#
# Copyright (c) 2016,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for APISet AVA CLI commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::AVA::CLI;

# Compiler Directives
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::AVA);

# CPAN imports
use Data::Dumper;
use Params::Validate qw(:all);
use NACL::APISet::_Mixins::CLI qw(:all);
use Tie::IxHash;

# NATE imports
use NATE::Log qw(log_global);
my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

our $AUTOLOAD;

# Exceptions imported
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;

###############################################################################
# Method Name: AUTOLOAD
# Objective: Catch all commands and call invoke
###############################################################################
sub AUTOLOAD {
    my $self = shift;
    $log->enter() if $may_enter;

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion

    tie my %args, 'Tie::IxHash', @_;

    $log->debug(sub { "Options to '$method' are:\n" . Dumper(\%args) });

    # Call invoke with the arguments given to execute the command
    my $_invoke = $self->_invoke(method => $method, args => \%args);

    $log->exit() if $may_exit;
    return $_invoke;
} ## end sub AUTOLOAD

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the AVA CLI APISet
# Details:   Initializes the APISet.  This method drives the workflow needed
#            to make the APISet ready for work.
##############################################################################
sub _init_API {
    my ($self, %args) = @_;
    $log->enter() if $may_enter;

    # dummy method
    $log->exit() if $may_exit;
} ## end sub _init_API

##############################################################################
# Method:    _get_connectrec_or_hostrec
# Objective: Sets the connectrec, hostrec, restobj in the arguments.
# Details:   This method is used by the _init_Session routine, to either set
#            the connection object, the host / rest object, whichever is defined.
###############################################################################
sub _get_connectrec_or_hostrec {
    my ($self, $args) = @_;
    $log->enter() if $may_enter;
    if (defined $self->{connection}) {
        $args->{connection} = $self->{connection};
    } elsif (defined $self->{hostobj}) {
        $args->{hostobj} = $self->{hostobj};
    }
    $log->exit() if $may_exit;
} ## end sub _get_connectrec_or_hostrec

##############################################################################
# Method:    _init_Session
# Objective: Initialize the AVA CLI Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################
sub _init_Session {

    my ($self, %args) = @_;
    $log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec(\%args);

    my $interface    = $self->{interface};
    my $session_type = $self->{shell_type};

    # Establish the session if not provided
    my $session_pkg
        = 'NACL::APISet::Session::' . $interface . "::" . $session_type;

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $log->exit() if $may_exit;
} ## end sub _init_Session

###############################################################################
# Method Name: _invoke
# Objective:   invoke the command
# Details:     This method invokes the AVA CLI procedure along with its input
#              argument object and returns its output (a response object).
###############################################################################
sub _invoke {
    my ($self, %args) = @_;
    $log->enter() if $may_enter;

    # Build the command
    my %connectrec;
    if ($args{args}->{'connectrec-match_table'}) {
        # All options prefixed with a connectrec are meant as options for
        # Connectrec and should not be translated by the CDEFs. These
        # are moved into a separate hash and delete from the args hash
        # so that they are not sent for translation
	$connectrec{'connectrec-match_table'} = $args{args}->{'connectrec-match_table'};
	delete $args{args}->{'connectrec-match_table'};
    }
    $self->_move_connectrec_options(
        source => \%connectrec,
        target => \%connectrec
    );
    my $method = $self->_translate_api_to_command($args{method});
    my $args = $self->_translate_args($args{args});
    my $command = $self->_translate(command => $method, options => $args);
    $log->debug("Final command is: " . $command);

    # send the command
    my $result_obj = $self->{session}->send_command(
	connectrec => \%connectrec,
	command    => $command
    );

    $log->exit() if $may_exit;
    return $result_obj;
} ## end sub _invoke

###############################################################################
# Method Name: _translate_api_to_command
# Objective:   translate API method name to command
###############################################################################
sub _translate_api_to_command {
    $log->enter() if $may_enter;
    my ($self, $cmd) = @_;

    $cmd =~ s/__/-/g;    # replace all '__' with '-'
    $cmd =~ s/_/ /g;    # replace all '_' with ' '

    $log->exit() if $may_exit;
    return $cmd;
} ## end sub _translate_api_to_command

###############################################################################
# Method Name: _translate_args
# Objective:   translate args to AVA args names
# Details:     This method translates __ to _
###############################################################################
sub _translate_args {
    $log->enter() if $may_enter;
    my ($self, $args) = @_;

    tie my %hash, 'Tie::IxHash';

    foreach my $key (keys %{$args}) {
        (my $new_key = $key) =~ s/__/-/g;    # replace all '__' with '-'
        $hash{$new_key} = $args->{$key};
    }

    $log->exit() if $may_exit;
    return \%hash;
} ## end sub _translate_args

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::AVA::CLI- Base for API Sets for AVA CLI calls

=head1 SYNOPSIS

 use NACL::APISet::AVA::CLI;

=head1 DESCRIPTION

A C<NACL::APISet::AVA::CLI> is used to connect to and send CLI command's.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
