#
# Copyright (c) 2016,... NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for AVA CLI AVAAdmin commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::AVA::CLI::AVAAdmin;

#
# Compiler Directives
#
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::AVA::CLI);

# Module import - APISets
use NACL::APISet::Session::CLI::AVAAdmin;

#
#Exceptions
#
use NACL::APISet::Exceptions::CommandFailedException;

#
# NATE imports
#
use Tie::IxHash;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    my $self = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec =>
            {version => {type => SCALAR, optional => 1, default => 99.99},}
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw CLI command on the filer and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $connection = $self->get_connection();

    $self->api_reconnect_session()
        if $connection->is_disconnected(verify => 0);

    my $output = $connection->execute(%args);

    $Log->exit() if $may_exit;
    return $output;
} ## end sub execute_raw_command

##############################################################################
# Method:    _translate
# Objective: Translate command and args to actual command
##############################################################################
sub _translate {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $command = $args{command};
    my $opts    = $args{options};

    # Here I am assuming that $opts is a hash tied to the Tie::IxHash class,
    # so that position-dependent arguments are preserved.
    my @args = map { "$_ $opts->{$_}" } (keys %{$opts});
    $command = join q{ }, $command, @args;

    $Log->exit() if $may_exit;
    return $command;
} ## end sub _translate

### Command overrides ###

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the AVA CLI AVAAdmin API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.
##############################################################################

sub _init_API {
    # NOTE: This is a dummy method left for future enhancements
    # during instantiation

    $Log->enter() if $may_enter;

    my ($self, %args) = @_;

    $self->SUPER::_init_API(%args);

    $Log->exit() if $may_exit;
} ## end sub _init_API

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::AVA::CLI::AVAAdmin - API Set for AVA AVAAdmin mode CLI commands

=head1 SYNOPSIS

 use NACL::APISet::AVA::CLI::AVAAdmin;

=head1 DESCRIPTION

A C<NACL::APISet::AVA::CLI::AVAAdmin> is used to connect to and send CLI commands to the
Normal interface of a AVA.

=head1 EXCEPTIONS

=over

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::AVA::CLI::AVAAdmin>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::AVA::CLI::AVAAdmin->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new API Set for AVA CLI Mode.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
