#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets Symbolapi commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Eseries::Symbolapi;

#
# Compiler Directives
#
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);

# Module import - APISets
use NACL::APISet::Schema::SymbolapiSchema;
use NACL::APISet::Session::Symbolapi;

our $AUTOLOAD;

#
# Exceptions imported
#
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Base class
use base qw(NACL::APISet::Eseries);


###############################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call invoke
###############################################################################

sub AUTOLOAD {
    my $self = shift;
    $Log->enter() if $may_enter;

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    my $spec = {};
    
    my %args = $self->validate_params(
        params => \@_,
        spec   => $spec,
        allow_extra => 1
    );
    $Log->debug( sub { "Options to '$method' are:\n" . Dumper( \%args ) } );

    my $_invoke;
        
    # Call invoke with the arguments given to execute the command
    $_invoke = $self->_invoke( method => $method, %args );

    $Log->exit() if $may_exit;
    return $_invoke;
        
} ## end sub AUTOLOAD

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw Symbolapi command on the Eseries and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    # $args{command} will contain the array having the procedure name, argument 
    # object and response object in the same order. 
    my $output = $self->{session}->send_command( command => $args{command},);

    $Log->exit() if $may_exit;
    return $output;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Eseries SYMbolapi API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path.
##############################################################################

sub _init_API {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $api_set   = $self->{set};
    my $path = $self->{cdef_path};
    
    $self->{cdef_path} = NATE::Inc::find_file( $path );

    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }
    
    my $cdef_path = $self->{cdef_path};
    
    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path.
    
    $self->{schema} = NACL::APISet::Schema::SymbolapiSchema->new();
    $self->{schema}->{searched_cdef_path} = $cdef_path;
    $self->{schema}->load( path => $cdef_path );

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Symbolapi Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec( \%args );

    my $interface = $self->{interface};

    # Establish the session if not provided

    my $session_pkg =
      "NACL::APISet::Session::" . $interface ;

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $Log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into input and response object and execute
# Details:     This method invokes the Symbolapi procedure along with its input
#              argument object and returns its output (a response object).
#              It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual procedure call to be invoked on
#              the service, and then invoking that on the Eseries using the 
#              Session object.
###############################################################################

sub _invoke {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $cdef_method = $args{method};

    my $schema_obj = $self->get_schema_obj();

    # translate cdef_method into command and get the procedure value, input object type
    # and response object type in an array
    my @cmd_input_output = $schema_obj->translate( cdef_method => $cdef_method, %args );
    
    # Form the input object and the response object
    my $translated_command = $schema_obj->input_output_object_creation( command_input_output => \@cmd_input_output );
   
    my $result_obj = $self->{session}->send_command(
        command          => $translated_command,
        method_name     => $cmd_input_output[0],
        actual_command  => $cdef_method,
    );

    $Log->exit() if $may_exit;
    return $result_obj;

} ## end sub _invoke

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Eseries::Symbolapi - Base for API Sets for Eseries procedure calls

=head1 SYNOPSIS

 use NACL::APISet::Eseries::Symbolapi;

=head1 DESCRIPTION

A C<NACL::APISet::Eseries::Symbolapi> is used to connect to and send Symbolapi
procedure calls.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument 

=back

=cut

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw Symbolapi commands on the Eseries . The input object
with the required arguments set and the response object should be provided by
the user. This will be invoked and the raw output will be returned as a string directly
without any processing.
This involves the user scripts to be written in Inline Java. For an example on using
execute_raw_command for Eseries, refer 
NACL/UnitTest/APISet/Eseries/Symbolapi/api_eseries_symbolapi.thpl  
This is useful for cases where we want to invoke a command without parsing the 
output or if the users want to invoke any Symbolapis whose entry is not in SYMbol.xml

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(command => @array_of_proc_input_obj_response_obj);

=item Arguments

=over

=item command

(Mandatory)
The procedure call along with the input and response objects
bundled in an array of Object[] type should be specified.

=back

=item Returns

Raw output

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
