#
# Copyright (c) 2014,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for APISet Webapi commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Eseries::Webapi;

# Compiler Directives
use strict;
use warnings;

# CPAN imports
use Data::Dumper;
use Params::Validate qw(:all);

# NATE imports
use NATE::Log qw(log_global);
my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

# NACL Module import - APISets
use NACL::APISet::Session::Webapi;

our $AUTOLOAD;

# Exceptions imported
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Base class
use base qw(NACL::APISet::Eseries);

###############################################################################
# Method Name: AUTOLOAD
# Objective: Catch all commands and call invoke
###############################################################################
sub AUTOLOAD {
    my $self = shift;
    $log->enter() if $may_enter;

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    $method =~ s/_/-/g;    # replace all '_' with '-'
    my $spec = {};
    
    my %args = $self->validate_params(
        params => \@_,
        spec   => $spec,
        allow_extra => 1
    );
    $log->debug( sub { "Options to '$method' are:\n" . Dumper( \%args ) } );

    my $_invoke;
        
    # Call invoke with the arguments given to execute the command
    $_invoke = $self->_invoke( method => $method, %args );

    $log->exit() if $may_exit;
    return $_invoke;
        
} ## end sub AUTOLOAD


##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Eseries Webapi APISet
# Details:   Initializes the APISet.  This method drives the workflow needed
#            to make the APISet ready for work.
##############################################################################
sub _init_API {
    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;
    # dummy method
    $log->exit() if $may_exit;
}

##############################################################################
# Method:    _get_connectrec_or_hostrec
# Objective: Sets the connectrec, hostrec, restobj in the arguments.
# Details:   This method is used by the _init_Session routine, to either set
#            the connection object, the host / rest object, whichever is defined.
###############################################################################
sub _get_connectrec_or_hostrec {
    my ( $self, $args ) = @_;
    $log->enter() if $may_enter;
    if ( defined $self->{connection} ) {
        $args->{connection} = $self->{connection};
    } elsif ( defined $self->{hostobj} ) {
        $args->{hostobj} = $self->{hostobj};
    } 
    if ( defined $self->{restobj} ) {
        $args->{restobj} = $self->{restobj};
    }
    $log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Webapi Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################
sub _init_Session {

    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec( \%args );

    my $interface = $self->{interface};

    # Establish the session if not provided
    my $session_pkg = 'NACL::APISet::Session::' . $interface ;

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   invoke the command
# Details:     This method invokes the Webapi procedure along with its input
#              argument object and returns its output (a response object).
###############################################################################
sub _invoke {
    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;

    # send the command
    my %send_args = %args;
    $send_args{cmd} = delete $send_args{method};
    my $result_obj = $self->{session}->send_command( %send_args );

    $log->exit() if $may_exit;
    return $result_obj;
    
} ## end sub _invoke

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Eseries::Webapi - Base for API Sets for Eseries Webapi calls

=head1 SYNOPSIS

 use NACL::APISet::Eseries::Webapi;

=head1 DESCRIPTION

A C<NACL::APISet::Eseries::Webapi> is used to connect to and send Webapi's.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument 

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
