# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary This exception gets thrown when multiple operations fail
##          This also gets NACL::APISet::Exceptions::ResponseException in its
##          inheritance hierarchy.
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::APISet::Exceptions::MultipleCommandFailedException

=head1 DESCRIPTION

Using either wildcard characters or extended query syntax, it is possible
to affect multiple elements with a single command. In the process, more than
one of these operations could fail, in which case the error text will really
consist of multiple error messages. This type of exception is thrown when
multiple operations fail.

Retrieval of each of the errors that occurred can be done by using the
new C<failures> accessor method. (See below)

=head1 EXAMPLES

=head2 Example error output

cluster1::> storage aggregate delete -aggregate *

Error: command failed on aggregate "aggr0": It is not allowed to delete an aggregate that contains volume(s). Please
       first remove all of the volumes on the aggregate.

Error: command failed on aggregate "component_flex_aggr": It is not allowed to delete an aggregate that contains
       volume(s). Please first remove all of the volumes on the aggregate.
[Job 73] Job succeeded: DONE
[Job 74] Job succeeded: DONE
2 entries were acted on.

cluster1::> storage aggregate modify { -state online } -state online

Error: command failed on aggregate "aggr0": Aggregate "aggr0" is already online.

Error: command failed on aggregate "component_flex_aggr": Aggregate "component_flex_aggr" is already online.
0 entries were modified.

=head2 Throwing exception

The throwing syntax is similar to that of other ResponseExceptions. In
addition to the arguments accepted by other exceptions subclassed from
C<ResponseException>, a new C<error_messages> argument needs to be provided.
This is expected to be an array-reference of all the error messages that
showed up in the output.

    NACL::APISet::Exceptions::MultipleCommandFailedException->throw(
        text            => $text,
        response_object => $response_obj,
        error_messages  => \@arr_of_err_msgs
    );

=head2 Catching exception

The methods exposed by all other exceptions subclassed off ResponseException
are exposed by this exception as well. Additionally, there is a new
C<failures> accessor method to obtain all of the error messages.
This is can be invoked in either scalar or list context.

    # In scalar context we get back all errors as an array-reference
    my $errors_array_ref = $exception->error_messages();

    # In list context we get back all errors as an array
    my @errors = $exception->error_messages();

=head3 Basic example

    # We're assuming the output is like case 2 shown in the examples section
    try {
        $apiset->storage_aggregate_modify(
            extended_query => { state => 'online' },
            state          => 'online'
        );
    } catch NACL::APISet::Exceptions::MultipleCommandFailedException with {
        my $exception = shift;

        # ->text() returns the B<complete> error message, i.e. it is one
        # string containing all error messages. Here it will be:
        #
        # Error: command failed on aggregate "aggr0": Aggregate "aggr0" is already online.
        #
        # Error: command failed on aggregate "component_flex_aggr": Aggregate
        # "component_flex_aggr" is already online.
        # 0 entries were modified.
        #
        $Log->comment('Full error is: ' . $exception->text());

        # C<get_response_object> works as like any other ResponseException
        my $response = $exception->get_response_object();

        # C<error_messages> is the new accessor method applicable only for this
        # exception
        my @failures = $exception->error_messages();
        $Log->comment("Failures:\n" . Dumper(\@failures));
    };

=cut

package NACL::APISet::Exceptions::MultipleCommandFailedException;

# Sub-class CommandFailed because earlier code would have thrown this
# exception. By sub-classing, we maintain backwards compatibility.
use base qw(NACL::APISet::Exceptions::CommandFailedException);

use Params::Validate qw(validate_with ARRAYREF CODEREF);

use Class::MethodMaker [ array => 'error_messages', ];

# "error_messages" was earlier called "failures". Maintain backwards
# compatibility by aliasing them
no strict 'refs';
*failures = *error_messages;
use strict 'refs';

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    $Log->enter() if $may_enter;
    my ($pkg, @args)  = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { error_messages => { type => ARRAYREF } },

        # Other options are those accepted and validated by the base classes
        allow_extra => 1
    );
    my $self = $pkg->SUPER::new(%opts);
    $Log->exit() if $may_exit;
    return $self;
}

=head1 METHODS FOR DEVELOPERS

The methods listed in this section are for developers of NACL exceptions.
It is suggested that these methods are not used outside of NACL code
since the interface of these methods is subject to change at any time.

=head2 convert_to_base_component_exception

    # Using NACL::C::Vserver->modify as example
    sub _modify_cmode_cli {
        ...
        try {
            $apiset->vserver_modify(%cmode_args);
        }  catch NACL::APISet::Exceptions::MultipleCommandFailedException with {
            my $exception = shift;
            my $sub = sub {
                my ($error) = validate_pos(@_, { type => SCALAR });
                $error =~ /failed on vserver "(.*?)"/;
                my $pk_hash = { vserver => $1 };
                return $pk_hash;
            };
            my $new_exception = $exception->convert_to_base_component_exception(
                extract_pk_sub => $sub);
            $Log->exit() if $may_exit;
            $new_exception->throw();
        };
    }

In CMode-CLI, if wildcard characters (such as "*") or extended-query syntax
is used, then multiple elements can be operated on simulataneously. In the
case of multiple failures occurring, the CMode-CLI APISet throws a
C<NACL::APISet::Exceptions::MultipleCommandFailedException>. For the
component, we need to convert this to a
L<NACL::C::Exceptions::MultipleCommandFailed|lib-NACL-C-Exceptions-MultipleCommandFailed-pm>
exception. This method does the necessary conversion.

=over

=item Options

=over

=item C<< extract_pk_sub => $coderef >>

A code-reference which gets passed the error message as argument. This
should extract the primary keys from the error message and return the
primary keys as a hash-reference.

=back

=back

=cut

sub convert_to_base_component_exception {
    $Log->enter() if $may_enter;
    require NACL::C::Exceptions::MultipleCommandFailed;
    my ($self, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            extract_pk_sub => { type => CODEREF },
        }
    );
    my $sub = $opts{extract_pk_sub};
    my @errors = $self->error_messages();
    my @all_failures;
    foreach my $error (@errors) {
        my $single_failure = NACL::C::Exceptions::MultipleCommandFailed->get_SingleCommandFailed;
        $single_failure->text($error);
        my $pk_hash = $sub->($error);
        $single_failure->primary_keys($pk_hash);
        $single_failure->response_object($self->get_response_object());
        push @all_failures, $single_failure;
    }

    # Sneaky re-bless
    my $obj = bless $self, 'NACL::C::Exceptions::MultipleCommandFailed';
    $obj->all_failures(@all_failures);
    $Log->exit() if $may_exit;
    return $obj;
}

1;
