#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when a command
##          execution fails or does not provide expected output.
##
## @author madhav.shekar@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

NACL::APISet::Exceptions::ResponseException.pm

=head1 Author

Madhav C Shekar

=head1 Description

An exception for when command execution fails or does not provide expected
output

=head1 See Also

Package inherits from the BaseException module in NATE.

=head1 NACL::APISet::Exceptions::ResponseException

=head2 Expected use

Use when command execution fails or does not provide expected output

=head2 Usage

=over

=item Throwing syntax

Normally, while throwing an exception we'd just need to provide the 
error text as argument. For ResponseException, and all exceptions sub-classed
off it, there is a slightly different syntax since we throw the response 
object along with the error text. We provide two named 
parameters - text and response_object. E.g:
NACL::APISet::Exceptions::ResponseException->throw(text => "Error text",
                                                   response_object => $resp);

=item Extracting the response object from the exception object

This can be done by invoking the method "get_response_object" on the
exception object.

catch NACL::APISet::Exceptions::ResponseException with {
    my $exception_object = shift;
	my $response_object = $exception_object->get_response_object();
};

=back

=head2 Base Class

NATE::BaseException

=cut

use strict;
use warnings;

package NACL::APISet::Exceptions::ResponseException;
use base qw(NATE::BaseException);
use NATE::Exceptions::Argument;

# Import methods common to various exceptions
use NACL::APISet::_Mixins::Exceptions::Common qw(:all);

use Params::Validate qw(validate validate_with SCALAR OBJECT UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 ATTRIBUTES

The following are attributes of the exception object and can be accessed
using the accessor methods of the same name as the attribute.

=head2 is_zapi

Is set to 1 if this exception was thrown because of a ZAPI call. Certain
exceptions have fields which are filled in only for ZAPI calls, such as
error_number (see below).

=head2 error_number

A ZAPI specific field. This will be the "error-number" value returned
in the ZAPI output XML.

=cut

use Class::MethodMaker [
    scalar => [
        { -default_ctor => sub { $_[0]->_is_zapi() } },
        'is_zapi'
    ],
    scalar => [
        { -default_ctor => sub { $_[0]->_get_error_number() } },
        'error_number'
    ]
];

sub new {
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            # Remove the UNDEF from 'text' once parsers are fixed
            text            => { type => SCALAR | UNDEF,         optional => 1 },
            response_object => { type => OBJECT | UNDEF, optional => 1 },
        },
        allow_extra => 1
    );

    my $text = delete $args{text};
    unless ($text) {
        $Log->warn( "The value for the 'text' field was not provided and "
                . "was hence defaulted to 'Error'. Check that the 'text' "
                . 'field is provided as a named argument' );
        $text = 'Error';
    }

    my $response_object = delete $args{response_object};
    if ( defined $response_object ) {

        # Is an object
        unless ( $response_object->isa('NACL::APISet::Response') ) {
            NATE::Exceptions::Argument->throw( "The 'response_object' "
                    . 'argument should have been an object which isa '
                    . "ResponseException but an object of type '"
                    . ref($response_object)
                    . "' was provided" );
        }
    } else {
        $Log->warn( "ResponseException should have 'response_object' provded "
                . 'as argument, but it was not provided in the call' );
    }

    $self->SUPER::new( $text, -response_object => $response_object, %args );
}

sub get_response_object {
    my $self = shift;
    return $self->{-response_object};
}

sub set_error_message {
    my $self = shift;
    $self->set_text(@_);
}

sub get_error_message {
    my $self = shift;
    return $self->text();
}

sub set_response_object {
    my $self            = shift;
    my $response_object = shift;
    $self->{-response_object}
        = ( ref($response_object) =~ /NACL::APISet::Response/ )
        ? $response_object
        : undef;
}

sub _is_zapi {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $resp = $self->get_response_object();
    if (defined $resp) {
        if ($resp->isa('NACL::APISet::Response::ZAPI')) {
            $Log->exit() if $may_exit;
            return 1;
        } else {
            $Log->exit() if $may_exit;
            return 0;
        }
    }
    $Log->exit() if $may_exit;
    return;
}

sub _get_error_number {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    if ($self->_is_zapi()) {
        my $resp = $self->get_response_object();
        $Log->exit() if $may_exit;
        return $resp->get_error_number();
    } else {
        $Log->exit() if $may_exit;
        return;
    }
}

1;
