#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown if
##          connection times out or is idle
##
## @author dl-nacl-dev@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

NACL::APISet::Exceptions::TimeoutException.pm

=head1 Author

dl-nacl-dev@netapp.com

=head1 Description

An exception to be thrown if connection times out or is idle

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

=head1 NACL::APISet::Exceptions::TimeoutException

=head2 Expected use

Use if connection times out or is idle.

=head2 Usage

Same as for exceptions defined by BaseException

=head2 Base Class

NATE::BaseException

=head2 Attributes

Each of the attributes listed below have accessor methods,
so (using the feature attribute as example) this can be done:

    $timeout = $exception_object->timeout();
    $output  = $exception_object->output();

=over

=item C<timeout>

Returns the current timeout value.

=item C<output>

Returns the output captured before the timeout occurred.

=back

=head2 Examples

=over

=item B<Throwing the exception>

    NACL::APISet::Exceptions::TimeoutException->throw(
        $error_text,
        timeout => $timeout,
        output =>$output
    );

=item B<Catching the exception>

    try {
        ...
    } catch NACL::APISet::Exceptions::TimeoutException with {
        my $exception = shift;
        # Getting the error text is the same as for other exceptions
        $Log->comment('Error is: ' . $exception->text());
        #Getting the timeout value set.
        my $timeout = $exception->timeout();
        #Getting the output obtained before the command timed out.
        my $output  = $exception->output();
    };

=back

=cut

package NACL::APISet::Exceptions::TimeoutException;
use base qw(NATE::BaseException);
use Params::Validate qw(validate_with SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Import methods common to various exceptions
use NACL::APISet::_Mixins::Exceptions::Common qw(:all);
use Class::MethodMaker [
    scalar => 'timeout',
    scalar => 'output'
];

sub new {
    my ( $pkg, $error_text, @args ) = @_;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            timeout => { type => SCALAR, optional => 1 },
            output  => { type => SCALAR, optional => 1 },
        },
    );
    my $self = $pkg->SUPER::new( $error_text, %opts );
    $Log->exit() if $may_exit;
    return $self;
}
1;
