#
# Copyright (c) 2015,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for APISet Flashray CLI commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Flashray::CLI;

# Compiler Directives
use strict;
use warnings;

# CPAN imports
use Data::Dumper;
use Params::Validate qw(:all);

# NATE imports
use NATE::Log qw(log_global);
my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

our $AUTOLOAD;

# Exceptions imported
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Base class
use base qw(NACL::APISet::Flashray);

###############################################################################
# Method Name: AUTOLOAD
# Objective: Catch all commands and call invoke
###############################################################################
sub AUTOLOAD {
    my $self = shift;
    $log->enter() if $may_enter;

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    my $spec = {};

    my %args = $self->validate_params(
        params => \@_,
        spec   => $spec,
        allow_extra => 1
    );
    $log->debug( sub { "Options to '$method' are:\n" . Dumper( \%args ) } );

    my $_invoke;

    # Call invoke with the arguments given to execute the command
    $_invoke = $self->_invoke( method => $method, %args );

    $log->exit() if $may_exit;
    return $_invoke;

} ## end sub AUTOLOAD


##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Flashray CLI APISet
# Details:   Initializes the APISet.  This method drives the workflow needed
#            to make the APISet ready for work.
##############################################################################
sub _init_API {
    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;
    # dummy method
    $log->exit() if $may_exit;
}

##############################################################################
# Method:    _get_connectrec_or_hostrec
# Objective: Sets the connectrec, hostrec, restobj in the arguments.
# Details:   This method is used by the _init_Session routine, to either set
#            the connection object, the host / rest object, whichever is defined.
###############################################################################
sub _get_connectrec_or_hostrec {
    my ( $self, $args ) = @_;
    $log->enter() if $may_enter;
    if ( defined $self->{connection} ) {
        $args->{connection} = $self->{connection};
    } elsif ( defined $self->{hostobj} ) {
        $args->{hostobj} = $self->{hostobj};
    }
    $log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Flashray CLI Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################
sub _init_Session {

    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec( \%args );

    my $interface = $self->{interface};
    my $session_type = $self->{shell_type};

    # Establish the session if not provided
    my $session_pkg = 'NACL::APISet::Session::' . $interface . "::" . $session_type;

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   invoke the command
# Details:     This method invokes the Flashray CLI procedure along with its input
#              argument object and returns its output (a response object).
###############################################################################
sub _invoke {
    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;

    # send the command
    my $method = $self->_translate_api_to_command(api=>delete $args{method});
    my %cmdArgs = $self->_translate_args(args=>\%args);
    $cmdArgs{command} = $method;

    my $command = $self->_translate(args=>\%cmdArgs);
    $log->debug("Final command is: ". $command);

    my $result_obj = $self->{session}->send_command(command=>$command);

    $log->exit() if $may_exit;
    return $result_obj;

} ## end sub _invoke


###############################################################################
# Method Name: _translate_api_to_command
# Objective:   translate API method name to command
###############################################################################
sub _translate_api_to_command {
    $log->enter() if $may_enter;

    my ($self, @args) = @_;
    my %opts = validate(@args, {api => {type => SCALAR}});
    my $cmd = $opts{api};
    $cmd =~ s/__/-/g;    # replace all '__' with '-'
    $cmd =~ s/_/\ /g;    # replace all '_' with ' '

    $log->exit() if $may_exit;
    return $cmd;
}

###############################################################################
# Method Name: _translate_args
# Objective:   translate args to Flashray args names
# Details:     This method translates __ to _
###############################################################################
sub _translate_args{
    my ( $self, %args ) = @_;
    $log->enter() if $may_enter;
    my %hash = %{$args{args}};
    my %retHash = ();
    foreach my $key ( keys %hash) {
        my $newKey = $key;
        $newKey =~ s/__/-/g;    # replace all '__' with '-'
        $retHash{$newKey} = $hash{$key};
    }

    $log->exit() if $may_exit;
    return %retHash;
}


1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Flashray::CLI- Base for API Sets for Flashray CLI calls

=head1 SYNOPSIS

 use NACL::APISet::Flashray::CLI;

=head1 DESCRIPTION

A C<NACL::APISet::Flashray::CLI> is used to connect to and send CLI command's.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
