#
# Copyright (c) 2015,... NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for Flashray CLI MarsMaint commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Flashray::CLI::MarsMaint;

#
# Compiler Directives
#
use strict;
use warnings;

# Module import - APISets
use NACL::APISet::Session::CLI::MarsMaint;

#
#Exceptions
#
use NACL::APISet::Exceptions::CommandFailedException;

#
# NATE imports
#
use Tie::IxHash;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Flashray::CLI);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;
    my $self = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            version => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw CLI command on the filer and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $connection = $self->get_connection();

    $self->api_reconnect_session()
        if $connection->is_disconnected( verify => 0 );

    my $output = $connection->execute(%args);

    $Log->exit() if $may_exit;
    return $output;
} ## end sub execute_raw_command


##############################################################################
# Method:    _translate
# Objective: Translate command and args to actual command
##############################################################################
sub _translate {
    my ( $self, %opts) = @_;
    $Log->enter() if $may_enter;
    my %args = %{$opts{args}};
    my $command = delete $args{command};

    foreach my $key ( keys %args ){
        $command = $command . " -$key $args{$key}";
    }

    $Log->exit() if $may_exit;
    return $command;
} ## end sub _translate

### Command overrides ###

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the MarsMaint API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API.
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    $self->SUPER::_init_API(%args);

    $Log->exit() if $may_exit;
} ## end sub _init_API


1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Flashray::CLI::MarsMaint- API Set for Flashray CLI Maint commands

=head1 SYNOPSIS

 use NACL::APISet::Flashray::CLI::MarsMaint;

=head1 DESCRIPTION

A C<NACL::APISet::Flashray::CLI::MarsMaint> is used to connect to and send CLI commands to the
Flashray maint mode

=head1 EXCEPTIONS

=over

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Flashray::CLI::MarsMaint>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Flashray::CLI::MarsMaint->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new API Set for CLI MarsMaint mode

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
