#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets CLI commands for Host

## @author dl-nacl-dev@netapp.com
## @change 2014 rohit4@netapp.com, Added set=>SMTT support

## @status Shared
## @pod here

package NACL::APISet::Host::CLI;

#
# Compiler Directives
#
use strict;
use warnings;
use Params::Validate qw(:all);

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

# Module import - APISets
use NACL::APISet::Schema;
use NACL::APISet::_Mixins::CLI qw(:all);
use NATE::ParamSet qw( param_global);

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::APISet::Exceptions::TimeoutException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

# Base class
use base qw(NACL::APISet::Host);

our $AUTOLOAD;

# This is what most constructors of Host APISet does
# - just bless the object
sub new {
    $Log->enter() if $may_enter;
    my $class = $_[0];
    my $self  = {};
    bless $self, $class;
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method Name: get_host
# Objective:  Return stored host name
# Details:    Refer to POD documentation at the end of the file
###############################################################################

sub get_host {
    my $self = shift;
    my $host;
    if ($self->{hostobj}) {
        $host = $self->{hostobj};
    } else {
        $host = $self->{connection}->hostp();
    }
    return $host->id();
} ## end sub get_host

###############################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call invoke
#             If the API call is prefixed with an "xlate_" prefix, it should
#             only be translated and not invoked, so we should call
#             _only_translate_method
###############################################################################

sub AUTOLOAD {
    my ($self, @opts) = @_;
    $Log->enter() if $may_enter;
    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    my %args = $self->_check_for_unnamed_parameters(
        options => \@opts,
        method  => $method
    );
    $Log->debug(sub { "Options to '$method' are:\n" . Dumper(\%args) });

    if ($method =~ /^xlate_/) {
        if ($method =~ /^xlate_execute(_raw)?_command/) {
            my $name = $1 || "";

            # Explicitly prevent xlate_ being used with execute_raw_command
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "xlate cannot be used with execute$name" . "_command");
        } else {
            $Log->exit() if $may_exit;
            return $self->_only_translate_method(method => $method, %args);
        }
    } else {
        my $_invoke;
        my $pre_method  = '_' . $method . '_pre';
        my $post_method = '_' . $method . '_post';
        my $result_hash = {};
        if ($self->can($pre_method)) {
            $result_hash = $self->$pre_method(args => \%args);
        }
        try {

            # Call invoke with the arguments given to execute the command
            $_invoke = $self->_invoke(method => $method, %args);
        }
        catch NATE::BaseException with {
            my $exception = shift;
            if ($exception->text() =~ /rsh server returned error:/) {
                NACL::APISet::Exceptions::ConnectionFailedException->throw(
                    $exception->text());
            } else {
                $exception->throw();
            }
        };
        if ($self->can($post_method)) {
            $self->$post_method(
                args       => \%args,
                pre_result => $result_hash,
                resp_obj   => $_invoke,
            );
        }
        $Log->exit() if $may_exit;
        return $_invoke;
    }
} ## end sub AUTOLOAD

##############################################################################
# Method:    answer_password
# Objective: Get the setting value for whether password should be answered
# Details:   See POD documentation at the end of this file.
##############################################################################

sub answer_password {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    $Log->exit() if $may_exit;
    return $self->{"answer-password"};
} ## end sub answer_password

##############################################################################
# Method:    set_privilege
# Objective: For running the command as the specified user
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_privilege {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            "privilege-level" => {type => SCALAR},
            "answer-password" => {type => BOOLEAN, optional => 1}
        }
    );

    my $priv = $args{"privilege-level"};
    if ($self->is_systemshellNGSH()) {
        $self->execute_raw_command(command => "set -privilege $priv");
    } else {
        $self->{"privilege-level"} = $priv;
        $self->{"answer-password"} = $args{"answer-password"};
    }

    $Log->exit() if $may_exit;
} ## end sub set_privilege

##############################################################################
# Method:    clear_privilege
# Objective: To default back to current user
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_privilege {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    $self->set_privilege("privilege-level" => "", "answer-password" => 0);
    $Log->exit() if $may_exit;
} ## end sub clear_privilege

sub is_systemshellNGSH {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $Log->exit() if $may_exit;
    return ($self->{actual_set} eq 'SystemshellNGSH') ? 1 : 0;
} ## end sub is_systemshellNGSH

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Host API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path.
##############################################################################

sub _init_API {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    # set default privilege level
    if (!defined $self->{'privilege-level'}) {
        $self->{'privilege-level'} = "";
    }

    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path. Load CDEFS after the check for
    # shell_type is done so that we need not unnessarily load CDEFs even in
    # case of invalid shell_type.
    # Note: Schema not applicable for SQL
    my $actual_set = $self->{actual_set};
    if ($actual_set ne 'SQL') {
        $self->{schema} = NACL::APISet::Schema->new(
            is_windows => $self->{set} eq 'Windows' ? 1 : 0);

        # Get the absolute paths for CDEF directories.
        $self->_get_absolute_cdef_paths();
        unless ($self->{cdef_path}) {
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                      "Error in finding CDEF directories "
                    . $self->{cdef_path}
                    . " and the reason: \n$@");
        }
        my $cdef_paths = $self->{cdef_path};

   # Get the list of th CDEF files to be loaded. The implementation of getting
   # the CDEF files list will go into the specific APISet class.
   # Handle linux, windows and HTTP paths as well.
        my @files
            = @{$self->{schema}->read_cdef_dir(directory => $cdef_paths)};

        # if files list is empty, cdef files have not been consolidated
        if (scalar(@files) == 0) {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                "No cdef data files found in: $cdef_paths ");
        }

        # Load the cdef files
        $self->{schema}->{searched_cdef_path} = $cdef_paths;

        # CDEFs currently not present for PowerShell, so can skip loading.
        if ($actual_set ne 'PowerShell') {
            $self->{schema}->load(files => \@files, path => $cdef_paths);
        }
    } ## end unless ($self->{actual_set...

    $Log->exit() if $may_exit;
} ## end sub _init_API

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Host Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec(\%args);

    my $interface = $self->{interface};

    # First check whether a valid shell_host (hostobj) is passed
    # for the context
    $self->_check_session_type() if (!($self->{set} eq "SMTT"));

    # Establish the session if not provided
    # Determine the type of the session based on the shell_type and context.
    my $session_type;
    if ($self->is_systemshellNGSH()) {
        $session_type = $self->{shell_type};
    } elsif (($self->{shell_type} eq "CMode")
        || ($self->{shell_type} eq "7Mode")) {

        # User had asked for system shell context and in this case we will
        # use the subclass of 7Mode/CMode session available for running
        # "system shell" commands.
        $session_type = $self->{shell_type} . "::Shell";
    } else {

        # A normal Host APISet is requested
        $session_type = $self->{set};
    }

    if ($self->{set} eq "SMTT") {
        if ($self->{shell_type} eq "Windows") {
            $session_type = "Windows";
        } elsif ($self->{shell_type} eq "Linux") {
            $session_type = "Linux";
        }

    }

    # Establish the session if not provided
    my $session_pkg
        = "NACL::APISet::Session::" . $interface . "::" . $session_type;

    # Create a Session object. This takes care of connecting to the service
    # and preparing it for use.
    $self->{session} = $session_pkg->new(%args);

    if ($self->is_systemshellNGSH()) {

        # FreeBSD command that are ported to NGSH will be
        # available in diag privilege-level. Setting
        # privilege-level to diag
        $self->set_privilege("privilege-level" => "diag");
    }

    $Log->exit() if $may_exit;
} ## end sub _init_Session

###############################################################################
# Method Name: _check_session_type
# Objective:   Check whether the shell_type passed by the user is appropriate
#              for the shell_host
###############################################################################
sub _check_session_type {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    # Valid combinations for shell_type and set passed by the user
    my %valid_hash = (

        #actual_set         shell_type
        "Freebsd"         => "Freebsd",
        "Linux"           => "Linux",
        "Windows"         => "Windows",
        "DFM"             => "Windows Linux",
        "Systemshell"     => "7Mode CMode",
        "Solaris"         => "Solaris",
        "SMO"             => "Linux Windows",
        "SQL"             => "Linux Windows",
        "PowerShell"      => "Windows",
        "HPUX"            => "Linux",
        "AIX"             => "Linux",
        "Kerberos"        => "Linux",
        "SystemshellNGSH" => "CMode",
        "SMTT"            => "Windows",
        "MarsRoot"        => "MarsRoot",
        "AVARoot"         => "AVARoot",
	"ONTAPSelectDeploy" => "ONTAPSelectDeploy",
    );
    if ($valid_hash{$self->{actual_set}} !~ /$self->{shell_type}/) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Invalid hostobj or shell_type "
                . "is sent for set '$self->{actual_set}'"
                . " and the shell_type obtained is "
                . $self->{shell_type});
    }
    $Log->exit() if $may_exit;
} ## end sub _check_session_type

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into actual command and execute
# Details:     This method invokes the command along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual command to be invoked on the
#              service, and then invoking that on the filer using the Session
#              object.
###############################################################################

sub _invoke {
    $Log->enter() if $may_enter;
    my ($self) = shift();

    $self->api_reconnect_session()
        if $self->get_connection()->is_disconnected(verify => 0);

    my (%connectrec, $actual_command, $mount_prefix);

    my %args = NACL::APISet->validate_params(
        params => \@_,
        spec   => {
            "privilege-level" => {type => SCALAR,  optional => 1},
            "answer-password" => {type => BOOLEAN, optional => 1},
        }
    );

    my $privilege              = delete $args{'privilege-level'};
    my $answer_password        = delete $args{'answer-password'};
    my $method                 = $args{method};
    my $return_response_object = delete $args{'return_response_object'};
    my $nacl_refresh_stale_connections_retry = 0;
    my $retry_value = param_global->get('NACL_REFRESH_STALE_CONNECTIONS_RETRY');
    if(defined $retry_value) {
        $nacl_refresh_stale_connections_retry = $retry_value if($retry_value == 1);
    }

    if (   $method eq "execute_command"
        && $return_response_object
        && $self->{actual_set} ne "Linux"
        && $self->{actual_set} ne "Solaris"
        && $self->{actual_set} ne "Freebsd"
        && $self->{actual_set} ne "PowerShell"
        && $self->{actual_set} ne "HPUX"
        && $self->{actual_set} ne "AIX"
        && $self->{actual_set} ne "Kerberos") {
        NATE::Exceptions::Argument->throw(
            "Invalid value: \"return_response_object\" cannot be '1' for \"$self->{actual_set}\" "
        );
    }
    # process parameters
    while (my ($key, $value) = each(%args)) {
        if ($key =~ /-?method$/) {
            if (!length($value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
        } elsif ($key =~ /-?debug_level$/) {
            if (!length($value) || $value !~ /\d+/) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /connectrec\-(.*)/ || $method eq 'execute_command') {

            # All options prefixed with a connectrec are meant as options for
            # Connectrec and should not be translated by the CDEFs. These
            # are moved into a separate hash and delete from the args hash
            # so that they are not sent for translation
            $connectrec{$key} = $value;
            delete $args{$key};
        }
    } ## end while (my ($key, $value) ...

    # verify mandatory cdef argument
    unless (defined $args{method} and length $args{method}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Mandatory argument \"cdef\" not specified ");
    }

    my ($command, $file);

    $self->_move_connectrec_options(
        source => \%connectrec,
        target => \%connectrec
    );

    if ($self->is_systemshellNGSH()) {
        $connectrec{actual_set} = $self->{actual_set};
    }

    undef $mount_prefix;
    # retrieve schema handle
    if ($method ne 'execute_command') {
       $mount_prefix = delete $args{'mount-prefix'} if (exists $args{'mount-prefix'});

       my $vsid;
       if ( ($args{'method'} =~ /^(ifstat|ifconfig|netstat)$/) && (defined $args{'vsid'}) ) {
	   $vsid = delete $args{'vsid'};
       }

       $command = $actual_command = $self->_translate_method_call(%args);

       $command = $mount_prefix.' '.$command if (defined $mount_prefix);

       # as per burt#1072529, specific handling is needed to execute
       # 'icacls' command with 'grant' and 'remove' option.
       $command =~ s/grant:/grant/ if ( $args{'method'} eq 'icacls' && ((defined $args{'grant'} && $args{'grant'} eq '') && defined $args{'sid-perm'}) );
       $command =~ s/remove:/remove/ if ( $args{'method'} eq 'icacls' && ((defined $args{'remove'} && $args{'remove'} eq '') && defined $args{'sid'}) );

       # as per burt#1058637 and burt#1062492, specific handling to execute
       # commands 'ifstat', 'ifconfig' and 'netstat' on vontext mode if vsid
       # is specified.
       if ( defined $vsid ) {
	   my $vcontext_command = "sudo vcontext -v $vsid \`which $args{'method'}\`";
	   $vcontext_command =~ s/^sudo // if ( $args{'method'} eq 'netstat') ;
	   if ( $command =~ /^sudo\s+jall\s+ifstat\s+(.*)/ ) {
		$vcontext_command .= " ".$args{'method'}." ".$1;
	   } else {
		$vcontext_command .= " ".$command;
	   }
	   $command = $actual_command = $vcontext_command;
       }

       # Find parser file
       $file = $self->_find_parser_file(method => $method);
    }

    $connectrec{"on_timeout"} = $self->_throw_timeout_exception(
        before => $connectrec{"on_timeout"});
    $connectrec{"on_idle"}
        = $self->_throw_timeout_exception(before => $connectrec{"on_idle"});

    # execute command

    my $result;

    # Retry send_command maximum 3 times
    my $redo = 3;
    # Attempt send_command
    SEND_CMND: {
    if ($method ne 'execute_command') {
	try {
        $result = $self->{session}->send_command(
            command          => $command,
            command_username => defined($privilege) ? $privilege
            : $self->get_privilege(),
            answer_password => defined($answer_password) ? $answer_password
            : $self->answer_password(),
            actual_command => $actual_command,
            method_name    => $method,
            cdef_path      => $self->{cdef_path},
            parser_file    => $file,
            connectrec     => \%connectrec,
	    );
        } catch NATE::BaseException with {
            my $exception = shift;
            if ($redo && ($exception->text() =~ /[can|could] not write|Write failed: Broken pipe/i) && $nacl_refresh_stale_connections_retry) {
                # Connection failed, wait and retry
                sleep(5);
                $redo--;
                $self->{session}->_reconnect();
                $self->{"session"}->{"connection"}->resync();
                $self->{session}->prep_service();
                $self->clear_privilege();
                # If failure is because of connectivity, re-run SEND_CMND.
                redo SEND_CMND ;
	    } else {
		$exception->throw();
	    }
	}; # end catch
    } elsif ($return_response_object) {
        $result = $self->{session}->send_command(
            command          => $connectrec{command},
            command_username => defined($privilege) ? $privilege
            : $self->get_privilege(),
            answer_password => defined($answer_password) ? $answer_password
            : $self->answer_password(),
            actual_command => $connectrec{command},
            method_name    => $method,
            cdef_path      => '',
            parser_file    => '',
            connectrec     => \%connectrec,
        );
    } else {
        delete $connectrec{method};
        $result = $self->{session}->send_command(
            command_username => defined($privilege) ? $privilege
            : $self->get_privilege(),
            answer_password => defined($answer_password) ? $answer_password
            : $self->answer_password(),
            connectrec => \%connectrec,
        );
    }
    } # End SEND_CMND
    $Log->exit() if $may_exit;
    return $result;
} ## end sub _invoke

##############################################################################
# Method:    _translate_method_call
# Objective: Returns the command translated from method call
# Details:   This method uses schema object to translate method call to
#            the actual command to be invoked and this is done with the
#            help of CDEFs. If the specific subclasses want to override
#            translation mechanism, then they can implement this method
##############################################################################

sub _translate_method_call {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my $command;

    # translate cdef schema into command
    my $schema_obj = $self->get_schema_obj();
    my $cdef_obj
        = $schema_obj->get_command_definition(command => $args{method});
    my $is_tool = $cdef_obj->get_special_properties()->{tool} || 0;
    my $is_path_specifiable
        = $cdef_obj->get_special_properties()->{path_specifiable} || 0;
    if ($is_tool || $is_path_specifiable) {
        unless ((exists $args{tool_dir_path} && defined $args{tool_dir_path})
            || (exists $args{tool_version} && defined $args{tool_version})) {
            my $override = 'tool' if $is_tool;
            $override = 'command' if $is_path_specifiable;
            $override .= '_full_path';
            unless (exists $args{$override}
                && defined $args{$override}) {
                $args{$override} = $self->_read_environment_path_overrides(
                    command => $args{method});

              # It is possible that _read_environment_path_overrides() returns
              # an 'undef'. Since $args{'override} is still present,
              #'$override' is translated as an option, which is incorrect.
              # That is why we need to delete $args{$override} if the value
              # is not defined.
                delete $args{$override} if !defined $args{$override};
            }
        }
    }
    $command = $schema_obj->translate(%args, cdef_obj => $cdef_obj);
    #to execute systemshell commands on console connection
    #prefix 'set -privilege test;system node systemshell -node local -command '
    #before command.
    if ( $self->{session}->{connid} =~ /^console$/ ) {
	$command = 'set -privilege test;system node systemshell -node local -command '.'"'.$command.'" ';
    }
    $Log->exit() if $may_exit;
    return $command;
} ## end sub _translate_method_call

sub _read_environment_path_overrides {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    #Look for parameters of the form NACL_<COMMAND>_<SET>_FULL_PATH
    if (defined Tharn::param(
                  'NACL_'
                . uc $args{command} . '_'
                . uc $self->{actual_set} . '_'
                . 'FULL_PATH'
        )
        ) {
        $Log->exit() if $may_exit;
        return Tharn::param('NACL_'
                . uc $args{command} . '_'
                . uc $self->{actual_set} . '_'
                . 'FULL_PATH');
    }

    #Look for parameters of the form <COMMAND>_FULL_PATH
    if (defined Tharn::param('NACL_' . uc $args{command} . '_' . 'FULL_PATH'))
    {
        $Log->exit() if $may_exit;
        return Tharn::param('NACL_' . uc $args{command} . '_' . 'FULL_PATH');
    }
    $Log->exit() if $may_exit;
    return undef;
} ## end sub _read_environment_path_overrides
###############################################################################
# Method Name: _only_translate_method
# Objective:   Translate the API call into the command invoked on the service
#              and return this without executing it.
# Details:     When the API is prefixed by an "xlate_", it must only translate
#              the API call without executing it. This will come in handy
#              while developing new APISets and for cases where undoing
#              API calls can be cumbersome.
#              This method strips the leading xlate_ prefix, translates
#              the API and returns the command
###############################################################################
sub _only_translate_method {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    $args{method} =~ s/^xlate_//;
    my $command = $self->_translate_method_call(%args);
    $Log->exit() if $may_exit;
    return $command;
} ## end sub _only_translate_method

sub _hammer_pre {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my $cmd_opts = $args{args};

   # Icky backwards compatibility hack made necessary by the inconsistency in
   # the way that 'filercore' option for 'hammer' is treated across Linux,
   # Solaris and Windows APISets. An option called 'filer' needs to be used in
   # conjunction with 'filercore' to specify the filer name.
   # 'filercore' is treated as a Boolean option in Linux. However, it is not a
   # Boolean option in Linux and  Solaris. To remove this inconsistency,
   # filercore has been converted to a non-Boolean option in Linux as well.
   # In order to ensure that scripts that use 'filercore' as a Boolean option
   # do not break for people using Linux APISets, we need this code.

    if (exists $cmd_opts->{filercore}
        && $cmd_opts->{filercore}) {
        $Log->warn(
            "Using -filercore as a Boolean option has been deprecated.\n"
                . "Please pass the <filer name> to filercore like so:\n"
                . '$Api_Set_Obj->hammer(filercore => <filer_name>);');
        $cmd_opts->{filercore} = delete $cmd_opts->{filer}
            if exists $cmd_opts->{filer} && defined $cmd_opts->{filer};
    }
    $Log->exit() if $may_exit;
} ## end sub _hammer_pre

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::CLI - Base for API Sets for CLI commands

=head1 SYNOPSIS

 use NACL::APISet::Host::CLI;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI> is used to connect to and send CLI commands. This
is further subclassed to send CLI commands specifically to Linux, Windows,
Solaris etc.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::TimeoutException

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw CLI commands on the filer. The entire command
line is to be provided exactly as it would be on the filer. This command is
executed and the raw output is returned as a scalar. This is useful for cases
where we want to invoke a command without parsing the output, or for cases
where the particular parser has not been defined yet but we want to see the
raw output.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(command => "system node show -node myvsim1");

=item Arguments

execute_raw_command accepts all arguments accepted by Connectrec->execute().
Of these, only command is mandatory.

=over

=item command

(Mandatory)
The command to be invoked. This is the full command line as it will be entered on the filer.

=back

=item Returns

(Scalar)
Raw output

=back

=head2 get_privilege

This methods returns the current privilege level at which commands are being run

=over

=item Synopsis

$privilege_level = $Api_Set_Obj->get_privilege();

=item Arguments

None.

=item Returns

(Scalar)
Privilege level

=back

=head2 set_privilege

Set the user you wish to run the commands as

=over

=item Synopsis

$Api_Set_Obj->set_privilege('privilege-level' => $privilege);

=item Arguments

=over

=item privilege-level

The name of the user you wish to run the commands as.

=item answer-password

Boolean value indicating whether password prompt must be answered

=back

=item Returns

Nothing.

=back

=head2 clear_privilege

Reset to privilege level to default (current user)

=over

=item Synopsis

$Api_Set_Obj->clear_privilege();

=item Arguments

None.

=item Returns

None.

=back

=head2 answer_password

This methods returns the current setting for answering password prompt

=over

=item Synopsis

$answer = $Api_Set_Obj->answer_password();

=item Arguments

None.

=item Returns

(Scalar)
Value of the current setting of 'answer-password'

=back

=head2 get_allowed_options_for_command

See
L<NACL::APISet::_Mixins::CLI::get_allowed_options_for_command|lib-NACL-APISet-_Mixins-CLI-pm/get_allowed_options_for_command>.

=head2 Command line overrides

Command line overrides can be used when we want to override or change the
path of a particular command or to define the path to the command whose
location (directory path) has not been defined as an environment variable.

These overrides are not applicable when execute_command() is used

There are two ways in which these overrides can be provided and they
are described below:

=over 4

=item NACL_<command>_FULL_PATH

This is applicable to every instance of <command> in the test script.

Example:

Test invocation:
ntest script.thpl CLIENT='some_client' NACL_EXPORTFS_FULL_PATH='/usr/sbin/exportfs'

So, every instance of 'exportfs' in the test script will try to use
/usr/sbin/exportfs irrespective of the 'set' that is used to call this.

=item NACL_<command>_<set>_FULL_PATH

This is applicable to commands that use the specific <set> only.

Example:

Test invocation:
ntest script.thpl CLIENT='some_client' NACL_SIO_WINDOWS_FULL_PATH='X:\bin\sio'

So, every instance of 'sio' command being executed using 'Windows' as the set,
will use the 'X:\bin\sio' as the path.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=over

=item NACL_REFRESH_STALE_CONNECTIONS_RETRY

  parameter NACL_REFRESH_STALE_CONNECTIONS_RETRY controls if retry on failure of connection is required
  possible values 0/1
  It will retry if NACL_REFRESH_STALE_CONNECTIONS_RETRY is 1

=back

=cut
