#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/lsdev.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX lsdev command
##
## @description
##    This is a library to parse the raw output of AIX lsdev command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lsdev" will lsdev()
##
## @synopsis
##    use lsdev;
##
##    lsdev(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::lsdev;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::lsdev
##
## @summary Parses the raw output from the "lsdev" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lsdev with any of its options as described in the
##    corresponding cdef file 'lsdev.cdefs'
## @synopsis
##      lsdev(%args);
## INPUTS TO METHOD CALL 'lsdev' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-P' </td> <td> 'predefined' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'parent' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'column-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'headers' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'customized' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'name' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sub-class' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'state' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'class' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lsdev
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;L2cache0   Available               L2 Cache
##&nbsp;cd0        Available 08-08-00      IDE DVD-ROM Drive
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Available',
##&nbsp;            'name' => 'L2cache0',
##&nbsp;            'description' => 'L2 Cache'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'location' => '08-08-00',
##&nbsp;            'status' => 'Available',
##&nbsp;            'name' => 'cd0',
##&nbsp;            'description' => 'IDE DVD-ROM Drive'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsdev -P
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;3dx            03000008        usbif          USB 3D mouse
##&nbsp;PCM            ds4k-ui         fcp            DS3/4/5K PCM User Interface
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'subclass' => 'usbif',
##&nbsp;            'type' => '03000008',
##&nbsp;            'class' => '3dx',
##&nbsp;            'description' => 'USB 3D mouse'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subclass' => 'fcp',
##&nbsp;            'type' => 'ds4k-ui',
##&nbsp;            'class' => 'PCM',
##&nbsp;            'description' => 'DS3/4/5K PCM User Interface'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsdev -r class
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;adapter
##&nbsp;bus
##&nbsp;cdrom
##&nbsp;container
##&nbsp;disk
##&nbsp;driver
##&nbsp;if
##&nbsp;iocp
##&nbsp;lft
##&nbsp;logical_volume
##&nbsp;lvm
##&nbsp;memory
##&nbsp;planar
##&nbsp;processor
##&nbsp;pseudo
##&nbsp;pty
##&nbsp;rcm
##&nbsp;sys
##&nbsp;tcpip
##&nbsp;tty
##&nbsp;usb
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          'adapter',
##&nbsp;          'bus',
##&nbsp;          'cdrom',
##&nbsp;          'container',
##&nbsp;          'disk',
##&nbsp;          'driver',
##&nbsp;          'if',
##&nbsp;          'iocp',
##&nbsp;          'lft',
##&nbsp;          'logical_volume',
##&nbsp;          'lvm',
##&nbsp;          'memory',
##&nbsp;          'planar',
##&nbsp;          'processor',
##&nbsp;          'pseudo',
##&nbsp;          'pty',
##&nbsp;          'rcm',
##&nbsp;          'sys',
##&nbsp;          'tcpip',
##&nbsp;          'tty',
##&nbsp;          'usb'
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsdev -F 'class description subclass'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;memory         L2 Cache                                            sys
##&nbsp;bsrpd          N/A                                                 node
##&nbsp;cdrom          IDE DVD-ROM Drive                                   ide
##&nbsp;logical_volume Volume group                                        vgsubclass
##&nbsp;if             Standard Ethernet Network Interface                 EN
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'subclass' => 'sys',
##&nbsp;            'class' => 'memory',
##&nbsp;            'description' => 'L2 Cache'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subclass' => 'node',
##&nbsp;            'class' => 'bsrpd',
##&nbsp;            'description' => 'N/A'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subclass' => 'ide',
##&nbsp;            'class' => 'cdrom',
##&nbsp;            'description' => 'IDE DVD-ROM Drive'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subclass' => 'vgsubclass',
##&nbsp;            'class' => 'logical_volume',
##&nbsp;            'description' => 'Volume group'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subclass' => 'EN',
##&nbsp;            'class' => 'if',
##&nbsp;            'description' => 'Standard Ethernet Network Interface'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lsdev {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @head_temp );
    my $i     = 0;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @no_option_header_arr  = ( "name", "status", "location", "description" );
    my @option_p_head_arr  = ( "class", "type", "subclass", "description" );
    my $c     = 0;

    if ( $cmd =~ /-F\s+(\'|\")(.*)(\'|\")/ ) {
        my $main = $2;
        ## To get the user delimiters
        my (@delimit, @pos_delim, $i, $j);
        my $temp_main = $main;
        while($temp_main =~ /\W/) {
            if($temp_main =~ /\w+(\W+)/) {
                push(@delimit,$1);
                $temp_main =~ s/$_//;
            }
        }
        ## Extracting the head based on the delimiter
        ## The command can have more than one delimiter
        for($i = 0; $i <= $#delimit; $i++) {
            $main =~ s/$delimit[$i]/ /;
        }

        @head = split(/\s+/, $main);
        @head_temp = split(/\s+/, $main);
        my @total_vars;

        ##Getting the position of the delimiter
        foreach my $line (@lines) {
            my @total_values = split(/\s\s+/, $line);
            if($#total_values == $#head) {
                @total_vars = split("",$line);
                for($j = 0; $j < $#total_vars; $j++) {
                    if($total_vars[$j-1] =~ /\s/ && $total_vars[$j] =~ /\s/ && $total_vars[$j+1] =~ /\w+/) {
                        push(@pos_delim, $j+1);
                    }
                }
                last;
            }
        }
        my $final = 0;
        my $loc   = undef;
        ## Getting the position for location
        for ( my $k = 0; $k <= $#head; $k++ ) {
            if ( $head[$k] eq "location" ) {
                $loc = $k;
            }
        }
        my (@temp, $head_loc, @each_values);
        my $start = 0;
        $start = 1 if ( $cmd =~ /-H/ );
        for ( $j = $start; $j <= $#lines; $j++ ) {
            @head = ();
            push(@head, @head_temp);
            for($i  = 0; $i <= $#delimit; $i++) {
                $lines[$j] =~ s/$delimit[$i]/ /;
            }
            my $curr_line;
            my $st_pos = 0;
            ##Each record is made such that its value delimited by 2 or more spaces
            for(my $y = 0; $y <= $#pos_delim; $y++) {
                $st_pos = $pos_delim[$y-1]  if($y);
                my $len = $pos_delim[$y] - $st_pos;
                $curr_line = $curr_line." ".substr($lines[$j],$st_pos,$len);
            }
            my @total_vars = split("", $lines[$j]);
            $curr_line = $curr_line." ".substr($lines[$j],$pos_delim[-1], $#total_vars);
            $curr_line =~ s/^\s//;
            @temp = split(/\s\s+/, $curr_line);
            if ( defined $loc ) {
                #If the value for location not present for specific record,
                #then that record header "location" is removed from common header list
                if ( $temp[$loc] !~ /^\d+\w+\-\d+/ ) {
                    for ( my $r = $loc; $r <= $#head; $r++ ) {
                        $head[$r] = $head[ $r + 1 ];
                    }
                    pop(@head);
                }
            }
            for ( my $w = 0; $w <= $#head; $w++ ) {
                $result_array[$c]->{ $head[$w] } = $temp[$w];
            }
            $c++;
        }
        return \@result_array;
    }

    foreach my $line (@lines) {
        my @values = split( /\s+/, $line );
        my $start = 0;
        if ( $cmd eq "lsdev -H" ) {
            $start = 1;
            ## Repopulating the header if present
            @no_option_header_arr =  split(/\s+/, $lines[0]);
            @option_p_head_arr =  split(/\s+/, $lines[0]);
        }
        $start = 1 if ( $cmd eq "lsdev -H" );

        if ( $cmd =~ /-r/ ) {
            $result_array[$c] = $line;
        } elsif ( $cmd =~ /-P/ ) {
            for ( my $i = $start; $i <= $#values; $i++ ) {
                $values[3] = join( " ", splice( @values, 3, $#values ) );
                $result_array[$c]->{ $option_p_head_arr[$i] } = $values[$i]
                    if ( $values[$i] );
            }
        } else {
            for ( my $i = $start; $i <= $#values; $i++ ) {
                if ( $values[2] !~ /^\d+/ ) {
                    $values[3] = join( " ", splice( @values, 2, $#values ) );
                    $values[3] =~ s/^\s+//;
                    $values[2] = "";
                } else {
                    $values[3] = join( " ", splice( @values, 3, $#values ) );
                    $values[3] =~ s/^\s+//;
                }
                $result_array[$c]->{ $no_option_header_arr[$i] } = $values[$i]
                    if ( $values[$i] );
            }
        }
        $c++;
    }
    return \@result_array;
} ## end sub lsdev

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(lsdev:|Error:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
