#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/lslpp.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX lslpp command
##
## @description
##    This is a library to parse the raw output of AIX lslpp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lslpp" lslpp()
##
## @synopsis
##    use lslpp;
##
##    lslpp(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::lslpp;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException;

## @name ::lslpp
##
## @summary Parses the raw output from the "lslpp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lslpp with any of its options as described in the
##    corresponding cdef file 'lslpp.cdefs'
## @synopsis
##      lslpp(%args);
## INPUTS TO METHOD CALL 'lslpp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-L' </td> <td> 'displays-specified-fileset' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'files' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'displays-installed-filesets' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filesetname' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'efixes-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'limits-the-inputs' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'history-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'product-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'filesets-most-recent-level' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'displays-dependents-filesets' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vendor-database-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'fileset-list-source' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr>
## <tr> <td> '-E' </td> <td> 'license-agreements' </td> <td> '1' </td> </tr>
## <tr> <td> '-J' </td> <td> 'list-output-format' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'additional-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-O' </td> <td> 'lists-specified-fileset-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'lists-fileset-that-owns' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'displays-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'requisites-installed-filesets' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lslpp -E Java14.sdk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;====================================================================
##&nbsp;                   Installed License Agreements
##&nbsp;====================================================================
##&nbsp;
##&nbsp;The installed software listed below contains license agreements
##&nbsp;which have been accepted.
##&nbsp;
##&nbsp;--------------------------------------------------------------------
##&nbsp;--------------------------------------------------------------------
##&nbsp;Fileset:  Java14.sdk
##&nbsp;Product ID:  5648-C9802
##&nbsp;Description:  Java SDK 32-bit
##&nbsp;Agreement File:  /usr/swlag/%L/Java14.la
##&nbsp;Date:  Wed Sep  7 23:21:07 sdafsaf 2011
##&nbsp;Machine ID:  00579F5A4C00
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'agreement_file' => '/usr/swlag/%L/Java14.la',
##&nbsp;            'product_id' => '5648-C9802',
##&nbsp;            'date' => 'Wed Sep  7 23:21:07 sdafsaf 2011',
##&nbsp;            'fileset' => 'Java14.sdk',
##&nbsp;            'machine_id' => '00579F5A4C00',
##&nbsp;            'description' => 'Java SDK 32-bit'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -l Java14.sdk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Fileset                      Level  State      Description
##&nbsp;  ----------------------------------------------------------------------------
##&nbsp;Path: /usr/lib/objrepos
##&nbsp;  Java14.sdk               1.4.2.420  COMMITTED  Java SDK 32-bit
##&nbsp;
##&nbsp;Path: /etc/objrepos
##&nbsp;  Java14.sdk               1.4.2.420  COMMITTED  Java SDK 32-bit
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'level' => '1.4.2.420',
##&nbsp;            'fileset' => 'Java14.sdk',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'description' => 'Java SDK 32-bit',
##&nbsp;            'state' => 'COMMITTED'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'level' => '1.4.2.420',
##&nbsp;            'fileset' => 'Java14.sdk',
##&nbsp;            'path' => '/etc/objrepos',
##&nbsp;            'description' => 'Java SDK 32-bit',
##&nbsp;            'state' => 'COMMITTED'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -L xlfrte.aix53
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Fileset                      Level  State  Type  Description (Uninstaller)
##&nbsp; ----------------------------------------------------------------------------
##&nbsp;xlfrte.msg.en_US          13.1.0.5    C     F    XL Fortran Runtime Messages -
##&nbsp;                                                    U.S. English
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp; State codes:
##&nbsp; A -- Applied.
##&nbsp; B -- Broken.
##&nbsp; C -- Committed.
##&nbsp; E -- EFIX Locked.
##&nbsp; O -- Obsolete.  (partially migrated to newer version)
##&nbsp; ? -- Inconsistent State...Run lppchk -v.
##&nbsp;
##&nbsp; Type codes:
##&nbsp; F -- Installp Fileset
##&nbsp; P -- Product
##&nbsp; C -- Component
##&nbsp; T -- Feature
##&nbsp; R -- RPM Package
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'Type' => 'Installp Fileset',
##&nbsp;            'Level' => '13.1.0.5',
##&nbsp;            'Description (Uninstaller)' => 'XL Fortran Runtime Messages - U.S. English',
##&nbsp;            'State' => 'Committed',
##&nbsp;            'Fileset' => 'xlfrte.msg.en_US'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -da Java14.sdk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Fileset               Dependents                            Dependents State
##&nbsp;  ----------------------------------------------------------------------------
##&nbsp;<Fileset> is a requisite of <Dependents>
##&nbsp;Path: /usr/lib/objrepos
##&nbsp;  Java14.sdk 1.4.2.20   sysmgtlib.framework.core 5.3.0.40     COMMITTED
##&nbsp;                        sysmgt.sguide.rte 5.3.0.0             COMMITTED
##&nbsp;                        csm.gui.dcem 1.7.1.0                  COMMITTED
##&nbsp;                        sysmgt.websm.framework 5.3.0.40       COMMITTED
##&nbsp;                        lum.base.gui 5.1.0.0                  COMMITTED
##&nbsp;
##&nbsp;Path: /etc/objrepos
##&nbsp;  Java14.sdk 1.4.2.20   sysmgt.websm.framework 5.3.0.40       COMMITTED
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.20',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'dependencies_info' => [
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'sysmgtlib.framework.core 5.3.0.40'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'sysmgt.sguide.rte 5.3.0.0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'csm.gui.dcem 1.7.1.0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'sysmgt.websm.framework 5.3.0.40'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'lum.base.gui 5.1.0.0'
##&nbsp;                                     }
##&nbsp;                                   ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.20',
##&nbsp;            'path' => '/etc/objrepos',
##&nbsp;            'dependencies_info' => [
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'sysmgt.websm.framework 5.3.0.40'
##&nbsp;                                     }
##&nbsp;                                   ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -p Java14.sdk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Fileset               Requisites
##&nbsp;  ----------------------------------------------------------------------------
##&nbsp;Path: /usr/lib/objrepos
##&nbsp;  Java14.sdk 1.4.2.20   *coreq  bos.rte  5.1.0.50
##&nbsp;                        *coreq  bos.rte.libpthreads 5.1.0.50
##&nbsp;                        *coreq  bos.net.tcp.client 5.1.0.50
##&nbsp;                        *coreq  bos.rte.libc 5.1.0.50
##&nbsp;                        *ifreq bos.iconv.Zh_TW (5.1.0.0) 5.1.0.35
##&nbsp;                        *ifreq bos.loc.com.utf (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.iso.Zh_TW (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.iso.zh_CN (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.iso.zh_TW (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.utf.ZH_CN (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.utf.ZH_TW (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.loc.utf.KO_KR (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.mp (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.mp64 (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.up (5.1.0.0) 5.1.0.50
##&nbsp;                        *ifreq   bos.mp (5.2.0.0) 5.2.0.10
##&nbsp;                        *ifreq   bos.mp64 (5.2.0.0) 5.2.0.10
##&nbsp;                        *ifreq   bos.up (5.2.0.0) 5.2.0.10
##&nbsp;                        *ifreq   bos.net.tcp.client (5.2.0.0) 5.2.0.10
##&nbsp;                        *ifreq   X11.base.rte (5.1.0.0)  5.1.0.50
##&nbsp;                        *ifreq   X11.base.lib (5.1.0.0)  5.1.0.50
##&nbsp;                        *ifreq   X11.motif.lib   (5.1.0.0) 5.1.0.50
##&nbsp;  Java14.sdk 1.4.2.420  NONE
##&nbsp;
##&nbsp;Path: /etc/objrepos
##&nbsp;  Java14.sdk 1.4.2.20   NONE
##&nbsp;  Java14.sdk 1.4.2.420  NONE
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'requisites' => [
##&nbsp;                              '*coreq  bos.rte  5.1.0.50',
##&nbsp;                              '*coreq bos.rte.libpthreads 5.1.0.50',
##&nbsp;                              '*coreq bos.net.tcp.client 5.1.0.50',
##&nbsp;                              '*coreq bos.rte.libc 5.1.0.50',
##&nbsp;                              '*ifreq bos.iconv.Zh_TW (5.1.0.0) 5.1.0.35',
##&nbsp;                              '*ifreq bos.loc.com.utf (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.iso.Zh_TW (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.iso.zh_CN (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.iso.zh_TW (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.utf.ZH_CN (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.utf.ZH_TW (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.loc.utf.KO_KR (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.mp (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.mp64 (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.up (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq bos.mp (5.2.0.0) 5.2.0.10',
##&nbsp;                              '*ifreq bos.mp64 (5.2.0.0) 5.2.0.10',
##&nbsp;                              '*ifreq bos.up (5.2.0.0) 5.2.0.10',
##&nbsp;                              '*ifreq bos.net.tcp.client (5.2.0.0) 5.2.0.10',
##&nbsp;                              '*ifreq X11.base.rte (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq X11.base.lib (5.1.0.0) 5.1.0.50',
##&nbsp;                              '*ifreq X11.motif.lib (5.1.0.0) 5.1.0.50'
##&nbsp;                            ],
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.20',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'requisites' => 'NONE',
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.420',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'requisites' => 'NONE',
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.20',
##&nbsp;            'path' => '/etc/objrepos'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'requisites' => 'NONE',
##&nbsp;            'fileset' => 'Java14.sdk 1.4.2.420',
##&nbsp;            'path' => '/etc/objrepos'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -p -c X11.apps.rte
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#Path:Fileset:Requisites
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:*prereq bos.rte 5.2.0.0 *prereq X11.base.common 5.2.0.0 *coreq X11.base.lib 5.2.0.0 *coreq X11.motif.mwm 5.2.0.0 *coreq X11.apps.aixterm 5.2.0.0 *coreq X11.apps.util 5.2.0.0 *coreq X11.loc.en_US.base.rte 5.2.0.0
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.10.0:*prereq bos.rte.install 5.3.10.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'requisites' => [
##&nbsp;                              '*prereq bos.rte 5.2.0.0',
##&nbsp;                              '*prereq X11.base.common 5.2.0.0',
##&nbsp;                              '*coreq X11.base.lib 5.2.0.0',
##&nbsp;                              '*coreq X11.motif.mwm 5.2.0.0',
##&nbsp;                              '*coreq X11.apps.aixterm 5.2.0.0',
##&nbsp;                              '*coreq X11.apps.util 5.2.0.0',
##&nbsp;                              '*coreq X11.loc.en_US.base.rte 5.2.0.0'
##&nbsp;                            ],
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.0.30',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'requisites' => [
##&nbsp;                              '*prereq bos.rte.install 5.3.10.0'
##&nbsp;                            ],
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.10.0',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -p -a -c X11.apps.rte
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#Path:Fileset:PTF Id:State:Requisites
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30::COMMITTED:*prereq bos.rte 5.2.0.0 *prereq X11.base.common 5.2.0.0 *coreq X11.base.lib 5.2.0.0 *coreq X11.motif.mwm 5.2.0.0 *coreq X11.apps.aixterm 5.2.0.0 *coreq X11.apps.util 5.2.0.0 *coreq X11.loc.en_US.base.rte 5.2.0.0
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.10.0::COMMITTED:*prereq bos.rte.install 5.3.10.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'requisites' => [
##&nbsp;                              '*prereq bos.rte 5.2.0.0',
##&nbsp;                              '*prereq X11.base.common 5.2.0.0',
##&nbsp;                              '*coreq X11.base.lib 5.2.0.0',
##&nbsp;                              '*coreq X11.motif.mwm 5.2.0.0',
##&nbsp;                              '*coreq X11.apps.aixterm 5.2.0.0',
##&nbsp;                              '*coreq X11.apps.util 5.2.0.0',
##&nbsp;                              '*coreq X11.loc.en_US.base.rte 5.2.0.0'
##&nbsp;                            ],
##&nbsp;            'ptf_id' => '',
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.0.30',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'state' => 'COMMITTED'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'requisites' => [
##&nbsp;                              '*prereq bos.rte.install 5.3.10.0'
##&nbsp;                            ],
##&nbsp;            'ptf_id' => '',
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.10.0',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'state' => 'COMMITTED'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -d -a -c X11.apps.rte
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#Path:Fileset:Dependents:Dependents State
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:X11.msg.en_US.apps.rte 5.3.0.0:COMMITTED
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:X11.fnt.iso_T1 5.3.0.0:COMMITTED
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:X11.fnt.iso1 5.3.0.0:COMMITTED
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:X11.fnt.defaultFonts 5.3.0.0:COMMITTED
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:X11.fnt.coreX 5.3.0.0:COMMITTED
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.0.30',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'dependencies_info' => [
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'X11.fnt.coreX 5.3.0.0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'X11.fnt.iso_T1 5.3.0.0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'X11.fnt.iso1 5.3.0.0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'dependents_state' => 'COMMITTED',
##&nbsp;                                       'dependents' => 'X11.fnt.defaultFonts 5.3.0.0'
##&nbsp;                                     }
##&nbsp;                                   ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -f -c X11.apps.rte
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#Path:Fileset:File
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/xclock
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/xinit
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xmodmap/POSIX -> /usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/xrdb
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/mkfontdir
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/Xdefaults.wheelmouse
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/querykbd
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/lib/X11/app-defaults/XClock
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/aixconsole
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/lib/X11/app-defaults/Aixconsole
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/lib/X11/app-defaults
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/xmodmap
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xmodmap
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/bin/xset
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xmodmap/C -> /usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xinitrc
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xserverrc
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/vtkeyboard
##&nbsp;/usr/lib/objrepos:X11.apps.rte 5.3.0.30:/usr/lpp/X11/defaults/xstartterm
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file' => [
##&nbsp;                        '/usr/lpp/X11/defaults',
##&nbsp;                        '/usr/lpp/X11/bin/xclock',
##&nbsp;                        '/usr/lpp/X11/bin/xinit',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/POSIX -> /usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/bin/xrdb',
##&nbsp;                        '/usr/lpp/X11/bin/mkfontdir',
##&nbsp;                        '/usr/lpp/X11/defaults/Xdefaults.wheelmouse',
##&nbsp;                        '/usr/lpp/X11/bin/querykbd',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults/XClock',
##&nbsp;                        '/usr/lpp/X11/bin/aixconsole',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults/Aixconsole',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults',
##&nbsp;                        '/usr/lpp/X11/bin/xmodmap',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap',
##&nbsp;                        '/usr/lpp/X11/bin/xset',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/C -> /usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/defaults/xinitrc',
##&nbsp;                        '/usr/lpp/X11/defaults/xserverrc',
##&nbsp;                        '/usr/lpp/X11/defaults/vtkeyboard',
##&nbsp;                        '/usr/lpp/X11/defaults/xstartterm'
##&nbsp;                      ],
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.0.30',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -f  X11.apps.rte
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Fileset               File
##&nbsp;  ----------------------------------------------------------------------------
##&nbsp;Path: /usr/lib/objrepos
##&nbsp;  X11.apps.rte 5.3.0.30
##&nbsp;                        /usr/lpp/X11/defaults
##&nbsp;                        /usr/lpp/X11/bin/xclock
##&nbsp;                        /usr/lpp/X11/bin/xinit
##&nbsp;                        /usr/lpp/X11/defaults/xmodmap/POSIX -> /usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;                        /usr/lpp/X11/bin/xrdb
##&nbsp;                        /usr/lpp/X11/bin/mkfontdir
##&nbsp;                        /usr/lpp/X11/defaults/Xdefaults.wheelmouse
##&nbsp;                        /usr/lpp/X11/bin/querykbd
##&nbsp;                        /usr/lpp/X11/lib/X11/app-defaults/XClock
##&nbsp;                        /usr/lpp/X11/bin/aixconsole
##&nbsp;                        /usr/lpp/X11/lib/X11/app-defaults/Aixconsole
##&nbsp;                        /usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;                        /usr/lpp/X11/lib/X11/app-defaults
##&nbsp;                        /usr/lpp/X11/bin/xmodmap
##&nbsp;                        /usr/lpp/X11/defaults/xmodmap
##&nbsp;                        /usr/lpp/X11/bin/xset
##&nbsp;                        /usr/lpp/X11/defaults/xmodmap/C -> /usr/lpp/X11/defaults/xmodmap/en_US
##&nbsp;                        /usr/lpp/X11/defaults/xinitrc
##&nbsp;                        /usr/lpp/X11/defaults/xserverrc
##&nbsp;                        /usr/lpp/X11/defaults/vtkeyboard
##&nbsp;                        /usr/lpp/X11/defaults/xstartterm
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file' => [
##&nbsp;                        '/usr/lpp/X11/defaults',
##&nbsp;                        '/usr/lpp/X11/bin/xclock',
##&nbsp;                        '/usr/lpp/X11/bin/xinit',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/POSIX -> /usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/bin/xrdb',
##&nbsp;                        '/usr/lpp/X11/bin/mkfontdir',
##&nbsp;                        '/usr/lpp/X11/defaults/Xdefaults.wheelmouse',
##&nbsp;                        '/usr/lpp/X11/bin/querykbd',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults/XClock',
##&nbsp;                        '/usr/lpp/X11/bin/aixconsole',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults/Aixconsole',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/lib/X11/app-defaults',
##&nbsp;                        '/usr/lpp/X11/bin/xmodmap',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap',
##&nbsp;                        '/usr/lpp/X11/bin/xset',
##&nbsp;                        '/usr/lpp/X11/defaults/xmodmap/C -> /usr/lpp/X11/defaults/xmodmap/en_US',
##&nbsp;                        '/usr/lpp/X11/defaults/xinitrc',
##&nbsp;                        '/usr/lpp/X11/defaults/xserverrc',
##&nbsp;                        '/usr/lpp/X11/defaults/vtkeyboard',
##&nbsp;                        '/usr/lpp/X11/defaults/xstartterm'
##&nbsp;                      ],
##&nbsp;            'fileset' => 'X11.apps.rte 5.3.0.30',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -i X11.adt.bitmaps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                        Vendor
##&nbsp;  Fileset               Code    Product Id  Feature Id  Package Name
##&nbsp;  ----------------------------------------------------------------------------
##&nbsp;Path: /usr/lib/objrepos
##&nbsp;  X11.adt.bitmaps 5.3.0.0
##&nbsp;                                5765-G0300              X11.adt
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'product_id' => '5765-G0300',
##&nbsp;            'package_name' => 'X11.adt',
##&nbsp;            'fileset' => 'X11.adt.bitmaps 5.3.0.0',
##&nbsp;            'path' => '/usr/lib/objrepos'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lslpp -l -c  X11.adt.bitmaps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#Fileset:Level:PTF Id:State:Type:Description:EFIX Locked
##&nbsp;/usr/lib/objrepos:X11.adt.bitmaps:5.3.0.0::COMMITTED:I:AIXwindows Application Development Toolkit Bitmap Files :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'level' => '5.3.0.0',
##&nbsp;            'efix_locked' => undef,
##&nbsp;            'ptf_id' => '',
##&nbsp;            'type' => 'I',
##&nbsp;            'fileset' => 'X11.adt.bitmaps',
##&nbsp;            'path' => '/usr/lib/objrepos',
##&nbsp;            'description' => 'AIXwindows Application Development Toolkit Bitmap Files ',
##&nbsp;            'state' => 'COMMITTED'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lslpp {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @head1, $c1, $path, @values, $temp );
    my $k = -1;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    if ( $cmd =~ /-d|-p/ ) {
        @lines = get_lines_tree(
            raw_output      => $output,
            response_object => $resp
        );
    }
    if ( $cmd =~ /-c/ ) {
        my $c = 0;
        my $k = -1;
        my $y = 0;
        my ( @values, @values1 );
        if ( $cmd =~ /-p/ ) {
            my $temp;
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @head = split( /:/, $lines[0] );
                if ( $head[0] =~ s/#// ) { }
                my @temp = split( /:/, $lines[$i] );
                @values = split( /\s\*/, $temp[-1] );
                for ( my $j = 0; $j <= $#head - 1; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $temp[$j];
                    for ( my $k = 0; $k <= $#values; $k++ ) {
                        if ( $values[$k] !~ /\*.*/ ) {
                            $temp = "*" . $values[$k];
                            $result_array[$c]
                                ->{ to_key_name( item => $head[-1] ) }->[$k]
                                = $temp;
                        } else {
                            $result_array[$c]
                                ->{ to_key_name( item => $head[-1] ) }->[$k]
                                = $values[$k];
                        }

                    }
                }
                $c++;
            }
        } elsif ( $cmd =~ /-d/ ) {
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @head = split( /:/, $lines[0] );
                if ( $head[0] =~ s/#// ) { }
                @values  = split( /:/, $lines[$i] );
                @values1 = split( /:/, $lines[ $i + 1 ] );
                if ( $values[0] eq $values1[0] ) {
                    $k++;
                    $result_array[$c]->{ to_key_name( item => $head[0] ) }
                        = $values[0];
                    $result_array[$c]->{ to_key_name( item => $head[1] ) }
                        = $values[1];
                    if ( $values[2] ) {
                        $result_array[$c]->{dependencies_info}->[$k]
                            ->{ to_key_name( item => $head[2] ) }
                            = $values[2];
                    }
                    if ( $values[3] ) {
                        $result_array[$c]->{dependencies_info}->[$k]
                            ->{ to_key_name( item => $head[3] ) }
                            = $values[3];
                    }
                } elsif ( $values[0] ne $values1[0] ) {
                    $result_array[$c]->{ to_key_name( item => $head[0] ) }
                        = $values[0];
                    $result_array[$c]->{ to_key_name( item => $head[1] ) }
                        = $values[1];
                    if ( $values[2] ) {
                        $result_array[$c]->{dependencies_info}->[$y]
                            ->{ to_key_name( item => $head[2] ) }
                            = $values[2];
                    }
                    if ( $values[3] ) {
                        $result_array[$c]->{dependencies_info}->[$y]
                            ->{ to_key_name( item => $head[3] ) }
                            = $values[3];
                    }
                    $c++;
                    $k = 0;
                }
            }
        } elsif ( $cmd =~ /-f/ ) {
            my $c = 0;
            my $k = 0;
            my $y = 0;
            my ( @values, @values1 );
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @head = split( /:/, $lines[0] );
                if ( $head[0] =~ s/#// ) { }
                @values  = split( /:/, $lines[$i] );
                @values1 = split( /:/, $lines[ $i + 1 ] );
                if ( $values[0] eq $values1[0] ) {
                    $result_array[$c]->{ to_key_name( item => $head[0] ) }
                        = $values[0];
                    $result_array[$c]->{ to_key_name( item => $head[1] ) }
                        = $values[1];
                    $result_array[$c]->{ to_key_name( item => $head[2] ) }
                        ->[$k] = $values[2];
                    $k++;
                } elsif ( $values[0] ne $values1[0] ) {
                    $result_array[$c]->{ to_key_name( item => $head[0] ) }
                        = $values[0];
                    $result_array[$c]->{ to_key_name( item => $head[1] ) }
                        = $values[1];
                    $result_array[$c]->{ to_key_name( item => $head[2] ) }
                        ->[$k] = $values[2];
                    $c++;
                    $k = 0;
                }
            }
        } elsif ( $cmd =~ /-l/ ) {
            my $c    = 0;
            $path = 'path';
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @head = split( /:/, $lines[0] );
                if ( $head[0] =~ s/#// ) { }
                unshift( @head, $path );
                @values = split( /:/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $c++;
            }
        } else {
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @head = split( /:/, $lines[0] );
                if ( $head[0] =~ s/#// ) { }
                @values = split( /:/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $c++;
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-i/ ) {
        my $c = -1;
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            my @values = split( /\s\s+/, $lines[$i] );
            if ( $values[0] =~ /^\w+\..*\s+(\d+\.){1,}\d+$/ ) {
                $c++;
                $result_array[$c]->{path}    = $path;
                $result_array[$c]->{fileset} = $values[0];
            }
            my $file;
            if ( $c >= 0 ) {
                for ( my $j = 0; $j < $#values; $j++ ) {
                    if ( $values[$j] =~ /^\w+\-\w+$/ ) {
                        $result_array[$c]->{product_id} = $values[$j];
                    } elsif ( $values[$j] =~ /^\d+$/ ) {
                        $result_array[$c]->{feature_id} = $values[$j];
                    }
                    $result_array[$c]->{package_name} = $values[-1];
                }
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-h/ ) {
        my $c = 0;
        my $k = 0;
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            @head   = split( /\s+/, $lines[0] );
            @values = split( /\s+/, $lines[$i] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            if (   $#values == 0
                && $lines[$i] !~ $lines[$0]
                && $lines[$i] !~ /----------------------..*/ ) {
                $c++;
                $k                              = 0;
                $result_array[$c]->{path}       = $path;
                $result_array[$c]->{ $head[0] } = $values[0];
            } elsif ( $lines[$i] =~ /\d+\.\d+/ ) {
                for ( my $j = 1; $j <= $#head; $j++ ) {
                    $result_array[$c]->{fileset_info}->[$k]->{ $head[$j] }
                        = $values[ $j - 1 ];
                }
                $k++;
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-l/ ) {
        my $c = -1;
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            my @values = split( /\s\s+/, $lines[$i] );
            my @head   = split( /\s+/,   $lines[0] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            if ( $#values > 2 ) {
                $c++;
                $result_array[$c]->{path} = $path;
                $temp = $values[-1];
                while ($lines[ $i + 1 ] =~ /^[A-Z]\s*.*\w*/
                    && $lines[ $i + 1 ] != /.*\d+.*/ ) {
                    $temp = $temp . " " . $lines[ $i + 1 ];
                    $i++;
                }
                for ( my $j = 0; $j < ( @values - 1 ); $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $result_array[$c]->{ to_key_name( item => $head[3] ) }
                    = $temp;
            }
            if (   $lines[$i] =~ /^[A-Z]+\w+\..*\w+$/
                && $lines[ $i + 1 ] =~ /^\d+.*/ ) {
                $c++;
                $result_array[$c]->{path}    = $path;
                $result_array[$c]->{fileset} = $lines[$i];
                my @val = split( /\s\s+/, $lines[ $i + 1 ] );
                $temp = $val[-1];
                while ( $lines[ $i + 2 ] =~ /^[A-Z]+.*\w*/ ) {
                    $temp = $temp . " " . $lines[ $i + 2 ];
                    $i++;
                }
                for ( my $j = 1; $j < @val; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $val[ $j - 1 ];
                }
                $result_array[$c]->{ to_key_name( item => $head[3] ) }
                    = $temp;
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-L/ ) {
        my $c = -1;
        my %states;
        my %type_codes;
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            my @values = split( /\s\s+/, $lines[$i] );
            my @head   = split( /\s\s+/, $lines[0] );
            for ( my $i = 2; $i <= $#lines; $i++ ) {
                if ( $lines[$i] =~ /State codes:/ ) {
                    while ( $lines[ $i + 1 ] !~ /^Type codes:/ ) {
                        my @values = split( /\s*--\s*/, $lines[ $i + 1 ] );
                        $states{"$values[0]"} = $values[1];
                        $i++;
                    }
                } elsif ( $lines[$i] =~ /Type codes:/ ) {
                    while ( $#lines > $i ) {
                        my @values = split( /\s*--\s*/, $lines[ $i + 1 ] );
                        $type_codes{"$values[0]"} = $values[1];
                        $i++;
                    }
                }
            }

            if ( $#values > 3 && $lines[$i] !~ /.*--|.*:$/ ) {
                $c++;
                $temp = $values[-1];
                while ( $lines[ $i + 1 ] =~ /^[A-Z]+/ ) {
                    my @t1 = split( /\s\s+/, $lines[ $i + 1 ] );
                    if ( $#t1 == 0 ) {
                        $temp = $temp . " " . $lines[ $i + 1 ];
                        $i++;
                    }
                }
                for ( my $j = 0; $j < ( @values - 1 ); $j++ ) {
                    if ( $head[$j] =~ /State/ ) {
                        $values[$j] = $states{ $values[$j] };
                    } elsif ( $head[$j] =~ /Type/ ) {
                        $values[$j] = $type_codes{ $values[$j] };
                    }
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $result_array[$c]->{ to_key_name( item => $head[4] ) }
                    = $temp;
            }
            if (   $lines[$i] =~ /^\w+\..*\w+$/
                && $lines[ $i + 1 ] =~ /^\d+.*/
                && $lines[ $i + 1 ] !~ /--|:$/ ) {
                $c++;
                $result_array[$c]->{fileset} = $lines[$i];
                my @val = split( /\s\s+/, $lines[ $i + 1 ] );
                $temp = $val[-1];
                while ($lines[ $i + 2 ] =~ /^[A-Z]+.*/
                    && $lines[ $i + 2 ] !~ /--|:$/ ) {
                    $temp = $temp . " " . $lines[ $i + 2 ];
                    $i++;
                }
                for ( my $j = 1; $j < @val; $j++ ) {
                    if ( $head[$j] =~ /State/ ) {
                        $val[ $j - 1 ] = $states{ $val[ $j - 1 ] };
                    } elsif ( $head[$j] =~ /Type/ ) {
                        $val[ $j - 1 ] = $type_codes{ $val[ $j - 1 ] };
                    }
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $val[ $j - 1 ];
                }
                $result_array[$c]->{ to_key_name( item => $head[4] ) }
                    = $temp;
            }

        }
        return \@result_array;
    }
    if ( $cmd =~ /-f/ ) {
        my $c = -1;
        my $k = -1;
        my $fileset;
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            my @head = split( /\s+/, $lines[0] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            my @values = split( /\s\s+/, $lines[$i] );
            if ( $#values == 1 ) {
                $c++;
                $k                        = 0;
                $fileset                  = $values[0];
                $result_array[$c]->{path} = $path;
                $result_array[$c]->{ to_key_name( item => $head[0] ) }
                    = $fileset;
                $result_array[$c]->{ to_key_name( item => $head[1] ) }->[$k]
                    = $values[1];
            } elsif ( $lines[$i] =~ /^\w+\..*\s+(\d+\.){1,}\d+$/ ) {
                $c++;
                $k       = -1;
                $fileset = $lines[$i];
            } elsif ( $lines[$i] =~ /^\/.*/ ) {
                $k++;
                $result_array[$c]->{path} = $path;
                $result_array[$c]->{ to_key_name( item => $head[0] ) }
                    = $fileset;
                $result_array[$c]->{ to_key_name( item => $head[1] ) }->[$k]
                    = $lines[$i];
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-d/ ) {
        my $c = -1;
        my $k = 0;
        my $fileset;
        for ( my $i = 3; $i <= $#lines; $i++ ) {
            $lines[0] =~ s/^\s\s//;
            my @head = split( /\s\s+/, $lines[0] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            if ( $lines[$i] =~ /^\s\s\w+/ ) {
                $lines[$i] =~ s/^\s\s//;
                my @values = split( /\s\s+/, $lines[$i] );
                if ( $values[0] ) {
                    $c++;
                    $k = 0;
                    $result_array[$c]->{path} = $path;
                    $result_array[$c]->{ to_key_name( item => $head[0] ) }
                        = $values[0];
                    $fileset = $values[0];
                }
                if ( $values[1] ) {
                    $result_array[$c]->{dependencies_info}->[$k]
                        ->{ to_key_name( item => $head[1] ) } = $values[1];
                }
                if ( $values[2] ) {
                    $result_array[$c]->{dependencies_info}->[$k]
                        ->{ to_key_name( item => $head[2] ) } = $values[2];
                }
            }
            if ( $lines[$i] =~ /^\s\s\s+\w+/ ) {
                $k++;
                $lines[$i] =~ s/^\s\s\s+//;
                my @values = split( /\s\s+/, $lines[$i] );
                if ( $values[0] ) {
                    $result_array[$c]->{dependencies_info}->[$k]
                        ->{ to_key_name( item => $head[1] ) } = $values[0];
                }
                if ( $values[1] ) {
                    $result_array[$c]->{dependencies_info}->[$k]
                        ->{ to_key_name( item => $head[2] ) } = $values[1];
                }

            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-p/ && $cmd =~ /-a/ ) {
        my $c = -1;
        my $k = 0;
        my ( @result, @array );
        my $temp1;
        my $var;
        for ( my $i = 2; $i <= $#lines; $i++ ) {

            my @head = split( /\s+/, $lines[0] );
            if ( $lines[$i] =~ s/^\s\s// )    { }
            if ( $lines[$i] =~ s/^\s\s\s+// ) { }

            my @values = split( /\s\s\s+/, $lines[$i] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }
            if (   $#values == 2
                && $values[2] eq 'NONE'
                && $values[0] =~ /^\w+\..*\s+(\d+\.){1,}\d+$/ ) {
                $c++;
                $result_array[$c]->{path}       = $path;
                $result_array[$c]->{fileset}    = $values[0];
                $result_array[$c]->{state}      = $values[1];
                $result_array[$c]->{requisites} = $values[2];
            }
            if ( $#values == 2 && $values[2] =~ /\*.*/ ) {
                $c++;
                $result_array[$c]->{path}    = $path;
                $result_array[$c]->{fileset} = $values[0];
                $result_array[$c]->{state}   = $values[1];
                $temp                        = $values[2];
                if ( $lines[ $i + 1 ] =~ s/^\s\s+// ) { }
                while ($lines[ $i + 1 ] =~ /\*.*/
                    || $lines[ $i + 1 ] =~ /^\s*\(*\d+\..*\d+\)*$/
                    || $lines[ $i + 1 ] eq 'NONE' ) {
                    if ( $lines[ $i + 1 ] =~ s/^\s\s+// ) { }
                    $temp = $temp . " " . $lines[ $i + 1 ];
                    $i++;
                }
                @result = split( /\s+/, $temp );
                for ( my $i = 0; $i <= $#result; $i++ ) {
                    $temp1 = $temp1 . " " . $result[$i];
                    if ( $result[ $i + 1 ] =~ /^\*\w+/ && $i ) {
                        push( @array, $temp1 );
                        $temp1 = "";
                    }
                }
                foreach my $var (@array) {
                    if ( $var =~ s/^\s// ) { }
                    $result_array[$c]->{requisites}->[$k] = $var;
                    $k++;
                }
            } elsif ( $values[0] =~ /^\w+.*\s+(\d+\.){1,}\d+$/
                && $#values == 0 ) {
                $c++;
                $result_array[$c]->{path}    = $path;
                $result_array[$c]->{fileset} = $values[0];
                if ( $lines[ $i + 1 ] =~ s/^\s\s+// ) { }
                if ( $lines[ $i + 1 ] =~ /^U.*/ ) {
                    my @val = split( /\s\s\s+/, $lines[ $i + 1 ] );
                    $result_array[$c]->{ptf_id} = $val[0];
                    $result_array[$c]->{state}  = $val[1];
                    $temp                       = $val[2];
                    while ($lines[ $i + 2 ] =~ /\*.*/
                        || $lines[ $i + 2 ] =~ /^\s*\(*\d+\..*\d+\)*$/
                        || $lines[ $i + 2 ] eq 'NONE' ) {
                        if ( $lines[ $i + 2 ] =~ s/^\s\s+// ) { }
                        $temp = $temp . " " . $lines[ $i + 2 ];
                        $i++;
                    }
                    @result = split( /\s+/, $temp );
                    for ( my $i = 0; $i <= $#result; $i++ ) {
                        $temp1 = $temp1 . " " . $result[$i];
                        if ( $result[ $i + 1 ] =~ /^\*\w+/ && $i ) {
                            push( @array, $temp1 );
                            $temp1 = "";
                        }
                    }
                    foreach my $var (@array) {
                        if ( $var =~ s/^\s// ) { }
                        $result_array[$c]->{requisites}->[$k] = $var;
                        $k++;
                    }
                } else {
                    my @val = split( /\s\s\s+/, $lines[ $i + 1 ] );
                    $result_array[$c]->{state} = $val[0];
                    $temp = $val[1];
                    while ($lines[ $i + 2 ] =~ /\*.*/
                        || $lines[ $i + 2 ] =~ /^\s*\(*\d+\..*\d+\)*$/
                        || $lines[ $i + 2 ] eq 'NONE' ) {
                        if ( $lines[ $i + 2 ] =~ s/^\s\s+// ) { }
                        $temp = $temp . " " . $lines[ $i + 2 ];
                        $i++;
                    }
                    @result = split( /\s+/, $temp );
                    for ( my $i = 0; $i <= $#result; $i++ ) {
                        $temp1 = $temp1 . " " . $result[$i];
                        if ( $result[ $i + 1 ] =~ /^\*\w+/ && $i ) {
                            push( @array, $temp1 );
                            $temp1 = "";
                        }
                    }
                    foreach my $var (@array) {
                        if ( $var =~ s/^\s// ) { }
                        $result_array[$c]->{requisites}->[$k] = $var;
                        $k++;
                    }
                }
            }
        }
        return \@result_array;
    }
    if ( $cmd =~ /-p/ ) {
        my $c = -1;
        my $k = 0;
        my $var;
        my ( @result, @array );
        my $temp1;
        for ( my $i = 2; $i <= $#lines; $i++ ) {

            my @head = split( /\s+/, $lines[0] );
            if ( $lines[$i] =~ s/^\s\s// )    { }
            if ( $lines[$i] =~ s/^\s\s\s+// ) { }
            my @values = split( /\s\s+/, $lines[$i] );
            if ( $lines[$i] =~ /Path:\s+(.*)/ ) {
                $path = $1;
            }

            if (   $values[0] =~ /^\w+\..*\s+(\d+\.){1,}\d+$/
                && $values[1] ne 'NONE' ) {
                $c++;
                $result_array[$c]->{path}    = $path;
                $result_array[$c]->{fileset} = $values[0];
            }
            if (   $values[0] =~ /^\w+\..*\s+(\d+\.){1,}\d+$/
                && $values[1] eq 'NONE' ) {
                $c++;
                $result_array[$c]->{path}       = $path;
                $result_array[$c]->{fileset}    = $values[0];
                $result_array[$c]->{requisites} = $values[1];
            }
            if ( $lines[$i] =~ /^\*.*\d+$/ || $lines[$i] eq 'NONE' ) {
                $temp = $lines[$i];

                while ( $lines[ $i + 1 ] =~ /\*.*\d+$/ ) {
                    if ( $lines[ $i + 1 ] =~ s/^\s\s\s+// ) { }
                    $temp = $temp . " " . $lines[ $i + 1 ];
                    $i++;
                }
                @result = split( /\s+/, $temp );
                for ( my $i = 0; $i <= $#result; $i++ ) {
                    $temp1 = $temp1 . " " . $result[$i];
                    if ( $result[ $i + 1 ] =~ /^\*\w+/ && $i ) {
                        push( @array, $temp1 );
                        $temp1 = "";
                    }
                }
                foreach my $var (@array) {
                    if ( $var =~ s/^\s// ) { }
                    $result_array[$c]->{requisites}->[$k] = $var;
                    $k++;
                }
            }

        }
        return \@result_array;
    }

    if ( $cmd =~ /-E/ ) {
        my $v = -1;
        foreach my $line (@lines) {
            if ( $line =~ /Fileset.*/ ) {
                $v++;
            }
            if ( $line =~ /(\w*\s*\w*):\s*(.*)/ ) {

                $result_array[$v]->{ to_key_name( item => $1 ) } = $2;
            }

        }
        return \@result_array;
    }
    if ( $cmd =~ /-S/ ) {
        my $head;
        my $c = -1;
        my $j = -1;
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /(Automatically Installed)/ ) {
                $head = $1;
                $i    = $i + 1;
            } elsif ( $lines[$i] =~ /(Optionally Installed)/ ) {
                $head = $1;
                $i    = $i + 1;
            } elsif ( $lines[$i] !~ /--------------------------------.*/
                && $lines[$i] !~ /bash.*/ ) {
                if ( $head =~ /Automatically Installed/ ) {
                    $c++;
                    $result_array[0]->{ to_key_name( item => $head ) }->[$c]
                        = $lines[$i];
                } elsif ( $head =~ /Optionally Installed/ ) {
                    $j++;
                    $result_array[0]->{ to_key_name( item => $head ) }->[$j]
                        = $lines[$i];
                }
            }
        }
        return \@result_array;
    }

} ## end sub lslpp

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(Usage:|lslpp:)\s+\w+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /(lslpp:)\s+\d+.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

sub get_lines_tree(%) {
    my %args       = @_;
    my $raw_output = $args{'raw_output'};
    my $first_line = $args{'first_line'};
    my $can_throw  = $args{'can_throw'};
    my @lines;

    if ($raw_output = snip_only_required_output(
            'raw_output' => $raw_output,
            'delimiter'  => $first_line
        )
        ) {
        @lines = split( /\r*\n+/, $raw_output );
        map( s/\s+$//, @lines );
        map( s/\cM//g, @lines );
        @lines = grep ( /\S+/, @lines );
    } elsif ($can_throw) {
        my $response_object = $args{'response_object'};
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "output is not in the expected format",
            response_object => $response_object
        );
    }
    return @lines;
} ## end sub get_lines_tree(%)

1;
