#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/lspath.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX lspath command
##
## @description
##    This is a library to parse the raw output of AIX lspath command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lspath" lspath()
##
## @synopsis
##    use lspath;
##
##    lspath(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::lspath;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::lspath
##
## @summary Parses the raw output from the "lspath" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lspath with any of its options as described in the
##    corresponding cdef file 'lspath.cdefs'
## @synopsis
##      lspath(%args);
## INPUTS TO METHOD CALL 'lspath' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-l' </td> <td> 'logical-device' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'indicates-attributes' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'connection' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'parent' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'headers' </td> <td> '1' </td> </tr>
## <tr> <td> '-E' </td> <td> 'current-values' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'default-values' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'identifies-attribute' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'displays-legal-values' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'command-usage-message' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file' </td> <td> '0' </td> </tr>
## <tr> <td> '-O' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'format' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lspath -H -l hdisk1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;status  name   parent
##&nbsp;
##&nbsp;Enabled hdisk1 scsi0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'parent' => 'scsi0',
##&nbsp;            'status' => 'Enabled',
##&nbsp;            'name' => 'hdisk1',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -A  -H -l hdisk1 -p scsi0 -E
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;attribute value description user_settable
##&nbsp;
##&nbsp;priority  1     Priority    True
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'attribute' => 'priority',
##&nbsp;            'user_settable' => 'True',
##&nbsp;            'value' => '1',
##&nbsp;            'description' => 'Priority'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -A -D -H -l hdisk1 -p scsi0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;attribute deflt description user_settable
##&nbsp;
##&nbsp;priority  1     Priority    True
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'deflt' => '1',
##&nbsp;            'attribute' => 'priority',
##&nbsp;            'user_settable' => 'True',
##&nbsp;            'description' => 'Priority'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -A  -l hdisk1 -p scsi0 -E
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;priority 1 Priority True
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'attribute' => 'priority',
##&nbsp;            'user_settable' => 'True',
##&nbsp;            'value' => '1',
##&nbsp;            'description' => 'Priority'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -s Enabled
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Enabled hdisk0 scsi0
##&nbsp;Enabled hdisk1 scsi0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'parent' => 'scsi0',
##&nbsp;            'status' => 'Enabled',
##&nbsp;            'name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'parent' => 'scsi0',
##&nbsp;            'status' => 'Enabled',
##&nbsp;            'name' => 'hdisk1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -l hdisk1 -p scsi0 -D -O
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#priority
##&nbsp;1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'priority' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -A -l hdisk1 -p scsi0 -H -R -a priority
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1...255 (+1)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'legal_values' => '1...255 (+1)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lspath -l hdisk1 -s available -F "connection-parent-path_status-status" -H
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;connection-parent-path_status-status
##&nbsp;
##&nbsp;11,0-scsi0-Available-Enabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'parent' => 'scsi0',
##&nbsp;            'connection' => '11,0',
##&nbsp;            'status' => 'Enabled',
##&nbsp;            'path_status' => 'Available'
##&nbsp;          }
##&nbsp;
sub lspath {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my $count = 0;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    # Default header for option -s, -l & -p
    @head = (qw( status name parent)) if ( $cmd =~ /-s|-l|-p/ );
    # Default header for option -E
    @head = (qw( attribute value description user_settable)) if ( $cmd =~ /-E/ );
    # Default header for option -D
    @head = (qw( attribute deflt description user_settable))if ( $cmd =~ /-D/ );

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /^\d+$/ ) {
            $result_array[$count]->{priority} = $lines[$i];
            $count++;
        } elsif ( $lines[$i] =~ /^(\d+...\d+)\s+\(.*\)$/ ) {
            $result_array[$count]->{legal_values} = $lines[$i];
            $count++;
        } elsif ( $cmd =~ /-F\s+(\'|\")(.*)(\'|\")/) {
                @head = split( /\W+/, $lines[0] );
                    if($lines[$i] ne $lines[0]){
                        if($lines[$i] =~ s/(\d+\,\d+)//){
                            $result_array[$count]->{connection} = $1;
                        }
                    @values = split( /\W+/, $lines[$i] );
                    for ( my $k = 0; $k <= $#values; $k++ ) {
                    $result_array[$count]->{ to_key_name( item => $head[$k] ) }
                            = $values[$k] if($head[$k] ne 'connection');
                }
                 $count++;
              }
        }
        elsif ( $cmd =~ /-H/ ) {
            # If the header is present repopulating it
            @head   = split( /\s+/, $lines[0] );
            if($lines[$i] ne $lines[0]){
            @values = split( /\s+/, $lines[$i] );
            for ( my $k = 0; $k <= $#values; $k++ ) {
                $result_array[$count]->{ to_key_name( item => $head[$k] ) }
                    = $values[$k];
            }
             $count++;
            }
        } else{
            if($lines[$i] ne $lines[0]){
            @values = split( /\s+/, $lines[$i] );
            for ( my $k = 0; $k <= $#values; $k++ ) {
                $result_array[$count]->{ to_key_name( item => $head[$k] ) }
                    = $values[$k];
            }
          $count++;
        }
        }
    }
    return \@result_array;
} ## end sub lspath

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(Usage:|lspath:|Error:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
