#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/lsvg.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX lsvg command
##
## @description
##    This is a library to parse the raw output of AIX lsvg command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lsvg" will lsvg()
##
## @synopsis
##    use lsvg;
##
##    lsvg(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::lsvg;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::lsvg
##
## @summary Parses the raw output from the "lsvg" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lsvg with any of its options as described in the
##    corresponding cdef file 'lsvg.cdefs'
## @synopsis
##      lsvg(%args);
## INPUTS TO METHOD CALL 'lsvg' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-P' </td> <td> 'mirror-pool-of-pv' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mirror-pool-of-lv' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'lock' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'lv-on-pv-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'active-vol-groups' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'standard-input' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'lv-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'descriptor-area-info' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volumegroup' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pv-info' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = lsvg -m rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Logical Volume    Copy 1            Copy 2            Copy 3
##&nbsp;hd5               None              None              None
##&nbsp;hd6               None              None              None
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'logical_volume' => 'hd5',
##&nbsp;            'copy_1' => 'None',
##&nbsp;            'copy_2' => 'None',
##&nbsp;            'copy_3' => 'None'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'logical_volume' => 'hd6',
##&nbsp;            'copy_1' => 'None',
##&nbsp;            'copy_2' => 'None',
##&nbsp;            'copy_3' => 'None'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsvg -P rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Physical Volume   Mirror Pool
##&nbsp;hdisk0            None
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_volume' => 'hdisk0',
##&nbsp;            'mirror_pool' => 'None'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsvg rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VOLUME GROUP:       rootvg                   VG IDENTIFIER:  000393860000d70000000138773c1be0
##&nbsp;VG STATE:           active                   PP SIZE:        128 megabyte(s)
##&nbsp;VG PERMISSION:      read/write               TOTAL PPs:      546 (69888 megabytes)
##&nbsp;MAX LVs:            256                      FREE PPs:       74 (9472 megabytes)
##&nbsp;LVs:                11                       USED PPs:       472 (60416 megabytes)
##&nbsp;OPEN LVs:           10                       QUORUM:         2 (Enabled)
##&nbsp;TOTAL PVs:          1                        VG DESCRIPTORS: 2
##&nbsp;STALE PVs:          0                        STALE PPs:      0
##&nbsp;ACTIVE PVs:         1                        AUTO ON:        yes
##&nbsp;MAX PPs per VG:     32512
##&nbsp;MAX PPs per PV:     1016                     MAX PVs:        32
##&nbsp;LTG size (Dynamic): 256 kilobyte(s)          AUTO SYNC:      no
##&nbsp;HOT SPARE:          no                       BB POLICY:      relocatable
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bb_policy' => 'relocatable',
##&nbsp;            'free_pps' => '74 (9472 megabytes)',
##&nbsp;            'lvs' => '11',
##&nbsp;            'open_lvs' => '10',
##&nbsp;            'hot_spare' => 'no',
##&nbsp;            'used_pps' => '472 (60416 megabytes)',
##&nbsp;            'auto_sync' => 'no',
##&nbsp;            'vg_permission' => 'read/write',
##&nbsp;            'total_pps' => '546 (69888 megabytes)',
##&nbsp;            'vg_identifier' => '000393860000d70000000138773c1be0',
##&nbsp;            'max_lvs' => '256',
##&nbsp;            'ltg_size_dynamic' => '256 kilobyte(s)',
##&nbsp;            'total_pvs' => '1',
##&nbsp;            'stale_pvs' => '0',
##&nbsp;            'active_pvs' => '1',
##&nbsp;            'max_pvs' => '32',
##&nbsp;            'vg_state' => 'active',
##&nbsp;            'vg_descriptors' => '2',
##&nbsp;            'stale_pps' => '0',
##&nbsp;            'max_pps_per_vg' => '32512',
##&nbsp;            'pp_size' => '128 megabyte(s)',
##&nbsp;            'auto_on' => 'yes',
##&nbsp;            'volume_group' => 'rootvg',
##&nbsp;            'quorum' => '2 (Enabled)',
##&nbsp;            'max_pps_per_pv' => '1016'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsvg -l rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;rootvg:
##&nbsp;LV NAME             TYPE       LPs     PPs     PVs  LV STATE      MOUNT POINT
##&nbsp;hd5                 boot       1       1       1    closed/syncd  N/A
##&nbsp;hd6                 paging     4       4       1    open/syncd    N/A
##&nbsp;hd8                 jfs2log    1       1       1    open/syncd    N/A
##&nbsp;hd4                 jfs2       126     126     1    open/syncd    /
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pps' => '1',
##&nbsp;            'lv_name' => 'hd5',
##&nbsp;            'type' => 'boot',
##&nbsp;            'mount_point' => 'N/A',
##&nbsp;            'lv_state' => 'closed/syncd',
##&nbsp;            'pvs' => '1',
##&nbsp;            'lps' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pps' => '4',
##&nbsp;            'lv_name' => 'hd6',
##&nbsp;            'type' => 'paging',
##&nbsp;            'mount_point' => 'N/A',
##&nbsp;            'lv_state' => 'open/syncd',
##&nbsp;            'pvs' => '1',
##&nbsp;            'lps' => '4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pps' => '1',
##&nbsp;            'lv_name' => 'hd8',
##&nbsp;            'type' => 'jfs2log',
##&nbsp;            'mount_point' => 'N/A',
##&nbsp;            'lv_state' => 'open/syncd',
##&nbsp;            'pvs' => '1',
##&nbsp;            'lps' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pps' => '126',
##&nbsp;            'lv_name' => 'hd4',
##&nbsp;            'type' => 'jfs2',
##&nbsp;            'mount_point' => '/',
##&nbsp;            'lv_state' => 'open/syncd',
##&nbsp;            'pvs' => '1',
##&nbsp;            'lps' => '126'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsvg -p rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;rootvg:
##&nbsp;PV_NAME           PV STATE          TOTAL PPs   FREE PPs    FREE DISTRIBUTION
##&nbsp;hdisk0            active            546         368         109..41..00..109..109
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'free_pps' => '368',
##&nbsp;            'free_distribution' => '109..41..00..109..109',
##&nbsp;            'pv_state' => 'active',
##&nbsp;            'total_pps' => '546',
##&nbsp;            'pv_name' => 'hdisk0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lsvg -M rootvg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;rootvg
##&nbsp;hdisk0:1        hd5:1
##&nbsp;hdisk0:2-110
##&nbsp;hdisk0:111      hd6:1
##&nbsp;hdisk0:112      hd6:2
##&nbsp;hdisk0:113      hd6:3
##&nbsp;hdisk0:114      hd6:4
##&nbsp;hdisk0:115      livedump:1
##&nbsp;hdisk0:116      livedump:2
##&nbsp;hdisk0:117-157
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'log_partition_num' => '1',
##&nbsp;            'log_vol_name' => 'hd5',
##&nbsp;            'phy_partition_num' => '1',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'phy_partition_num' => '2-110',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '1',
##&nbsp;            'log_vol_name' => 'hd6',
##&nbsp;            'phy_partition_num' => '111',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '2',
##&nbsp;            'log_vol_name' => 'hd6',
##&nbsp;            'phy_partition_num' => '112',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '3',
##&nbsp;            'log_vol_name' => 'hd6',
##&nbsp;            'phy_partition_num' => '113',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '4',
##&nbsp;            'log_vol_name' => 'hd6',
##&nbsp;            'phy_partition_num' => '114',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '1',
##&nbsp;            'log_vol_name' => 'livedump',
##&nbsp;            'phy_partition_num' => '115',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'log_partition_num' => '2',
##&nbsp;            'log_vol_name' => 'livedump',
##&nbsp;            'phy_partition_num' => '116',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'phy_partition_num' => '117-157',
##&nbsp;            'phy_vol_name' => 'hdisk0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lsvg {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my $head;
    my $m     = -1;
    my $c     = 0;
    my $i     = -1;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $cmd =~ /-l|-p/ ) {
            @head = split( /\s\s+/, $lines[1] );
            for ( my $i = 2; $i < @lines; $i++ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $c++;
            }
            return \@result_array;
        } elsif ( $cmd =~ /-o|-L/ ) {
            if ( $line =~ /(\w+)/ ) {
                $m++;
                $result_array[0]->{volume_goups}->[$m] = $1;
            }
        } elsif ( $cmd =~ /-m|-P/ ) {
            @head = split( /\s\s+/, $lines[0] );
            for ( my $i = 1; $i < @lines; $i++ ) {
                my @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $c++;
            }
            return \@result_array;
        } elsif ( $cmd =~ /-M/ ) {
            if ( $line =~ /(\w+):(\d+)\s+(\w+):(\d+)/ ) {
                $i++;
                $result_array[$i]->{phy_vol_name}      = $1;
                $result_array[$i]->{phy_partition_num} = $2;
                $result_array[$i]->{log_vol_name}      = $3;
                $result_array[$i]->{log_partition_num} = $4;
            } elsif ( $line =~ /(\w+):(\d+-\d+)/ ) {
                $i++;
                $result_array[$i]->{phy_vol_name}      = $1;
                $result_array[$i]->{phy_partition_num} = $2;
            }
        } elsif ( $lines[0] =~ /VOLUME GROUP:/ ) {
            my @array = split( /:\s+/, $line );
            if ( $#array == 2 ) {
                my @temp = split( /\s\s+/, $array[1] );
                $result_array[0]->{ to_key_name( item => $array[0] ) }
                    = $temp[0];
                $result_array[0]->{ to_key_name( item => $temp[1] ) }
                    = $array[2];
            } elsif ( $#array == 1 ) {
                my @temp = split( /\s\s+/, $array[1] );
                $result_array[0]->{ to_key_name( item => $array[0] ) }
                    = $temp[0];
            }
        }
    }
    return \@result_array;
}    ##end of lsvg

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /lsvg:\s+Not a recognized flag:.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /lsvg:\s+A flag requires a parameter:.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

1;

