#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/ntpdate.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse AIX ntpdate command
##
## @description
##    This is a library to parse the raw output of AIX ntpdate command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ntpdate" will ntpdate()
##
## @synopsis
##    use ntpdate;
##
##    ntpdate(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::ntpdate;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::ntpdate
##
## @summary Parses the raw output from the "ntpdate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ntpdate with any of its options as described in the
##    corresponding cdef file 'ntpdate.cdefs'
## @synopsis
##      ntpdate(%args);
## INPUTS TO METHOD CALL 'ntpdate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'slew-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'authentication-delay' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'samples' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'step-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'key-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'syslog' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'key-file' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'server' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'unprivileged-port' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
sub ntpdate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub ntpdate

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /option requires argument.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|ntpdate:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /.*can't find host.*|.*no servers.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

1;

