#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX oslevel command
##
## @description
##    This is a library to parse the raw output of AIX oslevel command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "oslevel" will oslevel
##
## @synopsis
##    use oslevel;
##
##    oslevel(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::oslevel;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::oslevel
##
## @summary Parses the raw output from the "oslevel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from oslevel with any of its options as described in the
##    corresponding cdef file 'oslevel.cdefs'
## @synopsis
##      oslevel(%args);
## INPUTS TO METHOD CALL 'oslevel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'applies-all-technology-levels' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force-rebuild-cache' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'lists-earlier-filesets' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'applies-all-service-packs' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'lists-greater-filesets' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'maintenance-levels' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'lists-known-technology-levels' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = oslevel -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;5.3.0.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_maintenance_levels' => [
##&nbsp;                                            '5.3.0.0'
##&nbsp;                                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;5300-12
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_recommended_maintenance_levels' => '5300-12'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;5300-12-04-1119
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_service_packs' => [
##&nbsp;                                       '5300-12-04-1119'
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -qr
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Known Recommended Maintenance Levels
##&nbsp;------------------------------------
##&nbsp;5300-12
##&nbsp;5300-11
##&nbsp;5300-10
##&nbsp;5300-09
##&nbsp;5300-08
##&nbsp;5300-07
##&nbsp;5300-06
##&nbsp;5300-05
##&nbsp;5300-04
##&nbsp;5300-03
##&nbsp;5300-02
##&nbsp;5300-01
##&nbsp;5300-00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_recommended_maintenance_levels' => [
##&nbsp;                                                        '5300-12',
##&nbsp;                                                        '5300-11',
##&nbsp;                                                        '5300-10',
##&nbsp;                                                        '5300-09',
##&nbsp;                                                        '5300-08',
##&nbsp;                                                        '5300-07',
##&nbsp;                                                        '5300-06',
##&nbsp;                                                        '5300-05',
##&nbsp;                                                        '5300-04',
##&nbsp;                                                        '5300-03',
##&nbsp;                                                        '5300-02',
##&nbsp;                                                        '5300-01',
##&nbsp;                                                        '5300-00'
##&nbsp;                                                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -qf
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Known Maintenance Levels
##&nbsp;------------------------
##&nbsp;5.3.0.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_maintenance_levels' => [
##&nbsp;                                            '5.3.0.0'
##&nbsp;                                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -qs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Known Service Packs
##&nbsp;-------------------
##&nbsp;5300-12-04-1119
##&nbsp;5300-12-03-1107
##&nbsp;5300-12-02-1036
##&nbsp;5300-12-01-1016
##&nbsp;5300-12-00-0000
##&nbsp;5300-11-07-1119
##&nbsp;5300-11-06-1107
##&nbsp;5300-11-05-1036
##&nbsp;5300-11-04-1015
##&nbsp;5300-11-03-1013
##&nbsp;5300-11-02-1007
##&nbsp;5300-11-01-0944
##&nbsp;5300-10-07-1119
##&nbsp;5300-10-06-1107
##&nbsp;5300-10-05-1036
##&nbsp;5300-10-04-1015
##&nbsp;5300-10-03-1013
##&nbsp;5300-10-02-0943
##&nbsp;5300-10-01-0921
##&nbsp;5300-09-08-1036
##&nbsp;5300-09-07-1015
##&nbsp;5300-09-06-1013
##&nbsp;5300-09-05-0943
##&nbsp;5300-09-04-0920
##&nbsp;5300-09-03-0918
##&nbsp;5300-09-02-0849
##&nbsp;5300-09-01-0847
##&nbsp;5300-08-10-1015
##&nbsp;5300-08-09-1013
##&nbsp;5300-08-08-0943
##&nbsp;5300-08-07-0920
##&nbsp;5300-08-06-0918
##&nbsp;5300-08-05-0846
##&nbsp;5300-08-04-0844
##&nbsp;5300-08-03-0831
##&nbsp;5300-08-02-0822
##&nbsp;5300-08-01-0819
##&nbsp;5300-07-10-0943
##&nbsp;5300-07-09-0920
##&nbsp;5300-07-08-0918
##&nbsp;5300-07-07-0846
##&nbsp;5300-07-06-0844
##&nbsp;5300-07-05-0831
##&nbsp;5300-07-04-0818
##&nbsp;5300-07-03-0811
##&nbsp;5300-07-02-0806
##&nbsp;5300-07-01-0748
##&nbsp;5300-06-12-0920
##&nbsp;5300-06-11-0918
##&nbsp;5300-06-10-0846
##&nbsp;5300-06-09-0844
##&nbsp;5300-06-08-0831
##&nbsp;5300-06-07-0818
##&nbsp;5300-06-06-0811
##&nbsp;5300-06-05-0806
##&nbsp;5300-06-04-0748
##&nbsp;5300-06-03-0732
##&nbsp;5300-06-02-0727
##&nbsp;5300-06-01-0000
##&nbsp;5300-05-CSP-0000
##&nbsp;5300-05-06-0000
##&nbsp;5300-05-05-0000
##&nbsp;5300-05-04-0000
##&nbsp;5300-05-03-0000
##&nbsp;5300-05-02-0000
##&nbsp;5300-05-01-0000
##&nbsp;5300-04-CSP-0000
##&nbsp;5300-04-03-0000
##&nbsp;5300-04-02-0000
##&nbsp;5300-04-01-0000
##&nbsp;5300-03-CSP-0000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_service_packs' => [
##&nbsp;                                       '5300-12-04-1119',
##&nbsp;                                       '5300-12-03-1107',
##&nbsp;                                       '5300-12-02-1036',
##&nbsp;                                       '5300-12-01-1016',
##&nbsp;                                       '5300-12-00-0000',
##&nbsp;                                       '5300-11-07-1119',
##&nbsp;                                       '5300-11-06-1107',
##&nbsp;                                       '5300-11-05-1036',
##&nbsp;                                       '5300-11-04-1015',
##&nbsp;                                       '5300-11-03-1013',
##&nbsp;                                       '5300-11-02-1007',
##&nbsp;                                       '5300-11-01-0944',
##&nbsp;                                       '5300-10-07-1119',
##&nbsp;                                       '5300-10-06-1107',
##&nbsp;                                       '5300-10-05-1036',
##&nbsp;                                       '5300-10-04-1015',
##&nbsp;                                       '5300-10-03-1013',
##&nbsp;                                       '5300-10-02-0943',
##&nbsp;                                       '5300-10-01-0921',
##&nbsp;                                       '5300-09-08-1036',
##&nbsp;                                       '5300-09-07-1015',
##&nbsp;                                       '5300-09-06-1013',
##&nbsp;                                       '5300-09-05-0943',
##&nbsp;                                       '5300-09-04-0920',
##&nbsp;                                       '5300-09-03-0918',
##&nbsp;                                       '5300-09-02-0849',
##&nbsp;                                       '5300-09-01-0847',
##&nbsp;                                       '5300-08-10-1015',
##&nbsp;                                       '5300-08-09-1013',
##&nbsp;                                       '5300-08-08-0943',
##&nbsp;                                       '5300-08-07-0920',
##&nbsp;                                       '5300-08-06-0918',
##&nbsp;                                       '5300-08-05-0846',
##&nbsp;                                       '5300-08-04-0844',
##&nbsp;                                       '5300-08-03-0831',
##&nbsp;                                       '5300-08-02-0822',
##&nbsp;                                       '5300-08-01-0819',
##&nbsp;                                       '5300-07-10-0943',
##&nbsp;                                       '5300-07-09-0920',
##&nbsp;                                       '5300-07-08-0918',
##&nbsp;                                       '5300-07-07-0846',
##&nbsp;                                       '5300-07-06-0844',
##&nbsp;                                       '5300-07-05-0831',
##&nbsp;                                       '5300-07-04-0818',
##&nbsp;                                       '5300-07-03-0811',
##&nbsp;                                       '5300-07-02-0806',
##&nbsp;                                       '5300-07-01-0748',
##&nbsp;                                       '5300-06-12-0920',
##&nbsp;                                       '5300-06-11-0918',
##&nbsp;                                       '5300-06-10-0846',
##&nbsp;                                       '5300-06-09-0844',
##&nbsp;                                       '5300-06-08-0831',
##&nbsp;                                       '5300-06-07-0818',
##&nbsp;                                       '5300-06-06-0811',
##&nbsp;                                       '5300-06-05-0806',
##&nbsp;                                       '5300-06-04-0748',
##&nbsp;                                       '5300-06-03-0732',
##&nbsp;                                       '5300-06-02-0727',
##&nbsp;                                       '5300-06-01-0000',
##&nbsp;                                       '5300-05-CSP-0000',
##&nbsp;                                       '5300-05-06-0000',
##&nbsp;                                       '5300-05-05-0000',
##&nbsp;                                       '5300-05-04-0000',
##&nbsp;                                       '5300-05-03-0000',
##&nbsp;                                       '5300-05-02-0000',
##&nbsp;                                       '5300-05-01-0000',
##&nbsp;                                       '5300-04-CSP-0000',
##&nbsp;                                       '5300-04-03-0000',
##&nbsp;                                       '5300-04-02-0000',
##&nbsp;                                       '5300-04-01-0000',
##&nbsp;                                       '5300-03-CSP-0000'
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = oslevel -g
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Fileset                                 Actual Level        Maintenance Level
##&nbsp;-----------------------------------------------------------------------------
##&nbsp;bos.rte                                 5.3.12.1            5.3.0.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'known_maintenance_levels' => [
##&nbsp;                                            '5.3.0.0'
##&nbsp;                                          ],
##&nbsp;            'actual_level' => '5.3.12.1',
##&nbsp;            'fileset' => 'bos.rte'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub oslevel {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head );
    my $i = 0;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    if ( $cmd =~ /-g|-l/ ) {
        for ( my $c = 2; $c <= $#lines; $c++ ) {
            $lines[0] =~ s/Maintenance Level/known_maintenance_levels/g;
            @head = split( /\s\s+/, $lines[0] );

            my @values = split( /\s+/, $lines[$c] );
            for ( my $j = 0; $j <= $#head - 1; $j++ ) {
                $result_array[$i]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $result_array[$i]->{ to_key_name( item => $head[2] ) }->[$i]
                = $values[-1];
            $i++;
        }
    }
    if ( $cmd =~ /-q/ ) {
        for my $line (@lines) {
            my $h = $lines[0];
            $i = $i + 2;
            my $c = 0;
            while ( $i <= $#lines ) {
                $result_array[0]->{ to_key_name( item => $h ) }->[$c]
                    = $lines[$i];
                $c++;
                $i++;
            }
        }
    }
    if ( $cmd =~ /-f/ ) {
        for my $line (@lines) {
            $result_array[0]->{known_maintenance_levels}->[$i] = $line;
        }
        $i++;
    }
    if ( $cmd =~ /-r/ ) {
        for my $line (@lines) {
            $result_array[$i]->{known_recommended_maintenance_levels} = $line;
        }
        $i++;
    }
    if ( $cmd =~ /-s/ ) {
        for my $line (@lines) {
            $result_array[$i]->{known_service_packs}->[$i] = $line;
        }
        $i++;
    }

    return \@result_array;
} ## end sub oslevel

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(Usage:|oslevel:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(Error:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
