#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/pagesize.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse AIX pagesize command
##
## @description
##    This is a library to parse the raw output of AIX pagesize command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "pagesize" will pagesize()
##
## @synopsis
##    use NACL::APISet::Host::CLI::AIX::default::pagesize;
##
##    pagesize(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::pagesize;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::pagesize
##
## @summary Parses the raw output from the "pagesize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pagesize with any of its options as described in the
##    corresponding cdef file 'pagesize.cdefs'
## @synopsis
##      pagesize(%args);
## INPUTS TO METHOD CALL 'pagesize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'formatted' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pagesize
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4096
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'default_pagesize' => '4096'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pagesize -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4K
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'default_pagesize' => '4K'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pagesize -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4096
##&nbsp;16777216
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'supported_pagesizes' => [
##&nbsp;                                       '4096',
##&nbsp;                                       '16777216'
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pagesize -af
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4K
##&nbsp;16M
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'supported_pagesizes' => [
##&nbsp;                                       '4K',
##&nbsp;                                       '16M'
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pagesize {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $cmd = $resp->get_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $c = -1;

    if ( $cmd =~ /-a/ ) {
        foreach (@lines) {
            $c++;
            $result_array[0]->{supported_pagesizes}->[$c] = $_;
        }
    } else {
        $result_array[0]->{default_pagesize} = $lines[0];
    }
    return \@result_array;
}    ##end of pagesize

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /pagesize: Not a recognized flag:.*|Usage:/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

1;

