#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/ps.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse AIX ps command"
##
## @description
##    This is a library to parse the raw output of AIX ps command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ps" will be ps()
##
## @synopsis
##    use ps;
##
##    ps(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::ps;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::ps
##
## @summary Parses the raw output from the "ps" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ps with any of its options as described in the
##    corresponding cdef file 'ps.cdefs'
## @synopsis
##      ps(%args);
## INPUTS TO METHOD CALL 'ps' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-L' </td> <td> 'pidlist' </td> <td> '0' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'page-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-processes-except-session-leaders' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'terminal-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'user-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-G' </td> <td> 'grouplists' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'pid' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'process-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'grouplist' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'project-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kernel-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'no-thread-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'format-specifier' </td> <td> '0' </td> </tr>
## <tr> <td> '-X' </td> <td> 'prints-all-characters' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'lists-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'users-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'writes-info-except-session-leaders' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'writes-info-except-kernel-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'namelist' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'processes-kernel-threads' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'classlist' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    PID    TTY  TIME CMD
##&nbsp; 151670  pts/3  0:00 /bin/sh -i
##&nbsp; 348164  pts/3  0:00 ps
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/3',
##&nbsp;            'process_id' => '151670',
##&nbsp;            'command' => '/bin/sh -i',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/3',
##&nbsp;            'process_id' => '348164',
##&nbsp;            'command' => 'ps',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -A
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    PID    TTY  TIME CMD
##&nbsp;      0      -  2:00 swapper
##&nbsp;      1      -  0:12 init
##&nbsp;   8196      - 8251:17 wait
##&nbsp;  12294      -  0:02 sched
##&nbsp;  16392      -  0:00 lrud
##&nbsp;  20490      -  0:00 vmptacrt
##&nbsp;  24588      -  0:00 xmfreed
##&nbsp;  28686      -  0:00 psgc
##&nbsp;  32784      -  0:01 pilegc
##&nbsp;  36882      -  0:48 xmgc
##&nbsp;  40980      - 8258:53 wait
##&nbsp;  45078      -  0:39 netm
##&nbsp;  49176      -  4:35 gil
##&nbsp;  53274      -  0:00 wlmsched
##&nbsp;  57372      -  0:00 armtrace_kproc
##&nbsp;  65594      -  0:00 lvmbb
##&nbsp;  69738      -  0:00 errdemon
##&nbsp;  82034      -  0:00 posix_aioserver
##&nbsp;  86194      -  0:11 random
##&nbsp;  90166      -  0:01 j2pg
##&nbsp;  94338      -  0:00 dog
##&nbsp;  98428      -  2:36 syncd
##&nbsp; 106688      -  0:00 inetd
##&nbsp; 110654      -  0:00 shlap64
##&nbsp; 114806      -  0:02 rgsr
##&nbsp; 118860      -  0:00 aioserver
##&nbsp; 123020      -  0:01 kbiod
##&nbsp; 127072      -  0:00 rtcmd
##&nbsp; 131142      -  0:00 nfsSM
##&nbsp; 143512      -  0:02 cron
##&nbsp; 147538      -  0:00 n4bg
##&nbsp; 151678      -  0:00 rlogind
##&nbsp; 155760      -  0:00 srcmstr
##&nbsp; 159824      -  0:00 rdpgc
##&nbsp; 163992      -  0:00 rpc.lockd
##&nbsp; 168044      -  0:01 snmpdv3ne
##&nbsp; 172122      -  0:00 snmpmibd64
##&nbsp; 176300      -  0:00 writesrv
##&nbsp; 180326      -  0:00 hostmibd64
##&nbsp; 188508      -  0:00 syslogd
##&nbsp; 192606      -  0:11 sendmail
##&nbsp; 196704      -  0:10 portmap
##&nbsp; 200802      -  0:03 aixmibd64
##&nbsp; 204968      -  0:00 nfsrgyd
##&nbsp; 209006      -  0:00 muxatmd
##&nbsp; 213182      -  0:00 biod
##&nbsp; 217216      -  0:00 rpc.statd
##&nbsp; 221354      0  0:00 getty
##&nbsp; 225402      -  0:00 ypbind
##&nbsp; 229566      -  1:04 topasrec
##&nbsp; 233610      -  0:38 rpc.lockd
##&nbsp; 237686      -  0:02 aump
##&nbsp; 241794      -  0:00 qdaemon
##&nbsp; 245882      -  0:02 automountd
##&nbsp; 249996      -  0:00 uprintfd
##&nbsp; 258178      -  0:00 sshd
##&nbsp; 270520      -  0:00 telnetd
##&nbsp; 278732      -  0:00 IBM.ServiceRMd
##&nbsp; 286778      -  0:01 rmcd
##&nbsp; 290984      -  0:00 diagd
##&nbsp; 299088  pts/3  0:01 snapdrive
##&nbsp; 303334  pts/4  0:00 ps
##&nbsp; 307338      -  0:00 telnetd
##&nbsp; 315616  pts/1  0:00 bash
##&nbsp; 323596  pts/3  0:00 bash
##&nbsp; 327840      -  0:00 IBM.CSMAgentRMd
##&nbsp; 332002      -  0:00 telnetd
##&nbsp; 348168      -  0:00 telnetd
##&nbsp; 352360      -  0:00 pofCmdProc
##&nbsp; 389278  pts/4  0:00 sh
##&nbsp; 401484      -  0:03 snapdrived
##&nbsp; 405692      -  0:00 snapdrived
##&nbsp; 413920  pts/2  0:00 bash
##&nbsp; 422040  pts/0  0:00 bash
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '0',
##&nbsp;            'command' => 'swapper',
##&nbsp;            'total_runtime' => '2:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '1',
##&nbsp;            'command' => 'init',
##&nbsp;            'total_runtime' => '0:12'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '8196',
##&nbsp;            'command' => 'wait',
##&nbsp;            'total_runtime' => '8251:17'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '12294',
##&nbsp;            'command' => 'sched',
##&nbsp;            'total_runtime' => '0:02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '16392',
##&nbsp;            'command' => 'lrud',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '20490',
##&nbsp;            'command' => 'vmptacrt',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '24588',
##&nbsp;            'command' => 'xmfreed',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '28686',
##&nbsp;            'command' => 'psgc',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '32784',
##&nbsp;            'command' => 'pilegc',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '36882',
##&nbsp;            'command' => 'xmgc',
##&nbsp;            'total_runtime' => '0:48'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '40980',
##&nbsp;            'command' => 'wait',
##&nbsp;            'total_runtime' => '8258:53'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '45078',
##&nbsp;            'command' => 'netm',
##&nbsp;            'total_runtime' => '0:39'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '49176',
##&nbsp;            'command' => 'gil',
##&nbsp;            'total_runtime' => '4:35'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '53274',
##&nbsp;            'command' => 'wlmsched',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '57372',
##&nbsp;            'command' => 'armtrace_kproc',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '65594',
##&nbsp;            'command' => 'lvmbb',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '69738',
##&nbsp;            'command' => 'errdemon',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '82034',
##&nbsp;            'command' => 'posix_aioserver',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '86194',
##&nbsp;            'command' => 'random',
##&nbsp;            'total_runtime' => '0:11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '90166',
##&nbsp;            'command' => 'j2pg',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '94338',
##&nbsp;            'command' => 'dog',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '98428',
##&nbsp;            'command' => 'syncd',
##&nbsp;            'total_runtime' => '2:36'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '106688',
##&nbsp;            'command' => 'inetd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '110654',
##&nbsp;            'command' => 'shlap64',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '114806',
##&nbsp;            'command' => 'rgsr',
##&nbsp;            'total_runtime' => '0:02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '118860',
##&nbsp;            'command' => 'aioserver',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '123020',
##&nbsp;            'command' => 'kbiod',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '127072',
##&nbsp;            'command' => 'rtcmd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '131142',
##&nbsp;            'command' => 'nfsSM',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '143512',
##&nbsp;            'command' => 'cron',
##&nbsp;            'total_runtime' => '0:02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '147538',
##&nbsp;            'command' => 'n4bg',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '151678',
##&nbsp;            'command' => 'rlogind',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '155760',
##&nbsp;            'command' => 'srcmstr',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '159824',
##&nbsp;            'command' => 'rdpgc',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '163992',
##&nbsp;            'command' => 'rpc.lockd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '168044',
##&nbsp;            'command' => 'snmpdv3ne',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '172122',
##&nbsp;            'command' => 'snmpmibd64',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '176300',
##&nbsp;            'command' => 'writesrv',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '180326',
##&nbsp;            'command' => 'hostmibd64',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '188508',
##&nbsp;            'command' => 'syslogd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '192606',
##&nbsp;            'command' => 'sendmail',
##&nbsp;            'total_runtime' => '0:11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '196704',
##&nbsp;            'command' => 'portmap',
##&nbsp;            'total_runtime' => '0:10'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '200802',
##&nbsp;            'command' => 'aixmibd64',
##&nbsp;            'total_runtime' => '0:03'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '204968',
##&nbsp;            'command' => 'nfsrgyd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '209006',
##&nbsp;            'command' => 'muxatmd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '213182',
##&nbsp;            'command' => 'biod',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '217216',
##&nbsp;            'command' => 'rpc.statd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '0',
##&nbsp;            'process_id' => '221354',
##&nbsp;            'command' => 'getty',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '225402',
##&nbsp;            'command' => 'ypbind',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '229566',
##&nbsp;            'command' => 'topasrec',
##&nbsp;            'total_runtime' => '1:04'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '233610',
##&nbsp;            'command' => 'rpc.lockd',
##&nbsp;            'total_runtime' => '0:38'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '237686',
##&nbsp;            'command' => 'aump',
##&nbsp;            'total_runtime' => '0:02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '241794',
##&nbsp;            'command' => 'qdaemon',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '245882',
##&nbsp;            'command' => 'automountd',
##&nbsp;            'total_runtime' => '0:02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '249996',
##&nbsp;            'command' => 'uprintfd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '258178',
##&nbsp;            'command' => 'sshd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '270520',
##&nbsp;            'command' => 'telnetd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '278732',
##&nbsp;            'command' => 'IBM.ServiceRMd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '286778',
##&nbsp;            'command' => 'rmcd',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '290984',
##&nbsp;            'command' => 'diagd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/3',
##&nbsp;            'process_id' => '299088',
##&nbsp;            'command' => 'snapdrive',
##&nbsp;            'total_runtime' => '0:01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'process_id' => '303334',
##&nbsp;            'command' => 'ps',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '307338',
##&nbsp;            'command' => 'telnetd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/1',
##&nbsp;            'process_id' => '315616',
##&nbsp;            'command' => 'bash',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/3',
##&nbsp;            'process_id' => '323596',
##&nbsp;            'command' => 'bash',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '327840',
##&nbsp;            'command' => 'IBM.CSMAgentRMd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '332002',
##&nbsp;            'command' => 'telnetd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '348168',
##&nbsp;            'command' => 'telnetd',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '352360',
##&nbsp;            'command' => 'pofCmdProc',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'process_id' => '389278',
##&nbsp;            'command' => 'sh',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '401484',
##&nbsp;            'command' => 'snapdrived',
##&nbsp;            'total_runtime' => '0:03'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => '-',
##&nbsp;            'process_id' => '405692',
##&nbsp;            'command' => 'snapdrived',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/2',
##&nbsp;            'process_id' => '413920',
##&nbsp;            'command' => 'bash',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'process_terminal' => 'pts/0',
##&nbsp;            'process_id' => '422040',
##&nbsp;            'command' => 'bash',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -Z
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    PID    TTY  TIME DPGSZ SPGSZ TPGSZ SHMPGSZ CMD
##&nbsp; 299102  pts/4  0:00    4K    4K    4K      4K sh
##&nbsp; 405606  pts/4  0:00    4K    4K    4K      4K ps
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'text_page_size' => '4K',
##&nbsp;            'shared_memory_page_size' => '4K',
##&nbsp;            'stack_page_size' => '4K',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_id' => '299102',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'data_page_size' => '4K',
##&nbsp;            'command' => 'sh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'text_page_size' => '4K',
##&nbsp;            'shared_memory_page_size' => '4K',
##&nbsp;            'stack_page_size' => '4K',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_id' => '405606',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'data_page_size' => '4K',
##&nbsp;            'command' => 'ps'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;     UID    PID   PPID   C    STIME    TTY  TIME CMD
##&nbsp;    root 303286 389292   1 05:50:52  pts/4  0:00 ps
##&nbsp;    root 389292 376844   3 05:50:48  pts/4  0:00 /bin/sh
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'parent_process_id' => '389292',
##&nbsp;            'proces_cpu_utilization' => '1',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_id' => '303286',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'user_id' => 'root',
##&nbsp;            'process_start_time' => '05:50:52',
##&nbsp;            'command' => 'ps'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'parent_process_id' => '376844',
##&nbsp;            'proces_cpu_utilization' => '3',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_id' => '389292',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'user_id' => 'root',
##&nbsp;            'process_start_time' => '05:50:48',
##&nbsp;            'command' => '/bin/sh'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       F S UID    PID   PPID   C PRI NI ADDR    SZ    WCHAN    TTY  TIME CMD
##&nbsp;  240001 A   0 376958 299112   4  62 20 2000650b   716           pts/4  0:00 sh
##&nbsp;  200001 A   0 405616 376958   2  61 20 2000e4af   740           pts/4  0:00 ps
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'priority' => '62',
##&nbsp;            'parent_process_id' => '299112',
##&nbsp;            'wait_channel' => '-',
##&nbsp;            'size_in_kb_units' => '716',
##&nbsp;            'nice_value' => '20',
##&nbsp;            'proces_cpu_utilization' => '4',
##&nbsp;            'proces_flag_number' => '240001',
##&nbsp;            'process_state' => 'A',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'process_id' => '376958',
##&nbsp;            'address' => '2000650b',
##&nbsp;            'user_id' => '0',
##&nbsp;            'command' => 'sh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => '61',
##&nbsp;            'parent_process_id' => '376958',
##&nbsp;            'wait_channel' => '-',
##&nbsp;            'size_in_kb_units' => '740',
##&nbsp;            'nice_value' => '20',
##&nbsp;            'proces_cpu_utilization' => '2',
##&nbsp;            'proces_flag_number' => '200001',
##&nbsp;            'process_state' => 'A',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'process_terminal' => 'pts/4',
##&nbsp;            'process_id' => '405616',
##&nbsp;            'address' => '2000e4af',
##&nbsp;            'user_id' => '0',
##&nbsp;            'command' => 'ps'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -t 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    PID    TTY  TIME CMD
##&nbsp; 221354      0  0:00 getty
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_terminal' => '0',
##&nbsp;            'process_id' => '221354',
##&nbsp;            'command' => 'getty',
##&nbsp;            'total_runtime' => '0:00'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -P
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; UID GID    PID    TTY  TIME PROJECT  ORIGIN  SUBPROJ  CMD
##&nbsp;  0   0  270534  pts/1  0:00 0        local        0  sh
##&nbsp;  0   0  389222  pts/1  0:00 0        local        0  ps
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'origin_name' => 'local',
##&nbsp;            'sub_project_identifier' => '0',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'group_id' => '0',
##&nbsp;            'process_id' => '270534',
##&nbsp;            'process_terminal' => 'pts/1',
##&nbsp;            'user_id' => '0',
##&nbsp;            'project_name' => '0',
##&nbsp;            'command' => 'sh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'origin_name' => 'local',
##&nbsp;            'sub_project_identifier' => '0',
##&nbsp;            'total_runtime' => '0:00',
##&nbsp;            'group_id' => '0',
##&nbsp;            'process_id' => '389222',
##&nbsp;            'process_terminal' => 'pts/1',
##&nbsp;            'user_id' => '0',
##&nbsp;            'project_name' => '0',
##&nbsp;            'command' => 'ps'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ps {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my %result = (
        PID     => 'process_id',
        TTY     => 'process_terminal',
        TIME    => 'total_runtime',
        CMD     => 'command',
        DPGSZ   => 'data_page_size',
        SPGSZ   => 'stack_page_size',
        TPGSZ   => 'text_page_size',
        SHMPGSZ => 'shared_memory_page_size',
        UID     => 'user_id',
        PPID    => 'parent_process_id',
        C       => 'proces_cpu_utilization',
        STIME   => 'process_start_time',
        F       => 'proces_flag_number',
        S       => 'process_state',
        PRI     => 'priority',
        NI      => 'nice_value',
        ADDR    => 'address',
        SZ      => 'size_in_kb_units',
        WCHAN   => 'wait_channel',
        GID     => 'group_id',
        PROJECT => 'project_name',
        ORIGIN  => 'origin_name',
        SUBPROJ => 'sub_project_identifier',
        COMMAND => 'command',
        RUSER   => 'real_user',
    );
    my $count = 0;
    my @head = split( /\s+/, $lines[0] );

    for ( my $i = 1; $i <= $#lines; $i++ ) {
        if ( $cmd =~ /-Z|-f|-l|-P|-u|-U/ ) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$count]->{ $result{ $head[$j] } } = $values[$j];
            }
            if ( $cmd =~ /-l/ ) {
                if ( $#head != $#values ) {
                    $result_array[$count]->{ $result{ $head[10] } } = '-';
                    $result_array[$count]->{ $result{ $head[11] } }
                        = $values[10];
                    $result_array[$count]->{ $result{ $head[12] } }
                        = $values[11];
                    $result_array[$count]->{ $result{ $head[13] } }
                        = $values[12];
                }
            }
        } elsif ( $lines[$i] =~ /(\w+)\s+(\S+)\s+(\d+.*\d+)\s+(.*)/ ) {
            $result_array[$count]->{ $result{ $head[0] } } = $1;
            $result_array[$count]->{ $result{ $head[1] } } = $2;
            $result_array[$count]->{ $result{ $head[2] } } = $3;
            $result_array[$count]->{ $result{ $head[3] } } = $4;
        }
        $count++;
    }
    return \@result_array;
} ## end sub ps

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /A flag requires a parameter.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif (
        $raw_output =~ /ps:\s+\d+.*invalid list|Invalid pid specified:.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Not a recognized flag.*|Usage:\s+ps\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
