#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/vmstat.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX vmstat command
##
## @description
##    This is a library to parse the raw output of AIX vmstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vmstat" will vmstat()
##
## @synopsis
##    use vmstat;
##
##    vmstat(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::vmstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::vmstat
##
## @summary Parses the raw output from the "vmstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vmstat with any of its options as described in the
##    corresponding cdef file 'vmstat.cdefs'
## @synopsis
##      vmstat(%args);
## INPUTS TO METHOD CALL 'vmstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'physical-volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'displays-vmm-statistics' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'total-summaries' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'display' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'forks' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wide-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interrupts' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'appends-vmm-statistics' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vmm-statistics' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'page-section' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sum-structure' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'time-stamp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vmstat -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;               470650 forks
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'forks' => '470650'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;             96332640 total address trans. faults
##&nbsp;               425082 page ins
##&nbsp;              3711586 page outs
##&nbsp;                 5780 paging space page ins
##&nbsp;                14872 paging space page outs
##&nbsp;                    0 total reclaims
##&nbsp;             42067275 zero filled pages faults
##&nbsp;                71065 executable filled pages faults
##&nbsp;              5360631 pages examined by clock
##&nbsp;                   10 revolutions of the clock hand
##&nbsp;              3284660 pages freed by the clock
##&nbsp;               654632 backtracks
##&nbsp;                    0 free frame waits
##&nbsp;                    0 extend XPT waits
##&nbsp;               349386 pending I/O waits
##&nbsp;              4102120 start I/Os
##&nbsp;               620827 iodones
##&nbsp;             80680803 cpu context switches
##&nbsp;             10611468 device interrupts
##&nbsp;               666008 software interrupts
##&nbsp;            205425386 decrementer interrupts
##&nbsp;               139675 mpc-sent interrupts
##&nbsp;               139675 mpc-receive interrupts
##&nbsp;                    0 phantom interrupts
##&nbsp;                    0 traps
##&nbsp;           2197951639 syscalls
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'page_outs' => '3711586',
##&nbsp;            'mpc_receive_interrupts' => '139675',
##&nbsp;            'paging_space_page_ins' => '5780',
##&nbsp;            'cpu_context_switches' => '80680803',
##&nbsp;            'zero_filled_pages_faults' => '42067275',
##&nbsp;            'backtracks' => '654632',
##&nbsp;            'pages_examined_by_clock' => '5360631',
##&nbsp;            'executable_filled_pages_faults' => '71065',
##&nbsp;            'device_interrupts' => '10611468',
##&nbsp;            'total_address_trans_faults' => '96332640',
##&nbsp;            'traps' => '0',
##&nbsp;            'pending_io_waits' => '349386',
##&nbsp;            'pages_freed_by_the_clock' => '3284660',
##&nbsp;            'extend_xpt_waits' => '0',
##&nbsp;            'syscalls' => '2197951639',
##&nbsp;            'iodones' => '620827',
##&nbsp;            'free_frame_waits' => '0',
##&nbsp;            'software_interrupts' => '666008',
##&nbsp;            'revolutions_of_the_clock_hand' => '10',
##&nbsp;            'start_ios' => '4102120',
##&nbsp;            'phantom_interrupts' => '0',
##&nbsp;            'mpc_sent_interrupts' => '139675',
##&nbsp;            'decrementer_interrupts' => '205425386',
##&nbsp;            'total_reclaims' => '0',
##&nbsp;            'paging_space_page_outs' => '14872',
##&nbsp;            'page_ins' => '425082'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;priority level    type   count module(handler)
##&nbsp;    0    2046   hardware  1420 i_mpc_int_handler(157b520)
##&nbsp;    1       4   hardware   840 /usr/lib/drivers/isa/rsdd_rspc(4255140)
##&nbsp;    3       1   hardware  5490 /usr/lib/drivers/pci/s_scsiddpin(40329d8)
##&nbsp;    3       2   hardware 9926204 /usr/lib/drivers/pci/phxentdd(4311908)
##&nbsp;    3       4   hardware 1412515 /usr/lib/drivers/pci/s_scsiddpin(40329d8)
##&nbsp;    3      25   hardware 698133 /etc/drivers/pci/efcddpin(3fe6ca8)
##&nbsp;    3      29   hardware 745680 /etc/drivers/pci/efcddpin(3fe6ca8)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'modulehandler' => 'i_mpc_int_handler(157b520)',
##&nbsp;            'priority' => '0',
##&nbsp;            'count' => '1420',
##&nbsp;            'level' => '2046',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/usr/lib/drivers/isa/rsdd_rspc(4255140)',
##&nbsp;            'priority' => '1',
##&nbsp;            'count' => '840',
##&nbsp;            'level' => '4',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/usr/lib/drivers/pci/s_scsiddpin(40329d8)',
##&nbsp;            'priority' => '3',
##&nbsp;            'count' => '5490',
##&nbsp;            'level' => '1',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/usr/lib/drivers/pci/phxentdd(4311908)',
##&nbsp;            'priority' => '3',
##&nbsp;            'count' => '9926204',
##&nbsp;            'level' => '2',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/usr/lib/drivers/pci/s_scsiddpin(40329d8)',
##&nbsp;            'priority' => '3',
##&nbsp;            'count' => '1412515',
##&nbsp;            'level' => '4',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/etc/drivers/pci/efcddpin(3fe6ca8)',
##&nbsp;            'priority' => '3',
##&nbsp;            'count' => '698133',
##&nbsp;            'level' => '25',
##&nbsp;            'type' => 'hardware'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'modulehandler' => '/etc/drivers/pci/efcddpin(3fe6ca8)',
##&nbsp;            'priority' => '3',
##&nbsp;            'count' => '745680',
##&nbsp;            'level' => '29',
##&nbsp;            'type' => 'hardware'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -t
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System configuration: lcpu=2 mem=2048MB
##&nbsp;
##&nbsp;kthr    memory              page              faults        cpu       time
##&nbsp;----- ----------- ------------------------ ------------ ----------- --------
##&nbsp; r  b   avm   fre  re  pi  po  fr   sr  cy  in   sy  cs us sy id wa hr mi se
##&nbsp; 1  1 143993 191687   0   0   0   0    0   0  21  316 103  1  1 98  1 06:47:42
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_configuration' => [
##&nbsp;                                        {
##&nbsp;                                          'number_of_logical_processors' => '2',
##&nbsp;                                          'memory' => '2048MB'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '191687',
##&nbsp;                            'active_virtual_pages' => '143993'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'cpu' => [
##&nbsp;                       {
##&nbsp;                         'system_time' => '1',
##&nbsp;                         'processor_idle_time_during_outstanding_disk' => '1',
##&nbsp;                         'processor_idle_time' => '98',
##&nbsp;                         'user_time' => '1'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'clock_cycles' => '0',
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'pager_io_list' => '0',
##&nbsp;                          'pages_freed' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'time' => [
##&nbsp;                        {
##&nbsp;                          'seconds' => '42',
##&nbsp;                          'hours' => '06',
##&nbsp;                          'minutes' => '47'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'kernel_thread_states' => [
##&nbsp;                                        {
##&nbsp;                                          'runnable_kernel_threads' => '1',
##&nbsp;                                          'kernel_threads_in_vmm_wait_queue' => '1'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'faults' => [
##&nbsp;                          {
##&nbsp;                            'system_calls' => '316',
##&nbsp;                            'device_interrupts' => '21',
##&nbsp;                            'context_switches' => '103'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System configuration: lcpu=2 mem=2048MB
##&nbsp;
##&nbsp;kthr    memory              page              faults        cpu     large-page
##&nbsp;----- ----------- ------------------------ ------------ ----------- -----------
##&nbsp; r  b   avm   fre  re  pi  po  fr   sr  cy  in   sy  cs us sy id wa   alp   flp
##&nbsp; 1  1 144879 190708   0   0   0   0    0   0  21  316 103  1  1 98  1     0     0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_configuration' => [
##&nbsp;                                        {
##&nbsp;                                          'number_of_logical_processors' => '2',
##&nbsp;                                          'memory' => '2048MB'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '190708',
##&nbsp;                            'active_virtual_pages' => '144879'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'cpu' => [
##&nbsp;                       {
##&nbsp;                         'system_time' => '1',
##&nbsp;                         'processor_idle_time_during_outstanding_disk' => '1',
##&nbsp;                         'processor_idle_time' => '98',
##&nbsp;                         'user_time' => '1'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'clock_cycles' => '0',
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'pager_io_list' => '0',
##&nbsp;                          'pages_freed' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'kernel_thread_states' => [
##&nbsp;                                        {
##&nbsp;                                          'runnable_kernel_threads' => '1',
##&nbsp;                                          'kernel_threads_in_vmm_wait_queue' => '1'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'large_page' => [
##&nbsp;                              {
##&nbsp;                                'no_of_large_pages_on_large_page_freelist' => '0',
##&nbsp;                                'no_of_large_pages_currently_in_use' => '0'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'faults' => [
##&nbsp;                          {
##&nbsp;                            'system_calls' => '316',
##&nbsp;                            'device_interrupts' => '21',
##&nbsp;                            'context_switches' => '103'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -P ALL
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System configuration: mem=2048MB
##&nbsp;
##&nbsp;pgsz            memory                           page
##&nbsp;----- -------------------------- ------------------------------------
##&nbsp;           siz      avm      fre    re    pi    po    fr     sr    cy
##&nbsp;   4K   524288   145165   190369     0     0     0     0      0     0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '190369',
##&nbsp;                            'no_frames_of_specified_page_size' => '524288',
##&nbsp;                            'active_virtual_pages' => '145165'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'clock_cycles' => '0',
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'pager_io_list' => '0',
##&nbsp;                          'pages_freed' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'pagesize' => '4K'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -p ALL
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System configuration: lcpu=2 mem=2048MB
##&nbsp;
##&nbsp;kthr    memory              page              faults        cpu
##&nbsp;----- ----------- ------------------------ ------------ -----------
##&nbsp; r  b   avm   fre  re  pi  po  fr   sr  cy  in   sy  cs us sy id wa
##&nbsp; 1  1 146097 189065   0   0   0   0    0   0  21  316 103  1  1 98  1
##&nbsp;
##&nbsp;  psz   avm   fre  re  pi  po  fr   sr  cy     siz
##&nbsp;   4K 146098 189064   0   0   0   0    0   0  524288
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_configuration' => [
##&nbsp;                                        {
##&nbsp;                                          'number_of_logical_processors' => '2',
##&nbsp;                                          'memory' => '2048MB'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '189065',
##&nbsp;                            'active_virtual_pages' => '146097'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'cpu' => [
##&nbsp;                       {
##&nbsp;                         'system_time' => '1',
##&nbsp;                         'processor_idle_time_during_outstanding_disk' => '1',
##&nbsp;                         'processor_idle_time' => '98',
##&nbsp;                         'user_time' => '1'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'clock_cycles' => '0',
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'pager_io_list' => '0',
##&nbsp;                          'pages_freed' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'kernel_thread_states' => [
##&nbsp;                                        {
##&nbsp;                                          'runnable_kernel_threads' => '1',
##&nbsp;                                          'kernel_threads_in_vmm_wait_queue' => '1'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'faults' => [
##&nbsp;                          {
##&nbsp;                            'system_calls' => '316',
##&nbsp;                            'device_interrupts' => '21',
##&nbsp;                            'context_switches' => '103'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '189064',
##&nbsp;                            'active_virtual_pages' => '146098'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'no_frames_of_specified_page_size' => '524288',
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'clock_cycles' => '0',
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'pager_io_list' => '0',
##&nbsp;                          'pages_freed' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'pagesize' => '4K'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -I
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System configuration: lcpu=2 mem=2048MB
##&nbsp;
##&nbsp;  kthr     memory              page              faults        cpu
##&nbsp;-------- ----------- ------------------------ ------------ -----------
##&nbsp; r  b  p   avm   fre  fi  fo  pi  po  fr   sr  in   sy  cs us sy id wa
##&nbsp; 1  1  0 161763 181200   0   6   0   0   0    0  21  321 104  1  1 98  1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_configuration' => [
##&nbsp;                                        {
##&nbsp;                                          'number_of_logical_processors' => '2',
##&nbsp;                                          'memory' => '2048MB'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'free_list_size' => '181200',
##&nbsp;                            'active_virtual_pages' => '161763'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'cpu' => [
##&nbsp;                       {
##&nbsp;                         'system_time' => '1',
##&nbsp;                         'processor_idle_time_during_outstanding_disk' => '1',
##&nbsp;                         'processor_idle_time' => '98',
##&nbsp;                         'user_time' => '1'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'pages_paged_out_paging_space' => '0',
##&nbsp;                          'pages_paged_in_paging_space' => '0',
##&nbsp;                          'pages_scanned' => '0',
##&nbsp;                          'file_page_outs_per_second' => '6',
##&nbsp;                          'pages_freed' => '0',
##&nbsp;                          'file_page_ins_per_second' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'kernel_thread_states' => [
##&nbsp;                                        {
##&nbsp;                                          'threads_waiting_on_io' => '0',
##&nbsp;                                          'runnable_kernel_threads' => '1',
##&nbsp;                                          'kernel_threads_in_vmm_wait_queue' => '1'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'faults' => [
##&nbsp;                          {
##&nbsp;                            'system_calls' => '321',
##&nbsp;                            'device_interrupts' => '21',
##&nbsp;                            'context_switches' => '104'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vmstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my %res = (
        kthr => 'kernel_thread_states',
        r    => 'runnable_kernel_threads',
        b    => 'kernel_threads_in_vmm_wait_queue',
        avm  => 'active_virtual_pages',
        fre  => 'free_list_size',
        siz  => 'no_frames_of_specified_page_size',
        Page => 'indicates_the_relevant_page_faults',
        re   => 'pager_io_list',
        pi   => 'pages_paged_in_paging_space',
        po   => 'pages_paged_out_paging_space',
        fr   => 'pages_freed',
        sr   => 'pages_scanned',
        cy   => 'clock_cycles',
        in   => 'device_interrupts',
        sy   => 'system_calls',
        sy1  => 'system_time',
        cs   => 'context_switches',
        us   => 'user_time',
        id   => 'processor_idle_time',
        wa   => 'processor_idle_time_during_outstanding_disk',
        pc   => 'no_of_physical_processors_consumed',
        ec   => 'percentage_of_entitled_capacity_consumed',
        p    => 'threads_waiting_on_io',
        fi   => 'file_page_ins_per_second',
        fo   => 'file_page_outs_per_second',
        psz  => 'pagesize',
        pgsz => 'pagesize',
        alp  => 'no_of_large_pages_currently_in_use',
        flp  => 'no_of_large_pages_on_large_page_freelist',
        hr   => 'hours',
        mi   => 'minutes',
        se   => 'seconds',
        lcpu => 'number_of_logical_processors',
        mem  => 'memory'
    );

    if ( $cmd =~ /-v|-s|-f/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /(\d+\.*\d*)\s+(.*)/ ) {
                my $key   = $2;
                my $value = $1;
                $key =~ s/\.|\///;
                $key =~ s/-/_/;
                $result_array[0]->{ to_key_name( item => $key ) } = $value;
            }
        }
    } elsif ( $cmd =~ /-i/ ) {
        my $c = 0;
        my @head = split( /\s+/, $lines[0] );
        for ( my $i = 1; $i <= $#lines; $i++ ) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $c++;
        }
    } elsif ( $cmd =~ /-P/ ) {
        my $count = -1;
        my ( @values, @head1, @head2 );
        @head1 = split( /\s+/, $lines[1] );
        @head2 = split( /\s+/, $lines[3] );
        if ( $lines[0] =~ /(\w+\s+\w+)\:\s+(\w+\=\d+)\s*(\w+\=\w+)/ ) {
            $count++;
            my @val  = split( /=/, $2 );
            my @val1 = split( /=/, $3 );
            $result_array[$count]->{ to_key_name( item => $1 ) }->[0]
                ->{ to_key_name( item => $res{ $val[0] } ) } = $val[1];
            $result_array[$count]->{ to_key_name( item => $1 ) }->[0]
                ->{ to_key_name( item => $res{ $val1[0] } ) } = $val1[1];
        }
        for ( my $i = 4; $i <= $#lines; $i++ ) {
            $count++;
            if ( $lines[$i] =~ /(\d+).*/ ) {
                @values = split( /\s+/, $lines[$i] );
                $result_array[$count]->{ $res{ $head1[0] } } = $values[0];
                for (
                    my $j = ( $#values - 9 );
                    $j <= ( $#values - 7 );
                    $j++
                    ) {
                    $result_array[$count]->{ $head1[1] }->[0]
                        ->{ $res{ $head2[$j] } } = $values[ $j + 1 ];
                }
                for (
                    my $j = ( $#values - 6 );
                    $j <= ( $#values - 1 );
                    $j++
                    ) {
                    $result_array[$count]->{ $head1[2] }->[0]
                        ->{ $res{ $head2[$j] } } = $values[ $j + 1 ];
                }
            }
        }
    } elsif ( $cmd =~ /-w|-l|-t|-p|-I/ ) {
        my $count = -1;
        my $index = 0;
        my ( @values, @head1, @head2, @head3 );
        my ( @kthr, @memory, @page, @faults, @pos );
        if ( $lines[0] =~ /(\w+\s+\w+)\:\s+(\w+\=\d+)\s*(\w+\=\w+)/ ) {
            $count++;
            my @val  = split( /=/, $2 );
            my @val1 = split( /=/, $3 );
            $result_array[$count]->{ to_key_name( item => $1 ) }->[0]
                ->{ to_key_name( item => $res{ $val[0] } ) } = $val[1];
            $result_array[$count]->{ to_key_name( item => $1 ) }->[0]
                ->{ to_key_name( item => $res{ $val1[0] } ) } = $val1[1];
        }
        @head1 = split( /\s+/, $lines[1] );
        $head1[5] =~ s/-/_/;
        @head3 = split( /\s+/, $lines[3] );
        for ( my $i = 4; $i <= $#lines; $i++ ) {
            $count++;
            while ( $lines[2] =~ m/\s/g ) {
                my $temp_pos = pos( $lines[2] );
                push( @pos, $temp_pos );
            }
            my $start = 0;
            my $temp  = $lines[3];
            for ( my $j = 0; $j <= $#pos; $j++ ) {
                my $value;
                my $end    = $pos[$j] - 2;
                my $start  = $pos[ $j - 1 ] if ( $j != 0 );
                my $length = $end - $start;
                $value = substr( $temp, $start, $length );
                $value =~ s/^\s+|\s+$//;
                push( @head2, $value );
            }
            @kthr   = split( /\s+/, $head2[0] );
            @memory = split( /\s+/, $head2[1] );
            @page   = split( /\s+/, $head2[2] );
            @faults = split( /\s+/, $head2[3] );
            $index  = ( $#kthr + 1 + $#memory + 1 + $#page + 1 );
            if ( $lines[$i] =~ /(\d+).*/ && $lines[$i] !~ /\d[A-Z]/ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#kthr; $j++ ) {
                    $result_array[$count]->{ $res{ $head1[0] } }->[0]
                        ->{ $res{ $kthr[$j] } } = $values[$j];
                }
                for ( my $j = 0; $j <= $#memory; $j++ ) {
                    $result_array[$count]->{ $head1[1] }->[0]
                        ->{ $res{ $memory[$j] } }
                        = $values[ $j + ( $#kthr + 1 ) ];
                }
                for ( my $j = 0; $j <= $#page; $j++ ) {
                    $result_array[$count]->{ $head1[2] }->[0]
                        ->{ $res{ $page[$j] } }
                        = $values[ $j + ( $#kthr + $#memory + 2 ) ];
                }
                for ( my $j = 0; $j <= $#faults; $j++ ) {
                    $result_array[$count]->{ $head1[3] }->[0]
                        ->{ $res{ $faults[$j] } }
                        = $values[ $j + ( $#kthr + $#memory + $#page + 3 ) ];
                }
            }
            if ( $cmd =~ /-I/ ) {
                for ( my $j = $index + 5; $j <= $index + 6; $j++ ) {
                    $result_array[$count]->{ $head1[4] }->[0]
                        ->{ $res{ $head3[$j] } } = $values[$j];
                }
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{us} }
                    = $values[ $index + 3 ];
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{sy1} }
                    = $values[ $index + 4 ];
            }
            if ( $cmd =~ /-w/ ) {
                for ( my $j = $index + 5; $j <= $index + 6; $j++ ) {
                    $result_array[$count]->{ $head1[4] }->[0]
                        ->{ $res{ $head3[$j] } } = $values[$j];
                }
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{us} }
                    = $values[ $index + 3 ];
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{sy1} }
                    = $values[ $index + 4 ];
            }
            if ( $cmd =~ /-t/ ) {
                for ( my $j = $index + 5; $j <= $index + 6; $j++ ) {
                    $result_array[$count]->{ $head1[4] }->[0]
                        ->{ $res{ $head3[$j] } } = $values[$j];
                }
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{us} }
                    = $values[ $index + 3 ];
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{sy1} }
                    = $values[ $index + 4 ];
                my @val = split( /:/, $values[17] ) if ( exists $values[17] );
                for ( my $j = $index + 7; $j <= $index + 9; $j++ ) {
                    $result_array[$count]->{ $head1[5] }->[0]
                        ->{ $res{ $head3[$j] } }
                        = $val[ $j - ( $index + 7 ) ];
                }
            }
            if ( $cmd =~ /-l/ ) {
                for ( my $j = $index + 5; $j <= $index + 6; $j++ ) {
                    $result_array[$count]->{ $head1[4] }->[0]
                        ->{ $res{ $head3[$j] } } = $values[$j];
                }
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{us} }
                    = $values[ $index + 3 ];
                $result_array[$count]->{ $head1[4] }->[0]->{ $res{sy1} }
                    = $values[ $index + 4 ];
                for ( my $k = ( $index + 7 ); $k <= ( $index + 8 ); $k++ ) {
                    $result_array[$count]->{ $head1[5] }->[0]
                        ->{ $res{ $head3[$k] } } = $values[$k];
                }
            }
            if ( $cmd =~ /-p/ ) {
                if ( $lines[$i] =~ /(\d+).*/ && $lines[$i] !~ /\d[A-Z]/ ) {
                    for ( my $j = $index + 5; $j <= $index + 6; $j++ ) {
                        $result_array[$count]->{ $head1[4] }->[0]
                            ->{ $res{ $head3[$j] } } = $values[$j];
                    }
                    $result_array[$count]->{ $head1[4] }->[0]->{ $res{us} }
                        = $values[ $index + 3 ];
                    $result_array[$count]->{ $head1[4] }->[0]->{ $res{sy1} }
                        = $values[ $index + 4 ];
                }
                if ( $lines[$i] =~ /psz.*/ ) {
                    my @temp_head = split( /\s+/, $lines[$i] );
                    my @val       = split( /\s+/, $lines[ $i + 1 ] );
                    $result_array[$count]->{ $res{ $temp_head[0] } }
                        = $val[0];
                    for ( my $j = ( $#val - 8 ); $j <= ( $#val - 7 ); $j++ ) {
                        $result_array[$count]->{ $head1[1] }->[0]
                            ->{ $res{ $temp_head[$j] } } = $val[$j];
                    }
                    for ( my $j = ( $#val - 6 ); $j <= ( $#val - 1 ); $j++ ) {
                        $result_array[$count]->{ $head1[2] }->[0]
                            ->{ $res{ $temp_head[$j] } } = $val[$j];
                    }
                    $result_array[$count]->{ $res{ $temp_head[-1] } }
                        = $val[-1];
                }
            }

        }
    }
    return \@result_array;
} ## end sub vmstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message );
    if ( $raw_output =~ /([Uu]sage:|vmstat:)\s+\w+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ( $raw_output
        =~ /vmstat:\s+\d+.*|vmstat: Invalid page.*|vmstat: A flag requires.*/
        ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

