#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#   Note originally written by Madhav Shekar

## @summary API Set for DFM CLI commands
## @author dl-nacl-dev@netapp.com, dl-sm-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::CLI::DFM;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::CLI);

#
# APISet imports
#
use NACL::APISet::Session::CLI::DFM;
use NACL::APISet::_Mixins::CLI qw(_load_version_xml);

#
#Exceptions imported
#
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Linux API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API, assigning a default CDEF path
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;
    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file( $self->{cdef_path} );

    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }
    my ( $release, $codename );
    if ( defined $self->{version_manager} ) {
        try {

            $release = $self->{version_manager}
                ->get_version_attribute( attribute => 'release' );
            $codename = $self->{version_manager}
                ->map_release_to_codename( release => $release );

        }
        catch NATE::BaseException with {

            my $exception = shift;
            if ( ( defined $release ) && !( defined $codename ) ) {
                $Log->warn(
                    "\nRelease info is found as $release but it is not mapped to a codename."
                        . "\nHence Loading the default parser"
                        . "\nPlease contact dl-sm-nacl-dev about this warning"
                );
            }
            $Log->debug(
                "Error in obtaining DFM version  " . $exception->text() );
        };
    }

    $self->SUPER::_init_API( %args, dfm_version_info => $codename );

    $Log->exit() if $may_exit;
}

###############################################################################
# Method:    find_cdef_directory
# Objective: To find the cdef directory for DFM based on various version related
#            attributes obtained from the service.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub find_cdef_directory {
    $Log->enter() if $may_enter;
    my $self = shift();

    # Load CDEFs from the CDEF directories which we maintain based on branch
    # and release information.
    my %required_attributes = ( "DFM" => ["release"], );
    my $cdef_dir;
    my $attr_info;
    my $type = $self->{set};

    if ( defined( $self->{version_manager} ) ) {

        $attr_info = $self->_find_cdef_directory(
            required_attributes => $required_attributes{$type} );
    }
    my $release = $attr_info->{release};
    if ( defined $release ) {

        # release should be normalized in lowercase so that its case
        # insensitive when we try to map it to its codename.
        $release = lc($release);
        try {
            $release = $self->{version_manager}
                ->map_release_to_codename( release => $release );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Caught an exception while trying to map $release "
                    . "to its codename: "
                    . $ex->text() );

            # Emit a warning, set the normalized release name to undef
            # and then proceed.
            $Log->warn( "Versioning hit the error: "
                    . $ex->text()
                    . "\nRelease mapping may be missing for release $release."
                    . "\nPlease contact dl-sm-nacl-dev about this warning."
                    . "\n Loading the default CDEF Directory." );
            $release = undef;
        };
    }
    
    # Load version.xml file
    my $libxml_obj = $self->_load_version_xml();
    
    # Discover the CDEF path
    $cdef_dir = $libxml_obj->find("/version/host/cli/dfm/default_cdef_dir")
        ->string_value;

    unless ( ( defined $release ) ) {
        $Log->debug("Release information could not be obtained");
        $Log->exit() if $may_exit;
        return $cdef_dir;
    }

    # Discover the CDEF path for specific release
    my $release_cdef_dir = $libxml_obj->find("/version/host/cli/dfm/release[\@value='" .
                    $release . "']/release_cdef_dir")->string_value();
    
    if (! defined $release_cdef_dir or $release_cdef_dir eq "") {
        $Log->debug("CDEF directory is not yet supported for $release");

        $Log->warn( "CDEF directory corresponding to release '$release' "
                . "doesn't exist. Please contact dl-sm-nacl-dev for this warning."
                . " Loading CDEFs from default CDEF directory defined for the"
                . " set '$self->{set}'." );
    } else {
        $Log->debug(
            "A release specific CDEF directory is found for " . $release );
        $cdef_dir = $release_cdef_dir;
    }

    $Log->exit() if $may_exit;
    return $cdef_dir;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::CLI::DFM - API Set for DFM CLI commands

=head1 SYNOPSIS

use NACL::APISet::Host::CLI::DFM;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI::DFM> is used to connect to and send CLI commands
to the DFM Host.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::CLI::DFM>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Host::CLI::DFM->new( %args );

=item Arguments

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Host::CLI for details.

=back

=head2 find_cdef_directory

This method gets the information of various version related attributes and uses
them to autodiscover the CDEF path. The data about the CDEF directory to
be loaded is maintained in a separate XML file from which it finds the CDEF path
based on the discovered version attributes. The version attributes to be
discovered will vary depending on the category and set.

=over

=item Synopsis

my $cdef_dir = $Api_Set_Obj->find_cdef_directory();

=item Arguments

Nothing.

=item Returns

Returns the relative path to CDEF directory for DFM.

=item Returns

A reference to a new DFM API Set.

=back

=head1 PARAMETERS

=head2 DFM_API_PERF_FLAG

=over

=item Synopsis
(Optional)
  This is an ENV value, which is used as a flag. 
  When the value it set to "TRUE" (case-insensitive) the raw execution time for the command 
  is calculated. If the value is set to something other than "TRUE" or if it not present at all,
  then by default there is no raw execution time calculated. The value returned by
  $response->get_raw_execution_time() would be undef in this case.
  The value for this can be set in the .ntest file, in the testbed file or the plan file. It can
  also be passed along the command line.
  
=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

