# Copyright (c) 2015 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse cgstat command in the systemshell
##
## @description
##    This is a library to parse the raw output of cgstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::cgstat;
##
##    cgstat(%args);
##
## @author parikshi@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::cgstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::cgstat
##
## @summary Parses the raw output from the "cgstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cgstat with any of its options as described in the
##    corresponding cdef file 'cgstat.cdefs'
## @synopsis
##      cgstat(%args);
## INPUTS TO METHOD CALL 'cgstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> BOOLEAN OPTION </tr>
## <tr> <td> '-s' </td> <td> 'stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'only_counters' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'only_histograms' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'cg_id' </td> <td> '0' </td> </tr>
## <tr> <td> '-G' </td> <td> 'global_stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'toeplitz' </td> <td> '1' </td> </tr>
## <tr> <td> '--hash' </td> <td> 'hash' </td> <td> '0' </td> </tr>
## <tr> <td> '-v4' </td> <td> 'v4' </td> <td> '1' </td> </tr>
## <tr> <td> '-v6' </td> <td> 'v6' </td> <td> '1' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'foreign_address' </td> <td> '0' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'foreign_port' </td> <td> '0' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'source_address' </td> <td> '0' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'source_port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command:
##&nbsp;cgstat -s -G
##&nbsp;$VAR1 = [
##&nbsp;			{
##&nbsp;				'cg_id_number' =>{
##&nbsp;									'counters' =>   {
##&nbsp;														'cntr_name1' => 'ctr_value1',
##&nbsp;														'cntr_name2' => 'ctr_value2',
##&nbsp;														...
##&nbsp;													},
##&nbsp;									'histograms' => {
##&nbsp;														'histo_name1' => {
##&nbsp;																			'sum' => 'sum_value',
##&nbsp;																			'col_heading1' => 'col_value1',
##&nbsp;																			'col_heading2' => 'col_value2',
##&nbsp;																		},
##&nbsp;														'histo_name2' => {
##&nbsp;																			'sum' => 'sum_value',
##&nbsp;																			'col_heading1' => 'col_value1',
##&nbsp;																			'col_heading2' => 'col_value2',
##&nbsp;																		},
##&nbsp;														...
##&nbsp;													},
##&nbsp;								}
##&nbsp;				...,
##&nbsp;				'global' =>     {
##&nbsp;									'global_cntr1' => 'value1',
##&nbsp;									'global_cntr2' => 'value2',
##&nbsp;									...
##&nbsp;								}
##&nbsp;			}
##&nbsp;		];
##&nbsp;
##&nbsp;Command:
##&nbsp;cgstat -t --hash 1234
##&nbsp;OR
##&nbsp;cgstat -t -v4 1.2.3.4 1234 5.6.7.8 5678
##&nbsp;$VAR1 = [
##&nbsp;			{
##&nbsp;				'toeplitz' =>   {
##&nbsp;									'hash' => 'value1',
##&nbsp;									'legacy_space_cg_id' => 'val2',
##&nbsp;									'default_space_cg_id' => 'val3',
##&nbsp;								}
##&nbsp;			}
##&nbsp;		];
sub cgstat {
    my %args = @_;

    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    _check_error(raw_output => $raw_output, response_object => $response);
    my $cmd = $response->get_command();

    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $response
    );
    my (@cgid_index_arr, @cgid_arr);
    for (my $i = 0; $i < scalar(@lines); $i++) {
        if ($lines[$i] =~ /^CG ID/) {
            push(@cgid_index_arr, $i);
            my @line_split = split(/:/, $lines[$i]);
            push(@cgid_arr, $line_split[1]);
        }
    }
    if (!@cgid_index_arr) {
        if (!($lines[0] =~ /CG Global Stats/) && !($lines[0] =~ /Toeplitz/)) {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "Command returned output: $raw_output",
                response_object => $response
            );
        }
    }
    my %ret_hash;
    if ($cmd =~ /-s/) {
        if ($cmd =~ /-g/) {
            my @global_lines = _copy_arr(\@lines, 2, 5);
            my $hashref = _parse_lines(\@global_lines);
            $ret_hash{global} = $hashref;
        }
        if ($cmd =~ /-C/ || !($cmd =~ /-H/)) {
            my @ctr_inds = _get_arr_indices(\@lines, 'Counters:');
            for (my $i = 0; $i < scalar(@cgid_index_arr); $i++) {
                my %ctr_hash;
                my @stat_ctr_lines
                    = _copy_arr(\@lines, $ctr_inds[$i] + 2, 32);
                my $hashref = _parse_lines(\@stat_ctr_lines);
                $ctr_hash{counters} = $hashref;
                my $temp = $ret_hash{$cgid_arr[$i]};
                $ret_hash{$cgid_arr[$i]}
                    = (defined $ret_hash{$cgid_arr[$i]})
                    ? {%{$temp}, %ctr_hash}
                    : \%ctr_hash;
            }
        }

        # TODO: Fix the histogram parser for HN.0 changes
        if ($cmd =~ /-H/ || !($cmd =~ /-C/)) {
            for (my $i = 0; $i < scalar(@cgid_index_arr); $i++) {
                my (%hist_hash, $hashref);
                my $cgid_block_end_length
                    = ($i == (scalar(@cgid_index_arr) - 1))
                    ? scalar(@lines) - $cgid_index_arr[$i]
                    : $cgid_index_arr[$i + 1] - $cgid_index_arr[$i];
                my @cgid_stat_lines = _copy_arr(\@lines, $cgid_index_arr[$i],
                    $cgid_block_end_length);
                my @hist_inds
                    = _get_arr_indices(\@cgid_stat_lines, 'histogram:');
                my @hist_end_inds = _get_arr_indices(\@cgid_stat_lines,
                    'Sum of all histogram values:');
                for (my $j = 0; $j < scalar(@hist_inds); $j++) {
                    my @stat_histo_lines
                        = _copy_arr(\@cgid_stat_lines, $hist_inds[$j],
                        $hist_end_inds[$j] - $hist_inds[$j] + 1);
                    my $temp_hashref = _parse_stat_histos(\@stat_histo_lines);
                    $hashref
                        = (defined $hashref)
                        ? {%{$hashref}, %{$temp_hashref}}
                        : $temp_hashref;
                }
                $hist_hash{histograms} = $hashref;
                my $temp = $ret_hash{$cgid_arr[$i]};
                $ret_hash{$cgid_arr[$i]}
                    = (defined $ret_hash{$cgid_arr[$i]})
                    ? {%{$temp}, %hist_hash}
                    : \%hist_hash;
            }
        }
    } elsif ($cmd =~ /-t/) {
        $ret_hash{toeplitz} = _parse_lines(\@lines);
    }
    my @ret_arr;
    push(@ret_arr, \%ret_hash);
    return \@ret_arr if @ret_arr;
    return undef;
} ## end sub cgstat

#checks for errors in the output and throws an appropriate exception
sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ($error, $error_message) = undef;
    if ($raw_output =~ /invalid option/) {
        $error         = 'InvalidParam';
        $error_message = $raw_output;
    } elsif ($raw_output =~ /Invalid/i && $raw_output =~ /Usage/i) {
        $error_message = $raw_output;
        $error         = 'InvalidParamValue';
    } elsif ($raw_output =~ /Usage/) {
        $error_message = $raw_output;
        $error         = 'UnexpectedOutput';
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => 'Execution status was' . $error_level
                . 'indicating that the command has failed',
            response_object => $resp
        );
    }
	return 0;
} ## end sub _check_error

#Takes as input an array of lines, each of which are of the form 
#"text goes here: 1234" and returns a hashref containing the line
#as the key and the number as the value.
sub _parse_lines {
    my $arr_ref = $_[0];

    my $hash;
    foreach my $line (@{$arr_ref}) {
        my @line_arr = split(':', $line);
        my $key = lc(_trim($line_arr[0]));
        $key =~ tr/ /_/;
        my %temphash;
        $temphash{$key} = _trim($line_arr[1]);
        $hash = (defined $hash) ? {%temphash, %{$hash}} : \%temphash;
    }
    return $hash;
} ## end sub _parse_lines

#Takes as input an array of lines which represent a histogram.
# This sub parses the lines into a hash and returns its reference
sub _parse_stat_histos {
    my $arr_ref = $_[0];

    my (%hash, %temphash);
    my @arr = @{$arr_ref};

    my @key_arr = split('histogram:', $arr[0]);
    my $key = lc(_trim($key_arr[0]));
    $key =~ s/\.//g;
    $key =~ tr/ /_/;

    my @sum_arr = split(':', $arr[-1]);
    my $sum = _trim($sum_arr[1]);
    $temphash{sum} = $sum;

    splice(@arr, 0, 1);
    splice(@arr, -1);

    for (my $i = 0; $i < scalar(@arr) / 2; $i++) {
        my @hist_keys   = split(/\t/, $arr[$i * 2]);
        my @hist_values = split(/\t/, $arr[$i * 2 + 1]);
        for (my $j = 0; $j < scalar(@hist_keys); $j++) {
            $temphash{$hist_keys[$j]} = $hist_values[$j];
        }
    }
    $hash{$key} = \%temphash;
    return \%hash;
} ## end sub _parse_stat_histos

sub _trim {
    my $str = $_[0];

    if ($str) {
        $str =~ s/^\s+//;
        $str =~ s/\s+$//;
    }
    return $str;
} ## end sub _trim

#Copies a block of array into another array by taking the starting point
#as the offset and the consequent number of lines to copy. Returns an array.
sub _copy_arr {
    my $arr_ref = $_[0];
    my $offset  = $_[1];
    my $length  = $_[2];

    my @arr = @{$arr_ref};
    my @ret_arr;
    for (my $i = 0; $i < $length; $i++) {
        push(@ret_arr, $arr[$offset + $i]);
    }
    return @ret_arr;
} ## end sub _copy_arr

sub _get_arr_indices {
    my $arr_ref = $_[0];
    my $regex   = $_[1];

    my @inds;
    my @arr = @{$arr_ref};
    for (my $i = 0; $i < scalar(@arr); $i++) {
        if ($arr[$i] =~ /$regex/) {
            push(@inds, $i);
        }
    }
    return @inds;
} ## end sub _get_arr_indices
1;
