# $Id$
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    cm_client(%args);
##
## @summary Routines to parse cm_client command output
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com

package NACL::APISet::Host::CLI::Freebsd::default::cm_client;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _parse_for_errors {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(Usage.*)/ || $output =~ /(Warning:.*)/i ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $output =~ /(.*not a known process.*)/
        || $output =~ /(.*Object \S+  was not found.*)/i ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
} ## end sub _parse_for_errors

## @name ::cm_client
##
## @summary Parses the raw output from the "cm_client" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cm_client with any of its options as described in the
##    corresponding cdef file 'cm_client.cdefs'
## @synopsis
##      cm_client(%args);
## INPUTS TO METHOD CALL 'cm_client' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-o' </td> <td> 'object' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'instances' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'priv' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'action' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'server' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'process' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'counters' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cm_client -o zapi -a object-instance-list-info -p mgwd
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Creating RPC client.
##&nbsp;Sucessfully created RPC client.
##&nbsp;/*******************************************/
##&nbsp;/* TEST OUT perf_object_instance_list_info */
##&nbsp;/*******************************************/
##&nbsp;	Calling perf_object_instance_list_info_1 (zapi, test)
##&nbsp;	Called: print_perf_object_instance_list_info_results
##&nbsp;	Result:
##&nbsp;		Num Instances: 13
##&nbsp;			object_name:"zapi" instance_name:"d-system-get-boot-state"
##&nbsp;			object_name:"zapi" instance_name:"d-system-get-env-status"
##&nbsp;			object_name:"zapi" instance_name:"d-system-get-info"
##&nbsp;			object_name:"zapi" instance_name:"perf-object-get-instances"
##&nbsp;			object_name:"zapi" instance_name:"sf-hwassist-status"
##&nbsp;			object_name:"zapi" instance_name:"sf-status"
##&nbsp;			object_name:"zapi" instance_name:"snapmirror-get-status"
##&nbsp;			object_name:"zapi" instance_name:"snapmirror-list-destinations"
##&nbsp;			object_name:"zapi" instance_name:"storage-array-get-config-summary"
##&nbsp;			object_name:"zapi" instance_name:"storage-disk-get-iter"
##&nbsp;			object_name:"zapi" instance_name:"storage-initiator-errors-list-info"
##&nbsp;			object_name:"zapi" instance_name:"storage-initiator-path-list-info"
##&nbsp;			object_name:"zapi" instance_name:"system-set-node-stable-after-startup"
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'num_instances' => '13',
##&nbsp;            'object_instance_list_info' => [
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'd-system-get-boot-state',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'd-system-get-env-status',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'd-system-get-info',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'perf-object-get-instances',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'sf-hwassist-status',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'sf-status',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'snapmirror-get-status',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'snapmirror-list-destinations',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'storage-array-get-config-summary',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'storage-disk-get-iter',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'storage-initiator-errors-list-info',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'storage-initiator-path-list-info',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'instance_name' => 'system-set-node-stable-after-startup',
##&nbsp;                                               'object_name' => 'zapi'
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cm_client {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $j = 0;
    my $heading;

    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines
        = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arraylines) {
        if ( $line
            =~ /((Timestamp):\s+(\d+))?.*(Num\s+(Objects|Instances|Counters)):\s+(\d+)/
            ) {
            $result_array[0]->{ to_key_name( item => $2 ) } = $3 if ($1);
            $result_array[0]->{ to_key_name( item => $4 ) } = $6;
        } elsif ( $line =~ /TEST\s+OUT\s+perf_(.*)\s+\*/ ) {
            $heading = "$1";
        } elsif ( $line =~ /(\S+:"\S+".*)(\s+translation:.*)/
            || $line =~ /(\S+:"\S+".*)/ ) {
            my $text             = $1;
            my $translation_text = $2;
            if ($translation_text) {
                $translation_text
                    =~ /translation: \{ input_counter_name:"(.*)" , function_name:"(.*)" \}/;
                $result_array[0]->{$heading}->[$j]->{translation}->[0]
                    ->{input_counter_name} = $1;
                $result_array[0]->{$heading}->[$j]->{translation}->[0]
                    ->{function_name} = $2;
            }
            $text =~ s/"\s+/"######/g;
            my @key_values = split( /######/, $text );
            foreach my $key_val (@key_values) {
                if ( $key_val =~ /(.*):"(.*)"/ || $key_val =~ /(.*):"(.*)/ ) {
                    $result_array[0]->{$heading}->[$j]
                        ->{ to_key_name( item => $1 ) } = $2;
                }
            }
            $j++;
        }
    }
    return \@result_array;
}    ##end of sub cm_client

1;
