#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Freebsd/default/coretool.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse coretool command
##
## @description
##    This is a library to parse the raw outputs of coretool command.
##    The function name will be the same as command name.
##
## @synopsis
##     use NACL::APISet::Host::CLI::Freebsd::default::coretool;
##
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::coretool;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _parse_for_errors {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $resp          = $args{"response_object"};
    my ($error);
    if ( $output =~ /(Command not found|Shared object.*not found)/){
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    elsif (( $output =~ /(Unable to open core file.*)/) || ($output =~ /(Could not initialize coretool)/) || ($output =~ /(No such file or directory)/)) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    elsif ($output =~ /(usage:.*)/i) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return;
}

## @name :: coretool
##
## @summary Parses the raw output from the "coretool" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coretool with core file name option as described in the
##    corresponding cdef file 'coretool.cdefs'
## @synopsis
##      coretool(%args);
## INPUTS TO METHOD CALL 'coretool' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <tr>
## <tr> <td> No switch for this key </td> <td> 'core-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /mroot/etc/crash/coretool /mroot/etc/crash/core.151755310.2012-04-19.12_19_50.nz
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'page_directory' => '0x0000000022ec4000',
##&nbsp;             'file_size' => '19,326,930,944',
##&nbsp;             'register_file_size' => '192',
##&nbsp;             'missing_blocks' => '0',
##&nbsp;             'processor_type' => 'CPU_AMD64 (4)',
##&nbsp;             'magic_number' => 'NETAPP_LMEM5_COREMAGIC (0xdddc1114)',
##&nbsp;             'nr_of_segments' => '6',
##&nbsp;             'registers' => [
##&nbsp;                              {
##&nbsp;                                'sp' => 'ffffff801d096574',
##&nbsp;                                'bp' => 'ffffff801d0965d0',
##&nbsp;                                '11' => 'ffffffff89326960',
##&nbsp;                                'di' => 'ffffffff8bacf4a0',
##&nbsp;                                'r9' => 'ffffffff886d7eb6',
##&nbsp;                               'ax' => '0000000000000000',
##&nbsp;                               'dx' => 'ffffffff886d82ba',
##&nbsp;                               'bx' => 'ffffff00241027e0',
##&nbsp;                               'r8' => '0000000000000000',
##&nbsp;                               'cx' => '0000000000000004',
##&nbsp;                               '13' => '0000000000000001',
##&nbsp;                               'si' => 'ffffffff886d82c7',
##&nbsp;                               '12' => '00007fffffffee73',
##&nbsp;                               '14' => '00007fffffffee67',
##&nbsp;                               '15' => '0000000000000002',
##&nbsp;                               '10' => 'ffffffff886d8173'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'ident_info' => [
##&nbsp;                              {
##&nbsp;                                'version' => '/x/eng/rlse23/DOT/nightly/sn_ipv6N_120319_1623/: Mon Mar 19 16:51:40 PDT 2012',
##&nbsp;                                'nvram_systemid' => '0151755310',
##&nbsp;                                'model' => 'FAS3170',
##&nbsp;                                'serialnumber' => '700000410005',
##&nbsp;                                'cc_flags' => 'amd64, x86_64.debug',
##&nbsp;                                'hostname' => 'fas3170-210-08',
##&nbsp;                                'sw_arch' => 'gx',
##&nbsp;                                'appliance' => 'filer'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'filename' => '/mroot/etc/crash/core.151755310.2012-04-19.12_19_50.nz',
##&nbsp;            'panic_message' => 'sysctl debug.panic: 1 in process sysctl on release sn_ipv6N_120319_1623 (C)',
##&nbsp;            'physical_memeory_segments_dumped' => [
##&nbsp;                                                    '0x0000000000000000-0x0000000000097fff (       622,592) @ file 4,096 1',
##&nbsp;                                                    '0x0000000000100000-0x000000002fffffff (   804,257,792) @ file 626,688 1',
##&nbsp;                                                    '0x0000000100000000-0x0000000204dbefff ( 4,376,489,984) @ file 804,884,480 1',
##&nbsp;                                                    '0x00000004bdeb3000-0x00000004cfffffff (   303,353,856) @ file 5,181,374,464 1',
##&nbsp;                                                    '0x0000000204dbf000-0x00000004bdeb2fff (11,694,718,976) @ file 5,484,728,320 1',
##&nbsp;                                                    '0x000000fc80000000-0x000000fcffffffff ( 2,147,483,648) @ file 17,179,447,296 5'
##&nbsp;                                                  ],
##&nbsp;            'panic_time' => 'Tue May 22 06:17:10 -836377 PST'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub coretool {
    my %args        = @_;
    my (@result,@segment, %hash, $i, $j);
    my $text = undef;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error = _parse_for_errors(
        "raw_output"    => $raw_output,
        "response_object"   => $resp,
        "command"       => $command,
        
    );
    my @lines =
      get_lines( raw_output => $raw_output, response_object => $resp );
    for ( $i = 0 ; $i < $#lines ; $i++ ) {
        if ( $lines[$i] =~ /(filename|missing blocks|file size|magic number|processor type|register file size|nr of segments|Page directory)\s+(.*)/ ) {
            $hash{to_key_name( item => $1 ) } = $2;
        }
        elsif ( $lines[$i] =~ /PANIC MESSAGE:/ ) {
            $i++;
            $hash{panic_message} = $lines[$i];
        }
        elsif ( $lines[$i] =~ /PANIC TIME:/ ) {
            $i++;
            $hash{panic_time}    = $lines[$i];
        }
        elsif ( $lines[$i] =~ /REGISTERS:/ ) {
            $i++;
            while ( $lines[$i] =~
                /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/
              )
            {
                $hash{registers}->[0]->{$1}    = $2;
                $hash{registers}->[0]->{$3}    = $4;
                $hash{registers}->[0]->{$5}    = $6;
                $hash{registers}->[0]->{$7}    = $8;
                $j++;
                $i++;
            }
            $i--;
        }
        elsif ( $lines[$i] =~ /IDENT INFO:/ ) {
            $i++;
            while ( $lines[$i] =~ /(\S+)=(.*)/ ) {
                $hash{ident_info}->[0]->{$1} = $2;
                $i++;
            }
            $i--;
        }
        elsif ( $lines[$i] =~ /PHYSICAL MEMORY SEGMENTS DUMPED:/ ) {
            $i++;
            $j = 0;
            while ( $lines[$i] =~ /\d+[.]\s(.*)/ ) {
                $hash{physical_memeory_segments_dumped}->[$j] = $1;
                push (@segment,$1);
                $i++;
                $j++;
            }
            $i--;
        }
    }
    push( @result, \%hash );
    return \@result;
} ## end sub coretool
1;

