#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd df command"
##
## @description
##    This is a library to parse the raw output of Freebsd df command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "df" will be df()
##
## @synodfis
##    use df;
##
##    df(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::df;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::df
##
## @summary Parses the raw output from the "df" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from df with any of its options as described in the
##    corresponding cdef file 'df.cdefs'
## @synopsis
##      df(%args);
## INPUTS TO METHOD CALL 'df' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'previous-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kilo-byte' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'base-two' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'base-ten' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'local' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'specific-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'free-inodes' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-points' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'byte' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'all-file-types' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'giga-byte' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mega-byte' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'posix' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'grand-total' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = df
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem                      1K-blocks   Used   Avail Capacity  Mounted on
##&nbsp;/dev/md0                             3405   3350      55    98%    /
##&nbsp;devfs                                   1      1       0   100%    /dev
##&nbsp;/dev/ad0s2                        1048320 269696  778624    26%    /cfcard
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/md0',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'used' => '3350',
##&nbsp;            'capacity' => '98%',
##&nbsp;            '1k_blocks' => '3405',
##&nbsp;            'avail' => '55'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'devfs',
##&nbsp;            'mounted_on' => '/dev',
##&nbsp;            'used' => '1',
##&nbsp;            'capacity' => '100%',
##&nbsp;            '1k_blocks' => '1',
##&nbsp;            'avail' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/ad0s2',
##&nbsp;            'mounted_on' => '/cfcard',
##&nbsp;            'used' => '269696',
##&nbsp;            'capacity' => '26%',
##&nbsp;            '1k_blocks' => '1048320',
##&nbsp;            'avail' => '778624'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df  -T
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem                       Type    1K-blocks   Used   Avail Capacity  Mounted on
##&nbsp;/dev/md0                         ufs          3405   3350      55    98%    /
##&nbsp;devfs                            devfs           1      1       0   100%    /dev
##&nbsp;/dev/ad0s2                       msdosfs   1048320 269696  778624    26%    /cfcard
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/md0',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'used' => '3350',
##&nbsp;            'type' => 'ufs',
##&nbsp;            'capacity' => '98%',
##&nbsp;            '1k_blocks' => '3405',
##&nbsp;            'avail' => '55'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'devfs',
##&nbsp;            'mounted_on' => '/dev',
##&nbsp;            'used' => '1',
##&nbsp;            'type' => 'devfs',
##&nbsp;            'capacity' => '100%',
##&nbsp;            '1k_blocks' => '1',
##&nbsp;            'avail' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/ad0s2',
##&nbsp;            'mounted_on' => '/cfcard',
##&nbsp;            'used' => '269696',
##&nbsp;            'type' => 'msdosfs',
##&nbsp;            'capacity' => '26%',
##&nbsp;            '1k_blocks' => '1048320',
##&nbsp;            'avail' => '778624'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df  -t devfs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem 1K-blocks Used Avail Capacity  Mounted on
##&nbsp;devfs              1    1     0   100%    /dev
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => 'devfs',
##&nbsp;            'mounted_on' => '/dev',
##&nbsp;            'used' => '1',
##&nbsp;            'capacity' => '100%',
##&nbsp;            '1k_blocks' => '1',
##&nbsp;            'avail' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df  -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem                      1K-blocks    Used   Avail Capacity  Mounted on
##&nbsp;/dev/md0                             3405    3350      55    98%    /
##&nbsp;devfs                                   1       1       0   100%    /dev
##&nbsp;total                             5341030 1766161 3574459    33%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/md0',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'used' => '3350',
##&nbsp;            'capacity' => '98%',
##&nbsp;            '1k_blocks' => '3405',
##&nbsp;            'avail' => '55'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'devfs',
##&nbsp;            'mounted_on' => '/dev',
##&nbsp;            'used' => '1',
##&nbsp;            'capacity' => '100%',
##&nbsp;            '1k_blocks' => '1',
##&nbsp;            'avail' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            '1k_blocks' => '5341030',
##&nbsp;            'filesystem' => 'total',
##&nbsp;            'mounted_on' => undef,
##&nbsp;            'used' => '1766161',
##&nbsp;            'capacity' => '33%',
##&nbsp;            'avail' => '3574459'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub df {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    $lines[0] =~ s/Mounted\s+on/mounted_on/;
    my @head = split( /\s+/, $lines[0] );

    for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                $head[$j] =~ s/%iused/iused_percent/;
                $head[$j] =~ s/-/_/;
                $head[$j] =~ s/\d+//;
                $result_array[$count]->{ to_key_name( item => $head[$j] ) } =
                  $values[$j];
            }
            $count++;
    }
    return \@result_array;
} ## end sub route

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /reject:.*|No\s+such.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/i ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
