#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Freebsd getXXbyYY command output
##
## @description
##    This is a library to parse the raw output of Freebsd getXXbyYY command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::getXXbyYY;
##
##    getXXbyYY(%args);
##
## @author Abhishek Kumar (kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::getXXbyYY;

use NACL::APISet::Response::CLI::ParserUtils;
use strict;


sub _check_error {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error_message = $raw_output;
    my $error_level = $resp->get_execution_status();
    my $error = "CommandFailed"; # we'll try to classify it if possible
    if($raw_output=~/.*:\s:.*/ || $error_level != 0 || $raw_output=~/Error:/i ||
        $raw_output=~/hostname nor servname provided, or not known/i ||
        $raw_output=~/Usage:/i) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
}

## @name ::getXXbyYY
##
## @summary Parses the raw output from the "getXXbyYY" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from getXXbyYY with any of its options as described in the
##    corresponding cdef file 'getXXbyYY.cdefs'
## @synopsis
##      getXXbyYY(%args);
## INPUTS TO METHOD CALL 'getXXbyYY' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-a' </td> <td> 'ip' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'prog' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'service-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'hostname' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'addrfamily' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sudo getXXbyYY getaddrinfo -h floyd;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;      Canon; floyd.ns.multistore.netapp.com
##&nbsp;      IPv4: 40.1.1.1
##&nbsp;      IPv4: 40.1.1.4
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'ipv4' => [
##&nbsp;                         '40.1.1.1',
##&nbsp;                         '40.1.1.4'
##&nbsp;                       ],
##&nbsp;             'canon' => 'floyd.ns.multistore.netapp.com'
##&nbsp;           }
##&nbsp;        ];

sub getXXbyYY {
    my %args = @_;
    my  @result_array;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    _check_error(response_object=>$resp);
    my @lines = get_lines(raw_output=>$output,first_line=>'');
    my %hash;
    my @key;
    my @value;
    my $i=0;
    my $j=0;
    my $tempkey;
    my $tempvalue;
    my @temparr;
    if($lines[0] =~ /getXXbyYY\s/) {
            shift(@lines);
        }
    if( $command =~ /getaddrinfo|
                     getipnodebyname|
                     gethostbyname|
                     gethostbyaddr|
                     getpwbyname|
                     getgrbyname|
                     getpwbyuid|
                     netgrp|
                     getnameinfo|
                     getgrbygid|
                     getgrlist
                    /x
        ) {
        foreach my $line(@lines) {
            if( $line=~ /(.*)[:|;]\s*(.*)/) {
                $key[$i]=$1;
                $value[$i]=$2;
            }
            $i++;
        }
        while($j<$i) {
            if ($key[$j] eq "IPv4") {
                push(@{$hash{to_key_name(item => $key[$j])}},$value[$j]);
                $j++;
            }elsif($key[$j] =~ /IPv6\s*[:]+/i){
                @temparr = split(/[:]/,$key[$j],2);
		$tempvalue  = $temparr[1].":".$value[$j];
		$tempvalue =~ s/^\s+//;
		push(@{$hash{to_key_name(item => $temparr[0])}},$tempvalue);
                $j++;
	    } else {
                $hash{to_key_name(item => $key[$j])} = $value[$j];
                $j++;
            }
        }
    }else{
        return [];
    }
    push (@result_array, \%hash);
    return \@result_array;
}
1;
