# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Freebsd/default/gxmsg.pm#1 $
#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd gxmsg command
##
## @description
##    This is a library to parse the raw output of Freebsd gxmsg command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::gxmsg;
##
##    gxmsg(%args);
##
## @author swati.bajaj@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::gxmsg;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::gxmsg
##
## @summary Parses the raw output from the "gxmsg" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from gxmsg with any of its options as described in the 
##    corresponding cdef file 'gxmsg.cdefs'
## @synopsis
##      gxmsg(%args);
## INPUTS TO METHOD CALL 'gxmsg' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-l' </td> <td> 'literal' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'args' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'message-number' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'catalog-number' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'code' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = gxmsg -c 1 -m 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_string' => 'Can\'t encode rpc arguments.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub gxmsg {
    my %args = @_;
    
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error_level = $response->get_execution_status();
    my $command    = $response->get_actual_command();
    
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed\n$raw_output",
            response_object => $response);
    }
    
    # For few errors which are not caught by execution status,
    # we are providing this additional error check
    my $error_message;
    my $error = _check_error(
        raw_output    => $raw_output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    my @lines = get_lines(raw_output => $raw_output, 
                          can_throw=>0, response_object => $response);
                          
     #Parsing the output.
    if ( $lines[0] =~ /(.+)/ ) {
        my %hash;
        $hash{'error_string'} = $1;
        return [ \%hash ];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $response
        );
    }
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(can't find message catalog entry for)/ ) {
        $$error_message = $1;
        $error = "InvalidParamValue";
    } 
    return $error;
}

1;
