# Copyright (c) 2011-16 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd ifconfig command
##
## @description
##    This is a library to parse the raw output of Freebsd ifconfig command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::ifconfig;
##
##    ifconfig(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ifconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException ;
use NACL::APISet::Exceptions::MissingValueException ;
use NACL::APISet::Exceptions::InvalidParamValueException ;

## @name ::ifconfig
##
## @summary Parses the raw output from the "ifconfig" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ifconfig with any of its options as described in the 
##    corresponding cdef file 'ifconfig.cdefs'
## @synopsis
##      ifconfig(%args);
## INPUTS TO METHOD CALL 'ifconfig' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'address-family' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'lifetime' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'cloners' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'groupname' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'down-interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'media' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'up-interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr>
## <tr> <td> 'No switch for this key' </td> <td> 'dest-address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /sbin/ifconfig e3b
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        {
##&nbsp;                          'vserver_id' => '0',
##&nbsp;                          'lif_type' => 'LOOPBACKLIF',
##&nbsp;                          'prefixlen' => '128',
##&nbsp;                          'address' => '::1'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'flags' => '80c9<UP,LOOPBACK,RUNNING,NOARP,MULTICAST>',
##&nbsp;            'options' => '6407bb<RXCSUM,TXCSUM,VLAN_MTU,VLAN_HWTAGGING,JUMBO_MTU,VLAN_HWCSUM,TSO4,TSO6,LRO,VLAN_HWTSO,RXCSUM_IPV6,TXCSUM_IPV6>',
##&nbsp;            'status' => 'active',
##&nbsp;            'nd6_options' => '23<PERFORMNUD,ACCEPT_RTADV,AUTO_LINKLOCAL>',
##&nbsp;            'media' => 'Ethernet autoselect (10Gbase-Twinax <full-duplex,rxpause,txpause>)',
##&nbsp;            'mac' => '90:e2:ba:7c:e5:b1',
##&nbsp;            'ipv4' => [
##&nbsp;                        {
##&nbsp;                          'broadcast' => undef,
##&nbsp;                          'vserver_id' => '0',
##&nbsp;                          'lif_type' => 'LOOPBACKLIF',
##&nbsp;                          'netmask' => '0xff000000',
##&nbsp;                          'address' => '127.0.0.1'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'metric' => '0',
##&nbsp;            'name' => 'e3b',
##&nbsp;            'mtu' => '8232'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifconfig {
    my %args = @_;
    
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    _check_error(raw_output => $raw_output, response_object => $response);
    my $cmd = $response->get_command();
    # Currently parses output in the output form of 'ifconfig -a' and  only
    my @lines = get_lines(raw_output => $raw_output, 
                          can_throw=>0, response_object => $response);
    my (@arr, %hash);
    foreach (@lines) {
        # start of a new section, so push the content of the old hash to arr 
        # and then reset it and also parse the first line of the new section
        if ($_ =~ /^(\S+):\s+(flags=\S+.*)/) {
            my ($name, %hash2) = ($1, %hash);
            push (@arr, \%hash2);
            %hash = split( /\s+|=/, $2 );
            $hash{name} = $name;
            $hash{ipv6} = [];
            $hash{ipv4} = [];
        } elsif ($_ =~ /(.*?options)=(.*)/) {
            $hash{to_key_name( item => $1 )} = $2;
        } elsif ($_ =~ /ether\s+(.+)/) {
            $hash{mac} = $1;
        } elsif ($_ =~ /inet\s+(\S+)\s+netmask\s+(\S+)\s+(?:broadcast\s+(\S+)\s+)?(\S+).+(\d+)/) {
            # there could be more than one of this kind
            # hence a hashref is pushed into $hash{ipv4}
            my %ipv4_hash;
            # ipv4 address
            $ipv4_hash{address} = $1;
            $ipv4_hash{netmask} = $2;
            $ipv4_hash{broadcast} = $3;
            $ipv4_hash{lif_type} = $4;
            $ipv4_hash{vserver_id} = $5;
            push @{$hash{ipv4}}, \%ipv4_hash;
        } elsif ($_ =~ /inet6\s+(\S+)\s+prefixlen\s+(\S+)\s+(\S+)\s+.+(\d+)/) {
            # similar to ipv4 parsing
            my %ipv6_hash;
            $ipv6_hash{address} = $1;
            $ipv6_hash{prefixlen} = $2;
            $ipv6_hash{lif_type} = $3;
            $ipv6_hash{vserver_id} = $4;
            push @{$hash{ipv6}}, \%ipv6_hash;
        } elsif ($_ =~ /media:\s+(.+)/) {
            $hash{media} = $1;
        } elsif ($_ =~ /status:\s+(.*)/) {
            $hash{status} = $1;
        }
    }
    if ($cmd =~ /-C|-l/) {
        my @array;
        my @values = split (/\s+/, $lines[0]);
        $array[0]->{interfaces} = \@values;
        return \@array;
    }
    # now push the last section and remove the first section (which is empty)
    push (@arr, \%hash);
    shift @arr;
    
    # don't forget to return the parsed output
    return \@arr if @arr;
    return undef;
}

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /interface.*does\s+not\s+exist|bad\s+address/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /options+requires\s+an\s+argument/i ) {
        $error_message = $raw_output;
        $error         = "MissingValue";
    } elsif ( $raw_output =~ /usage:.*|invalid\s+option|Invalid\s+argument|illegal\s+option/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }
}

1;
