#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode iic commands
##
## @description
##     This is a library to parse the raw output of 7Mode iic commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use iic;
##
##     iic(%args);
##
## @author sheethal@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::iic;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args            = @_;
    my $raw_output      = $args{raw_output};
    my $command         = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ($raw_output =~ /([Uu]sage: iic.*)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    } elsif ($raw_output =~ /Permission denied/) {
        $error_message = $1;
        $error         = "CommandFailed";
    } elsif ($raw_output =~ /No such file or directory/) {
        $error_message = $1;
        $error         = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }

} ## end sub _check_error

## @name ::iic
##
## @summary Parses the raw output from the "iic" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iic with any of its options as described in the
##    corresponding cdef file 'iic.cdefs'
## @synopsis
##      iic(%args);
## INPUTS TO METHOD CALL 'iic' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'retries' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bus' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'data' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'read_count' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'auto_verify' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'write_count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'block' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'write' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'offset' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = iic -b 3 -a 0xe2 -o 0 -r 4
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;0000: ff ff ff ff
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'value' => 'ff ff ff ff',
##&nbsp;            'data' => '0000: ff ff ff ff'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iic {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );

    # Now we can parse the output
    my @array
        = get_lines(raw_output => $raw_output, response_object => $resp);
    my @output;
    my $i = 0;
    my @heads;
    my @values;
    if (@array) {
        foreach my $input (@array) {
            if ($input =~ /:/) {
                @heads = split(/\s*:\s*/, $input);
                $output[$i]->{data}  = $input;
                $output[$i]->{value} = $heads[1];
            }
            $i++;
        }
    }
    return \@output;
} ## end sub iic
1;
