#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd iostat command"
##
## @description
##    This is a library to parse the raw output of Freebsd iostat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "iostat" will be iostat()
##
## @synoiostatis
##    use iostat;
##
##    iostat(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::iostat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingValueException;

## @name ::iostat
##
## @summary Parses the raw output from the "iostat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iostat with any of its options as described in the
##    corresponding cdef file 'iostat.cdefs'
## @synopsis
##      iostat(%args);
## INPUTS TO METHOD CALL 'iostat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-M' </td> <td> 'core' </td> <td> '0' </td> </tr>
## <tr> <td> '-K' </td> <td> 'kilobytes' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'drives' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'systems' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'timestamp' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'total' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'cpu-usage' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'nfs-report' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'xml-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'devs' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'device-usage' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'extended' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iostat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       tty             md0              ad0              ad1             cpu
##&nbsp; tin  tout  KB/t tps  MB/s   KB/t tps  MB/s   KB/t tps  MB/s  us ni sy in id
##&nbsp;   0     6 14.45   0  0.00  23.23   0  0.00  12.16   1  0.01   0  0  2  0 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ad0_kbt' => '23.23',
##&nbsp;            'md0_kbt' => '14.45',
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'ad0_mbs' => '0.00',
##&nbsp;            'md0_tps' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'ad1_tps' => '1',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'tty_tout' => '6',
##&nbsp;            'cpu_in' => '0',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'ad0_tps' => '0',
##&nbsp;            'md0_mbs' => '0.00',
##&nbsp;            'ad1_kbt' => '12.16',
##&nbsp;            'ad1_mbs' => '0.01'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iostat  -x
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                        extended device statistics
##&nbsp;device     r/s   w/s    kr/s    kw/s wait svc_t  %b
##&nbsp;md0        0.0   0.0     0.0     0.0    0   0.1   0
##&nbsp;ad0        0.0   0.0     0.1     0.2    0   1.8   0
##&nbsp;ad1        0.0   0.8     0.0     9.8    0   1.8   0
##&nbsp;ad2        0.0   0.0     0.2     0.0    0   0.6   0
##&nbsp;md1        0.0   0.0     0.1     0.0    0   1.5   0
##&nbsp;md2        0.0   0.0     0.0     0.0    0   1.7   0
##&nbsp;md3        0.0   0.2     0.0     1.3    0   0.0   0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'svc_t' => '0.1',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '0.0',
##&nbsp;            'w_s' => '0.0',
##&nbsp;            'device' => 'md0',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '1.8',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '0.2',
##&nbsp;            'w_s' => '0.0',
##&nbsp;            'device' => 'ad0',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '1.8',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '9.8',
##&nbsp;            'w_s' => '0.8',
##&nbsp;            'device' => 'ad1',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '0.6',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '0.0',
##&nbsp;            'w_s' => '0.0',
##&nbsp;            'device' => 'ad2',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '1.5',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '0.0',
##&nbsp;            'w_s' => '0.0',
##&nbsp;            'device' => 'md1',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '1.7',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '0.0',
##&nbsp;            'w_s' => '0.0',
##&nbsp;            'device' => 'md2',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'svc_t' => '0.0',
##&nbsp;            'r_s' => '0.0',
##&nbsp;            'kw_s' => '1.3',
##&nbsp;            'w_s' => '0.2',
##&nbsp;            'device' => 'md3',
##&nbsp;            'wait' => '0',
##&nbsp;            'b' => '0',
##&nbsp;            'kr_s' => '0.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iostat  -n md0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       tty            cpu
##&nbsp; tin  tout us ni sy in id
##&nbsp;   0     6  0  0  2  0 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'tty_tout' => '6',
##&nbsp;            'cpu_in' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iostat  -t pass
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       tty            cpu
##&nbsp; tin  tout us ni sy in id
##&nbsp;   0     6  0  0  2  0 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'tty_tout' => '6',
##&nbsp;            'cpu_in' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iostat  -c 2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       tty             md0              ad0              ad1             cpu
##&nbsp; tin  tout  KB/t tps  MB/s   KB/t tps  MB/s   KB/t tps  MB/s  us ni sy in id
##&nbsp;   0     6 14.45   0  0.00  23.23   0  0.00  12.15   1  0.01   0  0  2  0 98
##&nbsp;   0   233  0.00   0  0.00   0.00   0  0.00   0.00   0  0.00   0  0  2  0 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ad0_kbt' => '23.23',
##&nbsp;            'md0_kbt' => '14.45',
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'ad0_mbs' => '0.00',
##&nbsp;            'md0_tps' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'ad1_tps' => '1',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'tty_tout' => '6',
##&nbsp;            'cpu_in' => '0',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'ad0_tps' => '0',
##&nbsp;            'md0_mbs' => '0.00',
##&nbsp;            'ad1_kbt' => '12.15',
##&nbsp;            'ad1_mbs' => '0.01'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ad0_kbt' => '0.00',
##&nbsp;            'md0_kbt' => '0.00',
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'ad0_mbs' => '0.00',
##&nbsp;            'md0_tps' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'ad1_tps' => '0',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'tty_tout' => '233',
##&nbsp;            'cpu_in' => '0',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'ad0_tps' => '0',
##&nbsp;            'md0_mbs' => '0.00',
##&nbsp;            'ad1_kbt' => '0.00',
##&nbsp;            'ad1_mbs' => '0.00'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iostat  -o
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;       tty          md0           ad0           ad1           ad2             cpu
##&nbsp; tin  tout sps tps msps  sps tps msps  sps tps msps  sps tps msps  us ni sy in id
##&nbsp;   0     6   0   0  0.1    1   0  1.8   20   1  1.8    0   0  0.6   0  0  2  0 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ad2_msps' => '0.6',
##&nbsp;            'ad2_sps' => '0',
##&nbsp;            'cpu_sy' => '2',
##&nbsp;            'md0_msps' => '0.1',
##&nbsp;            'md0_tps' => '0',
##&nbsp;            'cpu_us' => '0',
##&nbsp;            'ad1_tps' => '1',
##&nbsp;            'tty_tout' => '6',
##&nbsp;            'ad0_sps' => '1',
##&nbsp;            'tty_tin' => '0',
##&nbsp;            'ad0_tps' => '0',
##&nbsp;            'ad2_tps' => '0',
##&nbsp;            'ad1_msps' => '1.8',
##&nbsp;            'ad0_msps' => '1.8',
##&nbsp;            'ad1_sps' => '20',
##&nbsp;            'cpu_id' => '98',
##&nbsp;            'cpu_ni' => '0',
##&nbsp;            'cpu_in' => '0',
##&nbsp;            'md0_sps' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iostat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count     = -1;
    my @main_head = split( /\s+/, $lines[0] );
    my @sub_head  = split( /\s+/, $lines[1] );

    for ( my $i = 2 ; $i <= $#lines ; $i++ ) {
        if ( $cmd =~ /-x/ ) {
            $count++;
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0 ; $j <= $#sub_head ; $j++ ) {
                $sub_head[$j] =~ s/%//;
                $sub_head[$j] =~ s/\//_/;
                $result_array[$count]
                  ->{ to_key_name( item => $sub_head[$j] ) } = $values[$j];
            }
        } else {
            my $main_count = 0;
            my $limit      = 0;
            $count++;
            my @values = split( /\s+/, $lines[$i] );
            foreach my $head (@sub_head) {
                $main_count++ if ( $head =~ /^sps|^KB|^us/ );
                $head =~ s/\///;
                $result_array[$count]->{ to_key_name( item =>"$main_head[$main_count]" . " $head" ) } = $values[$limit];
                $limit++;
            }
        }
    }
    return \@result_array;
} ## end sub iostat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Permission\s+denied/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/unknown\s+match\s+item|wait\s+time\s+is|count.*\s+is|No\s+such\s+file\s+or\s+directory/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
