#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd ipfstat command"
##
## @description
##    This is a library to parse the raw output of Freebsd ipfstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ipfstat" will be ipfstat()
##
## @synoipfstatis
##    use ipfstat;
##
##    ipfstat(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ipfstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::ipfstat
##
## @summary Parses the raw output from the "sudo ipfstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ipfstat with any of its options as described in the
##    corresponding cdef file 'ipfstat.cdefs'
## @synopsis
##      ipfstat(%args);
## INPUTS TO METHOD CALL 'ipfstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-P' </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'input' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fragment' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'resolve' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'authentication' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'dest-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'state' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'close' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'hits' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'rule-number' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'redraw' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'source-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'groups' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'state-table' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'inactive-active' </td> <td> '1' </td> </tr>
## <tr> <td> '-6' </td> <td> 'ip6' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'output' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'refresh-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'corefile' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'symbol-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'accounting' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sudo ipfstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;bad packets:		in 0	out 0
##&nbsp; IPv6 packets:		in 5116 out 7380
##&nbsp; input packets:		blocked 75617 passed 12124801 nomatch 0 counted 0 short 0
##&nbsp;output packets:		blocked 0 passed 12396696 nomatch 0 counted 0 short 0
##&nbsp; input packets logged:	blocked 0 passed 0
##&nbsp;output packets logged:	blocked 0 passed 0
##&nbsp; packets logged:	input 0 output 0
##&nbsp; log failures:		input 0 output 0
##&nbsp;fragment state(in):	kept 0	lost 0	not fragmented 0
##&nbsp;fragment state(out):	kept 0	lost 0	not fragmented 0
##&nbsp;packet state(in):	kept 807	lost 0
##&nbsp;packet state(out):	kept 369	lost 0
##&nbsp;ICMP replies:	6684	TCP RSTs sent:	68933
##&nbsp;Invalid source(in):	0
##&nbsp;Result cache hits(in):	1158107	(out):	990396
##&nbsp;IN Pullups succeeded:	7082	failed:	0
##&nbsp;OUT Pullups succeeded:	55	failed:	0
##&nbsp;Fastroute successes:	75617	failures:	0
##&nbsp;TCP cksum fails(in):	0	(out):	0
##&nbsp;IPF Ticks:	4453918
##&nbsp;Packet log flags set: (0)
##&nbsp;	none
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fragment_state_out_kept' => '0',
##&nbsp;            'packets_logged_output' => '0',
##&nbsp;            'input_packets_blocked' => '75617',
##&nbsp;            'input_packets_short' => '0',
##&nbsp;            'bad_packets_in' => '0',
##&nbsp;            'output_packets_logged_passed' => '0',
##&nbsp;            'packet_log_flags_set' => '0',
##&nbsp;            'bad_packets_out' => '0',
##&nbsp;            'fragment_state_in_lost' => '0',
##&nbsp;            'out_pullups_succeeded' => '55',
##&nbsp;            'result_cache_hits_in' => '1158107',
##&nbsp;            'icmp_replies' => '6684',
##&nbsp;            'input_packets_passed' => '12124801',
##&nbsp;            'fastroute_successes' => '75617',
##&nbsp;            'out_pullups_failed' => '0',
##&nbsp;            'result_cache_hits_out' => '990396',
##&nbsp;            'input_packets_logged_passed' => '0',
##&nbsp;            'fragment_state_out_lost' => '0',
##&nbsp;            'ipv6_packets_out' => '7380',
##&nbsp;            'tcp_cksum_fails_in' => '0',
##&nbsp;            'in_pullups_failed' => '0',
##&nbsp;            'output_packets_passed' => '12396696',
##&nbsp;            'tcp_rsts_sent' => '68933',
##&nbsp;            'output_packets_blocked' => '0',
##&nbsp;            'packets_logged_input' => '0',
##&nbsp;            'log_failures_input' => '0',
##&nbsp;            'packet_state_out_lost' => '0',
##&nbsp;            'fragment_state_out_not_fragmented' => '0',
##&nbsp;            'fragment_state_in_not_fragmented' => '0',
##&nbsp;            'packet_state_in_kept' => '807',
##&nbsp;            'input_packets_counted' => '0',
##&nbsp;            'log_failures_output' => '0',
##&nbsp;            'in_pullups_succeeded' => '7082',
##&nbsp;            'input_packets_nomatch' => '0',
##&nbsp;            'tcp_cksum_fails_out' => '0',
##&nbsp;            'fastroute_failures' => '0',
##&nbsp;            'output_packets_short' => '0',
##&nbsp;            'output_packets_counted' => '0',
##&nbsp;            'output_packets_nomatch' => '0',
##&nbsp;            'packet_state_in_lost' => '0',
##&nbsp;            'ipf_ticks' => '4453918',
##&nbsp;            'input_packets_logged_blocked' => '0',
##&nbsp;            'invalid_source_in' => '0',
##&nbsp;            'fragment_state_in_kept' => '0',
##&nbsp;            'output_packets_logged_blocked' => '0',
##&nbsp;            'ipv6_packets_in' => '5116',
##&nbsp;            'packet_state_out_kept' => '369'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = sudo ipfstat  -A
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Authorisation hits: 0	misses 0
##&nbsp;nospace 0
##&nbsp;added 0
##&nbsp;sendfail 0
##&nbsp;sendok 0
##&nbsp;queok 0
##&nbsp;quefail 0
##&nbsp;expire 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'added' => '0',
##&nbsp;            'authorisation_hits' => '0',
##&nbsp;            'queok' => '0',
##&nbsp;            'misses' => '0',
##&nbsp;            'quefail' => '0',
##&nbsp;            'nospace' => '0',
##&nbsp;            'sendfail' => '0',
##&nbsp;            'sendok' => '0',
##&nbsp;            'expire' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = sudo ipfstat  -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IP fragment states:
##&nbsp;	0 new
##&nbsp;	0 expired
##&nbsp;	0 hits
##&nbsp;	0 retrans
##&nbsp;	0 too short
##&nbsp;	0 no memory
##&nbsp;	0 already exist
##&nbsp;	0 inuse
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_fragment_states' => [
##&nbsp;                                      {
##&nbsp;                                        'already_exist' => '0',
##&nbsp;                                        'hits' => '0',
##&nbsp;                                        'inuse' => '0',
##&nbsp;                                        'expired' => '0',
##&nbsp;                                        'too_short' => '0',
##&nbsp;                                        'new' => '0',
##&nbsp;                                        'no_memory' => '0',
##&nbsp;                                        'retrans' => '0'
##&nbsp;                                      }
##&nbsp;                                    ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = sudo ipfstat  -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;opts 0x2 name /dev/ipl
##&nbsp;bad packets:		in 0	out 0
##&nbsp; IPv6 packets:		in 5116 out 7380
##&nbsp; input packets:		blocked 75617 passed 12125366 nomatch 0 counted 0 short 0
##&nbsp;output packets:		blocked 0 passed 12397295 nomatch 0 counted 0 short 0
##&nbsp; input packets logged:	blocked 0 passed 0
##&nbsp;output packets logged:	blocked 0 passed 0
##&nbsp; packets logged:	input 0 output 0
##&nbsp; log failures:		input 0 output 0
##&nbsp;fragment state(in):	kept 0	lost 0	not fragmented 0
##&nbsp;fragment state(out):	kept 0	lost 0	not fragmented 0
##&nbsp;packet state(in):	kept 807	lost 0
##&nbsp;packet state(out):	kept 369	lost 0
##&nbsp;ICMP replies:	6684	TCP RSTs sent:	68933
##&nbsp;Invalid source(in):	0
##&nbsp;Result cache hits(in):	1158130	(out):	990443
##&nbsp;IN Pullups succeeded:	7082	failed:	0
##&nbsp;OUT Pullups succeeded:	55	failed:	0
##&nbsp;Fastroute successes:	75617	failures:	0
##&nbsp;TCP cksum fails(in):	0	(out):	0
##&nbsp;IPF Ticks:	4454039
##&nbsp;Packet log flags set: (0)
##&nbsp;	none
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fragment_state_out_kept' => '0',
##&nbsp;            'packets_logged_output' => '0',
##&nbsp;            'input_packets_blocked' => '75617',
##&nbsp;            'input_packets_short' => '0',
##&nbsp;            'bad_packets_in' => '0',
##&nbsp;            'output_packets_logged_passed' => '0',
##&nbsp;            'packet_log_flags_set' => '0',
##&nbsp;            'opts' => '0x2',
##&nbsp;            'bad_packets_out' => '0',
##&nbsp;            'fragment_state_in_lost' => '0',
##&nbsp;            'out_pullups_succeeded' => '55',
##&nbsp;            'result_cache_hits_in' => '1158130',
##&nbsp;            'icmp_replies' => '6684',
##&nbsp;            'input_packets_passed' => '12125366',
##&nbsp;            'fastroute_successes' => '75617',
##&nbsp;            'name' => '/dev/ipl',
##&nbsp;            'out_pullups_failed' => '0',
##&nbsp;            'result_cache_hits_out' => '990443',
##&nbsp;            'input_packets_logged_passed' => '0',
##&nbsp;            'fragment_state_out_lost' => '0',
##&nbsp;            'ipv6_packets_out' => '7380',
##&nbsp;            'tcp_cksum_fails_in' => '0',
##&nbsp;            'in_pullups_failed' => '0',
##&nbsp;            'output_packets_passed' => '12397295',
##&nbsp;            'tcp_rsts_sent' => '68933',
##&nbsp;            'output_packets_blocked' => '0',
##&nbsp;            'packet_state_out_lost' => '0',
##&nbsp;            'packets_logged_input' => '0',
##&nbsp;            'log_failures_input' => '0',
##&nbsp;            'fragment_state_out_not_fragmented' => '0',
##&nbsp;            'fragment_state_in_not_fragmented' => '0',
##&nbsp;            'packet_state_in_kept' => '807',
##&nbsp;            'input_packets_counted' => '0',
##&nbsp;            'log_failures_output' => '0',
##&nbsp;            'in_pullups_succeeded' => '7082',
##&nbsp;            'input_packets_nomatch' => '0',
##&nbsp;            'tcp_cksum_fails_out' => '0',
##&nbsp;            'fastroute_failures' => '0',
##&nbsp;            'output_packets_short' => '0',
##&nbsp;            'output_packets_counted' => '0',
##&nbsp;            'output_packets_nomatch' => '0',
##&nbsp;            'packet_state_in_lost' => '0',
##&nbsp;            'ipf_ticks' => '4454039',
##&nbsp;            'input_packets_logged_blocked' => '0',
##&nbsp;            'invalid_source_in' => '0',
##&nbsp;            'fragment_state_in_kept' => '0',
##&nbsp;            'ipv6_packets_in' => '5116',
##&nbsp;            'output_packets_logged_blocked' => '0',
##&nbsp;            'packet_state_out_kept' => '369'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = sudo ipfstat  -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IP states added:
##&nbsp;	1134 TCP
##&nbsp;	42 UDP
##&nbsp;	0 ICMP
##&nbsp;	33045 hits
##&nbsp;	1396744 misses
##&nbsp;	0 bucket full
##&nbsp;	0 maximum rule references
##&nbsp;	0 maximum
##&nbsp;	0 no memory
##&nbsp;	0 bkts in use
##&nbsp;	0 active
##&nbsp;	42 expired
##&nbsp;	1134 closed
##&nbsp;State logging enabled
##&nbsp;
##&nbsp;State table bucket statistics:
##&nbsp;	0 in use
##&nbsp;	0% hash efficiency
##&nbsp;	0.00% bucket usage
##&nbsp;	0 minimal length
##&nbsp;	0 maximal length
##&nbsp;	0.000 average length
##&nbsp;
##&nbsp;TCP Entries per state
##&nbsp;     0     1     2     3     4     5     6     7     8     9    10    11
##&nbsp;     0     0     0     0     0     0     0     0     0     0     0     0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tcp_entries_per_state' => [
##&nbsp;                                         {
##&nbsp;                                           '6' => '0',
##&nbsp;                                           '11' => '0',
##&nbsp;                                           '3' => '0',
##&nbsp;                                           '7' => '0',
##&nbsp;                                           '9' => '0',
##&nbsp;                                           '2' => '0',
##&nbsp;                                           '8' => '0',
##&nbsp;                                           '1' => '0',
##&nbsp;                                           '4' => '0',
##&nbsp;                                           '0' => '0',
##&nbsp;                                           '10' => '0',
##&nbsp;                                           '5' => '0'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;            'state_table_bucket_statistics' => [
##&nbsp;                                                 {
##&nbsp;                                                   'in_use' => '0',
##&nbsp;                                                   'average_length' => '0.000',
##&nbsp;                                                   'hash_efficiency' => '0%',
##&nbsp;                                                   'maximal_length' => '0',
##&nbsp;                                                   'bucket_usage' => '0.00%',
##&nbsp;                                                   'minimal_length' => '0'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'ip_states_added' => [
##&nbsp;                                   {
##&nbsp;                                     'closed' => '1134',
##&nbsp;                                     'maximum' => '0',
##&nbsp;                                     'active' => '0',
##&nbsp;                                     'tcp' => '1134',
##&nbsp;                                     'misses' => '1396744',
##&nbsp;                                     'hits' => '33045',
##&nbsp;                                     'bucket_full' => '0',
##&nbsp;                                     'expired' => '42',
##&nbsp;                                     'bkts_in_use' => '0',
##&nbsp;                                     'maximum_rule_references' => '0',
##&nbsp;                                     'udp' => '42',
##&nbsp;                                     'no_memory' => '0',
##&nbsp;                                     'icmp' => '0'
##&nbsp;                                   }
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ipfstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values, @main_head );
    my ( $key1, $value1, $key2, $value2, $key3 );
    my $count = -1;
    my $main_head;

    if ( $cmd =~ /-s|-f/ ) {
        $count++;
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
            if ( $lines[$i] =~ /(.*)\:/ || $lines[$i] =~ /(TCP\s+Entries.*)/ ) {
                $main_head = to_key_name( item => $1 );
            } elsif ( $lines[$i] =~ /(\d+\s+\d+\s+\d+)/ ) {
                @head   = split( /\s+/, $lines[$i] );
                @values = split( /\s+/, $lines[ $i + 1 ] );
                for ( my $j = 0 ; $j <= $#values ; $j++ ) {
                    $result_array[$count]->{$main_head}[$count]
                      { to_key_name( item => $head[$j] ) } = $values[$j];
                }
            } elsif ( $lines[$i] =~ /(\d+\S*)\s+(.*)/ ) {
                $result_array[$count]
                  ->{$main_head}[$count]{ to_key_name( item => $2 ) } = $1;
            }
        }
    } elsif ( $cmd =~ /-A/ ) {
        $count++;
        foreach my $line (@lines) {
            if ( $line =~ /(\w+\s+\w+)\:\s+(\d+)\s+(\w+)\s+(\d+)/ ) {
                $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
                $result_array[$count]->{ to_key_name( item => $3 ) } = $4;
            } elsif ( $line =~ /(\w+)\s+(\d+)/ ) {
                $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } else {
        $count++;
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
            if ( $lines[$i] =~ /(.*)\((\w+)\)\:\s+(\d+)\s+\((.*)\)\:\s+(\d+)/ ) {
                ( $key1, $key2, $value1, $key3, $value2 ) = ( $1, $2, $3, $4, $5 );
                $result_array[$count]
                  ->{ to_key_name( item => "$key1" . " $key2" ) } = $value1;
                $result_array[$count]
                  ->{ to_key_name( item => "$key1" . " $key3" ) } = $value2;
            } elsif ( $lines[$i] =~ /(ICMP.*)\:\s+(\d+)\s+(.*)\:\s+(\d+)/ ) {
                $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
                $result_array[$count]->{ to_key_name( item => $3 ) } = $4;
            } elsif ( $lines[$i] =~ /(.*)\s+(\w+)\:\s+(\d+)\s+(.*)\:\s+(\d+)/ ) {
                ( $key1, $key2, $value1, $key3, $value2 ) = ( $1, $2, $3, $4, $5 );
                $result_array[$count]
                  ->{ to_key_name( item => "$key1" . " $key2" ) } = $value1;
                $result_array[$count]
                  ->{ to_key_name( item => "$key1" . " $key3" ) } = $value2;
            } elsif($lines[$i] =~ /(opts)\s+(.*)\s+(name)\s+(.*)/) {
                 $result_array[$count]->{ $1 } = $2;
                 $result_array[$count]->{ $3 } = $4;
            } elsif ( $lines[$i] =~ /(Invalid.*|IPF.*|Packet.*)\:\s+\(*(\d+)\)*/ ) {
                my $key   = $1;
                my $value = $2;
                $key =~ s/\(/_/;
                $key =~ s/\)//;
                $result_array[$count]->{ to_key_name( item => $key ) } = $value;

            } elsif ( $lines[$i] =~ /.*\S+\:.*/ ) {
                $lines[$i] =~ s/not fragmented/not_fragmented/;
                @head   = split( /:\s+/, $lines[$i] );
                @values = split( /\s+/,  $head[1] );
                for ( my $j = 0 ; $j <= $#values ; $j = $j + 2 ) {
                    $head[$j] =~ s/\(/_/;
                    $head[$j] =~ s/\)//;
                   $result_array[$count]->{to_key_name( item =>"$head[0]". " $values[$j]") } = $values[$j+1];
                }
            }
        }
    }

    return \@result_array;
} ## end sub ipfstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;

    if ( $raw_output =~ /No.*given\s+with.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/No\s+such\s+file\s+or\s+directory|Invalid\s+IP\s+address.*|Invalid\s+refreshtime\s+.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
