#
# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd is_fusen_mode command"
##
## @description
##    This is a library to parse the raw output of is_fusen_mode command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "is_fusen_mode" will be is_fusen_mode()
##
## @synoypwhichis
##    use is_fusen_mode;
##
##    is_fusen_mode(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::is_fusen_mode;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name :: is_fusen_mode
##
## @summary Parses the raw output from the "is_fusen_mode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from is_fusen_mode
##    cdef file is 'is_fusen_mode.cdefs'
## @synopsis
##      is_fusen_mode(%args);
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = systemshell local is_fusen_mode
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;FuseN mode is defaulting to disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fusen_mode' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub is_fusen_mode {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;

    foreach my $line (@lines) {
        if ( $line =~ /(.*)(disabled|enabled)/i ) {
            $result_array[0]->{to_key_name(item =>'FuseN mode')} = $2;
        }
    }
    return \@result_array;
} ## end sub is_fusen_mode

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();

    # error_level is 0/1 for diabled/enabled, so it should not throw exception for this
    if (! ($error_level == 1 || $error_level == 0)) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

