#
# Copyright (c) 2012-2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse kill command output
##
## @description
##    This library supports error checking for kill command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::kill;
##
##  kill(%args);
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::kill;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::kill
##
## @summary Parses the raw output from the "kill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from kill with any of its options as described in the
##    corresponding cdef file 'kill.cdefs'
## @synopsis
##      kill(%args);
## INPUTS TO METHOD CALL 'kill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'signal' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'commandname' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'signal-number' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'do-not-restrict' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'print-pid-only' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'signal-list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported

sub kill {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub kill

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;

    if ( $raw_output =~ /Too few arguments.*/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /No such process.*|Arguments should be jobs.*|.*invalid signal.*|Unknown signal.*|Bad signal.*/i
        ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /kill:\s+.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
