#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd killall command"
##
## @description
##    This is a library to parse the raw output of Freebsd killall command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "killall" will be killall()
##
## @synopsis
##    use killall;
##
##    killall(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::killall;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingValueException;

## @name ::killall
##
## @summary Parses the raw output from the "killall" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from killall with any of its options as described in the 
##    corresponding cdef file 'killall.cdefs'
## @synopsis
##      killall(%args);
## INPUTS TO METHOD CALL 'killall' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'show' </td> <td> '0' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'cmd' </td> <td> '0' </td> </tr> 
## <tr> <td> '-z' </td> <td> 'zombies' </td> <td> '0' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'match' </td> <td> '0' </td> </tr> 
## <tr> <td> '-e' </td> <td> 'user-id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'tty' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'message' </td> <td> '0' </td> </tr> 
## <tr> <td> '-SIGNAL' </td> <td> 'signal' </td> <td> '0' </td> </tr> 
## <tr> <td> '-j' </td> <td> 'jail' </td> <td> '0' </td> </tr> 
## <tr> <td> '-u' </td> <td> 'user' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub killall {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub killall

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Operation\s+not\s+permitted/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /must\s+specify.*|user.*does\s+not\s+exist/ ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/No\s+matching\s+processes.*|jail.*not\s+found|No\s+such\s+file\s+or\s+directory/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*|unknown\s+signal/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
