#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse ln  command output
##
## @description
##   This library supports error checking for ln command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::ln;
##
##    ln(%args);
##
## @author tanil@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ln;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~ /No such file or directory|File exists/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /usage:/) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::ln
##
## @summary Parses the raw output from the "ln" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ln with any of its options as described in the
##    corresponding cdef file 'ln.cdefs'
## @synopsis
##      ln(%args);
## INPUTS TO METHOD CALL 'ln' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'interact' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'symlink' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'files' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'target-directory' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'link-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-hnv' </td> <td> 'noderef-verbose' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##Returns: Output parsing is not supported
## @examples
##&nbsp;Command = ln -s api-set-dir/file SYMLINK
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ln {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'ln',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub ln

## @name ::link
##
## @summary Parses the raw output from the "link" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from link with any of its options as described in the
##    corresponding cdef file 'ln.cdefs'
## @synopsis
##      link(%args);
## INPUTS TO METHOD CALL 'link' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'interact' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'symlink' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'files' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'target-directory' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'link-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-hnv' </td> <td> 'noderef-verbose' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = link api-set-dir/file SYMLINK
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub link {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'link',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}    #end of link
1;

