#
# Copyright (c) 2012-13 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse ls command output
##
## @description
##   This library supports error checking for ls command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::ls;
##
##  ls(%args);
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ls;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::ls
##
## @summary Parses the raw output from the "ls" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ls with any of its options as described in the
##    corresponding cdef file 'ls.cdefs'
## @synopsis
##      ls(%args);
## INPUTS TO METHOD CALL 'ls' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'almost-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'raw-printing' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'sort-by-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'no-group-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'use-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'last-changed' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'dereference' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'mac-label' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'comma-separated' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'whiteouts' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'classify' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size-in-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'escape' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode-num' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'hide-control-chars' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'usetime-of-last-access' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'follow-symbolic-links' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'list-link' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-uid-gid' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'recursively' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'print-filename' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'unsorted-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'ignore-backups' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'block-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directories' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'prevent-super-user' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'directory-entries-with-dot' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reverse-order' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'display-complete-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-1' </td> <td> 'one-per-line' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-readable' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'sorted-multi-column-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'multi-column-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'sort-by-modification-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-G' </td> <td> 'colorized-output' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##Returns: Output parsing is not supported
sub ls {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my @result_array;
    my %hash = (files => []);
    my @lines = get_lines(raw_output=>$output, first_line=>'');

    foreach my $line (@lines) {
        chomp($line);
        my @files = split(/\s+/, $line);
        push @{$hash{files}}, @files;
    }

    push( @result_array, \%hash );

    return \@result_array;
} ## end sub ls

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;

    if ( $raw_output =~ /option requires an argument.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No such file or directory.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|ls:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
