# $Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/Freebsd/default/mbustest.pm#1 $
#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd mbustest command
##
## @description
##    This is a library to parse the raw output of Freebsd mbustest command.
##    This command is available only in debug builds for use of testing the mrt
##    task for Messaging Scheme BUS.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::mbustest;
##
##    mbustest(%args);
##
## @author clausner@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::mbustest;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::mbustest
##
## @summary Parses the raw output from the "mbustest" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mbustest with any of its options as described in the 
##    corresponding cdef file 'mbustest.cdefs'
## @synopsis
##      mbustest(%args);
## INPUTS TO METHOD CALL 'mbustest' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '--profile' </td> <td> 'profile' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'appid' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'time' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'delay' </td> </tr> 
## <tr> <td> '-e' </td> <td> 'exit' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'node' </td> </tr> 
## <tr> <td> '--tput' </td> <td> 'tput' </td> </tr> 
## <tr> <td> '--routes' </td> <td> 'routes' </td> </tr> 
## <tr> <td> '--msgid' </td> <td> 'msgid' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'controlid' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'log' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'count' </td> </tr> 
## <tr> <td> '--rpc' </td> <td> 'rpc' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'message' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'register' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'procid' </td> </tr> 
## <tr> <td> '--class' </td> <td> 'class' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'sleep' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mbustest -m broadcast -i 5 -t 10 -l verbose
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = {
##&nbsp;          'send1' => {
##&nbsp;                       'total_send' => '5',
##&nbsp;                       'appid' => 'ANY',
##&nbsp;                       'procid' => 'ANY',
##&nbsp;                       'send_num' => '2',
##&nbsp;                       'type_num' => '5',
##&nbsp;                       'type' => 'APP_MBUS_TEST',
##&nbsp;                       'host' => 'ANY'
##&nbsp;                     },
##&nbsp;          'message9' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x5',
##&nbsp;                          'time' => '13:54:03',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0x2',
##&nbsp;                          'from_uuid' => '79f68647-e2e3-11e0-beef-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0x1b'
##&nbsp;                        },
##&nbsp;          'mrt_max' => 2,
##&nbsp;          'message6' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x5',
##&nbsp;                          'time' => '13:54:03',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0x2',
##&nbsp;                          'from_uuid' => '79f68647-e2e3-11e0-beef-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0x1b'
##&nbsp;                        },
##&nbsp;          'message4' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000002',
##&nbsp;                          'time' => '13:54:02',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'message8' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x5',
##&nbsp;                          'time' => '13:54:03',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0x2',
##&nbsp;                          'from_uuid' => '79f68647-e2e3-11e0-beef-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0x1b'
##&nbsp;                        },
##&nbsp;          'send4' => {
##&nbsp;                       'total_send' => '5',
##&nbsp;                       'appid' => 'ANY',
##&nbsp;                       'procid' => 'ANY',
##&nbsp;                       'send_num' => '5',
##&nbsp;                       'type_num' => '5',
##&nbsp;                       'type' => 'APP_MBUS_TEST',
##&nbsp;                       'host' => 'ANY'
##&nbsp;                     },
##&nbsp;          'message1' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000002',
##&nbsp;                          'time' => '13:54:01',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'send2' => {
##&nbsp;                       'total_send' => '5',
##&nbsp;                       'appid' => 'ANY',
##&nbsp;                       'procid' => 'ANY',
##&nbsp;                       'send_num' => '3',
##&nbsp;                       'type_num' => '5',
##&nbsp;                       'type' => 'APP_MBUS_TEST',
##&nbsp;                       'host' => 'ANY'
##&nbsp;                     },
##&nbsp;          'message3' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000002',
##&nbsp;                          'time' => '13:54:01',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'message7' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x5',
##&nbsp;                          'time' => '13:54:03',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0x2',
##&nbsp;                          'from_uuid' => '79f68647-e2e3-11e0-beef-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0x1b'
##&nbsp;                        },
##&nbsp;          'send3' => {
##&nbsp;                       'total_send' => '5',
##&nbsp;                       'appid' => 'ANY',
##&nbsp;                       'procid' => 'ANY',
##&nbsp;                       'send_num' => '4',
##&nbsp;                       'type_num' => '5',
##&nbsp;                       'type' => 'APP_MBUS_TEST',
##&nbsp;                       'host' => 'ANY'
##&nbsp;                     },
##&nbsp;          'received_max' => 11,
##&nbsp;          'message2' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000002',
##&nbsp;                          'time' => '13:54:01',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'send_max' => 5,
##&nbsp;          'message5' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000002',
##&nbsp;                          'time' => '13:54:02',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'route' => {
##&nbsp;                       'csiqa-3070-47' => {
##&nbsp;                                            '0x1a' => {
##&nbsp;                                                        'procid' => '0x2',
##&nbsp;                                                        'status' => [
##&nbsp;                                                                      'Add',
##&nbsp;                                                                      'Delete'
##&nbsp;                                                                    ],
##&nbsp;                                                        'status_count' => 2
##&nbsp;                                                      },
##&nbsp;                                            '0x1b' => {
##&nbsp;                                                        'procid' => '0x2',
##&nbsp;                                                        'status' => [
##&nbsp;                                                                      'Add'
##&nbsp;                                                                    ],
##&nbsp;                                                        'status_count' => 1
##&nbsp;                                                      },
##&nbsp;                                            '0xffffffff' => {
##&nbsp;                                                              'procid' => '0x2',
##&nbsp;                                                              'status' => [
##&nbsp;                                                                            'Delete'
##&nbsp;                                                                          ],
##&nbsp;                                                              'status_count' => 1
##&nbsp;                                                            }
##&nbsp;                                          }
##&nbsp;                     },
##&nbsp;          'mrt1' => {
##&nbsp;                      'ip1' => '10.227.129.146',
##&nbsp;                      'mrt_name' => 'csiqa-3070-48',
##&nbsp;                      'ip0' => '10.227.128.146',
##&nbsp;                      'mrt_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412'
##&nbsp;                    },
##&nbsp;          'send0' => {
##&nbsp;                       'total_send' => '5',
##&nbsp;                       'appid' => 'ANY',
##&nbsp;                       'procid' => 'ANY',
##&nbsp;                       'send_num' => '1',
##&nbsp;                       'type_num' => '5',
##&nbsp;                       'type' => 'APP_MBUS_TEST',
##&nbsp;                       'host' => 'ANY'
##&nbsp;                     },
##&nbsp;          'mrt0' => {
##&nbsp;                      'ip1' => '10.227.129.145',
##&nbsp;                      'mrt_name' => 'csiqa-3070-47',
##&nbsp;                      'ip0' => '10.227.128.145',
##&nbsp;                      'mrt_uuid' => '79f68647-e2e3-11e0-beef-123478563412'
##&nbsp;                    },
##&nbsp;          'message0' => {
##&nbsp;                          'to_misc' => '0x0',
##&nbsp;                          'message_type' => '0x80000005',
##&nbsp;                          'time' => '13:54:01',
##&nbsp;                          'to_proc_id' => '0x2',
##&nbsp;                          'from_misc' => '0x0',
##&nbsp;                          'from_proc_id' => '0xffffffff',
##&nbsp;                          'from_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'to_app_id' => '0xe',
##&nbsp;                          'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                          'from_app_id' => '0xffffffff'
##&nbsp;                        },
##&nbsp;          'message10' => {
##&nbsp;                           'to_misc' => '0x0',
##&nbsp;                           'message_type' => '0x5',
##&nbsp;                           'time' => '13:54:03',
##&nbsp;                           'to_proc_id' => '0x2',
##&nbsp;                           'from_misc' => '0x0',
##&nbsp;                           'from_proc_id' => '0x2',
##&nbsp;                           'from_uuid' => '79f68647-e2e3-11e0-beef-123478563412',
##&nbsp;                           'to_app_id' => '0xe',
##&nbsp;                           'to_uuid' => '752b6292-e2e3-11e0-a8b3-123478563412',
##&nbsp;                           'from_app_id' => '0x1b'
##&nbsp;                         }
##&nbsp;        };
##&nbsp;
sub mbustest {
    my %args = @_;

    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error_level = $response->get_execution_status();
    my $result;

    # To check if there is any error
    if($error_level != 1) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed: $raw_output",
            response_object => $response);
    }
    my @lines = get_lines('raw_output'=>$raw_output,
        'response_object'=>$response);

    my %result_hash = ();

    # $received_count will increment when a message is received
    my $received_count = 0;

    # $mrt_count will increment when another mrt connection is seen
    my $mrt_count      = 0;

    # $send_count will increment when an outgoing message is seen.
    my $send_count     = 0;

    my $msg_type = undef;
    my $time;
    my $current_rcv_msg = 0;

    foreach my $line (@lines) {

        my $msg_str = "message$current_rcv_msg";

        #Grab the latest time no matter when (some are on separate lines)
        if( $line =~ /(\d+:\d+:\d+) /) {
            $time = $1;
        }

        # Example line:
        # 10:47:33 MBUS: mbus_session.c:807 Queue Insert 0x5:
        if( $line =~ /Queue Insert (.*):/ ) {
            $msg_type = $1;
            $current_rcv_msg = $received_count;
            $received_count++;

        # This follows the Queue Insert Line looks like:
        # From: 0055f105-bd16-11e0-867b-123478563412:0x8:0x2:0x0
        } elsif ( $line =~ /From: (.*):(.*):(.*):(.*)/) {
            if(!defined($msg_type)) {
                NACL::APISet::Exceptions::CommandFailedException->throw(
                    text=>"Command failed no type returned",
                    response_object => $response);
            }

            $result_hash{$msg_str}->{'message_type'} = $msg_type;
            $result_hash{$msg_str}->{'from_uuid'} = $1;
            $result_hash{$msg_str}->{'from_app_id'} = $2;
            $result_hash{$msg_str}->{'from_proc_id'} = $3;
            $result_hash{$msg_str}->{'from_misc'} = $4;
            $result_hash{$msg_str}->{'time'} = $time;

        # This follows the From and is paired up with the queue insert
        # To: 02015441-bd16-11e0-8564-123478563412:0x5:0x2:0x0
        } elsif ( $line =~ /To: (.*):(.*):(.*):(.*)/) {

            if(!defined($msg_type)) {
                NACL::APISet::Exceptions::CommandFailedException->throw(
                    text=>"Command failed no type returned",
                    response_object => $response);
            }

            $result_hash{$msg_str}->{'message_type'} = $msg_type;
            $result_hash{$msg_str}->{'to_uuid'} = $1;
            $result_hash{$msg_str}->{'to_app_id'} = $2;
            $result_hash{$msg_str}->{'to_proc_id'} = $3;
            $result_hash{$msg_str}->{'to_misc'} = $4;
            $result_hash{$msg_str}->{'time'} = $time;

            $msg_type = undef;

        # The line that shows which MRT the mbustest connects to:
        # 08:29:04 MBUS: mbus_session.c:1776 15e12f66-bd4a-11e0-9d00-123478563412 -> clausner-vsim1
        } elsif ( $line =~ /MBUS: .* (.*) -> (.*)/ ) {
            $result_hash{'mrt'.$mrt_count}->{'mrt_uuid'} = $1;
            $result_hash{'mrt'.$mrt_count}->{'mrt_name'} = $2;
            $mrt_count++;

        # The following line comes after the mrt connection and shows the ips
        # 08:29:04 MBUS: mbus_session.c:1778     ip[0]: 172.31.12.237
        } elsif ( $line =~ /ip\[(\d+)\]: (.*)/ ) {
            $result_hash{'mrt'.($mrt_count - 1)}->{'ip'.$1} = $2;

        # The following line (available normal logging) shows that it sends
        # Tx: APP_MBUS_TEST(5) to(host=ANY, appid=ANY, procid=ANY) 183 of 0
        } elsif ( $line =~ /Tx: (.*)\((\d+)\) to\(host=(.*), appid=(.*), procid=(.*)\) (\d+) of (\d+)/) {
            my $sender_str = 'send'.$send_count;
            $result_hash{$sender_str}->{'type'}       = $1;
            $result_hash{$sender_str}->{'type_num'}   = $2;
            $result_hash{$sender_str}->{'host'}       = $3;
            $result_hash{$sender_str}->{'appid'}      = $4;
            $result_hash{$sender_str}->{'procid'}     = $5;
            $result_hash{$sender_str}->{'send_num'}   = $6;
            $result_hash{$sender_str}->{'total_send'} = $7;
            $send_count++;

        # The routing information, which shows who all the client can send to
        # Route event [Add] 15e12f66-bd4a-11e0-9d00-123478563412 ( clausner-vsim1 ) procid=0x2 appid=0x2
        } elsif ( $line =~ /Route event \[(.*)\] (.*) \( (.*) \) procid=(.*) appid=(.*)/ ) {
            $result_hash{'route'}->{$3}->{$5}->{'procid'} = $4;
            if($result_hash{'route'}->{$3}->{$5}->{'status_count'} eq '' ) {
                $result_hash{'route'}->{$3}->{$5}->{'status_count'} = 0;
            }
            $result_hash{'route'}->{$3}->{$5}->{'status'}[$result_hash{'route'}->{$3}->{$5}->{'status_count'}] = $1;
            $result_hash{'route'}->{$3}->{$5}->{'status_count'}++;
        }
    }

    # Simple variables to show how many total each item has
    $result_hash{'received_max'}   = $received_count;
    $result_hash{'mrt_max'}        = $mrt_count;
    $result_hash{'send_max'}       = $send_count;
    my @result_array;
    push @result_array, \%result_hash;
    #return (\%result_hash);
    return (\@result_array);
}

1;
