#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Freebsd netstat command output
##
## @description
##    This is a library to parse the raw output of Freebsd netstat command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::netstat;
##
##    netstat(%args);
##
## @author Abhishek Kumar (kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::netstat;

use NACL::APISet::Response::CLI::ParserUtils;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;


sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Permission\s+denied|Operation\s+not\s+permitted/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/No\s+such\s+file\s+or\s+directory|unknown\s+address\s+family|unknown.*protocol/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::netstat
##
## @summary Parses the raw output from the "netstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netstat with any of its options as described in the
##    corresponding cdef file 'netstat.cdefs'
## @synopsis
##      netstat(%args);
## INPUTS TO METHOD CALL 'netstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'routing-tables' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'address-family' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-addr' </td> <td> '1' </td> </tr>
## <tr> <td> '-G' </td> <td> 'routing-group' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'core' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'listen-queues' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-sockets' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'executable' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'address' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group-address' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timers' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'counters' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'memory' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'pathi-mtu' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'show-network' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dropped-packets' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'all-interfaces' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netstat -r -n -G -f inet6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Routing tables
##&nbsp;
##&nbsp; Internet6:
##&nbsp; Destination                       Gateway                       Flags                                            Netif Expire
##&nbsp; ::1                               ::1                           UHL                                               lo0
##&nbsp; ::1001                            12:34:78:56:34:12             UHL                                               lo0
##&nbsp; ::2001                            ::1001                        UGHS                                             lofb
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'group_name' => 'default',
##&nbsp;             'addr-family-inet6' => [
##&nbsp;                                      {
##&nbsp;                                        'gateway' => '::1',
##&nbsp;                                        'destination' => '::1',
##&nbsp;                                        'flag' => 'UHL',
##&nbsp;                                        'netif' => 'lo0'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'gateway' => '12:34:78:56:34:12',
##&nbsp;                                        'destination' => '::1001',
##&nbsp;                                        'flag' => 'UHL',
##&nbsp;                                        'netif' => 'lo0'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'gateway' => '::1001',
##&nbsp;                                        'destination' => '::2001',
##&nbsp;                                        'flag' => 'UGHS',
##&nbsp;                                        'netif' => 'lofb'
##&nbsp;                                     }
##&nbsp;                                    ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = netstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Tcpcb    Proto Recv-Q Send-Q  Local Address      Foreign Address    VCTX Role         (state)
##&nbsp;       0 tcp4       0      0 subhasrv-vsi.http  cyclnb24.eng.btc.3  2    DATA         SYN_RCVD
##&nbsp;       0 tcp4       0      0 subhasrv-vsi.http  cyclnb24.eng.btc.3  2    DATA         SYN_RCVD
##&nbsp;ffffff000fea8a50 tcp4       0      0 localhost.telnet   localhost.52034     0    LOOPBACK     ESTABLISHED
##&nbsp;ffffff000ba90a50 tcp4       0      0 localhost.52034    localhost.telnet    0    ANY          ESTABLISHED
##&nbsp;Active UNIX domain sockets
##&nbsp;Address  Type   Recv-Q Send-Q    Inode     Conn     Refs  Nextref Addr
##&nbsp;ffffff002abc7d20 stream      0      0        0 ffffff000ad40b40        0        0
##&nbsp;ffffff000ad40b40 stream      0      0        0 ffffff002abc7d20        0        0
##&nbsp;ffffff0016dac000 stream      0      0        0 ffffff000da71c30        0        0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_internet_connections' => [
##&nbsp;                                               {
##&nbsp;                                                 'proto' => 'tcp4',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'tcpcb' => '0',
##&nbsp;                                                 'state' => 'SYN_RCVD',
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'foreign_address' => 'cyclnb24.eng.btc.3',
##&nbsp;                                                 'local_address' => 'subhasrv_vsi.http',
##&nbsp;                                                 'vctx' => '2',
##&nbsp;                                                 'role' => 'DATA'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'proto' => 'tcp4',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'tcpcb' => '0',
##&nbsp;                                                 'state' => 'SYN_RCVD',
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'foreign_address' => 'cyclnb24.eng.btc.3',
##&nbsp;                                                 'local_address' => 'subhasrv_vsi.http',
##&nbsp;                                                 'vctx' => '2',
##&nbsp;                                                 'role' => 'DATA'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'proto' => 'tcp4',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'tcpcb' => 'ffffff000fea8a50',
##&nbsp;                                                 'state' => 'ESTABLISHED',
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.52034',
##&nbsp;                                                 'local_address' => 'localhost.telnet',
##&nbsp;                                                 'vctx' => '0',
##&nbsp;                                                 'role' => 'LOOPBACK'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'proto' => 'tcp4',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'tcpcb' => 'ffffff000ba90a50',
##&nbsp;                                                 'state' => 'ESTABLISHED',
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.telnet',
##&nbsp;                                                 'local_address' => 'localhost.52034',
##&nbsp;                                                 'vctx' => '0',
##&nbsp;                                                 'role' => 'ANY'
##&nbsp;                                               },
##&nbsp;                                             ]
##&nbsp;            'active_unix_domain_sockets' => [
##&nbsp;                                              {
##&nbsp;                                                'nextref_address' => '0',
##&nbsp;                                                'refs' => '0',
##&nbsp;                                                'conn' => 'ffffff000ad40b40',
##&nbsp;                                                'recv_q' => '0',
##&nbsp;                                                'type' => 'stream',
##&nbsp;                                                'address' => 'ffffff002abc7d20',
##&nbsp;                                                'inode' => '0',
##&nbsp;                                                'send_q' => '0'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'nextref_address' => '0',
##&nbsp;                                                'refs' => '0',
##&nbsp;                                                'conn' => 'ffffff002abc7d20',
##&nbsp;                                                'recv_q' => '0',
##&nbsp;                                                'type' => 'stream',
##&nbsp;                                                'address' => 'ffffff000ad40b40',
##&nbsp;                                                'inode' => '0',
##&nbsp;                                                'send_q' => '0'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'nextref_address' => '0',
##&nbsp;                                                'refs' => '0',
##&nbsp;                                                'conn' => 'ffffff000da71c30',
##&nbsp;                                                'recv_q' => '0',
##&nbsp;                                                'type' => 'stream',
##&nbsp;                                                'address' => 'ffffff0016dac000',
##&nbsp;                                                'inode' => '0',
##&nbsp;                                                'send_q' => '0'
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;       }
##&nbsp;	];
##&nbsp;Command = netstat -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;138/1182/1320 mbufs in use (current/cache/total)
##&nbsp;1/681/682/16640 mbuf clusters in use (current/cache/total/max)
##&nbsp;0/299 mbuf+clusters out of packet secondary zone in use (current/cache)
##&nbsp;128/291/419/8320 4k (page size) jumbo clusters in use (current/cache/total/max)
##&nbsp;0/0/0/4160 9k jumbo clusters in use (current/cache/total/max)
##&nbsp;0/0/0/2080 16k jumbo clusters in use (current/cache/total/max)
##&nbsp;548K/2821K/3370K bytes allocated to network (current/cache/total)
##&nbsp;0/0/0 requests for mbufs denied (mbufs/clusters/mbuf+clusters)
##&nbsp;0/0/0 requests for jumbo clusters denied (4k/9k/16k)
##&nbsp;0/0/0 sfbufs in use (current/peak/max)
##&nbsp;0 requests for sfbufs denied
##&nbsp;0 requests for sfbufs delayed
##&nbsp;0 requests for I/O initiated by sendfile
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'calls_to_protocol_drain_routines' => '0',
##&nbsp;            'requests_for_sfbufs_delayed' => '0',
##&nbsp;            'requests_for_sfbufs_denied' => '0',
##&nbsp;            'requests_for_mbufs_denied' => [
##&nbsp;                                             {
##&nbsp;                                               'mbuf_clusters' => '0',
##&nbsp;                                               'clusters' => '0',
##&nbsp;                                               'mbufs' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'mbuf_clusters_out_of_packet_secondary_zone_in_use' => [
##&nbsp;                                                                     {
##&nbsp;                                                                       'current' => '0',
##&nbsp;                                                                       'cache' => '299'
##&nbsp;                                                                     }
##&nbsp;                                                                   ],
##&nbsp;            'requests_for_io_initiated_by_sendfile' => '0',
##&nbsp;            'mbufs_in_use' => [
##&nbsp;                                {
##&nbsp;                                  'current' => '138',
##&nbsp;                                  'total' => '1320',
##&nbsp;                                  'cache' => '1182'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'bytes_allocated_to_network' => [
##&nbsp;                                              {
##&nbsp;                                                'current' => '548K',
##&nbsp;                                                'total' => '3370K',
##&nbsp;                                                'cache' => '2821K'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'sfbufs_in_use' => [
##&nbsp;                                 {
##&nbsp;                                   'max' => '0',
##&nbsp;                                   'current' => '0',
##&nbsp;                                   'peak' => '0'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;           'requests_for_jumbo_clusters_denied' => [
##&nbsp;                                                     {
##&nbsp;                                                       '9k' => '0',
##&nbsp;                                                       '4k' => '0',
##&nbsp;                                                       '16k' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;           'mbuf_clusters_in_use' => [
##&nbsp;                                       {
##&nbsp;                                         'max' => '16640',
##&nbsp;                                         'current' => '1',
##&nbsp;                                         'total' => '682',
##&nbsp;                                         'cache' => '681'
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;           'jumbo_clusters_in_use' => [
##&nbsp;                                        {
##&nbsp;                                          'page_size' => '4k',
##&nbsp;                                          'max' => '8320',
##&nbsp;                                          'current' => '128',
##&nbsp;                                          'total' => '419',
##&nbsp;                                          'cache' => '291'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'page_size' => '9k',
##&nbsp;                                          'max' => '4160',
##&nbsp;                                          'current' => '0',
##&nbsp;                                          'total' => '0',
##&nbsp;                                          'cache' => '0'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'page_size' => '16k',
##&nbsp;                                          'max' => '2080',
##&nbsp;                                          'current' => '0',
##&nbsp;                                          'total' => '0',
##&nbsp;                                          'cache' => '0'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;         }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = netstat -L
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current listen queue sizes (qlen/incqlen/maxqlen)
##&nbsp;Proto Listen         Local Address
##&nbsp;unix  0/0/100        /var/run/mdnsd
##&nbsp;unix  0/0/128        /var/run/rpcbind.sock
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'proto' => 'unix',
##&nbsp;            'local_address' => '/var/run/mdnsd',
##&nbsp;            'listen' => '0/0/100'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'proto' => 'unix',
##&nbsp;            'local_address' => '/var/run/rpcbind.sock',
##&nbsp;            'listen' => '0/0/128'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Name    Mtu Network       Address              Ipkts Ierrs Idrop    Opkts Oerrs  Coll
##&nbsp;lo0    8232 <Link#1>                        44378816     0     0 44379015     0     0
##&nbsp;lo0    8232 your-net      localhost         44378218     -     -       45     -     -
##&nbsp;lo0    8232 localhost     ::1                      0     -     -     1031     -     -
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '<Link#1>',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'lo0',
##&nbsp;            'idrop' => '0',
##&nbsp;            'opkts' => '44379015',
##&nbsp;            'ipkts' => '44378816',
##&nbsp;            'address' => '-',
##&nbsp;            'coll' => '0',
##&nbsp;            'mtu' => '8232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '-',
##&nbsp;            'network' => 'your-net',
##&nbsp;            'ierrs' => '-',
##&nbsp;            'name' => 'lo0',
##&nbsp;            'idrop' => '-',
##&nbsp;            'opkts' => '45',
##&nbsp;            'ipkts' => '44378218',
##&nbsp;            'address' => 'localhost',
##&nbsp;            'coll' => '-',
##&nbsp;            'mtu' => '8232'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '-',
##&nbsp;            'network' => 'localhost',
##&nbsp;            'ierrs' => '-',
##&nbsp;            'name' => 'lo0',
##&nbsp;            'idrop' => '-',
##&nbsp;            'opkts' => '1031',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'address' => '::1',
##&nbsp;            'coll' => '-',
##&nbsp;            'mtu' => '8232'
##&nbsp;          }
##&nbsp;        ]
##&nbsp;
sub netstat {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
	my $command = $resp->get_actual_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
	my $group;
	my %hash;
	my $temp='abc';
	my $x=0;
	my $i=0;
    my $count = -1;
    my ( $main_head, $head, $value, $pagesize );
    my ( @head, @values );
	while($i<@lines) {
		if($command =~ /-r/) {
			if($temp =~ /Fibnum:\s*\d+/){ # to check if any group is existing
				$group=$temp;
			} elsif ( $temp eq 'Internet6:' || $lines[$i] =~ /Internet/ ) {
				$group='default';
			}
			if($lines[$i] =~ /Internet:/){ # For adress_family Internet
				$i++;
				$i++;
				while ($i) {
					my %internal_hash; # specific to one adress_family of a group
					my @values= split(' ', $lines[$i]);
					$internal_hash{'destination'} = $values[0];
					$internal_hash{'gateway'} = $values[1];
					$internal_hash{'flag'} = $values[2];
					$internal_hash{'ref'} = $values[3];
					$internal_hash{'use'} = $values[4];
					$internal_hash{'netif'} = $values[5];
					if(defined($values[6])){
						$internal_hash{'expire'} = $values[6];
					}
					push (@{$hash{$group}[0]{'addr-family-inet'}}, \%internal_hash);
					$i++;
					if($lines[$i] =~ /Fibnum:\s*(\d+)/) { # Changing the group name if new group found
						$temp='Fibnum: '.$1;
					} else {
						$temp='Internet6:'; # Changing the adress_family if new adress_family found
					}
					last if($lines[$i] =~ /Fibnum:\s\d+/ || $lines[$i] =~ /Internet6:/ || $lines[$i]=~ /Internet:/ || $i eq @lines); #To check for next group OR adress_family OR end-of-output
				}
			} elsif ($temp eq 'Internet6:' || $lines[$i] =~ /Internet6:/) { # For adress_family Internet6
				if($lines[$i] =~ /Internet6:/){
					$i++;
				}
				$i++;
				while ($i) {
					my %internal_hash; # specific to one adress_family of a group
					my @values= split(' ', $lines[$i]);
					$internal_hash{'destination'} = $values[0];
					$internal_hash{'gateway'} = $values[1];
					$internal_hash{'flag'} = $values[2];
					$internal_hash{'netif'} = $values[3];
					if(defined($values[4])){
						$internal_hash{'expire'} = $values[4];
					}
					push (@{$hash{$group}[0]{'addr-family-inet6'}}, \%internal_hash);
					$i++;
					if($lines[$i] =~ /Fibnum:\s*(\d+)/) { # Changing the group name if new group found
						$temp='Fibnum: '.$1;
					} else {
						$temp='Internet6:'; # Changing the adress_family if new adress_family found
					}
					last if($lines[$i] =~ /Fibnum:\s\d+/ || $lines[$i] =~ /Internet6:/ || $lines[$i] =~ /Internet:/ || $i eq @lines);#To check for next group OR adress_family OR end-of-output
				}
			}
        }
		$i++;
	}
	my @groups=keys %hash;

	while($x<@groups) {
		my %finalhash = %{$hash{$groups[$x]}->[0]};
		$finalhash{'group_name'}=$groups[$x];
		$x++;
		push (@result_array, \%finalhash);
	}
    if( $command =~ /-i/ ) {
        for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
    	    $count++;
            @head = split( /\s+/, $lines[0] );
            @values = split( /\s+/, $lines[$i] );
            if ( $#head != $#values ) {
                splice( @values, 3, 0, "-" );
            }
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
              $result_array[$count]->{to_key_name( item =>$head[$j])} =  $values[$j];
            }
        }
    } elsif ( $command =~ /-m/ ) {
      $count++;
      foreach my $line (@lines) {
          if ( $line =~ /(.*\d+)\s+(\w+)\s+\((page\s+size)\)\s+(.*\w+)\s+\((.*)\)/ ) {
              $pagesize = $2;
              ( $head, $main_head, $value ) = ( $5, $4, $1 );
          } elsif ( $line =~ /(.*\d+)\s+(\d+\w+)\s+(.*\w+)\s+\((.*)\)/ ) {
              $count++;
              $pagesize = $2;
              ( $head, $main_head, $value ) = ( $4, $3, $1 );
          } elsif ( $line =~ /(.*\d+[A-Z]*)\s+(.*\w+)\s+\((.*)\)/ ) {
              $count = 0;
              ( $head, $main_head, $value ) = ( $3, $2, $1 );
          } elsif ( $line =~ /(^\d+)\s+(.*)/ ) {
              my $key   = $2;
              my $value = $1;
              $key =~ s/\///;
              $result_array[$count]{to_key_name( item =>$key)} = $value;
          }
          @values = split( /\//, $value );
          @head   = split( /\//, $head );
          $main_head =~ s/\+/_/;
          for ( my $j = 0 ; $j <= $#head ; $j++ ) {
              $head[$j] =~ s/\+/_/;
              $result_array[0]{to_key_name( item =>$main_head)}[$count]
                  { to_key_name( item =>$head[$j]) } = $values[$j];
              $result_array[0]{to_key_name( item =>$main_head)}[$count]
                   {page_size} = $pagesize if ( $line =~ /jumbo.*use/ );
          }
        }
    } elsif ( $command =~ /-L/ ) {
         for ( my $i = 2 ; $i <= $#lines ; $i++ ) {
             $lines[1] =~ s/Local Address/local_address/;
             @head   = split( /\s+/, $lines[1] );
             @values = split( /\s+/, $lines[$i] );
             $count++;
             for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                 $result_array[$count]->{ to_key_name( item =>$head[$j]) } = $values[$j];
             }
         }
     } else {
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
            $lines[$i] =~ s/Local Address/local_address/;
            $lines[$i] =~ s/Foreign Address/foreign_address/;
            $lines[$i] =~ s/Nextref Addr/nextref_address/;
            $lines[$i] =~ s/-/_/;
            if ( $lines[$i] =~
                /Active\s+Internet\s+connections|Active\s+UNIX\s+domain\s+sockets/ ) {
                $main_head = $lines[$i];
            } elsif ( $lines[$i] =~ /Proto.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /Address\s+Type.*|Name\s+Mtu.*/ ) {
                @head = split( /\s+/, $lines[$i] );
                $count = -1;
            } elsif ( $lines[$i] =~ /(\w+)\s+(\w+)\s+(\S+).*/ ) {
                $count++;
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                    $head[$j] =~ s/\(//;
                    $head[$j] =~ s/\)//;
                    $head[$j] =~ s/-/_/;
                    $result_array[0]->{to_key_name( item =>$main_head)}->[$count]
                       ->{ to_key_name( item =>$head[$j])} = $values[$j];
                }
            }
        }
    }
    return \@result_array; #to be updated to return a reference
}
1;
