#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse ntpdate command output
##
## @description
##   This library supports error checking for ntpdate command, but output parsing is not supported as of now.
##
## @synopsis
##   use NACL::APISet::Host::CLI::Freebsd::default::ntpdate;
##
##  ntpdate(%args);
##
## @author asubhash@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ntpdate;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException ;
use NACL::APISet::Exceptions::MissingRequiredParamException ;

## @name ::ntpdate
##
## @summary Parses the raw output from the "ntpdate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ntpdate with any of its options as described in the
##    corresponding cdef file 'ntpdate.cdefs'
## @synopsis
##      ntpdate(%args);
## INPUTS TO METHOD CALL 'ntpdate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-6' </td> <td> 'ipv6' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'authdelay' </td> <td> '0' </td> </tr>
## <tr> <td> '-4' </td> <td> 'ipv4' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'query' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'authentication' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'samples' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'adjtime_call' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'unprivileged_port' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'syslog' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'keyfile' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'settimeofday_call' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rate' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'server' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;

    if ( $raw_output =~ /option\s+requires\s+an\s+argument.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /encryption\s+delay|number\s+of\s+samples|rate.*is\s+invalid/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

sub ntpdate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub ntpdate

1;
