#
# Copyright (c) 2013-2015 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse SystemShell nvmecontrol  commands
##
## @description
##     This is a library to parse the raw output of SystemShell nvmecontrol command.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use nvmecontrol ;
##
##     nvmecontrol (%args);
##
## @author dl-nacl-dev
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::nvmecontrol;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args            = @_;
    my $raw_output      = $args{raw_output};
    my $command         = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ($raw_output =~ /(nvmecontrol.*)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }

} ## end sub _check_error

## @name ::nvmecontrol_devlist
##
## @summary Parses the raw output from the "sudo nvmecontrol devlist" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nvmecontrol_devlist with any of its options as described in the 
##    corresponding cdef file 'nvmecontrol.cdefs'
## @synopsis
##      nvmecontrol_devlist(%args);
## INPUTS TO METHOD CALL 'nvmecontrol_devlist' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sudo nvmecontrol devlist
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; nvme0: SAMSUNG MZWEI400HAGM-00003
##&nbsp;    nvme0ns1 (381554MB)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nvme0' => {
##&nbsp;                         'model_information' => 'SAMSUNG MZWEI400HAGM-00003',
##&nbsp;                         'device_id' => 'nvme0',
##&nbsp;                         'namespace_id' => 'ns1',
##&nbsp;                         'size' => '381554MB'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub nvmecontrol_devlist{
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my @result_array;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    my @lines = get_lines(raw_output => $raw_output, response_object => $resp);

    while(@lines) {
        my $line = shift (@lines);
        my @values = split(/:\s+/, $line);
        my $next_line = shift (@lines);
        if ( $next_line =~ /(\w+)\s+\((.*)\)/ ) {
            $result_array[0]->{to_key_name(item => $values[0])}{namespace_id} = substr($1,-3);
            $result_array[0]->{to_key_name(item => $values[0])}{size} = $2;
        }
        $result_array[0]->{to_key_name(item => $values[0])}{device_id} = $values[0];
        $result_array[0]->{to_key_name(item => $values[0])}{model_information} = $values[1];
    }
    return \@result_array;
} ## end sub nvmecontrol_devlist

sub nvmecontrol_reset {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub nvmecontrol_reset

## @name ::nvmecontrol_logpage
##
## @summary Parses the raw output from the "sudo nvmecontrol logpage" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nvmecontrol_logpage with any of its options as described in the 
##    corresponding cdef file 'nvmecontrol.cdefs'
## @synopsis
##      nvmecontrol_logpage(%args);
## INPUTS TO METHOD CALL 'nvmecontrol_logpage' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'namespace_id' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'controller_id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'page_id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sudo nvmecontrol logpage -p 3 nvme0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Firmware Slot Log
##&nbsp;=================
##&nbsp;Slot 1: [  Active] IPM0CB3Q
##&nbsp;Slot 2: Empty
##&nbsp;Slot 3: Empty
##&nbsp;Slot 4: Empty
##&nbsp;Slot 5: Empty
##&nbsp;Slot 6: Empty
##&nbsp;Slot 7: Empty
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'firmware_slot_log' => {
##&nbsp;                                     'slot_5' => 'Empty',
##&nbsp;                                     'slot_1' => '[  Active] IPM0CB3Q',
##&nbsp;                                     'slot_7' => 'Empty',
##&nbsp;                                     'slot_3' => 'Empty',
##&nbsp;                                     'slot_6' => 'Empty',
##&nbsp;                                     'slot_2' => 'Empty',
##&nbsp;                                     'slot_4' => 'Empty'
##&nbsp;                                   }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub nvmecontrol_logpage {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my @result_array;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );

    my @lines
            = get_lines(raw_output => $raw_output, response_object => $resp);
    while (@lines) {
        my $line = shift @lines;
        if ( $line =~ /^Slot/ ) {
            my @values = split(/:\s+/, $line);
            $result_array[0]->{firmware_slot_log}{to_key_name(item => $values[0])} = $values[1];
        }
    }
    return \@result_array;
} ## end sub nvmecontrol_logpage
1;
