## @name ::pciconf
##
## @summary Parses the raw output from the "pciconf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pciconf with any of its options as described in the
##    corresponding cdef file 'pciconf.cdefs'
## @synopsis
##      pciconf(%args);
package NACL::APISet::Host::CLI::Freebsd::default::pciconf;

use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

use strict;
use warnings;
## INPUTS TO METHOD CALL 'pciconf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'halfword_operation' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'selector' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'errors_reported' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'assigned_driver' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vendor_device_info' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'from_offset_addr' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'byte_operation' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list_all_devices' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'read' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'base_address_register' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'capabilities' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'to_offset_addr' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'write' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pciconf -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hostb0@pci0:0:0:0:	class=0x060000 card=0x65c08086 chip=0x65c08086 rev=0x90 hdr=0x00
##&nbsp;pcib0@pci0:0:2:0:	class=0x060400 card=0x00000000 chip=0x65f78086 rev=0x90 hdr=0x01
##&nbsp;pcib1@pci0:0:3:0:	class=0x060400 card=0x00000000 chip=0x65e38086 rev=0x90 hdr=0x01
##&nbsp;pcib2@pci0:0:4:0:	class=0x060400 card=0x00000000 chip=0x65f88086 rev=0x90 hdr=0x01
##&nbsp;pcib3@pci0:0:5:0:	class=0x060400 card=0x00000000 chip=0x65e58086 rev=0x90 hdr=0x01
##&nbsp;pcib4@pci0:0:6:0:	class=0x060400 card=0x00000000 chip=0x65f98086 rev=0x90 hdr=0x01
##&nbsp;pcib5@pci0:0:7:0:	class=0x060400 card=0x00000000 chip=0x65e78086 rev=0x90 hdr=0x01
##&nbsp;ontap0@pci0:0:8:0:	class=0x088000 card=0x80868086 chip=0x65ff8086 rev=0x90 hdr=0x00
##&nbsp;hostb1@pci0:0:16:0:	class=0x060000 card=0x80868086 chip=0x65f08086 rev=0x90 hdr=0x00
##&nbsp;sanclem_fsbf10@pci0:0:16:1:	class=0x060000 card=0x80868086 chip=0x65f08086 rev=0x90 hdr=0x00
##&nbsp;hostb2@pci0:0:16:2:	class=0x060000 card=0x80868086 chip=0x65f08086 rev=0x90 hdr=0x00
##&nbsp;hostb3@pci0:0:17:0:	class=0x060000 card=0x80868086 chip=0x65f18086 rev=0x90 hdr=0x00
##&nbsp;hostb4@pci0:0:19:0:	class=0x060000 card=0x80868086 chip=0x65f38086 rev=0x90 hdr=0x00
##&nbsp;hostb5@pci0:0:21:0:	class=0x060000 card=0x80868086 chip=0x65f58086 rev=0x90 hdr=0x00
##&nbsp;hostb6@pci0:0:22:0:	class=0x060000 card=0x80868086 chip=0x65f68086 rev=0x90 hdr=0x00
##&nbsp;none0@pci0:0:26:0:	class=0x0c0300 card=0x00000000 chip=0x29378086 rev=0x02 hdr=0x00
##&nbsp;none1@pci0:0:26:7:	class=0x0c0320 card=0x27cc8086 chip=0x293c8086 rev=0x02 hdr=0x00
##&nbsp;pcib6@pci0:0:28:0:	class=0x060400 card=0x00000000 chip=0x29408086 rev=0x02 hdr=0x01
##&nbsp;pcib7@pci0:0:28:4:	class=0x060400 card=0x00000000 chip=0x29488086 rev=0x02 hdr=0x01
##&nbsp;pcib8@pci0:0:28:5:	class=0x060400 card=0x00000000 chip=0x294a8086 rev=0x02 hdr=0x01
##&nbsp;none2@pci0:0:29:0:	class=0x0c0300 card=0x00000000 chip=0x29348086 rev=0x02 hdr=0x00
##&nbsp;ehci0@pci0:0:29:7:	class=0x0c0320 card=0x27cc8086 chip=0x293a8086 rev=0x02 hdr=0x00
##&nbsp;pcib9@pci0:0:30:0:	class=0x060401 card=0x00000000 chip=0x244e8086 rev=0x92 hdr=0x01
##&nbsp;isab0@pci0:0:31:0:	class=0x060100 card=0x00000000 chip=0x29188086 rev=0x02 hdr=0x00
##&nbsp;ichsmb0@pci0:0:31:3:	class=0x0c0500 card=0x00000000 chip=0x29308086 rev=0x02 hdr=0x00
##&nbsp;none3@pci0:0:31:6:	class=0x118000 card=0x00000000 chip=0x29328086 rev=0x02 hdr=0x00
##&nbsp;ontap1@pci0:1:0:0:	class=0x020000 card=0x00011425 chip=0x00fe1425 rev=0x00 hdr=0x00
##&nbsp;plxsw86xx0@pci0:3:0:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;plxsw86xx1@pci0:4:4:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;plxsw86xx2@pci0:4:5:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;plxsw86xx3@pci0:4:8:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;ontap2@pci0:6:0:0:	class=0x020000 card=0x000c8086 chip=0x10fb8086 rev=0x01 hdr=0x00
##&nbsp;ontap3@pci0:6:0:1:	class=0x020000 card=0x000c8086 chip=0x10fb8086 rev=0x01 hdr=0x00
##&nbsp;ontap4@pci0:7:0:0:	class=0x020000 card=0x000c8086 chip=0x10fb8086 rev=0x01 hdr=0x00
##&nbsp;ontap5@pci0:7:0:1:	class=0x020000 card=0x000c8086 chip=0x10fb8086 rev=0x01 hdr=0x00
##&nbsp;plxsw86xx4@pci0:9:0:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;none4@pci0:10:0:0:	class=0x068000 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x00
##&nbsp;plxsw86xx5@pci0:10:4:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;plxsw86xx6@pci0:10:5:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;plxsw86xx7@pci0:10:6:0:	class=0x060400 card=0x862410b5 chip=0x862410b5 rev=0xbb hdr=0x01
##&nbsp;ontap6@pci0:12:0:0:	class=0x0c0400 card=0x01381077 chip=0x64321077 rev=0x02 hdr=0x00
##&nbsp;ontap7@pci0:12:0:1:	class=0x0c0400 card=0x01381077 chip=0x64321077 rev=0x02 hdr=0x00
##&nbsp;ontap8@pci0:13:0:0:	class=0x010700 card=0x00000000 chip=0x800111f8 rev=0x05 hdr=0x00
##&nbsp;ontap9@pci0:15:0:0:	class=0x020000 card=0x125e8086 chip=0x105e8086 rev=0x06 hdr=0x00
##&nbsp;ontap10@pci0:15:0:1:	class=0x020000 card=0x125e8086 chip=0x105e8086 rev=0x06 hdr=0x00
##&nbsp;ontap11@pci0:16:0:0:	class=0x020000 card=0x165914e4 chip=0x165914e4 rev=0x21 hdr=0x00
##&nbsp;ontap12@pci0:17:0:0:	class=0x020000 card=0x165914e4 chip=0x165914e4 rev=0x21 hdr=0x00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65c08086',
##&nbsp;            'card' => '0x65c08086',
##&nbsp;            'selector' => 'pci0:0:0:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb0',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65f78086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:2:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib0',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65e38086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:3:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib1',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65f88086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:4:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib2',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65e58086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:5:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib3',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65f98086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:6:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib4',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x65e78086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:7:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'pcib5',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65ff8086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:8:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'ontap0',
##&nbsp;            'class' => '0x088000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f08086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:16:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb1',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f08086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:16:1:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'sanclem_fsbf10',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f08086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:16:2:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb2',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f18086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:17:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb3',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f38086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:19:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb4',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f58086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:21:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb5',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x65f68086',
##&nbsp;            'card' => '0x80868086',
##&nbsp;            'selector' => 'pci0:0:22:0:',
##&nbsp;            'rev' => '0x90',
##&nbsp;            'driver_name_&_unit_number' => 'hostb6',
##&nbsp;            'class' => '0x060000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x29378086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:26:0:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'none0',
##&nbsp;            'class' => '0x0c0300'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x293c8086',
##&nbsp;            'card' => '0x27cc8086',
##&nbsp;            'selector' => 'pci0:0:26:7:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'none1',
##&nbsp;            'class' => '0x0c0320'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x29408086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:28:0:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'pcib6',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x29488086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:28:4:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'pcib7',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x294a8086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:28:5:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'pcib8',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x29348086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:29:0:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'none2',
##&nbsp;            'class' => '0x0c0300'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x293a8086',
##&nbsp;            'card' => '0x27cc8086',
##&nbsp;            'selector' => 'pci0:0:29:7:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'ehci0',
##&nbsp;            'class' => '0x0c0320'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x244e8086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:30:0:',
##&nbsp;            'rev' => '0x92',
##&nbsp;            'driver_name_&_unit_number' => 'pcib9',
##&nbsp;            'class' => '0x060401'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x29188086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:31:0:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'isab0',
##&nbsp;            'class' => '0x060100'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x29308086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:31:3:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'ichsmb0',
##&nbsp;            'class' => '0x0c0500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x29328086',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:0:31:6:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'none3',
##&nbsp;            'class' => '0x118000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x00fe1425',
##&nbsp;            'card' => '0x00011425',
##&nbsp;            'selector' => 'pci0:1:0:0:',
##&nbsp;            'rev' => '0x00',
##&nbsp;            'driver_name_&_unit_number' => 'ontap1',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:3:0:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx0',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:4:4:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx1',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:4:5:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx2',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:4:8:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx3',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x10fb8086',
##&nbsp;            'card' => '0x000c8086',
##&nbsp;            'selector' => 'pci0:6:0:0:',
##&nbsp;            'rev' => '0x01',
##&nbsp;            'driver_name_&_unit_number' => 'ontap2',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x10fb8086',
##&nbsp;            'card' => '0x000c8086',
##&nbsp;            'selector' => 'pci0:6:0:1:',
##&nbsp;            'rev' => '0x01',
##&nbsp;            'driver_name_&_unit_number' => 'ontap3',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x10fb8086',
##&nbsp;            'card' => '0x000c8086',
##&nbsp;            'selector' => 'pci0:7:0:0:',
##&nbsp;            'rev' => '0x01',
##&nbsp;            'driver_name_&_unit_number' => 'ontap4',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x10fb8086',
##&nbsp;            'card' => '0x000c8086',
##&nbsp;            'selector' => 'pci0:7:0:1:',
##&nbsp;            'rev' => '0x01',
##&nbsp;            'driver_name_&_unit_number' => 'ontap5',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:9:0:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx4',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:10:0:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'none4',
##&nbsp;            'class' => '0x068000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:10:4:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx5',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:10:5:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx6',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x01',
##&nbsp;            'chip' => '0x862410b5',
##&nbsp;            'card' => '0x862410b5',
##&nbsp;            'selector' => 'pci0:10:6:0:',
##&nbsp;            'rev' => '0xbb',
##&nbsp;            'driver_name_&_unit_number' => 'plxsw86xx7',
##&nbsp;            'class' => '0x060400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x64321077',
##&nbsp;            'card' => '0x01381077',
##&nbsp;            'selector' => 'pci0:12:0:0:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'ontap6',
##&nbsp;            'class' => '0x0c0400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x64321077',
##&nbsp;            'card' => '0x01381077',
##&nbsp;            'selector' => 'pci0:12:0:1:',
##&nbsp;            'rev' => '0x02',
##&nbsp;            'driver_name_&_unit_number' => 'ontap7',
##&nbsp;            'class' => '0x0c0400'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x800111f8',
##&nbsp;            'card' => '0x00000000',
##&nbsp;            'selector' => 'pci0:13:0:0:',
##&nbsp;            'rev' => '0x05',
##&nbsp;            'driver_name_&_unit_number' => 'ontap8',
##&nbsp;            'class' => '0x010700'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x105e8086',
##&nbsp;            'card' => '0x125e8086',
##&nbsp;            'selector' => 'pci0:15:0:0:',
##&nbsp;            'rev' => '0x06',
##&nbsp;            'driver_name_&_unit_number' => 'ontap9',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x105e8086',
##&nbsp;            'card' => '0x125e8086',
##&nbsp;            'selector' => 'pci0:15:0:1:',
##&nbsp;            'rev' => '0x06',
##&nbsp;            'driver_name_&_unit_number' => 'ontap10',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x165914e4',
##&nbsp;            'card' => '0x165914e4',
##&nbsp;            'selector' => 'pci0:16:0:0:',
##&nbsp;            'rev' => '0x21',
##&nbsp;            'driver_name_&_unit_number' => 'ontap11',
##&nbsp;            'class' => '0x020000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hdr' => '0x00',
##&nbsp;            'chip' => '0x165914e4',
##&nbsp;            'card' => '0x165914e4',
##&nbsp;            'selector' => 'pci0:17:0:0:',
##&nbsp;            'rev' => '0x21',
##&nbsp;            'driver_name_&_unit_number' => 'ontap12',
##&nbsp;            'class' => '0x020000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub pciconf {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my @result_array;
    my @lines = get_lines( raw_output => $output, can_throw => 1, response_object => $resp );
    foreach (@lines) {
    my %hash = ();
    if ($_ =~ /(.*)@(pci.*)\s+class=(.*)\s+card=(.*)\s+chip=(.*)\s+rev=(.*)\s+hdr=(.*)/) {
            $hash{'driver_name_&_unit_number'} = $1;
	    $hash{'selector'} = $2;
            $hash{'class'} = $3;
            $hash{'card'} = $4;
	    $hash{'chip'} = $5;
	    $hash{'rev'} = $6;
            $hash{'hdr'} = $7;
	    push(@result_array, \%hash);
	}
     }
    return \@result_array;
} ## end sub pciconf

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $resp          = $args{response_object};
    my $error_level   = $resp->get_execution_status();

    my $error_message;
    my $error         = undef;
    if ( $raw_output =~ /pciconf:/) {
        $error         = "InvalidParamValue";
	$error_message = $raw_output;
    }
    elsif($raw_output =~ /usage:/ ){
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
		text => "Execution status was $error_level "
              . "indicating that the command has failed",
		response_object => $resp
        );
    }
} ## end sub _check_error

1;
