#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ping command output
##
## @description
##    This is a library to parse the raw output of Linux ping command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::ping;
##
##    ping(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ping;

use NACL::APISet::Response::CLI::ParserUtils;
use strict;


## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'time-to-live' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'sweepincrsize' </td> </tr> 
## <tr> <td> '-G' </td> <td> 'sweepmaxsize' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'sweepminsize' </td> </tr> 
## <tr> <td> '-W' </td> <td> 'waittime' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'audible' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'packet-size' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'timeout' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'source-address' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'adaptive' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interval' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'so-debug' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pattern' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'numeric' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'preload' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ping -c 3 10.72.40.202
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stddev' => '0.576',
##&nbsp;            'avg' => '1.525',
##&nbsp;            'min' => '0.835',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'max' => '2.246',
##&nbsp;            'loss' => '0.0%',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '2.246',
##&nbsp;                           'ttl' => '255',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => '10.72.40.202',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                       ],
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub ping {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed\n$output",
            response_object => $resp);
    }

    my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'PING .+\s+data', 
                          response_object=>$resp);
    
    my $line = shift(@lines);
    my %hash;
    if($line =~ /PING ([^\s]+)\s+\(([^\s]+)\)\s+([^\s]+).+data/) {
        $hash{'host'} = $1;
        $hash{'ip'} = $2;
        $hash{'bytes'} = $3;
    }

    $hash{'stats'} = [];

    while (@lines) {
        my %internal_hash;
        $line = $lines[0];
        shift(@lines);
        if ($line =~ /([^\s]+)\s+bytes\s+from\s+([^:]+):\s+(.+)\sms/) {
            $internal_hash{'packet_size'} = $1;
            $internal_hash{'ip'} = $2;
            my $rest = $3;
            my @fields = split(/\s+/,$rest);
            foreach my $field(@fields) {
                my ($name,$value) = split(/=/,$field);
                $internal_hash{$name} = $value;
            }
            push (@{$hash{'stats'}}, \%internal_hash);
        }
        last if($line =~ /--- .+ ping statistics ---/);
    }

    $line = $lines[0];

    #3 packets transmitted, 3 packets received, 0.0% packet loss
    if ($line =~ /(\d+) packets transmitted, (\d+) packets received, (\d+\.\d+%) packet loss/) {

        $hash{packets_transmitted} = $1;
        $hash{received} = $2;
        $hash{loss} = $3;
        shift(@lines);
        $line = $lines[0];
    }
    
    #round-trip min/avg/max/stddev = 0.661/0.825/1.125/0.212 ms
    if($line =~ /round-trip (.+)\s+=\s+(.+)\s+ms/) {
        my @names = split(/\//, $1);
        my @values = split(/\//, $2);
        my $i;
        foreach my $name (@names) {
            $hash{$name}=$values[$i];
            $i++;
        }
    }

    my @result_array = (\%hash);
    return \@result_array;
}

## @name ::ping6
##
## @summary Parses the raw output from the "ping6" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping6 with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping6(%args);
## INPUTS TO METHOD CALL 'ping6' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '-N' </td> <td> 'probe-node' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'gateway' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'exit' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'audible-receive' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'flood' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'packet-size' </td> </tr> 
## <tr> <td> '-D' </td> <td> 'disable-v6' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'timeout' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'source-address' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'minimum-mtu' </td> </tr> 
## <tr> <td> '-P' </td> <td> 'policy' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'interface' </td> </tr>
## <tr> <td> '-H' </td> <td> 'reverse-lookup' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'hops' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'bufsiz' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'addrtype' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'audible-transmit' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'host' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'so-debug' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pattern' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'wait' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'hoplimit' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'preload' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'numeric' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ping6 -c 3 fe80::2a0:98ff:fe25:b9f1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING6(56=40+8+8 bytes) fe80::2a0:98ff:fe23:d0a3%default-ipspace --> fe80::2a0:98ff:fe25:b9f1
##&nbsp;16 bytes from fe80::2a0:98ff:fe25:b9f1%default-ipspace, icmp_seq=0 hlim=64 time=0.311 ms
##&nbsp;16 bytes from fe80::2a0:98ff:fe25:b9f1%default-ipspace, icmp_seq=1 hlim=64 time=0.281 ms
##&nbsp;16 bytes from fe80::2a0:98ff:fe25:b9f1%default-ipspace, icmp_seq=2 hlim=64 time=0.280 ms
##&nbsp;
##&nbsp;--- fe80::2a0:98ff:fe25:b9f1 ping6 statistics ---
##&nbsp;3 packets transmitted, 3 packets received, 0.0% packet loss
##&nbsp;round-trip min/avg/max/std-dev = 0.280/0.291/0.311/0.014 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56',
##&nbsp;            'avg' => '0.291',
##&nbsp;            'min' => '0.280',
##&nbsp;            'ip' => 'fe80::2a0:98ff:fe25:b9f1',
##&nbsp;            'max' => '0.311',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'hlim' => '64',
##&nbsp;                           'time' => '0.311',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'fe80::2a0:98ff:fe25:b9f1',
##&nbsp;                           'packet_size' => '16'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'hlim' => '64',
##&nbsp;                           'time' => '0.281',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'fe80::2a0:98ff:fe25:b9f1',
##&nbsp;                           'packet_size' => '16'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'hlim' => '64',
##&nbsp;                           'time' => '0.280',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'fe80::2a0:98ff:fe25:b9f1',
##&nbsp;                           'packet_size' => '16'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'fe80::2a0:98ff:fe23:d0a3',
##&nbsp;            'std-dev' => '0.014',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0.0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping6 {
    my %args = @_;

    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed\n$output",
            response_object => $resp);
    }

    my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'PING6\(.+=',
                          response_object=>$resp);

    my $line = shift(@lines);
    my %hash;
    if($line =~ /PING6\((.+)=.+\sbytes\)\s(.+)\s-->\s(.+)/) {
        $hash{'bytes'} = $1;
        $hash{'host'} = $2;
        $hash{'ip'} = $3;
        if( $hash{'host'} =~ /%/ ) {
            my $host = $`;
            $hash{'host'} = $host;
        }
    }

    $hash{'stats'} = [];

    while (@lines) {
        my %internal_hash;
        $line = $lines[0];
        shift(@lines);

        if ($line =~ /(\d+) bytes from (.+),\s+(.+)\sms/) {
            $internal_hash{'packet_size'} = $1;
            $internal_hash{'ip'} = $2;
            my $rest = $3;
            my @fields = split(/\s+/,$rest);
            foreach my $field(@fields) {
                my ($name,$value) = split(/=/,$field);
                $internal_hash{$name} = $value;
            }
            if($internal_hash{'ip'} =~ /%/) {
                my $ip = $`; 
                $internal_hash{'ip'} = $ip;
            }
            push (@{$hash{'stats'}}, \%internal_hash);
        }
        last if($line =~ /--- .+ ping6 statistics ---/);
    }

    $line = $lines[0];

    #3 packets transmitted, 3 packets received, 0.0% packet loss
    if ($line =~ /(\d+) packets transmitted, (\d+) packets received, (\d+\.\d+%) packet loss/) {

        $hash{packets_transmitted} = $1;
        $hash{received} = $2;
        $hash{loss} = $3;
        shift(@lines);
        $line = $lines[0];
    }

    #round-trip min/avg/max/stddev = 0.661/0.825/1.125/0.212 ms
    if($line =~ /round-trip (.+)\s+=\s+(.+)\s+ms/) {
        my @names = split(/\//, $1);
        my @values = split(/\//, $2);
        my $i;
        foreach my $name (@names) {
            $hash{$name}=$values[$i];
            $i++;
        }
    }

    my @result_array = (\%hash);
    return \@result_array;
}

1;
