#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd ps command"
##
## @description
##    This is a library to parse the raw output of Freebsd ps command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ps" will be ps()
##
## @synopsis
##    use ps;
##
##    ps(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ps;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::ps
##
## @summary Parses the raw output from the "ps" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ps with any of its options as described in the
##    corresponding cdef file 'ps.cdefs'
## @synopsis
##      ps(%args);
## INPUTS TO METHOD CALL 'ps' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'list-keywords' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'change-executable-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-X' </td> <td> 'controlling-terminal' </td> <td> '1' </td> </tr>
## <tr> <td> '-G' </td> <td> 'group-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'terminal' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'sort-cpu-usage' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'show-commandline' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'kernel-threads' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'mac-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wide' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'display-controlling-terminal' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'cpu-calculation' </td> <td> '1' </td> </tr>
## <tr> <td> '-j' </td> <td> 'short-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'process-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'parent-process' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'environment' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'descendancy' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'terminal-device' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'memory' </td> <td> '1' </td> </tr>
## <tr> <td> '-O' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'display-user-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'core-name-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'header' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PID  TT  STAT      TIME COMMAND
##&nbsp;42065   0  I+     0:00.03 -csh (csh)
##&nbsp;42546   1  S      0:00.05 bash --norc --noprofile -i
##&nbsp;42551   1  R+     0:00.00 ps
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pid' => '42065',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '42546',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '42551',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'command' => 'ps'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -Z
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;LABEL   PID  TT  STAT      TIME COMMAND
##&nbsp;  -   42065   0  I+     0:00.03 -csh (csh)
##&nbsp;  -   42596   1  S      0:00.05 bash --norc --noprofile -i
##&nbsp;  -   42601   1  R+     0:00.00 ps -Z
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pid' => '42065',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'label' => '-',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '42596',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'label' => '-',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '42601',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'label' => '-',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'command' => 'ps -Z'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -j
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;USER   PID  PPID  PGID   SID JOBC STAT  TT       TIME COMMAND
##&nbsp;diag 42065 42064 42065 42064    1 I+     0    0:00.03 -csh (csh)
##&nbsp;diag 42614 42613 42614 42613    1 S      1    0:00.05 bash --norc --noprofile -i
##&nbsp;diag 42619 42614 42619 42613    1 R+     1    0:00.00 ps -j
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sid' => '42064',
##&nbsp;            'jobc' => '1',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'pid' => '42065',
##&nbsp;            'ppid' => '42064',
##&nbsp;            'user' => 'diag',
##&nbsp;            'command' => '-csh (csh)',
##&nbsp;            'pgid' => '42065'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'sid' => '42613',
##&nbsp;            'jobc' => '1',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'pid' => '42614',
##&nbsp;            'ppid' => '42613',
##&nbsp;            'user' => 'diag',
##&nbsp;            'command' => 'bash --norc --noprofile -i',
##&nbsp;            'pgid' => '42614'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'sid' => '42613',
##&nbsp;            'jobc' => '1',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'pid' => '42619',
##&nbsp;            'ppid' => '42614',
##&nbsp;            'user' => 'diag',
##&nbsp;            'command' => 'ps -j',
##&nbsp;            'pgid' => '42619'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  UID   PID  PPID CPU PRI NI   VSZ   RSS MWCHAN STAT  TT       TIME COMMAND
##&nbsp; 1002 42065 42064   0  96  0 15364  2688 ttyin  I+     0    0:00.03 -csh (csh)
##&nbsp; 1002 42876 42875   2   8  0 10252  2388 wait   S      1    0:00.05 bash --norc --noprofile -i
##&nbsp; 1002 42881 42876   1  96  0 12144  1096 -      R+     1    0:00.00 ps -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cpu' => '0',
##&nbsp;            'uid' => '1002',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'mwchan' => 'ttyin',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'pid' => '42065',
##&nbsp;            'ppid' => '42064',
##&nbsp;            'rss' => '2688',
##&nbsp;            'pri' => '96',
##&nbsp;            'ni' => '0',
##&nbsp;            'vsz' => '15364',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'cpu' => '2',
##&nbsp;            'uid' => '1002',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'mwchan' => 'wait',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'pid' => '42876',
##&nbsp;            'ppid' => '42875',
##&nbsp;            'rss' => '2388',
##&nbsp;            'pri' => '8',
##&nbsp;            'ni' => '0',
##&nbsp;            'vsz' => '10252',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'cpu' => '1',
##&nbsp;            'uid' => '1002',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'mwchan' => '-',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'pid' => '42881',
##&nbsp;            'ppid' => '42876',
##&nbsp;            'rss' => '1096',
##&nbsp;            'pri' => '96',
##&nbsp;            'ni' => '0',
##&nbsp;            'vsz' => '12144',
##&nbsp;            'command' => 'ps -l'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;USER   PID %CPU %MEM   VSZ   RSS  TT  STAT STARTED      TIME COMMAND
##&nbsp;diag 42065  0.0  0.4 15364  2688   0  I+    9:05AM   0:00.03 -csh (csh)
##&nbsp;diag 42663  0.0  0.3 10252  2388   1  S     9:22AM   0:00.05 bash --norc --noprofile -i
##&nbsp;diag 42668  0.0  0.1 12144  1096   1  R+    9:22AM   0:00.00 ps -u
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mem' => '0.4',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'pid' => '42065',
##&nbsp;            'user' => 'diag',
##&nbsp;            'started' => '9:05AM',
##&nbsp;            'rss' => '2688',
##&nbsp;            'cpu' => '0.0',
##&nbsp;            'vsz' => '15364',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mem' => '0.3',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'pid' => '42663',
##&nbsp;            'user' => 'diag',
##&nbsp;            'started' => '9:22AM',
##&nbsp;            'rss' => '2388',
##&nbsp;            'cpu' => '0.0',
##&nbsp;            'vsz' => '10252',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mem' => '0.1',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'pid' => '42668',
##&nbsp;            'user' => 'diag',
##&nbsp;            'started' => '9:22AM',
##&nbsp;            'rss' => '1096',
##&nbsp;            'cpu' => '0.0',
##&nbsp;            'vsz' => '12144',
##&nbsp;            'command' => 'ps -u'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PID STAT      TIME  SL  RE PAGEIN   VSZ   RSS   LIM TSIZ %CPU %MEM COMMAND
##&nbsp;42065 I+     0:00.03 127 127      0 15364  2688     -  340  0.0  0.4 -csh (csh)
##&nbsp;42707 S      0:00.05   0   2      0 10252  2388     -  680  1.0  0.3 bash --norc --noprofile -i
##&nbsp;42712 R+     0:00.00 127   0      0 12144  1096     -   32  0.0  0.1 ps -v
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mem' => '0.4',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            're' => '127',
##&nbsp;            'pagein' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'lim' => '-',
##&nbsp;            'sl' => '127',
##&nbsp;            'pid' => '42065',
##&nbsp;            'rss' => '2688',
##&nbsp;            'cpu' => '0.0',
##&nbsp;            'vsz' => '15364',
##&nbsp;            'tsiz' => '340',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mem' => '0.3',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            're' => '2',
##&nbsp;            'pagein' => '0',
##&nbsp;            'stat' => 'S',
##&nbsp;            'lim' => '-',
##&nbsp;            'sl' => '0',
##&nbsp;            'pid' => '42707',
##&nbsp;            'rss' => '2388',
##&nbsp;            'cpu' => '1.0',
##&nbsp;            'vsz' => '10252',
##&nbsp;            'tsiz' => '680',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mem' => '0.1',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            're' => '0',
##&nbsp;            'pagein' => '0',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'lim' => '-',
##&nbsp;            'sl' => '127',
##&nbsp;            'pid' => '42712',
##&nbsp;            'rss' => '1096',
##&nbsp;            'cpu' => '0.0',
##&nbsp;            'vsz' => '12144',
##&nbsp;            'tsiz' => '32',
##&nbsp;            'command' => 'ps -v'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -t 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PID  TT  STAT      TIME COMMAND
##&nbsp;42064   0  Is     0:00.02 login [pam] (login)
##&nbsp;42065   0  I+     0:00.03 -csh (csh)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pid' => '42064',
##&nbsp;            'time' => '0:00.02',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'Is',
##&nbsp;            'command' => 'login [pam] (login)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '42065',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps  -N 2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PID  TT  STAT      TIME COMMAND
##&nbsp;42065   0  I+     0:00.03 -csh (csh)
##&nbsp;43080   1  S      0:00.05 bash --norc --noprofile -i
##&nbsp;43085   1  R+     0:00.00 ps -N 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pid' => '42065',
##&nbsp;            'time' => '0:00.03',
##&nbsp;            'tt' => '0',
##&nbsp;            'stat' => 'I+',
##&nbsp;            'command' => '-csh (csh)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '43080',
##&nbsp;            'time' => '0:00.05',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'S',
##&nbsp;            'command' => 'bash --norc --noprofile -i'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '43085',
##&nbsp;            'time' => '0:00.00',
##&nbsp;            'tt' => '1',
##&nbsp;            'stat' => 'R+',
##&nbsp;            'command' => 'ps -N 2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ps {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = -1;
    my ( @result_array, @values );
    my @head = split( /\s+/, $lines[0] );

    for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
        if($lines[$i] =~ /.*\d+\S+.*\d+\s+(.*)/ ) {
            $count++;
            $result_array[$count]->{to_key_name( item =>$head[$#head])} = $1;
        }
        @values = split( /\s+/, $lines[$i] );
        for ( my $j = 0 ; $j <= $#head - 1 ; $j++ ) {
            $head[$j] =~ s/%//;
            $result_array[$count]->{ to_key_name( item =>$head[$j]) } = $values[$j];
        }
    }
    return \@result_array;
} ## end sub ps

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/No\s+group\s+name|No\s+such\s+file\s+or\s+directory|Invalid\s+process\s+id|No\s+user\s+named|.*not\s+found.*|no\s+valid\s+keywords/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
