#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd pwd command
##
## @description
##    This is a library to parse the raw output of Freebsd pwd command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "pwd" will be pwd()
##
## @synopsis
##    use pwd;
##
##    pwd(%args);
##
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::pwd;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::pwd
##
## @summary Parses the raw output from "pwd" command
##
## @description
##    This method can be used for parsing the output from "pwd" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Freebsd/default/pwd.cdefs
##
## @synopsis
##    pwd(%args);
##
## @arg %args Required: Hash containing the response object
## response_object => used to get the raw output, actual command line invoked,
##                    execution status, to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object. 
##
## @return Returns an array reference which contains the parsed output data 
## from the output of pwd command

sub pwd {
    my %args = @_;
    
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command = $response->get_actual_command();
    my $error_level = $response->get_execution_status();
    my $result;

    my @lines = get_lines("raw_output"=>$raw_output,
        "first_line"=>"", "can_throw"=>1, "response_object"=>$response);

    my %hash;
    # To check if there is any error
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException(
            text=>"Command failed - Check command line and dependencies",
            response_object => $response);
    }
    # parse the output
    if (scalar(@lines) > 2) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text=>"Output is not in the expected format",
            response_object=>$response);
    } elsif (scalar (@lines) == 2) {    
        $hash{"warning"} = shift(@lines);
    }
    $hash{"current_directory"} = shift(@lines);
    $result = [\%hash];
    return $result;  
}
1;
