#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse route command output
##
## @description
##    This library supports error checking for route command
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::route;
##
##  route(%args);
##
## @author praveent@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::route;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::route
##
## @summary Parses the raw output from the "sudo route" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route(%args);
## INPUTS TO METHOD CALL 'route' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'supress' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'names' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'args' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'override' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'modifiers' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sudo route  -D get 10.72.219.45
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;   route to: 10.72.219.45
##&nbsp;destination: default
##&nbsp;       mask: default
##&nbsp;    gateway: 10.238.0.1
##&nbsp;  interface: default-ipspace
##&nbsp;      flags: <UP,GATEWAY,DONE,STATIC>
##&nbsp; recvpipe  sendpipe  ssthresh  rtt,msec    mtu        weight    expire
##&nbsp;       0         0         0         0      1500         1         0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ssthresh' => '0',
##&nbsp;            'flags' => [
##&nbsp;                         'UP',
##&nbsp;                         'GATEWAY',
##&nbsp;                         'DONE',
##&nbsp;                         'STATIC'
##&nbsp;                       ],
##&nbsp;            'gateway' => '10.238.0.1',
##&nbsp;            'destination' => 'default',
##&nbsp;            'recvpipe' => '0',
##&nbsp;            'sendpipe' => '0',
##&nbsp;            'weight' => '1',
##&nbsp;            'rtt_msec' => '0',
##&nbsp;            'route_to' => '10.72.219.45',
##&nbsp;            'interface' => 'default-ipspace',
##&nbsp;            'expire' => '0',
##&nbsp;            'mtu' => '1500',
##&nbsp;            'mask' => 'default'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub route {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @values, @head );

    for ( my $j = 0 ; $j <= $#lines ; ) {
        @values = split( /:\s+/, $lines[$j] );
        if ( $lines[$j] =~ /flags/ ) {
            my @array = split( /,/, $values[1] );
            $array[0] =~ s/<//;
            $array[$#array] =~ s/>//;
            $result_array[0]->{ $values[0] } = \@array;
        } elsif ( $lines[$j] =~ /:/ ) {
            $result_array[0]->{ to_key_name( item => $values[0] ) } =
              $values[1];
        } else {
            @head   = split( /\s+/, $lines[$j] );
            @values = split( /\s+/, $lines[ ++$j ] );
            for ( my $k = 0 ; $k <= $#head ; $k++ ) {
                $head[$k] =~ s/,/_/;
                $result_array[0]->{ to_key_name( item => $head[$k] ) } =
                  $values[$k];
            }
        }
        $j++;
    }
    return \@result_array;
} ## end sub route

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;

    if ( $raw_output =~
/illegal\s+.*|[Uu]sage:.*|Network\s+is\s+unreachable|must\s+be\s+root\s+to\s+alter\s+routing\s+table|Bad\s+file\s+descriptor/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    } elsif ( $raw_output =~
        /No\s+match.*|bad\s+keyword.*|Invalid\s+argument|No\s+such\s+process/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Modifying\s+networking\s+stack\s+.*/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
	if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
