#
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse SystemShell spd commands
##
## @description
##     This is a library to parse the raw output of SystemShell spd commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use spd;
##
##     spd(%args);
##
## @author sheethal@netapp.com , dl-nacl-dev
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::spd;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args            = @_;
    my $raw_output      = $args{raw_output};
    my $command         = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ($raw_output =~ /([Uu]sage: spd.*)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    } elsif ($raw_output =~ /Permission denied/) {
        $error_message = $1;
        $error         = "CommandFailed";
    } elsif ($raw_output =~ /No such file or directory/) {
        $error_message = $1;
        $error         = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }

} ## end sub _check_error

## @name ::spd
##
## @summary Parses the raw output from the "spd" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from spd with any of its options as described in the 
##    corresponding cdef file 'spd.cdefs'
## @synopsis
##      spd(%args);
## INPUTS TO METHOD CALL 'spd' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'bytes' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'number' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'data' </td> <td> '0' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'read_count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'run' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'temp' </td> <td> '0' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'kernel' </td> <td> '1' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'write_count' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'write' </td> <td> '0' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'offset' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = spd -n 1 -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SPD CRC(0x738e) is OK.
##&nbsp;Sig: <MES> Version: 0x1 
##&nbsp;Flag 0xc Id 0x1 Csum: 0x4e18
##&nbsp;Log Cnt:0x1
##&nbsp;UECC cnt:0x1
##&nbsp; UECC0 addr:0xe3b81eef Time: Sat Dec 15 15:12:29 2012
##&nbsp;
##&nbsp; UECC1 addr:0x0 
##&nbsp;CECC cnt:0x0
##&nbsp; CECC0 addr:0x0 bit:0x0
##&nbsp; CECC1 addr:0x0 bit:0x0
##&nbsp;Serial:BA001212000A4EB3
##&nbsp;
##&nbsp;CECC histogram:
##&nbsp; Daily cnt: 0x0
##&nbsp; Weekly cnt: 0x0
##&nbsp; Jan:0x0 Feb:0x0 Mar:0x0 April:0x0
##&nbsp; May:0x0 Jun:0x0 July:0x0 Aug:0x0
##&nbsp; Sept:0x0 Oct:0x0 Nov:0x0 Dec:0x0
##&nbsp;-----------------------------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cecc0' => {
##&nbsp;                         'bit' => '0x0',
##&nbsp;                         'addr' => '0x0'
##&nbsp;                       },
##&nbsp;            'cecc_cnt' => '0x0',
##&nbsp;            'cecc1' => {
##&nbsp;                         'bit' => '0x0',
##&nbsp;                         'addr' => '0x0'
##&nbsp;                       },
##&nbsp;            'serial' => 'BA001212000A4EB3',
##&nbsp;            'version' => '0x1',
##&nbsp;            'status' => 'OK',
##&nbsp;            'flag' => '0xc ',
##&nbsp;            'sig' => '<MES>',
##&nbsp;            'csum' => '0x4e18',
##&nbsp;            'uecc1' => {
##&nbsp;                         'addr' => '0x0'
##&nbsp;                       },
##&nbsp;            'cecc_histogram' => {
##&nbsp;                                  'feb' => '0x0',
##&nbsp;                                  'sept' => '0x0',
##&nbsp;                                  'may' => '0x0',
##&nbsp;                                  'mar' => '0x0',
##&nbsp;                                  'dec' => '0x0',
##&nbsp;                                  'daily_cnt' => '0x0',
##&nbsp;                                  'july' => '0x0',
##&nbsp;                                  'jan' => '0x0',
##&nbsp;                                  'aug' => '0x0',
##&nbsp;                                  'april' => '0x0',
##&nbsp;                                  'jun' => '0x0',
##&nbsp;                                  'weekly_cnt' => '0x0',
##&nbsp;                                  'nov' => '0x0',
##&nbsp;                                  'oct' => '0x0'
##&nbsp;                                },
##&nbsp;            'uecc_cnt' => '0x1',
##&nbsp;            'id' => '0x1',
##&nbsp;            'log_cnt' => '0x1',
##&nbsp;            'uecc0' => {
##&nbsp;                         'time' => 'Sat Dec 15 15:12:29 2012',
##&nbsp;                         'addr' => '0xe3b81eef'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub spd {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );

    # Now we can parse the output
    my @array
        = get_lines(raw_output => $raw_output, response_object => $resp);
    my @output;
    my $i = 0;
    my @heads;
    my @values;
    if (@array) {
        for (my $j = 0; $j <= $#array; $j++) {
            if ($array[$j] =~ /SPD CRC/) {
                @heads = split(/\s*is\s*/, $array[$j]);
                $heads[1] =~ s/\.//;
                $output[$i]->{status} = $heads[1];
            }
            if ($array[$j] =~ /Sig:/) {
                my @heads1 = split(/\s+/, $array[$j]);
                $output[$i]->{to_key_name('item' => $heads1[0])} = $heads1[1];
                $output[$i]->{to_key_name('item' => $heads1[2])} = $heads1[3];
            }
            if ($array[$j] =~ /(Flag)\s+(.*)(Id)\s+(.*)\s+(Csum:)\s+(.*)/) {
                $output[$i]->{to_key_name('item' => $1)} = $2;
                $output[$i]->{to_key_name('item' => $3)} = $4;
                $output[$i]->{to_key_name('item' => $5)} = $6;
            }
            if ($array[$j] =~ /Log\s+.*/) {
                my @heads2 = split(/:/, $array[$j]);
                $output[$i]->{to_key_name('item' => $heads2[0])} = $heads2[1];
            }
            if ($array[$j] =~ /UECC\s+.*/) {
                my @heads2 = split(/:/, $array[$j]);
                $output[$i]->{to_key_name('item' => $heads2[0])} = $heads2[1];
                $j++;
                until ($array[$j] =~ /CECC/) {
                    if ($array[$j] =~ /(UECC.*)\s+(addr):(.*)\s+(Time:)\s+(.*)/)
                    {
                        $output[$i]->{to_key_name('item' => $1)}->{addr} = $3;
                        $output[$i]->{to_key_name('item' => $1)}->{time} = $5;
                    } else {
                        my @heads3 = split(/:/,   $array[$j]);
                        my @val1   = split(/\s+/, $heads3[0]);
                        $output[$i]->{to_key_name('item' => $val1[0])}->{addr}
                            = $heads3[1];
                    }
                    $j++;
                }
            }
            if (   $array[$j] =~ /CECC\s+.*/
                && $array[$j] !~ /CECC\s+histogram.*/) {
                my @heads2 = split(/:/, $array[$j]);
                $output[$i]->{to_key_name('item' => $heads2[0])} = $heads2[1];
                $j++;
                until ($array[$j] =~ /Serial/) {
                    if ($array[$j] =~ /(CECC.*)\s+(addr):(.*)\s+(bit:)(.*)/) {
                        $output[$i]->{to_key_name('item' => $1)}->{addr} = $3;
                        $output[$i]->{to_key_name('item' => $1)}->{bit}  = $5;
                    } else {
                        my @heads3 = split(/:/, $array[$j]);
                        $output[$i]->{to_key_name('item' => $heads3[0])}
                            ->{addr} = $heads3[1];
                    }
                    $j++;
                }
            }
            if ($array[$j] =~ /CECC\s+histogram.*/) {
                my $histogram = to_key_name('item' => $array[$j]);
				$j++;
				     while ($j != $#array) {
                    if (   $array[$j] =~ /Daily cnt:/
                        || $array[$j] =~ /Weekly cnt:/) {
                        my @heads6 = split(/:\s+/, $array[$j]);
                        $output[$i]->{$histogram}->{to_key_name('item' => $heads6[0])}
                            = $heads6[1];
                    } elsif ($array[$j]
                        =~ /(.*):(.*)\s+(.*):(.*)\s+(.*):(.*)\s+(.*):(.*)/) {
                        $output[$i]->{$histogram}->{to_key_name('item' => $1)} = $2;
                        $output[$i]->{$histogram}->{to_key_name('item' => $3)} = $4;
                        $output[$i]->{$histogram}->{to_key_name('item' => $5)} = $6;
                        $output[$i]->{$histogram}->{to_key_name('item' => $7)} = $8;
                    }
                    $j++;
                }
            }
            if ($array[$j] =~ /Serial/) {
                my @heads5 = split(/:/, $array[$j]);
                $output[$i]->{to_key_name('item' => $heads5[0])} = $heads5[1];
            }
        }
    }
    return \@output;

} ## end sub spd
1;
