#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd spmctl command
##
## @description
##    This is a library to parse the raw output of Freebsd spmctl command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "spmctl" will be spmctl()
##
## @synopsis
##    use spmctl;
##
##    spmctl(%args);
##
## @author bhaskarm@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::spmctl;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException ;
use NACL::APISet::Exceptions::InvalidParamValueException ;
use NACL::APISet::Exceptions::MissingRequiredParamException ;

## @name ::spmctl
##
## @summary Parses the raw output from the "spmctl" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from spmctl with any of its options as described in the 
##    corresponding cdef file 'spmctl.cdefs'
## @synopsis
##      spmctl(%args);
## INPUTS TO METHOD CALL 'spmctl' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'add' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'stop' </td> </tr> 
## <tr> <td> '-e' </td> <td> 'env' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pid_t' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'handle' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'delete' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'idempotent' </td> </tr> 
## <tr> <td> '-e' </td> <td> 'exec' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'command' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'list' </td> </tr> 
## <tr> <td> '-H' </td> <td> 'host' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kill' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = spmctl -l
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'Pid' => '992',
##&nbsp;            'Exec' => '/usr/sbin/ucoreman --log',
##&nbsp;            'State' => 'Running',
##&nbsp;            'Handle' => '069264cf-377e-4f06-84c2-05fc01b4ee5f'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Pid' => '1837',
##&nbsp;            'Exec' => '/sbin/nchmd',
##&nbsp;            'State' => 'Running',
##&nbsp;            'Handle' => '3909a2b9-37c7-4009-bf7b-914892d4af8d'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = spmctl -h mgwd -s
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = spmctl -h mgwd -e
##&nbsp;$VAR1 = [];
##&nbsp;

sub spmctl {
    my %args = @_;
    
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command = $response->get_actual_command();
    _check_error( raw_output => $raw_output, response_object => $response );
    my $error_level = $response->get_execution_status();
    my @result_array;

    # To check if there is any error
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed\n$raw_output",
            response_object => $response);
    }
    my $first_line = '(?i)list\s+of\s+managed\s+processes';
    my @lines = get_lines("raw_output"=>$raw_output,
        "first_line"=>"$first_line", "response_object"=>$response);
    shift @lines;
    foreach my $line (@lines) {
        my @info_lines = split(/;/,$line);
        my %hash;
        foreach my $info_line (@info_lines) {
            if ($info_line =~ /(.+)=(.+)/){
                $hash{$1}=$2;
            }
        }
        push @result_array,\%hash if (scalar keys %hash);
    }
    return \@result_array;
}

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $response    = $args{response_object};
    my $error_level = $response->get_execution_status();
    my ( $error, $error_message ) = undef;

    if ( $raw_output =~
/option\s+requires\s+an\s+argument|Operation\s+requires.|No\s+command\s+line\s+supplied\s+for\s+.*/
      )
    {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Handle.*not\s+a\s+valid.*|Unknown\s+host/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /invalid\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $response
        );
    }

} ## end sub _check_error

1;
