#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse stat command output
##
## @description
##   This library supports error checking and parsing of stat command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::stat;
##
##  stat(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Freebsd::default::stat;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::stat
##
## @summary Parses the raw output from the "stat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from stat with any of its options as described in the 
##    corresponding cdef file 'stat.cdefs'
## @synopsis
##      stat(%args);
## INPUTS TO METHOD CALL 'stat' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-x' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'one-per-line' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'shell-output-format' </td> <td> '1' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'raw-output' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'dereference' </td> <td> '1' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'format' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'timefmt' </td> <td> '1' </td> </tr> 
## <tr> <td> '-F' </td> <td> 'classify' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = stat -x
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  File: "/dev/pts/1"
##&nbsp;  Size: 0            FileType: Character Device
##&nbsp;  Mode: (0600/crw-------)         Uid: ( 1002/    diag)  Gid: (    4/     tty)
##&nbsp;Device: 255,67108864   Inode: 120    Links: 1
##&nbsp;Access: Tue Jun 10 02:04:19 2014
##&nbsp;Modify: Tue Jun 10 02:04:19 2014
##&nbsp;Change: Tue Jun 10 02:04:19 2014
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'change' => 'Tue Jun 10 02:04:19 2014',
##&nbsp;            'FileType' => 'Character Device',
##&nbsp;            'access' => 'Tue Jun 10 02:04:19 2014',
##&nbsp;            'Inode' => '120',
##&nbsp;            'Links' => '1',
##&nbsp;            'Device' => '255,67108864',
##&nbsp;            'Size' => '0',
##&nbsp;            'modify' => 'Tue Jun 10 02:04:19 2014',
##&nbsp;            'File' => '"/dev/pts/1"',
##&nbsp;            'Mode' => '(0600/crw-------)',
##&nbsp;            'Gid' => '(    4/     tty)',
##&nbsp;            'Uid' => '( 1002/    diag)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;


sub stat {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    _check_error(raw_output => $output, response_object => $resp);

    # Now we can parse the output
    my @array = get_lines(raw_output => $output, response_object => $resp);

    my @result_array;

    if ($command =~ /stat -s/) {
        my $line = join(' ', @array);
        my @tmp = split /\s+/, $line;
        foreach (@tmp) {
            if ($_ =~ /(\S+)=(\S+)/) {
                $result_array[0]->{to_key_name(item => $1)} = $2;
            }
        }
        return \@result_array;
    }

    if ($command =~ /stat -F|stat -l/) {
        foreach (@array) {
            if ($_
                =~ /(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\w+\s+\d+\s+\d+:\d+:\d+\s+\d+)\s+(\S+)/
                ) {
                $result_array[0]->{'mode'}  = $1;
                $result_array[0]->{'links'} = $2;
                $result_array[0]->{'uid'}   = $3;
                $result_array[0]->{'gid'}   = $4;
                $result_array[0]->{'size'}  = $5;
                $result_array[0]->{'date'}  = $6;
                $result_array[0]->{'file'}  = $7;
            }
        }
        return \@result_array;
    }

    foreach my $line (@array) {
        if ($line =~ /^File:\s+(\S+)/) {
            $result_array[0]->{'File'} = $1;
        } elsif ($line =~ /^Size:\s+(\d+)\s+FileType:\s+(.*)/) {
            $result_array[0]->{'Size'}     = $1;
            $result_array[0]->{'FileType'} = $2;
        } elsif ($line =~ /^Mode:\s+(.*)\s+Uid:\s+(.*)\s+Gid:\s+(.*)/) {
            $result_array[0]->{'Mode'} = trim_string( 'string' => $1 );
            $result_array[0]->{'Uid'}  = trim_string( 'string' => $2 );
            $result_array[0]->{'Gid'}  = $3;
        } elsif ($line =~ /^Device:\s+(.*)\s+Inode:\s+(\d+)\s+Links:\s+(\d+)/)
        {
            $result_array[0]->{'Device'} = trim_string( 'string' => $1);
            $result_array[0]->{'Inode'}  = $2;
            $result_array[0]->{'Links'}  = $3;
        } elsif ($line =~ /^(\w+):\s+(\w+\s+\w+\s+\d+\s+\d+:\d+:\d+\s+\d{4})/)
        {
            $result_array[0]->{to_key_name(item => $1)} = $2;
        }
    }
    return \@result_array;
} ## end sub stat

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message, $error) = undef;

    if ($raw_output =~ /No such file or directory.*/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ($raw_output =~ /([Uu]sage:|stat:)\s+.*/) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
} ## end sub _check_error

1;
