#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd svstat command"
##
## @description
##    This is a library to parse the raw output of Freebsd svstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "svstat" will be svstat()
##
## @synosvstatis
##    use svstat;
##
##    svstat(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::svstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::svstat
##
## @summary Parses the raw output from the "svstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from svstat with any of its options as described in the
##    corresponding cdef file 'svstat.cdefs'
## @synopsis
##      svstat(%args);
## INPUTS TO METHOD CALL 'svstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-e' </td> <td> 'extended-rpc' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'core' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'display-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'display-histograms' </td> <td> '0' </td> </tr>
## <tr> <td> '-O' </td> <td> 'display-timings' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'display-spinvfs' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'widemode' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rpc-stats' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = svstat  -O
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;             OP   count TotalAvg  Pretime   OPtime  RPCtime   CbTime RestartTm  RelWait %RlWt   Max us   Min us
##&nbsp;         Access   12378      205        0       13      192        1       12        0  00.0%    19999     9916
##&nbsp;         Create      77      649        0        0      649      129        0        0  00.0%    10027     9986
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'optime' => '13',
##&nbsp;            'count' => '12378',
##&nbsp;            'relwait' => '0',
##&nbsp;            'cbtime' => '1',
##&nbsp;            'restarttm' => '12',
##&nbsp;            'min_us' => '9916',
##&nbsp;            'max_us' => '19999',
##&nbsp;            'pretime' => '0',
##&nbsp;            'rlwt' => '00.0%',
##&nbsp;            'op' => 'Access',
##&nbsp;            'totalavg' => '205',
##&nbsp;            'rpctime' => '192'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'optime' => '0',
##&nbsp;            'count' => '77',
##&nbsp;            'relwait' => '0',
##&nbsp;            'cbtime' => '129',
##&nbsp;            'restarttm' => '0',
##&nbsp;            'min_us' => '9986',
##&nbsp;            'max_us' => '10027',
##&nbsp;            'pretime' => '0',
##&nbsp;            'rlwt' => '00.0%',
##&nbsp;            'op' => 'Create',
##&nbsp;            'totalavg' => '649',
##&nbsp;            'rpctime' => '649'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = svstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Client Info:
##&nbsp;Rpc Counts:
##&nbsp;  Getattr   Setattr    Lookup  Readlink      Read     Write    Create    Remove
##&nbsp;   193002      1236     20220         2     64984     76191       111         0
##&nbsp;   Rename      Link  Readlink   Readdir    Access     Fsync    fstats    FsInfo
##&nbsp;      119         4         2         3     16245     35657       205         0
##&nbsp;   Pagein   Pageout
##&nbsp;        0         0
##&nbsp;Rpc Info:
##&nbsp; TimedOut   Invalid  Connects Reconnect   Retries  Requests
##&nbsp;        0         0         0         0         8    372236
##&nbsp;Cache Info:
##&nbsp;Attr Hits    Misses Lkup Hits    Misses BioR Hits    Misses BioW Hits    Misses
##&nbsp;  1504777    192797   1101400     20216    212040     64984        30     76191
##&nbsp;BioRLHits    Misses BioD Hits    Misses DirE Hits    Misses
##&nbsp;       10         2      1023         3       205         0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'rpc_info' => [
##&nbsp;                            {
##&nbsp;                              'retries' => '8',
##&nbsp;                              'timedout' => '0',
##&nbsp;                              'requests' => '372236',
##&nbsp;                              'connects' => '0',
##&nbsp;                              'reconnect' => '0',
##&nbsp;                              'invalid' => '0'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'rpc_counts' => [
##&nbsp;                              {
##&nbsp;                                'link' => '4',
##&nbsp;                                'create' => '111',
##&nbsp;                                'remove' => '0',
##&nbsp;                                'getattr' => '193002',
##&nbsp;                                'fstats' => '205',
##&nbsp;                                'access' => '16245',
##&nbsp;                                'setattr' => '1236',
##&nbsp;                                'readlink' => '2',
##&nbsp;                                'pagein' => '0',
##&nbsp;                                'readdir' => '3',
##&nbsp;                                'rename' => '119',
##&nbsp;                                'lookup' => '20220',
##&nbsp;                                'fsync' => '35657',
##&nbsp;                                'read' => '64984',
##&nbsp;                                'fsinfo' => '0',
##&nbsp;                                'pageout' => '0',
##&nbsp;                                'write' => '76191'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'cache_info' => [
##&nbsp;                              {
##&nbsp;                                'biod_misses' => '3',
##&nbsp;                                'bior_hits' => '212040',
##&nbsp;                                'lkup_hits' => '1101400',
##&nbsp;                                'dire_hits' => '205',
##&nbsp;                                'lkup_misses' => '20216',
##&nbsp;                                'biod_hits' => '1023',
##&nbsp;                                'biow_misses' => '76191',
##&nbsp;                                'biorlhits' => '10',
##&nbsp;                                'bior_misses' => '64984',
##&nbsp;                                'attr_misses' => '192797',
##&nbsp;                                'biorlmisses' => '2',
##&nbsp;                                'dire_misses' => '0',
##&nbsp;                                'attr_hits' => '1504777',
##&nbsp;                                'biow_hits' => '30'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = svstat  -e
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Client Info:
##&nbsp;Rpc Counts:
##&nbsp;  Getattr   Setattr    Lookup  Readlink      Read     Write    Create    Remove
##&nbsp;   193600      1237     20241         2     65071     76286       112         0
##&nbsp;   Rename      Link  Readlink   Readdir    Access     Fsync    fstats    FsInfo
##&nbsp;      119         4         2         3     16252     35686       206         0
##&nbsp;   Pagein   Pageout
##&nbsp;        0         0
##&nbsp;Rpc Info:
##&nbsp; TimedOut   Invalid  Connects Reconnect   Retries  Requests
##&nbsp;        0         0         0         0         8    373047
##&nbsp;Cache Info:
##&nbsp;Attr Hits    Misses Lkup Hits    Misses BioR Hits    Misses BioW Hits    Misses
##&nbsp;  1510440    193394   1106363     20237    211975     65071        30     76286
##&nbsp;BioRLHits    Misses BioD Hits    Misses DirE Hits    Misses
##&nbsp;       10         2      1029         3       206         0
##&nbsp;SpinOps max times (usec):
##&nbsp;  NullFlOp    Access     Close    Create  Dis_Cred    Pagein   Pageout
##&nbsp;         0     19999         0     10027         0         0         0
##&nbsp; Fold_File   Getattr Gatr_Bulk  Get_Cred Get_Parnt  Get_Rtfh      Link
##&nbsp;         0    250036         0         0         0         0         0
##&nbsp;      Lock  Lk_Grntd    Lookup  Mv_Locks      Open  Prefetch      Read
##&nbsp;         0         0     20275         0         0         0    249972
##&nbsp;   Readdir  Readlink Recl_Lcks    Remove    Rename Scn_M_Lks Scan_File
##&nbsp;      9982         0         0         0     10004         0         0
##&nbsp;   Setattr  Set_Cred     Share    Unlink   Unshare     Fsync     Write
##&nbsp;     40001         0         0     19999         0    259989    259987
##&nbsp; NrvGtAttr  FhToPth2  ReadLnk2   Create2 ChgFilTyp  RealRead RealWrite
##&nbsp;         0         0         0         0         0         0         0
##&nbsp; RplyRlBin     Audit GetVolLng GetFsAtrs    Cancel Get_Quota Free_Blks
##&nbsp;         0         0         0         0         0         0         0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'spinops_max_times' => [
##&nbsp;                                     {
##&nbsp;                                       'scn_m_lks' => '0',
##&nbsp;                                       'remove' => '0',
##&nbsp;                                       'getattr' => '250036',
##&nbsp;                                       'gatr_bulk' => '0',
##&nbsp;                                       'lock' => '0',
##&nbsp;                                       'create2' => '0',
##&nbsp;                                       'unlink' => '19999',
##&nbsp;                                       'rename' => '10004',
##&nbsp;                                       'recl_lcks' => '0',
##&nbsp;                                       'readdir' => '9982',
##&nbsp;                                       'fsync' => '259989',
##&nbsp;                                       'read' => '249972',
##&nbsp;                                       'get_quota' => '0',
##&nbsp;                                       'write' => '259987',
##&nbsp;                                       'get_cred' => '0',
##&nbsp;                                       'readlink' => '0',
##&nbsp;                                       'pagein' => '0',
##&nbsp;                                       'free_blks' => '0',
##&nbsp;                                       'audit' => '0',
##&nbsp;                                       'open' => '0',
##&nbsp;                                       'scan_file' => '0',
##&nbsp;                                       'rplyrlbin' => '0',
##&nbsp;                                       'fhtopth2' => '0',
##&nbsp;                                       'cancel' => '0',
##&nbsp;                                       'nullflop' => '0',
##&nbsp;                                       'unshare' => '0',
##&nbsp;                                       'getfsatrs' => '0',
##&nbsp;                                       'setattr' => '40001',
##&nbsp;                                       'lk_grntd' => '0',
##&nbsp;                                       'dis_cred' => '0',
##&nbsp;                                       'close' => '0',
##&nbsp;                                       'share' => '0',
##&nbsp;                                       'nrvgtattr' => '0',
##&nbsp;                                       'pageout' => '0',
##&nbsp;                                       'realwrite' => '0',
##&nbsp;                                       'link' => '0',
##&nbsp;                                       'create' => '10027',
##&nbsp;                                       'get_rtfh' => '0',
##&nbsp;                                       'access' => '19999',
##&nbsp;                                       'realread' => '0',
##&nbsp;                                       'chgfiltyp' => '0',
##&nbsp;                                       'prefetch' => '0',
##&nbsp;                                       'getvollng' => '0',
##&nbsp;                                       'lookup' => '20275',
##&nbsp;                                       'fold_file' => '0',
##&nbsp;                                       'mv_locks' => '0',
##&nbsp;                                       'set_cred' => '0',
##&nbsp;                                       'get_parnt' => '0',
##&nbsp;                                       'readlnk2' => '0'
##&nbsp;                                     }
##&nbsp;                                   ],
##&nbsp;            'rpc_info' => [
##&nbsp;                            {
##&nbsp;                              'retries' => '8',
##&nbsp;                              'timedout' => '0',
##&nbsp;                              'requests' => '373047',
##&nbsp;                              'connects' => '0',
##&nbsp;                              'reconnect' => '0',
##&nbsp;                              'invalid' => '0'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'rpc_counts' => [
##&nbsp;                              {
##&nbsp;                                'link' => '4',
##&nbsp;                                'create' => '112',
##&nbsp;                                'remove' => '0',
##&nbsp;                                'getattr' => '193600',
##&nbsp;                                'fstats' => '206',
##&nbsp;                                'access' => '16252',
##&nbsp;                                'setattr' => '1237',
##&nbsp;                                'readlink' => '2',
##&nbsp;                                'pagein' => '0',
##&nbsp;                                'readdir' => '3',
##&nbsp;                                'rename' => '119',
##&nbsp;                                'lookup' => '20241',
##&nbsp;                                'fsync' => '35686',
##&nbsp;                                'read' => '65071',
##&nbsp;                                'fsinfo' => '0',
##&nbsp;                                'pageout' => '0',
##&nbsp;                                'write' => '76286'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'cache_info' => [
##&nbsp;                              {
##&nbsp;                                'biod_misses' => '3',
##&nbsp;                                'bior_hits' => '211975',
##&nbsp;                                'lkup_hits' => '1106363',
##&nbsp;                                'dire_hits' => '206',
##&nbsp;                                'lkup_misses' => '20237',
##&nbsp;                                'biod_hits' => '1029',
##&nbsp;                                'biow_misses' => '76286',
##&nbsp;                                'biorlhits' => '10',
##&nbsp;                                'bior_misses' => '65071',
##&nbsp;                                'attr_misses' => '193394',
##&nbsp;                                'biorlmisses' => '2',
##&nbsp;                                'dire_misses' => '0',
##&nbsp;                                'attr_hits' => '1510440',
##&nbsp;                                'biow_hits' => '30'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = svstat  -h
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Client Info:
##&nbsp;Normal Priority Rpc Counts:
##&nbsp;  Getattr   Setattr    Lookup  Readlink      Read     Write    Create    Remove
##&nbsp;   193785      1241     20215         2     65292     76268       108         0
##&nbsp;   Rename      Link  Readlink   Readdir    Access     Fsync    fstats    FsInfo
##&nbsp;      111         0         2         3     16286     35529       206         0
##&nbsp;High Priority Rpc Counts:
##&nbsp;  Getattr   Setattr    Lookup  Readlink      Read     Write    Create    Remove
##&nbsp;       74         0        68         0         3       288         4         0
##&nbsp;   Rename      Link  Readlink   Readdir    Access     Fsync    Pagein   Pageout
##&nbsp;        8         4         0         0        40       297         0         0
##&nbsp;Rpc Info:
##&nbsp; TimedOut   Invalid  Connects Reconnect   Retries  Requests
##&nbsp;        0         0         0         0         8    373920
##&nbsp;Cache Info:
##&nbsp;Attr Hits    Misses Lkup Hits    Misses BioR Hits    Misses BioW Hits    Misses
##&nbsp;  1514304    193653   1107542     20279    214208     65295        30     76556
##&nbsp;BioRLHits    Misses BioD Hits    Misses DirE Hits    Misses
##&nbsp;       10         2      1029         3       206         0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'rpc_info' => [
##&nbsp;                            {
##&nbsp;                              'retries' => '8',
##&nbsp;                              'timedout' => '0',
##&nbsp;                              'requests' => '373920',
##&nbsp;                              'connects' => '0',
##&nbsp;                              'reconnect' => '0',
##&nbsp;                              'invalid' => '0'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'normal_priority_rpc_counts' => [
##&nbsp;                                              {
##&nbsp;                                                'link' => '0',
##&nbsp;                                                'create' => '108',
##&nbsp;                                                'remove' => '0',
##&nbsp;                                                'getattr' => '193785',
##&nbsp;                                                'fstats' => '206',
##&nbsp;                                                'access' => '16286',
##&nbsp;                                                'setattr' => '1241',
##&nbsp;                                                'readlink' => '2',
##&nbsp;                                                'readdir' => '3',
##&nbsp;                                                'rename' => '111',
##&nbsp;                                                'lookup' => '20215',
##&nbsp;                                                'fsync' => '35529',
##&nbsp;                                                'read' => '65292',
##&nbsp;                                                'fsinfo' => '0',
##&nbsp;                                                'write' => '76268'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'cache_info' => [
##&nbsp;                              {
##&nbsp;                                'biod_misses' => '3',
##&nbsp;                                'bior_hits' => '214208',
##&nbsp;                                'lkup_hits' => '1107542',
##&nbsp;                                'dire_hits' => '206',
##&nbsp;                                'lkup_misses' => '20279',
##&nbsp;                                'biod_hits' => '1029',
##&nbsp;                                'biow_misses' => '76556',
##&nbsp;                                'biorlhits' => '10',
##&nbsp;                                'bior_misses' => '65295',
##&nbsp;                                'attr_misses' => '193653',
##&nbsp;                                'biorlmisses' => '2',
##&nbsp;                                'dire_misses' => '0',
##&nbsp;                                'attr_hits' => '1514304',
##&nbsp;                                'biow_hits' => '30'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'high_priority_rpc_counts' => [
##&nbsp;                                            {
##&nbsp;                                              'link' => '4',
##&nbsp;                                              'create' => '4',
##&nbsp;                                              'remove' => '0',
##&nbsp;                                              'getattr' => '74',
##&nbsp;                                              'access' => '40',
##&nbsp;                                              'setattr' => '0',
##&nbsp;                                              'readlink' => '0',
##&nbsp;                                              'pagein' => '0',
##&nbsp;                                              'readdir' => '0',
##&nbsp;                                              'rename' => '8',
##&nbsp;                                              'lookup' => '68',
##&nbsp;                                              'fsync' => '297',
##&nbsp;                                              'read' => '3',
##&nbsp;                                              'pageout' => '0',
##&nbsp;                                              'write' => '288'
##&nbsp;                                            }
##&nbsp;                                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = svstat -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Paging Sizes:
##&nbsp;     4K:          0   8K:          0  12K:          0  16K:          0
##&nbsp;    20K:          0  24K:          0  28K:          0  32K:          0
##&nbsp;    36K:          0  40K:          0  44K:          0  48K:          0
##&nbsp;    52K:          0  56K:          0  60K:          0  64K:          0
##&nbsp;  > 64K: 0
##&nbsp;
##&nbsp;     OP/ms         < 10  < 20  < 50 < 100 < 150 < 200 < 250 < 300 < 350 < 400 < 450 < 500 < 1 s < 1.5   < 2   < 3   < 4   < 5   < 6   < 8  < 10  < 15  > 15
##&nbsp;     Pagein           0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0
##&nbsp;    Pageout           0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0     0
##&nbsp;
##&nbsp;  pagein pagout pi_avg po_avg  faults   free   pi/rd  po/wr
##&nbsp;       0      0      0      0 96332621     29M     0%     0%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'paging_sizes' => [
##&nbsp;                                {
##&nbsp;                                  '32k' => '0',
##&nbsp;                                  '24k' => '0',
##&nbsp;                                  '60k' => '0',
##&nbsp;                                  '8k' => '0',
##&nbsp;                                  '16k' => '0',
##&nbsp;                                  '36k' => '0',
##&nbsp;                                  '52k' => '0',
##&nbsp;                                  '4k' => '0',
##&nbsp;                                  '12k' => '0',
##&nbsp;                                  '64k' => '0',
##&nbsp;                                  '48k' => '0',
##&nbsp;                                  '20k' => '0',
##&nbsp;                                  '56k' => '0',
##&nbsp;                                  '>_64k' => '0',
##&nbsp;                                  '28k' => '0',
##&nbsp;                                  '44k' => '0',
##&nbsp;                                  '40k' => '0'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'op_per_ms' => [
##&nbsp;                             {
##&nbsp;                               '<_450' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_1s' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_3' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_10' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_5' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_500' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_200' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_4' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_20' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_1.5' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_100' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_150' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '>_15' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_6' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_350' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_50' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_15' => [
##&nbsp;                                           {
##&nbsp;                                             'page_in' => '0',
##&nbsp;                                             'page_out' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               '<_300' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_2' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_8' => [
##&nbsp;                                          {
##&nbsp;                                            'page_in' => '0',
##&nbsp;                                            'page_out' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                               '<_250' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               '<_400' => [
##&nbsp;                                            {
##&nbsp;                                              'page_in' => '0',
##&nbsp;                                              'page_out' => '0'
##&nbsp;                                            }
##&nbsp;                                          ]
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vmstat_info' => [
##&nbsp;                               {
##&nbsp;                                 'pi_avg' => '0',
##&nbsp;                                 'pi_rd' => '0%',
##&nbsp;                                 'po_avg' => '0',
##&nbsp;                                 'faults' => '96332621',
##&nbsp;                                 'pagein' => '0',
##&nbsp;                                 'pagout' => '0',
##&nbsp;                                 'free' => '29M',
##&nbsp;                                 'po_wr' => '0%'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub svstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( $count, $index ) = ( -1, 0 );
    my ( $sub_head, $main_head, @values, @head, @result_array );

    if ( $cmd =~ /-O/ ) {
        $lines[0] =~ s/Max\s+us/Max_us/;
        $lines[0] =~ s/Min\s+us/Min_us/;
        @head = split( /\s+/, $lines[0] );
        for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
            $count++;
            @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                $head[$j] =~ s/%//;
                $result_array[$count]->{ to_key_name( item => $head[$j] ) } =
                  $values[$j];
            }
        }
    } elsif( $cmd =~ /-v/ ) {
        for ( my $i = 0; $i < @lines ; $i++ ) {
            $lines[$i] =~ s/pi\/rd/pi_rd/;
            $lines[$i] =~ s/po\/wr/po_wr/;
            if( my @sizes = ( $lines[$i] =~ /([<>]?\s?\d+K:\s+\d+)/g ) ) {
                foreach ( @sizes ) {
                    my ($a, $b) = map { s/^[ \t]+|[ \t]+$//g; $_ } split(/:/, $_);
                    $result_array[$index]->{paging_sizes}->[0]->
                      {to_key_name( item => $a) } = $b;
                }
            } elsif ( $lines[$i] =~ /pagein\s+pagout\s+pi_avg\s+po_avg.*/ )
               {
                $index++;
                my @sub_head = split( /\s+/, $lines[$i] );
                my @sub_values = split( /\s+/, $lines[$i+1] );
                for ( my $j=0 ; $j < @sub_head ; $j++ ) {
                    $result_array[$index]->{vmstat_info}->[0]->
                      { to_key_name( item => $sub_head[$j] )} = $sub_values[$j];
                }
            } elsif ( $lines[$i] =~ /^OP\/ms.*/i ) {
                my ( %key_found, $cnt );
                $index++;
                my @main_head = ( $lines[$i] =~ /([<>]?\s+\d+\.?\d*\s?\w?)/g);
                my @pagein_value = ($lines[$i+1] =~ /(\d+)/g)
                  if($lines[$i+1] =~ /Pagein\s+\d+\s+.*/);
                my @pageout_value = ($lines[$i+2] =~ /(\d+)/g)
                  if($lines[$i+2] =~ /Pageout\s+\d+\s+.*/);
                for ( my $k = 0; $k < @main_head ; $k++) {
                   if( $key_found{$main_head[$k]} ){
                       $cnt++;
                   } else {
                       $cnt = 0;
                       $key_found{$main_head[$k]} = 1;
                   }
                   $main_head[$k] =~ s/\s+$//;
                   $main_head[$k] =~ s/(\d+)\s+s/$1s/;
                   $result_array[$index]->{op_per_ms}->[0]->
                     { to_key_name( item => $main_head[$k] ) }->[$cnt]->
                       {page_in} = $pagein_value[$k];
                   $result_array[$index]->{op_per_ms}->[0]->
                     { to_key_name( item => $main_head[$k] ) }->[$cnt]->
                       {page_out} = $pageout_value[$k];
                }
            }
        }
    } else {
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
            $lines[$i] =~ s/Attr\s+Hits\s+Misses/Attr_Hits Attr_Misses/;
            $lines[$i] =~ s/Lkup\s+Hits\s+Misses/Lkup_Hits Lkup_Misses/;
            $lines[$i] =~ s/BioR\s+Hits\s+Misses/BioR_Hits BioR_Misses/;
            $lines[$i] =~ s/BioW\s+Hits\s+Misses/BioW_Hits BioW_Misses/;
            $lines[$i] =~ s/BioRLHits\s+Misses/BioRLHits BioRLMisses/;
            $lines[$i] =~ s/BioD\s+Hits\s+Misses/BioD_Hits BioD_Misses/;
            $lines[$i] =~ s/DirE\s+Hits\s+Misses/DirE_Hits DirE_Misses/;
            if ($lines[$i] =~ /(^Rpc\s+Counts):/
                || $lines[$i] =~ /(^Rpc\s+Info):/
                || $lines[$i] =~ /(^Normal\s+Priority\s+Rpc\s+Counts):/
                || $lines[$i] =~ /(^High\s+Priority\s+Rpc\s+Counts):/
                ||$lines[$i] =~ /(^Cache\s+Info):/
                || $lines[$i] =~ /(^SpinOps\s+max\s+times)/) {
                $sub_head = $1;
                $count=0;
            }
            elsif ( $lines[$i] =~ /\d+\s+\d+.*/ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                     $result_array[0]->{to_key_name( item => $sub_head)}[$count]
                              ->{to_key_name( item => $head[$j])} = $values[$j];
                }
            }
            elsif ( $lines[$i] =~ /\w+\s+\w+.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            }
        }
    }
    return \@result_array;
} ## end sub svstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Permission\s+denied|Operation\s+not\s+permitted/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
        /No\s+such\s+file\s+or\s+directory|.*can't\s+get\s+names/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
