#
# Copyright (c) 2012-13 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse sysctl command output
##
## @description
##    This is a library to parse the raw output of sysctl command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::sysctl;
##
##  sysctl(%args);
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::sysctl;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::sysctl
##
## @summary Parses the raw output from the "sysctl" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sysctl with any of its options as described in the
##    corresponding cdef file 'sysctl.cdefs'
## @synopsis
##      sysctl(%args);
## INPUTS TO METHOD CALL 'sysctl' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'e' </td> <td> 'ignore-errors' </td> <td> '1' </td> </tr>
## <tr> <td> '=' </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> 'q' </td> <td> 'no-stdout' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> 'n' </td> <td> 'only-value' </td> <td> '1' </td> </tr>
## <tr> <td> 'N' </td> <td> 'only-name' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'idescription' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'binary-format' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-format' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'index' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'opaque' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'hex' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sysctl debug.watchdog.disable
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;debug.watchdog.disable: 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'debug.watchdog.disable' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sysctl {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my (@result_array);
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line =~ /(.*)[=:]\s*(.*)/ ) {
            $result_array[0]->{$1} = $2;
        }
    }
    return \@result_array;
} ## end sub sysctl

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;

    if ( $raw_output =~ /Operation\s+not\s+permitted.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /sysctl:\s+unknown\s+oid.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
