#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Freebsd/default/tcp_client.pm#$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode tcp_client command output
##
## @description
##    This is a library to parse the raw output of 7Mode tcp_client command.
##
## @synopsis
##    use tcp_client;
##
##    tcp_client(%args);
##
## @author benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::tcp_client;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /Usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ( $output =~ /(bind failed)|(connect failed)|(recv failed:.*)/i )
    {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ( $output
        =~ /(Ipv6 addres is in invalid form:.*)|(Wrong payload size is entered.*)|(Wrong packet count is entered.*)|(Wrong option is entered.*)|(Enter 4or6 address family.*)/i
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::tcp_client
##
## @summary Parses the raw output from the "tcp_client" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tcp_client with any of its options as described in the
##    corresponding cdef file 'tcp_client.cdefs'
## @synopsis
##      tcp_client(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes_received' => '25',
##&nbsp;            'bytes_sent' => '25'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##
sub tcp_client {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ( $error_message, $error, $firstline, @lines, @result );

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp
    );
    my $i = -1;
    foreach my $line (@lines) {
        if ( $line =~ /\*\*\*(\d+)\s+bytes sent/i ) {
            $i++;
            $result[$i]->{'bytes_sent'} = $1;
        } elsif ( $line =~ /^(\d+)\s+bytes received/ ) {
            $result[$i]->{'bytes_received'} = $1;
            last;
        }
    }
    return \@result;
} ## end sub tcp_client
1;
