#
# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd tcpdump command"
##
## @description
##    This is a library to parse the raw output of tcpdump command.
##    The function name will be the same as command name. 
##    For Example, the parser function for "tcpdump" will be tcpdump()
##
## @synoypis
##    use tcpdump;
##
##    tcpdump(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::tcpdump;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name :: tcpdump
##
## @summary Parses the raw output from the "tcpdump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method tcpdump 
##    cdef file is 'tcpdump.cdefs'
## @synopsis
##      tcpdump(%args);
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sudo tcpdump
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;tcpdump: listening on e0a, link-type EN10MB (Ethernet), capture size 65535 bytes
##&nbsp;5 packets captured
##&nbsp;8 packets received by filter
##&nbsp;0 packets dropped by kernel
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'e0a' => {
##&nbsp;                       'packets_dropped_by_kernel' => '0',
##&nbsp;                       'packets_received_by_filter' => '8',
##&nbsp;                       'packets_captured' => '5'
##&nbsp;                     }
##&nbsp;          }
##&nbsp;        ];

sub tcpdump {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $interface;
    my %hash;
    foreach my $line (@lines) {
        if ( $line =~ /listening\s+on\s+(\w+).+/i ) {
	    $interface = $1;
        } elsif ( $line =~ /^(\d+)\s+(.+)/ ) {
	    $hash{to_key_name(item => $2)} = $1;
	}
    }
    $result_array[0]->{to_key_name(item => $interface)} = \%hash;
    return \@result_array;
} ## end sub tcpdump

sub _check_error {
    my %args        = @_;
    my $raw_output = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ($error_message, $error) = undef;

    if ($raw_output =~ /You don't have permission to capture on that device/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ($raw_output =~ /no suitable device found/) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

