#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd top command"
##
## @description
##    This is a library to parse the raw output of Freebsd top command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "top" will be top()
##
## @synotopis
##    use top;
##
##    top(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::top;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::top
##
## @summary Parses the raw output from the "top" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from top with any of its options as described in the
##    corresponding cdef file 'top.cdefs'
## @synopsis
##      top(%args);
## INPUTS TO METHOD CALL 'top' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'time' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'statistics' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'renice' </td> <td> '1' </td> </tr>
## <tr> <td> '-j' </td> <td> 'jail-id' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'command-names' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'batch-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'active-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interactive-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'non-interactive-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'field' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'iterations' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'usernames' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'top-process' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'threads-toggle' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'cpu-display' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'per-cpu' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'system-processes' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = top  -d 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[H[2Jlast pid: 47169;  load averages:  3.86,  4.16,  4.22[1;9974H up 10+02:19:13[1;9991H11:49:01
##&nbsp;237 processes: 1 running, 236 sleeping
##&nbsp;CPU: [3;6H    % user,     % nice,     % system,     % interrupt,     % idle
##&nbsp;Mem: 251M Active, 273M Inact, 156M Wired, 13M Cache, 7472K Buf, 16M Free
##&nbsp;Swap: 1536M Total, 1536M Free
##&nbsp;
##&nbsp;  PID USERNAME     THR PRI NICE   SIZE    RES STATE   C   TIME   WCPU COMMAND
##&nbsp;47164 diag           1   8    0 10252K  2388K wait    1   0:00  1.40% bash
##&nbsp; 2668 root          33   4    0   637M   149M accept  0  12:00  0.00% mgwd
##&nbsp;  906 root          18  97    0   154M 51956K ucond   0  11:20  0.00% notifyd
##&nbsp; 2766 root           1  96    0 29440K  4196K select  1   7:55  0.00% bip_nitro
##&nbsp; 3564 root          12  96    0   148M 46652K ucond   1   6:53  0.00% nchmd
##&nbsp; 3302 root           1  96    0 17484K  1956K select  1   1:42  0.00% vmtools
##&nbsp;  714 root           1  96    0 12088K  1548K select  0   0:44  0.00% rpcbind
##&nbsp;  671 root           3  96    0 31748K  3724K ucond   0   0:43  0.00% mlogd
##&nbsp; 3523 root           4  96    0   141M 33676K select  0   0:37  0.00% ndmpd
##&nbsp; 3076 root           1  96    0 29440K  3324K select  0   0:27  0.00% mhostexecd
##&nbsp; 3589 root           6  20    0   131M 38972K pause   0   0:17  0.00% servprocd
##&nbsp; 3176 root           2  96    0 84404K 19592K ucond   1   0:08  0.00% raid_lm
##&nbsp; 3550 root          11  96    0   133M 39996K ucond   1   0:07  0.00% shmd
##&nbsp; 3882 root           1   8    0 12080K  1344K nanslp  0   0:05  0.00% cron
##&nbsp; 3612 root           7  97    0   122M 35216K ucond   1   0:05  0.00% coresegd
##&nbsp; 3577 root           3  97    0   133M 35712K select  0   0:03  0.00% nphmd
##&nbsp;  990 root           2  96    0 31620K  3732K select  1   0:02  0.00% time_state[6;1H[24;1H[K
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '0:00',
##&nbsp;            'wcpu' => '1.40%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'wait',
##&nbsp;            'res' => '2388K',
##&nbsp;            'size' => '10252K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'diag',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '47164',
##&nbsp;            'pri' => '8',
##&nbsp;            'command' => 'bash'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '12:00',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '33',
##&nbsp;            'state' => 'accept',
##&nbsp;            'res' => '149M',
##&nbsp;            'size' => '637M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '2668',
##&nbsp;            'pri' => '4',
##&nbsp;            'command' => 'mgwd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '11:20',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '18',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '51956K',
##&nbsp;            'size' => '154M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '906',
##&nbsp;            'pri' => '97',
##&nbsp;            'command' => 'notifyd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '7:55',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '4196K',
##&nbsp;            'size' => '29440K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '2766',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'bip_nitro'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '6:53',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '12',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '46652K',
##&nbsp;            'size' => '148M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '3564',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'nchmd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '1:42',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '1956K',
##&nbsp;            'size' => '17484K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '3302',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'vmtools'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:44',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '1548K',
##&nbsp;            'size' => '12088K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '714',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'rpcbind'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:43',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '3',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '3724K',
##&nbsp;            'size' => '31748K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '671',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'mlogd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:37',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '4',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '33676K',
##&nbsp;            'size' => '141M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '3523',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'ndmpd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:27',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '3324K',
##&nbsp;            'size' => '29440K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '3076',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'mhostexecd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:17',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '6',
##&nbsp;            'state' => 'pause',
##&nbsp;            'res' => '38972K',
##&nbsp;            'size' => '131M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '3589',
##&nbsp;            'pri' => '20',
##&nbsp;            'command' => 'servprocd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:08',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '2',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '19592K',
##&nbsp;            'size' => '84404K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '3176',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'raid_lm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:07',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '11',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '39996K',
##&nbsp;            'size' => '133M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '3550',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'shmd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:05',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '1',
##&nbsp;            'state' => 'nanslp',
##&nbsp;            'res' => '1344K',
##&nbsp;            'size' => '12080K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '3882',
##&nbsp;            'pri' => '8',
##&nbsp;            'command' => 'cron'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:05',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '7',
##&nbsp;            'state' => 'ucond',
##&nbsp;            'res' => '35216K',
##&nbsp;            'size' => '122M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '3612',
##&nbsp;            'pri' => '97',
##&nbsp;            'command' => 'coresegd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:03',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '3',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '35712K',
##&nbsp;            'size' => '133M',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '0',
##&nbsp;            'pid' => '3577',
##&nbsp;            'pri' => '97',
##&nbsp;            'command' => 'nphmd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time' => '0:02',
##&nbsp;            'wcpu' => '0.00%',
##&nbsp;            'thr' => '2',
##&nbsp;            'state' => 'select',
##&nbsp;            'res' => '3732K',
##&nbsp;            'size' => '31620K',
##&nbsp;            'nice' => '0',
##&nbsp;            'username' => 'root',
##&nbsp;            'c' => '1',
##&nbsp;            'pid' => '990',
##&nbsp;            'pri' => '96',
##&nbsp;            'command' => 'time_state'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub top {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = -1;
    my ( @result_array, @values, @head );
    foreach my $line (@lines) {
        if ( $line =~ /PID.*/ ) {
           @head = split( /\s+/, $line );
         } elsif($line =~ /\d+\s+\w+\s+\w+.*/ ) {
            $count++;
            @values = split( /\s+/, $line );
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
               $values[$j] =~ s/\e.*//;
               $result_array[$count]->{ to_key_name( item =>$head[$j]) } = $values[$j];
            }
        }
    }
    return \@result_array;
} ## end sub top

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /unknown\s+user|.*recognized\s+sorting\s+order/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
