#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd vmstat command"
##
## @description
##    This is a library to parse the raw output of Freebsd vmstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vmstat" will be vmstat()
##
## @synovmstatis
##    use vmstat;
##
##    vmstat(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::vmstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingValueException;

## @name ::vmstat
##
## @summary Parses the raw output from the "vmstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vmstat with any of its options as described in the
##    corresponding cdef file 'vmstat.cdefs'
## @synopsis
##      vmstat(%args);
## INPUTS TO METHOD CALL 'vmstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-M' </td> <td> 'core' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kernel-zone' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'numbers' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'interrupts' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'read' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sum' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'usage' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'startup' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'device-types' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'devs' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fork' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vmstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; procs      memory      page                    disks     faults         cpu
##&nbsp; r b w     avm    fre   flt  re  pi  po    fr  sr md0 ad0   in   sy   cs us sy id
##&nbsp; 0 3 0   2662M    40M    53   0   0   0    58   0   0   0  446  391 2602  0  2 98
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disks' => [
##&nbsp;                         {
##&nbsp;                           'ad0' => '0',
##&nbsp;                           'md0' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'memory' => [
##&nbsp;                          {
##&nbsp;                            'fre' => '40M',
##&nbsp;                            'avm' => '2662M'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'cpu' => [
##&nbsp;                       {
##&nbsp;                         'sy' => '2',
##&nbsp;                         'us' => '0',
##&nbsp;                         'id' => '98'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'page' => [
##&nbsp;                        {
##&nbsp;                          'fr' => '58',
##&nbsp;                          'po' => '0',
##&nbsp;                          're' => '0',
##&nbsp;                          'pi' => '0',
##&nbsp;                          'flt' => '53',
##&nbsp;                          'sr' => '0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'procs' => [
##&nbsp;                         {
##&nbsp;                           'w' => '0',
##&nbsp;                           'r' => '0',
##&nbsp;                           'b' => '3'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'faults' => [
##&nbsp;                          {
##&nbsp;                            'sy' => '391',
##&nbsp;                            'in' => '446',
##&nbsp;                            'cs' => '2602'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat  -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;345715 forks, 171700890 pages, average 496.65
##&nbsp;24154 vforks, 12037703 pages, average 498.37
##&nbsp;0 rforks, 0 pages, average 0.00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'forks' => '345715',
##&nbsp;            'average' => '496.65',
##&nbsp;            'pages' => '171700890'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'average' => '498.37',
##&nbsp;            'vforks' => '24154',
##&nbsp;            'pages' => '12037703'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'rforks' => '0',
##&nbsp;            'average' => '0.00',
##&nbsp;            'pages' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat  -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;interrupt                          total       rate
##&nbsp;irq1: atkbd0                           2          0
##&nbsp;irq3: uart1                            1          0
##&nbsp;irq4: uart0                       262939          0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'rate' => '0',
##&nbsp;            'total' => '2',
##&nbsp;            'interrupt' => 'irq1: atkbd0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'rate' => '0',
##&nbsp;            'total' => '1',
##&nbsp;            'interrupt' => 'irq3: uart1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'rate' => '0',
##&nbsp;            'total' => '262939',
##&nbsp;            'interrupt' => 'irq4: uart0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat  -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                     Type  InUse  MemUse HighUse Requests  Size(s)
##&nbsp;                  CAM SIM      1      1K       -        1  256
##&nbsp;                  CAM periph      2      1K       -        9  16,32,64,128,256
##&nbsp;                  D-openssh    127    477K       -      186  H/PSEG      364K/       84K
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memuse' => '1K',
##&nbsp;            'inuse' => '1',
##&nbsp;            'requests' => '1',
##&nbsp;            'highuse' => '-',
##&nbsp;            'type' => 'CAM SIM',
##&nbsp;            'sizes' => '256'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memuse' => '1K',
##&nbsp;            'inuse' => '2',
##&nbsp;            'requests' => '9',
##&nbsp;            'highuse' => '-',
##&nbsp;            'type' => 'CAM periph',
##&nbsp;            'sizes' => '1632,64,128,256'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memuse' => '477K',
##&nbsp;            'inuse' => '127',
##&nbsp;            'requests' => '186',
##&nbsp;            'highuse' => '-',
##&nbsp;            'type' => 'D-openssh',
##&nbsp;            'sizes' => 'H/PSEG      364K/       84K'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = vmstat  -z
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ITEM                     SIZE     LIMIT      USED      FREE  REQUESTS  FAILURES
##&nbsp;
##&nbsp;UMA Kegs:                 208,        0,       86,       16,       87,        0
##&nbsp;UMA Zones:                264,        0,       86,        5,       87,        0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'failures' => '0',
##&nbsp;            'free' => '16',
##&nbsp;            'requests' => '87',
##&nbsp;            'used' => '86',
##&nbsp;            'item' => 'UMA Kegs',
##&nbsp;            'limit' => '0',
##&nbsp;            'size' => '208'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'failures' => '0',
##&nbsp;            'free' => '5',
##&nbsp;            'requests' => '87',
##&nbsp;            'used' => '86',
##&nbsp;            'item' => 'UMA Zones',
##&nbsp;            'limit' => '0',
##&nbsp;            'size' => '264'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat  -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;2664675519 cpu context switches
##&nbsp;456468795 device interrupts
##&nbsp;143692106 software interrupts
##&nbsp; 66788431 traps
##&nbsp;400786364 system calls
##&nbsp;   140652 kernel threads created
##&nbsp;   346691  fork() calls
##&nbsp;    24222 vfork() calls
##&nbsp;        0 rfork() calls
##&nbsp;        0 swap pager pageins
##&nbsp;        0 swap pager pages paged in
##&nbsp;        0 swap pager pageouts
##&nbsp;        0 swap pager pages paged out
##&nbsp;     5137 vnode pager pageins
##&nbsp;    53094 vnode pager pages paged in
##&nbsp;        0 vnode pager pageouts
##&nbsp;        0 vnode pager pages paged out
##&nbsp;       18 page daemon wakeups
##&nbsp;   103491 pages examined by the page daemon
##&nbsp;     6388 pages reactivated
##&nbsp; 14331445 copy-on-write faults
##&nbsp;     4165 copy-on-write optimized faults
##&nbsp; 24130763 zero fill pages zeroed
##&nbsp;     3125 zero fill pages prezeroed
##&nbsp;      714 intransit blocking page faults
##&nbsp; 54164984 total VM faults taken
##&nbsp;        0 pages affected by kernel thread creation
##&nbsp;172187032 pages affected by  fork()
##&nbsp; 12071214 pages affected by vfork()
##&nbsp;        0 pages affected by rfork()
##&nbsp;   103678 pages cached
##&nbsp; 59630824 pages freed
##&nbsp;        0 pages freed by daemon
##&nbsp; 22678568 pages freed by exiting processes
##&nbsp;    65129 pages active
##&nbsp;    66113 pages inactive
##&nbsp;     3230 pages in VM cache
##&nbsp;    40297 pages wired down
##&nbsp;     6900 pages free
##&nbsp;     4096 bytes per page
##&nbsp; 73825576 total name lookups
##&nbsp;          cache hits (89% pos + 9% neg) system 0% per-directory
##&nbsp;          deletions 0%, falsehits 0%, toolong 0%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_name_lookups' => '73825576',
##&nbsp;            'pages_wired_down' => '40297',
##&nbsp;            'kernel_threads_created' => '140652',
##&nbsp;            'zero_fill_pages_zeroed' => '24130763',
##&nbsp;            'vnode_pager_pages_paged_out' => '0',
##&nbsp;            'cache_hits' => '89% pos + 9% neg',
##&nbsp;            'cpu_context_switches' => '2664675519',
##&nbsp;            'system_calls' => '400786364',
##&nbsp;            'pages_examined_by_the_page_daemon' => '103491',
##&nbsp;            'swap_pager_pageouts' => '0',
##&nbsp;            'traps' => '66788431',
##&nbsp;            'pages_active' => '65129',
##&nbsp;            'pages_affected_by_fork' => '172187032',
##&nbsp;            'fork_calls' => '346691',
##&nbsp;            'rfork_calls' => '0',
##&nbsp;            'pages_inactive' => '66113',
##&nbsp;            'intransit_blocking_page_faults' => '714',
##&nbsp;            'pages_free' => '6900',
##&nbsp;            'bytes_per_page' => '4096',
##&nbsp;            'software_interrupts' => '143692106',
##&nbsp;            'toolong' => '0%',
##&nbsp;            'pages_reactivated' => '6388',
##&nbsp;            'pages_affected_by_rfork' => '0',
##&nbsp;            'vnode_pager_pages_paged_in' => '53094',
##&nbsp;            'vnode_pager_pageins' => '5137',
##&nbsp;            'pages_affected_by_kernel_thread_creation' => '0',
##&nbsp;            'falsehits' => '0%',
##&nbsp;            'system' => '0% per-directory',
##&nbsp;            'page_daemon_wakeups' => '18',
##&nbsp;            'vfork_calls' => '24222',
##&nbsp;            'copy-on-write_faults' => '14331445',
##&nbsp;            'device_interrupts' => '456468795',
##&nbsp;            'zero_fill_pages_prezeroed' => '3125',
##&nbsp;            'pages_freed_by_exiting_processes' => '22678568',
##&nbsp;            'swap_pager_pageins' => '0',
##&nbsp;            'pages_cached' => '103678',
##&nbsp;            'pages_affected_by_vfork' => '12071214',
##&nbsp;            'total_vm_faults_taken' => '54164984',
##&nbsp;            'pages_freed' => '59630824',
##&nbsp;            'copy-on-write_optimized_faults' => '4165',
##&nbsp;            'deletions' => '0%',
##&nbsp;            'vnode_pager_pageouts' => '0',
##&nbsp;            'pages_freed_by_daemon' => '0',
##&nbsp;            'swap_pager_pages_paged_in' => '0',
##&nbsp;            'pages_in_vm_cache' => '3230',
##&nbsp;            'swap_pager_pages_paged_out' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vmstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count     = -1;
    my @main_head = split( /\s+/, $lines[0] );
    my @sub_head  = split( /\s+/, $lines[1] );

    if ( $cmd =~ /-s|-f/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(\d+)\s+(\w+)\,\s+(\d+)\s+(\w+)\,\s+(\w+)\s+(\S+)/ ) {
                $count++;
                $result_array[$count]->{ to_key_name( item => $2 ) } = $1;
                $result_array[$count]->{ to_key_name( item => $4 ) } = $3;
                $result_array[$count]->{ to_key_name( item => $5 ) } = $6;
            } elsif ( $line =~ /(\d+)\s+(.*)/ ) {
                my $key   = $2;
                my $temp_value = $1;
                $key =~ s/\(//;
                $key =~ s/\)//;
                $result_array[0]->{ to_key_name( item => $key ) } = $temp_value;
            } elsif ( $line =~ /(cache\s+hits)\s+\((.*)\)\s+(system)\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
                $result_array[0]->{ to_key_name( item => $3 ) } = $4;
            } elsif ( $line =~ /(\w+)\s+(.*)\,\s+(\w+)\s+(.*)\,\s+(\w+)\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
                $result_array[0]->{ to_key_name( item => $3 ) } = $4;
                $result_array[0]->{ to_key_name( item => $5 ) } = $6;
            }
        }
    } elsif ( $cmd =~ /-i|-z/ ) {
        for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
            $count++;
            my @values = split( /\s\s+/, $lines[$i] );
            for ( my $j = 0 ; $j <= $#main_head ; $j++ ) {
                $values[$j]    =~ s/\,|\:// if($cmd !~ /-i/);
                $result_array[$count]
                  ->{ to_key_name( item => $main_head[$j] ) } = $values[$j];
            }
        }
    } elsif($cmd =~ /-m/) {
       for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
        $count++;
        my @values;
        if($lines[$i] =~ /(.*\w+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\d+)\s*(.*)/) {
        push(@values, $1,$2,$3,$4,$5,$6);
        for(my $j=0;$j<=$#main_head;$j++) {
            $main_head[$j] =~ s/\(//;
            $main_head[$j] =~ s/\)//;		
            $result_array[$count]
               ->{to_key_name( item =>$main_head[$j])} = $values[$j];
        }
      }
    } 
   } else {
        for ( my $i = 2 ; $i <= $#lines ; $i++ ) {
            my $main_count = 0;
            my $limit      = 0;
            $count++;
            my @values = split( /\s+/, $lines[$i] );
            foreach my $head (@sub_head) {
                $main_count++ if ( $head =~ /^avm|^flt|^md0|^in|^us/ );
                $head =~ s/\///;
                $result_array[$count]->{ $main_head[$main_count] }[0]
                  ->{ to_key_name( item => $head ) } = $values[$limit];
                $limit++;
            }
        }
    }
    return \@result_array;
} ## end sub vmstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;

    if ( $raw_output =~ /illegal\s+option|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
        /No\s+such\s+file\s+or\s+directory|unknown\s+match\s+item/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
