#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd wget command"
##
## @description
##    This is a library to parse the raw output of Freebsd wget command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "wget" will be wget()
##
## @synowgetis
##    use wget;
##
##    wget(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::wget;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::wget
##
## @summary Parses the raw output from the "wget" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wget with any of its options as described in the
##    corresponding cdef file 'wget.cdefs'
## @synopsis
##      wget(%args);
## INPUTS TO METHOD CALL 'wget' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--verbose' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '--limit-rate' </td> <td> 'limit-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '--post-file' </td> <td> 'post-file' </td> <td> '0' </td> </tr>
## <tr> <td> '--spider' </td> <td> 'spider' </td> <td> '1' </td> </tr>
## <tr> <td> '--save-headers' </td> <td> 'save-headers ' </td> <td> '0' </td> </tr>
## <tr> <td> '--timestamping' </td> <td> 'timestamping' </td> <td> '1' </td> </tr>
## <tr> <td> '--password' </td> <td> 'password' </td> <td> '0' </td> </tr>
## <tr> <td> '--follow-ftp' </td> <td> 'follow-ftp' </td> <td> '1' </td> </tr>
## <tr> <td> '--page-requisites' </td> <td> 'page-requisites' </td> <td> '1' </td> </tr>
## <tr> <td> '--timeout' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '--ftp-password' </td> <td> 'ftp-password' </td> <td> '0' </td> </tr>
## <tr> <td> '--post-data' </td> <td> 'post-data' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-cache' </td> <td> 'no-cache' </td> <td> '1' </td> </tr>
## <tr> <td> '--http-password' </td> <td> 'http-password' </td> <td> '0' </td> </tr>
## <tr> <td> '--tries' </td> <td> 'tries' </td> <td> '0' </td> </tr>
## <tr> <td> '--base' </td> <td> 'base' </td> <td> '0' </td> </tr>
## <tr> <td> '--relative' </td> <td> 'relative' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignore-length' </td> <td> 'ignore-length' </td> <td> '1' </td> </tr>
## <tr> <td> '--exclude-directories' </td> <td> 'exclude-directories' </td> <td> '0' </td> </tr>
## <tr> <td> '--level' </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> '--load-cookies' </td> <td> 'load-cookies' </td> <td> '0' </td> </tr>
## <tr> <td> '--retr-symlinks' </td> <td> 'retr-symlinks' </td> <td> '1' </td> </tr>
## <tr> <td> '--delete-after' </td> <td> 'delete-after' </td> <td> '1' </td> </tr>
## <tr> <td> '--no-glob' </td> <td> 'no-glob' </td> <td> '1' </td> </tr>
## <tr> <td> '--mirror' </td> <td> 'mirror' </td> <td> '1' </td> </tr>
## <tr> <td> '--backup-converted' </td> <td> 'backup-converted' </td> <td> '1' </td> </tr>
## <tr> <td> '--connect-timeout' </td> <td> 'connect-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '--dns-timeout' </td> <td> 'dns-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-passive-ftp' </td> <td> 'no-passive-ftp' </td> <td> '1' </td> </tr>
## <tr> <td> '--no-dns-cache' </td> <td> 'no-dns-cache' </td> <td> '1' </td> </tr>
## <tr> <td> '--referer' </td> <td> 'referer' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-directories' </td> <td> 'no-directories' </td> <td> '1' </td> </tr>
## <tr> <td> '--force-html' </td> <td> 'force-html' </td> <td> '1' </td> </tr>
## <tr> <td> '--progress' </td> <td> 'progress' </td> <td> '0' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--background' </td> <td> 'background' </td> <td> '1' </td> </tr>
## <tr> <td> '--force-directories' </td> <td> 'force-directories' </td> <td> '1' </td> </tr>
## <tr> <td> '--execute' </td> <td> 'execute' </td> <td> '0' </td> </tr>
## <tr> <td> '--recursive' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> '--domains' </td> <td> 'domains' </td> <td> '0' </td> </tr>
## <tr> <td> '--wait' </td> <td> 'wait' </td> <td> '0' </td> </tr>
## <tr> <td> '--proxy-user' </td> <td> 'proxy-user' </td> <td> '0' </td> </tr>
## <tr> <td> '--keep-session-cookies' </td> <td> 'keep-session-cookies' </td> <td> '1' </td> </tr>
## <tr> <td> '--input-file' </td> <td> 'input-file' </td> <td> '0' </td> </tr>
## <tr> <td> '--http-user' </td> <td> 'http-user' </td> <td> '0' </td> </tr>
## <tr> <td> '--span-hosts' </td> <td> 'span-hosts' </td> <td> '1' </td> </tr>
## <tr> <td> '--directory-prefix' </td> <td> 'directory-prefix' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-proxy' </td> <td> 'no-proxy' </td> <td> '1' </td> </tr>
## <tr> <td> '--retry-connrefused' </td> <td> 'retry-connrefused' </td> <td> '1' </td> </tr>
## <tr> <td> '--preserve-permissions' </td> <td> 'preserve-permissions' </td> <td> '1' </td> </tr>
## <tr> <td> '--user' </td> <td> 'user' </td> <td> '0' </td> </tr>
## <tr> <td> '--waitretry' </td> <td> 'waitretry' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'URL' </td> <td> '0' </td> </tr>
## <tr> <td> '--cut-dirs' </td> <td> 'cut-dirs' </td> <td> '0' </td> </tr>
## <tr> <td> '--ftp-user' </td> <td> 'ftp-user' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-clobber' </td> <td> 'no-clobber' </td> <td> '1' </td> </tr>
## <tr> <td> '--random-wait' </td> <td> 'random-wait' </td> <td> '1' </td> </tr>
## <tr> <td> '--include-directories' </td> <td> 'include-directories' </td> <td> '0' </td> </tr>
## <tr> <td> '--server-response' </td> <td> 'server-response' </td> <td> '1' </td> </tr>
## <tr> <td> '--follow-tags' </td> <td> 'follow-tags' </td> <td> '0' </td> </tr>
## <tr> <td> '--exclude-domains' </td> <td> 'exclude-domains' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-http-keep-alive' </td> <td> 'no-http-keep-alive' </td> <td> '1' </td> </tr>
## <tr> <td> '--no-verbose' </td> <td> 'no-verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '--proxy-password' </td> <td> 'proxy-password' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-cookies' </td> <td> 'no-cookies' </td> <td> '1' </td> </tr>
## <tr> <td> '--output-file' </td> <td> 'output-file' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-host-directories' </td> <td> 'no-host-directories' </td> <td> '1' </td> </tr>
## <tr> <td> '--convert-links' </td> <td> 'convert-links' </td> <td> '1' </td> </tr>
## <tr> <td> '--continue' </td> <td> 'continue' </td> <td> '1' </td> </tr>
## <tr> <td> '--proxy' </td> <td> 'proxy' </td> <td> '1' </td> </tr>
## <tr> <td> '--user-agent' </td> <td> 'user-agent' </td> <td> '0' </td> </tr>
## <tr> <td> '--quota' </td> <td> 'quota' </td> <td> '0' </td> </tr>
## <tr> <td> '--strict-comments' </td> <td> 'strict-comments' </td> <td> '1' </td> </tr>
## <tr> <td> '--bind-address' </td> <td> 'bind-address' </td> <td> '0' </td> </tr>
## <tr> <td> '--read-timeout' </td> <td> 'read-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '--html-extension' </td> <td> 'html-extension' </td> <td> '1' </td> </tr>
## <tr> <td> '-help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '--save-cookies' </td> <td> 'save-cookies' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-parent' </td> <td> 'no-parent' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignore-tags' </td> <td> 'ignore-tags' </td> <td> '0' </td> </tr>
## <tr> <td> '--quiet' </td> <td> 'quiet' </td> <td> '1' </td> </tr>
## <tr> <td> '--protocol-directories' </td> <td> 'protocol-directories' </td> <td> '1' </td> </tr>
## <tr> <td> '--accept' </td> <td> 'accept' </td> <td> '0' </td> </tr>
## <tr> <td> '--restrict-file-names' </td> <td> 'restrict-file-names' </td> <td> '0' </td> </tr>
## <tr> <td> '--append-output' </td> <td> 'append-output' </td> <td> '0' </td> </tr>
## <tr> <td> '--output-document' </td> <td> 'output-document' </td> <td> '0' </td> </tr>
## <tr> <td> '--no-remove-listing' </td> <td> 'no-remove-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '--reject' </td> <td> 'reject' </td> <td> '0' </td> </tr>
## <tr> <td> '--header' </td> <td> 'header' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub wget {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub wget

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /option\s+requires\s+an\s+argument|option\s+.*\s+is\s+ambiguous|missing\s+URL/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/No\s+match|No\s+such\s+file\s+or\s+directory|No\s+URLs\s+found\s+in\s+.*|Invalid\s+.*|Unknown\s+host.*|unrecognized\s+option\s+.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|invalid\s+option\s+|[Uu]sage:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
