#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Freebsd ypwhich command"
##
## @description
##    This is a library to parse the raw output of Freebsd ypwhich command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ypwhich" will be ypwhich()
##
## @synoypwhichis
##    use ypwhich;
##
##    ypwhich(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ypwhich;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::ypwhich
##
## @summary Parses the raw output from the "ypwhich" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ypwhich with any of its options as described in the
##    corresponding cdef file 'ypwhich.cdefs'
## @synopsis
##      ypwhich(%args);
## INPUTS TO METHOD CALL 'ypwhich' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'domain' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'map-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'translation' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'display' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ypwhich -x
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;"passwd" is an alias for "passwd.byname"
##&nbsp;"master.passwd" is an alias for "master.passwd.byname"
##&nbsp;"shadow" is an alias for "shadow.byname"
##&nbsp;"group" is an alias for "group.byname"
##&nbsp;"networks" is an alias for "networks.byaddr"
##&nbsp;"hosts" is an alias for "hosts.byaddr"
##&nbsp;"protocols" is an alias for "protocols.bynumber"
##&nbsp;"services" is an alias for "services.byname"
##&nbsp;"aliases" is an alias for "mail.aliases"
##&nbsp;"ethers" is an alias for "ethers.byname"
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hosts' => 'hosts.byaddr',
##&nbsp;            'master.passwd' => 'master.passwd.byname',
##&nbsp;            'shadow' => 'shadow.byname',
##&nbsp;            'services' => 'services.byname',
##&nbsp;            'networks' => 'networks.byaddr',
##&nbsp;            'passwd' => 'passwd.byname',
##&nbsp;            'group' => 'group.byname',
##&nbsp;            'protocols' => 'protocols.bynumber',
##&nbsp;            'ethers' => 'ethers.byname',
##&nbsp;            'aliases' => 'mail.aliases'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ypwhich {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;

    foreach my $line (@lines) {
        if ( $line =~ /\"(.*)\".*\"(.*)\"/ ) {
            $result_array[0]->{$1} = $2;
        }
    }
    return \@result_array;
} ## end sub ypwhich

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~
/Domain\s+not\s+bound.*|.*Can't\s+bind\s+to\s+server\s+which\s+serves\s+this\s+domain|host.*unknown/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /illegal\s+option|[Uu]sage.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
