#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX bdf command
##
## @description
##    This is a library to parse the raw output of HPUX bdf command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "bdf" will bdf
##
## @synopsis
##    use bdf;
##
##    bdf(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::bdf;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::bdf
##
## @summary Parses the raw output from the "bdf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from bdf with any of its options as described in the
##    corresponding cdef file 'bdf.cdefs'
## @synopsis
##      bdf(%args);
## INPUTS TO METHOD CALL 'bdf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'display-file-system-swapping' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'report-used-and-free-inodes' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'report-on-the-file-systems' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'display-local-file-systems' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'donot-sync-file-system-data' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = bdf -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem          kbytes    used   avail %used  iused  ifree %iuse Mounted on
##&nbsp;/dev/vg00/lvol3    3170304 2459264  707904   78%   5469  22211   20% /
##&nbsp;/dev/vg00/lvol1    1696696   93312 1433712    6%    196 270140    0% /stand
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kbytes' => '3170304',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'used_percentage' => '78%',
##&nbsp;            'avail' => '707904',
##&nbsp;            'ifree' => '22211',
##&nbsp;            '%iuse' => '20%',
##&nbsp;            'filesystem' => '/dev/vg00/lvol3',
##&nbsp;            'used' => '2459264',
##&nbsp;            'iused' => '5469'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '1696696',
##&nbsp;            'mounted_on' => '/stand',
##&nbsp;            'used_percentage' => '6%',
##&nbsp;            'avail' => '1433712',
##&nbsp;            'ifree' => '270140',
##&nbsp;            '%iuse' => '0%',
##&nbsp;            'filesystem' => '/dev/vg00/lvol1',
##&nbsp;            'used' => '93312',
##&nbsp;            'iused' => '196'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = bdf -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem          kbytes    used   avail %used Mounted on
##&nbsp;/dev/vg00/lvol3    3170304 2459704  707464   78% /
##&nbsp;/dev/vg00/lvol1    1696696   93312 1433712    6% /stand
##&nbsp;/dev/vg00/lvol8    5242880 4143176 1091152   79% /var
##&nbsp;/dev/vg00/lvol7    7340032 2644552 4658864   36% /usr
##&nbsp;/dev/vg00/lvol6    3170304  228936 2923952    7% /tmp
##&nbsp;/dev/vg00/lvol5    8388608 4395336 3962712   53% /opt
##&nbsp;/dev/vg00/lvol4    1048576   25008 1015688    2% /home
##&nbsp;DevFS                    3       3       0  100% /dev/deviceFileSystem
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kbytes' => '3170304',
##&nbsp;            'filesystem' => '/dev/vg00/lvol3',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'used_percentage' => '78%',
##&nbsp;            'used' => '2459704',
##&nbsp;            'avail' => '707464'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '1696696',
##&nbsp;            'filesystem' => '/dev/vg00/lvol1',
##&nbsp;            'mounted_on' => '/stand',
##&nbsp;            'used_percentage' => '6%',
##&nbsp;            'used' => '93312',
##&nbsp;            'avail' => '1433712'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '5242880',
##&nbsp;            'filesystem' => '/dev/vg00/lvol8',
##&nbsp;            'mounted_on' => '/var',
##&nbsp;            'used_percentage' => '79%',
##&nbsp;            'used' => '4143176',
##&nbsp;            'avail' => '1091152'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '7340032',
##&nbsp;            'filesystem' => '/dev/vg00/lvol7',
##&nbsp;            'mounted_on' => '/usr',
##&nbsp;            'used_percentage' => '36%',
##&nbsp;            'used' => '2644552',
##&nbsp;            'avail' => '4658864'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '3170304',
##&nbsp;            'filesystem' => '/dev/vg00/lvol6',
##&nbsp;            'mounted_on' => '/tmp',
##&nbsp;            'used_percentage' => '7%',
##&nbsp;            'used' => '228936',
##&nbsp;            'avail' => '2923952'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '8388608',
##&nbsp;            'filesystem' => '/dev/vg00/lvol5',
##&nbsp;            'mounted_on' => '/opt',
##&nbsp;            'used_percentage' => '53%',
##&nbsp;            'used' => '4395336',
##&nbsp;            'avail' => '3962712'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '1048576',
##&nbsp;            'filesystem' => '/dev/vg00/lvol4',
##&nbsp;            'mounted_on' => '/home',
##&nbsp;            'used_percentage' => '2%',
##&nbsp;            'used' => '25008',
##&nbsp;            'avail' => '1015688'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '3',
##&nbsp;            'filesystem' => 'DevFS',
##&nbsp;            'mounted_on' => '/dev/deviceFileSystem',
##&nbsp;            'used_percentage' => '100%',
##&nbsp;            'used' => '3',
##&nbsp;            'avail' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub bdf {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head );
    my $i = 0;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        $lines[0] =~ s/Mounted on/mounted_on/g;
        $lines[0] =~ s/%used/used_percentage/g;
        if ( $cmd =~ /(-b|-i|-l|-s)\s*.*/ ) {
            @head = split( /\s+/, $lines[0] );
        }
        if ( $line
            =~ /(\/*\S+\/*\S+\/*\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+\%)\s+(\d*)\s*(\d*)\s*(\d*\%*)\s*(\/*)(\/*\S*\/*\S*)/
            ) {
            my @values = split( /\s+/, $line );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$i]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $i++;
        }
    }

    return \@result_array;

} ## end sub bdf

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(bdf:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;

