#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/diskinfo.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX diskinfo command
##
## @description
##    This is a library to parse the raw output of HPUX diskinfo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "diskinfo" will diskinfo()
##
## @synopsis
##    use diskinfo;
##
##    diskinfo(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::diskinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::diskinfo
##
## @summary Parses the raw output from the "diskinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from diskinfo with any of its options as described in the
##    corresponding cdef file 'diskinfo.cdefs'
## @synopsis
##      diskinfo(%args);
## INPUTS TO METHOD CALL 'diskinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'disksize' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = diskinfo /dev/rdisk/disk3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SCSI describe of /dev/rdisk/disk3:
##&nbsp;             vendor: HP 36.4G
##&nbsp;         product id: ST336754LC
##&nbsp;               type: direct access
##&nbsp;               size: 35566480 Kbytes
##&nbsp;   bytes per sector: 512
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes_per_sector' => '512',
##&nbsp;            'product_id' => 'ST336754LC',
##&nbsp;            'type' => 'direct access',
##&nbsp;            'device' => '/dev/rdisk/disk3:',
##&nbsp;            'size' => '35566480 Kbytes',
##&nbsp;            'vendor' => 'HP 36.4G'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = diskinfo -b /dev/rdisk/disk3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;35566480
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk_size' => '35566480'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = diskinfo -v /dev/rdisk/disk3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SCSI describe of /dev/rdisk/disk3:
##&nbsp;             vendor: HP 36.4G
##&nbsp;         product id: ST336754LC
##&nbsp;               type: direct access
##&nbsp;               size: 35566480 Kbytes
##&nbsp;   bytes per sector: 512
##&nbsp;          rev level: HPC3
##&nbsp;    blocks per disk: 71132960
##&nbsp;        ISO version: 0
##&nbsp;       ECMA version: 0
##&nbsp;       ANSI version: 3
##&nbsp;    removable media: no
##&nbsp;    response format: 2
##&nbsp;   (Additional inquiry bytes: (32)48 (33)50 (34)43 (35)33 (36)33 (37)4b (38)51 (39)31 (40)4d (41)43 (42)44 (43)48 (44)20 (45)20 (46)20 (47)20 (48)20 (49)20 (50)20 (51)20 (52)20 (53)20 (54)20 (55)20 (56)f (57)0 (58)0 (59)60 (60)a (61)dc (62)b (63)59 (64)3 (65)1 (66)3 (67)22 (68)0 (69)0 (70)0 (71)0 (72)0 (73)0 (74)0 (75)0 (76)0 (77)0 (78)0 (79)0 (80)0 (81)0 (82)0 (83)0 (84)0 (85)0 (86)0 (87)0 (88)0 (89)0 (90)0 (91)0 (92)0 (93)0 (94)0 (95)0 (96)0 (97)43 (98)6f (99)70 (100)79 (101)72 (102)69 (103)67 (104)68 (105)74 (106)20 (107)28 (108)63 (109)29 (110)20 (111)32 (112)30 (113)30 (114)36 (115)20 (116)53 (117)65 (118)61 (119)67 (120)61 (121)74 (122)65 (123)20 (124)41 (125)6c (126)6c (127)20 (128)0 (129)0 (130)0 (131)0 (132)4 (133)3d (134)67 (135)20 (136)0 (137)0 (138)2 )
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'response_format' => '2',
##&nbsp;            'ecma_version' => '0',
##&nbsp;            'blocks_per_disk' => '71132960',
##&nbsp;            'additional_inquiry_bytes' => '(32)48 (33)50 (34)43 (35)33 (36)33 (37)4b (38)51 (39)31 (40)4d (41)43 (42)44 (43)48 (44)20 (45)20 (46)20 (47)20 (48)20 (49)20 (50)20 (51)20 (52)20 (53)20 (54)20 (55)20 (56)f (57)0 (58)0 (59)60 (60)a (61)dc (62)b (63)59 (64)3 (65)1 (66)3 (67)22 (68)0 (69)0 (70)0 (71)0 (72)0 (73)0 (74)0 (75)0 (76)0 (77)0 (78)0 (79)0 (80)0 (81)0 (82)0 (83)0 (84)0 (85)0 (86)0 (87)0 (88)0 (89)0 (90)0 (91)0 (92)0 (93)0 (94)0 (95)0 (96)0 (97)43 (98)6f (99)70 (100)79 (101)72 (102)69 (103)67 (104)68 (105)74 (106)20 (107)28 (108)63 (109)29 (110)20 (111)32 (112)30 (113)30 (114)36 (115)20 (116)53 (117)65 (118)61 (119)67 (120)61 (121)74 (122)65 (123)20 (124)41 (125)6c (126)6c (127)20 (128)0 (129)0 (130)0 (131)0 (132)4 (133)3d (134)67 (135)20 (136)0 (137)0 (138)2',
##&nbsp;            'iso_version' => '0',
##&nbsp;            'device' => '/dev/rdisk/disk3:',
##&nbsp;            'rev_level' => 'HPC3',
##&nbsp;            'size' => '35566480 Kbytes',
##&nbsp;            'ansi_version' => '3',
##&nbsp;            'bytes_per_sector' => '512',
##&nbsp;            'removable_media' => 'no',
##&nbsp;            'product_id' => 'ST336754LC',
##&nbsp;            'type' => 'direct access',
##&nbsp;            'vendor' => 'HP 36.4G'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub diskinfo {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $cmd = $resp->get_command();
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    $output =~ s/\s+$//;

    if ( $cmd =~ /-b/ ) {
        $result_array[0]->{disk_size} = $output;
        return \@result_array;
    } else {
        foreach my $line (@lines) {
            if ( $line =~ /SCSI describe of (.*)/ ) {
                $result_array[0]->{device} = $1;
            } else {
                my @temp = split( /:\s/, $line );
                $temp[0] =~ s/^\(//;
                $temp[1] =~ s/\s+\)$//;
                $result_array[0]->{ to_key_name( item => $temp[0] ) }
                    = $temp[1];
            }
        }
    }
    return \@result_array;

} ## end sub diskinfo

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(Usage:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(diskinfo:)\s+.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

