#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/fstyp.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HP-UX fstyp command
##
## @description
##    This is a library to parse the raw output of HP-UX fstyp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "fstyp" will fstyp()
##
## @synopsis
##    use fstyp;
##
##    fstyp(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::fstyp;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::fstyp
##
## @summary Parses the raw output from the "fstyp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fstyp with any of its options as described in the
##    corresponding cdef file 'fstyp.cdefs'
## @synopsis
##      fstyp(%args);
## INPUTS TO METHOD CALL 'fstyp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fstyp -v /dev/vg00/lvol5
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;vxfs
##&nbsp;version: 6
##&nbsp;f_bsize: 8192
##&nbsp;f_frsize: 8192
##&nbsp;f_blocks: 1048576
##&nbsp;f_bfree: 499159
##&nbsp;f_bavail: 495260
##&nbsp;f_files: 189056
##&nbsp;f_ffree: 124768
##&nbsp;f_favail: 124768
##&nbsp;f_fsid: 1073741829
##&nbsp;f_basetype: vxfs
##&nbsp;f_namemax: 254
##&nbsp;f_magic: a501fcf5
##&nbsp;f_featurebits: 0
##&nbsp;f_flag: 16
##&nbsp;f_fsindex: 10
##&nbsp;f_size: 1048576
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'f_size' => '1048576',
##&nbsp;            'f_fsindex' => '10',
##&nbsp;            'f_files' => '189056',
##&nbsp;            'f_frsize' => '8192',
##&nbsp;            'f_flag' => '16',
##&nbsp;            'f_bfree' => '499159',
##&nbsp;            'f_namemax' => '254',
##&nbsp;            'f_bsize' => '8192',
##&nbsp;            'f_basetype' => 'vxfs',
##&nbsp;            'f_fsid' => '1073741829',
##&nbsp;            'f_favail' => '124768',
##&nbsp;            'f_featurebits' => '0',
##&nbsp;            'f_ffree' => '124768',
##&nbsp;            'version' => '6',
##&nbsp;            'f_blocks' => '1048576',
##&nbsp;            'f_bavail' => '495260',
##&nbsp;            'f_magic' => 'a501fcf5',
##&nbsp;            'fstype' => 'vxfs'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fstyp -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hfs
##&nbsp;nfs
##&nbsp;cdfs
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;DevFS
##&nbsp;ffs
##&nbsp;lofs
##&nbsp;procfs
##&nbsp;vxfs
##&nbsp;nfs3
##&nbsp;nfs4
##&nbsp;autofs
##&nbsp;cachefs
##&nbsp;cifs
##&nbsp;cfsm
##&nbsp;pipefs
##&nbsp;dnlc_neg_fs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'supported_fstypes' => [
##&nbsp;                                     'hfs',
##&nbsp;                                     'nfs',
##&nbsp;                                     'cdfs',
##&nbsp;                                     'DevFS',
##&nbsp;                                     'ffs',
##&nbsp;                                     'lofs',
##&nbsp;                                     'procfs',
##&nbsp;                                     'vxfs',
##&nbsp;                                     'nfs3',
##&nbsp;                                     'nfs4',
##&nbsp;                                     'autofs',
##&nbsp;                                     'cachefs',
##&nbsp;                                     'cifs',
##&nbsp;                                     'cfsm',
##&nbsp;                                     'pipefs',
##&nbsp;                                     'dnlc_neg_fs'
##&nbsp;                                   ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fstyp /dev/vg00/lvol5
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;vxfs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fstype' => 'vxfs'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fstyp {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my $i     = -1;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $cmd   = $resp->get_command();

    foreach my $line (@lines) {
        if ( $cmd =~ /-v/ ) {
            $result_array[0]->{fstype} = $lines[0];
            if ( $line =~ /(\w+):\s+(\w+)/ ) {
                $result_array[0]->{$1} = $2;
            }
        } elsif ( $line =~ /(\w+)/ ) {
            $i++;
            my $type = $1;
            if ( $cmd =~ /-l/ ) {
                $result_array[0]->{supported_fstypes}->[$i] = $type;
            } else {
                $result_array[0]->{fstype} = $line;
            }
        }
    }
    return \@result_array;

} ## end sub fstyp

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();

    if ( $raw_output =~ /usage:|fstyp:/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /unknown_fstyp|No such file or directory/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

