#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/ioscan.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX ioscan command
##
## @description
##    This is a library to parse the raw output of HPUX ioscan command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ioscan" will be ioscan()
##
## @synopsis
##    use ioscan;
##
##    ioscan(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::ioscan;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::ioscan
##
## @summary Parses the raw output from the "ioscan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ioscan with any of its options as described in the
##    corresponding cdef file 'ioscan.cdefs'
## @synopsis
##      ioscan(%args);
## INPUTS TO METHOD CALL 'ioscan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'remove' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mapping' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'compact-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'thread-zero-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'time' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'hw-path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'devfile' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'device-path' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'unclaimed' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'property' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'software-driver' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'alias-path' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list-locally-connected-devices' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'usable' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'stale' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'pending-deferred-bindings' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'agile-view' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'list-device-file-names' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'class' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kernel-io' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'driver' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'defer-driver-binding' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ioscan -t
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Thu Jul 19 00:37:11 2012
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'scanned_time' => '00:37:11',
##&nbsp;            'date' => '19',
##&nbsp;            'month' => 'Jul',
##&nbsp;            'day' => 'Thu',
##&nbsp;            'year' => '2012'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ioscan -P bus_type -C disk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Class     I  H/W Path  bus_type
##&nbsp;===============================
##&nbsp;disk      3  64000/0xfa00/0x0  virtbus
##&nbsp;disk      4  64000/0xfa00/0x1  virtbus
##&nbsp;disk      5  64000/0xfa00/0x2  virtbus
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;              'hardware_path' => '64000/0xfa00/0x0',
##&nbsp;              'class' => 'disk',
##&nbsp;              'bus_type' => 'virtbus',
##&nbsp;              'instance' => '3'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '64000/0xfa00/0x1',
##&nbsp;              'class' => 'disk',
##&nbsp;              'bus_type' => 'virtbus',
##&nbsp;              'instance' => '4'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '64000/0xfa00/0x2',
##&nbsp;              'class' => 'disk',
##&nbsp;              'bus_type' => 'virtbus',
##&nbsp;              'instance' => '5'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ioscan -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;H/W Path     Class                     Description
##&nbsp;==================================================
##&nbsp;0            ioa                       System Bus Adapter (880)
##&nbsp;0/0              ba                    Local PCI-X Bus Adapter (783)
##&nbsp;0/0/1/0                OO              PCI SerialBus (10330035)
##&nbsp;0/0/1/1                OO              PCI SerialBus (10330035)
##&nbsp;0/0/1/2                OO              PCI SerialBus (103300e0)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;              'hardware_path' => '0',
##&nbsp;              'class' => 'ioa',
##&nbsp;              'description' => 'System Bus Adapter (880)'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '0/0',
##&nbsp;              'class' => 'ba',
##&nbsp;              'description' => 'Local PCI-X Bus Adapter (783)'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '0/0/1/0',
##&nbsp;              'class' => 'OO',
##&nbsp;              'description' => 'PCI SerialBus (10330035)'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '0/0/1/1',
##&nbsp;              'class' => 'OO',
##&nbsp;              'description' => 'PCI SerialBus (10330035)'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'hardware_path' => '0/0/1/2',
##&nbsp;              'class' => 'OO',
##&nbsp;              'description' => 'PCI SerialBus (103300e0)'
##&nbsp;            },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ioscan -m lun
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Class     I  Lun H/W Path  Driver  S/W State   H/W Type     Health  Description
##&nbsp;======================================================================
##&nbsp;disk      3  64000/0xfa00/0x0   esdisk  CLAIMED     DEVICE       online  HP 36.4GST336754LC
##&nbsp;             0/1/1/0.0x1.0x0
##&nbsp;                      /dev/disk/disk3   /dev/rdisk/disk3
##&nbsp;disk      4  64000/0xfa00/0x1   esdisk  CLAIMED     DEVICE       online  HP 36.4GST336754LC
##&nbsp;             0/1/1/0.0x0.0x0
##&nbsp;                      /dev/disk/disk4   /dev/rdisk/disk4
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'lun_hardware_path' => '64000/0xfa00/0x0',
##&nbsp;              'description' => 'HP 36.4GST336754LC 0/1/1/0.0x1.0x0 /dev/disk/disk3   /dev/rdisk/disk3',
##&nbsp;              'health' => 'online',
##&nbsp;              'class' => 'disk',
##&nbsp;              'hardware_type' => 'DEVICE',
##&nbsp;              'instance' => '3',
##&nbsp;              'driver' => 'esdisk'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'lun_hardware_path' => '64000/0xfa00/0x1',
##&nbsp;              'description' => 'HP 36.4GST336754LC 0/1/1/0.0x0.0x0 /dev/disk/disk4   /dev/rdisk/disk4',
##&nbsp;              'health' => 'online',
##&nbsp;              'class' => 'disk',
##&nbsp;              'hardware_type' => 'DEVICE',
##&nbsp;              'instance' => '4',
##&nbsp;              'driver' => 'esdisk'
##&nbsp;            },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ioscan -A -F
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;:central_bus:F:F:F:-1:-1:4294967295:root:root:::0:root:root:CLAIMED:BUS_NEXUS::0::
##&nbsp;pa:central_bus:F:F:F:-1:-1:4294967295:ioa:sba:0:136 0 0 76 0 0 12 16 0 0 0 0 0 0 0 0 :0:root.sba:sba:CLAIMED:BUS_NEXUS:System Bus Adapter (880):0::
##&nbsp;pa:lba:F:F:F:-1:-1:4294967295:ba:lba_adapter:0/0:120 48 0 77 0 0 10 0 0 0 0 0 0 0 0 0 :0:root.sba.lba:lba:CLAIMED:BUS_NEXUS:Local PCI-X Bus Adapter (783):0::
##&nbsp;pci:wsio:F:T:F:-1:124:0:OO:UsbOhci:0/0/1/0:16 51 0 53 0 0 0 3 :0:root.sba.lba.UsbOhci:UsbOhci:CLAIMED:INTERFACE:PCI SerialBus (10330035):0::
##&nbsp;pci:wsio:F:T:F:-1:124:65536:OO:UsbOhci:0/0/1/1:16 51 0 53 0 0 0 3 :1:root.sba.lba.UsbOhci:UsbOhci:CLAIMED:INTERFACE:PCI SerialBus (10330035):1::
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;              'identify_bytes' => '-',
##&nbsp;              'character_major_number' => '-1',
##&nbsp;              'a_brief_description' => '-',
##&nbsp;              'hardware_type' => 'BUS_NEXUS',
##&nbsp;              'minor_number' => '4294967295',
##&nbsp;              'module_name' => 'root',
##&nbsp;              'instance_number' => '0',
##&nbsp;              'module_path' => 'root',
##&nbsp;              'alias_path' => undef,
##&nbsp;              'is_block' => 'F',
##&nbsp;              'is_pseudo' => 'F',
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'card_instance' => '0',
##&nbsp;              'is_char' => 'F',
##&nbsp;              'class' => 'root',
##&nbsp;              'cdio' => 'central_bus',
##&nbsp;              'hardware_path' => '-',
##&nbsp;              'block_major_number' => '-1',
##&nbsp;              'bus_type' => '-',
##&nbsp;              'driver' => 'root'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'identify_bytes' => '136 0 0 76 0 0 12 16 0 0 0 0 0 0 0 0 ',
##&nbsp;              'character_major_number' => '-1',
##&nbsp;              'a_brief_description' => 'System Bus Adapter (880)',
##&nbsp;              'hardware_type' => 'BUS_NEXUS',
##&nbsp;              'minor_number' => '4294967295',
##&nbsp;              'module_name' => 'sba',
##&nbsp;              'instance_number' => '0',
##&nbsp;              'module_path' => 'root.sba',
##&nbsp;              'alias_path' => undef,
##&nbsp;              'is_block' => 'F',
##&nbsp;              'is_pseudo' => 'F',
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'card_instance' => '0',
##&nbsp;              'is_char' => 'F',
##&nbsp;              'class' => 'ioa',
##&nbsp;              'cdio' => 'central_bus',
##&nbsp;              'hardware_path' => '0',
##&nbsp;              'block_major_number' => '-1',
##&nbsp;              'bus_type' => 'pa',
##&nbsp;              'driver' => 'sba'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'identify_bytes' => '120 48 0 77 0 0 10 0 0 0 0 0 0 0 0 0 ',
##&nbsp;              'character_major_number' => '-1',
##&nbsp;              'a_brief_description' => 'Local PCI-X Bus Adapter (783)',
##&nbsp;              'hardware_type' => 'BUS_NEXUS',
##&nbsp;              'minor_number' => '4294967295',
##&nbsp;              'module_name' => 'lba',
##&nbsp;              'instance_number' => '0',
##&nbsp;              'module_path' => 'root.sba.lba',
##&nbsp;              'alias_path' => undef,
##&nbsp;              'is_block' => 'F',
##&nbsp;              'is_pseudo' => 'F',
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'card_instance' => '0',
##&nbsp;              'is_char' => 'F',
##&nbsp;              'class' => 'ba',
##&nbsp;              'cdio' => 'lba',
##&nbsp;              'hardware_path' => '0/0',
##&nbsp;              'block_major_number' => '-1',
##&nbsp;              'bus_type' => 'pa',
##&nbsp;              'driver' => 'lba_adapter'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'identify_bytes' => '16 51 0 53 0 0 0 3 ',
##&nbsp;              'character_major_number' => '124',
##&nbsp;              'a_brief_description' => 'PCI SerialBus (10330035)',
##&nbsp;              'hardware_type' => 'INTERFACE',
##&nbsp;              'minor_number' => '0',
##&nbsp;              'module_name' => 'UsbOhci',
##&nbsp;              'instance_number' => '0',
##&nbsp;              'module_path' => 'root.sba.lba.UsbOhci',
##&nbsp;              'alias_path' => undef,
##&nbsp;              'is_block' => 'F',
##&nbsp;              'is_pseudo' => 'F',
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'card_instance' => '0',
##&nbsp;              'is_char' => 'T',
##&nbsp;              'class' => 'OO',
##&nbsp;              'cdio' => 'wsio',
##&nbsp;              'hardware_path' => '0/0/1/0',
##&nbsp;              'block_major_number' => '-1',
##&nbsp;              'bus_type' => 'pci',
##&nbsp;              'driver' => 'UsbOhci'
##&nbsp;            },
##&nbsp;            {
##&nbsp;              'identify_bytes' => '16 51 0 53 0 0 0 3 ',
##&nbsp;              'character_major_number' => '124',
##&nbsp;              'a_brief_description' => 'PCI SerialBus (10330035)',
##&nbsp;              'hardware_type' => 'INTERFACE',
##&nbsp;              'minor_number' => '65536',
##&nbsp;              'module_name' => 'UsbOhci',
##&nbsp;              'instance_number' => '1',
##&nbsp;              'module_path' => 'root.sba.lba.UsbOhci',
##&nbsp;              'alias_path' => undef,
##&nbsp;              'is_block' => 'F',
##&nbsp;              'is_pseudo' => 'F',
##&nbsp;              'software_state' => 'CLAIMED',
##&nbsp;              'card_instance' => '1',
##&nbsp;              'is_char' => 'T',
##&nbsp;              'class' => 'OO',
##&nbsp;              'cdio' => 'wsio',
##&nbsp;              'hardware_path' => '0/0/1/1',
##&nbsp;              'block_major_number' => '-1',
##&nbsp;              'bus_type' => 'pci',
##&nbsp;              'driver' => 'UsbOhci'
##&nbsp;            },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ioscan -P ms_scan_time -H 0/0/2/0.0.0x0.0x0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Class     I  H/W Path  ms_scan_time
##&nbsp;===============================
##&nbsp;lunpath   2  0/0/2/0.0.0x0.0x0  0 min 0 sec 2 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ms_scan_time' => '0 min 0 sec 2 ms',
##&nbsp;            'hardware_path' => '0/0/2/0.0.0x0.0x0',
##&nbsp;            'class' => 'lunpath',
##&nbsp;            'instance' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ioscan {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $cmd   = $resp->get_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $c     = 0;
    my ( @result_array, @head, @values );
    @head = split( /\s\s+/, $lines[0] );

    foreach (@head) {
        $_ =~ s/H\/W/hardware/;
        $_ =~ s/S\/W/software/;
        $_ =~ s/I/instance/;
    }
    if ( $cmd =~ /-m/ ) {
        for ( my $i = 2; $i <= $#lines; $i += 3 ) {
            @values = split( /\s\s+/, $lines[$i] );
            my $size = @values;
            if ( $#values == 8 ) {
                $values[ $size - 2 ]
                    = $values[ $size - 2 ] . " " . $values[ $size - 1 ];
                pop(@values);
                $size = @values;
            }
            $lines[ $i + 1 ] =~ s/^\s+//;
            $lines[ $i + 2 ] =~ s/^\s+//;
            my $temp = $lines[ $i + 1 ] . $lines[ $i + 2 ];
            $values[ $size - 1 ] = $values[ $size - 1 ] . $temp;
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $c++;
        }
        return \@result_array;
    } elsif ( $cmd =~ /-t/ ) {
        my @array = split( /\s+/, $output );
        $result_array[0]->{day}          = $array[0];
        $result_array[0]->{month}        = $array[1];
        $result_array[0]->{date}         = $array[2];
        $result_array[0]->{scanned_time} = $array[3];
        $result_array[0]->{year}         = $array[4];
        return \@result_array;
    } elsif ( $cmd =~ /-F/ ) {
        my @heading = (
            "bus type",
            "cdio",
            "is_block",
            "is_char",
            "is_pseudo",
            "block major number",
            "character major number",
            "minor number",
            "class",
            "driver",
            "hardware path",
            "identify bytes",
            "instance number",
            "modulepath",
            "module name",
            "software state",
            "hardware type",
            "a brief description",
            "card instance"
        );
        my @head1 = (
            "is_remote",
            "EFI device path or PA device path",
            "health",
            "alias_path"
        );
        foreach my $line (@lines) {
            @values = split( /:/, $line );
            foreach (@values) {
                $_ = "-" if ( $_ eq "" );
            }

            if ( $cmd =~ /-FF | -F -F/ ) {
                push( @heading, $head1[0] );
            } elsif ( $cmd =~ /-e -F|-F -e/ ) {
                push( @heading, $head1[1] );
            } elsif ( $cmd =~ /-N -F|-F -N/ ) {
                push( @heading, $head1[2] );
            } elsif ( $cmd =~ /-A -F|-F -A/ ) {
                push( @heading, $head1[3] );
            }
            for ( my $i = 0; $i <= $#heading; $i++ ) {
                $result_array[$c]->{ to_key_name( item => $heading[$i] ) }
                    = $values[$i];
            }
            $c++;
        }
        return \@result_array;
    } else {
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            @values = split( /\s\s+/, $lines[$i] );
            if ( $lines[$i] !~ /^\w+\s+\d+\s+\S+\s+\w+/ ) {
                $lines[ $i - 1 ] = $lines[ $i - 1 ] . " " . $lines[$i];
                @values = split( /\s\s+/, $lines[ $i - 1 ] );
                $c--;
            }
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $c++;
        }
        return \@result_array;
    }
} ## end sub ioscan

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /illegal|Invalid/i ) {
        $error = "InvalidParam";
        $error_message
            = "Illegal or Invalid parameters are passed to the Command";

    } elsif ( $raw_output =~ /requires/i ) {
        $error         = "MissingRequiredParam";
        $error_message = "Command is missing the required parameters";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
