#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/pvdisplay.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HP-UX pvdisplay command
##
## @description
##    This is a library to parse the raw output of HP-UX pvdisplay command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "pvdisplay" will pvdisplay()
##
## @synopsis
##    use pvdisplay;
##
##    pvdisplay(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::pvdisplay;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::pvdisplay
##
## @summary Parses the raw output from the "pvdisplay" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvdisplay with any of its options as described in the
##    corresponding cdef file 'pvdisplay.cdefs'
## @synopsis
##      pvdisplay(%args);
## INPUTS TO METHOD CALL 'pvdisplay' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'block_info' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'off-set-info' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pv-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'hp-lvm' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pvdisplay -v /dev/disk/disk3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;--- Physical volumes ---
##&nbsp;PV Name                     /dev/disk/disk3
##&nbsp;VG Name                     /dev/vg00
##&nbsp;PV Status                   available
##&nbsp;Allocatable                 yes
##&nbsp;VGDA                        2
##&nbsp;Cur LV                      8
##&nbsp;PE Size (Mbytes)            8
##&nbsp;Total PE                    4341
##&nbsp;Free PE                     153
##&nbsp;Allocated PE                4188
##&nbsp;Stale PE                    0
##&nbsp;IO Timeout (Seconds)        default
##&nbsp;Autoswitch                  On
##&nbsp;Proactive Polling           On
##&nbsp;
##&nbsp;   --- Distribution of physical volume ---
##&nbsp;   LV Name                 LE of LV  PE for LV
##&nbsp;   /dev/vg00/lvol1         214       214
##&nbsp;   /dev/vg00/lvol2         512       512
##&nbsp;   /dev/vg00/lvol3         387       387
##&nbsp;   /dev/vg00/lvol4         128       128
##&nbsp;   /dev/vg00/lvol5         1024      1024
##&nbsp;   /dev/vg00/lvol6         387       387
##&nbsp;   /dev/vg00/lvol7         896       896
##&nbsp;   /dev/vg00/lvol8         640       640
##&nbsp;
##&nbsp;   --- Physical extents ---
##&nbsp;   PE    Status   LV                      LE
##&nbsp;   00000 current  /dev/vg00/lvol1         00000
##&nbsp;   00001 current  /dev/vg00/lvol1         00001
##&nbsp;   00002 current  /dev/vg00/lvol1         00002
##&nbsp;   00003 current  /dev/vg00/lvol1         00003
##&nbsp;   00004 current  /dev/vg00/lvol1         00004
##&nbsp;   00005 current  /dev/vg00/lvol1         00005
##&nbsp;   00006 current  /dev/vg00/lvol1         00006
##&nbsp;   00007 current  /dev/vg00/lvol1         00007
##&nbsp;   00008 current  /dev/vg00/lvol1         00008
##&nbsp;   00009 current  /dev/vg00/lvol1         00009
##&nbsp;   00010 current  /dev/vg00/lvol1         00010
##&nbsp;   04190 free                             00000
##&nbsp;   04191 free                             00000
##&nbsp;   04192 free                             00000
##&nbsp;   04193 free                             00000
##&nbsp;   04194 free                             00000
##&nbsp;   04195 free                             00000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'distribution_of_physical_volume' => [
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol1',
##&nbsp;                                                     'le_of_lv' => '214',
##&nbsp;                                                     'pe_for_lv' => '214'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol2',
##&nbsp;                                                     'le_of_lv' => '512',
##&nbsp;                                                     'pe_for_lv' => '512'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol3',
##&nbsp;                                                     'le_of_lv' => '387',
##&nbsp;                                                     'pe_for_lv' => '387'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol4',
##&nbsp;                                                     'le_of_lv' => '128',
##&nbsp;                                                     'pe_for_lv' => '128'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol5',
##&nbsp;                                                     'le_of_lv' => '1024',
##&nbsp;                                                     'pe_for_lv' => '1024'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol6',
##&nbsp;                                                     'le_of_lv' => '387',
##&nbsp;                                                     'pe_for_lv' => '387'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol7',
##&nbsp;                                                     'le_of_lv' => '896',
##&nbsp;                                                     'pe_for_lv' => '896'
##&nbsp;                                                   },
##&nbsp;                                                   {
##&nbsp;                                                     'lv_name' => '/dev/vg00/lvol8',
##&nbsp;                                                     'le_of_lv' => '640',
##&nbsp;                                                     'pe_for_lv' => '640'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;            'physical_volumes' => {
##&nbsp;                                    'proactive_polling' => 'On',
##&nbsp;                                    'vg_name' => '/dev/vg00',
##&nbsp;                                    'allocated_pe' => '4188',
##&nbsp;                                    'autoswitch' => 'On',
##&nbsp;                                    'cur_lv' => '8',
##&nbsp;                                    'vgda' => '2',
##&nbsp;                                    'stale_pe' => '0',
##&nbsp;                                    'free_pe' => '153',
##&nbsp;                                    'pe_size_(mbytes)' => '8',
##&nbsp;                                    'io_timeout_(seconds)' => 'default',
##&nbsp;                                    'allocatable' => 'yes',
##&nbsp;                                    'total_pe' => '4341',
##&nbsp;                                    'pv_name' => '/dev/disk/disk3',
##&nbsp;                                    'pv_status' => 'available'
##&nbsp;                                  },
##&nbsp;            'physical_extents' => [
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00000',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00001',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00001',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00002',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00002',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00003',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00003',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00004',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00004',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00005',
##&nbsp;                                      'logical_vol' => '/dev/vg00/lvol1',
##&nbsp;                                      'physical_extent' => '00005',
##&nbsp;                                      'status' => 'current'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04190',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04191',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04192',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04193',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04194',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'logical_extent' => '00000',
##&nbsp;                                      'logical_vol' => '-',
##&nbsp;                                      'physical_extent' => '04195',
##&nbsp;                                      'status' => 'free'
##&nbsp;                                    },
##&nbsp;                                  ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay -l /dev/disk/disk3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/disk/disk3:LVM_Disk=yes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk_name' => '/dev/disk/disk3',
##&nbsp;            'lvm_disk' => 'yes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pvdisplay {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my $i = -1;
    my $j = -1;
    my $k = -1;
    my $head;
    my ( @array, @arr1, @arr2 );
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $cmd =~ /-l/ ) {
            if ( $line =~ /(\S+):(\w+)=(\w+)/ ) {
                $result_array[0]->{disk_name} = $1;
                $result_array[0]->{ to_key_name( item => $2 ) } = $3;
            }
            return \@result_array;
        }
        if ( $line =~ /\---\s(.*)\s\---/ ) {
            $head = to_key_name( item => $1 );
        }
        if ( $line =~ /LV\s+Name/ ) {
            @arr1 = split( /\s\s+/, $line );
        } elsif ( $line =~ /PE\s+Status/ ) {
            @arr2 = split( /\s+/, $line );
        }

        #Hash is used to make keys more redable
        my %hash = (
            "PE"     => "physical_extent",
            "Status" => "status",
            "LV"     => "logical_vol",
            "LE"     => "logical_extent"
        );
        @array = split( /\s\s+/, $line );
        if ( $#array == 1 ) {
            if ( $array[0] =~ /^\D+/ ) {
                $result_array[0]->{$head}
                    ->{ to_key_name( item => $array[0] ) } = $array[1];
            } elsif ( $array[0] =~ /^\d+/ ) {
                $i++;
                my @temp = split( /\s+/, $line );
                my $swap = $temp[2];
                $temp[2] = "-";
                $temp[3] = $swap;
                for ( my $j = 0; $j <= $#temp; $j++ ) {
                    $result_array[0]->{$head}->[$i]->{ $hash{ $arr2[$j] } }
                        = $temp[$j];
                }
            }
        } elsif ( $#array == 2 ) {
            if ( $array[0] =~ /^\d+/ ) {
                $j++;
                my @temp1 = split( /\s+/, $line );
                for ( my $i = 0; $i <= $#temp1; $i++ ) {
                    $result_array[0]->{$head}->[$j]->{ $hash{ $arr2[$i] } }
                        = $temp1[$i];
                }
            } elsif ( $array[0] =~ /^\/dev/ ) {
                $k++;
                for ( my $l = 0; $l <= $#array; $l++ ) {
                    $result_array[0]->{$head}->[$k]
                        ->{ to_key_name( item => $arr1[$l] ) } = $array[$l];
                }
            }
        }
    }
    return \@result_array;
} ## end sub pvdisplay

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /Illegal option/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /pvdisplay:\s\S+:\sNo such file or directory/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /More arguments required/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
