#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/setboot.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX setboot command
##
## @description
##    This is a library to parse the raw output of HPUX setboot command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "setboot" will be setboot
##
## @synopsis
##    use setboot;
##
##    setboot(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::setboot;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::setboot
##
## @summary Parses the raw output from the "setboot" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from setboot with any of its options as described in the
##    corresponding cdef file 'setboot.cdefs'
## @synopsis
##      setboot(%args);
## INPUTS TO METHOD CALL 'setboot' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-p ' </td> <td> 'primary-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'alternate-path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'auto-boot-sequence-flag' </td> <td> '0' </td> </tr>
## <tr> <td> '=' </td> <td> 'match' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ha-alternate-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'change-testname-for-next-system-boot' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'auto-search-sequence-flag' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'change-testname-for-following-boots' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = setboot -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Primary bootpath : 0/1/1/0.0x1.0x0 (/dev/rdisk/disk3)
##&nbsp;Alternate bootpath :
##&nbsp;
##&nbsp;Autoboot is ON (enabled)
##&nbsp;Autosearch is ON (enabled)
##&nbsp;
##&nbsp;TEST            CURRENT        SUPPORTED      DEFAULT        NEXT BOOT
##&nbsp;----            -------        ---------      -------        ---------
##&nbsp;all             on             yes            on             on
##&nbsp;  SELFTESTS     on             yes            on             on
##&nbsp;    early_cpu   on             yes            on             on
##&nbsp;    late_cpu    on             yes            on             on
##&nbsp;  FASTBOOT      on             yes            on             on
##&nbsp;    full_memory on             yes            on             on
##&nbsp;    PDH         on             yes            on             on
##&nbsp;  CEC           on             yes            on             on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bootpaths' => [
##&nbsp;                             {
##&nbsp;                               'alternate_bootpath' => undef,
##&nbsp;                               'primary_bootpath' => '0/1/1/0.0x1.0x0 (/dev/rdisk/disk3)'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'autosearch' => 'ON (enabled)',
##&nbsp;                           'autoboot' => 'ON (enabled)'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'status_table' => [
##&nbsp;                                {
##&nbsp;                                  'test' => 'all',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'SELFTESTS',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'early_cpu',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'late_cpu',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'FASTBOOT',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'full_memory',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'PDH',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'test' => 'CEC',
##&nbsp;                                  'current' => 'on',
##&nbsp;                                  'next_boot' => 'on',
##&nbsp;                                  'default' => 'on',
##&nbsp;                                  'supported' => 'yes'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = setboot -p 0/1/1/0.0x1.0x0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = setboot -b on
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = setboot -t early_cpu=on
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub setboot {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my $c = 0;
    my ( @head, @values, @array );
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /^TEST\s+CURRENT/ ) {
            @head = split( /\s\s+/, $line );
        } elsif ( $line =~ /^\w+\s+on|off|partial\s+yes|no|patial/ ) {
            @values = split( /\s+/, $line );
            for ( my $i = 0; $i <= $#head; $i++ ) {
                $result_array[0]->{status_table}->[$c]
                    ->{ to_key_name( item => $head[$i] ) } = $values[$i];
            }
            $c++;
        }
        if ( $line =~ /bootpath/ ) {
            @array = split( /\s+:/, $line );
            $array[1] =~ s/^\s+//;
            $result_array[0]->{bootpaths}->[0]
                ->{ to_key_name( item => $array[0] ) } = $array[1];
        } elsif ( $line =~ /(\w+)\s+\w+\s+(\w+\s+\(\w+\))/ ) {
            $result_array[0]->{flags}->[0]->{ to_key_name( item => $1 ) }
                = $2;
        }
    }
    return \@result_array;
} ## end sub setboot

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([uU]sage:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;

    } elsif ( $raw_output =~ /setboot:\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
