# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/swapinfo.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX swapinfo command
##
## @description
##    This is a library to parse the raw output of HPUX swapinfo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "swapinfo" will swapinfo()
##
## @synopsis
##    use swapinfo;
##
##    swapinfo(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::swapinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::swapinfo
##
## @summary Parses the raw output from the "swapinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from swapinfo with any of its options as described in the
##    corresponding cdef file 'swapinfo.cdefs'
## @synopsis
##      swapinfo(%args);
## INPUTS TO METHOD CALL 'swapinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'totals-only-paging-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'print-reserved-paging-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'print-primary-paging-area' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'print-warning-about-wasted-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'print-filesystem-paging-areas' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'display-values-in-mbytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'categorize-localfs-and-network-areas' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'print-device-paging-areas' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'print-memory-paging-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'show-all-device-paging-areas' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = swapinfo -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Primary swap (for next boot) :
##&nbsp;Primary swap for next boot was not set using swapctl()
##&nbsp;
##&nbsp;             Kb      Kb      Kb   PCT  START/      Kb
##&nbsp;TYPE      AVAIL    USED    FREE  USED   LIMIT RESERVE  PRI  NAME
##&nbsp;dev     4194304       0 4194304    0%       0       -    1  /dev/vg00/lvol2
##&nbsp;reserve       -  523316 -523316
##&nbsp;memory  3987456  828628 3158828   21%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pct_used' => '0%',
##&nbsp;            'kb_used' => '0',
##&nbsp;            'name' => '/dev/vg00/lvol2',
##&nbsp;            'kb_reserve' => '-',
##&nbsp;            'kb_free' => '4194304',
##&nbsp;            'type' => 'dev',
##&nbsp;            'kb_avail' => '4194304',
##&nbsp;            'pri' => '1',
##&nbsp;            'start_limit' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pct_used' => undef,
##&nbsp;            'kb_used' => '523316',
##&nbsp;            'name' => undef,
##&nbsp;            'kb_reserve' => undef,
##&nbsp;            'kb_free' => '-523316',
##&nbsp;            'type' => 'reserve',
##&nbsp;            'kb_avail' => '-',
##&nbsp;            'pri' => undef,
##&nbsp;            'start_limit' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pct_used' => '21%',
##&nbsp;            'kb_used' => '828628',
##&nbsp;            'name' => undef,
##&nbsp;            'kb_reserve' => undef,
##&nbsp;            'kb_free' => '3158828',
##&nbsp;            'type' => 'memory',
##&nbsp;            'kb_avail' => '3987456',
##&nbsp;            'pri' => undef,
##&nbsp;            'start_limit' => undef
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = swapinfo -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;             Mb      Mb      Mb   PCT  START/      Mb
##&nbsp;TYPE      AVAIL    USED    FREE  USED   LIMIT RESERVE  PRI  NAME
##&nbsp;dev        4096       0    4096    0%       0       -    1  /dev/vg00/lvol2
##&nbsp;reserve       -     510    -510
##&nbsp;memory     3894     809    3085   21%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pct_used' => '0%',
##&nbsp;            'mb_free' => '4096',
##&nbsp;            'mb_used' => '0',
##&nbsp;            'name' => '/dev/vg00/lvol2',
##&nbsp;            'mb_reserve' => '-',
##&nbsp;            'mb_avail' => '4096',
##&nbsp;            'type' => 'dev',
##&nbsp;            'pri' => '1',
##&nbsp;            'start_limit' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pct_used' => undef,
##&nbsp;            'mb_free' => '-510',
##&nbsp;            'mb_used' => '510',
##&nbsp;            'name' => undef,
##&nbsp;            'mb_reserve' => undef,
##&nbsp;            'mb_avail' => '-',
##&nbsp;            'type' => 'reserve',
##&nbsp;            'pri' => undef,
##&nbsp;            'start_limit' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pct_used' => '21%',
##&nbsp;            'mb_free' => '3085',
##&nbsp;            'mb_used' => '809',
##&nbsp;            'name' => undef,
##&nbsp;            'mb_reserve' => undef,
##&nbsp;            'mb_avail' => '3894',
##&nbsp;            'type' => 'memory',
##&nbsp;            'pri' => undef,
##&nbsp;            'start_limit' => undef
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = swapinfo -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;             Kb      Kb      Kb
##&nbsp;TYPE      AVAIL    USED    FREE
##&nbsp;reserve       -  522220 -522220
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kb_used' => '522220',
##&nbsp;            'kb_free' => '-522220',
##&nbsp;            'kb_avail' => '-',
##&nbsp;            'type' => 'reserve'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = swapinfo -M
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;             Kb      Kb      Kb   PCT
##&nbsp;TYPE      AVAIL    USED    FREE  USED
##&nbsp;memory  3987456  828628 3158828   21%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pct_used' => '21%',
##&nbsp;            'kb_used' => '828628',
##&nbsp;            'kb_free' => '3158828',
##&nbsp;            'kb_avail' => '3987456',
##&nbsp;            'type' => 'memory'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = swapinfo -q
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;8181760
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'value' => '8181760'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub swapinfo {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head1, @head2 );
    my $c = 0;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /(Kb|Mb).*/ ) {
            @head1 = split( /\s+/, $lines[$i] );
            @head2 = split( /\s+/, $lines[ $i + 1 ] );
        }
        if (   $lines[$i] =~ /\w+\s+\d*\s+\d*.*/
            && $lines[$i] !~ /(Kb|TYPE|Primary|Mb).*/ ) {
            my @values = split( /\s+/, $lines[$i] );
            $result_array[$c]->{ to_key_name( item => $head2[0] ) }
                = $values[0];
            for ( my $j = 0; $j <= $#head1; $j++ ) {
                my $res = $head1[$j] . "_" . $head2[ $j + 1 ];
                $result_array[$c]->{ to_key_name( item => $res ) }
                    = $values[ $j + 1 ];
            }
            if ( $cmd =~ /(-s|-m|-t|-a|-d|-f|-n|-w)/ ) {
                $result_array[$c]->{ to_key_name( item => $head2[7] ) }
                    = $values[7];
                $result_array[$c]->{ to_key_name( item => $head2[8] ) }
                    = $values[8];

            }

            $c++;
        }

    }
    if ( $cmd =~ /(-q)/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            $result_array[$c]->{value} = $lines[$i];
        }
    }
    return \@result_array;
} ## end sub swapinfo

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:|swapinfo:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
       }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

