#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @description
##    This is a library to parse the raw output of listprincs command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "listprincs" will be listprincs)
##
## @synopsis
##    use listprincs;
##
##    listprincs(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

#
package NACL::APISet::Host::CLI::Kerberos::default::listprincs;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;
no warnings 'redefine';
use Data::Dumper;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $command = $resp->get_actual_command();
    my $error_message;
    my $error = undef;
   if ( $raw_output =~ /Usage\s*:/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::listprincs
##
## @summary Parses the raw output from the "listprincs" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from listprincs with any of its options as described in the 
##    corresponding cdef file 'kadmin.cdefs'
## @synopsis
##      listprincs(%args);
## INPUTS TO METHOD CALL 'listprincs' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'principal_name' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = listprincs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;K/M@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;amitb@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;changepw/vmsun66-132.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;kadmin/admin@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;kadmin/changepw@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;kadmin/history@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;kadmin/vmsun66-132.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;kiprop/vmsun66-132.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;krbtgt/MITKDC01.NFSDEV.NB.NETAPP.COM@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;nfs/lnx219-104.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;nfs/lnx219-41.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;nfs/lnx219-42.lab.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;nfs/lnx219-42.lab.eng.btc.netapp.in@MITKDC01.NFSQA.NB.NETAPP.COM
##&nbsp;nfs/vineethk-vsim3-d2.sim.eng.btc.netapp.in@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;root/admin@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;testu1@MITKDC01.NFSDEV.NB.NETAPP.COM
##&nbsp;  
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'K',
##&nbsp;            'instance' => 'M'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'amitb',
##&nbsp;            'instance' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'changepw',
##&nbsp;            'instance' => 'vmsun66-132.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'kadmin',
##&nbsp;            'instance' => 'admin'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'kadmin',
##&nbsp;            'instance' => 'changepw'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'kadmin',
##&nbsp;            'instance' => 'history'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'kadmin',
##&nbsp;            'instance' => 'vmsun66-132.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'kiprop',
##&nbsp;            'instance' => 'vmsun66-132.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'krbtgt',
##&nbsp;            'instance' => 'MITKDC01.NFSDEV.NB.NETAPP.COM'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'nfs',
##&nbsp;            'instance' => 'lnx219-104.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'nfs',
##&nbsp;            'instance' => 'lnx219-41.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'nfs',
##&nbsp;            'instance' => 'lnx219-42.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSQA.NB.NETAPP.COM',
##&nbsp;            'primary' => 'nfs',
##&nbsp;            'instance' => 'lnx219-42.lab.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'nfs',
##&nbsp;            'instance' => 'vineethk-vsim3-d2.sim.eng.btc.netapp.in'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'root',
##&nbsp;            'instance' => 'admin'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'realm' => 'MITKDC01.NFSDEV.NB.NETAPP.COM',
##&nbsp;            'primary' => 'testu1',
##&nbsp;            'instance' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub listprincs {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, );
    foreach my $line (@lines) {
        my %parsed_output;
        next if ( $line =~ /listprincs/ );
        if ( $line =~ /(\w+)\/(.*)@(.*)/ ) {

           #To parse output that looks like: K/M@MITKDC01.NFSDEV.NB.NETAPP.COM
            $parsed_output{primary}  = $1;
            $parsed_output{instance} = $2;
            $parsed_output{realm}    = $3;
        } elsif ( $line =~ /(\w+)@(.*)/ ) {

        #To parse output that looks like: amitb@MITKDC01.NFSDEV.NB.NETAPP.COM.
            $parsed_output{primary} = $1;
            $parsed_output{realm}   = $2;
            $parsed_output{instance} = '';
        }
        push( @result_array, \%parsed_output );
    }
    return \@result_array;
}

1;
