#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for Linux CLI commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::CLI::Linux;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::CLI);

#
# Module imports - APISet
#
use NACL::APISet::Session::CLI::Linux;

#
##Exceptions
##
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

sub shutdown {
    my ( $self, %opts ) = @_;
    my $return_value = "";

    my $major_version = $self->{version_manager}->get_version_attribute( attribute => 'os_revision' )->{major};
    my $minor_version = $self->{version_manager}->get_version_attribute( attribute => 'os_revision' )->{minor};

    if ( ( !$major_version ) || ( !$minor_version ) ) {
        $major_version = $self->{hostobj}->{os_major};
        $minor_version = $self->{hostobj}->{os_minor};
    }

    if ( ( (defined $opts{time}) && ( ($opts{time} eq 'now') || ($opts{time} == 0) ) ) && ( ( $major_version . "." . $minor_version ) ge 7.0 ) ) {
        try {
	    #as per burt946275, RHEL versions >=7.0 shuts down ungracefully hence invoking shutdown api with a lesser timeout value and catching the exception and suppressing it
	    #as per burt980018, increasing 'connectrec-timeout' from 1 to 10 since some RHEL7.X configuration clients get resync error and hangs intermittently during shutdown
            $return_value = $self->_invoke( method => 'shutdown', %opts, 'connectrec-timeout' => 10 );
        }catch NATE::BaseException with {
            my $exception = shift;
            if ( ( $exception->text() =~ /Unexpected EOF while reading output from command/i ) || ( $exception->text() =~ /Timeout waiting for command to complete/i ) ) {
                # Expected, so suppress
            } elsif ( $exception->text() =~ /Could not write to connection/i ){
		NATE::BaseException->throw( $exception->text() )
	    }else{
		NACL::APISet::Exceptions::ResponseException->throw( text => $exception->text(), response_object => $exception->get_response_object() )
	    }
        };
    } else {
        $return_value = $self->_invoke( method => 'shutdown', %opts );
    }
    return $return_value;
}

sub reboot {
    my ( $self, %opts ) = @_;
    my $return_value = "";

    my $major_version = $self->{version_manager}->get_version_attribute( attribute => 'os_revision' )->{major};
    my $minor_version = $self->{version_manager}->get_version_attribute( attribute => 'os_revision' )->{minor};

    if ( ( !$major_version ) || ( !$minor_version ) ) {
        $major_version = $self->{hostobj}->{os_major};
        $minor_version = $self->{hostobj}->{os_minor};
    }

    if ( ( $major_version . "." . $minor_version ) ge 7.0 ) {
        try {
            #as per burt1086892, RHEL versions >=7.0 reboots ungracefully hence invoking 'reboot' api with a lesser timeout value and catching the exception and suppressing it
            $return_value = $self->_invoke( method => 'reboot', %opts, 'connectrec-timeout' => 10 );
        }catch NATE::BaseException with {
            my $exception = shift;
            if ( ( $exception->text() =~ /Unexpected EOF while reading output from command/i ) || ( $exception->text() =~ /Timeout waiting for command to complete/i ) ) {
                # Expected, so suppress
            } elsif ( $exception->text() =~ /Could not write to connection/i ){
                NATE::BaseException->throw( $exception->text() )
            }else{
                NACL::APISet::Exceptions::ResponseException->throw( text => $exception->text(), response_object => $exception->get_response_object() )
            }
        };
    } else {
        $return_value = $self->_invoke( method => 'reboot', %opts );
    }
    return $return_value;

}

=head1 NAME

NACL::APISet::Host::CLI::Linux - API Set for Linux CLI commands

=head1 SYNOPSIS

use NACL::APISet::Host::CLI::Linux;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI::Linux> is used to connect to and send CLI commands
to the Linux Host.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::CLI::Linux>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Host::CLI::Linux->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Host::CLI for details.

=back

=item Returns

A reference to a new API Set.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
