#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/chkconfig.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux chkconfig command
##
## @description
##    This is a library to parse the raw output of Linux chkconfig command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "chkconfig" will be chkconfig
##
## @synopsis
##    use chkconfig;
##
##    chkconfig(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::chkconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::chkconfig_list
##
## @summary Parses the raw output from the "chkconfig --list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chkconfig_list with any of its options as described in the
##    corresponding cdef file 'chkconfig.cdefs'
## @synopsis
##      chkconfig_list(%args);
## INPUTS TO METHOD CALL 'chkconfig_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = chkconfig --list rawdevices
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;rawdevices     	0:off	1:off	2:off	3:on	4:on	5:on	6:off
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'levels' => {
##&nbsp;                          '6' => 'off',
##&nbsp;                          '4' => 'on',
##&nbsp;                          '1' => 'off',
##&nbsp;                          '3' => 'on',
##&nbsp;                          '0' => 'off',
##&nbsp;                          '2' => 'off',
##&nbsp;                          '5' => 'on'
##&nbsp;                        },
##&nbsp;            'service' => 'rawdevices'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = chkconfig --list daytime-stream
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;daytime-stream 	on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'xinetd_based' => 1,
##&nbsp;            'value' => 'on',
##&nbsp;            'service' => 'rlogin'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub chkconfig_list {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $c = 0;
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    for my $line (@lines) {
        if ( $line =~ /(\w+\-*\w*)\s+\d+:/ ) {
            $result_array[$c]->{service} = $1;
            my @arr = split( /\s+/, $line );
            foreach my $t (@arr) {
                if ( $t =~ /:/ ) {
                    my @temp = split( /:/, $t );
                    $result_array[$c]->{levels}->{ $temp[0] } = $temp[1];
                }
            }
            $c++;
        } elsif ( $line =~ /(\w+\-*\w*):*\s+(\w+)/
            && $line ne 'xinetd based service:' ) {
            $result_array[$c]->{xinetd_based} = 1;
            $result_array[$c]->{service}      = $1;
            $result_array[$c]->{value}        = $2;
            $c++;
        }

    }
    return \@result_array;

} ## end sub chkconfig_list

## @name ::chkconfig_add
##
## @summary Parses the raw output from the "chkconfig --add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chkconfig_add with any of its options as described in the
##    corresponding cdef file 'chkconfig.cdefs'
## @synopsis
##      chkconfig_add(%args);
## INPUTS TO METHOD CALL 'chkconfig_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = chkconfig --add dovecot
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub chkconfig_add {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub chkconfig_add

## @name ::chkconfig_del
##
## @summary Parses the raw output from the "chkconfig --del" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chkconfig_del with any of its options as described in the
##    corresponding cdef file 'chkconfig.cdefs'
## @synopsis
##      chkconfig_del(%args);
## INPUTS TO METHOD CALL 'chkconfig_del' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = chkconfig --del dovecot
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub chkconfig_del {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub chkconfig_del

## @name ::chkconfig
##
## @summary Parses the raw output from the "chkconfig" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chkconfig with any of its options as described in the
##    corresponding cdef file 'chkconfig.cdefs'
## @synopsis
##      chkconfig(%args);
## INPUTS TO METHOD CALL 'chkconfig' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--level' </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = chkconfig --level 1 acpid on
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub chkconfig {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub chkconfig

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(usage:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(chkconfig).* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;

