#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux cmp command
##
## @description
##    This is a library to parse the raw output of Linux cmp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "cmp" will be cmp
##
## @synopsis
##    use cmp;
##
##    cmp(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::cmp;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /$command.*:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}

## @name ::cmp
##
## @summary Parses the raw output from the "cmp" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from cmp with any of its options as described in the 
##    corresponding cdef file 'cmp.cdefs'
## @synopsis
##      cmp(%args);
## INPUTS TO METHOD CALL 'cmp' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file2' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file1' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'skip' </td> <td> '0' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'print-bytes' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'byte-num-val' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'silent' </td> <td> '1' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'byte-limit' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cmp -l -b abcd.txt defg.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; 9 141 a    144 d
##&nbsp;10 142 b    145 e
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'actual_value_firstFile' => 'a',
##&nbsp;            'differing_byte' => '9',
##&nbsp;            'actual_value_SecondFile' => 'd',
##&nbsp;            'value_firstFile' => '141',
##&nbsp;            'value_SecondFile' => '144'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'actual_value_firstFile' => 'b',
##&nbsp;            'differing_byte' => '10',
##&nbsp;            'actual_value_SecondFile' => 'e',
##&nbsp;            'value_firstFile' => '142',
##&nbsp;            'value_SecondFile' => '145'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cmp -l abcd.txt defg.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; 9 141 144
##&nbsp;10 142 145
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'differing_byte' => '9',
##&nbsp;            'value_firstFile' => '141',
##&nbsp;            'value_SecondFile' => '144'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'differing_byte' => '10',
##&nbsp;            'value_firstFile' => '142',
##&nbsp;            'value_SecondFile' => '145'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cmp -b abcd.txt defg.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;abcd.txt defg.txt differ: byte 9, line 1 is 141 a 144 d
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'differing_value_File1_File2' => '141 a 144 d',
##&nbsp;            'line_no' => '1',
##&nbsp;            'differing_byte' => '9'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cmp {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
	print " Error level is".$error_level;
    my $error;  
    if($error_level>1) {
        $error = _check_error(raw_output=>$output,command=>"cmp",
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
    my @lines = get_lines(raw_output=>$output,first_line=>"");
    foreach my $line (@lines)  {
        my %hash;
		if($command =~ /-b/ && $command =~ /-l/) {
			if($line =~ /cmp/){
				$hash{'end_of_a_file'} =$line ;
				push (@result_array, \%hash);
			} else {
				my @values= split(' ', $line);
				if(@values==4) {
					$hash{'differing_byte'} = $values[0];
					if($values[1]==40) {
						$hash{'actual_value_firstfile'} = " ";
						$hash{'value_firstfile'} = $values[1];
						$hash{'value_secondfile'} = $values[2];
						$hash{'actual_value_secondfile'} = $values[3];
					} elsif($values[3]==40) {
						$hash{'actual_value_secondfile'} = " ";
						$hash{'value_firstfile'} = $values[1];
						$hash{'value_secondfile'} = $values[3];
						$hash{'actual_value_firstfile'} = $values[2];
					} else {
						$hash{'actual_value_firstfile'} = $values[2];
						$hash{'actual_value_secondfile'} = $values[4];
					}
					push (@result_array, \%hash);
				} else {
					$hash{'differing_byte'} = $values[0];
					$hash{'value_firstFile'} = $values[1];
					$hash{'actual_value_firstfile'} = $values[2];
					$hash{'value_secondfile'} = $values[3];
					$hash{'actual_value_secondfile'} = $values[4];
					push (@result_array, \%hash);
				}
				
            }
		} elsif ($command =~ /-l/){
			if($line =~ /cmp/){
				$hash{'end_of_a_file'} =$line ;
				push (@result_array, \%hash);
			} else {
				my @values1= split(' ', $line);
				$hash{'differing_byte'} = $values1[0];
				$hash{'value_firstfile'} = $values1[1];
				$hash{'value_secondfile'} = $values1[2];
				push (@result_array, \%hash);
			}
		} else {
			if($line =~ /.* byte\s(\d).* line\s(\d)$/) {
				$hash{'differing_byte'} =$1;
				$hash{'line_no'} =$2;
				push (@result_array, \%hash);
			} elsif ($line =~ /.* byte\s(\d).* line\s(\d)\sis\s(.*)/) {
				$hash{'differing_byte'} =$1;
				$hash{'line_no'} =$2;
				$hash{'differing_value_file1_file2'} =$3;
				push (@result_array, \%hash);
			}
		}
	}
    return \@result_array; #to be updated to return a reference
}

1;
