## @summary Routines to parse Linux compare ( /usr/software/ctest/bin/compare 
##                                            command output)
##
## @description
##    This is a library to parse the raw output of the 
##     /usr/software/ctest/bin/compare command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::compare
##
##    compare(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::compare;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
        
## @name ::compare
##
## @summary Parses the raw output from the "/usr/software/test/bin/compare" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from compare with any of its options as described in the 
##    corresponding cdef file 'compare.cdefs'
## @synopsis
##      compare(%args);
## INPUTS TO METHOD CALL 'compare' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'tree2' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tree1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/software/test/bin/compare x y
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_time' => '00.000',
##&nbsp;            'differing_files' => [
##&nbsp;                                   'f.txt'
##&nbsp;                                 ],
##&nbsp;            'files_in_tree2_only' => [
##&nbsp;                                       'c',
##&nbsp;                                       'd'
##&nbsp;                                     ],
##&nbsp;            'files_in_tree1_only' => [
##&nbsp;                                       'b',
##&nbsp;                                       'a'
##&nbsp;                                     ],
##&nbsp;            'completion_time' => '00.000',
##&nbsp;            'user_time' => '00.000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub compare {
    my %args = @_;
    
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,response_object => $resp);
    }

    my @lines = get_lines(raw_output=>$output, first_line=>"Comparison completed in", 
                          response_object=>$resp);
                          
    my %hash;
    my $line = shift @lines;
    $line =~ /Comparison completed in ([\.\d]+) \(([\.\d]+) user, ([\.\d]+) system\)/;
    $hash{'completion_time'} = $1;
    $hash{'user_time'} = $2;
    $hash{'system_time'} = $3;
    
    $line = shift @lines;
    if ($line =~ /files in .+ but not .+:$/) {
        $hash{'files_in_tree1_only'} = [];
        $hash{'files_in_tree2_only'} = [];
        $hash{'differing_files'} = [];
        
        my $i;
        for ($i = 0; $lines[$i] !~ /files .+ but not.+:$/; $i++) {
            push @{$hash{'files_in_tree1_only'}}, $lines[$i];
        }
        
        $i++;
        
        for (; $lines[$i] !~ /files differed:/; $i++) {
            push @{$hash{'files_in_tree2_only'}}, $lines[$i];
        }
        
        $i++;
        
        for (; $i < @lines; $i++) {
            push @{$hash{'differing_files'}}, $lines[$i];
        }
    }
    
    return [ \%hash ];
}

1;
