#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux dd command output
##
## @description
##    This is a library to parse the raw output of Linux dd command.
##
## @synopsis
##    use dd;
##
##    dd(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::dd;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid)|([Uu]nrecognized)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(unknown)/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}

## @name ::dd
##
## @summary Parses the raw output from "dd" command
##
## @description
##    This method can be used for parsing the output from "dd" command
##    called with arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/dd.cdefs
##
## @synopsis
##    dd (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of the dd command

sub dd {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @lines = get_lines(raw_output=>$output, first_line=>"", 
                          response_object=>$resp);
    
    my ($line, $name, $value, %hash);
    $line = $lines[$#lines];
    
    if($line =~ /copied,/) {
        pop(@lines);
        my @items = split(/,\s+/,$line);
        foreach my $item(@items) {
            my @words = split(/\s+/,$item);
            $name = pop(@words);
            $value = join(' ',@words);
            $hash{$name} = $value;
        }
    }
    
    my $count = 0;
    while($count<2) {
        $line = pop(@lines);
        if($line =~ /(.+)\s+(records.+)/) {
            $name = $2;
            $value = $1;
            $name=~ s/ /_/g;
            $hash{$name} = $value;
        }
        $count++;
    }
    
    $hash{'rest_of_output'} = join("\n",@lines);
    my @result_array = (\%hash);
    return \@result_array;
}

1;
