#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/dmidecode.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Linux dmidecode commands
##
## @description
##    This is a library to parse the raw output of Linux dmidecode commands.
##
## @synopsis
##    use dmidecode;
##
## dmidecode(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::dmidecode;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{response_object};
    my $error_message;
    my $error;
    if ( $output =~ /invalid option/i ) {
        $error         = "InvalidParam";
        $error_message = "Invalid options are used in the command";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

## @name ::dmidecode
##
## @summary Parses the raw output from the "dmidecode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dmidecode with any of its options as described in the
##    corresponding cdef file 'dmidecode.cdefs'
## @synopsis
##      dmidecode(%args);
## INPUTS TO METHOD CALL 'dmidecode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'dump' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--from-dump' </td> <td> 'dump-from' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'string' </td> <td> '0' </td> </tr>
## <tr> <td> '--dump-bin' </td> <td> 'dump-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dev-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = dmidecode -V
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;2.9
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => '2.9'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dmidecode -q
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# dmidecode 2.9
##&nbsp;SMBIOS 2.4 present.
##&nbsp;45 structures occupying 1607 bytes.
##&nbsp;Table at 0x7FF6C000.
##&nbsp;
##&nbsp;Handle 0x0000, DMI type 0, 24 bytes
##&nbsp;BIOS Information
##&nbsp;	Vendor: IBM
##&nbsp;	Version: IBM BIOS Version 1.35-[C0E135AUS-1.35]-
##&nbsp;	Release Date: 02/26/2007
##&nbsp;	Address: 0xE4D80
##&nbsp;	Runtime Size: 111232 bytes
##&nbsp;	ROM Size: 512 kB
##&nbsp;	Characteristics:
##&nbsp;		PCI is supported
##&nbsp;		PNP is supported
##&nbsp;		BIOS is upgradeable
##&nbsp;		BIOS shadowing is allowed
##&nbsp;		ESCD support is available
##&nbsp;		Boot from CD is supported
##&nbsp;		Selectable boot is supported
##&nbsp;		BIOS ROM is socketed
##&nbsp;		EDD is supported
##&nbsp;
##&nbsp;Handle 0x0001, DMI type 1, 27 bytes
##&nbsp;System Information
##&nbsp;	Manufacturer: IBM
##&nbsp;	Product Name: IBM System x3455-[798451A]-
##&nbsp;	Version: Not Specified
##&nbsp;	Serial Number: 99A2588
##&nbsp;	UUID: 00D31C51-9964-0010-8BB7-938516FA2FB5
##&nbsp;	Wake-up Type: AC Power Restored
##&nbsp;	SKU Number: Not Specified
##&nbsp;	Family: Not Specified
##&nbsp;
##&nbsp;End Of Table
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'rom_size' => '512 kB',
##&nbsp;            'occupied' => '1607 bytes.',
##&nbsp;            'release_date' => '02/26/2007',
##&nbsp;            'version' => 'IBM BIOS Version 1.35-[C0E135AUS-1.35]-',
##&nbsp;            'information' => 'BIOS Information',
##&nbsp;            'table' => '0x7FF6C000.',
##&nbsp;            'dmidecode' => '2.9',
##&nbsp;            'structures' => '45',
##&nbsp;            'smbios' => '2.4 present.',
##&nbsp;            'handle' => '0x0000',
##&nbsp;            'characteristics' => [
##&nbsp;                                   'PCI is supported',
##&nbsp;                                   'PNP is supported',
##&nbsp;                                   'BIOS is upgradeable',
##&nbsp;                                   'BIOS shadowing is allowed',
##&nbsp;                                   'ESCD support is available',
##&nbsp;                                   'Boot from CD is supported',
##&nbsp;                                   'Selectable boot is supported',
##&nbsp;                                   'BIOS ROM is socketed',
##&nbsp;                                   'EDD is supported',
##&nbsp;                                 ],
##&nbsp;            'space_occupied' => '24 bytes',
##&nbsp;            'runtime_size' => '111232 bytes',
##&nbsp;            'address' => '0xE4D80',
##&nbsp;            'dmi_type' => '0',
##&nbsp;            'vendor' => 'IBM'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => '99A2588',
##&nbsp;            'wake_up_type' => 'AC Power Restored',
##&nbsp;            'version' => 'Not Specified',
##&nbsp;            'information' => 'System Information',
##&nbsp;            'uuid' => '00D31C51-9964-0010-8BB7-938516FA2FB5',
##&nbsp;            'sku_number' => 'Not Specified',
##&nbsp;            'handle' => '0x0001',
##&nbsp;            'manufacturer' => 'IBM',
##&nbsp;            'space_occupied' => '27 bytes',
##&nbsp;            'dmi_type' => '1',
##&nbsp;            'family' => 'Not Specified',
##&nbsp;            'product_name' => 'IBM System x3455-[798451A]-'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dmidecode -t baseboard
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# dmidecode 2.9
##&nbsp;SMBIOS 2.4 present.
##&nbsp;
##&nbsp;Handle 0x0002, DMI type 2, 9 bytes
##&nbsp;Base Board Information
##&nbsp;	Manufacturer: IBM
##&nbsp;	Product Name: 7984
##&nbsp;	Version: Not Specified
##&nbsp;	Serial Number: #A123456789
##&nbsp;
##&nbsp;Handle 0x000C, DMI type 10, 6 bytes
##&nbsp;On Board Device Information
##&nbsp;	Type: Other
##&nbsp;	Status: Disabled
##&nbsp;	Description: IBM Automatic Server Restart - Integrated System Management Processor
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => '#A123456789',
##&nbsp;            'version' => 'Not Specified',
##&nbsp;            'information' => 'Base Board Information',
##&nbsp;            'dmidecode' => '2.9',
##&nbsp;            'smbios' => '2.4 present.',
##&nbsp;            'handle' => '0x0002',
##&nbsp;            'manufacturer' => 'IBM',
##&nbsp;            'space_occupied' => '9 bytes',
##&nbsp;            'dmi_type' => '2',
##&nbsp;            'product_name' => '7984'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'handle' => '0x000C',
##&nbsp;            'status' => 'Disabled',
##&nbsp;            'information' => 'On Board Device Information',
##&nbsp;            'space_occupied' => '6 bytes',
##&nbsp;            'type' => 'Other',
##&nbsp;            'dmi_type' => '10',
##&nbsp;            'description' => 'IBM Automatic Server Restart - Integrated System Management Processor'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dmidecode -s chassis-type
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Main Server Chassis
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'chassis_type' => 'Main Server Chassis'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dmidecode -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# dmidecode 2.9
##&nbsp;SMBIOS 2.4 present.
##&nbsp;45 structures occupying 1607 bytes.
##&nbsp;Table at 0x7FF6C000.
##&nbsp;
##&nbsp;Handle 0x0000, DMI type 0, 24 bytes
##&nbsp;	Header and Data:
##&nbsp;		00 18 00 00 01 02 D8 E4 03 07 80 DA CB 3F 00 00
##&nbsp;		00 00 03 01 FF FF FF FF
##&nbsp;	Strings:
##&nbsp;		49 42 4D 00
##&nbsp;		"IBM"
##&nbsp;		49 42 4D 20 42 49 4F 53 20 56 65 72 73 69 6F 6E
##&nbsp;		20 31 2E 33 35 2D 5B 43 30 45 31 33 35 41 55 53
##&nbsp;		2D 31 2E 33 35 5D 2D 00
##&nbsp;		"IBM BIOS Version 1.35-[C0E135AUS-1.35]-"
##&nbsp;		30 32 2F 32 36 2F 32 30 30 37 00
##&nbsp;		"02/26/2007"
##&nbsp;
##&nbsp;Handle 0x0001, DMI type 1, 27 bytes
##&nbsp;	Header and Data:
##&nbsp;		01 1B 01 00 01 02 00 03 00 D3 1C 51 99 64 00 10
##&nbsp;		8B B7 93 85 16 FA 2F B5 08 00 00
##&nbsp;	Strings:
##&nbsp;		49 42 4D 00
##&nbsp;		"IBM"
##&nbsp;		49 42 4D 20 53 79 73 74 65 6D 20 78 33 34 35 35
##&nbsp;		2D 5B 37 39 38 34 35 31 41 5D 2D 00
##&nbsp;		"IBM System x3455-[798451A]-"
##&nbsp;		39 39 41 32 35 38 38 00
##&nbsp;		"99A2588"
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'occupied' => '1607 bytes.',
##&nbsp;            'table' => '0x7FF6C000.',
##&nbsp;            'dmidecode' => '2.9',
##&nbsp;            'strings' => [
##&nbsp;                           '49 42 4D 00',
##&nbsp;                           '"IBM"',
##&nbsp;                           '49 42 4D 20 42 49 4F 53 20 56 65 72 73 69 6F 6E',
##&nbsp;                           '20 31 2E 33 35 2D 5B 43 30 45 31 33 35 41 55 53',
##&nbsp;                           '2D 31 2E 33 35 5D 2D 00',
##&nbsp;                           '"IBM BIOS Version 1.35-[C0E135AUS-1.35]-"',
##&nbsp;                           '30 32 2F 32 36 2F 32 30 30 37 00',
##&nbsp;                           '"02/26/2007"'
##&nbsp;                         ],
##&nbsp;            'structures' => '45',
##&nbsp;            'smbios' => '2.4 present.',
##&nbsp;            'header_and_data' => [
##&nbsp;                                   '00 18 00 00 01 02 D8 E4 03 07 80 DA CB 3F 00 00',
##&nbsp;                                   '00 00 03 01 FF FF FF FF'
##&nbsp;                                 ],
##&nbsp;            'handle' => '0x0000',
##&nbsp;            'space_occupied' => '24 bytes',
##&nbsp;            'dmi_type' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'header_and_data' => [
##&nbsp;                                   '01 1B 01 00 01 02 00 03 00 D3 1C 51 99 64 00 10',
##&nbsp;                                   '8B B7 93 85 16 FA 2F B5 08 00 00'
##&nbsp;                                 ],
##&nbsp;            'handle' => '0x0001',
##&nbsp;            'space_occupied' => '27 bytes',
##&nbsp;            'dmi_type' => '1',
##&nbsp;            'strings' => [
##&nbsp;                           '49 42 4D 00',
##&nbsp;                           '"IBM"',
##&nbsp;                           '49 42 4D 20 53 79 73 74 65 6D 20 78 33 34 35 35',
##&nbsp;                           '2D 5B 37 39 38 34 35 31 41 5D 2D 00',
##&nbsp;                           '"IBM System x3455-[798451A]-"',
##&nbsp;                           '39 39 41 32 35 38 38 00',
##&nbsp;                           '"99A2588"'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dmidecode {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my ( $key );
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }
    my @lines
        = get_lines( 'raw_output' => $output, response_object => $resp );
    my %hash = ();
    my $c    = -1;
    if ( $command =~ /-s (.*)/ ) {
        my $t = $1;
        $t =~ s/-/_/g;
        $result_array[0]->{ to_key_name( item => $t ) } = $lines[0];
        return \@result_array;
    } elsif ( $command =~ /-V/ ) {
        $result_array[0]->{version} = $lines[0];
        return \@result_array;
    }

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        $lines[$i] =~ s/\.$//;
        if ( $lines[$i] =~ /dmidecode\s(.*)/ ) {
            $result_array[0]->{dmidecode} = $1;
        } elsif ( $lines[$i] =~ /SMBIOS\s(.*)/ ) {
            $result_array[0]->{smbios} = $1;
        } elsif ( $lines[$i] =~ /(\d+)\s+structures occupying\s+(\d+\s+.*)/ )
        {
            $result_array[0]->{structures} = $1;
            $result_array[0]->{occupied}   = $2;
        } elsif ( $lines[$i] =~ /Table at (.*)/ ) {
            $result_array[0]->{table} = $1;
        }
        if ( $lines[$i] =~ /Handle\s(.*),\s+(\w+\s+\w+)\s+(\d+),\s+(.*)/ ) {
            $c++;
            $result_array[$c]->{handle} = $1;
            $result_array[$c]->{ to_key_name( item => $2 ) } = $3;
            $result_array[$c]->{space_occupied} = $4;
            if ( $lines[ $i + 1 ] !~ /\:$/ ) {
                $result_array[$c]->{information} = $lines[ $i + 1 ];
                $i = $i + 2;
            }
        } elsif ( $lines[$i] !~ /\:$/
            && $lines[$i] =~ /^([A-Z]+[a-z]*\s+){1,}[A-Z]+[a-z]*$/ ) {
            $c++;
            $result_array[$c]->{information} = $lines[$i];
        }
        if ( $lines[$i] =~ /:\s+/ ) {
            my @temp = split( /:\s+/, $lines[$i] );
            $temp[0] =~ s/-/_/;
            $result_array[$c]->{ to_key_name( item => $temp[0] ) } = $temp[1];
        } elsif ( $lines[$i] =~ /(.*):$/ ) {
            my $head = $1;
            $head = to_key_name( item => $head );
            $head =~ s/-/_/;
            $i++;
            my $sub_c = 0;
            while (
                $lines[$i] !~ /Handle|:|^([A-Z]+[a-z]*\s+){1,}[A-Z]+[a-z]*$/
                && $lines[$i] ) {
                $lines[$i] =~ s/"//;
                $result_array[$c]->{$head}->[$sub_c] = $lines[$i];
                $i++;
                $sub_c++;
            }
            $i--;
        }
    }
    return \@result_array;    #to be updated to return a reference
} ## end sub dmidecode

1;
