#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse do_mount command
##
## @description
##    This is a library to parse the raw output of the do_mount command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "do_mount" will be do_mount
##
## @synopsis
##    use  NACL::APISet::Host::CLI::Linux::Redhat::do_mount;
##
##    do_mount(%args);
##
## @author Eric Osgood (osgood@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
################################################################################

package NACL::APISet::Host::CLI::Linux::Redhat::do_mount;

###################
# Everytime
use strict;
use warnings;

###################
# Module includes
use NACL::APISet::Response::CLI::ParserUtils;

####################
# Exceptions
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::TimeoutException qw(:try);

###################
# Sub Routines    #
###################

## @name ::do_mount
##
## @summary Parses the raw output from "do_mount" command
##
## @description
##    This method can be used for parsing the output from "do_mount" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/do_mount.cdefs
##
## @synopsis
##    do_mount (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing hash reference(s) which contains the
##  parsed output data from the output of the mount comman

sub do_mount {
    my %args = @_;
    my (@res_arr);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $exception;

    # do_mount either failed with a timeout/command line failure or succeeded
    if ($error_level) {
	my $message = ("Command: $command\n"
		       ."Output from command:\n"
		       ."  ".indent(2,$output)."\n");
	if ($output =~ /timed out/){
	    $exception = NACL::APISet::Exceptions::TimeoutException->new
		("Timed out attempting to mount:\n$message");
	} else {
	    $exception = NACL::APISet::Exceptions::CommandFailedException->new
		(text=>"Error attempting to mount:\n$message",
		 response_object=>$resp);
	}
	$exception->throw();
    } else {
	# do_mount returned OK, parse its ouput
        my %hash;
	if ($output=~ /(.*) on (.*) type (.*) (.*)/){
            $hash{'source_path'} = $1;
            $hash{'mount_point'} = $2;
            $hash{'type'} = $3;
            $hash{'options'} = $4;
            $hash{'result'} = 'mounted';
            push (@res_arr, \%hash);
	}
    }
    return \@res_arr;
}

###############################################################################
# Subroutine: indent
#
# Indent by a given amount all but the first line with $amount spaces
#
# Arguments: Scalar String
#
# Returns: Scalar String
sub indent($$) {
    my ( $amount, $text ) = @_;
    chomp $text;
    return join( "\n" . ( " " x $amount ), split( /\n/, $text ) ) . "\n";
}


###################
# End Subroutines #
###################

1;
