#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ethtool commands
##
## @description
##    This is a library to parse the raw output of Linux ethtool commands.
##
## @synopsis
##    use ethtool;
##
## ethtool(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ethtool;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /(.* Command not found.*)/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid Parameter is passed";
    } elsif ( $output =~ /((.*No such file.*) | (.*invalid.*) | (.*No such device.*) | (.*no offload info available.*) | (.*no data.*))/si
        || $output =~ /ethtool:\s+bad\s+command\s+line.*/i ) {
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /(.*(Cannot get device).*)/si ) {
	$error          = "CommandFailed";
	$$error_message = $output;
    }

    return $error;
} ## end sub _check_error
## @name ::ethtool
##
## @summary Parses the raw output from the "/sbin/ethtool" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ethtool with any of its options as described in the
##    corresponding cdef file 'ethtool.cdefs'
## @synopsis
##      ethtool(%args);
## INPUTS TO METHOD CALL 'ethtool' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'gro' </td> <td> 'generic_receive_offload' </td> <td> '0' </td> </tr>
## <tr> <td> 'gso' </td> <td> 'generic_segmentation_offload' </td> <td> '0' </td> </tr>
## <tr> <td> 'tso' </td> <td> 'tcp_segmentation_offload' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'show_offload' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ethernet_name' </td> <td> '0' </td> </tr>
## <tr> <td> '-K' </td> <td> 'set_offload' </td> <td> '0' </td> </tr>
## <tr> <td> 'rx' </td> <td> 'rx_checksum' </td> <td> '0' </td> </tr>
## <tr> <td> 'ufo' </td> <td> 'udp_fragmentation_offload' </td> <td> '0' </td> </tr>
## <tr> <td> 'sg' </td> <td> 'scatter_gather' </td> <td> '0' </td> </tr>
## <tr> <td> 'tx' </td> <td> 'tx_checksum' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = /sbin/ethtool -k eth0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Offload parameters for eth0:
##&nbsp;Cannot get device rx csum settings: Operation not permitted
##&nbsp;Cannot get device udp large send offload settings: Operation not supported
##&nbsp;rx_checksumming: off
##&nbsp;tx_checksumming: on
##&nbsp;scatter_gather: on
##&nbsp;tcp segmentation offload: on
##&nbsp;udp fragmentation offload: off
##&nbsp;generic segmentation offload: off
##&nbsp;generic_receive_offload: off
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'offload_parameters' => [
##&nbsp;                                      {
##&nbsp;                                        'generic_receive_offload' => 'off',
##&nbsp;                                        'cannot_get_device_udp_large_send_offload_settings' => 'Operation not supported',
##&nbsp;                                        'tcp_segmentation_offload' => 'off',
##&nbsp;                                        'cannot_get_device_rx_csum_settings' => 'Operation not supported',
##&nbsp;                                        'udp_fragmentation_offload' => 'off',
##&nbsp;                                        'scatter_gather' => 'off',
##&nbsp;                                        'rx_checksumming' => 'off',
##&nbsp;                                        'tx_checksumming' => 'off',
##&nbsp;                                        'generic_segmentation_offload' => 'off'
##&nbsp;                                      }
##&nbsp;                                    ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = /sbin/ethtool eth0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Settings for eth0:
##&nbsp;        Link detected: yes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;           'settings_for_eth0' => [
##&nbsp;                    {
##&nbsp;                                       'link_detected' => 'yes'
##&nbsp;                                     }
##&nbsp;                                   ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ethtool {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my ( @Res, $key );
    $error = _check_error(
        raw_output    => $output,
        command       => "/sbin/ethtool",
        error_message => \$error_message
    );
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        my $can_throw = 0;
    }

    my @lines = get_lines( 'raw_output' => $output );
    my %hash = ();
    if ( $command =~ /-k/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^(Offload parameters).*/ ) {
                $key = $1;
            } elsif ( $line =~ /^(\S+(\s+\w+)*):\s+(.*)/ ) {
		if ( $line =~ /(Cannot get device.*)/si) {
			next;
		}
		my $temp = $1;
                my @words = split( /-/, $temp );
                $temp = join( "_", @words );
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $temp ) } = $3;
            }
        }
    } else {
        foreach my $line (@lines) {
            if ( $line =~ /^(Settings.*)/ ) {
                $key = $1;
            } elsif ( $line =~ /^(\S+(\s+\w+)*):\s+(.*)/ ) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $1 ) } = $3;
            } elsif ( $line =~ /^(no offload.*)/ ) {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    if($error) {
        $resp->set_parsed_output(parsed_output=>\@Res);
        $error->throw(text=>$error_message,response_object => $resp);
    }
    return \@Res;    #to be updated to return a reference
} ## end sub ethtool

1;
