#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux group sub command
##
## @description
##    This is a library to parse the raw output of Linux group command.
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::group;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::groupadd
##
## @summary Parses the raw output from the "groupadd" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from groupadd with any of its options as described in the 
##    corresponding cdef file 'group.cdefs'
## @synopsis
##      groupadd(%args);
## INPUTS TO METHOD CALL 'groupadd' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '0' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'gid' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = groupadd group
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub groupadd {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub groupadd

## @name ::groupdel
##
## @summary Parses the raw output from the "groupdel" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from groupdel with any of its options as described in the 
##    corresponding cdef file 'group.cdefs'
## @synopsis
##      groupdel(%args);
## INPUTS TO METHOD CALL 'groupdel' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = groupdel group
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub groupdel {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::groupmod
##
## @summary Parses the raw output from the "groupmod" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from groupmod with any of its options as described in the 
##    corresponding cdef file 'group.cdefs'
## @synopsis
##      groupmod(%args);
## INPUTS TO METHOD CALL 'groupmod' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'new-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '0' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'non-unique' </td> <td> '0' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'gid' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = groupmod group
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub groupmod {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::getent_group
##
## @summary Parses the raw output from the "getent group" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from getent_group with any of its options as described in the 
##    corresponding cdef file 'group.cdefs'
## @synopsis
##      getent_group(%args);
## INPUTS TO METHOD CALL 'getent_group' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = getent group
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;root:x:0:
##&nbsp;bin:x:1:bin,daemon
##&nbsp;ip3poee87:*:60063:hdawn,bian,shailvi,alirezam,bsomenat,groot,kunalk,tanjali,vishwane,psp,harshh,andyz,jiron,rcousins,cgiardin,bpulluri,hronald,msudheen,antp,veroniqu,fcastill,jgiardin,mmanenko,trobbins,fulj,ahopkins,bmasingo,martinej,steb,mnorman,dkogel,rrucker,jlantz,jwenners,suguna,saikrish,haiming,qifeng,dnixon,pmadhu,kgoodwin,rostysla,ameade,saptarsh,adhitya,kaslin,wrudd,arjuns,itai,shashidg,mamtap,mkrehbie,bipins,adamou,vladimik,dandrusm,enochj,rciacche,ahmadd,mandarn,newtonl,mzim,lvenkate,dhwani,abhisheh,rcarr,janeal,bgarling,mmckee,manasip
##&nbsp;dstage:*:25002:sdamo,bee,dsadm,wasdmin,admin,gopalkri,shivp
##&nbsp;ip3poee36:*:60063:jstover,jbeadles,brittor,chekt,mgerald,ggaston,kcharles,juditht,tracyt,durairam,amyv,chetanbl,nelsond,girishkb,mooj,tstaggs,dstein,kentp,swaltner,stanleyk,aprahlad,rwinslow,rlandkam,geoffreb,jlessard,bernardc,tdonald,ndan,bderek,sreymoct,swei,jlara,spadmana,jkloeppn,farihab,kumak,jhoaster,ghulpieu,mlynn,bcorriga,gsunil,mzorn,elarry,tsnider,weberm,darrenv,keirj,shej,mjess,chiaho,arunkumr,mbostock,wwarren,montee,gjulie,satindeb,deanl,nghiam,vjason,tahirm,jsattler,shadaksc,wilm,odavid,jcrameri,miriams,springh,waikwong,emiller,darrells
##&nbsp;operator:*:5:
##&nbsp;apache:*:80:
##&nbsp;vsagroup:*:60084:agelsomi,ashrut,atai,badre,cattel,cfreitas,crepunte,dares,dcathy,flynng,gsingodi,hannans,hvijay,hxu,jackm,jeanr,jgoldsch,jmcanult,kevinl,labonte,mbrian,mitchelb,mohinish,monalish,mschwart,nviet,pelle,pudayan,rajarams,rajeshr,ruitao,sangram,saywell,schs,silva,stank,sudhansh,thor,trick,wdallas,xmhan,yohannes,yumeng
##&nbsp;ip3poee21:*:60063:puneet,hiuling,jose1,rpardorl,thomase,minhp,fjohn,tdoan,kortas,lamont,enavalta,carrillo,wa,calder,hanness,rpratt,jsummers,toby,kaufman,adrian,wjoe,dguy,skovrup,dgustafs,rozboril,gloverr,spinks,flannery,nollman,mbaker,lung,jameslm,cbutters,mcramer,mctague,parr,meganj,davidsh,soffa,douglasm,chadc,donp,reddy,subram,johnk,tonyd,johnnys,gautreau,geneh,sedillo,jennyb,alvin,ronc,pmeidell,rajdas,seshu,davec,sivanath,davidlb,sgressel,msiemens,jbennett,afore,schlegel,blanchet,mbrolley,bingen,rustyb,blose,kavuri
##&nbsp;sso_dev:*:658:ashen,thirurag
##&nbsp;ftpers4:*:215:rzink,min,lionel,btaylor,asupdw2,dhoward,jscott,rayf,pranoop,alberty,saadia,bswartz,vantive,ndias,kunal,crosson,dougc,stocks,paloma,psrao,bbilly,marlonh,vijayk,sushilat,marcellc,tettehh,asiemer,indika,vut
##&nbsp;guess:*:99:
##&nbsp;cft_dba_sysadmin:*:60008:
##&nbsp;ip3poee86:*:60063:kbock,thompsos,joshua3,jkatter,almad,faustin,dustinh,vidhib,npatty,dbeahm,ccontrer,akaryeka,deepikar,balasunl,indushre,rahuld,morab,jkaunds,rcp,jshourea,grantj,wfagan,gopig,saranyac,aninditr,ppriya,donniet,drewr,priced,ratena,nvan,dfrase,rwhitney,shailesp,sudhansg,sean3,sanjaym,mtorelli,tall,dpollak,venkatec,chec,giannim,viola,ajithr,sjennife,shreeg,satyana1,richies,twallace,asarthi,raymondg,joshuao,suzanneo,jtaylor,balakrit,gopinatp,batemanj,dkoilada,kratan,harishac,gaurabb,kathern,raghuras,scattrel,kirstenl,nsimha,srivasts,sreenand,mschuman
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'password' => 'x',
##&nbsp;            'group_id' => '0',
##&nbsp;            'group' => 'root',
##&nbsp;            'users' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => 'x',
##&nbsp;            'group_id' => '1',
##&nbsp;            'group' => 'bin',
##&nbsp;            'users' => 'bin,daemon'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60063',
##&nbsp;            'group' => 'ip3poee87',
##&nbsp;            'users' => 'hdawn,bian,shailvi,alirezam,bsomenat,groot,kunalk,tanjali,vishwane,psp,harshh,andyz,jiron,rcousins,cgiardin,bpulluri,hronald,msudheen,antp,veroniqu,fcastill,jgiardin,mmanenko,trobbins,fulj,ahopkins,bmasingo,martinej,steb,mnorman,dkogel,rrucker,jlantz,jwenners,suguna,saikrish,haiming,qifeng,dnixon,pmadhu,kgoodwin,rostysla,ameade,saptarsh,adhitya,kaslin,wrudd,arjuns,itai,shashidg,mamtap,mkrehbie,bipins,adamou,vladimik,dandrusm,enochj,rciacche,ahmadd,mandarn,newtonl,mzim,lvenkate,dhwani,abhisheh,rcarr,janeal,bgarling,mmckee,manasip'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '25002',
##&nbsp;            'group' => 'dstage',
##&nbsp;            'users' => 'sdamo,bee,dsadm,wasdmin,admin,gopalkri,shivp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60063',
##&nbsp;            'group' => 'ip3poee36',
##&nbsp;            'users' => 'jstover,jbeadles,brittor,chekt,mgerald,ggaston,kcharles,juditht,tracyt,durairam,amyv,chetanbl,nelsond,girishkb,mooj,tstaggs,dstein,kentp,swaltner,stanleyk,aprahlad,rwinslow,rlandkam,geoffreb,jlessard,bernardc,tdonald,ndan,bderek,sreymoct,swei,jlara,spadmana,jkloeppn,farihab,kumak,jhoaster,ghulpieu,mlynn,bcorriga,gsunil,mzorn,elarry,tsnider,weberm,darrenv,keirj,shej,mjess,chiaho,arunkumr,mbostock,wwarren,montee,gjulie,satindeb,deanl,nghiam,vjason,tahirm,jsattler,shadaksc,wilm,odavid,jcrameri,miriams,springh,waikwong,emiller,darrells'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '5',
##&nbsp;            'group' => 'operator',
##&nbsp;            'users' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '80',
##&nbsp;            'group' => 'apache',
##&nbsp;            'users' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60084',
##&nbsp;            'group' => 'vsagroup',
##&nbsp;            'users' => 'agelsomi,ashrut,atai,badre,cattel,cfreitas,crepunte,dares,dcathy,flynng,gsingodi,hannans,hvijay,hxu,jackm,jeanr,jgoldsch,jmcanult,kevinl,labonte,mbrian,mitchelb,mohinish,monalish,mschwart,nviet,pelle,pudayan,rajarams,rajeshr,ruitao,sangram,saywell,schs,silva,stank,sudhansh,thor,trick,wdallas,xmhan,yohannes,yumeng'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60063',
##&nbsp;            'group' => 'ip3poee21',
##&nbsp;            'users' => 'puneet,hiuling,jose1,rpardorl,thomase,minhp,fjohn,tdoan,kortas,lamont,enavalta,carrillo,wa,calder,hanness,rpratt,jsummers,toby,kaufman,adrian,wjoe,dguy,skovrup,dgustafs,rozboril,gloverr,spinks,flannery,nollman,mbaker,lung,jameslm,cbutters,mcramer,mctague,parr,meganj,davidsh,soffa,douglasm,chadc,donp,reddy,subram,johnk,tonyd,johnnys,gautreau,geneh,sedillo,jennyb,alvin,ronc,pmeidell,rajdas,seshu,davec,sivanath,davidlb,sgressel,msiemens,jbennett,afore,schlegel,blanchet,mbrolley,bingen,rustyb,blose,kavuri'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '658',
##&nbsp;            'group' => 'sso_dev',
##&nbsp;            'users' => 'ashen,thirurag'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '215',
##&nbsp;            'group' => 'ftpers4',
##&nbsp;            'users' => 'rzink,min,lionel,btaylor,asupdw2,dhoward,jscott,rayf,pranoop,alberty,saadia,bswartz,vantive,ndias,kunal,crosson,dougc,stocks,paloma,psrao,bbilly,marlonh,vijayk,sushilat,marcellc,tettehh,asiemer,indika,vut'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '99',
##&nbsp;            'group' => 'guess',
##&nbsp;            'users' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60008',
##&nbsp;            'group' => 'cft_dba_sysadmin',
##&nbsp;            'users' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'password' => '*',
##&nbsp;            'group_id' => '60063',
##&nbsp;            'group' => 'ip3poee86',
##&nbsp;            'users' => 'kbock,thompsos,joshua3,jkatter,almad,faustin,dustinh,vidhib,npatty,dbeahm,ccontrer,akaryeka,deepikar,balasunl,indushre,rahuld,morab,jkaunds,rcp,jshourea,grantj,wfagan,gopig,saranyac,aninditr,ppriya,donniet,drewr,priced,ratena,nvan,dfrase,rwhitney,shailesp,sudhansg,sean3,sanjaym,mtorelli,tall,dpollak,venkatec,chec,giannim,viola,ajithr,sjennife,shreeg,satyana1,richies,twallace,asarthi,raymondg,joshuao,suzanneo,jtaylor,balakrit,gopinatp,batemanj,dkoilada,kratan,harishac,gaurabb,kathern,raghuras,scattrel,kirstenl,nsimha,srivasts,sreenand,mschuman'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub getent_group {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines(raw_output=>$output, response_object=>$resp);
    my (@split_line, $line, @result_array);

    my $i = 0;
    while(@lines){
       $line = shift(@lines);
       @split_line = split(/[:]+/,$line);
       $result_array[$i]->{'group'} = $split_line[0];
       $result_array[$i]->{'password'} = $split_line[1];
       $result_array[$i]->{'group_id'} = $split_line[2];
       $result_array[$i]->{'users'} = $split_line[3];
       $i++;
	}
    return \@result_array;
}

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Usage.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /.*does not exist/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ($raw_output =~ /invalid group ID/){
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ($raw_output =~ /already exists/i){
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif($raw_output =~ /not a unique name/){
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif($raw_output =~ /option requires an argument/i) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error


1;
