#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/hwclock.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary hwclock command output
##
## @description
##    This is a library to parse the raw output of hwclock command.
##
## @synopsis
##    use hwclock;
##
##    hwclock(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::hwclock;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

use strict;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /invalid option/ ) {
        $error         = "InvalidParam";
        $error_message = "Invalid Options are used in the Command";
    } elsif ( $raw_output =~ /option\s+.*\s+requires an argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = "Required Parameter is Missing";
    } elsif ( $raw_output =~ /U[u]sage:.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

## @name ::hwclock
##
## @summary Parses the raw output from the "hwclock" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from hwclock with any of its options as described in the 
##    corresponding cdef file 'hwclock.cdefs'
## @synopsis
##      hwclock(%args);
## INPUTS TO METHOD CALL 'hwclock' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> ' --adjfile' </td> <td> 'override_adjtime' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --directisa' </td> <td> 'use_explict_i/o_instruction_to_access_hwclock' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'date_string' </td> <td> '0' </td> </tr> 
## <tr> <td> ' --systz' </td> <td> 'reset_system_time' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --epoch' </td> <td> 'kernel_hwclock_value' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --localtime' </td> <td> 'local_time' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --set' </td> <td> 'set_hwclock' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -D' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --badyear' </td> <td> 'outside_the_range' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -r' </td> <td> 'show' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --setepoch' </td> <td> 'set_kernel_hwclock_epoch_value' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --srm' </td> <td> 'epoch_on_alphas_with_SRM_console' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -v' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '=' </td> <td> 'match' </td> <td> '0' </td> </tr> 
## <tr> <td> ' -u' </td> <td> 'utc' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --funky-toy' </td> <td> 'aplha_machine_type' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --arc' </td> <td> 'epoch_on_alphas_with_ARC_console' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --test' </td> <td> 'test_mode' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -f' </td> <td> 'override_default_dev_filename' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --date' </td> <td> 'set_hwclock_time' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --noadjfile' </td> <td> 'disable_adjustfile_facilities' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -w' </td> <td> 'systohc' </td> <td> '1' </td> </tr> 
## <tr> <td> ' -s' </td> <td> 'hctosys' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr> 
## <tr> <td> ' --getepoch' </td> <td> 'show_kernel_hwclock_epoch_value' </td> <td> '1' </td> </tr> 
## <tr> <td> ' --adjust' </td> <td> 'adjust' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'year' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = hwclock -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Sat 28 Sep 1996 04:29:09 PM IST  -0.914949 seconds
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hwclock_time' => '04:29:09 PM',
##&nbsp;            'date' => 'Sat 28 Sep',
##&nbsp;            'time_zone' => 'IST',
##&nbsp;            'drift' => '-0.914949 seconds',
##&nbsp;            'year' => '1996'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hwclock from util-linux-2.13-pre7
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'util-linux-2.13-pre7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Sat 28 Sep 1996 09:59:10 PM IST  -0.862831 seconds
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hwclock_time' => '09:59:10 PM',
##&nbsp;            'date' => 'Sat 28 Sep',
##&nbsp;            'time_zone' => 'IST',
##&nbsp;            'drift' => '-0.862831 seconds',
##&nbsp;            'year' => '1996'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock --localtime
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Sat 28 Sep 1996 04:29:11 PM IST  -0.945593 seconds
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hwclock_time' => '04:29:11 PM',
##&nbsp;            'date' => 'Sat 28 Sep',
##&nbsp;            'time_zone' => 'IST',
##&nbsp;            'drift' => '-0.945593 seconds',
##&nbsp;            'year' => '1996'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock --debug --badyear
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hwclock from util-linux-2.13-pre7
##&nbsp;Using /dev/rtc interface to clock.
##&nbsp;Last drift adjustment done at 843478277 seconds after 1969
##&nbsp;Last calibration done at 843478277 seconds after 1969
##&nbsp;Hardware clock is on local time
##&nbsp;Assuming hardware clock is kept in local time.
##&nbsp;Waiting for clock tick...
##&nbsp;/dev/rtc does not have interrupt functions. Waiting in loop for time from /dev/rtc to change
##&nbsp;...got clock tick
##&nbsp;Time read from Hardware Clock: 2012/09/28 16:29:12
##&nbsp;Hw clock time : 2012/09/28 16:29:12 = 1348829952 seconds since 1969
##&nbsp;Fri 28 Sep 2012 04:29:12 PM IST  -0.948685 seconds
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_read_from_hardware_clock' => '2012/09/28 16:29:12',
##&nbsp;            'hwclock_time' => '04:29:12 PM',
##&nbsp;            'date' => 'Fri 28 Sep',
##&nbsp;            'time' => 'local',
##&nbsp;            'version' => 'util-linux-2.13-pre7',
##&nbsp;            'last_drift_adjustment' => '843478277 seconds after 1969',
##&nbsp;            'last_calibration' => '843478277 seconds after 1969',
##&nbsp;            'time_zone' => 'IST',
##&nbsp;            'interface' => '/dev/rtc',
##&nbsp;            'drift' => '-0.948685 seconds',
##&nbsp;            'hw_clock_time' => {
##&nbsp;                                 'date' => '2012/09/28',
##&nbsp;                                 'hwtime' => '16:29:12 = 1348829952 seconds since 1969'
##&nbsp;                               },
##&nbsp;            'year' => '2012'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock --debug --noadjfile --utc
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hwclock from util-linux-2.13-pre7
##&nbsp;Using /dev/rtc interface to clock.
##&nbsp;Assuming hardware clock is kept in UTC time.
##&nbsp;Waiting for clock tick...
##&nbsp;/dev/rtc does not have interrupt functions. Waiting in loop for time from /dev/rtc to change
##&nbsp;...got clock tick
##&nbsp;Time read from Hardware Clock: 1996/09/28 16:29:13
##&nbsp;Hw clock time : 1996/09/28 16:29:13 = 843928153 seconds since 1969
##&nbsp;Sat 28 Sep 1996 09:59:13 PM IST  -0.945763 seconds
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_read_from_hardware_clock' => '1996/09/28 16:29:13',
##&nbsp;            'hwclock_time' => '09:59:13 PM',
##&nbsp;            'date' => 'Sat 28 Sep',
##&nbsp;            'time' => 'UTC',
##&nbsp;            'version' => 'util-linux-2.13-pre7',
##&nbsp;            'time_zone' => 'IST',
##&nbsp;            'interface' => '/dev/rtc',
##&nbsp;            'drift' => '-0.945763 seconds',
##&nbsp;            'hw_clock_time' => {
##&nbsp;                                 'date' => '1996/09/28',
##&nbsp;                                 'hwtime' => '16:29:13 = 843928153 seconds since 1969'
##&nbsp;                               },
##&nbsp;            'year' => '1996'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = hwclock --debug -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;hwclock from util-linux-2.13-pre7
##&nbsp;Using /dev/rtc interface to clock.
##&nbsp;Last drift adjustment done at 843478277 seconds after 1969
##&nbsp;Last calibration done at 843478277 seconds after 1969
##&nbsp;Hardware clock is on local time
##&nbsp;Assuming hardware clock is kept in local time.
##&nbsp;Waiting for clock tick...
##&nbsp;/dev/rtc does not have interrupt functions. Waiting in loop for time from /dev/rtc to change
##&nbsp;...got clock tick
##&nbsp;Time read from Hardware Clock: 1996/09/28 16:29:14
##&nbsp;Hw clock time : 1996/09/28 16:29:14 = 843908354 seconds since 1969
##&nbsp;Calling settimeofday:
##&nbsp;	tv.tv_sec = 843908354, tv.tv_usec = 0
##&nbsp;	tz.tz_minuteswest = -330
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_read_from_hardware_clock' => '1996/09/28 16:29:14',
##&nbsp;            'last_calibration' => '843478277 seconds after 1969',
##&nbsp;            'last_drift_adjustment' => '843478277 seconds after 1969',
##&nbsp;            'settimeofday' => {
##&nbsp;                                'tz_tz_minuteswest' => '-330',
##&nbsp;                                'tv_tv_sec' => '843908354',
##&nbsp;                                'tv_tv_usec' => '0'
##&nbsp;                              },
##&nbsp;            'time' => 'local',
##&nbsp;            'version' => 'util-linux-2.13-pre7',
##&nbsp;            'interface' => '/dev/rtc',
##&nbsp;            'hw_clock_time' => {
##&nbsp;                                 'date' => '1996/09/28',
##&nbsp;                                 'hwtime' => '16:29:14 = 843908354 seconds since 1969'
##&nbsp;                               }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub hwclock {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    if ( $cmd eq "hwclock -w" && $cmd eq "hwclock -s" ) {
        return [];
    }

    foreach my $line (@lines) {
        if ( $cmd =~ /-v/ ) {
            if($line=~/hwclock from (.*)/)
            {
            $result_array[0]->{version} = $1;
            }
            return \@result_array;
        }

        if ( $cmd =~ /-D|--debug/ ) {
            if ( $line =~ /hwclock from (.*)/ ) {
                $result_array[0]->{version} = $1;
            } elsif ( $line =~ /Using\s+(\S+)\s+(\w+)\sto clock./ ) {
                $result_array[0]->{$2} = $1;
            } elsif ( $line =~ /hardware clock is kept in (\w+) (\w+)/ ) {
                $result_array[0]->{$2} = $1;
            } elsif ( $line =~ /Time read from.*/ ) {
                my @temp = split( /:\s+/, $line );
                $result_array[0]->{ to_key_name( item => $temp[0] ) }
                    = $temp[1];
            } elsif ( $line =~ /(Hw clock time) :\s(\S+)\s(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) }->{date} = $2;
                $result_array[0]->{ to_key_name( item => $1 ) }->{hwtime}
                    = $3;
            } elsif ( $line =~ /(Last drift adjustment) done at (.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /(Last calibration) done at (.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            } elsif (
                $line =~ /(tv.tv_sec)\s=\s(\d+),\s+(tv.tv_usec)\s=\s(\d+)/ ) {
                my $t1 = $1;
                my $t2 = $3;
                $t1 =~ tr/./_/;
                $t2 =~ tr/./_/;
                $result_array[0]->{settimeofday}
                    ->{ to_key_name( item => $t1 ) } = $2;
                $result_array[0]->{settimeofday}
                    ->{ to_key_name( item => $t2 ) } = $4;
            } elsif ( $line =~ /(tz.tz_minuteswest)\s=\s(\S+)/ ) {
                my $t = $1;
                $t =~ tr/./_/;
                $result_array[0]->{settimeofday}
                    ->{ to_key_name( item => $t ) } = $2;
            }
        }
        if ( $line
            =~ /(\w+\s+\d{1,2}\s+\w+)\s+(\d{1,4})\s+(\d{1,2}:\d{1,2}:\d{1,2}\s+\w+)\s+(\w+)\s+(\S+\s+\w+)/
            ) {

            $result_array[0]->{date}         = $1;
            $result_array[0]->{year}         = $2;
            $result_array[0]->{hwclock_time} = $3;
            $result_array[0]->{time_zone}    = $4;
            $result_array[0]->{drift}        = $5;
        }
    }
    return \@result_array;
} ## end sub hwclock
1;

