#Copyright (c) 2010-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux vconfig command output
##
## @description
##    This is a library to parse the raw output of Linux ifconfig command.
##
## @synopsis
##    use ifconfig;
##
##    ifconfig(%args);
##
## @author ksmahesh@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ifconfig;

use Data::Dumper;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed";    # we'll try to classify it if possible
    $$error_message = $raw_output;

    if ($raw_output =~ /ERROR|Usage:\s.*/) {
        if ($raw_output =~ /Missing/) {
            $error = "MissingRequiredParam";
        } elsif ($raw_output =~ /[Ii]nvalid|[Uu]nrecognized\s+option/) {
            $error = "InvalidParam";
        } elsif ($raw_output =~ /argument\s+to|unknown/) {
            $error = "InvalidParamValue";
        }
    }
    return $error;
}

## @name ::ifconfig
##
## @summary Parses the raw output from "vconfig add" command
##
## @description
##    This method can be used for parsing the output from "ifconfig add" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/ifconfig.cdefs
##
## @synopsis
##    ifconfig (%args);
##
## INPUTS TO METHOD CALL 'ifconfig' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## command_full_path
## (Optional)
## The complete path to the command.
## Eg: /sbin/ifconfig
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
##
## @return Returns an array ref containing a hash reference which contains the
##  parsed output data from the output of vconfig add command

sub ifconfig {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 0,
        response_object => $resp
    );

    my @return_array;
    my $regex;
    my $temp_hash_ptr;
    while ( my $line = shift(@lines) ) {

        if ( $line =~ /(\S+)\s+Link\s+encap:/) {
	    # eth0      Link encap:Ethernet  HWaddr 00:1D:D7:03:5B:A1
	    # lo        Link encap:Local Loopback  \
            push( @return_array, $temp_hash_ptr )
                if ( defined( $temp_hash_ptr->{'interface'} ) );
            $temp_hash_ptr                 = {};
            $temp_hash_ptr->{'interface'}  = $1;
            if ( $line =~ /\S+\s+Link\s+encap:(Local Loopback)/) {
                $temp_hash_ptr->{'link_encap'} = $1;
	    } elsif ($line =~ /\S+\s+Link\s+encap:(\S+)\s+\S+\s+(\S+)/ ) {
                $temp_hash_ptr->{'link_encap'} = $1;
                $temp_hash_ptr->{'hwaddr'} = $2;
	    }
        } elsif ( $line =~ /(\S+):\s+flags=\d+\S+\s+mtu\s+(\d+)?$/ ) {
	    # eth0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
            push( @return_array, $temp_hash_ptr )
                if ( defined( $temp_hash_ptr->{'interface'} ) );
            $temp_hash_ptr                 = {};
            $temp_hash_ptr->{'interface'}  = $1;
            $temp_hash_ptr->{'mtu'}    = $2;
	} elsif ($line =~ /loop\s+txqueuelen\s+\d+\s+\((\w+)\)/ ) {
            # loop  txqueuelen 0  (Local Loopback)
	    $temp_hash_ptr->{'link_encap'} = $1; 
        } elsif ( $line =~ /inet addr:(\S+)\s+Bcast:(\S+)\s+Mask:(\S+)/ ) {
            $temp_hash_ptr->{'inet_addr'} = $1;
            $temp_hash_ptr->{'bcast'}     = $2;
            $temp_hash_ptr->{'mask'}      = $3;
        } elsif ( $line =~ /inet\s+(\S+)\s+netmask\s+(\S+)\s+broadcast\s+(\S+)/ ) {
	    # inet 10.234.12.210  netmask 255.255.252.0  broadcast 10.234.15.255
            $temp_hash_ptr->{'inet_addr'} = $1;
            $temp_hash_ptr->{'mask'}      = $2;
            $temp_hash_ptr->{'bcast'}     = $3;
        } elsif ( $line =~ /inet addr:(\S+)\s+Mask:(\S+)/ ) {
            $temp_hash_ptr->{'inet_addr'} = $1;
            $temp_hash_ptr->{'mask'}      = $2;
        } elsif ( $line =~ /inet6 addr:\s?(\S+) Scope:(.*)/ ) {
            my %temp_hash;
            $temp_hash{'addr'}  = $1;
            $temp_hash{'scope'} = $2;
            push( @{ $temp_hash_ptr->{'inet6_info'} }, \%temp_hash );
        } elsif ( $line =~ /inet6\s?(\S+)\s+prefixlen\s+(\d+)\s+scopeid\s+(\S+)/ ) {
	    # inet6 fe80::21d:d7ff:fe05:322d  prefixlen 64  scopeid 0x20<link>
            my %temp_hash;
            $temp_hash{'addr'}  = "$1/$2";
            $temp_hash{'scope'} = $3;
            push( @{ $temp_hash_ptr->{'inet6_info'} }, \%temp_hash );
        } elsif ( $line =~ /(.*)\s+MTU:(\d+)\s+Metric:(\d+)/ ) {
            my $value = $1;
            $temp_hash_ptr->{'mtu'}    = $2;
            $temp_hash_ptr->{'metric'} = $3;
            my ( $status, @property ) = split( /\s+/, $value );
            $temp_hash_ptr->{'status'} = $status
                if ( $status =~ /(UP|DOWN)/ );
            push( @property, $status ) if ( $status !~ /(UP|DOWN)/ );
            $temp_hash_ptr->{'property'} = join( ',', @property );
        } elsif ( $line
            =~ /RX packets:(\d+)\s+errors:(\d+)\s+dropped:(\d+)\s+overruns:(\d+)\s+frame:(\d+)/
            )
        {
            $temp_hash_ptr->{'rx_packets'}  = $1;
            $temp_hash_ptr->{'rx_errors'}   = $2;
            $temp_hash_ptr->{'rx_dropped'}  = $3;
            $temp_hash_ptr->{'rx_overruns'} = $4;
            $temp_hash_ptr->{'rx_frame'}    = $5;
        } elsif ( $line
            =~ /TX packets:(\d+)\s+errors:(\d+)\s+dropped:(\d+)\s+overruns:(\d+)\s+carrier:(\d+)/
            )
        {
            $temp_hash_ptr->{'tx_packets'}  = $1;
            $temp_hash_ptr->{'tx_errors'}   = $2;
            $temp_hash_ptr->{'tx_dropped'}  = $3;
            $temp_hash_ptr->{'tx_overruns'} = $4;
            $temp_hash_ptr->{'tx_carrier'}  = $5;
        } elsif ( $line =~ /collisions:(\d+)\s+txqueuelen:(\d+)/ ) {
            $temp_hash_ptr->{'collisions'} = $1;
            $temp_hash_ptr->{'txqueuelen'} = $2;
        } elsif ( $line =~ /RX bytes:(.*)\s+TX bytes:(.*)/ ) {
            my $value = $1;
            $value = trim_string( 'string' => $value );
            $temp_hash_ptr->{'rx_bytes'} = $value;
            $temp_hash_ptr->{'tx_bytes'} = $2;
        } elsif ( $line =~ /Interrupt:(\S+)\s+Base\s+address:(\S+)/ ) {
            $temp_hash_ptr->{'interrupt'}    = $1;
            $temp_hash_ptr->{'base_address'} = $2;
        } elsif ( $line =~ /Interrupt:(\S+)\s+Memory:(\S+)/ ) {
            $temp_hash_ptr->{'interrupt'}    = $1;
            $temp_hash_ptr->{'memory'} = $2;
        } elsif ($line =~ /RX packets\s+(.*)\s+bytes\s+(.*)/){
	    $temp_hash_ptr->{'rx_packets'} =  trim_string( 'string' => $1) ;
	    $temp_hash_ptr->{'rx_bytes'} = $2 ;
	} elsif($line =~ /device\s+interrupt\s+(\d+)\s+memory\s+(.*)/) {
	    $temp_hash_ptr->{'interrupt'} = $1;
	    $temp_hash_ptr->{'memory'}    = $2 ;
	} elsif($line =~ /TX packets\s+(.*)\s+bytes\s+(.*)/){
	    $temp_hash_ptr->{'tx_packets'} =  trim_string( 'string' => $1) ;
	    $temp_hash_ptr->{'tx_bytes'} = $2 ;
	} elsif($line  =~
		/TX errors\s+(\d+)\s+dropped\s+(\d+)\s+overruns\s+(\d+)\s+carrier\s+(\d+)\s+collisions\s+(\d+)/){
            $temp_hash_ptr->{'tx_errors'}   = $1;
            $temp_hash_ptr->{'tx_dropped'}  = $2;
            $temp_hash_ptr->{'tx_overruns'} = $3;
            $temp_hash_ptr->{'tx_carrier'}  = $4;
	    $temp_hash_ptr->{'tx_collisions'} = $5 ;
	}elsif ($line =~ /RX errors\s+(\d+)\s+dropped\s+(\d+)\s+overruns\s+(\d+)\s+frame\s+(\d+)/) {
	    $temp_hash_ptr->{'rx_errors'}   = $1;
            $temp_hash_ptr->{'rx_dropped'}  = $2;
            $temp_hash_ptr->{'rx_overruns'} = $3;
            $temp_hash_ptr->{'rx_frame'}  = $4;
	} elsif ($line =~ /ether\s+(\S+)\s+txqueuelen\s+(\d+)\s+\((\w+)\)/) {
            # ether 00:1d:d7:05:32:2d  txqueuelen 1000  (Ethernet)
	    $temp_hash_ptr->{'hwaddr'}   =  trim_string( 'string' => $1);
            $temp_hash_ptr->{'txqueuelen'}  = $2;
	    $temp_hash_ptr->{'link_encap'} = $3; 
	}elsif ( $line =~ /inet\s+(\S+)\s+netmask\s+(\S+)\s+broadcast\s+(\S+)/ ) {
            $temp_hash_ptr->{'inet_addr'} = $1;
            $temp_hash_ptr->{'mask'}      = $2;
            $temp_hash_ptr->{'bcast'}     = $3;
	} elsif ( $line =~ /flags[=](.*)\s+mtu\s+(.*)/) {
	    $temp_hash_ptr->{'flags'} = trim_string( 'string' => $1) ;
            $temp_hash_ptr->{'mtu'} =$2 ;
	}
    }
    push( @return_array, $temp_hash_ptr );
    return \@return_array;

    #    if ($line =~ /Added VLAN with VID == \d+ to IF -:\w+:-/) {
    #        my @result_array;
    #        my %temp_hash = ("result","successful");
    #        push (@result_array,\%temp_hash);
    #        return \@result_array;
    #    } else {
    #        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
    #                    text => "Unexpected Output while parsing",
    #                        response_object => $resp);
    #    }
}
1;
