#
#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse dd command output
##
## @description
##    This is a library to parse the raw output of ip command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::ip;
##
##  ip(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Linux::Redhat::ip;

use strict;
use warnings;

use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::ip
##
## @summary Parses the raw output from the "ip" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ip with any of its options as described in the 
##    corresponding cdef file 'ip.cdefs'
## @synopsis
##      ip(%args);
## INPUTS TO METHOD CALL 'ip' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'link' </td> <td> 'link' </td> <td> '0' </td> </tr> 
## <tr> <td> 'show' </td> <td> 'show' </td> <td> '0' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'neline' </td> <td> '0' </td> </tr> 
## <tr> <td> 'addr' </td> <td> 'address' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ip -o addr show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1: lo    inet 127.0.0.1/8 scope host lo\       valid_lft forever preferred_lft forever
##&nbsp;1: lo    inet6 ::1/128 scope host \       valid_lft forever preferred_lft forever
##&nbsp;2: eth0    inet 10.235.232.199/22 brd 10.235.235.255 scope global dynamic eth0\       valid_lft 83761sec preferred_lft 83761sec
##&nbsp;2: eth0    inet6 fd20:8b1e:b255:9074:21d:d7ff:fe01:b7fc/64 scope global dynamic \       valid_lft 2591949sec preferred_lft 604749sec
##&nbsp;2: eth0    inet6 fe80::21d:d7ff:fe01:b7fc/64 scope link \       valid_lft forever preferred_lft forever
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'inet',
##&nbsp;            'valid_lft' => 'forever',
##&nbsp;            'preferred_lft' => 'forever',
##&nbsp;            'ip_addr_port' => '127.0.0.1/8',
##&nbsp;            'network_interface' => 'lo',
##&nbsp;            'seq' => '1',
##&nbsp;            'scope' => 'host'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'inet6',
##&nbsp;            'valid_lft' => 'forever',
##&nbsp;            'preferred_lft' => 'forever',
##&nbsp;            'ip_addr_port' => '::1/128',
##&nbsp;            'network_interface' => 'lo',
##&nbsp;            'seq' => '1',
##&nbsp;            'scope' => 'host'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'inet',
##&nbsp;            'valid_lft' => '83761sec',
##&nbsp;            'brd' => '10.235.235.255',
##&nbsp;            'preferred_lft' => '83761sec',
##&nbsp;            'ip_addr_port' => '10.235.232.199/22',
##&nbsp;            'network_interface' => 'eth0',
##&nbsp;            'seq' => '2',
##&nbsp;            'scope' => 'global'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'inet6',
##&nbsp;            'valid_lft' => '2591949sec',
##&nbsp;            'preferred_lft' => '604749sec',
##&nbsp;            'ip_addr_port' => 'fd20:8b1e:b255:9074:21d:d7ff:fe01:b7fc/64',
##&nbsp;            'network_interface' => 'eth0',
##&nbsp;            'seq' => '2',
##&nbsp;            'scope' => 'global'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'inet6',
##&nbsp;            'valid_lft' => 'forever',
##&nbsp;            'preferred_lft' => 'forever',
##&nbsp;            'ip_addr_port' => 'fe80::21d:d7ff:fe01:b7fc/64',
##&nbsp;            'network_interface' => 'eth0',
##&nbsp;            'seq' => '2',
##&nbsp;            'scope' => 'link'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ip {
    my %args = @_;
    my $resp = $args{response_object};
    my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();

    _check_error( raw_output => $raw_output, response_object => $resp );

    if ( $command =~ /ip\s+-o\s+addr\s+show/ ) {
	_ip_neline_addr_show(%args);
    }elsif ( $command =~ /ip\s+-o\s+link\s+show/ ) {
	_ip_neline_link_show(%args);
    }else {
	return [];
    }
}

sub _ip_neline_addr_show {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    _check_error( raw_output => $raw_output, response_object => $resp );

    my @array_lines = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => '1',
        "response_object" => $resp,
    );

    my @result;
    for ( my $line = 0; $line <= $#array_lines; $line++ ) {
        my @values = split( /\s+/, $array_lines[$line] );
        my %hash;
        for ( my $field = 0; $field <= $#values; $field++ ) {
            if ( $field == 0 ) {
		$values[0] =~ s/://;
                $hash{'seq'} = $values[0];
            } elsif ( $field == 1 ) {
                $hash{'network_interface'} = $values[1];
            } elsif ( $field == 2 ) {
                $hash{'protocol'} = $values[2];
            } elsif ( $field == 3 ) {
                $hash{'ip_addr_port'} = $values[3];
            } else {
                if (   $values[$field] !~ /scope/
                    && $values[$field] !~ /brd/
                    && $values[$field] !~ /valid_lft/
                    && $values[$field] !~ /preferred_lft/ )
                {
                    #Skip unknown fields
                    #do nothing just go to next iteration
                } else {
                    $hash{ to_key_name( item => $values[$field] ) } = $values[ $field + 1 ];
                    $field++;
                }
            }
        }
        push( @result, \%hash );
    }
    return \@result;
}

sub _ip_neline_link_show {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    _check_error( raw_output => $raw_output, response_object => $resp );

    my @array_lines = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => '1',
        "response_object" => $resp,
    );

    my @result;

    for ( my $line = 0; $line <= $#array_lines; $line++ ) {
        my @values = split( /\s+/, $array_lines[$line] );
        my %hash;
        for ( my $field = 0; $field <= $#values; $field++ ) {
            if ( $field == 0 ) {
		$values[0] =~ s/://;
                $hash{'seq'} = $values[0];
            } elsif ( $field == 1 ) {
                $hash{'network_interface'} = $values[1];
            } elsif ( $field == 2 ) {
                $hash{'features'} = $values[2];
            } elsif ( $values[$field] =~ /link\// ) {
                my @link_data = split( '/', $values[$field] );
                $hash{'link'} = $link_data[1];
            } elsif ( $values[ $field - 1 ] =~ /link\// ) {
                $hash{'current_hw_address'} = $values[$field];
            } else {
                if (   $values[$field] !~ /mtu/
                    && $values[$field] !~ /qdisc/
                    && $values[$field] !~ /state/
                    && $values[$field] !~ /mode/
                    && $values[$field] !~ /qlen/
                    && $values[$field] !~ /brd/ )
                {
                    #Skip unknown fields
                    #do nothing just go to next iteration
                } else {
                    $hash{ to_key_name( item => $values[$field] ) } = $values[ $field + 1 ];
                    $field++;
                }
            }
        }
        push( @result, \%hash );
    }
    return \@result;
}

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};

    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Usage: ip(.*)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $1;
    } elsif ( $raw_output =~ /(.*is unknown, try "ip.*.help".*)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $1;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
