#Copyright (c) 2012-17 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux iscsiadm command output
##
## @description
##    This is a library to parse the raw output of Linux iscsiadm command.
##
## @synopsis
##    use iscsiadm;
##
##    iscsiadm(%args);
##
## @author kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::iscsiadm;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingValueException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    if ( $raw_output
        =~ /(iscsiadm: discovery record [0,3260] not found!|option requires an argument)/i
        ) {
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /iscsiadm\s+-m\s+discovery\s+\[\s+-hV\s\]|iscsiadm --help/si ) {
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /iscsiadm:\s+update requires name and value/si ) {
        $error         = "MissingValue";
    } elsif ( $raw_output
        =~ /(iscsiadm: Maybe you are not root|no\s+records\s+found|Could\s+not\s+login|connection\s+to\s+discovery\s+address.*failed|discovery\s+login\s+to\s+\S+\s+rejected|iscsiadm:\s+initiator\s+reported\s+error|Could\s+not\s+perform\s+SendTargets\s+discovery|Cannot\s+perform\s+discovery|failed\s+to\s+create\s+a\s+discovery)/si
        ) {
        $error         = "CommandFailed";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
} ## end sub _check_error

## @name ::iscsiadm
##
## @summary Parses the raw output from the "/sbin/iscsiadm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsiadm with any of its options as described in the
##    corresponding cdef file 'iscsiadm.cdefs'
## @synopsis
##      iscsiadm(%args);
## INPUTS TO METHOD CALL 'iscsiadm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-k' </td> <td> 'priority' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '1' </td> </tr>
## <tr> <td> 'fw' </td> <td> 'firmware' </td> <td> '1' </td> </tr>
## <tr> <td> 'node' </td> <td> 'node' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'sysfsdir' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'hostno' </td> <td> '0' </td> </tr>
## <tr> <td> 'session' </td> <td> 'session' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'logout-all' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'logout' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> 'iface' </td> <td> 'iface' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'operation' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'login-all' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'print-level' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'show' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug-level' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'target-name' </td> <td> '0' </td> </tr>
## <tr> <td> 'discovery' </td> <td> 'discovery' </td> <td> '1' </td> </tr>
## <tr> <td> 'host' </td> <td> 'host' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ip-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'login' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'iface-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'rescan' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'sessionid' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'specific-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /sbin/iscsiadm -m node -T iqn.1992-08.com.netapp:sn.4093676661 -p 10.238.3.164 -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Logging in to [iface: default, target: iqn.1992-08.com.netapp:sn.4093676661, portal: 10.238.3.164,3260]
##&nbsp;Login to [iface: default, target: iqn.1992-08.com.netapp:sn.4093676661, portal: 10.238.3.164,3260]: successful
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'target' => 'iqn.1992-08.com.netapp:sn.4093676661',
##&nbsp;            'status' => 'successful',
##&nbsp;            'iface' => 'default',
##&nbsp;            'portal' => '10.238.3.164,3260'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iscsiadm {
    my %args = @_;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $command     = $resp->get_actual_command();

    if($error_level) {
	_check_error( raw_output => $output, response_object => $resp );
    }

    my @Result_Array;
    my @lines  = get_lines( raw_output => $output, response_object => $resp );
    my $i      = 0;
    my $option = $command;
    my @values;
    foreach my $line (@lines) {
	    if ( $line =~ /iscsiadm\s+version\s+(.*)/i ) {
		    $Result_Array[ $i++ ]->{version} = $1;
	    } elsif ( $line =~ /^(.*):(\d+,?\d*)+\s+(.*)$/ ) {
		    my $str1 = $1;
		    my $str2 = $2;
		    my $str3 = $3;
		    if ($str1 =~ /\[.*:.*\]/) {
			@values = split(/\[|\]/, $str1);
			if ($str1 =~ /^tcp/) {
			    $Result_Array[$i]->{'ipaddress'} =  $values[3];
			}
			else {
			    $Result_Array[$i]->{'ipaddress'} =  $values[1];
			}
		    } elsif ($str1 =~ /.*:.*:.*/) {
			    $Result_Array[$i]->{'ipaddress'} = $str1;
		    } elsif ($str1 =~ /.*:.*/) {
			    @values = split( /:/, $str1 );
			    my @ipadd = split(/\s+/ , $values[1]);
			    $Result_Array[$i]->{'ipaddress'} =  $ipadd[2];
		    } else {
			    $Result_Array[$i]->{'ipaddress'} = $str1;
		    }
		    if($str2 =~ /.*,.*/)
		    {
			    @values = split( /,/, $str2 );
			    $Result_Array[$i]->{'ip_port'} = $values[0];
			    $Result_Array[$i]->{'tpgt'} = $values[1];
		    } else {
			    $Result_Array[$i]->{'ip_port'} = $str2;
		    }
		    if($str3 =~ /via/)
		    {
			    @values = split( /\s+/, $str3 );
			    $Result_Array[$i++]->{'discovery_type'} = $values[1];
		    } else {
			    $Result_Array[$i++]->{'iqn'} = $str3;
		    }
	    }

	    elsif ( $line
            =~ /Login.*iface:\s+(.*),\s+target:\s+(.*\d+),\s+portal:\s+(.*,\d+).*:\s*(\w+)/i
            ) {
            $Result_Array[$i]->{'iface'}      = $1;
            $Result_Array[$i]->{'target'}     = $2;
            $Result_Array[$i]->{'portal'}     = $3;
            $Result_Array[ $i++ ]->{'status'} = $4;
        } elsif ( $line
            =~ /Logout.*sid:\s+(.*),\s+target:\s+(.*\d+),\s+portal:\s+(.*,\d+).*:\s*(\w+)/i
            ) {
            $Result_Array[$i]->{'sid'}        = $1;
            $Result_Array[$i]->{'target'}     = $2;
            $Result_Array[$i]->{'portal'}     = $3;
            $Result_Array[ $i++ ]->{'status'} = $4;
        }
    }
    return \@Result_Array;
} ## end sub iscsiadm

1;
