#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux lldptool command
##
## @description
##    This is a library to parse the raw output of Linux lldptool command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lldptool" will be lldptool()
##
## @synopsis
##    use lldptool;
##
##    lldptool(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::lldptool;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::lldptool
##
## @summary Parses the raw output from the "lldptool" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lldptool with any of its options as described in the
##    corresponding cdef file 'lldptool.cdefs'
## @synopsis
##      lldptool(%args);
## INPUTS TO METHOD CALL 'lldptool' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-V' </td> <td> 'tlvid' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'set-lldp' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ifname' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'message' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lldptooll' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'get-lldp' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'get-tlv' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'set-tlv' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'neighbour' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lldptool -l -i eth1 adminStatus
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;adminStatus=rxtx
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'adminstatus' => 'rxtx'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lldptool -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;lldptool v0.9.43
##&nbsp;Copyright (c) 2007-2010, Intel Corporation
##&nbsp;
##&nbsp;Substantially modified from:  hostapd_cli v 0.5.7
##&nbsp;Copyright (c) 2004-2007, Jouni Malinen <j@w1.fi> and contributors
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => '0.9.43'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lldptool -t -n -i eth1 -V portDesc
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Port Description TLV
##&nbsp;	GigabitEthernet1/40
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'port_description_tlv' => 'GigabitEthernet1/40'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lldptool -S -i eth1 adminStatus
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Total Frames Transmitted        = 19
##&nbsp;Total Discarded Frames Received = 0
##&nbsp;Total Error Frames Received     = 0
##&nbsp;Total Frames Received           = 35250
##&nbsp;Total Discarded TLVs            = 0
##&nbsp;Total Unrecognized TLVs         = 342
##&nbsp;Total Ageouts                   = 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_ageouts' => '0',
##&nbsp;            'total_discarded_frames_received' => '0',
##&nbsp;            'total_frames_transmitted' => '19',
##&nbsp;            'total_discarded_tlvs' => '0',
##&nbsp;            'total_frames_received' => '35250',
##&nbsp;            'total_unrecognized_tlvs' => '342',
##&nbsp;            'total_error_frames_received' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lldptool -t -n -i eth1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Chassis ID TLV
##&nbsp;	MAC: 00:22:90:74:9c:7f
##&nbsp;Port ID TLV
##&nbsp;	Ifname: Gi1/40
##&nbsp;Time to Live TLV
##&nbsp;	120
##&nbsp;System Name TLV
##&nbsp;	net-4948-6.lab.netapp.com
##&nbsp;System Description TLV
##&nbsp;	Cisco IOS Software, Catalyst 4500 L3 Switch Software (cat4500-ENTSERVICESK9-M), Version 12.2(53)SG8, RELEASE SOFTWARE (fc2)
##&nbsp;Technical Support: http://www.cisco.com/techsupport
##&nbsp;Copyright (c) 1986-2012 by Cisco Systems, Inc.
##&nbsp;Compiled Wed 05-Sep-12 12:21 by
##&nbsp;Port Description TLV
##&nbsp;	GigabitEthernet1/40
##&nbsp;System Capabilities TLV
##&nbsp;	System capabilities:  Bridge, Router
##&nbsp;	Enabled capabilities: Bridge, Router
##&nbsp;Management Address TLV
##&nbsp;	IPv4: 172.22.200.114
##&nbsp;	Ifindex: 62
##&nbsp;Unidentified Org Specific TLV
##&nbsp;	OUI: 0x0080c2, Subtype: 1, Info: 0001
##&nbsp;MAC/PHY Configuration Status TLV
##&nbsp;	Auto-negotiation supported and enabled
##&nbsp;	PMD auto-negotiation capabilities: 0x6c03
##&nbsp;	MAU type: 1000 BaseTFD
##&nbsp;End of LLDPDU TLV
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'chassis_id_tlv' => [
##&nbsp;                                  {
##&nbsp;                                    'mac' => '00:22:90:74:9c:7f'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'management_address_tlv' => [
##&nbsp;                                          {
##&nbsp;                                            'ifindex' => '62',
##&nbsp;                                            'ipv4' => '172.22.200.114'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'system_name_tlv' => 'net-4948-6.lab.netapp.com',
##&nbsp;            'time_to_live_tlv' => '120',
##&nbsp;            'mac_phy_configuration_status_tlv' => [
##&nbsp;                                                    {
##&nbsp;                                                      'mau_type' => '1000 BaseTFD',
##&nbsp;                                                      'auto_negotiation_supported' => 'enabled',
##&nbsp;                                                      'pmd_auto-negotiation_capabilities' => '0x6c03'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;            'system_description_tlv' => 'Cisco IOS Software, Catalyst 4500 L3 Switch Software (cat4500-ENTSERVICESK9-M), Version 12.2(53)SG8, RELEASE SOFTWARE (fc2) Technical Support: http://www.cisco.com/techsupport Copyright (c) 1986-2012 by Cisco Systems, Inc. Compiled Wed 05-Sep-12 12:21 by',
##&nbsp;            'port_id_tlv' => [
##&nbsp;                               {
##&nbsp;                                 'ifname' => 'Gi1/40'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'port_description_tlv' => 'GigabitEthernet1/40',
##&nbsp;            'unidentified_org_specific_tlv' => [
##&nbsp;                                                 {
##&nbsp;                                                   'info' => '0001',
##&nbsp;                                                   'subtype' => '1',
##&nbsp;                                                   'oui' => '0x0080c2'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'system_capabilities_tlv' => [
##&nbsp;                                           {
##&nbsp;                                             'enabled_capabilities' => [
##&nbsp;                                                                         'Bridge',
##&nbsp;                                                                         'Router'
##&nbsp;                                                                       ],
##&nbsp;                                             'system_capabilities' => [
##&nbsp;                                                                        'Bridge',
##&nbsp;                                                                        'Router'
##&nbsp;                                                                      ]
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
sub lldptool {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my (
        @lines, $line, @result_array, $cmd, $main_head,
        @key2,  @temp, @array1,       @array2
    );
    @lines = get_lines( raw_output => $output, response_object => $resp );
    my $cmd = $resp->get_command();
    if ( $cmd =~ /-t|-V/ ) {
        @lines = get_lines_tree(
            raw_output      => $output,
            response_object => $resp
        );
        foreach $line (@lines) {
            if (   $line =~ /^(\w+.*)/
                && $line !~ /Technical\s+Support|Copyright|Compiled/ )
            {
                $main_head = $1;
                $main_head =~ s/\//_/;
                $main_head = to_key_name( item => $main_head );
            } elsif ($line =~ /^\s+(.*)/
                || $line =~ /((?:Technical\s+Support|Copyright|Compiled).*)/ )
            {
                my $value = $1;
                if ( $value =~ /^(\w+\s*\w*):\s+(\S+)$/ && $value !~ /Technical Support/ ) {
                     $result_array[0]->{$main_head}[0]
                      ->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $value =~
                    /(System\s+capabilities|Enabled\s+capabilities):\s+(.*)/ )
                {
                    my $key    = to_key_name( item => $1 );
                    my $value  = $2;
                    my @values = split( /,\s+/, $value );
                    $result_array[0]->{$main_head}[0]->{$key} = \@values;
                } elsif ( $value =~ /(OUI.*)/ ) {
                    @key2 = split( /,\s+/, $value );
                    foreach my $value1 (@key2) {
                        my @temp = split( /:\s+/, $value1 );
                        $result_array[0]->{$main_head}[0]
                          ->{ to_key_name( item => $temp[0] ) } = $temp[1];
                    }
                } elsif ( $value =~
                    /((?:Cisco IOS|Technical\s+Support|Copyright|Compiled).*)/ )
                {
                    push( @array1, $1 );
                    my $index = join( ' ', @array1 );
                    $result_array[0]->{$main_head} = $index;
                } elsif ( $value =~ /(?:(Auto-negotiation|PMD|MAU).*)/ ) {
                    push( @array2, $1 );
                    my $regex = ( $1 =~ /Auto-negotiation/ ) ? 'and' : ':';
                    my ( $key, $value ) = split( $regex, $value );
                    $key =~ s/-/_/g;
                    $key =~
s/Auto_negotiation\s+supported\s+/auto_negotiation_supported/g;
                    $value =~ s/^\s+//;
                    $result_array[0]->{$main_head}->[0]
                      ->{ to_key_name( item => $key ) } = $value;
                } else {
                    $result_array[0]->{$main_head} = $1;
                }
            }
        }
    }
    elsif ( $cmd =~ /-v/ ) {
        $output =~ /lldptool\s+v(\d+\.\d+\.\d+)/i;
        $result_array[0]->{version} = $1;
    }
    elsif ( $cmd =~ /-S|-L|-l/ ) {
        foreach $line (@lines) {
            my @values = split( /\s*=\s*/, $line );
            $result_array[0]->{ to_key_name( item => $values[0] ) } =
              $values[1];
        }
    }
    return \@result_array;
} ## end sub lldptool

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error       = undef;
    if ( $raw_output =~ /missing\s+argument\s+for\s+option.*/i ) {
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output =~
/Device\s+not\s+found.*|invalid\s+option|[Uu]sage|Invalid\s+agent.*/i
      )
    {
        $error         = "InvalidParam";
    } elsif ( $raw_output =~/command\s+not\s+found/) {
          $error         = "CommandNotFound";
      }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

sub get_lines_tree(%) {
    my %args       = @_;
    my $raw_output = $args{'raw_output'};
    my $first_line = $args{'first_line'};
    my $can_throw  = $args{'can_throw'};
    my @lines;

    if (
        $raw_output = snip_only_required_output(
            'raw_output' => $raw_output,
            'delimiter'  => $first_line
        )
      )
    {
        @lines = split( /\r*\n+/, $raw_output );
        map( s/\s+$//, @lines );
        map( s/\cM//g, @lines );
        @lines = grep ( /\S+/, @lines );
    }
    elsif ($can_throw) {
        my $response_object = $args{'response_object'};
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "output is not in the expected format",
            response_object => $response_object
        );
    }
    return @lines;
} ## end sub get_lines_tree(%)

1;
