#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/lvmdiskscan.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux lvmdiskscan command output
##
## @description
##    This is a library to parse the raw output of Linux lvmdiskscan command.
##
## @synopsis
##    use lvmdiskscan;
##
##    lvmdiskscan(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::lvmdiskscan;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    my $error_message;
    if ( $raw_output =~ /invalid option/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

    return $error;
} ## end sub _check_error

## @name ::lvmdiskscan
##
## @summary Parses the raw output from the "lvmdiskscan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lvmdiskscan with any of its options as described in the
##    corresponding cdef file 'lvmdiskscan.cdefs'
## @synopsis
##      lvmdiskscan(%args);
## INPUTS TO METHOD CALL 'lvmdiskscan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'partition' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lvmdiskscan -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  /dev/ramdisk                    [       16.00 MB]
##&nbsp;  /dev/vol_grp2/lvol0             [       20.00 MB]
##&nbsp;  /dev/ram                        [       16.00 MB]
##&nbsp;  /dev/sda1                       [      101.94 MB] LVM physical volume
##&nbsp;  /dev/vol_grp2/lvol1             [       20.00 MB]
##&nbsp;  /dev/ram2                       [       16.00 MB]
##&nbsp;  /dev/sda2                       [        4.88 GB]
##&nbsp;  /dev/mapper/vol_grp2-lvol0-real [       20.00 MB]
##&nbsp;  /dev/ram3                       [       16.00 MB]
##&nbsp;  /dev/root                       [       29.29 GB]
##&nbsp;  /dev/mapper/vol_grp2-lvol1-cow  [       20.00 MB]
##&nbsp;  /dev/ram4                       [       16.00 MB]
##&nbsp;  /dev/sda4                       [       40.26 GB]
##&nbsp;  /dev/ram5                       [       16.00 MB]
##&nbsp;  /dev/ram6                       [       16.00 MB]
##&nbsp;  /dev/ram7                       [       16.00 MB]
##&nbsp;  /dev/ram8                       [       16.00 MB]
##&nbsp;  /dev/ram9                       [       16.00 MB]
##&nbsp;  /dev/ram10                      [       16.00 MB]
##&nbsp;  /dev/ram11                      [       16.00 MB]
##&nbsp;  /dev/ram12                      [       16.00 MB]
##&nbsp;  /dev/ram13                      [       16.00 MB]
##&nbsp;  /dev/ram14                      [       16.00 MB]
##&nbsp;  /dev/ram15                      [       16.00 MB]
##&nbsp;  /dev/sdb1                       [        2.00 GB]
##&nbsp;  /dev/sdb2                       [      147.05 GB]
##&nbsp;  5 disks
##&nbsp;  20 partitions
##&nbsp;  0 LVM physical volume whole disks
##&nbsp;  1 LVM physical volume
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm_physical_volume' => '1',
##&nbsp;            'lvm_physical_volume_whole_disks' => '0',
##&nbsp;            'scanned_devices' => [
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ramdisk',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/vol_grp2/lvol0',
##&nbsp;                                     'size' => '20.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/sda1',
##&nbsp;                                     'state' => 'LVM physical volume',
##&nbsp;                                     'size' => '101.94 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/vol_grp2/lvol1',
##&nbsp;                                     'size' => '20.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram2',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/sda2',
##&nbsp;                                     'size' => '4.88 GB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/mapper/vol_grp2-lvol0-real',
##&nbsp;                                     'size' => '20.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram3',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/root',
##&nbsp;                                     'size' => '29.29 GB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/mapper/vol_grp2-lvol1-cow',
##&nbsp;                                     'size' => '20.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram4',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/sda4',
##&nbsp;                                     'size' => '40.26 GB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram5',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram6',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram7',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram8',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram9',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram10',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram11',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram12',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram13',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram14',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/ram15',
##&nbsp;                                     'size' => '16.00 MB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/sdb1',
##&nbsp;                                     'size' => '2.00 GB'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'device' => '/dev/sdb2',
##&nbsp;                                     'size' => '147.05 GB'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'disk_count' => '5',
##&nbsp;            'partitions' => '20'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lvmdiskscan {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $c = 0;
    my ( @values, @result_array );
    foreach my $line (@lines) {

        if ( $line =~ /\[.*\]/ ) {
            $line =~ s/\[//;
            $line =~ s/\]//;
            @values = split( /\s+/, $line );
            my $len = @values - 3;
            my @temp = splice( @values, 3, $len );
            $values[3] = join( ' ', @temp );
            $result_array[0]->{scanned_devices}->[$c]->{device} = $values[0];
            $result_array[0]->{scanned_devices}->[$c]->{size}
                = "$values[1] $values[2]";
            $result_array[0]->{scanned_devices}->[$c]->{state} = $values[3]
                if ( $values[3] );
            $c++;
        } elsif ( $line =~ /(\d+)\s+(.*)/ ) {
            my $key = $2;
            $key = "disk_count" if ( $key eq "disks" );
            $result_array[0]->{ to_key_name( item => $key ) } = $1;
        }
    }
    return \@result_array;
} ## end sub lvmdiskscan

1;
