#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/lvs.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse lvs command
##
## @description
##    This is a library to parse the raw output of Linux lvs command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lvs" will lvs()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::lvs;
##
##    lvs(%args);
##
## @author vidyat@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::lvs;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidOptionException;
use NACL::APISet::Exceptions::MissingValueException;
use NACL::APISet::Exceptions::FailedAndNotFoundException;
use NACL::APISet::Exceptions::NoMatchingEntriesException;
use NACL::APISet::Response::CLI::ParserUtils;

use strict;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /invalid option|unrecognized option/i ) {
        $error         = "InvalidOption";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /Invalid path for Logical Volume|Can't report LV and PV fields at the same time/i
        ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif (
        $raw_output =~ /Invalid units specification|Unrecognised field/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option requires an argument/i ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Volume group \S+ not found/i ) {
        $error         = "FailedAndNotFound";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No volume groups found/i ) {
        $error         = "NoMatchingEntries";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::lvs
##
## @summary Parses the raw output from the "lvs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lvs with any of its options as described in the
##    corresponding cdef file 'lvs.cdefs'
## @synopsis
##      lvs(%args);
## INPUTS TO METHOD CALL 'lvs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> '--trustcache' </td> <td> 'trustcache' </td> <td> '1' </td> </tr>
## <tr> <td> '--unbuffered' </td> <td> 'unbuffered' </td> <td> '1' </td> </tr>
## <tr> <td> '--segments' </td> <td> 'segments' </td> <td> '1' </td> </tr>
## <tr> <td> '--rows' </td> <td> 'rows' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'partial' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '--unquoted' </td> <td> 'unquoted' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '--nosuffix' </td> <td> 'nosuffix' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignorelockingfailure' </td> <td> '1' </td> </tr>
## <tr> <td> '--nameprefixes' </td> <td> 'nameprefixes' </td> <td> '1' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--noheadings' </td> <td> 'noheadings' </td> <td> '1' </td> </tr>
## <tr> <td> '-O' </td> <td> 'sort' </td> <td> '0' </td> </tr>
## <tr> <td> '--aligned' </td> <td> 'aligned' </td> <td> '1' </td> </tr>
## <tr> <td> '--separator' </td> <td> 'separator' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume_group_name' </td> <td> '0' </td> </tr>
## <tr> <td> '--units' </td> <td> 'units' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lvs -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   LSize  Origin Snap%  Move Log Copy%  Convert
##&nbsp;  lv_home vg_lnx21945 -wi-ao 20.11g
##&nbsp;  lv_root vg_lnx21945 -wi-ao 50.00g
##&nbsp;  lv_swap vg_lnx21945 -wi-ao  3.94g
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --nosuffix --units M
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   LSize    Origin Snap%  Move Log Copy%  Convert
##&nbsp;  lv_home vg_lnx21945 -wi-ao 21588.08
##&nbsp;  lv_root vg_lnx21945 -wi-ao 53687.09
##&nbsp;  lv_swap vg_lnx21945 -wi-ao  4227.86
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '21588.08',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '53687.09',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '4227.86',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs -o lv_uuid
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV UUID
##&nbsp;  S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF
##&nbsp;  vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN
##&nbsp;  u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lv_uuid' => [
##&nbsp;                           'S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF',
##&nbsp;                           'vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN',
##&nbsp;                           'u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --segments
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   #Str Type   SSize
##&nbsp;  lv_home vg_lnx21945 -wi-ao    1 linear 20.11g
##&nbsp;  lv_root vg_lnx21945 -wi-ao    1 linear 50.00g
##&nbsp;  lv_swap vg_lnx21945 -wi-ao    1 linear  3.94g
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ssize' => '20.11g',
##&nbsp;            'str' => '1',
##&nbsp;            'type' => 'linear',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ssize' => '50.00g',
##&nbsp;            'str' => '1',
##&nbsp;            'type' => 'linear',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ssize' => '3.94g',
##&nbsp;            'str' => '1',
##&nbsp;            'type' => 'linear',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs -O -Attr
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   LSize  Origin Snap%  Move Log Copy%  Convert
##&nbsp;  lv_root vg_lnx21945 -wi-ao 50.00g
##&nbsp;  lv_home vg_lnx21945 -wi-ao 20.11g
##&nbsp;  lv_swap vg_lnx21945 -wi-ao  3.94g
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --rows
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV lv_home lv_root lv_swap
##&nbsp;  VG vg_lnx21945 vg_lnx21945 vg_lnx21945
##&nbsp;  Attr -wi-ao -wi-ao -wi-ao
##&nbsp;  LSize 20.11g 50.00g 3.94g
##&nbsp;  Origin
##&nbsp;  Snap%
##&nbsp;  Move
##&nbsp;  Log
##&nbsp;  Copy%
##&nbsp;  Convert
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => [
##&nbsp;                         '20.11g',
##&nbsp;                         '50.00g',
##&nbsp;                         '3.94g'
##&nbsp;                       ],
##&nbsp;            'attr' => [
##&nbsp;                        '-wi-ao',
##&nbsp;                        '-wi-ao',
##&nbsp;                        '-wi-ao'
##&nbsp;                      ],
##&nbsp;            'vg' => [
##&nbsp;                      'vg_lnx21945',
##&nbsp;                      'vg_lnx21945',
##&nbsp;                      'vg_lnx21945'
##&nbsp;                    ],
##&nbsp;            'lv' => [
##&nbsp;                      'lv_home',
##&nbsp;                      'lv_root',
##&nbsp;                      'lv_swap'
##&nbsp;                    ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --units k
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   LSize        Origin Snap%  Move Log Copy%  Convert
##&nbsp;  lv_home vg_lnx21945 -wi-ao 21082112.00k
##&nbsp;  lv_root vg_lnx21945 -wi-ao 52428800.00k
##&nbsp;  lv_swap vg_lnx21945 -wi-ao  4128768.00k
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '21082112.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '52428800.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '4128768.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Finding all logical volumes
##&nbsp;  LV      VG          #Seg Attr   LSize  Maj Min KMaj KMin Origin Snap%  Move Copy%  Log Convert LV UUID
##&nbsp;  lv_home vg_lnx21945    1 -wi-ao 20.11g  -1  -1 253  2                                          S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF
##&nbsp;  lv_root vg_lnx21945    1 -wi-ao 50.00g  -1  -1 253  0                                          vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN
##&nbsp;  lv_swap vg_lnx21945    1 -wi-ao  3.94g  -1  -1 253  1                                          u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '2',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF',
##&nbsp;            'maj' => '-1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '0',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN',
##&nbsp;            'maj' => '-1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '1',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ',
##&nbsp;            'maj' => '-1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LVM version:     2.02.72(2) (2010-07-28)
##&nbsp;  Library version: 1.02.53 (2010-07-28)
##&nbsp;  Driver version:  4.17.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'library_version' => '1.02.53 (2010-07-28)',
##&nbsp;            'driver_version' => '4.17.0',
##&nbsp;            'lvm_version' => '2.02.72(2) (2010-07-28)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs /dev/vg_lnx21945
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV      VG          Attr   LSize  Origin Snap%  Move Log Copy%  Convert
##&nbsp;  lv_home vg_lnx21945 -wi-ao 20.11g
##&nbsp;  lv_root vg_lnx21945 -wi-ao 50.00g
##&nbsp;  lv_swap vg_lnx21945 -wi-ao  3.94g
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --separator :
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV:VG:Attr:LSize:Origin:Snap%:Move:Log:Copy%:Convert
##&nbsp;  lv_home:vg_lnx21945:-wi-ao:20.11g::::::
##&nbsp;  lv_root:vg_lnx21945:-wi-ao:50.00g::::::
##&nbsp;  lv_swap:vg_lnx21945:-wi-ao:3.94g::::::
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs --nameprefixes
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV VG Attr LSize Origin Snap% Move Log Copy% Convert
##&nbsp;  LVM2_LV_NAME='lv_home' LVM2_VG_NAME='vg_lnx21945' LVM2_LV_ATTR='-wi-ao' LVM2_LV_SIZE='20.11g' LVM2_ORIGIN='' LVM2_SNAP_PERCENT='' LVM2_MOVE_PV='' LVM2_MIRROR_LOG='' LVM2_COPY_PERCENT='' LVM2_CONVERT_LV=''
##&nbsp;  LVM2_LV_NAME='lv_root' LVM2_VG_NAME='vg_lnx21945' LVM2_LV_ATTR='-wi-ao' LVM2_LV_SIZE='50.00g' LVM2_ORIGIN='' LVM2_SNAP_PERCENT='' LVM2_MOVE_PV='' LVM2_MIRROR_LOG='' LVM2_COPY_PERCENT='' LVM2_CONVERT_LV=''
##&nbsp;  LVM2_LV_NAME='lv_swap' LVM2_VG_NAME='vg_lnx21945' LVM2_LV_ATTR='-wi-ao' LVM2_LV_SIZE='3.94g' LVM2_ORIGIN='' LVM2_SNAP_PERCENT='' LVM2_MOVE_PV='' LVM2_MIRROR_LOG='' LVM2_COPY_PERCENT='' LVM2_CONVERT_LV=''
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_home'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'vg' => 'vg_lnx21945',
##&nbsp;            'lv' => 'lv_swap'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvs -o lv_all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV UUID                                LV      Path                     Attr   Maj Min Rahead KMaj KMin KRahead LSize  #Seg Origin OSize Snap%  Copy%  Move Convert LV Tags Log Modules
##&nbsp;  S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF lv_home /dev/vg_lnx21945/lv_home -wi-ao  -1  -1   auto 253  2    128.00k 20.11g    1           0
##&nbsp;  vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN lv_root /dev/vg_lnx21945/lv_root -wi-ao  -1  -1   auto 253  0    128.00k 50.00g    1           0
##&nbsp;  u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ lv_swap /dev/vg_lnx21945/lv_swap -wi-ao  -1  -1   auto 253  1    128.00k  3.94g    1           0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'osize' => '0',
##&nbsp;            'lsize' => '20.11g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '2',
##&nbsp;            'path' => '/dev/vg_lnx21945/lv_home',
##&nbsp;            'krahead' => '128.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'lv' => 'lv_home',
##&nbsp;            'rahead' => 'auto',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'S4DY8Y-8Eot-RsFH-xuvi-xKkM-JQqE-huqxaF',
##&nbsp;            'maj' => '-1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'osize' => '0',
##&nbsp;            'lsize' => '50.00g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '0',
##&nbsp;            'path' => '/dev/vg_lnx21945/lv_root',
##&nbsp;            'krahead' => '128.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'lv' => 'lv_root',
##&nbsp;            'rahead' => 'auto',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'vHB6gq-8hLj-p8in-4Oy6-KdeX-COh1-ZMd7fN',
##&nbsp;            'maj' => '-1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'osize' => '0',
##&nbsp;            'lsize' => '3.94g',
##&nbsp;            'kmaj' => '253',
##&nbsp;            'min' => '-1',
##&nbsp;            'kmin' => '1',
##&nbsp;            'path' => '/dev/vg_lnx21945/lv_swap',
##&nbsp;            'krahead' => '128.00k',
##&nbsp;            'attr' => '-wi-ao',
##&nbsp;            'lv' => 'lv_swap',
##&nbsp;            'rahead' => 'auto',
##&nbsp;            'seg' => '1',
##&nbsp;            'lv_uuid' => 'u2S80z-1iDT-OYZ7-5Gv0-ERxf-UkOE-1UCWOJ',
##&nbsp;            'maj' => '-1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub lvs {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @key, @value, @pos );
    my ( $count1, $count2 ) = ( -1, 0 );
    my ( $head, $heading, $lines );

    shift @lines
        if ( $lines[0] =~ /Partial mode|Finding all logical volumes/ );
    foreach my $line (@lines) {
        $line =~ s/LV UUID/LV_UUID/g;
        $line =~ s/LV Tags/LV_TAGS/g;
        $line =~ s/#/ /g;
        if ( $cmd =~ /--version/ ) {
            if ( $line =~ /(.*):\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        } elsif ( $cmd =~ /-o (\S+)/ && $1 !~ /lv_all/ ) {
            $head = $1;
            if ( $line !~ /$lines[0]/ ) {
                $result_array[0]->{$head}->[$count2] = $line;
                $count2++;
            }
        } elsif ( $cmd =~ /--nameprefixes/ ) {
            my @values;
            @key = split( /\s+/, $lines[0] );
            if ( $line =~ /^LVM2_/ ) {
                $count1++;
                my @array = split( /\s+/, $line );
                foreach (@array) {
                    @value = split( /=/, $_ );
                    $value[1] = $1 if ( $value[1] =~ /\'(\S+)?\'/ );
                    push( @values, $value[1] );
                }
            }
            for ( my $j = 0; $j <= $#key; $j++ ) {
                $result_array[$count1]->{ to_key_name( item => $key[$j] ) }
                    = $values[$j]
                    if ( $values[$j] );
            }
        } elsif ( $cmd =~ /--rows/ ) {
            @value = split( /\s+/, $line );
            $head = $value[0];
            shift(@value);
            for ( my $k = 0; $k <= $#value; $k++ ) {
                $result_array[0]->{ to_key_name( item => $head ) }->[$k]
                    = $value[$k]
                    if ( defined( $value[$k] ) );
            }
        } elsif ( $cmd =~ /--separator (\S+)/ ) {
            my $var = $1;
            @key = split( $var, $lines[0] );
            if ( $line !~ /^LV|LV_UUID/ ) {
                $count1++;
                @value = split( $var, $line );
            }
            for ( my $n = 0; $n <= $#key; $n++ ) {
                $result_array[$count1]->{ to_key_name( item => $key[$n] ) }
                    = $value[$n]
                    if defined( $value[$n] );
            }
        } else {
            @key = split( /\s+/, $lines[0] );
            if ( $line =~ /^LV|LV_UUID/ ) {
                while ( $line =~ m/\s+/g ) {
                    my $p = pos($line);
                    push( @pos, $p );
                }
            } elsif ( $line !~ /^LV|LV_UUID/ ) {
                $count1++;
                my $start = 0;
                my @values;
                for ( my $i = 0; $i <= @pos; $i++ ) {
                    my $value;
                    my $end    = $pos[$i] - 1;
                    my $start  = $pos[ $i - 1 ] if ( $i != 0 );
                    my $length = $end - $start;
                    if ( $i != @pos ) {
                        $value = substr( $line, $start, $length );
                    } else {
                        $value = substr( $line, $start );
                    }
                    $value =~ s/^\s+//;
                    $value =~ s/\s+$//;
                    push( @values, $value );
                }
                for ( my $m = 0; $m <= $#key; $m++ ) {
                    $result_array[$count1]
                        ->{ to_key_name( item => $key[$m] ) } = $values[$m]
                        if ( ( defined $values[$m] )
                        && ( $values[$m] ne '' ) );
                }
            }
        }
    }
    return \@result_array;
} ## end sub lvs
1;

