#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux million command
##
## @description
##    This is a library to parse the raw output of Linux million command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "million" will be million()
##
## @synopsis
##    use million;
##
##    million(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::million;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidOptionException;
use NACL::APISet::Exceptions::MissingValueException;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::million
##
## @summary Parses the raw output from the "million.pl" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from million with any of its options as described in the
##    corresponding cdef file 'million.cdefs'
## @synopsis
##      million(%args);
## INPUTS TO METHOD CALL 'million' ON APISET OBJECT
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /x/eng/localtest/noarch/agnostic/cluster/fs/million/, /u/username/p4/CLIENTNAME/test/cluster/fs/million/
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-number' </td> <td> 'number' </td> <td> '0' </td> </tr>
## <tr> <td> '-iterations' </td> <td> 'iterations' </td> <td> '0' </td> </tr>
## <tr> <td> '-testdir' </td> <td> 'testdir' </td> <td> '0' </td> </tr>
## <tr> <td> '-namelen' </td> <td> 'namelen' </td> <td> '0' </td> </tr>
## <tr> <td> '-size' </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> '-debug' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /x/eng/localtest/noarch/agnostic/cluster/fs/million/million.pl -type S -number 100 -size 15 -namelen 20 -iterations 1 -debug
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;testdir is /u/vidyat
##&nbsp;namelen is 20
##&nbsp;nof is 100
##&nbsp;filesize is 15
##&nbsp;iterations is 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesize' => '15',
##&nbsp;            'iterations' => '1',
##&nbsp;            'testdir' => '/u/vidyat',
##&nbsp;            'nof' => '100',
##&nbsp;            'namelen' => '20'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = /x/eng/localtest/noarch/agnostic/cluster/fs/million/million.pl -type H -number 1000 -iterations 2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub million {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my ( @result_array, @lines );
    @lines = get_lines( raw_output => $output, response_object => $resp );
    my $cmd = $resp->get_command();
    if ( $cmd =~ /debug/ ) {
        foreach my $line (@lines) {
            my ( $head, $value ) = split( /\s+is\s+/, $line );
            $result_array[0]->{ to_key_name( item => $head ) } = $value;
        }
        return \@result_array;
    } else {
        return [];
    }
} ## end sub million

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error       = undef;
    if ( $raw_output =~ /Option\s+\w+\s+requires an argument/i ) {
        $error = "MissingValue";
    } elsif ( $raw_output =~ /Unknown option/i ) {
        $error = "InvalidOption";
    } elsif ( $raw_output
        =~ /Value\s+\"\w+\"\s+invalid for option\s+\w+\s+\(number expected\)|Unknown filetype/i
        ) {
        $error = "InvalidParamValue";
    } elsif ( $raw_output =~ /Command not found/i ) {
        $error = "CommandNotFound";
    } elsif (
        $raw_output =~ /cannot create directory|Error parsing command line/i )
    {
        $error = "CommandFailed";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
