#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse mkfifo command
##
## @description
##    This is a library to parse the raw output of mkfifo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::mkfifo;
##
##    mkfifo(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::mkfifo;

use strict;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Include exceptions which will be thrown
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

sub _check_error {
    my %args          = @_;
    my $resp          = $args{response_object};
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = undef;
    my $error         = undef;
    if ( $raw_output =~ /((mkfifo:(\s*(.*)File\s+exists))|\s*Sorry)/ )
    {
        $error = "CommandFailed";
    }elsif ( $raw_output =~ /mkfifo:(\s*invalid\s+mode)/ ) {
        $error = "InvalidParamValue";
    }elsif ( $raw_output =~ /\s*invalid\s+option:/ ) {
        $error = "InvalidParam";
    }elsif($raw_output =~ /mkfifo:(\s*missing\s+operand|)/){
	    $error = "MissingRequiredParam";
	}
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
}

## @name ::mkfifo
##
## @summary Parses the raw output from the "mkfifo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mkfifo with any of its options as described in the
##    corresponding cdef file 'mkfifo.cdefs'
## @synopsis
##      mkfifo(%args);
## INPUTS TO METHOD CALL 'mkfifo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'namepipe' </td> <td> '0' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'context' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mkfifo
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;: missing operand
##&nbsp;Try `mkfifo --help' for more information.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mkfifo temp3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub mkfifo {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my @result_array;
    _check_error(
        "response_object" => $resp,
        "raw_output"      => $output,
        "command"         => $command,
    );

    return \@result_array;
}
1;

