#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/mkfs.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux mkfs commands
##
## @description
##    This is a library to parse the raw output of Linux mkfs commands.
##
## @synopsis
##    use mkfs;
##
## mkfs(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::mkfs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ( $output =~ /(.* Command not found.*)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $output =~ /(.*(No such file).*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /(.*(invalid).*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /(.*(mounted).*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /(.*will not make a filesystem here.*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::mkfs
##
## @summary Parses the raw output from the "mkfs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mkfs with any of its options as described in the
##    corresponding cdef file 'mkfs.cdefs'
## @synopsis
##      mkfs(%args);
## INPUTS TO METHOD CALL 'mkfs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'bytes-per-inode' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fragment-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'file-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'volume-label' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'creator-os' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'block-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'blocks-per-group' </td> <td> '0' </td> </tr>
## <tr> <td> '-O' </td> <td> 'feature' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'fs-revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'previous-mounted-directory' </td> <td> '0' </td> </tr>
## <tr> <td> '-j' </td> <td> 'journal' </td> <td> '1' </td> </tr>
## <tr> <td> '-qvSV' </td> <td> 'library-version' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'reserved-blocks-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'number-of-inodes' </td> <td> '0' </td> </tr>
## <tr> <td> '-J' </td> <td> 'journal-options' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'blocks-count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'fstype' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /sbin/mkfs -V
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
#&nbsp;             'linux_release' => 'util-linux 2.13-pre7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = /sbin/mkfs -b 2048 -f 1024 -i 1024 -j -J size=2 -N 2 -m 20
##&nbsp;         -o Linux -g 16384 -L vol2 -M /mnt_fs -O has_journal -r 1 /dev/sda2 5379
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fragment_size' => '4096',
##&nbsp;            'block_groups' => '12',
##&nbsp;             'filesystem_status' => 'automatically checked',
##&nbsp;             'blocks' => '375519',
##&nbsp;             'reserved_blocks' => '18775',
##&nbsp;             'block_size' => '4096',
##&nbsp;             'blocks_per_group' => '32768',
##&nbsp;             'overriding_command' => 'tune2fs -c or -i ',
##&nbsp;             'superblock_backups_stored_on_blocks' => '98304, 163840, 229376, 294912',
##&nbsp;             'command_info' => {
##&nbsp;                                 'library_version' => '1.39',
##&nbsp;                                 'date_of_release' => '29-May-2006',
##&nbsp;                                 'name' => 'mke2fs'
##&nbsp;                               },
##&nbsp;             'os_type' => 'Linux',
##&nbsp;             'journal' => {
##&nbsp;                            'info' => 'Creating journal ',
##&nbsp;                            'no_of_blocks' => '8192',
##&nbsp;                            'status' => 'done'
##&nbsp;                          },
##&nbsp;             'no_of_days' => '180',
##&nbsp;             'inodes' => '187776',
##&nbsp;             'filesystem_type' => 'backups',
##&nbsp;             'inodes_per_group' => '15648',
##&nbsp;             'no_of_mounts' => '21',
##&nbsp;             'fragments_per_group' => '32768',
##&nbsp;             'first_data_block' => '0',
##&nbsp;             'maximum_filesystem_blocks' => '385875968',
##&nbsp;             'reserved_blocks_percentage' => '5.00%',
##&nbsp;             'writing_inode_tables' => '0',
##&nbsp;             'writing_superblocks_and_filesystem_accounting_information' => 'done'
##&nbsp;           }
##&nbsp;         ];
sub mkfs {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "/sbin/mkfs",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );

    }

    my @lines = get_lines( 'raw_output' => $output );
    my %hash = ();
    foreach my $line (@lines) {
        if ( $command =~ /-V/ ) {
            if ( $line =~ /.*\((\w+.*)\)/ ) {
                $hash{'linux_release'} = $1;
            }
        } else {
            if ( $line =~ /^(Warning):([\s+\w+]*).\s+\w+\s+(-\w).*/ ) {
                $hash{$1} = $2;
                $hash{'unsupported_option'} = $3;
            } elsif ( $line =~ /^([\w+\s+?]*\w+)[=:]\s*(\w+)/ ) {
                my $temp = $1;
                my @words = split( /\s+/, $temp );
                $temp = join( "_", @words );
                $temp =~ tr/A-Z/a-z/;
                $hash{$temp} = $2;
            } elsif (
                $line =~ /^(\d+)\s+((\w+\s*)+)\,*\s+(\d+)\s+((\w+\s*)+)$/ ) {
                my $temp = $2;
                my @words = split( /\s+/, $temp );
                $temp = join( "_", @words );
                $temp =~ tr/A-Z/a-z/;
                $hash{$temp} = $1;
                $temp = $5;
                @words = split( /\s+/, $temp );
                $temp = join( "_", @words );
                $temp =~ tr/A-Z/a-z/;
                $hash{$temp} = $4;
            } elsif ( $line =~ /^(\d+)\s+((\w+\s+)+\w+)/ ) {
                my $temp = $2;
                my @words = split( /\s+/, $temp );
                $temp = join( "_", @words );
                $temp =~ tr/A-Z/a-z/;
                $hash{$temp} = $1;
            } elsif ( $line =~ /(\d+)(\s+\w+\s+)\((\S+)\).*/ ) {
                $hash{'reserved_blocks_percentage'} = $3;
                $hash{'reserved_blocks'}            = $1;
            } elsif ( $line =~ /((\w+\s+)*)\((\d+).*:\s+(\w+)/ ) {
                $hash{'journal'}->{'info'}         = $1;
                $hash{'journal'}->{'status'}       = $4;
                $hash{'journal'}->{'no_of_blocks'} = $3;
            } elsif ( $line =~ /^.*(automatically checked).*?(\d+).*/ ) {
                $hash{'filesystem_status'} = $1;
                $hash{'no_of_mounts'}      = $2;
            } elsif (
                $line =~ /^(\d+)\s+\w+[,]\D+\.\s+\w+\s((\S+\s[-]\w+\s+)*).*/ )
            {
                $hash{'no_of_days'}         = $1;
                $hash{'overriding_command'} = $2;
            } elsif ( $line =~ /((\w+\s+)+)\((\S+\s+\w+)\):\s+(\w+)/ ) {
                $hash{'bad_blocks'}->{'info'}      = $1;
                $hash{'bad_blocks'}->{'status'}    = $4;
                $hash{'bad_blocks'}->{'test_type'} = $3;
            } elsif ( $line =~ /^(mk\S+)\s+(\S+)\s+\((\S+)\)/ ) {
                $hash{'command_info'}->{'name'}            = $1;
                $hash{'command_info'}->{'library_version'} = $2;
                $hash{'command_info'}->{'date_of_release'} = $3;
            } elsif ( $line =~ /\s+(\w+)\s+\w+\s+(\w+\s+)+(\S+)/ ) {
                $hash{'filesystem_type'} = $1;
            } elsif ( $line =~ /\s+((\S+\s+)+\d+)/ ) {
                $hash{'superblock_backups_stored_on_blocks'} = $1;
            }
        }
    }

    push( @result_array, \%hash );
    return \@result_array;    #to be updated to return a reference
} ## end sub mkfs

1;
