#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse modprobe command output
##
## @description
##    This is a library to parse the raw output of  modprobe command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::modprobe;
##
##    modprobe(%args);
##
## @author sheethal@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::modprobe;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;

## @name ::modprobe
##
## @summary Parses the raw output from the "modprobe" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from modprobe with any of its options as described in the 
##    corresponding cdef file 'modprobe.cdefs'
## @synopsis
##      modprobe(%args);
## INPUTS TO METHOD CALL 'modprobe' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'dir_name' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'remove' </td> <td> '1' </td> </tr> 
## <tr> <td> '--ignore-remove' </td> <td> 'ignore_remove' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'config_file' </td> <td> '0' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'use_blacklist' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'ignore' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'wild_card' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'module' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'set_version' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'directory_name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'name' </td> <td> '1' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'dry_run' </td> <td> '1' </td> </tr> 
## <tr> <td> '--ignore-install' </td> <td> 'ignore_install' </td> <td> '1' </td> </tr> 
## <tr> <td> '--dump-modversions' </td> <td> 'dump_modversions' </td> <td> '1' </td> </tr> 
## <tr> <td> '-D' </td> <td> 'show_depends' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'showconfig' </td> <td> '1' </td> </tr> 
## <tr> <td> '--force-vermagic' </td> <td> 'force_vermagic' </td> <td> '1' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '1' </td> </tr> 
## <tr> <td> '--first-time' </td> <td> 'first_time' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'syslog' </td> <td> '1' </td> </tr> 
## <tr> <td> '-C' </td> <td> 'config' </td> <td> '1' </td> </tr> 
## <tr> <td> '--force-modversion' </td> <td> 'force_modversion' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub modprobe {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    return [];
} ## end sub modprobe

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ($error, $error_message) = undef;
    if ($raw_output =~ /(Usage:)/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
