#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux mount command
##
## @description
##    This is a library to parse the raw output of Linux mount command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mount" will be mount
##
## @synopsis
##    use mount;
##
##    mount(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::mount;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = $raw_output;
    if ($raw_output =~ /$command.*:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}

## @name ::mount
##
## @summary Parses the raw output from the "mount" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mount with any of its options as described in the 
##    corresponding cdef file 'mount.cdefs'
## @synopsis
##      mount(%args);
## INPUTS TO METHOD CALL 'mount' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'file-system-type' </td> </tr> 
## <tr> <td> '--move' </td> <td> 'move' </td> </tr> 
## <tr> <td> '--make-rshared' </td> <td> 'make-rshared' </td> </tr> 
## <tr> <td> '--make-slave' </td> <td> 'make-slave' </td> </tr> 
## <tr> <td> '-U' </td> <td> 'uuid' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> </tr> 
## <tr> <td> '--make-private' </td> <td> 'make-private' </td> </tr> 
## <tr> <td> '--make-unbindable' </td> <td> 'make-unbindable' </td> </tr> 
## <tr> <td> '--make-rprivate' </td> <td> 'make-rprivate' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'mount-point' </td> </tr> 
## <tr> <td> '-O' </td> <td> 'optlist' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'list' </td> </tr> 
## <tr> <td> '--make-runbindable' </td> <td> 'make-runbindable' </td> </tr> 
## <tr> <td> '--bind' </td> <td> 'bind' </td> </tr> 
## <tr> <td> '--make-shared' </td> <td> 'make-shared' </td> </tr> 
## <tr> <td> '--make-rslave' </td> <td> 'make-rslave' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'label' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mount 10.72.39.178:/ ~/api-test-dir
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mount -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/sda6',
##&nbsp;            'options' => '[/]',
##&nbsp;            'type' => 'ext3 (rw)',
##&nbsp;            'mount_point' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'proc',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'proc',
##&nbsp;            'mount_point' => '/proc'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'sysfs',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'sysfs',
##&nbsp;            'mount_point' => '/sys'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'devpts',
##&nbsp;            'options' => '(rw,gid=5,mode=620)',
##&nbsp;            'type' => 'devpts',
##&nbsp;            'mount_point' => '/dev/pts'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/sda5',
##&nbsp;            'options' => '[/tmp]',
##&nbsp;            'type' => 'ext3 (rw)',
##&nbsp;            'mount_point' => '/tmp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/sda2',
##&nbsp;            'options' => '[/var]',
##&nbsp;            'type' => 'ext3 (rw)',
##&nbsp;            'mount_point' => '/var'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/sda1',
##&nbsp;            'options' => '[/boot]',
##&nbsp;            'type' => 'ext3 (rw)',
##&nbsp;            'mount_point' => '/boot'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'tmpfs',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'tmpfs',
##&nbsp;            'mount_point' => '/dev/shm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'none',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'binfmt_misc',
##&nbsp;            'mount_point' => '/proc/sys/fs/binfmt_misc'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'sunrpc',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'rpc_pipefs',
##&nbsp;            'mount_point' => '/var/lib/nfs/rpc_pipefs'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'rtpinf02.rtp.netapp.com:/vol/local/i386-linux2.0',
##&nbsp;            'options' => '(rw,intr,bg,rsize=8192,wsize=8192,nfsvers=3,addr=10.60.132.47)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/usr/local'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'rtpinf02.rtp.netapp.com:/vol/local/share',
##&nbsp;            'options' => '(rw,intr,bg,rsize=8192,wsize=8192,nfsvers=3,addr=10.60.132.47)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/usr/local/share'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'rtpinf01.rtp.netapp.com:/vol/software/arch/x86_64-redhat-rhel5',
##&nbsp;            'options' => '(ro,noatime,intr,rsize=32768,wsize=32768,nfsvers=3,tcp,timeo=600,addr=10.60.132.45)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/usr/software'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'nfsd',
##&nbsp;            'options' => '(rw)',
##&nbsp;            'type' => 'nfsd',
##&nbsp;            'mount_point' => '/proc/fs/nfsd'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '10.72.40.177:/jun',
##&nbsp;            'options' => '(rw,addr=10.72.40.177)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/root/mount/mount_vol_issh_vsim'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'rtpinf02.rtp.netapp.com:/vol/localtest',
##&nbsp;            'options' => '(rw,intr,hard,rsize=8192,wsize=8192,nfsvers=3,timeo=600,tcp,addr=10.60.132.47)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/x/eng/localtest'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '10.72.39.178:/',
##&nbsp;            'options' => '(rw,addr=10.72.39.178)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/root/e'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '10.72.39.178:/',
##&nbsp;            'options' => '(rw,hard,intr,addr=10.72.39.178)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/t/10.72.39.178/root'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => 'agate-be.rtp.netapp.com:/vol/rtpeng_testbeds',
##&nbsp;            'options' => '(rw,intr,hard,rsize=8192,wsize=8192,nfsvers=3,timeo=600,tcp,addr=10.60.131.13)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/x/eng/rtpeng-testbeds'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '10.72.39.178:/',
##&nbsp;            'options' => '(rw,addr=10.72.39.178)',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/root/api-test-dir'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mount {
    my %args = @_;
    
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output,command=>"mount",
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @lines = get_lines(raw_output=>$output,first_line=>"");
    
    foreach my $line (@lines) {
        my %hash;
            
        if ( $command eq 'mount' || $command eq 'mount -l') { # -l is specific to linux
            if ($line =~ /(.*) on (.*) type (.*) (.*)/) { # Linux
                $hash{'source_path'} = $1;
                $hash{'mount_point'} = $2;
                $hash{'type'} = $3;
                $hash{'options'} = $4;			
                push (@result_array, \%hash);
            } elsif ($line =~ /(.*) on (.*)\s(.*) on (.*)/ ) { # solaris
                $hash{'mount_point'} = $1;
                $hash{'source_path'} = $2;
                $hash{'type'} = $3;
                $hash{'timestamp'} = $4;
                push (@result_array, \%hash);
            }
        } elsif ($command eq 'mount -v') { # solaris specific
            if ($line =~ /(.*) on (.*) type (.*) (.*) on (.*)/) { 
                $hash{'source_path'} = $1;
                $hash{'mount_point'} = $2;
                $hash{'type'} = $3;
                $hash{'options'} = $4;
                $hash{'timestamp'} = $5;
                push (@result_array, \%hash);
            }
        } else {
            return [];
		}
    }
    return \@result_array; #to be updated to return a reference
}

1;
