#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/netstat.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Linux netstat command"
##
## @description
##    This is a library to parse the raw output of Linux netstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "netstat" will be netstat()
##
## @synopsis
##    use netstat;
##
##    netstat(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::netstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

## @name ::netstat
##
## @summary Parses the raw output from the "netstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netstat with any of its options as described in the
##    corresponding cdef file 'netstat.cdefs'
## @synopsis
##      netstat(%args);
## INPUTS TO METHOD CALL 'netstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'numeric-hosts' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'listening' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'statistics' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'numeric-users' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'masquerade' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'delay' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'program' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'routing-info-from-cache' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'context' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'groups' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'udp' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'iface' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'timers' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'extend' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'notrim' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'numeric-ports' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'tcp' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'route' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'symbolic' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'continuous' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'routing-info-from-FIB' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'raw' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active Internet connections (w/o servers)
##&nbsp;Proto Recv-Q Send-Q Local Address               Foreign Address             State
##&nbsp;tcp        0      0 localhost.localdomain:54105 localhost.localdomai:sunrpc TIME_WAIT
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc:login qalnb02.eng.btc.netapp.:870 ESTABLISHED
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc.n:ssh 10.74.231.177:emc-gateway   ESTABLISHED
##&nbsp;tcp        0      0 localhost.localdomain:929   localhost.local:fujitsu-dev TIME_WAIT
##&nbsp;Active UNIX domain sockets (w/o servers)
##&nbsp;Proto RefCnt Flags       Type       State         I-Node Path
##&nbsp;unix  26     [ ]         DGRAM                    12126  /dev/log
##&nbsp;unix  2      [ ]         DGRAM                    8819   @/org/kernel/udev/udevd
##&nbsp;unix  3      [ ]         DGRAM                    12540  /var/run/lldpad/clif
##&nbsp;unix  2      [ ]         DGRAM                    13354  @/org/freedesktop/hal/udev_event
##&nbsp;unix  2      [ ]         DGRAM                    12597  /tmp/fcoemon.dcbd.1618
##&nbsp;unix  2      [ ]         DGRAM                    12600  /var/run/fcm/fcm_clif
##&nbsp;unix  2      [ ]         DGRAM                    156693
##&nbsp;unix  2      [ ]         DGRAM                    154426
##&nbsp;unix  3      [ ]         STREAM     CONNECTED     44126
##&nbsp;unix  3      [ ]         STREAM     CONNECTED     44125
##&nbsp;unix  2      [ ]         DGRAM                    44122
##&nbsp;unix  3      [ ]         STREAM     CONNECTED     19281  @/tmp/dbus-EBOy5mlfG5
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_internet_connections' => [
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.localdomai:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:54105',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'qalnb02.eng.btc.netapp.:870',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc:login',
##&nbsp;                                                 'state' => 'ESTABLISHED'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => '10.74.231.177:emc-gateway',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc.n:ssh',
##&nbsp;                                                 'state' => 'ESTABLISHED'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.local:fujitsu-dev',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:929',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;            'active_unix_domain_sockets' => [
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12126',
##&nbsp;                                                'path' => '/dev/log',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '26',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '8819',
##&nbsp;                                                'path' => '@/org/kernel/udev/udevd',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12540',
##&nbsp;                                                'path' => '/var/run/lldpad/clif',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '13354',
##&nbsp;                                                'path' => '@/org/freedesktop/hal/udev_event',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12597',
##&nbsp;                                                'path' => '/tmp/fcoemon.dcbd.1618',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12600',
##&nbsp;                                                'path' => '/var/run/fcm/fcm_clif',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '156693',
##&nbsp;                                                'path' => '',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '154426',
##&nbsp;                                                'path' => '',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '44126',
##&nbsp;                                                'path' => '',
##&nbsp;                                                'type' => 'STREAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => 'CONNECTED'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '44125',
##&nbsp;                                                'path' => '',
##&nbsp;                                                'type' => 'STREAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => 'CONNECTED'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '44122',
##&nbsp;                                                'path' => '',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '19281',
##&nbsp;                                                'path' => '@/tmp/dbus-EBOy5mlfG5',
##&nbsp;                                                'type' => 'STREAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => 'CONNECTED'
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -e
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active Internet connections (w/o servers)
##&nbsp;Proto Recv-Q Send-Q Local Address               Foreign Address             State       User       Inode
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc.n:ssh 10.74.231.177:emc-gateway   ESTABLISHED root       44006
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc.n:rmc ns0.lab.eng.btc.netap:43890 TIME_WAIT   root       0
##&nbsp;tcp        0      0 localhost.localdomain:43034 localhost.localdomai:sunrpc TIME_WAIT   root       0
##&nbsp;tcp        0      0 localhost.localdomain:43838 localhost.localdomai:sunrpc TIME_WAIT   root       0
##&nbsp;Active UNIX domain sockets (w/o servers)
##&nbsp;Proto RefCnt Flags       Type       State         I-Node Path
##&nbsp;unix  26     [ ]         DGRAM                    12126  /dev/log
##&nbsp;unix  2      [ ]         DGRAM                    8819   @/org/kernel/udev/udevd
##&nbsp;unix  3      [ ]         DGRAM                    12540  /var/run/lldpad/clif
##&nbsp;unix  2      [ ]         DGRAM                    13354  @/org/freedesktop/hal/udev_event
##&nbsp;unix  2      [ ]         DGRAM                    12597  /tmp/fcoemon.dcbd.1618
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_internet_connections' => [
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => '10.74.231.177:emc-gateway',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc.n:ssh',
##&nbsp;                                                 'user' => 'root',
##&nbsp;                                                 'inode' => '44006',
##&nbsp;                                                 'state' => 'ESTABLISHED'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'ns0.lab.eng.btc.netap:43890',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc.n:rmc',
##&nbsp;                                                 'user' => 'root',
##&nbsp;                                                 'inode' => '0',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.localdomai:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:43034',
##&nbsp;                                                 'user' => 'root',
##&nbsp;                                                 'inode' => '0',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.localdomai:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:43838',
##&nbsp;                                                 'user' => 'root',
##&nbsp;                                                 'inode' => '0',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;            'active_unix_domain_sockets' => [
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12126',
##&nbsp;                                                'path' => '/dev/log',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '26',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '8819',
##&nbsp;                                                'path' => '@/org/kernel/udev/udevd',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12540',
##&nbsp;                                                'path' => '/var/run/lldpad/clif',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '13354',
##&nbsp;                                                'path' => '@/org/freedesktop/hal/udev_event',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12597',
##&nbsp;                                                'path' => '/tmp/fcoemon.dcbd.1618',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -o
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active Internet connections (w/o servers)
##&nbsp;Proto Recv-Q Send-Q Local Address               Foreign Address             State       Timer
##&nbsp;tcp        0      0 localhost.localdomain:44374 localhost.localdomai:sunrpc TIME_WAIT   timewait (43.17/0/0)
##&nbsp;tcp        0      0 lnx219-45.lab.eng.bt:nlogin ns0.lab.eng.btc.netap:43890 TIME_WAIT   timewait (57.41/0/0)
##&nbsp;tcp        0      0 localhost.localdomain:756   localhost.local:fujitsu-dev TIME_WAIT   timewait (57.38/0/0)
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc:49478 ns0.lab.eng.btc.neta:sunrpc TIME_WAIT   timewait (57.33/0/0)
##&nbsp;Active UNIX domain sockets (w/o servers)
##&nbsp;Proto RefCnt Flags       Type       State         I-Node Path
##&nbsp;unix  25     [ ]         DGRAM                    12126  /dev/log
##&nbsp;unix  2      [ ]         DGRAM                    8819   @/org/kernel/udev/udevd
##&nbsp;unix  3      [ ]         DGRAM                    12540  /var/run/lldpad/clif
##&nbsp;unix  2      [ ]         DGRAM                    13354  @/org/freedesktop/hal/udev_event
##&nbsp;unix  2      [ ]         DGRAM                    12597  /tmp/fcoemon.dcbd.1618
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_internet_connections' => [
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.localdomai:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:44374',
##&nbsp;                                                 'timer' => 'timewait (43.17/0/0)',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'ns0.lab.eng.btc.netap:43890',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.bt:nlogin',
##&nbsp;                                                 'timer' => 'timewait (57.41/0/0)',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.local:fujitsu-dev',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:756',
##&nbsp;                                                 'timer' => 'timewait (57.38/0/0)',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'ns0.lab.eng.btc.neta:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc:49478',
##&nbsp;                                                 'timer' => 'timewait (57.33/0/0)',
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;            'active_unix_domain_sockets' => [
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12126',
##&nbsp;                                                'path' => '/dev/log',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '25',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '8819',
##&nbsp;                                                'path' => '@/org/kernel/udev/udevd',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12540',
##&nbsp;                                                'path' => '/var/run/lldpad/clif',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '13354',
##&nbsp;                                                'path' => '@/org/freedesktop/hal/udev_event',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12597',
##&nbsp;                                                'path' => '/tmp/fcoemon.dcbd.1618',
##&nbsp;                                                'type' => 'DGRAM',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'state' => '-'
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active Internet connections (w/o servers)
##&nbsp;Proto Recv-Q Send-Q Local Address               Foreign Address             State       PID/Program name
##&nbsp;tcp        0      0 lnx219-45.lab.eng.btc:60342 ns0.lab.eng.btc.neta:sunrpc TIME_WAIT   -
##&nbsp;tcp        0      0 lnx219-45.l:netconfsoapbeep ns0.lab.eng.btc.netap:43890 TIME_WAIT   -
##&nbsp;tcp        0      0 localhost.localdomain:37650 localhost.localdomai:sunrpc TIME_WAIT   -
##&nbsp;tcp        0    550 lnx219-45.lab.eng.btc:login qalnb02.eng.btc.netapp.:839 ESTABLISHED 21007/in.rlogind
##&nbsp;Active UNIX domain sockets (w/o servers)
##&nbsp;Proto RefCnt Flags       Type       State         I-Node PID/Program name    Path
##&nbsp;unix  25     [ ]         DGRAM                    12126  1508/rsyslogd       /dev/log
##&nbsp;unix  2      [ ]         DGRAM                    8819   530/udevd           @/org/kernel/udev/udevd
##&nbsp;unix  3      [ ]         DGRAM                    12540  1595/lldpad         /var/run/lldpad/clif
##&nbsp;unix  3      [ ]         STREAM     CONNECTED     12672  1627/dbus-daemon    /var/run/dbus/system_bus_socket
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_internet_connections' => [
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'ns0.lab.eng.btc.neta:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'pid' => '-',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc:60342',
##&nbsp;                                                 'program_name' => undef,
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'ns0.lab.eng.btc.netap:43890',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'pid' => '-',
##&nbsp;                                                 'local_address' => 'lnx219-45.l:netconfsoapbeep',
##&nbsp;                                                 'program_name' => undef,
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '0',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'localhost.localdomai:sunrpc',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'pid' => '-',
##&nbsp;                                                 'local_address' => 'localhost.localdomain:37650',
##&nbsp;                                                 'program_name' => undef,
##&nbsp;                                                 'state' => 'TIME_WAIT'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'send_q' => '550',
##&nbsp;                                                 'recv_q' => '0',
##&nbsp;                                                 'foreign_address' => 'qalnb02.eng.btc.netapp.:839',
##&nbsp;                                                 'proto' => 'tcp',
##&nbsp;                                                 'pid' => '21007',
##&nbsp;                                                 'local_address' => 'lnx219-45.lab.eng.btc:login',
##&nbsp;                                                 'program_name' => 'in.rlogind',
##&nbsp;                                                 'state' => 'ESTABLISHED'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;            'active_unix_domain_sockets' => [
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12126',
##&nbsp;                                                'path' => '/dev/log',
##&nbsp;                                                'program_name' => 'rsyslogd',
##&nbsp;                                                'state' => '-',
##&nbsp;                                                'pid' => '1508',
##&nbsp;                                                'refcnt' => '25',
##&nbsp;                                                'type' => 'DGRAM'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '8819',
##&nbsp;                                                'path' => '@/org/kernel/udev/udevd',
##&nbsp;                                                'program_name' => 'udevd',
##&nbsp;                                                'state' => '-',
##&nbsp;                                                'pid' => '530',
##&nbsp;                                                'refcnt' => '2',
##&nbsp;                                                'type' => 'DGRAM'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12540',
##&nbsp;                                                'path' => '/var/run/lldpad/clif',
##&nbsp;                                                'program_name' => 'lldpad',
##&nbsp;                                                'state' => '-',
##&nbsp;                                                'pid' => '1595',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'type' => 'DGRAM'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'proto' => 'unix',
##&nbsp;                                                'flags' => '',
##&nbsp;                                                'i_node' => '12672',
##&nbsp;                                                'path' => '/var/run/dbus/system_bus_socket',
##&nbsp;                                                'program_name' => 'dbus-daemon',
##&nbsp;                                                'state' => 'CONNECTED',
##&nbsp;                                                'pid' => '1627',
##&nbsp;                                                'refcnt' => '3',
##&nbsp;                                                'type' => 'STREAM'
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel IP routing table
##&nbsp;Destination     Gateway         Genmask         Flags   MSS Window  irtt Iface
##&nbsp;10.72.219.0     *               255.255.255.0   U         0 0          0 eth0
##&nbsp;192.168.122.0   *               255.255.255.0   U         0 0          0 virbr0
##&nbsp;link-local      *               255.255.0.0     U         0 0          0 eth0
##&nbsp;link-local      *               255.255.0.0     U         0 0          0 eth1
##&nbsp;default         sw-219-cr1.lab. 0.0.0.0         UG        0 0          0 eth0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_ip_routing_table' => [
##&nbsp;                                           {
##&nbsp;                                             'window' => '0',
##&nbsp;                                             'flags' => 'U',
##&nbsp;                                             'gateway' => '*',
##&nbsp;                                             'destination' => '10.72.219.0',
##&nbsp;                                             'irtt' => '0',
##&nbsp;                                             'mss' => '0',
##&nbsp;                                             'iface' => 'eth0',
##&nbsp;                                             'genmask' => '255.255.255.0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'window' => '0',
##&nbsp;                                             'flags' => 'U',
##&nbsp;                                             'gateway' => '*',
##&nbsp;                                             'destination' => '192.168.122.0',
##&nbsp;                                             'irtt' => '0',
##&nbsp;                                             'mss' => '0',
##&nbsp;                                             'iface' => 'virbr0',
##&nbsp;                                             'genmask' => '255.255.255.0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'window' => '0',
##&nbsp;                                             'flags' => 'U',
##&nbsp;                                             'gateway' => '*',
##&nbsp;                                             'destination' => 'link-local',
##&nbsp;                                             'irtt' => '0',
##&nbsp;                                             'mss' => '0',
##&nbsp;                                             'iface' => 'eth0',
##&nbsp;                                             'genmask' => '255.255.0.0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'window' => '0',
##&nbsp;                                             'flags' => 'U',
##&nbsp;                                             'gateway' => '*',
##&nbsp;                                             'destination' => 'link-local',
##&nbsp;                                             'irtt' => '0',
##&nbsp;                                             'mss' => '0',
##&nbsp;                                             'iface' => 'eth1',
##&nbsp;                                             'genmask' => '255.255.0.0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'window' => '0',
##&nbsp;                                             'flags' => 'UG',
##&nbsp;                                             'gateway' => 'sw-219-cr1.lab.',
##&nbsp;                                             'destination' => 'default',
##&nbsp;                                             'irtt' => '0',
##&nbsp;                                             'mss' => '0',
##&nbsp;                                             'iface' => 'eth0',
##&nbsp;                                             'genmask' => '0.0.0.0'
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel Interface table
##&nbsp;Iface       MTU Met    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg
##&nbsp;eth0       1500   0   852414      0 287067      0   135908      0      0      0 BMRU
##&nbsp;eth1       1500   0 534633054      0      0      0        9      0      0      0 BMRU
##&nbsp;lo        16436   0    18774      0      0      0    18774      0      0      0 LRU
##&nbsp;virbr0     1500   0        0      0      0      0     1421      0      0      0 BMRU
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_interface_table' => [
##&nbsp;                                          {
##&nbsp;                                            'met' => '0',
##&nbsp;                                            'flg' => 'BMRU',
##&nbsp;                                            'tx_ovr' => '0',
##&nbsp;                                            'rx_drp' => '287067',
##&nbsp;                                            'tx_ok' => '135908',
##&nbsp;                                            'tx_drp' => '0',
##&nbsp;                                            'iface' => 'eth0',
##&nbsp;                                            'tx_err' => '0',
##&nbsp;                                            'rx_err' => '0',
##&nbsp;                                            'rx_ovr' => '0',
##&nbsp;                                            'mtu' => '1500',
##&nbsp;                                            'rx_ok' => '852414'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'met' => '0',
##&nbsp;                                            'flg' => 'BMRU',
##&nbsp;                                            'tx_ovr' => '0',
##&nbsp;                                            'rx_drp' => '0',
##&nbsp;                                            'tx_ok' => '9',
##&nbsp;                                            'tx_drp' => '0',
##&nbsp;                                            'iface' => 'eth1',
##&nbsp;                                            'tx_err' => '0',
##&nbsp;                                            'rx_err' => '0',
##&nbsp;                                            'rx_ovr' => '0',
##&nbsp;                                            'mtu' => '1500',
##&nbsp;                                            'rx_ok' => '534633054'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'met' => '0',
##&nbsp;                                            'flg' => 'LRU',
##&nbsp;                                            'tx_ovr' => '0',
##&nbsp;                                            'rx_drp' => '0',
##&nbsp;                                            'tx_ok' => '18774',
##&nbsp;                                            'tx_drp' => '0',
##&nbsp;                                            'iface' => 'lo',
##&nbsp;                                            'tx_err' => '0',
##&nbsp;                                            'rx_err' => '0',
##&nbsp;                                            'rx_ovr' => '0',
##&nbsp;                                            'mtu' => '16436',
##&nbsp;                                            'rx_ok' => '18774'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'met' => '0',
##&nbsp;                                            'flg' => 'BMRU',
##&nbsp;                                            'tx_ovr' => '0',
##&nbsp;                                            'rx_drp' => '0',
##&nbsp;                                            'tx_ok' => '1421',
##&nbsp;                                            'tx_drp' => '0',
##&nbsp;                                            'iface' => 'virbr0',
##&nbsp;                                            'tx_err' => '0',
##&nbsp;                                            'rx_err' => '0',
##&nbsp;                                            'rx_ovr' => '0',
##&nbsp;                                            'mtu' => '1500',
##&nbsp;                                            'rx_ok' => '0'
##&nbsp;                                          }
##&nbsp;                                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -g
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPv6/IPv4 Group Memberships
##&nbsp;Interface       RefCnt Group
##&nbsp;--------------- ------ ---------------------
##&nbsp;lo              1      all-systems.mcast.net
##&nbsp;eth0            1      224.0.0.251
##&nbsp;eth0            1      all-systems.mcast.net
##&nbsp;eth1            1      all-systems.mcast.net
##&nbsp;virbr0          1      224.0.0.251
##&nbsp;virbr0          1      all-systems.mcast.net
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipv6_ipv4_group_memberships' => [
##&nbsp;                                               {
##&nbsp;                                                 'group' => 'all-systems.mcast.net',
##&nbsp;                                                 'interface' => 'lo',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'group' => '224.0.0.251',
##&nbsp;                                                 'interface' => 'eth0',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'group' => 'all-systems.mcast.net',
##&nbsp;                                                 'interface' => 'eth0',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'group' => 'all-systems.mcast.net',
##&nbsp;                                                 'interface' => 'eth1',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'group' => '224.0.0.251',
##&nbsp;                                                 'interface' => 'virbr0',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'group' => 'all-systems.mcast.net',
##&nbsp;                                                 'interface' => 'virbr0',
##&nbsp;                                                 'refcnt' => '1'
##&nbsp;                                               }
##&nbsp;                                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub netstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    my $temp;

    if ( $cmd =~ /-t/ ) {
        $lines[1] =~ s/Local Address/local_address/;
        $lines[1] =~ s/Foreign Address/foreign_address/;
        @head = split( /\s+/, $lines[1] );
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[0]->{active_internet_connections}->[$count]
                    ->{ to_key_name( item => $head[$j] ) } = $values[$j];
            }
            $count++;
        }
    } elsif ( $cmd =~ /-r|-i/ ) {
        my $temp_head = $lines[0];
        @head = split( /\s+/, $lines[1] );
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            @values = split( /\s+/, $lines[$i] );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $head[$j] =~ s/-/_/;
                $result_array[0]->{ to_key_name( item => $temp_head ) }
                    ->[$count]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            $count++;
        }
    } elsif ( $cmd =~ /-g/ ) {
        my $temp_head = $lines[0];
        $temp_head =~ s/\//_/;
        @head = split( /\s+/, $lines[1] );
        for ( my $i = 3; $i <= $#lines; $i++ ) {
            @values = split( /\s+/, $lines[$i] );
            if ( $lines[$i] =~ /(\S+)\s+(\S+)\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $temp_head ) }
                    ->[$count]->{ to_key_name( item => $head[0] ) } = $1;
                $result_array[0]->{ to_key_name( item => $temp_head ) }
                    ->[$count]->{ to_key_name( item => $head[1] ) } = $2;
                $result_array[0]->{ to_key_name( item => $temp_head ) }
                    ->[$count]->{ to_key_name( item => $head[2] ) } = $3;
                $count++;
            }
        }
    } else {
        for ( my $i = 1; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/Local Address/local_address/;
            $lines[$i] =~ s/Foreign Address/foreign_address/;
            $lines[$i] =~ s/PID\/Program name/pid program_name/;
            $lines[$i] =~ s/I-Node/i_node/;
            if ( $lines[$i] =~ /Proto\s+Recv-Q.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /Proto\s+RefCnt\s+Flags.*/ ) {
                $count = 0;
                @head = split( /\s+/, $lines[$i] );
            }
            if ( $cmd =~ /-p/ ) {
                if ( $lines[$i]
                    =~ /(\w+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\w+)\s*(\S*\/*\s*\S*)/
                    ) {
                    $temp = $7;
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#head - 2; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{active_internet_connections}
                            ->[$count]->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    my @val = split( /\//, $temp );
                    $result_array[0]->{active_internet_connections}->[$count]
                        ->{ to_key_name( item => $head[ $#head - 1 ] ) }
                        = $val[0];
                    $result_array[0]->{active_internet_connections}->[$count]
                        ->{ to_key_name( item => $head[$#head] ) } = $val[1];
                    $count++;
                } elsif ( $lines[$i]
                    =~ /(\w+)\s+(\d+)\s+\[\s*(.*)\s+\]\s+(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\/*(\S+)\s*(.*)/
                    ) {
                    splice(
                        @values, 0, $#head,
                        (   "$1", "$2", "$3", "$4", "$5", "$6",
                            "$7", "$8", "$9"
                        )
                    );
                    for ( my $j = 0; $j <= $#head; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{active_unix_domain_sockets}
                            ->[$count]->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $count++;
                } elsif ( $lines[$i]
                    =~ /(\w+)\s+(\d+)\s+\[\s*(.*)\s+\]\s+(\w+)\s+(\d+)\s+(\d+)\/(\S+)\s*(\S*)/
                    ) {
                    splice(
                        @values, 0, $#head,
                        (   "$1", "$2", "$3", "$4", "-", "$5",
                            "$6", "$7", "$8"
                        )
                    );

                    for ( my $j = 0; $j <= $#head; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{active_unix_domain_sockets}
                            ->[$count]->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $count++;
                }
            } elsif ( $lines[$i]
                =~ /(\w+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S+)\s*(\S*)\s*(.*)/ ) {
                $temp = $7;
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $head[$j] =~ s/-/_/;
                    $result_array[0]->{active_internet_connections}->[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $result_array[0]->{active_internet_connections}->[$count]
                    ->{ to_key_name( item => $head[$#head] ) } = $temp
                    if ( $cmd =~ /-o/ );
                $count++;
            } elsif ( $lines[$i]
                =~ /(\w+)\s+(\d+)\s+\[\s*(.*)\s+\]\s+(\w+)\s+(\w+)\s+(\d+)\s*(.*)/
                ) {
                splice( @values, 0, $#head,
                    ( "$1", "$2", "$3", "$4", "$5", "$6", "$7" ) );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $head[$j] =~ s/-/_/;
                    $result_array[0]->{active_unix_domain_sockets}->[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            } elsif ( $lines[$i]
                =~ /(\w+)\s+(\d+)\s+\[\s*(.*)\s+\]\s+(\w+)\s+(\d+)\s*(.*)/ ) {
                splice( @values, 0, $#head,
                    ( "$1", "$2", "$3", "$4", "-", "$5", "$6" ) );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $head[$j] =~ s/-/_/;
                    $result_array[0]->{active_unix_domain_sockets}->[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            }
        }
    }
    return \@result_array;
} ## end sub netstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /netstat:\s+no\s+support.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|netstat).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

