#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ping command output
##
## @description
##    This is a library to parse the raw output of Linux ping command.
##
## @synopsis
##    use ping;
##
##    ping(%args);
##    ping6(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ping;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    
    if ($raw_output =~ /:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(unknown)/) {
            $error = "InvalidParamValue";
        } 
    } elsif ($raw_output =~ /Flow labels are not supported./) {
        $$error_message = $raw_output;
    } 
    return $error;
}

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'time-to-live' </td> <td> '0' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-U' </td> <td> 'user-to-user-latency' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'sndbuf' </td> <td> '0' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'allow-broadcast' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'audible' </td> <td> '1' </td> </tr> 
## <tr> <td> '-Q' </td> <td> 'tos' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'packet-size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-T' </td> <td> 'timestamp' </td> <td> '0' </td> </tr> 
## <tr> <td> '-W' </td> <td> 'timeout' </td> <td> '0' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'hop' </td> <td> '0' </td> </tr> 
## <tr> <td> '-B' </td> <td> 'bind-source' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'deadline' </td> <td> '0' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'adaptive' </td> <td> '1' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'bypass-routing-tables' </td> <td> '1' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'interface-address' </td> <td> '0' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'record-route' </td> <td> '1' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interval' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'so-debug' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'flood-ping' </td> <td> '1' </td> </tr> 
## <tr> <td> '-M' </td> <td> 'hint' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'preload' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'numeric' </td> <td> '1' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'supress-loopback-multicast' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ping -c 3 -T tsonly localhost
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING localhost.localdomain (127.0.0.1) 56(124) bytes of data.
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=1 ttl=64 time=0.052 ms
##&nbsp;TS: 	26907624 absolute
##&nbsp;	0
##&nbsp;	0
##&nbsp;	0
##&nbsp;
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=2 ttl=64 time=0.034 ms
##&nbsp;TS: 	26908624 absolute
##&nbsp;	0
##&nbsp;	0
##&nbsp;	0
##&nbsp;
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=3 ttl=64 time=0.056 ms
##&nbsp;TS: 	26909625 absolute
##&nbsp;	0
##&nbsp;	1
##&nbsp;	0
##&nbsp;
##&nbsp;
##&nbsp;--- localhost.localdomain ping statistics ---
##&nbsp;3 packets transmitted, 3 received, 0% packet loss, time 2001ms
##&nbsp;rtt min/avg/max/mdev = 0.034/0.047/0.056/0.011 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(124)',
##&nbsp;            'avg' => '0.047',
##&nbsp;            'min' => '0.034',
##&nbsp;            'time' => '2001ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.011',
##&nbsp;            'max' => '0.056',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.052',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.034',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.056',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '3',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost.localdomain',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -c 3 -R localhost
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING localhost.localdomain (127.0.0.1) 56(124) bytes of data.
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=1 ttl=64 time=0.054 ms
##&nbsp;RR: 	localhost.localdomain (127.0.0.1)
##&nbsp;	localhost.localdomain (127.0.0.1)
##&nbsp;	localhost.localdomain (127.0.0.1)
##&nbsp;	localhost.localdomain (127.0.0.1)
##&nbsp;
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=2 ttl=64 time=0.024 ms	(same route)
##&nbsp;64 bytes from localhost.localdomain (127.0.0.1): icmp_seq=3 ttl=64 time=0.036 ms	(same route)
##&nbsp;
##&nbsp;--- localhost.localdomain ping statistics ---
##&nbsp;3 packets transmitted, 3 received, 0% packet loss, time 2006ms
##&nbsp;rtt min/avg/max/mdev = 0.024/0.038/0.054/0.012 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(124)',
##&nbsp;            'avg' => '0.038',
##&nbsp;            'min' => '0.024',
##&nbsp;            'time' => '2006ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.012',
##&nbsp;            'max' => '0.054',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.054',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.024',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.036',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '3',
##&nbsp;                           'ip' => 'localhost.localdomain (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost.localdomain',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -V
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ping utility, iputils-ss020927
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'iputils-ss020927
##&nbsp;'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -L -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.017',
##&nbsp;            'min' => '0.010',
##&nbsp;            'time' => '2000ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.005',
##&nbsp;            'max' => '0.022',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.019',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.010',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.022',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -R -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(124)',
##&nbsp;            'avg' => '0.019',
##&nbsp;            'min' => '0.008',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.008',
##&nbsp;            'max' => '0.027',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.022',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.027',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.008',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -U -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.016',
##&nbsp;            'min' => '0.012',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.004',
##&nbsp;            'max' => '0.022',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.022',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.012',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.014',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -b -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.014',
##&nbsp;            'min' => '0.013',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.000',
##&nbsp;            'max' => '0.015',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.015',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.014',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.013',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -d -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.012',
##&nbsp;            'min' => '0.009',
##&nbsp;            'time' => '2000ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.003',
##&nbsp;            'max' => '0.016',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.016',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.009',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.011',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -n -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.011',
##&nbsp;            'min' => '0.008',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.005',
##&nbsp;            'max' => '0.016',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.016',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => '127.0.0.1',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.011',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => '127.0.0.1',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.008',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => '127.0.0.1',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -q -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.018',
##&nbsp;            'min' => '0.008',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.007',
##&nbsp;            'max' => '0.025',
##&nbsp;            'stats' => [],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -v -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.010',
##&nbsp;            'min' => '0.007',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.004',
##&nbsp;            'max' => '0.015',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.015',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.009',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.007',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -V
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'iputils-ss020927
##&nbsp;'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -a -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.012',
##&nbsp;            'min' => '0.008',
##&nbsp;            'time' => '2000ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.004',
##&nbsp;            'max' => '0.017',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.017',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.012',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.008',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -A -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.010',
##&nbsp;            'min' => '0.007',
##&nbsp;            'time' => '402ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.004 ms, pipe 2, ipg',
##&nbsp;            'max' => '0.016',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.016',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.007',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.007',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -r -c 3 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56(84)',
##&nbsp;            'avg' => '0.009',
##&nbsp;            'min' => '0.007',
##&nbsp;            'time' => '1999ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'mdev' => '0.004',
##&nbsp;            'max' => '0.014',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.014',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.008',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.007',
##&nbsp;                           'ttl' => '64',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'localhost (127.0.0.1)',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'localhost',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @result_array; 
    my %hash;
    if($output =~ /ping utility, /) {
       $hash{'version'} = $';
       @result_array = (\%hash);
       return \@result_array;
    } 
    my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>"PING .+ bytes of data.", 
                          response_object=>$resp);
    
    my $line = shift(@lines);
    if($line =~ /PING ([^\s]+)\s+\(([^\s]+)\)\s+([^\s]+)\s+bytes\s+of\s+data/) {
        $hash{'host'} = $1;
        $hash{'ip'} = $2;
        $hash{'bytes'} = $3;
    }

    $hash{'stats'} = [];

    while (@lines) {
        my %internal_hash;
        $line = $lines[0];
        shift(@lines);
        if ($line =~ /([^\s]+)\s+bytes\s+from\s+([^:]+):\s+(.+)\sms/) {
            $internal_hash{'packet_size'} = $1;
            $internal_hash{'ip'} = $2;
            my $rest = $3;
            my @fields = split(/\s+/,$rest);
            foreach my $field(@fields) {
                my ($name,$value) = split(/=/,$field);
                $internal_hash{$name} = $value;
            }
            push (@{$hash{'stats'}}, \%internal_hash);
        }
        last if($line =~ /--- .+ ping statistics ---/);
    }

    $line = $lines[0];
    if ($line =~ /(\d+) packets transmitted, (\d+) received, (\d+%) packet loss, time (\d+ms)/) {
        $hash{packets_transmitted} = $1;
        $hash{received} = $2;
        $hash{loss} = $3;
        $hash{time} = $4;
        shift(@lines);
        $line = $lines[0];
    }
    
    if($line =~ /rtt (.+)\s+=\s+(.+)\sms/) {
        my @names = split(/\//, $1);
        my @values = split(/\//, $2);
        my $i;
        foreach my $name (@names) {
            $hash{$name}=$values[$i];
            $i++;
        }
    }

    @result_array = (\%hash);
    return \@result_array;
}

## @name ::ping6
##
## @summary Parses the raw output from the "ping6" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping6 with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping6(%args);
## INPUTS TO METHOD CALL 'ping6' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'time-to-live' </td> </tr> 
## <tr> <td> '-U' </td> <td> 'user-to-user-latency' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'sndbuf' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'audible' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'packet-size' </td> </tr> 
## <tr> <td> '-Q' </td> <td> 'tos' </td> </tr> 
## <tr> <td> '-W' </td> <td> 'timeout' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'hop' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'version' </td> </tr> 
## <tr> <td> '-F' </td> <td> 'flow-label' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'deadline' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'adaptive' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'bypass-routing-tables' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interval' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'interface-address' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'so-debug' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'flood-ping' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pattern' </td> </tr> 
## <tr> <td> '-M' </td> <td> 'hint' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'preload' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'numeric' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'supress-loopback-multicast' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ping6 -c 3 fd20:8b1e:b255:8305:250:56ff:feae:4ea8
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING fd20:8b1e:b255:8305:250:56ff:feae:4ea8(fd20:8b1e:b255:8305:250:56ff:feae:4ea8) 56 data bytes
##&nbsp;64 bytes from fd20:8b1e:b255:8305:250:56ff:feae:4ea8: icmp_seq=0 ttl=57 time=233 ms
##&nbsp;64 bytes from fd20:8b1e:b255:8305:250:56ff:feae:4ea8: icmp_seq=1 ttl=57 time=232 ms
##&nbsp;64 bytes from fd20:8b1e:b255:8305:250:56ff:feae:4ea8: icmp_seq=2 ttl=57 time=233 ms
##&nbsp;
##&nbsp;--- fd20:8b1e:b255:8305:250:56ff:feae:4ea8 ping statistics ---
##&nbsp;3 packets transmitted, 3 received, 0% packet loss, time 2000ms
##&nbsp;rtt min/avg/max/mdev = 232.860/233.223/233.465/0.261 ms, pipe 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56',
##&nbsp;            'avg' => '233.223',
##&nbsp;            'min' => '232.860',
##&nbsp;            'time' => '2000ms',
##&nbsp;            'ip' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;            'mdev' => '0.261',
##&nbsp;            'max' => '233.465',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '233',
##&nbsp;                           'ttl' => '57',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '232',
##&nbsp;                           'ttl' => '57',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '233',
##&nbsp;                           'ttl' => '57',
##&nbsp;                           'icmp_seq' => '2',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;                           'packet_size' => '64'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;            'packets_transmitted' => '3',
##&nbsp;            'loss' => '0%',
##&nbsp;            'received' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping6 -V
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ping6 utility, iputils-ss020927
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'iputils-ss020927
##&nbsp;'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping6 {
    my %args = @_;

    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @result_array;
    my %hash;
    if($output =~ /ping6 utility, /) {
       $hash{'version'} = $';
       @result_array = (\%hash);
       return \@result_array;
    }
    my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>"PING .+ data bytes",
                          response_object=>$resp);
    my $line = shift(@lines);
    if($line =~ /PING ([^\s]+)\(([^\s]+)\)\s+([^\s]+)\s+data\s+bytes/) {
        $hash{'host'} = $1;
        $hash{'ip'} = $2;
        $hash{'bytes'} = $3;
    }

    $hash{'stats'} = [];

    while (@lines) {
        my %internal_hash;
        $line = $lines[0];
        shift(@lines);
        if ($line =~ /([^\s]+)\s+bytes\s+from\s+(.+):\s+(.+)\sms/) {
            $internal_hash{'packet_size'} = $1;
            $internal_hash{'ip'} = $2;
            my $rest = $3;
            my @fields = split(/\s+/,$rest);
            foreach my $field(@fields) {
                my ($name,$value) = split(/=/,$field);
                $internal_hash{$name} = $value;
            }
            push (@{$hash{'stats'}}, \%internal_hash);
        }
        last if($line =~ /--- .+ ping statistics ---/);
    }

    $line = $lines[0];
    if ($line =~ /(\d+) packets transmitted, (\d+) received, (\d+%) packet loss, time (\d+ms)/) {
        $hash{packets_transmitted} = $1;
        $hash{received} = $2;
        $hash{loss} = $3;
        $hash{time} = $4;
        shift(@lines);
        $line = $lines[0];
    }
    if($line =~ /rtt (.+)\s+=\s+(.+)\sms/) {
        my @names = split(/\//, $1);
        my @values = split(/\//, $2);
        my $i;
        foreach my $name (@names) {
            $hash{$name}=$values[$i];
            $i++;
        }
    }

    @result_array = (\%hash);
    return \@result_array;
}
1;
