#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux populate command output
##
## @description
##    This is a library to parse the raw output of Linux populate command.
##
## @synopsis
##    use populate;
##
##    populate(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::populate;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    return $error;
}



## @name ::populate
##
## @summary Parses the raw output from the "populate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from populate with any of its options as described in the
##    corresponding cdef file 'populate.cdefs'
## @synopsis
##      populate(%args);
## INPUTS TO METHOD CALL 'populate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'FORK_COUNT' </td> <td> 'fork_count' </td> </tr> 
## <tr> <td> 'BREADTH_AVG' </td> <td> 'breadth_avg' </td> </tr>
## <tr> <td> 'DEPTH_DEV' </td> <td> 'depth_dev' </td> </tr>
## <tr> <td> 'MIN_DIR_SIZE' </td> <td> 'min_dir_size' </td> </tr>
## <tr> <td> 'DEPTH_AVG' </td> <td> 'depth_avg' </td> </tr>
## <tr> <td> 'DEPTH_BASE' </td> <td> 'depth_base' </td> </tr>
## <tr> <td> 'FLOOD' </td> <td> 'flood' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'directory' </td> </tr>
## <tr> <td> '-e' </td> <td> 'std-dev-effect-estimate' </td> </tr>
## <tr> <td> 'FILE_DEV' </td> <td> 'file_dev' </td> </tr>
## <tr> <td> 'SEED' </td> <td> 'seed' </td> </tr>
## <tr> <td> 'OFFSET' </td> <td> 'offset' </td> </tr>
## <tr> <td> 'SCRAMBLE' </td> <td> 'scramble' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr>
## <tr> <td> 'ENVIRONMENT_TYPE' </td> <td> 'environment_type' </td> </tr>
## <tr> <td> 'DIR_AVG' </td> <td> 'dir_avg' </td> </tr>
## <tr> <td> 'LOG_FILE' </td> <td> 'log_file' </td> </tr>
## <tr> <td> 'FILE_AVG' </td> <td> 'file_avg' </td> </tr>
## <tr> <td> 'DIR_DEV' </td> <td> 'dir_dev' </td> </tr>
## <tr> <td> 'PERSEVERE' </td> <td> 'persevere' </td> </tr>
## <tr> <td> 'BREADTH_DEV' </td> <td> 'breadth_dev' </td> </tr>
## <tr> <td> 'RETURN_PATHS' </td> <td> 'return_paths' </td> </tr>
## <tr> <td> 'PRECREATE' </td> <td> 'precreate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub populate {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $command_line = $resp->get_actual_command();
    my $error;
    
    if($error_level) {
        $error= _check_error(raw_output=>$output,error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @lines = get_lines(raw_output=>$output,
                          first_line=>"(Population complete|ENVIRONMENT_TYPE|Base population completed)", 
                          response_object=>$resp);
                          
    my (%hash, $line, $name, $value);
    
    if($command_line =~ /\s+-e\s+/) {
        while(@lines) {
            $line = shift(@lines);
            if($line=~/(\w+)\s+=\s+(.+)/) {
                $name = to_key_name(item=>$1);
                $hash{$name} = $2;
            } elsif ($line=~/tree depth of (.+) \(on average, (min=.+)\)/) {
                $hash{'other_stats'}=[];
                my %internal_hash;
                $internal_hash{'name'}='tree_depth';
                $internal_hash{'average'}=$1;
                %internal_hash = (%internal_hash,
                                  _populate_hash_line('line'=>$2));
                push(@{$hash{'other_stats'}}, \%internal_hash);
                last;
            }
        }
        foreach $line (@lines) {
            if($line=~/([^\s]+)\s(.+)\s+\(.+(min.+)\)/) {
                my %internal_hash;
                $name = to_key_name(item=>$2);
                $internal_hash{'name'} = $name;
                $internal_hash{'average'} = $1;
                %internal_hash = (%internal_hash,
                                  _populate_hash_line('line'=>$3));
                push(@{$hash{'other_stats'}}, \%internal_hash);
            }
        }
    } else {
        my $flag = 0;
        foreach $line (@lines) {
            if ($flag == 1 && $line =~ /^\w+$/) {
                push @{$hash{'sub_populations'}}, $line;
            } elsif ($flag == 2 && $line =~ /^\d+$/) {
                push @{$hash{'subseeds'}}, $line;
            } elsif($line=~/Population completed in (.+)\s\((.+)\suser,\s(.+)\ssystem\)/) {
                $hash{'population_completed'} = $1;
                $hash{'user'} = $2;
                $hash{'system'} = $3;
            } elsif ($line=~/Population of (.+) succeeded.\s+\(Seed was (.+)\./) {
                $hash{'directory'} = $1;
                $hash{'seed'} = $2;
            } elsif ($line =~ /Base population completed, (starting \d+ sub-populations:|returned \d+ subpaths)/) {
                $hash{'sub_populations'} = [];
                $flag = 1;
            } elsif ($line =~ /Received \d+ subseeds/) {
                $hash{'subseeds'} = [];
                $flag = 2;
            }
        }
    }
    
    my @result_array = (\%hash);
    return \@result_array;
}

sub _populate_hash_line {
    # returns a hash constructed from comma separated name-value pairs 
    # present in a string

    my %args = @_;
    my $line = $args{'line'};
    my %hash;
    my @items = split(/\s*,\s*/,$line);
    my ($name,$value);
    foreach my $item (@items) {
        ($name,$value) = split(/=/, $item);
        $hash{$name} = $value;
    }
    return %hash;
}

1;
