
#Copyright (c) 2012-13 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux pv command output
##
## @description
##    This is a library to parse the raw output of Linux pv command.
##
## @synopsis
##    use pv;
##
##    pvcreate(%args);
##    pvremove(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::pv;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingValueException ;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    my $error_message;
    if ( $raw_output =~
/Couldn't\s+find\s+device|Option.*not\s+be\s+repeated|Failed\s+to\s+read\s+physical\s+volume/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
        /Please\s+give\s+one\s+or\s+more.*/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /invalid\s+option|Invalid\s+argument/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output
        =~ /Run `(pvcreate|pvremove|pvdisplay) --help' for more information|Device.*not found|No such file or directory|Invalid\s+units\s+specification|Failed\s+to\s+read\s+physical\s+volume.*/
        ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /option.*requires\s+an\s+argument|Please\s+give\s+a\s+physical\s+volume\s+path/i ) {
        $error_message = $raw_output;
        $error         = "MissingValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::pvcreate
##
## @summary Parses the raw output from the "pvcreate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvcreate with any of its options as described in the
##    corresponding cdef file 'pvcreate.cdefs'
## @synopsis
##      pvcreate(%args);
## INPUTS TO METHOD CALL 'pvcreate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-y' </td> <td> 'yes' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '--restorefile' </td> <td> 'restorefile' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'physical-volumes' </td> <td> '0' </td> </tr>
## <tr> <td> '--dataalignment' </td> <td> 'dataalignment' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'zero' </td> <td> '0' </td> </tr>
## <tr> <td> '--setphysicalvolumesize' </td> <td> 'setphysicalvolumesize' </td> <td> '0' </td> </tr>
## <tr> <td> '--metadatacopies' </td> <td> 'metadatacopies' </td> <td> '0' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '--labelsector' </td> <td> 'labelsector' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'metadatatype' </td> <td> '0' </td> </tr>
## <tr> <td> '--metadatasize' </td> <td> 'metadatasize' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
sub pvcreate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub pvcreate

## @name ::pvremove
##
## @summary Parses the raw output from the "pvremove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvremove with any of its options as described in the
##    corresponding cdef file 'pvremove.cdefs'
## @synopsis
##      pvremove(%args);
## INPUTS TO METHOD CALL 'pvremove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-y' </td> <td> 'yes' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'physical-volumes' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub pvremove {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub pvremove

## @name ::pvdisplay
##
## @summary Parses the raw output from the "pvdisplay" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvdisplay with any of its options as described in the
##    corresponding cdef file 'pv.cdefs'
## @synopsis
##      pvdisplay(%args);
## INPUTS TO METHOD CALL 'pvdisplay' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--maps' </td> <td> 'maps' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignore-locking-failure' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'physical-volumes' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'colon' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'short' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'columns' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '--nosuffix' </td> <td> 'nosuffix' </td> <td> '1' </td> </tr>
## <tr> <td> '--units' </td> <td> 'units' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pvdisplay --short
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Device "/dev/sdb" has a capacity of 149.03 GiB
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pv_size' => '149.03 GiB',
##&nbsp;            'pv_name' => '/dev/sdb'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay  --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LVM version:     2.02.95(2)-RHEL6 (2012-05-16)
##&nbsp;  Library version: 1.02.74-RHEL6 (2012-05-16)
##&nbsp;  Driver version:  4.22.6
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'library_version' => '1.02.74-RHEL6 (2012-05-16)',
##&nbsp;            'driver_version' => '4.22.6',
##&nbsp;            'lvm_version' => '2.02.95(2)-RHEL6 (2012-05-16)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay  -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Scanning for physical volume names
##&nbsp;  --- Physical volume ---
##&nbsp;  PV Name               /dev/sdb
##&nbsp;  VG Name               vg1
##&nbsp;  PV Size               149.05 GiB / not usable 3.84 MiB
##&nbsp;  Allocatable           yes
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Free PE               38151
##&nbsp;  Allocated PE          5
##&nbsp;  PV UUID               9j2kzI-AJRt-6CCU-tXTm-tEXp-GaDZ-96cftQ
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'allocated_pe' => '5',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38151',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'pv_uuid' => '9j2kzI-AJRt-6CCU-tXTm-tEXp-GaDZ-96cftQ',
##&nbsp;            'pv_size' => '149.05 GiB / not usable 3.84 MiB',
##&nbsp;            'allocatable' => 'yes',
##&nbsp;            'pv_name' => '/dev/sdb',
##&nbsp;            'total_pe' => '38156'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay  -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  /dev/sdb:vg1:312581808:-1:8:8:-1:4096:38156:38151:5:9j2kzI-AJRt-6CCU-tXTm-tEXp-GaDZ-96cftQ
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'allocated_pe' => '5',
##&nbsp;            'number_of_lv' => '-1',
##&nbsp;            'pv_status' => '8',
##&nbsp;            'pv_size_kb' => '312581808',
##&nbsp;            'pe_size' => '4096',
##&nbsp;            'free_pe' => '38151',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'pv_number' => '-1',
##&nbsp;            'pv_uuid' => '9j2kzI-AJRt-6CCU-tXTm-tEXp-GaDZ-96cftQ',
##&nbsp;            'pv_not_allocatable' => '8',
##&nbsp;            'pv_name' => '/dev/sdb',
##&nbsp;            'total_pe' => '38156'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay  -C
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV         VG   Fmt  Attr PSize   PFree
##&nbsp;  /dev/sdb    lvm2 a--  149.05g 149.03g
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pv_free' => '149.03g',
##&nbsp;            'vg_name' => '-',
##&nbsp;            'attr' => 'a--',
##&nbsp;            'pv_size' => '149.05g',
##&nbsp;            'fmt' => 'lvm2',
##&nbsp;            'pv_name' => '/dev/sdb'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvdisplay  --maps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Physical volume ---
##&nbsp;  PV Name               /dev/sdb
##&nbsp;  VG Name               vg1
##&nbsp;  PV Size               149.05 GiB / not usable 3.84 MiB
##&nbsp;  Allocatable           yes
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Free PE               38151
##&nbsp;  Allocated PE          5
##&nbsp;  PV UUID               xcGwPL-e6uj-IGsu-Y88P-JEvP-MRod-kLIsH0
##&nbsp;
##&nbsp;  --- Physical Segments ---
##&nbsp;  Physical extent 0 to 4:
##&nbsp;    Logical volume	/dev/vg1/lvol0
##&nbsp;    Logical extents	0 to 4
##&nbsp;  Physical extent 5 to 38155:
##&nbsp;    FREE
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'allocated_pe' => '5',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38151',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'pv_uuid' => 'xcGwPL-e6uj-IGsu-Y88P-JEvP-MRod-kLIsH0',
##&nbsp;            'pv_size' => '149.05 GiB / not usable 3.84 MiB',
##&nbsp;            'physical_segments' => [
##&nbsp;                                     {
##&nbsp;                                       'physical_extent_0_to_4' => {
##&nbsp;                                                                     'logical_extents' => '0 to 4',
##&nbsp;                                                                     'logical_volume' => '/dev/vg1/lvol0'
##&nbsp;                                                                   }
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'physical_extent_5_to_38155' => 'FREE'
##&nbsp;                                     }
##&nbsp;                                   ],
##&nbsp;            'allocatable' => 'yes',
##&nbsp;            'pv_name' => '/dev/sdb',
##&nbsp;            'total_pe' => '38156'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pvdisplay {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    my @result_array;
    my $limit = -1;
    my $cmd = $resp->get_command();
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    @lines = grep(! /Input\/output\s+error/, @lines);
    my %res = (
          PV  => 'pv_name',
          VG  => 'vg_name',
          Fmt  => 'fmt',
          Attr => 'attr',
          PSize => 'pv_size',
          PFree => 'pv_free',
    );
    if ( $cmd =~ /-C/ ) {
        my @head = split( /\s+/, $lines[0] );
        for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
            $limit++;
            my @values = split( /\s+/, $lines[$i] );
            if ( $#head != $#values ) {
                splice( @values, 1, 0, "-" );
            }
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                $result_array[$limit]->{ $res{ $head[$j] } } = $values[$j];
            }
        }
    } elsif ($cmd =~ /--version/) {
        $limit++;
        foreach my $line (@lines) {
            my @temp = split(/:\s+/, $line);
            $result_array[$limit]->{to_key_name(item => $temp[0])} = $temp[1];
        }
    } elsif ($cmd =~ /-c/) {
	    my $pv_size;
        foreach my $line (@lines) {
           # The below regex matches the lines like "/dev/sdb" is a new physical volume of "149.05 GiB"
           if ($line =~ /\".*\".*\s+volume\s+\w+\s+\"(.*)\"/) {
                $pv_size = $1;
            } else {
             $limit++;
            my @head = (
                qw( pv_name vg_name pv_size_kb pv_number pv_status pv_not_allocatable
                    number_of_lv pe_size total_pe free_pe allocated_pe pv_uuid )
            );
            $result_array[$limit]->{pv_size} = $pv_size if(defined $pv_size );
            my @var = split(/:/, $line);
            for (my $j = 0; $j <= $#var; $j++) {
                $result_array[$limit]->{$head[$j]} = $var[$j];
            }
          }
        }
    } else {
        my $count = -1;
        my ($head,$temp_head);
        foreach my $line (@lines) {
            # The below regex matches the lines like Device "/dev/sdb" has a capacity of 149.05 GiB
            if ($line
                =~ /\w+\s+\"(.*)\"\s+\w+\s+\w+\s+\w+\s+\w+\s+(.*)/) {
                $limit++;
                $result_array[$limit]->{pv_name} = $1;
                $result_array[$limit]->{pv_size} = $2;
            } else {
                if ($line =~ /\---\s(.*)\s\---/) {
                    $head = $1;
                    $limit++ if($head !~ /Physical\s+Segments/);
                }
                my @arr = split(/\s\s+/, $line);
                if ($line !~/--*--/ && $head !~ /Physical\s+Segments/ && $line !~ /Scanning.*|\".*/ ) {
                    $result_array[$limit]->{to_key_name(item =>$arr[0])}
                        = defined($arr[1]) ? $arr[1] : '-';
                } elsif ($head =~ /Physical\s+Segments/
                    && $line =~ /(Physical\s+extent\s+.*):/) {
                    $count++;
                    $temp_head = to_key_name(item => $1);
                } elsif ($line =~ /(Logical\s+volume|Logical\s+extents|FREE)\s*(.*)/) {
                    $result_array[$limit]{to_key_name(item =>$head)}[$count]{$temp_head}{to_key_name(item => $1)}
                         = $2 if ($line !~ /FREE/);
                    $result_array[$limit]{to_key_name(item =>$head)}[$count]{$temp_head} = $line
                        if ($line =~ /FREE/);
                }
            }
        }
    }
    return \@result_array;
} ## end sub pvdisplay

## @name ::pvchange
##
## @summary Parses the raw output from the "pvchange" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvchange with any of its options as described in the
##    corresponding cdef file 'pvchange.cdefs'
## @synopsis
##      pvchange(%args);
## INPUTS TO METHOD CALL 'pvchange' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '--metadataignore' </td> <td> 'metadataignore' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uuid' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '--addtag' </td> <td> 'addtag' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pv-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'allocatable' </td> <td> '0' </td> </tr>
## <tr> <td> '--deltag' </td> <td> 'deltag' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub pvchange {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub pvchange

1;
