#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/route.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux route command
##
## @description
##    This is a library to parse the raw output of Linux route command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "route" will be route()
##
## @synopsis
##    use route;
##
##    route(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::route;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::route
##
## @summary Parses the raw output from the "route" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route(%args);
## INPUTS TO METHOD CALL 'route' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-ee' </td> <td> 'all-parameters' </td> <td> '1' </td> </tr>
## <tr> <td> 'W' </td> <td> 'window' </td> <td> '0' </td> </tr>
## <tr> <td> 'add' </td> <td> 'add' </td> <td> '1' </td> </tr>
## <tr> <td> 'gw' </td> <td> 'gateway' </td> <td> '1' </td> </tr>
## <tr> <td> 'N' </td> <td> 'metric' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reinstate' </td> <td> '0' </td> </tr>
## <tr> <td> '-net' </td> <td> 'net' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'kernel-FIB' </td> <td> '1' </td> </tr>
## <tr> <td> 'mod' </td> <td> 'modified-route' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'target' </td> <td> '0' </td> </tr>
## <tr> <td> 'I' </td> <td> 'initial-round-trip-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numerical-addresses' </td> <td> '1' </td> </tr>
## <tr> <td> 'netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> 'M' </td> <td> 'maximum-segment-size' </td> <td> '0' </td> </tr>
## <tr> <td> 'del' </td> <td> 'del' </td> <td> '1' </td> </tr>
## <tr> <td> 'dyn' </td> <td> 'dynamic-route' </td> <td> '1' </td> </tr>
## <tr> <td> 'dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> 'If' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-host' </td> <td> 'host' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'netstat-format' </td> <td> '1' </td> </tr>
## <tr> <td> 'reject' </td> <td> 'reject' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'kernel-routing-cache' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'family' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = route -n
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel IP routing table
##&nbsp;Destination     Gateway         Genmask         Flags Metric Ref    Use Iface
##&nbsp;10.72.219.0     0.0.0.0         255.255.255.0   U     0      0        0 eth0
##&nbsp;192.168.122.0   0.0.0.0         255.255.255.0   U     0      0        0 virbr0
##&nbsp;169.254.0.0     0.0.0.0         255.255.0.0     U     1002   0        0 eth0
##&nbsp;169.254.0.0     0.0.0.0         255.255.0.0     U     1003   0        0 eth1
##&nbsp;0.0.0.0         10.72.219.1     0.0.0.0         UG    0      0        0 eth0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '0.0.0.0',
##&nbsp;            'destination' => '10.72.219.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '0.0.0.0',
##&nbsp;            'destination' => '192.168.122.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'virbr0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '0.0.0.0',
##&nbsp;            'destination' => '169.254.0.0',
##&nbsp;            'metric' => '1002',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '0.0.0.0',
##&nbsp;            'destination' => '169.254.0.0',
##&nbsp;            'metric' => '1003',
##&nbsp;            'iface' => 'eth1',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'UG',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '10.72.219.1',
##&nbsp;            'destination' => '0.0.0.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '0.0.0.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = route -F
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel IP routing table
##&nbsp;Destination     Gateway         Genmask         Flags Metric Ref    Use Iface
##&nbsp;10.72.219.0     *               255.255.255.0   U     0      0        0 eth0
##&nbsp;192.168.122.0   *               255.255.255.0   U     0      0        0 virbr0
##&nbsp;link-local      *               255.255.0.0     U     1002   0        0 eth0
##&nbsp;link-local      *               255.255.0.0     U     1003   0        0 eth1
##&nbsp;default         sw-219-cr1.lab. 0.0.0.0         UG    0      0        0 eth0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => '10.72.219.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => '192.168.122.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'virbr0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => 'link-local',
##&nbsp;            'metric' => '1002',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => 'link-local',
##&nbsp;            'metric' => '1003',
##&nbsp;            'iface' => 'eth1',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'UG',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => 'sw-219-cr1.lab.',
##&nbsp;            'destination' => 'default',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '0.0.0.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = route -ee
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel IP routing table
##&nbsp;Destination     Gateway         Genmask         Flags Metric Ref    Use Iface    MSS   Window irtt
##&nbsp;10.72.219.0     *               255.255.255.0   U     0      0        0 eth0     0     0      0
##&nbsp;192.168.122.0   *               255.255.255.0   U     0      0        0 virbr0   0     0      0
##&nbsp;link-local      *               255.255.0.0     U     1002   0        0 eth0     0     0      0
##&nbsp;link-local      *               255.255.0.0     U     1003   0        0 eth1     0     0      0
##&nbsp;default         sw-219-cr1.lab. 0.0.0.0         UG    0      0        0 eth0     0     0      0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'window' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => '10.72.219.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'irtt' => '0',
##&nbsp;            'use' => '0',
##&nbsp;            'mss' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'window' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => '192.168.122.0',
##&nbsp;            'metric' => '0',
##&nbsp;            'irtt' => '0',
##&nbsp;            'use' => '0',
##&nbsp;            'mss' => '0',
##&nbsp;            'iface' => 'virbr0',
##&nbsp;            'genmask' => '255.255.255.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'window' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => 'link-local',
##&nbsp;            'metric' => '1002',
##&nbsp;            'irtt' => '0',
##&nbsp;            'use' => '0',
##&nbsp;            'mss' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'window' => '0',
##&nbsp;            'flags' => 'U',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '*',
##&nbsp;            'destination' => 'link-local',
##&nbsp;            'metric' => '1003',
##&nbsp;            'irtt' => '0',
##&nbsp;            'use' => '0',
##&nbsp;            'mss' => '0',
##&nbsp;            'iface' => 'eth1',
##&nbsp;            'genmask' => '255.255.0.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'window' => '0',
##&nbsp;            'flags' => 'UG',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => 'sw-219-cr1.lab.',
##&nbsp;            'destination' => 'default',
##&nbsp;            'metric' => '0',
##&nbsp;            'irtt' => '0',
##&nbsp;            'use' => '0',
##&nbsp;            'mss' => '0',
##&nbsp;            'iface' => 'eth0',
##&nbsp;            'genmask' => '0.0.0.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = route -C
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel IP routing cache
##&nbsp;Source          Destination     Gateway         Flags Metric Ref    Use Iface
##&nbsp;10.72.219.49    10.72.219.255   10.72.219.255   ibl   0      0        0 lo
##&nbsp;localhost.local localhost.local localhost.local l     0      0        5 lo
##&nbsp;10.72.219.150   10.72.219.255   10.72.219.255   ibl   0      0        1 lo
##&nbsp;10.72.219.236   255.255.255.255 255.255.255.255 ibl   0      0        0 lo
##&nbsp;lnx219-45.lab.e ns0.eng.btc.net sw-219-cr1.lab.       0      0       11 eth0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source' => '10.72.219.49',
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'ibl',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '10.72.219.255',
##&nbsp;            'destination' => '10.72.219.255',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'lo'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source' => 'localhost.local',
##&nbsp;            'use' => '5',
##&nbsp;            'flags' => 'l',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => 'localhost.local',
##&nbsp;            'destination' => 'localhost.local',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'lo'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source' => '10.72.219.150',
##&nbsp;            'use' => '1',
##&nbsp;            'flags' => 'ibl',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '10.72.219.255',
##&nbsp;            'destination' => '10.72.219.255',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'lo'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source' => '10.72.219.236',
##&nbsp;            'use' => '0',
##&nbsp;            'flags' => 'ibl',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => '255.255.255.255',
##&nbsp;            'destination' => '255.255.255.255',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'lo'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source' => 'lnx219-45.lab.e',
##&nbsp;            'use' => '11',
##&nbsp;            'flags' => '-',
##&nbsp;            'ref' => '0',
##&nbsp;            'gateway' => 'sw-219-cr1.lab.',
##&nbsp;            'destination' => 'ns0.eng.btc.net',
##&nbsp;            'metric' => '0',
##&nbsp;            'iface' => 'eth0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub route {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    my @head  = split( /\s+/, $lines[1] );

    for ( my $i = 2; $i <= $#lines; $i++ ) {
        my @values = split( /\s+/, $lines[$i] );
        if ( $cmd =~ /-C/ ) {
            if ( $#head != $#values ) {
                splice( @values, 3, 0, "-" );
            }
        }
        for ( my $j = 0; $j <= $#head; $j++ ) {
            $result_array[$count]->{ to_key_name( item => $head[$j] ) }
                = $values[$j];
        }
        $count++;
    }
    return \@result_array;
} ## end sub route

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output
        =~ /reject:.*|No\s+such.*|Unknown\s+host.*|File\s+exists.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|route:).*|Invalid argument.* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

