#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux  rpower commands
##
## @description
##    This is a library to parse the raw outputs of any of the Linux rpower commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rpower" will be rpower()
##
## @synopsis
##    use NACL::APISet::Node::CLI::Linux::Redhat::rpower;
##
##    rpower(%agrs);
##
## INPUTS TO METHOD CALL 'rpower' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/utils/bin/rpower
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like Operating Systems and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/utils/bin
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
##

package NACL::APISet::Host::CLI::Linux::Redhat::rpower;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $error_message;
    my $error;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /(FATAL:.*)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $1;
    }
    elsif ( $raw_output =~ /usage:/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(Unknown.*)/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(bash.*|.*[cC]ommand not found.*)/ ) {
        $$error_message = $1;
        $error          = "CommandNotFound";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::rpower
##
## @summary Parses the raw output from the "rpower" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rpower with any of its options as described in the 
##    corresponding cdef file 'rpower.cdefs'
## @synopsis
##      rpower(%args);
## INPUTS TO METHOD CALL 'rpower' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> 'reboot' </td> <td> 'reboot' </td> <td> '1' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'no-action' </td> <td> '1' </td> </tr> 
## <tr> <td> 'status' </td> <td> 'status' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'switchname' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'devicename' </td> <td> '0' </td> </tr> 
## <tr> <td> 'off' </td> <td> 'off' </td> <td> '1' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'find' </td> <td> '1' </td> </tr> 
## <tr> <td> 'on' </td> <td> 'on' </td> <td> '1' </td> </tr> 
## <tr> <td> 'reset' </td> <td> 'reset' </td> <td> '1' </td> </tr> 
## <tr> <td> 'shutdown' </td> <td> 'shutdown' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rpower plat-3040-1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;found entry plat-3040-1(=>Plat-3040-1.lab.netapp.com) - reboot requested for 1 power outlets
##&nbsp;sending 'off' request to sentry CDU at j2dc-pdu-ii12.lab.netapp.com(36/36).
##&nbsp;sleeping for 5 seconds...
##&nbsp;sending 'on' request to sentry CDU at j2dc-pdu-ii12.lab.netapp.com(36/36).
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sleeping_for' => '5 seconds',
##&nbsp;            'sending_off_request_to' => 'j2dc-pdu-ii12.lab.netapp.com(36/36)',
##&nbsp;            'sending_on_request_to' => 'j2dc-pdu-ii12.lab.netapp.com(36/36)',
##&nbsp;            'reboot_requested_for' => 'Plat-3040-1.lab.netapp.com'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = rpower -v plat-3040-1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;found entry plat-3040-1(=>Plat-3040-1.lab.netapp.com) - reboot requested for 1 power outlets
##&nbsp;sending 'off' request to sentry CDU at j2dc-pdu-ii12.lab.netapp.com(36/36).
##&nbsp;Trying 172.19.66.12...
##&nbsp;Connected to j2dc-pdu-ii12.lab.netapp.com (172.19.66.12).
##&nbsp;Escape character is '^]'.
##&nbsp;
##&nbsp;
##&nbsp;Sentry Switched CDU Version 6.0k (100707)
##&nbsp;
##&nbsp;Username: admn
##&nbsp;Password: 
##&nbsp;
##&nbsp;Location: 
##&nbsp;
##&nbsp;Switched CDU: OFF .bb4
##&nbsp;
##&nbsp;   Outlet   Outlet                    Outlet     Control
##&nbsp;   ID       Name                      Status     State
##&nbsp;
##&nbsp;   .BB4     TowerB_InfeedB_Outlet4    Off        Off       
##&nbsp;
##&nbsp;   Command successful
##&nbsp;
##&nbsp;Switched CDU: quit
##&nbsp;
##&nbsp;Session ended
##&nbsp;[closed]
##&nbsp;sleeping for 5 seconds...
##&nbsp;sending 'on' request to sentry CDU at j2dc-pdu-ii12.lab.netapp.com(36/36).
##&nbsp;Trying 172.19.66.12...
##&nbsp;Connected to j2dc-pdu-ii12.lab.netapp.com (172.19.66.12).
##&nbsp;Escape character is '^]'.
##&nbsp;
##&nbsp;
##&nbsp;Sentry Switched CDU Version 6.0k (100707)
##&nbsp;
##&nbsp;Username: admn
##&nbsp;Password: 
##&nbsp;
##&nbsp;Location: 
##&nbsp;
##&nbsp;Switched CDU: ON .bb4
##&nbsp;
##&nbsp;   Outlet   Outlet                    Outlet     Control
##&nbsp;   ID       Name                      Status     State
##&nbsp;
##&nbsp;   .BB4     TowerB_InfeedB_Outlet4    On         On        
##&nbsp;
##&nbsp;   Command successful
##&nbsp;
##&nbsp;Switched CDU: quit
##&nbsp;
##&nbsp;Session ended
##&nbsp;[closed]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sleeping_for' => '5 seconds',
##&nbsp;            'trying' => '172.19.66.12',
##&nbsp;            'switched_cdu' => [
##&nbsp;                                {
##&nbsp;                                  'on' => [
##&nbsp;                                            {
##&nbsp;                                              'outlet_state' => 'On',
##&nbsp;                                              'outlet_status' => 'On',
##&nbsp;                                              'outlet_id' => '.BB4',
##&nbsp;                                              'outlet_name' => 'TowerB_InfeedB_Outlet4'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                                  'show_system' => [
##&nbsp;                                                     {
##&nbsp;                                                       'username' => 'admn'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                  'quit' => [
##&nbsp;                                              {
##&nbsp;                                                'status' => 'Session ended'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                                  'off' => [
##&nbsp;                                             {
##&nbsp;                                               'outlet_state' => 'Off',
##&nbsp;                                               'outlet_status' => 'Off',
##&nbsp;                                               'outlet_id' => '.BB4',
##&nbsp;                                               'outlet_name' => 'TowerB_InfeedB_Outlet4'
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'switched_cdu_version' => '6.0k',
##&nbsp;            'connected_to' => 'j2dc-pdu-ii12.lab.netapp.com (172.19.66.12)',
##&nbsp;            'sending_off_request_to' => 'j2dc-pdu-ii12.lab.netapp.com(36/36)',
##&nbsp;            'reboot_requested_for' => 'Plat-3040-1.lab.netapp.com',
##&nbsp;            'sending_on_request_to' => 'j2dc-pdu-ii12.lab.netapp.com(36/36)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = rpower -v plat-3040-1 status
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;found entry plat-3040-1(=>Plat-3040-1.lab.netapp.com) - status requested for 1 power outlets
##&nbsp;sending 'status' request to sentry CDU at j2dc-pdu-ii12.lab.netapp.com(36/36).
##&nbsp;Trying 172.19.66.12...
##&nbsp;Connected to j2dc-pdu-ii12.lab.netapp.com (172.19.66.12).
##&nbsp;Escape character is '^]'.
##&nbsp;
##&nbsp;
##&nbsp;Sentry Switched CDU Version 6.0k (100707)
##&nbsp;
##&nbsp;Username: admn
##&nbsp;Password: 
##&nbsp;
##&nbsp;Location: 
##&nbsp;
##&nbsp;Switched CDU: show system
##&nbsp;
##&nbsp;   System Information
##&nbsp;
##&nbsp;      F/W Version:       Sentry Switched CDU Version 6.0k (100707)
##&nbsp;      NIC S/N:           8216345  
##&nbsp;      MAC Address:       00-0a-9c-52-3f-d9  
##&nbsp;      H/W Rev Code:      32
##&nbsp;      Flash Size:        2 MB
##&nbsp;      Uptime:            257 days 19 hours 19 minutes 48 seconds
##&nbsp;
##&nbsp;      Location:          
##&nbsp;
##&nbsp;      Area (Footprint):  (disabled) 
##&nbsp;
##&nbsp;      Power Factor:      1.0
##&nbsp;
##&nbsp;      3-Phase Load
##&nbsp;       Out-of-Balance
##&nbsp;       Threshold:        20 percent
##&nbsp;
##&nbsp;   Command successful
##&nbsp;
##&nbsp;Switched CDU: status
##&nbsp;
##&nbsp;   Outlet   Outlet                    Outlet     Control
##&nbsp;   ID       Name                      Status     State
##&nbsp;
##&nbsp;   .AA1     TowerA_InfeedA_Outlet1    On         Idle On   
##&nbsp;   .AA2     TowerA_InfeedA_Outlet2    On         Idle On   
##&nbsp;   .AA3     TowerA_InfeedA_Outlet3    On         Idle On   
##&nbsp;   .AA4     TowerA_InfeedA_Outlet4    On         Idle On   
##&nbsp;   .AA5     TowerA_InfeedA_Outlet5    On         Idle On   
##&nbsp;   .AA6     TowerA_InfeedA_Outlet6    On         Idle On   
##&nbsp;   .AA7     TowerA_InfeedA_Outlet7    On         Idle On   
##&nbsp;   .AA8     TowerA_InfeedA_Outlet8    On         Idle On   
##&nbsp;   .AB1     TowerA_InfeedB_Outlet1    On         Idle On   
##&nbsp;   .AB2     TowerA_InfeedB_Outlet2    On         Idle On   
##&nbsp;   .AB3     TowerA_InfeedB_Outlet3    On         Idle On   
##&nbsp;   .AB4     TowerA_InfeedB_Outlet4    On         Idle On   
##&nbsp;   .AB5     TowerA_InfeedB_Outlet5    On         Idle On   
##&nbsp;   .AB6     TowerA_InfeedB_Outlet6    On         On        
##&nbsp;   .AB7     TowerA_InfeedB_Outlet7    On         Idle On   
##&nbsp;   .AB8     TowerA_InfeedB_Outlet8    On         Idle On   
##&nbsp;
##&nbsp;   More (Y/es N/o): y
##&nbsp;
##&nbsp;   .AC1     TowerA_InfeedC_Outlet1    On         Idle On   
##&nbsp;   .AC2     TowerA_InfeedC_Outlet2    On         On        
##&nbsp;   .AC3     TowerA_InfeedC_Outlet3    On         On        
##&nbsp;   .AC4     TowerA_InfeedC_Outlet4    On         Idle On   
##&nbsp;   .AC5     TowerA_InfeedC_Outlet5    On         Idle On   
##&nbsp;   .AC6     TowerA_InfeedC_Outlet6    On         Idle On   
##&nbsp;   .AC7     TowerA_InfeedC_Outlet7    On         Idle On   
##&nbsp;   .AC8     TowerA_InfeedC_Outlet8    On         Idle On   
##&nbsp;   .BA1     TowerB_InfeedA_Outlet1    On         Idle On   
##&nbsp;   .BA2     TowerB_InfeedA_Outlet2    On         Idle On   
##&nbsp;   .BA3     TowerB_InfeedA_Outlet3    On         Idle On   
##&nbsp;   .BA4     TowerB_InfeedA_Outlet4    On         Idle On   
##&nbsp;   .BA5     TowerB_InfeedA_Outlet5    On         Idle On   
##&nbsp;   .BA6     TowerB_InfeedA_Outlet6    On         Idle On   
##&nbsp;   .BA7     TowerB_InfeedA_Outlet7    On         Idle On   
##&nbsp;   .BA8     TowerB_InfeedA_Outlet8    On         Idle On   
##&nbsp;
##&nbsp;   More (Y/es N/o): y
##&nbsp;
##&nbsp;   .BB1     TowerB_InfeedB_Outlet1    On         Idle On   
##&nbsp;   .BB2     TowerB_InfeedB_Outlet2    On         Idle On   
##&nbsp;   .BB3     TowerB_InfeedB_Outlet3    On         Idle On   
##&nbsp;   .BB4     TowerB_InfeedB_Outlet4    On         On        
##&nbsp;   .BB5     TowerB_InfeedB_Outlet5    On         On        
##&nbsp;   .BB6     TowerB_InfeedB_Outlet6    On         On        
##&nbsp;   .BB7     TowerB_InfeedB_Outlet7    On         Idle On   
##&nbsp;   .BB8     TowerB_InfeedB_Outlet8    On         Idle On   
##&nbsp;   .BC1     TowerB_InfeedC_Outlet1    On         Idle On   
##&nbsp;   .BC2     TowerB_InfeedC_Outlet2    On         On        
##&nbsp;   .BC3     TowerB_InfeedC_Outlet3    On         Idle On   
##&nbsp;   .BC4     TowerB_InfeedC_Outlet4    On         Idle On   
##&nbsp;   .BC5     TowerB_InfeedC_Outlet5    On         Idle On   
##&nbsp;   .BC6     TowerB_InfeedC_Outlet6    On         Idle On   
##&nbsp;   .BC7     TowerB_InfeedC_Outlet7    On         Idle On   
##&nbsp;   .BC8     TowerB_InfeedC_Outlet8    On         Idle On   
##&nbsp;
##&nbsp;   Command successful
##&nbsp;
##&nbsp;Switched CDU: quit
##&nbsp;
##&nbsp;Session ended
##&nbsp;Outlet: 36(.bb4) State: On - On        
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'trying' => '172.19.66.12',
##&nbsp;            'switched_cdu' => [
##&nbsp;                                {
##&nbsp;                                  'show_system' => [
##&nbsp;                                                     {
##&nbsp;                                                       'threshold' => '20 percent',
##&nbsp;                                                       'power_factor' => '1.0',
##&nbsp;                                                       'nic_s/n' => '8216345',
##&nbsp;                                                       'username' => 'admn',
##&nbsp;                                                       'mac_address' => '00-0a-9c-52-3f-d9',
##&nbsp;                                                       'flash_size' => '2 MB',
##&nbsp;                                                       'outlet' => 'On - On',
##&nbsp;                                                       'area_footprint' => '(disabled)',
##&nbsp;                                                       'uptime' => '257 days 19 hours 19 minutes 48 seconds',
##&nbsp;                                                       'h/w_rev_code' => '32'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                  'quit' => [
##&nbsp;                                              {
##&nbsp;                                                'status' => 'Session ended'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                                  'status' => [
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA1',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA2',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA3',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA4',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA5',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA6',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA7',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AA8',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedA_Outlet8'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB1',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB2',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB3',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB4',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB5',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB6',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB7',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AB8',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedB_Outlet8'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC1',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC2',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC3',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC4',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC5',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC6',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC7',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.AC8',
##&nbsp;                                                  'outlet_name' => 'TowerA_InfeedC_Outlet8'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA1',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA2',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA3',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA4',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA5',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA6',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA7',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BA8',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedA_Outlet8'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB1',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB2',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB3',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB4',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB5',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB6',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB7',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BB8',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedB_Outlet8'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC1',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet1'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC2',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet2'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC3',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet3'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC4',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet4'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC5',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet5'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC6',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet6'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC7',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet7'
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'outlet_state' => 'Idle On',
##&nbsp;                                                  'outlet_status' => 'On',
##&nbsp;                                                  'outlet_id' => '.BC8',
##&nbsp;                                                  'outlet_name' => 'TowerB_InfeedC_Outlet8'
##&nbsp;                                                }
##&nbsp;                                              ]
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'status_requested_for' => 'Plat-3040-1.lab.netapp.com',
##&nbsp;            'switched_cdu_version' => '6.0k',
##&nbsp;            'connected_to' => 'j2dc-pdu-ii12.lab.netapp.com (172.19.66.12)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rpower {
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my @result_array;
    my $error = _check_error(
        "raw_output"      => $raw_output,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $raw_output, response_object => $response );
    my $key="";
    my $key1="";
    my $key2="";
    my $i=0;
    my $j=0;
    foreach my $line (@lines) {
         if($line =~ /^(found\sentry)\s\w+-\d+\-\d\(=>(\S+)\)\s-\s(\w+\s\w+\s\w+).*/){
                $result_array[0]->{ to_key_name( item => $3 )}=$2;
         }elsif($line =~ /sending\s('on'|'off')\srequest\sto(.*at)\s+(\S+)\./){
                if($1 eq "'on'"){
                    $result_array[0]->{"sending_on_request_to"}=$3;
                }else{
                    $result_array[0]->{"sending_off_request_to"}=$3;
                }       
         }elsif($line =~ /^(Connected\sto)\s(.*)\./){
                $result_array[0]->{"connected_to"}=$2;
         }elsif($line =~ /\w+\s(Switched\sCDU)\s(Version)\s(\S+)\s\S+/){
                $result_array[0]->{"switched_cdu_version"}=$3;
         }elsif($line =~ /^(Switched\sCDU):\s((status)|(ON)|(OFF)|(quit)|(show)).*/){
                $key="switched_cdu";
                $key1= to_key_name( item => $2 );
         }elsif($line =~ /(\.\S+)\s+(\S+)\s+(On|Off)\s+(Wake.*|Idle.*|On|Off)/){
                if($key1 eq "status"){
                    $result_array[0]->{$key}->[0]->{$key1}->[$i]->{"outlet_id"} = $1;
                    $result_array[0]->{$key}->[0]->{$key1}->[$i]->{"outlet_name"} = $2;
                    $result_array[0]->{$key}->[0]->{$key1}->[$i]->{"outlet_status"} = $3;
                    $result_array[0]->{$key}->[0]->{$key1}->[$i]->{"outlet_state"} = $4;
                    $i++;
                }else{
                    $result_array[0]->{$key}->[0]->{$key1}->[0]->{"outlet_id"} = $1;
                    $result_array[0]->{$key}->[0]->{$key1}->[0]->{"outlet_name"} = $2;
                    $result_array[0]->{$key}->[0]->{$key1}->[0]->{"outlet_status"} = $3;
                    $result_array[0]->{$key}->[0]->{$key1}->[0]->{"outlet_state"} = $4;    
                }
         }elsif( $line =~ /(.*):\s+((.*)|)/ ) {
                $key2="show_system";
                $key="switched_cdu";
                    if(($2) && ($2 ne "y")){     
                        $result_array[0]->{$key}->[0]->{$key2}->[0]->{to_key_name(item => $1)} = $2;
                    }
         }elsif($line =~ /^(Session\s\S+)/ ){
                $key1="quit";
                $result_array[0]->{$key}->[0]->{$key1}->[0]->{"status"} = $1;
         }elsif($line =~ /^(\S+\.\S+\.\S+.*)/ ){
                $result_array[0]->{"machine"}->[$j]=$1;
                $j++;
         }elsif($line =~ /^(sleeping\sfor|Trying)\s(.*).../){
                $result_array[0]->{to_key_name( item => $1 )}=$2;
         }elsif($line =~ /""/){
                next;
         }
}
return \@result_array;
}    # end of rpower

1;

