#$Id$
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux sanlun command
##
## @description
##    This is a library to parse the raw output of Linux sanlun command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sanlun lun show"
##    will be sanlun_lun_show)
##
## @synopsis
##    use sanlun;
##
##    sanlun_lun_show(%args);
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/sanlun lun show
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'experimental', '6.22'
## This would select sanlun lun show.experimental, sanlun lun show.6.22 respectively.
##
##
## TEST PARAMETERS
##
## NACL_SANLUN_LUN_SHOW_FULL_PATH=<path to the sanlun tool>.
## Eg: NACL_SANLUN_LUN_SHOW_FULL_PATH=/usr/sbin/sanlun lun show
## This override is applicable to every sanlun lun show command invoked
## irrespective of the 'set' used.
##
## NACL_LINUX_SANLUN_LUN_SHOW_FULL_PATH=<path to the sanlun tool>.
## Eg: NACL_LINUX_SANLUN_LUN_SHOW_FULL_PATH=/usr/sbin/sanlun lun show
## This is applicable to commands that use Linux APISets only.
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::sanlun;

use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~
/(device filename.*does not exist|no LUNs available from controller|LUN.*is not available|No supported adapters are present)/si
      )
    {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    elsif ( $output =~ /(sanlun lun show .*|No information available for adapter .*)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    }
    return $error;
}

## @name ::sanlun_lun_show
##
## @summary Parses the raw output from the "/usr/sbin/sanlun lun show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sanlun_lun_show with any of its options as described in the 
##    corresponding cdef file 'sanlun.cdefs'
## @synopsis
##      sanlun_lun_show(%args);
## INPUTS TO METHOD CALL 'sanlun_lun_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'multipath-info' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'controller-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'host-dev-filename' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'pathname' </td> <td> '0' </td> </tr> 
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'query-native' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /usr/sbin/sanlun lun show  -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                    ONTAP Path: rahula-vsim-3:/vol/vol0/iscsi_lun
##&nbsp;                           LUN: 0
##&nbsp;                      LUN Size: 40m
##&nbsp;           Controller CF State: Cluster Disabled
##&nbsp;            Controller Partner:
##&nbsp;                          Mode: 7
##&nbsp;            Multipath Provider: None
##&nbsp;--------- ---------- ------- ------------ ----------------------------------------------
##&nbsp;host      controller                      controller
##&nbsp;path      path       /dev/   host         target
##&nbsp;state     type       node    adapter	  port
##&nbsp;--------- ---------- ------- ------------ ----------------------------------------------
##&nbsp;up        iscsi      sdc     host5        10.238.3.164
##&nbsp;
##&nbsp;                    ONTAP Path: rahula-vsim-3:/vol/vol0/lin_lun1
##&nbsp;                           LUN: 0
##&nbsp;                      LUN Size: 40m
##&nbsp;           Controller CF State: Cluster Disabled
##&nbsp;            Controller Partner:
##&nbsp;                          Mode: 7
##&nbsp;            Multipath Provider: None
##&nbsp;--------- ---------- ------- ------------ ----------------------------------------------
##&nbsp;host      controller                      controller
##&nbsp;path      path       /dev/   host         target
##&nbsp;state     type       node    adapter	  port
##&nbsp;--------- ---------- ------- ------------ ----------------------------------------------
##&nbsp;up        iscsi      sdd     host5        10.238.3.164
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ontap_path' => 'rahula-vsim-3:/vol/vol0/iscsi_lun',
##&nbsp;            'dev_node' => 'sdc',
##&nbsp;            'mode' => '7',
##&nbsp;            'host_path_state' => 'up',
##&nbsp;            'multipath_provider' => 'None',
##&nbsp;            'controller_target_port' => '10.238.3.164',
##&nbsp;            'host_hba' => 'host5',
##&nbsp;            'lun' => '0',
##&nbsp;            'lun_size' => '40m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'controller_partner' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ontap_path' => 'rahula-vsim-3:/vol/vol0/lin_lun1',
##&nbsp;            'dev_node' => 'sdd',
##&nbsp;            'mode' => '7',
##&nbsp;            'host_path_state' => 'up',
##&nbsp;            'multipath_provider' => 'None',
##&nbsp;            'controller_target_port' => '10.238.3.164',
##&nbsp;            'host_hba' => 'host5',
##&nbsp;            'lun' => '0',
##&nbsp;            'lun_size' => '40m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'controller_partner' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = /usr/sbin/sanlun lun show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;controller(7mode)/                       device          host                  lun
##&nbsp;vserver(Cmode)       lun-pathname        filename        adapter    protocol   size    mode
##&nbsp;-------------------------------------------------------------------------------------------
##&nbsp;rahula-vsim-3        /vol/vol0/iscsi_lun /dev/sdc        host5      iSCSI      40m     7
##&nbsp;rahula-vsim-3        /vol/vol0/lin_lun1  /dev/sdd        host5      iSCSI      40m     7
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m',
##&nbsp;            'adapter' => 'host5',
##&nbsp;            'device_filename' => '/dev/sdc',
##&nbsp;            'lun_pathname' => '/vol/vol0/iscsi_lun',
##&nbsp;            'lun_state' => '7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m',
##&nbsp;            'adapter' => 'host5',
##&nbsp;            'device_filename' => '/dev/sdd',
##&nbsp;            'lun_pathname' => '/vol/vol0/lin_lun1',
##&nbsp;            'lun_state' => '7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = /usr/sbin/sanlun lun show -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;gsrikant-vsim1:/vol/vol0/lun_test2 (LUN 1)		Lun state: GOOD
##&nbsp;Lun Size:     10m (10485760)     Controller_CF_State: Cluster Disabled
##&nbsp;Protocol: iSCSI		Controller Partner: 
##&nbsp;Multipath-provider: NONE
##&nbsp;--------- ---------- ------- ------------ --------------------------------------------- ---------------
##&nbsp;   sanlun Controller                                                            Primary         Partner
##&nbsp;   path         Path   /dev/         Host                                    Controller      Controller
##&nbsp;   state        type    node          HBA                                          port            port
##&nbsp;--------- ---------- ------- ------------ --------------------------------------------- ---------------
##&nbsp;     GOOD  iscsi         sdd        host8                                  10.238.35.63              --
##&nbsp;     GOOD  iscsi         sde        host9                                  10.238.35.62              --
##&nbsp;
##&nbsp;gsrikant-vsim1:/vol/vol0/lun_test (LUN 0)		Lun state: GOOD
##&nbsp;Lun Size:     10m (10485760)     Controller_CF_State: Cluster Disabled
##&nbsp;Protocol: iSCSI		Controller Partner: 
##&nbsp;Multipath-provider: NONE
##&nbsp;--------- ---------- ------- ------------ --------------------------------------------- ---------------
##&nbsp;   sanlun Controller                                                            Primary         Partner
##&nbsp;   path         Path   /dev/         Host                                    Controller      Controller
##&nbsp;   state        type    node          HBA                                          port            port
##&nbsp;--------- ---------- ------- ------------ --------------------------------------------- ---------------
##&nbsp;     GOOD  iscsi         sdb        host8                                  10.238.35.63              --
##&nbsp;     GOOD  iscsi         sdc        host9                                  10.238.35.62              --
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'ontap_path' => 'gsrikant-vsim1:/vol/vol0/lun_test2',
##&nbsp;            'dev_node' => 'sdd',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'multipath_provider' => 'NONE',
##&nbsp;            'primary_controller_port' => '10.238.35.63',
##&nbsp;            'host_hba' => 'host8',
##&nbsp;            'lun_size' => '10m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'sanlun_path_state' => 'GOOD',
##&nbsp;            'controller_partner' => '',
##&nbsp;            'partner_controller_port' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'ontap_path' => 'gsrikant-vsim1:/vol/vol0/lun_test2',
##&nbsp;            'dev_node' => 'sde',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'multipath_provider' => 'NONE',
##&nbsp;            'primary_controller_port' => '10.238.35.62',
##&nbsp;            'host_hba' => 'host9',
##&nbsp;            'lun_size' => '10m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'sanlun_path_state' => 'GOOD',
##&nbsp;            'controller_partner' => '',
##&nbsp;            'partner_controller_port' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'ontap_path' => 'gsrikant-vsim1:/vol/vol0/lun_test',
##&nbsp;            'dev_node' => 'sdb',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'multipath_provider' => 'NONE',
##&nbsp;            'primary_controller_port' => '10.238.35.63',
##&nbsp;            'host_hba' => 'host8',
##&nbsp;            'lun_size' => '10m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'sanlun_path_state' => 'GOOD',
##&nbsp;            'controller_partner' => '',
##&nbsp;            'partner_controller_port' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'ontap_path' => 'gsrikant-vsim1:/vol/vol0/lun_test',
##&nbsp;            'dev_node' => 'sdc',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'multipath_provider' => 'NONE',
##&nbsp;            'primary_controller_port' => '10.238.35.62',
##&nbsp;            'host_hba' => 'host9',
##&nbsp;            'lun_size' => '10m',
##&nbsp;            'controller_cf_state' => 'Cluster Disabled',
##&nbsp;            'controller_path_type' => 'iscsi',
##&nbsp;            'sanlun_path_state' => 'GOOD',
##&nbsp;            'controller_partner' => '',
##&nbsp;            'partner_controller_port' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sanlun_lun_show {
    my %args = @_;
    my ( $error_message, @result_array );

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "sanlun_lun_show",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp
    );
    my $i      = -1;
    my $option = $command;
    my @temp;
    my $controller_info_hashref = ();
    my ( $temp1, $temp2, $temp3, $temp4, $temp5, $temp6 ) = undef;
    my $counter = -1 ;
    foreach my $line (@lines) {
        if($counter > -1){
            $counter --;
            next;
        }
        if ( $line =~ /controller.*device\s+host\s+.*/ ) {
            next;
        }
        elsif (( $line =~ /^vserver\(.*\)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+).*/ )
            || ( $line =~ /.*lun-pathname.*protocol.*/ ) )
        {
            next;
        }
        elsif ( $line =~ /(Lun\s+Size):\s+(\S+)\s+?\S+\s+(.*CF.*):\s+(.*)/ ) {
            $temp[$i]->{ to_key_name( item => $1 ) } =
              trim_string( 'string' => "$2" );
            $temp[$i]->{ to_key_name( item => $3 ) } =
              trim_string( 'string' => "$4" );
        }
        elsif ( $line =~ /(Protocol):\s+(\S+)\s+(\S+\s+Partner):\s*(.*)/ ) {
            $temp[$i]->{ to_key_name( item => $1 ) } =
              trim_string( 'string' => "$2" );
            $temp[$i]->{ to_key_name( item => $3 ) } =
              trim_string( 'string' => "$4" );
        }
        elsif ( $line =~ /(.*IP\s+address:)\s+(\S+)\s+(.*number)\s+(\S+)/ ) {
            $temp[$i]->{ to_key_name( item => $1 ) } =
              trim_string( 'string' => "$2" );
            $temp[$i]->{ to_key_name( item => $3 ) } =
              trim_string( 'string' => "$4" );

        }
        elsif ( $line =~
            /^(\S+)?\:\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+\s+\S+)\s+(\w+)/ )
        {
            $i++;
            $temp[$i]->{'controller'}      = $1;
            $temp[$i]->{'lun_pathname'}    = $2;
            $temp[$i]->{'device_filename'} = $3;
            $temp[$i]->{'adapter'}         = $4;
            $temp[$i]->{'protocol'}        = $5;
            $temp[$i]->{'lun_size'}        = $6;
            $temp[$i]->{'lun_state'}       = $7;
        }
        elsif (
            (
                $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/
            )
            || ( $line =~
                /(\w+)\:\s+(\S+)\s+(\S+)\s+(\w+)\s+(\w+)\s+(\S+\s+\S+)\s+(\w+)$/
            )
          )
        {
            $i++;
            $temp[$i]->{'controller'}      = $1;
            $temp[$i]->{'lun_pathname'}    = $2;
            $temp[$i]->{'device_filename'} = $3;
            $temp[$i]->{'adapter'}         = $4;
            $temp[$i]->{'protocol'}        = $5;
            $temp[$i]->{'lun_size'}        = $6;
            $temp[$i]->{'lun_state'}       = $7;
        }
        elsif ( $line =~ /ONTAP\s+Path\:\s+(\S+)/ ) {
            $i++;
            $temp[$i]->{'ontap_path'} = $1;
        }
        elsif ( $line =~ /(\S+)\s+\(\S+\s+\S+\)\s+Lun state:?\s+(\S+)/ ) {
            $i++;
            $temp[$i]->{'ontap_path'} = $1;
            $temp[$i]->{'lun_state'}  = $2;
        }
        elsif ( $line =~ /.*\s+volume\s+name:\s*(\S+)\s+FSID:(\w+)/ ) {
            $temp[$i]->{'volume_name'} = $1;
            $temp[$i]->{'fsid'}        = $2;

        }
        elsif ( $line =~ /.*\s+qtree\s+name:\s+(.*)\s+ID:(.*)/ ) {
            $temp[$i]->{'qtree_name'} = trim_string( 'string' => "$1" );
            $temp[$i]->{'qtree_id'} = $2;

        }
        elsif ( $line =~ /.*\s+snapshot\s+name:(.*)\s+(ID:(.*))?/ ) {
            $temp[$i]->{'snapshot_name'} = trim_string( 'string' => "$1" );
            $temp[$i]->{'snapshot_id'} = $3;

        }
        elsif ( $line =~ /Multipath-provider:\s*(.*)/i ) {
            $temp[$i]->{'multipath_provider'} = $1;
        }
        elsif($line =~ /(Vserver\s+IP\s+address)\:(.*)/i){
            $temp1 = $1;
            $temp2 = $2;
            if ( $i == -1 ) { $i = 0; }
            $temp1 =~s/-/_/g;
            my $key = to_key_name( item => $temp1 ) ;
            $temp[$i]->{$key} = [$temp2];
            my @array = @lines ;
            my $row = shift(@array) ;
            while($row !~  /(Vserver\s+IP\s+address)\:(.*)/) {
	        $row = shift(@array) ;
            }
            $row = shift(@array) ;
            while($row =~ /^\[*(\w+\d+:\d+\w+.*)\]/) {
                push  @{$temp[$i]->{$key}}, $1;
                $counter ++;
                $row = shift(@array);
            }
        }
	elsif( $line =~ /^\[(\w+\d+:\d+\w+.*)\]$/) {
	     next ;
	}
        elsif ( $line =~ /(.*)\:(.*)/ ) {
            $temp2 = $2;
            if ( $i == -1 ) { $i = 0; }
            $temp1 = $1;
            $temp1 =~s/-/_/g;
            if($line =~ /(Vserver volume name)/i) {
	        my @arr = split(/\\t/,$line);
		foreach(@arr) {
                    if($_ =~ /(.*):(.*)/) {
                        $temp[$i]->{ to_key_name( item => $1) }= trim_string( 'string' => $2 );
                    }
		}
            }
            else {
                $temp[$i]->{ to_key_name( item => $temp1 ) } =
                   trim_string( 'string' => $temp2 );
		 }
        }
        elsif (( $line =~ /host\s+controller.*/ )
            || ( $line =~ /path\s+path\s+.*target/ )
            || ( $line =~ /state\s+type\s+node.*/ ) )
        {
            next;
        }
        elsif ( $line =~ /^(\w+)\s+(\w+)\s+(\S+)\s+(\S+)\s+(\S+)$/ ) {
            $temp[$i]->{'host_path_state'}        = $1;
            $temp[$i]->{'controller_path_type'}   = $2;
            $temp[$i]->{'dev_node'}               = $3;
            $temp[$i]->{'host_hba'}               = $4;
            $temp[$i]->{'controller_target_port'} = $5;
        }
        elsif ( $line =~ /^\s+((\w+\s*)+):\s*(\S+)$/ ) {
            $temp1 = $1;
            $temp2 = $3;
            $temp1 =~ s/\s+/_/g;
            $temp1 =~ tr/[A-Z]/[a-z]/;
            $temp[$i]->{$temp1} = $temp2;
        }
        elsif ( $line =~ /^\s+((\w+\s*)+):\s*(\S+)\s+(\S+):(\S+)$/ ) {
            $temp1 = $1;
            $temp2 = $3;
            $temp3 = $4;
            $temp4 = $5;
            $temp1 =~ s/\s+/_/g;
            $temp3 =~ s/\s+/_/g;
            $temp1 =~ tr/[A-Z]/[a-z]/;
            $temp3 =~ tr/[A-Z]/[a-z]/;
            $temp[$i]->{$temp1} = $temp2;
            $temp[$i]->{$temp3} = $temp4;
        }
        elsif ( $line =~ /^\s+((\w+\s*)+):\s*(\S+)\s+(.+)\s+(\S+)$/ ) {
            $temp1 = $1;
            $temp2 = $3;
            $temp3 = $4;
            $temp4 = $5;
            $temp1 =~ s/\s+/_/g;
            $temp3 =~ s/\s+/_/g;
            $temp1 =~ tr/[A-Z]/[a-z]/;
            $temp3 =~ tr/[A-Z]/[a-z]/;
            $temp[$i]->{$temp1} = $temp2;
            $temp[$i]->{$temp3} = $temp4;
        }
        elsif ( $line =~ /(\S+):(\S+)\s+\((\w+)\s+(\w+)\)\s+(.*):\s+(.*)/ ) {
            $i++;
            $temp1 = $1;
            $temp2 = $2;
            $temp3 = $3;
            $temp4 = $4;
            $temp5 = $5;
            $temp6 = $6;
            $temp5 =~ s/\s+/_/g;
            $temp5 =~ tr/[A-Z]/[a-z]/;
            $temp[$i]->{'controller'}   = $temp1;
            $temp[$i]->{'lun_pathname'} = $temp2;
            $temp[$i]->{$temp3}         = $temp4;
            $temp[$i]->{$temp5}         = $temp6;
        }
        elsif ( $line =~ /((\S+\s*)+):\s+(\S+(\s+\(\S+\))?)\s+(.*):\s+(.*)/ ) {
            $temp1 = $1;
            $temp2 = $3;
            $temp3 = $5;
            $temp4 = $6;
            $temp1 =~ s/\s+/_/g;
            $temp3 =~ s/\s+/_/g;
            $temp1 =~ tr/[A-Z]/[a-z]/;
            $temp3 =~ tr/[A-Z]/[a-z]/;
            $temp[$i]->{$temp1} = $temp2;
            $temp[$i]->{$temp3} = $temp4;
        }
        elsif ( $line =~
        /\s*(\S+)\s+(\S+)\s+(\S+)\s+(\d+):(\d+):(\d+):(\d+)\s+(\S+)\s+(\S+)\s+(\d+):(\d+)/
          )
        {
            $temp[$i]->{'sanlun_path_state'}       = $1;
            $temp[$i]->{'controller_path_type'}    = $2;
            $temp[$i]->{'dev_node'}                = $3;
            $temp[$i]->{'host'}                    = $4;
            $temp[$i]->{'channel'}                 = $5;
            $temp[$i]->{'id'}                      = $6;
            $temp[$i]->{'lun_number'}              = $7;
            $temp[$i]->{'primary_controller_port'} = $8;
            $temp[$i]->{'partner_controller_port'} = $9;
            $temp[$i]->{'major'}                   = $10;
            $temp[$i]->{'minor'}                   = $11;
        }
        elsif ( $line =~ /\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*/ )
        {
            next if $line =~ /---+/ or $line =~ /path\s+/i;
            my @keys = keys %{$temp[$i]};
            my @keys_expected
                    = qw/protocol cf_state partner lun_size ontap_path multipath_provider lun_state/;

            my $diff = 1;
            my $value;
            # The 'if' block below pulls out parsed output of lines like these:
            #"gsrikant-vsim1:/vol/vol0/lun_test2 (LUN 1)     Lun state: GOOD
            #Lun Size:     10m (10485760)     Controller_CF_State: Cluster Disabled
            #Protocol: iSCSI     Controller Partner: 
            #Multipath-provider: NONE"
            # The parsed output is then pushed into every other hashref in the
            # parsed output to maintain backward compatibility.
            if (scalar @keys == scalar @keys_expected){
                foreach my $key (@keys_expected){
                    $value = grep {$_ =~ /$key/;} @keys;
                    if ($value){
                        $diff = 0;
                    }
                }
                $controller_info_hashref = splice(@temp, $i, 1) if !$diff;
            }
            my $sanlun_path_state = $1;
            my $controller_path_type = $2;
            my $dev_node = $3;
            my $host_hba = $4;
            my $primary_controller_port = $5;
            my $partner_controller_port = $6;
            $i++;
            if ($sanlun_path_state =~ /-+/) {
                $temp[$i]->{'sanlun_path_state'} = '';
            } else {
                $temp[$i]->{'sanlun_path_state'} = $sanlun_path_state;
            }
            $temp[$i]->{'controller_path_type'} = $controller_path_type;
            $temp[$i]->{'dev_node'}             = $dev_node;
            $temp[$i]->{'host_hba'}             = $host_hba;
            if ( $primary_controller_port =~ /-+/ ) { $temp[$i]->{'primary_controller_port'} = ''; }
            else {
                $temp[$i]->{'primary_controller_port'} = $primary_controller_port;
            }
            if ( $sanlun_path_state =~ /-*/ ) { $temp[$i]->{'partner_controller_port'} = ''; }
            else {
                $temp[$i]->{'partner_controller_port'} = $partner_controller_port;
            }
            my $lun_info_hashref = $temp[$i];
            $temp[$i] = {%{$lun_info_hashref}, %{$controller_info_hashref}} if $controller_info_hashref;
        }
    }
    @temp = grep {defined $_} @temp;
    return \@temp;    #to be updated to return a reference
}

## @name ::sanlun_fcp_show_adapter
##
## @summary Parses the raw output from the "/usr/sbin/sanlun fcp show adapter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sanlun_fcp_show_adapter with any of its options as
##    described in the corresponding cdef file 'sanlun.cdefs'
## @synopsis
##      sanlun_fcp_show_adapter(%args);
## INPUTS TO METHOD CALL 'sanlun_fcp_show_adapter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /usr/sbin/sanlun fcp show adapter -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; adapter name:      host0
##&nbsp; WWPN:              210000e08b86b783
##&nbsp; WWNN:              200000e08b86b783
##&nbsp; driver name:       qla2xxx
##&nbsp; model:             QLA2342
##&nbsp; model description: QLogic QLA2342
##&nbsp; serial number:     E26935
##&nbsp; hardware version:  FC5010409-10
##&nbsp; driver version:    v.8.05.00.03.06.5-k2
##&nbsp; firmware version:  v.3.03.27 IPX
##&nbsp; Number of ports:   1
##&nbsp; port type:         Fabric
##&nbsp; port state:        Operational
##&nbsp; supported speed:   1 GBit/sec, 2 GBit/sec
##&nbsp; negotiated speed:  2 GBit/sec
##&nbsp; OS device name:    /sys/class/scsi_host/host1
##&nbsp;
##&nbsp; adapter name:      host1
##&nbsp; WWPN:              210100e08ba6b783
##&nbsp; WWNN:              200100e08ba6b783
##&nbsp; driver name:       qla2xxx
##&nbsp; model:             QLA2342
##&nbsp; model description: QLogic QLA2342
##&nbsp; serial number:     E26935
##&nbsp; hardware version:  FC5010409-10
##&nbsp; driver version:    v.8.05.00.03.06.5-k2
##&nbsp; firmware version:  v.3.03.27 IPX
##&nbsp; Number of ports:   1
##&nbsp; port type:         Fabric
##&nbsp; port state:        Operational
##&nbsp; supported speed:   1 GBit/sec, 2 GBit/sec
##&nbsp; negotiated speed:  2 GBit/sec
##&nbsp; OS device name:    /sys/class/scsi_host/host2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => 'E26935',
##&nbsp;            'negotiated_speed' => '2 GBit/sec',
##&nbsp;            'model_description' => 'QLogic QLA2342',
##&nbsp;            'model' => 'QLA2342',
##&nbsp;            'supported_speed' => '1 GBit/sec, 2 GBit/sec',
##&nbsp;            'port_state' => 'Operational',
##&nbsp;            'port_type' => 'Fabric',
##&nbsp;            'driver_version' => 'v.8.05.00.03.06.5-k2',
##&nbsp;            'driver_name' => 'qla2xxx',
##&nbsp;            'os_device_name' => '/sys/class/scsi_host/host1',
##&nbsp;            'hardware_version' => 'FC5010409-10',
##&nbsp;            'wwpn' => '210000e08b86b783',
##&nbsp;            'firmware_version' => 'v.3.03.27 IPX',
##&nbsp;            'number_of_ports' => '1',
##&nbsp;            'wwnn' => '200000e08b86b783',
##&nbsp;            'adapter-name' => 'host0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => 'E26935',
##&nbsp;            'negotiated_speed' => '2 GBit/sec',
##&nbsp;            'model_description' => 'QLogic QLA2342',
##&nbsp;            'model' => 'QLA2342',
##&nbsp;            'supported_speed' => '1 GBit/sec, 2 GBit/sec',
##&nbsp;            'port_state' => 'Operational',
##&nbsp;            'port_type' => 'Fabric',
##&nbsp;            'driver_version' => 'v.8.05.00.03.06.5-k2',
##&nbsp;            'driver_name' => 'qla2xxx',
##&nbsp;            'os_device_name' => '/sys/class/scsi_host/host2',
##&nbsp;            'hardware_version' => 'FC5010409-10',
##&nbsp;            'wwpn' => '210100e08ba6b783',
##&nbsp;            'firmware_version' => 'v.3.03.27 IPX',
##&nbsp;            'number_of_ports' => '1',
##&nbsp;            'wwnn' => '200100e08ba6b783',
##&nbsp;            'adapter-name' => 'host1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = /usr/sbin/sanlun fcp show adapter
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;host0             WWPN:210000e08b86b783
##&nbsp;
##&nbsp;host1             WWPN:210100e08ba6b783
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wwpn' => '210000e08b86b783',
##&nbsp;            'adapter-name' => 'host0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'wwpn' => '210100e08ba6b783',
##&nbsp;            'adapter-name' => 'host1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub sanlun_fcp_show_adapter {
    my %args = @_;
    my ( $error_message, @result_array );

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "sanlun_fcp_show_adapter",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp
    );
    my $j = 0;
    my (@result_array,$i);

    if ($command =~ /-v/ ) {
        foreach my $line (@lines) {
            if($line =~ /adapter name:\s+(\S+)/) {
                $i = $j;
                $j++;
                $result_array[$i]->{ to_key_name( item => "adapter-name") } = $1;
            }elsif($line =~ /(.*?)\:\s+(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } else {
        foreach my $line (@lines) {
            if($line =~ /(.*?)\s+WWPN:(\w+)/) {
                $result_array[$j]->{ to_key_name( item => "adapter-name") } = $1;
                $result_array[$j]->{ to_key_name( item => "WWPN") } = $2;
                $j++;
            }
        }
    }
    return \@result_array;

}

1;
