# Copyright (c) 2012-13 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux command 'service'
##
## @description
##    This is a library to parse the raw output of Linux command 'service'.
##
## @synopsis
##    use service;
##
##    service(%args);
##
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::service;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::service
##
## @summary Parses the raw output from the "service" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from service with any of its options as described in the 
##    corresponding cdef file 'service.cdefs'
## @synopsis
##      service(%args);
## INPUTS TO METHOD CALL 'service' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'script' </td> <td> '0' </td> </tr> 
## <tr> <td> '--full-restart' </td> <td> 'full-restart' </td> <td> '1' </td> </tr> 
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '--status-all' </td> <td> 'status-all' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = service iscsi status
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;iscsid (pid  8401) is running...
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub service {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
} 

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /unrecognized service/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
