#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/service_iscsi.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux service iscsi commands
##
## @description
##    This is a library to parse the raw output of Linux service iscsi commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "service iscsi start" will be
##    service_iscsi_start
##
## @synopsis
##    use service_iscsi;
##
##service_iscsi(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::service_iscsi;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(.* Command not found.*)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $output =~ /.*(Module iscsi_sfnet is in use).*/si ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::service_iscsi_start
##
## @summary Parses the raw output from the "service_iscsi_start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from service_iscsi_start with any of its options as described in the
##    corresponding cdef file 'service_iscsi.cdefs'
## @synopsis
##      service_iscsi_start(%args);
## INPUTS TO METHOD CALL 'service_iscsi' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /sbin/service iscsi start
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/etc/init.d/iscsi: line 9: /etc/sysconfig/iscsi: Permission denied
##&nbsp;Starting iscsi:                                            [  OK  ]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;		'status' => 'OK'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub service_iscsi_start {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "/sbin/service iscsi start",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }
    my @lines = get_lines( 'raw_output' => $output );
    my %hash = ();
    foreach my $line (@lines) {
        if ( $line =~ /^(\/\S+):\s+(\w+.\d):\s+(\/\S+):\s+(\w+.*)/ ) {
            $hash{$1} = $2;
            $hash{$3} = $4;
        } elsif ( $line =~ /^Starting.*(OK|FAILED).*/ ) {
            $hash{'status'} = $1;
        } elsif ( $line =~ /^(\w+):\s+(\S+):(.*)/ ) {
            $hash{$1} = $2;
            $hash{'file_status'} = $3;
        } elsif ( $line =~ /^(\w+)\s+\((\w+)\s+(\d+)\)\s+\w+\s+(\w+).*/ ) {
            $hash{$2} = $3;
            $hash{'status'} = $4;
        } elsif ( $line
            =~ /(\w+\s+\w+)\s+network shutdown[.]\s+\w+\s+\w+\s+\w+:.*(OK|FAILED).*/i
            ) {
            $hash{'network_shutdown'}      = $1;
            $hash{'starting_iscsi_daemon'} = $2;
        } elsif ( $line =~ /^(\/\S+):\s+(\w+.\d+):\s+(\S+):\s+(\w+\s+\w+)/ ) {
            $hash{'file_name'}   = $1;
            $hash{'file_status'} = $4;
            $hash{'pid_file'}    = $2;
        } elsif ( $line =~ /iscsid\s+(\w+).*file\s+(\w+)/i ) {
            $hash{'iscsid'}   = $1;
            $hash{'pid_file'} = $2;
        }
    }
    push( @result_array, \%hash );
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@result_array );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub service_iscsi_start

## @name ::service_iscsi_stop
##
## @summary Parses the raw output from the "service_iscsi_stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from service_iscsi_stop with any of its options as described in
##    the corresponding cdef file 'service_iscsi.cdefs'
## @synopsis
##      service_iscsi_stop(%args);
## INPUTS TO METHOD CALL 'service_iscsi' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /sbin/service iscsi stop
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'status' => 'ok',
##&nbsp;             'iscsi_daemon' => 'stopping'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;

sub service_iscsi_stop {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "/sbin/service iscsi stop",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }
    my @lines = get_lines( 'raw_output' => $output );
    my %hash = ();
    foreach my $line (@lines) {
        if ($line =~ /stopping iscsi daemon:(\s+\[?\s+(OK|FAILED)\s+\]?)?/si )
        {
            $hash{'iscsi_daemon'} = 'stopping';
            $hash{'status'}       = 'ok';
        } elsif (
            $line =~ /stopping iscsi daemon:\s+iscsiadm:\s+(.*)\s+\(\d+\)!/i )
        {
            $hash{'iscsi_daemon'} = 'stopping';
            $hash{'status'}       = $1;
        } elsif ( $line =~ /iscsiadm:\s+(.*)[.]\s+(.*)/ ) {
            $hash{'iscsiadm'} = $1;
            $hash{'status'}   = $2;
        } elsif ( $line =~ /Searching for iscsi-based multipath maps/i ) {
            $hash{'iscsi_based_multipath_maps'} = 'searching';
        } elsif ( $line =~ /Found\s+(\d+)\s+maps/i ) {
            $hash{'maps_found'} = $1;
        } elsif ( $line =~ /stopping iscsid:.*((OK|FAILED)).*/i ) {
            $hash{'stopping_iscsid'} = $1;
        }
    }
    push( @result_array, \%hash );
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@result_array );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub service_iscsi_stop

## @name ::service_iscsi_restart
##
## @summary Parses the raw output from the "service_iscsi_restart" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from service_iscsi_restart with any of its options as described
##    in the corresponding cdef file 'service_iscsi.cdefs'
## @synopsis
##      service_iscsi_restart(%args);
## INPUTS TO METHOD CALL 'service_iscsi' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /sbin/service iscsi restart
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'network_shutdown' => 'Turning off',
##&nbsp;            'iscsiadm' => 'Could not stop iscsid',
##&nbsp;            'starting_iscsi_daemon' => 'm',
##&nbsp;            'status' => 'Trying sending iscsid SIGTERM or SIGKILL signals manually',
##&nbsp;            'iscsi_daemon' => 'stopping'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub service_iscsi_restart {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "/sbin/service iscsi restart",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }
    my @lines = get_lines( 'raw_output' => $output );
    my %hash = ();
    foreach my $line (@lines) {
        if ( $line =~ /stopping iscsi daemon:\s+\[?\s+(\w+)\s+\]?/i ) {
            $hash{'iscsi_daemon'} = 'stopping';
            $hash{'status'}       = 'ok';
        } elsif (
            $line =~ /stopping iscsi daemon:\s+iscsiadm:\s+(.*)\s+\(\d+\)!/i )
        {
            $hash{'iscsi_daemon'} = 'stopping';
            $hash{'status'}       = $1;
        } elsif ( $line =~ /iscsiadm:\s+(.*)[.]\s+(.*)/ ) {
            $hash{'iscsiadm'} = $1;
            $hash{'status'}   = $2;
        } elsif ( $line =~ /Searching for iscsi-based multipath maps/i ) {
            $hash{'iscsi_based_multipath_maps'} = 'searching';
        } elsif ( $line =~ /Found\s+(\d+)\s+maps/i ) {
            $hash{'maps_found'} = $1;
        } elsif ( $line =~ /stopping iscsid:.*((OK|FAILED)).*/i ) {
            $hash{'stopping_iscsid'} = $1;
        } elsif ( $line =~ /iscsid\s+(\w+).*file\s+(\w+).*\[?\s+\w+\s+\]?/i )
        {
            $hash{'iscsid'}   = $1;
            $hash{'pid_file'} = $2;
        } elsif ( $line
            =~ /(\w+\s+\w+)\s+network shutdown[.]\s+\w+\s+\w+\s+\w+:.*(OK|FAILED).*/i
            ) {
            $hash{'network_shutdown'}      = $1;
            $hash{'starting_iscsi_daemon'} = $2;
        } elsif ( $line =~ /Stopping iscsid: (iscsid not running)/i ) {
            $hash{'stopping_iscsid'} = $1;
        }
    }
    push( @result_array, \%hash );
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@result_array );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub service_iscsi_restart
1;
