#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux setacl command
##
## @description
##    This is a library to parse the raw output of Linux setacl command.
##
## @synopsis
##    use setacl;
##
##    setacl(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::setacl;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::setacl
##
## @summary Parses the raw output from the "/usr/software/test/bin/v4t/v4t_setacl" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from setacl with any of its options as described in the 
##    corresponding cdef file 'setacl.cdefs'
## @synopsis
##      setacl(%args);
## INPUTS TO METHOD CALL 'setacl' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-F' </td> <td> 'file-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-C' </td> <td> 'volume-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'acl-input-file-path' </td> <td> '0' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'ip-address' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/software/test/bin/v4t/v4t_setacl -I 10.227.67.75 -C vol99 -F /data_vol99/file1 -f /u/akhand/audit1.acl
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Step 1.0: 	set attr -- went OK
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = 'Step 1.0: set attr -- went OK';
##&nbsp;
sub setacl{
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array);
    $result_array[0]->{'result'} = $output;
    return \@result_array;
} ## end sub setacl

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /.*Permission\s+denied/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No\s+such\s+file\s+or\s+directory.*|Too\s+many\s+arguments.*|set\s+attr\s+--failed\s+(unexpectedly).*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
