#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse stat command
##
## @description
##    This is a library to parse the raw output of stat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::stat;
##
##    stat(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::stat;

use strict;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Include exceptions which will be thrown
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

sub _check_error {
    my %args          = @_;
    my $resp          = $args{response_object};
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = undef;
    my $error         = undef;
    if ( $raw_output =~
/stat:(\s*cannot\s+stat|\s*Could\s+not\s+find|\s*No\s+file\s+specified|\s*No\s+such\s+file\s+or\s+directory|\s*cannot\s+read\s+file\s+system)/
      )
    {
        $error = "CommandFailed";
    }
    elsif ( $raw_output =~ /stat:(\s*illegal\s+option|\s*invalid\s+option)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $raw_output =~ /\s*Unknown\s+option:/ ) {
        $error = "InvalidParam";
    }elsif($raw_output =~ /stat:(\s*missing\s+operand|)/){
	    $error = "MissingRequiredParam";
	}
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
}

## @name ::stat
##
## @summary Parses the raw output from the "stat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from stat with any of its options as described in the 
##    corresponding cdef file 'stat.cdefs'
## @synopsis
##      stat(%args);
## INPUTS TO METHOD CALL 'stat' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'terse' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'dereference' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '--file-system' </td> <td> 'filesystem' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'context' </td> <td> '1' </td> </tr>
## <tr> <td> '--printf' </td> <td> 'printf' </td> <td> '0' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = stat tempfile tempdir
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  File: `tempfile'
##&nbsp;  Size: 0         	Blocks: 0          IO Block: 4096   regular empty file
##&nbsp;Device: 1dh/29d	Inode: 1253676976  Links: 1
##&nbsp;Access: (0644/-rw-r--r--)  Uid: (45172/ someshk)   Gid: (   30/  gopher)
##&nbsp;Access: 2014-07-07 03:57:24.847513000 -0400
##&nbsp;Modify: 2014-07-07 03:57:24.847513000 -0400
##&nbsp;Change: 2014-07-07 03:57:24.847513000 -0400
##&nbsp;  File: `tempdir'
##&nbsp;  Size: 4096      	Blocks: 8          IO Block: 4096   directory
##&nbsp;Device: 1dh/29d	Inode: 1253676973  Links: 2
##&nbsp;Access: (0755/drwxr-xr-x)  Uid: (45172/ someshk)   Gid: (   30/  gopher)
##&nbsp;Access: 2014-07-02 01:21:09.076502000 -0400
##&nbsp;Modify: 2014-07-02 01:21:09.076502000 -0400
##&nbsp;Change: 2014-07-02 01:21:09.076502000 -0400
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tempdir' => {
##&nbsp;                           'access_permissions' => '-0400',
##&nbsp;                           'blocks' => '8',
##&nbsp;                           'modify_permissions' => '-0400',
##&nbsp;                           'access_date' => '2014-07-02',
##&nbsp;                           'size' => '4096',
##&nbsp;                           'uid' => '45172/someshk',
##&nbsp;                           'access' => '0755/drwxr-xr-x',
##&nbsp;                           'access_time' => '01:21:09.076502000',
##&nbsp;                           'name' => 'tempdir',
##&nbsp;                           'io_block' => '4096',
##&nbsp;                           'change_date' => '2014-07-02',
##&nbsp;                           'device' => '1dh/29d',
##&nbsp;                           'inode' => '1253676973',
##&nbsp;                           'modify_time' => '01:21:09.076502000',
##&nbsp;                           'type' => 'directory',
##&nbsp;                           'change_time' => '01:21:09.076502000',
##&nbsp;                           'change_permissions' => '-0400',
##&nbsp;                           'gid' => '30/gopher',
##&nbsp;                           'modify_date' => '2014-07-02'
##&nbsp;                         },
##&nbsp;            'tempfile' => {
##&nbsp;                            'access_permissions' => '-0400',
##&nbsp;                            'blocks' => '0',
##&nbsp;                            'modify_permissions' => '-0400',
##&nbsp;                            'access_date' => '2014-07-07',
##&nbsp;                            'size' => '0',
##&nbsp;                            'uid' => '45172/someshk',
##&nbsp;                            'access' => '0644/-rw-r--r--',
##&nbsp;                            'access_time' => '03:57:24.847513000',
##&nbsp;                            'name' => 'tempfile',
##&nbsp;                            'io_block' => '4096',
##&nbsp;                            'change_date' => '2014-07-07',
##&nbsp;                            'device' => '1dh/29d',
##&nbsp;                            'inode' => '1253676976',
##&nbsp;                            'modify_time' => '03:57:24.847513000',
##&nbsp;                            'type' => 'regular_empty_file',
##&nbsp;                            'change_time' => '03:57:24.847513000',
##&nbsp;                            'change_permissions' => '-0400',
##&nbsp;                            'gid' => '30/gopher',
##&nbsp;                            'modify_date' => '2014-07-07'
##&nbsp;                          }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stat {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    _check_error(
        "response_object" => $resp,
        "raw_output"      => $output,
        "command"         => $command,
    );
    my @lines = get_lines( raw_output => $output );
    my (
        $line,      $new_line,     %hash,     $key, $value,
        @split_arr, @result_array, $main_key, $value2
    );

    while (@lines) {
        $line = shift(@lines);
        $line =~ s/:\s*/:/g;
        $line =~ s/\/\s+/\//g;
        if (   $line =~ /(.*)[:]+(.*)/
            && $line !~ /\d+[:]\d+[:]\d+/i
            && $line !~ /^(Inodes:|Blocks:)/ )
        {
            $new_line = $1;
            $value    = $2;
            $new_line =~ s/\(//g;
            $new_line =~ s/\)//g;
            $new_line =~ s/:\s*/:/g;
            $value    =~ s/\(//g;
            $new_line =~ s/\s{2,}/&/g;
            @split_arr = split( /[:|&|\t]+/, $new_line );

            if ( $split_arr[0] =~ /[n|N]ame|[f|F]ile/i ) {
                $main_key = $split_arr[1];
                $main_key =~ s/"//g;
				$main_key =~ s/'//g;
				$main_key =~ s/`//g;
				$main_key =~ s/\\//g;
            }
			my $i = 0;
			if($#split_arr){
				for ( $i = 0 ; $i < $#split_arr ; $i = $i + 2 ) {
					$hash{$main_key}{ to_key_name( item => $split_arr[$i] ) } =
					  $split_arr[ $i + 1 ];
				}
				if ( $value =~ /(\d+[\/]+(\w*-*\w+-*\w*)*\s*.*)/ ){
					$value  = $1;
					$value =~ s/\)//g;
					$hash{$main_key}{ to_key_name( item => $split_arr[$i] ) } =
					 $value;
				}elsif($value =~ /(\d+)\s+(.*)/){
					$value  = $1;
					$value2 = $2;
					$value =~ s/\)//g;
					$value2 =~ s/\s+/_/g;
					$hash{$main_key}{'type'} = $value2  if ($value2);
					$hash{$main_key}{ to_key_name( item => $split_arr[$i] ) } =
					  $value;
				}
			}elsif ( !$#split_arr ) {
                if ( $main_key && $value ) {
                    $hash{$main_key}{ to_key_name( item => $split_arr[$i] ) } =
                      $value;
                }
                elsif ($value) {
                    $main_key = $value;
                    $main_key =~ s/"//g;
					$main_key =~ s/'//g;
					$main_key =~ s/`//g;
					$main_key =~ s/\\//g;
					$hash{$main_key}{'name'} = $main_key;
                }
            }
        }
        elsif ( $line =~
            /(.*)[:]+(\d+[-]\d+[-]\d+)\s+(\d+[:]\d+[:]\d+[.\d+]*)\s+(.*)/ )
        {
            my $value_type = $1;
            my $date       = $2;
            my $time       = $3;
            my $permission = $4;
            $hash{$main_key}{ to_key_name( item => $value_type . '_date' ) } =
              $2;
            $hash{$main_key}{ to_key_name( item => $value_type . '_time' ) } =
              $3;
            $hash{$main_key}
              { to_key_name( item => $value_type . '_permissions' ) } = $4;
        }
        elsif ( $line =~
/(access|modify|change).*[:?]+(\w+\s+\w+\s+\d+)\s+(\d+[:]\d+[:]\d+[.\d+]*)\s+(\d+)\s+(.*)/i
          )
        {
            my $value_type = $1;
            my $date       = $2 . "_" . $4;
            my $time       = $3;
            $new_line = $5;
            $date =~ s/\s+/_/g;
            $hash{$main_key}{ to_key_name( item => $value_type . '_date' ) } =
              $date;
            $hash{$main_key}{ to_key_name( item => $value_type . '_time' ) } =
              $time;
            @split_arr = split( /[:]/, $new_line );
            my $j = 0;

            for ( $j = 0 ; $j < $#split_arr ; $j = $j + 2 ) {
                $split_arr[$j]       =~ s/\//_/g;
                $split_arr[$j]       =~ s/\s+/_/g;
                $hash{$main_key}{ to_key_name( item => $split_arr[$j] ) } =
                  $split_arr[ $j + 1 ];
            }
        }elsif ( $line =~ /^(Inodes|Blocks)\s*[:]\s*(.*)/i ) {
            my $main_key2 = $1;
            $new_line = $2;
            $new_line =~ s/\s{2,}/&/g;
            @split_arr = split( /[:|&]+/, $new_line );
            my $j = 0;
            for ( $j = 0 ; $j < $#split_arr ; $j = $j + 2 ) {
                $split_arr[$j]       =~ s/\//_/g;
                $split_arr[$j]       =~ s/\s+/_/g;
                $split_arr[ $j + 1 ] =~ s/\//_/g;
                $hash{$main_key}{$main_key2}
                  { to_key_name( item => $split_arr[$j] ) } =
                  $split_arr[ $j + 1 ];
            }
        }elsif($line =~ /(.*)/){
		    $hash{'result'} = $1;
		}
    }
    push( @result_array, \%hash );
    return \@result_array;
}
1;
