#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/top.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux top command
##
## @description
##    This is a library to parse the raw output of Linux top command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "top" will be top
##
## @synopsis
##    use top;
##
##    top(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::top;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::top
##
## @summary Parses the raw output from the "top" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from top with any of its options as described in the
##    corresponding cdef file 'top.cdefs'
## @synopsis
##      top(%args);
## INPUTS TO METHOD CALL 'top' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-b' </td> <td> 'batch-mode-operation' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'command-line-toggle' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delay-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'threads-toggle' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'idle-processes-toggle' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'iterations-limit' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'processes-with-effective-uid' </td> <td> '0' </td> </tr>
## <tr> <td> '-U' </td> <td> 'processes-with-uid' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'monitor-pid' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'secure-mode-operation' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'cumulative-mode-toggle' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'detect-memory-units' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = top -b
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1 root      15   0 10348  696  584 S  0.0  0.0   0:01.22 init
##&nbsp;2 root      RT  -5     0    0    0 S  0.0  0.0   0:00.11 migration/0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'priority' => '15',
##&nbsp;             'process_status' => 'S',
##&nbsp;             'resident_size' => '696',
##&nbsp;             'nice_value' => '0',
##&nbsp;             'memory_usage' => '0.0',
##&nbsp;             'virtual_image' => '10348',
##&nbsp;             'program_name' => 'init',
##&nbsp;             'user_name' => 'root',
##&nbsp;             'shared_mem_size' => '584',
##&nbsp;             'process_id' => '1',
##&nbsp;             'cpu_time' => '0:01.22',
##&nbsp;             'cpu_usage' => '0.0'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'priority' => 'RT',
##&nbsp;             'process_status' => 'S',
##&nbsp;             'resident_size' => '0',
##&nbsp;             'nice_value' => '-5',
##&nbsp;             'memory_usage' => '0.0',
##&nbsp;             'virtual_image' => '0',
##&nbsp;             'program_name' => 'migration/0',
##&nbsp;             'user_name' => 'root',
##&nbsp;             'shared_mem_size' => '0',
##&nbsp;             'process_id' => '2',
##&nbsp;             'cpu_time' => '0:00.11',
##&nbsp;             'cpu_usage' => '0.0'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = top -n 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1 root      15   0 10348  696  584 S  0.0  0.0   0:01.22 init
##&nbsp;2 root      RT  -5     0    0    0 S  0.0  0.0   0:00.11 migration/0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'priority' => '15',
##&nbsp;             'process_status' => 'S',
##&nbsp;             'resident_size' => '696',
##&nbsp;             'nice_value' => '0',
##&nbsp;             'memory_usage' => '0.0',
##&nbsp;             'virtual_image' => '10348',
##&nbsp;             'program_name' => 'init',
##&nbsp;             'user_name' => 'root',
##&nbsp;             'shared_mem_size' => '584',
##&nbsp;             'process_id' => '1',
##&nbsp;             'cpu_time' => '0:01.22',
##&nbsp;             'cpu_usage' => '0.0'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'priority' => 'RT',
##&nbsp;             'process_status' => 'S',
##&nbsp;             'resident_size' => '0',
##&nbsp;             'nice_value' => '-5',
##&nbsp;             'memory_usage' => '0.0',
##&nbsp;             'virtual_image' => '0',
##&nbsp;             'program_name' => 'migration/0',
##&nbsp;             'user_name' => 'root',
##&nbsp;             'shared_mem_size' => '0',
##&nbsp;             'process_id' => '2',
##&nbsp;             'cpu_time' => '0:00.11',
##&nbsp;             'cpu_usage' => '0.0'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
sub top {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $c = 0;
    my ( @result_array,, @temp, @var, %res );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @head;
    %res = (
        PID     => 'process_id',
        USER    => 'user_name',
        PR      => 'priority',
        NI      => 'nice_value',
        VIRT    => 'virtual_image',
        RES     => 'resident_size',
        SHR     => 'shared_mem_size',
        S       => 'process_status',
        '%CPU'  => 'cpu_usage_percentage',
        '%MEM'  => 'memory_usage_percentage',
        'TIME+' => 'cpu_time',
        COMMAND => 'program_name'
    );

    for ( my $j = 0; $j <= $#lines; $j++ ) {
        if ( $lines[$j] =~ /.*(PID.*COMMAND)/ ) {
            @temp = split( /\s+/, $1 );
        }
        if ( $lines[$j]
            =~ /.*(\d+\s\w+\s+[A-Z|0-9]+\s+\-*\d+\s+\d+\s+\d+.*\:\d+\d+\.\d+\s+\w+\/*\d*)/
            ) {
            $lines[$j] = $1;
            my @var = split( /\s+/, $lines[$j] );
            for ( my $k = 0; $k <= $#var; $k++ ) {
                $result_array[$c]->{ $res{$temp[$k]} } = $var[$k];
            }
            $c++;
        }
    }
    return \@result_array;
} ## end sub top

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /\s*(top:)\s+.* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

