#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/type.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux type command
##
## @description
##    This is a library to parse the raw output of Linux type command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "type" will be type
##
## @synopsis
##    use type;
##
##    type(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::type;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::type
##
## @summary Parses the raw output from the "type" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from type with any of its options as described in the
##    corresponding cdef file 'type.cdefs'
## @synopsis
##      type(%args);
## INPUTS TO METHOD CALL 'type' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-P' </td> <td> 'path-for-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'diskfile-name-or-nothing' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'suppresses-shell-func' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'single-word-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'displays-all-places' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = type gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;gzip is /bin/gzip
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = type -t gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;file
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'type' => 'file',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = type -a gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;gzip is /bin/gzip
##&nbsp;gzip is /usr/bin/gzip
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'path' => '/usr/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = type -f gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;gzip is /bin/gzip
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = type -p gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/bin/gzip
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = type -P gzip
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/bin/gzip
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/bin/gzip',
##&nbsp;            'command' => 'gzip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub type {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $comm;
    my @var = split( /\s+/, $command );
    if ( $#var == 2 ) {
        $comm = $var[2];
    } else {
        $comm = $var[1];
    }
    _check_error( raw_output => $output, response_object => $resp );
    my (@result_array);
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $c = 0;

    for ( my $i = 0; $i <= $#lines; $i++ ) {

        if ( $#lines == 2 ) {
            if ( $lines[$i] =~ /(\w+)\s+(is)\s+(\/.*)/ ) {
                $result_array[$c]->{path}    = $3;
                $result_array[$c]->{command} = $comm;
                if ( $lines[ $i + 1 ] =~ /(\w+)\s+(is)\s+(\/.*)/ ) {
                    $result_array[ ++$c ]->{path} = $3;
                    $result_array[$c]->{command} = $comm;
                }
            } elsif ( $lines[$i] =~ /^(\w+.*a)\s+\`*(.*)/ ) {
                $result_array[$c]->{command} = $comm;
                $result_array[$c]->{type}    = $2;
                if ( $lines[ $i + 1 ] =~ /(\w+)\s+(is)\s+(\/.*)/ ) {
                    $result_array[$c]->{path} = $3;
                }
            }
        }
        if ( $lines[$i] =~ /(\w+)\s+(is)\s+(\/.*)/ ) {
            $result_array[$c]->{path}    = $3;
            $result_array[$c]->{command} = $comm;
        } elsif ( $lines[$i] =~ /^(\w+.*a)\s+\`*(.*)/ ) {
            $result_array[$c]->{command} = $comm;
            $result_array[$c]->{type}    = $2;
        } elsif ( $lines[$i] =~ /^(\w+)$/i ) {
            $result_array[$c]->{type}    = $1;
            $result_array[$c]->{command} = $comm;
        } elsif ( $lines[$i] =~ /(^\/.*)/ ) {
            $result_array[$c]->{path}    = $1;
            $result_array[$c]->{command} = $comm;
        }
    }
    return \@result_array;

} ## end sub type

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(-bash:)\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(type:)\s+.* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

