#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux umount command
##
## @description
##    This is a library to parse the raw output of Linux umount command.
##
## @synopsis
##    use umount;
##
##    umount(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::umount;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /$command.*:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}

## @name ::umount
##
## @summary Parses the raw output from "umount" command
##
## @description
##    This method can be used for parsing the output from "umount" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/umount.cdefs
##
## @synopsis
##    umount (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing hash reference(s) which contains the 
##  parsed output data from the output of the umount command

sub umount {
    my %args = @_;
    
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output,command=>"umount",
                                 error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        }
    }

    my @lines = get_lines(raw_output=>$output,first_line=>'umount:\s');

    
    foreach my $line (@lines) {
            my %hash;
            if ($line =~ /umount:\s(.*)\s(.*) - remounted read-only/) {
                $hash{'source_path'} = $1;
                $hash{'reason'} = $2;
                $hash{'result'} = 'remounted read-only';
                push (@result_array, \%hash);
            }
    }
    
    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error->throw(text=>$error_message,response_object => $resp);
    }
    
    return \@result_array;
}

1;
