#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux uname command
##
## @description
##    This is a library to parse the raw output of Linux uname  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use uname;
##
##  uname(%args);
##
## @author anupamap@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::uname;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error         = "CommandFailed";
    $$error_message = $raw_output;
    if ( $raw_output =~ /$command.*:\s(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message =~ /([Ii]nvalid option)/ ) {
            $error = "InvalidParam";
        }
        elsif ( $$error_message =~ /([Uu]nrecognized option)/ ) {
            $error = "InvalidParam";
        }
        elsif ( $$error_message =~ /([Ee]xtra option)/ ) {
            $error = "InvalidParam";
        }
    }
    return $error;
} ## end sub _check_error

## @name ::uname
##
## @summary Parses the raw output from the "uname" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from uname with any of its options as described in the 
##    corresponding cdef file 'uname.cdefs'
## @synopsis
##      uname(%args);
## INPUTS TO METHOD CALL 'uname' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'hardware-platform' </td> <td> '1' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'nodename' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'processor' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'kernel-name' </td> <td> '1' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'machine_type' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'operating-system' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'kernel-version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'kernel-release' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = uname
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Linux
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_name' => 'Linux'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = uname -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Linux lnx219-44 2.6.18-164.el5xen #1 SMP Tue Aug 18 15:59:52 EDT 2009 x86_64 x86_64 x86_64 GNU/Linux
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_version' => 'SMP',
##&nbsp;            'nodename' => 'lnx219-44',
##&nbsp;            'processor' => 'x86_64',
##&nbsp;            'kernel_release' => '2.6.18-164.el5xen',
##&nbsp;            'operating_system' => 'GNU/Linux',
##&nbsp;            'hardware_platform' => 'x86_64',
##&nbsp;            'machine_type' => 'x86_64',
##&nbsp;            'kernel_name' => 'Linux'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub uname {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            command       => "uname",
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    }
    my @lines = get_lines( raw_output => $output, first_line => "" );
    foreach my $line (@lines) {
        my %hash;
        if ( $command eq 'uname' ) {
            if ( $line =~ /(^Linux).*/ ) {
                $hash{'kernel_name'} = $1;
                push( @result_array, \%hash );
            }
        }
        elsif ( $command eq 'uname -a' ) {
            if ( $line =~
                 /(^Linux)\s(\S+)\s(\S+)\s\S+\s(\S+)\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s(\S+)\s(\S+)\s(\S+)\s(\S+)/
              )
            {
                $hash{'kernel_name'}       = $1;
                $hash{'nodename'}          = $2;
                $hash{'kernel_release'}    = $3;
                $hash{'kernel_version'}    = $4;
                $hash{'machine_type'}      = $5;
                $hash{'processor'}         = $6;
                $hash{'hardware_platform'} = $7;
                $hash{'operating_system'}  = $8;
                push( @result_array, \%hash );
            }

        }
        elsif ($command eq 'uname -s'
            || $command eq 'uname -n'
            || $command eq 'uname -r'
            || $command eq 'uname -v'
            || $command eq 'uname -m'
            || $command eq 'uname -p'
            || $command eq 'uname -i'
            || $command eq 'uname -o' )
        {
            if ( $line =~ /(^Linux)/ ) {
                $hash{'kernel_name'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'nodename'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'kernel_release'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /\S+\s(\S+)\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+/ ) {
                $hash{'kernel_version'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'machine_type'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'processor'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'hardware_platform'} = $1;
                push( @result_array, \%hash );
            }
            elsif ( $line =~ /(\S+)/ ) {
                $hash{'operating_system'} = $1;
                push( @result_array, \%hash );
            }
        }
        else {
            return [];
        }
    }
    return \@result_array;
} ## end sub uname

1;
