#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux vconfig command output
##
## @description
##    This is a library to parse the raw output of Linux vconfig command.
##
## @synopsis
##    use vconfig;
##
##    vconfig_add(%args);
##    vconfig_rem(%args);
##    vconfig_set_egress_map(%args);
##    vconfig_set_flag(%args);
##    vconfig_set_ingress_map(%args);
##    vconfig_set_name_type(%args);
##
## @author ksmahesh@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::vconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = $raw_output;

    if ($raw_output =~ /ERROR|Usage:\s.*/) {
        if($raw_output =~ /Missing/) {
            $error = "MissingRequiredParam";
        } elsif($raw_output =~ /[Ii]nvalid|[Uu]nrecognized\s+option/) {
            $error = "InvalidParam";
        } elsif($raw_output =~ /argument\s+to|unknown/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}

## @name ::vconfig_add
##
## @summary Parses the raw output from "vconfig add" command
##
## @description
##    This method can be used for parsing the output from "vconfig add" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_add (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig add command

sub vconfig_add {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Added VLAN', 
                          response_object=>$resp);

    my $line= shift (@lines);
    if ($line =~ /Added VLAN with VID == \d+ to IF -:\w+:-/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

## @name ::vconfig_rem
##
## @summary Parses the raw output from "vconfig rem" command
##
## @description
##    This method can be used for parsing the output from "vconfig rem" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_rem (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig rem command

sub vconfig_rem {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Removed VLAN', 
                          response_object=>$resp);

    my $line= shift (@lines);
    if ($line =~ /Removed VLAN -:.*:-/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

## @name ::vconfig_set_name_type
##
## @summary Parses the raw output from "vconfig set_name_type" command
##
## @description
##    This method can be used for parsing the output from "vconfig set_name_type" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_set_name_type (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig set_name_type command

sub vconfig_set_name_type {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Set name-type', 
                          response_object=>$resp);

    my $line= shift (@lines);
    if ($line =~ /Set name-type for VLAN subsystem. Should be visible in \/proc\/net\/vlan\/config/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

## @name ::vconfig_set_flag
##
## @summary Parses the raw output from "vconfig set_flag" command
##
## @description
##    This method can be used for parsing the output from "vconfig set_flag" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_set_flag(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig set_flag command

sub vconfig_set_flag {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Set flag', 
                          response_object=>$resp);

    my $line= shift (@lines);
    if ($line =~ /Set flag on device -:.*:- Should be visible in \/proc\/net\/vlan\/.*/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

## @name ::vconfig_set_egress_map
##
## @summary Parses the raw output from "vconfig set_egress_map" command
##
## @description
##    This method can be used for parsing the output from "vconfig set_egress_map" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_set_egress_map(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig set_egress_map command

sub vconfig_set_egress_map {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Set egress', 
                          response_object=>$resp);
                          
    my $line= shift (@lines);
    if ($line =~ /Set egress mapping on device -:.*:- Should be visible in \/proc\/net\/vlan\/.*/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

## @name ::vconfig_set_ingress_map
##
## @summary Parses the raw output from "vconfig set_ingress_map" command
##
## @description
##    This method can be used for parsing the output from "vconfig set_ingress_map" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/vconfig.cdefs
##
## @synopsis
##    vconfig_set_ingress_map(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object 
##                    with the parsed output incase of partial success in the 
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the 
##  parsed output data from the output of vconfig set_ingress_map command

sub vconfig_set_ingress_map {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output, 
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }
     my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Set ingress', 
                          response_object=>$resp);
                          
    my $line= shift (@lines);
    if ($line =~ /Set ingress mapping on device -:.*:- Should be visible in \/proc\/net\/vlan\/.*/) {
        my @result_array;
        my %temp_hash = ("result","successful");
        push (@result_array,\%temp_hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Unexpected Output while parsing", 
                        response_object => $resp);
    }
}

1;
