#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux vgexport command
##
## @description
##    This is a library to parse the raw output of Linux vgexport command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vgexport" will be vgexport()
##
## @synopsis
##    use vgexport;
##
##    vgexport(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::vgexport;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingValueException;
use NACL::APISet::Exceptions::NoMatchingEntriesException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::vgexport
##
## @summary Parses the raw output from the "vgexport" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vgexport with any of its options as described in the
##    corresponding cdef file 'vgexport.cdefs'
## @synopsis
##      vgexport(%args);
## INPUTS TO METHOD CALL 'vgexport' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vg-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub vgexport {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub vgexport

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /Please\s+supply\s+volume\s+groups/i ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Volume\s+group.*has\s+active\s+logical\s+volumes/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No\s+volume\s+groups\s+found/i ) {
        $error         = "NoMatchingEntries";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/Invalid\s+volume\s+group|Volume\s+group.*not\s+found|invalid\s+characters/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /invalid\s+option/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

