#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux vgreduce command
##
## @description
##    This is a library to parse the raw output of Linux vgreduce command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vgreduce" will be vgreduce()
##
## @synopsis
##    use vgreduce;
##
##    vgreduce(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::vgreduce;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::vgreduce
##
## @summary Parses the raw output from the "vgreduce" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vgreduce with any of its options as described in the
##    corresponding cdef file 'vgreduce.cdefs'
## @synopsis
##      vgreduce(%args);
## INPUTS TO METHOD CALL 'vgreduce' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '--removemissing' </td> <td> 'removemissing' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vg-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pv-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub vgreduce {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub vgreduce

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /Please\s+give\s+volume\s+group\s+name|Please\s+enter\s+physical\s+volume\s+paths/i ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Can't\s+remove\s+final\s+physical\s+volume|Physical\s+volume.*still\s+in\s+use/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid\s+volume\s+group|Volume\s+group.*not\s+found/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /invalid\s+option/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

