#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/vmstat.pm#1 $
# Copyright (c) 2012 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse Linux vmstat command
##
## @description
##    This is a library to parse the raw output of Linux vmstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vmstat" will be vmstat()
##
## @synopsis
##    use NACL::APISet::Host::CLI::HPUX::default::vmstat;
##
##    vmstat(%args);
##
## @author sheethal@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
package NACL::APISet::Host::CLI::Linux::Redhat::vmstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

## @name ::vmstat
##
## @summary Parses the raw output from the "vmstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vmstat with any of its options as described in the
##    corresponding cdef file 'vmstat.cdefs'
## @synopsis
##      vmstat(%args);
## INPUTS TO METHOD CALL 'vmstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'forks_count' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'slabinfo' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'memory_info' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'disk_table' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disk_statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'partition_stats' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'active_stat' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vmstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;procs -----------memory---------- ---swap-- -----io---- --system-- -----cpu-----
##&nbsp; r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa st
##&nbsp; 0  0      0 2829080 289304 3054120    0    0     0    39    2    1  0  0 97  3  0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'swap_out' => '0',
##&nbsp;            'cpu_idle' => '97',
##&nbsp;            'memory_swpd' => '0',
##&nbsp;            'system_interrupts' => '2',
##&nbsp;            'procs_wait_runtime' => '0',
##&nbsp;            'blocks_recieved' => '0',
##&nbsp;            'system_context_switches' => '1',
##&nbsp;            'cpu_stolen_time' => '0',
##&nbsp;            'procs_uninterruptible_sleep' => '0',
##&nbsp;            'memory_cache' => '3054120',
##&nbsp;            'cpu_waiting' => '3',
##&nbsp;            'memory_buff' => '289304',
##&nbsp;            'swap_in' => '0',
##&nbsp;            'blocks_sent' => '39',
##&nbsp;            'cpu_user_time' => '0',
##&nbsp;            'memory_free' => '2829080',
##&nbsp;            'cpu_sys_time' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -V
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;procps version 3.2.8
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'procps version 3.2.8'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;procs -----------memory---------- ---swap-- -----io---- --system-- -----cpu-----
##&nbsp; r  b   swpd   free  inact active   si   so    bi    bo   in   cs us sy id wa st
##&nbsp; 0  0      0 2829080 1707152 3117068    0    0     0    39    2    1  0  0 97  3  0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'swap_out' => '0',
##&nbsp;            'cpu_idle' => '97',
##&nbsp;            'memory_swpd' => '0',
##&nbsp;            'system_interrupts' => '2',
##&nbsp;            'procs_wait_runtime' => '0',
##&nbsp;            'blocks_recieved' => '0',
##&nbsp;            'system_context_switches' => '1',
##&nbsp;            'cpu_stolen_time' => '0',
##&nbsp;            'procs_uninterruptible_sleep' => '0',
##&nbsp;            'cpu_waiting' => '3',
##&nbsp;            'swap_in' => '0',
##&nbsp;            'memory_active' => '3117068',
##&nbsp;            'blocks_sent' => '39',
##&nbsp;            'cpu_user_time' => '0',
##&nbsp;            'memory_free' => '2829080',
##&nbsp;            'cpu_sys_time' => '0',
##&nbsp;            'memory_inact' => '1707152'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;disk- ------------reads------------ ------------writes----------- -----IO------
##&nbsp;       total merged sectors      ms  total merged sectors      ms    cur    sec
##&nbsp;ram0       0      0       0       0      0      0       0       0      0      0
##&nbsp;ram1       0      0       0       0      0      0       0       0      0      0
##&nbsp;ram2       0      0       0       0      0      0       0       0      0      0
##&nbsp;ram3       0      0       0       0      0      0       0       0      0      0
##&nbsp
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'ram0',
##&nbsp;            'merged_writes' => '0',
##&nbsp;            'total_writes' => '0',
##&nbsp;            'sectors_reads' => '0',
##&nbsp;            'total_reads' => '0',
##&nbsp;            'ms_writes' => '0',
##&nbsp;            'merged_reads' => '0',
##&nbsp;            'sectors_writes' => '0',
##&nbsp;            'cur_IO' => '0',
##&nbsp;            'IO_duration' => '0',
##&nbsp;            'ms_reads' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'ram1',
##&nbsp;            'merged_writes' => '0',
##&nbsp;            'total_writes' => '0',
##&nbsp;            'sectors_reads' => '0',
##&nbsp;            'total_reads' => '0',
##&nbsp;            'ms_writes' => '0',
##&nbsp;            'merged_reads' => '0',
##&nbsp;            'sectors_writes' => '0',
##&nbsp;            'cur_IO' => '0',
##&nbsp;            'IO_duration' => '0',
##&nbsp;            'ms_reads' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'ram2',
##&nbsp;            'merged_writes' => '0',
##&nbsp;            'total_writes' => '0',
##&nbsp;            'sectors_reads' => '0',
##&nbsp;            'total_reads' => '0',
##&nbsp;            'ms_writes' => '0',
##&nbsp;            'merged_reads' => '0',
##&nbsp;            'sectors_writes' => '0',
##&nbsp;            'cur_IO' => '0',
##&nbsp;            'IO_duration' => '0',
##&nbsp;            'ms_reads' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'ram3',
##&nbsp;            'merged_writes' => '0',
##&nbsp;            'total_writes' => '0',
##&nbsp;            'sectors_reads' => '0',
##&nbsp;            'total_reads' => '0',
##&nbsp;            'ms_writes' => '0',
##&nbsp;            'merged_reads' => '0',
##&nbsp;            'sectors_writes' => '0',
##&nbsp;            'cur_IO' => '0',
##&nbsp;            'IO_duration' => '0',
##&nbsp;            'ms_reads' => '0'
##&nbsp;          },
##&nbsp;
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -D
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;           26 disks
##&nbsp;            3 partitions
##&nbsp;        74830 total reads
##&nbsp;         2481 merged reads
##&nbsp;      2416918 read sectors
##&nbsp;       332899 milli reading
##&nbsp;     68178451 writes
##&nbsp;    137567697 merged writes
##&nbsp;   1505196446 written sectors
##&nbsp;   1142200612 milli writing
##&nbsp;            0 inprogress IO
##&nbsp;       426152 milli spent IO
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'milli_writing' => '1142200612',
##&nbsp;            'milli_spent_io' => '426152',
##&nbsp;            'inprogress_io' => '0',
##&nbsp;            'merged_writes' => '137567697',
##&nbsp;            'partitions' => '3',
##&nbsp;            'total_reads' => '74830',
##&nbsp;            'merged_reads' => '2481',
##&nbsp;            'disks' => '26',
##&nbsp;            'read_sectors' => '2416918',
##&nbsp;            'milli_reading' => '332899',
##&nbsp;            'writes' => '68178451',
##&nbsp;            'written_sectors' => '1505196446'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -p sda1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;sda1          reads   read sectors  writes    requested writes
##&nbsp;                 728      12764         71       8718
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'partition_writes' => '71',
##&nbsp;            'partition_read_sectors' => '12764',
##&nbsp;            'partition_reads' => '728',
##&nbsp;            'partition_requested_writes' => '8718'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;      8058716  total memory
##&nbsp;      5229380  used memory
##&nbsp;      3116912  active memory
##&nbsp;      1707152  inactive memory
##&nbsp;      2829336  free memory
##&nbsp;       289304  buffer memory
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'active_memory' => '3116912',
##&nbsp;            'inactive_memory' => '1707152',
##&nbsp;            'used_memory' => '5229380',
##&nbsp;            'buffer_memory' => '289304',
##&nbsp;            'total_memory' => '8058716',
##&nbsp;            'free_memory' => '2829336',
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Cache                       Num  Total   Size  Pages
##&nbsp;pid_2                         0      0    128     30
##&nbsp;nf_conntrack_expect           0      0    240     16
##&nbsp;nf_conntrack_ffffffff82007d40     47     91    304     13
##&nbsp;bridge_fdb_cache              1     59     64     59
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;             {
##&nbsp;                         'cache'=>'pid_2',
##&nbsp;                         'num' => '2',
##&nbsp;                         'total' => '0',
##&nbsp;                         'pages' => '128',
##&nbsp;              },
##&nbsp;             {
##&nbsp;                        'cache'=>'nf_conntrack_expect',
##&nbsp;                        'num' => 't',
##&nbsp;                        'total' => '0',
##&nbsp;                        'pages' => '240',
##&nbsp;                        'size' => '0'
##&nbsp;              },
##&nbsp;            {
##&nbsp;              'cache'=>'nf_conntrack_ffffffff82007d40',
##&nbsp;              'num' => '0',
##&nbsp;              'total' => '47',
##&nbsp;              'pages' => '304',
##&nbsp;              'size' => '91'
##&nbsp;             },
##&nbsp;            {
##&nbsp;              'cache'=>'bridge_fdb_cache',
##&nbsp;              'num' => 'e',
##&nbsp;              'total' => '1',
##&nbsp;              'pages' => '64',
##&nbsp;              'size' => '59'
##&nbsp;            }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vmstat -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;      1147007 forks
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fork_count' => '1147007'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vmstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    my $str;
    my @head;
    my @proc_info;
    _check_error( raw_output => $output, response_object => $resp );
    my (@result_array);
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;

    if ( $cmd =~ /^vmstat -D.*/ ) {
        @lines = grep ( !(/^\s*disk-\s+\S+/), @lines );
        while (@lines) {
            $str = shift(@lines);
            @proc_info = split( /\s+/, $str );
            $result_array[$count]->{ to_key_name( item => $proc_info[1] ) } =
              $proc_info[0];

        }
    }
    elsif ( $cmd =~ /^vmstat -s.*/ ) {
        while (@lines) {
            $str = shift(@lines);
            @proc_info = split( /\s+/, $str );
            if ( $str =~ /(^\d+)\s+([k m])\s+((\w|\s)*)/i ) {

                $result_array[$count]
                  ->{ to_key_name( item => $proc_info[2] ) } =
                  $proc_info[0] . $proc_info[1];
            }
            elsif ( $str =~ /^(\d+)\s+((\w|\s)*)/ ) {
                $result_array[$count]
                  ->{ to_key_name( item => $proc_info[1] ) } = $proc_info[0];
            }

        }
    }

    elsif ( $cmd =~ /-m.*/ ) {
        @lines = grep ( !(/^\s*cache\s+\S+/), @lines );

        while (@lines) {
            $str                           = shift(@lines);
            @proc_info                     = split( /\s+/, $str );
            $result_array[$count]->{cache} = $proc_info[0];
            $result_array[$count]->{num}   = $proc_info[1];
            $result_array[$count]->{total} = $proc_info[2];
            $result_array[$count]->{size}  = $proc_info[3];
            $result_array[$count]->{pages} = $proc_info[4];
            $count++;
        }

    }

    elsif ( $cmd =~ /^vmstat -V.*/ ) {
        foreach my $line (@lines) {

            if ( $line =~ /^procps version.*/ ) {
                push( @result_array, { version => $line } );

            }
        }
    }
    elsif ( $cmd =~ /^vmstat -f.*/ ) {
        my $count1 = 0;
        foreach my $line (@lines) {

            if ( $line =~ /forks/ ) {
                my @head = split( /\s+/, $line );
                for ( my $i = 0 ; $i <= $#head ; $i++ ) {

                    if ( $head[$i] =~ /\d+/ ) {

                        $count1 = $head[$i];
                        last;
                    }

                }

                $result_array[$count]->{fork_count} = $count1;

            }
        }
    }

    elsif ( $cmd =~ /^vmstat -p.*/ ) {

        while (@lines) {
            $str = shift(@lines);
            if ( $str =~ /^(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ ) {

                $result_array[$count]->{partition_reads}            = $1;
                $result_array[$count]->{partition_read_sectors}     = $2;
                $result_array[$count]->{partition_writes}           = $3;
                $result_array[$count]->{partition_requested_writes} = $4;

            }
        }
    }

    else {

        @lines = grep ( !(/^\s*procs\s+\S+/), @lines );
        @lines = grep ( !(/^\s*r\s+\S+/),     @lines );
        if ( $cmd =~ /^vmstat -a.*/ ) {
            @head =
              qw(procs_wait_runtime procs_uninterruptible_sleep memory_swpd memory_free memory_inact memory_active swap_in swap_out blocks_recieved blocks_sent system_interrupts
              system_context_switches cpu_user_time cpu_sys_time cpu_idle cpu_waiting cpu_stolen_time);
        }
        elsif ( $cmd =~ /^vmstat -d.*/ ) {
            @lines = grep ( !(/^\s*disk-\s+\S+/), @lines );
            @lines = grep ( !(/^\s*total\s+\S+/), @lines );
            @head =
              qw(disk total_reads merged_reads sectors_reads ms_reads total_writes merged_writes sectors_writes ms_writes cur_io io_duration);
        }
        else {
            @head =
              qw(procs_wait_runtime procs_uninterruptible_sleep memory_swpd memory_free memory_buff memory_cache swap_in swap_out blocks_recieved blocks_sent system_interrupts
              system_context_switches cpu_user_time cpu_sys_time cpu_idle cpu_waiting cpu_stolen_time);
        }
        while (@lines) {
            $str = shift(@lines);

            @proc_info = split( /\s+/, $str );
            for ( my $i = 0 ; $i < scalar(@proc_info) ; $i++ ) {
                $result_array[$count]->{ $head[$i] } = $proc_info[$i];
            }

            $count++;

        }
    }
    return \@result_array;
} ## end sub netstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /vmstat:\s+no\s+support.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /([Uu]sage:|vmstat).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error
1;
