# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/PowerShell.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for PowerShell CLI commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::CLI::PowerShell;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::CLI);

#
# Module imports - APISet
#
use NACL::APISet::Session::CLI::PowerShell;
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::TimeoutException;
use NACL::APISet::Exceptions::CouldNotResyncConnectionException;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;
use NACL::APISet::_Mixins::CLI qw(_get_absolute_cdef_paths);

sub execute_raw_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    # We need prefixing, suffixing (handling prompts) stuff to be done. Hence
    # we invoke execute_command internally. This also handles removing
    # command echo
    my $output
        = $self->execute_command( %args, 'return_response_object' => 0 );

    $Log->exit() if $may_exit;
    return $output;
}

##############################################################################
#                               Private Methods
##############################################################################

###############################################################################
# Method Name: _read_default_cdef_dir
# Objective:   finds the cdef dir
# Details: This method overrides the shared cdef dir logic of base class
#           implementation and thus prevents loading of unnecessary CDEFs
#           which are not applicable for this APISet.
###############################################################################

sub _read_default_cdef_dir {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift();
    my %args       = $pkg_or_obj->validate_params(
        params => \@_,
        spec   => $pkg_or_obj->_read_default_cdef_dir_spec,
    );

    my $libxml_obj = $args{libxml_obj};
    my $powershell_default
        = $libxml_obj->find(
        "/version/$args{category}/$args{interface}/$args{set}/default_cdef_dir"
        )->string_value;
    $Log->exit() if $may_exit;

    # Paths are listed in the order of precedence as the schema will not
    # overwrite the entry which is already present
    return ($powershell_default);

}

##############################################################################
# Method:    _translate_method_call
# Objective: Returns the command translated from method call
# Details:   This method uses schema object to translate method call to
#            the actual command to be invoked and this is done with the
#            help of CDEFs. If the specific CDEF doesn't exist, then it
#            assumes default translation in which underscores in the method
#            name are replaced by hyphens and options are translated
#            by adding a leading hyphen followed by the corresponding value
##############################################################################

sub _translate_method_call {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $command;
    %args = $self->validate_params(
        params => [%args],
        spec => { apiset_show_fields => { type => ARRAYREF, default => [] }, }
    );
    my @fields = @{ delete $args{apiset_show_fields} };
    try {
        my $schema_obj = $self->get_schema_obj();
        $command = $schema_obj->translate(%args);
    }
    catch NACL::APISet::Exceptions::MethodNotFoundException with {

        # Perform the default simple translation
        $Log->trace("Performing default translation");
        $command = delete $args{method};
        $command =~ s/_/-/g;
        my $pass_through = delete $args{pass_through};
        foreach my $opt ( keys %args ) {
            $command .= " -${opt} $args{$opt}";
        }
        if ($pass_through) {
            $command .= ' ' . $pass_through;
        }
    };

    # Add the output format specifier.
    if ( scalar @fields ) {
        my $field = join( ',', @fields );
        $command .= "| Format-List -Property $field";
    } else {
        $command .= "| Format-List";
    }

    $Log->debug("Translated command is $command");
    $Log->exit() if $may_exit;
    return $command;
}

##############################################################################
# Method:    _find_parser_file
# Objective: To find the parser file
# Details:   It calls the base class implementation and if the command
#            was executed without having a CDEF, it results in MethodNotFound
#            exception. It handles the failure and returns undef in that case.
###############################################################################

sub _find_parser_file {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $file;
    try {
        $file = $self->SUPER::_find_parser_file(%args);
    }
    catch NACL::APISet::Exceptions::MethodNotFoundException with {};
    $Log->exit() if $may_exit;
    return $file;
}

##############################################################################
# Method:    _throw_timeout_exception
# Objective: Throws TimeoutException on timeout or idle connection
# Details:   On timeout of connection,or idle connection this subroutine is
#             executed to throw the TimeoutException
###############################################################################

sub _throw_timeout_exception {
    my ( $self, %args ) = @_;
    my $execute_before = $args{before};
    return sub {

        # the user's timeout handler is executed first
        &$execute_before if $execute_before;
        my $connection = delete $self->{session}->{connection};

        try {
            $connection->send_intr();
        } catch NATE::BaseException with {
            my $exception = $_[0];
            NACL::APISet::Exceptions::CouldNotResyncConnectionException
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
        };

        # In case of persistent connection with powershell, we are not able
        # to kill the running command alone without exiting out of the
        # interactive session started earlier through 'powershell -Command -'
        # ctrl-C results in session being closed completely. This is a
        # limitation with powershell. To make the APISet usable further,
        # we drop the current connection and establish a new one, but
        # unfortunately the history of things done in the previous session is
        # lost.
        $Log->trace(
            "Establishing a new connection from PowerShell timeout handler");
        $self->{session}->connect_and_prep();

        my $message = 'User defined timeout handler was executed';

      # Output will be available only if the default handler alone is executed
      # This is because when custom handlers are specified, we cannot be sure
      # that the output was not already read in the custom handler itself.

        $message = 'Timeout has occured' if !$execute_before;

        NACL::APISet::Exceptions::TimeoutException->throw($message);
        }
} ## end sub _throw_timeout_exception

1;

=head1 NAME

NACL::APISet::Host::CLI::PowerShell - API Set for PowerShell CLI commands

=head1 SYNOPSIS

use NACL::APISet::Host::CLI::PowerShell;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI::PowerShell> is used to connect to and send CLI commands
to the PowerShell Host.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::CLI::PowerShell>.
The constructore of this package should not be directly called and the factory class
NACL::APISet should be used instead.

=over

=item Synopsis

 $api_set_obj = NACL::APISet->new( hostobj => $hostrec, category => 'Host', interface => 'CLI', set => 'PowerShell' );

=item Arguments

=over

=item %args

Parameters passed to parent classes.  See NACL::APISet for details.

=back

=item Returns

A reference to a new PowerShell API Set (NACL::APISet::Host::CLI::PowerShell)

=back

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw CLI commands on the powershell. Unlike
the execute_raw_command support in other APISets, it modifies the 
'command' which you have sent by adding necessary prefixes and suffixes
to facilitate the execution of the command in PowerShell. This can be
used for the cases in which you would like the output to be obtained with
different output format. (For method calls, APISet uses Format-List
as the output format by default).

Note that execute_raw_command is a thin wrapper over execute_command with
return_response_object => 0. This is supported to provide parity with
other APIset implementations.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(command => "Get-Process -Name powershell | Format-Wide -Property Id");

=item Arguments

execute_raw_command accepts all arguments accepted by Connectrec->execute().
Of these, only command is mandatory.

=over

=item command

(Mandatory)
The command to be invoked. This is the full command line as it will be entered
on the PowerShell commandline.

=back

=item Returns

(Scalar)
Raw output

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

