#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @name ::transition
##
## @summary Routines to parse the output of the Windows transition command output
##
## @description
##    This is a library to parse the raw output of the Windows transition command
##
## @synopsis
##    use NACL::APISet::Host::CLI::SMTT::V1_3::transition;
##
##    transition_credentials_add(%args);
##    transition_create(%args);
##    transition_volumepair_add(%args);
##    transition_add_primary_seven_mode_system(%args);
##    transition_add_secondary_cluster_mode_system(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_schedule_add(%args);
##    transition_schedule_modify(%args);
##    transition_schedule_remove(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_precutover(%args);
##    transition_resync(%args);
##    transition_update(%args);
##    transition_cutover(%args);
##    transition_abort(%args);
##    transition_delete(%args);
##    transition_show(%args);
##    transition_schedule_show(%args);
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @status Review
#

###################
# Package
package NACL::APISet::Host::CLI::SMTT::V1_3::transition;

###################
# Everytime
no strict 'refs';
use NACL::APISet::Response::CLI::ParserUtils;

###################
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::transition_credentials_add
##
## @summary Parses the raw output from the "transition credentials add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition credentials add -h 10.227.202.110 -u root :
##&nbsp;Enter password for 'root@10.227.202.110':
##&nbsp;netapp1!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;
sub transition_credentials_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_credentials_add

## @name ::transition_create
##
## @summary Parses the raw output from the "transition_create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_create with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'svm-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
##&nbsp;transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp; Error(31101) : The session type 'standfsdfalone' is invalid.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31101',
##&nbsp;                           'message' => 'The session type \'standfsdfalone\' is invalid.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
sub transition_create {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args); 
    return $result_array;

} ## end sub transition_create

## @name ::transition_volumepair_add
##
## @summary Parses the raw output from the "transition_volumepair_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;Error(31248) : Aggregate must not be provided along with the existing Cluster-Mode volume.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31248',
##&nbsp;                           'message' => 'Aggregate must not be provided along with the existing Cluster-Mode volume.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
sub transition_volumepair_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumepair_add

## @name :: transition_add_primary_seven_mode_system
##
## @summary Parses the raw output from the "transition_add_primary_seven_mode_system" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_add_primary_seven_mode_system with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_add_primary_seven_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_add_primary_seven_mode_system' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'source-host' </td> <td> </td> '0' </tr>
## <tr> <td> '-f' </td> <td> 'management-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'data-copy-ipaddress-staggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition add-primary-seven-mode-system -s S1 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-primary-seven-mode-system -s S1 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110
##&nbsp;
##&nbsp;transition add-primary-seven-mode-system -s S5 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110 :
##&nbsp;Error(31115) : Transition session with name \'S5\' does not exist.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31115',
##&nbsp;                           'message' => 'Transition session with name \'S5\' does not exist.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-primary-seven-mode-system -s S5 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110
##&nbsp;

sub transition_add_primary_seven_mode_system{

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}## end transition_add_primary_seven_mode_system

## @name :: transition_add_secondary_cluster_mode_system
##
## @summary Parses the raw output from the "transition_add_secondary_cluster_mode_system" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_add_secondary_cluster_mode_system with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_add_secondary_cluster_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_secondary_cluster_mode_system' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vserver-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec
##&nbsp;
##&nbsp;transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec1 :
##&nbsp;Error(31283) : Specified Vserver \'vs_sec1\' does not exists on the Cluster-Mode storage system \'10.226.99.61\'.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31283',
##&nbsp;                           'message' => 'Specified Vserver \'vs_sec1\' does not exists on the Cluster-Mode storage system \'10.226.99.61\'.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec1
##&nbsp;
sub transition_add_secondary_cluster_mode_system{

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}## end tranition_add_secondary_cluster_mode_system

## @name ::transition_precheck
##
## @summary Parses the raw output from the "transition_precheck" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_precheck with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition precheck -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'next_suggested_steps' => '1. Use the \'transition job-results -j 170c0cbe-c5f2-464e-969e-37ec1424ffa0\' command to see the results of this operation. 2. Carefully review the precheck operation results before proceeding with the next step. 3. Select \'Start data copy\' from the main menu or run the transition CLI command \'transition start -s S21\' when you are ready to start data copy to Cluster-Mode volumes.',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition precheck -r no -s S21
##&nbsp;
sub transition_precheck {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_precheck

## @name ::transition_start
##
## @summary Parses the raw output from the "transition_start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_start with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition start -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90051',
##&nbsp;                                   'message' => 'Creating the required clustered Data ONTAP volumes on the Vserver \'vs_sec\'.Successfully created the following clustered Data ONTAP volumes on the Vserver \'vs_sec\'.'
##&nbsp;                                 },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition start -r no -s S21
##&nbsp;
sub transition_start {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_start

## @name :: transition_schedule_add
##
## @summary Parses the raw output from the "transition_schedule_add" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_add with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_add(%args);
## INPUTS TO METHOD CALL 'transition_shedule_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##@examples
##&nbsp;transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2
##&nbsp;
##&nbsp;transition schedule add -s SS3 -n sc4 -d 0 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;Error(31327) : Schedule \'sc4\' interferes with the schedule \'sc1\' which is already existing in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31327',
##&nbsp;                           'message' => 'Schedule \'sc4\' interferes with the schedule \'sc1\' which is already existing in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule add -s SS3 -n sc4 -d 0 -b 12:00 -e 10:00 -t 2 -x 2
##&nbsp;
sub transition_schedule_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_add

## @name :: transition_schedule_modify
##
## @summary Parses the raw output from the "transition_schedule_modify" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_modify with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_shedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:30 -t 2 -x 2 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule modify -s S21 -n sc1-d 1-5 -b 12:00 -e 10:30 -t 2 -x 2
##&nbsp;
sub transition_schedule_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_modify

## @name :: transition_schedule_remove
##
## @summary Parses the raw output from the "transition_schedule_remove" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_remove with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_remove(%args);
## INPUTS TO METHOD CALL 'transition_shedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;transition schedule remove -s S21 -n sc1 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule remove -s S21 -n sc1
##&nbsp;
##&nbsp;transition schedule remove -s S21 -n sc4 :
##&nbsp;Error(31328) : Schedule \'sc4\' is not available in the session \'S21\'.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31328',
##&nbsp;                           'message' => 'Schedule \'sc4\' is not available in the session \'S21\'.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule remove -s S21 -n sc4
##&nbsp;
sub transition_schedule_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_remove

## @name ::transition_pause
##
## @summary Parses the raw output from the "transition_pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_pause with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition pause -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = transition pause -r no -s S21
##&nbsp;
##&nbsp;
sub transition_pause {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_pause

## @name ::transition_resume
##
## @summary Parses the raw output from the "transition_resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_resume with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition resume -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90071',
##&nbsp;                                   'message' => 'Resuming data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume\'vol_compo1\'. Successfully resumed data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition resume -r no -s S21
##&nbsp;
sub transition_resume {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_resume

## @name :: transition_precutover
##
## @summary Parses the raw output from the "transition_precutover" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_precutover with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_precutover(%args);
## INPUTS TO METHOD CALL 'transition_precutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapmirror-throttle' <td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition precutover -r no -s gay_nearer_1392898178 -m rw_test -t 4 -b 240 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol1 }',
##&nbsp;                                   'code' => '70141',
##&nbsp;                                   'message' => 'Collecting space reservation information from the 7-Mode storage system\'vfiler0@10.238.24.232\'. Successfully collected space reservation information for the following 7-Mode volumes from the 7-Mode storage system \'vfiler0@10.238.24.232\'.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ nfs.response.trigger , nfs.udp.xfersize , nfs.v4.acl.max.aces , nfs.rpcsec.ctx.idle , nfs.tcp.xfersize , nfs.rpcsec.ctx.high , wafl.default_nt_user , nfs.v4.id.domain , nfs.mount_rootonly , nfs.response.trace , nfs.v3.enable , nfs.v4.acl.enable , nfs.v4.write_delegation , nfs.v4.snapshot.active.fsid.enable , nfs.v4.enable , nfs.tcp.enable , nfs.v4.read_delegation , nfs.vstorage.enable , nfs.udp.enable }',
##&nbsp;                                   'code' => '10361',
##&nbsp;                                   'message' => 'Collecting the NFS options settings from the 7-Mode storage system \'vfiler0@10.238.24.232\'.Successfully collected the following NFS options settings from the 7-Mode storage system \'vfiler0@10.238.24.232\'.'
##&nbsp;                                 },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '{ 10.238.24.232 santosh6-vsim1 santosh6-vsim1-e0c Error Code: 13130 Reason: duplicate entry , 127.0.10.1 localhost-10 localhost-bsd Error Code: 13130 Reason: duplicate entry , 127.0.0.1 localhost localhost-stack Error Code: 13130 Reason: duplicate entry , 127.0.20.1 localhost-20 localhost-sk Error Code: 13130 Reason: duplicate entry }',
##&nbsp;                             'code' => '40111',
##&nbsp;                             'message' => 'Applying the hosts configuration to the Vserver \'vs_trans\'.Failed to create the following hosts configuration on the Vserver \'vs_trans\'.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '{ vol_compo1 }',
##&nbsp;                             'code' => '92021',
##&nbsp;                             'message' => 'Resynchronizing the SnapMirror relationship between the 7-mode source volumes and the clusteredData ONTAP destination volumes. The SnapMirror resync operation for the following clustered Data ONTAP destination volumes are triggered successfully. The clustered Data ONTAP destination volumes will be available for read-only operations after the resync operation is completed successfully. This might take several minutes.'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition precutover -r no -s gay_nearer_1392898178 -m rw_test -t 4 -b 240
##&nbsp;
sub transition_precutover {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_precutover

## @name :: transition_resync
##
## @summary Parses the raw output from the "transition_resync" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_resync with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_resync(%args);
## INPUTS TO METHOD CALL 'transition_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition resync -r no -s charm_tulip_1392896242 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => 0,
##&nbsp;            'informational' => [],
##&nbsp;            'error_count' => 0,
##&nbsp;            'warning_count' => '1',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '92021',
##&nbsp;                             'message' => 'Resynchronizing the SnapMirror relationship between the 7-mode source volumes and the clusteredData ONTAP destination volumes. The SnapMirror resync operation for all the clustered Data ONTAP destination volumes is triggered successfully. The clustered Data ONTAP destination volumes will be available for read-only operations after the resync operation is completed successfully. This might take several minutes.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition resync -r no -s charm_tulip_1392896242
##&nbsp;

sub transition_resync {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_resync

## @name :: transition_update
##
## @summary Parses the raw output from the "transition_update" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_update with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_update(%args);
## INPUTS TO METHOD CALL 'transition_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition update -r no -s gay_nearer_1392898178 -t 4 -x 240 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => 0,
##&nbsp;            'informational' => [],
##&nbsp;            'error_count' => 0,
##&nbsp;            'warning_count' => '0',
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition update -r no -s gay_nearer_1392898178 -t 4 -x 240
##&nbsp;

sub transition_update {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_update

## @name :: transition_cutover
##
## @summary Parses the raw output from the "transition_cutover" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_cutover with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;transition cutover -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90821',
##&nbsp;                                   'message' => 'Performing the final update transfer for the clustered Data ONTAP volumes. Final update succeeded for the following clustered Data ONTAP volumes.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '96801',
##&nbsp;                                   'message' => 'Taking all the 7-Mode volumes offline. Offlined all the 7-Mode volumes.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => '0',
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition cutover -r no -s S21
##&nbsp;
sub transition_cutover {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_cutover

## @name ::transition_abort
##
## @summary Parses the raw output from the "transition_abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_abort with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition abort -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '3',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 },
##&nbsp;                                {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90011',
##&nbsp;                                   'message' => 'Breaking the SnapMirror relationships. Successfully broken all SnapMirror relationships.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                  'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90021',
##&nbsp;                                   'message' => 'Deleting the SnapMirror relationships. Successfully deleted all SnapMirror relationships.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition abort -r no -s S21
##&nbsp;
sub transition_abort {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_abort

## @name ::transition_delete
##
## @summary Parses the raw output from the "transition_delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_delete with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition delete -s S21
##&nbsp;
sub transition_delete {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_delete

## @name ::transition_show
##
## @summary Parses the raw output from the "transition_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_show(%args);
## INPUTS TO METHOD CALL 'transition_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition show -r no -s secondary :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;            'vserver_comment' => 'Pre-existing',
##&nbsp;            'multipath_ip_address' => '-',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.108',
##&nbsp;            'data_copy_ip_address' => '10.227.65.87',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'snapmirror_status' => [
##&nbsp;                {
##&nbsp;                'snapmirror_state' => '-/-',
##&nbsp;                '7_mode_volume' => 'dst',
##&nbsp;                'cluster_mode_volume' => 'dst',
##&nbsp;                'comments' => '-'
##&nbsp;                }
##&nbsp;            ],
##&nbsp;            'schedule_info' => [
##&nbsp;                {
##&nbsp;                'update_frequency' => '00:30',
##&nbsp;                'streams_count' => '10',
##&nbsp;                'days_range' => '0',
##&nbsp;                'throttle' => '-1',
##&nbsp;                'start_time' => '01:00',
##&nbsp;                'schedule_name' => 'secondary_schedule',
##&nbsp;                'duration' => '02:00'
##&nbsp;                }
##&nbsp;            ],
##&nbsp;            'lif_info' => [
##&nbsp;                 {
##&nbsp;                 'home_node' => 'sti3270-020',
##&nbsp;                 'gateway' => '10.72.219.1',
##&nbsp;                 'lif_status' => 'Created',
##&nbsp;                 'netmask' => '255.255.255.0',
##&nbsp;                 'ip_address' => '10.72.219.200',
##&nbsp;                 'home_port' => 'e1b'
##&nbsp;                 }
##&nbsp;             ],
##&nbsp;             '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;             'snapmirror_schedule' => '-',
##&nbsp;             'session_type' => 'secondary',
##&nbsp;             'version' => '1.3.0',
##&nbsp;             'session_name' => 'secondary',
##&nbsp;             'apply_configuration' => 'Yes',
##&nbsp;             'session_state' => 'Completed',
##&nbsp;             'primary_7_mode_details' => '10.227.66.88:10.227.66.88:10.227.66.88:-',
##&nbsp;             'cluster_mode_system_version' => 'NetApp Release SierraNevada__8.2.2 Cluster-Mode: Sat Jan 25 15:07:32 PST 2014',
##&nbsp;             '7_mode_version' => 'NetApp Release R8.2.1xN_131112_1507 7-Mode: Tue Nov 12 15:56:49 PST 2013',
##&nbsp;             'vserver_name' => 'vsec',
##&nbsp;             'session_status' => '-'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no -s secondary
##&nbsp;
##&nbsp;transition show -r no:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.123',
##&nbsp;            'vserver_name' => 'vserver4',
##&nbsp;            'session_status' => '-',
##&nbsp;            'session_name' => 'cifssession',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'session_state' => 'Completed'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.123',
##&nbsp;            'vserver_name' => 'vserver4',
##&nbsp;            'session_status' => '-',
##&nbsp;            'session_name' => 'cifssession2',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'session_state' => 'Aborted'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no
##&nbsp;
##&nbsp;
##&nbsp;
sub transition_show {

    my %args = @_;
    my ($first_line, $vol7mode, $volcmode,  $snapmirror,
        $comments,   $temp,     $aggregate, $cmodestatus,
        %hash,       %entry,    %comment,   $error_message,
    );
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    my $result_ref=[];
    if (!$error){
        if ($output =~ /Apply configuration/){

            #Let us remove the schedule part and parse it separately
            my ( $schedule, @schedule_arr);
            if($output=~s/(.*)(Name\s\s+Days Range\s\s+Start Time.*)(7-Mode volume\s\s+Cluster-Mode volume.*)/$1$3/s){
                $schedule=$2;
            }
            if(defined $schedule){
                $schedule=~s/Name\s+Days Range\s+Start Time\s+Duration\s+Streams Count.*----------//s;
                @schedule_arr=split(/\s+/,$schedule);
                foreach (0..@schedule_arr-1){
                    splice(@schedule_arr, $_, 1) if @schedule_arr[$_] eq '';
                }
                for(my $i=0; $i<@schedule_arr;){
                    push(
                        @{ $hash{schedule_info} },
                        {   'schedule_name'       => $schedule_arr[$i],
                            'days_range'          => $schedule_arr[$i+1],
                            'start_time'          => $schedule_arr[$i+2],
                            'duration'            => $schedule_arr[$i+3],
                            'streams_count'       => $schedule_arr[$i+4],
                            'throttle'            => $schedule_arr[$i+5],
                            'update_frequency'    => $schedule_arr[$i+6],
                        }
                    );
                    $i=$i+7;
                }
            }

            #Let us remove LIF information and parse it separately
            my ( $lif, @lif_arr );
            if($output=~s/(.*)(IP address\s\s+Netmask\s\s+Gateway.*)/$1/s){
                $lif=$2;
            }
            if(defined $lif){
                $lif=~s/IP address\s+Netmask.*----------//s;
                @lif_arr=split(/\s\s+/,$lif);
                foreach (0..@lif_arr-1){
                    splice(@lif_arr, $_, 1) if @lif_arr[$_] eq '';
                }
                for(my $i=0; $i<@lif_arr;){
                    push(
                        @{ $hash{lif_info} },
                        {   'ip_address'       => $lif_arr[$i],
                            'netmask'          => $lif_arr[$i+1],
                            'gateway'          => $lif_arr[$i+2],
                            'home_node'        => $lif_arr[$i+3],
                            'home_port'        => $lif_arr[$i+4],
                            'lif_status'       => $lif_arr[$i+5],
                        }
                    );
                    $i=$i+6;
                }
            }

            my @summary_lines = get_lines(
                raw_output      => $output,
                response_object => $resp,
                first_line      => "SUMMARY:"
            );

            #remove summary lines from the output
            foreach my $line (@summary_lines) {
                $output =~ s/\Q$line\E//g;
            }
            if ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments';
            } elsif ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status';
            }

            #grep snapmirror status lines from output and parse the fields
            $output =~ m/$first_line(.*)/si;
            my $snapmirror_string = $1 if $1;
            my @snapmirror_status_lines = split( "\n", $snapmirror_string );
            foreach my $line (@snapmirror_status_lines) {
                if ( $line =~ /(^\w+)/ ) {
                    $temp = $1;
                    $entry{$temp} = $line;
                } elsif ( $line =~ /^\s+/ ) {
                    $entry{$temp} = $entry{$temp} . $line;
                }
            }
            foreach my $en ( keys(%entry) ) {
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments'
                    )
                {
                    ( $vol7mode, $volcmode, $snapmirror, $comments )
                        = split( /\s+/, $entry{$en}, 4 );
                    if (    defined $vol7mode
                        and defined $volcmode
                        and defined $snapmirror
                        and defined $comments )
                    {
                        #remove extra spaces from comments field
                        $comments =~ s/\s{2,}//g;
                        $comments =~ s/\*/ */g;
                        push(
                            @{ $hash{snapmirror_status} },
                            {   '7_mode_volume'       => $vol7mode,
                                'cluster_mode_volume' => $volcmode,
                                'snapmirror_state'    => $snapmirror,
                                'comments'            => $comments
                            }
                        ) if ( $vol7mode || $volcmode || $snapmirror || $comments);
                    }
                }
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status'
                    )
                {
                    ( $vol7mode, $volcmode, $aggregate, $snapmirror, $cmodestatus )
                        = split( /\s+/, $entry{$en} );
                    push(
                        @{ $hash{snapmirror_status} },
                        {   '7_mode_volume'              => $vol7mode,
                            'cluster_mode_volume'        => $volcmode,
                            'snapmirror_state'           => $snapmirror,
                            'aggregate'                  => $aggregate,
                            'cluster_mode_volume_status' => $cmodestatus
                        }
                    ) if ( $vol7mode || $volcmode || $snapmirror
                        || $comments || $aggregate );
                }
            }

            #remove snapmirror status lines from output
            $output =~ s/($first_line.*)//s if $first_line;

            #parse rest of the show output
            my @lines = get_lines(
                raw_output      => $output,
                response_object => $resp
            );
            foreach my $line (@lines) {
                if ( $line !~ /----------/ ) {
                    my ( $key, $val ) = split( ':', $line, 2 );
                    trim( string_ptr => \$key );
                    trim( string_ptr => \$val );
                    $key =~ s/\s/_/g;
                    if(_upper_to_lower($key) eq "vserver_information"){
                        ($hash{"vserver_name"},$hash{vserver_comment})=split(/:/,$val);
                    }else{
                        $hash{_upper_to_lower($key)} = $val;
                    }
                }
            }
        }elsif($output =~ /7-Mode system/){
            my (@arr,%chunk_hash,@new_lines);
            @lines=split(/\n/,$output);
            my $i=0;
            foreach my $line (@lines){
                next if($line =~ /Name\s+Type/ || $line =~ /-----/ || $line !~ /\w+/) ;
                $new_lines[$i]=$line;
                $i++;
            }
            my @data;
            foreach my $line (@new_lines){
                push(@data,split(/\s+/,$line));
            }
            foreach (0..@data-1){
                splice(@data, $_, 1) if $data[$_] eq '';
            }
            for(my $i=0; $i<@data;){
                $chunk_hash{"session_name"}=$data[$i];
                $chunk_hash{"session_type"}=$data[$i+1];
                ($chunk_hash{"7_mode_system_fqdn"},$chunk_hash{"vfiler_unit"})
                    =split(/:/,$data[$i+2]);
                ($chunk_hash{"cluster_mode_system_fqdn"},$chunk_hash{"vserver_name"})
                    =split(/:/,$data[$i+3]);
                $chunk_hash{"session_state"}=$data[$i+4];
                $chunk_hash{"session_status"}=$data[$i+5];
                push @$result_ref , {%chunk_hash} ;
                $i=$i+6;
            }
        }
    }

    push @{$result_ref} , {%hash};

    if($error) {
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }
    return $result_ref;

} ## end sub transition_show

## @name ::transition_schedule_show
##
## @summary Parses the raw output from the "transition_schedule_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_schedule_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_schedule_show(%args);
## INPUTS TO METHOD CALL 'transition_schedule_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition schedule show -s Arragon_languished_1392735210 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '0',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse1',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '1',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse2',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule show -s Arragon_languished_1392735210
##&nbsp;
##&nbsp;transition schedule show -s Arragon_languished_1392735210 -n Betelgeuse1:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '0',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse1',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule show -s Arragon_languished_1392735210 -n Betelgeuse1
##&nbsp;
##&nbsp;
##&nbsp;

sub transition_schedule_show {

    my %args = @_;
    my %hash ;
    my $result_ref=[];
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    if (!$error){
        my ( @schedule_arr);
        $output=~s/Name\s+Days Range\s+Start Time\s+Duration\s+Streams Count.*----------//s;
        @schedule_arr=split(/\s+/,$output);
        foreach (0..@schedule_arr-1){
            splice(@schedule_arr, $_, 1) if @schedule_arr[$_] eq '';
        }
        for(my $i=0; $i<@schedule_arr;){
            my %chunk_hash = (
                'schedule_name'       => $schedule_arr[$i],
                'days_range'          => $schedule_arr[$i+1],
                'start_time'          => $schedule_arr[$i+2],
                'duration'            => $schedule_arr[$i+3],
                'streams_count'       => $schedule_arr[$i+4],
                'throttle'            => $schedule_arr[$i+5],
                'update_frequency'    => $schedule_arr[$i+6],
            );
            $i=$i+7;
            push @{$result_ref} , {%chunk_hash};
        }
    }
    push @{$result_ref} , {%hash};

    if($error) {
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return $result_ref;

} ## end sub transition_schedule_show

sub _upper_to_lower{

    my $string=shift;
    $string =~ tr/A-Z/a-z/;
    $string =~ s/-/_/;
    return $string;

} ## end sub _upper_to_lower

sub _formatter{

    my $string=shift;
    $string=~s/^\s+|\s+$//g;
    return $string;

} ## end sub _formatter

sub _get_count{

    my $string=shift;
    my $var;
    $var=$2 if shift =~ /((\d+)\s+$string)/;
    if (!defined $var){
        $var=0;
    }
    return $var;

} ## end sub _get_count

sub _check_error {

    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $hash_ref=$args{hash};
    my %hash=%{$hash_ref};
    my $error;
    if($output =~ /No value specified for .*/) {
        $error = "MissingRequiredParam";
    } elsif ($output =~ /The .* command takes .* arguments. You gave .*.  Extra arguments: .*/) {
        $error = "InvalidParam";
    } elsif ($output =~ /Transition session with name .* does not exist/ ||
        $output =~ /The session name .* is invalid. Special characters .* are not allowed in the session name/ ||
        $output =~ /.* storage system volume .*' already used in the transition session .*/){
        $error = "InvalidParamValue";
    } elsif ($output =~ /(Operation summary.*?(\d+)\s+Error.*)/is && $2>0
            || $output =~ /.*Failed to run the.*/
            || $output =~ /Transition session.*status is not.*/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(SYNOPSIS)/ || $output =~ /session with name .* already exists/
        || $output =~ /Aggregate must not be provided along with the existing Cluster-Mode volume/
        || $output =~ /.*must be provided.*]/
        || $output =~ /Failed to communicate with 7-Mode Transition Tool server/
        || $output =~ /.*not recognized as an internal or external command.*/
        || $output =~ /Host .* not found in credential bank/
        || $output =~ /There are no sessions/
        || $output =~ /Volume.*does not exists.*/
        || $output =~ /Credentials provided for host.* are invalid/
        ){
        $error = "CommandFailed";
    }

    $$error_message = $output;

    # This is used to catch the errors which went unnoticed above
    if ($output =~ /(Error)(?:(\([-]?\d+\)))?(.*)/ &&
        $output !~ /Name.*Type.*7-Mode system.*Cluster-Mode system.*Status/is
        && $output !~ /Apply configuration.*7-Mode system FQDN/is &&
        $output !~ /Operation summary/is ){
        my $a1=$1 if $1;
        my $a2=$2 if $2;
        my $a3=$3 if $3;
        $a2=~ s/\(//;
        $a2=~ s/\)//;
        $a3=~ s/:\s//;
        my $member=$hash{_upper_to_lower($a1)}=[];
        my %chunk_hash;
        $chunk_hash{"code"}=$a2;
        $chunk_hash{"message"}=_formatter($a3);
        push @$member , {%chunk_hash};
        $hash{"error_count"}=1;
    }else{
        $hash{"error_count"}=0;
    }

    %$hash_ref=%hash;

    return $error;

} ## end sub _check_error

sub _no_precheck_type_parser{

    my $ref=shift;
    my ($error_message , @result_array , %hash , $resp , $output , %args );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);

    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return \@result_array;

} ## end sub _no_precheck_type_parser

sub _precheck_type_parser{

    my $ref=shift;
    my ($error_message, @result_array, $resp,       $output, %args,
        @chunks,        $last_chunk,   $op_string,  %hash,   $first_line,
        @lines,         $error
    );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    $first_line = '.*';
    @lines = get_lines(raw_output=>$output, first_line=>$first_line);

    # Create a string version from Array of o/p Lines
    $op_string = join (" ", @lines);
    #Get the last Operation summary chunk separate from chunk array
    $last_chunk=$1 if $op_string =~ s/(Operation summary.*)//gi;
    $op_string =~ s/(Job with.*has started. )//gi;

    #Prepare an array of chunks one for each error code
    my $i=-1;
    do{
        $i++;
        push(@chunks,$1) if $op_string =~ s/^(\w{0,4}\d{1,5}\s+:\s.*?)(\w{0,4}\d{1,5}\s+:\s)/$2/g ;
    }while(defined $chunks[$i]);
    push(@chunks,$op_string);

    $hash{"error"}=[];
    $hash{"warning"}=[];
    $hash{"informational"}=[];

    #Format the chunks and push them into the complex data structure
    foreach my $i (0..@chunks-1){
        my (%chunk_hash,@msg,$chunk_type,$chunk_code,$members);
        $chunks[$i] =~ s/\d+\s+:$//g;
        $chunks[$i] =~ s/\s+> / /g;
        $chunk_type=$1 if $chunks[$i] =~ s/\[\s+(\w+)\s+\]//g;
        $chunks[$i] =~ s/  //g;
        $chunk_code=$1 if $chunks[$i] =~ s/^(\w{0,4}\d{1,5})\s+://g;
        $chunk_hash{"code"}=$chunk_code;
        if($chunks[$i] =~ /CORRECTIVE-ACTION/
           && $chunks[$i] !~ /AFFECTED-OBJECTS/) {
            @msg=split(/CORRECTIVE-ACTION:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"corrective_action"}=_formatter($msg[1]);
            $chunk_hash{"affected_objects"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] !~ /CORRECTIVE-ACTION/){
            @msg=split(/AFFECTED-OBJECTS:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"affected_objects"}=_formatter($msg[1]);
            $chunk_hash{"corrective_action"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] =~ /CORRECTIVE-ACTION/){
            $chunks[$i] =~ s/\s  //g;
            $chunk_hash{"affected_objects"}=_formatter($1)
                if $chunks[$i] =~ s/AFFECTED-OBJECTS:(.*)//gims;
            $chunk_hash{"corrective_action"}=_formatter($1)
                if $chunks[$i] =~ s/CORRECTIVE-ACTION:(.*)//gims;
            $chunk_hash{"message"}=_formatter($chunks[$i]);
        }else{
            $chunk_hash{"message"}=_formatter($chunks[$i]);
            $chunk_hash{"affected_objects"}="";
            $chunk_hash{"corrective_action"}="";
        }
        $members=$hash{_upper_to_lower($chunk_type)};
        push @$members , {%chunk_hash};

    }
    my @keys = keys (%hash);

    #Push the last chunk onto the complex data structure alongwith other details
    $hash{"error_count"}=_get_count("Errors",$last_chunk) ;
    $hash{"warning_count"}=_get_count("Warnings",$last_chunk) ;
    $hash{"informational_count"}=_get_count("Informational messages",$last_chunk);
    $hash{"next_suggested_steps"}=_formatter($1) if $last_chunk =~ /Next suggested steps: [-]+(.*)(Ran.*)/;

    #Clear  empty keys
    foreach (@keys){
        delete $hash{$_} if (!$_);
    }

    push (@result_array, \%hash);

    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return \@result_array;

} ## end sub _precheck_type_parser

1;
