#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @name ::transition
##
## @summary Routines to parse the output of the Windows transition command output
##
## @description
##    This is a library to parse the raw output of the Windows transition command
##
## @synopsis
##    use NACL::APISet::Host::CLI::SMTT::V2_1::transition;
##
##    transition_create(%args);
##    transition_credentials_add(%args);
##    transition_credentials_get(%args);
##    transition_credentials_modify(%args);
##    transition_credentials_remove(%args);
##    transition_version(%args);
##    transition_lif_add(%args);
##    transition_lif_modify(%args);
##    transition_lif_remove(%args);
##    transition_lif_show(%args);
##    transition_volumepair_add(%args);
##    transition_volumepair_modify(%args);
##    transition_volumepair_remove(%args);
##    transition_volumepair_show(%args);
##    transition_export_lunmap(%args);
##    transition_add_primary_seven_mode_system(%args);
##    transition_add_secondary_cluster_mode_system(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_schedule_add(%args);
##    transition_schedule_modify(%args);
##    transition_schedule_remove(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_precutover(%args);
##    transition_resync(%args);
##    transition_update(%args);
##    transition_cutover(%args);
##    transition_abort(%args);
##    transition_delete(%args);
##    transition_show(%args);
##    transition_schedule_show(%args);
##    transition_add_cluster_info(%args);
##    transition_modify(%args);
##    transition_property_set(%args);
##    transition_property_get(%args);
##    transition_property_reset(%args);
##    transition_remove_primary_seven_mode_system(%args);
##    transition_remove_secondary_cluster_mode_system(%args);
##    transition_bundle_tool_logs(%args);
##    transition_jobs(%args);
##    transition_show_primary_seven_mode_systems(%args);
##    transition_show_secondary_cluster_mode_systems(%args);
##    transition_job_status(%args);
##    transition_job_results(%args);
##    transition_qtreepair_add(%args);
##    transition_qtreepair_modify(%args);
##    transition_qtreepair_remove(%args);
##    transition_stagingsystems_add(%args);
##    transition_stagingsystems_modify(%args);
##    transition_stagingsystems_remove(%args);
##    transition_volumeaggregatemapping_add(%args);
##    transition_volumeaggregatemapping_modify(%args);
##    transition_volumeaggrgatemapping_remove(%args);
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @change     2014-11-28 dinoc     :    12 Transition subroutines were added.
## @change     2014-11-27 ppranav   :     Added 13 new subroutines for respective
##                                          commands along with parsers.
## @change     2015-01-30 santosh6  :    Added new subroutines to support Qtree breakout
## @status Review
####################
# Package
package NACL::APISet::Host::CLI::SMTT::V2_1::transition;

###################
# Everytime
no strict 'refs';
use NACL::APISet::Response::CLI::ParserUtils;

###################
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::transition_create
##
## @summary Parses the raw output from the "transition_create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_create with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'svm-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
##&nbsp;transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp; Error(31101) : The session type 'standfsdfalone' is invalid.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31101',
##&nbsp;                           'message' => 'The session type \'standfsdfalone\' is invalid.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
sub transition_create {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_create
## @name ::transition_credentials_add
##
## @summary Parses the raw output from the "transition credentials add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition credentials add -h 10.227.202.110 -u root :
##&nbsp;Enter password for 'root@10.227.202.110':
##&nbsp;netapp1!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;
sub transition_credentials_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_credentials_add
## @name ::transition_credentials_get
##
## @summary Parses the raw output from the "transition credentials get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_get with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_get(%args);
## INPUTS TO METHOD CALL 'transition_credentials_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;                  'parsed_output' => [
##&nbsp;                                       {
##&nbsp;                                        'username' => '\'root\'.',
##&nbsp;                                         'host' => '\'10.63.36.166\''
##&nbsp;                                      },
##&nbsp;                                       {
##&nbsp;                                        'error_count' => 0
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;                'command' => 'transition credentials get -h 10.63.36.166'
##&nbsp;
sub transition_credentials_get {
    my %args = @_;
    my ($error_message , @result_array, %hash , $resp , $output);
    $resp = $args{response_object};
    $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    if (!$error){
        my ( @schedule_arr);
        if($output=~ m/Username/){
	    @schedule_arr=split(/\s+/,$output);
	    #splitting output with respect to space between words
	    foreach (0..@schedule_arr-1){
		splice(@schedule_arr, $_, 1) if @schedule_arr[$_] eq '';
	    }
	    for(my $i=0; $i<@schedule_arr;){
		my %chunk_hash = (
		    'host'               => $schedule_arr[$i+3],
		    'username'           => $schedule_arr[$i+5],
		);
		push(@result_array, \%chunk_hash);
		last;
	    }
	}
    }
    push(@result_array, \%hash);
    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );
    return \@result_array;
} ## end sub transition_credentials_get
## @name ::transition_credentials_modify
##
## @summary Parses the raw output from the "transition credentials modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_modify with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_modify(%args);
## INPUTS TO METHOD CALL 'transition_credentials_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;           'parsed_output' => [
##&nbsp;                                {
##&nbsp;                                 'error_count' => 1                                                                        'error' =>
##&nbsp;                                   {
##&nbsp;                                    'code' => undef,
##&nbsp;                                     'message' => 'No value specified for required option: -h (--host-name)'
##&nbsp;                                    }
##&nbsp;                                 }
##&nbsp;                              ],
##&nbsp;                              'command' => 'transition credentials modify -u root'
sub transition_credentials_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_credentials_modify

## @name ::transition_credentials_remove
##
## @summary Parses the raw output from the "transition credentials remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_remove with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_remove(%args);
## INPUTS TO METHOD CALL 'transition_credentials_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;              'parsed_output' => [
##&nbsp;                   {
##&nbsp;                    'error_count' => 1,
##&nbsp;                        'error' => [
##&nbsp;                                    {
##&nbsp;                                     'code' => '21500',
##&nbsp;                                     'message' => 'Host \'10.63.36.166\' not found in credential bank.'
##&nbsp;                                    }
##&nbsp;                                   ]
##&nbsp;                   }
##&nbsp;                                                                    ],
##&nbsp;                   'command' => 'transition credentials remove -h 10.63.36.166'
##&nbsp;
sub transition_credentials_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_credentials_remove

## @name ::transition_version
##
## @summary Parses the raw output from the "transition version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_version with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_version(%args);
## INPUTS TO METHOD CALL 'transition_version' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;    'parsed_output' => [
##&nbsp;                         {
##&nbsp;                          'client' => 'RTrans_2.0N_141110_0703',
##&nbsp;                          'version' => 'NetApp Release 7-Mode Transition Tool FullSteam__2.0.0: Mon Nov 10 10:16:05 EST 2014',
##&nbsp;                          'client_path' => '/x/eng/rlse11/transition/nightly/RTrans_2.0N_141110_0703',
##&nbsp;                          'change_number' => '2598139'
##&nbsp;                          },
##&nbsp;                         {
##&nbsp;                          'error_count' => 0,
##&nbsp;                         }
##&nbsp;                      ],
##&nbsp;                     'command' => 'transition version -v hgh'
##&nbsp;
sub transition_version {
    my %args = @_;
    my $i=0;
    my ($error_message , @result_array, %hash , $resp , $output);
    $resp = $args{response_object};
    $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    if($output =~ /NetApp/){
        my (@arr,%chunk_hash,@new_lines);
        @lines = split(/\n/,$output);
        my @data;
	foreach my $line (@lines){
            if($line !~ m/^NetApp/ && ($line =~ /Client/ || $line =~ /Change/)){
                push(@data,split(/:/,$line));
            }#splitting with ':' between words.
	}#execute if '-v' is present in command
	if(@data){
            for($i=0; $i<@data; $i++){
                $data[$i] =~ _formatter($data[$i]);
	    }#removing leading and trailing spaces.
	}
        for($i=0; $i<=@data;){
	    $chunk_hash{"version"}	=	$lines[0];
	    if(@data){
		$chunk_hash{"client"}	= $data[$i+1];
		$chunk_hash{"client_path"}	= $data[$i+3];
		$chunk_hash{"change_number"}	= $data[$i+5];
	    }
            push (@result_array,\%chunk_hash);
            last;
        }#Printing style of parsed output.
    }
        push(@result_array, \%hash);
    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );
    return \@result_array;
} ## end sub transition_version

## @name ::transition_lif_add
##
## @summary Parses the raw output from the "transition lif add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_lif_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_lif_add(%args);
## INPUTS TO METHOD CALL 'transition_lif_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'default-gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'home-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'home-node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp; 'parsed_output' => [
##&nbsp;                      {
##&nbsp;                        'error_count' => 1,
##&nbsp;                                        'error' => [
##&nbsp;                                                    {
##&nbsp;                                                        'code' => '21501',
##&nbsp;                                                        'message' => 'IPv4 address \'10.63.58.116\' has already been added to the session.'
##&nbsp;                                                     }
##&nbsp;                                                   ]
##&nbsp;                      }
##&nbsp;                    ],
##&nbsp;                    'command' => 'transition lif add -s session2 -i 10.63.58.116 -m 255.255.192.0 -g 10.63.0.1 -p e0c -n ccluster1-02'
##&nbsp;
sub transition_lif_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_lif_add

## @name ::transition_lif_modify
##
## @summary Parses the raw output from the "transition lif modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_lif_modify with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_lif_modify(%args);
## INPUTS TO METHOD CALL 'transition_lif_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'default-gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'home-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'home-node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;                  'parsed_output' => [
##&nbsp;                                       {
##&nbsp;                                         'error_count' => 0
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;                  'command' => 'transition lif modify -s session2 -i 10.63.58.116 -m 255.255.192.0 -g 10.63.0.1 -p e0c -n ccluster1-02'
##&nbsp;
sub transition_lif_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_lif_modify
## @name ::transition_lif_remove
##
## @summary Parses the raw output from the "transition lif remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_lif_remove with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_lif_remove(%args);
## INPUTS TO METHOD CALL 'transition_lif_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;                  'parsed_output' => [
##&nbsp;                                       {
##&nbsp;                                         'error_count' => 0
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;                  'command' => 'transition lif remove -s session2 -i 10.63.58.116'
##&nbsp;                }, 'NACL::APISet::Response::CLI::Windows' );
##&nbsp;
sub transition_lif_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_lif_remove
## @name ::transition_lif_show
##
## @summary Parses the raw output from the "transition lif show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_lif_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_lif_show(%args);
## INPUTS TO METHOD CALL 'transition_lif_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;                  'parsed_output' => [
##&nbsp;									  {
##&nbsp;                                        'gateway' => '10.63.0.1',
##&nbsp;                                         'home_node' => 'ccluster1-02',
##&nbsp;                                         'ip_address' => '10.63.58.116',
##&nbsp;                                         'netmask' => '255.255.192.0',
##&nbsp;                                         'home_port' => 'e0c',
##&nbsp;                                         'lif_state' => 'Created'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'error_count' => 0
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;                  'command' => 'transition lif show -s session2 -i 10.63.58.116'
##&nbsp;                }, 'NACL::APISet::Response::CLI::Windows' );
##&nbsp;
sub transition_lif_show {
    my %args = @_;
    my ($error_message , @result_array, %hash , $resp , $output);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    if($output =~ m/LIF state/){
	$result_array=_row_type_parser(\%args);
	return $result_array;
    }else{
	my $error = _check_error(
            raw_output=>$output,
            error_message=>\$error_message,
            hash=>\%hash,
	);
	if (!$error){
	    if ($output =~ /No IP addresses to display/) {
		$hash{"output"} = "No IP addresses to display.";
	    }else {
		#remove all spaces or new line char from beginning of output
		$output =~ s/^(\s+|\n+)//;
		#divide output into array of lines
		my @lines = split(/\n/, $output);
		#first 2 lines needs to be ignored.
		#process remaining lines of table one by one
		foreach $line (@lines[2..(@lines-1)]) {
		#if line does not contain ip address then it is end of output
                    if ($line !~ /\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}/) {
                        last;
                    }
                    #remove leading and trailing spaces
                    $line = _formatter($line);
                    my @values = split(/\s+/,$line);
                    my %entry = (
			'ip_address'     => $values[0],
			'netmask'        => $values[1],
			'gateway'        => $values[2],
			'home_node'      => $values[3],
			'home_port'      => $values[4],
			'lif_status'     => "$values[5] $values[6] $values[7]",
                    );
                    push(@result_array, \%entry);
                }
            }
        }
        push (@result_array, \%hash);
        _error_exception(
            error_message=>\$error_message,
            error=>\$error,
            hash=>\%hash,
            result_array=>\@result_array,
            response_obj=>$resp,
        );
        return \@result_array;
    }
} ## end sub transition_lif_show

## @name ::transition_volumepair_add
##
## @summary Parses the raw output from the "transition_volumepair_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31248',
##&nbsp;                           'message' => 'Aggregate must not be provided along with the existing Cluster-Mode volume.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
sub transition_volumepair_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumepair_add

## @name ::transition_volumepair_modify
##
## @summary Parses the raw output from the "transition_volumepair_modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_modify with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_modify(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair modify -s session2 -v vol02 -c vol1234 -g aggr_trans :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = 'command' => 'transition volumepair modify -s session2 -v vol02 -c vol1234 -g aggr_trans'
##&nbsp;
sub transition_volumepair_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumepair_modify

## @name ::transition_volumepair_remove
##
## @summary Parses the raw output from the "transition_volumepair_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_remove with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair remove -s S1 -v vol02
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = 'command' => 'transition volumepair remove -s session2 -v vol02'
##&nbsp;
sub transition_volumepair_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumepair_remove

## @name ::transition_volumepair_show
##
## @summary Parses the raw output from the "transition_volumepair_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_show(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair show -s S1 -v vol02
##&nbsp;-------- Parsed Output ------
##&nbsp;                      $VAR1 = [
##&nbsp;                               {
##&nbsp;                                'Cluster-Mode Volume' => 'vol123',
##&nbsp;                                'Cluster-Mode volume status' => 'Created',
##&nbsp;                                '7-Mode Volume' => 'vol02',
##&nbsp;                                'Snapmirror state' => '-/-',
##&nbsp;                                'Aggregate' => 'aggr_trans'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                'error_count' => 0
##&nbsp;                               },
##&nbsp;                              ];
##&nbsp;  Command = 'command' => 'transition volumepair show -s session2 -v vol02'
##&nbsp;transition volumepair show -s session2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;                      $VAR1 = [
##&nbsp;                               {
##&nbsp;                                'current_operation' => '-',
##&nbsp;                                'snapmirror_state' => '-/-',
##&nbsp;                                '7_mode_volume' => 'vol02',
##&nbsp;                                'cluster_mode_vol' => 'vol123',
##&nbsp;                                'comments' => '-'
##&nbsp;                                },
##&nbsp;                               {
##&nbsp;                                'error_count' => 0
##&nbsp;                               },
##&nbsp;                              ];
##&nbsp;  Command = 'command' => 'transition volumepair show -s session2
##&nbsp;
##&nbsp;
sub transition_volumepair_show {
    my %args = @_;
    my ($error_message , @result_array, %hash , $resp , $output);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $comand = $resp->get_command();
    if($comand =~ m/\-v/){
	$result_array=_row_type_parser(\%args);
	return $result_array;
    }else{
	my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
	);
	if (!$error){
            if ($output =~ /No information to display./) {
		$hash{"info"} = "No information to display.";
            }elsif($comand !~ /\-v/) {
                #remove all spaces or new line char from beginning of output
		$output =~ s/^(\s+|\n+)//;
		#divide output into array of lines
		my @lines = split(/\n/, $output);
		#first 2 lines needs to be ignored.
		#process remaining lines of table one by one
		foreach $line (@lines[2..(@lines-1)]) {
                    #if line does not contain valid snapmirror status then
                    #it is end of output
                    if ($line !~ /\-\/\-/ && $line !~ /mirrored/ && $line !~ /idle/) {
			last;
                    }
                    #remove leading and trailing spaces of this line
                    $line =~ _formatter($line);
                    my @values = split(/\s+/,$line);
                    my %entry = (
			'7_mode_volume' => $values[0],
			'cluster_mode_vol'  => $values[1],
			'aggregate'     =>  $values[2],
			'snapmirror_state'  => $values[3],
			'cluster_mode_volume_status'    => "$values[4] $values[5] $values[6]",
                    );
                    push(@result_array, \%entry);
                }
	    }
        }
	push (@result_array, \%hash);
	_error_exception(
            error_message=>\$error_message,
            error=>\$error,
            hash=>\%hash,
            result_array=>\@result_array,
            response_obj=>$resp,
        );
	return \@result_array;
    }
} ## end sub transition_volumepair_show

## @name ::transition_export_lunmap
##
## @summary Parses the raw output from the "transition_export_lunmap" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_export_lunmap with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_export_lunmap(%args);
## INPUTS TO METHOD CALL 'transition_export_lunmap' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'output-file' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp; 'parsed_output' => [
##&nbsp;                     {
##&nbsp;                      'error_count' => 1,
##&nbsp;                        'error' => [
##&nbsp;                                    {
##&nbsp;                                    'code' => '5001',
##&nbsp;                                    'message' => 'Session must be completed for running the transition export-lun-map command'
##&nbsp;                                     }
##&nbsp;                                    ]
##&nbsp;                     }
##&nbsp;                    ],
##&nbsp;  'command' => 'transition export lunmap -s session2 -o /u/username/file.txt'
##&nbsp;'parsed_output' => [
##&nbsp;                     {
##&nbsp;                      'error_count' => 1,
##&nbsp;                       'error' => [
##&nbsp;                                  {
##&nbsp;                                   'code' => '5000',
##&nbsp;                               'message' => 'No LUNs found in the session'
##&nbsp;                                  }
##&nbsp;                                  ]
##&nbsp;                    }
##&nbsp;                  ],
##&nbsp;  'command' => 'transition export lunmap -s session2 -o /u/username/file.txt'
##&nbsp;
sub transition_export_lunmap {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_export_lunmap

## @name :: transition_add_primary_seven_mode_system
##
## @summary Parses the raw output from the "transition_add_primary_seven_mode_system" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_add_primary_seven_mode_system with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_add_primary_seven_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_add_primary_seven_mode_system' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'source-host' </td> <td> </td> '0' </tr>
## <tr> <td> '-f' </td> <td> 'management-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'data-copy-ipaddress-staggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition add-primary-seven-mode-system -s S1 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-primary-seven-mode-system -s S1 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110
##&nbsp;
##&nbsp;transition add-primary-seven-mode-system -s S5 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110 :
##&nbsp;Error(31115) : Transition session with name \'S5\' does not exist.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31115',
##&nbsp;                           'message' => 'Transition session with name \'S5\' does not exist.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-primary-seven-mode-system -s S5 -h 10.227.202.110 -f 10.227.202.110 -d 10.227.202.110
##&nbsp;

sub transition_add_primary_seven_mode_system{

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}## end transition_add_primary_seven_mode_system

## @name :: transition_add_secondary_cluster_mode_system
##
## @summary Parses the raw output from the "transition_add_secondary_cluster_mode_system" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_add_secondary_cluster_mode_system with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_add_secondary_cluster_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_secondary_cluster_mode_system' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vserver-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec
##&nbsp;
##&nbsp;transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec1 :
##&nbsp;Error(31283) : Specified Vserver \'vs_sec1\' does not exists on the Cluster-Mode storage system \'10.226.99.61\'.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31283',
##&nbsp;                           'message' => 'Specified Vserver \'vs_sec1\' does not exists on the Cluster-Mode storage system \'10.226.99.61\'.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-secondary-cluster-mode-system -s S1 -h 10.226.99.61 -v vs_sec1
##&nbsp;
sub transition_add_secondary_cluster_mode_system{

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}## end tranition_add_secondary_cluster_mode_system

## @name ::transition_precheck
##
## @summary Parses the raw output from the "transition_precheck" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_precheck with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition precheck -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'next_suggested_steps' => '1. Use the \'transition job-results -j 170c0cbe-c5f2-464e-969e-37ec1424ffa0\' command to see the results of this operation. 2. Carefully review the precheck operation results before proceeding with the next step. 3. Select \'Start data copy\' from the main menu or run the transition CLI command \'transition start -s S21\' when you are ready to start data copy to Cluster-Mode volumes.',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition precheck -r no -s S21
##&nbsp;
sub transition_precheck {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_precheck

## @name ::transition_start
##
## @summary Parses the raw output from the "transition_start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_start with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition start -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90051',
##&nbsp;                                   'message' => 'Creating the required clustered Data ONTAP volumes on the Vserver \'vs_sec\'.Successfully created the following clustered Data ONTAP volumes on the Vserver \'vs_sec\'.'
##&nbsp;                                 },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition start -r no -s S21
##&nbsp;
sub transition_start {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_start

## @name :: transition_schedule_add
##
## @summary Parses the raw output from the "transition_schedule_add" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_add with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_add(%args);
## INPUTS TO METHOD CALL 'transition_shedule_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> '7-Mode-host' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##@examples
##&nbsp;transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2
##&nbsp;
##&nbsp;transition schedule add -s SS3 -n sc4 -d 0 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;Error(31327) : Schedule \'sc4\' interferes with the schedule \'sc1\' which is already existing in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31327',
##&nbsp;                           'message' => 'Schedule \'sc4\' interferes with the schedule \'sc1\' which is already existing in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule add -s SS3 -n sc4 -d 0 -b 12:00 -e 10:00 -t 2 -x 2
##&nbsp;
sub transition_schedule_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_add

## @name :: transition_schedule_modify
##
## @summary Parses the raw output from the "transition_schedule_modify" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_modify with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_shedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> '7-Mode-host' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:30 -t 2 -x 2 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule modify -s S21 -n sc1-d 1-5 -b 12:00 -e 10:30 -t 2 -x 2
##&nbsp;
sub transition_schedule_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_modify

## @name :: transition_schedule_remove
##
## @summary Parses the raw output from the "transition_schedule_remove" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_schedule_remove with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_schedule_remove(%args);
## INPUTS TO METHOD CALL 'transition_shedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> '7-Mode-host' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;transition schedule remove -s S21 -n sc1 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule remove -s S21 -n sc1
##&nbsp;
##&nbsp;transition schedule remove -s S21 -n sc4 :
##&nbsp;Error(31328) : Schedule \'sc4\' is not available in the session \'S21\'.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31328',
##&nbsp;                           'message' => 'Schedule \'sc4\' is not available in the session \'S21\'.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule remove -s S21 -n sc4
##&nbsp;
sub transition_schedule_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_schedule_remove

## @name ::transition_pause
##
## @summary Parses the raw output from the "transition_pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_pause with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition pause -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = transition pause -r no -s S21
##&nbsp;
##&nbsp;
sub transition_pause {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_pause

## @name ::transition_resume
##
## @summary Parses the raw output from the "transition_resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_resume with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition resume -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90071',
##&nbsp;                                   'message' => 'Resuming data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume\'vol_compo1\'. Successfully resumed data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition resume -r no -s S21
##&nbsp;
sub transition_resume {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_resume

## @name :: transition_precutover
##
## @summary Parses the raw output from the "transition_precutover" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_precutover with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_precutover(%args);
## INPUTS TO METHOD CALL 'transition_precutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapmirror-throttle' <td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition precutover -r no -s gay_nearer_1392898178 -m rw_test -t 4 -b 240 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol1 }',
##&nbsp;                                   'code' => '70141',
##&nbsp;                                   'message' => 'Collecting space reservation information from the 7-Mode storage system\'vfiler0@10.238.24.232\'. Successfully collected space reservation information for the following 7-Mode volumes from the 7-Mode storage system \'vfiler0@10.238.24.232\'.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ nfs.response.trigger , nfs.udp.xfersize , nfs.v4.acl.max.aces , nfs.rpcsec.ctx.idle , nfs.tcp.xfersize , nfs.rpcsec.ctx.high , wafl.default_nt_user , nfs.v4.id.domain , nfs.mount_rootonly , nfs.response.trace , nfs.v3.enable , nfs.v4.acl.enable , nfs.v4.write_delegation , nfs.v4.snapshot.active.fsid.enable , nfs.v4.enable , nfs.tcp.enable , nfs.v4.read_delegation , nfs.vstorage.enable , nfs.udp.enable }',
##&nbsp;                                   'code' => '10361',
##&nbsp;                                   'message' => 'Collecting the NFS options settings from the 7-Mode storage system \'vfiler0@10.238.24.232\'.Successfully collected the following NFS options settings from the 7-Mode storage system \'vfiler0@10.238.24.232\'.'
##&nbsp;                                 },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '{ 10.238.24.232 santosh6-vsim1 santosh6-vsim1-e0c Error Code: 13130 Reason: duplicate entry , 127.0.10.1 localhost-10 localhost-bsd Error Code: 13130 Reason: duplicate entry , 127.0.0.1 localhost localhost-stack Error Code: 13130 Reason: duplicate entry , 127.0.20.1 localhost-20 localhost-sk Error Code: 13130 Reason: duplicate entry }',
##&nbsp;                             'code' => '40111',
##&nbsp;                             'message' => 'Applying the hosts configuration to the Vserver \'vs_trans\'.Failed to create the following hosts configuration on the Vserver \'vs_trans\'.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '{ vol_compo1 }',
##&nbsp;                             'code' => '92021',
##&nbsp;                             'message' => 'Resynchronizing the SnapMirror relationship between the 7-mode source volumes and the clusteredData ONTAP destination volumes. The SnapMirror resync operation for the following clustered Data ONTAP destination volumes are triggered successfully. The clustered Data ONTAP destination volumes will be available for read-only operations after the resync operation is completed successfully. This might take several minutes.'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition precutover -r no -s gay_nearer_1392898178 -m rw_test -t 4 -b 240
##&nbsp;
sub transition_precutover {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_precutover

## @name :: transition_resync
##
## @summary Parses the raw output from the "transition_resync" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_resync with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_resync(%args);
## INPUTS TO METHOD CALL 'transition_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition resync -r no -s charm_tulip_1392896242 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => 0,
##&nbsp;            'informational' => [],
##&nbsp;            'error_count' => 0,
##&nbsp;            'warning_count' => '1',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '92021',
##&nbsp;                             'message' => 'Resynchronizing the SnapMirror relationship between the 7-mode source volumes and the clusteredData ONTAP destination volumes. The SnapMirror resync operation for all the clustered Data ONTAP destination volumes is triggered successfully. The clustered Data ONTAP destination volumes will be available for read-only operations after the resync operation is completed successfully. This might take several minutes.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition resync -r no -s charm_tulip_1392896242
##&nbsp;

sub transition_resync {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_resync

## @name :: transition_update
##
## @summary Parses the raw output from the "transition_update" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_update with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_update(%args);
## INPUTS TO METHOD CALL 'transition_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition update -r no -s gay_nearer_1392898178 -t 4 -x 240 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => 0,
##&nbsp;            'informational' => [],
##&nbsp;            'error_count' => 0,
##&nbsp;            'warning_count' => '0',
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition update -r no -s gay_nearer_1392898178 -t 4 -x 240
##&nbsp;

sub transition_update {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_update

## @name :: transition_cutover
##
## @summary Parses the raw output from the "transition_cutover" command
##
## @description
##  This method can be used for parsing the raw output after calling
##  the method from transition_cutover with any of its
##  options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'offline-source-volumes' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;transition cutover -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90821',
##&nbsp;                                   'message' => 'Performing the final update transfer for the clustered Data ONTAP volumes. Final update succeeded for the following clustered Data ONTAP volumes.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '96801',
##&nbsp;                                   'message' => 'Taking all the 7-Mode volumes offline. Offlined all the 7-Mode volumes.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => '0',
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition cutover -r no -s S21
##&nbsp;
sub transition_cutover {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

}# end sub transition_cutover

## @name ::transition_abort
##
## @summary Parses the raw output from the "transition_abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_abort with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition abort -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '3',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 },
##&nbsp;                                {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90011',
##&nbsp;                                   'message' => 'Breaking the SnapMirror relationships. Successfully broken all SnapMirror relationships.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                  'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90021',
##&nbsp;                                   'message' => 'Deleting the SnapMirror relationships. Successfully deleted all SnapMirror relationships.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition abort -r no -s S21
##&nbsp;
sub transition_abort {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_abort

## @name ::transition_delete
##
## @summary Parses the raw output from the "transition_delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_delete with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition delete -s S21
##&nbsp;
sub transition_delete {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_delete

## @name ::transition_add_cluster_info
##
## @summary Parses the raw output from the "transition_add_cluster_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_add_cluster_info with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_add_cluster_info(%args);
## INPUTS TO METHOD CALL 'transition_add_cluster_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition add-cluster-info -s ses1 -h 10.63.57.180 -v vs1 -g aggr1:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition add-cluster-info -s ses1 -h 10.63.57.180 -v vs1
##&nbsp;            -g aggr1
##&nbsp;
sub transition_add_cluster_info {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_add_cluster_info

## @name ::transition_modify
##
## @summary Parses the raw output from the "transition_modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_modify with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_modify(%args);
## INPUTS TO METHOD CALL 'transition_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition modify -s ses1 -h 10.63.53.28 -c 10.63.53.28 -m 10.63.53.199
##&nbsp;            -v vs1 -g aggr1:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition modify -s ses1 -h 10.63.53.28 -c 10.63.53.28
##&nbsp;            -m 10.63.53.199 -v vs1 -g aggr1
##&nbsp;
sub transition_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_modify

## @name ::transition_property_set
##
## @summary Parses the raw output from the "transition_property_set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_property_set with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_property_set(%args);
## INPUTS TO METHOD CALL 'transition_property_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'property-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'property-value' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition property-set -s ses1 -p cluster-audit-saveas-path
##&nbsp;            -v /vol/vol1 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition property-set -s ses1 -p cluster-audit-saveas-path
##&nbsp;            -v /vol/vol1
##&nbsp;
sub transition_property_set {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_property_set

## @name ::transition_property_get
##
## @summary Parses the raw output from the "transition_property_get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_property_get with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_property_get(%args);
## INPUTS TO METHOD CALL 'transition_property_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'property-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition property-get -s ses1 -p cluster-audit-saveas-path:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;            'property_value' => '/vol/vol1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition property-get -s ses1 -p cluster-audit-saveas-path
##&nbsp;
sub transition_property_get {

    my %args = @_;
    my ($error_message , @result_array, %hash , $resp , $output);

    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array , \%hash);

    _error_exception(
            error_message=>\$error_message,
            error=>\$error,
            hash=>\%hash,
            result_array=>\@result_array,
            response_obj=>$resp,
        );

    my @lines = split(/\n/, $output);

    if ($lines[0] =~ /\/vol\//) {
        $lines[0] = _formatter($lines[0]);
        $hash{"property_value"} = $lines[0];
    }
    else {
        _invalid_output_exception($resp,\@result_array,\%hash,$error,
                                  $error_message);
    }

    return \@result_array;

} ## end sub transition_property_get

## @name ::transition_property_reset
##
## @summary Parses the raw output from the "transition_property_reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_property_reset with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_property_reset(%args);
## INPUTS TO METHOD CALL 'transition_property_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'property-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition property-reset -s ses1 -p cluster-audit-saveas-path:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition property-reset -s ses1 -p cluster-audit-saveas-path
##&nbsp;
sub transition_property_reset {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_property_reset

## @name ::transition_remove_primary_seven_mode_system
##
## @summary Parses the raw output from the
##    "transition_remove_primary_seven_mode_system" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_remove_primary_seven_mode_system with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_remove_primary_seven_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_remove_primary_seven_mode_system' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'source-host' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition remove-primary-seven-mode-system -s ses_secon
##&nbsp;            -h 10.63.54.196:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition remove-primary-seven-mode-system -s ses_secon
##&nbsp;            -h 10.63.54.196
##&nbsp;
sub transition_remove_primary_seven_mode_system {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_remove_primary_seven_mode_system

## @name ::transition_remove_secondary_cluster_mode_system
##
## @summary Parses the raw output from the
##    "transition_remove_secondary_cluster_mode_system" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_remove_secondary_cluster_mode_system with any
##    of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_remove_secondary_cluster_mode_system(%args);
## INPUTS TO METHOD CALL 'transition_remove_secondary_cluster_mode_system' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'secondary-svm' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition remove-secondary-cluster-mode-system -s ses_prim
##&nbsp;        -v vs_secondary :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition remove-secondary-cluster-mode-system -s ses_prim
##&nbsp;            -v vs_secondary
##&nbsp;
sub transition_remove_secondary_cluster_mode_system {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_remove_secondary_cluster_mode_system

## @name ::transition_bundle_tool_logs
##
## @summary Parses the raw output from the "transition_bundle_tool_logs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_bundle_tool_logs with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_bundle_tool_logs(%args);
## INPUTS TO METHOD CALL 'transition_bundle_tool_logs' ON APISET OBJECT
## Currently no input
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition bundle_tool_logs:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;            'info' => 'Successfully bundled tool logs on the server
##&nbsp;                        at the following location : C:\Program Files\
##&nbsp;                        NetApp\7-Mode Transition Tool\asup\
##&nbsp;                        7MTT2.0.0-20141126050939787.zip'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition bundle_tool_logs
##&nbsp;
sub transition_bundle_tool_logs {

    my %args = @_;
    my ($error_message , @result_array, %hash , $resp , $output);

    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array , \%hash);

    _error_exception(
            error_message=>\$error_message,
            error=>\$error,
            hash=>\%hash,
            result_array=>\@result_array,
            response_obj=>$resp,
        );

    if ($output =~ /INFO\s*\:s*(.*)/){
        my $info = $1;
        $info = _formatter($info);
        $hash{"info"} = $info;
    }
    else {
        _invalid_output_exception($resp,\@result_array,\%hash,
                                  $error,$error_message);
    }

    return \@result_array;

} ## end sub transition_bundle_tool_logs

## @name ::transition_show_primary_seven_mode_systems
##
## @summary Parses the raw output from the
##    "transition_show_primary_seven_mode_systems" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_show_primary_seven_mode_systems with any of
##    its options as described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_show_primary_seven_mode_systems(%args);
## INPUTS TO METHOD CALL 'transition_show_primary_seven_mode_systems' ON APISET
##  OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition show-primary-seven-mode-systems -s ses_secon :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'multipathing_ip_address' => '-',
##&nbsp;            'snapmirror_source_host' => '10.63.53.28',
##&nbsp;            'data_copy_ip_address' => '10.63.53.28',
##&nbsp;            'filer_ip_address' => '10.63.53.28'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show-primary-seven-mode-systems -s ses_secon
##&nbsp;
sub transition_show_primary_seven_mode_systems {

    my %args = @_;
    my $result_array=_table_type_parser(\%args);
    return $result_array;

} ## end sub transition_show_primary_seven_mode_systems

## @name ::transition_show_secondary_cluster_mode_systems
##
## @summary Parses the raw output from the
##    "transition_show_secondary_cluster_mode_systems" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_show_secondary_cluster_mode_systems with any
##    of its options as described in the corresponding cdef file
##    'transition.cdefs'
## @synopsis
##      transition_show_secondary_cluster_mode_systems(%args);
## INPUTS TO METHOD CALL 'transition_show_secondary_cluster_mode_systems' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition show-secondary-cluster-mode-systems -s ses_prim :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cluster-mode_host_name' => '10.63.57.150',
##&nbsp;            'svm' => 'vs_eng'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show-secondary-cluster-mode-systems -s ses_prim
##&nbsp;
sub transition_show_secondary_cluster_mode_systems {

    my %args = @_;
    my $result_array=_table_type_parser(\%args);
    return $result_array;

} ## end sub transition_show_secondary_cluster_mode_systems

## @name ::transition_job_status
##
## @summary Parses the raw output from the "transition_job_status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_job_status with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_job_status(%args);
## INPUTS TO METHOD CALL 'transition_job_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-j' </td> <td> 'job-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition job-status -j 4853f6ab-6c88-4704-8637-ce2542ba229a :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'end_time' => '12-05-2014 06',
##&nbsp;            'job_id' => '36290753-3209-4e5f-a80b-80a69db6a5e2',
##&nbsp;            'job_type' => 'start',
##&nbsp;            'start_time' => '12-05-2014 06',
##&nbsp;            'operation_status' => 'Failed',
##&nbsp;            'job_status' => 'completed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition job-status -j 4853f6ab-6c88-4704-8637-ce2542ba229a
##&nbsp;
sub transition_job_status {

    my %args = @_;
    my $result_array=_row_type_parser(\%args);
    return $result_array;

} ## end sub transition_job_status

## @name ::transition_jobs
##
## @summary Parses the raw output from the "transition_jobs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_jobs with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_jobs(%args);
## INPUTS TO METHOD CALL 'transition_jobs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-j' </td> <td> 'job-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition jobs -s ses1 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;                'error_count' => 0
##&nbsp;            },
##&nbsp;            {
##&nbsp;                'end_time' => '12-05-2014 06:46:03 EST',
##&nbsp;                'start_time' => '12-05-2014 06:45:59 EST',
##&nbsp;                'job_status' => 'completed',
##&nbsp;                'operation' => 'start',
##&nbsp;                'op_status' => 'Failed',
##&nbsp;                'unique_job_identifier' => '1850e869-b031-4bc6-8495-aa7b8c95e31c'
##&nbsp;            },
##&nbsp;            {
##&nbsp;                'end_time' => '12-05-2014 06:45:54 EST',
##&nbsp;                'start_time' => '12-05-2014 06:45:49 EST',
##&nbsp;                'job_status' => 'completed',
##&nbsp;                'operation' => 'start',
##&nbsp;                'op_status' => 'Failed',
##&nbsp;                'unique_job_identifier' => '832aa494-e4c3-4bc6-84aa-47bff25469af'
##&nbsp;            },
##&nbsp;            {
##&nbsp;                'end_time' => '12-05-2014 06:45:44 EST',
##&nbsp;                'start_time' => '12-05-2014 06:45:43 EST',
##&nbsp;                'job_status' => 'completed',
##&nbsp;                'operation' => 'start',
##&nbsp;                'op_status' => 'Failed',
##&nbsp;                'unique_job_identifier' => '1963709d-8dad-44fd-aa58-025ff59ae1c6'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition jobs -s ses1
##&nbsp;
sub transition_jobs {

    my %args = @_;
    my ($error_message , @result_array , %hash , $resp , $output);

    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);

    _error_exception(
            error_message=>\$error_message,
            error=>\$error,
            hash=>\%hash,
            result_array=>\@result_array,
            response_obj=>$resp,
        );

    if ($output =~ /No information to display./) {
        $hash{"info"} = "No information to display.";
    }
    else {
        #remove all spaces or new line char from begining of output
        $output =~ s/^(\s+|\n+)//;
        #devide output into array of lines
        my @lines = split(/\n/, $output);
        #first 2 lines needs to be ignored.
        #process remaning lines of table one by one
        foreach $line (@lines[2..(@lines-1)]) {
            #if line does not contain valid job id then it is end of output
            if ($line !~ /\w{8}\-\w{4}\-\w{4}\-\w{4}\-\w{12}/) {
                last;
            }
            #remove leading and trailing spaces of this line
            $line = _formatter($line);

            my @values = split(/\s+/,$line);
            my %entry = (
                            'unique_job_identifier'     => $values[0],
                            'operation'                 => $values[1],
                            'job_status'                => $values[2],
                            'op_status'                 => $values[3],
                            'start_time'                => "$values[4] ".
                                                            "$values[5] ".
                                                            "$values[6]",
                            'end_time'                  => "$values[7] ".
                                                            "$values[8] ".
                                                            "$values[9]",
            );
            push(@result_array, \%entry);
        }
    }

    return \@result_array;

} ## end sub transition_jobs

## @name ::transition_job_results
##
## @summary Parses the raw output from the "transition_job_results" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_job_results with any of its options as
##    described in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_job_results(%args);
## INPUTS TO METHOD CALL 'transition_job_results' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-j' </td> <td> 'job-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition job-results -j 4853f6ab-6c88-4704-8637-ce2542ba229a :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;           {
##&nbsp;             'informational_count' => '1',
##&nbsp;             'informational' => [
##&nbsp;                                  {
##&nbsp;                                    'corrective_action' => 'LIFs need to
##&nbsp;                                       be transitioned manually. For
##&nbsp;                                       more information, see the 7-Mode
##&nbsp;                                       Transition Tool Data and
##&nbsp;                                       Configuration Transition Guide.',
##&nbsp;                                    'affected_objects' => '',
##&nbsp;                                    'code' => '99162',
##&nbsp;                                                   information is added
##&nbsp;                                                   to the session
##&nbsp;                                                   \'ses1\'.Data LIFs
##&nbsp;                                                   are not added to the
##&nbsp;                                                   session \'ses1\'.'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;             'error_count' => '2',
##&nbsp;             'warning_count' => '8',
##&nbsp;             'warning' => [
##&nbsp;                            {
##&nbsp;                              'corrective_action' => 'Manually
##&nbsp;                                transition the SNMP configuration to the
##&nbsp;                                clustered Data ONTAP storage system.
##&nbsp;                                For more information, refer to the
##&nbsp;                                Clustered Data ONTAP Network Management
##&nbsp;                                Guide.',
##&nbsp;                              'affected_objects' => '',
##&nbsp;                              'code' => '94101',
##&nbsp;                              'message' => 'Checking whether SNMP is
##&nbsp;                                            enabled on the 7-Mode
##&nbsp;                                            storage system \'10.63.53.28
##&nbsp;                                            \'.SNMP is enabled on the
##&nbsp;                                            7-Mode storage system \'10.
##&nbsp;                                            63.53.28\'. However, 7-Mode
##&nbsp;                                            Transition Tool does not
##&nbsp;                                            support the transition of
##&nbsp;                                            SNMP configuration.'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'corrective_action' => 'The following
##&nbsp;                                roles must be manually transitioned to
##&nbsp;                                the SVM. For more information about how
##&nbsp;                                to configure the roles, refer to the
##&nbsp;                                Clustered Data ONTAP System
##&nbsp;                                Administration Guide for Cluster
##&nbsp;                                Administrators.',
##&nbsp;                              'affected_objects' => '{ audit , backup ,
##&nbsp;                                root , power , admin , compliance ,
##&nbsp;                                none }',
##&nbsp;                              'code' => '94001',
##&nbsp;                              'message' => 'Checking whether any roles
##&nbsp;                                are defined on the 7-Mode storage system
##&nbsp;                                \'10.63.53.28\'.The following roles are
##&nbsp;                                defined on the 7-Mode storage system
##&nbsp;                                \'10.63.53.28\'. However, 7-Mode
##&nbsp;                                Transition Tool does not support the
##&nbsp;                                transition of roles.'
##&nbsp;                            },
##&nbsp;                          ],
##&nbsp;             'error' => [
##&nbsp;                          {
##&nbsp;                            'corrective_action' => 'Add one or more
##&nbsp;                                7-Mode volumes to the session \'ses1\'.
##&nbsp;                                    For more information, see the 7-Mode
##&nbsp;                                    Transition Tool Data and
##&nbsp;                                    Configuration Transition Guide.',
##&nbsp;                            'affected_objects' => '',
##&nbsp;                            'code' => '99122',
##&nbsp;                            'message' => 'Checking whether one or more
##&nbsp;                                    7-Mode volumes are added to the
##&nbsp;                                    session \'ses1\'.No 7-Mode volumes
##&nbsp;                                    are added to the session \'ses1\'.'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'corrective_action' => 'Enter the
##&nbsp;                                    information of the clustered Data
##&nbsp;                                    ONTAP volumes to the session
##&nbsp;                                    \'ses1\'. For more information,
##&nbsp;                                    see the 7-Mode Transition Tool Data
##&nbsp;                                    and Configuration
##&nbsp;                                    Transition Guide.',
##&nbsp;                            'affected_objects' => '',
##&nbsp;                            'code' => '99142',
##&nbsp;                            'message' => 'Checking whether the
##&nbsp;                                    information of the clustered Data
##&nbsp;                                    ONTAP volumes is added to the
##&nbsp;                                    session\'ses1\'. Information of
##&nbsp;                                    the clustered Data ONTAP volumes
##&nbsp;                                    is not added to the
##&nbsp;                                    session \'ses1\'.'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition job-results -j 4853f6ab-6c88-4704-8637-ce2542ba229a
##&nbsp;
sub transition_job_results {

    my %args = @_;

    my ($result_array , %hash , $resp , $output);

    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    if ($output =~ /Operation summary/i) {
        eval {
            $result_array=_precheck_type_parser(\%args);
        };
        $result_array = $resp->get_parsed_output();
    }
    else {
        $result_array=_no_precheck_type_parser(\%args);
    }

    return $result_array;

} ## end sub transition_job_results

## @name ::transition_qtreepair_add
##
## @summary Parses the raw output from the
##    "transition_qtreepair_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_qtreepair_add with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_qtreepair_add(%args);
## INPUTS TO METHOD CALL 'transition_qtreepair_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'qtree-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'qtree-name-c-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition qtreepair add -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;            -c vol_trans1 -d qtree1_vol_trans1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair add -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;            -c vol_trans1 -d qtree1_vol_trans1
##&nbsp;
##&nbsp;transition qtreepair add -s S1 -v vol_trans1 -t qtree1 -c vol_trans1 -d qtree1_vol_trans1 :
##&nbsp;Error(31347) : Qtree \'qtree1\' belonging to the volume \'vol_trans1\' does not exist on the 7-Mode system \'vfiler0@10.226.151.38\'.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31347',
##&nbsp;                           'message' => 'Qtree \'qtree1\' belonging to the volume \'vol_trans1\' does not exist on the 7-Mode system \'vfiler0@10.226.151.38\'.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair add -s S1 -v vol_trans1 -t qtree1 -c vol_trans1 -d qtree1_vol_trans1
##&nbsp;
sub transition_qtreepair_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_qtreepair_add

## @name ::transition_qtreepair_modify
##
## @summary Parses the raw output from the
##    "transition_qtreepair_modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_qtreepair_modify with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_qtreepair_modify(%args);
## INPUTS TO METHOD CALL 'transition_qtreepair_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'qtree-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'qtree-name-c-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition qtreepair modify -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;            -c vol_trans1modified -d qtree2_vol_trans1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair modify -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;            -c vol_trans1modified -d qtree2_vol_trans1
##&nbsp;
##&nbsp;transition qtreepair modify -s S1 -v vol1 -t qtree1_vol_trans1 -c vol_trans1modified -d qtree2_vol_trans1 :
##&nbsp;Error(21500) : Qtree pair mapping is not configured for the source volume \'vol1\' and source qtree \'qtree1_vol_trans1\' in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '21500',
##&nbsp;                           'message' => 'Qtree pair mapping is not configured for the source volume \'vol1\' and source qtree \'qtree1_vol_trans1\' in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair modify -s S1 -v vol1 -t qtree1_vol_trans1 -c vol_trans1modified -d qtree2_vol_trans1
##&nbsp;
sub transition_qtreepair_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_qtreepair_modify

## @name ::transition_qtreepair_remove
##
## @summary Parses the raw output from the
##    "transition_qtreepair_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_qtreepair_remove with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_qtreepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_qtreepair_remove' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'qtree-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition qtreepair remove -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair remove -s S1 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;
##&nbsp;transition qtreepair remove -s S2 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;Error(31115) : Transition session with name \'S2\' does not exist.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31115',
##&nbsp;                           'message' => 'Transition session with name \'S2\' does not exist.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair remove -s S2 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;

sub transition_qtreepair_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_qtrepair_remove

## @name ::transition_stagingsystems_add
##
## @summary Parses the raw output from the
##    "transition_stagingsystems_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_stagingsystems_add with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_stagingsystems_add(%args);
## INPUTS TO METHOD CALL 'transition_stagingsystems_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'aggregates' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'datacopy-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipath-ip-address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition stagingsystems add -s S1 -h 10.226.151.40 -v vfiler0
##&nbsp;            -a aggr_trans -d 10.226.151.40
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition stagingsystems add -s S1 -h 10.226.151.40 -v vfiler0
##&nbsp;            -a aggr_trans -d 10.226.151.40
##&nbsp;
##&nbsp;transition qtreepair remove -s S2 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;Error(31115) : Transition session with name \'S2\' does not exist.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31115',
##&nbsp;                           'message' => 'Transition session with name \'S2\' does not exist.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition qtreepair remove -s S2 -v vol_trans1 -t qtree1_vol_trans1
##&nbsp;

sub transition_stagingsystems_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_stagingsystems_add

## @name ::transition_stagingsystems_modify
##
## @summary Parses the raw output from the
##    "transition_stagingsystems_modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_stagingsystems_modify with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_stagingsystems_modify(%args);
## INPUTS TO METHOD CALL 'transition_stagingsystems_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'aggregates' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'datacopy-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipath-ip-address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition stagingsystems modify -s S1 -h 10.226.151.40 -v vfiler1
##&nbsp;            -a aggr_trans_raid_dp_1 -d 10.226.151.40
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition stagingsystems modify -s S1 -h 10.226.151.40 -v vfiler1
##&nbsp;            -a aggr_trans_raid_dp_1 -d 10.226.151.40
##&nbsp;
##&nbsp;transition stagingsystems modify -s S1 -h 10.226.151.40 -v vfiler2 -a aggr_trans_raid_dp_1 -d 10.226.151.40
##&nbsp;Error(31282) : Provided vFiler \'vfiler2\' does not exist.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31282',
##&nbsp;                           'message' => 'Provided vFiler \'vfiler2\' does not exist.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition stagingsystems modify -s S1 -h 10.226.151.40 -v vfiler2 -a aggr_trans_raid_dp_1 -d 10.226.151.40
##&nbsp;
sub transition_stagingsystems_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_stagingsystems_modify

## @name ::transition_stagingsystems_remove
##
## @summary Parses the raw output from the
##    "transition_stagingsystems_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_stagingsystems_remove with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_stagingsystems_remove(%args);
## INPUTS TO METHOD CALL 'transition_stagingsystems_remove' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'source-host' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition stagingsystems remove -s S1 -h 10.226.151.40
##&nbsp;            -a aggr_trans -d 10.226.151.40
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition stagingsystems remove -s S1 -h 10.226.151.40
##&nbsp;
##&nbsp;transition stagingsystems remove -s S1 -h 10.235.126.34
##&nbsp;Error(31341) : Staging system information for the specified host name \'10.235.126.34\' is not configured in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31341',
##&nbsp;                           'message' => 'Staging system information for the specified host name \'10.235.126.34\' is not configured in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition stagingsystems remove -s S1 -h 10.235.126.34
##&nbsp;
sub transition_stagingsystems_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_stagingsystems_remove

## @name ::transition_volumeaggregatemapping_add
##
## @summary Parses the raw output from the
##    "transition_volumeaggregatemapping_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumeaggregatemapping_add with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumeaggregatemapping_add(%args);
## INPUTS TO METHOD CALL 'transition_stagingsystems_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'target-cluster-aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'volume-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'staging-system-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'staging-aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumeaggregatemapping add -s S1 -c vol_trans1 -g aggr_trans
##&nbsp;            -h 10.226.151.40 -a aggr_trans
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumeaggregatemapping add -s S1 -c vol_trans1 -g aggr_trans
##&nbsp;            -h 10.226.151.40 -a aggr_trans
##&nbsp;
##&nbsp;transition volumeaggregatemapping add -s S1 -c vol_trans2 -g aggr_trans -h 10.226.151.40 -a aggr_trans
##&nbsp;Error(31350) : Qtree mapping information for the clustered Data ONTAP volume vol_trans2 is not configured in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31350',
##&nbsp;                           'message' => 'Qtree mapping information for the clustered Data ONTAP volume vol_trans2 is not configured in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumeaggregatemapping add -s S1 -c vol_trans2 -g aggr_trans -h 10.226.151.40 -a aggr_trans
##&nbsp;
sub transition_volumeaggregatemapping_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumeaggregatemapping_add

## @name ::transition_volumeaggregatemapping_modify
##
## @summary Parses the raw output from the
##    "transition_volumeaggregatemapping_modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumeaggregatemapping_modify with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumeaggregatemapping_modify(%args);
## INPUTS TO METHOD CALL 'transition_stagingsystems_add' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'target-cluster-aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'volume-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'staging-system-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'staging-aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumeaggregatemapping modify -s S1 -c vol_trans1 -g aggr_trans
##&nbsp;             -h 10.226.151.40 -a aggr_trans_raid_dp_1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumeaggregatemapping modify -s S1 -c vol_trans1 -g aggr_trans
##&nbsp;             -h 10.226.151.40 -a aggr_trans_raid_dp_1
##&nbsp;
##&nbsp;transition volumeaggregatemapping add -s S1 -c vol_trans2 -g aggr_trans -h 10.226.151.40 -a aggr_trans
##&nbsp;Error(31350) : Qtree mapping information for the clustered Data ONTAP volume vol_trans2 is not configured in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31354',
##&nbsp;                           'message' => 'Aggregate name aggr_trans_raid_dp_2 specified is not configured as part of the staging system information for the host 10.226.151.40 in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumeaggregatemapping modify -s S1 -c vol_trans1 -g aggr_trans -h 10.226.151.40 -a aggr_trans_raid_dp_2
##&nbsp;
sub transition_volumeaggregatemapping_modify {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumeaggregatemapping_modify

## @name ::transition_volumeaggregatemapping_remove
##
## @summary Parses the raw output from the
##    "transition_volumeaggregatemapping_remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumeaggregatemapping_remove with any of
##    its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumeaggregatemapping_remove(%args);
## INPUTS TO METHOD CALL 'transition_volumeaggregatemapping_remove' ON
##  APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumeaggregatemapping remove -s S1 -c vol_trans1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumeaggregatemapping remove -s S1 -c vol_trans1
##&nbsp;
##&nbsp;transition volumeaggregatemapping remove -s S1 -c vol1
##&nbsp;Error(31351) : Aggregate mapping information for the clustered Data ONTAP volume vol1 is not configured in the session.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31351',
##&nbsp;                           'message' => 'Aggregate mapping information for the clustered Data ONTAP volume vol1 is not configured in the session.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = Error(31351) : Aggregate mapping information for the clustered Data ONTAP volume vol1 is not configured in the session.
##&nbsp;
sub transition_volumeaggregatemapping_remove {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumeaggregatemapping_remove

## @name ::transition_show
##
## @summary Parses the raw output from the "transition_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_show(%args);
## INPUTS TO METHOD CALL 'transition_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition show -r no -s secondary :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;            'vserver_comment' => 'Pre-existing',
##&nbsp;            'multipath_ip_address' => '-',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.108',
##&nbsp;            'data_copy_ip_address' => '10.227.65.87',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'snapmirror_status' => [
##&nbsp;                {
##&nbsp;                'snapmirror_state' => '-/-',
##&nbsp;                '7_mode_volume' => 'dst',
##&nbsp;                'cluster_mode_volume' => 'dst',
##&nbsp;                'comments' => '-'
##&nbsp;                }
##&nbsp;            ],
##&nbsp;            'schedule_info' => [
##&nbsp;                {
##&nbsp;                'update_frequency' => '00:30',
##&nbsp;                'streams_count' => '10',
##&nbsp;                'days_range' => '0',
##&nbsp;                'throttle' => '-1',
##&nbsp;                'start_time' => '01:00',
##&nbsp;                'schedule_name' => 'secondary_schedule',
##&nbsp;                'duration' => '02:00'
##&nbsp;                }
##&nbsp;            ],
##&nbsp;            'lif_info' => [
##&nbsp;                 {
##&nbsp;                 'home_node' => 'sti3270-020',
##&nbsp;                 'gateway' => '10.72.219.1',
##&nbsp;                 'lif_status' => 'Created',
##&nbsp;                 'netmask' => '255.255.255.0',
##&nbsp;                 'ip_address' => '10.72.219.200',
##&nbsp;                 'home_port' => 'e1b'
##&nbsp;                 }
##&nbsp;             ],
##&nbsp;             '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;             'snapmirror_schedule' => '-',
##&nbsp;             'session_type' => 'secondary',
##&nbsp;             'version' => '1.3.0',
##&nbsp;             'session_name' => 'secondary',
##&nbsp;             'apply_configuration' => 'Yes',
##&nbsp;             'session_state' => 'Completed',
##&nbsp;             'primary_7_mode_details' => '10.227.66.88:10.227.66.88:10.227.66.88:-',
##&nbsp;             'cluster_mode_system_version' => 'NetApp Release SierraNevada__8.2.2 Cluster-Mode: Sat Jan 25 15:07:32 PST 2014',
##&nbsp;             '7_mode_version' => 'NetApp Release R8.2.1xN_131112_1507 7-Mode: Tue Nov 12 15:56:49 PST 2013',
##&nbsp;             'vserver_name' => 'vsec',
##&nbsp;             'session_status' => '-'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no -s secondary
##&nbsp;
##&nbsp;transition show -r no:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.123',
##&nbsp;            'vserver_name' => 'vserver4',
##&nbsp;            'session_status' => '-',
##&nbsp;            'session_name' => 'cifssession',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'session_state' => 'Completed'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.65.87',
##&nbsp;            'cluster_mode_system_fqdn' => '10.229.180.123',
##&nbsp;            'vserver_name' => 'vserver4',
##&nbsp;            'session_status' => '-',
##&nbsp;            'session_name' => 'cifssession2',
##&nbsp;            'vfiler_unit' => 'vfiler0',
##&nbsp;            'session_state' => 'Aborted'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no
##&nbsp;
##&nbsp;
##&nbsp;
sub transition_show {
    my %args = @_;
    my ($first_line, $vol7mode, $volcmode,  $snapmirror,
        $comments,   $temp,     $aggregate, $cmodestatus,
        %hash,       %entry,    %comment,   $error_message,
        $current_operation,
    );
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    my $result_ref=[];
    if (!$error){
        if ($output =~ /Apply configuration/){

            #Let us remove the schedule part and parse it separately
            my ( $schedule, @schedule_arr, @temp1);
            if($output=~s/(.*)(Name\s\s+Days Range\s\s+Start Time.*)(7-Mode volume\s\s+Cluster-Mode volume.*)/$1$3/s){
                $schedule=$2;
            }
            if(defined $schedule){
                $schedule=~s/Name\s+Days Range\s+Start Time\s+Duration\s+% of Available  Maximum Cap   Throttle  Update Frequency.*----------//s;
                @temp1=split(/\s+/,$schedule);
                foreach (0..@temp1-1){
                    splice(@temp1, $_, 1) if @temp1[$_] eq '';
                }
                foreach (0..@temp1-1){
                    next if ($temp1[$_] eq "Available");
                    next if ($temp1[$_] eq "MBps");
                    push (@schedule_arr,$temp1[$_]);
                    
                }
                for(my $i=0; $i<@schedule_arr;){
                    push(
                        @{ $hash{schedule_info} },
                        {   'schedule_name'             => $schedule_arr[$i],
                            'days_range'                => $schedule_arr[$i+1],
                            'start_time'                => $schedule_arr[$i+2],
                            'duration'                  => $schedule_arr[$i+3],
                            'percentage_of_available'   => $schedule_arr[$i+4],
                            'maximum_cap'               => $schedule_arr[$i+5],
                            'throttle'                  => $schedule_arr[$i+6],
                            'update_frequency'          => $schedule_arr[$i+7],
                        }
                    );
                    $i=$i+8;
                }
            }

            #Let us remove LIF information and parse it separately
            my ( $lif, @lif_arr );
            if($output=~s/(.*)(IP address\s\s+Netmask\s\s+Gateway.*)/$1/s){
                $lif=$2;
            }
            if(defined $lif){
                $lif=~s/IP address\s+Netmask.*----------//s;
                @lif_arr=split(/\s\s+/,$lif);
                foreach (0..@lif_arr-1){
                    splice(@lif_arr, $_, 1) if @lif_arr[$_] eq '';
                }
                for(my $i=0; $i<@lif_arr;){
                    push(
                        @{ $hash{lif_info} },
                        {   'ip_address'       => $lif_arr[$i],
                            'netmask'          => $lif_arr[$i+1],
                            'gateway'          => $lif_arr[$i+2],
                            'home_node'        => $lif_arr[$i+3],
                            'home_port'        => $lif_arr[$i+4],
                            'lif_status'       => $lif_arr[$i+5],
                        }
                    );
                    $i=$i+6;
                }
            }
            my @summary_lines = get_lines(
                raw_output      => $output,
                response_object => $resp,
                first_line      => "SUMMARY:"
            );

            #remove summary lines from the output
            foreach my $line (@summary_lines) {
                $output =~ s/\Q$line\E//g;
            }
            if ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Current Operation\s+Comments/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Current Operation\s+Comments';
            } elsif ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status';
            }
            #grep snapmirror status lines from output and parse the fields
            $output =~ m/$first_line(.*)/si;
            my $snapmirror_string = $1 if $1;
            my @snapmirror_status_lines = split( "\n", $snapmirror_string );
            foreach my $line (@snapmirror_status_lines) {
                next if $line =~ /^\s+$/;
                next if $line =~ /^--/;
                next if $line eq '';
                if ( $line =~ /(^\w+)/ ) {
                    $temp = $1;
                    $entry{$temp} = $line;
                } elsif ( $line =~ /^\s+/ ) {
                    $entry{$temp} = $entry{$temp} . $line;
                }
            }
            foreach my $en ( keys(%entry) ) {
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Current Operation\s+Comments'
                    )
                {
                    ( $vol7mode, $volcmode, $snapmirror, $current_operation, $comments )
                        = split( /\s+/, $entry{$en}, 5 );
                    if (    defined $vol7mode
                        and defined $volcmode
                        and defined $snapmirror
                        and defined $current_operation
                        and defined $comments )
                    {
                        #remove extra spaces from comments field
                        $comments =~ s/\s{2,}//g;
                        $comments =~ s/\*/ */g;
                        push(
                            @{ $hash{snapmirror_status} },
                            {   '7_mode_volume'       => $vol7mode,
                                'cluster_mode_volume' => $volcmode,
                                'snapmirror_state'    => $snapmirror,
                                'current_operation'   => $current_operation,
                                'comments'            => $comments
                            }
                        ) if ( $vol7mode || $volcmode || $snapmirror || $current_operation || $comments);
                    }
                }
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status'
                    )
                {
                    ( $vol7mode, $volcmode, $aggregate, $snapmirror, $cmodestatus )
                        = split( /\s+/, $entry{$en} );
                    push(
                        @{ $hash{snapmirror_status} },
                        {   '7_mode_volume'              => $vol7mode,
                            'cluster_mode_volume'        => $volcmode,
                            'snapmirror_state'           => $snapmirror,
                            'aggregate'                  => $aggregate,
                            'cluster_mode_volume_status' => $cmodestatus
                        }
                    ) if ( $vol7mode || $volcmode || $snapmirror
                        || $comments || $aggregate );
                }
            }

            #remove snapmirror status lines from output
            $output =~ s/($first_line.*)//s if $first_line;

            #parse rest of the show output
            my @lines = get_lines(
                raw_output      => $output,
                response_object => $resp
            );
            foreach my $line (@lines) {
                if ( $line !~ /----------/ ) {
                    my ( $key, $val ) = split( ':', $line, 2 );
                    trim( string_ptr => \$key );
                    trim( string_ptr => \$val );
                    $key =~ s/\s/_/g;
                    if(_upper_to_lower($key) eq "vserver_information"){
                        ($hash{"vserver_name"},$hash{vserver_comment})=split(/:/,$val);
                    }else{
                        $hash{_upper_to_lower($key)} = $val;
                    }
                }
            }
        }elsif($output =~ /7-Mode system/){
            my (@arr,%chunk_hash,@new_lines);
            @lines=split(/\n/,$output);
            my $i=0;
            foreach my $line (@lines){
                next if($line =~ /Name\s+Type/ || $line =~ /-----/ || $line !~ /\w+/) ;
                $new_lines[$i]=$line;
                $i++;
            }
            my @data;
            foreach my $line (@new_lines){
                push(@data,split(/\s+/,$line));
            }
            foreach (0..@data-1){
                splice(@data, $_, 1) if $data[$_] eq '';
            }
            for(my $i=0; $i<@data;){
                $chunk_hash{"session_name"}=$data[$i];
                $chunk_hash{"session_type"}=$data[$i+1];
                ($chunk_hash{"7_mode_system_fqdn"},$chunk_hash{"vfiler_unit"})
                    =split(/:/,$data[$i+2]);
                ($chunk_hash{"cluster_mode_system_fqdn"},$chunk_hash{"vserver_name"})
                    =split(/:/,$data[$i+3]);
                $chunk_hash{"session_state"}=$data[$i+4];
                $chunk_hash{"session_status"}=$data[$i+5];
                push @$result_ref , {%chunk_hash} ;
                $i=$i+6;
            }
        }
    }

    push @{$result_ref} , {%hash};

    if($error) {
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }
    return $result_ref;

} ## end sub transition_show

## @name ::transition_schedule_show
##
## @summary Parses the raw output from the "transition_schedule_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_schedule_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_schedule_show(%args);
## INPUTS TO METHOD CALL 'transition_schedule_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition schedule show -s Arragon_languished_1392735210 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '0',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse1',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '1',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse2',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule show -s Arragon_languished_1392735210
##&nbsp;
##&nbsp;transition schedule show -s Arragon_languished_1392735210 -n Betelgeuse1:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'update_frequency' => '01:30',
##&nbsp;            'days_range' => '0',
##&nbsp;            'streams_count' => '4',
##&nbsp;            'schedule_name' => 'Betelgeuse1',
##&nbsp;            'start_time' => '09:00',
##&nbsp;            'throttle' => '240',
##&nbsp;            'duration' => '09:00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition schedule show -s Arragon_languished_1392735210 -n Betelgeuse1
##&nbsp;
##&nbsp;
##&nbsp;
sub transition_schedule_show {

    my %args = @_;
    my %hash ;
    my $result_ref=[];
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    if (!$error){
        my ( @schedule_arr);
        $output=~s/Name\s+Days Range\s+Start Time\s+Duration\s+Streams Count.*----------//s;
        @schedule_arr=split(/\s+/,$output);
        foreach (0..@schedule_arr-1){
            splice(@schedule_arr, $_, 1) if @schedule_arr[$_] eq '';
        }
        for(my $i=0; $i<@schedule_arr;){
            my %chunk_hash = (
                'schedule_name'       => $schedule_arr[$i],
                'days_range'          => $schedule_arr[$i+1],
                'start_time'          => $schedule_arr[$i+2],
                'duration'            => $schedule_arr[$i+3],
                'streams_count'       => $schedule_arr[$i+4],
                'throttle'            => $schedule_arr[$i+5],
                'update_frequency'    => $schedule_arr[$i+6],
            );
            $i=$i+7;
            push @{$result_ref} , {%chunk_hash};
        }
    }
    push @{$result_ref} , {%hash};

    if($error) {
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return $result_ref;

} ## end sub transition_schedule_show

sub _upper_to_lower{

    my $string=shift;
    $string =~ tr/A-Z/a-z/;
    $string =~ s/-/_/;
    return $string;

} ## end sub _upper_to_lower

sub _formatter{

    my $string=shift;
    $string=~s/^\s+|\s+$//g;
    return $string;

} ## end sub _formatter

sub _get_count{

    my $string=shift;
    my $var;
    $var=$2 if shift =~ /((\d+)\s+$string)/;
    if (!defined $var){
        $var=0;
    }
    return $var;

} ## end sub _get_count

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $hash_ref=$args{hash};
    my %hash=%{$hash_ref};
    my $error;
    if($output =~ /No value specified for .*/) {
        $error = "MissingRequiredParam";
    } elsif ($output =~ /The .* command takes .* arguments. You gave .*.  Extra arguments: .*/) {
        $error = "InvalidParam";
    } elsif ($output =~ /Transition session with name .* does not exist/ ||
        $output =~ /The session name .* is invalid. Special characters .* are not allowed in the session name/ ||
        $output =~ /Job with ID.*does not exist./ ||
        $output =~ /.* storage system volume .*' already used in the transition session .*/){
        $error = "InvalidParamValue";
    } elsif ($output =~ /(Operation summary.*?(\d+)\s+Error.*)/is && $2>0
            || $output =~ /.*Failed to run the.*/
            || $output =~ /Transition session.*status is not.*/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(SYNOPSIS)/ || $output =~ /session with name .* already exists/
        || $output =~ /Aggregate must not be provided along with the existing Cluster-Mode volume/
        || $output =~ /.*must be provided.*]/
        || $output =~ /Failed to communicate with 7-Mode Transition Tool server/
        || $output =~ /.*not recognized as an internal or external command.*/
        || $output =~ /Host .* not found in credential bank/
        || $output =~ /There are no sessions/
        || $output =~ /Volume.*does not exists.*/
        || $output =~ /Credentials provided for host.* are invalid/
        ){
        $error = "CommandFailed";
    }

    $$error_message = $output;

    # This is used to catch the errors which went unnoticed above
    if ($output =~ /(Error)(?:(\([-]?\d+\)))?(.*)/ &&
        $output !~ /Name.*Type.*7-Mode system.*Cluster-Mode system.*Status/is
        && $output !~ /Apply configuration.*7-Mode system FQDN/is &&
        $output !~ /Operation summary/is ){
        my $a1=$1 if $1;
        my $a2=$2 if $2;
        my $a3=$3 if $3;
        $a2=~ s/\(//;
        $a2=~ s/\)//;
        $a3=~ s/:\s//;
        my $member=$hash{_upper_to_lower($a1)}=[];
        my %chunk_hash;
        $chunk_hash{"code"}=$a2;
        $chunk_hash{"message"}=_formatter($a3);
        push @$member , {%chunk_hash};
        $hash{"error_count"}=1;
    }else{
        $hash{"error_count"}=0;
    }

    %$hash_ref=%hash;

    return $error;

} ## end sub _check_error

sub _no_precheck_type_parser{

    my $ref=shift;
    my ($error_message , @result_array , %hash , $resp , $output , %args );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);
    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );
    return \@result_array;

} ## end sub _no_precheck_type_parser

sub _error_exception{
    my %args = @_;
    my $hash_ref=$args{hash};
    my %hash=%{$hash_ref};
    my $error=$args{error};
    my $resp=$args{response_obj};
    my $result_array = $args{result_array};
    my $error_message = $args{error_message};
    if ($$error){
        $resp->set_parsed_output(parsed_output=>$result_array);
        $$error = 'NACL::APISet::Exceptions::'.$$error.'Exception';
        $$error->throw(text=>$$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_array);
        $$error = 'NACL::APISet::Exceptions::CommandFailedException';
        $$error->throw(text=>"Command failed.Transition library internal error."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $$error_message",response_object => $resp);
    }
}

sub _invalid_output_exception {
    $resp = shift;
    $result_array = shift;
    $hash = shift;
    $error = shift;
    $error_message = shift;

    $hash->{error_count} = $hash->{error_count}+1;
    $hash->{error} = [{
                       "message" => 'Output given by the command is not valid.',
                       "code" => undef,
                       }];

    $resp->set_parsed_output(parsed_output=>$result_array);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed or Invalid output."
        ." Original output:\n $error_message",response_object => $resp);
}

sub _precheck_type_parser{

    my $ref=shift;
    my ($error_message, @result_array, $resp,       $output, %args,
        @chunks,        $last_chunk,   $op_string,  %hash,   $first_line,
        @lines,         $error
    );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    $first_line = '.*';
    @lines = get_lines(raw_output=>$output, first_line=>$first_line);

    # Create a string version from Array of o/p Lines
    $op_string = join (" ", @lines);
    #Get the last Operation summary chunk separate from chunk array
    $last_chunk=$1 if $op_string =~ s/(Operation summary.*)//gi;
    $op_string =~ s/(Job with.*has started. )//gi;

    #Prepare an array of chunks one for each error code
    my $i=-1;
    do{
        $i++;
        push(@chunks,$1) if $op_string =~ s/^(\w{0,4}\d{1,5}\s+:\s.*?)(\w{0,4}\d{1,5}\s+:\s)/$2/g ;
    }while(defined $chunks[$i]);
    push(@chunks,$op_string);

    $hash{"error"}=[];
    $hash{"warning"}=[];
    $hash{"informational"}=[];

    #Format the chunks and push them into the complex data structure
    foreach my $i (0..@chunks-1){
        my (%chunk_hash,@msg,$chunk_type,$chunk_code,$members);
        $chunks[$i] =~ s/\d+\s+:$//g;
        $chunks[$i] =~ s/\s+> / /g;
        $chunk_type=$1 if $chunks[$i] =~ s/\[\s+(\w+)\s+\]//g;
        $chunks[$i] =~ s/  //g;
        $chunk_code=$1 if $chunks[$i] =~ s/^(\w{0,4}\d{1,5})\s+://g;
        $chunk_hash{"code"}=$chunk_code;
        if($chunks[$i] =~ /CORRECTIVE-ACTION/
           && $chunks[$i] !~ /AFFECTED-OBJECTS/) {
            @msg=split(/CORRECTIVE-ACTION:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"corrective_action"}=_formatter($msg[1]);
            $chunk_hash{"affected_objects"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] !~ /CORRECTIVE-ACTION/){
            @msg=split(/AFFECTED-OBJECTS:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"affected_objects"}=_formatter($msg[1]);
            $chunk_hash{"corrective_action"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] =~ /CORRECTIVE-ACTION/){
            $chunks[$i] =~ s/\s  //g;
            $chunk_hash{"affected_objects"}=_formatter($1)
                if $chunks[$i] =~ s/AFFECTED-OBJECTS:(.*)//gims;
            $chunk_hash{"corrective_action"}=_formatter($1)
                if $chunks[$i] =~ s/CORRECTIVE-ACTION:(.*)//gims;
            $chunk_hash{"message"}=_formatter($chunks[$i]);
        }else{
            $chunk_hash{"message"}=_formatter($chunks[$i]);
            $chunk_hash{"affected_objects"}="";
            $chunk_hash{"corrective_action"}="";
        }
        $members=$hash{_upper_to_lower($chunk_type)};
        push @$members , {%chunk_hash};

    }
    my @keys = keys (%hash);

    #Push the last chunk onto the complex data structure alongwith other details
    $hash{"error_count"}=_get_count("Errors",$last_chunk) ;
    $hash{"warning_count"}=_get_count("Warnings",$last_chunk) ;
    $hash{"informational_count"}=_get_count("Informational messages",$last_chunk);
    $hash{"next_suggested_steps"}=_formatter($1) if $last_chunk =~ /Next suggested steps: [-]+(.*)(Ran.*)/;

    #Clear  empty keys
    foreach (@keys){
        delete $hash{$_} if (!$_);
    }

    push (@result_array, \%hash);

    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );

    return \@result_array;

} ## end sub _precheck_type_parser

sub _row_type_parser{
    my $ref=shift;
    my ($error_message , @result_array , %hash , $resp , $output , %args );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);

    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );

    if ($output =~ /No information to display./) {
        $hash{"info"} = "No information to display.";
    }
    else {
        #remove all spaces or new line char from begining of output
        $output =~ s/^(\s+|\n+)//;
        #divide output into array of lines
        my @lines = split(/\n/, $output);
        my %entry;
        #now process all lines form 3nd line
        foreach $line (@lines[0..(@lines-1)]) {
            #if line does not contain : (separator) then it is end of output
            if ($line !~ /\:/) {
                last;
            }
            my @values = split(/\:/,$line);

            #formate hash index names
            $values[0] = _formatter($values[0]);
            $values[0] = to_key_name(item => $values[0]);
            $values[1] = _formatter($values[1]);

            #add values in hash
            $entry{$values[o]} = $values[1];
        }
        push(@result_array, \%entry);
    }

    return \@result_array;
} ## end sub _row_type_parser

sub _table_type_parser{

    my $ref=shift;
    my ($error_message , @result_array , %hash , $resp , $output , %args );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);

    _error_exception(
        error_message=>\$error_message,
        error=>\$error,
        hash=>\%hash,
        result_array=>\@result_array,
        response_obj=>$resp,
    );

    if ($output =~ /No information to display./) {
        $hash{"output"} = "No information to display.";
    }
    else {
        #remove all spaces or new line char from begining of output
        $output =~ s/^(\s+|\n+)//;

        #devide output into array of lines
        my @lines = split(/\n/, $output);

        #remove leading and trailing space of 1st line for
        #getting proper col titles
        $lines[0] = _formatter($lines[0]);

        #get array of column titles
        my @col_names = split(/\s{2,}/,$lines[0]);

        #ignore 2nd line and process remaning lines of table one by one
        foreach $line (@lines[2..(@lines-1)]) {
            #if line does not contain any word char then end of processing
                if ($line !~ /\w/) {
                    last;
                }
            #removeing leading and trailing space
            $line = _formatter($line);

            my @values = split(/\s+/,$line);
            my %entry;

            #fill values in hash
            foreach $index (0..(@col_names-1)) {
                $entry{to_key_name(item => $col_names[$index])} =
                                                                $values[$index];
            }
            push(@result_array, \%entry);
        }
    }

    return \@result_array;

} ## end sub _table_type_parser

1;
