#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/cfgadm.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse solaris cfgadm command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris cfgadm command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "cfgadm" will cfgadm()
##
## @synopsis
##    use cfgadm;
##    cfgadm(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::cfgadm;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(cfgadm:.*)/ ) {
        $error         = "InvalidParam";
        $error_message = "Invalid Parameters are Used in the command";
    } elsif ( $raw_output =~ /(U[u]sage:.*)/ ) {
        $error         = "CommandFailed";
        $error_message = "Command Failed to Execute";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

## @name ::cfgadm
##
## @summary Parses the raw output from the "cfgadm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cfgadm with any of its options as described in the
##    corresponding cdef file 'cfgadm.cdefs'
## @synopsis
##      cfgadm(%args);
## INPUTS TO METHOD CALL 'cfgadm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-y' </td> <td> 'yes' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'state_change' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'hardware_function' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'hardware_options' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'dynamic_attachment_points' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'supply_list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cfgadm -al
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Ap_Id                          Type         Receptacle   Occupant     Condition
##&nbsp;c0                             scsi-bus     connected    configured   unknown
##&nbsp;c0::dsk/c0t2d0                 disk         connected    configured   unknown
##&nbsp;c0::dsk/c0t3d0                 disk         connected    configured   unknown
##&nbsp;usb0/1                         unknown      empty        unconfigured ok
##&nbsp;usb0/2                         unknown      empty        unconfigured ok
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'type' => 'scsi-bus',
##&nbsp;            'ap_id' => 'c0',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'condition' => 'unknown',
##&nbsp;            'receptacle' => 'connected'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'disk',
##&nbsp;            'ap_id' => 'c0::dsk/c0t2d0',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'condition' => 'unknown',
##&nbsp;            'receptacle' => 'connected'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'disk',
##&nbsp;            'ap_id' => 'c0::dsk/c0t3d0',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'condition' => 'unknown',
##&nbsp;            'receptacle' => 'connected'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'unknown',
##&nbsp;            'ap_id' => 'usb0/1',
##&nbsp;            'occupant' => 'unconfigured',
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'empty'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'unknown',
##&nbsp;            'ap_id' => 'usb0/2',
##&nbsp;            'occupant' => 'unconfigured',
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'empty'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cfgadm -v -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Ap_Id                          Receptacle   Occupant     Condition  Information
##&nbsp;When         Type         Busy     Phys_Id
##&nbsp;c0                             connected    configured   unknown
##&nbsp;unavailable  scsi-bus     n        /devices/pci@0,0/pci1022,7450@2/pci1000,3060@3:scsi
##&nbsp;usb0/1                         empty        unconfigured ok
##&nbsp;unavailable  unknown      n        /devices/pci@0,0/pci1022,7460@6/pci1022,7464@0:1
##&nbsp;usb0/2                         empty        unconfigured ok
##&nbsp;unavailable  unknown      n        /devices/pci@0,0/pci1022,7460@6/pci1022,7464@0:2
##&nbsp;usb1/1                         connected    configured   ok         Mfg: American Megatrends Inc.  Product: Virtual Cdrom Device  NConfigs: 1  Config: 0  <no cfg str descr>
##&nbsp;unavailable  usb-storage  n        /devices/pci@0,0/pci1022,7460@6/pci1022,7464@0,1:1
##&nbsp;usb1/2                         connected    configured   ok         Mfg: American Megatrends Inc.  Product: Virtual Floppy Device  NConfigs: 1  Config: 0  <no cfg str descr>
##&nbsp;unavailable  usb-floppy   n        /devices/pci@0,0/pci1022,7460@6/pci1022,7464@0,1:2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'type' => 'scsi-bus',
##&nbsp;            'when' => 'unavailable',
##&nbsp;            'ap_id' => 'c0',
##&nbsp;            'busy' => 'n',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'condition' => 'unknown',
##&nbsp;            'receptacle' => 'connected',
##&nbsp;            'phys_id' => '/devices/pci@0,0/pci1022,7450@2/pci1000,3060@3:scsi'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'unknown',
##&nbsp;            'when' => 'unavailable',
##&nbsp;            'ap_id' => 'usb0/1',
##&nbsp;            'busy' => 'n',
##&nbsp;            'occupant' => 'unconfigured',
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'empty',
##&nbsp;            'phys_id' => '/devices/pci@0,0/pci1022,7460@6/pci1022,7464@0:1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'type' => 'unknown',
##&nbsp;            'when' => 'unavailable',
##&nbsp;            'ap_id' => 'usb0/2',
##&nbsp;            'busy' => 'n',
##&nbsp;            'occupant' => 'unconfigured',
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'empty',
##&nbsp;            'phys_id' => '/devices/pci@0,0/pci1022,7460@6/pci1022,7464@0:2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ap_id' => 'usb1/1',
##&nbsp;            'busy' => 'n',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'information' => {
##&nbsp;                               'nconfigs' => '1',
##&nbsp;                               'product' => 'Virtual Cdrom Device',
##&nbsp;                               'config' => '0 <no cfg str descr>',
##&nbsp;                               'mfg' => 'American Megatrends Inc.'
##&nbsp;                             },
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'connected',
##&nbsp;            'type' => 'usb-storage',
##&nbsp;            'when' => 'unavailable',
##&nbsp;            'phys_id' => '/devices/pci@0,0/pci1022,7460@6/pci1022,7464@0,1:1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ap_id' => 'usb1/2',
##&nbsp;            'busy' => 'n',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'information' => {
##&nbsp;                               'nconfigs' => '1',
##&nbsp;                               'product' => 'Virtual Floppy Device',
##&nbsp;                               'config' => '0 <no cfg str descr>',
##&nbsp;                               'mfg' => 'American Megatrends Inc.'
##&nbsp;                             },
##&nbsp;            'condition' => 'ok',
##&nbsp;            'receptacle' => 'connected',
##&nbsp;            'type' => 'usb-floppy',
##&nbsp;            'when' => 'unavailable',
##&nbsp;            'phys_id' => '/devices/pci@0,0/pci1022,7460@6/pci1022,7464@0,1:2'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cfgadm -f -c configure c0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = cfgadm  -s 'match=partial,select=class(scsi):ap_id(c):type(scsi)'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Ap_Id                          Type         Receptacle   Occupant     Condition
##&nbsp;c0                             scsi-bus     connected    configured   unknown
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'type' => 'scsi-bus',
##&nbsp;            'ap_id' => 'c0',
##&nbsp;            'occupant' => 'configured',
##&nbsp;            'condition' => 'unknown',
##&nbsp;            'receptacle' => 'connected'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cfgadm {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $cmd        = $resp->get_command();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    my @result_array;
    my $arr_count = -1;
    my @lines
        = get_lines( raw_output => $raw_output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $cmd =~ /-v/ ) {
            @head1 = split( /\s+/, $lines[0] );
            @head2 = split( /\s+/, $lines[1] );
            $c     = 0;
            for ( my $i = 2; $i <= $#lines; $i = $i + 2 ) {
                if ( $lines[$i] =~ /:/ ) {
                    @values1 = split( /\s\s+/, $lines[$i] );
                } else {
                    @values1 = split( /\s+/, $lines[$i] );
                }
                @values2 = split( /\s+/, $lines[ $i + 1 ] );
                for ( my $j = 0; $j <= $#values1; $j++ ) {
                    if ( $values1[$j] =~ /:/ ) {
                        my $flag = 0;
                        if ( $values1[ $j + 1 ] !~ /:/ ) {
                            $values1[$j] = "$values1[$j] $values1[$j+1]";
                            $flag = 1;
                        }
                        my @t1 = split( /:\s+/, $values1[$j] );
                        $result_array[$c]->{information}->{ to_key_name( item => $t1[0] ) }  = $t1[1];
                        $j++ if ($flag);
                    } else {
                        $result_array[$c]->{to_key_name( item =>  $head1[$j]) } = $values1[$j];
                    }
                }
                for ( my $k = 0; $k <= $#values2; $k++ ) {
                    $result_array[$c]->{to_key_name( item =>  $head2[$k]) } = $values2[$k];
                }
                $c++;
            }

        } elsif ( $line !~ /Ap_Id\s+\w+\s+\w+\s+\w+\s+\w+/ ) {
            @head = split( /\s+/, $lines[0] );
            $arr_count++;
            my @values = split( /\s+/, $line );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$arr_count]->{to_key_name( item =>  $head[$j]) } = $values[$j];
            }
        }
    }
    return \@result_array;
} ## end sub cfgadm
1;
