#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/crontab.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse solaris crontab command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris crontab command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "crontab" will crontab()
##
## @synopsis
##    use crontab;
##    crontab(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::crontab;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::crontab
##
## @summary Parses the raw output from the "crontab" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from crontab with any of its options as described in the
##    corresponding cdef file 'crontab.cdefs'
## @synopsis
##      crontab(%args);
## INPUTS TO METHOD CALL 'crontab' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'remove' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'edit' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = crontab -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;#ident  "@(#)root       1.21    04/03/23 SMI"
##&nbsp;#
##&nbsp;# The root crontab should be used to perform accounting data collection.
##&nbsp;#
##&nbsp;#
##&nbsp;10 3 * * * /usr/sbin/logadm
##&nbsp;15 3 * * 0 /usr/lib/fs/nfs/nfsfind
##&nbsp;30 3 * * * [ -x /usr/lib/gss/gsscred_clean ] && /usr/lib/gss/gsscred_clean
##&nbsp;#
##&nbsp;# The rtc command is run to adjust the real time clock if and when
##&nbsp;# daylight savings time changes.
##&nbsp;#
##&nbsp;1 2 * * * [ -x /usr/sbin/rtc ] && /usr/sbin/rtc -c > /dev/null 2>&1
##&nbsp;#10 3 * * * /usr/lib/krb5/kprop_script ___slave_kdcs___
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hour' => '3',
##&nbsp;            'minute' => '10',
##&nbsp;            'month' => '*',
##&nbsp;            'day_of_the_week' => '*',
##&nbsp;            'job' => '/usr/sbin/logadm',
##&nbsp;            'day_of_the_month' => '*'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hour' => '3',
##&nbsp;            'minute' => '15',
##&nbsp;            'month' => '*',
##&nbsp;            'day_of_the_week' => '0',
##&nbsp;            'job' => '/usr/lib/fs/nfs/nfsfind',
##&nbsp;            'day_of_the_month' => '*'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hour' => '3',
##&nbsp;            'minute' => '30',
##&nbsp;            'month' => '*',
##&nbsp;            'day_of_the_week' => '*',
##&nbsp;            'job' => '[ -x /usr/lib/gss/gsscred_clean ] && /usr/lib/gss/gsscred_clean',
##&nbsp;            'day_of_the_month' => '*'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hour' => '2',
##&nbsp;            'minute' => '1',
##&nbsp;            'month' => '*',
##&nbsp;            'day_of_the_week' => '*',
##&nbsp;            'job' => '[ -x /usr/sbin/rtc ] && /usr/sbin/rtc -c > /dev/null 2>&1',
##&nbsp;            'day_of_the_month' => '*'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub crontab {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( response_object => $resp, raw_output => $raw_output );
    my @result_array;
    my @lines
        = get_lines( response_object => $resp, raw_output => $raw_output );
    my @arr = ( 'minute', 'hour', 'day_of_the_month', 'month',
        'day_of_the_week' );
    my $count = 0;

    foreach my $line (@lines) {
        if ( $line !~ /^#/ ) {
            my @values = split( /\s+/, $line );
            for ( my $i = 0; $i < 5; $i++ ) {
                $result_array[$count]->{ $arr[$i] } = $values[$i];
            }
            splice( @values, 0, 5 );
            my $job = join( " ", @values );
            $result_array[$count]->{job} = $job;
            $count++;
        }
    }
    return \@result_array;
} ## end sub crontab

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message, $error);
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(crontab):\s+.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
