# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/devinfo.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Solaris devinfo command
##
## @description
##    This is a library to parse the raw output of Solaris devinfo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "devinfo" will be devinfo()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::devinfo;
##
##    devinfo(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::devinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::devinfo
##
## @summary Parses the raw output from the "devinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from devinfo with any of its options as described in the
##    corresponding cdef file 'devinfo.cdefs'
## @synopsis
##      devinfo(%args);
## INPUTS TO METHOD CALL 'devinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'device-partition' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'device-info' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = devinfo -i /dev/rdsk/c0t2d0p0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/rdsk/c0t2d0p0	0	0	16065	512	5
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_name' => '/dev/rdsk/c0t2d0p0',
##&nbsp;            'software_version' => '0',
##&nbsp;            'device_partitions' => '5',
##&nbsp;            'drive_id' => '0',
##&nbsp;            'device_bytes_per_block' => '512',
##&nbsp;            'device_blocks_per_cylinder' => '16065'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = devinfo -p /dev/rdsk/c0t2d0p0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/rdsk/c0t2d0p0	1d	d0	0	0	0	0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'partition_tag' => '0',
##&nbsp;            'device_major_no' => '1d',
##&nbsp;            'partition_start' => '0',
##&nbsp;            'device_name' => '/dev/rdsk/c0t2d0p0',
##&nbsp;            'partition_flag' => '0',
##&nbsp;            'no_of_blocks' => '0',
##&nbsp;            'device_minor_no' => 'd0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub devinfo {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line
            =~ /(\/\w+\/\w+\/\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
            $result_array[0]->{device_name}                = $1;
            $result_array[0]->{software_version}           = $2;
            $result_array[0]->{drive_id}                   = $3;
            $result_array[0]->{device_blocks_per_cylinder} = $4;
            $result_array[0]->{device_bytes_per_block}     = $5;
            $result_array[0]->{device_partitions}          = $6;
        } elsif ( $line
            =~ /(\/\w+\/\w+\/\S+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
            ) {
            $result_array[0]->{device_name}     = $1;
            $result_array[0]->{device_major_no} = $2;
            $result_array[0]->{device_minor_no} = $3;
            $result_array[0]->{partition_start} = $4;
            $result_array[0]->{no_of_blocks}    = $5;
            $result_array[0]->{partition_flag}  = $6;
            $result_array[0]->{partition_tag}   = $7;
        }

    }
    return \@result_array;

} ## end sub devinfo

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([uU]sage):\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(devinfo):\s+.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;
