#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris hammer command
##
## @description
##    This is a library to parse the raw output of Solaris hammer command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "hammer" will be hammer)
##
## @synopsis
##    use hammer;
## INPUTS TO METHOD CALL 'hammer' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/hammer_6_22, /u/myuser/dir/hammer.experimental
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'experimental', '6.22'
## This would select hammer.experimental, hammer.6.22 respectively.
##
##
##    hammer(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::hammer;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~ /(.*Usage:.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = "Invalid Usage of hammer command";
    }
    return $error;
} ## end sub _check_error

## @name ::hammer
##
## @summary Parses the raw output from "hammer" command
##
## @description
##    This method can be used for parsing the output from "hammer" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/Linux/Redhat/hammer.cdefs
##
## @synopsis
##    hammer (%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the
##  parsed output data from the output of hammer command

sub hammer {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'hammer',
                             "ptr_error_message" => \$error_message,);
    if($error) {
        if($error_level == 0) {
            $Log->warn ( "Due to a bug in Hammer, a zero execution status"
                        ."\nwas returned even when the command execution"
                        ."\nhas failed.\n" );
        }
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text =>  "Execution status was $error_level(>0) "
                    ."indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    my @lines = get_lines(raw_output => $raw_output, response_object => $resp);
    my ($curr_word, $flag);
    my %outer_hash = ();
    foreach my $line (@lines) {
        my %hash;
        if ($line =~
            /(.*) (CREATE|DELETE|RENAME|OWRITE|TRUNC|READ) (.*) fileid/) {
            $hash{'started'}   = $1;
            $hash{'operation'} = $2;
            $hash{'file_name'} = $3;
            $curr_word         = $hash{'file_name'};
            while ($curr_word =
                   _get_next_word(line      => $line,
                                  prev_word => $curr_word)
              ) {
                if ($curr_word =~ /(.*)=(.*)/) {
                    $hash{$1} = $2;
                } elsif ($curr_word =~ /\((.*)\)/) {
                    $hash{'info'} = $1;
                } elsif ($curr_word =~ /\//) {
                    $hash{'speed'} = $curr_word;
                } else {
                    $hash{'time'} = $curr_word;
                }
            } ## end while ($curr_word = _get_next_word...
            push(@{$outer_hash{'operations'}}, \%hash);
        } elsif ($line =~ /=/ && $line !~ /[Vv]ersion/) {
            $line      = "dummy " . $line;
            $curr_word = 'dummy';
            while ($curr_word =
                   _get_next_word(line      => $line,
                                  prev_word => $curr_word)
              ) {
                if ($curr_word =~ /(.*)=(.*)/) {
                    $outer_hash{$1} = $2;
                }
            }
        } elsif ($line =~ /(hammer\s+)?version(\s+|=)(.*)/) {
            $outer_hash{'version'} = $3;
        }
    } ## end foreach my $line (@lines)
    @result_array = (\%outer_hash);
    return \@result_array;    #to be updated to return a reference
} ## end sub hammer

## @name ::get_next_word
##
## @summary : returns next word in a line having unique words
##
## @description
##    This method is used for getting the next word iteratively over a line
##
## @synopsis
##    _get_next_word (%args);
##
## @arg %args Required:
## line => the line from which the word is to be extracted
## prev_word    => the previous word that was extracted
##
## @return Returns the next word (scalar)

sub _get_next_word {
    my %args      = @_;
    my $line      = $args{line};
    my $prev_word = $args{prev_word};
    if ($line =~ /$prev_word (\(.*\)|[^\s]*)/) {
        my $word = $1;

        #$word =~ s/\(|\)//g;
        return $word;
    }
    return undef;
} ## end sub _get_next_word
1;
