#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris ifconfig command output
##
## @description
##    This is a library to parse the raw output of Solaris ifconfig command.
##
## @synopsis
##    use ifconfig;
##
##    ifconfig(%args);
##
## @author ksmahesh@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::ifconfig;

use Data::Dumper;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed";    # we'll try to classify it if possible
    $$error_message = $raw_output;

    if ($raw_output =~ /ERROR|Usage:\s.*/) {
        if ($raw_output =~ /Missing/) {
            $error = "MissingRequiredParam";
        } elsif ($raw_output =~ /[Ii]nvalid|[Uu]nrecognized\s+option/) {
            $error = "InvalidParam";
        } elsif ($raw_output =~ /argument\s+to|unknown/) {
            $error = "InvalidParamValue";
        }
    } elsif ($raw_output =~ /already\s+exists/) {
        $error = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::ifconfig
##
## @summary Parses the raw output from the "ifconfig" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ifconfig with any of its options as described in the 
##    corresponding cdef file 'ifconfig.cdefs'
## @synopsis
##      ifconfig(%args);
## INPUTS TO METHOD CALL 'ifconfig' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-a6' </td> <td> 'all-IPv6' </td> </tr> 
## <tr> <td> 'up' </td> <td> 'up' </td> </tr> 
## <tr> <td> 'addif' </td> <td> 'addif-address' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> </tr> 
## <tr> <td> '-a4' </td> <td> 'all-IPv4' </td> </tr> 
## <tr> <td> 'unplumb' </td> <td> 'unplumb' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr> 
## <tr> <td> 'down' </td> <td> 'down' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'address' </td> </tr> 
## <tr> <td> 'netmask' </td> <td> 'netmask' </td> </tr> 
## <tr> <td> 'removeif' </td> <td> 'removeif-address' </td> </tr> 
## <tr> <td> 'plumb' </td> <td> 'plumb' </td> </tr> 
## <tr> <td> 'mtu' </td> <td> 'max-transfer-unit' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ifconfig -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;lo0: flags=2001000849<UP,LOOPBACK,RUNNING,MULTICAST,IPv4,VIRTUAL> mtu 8232 index 1
##&nbsp;	inet 127.0.0.1 netmask ff000000 
##&nbsp;vmxnet0: flags=1000843<UP,BROADCAST,RUNNING,MULTICAST,IPv4> mtu 1500 index 2
##&nbsp;	inet 10.72.217.12 netmask ff000000 broadcast 10.255.255.255
##&nbsp;	ether 0:50:56:98:2d:d6 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'interface' => 'lo0',
##&nbsp;            'configuration' => [
##&nbsp;                                 {
##&nbsp;                                   'index' => '1',
##&nbsp;                                   'flags' => '2001000849<UP,LOOPBACK,RUNNING,MULTICAST,IPv4,VIRTUAL>',
##&nbsp;                                   'inet' => '127.0.0.1',
##&nbsp;                                   'netmask' => 'ff000000',
##&nbsp;                                   'mtu' => '8232'
##&nbsp;                                 }
##&nbsp;                               ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'interface' => 'vmxnet0',
##&nbsp;            'configuration' => [
##&nbsp;                                 {
##&nbsp;                                   'broadcast' => '10.255.255.255',
##&nbsp;                                   'ether' => '0:50:56:98:2d:d6',
##&nbsp;                                   'index' => '2',
##&nbsp;                                   'flags' => '1000843<UP,BROADCAST,RUNNING,MULTICAST,IPv4>',
##&nbsp;                                   'inet' => '10.72.217.12',
##&nbsp;                                   'netmask' => 'ff000000',
##&nbsp;                                   'mtu' => '1500'
##&nbsp;                                 }
##&nbsp;                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifconfig {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $command = $resp->get_actual_command();
    my $error;

    #error_level is zero, when the
    #vlan with the same vlan-id
    #already exists.
    if ($output =~/already\s+exists/) {
        $error_level = 1;
    }

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 0,
        first_line      => '',
        response_object => $resp
    );

    my @return_array;
    my $regex;
    my $i=0;
	while($i<@lines) {
	    my %hash;
		my %internal_hash;
		if ($lines[$i]=~/(.*):\s\w*=(.*)\s\w*\s(\d*)\s\w*\s(\d*)/){
			$hash{'interface'}=$1;
			$internal_hash{'flags'}=$2;
			$internal_hash{'mtu'}=$3;
			$internal_hash{'index'}=$4;
			$i++;
			if ($lines[$i]=~/(\w*)\s(.*)\s(\w*)\s(.*)\s(\w*)\s(.*)/){
				$internal_hash{$1}=$2;
				$internal_hash{$3}=$4;
				$internal_hash{$5}=$6;
				$i++;
			} elsif ($lines[$i]=~/(\w*)\s(.*)\s(\w*)\s(.*)/){
				$internal_hash{$1}=$2;
				$internal_hash{$3}=$4;
				$i++;
			} elsif ($lines[$i]=~/(\w*)\s(.*)/){
				$internal_hash{$1}=$2;
				$i++;
			}
			if ($lines[$i]=~/ether\s(.*)/){
				$internal_hash{'ether'}=$1;
				$i++;
			}
	        } else {
                    return [];
		}
		push(@{$hash{'configuration'}},\%internal_hash);
        push( @return_array, \%hash );
    }
    return \@return_array;
}
1;
