#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode iscsiadm command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode iscsiadm command.
##
## @synopsis
##    use iscsiadm;
##    iscsiadm_add(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::iscsiadm;

use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~
        /(unexpected OS error|missing target address|invalid IP address|not found|unknown iSCSI name type)/
      ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(invalid object|iscsiadm:\s+Unknown\s+key)/) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(iscsiadm: at least one option required)/) {
        $error          = "MissingRequiredParam";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(Unable to complete operation|iscsiadm:\s+logical\s+unit\s+in\s+use)/) {
        $error = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::iscsiadm_add
##
## @summary Parses the raw output from the "iscsiadm_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsiadm_add with any of its options as described in the
##    corresponding cdef file 'iscsiadm.cdefs'
## @synopsis
##      iscsiadm_add(%args);
## INPUTS TO METHOD CALL 'iscsiadm_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'direct-object' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'discovery-address' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'isns-server' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'static-target' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = iscsiadm add discovery-address 10.0.0.10:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm add isns-server 10.0.0.11:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm add static-config iqn.1999-08.com.array:sn.01234567,10.0.0.12:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
sub iscsiadm_add {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'iscsiadm add',
                             "ptr_error_message" => \$error_message,);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub iscsiadm_add


## @name ::iscsiadm_list
##
## @summary Parses the raw output from the "iscsiadm list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from iscsiadm_list with any of its options as described in the 
##    corresponding cdef file 'iscsiadm.cdefs'
## @synopsis
##      iscsiadm_list(%args);
## INPUTS TO METHOD CALL 'iscsiadm_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'target' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'isns-server' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'specific' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'direct-object' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'discovery-address' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'static-target' </td> <td> '0' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'more-details' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = iscsiadm list discovery
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Discovery:
##&nbsp;	Static: disabled
##&nbsp;	Send Targets: enabled
##&nbsp;	iSNS: enabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'isns' => 'enabled',
##&nbsp;            'send_targets' => 'enabled',
##&nbsp;            'static' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iscsiadm list target
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Target: iqn.1992-08.com.netapp:sn.ae0cdf6645be11e2b5f9123478563412:vs.5
##&nbsp;	Alias: vs0
##&nbsp;	TPGT: 1026
##&nbsp;	ISID: 4000002a0000
##&nbsp;	Connections: 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'connections' => '1',
##&nbsp;            'tpgt' => '1026',
##&nbsp;            'isid' => '4000002a0000',
##&nbsp;            'alias' => 'vs0',
##&nbsp;            'iqn' => 'iqn.1992-08.com.netapp:sn.ae0cdf6645be11e2b5f9123478563412:vs.5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iscsiadm list discovery-address
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Discovery Address: 10.238.9.30:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipaddress' => '10.238.9.30',
##&nbsp;            'ip_port' => '3260'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iscsiadm list initiator-node
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Initiator node name: iqn.1986-03.com.sun:01:eafec4f5ffff.4d3a0209
##&nbsp;Initiator node alias: sti-solaris10-0173
##&nbsp;	Login Parameters (Default/Configured):
##&nbsp;		Header Digest: NONE/NONE
##&nbsp;		Data Digest: NONE/NONE
##&nbsp;	Authentication Type: NONE
##&nbsp;	RADIUS Server: NONE
##&nbsp;	RADIUS access: unknown
##&nbsp;	Tunable Parameters (Default/Configured):
##&nbsp;		Session Login Response Time: 60/-
##&nbsp;		Maximum Connection Retry Time: 180/-
##&nbsp;		Login Retry Time Interval: 60/-
##&nbsp;	Configured Sessions: 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'radius_server' => 'NONE',
##&nbsp;            'radius_access' => 'unknown',
##&nbsp;            'login_parameters' => [
##&nbsp;                                    {
##&nbsp;                                      'parameter' => 'Header Digest',
##&nbsp;                                      'configured' => 'NONE',
##&nbsp;                                      'default' => 'NONE'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'parameter' => 'Data Digest',
##&nbsp;                                      'configured' => 'NONE',
##&nbsp;                                      'default' => 'NONE'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'tunable_parameters' => [
##&nbsp;                                      {
##&nbsp;                                        'parameter' => 'Session Login Response Time',
##&nbsp;                                        'configured' => '-',
##&nbsp;                                        'default' => '60'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'parameter' => 'Maximum Connection Retry Time',
##&nbsp;                                        'configured' => '-',
##&nbsp;                                        'default' => '180'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'parameter' => 'Login Retry Time Interval',
##&nbsp;                                        'configured' => '-',
##&nbsp;                                        'default' => '60'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'initiator_node' => [
##&nbsp;                                  {
##&nbsp;                                    'initiator_node_alias' => 'sti-solaris10-0173',
##&nbsp;                                    'authentication_type' => 'NONE',
##&nbsp;                                    'name' => 'iqn.1986-03.com.sun:01:eafec4f5ffff.4d3a0209',
##&nbsp;                                    'configured_sessions' => '1'
##&nbsp;                                  }
##&nbsp;                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = iscsiadm list static-config
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm list isns-server
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;iSNS Server IP Address: 10.0.0.11:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipaddress' => '10.0.0.11',
##&nbsp;            'ip_port' => '3260'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iscsiadm_list {
	my %args = @_;
	my $error_message;
	my $resp       = $args{response_object};
	my $command    = $resp->get_actual_command();
	my $raw_output = $resp->get_processed_output();
	my $error = _check_error("raw_output"        => $raw_output,
			"command"           => 'iscsiadm list',
			"ptr_error_message" => \$error_message,);
	if ($error) {
		$error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
		$error->throw(text => $error_message, response_object => $resp);
	}
	my @result_array = ();
	my ($name, $i, $j, $name1,$key,$val);
	$j = 0;
	$i = -1;
	my $flag1 =0;
	my $flag2=0;
	my (@values,@values1);
	my $k;
	my @lines = get_lines(raw_output      => $raw_output,
			can_throw       => 0,
			response_object => $resp);
	foreach my $line (@lines) {
		if ($line =~ /^(Discovery):$/) {
			$name = to_key_name(item => $1);
		} elsif ($line =~ /^(Static|Send Targets|iSNS): (enabled|disabled)$/) {
			$result_array[0]->{to_key_name(item => $1)} = $2;
		} elsif ($line =~ /^(Target name|Target address):\s+([\w\.\:]+)$/) {
			$result_array[$i]->{"target"}->[$j]->{to_key_name(item => $1)} = $2;
			$j++ if ($1 =~ /address/);
		} elsif ($line =~ /(Unable to get targets)/) {
			$result_array[0]->{"target"}->[0] = to_key_name(item => $1);
		} elsif ($line =~/^(Discovery Address):\s+([\w\.\-\:\,]+)$/) {
			@values = split (/:\s*/,$line);
			$i++;
			$result_array[$i]->{ipaddress} = $values[1];
			$result_array[$i]->{ip_port} = $values[2];
		} elsif ($line =~/^(Static Configuration Target|Static targetiSNS Server IP Address):\s+([\w\.\-\:\,]+)$/) {
			$result_array[0]->{to_key_name(item => $1)}->[$j++] = $2;
		} elsif ($line =~/^(iSNS Server IP Address):\s+([\w\.\-\:\,]+)$/) {
			@values = split (/:\s*/,$line);
			$i++;
			$result_array[$i]->{ipaddress} = $values[1];
			$result_array[$i]->{ip_port} = $values[2];
		} elsif ($line =~ /(Initiator node name):\s+([\w\.\-\:\,]+)$/) {
			$flag1=1;
			$i++;
			$val = $2;
			$name1 = to_key_name(item => $1);
			$name1 =~ s/_name//g;
			$result_array[$i]->{"$name1"}->[0]->{"name"} = $val;
		} elsif ($line =~ /(Target):\s+([\w\.\-\:\,]+)$/) {
			$flag2=1;
			$i++;
			@values = split(/:\s+/,$line);
			$result_array[$i]->{"iqn"} = $values[1];
		} elsif ($line =~ /(Alias|ISID|Connections|CID|RADIUS Server|RADIUS access|TPGT):\s+([\w\-\:\.]+)$/) {
			$result_array[$i]->{to_key_name(item => $1)} = $2;
		} elsif ($line =~ /(IP address \(Local\)|IP address \(Peer\)|Discovery Method):\s+([\w\.\:]+)$/) {
			$result_array[$i]->{"$name1"}->[0]->{to_key_name(item => $1)} = $2;
		} elsif ($line =~ /(Initiator node alias|Authentication Type|Configured Sessions|Bi-directional Authentication):\s+([\w\.\-\:\,]+)$/) {
			if($flag2) {
				$key = $1;
				$val = $2;
				$key =~ s/-//g;
				$result_array[$i]->{to_key_name(item => $key)} = $val;
			} elsif ($flag1) {
				$key = $1;
				$val = $2;
				$key =~ s/-//g;
				$result_array[$i]->{"$name1"}->[0]->{to_key_name(item => $key)} = $val;
			}
		} elsif ($line =~ /(Login Parameters \(Negotiated\)|Tunable Parameters \(Default\/Configured\)|Login Parameters \(Default\/Configured\))/) {
			$k = 0;
			@values = split(/\s+\(/,$line);
            $name = $values[0];
        } elsif ($line =~ /([\w\s\(\)]+):\s+([\w\-\/]+)$/) {
            @values = split (/:\s+/,$line);
            @values1 = split(/\//,$values[1]);
            $result_array[$i]->{to_key_name(item => $name)}->[$k]->{"parameter"} = $values[0];
            $result_array[$i]->{to_key_name(item => $name)}->[$k]->{"default"} = $values1[0];
            $result_array[$i]->{to_key_name(item => $name)}->[$k]->{"configured"} = $values1[1];
            $k++;
        }
    }
    
    return \@result_array;
} ## end sub iscsiadm_list

## @name ::iscsiadm_remove
##
## @summary Parses the raw output from the "iscsiadm remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsiadm_remove with any of its options as described in the
##    corresponding cdef file 'iscsiadm.cdefs'
## @synopsis
##      iscsiadm_remove(%args);
## INPUTS TO METHOD CALL 'iscsiadm_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'direct-object' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'discovery-address' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'isns-server' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'static-target' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'target' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = iscsiadm remove discovery-address 10.0.0.10:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm remove isns-server 10.0.0.11:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm remove static-config iqn.1999-08.com.array:sn.01234567,10.0.0.12:3260
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm remove target-param iqn.1999-08.com.array:sn.01234567
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
sub iscsiadm_remove {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'iscsiadm remove',
                             "ptr_error_message" => \$error_message,);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub iscsiadm_remove

## @name ::iscsiadm_modify
##
## @summary Parses the raw output from the "iscsiadm modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsiadm_modify with any of its options as described in the
##    corresponding cdef file 'iscsiadm.cdefs'
## @synopsis
##      iscsiadm_modify(%args);
## INPUTS TO METHOD CALL 'iscsiadm_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'direct-object' </td> </tr>
## <tr> <td> '-i' </td> <td> 'iSNS' </td> </tr>
## <tr> <td> '-s' </td> <td> 'static' </td> </tr>
## <tr> <td> '-t' </td> <td> 'sendtargets' </td> </tr>
## <tr> <td> '-A' </td> <td> 'node-alias' </td> </tr>
## <tr> <td> '-a' </td> <td> 'authentication' </td> </tr>
## <tr> <td> '-C' </td> <td> 'CHAP-secret' </td> </tr>
## <tr> <td> '-c' </td> <td> 'configured-sessions' </td> </tr>
## <tr> <td> '-d' </td> <td> 'datadigest' </td> </tr>
## <tr> <td> '-H' </td> <td> 'CHAP-name' </td> </tr>
## <tr> <td> '-h' </td> <td> 'headerdigest' </td> </tr>
## <tr> <td> '-N' </td> <td> 'node-name' </td> </tr>
## <tr> <td> '-P' </td> <td> 'radius-shared-secret' </td> </tr>
## <tr> <td> '-R' </td> <td> 'radius-access' </td> </tr>
## <tr> <td> '-r' </td> <td> 'radius-server' </td> </tr>
## <tr> <td> '-T' </td> <td> 'tunable-param' </td> </tr>
## <tr> <td> '-B' </td> <td> 'bi-directional-authentication' </td> </tr>
## <tr> <td> '-p' </td> <td> 'login-param' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'static-target' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = iscsiadm modify initiator-node -h NONE
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm modify discovery -s enable
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm modify initiator-node -r 10.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = iscsiadm modify target-param -d NONE iqn.1999-08.com.array:sn.01234567
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
sub iscsiadm_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'iscsiadm modify',
                             "ptr_error_message" => \$error_message,);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub iscsiadm_modify
1;
