#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/ls.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Solaris ls command output
##
## @description
##     This library supports error checking for ls command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::ls;
##
##    ls(%args);
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::ls;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidOptionException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error       = undef;
    my $error_message;
    if ( $raw_output =~ /(ls:\s+illegal option.*)/ ) {
        $error         = "InvalidOption";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(\S+:\s+No such file or directory)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::ls
##
## @summary Parses the raw output from the "ls" command
##
## @description
##    This library supports error checking for ls command, but output parsing is not supported as of now.
## @synopsis
##      ls(%args);
## INPUTS TO METHOD CALL 'ls' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-A' </td> <td> 'almost-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'sort-mod-time-i-node' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'list-by-column' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directory' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'no-group-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'do-not-list-owner' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'dereference' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'comma-separated' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force-to-dir' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size-in-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-1' </td> <td> 'one-per-line' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'hide-control-chars' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-readable' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'display-compact-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reverse' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'follow-symbolic-links' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'display-time-to-sec' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-uid-gid' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'sort-by-modification-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'sort-by-access-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'put-a-slash' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'force-print' </td> <td> '1' </td> </tr>
## <tr> <td> '-E' </td> <td> 'display-time-to-nanosec' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'display-verbose-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'ignore-backups' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##Returns: Output parsing is not supported

sub ls {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    return [];
} ## end sub ls

1;
