#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/mkfs.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris mkfs command
##
## @description
##    This is a library to parse the raw output of Solaris mkfs command.
##
## @synopsis
##    use mkfs;
##
## mkfs(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::mkfs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(mkfs:\s+FSType\s+.*\s+not\s+installed\s+in\s+the\s+kernel|bad value for size|bad numeric arg for size|.*(invalid).*|.*mounted;\s+will\s+not\s+make\s+a\s+filesystem\s+here|.*(No such file).*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = "$1";
    } elsif( $output =~ /size not specified/) {
        $error          = "MissingRequiredParam";
        $$error_message = $output;
    }
    return $error;
} ## end sub _check_error

## @name ::mkfs
##
## @summary Parses the raw output from the "mkfs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mkfs with any of its options as described in the
##    corresponding cdef file 'mkfs.cdefs'
## @synopsis
##      mkfs(%args);
## INPUTS TO METHOD CALL 'mkfs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-o' </td> <td> 'fs-options' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'validate' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'fstype' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mkfs \dev\rsd1i 16065
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/rsd1i:     16064 sectors in 3 cylinders of 48 tracks, 128 sectors\r\n
##&nbsp;        7.8MB in 1 cyl groups (13 c/g, 39.00MB/g, 18624 i/g)\r\n
##&nbsp;super-block backups (for fsck -F ufs -o b=#) at:\r\n
##&nbsp; 32,\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'sectors_per_track' => '128',
##&nbsp;             'tracks_per_cylinder' => '48',
##&nbsp;             'space_in_1_cyl_groups' => '7.8MB',
##&nbsp;             'total_sectors' => '16064',
##&nbsp;             'inodes_per_group' => '18624',
##&nbsp;             'space_per_group' => '39.00MB',
##&nbsp;             'cylinders_per_group' => '13',
##&nbsp;             'device' => '/dev/rsd1i',
##&nbsp;             'super_block_backups' => '32,'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub mkfs {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "mkfs",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( 'raw_output' => $output );
    while(my $line = shift(@lines)) {
        if($line =~ /(.*):\s+(\d+)\s+sectors\s+in\s+(\d+)\s+cylinders\s+of\s+(\d+)\s+tracks,\s+(\d+)\s+sectors/){
            $result_array[0]->{total_sectors} = $2;
            $result_array[0]->{device} = $1;
            $result_array[0]->{tracks_per_cylinder} = $4;
            $result_array[0]->{sectors_per_track} = $5;
        } elsif($line =~ /([\d+.MGB]+)\s+(in\s+\d+\s+cyl\s+groups)\s+\((\d+)\s+c\/g,\s+([\d+.MGB]+)\/g,\s+(\d+)\s+i\/g\)/) {
            my $key = "space "."$2";
            $result_array[0]->{to_key_name(item => $key)} = $1;
            $result_array[0]->{cylinders_per_group} = $3 ;
            $result_array[0]->{space_per_group} = $4;
            $result_array[0]->{inodes_per_group} = $5;
        } elsif($line =~ /super-block\s+backups.*/){
            $line = shift(@lines);
            $result_array[0]->{super_block_backups} = $line;
        }

    }
    return \@result_array;    #to be updated to return a reference
} ## end sub mkfs

1;
