# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/modinfo.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Solaris modinfo command
##
## @description
##    This is a library to parse the raw output of Solaris modinfo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "modinfo" will be modinfo()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::modinfo;
##
##    modinfo(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::modinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::modinfo
##
## @summary Parses the raw output from the "modinfo" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from modinfo with any of its options as described in the 
##    corresponding cdef file 'modinfo.cdefs'
## @synopsis
##      modinfo(%args);
## INPUTS TO METHOD CALL 'modinfo' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'current-state' </td> <td> '1' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'truncate' </td> <td> '1' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'module-id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = modinfo -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Id    Loadcnt Module Name                            State
##&nbsp;  0          1 unix                             LOADED/INSTALLED
##&nbsp;  1          1 krtld                            LOADED/INSTALLED
##&nbsp;  2          1 genunix                          LOADED/INSTALLED
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'module_name' => 'unix',
##&nbsp;            'module_id' => '0',
##&nbsp;            'state' => 'LOADED/INSTALLED',
##&nbsp;            'loadcnt' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_name' => 'krtld',
##&nbsp;            'module_id' => '1',
##&nbsp;            'state' => 'LOADED/INSTALLED',
##&nbsp;            'loadcnt' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_name' => 'genunix',
##&nbsp;            'module_id' => '2',
##&nbsp;            'state' => 'LOADED/INSTALLED',
##&nbsp;            'loadcnt' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = modinfo -w
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Id         Loadaddr   Size Info Rev Module Name
##&nbsp;  0 fffffffffb800000 131a28   -   0  unix ()
##&nbsp;  1 fffffffffb8c3728  36a82   -   0  krtld ()
##&nbsp;  2 fffffffffb8f4a20 202e58   -   0  genunix ()
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => '-',
##&nbsp;            'revision' => '0',
##&nbsp;            'module_name' => 'unix ()',
##&nbsp;            'module_id' => '0',
##&nbsp;            'loadaddr' => 'fffffffffb800000',
##&nbsp;            'size' => '131a28'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '-',
##&nbsp;            'revision' => '0',
##&nbsp;            'module_name' => 'krtld ()',
##&nbsp;            'module_id' => '1',
##&nbsp;            'loadaddr' => 'fffffffffb8c3728',
##&nbsp;            'size' => '36a82'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '-',
##&nbsp;            'revision' => '0',
##&nbsp;            'module_name' => 'genunix ()',
##&nbsp;            'module_id' => '2',
##&nbsp;            'loadaddr' => 'fffffffffb8f4a20',
##&nbsp;            'size' => '202e58'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub modinfo {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    my @result_array;
    my $i = -1;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{module_id}   = $1;
            $result_array[$i]->{loadaddr}    = $2;
            $result_array[$i]->{size}        = $3;
            $result_array[$i]->{info}        = $4;
            $result_array[$i]->{revision}    = $5;
            $result_array[$i]->{module_name} = $6;
        } elsif ( $line =~ /(\d+)\s+(\d+)\s+(\S+)\s+(\S+)/ ) {
            $i++;
            $result_array[$i]->{module_id}   = $1;
            $result_array[$i]->{loadcnt}     = $2;
            $result_array[$i]->{module_name} = $3;
            $result_array[$i]->{state}       = $4;
        }

    }
    return \@result_array;

} ## end sub modinfo

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([uU]sage):\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
        if ( $raw_output =~ /(illegal option)/ ) {
            $error         = "InvalidParam";
            $error_message = $1;
        }
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;
