#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris mount command
##
## @description
##    This is a library to parse the raw output of Solaris mount command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mount" will be mount
##
## @synopsis
##    use mount;
##
##    mount(%args);
##
## @author Abhishek Kumar(kabhishe@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Solaris::default::mount;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException ;
use NACL::APISet::Exceptions::MethodNotFoundException ;
use NACL::APISet::Exceptions::CommandFailedException ;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /$command.*:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        }
    }
    return $error;
}

## @name ::mount
##
## @summary Parses the raw output from the "mount" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mount with any of its options as described in the 
##    corresponding cdef file 'mount.cdefs'
## @synopsis
##      mount(%args);
## INPUTS TO METHOD CALL 'mount' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'options' </td> </tr> 
## <tr> <td> '-F' </td> <td> 'file-system-type' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'mount-point' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'check-cmd' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = mount -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/dsk/c0t2d0s0 - / ufs - no rw,intr,largefiles,logging,xattr,onerror=panic
##&nbsp;/devices - /devices devfs - no 
##&nbsp;ctfs - /system/contract ctfs - no 
##&nbsp;proc - /proc proc - no 
##&nbsp;mnttab - /etc/mnttab mntfs - no 
##&nbsp;swap - /etc/svc/volatile tmpfs - no xattr
##&nbsp;objfs - /system/object objfs - no 
##&nbsp;/usr/lib/libc/libc_hwcap2.so.1 - /lib/libc.so.1 lofs - no 
##&nbsp;fd - /dev/fd fd - no rw
##&nbsp;/dev/dsk/c0t2d0s3 - /var ufs - no rw,intr,largefiles,logging,xattr,onerror=panic
##&nbsp;swap - /tmp tmpfs - no xattr
##&nbsp;swap - /var/run tmpfs - no xattr
##&nbsp;[fd20:8b1e:b255:4219:2a0:98ff:fe08:618e]:/vol/test_vol2 - /mnt/vol2 nfs - no rw,xattr
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source_path' => '/ ufs',
##&nbsp;            'options' => 'no rw,intr,largefiles,logging,xattr,onerror=panic',
##&nbsp;            'mount_point' => '/dev/dsk/c0t2d0s0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/devices devfs',
##&nbsp;            'options' => 'no',
##&nbsp;            'mount_point' => '/devices'
##&nbsp;          },
##&nbsp;          .. 
##&nbsp;        ];           
##&nbsp;
##&nbsp;Command = mount -V 10.227.71.55:/vol/vol0 /t/10.227.71.55//vol/vol0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;mount -F nfs 10.227.71.55:/vol/vol0 /t/10.227.71.55/vol/vol0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source_path' => '10.227.71.55:/vol/vol0',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'mount_point' => '/t/10.227.71.55/vol/vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = mount -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/dev/dsk/c0t2d0s0 on / type ufs read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c0 on Thu Oct  6 12:14:25 2011
##&nbsp;/devices on /devices type devfs read/write/setuid/devices/dev=45c0000 on Thu Oct  6 12:14:07 2011
##&nbsp;ctfs on /system/contract type ctfs read/write/setuid/devices/dev=4600001 on Thu Oct  6 12:14:07 2011
##&nbsp;proc on /proc type proc read/write/setuid/devices/dev=4640000 on Thu Oct  6 12:14:07 2011
##&nbsp;mnttab on /etc/mnttab type mntfs read/write/setuid/devices/dev=4680001 on Thu Oct  6 12:14:07 2011
##&nbsp;swap on /etc/svc/volatile type tmpfs read/write/setuid/devices/xattr/dev=46c0001 on Thu Oct  6 12:14:07 2011
##&nbsp;objfs on /system/object type objfs read/write/setuid/devices/dev=4700001 on Thu Oct  6 12:14:07 2011
##&nbsp;/usr/lib/libc/libc_hwcap2.so.1 on /lib/libc.so.1 type lofs read/write/setuid/devices/dev=7400c0 on Thu Oct  6 12:14:18 2011
##&nbsp;fd on /dev/fd type fd read/write/setuid/devices/dev=48c0001 on Thu Oct  6 12:14:25 2011
##&nbsp;/dev/dsk/c0t2d0s3 on /var type ufs read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c3 on Thu Oct  6 12:14:28 2011
##&nbsp;swap on /tmp type tmpfs read/write/setuid/devices/xattr/dev=46c0002 on Thu Oct  6 12:14:28 2011
##&nbsp;swap on /var/run type tmpfs read/write/setuid/devices/xattr/dev=46c0003 on Thu Oct  6 12:14:28 2011
##&nbsp;[fd20:8b1e:b255:4219:2a0:98ff:fe08:618e]:/vol/test_vol2 on /mnt/vol2 type nfs remote/read/write/setuid/devices/xattr/dev=4940005 on Thu Oct  6 13:37:11 2011
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/dsk/c0t2d0s0',
##&nbsp;            'options' => 'read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c0',
##&nbsp;            'type' => 'ufs',
##&nbsp;            'mount_point' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/devices',
##&nbsp;            'options' => 'read/write/setuid/devices/dev=45c0000',
##&nbsp;            'type' => 'devfs',
##&nbsp;            'mount_point' => '/devices'
##&nbsp;          },
##&nbsp;          ... 
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = mount
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/ on /dev/dsk/c0t2d0s0 read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c0 on Thu Oct  6 12:14:25 2011
##&nbsp;/devices on /devices read/write/setuid/devices/dev=45c0000 on Thu Oct  6 12:14:07 2011
##&nbsp;/system/contract on ctfs read/write/setuid/devices/dev=4600001 on Thu Oct  6 12:14:07 2011
##&nbsp;/proc on proc read/write/setuid/devices/dev=4640000 on Thu Oct  6 12:14:07 2011
##&nbsp;/etc/mnttab on mnttab read/write/setuid/devices/dev=4680001 on Thu Oct  6 12:14:07 2011
##&nbsp;/etc/svc/volatile on swap read/write/setuid/devices/xattr/dev=46c0001 on Thu Oct  6 12:14:07 2011
##&nbsp;/system/object on objfs read/write/setuid/devices/dev=4700001 on Thu Oct  6 12:14:07 2011
##&nbsp;/lib/libc.so.1 on /usr/lib/libc/libc_hwcap2.so.1 read/write/setuid/devices/dev=7400c0 on Thu Oct  6 12:14:18 2011
##&nbsp;/dev/fd on fd read/write/setuid/devices/dev=48c0001 on Thu Oct  6 12:14:25 2011
##&nbsp;/var on /dev/dsk/c0t2d0s3 read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c3 on Thu Oct  6 12:14:28 2011
##&nbsp;/tmp on swap read/write/setuid/devices/xattr/dev=46c0002 on Thu Oct  6 12:14:28 2011
##&nbsp;/var/run on swap read/write/setuid/devices/xattr/dev=46c0003 on Thu Oct  6 12:14:28 2011
##&nbsp;/mnt/vol2 on [fd20:8b1e:b255:4219:2a0:98ff:fe08:618e]:/vol/test_vol2 remote/read/write/setuid/devices/xattr/dev=4940005 on Thu Oct  6 13:37:11 2011
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source_path' => '/dev/dsk/c0t2d0s0',
##&nbsp;            'options' => 'read/write/setuid/devices/intr/largefiles/logging/xattr/onerror=panic/dev=7400c0',
##&nbsp;            'mount_point' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'source_path' => '/devices',
##&nbsp;            'options' => 'read/write/setuid/devices/dev=45c0000',
##&nbsp;            'mount_point' => '/devices'
##&nbsp;          },
##&nbsp;          ... 
##&nbsp;          {
##&nbsp;            'source_path' => '[fd20:8b1e:b255:4219:2a0:98ff:fe08:618e]:/vol/test_vol2',
##&nbsp;            'options' => 'remote/read/write/setuid/devices/xattr/dev=4940005',
##&nbsp;            'mount_point' => '/mnt/vol2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mount {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    if($error_level) {
        $error = _check_error(raw_output=>$output,command=>"mount",
                              error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
            $error->throw(text=>$error_message,response_object => $resp);
        }
    }

    my @lines = get_lines(raw_output=>$output,first_line=>"");

    if($command =~ /mount -v\s*$/) {
        foreach my $line (@lines) {
            my %hash;
            if ($line =~ /(.*) on (.*) type (.*) (.*) on/) {
                $hash{'source_path'} = $1;
                $hash{'mount_point'} = $2;
                $hash{'type'} = $3;
                $hash{'options'} = $4;
                push (@result_array, \%hash);
            }
        }
    } elsif ($command =~ /mount -V (.*)/) {
        my $check_line = $lines[0];
        my %attr;
        if ($check_line  =~ /mount -F (.*) (.*) (.*)/) {
            $attr{'type'} = $1;
            $attr{'source_path'} = $2;
            $attr{'mount_point'} = $3;
            push (@result_array, \%attr);
        }
    } elsif ($command =~ /mount\s*$/) {
        foreach my $row (@lines) {
            my %attr_all;
            if ($row =~ /(.*) on (.*) (.*) on/) {
                $attr_all{'mount_point'} = $1;
                $attr_all{'source_path'} = $2;
                $attr_all{'options'} = $3;
                push (@result_array, \%attr_all);
            }
        }
    } elsif ($command =~ /mount -p\s*$/) { 
        foreach my $entry (@lines) { 
            my %vfstab;
            if ($entry =~ /(.*) - (.*) - (.*)/) {
                $vfstab{'mount_point'} = $1;
                $vfstab{'source_path'} = $2;
                $vfstab{'options'} = $3;
                push (@result_array, \%vfstab);
            }
        }
    } else {
        return [];
    }
    return \@result_array; #to be updated to return a reference

}
1;
