# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/netstat.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Solaris netstat command"
##
## @description
##    This is a library to parse the raw output of Solaris netstat command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "netstat" will be netstat()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::netstat;
##
##    netstat(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::netstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::netstat
##
## @summary Parses the raw output from the "netstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netstat with any of its options as described in the
##    corresponding cdef file 'netstat.cdefs'
## @synopsis
##      netstat(%args);
## INPUTS TO METHOD CALL 'netstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'dhcp-interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'multicast' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'memory' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'net-show' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'security-attributes' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numbers' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'route' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'address-family' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'filter' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netstat -D
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Interface  State         Sent  Recv  Declined  Flags
##&nbsp;e1000g0    SELECTING      272     0         0  [PRIMARY]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => 'PRIMARY',
##&nbsp;            'declined' => '0',
##&nbsp;            'sent' => '272',
##&nbsp;            'state' => 'SELECTING',
##&nbsp;            'recv' => '0',
##&nbsp;            'interface' => 'e1000g0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Net to Media Table: IPv4
##&nbsp;Device   IP Address               Mask      Flags      Phys Addr
##&nbsp;------ -------------------- --------------- -------- ---------------
##&nbsp;e1000g0 sw-219-cr1.lab.eng.btc.netapp.in 255.255.255.255          00:00:0c:9f:f0:db
##&nbsp;e1000g0 sun219-32            255.255.255.255 SPLA     00:14:4f:40:cc:34
##&nbsp;Net to Media Table: IPv6
##&nbsp;If   Physical Address    Type      State      Destination/Mask
##&nbsp;----- -----------------  ------- ------------ ---------------------------
##&nbsp;e1000g0 33:33:00:00:00:01  other   REACHABLE    ff02::1
##&nbsp;e1000g0 00:26:98:09:be:41  dynamic REACHABLE    fd20:8b1e:b255:4219::2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        {
##&nbsp;                          'type' => 'other',
##&nbsp;                          'destination' => 'ff02::1',
##&nbsp;                          'if' => 'e1000g0',
##&nbsp;                          'state' => 'REACHABLE',
##&nbsp;                          'physical_address' => '33:33:00:00:00:01',
##&nbsp;                          'mask' => undef
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'type' => 'dynamic',
##&nbsp;                          'destination' => 'fd20:8b1e:b255:4219::2',
##&nbsp;                          'if' => 'e1000g0',
##&nbsp;                          'state' => 'REACHABLE',
##&nbsp;                          'physical_address' => '00:26:98:09:be:41',
##&nbsp;                          'mask' => undef
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'ipv4' => [
##&nbsp;                        {
##&nbsp;                          'device' => 'e1000g0',
##&nbsp;                          'flags' => '-',
##&nbsp;                          'mask' => '255.255.255.255',
##&nbsp;                          'ip_address' => 'sw-219-cr1.lab.eng.btc.netapp.in',
##&nbsp;                          'physical_address' => '00:00:0c:9f:f0:db'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'device' => 'e1000g0',
##&nbsp;                          'flags' => 'SPLA',
##&nbsp;                          'mask' => '255.255.255.255',
##&nbsp;                          'ip_address' => 'sun219-32',
##&nbsp;                          'physical_address' => '00:14:4f:40:cc:34'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = netstat -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Routing Table: IPv4
##&nbsp;Destination           Gateway           Flags  Ref     Use     Interface
##&nbsp;-------------------- -------------------- ----- ----- ---------- ---------
##&nbsp;default              10.72.219.1          UG        1          8
##&nbsp;1.2.3.5              localhost            UH        1          0 lo0
##&nbsp;1.2.3.6              localhost            UH        1          0 lo0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'ipv4' => [
##&nbsp;                         {
##&nbsp;                           'use' => '8',
##&nbsp;                           'flags' => 'UG',
##&nbsp;                           'ref' => '1',
##&nbsp;                           'gateway' => '10.72.219.1',
##&nbsp;                           'destination' => 'default',
##&nbsp;                           'interface' => undef
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'use' => '0',
##&nbsp;                           'flags' => 'UH',
##&nbsp;                           'ref' => '1',
##&nbsp;                           'gateway' => 'localhost',
##&nbsp;                           'destination' => '1.2.3.5',
##&nbsp;                           'interface' => 'lo0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'use' => '0',
##&nbsp;                           'flags' => 'UH',
##&nbsp;                           'ref' => '1',
##&nbsp;                           'gateway' => 'localhost',
##&nbsp;                           'destination' => '1.2.3.6',
##&nbsp;                           'interface' => 'lo0'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = netstat -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Name  Mtu  Net/Dest      Address        Ipkts  Ierrs Opkts  Oerrs Collis Queue
##&nbsp;lo0   8232 loopback      localhost      119    0     119    0     0      0
##&nbsp;e1000g0 1500 sun219-32     sun219-32      34453  0     1527   0     0      0
##&nbsp;Name  Mtu  Net/Dest                    Address                     Ipkts  Ierrs Opkts  Oerrs Collis
##&nbsp;lo0   8252 ::1                         ::1                         119    0     119    1     2
##&nbsp;e1000g0 1500 fe80::214:4fff:fe40:cc34/10 fe80::214:4fff:fe40:cc34    34455  0     1529   0     0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mtu' => '8232',
##&nbsp;            'dest' => '-',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'address' => 'localhost',
##&nbsp;            'ipkts' => '119',
##&nbsp;            'queue' => '0',
##&nbsp;            'net' => 'loopback',
##&nbsp;            'opkts' => '119',
##&nbsp;            'collis' => '0',
##&nbsp;            'oerrs' => '0',
##&nbsp;            'name' => 'lo0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mtu' => '1500',
##&nbsp;            'dest' => '-',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'address' => 'sun219-32',
##&nbsp;            'ipkts' => '34453',
##&nbsp;            'queue' => '0',
##&nbsp;            'net' => 'sun219-32',
##&nbsp;            'opkts' => '1527',
##&nbsp;            'collis' => '0',
##&nbsp;            'oerrs' => '0',
##&nbsp;            'name' => 'e1000g0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mtu' => '8252',
##&nbsp;            'dest' => '',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'address' => '::1',
##&nbsp;            'ipkts' => '119',
##&nbsp;            'net' => '::1',
##&nbsp;            'opkts' => '119',
##&nbsp;            'collis' => '2',
##&nbsp;            'oerrs' => '1',
##&nbsp;            'name' => 'lo0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mtu' => '1500',
##&nbsp;            'dest' => '10',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'address' => 'fe80::214:4fff:fe40:cc34',
##&nbsp;            'ipkts' => '34455',
##&nbsp;            'net' => 'fe80::214:4fff:fe40:cc34',
##&nbsp;            'opkts' => '1529',
##&nbsp;            'collis' => '0',
##&nbsp;            'oerrs' => '0',
##&nbsp;            'name' => 'e1000g0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = netstat -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;streams allocation:\r\n
##&nbsp;                                         cumulative  allocation
##&nbsp;                      current   maximum       total    failures
##&nbsp;streams                    97       120        1207           0
##&nbsp;queues                    199       222        1930           0
##&nbsp;mblk                     6607     25893       26312           0
##&nbsp;dblk                     6607     25864       52415           0
##&nbsp;2565 Kbytes allocated for streams data
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'streams_allocation' => [
##&nbsp;                                       {
##&nbsp;                                         'maximum' => '120',
##&nbsp;                                         'allocation_failures' => '0',
##&nbsp;                                         'current' => '97',
##&nbsp;                                         'stream' => 'streams',
##&nbsp;                                         'cumulative_total' => '1207'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'maximum' => '222',
##&nbsp;                                         'allocation_failures' => '0',
##&nbsp;                                         'current' => '199',
##&nbsp;                                         'stream' => 'queues',
##&nbsp;                                         'cumulative_total' => '1930'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'maximum' => '25893',
##&nbsp;                                         'allocation_failures' => '0',
##&nbsp;                                         'current' => '6607',
##&nbsp;                                         'stream' => 'mblk',
##&nbsp;                                         'cumulative_total' => '26312'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'maximum' => '25864',
##&nbsp;                                         'allocation_failures' => '0',
##&nbsp;                                         'current' => '6607',
##&nbsp;                                         'stream' => 'dblk',
##&nbsp;                                         'cumulative_total' => '52415'
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;             'kbytes_allocated_for_streams_data' => '2565'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = netstat -g
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Group Memberships: IPv4
##&nbsp;Interface Group                RefCnt
##&nbsp;--------- -------------------- ------
##&nbsp;lo0       224.0.0.1                 1
##&nbsp;e1000g1   224.0.0.1                 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'ipv4' => [
##&nbsp;                         {
##&nbsp;                           'group' => '224.0.0.1',
##&nbsp;                           'interface' => 'lo0',
##&nbsp;                           'refcnt' => '1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'group' => '224.0.0.1',
##&nbsp;                           'interface' => 'e1000g1',
##&nbsp;                           'refcnt' => '1'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = netstat -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;TCP: IPv4
##&nbsp;Local/Remote Address Swind  Snext     Suna   Rwind  Rnext     Rack    Rto   Mss     State
##&nbsp;-------------------- ----- -------- -------- ----- -------- -------- ----- ----- -----------
##&nbsp;sun219-32.telnet
##&nbsp;10.72.8.242.48863     6144 1b1848e3 1b1848e3 49232 b451b0f6 b451b0f6   407  1448 ESTABLISHED
##&nbsp;sun219-32.login
##&nbsp;10.72.8.242.788      10240 3a1b6f69 3a1b6f69 49232 3adb4024 3adb4006  1970  1448 ESTABLISHED
##&nbsp;Active UNIX domain sockets
##&nbsp;Address  Type          Vnode     Conn  Local Addr      Remote Addr
##&nbsp;ffffffff94abfc88 stream-ord 00000000 ffffffff95140cc0                /tmp/.X11-unix/X0
##&nbsp;ffffffff94df61d0 stream-ord 00000000 00000000 /tmp/.X11-unix/X0
##&nbsp;ffffffff94df6398 stream-ord 00000000 00000000 /tmp/.X11-unix/X0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'ipv4' => [
##&nbsp;                         {
##&nbsp;                           'rwind' => '49232',
##&nbsp;                           'rnext' => 'b451b0f6',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '10.72.8.242.48863',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'suna' => '1b1848e3',
##&nbsp;                           'mss' => '1448',
##&nbsp;                           'local_address' => 'sun219-32.telnet',
##&nbsp;                           'rack' => 'b451b0f6',
##&nbsp;                           'rto' => '407',
##&nbsp;                           'snext' => '1b1848e3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'rwind' => '49232',
##&nbsp;                           'rnext' => '3adb4024',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '10.72.8.242.788',
##&nbsp;                           'swind' => '10240',
##&nbsp;                           'suna' => '3a1b6f69',
##&nbsp;                           'mss' => '1448',
##&nbsp;                           'local_address' => 'sun219-32.login',
##&nbsp;                           'rack' => '3adb4006',
##&nbsp;                           'rto' => '1970',
##&nbsp;                           'snext' => '3a1b6f69'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;             'active_unix_domain_sockets' => [
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => 'ffffffff95140cc0',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94abfc88'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => '00000000',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94df61d0'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => '00000000',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94df6398'
##&nbsp;                                               }
##&nbsp;                                             ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = netstat
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;TCP: IPv4
##&nbsp;Local Address        Remote Address    Swind Send-Q Rwind Recv-Q    State
##&nbsp;-------------------- -------------------- ----- ------ ----- ------ -----------
##&nbsp;sun219-32.telnet     10.72.8.242.48863     6144      0 49232      0 ESTABLISHED
##&nbsp;sun219-32.login      10.72.8.242.788       7168      0 49232     27 ESTABLISHED
##&nbsp;Active UNIX domain sockets
##&nbsp;Address  Type          Vnode     Conn  Local Addr      Remote Addr
##&nbsp;ffffffff94abfc88 stream-ord 00000000 ffffffff95140cc0                /tmp/.X11-unix/X0
##&nbsp;ffffffff94df61d0 stream-ord 00000000 00000000 /tmp/.X11-unix/X0
##&nbsp;ffffffff94df6398 stream-ord 00000000 00000000 /tmp/.X11-unix/X0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'ipv4' => [
##&nbsp;                         {
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.242.48863',
##&nbsp;                           'local_address' => 'sun219-32.telnet',
##&nbsp;                           'rwind' => '49232',
##&nbsp;                           'recv_q' => '0',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'send_q' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'swind' => '7168',
##&nbsp;                           'remote_address' => '10.72.8.242.788',
##&nbsp;                           'local_address' => 'sun219-32.login',
##&nbsp;                           'rwind' => '49232',
##&nbsp;                           'recv_q' => '27',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'send_q' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;             'active_unix_domain_sockets' => [
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => 'ffffffff95140cc0',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94abfc88'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => '00000000',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94df61d0'
##&nbsp;                                               },
##&nbsp;                                               {
##&nbsp;                                                 'remote_address' => '-',
##&nbsp;                                                 'local_address' => '/tmp/.X11-unix/X0',
##&nbsp;                                                 'conn' => '00000000',
##&nbsp;                                                 'vnode' => '00000000',
##&nbsp;                                                 'type' => 'stream-ord',
##&nbsp;                                                 'address' => 'ffffffff94df6398'
##&nbsp;                                               }
##&nbsp;                                             ]
##&nbsp;           }
##&nbsp;         ];
sub netstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values, @head1 );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( $count, $ip_version ) = 0;

    if ( $cmd =~ /-D/ ) {
        @head = split( /\s+/, $lines[0] );
        foreach my $line (@lines) {
            if ( $line =~ /(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+\[(.*)\]/ )
            {
                splice(
                    @values, 0,    $#head, "$1", "$2",
                    "$3",    "$4", "$5",   "$6"
                );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            }
        }

    } elsif ( $cmd =~ /-p/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/IP Address/ip_address/;
            $lines[$i] =~ s/Destination\/Mask/destination mask/;
            $lines[$i] =~ s/Phys Addr/physical_address/;
            $lines[$i] =~ s/Physical Address/physical_address/;
            if ( $lines[$i] =~ /^Device.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /^If.*/ ) {
                $count = 0;
                @head = split( /\s+/, $lines[$i] );
            }
            if (   $lines[$i] =~ /(\w+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(\S*)/
                && $lines[$i] !~ /$head[0]/ ) {
                if ( $head[0] =~ /Device/ ) {
                    if ( $lines[$i] =~ /^(\w+)\s+(\S+)\s+(\S+)\s+(\S+)$/ ) {
                        splice( @values, 0, $#head, "$1", "$2", "$3", "-",
                            "$4" );
                        for ( my $j = 0; $j <= $#head; $j++ ) {
                            $head[$j] =~ s/-/_/;
                            $result_array[0]->{ipv4}->[$count]
                                ->{ to_key_name( item => $head[$j] ) }
                                = $values[$j];
                        }
                        $count++;
                    } else {
                        @values = split( /\s+/, $lines[$i] );
                        for ( my $j = 0; $j <= $#head; $j++ ) {
                            $result_array[0]->{ipv4}->[$count]
                                ->{ to_key_name( item => $head[$j] ) }
                                = $values[$j];
                        }
                        $count++;
                    }
                } elsif ( $head[0] =~ /If/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    my @val = split( /\//, $values[$#values] );
                    for ( my $j = 0; $j <= $#head - 1; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{ipv6}->[$count]
                            ->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $result_array[0]->{ipv6}->[$count]
                        ->{ to_key_name( item => $head[ $#head - 1 ] ) }
                        = $val[0];
                    $result_array[0]->{ipv6}->[$count]
                        ->{ to_key_name( item => $head[$#head] ) } = $val[1];
                    $count++;
                }
            }
        }
    } elsif ( $cmd =~ /-r/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/Destination\/Mask/destination mask/;
            if ( $lines[$i] =~ /^Destination.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /^destination\s+mask.*/ ) {
                $count = 0;
                @head1 = split( /\s+/, $lines[$i] );
            }
            if ( $lines[$i]
                =~ /(\S+)\s+(\S+)\s+(\w+)\s+(\d+)\s+(\d+)\s*(\w*)/ ) {
                if ( $head1[0] =~ /destination/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    my @val = split( /\//, $values[0] );
                    for ( my $j = 2; $j <= $#head1; $j++ ) {
                        $result_array[0]->{ipv6}->[$count]
                            ->{ to_key_name( item => $head1[$j] ) }
                            = $values[ $j - 1 ];
                    }
                    $result_array[0]->{ipv6}->[$count]
                        ->{ to_key_name( item => $head1[0] ) } = $val[0];
                    $result_array[0]->{ipv6}->[$count]->{ $head1[1] }
                        = $val[1];
                    $count++;
                } elsif ( $head[0] =~ /Destination/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#head; $j++ ) {
                        $result_array[0]->{ipv4}->[$count]
                            ->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $count++;
                }
            }
        }
    } elsif ( $cmd =~ /-i/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/Net\/Dest/net dest/;
            if ( $lines[$i] =~ /.*Queue$/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /.*Collis$/ ) {
                @head = split( /\s+/, $lines[$i] );
            }
            if ( $lines[$i]
                =~ /(\w+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
                ) {
                splice(
                    @values, 0,    $#head, "$1", "$2", "$3", "-",
                    "$4",    "$5", "$6",   "$7", "$8", "$9", "$10"
                );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            } elsif ( $lines[$i]
                =~ /(\w+)\s+(\d+)\s+(\w*\S+\w*\/*\d*)\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s*(\d*)/
                ) {
                my @val = split( /\//, $3 );
                splice(
                    @values,   0,         $#head, "$1",  "$2",
                    "$val[0]", "$val[1]", "$4",   "$5",  "$6",
                    "$7",      "$8",      "$9",   "$10", "$11"
                );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;

            }
        }
    } elsif ( $cmd =~ /-m/ ) {
        for ( my $i = 2; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/total/cumulative_total/;
            $lines[$i] =~ s/failures/allocation_failures/;
            if ( $lines[$i] =~ /^current.*/ ) {
                @head = split( /\s+/, $lines[$i] );
                unshift( @head, 'stream' );
            }
            if ( $lines[$i] =~ /(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[0]->{streams_allocation}
                        ->[$count]{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $count++;
            } elsif ( $lines[$i] =~ /(\d+)\s+(\w+.*\w+)/ ) {
                $result_array[0]->{ to_key_name( item => $2 ) } = $1;

            }
        }
    } elsif ( $cmd =~ /-g/ ) {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /Group Memberships:\s+(.*)/ ) {
                $ip_version = to_key_name( item =>$1);
            }
            if ( $lines[$i] =~ /^Interface.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /^If.*/ ) {
                $count = 0;
                @head = split( /\s+/, $lines[$i] );
            }
            if ( $lines[$i] =~ /(\w+)\s+(\w+[-]\S+|\S+)\s+(\d+)/ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[0]->{$ip_version}->[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            } elsif ( $lines[$i] =~ /(\w+)\s+(\w*[::].*\w+)\s+(\d+)/
                && $lines[$i] !~ /$head[0]/
                && $lines[$i] !~ /^-----.*/ ) {
                @values = split( /\s+/, $lines[$i] );
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[0]->{$ip_version}->[$count]
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                }
                $count++;
            }
        }
    } else {
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            $lines[$i] =~ s/Local Address/local_address/;
            $lines[$i] =~ s/Local\//local_address/;
            $lines[$i] =~ s/Remote Address/ remote_address/;
            $lines[$i] =~ s/Local Addr/local_address/;
            $lines[$i] =~ s/Remote Addr/remote_address/;
            if ( $lines[$i] =~ /TCP:\s+(.*)/ ) {
                $ip_version = to_key_name( item =>$1);
            }
            if ( $lines[$i] =~ /^local_address.*/ ) {
                @head = split( /\s+/, $lines[$i] );
            } elsif ( $lines[$i] =~ /^Address\s+Type.*/ ) {
                $count = 0;
                @head = split( /\s+/, $lines[$i] );
            }
            if ( $cmd =~ /-v/ ) {
                if ( $lines[$i] =~ /^(\S+)$/ ) {
                    $result_array[0]->{$ip_version}->[$count]
                        ->{ to_key_name( item => $head[0] ) } = $1;
                } elsif ( $lines[$i]
                    =~ /(\S+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(.*)/
                    ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 1; $j <= $#head; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{$ip_version}->[$count]
                            ->{ to_key_name( item => $head[$j] ) }
                            = $values[ $j - 1 ];
                    }
                    $count++;
                } elsif ( $lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/
                    && $lines[$i] !~ /$head[0]/
                    && $lines[$i] !~ /^-----.*/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#head - 1; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{active_unix_domain_sockets}
                            ->[$count]->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $result_array[0]->{active_unix_domain_sockets}->[$count]
                        ->{ to_key_name( item => $head[$#head] ) }
                        = $values[$#head] ? $values[$#head] : '-';
                    $count++;
                }
            } else {

                if ( $lines[$i]
                    =~ /(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(.*)/
                    ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#head; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $result_array[0]->{$ip_version}->[$count]
                            ->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $count++;
                } elsif ( $lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*(.*)/
                    && $lines[$i] !~ /$head[0]/
                    && $lines[$i] !~ /^-----.*/
                    && $lines[$i] !~ /Active UNIX domain.*/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#head - 1; $j++ ) {
                        $head[$j] =~ s/-/_/;
                        $values[$j] =~ s/\cH// if($cmd =~ /-n/);
                        $result_array[0]->{active_unix_domain_sockets}
                            ->[$count]->{ to_key_name( item => $head[$j] ) }
                            = $values[$j];
                    }
                    $result_array[0]->{active_unix_domain_sockets}->[$count]
                        ->{ to_key_name( item => $head[$#head] ) }
                        = $values[$#head] ? $values[$#head] : '-';
                    $count++;
                }
            }
        }
    }
    return \@result_array;
} ## end sub netstat

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /unknown.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(netstat:|Usage:)\s+.*|illegal option.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
