#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris ping command output
##
## @description
##    This is a library to parse the raw output of Solaris ping command.
##    This command will handle both ipv4 & ipv6. 
##
## @synopsis
##    use ping;
##
##    ping(%args);
##
## @author Abhishek Kumar(kabhishe@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::ping;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(ping|[Uu]sage):\s(.*)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
        if ( $$error_message =~ /(illegal) option/ ) {
            $error = "InvalidParam";
        } elsif ( $$error_message !~ /unknown host/i ) {
            $$error_message
                = "Command failed due to wrong usage or dependencies";
        }

    }elsif ( $raw_output =~ /(no\s+answer\s+from\s+(.*))/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    return $error;
}

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'ttl' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'gateway' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'packet-size' </td> </tr> 
## <tr> <td> '-P' </td> <td> 'tos' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'addr-family' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'loose-source' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'count' </td> </tr> 
## <tr> <td> '-F' </td> <td> 'flow-label' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'data-size' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'bypass-routing-tables' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'interval' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'record-route' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'traffic-class' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interface-address' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'port' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'so-debug' </td> </tr> 
## <tr> <td> '-U' </td> <td> 'udp-packets' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all-addresses' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'numeric' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'supress-loopback-multicast' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ping 10.72.219.44
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;10.72.219.44 is alive
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'state' => 'alive',
##&nbsp;            'host' => '10.72.219.44'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping fd20:8b1e:b255:8305:250:56ff:feae:4ea8
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fd20:8b1e:b255:8305:250:56ff:feae:4ea8 is alive
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'state' => 'alive',
##&nbsp;            'host' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -s 10.72.219.44 56 2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING 10.72.219.44: 56 data bytes
##&nbsp;64 bytes from lnx219-44 (10.72.219.44): icmp_seq=0. time=0.517 ms
##&nbsp;64 bytes from lnx219-44 (10.72.219.44): icmp_seq=1. time=0.241 ms
##&nbsp;
##&nbsp;----10.72.219.44 PING Statistics----
##&nbsp;2 packets transmitted, 2 packets received, 0% packet loss
##&nbsp;round-trip (ms)  min/avg/max/stddev = 0.241/0.379/0.517/0.20
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '56',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '0.517 ms',
##&nbsp;                           'icmp_seq' => '0',
##&nbsp;                           'ip' => '10.72.219.44',
##&nbsp;                           'packet_size' => '64',
##&nbsp;                           'host' => 'lnx219-44'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '0.241 ms',
##&nbsp;                           'icmp_seq' => '1',
##&nbsp;                           'ip' => '10.72.219.44',
##&nbsp;                           'packet_size' => '64',
##&nbsp;                           'host' => 'lnx219-44'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'host' => '10.72.219.44'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my %hash;
    if ( $output =~ /(.*)\s+is\s+alive\s/ ) {
        $hash{host}  = $1;
        $hash{state} = "alive";
    } elsif ( $output =~ /PING (.+): (\d+) data bytes/ ) {
        $hash{'host'} = $1;
        $hash{'bytes'} = $2;
        my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>"PING .+: .+ data bytes",
                          response_object=>$resp);
        my $line = shift(@lines);

        $hash{'stats'} = [];

        while (@lines) {
            my %internal_hash;
            $line = $lines[0];
            shift(@lines);
            if ($line =~ /(.+)\s+bytes\s+from\s+(.+)\s+\((.+)\):\s+(.+)/) {
                $internal_hash{'packet_size'} = $1;
                $internal_hash{'host'} = $2;
                $internal_hash{'ip'} = $3;
                my $rest = $4;
                my @fields = split(/\s+/,$rest);
                my $time_val;          
                foreach my $field(@fields) {
                    my ($name,$value) = split(/=/,$field);
                    if(($name =~ /icmp_seq/) && ($value =~ /\./)){
                        chop($value);
                    }
                    if($name =~ /time/) {
                        $time_val = $value; 
                        next;
                    } 
                    if($name =~ /ms/) {
                       $value = $time_val.' '. $name;   
                       $name = 'time';
                    } 
                    $internal_hash{$name} = $value;
                }
                push (@{$hash{'stats'}}, \%internal_hash);
            }
            last if($line =~ /----.+ PING Statistics----/);
        }

        $line = $lines[0];
        # 2 packets transmitted, 2 packets received, 0% packet loss
        if ($line =~ /(\d+) packets transmitted, (\d+) received, (\d+%) packet loss/) {
            $hash{packets_transmitted} = $1;
            $hash{received} = $2;
            $hash{loss} = $3;
            shift(@lines);
            $line = $lines[0];
        }
        # round-trip (ms)  min/avg/max/stddev = 0.199/0.227/0.255/0.040
        if($line =~ /round-trip \(ms\)\s+(.+)\s+=\s+(.+)\s+ms/) {
            my @names = split(/\//, $1);
            my @values = split(/\//, $2);
            my $i;
            foreach my $name (@names) {
                $hash{$name}=$values[$i];
                $i++;
            }
        }
    }
    @result_array = ( \%hash );
    return \@result_array;
}

1;
