#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/pkginfo.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris pkginfo command output
##
## @description
##    This is a library to parse the raw output of Solaris pkginfo command.
##
## @synopsis
##    use pkginfo;
##
##    pkginfo(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::pkginfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error      = undef;
    my $error_message;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /([Uu]sage):\s(.*)/ ) {
        $error_message = $raw_output;
        $error         = "CommandFailed";
    } elsif ( $raw_output =~ /ERROR: / ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level(>0) "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

## @name ::pkginfo
##
## @summary Parses the raw output from the "pkginfo" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from pkginfo with any of its options as described in the 
##    corresponding cdef file 'pkginfo.cdefs'
## @synopsis
##      pkginfo(%args);
## INPUTS TO METHOD CALL 'pkginfo' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-R' </td> <td> 'root-path' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '0' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'check-install' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'arch' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'pkg-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'full-install' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'device' </td> <td> '0' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'extracted-list' </td> <td> '1' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'long-list' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'partial-install' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'category' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pkginfo
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      BRCMbnx                          Broadcom NetXtreme II Gigabit Ethernet Adapter Driver
##&nbsp;system      CADP160                          Adaptec Ultra160 SCSI Host Adapter Driver
##&nbsp;system      HPFC                             Agilent Fibre Channel HBA Driver
##&nbsp;system      SUNWzip                          The Info-Zip (zip) compression utility
##&nbsp;system      SUNWzlib                         The Zip compression library
##&nbsp;system      SUNWzsh                          Z shell (zsh)
##&nbsp;system      SYMhisl                          Symbios 895A, 896 and 1010 SCSI driver
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Broadcom NetXtreme II Gigabit Ethernet Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'BRCMbnx'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Adaptec Ultra160 SCSI Host Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'CADP160'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Agilent Fibre Channel HBA Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'HPFC'
##&nbsp;          },
##&nbsp;            'name' => 'The Info-Zip (zip) compression utility',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzip'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'The Zip compression library',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzlib'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Z shell (zsh)',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzsh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Symbios 895A, 896 and 1010 SCSI driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SYMhisl'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -x
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;BRCMbnx                           Broadcom NetXtreme II Gigabit Ethernet Adapter Driver
##&nbsp;                                  (i386) 11.10.0,REV=2007.06.20.13.12
##&nbsp;CADP160                           Adaptec Ultra160 SCSI Host Adapter Driver
##&nbsp;                                  (i386) 1.21,REV=2005.01.17.23.31
##&nbsp;HPFC                              Agilent Fibre Channel HBA Driver
##&nbsp;                                  (i386) 1.1.0,REV=2005.01.06.07.16
##&nbsp;SUNWzsh                           Z shell (zsh)
##&nbsp;                                  (i386) 11.10.0,REV=2005.01.08.01.09
##&nbsp;SYMhisl                           Symbios 895A, 896 and 1010 SCSI driver
##&nbsp;                                  (i386) 11.9.0,REV=2005.01.06.07.16
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'revision' => '2007.06.20.13.12',
##&nbsp;            'version' => '11.10.0',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'Broadcom NetXtreme II Gigabit Ethernet Adapter Driver',
##&nbsp;            'pkginst' => 'BRCMbnx'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'revision' => '2005.01.17.23.31',
##&nbsp;            'version' => '1.21',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'Adaptec Ultra160 SCSI Host Adapter Driver',
##&nbsp;            'pkginst' => 'CADP160'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'revision' => '2005.01.06.07.16',
##&nbsp;            'version' => '1.1.0',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'Agilent Fibre Channel HBA Driver',
##&nbsp;            'pkginst' => 'HPFC'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'revision' => '2005.01.08.01.09',
##&nbsp;            'version' => '11.10.0',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'Z shell (zsh)',
##&nbsp;            'pkginst' => 'SUNWzsh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'revision' => '2005.01.06.07.16',
##&nbsp;            'version' => '11.9.0',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'Symbios 895A, 896 and 1010 SCSI driver',
##&nbsp;            'pkginst' => 'SYMhisl'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/
##&nbsp;/usr
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'basedir' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'basedir' => '/usr'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -a i386
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      BRCMbnx                          Broadcom NetXtreme II Gigabit Ethernet Adapter Driver
##&nbsp;system      CADP160                          Adaptec Ultra160 SCSI Host Adapter Driver
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Broadcom NetXtreme II Gigabit Ethernet Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'BRCMbnx'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Adaptec Ultra160 SCSI Host Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'CADP160'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -c system
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      BRCMbnx                          Broadcom NetXtreme II Gigabit Ethernet Adapter Driver
##&nbsp;system      CADP160                          Adaptec Ultra160 SCSI Host Adapter Driver
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Broadcom NetXtreme II Gigabit Ethernet Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'BRCMbnx'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'Adaptec Ultra160 SCSI Host Adapter Driver',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'CADP160'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -i SUNWadmj
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      SUNWadmj Admin/Install Java Extension Libraries
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Admin/Install Java Extension Libraries',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWadmj'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -v 11.10.0,REV=2006.05.18.01.46
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      SUNWzfskr                        ZFS Kernel (Root)
##&nbsp;system      SUNWzfsr                         ZFS (Root)
##&nbsp;system      SUNWzfsu                         ZFS (Usr)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'ZFS Kernel (Root)',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzfskr'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'ZFS (Root)',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzfsr'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'ZFS (Usr)',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWzfsu'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -R / SUNWxwpls
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system      SUNWxwpls X Server x86 platform software
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'X Server x86 platform software',
##&nbsp;            'category' => 'system',
##&nbsp;            'pkginst' => 'SUNWxwpls'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -l SUNWxwpls
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;   PKGINST:  SUNWxwpls
##&nbsp;      NAME:  X Server x86 platform software
##&nbsp;  CATEGORY:  system
##&nbsp;      ARCH:  i386
##&nbsp;   VERSION:  6.6.2.0,REV=0.2004.12.10
##&nbsp;   BASEDIR:  /usr
##&nbsp;    VENDOR:  Sun Microsystems, Inc.
##&nbsp;      DESC:  X Server x86 platform software
##&nbsp;    PSTAMP:  xbuild1020070502141344
##&nbsp;  INSTDATE:  Feb 08 2011 22:57
##&nbsp;   HOTLINE:  Please contact your local service provider
##&nbsp;    STATUS:  completely installed
##&nbsp;     FILES:      516 installed pathnames
##&nbsp;                  11 shared pathnames
##&nbsp;                  55 directories
##&nbsp;               16518 blocks used (approx)
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hotline' => 'Please contact your local service provider',
##&nbsp;            'files' => {
##&nbsp;                         'blocks_used_approx' => '16518',
##&nbsp;                         'directories' => '55',
##&nbsp;                         'shared_pathnames' => '11',
##&nbsp;                         'installed_pathnames' => '516'
##&nbsp;                       },
##&nbsp;            'status' => 'completely installed',
##&nbsp;            'version' => '6.6.2.0',
##&nbsp;            'arch' => 'i386',
##&nbsp;            'name' => 'X Server x86 platform software',
##&nbsp;            'basedir' => '/usr',
##&nbsp;            'pkginst' => 'SUNWxwpls',
##&nbsp;            'revision' => '0.2004.12.10',
##&nbsp;            'instdate' => 'Feb 08 2011 22:57',
##&nbsp;            'desc' => 'X Server x86 platform software',
##&nbsp;            'category' => 'system',
##&nbsp;            'pstamp' => 'xbuild1020070502141344',
##&nbsp;            'vendor' => 'Sun Microsystems, Inc.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pkginfo -d /var/spool/pkg/NTAPSANTool.pkg
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;application NTAPSANTool Solaris SAN Host Utilities for NetApp Storage.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Solaris SAN Host Utilities for NetApp Storage.',
##&nbsp;            'category' => 'application',
##&nbsp;            'pkginst' => 'NTAPSANTool'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pkginfo {
    my %args = @_;
    my (@result_array);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $cmd         = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my $count = -1;
    for ( my $i = 0; $i < (@lines); $i++ ) {
        if ( $lines[$i] =~ /PKGINST/ ) {
            $count++;
        }
        if ( $cmd =~ /-l/ && $lines[$i] =~ /[A-Z]+:\s+/ ) {
            my @temp = split( /:\s+/, $lines[$i] );
            my $file;
            if ( $temp[0] =~ /FILES/ ) {
                if($temp[1] =~ /^(\d+)\s+(.*)/) {
                    $result_array[$count]->{ to_key_name( item => $temp[0] ) }
                    ->{ to_key_name( item => $2)} = $1;
                }
                $i++;
                while( $lines[$i] =~ /^(\d+)\s+(.*)/) {
                    $result_array[$count]->{ to_key_name( item => $temp[0] ) }
                        ->{ to_key_name( item => $2)} = $1;
                    $i++;
                }
                $i--;
            } elsif ( $temp[1] =~ /(\d+.*\d+)\,REV=(.*)/) { 
                $result_array[$count]->{version} = $1;
                $result_array[$count]->{revision} = $2;
            } else {
                $result_array[$count]->{ to_key_name( item => $temp[0] ) }
                    = $temp[1];
            }
        }
        if ( $lines[$i] =~ /^\/\w*$/ ) {
            $result_array[$i]->{basedir} = $lines[$i];
        }
        if ( $cmd !~ /-x|-l/ && $lines[$i] =~ /(\w+)\s+([A-Z]+\w+)\s+(.*)/ ) {
            $result_array[$i]->{category} = $1;
            $result_array[$i]->{pkginst}  = $2;
            $result_array[$i]->{name}     = $3;
        }
        if ( $cmd =~ /-x/ && $lines[$i] =~ /([A-Z]+\w+)\s+([A-Z]+.*)/ ) {
            $count++;
            $result_array[$count]->{pkginst} = $1;
            $result_array[$count]->{name}    = $2;
            $i++;
            if ( $lines[$i] =~ /\((.*)\)\s+(\d+.*\d+)\,REV=(.*)/ ) {
                $result_array[$count]->{arch}    = $1;
                $result_array[$count]->{version} = $2;
                $result_array[$count]->{revision} = $3;
            }
        }
    }
    return \@result_array;
} ## end sub pkginfo

1;
