#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/prtdiag.pm#3 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris prtdiag command
##
## @description
##    This is a library to parse the raw output of Solaris prtdiag command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "prtdiag" will be prtdiag)
##
## @synopsis
##    use prtdiag;
## INPUTS TO METHOD CALL 'prtdiag' ON APISET OBJECT
####
##    prtdiag(%args);
##
## @author sheethal@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::prtdiag;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~ /(.*Usage:.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = "Invalid Usage of prtdiag command";
    }
    return $error;
} ## end sub _check_error

## @name ::prtdiag
##
## @summary Parses the raw output from the "prtdiag" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from prtdiag with any of its options as described in the
##    corresponding cdef file 'prtdiag.cdefs'
## @synopsis
##      prtdiag(%args);
## INPUTS TO METHOD CALL 'prtdiag' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'log_output' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = prtdiag
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;System Configuration: Sun Microsystems Sun Fire X4200 Server
##&nbsp;BIOS Configuration: American Megatrends Inc. 080010  08/01/2006
##&nbsp;BMC Configuration: IPMI 2.0 (KCS: Keyboard Controller Style)
##&nbsp;
##&nbsp;==== Processor Sockets ====================================
##&nbsp;
##&nbsp;Version                          Location Tag
##&nbsp;-------------------------------- --------------------------
##&nbsp;Dual Core AMD Opteron(tm) Processor 280 H0
##&nbsp;Dual Core AMD Opteron(tm) Processor 280 H1
##&nbsp;Dual Core AMD Opteron(tm) Processor 280 H2
##&nbsp;Dual Core AMD Opteron(tm) Processor 280 H3
##&nbsp;
##&nbsp;==== Memory Device Sockets ================================
##&nbsp;
##&nbsp;Type    Status Set Device Locator      Bank Locator
##&nbsp;------- ------ --- ------------------- --------------------
##&nbsp;DDR     in use 0   H0_DIMM0            BANK0
##&nbsp;DDR     in use 0   H0_DIMM1            BANK1
##&nbsp;unknown empty  0   H0_DIMM2            BANK2
##&nbsp;unknown empty  0   H0_DIMM3            BANK3
##&nbsp;DDR     in use 0   H1_DIMM0            BANK4
##&nbsp;DDR     in use 0   H1_DIMM1            BANK5
##&nbsp;unknown empty  0   H1_DIMM2            BANK6
##&nbsp;unknown empty  0   H1_DIMM3            BANK7
##&nbsp;
##&nbsp;==== On-Board Devices =====================================
##&nbsp; LSI serial-ATA #1
##&nbsp; Gigabit Ethernet #1
##&nbsp; Gigabit Ethernet #2
##&nbsp; ATI Rage XL VGA
##&nbsp;
##&nbsp;==== Upgradeable Slots ====================================
##&nbsp;
##&nbsp;ID  Status    Type             Description
##&nbsp;--- --------- ---------------- ----------------------------
##&nbsp;0   in use    PCI-X            PCIX SLOT0
##&nbsp;1   available PCI-X            PCIX SLOT1
##&nbsp;2   available PCI-X            PCIX SLOT2
##&nbsp;3   available PCI-X            PCIX SLOT3
##&nbsp;4   available PCI-X            PCIX SLOT4
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bmc_configuration' => ' IPMI 2.0 (KCS: Keyboard Controller Style)',
##&nbsp;            'on_board_devices' => [
##&nbsp;                                    'LSI serial-ATA #1',
##&nbsp;                                    'Gigabit Ethernet #1',
##&nbsp;                                    'Gigabit Ethernet #2',
##&nbsp;                                    'ATI Rage XL VGA'
##&nbsp;                                  ],
##&nbsp;            'upgradeble_slots' => [
##&nbsp;                                    {
##&nbsp;                                      'status' => 'in use',
##&nbsp;                                      'type' => 'PCI X',
##&nbsp;                                      'id' => '0',
##&nbsp;                                      'description' => 'PCIX SLOT0'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'status' => 'available',
##&nbsp;                                      'type' => 'PCI X',
##&nbsp;                                      'id' => '1',
##&nbsp;                                      'description' => 'PCIX SLOT1'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'status' => 'available',
##&nbsp;                                      'type' => 'PCI X',
##&nbsp;                                      'id' => '2',
##&nbsp;                                      'description' => 'PCIX SLOT2'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'status' => 'available',
##&nbsp;                                      'type' => 'PCI X',
##&nbsp;                                      'id' => '3',
##&nbsp;                                      'description' => 'PCIX SLOT3'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'status' => 'available',
##&nbsp;                                      'type' => 'PCI X',
##&nbsp;                                      'id' => '4',
##&nbsp;                                      'description' => 'PCIX SLOT4'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'bios_configuration' => ' American Megatrends Inc. 080010  08/01/2006',
##&nbsp;            'processor_sockets' => [
##&nbsp;                                     {
##&nbsp;                                       'version' => 'Dual Core AMD Opteron tm  Processor 280',
##&nbsp;                                       'location_tag' => 'H0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'version' => 'Dual Core AMD Opteron tm  Processor 280',
##&nbsp;                                       'location_tag' => 'H1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'version' => 'Dual Core AMD Opteron tm  Processor 280',
##&nbsp;                                       'location_tag' => 'H2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'version' => 'Dual Core AMD Opteron tm  Processor 280',
##&nbsp;                                       'location_tag' => 'H3'
##&nbsp;                                     }
##&nbsp;                                   ],
##&nbsp;            'system_configuration' => 'Sun Microsystems Sun Fire X4200 Server',
##&nbsp;            'memory_device_sockets' => [
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK0',
##&nbsp;                                           'device_locator' => 'H0_DIMM0',
##&nbsp;                                           'status' => 'in use',
##&nbsp;                                           'type' => 'DDR',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK1',
##&nbsp;                                           'device_locator' => 'H0_DIMM1',
##&nbsp;                                           'status' => 'in use',
##&nbsp;                                           'type' => 'DDR',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK2',
##&nbsp;                                           'device_locator' => 'H0_DIMM2',
##&nbsp;                                           'status' => 'empty',
##&nbsp;                                           'type' => 'unknown',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK3',
##&nbsp;                                           'device_locator' => 'H0_DIMM3',
##&nbsp;                                           'status' => 'empty',
##&nbsp;                                           'type' => 'unknown',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK4',
##&nbsp;                                           'device_locator' => 'H1_DIMM0',
##&nbsp;                                           'status' => 'in use',
##&nbsp;                                           'type' => 'DDR',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK5',
##&nbsp;                                           'device_locator' => 'H1_DIMM1',
##&nbsp;                                           'status' => 'in use',
##&nbsp;                                           'type' => 'DDR',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK6',
##&nbsp;                                           'device_locator' => 'H1_DIMM2',
##&nbsp;                                           'status' => 'empty',
##&nbsp;                                           'type' => 'unknown',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'bank_locator' => 'BANK7',
##&nbsp;                                           'device_locator' => 'H1_DIMM3',
##&nbsp;                                           'status' => 'empty',
##&nbsp;                                           'type' => 'unknown',
##&nbsp;                                           'set' => '0'
##&nbsp;                                         }
##&nbsp;                                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub prtdiag {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    my $str;
    my @head;
    my @proc_info;
    my $i = 0;
    my $key;
    _check_error(raw_output => $output, response_object => $resp);
    my (@result_array);
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    my $count = 0;

    while (@lines) {
        $str = shift(@lines);
        if ($str =~ /^(System.*:\s+((\w|\s)*))/) {
            $key = $1;
            $result_array[$count]->{to_key_name(item => $key)} = $2;
        }
        if ($str =~ /^(BIOS.*:\s+((\w|\s)*))/) {
            my @head = split(/:/, $str, 2);

            $key = $head[0];
            $result_array[$count]->{to_key_name(item => $key)} = $head[1];
        }
        if ($str =~ /^(BMC.*:\s+((\w|\s)*))/) {
            my @head = split(/:/, $str, 2);
            $key = $head[0];
            $result_array[$count]->{to_key_name(item => $key)} = $head[1];
        }
        if ($str =~ /^(Version)/) {
            $i = 0;
            until ($str =~ /^(=.*)/) {
                $str = shift(@lines);
                $str =~ s/\)/ /;
                $str =~ s/\(/ /;

                if ($str
                    =~ /(\w+\s+\w+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+)\s+(\w+)/) {

                    $result_array[$count]->{processor_sockets}->[$i]
                        ->{version} = $1;
                    $result_array[$count]->{processor_sockets}->[$i]
                        ->{location_tag} = $2;

                    $i++;
                }
            }
        }
        if ($str =~ /^(Type)/) {
            $i = 0;
            until ($str =~ /^(=.*)/) {
                $str = shift(@lines);

                if (   $str =~ /(\w+)\s+(\w+\s+\w+)\s+(\d+)\s+(\w+)\s+(\w+)/
                    || $str =~ /(\w+)\s+(\w+)\s+(\d+)\s+(\w+)\s+(\w+)/) {

                    $result_array[$count]->{memory_device_sockets}->[$i]
                        ->{type} = $1;
                    $result_array[$count]->{memory_device_sockets}->[$i]
                        ->{status} = $2;
                    $result_array[$count]->{memory_device_sockets}->[$i]
                        ->{set} = $3;
                    $result_array[$count]->{memory_device_sockets}->[$i]
                        ->{device_locator} = $4;
                    $result_array[$count]->{memory_device_sockets}->[$i]
                        ->{bank_locator} = $5;
                    $i++;
                }
            }
        }

        if ($str =~ /=.*-Board/) {
            $i   = 0;
            $str = shift(@lines);
            until ($str =~ /^(=.*)/) {
                if ($str =~ /^(\w+)*/) {
                    $result_array[$count]->{on_board_devices}->[$i] = $str;
                    $i++;
                    $str = shift(@lines);
                }

            }
        } elsif ($str =~ /^(ID)/) {
            $i = 0;

            while (@lines) {
                $str = shift(@lines);
                $str =~ s\-\ \;

                if ($str =~ /(\d+)\s+(\w+\s+\w+)\s+(\w+\s+\w+)\s+(\w+\s+\w+)/
                    || $str =~ /(\d+)\s+(\w+)\s+(\w+\s+\w+)\s+(\w+\s+\w+)/) {

                    $result_array[$count]->{upgradeble_slots}->[$i]->{id}
                        = $1;
                    $result_array[$count]->{upgradeble_slots}->[$i]->{status}
                        = $2;
                    $result_array[$count]->{upgradeble_slots}->[$i]->{type}
                        = $3;
                    $result_array[$count]->{upgradeble_slots}->[$i]
                        ->{description} = $4;
                    $i++;
                }
            }
        }

    }


    return \@result_array;
} ## end sub prtdiag
1;

