# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/psrinfo.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse Solaris psrinfo command
##
## @description
##    This is a library to parse the raw output of Solaris psrinfo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "psrinfo" will be psrinfo()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::psrinfo;
##
##    psrinfo(%args);
##
## @author Veerabhadraswamy.P@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::psrinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Exceptions::InvalidParamException;

## @name ::psrinfo
##
## @summary Parses the raw output from the "psrinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from psrinfo with any of its options as described in the
##    corresponding cdef file 'psrinfo.cdefs'
## @synopsis
##      psrinfo(%args);
## INPUTS TO METHOD CALL 'psrinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'processor-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'physical-processor-count' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'silent' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = psrinfo
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;0	on-line   since 09/11/2012 15:57:11
##&nbsp;1	on-line   since 09/11/2012 15:57:14
##&nbsp;2	on-line   since 09/11/2012 15:57:16
##&nbsp;3	on-line   since 09/11/2012 15:57:18
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'started_since' => '09/11/2012 15:57:11',
##&nbsp;            'processor_id' => '0',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'started_since' => '09/11/2012 15:57:14',
##&nbsp;            'processor_id' => '1',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'started_since' => '09/11/2012 15:57:16',
##&nbsp;            'processor_id' => '2',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'started_since' => '09/11/2012 15:57:18',
##&nbsp;            'processor_id' => '3',
##&nbsp;            'status' => 'on-line'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = psrinfo -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Status of virtual processor 0 as of: 11/12/2012 14:41:41
##&nbsp;  on-line since 09/11/2012 15:57:11.
##&nbsp;  The i386 processor operates at 2393 MHz,
##&nbsp;	and has an i387 compatible floating point processor.
##&nbsp;Status of virtual processor 1 as of: 11/12/2012 14:41:41
##&nbsp;  on-line since 09/11/2012 15:57:14.
##&nbsp;  The i386 processor operates at 2393 MHz,
##&nbsp;	and has an i387 compatible floating point processor.
##&nbsp;Status of virtual processor 2 as of: 11/12/2012 14:41:41
##&nbsp;  on-line since 09/11/2012 15:57:16.
##&nbsp;  The i386 processor operates at 2393 MHz,
##&nbsp;	and has an i387 compatible floating point processor.
##&nbsp;Status of virtual processor 3 as of: 11/12/2012 14:41:41
##&nbsp;  on-line since 09/11/2012 15:57:18.
##&nbsp;  The i386 processor operates at 2393 MHz,
##&nbsp;	and has an i387 compatible floating point processor.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'floating_point_processor' => 'i387',
##&nbsp;            'started_since' => '09/11/2012 15:57:11',
##&nbsp;            'processor_id' => '0',
##&nbsp;            'speed' => '2393 MHz',
##&nbsp;            'processor' => 'i386',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'floating_point_processor' => 'i387',
##&nbsp;            'started_since' => '09/11/2012 15:57:14',
##&nbsp;            'processor_id' => '1',
##&nbsp;            'speed' => '2393 MHz',
##&nbsp;            'processor' => 'i386',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'floating_point_processor' => 'i387',
##&nbsp;            'started_since' => '09/11/2012 15:57:16',
##&nbsp;            'processor_id' => '2',
##&nbsp;            'speed' => '2393 MHz',
##&nbsp;            'processor' => 'i386',
##&nbsp;            'status' => 'on-line'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'floating_point_processor' => 'i387',
##&nbsp;            'started_since' => '09/11/2012 15:57:18',
##&nbsp;            'processor_id' => '3',
##&nbsp;            'speed' => '2393 MHz',
##&nbsp;            'processor' => 'i386',
##&nbsp;            'status' => 'on-line'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = psrinfo -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_processors_count' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = psrinfo -p -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The physical processor has 2 virtual processors (0 1)
##&nbsp;  x86 (chipid 0x0 AuthenticAMD family 15 model 33 step 2 clock 2393 MHz)
##&nbsp;	Dual Core AMD Opteron(tm) Processor 280
##&nbsp;The physical processor has 2 virtual processors (2 3)
##&nbsp;  x86 (chipid 0x1 AuthenticAMD family 15 model 33 step 2 clock 2393 MHz)
##&nbsp;	Dual Core AMD Opteron(tm) Processor 280
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'architecure_based' => 'x86 (chipid 0x0 AuthenticAMD family 15 model 33 step 2 clock 2393 MHz)',
##&nbsp;            'processor_ids' => [
##&nbsp;                                '0',
##&nbsp;                                '1'
##&nbsp;                              ],
##&nbsp;            'virtual_processors_count' => '2',
##&nbsp;            'model' => 'Dual Core AMD Opteron(tm) Processor 280'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'architecure_based' => 'x86 (chipid 0x1 AuthenticAMD family 15 model 33 step 2 clock 2393 MHz)',
##&nbsp;            'processor_ids' => [
##&nbsp;                                '2',
##&nbsp;                                '3'
##&nbsp;                              ],
##&nbsp;            'virtual_processors_count' => '2',
##&nbsp;            'model' => 'Dual Core AMD Opteron(tm) Processor 280'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = psrinfo -s 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'on-line'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub psrinfo {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    my $count = 0;
    my @result_array;
    my $cmd = $resp->get_command();
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    my @values;
    my $str;

    if ($cmd =~ /^psrinfo -p$/) {
        $result_array[$count]->{physical_processors_count} = $lines[0];
    } elsif ($cmd =~ /^psrinfo -p -v$/) {
        while (@lines) {
            $str = shift(@lines);
            @values = ($str =~ /\d+/g);
            $result_array[$count]->{virtual_processors_count} = $values[0];
            $result_array[$count]->{processor_ids} = [$values[1], $values[2]];
            $str = shift(@lines);
            $result_array[$count]->{architecure_based} = $str;
            $str                                       = shift(@lines);
            $result_array[$count]->{model}             = $str;

            $count++;
        }

    } elsif ($cmd =~ /^psrinfo -v$/) {

        while (@lines) {
            $str = shift(@lines);
            if ($str =~ /(\d+)/) {
                $result_array[$count]->{processor_id} = $1;
            }
            $str = shift(@lines);
            if ($str =~ /(\w+-\w+)\s+\w+\s+(.*)\./) {
                $result_array[$count]->{status}        = $1;
                $result_array[$count]->{started_since} = $2;
            }
            $str = shift(@lines);
            if ($str =~ /\w+\s+(\w+)(.*)\s+(\d+ \w+)\,/) {
                $result_array[$count]->{processor} = $1;
                $result_array[$count]->{speed}     = $3;
            }
            $str = shift(@lines);
            if ($str =~ /(.*)\s+(i\d+)/) {
                $result_array[$count]->{floating_point_processor} = $2;
            }
            $count++;
        }
    } elsif ($cmd =~ /^psrinfo -s/) {
        if ($lines[0] == 1) {
            $result_array[0]->{status} = 'on-line';
        } else {
            $result_array[0]->{status} = 'off-line';
        }

    } else {
        while (@lines) {
            $str = shift(@lines);
            if ($str =~ /(\d+)\s+(\w+-\w+)\s+\w+\s+(.*)/) {
                $result_array[$count]->{processor_id}  = $1;
                $result_array[$count]->{status}        = $2;
                $result_array[$count]->{started_since} = $3;
                $count++;
            }
        }
    }

    return \@result_array;

} ## end sub psrinfo

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ($raw_output =~ /([uU]sage):\s+.*/) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ($raw_output =~ /not found/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

} ## end sub _check_error

1;
