# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/route.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary "Routines to parse Solaris route command"
##
## @description
##    This is a library to parse the raw output of Solaris route command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "route" will be route()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::route;
##
##    route(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::route;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::route
##
## @summary Parses the raw output from the "route" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route(%args);
## INPUTS TO METHOD CALL 'route' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'modifiers' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'args' </td> <td> '0' </td> </tr>
## <tr> <td> 'add' </td> <td> 'add' </td> <td> '1' </td> </tr>
## <tr> <td> 'change' </td> <td> 'change' </td> <td> '1' </td> </tr>
## <tr> <td> 'delete' </td> <td> 'delete' </td> <td> '1' </td> </tr>
## <tr> <td> 'flush' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> 'get' </td> <td> 'get' </td> <td> '1' </td> </tr>
## <tr> <td> 'monitor' </td> <td> 'monitor' </td> <td> '1' </td> </tr>
## <tr> <td> 'show' </td> <td> 'show' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'root-dir' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numerical-addresses' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'suppress-output' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'persistent' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = route -p add 1.2.3.4 127.0.0.1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;add host 1.2.3.4: gateway 127.0.0.1
##&nbsp;add persistent host 1.2.3.4: gateway 127.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;             'persistent' => 0
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4',
##&nbsp;             'persistent' => 1
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = route -p show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;persistent: route add 1.2.3.5 127.0.0.1
##&nbsp;persistent: route add 1.2.3.6 127.0.0.1
##&nbsp;persistent: route add 1.2.3.8 127.0.0.1
##&nbsp;persistent: route add 1.2.3.4 127.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'persistent' => 1,
##&nbsp;             'host' => '1.2.3.5'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'persistent' => 1,
##&nbsp;             'host' => '1.2.3.6'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'persistent' => 1,
##&nbsp;             'host' => '1.2.3.8'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'persistent' => 1,
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = route -p delete 1.2.3.4 127.0.0.1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;delete host 1.2.3.4: gateway 127.0.0.1
##&nbsp;delete persistent host 1.2.3.4: gateway 127.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;             'persistent' => 0
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4',
##&nbsp;             'persistent' => 1
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = route add 1.2.4.5 127.0.0.1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;add host 1.2.4.5: gateway 127.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;             'persistent' => 0
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = route  delete 1.2.3.4 127.0.0.1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;delete host 1.2.3.4: gateway 127.0.0.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '127.0.0.1',
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;             'persistent' => 0
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub route {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    foreach my $line (@lines) {
        if ( $line =~ /^\w+\s+(\w+)\s+(\S+)[:]\s+(\w+)\s+(\S+)$/ ) {
            $result_array[$count]->{$1}         = $2;
            $result_array[$count]->{$3}         = $4;
            $result_array[$count]->{persistent} = 0;
        } elsif ( $line =~ /\w+\s+(\w+)\s+(\w+)\s+(\S+)[:]\s+(\w+)\s+(\S+)/ )
        {
            $result_array[$count]->{$1} = 1;
            $result_array[$count]->{$2} = $3;
            $result_array[$count]->{$4} = $5;
        } elsif ( $line =~ /\w+[:]\s+\w+\s+\w+\s+(\S+)\s+(\S+)/ ) {
            $result_array[$count]->{host}       = $1;
            $result_array[$count]->{gateway}    = $2;
            $result_array[$count]->{persistent} = 1;
        }
        $count++;
    }
    return \@result_array;
} ## end sub route

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /gateway required.*|destination required.*/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No persistent routes are defined.*/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /Network is unreachable.*|not in table.*|.*bad value.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(usage:|route:)\s+.*|invalid arguments.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
