#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris sanlun command
##
## @description
##    This is a library to parse the raw output of Solaris sanlun command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sanlun lun show"
##    will be sanlun_lun_show)
##
## @synopsis
##    use sanlun;
##
##    sanlun_lun_show(%args);
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/sanlun lun show
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'experimental', '6.22'
## This would select sanlun lun show.experimental, sanlun lun show.6.22 respectively.
##
## TEST PARAMETERS
##
## NACL_SANLUN_LUN_SHOW_FULL_PATH=<path to the sanlun tool>.
## Eg: NACL_SANLUN_LUN_SHOW_FULL_PATH=/usr/sbin/sanlun lun show
## This override is applicable to every sanlun lun show command invoked
## irrespective of the 'set' used.
##
## NACL_SOLARIS_SANLUN_LUN_SHOW_FULL_PATH=<path to the sanlun tool>.
## Eg: NACL_SOLARIS_SANLUN_LUN_SHOW_FULL_PATH=/usr/sbin/sanlun lun show
## This is applicable to commands that use Solaris APISet only.
##
## @author gupendra@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::sanlun;

use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~
/(device filename.*does not exist|no LUNs available from controller|LUN.*is not available)/si
      )
    {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    elsif ( $output =~ /(sanlun lun show .* )/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    }
    return $error;
}

## @name ::sanlun_lun_show
##
## @summary Parses the raw output from the "/opt/NTAP/SANToolkit/bin/sanlun lun show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sanlun_lun_show with any of its options as described in the
##    corresponding cdef file 'sanlun.cdefs'
## @synopsis
##      sanlun_lun_show(%args);
## INPUTS TO METHOD CALL 'sanlun_lun_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'multipath-info' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'controller-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'host-dev-filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pathname' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /opt/NTAP/SANToolkit/bin/sanlun lun show -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; controller:      lun-pathname       device filename   adapter  protocol          lun size         lun state
##&nbsp;rahula-vsim-3:  /vol/vol0/sollun2   /dev/rdsk/c2t4d0s2  iscsi0   iSCSI         40m (41943040)       GOOD
##&nbsp;	Serial number: C02BS+ApqRbd
##&nbsp;	Controller iSCSI IP address:	10.238.3.164  Controller iSCSI port number 3260
##&nbsp;	Controller IP address:	10.238.3.164
##&nbsp;	Controller volume name:vol0   FSID:0xadf81f75
##&nbsp;	Controller qtree name:/vol/vol0   ID:0x0
##&nbsp;	Controller snapshot name:   ID:0x0
##&nbsp;	LUN partition table permits multiprotocol access: no
##&nbsp;		why: there is no valid disk label on this disk.
##&nbsp;	LUN has valid label: no
##&nbsp;rahula-vsim-3:  /vol/vol0/sollun11  /dev/rdsk/c2t2d0s2  iscsi0   iSCSI         40m (41943040)       GOOD
##&nbsp;	Serial number: C02BS+ApqRbv
##&nbsp;	Controller iSCSI IP address:	10.238.3.164  Controller iSCSI port number 3260
##&nbsp;	Controller IP address:	10.238.3.164
##&nbsp;	Controller volume name:vol0   FSID:0xadf81f75
##&nbsp;	Controller qtree name:/vol/vol0   ID:0x0
##&nbsp;	Controller snapshot name:   ID:0x0
##&nbsp;	LUN partition table permits multiprotocol access: no
##&nbsp;		why: there is no valid disk label on this disk.
##&nbsp;	LUN has valid label: no
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => 'C02BS+ApqRbd',
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'multiprotocol_access' => 'no',
##&nbsp;            'valid_label' => 'no',
##&nbsp;            'volume_name' => 'vol0',
##&nbsp;            'device_filename' => '/dev/rdsk/c2t4d0s2',
##&nbsp;            'lun_pathname' => '/vol/vol0/sollun2',
##&nbsp;            'snapshot_name' => '  ',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'ip_address' => '10.238.3.164',
##&nbsp;            'fsid' => '0xadf81f75',
##&nbsp;            'iscsi_ip_address' => '10.238.3.164',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'iscsi_port_number' => '3260',
##&nbsp;            'adapter' => 'iscsi0',
##&nbsp;            'snapshot_id' => '0x0'
##&nbsp;            'qtree_name' => '/vol/vol0'
##&nbsp;            'qtree_id' => '0x0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => 'C02BS+ApqRbv',
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'multiprotocol_access' => 'no',
##&nbsp;            'valid_label' => 'no',
##&nbsp;            'volume_name' => 'vol0',
##&nbsp;            'device_filename' => '/dev/rdsk/c2t2d0s2',
##&nbsp;            'lun_pathname' => '/vol/vol0/sollun11',
##&nbsp;            'snapshot_name' => '  ',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'ip_address' => '10.238.3.164',
##&nbsp;            'fsid' => '0xadf81f75',
##&nbsp;            'iscsi_ip_address' => '10.238.3.164',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'iscsi_port_number' => '3260',
##&nbsp;            'adapter' => 'iscsi0',
##&nbsp;            'snapshot_id' => '0x0'
##&nbsp;            'qtree_name' => '/vol/vol0'
##&nbsp;            'qtree_id' => '0x0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = /opt/NTAP/SANToolkit/bin/sanlun lun show -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ONTAP_PATH: rahula-vsim-3:/vol/vol0/sollun11
##&nbsp;LUN: 0
##&nbsp;LUN Size:    40m (41943040)
##&nbsp;Host Device: /dev/rdsk/c2t2d0s2
##&nbsp;LUN State: GOOD  Controller_CF_State: Cluster Enabled  Controller Partner:
##&nbsp;Multipath_Policy: Native  Multipath-provider: Sun Microsystems
##&nbsp; TPGS flag: 0x8
##&nbsp;ONTAP_PATH: rahula-vsim-3:/vol/vol0/sollun2
##&nbsp;LUN: 1
##&nbsp;LUN Size:    40m (41943040)
##&nbsp;Host Device: /dev/rdsk/c2t4d0s2
##&nbsp;LUN State: GOOD  Controller_CF_State: Cluster Enabled  Controller Partner:
##&nbsp;Multipath_Policy: Native  Multipath-provider: Sun Microsystems
##&nbsp; TPGS flag: 0x8
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'host_device' => '/dev/rdsk/c2t2d0s2',
##&nbsp;            'ontap_path' => 'rahula-vsim-3:/vol/vol0/sollun11',
##&nbsp;            'multipath_policy' => 'Native',
##&nbsp;            'multipath_provider' => 'Sun Microsystems',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'tpgs_flag' => '0x8',
##&nbsp;            'lun' => '0',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'controller_cf_state' => 'Cluster Enabled',
##&nbsp;            'controller_partner' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'host_device' => '/dev/rdsk/c2t4d0s2',
##&nbsp;            'ontap_path' => 'rahula-vsim-3:/vol/vol0/sollun2',
##&nbsp;            'multipath_policy' => 'Native',
##&nbsp;            'multipath_provider' => 'Sun Microsystems',
##&nbsp;            'lun_state' => 'GOOD',
##&nbsp;            'tpgs_flag' => '0x8',
##&nbsp;            'lun' => '1',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'controller_cf_state' => 'Cluster Enabled',
##&nbsp;            'controller_partner' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = /opt/NTAP/SANToolkit/bin/sanlun lun show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  controller:      lun-pathname       device filename   adapter  protocol          lun size         lun state
##&nbsp;gupendra-vsim2:  /vol/vol0/sollun1   /dev/rdsk/c2t3d0s2  iscsi0   iSCSI         40m (41943040)       GOOD
##&nbsp; rahula-vsim-3:  /vol/vol0/sollun2   /dev/rdsk/c2t4d0s2  iscsi0   iSCSI         40m (41943040)       GOOD
##&nbsp; rahula-vsim-3:  /vol/vol0/sollun11  /dev/rdsk/c2t2d0s2  iscsi0   iSCSI         40m (41943040)       GOOD
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'controller' => 'gupendra-vsim2',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'adapter' => 'iscsi0',
##&nbsp;            'device_filename' => '/dev/rdsk/c2t3d0s2',
##&nbsp;            'lun_pathname' => '/vol/vol0/sollun1',
##&nbsp;            'lun_state' => 'GOOD'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'adapter' => 'iscsi0',
##&nbsp;            'device_filename' => '/dev/rdsk/c2t4d0s2',
##&nbsp;            'lun_pathname' => '/vol/vol0/sollun2',
##&nbsp;            'lun_state' => 'GOOD'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'controller' => 'rahula-vsim-3',
##&nbsp;            'lun_size' => '40m (41943040)',
##&nbsp;            'adapter' => 'iscsi0',
##&nbsp;            'device_filename' => '/dev/rdsk/c2t2d0s2',
##&nbsp;            'lun_pathname' => '/vol/vol0/sollun11',
##&nbsp;            'lun_state' => 'GOOD'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sanlun_lun_show {
    my %args = @_;
    my ( $error_message, @result_array );

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        command       => "sanlun_lun_show",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $option = $command;
    my ( @temp, $value );
    my $i = 0;
    if ( $option =~ /-p\s?/ ) {
        my @lines =
          get_lines( "raw_output" => $output, "response_object" => $resp );
        foreach $value (@lines) {
            if ( $value =~
     /(LUN\s+State)\:\s+(\w+)\s+(\w+)\:\s+(\w+\s+\w+?)\s+(\w+\s+\w+)\:(.*)/
              )
            {
                $temp[$i]->{ to_key_name( item => $1 ) } =
                  trim_string( 'string' => "$2" );
                $temp[$i]->{ to_key_name( item => $3 ) } =
                  trim_string( 'string' => "$4" );
                $temp[$i]->{ to_key_name( item => $5 ) } =
                  trim_string( 'string' => "$6" );
            }
            elsif ( $value =~ /(Multipath_Policy)\:\s+(\w+)\s+.*\:(.*)/ ) {
                $temp[$i]->{ to_key_name( item => $1 ) } =
                  trim_string( 'string' => "$2" );
                $temp[$i]->{'multipath_provider'}  =
                  trim_string( 'string' => "$3" );
            }
           elsif ( $value =~ /(ONTAP_PATH)\:\s+(.*)/ ) {
                $temp[$i]->{ to_key_name( item => $1 ) } =
                  trim_string( 'string' => "$2" );
            }
            elsif ( $value =~ /(.*)\:(.*)/ ) {
                $temp[$i]->{ to_key_name( item => $1 ) } =
                  trim_string( 'string' => "$2" );
                $i++ if ( $value =~ /TPGS\s+flag/ );
            }
        }
    }
    elsif ( $option =~ /-v\s?/ ) {
        my @lines =
          get_lines( "raw_output" => $output, "response_object" => $resp );
        foreach $value (@lines) {
            if ( $value =~
                /controller:\s+lun-pathname\s+device\s+filename.*/ )
            {
                next;
            }
            if ( $value =~
                /(.*)\:\s+(\/\S+)\s+(\S+)\s+(\S+)\s+(\w+)\s+(\S+\s+\S+)\s+(\w+)/
              )
            {
                $temp[$i]->{'controller'}      = $1;
                $temp[$i]->{'lun_pathname'}    = $2;
                $temp[$i]->{'device_filename'} = $3;
                $temp[$i]->{'adapter'}         = $4;
                $temp[$i]->{'protocol'}        = $5;
                $temp[$i]->{'lun_size'}        = $6;
                $temp[$i]->{'lun_state'}       = $7;
            }
            elsif ( $value =~ /Serial\s+number\:\s+(\S+)/ ) {
                $temp[$i]->{'serial_number'} = trim_string( 'string' => "$1" );
            }
            elsif ( $value =~
            /Controller\s+iSCSI\s+IP\s+address\:\s+(\S+)\s+Controller\s+iSCSI\s+port\s+number\s+(\S+)/
              )
            {
                $temp[$i]->{'iscsi_ip_address'}  = $1;
                $temp[$i]->{'iscsi_port_number'} = $2;
            }
            elsif ( $value =~ /Controller\s+IP\s+address:\s+(\S+)/ ) {
                $temp[$i]->{'ip_address'} = $1;
            }
            elsif ( $value =~ /Controller\s+volume\s+name\:(\S+)\s+FSID\:(.*)/ )
            {
                $temp[$i]->{'volume_name'} = $1;
                $temp[$i]->{'fsid'}        = $2;
            }
            elsif ( $value =~ /Controller\s+qtree\s+name\:(\S+)\s+ID:(.*)/ ) {
                $temp[$i]->{'qtree_name'} = $1;
                $temp[$i]->{'qtree_id'} = $2;
            }
            elsif ( $value =~ /Controller\s+snapshot\s+name:(.*)\s+ID:(\S+)/ ) {
                $temp[$i]->{'snapshot_name'} = $1;
                $temp[$i]->{'snapshot_id'} = $2;
            }

            elsif ( $value =~ /LUN\s+partition\s+.*\:\s+(\w+)/ ) {
                $temp[$i]->{'multiprotocol_access'} = $1;

            }
            elsif ( $value =~ /why:\s+there\s+is\s+no.*/ ) {
                next;
            }
            elsif ( $value =~ /LUN\s+has\s+valid\s+label\:\s+(\w+)/ ) {
                $temp[$i]->{'valid_label'} = $1;
                $i++;
            }
        }
    }
    else {
        my @lines =
          get_lines( "raw_output" => $output, "response_object" => $resp );
        foreach $value (@lines) {
            if ( $value =~
                /controller.*lun-pathname.*device.*lun size.*lun state/ )
            {
                next;
            }
            if ( $value =~
                /(\S+):\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+\s+\S+)\s+(\w+)/ )
            {
                $temp[$i]->{'controller'}      = $1;
                $temp[$i]->{'lun_pathname'}    = $2;
                $temp[$i]->{'device_filename'} = $3;
                $temp[$i]->{'adapter'}         = $4;
                $temp[$i]->{'protocol'}        = $5;
                $temp[$i]->{'lun_size'}        = $6;
                $temp[$i]->{'lun_state'}       = $7;
                $i++;
            }
        }
    }
    return \@temp;
}
1;
