#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary shutdown command output
##
## @description
##    This is a library to parse the raw output of shutdown command.
##
## @synopsis
##    use shutdown;
##
##    shutdown(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Solaris::default::shutdown;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error = undef;

    if ( $raw_output =~ /[Uu]sage/ ) {
        $error         = "MissingRequiredParam";
    }
    elsif ( $raw_output =~ /command not found.*/i ) {
        $error         = "CommandNotFound";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::shutdown
##
## @summary Parses the raw output from the "shutdown" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from shutdown with any of its options as described in the
##    corresponding cdef file 'shutdown.cdefs'
## @synopsis
##      shutdown(%args);
## INPUTS TO METHOD CALL 'shutdown' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-y' </td> <td> 'pre-answer' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'grace-period' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'init-state' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'message' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples

##&nbsp;Command = shutdown -y -g 2 -i 6; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;The system is going down for reboot NOW!
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub shutdown {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
}
1;

