#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse solaris svciadm command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris svcadm command.
##
## @synopsis
##    use svcadm;
##    svcadm(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::svcadm;

use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~
        /doesn't match any instances|not in a maintenance or degraded state/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /may not be used with degraded/) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /unsatisfied dependencies/) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::svcadm
##
## @summary Parses the raw output from the "svcadm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from svcadm with any of its options as described in the
##    corresponding cdef file 'svcadm.cdefs'
## @synopsis
##      svcadm(%args);
## INPUTS TO METHOD CALL 'svcadm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'cmd' </td> </tr>
## <tr> <td> '-r' </td> <td> 'recursive' </td> </tr>
## <tr> <td> '-t' </td> <td> 'temporary' </td> </tr>
## <tr> <td> '-I' </td> <td> 'immediate' </td> </tr>
## <tr> <td> '-s' </td> <td> 'wait' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'milestone-fmri' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'instance-state' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fmri-pattern' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = svcadm -v enable svc:/application/print/ipp-listener:default
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'listener:default' => 'enabled'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = svcadm -v refresh svc:/application/print/ipp-listener:default
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'svc:/application/print/ipp' => 'refresh'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = svcadm -v disable svc:/application/print/ipp-listener:default
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'listener:default' => 'disabled'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = iscsiadm modify target-param -d NONE iqn.1999-08.com.array:sn.01234567
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'svc:/application/print/ipp' => 'maint_on'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = svcadm -v clear svc:/application/print/ipp-listener:default
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'svc:/application/print/ipp' => 'maint_off'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = svcadm restart svc:/application/print/ipp-listener:default
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
##&nbsp;Command = svcadm milestone all
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;
sub svcadm {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'svcadm',
                             "ptr_error_message" => \$error_message,);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array = ();
    my @lines = get_lines(raw_output      => $raw_output,
                          can_throw       => 0,
                          response_object => $resp);
    foreach my $line (@lines) {
        if ($line =~ /([\w\:\/\-]+)\s+(enabled|disabled)/) {
            $result_array[0]->{$1} = $2;
        } elsif ($line =~
            /Action (refresh|restart|maint_off|maint_on) set for ([\w\:\/\-]+)/) {
            $result_array[0]->{$2} = $1;
        }
    }
    return \@result_array;
} ## end sub svcadm
1;
