#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris svcs command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris svcs command.
##
## @synopsis
##    use svcs;
##    svcs(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::svcs;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /(Pattern '\w+' doesn't match any instances)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
}

## @name ::svcs
##
## @summary Parses the raw output from the "svcs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from svcs with any of its options as described in the
##    corresponding cdef file 'svcs.cdefs'
## @synopsis
##      svcs(%args);
## INPUTS TO METHOD CALL 'svcs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = svcs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rcS_d/S50sk98sol',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S10lu',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S20sysetup',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S40llc2',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S42ncakmod',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S47pppd',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S70uucp',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S72autoinstall',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S73cachefs_daemon',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S81dodatadm_udaplt',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S89PRESERVE',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S94ncalogd',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc2_d/S98deallocate',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc3_d/S16boot_server',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc3_d/S50apache',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc3_d/S52imq',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'fmri' => 'lrc:/etc/rc3_d/S80mipagent',
##&nbsp;             'stime' => 'Aug_24',
##&nbsp;             'state' => 'legacy_run'
##&nbsp;           },
##&nbsp;        ];
##&nbsp;
sub svcs {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'svcs',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array = ();
    my $i = 0;
    my @lines = get_lines( raw_output => $raw_output,
	                       can_throw => 0, response_object => $resp );
    foreach my $line (@lines) {
		if($line =~ /STATE\s+STIME\s+FMRI/) {
			next;
		} elsif($line =~ /([\w\_]+)\s+([\w\_\:]+)\s+([\w\:\/\_\-]+)$/) {
			$result_array[$i] -> {"state"} = $1;
			$result_array[$i] -> {"stime"} = $2;
			$result_array[$i] -> {"fmri"} = $3;
		}
		$i++;
	}
	return \@result_array;
}
1;
