#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris umount command
##
## @description
##    This is a library to parse the raw output of Solaris umount command.
##
## @synopsis
##    use umount;
##
##    umount(%args);
##
## @author Abhishek Kumar(kabhishe@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::umount;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = $raw_output;
    if ($raw_output =~ /$command.*:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        } 
    }
    return $error;
}


## @name ::umount
##
## @summary Parses the raw output from the "umount" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from umount with any of its options as described in the 
##    corresponding cdef file 'umount.cdefs'
## @synopsis
##      umount(%args);
## INPUTS TO METHOD CALL 'umount' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'echo-command' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'paths' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'forcily-umount' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub umount {
    my %args = @_;
    
    my ($error_message);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    
    if($error_level) {
        $error = _check_error(raw_output=>$output,command=>"umount",
                                 error_message=>\$error_message);
        if($error) {
            $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
			$error->throw(text=>$error_message,response_object => $resp);
        }
    }
    
    return [];
}

1;
