#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/uname.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris uname command output
##
## @description
##    This is a library to parse the raw output of Solaris uname command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::uname;
##
##    uname(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::uname;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );
    if ( $raw_output =~ /(uname|[Uu]sage):/ ) {
        $error_message = $1;
        $error         = "CommandFailed";
        if ( $raw_output =~ /(illegal option)/ ) {
            $error         = "InvalidParam";
            $error_message = $1;
        }
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::uname
##
## @summary Parses the raw output from the "uname" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from uname with any of its options as described in the 
##    corresponding cdef file 'uname.cdefs'
## @synopsis
##      uname(%args);
## INPUTS TO METHOD CALL 'uname' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-X' </td> <td> 'expanded-system-info' </td> <td> '1' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'system-name' </td> <td> '1' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'nodename' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'processor' </td> <td> '1' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'release' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'os' </td> <td> '1' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'hardware' </td> <td> '1' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'platform' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = uname
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SunOS
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system' => 'SunOS'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = uname -X
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;System = SunOS
##&nbsp;Node = sun219-31
##&nbsp;Release = 5.10
##&nbsp;KernelID = Generic_127128-11
##&nbsp;Machine = i86pc
##&nbsp;BusType = <unknown>
##&nbsp;Serial = <unknown>
##&nbsp;Users = <unknown>
##&nbsp;OEM# = 0
##&nbsp;Origin# = 1
##&nbsp;NumCPU = 4
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bustype' => '<unknown>',
##&nbsp;            'system' => 'SunOS',
##&nbsp;            'serial' => '<unknown>',
##&nbsp;            'kernelid' => 'Generic_127128-11',
##&nbsp;            'origin#' => '1',
##&nbsp;            'users' => '<unknown>',
##&nbsp;            'release' => '5.10',
##&nbsp;            'node' => 'sun219-31',
##&nbsp;            'numcpu' => '4',
##&nbsp;            'oem#' => '0',
##&nbsp;            'machine' => 'i86pc'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = uname -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SunOS sun219-31 5.10 Generic_127128-11 i86pc i386 i86pc
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system' => 'SunOS',
##&nbsp;            'processor' => 'i386',
##&nbsp;            'release' => '5.10',
##&nbsp;            'kernel_id' => 'Generic_127128-11',
##&nbsp;            'node' => 'sun219-31',
##&nbsp;            'platform' => 'i86pc',
##&nbsp;            'machine' => 'i86pc'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = uname -s -n -m -i -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SunOS sun219-31 i86pc i386 i86pc
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system' => 'SunOS',
##&nbsp;            'processor' => 'sun219-31',
##&nbsp;            'platform' => 'i86pc',
##&nbsp;            'node' => 'i86pc',
##&nbsp;            'machine' => 'i386'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub uname {
    my %args = @_;
    my (@result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my %hash_info = (
        'system'    => '-s',
        'node'      => '-n',
        'release'   => '-r',
        'kernel_id' => '-v',
        'machine'   => '-m',
        'processor' => '-p',
        'platform'  => '-i'
    );

    my $line = $lines[0];
    if ( $command =~ /-X/ ) {
        foreach my $var (@lines) {
            my @values = split( /\s=\s/, $var );
            $values[0] =~ s/#//g;
            $result_array[0]->{ to_key_name( item => $values[0] ) }
                = $values[1];
        }
        return \@result_array;
    }
    my ( @info, @label, $i );
    foreach my $key ( keys(%hash_info) ) {
        my $cmd = $hash_info{$key};
        if ( $command =~ /$cmd/ ) {
            push( @label, $key );
            my $temp = $line;
            if ( $temp =~ /\w+\s+\w+/ ) {
                @info = split( /\s+/, $temp );
            }
        }
    }

    for ( $i = 0; $i < @info; $i++ ) {
        $result_array[0]->{ $label[$i] } = $info[$i];
    }
    if ( $command =~ /-a/ ) {
        my $temp  = $line;
        my @items = split( /\s+/, $temp );
        my $j     = 0;
        foreach my $key (
            'system',  'node',      'release', 'kernel_id',
            'machine', 'processor', 'platform'
            ) {
            $result_array[0]->{$key} = $items[$j];
            $j++;
        }
        return \@result_array;
    }
    if ( $command eq "uname" ) {
        $result_array[0]->{'system'} = $lines[0];
    }

    return \@result_array;
} ## end sub uname

1;
