#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/chmod.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux chmod command
##
## @description
##    This is a library to parse the raw output of Linux chmod command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "chmod" will be chmod()
##
## @synopsis
##    use chmod;
##
##    chmod(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::chmod;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::chmod
##
## @summary Parses the raw output from the "chmod" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chmod with any of its options as described in the
##    corresponding cdef file 'chmod.cdefs'
## @synopsis
##      chmod(%args);
## INPUTS TO METHOD CALL 'chmod' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub chmod {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub chmod

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /missing operand.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No such file or directory.*|invalid mode.* / ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /chmod:\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

1;

