#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/df.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux & Solaris df command
##
## @description
##    This is a library to parse the raw output of Linux df command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "df" will be df()
##
## @synopsis
##    use df;
##
##    df(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::df;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::df
##
## @summary Parses the raw output from the "df" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from df with any of its options as described in the
##    corresponding cdef file 'df.cdefs'
## @synopsis
##      df(%args);
## INPUTS TO METHOD CALL 'df' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-B' </td> <td> 'block-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'files-free' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'all-visible-zones' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'directory' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'portability' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'total-or-type' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'fstype-specific-options' </td> <td> '0' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'exclude-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'tatvfs-structure' </td> <td> '1' </td> </tr>
## <tr> <td> '--no-sync' </td> <td> 'no-sync' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'kilobytes-free' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kbytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'smallest-block-size' </td> <td> '1' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--sync' </td> <td> 'sync' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inodes' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'echo' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'local-file-systems' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-readable-format' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'block-device' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'print-type' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'powers-of-1000' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'fstype-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'fstype' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = df -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem           1K-blocks      Used Available Use% Mounted on
##&nbsp;/dev/mapper/VolGroup00-LogVol00
##&nbsp;                     224953628   7584068 205758220   4% /
##&nbsp;proc                         0         0         0   -  /proc
##&nbsp;/dev/sda1               101086     12544     83323  14% /boot
##&nbsp;tmpfs                   512916         0    512916   0% /dev/shm
##&nbsp;btcinf02.eng.btc.netapp.in:/vol/local/i386-linux2.0
##&nbsp;                     300764904 104351968 196412936  35% /usr/local
##&nbsp;/dev/mapper/mpath67     495844    105121    365123  23% /scal/multiVS1
##&nbsp;/dev/mapper/mpath16     495844      2318    467926   1% /scal/multiVS3
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/mapper/VolGroup00-LogVol00',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'percentage_used' => '4%',
##&nbsp;            'used' => '7584068',
##&nbsp;            '1k_blocks' => '224953628',
##&nbsp;            'available' => '205758220'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'proc',
##&nbsp;            'mounted_on' => '/proc',
##&nbsp;            'percentage_used' => '-',
##&nbsp;            'used' => '0',
##&nbsp;            '1k_blocks' => '0',
##&nbsp;            'available' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/sda1',
##&nbsp;            'mounted_on' => '/boot',
##&nbsp;            'percentage_used' => '14%',
##&nbsp;            'used' => '12544',
##&nbsp;            '1k_blocks' => '101086',
##&nbsp;            'available' => '83323'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'tmpfs',
##&nbsp;            'mounted_on' => '/dev/shm',
##&nbsp;            'percentage_used' => '0%',
##&nbsp;            'used' => '0',
##&nbsp;            '1k_blocks' => '512916',
##&nbsp;            'available' => '512916'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'btcinf02.eng.btc.netapp.in:/vol/local/i386-linux2.0',
##&nbsp;            'mounted_on' => '/usr/local',
##&nbsp;            'percentage_used' => '35%',
##&nbsp;            'used' => '104351968',
##&nbsp;            '1k_blocks' => '300764904',
##&nbsp;            'available' => '196412936'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/mapper/mpath67',
##&nbsp;            'mounted_on' => '/scal/multiVS1',
##&nbsp;            'percentage_used' => '23%',
##&nbsp;            'used' => '105121',
##&nbsp;            '1k_blocks' => '495844',
##&nbsp;            'available' => '365123'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/mapper/mpath16',
##&nbsp;            'mounted_on' => '/scal/multiVS3',
##&nbsp;            'percentage_used' => '1%',
##&nbsp;            'used' => '2318',
##&nbsp;            '1k_blocks' => '495844',
##&nbsp;            'available' => '467926'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -T
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem    Type   1K-blocks      Used Available Use% Mounted on
##&nbsp;/dev/mapper/VolGroup00-LogVol00
##&nbsp;              ext3   224953628   7584068 205758220   4% /
##&nbsp;/dev/sda1     ext3      101086     12544     83323  14% /boot
##&nbsp;tmpfs        tmpfs      512916         0    512916   0% /dev/shm
##&nbsp;btcinf02.eng.btc.netapp.in:/vol/local/i386-linux2.0
##&nbsp;               nfs   300764904 104351968 196412936  35% /usr/local
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/mapper/VolGroup00-LogVol00',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'percentage_used' => '4%',
##&nbsp;            'used' => '7584068',
##&nbsp;            '1k_blocks' => '224953628',
##&nbsp;            'type' => 'ext3',
##&nbsp;            'available' => '205758220'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/sda1',
##&nbsp;            'mounted_on' => '/boot',
##&nbsp;            'percentage_used' => '14%',
##&nbsp;            'used' => '12544',
##&nbsp;            '1k_blocks' => '101086',
##&nbsp;            'type' => 'ext3',
##&nbsp;            'available' => '83323'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'tmpfs',
##&nbsp;            'mounted_on' => '/dev/shm',
##&nbsp;            'percentage_used' => '0%',
##&nbsp;            'used' => '0',
##&nbsp;            '1k_blocks' => '512916',
##&nbsp;            'type' => 'tmpfs',
##&nbsp;            'available' => '512916'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'btcinf02.eng.btc.netapp.in:/vol/local/i386-linux2.0',
##&nbsp;            'mounted_on' => '/usr/local',
##&nbsp;            'percentage_used' => '35%',
##&nbsp;            'used' => '104351968',
##&nbsp;            '1k_blocks' => '300764904',
##&nbsp;            'type' => 'nfs',
##&nbsp;            'available' => '196412936'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -x nfs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Filesystem           1K-blocks      Used Available Use% Mounted on
##&nbsp;/dev/mapper/VolGroup00-LogVol00
##&nbsp;                     224953628   7584068 205758220   4% /
##&nbsp;/dev/sda1               101086     12544     83323  14% /boot
##&nbsp;tmpfs                   512916         0    512916   0% /dev/shm
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/mapper/VolGroup00-LogVol00',
##&nbsp;            'mounted_on' => '/',
##&nbsp;            'percentage_used' => '4%',
##&nbsp;            'used' => '7584068',
##&nbsp;            '1k_blocks' => '224953628',
##&nbsp;            'available' => '205758220'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => '/dev/sda1',
##&nbsp;            'mounted_on' => '/boot',
##&nbsp;            'percentage_used' => '14%',
##&nbsp;            'used' => '12544',
##&nbsp;            '1k_blocks' => '101086',
##&nbsp;            'available' => '83323'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem' => 'tmpfs',
##&nbsp;            'mounted_on' => '/dev/shm',
##&nbsp;            'percentage_used' => '0%',
##&nbsp;            'used' => '0',
##&nbsp;            '1k_blocks' => '512916',
##&nbsp;            'available' => '512916'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;df (GNU coreutils) 5.97
##&nbsp;Copyright (C) 2006 Free Software Foundation, Inc.
##&nbsp;This is free software.  You may redistribute copies of it under the terms of
##&nbsp;the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.
##&nbsp;There is NO WARRANTY, to the extent permitted by law.
##&nbsp;
##&nbsp;Written by Torbjorn Granlund, David MacKenzie, and Paul Eggert.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'website' => '<http://www.gnu.org/licenses/gpl.html>',
##&nbsp;            'version' => '(GNU coreutils) 5.97',
##&nbsp;            'written_by' => 'Torbjorn Granlund, David MacKenzie, and Paul Eggert',
##&nbsp;            'copyright' => '2006 Free Software Foundation, Inc.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -F ufs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/                  (/dev/dsk/c0t2d0s0 ):128204410 blocks  7974979 files
##&nbsp;/var               (/dev/dsk/c0t2d0s3 ): 3964458 blocks   314517 files
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'free_files' => '7974979 ',
##&nbsp;            'free_blocks' => '128204410 ',
##&nbsp;            'mount_point' => '/',
##&nbsp;            'device_or_filesystem' => '/dev/dsk/c0t2d0s0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'free_files' => '314517 ',
##&nbsp;            'free_blocks' => '3964458 ',
##&nbsp;            'mount_point' => '/var',
##&nbsp;            'device_or_filesystem' => '/dev/dsk/c0t2d0s3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -g
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/                  (/dev/dsk/c0t2d0s0 ):         8192 block size          1024 frag size
##&nbsp;134911906 total blocks  128204410 free blocks 126855292 available        8118656 total files
##&nbsp; 7974979 free files      7602368 filesys id
##&nbsp;     ufs fstype       0x00000004 flag             255 filename length
##&nbsp;
##&nbsp;/devices           (/devices          ):          512 block size           512 frag size
##&nbsp;       0 total blocks          0 free blocks        0 available            137 total files
##&nbsp;       0 free files     73138176 filesys id  devices
##&nbsp;   devfs fstype         00000000 flag             255 filename length
##&nbsp;
##&nbsp;/system/contract   (ctfs              ):          512 block size           512 frag size
##&nbsp;       0 total blocks          0 free blocks        0 available             33 total files
##&nbsp;2147483614 free files     73400321 filesys id  contract
##&nbsp;    ctfs fstype         00000000 flag              32 filename length
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_files' => '8118656',
##&nbsp;            'total_blocks' => '134911906',
##&nbsp;            'block_size' => '8192',
##&nbsp;            'flag' => '0x00000004',
##&nbsp;            'free_files' => '7974979',
##&nbsp;            'free_blocks' => '128204410',
##&nbsp;            'device_or_filesystem' => '/dev/dsk/c0t2d0s0',
##&nbsp;            'frag_size' => '1024',
##&nbsp;            'filesys_id' => '7602368',
##&nbsp;            'mount_point' => '/',
##&nbsp;            'filename_length' => '255',
##&nbsp;            'fstype' => 'ufs',
##&nbsp;            'available' => '126855292'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_files' => '137',
##&nbsp;            'total_blocks' => '0',
##&nbsp;            'block_size' => '512',
##&nbsp;            'flag' => '00000000',
##&nbsp;            'free_files' => '0',
##&nbsp;            'free_blocks' => '0',
##&nbsp;            'device_or_filesystem' => '/devices',
##&nbsp;            'frag_size' => '512',
##&nbsp;            'filesys_id' => '73138176',
##&nbsp;            'mount_point' => '/devices',
##&nbsp;            'filename_length' => '255',
##&nbsp;            'fstype' => 'devfs',
##&nbsp;            'available' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_files' => '33',
##&nbsp;            'total_blocks' => '0',
##&nbsp;            'block_size' => '512',
##&nbsp;            'flag' => '00000000',
##&nbsp;            'free_files' => '2147483614',
##&nbsp;            'free_blocks' => '0',
##&nbsp;            'device_or_filesystem' => 'ctfs',
##&nbsp;            'frag_size' => '512',
##&nbsp;            'filesys_id' => '73400321',
##&nbsp;            'mount_point' => '/system/contract',
##&nbsp;            'filename_length' => '32',
##&nbsp;            'fstype' => 'ctfs',
##&nbsp;            'available' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -n
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/                  : ufs
##&nbsp;/devices           : devfs
##&nbsp;/system/contract   : ctfs
##&nbsp;/proc              : proc
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filesystem_type' => 'ufs',
##&nbsp;            'filesystem' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem_type' => 'devfs',
##&nbsp;            'filesystem' => '/devices'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem_type' => 'ctfs',
##&nbsp;            'filesystem' => '/system/contract'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filesystem_type' => 'proc',
##&nbsp;            'filesystem' => '/proc'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -t
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/                  (/dev/dsk/c0t2d0s0 ): 128204410 blocks  7974979 files
##&nbsp;                                  total: 134911906 blocks  8118656 files
##&nbsp;/devices           (/devices          ):        0 blocks        0 files
##&nbsp;                                  total:        0 blocks      137 files
##&nbsp;/system/contract   (ctfs              ):        0 blocks 2147483614 files
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'free_files' => '7974979',
##&nbsp;            'free_blocks' => '128204410',
##&nbsp;            'total' => [
##&nbsp;                         {
##&nbsp;                           'blocks' => '134911906',
##&nbsp;                           'files' => '8118656'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'mount_point' => '/',
##&nbsp;            'device_or_filesystem' => '/dev/dsk/c0t2d0s0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'free_files' => '0',
##&nbsp;            'free_blocks' => '0',
##&nbsp;            'total' => [
##&nbsp;                         {
##&nbsp;                           'blocks' => '0',
##&nbsp;                           'files' => '137'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'mount_point' => '/devices',
##&nbsp;            'device_or_filesystem' => '/devices'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'free_files' => '2147483614',
##&nbsp;            'free_blocks' => '0',
##&nbsp;            'total' => [
##&nbsp;                         {
##&nbsp;                           'blocks' => '0',
##&nbsp;                           'files' => '33'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'mount_point' => '/system/contract',
##&nbsp;            'device_or_filesystem' => 'ctfs'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = df -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Mount Dir  Filesystem        blocks     used     free  %used
##&nbsp;/          /dev/dsk/c0t2d0 67455953  3353748 63427646     6%
##&nbsp;/devices   /devices               0        0        0     0%
##&nbsp;/system/co ctfs                   0        0        0     0%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'free' => '63427646',
##&nbsp;            'blocks' => '67455953',
##&nbsp;            'percentage_used' => '6%',
##&nbsp;            'filesystem' => '/dev/dsk/c0t2d0',
##&nbsp;            'used' => '3353748',
##&nbsp;            'mount_dir' => '/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'free' => '0',
##&nbsp;            'blocks' => '0',
##&nbsp;            'percentage_used' => '0%',
##&nbsp;            'filesystem' => '/devices',
##&nbsp;            'used' => '0',
##&nbsp;            'mount_dir' => '/devices'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'free' => '0',
##&nbsp;            'blocks' => '0',
##&nbsp;            'percentage_used' => '0%',
##&nbsp;            'filesystem' => 'ctfs',
##&nbsp;            'used' => '0',
##&nbsp;            'mount_dir' => '/system/co'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub df {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my $c = 0;
    my $m = -1;

    foreach my $line (@lines) {
        if ( $lines[0] =~ /^Filesystem |^Mount Dir/ ) {
            @head = split( /\s+/, $lines[0] );
            foreach (@head) {
                $_ =~ s/-/_/;
                $_ =~ s/Use%/percentage_used/;
            }
            my $size = @head;
            if ( $head[ $size - 2 ] eq "Mounted" ) {
                $head[ $size - 2 ]
                    = $head[ $size - 2 ] . "_" . $head[ $size - 1 ];
                pop @head;
            }
            if ( $head[0] eq "Mount" ) {
                $head[1] = $head[0] . "_" . $head[1];
                shift @head;
            }
            for ( my $i = 1; $i <= $#lines; $i++ ) {
                @values = split( /\s+/, $lines[$i] );
                if ( $#values == 0 ) {
                    $lines[$i] = $lines[$i] . " " . $lines[ $i + 1 ];
                    @values = split( /\s+/, $lines[$i] );
                    $i++;
                }
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
                $c++;
            }
            return \@result_array;
        }
        if ( $cmd =~ /--version/ ) {
            if ( $line =~ /df\s+(.*)/ ) {
                $result_array[0]->{version} = $1;
            } elsif ( $line =~ /(Copyright)\s+\S+\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /General Public License\s+(\<\S+\>)/ ) {
                $result_array[0]->{website} = $1;
            } elsif ( $line
                =~ /(Written by)\s+(\w+\s\w+,\s\w+\w\s\w+,\s\w+\s\w+\s\w+)/ )
            {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        } elsif ( $cmd =~ /-g/ ) {
            if ( $line
                =~ /(\S+)\s+\((\S+)\s+\):\s+(\d+)\s+(block size)\s+(\d+)\s+(frag size)/
                ) {
                $m++;
                $result_array[$m]->{mount_point}          = $1;
                $result_array[$m]->{device_or_filesystem} = $2;
                $result_array[$m]->{ to_key_name( item => $4 ) } = $3;
                $result_array[$m]->{ to_key_name( item => $6 ) } = $5;

            } elsif ( $line
                =~ /(\d+)\s+(total blocks)\s+(\d+)\s+(free blocks)\s+(\d+)\s+(available)\s+(\d+)\s+(total files)/
                ) {
                $result_array[$m]->{ to_key_name( item => $2 ) } = $1;
                $result_array[$m]->{ to_key_name( item => $4 ) } = $3;
                $result_array[$m]->{ to_key_name( item => $6 ) } = $5;
                $result_array[$m]->{ to_key_name( item => $8 ) } = $7;
            } elsif ( $line =~ /(\d+)\s+(free files)\s+(\d+)\s+(filesys id)/ )
            {
                $result_array[$m]->{ to_key_name( item => $2 ) } = $1;
                $result_array[$m]->{ to_key_name( item => $4 ) } = $3;
            } elsif ( $line
                =~ /(\w+)\s+(fstype)\s+(\w+)\s+(flag)\s+(\d+)\s+(filename length)/
                ) {
                $result_array[$m]->{ to_key_name( item => $2 ) } = $1;
                $result_array[$m]->{ to_key_name( item => $4 ) } = $3;
                $result_array[$m]->{ to_key_name( item => $6 ) } = $5;
            }
        } elsif ( $cmd =~ /-n/ ) {
            $m++;
            my @array = split( /\s+:\s+/, $line );
            $result_array[$m]->{filesystem}      = $array[0];
            $result_array[$m]->{filesystem_type} = $array[1];
        } elsif ( $lines[0] =~ /^\S+\s+\(.*\):/ ) {
            if ( $line =~ /(\S+)\s+\((.*)\):(.*)\s+blocks\s+(\S+)\s+files/ ) {
                my ( $temp1, $temp2, $temp3, $temp4 ) = ( $1, $2, $3, $4 );
                $temp2 =~ s/\s+$//;
                $temp3 =~ s/^\s+//;
                $m++;
                $result_array[$m]->{mount_point}          = $temp1;
                $result_array[$m]->{device_or_filesystem} = $temp2;
                $result_array[$m]->{free_blocks}          = $temp3;
                $result_array[$m]->{free_files}           = $temp4;
            }
            if ( $line =~ /(total):\s+(\d+)\s+blocks\s+(\d+)\s+files/ ) {
                $result_array[$m]->{$1}->[0]->{blocks} = $2;
                $result_array[$m]->{$1}->[0]->{files}  = $3;
            }
        }
    }
    return \@result_array;
} ## end sub df

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /df:\s+(invalid|unknown)\s+option.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /df:\s+option requires an argument.*/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No such file or directory/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

