#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/exportfs.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux & Solaris exportfs command
##
## @description
##    This is a library to parse the raw output of Linux exportfs command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "exportfs" will be exportfs
##
## @synopsis
##    use exportfs;
##
##    exportfs(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::exportfs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::exportfs
##
## @summary Parses the raw output from the "exportfs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from exportfs with any of its options as described in the
##    corresponding cdef file 'exportfs.cdefs'
## @synopsis
##      exportfs(%args);
## INPUTS TO METHOD CALL 'exportfs' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## command_full_path
## (Optional)
## The complete path to the command.
## Eg: /usr/sbin/exportfs
##
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option are supported in Linux & Solaris
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ignore' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'unexport' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'export' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
##
## The below option support Linux only
## <tr> <td> '-r' </td> <td> 'reexport' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'flush' </td> <td> '1' </td> </tr>
##
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = exportfs
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -o  async 10.72.219.44:/usr/tmp
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -r
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = exportfs  -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub exportfs {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub exportfs

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:|exportfs:) .*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid exporting option:.* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

