#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary find command output
##
## @description
## @synopsis
##    use NACL::APISet::Host::CLI::Unix::find;
##
##    find(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Unix::find;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidOptionException qw(:try);
use strict;

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error = undef;
    if ( $raw_output =~ /(?:unknown|invalid)\s+predicate/ ) {
        $error         = "InvalidOption";
    } elsif ( $raw_output =~ /No\s+such\s+file\s+or\s+directory/ ) {
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /missing\s+argument\s+to/ ) {
        $error         = "MissingRequiredParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

## @name ::find
##
## @summary Parses the raw output from the "find" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from find with any of its options as described in the 
##    corresponding cdef file 'find.cdefs'
## @synopsis
##      find(%args);
## INPUTS TO METHOD CALL 'find' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-exec rm -rf {} +' </td> <td> 'rm' </td> <td> '1' </td> </tr> 
## <tr> <td> '-not' </td> <td> 'exclude' </td> <td> '1' </td> </tr> 
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = find /tmp -name "*" -type f
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/tmp/RLNxCa02sE.dat
##&nbsp;/tmp/r8BCejzx1k.dat
##&nbsp;/tmp/rO2zUof2zZ.dat
##&nbsp;/tmp/g_f
##&nbsp;/tmp/abc
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$var [
##&nbsp;        /tmp/RLNxCa02sE.dat,
##&nbsp;        /tmp/r8BCejzx1k.dat,
##&nbsp;        /tmp/rO2zUof2zZ.dat,
##&nbsp;        /tmp/g_f,
##&nbsp;        /tmp/abc,
##&nbsp;     ]
##&nbsp;

sub find {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output,
                 "command" => $command,
                 response_object => $resp );
    my @files = split /\n/,$output;
    return \@files;
} ## end sub find
1;

