# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/logger.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse logger command
##
## @description
##    This is a library to parse logger command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "logger" will be logger()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Unix::logger;
##
##    logger(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::logger;
use strict;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::logger
##
## @summary Parses the raw output from the "logger" command
##
## @description 
##   This library supports error checking for kenv command, but output parsing is not supported as of now.
## @synopsis
##      logger(%args);
## INPUTS TO METHOD CALL 'logger' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option supports both Linux & Solaris
##
## <tr> <td> '-p' </td> <td> 'priority' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'tag' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'process-id' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'message' </td> <td> '0' </td> </tr> 
##
## The below option support Linux only
##
## <tr> <td> '-d' </td> <td> 'datagram' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'system-log' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'socket' </td> <td> '0' </td> </tr> 
## <tr> <td> '--' </td> <td> 'end-argument-list' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported
## @examples
##&nbsp;Command = logger  -isd -p local0.notice -t Testing -f  /root/vidya/loggerfile.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Testing[4108]: Hi!!
##&nbsp;Testing[4108]: Welcome to HCL....
##&nbsp;Testing[4108]: Bye!!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = logger -is -p local7.info -t Testing Practical Extraction and Reporting Language
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Testing[4421]: logger Practical Extraction and Reporting Language
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub logger {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub logger

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /logger:\s+invalid option.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /[uU]sage:\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
