#
#Copyright (c) 2012-13 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux and HPUX lvdisplay command
##
## @description
##    This is a library to parse the raw output of HP-UX and Linux lvdisplay command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lvdisplay" will lvdisplay
##
## @synopsis
##    use lvdisplay;
##
##    lvdisplay(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::lvdisplay;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::lvdisplay
##
## @summary Parses the raw output from the "lvdisplay" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lvdisplay with any of its options as described in the
##    corresponding cdef file 'lvdisplay.cdefs'
## @synopsis
##      lvdisplay(%args);
## INPUTS TO METHOD CALL 'lvdisplay' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below options support only HP-UX
## <tr> <td> '-F' </td> <td> 'compact-listing' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'pvkey' </td> <td> '1' </td> </tr>
##
## ## The below options support both Linux and HP-UX
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lv-path' </td> <td> '0' </td> </tr>
##
## The below options support only Linux
## <tr> <td> '-v' </td> <td> 'mapping' </td> <td> '1' </td> </tr>
## <tr> <td> '--maps' </td> <td> 'maps' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignorelockfail' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'partial' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'colon' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '--units' </td> <td> 'units' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'columns' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lvdisplay
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Logical volume ---
##&nbsp;  LV Name                /dev/vol_grp2/lvol0
##&nbsp;  VG Name                vol_grp2
##&nbsp;  LV UUID                utiIs4-DBvf-2fnt-hUd6-pc1r-6f9f-KpkesC
##&nbsp;  LV Write Access        read/write
##&nbsp;  LV snapshot status     source of
##&nbsp;                         /dev/vol_grp2/lvol1 [active]
##&nbsp;  LV Status              available
##&nbsp;  # open                 0
##&nbsp;  LV Size                20.00 MB
##&nbsp;  Current LE             5
##&nbsp;  Segments               1
##&nbsp;  Allocation             inherit
##&nbsp;  Read ahead sectors     auto
##&nbsp;  - currently set to     256
##&nbsp;  Block device           253:0
##&nbsp;
##&nbsp;  --- Logical volume ---
##&nbsp;  LV Name                /dev/vol_grp2/lvol1
##&nbsp;  VG Name                vol_grp2
##&nbsp;  LV UUID                7j8dhC-d5gs-3dMo-NSvM-rGfL-bz7J-i5c1Wt
##&nbsp;  LV Write Access        read/write
##&nbsp;  LV snapshot status     active destination for /dev/vol_grp2/lvol0
##&nbsp;  LV Status              available
##&nbsp;  # open                 0
##&nbsp;  LV Size                20.00 MB
##&nbsp;  Current LE             5
##&nbsp;  COW-table size         20.00 MB
##&nbsp;  COW-table LE           5
##&nbsp;  Allocated to snapshot  0.04%
##&nbsp;  Snapshot chunk size    4.00 KB
##&nbsp;  Segments               1
##&nbsp;  Allocation             inherit
##&nbsp;  Read ahead sectors     auto
##&nbsp;  - currently set to     256
##&nbsp;  Block device           253:1
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lv_size' => '20.00 MB',
##&nbsp;            'lv_status' => 'available',
##&nbsp;            'open_count_of_lv' => '0',
##&nbsp;            'vg_name' => 'vol_grp2',
##&nbsp;            'segments' => '1',
##&nbsp;            'read_ahead_sectors' => 'auto',
##&nbsp;            'currently_set_to' => '256',
##&nbsp;            'lv_name' => '/dev/vol_grp2/lvol0',
##&nbsp;            'lv_snapshot_status' => 'source of',
##&nbsp;            'block_device' => '253:0',
##&nbsp;            'current_le' => '5',
##&nbsp;            'lv_uuid' => 'utiIs4-DBvf-2fnt-hUd6-pc1r-6f9f-KpkesC',
##&nbsp;            'allocation' => 'inherit',
##&nbsp;            'lv_write_access' => 'read/write'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'cow_table_le' => '5',
##&nbsp;            'lv_size' => '20.00 MB',
##&nbsp;            'vg_name' => 'vol_grp2',
##&nbsp;            'segments' => '1',
##&nbsp;            'read_ahead_sectors' => 'auto',
##&nbsp;            'currently_set_to' => '256',
##&nbsp;            'snapshot_chunk_size' => '4.00 KB',
##&nbsp;            'lv_name' => '/dev/vol_grp2/lvol1',
##&nbsp;            'lv_snapshot_status' => 'active destination for /dev/vol_grp2/lvol0',
##&nbsp;            'lv_uuid' => '7j8dhC-d5gs-3dMo-NSvM-rGfL-bz7J-i5c1Wt',
##&nbsp;            'allocation' => 'inherit',
##&nbsp;            'lv_write_access' => 'read/write',
##&nbsp;            'lv_status' => 'available',
##&nbsp;            'allocated_to_snapshot' => '0.04%',
##&nbsp;            'open_count_of_lv' => '0',
##&nbsp;            'block_device' => '253:1',
##&nbsp;            'current_le' => '5',
##&nbsp;            'cow_table_size' => '20.00 MB'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvdisplay -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  /dev/vol_grp2/lvol0:vol_grp2:3:1:-1:0:40960:5:-1:0:-1:253:0
##&nbsp;  /dev/vol_grp2/lvol1:vol_grp2:3:1:-1:0:40960:5:-1:0:-1:253:1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvsize_in_sectors' => '40960',
##&nbsp;            'lv_status' => '1',
##&nbsp;            'open_count_of_lv' => '0',
##&nbsp;            'vg_name' => 'vol_grp2',
##&nbsp;            'minor_dev_num_of_lv' => '0',
##&nbsp;            'read_ahead_sectors' => '-1',
##&nbsp;            'allocated_le_of_lv' => '-1',
##&nbsp;            'current_le_associated_to_lv' => '5',
##&nbsp;            'lv_acess' => '3',
##&nbsp;            'lv_name' => '/dev/vol_grp2/lvol0',
##&nbsp;            'internal_lv_number' => '-1',
##&nbsp;            'alloc_policy_of_lv' => '0',
##&nbsp;            'major_dev_num_of_lv' => '253'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvsize_in_sectors' => '40960',
##&nbsp;            'lv_status' => '1',
##&nbsp;            'open_count_of_lv' => '0',
##&nbsp;            'vg_name' => 'vol_grp2',
##&nbsp;            'minor_dev_num_of_lv' => '1',
##&nbsp;            'read_ahead_sectors' => '-1',
##&nbsp;            'allocated_le_of_lv' => '-1',
##&nbsp;            'current_le_associated_to_lv' => '5',
##&nbsp;            'lv_acess' => '3',
##&nbsp;            'lv_name' => '/dev/vol_grp2/lvol1',
##&nbsp;            'internal_lv_number' => '-1',
##&nbsp;            'alloc_policy_of_lv' => '0',
##&nbsp;            'major_dev_num_of_lv' => '253'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvdisplay -v /dev/vg00/lvol3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;--- Logical volumes ---
##&nbsp;LV Name                     /dev/vg00/lvol3
##&nbsp;VG Name                     /dev/vg00
##&nbsp;LV Permission               read/write
##&nbsp;LV Status                   available/syncd
##&nbsp;Mirror copies               0
##&nbsp;Consistency Recovery        MWC
##&nbsp;Schedule                    parallel
##&nbsp;LV Size (Mbytes)            3096
##&nbsp;Current LE                  387
##&nbsp;Allocated PE                387
##&nbsp;Stripes                     0
##&nbsp;Stripe Size (Kbytes)        0
##&nbsp;Bad block                   off
##&nbsp;Allocation                  strict/contiguous
##&nbsp;IO Timeout (Seconds)        default
##&nbsp;
##&nbsp;   --- Distribution of logical volume ---
##&nbsp;   PV Name                 LE on PV  PE on PV
##&nbsp;   /dev/disk/disk3         387       387
##&nbsp;
##&nbsp;   --- Logical extents ---
##&nbsp;   LE    PV1                     PE1   Status 1
##&nbsp;   00000 /dev/disk/disk3         00726 current
##&nbsp;   00001 /dev/disk/disk3         00727 current
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'logical_extents' => [
##&nbsp;                                   {
##&nbsp;                                     'status' => 'current',
##&nbsp;                                     'logical_extents' => '00000',
##&nbsp;                                     'physical_extents' => '00726',
##&nbsp;                                     'physcial_volume' => '/dev/disk/disk3'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'status' => 'current',
##&nbsp;                                     'logical_extents' => '00001',
##&nbsp;                                     'physical_extents' => '00727',
##&nbsp;                                     'physcial_volume' => '/dev/disk/disk3'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'logical_volumes' => [
##&nbsp;                                   {
##&nbsp;                                     'allocated_pe' => '387',
##&nbsp;                                     'lv_status' => 'available/syncd',
##&nbsp;                                     'lv_size_mbytes' => '3096',
##&nbsp;                                     'bad_block' => 'off',
##&nbsp;                                     'consistency_recovery' => 'MWC',
##&nbsp;                                     'vg_name' => '/dev/vg00',
##&nbsp;                                     'schedule' => 'parallel',
##&nbsp;                                     'stripe_size_kbytes' => '0',
##&nbsp;                                     'stripes' => '0',
##&nbsp;                                     'lv_name' => '/dev/vg00/lvol3',
##&nbsp;                                     'io_timeout_seconds' => 'default',
##&nbsp;                                     'mirror_copies' => '0',
##&nbsp;                                     'current_le' => '387',
##&nbsp;                                     'allocation' => 'strict/contiguous',
##&nbsp;                                     'lv_permission' => 'read/write'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'distribution_of_logical_volume' => {
##&nbsp;                                                  'le_on_pv' => '387',
##&nbsp;                                                  'pe_on_pv' => '387',
##&nbsp;                                                  'pv_name' => '/dev/disk/disk3'
##&nbsp;                                                }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lvdisplay --maps
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Logical volume ---
##&nbsp;  LV Name                /dev/n_vg/nvol-1
##&nbsp;  VG Name                n_vg
##&nbsp;  LV UUID                TX2Dav-wJTX-uyfD-ihO6-Wl0u-KCLi-ag22ez
##&nbsp;  LV Write Access        read/write
##&nbsp;  LV Status              available
##&nbsp;  # open                 1
##&nbsp;  LV Size                3.01 GiB
##&nbsp;  Current LE             770
##&nbsp;  Segments               1
##&nbsp;  Allocation             inherit
##&nbsp;  Read ahead sectors     auto
##&nbsp;  - currently set to     2560
##&nbsp;  Block device           253:13
##&nbsp;
##&nbsp;  --- Segments ---
##&nbsp;  Logical extent 0 to 769:
##&nbsp;    Type  striped
##&nbsp;    Stripes  10
##&nbsp;    Stripe size  64.00 KiB
##&nbsp;    Stripe 0:
##&nbsp;      Physical volume /dev/mapper/360a9800032466648455d43323071566d
##&nbsp;      Physical extents 0 to 76
##&nbsp;    Stripe 1:
##&nbsp;      Physical volume /dev/mapper/360a9800032466648455d43323071566f
##&nbsp;      Physical extents 0 to 76
##&nbsp;    Stripe 2:
##&nbsp;      Physical volume /dev/mapper/360a9800032466648455d433230715671
##&nbsp;      Physical extents 0 to 76
##&nbsp;    Stripe 3:
##&nbsp;      Physical volume /dev/mapper/360a9800032466648455d433230715673
##&nbsp;      Physical extents 0 to 76
##&nbsp;    Stripe 4:
##&nbsp;      Physical volume /dev/mapper/360a9800032466648455d433230715675
##&nbsp;      Physical extents 0 to 76
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lv_size' => '3.01 GiB',
##&nbsp;            'lv_status' => 'available',
##&nbsp;            'open_count_of_lv' => '1',
##&nbsp;            'vg_name' => 'n_vg',
##&nbsp;            'segments' => '1',
##&nbsp;            'read_ahead_sectors' => 'auto',
##&nbsp;            'logical_extent_0_to_769' => [
##&nbsp;                                           {
##&nbsp;                                             'stripes_info' => [
##&nbsp;                                                                 {
##&nbsp;                                                                   'physical_extents' => '0 to 76',
##&nbsp;                                                                   'physical_volume' => '/dev/mapper/360a9800032466648455d43323071566d'
##&nbsp;                                                                 },
##&nbsp;                                                                 {
##&nbsp;                                                                   'physical_extents' => '0 to 76',
##&nbsp;                                                                   'physical_volume' => '/dev/mapper/360a9800032466648455d43323071566f'
##&nbsp;                                                                 },
##&nbsp;                                                                 {
##&nbsp;                                                                   'physical_extents' => '0 to 76',
##&nbsp;                                                                   'physical_volume' => '/dev/mapper/360a9800032466648455d433230715671'
##&nbsp;                                                                 },
##&nbsp;                                                                 {
##&nbsp;                                                                   'physical_extents' => '0 to 76',
##&nbsp;                                                                   'physical_volume' => '/dev/mapper/360a9800032466648455d433230715673'
##&nbsp;                                                                 },
##&nbsp;                                                                 {
##&nbsp;                                                                   'physical_extents' => '0 to 76',
##&nbsp;                                                                   'physical_volume' => '/dev/mapper/360a9800032466648455d433230715675'
##&nbsp;                                                                 }
##&nbsp;                                                                ],
##&nbsp;                                             'stripe_size' => '64.00 KiB',
##&nbsp;                                             'type' => 'striped',
##&nbsp;                                             'stripes' => '10'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;           'currently_set_to' => '2560',
##&nbsp;           'lv_name' => '/dev/n_vg/nvol-1',
##&nbsp;           'block_device' => '253:13',
##&nbsp;           'lv_uuid' => 'TX2Dav-wJTX-uyfD-ihO6-Wl0u-KCLi-ag22ez',
##&nbsp;           'current_le' => '770',
##&nbsp;           'allocation' => 'inherit',
##&nbsp;           'lv_write_access' => 'read/write'
##&nbsp;        }
##&nbsp;     ];
##&nbsp;
##&nbsp;Command = lvdisplay -C
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LV        VG          Attr     LSize  Pool Origin Data%  Move Log Copy%  Convert
##&nbsp;  lvol_samp vol_grpsamp -wi-a--- 20.00m
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'convert' => '-',
##&nbsp;            'lv_size' => '20.00m',
##&nbsp;            'move' => '-',
##&nbsp;            'origin' => '-',
##&nbsp;            'data' => '-',
##&nbsp;            'vg_name' => 'vol_grpsamp',
##&nbsp;            'attr' => '-wi-a---',
##&nbsp;            'copy' => '-',
##&nbsp;            'lv_name' => 'lvol_samp',
##&nbsp;            'pool' => '-',
##&nbsp;            'log' => '-'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lvdisplay {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my $i = -1;
    my $m = -1;
    my $k = -1;
    my $count = -1;
    my $limit = -1;
    my ($head,$temp_head,$stripe,$cost, $time);
    my (@array, @head);
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    @lines = grep(! /Input\/output\s+error|echo/, @lines);
    my $cmd = $resp->get_command();

    my @header = (
        "lv_name",            "vg_name",
        "lv_acess",           "lv_status",
        "internal_lv_number", "open_count_of_lv",
        "lvsize_in_sectors",  "current_le_associated_to_lv",
        "allocated_le_of_lv", "alloc_policy_of_lv",
        "read_ahead_sectors", "major_dev_num_of_lv",
        "minor_dev_num_of_lv"
    );
    my %res   = (
        LV      => 'lv_name',
        VG      => 'vg_name',
        Attr    => 'attr',
        LSize   => 'lv_size',
        Pool    => 'pool',
        Origin  => 'origin',
        'Data%' => 'data',
        Move    => 'move',
        Log     => 'log',
        'Copy%' => 'copy',
        Convert => 'convert',
        'Snap%' => 'snap',       
    );
    my ( @arr1, @arr2 );
    if ($cmd =~ /-C/) {
          @head = split( /\s+/, $lines[0] );
          for (my $i = 1; $i <= $#lines; $i++) {
                  $count++;
                  my @values = split(/\s+/, $lines[$i]);
                  for (my $j = 0; $j <= $#head; $j++) {
                      $result_array[$count]->{$res{$head[$j]}} = $values[$j]?$values[$j]:'-';
              }
           }
    }
    foreach my $line (@lines) {
        if ( $line =~ /\---\s(.*)\s\---/ ) {
            $head = to_key_name( item => $1 );
            $m++ if($head !~ /segments/);
         # The below regex matches the output from 'lvdisplay -F /dev/vg00/lvol3' command in HPUX
        } elsif ( $line =~ /.*=.*:.*.*=.*:.*/ ) {
            $k++;
            my @array = split(/:/, $line);
            for (my $j = 0; $j <= $#array; $j++) {
                my @array1 = split(/=/, $array[$j]);
                $result_array[$k]->{to_key_name(item => $array1[0])}
                    = $array1[1];
            }
        } elsif ( $line =~ /.*:.*:.*:/ ) {
            $k++;
            @array = split( /:/, $line );
            for ( my $i = 0; $i <= $#header; $i++ ) {
                $header[$i] =~ s/-\s+//;
                $header[$i] =~ s/-/_/;
                $result_array[$k]->{ $header[$i] } = $array[$i];
            }
        } elsif ($line =~ /LV.*\,\s+time\s+(.*)\,\s+(.*)/) {
            $cost = $1;
            $time = $2;
        }
        @array = split( /\s\s+/, $line );
        if ( $line =~ /LE\s+PV1/ ) {
            @arr1 = split( /\s\s+/, $line );
            foreach (@arr1) {
            }
        } elsif ( $line =~ /PV\sName/ ) {
            @arr2 = split( /\s\s+/, $line );
        }
        my %hash1 = (
            "LE"       => "logical_extents",
            "PV1"      => "physcial_volume",
            "PE1"      => "physical_extents",
            "Status 1" => "status"
        );
        my %hash2 = (
            "PV Name"  => "pv_name",
            "LE on PV" => "le_on_pv",
            "PE on PV" => "pe_on_pv"
        );
        if ( $#array == 1 || $line =~ /^Logical.*|^Stripe.*|^Physical/ ) {
        if ( $array[0] =~ /^\D+/ ) {
            $array[0] =~ s/#\s+open/open_count_of_lv/;
            $array[0] =~ s/-\s+//;
            $array[0] =~ s/-/_/;
            if ( $cmd =~ /-v/ ) {
                $result_array[0]->{logical_volumes}->[$m]->{lv_creation_host} = $cost if(defined $cost);
                $result_array[0]->{logical_volumes}->[$m]->{time} = $time if(defined $time);
                $result_array[0]->{logical_volumes}->[$m]->{to_key_name( item =>$array[0])} =
                  $array[1];
            } else {
                if ( $head =~ /segments/ ) {
                    if ( $line =~ /(Logical\s+extent.*)\:/ ) {
                        $temp_head = to_key_name( item =>$1);
                        $count=-1 ;
                        $limit++;
                    } elsif ( $line =~ /(Type|Stripes|Stripe\s+size)\s+(.*)/ ) {
                        $result_array[$m]->{$temp_head}[0]->{to_key_name( item =>$1)} = $2;
                    } elsif ( $line =~ /(Stripe\s+\d+):/ ) {
                        $stripe = to_key_name( item =>$1);
                        $count++;
                    } elsif ( $line =~
                        /(Physical\s+volume|Physical\s+extents)\s+(.*)/ ) {
                        $result_array[$m]->{$temp_head}[0]->{stripes_info}[$count]->{to_key_name( item =>$1)} 
                          = $2 if ( defined $stripe );
                        $result_array[$m]->{$temp_head}[0]->{stripes_info}[$limit]->{to_key_name( item =>$1)} 
                          = $2 if ( !defined $stripe );
                    }
                } else {
                    $result_array[$m]->{lv_creation_host} = $cost if(defined $cost);
                    $result_array[$m]->{time} = $time if(defined $time);
                    $result_array[$m]->{to_key_name( item =>$array[0])} = $array[1];
                }
            }
        } elsif ( $array[0] =~ /^\d+/ ) {
                $i++;
                my @temp1 = split( /\s+/, $line );
                for ( my $j = 0; $j <= $#temp1; $j++ ) {
                    $arr1[$j] =~ s/-/_/;
                    $result_array[0]->{$head}->[$i]->{ $hash1{ $arr1[$j] } }
                        = $temp1[$j];
                }
            }
        } elsif ( $#array == 2 && $line =~ /^\// ) {
            for ( my $l = 0; $l <= $#array; $l++ ) {
                $arr2[$l] =~ s/-/_/;
                $result_array[0]->{$head}->{ $hash2{ $arr2[$l] } }
                    = $array[$l];
            }
        }
    }
    return \@result_array;
} ## end sub lvdisplay

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(u[U]sage:)\s+.*|\s+Volume group.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /lvdisplay:\s+.*|.*:\s+Illegal option./ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;

