#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux multipath command output
##
## @description
##    This is a library to parse the raw output of Linux multipath command.
##
## @synopsis
##    use multipath;
##
##    multipath(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::multipath;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error, $error_message) = undef;
    my $error_level = $resp->get_execution_status();
    if ($raw_output
        =~ /Missing\s+option\s+arguement|must\s+provide\s+a\s+map\s+name\s+to\s+remove/i ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif ($raw_output =~ /Unknown\s+switch:.*|.*not\s+a\s+valid\s+policy/i) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ($raw_output =~ /[Uu]sage:.*/i) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::multipath
##
## @summary Parses the raw output from the "multipath" command
##
## @description
##    This library supports error checking for multipath command, but output parsing is not supported as of now.
## @synopsis
##      multipath(%args);
## INPUTS TO METHOD CALL 'multipath' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'dry-run' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'flush-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbosity-level' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'policy' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reload' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'specific-topology-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-ll' </td> <td> 'all-topology-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bindings-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'internal-hardware-table' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
##
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = multipath -ll
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;\e[1m360a9800032466648455d433230715671 dm-3 NETAPP,LUN\e[0m
##&nbsp;size=2.0G features='3 queue_if_no_path pg_init_retries 50' hwhandler='1 alua' wp=rw
##&nbsp;|-+- policy='round-robin 0' prio=50 status=active
##&nbsp;| `- 2:0:1:3  sdo  8:224  active ready running
##&nbsp;|-+- policy='round-robin 0' prio=50 status=enabled
##&nbsp;| `- 3:0:1:3  sdak 66:64  active ready running
##&nbsp;|-+- policy='round-robin 0' prio=10 status=enabled
##&nbsp;| `- 2:0:0:3  sdd  8:48   active ready running
##&nbsp;`-+- policy='round-robin 0' prio=10 status=enabled
##&nbsp;  `- 3:0:0:3  sdz  65:144 active ready running
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'features' => '3 queue_if_no_path pg_init_retries 50',
##&nbsp;            'paths' => [
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdo',
##&nbsp;                           'status' => 'active',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:224',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '2:0:1:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdak',
##&nbsp;                           'status' => 'enabled',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '66:64',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '3:0:1:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '10',
##&nbsp;                           'dev' => 'sdd',
##&nbsp;                           'status' => 'enabled',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:48',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '2:0:0:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '10',
##&nbsp;                           'dev' => 'sdz',
##&nbsp;                           'status' => 'enabled',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '65:144',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '3:0:0:3'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'size' => '2.0G',
##&nbsp;            'wwid' => '360a9800032466648455d433230715671',
##&nbsp;            'devnode' => 'dm-3',
##&nbsp;            'hardware_handler' => '1 alua',
##&nbsp;            'wp' => 'rw',
##&nbsp;            'vend' => 'NETAPP',
##&nbsp;            'product' => 'LUN'
##&nbsp;          },
##&nbsp;
##&nbsp;Command = multipath -d -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;\e[1m360a9800032466648455d433230715671 dm-3 NETAPP,LUN\e[0m
##&nbsp;size=2.0G features='3 queue_if_no_path pg_init_retries 50' hwhandler='1 alua' wp=rw
##&nbsp;|-+- policy='round-robin 0' prio=50 status=active
##&nbsp;| |- 2:0:1:3  sdo  8:224  active ready running
##&nbsp;| `- 3:0:1:3  sdak 66:64  active ready running
##&nbsp;`-+- policy='round-robin 0' prio=10 status=enabled
##&nbsp;  |- 2:0:0:3  sdd  8:48   active ready running
##&nbsp;  `- 3:0:0:3  sdz  65:144 active ready running
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'features' => '3 queue_if_no_path pg_init_retries 50',
##&nbsp;            'paths' => [
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdo',
##&nbsp;                           'status' => 'active',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:224',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '2:0:1:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdak',
##&nbsp;                           'status' => 'active',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '66:64',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '3:0:1:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '10',
##&nbsp;                           'dev' => 'sdd',
##&nbsp;                           'status' => 'enabled',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:48',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '2:0:0:3'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '10',
##&nbsp;                           'dev' => 'sdz',
##&nbsp;                           'status' => 'enabled',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '65:144',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '3:0:0:3'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'size' => '2.0G',
##&nbsp;            'wwid' => '360a9800032466648455d433230715671',
##&nbsp;            'devnode' => 'dm-3',
##&nbsp;            'hardware_handler' => '1 alua',
##&nbsp;            'wp' => 'rw',
##&nbsp;            'vend' => 'NETAPP',
##&nbsp;            'product' => 'LUN'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = multipath -d -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;3600a0980433032426f24434a54666c77 dm-0 NETAPP,LUN C-Mode
##&nbsp;[size=50M][features=1 queue_if_no_path][hwhandler=1 alua][rw]
##&nbsp;\\_ round-robin 0 [prio=50][active]
##&nbsp;\\_ 7:0:0:0 sdb 8:16  [active][ready]
##&nbsp;\\_ 7:0:0:0 sdb 8:16  [active][ready][running]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'features' => '1 queue_if_no_path',
##&nbsp;            'paths' => [
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdb',
##&nbsp;                           'status' => 'active',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:16',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '7:0:0:0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'priority' => '50',
##&nbsp;                           'dev' => 'sdb',
##&nbsp;                           'status' => 'active',
##&nbsp;                           'path_selector' => 'round-robin 0',
##&nbsp;                           'chk_st' => 'ready',
##&nbsp;                           'dev_t' => '8:16',
##&nbsp;                           'device_status' => 'running',
##&nbsp;                           'dm_st' => 'active',
##&nbsp;                           'hcil' => '7:0:0:0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'size' => '50M',
##&nbsp;            'wwid' => '3600a0980433032426f24434a54666c77',
##&nbsp;            'devnode' => 'dm-0',
##&nbsp;            'hardware_handler' => '1 alua',
##&nbsp;            'wp' => 'rw',
##&nbsp;            'vend' => 'NETAPP',
##&nbsp;            'product' => 'LUN C-Mode'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub multipath {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error(raw_output => $output, response_object => $resp);
    my @result_array;
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    @lines = grep(! /Input\/output\s+error/, @lines);
    if (($cmd =~ /-l/ && $cmd =~ /-d/) || ($cmd =~ /-ll/ && $cmd =~ /-d/) || $cmd =~ /-ll|-l/ ) {
        my $first_time = 1;
        my $count = 0;
        my $limit = 0;
        my ($path_selector, $priority, $status );
        foreach my $line (@lines) {
            $line =~ s/\e\[1m//;
            $line =~ s/\e\[\d+m//;
            # The below regex matches the lines like "360a9800032466648455d433230715671 dm-3 NETAPP,LUN"
            if ($line =~ /(.*)\s+(dm-\d+)\s+(.*)\,(\w+.*)\,*(\w*)/) {
                if(! $first_time) {
                    $count++;
                }
                $result_array[$count]->{wwid}     = $1;
                $result_array[$count]->{devnode}  = $2;
                $result_array[$count]->{vend}     = $3;
                $result_array[$count]->{product}  = $4;
                $result_array[$count]->{revision} = $5 if ($5);
                $first_time = 0;
                $limit=0;
            # The below regex matches the lines like "size=2.0G features='3 queue_if_no_path pg_init_retries 50' hwhandler='1 alua' wp=rw"
            } elsif ($line
                =~ /(\w+)[=](\S+)\s+(\w+)[=]\'(.*)\'\s+\w+[=]\'(.*)\'\s+(\w+)[=](\w+)/ ) {
                $result_array[$count]->{$1}               = $2;
                $result_array[$count]->{$3}               = $4;
                $result_array[$count]->{hardware_handler} = $5;
                $result_array[$count]->{$6}               = $7;
            # The below regex matches the lines like [size=50M][features=1 queue_if_no_path][hwhandler=1 alua][rw]
            } elsif ($line
                =~ /\[(\w+)=(\w+)\]\[(\w+)[=](.*)\]\[hwhandler[=](.*)\]\[(.*)\]/ ) {
                $result_array[$count]->{$1}               = $2;
                $result_array[$count]->{$3}               = $4;
                $result_array[$count]->{hardware_handler} = $5;
                $result_array[$count]->{wp}               = $6;
            # The below regex matches the lines like "|-+- policy='round-robin 0' prio=50 status=active"
            # The below regex matches the lines like \\_ round-robin 0 [prio=50][active]
            } elsif ($line =~ /\w+[=]\'(.*)\'\s+\w+[=](\d+)\s+\w+[=](\w+)/ || $line =~ /\_*\s*(\w+.*\w+)\s+\[\w+[=](.*)\]\[(.*)\]/) {
                $path_selector = $1;
                $priority = $2;
                $status = $3;
            # The below regex matches the lines like "| `- 2:0:1:3  sdo  8:224  active ready running"
            # The below regex also matches the lines like \\_ 7:0:0:0 sdb 8:16  [active][ready]
            } elsif ($line
                =~ /(\d+.*\d+)\s+(\w+)\s+(\d+.*\d+)\s+\[*(\w+)\]*\s*\[*(\w+)\]*\s*\[*(\w*)\]*/) {
                $result_array[$count]->{paths}->[$limit]->{path_selector} =
                  $path_selector;
                $result_array[$count]->{paths}->[$limit]->{priority} =
                  $priority;
                $result_array[$count]->{paths}->[$limit]->{status} = $status;
                $result_array[$count]->{paths}->[$limit]->{hcil} = $1;
                $result_array[$count]->{paths}->[$limit]->{dev} = $2;
                $result_array[$count]->{paths}->[$limit]->{dev_t} = $3;
                $result_array[$count]->{paths}->[$limit]->{dm_st} = $4;
                $result_array[$count]->{paths}->[$limit]->{chk_st} = $5;
                $result_array[$count]->{paths}->[$limit]->{device_status} = $6 if ($6);
                $limit++;
            }
        }
    }
    return \@result_array;
} ## end sub multipath

1;
