#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/mv.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse mv command
##
## @description
##    This is a library to parse mv command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mv" will mv()
##
## @synopsis
##    use mv;
##    mv(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Unix::mv;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /((mv|Usage):.*)/ && $raw_output !~ /overwrite/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
        if ( $raw_output =~ /(Insufficient arguments)/ ) {
            $error         = "MissingRequiredParam";
            $error_message = $1;
        } elsif ( $raw_output =~ /(cannot access)/ ) {
            $error         = "InvalidParamValue";
            $error_message = $1;
        } elsif ( $raw_output =~ /(command not found)/ ) {
            $error         = "CommandFailed";
            $error_message = $1;
        }
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    }
} ## end sub _check_error

## @name ::mv
##
## @summary Parses the raw output from the "mv" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mv with any of its options as described in the 
##    corresponding cdef file 'mv.cdefs'
## @synopsis
##      mv(%args);
## INPUTS TO METHOD CALL 'mv' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option supports both Linux & Solaris
## <tr> <td> No switch for this key </td> <td> 'source' </td> <td> '0' </td> </tr> 
##
## The below options support only Linux
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '--backup' </td> <td> 'specific-backup' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'no-clobber' </td> <td> '1' </td> </tr> 
##
## The below option support both Linux & Solaris
## <tr> <td> '-i' </td> <td> 'interactive' </td> <td> '1' </td> </tr> 
##
## The below options support only Linux
## <tr> <td> '-T' </td> <td> 'no-target-directory' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> '--strip-trailing-slashes' </td> <td> '1' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'override-backup-suffix' </td> <td> '0' </td> </tr> 
##
## The below option support both Linux & Solaris
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr> 
##
## The below options support only Linux
## <tr> <td> No switch for this key </td> <td> '--version' </td> <td> '1' </td> </tr> 
##
## The below option support both Linux & Solaris
## <tr> <td> No switch for this key </td> <td> 'target' </td> <td> '0' </td> </tr> 
##
## The below options support only Linux
## <tr> <td> '=' </td> <td> 'suffix' </td> <td> '0' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'normal-backup' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'target-directory' </td> <td> '1' </td> </tr> 
## <tr> <td> '-u' </td> <td> 'update' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> '--help' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mv /root/dir/file1.txt /root/dir/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -f /root/dir/file1.txt /root/dir/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -i /root/dir/file1.txt /root/dir/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;mv: overwrite '/root/dir/file2.txt' ?
##&nbsp;y\r
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -n /root/dir/file1.txt /root/dir/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -t /root/dir1 /root/dir2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -T /root/dir123 /root/file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -u /root/dir123/file1.txt /root/dir1/file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -v /root/dir1/file2.txt /root/dir1/file3.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;`/root/dir1/file2.txt' -> `/root/dir1/file3.txt'
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -S 2012 file1.txt dir1/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv --strip-trailing-slashes file1.txt/ dir123/file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv -b file1.txt file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv --backup=numbered sample1 sample2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv --help
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Usage: mv [OPTION]... [-T] SOURCE DEST
##&nbsp;  or:  mv [OPTION]... SOURCE... DIRECTORY
##&nbsp;  or:  mv [OPTION]... -t DIRECTORY SOURCE...
##&nbsp;Rename SOURCE to DEST, or move SOURCE(s) to DIRECTORY.
##&nbsp;
##&nbsp;Mandatory arguments to long options are mandatory for short options too.
##&nbsp;      --backup[=CONTROL]       make a backup of each existing destination file
##&nbsp;  -b                           like --backup but does not accept an argument
##&nbsp;  -f, --force                  do not prompt before overwriting
##&nbsp;  -i, --interactive            prompt before overwrite
##&nbsp;  -n, --no-clobber             do not overwrite an existing file
##&nbsp;If you specify more than one of -i, -f, -n, only the final one takes effect.
##&nbsp;      --strip-trailing-slashes  remove any trailing slashes from each SOURCE
##&nbsp;                                 argument
##&nbsp;  -S, --suffix=SUFFIX          override the usual backup suffix
##&nbsp;  -t, --target-directory=DIRECTORY  move all SOURCE arguments into DIRECTORY
##&nbsp;  -T, --no-target-directory    treat DEST as a normal file
##&nbsp;  -u, --update                 move only when the SOURCE file is newer
##&nbsp;                                 than the destination file or when the
##&nbsp;                                 destination file is missing
##&nbsp;  -v, --verbose                explain what is being done
##&nbsp;      --help     display this help and exit
##&nbsp;      --version  output version information and exit
##&nbsp;
##&nbsp;The backup suffix is `~', unless set with --suffix or SIMPLE_BACKUP_SUFFIX.
##&nbsp;The version control method may be selected via the --backup option or through
##&nbsp;the VERSION_CONTROL environment variable.  Here are the values:
##&nbsp;
##&nbsp;  none, off       never make backups (even if --backup is given)
##&nbsp;  numbered, t     make numbered backups
##&nbsp;  existing, nil   numbered if numbered backups exist, simple otherwise
##&nbsp;  simple, never   always make simple backups
##&nbsp;
##&nbsp;Report mv bugs to bug-coreutils@gnu.org
##&nbsp;GNU coreutils home page: <http://www.gnu.org/software/coreutils/>
##&nbsp;General help using GNU software: <http://www.gnu.org/gethelp/>
##&nbsp;For complete documentation, run: info coreutils 'mv invocation'
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = mv --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;mv (GNU coreutils) 8.4
##&nbsp;Copyright (C) 2010 Free Software Foundation, Inc.
##&nbsp;License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
##&nbsp;This is free software: you are free to change and redistribute it.
##&nbsp;There is NO WARRANTY, to the extent permitted by law.
##&nbsp;
##&nbsp;Written by Mike Parker, David MacKenzie, and Jim Meyering.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub mv {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    return [];
} ## end sub mv
1;
