#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/sleep.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux & Unix(Solaris, Freebsd) sleep command
##
## @description
##    This is a library to parse the raw output of Linux sleep command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sleep" will be sleep
##
## @synopsis
##    use sleep;
##
##    sleep(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::sleep;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::sleep
##
## @summary Parses the raw output from the "sleep" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sleep with any of its options as described in the
##    corresponding cdef file 'sleep.cdefs'
## @synopsis
##      sleep(%args);
## INPUTS TO METHOD CALL 'sleep' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'time-interval' </td> <td> '0' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sleep  5s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = sleep --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;sleep (GNU coreutils) 5.97
##&nbsp;Copyright (C) 2006 Free Software Foundation, Inc.
##&nbsp;This is free software.  You may redistribute copies of it under the terms of
##&nbsp;the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.
##&nbsp;There is NO WARRANTY, to the extent permitted by law.
##&nbsp;
##&nbsp;Written by Jim Meyering and Paul Eggert.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sleep {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub sleep

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(sleep:)\s+.* / ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(Usage:).*/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

