#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/ulimit.pm#1 $#
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse ulimit command
##
## @description
##    This is a library to parse the raw output of ulimit command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ulimit" will ulimit()
##
## @synopsis
##    use ulimit;
##
##    ulimit(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::ulimit;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::ulimit
##
## @summary Parses the raw output from the "ulimit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ulimit with any of its options as described in the
##    corresponding cdef file 'ulimit.cdefs'
## @synopsis
##      ulimit(%args);
## INPUTS TO METHOD CALL 'ulimit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below options support AIX, Linux & Solaris
## <tr> <td> '-p' </td> <td> 'pipe-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'virtual-memory' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-open-files' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'max-user-processes' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-current-resource' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'hard-limit' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'stack-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'data-segment-size' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'limit' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'core-file-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'max-memory-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'soft-limit' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'cpu-time' </td> <td> '1' </td> </tr>
##
## The below options support only AIX
## <tr> <td> No switch for this key </td> <td> 'file-path' </td> <td> '1' </td> </tr>
##
## The below options support only Linux
## <tr> <td> '-e' </td> <td> 'scheduling-priority' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'no-of-pending-signals' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'max-memory-lock-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'bytes-in-posix-message-queues' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'real-time-scheduling-priority' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'file-locks' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ulimit -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;131072
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'limit' => '131072'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ulimit -d 1024
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = ulimit -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;time(seconds)        unlimited
##&nbsp;file(blocks)         2097151
##&nbsp;data(kbytes)         131072
##&nbsp;stack(kbytes)        32768
##&nbsp;memory(kbytes)       32768
##&nbsp;coredump(blocks)     2097151
##&nbsp;nofiles(descriptors) 2000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nofiles(descriptors)' => '2000',
##&nbsp;            'memory(kbytes)' => '32768',
##&nbsp;            'data(kbytes)' => '131072',
##&nbsp;            'file(blocks)' => '2097151',
##&nbsp;            'time(seconds)' => 'unlimited',
##&nbsp;            'coredump(blocks)' => '2097151',
##&nbsp;            'stack(kbytes)' => '32768'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ulimit -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;2097151
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'limit' => '2097151'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ulimit -i
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;16376
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'limit' => '16376'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ulimit -q 1024
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = ulimit -x 100
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ulimit {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @var;
    my $c = 0;

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $cmd =~ /(-a)/ ) {
            if ( $lines[$i] =~ /^\w+\(.*\)\s+(\w+)$/ ) {
                @var = split( /\s+/, $lines[$i] );
                my @values = split( /\s+/, $lines[$i] );
                $result_array[$c]->{ $var[0] } = $values[$#values];
            } else {
                @var = split( /\s\s+/, $lines[$i] );
                my @values = split( /\s+/, $lines[$i] );
                $result_array[$c]->{ to_key_name( item => $var[0] ) }
                    = $values[$#values];
            }
        } else {
            $result_array[0]->{limit} = $lines[$i];
        }
    }
    return \@result_array;
} ## end sub ulimit

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:|ulimit:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
