#Copyright (c) 2012-13 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse vgdisplay command
##
## @description
##    This is a library to parse the raw output of vgdisplay command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vgdisplay" will be vgdisplay()
##
## @synopsis
##    use use NACL::APISet::Host::CLI::Unix::vgdisplay;
##
##    vgdisplay(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vgdisplay;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
## @name ::vgdisplay
##
## @summary Parses the raw output from the "vgdisplay" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vgdisplay with any of its options as described in the 
##    corresponding cdef file 'vgdisplay.cdefs'
## @synopsis
##      vgdisplay(%args);
## INPUTS TO METHOD CALL 'vgdisplay' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-F' </td> <td> 'compact-listing' </td> <td> '1' </td> </tr> 
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'volume-group-name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'display-volume-group-info' </td> <td> '0' </td> </tr> 
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignore-locking-failure' </td> <td> '1' </td> </tr> 
## <tr> <td> '-P' </td> <td> 'partial' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'colon' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'short' </td> <td> '1' </td> </tr> 
## <tr> <td> '-C' </td> <td> 'columns' </td> <td> '1' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'activevolumegroups' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vgdisplay  -A
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Volume group ---
##&nbsp;  VG Name               vg1
##&nbsp;  System ID             
##&nbsp;  Format                lvm2
##&nbsp;  Metadata Areas        1
##&nbsp;  Metadata Sequence No  3
##&nbsp;  VG Access             read/write
##&nbsp;  VG Status             resizable
##&nbsp;  MAX LV                0
##&nbsp;  Cur LV                2
##&nbsp;  Open LV               0
##&nbsp;  Max PV                0
##&nbsp;  Cur PV                1
##&nbsp;  Act PV                1
##&nbsp;  VG Size               149.05 GiB
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Alloc PE / Size       2 / 8.00 MiB
##&nbsp;  Free  PE / Size       38154 / 149.04 GiB
##&nbsp;  VG UUID               Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;   
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'system_id' => '-',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'alloc_pe_size' => '8.00 MiB',
##&nbsp;            'max_pv' => '0',
##&nbsp;            'vg_status' => 'resizable',
##&nbsp;            'alloc_pe' => '2',
##&nbsp;            'vg_size' => '149.05 GiB',
##&nbsp;            'cur_lv' => '2',
##&nbsp;            'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;            'total_pe' => '38156',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38154',
##&nbsp;            'vg_access' => 'read/write',
##&nbsp;            'metadata_areas' => '1',
##&nbsp;            'metadata_sequence_no' => '3',
##&nbsp;            'format' => 'lvm2',
##&nbsp;            'free_pe_size' => '149.04 GiB',
##&nbsp;            'max_lv' => '0',
##&nbsp;            'open_lv' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  vg1:r/w:772:-1:0:2:0:-1:0:1:1:156286976:4096:38156:2:38154:Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'vg_mode' => '772',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'max_pv' => '0',
##&nbsp;            'alloc_pe' => '2',
##&nbsp;            'cur_lv' => '2',
##&nbsp;            'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;            'total_pe' => '38156',
##&nbsp;            'internal_vg_number' => '-1',
##&nbsp;            'max_lv_size' => '-1',
##&nbsp;            'pe_size_kb' => '4096',
##&nbsp;            'free_pe' => '38154',
##&nbsp;            'vg_size_kb' => '156286976',
##&nbsp;            'vg_access' => 'r/w',
##&nbsp;            'max_lv' => '0',
##&nbsp;            'open_lv' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Volume group ---
##&nbsp;  VG Name               vg1
##&nbsp;  System ID             
##&nbsp;  Format                lvm2
##&nbsp;  Metadata Areas        1
##&nbsp;  Metadata Sequence No  3
##&nbsp;  VG Access             read/write
##&nbsp;  VG Status             resizable
##&nbsp;  MAX LV                0
##&nbsp;  Cur LV                2
##&nbsp;  Open LV               0
##&nbsp;  Max PV                0
##&nbsp;  Cur PV                1
##&nbsp;  Act PV                1
##&nbsp;  VG Size               149.05 GiB
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Alloc PE / Size       2 / 8.00 MiB
##&nbsp;  Free  PE / Size       38154 / 149.04 GiB
##&nbsp;  VG UUID               Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;   
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'system_id' => '-',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'alloc_pe_size' => '8.00 MiB',
##&nbsp;            'max_pv' => '0',
##&nbsp;            'vg_status' => 'resizable',
##&nbsp;            'alloc_pe' => '2',
##&nbsp;            'vg_size' => '149.05 GiB',
##&nbsp;            'cur_lv' => '2',
##&nbsp;            'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;            'total_pe' => '38156',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38154',
##&nbsp;            'vg_access' => 'read/write',
##&nbsp;            'metadata_areas' => '1',
##&nbsp;            'metadata_sequence_no' => '3',
##&nbsp;            'format' => 'lvm2',
##&nbsp;            'free_pe_size' => '149.04 GiB',
##&nbsp;            'max_lv' => '0',
##&nbsp;            'open_lv' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  --ignorelockingfailure
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  --- Volume group ---
##&nbsp;  VG Name               vg1
##&nbsp;  System ID             
##&nbsp;  Format                lvm2
##&nbsp;  Metadata Areas        1
##&nbsp;  Metadata Sequence No  3
##&nbsp;  VG Access             read/write
##&nbsp;  VG Status             resizable
##&nbsp;  MAX LV                0
##&nbsp;  Cur LV                2
##&nbsp;  Open LV               0
##&nbsp;  Max PV                0
##&nbsp;  Cur PV                1
##&nbsp;  Act PV                1
##&nbsp;  VG Size               149.05 GiB
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Alloc PE / Size       2 / 8.00 MiB
##&nbsp;  Free  PE / Size       38154 / 149.04 GiB
##&nbsp;  VG UUID               Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;   
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'system_id' => '-',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'alloc_pe_size' => '8.00 MiB',
##&nbsp;            'max_pv' => '0',
##&nbsp;            'vg_status' => 'resizable',
##&nbsp;            'alloc_pe' => '2',
##&nbsp;            'vg_size' => '149.05 GiB',
##&nbsp;            'cur_lv' => '2',
##&nbsp;            'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;            'total_pe' => '38156',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38154',
##&nbsp;            'vg_access' => 'read/write',
##&nbsp;            'metadata_areas' => '1',
##&nbsp;            'metadata_sequence_no' => '3',
##&nbsp;            'format' => 'lvm2',
##&nbsp;            'free_pe_size' => '149.04 GiB',
##&nbsp;            'max_lv' => '0',
##&nbsp;            'open_lv' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -P
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Partial mode. Incomplete logical volumes will be processed.
##&nbsp;  --- Volume group ---
##&nbsp;  VG Name               vg1
##&nbsp;  System ID             
##&nbsp;  Format                lvm2
##&nbsp;  Metadata Areas        1
##&nbsp;  Metadata Sequence No  3
##&nbsp;  VG Access             read/write
##&nbsp;  VG Status             resizable
##&nbsp;  MAX LV                0
##&nbsp;  Cur LV                2
##&nbsp;  Open LV               0
##&nbsp;  Max PV                0
##&nbsp;  Cur PV                1
##&nbsp;  Act PV                1
##&nbsp;  VG Size               149.05 GiB
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Alloc PE / Size       2 / 8.00 MiB
##&nbsp;  Free  PE / Size       38154 / 149.04 GiB
##&nbsp;  VG UUID               Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;   
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'system_id' => '-',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => 'vg1',
##&nbsp;            'alloc_pe_size' => '8.00 MiB',
##&nbsp;            'max_pv' => '0',
##&nbsp;            'vg_status' => 'resizable',
##&nbsp;            'alloc_pe' => '2',
##&nbsp;            'vg_size' => '149.05 GiB',
##&nbsp;            'cur_lv' => '2',
##&nbsp;            'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;            'total_pe' => '38156',
##&nbsp;            'pe_size' => '4.00 MiB',
##&nbsp;            'free_pe' => '38154',
##&nbsp;            'vg_access' => 'read/write',
##&nbsp;            'metadata_areas' => '1',
##&nbsp;            'metadata_sequence_no' => '3',
##&nbsp;            'format' => 'lvm2',
##&nbsp;            'free_pe_size' => '149.04 GiB',
##&nbsp;            'max_lv' => '0',
##&nbsp;            'open_lv' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  "vg1" 149.05 GiB [8.00 MiB  used / 149.04 GiB free]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'free_space' => '149.04 GiB',
##&nbsp;            'vg_size' => '149.05 GiB',
##&nbsp;            'used_space' => '8.00 MiB ',
##&nbsp;            'vg_name' => 'vg1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Finding all volume groups
##&nbsp;    Finding volume group "vg1"
##&nbsp;  --- Volume group ---
##&nbsp;  VG Name               vg1
##&nbsp;  System ID             
##&nbsp;  Format                lvm2
##&nbsp;  Metadata Areas        1
##&nbsp;  Metadata Sequence No  3
##&nbsp;  VG Access             read/write
##&nbsp;  VG Status             resizable
##&nbsp;  MAX LV                0
##&nbsp;  Cur LV                2
##&nbsp;  Open LV               0
##&nbsp;  Max PV                0
##&nbsp;  Cur PV                1
##&nbsp;  Act PV                1
##&nbsp;  VG Size               149.05 GiB
##&nbsp;  PE Size               4.00 MiB
##&nbsp;  Total PE              38156
##&nbsp;  Alloc PE / Size       2 / 8.00 MiB
##&nbsp;  Free  PE / Size       38154 / 149.04 GiB
##&nbsp;  VG UUID               Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww
##&nbsp;   
##&nbsp;  --- Logical volume ---
##&nbsp;  LV Path                /dev/vg1/lvol0
##&nbsp;  LV Name                lvol0
##&nbsp;  VG Name                vg1
##&nbsp;  LV UUID                hHlf4f-HDoG-Jxyu-Kj4n-1Cwi-VUPo-FR9qAl
##&nbsp;  LV Write Access        read/write
##&nbsp;  LV Creation host, time lnx219-45.lab.eng.btc.netapp.in, 2013-03-08 14:54:19 +0530
##&nbsp;  LV Status              available
##&nbsp;  # open                 0
##&nbsp;  LV Size                4.00 MiB
##&nbsp;  Current LE             1
##&nbsp;  Segments               1
##&nbsp;  Allocation             inherit
##&nbsp;  Read ahead sectors     auto
##&nbsp;  - currently set to     256
##&nbsp;  Block device           253:0
##&nbsp;   
##&nbsp;  --- Logical volume ---
##&nbsp;  LV Path                /dev/vg1/lvol1
##&nbsp;  LV Name                lvol1
##&nbsp;  VG Name                vg1
##&nbsp;  LV UUID                FMWbgS-Ypwt-0K1G-oDZx-6P1W-nujr-xj12Vd
##&nbsp;  LV Write Access        read/write
##&nbsp;  LV Creation host, time lnx219-45.lab.eng.btc.netapp.in, 2013-03-08 14:54:22 +0530
##&nbsp;  LV Status              available
##&nbsp;  # open                 0
##&nbsp;  LV Size                4.00 MiB
##&nbsp;  Current LE             1
##&nbsp;  Segments               1
##&nbsp;  Allocation             inherit
##&nbsp;  Read ahead sectors     auto
##&nbsp;  - currently set to     256
##&nbsp;  Block device           253:1
##&nbsp;   
##&nbsp;  --- Physical volumes ---
##&nbsp;  PV Name               /dev/sdb     
##&nbsp;  PV UUID               98TIza-0bPf-1G5t-K6jC-L3fc-s0Ei-MyHNcn
##&nbsp;  PV Status             allocatable
##&nbsp;  Total PE / Free PE    38156 / 38154
##&nbsp;
##&nbsp;  PV Name               /dev/mapper/360a9800032466648455d43323071566f
##&nbsp;  PV UUID               gWOHkk-CbpE-Szu0-Cewm-ZVyg-zqIX-shAZif
##&nbsp;  PV Status             allocatable
##&nbsp;  Total PE / Free PE    511 / 280
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'logical_volume' => [
##&nbsp;                                  {
##&nbsp;                                    'lv_size' => '4.00 MiB',
##&nbsp;                                    'time' => '2013-03-08 14:54:19 +0530',
##&nbsp;                                    'lv_creation_host' => 'lnx219-45.lab.eng.btc.netapp.in',
##&nbsp;                                    'vg_name' => 'vg1',
##&nbsp;                                    'segments' => '1',
##&nbsp;                                    'read_ahead_sectors' => 'auto',
##&nbsp;                                    'currently_set_to' => '256',
##&nbsp;                                    'lv_name' => 'lvol0',
##&nbsp;                                    'lv_uuid' => 'hHlf4f-HDoG-Jxyu-Kj4n-1Cwi-VUPo-FR9qAl',
##&nbsp;                                    'allocation' => 'inherit',
##&nbsp;                                    'lv_write_access' => 'read/write',
##&nbsp;                                    'lv_status' => 'available',
##&nbsp;                                    'open' => '0',
##&nbsp;                                    'lv_path' => '/dev/vg1/lvol0',
##&nbsp;                                    'current_le' => '1',
##&nbsp;                                    'block_device' => '253:0'
##&nbsp;                                  },
##&nbsp;                                  {
##&nbsp;                                    'lv_size' => '4.00 MiB',
##&nbsp;                                    'time' => '2013-03-08 14:54:22 +0530',
##&nbsp;                                    'lv_creation_host' => 'lnx219-45.lab.eng.btc.netapp.in',
##&nbsp;                                    'vg_name' => 'vg1',
##&nbsp;                                    'segments' => '1',
##&nbsp;                                    'read_ahead_sectors' => 'auto',
##&nbsp;                                    'currently_set_to' => '256',
##&nbsp;                                    'lv_name' => 'lvol1',
##&nbsp;                                    'lv_uuid' => 'FMWbgS-Ypwt-0K1G-oDZx-6P1W-nujr-xj12Vd',
##&nbsp;                                    'allocation' => 'inherit',
##&nbsp;                                    'lv_write_access' => 'read/write',
##&nbsp;                                    'lv_status' => 'available',
##&nbsp;                                    'open' => '0',
##&nbsp;                                    'lv_path' => '/dev/vg1/lvol1',
##&nbsp;                                    'current_le' => '1',
##&nbsp;                                    'block_device' => '253:1'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'physical_volumes' => [
##&nbsp;                                    {
##&nbsp;                                      'pv_status' => 'allocatable',
##&nbsp;                                      'free_pe' => '38154',
##&nbsp;                                      'total_pe' => '38156',
##&nbsp;                                      'pv_uuid' => '98TIza-0bPf-1G5t-K6jC-L3fc-s0Ei-MyHNcn',
##&nbsp;                                      'pv_name' => '/dev/sdb'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'pv_status' => 'allocatable',
##&nbsp;                                      'free_pe' => '280',
##&nbsp;                                      'total_pe' => '511',
##&nbsp;                                      'pv_uuid' => 'pOdNdm-eHeO-BIWQ-TA8D-fQnF-OSRx-vcQ99H',
##&nbsp;                                      'pv_name' => '/dev/mapper/360a9800032466648455d43323071566d'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'volume_group' => [
##&nbsp;                                {
##&nbsp;                                  'cur_pv' => '1',
##&nbsp;                                  'system_id' => '-',
##&nbsp;                                  'act_pv' => '1',
##&nbsp;                                  'vg_name' => 'vg1',
##&nbsp;                                  'alloc_pe_size' => '8.00 MiB',
##&nbsp;                                  'max_pv' => '0',
##&nbsp;                                  'vg_status' => 'resizable',
##&nbsp;                                  'alloc_pe' => '2',
##&nbsp;                                  'vg_size' => '149.05 GiB',
##&nbsp;                                  'cur_lv' => '2',
##&nbsp;                                  'vg_uuid' => 'Tvkigp-NDxZ-ZOQS-t9da-0yVX-myMQ-6sPVww',
##&nbsp;                                  'total_pe' => '38156',
##&nbsp;                                  'pe_size' => '4.00 MiB',
##&nbsp;                                  'free_pe' => '38154',
##&nbsp;                                  'vg_access' => 'read/write',
##&nbsp;                                  'metadata_areas' => '1',
##&nbsp;                                  'metadata_sequence_no' => '3',
##&nbsp;                                  'format' => 'lvm2',
##&nbsp;                                  'free_pe_size' => '149.04 GiB',
##&nbsp;                                  'max_lv' => '0',
##&nbsp;                                  'open_lv' => '0'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LVM version:     2.02.95(2)-RHEL6 (2012-05-16)
##&nbsp;  Library version: 1.02.74-RHEL6 (2012-05-16)
##&nbsp;  Driver version:  4.22.6
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'library_version' => '1.02.74-RHEL6 (2012-05-16)',
##&nbsp;            'driver_version' => '4.22.6',
##&nbsp;            'lvm_version' => '2.02.95(2)-RHEL6 (2012-05-16)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp; The Below outputs is from HPUX machine
##&nbsp;
##&nbsp;Command = vgdisplay  -V  1.0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;--- Volume groups ---
##&nbsp;VG Name                     /dev/vg00
##&nbsp;VG Write Access             read/write
##&nbsp;VG Status                   available
##&nbsp;Max LV                      255
##&nbsp;Cur LV                      3
##&nbsp;Open LV                     3
##&nbsp;Max PV                      16
##&nbsp;Cur PV                      1
##&nbsp;Act PV                      1
##&nbsp;Max PE per PV               4328
##&nbsp;VGDA                        2
##&nbsp;PE Size (Mbytes)            16
##&nbsp;Total PE                    4319
##&nbsp;Alloc PE                    4318
##&nbsp;Free PE                     1
##&nbsp;Total PVG                   0
##&nbsp;Total Spare PVs             0
##&nbsp;Total Spare PVs in use      0
##&nbsp;VG Version                  1.0
##&nbsp;VG Max Size                 1082g
##&nbsp;VG Max Extents              69248
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'total_spare_pvs_in_use' => '0',
##&nbsp;            'vgda' => '2',
##&nbsp;            'vg_version' => '1.0',
##&nbsp;            'max_pe_per_pv' => '4328',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => '/dev/vg00',
##&nbsp;            'vg_status' => 'available',
##&nbsp;            'max_pv' => '16',
##&nbsp;            'alloc_pe' => '4318',
##&nbsp;            'vg_write_access' => 'read/write',
##&nbsp;            'cur_lv' => '3',
##&nbsp;            'vg_max_extents' => '69248',
##&nbsp;            'total_pvg' => '0',
##&nbsp;            'total_pe' => '4319',
##&nbsp;            'total_spare_pvs' => '0',
##&nbsp;            'free_pe' => '1',
##&nbsp;            'vg_max_size' => '1082g',
##&nbsp;            'max_lv' => '255',
##&nbsp;            'pe_size_mbytes' => '16',
##&nbsp;            'open_lv' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgdisplay  -F /dev/vg00/lvol3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;vg_name=/dev/vg00:vg_write_access=read,write:vg_status=available:max_lv=255:cur_lv=3:open_lv=3:max_pv=16:cur_pv=1:act_pv=1:max_pe_per_pv=4328:vgda=2:pe_size=16:total_pe=4319:alloc_pe=4318:free_pe=1:total_pvg=0:total_spare_pvs=0:total_spare_pvs_in_use=0:vg_version=1.0:vg_max_size=1082g:vg_max_extents=69248
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cur_pv' => '1',
##&nbsp;            'total_spare_pvs_in_use' => '0',
##&nbsp;            'vgda' => '2',
##&nbsp;            'vg_version' => '1.0',
##&nbsp;            'max_pe_per_pv' => '4328',
##&nbsp;            'act_pv' => '1',
##&nbsp;            'vg_name' => '/dev/vg00',
##&nbsp;            'vg_status' => 'available',
##&nbsp;            'max_pv' => '16',
##&nbsp;            'alloc_pe' => '4318',
##&nbsp;            'vg_write_access' => 'read,write',
##&nbsp;            'cur_lv' => '3',
##&nbsp;            'vg_max_extents' => '69248',
##&nbsp;            'total_pvg' => '0',
##&nbsp;            'total_pe' => '4319',
##&nbsp;            'pe_size' => '16',
##&nbsp;            'total_spare_pvs' => '0',
##&nbsp;            'free_pe' => '1',
##&nbsp;            'vg_max_size' => '1082g',
##&nbsp;            'max_lv' => '255',
##&nbsp;            'open_lv' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = vgdisplay  -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;--- Volume groups ---
##&nbsp;VG Name                     /dev/vg00
##&nbsp;VG Write Access             read/write
##&nbsp;VG Status                   available
##&nbsp;Max LV                      255
##&nbsp;Cur LV                      3
##&nbsp;Open LV                     3
##&nbsp;Max PV                      16
##&nbsp;Cur PV                      1
##&nbsp;Act PV                      1
##&nbsp;Max PE per PV               4328
##&nbsp;VGDA                        2
##&nbsp;PE Size (Mbytes)            16
##&nbsp;Total PE                    4319
##&nbsp;Alloc PE                    4318
##&nbsp;Free PE                     1
##&nbsp;Total PVG                   0
##&nbsp;Total Spare PVs             0
##&nbsp;Total Spare PVs in use      0
##&nbsp;VG Version                  1.0
##&nbsp;VG Max Size                 1082g
##&nbsp;VG Max Extents              69248
##&nbsp;
##&nbsp;--- Logical volumes ---
##&nbsp;LV Name                     /dev/vg00/lvol1
##&nbsp;LV Status                   available/syncd
##&nbsp;LV Size (Mbytes)            1792
##&nbsp;Current LE                  112
##&nbsp;Allocated PE                112
##&nbsp;Used PV                     1
##&nbsp;
##&nbsp;LV Name                     /dev/vg00/lvol2
##&nbsp;LV Status                   available/syncd
##&nbsp;LV Size (Mbytes)            8192
##&nbsp;Current LE                  512
##&nbsp;Allocated PE                512
##&nbsp;Used PV                     1
##&nbsp;
##&nbsp;LV Name                     /dev/vg00/lvol3
##&nbsp;LV Status                   available/syncd
##&nbsp;LV Size (Mbytes)            59104
##&nbsp;Current LE                  3694
##&nbsp;Allocated PE                3694
##&nbsp;Used PV                     1
##&nbsp;
##&nbsp;
##&nbsp;--- Physical volumes ---
##&nbsp;PV Name                     /dev/disk/disk5_p2
##&nbsp;PV Status                   available
##&nbsp;Total PE                    4319
##&nbsp;Free PE                     1
##&nbsp;Autoswitch                  On
##&nbsp;Proactive Polling           On
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'logical_volumes' => [
##&nbsp;                                   {
##&nbsp;                                     'allocated_pe' => '112',
##&nbsp;                                     'lv_status' => 'available/syncd',
##&nbsp;                                     'lv_size_mbytes' => '1792',
##&nbsp;                                     'used_pv' => '1',
##&nbsp;                                     'lv_name' => '/dev/vg00/lvol1',
##&nbsp;                                     'current_le' => '112'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'allocated_pe' => '512',
##&nbsp;                                     'lv_status' => 'available/syncd',
##&nbsp;                                     'lv_size_mbytes' => '8192',
##&nbsp;                                     'used_pv' => '1',
##&nbsp;                                     'lv_name' => '/dev/vg00/lvol2',
##&nbsp;                                     'current_le' => '512'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'allocated_pe' => '3694',
##&nbsp;                                     'lv_status' => 'available/syncd',
##&nbsp;                                     'lv_size_mbytes' => '59104',
##&nbsp;                                     'used_pv' => '1',
##&nbsp;                                     'lv_name' => '/dev/vg00/lvol3',
##&nbsp;                                     'current_le' => '3694'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'volume_groups' => [
##&nbsp;                                 {
##&nbsp;                                   'cur_pv' => '1',
##&nbsp;                                   'total_spare_pvs_in_use' => '0',
##&nbsp;                                   'vgda' => '2',
##&nbsp;                                   'vg_version' => '1.0',
##&nbsp;                                   'max_pe_per_pv' => '4328',
##&nbsp;                                   'act_pv' => '1',
##&nbsp;                                   'vg_name' => '/dev/vg00',
##&nbsp;                                   'vg_status' => 'available',
##&nbsp;                                   'max_pv' => '16',
##&nbsp;                                   'alloc_pe' => '4318',
##&nbsp;                                   'vg_write_access' => 'read/write',
##&nbsp;                                   'cur_lv' => '3',
##&nbsp;                                   'vg_max_extents' => '69248',
##&nbsp;                                   'total_pvg' => '0',
##&nbsp;                                   'total_pe' => '4319',
##&nbsp;                                   'total_spare_pvs' => '0',
##&nbsp;                                   'free_pe' => '1',
##&nbsp;                                   'vg_max_size' => '1082g',
##&nbsp;                                   'max_lv' => '255',
##&nbsp;                                   'pe_size_mbytes' => '16',
##&nbsp;                                   'open_lv' => '3'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'physical_volumes' => [
##&nbsp;                                    {
##&nbsp;                                      'pv_status' => 'available',
##&nbsp;                                      'autoswitch' => 'On',
##&nbsp;                                      'free_pe' => '1',
##&nbsp;                                      'proactive_polling' => 'On',
##&nbsp;                                      'total_pe' => '4319',
##&nbsp;                                      'pv_name' => '/dev/disk/disk5_p2'
##&nbsp;                                    }
##&nbsp;                                  ]
##&nbsp;          }
##&nbsp;        ];
sub vgdisplay {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $cmd    = $resp->get_command();
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    my ($head, @result_array);
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    @lines = grep(! /Input\/output\s+error|echo/, @lines);
    my $count = -1;
    my %res   = (
        '#PV' => 'cur_pv',
        VG    => 'vg_name',
        '#LV' => 'cur_lv',
        '#SN' => 'sn',
        Attr  => 'attr',
        VSize => 'vg_size',
        VFree => 'vg_free',
    );

    if ($cmd =~ /--version/) {
        $count++;
        for (my $i = 0; $i <= $#lines; $i++) {
            my @temp = split(/:\s+/, $lines[$i]) ;
            $result_array[$count]->{to_key_name(item => $temp[0])} = $temp[1];
        }
    } elsif ($cmd =~ /-c/) {
        for (my $i = 0; $i <= $#lines; $i++) {
            my @head = (
                qw( vg_name vg_access vg_mode internal_vg_number max_lv cur_lv open_lv max_lv_size max_pv cur_pv act_pv
                    vg_size_kb pe_size_kb total_pe alloc_pe free_pe vg_uuid )
            );
            $count++;
            my @var = split(/:/, $lines[$i]);
            for (my $j = 0; $j <= $#var; $j++) {
                 $result_array[$count]->{to_key_name(item => $head[$j])}
                    = $var[$j];
            }
        }
    } elsif ($cmd =~ /-s/) {
        for (my $i = 0; $i <= $#lines; $i++) {
            if ($lines[$i]
                =~ /\"(.*)\"\s+(.*)\s+\[(.*)\s+used\s+\/\s+(.*)\s+free\]/) {
                $count++;
                $result_array[$count]->{vg_name}    = $1;
                $result_array[$count]->{vg_size}    = $2;
                $result_array[$count]->{used_space} = $3;
                $result_array[$count]->{free_space} = $4;
            }
        }
    } elsif ($cmd =~ /-F/) {
        for (my $i = 0; $i <= $#lines; $i++) {
            $count++ ;
            my @arr = split(/:/, $lines[$i]) ;
            for (my $j = 0; $j <= $#arr; $j++) {
                my @arr1 = split(/=/, $arr[$j]);
                $result_array[$count]->{to_key_name(item => $arr1[0])}
                    = $arr1[1];
            }
        }
    } elsif ($cmd =~ /-C/) {
        my @head = split( /\s+/, $lines[0] );
        for (my $i = 1; $i <= $#lines; $i++) {
                $count++;
                my @values = split(/\s+/, $lines[$i]);
                for (my $j = 0; $j <= $#head; $j++) {
                    $result_array[$count]->{$res{$head[$j]}} = $values[$j];
                }
            }
    } elsif ( $cmd =~ /-v/ ) {
        $count++;
        my $limit = -1;
        my $flag  = -1;
        my $loop  = -1;
        my $set   = 1;
        my ( $pv_name, $lv_name );
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
    
            if ( $lines[$i] =~ /\---\s(.*)\s\---/ ) {
                $head = $1;
                if ( $lines[$i] =~ /\---\s(Logical\s+volume)\s\---/ ) {
                    $limit++;
                    $set = 0;
                }
                $loop++;
            }
            my @arr = split( /\s\s+/, $lines[$i] );
            $arr[0] =~ s/#\s+|-\s+//;
            $arr[0] =~ s/\)//;
            $arr[0] =~ s/\(//;
            foreach (@arr) {
                $_ = "_" if ( not defined $_ );
            }
            if ( $head =~ /Logical\s+volume/ && $lines[$i] !~ /--.*--/) {
                if ( $lines[$i] =~ /LV\s+Name\s+(.*)/ ) {
                    $lv_name = $1;
                    $limit++ if ($set);
                }
                elsif ( $lines[$i] =~ /(LV.*)\,\s+(time)\s+(.*)\,\s+(.*)/ ) {
                    $result_array[$count]->{ to_key_name( item => $head ) }
                      ->[$limit]->{ to_key_name( item => $1 ) } = $3;
                    $result_array[$count]->{ to_key_name( item => $head ) }
                      ->[$limit]->{ to_key_name( item => $2 ) } = $4;
                }
                else {
                    $result_array[$count]->{ to_key_name( item => $head ) }
                      ->[$limit]->{lv_name} = $lv_name
                      if ( defined($lv_name) );
                    $result_array[$count]->{ to_key_name( item => $head ) }
                      ->[$limit]->{ to_key_name( item => $arr[0] ) } =
                      defined( $arr[1] ) ? $arr[1] : '-';
                }
            } elsif ( $head =~ /Physical\s+volumes/ && $lines[$i] !~ /--.*--/ ) {
                if ( $lines[$i] =~ /PV\s+Name\s+(.*)/ ) {
                    $pv_name = $1;
                    $flag++;
                }
                elsif ( $lines[$i] =~ /(.*)\s+\/\s+(.*\w+)\s+(\d+)\s+\/\s+(.*)/ ) {
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$flag]
                      ->{ to_key_name( item => $1 ) } = $3;
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$flag]
                      ->{ to_key_name( item => $2 ) } = $4;
                }
                else {
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$flag]
                      ->{pv_name} = $pv_name;
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$flag]
                      ->{ to_key_name( item => $arr[0] ) } =
                      defined( $arr[1] ) ? $arr[1] : '-';
                }
            } elsif ( $head =~ /Volume\s+group/ && $lines[$i] !~ /--.*--/ ) {
                if ( $lines[$i] =~ /(.*)\s+\/\s+(.*\w+)\s+(\d+)\s+\/\s+(.*)/ ) {
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$loop]
                      ->{ to_key_name( item => $1 ) } = $3;
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$loop]
                      ->{ to_key_name( item => $1 . "_" . $2 ) } = $4;
                }
                else {
                    $result_array[$count]->{ to_key_name( item => $head ) }->[$loop]
                      ->{ to_key_name( item => $arr[0] ) } =
                      defined( $arr[1] ) ? $arr[1] : '-';
                }
            }
        }
    } else {
        my $vg_name;
        foreach my $line (@lines) {
            my @arr = split( /\s\s+/, $line );
            $arr[0] =~ s/\)//;
            $arr[0] =~ s/\(//;
            if ( $line =~ /VG\s+Name\s+(.*)/i ) {
                $vg_name = $1;
                $count++;
            } elsif ( $line =~ /(.*)\s+\/\s+(.*\w+)\s+(\d+)\s+\/\s+(.*)/ ) {
                $result_array[$count]->{ to_key_name( item => $1 ) } = $3;
                $result_array[$count]->{ to_key_name( item => $1 . "_" . $2 ) } =
                  $4;
            } elsif ( $line !~ /Finding.*|Partial.*/ && $line !~ /--.*--/ ) {
                $result_array[$count]->{vg_name} = $vg_name;
                $result_array[$count]->{ to_key_name( item => $arr[0] ) } =
                  defined( $arr[1] ) ? $arr[1] : '-';
            }
        }
    }
    return \@result_array;
} ## end sub vgdisplay

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Usage: vgdisplay.*|vgdisplay:\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid volume group name|Illegal path/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

