#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux vgremove command output
##
## @description
##    This is a library to parse the raw output of Linux vgremove command.
##
## @synopsis
##    use vgremove;
##
##    vgremove(%args);
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vgremove;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /invalid\s+option|Invalid argument/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~
/Run\s+`vgremove\s+--help'\s+for\s+more\s+information|Volume\s+group.*not\s+found/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::vgremove
##
## @summary Parses the raw output from the "vgremove" command
##
## @description
##    This library supports error checking for vgremove command, but output parsing is not supported as of now.
## @synopsis
##      vgremove(%args);
## INPUTS TO METHOD CALL 'vgremove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
##
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub vgremove {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub vgremove

1;
