#$Id:$
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux vsim command output
##
## @description
##    This is a library to parse the raw output of Linux vsim command.
##
## @synopsis
##    use vsim;
##
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vsim;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed";    # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";

    if ( $raw_output =~ /:\s(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message =~ /(Missing)/ ) {
            $error = "MissingRequiredParam";
        } elsif ( $$error_message
            =~ /([Ii]nvalid)|([Uu]nrecognized option|[Uu]nknown option)/ )
        {
            $error = "InvalidParam";
        } elsif ( $$error_message =~ /(argument to)|(unknown)/ ) {
            $error = "InvalidParamValue";
        }
    } ## end if ( $raw_output =~ /:\s(.*)/)
    return $error;
} ## end sub _check_error

## @name ::vsim_snap
##
## @summary Parses the raw output from the "/usr/local/test/bin/vsim snap" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vsim_snap with any of its options as described in the 
##    corresponding cdef file 'vsim.cdefs'
## @synopsis
##      vsim_snap(%args);
## INPUTS TO METHOD CALL 'vsim_snap' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsimid' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/local/test/bin/vsim snap 4 snapshot
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Creating snapshot for Simulator /u/sushantg/vsims/sushantg_vsim_4/DataONTAP.vmx
##&nbsp;
##&nbsp;Snapshot for virtual machine 'sushantg_vsim_4' successfully created 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsim_name' => 'sushantg_vsim_4',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vsim_snap {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } ## end if ($error_level)

    my @result_array;
    my %hash;
    if ( $output
        =~ /Snapshot for virtual machine '(\S+)' successfully created/i )
    {
        $hash{'vsim_name'} = $1;
        @result_array      = ( \%hash );
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text=>"Output is not in the expected format",
            response_object=>$resp);
    }
    return \@result_array;
} ## end sub vsim_snap

## @name ::vsim_snapback
##
## @summary Parses the raw output from the "/usr/local/test/bin/vsim snapback" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vsim_snapback with any of its options as described in the 
##    corresponding cdef file 'vsim.cdefs'
## @synopsis
##      vsim_snapback(%args);
## INPUTS TO METHOD CALL 'vsim_snapback' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsimid' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/local/test/bin/vsim snapback 4 snapshot
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Reverting to snapshot snapshot for Simulator /u/sushantg/vsims/sushantg_vsim_4/DataONTAP.vmx
##&nbsp;
##&nbsp;virtual machine 'sushantg_vsim_4' successfully reverted to snapshot snapshot
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'snapshot_name' => 'snapshot',
##&nbsp;            'vsim_name' => 'sushantg_vsim_4',
##&nbsp;            'result' => 'successful'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vsim_snapback {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    } ## end if ($error_level)

    my @result_array;
    my %hash;
    if ( $output
        =~ /virtual machine '(\S+)' successfully reverted to snapshot (\S+)/i )
    {
        $hash{'vsim_name'}     = $1;
        $hash{'snapshot_name'} = $2;
        $hash{'result'}        = 'successful';
        @result_array          = ( \%hash );
    } ## end if ( $output =~ ...)
    return \@result_array;
} ## end sub vsim_snapback

## @name ::vsim_snapdelete
##
## @summary Parses the raw output from the "/usr/local/test/bin/vsim snapdelete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vsim_snapdelete with any of its options as described in the 
##    corresponding cdef file 'vsim.cdefs'
## @synopsis
##      vsim_snapdelete(%args);
## INPUTS TO METHOD CALL 'vsim_snapdelete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsimid' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/local/test/bin/vsim snapdelete 4 snapshot
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Removing snapshot snapshot for Simulator /u/sushantg/vsims/sushantg_vsim_4/DataONTAP.vmx
##&nbsp;
##&nbsp;Snapshot snapshot of virtual machine 'sushantg_vsim_4' successfully removed 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'snapshot_name' => 'snapshot',
##&nbsp;            'vsim_name' => 'sushantg_vsim_4',
##&nbsp;            'result' => 'successful'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vsim_snapdelete {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    } ## end if ($error_level)

    my @result_array;
    my %hash;
    if ( $output
        =~ /Snapshot (\S+) of virtual machine '(\S+)' successfully removed/i )
    {
        $hash{'snapshot_name'} = $1;
        $hash{'vsim_name'}     = $2;
        $hash{'result'}        = 'successful';
        @result_array          = ( \%hash );
    } ## end if ( $output =~ ...)
    return \@result_array;
} ## end sub vsim_snapdelete

## @name ::vsim_snaplist
##
## @summary Parses the raw output from the "/usr/local/test/bin/vsim snaplist" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vsim_snaplist with any of its options as described in the 
##    corresponding cdef file 'vsim.cdefs'
## @synopsis
##      vsim_snaplist(%args);
## INPUTS TO METHOD CALL 'vsim_snaplist' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsimid' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /usr/local/test/bin/vsim snaplist 4
##&nbsp;-------- Raw Output ---------
##&nbsp; Getting snapshot list for Simulator /u/sushantg/vsims/sushantg_vsim_4/DataONTAP.vmx
##&nbsp;snapshot
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'snapshot' => [
##&nbsp;                            'snapshot'
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vsim_snaplist {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            error_message => \$error_message
        );
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
    } ## end if ($error_level)

    my @result_array;
    my %hash = ( 'snapshot' => [] );
    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp
    );

    # starting loop from 1 to getting rid of 'Getting snapshot list for Simulator' line
    for ( my $i = 1; $i <= $#lines; $i++ ) {
        chomp( $lines[$i] );
        push @{ $hash{'snapshot'} }, $lines[$i];
    }
    @result_array = ( \%hash );
    return \@result_array;
} ## end sub vsim_snaplist

1;
