#$Id$
#
#Copyright (c) 2011-12 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse attrib command on Windows 7
##
## @description
##    This is a library to parse the raw output of command attrib on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::attrib
##  
##  attrib(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::attrib;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    if ( $raw_output =~
/(only valid with)|(Invalid switch)|(File not found)|Path\s+not\s+found|Not\s+resetting\s+system\s+file|Parameter\s+format\s+not\s+correct/i){
        $error         = "InvalidParam";
        $$error_message = $raw_output;
    }

    return $error;
}

## @name ::attrib
##
## @summary Parses the raw output from the "attrib" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from attrib with any of its options as described in the 
##    corresponding cdef file 'attrib.cdefs'
## @synopsis
##      attrib(%args);
## INPUTS TO METHOD CALL 'attrib' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '+A' </td> <td> 'set-attrib-archive' </td> </tr> 
## <tr> <td> '/S' </td> <td> 'files-only-recursive' </td> </tr> 
## <tr> <td> '+I' </td> <td> 'set-attrib-not-content-indexed' </td> </tr> 
## <tr> <td> '+H' </td> <td> 'set-attrib-hidden' </td> </tr> 
## <tr> <td> '/L' </td> <td> 'use-simlink' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'clear-attrib-read' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'clear-attrib-not-content-indexed' </td> </tr> 
## <tr> <td> '-H' </td> <td> 'clear-attrib-hidden' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'clear-attrib-archive' </td> </tr> 
## <tr> <td> '/D' </td> <td> 'files-and-folders' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'clear-attrib-system' </td> </tr> 
## <tr> <td> '+R' </td> <td> 'set-attrib-read' </td> </tr> 
## <tr> <td> '+S' </td> <td> 'set-attrib-system' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = attrib C:\Users\Administrator\Music\ab\* /S
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;A       I    C:\Users\Administrator\Music\ab\a
##&nbsp;A       I    C:\Users\Administrator\Music\ab\ab.txt
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab\\a',
##&nbsp;            'attribs' => [
##&nbsp;                           'A',
##&nbsp;                           'I'
##&nbsp;                         ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab\\ab.txt',
##&nbsp;            'attribs' => [
##&nbsp;                           'A',
##&nbsp;                           'I'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub attrib {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(raw_output => $output,command => "attrib", error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $output, response_object => $resp);
    }

    my @lines = split(/\n/, $output);

    #my (%hash, $line);
    # Get rid of newlines
    @lines = grep(! /^\s+$/i, @lines);

    # There is nothing to parse
    return [] if !@lines;

    foreach my $line (@lines) {
        chomp($line);
        $line =~ s/^\s+//;
        $line =~ s/\s+$//;
        if($line =~ /(.*)\s*(\w+\:.*)/ ){
             my $attribs = $1;
             my $path = $2;
             my @tmp = split( /\s+/, $attribs );
             my %file_attribs;
             $file_attribs{'file_path'} = $path;
             push( @{$file_attribs{'attribs'} }, @tmp );
             push (@result_array,                 \%file_attribs );
        }
    }
    return wantarray ? @result_array : \@result_array;
}

1;
