#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Windows dir command output
##
## @description
##    This is a library to parse the raw output of dir command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::domain;
##
##    domain(%args);
##
## @author ssanjeev@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::domain;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ($raw_output =~ /(The\s+syntax\s+of\s+this\s+command\s+is)|(.*has incorrect format)/i) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($raw_output =~ /not\s+recognized\s+as\s+an\s+internal\s+or\s+external\s+command/i) {
        $error         = "CommandFailed";
        $error_message = $raw_output;  
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name :: net_user_delete
##
## @summary Parses the raw output from the "net user add" command
##
## @description
## This method can be used for parsing the raw output obtained after calling
## the method from net user add  with any of its options as described in the
## corresponding cdef file 'domain.cdefs'
## @synopsis
##      domain(%args);
## INPUTS TO METHOD CALL 'net_user_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net user username /delete /domain; result -l
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = []
##&nbsp;

sub net_user_delete {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error("raw_output" => $raw_output, response_object => $resp);
    return [];
} ## end sub net_user_delete

## @name :: net_user_add
##
## @summary Parses the raw output from the "net user add" command
##
## @description
## This method can be used for parsing the raw output obtained after calling
## the method from net user add  with any of its options as described in the
## corresponding cdef file 'domain.cdefs'
## @synopsis
##      domain(%args);
## INPUTS TO METHOD CALL 'net_user_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net user username password /add /domain; result -l
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = []
##&nbsp;

sub net_user_add {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error("raw_output" => $raw_output, response_object => $resp);
    return [];
} ## end sub net_user_add

## @name :: dsadd_group
##
## @summary Parses the raw output from the "dsadd group" command
##
## @description
## This method can be used for parsing the raw output obtained after calling
## the method from net user add  with any of its options as described in the
## corresponding cdef file 'domain.cdefs'
## @synopsis
##      domain(%args);
## INPUTS TO METHOD CALL 'dsadd group' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = dsadd group groupdn
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = []
##&nbsp;

sub dsadd_group {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error("raw_output" => $raw_output, response_object => $resp);
    return [];
} ## end sub dsadd_group

## @name :: net_group_add
##
## @summary Parses the raw output from the "dsadd group" command
##
## @description
## This method can be used for parsing the raw output obtained after calling
## the method from net user add  with any of its options as described in the
## corresponding cdef file 'domain.cdefs'
## @synopsis
##      domain(%args);
## INPUTS TO METHOD CALL 'net_group_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net group groupname groupuser /add
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = []
##&nbsp;

sub net_group_add {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error("raw_output" => $raw_output, response_object => $resp);
    return [];
} ## end sub net_group_add

## @name :: net_group_delete
##
## @summary Parses the raw output from the "net group delete" command
##
## @description
## This method can be used for parsing the raw output obtained after calling
## the method from net user add  with any of its options as described in the
## corresponding cdef file 'domain.cdefs'
## @synopsis
##      domain(%args);
## INPUTS TO METHOD CALL 'net_group_delet' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net group groupname netgrp_user /delete
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = []
##&nbsp;

sub net_group_delete {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error("raw_output" => $raw_output, response_object => $resp);
    return [];
} ## end sub net_group_delete

1;

