# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/eventcreate.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Windows eventcreate command output
##
## @description
##    This library supports error checking for eventcreate command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::eventcreate;
##
##    eventcreate(%args);
##
## @author  sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::eventcreate;

use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::eventcreate
##
## @summary Parses the raw output from the "eventcreate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from eventcreate with any of its options as described in the
##    corresponding cdef file 'eventcreate.cdefs'
## @synopsis
##      eventcreate(%args);
## INPUTS TO METHOD CALL 'eventcreate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '/SO' </td> <td> 'source' </td> <td> '0' </td> </tr>
## <tr> <td> '/L' </td> <td> 'logname' </td> <td> '0' </td> </tr>
## <tr> <td> '/P' </td> <td> 'password' </td> <td> '0' </td> </tr>
## <tr> <td> '/U' </td> <td> 'domain-or-user' </td> <td> '0' </td> </tr>
## <tr> <td> '/S' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '/?' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '/ID' </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> '/T' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '/D' </td> <td> 'description' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported.
## @examples
##&nbsp;Command = eventcreate /T ERROR /ID 1000 /L APPLICATION /D "My custom error event for the application log"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;SUCCESS: An event of type 'ERROR' was created in the 'APPLICATION' log with 'EventCreate' as the source.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = eventcreate /T ERROR /ID 999 /L APPLICATION /SO WinWord /D "Winword event 999 happened due to low diskspace"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;SUCCESS: An event of type 'ERROR' was created in the 'APPLICATION' log with 'WinWord' as the source.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = eventcreate /S CSIQA-AD1 /U administrator /P netapp /T ERROR /ID 999 /L APPLICATION /SO WinWord /D "Winword event 999 happened due to low diskspace"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;SUCCESS: An event of type 'ERROR' was created in the 'APPLICATION' log with 'WinWord' as the source.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub eventcreate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub eventcreate

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /ERROR: Access.*|ERROR: No network.*|ERROR: Invalid syntax.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
