#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserve
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse format command for windows
##
## @description
##    This is a library to parse the raw output of windows format command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "format" will be format()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::format;
##
##    format(%args);
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::format;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_message;
    my $error_level = $resp->get_execution_status();
    my $error = undef;

    if ( $raw_output =~ /(An incorrect volume label was entered for this drive.*|Required parameter missing.*)/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    return $error;
} ## end sub _check_error

## @name ::format
##
## @summary Parses the raw output from the "format" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from format with any of its options as described in the
##    corresponding cdef file 'format.cdefs'
## @synopsis
##      format(%args);
## INPUTS TO METHOD CALL 'format' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> /V </td> <td> 'label' </td> <td> '0' </td> </tr>
## <tr> <td> /FS </td> <td> 'file-system' </td> <td> '0' </td> </tr>
## <tr> <td> /Q </td> <td> 'quick-format' </td> <td> '0' </td> </tr>
## <tr> <td> /A </td> <td> 'override-size' </td> <td> '0' </td> </tr>
## <tr> <td> /C </td> <td> 'compresssed' </td> <td> '0' </td> </tr>
## <tr> <td> /X </td> <td> 'dismount' </td> <td> '0' </td> </tr>
## <tr> <td> /P </td> <td> 'passes' </td> <td> '0' </td> </tr>
## <tr> <td> /D </td> <td> 'duplicate' </td> <td> '0' </td> </tr>
## <tr> <td> /R </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> /F </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> /T </td> <td> 'tracks' </td> <td> '0' </td> </tr>
## <tr> <td> /N </td> <td> 'sectors' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = format D: /FS:FAT32 /V:new_label /Q /A:512
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; The type of the file system is NTFS.
##&nbsp; The new file system is FAT32.
##&nbsp; Enter current volume label for drive D:
##&nbsp; New_Volume
##&nbsp; WARNING, ALL DATA ON NON-REMOVABLE DISK
##&nbsp; DRIVE D: WILL BE LOST!
##&nbsp; Proceed with Format (Y/N)?
##&nbsp; Y
##&nbsp; QuickFormatting 53M
##&nbsp; Initializing the File Allocation Table (FAT)...
##&nbsp; Format complete.
##&nbsp;
##&nbsp;    51,380,224 bytes total disk space.
##&nbsp;    51,379,712 bytes available on disk.
##&nbsp;
##&nbsp;           512 bytes in each allocation unit.
##&nbsp;       100,351 allocation units available on disk.
##&nbsp;
##&nbsp;            32 bits in each FAT entry.
##&nbsp;
##&nbsp; Volume Serial Number is DE4A-D185
##&nbsp;
##&nbsp; C:\\Windows\\system32>
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub format {
    my %args = @_;
    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

1;
