#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse icacls command on Windows 7
##
## @description
##    This is a library to parse the raw output of command icacls on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::icacls
##  
##  icacls(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::icacls;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Invalid parameter)|(Invalid arguments)/i) {
            $error = "InvalidParam";
        } elsif ($$error_message =~ /cannot find the file specified/i) {
            $error = "MissingRequiredParam";
        }
    }
    return $error;
}

## @name ::icacls
##
## @summary Parses the raw output from the "icacls" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from icacls with any of its options as described in the 
##    corresponding cdef file 'icacls.cdefs'
## @synopsis
##      icacls(%args);
## INPUTS TO METHOD CALL 'icacls' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'sid' </td> </tr> 
## <tr> <td> '/remove' </td> <td> 'remove' </td> </tr> 
## <tr> <td> '/setowner' </td> <td> 'setowner' </td> </tr> 
## <tr> <td> '/substitute' </td> <td> 'substitute' </td> </tr> 
## <tr> <td> '/restore' </td> <td> 'restore' </td> </tr> 
## <tr> <td> '/C' </td> <td> 'continue-on-err' </td> </tr> 
## <tr> <td> '/inheritance' </td> <td> 'inheritance' </td> </tr> 
## <tr> <td> '/save' </td> <td> 'save' </td> </tr> 
## <tr> <td> '/reset' </td> <td> 'reset' </td> </tr> 
## <tr> <td> '/verify' </td> <td> 'verify' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'user' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'policy' </td> </tr> 
## <tr> <td> '/Q' </td> <td> 'quite' </td> </tr> 
## <tr> <td> '/findsid' </td> <td> 'findsid' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'old-sid' </td> </tr> 
## <tr> <td> '/L' </td> <td> 'use-simlink' </td> </tr> 
## <tr> <td> '/deny' </td> <td> 'deny' </td> </tr> 
## <tr> <td> '/setintegritylevel' </td> <td> 'setintegritylevel' </td> </tr> 
## <tr> <td> '/T' </td> <td> 'recursive' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'sid-perm' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'level' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'aclfile' </td> </tr> 
## <tr> <td> '/grant' </td> <td> 'grant' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = icacls C:\Users\Administrator\Music\ab /T
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;C:\Users\Administrator\Music\ab Everyone:(F)
##&nbsp;                                NT AUTHORITY\SYSTEM:(I)(OI)(CI)(F)
##&nbsp;                                BUILTIN\Administrators:(I)(OI)(CI)(F)
##&nbsp;                                VMWIN64-75\Administrator:(I)(OI)(CI)(F)
##&nbsp;
##&nbsp;C:\Users\Administrator\Music\ab\a NT AUTHORITY\SYSTEM:(I)(F)
##&nbsp;                                  BUILTIN\Administrators:(I)(F)
##&nbsp;                                  VMWIN64-75\Administrator:(I)(F)
##&nbsp;
##&nbsp;Successfully processed 2 files; Failed processing 0 files
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab',
##&nbsp;            'permissions' => [
##&nbsp;                               {
##&nbsp;                                 'permission' => '(F)',
##&nbsp;                                 'user_group' => 'Everyone'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(OI)(CI)(F)',
##&nbsp;                                 'user_group' => 'NT AUTHORITY\\SYSTEM'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(OI)(CI)(F)',
##&nbsp;                                 'user_group' => 'BUILTIN\\Administrators'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(OI)(CI)(F)',
##&nbsp;                                 'user_group' => 'VMWIN64-75\\Administrator'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab\\a',
##&nbsp;            'permissions' => [
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(F)',
##&nbsp;                                 'user_group' => 'NT AUTHORITY\\SYSTEM'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(F)',
##&nbsp;                                 'user_group' => 'BUILTIN\\Administrators'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(I)(F)',
##&nbsp;                                 'user_group' => 'VMWIN64-75\\Administrator'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = icacls C:\Users\Administrator\Music\* /findsid *S-1-1-0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;SID Found: C:\Users\Administrator\Music\ab.
##&nbsp;SID Found: C:\Users\Administrator\Music\abcd.txt.
##&nbsp;SID Found: C:\Users\Administrator\Music\c.
##&nbsp;SID Found: C:\Users\Administrator\Music\d.
##&nbsp;SID Found: C:\Users\Administrator\Music\desktop.ini.
##&nbsp;SID Found: C:\Users\Administrator\Music\e.
##&nbsp;Successfully processed 6 files; Failed processing 0 files
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_list' => [
##&nbsp;                             'C:\\Users\\Administrator\\Music\\ab.',
##&nbsp;                             'C:\\Users\\Administrator\\Music\\abcd.txt.',
##&nbsp;                             'C:\\Users\\Administrator\\Music\\c.',
##&nbsp;                             'C:\\Users\\Administrator\\Music\\d.',
##&nbsp;                             'C:\\Users\\Administrator\\Music\\desktop.ini.',
##&nbsp;                             'C:\\Users\\Administrator\\Music\\e.'
##&nbsp;                           ],
##&nbsp;            'no_of_files_found' => 6
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub icacls {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    if($error_level) {
        $error = _check_error(raw_output=>$output,command=>"icacls",
                              error_message=>\$error_message);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    my @lines = split(/\n/, $output);

    my (%hash, $line);
    @lines = grep(! /^processed\s+/i, @lines);
    ## output of this command will always have following line as last line
    #---Successfully processed <count> files; Failed processing <count> files-------#
    ## Skipping it
    @lines = grep(! /^Successfully\s+processed\s+/i, @lines);

    ## Getting rid of line
    #-----__status__=!errorlevel----------------#
    shift(@lines) if ($lines[0] =~ /__status__=/i);

    my @without_new_lines = grep(! /^$/i, @lines);
    # There is nothing to parse
    return [] if !@without_new_lines;

    my $no_of_files_with_sid;
    if ($command =~ /\/findsid/) {
        my (@file_list);
        foreach $line (@lines) {
            chomp($line);
            if ($line =~ /No files with a matching SID was found/i){
                last;
            } else {
                $no_of_files_with_sid++;
                $line =~ /SID\s+Found:\s+(\S+)/i;
                push(@file_list, $1);
            }
        }
        $hash{'no_of_files_found'} = $no_of_files_with_sid;
        $hash{'file_list'} = \@file_list;
        push (@result_array, \%hash);
    } else {
        while(@lines) {
            $line = shift(@lines);
            chomp($line);
            my ($file_path, @permissions);
            while($line !~ /^$/){
                if ($line =~ /^(\S+)\s+(.+)$/) {
                    $file_path = $1;
                    $line = $2;
                }

                $line =~ s/^\s+//;
                $line =~ s/\s+$//;
                my %permission;
                my @tmp = split(/:/, $line);
                $permission{'user_group'} = $tmp[0];
                $permission{'permission'} = $tmp[1];
                push(@permissions, \%permission);

                $line = shift(@lines);
                chomp($line);
            }
            my %icacl;
            $icacl{'file_path'} = $file_path;
            $icacl{'permissions'} = \@permissions;
            push (@result_array, \%icacl);
        }
    }

    return wantarray ? @result_array : \@result_array;
}

1;
