#$Id: //depot/prod/DOT/dev/test/lib/NACL/UnitTest/APISet/Host/CLI/api_windows.thpl#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @name ::ipconfig
##
## @summary Routines to parse the output of the Windows ipconfig command output
##
## @description
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipconfig with any of its options as described in the 
##    corresponding cdef file 'ipconfig.cdefs'
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::ipconfig;
##
##    ipconfig(%args);
##
## @author leahk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::ipconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException;

# Required to determine if an IP is IPv6 or not
use Net::IP qw(ip_is_ipv4 ip_is_ipv6);

## @name ::ipconfig
##
## @summary Parses the raw output from the "ipconfig" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipconfig with any of its options as described in the 
##    corresponding cdef file 'ipconfig.cdefs'
## @synopsis
##      ipconfig(%args);
## INPUTS TO METHOD CALL 'ipconfig' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '/flushdns' </td> <td> 'flushdns' </td> </tr> 
## <tr> <td> '/all' </td> <td> 'all' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ipconfig
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;Windows IP Configuration
##&nbsp;
##&nbsp;
##&nbsp;Ethernet adapter Local Area Connection:
##&nbsp;
##&nbsp;   Connection-specific DNS Suffix  . : 
##&nbsp;   Link-local IPv6 Address . . . . . : fe80::519d:de82:5b55:f89b%11
##&nbsp;   IPv4 Address. . . . . . . . . . . : 172.17.137.197
##&nbsp;   Subnet Mask . . . . . . . . . . . : 255.255.252.0
##&nbsp;   Default Gateway . . . . . . . . . : 172.17.136.1
##&nbsp;
##&nbsp;Tunnel adapter isatap.{8218E4B1-0EAF-4E2D-8867-12053A68E4E5}:
##&nbsp;
##&nbsp;   Media State . . . . . . . . . . . : Media disconnected
##&nbsp;   Connection-specific DNS Suffix  . : 
##&nbsp;
##&nbsp;Tunnel adapter Teredo Tunneling Pseudo-Interface:
##&nbsp;
##&nbsp;   Media State . . . . . . . . . . . : Media disconnected
##&nbsp;   Connection-specific DNS Suffix  . : 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'dns_servers' => '',
##&nbsp;            'gateway' => '',
##&nbsp;            'netbios' => '',
##&nbsp;            'primary_wins' => '',
##&nbsp;            'state' => 'Media disconnected',
##&nbsp;            'adapter' => 'Teredo Tunneling Pseudo-Interface',
##&nbsp;            'dns_suffix' => '',
##&nbsp;            'mask' => '',
##&nbsp;            'secondary_wins' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'dns_servers' => '',
##&nbsp;            'gateway' => '',
##&nbsp;            'netbios' => '',
##&nbsp;            'primary_wins' => '',
##&nbsp;            'state' => 'Media disconnected',
##&nbsp;            'adapter' => 'isatap.{8218E4B1-0EAF-4E2D-8867-12053A68E4E5}',
##&nbsp;            'dns_suffix' => '',
##&nbsp;            'mask' => '',
##&nbsp;            'secondary_wins' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'dns_servers' => '',
##&nbsp;            'ipv4' => [
##&nbsp;                        '172.17.137.197'
##&nbsp;                      ],
##&nbsp;            'gateway' => '172.17.136.1',
##&nbsp;            'netbios' => '',
##&nbsp;            'primary_wins' => '',
##&nbsp;            'state' => '',
##&nbsp;            'adapter' => 'Local Area Connection',
##&nbsp;            'dns_suffix' => '',
##&nbsp;            'link_local_ipv6' => 'fe80::519d:de82:5b55:f89b%11',
##&nbsp;            'mask' => '255.255.252.0',
##&nbsp;            'secondary_wins' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ipconfig {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    if ($error_level) {
        # XP has a typo
        if ($output =~ /Error: unrecon?gnized or incomplete command line/) {
            NACL::APISet::Exceptions::InvalidParamValueException->throw(text=>$output,
                                                       response_object => $resp);
        }
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                               response_object => $resp);
    }

    my @lines = get_lines(raw_output => $output);

    # much of this parsing code is copied from CIFS::Client->get_ip_config
    my @return_array;
    my %ipInfo = ();
    my $interface;
    while ( my $line = shift(@lines) ) {
        if  ($line =~ /\s+adapter\s+(.+):/) {
            $interface                            = $1;
            $ipInfo{$interface}{'adapter'}      = $interface;
            $ipInfo{$interface}{'state'}          = '';
            $ipInfo{$interface}{'mask'}           = '';
            $ipInfo{$interface}{'gateway'}        = '';
            $ipInfo{$interface}{'dns_suffix'}     = '';
            $ipInfo{$interface}{'secondary_wins'} = '';
            $ipInfo{$interface}{'primary_wins'}   = '';
            $ipInfo{$interface}{'dns_servers'}    = '';
            $ipInfo{$interface}{'netbios'}        = '';
        }

        if ($interface) {
            if ($line =~ /Media State.+:\s+(Media disconnected)/) {
                $ipInfo{$interface}{'state'} = $1;
            } elsif ($line =~ /Physical Address.+:\s+(.*)$/) {
                my $mac = $1;
                $mac =~ s/-/:/g;
                $ipInfo{$interface}{'mac'} = $mac;
            } elsif ($line =~ /Address.+:\s+(.*)$/) {

                # Vista/2k8
                # Link-local IPv6 Address . . . . . : fe80::3095:394d:d251:6ca1%12(Preferred)
                # IPv4 Address. . . . . . . . . . . : 10.60.76.96(Preferred)
                # IPv6 Address. . . . . . . . . . . : fd20:81be:b255:8100:0:10:0:2(Preferred)
                # Link-local IPv6 Address . . . . . : fe80::3181:f361:3d14:85bd%11(Preferred)
                # IPv4 Address. . . . . . . . . . . : 172.16.13.252(Preferred)
                # 2003
                # IP Address. . . . . . . . . . . . : 10.60.92.59
                # IP Address. . . . . . . . . . . . : fe80::214:5eff:fe3c:b8db%4
                # IP Address. . . . . . . . . . . . : 172.16.14.194
                # IP Address. . . . . . . . . . . . : fd20:81be:b255:8100:0:10:0:29
                # IP Address. . . . . . . . . . . . : fe80::214:5eff:fe3c:b8da%5
                # IP Address. . . . . . . . . . . . : fe80::ffff:ffff:fffd%6
                # IP Address. . . . . . . . . . . . : fe80::5efe:172.16.14.194%2
                # IP Address. . . . . . . . . . . . : fe80::5efe:10.60.92.59%2
                # XP
                # IP Address. . . . . . . . . . . . : 10.60.89.160
                # IP Address. . . . . . . . . . . . : fe80::214:5eff:fe3c:9915%4
                # IP Address. . . . . . . . . . . . : 172.16.12.131
                # IP Address. . . . . . . . . . . . : fd20:81be:b255:8100:0:10:0:34
                # IP Address. . . . . . . . . . . . : fe80::214:5eff:fe3c:9914%5
                # IP Address. . . . . . . . . . . . : fe80::ffff:ffff:fffd%6
                # IP Address. . . . . . . . . . . . : fe80::5efe:172.16.12.131%2
                # IP Address. . . . . . . . . . . . : fe80::5efe:10.60.89.160%2
                #

                my $addr = $1;
                my $is_link_local;
                my $actual_addr = $addr;
                if ($actual_addr =~ /%/) {
                    $actual_addr =~ s/%\d+$//;
                    $actual_addr =~ s/%\d+\(Preferred\)//;
                    $is_link_local = 1;
                } elsif ($actual_addr =~ /\(Preferred\)$/) {
                    $actual_addr =~ s/\(Preferred\)//;
                } elsif ($actual_addr =~ /\(Deprecated\)$/) {
                    $actual_addr =~ s/\(Deprecated\)// ;
                } elsif ($actual_addr =~ /\(Duplicate\)$/) {
		    $actual_addr =~ s/\(Duplicate\)// ;
		}

                # We perform both checks because we want to ensure
                # that what we have got is really an IP address
                my $is_ipv4 = ip_is_ipv4($actual_addr);
                my $is_ipv6 = ip_is_ipv6($actual_addr);

                if (!$is_ipv4 && !$is_ipv6) {
                    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                        text => "Address $addr is neither IPv4 nor IPv6. "
                            . 'This is probably because the IP address '
                            . 'has been wrongly extracted from this line '
                            . "of raw output:\n$line",
                        response_object => $resp
                    );
                }

                if ($is_ipv4) {
                    push(@{$ipInfo{$interface}{'ipv4'}}, $actual_addr);
                } elsif ($is_ipv6 && !$is_link_local) {
                    push(@{$ipInfo{$interface}{'ipv6'}}, $actual_addr);
                } elsif ($is_ipv6 && $is_link_local) {
                     $actual_addr=$addr;
                     $actual_addr=~s/\(Preferred\)$//;
                    $ipInfo{$interface}{'link_local_ipv6'} = $actual_addr;
                } else {
                    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                        text => "The line:\n$line\ncould not be parsed. "
                        . "The IP address $addr was not IPv4; IPv6; or "
                        . 'a link-local IPv6 address',
                        response_object => $resp
                    );
                }
            } elsif ($line =~ /Subnet Mask.+:\s+(.*)$/) {
                $ipInfo{$interface}{'mask'} = $1;
            } elsif ($line =~ /Default Gateway.+:\s+(.*)$/) {
                $ipInfo{$interface}{'gateway'} = $1;
            } elsif ($line =~ /DNS Servers.+:\s+(.*)$/) {
                $ipInfo{$interface}{'dns_servers'} = $1;
            } elsif ($line =~ /Primary\s+WINS\s+Server.+:\s+(.*)$/) {
                $ipInfo{$interface}{'primary_wins'} = $1;
            } elsif ($line =~ /Secondary\s+WINS\s+Server.+:\s+(.*)$/) {
                $ipInfo{$interface}{'secondary_wins'} = $1;
            } elsif ($line =~ /Connection-specific\s+DNS\s+Suffix.+:\s+(.*)$/) {
                $ipInfo{$interface}{'dns_suffix'} = $1;
            } elsif ($line =~ /NetBIOS\s+over\s+Tcpip.+:\s+(.*)$/) {
                $ipInfo{$interface}{'netbios'} = $1;
            }
        }
    }
    
    # seperate into records
    foreach my $if (keys(%ipInfo)) { push(@return_array, $ipInfo{$if}) }
    return \@return_array;
}

1;
