#
# Copyright (c) 2013, 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse mpc command for windows
##
## @description
##    This is a library to parse the raw output of windows mpc command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mpc show luns" will be mpc_show_luns()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::mpc;
##
##    mpc(%args);
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::mpc;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_message;
    my $error_level = $resp->get_execution_status();
    my $error = undef;
    if ( $raw_output =~ /(MESSAGE: Generic failure.*|Failed:.*)/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /is not recognized.*/i ) {
        $error         = "CommandNotFound";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /(SYNTAX ERROR:.*|The system cannot find the drive specified)/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    return $error;
} ## end sub _check_error

## @name ::mpc_show_luns
##
## @summary Parses the raw output from the "mpc show luns" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_show_luns with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_show_luns(%args);
## INPUTS TO METHOD CALL 'mpc_show_luns' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc show luns
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Disk	LUN	DSM	Policy	Paths	Serial		CFMode	Filer		Proto	LunType		Path
##&nbsp;----	---	-----	------	------	-----		------	-----		-----	-------		----
##&nbsp;Disk1	0	None	UNK	1	C02Bs?CVufB4	None	vs0	iSCSI	windows	/vol/component_flex_vol/lun_iscsi6673
##&nbsp;
##&nbsp;Success: show luns
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'iSCSI',
##&nbsp;            'disk' => 'Disk1',
##&nbsp;            'lun_pathname' => '/vol/component_flex_vol/lun_iscsi6673',
##&nbsp;            'serial' => 'C02Bs?CVufB4',
##&nbsp;            'cfmode' => 'None',
##&nbsp;            'lun_id' => '0',
##&nbsp;            'paths' => '1',
##&nbsp;            'controller' => 'vs0',
##&nbsp;            'luntype' => 'windows',
##&nbsp;            'policy' => 'UNK',
##&nbsp;            'dsm' => 'None'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mpc_show_luns {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my @result_array;
    my $limit = 0;

    my $error = _check_error( "raw_output" => $output,
                               response_object => $resp );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output => $output, response_obj => $resp);
    my @head = qw( disk lun_id dsm policy paths serial cfmode controller
                   protocol luntype lun_pathname  );

    for ( my $i = 2; $i <= $#lines; $i++ ) {
        if( $lines[$i] !~ /^Success:/) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j=0; $j <= $#head; $j++) {
                $result_array[$limit]->{to_key_name(item =>$head[$j])}
                    = $values[$j];
            }
        }
        $limit++;
    }
    return \@result_array;
}

## @name ::mpc_iscsi_show_adapters
##
## @summary Parses the raw output from the "mpc iscsi show adapters" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mpc_iscsi_show_adapters with any of its options as described in the 
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_show_adapters(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_show_adapters' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi show adapters
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;iSCSINodeName: iqn.1991-05.com.microsoft:ncat-lw-1
##&nbsp;
##&nbsp;AdapterName	Index	Tag	Type	Address				Full Address
##&nbsp;-----------	-----	---	----	-------				------------
##&nbsp;MS-iSCSI-0	0	0	IPv4	10.225.152.60	
##&nbsp;
##&nbsp;Success: iscsi show adapters
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'adapters' => [
##&nbsp;                            {
##&nbsp;                              'full_address' => undef,
##&nbsp;                              'index' => '0',
##&nbsp;                              'address' => '10.225.152.60',
##&nbsp;                              'type' => 'IPv4',
##&nbsp;                              'tag' => '0',
##&nbsp;                              'adaptername' => 'MS-iSCSI-0'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'initiator_name' => 'iqn.1991-05.com.microsoft:ncat-lw-1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mpc_iscsi_show_adapters {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    my $error = _check_error( "raw_output" => $output, response_object => $resp );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output => $output, response_obj => $resp);

    @lines = grep (! /^Success:/, @lines);
    my (@result_array,@head);
    my $cnt=0;
    my $new_cnt = 0;
    my $i = 0;
    for($i=0;$i <= $#lines;$i++) {
        if($lines[$i] =~ /iSCSINodeName:\s+(.*)/i) {
            $result_array[0]->{'initiator_name'} = $1; $i=$i+1;
            if( $lines[$i] =~
                /(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(Full.*)/ ) {
                @head = ($1,$2,$3,$4,$5,$6);
            }
            $new_cnt = $i+2;
        }
    }

    for( $i = $new_cnt; $i <= $#lines; $i++ ) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j=0; $j <= $#head; $j++) {
                $result_array[0]->{adapters}->[$cnt]
                    ->{to_key_name(item => $head[$j])} = $values[$j];
            }
            $cnt++;
    }

    return \@result_array;
}
## @name ::mpc_iscsi_show_targets
##
## @summary Parses the raw output from the "mpc iscsi show targets" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_iscsi_show_targets with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_show_targets(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_show_targets' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi show targets
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;TargetName					Index	IP Addr				Port	Auth	DDigest	HDigest	MaxConn	Type
##&nbsp;----------					-----	-------				----	----	-------	-------	-------	----
##&nbsp;iqn.1992-08.com.netapp:sn.9b2a2560957311e29d1d123478563412:vs.11	0:0	10.238.3.168	3260	None	None	None	0	SendTargets
##&nbsp;iqn.1992-08.com.netapp:sn.6bf1ca218fed11e29d1b123478563412:vs.2	0:0	10.238.3.167	3260	None	None	None	0	SendTargets
##&nbsp;
##&nbsp;Success: iscsi show targets
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ddigest' => 'None',
##&nbsp;            'auth' => 'None',
##&nbsp;            'targetname' => 'iqn.1992-08.com.netapp:sn.9b2a2560957311e29d1d123478563412:vs.11',
##&nbsp;            'ip_addr' => '10.238.3.168',
##&nbsp;            'index' => '0:0',
##&nbsp;            'maxconn' => '0',
##&nbsp;            'hdigest' => 'None',
##&nbsp;            'type' => 'SendTargets',
##&nbsp;            'port' => '3260'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ddigest' => 'None',
##&nbsp;            'auth' => 'None',
##&nbsp;            'targetname' => 'iqn.1992-08.com.netapp:sn.6bf1ca218fed11e29d1b123478563412:vs.2',
##&nbsp;            'ip_addr' => '10.238.3.167',
##&nbsp;            'index' => '0:0',
##&nbsp;            'maxconn' => '0',
##&nbsp;            'hdigest' => 'None',
##&nbsp;            'type' => 'SendTargets',
##&nbsp;            'port' => '3260'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mpc_iscsi_show_targets {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my $error = _check_error( "raw_output"    => $raw_output,
                           response_object => $resp );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my (@result_array,@head);
    my @lines = get_lines(raw_output => $raw_output, response_obj => $resp);
    @lines = grep (! /^Success:/, @lines);
    my $cnt=0;
    my $i;
    if( $lines[0] =~
        /(\w+)\s+(\w+)\s+(IP\s\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)/
        ) {
            @head = ($1,$2,$3,$4,$5,$6,$7,$8,$9);
    }

    for($i=2;$i <= $#lines;$i++) {
            my @values = split( /\s+/, $lines[$i] );
            for ( my $j=0; $j <= $#head; $j++) {
                $result_array[$cnt]->{to_key_name(item => $head[$j])}
                    = $values[$j];
            }
            $cnt++;
    }

    return \@result_array;
}

## @name ::mpc_iscsi_login
##
## @summary Parses the raw output from the "mpc iscsi login" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_iscsi_login with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_login(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_login' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'target_index' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'target_name' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'initiator_index' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi login iqn.1992-08.com.netapp:sn.6bf1ca218fed11e29d1b123478563412:vs.2 0:0 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;SessionId: ffffffff874ac8d4-4000013700000048
##&nbsp;ConnectionId: ffffffff874ac8d4-00000000000000a0
##&nbsp;
##&nbsp;Success: iscsi login target
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'connectionid' => 'ffffffff874ac8d4-00000000000000a0',
##&nbsp;            'sessionid' => 'ffffffff874ac8d4-4000013700000048'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub mpc_iscsi_login {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        response_object     => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output => $raw_output, response_obj => $resp);
    foreach my $line (@lines) {
        if($line =~ /(.*):\s(.*)/ && $line !~ /Success:/) {
            $result_array[0]->{to_key_name(item =>$1)} = $2;
        }
    }
    return \@result_array;
} ## end sub mpc_iscsi_login

## @name ::mpc_iscsi_logout
##
## @summary Parses the raw output from the "mpc iscsi login" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_iscsi_logout with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_logout(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_logout' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'session_id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi logout ffffffff874ac8d4-400001370000006b
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Success: iscsi logout target
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub mpc_iscsi_logout {
    my %args = @_;

    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse anything because no output is generated
    return [];
}

## @name ::mpc_iscsi_remove_targetportal
##
## @summary Parses the raw output from the "mpc iscsi login" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_iscsi_remove_targetportal with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_remove_targetportal(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_remove_targetportal' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filername_or_ip' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi remove targetportal 10.238.3.167
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Success: iscsi remove targetportal
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub mpc_iscsi_remove_targetportal {
    my %args = @_;
    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse anything because no output is generated
    return [];
}

## @name ::mpc_iscsi_add_targetportal
##
## @summary Parses the raw output from the "mpc iscsi login" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_iscsi_add_targetportal with any of its options as described in the
##    corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_iscsi_add_targetportal(%args);
## INPUTS TO METHOD CALL 'mpc_iscsi_add_targetportal' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filername_or_ip' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator_index' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = C:\Users\Administrator\Desktop/mpc iscsi add targetportal 10.238.3.167 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Success: iscsi add targetportals
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub mpc_iscsi_add_targetportal {
    my %args = @_;

    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse anything because no output is generated
    return [];
}

sub mpc_show_disks {
    my %args = @_;

    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my (@result_array,@head);
    my @lines = get_lines(raw_output => $output, response_obj => $resp);

    @lines = grep (! /(^Success:)|(----)/, @lines);
    my $cnt=0;

    @head = split(/\s+/,$lines[0]);

    for my $i (1 .. $#lines) {
        my @values = split( /\s+/, $lines[$i] );
        $values[2] .= " $values[3]";
        splice(@values,3,1);
        for my $j (0 .. $#head) {
            $result_array[$cnt]->{to_key_name(item => $head[$j])}
                 = $values[$j];
        }
        $cnt++;
    }

    return \@result_array;
}

## @name ::mpc_show_driveletters
##
## @summary Parses the raw output from the "mpc show driveletters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method mpc_show_driveletters 
## @synopsis
##      mpc_show_driveletters(%args);
##
## INPUTS TO METHOD CALL 'mpc_fcp_show_adapters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = X:\SAN\common\bin\mpc show driveletters
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Used Drive Letters (6): A C D E F X
##&nbsp;Available Driver Letters (20): B G H I J K L M N O P Q R S T U V W Y Z
##&nbsp;
##&nbsp;Success: show driveletters
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'used' => 'A C D E F X',
##&nbsp;            'available' => 'B G H I J K L M N O P Q R S T U V W Y Z', 
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub mpc_show_driveletters { 
    my %args = @_;

    my ($error_message);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my @lines = get_lines(raw_output => $output, response_obj => $resp);

    my %drive_letters;
    for my $line (@lines) {
	# skip the first four fields to extract the available or used drive
	# letters
	if ($line =~ /Used.+:\s+(.+$)/) {
	    $drive_letters{'used'} = $1;
	}
	elsif ($line =~ /Available.+:\s+(.+$)/) {
	    $drive_letters{'available'} = $1;
	}
    }
    push(@result_array, \%drive_letters);
    return \@result_array;
}
## @name ::mpc_fcp_show_adapters
##
## @summary Parses the raw output from the "mpc fcp show adapters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mpc_fcp_show_targets with any of its options as described
##    in the corresponding cdef file 'mpc.cdefs'
## @synopsis
##      mpc_fcp_show_adapters(%args);
## INPUTS TO METHOD CALL 'mpc_fcp_show_adapters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = X:\SAN\common\bin\mpc fcp show adapters
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; AdapterName            State         Type    NodeName                   PortName
##&nbsp; -----------            -----         ----    --------                   --------
##&nbsp; com.emulex-LPe12002-0  Operational   Fabric  20:00:00:00:90:fa:1e:38:f4  10:00:00:00:90:fa:1e:38:f4
##&nbsp; com.emulex-LPe12002-1  Operational   Fabric  20:00:00:00:90:fa:1e:38:f5  10:00:00:00:90:fa:1e:38:f5
##&nbsp;Success: fcp show adapters
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'adaptername' => 'com.emulex-LPe12002-0',
##&nbsp;             'state' => 'operational',
##&nbsp;             'type' => 'fabric',
##&nbsp;             'nodename' => '20:00:00:00:90:fa:1e:38:f4',
##&nbsp;             'portname' => '10:00:00:00:90:fa:1e:38:f4',
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'adaptername' => 'com.emulex-LPe12002-1',
##&nbsp;             'state' => 'operational',
##&nbsp;             'type' => 'fabric',
##&nbsp;             'nodename' => '20:00:00:00:90:fa:1e:38:f5',
##&nbsp;             'portname' => '10:00:00:00:90:fa:1e:38:f5',
##&nbsp;           },
##&nbsp;         ];
##&nbsp;

sub mpc_fcp_show_adapters {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    my $error = _check_error( "raw_output" => $output, response_object => $resp );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output => $output, response_obj => $resp);
    my @head = split /\s+/, $lines[0];
    my @result_array;
    my $ix = 0;

    # remove Success line from output
    @lines = grep (! /^Success:/, @lines);

    # start on the third line of output to skip header
    for (my $i = 2; $i <= $#lines; $i++) {
	my @values = split( /\s+/, $lines[$i] );
	for (my $j = 0; $j <= $#head; $j++) {
            $result_array[$ix]->{to_key_name(item =>$head[$j])} = $values[$j];
            }
	$ix++; 
    }
    return \@result_array;

}


sub mpc_iscsi_show_connections {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    my $error = _check_error( "raw_output" => $output, response_object => $resp );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output => $output, response_obj => $resp);
    my @head = split /\s+/, $lines[0];
    my @result_array;
    my $ix = 0;

    # remove Success line from output
    @lines = grep (! /^Success:/, @lines);

    # start on the third line of output to skip header
    for (my $i = 2; $i <= $#lines; $i++) {
	my @values = split( /\s+/, $lines[$i] );
	for (my $j = 0; $j <= $#head; $j++) {
            $result_array[$ix]->{to_key_name(item =>$head[$j])} = $values[$j];
            }
	$ix++; 
    }
    return \@result_array;

}

1;
