#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @name ::net
##
## @summary Routines to parse the output of the Windows net command output
##
## @description
##    This method can be used for parsing the raw output obtained after calling 
##    the method from net with any of its options as described in the 
##    corresponding cdef file 'net.cdefs'
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::net;
##
##    net_use(%args);
##    net_start(%args);
##    net_stop(%args);
##    net_stats(%args);
##    net_time(%args);
##
## @author anerudh@netapp.com, leahk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::net;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

# Shouldn't log warnings for redefinition
# as this module is supposed to override these methods
no warnings 'redefine';

## @name ::net_use
##
## @summary Parses the raw output from the "net use" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from net_use with any of its options as described in the 
##    corresponding cdef file 'net.cdefs'
## @synopsis
##      net_use(%args);
## INPUTS TO METHOD CALL 'net_use' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'devicename' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'sharename' </td> </tr> 
## <tr> <td> '/PERSISTENT' </td> <td> 'persistent' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'password' </td> </tr> 
## <tr> <td> '/USER' </td> <td> 'user' </td> </tr> 
## <tr> <td> '/SAVECRED' </td> <td> 'savecred' </td> </tr> 
## <tr> <td> '/DELETE' </td> <td> 'delete' </td> </tr> 
## <tr> <td> '/HOME' </td> <td> 'home' </td> </tr> 
## <tr> <td> '/SMARTCARD' </td> <td> 'smartcard' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net use
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;New connections will not be remembered.
##&nbsp;
##&nbsp;
##&nbsp;Status       Local     Remote                    Network
##&nbsp;
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;Disconnected U:        \\172.17.208.61\test_limon 
##&nbsp;                                                Microsoft Windows Network
##&nbsp;             V:        \\172.17.208.61\test_limon 
##&nbsp;                                                Microsoft Windows Network
##&nbsp;             W:        \\ABSOLUT-LIMON\test_limon 
##&nbsp;                                                Microsoft Windows Network
##&nbsp;             X:        \\svlinf06.na.ead.netapp.com\localtest\arch\win32 
##&nbsp;                                                Microsoft Windows Network
##&nbsp;Disconnected Y:        \\ntl1\go                 Microsoft Windows Network
##&nbsp;Disconnected Z:        \\ntl1\c$                 Microsoft Windows Network
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => 'Disconnected',
##&nbsp;            'local' => 'U:',
##&nbsp;            'remote' => '\\\\172.17.208.61\\test_limon'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => '',
##&nbsp;            'local' => 'V:',
##&nbsp;            'remote' => '\\\\172.17.208.61\\test_limon'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => '',
##&nbsp;            'local' => 'W:',
##&nbsp;            'remote' => '\\\\ABSOLUT-LIMON\\test_limon'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => '',
##&nbsp;            'local' => 'X:',
##&nbsp;            'remote' => '\\\\svlinf06.na.ead.netapp.com\\localtest\\arch\\win32'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => 'Disconnected',
##&nbsp;            'local' => 'Y:',
##&nbsp;            'remote' => '\\\\ntl1\\go'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network' => 'Microsoft Windows Network',
##&nbsp;            'status' => 'Disconnected',
##&nbsp;            'local' => 'Z:',
##&nbsp;            'remote' => '\\\\ntl1\\c$'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub net_use {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $parsed_output = [];

    if($error_level) {
        if($output =~ /The network .+not be found/) {
            NACL::APISet::Exceptions::InvalidParamValueException->throw(text=>$output,
                                                       response_object => $resp);
        }else{
            $output ||= 'Command reported a non-zero errorlevel';
            NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
        }
    }

    $output =~ s/The command completed successfully.//;
    if ($output =~ /Status\s+Local\s+Remote\s+Network/) {

        $output =~ s/The command completed successfully.//;
        my @lines = get_lines(raw_output => $output,
                              first_line => 'Status\s+Local\s+Remote\s+Network');
        while(shift(@lines) !~ /------------/) {
        }

        while(@lines) {

            my %hash = _read_section(\@lines);
            push(@$parsed_output, \%hash);
            
        }
    } elsif ($output =~ /Local name.+/) {
        my @lines = get_lines(raw_output => $output, 
                              first_line => 'Local name');
        my %hash;
        foreach(@lines) {
            my @values = split /\s+/, $_;
            shift @values if $values[0] =~ /\W+/;
            $hash{to_key_name(item => join(' ', @values[0..$#values-1]))} = $values[-1];
        }
        if ( defined $hash{'local'} && $hash{'local'} eq 'name') {
            $hash{'local'} = $hash{local_name} = '';
        } elsif ($hash{local_name}) {
            $hash{'local'} = $hash{local_name};
        }
        
        $hash{remote} = $hash{remote_name};
        @$parsed_output = (\%hash);

    } elsif ($output =~ /Drive ([A-Z]:) is now connected to (.+)./) {
        $parsed_output = [{'local' => $1, remote => $2}];
    }
    return $parsed_output;
}

sub _read_section {
    my @lines = @{$_[0]};
    
    my @values = split /\s+/, $lines[0];
    shift @lines; # now we don't need this line
    
    # look for the value which begins with "\\" - this is the value for 'remote'
    my $i = 0;
    my %hash;
    
    foreach(@values) {
        if($_ =~ /^\\\\/) {
            $hash{remote} = $_;
            last;
        } elsif ($_ =~ /^[A-Z]:$/) {
            $hash{'local'} = $_;
        }
        $i++;
    }
    
    # Now find out the 'status'
    if (exists $hash{'local'}) {
        if($values[0] eq $hash{'local'}) {
            # there is no value for 'status'
            $hash{status} = '';
        } else {
            $hash{status} = $values[0];
        }
    } else { 
        if($values[0] ne $hash{remote} ) {
            $hash{status} = $values[0];
        } else {
            $hash{status} = '';
        }
        $hash{'local'} = '';
    }
    
    # Now find the 'network'
    if($values[-1] eq $values[$i]) {
        # network value is on the next line 
        $hash{network} = shift @lines;
    } else {
        $hash{network} = join (' ', @values[$i+1..$#values]);
    }
    @{$_[0]} = @lines;
    return %hash;
}

## @name ::net_start
##
## @summary Parses the raw output from the "net start" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from net_start with any of its options as described in the 
##    corresponding cdef file 'net.cdefs'
## @synopsis
##      net_start(%args);
## INPUTS TO METHOD CALL 'net_start' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net start
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;These Windows services are started:
##&nbsp;
##&nbsp;   Background Intelligent Transfer Service
##&nbsp;   Base Filtering Engine
##&nbsp;   Certificate Propagation
##&nbsp;   COM+ Event System
##&nbsp;   COM+ System Application
##&nbsp;   Computer Browser
##&nbsp;   Cryptographic Services
##&nbsp;   DCOM Server Process Launcher
##&nbsp;   Desktop Window Manager Session Manager
##&nbsp;   DHCP Client
##&nbsp;   Diagnostic Policy Service
##&nbsp;   Diagnostic Service Host
##&nbsp;   Distributed Link Tracking Client
##&nbsp;   Distributed Transaction Coordinator
##&nbsp;   DNS Client
##&nbsp;   Function Discovery Resource Publication
##&nbsp;   Group Policy Client
##&nbsp;   IKE and AuthIP IPsec Keying Modules
##&nbsp;   IP Helper
##&nbsp;   IPsec Policy Agent
##&nbsp;   Netlogon
##&nbsp;   Network Connections
##&nbsp;   Network List Service
##&nbsp;   Network Location Awareness
##&nbsp;   Network Store Interface Service
##&nbsp;   Offline Files
##&nbsp;   Plug and Play
##&nbsp;   Power
##&nbsp;   Print Spooler
##&nbsp;   Remote Desktop Configuration
##&nbsp;   Remote Desktop Services
##&nbsp;   Remote Desktop Services UserMode Port Redirector
##&nbsp;   Remote Procedure Call (RPC)
##&nbsp;   RPC Endpoint Mapper
##&nbsp;   Security Accounts Manager
##&nbsp;   Security Center
##&nbsp;   Server
##&nbsp;   Shell Hardware Detection
##&nbsp;   SSDP Discovery
##&nbsp;   Superfetch
##&nbsp;   System Event Notification Service
##&nbsp;   Task Scheduler
##&nbsp;   TCP/IP NetBIOS Helper
##&nbsp;   Themes
##&nbsp;   User Profile Service
##&nbsp;   VMware Tools Service
##&nbsp;   VMware Upgrade Helper
##&nbsp;   Windows Audio
##&nbsp;   Windows Audio Endpoint Builder
##&nbsp;   Windows Defender
##&nbsp;   Windows Event Log
##&nbsp;   Windows Firewall
##&nbsp;   Windows Font Cache Service
##&nbsp;   Windows Management Instrumentation
##&nbsp;   Windows Search
##&nbsp;   Windows Time
##&nbsp;   Windows Update
##&nbsp;   Workstation
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub net_start {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    if($error_level) {
        if($output =~ /The service name is invalid/) {
            NACL::APISet::Exceptions::InvalidParamValueException->throw(text=>$output,
                                                       response_object => $resp);
        }
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
    }
	return [];
}

## @name ::net_stop
##
## @summary Parses the raw output from the "net stop" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from net_stop with any of its options as described in the 
##    corresponding cdef file 'net.cdefs'
## @synopsis
##      net_stop(%args);
## INPUTS TO METHOD CALL 'net_stop' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub net_stop {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
    }
	return [];
}

## @name ::net_stats
##
## @summary Parses the raw output from the "net stats" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from net_stats with any of its options as described in the 
##    corresponding cdef file 'net.cdefs'
## @synopsis
##      net_stats(%args);
## INPUTS TO METHOD CALL 'net_stats' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'workstation' </td> <td> 'workstation' </td> </tr> 
## <tr> <td> 'server' </td> <td> 'server' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = net stats server
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;Server Statistics for \\WIN7-CIFSQA-7
##&nbsp;
##&nbsp;
##&nbsp;Statistics since 9/13/2011 1:58:56 PM
##&nbsp;
##&nbsp;
##&nbsp;Sessions accepted                  1
##&nbsp;Sessions timed-out                 0
##&nbsp;Sessions errored-out               0
##&nbsp;
##&nbsp;Kilobytes sent                     0
##&nbsp;Kilobytes received                 0
##&nbsp;
##&nbsp;Mean response time (msec)          0
##&nbsp;
##&nbsp;System errors                      0
##&nbsp;Permission violations              0
##&nbsp;Password violations                0
##&nbsp;
##&nbsp;Files accessed                     0
##&nbsp;Communication devices accessed     0
##&nbsp;Print jobs spooled                 0
##&nbsp;
##&nbsp;Times buffers exhausted
##&nbsp;
##&nbsp;  Big buffers                      0
##&nbsp;  Request buffers                  0
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sessions_errored_out' => '0',
##&nbsp;            'sessions_timed_out' => '0',
##&nbsp;            'kb_sent' => '0',
##&nbsp;            'hostname' => 'WIN7-CIFSQA-7',
##&nbsp;            'password_violations' => '0',
##&nbsp;            'files_accessed' => '0',
##&nbsp;            'up_since' => '9/13/2011 1:58:56 PM',
##&nbsp;            'request_buffers_exhausted' => '0',
##&nbsp;            'system_errors' => '0',
##&nbsp;            'permission_violations' => '0',
##&nbsp;            'mean_resp_time' => '0',
##&nbsp;            'kb_rcvd' => '0',
##&nbsp;            'com_devices_accessed' => '0',
##&nbsp;            'big_buffers_exhausted' => '0',
##&nbsp;            'print_jobs_spooled' => '0',
##&nbsp;            'sessions_accepted' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = net stats workstation
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;Workstation Statistics for \\WIN7-CIFSQA-7
##&nbsp;
##&nbsp;
##&nbsp;Statistics since 9/13/2011 1:58:55 PM
##&nbsp;
##&nbsp;
##&nbsp;  Bytes received                               14980995173
##&nbsp;  Server Message Blocks (SMBs) received        923435
##&nbsp;  Bytes transmitted                            159901162
##&nbsp;  Server Message Blocks (SMBs) transmitted     905846
##&nbsp;  Read operations                              506313
##&nbsp;  Write operations                             871160
##&nbsp;  Raw reads denied                             0
##&nbsp;  Raw writes denied                            0
##&nbsp;
##&nbsp;  Network errors                               0
##&nbsp;  Connections made                             3027
##&nbsp;  Reconnections made                           0
##&nbsp;  Server disconnects                           22
##&nbsp;
##&nbsp;  Sessions started                             0
##&nbsp;  Hung sessions                                0
##&nbsp;  Failed sessions                              0
##&nbsp;  Failed operations                            0
##&nbsp;  Use count                                    3480
##&nbsp;  Failed use count                             748
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'network_errors' => '0',
##&nbsp;            'failed_ops' => '0',
##&nbsp;            'smb_rcvd' => '923435',
##&nbsp;            'hostname' => 'WIN7-CIFSQA-7',
##&nbsp;            'reconnections_made' => '0',
##&nbsp;            'up_since' => '9/13/2011 1:58:55 PM',
##&nbsp;            'raw_reads_denied' => '0',
##&nbsp;            'connections_made' => '3027',
##&nbsp;            'use_count' => '3480',
##&nbsp;            'sessions_started' => '0',
##&nbsp;            'server_disconnects' => '22',
##&nbsp;            'read_ops' => '506313',
##&nbsp;            'hung_sessions' => '0',
##&nbsp;            'smb_tx' => '905846',
##&nbsp;            'raw_writes_denied' => '0',
##&nbsp;            'write_ops' => '871160',
##&nbsp;            'bytes_rcvd' => '14980995173',
##&nbsp;            'failed_use_count' => '748',
##&nbsp;            'bytes_tx' => '159901162',
##&nbsp;            'failed_sessions' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub net_stats {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
    }
    my %data;
    my @lines = get_lines(raw_output => $output);
    while ( my $line = shift(@lines) ) {
        if ($line =~ /Statistics for ..(.*)$/ ) {
            $data{hostname} = $1;
        } elsif ( $line =~ /Statistics since (\d+.*)$/ ) {
            # convert this to seconds?
            $data{up_since} = $1;
        } elsif ( $line =~ /Sessions accepted\s*(\d+)$/ ) {
            $data{sessions_accepted} = $1;
        } elsif ( $line =~ /Sessions timed-out\s*(\d+)$/ ) {
            $data{sessions_timed_out} = $1;
        } elsif ( $line =~ /Sessions errored-out\s*(\d+)$/ ) {
            $data{sessions_errored_out} = $1;
        } elsif ( $line =~ /Kilobytes sent\s*(\d+)$/ ) {
            $data{kb_sent} = $1;
        } elsif ( $line =~ /Kilobytes received\s*(\d+)$/ ) {
            $data{kb_rcvd} = $1;
        } elsif ( $line =~ /Mean response time \(msec\)\s*(\d+)$/ ) {
            $data{mean_resp_time} = $1;
        } elsif ( $line =~ /System errors\s*(\d+)$/ ) {
            $data{system_errors} = $1;
        } elsif ( $line =~ /Permission violations\s*(\d+)$/ ) {
            $data{permission_violations} = $1;
        } elsif ( $line =~ /Password violations\s*(\d+)$/ ) {
            $data{password_violations} = $1;
        } elsif ( $line =~ /Files accessed\s*(\d+)$/ ) {
            $data{files_accessed} = $1;
        } elsif ( $line =~ /Communication devices accessed\s*(\d+)$/ ) {
            $data{com_devices_accessed} = $1;
        } elsif ( $line =~ /Print jobs spooled\s*(\d+)$/ ) {
            $data{print_jobs_spooled} = $1;
        } elsif ( $line =~ /Big buffers\s*(\d+)$/ ) {
            $data{big_buffers_exhausted} = $1;
        } elsif ( $line =~ /Request buffers\s*(\d+)$/ ) {
            $data{request_buffers_exhausted} = $1;
        } elsif ( $line =~ /Bytes received\s*(\d+)$/ ) {
            $data{bytes_rcvd} = $1;
        } elsif ( $line =~ /SMBs. received\s*(\d+)$/ ) {
            $data{smb_rcvd} = $1;
        } elsif ( $line =~ /Bytes transmitted\s*(\d+)$/ ) {
            $data{bytes_tx} = $1;
        } elsif ( $line =~ /SMBs. transmitted\s*(\d+)$/ ) {
            $data{smb_tx} = $1;
        } elsif ( $line =~ /Read operations\s*(\d+)$/ ) {
            $data{read_ops} = $1;
        } elsif ( $line =~ /Write operations\s*(\d+)$/ ) {
            $data{write_ops} = $1;
        } elsif ( $line =~ /Raw reads denied\s*(\d+)$/ ) {
            $data{raw_reads_denied} = $1;
        } elsif ( $line =~ /Raw writes denied\s*(\d+)$/ ) {
            $data{raw_writes_denied} = $1;
        } elsif ( $line =~ /Network errors\s*(\d+)$/ ) {
            $data{network_errors} = $1;
        } elsif ( $line =~ /Connections made\s*(\d+)$/ ) {
            $data{connections_made} = $1;
        } elsif ( $line =~ /Reconnections made\s*(\d+)$/ ) {
            $data{reconnections_made} = $1;
        } elsif ( $line =~ /Server disconnects\s*(\d+)$/ ) {
            $data{server_disconnects} = $1;
        } elsif ( $line =~ /Sessions started\s*(\d+)$/ ) {
            $data{sessions_started} = $1;
        } elsif ( $line =~ /Hung sessions\s*(\d+)$/ ) {
            $data{hung_sessions} = $1;
        } elsif ( $line =~ /Failed sessions\s*(\d+)$/ ) {
            $data{failed_sessions} = $1;
        } elsif ( $line =~ /Failed operations\s*(\d+)$/ ) {
            $data{failed_ops} = $1;
        } elsif ( $line =~ /Use count\s*(\d+)$/ ) {
            $data{use_count} = $1;
        } elsif ( $line =~ /Failed use count\s*(\d+)$/ ) {
            $data{failed_use_count} = $1;
        }
    }
    my @return;
    push(@return,\%data);
    return \@return;
}

## @name ::net_time
##
## @summary Parses the raw output from the "net time" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from net_time with any of its options as described in the
##    corresponding cdef file 'net_time.cdefs'
## @synopsis
##      net_time(%args);
## INPUTS TO METHOD CALL 'net_time' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> <td> '0' </td> </tr>
## <tr> <td> '/querysntp' </td> <td> 'querysntp' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'rtsdomain' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'computername' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'setsntp' </td> <td> '0' </td> </tr>
## <tr> <td> '\localhost' </td> <td> 'localhost' </td> <td> '1' </td> </tr>
## <tr> <td> '/set' </td> <td> 'set' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = net time
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current time at \\CSIQA-AD1 is 7/31/2012 2:14 AM
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '2:14 AM',
##&nbsp;            'date' => '7/31/2012',
##&nbsp;            'server' => 'CSIQA-AD1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = net time /domain
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current time at \\csiqa-ad1.csiqa.rtp.netapp.com is 7/31/2012 2:14 AM
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '2:14 AM',
##&nbsp;            'date' => '7/31/2012',
##&nbsp;            'server' => 'csiqa-ad1.csiqa.rtp.netapp.com'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = net time /rtsdomain
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current time at \\csiqa-ad1.csiqa.rtp.netapp.com is 7/31/2012 2:14 AM
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '2:14 AM',
##&nbsp;            'date' => '7/31/2012',
##&nbsp;            'server' => 'csiqa-ad1.csiqa.rtp.netapp.com'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = net time /SETSNTP
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = net time /SET
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current time at \\CSIQA-AD1 is 7/31/2012 2:14 AM
##&nbsp;
##&nbsp;The current local clock is 7/31/2012 2:14 AM
##&nbsp;Do you want to set the local computer's time to match the
##&nbsp;time at \\CSIQA-AD1? (Y/N) [Y]: y
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'local_date' => '7/31/2012',
##&nbsp;            'local_time' => '2:14 AM',
##&nbsp;            'server' => 'CSIQA-AD1',
##&nbsp;            'date'  => '7/31/2012',
##&nbsp;            'time' => '2:14 AM'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = net time \\localhost
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Current time at \\localhost is 7/31/2012 6:17 AM
##&nbsp;
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '6:17 AM',
##&nbsp;            'date' => '7/31/2012',
##&nbsp;            'server' => 'localhost'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub net_time {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    foreach my $line (@lines) {
        if ( $line =~ /Current time\s+at\s+(\S+)\s+is\s+(\S+)\s+(\S+\s+\w+)/ )
        {
            $result_array[0]->{date} = $2;
            $result_array[0]->{'time'} = $3;
            my $temp = $1;
            $temp =~ s/\\*//;
            $result_array[0]->{server} = $temp;
        } elsif (
            $line =~ /The current local clock\s+is\s+(\S+)\s+(\S+\s+\w+)/ ) {
            $result_array[0]->{local_time} = $2;
            $result_array[0]->{local_date} = $1;
        }
    }
    return \@result_array;
} ## end sub net_time

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /The option\s+\S+\s+is unknown/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /The syntax of this command is:.*|Could not.*/ )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
