#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse netsh command on Windows 7
##
## @description
##    This is a library to parse the raw output of command netsh on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::netsh
##
##  netsh(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::netsh;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    $$error_message = $raw_output;
    if ( $$error_message =~ /(.*parameters not specified.*)/i ) {
        $$error_message = $1;
        $error          = "MissingRequiredParam";
    }
    elsif (( $$error_message =~ /(.*name is not registered.*)/i )
        or ( $$error_message =~ /(.*not an acceptable value.*)/i ) )
    {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    }
    elsif ( $$error_message =~ /(.*interfaces can not be.*)/i ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    elsif ( $$error_message =~
/(The syntax supplied for this command is not valid.*)|(Cannot alter this setting because it is configured by Group Policy)|(This command can only be executed from an elevated command prompt.*)/i
      )
    {
        $$error_message = $1;
        $error          = "CommandFailed";
    }

    return $error;

}

## @name ::netsh_interface_set_interface
##
## @summary Parses the raw output from the "netsh interface set interface" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netsh_interface_set_interface with any of its options as described in the
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_interface_set_interface(%args);
## INPUTS TO METHOD CALL 'netsh_interface_set_interface' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'admin' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-interface-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'connect' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh interface set interface "Wireless Network Connection" ENABLED CONNECTED "Wireless Network Connection 1"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub netsh_interface_set_interface {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        command       => "netsh interface set interface",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse because no output is generated
    return [];
}

## @name ::netsh_branchcache_set_service
##
## @summary Parses the raw output from the "netsh branchcache set service" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netsh_branchcache_set_service with any of its options as described in the
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_branchcache_set_service(%args);
## INPUTS TO METHOD CALL 'netsh_branchcache_set_service' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'clientauthentication' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'location' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'mode' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'serveonbattery' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh branchcache set service DISTRIBUTED
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = netsh branchcache set service DISTRIBUTED
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub netsh_branchcache_set_service {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        command       => "netsh branchcache set service",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse anything because no output is generated
    return [];
}

## @name ::netsh_branchcache_show_status
##
## @summary Parses the raw output from the "netsh branchcache show status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netsh_branchcache_show_status with any of its options as described in the
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_branchcache_show_status(%args);
## INPUTS TO METHOD CALL 'netsh_branchcache_show_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh branchcache show status
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Running',
##&nbsp;            'mode' => ' Distributed Caching',
##&nbsp;            'serveonbattery' => 'Disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub netsh_branchcache_show_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $value;

    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @array_lines =
      get_lines( raw_output => $output, response_object => $resp );
    my %hash;

    foreach my $line (@array_lines) {
        if ( $line =~ /Service Mode\s+=((\s\w+)+)/ ) {
            $hash{'mode'} = $1;
        }
        elsif ( $line =~ /Serve peers on battery power =\s+(\w+)/ ) {
            $hash{'serveonbattery'} = $1;
        }
        elsif ( $line =~ /Current Status\s+=\s+(\w+)/ ) {
            $hash{'status'} = $1;
        }
    }

    @result_array = ( \%hash );
    return \@result_array;
}

## @name ::netsh_branchcache_show_localcache
##
## @summary Parses the raw output from the "netsh branchcache show localcache" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netsh_branchcache_show_localcache with any of its options as described in the
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_branchcache_show_localcache(%args);
## INPUTS TO METHOD CALL 'netsh_branchcache_show_localcache' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh branchcache show localcache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'maxcachesize' => '5% of hard disk',
##&nbsp;            'cachelocation' => 'C:\\Windows\\ServiceProfiles\\NetworkService\\AppData\\Local\\PeerDistRepub',
##&nbsp;            'currcachesize' => '4008628 Bytes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub netsh_branchcache_show_localcache {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $value;

    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @array_lines =
      get_lines( raw_output => $output, response_object => $resp );
    my %hash;

    foreach my $line (@array_lines) {
        if ( $line =~ /Maximum Cache Size\s+=\s(\d+%\D+)/ ) {
            $hash{'maxcachesize'} = $1;
        }
        elsif ( $line =~ /Active Current Cache Size\s+=\s(\d+\D+)/ ) {
            $hash{'currcachesize'} = $1;
        }
        elsif ( $line =~ /Local Cache Location\s+=\s(\w[:](\\\S+))/ ) {
            $hash{'cachelocation'} = $1;
        }
    }

    @result_array = ( \%hash );
    return \@result_array;
}

## @name ::netsh_branchcache_show_status_all
##
## @summary Parses the raw output from the "netsh branchcache show status all" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from netsh_branchcache_show_status_all with any of its options as described in the 
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_branchcache_show_status_all(%args);
## INPUTS TO METHOD CALL 'netsh_branchcache_show_status_all' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh branchcache show status all
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'localcache' => {
##&nbsp;                              'maxcachesize' => '5% of hard disk',
##&nbsp;                              'cachelocation' => 'C:\\Windows\\ServiceProfiles\\NetworkService\\AppData\\Local\\PeerDistRepub',
##&nbsp;                              'currcachesize' => '4008628 Bytes'
##&nbsp;                            },
##&nbsp;            'branchcache' => {
##&nbsp;                               'status' => 'Running',
##&nbsp;                               'mode' => ' Distributed Caching',
##&nbsp;                               'starttype' => 'Manual',
##&nbsp;                               'serveonbattery' => 'Disabled'
##&nbsp;                             },
##&nbsp;            'publicationcache' => {
##&nbsp;                                    'maxcachesize' => '1% of hard disk',
##&nbsp;                                    'cachelocation' => 'C:\\Windows\\ServiceProfiles\\NetworkService\\AppData\\Local\\PeerDistPub',
##&nbsp;                                    'currcachesize' => '0 Bytes'
##&nbsp;                                  },
##&nbsp;            'networking' => {
##&nbsp;                                          'hosted_cache_url_reservation' => ' Configured',
##&nbsp;                                          'content_retrieval_url_reservation' => ' Configured',
##&nbsp;                                          'peer_discovery_firewall_rules' => ' Enabled',
##&nbsp;                                          'ssl_certificate_bound_to_hosted_cache_port' => ' Not Configured',
##&nbsp;                                          'hosted_cache_client_firewall_rules' => ' Enabled',
##&nbsp;                                          'content_retrieval_firewall_rules' => ' Enabled',
##&nbsp;                                          'hosted_cache_server_firewall_rules' => ' Enabled'
##&nbsp;                                        } 
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub netsh_branchcache_show_status_all {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $value;

    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @array_lines =
      get_lines( raw_output => $output, response_object => $resp );
    my ( %hash, $i, $j, $k, $l ,$name);
	my %localhash;
    $i = $j = $k = $l = 0;

    my @networkarray = (
        'Content Retrieval URL Reservation',
        'Hosted Cache URL Reservation',
        'SSL Certificate Bound To Hosted Cache Port',
        'Content Retrieval Firewall Rules',
        'Peer Discovery Firewall Rules',
        'Hosted Cache Server Firewall Rules',
        'Hosted Cache Client Firewall Rules'
    );

    foreach my $line (@array_lines) {
        if ( $line !~ /Publication Cache Status:/ ) {
            $i++;
        }
        else {
            $i += 2;
            if ( $array_lines[$i] =~ /Maximum Cache Size\s+=\s(\d+%\D+)/ ) {
                $hash{'publicationcache'}{'maxcachesize'} = $1;
            }
            $i++;
            if ( $array_lines[$i] =~ /Active Current Cache Size\s+=\s(\d+\D+)/ )
            {
                $hash{'publicationcache'}{'currcachesize'} = $1;
            }
            $i++;
            if ( $array_lines[$i] =~
                /Publication Cache Location\s+=\s(\w[:](\\\S+))/ )
            {
                $hash{'publicationcache'}{'cachelocation'} = $1;
            }
        }
        if ( $line !~ /Local Cache Status:/ ) {
            $j++;
        }
        else {
            $j += 2;
            if ( $array_lines[$j] =~ /Maximum Cache Size\s+=\s(\d+%\D+)/ ) {
                $hash{'localcache'}{'maxcachesize'} = $1;
            }
            $j++;
            if ( $array_lines[$j] =~ /Active Current Cache Size\s+=\s(\d+\D+)/ )
            {
                $hash{'localcache'}{'currcachesize'} = $1;
            }
            $j++;
            if (
                $array_lines[$j] =~ /Local Cache Location\s+=\s(\w[:](\\\S+))/ )
            {
                $hash{'localcache'}{'cachelocation'} = $1;
            }
        }
        if ( $line !~ /BranchCache Service Status:/ ) {
            $k++;
        }
        else {
            $k += 2;
            if ( $array_lines[$k] =~ /Service Mode\s+=((\s\w+)+)/ ) {
                $hash{'branchcache'}{'mode'} = $1;
            }
            $k++;
            if ( $array_lines[$k] =~ /Serve peers on battery power =\s+(\w+)/ )
            {
                $hash{'branchcache'}{'serveonbattery'} = $1;
            }
            $k++;
            if ( $array_lines[$k] =~ /Current Status\s+=\s+(\w+)/ ) {
                $hash{'branchcache'}{'status'} = $1;
            }
            $k++;
            if ( $array_lines[$k] =~ /Service Start Type\s+=\s+(\w+)/ ) {
                $hash{'branchcache'}{'starttype'} = $1;
            }
        }
        if ( $line !~ /Networking Status:/ ) {
            $l++;
        }
        else {
            $l += 2;
            foreach my $line (@networkarray) {
                if ( $array_lines[$l] =~ /($line)\s+=((\s+\w+)+)/ ) {
                    $hash{'networking'}{to_key_name(item=>"$1")} = $2;
                    $l++;
                }
            }
        }
    }

    @result_array = ( \%hash);
    return \@result_array;
}

## @name ::netsh_branchcache_reset
##
## @summary Parses the raw output from the "netsh branchcache reset" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from netsh_branchcache_reset with any of its options as described in the 
##    corresponding cdef file 'netsh.cdefs'
## @synopsis
##      netsh_branchcache_reset(%args);
## INPUTS TO METHOD CALL 'netsh_branchcache_reset' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = netsh branchcache reset
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub netsh_branchcache_reset {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        command       => "netsh branchcache reset",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse anything because no output is generated
    return [];
}

1;
