#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/ping.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse the output of the Windows ping command output
##
## @description
##    This is a library to parse the raw output of the Windows ping command
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::ping;
##
##    ping(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::ping;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'ttl' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'fragment' </td> <td> '1' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'hostname' </td> <td> '1' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'tos' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'timestamp' </td> <td> '0' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'timeout' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'source-addr' </td> <td> '0' </td> </tr> 
## <tr> <td> '-j' </td> <td> 'loose-source' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-6' </td> <td> 'ipv6' </td> <td> '1' </td> </tr> 
## <tr> <td> '-4' </td> <td> 'ipv4' </td> <td> '1' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'record-route' </td> <td> '0' </td> </tr> 
## <tr> <td> '-k' </td> <td> 'strict-source' </td> <td> '0' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'reverse-route' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Pinging 10.72.219.44 with 32 bytes of data:
##&nbsp;
##&nbsp;Reply from 10.72.219.44: bytes=32 time=227ms TTL=51
##&nbsp;Reply from 10.72.219.44: bytes=32 time=227ms TTL=51
##&nbsp;Reply from 10.72.219.44: bytes=32 time=227ms TTL=51
##&nbsp;Reply from 10.72.219.44: bytes=32 time=226ms TTL=51
##&nbsp;
##&nbsp;Ping statistics for 10.72.219.44:
##&nbsp;    Packets: Sent = 4, Received = 4, Lost = 0 (0% loss),
##&nbsp;Approximate round trip times in milli-seconds:
##&nbsp;    Minimum = 226ms, Maximum = 227ms, Average = 226ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '226ms',
##&nbsp;            'min' => '226ms',
##&nbsp;            'ip' => '10.72.219.44',
##&nbsp;            'max' => '227ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '51',
##&nbsp;                           'time' => '227ms',
##&nbsp;                           'ip' => '10.72.219.44'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '51',
##&nbsp;                           'time' => '227ms',
##&nbsp;                           'ip' => '10.72.219.44'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '51',
##&nbsp;                           'time' => '227ms',
##&nbsp;                           'ip' => '10.72.219.44'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '51',
##&nbsp;                           'time' => '226ms',
##&nbsp;                           'ip' => '10.72.219.44'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;Pinging fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a from fd20:8b1e:b255:8305:250:56ff:feae:4ea8 with 32 bytes of data:
##&nbsp;
##&nbsp;Reply from fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a: time=548ms 
##&nbsp;Reply from fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a: time<1ms 
##&nbsp;Reply from fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a: time<1ms 
##&nbsp;Reply from fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a: time<1ms 
##&nbsp;
##&nbsp;Ping statistics for fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a:
##&nbsp;    Packets: Sent = 4, Received = 4, Lost = 0 (0% loss),
##&nbsp;Approximate round trip times in milli-seconds:
##&nbsp;    Minimum = 0ms, Maximum = 548ms, Average = 137ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '137ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => 'fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a',
##&nbsp;            'max' => '548ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '548ms',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => 'fd20:8b1e:b255:8305:2a0:98ff:fe25:3e6a'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'host' => 'fd20:8b1e:b255:8305:250:56ff:feae:4ea8',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -a localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '0ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => '::1',
##&nbsp;            'max' => '0ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'hostname' => 'csiqa-ad1.csiqa.rtp.netapp.com',
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'host' => '::1',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -f localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '0ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'max' => '0ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'hostname' => 'csiqa-ad1.csiqa.rtp.netapp.com',
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -R ::1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '0ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => '::1',
##&nbsp;            'max' => '1ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'host' => '::1',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -4 localhost
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '0ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => '127.0.0.1',
##&nbsp;            'max' => '0ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'bytes' => '32',
##&nbsp;                           'ttl' => '128',
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '127.0.0.1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'hostname' => 'csiqa-ad1.csiqa.rtp.netapp.com',
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -6 ::1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '32',
##&nbsp;            'avg' => '0ms',
##&nbsp;            'min' => '0ms',
##&nbsp;            'ip' => '::1',
##&nbsp;            'max' => '0ms',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'time' => '1ms',
##&nbsp;                           'ip' => '::1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'loss_percent' => '0%',
##&nbsp;            'host' => '::1',
##&nbsp;            'packets_transmitted' => '4',
##&nbsp;            'loss' => '0',
##&nbsp;            'received' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    if($error_level) {
        if($output =~ /Ping request could not find host/) {
            NACL::APISet::Exceptions::InvalidParamValueException->throw(text=>$output,
                                                       response_object => $resp);
        }
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
    }
    my @lines = get_lines(raw_output=>$output, can_throw=>1,
                          first_line=>'Pinging',
                          response_object=>$resp);

    my $line = shift(@lines);
    my %hash;
    if($line =~ /Pinging (.+) with (\d+) bytes of data:/) {
        $hash{'ip'} = $1;
        $hash{'bytes'} = $2;
        if( $hash{'ip'} =~ /(.+) from (.+)/) {
            $hash{'ip'} = $1;
            $hash{'host'} = $2;
        }
        if( $hash{'ip'} =~ /(.+) \[(.+)\]/) {
            $hash{'hostname'} = $1;
            $hash{'ip'} = $2;
        }
    }

    $hash{'stats'} = [];

    while (@lines) {
        $line = $lines[0];
        shift(@lines);
        my %internal_hash;

        if ($line =~ /Reply from (.+):\s+(.+)/) {
            $internal_hash{'ip'} = $1;
            my $rest = $2;
            my @fields = split(/\s+/,$rest);
            foreach my $field(@fields) {
                my $seperator;
                foreach my $sep ('=','>','<') {
                   if($field =~ /$sep/) {
                       $seperator = $sep;
                   }
                }
                my ($name,$value) = split(/$seperator/,$field);
                $name = to_key_name(item => $name) ;
                $internal_hash{$name} = $value;
            }
            push (@{$hash{'stats'}}, \%internal_hash);
        }
        last if($line =~ /Ping statistics for .+:/);
    }
    $line = $lines[0];

    #Packets: Sent = 4, Received = 4, Lost = 0 (0% loss),
    #3 packets transmitted, 3 packets received, 0.0% packet loss
    if ($line =~ /Packets: Sent = (\d+), Received = (\d+), Lost = (\d+) \((\d+%) loss\),/) {

        $hash{packets_transmitted} = $1;
        $hash{received} = $2;
        $hash{loss} = $3;
        $hash{loss_percent} = $4;
        shift(@lines);
        $line = $lines[0];
    }

    #Approximate round trip times in milli-seconds:
    #Minimum = 0ms, Maximum = 5ms, Average = 2ms

    $line = $lines[1];
    if($line =~ /Minimum/) {
        my @round_trip = split(/,\s/,$line);
        foreach my $field_trip (@round_trip) {
            my($name,$value) = split(/\s=\s/,$field_trip);
            $name =~ tr/[A-Z]/[a-z]/;
            $name = substr($name,0,3);
            $name = "avg" if($name =~ /ave/); 
            $hash{$name}=$value;
                     
        }
    }

    my @result_array = (\%hash);
    return \@result_array;

}
1;
