#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/systeminfo.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary systeminfo command
##
## @description
##    This is a library to parse the raw output of systeminfo command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::systeminfo;
##
##    systeminfo(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::systeminfo;

use strict;

use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Invalid argument/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid syntax/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::systeminfo
##
## @summary Parses the raw output from the "systeminfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from systeminfo with any of its options as described in the
##    corresponding cdef file 'systeminfo.cdefs'
## @synopsis
##      systeminfo(%args);
## INPUTS TO METHOD CALL 'systeminfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '/P' </td> <td> 'password' </td> <td> '0' </td> </tr>
## <tr> <td> '/FO' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '/S' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '/?' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '/U' </td> <td> 'user' </td> <td> '0' </td> </tr>
## <tr> <td> '/NH' </td> <td> 'no-header' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = systeminfo /FO CSV
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;"Host Name","OS Name","OS Version","OS Manufacturer","OS Configuration","OS Build Type","Registered Owner","Registered Organization","Product ID","Original Install Date","System Up Time","System Manufacturer","System Model","System Type","Processor(s)","BIOS Version","Windows Directory","System Directory","Boot Device","System Locale","Input Locale","Time Zone","Total Physical Memory","Available Physical Memory","Page File: Max Size","Page File: Available","Page File: In Use","Page File Location(s)","Domain","Logon Server","Hotfix(s)","Network Card(s)"
##&nbsp;"CSIQA-AD1","Microsoft(R) Windows(R) Server 2003, Standard Edition","5.2.3790 Service Pack 2 Build 3790","Microsoft Corporation","Additional/Backup Domain Controller","Multiprocessor Free","netapp","netapp","69712-640-0778677-45459","10/19/2009, 11:30:26 AM","25 Days, 18 Hours, 14 Minutes, 37 Seconds","VMware, Inc.","VMware Virtual Platform","X86-based PC","2 Processor(s) Installed.,[01]: x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2334 Mhz,[02]: x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2335 Mhz","INTEL  - 6040000","C:\WINDOWS","C:\WINDOWS\system32","\Device\HarddiskVolume1","en-us;English (United States)","en-us;English (United States)","(GMT-05:00) Eastern Time (US & Canada)","2,047 MB","1,116 MB","3,944 MB","3,241 MB","703 MB","C:\pagefile.sys","csiqa.rtp.netapp.com","N/A","414 Hotfix(s) Installed.,[01]: File 1,[02]: File 1,[03]: File 1,[04]: File 1,[05]: File 1,[06]: File 1,[07]: File 1,[08]: File 1,[09]: File 1,[10]: File 1","1 NIC(s) Installed.,[01]: VMware Accelerated AMD PCNet Adapter,      Connection Name: Local Area Connection,      DHCP Enabled:    No,      IP address(es),      [01]: 10.227.64.21"
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_type' => 'X86-based PC',
##&nbsp;            'system_manufacturer' => 'VMware, Inc.',
##&nbsp;            'registered_owner' => 'netapp',
##&nbsp;            'bios_version' => 'INTEL  - 6040000',
##&nbsp;            'processors_installed' => [
##&nbsp;                                          'x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2334 Mhz',
##&nbsp;                                          'x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2335 Mhz'
##&nbsp;                                        ],
##&nbsp;            'system_directory' => 'C:\\WINDOWS\\system32',
##&nbsp;            'host_name' => 'CSIQA-AD1',
##&nbsp;            'os_version' => '5.2.3790 Service Pack 2 Build 3790',
##&nbsp;            'boot_device' => '\\Device\\HarddiskVolume1',
##&nbsp;            'total_physical_memory' => '2,047 MB',
##&nbsp;            'number_of_processors_installed' => '2',
##&nbsp;            'system_up_time' => '25 Days, 18 Hours, 14 Minutes, 37 Seconds',
##&nbsp;            'system_locale' => 'en-us;English (United States)',
##&nbsp;            'os_configuration' => 'Additional/Backup Domain Controller',
##&nbsp;            'registered_organization' => 'netapp',
##&nbsp;            'windows_directory' => 'C:\\WINDOWS',
##&nbsp;            'os_build_type' => 'Multiprocessor Free',
##&nbsp;            'original_install_date' => '10/19/2009, 11:30:26 AM',
##&nbsp;            'page_file_locations' => 'C:\\pagefile.sys',
##&nbsp;            'domain' => 'csiqa.rtp.netapp.com',
##&nbsp;            'product_id' => '69712-640-0778677-45459',
##&nbsp;            'os_manufacturer' => 'Microsoft Corporation',
##&nbsp;            'number_of_hotfixs_installed' => '414',
##&nbsp;            'input_locale' => 'en-us;English (United States)',
##&nbsp;            'os_name' => 'Microsoft(R) Windows(R) Server 2003, Standard Edition',
##&nbsp;            'available_physical_memory' => '1,116 MB',
##&nbsp;            'hotfixs_installed' => [
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                       'File 1',
##&nbsp;                                     ],
##&nbsp;            'number_of_nics_installed' => '1',
##&nbsp;            'system_model' => 'VMware Virtual Platform',
##&nbsp;            'time_zone' => '(GMT-05:00) Eastern Time (US & Canada)',
##&nbsp;            'page_file' => '703 MB',
##&nbsp;            'logon_server' => 'N/A',
##&nbsp;            'nics_installed' => [
##&nbsp;                                    {
##&nbsp;                                        'dhcp_enabled' => 'No',
##&nbsp;                                        'name' => 'VMware Accelerated AMD PCNet Adapter',
##&nbsp;                                        'ip_address' => [
##&nbsp;                                                          '10.227.64.21'
##&nbsp;                                                        ],
##&nbsp;                                        'connection_name' => 'Local Area Connection'
##&nbsp;                                    }
##&nbsp;                                  ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = systeminfo /FO table
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Host Name                      OS Name                        OS Version                     OS Manufacturer                          OS Configuration                         OS Build Type        Registered Owner                                   Registered Organization                            Product ID                     Original Install Date     System Up Time                 System Manufacturer            System Model         System Type               Processor(s)                                  BIOS Version                   Windows Directory    System Directory     Boot Device                         System Locale                       Input Locale                        Time Zone                                          Total Physical Memory     Available Physical Memory Page File: Max Size       Page File: Available      Page File: In Use         Page File Location(s)          Domain               Logon Server         Hotfix(s)            Network Card(s)
##&nbsp;============================== ============================== ============================== ======================================== ======================================== ==================== ================================================== ================================================== ============================== ========================= ============================== ============================== ==================== ========================= ============================================= ============================== ==================== ==================== =================================== =================================== =================================== ================================================== ========================= ========================= ========================= ========================= ========================= ============================== ==================== ==================== ==================== ====================================================================================================
##&nbsp;CSIQA-AD1                      Microsoft(R) Windows(R) Server 5.2.3790 Service Pack 2 Build  Microsoft Corporation                    Additional/Backup Domain Controller      Multiprocessor Free  netapp                                             netapp                                             69712-640-0778677-45459        10/19/2009, 11:30:26 AM   6 Days, 7 Hours, 32 Minutes, 3 VMware, Inc.                   VMware Virtual Platf X86-based PC              2 Processor(s) Installed., [01]: x86 Family 6 INTEL  - 6040000               C:\WINDOWS           C:\WINDOWS\system32  \Device\HarddiskVolume1             en-us;English (United States)       en-us;English (United States)       (GMT-05:00) Eastern Time (US & Canada)             2,047 MB                  1,320 MB                  3,944 MB                  3,439 MB                  505 MB                    C:\pagefile.sys                csiqa.rtp.netapp.com N/A                  414 Hotfix(s) Insta, 1 NIC(s) Installed., [01]: VMware Accelerated AMD PCNet Adapter,       Connection Name: Local Area C
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_type' => 'X86-based PC',
##&nbsp;            'os_build_type' => 'Multiprocessor Free',
##&nbsp;            'original_install_date' => '10/19/2009, 11:30:26 AM',
##&nbsp;            'system_manufacturer' => 'VMware, Inc.',
##&nbsp;            'registered_owner' => 'netapp',
##&nbsp;            'bios_version' => 'INTEL  - 6040000',
##&nbsp;            'system_directory' => 'C:\\WINDOWS\\system32',
##&nbsp;            'domain' => 'csiqa.rtp.netapp.com',
##&nbsp;            'page_file_locations' => 'C:\\pagefile.sys',
##&nbsp;            'os_version' => '5.2.3790 Service Pack 2 Build',
##&nbsp;            'boot_device' => '\\Device\\HarddiskVolume1',
##&nbsp;            'product_id' => '69712-640-0778677-45459',
##&nbsp;            'processors' => '2 Processor(s) Installed., [01]: x86 Family 6',
##&nbsp;            'os_manufacturer' => 'Microsoft Corporation',
##&nbsp;            'total_physical_memory' => '2,047 MB',
##&nbsp;            'input_locale' => 'en-us;English (United States)',
##&nbsp;            'system_up_time' => '6 Days, 7 Hours, 32 Minutes, 3',
##&nbsp;            'os_name' => 'Microsoft(R) Windows(R) Server',
##&nbsp;            'page_file_max_size' => '3,944 MB',
##&nbsp;            'available_physical_memory' => '1,320 MB',
##&nbsp;            'page_file_in_use' => '505 MB',
##&nbsp;            'system_locale' => 'en-us;English (United States)',
##&nbsp;            'os_configuration' => 'Additional/Backup Domain Controller',
##&nbsp;            'page_file_available' => '3,439 MB',
##&nbsp;            'registered_organization' => 'netapp',
##&nbsp;            'system_model' => 'VMware Virtual Platf',
##&nbsp;            'hotfixs' => '414 Hotfix(s) Insta,',
##&nbsp;            'time_zone' => '(GMT-05:00) Eastern Time (US & Canada)',
##&nbsp;            'host_name' => 'CSIQA-AD1',
##&nbsp;            'logon_server' => 'N/A',
##&nbsp;            'windows_directory' => 'C:\\WINDOWS'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = systeminfo
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Host Name:                 CSIQA-AD1
##&nbsp;OS Name:                   Microsoft(R) Windows(R) Server 2003, Standard Edition
##&nbsp;OS Version:                5.2.3790 Service Pack 2 Build 3790
##&nbsp;OS Manufacturer:           Microsoft Corporation
##&nbsp;OS Configuration:          Additional/Backup Domain Controller
##&nbsp;OS Build Type:             Multiprocessor Free
##&nbsp;Registered Owner:          netapp
##&nbsp;Registered Organization:   netapp
##&nbsp;Product ID:                69712-640-0778677-45459
##&nbsp;Original Install Date:     10/19/2009, 11:30:26 AM
##&nbsp;System Up Time:            6 Days, 8 Hours, 2 Minutes, 46 Seconds
##&nbsp;System Manufacturer:       VMware, Inc.
##&nbsp;System Model:              VMware Virtual Platform
##&nbsp;System Type:               X86-based PC
##&nbsp;Processor(s):              2 Processor(s) Installed.
##&nbsp;                           [01]: x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2334 Mhz
##&nbsp;                           [02]: x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2335 Mhz
##&nbsp;BIOS Version:              INTEL  - 6040000
##&nbsp;Windows Directory:         C:\WINDOWS
##&nbsp;System Directory:          C:\WINDOWS\system32
##&nbsp;Boot Device:               \Device\HarddiskVolume1
##&nbsp;System Locale:             en-us;English (United States)
##&nbsp;Input Locale:              en-us;English (United States)
##&nbsp;Time Zone:                 (GMT-05:00) Eastern Time (US & Canada)
##&nbsp;Total Physical Memory:     2,047 MB
##&nbsp;Available Physical Memory: 1,321 MB
##&nbsp;Page File: Max Size:       3,944 MB
##&nbsp;Page File: Available:      3,439 MB
##&nbsp;Page File: In Use:         505 MB
##&nbsp;Page File Location(s):     C:\pagefile.sys
##&nbsp;Domain:                    csiqa.rtp.netapp.com
##&nbsp;Logon Server:              N/A
##&nbsp;Hotfix(s):                 414 Hotfix(s) Installed.
##&nbsp;                           [01]: File 1
##&nbsp;                           [02]: File 1
##&nbsp;                           [03]: File 1
##&nbsp;                           [04]: File 1
##&nbsp;                           [05]: File 1
##&nbsp;Network Card(s):           1 NIC(s) Installed.
##&nbsp;                           [01]: VMware Accelerated AMD PCNet Adapter
##&nbsp;                                 Connection Name: Local Area Connection
##&nbsp;                                 DHCP Enabled:    No
##&nbsp;                                 IP address(es)
##&nbsp;                                 [01]: 10.227.64.21
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_type' => 'X86-based PC',
##&nbsp;            'processors_installed' => [
##&nbsp;                                        'x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2334 Mhz',
##&nbsp;                                        'x86 Family 6 Model 15 Stepping 1 GenuineIntel ~2335 Mhz'
##&nbsp;                                      ],
##&nbsp;            'hotfixs_installed' => [
##&nbsp;                                     'File 1',
##&nbsp;                                     'File 1',
##&nbsp;                                     'File 1',
##&nbsp;                                     'File 1',
##&nbsp;                                     'File 1',
##&nbsp;                                   ],
##&nbsp;            'system_manufacturer' => 'VMware, Inc.',
##&nbsp;            'registered_owner' => 'netapp',
##&nbsp;            'bios_version' => 'INTEL  - 6040000',
##&nbsp;            'system_directory' => 'C:\\WINDOWS\\system32',
##&nbsp;            'os_version' => '5.2.3790 Service Pack 2 Build 3790',
##&nbsp;            'boot_device' => '\\Device\\HarddiskVolume1',
##&nbsp;            'total_physical_memory' => '2,047 MB',
##&nbsp;            'nics_installed' => [
##&nbsp;                                  {
##&nbsp;                                    'connection_name' => 'Local Area Connection',
##&nbsp;                                    'name' => 'VMware Accelerated AMD PCNet Adapter',
##&nbsp;                                    'ip_address' => [
##&nbsp;                                                      '10.227.64.21'
##&nbsp;                                                    ],
##&nbsp;                                    'dhcp_enabled' => 'No'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'system_up_time' => '6 Days, 8 Hours, 2 Minutes, 46 Seconds',
##&nbsp;            'number_of_hotfixs_installed' => '414',
##&nbsp;            'page_file_max_size' => '3,944 MB',
##&nbsp;            'page_file_in_use' => '505 MB',
##&nbsp;            'system_locale' => 'en-us;English (United States)',
##&nbsp;            'os_configuration' => 'Additional/Backup Domain Controller',
##&nbsp;            'registered_organization' => 'netapp',
##&nbsp;            'number_of_nics_installed' => '1',
##&nbsp;            'host_name' => 'CSIQA-AD1',
##&nbsp;            'windows_directory' => 'C:\\WINDOWS',
##&nbsp;            'os_build_type' => 'Multiprocessor Free',
##&nbsp;            'original_install_date' => '10/19/2009, 11:30:26 AM',
##&nbsp;            'page_file_locations' => 'C:\\pagefile.sys',
##&nbsp;            'domain' => 'csiqa.rtp.netapp.com',
##&nbsp;            'product_id' => '69712-640-0778677-45459',
##&nbsp;            'os_manufacturer' => 'Microsoft Corporation',
##&nbsp;            'input_locale' => 'en-us;English (United States)',
##&nbsp;            'os_name' => 'Microsoft(R) Windows(R) Server 2003, Standard Edition',
##&nbsp;            'available_physical_memory' => '1,321 MB',
##&nbsp;            'page_file_available' => '3,439 MB',
##&nbsp;            'system_model' => 'VMware Virtual Platform',
##&nbsp;            'number_of_processors_installed' => '2',
##&nbsp;            'time_zone' => '(GMT-05:00) Eastern Time (US & Canada)',
##&nbsp;            'logon_server' => 'N/A'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub systeminfo {

    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    my @lines
        = get_lines( raw_output => $raw_output, response_object => $resp );
    my $cmd = $resp->get_command();
    my ( @result_array, @array, @values, @head, @pos );
    my ( $c, $pkg_installed );

    if ( $cmd =~ /TABLE/i ) {
        while ( $lines[1] =~ m/\s/g ) {
            my $p = pos( $lines[1] );
            push( @pos, $p );
        }
        my $start = 0;
        my $temp1 = $lines[0];
        my $temp2 = $lines[2];
        for ( my $j = 0; $j <= $#pos; $j++ ) {
            my $value;
            my $head;
            my $end    = $pos[$j] - 1;
            my $start  = $pos[ $j - 1 ] if ( $j != 0 );
            my $length = $end - $start;
            $head = substr( $temp1, $start, $length );
            $head =~ s/://;
            $head =~ s/\s+$//;
            $value = substr( $temp2, $start, $length );
            $value =~ s/\s+$//;
            push( @head,   $head );
            push( @values, $value );
        }
        for ( my $k = 0; $k <= $#head; $k++ ) {
            $result_array[0]->{ to_key_name( item => $head[$k] ) }
                = $values[$k];
        }
        return \@result_array;
    } elsif ( $cmd =~ /CSV/i ) {
        @head   = split( /,/,   $lines[0] );
        @values = split( /","/, $lines[1] );
        foreach (@head) {
            $_ =~ s/"//g;
            $_ =~ s/://;
        }
        foreach (@values) {
            $_ =~ s/"//g;
        }

        for ( my $i = 0; $i <= $#values; $i++ ) {
            if ( $values[$i] =~ /(\d+)\s+(\w+\(s\)\s+Installed)/ ) {
                my $c = 0;
                $pkg_installed = to_key_name( item => $2 );
                my $no_of_pkg = "number_of_$pkg_installed";
                my @array = split( /,/, $values[$i] );
                $result_array[0]->{$no_of_pkg} = $1;
                for ( my $j = 0; $j <= $#array; $j++ ) {
                    if ( $array[$j] =~ /\[\d+\]:.*/ ) {
                        my @temp = split( /:/, $array[$j] );
                        $temp[1] =~ s/^\s+//;
                        $result_array[0]->{$pkg_installed}->[$c] = $temp[1];
                        $c++;
                    }
                }
            } else {
                $result_array[0]->{ to_key_name( item => $head[$i] ) }
                    = $values[$i];
            }
        }

        return \@result_array;
    } else {
        my $k = 0;
        my $m = -1;
        for ( my $i = 0; $i < @lines; $i++ ) {
            $lines[$i] =~ s/:\s+/_/ if ( $lines[$i] =~ /^Page File:\s+/ );
            @values = split( /:\s+/, $lines[$i] );
            my $flag = 0;
            if ( $values[1] =~ /(\d+)\s+(\w+\(s\)\s+Installed)/ ) {
                $c = -1;
                $pkg_installed = to_key_name( item => $2 );
                my $no_of_pkg = "number_of_$pkg_installed";
                $result_array[0]->{$no_of_pkg} = $1;
            } elsif ( $pkg_installed eq "nics_installed" ) {
                if ( $lines[$i] =~ /\[\d+]:\s+(\w+\s*.*)/ && $lines[$i] !~ /\[\d+]:\s+\S+[:.]/ ) {
                    $m++;
                    $result_array[0]->{$pkg_installed}->[$m]->{name} = $1;
                } elsif ( $lines[$i] =~ /(\w+\s?\w+):\s+(.*)/ ){
                    $result_array[0]->{$pkg_installed}->[$m]
                        ->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $lines[$i] =~ /\[\d+\]:\s+\d+./
                    | $lines[$i] =~ /\[\d+\]:\s+\S+:/ ) {
                    my @a = split( /:\s+/, $lines[$i] );
                    $result_array[0]->{$pkg_installed}->[$m]->{ip_address}
                        ->[$k] = $a[1];
                    $k++;
                }
            } elsif ( $values[0] =~ /^\[\d+\]$/ ) {
                $c++;
                $result_array[0]->{$pkg_installed}->[$c] = $values[1];
            } else {
                $result_array[0]->{ to_key_name( item => $values[0] ) }
                    = $values[1] if (defined $values[1]);
            }
        }
        return \@result_array;
    }

} ## end sub systeminfo

1;

