#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/takeown.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse takeown command on Windows
##
## @description
##    This is a library to parse the raw output of command takeown on Windows client.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::takeown
##
##  takeown(%args)
##
## @author anupamap@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::takeown;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~
        /No\s+network\s+provider\s+accepted\s+the\s+given\s+network\s+path/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
/The\s+system\s+cannot\s+find\s+the\s+path\s+specified|File\s+or\s+Directory\s+not\s+found|A\s+subdirectory\s+or\s+file\s+.*\s+already\s+exists/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /ERROR:.*|Invalid\s+syntax.*|Invalid\s+argument.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
}

## @name ::takeown
##
## @summary Parses the raw output from the "takeown" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from takeown with any of its options as described in the
##    corresponding cdef file 'takeown.cdefs'
## @synopsis
##      takeown(%args);
## INPUTS TO METHOD CALL 'takeown' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '/P' </td> <td> 'password' </td> <td> '0' </td> </tr>
## <tr> <td> '/R' </td> <td> 'recurse' </td> <td> '1' </td> </tr>
## <tr> <td> '/F' </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '/D' </td> <td> 'prompt' </td> <td> '0' </td> </tr>
## <tr> <td> '/U' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> '/A' </td> <td> 'admininstrators' </td> <td> '1' </td> </tr>
## <tr> <td> '/S' </td> <td> 'system' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = takeown /F file123
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SUCCESS: The file (or folder): "C:\\Documents and Settings\\CTLAdmin.CTL\\file123" now owned by user "CTL\\CTLAdmin".
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'owner' => 'CTL\\CTLAdmin',
##&nbsp;             'status' => 'SUCCESS',
##&nbsp;             'file' => 'C:\\Documents and Settings\\CTLAdmin.CTL\\file123'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = takeown /F file123 /A
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SUCCESS: The file (or folder): "C:\\Documents and Settings\\CTLAdmin.CTL\\file123" now owned by the administrators group.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;  $VAR1 = [
##&nbsp;           {
##&nbsp;             'owner' => 'administrators group',
##&nbsp;             'status' => 'SUCCESS',
##&nbsp;             'file' => 'C:\\Documents and Settings\\CTLAdmin.CTL\\file123'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub takeown {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count = 0;
    foreach my $line (@lines) {
        if ( $line =~
            /(\w+)\:.*\:\s+\"(.*)\"(.*\"(.*)\"|.*(administrators\s+group))/ ) {
            $result_array[$count]->{status} = $1;
            $result_array[$count]->{file}   = $2;
            $result_array[$count]->{owner}  = $4;
            $result_array[$count]->{owner}  = $5 if ($5);
        }
        $count++;
    }
    return \@result_array;
} ## end sub takeown

1;
