#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse taskkill command on Windows 7
##
## @description
##    This is a library to parse the raw output of command taskkill on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::taskkill
##  
##  taskkill(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::taskkill;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    return $error;
}

## @name ::taskkill
##
## @summary Parses the raw output from the "taskkill" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from taskkill with any of its options as described in the 
##    corresponding cdef file 'taskkill.cdefs'
## @synopsis
##      taskkill(%args);
## INPUTS TO METHOD CALL 'taskkill' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '/F' </td> <td> 'terminate-by-force' </td> </tr> 
## <tr> <td> '/FI' </td> <td> 'filter' </td> </tr> 
## <tr> <td> '/S' </td> <td> 'system-to-connect' </td> </tr> 
## <tr> <td> '/IM' </td> <td> 'image-name' </td> </tr> 
## <tr> <td> '/P' </td> <td> 'password' </td> </tr> 
## <tr> <td> '/T' </td> <td> 'tree-kill' </td> </tr> 
## <tr> <td> '/PID' </td> <td> 'pid' </td> </tr> 
## <tr> <td> '/U' </td> <td> 'user-context' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = taskkill /F /FI "imagename eq notepad.exe"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;SUCCESS: The process with PID 1944 has been terminated.
##&nbsp;SUCCESS: The process with PID 3236 has been terminated.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'SUCCESS: The process with PID 1944 has been terminated.'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'SUCCESS: The process with PID 3236 has been terminated.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = taskkill /F /FI "imagename eq notepad.exe" /PID 123
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;INFO: No tasks running with the specified criteria.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'No tasks running with the specified criteria.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub taskkill {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
	my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
	}
	my @lines = get_lines(raw_output=>$output,first_line=>"");

	foreach my $line (@lines)  {
        my %hash;
		if($command =~ /(\/pid)/i || $command =~ /(\/fi)/i) {
			if($line =~ /(\w+):\s+/){
				if($1 eq "INFO") {
					$hash{'status'} = 'No tasks running with the specified criteria.';
				} else {
					$hash{'status'} = $line;
				}
			}
			push (@result_array, \%hash);
		}
	}
	return \@result_array;			
}

1;
