#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse tasklist command on Windows 7
##
## @description
##    This is a library to parse the raw output of command tasklist on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::tasklist
##  
##  tasklist(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::tasklist;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    return $error;
}


## @name ::tasklist
##
## @summary Parses the raw output from the "tasklist" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from tasklist with any of its options as described in the 
##    corresponding cdef file 'tasklist.cdefs'
## @synopsis
##      tasklist(%args);
## INPUTS TO METHOD CALL 'tasklist' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '/FI' </td> <td> 'filter' </td> </tr> 
## <tr> <td> '/SVC' </td> <td> 'display-services' </td> </tr> 
## <tr> <td> '/S' </td> <td> 'system-to-connect' </td> </tr> 
## <tr> <td> '/P' </td> <td> 'password' </td> </tr> 
## <tr> <td> '/M' </td> <td> 'module-list' </td> </tr> 
## <tr> <td> '/NH' </td> <td> 'no-header' </td> </tr> 
## <tr> <td> '/U' </td> <td> 'user-context' </td> </tr> 
## <tr> <td> '/FO' </td> <td> 'output-format' </td> </tr> 
## <tr> <td> '/V' </td> <td> 'display-verbose-info' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = tasklist /FI "imagename eq notepad.exe"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;Image Name                     PID Session Name        Session#    Mem Usage
##&nbsp;========================= ======== ================ =========== ============
##&nbsp;notepad.exe                   1500 Console                    0      2,212 K
##&nbsp;notepad.exe                    680 Console                    0      1,412 K
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pid' => '1500',
##&nbsp;            'image_name' => 'notepad.exe',
##&nbsp;            'memory_usage' => '2,212K',
##&nbsp;            'session_name' => 'Console',
##&nbsp;            'session_number' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pid' => '680',
##&nbsp;            'image_name' => 'notepad.exe',
##&nbsp;            'memory_usage' => '1,412K',
##&nbsp;            'session_name' => 'Console',
##&nbsp;            'session_number' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub tasklist {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
	my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
	}
	my @lines = get_lines(raw_output=>$output,first_line=>"");

	if($command =~ /(\/nh)/i || $lines[0] =~ /INFO:\s+/) {
	} else {
		shift(@lines);
		shift(@lines);
	}
	foreach my $line (@lines)  {
        my %hash;
		if($command =~ /(\/svc)/i) {
		} else {
			if ($command =~ /(\/fi)/i) {
				if($line =~ /(\w+):\s+/){
					if($1 eq "INFO") {
						$hash{'status'} = 'No tasks are running which match the specified criteria.';
					}
					push (@result_array, \%hash);
				} else {
					my @values= split(' ', $line);
					$hash{'image_name'} = $values[0];
					$hash{'pid'} = $values[1];
					if(@values<6){				
						$hash{'session_name'} = '';
						$hash{'session_number'} = $values[2];
						$hash{'memory_usage'} = $values[3].$values[4];
					} else {
						$hash{'session_name'} = $values[2];
						$hash{'session_number'} = $values[3];
						$hash{'memory_usage'} = $values[4].$values[5];
					}
					push (@result_array, \%hash);
				}
			}
		}
	}
	return \@result_array;
			
}

1;
