#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @name ::transition
##
## @summary Routines to parse the output of the Windows transition command output
##
## @description
##    This is a library to parse the raw output of the Windows transition command
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::transition;
##
##    transition_credentials_add(%args);
##    transition_create(%args);
##    transition_volumepair_add(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_complete(%args);
##    transition_abort(%args);
##    transition_delete(%args);
##    transition_show(%args);
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @status review
#

###################
# Package
package NACL::APISet::Host::CLI::Windows::default::transition;

###################
# Everytime
no strict 'refs';
use NACL::APISet::Response::CLI::ParserUtils;

###################
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::transition_credentials_add
##
## @summary Parses the raw output from the "transition credentials add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_credentials_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition credentials add -h 10.227.202.110 -u root :
##&nbsp;Enter password for 'root@10.227.202.110':
##&nbsp;netapp1!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;
sub transition_credentials_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_credentials_add

## @name ::transition_create
##
## @summary Parses the raw output from the "transition_create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_create with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'c-mode-vserver-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'vserver-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'apply-config' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
##&nbsp;transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true :
##&nbsp; Error(31101) : The session type 'standfsdfalone' is invalid.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31101',
##&nbsp;                           'message' => 'The session type \'standfsdfalone\' is invalid.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition create -s S21 -t standfsdfalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily -a true
##&nbsp;
sub transition_create {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_create

## @name ::transition_volumepair_add
##
## @summary Parses the raw output from the "transition_volumepair_add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_volumepair_add with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;         }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;Error(31248) : Aggregate must not be provided along with the existing Cluster-Mode volume.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 1,
##&nbsp;            'error' => [
##&nbsp;                         {
##&nbsp;                           'code' => '31248',
##&nbsp;                           'message' => 'Aggregate must not be provided along with the existing Cluster-Mode volume.'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans
##&nbsp;
sub transition_volumepair_add {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_volumepair_add

## @name ::transition_precheck
##
## @summary Parses the raw output from the "transition_precheck" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_precheck with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition precheck -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'next_suggested_steps' => '1. Use the \'transition job-results -j 170c0cbe-c5f2-464e-969e-37ec1424ffa0\' command to see the results of this operation. 2. Carefully review the precheck operation results before proceeding with the next step. 3. Select \'Start data copy\' from the main menu or run the transition CLI command \'transition start -s S21\' when you are ready to start data copy to Cluster-Mode volumes.',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition precheck -r no -s S21
##&nbsp;
sub transition_precheck {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end transition_precheck

## @name ::transition_start
##
## @summary Parses the raw output from the "transition_start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_start with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition start -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'informational_count' => '2',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => 'LIFs need to be transitioned manually. For more information, see the Data and Configuration Transition Guide.',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '99162',
##&nbsp;                                   'message' => 'Checking to see if LIF information is added to the session \'S21\'.Data LIFs are not added to the session \'S21\'.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90051',
##&nbsp;                                   'message' => 'Creating the required clustered Data ONTAP volumes on the Vserver \'vs_sec\'.Successfully created the following clustered Data ONTAP volumes on the Vserver \'vs_sec\'.'
##&nbsp;                                 },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '2',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60532',
##&nbsp;                             'message' => 'Checking the \'snapmirror.window_size\' option in the 7-Mode storage system. Ignoring transition of TCP window size configuration for the SnapMirror replications because it is not supported in clustered Data ONTAP.'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'corrective_action' => 'The \'snapmirror.checkip.enable\' option is not supported in clustered Data ONTAP.',
##&nbsp;                             'affected_objects' => '',
##&nbsp;                             'code' => '60542',
##&nbsp;                             'message' => 'Checking the \'snapmirror.checkip.enable\' option in the 7-Mode storage system. Expected value is\'off\'. The \'snapmirror.checkip.enable\' option is enabled on the 7-Mode storage system.'
##&nbsp;                           },
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition start -r no -s S21
##&nbsp;
sub transition_start {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_start

## @name ::transition_pause
##
## @summary Parses the raw output from the "transition_pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_pause with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition pause -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = transition pause -r no -s S21
##&nbsp;
##&nbsp;
sub transition_pause {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_pause

## @name ::transition_resume
##
## @summary Parses the raw output from the "transition_resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_resume with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition resume -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90071',
##&nbsp;                                   'message' => 'Resuming data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume\'vol_compo1\'. Successfully resumed data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition resume -r no -s S21
##&nbsp;
sub transition_resume {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_resume

## @name ::transition_complete
##
## @summary Parses the raw output from the "transition_complete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_complete with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_complete(%args);
## INPUTS TO METHOD CALL 'transition_complete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'safe-mode' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition complete -r no -s S21 -m :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '1',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90811',
##&nbsp;                                   'message' => 'This complete operation is running in safe-mode.'
##&nbsp;                                },
##&nbsp;                               ],
##&nbsp;            'warning_count' => '1',
##&nbsp;            'warning' => [
##&nbsp;                           {
##&nbsp;                             'corrective_action' => '',
##&nbsp;                             'affected_objects' => '{ 10.227.202.109 csiqa-2040-10aError Code: 13130 Reason: duplicate entry , 10.227.202.110 csiqa-2040-10b csiqa-2040-10b-e0c Error Code: 13130 Reason: duplicate entry , 127.0.10.1 localhost-10 localhost-bsd Error Code: 13130 Reason: duplicate entry , 127.0.0.1 localhost localhost-stack Error Code: 13130 Reason: duplicate entry , 127.0.20.1 localhost-20 localhost-sk Error Code: 13130 Reason: duplicate entry }',
##&nbsp;                             'code' => '40111',
##&nbsp;                             'message' => 'Applying the hosts configuration to the Vserver \'vs_sec\'.Failed to create the following hosts configuration on the Vserver \'vs_sec\'.'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition complete -r no -s S21 -m
##&nbsp;
sub transition_complete {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_complete

## @name ::transition_abort
##
## @summary Parses the raw output from the "transition_abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_abort with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition abort -r no -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'informational_count' => '3',
##&nbsp;            'error_count' => 0,
##&nbsp;            'informational' => [
##&nbsp;                                 {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90061',
##&nbsp;                                   'message' => 'Pausing data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'. Successfully paused data transfer between 7-Mode volume \'vol1\' and clustered Data ONTAP volume \'vol_compo1\'.'
##&nbsp;                                 },
##&nbsp;                                {
##&nbsp;                                   'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '',
##&nbsp;                                   'code' => '90011',
##&nbsp;                                   'message' => 'Breaking the SnapMirror relationships. Successfully broken all SnapMirror relationships.'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                  'corrective_action' => '',
##&nbsp;                                   'affected_objects' => '{ vol_compo1 }',
##&nbsp;                                   'code' => '90021',
##&nbsp;                                   'message' => 'Deleting the SnapMirror relationships. Successfully deleted all SnapMirror relationships.'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'warning_count' => 0,
##&nbsp;            'warning' => [],
##&nbsp;            'error' => []
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition abort -r no -s S21
##&nbsp;
sub transition_abort {

    my %args = @_;
    my $result_array=_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_abort

## @name ::transition_delete
##
## @summary Parses the raw output from the "transition_delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_delete with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_count' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition delete -s S21
##&nbsp;
sub transition_delete {

    my %args = @_;
    my $result_array=_no_precheck_type_parser(\%args);
    return $result_array;

} ## end sub transition_delete

## @name ::transition_show
##
## @summary Parses the raw output from the "transition_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from transition_show with any of its options as described
##    in the corresponding cdef file 'transition.cdefs'
## @synopsis
##      transition_show(%args);
## INPUTS TO METHOD CALL 'transition_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'copy-status' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;transition show -r no -s S1 -c :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'error_count' => 0,
##&nbsp;             'vserver_comment' => 'Pre-existing',
##&nbsp;             'cluster_mode_system_fqdn' => '10.226.99.61',
##&nbsp;             'multipath_ip_address' => '-',
##&nbsp;             'data_copy_ip_address' => '10.227.202.109',
##&nbsp;             'snapmirror_status' => [
##&nbsp;                                      {
##&nbsp;                                        'snapmirror_state' => '-/-',
##&nbsp;                                        '7_mode_volume' => 'vol_stag_7cm_16sess60',
##&nbsp;                                        'cluster_mode_volume' => 'vol_stag_7cm_16sess60',
##&nbsp;                                        'comments' => ' * Error: SnapMirror relationship does not exist.'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'snapmirror_state' => '-/-',
##&nbsp;                                        '7_mode_volume' => 'vol_stag_7cm_16sess92',
##&nbsp;                                        'cluster_mode_volume' => 'vol_stag_7cm_16sess92',
##&nbsp;                                        'comments' => ' * Error: SnapMirror relationship does not exist.'
##&nbsp;                                      },
##&nbsp;                                    ],
##&nbsp;             'vfiler_unit' => 'vfiler0',
##&nbsp;             'c_mode_volumes' => [
##&nbsp;                                   'vol_stag_7cm_16sess4',
##&nbsp;                                   'vol_stag_7cm_16sess12',
##&nbsp;                                   'vol_stag_7cm_16sess20',
##&nbsp;                                   'vol_stag_7cm_16sess28',
##&nbsp;                                   'vol_stag_7cm_16sess36',
##&nbsp;                                   'vol_stag_7cm_16sess44',
##&nbsp;                                   'vol_stag_7cm_16sess52',
##&nbsp;                                   'vol_stag_7cm_16sess60',
##&nbsp;                                   'vol_stag_7cm_16sess68',
##&nbsp;                                   'vol_stag_7cm_16sess76',
##&nbsp;                                   'vol_stag_7cm_16sess84',
##&nbsp;                                   'vol_stag_7cm_16sess92',
##&nbsp;                                   'vol_stag_7cm_16sess100',
##&nbsp;                                   'vol_stag_7cm_16sess108',
##&nbsp;                                   'vol_stag_7cm_16sess116',
##&nbsp;                                   'vol_stag_7cm_16sess124',
##&nbsp;                                   'vol_stag_7cm_16sess132',
##&nbsp;                                   'vol_stag_7cm_16sess140',
##&nbsp;                                   'vol_stag_7cm_16sess148',
##&nbsp;                                   'vol_stag_7cm_16sess156'
##&nbsp;                                 ],
##&nbsp;             '7_mode_system_fqdn' => '10.227.202.109',
##&nbsp;             'vserver_data_lifs' => [
##&nbsp;                                      '-'
##&nbsp;                                    ],
##&nbsp;             'snapmirror_schedule' => '-',
##&nbsp;             'session_type' => 'standalone',
##&nbsp;             'session_name' => 'S1',
##&nbsp;             'apply_configuration' => 'Yes',
##&nbsp;             'volume_pairs' => 'vol_stag_7cm_16sess4:vol_stag_7cm_16sess4, vol_stag_7cm_16sess12:vol_stag_7cm_16sess12, vol_stag_7cm_16sess20:vol_stag_7cm_16sess20, vol_stag_7cm_16sess28:vol_stag_7cm_16sess28, vol_stag_7cm_16sess36:vol_stag_7cm_16sess36, vol_stag_7cm_16sess44:vol_stag_7cm_16sess44, vol_stag_7cm_16sess52:vol_stag_7cm_16sess52, vol_stag_7cm_16sess60:vol_stag_7cm_16sess60, vol_stag_7cm_16sess68:vol_stag_7cm_16sess68, vol_stag_7cm_16sess76:vol_stag_7cm_16sess76, vol_stag_7cm_16sess84:vol_stag_7cm_16sess84, vol_stag_7cm_16sess92:vol_stag_7cm_16sess92, vol_stag_7cm_16sess100:vol_stag_7cm_16sess100, vol_stag_7cm_16sess108:vol_stag_7cm_16sess108, vol_stag_7cm_16sess116:vol_stag_7cm_16sess116, vol_stag_7cm_16sess124:vol_stag_7cm_16sess124, vol_stag_7cm_16sess132:vol_stag_7cm_16sess132, vol_stag_7cm_16sess140:vol_stag_7cm_16sess140, vol_stag_7cm_16sess148:vol_stag_7cm_16sess148, vol_stag_7cm_16sess156:vol_stag_7cm_16sess156',
##&nbsp;             'cluster_mode_system_version' => 'NetApp Release 8.2 Cluster-Mode: Tue May 21 05:47:21 PDT 2013',
##&nbsp;             '7_mode_version' => 'NetApp Release 8.1.2 7-Mode: Tue Oct 30 19:56:16 PDT 2012',
##&nbsp;             'vserver_name' => 'vs_sec',
##&nbsp;             'session_status' => 'Copy-unknown',
##&nbsp;             '7_mode_volumes' => [
##&nbsp;                                   'vol_stag_7cm_16sess4',
##&nbsp;                                   'vol_stag_7cm_16sess12',
##&nbsp;                                   'vol_stag_7cm_16sess20',
##&nbsp;                                   'vol_stag_7cm_16sess28',
##&nbsp;                                   'vol_stag_7cm_16sess36',
##&nbsp;                                   'vol_stag_7cm_16sess44',
##&nbsp;                                   'vol_stag_7cm_16sess52',
##&nbsp;                                   'vol_stag_7cm_16sess60',
##&nbsp;                                   'vol_stag_7cm_16sess68',
##&nbsp;                                   'vol_stag_7cm_16sess76',
##&nbsp;                                   'vol_stag_7cm_16sess84',
##&nbsp;                                   'vol_stag_7cm_16sess92',
##&nbsp;                                   'vol_stag_7cm_16sess100',
##&nbsp;                                   'vol_stag_7cm_16sess108',
##&nbsp;                                   'vol_stag_7cm_16sess116',
##&nbsp;                                   'vol_stag_7cm_16sess124',
##&nbsp;                                   'vol_stag_7cm_16sess132',
##&nbsp;                                   'vol_stag_7cm_16sess140',
##&nbsp;                                   'vol_stag_7cm_16sess148',
##&nbsp;                                   'vol_stag_7cm_16sess156'
##&nbsp;                                 ]
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no -s S1 -c
##&nbsp;
##&nbsp;transition show -r no -c :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.202.110',
##&nbsp;            'cluster_mode_system_fqdn' => '10.226.99.61',
##&nbsp;            'vserver_name' => 'vs_sec',
##&nbsp;            'session_status' => 'Copy-update',
##&nbsp;            'session_name' => 'R_1379479156',
##&nbsp;            'vfiler_unit' => 'vfiler0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.202.109',
##&nbsp;            'cluster_mode_system_fqdn' => '10.226.99.61',
##&nbsp;            'vserver_name' => 'vs_sec',
##&nbsp;            'session_status' => 'Copy-unknown',
##&nbsp;            'session_name' => 'S1',
##&nbsp;            'vfiler_unit' => 'vfiler0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;        ],
##&nbsp;
##&nbsp;Command = transition show -r no -c
##&nbsp;
##&nbsp;transition show -r no:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.202.110',
##&nbsp;            'cluster_mode_system_fqdn' => '10.226.99.61',
##&nbsp;            'vserver_name' => 'vs_sec',
##&nbsp;            'session_status' => 'Copy-update',
##&nbsp;            'session_name' => 'R_1379479156',
##&nbsp;            'vfiler_unit' => 'vfiler0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'session_type' => 'standalone',
##&nbsp;            '7_mode_system_fqdn' => '10.227.202.109',
##&nbsp;            'cluster_mode_system_fqdn' => '10.226.99.61',
##&nbsp;            'vserver_name' => 'vs_sec',
##&nbsp;            'session_status' => 'Copy-unknown',
##&nbsp;            'session_name' => 'S1',
##&nbsp;            'vfiler_unit' => 'vfiler0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'error_count' => 0,
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = transition show -r no
##&nbsp;
##&nbsp;
##&nbsp;Command = transition show -r no -c
##&nbsp;
##&nbsp;
sub transition_show {

    my %args = @_;
    my ($first_line, $vol7mode, $volcmode,  $snapmirror,
        $comments,   $temp,     $aggregate, $cmodestatus,
        %hash,       %entry,    %comment,   $error_message,
    );
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    my $result_ref=[];
    if (!$error){
        if ($output =~ /SnapMirror schedule/){
            my @summary_lines = get_lines(
                raw_output      => $output,
                response_object => $resp,
                first_line      => "SUMMARY:"
            );

            #remove summary lines from the output
            foreach my $line (@summary_lines) {
                $output =~ s/\Q$line\E//g;
            }
            if ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments';
            } elsif ( $output
                =~ /7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status/
                )
            {
                $first_line
                    = '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status';
            }

            #grep snapmirror status lines from output and parse the fields
            $output =~ m/$first_line(.*)/si;
            my $snapmirror_string = $1 if $1;
            my @snapmirror_status_lines = split( "\n", $snapmirror_string );
            foreach my $line (@snapmirror_status_lines) {
                if ( $line =~ /(^\w+)/ ) {
                    $temp = $1;
                    $entry{$temp} = $line;
                } elsif ( $line =~ /^\s+/ ) {
                    $entry{$temp} = $entry{$temp} . $line;
                }
            }
            foreach my $en ( keys(%entry) ) {
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Snapmirror state\s+Comments'
                    )
                {
                    ( $vol7mode, $volcmode, $snapmirror, $comments )
                        = split( /\s+/, $entry{$en}, 4 );
                    if (    defined $vol7mode
                        and defined $volcmode
                        and defined $snapmirror
                        and defined $comments )
                    {
                        #remove extra spaces from comments field
                        $comments =~ s/\s{2,}//g;
                        $comments =~ s/\*/ */g;
                        push(
                            @{ $hash{snapmirror_status} },
                            {   '7_mode_volume'       => $vol7mode,
                                'cluster_mode_volume' => $volcmode,
                                'snapmirror_state'    => $snapmirror,
                                'comments'            => $comments
                            }
                        ) if ( $vol7mode || $volcmode || $snapmirror || $comments);
                    }
                }
                if ( $first_line eq
                    '7-Mode volume\s+Cluster-Mode volume\s+Aggregate\s+Snapmirror state\s+Cluster-Mode volume status'
                    )
                {
                    ( $vol7mode, $volcmode, $aggregate, $snapmirror, $cmodestatus )
                        = split( /\s+/, $entry{$en} );
                    push(
                        @{ $hash{snapmirror_status} },
                        {   '7_mode_volume'              => $vol7mode,
                            'cluster_mode_volume'        => $volcmode,
                            'snapmirror_state'           => $snapmirror,
                            'aggregate'                  => $aggregate,
                            'cluster_mode_volume_status' => $cmodestatus
                        }
                    ) if ( $vol7mode || $volcmode || $snapmirror
                        || $comments || $aggregate );
                }
            }

            #remove snapmirror status lines from output
            $output =~ s/($first_line.*)//s if $first_line;

            #parse rest of the show output
            my @lines = get_lines(
                raw_output      => $output,
                response_object => $resp
            );
            foreach my $line (@lines) {
                if ( $line !~ /----------/ ) {
                    my ( $key, $val ) = split( ':', $line, 2 );
                    trim( string_ptr => \$key );
                    trim( string_ptr => \$val );
                    $key =~ s/\s/_/g;
                    if(_upper_to_lower($key) eq "vserver_information"){
                        ($hash{"vserver_name"},$hash{vserver_comment})=split(/:/,$val);
                    }elsif(_upper_to_lower($key) eq "vserver_data_lifs"){
                        my @t=split(/,/,$val);
                        $hash{"vserver_data_lifs"} = \@t;
                    }elsif(_upper_to_lower($key) eq "volume_pairs"){
                        my @c7=split(/,/,$val);
                        my (@m7,@mc);
                        foreach (0..@c7-1){
                            ($m7,$mc)=split(/:/,$c7[$_]);
                            $mc='-' if(not defined $mc);
                            $m7='-' if(not defined $m7);
                            push (@m7,_formatter($m7)) ;
                            push (@mc,_formatter($mc));
                        }
                        $hash{"7_mode_volumes"} = \@m7;
                        $hash{"c_mode_volumes"} = \@mc;
                        $hash{_upper_to_lower($key)} = $val;
                    }else{
                        $hash{_upper_to_lower($key)} = $val;
                    }
                }
            }
        }elsif($output =~ /7-Mode system/){
            my (@arr,%chunk_hash,%session_hash);
            @lines=split(/\n/,$output);
            foreach my $line (@lines){
                next if($line =~ /Name/ || $line =~ /---/ || $line !~ /\w+/) ;
                @arr = split(/\s+/,$line);
                $chunk_hash{"session_name"}=$arr[0];
                $chunk_hash{"session_type"}=$arr[1];
                ($chunk_hash{"7_mode_system_fqdn"},$chunk_hash{"vfiler_unit"})
                    =split(/:/,$arr[2]);
                ($chunk_hash{"cluster_mode_system_fqdn"},$chunk_hash{"vserver_name"})
                    =split(/:/,$arr[3]);
                $chunk_hash{"session_status"}=$arr[4];
                push @$result_ref , {%chunk_hash} ;
            }
        }
    }

    push @{$result_ref} , {%hash};

    if($error) {
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>$result_ref);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Reason to be updated in API library."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }
    return $result_ref;

} ## end sub transition_show

sub _upper_to_lower{

    my $string=shift;
    $string =~ tr/A-Z/a-z/;
    $string =~ s/-/_/;
    return $string;

} ## end sub _upper_to_lower

sub _formatter{

    my $string=shift;
    $string=~s/^\s+|\s+$//g;
    return $string;

} ## end sub _formatter

sub _get_count{

    my $string=shift;
    my $var;
    $var=$2 if shift =~ /((\d+)\s+$string)/;
    if (!defined $var){
        $var=0;
    }
    return $var;

} ## end sub _get_count

sub _check_error {

    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $hash_ref=$args{hash};
    my %hash=%{$hash_ref};
    my $error;
    if($output =~ /No value specified for .*/) {
        $error = "MissingRequiredParam";
    } elsif ($output =~ /The .* command takes .* arguments. You gave .*.  Extra arguments: .*/) {
        $error = "InvalidParam";
    } elsif ($output =~ /Transition session with name .* does not exist/ ||
        $output =~ /The session name .* is invalid. Special characters .* are not allowed in the session name/ ||
        $output =~ /.* storage system volume .*' already used in the transition session .*/){
        $error = "InvalidParamValue";
    } elsif ($output =~ /(Operation summary.*?(\d+)\s+Error.*)/is && $2>0
            || $output =~ /.*Failed to run the.*/
            || $output =~ /Transition session.*status is not.*/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(SYNOPSIS)/ || $output =~ /session with name .* already exists/
        || $output =~ /Aggregate must not be provided along with the existing Cluster-Mode volume/
        || $output =~ /.*must be provided.*]/
        || $output =~ /Failed to communicate with 7-Mode Transition Tool server/
        || $output =~ /.*not recognized as an internal or external command.*/
        || $output =~ /Host .* not found in credential bank/
        || $output =~ /There are no sessions/
        || $output =~ /Volume.*does not exists.*/
        || $output =~ /Credentials provided for host.* are invalid/
        ){
        $error = "CommandFailed";
    }

    $$error_message = $output;

    # This is used to catch the errors which went unnoticed above
    if ($output =~ /(Error)(?:(\([-]?\d+\)))?(.*)/ &&
        $output !~ /Name.*Type.*7-Mode system.*Cluster-Mode system.*Status/is
        && $output !~ /Apply configuration.*7-Mode system FQDN/is &&
        $output !~ /Operation summary/is ){
        my $a1=$1 if $1;
        my $a2=$2 if $2;
        my $a3=$3 if $3;
        $a2=~ s/\(//;
        $a2=~ s/\)//;
        $a3=~ s/:\s//;
        my $member=$hash{_upper_to_lower($a1)}=[];
        my %chunk_hash;
        $chunk_hash{"code"}=$a2;
        $chunk_hash{"message"}=_formatter($a3);
        push @$member , {%chunk_hash};
        $hash{"error_count"}=1;
    }else{
        $hash{"error_count"}=0;
    }

    %$hash_ref=%hash;

    return $error;

} ## end sub _check_error

sub _no_precheck_type_parser{

    my $ref=shift;
    my ($error_message , @result_array , %hash , $resp , $output , %args );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );

    push (@result_array, \%hash);

    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Reason to be updated in API library."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return \@result_array;

} ## end sub _no_precheck_type_parser

sub _precheck_type_parser{

    my $ref=shift;
    my ($error_message, @result_array, $resp,       $output, %args,
        @chunks,        $last_chunk,   $op_string,  %hash,   $first_line,
        @lines,         $error
    );
    %args=%{$ref};
    $resp = $args{response_object};
    $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output=>$output,
        error_message=>\$error_message,
        hash=>\%hash,
    );
    $first_line = '.*';
    @lines = get_lines(raw_output=>$output, first_line=>$first_line);

    # Create a string version from Array of o/p Lines
    $op_string = join (" ", @lines);
    #Get the last Operation summary chunk separate from chunk array
    $last_chunk=$1 if $op_string =~ s/(Operation summary.*)//gi;
    $op_string =~ s/(Job with.*has started. )//gi;

    #Prepare an array of chunks one for each error code
    my $i=-1;
    do{
        $i++;
        push(@chunks,$1) if $op_string =~ s/^(\w{0,4}\d{1,5}:\s.*?)(\w{0,4}\d{1,5}:\s)/$2/g ;
    }while(defined $chunks[$i]);
    push(@chunks,$op_string);

    $hash{"error"}=[];
    $hash{"warning"}=[];
    $hash{"informational"}=[];

    #Format the chunks and push them into the complex data structure
    foreach my $i (0..@chunks-1){
        my (%chunk_hash,@msg,$chunk_type,$chunk_code,$members);
        $chunks[$i] =~ s/\d+:$//g;
        $chunks[$i] =~ s/\s+> / /g;
        $chunk_type=$1 if $chunks[$i] =~ s/\[\s+(\w+)\s+\]//g;
        $chunks[$i] =~ s/  //g;
        $chunk_code=$1 if $chunks[$i] =~ s/^(\w{0,4}\d{1,5})://g;
        $chunk_hash{"code"}=$chunk_code;
        if($chunks[$i] =~ /CORRECTIVE-ACTION/
           && $chunks[$i] !~ /AFFECTED-OBJECTS/) {
            @msg=split(/CORRECTIVE-ACTION:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"corrective_action"}=_formatter($msg[1]);
            $chunk_hash{"affected_objects"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] !~ /CORRECTIVE-ACTION/){
            @msg=split(/AFFECTED-OBJECTS:/,$chunks[$i]);
            $chunk_hash{"message"}=_formatter($msg[0]);
            $chunk_hash{"affected_objects"}=_formatter($msg[1]);
            $chunk_hash{"corrective_action"}="";
        }elsif($chunks[$i] =~ /AFFECTED-OBJECTS/
               && $chunks[$i] =~ /CORRECTIVE-ACTION/){
            $chunks[$i] =~ s/\s  //g;
            $chunk_hash{"affected_objects"}=_formatter($1)
                if $chunks[$i] =~ s/AFFECTED-OBJECTS:(.*)//gims;
            $chunk_hash{"corrective_action"}=_formatter($1)
                if $chunks[$i] =~ s/CORRECTIVE-ACTION:(.*)//gims;
            $chunk_hash{"message"}=_formatter($chunks[$i]);
        }else{
            $chunk_hash{"message"}=_formatter($chunks[$i]);
            $chunk_hash{"affected_objects"}="";
            $chunk_hash{"corrective_action"}="";
        }
        $members=$hash{_upper_to_lower($chunk_type)};
        push @$members , {%chunk_hash};

    }
    my @keys = keys (%hash);

    #Push the last chunk onto the complex data structure alongwith other details
    $hash{"error_count"}=_get_count("Errors",$last_chunk) ;
    $hash{"warning_count"}=_get_count("Warnings",$last_chunk) ;
    $hash{"informational_count"}=_get_count("Informational messages",$last_chunk);
    $hash{"next_suggested_steps"}=_formatter($1) if $last_chunk =~ /Next suggested steps: [-]+(.*)(Ran.*)/;

    #Clear  empty keys
    foreach (@keys){
        delete $hash{$_} if (!$_);
    }

    push (@result_array, \%hash);

    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if($hash{"error_count"}==1){
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw(text=>"Command failed.Reason to be updated in API library."
        ."Report this to dl-merlin2-migqa.\n Original output:\n $error_message",response_object => $resp);
    }

    return \@result_array;

} ## end sub _precheck_type_parser

1;
