#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse the output of the Windows typeperf command output
##
## @description
##    This is a library to parse the raw output of the Windows typeperf command
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::typeperf;
##
##    net_use(%args);
##
## @author Abhishek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::typeperf;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::typeperf
##
## @summary Parses the raw output from the "typeperf" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from typeperf with any of its options as described in the 
##    corresponding cdef file 'typeperf.cdefs'
## @synopsis
##      typeperf(%args);
## INPUTS TO METHOD CALL 'typeperf' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-y' </td> <td> 'yes-to-all' </td> </tr> 
## <tr> <td> '-si' </td> <td> 'time' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'computer-name' </td> </tr> 
## <tr> <td> '-qx' </td> <td> 'object-with-instance' </td> </tr> 
## <tr> <td> '-config' </td> <td> 'config-file' </td> </tr> 
## <tr> <td> '-cf' </td> <td> 'counter-file' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'file-format' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'object-without-instance' </td> </tr> 
## <tr> <td> '-sc' </td> <td> 'samples' </td> </tr> 
## <tr> <td> '-o' </td> <td> 'output-file' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = typeperf "Processor(_Total)\% Processor Time" "Processor(_Total)\DPC Rate" -sc 4
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;"(PDH-CSV 4.0)","\\VMWIN64-190\Processor(_Total)\% Processor Time","\\VMWIN64-190\Processor(_Total)\DPC Rate"
##&nbsp;"08/10/2011 16:39:21.912","0.390620","0.000000"
##&nbsp;"08/10/2011 16:39:22.912","0.000000","0.000000"
##&nbsp;"08/10/2011 16:39:23.912","0.781250","0.000000"
##&nbsp;"08/10/2011 16:39:24.912","0.390630","0.000000"
##&nbsp;Exiting please wait...                        
##&nbsp;The command completed successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_stamp' => '"08/10/2011 16:39:21.912"',
##&nbsp;            'value-counter2' => '"0.000000"',
##&nbsp;            'value-counter1' => '"0.390620"'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time_stamp' => '"08/10/2011 16:39:22.912"',
##&nbsp;            'value-counter2' => '"0.000000"',
##&nbsp;            'value-counter1' => '"0.000000"'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time_stamp' => '"08/10/2011 16:39:23.912"',
##&nbsp;            'value-counter2' => '"0.000000"',
##&nbsp;            'value-counter1' => '"0.781250"'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'time_stamp' => '"08/10/2011 16:39:24.912"',
##&nbsp;            'value-counter2' => '"0.000000"',
##&nbsp;            'value-counter1' => '"0.390630"'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub typeperf {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
	my $command = $resp->get_actual_command();
    my @result_array;
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                                       response_object => $resp);
    }
	my @lines = get_lines(raw_output=>$output,first_line=>'\"\(PDH.*\"');
	shift(@lines);
	foreach my $line (@lines)  {
        my %hash;
		my @values= split(',', $line);
		my $count=@values;
		$hash{'time_stamp'} = $values[0];
		while($count>1){
			$count--;
			$hash{"value-counter$count"}    = $values[$count];
		}
		last if($line=~/Exiting please wait.../);
		push (@result_array, \%hash);
	}
    return \@result_array;
}

1;
