#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux cd command
##
## @description
##    This is a library to parse the raw output of Linux cd command.
##
## @synopsis
##    use cd;
##
##    cd(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::cd;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::cd
##
## @summary Parses the raw output from the "cd" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cd with any of its options as described in the
##    corresponding cdef file 'cd.cdefs'
## @synopsis
##      cd(%args);
## INPUTS TO METHOD CALL 'cd' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>

## The below options are supported in Linux and FreeBsd
## <tr> <td> '-' </td> <td> 'old-path' </td> <td> '1' </td> </tr>

## The below options are supported in Linux,Solaris and FreeBsd
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>

## The below options support Linux and Solaris
## <tr> <td> '-L' </td> <td> 'linkfollow' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'physical-dir' </td> <td> '1' </td> </tr>

## The below options support FreeBsd only
## <tr> <td> '-p' </td> <td> 'physical-directory' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'link-follow' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'newest' </td> <td> '1' </td> </tr>

## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub cd {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub cd

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /.*Permission\s+denied/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No\s+such\s+file\s+or\s+directory.*|Too\s+many\s+arguments.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|cd:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
