#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse crud command
##
## @description
##    This is a library to parse the raw output of crud command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::crud;
##
##    crud(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::crud;

use strict;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Include exceptions which will be thrown
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::TimeoutException;

sub _check_error {
    my %args          = @_;
    my $resp          = $args{response_object};
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = undef;
    my $error         = undef;
    if ( $raw_output =~ /ERROR:/ ) {
        $error = "CommandFailed";
    }
    elsif ( $raw_output =~ /crud: illegal option.*/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $raw_output =~ /Timeout.*/ ) {
        $error = "Timeout";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
}

## @name ::crud
##
## @summary Parses the raw output from the "crud" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from crud with any of its options as described in the
##    corresponding cdef file 'crud.cdefs'
## @synopsis
##      crud(%args);
## INPUTS TO METHOD CALL 'crud' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-S' </td> <td> 'S' </td> <td> '1' </td> </tr>
## <tr> <td> '--verbose' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '--read_size_min' </td> <td> 'read_size_min' </td> <td> '0' </td> </tr>
## <tr> <td> '--read_size' </td> <td> 'read_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'T' </td> <td> '0' </td> </tr>
## <tr> <td> '--max_hl' </td> <td> 'max_hl' </td> <td> '0' </td> </tr>
## <tr> <td> '-7' </td> <td> '7' </td> <td> '0' </td> </tr>
## <tr> <td> '--write_size' </td> <td> 'write_size' </td> <td> '0' </td> </tr>
## <tr> <td> '--write_size_max' </td> <td> 'write_size_max' </td> <td> '0' </td> </tr>
## <tr> <td> '-2' </td> <td> '2' </td> <td> '0' </td> </tr>
## <tr> <td> '--read_offset_min' </td> <td> 'read_offset_min' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'y' </td> <td> '0' </td> </tr>
## <tr> <td> '-1' </td> <td> '1' </td> <td> '0' </td> </tr>
## <tr> <td> '--closedelete' </td> <td> 'closedelete' </td> <td> '1' </td> </tr>
## <tr> <td> '--timeout' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '--write_offset_min' </td> <td> 'write_offset_min' </td> <td> '0' </td> </tr>
## <tr> <td> '--stripe_width_min' </td> <td> 'stripe_width_min' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 't' </td> <td> '0' </td> </tr>
## <tr> <td> '--read_offset' </td> <td> 'read_offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-W' </td> <td> 'W' </td> <td> '0' </td> </tr>
## <tr> <td> '--locking' </td> <td> 'locking' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'v' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 's' </td> <td> '0' </td> </tr>
## <tr> <td> '--interval' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'c' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'z' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'w' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'r' </td> <td> '0' </td> </tr>
## <tr> <td> '--stripe_width_max' </td> <td> 'stripe_width_max' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'x' </td> <td> '0' </td> </tr>
## <tr> <td> '--seed' </td> <td> 'seed' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'h' </td> <td> '1' </td> </tr>
## <tr> <td> '-0' </td> <td> '0' </td> <td> '0' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '--write_offset' </td> <td> 'write_offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'f' </td> <td> '0' </td> </tr>
## <tr> <td> '--stripe_width' </td> <td> 'stripe_widht' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'i' </td> <td> '0' </td> </tr>
## <tr> <td> '-6' </td> <td> '6' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'n' </td> <td> '0' </td> </tr>
## <tr> <td> '-3' </td> <td> '3' </td> <td> '0' </td> </tr>
## <tr> <td> '--style' </td> <td> 'style' </td> <td> '0' </td> </tr>
## <tr> <td> '-9' </td> <td> '9' </td> <td> '0' </td> </tr>
## <tr> <td> '--write_size_min' </td> <td> 'write_size_min' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'm' </td> <td> '0' </td> </tr>
## <tr> <td> '--threads' </td> <td> 'threads' </td> <td> '0' </td> </tr>
## <tr> <td> '-8' </td> <td> '8' </td> <td> '0' </td> </tr>
## <tr> <td> '--write_offset_max' </td> <td> 'write_offset_max' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'l' </td> <td> '1' </td> </tr>
## <tr> <td> '-4' </td> <td> '4' </td> <td> '0' </td> </tr>
## <tr> <td> '--max_sl' </td> <td> 'max_sl' </td> <td> '0' </td> </tr>
## <tr> <td> '--filename' </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '--read_offset_max' </td> <td> 'read_offset_max' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'R' </td> <td> '0' </td> </tr>
## <tr> <td> '--min_hl' </td> <td> 'min_hl' </td> <td> '0' </td> </tr>
## <tr> <td> '--read_size_max' </td> <td> 'read_size_max' </td> <td> '0' </td> </tr>
## <tr> <td> '--min_sl' </td> <td> 'min_sl' </td> <td> '0' </td> </tr>
## <tr> <td> '-5' </td> <td> '5' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /u/rawat/p4/test/cluster/fs/crud/bin/linux/crud -f sample -t 10
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Fri Jun 13 06:15:14 2014: Using the following parameters:
##&nbsp;Fri Jun 13 06:15:14 2014:       run type.............crud
##&nbsp;Fri Jun 13 06:15:14 2014:       interval.............0
##&nbsp;Fri Jun 13 06:15:14 2014:       verbose..............0
##&nbsp;Fri Jun 13 06:15:14 2014:       locking..............0
##&nbsp;Fri Jun 13 06:15:14 2014:       seed.................1402654514
##&nbsp;Fri Jun 13 06:15:14 2014:       filename.............sample
##&nbsp;Fri Jun 13 06:15:14 2014:       thread count.........32
##&nbsp;Fri Jun 13 06:15:14 2014:       timeout..............10
##&nbsp;Fri Jun 13 06:15:14 2014:       stripe width(fixed)..67108864
##&nbsp;Fri Jun 13 06:15:14 2014:       write offset.........4096
##&nbsp;Fri Jun 13 06:15:14 2014:       write size...........8192
##&nbsp;Fri Jun 13 06:15:14 2014:       read offset..........4096
##&nbsp;Fri Jun 13 06:15:14 2014:       read size............8192
##&nbsp;Fri Jun 13 06:15:14 2014:       min hardlinks........5
##&nbsp;Fri Jun 13 06:15:14 2014:       max hardlinks........50
##&nbsp;Fri Jun 13 06:15:14 2014:       min symlinks........5
##&nbsp;Fri Jun 13 06:15:14 2014:       max symlinks........50
##&nbsp;Fri Jun 13 06:15:24 2014: Ending stats:
##&nbsp;Fri Jun 13 06:15:24 2014:
##&nbsp;Fri Jun 13 06:15:24 2014: Fileop counts:
##&nbsp;Fri Jun 13 06:15:24 2014:                close: 3243
##&nbsp;Fri Jun 13 06:15:24 2014:                fsync: 12972
##&nbsp;Fri Jun 13 06:15:24 2014:            ftruncate: 3243
##&nbsp;Fri Jun 13 06:15:24 2014:                 open: 3243
##&nbsp;Fri Jun 13 06:15:24 2014:                 link: 0
##&nbsp;Fri Jun 13 06:15:24 2014:           lock(excl): 0
##&nbsp;Fri Jun 13 06:15:24 2014:         lock(shared): 0
##&nbsp;Fri Jun 13 06:15:24 2014:               rename: 0
##&nbsp;Fri Jun 13 06:15:24 2014:                 seek: 12972
##&nbsp;Fri Jun 13 06:15:24 2014:               unlink: 3243
##&nbsp;Fri Jun 13 06:15:24 2014:               unlock: 0
##&nbsp;Fri Jun 13 06:15:24 2014:                write: 12972
##&nbsp;Fri Jun 13 06:15:24 2014:              symlink: 0
##&nbsp;Fri Jun 13 06:15:24 2014:
##&nbsp;Fri Jun 13 06:15:24 2014: Error counts:
##&nbsp;Fri Jun 13 06:15:24 2014:         close errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:         fsync errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:     ftruncate errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:          link errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:          lock errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:          open errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:        rename errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:          seek errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:        unlink errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:        unlock errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:         write errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014:       symlink errors: 0
##&nbsp;Fri Jun 13 06:15:24 2014: program complete, calling exit.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'verbose' => '0',
##&nbsp;            'max_symlinks' => '50',
##&nbsp;            'read_size' => '8192',
##&nbsp;            'fileop_counts' => [
##&nbsp;                                 {
##&nbsp;                                   'link' => '0',
##&nbsp;                                   'unlock' => '0',
##&nbsp;                                   'lock_excl' => '0',
##&nbsp;                                   'lock_shared' => '0',
##&nbsp;                                   'unlink' => '3243',
##&nbsp;                                   'open' => '3243',
##&nbsp;                                   'rename' => '0',
##&nbsp;                                   'close' => '3243',
##&nbsp;                                   'ftruncate' => '3243',
##&nbsp;                                   'symlink' => '0',
##&nbsp;                                   'fsync' => '12972',
##&nbsp;                                   'write' => '12972',
##&nbsp;                                   'seek' => '12972'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'write_size' => '8192',
##&nbsp;            'seed' => '1402654514',
##&nbsp;            'min_symlinks' => '5',
##&nbsp;            'timeout' => '10',
##&nbsp;            'max_hardlinks' => '50',
##&nbsp;            'write_offset' => '4096',
##&nbsp;            'read_offset' => '4096',
##&nbsp;            'error_counts' => [
##&nbsp;                                {
##&nbsp;                                  'unlock_errors' => '0',
##&nbsp;                                  'fsync_errors' => '0',
##&nbsp;                                  'link_errors' => '0',
##&nbsp;                                  'rename_errors' => '0',
##&nbsp;                                  'lock_errors' => '0',
##&nbsp;                                  'unlink_errors' => '0',
##&nbsp;                                  'ftruncate_errors' => '0',
##&nbsp;                                  'close_errors' => '0',
##&nbsp;                                  'open_errors' => '0',
##&nbsp;                                  'symlink_errors' => '0',
##&nbsp;                                  'write_errors' => '0',
##&nbsp;                                  'seek_errors' => '0'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'locking' => '0',
##&nbsp;            'thread_count' => '32',
##&nbsp;            'interval' => '0',
##&nbsp;            'stripe_width_fixed' => '67108864',
##&nbsp;            'min_hardlinks' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub crud {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    _check_error(
        "response_object" => $resp,
        "raw_output"      => $output,
        "command"         => $command,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash,   @result_array );
    my ( @array1, @array2 );
    my $continue_flag = 0;

    while (@lines) {
        my $line = shift(@lines);
        chomp($line);
        if ( $line =~ /Using\s+the\s+following\s+parameters:.*/i ) {
            while ( !( $line =~ /Ending\s+stats.*/i ) ) {
                $line = shift(@lines);
                if ( $line =~ /([\s\w\s]*[\(\s\w\s\)]*)\.\.+([\d]+)/ ) {
                    my $key   = $1;
                    my $value = $2;
                    $key =~ s/^\s+//;
                    $key =~ s/\(/_/g;
                    $key =~ s/\)//g;
                    $hash{ to_key_name( item => $key ) } = $value;
                }
            }
            unshift( @lines, $line );
        }
        elsif ( $line =~ /Fileop\s+counts:.*/i ) {
            while ( !( $line =~ /Error\s+counts:.*/i ) ) {
                $line = shift(@lines);
                if ( $line =~ /([\s\w\s]*[\(\s\w\s\)]*):\s+([\d]+)$/ ) {
                    my $key   = $1;
                    my $value = $2;
                    $key =~ s/^\s+//;
                    $key =~ s/\(/_/g;
                    $key =~ s/\)//g;
                    $array1[0]->{ to_key_name( item => $key ) } = $value;
                }
            }
            $hash{"fileop_counts"} = \@array1;
            unshift( @lines, $line );
        }
        elsif ( $line =~ /Error\scounts:.*/i ) {
            while ( !( $line =~ /calling\s+exit.*/ ) ) {
                $line = shift(@lines);
                if ( $line =~ /([\s\w\s]*[\(\s\w\s\)]*):\s+([\d]+)$/ ) {
                    my $key   = $1;
                    my $value = $2;
                    $key =~ s/^\s+//;
                    $key =~ s/\(/_/g;
                    $key =~ s/\)//g;
                    $array2[0]->{ to_key_name( item => $key ) } = $value;
                }
            }
            $hash{"error_counts"} = \@array2;
            shift(@lines);
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}
1;

