#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/dwim.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux dwim command output
##
## @description
##    This is a library to parse the raw output of Linux dwim command.
##
## @synopsis
##    use dwim;
##
##    dwim(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::dwim;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    my $error_message;
    if ( $raw_output =~ /(Usage|dwim):\s(.*)/ ) {
        $error_message = $1;
        $error         = "CommandFailed";
        if ( $raw_output =~ /(invalid)/ ) {
            $error_message = $1;
            $error         = "InvalidParam";
        } elsif ( $raw_output =~ /(requires an argument)/ ) {
            $error         = "InvalidParamValue";
            $error_message = $1;
        }
    } elsif ( $raw_output =~ /(could not open file)/ ) {
        $error_message = $1;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

    return $error;
} ## end sub _check_error

## @name ::dwim
##
## @summary Parses the raw output from the "dwim" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dwim with any of its options as described in the 
##    corresponding cdef file 'dwim.cdefs'
## @synopsis
##      dwim(%args);
## INPUTS TO METHOD CALL 'dwim' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-U' </td> <td> 'rand' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'num-clients' </td> <td> '0' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'time' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'size' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr> 
## <tr> <td> '-C' </td> <td> 'server-ip' </td> <td> '0' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'read' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'throughput' </td> <td> '0' </td> </tr> 
## <tr> <td> '-q' </td> <td> 'qdepth' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = dwim -t 10 hamlog.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[dwim:v1.49]
##&nbsp;Target files: 1. Size (59847B) (58.44 KB)
##&nbsp;Limiting max qdepth to 2048
##&nbsp;Run time: 10
##&nbsp;Target iops: 100
##&nbsp;Block size: 4096 (4K)
##&nbsp;Read %: 100
##&nbsp;Random %: 100
##&nbsp;
##&nbsp;Time(s)		TPUT(iops)	Lat.(ms)	MaxQdepth :: Tue Jun 12 14:21:18 2012
##&nbsp;    1		   103		    0.707	      4
##&nbsp;    2		   100		    0.130	      1
##&nbsp;    3		   100		    1.111	      4
##&nbsp;    4		   100		    0.126	      1
##&nbsp;    5		   100		    0.125	      1
##&nbsp;    6		   101		    0.125	      1
##&nbsp;    7		   100		    0.125	      1
##&nbsp;    8		   100		    0.125	      1
##&nbsp;    9		   100		    0.212	      1
##&nbsp;   10		    97		    0.126	      1
##&nbsp;
##&nbsp;Threads have terminated. About to destroy AIO context
##&nbsp;
##&nbsp;Run time (s): 10.025
##&nbsp;Avg. TPUT (iops): 99.853
##&nbsp;Avg. TPUT (kbps): 399.410
##&nbsp;Avg. RT     (ms): 0.293
##&nbsp;Max. RT     (ms): 39.616
##&nbsp;
##&nbsp;Average latency of lowest 98.70% latencies: 0.15ms
##&nbsp;Avg qdepth : 0.029
##&nbsp;Max qdepth : 4 (2048)
##&nbsp;Percent target throughput achieved: 99.853
##&nbsp;Dwim workload complete!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result_summary' => {
##&nbsp;                                  'runtime_in_secs' => '10.025',
##&nbsp;                                  'max_qdepth' => '4 (2048)',
##&nbsp;                                  'avg_latency' => [
##&nbsp;                                                     {
##&nbsp;                                                       'lowest_percentage' => '98.70%',
##&nbsp;                                                       'latency' => '0.15ms'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                  'percent_target_throughput_achieved' => '99.853',
##&nbsp;                                  'avg_qdepth' => '0.029',
##&nbsp;                                  'avg_tput_kbps' => '399.410',
##&nbsp;                                  'avg_rt_ms' => '0.293',
##&nbsp;                                  'avg_tput_iops' => '99.853',
##&nbsp;                                  'max_rt_ms' => '39.616'
##&nbsp;                                },
##&nbsp;            'version' => '1.49',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '103',
##&nbsp;                           'max_qdepth_for_sec' => '4',
##&nbsp;                           'time_in_secs' => '1',
##&nbsp;                           'lat_ms' => '0.707'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '2',
##&nbsp;                           'lat_ms' => '0.130'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '4',
##&nbsp;                           'time_in_secs' => '3',
##&nbsp;                           'lat_ms' => '1.111'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '4',
##&nbsp;                           'lat_ms' => '0.126'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '5',
##&nbsp;                           'lat_ms' => '0.125'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '101',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '6',
##&nbsp;                           'lat_ms' => '0.125'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '7',
##&nbsp;                           'lat_ms' => '0.125'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '8',
##&nbsp;                           'lat_ms' => '0.125'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '9',
##&nbsp;                           'lat_ms' => '0.212'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '97',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '10',
##&nbsp;                           'lat_ms' => '0.126'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'inputs' => {
##&nbsp;                          'target_iops' => '100',
##&nbsp;                          'file_size' => '59847B',
##&nbsp;                          'target_files' => '1',
##&nbsp;                          'runtime_in_secs' => '10',
##&nbsp;                          'block_size' => '4096',
##&nbsp;                          'read%' => '100',
##&nbsp;                          'random%' => '100'
##&nbsp;                        }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dwim -S 1 -t 10
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[dwim:v1.49]
##&nbsp;dwim server: waiting for connections on port 6336...
##&nbsp;dwim server: got connection from 127.0.0.1 (1/1)
##&nbsp;Limiting max qdepth to 2048
##&nbsp;Run time: 10
##&nbsp;Target iops: 100
##&nbsp;Block size: 4096 (4K)
##&nbsp;Read %: 100
##&nbsp;Random %: 100
##&nbsp;
##&nbsp;Time(s)		TPUT(iops)	Lat.(ms)	MaxQdepth :: Fri Jun 15 11:48:34 2012
##&nbsp;    1		   103		    0.787	      5
##&nbsp;    2		   100		    0.517	      3
##&nbsp;    3		   101		    0.137	      1
##&nbsp;    4		   100		    0.136	      1
##&nbsp;    5		   100		    0.137	      1
##&nbsp;    6		   100		    0.138	      1
##&nbsp;    7		   100		    0.236	      2
##&nbsp;    8		   100		    0.134	      1
##&nbsp;    9		   100		    0.133	      1
##&nbsp;   10		    97		    0.133	      1
##&nbsp;
##&nbsp;Threads have terminated. About to destroy AIO context
##&nbsp;
##&nbsp;========== CLIENT 0 (127.0.0.1) ==========
##&nbsp;
##&nbsp;Run time (s): 10.031
##&nbsp;Avg. TPUT (iops): 99.791
##&nbsp;Avg. TPUT (kbps): 399.162
##&nbsp;Avg. RT     (ms): 0.251
##&nbsp;Max. RT     (ms): 22.643
##&nbsp;
##&nbsp;Average latency of lowest 99.00% latencies: 0.15ms
##&nbsp;Avg qdepth : 0.025
##&nbsp;Max qdepth : 5 (2048)
##&nbsp;Percent target throughput achieved: 99.791
##&nbsp;
##&nbsp;========== TOTAL ==========
##&nbsp;
##&nbsp;Run time (s): 10.031
##&nbsp;Avg. TPUT (iops): 99.790
##&nbsp;Avg. TPUT (kbps): 399.160
##&nbsp;Avg. RT     (ms): 0.251
##&nbsp;Max. RT     (ms): 22.643
##&nbsp;
##&nbsp;Average latency of lowest 99.00% latencies: 0.15ms
##&nbsp;Avg qdepth : 0.025
##&nbsp;Max qdepth : 5 (2048)
##&nbsp;Percent target throughput achieved: 99.790
##&nbsp;Dwim workload complete!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result_summary' => {
##&nbsp;                                  'client0' => {
##&nbsp;                                                 'runtime_in_secs' => '10.031',
##&nbsp;                                                 'max_qdepth' => '5 (2048)',
##&nbsp;                                                 'avg_latency' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'lowest_percentage' => '99.00%',
##&nbsp;                                                                      'latency' => '0.15ms'
##&nbsp;                                                                    }
##&nbsp;                                                                  ],
##&nbsp;                                                 'percent_target_throughput_achieved' => '99.791',
##&nbsp;                                                 'avg_qdepth' => '0.025',
##&nbsp;                                                 'ipaddress' => '127.0.0.1',
##&nbsp;                                                 'avg_tput_kbps' => '399.162',
##&nbsp;                                                 'avg_rt_ms' => '0.251',
##&nbsp;                                                 'avg_tput_iops' => '99.791',
##&nbsp;                                                 'max_rt_ms' => '22.643'
##&nbsp;                                               },
##&nbsp;                                  'total' => {
##&nbsp;                                               'runtime_in_secs' => '10.031',
##&nbsp;                                               'max_qdepth' => '5 (2048)',
##&nbsp;                                               'avg_latency' => [
##&nbsp;                                                                  {
##&nbsp;                                                                    'lowest_percentage' => '99.00%',
##&nbsp;                                                                    'latency' => '0.15ms'
##&nbsp;                                                                  }
##&nbsp;                                                                ],
##&nbsp;                                               'percent_target_throughput_achieved' => '99.790',
##&nbsp;                                               'avg_qdepth' => '0.025',
##&nbsp;                                               'avg_tput_kbps' => '399.160',
##&nbsp;                                               'avg_rt_ms' => '0.251',
##&nbsp;                                               'avg_tput_iops' => '99.790',
##&nbsp;                                               'max_rt_ms' => '22.643'
##&nbsp;                                             }
##&nbsp;                                },
##&nbsp;            'version' => '1.49',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '103',
##&nbsp;                           'max_qdepth_for_sec' => '5',
##&nbsp;                           'time_in_secs' => '1',
##&nbsp;                           'lat_ms' => '0.787'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '3',
##&nbsp;                           'time_in_secs' => '2',
##&nbsp;                           'lat_ms' => '0.517'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '101',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '3',
##&nbsp;                           'lat_ms' => '0.137'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '4',
##&nbsp;                           'lat_ms' => '0.136'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '5',
##&nbsp;                           'lat_ms' => '0.137'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '6',
##&nbsp;                           'lat_ms' => '0.138'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '2',
##&nbsp;                           'time_in_secs' => '7',
##&nbsp;                           'lat_ms' => '0.236'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '8',
##&nbsp;                           'lat_ms' => '0.134'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '9',
##&nbsp;                           'lat_ms' => '0.133'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '97',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '10',
##&nbsp;                           'lat_ms' => '0.133'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'inputs' => {
##&nbsp;                          'target_iops' => '100',
##&nbsp;                          'runtime_in_secs' => '10',
##&nbsp;                          'block_size' => '4096',
##&nbsp;                          'read%' => '100',
##&nbsp;                          'random%' => '100'
##&nbsp;                        }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dwim -C localhost 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[dwim:v1.49]
##&nbsp;Target files: 1. Size (6903B) (6.74 KB)
##&nbsp;dwim client: connecting to 127.0.0.1
##&nbsp;dwim client: got server id: dwim
##&nbsp;dwim client: got server version: 1.49
##&nbsp;dwim client: waiting for server...
##&nbsp;dwim client: server ready, starting
##&nbsp;Limiting max qdepth to 2048
##&nbsp;Run time: 10
##&nbsp;Target iops: 100
##&nbsp;Block size: 4096 (4K)
##&nbsp;Read %: 100
##&nbsp;Random %: 100
##&nbsp;
##&nbsp;Time(s)		TPUT(iops)	Lat.(ms)	MaxQdepth :: Fri Jun 15 11:50:07 2012
##&nbsp;    1		   103		    1.436	      5
##&nbsp;    2		   100		    0.141	      1
##&nbsp;    3		   101		    0.139	      1
##&nbsp;    4		   100		    0.135	      1
##&nbsp;    5		   100		    0.131	      1
##&nbsp;    6		   100		    0.188	      1
##&nbsp;    7		   100		    0.132	      1
##&nbsp;    8		   100		    0.133	      1
##&nbsp;    9		   100		    0.132	      1
##&nbsp;   10		    97		    0.132	      1
##&nbsp;
##&nbsp;Threads have terminated. About to destroy AIO context
##&nbsp;
##&nbsp;Run time (s): 10.029
##&nbsp;Avg. TPUT (iops): 99.812
##&nbsp;Avg. TPUT (kbps): 399.247
##&nbsp;Avg. RT     (ms): 0.274
##&nbsp;Max. RT     (ms): 46.382
##&nbsp;
##&nbsp;Average latency of lowest 98.80% latencies: 0.15ms
##&nbsp;Avg qdepth : 0.027
##&nbsp;Max qdepth : 5 (2048)
##&nbsp;Percent target throughput achieved: 99.812
##&nbsp;Dwim workload complete!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result_summary' => {
##&nbsp;                                  'runtime_in_secs' => '10.029',
##&nbsp;                                  'max_qdepth' => '5 (2048)',
##&nbsp;                                  'avg_latency' => [
##&nbsp;                                                     {
##&nbsp;                                                       'lowest_percentage' => '98.80%',
##&nbsp;                                                       'latency' => '0.15ms'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                  'percent_target_throughput_achieved' => '99.812',
##&nbsp;                                  'avg_qdepth' => '0.027',
##&nbsp;                                  'avg_tput_kbps' => '399.247',
##&nbsp;                                  'avg_rt_ms' => '0.274',
##&nbsp;                                  'avg_tput_iops' => '99.812',
##&nbsp;                                  'max_rt_ms' => '46.382'
##&nbsp;                                },
##&nbsp;            'version' => '1.49',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '103',
##&nbsp;                           'max_qdepth_for_sec' => '5',
##&nbsp;                           'time_in_secs' => '1',
##&nbsp;                           'lat_ms' => '1.436'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '2',
##&nbsp;                           'lat_ms' => '0.141'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '101',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '3',
##&nbsp;                           'lat_ms' => '0.139'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '4',
##&nbsp;                           'lat_ms' => '0.135'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '5',
##&nbsp;                           'lat_ms' => '0.131'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '6',
##&nbsp;                           'lat_ms' => '0.188'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '7',
##&nbsp;                           'lat_ms' => '0.132'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '8',
##&nbsp;                           'lat_ms' => '0.133'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '100',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '9',
##&nbsp;                           'lat_ms' => '0.132'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'tput_iops' => '97',
##&nbsp;                           'max_qdepth_for_sec' => '1',
##&nbsp;                           'time_in_secs' => '10',
##&nbsp;                           'lat_ms' => '0.132'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'inputs' => {
##&nbsp;                          'target_iops' => '100',
##&nbsp;                          'file_size' => '6903B',
##&nbsp;                          'target_files' => '1',
##&nbsp;                          'runtime_in_secs' => '10',
##&nbsp;                          'block_size' => '4096',
##&nbsp;                          'read%' => '100',
##&nbsp;                          'random%' => '100'
##&nbsp;                        }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dwim {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );
    my @result_array;
    my @stats;
    my $pos   = 0;
    my $stats = 0;
    for ( my $i = 0; $i < (@lines); $i++ ) {
        my $client;
        if ( $lines[$i] =~ /dwim:v(\d+.*)\]/ ) {
            $result_array[0]->{version} = $1;
        }
        if ( ( $lines[$i] =~ /CLIENT/ ) || ( $lines[$i] =~ /TOTAL/ ) ) {
            if ( $lines[$i] =~ /CLIENT\s(\d+)\s\((.*)\)/ ) {
                $client = "client" . $1;
                $result_array[0]->{result_summary}->{$client}->{ipaddress}
                    = $2;
            } else {
                $client = "total";
            }
            $i++;
            while ( $lines[$i] =~ /:/ ) {
                my @cli_stats = split( /\s*:\s+/, $lines[$i] );
                $cli_stats[0] =~ s/\.//g
                    if ( $cli_stats[0] !~ /Average\s+latency\s+of/ );
                if ( $cli_stats[0] =~ /(.*)\s+%/ ) {
                    $cli_stats[0] = "$1%";
                }
                $cli_stats[0] = "runtime_in_secs"
                    if ( $cli_stats[0] =~ /Run time/ );
                if ( $cli_stats[0]
                    =~ /Average\s+latency\s+of\s+lowest\s+(.*\%)\s+latencies/
                    ) {
                    $result_array[0]->{result_summary}->{$client}
                        ->{avg_latency}->[0]->{lowest_percentage} = $1;
                    $result_array[0]->{result_summary}->{$client}
                        ->{avg_latency}->[0]->{latency} = $cli_stats[1];
                } else {
                    $result_array[0]->{result_summary}->{$client}
                        ->{ to_key_name( item => $cli_stats[0] ) }
                        = $cli_stats[1];
                }
                $i++;
            }
            $i--;
        }
        if ( $lines[$i] =~ /Time\(s\)\t\tTPUT/ ) {
            $stats = 1;
        }
        if (   $lines[$i] =~ /:/
            && $lines[$i] !~ /dwim|Time\(s\)\t\tTPUT/
            && !$client ) {
            my $key_var;
            if ($stats) {
                $key_var = "result_summary";
            } else {
                $key_var = "inputs";
            }
            my @items = split( /\s*:\s+/, $lines[$i] );
            $items[0] =~ s/\.//g if ( $items[0] !~ /Average\s+latency\s+of/ );
            if ( $items[0] =~ /(.*)\s+%/ ) {
                $items[0] = "$1%";
            }
            $items[0] = "runtime_in_secs" if ( $items[0] =~ /Run time/ );
            if ( $items[0] =~ /Block size/ && $items[1] =~ /(\d+)\s+.*\d/ ) {
                $items[1] = $1;
            }
            if ( $items[0] =~ /Target files/ ) {
                if ( $items[1] =~ /(\d+)\.\s/ ) {
                    my $rest = $';
                    $result_array[0]->{$key_var}
                        ->{ to_key_name( item => $items[0] ) } = $1;
                    if ( $rest =~ /(Size|Size range:)\s(.*)/ ) {
                        my $key  = "file_$1";
                        my $temp = $2;
                        if ( $temp =~ /\((\d+.*)\)\s+\(\d+.*\)/ ) {
                            $result_array[0]->{$key_var}
                                ->{ to_key_name( item => $key ) } = $1;
                        } else {
                            $result_array[0]->{$key_var}
                                ->{ to_key_name( item => $key ) } = $temp;
                        }
                    }
                }
            } elsif ( $items[0]
                =~ /Average\s+latency\s+of\s+lowest\s+(.*\%)\s+latencies/ ) {
                $result_array[0]->{$key_var}->{avg_latency}->[0]
                    ->{lowest_percentage} = $1;
                $result_array[0]->{$key_var}->{avg_latency}->[0]->{latency}
                    = $items[1];
            } else {
                $result_array[0]->{$key_var}
                    ->{ to_key_name( item => $items[0] ) } = $items[1];
            }
        }
        if ( $lines[$i] =~ /Time\(s\)\t\tTPUT/ ) {
            @stats = split( /\t+/, $lines[$i] );
        }
        if ( $lines[$i] =~ /\d+\s+\d+\s+/ ) {
            my @stat_values = split( /\s+/, $lines[$i] );
            for ( my $j = 0; $j < scalar(@stats); $j++ ) {
                $stats[$j] = "time_in_secs" if ( $stats[$j] =~ /Time\(s\)/ );
                $stats[$j] = "tput_iops"    if ( $stats[$j] =~ /TPUT\(/ );
                $stats[$j] = "max_qdepth_for_sec"
                    if ( $stats[$j] =~ /MaxQdepth/ );
                $stats[$j] = to_key_name( item => $stats[$j] );
                $stats[$j] =~ s/_$//g;
                $stats[$j] =~ s/\./_/g;
                $result_array[0]->{stats}->[$pos]
                    ->{ to_key_name( item => $stats[$j] ) }
                    = $stat_values[$j];
            }
            $pos++;
        }
    }
    return \@result_array;
} ## end sub dwim

1;
