# $Id: $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse fops command output
##
## @description
##    This is a library to parse the raw output of fops command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::fops;
##
##    fops(%args);
##
## @author Abhsihek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::fops;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::fops
##
## @summary Parses the raw output from the "fops" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fops with any of its options as described in the
##    corresponding cdef file 'fops.cdefs'
## @synopsis
##      fops(%args);
## INPUTS TO METHOD CALL 'fops' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/fops_1_2, /u/myuser/dir/fops.experimental
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'current'
## This would select fops.current respectively.
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'offset-string' </td> <td> '0' </td> </tr>
## <tr> <td> '-W' </td> <td> 'width' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'block-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'file-prefix' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size-string' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'metrics-string' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'num-of-threads' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'pick-random-files' </td> <td> '1' </td> </tr>
## <tr> <td> '-X' </td> <td> 'error-tolerance-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'rename-file-prefix' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'dir-prefix' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'num-of-files' </td> <td> '0' </td> </tr>
## <tr> <td> '-O' </td> <td> 'file-open-options' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'height' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'operations' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'working-dir' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'use-existing-dir' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delay-in-operation' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'delay-between-operations' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array ref as the parsed output.

sub fops {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }
    return [];
}

1;
