#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/hammer.pm#3 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Unix like and Windows hammer command
##
## @description
##    This is a library to parse the raw output of Unix like and Windows hammer command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "hammer" will be hammer)
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::hammer;
##
##    hammer(%args);
## INPUTS TO METHOD CALL 'hammer' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/hammer_6_22, /u/myuser/dir/hammer.experimental, Y:\bin\hammer.exe
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows
##Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes.
## in Unix.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'experimental', '6.22'
## This would select hammer.experimental, hammer.6.22 respectively.
##
##
## @author dl-nacl-dev
##
## @status public
#
package NACL::APISet::Host::CLI::default::hammer;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::UnexpectedOutputException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ( $raw_output =~ /(.*Usage:.*)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = "Invalid Usage of hammer command";
    } elsif ( $raw_output =~ /usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ( $raw_output =~ /(.*:\s+No\s+such\s+file\s+or\s+directory)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = "The directory name given for hammer is invalid";
    } elsif ( $raw_output =~ /(.*:\s+Not\s+a\s+directory)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(Sorry,.*\s+API\s+is\s+not\s+available\s+on\s+.*)/ )
    {
        $error          = "InvalidParamValue";
        $$error_message = "invalid api value";
    } elsif ( $raw_output =~ /No\s+space\s+left\s+on\s+device/i )
    {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::hammer
##
## @summary Parses the raw output from "hammer" command
##
## @description
##    This method can be used for parsing the output from "hammer" command
##    called with no arguments as described in the cdef file at
##    lib/NACL/APISet/Host/CLI/default/hammer.cdefs
##
## @synopsis
##    hammer (%args);
##
## INPUTS TO METHOD CALL 'hammer' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-filercore' </td> <td> 'filercore' </td> <td> '0' </td> </tr>
## <tr> <td> '-nofilercore' </td> <td> 'nofilercore' </td> <td> '1' </td> </tr>
## <tr> <td> '-mode' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-fill' </td> <td> 'fill' </td> <td> '1' </td> </tr>
## <tr> <td> '-api' </td> <td> 'api' </td> <td> '0' </td> </tr>
## <tr> <td> '-bg' </td> <td> 'background' </td> <td> '1' </td> </tr>
## <tr> <td> '-checkinodes' </td> <td> 'checkinodes' </td> <td> '1' </td> </tr>
## <tr> <td> '-interactive' </td> <td> 'interactive' </td> <td> '1' </td> </tr>
## <tr> <td> '-iterations' </td> <td> 'iterations' </td> <td> '0' </td> </tr>
## <tr> <td> '-logfile' </td> <td> 'logfile' </td> <td> '0' </td> </tr>
## <tr> <td> '-nocleanup' </td> <td> 'nocleanup' </td> <td> '1' </td> </tr>
## <tr> <td> '-noflush' </td> <td> 'noflush' </td> <td> '1' </td> </tr>
## <tr> <td> '-onlyflush' </td> <td> 'onlyflush' </td> <td> '1' </td> </tr>
## <tr> <td> '-nostreams' </td> <td> 'nostreams' </td> <td> '1' </td> </tr>
## <tr> <td> '-seed' </td> <td> 'seed' </td> <td> '0' </td> </tr>
## <tr> <td> '-runtime' </td> <td> 'runtime' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-blocksize' </td> <td> 'blocksize' </td> <td> '0' </td> </tr>
## <tr> <td> '-blocksize random' </td> <td> 'blocksize_random' </td> <td> '1' </td> </tr>
## <tr> <td> '-minbsize' </td> <td> 'min_block_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-maxbsize' </td> <td> 'max_block_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-minfsize' </td> <td> 'min_file_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-maxfsize' </td> <td> 'max_file_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-ignorelockerrors' </td> <td> 'ignorelockerrors' </td> <td> '1' </td> </tr>
## <tr> <td> '-ignorefileerrors' </td> <td> 'ignorefileerrors' </td> <td> '1' </td> </tr>
## <tr> <td> '-ignoredatacorruption' </td> <td> 'ignoredatacorruption' </td> <td> '1' </td> </tr>
## <tr> <td> '-ignoreallerrors' </td> <td> 'ignoreallerrors' </td> <td> '1' </td> </tr>
## </table>
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the client.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref containing a hash reference which contains the
##  parsed output data from the output of hammer command
##
##&nbsp;Command = hammer -filercore csiqa-2040-11a -iterations 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Copyright (c) 2007 Network Appliance, Inc. All rights reserved.
##&nbsp;hammer started at Thu Dec 01 05:13:11 2011
##&nbsp;version=$Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/default/hammer.pm#2 $
##&nbsp;path=C:\Documents and Settings\Administrator\5e313cdb pid=0x0F98
##&nbsp;client=CSIQA-AD1 (Windows Server 2003 [5.2.3790 Service Pack 2])
##&nbsp;minfsize=0x00000001 maxfsize=0x00500000
##&nbsp;minbsize=0x00000200 maxbsize=0x00020000 blocksize=random
##&nbsp;api=win32 mode=mixed streams=on flush=random nocleanup=false seed=2925582890
##&nbsp;logfile=hamlog.txt timezone=Eastern Standard Time
##&nbsp;num_iterations=0 max_iterations=1 cur_runtime=3 max_runtime=-1
##&nbsp;2011/12/01-05:13:14 CREATE 00000001.ham:s00000001 fileid=0x737CEA05 blocksize=0x0001ECE7
##&nbsp;filesize=0x00329256 timestamp=0x4ED7533A (async then flush) 25892.7K/s
##&nbsp;iterations limit (1 iterations) reached.
##&nbsp;cleaning up...
##&nbsp;2011/12/01-05:13:14 READ   00000001.ham:s00000001 fileid=0x737CEA05 blocksize=0x000143C7
##&nbsp;filesize=0x00329256 3.23658e+006K/s
##&nbsp;2011/12/01-05:13:14 DELETE 00000001.ham:s00000001 fileid=0x737CEA05 0.016sec
##&nbsp;version=$Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/default/hammer.pm#2 $
##&nbsp;path=C:\Documents and Settings\Administrator\5e313cdb pid=0x0F98
##&nbsp;client=CSIQA-AD1 (Windows Server 2003 [5.2.3790 Service Pack 2])
##&nbsp;minfsize=0x00000001 maxfsize=0x00500000
##&nbsp;minbsize=0x00000200 maxbsize=0x00020000 blocksize=random
##&nbsp;api=win32 mode=mixed streams=on flush=random nocleanup=false seed=2925582890
##&nbsp;logfile=hamlog.txt timezone=Eastern Standard Time
##&nbsp;num_iterations=1 max_iterations=1 cur_runtime=6 max_runtime=-1
##&nbsp;hammer stopped at Thu Dec 01 05:13:17 2011
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'blocksize' => 'random',
##&nbsp;             'maxbsize' => '0x00020000',
##&nbsp;             'num_iterations' => '3',
##&nbsp;             'mode' => 'mixed',
##&nbsp;             'minbsize' => '0x00000200',
##&nbsp;             'maxfsize' => '0x00500000',
##&nbsp;             'seed' => '2742802126',
##&nbsp;             'api' => 'win32',
##&nbsp;             'cur_runtime' => '7',
##&nbsp;             'streams' => 'on',
##&nbsp;             'logfile' => 'hamlog.txt',
##&nbsp;             'timezone' => 'Eastern',
##&nbsp;             'max_iterations' => '3',
##&nbsp;             'version' => '$Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/default/hammer.pm#2 $',
##&nbsp;             'path' => 'C:\\Documents',
##&nbsp;             'nocleanup' => 'false',
##&nbsp;             'operations' => [
##&nbsp;                                            {
##&nbsp;                                              'info' => 'sync',
##&nbsp;                                              'blocksize' => '0x0001AE5A',
##&nbsp;                                              'fileid' => '0x35B74026',
##&nbsp;                                              'filesize' => '0x0048D3F2',
##&nbsp;                                              'file_name' => '00000001.ham:s00000001',
##&nbsp;                                              'timestamp' => '0x4EA543D8',
##&nbsp;                                              'started' => '2011/12/01-05:13:14',
##&nbsp;                                              'operation' => 'CREATE'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'info' => 'async then flush',
##&nbsp;                                              'blocksize' => '0x0001B149',
##&nbsp;                                              'fileid' => '0x1264BD07',
##&nbsp;                                              'filesize' => '0x00377C05',
##&nbsp;                                              'file_name' => '00000002.ham',
##&nbsp;                                              'timestamp' => '0x4EA543D8',
##&nbsp;                                              'started' => '2011/12/01-05:13:14',
##&nbsp;                                              'operation' => 'CREATE'
##&nbsp;                                               },
##&nbsp;                               ],
##&nbsp;             'client' => 'CSIQA-AD1',
##&nbsp;             'flush' => 'random',
##&nbsp;             'minfsize' => '0x00000001',
##&nbsp;             'max_runtime' => '-1'
##&nbsp;           }
##&nbsp;	];
sub hammer {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'hammer',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {

        if ( $error_level == 0 ) {
            $Log->warn( "Due to a bug in Hammer, a zero execution status"
                  . "\nwas returned even when the command execution"
                  . "\nhas failed.\n" );
        }
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level(>0) "
              . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    my @lines =
      get_lines( raw_output => $raw_output, response_object => $resp );
    my ( $curr_word, $flag );
    my %outer_hash = ();
    foreach my $line (@lines) {
        my %hash;
        if ( $line =~
            /(.*) (CREATE|DELETE|RENAME|OWRITE|TRUNC|READ) (.*) fileid/ )
        {
            $hash{'started'}   = $1;
            $hash{'operation'} = $2;
            $hash{'file_name'} = $3;
            $curr_word         = $hash{'file_name'};
            while (
                $curr_word = _get_next_word(
                    line      => $line,
                    prev_word => $curr_word
                )
              )
            {
                if ( $curr_word =~ /(.*)=(.*)/ ) {
                    $hash{$1} = $2;
                }
                elsif ( $curr_word =~ /\((.*)\)/ ) {
                    $hash{'info'} = $1;
                }
                elsif ( $curr_word =~ /\// ) {
                    $hash{'speed'} = $curr_word;
                }
                else {
                    $hash{'time'} = $curr_word;
                }
            } ## end while ($curr_word = _get_next_word...
            push( @{ $outer_hash{'operations'} }, \%hash );
        }
        elsif ( $line =~ /=/ && $line !~ /[Vv]ersion/ ) {
            $line      = "dummy " . $line;
            $curr_word = 'dummy';
            while (
                $curr_word = _get_next_word(
                    line      => $line,
                    prev_word => $curr_word
                )
              )
            {
                if ( $curr_word =~ /(.*)=(.*)/ ) {
                    $outer_hash{$1} = $2;
                }
            }
        }
        elsif ( $line =~ /(hammer\s+)?version(\s+|=)(.*)/ ) {
            $outer_hash{'version'} = $3;
        }
    } ## end foreach my $line (@lines)
    @result_array = ( \%outer_hash );
    return \@result_array;    #to be updated to return a reference
} ## end sub hammer

sub _get_next_word {
    my %args      = @_;
    my $line      = $args{line};
    my $prev_word = $args{prev_word};
    if ( $line =~ /\Q$prev_word\E\s+(\(.*\)|[^\s]*)/ ) {
        my $word = $1;
        $Log->debug("Line - $line\n prev_word - $prev_word\n Word is - $word");

        return $word;
    }
    return undef;
} ## end sub _get_next_word
1;
