#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux  netmist commands
##
## @description
##    This is a library to parse the raw outputs of any of the Linux netmist commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "netmist" will be netmist()
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::netmist;
##
##    netmist(%agrs);
##
## @author dl-nacl-dev@netapp.com, pbala@netapp.com
##
## @status public
##

package NACL::APISet::Host::CLI::default::netmist;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $error_message;
    my $error;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /(ssh:\serror.*)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $1;
    }
    elsif ( $raw_output =~ /\s+User\soptions:/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(bash:.*|.*[cC]ommand not found.*)/i ) {
        $error          = "CommandNotFound";
        $error_message  = $1;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::netmist
##
## @summary Parses the raw output from the "/usr/software/test/projects/perf/binaries.linux/netmist" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netmist with any of its options as described in the
##    corresponding cdef file 'netmist.cdefs'
## @synopsis
##      netmist(%args);
## INPUTS TO METHOD CALL 'netmist' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-E' </td> <td> 'export-mix' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'install-benchmark' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'working-dirpath' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'use-ipv6' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kill' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'run-locally' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'files-per-dir' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'import-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-K' </td> <td> 'disable-file-unlink' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'enable-op-latency' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'workload-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-G' </td> <td> 'enable-notifications' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'additional-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'file-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'ops-per-second' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'master-result-dir' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'master-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'client-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'log-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'record-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'master-flag' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'dirs-per-proc' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'processes' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'local-filesystem' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'warm-seconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'aggr-data' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'pit-server-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'run-seconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'client-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'client-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'client-token-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-U' </td> <td> 'log-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'disable-fsyncs' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'info' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'pit-server-hostname' </td> <td> '0' </td> </tr>
## <tr> <td> '6' </td> <td> 'ipv6' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'use-rsh' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'client-id-number' </td> <td> '0' </td> </tr>
## <tr> <td> '-O' </td> <td> 'op-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'master-listen-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'clients' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'execdir' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = /usr/software/test/projects/perf/binaries.linux/netmist -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Netmist         $Revision: #1 $
##&nbsp;

sub netmist {
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $error_level  = $response->get_execution_status();
    my $error = _check_error(
        "raw_output"      => $raw_output,
        "response_object" => $response,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text =>  "Execution status was $error_level "
                    ."indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    return [];
}    # end of netmist

1;
