#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse pkill command output
##
## @description
##    This library supports error checking for pkill command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::pkill;
##
##  pkill(%args);
##
## @author praveent@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::pkill;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::pkill
##
## @summary Parses the raw output from the "pkill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pkill with any of its options as described in the
##    corresponding cdef file 'pkill.cdefs'
## @synopsis
##      pkill(%args);
## INPUTS TO METHOD CALL 'pkill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## The below options are supported in Linux, Solaris and FreeBsd
## <tr> <td> '-s' </td> <td> 'sid' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'oldest' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'terminal' </td> <td> '0' </td> </tr>
## <tr> <td> '-G' </td> <td> 'groupid' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'newest' </td> <td> '1' </td> </tr>
## <tr> <td> '-U' </td> <td> 'uid' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'exact' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'negate' </td> <td> '1' </td> </tr>
## <tr> <td> '-signal' </td> <td> 'signal' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'ppid' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'pgrp' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'effective_uid' </td> <td> '0' </td> </tr>

## The below option support FreeBsd only
## <tr> <td> '-I' </td> <td> 'request' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'lock' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ignore' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'pidfile' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'core' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-j' </td> <td> 'jid' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'vsid' </td> <td> '0' </td> </tr>

## The below option support Solaris only
## <tr> <td> '-J' </td> <td> 'project-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'task-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'ctid-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zone-list' </td> <td> '0' </td> </tr>

## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub pkill {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub pkill

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Operation\s+not\s+permitted/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } 
    elsif ( $raw_output =~ /option\s+requires\s+an\s+argument/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif (
        $raw_output =~ /No\s+such\s+.*|Unknown\s+group.*|Unknown\s+user.*/i )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /[Uu]sage:.*|invalid.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
