# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/sio.pm#1 $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse sio command
##
## @description
##    This is a library to parse the raw output of sio command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sio" will be sio)
##
## @synopsis
##    use sio;
##
##    sio(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::sio;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my (%args) = @_;
    my $resp = $args{response_object};
    my $raw_output = $args{output};
    my $error_level = $resp->get_execution_status();
    my $error         = undef;
    if ( $raw_output =~ /(option .+ not found|The system cannot find the drive specified)/ ) {
        $error          = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$raw_output,response_object => $resp);
    }
    if ($error_level) {
        my ($error_message) = ( $raw_output =~ /(SIO_ERROR:.*)/ );
        my $text
            = defined($error_message)
            ? "sio command failed with error message: $error_message\n"
            : "sio command exited with status $error_level\n";
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $text,
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::sio
##
## @summary Parses the raw output from the "sio" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sio with any of its options as described in the
##    corresponding cdef file 'sio.cdefs'
## @synopsis
##      sio(%args);
##
## INPUTS TO METHOD CALL 'sio' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/sio_6_22, /u/myuser/dir/sio.experimental
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'experimental', '6.22'
## This would select sio.experimental, sio.6.22 respectively.
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-ramp' </td> <td> 'ramp' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'file_size' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start_byte' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'block_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-magic_trigger' </td> <td> 'magic_trigger' </td> <td> '0' </td> </tr>
## <tr> <td> '-fixedfill' </td> <td> 'fixedfill' </td> <td> '0' </td> </tr>
## <tr> <td> '-verify_retry' </td> <td> 'verify_retry' </td> <td> '0' </td> </tr>
## <tr> <td> '-stream_per_thread' </td> <td> 'stream_per_thread' </td> <td> '1' </td> </tr>
## <tr> <td> '-affine' </td> <td> 'affine' </td> <td> '1' </td> </tr>
## <tr> <td> '-shmprot' </td> <td> 'shared_mem_prot' </td> <td> '0' </td> </tr>
## <tr> <td> '-align' </td> <td> 'align' </td> <td> '0' </td> </tr>
## <tr> <td> '-nopread' </td> <td> 'nopread' </td> <td> '1' </td> </tr>
## <tr> <td> '-heap' </td> <td> 'use_heap' </td> <td> '1' </td> </tr>
## <tr> <td> '-no_dsync' </td> <td> 'no_dsync' </td> <td> '1' </td> </tr>
## <tr> <td> '-shmI' </td> <td> 'use_intimate_shared_mem' </td> <td> '1' </td> </tr>
## <tr> <td> '-shm' </td> <td> 'use_shared_mem' </td> <td> '1' </td> </tr>
## <tr> <td> '-iofailok' </td> <td> 'iofailok' </td> <td> '1' </td> </tr>
## <tr> <td> '-globalfds' </td> <td> 'globalfds' </td> <td> '1' </td> </tr>
## <tr> <td> '-stats_reset' </td> <td> 'stats_reset' </td> <td> '0' </td> </tr>
## <tr> <td> '-hard' </td> <td> 'hard' </td> <td> '1' </td> </tr>
## <tr> <td> '-fillonce' </td> <td> 'fillonce' </td> <td> '1' </td> </tr>
## <tr> <td> '-mmap' </td> <td> 'use_mem_mapped_io' </td> <td> '0' </td> </tr>
## <tr> <td> '-partition_among_threads' </td> <td> 'partition_among_threads' </td> <td> '1' </td> </tr>
## <tr> <td> '-noheader' </td> <td> 'noheader' </td> <td> '1' </td> </tr>
## <tr> <td> '-soak' </td> <td> 'soak' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num_of_threads' </td> <td> '0' </td> </tr>
## <tr> <td> '-think' </td> <td> 'think' </td> <td> '0' </td> </tr>
## <tr> <td> '-lockall' </td> <td> 'lockall' </td> <td> '1' </td> </tr>
## <tr> <td> '-rep_latency' </td> <td> 'rep_latency' </td> <td> '0' </td> </tr>
## <tr> <td> '-nolseek' </td> <td> 'nolseek' </td> <td> '1' </td> </tr>
## <tr> <td> '-rshhostfirst' </td> <td> 'rshhostfirst' </td> <td> '1' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-no_sync' </td> <td> 'no_sync' </td> <td> '0' </td> </tr>
## <tr> <td> '-truncate' </td> <td> 'truncate' </td> <td> '1' </td> </tr>
## <tr> <td> '-aio' </td> <td> 'async_io' </td> <td> '0' </td> </tr>
## <tr> <td> '-rshuserfirst' </td> <td> 'rshuserfirst' </td> <td> '1' </td> </tr>
## <tr> <td> '-noflock' </td> <td> 'noflock' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'devices_list' </td> <td> '0' </td> </tr>
## <tr> <td> '-kstat' </td> <td> 'enable_solaris_kstat' </td> <td> '1' </td> </tr>
## <tr> <td> '-openfailok' </td> <td> 'ignore_open_fails' </td> <td> '1' </td> </tr>
## <tr> <td> '-detailed_logging' </td> <td> 'detailed_logging' </td> <td> '1' </td> </tr>
## <tr> <td> '-filer' </td> <td> 'filer' </td> <td> '0' </td> </tr>
## <tr> <td> '-nosysinfo' </td> <td> 'disable_host_monitoring' </td> <td> '1' </td> </tr>
## <tr> <td> '-niceoutput' </td> <td> 'nice_output' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'read' </td> <td> '0' </td> </tr>
## <tr> <td> '-debug' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-metadata' </td> <td> 'metadata' </td> <td> '1' </td> </tr>
## <tr> <td> '-partial_write_test' </td> <td> 'partial_write_test' </td> <td> '1' </td> </tr>
## <tr> <td> '-shmkey' </td> <td> 'shared_mem_key' </td> <td> '0' </td> </tr>
## <tr> <td> '-fileperthread' </td> <td> 'one_file_per_thread' </td> <td> '1' </td> </tr>
## <tr> <td> '-iscsi' </td> <td> 'iscsi' </td> <td> '1' </td> </tr>
## <tr> <td> '-max_blksize' </td> <td> 'max_blksize' </td> <td> '0' </td> </tr>
## <tr> <td> '-blockno' </td> <td> 'blockno' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'duration_in_secs' </td> <td> '0' </td> </tr>
## <tr> <td> '-numops' </td> <td> 'numops' </td> <td> '0' </td> </tr>
## <tr> <td> '-mfiler' </td> <td> 'mfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-nomutex' </td> <td> 'nomutex' </td> <td> '1' </td> </tr>
## <tr> <td> '-instrumentation' </td> <td> 'instrumentation' </td> <td> '1' </td> </tr>
## <tr> <td> '-rshuser' </td> <td> 'rsh_user_name' </td> <td> '0' </td> </tr>
## <tr> <td> '-direct' </td> <td> 'direct' </td> <td> '1' </td> </tr>
## <tr> <td> '-profile' </td> <td> 'collect_filer_profile' </td> <td> '1' </td> </tr>
## <tr> <td> '-verify' </td> <td> 'verify' </td> <td> '1' </td> </tr>
## <tr> <td> '-max_latency' </td> <td> 'max_latency' </td> <td> '0' </td> </tr>
## <tr> <td> '-help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-break_on_dc' </td> <td> 'break_on_dc' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'rand' </td> <td> '0' </td> </tr>
## <tr> <td> '-stop' </td> <td> 'stop' </td> <td> '0' </td> </tr>
## <tr> <td> '-iops' </td> <td> 'iops' </td> <td> '0' </td> </tr>
## <tr> <td> '-dafs' </td> <td> 'dafs' </td> <td> '0' </td> </tr>
## <tr> <td> '-no_statit' </td> <td> 'no_statit' </td> <td> '1' </td> </tr>
## <tr> <td> '-create' </td> <td> 'create' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sio 0 50 4k 0 100m 10 2 a.file b.file
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Version: 6.29
##&nbsp;Read: 0 Rand: 50 BlkSz: 4096 BegnBlk: 0 EndBlk: 25600 Secs: 10 Threads: 2 Devs: 2  a.file b.file
##&nbsp;Terminating threads ...
##&nbsp;IOPS,TPUT(KB/s),LAT(ms)Calc'd,LAT(ms)Actual,READ,RAND,IOS,SEC,THDS,BLKSZ,CSW_IO,INT_IO,SYS,USR,WT,IDL,uIO,uKB,uuIO,tuIO
##&nbsp;48,192,41.589,[41.696],0,50,433,9,2,4096,17.2,15.0,1,0,38,62,832,207.9,0,832
##&nbsp;Thread Latency Stats:
##&nbsp; Thread:        0
##&nbsp;  ios:       217
##&nbsp;  latency(us):   9026979
##&nbsp;  sumofsquares:     17528
##&nbsp;  min(ms):     10.00
##&nbsp;  max(ms):     77.66
##&nbsp;  avg(ms):     41.60
##&nbsp;  stddev(ms):      8.99
##&nbsp; Thread:         1
##&nbsp;  ios:       216
##&nbsp;  latency(us):   9027431
##&nbsp;  sumofsquares:     18699
##&nbsp;  min(ms):     10.00
##&nbsp;  max(ms):     92.56
##&nbsp;  avg(ms):     41.79
##&nbsp;  stddev(ms):      9.30
##&nbsp;
##&nbsp;Global Latency Stats:
##&nbsp; ios:       433
##&nbsp; latency(us):  18054410
##&nbsp; sumofsquares:     36227
##&nbsp; min(ms):     10.00
##&nbsp; max(ms):     92.56
##&nbsp; avg(ms):     41.70
##&nbsp; stddev:      9.15
##&nbsp;
##&nbsp;global_reads = 0; global_bytes_read = 0 (0 KB)
##&nbsp;global_writes = 433; global_bytes_written = 1773568 (1732 KB)
##&nbsp;global_time_start = 1338802199, global_time_stop = 1338802209
##&nbsp;measurement start = 1338802200, measurement stop = 1338802209
##&nbsp;Computed run time seconds = 10
##&nbsp;Computed measurement seconds = 9
##&nbsp;Computed IOPS = 48.11
##&nbsp;Computed KB/s = 192.44
##&nbsp;Computed bytes/IO = 4096
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'global_bytes_read' => '0 (0 KB)',
##&nbsp;             'micro_io' => '832',
##&nbsp;             'block_size' => '4096',
##&nbsp;             'percentage_read' => '0',
##&nbsp;             'computed_run_time_seconds' => '10',
##&nbsp;             'computed_bytes_per_io' => '4096',
##&nbsp;             'global_writes' => '433',
##&nbsp;             'micro_micro_io' => '0',
##&nbsp;             'global_reads' => '0',
##&nbsp;             'latency_calculated' => '41.589',
##&nbsp;             'csw_io' => '17.2',
##&nbsp;             'measurement_start' => '1338802200',
##&nbsp;             'duration' => '9',
##&nbsp;             'global_bytes_written' => '1773568 (1732 KB)',
##&nbsp;             'computed_iops' => '48.11',
##&nbsp;             'throughput' => '192',
##&nbsp;             'total_ios' => '433',
##&nbsp;             'user_time' => '0',
##&nbsp;             'global_latency_stats' => [
##&nbsp;                                         {
##&nbsp;                                           'min_ms' => '10.00',
##&nbsp;                                           'stddev' => '9.15',
##&nbsp;                                           'sumofsquares' => '36227',
##&nbsp;                                           'avg_ms' => '41.70',
##&nbsp;                                           'latency_us' => '18054410',
##&nbsp;                                           'max_ms' => '92.56',
##&nbsp;                                           'ios' => '433'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;             'micro_kb' => '207.9',
##&nbsp;             'io_per_secs' => '48',
##&nbsp;             'computed_kb_per_s' => '192.44',
##&nbsp;             'thread_latency_stats' => [
##&nbsp;                                         {
##&nbsp;                                           'sumofsquares' => '17528',
##&nbsp;                                           'avg_ms' => '41.60',
##&nbsp;                                           'max_ms' => '77.66',
##&nbsp;                                           'ios' => '217',
##&nbsp;                                           'min_ms' => '10.00',
##&nbsp;                                           'stddev_ms' => '8.99',
##&nbsp;                                           'thread' => '0',
##&nbsp;                                           'latency_us' => '9026979'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'sumofsquares' => '18699',
##&nbsp;                                           'avg_ms' => '41.79',
##&nbsp;                                           'max_ms' => '92.56',
##&nbsp;                                           'ios' => '216',
##&nbsp;                                           'min_ms' => '10.00',
##&nbsp;                                           'stddev_ms' => '9.30',
##&nbsp;                                           'thread' => '1',
##&nbsp;                                           'latency_us' => '9027431'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;             'int_io' => '15.0',
##&nbsp;             'random' => '50',
##&nbsp;             'num_of_threads' => '2',
##&nbsp;             'computed_measurement_seconds' => '9',
##&nbsp;             'global_time_start' => '1338802199',
##&nbsp;             'version' => '6.29',
##&nbsp;             'latency_actual' => '41.696',
##&nbsp;             'system_time' => '1',
##&nbsp;             'idle_time' => '62',
##&nbsp;             'wait' => '38',
##&nbsp;             'measurement_stop' => '1338802209',
##&nbsp;             'tu_io' => '832',
##&nbsp;             'global_time_stop' => '1338802209'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub sio {
    my %args = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error;

    _check_error(
        response_object => $resp,
        output => $output,
    );
    
    my @lines = get_lines( raw_output => $output );
    my (%hash, @result_array, $element);
    while (@lines) {
        my $line = shift(@lines);
        chomp($line);
        if ( $line =~ /Version:\s*(\S+)/i ) {
            # Get the version info
            $hash{version} = $1;
        } elsif ( $line =~ /^Read\:/i ) {
            # No need to parse this line because we already have this info
            next;
        } elsif ( $line =~ /^IOPS\,/ ) {
            # Expanding the headings names
            my @headings
                = qw(io_per_secs throughput latency_calculated latency_actual
                percentage_read random total_ios duration num_of_threads
                block_size csw_io int_io system_time user_time wait idle_time
                micro_io micro_kb micro_micro_io tu_io
            );
            # Get the next line;
            $line = shift(@lines);
            my @values = split( /\,\s*/, $line );
            for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                if ( $values[$i] =~ /\[(\S+)\]/ ) {
                    $values[$i] = $1;
                }
                $hash{ $headings[$i] } = $values[$i];
            }
        } elsif ( $line =~ /^(Maximum\s+latency\s+observed):\s*(\S+)/i ) {
            $hash{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /^(Thread\s+Latency\s+Stats):/i ) {
            my %hash1;
            my @array1;
            my $ii = -1;
            foreach my $k (@lines) {
               if ($k =~ /^(Global\s+Latency\s+Stats:.*)/i) {
                  $element = $1;
                  last;
               }
               if ($k =~ /Thread:\s+(\d+(.\d+)?)/) {
                  $ii++;
                  $array1[$ii]->{"thread"} = $1;
               } elsif($k =~ /(stddev\(ms\)|ios|latency\(us\)|sumofsquares|min\(ms\)|max\(ms\)|avg\(ms\)):\s+(\d+(.\d+)?)/) {
                  my $key = $1;
                  my $value = $2;
                  $key =~ s/\(/_/g;
                  $key =~ s/\)//g;
                  $array1[$ii]->{$key} = $value;
               }
            }
            $hash{"thread_latency_stats"} = \@array1;
            unshift(@lines, $element);
        } elsif ( $line =~ /^(Global\s+Latency\s+Stats):/i ) {
            my @array1;
            foreach my $k (@lines) {
                if($k =~ /(stddev|ios|latency\(us\)|sumofsquares|min\(ms\)|max\(ms\)|avg\(ms\)):\s+(\d+(.\d+)?)/) {
                     my $key = $1;
                     my $value = $2;
                     $key =~ s/\(/_/g;
                     $key =~ s/\)//g;
                     $array1[0]->{$key} = $value;
                } else {
                    last;
                }
            }
            $hash{"global_latency_stats"} = \@array1;
        } elsif ( $line =~ /(.*)\s+=\s+(\d+)[;|,]\s+(.*)\s+=\s+(\d+.*)/ ) {
            $hash{ to_key_name( item => $1 ) } = $2;
            $hash{ to_key_name( item => $3 ) } = $4;
        } elsif ( $line =~ /(.*)\s+=\s+(.*)/ ) {
            my $key = $1;
            my $value = $2;
            $key =~ s/\//_per_/;
            $hash{to_key_name( item => $key ) } = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}
1;
