#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse spt command
##
## @description
##    This is a library to parse the raw output of spt command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "spt" will be spt)
##
## @synopsis
##    use spt;
##
##    spt(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::spt;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;

    if ($raw_output =~ /(Invalid\s+option|Invalid\s+command\s+operation\s+code
                            |Error\s+parsing|open.*\s+of\s+.*\s+failed!)/si
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(Please\s+specify)/si) {
        $error          = "MissingRequiredParam";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::spt
##
## @summary Parses the raw output from the "spt" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from spt with any of its options as described in the
##    corresponding cdef file 'spt.cdefs'
## @synopsis
##      spt(%args);
## INPUTS TO METHOD CALL 'spt' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'pin' </td> <td> 'pin' </td> <td> '0' </td> </tr>
## <tr> <td> 'exit' </td> <td> 'exit' </td> <td> '1' </td> </tr>
## <tr> <td> 'iomode' </td> <td> 'iomode' </td> <td> '0' </td> </tr>
## <tr> <td> 'emit' </td> <td> 'emit' </td> <td> '0' </td> </tr>
## <tr> <td> 'dsf1' </td> <td> 'dsf1' </td> <td> '0' </td> </tr>
## <tr> <td> 'timeout' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> 'abort_timeout' </td> <td> 'abort_timeout' </td> <td> '0' </td> </tr>
## <tr> <td> 'pout' </td> <td> 'pout' </td> <td> '0' </td> </tr>
## <tr> <td> 'retry' </td> <td> 'retry' </td> <td> '0' </td> </tr>
## <tr> <td> 'runtime' </td> <td> 'runtime' </td> <td> '0' </td> </tr>
## <tr> <td> 'aborts' </td> <td> 'aborts' </td> <td> '0' </td> </tr>
## <tr> <td> 'repeat' </td> <td> 'repeat' </td> <td> '0' </td> </tr>
## <tr> <td> 'din' </td> <td> 'din' </td> <td> '0' </td> </tr>
## <tr> <td> 'cdb' </td> <td> 'cdb' </td> <td> '0' </td> </tr>
## <tr> <td> 'op' </td> <td> 'op' </td> <td> '0' </td> </tr>
## <tr> <td> 'usleep' </td> <td> 'usleep' </td> <td> '0' </td> </tr>
## <tr> <td> 'dlimit' </td> <td> 'dlimit' </td> <td> '0' </td> </tr>
## <tr> <td> 'dir' </td> <td> 'dir' </td> <td> '0' </td> </tr>
## <tr> <td> 'ranges' </td> <td> 'ranges' </td> <td> '0' </td> </tr>
## <tr> <td> 'msleep' </td> <td> 'msleep' </td> <td> '0' </td> </tr>
## <tr> <td> 'sname' </td> <td> 'sname' </td> <td> '0' </td> </tr>
## <tr> <td> 'help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> 'dsf' </td> <td> 'dsf' </td> <td> '0' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> <td> '0' </td> </tr>
## <tr> <td> 'quit' </td> <td> 'quit' </td> <td> '1' </td> </tr>
## <tr> <td> 'script' </td> <td> 'script' </td> <td> '0' </td> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> 'scsiopcodes' </td> <td> 'scsiopcodes' </td> <td> '1' </td> </tr>
## <tr> <td> 'version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> 'path' </td> <td> 'path' </td> <td> '1' </td> </tr>
## <tr> <td> 'qtag' </td> <td> 'qtag' </td> <td> '0' </td> </tr>
## <tr> <td> 'threads' </td> <td> 'threads' </td> <td> '0' </td> </tr>
## <tr> <td> 'pattern' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> 'length' </td> <td> 'length' </td> <td> '0' </td> </tr>
## <tr> <td> 'onerr' </td> <td> 'onerr' </td> <td> '0' </td> </tr>
## <tr> <td> 'dout' </td> <td> 'dout' </td> <td> '0' </td> </tr>
## <tr> <td> 'sleep' </td> <td> 'sleep' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /usr/software/test/bin/spt dsf=/dev/sda cdb='12 00 00 00 ff 00' dir=read length=255
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Offset  Data
##&nbsp;000000  00 00 02 02 1f 00 00 73 56 4d 77 61 72 65 20 20 "       sVMware  "
##&nbsp;000016  56 69 72 74 75 61 6c 20 64 69 73 6b 20 20 20 20 "Virtual disk    "
##&nbsp;000032  31 2e 30 20                                     "1.0 "
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          '00',
##&nbsp;          '00',
##&nbsp;          '02',
##&nbsp;          '02',
##&nbsp;          '1f',
##&nbsp;          '00',
##&nbsp;          '00',
##&nbsp;          '73',
##&nbsp;          '56',
##&nbsp;          '4d',
##&nbsp;          '77',
##&nbsp;          '61',
##&nbsp;          '72',
##&nbsp;          '65',
##&nbsp;          '20',
##&nbsp;          '20',
##&nbsp;          '56',
##&nbsp;          '69',
##&nbsp;          '72',
##&nbsp;          '74',
##&nbsp;          '75',
##&nbsp;          '61',
##&nbsp;          '6c',
##&nbsp;          '20',
##&nbsp;          '64',
##&nbsp;          '69',
##&nbsp;          '73',
##&nbsp;          '6b',
##&nbsp;          '20',
##&nbsp;          '20',
##&nbsp;          '20',
##&nbsp;          '20',
##&nbsp;          '31',
##&nbsp;          '2e',
##&nbsp;          '30',
##&nbsp;          '20'
##&nbsp;        ];
##&nbsp;

use Data::Dumper;
my @result_array = ();

sub spt {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error_message;

    my $error = _check_error(
        "raw_output"        => $output,
        "command"           => $command,
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        if ($error_level == 0) {
            $Log->warn("Due to a bug in 'spt', a zero execution status"
                    . "\n was returned even when the command"
                    . "\nexecution has failed.\n");
        }
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

    my @lines
        = get_lines(raw_output => $output, first_line => 'Offset\s+Data');
    shift @lines;

    foreach my $line (@lines) {
        if ($line =~ /\S+\s+([0-9a-f\s]+)\b\s+".*"/) {
            my @tmp = split /\s+/, $1;
            push @result_array, @tmp;
        }
    }
    return \@result_array;
} ## end sub spt
1;
