#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse touch command
##
## @description
##    This is a library to parse touch command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "touch" will be touch()
##
## @synopsis
##    use touch;
##    touch(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::touch;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::touch
##
## @summary Parses the raw output from the "touch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from touch with any of its options as described in the
##    corresponding cdef file 'touch.cdefs'
## @synopsis
##      touch(%args);
## INPUTS TO METHOD CALL 'touch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>

## The below options are supported in Linux,Solaris and FreeBsd
## <tr> <td> '-a' </td> <td> 'access_time' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'no_create' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'modification_time' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reference' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'stamp' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>

## The below options support Linux only
## <tr> <td> '-d' </td> <td> 'set_date' </td> <td> '0' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'no-dereference' </td> <td> '1' </td> </tr>
## <tr> <td> '--time' </td> <td> 'time' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '1' </td> </tr>

## The below options support FreeBsd only
## <tr> <td> '-A' </td> <td> 'adjust-access-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'symbolic-link' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>

## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = touch file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -a file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -c file1.txt file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -d 12121212 file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -m file1.txt file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -r file1.txt file2.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch -t 12121212 file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch --time=atime file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;touch (GNU coreutils) 5.97
##&nbsp;Copyright (C) 2006 Free Software Foundation, Inc.
##&nbsp;This is free software.  You may redistribute copies of it under the terms of
##&nbsp;the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.
##&nbsp;There is NO WARRANTY, to the extent permitted by law.
##&nbsp;
##&nbsp;Written by Paul Rubin, Arnold Robbins, Jim Kingdon, David MacKenzie, and Randy Smith.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = touch --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;touch (GNU coreutils) 5.97
##&nbsp;Copyright (C) 2006 Free Software Foundation, Inc.
##&nbsp;This is free software.  You may redistribute copies of it under the terms of
##&nbsp;the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.
##&nbsp;There is NO WARRANTY, to the extent permitted by law.
##&nbsp;
##&nbsp;Written by Paul Rubin, Arnold Robbins, Jim Kingdon, David MacKenzie, and Randy Smith.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub touch {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    return [];

} ## end sub touch

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(touch):\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /([uU]sage):\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /touch:\s+missing file operand.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
