#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @name ::Transition
##
## @summary Contains methods to execute transition commands
##
## @description
##    Using this library, user can perform transition related task.
##    Methods in this library has a one to one mapping to the CLI parsers
##
## @synopsis
##    use NACL::APISet::Host::GUI::SMTT::V1_3::Transition;
##
##    transition_credentials_add(%args);
##    transition_create(%args);
##    transition_volumepair_add(%args);
##    transition_volumepair_remove(%args);
##    transition_lif_add(%args);
##    transition_lif_remove(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_schedule_add(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_precutover(%args);
##    transition_resync(%args);
##    transition_update(%args);
##    transition_cutover(%args);
##    transition_abort(%args);
##    transition_schedule_modify(%args);
##    transition_schedule_remove(%args);
##    transition_delete(%args);
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @status Review
#

package NACL::APISet::Host::GUI::SMTT::V1_3::Transition;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SMTT::V1_3::GlobalConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    transition_credentials_add
    transition_create
    transition_volumepair_add
    transition_volumepair_remove
    transition_lif_add
    transition_lif_remove
    transition_schedule_add
    transition_schedule_modify
    transition_schedule_remove
    transition_precheck
    transition_start
    transition_precutover
    transition_resync
    transition_cutover
    transition_update
    transition_pause
    transition_resume
    transition_abort
    transition_delete
);

## @name ::transition_credentials_add
##
## @summary Executes "transition credentials add" command
##
## @description
##    This method is used to execute transition credentials add command
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##      $Api_Set_Obj2->transition_credentials_add(
##            "host-name" => "10.227.202.109",
##            "username"  => "root",
##            "password"  => "netapp1!",
##      );
##
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;

sub transition_credentials_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "host-name"  => { type => SCALAR },
            "username"   => { type => SCALAR },
            "password"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Credentials add page");
    $self->_smtt_navigate($NAVIGATE_TRANSITION_STORAGE);

    $self->_add_credentials(%args);

    $self->_close_tab(
        string => "Storage",
    );
    $Log->comment("Credentials added successfully");
    $Log->exit();

    return 1;
} ## end sub transition_credentials_add

## @name ::transition_create
##
## @summary Executes "transition_create" command
##
## @description
##    This method is used to execute transition create command
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'svm-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'S_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'C_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'volumepair_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'schedule_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'seven-mode-only' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##       my %S_mode_credentials=(
##            "host-name"                        => "10.227.202.109",
##            username                           => "root",
##            password                           => "netapp1!"
##       );
##       my %C_mode_credentials=(
##            "host-name"                        => "10.227.65.55",
##            username                           => "admin",
##            password                           => "netapp1!"
##       );
##       my %volume=(
##            "volume-name-7-mode"               => [qw(r1 r2)],
##            "volume-name-c-mode"               => [qw(r1234 r2234)],
##            "aggregate-name"                   => [qw(aggr_trans aggr_trans)],
##       );
##
##       my %schedule1=(
##            "schedule-name"                    => "new1",
##            "days-range"                       => "1-4",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##       );
##       my %schedule2=(
##            "schedule-name"                    => "new2",
##           "days-range"                        => "0,5,6",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##        );
##        my @schedule=(\%schedule1,\%schedule2);
##        $Api_Set_Obj2->transition_create(
##             "session-name"              => "session2",
##             "session-type"              => "standalone",
##             "7-mode-host-name"          => "10.227.202.109",
##             "data-copy-ipaddress"       => "10.227.202.109",
##             "vfiler"                    => "vfiler0",
##             "multipathing-ipaddress"    => "10.227.202.110",
##             "c-mode-host-name"          => "10.227.65.55",
##             "c-mode-vserver-name"       => "r1",
##             "vserver-rootvol-aggr"      => { type => SCALAR },
##             S_mode_credentials          => \%S_mode_credentials,
##             C_mode_credentials          => \%C_mode_credentials,
##             volumepair_add_args         => \%volume,
##             schedule_add_args           => \@schedule,
##             "seven-mode-only"           => 1,
##        );
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily :
##&nbsp;

sub transition_create {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR , optional => 1},
            "session-type"              => { type => SCALAR , optional => 1},
            "7-mode-host-name"          => { type => SCALAR , optional => 1},
            "data-copy-ipaddress"       => { type => SCALAR , optional => 1},
            "vfiler"                    => { type => SCALAR , optional => 1},
            "multipathing-ipaddress"    => { type => SCALAR , optional => 1},
            "c-mode-host-name"          => { type => SCALAR , optional => 1},
            "svm-name"                  => { type => SCALAR , optional => 1},

            "S_mode_credentials"        => { optional => 1  },
            "C_mode_credentials"        => { optional => 1  },
            "volumepair_add_args"       => { optional => 1  },
            "schedule_add_args"         => { type => ARRAYREF, optional => 1},

            "seven-mode-only"           => { type => SCALAR , optional => 1},

            "primary_info"              => { optional => 1 },
            "secondary_info"            => { optional => 1 },

        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if (defined $args{primary_info} && defined $args{secondary_info} ) {

        my $primary_info        = delete $args{primary_info};
        my $secondary_info      = delete $args{secondary_info};

        my $S_mode_credentials_P = delete $primary_info->{S_mode_credentials};
        my $C_mode_credentials_P = delete $primary_info->{C_mode_credentials};

        $self->_click(
            id                      => $STORAGE_SYSTEMS,
            error_checking_needed   => 0,
        ) ;
        $self->_add_credentials( %{$S_mode_credentials_P} );
        $self->_add_credentials( %{$C_mode_credentials_P} );
        $self->_close_tab(
            string => "Storage",
        );

        $secondary_info->{volumepair_add_args}->{"volume-name-7-mode"}
            = $primary_info->{volumepair_add_args}->{"volume-name-7-mode"};
        $secondary_info->{"7-mode-host-name"} = $primary_info->{"7-mode-host-name"};
        $self->_create_helper($secondary_info);
        my $prim_sess = $primary_info->{"session-name"};
        $primary_info->{"session-name"} = $secondary_info->{"session-name"}."_primary";
        $self->_create_primary($primary_info,$prim_sess);
    } else {
        $self->_create_helper(\%args);
    }

    $Log->exit();

    return 1;
} ## end sub transition_create

## @name ::transition_volumepair_add
##
## @summary Executes "transition_volumepair_add" command
##
## @description
##    This method is used to execute transition volumepair add command
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_add(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1)],
##            "volume-name-c-mode"        => [qw(r1_1)],
##            "aggregate-name"            => [qw(aggr_trans)],
##         );
##
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;

sub transition_volumepair_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
            "volume-name-c-mode"        => { type => ARRAYREF , optional => 1},
            "aggregate-name"            => { type => ARRAYREF , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Storing volume pair add arguments in the object
    foreach (@{$self->{$args{"session-name"}}}){
        undef $_ if (defined $_->{transition_volumepair_add_args});
    }
    @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
    push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => \%args} ;

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 1,
        ) ;

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        $self->_add_volumepairs_for_session($args{"session-name"},1);

        $self->_click(
            id                      => $SAVE_PROJECT_AND_CONTINUE,
            error_checking_needed   => 1,
        ) ;

        if ( (defined $args{"aggregate-name"}) || (defined $args{"volume-name-c-mode"}) ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;
        
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});
        
            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }

        $self->_close_tab(
            session => $args{"session-name"},
        );

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        if($self->_is_key_x_present_for_session_y("transition_create_args",$args{"session-name"})){

            $Log->comment("Creating session and adding volumepairs");

            # Session does not exist
            $self->_close_tab(
                string => "Dashboardtab",
            ) if( $dashboard_enabled ) ;

            # Get hostname from ip and store it in APISet object in session hash
            my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","7-mode-host-name",$args{"session-name"});
            my $host = $self->_get_host_from_ip($ip,"s");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","s_hostname",$args{"session-name"},$host);

            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","c-mode-host-name",$args{"session-name"});
            $host = $self->_get_host_from_ip($ip,"c");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","c_hostname",$args{"session-name"},$host);

            # Project details
            $self->_smtt_navigate($NAVIGATE_TRANSITION_CREATE);

            # Create new project here
            $Log->debug("Creating new project");
            $self->_type( $PROJECT_NAME_INPUT ,$args{"session-name"} );
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;

            # Wait till volume pairs are loaded
            $self->_check_for_error();

            # Select volumepairs
            $self->_add_volumepairs_for_session($args{"session-name"});

            # Continuing with the remaining part of session creation
            $self->_click(
                id                      => $CREATE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;

            # Changing session name to desired value
            $self->_change_session_name_to_given_value($args{"session-name"});

            my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","data-copy-ipaddress",$args{"session-name"});
            $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

            my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","multipathing-ipaddress",$args{"session-name"});
            $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;

            # Vserver selection page
            $self->_add_vserver($args{"session-name"});
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_close_tab(
                session => $args{"session-name"},
            );
            
            $Log->comment("Session created and volumepairs added successfully");
            
        }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist "."
                and create arguments not present in the APISet object.\n ".
                "Make sure that you invoke transition_volumepair_add() on the same APISet".
                " object \n on which you invoked transition_create() or session should exist"
            );
        }
    }

    $Log->exit();

    return 1;
} ## end sub transition_volumepair_add

## @name :: transition_volumepair_remove
##
## @summary Executes "transition_volumepair_remove" command
##
## @description
##  This method is used to execute transition volume remove from session
## @synopsis
##  transition_volumepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1 r2)],
##         );
##
##&nbsp;transition volumepair remove -s S1 -v vol1 :
##&nbsp;

sub transition_volumepair_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->comment("Navigating to Dashboard and selecting a subproject");

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;

        my $vol_ar = $args{"volume-name-7-mode"};

        foreach (0..@{$vol_ar}-1){
            my $vol = $vol_ar->[$_];
            my $remove=$VOLUME_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                ) ;
            }else{
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Probably the volume requested for does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed volumes from session");
        }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }

    $Log->exit();

    return 1;
} ## end sub transition_volumepair_remove

## @name :: transition_lif_add
##
## @summary Executes "transition_lif_add" command
##
## @description
##  This method is used to execute transition lif add command
## @synopsis
##  transition_lif_add(%args);
## INPUTS TO METHOD CALL 'transition_lif_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'default-gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'home-node' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'home-port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"             => "session2",
##            "ip-address"        => "1.12.255.7",
##            "default-gateway"           => "1.12.255.1",
##            "netmask"           => "255.0.0.0",
##            "home-node"              => "csiqa-2240-03",
##            "home-port"              => "e0d",
##            "origin-ip-address"        => "1.12.255.7",
##         );
##&nbsp;
sub transition_lif_add {

    my $self = shift;
    my $sleep_count = $GENERIC_WAIT / 1000;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                          => { type => SCALAR },
            "ip-address"                            => { type => SCALAR },
            "default-gateway"                       => { type => SCALAR , optional => 1},
            "netmask"                               => { type => SCALAR , optional => 1},
            "home-node"                             => { type => SCALAR , optional => 1},
            "home-port"                             => { type => SCALAR , optional => 1},
            "origin-ip-address"                     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 0,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );

        $self->_click(
            id                      => $NETWORKING_TAB,
            error_checking_needed   => 0,
        );

        #Invoke Add LIF window
        $self->_click(
            id                      => $GENERIC_ADDLIF_BUTTON,
            error_checking_needed   => 0,
        );

        sleep $sleep_count;
        #If origin IP address speicifed choose IP address from 7mode
        if (defined $args{"origin-ip-address"}) {

            $self->_click(
                id                      => $EXISTING_LIF_RADIO_BUTTON,
                error_checking_needed   => 0,
                );

            $self->_click(
                id                      => $GENERIC_ORIGIN_IP_INPUT,
                error_checking_needed   => 0,
                );

            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $ipaddress = $args{"origin-ip-address"};
            my $originIPselector =  $GENERIC_LISTITEM_SELECTOR;
            $originIPselector =~ s/(.*)XXYYZZ(.*)/$1$ipaddress$2/;

            $self->_click(
                id                      => $originIPselector,
                error_checking_needed   => 0,
                );

        }   else{
            #Choose add new LIF option
            $self->_click(
                id                      => $ADD_NEW_LIF_RADIO_BUTTON,
                error_checking_needed   => 0,
                );

            $self->_type( $GENERIC_TARGET_IP_INPUT ,$args{"ip-address"});
        }

        if (defined $args{"default-gateway"}){
            $self->_type( $GENERIC_TARGET_GATEWAY_INPUT ,$args{"default-gateway"});
        }

        if (defined $args{"netmask"}){
            $self->_type( $GENERIC_TARGET_NETMASK_INPUT ,$args{"netmask"});
        }

        if (defined $args{"home-node"}){
            $self->_click(
                id                      => $GENERIC_TARGET_NODE_INPUT,
                error_checking_needed   => 0,
                );
            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $node = $args{"home-node"};
            my $nodeselector =  $GENERIC_LISTITEM_SELECTOR;
            $nodeselector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;
            $self->_click(
                id                      => $nodeselector,
                error_checking_needed   => 0,
                );
        }

        if (defined $args{"home-port"}){
            $self->_click(
                id                      => $GENERIC_TARGET_PORT_INPUT,
                error_checking_needed   => 0,
                );
             # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $port = $args{"home-port"};
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$port$2/;
            $self->_click(
                id                      => $portselector,
                error_checking_needed   => 0,
                );
        }

        $self->_click(
            id                      => $GENERIC_SAVELIF_BUTTON,
            error_checking_needed   => 1,
            );
        $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
        sleep $sleep_count;

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_lif_add

## @name :: transition_lif_remove
##
## @summary Executes "transition_lif_remove" command
##
## @description
##  This method is used to execute transition lif remove from session
## @synopsis
##  transition_lif_remove(%args);
## INPUTS TO METHOD CALL 'transition_lif_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_lif_remove(
##            "session-name"              => "session2",
##            "ip-address"                => [qw(1.12.255.2 1.12.255.3)],
##         );
##
##&nbsp;
##&nbsp;

sub transition_lif_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "ip-address"                => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->comment("Navigating to Dashboard and selecting a subproject");

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $NETWORKING_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;

        my $ip_ar = $args{"ip-address"};

        foreach (0..@{$ip_ar}-1){
            my $ipaddress = $ip_ar->[$_];
            my $remove=$LIF_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$ipaddress$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                );
            }else{
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Probably the lif requested does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed lifs from session");
        }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }

    $Log->exit();

    return 1;
} ## end sub transition_lif_remove

## @name :: transition_schedule_add
##
## @summary Executes "transition_schedule_add" command
##
## @description
##  This method is used to execute transition schedule add command
## @synopsis
##  transition_schedule_add(%args);
## INPUTS TO METHOD CALL 'transition_shedule_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR },
            "start-time"                        => { type => SCALAR },
            "duration"                          => { type => SCALAR },
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $CREATE_SCHEDULE_DASHBOARD,
            error_checking_needed   => 0,
        ) ;

        $self->_type( $GENERIC_SCHEDULE_NAME ,$args{"schedule-name"});

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        my $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/; 
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_add

## @name ::transition_precheck
##
## @summary Executes "transition_precheck" command
##
## @description
##    This method is used to execute transition precheck command
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##        $Api_Set_Obj2->transition_precheck(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##        );
##&nbsp;

sub transition_precheck {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'seven-mode-only'               => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Running prechecks . Will throw an exception if found errors
        if( $self->_is_button_enabled($RUN_PRECHECKS) ) {

            $aref = $self->_click(
                id                      => $RUN_PRECHECKS,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till precheck operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_PRECHECK,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception(
                "Transition precheck on ".$args{"session-name"}." can not be "
                ."run as precheck button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_precheck

## @name ::transition_start
##
## @summary Executes "transition_start" command
##
## @description
##    This method is used to execute transition start command
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_start(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##         );
##&nbsp;transition start -r no -s S21 :
##&nbsp;

sub transition_start {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'no-precheck'                   => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($START) ) {

            $aref = $self->_click(
                id                      => $START,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till start operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_START,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition start on ".$args{"session-name"}
                ." can not be run as start button is disabled");
        }

        Tharn::snooze(5);

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_start

## @name :: transition_update
##
## @summary executes "transition_update" command
##
## @description
##  This method is used to execute transition update command
## @synopsis
##  transition_update(%args);
## INPUTS TO METHOD CALL 'transition_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_update {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'available-transfers-percentage'    => { type => SCALAR },
            'session-snapmirror-throttle'       => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($UPDATE_NOW) ) {
            $aref = $self->_click(
                id                      => $UPDATE_NOW,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition update on ".$args{"session-name"}
                ." can not be run as Update Now button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_update

## @name ::transition_pause
##
## @summary executes "transition_pause" command
##
## @description
##    This method is used to execute transition pause command
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_pause {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($PAUSE) ) {
    
            $aref = $self->_click(
                id                      => $PAUSE,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till pause operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                my $pause_completed = 0;
                while (1){
                    if ( ( $self->_is_element_present($WAIT_FOR_PAUSE_IN_BASELINE) ) ||
                         ( $self->_is_element_present($WAIT_FOR_PAUSE_IN_INCREMENTAL_D_XFER) ) 
                    ) {
                        $pause_completed = 1;
                        last;
                    }
                    $timeout--;
                    last unless $timeout;
                }

                unless ( $pause_completed ){
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Even after waiting for "
                        ."$args{'method-timeout'} secs , still the pause did not succeed");
                }

            }
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition pause on ".$args{"session-name"}
                ." can not be run as Pause button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_pause

## @name ::transition_resume
##
## @summary executes "transition_resume" command
##
## @description
##    This method is used to execute transition resume command
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resume {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($RESUME) ) {

            $aref = $self->_click(
                id                      => $RESUME,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till resume operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                my $resume_completed = 0;
                while (1){
                    if ( !( $self->_is_element_present($WAIT_FOR_PAUSE_IN_BASELINE) ) &&
                         !( $self->_is_element_present($WAIT_FOR_PAUSE_IN_INCREMENTAL_D_XFER) ) 
                    ) {
                        $resume_completed = 1;
                        last;
                    }
                    $timeout--;
                    last unless $timeout;
                }

                unless ( $resume_completed ){
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Even after waiting for"
                        ." $args{'method-timeout'} secs , still the resume did not succeed");
                }

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition resume on ".$args{"session-name"}
                ." can not be run as Resume button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_resume

## @name :: transition_precutover
##
## @summary Executes "transition_precutover" command
##
## @description
##  This method is used to execute transition precutover command
## @synopsis
##  transition_precutover(%args);
## INPUTS TO METHOD CALL 'transition_precutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapmirror-throttle' <td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_precutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'mode'                          => { type => SCALAR },
            'available-transfers-percentage'=> { type => SCALAR },
            "max-cap-concurrent-transfers"  => { type => SCALAR , optional => 1},
            'snapmirror-throttle'           => { type => SCALAR , optional => 1},
            'ad-server-host-name'           => { type => SCALAR , optional => 1},
            'ad-server-ip-address'          => { type => SCALAR , optional => 1},
            'ad-server-admin-user-name'     => { type => SCALAR , optional => 1},
            'ad-server-password'            => { type => SCALAR , optional => 1},
            'kdc-server-host-name'          => { type => SCALAR , optional => 1},
            'kdc-server-admin-user-name'    => { type => SCALAR , optional => 1},
            'kdc-server-password'           => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($PRECUTOVER) ) {

            if( $args{mode} eq "rw_test" ){
                $self->_click(
                    id                      => $TEST_MODE,
                    error_checking_needed   => 1,
                ) ;
            }

            $aref = $self->_click(
                id                      => $PRECUTOVER,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till precutover operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                if( $args{mode} eq "rw_test" ){
                    $self->_wait_on_attribute(
                        attribute_to_check   => $RESYNC,
                        "method-timeout"     => $timeout,
                    ) ;
                } else {
                    $self->_wait_on_attribute(
                        attribute_to_check   => $WAIT_FOR_PRECUTOVER,
                        "method-timeout"     => $timeout,
                    ) ;
                }

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition precutover on ".$args{"session-name"}
                ." can not be run as Copy-Configuration button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_precutover

## @name :: transition_resync
##
## @summary Executes "transition_resync" command
##
## @description
##  This method is used to execute re-sync on a transition session
## @synopsis
##  transition_resync(%args);
## INPUTS TO METHOD CALL 'transition_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resync {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($RESYNC) ) {
            
            $aref = $self->_click(
                id                      => $RESYNC,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till resync operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_RESYNC,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition resync on ".$args{"session-name"}
                ." can not be run as Finish Testing button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_resync

## @name :: transition_cutover
##
## @summary Executes "transition_cutover" command
##
## @description
##  This method is used to execute transition cutover command
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_cutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'available-transfers-percentage'    => { type => SCALAR },
            'session-snapmirror-throttle'       => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($CUTOVER) ) {

            $aref = $self->_click(
                id                      => $CUTOVER,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till cutover operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_CUTOVER,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition cutover on ".$args{"session-name"}
                ." can not be run as Complete Transition button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_cutover

## @name ::transition_abort
##
## @summary Executes "transition_abort" command
##
## @description
##    This method is used to execute transition abort command
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_abort {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {

        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($ABORT) ) {

            $aref = $self->_click(
                id                      => $ABORT,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till abort operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_ABORT,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition abort on ".$args{"session-name"}
                ." can not be run as Abort button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_abort

## @name :: transition_schedule_modify
##
## @summary Executes "transition_schedule_modify" command
##
## @description
##  This method is used to execute transition schedule modify command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_modify {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR , optional => 1},
            "start-time"                        => { type => SCALAR , optional => 1},
            "duration"                          => { type => SCALAR , optional => 1},
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_MODIFIER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        # Clearing off all the days
        foreach (0..6){
            my $days = {
                0 => "sunday",
                1 => "monday",
                2 => "tuesday",
                3 => "wednesday",
                4 => "thursday",
                5 => "friday",
                6 => "saturday",
            };
            my $day = $days->{$_};
            my $selector =  $GENERIC_SCHEDULE_DAYS_SELECT;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$day$2/;
            my $checked = $selector.'[@checked=""]';
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) if ( $self->_is_element_present($checked) );
        }

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        my $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/; 
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_modify

## @name :: transition_schedule_remove
##
## @summary Executes "transition_schedule_remove" command
##
## @description
##  This method is used to execute transition schedule remove command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_REMOVER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_remove

## @name :: transition_delete
##
## @summary Executes "transition_delete" command
##
## @description
##  This method is used to execute transition delete command
## @synopsis
##  transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;

sub transition_delete {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        my $project_id = $self->_get_project_id($id);

        $self->_click(
            id                      => $project_id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $DELETE_PROJECT,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_delete

1;
