# $Id$
#---------------------------------------------------
# Copyright (c) 2014 Network Appliance, Inc.
# All rights reserved.
#---------------------------------------------------
## @summary
##      Wrapper Utility for Selnium webDriver functions.
##
## @description
##      This modules contains all helper functions for SMTT libraries
##      to interact with selenium webdriver
##
## @synopsis
##      use NACL::APISet::Host::GUI::SMTT::V1_4::SMTTUtils;
##
##          _login(%args);
##          _wait_for_element_not_present(%args);
##          _wait_for_element_present(%args);
##          _wait_for_loading(%args);
##          _is_element_present(%args);
##          _is_element_enabled(%args);
##          _find_element(%args);
##          _find_elements(%args);
##          _type(%args);
##          _click(%args);
##          _check_for_error(%args);
##          _get_all_window_names(%args);
##          _select_window(%args);
##          _is_checked(%args);
##          _get_value(%args);
##          _get_text(%args);
##          _clear_window(%args);
##          _click_button_having_specific_text(%args);
##          _click_button_having_specific_text_no_error_check(%args);
##          _smtt_navigate(%args);
##          _close_tab(%args);
##          _throw_exception(%args);
##          _take_screenshot(%args);
##          _repair_session(%args);
##          _get_host_from_ip(%args);
##          _session_exist(%args);
##          _get_all_sessions(%args);
##          _add_volumepairs_for_session(%args);
##          _add_vserver(%args);
##          _select_aggregate(%args);
##          _do_volume_mapping(%args);
##          _change_session_name_to_given_value(%args);
##          _add_volumes_given_vol_ref_array(%args);
##          _is_button_enabled(%args);
##          _wait_on_attribute(%args);
##          _add_credentials(%args);
##          _add_schedule_days(%args)
##          _is_key_x_present_for_session_y(%args);
##          _get_value_for_key_x_key_y_in_session_z(%args);
##          _put_value_for_key_x_key_y_in_session_z(%args);
##          _get_volume_sm_details(%args);
##          _get_data_copy_schedule_details(%args);
##          _create_helper(%args);
##          _create_primary(%args);
##          _get_volume_selection_details(%args);
##          _get_volume_selection_details_helper(%args);
##
##          is_element_present(%args);
##          is_element_enabled(%args);
##          is_checked(%args);
##          get_value(%args);
##          get_text(%args);
##          get_all_sessions(%args);
##          is_button_enabled(%args);
##          findElement(%args);
##          findElements(%args);
##          get_all_window_names(\%args);
##          session_exist(\%args);
##          set_page(\%args);
##          get_session_details(\%args);
##
## @status Review
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com

package NACL::APISet::Host::GUI::SMTT::V1_4::SMTTUtils;

#
## Compiler directives.
#
use strict;
use warnings;

## Module import.
use Selenium::Remote::WebElement;
use Selenium::Remote::WDKeys;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Host::GUI::SMTT::V1_4::GlobalConst;
use NACL::APISet::Host::GUI::SMTT::GConst;
use NATE::BaseException qw(:try);

use base 'Selenium::Remote::Driver';

################################################################################
# P U B L I C   M E T H O D S
################################################################################

###############################################################################
## @name ::_login
##
## @summary
##     Logs in to SMTT
## @description
##     This method can be used to log into the SMTT
## @synopsis
##     _login(%args);

sub _login {

    my ( $self ) = @_;

    $Log->enter();

    $self->_type( $LOGIN_USERNAME ,$self->{username});
    $self->_type( $LOGIN_PASSWORD ,$self->{password});

    $self->_click(
        id                      => $LOGIN_BUTTON,
        error_checking_needed   => 0,
    ) ;
    my @elems=$self->find_elements("//*[\@type='button']");
    $elems[1]->click();
    $Log->exit();

    return 1;
} ## end sub _login

###############################################################################
## @name ::_wait_for_element_not_present
##
## @summary
##     This method is used to wait till the element disappears or timeout occurs
## @synopsis
##     _wait_for_element_not_present(%args);

sub _wait_for_element_not_present {

    my ( $self, $debugID, $timeout ) = @_;

    $Log->enter();

    my $retVal = 1;
    $self->_wait_for_loading();

    my $sleep_count = $timeout ? $timeout : ($GENERIC_WAIT / 1000);
    for ( my $i = 0 ; $i < $sleep_count ; $i++ ) {
        eval {
            $self->_find_element($debugID);
            $Log->comment("Element $debugID still exists");
            Tharn::snooze(1);
            $retVal = 0;
        };
        if ($@) {
            $retVal = 1;
            last;
        }
    }

    $Log->exit();

    return $retVal;
} ## end sub _wait_for_element_not_present

###############################################################################
## @name ::_wait_for_element_present
##
## @summary
##     This method is used to wait till the element appears or timeout occurs
## @synopsis
##     _wait_for_element_present(%args);

sub _wait_for_element_present {

    my ( $self, $debugID, $timeout ) = @_;

    $Log->enter();

    my $retVal = 0;
    $self->_wait_for_loading();

    my $sleep_count = $timeout ? $timeout : ($GENERIC_WAIT / 1000);
    for ( my $i = 0 ; $i < $sleep_count ; $i++ ) {
        eval {
            $self->_find_element($debugID);
            $retVal = 1;
        };
        if ($@) {
            $Log->comment( "Waiting for $debugID : " . $@ );
            Tharn::snooze(1);
        } else {
            last;
        }
    }

    $Log->exit();

    return $retVal;
} ## end sub _wait_for_element_present

###############################################################################
## @name ::_wait_for_loading
##
## @summary
##     This method is used to wait till the page loads
## @synopsis
##     _wait_for_loading(%args);

sub _wait_for_loading {

    my ( $self ) = @_;

    $Log->enter();

    my $sleep_count = $self->{timeout} || $GENERIC_WAIT / 1000;
    Tharn::snooze(2);

    $self->set_implicit_wait_timeout(1000);

    while ($self->_is_element_present($GENERIC_PLEASE_WAIT) ) {
        $Log->comment("Waiting for Page to load..");
        Tharn::snooze(1);
        $sleep_count = $sleep_count - 1;
        if ($sleep_count == 0) {
            $self->_throw_exception("Page load timeout in _wait_for_loading()");
        }
    }

    $self->set_implicit_wait_timeout(5000);

    $Log->exit();

    return 1;
} ## end sub _wait_for_loading

###############################################################################
## @name ::_is_element_present
##
## @summary
##     This method is used check if the element exists on the page
## @synopsis
##     _is_element_present(%args);

sub _is_element_present {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    eval { $self->_find_element($debugID); };
    if ($@) {
        $Log->exit();
        return 0;
    }

    $Log->exit();

    return 1;
} ## end sub _is_element_present

###############################################################################
## @name ::is_element_present
##
## @summary
##     This method is used check if the element exists on the page
## @synopsis
##     is_element_present(%args);

sub is_element_present {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_is_element_present($debugID);

    $Log->exit();

    return $output;
} ## end sub is_element_present

###############################################################################
## @name ::_is_element_enabled
##
## @summary
##     This method is used check if the element is enabled
## @synopsis
##     _is_element_enabled(%args);

sub _is_element_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $res;
    eval {
        my $elem = $self->_find_element($debugID);
        $res = $elem->is_enabled();
    };

    if ($@) {
        $Log->exit();
        return 0 ;
    }
    unless ($res) {
        $Log->exit();
        return 0 ;
    }

    $Log->exit();

    return 1;
} ## end sub _is_element_enabled

###############################################################################
## @name ::is_element_enabled
##
## @summary
##     This method is used check if the element is enabled
## @synopsis
##     is_element_enabled(%args);

sub is_element_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_is_element_enabled($debugID);

    $Log->exit();

    return $output;
} ## end sub is_element_enabled

###############################################################################
## @name ::_find_element
##
## @summary
##     This method returns the element if it is found on the page
## @synopsis
##     _find_element(%args);

sub _find_element {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $elem_obj;
    if ( $debugID =~ /^\/\// ) {
        eval { $elem_obj = $self->find_element($debugID); };
        if ($@) {
            $self->_throw_exception("$@");
        }
    } else {
        eval { $elem_obj = $self->find_element( $debugID, 'id' ); };
        if ($@) {
            $self->_throw_exception("$@");
        }
    }

    $Log->exit();

    return $elem_obj;
} ## end sub _find_element

###############################################################################
## @name ::findElement
##
## @summary
##     This method returns the element if it is found on the page
## @synopsis
##     findElement(%args);

sub findElement {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_find_element($debugID);

    $Log->exit();

    return $output;
} ## end sub findElement

###############################################################################
## @name ::_find_elements
##
## @summary
##     This method returns the elements if they are found on the page
## @synopsis
##     _find_elements(%args);

sub _find_elements {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my @elem_obj ;
    if ( $debugID =~ /^\/\// ) {
        eval {
            @elem_obj = $self->find_elements($debugID);
        };
    } else {
        eval {
            @elem_obj = $self->find_elements( $debugID, 'id' );;
        };
    }

    $Log->exit();

    return @elem_obj;
} ## end sub _find_elements

###############################################################################
## @name ::findElements
##
## @summary
##     This method returns the elements if they are found on the page
## @synopsis
##     findElements(%args);

sub findElements {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my @elemt = $self->_find_elements($debugID);
    my $output->{parsed_output}->{response} = \@elemt ;

    $Log->exit();

    return $output;
} ## end sub findElements

###############################################################################
## @name ::_type
##
## @summary
##     This method is used for typing the text in the input box specified
## @synopsis
##     _type(%args);

sub _type {

    my ( $self, $debugID, $typeText ) = @_;

    $Log->enter();

    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);

    $Log->exit();

    return 1;
} ## end sub _type

###############################################################################
## @name ::_click
##
## @summary
##     This method is used to click the element specified
## @synopsis
##     _click(%args);

sub _click {

    my ( $self, %args ) = @_;

    $Log->enter();

    my $aref;

    if ( ref($self) =~ /WebElement/ ) {
        $self->click();
        $self->_wait_for_loading();
        $aref = $self->_check_for_error( args => $args{info_hash} )
            if ( $args{error_checking_needed}
        );
        $Log->exit();
        return $aref;
    }

    $Log->comment("Click Element : $args{id}");
    my $elem = $self->_find_element($args{id});
    $elem->click();
    $self->_wait_for_loading();
    $aref = $self->_check_for_error( info_hash => $args{info_hash} )
        if ( $args{error_checking_needed}
    );
    $Log->comment("Click on Element $args{id} successfull");

    $Log->exit();

    return $aref;
} ## end sub _click

###############################################################################
## @name ::_check_for_error
##
## @summary
##     This method is used to check for errors on the page after each click
## @synopsis
##     _check_for_error(%args);

sub _check_for_error{

    my ( $self, %args ) = @_;

    $Log->enter();

    # DS to store the parsed output
    my $aref=[];

    $self->set_implicit_wait_timeout(4000);

    while( $self->_is_element_present($GENERIC_POPUP) ){

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        my $popup_title;

        # Putting an eval, maybe popup was there but not present now
        eval{
            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
        };

        # Since popup was present earlier but not present now so skipping checking
        unless ($@) {

            if ( $popup_title =~ /Error/ ) {

                # Dying here with the error message
                my $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception(
                    "Popup Title : $popup_title , Popup error message : $popup_text");
    
            }elsif ( $popup_title =~ /Volume loading/ ){
    
                # Wait till all volumes are being loaded
                my $wait = $self->{timeout} || $GENERIC_VOLUMEPAIR_LOAD;
                unless( $self->_wait_for_element_not_present( $GENERIC_POPUP,$wait ) ){
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception(
                        "Volumes did not get loaded even after $wait s of wait");
                }
    
            }elsif ( $popup_title =~ /Warning/i || $popup_title =~ /Information/i
                || $popup_title =~ /Incomplete Subproject/i || $popup_title =~  /Add\/Remove Volumes list/i
                || $popup_title =~  /Confirmation/i || $popup_title =~  /Transition Pause/i
                || $popup_title =~  /Transition Abort/i ){
    
                # Handle all Generic pop-ups here
                # We dont want to fail if any click fails
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };

            }elsif ( $popup_title =~ /audit logs destination path/i ){

                # Handle Audit logging here
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };

            }elsif ( $popup_title =~ /New Transition Project Name/i ) {

                # Returning back from here and handling this in the caller
                $Log->exit();
                return 1;

            }elsif ( $popup_title =~ /Create Data Copy Schedule/i ) {

                # Returning back from here and handling this in the caller
                $Log->exit();
                return 1;

            }elsif ($popup_title =~ /Operation Progress/i || $popup_title =~ /Precheck Status/i ) {

                my $timer = $self->{timeout} || $GENERIC_PRECHECKS_WAIT;
                my $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                while ( ($new_popup_title !~ /Precheck Status/) && ($timer > 0) ) {

                    # Waiting here for prechecks to get completed
                    $timer--;
                    my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);
                    my $sleep_count = $GENERIC_WAIT / 1000;
                    Tharn::snooze($sleep_count);
                    $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                    if ( ($new_popup_title =~ /Operation Progress/i) && ($summary_text =~ /finished successfully/) ){
                        last;
                    }
                    if ( ($new_popup_title =~ /Operation Progress/i) && ($summary_text =~ /has failed/) ){
                        last;
                    }

                }

                if($timer == 0){
                    eval {
                        $self->_click(
                            id                      => $CLOSE_BUTTON,
                            error_checking_needed   => 0,
                        ) ;
                    };

                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception(
                        "Operation did not get completed even after 1800 secs");
                }

                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) if ($new_popup_title =~ /Precheck Status/);
                };

                my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);

                unless ($summary_text =~ /finished successfully/){
                    eval{
                        $self->_click(
                            id                      => $CLOSE_BUTTON,
                            error_checking_needed   => 0,
                        ) ;
                    };
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Operation completed with error : ".$summary_text);
                }

                # Getting job results data in a DS
                my $href={};
                $href->{Informational}=[];
                $href->{Error}=[];
                $href->{Warning}=[];
                eval {
                    $self->_click(
                        id                      => $VIEW_RESULTS,
                        error_checking_needed   => 0,
                    ) ;
                };
                if (! $@ ){
                    my @keys = ("code","type","message","corrective_action","affected_objects");

                    my $row = 1;
                    while (1){
                        my ( $type, %chunk_hash );
                        my $x=2;
                        my $y = $JOB_RESULTS;
                        $y =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$x$3/;
                        eval{
                            $type = $self->_get_text($y) ;
                        };
                        if ($@) {
                            last;
                        }
                        foreach my $col (1..@keys){
                            my $job_results = $JOB_RESULTS;
                            $job_results =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                            if ($col == 5){
                                if($self->_is_element_present($job_results)){
                                    $chunk_hash{"affected_objects"} = $self->_get_text($job_results);
                                } 
                            } else {
                                $chunk_hash{$keys[$col-1]} = $self->_get_text($job_results);
                            }
                        }

                        push @{$href->{$type}} , {%chunk_hash};

                        $row++;
                    }
                    push @$aref , $href;

                }

                eval{
                    $self->_click(
                        id                      => $CLOSE_BUTTON,
                        error_checking_needed   => 0,
                    ) ;
                };

            } else {
                # Closing the pop-up if none of the above conditions meet
                # We dont want to fail in case if the pop-up can't be closed
                eval {
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }
        }

    }

    $self->set_implicit_wait_timeout(1000);

    if($self->_is_element_present($GENERIC_PRECUTOVER)){
        # Add precutover arguments here
        eval {
             $self->_click(
                id                      => $ADD_INFO_DURING_PRECUTOVER,
                error_checking_needed   => 0,
            ) ;
        };

        if ( defined $args{"info_hash"}{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,
                $args{"info_hash"}{"available-transfers-percentage"}
            );
        }

        if ( defined $args{"info_hash"}{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_STREAM_SELECTOR ,
                $args{"info_hash"}{"max-cap-concurrent-transfers"}
            );
        }

        # Adding throttle
        if ( defined $args{"info_hash"}{"snapmirror-throttle"} ){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) ;
            $self->_type(
                $GENERIC_THROTTLE_SELECTOR ,
                $args{"info_hash"}{"snapmirror-throttle"}
            );
        }

        $aref = $self->_click(
            id                      => $CONTINUE_BUTTON,
            error_checking_needed   => 1,
        ) ;

    }

    if($self->_is_element_present($GENERIC_CUTOVER) || $self->_is_element_present($GENERIC_UPDATE)){

        # Add precutover arguments here
        eval {
             $self->_click(
                id                      => $ADD_INFO_DURING_PRECUTOVER,
                error_checking_needed   => 0,
            ) ;
        };

        if ( defined $args{"info_hash"}{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,
                $args{"info_hash"}{"available-transfers-percentage"}
            );
        }

        if ( defined $args{"info_hash"}{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type( $GENERIC_STREAM_SELECTOR ,
                $args{"info_hash"}{"max-cap-concurrent-transfers"}
            );
        }

        # Adding throttle
        if ( defined $args{"info_hash"}{"session-snapmirror-throttle"} ){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,
                $args{"info_hash"}{"session-snapmirror-throttle"}
            );
        }

        $aref = $self->_click_button_having_specific_text("Continue");

    }

    if($self->_is_element_present($GENERIC_SVM_AUDIT_PATH)){
        eval {
            $self->_click_button_having_specific_text("Skip");
        };
    }

    $self->set_implicit_wait_timeout(5000);

    $Log->exit();

    return $aref;
} ## end sub _check_for_error


###############################################################################
## @name ::_get_all_window_names
##
## @summary
##     This method is used to get the names of all windows which are opened
## @synopsis
##     _get_all_window_names(%args);

sub _get_all_window_names {

    my ( $self ) = @_;

    $Log->enter();

    my $handles = $self->get_window_handles;

    $Log->exit();

    return @$handles;
} ## end sub _get_all_window_names

###############################################################################
## @name ::get_all_window_names
##
## @summary
##     This method is used to get the names of all windows which are opened
## @synopsis
##     get_all_window_names(%args);

sub get_all_window_names {

    my ( $self ) = @_;

    $Log->enter();

    my @w_h = $self->_get_all_window_names();
    my $output->{parsed_output}->{response} = \@w_h ;

    $Log->exit();

    return $output;
} ## end sub get_all_window_names

###############################################################################
## @name ::_select_window
##
## @summary
##     This method is used to switch to the window specified
## @synopsis
##     _select_window(%args);

sub _select_window {

    my ( $self, $selWin ) = @_;

    $Log->enter();

    $self->switch_to_window($selWin);

    $Log->exit();

    return 1;
} ## end sub _select_window

###############################################################################
## @name ::_is_checked
##
## @summary
##     This method returns true if the element is selected/checked
## @synopsis
##     _is_checked(%args);

sub _is_checked {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $value;
    if ( ref($self) =~ /WebElement/ ) {
        $value = $self->is_selected();
        $Log->exit();
        return $value;
    }

    $value = $self->_find_element($debugID)->is_selected();

    $Log->exit();

    return $value;
} ## end sub _is_checked

###############################################################################
## @name ::is_checked
##
## @summary
##     This method returns true if the element is selected/checked
## @synopsis
##     is_checked(%args);

sub is_checked {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_is_checked($debugID);

    $Log->exit();

    return $output;

} ## end sub is_checked

###############################################################################
## @name ::_get_value
##
## @summary
##     This method returns the value of the element specified
## @synopsis
##     _get_value(%args);

sub _get_value {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output_value;
    if ( ref($self) =~ /WebElement/ ) {
        $output_value = $self->get_value();
        $Log->exit();
        return $output_value;
    }

    my $elem = $self->_find_element($debugID);
    $output_value = $elem->get_value();

    $Log->exit();

    return $output_value;
} ## end sub _get_value

###############################################################################
## @name ::get_value
##
## @summary
##     This method returns the value of the element specified
## @synopsis
##     get_value(%args);

sub get_value {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_get_value($debugID);

    $Log->exit();

    return $output;
} ## end sub get_value

###############################################################################
## @name ::_get_text
##
## @summary
##     This method returns the text of the element specified
## @synopsis
##     _get_text(%args);

sub _get_text {
# _find_element is not eval'ed so it can raise exception without capturing snapshot
    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output_text;
    if ( ref($self) =~ /WebElement/ ) {
        $output_text = $self->get_text();
        $Log->exit();
        return $output_text;
    }

    my $elem = $self->_find_element($debugID);
    $output_text = $elem->get_text();

    $Log->exit();

    return $output_text;
} ## end sub _get_text

###############################################################################
## @name ::get_text
##
## @summary
##     This method returns the text of the element specified
## @synopsis
##     get_text(%args);

sub get_text {
# _find_element is not eval'ed so it can raise exception without capturing snapshot
    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_get_text($debugID);

    $Log->exit();

    return $output;

} ## end sub get_text

###############################################################################
## @name ::_clear_window
##
## @summary
##     This method is used to close the pop-ups
## @synopsis
##     _clear_window(%args);

sub _clear_window {

    my ( $self ) = @_;

    $Log->enter();

    if($self->_is_element_present($GENERIC_POPUP)){
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;
    }

    $Log->exit();

    return 1;
} ## end sub _clear_window

###############################################################################
## @name ::_click_button_having_specific_text
##
## @summary
##     This method is used to click the button having the specified text
## @synopsis
##     _click_button_having_specific_text(%args);

sub _click_button_having_specific_text{

    my ( $self, $text ) = @_;

    $Log->enter();

    my $aref;

    my @elems=$self->find_elements("//*[\@type='button']");
    foreach (@elems){
        # $_ is a webelement object so cant use private methods
        if(($_->get_text()) =~ /$text/i){
            $_->click();
            $aref = $self->_check_for_error();
            last;
        }
    }

    $Log->exit();

    return $aref;
} ## end sub _click_button_having_specific_text

###############################################################################
## @name ::_click_button_having_specific_text_no_error_check
##
## @summary
##     This method is used to click the button having the specified text
## @synopsis
##     _click_button_having_specific_text_no_error_check(%args);

sub _click_button_having_specific_text_no_error_check{

    my ( $self, $text ) = @_;

    $Log->enter();

    my @elems=$self->find_elements("//*[\@type='button']");
    foreach (@elems){
        # $_ is a webelement object so cant use private methods
        if(($_->get_text()) =~ /$text/i){
            $_->click();
            last;
        }
    }

    $Log->exit();

    return 1;
} ## end sub _click_button_having_specific_text_no_error_check


###############################################################################
## @name ::_smtt_navigate
##
## @summary
##     This method is used to navigate in the 7MTT tool based on navigation steps
## @synopsis
##     _smtt_navigate(%args);

sub _smtt_navigate{

    my ( $self, $navigation_steps ) = @_;

    $Log->enter();

    while ( $navigation_steps =~ /([\w\s]+):?/g ){
        if ( $1 eq "Edit Configuration" ){
            $self->_click(
                id                      => $EDIT_CONFIGURATION,
                error_checking_needed   => 0,
            ) ;
        }elsif ( $1 eq "Save Project and Continue" ){
            $self->_click(
                id                      => $SAVE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "Next" ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "Skip" ){
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "Storage Systems" ){
            $self->_click(
                id                      => $STORAGE_SYSTEMS,
                error_checking_needed   => 0,
            ) ;
        }elsif ( $1 eq "Get Started" ){
            $self->_click(
                id                      => $GET_STARTED,
                error_checking_needed   => 0,
            ) ;
        }else{
            $self->_click_button_having_specific_text($1);
        }

    }

    $Log->exit();

    return 1;
} ## end sub _smtt_navigate

###############################################################################
## @name ::_close_tab
##
## @summary
##     This method is used to close the unwanted tabs
## @synopsis
##     _close_tab(%args);

sub _close_tab{

    my ( $self, %args ) = @_;

    $Log->enter();

    if (defined $args{session}){

        my $tab=$GENERIC_SESSION_TAB;
        my $close=$GENERIC_SESSION_TAB_CLOSE;
        my $project = $args{session}."_project";
        $tab =~ s/(.*)XXYYZZ(.*)/$1$project$2/;
        $close =~ s/(.*)XXYYZZ(.*)/$1$project$2/;
        if($self->_is_element_present($tab)){
            $self->_click(
                id                      => $close,
                error_checking_needed   => 1,
            ) ;
        }

    }elsif (defined $args{string}){

        my ( $tab, $close );
        if($args{string} =~ /^Dashboard$/){
            $tab=$GENERIC_SESSION_TAB;
            $close=$GENERIC_SESSION_TAB_CLOSE;
        }else{
            $tab=$GENERIC_TAB;
            $close=$GENERIC_TAB_CLOSE;
        }
        my $tab_name = $args{string};
        $tab =~ s/(.*)XXYYZZ(.*)/$1$tab_name$2/;
        $close =~ s/(.*)XXYYZZ(.*)/$1$tab_name$2/;
        if($self->_is_element_present($tab)){
            $self->_click(
                id                      => $close,
                error_checking_needed   => 1,
            ) ;
        }

    }

    $Log->exit();

    return 1;
} ## end sub _close_tab

###############################################################################
## @name ::_throw_exception
##
## @summary
##     This method is used to throw exception after capturing screenshot
## @synopsis
##     _throw_exception(%args);

sub _throw_exception{

    my ( $self, $exception_text ) = @_;

    $Log->enter();

    $self->_take_screenshot();

    $Log->exit();

    NATE::BaseException->throw($exception_text);

} ## end sub _throw_exception

###############################################################################
## @name ::_take_screenshot
##
## @summary
##     This method is used to capture screenshot
## @synopsis
##     _take_screenshot(%args);

sub _take_screenshot{

    my ( $self ) = @_;

    $Log->enter();

    $Log->comment("Taking a base-64 encoded screenshot.. Use freeformatter to decode");
    my $img = $self->screenshot();
    $Log->comment($img);

    $Log->exit();

} ## end sub _take_screenshot

###############################################################################
## @name ::_repair_session
##
## @summary
##     This method is used to repair session object
## @synopsis
##     _repair_session(%args);

sub _repair_session{

    my ( $self ) = @_;

    $Log->enter();

    $self->refresh();
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    $self->_login();

    $Log->exit();

    return 1;
} ## end sub _repair_session

###############################################################################
## @name ::_get_host_from_ip
##
## @summary
##     This method returns hostname from the given IP address
## @synopsis
##     _get_host_from_ip(%args);

sub _get_host_from_ip{

    my ( $self, $ip, $mode ) = @_;

    $Log->enter();

    # Navigating to storage tab
     $self->_click(
        id                      => $STORAGE_SYSTEMS,
        error_checking_needed   => 0,
    ) ;

    my $selector;
    if ($mode =~ /c/){
        $selector=$GENERIC_CMODE_HOSTNAME;
    }elsif ($mode =~ /s/){
        $selector=$GENERIC_SMODE_HOSTNAME;
    }
    
    $selector =~ s/(.*)XXYYZZ(.*)/$1$ip$2/;

    my $host;
    if( $self->_is_element_present($selector) ){
        $host = $self->_get_text($selector);
        $self->_close_tab(
            string => "Storage",
        );
    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Did not get the hostname from the IP in the storage "
            ."systems.. Prabably the credentials are not added .. "
            ."Unable to find the element with ".$selector);
    }

    $Log->exit();

    return $host;
} ## end sub _get_host_from_ip

###############################################################################
## @name ::_session_exist
##
## @summary
##     This method returns true if session exists
## @synopsis
##     _session_exist(%args);

sub _session_exist{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    my %projects_sub_projects_details = $self->_get_all_sessions();

    my ($key, $value );

    while ( ($key, $value) = each %projects_sub_projects_details ) {
        foreach ( @{$value} ) {
            if ($_ eq $session_name){
                $Log->exit();
                return "gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_".$session_name.".$key";
            }
        }
    }

    $Log->exit();

    return 0;
} ## end sub _session_exist

###############################################################################
## @name ::session_exist
##
## @summary
##     This method returns true if session exists
## @synopsis
##     session_exist(%args);

sub session_exist{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_session_exist($session_name);

    $Log->exit();

    return $output;
} ## end sub session_exist

###############################################################################
## @name ::_get_project_id
##
## @summary
##     This method returns the project ID from the session ID
## @synopsis
##     _get_project_id(%args);

sub _get_project_id{

    my ( $self, $session_id ) = @_;

    $Log->enter();

    $session_id =~ s/(gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_)(.*)\.(.*)/$1$3/gi;

    return $session_id;

    $Log->exit();

    return 0;
} ## end sub _get_project_id

###############################################################################
## @name ::_get_all_sessions
##
## @summary
##     This method returns all the session names
## @synopsis
##     _get_all_sessions(%args);

sub _get_all_sessions{

    my ( $self ) = @_;

    $Log->enter();

    my %projects_sub_projects_details;

    my @sess_objs = $self->_find_elements( $PROJECT_SELECTOR ) ;
    foreach (@sess_objs){
        my $id = $_->get_attribute('id');
        if ($id =~ /\./){
            # It's a sub-project
            if ($id =~ /gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_(.*)\.(.*)/){
                push @{$projects_sub_projects_details{$2}},$1;
            }
        }
    }

    $Log->exit();

    return %projects_sub_projects_details;
} ## end sub _get_all_sessions

###############################################################################
## @name ::get_all_sessions
##
## @summary
##     This method returns all the session names
## @synopsis
##     get_all_sessions(%args);

sub get_all_sessions{

    my ( $self ) = @_;

    $Log->enter();

    my %hash = $self->_get_all_sessions();
    my $output->{parsed_output}->{response} = \%hash ;

    $Log->exit();

    return $output;
} ## end sub get_all_sessions

###############################################################################
## @name ::_add_volumepairs_for_session
##
## @summary
##     This method is used to add volumepairs to the session
## @synopsis
##     _add_volumepairs_for_session(%args);

sub _add_volumepairs_for_session{

    my ( $self, $session_name, $skip_navigation ) = @_;

    $Log->enter();

    if( ! $skip_navigation){
        # Navigating to the correct vfiler to add volumes
        my $text;
        my $vfiler = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","vfiler",$session_name
        );
        my $node = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_volumepair_add_args","s_hostname",$session_name
        );
        if ($vfiler eq "vfiler0"){
            $text = $node;
        }else{
            $text = $vfiler.".".$node;
        }
        my $selector=$GENERIC_VFILER_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$text$2/;
    
        if( $self->_is_element_present($selector) ){
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 1,
            ) ;
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Probably the vfiler requested for does not "
                ."exist.. Unable to find the element with ".$selector);
        }
    }

    # Adding volumes now
    my $session_type = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_create_args","session-type",$session_name
    );
    my $vol_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name
    );
    if($session_type eq "standalone"){
        $self->_add_volumes_given_vol_ref_array($vol_ar);
    }elsif ($session_type eq "secondary"){
        $self->_add_volumes_given_vol_ref_array($vol_ar,"staggered");
    }else{
        # Create arguments not present. Adding volumes standalone style
        $self->_add_volumes_given_vol_ref_array($vol_ar);
    }

    $Log->exit();

    return 1;
} ## end sub _add_volumepairs_for_session

###############################################################################
## @name ::_add_vserver
##
## @summary
##     This method is used to add vserver to the session
## @synopsis
##     _add_vserver(%args);

sub _add_vserver{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    my $node = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","c_hostname",$session_name
    );
    my $clus_selector=$GENERIC_CLUSTER_SELECTOR;
    $clus_selector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;

    if( $self->_is_element_present($clus_selector) ){
        $self->_click(
            id                      => $clus_selector,
            error_checking_needed   => 1,
        ) ;
    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Probably the Cluster requested for does not exist"
            .".. Unable to find the element with ".$clus_selector);
    }

    my $vserver = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_create_args","svm-name",$session_name
    );
    my $selector=$GENERIC_VSERVER_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$vserver$2/;

    if( $self->_is_element_present($selector) ) {
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) if(! $self->_is_checked($selector) );
    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Probably the vserver requested for does not exist"
            .".. Unable to find the element with ".$selector);
    }

    $Log->exit();

    return 1;
} ## end sub _add_vserver

###############################################################################
## @name ::_select_aggregate
##
## @summary
##     This method is used for aggregate selection
## @synopsis
##     _select_aggregate(%args);

sub _select_aggregate{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    # Sleeping here for sometime
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    Tharn::snooze($sleep_count);
    my $aggr_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","aggregate-name",$session_name
    );
    my $vol_7_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name
    );

    foreach (0..@{$aggr_ar}-1){

        my $aggregate = $aggr_ar->[$_];
        my $vol_7 = $vol_7_ar->[$_];

        my $aggr_sel = $GENERIC_AGGR;
        $aggr_sel =~ s/(.*)XXYYZZ(.*)/$1$vol_7$2/;
        my $elem = $self->_find_element( $aggr_sel ) ;
        $elem->click();
        # Sleeping to allow drop-down some time to appear
        Tharn::snooze($sleep_count);
        my $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$aggregate$2/;

        if( $self->_is_element_present($selector) ){
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 1,
            ) ;
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Probably the aggregate requested for does "
                ."not exist.. Unable to find the element with ".$selector);
        }

    }

    $Log->exit();

    return 1;
} ## end sub _select_aggregate

###############################################################################
## @name ::_do_volume_mapping
##
## @summary
##     This method is used for volume mapping
## @synopsis
##     _do_volume_mapping(%args);

sub _do_volume_mapping{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    my $vol_7_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name);
    my $vol_c_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-c-mode",$session_name);

    foreach (0..@{$vol_c_ar}-1){

        my $vol_7 = $vol_7_ar->[$_];
        my $vol_c = $vol_c_ar->[$_];

        # Changing C-Mode volumes names to requested values
        my $cmode_vol_sel = $GENERIC_CMODE_VOLUME_SELECTOR;
        $cmode_vol_sel =~ s/(.*)XXYYZZ(.*)/$1$vol_7$2/;
        $self->_type( $cmode_vol_sel ,$vol_c);

    }

    $Log->exit();

    return 1;
} ## end sub _do_volume_mapping

###############################################################################
## @name ::_change_session_name_to_given_value
##
## @summary
##     This method is used for changing session names
## @synopsis
##     _change_session_name_to_given_value(%args);

sub _change_session_name_to_given_value{

    my ( $self, $session_name ) = @_;

    $Log->enter();

    my @edits = $self->_find_elements( $PROJECT_NAME_CHANGE ) ;
    $edits[0]->click();
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    my $project_name = $session_name."_project";
    $self->_type( $PROJECT_NAME_INPUT ,$project_name);
    $edits[1]->click();
    Tharn::snooze($sleep_count);
    $self->_type( $SUBPROJECT_NAME_INPUT ,$session_name);

    my @saves = $self->_find_elements( $PROJECT_NAME_SAVE ) ;
    $saves[0]->click();
    Tharn::snooze($sleep_count);
    $saves[1]->click();
    Tharn::snooze($sleep_count);

    $Log->exit();

    return 1;
} ## end sub _change_session_name_to_given_value

###############################################################################
## @name ::_add_volumes_given_vol_ref_array
##
## @summary
##     This is a helper method for adding volumepairs
## @synopsis
##     _add_volumes_given_vol_ref_array(%args);

sub _add_volumes_given_vol_ref_array{

    my ( $self, $vol_ar, $ses_type ) = @_;

    $Log->enter();

    # Add support for qtree transition also
    foreach (0..@{$vol_ar}-1){
        my $vol = $vol_ar->[$_];
        my $checkbox=$GENERIC_VOLUME_STANDALONE_SELECTOR;
        $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
        if($self->_is_element_enabled($checkbox) ){
            if( $self->_is_checked($checkbox) ){
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Volume $vol is already been added to the session");
            }
            if ($ses_type eq "staggered"){
                $checkbox=$GENERIC_VOLUME_STAGGERED_SELECTOR;
                $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
                
            }
            $self->_click(
                id                      => $checkbox,
                error_checking_needed   => 1,
            ) ;
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Probably the volume requested for does not exist"
                ." or is offline/involved in some other subproject.. "
                ."Unable to find the element with ".$checkbox);
        }
    }

    $Log->exit();

    return 1;
} ## end sub _add_volumes_given_vol_ref_array

###############################################################################
## @name ::_is_button_enabled
##
## @summary
##     This method returns true if button is enabled
## @synopsis
##     _is_button_enabled(%args);

sub _is_button_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $retStatus = 1;

    if (
        $self->_is_element_present(
            "//table[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      ) {
        $retStatus = 0;
    }

    ( $retStatus == 1 ) ? $Log->comment("Button: $debugID is Enabled")
        : $Log->comment("Button: $debugID is Disabled");

    $Log->exit();

    return $retStatus;
} ## end sub _is_button_enabled

###############################################################################
## @name ::is_button_enabled
##
## @summary
##     This method returns true if button is enabled
## @synopsis
##     is_button_enabled(%args);

sub is_button_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter();

    my $output->{parsed_output}->{response} = $self->_is_button_enabled($debugID);

    $Log->exit();

    return $output;
} ## end sub is_button_enabled

###############################################################################
## @name ::_wait_on_attribute
##
## @summary
##     This method is used to wait on a given attribute
## @synopsis
##     _wait_on_attribute(%args);

sub _wait_on_attribute {

    my ( $self , %args ) = @_;

    $Log->enter();

    my $attribute_to_check = $args{attribute_to_check};
    my $method_timeout     = $args{"method-timeout"};

    $self->_click(
        id                      => $AUTO_REFRESH,
        error_checking_needed   => 0,
    ) if( $self->_is_checked($AUTO_REFRESH) ); ;

    my $bool=0;
    foreach (1..$method_timeout) {
        $self->_click(
            id                      => $REFRESH,
            error_checking_needed   => 0,
        ) ;
        if ( $self->_is_element_present( $attribute_to_check ) ) {
            $bool = 1;
            last;
        }
    }

    unless ( $bool ){
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Even after waiting for $method_timeout secs "
            .", still the desired attribute $attribute_to_check did not appear");
    }

    $Log->exit();

    return 1;
} ## end sub wait_on_attribute

###############################################################################
## @name ::_add_schedule_days
##
## @summary
##     This method is used to add days to schedule
## @synopsis
##     _add_schedule_days(%args);

sub _add_schedule_days {

    my ( $self, $days_range ) = @_;

    if ($days_range ne "0-6" ){

        # Adding days-range
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DAYS_SELECT_RADIO,
            error_checking_needed   => 0,
        ) ;

        my $days = {
            0 => "sunday",
            1 => "monday",
            2 => "tuesday",
            3 => "wednesday",
            4 => "thursday",
            5 => "friday",
            6 => "saturday",
        };
        my @arr;
        foreach (split(",",$days_range)){
            if($_ =~ /-/){
                my @temp = split("-",$_);
                if( $temp[0] > $temp[1] ){
                    push @arr,$temp[0];
                    while( $temp[0] ne $temp[1] ){
                        if (++$temp[0] eq 7){
                            $temp[0]=0;
                            push @arr,$temp[0];
                        }else{
                            push @arr,$temp[0];
                        }
                    }
                }else{
                    my $i=0;
                    push @arr,$temp[0];
                    while( $temp[0] ne $temp[1] ){
                        push @arr,++$temp[0];
                        $i++;
                    }
                }
            }else{
                push @arr,$_;
            }
        }

        my %unique = ();
        foreach my $item (@arr) {
            $unique{$item} ++ ;
        }
        my @myuniquearray = keys %unique;
        foreach (@myuniquearray){
            my $day = $days->{$_};
            my $selector =  $GENERIC_SCHEDULE_DAYS_SELECT;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$day$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

    }

}

###############################################################################
## @name ::_add_credentials
##
## @summary
##     This method is used to add credentials
## @synopsis
##     _add_credentials(%args);

sub _add_credentials {

    my ( $self , %args ) = @_;

    $Log->enter();

    $Log->comment("Adding credentials");
    $self->_type( $HOSTNAME ,$args{"host-name"});
    $self->_type( $HOST_USERNAME ,$args{"username"});
    $self->_type( $HOST_PASSWORD ,$args{"password"});
    $self->_click(
        id                      => $HOST_ADD,
        error_checking_needed   => 1,
    ) ;

    $Log->exit();

    return 1;
} ## end sub _add_credentials

###############################################################################
## @name ::_is_key_x_present_for_session_y
##
## @summary
##     This method is internal helper method
## @synopsis
##     _is_key_x_present_for_session_y(%args);

sub _is_key_x_present_for_session_y{

    my ( $self , $x , $y ) = @_;

    $Log->enter();

    foreach (@{$self->{$y}}){
        if ( defined $_->{$x}){
            $Log->exit();
            return 1;
        }
    }

    $Log->exit();

    return 0;
} ## end sub _is_key_x_present_for_session_y

###############################################################################
## @name ::_get_value_for_key_x_key_y_in_session_z
##
## @summary
##     This method is internal helper method
## @synopsis
##     _get_value_for_key_x_key_y_in_session_z(%args);

sub _get_value_for_key_x_key_y_in_session_z{

    my ( $self, $x , $y, $z) = @_;

    $Log->enter();

    foreach (@{$self->{$z}}){
        if(defined $_->{$x} ){
            $Log->exit();
            return $_->{$x}->{$y};
        }
    }

    $Log->exit();

    return undef;

} ## end sub _get_value_for_key_x_key_y_in_session_z

###############################################################################
## @name ::_put_value_for_key_x_key_y_in_session_z
##
## @summary
##     This method is internal helper method
## @synopsis
##     _put_value_for_key_x_key_y_in_session_z(%args);

sub _put_value_for_key_x_key_y_in_session_z{

    my ( $self, $x , $y, $z, $val) = @_;

    $Log->enter();

    foreach (@{$self->{$z}}){
        if(defined $_->{$x} ){
            $_->{$x}->{$y}=$val;
            last;
        }
    }

    $Log->exit();

    return 1;

} ## end sub _put_value_for_key_x_key_y_in_session_z

###############################################################################
## @name ::_get_volume_sm_details
##
## @summary
##     This method returns volume snapmirror status for a given session
## @synopsis
##     _get_volume_sm_details(%args);

sub _get_volume_sm_details{

    my ( $self, %args ) = @_;

    $Log->enter();

    my $id = undef ;
    $self->_repair_session();
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Click on volume tab
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 1,
        ) ;

        # Populate this array if you want to fetch any new entries
        my @keys = ("Origin_Volume","Target_Volume","Current_Operation","Last_Operation",
            "Mirror_State","SM_Status","Data_Pending"
        );

        my $row = 1;
        while (1){
            eval {
                my $href = undef;
                foreach my $col (1..@keys){
                    my $volume_table_selector = $VOLUME_TABLE_SELECTOR;
                    $volume_table_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                    $href->{$keys[$col-1]} = $self->_get_text($volume_table_selector);
                }
                push (@$aref,$href);
            };
            if ($@) {
                $self->_close_tab(
                    string => "Dashboardtab",
                );
                $output->{parsed_output}->{response} = $aref ;
                $Log->exit();
                return $output;
            }
            $row++;
        }

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit();

    return $output;

} ## end sub _get_volume_sm_details

###############################################################################
## @name ::_get_data_copy_schedule_details
##
## @summary
##     This method returns data copy schedule_details for a given session
## @synopsis
##     _get_data_copy_schedule_details(%args);

sub _get_data_copy_schedule_details{

    my ( $self, %args ) = @_;

    $Log->enter();

    my $id = undef ;
    $self->_repair_session();
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Click on schedule tab
        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 1,
        ) ;

        # Populate this array if you want to fetch any new entries
        my @keys = ("Status","Name","Recurring_Days","Start_Time",
            "Duration","End_Time","Update_Frequency","Available_Transfers_Percentage",
            "Currently_Used","Throttle"
        );

        my $row = 1;
        while (1){
            eval {
                my $href = undef;
                foreach my $col (1..@keys){
                    my $schedule_selector = $SCHEDULE_SELECTOR;
                    $schedule_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                    $href->{$keys[$col-1]} = $self->_get_text($schedule_selector);
                }
                push (@$aref,$href);
            };
            if ($@) {
                $self->_close_tab(
                    string => "Dashboardtab",
                );
                $output->{parsed_output}->{response} = $aref ;
                $Log->exit();
                return $output;
            }
            $row++;
        }

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit();

    return $output;

} ## end sub _get_data_copy_schedule_details

###############################################################################
## @name ::_create_helper
##
## @summary
##     This method is a helper method for create subroutine
## @synopsis
##     _create_helper(%args);

sub _create_helper {

    my ($self, $args) = @_;
    my %args=%{$args};

    $Log->enter();

    my $S_mode_credentials      = delete $args{S_mode_credentials};
    my $C_mode_credentials      = delete $args{C_mode_credentials};
    my $volumepair_add_args     = delete $args{volumepair_add_args};
    my $schedule_add_args       = delete $args{schedule_add_args};

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if( $id ){
        # Session exists .. Raise an exception.. Session repair not required
        $self->_close_tab(
            string => "Dashboardtab",
        );
        $self->_throw_exception("session with name ".$args{"session-name"}." already exists");
    }

    # Storing create arguments in the object and returning back as we can not
    # apply all the given attributes unless transition_volumepair_add is called for
    push @{$self->{$args{"session-name"}}} , { transition_create_args => \%args} ;

    $self->_close_tab(
        string => "Dashboardtab",
    ) if( $dashboard_enabled ) ;

    if ( $S_mode_credentials || $C_mode_credentials || $volumepair_add_args ) {

        # Lets Get Started
        if ( $S_mode_credentials || $C_mode_credentials ) {
            $self->_click(
                id                      => $GET_STARTED,
                error_checking_needed   => 0,
            ) ;
        }

        # Adding credentials
        $self->_add_credentials( %{$S_mode_credentials} ) if ( $S_mode_credentials ) ;
        $self->_add_credentials( %{$C_mode_credentials} ) if ( $C_mode_credentials ) ;

        if ( ($S_mode_credentials || $C_mode_credentials) &&  !($volumepair_add_args) ) {

            # Closing the Untitled project
            my $selector=$GENERIC_TAB_CLOSE;
            $selector =~ s/(.*)XXYYZZ(.*)/$1Untitled$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

        }

        if ( $volumepair_add_args ) {

            # Storing volume pair add arguments in the object
            foreach (@{$self->{$args{"session-name"}}}){
                undef $_ if (defined $_->{transition_volumepair_add_args});
            }
            @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
            push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => $volumepair_add_args} ;

            if ( $S_mode_credentials || $C_mode_credentials ) {

                # Lets go to Home Tab -> Storage systems and fetch the hostnames
                my $selector=$GENERIC_TAB;
                $selector =~ s/(.*)XXYYZZ(.*)/$1Hometab$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                # Closing the Untitled project
                my $selector=$GENERIC_TAB_CLOSE;
                $selector =~ s/(.*)XXYYZZ(.*)/$1Untitled$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

            }

            # Get hostname from ip and store it in APISet object in session hash
            my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","7-mode-host-name",$args{"session-name"});
            my $host = $self->_get_host_from_ip($ip,"s");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","s_hostname",$args{"session-name"},$host);

            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","c-mode-host-name",$args{"session-name"});
            $host = $self->_get_host_from_ip($ip,"c");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","c_hostname",$args{"session-name"},$host);

            # Lets Get Started
            $self->_click(
                id                      => $GET_STARTED,
                error_checking_needed   => 0,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Create new project here
            $Log->debug("Creating new project");
            $self->_type( $PROJECT_NAME_INPUT ,$args{"session-name"} );
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;

            # Wait till volume pairs are loaded
            $self->_check_for_error();

            # Select volumepairs
            $self->_add_volumepairs_for_session($args{"session-name"});

            # Continuing with the remaining part of session creation
            $self->_click(
                id                      => $CREATE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;

            # Changing session name to desired value
            $self->_change_session_name_to_given_value($args{"session-name"});

            my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","data-copy-ipaddress",$args{"session-name"});
            $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

            my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","multipathing-ipaddress",$args{"session-name"});
            $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            if ( $args{"seven-mode-only"} ) {
                $self->_click(
                    id                      => $SMODE_PRECHECK,
                    error_checking_needed   => 1,
                ) ;
                $self->_click(
                    id                      => $SMODE_PRECHECK_NEXT_BUTTON,
                    error_checking_needed   => 0,
                ) ;
            }else {
                # Skipping prechecks
                $self->_click(
                    id                      => $SKIP,
                    error_checking_needed   => 0,
                ) ;
            }

            # Vserver selection page
            $self->_add_vserver($args{"session-name"});
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($volumepair_add_args->{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($volumepair_add_args->{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

        }

        if ( $volumepair_add_args && $schedule_add_args ) {

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            foreach ( @{$schedule_add_args} ) {

                $self->_click(
                    id                      => $CREATE_SCHEDULE,
                    error_checking_needed   => 0,
                ) ;

                $self->_type( $GENERIC_SCHEDULE_NAME ,$_->{"schedule-name"});

                $self->_add_schedule_days($_->{"days-range"});

                # Adding start time
                my @s_t = split(":",$_->{"start-time"});
                my $selector=$GENERIC_LISTITEM_SELECTOR;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_LISTITEM_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
        
                # Adding duration
                my @duration = split(":",$_->{"duration"});
                $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_LISTITEM_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                if ( defined $_->{"available-transfers-percentage"} ) {
                    # Adding available-transfers-percentage
                    $self->_click(
                        id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
                    $self->_type(
                        $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
                }

                if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                    # Adding stream count
                    $self->_click(
                        id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
                    $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
                }
        
                # Adding throttle
                if (defined $_->{"session-snapmirror-throttle"}){
                    $self->_click(
                        id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                        error_checking_needed   => 1,
                    ) ;
                    $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
                }

                # Adding update frequency
                if (defined $_->{"update-frequency"}){
                    my @u_f = split(":",$_->{"update-frequency"});
                    $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                    $self->_click(
                        id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                        error_checking_needed   => 0,
                    ) ;
                    $selector=$GENERIC_AGGR_SELECTOR;
                    $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/; 
                    $self->_click(
                        id                      => $selector,
                        error_checking_needed   => 0,
                    ) ;
                }

                $self->_click(
                    id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                    error_checking_needed   => 1,
                ) ;
    
                if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                    my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Error while adding schedule : ".$text);
                }

            }

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $GO_TO_DASHBOARD,
                error_checking_needed   => 1,
            ) ;
    
            $self->_close_tab(
                string => "Dashboardtab",
            );

        } else {

            # Since schedule add args not present , lets close the tab
            $self->_close_tab(
                session => $args{"session-name"},
            );

        }
    }

    $Log->exit();
} ## end sub _create_helper

###############################################################################
## @name ::_create_primary
##
## @summary
##     This method is a helper method for create primary subroutine
## @synopsis
##     _create_helper(%args);

sub _create_primary {

    my ( $self, $args, $prim_sess ) = @_;
    my %args=%{$args};

    $Log->enter();

    my $volumepair_add_args     = delete $args{volumepair_add_args};
    my $schedule_add_args       = delete $args{schedule_add_args};

    my $id = undef ;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
        $args{"session-name"} = $prim_sess;
        push @{$self->{$args{"session-name"}}} , { transition_create_args => \%args} ;
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 1,
        ) ;

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        $self->_click(
            id                      => $SAVE_PROJECT_AND_CONTINUE,
            error_checking_needed   => 1,
        ) ;

        # Changing session name to desired value
        $self->_change_session_name_to_given_value($args{"session-name"});

        my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","data-copy-ipaddress",$args{"session-name"});
        $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

        my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","multipathing-ipaddress",$args{"session-name"});
        $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;

        if ( $args{"seven-mode-only"} ) {
            $self->_click(
                id                      => $SMODE_PRECHECK,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $SMODE_PRECHECK_NEXT_BUTTON,
                error_checking_needed   => 0,
            ) ;
        }else {
            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;
        }

        # Vserver selection page
        $self->_add_vserver($args{"session-name"});
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        # Storing volume pair add arguments in the object
        foreach (@{$self->{$args{"session-name"}}}){
            undef $_ if (defined $_->{transition_volumepair_add_args});
        }
        @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
        push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => $volumepair_add_args} ;

        # Aggr selection page
        $self->_select_aggregate($args{"session-name"})
            if defined ($volumepair_add_args->{"aggregate-name"});

        # Volume mapping
        $self->_do_volume_mapping($args{"session-name"})
            if defined ($volumepair_add_args->{"volume-name-c-mode"});

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        if ( $volumepair_add_args && $schedule_add_args ) {

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            foreach ( @{$schedule_add_args} ) {

                $self->_click(
                    id                      => $CREATE_SCHEDULE,
                    error_checking_needed   => 0,
                ) ;

                $self->_type( $GENERIC_SCHEDULE_NAME ,$_->{"schedule-name"});

                $self->_add_schedule_days($_->{"days-range"});

                # Adding start time
                my @s_t = split(":",$_->{"start-time"});
                my $selector=$GENERIC_AGGR_SELECTOR;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
        
                # Adding duration
                my @duration = split(":",$_->{"duration"});
                $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/; 
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                if ( defined $_->{"available-transfers-percentage"} ) {
                    # Adding available-transfers-percentage
                    $self->_click(
                        id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
                    $self->_type(
                        $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
                }

                if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                    # Adding stream count
                    $self->_click(
                        id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
                    $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
                }
        
                # Adding throttle
                if (defined $_->{"session-snapmirror-throttle"}){
                    $self->_click(
                        id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                        error_checking_needed   => 1,
                    ) ;
                    $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
                }

                # Adding update frequency
                if (defined $_->{"update-frequency"}){
                    my @u_f = split(":",$_->{"update-frequency"});
                    $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                    $self->_click(
                        id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                        error_checking_needed   => 0,
                    ) ;
                    $selector=$GENERIC_AGGR_SELECTOR;
                    $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/; 
                    $self->_click(
                        id                      => $selector,
                        error_checking_needed   => 0,
                    ) ;
                }

                $self->_click(
                    id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                    error_checking_needed   => 1,
                ) ;
    
                if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                    my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Error while adding schedule : ".$text);
                }

            }

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $GO_TO_DASHBOARD,
                error_checking_needed   => 1,
            ) ;
    
            $self->_close_tab(
                string => "Dashboard",
            );
        } else {

            # Since schedule add args not present , lets close the tab
            $self->_close_tab(
                session => $args{"session-name"},
            );

        }

    }else{
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist ");
    }

    $Log->exit();
} ## end sub _create_primary

###############################################################################
## @name ::_get_volume_selection_details
##
## @summary
##     This method returns volume selection details in an array ref from Volume selection page
## @synopsis
##     _get_volume_selection_details(%args);

sub _get_volume_selection_details{

    my ( $self, %args ) = @_;

    $Log->enter();

    my $sess_name = $args{"session-name"};
    my $ip = $args{"7-mode-host-name"};

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if ( $sess_name ) {

        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        my $id = $self->_session_exist($sess_name);
        if ($id){
            $self->_click(
                id                      => $id,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $EDIT_CONFIGURATION,
                error_checking_needed   => 1,
            ) ;
            $aref = $self->_get_volume_selection_details_helper($sess_name);

        }else {
            $self->_close_tab(
                string => "Dashboardtab",
            ) ;
            $self->_throw_exception("Session $sess_name does not exist ");
        }

    } else {
        my $host = $self->_get_host_from_ip($ip,"s");
        # Lets Get Started
        $self->_click(
            id                      => $GET_STARTED,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        # Create new project here
        $Log->debug("Creating new project");
        $self->_type( $PROJECT_NAME_INPUT ,$args{temp_session_name} );
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;

        # Wait till volume pairs are loaded
        $self->_check_for_error();
        $aref = $self->_get_volume_selection_details_helper();
    }

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit();

    return $output;

} ## end sub _get_volume_selection_details

###############################################################################
## @name ::_get_volume_selection_details_helper
##
## @summary
##     Helper method for _get_volume_selection_details
## @synopsis
##     _get_volume_selection_details_helper(%args);

sub _get_volume_selection_details_helper{

    my ( $self, $sess_name ) = @_;

    my $aref=[];

    # Populate this array if you want to fetch any new entries
    my @keys = ("Name","Used_Size(Gb)","Total_Size(Gb)","Snapmirror_Relationship",
        "Qtree_Snapmirror_Relationship","Snapmirror_Source_Or_Destination",
        "Transition_As_Standalone","Transition_With_Snapmirror_Relationship",
    );

    my $row = 1;
    while (1){
        eval {
            my $href = undef;
            foreach my $col (1..@keys){

                my $volume_info_table_selector = $VOLUME_INFO_TABLE_SELECTOR;
                $volume_info_table_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;

                my ( $stand, $stag, $checkbox, $text );
                if ( $keys[$col-1] eq "Transition_As_Standalone" ) {
                    $stand=1;
                    $href->{Transition_As_Standalone}={};
                    $checkbox=$GENERIC_VOLUME_STANDALONE_SELECTOR;
                    $text="Transition_As_Standalone";
                }
                if ( $keys[$col-1] eq "Transition_With_Snapmirror_Relationship" ) {
                    $stag=1;
                    $href->{Transition_With_Snapmirror_Relationship}={};
                    $checkbox=$GENERIC_VOLUME_STAGGERED_SELECTOR;
                    $text="Transition_With_Snapmirror_Relationship";
                }
                if ( $stand || $stag ) {

                    my $vol=$href->{Name};
                    $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
                    if($self->_is_element_enabled($checkbox) ){
                        $href->{$text}->{is_enabled} = "true";
                        if($self->_is_checked($checkbox) ){
                            $href->{$text}->{is_checked} = "true";
                        }else {
                            $href->{$text}->{is_checked} = "false";
                        }
                    }else {
                        $href->{$text}->{is_enabled} = "false";
                        $href->{$text}->{is_checked} = "false";
                    }
                    my $elem = $self->_find_element($volume_info_table_selector."/div");
                    my $id = $elem->get_attribute('id');
                    $href->{$text}->{tool_tip} = $id;

                }else {
                    $href->{$keys[$col-1]} = $self->_get_text($volume_info_table_selector);
                }
            }
            push (@$aref,$href);
        };
        if ($@) {
            if ($sess_name){
                $self->_close_tab(
                    string => $sess_name."tab",
                );
                $self->_close_tab(
                    string => "Dashboardtab",
                ) ;
            } else {
                # Closing the Untitled project
                my $selector=$GENERIC_TAB_CLOSE;
                $selector =~ s/(.*)XXYYZZ(.*)/$1Untitled$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
            }
            return $aref;
        }
        $row++;
    }

} ## end sub _get_volume_selection_details_helper

###############################################################################
## @name ::set_page
##
## @summary
##     Takes the user to the page requested for
## @synopsis
##     set_page(%args);

sub set_page{

    my ( $self, %args ) = @_;

    my $sess_name = $args{"session-name"};
    my $navigate_to = $args{"navigate_to"};

    $Log->enter();

    my $id = undef;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($sess_name);
    }

    if( $id ){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Any new navigation steps should be added to this map
        my $map = {
            VOLUME_SELECTION_PAGE       => $NAVIGATE_VOLUME_SELECTION_PAGE,
            IP_CONFIGURATION            => $NAVIGATE_IP_CONFIGURATION,
            PRECHECK                    => $NAVIGATE_PRECHECK,
            SVM_MAPPING                 => $NAVIGATE_SVM_MAPPING,
            VOLUME_MAPPING              => $NAVIGATE_VOLUME_MAPPING,
            INTERFACE_MAPPING           => $NAVIGATE_INTERFACE_MAPPING,
            DATA_COPY_SCHEDULE          => $NAVIGATE_DATA_COPY_SCHEDULE,
            FINAL_PRECHECK              => $NAVIGATE_FINAL_PRECHECK,
            SUBPROJECT_SUMMARY          => $NAVIGATE_SUBPROJECT_SUMMARY,
        };

        my $navigation_steps = $map->{$navigate_to};

        while ( $navigation_steps =~ /([\w\s-]+):?/g ){
            $self->_click(
                id                      => $1,
                error_checking_needed   => 1,
            ) ;
        }
    } elsif ( $navigate_to eq "VOLUME_SELECTION_PAGE" ){
        $self->_close_tab(
            string => "Dashboardtab",
        );
        $self->_smtt_navigate($NAVIGATE_TRANSITION_CREATE);
        # Create new project here
        my $ses_name="t".int (rand(time));
        $self->_type( $PROJECT_NAME_INPUT ,$ses_name );
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;
    } else {
        $self->_close_tab(
            string => "Dashboardtab",
        );
    }

    $Log->exit();

    return 1;

} ## end sub set_page

###############################################################################
## @name ::get_session_details
##
## @summary
##     Returns the session details to the user
## @synopsis
##     get_session_details(%args);

sub get_session_details{

    my ( $self, %args ) = @_;

    my $sess_name = $args{"session-name"};
    my $output ;

    $Log->enter();

    my $id = undef;
    my $dashboard_enabled = undef;
    my $hash_ref={};

    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($sess_name);
    }

    if( $id ){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        my $arr_ref = [
            "precheck-status:prechecks not run yet",
            "precheck-status:prechecks completed",
            "baseline-status:baseline data copy not started",
            "baseline-status:baseline data copy is scheduled",
            "baseline-status:baseline data copy paused",
            "baseline-status:baseline data copy completed",
            "incremental-status:incremental data copy is scheduled",
            "incremental-status:incremental data copy paused",
            "config-status:apply configuration not started",
            "config-status:apply configuration in progress",
            "config-status:copy resync in progress",
            "config-status:apply configuration completed",
            "cutover-status:cutover not started",
            "cutover-status:cutover in progress",
            "cutover-status:transition completed",
            "cutover-status:transition aborted",
        ];

        foreach my $id ( @$arr_ref ) {
            if ( $self->_is_element_present($id) ){
                my $elem  = $self->_find_element( "//div[\@id='".$id."']/div/div") ;
                my $style = $elem->get_attribute('style');
                $id =~ /([\w\s-]+):(.*)/g;
                $hash_ref->{$1}="$2:$style";
            }
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );
    } else {
        $self->_close_tab(
            string => "Dashboardtab",
        );
    }

    $output->{parsed_output} = $hash_ref ;

    $Log->exit();

    return $output;

} ## end sub get_session_details

1;
