#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @name ::Transition
##
## @summary Contains methods to execute transition commands
##
## @description
##    Using this library, user can perform transition related task.
##    Methods in this library has a one to one mapping to the CLI parsers
##
## @synopsis
##    use NACL::APISet::Host::GUI::SMTT::V2_3::Transition;
##
##    transition_credentials_add(%args);
##    transition_create(%args);
##    transition_volumepair_add(%args);
##    transition_volumepair_remove(%args);
##    transition_lif_add(%args);
##    transition_lif_remove(%args);
##    transition_precheck(%args);
##    transition_start(%args);
##    transition_schedule_add(%args);
##    transition_pause(%args);
##    transition_resume(%args);
##    transition_precutover(%args);
##    transition_resync(%args);
##    transition_update(%args);
##    transition_cutover(%args);
##    transition_abort(%args);
##    transition_schedule_modify(%args);
##    transition_schedule_remove(%args);
##    transition_delete(%args);
##    transition_cft_create(%args);
##    transition_cft_clusterinfo_add(%args);
##    transition_cft_nodemapping_set(%args);
##    transition_cft_svmmapping_add(%args);
##    transition_cft_precheck(%args);
##    transition_cft_apply_svm_config(%args);
##    transition_cft_start(%args);
##    transition_cft_verify_cabling(%args);
##    transition_cft_import(%args);
##    transition_cft_commit(%args);
##    transition_cft_revert_precheck(%args);
##    transition_cft_revert_generate_manual_steps(%args);
##    transition_cft_delete(%args);
##    transition_cft_check_readiness(%args);
##
## @author santosh6@netapp.com
##
## @change     2015-07-13 santosh6 : Added cft delete command for GUI
##
## @status Review
#

package NACL::APISet::Host::GUI::SMTT::V2_3::Transition;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SMTT::V2_3::GlobalConst;
use NATE::BaseException qw(:try);

use Selenium::Remote::WDKeys;

use Exporter 'import';

our @EXPORT = qw (
    transition_credentials_add
    transition_create
    transition_volumepair_add
    transition_volumepair_remove
    transition_lif_add
    transition_lif_remove
    transition_schedule_add
    transition_schedule_modify
    transition_schedule_remove
    transition_precheck
    transition_start
    transition_precutover
    transition_resync
    transition_cutover
    transition_update
    transition_pause
    transition_resume
    transition_abort
    transition_delete
    transition_cft_create
    transition_cft_clusterinfo_add
    transition_cft_nodemapping_set
    transition_cft_svmmapping_add
    transition_cft_precheck
    transition_cft_apply_svm_config
    transition_cft_start
    transition_cft_verify_cabling
    transition_cft_import
    transition_cft_commit
    transition_cft_revert_precheck
    transition_cft_revert_generate_manual_steps
    transition_cft_revert_verify7_mode
    transition_cft_delete
    transition_view_results
    transition_cft_confirm_execution_revert_manual_steps
    transition_apt_add_systems
    transition_apt_generate_report
    transition_cft_revert_abort
    transition_cft_check_readiness
);

## @name ::transition_credentials_add
##
## @summary Executes "transition credentials add" command
##
## @description
##    This method is used to execute transition credentials add command
## @synopsis
##      transition_credentials_add(%args);
## INPUTS TO METHOD CALL 'transition_credentials_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'username' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##      $Api_Set_Obj2->transition_credentials_add(
##            "host-name" => "10.227.202.109",
##            "username"  => "root",
##            "password"  => "netapp1!",
##      );
##
##&nbsp;Command = transition credentials add -h 10.227.202.110 -u root
##&nbsp;

sub transition_credentials_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "host-name"  => { type => SCALAR },
            "username"   => { type => SCALAR },
            "password"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Credentials add page");
    $self->_smtt_navigate($NAVIGATE_TRANSITION_STORAGE);

    $self->_add_credentials(%args);

    $self->_close_tab(
        string => "Storage",
    );
    $Log->comment("Credentials added successfully");
    $Log->exit();

    return 1;
} ## end sub transition_credentials_add

## @name ::transition_create
##
## @summary Executes "transition_create" command
##
## @description
##    This method is used to execute transition create command
## @synopsis
##      transition_create(%args);
## INPUTS TO METHOD CALL 'transition_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'session-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'data-copy-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multipathing-ipaddress' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'svm-rootvol-aggr' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'S_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'C_mode_credentials' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'volumepair_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'schedule_add_args' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'seven-mode-only' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##       my %S_mode_credentials=(
##            "host-name"                        => "10.227.202.109",
##            username                           => "root",
##            password                           => "netapp1!"
##       );
##       my %C_mode_credentials=(
##            "host-name"                        => "10.227.65.55",
##            username                           => "admin",
##            password                           => "netapp1!"
##       );
##       my %volume=(
##            "volume-name-7-mode"               => [qw(r1 r2)],
##            "volume-name-c-mode"               => [qw(r1234 r2234)],
##            "aggregate-name"                   => [qw(aggr_trans aggr_trans)],
##       );
##
##       my %schedule1=(
##            "schedule-name"                    => "new1",
##            "days-range"                       => "1-4",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##       );
##       my %schedule2=(
##            "schedule-name"                    => "new2",
##           "days-range"                        => "0,5,6",
##            "start-time"                       => "12:00",
##            "duration"                         => "20:30",
##            "available-transfers-percentage"   => "4",
##            "session-snapmirror-throttle"      => "230",
##            "update-frequency"                 => "1:30",
##        );
##        my @schedule=(\%schedule1,\%schedule2);
##        $Api_Set_Obj2->transition_create(
##             "session-name"              => "session2",
##             "session-type"              => "standalone",
##             "7-mode-host-name"          => "10.227.202.109",
##             "data-copy-ipaddress"       => "10.227.202.109",
##             "vfiler"                    => "vfiler0",
##             "multipathing-ipaddress"    => "10.227.202.110",
##             "c-mode-host-name"          => "10.227.65.55",
##             "svm-name"                  => "r1",
##             "vserver-rootvol-aggr"      => { type => SCALAR },
##             S_mode_credentials          => \%S_mode_credentials,
##             C_mode_credentials          => \%C_mode_credentials,
##             volumepair_add_args         => \%volume,
##             schedule_add_args           => \@schedule,
##             "seven-mode-only"           => 1,
##        );
##&nbsp;transition create -s S21 -t standalone -n 10.227.202.110 -c 10.227.202.110 -f vfiler0 -h 10.226.99.61 -v vs_sec -d daily :
##&nbsp;

sub transition_create {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR , optional => 1},
            "session-type"              => { type => SCALAR , optional => 1},
            "7-mode-host-name"          => { type => SCALAR , optional => 1},
            "data-copy-ipaddress"       => { type => SCALAR , optional => 1},
            "vfiler"                    => { type => SCALAR , optional => 1},
            "multipathing-ipaddress"    => { type => SCALAR , optional => 1},
            "c-mode-host-name"          => { type => SCALAR , optional => 1},
            "svm-name"                  => { type => SCALAR , optional => 1},

            "S_mode_credentials"        => { optional => 1  },
            "C_mode_credentials"        => { optional => 1  },
            "volumepair_add_args"       => { optional => 1  },
            "schedule_add_args"         => { type => ARRAYREF, optional => 1},

            "seven-mode-only"           => { type => SCALAR , optional => 1},

            "primary_info"              => { optional => 1 },
            "secondary_info"            => { optional => 1 },

        },
        allow_extra => 1
    );

    $Log->enter();

    $self->send_keys_to_active_element(KEYS->{'control'}, KEYS->{'subtract'}, KEYS->{'subtract'});
    $self->send_keys_to_active_element(KEYS->{'control'}, KEYS->{'subtract'}, KEYS->{'subtract'});
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $self->_click(
	id                      =>  $MIGRATE_BUTTON,
	error_checking_needed   =>  0,
    );

    if (defined $args{primary_info} && defined $args{secondary_info} ) {

        my $primary_info        = delete $args{primary_info};
        my $secondary_info      = delete $args{secondary_info};

        my $S_mode_credentials_P = delete $primary_info->{S_mode_credentials};
        my $C_mode_credentials_P = delete $primary_info->{C_mode_credentials};

        $self->_click(
            id                      => $CBT_STORAGE_SYSTEMS,
            error_checking_needed   => 0,
        ) ;
        $self->_add_credentials( %{$S_mode_credentials_P} );
        $self->_add_credentials( %{$C_mode_credentials_P} );
        $self->_close_tab(
            string => "Storage",
        );

        $secondary_info->{volumepair_add_args}->{"volume-name-7-mode"}
            = $primary_info->{volumepair_add_args}->{"volume-name-7-mode"};
        $secondary_info->{"7-mode-host-name"} = $primary_info->{"7-mode-host-name"};

	my $host = $self->_cbt_get_host_from_ip($primary_info->{"c-mode-host-name"},"c");
        push @{$self->{$primary_info->{"session-name"}}} , { transition_volumepair_add_args =>$primary_info->{"volumepair_add_args"}} ;
	$self->_put_value_for_key_x_key_y_in_session_z(
	    "transition_volumepair_add_args","c_hostname",$primary_info->{"session-name"},$host);

	$self->_create_helper($secondary_info);
        my $prim_sess = $primary_info->{"session-name"};
        $primary_info->{"session-name"} = $secondary_info->{"session-name"}."_primary";
        $self->_create_primary($primary_info,$prim_sess);
    } else {
        $self->_create_helper(\%args);
    }

    $Log->exit();

    return 1;
} ## end sub transition_create

## @name ::transition_volumepair_add
##
## @summary Executes "transition_volumepair_add" command
##
## @description
##    This method is used to execute transition volumepair add command
## @synopsis
##      transition_volumepair_add(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'volume-name-c-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_add(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1)],
##            "volume-name-c-mode"        => [qw(r1_1)],
##            "aggregate-name"            => [qw(aggr_trans)],
##         );
##
##&nbsp;transition volumepair add -s S1 -v vol1 -c vol_compo1 -g aggr_trans :
##&nbsp;

sub transition_volumepair_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
            "volume-name-c-mode"        => { type => ARRAYREF , optional => 1},
            "aggregate-name"            => { type => ARRAYREF , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Storing volume pair add arguments in the object
    foreach (@{$self->{$args{"session-name"}}}){
        undef $_ if (defined $_->{transition_volumepair_add_args});
    }
    @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
    push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => \%args} ;

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $EDIT_CONFIGURATION,
            error_checking_needed   => 1,
        ) ;

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        $self->_add_volumepairs_for_session($args{"session-name"},1);

        $self->_click(
            id                      => $SAVE_PROJECT_AND_CONTINUE,
            error_checking_needed   => 1,
        ) ;

        if ( (defined $args{"aggregate-name"}) || (defined $args{"volume-name-c-mode"}) ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }

        $self->_close_tab(
            session => $args{"session-name"},
        );

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        if($self->_is_key_x_present_for_session_y("transition_create_args",$args{"session-name"})){

            $Log->comment("Creating session and adding volumepairs");

            # Session does not exist
            $self->_close_tab(
                string => "Dashboardtab",
            ) if( $dashboard_enabled ) ;

            # Get hostname from ip and store it in APISet object in session hash
            my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","7-mode-host-name",$args{"session-name"});
            my $host = $self->_get_host_from_ip($ip,"s");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","s_hostname",$args{"session-name"},$host);

            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","c-mode-host-name",$args{"session-name"});
            $host = $self->_get_host_from_ip($ip,"c");
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","c_hostname",$args{"session-name"},$host);

            # Project details
            $self->_smtt_navigate($NAVIGATE_TRANSITION_CREATE);

            # Create new project here
            $Log->debug("Creating new project");
            $self->_type( $PROJECT_NAME_INPUT ,$args{"session-name"} );
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;

            # Wait till volume pairs are loaded
            $self->_check_for_error();

            # Select volumepairs
            $self->_add_volumepairs_for_session($args{"session-name"});

            # Continuing with the remaining part of session creation
            $self->_click(
                id                      => $CREATE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;

            # Changing session name to desired value
            $self->_change_session_name_to_given_value($args{"session-name"});

            my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","data-copy-ipaddress",$args{"session-name"});
            $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

            my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","multipathing-ipaddress",$args{"session-name"});
            $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            # Skipping prechecks
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 0,
            ) ;

            # Vserver selection page
            $self->_add_vserver($args{"session-name"});
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            # Aggr selection page
            $self->_select_aggregate($args{"session-name"})
                if defined ($args{"aggregate-name"});

            # Volume mapping
            $self->_do_volume_mapping($args{"session-name"})
                if defined ($args{"volume-name-c-mode"});

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_close_tab(
                session => $args{"session-name"},
            );

            $Log->comment("Session created and volumepairs added successfully");

        }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist "."
                and create arguments not present in the APISet object.\n ".
                "Make sure that you invoke transition_volumepair_add() on the same APISet".
                " object \n on which you invoked transition_create() or session should exist"
            );
        }
    }

    $Log->exit();

    return 1;
} ## end sub transition_volumepair_add

## @name :: transition_volumepair_remove
##
## @summary Executes "transition_volumepair_remove" command
##
## @description
##  This method is used to execute transition volume remove from session
## @synopsis
##  transition_volumepair_remove(%args);
## INPUTS TO METHOD CALL 'transition_volumepair_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'volume-name-7-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"              => "session2",
##            "volume-name-7-mode"        => [qw(r1 r2)],
##         );
##
##&nbsp;transition volumepair remove -s S1 -v vol1 :
##&nbsp;

sub transition_volumepair_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "volume-name-7-mode"        => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->comment("Navigating to Dashboard and selecting a subproject");

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;

        my $vol_ar = $args{"volume-name-7-mode"};

        foreach (0..@{$vol_ar}-1){
            my $vol = $vol_ar->[$_];
            my $remove=$VOLUME_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                ) ;
            }else{
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Probably the volume requested for does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed volumes from session");
        }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }

    $Log->exit();

    return 1;
} ## end sub transition_volumepair_remove

## @name :: transition_lif_add
##
## @summary Executes "transition_lif_add" command
##
## @description
##  This method is used to execute transition lif add command
## @synopsis
##  transition_lif_add(%args);
## INPUTS TO METHOD CALL 'transition_lif_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'default-gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'home-node' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'home-port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_volumepair_remove(
##            "session-name"             => "session2",
##            "ip-address"        => "1.12.255.7",
##            "default-gateway"           => "1.12.255.1",
##            "netmask"           => "255.0.0.0",
##            "home-node"              => "csiqa-2240-03",
##            "home-port"              => "e0d",
##            "origin-ip-address"        => "1.12.255.7",
##         );
##&nbsp;
sub transition_lif_add {

    my $self = shift;
    my $sleep_count = $GENERIC_WAIT / 1000;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                          => { type => SCALAR },
            "ip-address"                            => { type => SCALAR },
            "default-gateway"                       => { type => SCALAR , optional => 1},
            "netmask"                               => { type => SCALAR , optional => 1},
            "home-node"                             => { type => SCALAR , optional => 1},
            "home-port"                             => { type => SCALAR , optional => 1},
            "origin-ip-address"                     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 0,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );

        $self->_click(
            id                      => $NETWORKING_TAB,
            error_checking_needed   => 0,
        );

        #Invoke Add LIF window
        $self->_click(
            id                      => $GENERIC_ADDLIF_BUTTON,
            error_checking_needed   => 0,
        );

        sleep $sleep_count;
        #If origin IP address speicifed choose IP address from 7mode
        if (defined $args{"origin-ip-address"}) {

            $self->_click(
                id                      => $EXISTING_LIF_RADIO_BUTTON,
                error_checking_needed   => 0,
                );

            $self->_click(
                id                      => $GENERIC_ORIGIN_IP_INPUT,
                error_checking_needed   => 0,
                );

            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $ipaddress = $args{"origin-ip-address"};
            my $originIPselector =  $GENERIC_LISTITEM_SELECTOR;
            $originIPselector =~ s/(.*)XXYYZZ(.*)/$1$ipaddress$2/;

            $self->_click(
                id                      => $originIPselector,
                error_checking_needed   => 0,
                );

        }   else{
            #Choose add new LIF option
            $self->_click(
                id                      => $ADD_NEW_LIF_RADIO_BUTTON,
                error_checking_needed   => 0,
                );

            $self->_type( $GENERIC_TARGET_IP_INPUT ,$args{"ip-address"});
        }

        if (defined $args{"default-gateway"}){
            $self->_type( $GENERIC_TARGET_GATEWAY_INPUT ,$args{"default-gateway"});
        }

        if (defined $args{"netmask"}){
            $self->_type( $GENERIC_TARGET_NETMASK_INPUT ,$args{"netmask"});
        }

        if (defined $args{"home-node"}){
            $self->_click(
                id                      => $GENERIC_TARGET_NODE_INPUT,
                error_checking_needed   => 0,
                );
            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $node = $args{"home-node"};
            my $nodeselector =  $GENERIC_LISTITEM_SELECTOR;
            $nodeselector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;
            $self->_click(
                id                      => $nodeselector,
                error_checking_needed   => 0,
                );
        }

        if (defined $args{"home-port"}){
            $self->_click(
                id                      => $GENERIC_TARGET_PORT_INPUT,
                error_checking_needed   => 0,
                );
             # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $port = $args{"home-port"};
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$port$2/;
            $self->_click(
                id                      => $portselector,
                error_checking_needed   => 0,
                );
        }

        $self->_click(
            id                      => $GENERIC_SAVELIF_BUTTON,
            error_checking_needed   => 1,
            );
        $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
        sleep $sleep_count;

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_lif_add

## @name :: transition_lif_remove
##
## @summary Executes "transition_lif_remove" command
##
## @description
##  This method is used to execute transition lif remove from session
## @synopsis
##  transition_lif_remove(%args);
## INPUTS TO METHOD CALL 'transition_lif_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_lif_remove(
##            "session-name"              => "session2",
##            "ip-address"                => [qw(1.12.255.2 1.12.255.3)],
##         );
##
##&nbsp;
##&nbsp;

sub transition_lif_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"              => { type => SCALAR },
            "ip-address"                => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->comment("Navigating to Dashboard and selecting a subproject");

    my $id = undef ;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        );
        $self->_click(
            id                      => $NETWORKING_TAB,
            error_checking_needed   => 0,
        );
        my $sleep_count = $GENERIC_WAIT / 1000;
        sleep $sleep_count;

        my $ip_ar = $args{"ip-address"};

        foreach (0..@{$ip_ar}-1){
            my $ipaddress = $ip_ar->[$_];
            my $remove=$LIF_REMOVE;
            $remove =~ s/(.*)XXYYZZ(.*)/$1$ipaddress$2/;
            if($self->_is_element_enabled($remove) ){
                $self->_click(
                    id                      => $remove,
                    error_checking_needed   => 1,
                );
            }else{
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Probably the lif requested does not exist or already removed.. Unable to find the element with ".$remove);
            }
            $Log->comment("Successfully removed lifs from session");
        }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    }else{
            $self->_throw_exception("Session ".$args{"session-name"}." does not exist ")
    }

    $Log->exit();

    return 1;
} ## end sub transition_lif_remove

## @name :: transition_schedule_add
##
## @summary Executes "transition_schedule_add" command
##
## @description
##  This method is used to execute transition schedule add command
## @synopsis
##  transition_schedule_add(%args);
## INPUTS TO METHOD CALL 'transition_shedule_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule add -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_add {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR },
            "start-time"                        => { type => SCALAR },
            "duration"                          => { type => SCALAR },
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $CREATE_SCHEDULE_DASHBOARD,
            error_checking_needed   => 0,
        ) ;

        $self->_type( $GENERIC_SCHEDULE_NAME ,$args{"schedule-name"});

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        my $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_add

## @name ::transition_precheck
##
## @summary Executes "transition_precheck" command
##
## @description
##    This method is used to execute transition precheck command
## @synopsis
##      transition_precheck(%args);
## INPUTS TO METHOD CALL 'transition_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'seven-mode-only' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##
## @examples
##        $Api_Set_Obj2->transition_precheck(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##        );
##&nbsp;

sub transition_precheck {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'seven-mode-only'               => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Running prechecks . Will throw an exception if found errors
        if( $self->_is_button_enabled($RUN_PRECHECKS) ) {

            $aref = $self->_click(
                id                      => $RUN_PRECHECKS,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till precheck operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_PRECHECK,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception(
                "Transition precheck on ".$args{"session-name"}." can not be "
                ."run as precheck button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_precheck

## @name ::transition_start
##
## @summary Executes "transition_start" command
##
## @description
##    This method is used to execute transition start command
## @synopsis
##      transition_start(%args);
## INPUTS TO METHOD CALL 'transition_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-precheck' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj2->transition_start(
##            "session-name"                  => "session2",
##            'wait_for_operation'            => 1,
##            "method-timeout"                => 1000
##         );
##&nbsp;transition start -r no -s S21 :
##&nbsp;

sub transition_start {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'no-precheck'                   => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($START) ) {

            $aref = $self->_click(
                id                      => $START,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till start operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_START,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition start on ".$args{"session-name"}
                ." can not be run as start button is disabled");
        }

        Tharn::snooze(5);

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_start

## @name :: transition_update
##
## @summary executes "transition_update" command
##
## @description
##  This method is used to execute transition update command
## @synopsis
##  transition_update(%args);
## INPUTS TO METHOD CALL 'transition_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_update {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'available-transfers-percentage'    => { type => SCALAR },
            'session-snapmirror-throttle'       => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($UPDATE_NOW) ) {
            $aref = $self->_click(
                id                      => $UPDATE_NOW,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition update on ".$args{"session-name"}
                ." can not be run as Update Now button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_update

## @name ::transition_pause
##
## @summary executes "transition_pause" command
##
## @description
##    This method is used to execute transition pause command
## @synopsis
##      transition_pause(%args);
## INPUTS TO METHOD CALL 'transition_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_pause {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($PAUSE) ) {

            $aref = $self->_click(
                id                      => $PAUSE,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till pause operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                my $pause_completed = 0;
                while (1){
                    if ( ( $self->_is_element_present($WAIT_FOR_PAUSE_IN_BASELINE) ) ||
                         ( $self->_is_element_present($WAIT_FOR_PAUSE_IN_INCREMENTAL_D_XFER) )
                    ) {
                        $pause_completed = 1;
                        last;
                    }
                    $timeout--;
                    last unless $timeout;
                }

                unless ( $pause_completed ){
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Even after waiting for "
                        ."$args{'method-timeout'} secs , still the pause did not succeed");
                }

            }
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition pause on ".$args{"session-name"}
                ." can not be run as Pause button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_pause

## @name ::transition_resume
##
## @summary executes "transition_resume" command
##
## @description
##    This method is used to execute transition resume command
## @synopsis
##      transition_resume(%args);
## INPUTS TO METHOD CALL 'transition_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resume {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($RESUME) ) {

            $aref = $self->_click(
                id                      => $RESUME,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till resume operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                my $resume_completed = 0;
                while (1){
                    if ( !( $self->_is_element_present($WAIT_FOR_PAUSE_IN_BASELINE) ) &&
                         !( $self->_is_element_present($WAIT_FOR_PAUSE_IN_INCREMENTAL_D_XFER) )
                    ) {
                        $resume_completed = 1;
                        last;
                    }
                    $timeout--;
                    last unless $timeout;
                }

                unless ( $resume_completed ){
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Even after waiting for"
                        ." $args{'method-timeout'} secs , still the resume did not succeed");
                }

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition resume on ".$args{"session-name"}
                ." can not be run as Resume button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_resume

## @name :: transition_precutover
##
## @summary Executes "transition_precutover" command
##
## @description
##  This method is used to execute transition precutover command
## @synopsis
##  transition_precutover(%args);
## INPUTS TO METHOD CALL 'transition_precutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapmirror-throttle' <td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'ad-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ad-server-ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'ad-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'ad-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'kdc-server-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'kdc-server-admin-user-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'kdc-server-password' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
##&nbsp;

sub transition_precutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                  => { type => SCALAR },
            'mode'                          => { type => SCALAR },
            'available-transfers-percentage'=> { type => SCALAR },
            "max-cap-concurrent-transfers"  => { type => SCALAR , optional => 1},
            'snapmirror-throttle'           => { type => SCALAR , optional => 1},
            'ad-server-host-name'           => { type => SCALAR , optional => 1},
            'ad-server-ip-address'          => { type => SCALAR , optional => 1},
            'ad-server-admin-user-name'     => { type => SCALAR , optional => 1},
            'ad-server-password'            => { type => SCALAR , optional => 1},
            'kdc-server-host-name'          => { type => SCALAR , optional => 1},
            'kdc-server-admin-user-name'    => { type => SCALAR , optional => 1},
            'kdc-server-password'           => { type => SCALAR , optional => 1},
            'wait_for_operation'            => { type => SCALAR , optional => 1},
            'method-timeout'                => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($PRECUTOVER) ) {

            if( $args{mode} eq "rw_test" ){
                $self->_click(
                    id                      => $TEST_MODE,
                    error_checking_needed   => 1,
                ) ;
            }

            $aref = $self->_click(
                id                      => $PRECUTOVER,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till precutover operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                if( $args{mode} eq "rw_test" ){
                    $self->_wait_on_attribute(
                        attribute_to_check   => $RESYNC,
                        "method-timeout"     => $timeout,
                    ) ;
                } else {
                    $self->_wait_on_attribute(
                        attribute_to_check   => $WAIT_FOR_PRECUTOVER,
                        "method-timeout"     => $timeout,
                    ) ;
                }

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition precutover on ".$args{"session-name"}
                ." can not be run as Copy-Configuration button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_precutover

## @name :: transition_resync
##
## @summary Executes "transition_resync" command
##
## @description
##  This method is used to execute re-sync on a transition session
## @synopsis
##  transition_resync(%args);
## INPUTS TO METHOD CALL 'transition_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_resync {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($RESYNC) ) {

            $aref = $self->_click(
                id                      => $RESYNC,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till resync operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_RESYNC,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition resync on ".$args{"session-name"}
                ." can not be run as Finish Testing button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_resync

## @name :: transition_cutover
##
## @summary Executes "transition_cutover" command
##
## @description
##  This method is used to execute transition cutover command
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_cutover {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'available-transfers-percentage'    => { type => SCALAR },
            'session-snapmirror-throttle'       => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($CUTOVER) ) {

            $aref = $self->_click(
                id                      => $CUTOVER,
                error_checking_needed   => 1,
                info_hash               => \%args,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till cutover operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_CUTOVER,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition cutover on ".$args{"session-name"}
                ." can not be run as Complete Transition button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_cutover

## @name ::transition_abort
##
## @summary Executes "transition_abort" command
##
## @description
##    This method is used to execute transition abort command
## @synopsis
##      transition_abort(%args);
## INPUTS TO METHOD CALL 'transition_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##&nbsp;

sub transition_abort {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            'wait_for_operation'                => { type => SCALAR , optional => 1},
            'method-timeout'                    => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output ;

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {

        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        if( $self->_is_button_enabled($ABORT) ) {

            $aref = $self->_click(
                id                      => $ABORT,
                error_checking_needed   => 1,
            ) ;

            if ( $args{wait_for_operation} ) {
                # Waiting till abort operation is completed

                my $timeout = $self->{timeout};
                if ( defined $args{'method-timeout'} ){
                    $timeout = $args{'method-timeout'};
                }

                $self->_wait_on_attribute(
                    attribute_to_check   => $WAIT_FOR_ABORT,
                    "method-timeout"     => $timeout,
                );

            }

        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Transition abort on ".$args{"session-name"}
                ." can not be run as Abort button is disabled");
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $output->{parsed_output} = $aref->[0] ;

    $Log->exit();

    return $output;
} ## end sub transition_abort

## @name :: transition_schedule_modify
##
## @summary Executes "transition_schedule_modify" command
##
## @description
##  This method is used to execute transition schedule modify command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'days-range' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'start-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'duration' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'max-cap-concurrent-transfers' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'available-transfers-percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'session-snapmirror-throttle' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'update-frequency' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_modify {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
            "days-range"                        => { type => SCALAR , optional => 1},
            "start-time"                        => { type => SCALAR , optional => 1},
            "duration"                          => { type => SCALAR , optional => 1},
            "update-frequency"                  => { type => SCALAR , optional => 1},
            "available-transfers-percentage"    => { type => SCALAR , optional => 1},
            "max-cap-concurrent-transfers"      => { type => SCALAR , optional => 1},
            "session-snapmirror-throttle"       => { type => SCALAR , optional => 1},
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_MODIFIER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        # Clearing off all the days
        foreach (0..6){
            my $days = {
                0 => "sunday",
                1 => "monday",
                2 => "tuesday",
                3 => "wednesday",
                4 => "thursday",
                5 => "friday",
                6 => "saturday",
            };
            my $day = $days->{$_};
            my $selector =  $GENERIC_SCHEDULE_DAYS_SELECT;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$day$2/;
            my $checked = $selector.'[@checked=""]';
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) if ( $self->_is_element_present($checked) );
        }

        $self->_add_schedule_days($args{"days-range"});

        # Adding start time
        my @s_t = split(":",$args{"start-time"});
        $selector=$GENERIC_AGGR_SELECTOR;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/; 
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Adding duration
        my @duration = split(":",$args{"duration"});
        $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
            error_checking_needed   => 0,
        ) ;
        $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        if ( defined $args{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$args{"available-transfers-percentage"} );
        }

        if ( defined $args{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
            $self->_type( $GENERIC_STREAM_SELECTOR ,$args{"max-cap-concurrent-transfers"} );
        }

        # Adding throttle
        if (defined $args{"session-snapmirror-throttle"}){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 1,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,$args{"session-snapmirror-throttle"} );
        }

        # Adding update frequency
        if (defined $args{"update-frequency"}){
            my @u_f = split(":",$args{"update-frequency"});
            $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
            $self->_click(
                id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_AGGR_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

        $self->_click(
            id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
            error_checking_needed   => 1,
        ) ;

        if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
            my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Error while adding schedule : ".$text);
        }

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_modify

## @name :: transition_schedule_remove
##
## @summary Executes "transition_schedule_remove" command
##
## @description
##  This method is used to execute transition schedule remove command
## @synopsis
##  transition_schedule_modify(%args);
## INPUTS TO METHOD CALL 'transition_schedule_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'schedule-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition schedule modify -s S21 -n sc1 -d 1-5 -b 12:00 -e 10:00 -t 2 -x 2 :
##&nbsp;
##&nbsp;

sub transition_schedule_remove {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
            "schedule-name"                     => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 0,
        ) ;

        my $selector=$GENERIC_SCHEDULE_REMOVER;
        my $ses_name=$args{"schedule-name"};
        $selector =~ s/(.*)XXYYZZ(.*)/$1$ses_name$2/;

        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_schedule_remove

## @name :: transition_delete
##
## @summary Executes "transition_delete" command
##
## @description
##  This method is used to execute transition delete command
## @synopsis
##  transition_delete(%args);
## INPUTS TO METHOD CALL 'transition_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'session-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
##@examples
##&nbsp;transition delete -s S21 :
##&nbsp;
##&nbsp;

sub transition_delete {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "session-name"                      => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $id = undef ;
    if( $self->_is_button_enabled_cbt($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        my $project_id = $self->_get_project_id($id);

        $self->_click(
            id                      => $project_id,
            error_checking_needed   => 1,
        ) ;

        $self->_click(
            id                      => $DELETE_PROJECT,
            error_checking_needed   => 1,
        ) ;

        $self->_close_tab(
            string => "Dashboardtab",
        );

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $Log->exit();

    return 1;
} ## end sub transition_delete

sub transition_apt_add_systems {

    my $self = shift;

    my %args = validate_with(
	params => \@_,
	spec   => {
            "host-name"  => { type => SCALAR },
            "username"   => { type => SCALAR },
            "password"   => { type => SCALAR },
            "type"       => { type => SCALAR },
	},
	allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $self->_click(
	id                      =>  $ASSESSMENT_BUTTON,
	error_checking_needed   =>  1,
    );

    $self->_click_button_having_specific_text_no_error_check($APT_ADD_SYSTEMS);

    $self->_type( $APT_ADD_SYSTEMS_HOSTNAME ,$args{"host-name"});
    $self->_type( $APT_ADD_SYSTEMS_USERNAME ,$args{"username"});
    $self->_type( $APT_ADD_SYSTEMS_PASSWORD ,$args{"password"});

    $self->_click(
	id                      =>  $APT_ADD_SYSTEMS_TYPE,
	error_checking_needed   =>  0,
    );

    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);

    my $selector = $GENERIC_LISTITEM_SELECTOR;
    my $type = $args{"type"};
    $selector =~ s/(.*)XXYYZZ(.*)/$1$type$2/;

    if( $self->_is_element_present($selector) ){
	$self->_click(
	    id                      => $selector,
	    error_checking_needed   => 0,
	);
        $self->_click(
	    id                      => $APT_ADD_SYSTEMS_BUTTON,
	    error_checking_needed   => 0,
	);
    }else{
	$self->_take_screenshot();
	$self->_repair_session();
	$self->_throw_exception("Type requested for does not exist : ".$selector);
    }

    $self->_click(
	id                      =>  $MIGRATE_BUTTON,
	error_checking_needed   =>  0,
    );

    $Log->exit();

    return 1;
} ## end transition_apt_add_systems

sub transition_apt_generate_report {

    my $self = shift;

    my %args = validate_with(
	params => \@_,
	spec   => {
            "host-name"        => { type => SCALAR },
            "targer-cluster"   => { type => SCALAR },
            "file-prefix"      => { type => SCALAR , optional => 1},
            "timeout"          => { type => SCALAR , optional => 1},
	},
	allow_extra => 1
    );

    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $self->_click(
	id                      =>  $ASSESSMENT_BUTTON,
	error_checking_needed   =>  1,
    );

    my $host = $args{"host-name"};
    my $tc = $args{"targer-cluster"};
    my $fp = $args{"file-prefix"};
    my $timeout = $args{'timeout'} || ($GENERIC_WAIT / 1000);

    my $selector = $APT_SELECT_SYSTEM;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$host$2/;

    $self->_click(
	id                      =>  $selector,
	error_checking_needed   =>  0,
    );

    $selector = $GENERIC_LISTITEM_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$tc$2/;

    $self->_click_button_having_specific_text_no_error_check("Create Transition Assessment Report");

    my $elem = $self->_find_element( $APT_SELECT_TARGET_CLUSTER ) ;
    $elem->click();

    if( $self->_is_element_present($selector) ){
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Target cluster requested for does "
            ."not exist.. Unable to find the element with ".$selector);
    }

    $self->_type( $APT_FILE_PREFIX_SYSTEM ,$fp);

    $self->_click_button_having_specific_text_no_error_check("Generate Report");

    unless( $self->_wait_for_element_not_present( $APT_WAIT_FOR_REPORT,$timeout ) ){
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception(
            "Report did not get generated within given timeout !"
        );
    }

    $self->_click(
	id                      =>  $MIGRATE_BUTTON,
	error_checking_needed   =>  0,
    );

    $Log->exit();

    return 1;
} ## end transition_apt_generate_report

## @name :: transition_cft_create
##
## @summary Executes "transition_cft_create" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cutover(%args);
## INPUTS TO METHOD CALL 'transition_cutover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> '7-mode1-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> '7-mode2-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'wait_for_operation' </td> <td> '1' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##      my %S_mode_credentials = (
##                  'host-name'        => $Host7Mode[0]->{MGMT_IP},
##                  username           => "root",
##                  'password'         => $Host7Mode[0]->{default_password}
##              );
##          my %S_mode_credentials_HA = (
##                  'host-name'        => $Host7Mode[1]->{MGMT_IP},
##                  username           => "root",
##                  'password'         => $Host7Mode[1]->{default_password}
##              );
##          my %C_mode_credentials = (
##                  "host-name"        => $HostCMode[0]->{CLUSTER_MGMT_IP},
##                  username           => "admin",
##                  'password'         => $HostCMode[0]->{default_password}
##              );
##          my %cft_clusterinfo_add=(
##                  "c-mode-host-name" =>$HostCMode[0]->{CLUSTER_MGMT_IP},
##              );
##          my %cft_svmmapping_add_pri = (
##                  "7-mode-ip-or-fqdn"         => $Host7Mode[0]->{MGMT_IP},
##                  "vfiler-name"               => $VFILER_PRI,
##                  "svm-name"                  => $VSERVER_PRI,
##              );
##          my %cft_svmmapping_add_sec = (
##                  "7-mode-ip-or-fqdn"         => $Host7Mode[1]->{MGMT_IP},
##                  "vfiler-name"               => $VFILER_SEC,
##                  "svm-name"                  => $VSERVER_SEC,
##              );
##          my %cft_nodemapping_set = (
##                  "cluster-node1-name"        => $NodeCMode[0]->name(),
##                  "cluster-node2-name"        => $NodeCMode[1]->name(),
##              );
##
##          $Api_Set_Obj->transition_cft_create(
##              command_interface       => $HostWindows,
##              "project-name"          => $PROJECT_NAME,
##              "7-mode1-host-name"     => $Host7Mode[0]->{MGMT_IP},
##              "7-mode2-host-name"     => $Host7Mode[1]->{MGMT_IP},
##              "group-name"            => $GROUP_NAME,
##              S_mode_credentials      => \%S_mode_credentials,
##              C_mode_credentials      => \%C_mode_credentials,
##              S_mode_credentials_HA   => \%S_mode_credentials_HA,
##              cft_clusterinfo_add     => \%cft_clusterinfo_add,
##              cft_svm_mapping_add_pri => \%cft_svmmapping_add_pri,
##              cft_svm_mapping_add_sec => \%cft_svmmapping_add_sec,
##              cft_nodemapping_set     => \%cft_nodemapping_set,
##          );
##
##&nbsp;transition cft revert precheck -p Test -t new :
##&nbsp;

sub transition_cft_create {

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"            => { type => SCALAR , optional => 1},
            "group-name"              => { type => SCALAR , optional => 1},
            "7-mode1-host-name"       => { type => SCALAR , optional => 1},
            "7-mode2-host-name"       => { type => SCALAR , optional => 1},
            "S_mode_credentials"      => { optional => 1  },
            "C_mode_credentials"      => { optional => 1  },
            "S_mode_credentials_HA"   => { optional => 1  },
            "cft_clusterinfo_add"     => { optional => 1  },
            "cft_svm_mapping_add_pri" => { optional => 1  },
            "cft_svm_mapping_add_sec" => { optional => 1  },
            "cft_nodemapping_set"     => { optional => 1  },
        },
        allow_extra => 1,
    );

    
    $Log->enter();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_cft_create_helper(\%args);

    $Log->exit();

    return 1;
}

## @name :: transition_cft_clusterinfo_add
##
## @summary Executes "transition_cft_clusterinfo_add" command
##
## @description
##  This method is used to execute transition cft clusterinfo add command
## @synopsis
##  transition_cft_clusterinfo_add(%args);
## INPUTS TO METHOD CALL 'transition_cft_clusterinfo_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'c-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_clusterinfo_add(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "c-mode-host-name"  => $HostCMode[0]->{CLUSTER_MGMT_IP},
##         );
##
##&nbsp;transition cft clusterinfo add -p Test -h 10.72.213.45 :
##&nbsp;

sub transition_cft_clusterinfo_add {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"      => { type => SCALAR , optional => 1},
            "c-mode-host-name"  => { type => SCALAR , optional => 0},
        },
        allow_extra => 1,
    );
    $Log->enter();

        push @{$self->{$args{"project-name"}}} , { cft_clusterinfo_add_args => \%args} ;
        use Data::Dumper;
        $Log->comment( sub { "Input contains after addition " . Dumper( \$self ) } );
        my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_clusterinfo_add_args","c-mode-host-name",$args{"project-name"});
        my $host_cmode = $self->_get_host_from_ip($ip,"c");
        $self->_put_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args",$ip,$args{"project-name"},$host_cmode);

    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        my $cft_edit_config = $CFT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;

        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $EDIT_SELECT_TARGET_CLUSTER,
            error_checking_needed   => 0,
        ) ;
        my $cft_cluster = $CFT_CLUSTER;
        $cft_cluster =~ s/(.*)XXYYZZ(.*)/$1$host_cmode$2/;
        if($self->is_element_present($cft_cluster)){
            $self->_click(
                id                      => $cft_cluster,
                error_checking_needed   => 0,
            ) ;
        }
        my $var = 0;
        while (1) {
            my $c_ha_selector =$CLUSTER_HA_SELECTOR;
            $c_ha_selector =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
            if ($self->_is_element_present($c_ha_selector)) {
                $self->_click(
                    id                      => $c_ha_selector,
                    error_checking_needed   => 0,
                )if(! $self->_is_checked($c_ha_selector) );
                last;
            }
            $var++;
        }
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;
        $self->_close_tab(
            project => $args{"project-name"},
        );
        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }
    $Log->exit();

    return 1;
}

## @name :: transition_cft_nodemapping_set
##
## @summary Executes "transition_cft_nodemapping_set" command
##
## @description
##  This method is used to execute transition cft nodemapping set command
## @synopsis
##  transition_cft_nodemapping_set(%args);
## INPUTS TO METHOD CALL 'transition_cft_nodemapping_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'cluster-node1-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'cluster-node2-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_precheck(
##            "project-name"       => $PROJECT_NAME,
##            "group-name"         => $GROUP_NAME,
##            "cluster-node1-name" => $NodeCMode[0]->name(),
##            "cluster-node2-name" => $NodeCMode[1]->name(),
##         );
##
##&nbsp;transition cft nodemapping set -p Test -u 10.71.213.101 -v 10.71.213.102 :
##&nbsp;

sub transition_cft_nodemapping_set{

    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "cluster-node1-name" => { type => SCALAR , optional => 1},
            "cluster-node2-name" => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();

    my $pri_clus_node = $PRIMARY_CLUS_NODE;
    my $pri_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
    "cft_nodemapping_set_args","cluster-node1-name",$args{"project-name"});
    $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
    my $sec_clus_node = $SECONDARY_CLUS_NODE;
    my $sec_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
        "cft_nodemapping_set_args","cluster-node2-name",$args{"project-name"});
    $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;

    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_nodemapping_set_args => \%args} ;
        my $cft_edit_config = $CFT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $EDIT_SELECT_TARGET_CLUSTER,
            error_checking_needed   => 0,
        ) ;

        my $pri_clus_node = $PRIMARY_CLUS_NODE;
        my $pri_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node1-name",$args{"project-name"});
        $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
        my $sec_clus_node = $SECONDARY_CLUS_NODE;
        my $sec_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node2-name",$args{"project-name"});
        $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;
        if (($self->_is_element_present($pri_clus_node)) && ($self->_is_element_present($sec_clus_node))) {
            print"the node mapping is already done\n";
        }else{
            $self->_click(
                id                      => $SWAP_BUTTON,
                error_checking_needed   => 0,
            ) ;
        }
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;
        $self->_close_tab(
            project => $args{"project-name"},
        );
        $self->_close_tab(
            string => "Migration",
        );
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit();

    return 1;
}

## @name :: transition_cft_svmmapping_add
##
## @summary Executes "transition_cft_svmmapping_add" command
##
## @description
##  This method is used to execute transition cft svmmapping add command
## @synopsis
##  transition_cft_svmmapping_add(%args);
## INPUTS TO METHOD CALL 'transition_cft_svmmapping_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> '7-mode-host-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'svm-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_svmmapping_add(
##            "7-mode-ip-or-fqdn"         => $Host7Mode[0]->{MGMT_IP},
##            "vfiler-name"               => $VFILER_PRI,
##            "svm-name"                  => $VSERVER_PRI,
##         );
##
##&nbsp;transition cft svmmapping add -p Test -n 10.71.213.101 -f vfiler0 -v vs1 :
##&nbsp;

sub transition_cft_svmmapping_add {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "7-mode-host-name"   => { type => SCALAR , optional => 1},
            "vfiler-name"        => { type => SCALAR , optional => 1},
            "svm-name"           => { type => SCALAR , optional => 1},
            "svm-type"           => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_svmmapping_set_args => \%args} ;
        $Log->comment( sub { "Input Hash contains  after adding" . Dumper( \$self ) } );
        print "The Project exists\n";
        my $cft_edit_config = $CFT_EDIT_CONFIG;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;
    
        $self->_click(
            id                      => $EDIT_SVM_MAPPING,
            error_checking_needed   => 0,
        ) ;

        $self->_select_vsm($args{"7-mode-host-name"},$args{"vfiler-name"},$args{"svm-name"});
        $self->_close_tab(
            project => $args{"project-name"},
        );
        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }
    
    $Log->exit();

    return 1;

}

## @name :: transition_cft_delete
##
## @summary Executes "transition_cft_delete" command
##
## @description
##  This method is used to execute transition cft delete command
## @synopsis
##  transition_cft_delete(%args);
## INPUTS TO METHOD CALL 'transition_cft_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_delete(
##            "project-name"        => $project_name,
##            "group-name"          => $group_name,
##         );
##
##&nbsp;transition cft delete -p Test  :
##&nbsp;

sub transition_cft_delete {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        push @{$self->{$args{"project-name"}}} , { cft_svmmapping_set_args => \%args} ;
        $Log->comment( sub { "Input Hash contains  after adding" . Dumper( \$self ) } );
        print "The Project exists\n";
        my $cft_edit_config = $CFT_MODIFY;
        $cft_edit_config =~ s/(.*)XXYYZZ(.*)/$1$args{"project-name"}$2/;
        $self->_click(
            id                      => $cft_edit_config,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $PROJECT_DELETE,
            error_checking_needed   => 1,
        ) ;

        $self->_create_screenshot($args{"project-name"}."_deleted");
        $self->_close_tab(
            string => "Migration",
        );

    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit();

    return 1;

}

## @name :: transition_cft_precheck
##
## @summary Executes "transition_cft_precheck" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_precheck(%args);
## INPUTS TO METHOD CALL 'transition_cft_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_precheck(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsedoutput is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft precheck -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_precheck {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CFT_Precheck";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }

    my $state = undef;
    if ($parsed_output != 1) {
        $state = $PRECHECK_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_PRECHECK_BUTTON,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

sub transition_cft_check_readiness {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "CFT_Check_Readiness";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $PRECHECK_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_CHECK_READINESS,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_apply_svm_config
##
## @summary Executes "transition_cft_apply_svm_config" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_precheck(%args);
## INPUTS TO METHOD CALL 'transition_cft_apply_svm_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_apply_svm_config(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsedoutput is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft apply-svm-config -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_apply_svm_config {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Apply_svm_config";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $APPLY_SVM_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_APPLY_CONFIG,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_start
##
## @summary Executes "transition_cft_start" command
##
## @description
##  This method is used to execute transition cft start command
## @synopsis
##  transition_cft_start(%args);
## INPUTS TO METHOD CALL 'transition_cft_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_start(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft start -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_start {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Export";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $EXPORT_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_EXPORT,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;
}

## @name :: transition_cft_verify_cabling
##
## @summary Executes "transition_cft_verify_cabling" command
##
## @description
##  This method is used to execute transition cft verify cabling command
## @synopsis
##  transition_cft_start(%args);
## INPUTS TO METHOD CALL 'transition_cft_verify_cabling' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_verify_cabling(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft verify-cabling -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_verify_cabling {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Verify_Cabling";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $VERIFY_CABLE_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_VERIFY_CABLING,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_import
##
## @summary Executes "transition_cft_import" command
##
## @description
##  This method is used to execute transition cft import command
## @synopsis
##  transition_cft_import(%args);
## INPUTS TO METHOD CALL 'transition_cft_import' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_import(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft import -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_import {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Import";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $IMPORT_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_IMPORT,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;
}

## @name :: transition_cft_commit
##
## @summary Executes "transition_cft_commit" command
##
## @description
##  This method is used to execute transition cft commit command
## @synopsis
##  transition_cft_commit(%args);
## INPUTS TO METHOD CALL 'transition_cft_commit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_commit(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft commit -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_commit {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Commit";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $COMMIT_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$CFT_COMMIT,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_revert_precheck
##
## @summary Executes "transition_cft_revert_precheck" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_commit(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_precheck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_precheck(
##            "project-name"      => "Test",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft revert precheck -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_precheck {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    if (defined $args{"group-name"}) {
        $group = $args{"group-name"};
    }else{
        $group = "Default";
    }
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Revert_precheck";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;

    my $result_array = $self->_perform_operation($args{"project-name"},$REVERT_PRECHECK,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}


sub transition_cft_revert_abort {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();

    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;

    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $var=1;
    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        $self->_click(
            id                      => $locator,
            error_checking_needed   => 0,
        ) ;
        my $popup_title;
        if( $self->_is_button_enabled($CANCEL_ROLLBACK) ) {
            $self->_click(
                id                      => $CANCEL_ROLLBACK,
                error_checking_needed   => 0,
            ) ;
            eval{
            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
            };
            if ( ($popup_title =~ /Confirmation/i )){
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }
            $self->_create_screenshot("rollback_abort_success");
            $self->_close_tab(
                string => "Migration",
            );

        }else{
            $self->_create_screenshot("rollback_cancel_disabled");
            $self->_close_tab(
                string => "Migration",
            );
            $self->_throw_exception("rollback cancel button is disabled");
        }
    }else{
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("Project with name ".$args{"project-name"}." does not exists");
    }

    $Log->exit();

    return 1;

}


## @name :: transition_cft_revert_generate_manual_steps
##
## @summary Executes "transition_cft_revert_generate_manual_steps" command
##
## @description
##  This method is used to execute transition cft revert generate-manual-steps command
## @synopsis
##  transition_cft_revert_generate_manual_steps(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_generate_manual_steps' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> '' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_generate_manual_steps(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "parsed_output"     => 1,
##         );
##
##&nbsp;transition cft revert generate-manual-steps -p Test :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [],
##&nbsp;           'Warning' => [
##&nbsp;                         {
##&nbsp;                           'corrective_action' => '',
##&nbsp;                           'type' => 'Warning',
##&nbsp;                           'code' => '08023',
##&nbsp;                           'message' => 'If any new files, directories or qtrees were created on the transitioned 7-Mode aggregates during the Precommit Testing phase, those objects will be deleted during the revert operation.'
##&nbsp;                         }
##&nbsp;                        ],
##&nbsp;           'Informational' => [
##&nbsp;                                {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'f8020-213-101-102(10.238.219.45)' => [
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr cc61db8e-87af-11df-8350-123478563412:1574236446:AGGR_TRANS_RAID4_1_CFT_SNAPSHOT_ON_7MODE_20150706105757909:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr 1e531da9-879b-11df-b134-123478563412:1574236446:AGGR0_0_CFT_SNAPSHOT_ON_7MODE_20150706105757331:true',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-102 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr a4dd55b0-1fdd-11e5-9f77-123478563412:1574236413:AGGR1_CFT_SNAPSHOT_ON_7MODE_20150706105758565:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-101 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr c61483bb-87af-11df-8350-123478563412:1574236446:AGGR_TRANS_RAID_DP_1_CFT_SNAPSHOT_ON_7MODE_20150706105757800:false',
##&nbsp;                                                                         'storage transition revert start -session-id 011b4fd2-f7eb-492a-beaa-2849fd6529b8 -node-name f8020-213-102 -kernel-version kern_vers_8_1 -aggr-trans-revert-attr 34f7c768-1fc2-11e5-b14f-123478563412:1574236413:AGGR0_1_CFT_SNAPSHOT_ON_7MODE_20150706105758019:true'
##&nbsp;                                                                       ],
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08081',
##&nbsp;                                 'message' => 'Required: Run the following diagnostic privilege level commands to revert the aggregates to their original 7-Mode state. You can verify the revert status of the aggregates by using the following command at the diagnostic privilege level: \'storage transition aggregates show-status\'.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_generate_manual_steps {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "Generate_manual_step";
    my $parsed_output = 1;
    #$parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;

    my $result_array = $self->_perform_operation($args{"project-name"},$GENERATE_MANUAL_STEPS,$group,$operation,
                                                 $parsed_output);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_revert_verify7_mode
##
## @summary Executes "transition_cft_revert_verify7_mode" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_revert_verify7_mode(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_verify7_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'parsed-output' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_revert_verify7_mode(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "parsed_output"     => 1,
##         );
##
## By default parsed_output will be "1",it will process all the precheck message and send it as
## parsed output. If in case parsed output is not need, it can be skipped by specifying
## parsed_output as "0"(Here the command does not return any parsed output)
##
##&nbsp;transition cft revert verify7-mode -p Test:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error' => [
##&nbsp;                          {
##&nbsp;                         'corrective_action' => 'To continue with the transition, create these SVMs on the target cluster. For more information, refer to the 7-Mode Transition Tool Copy-Free Transiustered Data ONTAP Administration Guide.',
##&nbsp;                         'type' => 'Error',
##&nbsp;                         'code' => '52250',
##&nbsp;                         'message' => 'The following target SVMs that are added to the project do not exist on the cluster.'
##&nbsp;                       },
##&nbsp;                       ]
##&nbsp;           'Warning' => [],
##&nbsp;           'Informational' => [
##&nbsp;                               {
##&nbsp;                                 'corrective_action' => '',
##&nbsp;                                 'type' => 'Informational',
##&nbsp;                                 'code' => '08021',
##&nbsp;                                 'message' => 'Successfully verified that all the disk shelves that contain 7-Mode aggregate disks does not contain any native aggregates and are eligible for the revert operation.'
##&nbsp;                               },
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub transition_cft_revert_verify7_mode {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
            'run-background'     => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "verify7-mode";
    my $parsed_output = undef;
    my $run_background = undef;

    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    $run_background = (defined $args{"run-background"}) ? $args{"run-background"}: undef;
    if ($run_background == 1) {
        $parsed_output = undef;
    }
    
    my $state = undef;
    if ($parsed_output != 1) {
        $state = $REVERT_SUCCESS;
    }

    my $result_array = $self->_perform_operation($args{"project-name"},$VERIFY_SEVEN_MODE,$group,$operation,
                                                 $parsed_output,$run_background,$state);

    $Log->exit();

    return (defined $result_array) ? $result_array : 1;

}

## @name :: transition_cft_confirm_execution_revert_manual_steps
##
## @summary Executes "transition_cft_confirm_execution_revert_manual_steps" command
##
## @description
##  This method is used to execute transition cft create command
## @synopsis
##  transition_cft_confirm_execution_revert_manual_steps(%args);
## INPUTS TO METHOD CALL 'transition_cft_revert_verify7_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'project-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'group-name' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'method-timeout' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'seven-mode' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'cabling' </td> <td> '0' </td> </tr>
## <tr> <td> ' ' </td> <td> 'cluster-mode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @examples
##         $Api_Set_Obj->transition_cft_confirm_execution_revert_manual_steps(
##            "project-name"      => "Test",
##            "group-name"        => "new",
##            "cluster-mode"      => "yes",
##            "cabling"           => "yes",
##            "seven-mode"        => "yes",
##         );
##
##&nbsp;
##&nbsp;

sub transition_cft_confirm_execution_revert_manual_steps {
    my $self = shift;

    my %args = validate_with(
        params => \@_,
        spec   => {
            "project-name"       => { type => SCALAR , optional => 1},
            "group-name"         => { type => SCALAR , optional => 1},
            'method-timeout'     => { type => SCALAR , optional => 1},
            'seven-mode'         => { type => SCALAR , optional => 1},
            'cabling'            => { type => SCALAR , optional => 1},
            'cluster-mode'       => { type => SCALAR , optional => 1},
            'parsed-output'      => { type => SCALAR , optional => 1},
        },
        allow_extra => 1,
    );

    $Log->enter();
    my $group = undef;
    my $var=1;
    $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    $Log->comment( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $operation = "verify7-mode";
    my $parsed_output = undef;
    $parsed_output = (defined $args{"parsed-output"}) ? $args{"parsed-output"}: 1;
    my $state = undef;

    $self->_click(
        id                      => $MIGRATION_DASHBOARD_LINK,
        error_checking_needed   => 0,
    ) ;

    while (1) {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }

    my $locator = "gwt-debug-".$args{"project-name"};
    if ($self->_is_element_present($locator)) {
        $self->_click(
            id                      => $locator,
            error_checking_needed   => 0,
        ) ;
        if ( lc $args{"cluster-mode"} eq "yes" ) {
            if ($self->_is_button_enabled($CMODE_CONFIRM_EXECUTION)) {
                $self->_click(
                    id                      => $CMODE_CONFIRM_EXECUTION,
                    error_checking_needed   => 1,
                ) ;
                if ( lc $args{"cabling"} eq "yes" ) {
                    if ($self->_is_button_enabled($CABLING_CONFIRM)) {
                        $self->_click(
                            id                      => $CABLING_CONFIRM,
                            error_checking_needed   => 1,
                        ) ;
                        if ( lc $args{"seven-mode"} eq "yes" ) {
                            if ($self->_is_button_enabled($SMODE_CONFIRM_EXECUTION)) {
                                $self->_click(
                                    id                      => $SMODE_CONFIRM_EXECUTION,
                                    error_checking_needed   => 1,
                                ) ;
                                $self->_close_tab(
                                    string => "Migration",
                                );
                            }else{
                                $self->_create_screenshot("Smode_confirmation_button_disabled");
                                $self->_close_tab(
                                    string => "Migration",
                                );
                                $self->_throw_exception("The smode confirmation button is disbaled");
                            }
                        }
                    }else{
                        $self->_create_screenshot("Cabling_confirmation_button_disabled");
                        $self->_close_tab(
                            string => "Migration",
                        );
                        $self->_throw_exception("The cabling confirmation button is disbaled");
                    }
                }
            }else{
                $self->_create_screenshot("Cmode_confirmation_button_disabled");
                $self->_close_tab(
                    string => "Migration",
                );
                $self->_throw_exception("The cmode confirmation button is disbaled");
            }
        }
    }else{
        $self->_create_screenshot("Project_does_not_exits");
        $self->_close_tab(
            string => "Migration",
        );
        my $project =$args{"project-name"};
        $self->_throw_exception("The  project $project is not present");
    }

    $Log->exit();

    return 1;

}

1;
