#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets related to GUI operations
## @author sindhe@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::GUI::SysManager;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#use Module Imports
use NACL::APISet::Response::GUI;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use Params::Validate qw(validate_pos SCALAR );

# Import Exception class
use NATE::BaseException qw(:try);

# Base class
use base qw(NACL::APISet::Host);

our $AUTOLOAD;

###############################################################################
# Method Name: new
# Objective:  This function creates a CMode GUI object
###############################################################################

sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;

    my $self = {};
    $self =
      NACL::APISet::validate_params($class,
                                    params => \%args,
                                    spec   => {},);

    bless $self, $class;

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method Name: AUTOLOAD
# Objective:  This function will be called when method is missing
###############################################################################

sub AUTOLOAD {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my ($response, $auto);
    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    if ($method eq "DESTROY") {
        $Log->exit() if $may_exit;
        return;
    }

    # Call invoke with the arguments given to execute the command
    $Log->exit() if $may_exit;
    eval {
        my $exception;
        try {
            $auto = $self->{session}->$method(%args);
        } otherwise {
            $exception = shift;
        };
        die $exception->text() if ($exception);
        unless (ref $auto) {
            $auto = {};
        }
        $auto->{'method_name'} = $method;
        $auto->{execution_status} = "0";
    };
    if ($@) {
        $auto                     = {};
        $auto->{'method_name'}    = $method;
        $auto->{execution_status} = "FAIL";
        $auto->{output}           = $@;
    }

    $response = NACL::APISet::Response::GUI->new($auto);
    $Log->exit() if $may_exit;
    return $response;
} ## end sub AUTOLOAD

##############################################################################
# Method Name:    parse_system_details
# Objective: Creates hash with System IP, System Name, Username and Password.
##############################################################################

sub parse_system_details {
    my $self = shift;
    my ($systemDetails) = validate_pos(@_, { type => SCALAR });
    $Log->enter() if $may_enter;

    my @sysDetail = split(/\|/, $systemDetails);
    if (scalar(@sysDetail) != 4) {
        my $cnt = scalar(@sysDetail);
        NATE::BaseException->throw(
           "Incorrect System Details.\n$cnt\n@sysDetail\n");
    }

    $Log->exit() if $may_exit;
    return ($SYSTEM_IP_KEY       => $sysDetail[0],
            $SYSTEM_NAME_KEY     => $sysDetail[1],
            $SYSTEM_USERNAME_KEY => $sysDetail[2],
            $SYSTEM_PASSWORD_KEY => $sysDetail[3]);

} ## end sub parse_system_details

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::GUI::SysManager - Base class for APISets for GUI operations.

=head1 SYNOPSIS

use NACL::APISet::Host::GUI::SysManager;

=head1 DESCRIPTION

Contains Common routines for libraries under NACL/APISet/Host/GUI/SysManager

=head1 METHODS

=head2 new

This method is used to create a NACL::APISet::Host::GUI::SysManager object.

=over

=item Synopsis

$output = $GuiObj->new(%args);;

=item Arguments

new function accepts all arguments.

=item Returns

(Scalar)
GUI object

=back

=head2 parse_system_details

This function parses input parameter and creates hash with System IP, System Name, Username and Password.

=over

=item Synopsis

%ouput = $GuiObj->parse_system_details($host_detail_string);

=item Arguments

accepts input string containing the detail of a host.

=item Returns

(Hash)
A hash containing System IP, System Name, Username and Password as keys.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (sindhe@netapp.com, dl-nacl-dev@netapp.com)

=back

=cut

