#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets related to CMode GUI operations
## @author sindhe@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::GUI::SysManager::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use Tharn qw( host);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#use Module Imports
use NACL::APISet::Session::GUI::SysManager::CMode;
use NACL::APISet::Response::GUI;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use Params::Validate qw(validate_pos SCALAR );

my $nate_params = NATE::ParamSet->new(global => 1);

# Import Exception class
use NATE::BaseException qw(:try);

# Base class
use base 'NACL::APISet::Host::GUI::SysManager';

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Sysmanager Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ($self, %args) = @_;
    my $SystemDetails;
    my %MainSystem = ();
    my %SystemNodes= ();

    $Log->enter() if $may_enter;

    # Read all the parameters from Tharnhost
    my $hostobj = $self->{hostobj};

    my ( $hostname, $result, $cluster_mgmt_ip );

    # Set argument hash with the current System details
    # ex: Ip,username , password etc.
    # Get hostname and cluster managment ip form CLI connection

    my $cliApisetObj = NACL::APISet->new(
            hostobj   => $hostobj,
            category  => "Node",
            interface => "CLI",
            set       => "CMode",
    );

    $Log->comment("Getting the cluster name of the Node");
    $result     = $cliApisetObj->hostname();
    ($hostname) = $result->get_raw_output() =~ /(\S+)/;
    $Log->comment("Getting the cluster-management-ip to open SM");
    $result     = $cliApisetObj->network_interface_show(
                                     vserver => $hostname,
                                     role    => "cluster-mgmt" );

    my @parsed_output = @{ $result->get_parsed_output() };
    $cluster_mgmt_ip  = $parsed_output[0]->{'address'};

    $args{remote_server_addr} //= $nate_params->get('SMSERVERIP');

    unless ( $args{remote_server_addr} ) {
        if ( $nate_params->get('WINCLIENT') ) {
            my $c_hostobj = host($nate_params->get('WINCLIENT'));
            $args{remote_server_addr} = $c_hostobj->{'default_ip'};
        } elsif ( $nate_params->get('CLIENT') ) {
            my $c_hostobj = host($nate_params->get('CLIENT'));
            $args{remote_server_addr} = $c_hostobj->{'default_ip'};
        } else {
            $args{remote_server_addr} = $nate_params->get('SMSERVERIP') ||
                NACL::APISet::Exceptions::MissingRequiredParamException->throw(
                "Param 'WINCLIENT' or 'CLIENT' or SMSERVERIP'"
              . " is required to use GUI interface" );
        }
    }

    $args{browser_url} //= $nate_params->get('SMURL');
    $args{port} //= $nate_params->get('SELPORT') || "4444";
    $args{browser_name} //= $nate_params->get('BROWSER')
        ||  $nate_params->get("SM_BROWSER") || "firefox";
    $args{vserver} //= $nate_params->get('VSERVER')
          || $nate_params->get("DEFAULT_VSR");

    if ( exists($args{selenium_port}) and $args{selenium_port} ) {
        $args{port} = $args{selenium_port};
    }

    my $session_pkg =
        "NACL::APISet::Session::GUI::SysManager::CMode";

    #Create hash with the cluster information
    $MainSystem{'systemIp'}       = $args{ipaddress} || $cluster_mgmt_ip ;
    $MainSystem{'systemName'}     = $hostname || $hostobj->{hostname};
    $MainSystem{'systemUsername'} = $args{username}
                                    || $hostobj->{default_username};
    $MainSystem{'systemPassword'} = $args{password}
                                    || $hostobj->{default_password};

    unless ( defined($args{'browser_url'})
             and $args{'browser_url'} =~ /http/ ) {
        $args{browser_url} = "https://".$MainSystem{'systemIp'};
    }

    $SystemDetails = "$MainSystem{'systemIp'}|"
                   . "$MainSystem{'systemName'}|"
                   . "$MainSystem{'systemUsername'}|"
                   . "$MainSystem{'systemPassword'}";

    # Adding cluster details to $SystemNodes Hash.
    $SystemNodes{uc($PARAM_CLUSTERDETAIL)} = $SystemDetails;

    $args{'SystemNodes'} = \%SystemNodes;
    $args{'MainSystem'}  = \%MainSystem;

    # Create a Session object. The Session object automatically takes care of
    # connecting to the service and preparing the service for use
    delete $args{context_definition};
    delete $args{prompt_password};
    delete $args{prompt_username};

    try {
        $self->{session} = $session_pkg->new(%args);
    }
    otherwise {
        my $exception = shift;
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ConnectionFailedException->throw(
            "Failed to Create Selenium Session object : " . $exception->text());
    }

    $Log->exit() if $may_exit;
} ## end sub _init_Session

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::GUI::SysManager::CMode - Base for CMode API Sets for GUI operations

=head1 SYNOPSIS

use NACL::APISet::Host::GUI::SysManager::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Host::GUI::SysManager::CMode> is used Initializes the API Set and make the API Set ready for work.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (neelkant@netapp.com, dl-nacl-dev@netapp.com)

=back

 $Id$
 Last revised $Date: 2014/01/21 $ by $Author: neelkant $

=cut

