# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Aggregate GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode Aggregate related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::7Mode::SmAggr;
##
##      create_aggr(%args);
##      aggr_edit(%args);
##      aggr_adddisk(%args);
##      aggr_status(%args);
##      aggr_mirror(%args);
##      aggr_delete(%args);
##      get_aggr_info(%args);
##      refresh_aggr(%args);
##      aggr_plexes(%args);
##
## @status public
##
## @author Afsar Syed. afsars@netapp.com
## @change
##  4/26/2010 : Afsar Syed: Initial version
## @change
##  3/20/2012 : Vikas : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmAggr;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    create_aggr
    aggr_edit
    aggr_adddisk
    aggr_status
    aggr_mirror
    aggr_delete
    get_aggr_info
    refresh_aggr
    aggr_plexes
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::create_aggr
## @summary
##     Creation of Aggregate using GUI.
## @description
##     This method can be used to create aggregate using GUI.
## @synopsis
##     create_aggr(%args);
## INPUTS TO METHOD CALL 'create_aggr' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be created </td> </tr>
## <tr> <td> -disktype </td> <td> (Required)Type of Disk to be used </td> </tr>
## <tr> <td> -diskcount </td> <td> (Required)No of the Disks to be used </td> </tr>
## <tr> <td> -raidlevel </td> <td> Raid level to be used for creating the aggregate</td> </tr>
## <tr> <td> -blocktype </td> <td> Block type to be used for creating the aggregate </td> </tr>
## <tr> <td> -synmirror</td> <td> Check if sync mirror has to enabled </td> </tr>
## <tr> <td> -snaplock</td> <td> Check if snaplock has to enabled </td> </tr>
## <tr> <td> -nodename </td> <td> Name of the node to be used </td> </tr>
## <tr> <td> -diskcount </td> <td> Disk count value for respective disk type </td> </tr>
## <tr> <td> -zerroing </td> <td> Enable Zerroing for the Aggregate </td> </tr>
## <tr> <td> -flashpool </td> <td> Enable Flash pool </td> </tr>
## <tr> <td> -change_hdd_count </td> <td> Change the HDD Count </td> </tr>
## <tr> <td> -change_raid_size </td> <td> raid size to be changed to </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.
sub create_aggr {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-aggrname"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_loading();
    $self->wait_for_element_present("$AGGR_CREATE_BUTTON");

    $Log->comment("Selecting create aggr button");
    $self->click("$AGGR_CREATE_BUTTON");
    
    $self->wait_for_loading();
    $self->wait_for_element_present("$AGGR_CREATE_NEXT_BUTTON");
    $Log->comment("Selecting create aggr next button");
    $self->click("$AGGR_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Enter aggr name: $args{'-aggrname'}");
    $self->type_keys( "$AGGR_CREATE_NAME_INPUT", $args{'-aggrname'} );
    $self->type( "$AGGR_CREATE_NAME_INPUT", $args{'-aggrname'} );

    if ( exists( $args{-raidtype} ) ) {
        if($args{-raidtype} =~ /[RAID-DP|RAID4|RAID0]/i){
            $Log->comment("Selecting raid type");
            $self->set_combobox_value($AGGR_CREATE_RAID_TYPE_COMBOBOX, uc($args{-raidtype}));
        } else {
            NATE::BaseException->throw(
                "Invalid value for raid type, valid values are 'RAID-DP' or 'RAID4' or 'RAID0'");
        }
    } ## end if ( exists( $args{-raidtype...}))
    else {
        NATE::BaseException->throw(
            "Invalid value for raid type, valid values are 'raid0' or 'dual' or 'raid4'"
        );
    }

    if(exists($args{-flashpool})) {
        $Log->comment("Enabling flash pool");
        $self->click($AGGR_CREATE_SSD_CHECKBOX);
    }

    $Log->comment("Clicking next button in aggr wizard");
    $self->click($AGGR_CREATE_NEXT_BUTTON);
    $self->wait_for_loading();

    if(exists($args{-change_hdd_count})) {
        $Log->comment("Clicking on change button for disk count");
        $self->click($AGGR_CREATE_CHANGE_DISK_SEL);

        $Log->comment("Selecting disk group");
        $self->click_row_with_texts($AGGR_CREATE_CHANGE_DISK_DIALOG, $args{-node}, $args{-disktype}, $args{-checksum});

        $Log->comment("Entering HDD disk count for selected disk group");
        $self->type($AGGR_CREATE_CHANGE_DISK_DIALOG_HDD_COUNT, $args{-change_hdd_count});

        if ($args{-change_ssd_count}) {
            $Log->comment("Entering SSD disk count for selected disk group");
            $self->type($AGGR_CREATE_CHANGE_DISK_DIALOG_SSD_COUNT, $args{-change_ssd_count});
        }

        $Log->comment("Clicking on save close button for disk dialog");
        $self->click($AGGR_CREATE_CHANGE_DISK_DIALOG_SAVE_BUTTON);
        $self->wait_for_loading();
    }

    if(exists($args{-change_raid_size})) {
        $Log->comment("Clicking on change button for raid");
        $self->click($AGGR_CREATE_CHANGE_RAID_SEL);

        $Log->comment("Modifying raid group size");
        $self->type($AGGR_CREATE_CHANGE_RAID_SEL_DIALOG_RAID_SIZE_INPUT, $args{-change_raid_size});
        $self->fire_event($AGGR_CREATE_CHANGE_RAID_SEL_DIALOG_RAID_SIZE_INPUT, "blur");

        $Log->comment("Clicking on save close button for disk dialog");
        $self->click($AGGR_CREATE_CHANGE_RAID_DIALOG_SAVE_BUTTON);
    }

    if(exists($args{-change_raid_type})) {
        $Log->comment("Clicking on change button for raid");
        $self->click($AGGR_CREATE_CHANGE_RAID_SEL);

        $Log->comment("Modifying raid type");
        $self->set_combobox_value($AGGR_CREATE_CHANGE_RAID_SEL_DIALOG_RAID_TYPE, $args{-change_raid_type});

        $Log->comment("Clicking on save close button for disk dialog");
        $self->click($AGGR_CREATE_CHANGE_RAID_DIALOG_SAVE_BUTTON);
    }
    
    $Log->comment("Clicking on create button");
    $self->click($AGGR_CREATE_NEXT_BUTTON);
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Waiting for finish button");
    $self->wait_for_element_present($AGGR_CREATE_NEXT_BUTTON, $GENERIC_WAIT);

    if ((lc($args{-zerroing})) eq 'wait' && (lc($args{-aggr_type})) eq "hya") {
        my $state = $self->get_text("$AGGR_CREATE_FINISH_LABEL");

        $Log->comment("*** Script will wait until disks disk zeroing is complete ***");
        while($state !~ /Successful/) {
            sleep(1);
            $state = $self->get_text("$AGGR_CREATE_FINISH_LABEL");
        }

        $Log->comment("Enter finish for Aggr create");
        $self->click("$AGGR_CREATE_NEXT_BUTTON");
    } else {
        $Log->comment("Clicking on finish button");
        $self->click_nowait($AGGR_CREATE_NEXT_BUTTON);
        
        if($self->is_element_present($AGGR_CREATE_NO_WAIT_OK_BUTTON)){
            $Log->comment("Clicking on OK button to zero disks in background");
            $self->click_nowait($AGGR_CREATE_NO_WAIT_OK_BUTTON);
        }
    }
    
    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );
    $self->wait_for_loading();
    $Log->comment("Create aggr is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub create_aggr

################################################################################
## @name ::aggr_edit
## @summary
##     Edit Aggregate using GUI.
## @description
##     This method can be used to edit the aggregate raid value and size using GUI.
## @synopsis
##     aggr_edit(%args);
## INPUTS TO METHOD CALL 'aggr_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be edited </td> </tr>
## <tr> <td> -renameing </td> <td> Changing the name of the Aggregate </td> </tr>
## <tr> <td> -raidlevel </td> <td> Raid level to be used for creating the aggregate</td> </tr>
## <tr> <td> -resize </td> <td> Changing the size of the Aggregate </td> </tr>

## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub aggr_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-aggrname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_GRID");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );

    $Log->comment("Selecting edit button to change settings");
    $self->click("$AGGR_EDIT_BUTTON");
    $self->wait_for_loading();

    if ( exists( $args{-renameing} ) ) {
        $Log->comment("Changing the name of the aggr");
        $self->type( "$AGGR_EDIT_AGGRNAME_INPUT", $args{-renameing} );
    }

    if(exists($args{-flashpool})) {
        $Log->comment("Select Enable flash pool");
        $self->click($AGGR_EDIT_ENABLE_FLASH_POOL);
    }

    if(exists($args{-ssdcount})) {
        $Log->comment("Entering SSD disk count");
        $self->type( "$AGGR_EDIT_FLASH_POOL_COUNT", "");
        $self->type_keys( "$AGGR_EDIT_FLASH_POOL_COUNT", $args{-ssdcount} );
        $self->type( "$AGGR_EDIT_FLASH_POOL_COUNT", $args{-ssdcount} );
        $self->fire_event( "$AGGR_EDIT_FLASH_POOL_COUNT", "blur");
    }

    if(exists($args{-hdd_count})) {
        $Log->comment("Entering HDD disk count");
        $self->type_keys( "$AGGR_EDIT_HDD_COUNT", $args{-hdd_count} );
        $self->type( "$AGGR_EDIT_HDD_COUNT", $args{-hdd_count} );
        $self->fire_event( "$AGGR_EDIT_HDD_COUNT", "blur");
        $Log->comment("Clicking advanced button for add disks");
        $self->click($AGGR_EDIT_ADVANCED_BUTTON);

        $Log->comment("Selecting disk group");
        $self->click_row_with_texts($AGGR_EDIT_ADVANCED_DISK_GRID, $args{-node},$args{-disktype},$args{-checksum} );

        $Log->comment("Entering HDD disk count in advanced disk grid");
        $self->type_keys( "$AGGR_EDIT_ADVANCED_DISK_COUNT", $args{-hdd_count} );
        $self->type( "$AGGR_EDIT_ADVANCED_DISK_COUNT", $args{-hdd_count} );
        $self->fire_event( "$AGGR_EDIT_ADVANCED_DISK_COUNT", "blur" );

        if(exists($args{-raidgroup})) {
            if(lc($args{-raidgroup}) eq "all") {
                $Log->comment("Selecting Raid groups all");
                $self->click($AGGR_EDIT_ALL_RAID_GROUPS_RADIO_BUTTON);
            } elsif(lc($args{-raidgroup}) eq "specific") {
                $Log->comment("Selecting specific Raid group");
                $self->click($AGGR_EDIT_SPECIFIC_RAID_GROUP_RADIO_BUTTON);

                if(exists($args{-specificRaidGroup})) {
                    $Log->comment("Selecting specific raid group");
                    $self->set_combobox_value($AGGR_EDIT_SPECIFIC_RAID_GROUP_COMBOBOX, $args{-specificRaidGroup});
                }
            } elsif(lc($args{-raidgroup}) eq "new") {
                $Log->comment("Selecting New Raid group");
                $self->click($AGGR_EDIT_NEW_RAID_GROUP_RADIO_BUTTON);
            } else {
                NATE::BaseException->throw(
                    "Invalid value for raid type, valid values are 'all' or 'specific' or 'new'");
            }
        }
        $Log->comment("Clicking Save and Close button");
        $self->click($AGGR_EDIT_ADVANCED_SAVE_CLOSE_BUTTON);
    }
    
    if(exists($args{-raidtype})) {
        $Log->comment("Clicking change button for raid details");
        $self->click($AGGR_EDIT_RAID_CHANGE_BUTTON);

        $Log->comment("Modifying raid type");
        $self->set_combobox_value($AGGR_EDIT_RAID_TYPE_COMBOBOX, $args{-raidtype});

        $Log->comment("Clicking Save and Close button for raid change dialog");
        $self->click($AGGR_EDIT_RAID_SAVE_CLOSE_BUTTON);
    }

    if(exists($args{-raidsize})) {
        $Log->comment("Clicking change button for raid details");
        $self->click($AGGR_EDIT_RAID_CHANGE_BUTTON);

        $Log->comment("Modifying raid group size");
        $self->type($AGGR_EDIT_RAID_SIZE_MODIFY, $args{-raidsize});
        $self->fire_event($AGGR_EDIT_RAID_SIZE_MODIFY, "blur");

        $Log->comment("Clicking Save and Close button for raid change dialog");
        $self->click($AGGR_EDIT_RAID_SAVE_CLOSE_BUTTON);
    }
    
    $Log->comment("Clicking Save and Close button for edit aggr dialog");
    $self->click($AGGR_EDIT_SAVE_CLOSE_BUTTON);
    $self->check_for_error();
     
    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("Edit for aggr is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub aggr_edit

################################################################################
## @name ::aggr_status
## @summary
##     Change status of the aggregate using GUI.
## @description
##     This method can be used to change status of the aggregate using GUI
## @synopsis
##     aggr_status(%args);
## INPUTS TO METHOD CALL 'aggr_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate whose status has to be changed</td> </tr>
## <tr> <td> -status </td> <td> Changing the status of the Aggregate </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub aggr_status {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-aggrname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_GRID");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );

    if ( exists( $args{-status} ) ) {
        $Log->comment("Selecting status button to change settings");
        $self->click("$AGGR_STATUS_BUTTON");
        if ( lc( $args{-status} ) eq 'online' ) {
            $Log->comment(
                "Selecting Online status button to change settings");
            $self->click("$AGGR_STATUS_ONLINE_BUTTON");
            $Log->comment("Selecting conform status button ");

            $self->wait_for_loading();
            $self->wait_for_element_not_present(
                $AGGR_STATUS_ONLINE_OK_BUTTON, $GENERIC_WAIT );

        } elsif ( lc( $args{-status} ) eq 'offline' ) {
            $Log->comment(
                "Selecting Offline status button to change settings");
            $self->click("$AGGR_STATUS_OFFLINE_BUTTON");

            $Log->comment("Selecting conform status button ");
            $self->wait_for_element_present("$AGGR_STATUS_OFFLINE_OK_BUTTON");
            $self->click("$AGGR_STATUS_OFFLINE_OK_BUTTON");

            $self->wait_for_loading();
            $self->wait_for_element_not_present(
                $AGGR_STATUS_OFFLINE_OK_BUTTON,
                $GENERIC_WAIT );
        } elsif ( lc( $args{-status} ) eq 'restrict' ) {
            $Log->comment(
                "Selecting restrict status button to change settings");
            $self->click("$AGGR_STATUS_RESTRICT_BUTTON");

            $Log->comment("Selecting conform status button ");
            $self->wait_for_element_present(
                "$AGGR_STATUS_RESTRICT_OK_BUTTON");

            $self->click("$AGGR_STATUS_RESTRICT_OK_BUTTON");
            $self->wait_for_loading();
            $self->wait_for_element_not_present(
                $AGGR_STATUS_RESTRICT_OK_BUTTON,
                $GENERIC_WAIT );
        } ## end elsif ( lc( $args{-status...}))
    } ## end if ( exists( $args{-status...}))

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $self->check_for_error();
    $Log->comment("Status change for aggr is successfull");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub aggr_status

################################################################################
## @name ::aggr_mirror
## @summary
##     Create mirror Aggregate using GUI
## @description
##     This method can be used to mirror an aggregate using GUI.
## @synopsis
##     aggr_mirror(%args);
## INPUTS TO METHOD CALL 'aggr_mirror' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate for mirroring  </td> </tr>
## <tr> <td> -aggrdisk</td> <td> Selecting one disk from Aggregate disk grid </td> </tr>
## <tr> <td> -mirrordisk</td> <td> Selecting one disk from mirror disk grid </td> </tr>
## <tr> <td> -autoselect</td> <td> Selecting auto disk selection button to change settings </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub aggr_mirror {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-aggrname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_GRID");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );

    $Log->comment("Selecting mirror button to change settings");
    $self->click("$AGGR_MIRROR_BUTTON");

    $self->wait_for_loading();
    $self->wait_for_element_present("$AGGR_MIRROR_AGGRDISK_GRID");

    if ( exists( $args{-aggrdisk} ) ) {
        foreach my $disk ( @{ $args{-aggrdisk} } ) {
            my $temp = "enable";
            $temp = "disable" if ( $disk =~ /-/ );
            $Log->comment("Selecting one disk from Aggr disk grid : $disk");
            $self->set_checkbox_in_row_with_text(
                "$AGGR_MIRROR_AGGRDISK_GRID", $temp, ("$disk") );
        } ## end foreach my $disk ( @{ $args...})
    } ## end if ( exists( $args{-aggrdisk...}))

    if ( exists( $args{-mirrordisk} ) ) {
        foreach my $disk ( @{ $args{-mirrordisk} } ) {
            my $temp = "enable";
            $temp = "disable" if ( $disk =~ /-/ );
            $Log->comment("Selecting one disk from mirror disk grid : $disk");
            $self->set_checkbox_in_row_with_text(
                "$AGGR_MIRROR_MIRROREDDISK_GRID",
                $temp, ("$disk") );
        } ## end foreach my $disk ( @{ $args...})
    } ## end if ( exists( $args{-mirrordisk...}))

    if ( exists( $args{-autoselect} ) ) {
        if ( lc( $args{-autoselect} ) eq 'yes' ) {
            $Log->comment(
                "Selecting auto disk selection button to change settings");
            $self->click("$AGGR_MIRROR_AUTOSELECT_BUTTON");
        }
    } ## end if ( exists( $args{-autoselect...}))

    $Log->comment("Selecting ok button for mirroring");
    $self->click("$AGGR_MIRROR_OK_BUTTON");

    $self->check_for_error();
    $self->wait_for_element_not_present( $AGGR_MIRROR_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("Mirroring  for aggr is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub aggr_mirror

################################################################################
## @name ::aggr_delete
## @summary
##     Deleting the aggregate using GUI.
## @description
##     This method can be used to delete the aggregate using GUI.
## @synopsis
##     aggr_delete(%args);
## INPUTS TO METHOD CALL 'aggr_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be deleted  </td> </tr>
## <tr> <td> -offline</td> <td> Selecting Offline status button to change settings </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub aggr_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-aggrname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_CREATE_BUTTON");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );
    $self->wait_for_loading();

    if ( exists( $args{-offline} ) ) {
        if ( lc( $args{-offline} ) eq 'yes' ) {
            $Log->comment("Selecting status button to change settings");
            $self->click("$AGGR_STATUS_BUTTON");

            $Log->comment(
                "Selecting Offline status button to change settings");
            $self->click("$AGGR_STATUS_OFFLINE_BUTTON");

            $Log->comment("Selecting conform status button ");
            $self->wait_for_element_present("$AGGR_STATUS_OFFLINE_OK_BUTTON");
            $self->click("$AGGR_STATUS_OFFLINE_OK_BUTTON");

            $self->wait_for_loading();
            $self->wait_for_element_not_present(
                $AGGR_STATUS_OFFLINE_OK_BUTTON,
                $GENERIC_WAIT );
        } ## end if ( lc( $args{-offline...}))
        $Log->comment("Selecting one Aggr from the grid");
        $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );
        $self->wait_for_loading();
    } ## end if ( exists( $args{-offline...}))

    $Log->comment("Selecting delete aggr button");
    $self->click("$AGGR_DELETE_BUTTON");

    $Log->comment("Selecting delete aggr conform checkbox");
    $self->set_checkbox( "$AGGR_DELETE_CONF_CHECKBOX", "enable" );
    $self->wait_for_element_present("$AGGR_DELETE_CONF_CHECKBOX");

    $self->click("$AGGR_DELETE_OK_BUTTON");

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $self->check_for_error();
    $Log->comment("Aggregate deletion is successfull");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub aggr_delete

################################################################################
## @name ::get_aggr_info
## @summary
##    Get information about the aggregate using GUI.
## @description
##     This method can be used to get information about the aggregate using GUI.
## @synopsis
##     get_aggr_info(%args);
## INPUTS TO METHOD CALL 'get_aggr_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate whose information is required </td> </tr>
## </table>
## @return
##     Returns an array reference which contains the Aggregate status details.

sub get_aggr_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-aggrname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    my ( %values, $aggr_info );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY}, $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_GRID");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );

    $Log->comment("Navigating to Storage>Aggregate details tab");
    $self->click("$AGGR_INFO_DETAILS_TAB");

    $values{$AGGR_INFO_DETAILS_NAME_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_NAME_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_NAME_LABEL") );
    $values{$AGGR_INFO_DETAILS_STATUS_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_STATUS_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_STATUS_LABEL") );
    $values{$AGGR_INFO_DETAILS_RAIDTYPE_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_RAIDTYPE_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_RAIDTYPE_LABEL") );
    $values{$AGGR_INFO_DETAILS_TYPE_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_TYPE_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_TYPE_LABEL") );
    $values{$AGGR_INFO_DETAILS_ROOT_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_ROOT_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_ROOT_LABEL") );
    $values{$AGGR_INFO_DETAILS_FILES_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_FILES_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_FILES_LABEL") );
    $values{$AGGR_INFO_DETAILS_MAXFILES_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_MAXFILES_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_MAXFILES_LABEL") );
    $values{$AGGR_INFO_DETAILS_CHECKSUM_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_CHECKSUM_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_CHECKSUM_LABEL") );
    $values{$AGGR_INFO_DETAILS_64BIT_KEY}
        = $self->get_text("$AGGR_INFO_DETAILS_64BIT_LABEL")
        if ( $self->is_element_present("$AGGR_INFO_DETAILS_64BIT_LABEL") );

    $Log->comment("Info collection for aggr is successfull");
    $aggr_info->{parsed_output} = \%values;
    $Log->exit() if $may_exit;

    return $aggr_info;

} ## end sub get_aggr_info

################################################################################
## @name ::refresh_aggr
## @summary
##    Refresh the aggregate using GUI.
## @description
##     This method can be used to refresh the aggregate using GUI.
## @synopsis
##     refresh_aggr(%args);
## INPUTS TO METHOD CALL 'refresh_aggr' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## </table>
## @return
##     Returns an array reference which contains the Loading time.

sub refresh_aggr {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present( $AGGR_REFRESH_BUTTON, $GENERIC_WAIT );

    if ( not $self->is_button_enabled($AGGR_REFRESH_BUTTON) ) {
        $self->wait_for_loading();
    }

    $self->click_button("$AGGR_REFRESH_BUTTON");
    $self->wait_for_loading();
    $Log->comment("Loading time and commit time are $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub refresh_aggr

################################################################################
## @name ::aggr_plexes
## @summary
##    Perform operation on the plexes of the aggregate using GUI.
## @description
##     This method can be used to perform operation on the plexes of aggregate using GUI.
## @synopsis
##     aggr_plexes(%args);
## INPUTS TO METHOD CALL 'aggr_plexes' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate on which plex operations to be done </td> </tr>
## <tr> <td> -plexname </td> <td> (Required)Plex Name on which operations have to be performed </td> </tr>
## <tr> <td> -status </td> <td> Selecting status button to change settings </td> </tr>
## <tr> <td> -splitplex </td> <td> Selecting split button to change settings </td> </tr>
## <tr> <td> -destroy </td> <td> Selecting destroy button to change settings </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub aggr_plexes {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-aggrname' => { type => SCALAR },
            "-plexname" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>Aggregate");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_element_present("$AGGR_GRID");

    $Log->comment("Selecting one Aggr from the grid");
    $self->click_row_with_texts( "$AGGR_GRID", $args{'-aggrname'} );
    $self->wait_for_loading();
    $self->wait_for_element_present("$AGGR_INFO_PLEXES_TAB");

    $Log->comment("Selecting plexes tab for the selected aggr");
    $self->click("$AGGR_INFO_PLEXES_TAB");
    $self->wait_for_loading();
    $self->wait_for_element_present("$AGGR_INFO_PLEXES_GRID");

    $Log->comment("Selecting one plex from the grid");
    $self->click_row_with_texts( "$AGGR_INFO_PLEXES_GRID",
        $args{'-plexname'} );

    if ( lc( $args{-status} ) eq 'online' ) {
        $Log->comment("Selecting Online status button to change settings");
        $self->click("$AGGR_INFO_PLEXES_ONLINE_BUTTON");
    } elsif ( lc( $args{-status} ) eq 'offline' ) {
        $Log->comment("Selecting Offline status button to change settings");
        $self->click("$AGGR_INFO_PLEXES_OFFLINE_BUTTON");

        $Log->comment("Selecting conform status button ");
        $self->wait_for_element_present(
            "$AGGR_INFO_PLEXES_OFFLINE_CONF_BUTTON");
        $self->click("$AGGR_INFO_PLEXES_OFFLINE_CONF_BUTTON");
    } ## end elsif ( lc( $args{-status...}))

    if ( lc( $args{-splitplex} ) eq 'yes' ) {
        $Log->comment("Selecting split button to change settings");
        $self->click("$AGGR_INFO_PLEXES_SPLIT_BUTTON");

        $Log->comment("Selecting conform status button ");
        $self->wait_for_element_present(
            "$AGGR_INFO_PLEXES_SPLIT_CONF_BUTTON");

        $self->click("$AGGR_INFO_PLEXES_SPLIT_CONF_BUTTON");
        $self->wait_for_element_not_present(
            $AGGR_INFO_PLEXES_SPLIT_CONF_BUTTON,
            $GENERIC_WAIT );

        if ( $self->is_element_present("$AGGR_INFO_PLEXES_SPLIT_NAME_INPUT") )
        {
            $Log->comment("Enter the name for new Plex :$args{-newplex}");
            $self->type( "$AGGR_INFO_PLEXES_SPLIT_NAME_INPUT",
                $args{-newplex} )
                if ( exists( $args{-newplex} ) );
            $self->click("$AGGR_INFO_PLEXES_SPLIT_OK_BUTTON");
        } ## end if ( $self->is_element_present...)
    } ## end if ( lc( $args{-splitplex...}))

    if ( lc( $args{-destroy} ) eq 'yes' ) {
        $Log->comment("Selecting destroy button to change settings");
        $self->click("$AGGR_INFO_PLEXES_DESTROY_BUTTON");

        $Log->comment("Selecting conform status button ");
        $self->wait_for_element_present(
            "$AGGR_INFO_PLEXES_DESTROY_CONF_BUTTON");
        $self->click("$AGGR_INFO_PLEXES_DESTROY_CONF_BUTTON");
    } ## end if ( lc( $args{-destroy...}))

    $self->check_for_error();
    $Log->comment("plexes operation on aggr is successfull");

    $Log->comment("Loading time and commit time are $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub aggr_plexes

################################################################################
## @name ::aggr_adddisk
## @summary
##     Edit Aggregate using GUI.
## @description
##     This method can be used to edit the raidsize and raidgroup
##     of the aggregate when a new disk is added using GUI.
## @synopsis
##     aggr_adddisk(%args);
## INPUTS TO METHOD CALL 'aggr_adddisk' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be edited </td> </tr>
## <tr> <td> -raidsize </td> <td> Entering the raidsize value of the Aggregate </td> </tr>
## <tr> <td> -raidgroup </td> <td>  Entering the raidgroup value of the aggregate</td> </tr>
## <tr> <td> -diskcount_pool0 </td> <td> Disk count value for respective disk type </td> </tr>
## <tr> <td> -diskcount_pool1 </td> <td> Disk count value for respective disk type </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.
#
#sub aggr_adddisk {
#
#    my $self = shift;
#    my %args = validate_with(
#            params      => \@_,
#            spec        => { "-aggrname" => { type => SCALAR }, },
#            allow_extra => 1
#            );
#
#    $Log->enter() if $may_enter;
#    $self->clear_window();
#
#    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
#
#    $Log->comment("Navigating to Storage>Aggregate");
#    my $loading_time
#        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
#                $AGGR_TREE );
#    $self->wait_for_element_present("$AGGR_GRID");
#
#    $Log->comment("Selecting one Aggr from the grid");
#    $self->click_row_with_texts( "$AGGR_GRID", "1", "0", $args{'-aggrname'} );
#
#    $Log->comment("Selecting Add disk button to change settings");
#    $self->click("$AGGR_ADDISK_BUTTON");
##$self->wait_for_element_present("$AGGR_ADDISK_TEXT_FIELD");
#    $self->wait_for_loading();
#
#    $Log->comment("Expand Advanced option");
#    $self->click("$AGGR_ADDISK_ADVANCED_OPTION");
#
#    if ( exists( $args{-raidsize} ) ) {
#        $Log->comment("Entering raid size value : $args{-raidsize}");
#        $self->type( "$AGGR_ADDISK_RAID_SIZE", $args{-raidsize} );
#    }
#
#    if ( exists( $args{-raidgroup} ) ) {
#        $Log->comment("Selecting raid group");
#        if ( lc( $args{-raidgroup} ) eq 'specific' ) {
#            $Log->comment("Changing raid group to specific");
#            $self->click("$AGGR_ADDISK_SELECT_SPECIFIC");
#            if ( exists( $args{-raidspecificgrp} ) ) {
#                $Log->comment("Selecting specific raid group from combo box");
#                $self->set_combobox_value(
#                        "$AGGR_ADDISK_SELECT_SPECIFIC_RAID_GROUP",
#                        $args{-raidspecificgrp} );
#            } ## end if ( exists( $args{-raidspecificgrp...}))
#        } elsif ( lc( $args{-raidgroup} ) eq 'all' ) {
#            $Log->comment("Changing raid group to all");
#            $self->click("$AGGR_ADDISK_SELECT_ALL");
#        }
#    } ## end if ( exists( $args{-raidgroup...}))
#
#    $Log->comment("Clicking Select disks");
#    $self->click("$AGGR_ADDISK_SELECT_BUTTON");
#
#    sleep(10);
#
#    if ( exists( $args{-diskcount_pool0} ) ) {
#        foreach my $disk ( @{ $args{-diskcount_pool0} } ) {
#            $Log->comment(
#                    "Entering disk count value for the respective disk type");
#            my @disk_values = split( /:/, $disk );
#            $self->type_in_row_with_text( "$AGGR_ADDISK_SELECT_TABLE_POOL0",
#                    $disk_values[1], $disk_values[0] );
#        } ## end foreach my $disk ( @{ $args...})
#    } ## end if ( exists( $args{-diskcount_pool0...}))
#
#    if ( exists( $args{-diskcount_pool1} ) ) {
#        foreach my $disk1 ( @{ $args{-diskcount_pool1} } ) {
#            $Log->comment(
#                    "Entering disk count value for the respective disk type");
#            my @disk_values1 = split( /:/, $disk1 );
#            $self->type_in_row_with_text( "$AGGR_ADDISK_SELECT_TABLE_POOL1",
#                    $disk_values1[1], $disk_values1[0] );
#        } ## end foreach my $disk1 ( @{ $args...})
#    } ## end if ( exists( $args{-diskcount_pool1...}))
#
#    $Log->comment("Clicking Ok button in select disk dialog");
#    $self->click("$AGGR_ADDISK_SELECT_INPUT_OK_BUTTON");
#    if ( $self->is_button_enabled("$AGGR_ADDISK_OK_BUTTON") ) {
#        $Log->comment("Clicking Add button in add disk dialog");
#        $self->click("$AGGR_ADDISK_OK_BUTTON");
#        $self->check_for_error();
#        $self->wait_for_element_not_present( $AGGR_ADDISK_OK_BUTTON,
#                $GENERIC_WAIT );
#    } ## end if ( $self->is_button_enabled...)
#
#    $Log->comment("Loading time and commit time are $loading_time sec!");
#    my %times = ( loading_time => $loading_time );
#
#    $Log->comment("Add disk for aggr is successfull");
#    $Log->exit() if $may_exit;
#
#    return \%times;
#
#} ## end sub aggr_adddisk

1;
