# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Dedupe Policy GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode Aggregate related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::7Mode::SmDedupePolicy;
##
##      dedupe_policy_add(%args);
##		dedupe_policy_edit(%args);
##		dedupe_policy_delete(%args);
##		dedupe_policy_status(%args);
##		dedupe_policy_refresh(%args);
##      get_dedupe_policy_info(%args);
##
##
## @status public
##
## @author afsars@netapp.com
## @change
##  6/7/2011 : afsars : Initial version
## @change
##  2012-03-20 : vikasp : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmDedupePolicies;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    dedupe_policy_add
    dedupe_policy_edit
    dedupe_policy_delete
    dedupe_policy_status
    dedupe_policy_refresh
    get_dedupe_policy_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::dedupe_policy_add
## @summary
##     Creation of dedupe policy using GUI.
## @description
##     This method can be used to create dedupe policy using GUI.
## @synopsis
##     dedupe_policy_add(%args);
## INPUTS TO METHOD CALL 'dedupe_policy_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -maxruntime </td> <td> Selecting maxruntime </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub dedupe_policy_add {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-policyName' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $DEDUPLICATION_POLICIES_TREE );

    $Log->comment("Clicking dedupe policy add button");
    $self->click($DEDUPE_POLICY_ADD);
    $self->wait_for_loading();

    $Log->comment("Waiting for dedupe policy add Dialog");
    $self->wait_for_element_present( $DEDUPE_POLICY_NAME_TEXTBOX,
        $GENERIC_WAIT );

    $Log->comment("Entering policy name: $args{'-policyName'}");
    $self->type( $DEDUPE_POLICY_NAME_TEXTBOX, $args{'-policyName'} );
    $self->type_keys( $DEDUPE_POLICY_NAME_TEXTBOX, $args{'-policyName'} );
    $self->fire_event( "$DEDUPE_POLICY_NAME_TEXTBOX", "blur" );

    #Selecting schdeule
    if ( exists( $args{-schedule} ) ) {
        $Log->comment("select schedule : $args{-schedule}");
        $self->set_combobox_value( $DEDUPE_POLICY_ADD_SCHEDULE,
            $args{-schedule} );
    }

    #Entering value for maximum runtime
    if ( exists( $args{-maxruntime} ) ) {
        $Log->comment("select schedule : $args{-maxruntime}");
        $self->type_keys( $DEDUPE_POLICY_ADD_MAXRUNTIME, $args{-maxruntime} );
    }

    sleep 5;
    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($DEDUPE_POLICY_ADD_OK_BUTTON);
    $self->wait_for_loading();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("dedupe policy created successfully");

    $Log->exit() if $may_exit;
    return \%times;
} ## end sub dedupe_policy_add

################################################################################
## @name ::dedupe_policy_edit
## @summary
##     Editing of dedupe policy using GUI.
## @description
##     This method can be used to edit dedupe policy using GUI.
## @synopsis
##    dedupe_policy_edit(%args);
## INPUTS TO METHOD CALL 'dedupe_policy_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -maxruntime </td> <td> Selecting maxruntime </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub dedupe_policy_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-policyName' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $DEDUPLICATION_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $DEDUPE_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking dedupe policy edit button");
    $self->click($DEDUPE_POLICY_EDIT);

    $Log->comment("Waiting for dedupe policy edit Dialog");
    #$self->wait_for_element_present( $DEDUPE_POLICY_EDIT_SCHEDULE,
    #    $GENERIC_WAIT );

    $self->wait_for_loading();

    #Selecting schdeule
    if ( exists( $args{-schedule} ) ) {
        $Log->comment("select schedule : $args{-schedule}");
        $self->set_combobox_value( $DEDUPE_POLICY_EDIT_SCHEDULE,
            $args{-schedule} );
    }

    #Entering value for maximum runtime
    if ( exists( $args{-maxruntime} ) ) {
        $Log->comment("enter maxruntime : $args{-maxruntime}");
        $self->type( $DEDUPE_POLICY_EDIT_MAXRUNTIME, "" );
        $self->type_keys( $DEDUPE_POLICY_EDIT_MAXRUNTIME,
            $args{-maxruntime} );
    } ## end if ( exists( $args{-maxruntime...}))

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($DEDUPE_POLICY_EDIT_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("policy edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub dedupe_policy_edit

################################################################################
## @name ::dedupe_policy_delete
## @summary
##    Deletion of dedupe policy using GUI.
## @description
##     This method can be used to delete dedupe policy using GUI.
## @synopsis
##    dedupe_policy_delete(%args);
## INPUTS TO METHOD CALL 'dedupe_policy_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -checkbox </td> <td> Selecting checkbox </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub dedupe_policy_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-policyName' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $DEDUPLICATION_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $DEDUPE_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking dedupe policy delete button");
    $self->click($DEDUPE_POLICY_DELETE);

    $Log->comment("Waiting for dedupe policy delete Dialog");
    $self->wait_for_element_present( $DEDPE_POLICY_DELETE_CONFIRM_CHECKBOX,
        $GENERIC_WAIT );

    #Selecting confirm checkbox
    if ( exists( $args{-checkbox} ) ) {
        $Log->comment("selecting confirm checkbox");
        $self->click($DEDPE_POLICY_DELETE_CONFIRM_CHECKBOX);
    }

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($DEDUPE_POLICY_DELETE_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("policy deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub dedupe_policy_delete

################################################################################
## @name ::dedupe_policy_status
## @summary
##    Get status of dedupe policy using GUI.
## @description
##     This method can be used to get status using GUI.
## @synopsis
##    dedupe_policy_status(%args);
## INPUTS TO METHOD CALL 'dedupe_policy_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -status </td> <td> Selecting status </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub dedupe_policy_status {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-policyName' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my ( $commit_time, );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $DEDUPLICATION_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $DEDUPE_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking dedupe status button");
    $self->click($DEDUPE_POLICY_STATUS_BUTTON);

    if ( lc( $args{-status} ) eq "enable" ) {
        if ( $self->is_button_enabled("$DEDUPE_POLICY_STATUS_ENABLE") ) {
            $Log->comment("Click enable Status Button");
            $commit_time
                = $self->click_timecount($DEDUPE_POLICY_STATUS_ENABLE);
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("enable button is not enabled");
        }
    } ## end if ( lc( $args{-status...}))

    elsif ( lc( $args{-status} ) eq "disable" ) {
        if ( $self->is_button_enabled("$DEDUPE_POLICY_STATUS_DISABLE") ) {
            $Log->comment("Click disable Status Button");
            $self->click($DEDUPE_POLICY_STATUS_DISABLE);
            $self->click($DEDUPE_POLICY_STATUS_CONFIRMATION_CHECKBOX);
            $commit_time = $self->click_timecount(
                $DEDUPE_POLICY_STATUS_CONFIRMDISABLE_BUTTON);
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("disable button is not enabled");
        }
    } ## end elsif ( lc( $args{-status...}))

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("policy status changed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub dedupe_policy_status

################################################################################
## @name ::dedupe_policy_refresh
## @summary
##    Refresh dedupe policy using GUI.
## @description
##     This method can be used to refresh status using GUI.
## @synopsis
##    dedupe_policy_refresh(%args);
## INPUTS TO METHOD CALL 'dedupe_policy_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub dedupe_policy_refresh {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $DEDUPLICATION_POLICIES_TREE );
    $self->wait_for_element_present($DEDUPE_POLICY_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $DEDUPE_POLICY_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub dedupe_policy_refresh

################################################################################
## @name ::get_dedupe_policy_info
## @summary
##    Fetch dedupe policy details using GUI.
## @description
##     This method can be used to fetch dedupe policy status using GUI.
## @synopsis
##    get_dedupe_policy_info(%args);
## INPUTS TO METHOD CALL 'get_dedupe_policy_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver</td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the DedupePolicy details.

sub get_dedupe_policy_info {
    
    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { '-policyName' => { type => SCALAR }, },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;
    $self->clear_window();
    
    my $policyName = $args{'-policyName'};
    my $dedupe_policy_info;
    
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    

    $Log->step("Navigating to $DEDUPLICATION_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
            $DEDUPLICATION_POLICIES_TREE );

    $Log->step("Selecting policy: $policyName");
    $self->click_row_with_texts( $DEDUPE_POLICY_GRID,
            $policyName );

    $Log->step("Getting row data for dedupe policy $policyName from grid");
    my $rowdata = $self->get_row_data_with_text($DEDUPE_POLICY_GRID, $policyName);

    my @rowarray = split(' ',$rowdata);
    my %temp_dedupe_policy_info;

    %temp_dedupe_policy_info = (
            "name"        => $rowarray[0],
            "status"      => $rowarray[1],
            "schedule"    => $rowarray[2],
            "max_runtime" => $rowarray[3],
            );

    $Log->comment("Collection of dedupe policy info is successfull");
    $dedupe_policy_info->{loading_time} = $loading_time;
    $dedupe_policy_info->{parsed_output} = \%temp_dedupe_policy_info;

    $Log->exit() if $may_exit;

    return $dedupe_policy_info;
} ## end sub get_dedupe_policy_info

1;
