# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##    Routines to parse Cmode Disks GUI Functionalities
##
## @description
## Using this library, user can perform Cmode disks related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmDisks.pm
##
##      create_aggregate_from_disks(%args);
##      add_disks_to_aggregate(%args);
##      get_disk_info(%args);
##      disk_refresh(%args);
##
## @status public
##
## @author Arun William - arunw@netapp.com
## @change
##  6/7/2010 : Arun William: Initial version
## @change
##  3/20/2012 : Vikas P : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmDisks;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    create_aggregate_from_disks
    add_disks_to_aggregate
    get_disk_info
    disk_refresh
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::create_aggregate_from_disks
## @summary
##     Creation of Aggregate from disks using GUI.
## @description
##     This method can be used to create aggregate from disks using GUI.
## @synopsis
##     create_aggregate_from_disks(%args);
## INPUTS TO METHOD CALL 'create_aggregate_from_disks' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -disks </td> <td> (Required)No of disks to be used </td> </tr>
## <tr> <td> -raidtype </td> <td> Raid type to be used for creating the aggregate</td> </tr>
## <tr> <td> -snaplock </td> <td> Selecting snaplock type on input parameter </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub create_aggregate_from_disks {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-aggrname" => { type => SCALAR },
            "-disks"    => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my ( $disk, $diskcount, $diskname, $diskusablesize, $disktotalsize,
        $vserverName );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $DISK_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DISK_TREE );

    $disk = $args{-disks}[0];
    for my $count ( 1 .. $#{ $args{-disks} } ) {
        $diskname = $args{-disks}[$count];
        $disk     = $disk . "|" . $diskname;
    }

    #Filter using disknames
    if ( exists( $args{-disks} ) ) {
        $Log->step("Selecting disk name tab");
        $self->click("$DISKS_NAME_FILTER_TAB");
        $Log->step("Filtering by disk names $disk");
        $self->type( $DISKS_NAME_FILTER_TEXT, $disk );
        $self->fire_event( $DISKS_NAME_FILTER_TEXT, "focus" );
        $self->key_down( $DISKS_NAME_FILTER_TEXT, "\13" );
        $self->key_up( $DISKS_NAME_FILTER_TEXT, "\13" );
        $Log->step("Filtered by disk names");
        $self->wait_for_loading();
    } ## end if ( exists( $args{-disks...}))

    # Selecting disks after filter
    for my $count ( 0 .. $#{ $args{-disks} } ) {
        my $diskname = $args{-disks}[$count];
        $Log->step("Selecting disks");
        $self->wait_for_loading();
        $self->click_row_with_texts( $DISKS_TAB_GRID, $diskname );
        $self->control_key_down();
        $self->wait_for_loading();
    } ## end for my $count ( 0 .. ( ...))
    $self->control_key_up();
    $self->wait_for_loading();

    #Create aggregate
    $Log->step("Checking if Create Aggregate button is enabled");
    if ( not $self->is_button_enabled($DISKS_CREATE_AGGR_BUTTON) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Create Aggregate button not enabled");
    }

    $Log->comment("Clicking Create Aggregate button");
    $self->click($DISKS_CREATE_AGGR_BUTTON);

    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Checking for warning popup");
    if ( $self->is_element_present($DISKS_AGGR_WARNING) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Minimum of one disk should be left as spare.");
    }

    if ($self->is_element_present ($DISK_AGGR_CREATE_WARNING_DIALOG)) {
        $Log->comment("Checking for SM does not recommend warning pop up");
        $self->click($DISK_AGGR_CREATE_WARNING_DIALOG);
    }        
 
    $Log->comment("Waiting for Create Aggregate dialog to open");
    #$self->wait_for_element_present( $DISKS_CREATE_AGGR_NAME_INPUT,
    #    $GENERIC_WAIT );

    $self->wait_for_loading();

    $Log->comment("Entering AggrName : $args{'-aggrname'}");
    $self->type( $DISKS_CREATE_AGGR_NAME_INPUT, $args{'-aggrname'} );

    if ( exists( $args{-raidtype} ) ) {
        my $raidtype = $args{-raidtype};
        $Log->comment("Selecting $raidtype as RAID type");
        if ( $raidtype eq "raiddp" ) {
            $self->set_option($DISKS_CREATE_AGGR_RAIDDP_OPTION);
        } elsif ( $raidtype eq "raid4" ) {
            $self->set_option($DISKS_CREATE_AGGR_RAID4_OPTION);
        } elsif ( $raidtype eq "raid0" ) {
            $self->set_option($DISKS_CREATE_AGGR_RAID0_OPTION);
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid raid type. Use raid4 / raiddp / raid0.");
        }
    } ## end if ( exists( $args{-raidtype...}))

    $diskcount      = $self->get_text($DISKS_CREATE_AGGR_DISK_COUNT);
    $diskusablesize = $self->get_text($DISKS_CREATE_AGGR_DISK_USABLESIZE);
    $disktotalsize  = $self->get_text($DISKS_CREATE_AGGR_DISK_TOTALSIZE);
    $Log->comment("Disk: Count / Usable size / Total size");
    $Log->comment($diskcount);
    $Log->comment($diskusablesize);
    $Log->comment($disktotalsize);

 #Select Snaplock tab based on user input (Added by Raja Rao on sep 16th 2010)
    if ( exists( $args{'-snaplock'} ) ) {
        $Log->comment("Selecting Snaplock tab");
        $self->click("$DISKS_CREATE_SNAPLOCK_TAB");

        $Log->comment("Selecting snaplock checkbox");
        $self->click("$DISKS_SNAPLOCK_CHECKBOX");

        # Selecting snaplock type based on input parameter
        $Log->comment("Selecting snaplock type on input parameter");
        if ( lc( $args{'-snaplock'} eq "enterprise" ) ) {
            $Log->comment("Selecting Enterprise option");
            $self->set_option("$DISKS_SNAPLOCK_ENTERPRISE_OPTION");
        } elsif ( lc( $args{'-snaplock'} eq "compliance" ) ) {
            $Log->comment("Selecting Compliance option");
            $self->set_option("$DISKS_SNAPLOCK_COMPLIANCE_OPTION");
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid value for snaplock type, valid values are 'enterprise' or 'compliance'"
            );
        } ## end else [ if ( lc( $args{'-snaplock'...}))]
    } ## end if ( exists( $args{'-snaplock'...}))

    $Log->comment("Clicking on OK button to complete aggregate create");

    my $commit_time = $self->click_timecount($DISKS_CREATE_AGGR_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("aggregate created successfully.");

    # Clearing disks filter
    $Log->step("Navigating to $DISK_TREE");
    $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY}, $DISK_TREE );
    $self->wait_for_loading();
    $self->sort_filter( "$DISKS_TAB_GRID", ['Name:'] );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub create_aggregate_from_disks

################################################################################
## @name ::add_disks_to_aggregate
## @summary
##     Add disk to existing Aggregate using GUI.
## @description
##     This method can be used to add disk to existing aggregate using GUI.
## @synopsis
##     add_disks_to_aggregate(%args);
## INPUTS TO METHOD CALL 'add_disks_to_aggregate' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Name of the aggregate to be edited </td> </tr>
## <tr> <td> -disks </td> <td> (Required)No of disks to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub add_disks_to_aggregate {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-aggrname" => { type => SCALAR },
            "-disks"    => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    my ( $disk, $diskname, $vserverName, );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $DISK_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DISK_TREE );

    $disk = $args{-disks}[0];
    for my $count ( 1 .. $#{ $args{-disks} } ) {
        $diskname = $args{-disks}[$count];
        $disk     = $disk . "|" . $diskname;
    }

    #Filter using disknames
    if ( exists( $args{-disks} ) ) {
        $Log->step("Selecting disk name tab");
        sleep 10;
        $self->wait_for_loading();
        $self->click("$DISKS_NAME_FILTER_TAB");
        $Log->step("Filtering by disk names $disk");
        $self->type( $DISKS_NAME_FILTER_TEXT, $disk );
        $self->fire_event( $DISKS_NAME_FILTER_TEXT, "focus" );
        $self->key_down( $DISKS_NAME_FILTER_TEXT, "\13" );
        $self->key_up( $DISKS_NAME_FILTER_TEXT, "\13" );
        $Log->step("Filtered by disk names");
        $self->wait_for_loading();
    } ## end if ( exists( $args{-disks...}))
    
    # Selecting disks after filter
    for my $count ( 0 .. $#{ $args{-disks}} ) {
        my $diskname = $args{-disks}[$count];
        $Log->step("Selecting disks");
        sleep 5;
        $self->click_row_with_texts( $DISKS_TAB_GRID, $diskname );
        $self->control_key_down();
        sleep 5;
        $self->wait_for_loading();
    } ## end for my $count ( 0 .. $#...)
    $self->control_key_up();
    sleep 10;
    
    $self->wait_for_loading();
    $Log->step("Checking if Add to Aggregate button is enabled");
    if ( not $self->is_button_enabled($DISKS_ADD_TO_AGGR_BUTTON) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Add Disks to Aggregate button not enabled");
    }

    $Log->comment("Clicking Add to Aggregate button");
    $self->click($DISKS_ADD_TO_AGGR_BUTTON);

    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Checking for warning popup");
    if ( $self->is_element_present($DISKS_AGGR_WARNING) ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Minimum of one disk should be left as spare.");
    }

    if ($self->is_element_present ($DISK_AGGR_CREATE_WARNING_DIALOG)) {
        $Log->comment("Checking for SM does not recommend warning pop up");
        $self->click($DISK_AGGR_CREATE_WARNING_DIALOG);
    }
      
    $Log->comment("Waiting for Add Disks to Aggregate dialog to open");
    $self->wait_for_element_present( $DISKS_ADD_TO_AGGR_NAME_COMBOBOX,
        $GENERIC_WAIT );

    $Log->comment("Selecting AggrName : $args{-aggrname}");
    $self->set_combobox_value( $DISKS_ADD_TO_AGGR_NAME_COMBOBOX,
        $args{-aggrname} );

    my ( $diskcount, $diskusablesize, $disktotalsize );
    $diskcount      = $self->get_text($DISKS_ADD_TO_AGGR_DISK_COUNT);
    $diskusablesize = $self->get_text($DISKS_ADD_TO_AGGR_DISK_USABLESIZE);
    $disktotalsize  = $self->get_text($DISKS_ADD_TO_AGGR_DISK_TOTALSIZE);
    $Log->comment("Disk: Count / Usable size / Total size");
    $Log->comment($diskcount);
    $Log->comment($diskusablesize);
    $Log->comment($disktotalsize);

    $Log->comment("Clicking on OK button to complete aggregate create");
    my $commit_time
        = $self->click_timecount($DISKS_ADD_TO_AGGR_DISK_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("PASS: Successfully added disk to aggregate.");

    # Clearing disks filter
    $Log->step("Navigating to $DISK_TREE");
    $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY}, $DISK_TREE );
    $self->wait_for_loading();
    $self->sort_filter( "$DISKS_TAB_GRID", ['Name:'] );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub add_disks_to_aggregate

################################################################################
## @name ::get_disk_info
## @summary
##     Get information related to disk using GUI.
## @description
##     This method can be used to get information related to disk using GUI.
## @synopsis
##     get_disk_info(%args);
## INPUTS TO METHOD CALL 'get_disk_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -diskname </td> <td> (Required)Name of the aggregate to be edited </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Disk status details.

sub get_disk_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-diskname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my %values;
    my $disk_info = {};

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $DISK_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DISK_TREE );
    $self->wait_for_loading();
    
    #Filter using disknames
    $Log->step("Selecting disk name tab");
    $self->click("$DISKS_NAME_FILTER_TAB");
    $Log->step("Filtering by disk names $args{'-diskname'}");
    $self->type($DISKS_NAME_FILTER_TEXT, $args{'-diskname'});
    $self->fire_event($DISKS_NAME_FILTER_TEXT, "focus");
    $self->key_down($DISKS_NAME_FILTER_TEXT, "\13");
    $self->key_up($DISKS_NAME_FILTER_TEXT, "\13");
    $Log->step("Filtered by disk names");
    $self->wait_for_loading();

    $Log->step("Selecting disk : $args{'-diskname'}");
    $self->click_row_with_texts( $DISKS_TAB_GRID, $args{'-diskname'} );

    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Fetching Disk Aggregate Name ");
    $values{$DISK_AGGREGATE_KEY} = $self->get_text($DISK_AGGREGATE);
    $Log->comment("Fetching Disk Vendor ID");
    $values{$DISK_VENDOR_ID_KEY} = $self->get_text($DISK_VENDOR_ID);
    $Log->comment("Fetching Disk Serial No");
    $values{$DISK_SERIAL_NO_KEY} = $self->get_text($DISK_SERIAL_NO);
    $Log->comment("Fetching Disk Zeroing %");
    $values{$DISK_ZEROING_PERCENTAGE_KEY}
        = $self->get_text($DISK_ZEROING_PERCENTAGE);
    $Log->comment("Fetching Disk Broken Details");
    $values{$DISK_BROKEN_DETAILS_KEY} = $self->get_text($DISK_BROKEN_DETAILS);

    $Log->comment("Collection of Disk info is successfull");
    $disk_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    return $disk_info;

} ## end sub get_disk_info

################################################################################
## @name ::disk_refresh
## @summary
##     Refresh aggregate disks using GUI.
## @description
##     This method can be used to refresh aggregate disks using GUI.
## @synopsis
##     disk_refresh(%args);
## INPUTS TO METHOD CALL 'disk_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub disk_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        my $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DISK_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DISK_TREE );

    $self->wait_for_element_present($DISKS_TAB_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $DISKS_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time => $commit_time
    );
    $Log->comment("disk page refresh completed successfully");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub disk_refresh

1;

