#  Copyright (c) 2002 Network Appliance, Inc.
#  All rights reserved.
## @summary
##    Routines to parse Cmode export policy related GUI Functionalities
##
## @description
##    Using this library, user can perform Cmode exports related related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmExportsPolicies.pm
##
##      export_policy_create(%args);
##      export_policy_rename(%args);
##      export_policy_delete(%args);
##      export_policy_add_rule(%args);
##      export_policy_modify_rule(%args);
##      export_policy_rule_delete(%args);
##      export_policy_refresh(%args);
##      get_export_policy_info(%args);
##      get_export_policy_rule_info(%args);
##
## @status
##
## @author afsars@netapp.com
## @change
##  2011-06-07 : afsars : Initial version
## @change
##  2012-01-25 : vikasp : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmExportPolicies;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    export_policy_create
    export_policy_rename
    export_policy_delete
    export_policy_add_rule
    export_policy_modify_rule
    export_policy_rule_delete
    export_policy_refresh
    get_export_policy_info
    get_export_policy_rule_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::export_policy_create
## @summary
##     Creation of export policy using GUI.
## @description
##     This method can be used to create exportpolicy using GUI.
## @synopsis
##    export_policy_create(%args);
## INPUTS TO METHOD CALL 'export_policy_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -exportpath </td> <td> (Required) Path where the export has to be created </td> </tr>
## <tr> <td> -addeditpermissions </td> <td> Adding new client permission rule </td> </tr>
## <tr> <td> -permissionaction </td> <td> Select the permissions to be edited from the grid </td> </tr>
## <tr> <td> -secflavournone</td> <td> Update security flavour </td> </tr>
## <tr> <td> -secflavourunix </td> <td> Update security flavour as unix  </td> </tr>
## <tr> <td> -secflavourkerb5 </td> <td> Update security flavour as kerb5 </td> </tr>
## <tr> <td> -secflavourkerb5int </td> <td> Update security flavour as kerb5int </td> </tr>
## <tr> <td> -secflavourkerb5priv </td> <td> Update security flavour as kerb5priv </td> </tr>
## <tr> <td> -addclients </td> <td> Adding clients to the export </td> </tr>
## <tr> <td> -deleteclients </td> <td> Deleting clients from the export </td> </tr>
## <tr> <td> -editclients </td> <td> Editing clients from the export </td> </tr>
## <tr> <td> -annonaccess </td> <td> Selecting anonymous access based on input parameter </td> </tr>
## <tr> <td> -enablesetid  </td> <td> Enabling setuid </td> </tr>
## <tr> <td> -deletepermissions </td> <td> Selecting permission to be deleted from the grid </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_create {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $Log->comment("Clicking export policy add button");
    $self->click($EXPORT_POLICY_CREATE_BUTTON);

    $Log->comment("Waiting for export policy add Dialog");
    $self->wait_for_loading();

    $Log->comment("Entering policy name: $args{'-policyName'}");
    $self->type_keys( $EXPORT_POLICY_NAME_TEXTBOX, $args{'-policyName'} );
    $self->type( $EXPORT_POLICY_NAME_TEXTBOX, $args{'-policyName'} );
    
    if ( exists( $args{-copyrules} ) ) {
        $Log->comment("selecting copy rules");
        $self->click($EXPORT_POLICY_COPY_RULES_CHECKBOX);

        if ( exists( $args{-vserver} ) ) {
            $Log->comment("select vserver : $args{-vserver}");
            $self->set_combobox_value( $EXPORT_POLICY_RULES_VSERVER,
                $args{-vserver} );
        }

        if ( exists( $args{-sourcepolicy} ) ) {
            $Log->comment("select sourcepolicy : $args{-sourcepolicy}");
            $self->set_combobox_value( $EXPORT_POLICY_RULES_SOURCE_POLICY,
                $args{-sourcepolicy} );
        }
    } ## end if ( exists( $args{-copyrules...}))

    $self->wait_for_loading();

    if ( exists( $args{-addrules} ) ) {
        $Log->comment("clicking add button");
        $self->click($EXPORT_POLICY_RULES_ADD);

        if ( exists( $args{-clientspec} ) ) {
            $Log->comment("entering client spec");
            $self->type_keys( $EXPORT_RULES_CLIENT_TEXTBOX,
                $args{-clientspec} );
            $self->type( $EXPORT_RULES_CLIENT_TEXTBOX,
                $args{-clientspec} );
        }

        if ( exists( $args{-protocolcifs} ) ) {
            $Log->comment("Enter protocol : $args{-protocolcifs}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_CIFS",
                $args{-protocolcifs} );
        }

        if ( exists( $args{-protocolnfs} ) ) {
            $Log->comment("Enter protocol : $args{-protocolnfs}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_NFS",
                $args{-protocolnfs} );
        }

        if ( exists( $args{-protocolnfsv2} ) ) {
            $Log->comment("Enter protocol : $args{-protocolnfsv2}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_NFSV2",
                $args{-protocolnfsv2} );
        }

        if ( exists( $args{-protocolnfsv3} ) ) {
            $Log->comment("Enter protocol : $args{-protocolnfsv3}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_NFSV3",
                $args{-protocolnfsv3} );
        }

        if ( exists( $args{-protocolnfsv4} ) ) {
            $Log->comment("Enter protocol : $args{-protocolnfsv4}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_NFSV4",
                $args{-protocolnfsv4} );
        }

        if ( exists( $args{-protocolflex} ) ) {
            $Log->comment("Enter protocol : $args{-protocolflex}");
            $self->set_checkbox( "$EXPORT_RULES_PROTOCOL_FLEXCACHE",
                $args{-protocolflex} );
        }

        if ( exists( $args{-readonly} ) eq 'disable' ) {
            $Log->comment("clicking readonly option");
            $self->click($EXPORT_RULES_READONLY_CHECKBOX);
        } elsif ( exists( $args{-readonly} ) eq 'enable' ) {
            if ( lc( $args{-addreadonlyopt} ) eq 'any' ) {
                $Log->comment("select : $args{-addreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            } elsif ( lc( $args{-addreadonlyopt} ) eq 'none' ) {
                $Log->comment("select : $args{-addreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_NONE_CHECKBOX");
            } elsif ( lc( $args{-addreadonlyopt} ) eq 'unix' ) {
                $Log->comment("select : $args{-addreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_UNIX_CHECKBOX");
            } elsif ( lc( $args{-addreadonlyopt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-addreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-addreadonlyopt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-addreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-readonly...}))

        if ( exists( $args{-readwrite} ) eq 'disable' ) {
            $Log->comment("clicking readwrite option");
            $self->click($EXPORT_RULES_READWRITE_CHECKBOX);
        } elsif ( exists( $args{-readwrite} ) eq 'enable' ) {
            if ( lc( $args{-addreadwriteopt} ) eq 'any' ) {
                $Log->comment("select : $args{-addreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            } elsif ( lc( $args{-addreadwriteopt} ) eq 'none' ) {
                $Log->comment("select : $args{-addreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_NONE_CHECKBOX");
            } elsif ( lc( $args{-addreadwriteopt} ) eq 'unix' ) {
                $Log->comment("select : $args{-addreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_UNIX_CHECKBOX");
            } elsif ( lc( $args{-addreadwriteopt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-addreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-addreadwriteopt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-addreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-readwrite...}))

        if ( exists( $args{-superuser} ) eq 'disable' ) {
            $Log->comment("clicking superuser option");
            $self->click($EXPORT_RULES_SUPERUSER_CHECKBOX);
        } elsif ( exists( $args{-superuser} ) eq 'enable' ) {
            if ( lc( $args{-addsuperuseropt} ) eq 'any' ) {
                $Log->comment("select : $args{-addsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            } elsif ( lc( $args{-addsuperuseropt} ) eq 'none' ) {
                $Log->comment("select : $args{-addsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_NONE_CHECKBOX");
            } elsif ( lc( $args{-addsuperuseropt} ) eq 'unix' ) {
                $Log->comment("select : $args{-addsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_UNIX_CHECKBOX");
            } elsif ( lc( $args{-addsuperuseropt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-addsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-addsuperuseropt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-addsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-superuser...}))

        $Log->comment("Selecting OK button to complete.");
        $self->click($EXPORT_RULES_OK_BUTTON);
    } ## end if ( exists( $args{-addrules...}))

    if ( exists( $args{-editrules} ) ) {
        $Log->comment("Selecting one rule to edit");
        $self->click($EXPORT_POLICY_CREATE_RULES_GRID);

        $Log->comment("clicking edit button");
        $self->click($EXPORT_POLICY_RULES_EDIT);

        if ( exists( $args{-clientspec} ) ) {
            $Log->comment("entering client spec");
            $self->type_keys( $EXPORT_RULES_CLIENT_TEXTBOX,
                $args{-clientspec} );
        }

        if ( exists( $args{-editprotocol} ) ) {
            if ( lc( $args{-editprotocol} ) eq 'cifs' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_CIFS");
            } elsif ( lc( $args{-editprotocol} ) eq 'nfs' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_NFS");
            } elsif ( lc( $args{-editprotocol} ) eq 'nfsv2' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_NFSV2");
            } elsif ( lc( $args{-editprotocol} ) eq 'nfsv3' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_NFSV3");
            }
            elsif ( lc( $args{-editprotocol} ) eq 'nfsv4' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_NFSV4");
            }
            elsif ( lc( $args{-editprotocol} ) eq 'flexcache' ) {
                $Log->comment("Enter protocol : $args{-editprotocol}");
                $self->click("$EXPORT_RULES_PROTOCOL_FLEXCACHE");
            }
        } ## end if ( exists( $args{-editprotocol...}))

        if ( exists( $args{-readonly} ) eq 'disable' ) {
            $Log->comment("clicking readonly option");
            $self->click($EXPORT_RULES_READONLY_CHECKBOX);
        } elsif ( exists( $args{-readonly} ) eq 'enable' ) {
            if ( lc( $args{-editreadonlyopt} ) eq 'any' ) {
                $Log->comment("select : $args{-editreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            } elsif ( lc( $args{-editreadonlyopt} ) eq 'none' ) {
                $Log->comment("select : $args{-editreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_NONE_CHECKBOX");
            } elsif ( lc( $args{-editreadonlyopt} ) eq 'unix' ) {
                $Log->comment("select : $args{-editreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_UNIX_CHECKBOX");
            } elsif ( lc( $args{-editreadonlyopt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-editreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-editreadonlyopt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-editreadonlyopt}");
                $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READONLY_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-readonly...}))

        if ( exists( $args{-readwrite} ) eq 'disable' ) {
            $Log->comment("clicking readwrite option");
            $self->click($EXPORT_RULES_READWRITE_CHECKBOX);
        } elsif ( exists( $args{-readwrite} ) eq 'enable' ) {
            if ( lc( $args{-editreadwriteopt} ) eq 'any' ) {
                $Log->comment("select : $args{-editreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            } elsif ( lc( $args{-editreadwriteopt} ) eq 'none' ) {
                $Log->comment("select : $args{-editreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_NONE_CHECKBOX");
            } elsif ( lc( $args{-editreadwriteopt} ) eq 'unix' ) {
                $Log->comment("select : $args{-editreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_UNIX_CHECKBOX");
            } elsif ( lc( $args{-editreadwriteopt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-editreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-editreadwriteopt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-editreadwriteopt}");
                $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_READWRITE_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-readwrite...}))

        if ( exists( $args{-superuser} ) eq 'disable' ) {
            $Log->comment("clicking superuser option");
            $self->click($EXPORT_RULES_SUPERUSER_CHECKBOX);
        } elsif ( exists( $args{-superuser} ) eq 'enable' ) {
            if ( lc( $args{-editsuperuseropt} ) eq 'any' ) {
                $Log->comment("select : $args{-editsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            } elsif ( lc( $args{-editsuperuseropt} ) eq 'none' ) {
                $Log->comment("select : $args{-editsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_NONE_CHECKBOX");
            } elsif ( lc( $args{-editsuperuseropt} ) eq 'unix' ) {
                $Log->comment("select : $args{-editsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_UNIX_CHECKBOX");
            } elsif ( lc( $args{-editsuperuseropt} ) eq 'kerberos' ) {
                $Log->comment("select : $args{-editsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_KERBEROS_CHECKBOX");
            } elsif ( lc( $args{-editsuperuseropt} ) eq 'ntlm' ) {
                $Log->comment("select : $args{-editsuperuseropt}");
                $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
                $self->click("$EXPORT_RULES_SUPERUSER_NTLM_CHECKBOX");
            }
        } ## end elsif ( exists( $args{-superuser...}))

        $Log->comment("Selecting OK button to complete.");
        $self->click($EXPORT_RULES_OK_BUTTON);
    } ## end if ( exists( $args{-editrules...}))

    if ( exists( $args{-deleterules} ) ) {
        $Log->comment("Selecting one rule to edit");
        $self->click($EXPORT_POLICY_CREATE_RULES_GRID);

        $Log->comment("clicking delete button");
        $self->click($EXPORT_POLICY_RULES_DELETE);

        $Log->comment("Selecting OK button to complete.");
        $self->click($EXPORT_RULES_OK_BUTTON);
    } ## end if ( exists( $args{-deleterules...}))

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_POLICY_CREATE_OK_BUTTON);
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );

    $Log->comment("snapshot policy created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_create

################################################################################
## @name ::export_policy_rename
## @summary
##     Rename the export policy using GUI.
## @description
##     This method can be used to rename the export policy using GUI.
## @synopsis
##    export_policy_rename(%args);
## INPUTS TO METHOD CALL 'export_policy_rename' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -rename </td> <td> Changing policy name  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_rename {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking export policy rename button");
    $self->click($EXPORT_POLICY_RENAME_BUTTON);

    $Log->comment("Waiting for export policy rename Dialog");
    $self->wait_for_element_present( $EXPORT_POLICY_RENAME_NAME_TEXTBOX,
        $GENERIC_WAIT );

    if ( exists( $args{-rename} ) ) {
        $Log->comment("changing policy name");
        $self->type( $EXPORT_POLICY_RENAME_NAME_TEXTBOX, "" );
        $self->type_keys( $EXPORT_POLICY_RENAME_NAME_TEXTBOX,
            $args{-rename} );

    } ## end if ( exists( $args{-rename...}))

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_POLICY_RENAME_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $EXPORT_POLICY_RENAME_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );

    $Log->comment("policy renamed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_rename

################################################################################
## @name ::export_policy_delete
## @summary
##     Delete the export policy using GUI.
## @description
##     This method can be used to delete the export policy using GUI.
## @synopsis
##    export_policy_delete(%args);
## INPUTS TO METHOD CALL 'export_policy_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -checkbox </td> <td> Selecting confirm checkbox  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking export policy delete button");
    $self->click($EXPORT_POLICY_DELETE_BUTTON);

    $Log->comment("Waiting for export policy delete Dialog");
    $self->wait_for_element_present( $EXPORT_POLICY_DELETE_CONFORM_CHECKBOX,
        $GENERIC_WAIT );

    #Selecting confirm checkbox
    if ( exists( $args{-checkbox} ) ) {
        $Log->comment("selecting confirm checkbox");
        $self->click($EXPORT_POLICY_DELETE_CONFORM_CHECKBOX);
    }

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_POLICY_DELETE_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $EXPORT_POLICY_DELETE_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );

    $Log->comment("policy deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_delete

################################################################################
## @name ::export_policy_add_rule
## @summary
##     Add rule to export policy using GUI.
## @description
##     This method can be used to add rule to export policy using GUI.
## @synopsis
##    export_policy_add_rule(%args);
## INPUTS TO METHOD CALL 'export_policy_add_rule' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -clientspec </td> <td> Entering client spec  </td> </tr>
## <tr> <td> -ruleindex </td> <td>  Changing rule index </td> </tr>
## <tr> <td> -protocol </td> <td>   Selecting protocol  </td> </tr>
## <tr> <td> -readonly </td> <td> Clicking readonly option </td> </tr>
## <tr> <td> -addreadonlyopt </td> <td> Adding readonly option </td> </tr>
## <tr> <td> -readwrite </td> <td> Clicking readwrite option  </td> </tr>
## <tr> <td> -addreadwriteopt </td> <td> Adding readwrite option  </td> </tr>
## <tr> <td> -superuser </td> <td> Clicking superuser option  </td> </tr>
## <tr> <td> -addsuperuseropt </td> <td> Adding superuser option  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_add_rule {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $self->wait_for_loading();

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking add rule button");
    $self->click($EXPORT_RULES_ADD_BUTTON);

    if ( exists( $args{-clientspec} ) ) {
        $Log->comment("entering client spec");
        $self->type_keys( $EXPORT_RULES_CLIENT_TEXTBOX, $args{-clientspec} );
    }
    if ( exists( $args{-ruleindex} ) ) {
        $Log->comment("changing rule index");
        $self->type( $EXPORT_RULES_INDEX, $args{-ruleindex} );
    }
    if ( exists( $args{-protocol} ) ) {
        if ( lc( $args{-protocol} ) eq 'cifs' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_CIFS");
        } elsif ( lc( $args{-protocol} ) eq 'nfs' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFS");
        } elsif ( lc( $args{-protocol} ) eq 'nfsv2' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV2");
        } elsif ( lc( $args{-protocol} ) eq 'nfsv3' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV3");
        }
        elsif ( lc( $args{-protocol} ) eq 'nfsv4' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV4");
        }
        elsif ( lc( $args{-protocol} ) eq 'flexcache' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_FLEXCACHE");
        }
    } ## end if ( exists( $args{-protocol...}))

    if ( exists( $args{-readonly} ) eq 'disable' ) {
        $Log->comment("clicking readonly option");
        $self->click($EXPORT_RULES_READONLY_CHECKBOX);
    } elsif ( exists( $args{-readonly} ) eq 'enable' ) {
        if ( lc( $args{-addreadonlyopt} ) eq 'any' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'none' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_NONE_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-readonly...}))

    if ( exists( $args{-readwrite} ) eq 'disable' ) {
        $Log->comment("clicking readwrite option");
        $self->click($EXPORT_RULES_READWRITE_CHECKBOX);
    } elsif ( exists( $args{-readwrite} ) eq 'enable' ) {
        if ( lc( $args{-addreadwriteopt} ) eq 'any' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'none' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_NONE_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-readwrite...}))

    if ( exists( $args{-superuser} ) eq 'disable' ) {
        $Log->comment("clicking superuser option");
        $self->click($EXPORT_RULES_SUPERUSER_CHECKBOX);
    } elsif ( exists( $args{-superuser} ) eq 'enable' ) {
        if ( lc( $args{-addsuperuseropt} ) eq 'any' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'none' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_NONE_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-superuser...}))

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_RULES_OK_BUTTON);

    $self->wait_for_loading();
    $self->wait_for_element_not_present( $EXPORT_RULES_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("policy status changed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_add_rule

################################################################################
## @name ::export_policy_modify_rule
## @summary
##     Modify rule to export policy using GUI.
## @description
##     This method can be used to modify rule to export policy using GUI.
## @synopsis
##    export_policy_modify_rule(%args);
## INPUTS TO METHOD CALL 'export_policy_modify_rule' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the export to be modified </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -clientspec </td> <td> Entering client spec  </td> </tr>
## <tr> <td> -ruleindex </td> <td>  Changing rule index </td> </tr>
## <tr> <td> -protocol </td> <td>   Selecting protocol  </td> </tr>
## <tr> <td> -readonly </td> <td> Clicking readonly option </td> </tr>
## <tr> <td> -addreadonlyopt </td> <td> Adding readonly option </td> </tr>
## <tr> <td> -readwrite </td> <td> Clicking readwrite option  </td> </tr>
## <tr> <td> -addreadwriteopt </td> <td> Adding readwrite option  </td> </tr>
## <tr> <td> -superuser </td> <td> Clicking superuser option  </td> </tr>
## <tr> <td> -addsuperuseropt </td> <td> Adding superuser option  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_modify_rule {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ruleName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    if ( exists($args{'-policyName'})) {
        $Log->step("Selecting policy: $args{'-policyName'}");
        $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );
    }
    
    $Log->step("Selecting rule : $args{'-ruleName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_RULES_GRID, $args{'-ruleName'} );

    $Log->comment("Clicking modify rule button");
    $self->click($EXPORT_RULES_MODIFY_BUTTON);

    if ( exists( $args{-clientspec} ) ) {
        $Log->comment("entering client spec");
        $self->type_keys( $EXPORT_RULES_CLIENT_TEXTBOX, $args{-clientspec} );
    }

    if ( exists( $args{-protocol} ) ) {
        if ( lc( $args{-protocol} ) eq 'cifs' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_CIFS");
        } elsif ( lc( $args{-protocol} ) eq 'nfs' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFS");
        } elsif ( lc( $args{-protocol} ) eq 'nfsv2' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV2");
        } elsif ( lc( $args{-protocol} ) eq 'nfsv3' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV3");
        }
        elsif ( lc( $args{-protocol} ) eq 'nfsv4' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_NFSV4");
        }
        elsif ( lc( $args{-protocol} ) eq 'flexcache' ) {
            $Log->comment("Enter protocol : $args{-protocol}");
            $self->click("$EXPORT_RULES_PROTOCOL_FLEXCACHE");
        }
    } ## end if ( exists( $args{-protocol...}))

    if ( exists( $args{-readonly} ) eq 'disable' ) {
        $Log->comment("clicking readonly option");
        $self->click($EXPORT_RULES_READONLY_CHECKBOX);
    } elsif ( exists( $args{-readonly} ) eq 'enable' ) {
        if ( lc( $args{-addreadonlyopt} ) eq 'any' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'none' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_NONE_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addreadonlyopt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addreadonlyopt}");
            $self->click("$EXPORT_RULES_READONLY_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READONLY_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-readonly...}))

    if ( exists( $args{-readwrite} ) eq 'disable' ) {
        $Log->comment("clicking readwrite option");
        $self->click($EXPORT_RULES_READWRITE_CHECKBOX);
    } elsif ( exists( $args{-readwrite} ) eq 'enable' ) {
        if ( lc( $args{-addreadwriteopt} ) eq 'any' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'none' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_NONE_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addreadwriteopt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addreadwriteopt}");
            $self->click("$EXPORT_RULES_READWRITE_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_READWRITE_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-readwrite...}))

    if ( exists( $args{-superuser} ) eq 'disable' ) {
        $Log->comment("clicking superuser option");
        $self->click($EXPORT_RULES_SUPERUSER_CHECKBOX);
    } elsif ( exists( $args{-superuser} ) eq 'enable' ) {
        if ( lc( $args{-addsuperuseropt} ) eq 'any' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'none' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_NONE_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'unix' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_UNIX_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'kerberos' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_KERBEROS_CHECKBOX");
        } elsif ( lc( $args{-addsuperuseropt} ) eq 'ntlm' ) {
            $Log->comment("select : $args{-addsuperuseropt}");
            $self->click("$EXPORT_RULES_SUPERUSER_ANY_CHECKBOX");
            $self->click("$EXPORT_RULES_SUPERUSER_NTLM_CHECKBOX");
        }
    } ## end elsif ( exists( $args{-superuser...}))

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_RULES_OK_BUTTON);

    $self->wait_for_loading();
    $self->wait_for_element_not_present( $EXPORT_RULES_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("policy status changed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_modify_rule

################################################################################
## @name ::export_policy_rule_delete
## @summary
##     Delete the export policy ruleusing GUI.
## @description
##     This method can be used to delete the export policy rule using GUI.
## @synopsis
##    export_policy_rule_delete(%args);
## INPUTS TO METHOD CALL 'export_policy_rule_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ruleName </td> <td> (Required) Name of the rule to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -clientspec </td> <td> Entering client spec  </td> </tr>
## <tr> <td> -ruleindex </td> <td>  Changing rule index </td> </tr>
## <tr> <td> -protocol </td> <td>   Selecting protocol  </td> </tr>
## <tr> <td> -readonly </td> <td> Clicking readonly option </td> </tr>
## <tr> <td> -addreadonlyopt </td> <td> Adding readonly option </td> </tr>
## <tr> <td> -readwrite </td> <td> Clicking readwrite option  </td> </tr>
## <tr> <td> -addreadwriteopt </td> <td> Adding readwrite option  </td> </tr>
## <tr> <td> -superuser </td> <td> Clicking superuser option  </td> </tr>
## <tr> <td> -addsuperuseropt </td> <td> Adding superuser option  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.
sub export_policy_rule_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ruleName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    if ( exists($args{'-policyName'})) {
        $Log->step("Selecting policy: $args{'-policyName'}");
        $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );
    }

    $Log->step("Selecting rule : $args{'-ruleName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_RULES_GRID, $args{'-ruleName'} );

    $Log->comment("Clicking export policy rule delete button");
    $self->click($EXPORT_RULES_DELETE_BUTTON);

    $Log->comment("Waiting for export policy rule delete Dialog");
    $self->wait_for_element_present( $EXPORT_RULES_DELETE_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Selecting OK button to complete.");
    $self->click($EXPORT_RULES_DELETE_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $EXPORT_RULES_DELETE_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("policy deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_rule_delete

################################################################################
## @name ::export_policy_refresh
## @summary
##     Refresh the export policy ruleusing GUI.
## @description
##     This method can be used to refresh the export policy rule using GUI.
## @synopsis
##    export_policy_refresh(%args);
## INPUTS TO METHOD CALL 'export_policy_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ruleName </td> <td> (Required) Name of the rule to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub export_policy_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $self->wait_for_element_present($EXPORT_POLICY_GRID);

    $Log->step("Click Refresh button");
    $self->click($EXPORT_REFRESH_BUTTON);
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub export_policy_refresh

################################################################################
## @name ::get_export_policy_info
## @summary
##      Get export policy information from GUI.
## @description
##     This method can be used to get export policy information from GUI.
## @synopsis
##    get_export_policy_info(%args);
## INPUTS TO METHOD CALL 'get_export_policy_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the rule to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Export Policy Status details.

sub get_export_policy_info {

    my %values;
    my $exportpolicy_info = {};

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $EXPORT_POLICY_GRID, $args{'-policyName'} );

    $Log->step("Getting row data for export policy from grid");
    my $rowdata = $self->get_row_data_with_text( $EXPORT_POLICY_GRID,
        $args{'-policyName'} );

    my @rowarray = split( ' ', $rowdata );
    %values = ( "$EXPORTPOLICY_INFO_POLICY_KEY" => $rowarray[0], );

    $exportpolicy_info->{parsed_output} = \%values;

    $Log->comment("Collection of Export Policy is done");
    $Log->exit() if $may_exit;

    return $exportpolicy_info;
} ## end sub get_export_policy_info

################################################################################
## @name ::get_export_policy_rule_info
## @summary
##      Get export policy rule information from GUI.
## @description
##     This method can be used to get export policy rule information from GUI.
## @synopsis
##    get_export_policy_rule_info(%args);
## INPUTS TO METHOD CALL 'get_export_policy_rule_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required) Name of the policy where rule exists </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -ruleName </td> <td> (Required) Name of the rule to be deleted </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub get_export_policy_rule_info {

    my %values;
    my $exportpolicyrule_info = {};

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-policyName" => { type => SCALAR },
            "-ruleName"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $EXPORT_POLICIES_TREE");
    $self->cluster_tree_navigate( $vserverName, $EXPORT_POLICIES_TREE );

    $Log->step("Selecting rule: $args{'-ruleName'}");
    $self->click_child_row_with_texts( $EXPORT_POLICY_GRID,
        $args{'-policyName'}, ["$args{'-ruleName'}"] );

    $Log->step("Getting row data for export policy rule from grid");
    my $rowdata = $self->get_row_data_with_text( $EXPORT_POLICY_GRID,
        $args{'-ruleName'} );

    my @rowarray = split( ' ', $rowdata );
    %values = (
        "$EXPORTPOLICYRULE_INFO_POLICY_KEY"     => $rowarray[0],
        "$EXPORTPOLICYRULE_INFO_CLIENTSPEC_KEY" => $rowarray[1],
        "$EXPORTPOLICYRULE_INFO_ACCESSPROT_KEY" => $rowarray[2],
        "$EXPORTPOLICYRULE_INFO_READONLY_KEY"   => $rowarray[3],
        "$EXPORTPOLICYRULE_INFO_READWRITE_KEY"  => $rowarray[4],
        "$EXPORTPOLICYRULE_INFO_SUPERUSER_KEY"  => $rowarray[5],
    );

    $exportpolicyrule_info->{parsed_output} = \%values;

    $Log->comment("Collection of Export Policy Rule is done");
    $Log->exit() if $may_exit;

    return $exportpolicyrule_info;

} ## end sub get_export_policy_rule_info

1;

