# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##    Routines to parse FCP and iSCSI GUI Functionalities.
##
## @description
##      Using this library, user can perform FCP and iSCSI  related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmFcIscsi
##
##       iscsiservice_edit(%args);
##		 iscsiservice(%args)
##       iscsiservice_refresh(%args);
##       iscsiinterface_status(%args);
##       get_iscsi_info(%args);
##       tpg_create(%args);
##       tpg_edit(%args);
##       tpg_delete(%args);
##       tpg_refresh(%args);
##       fcp_status(%args);
##       fcp_refresh(%args);
##       fcp_edit(%args);
##       fcp_info(%args);
##		 get_fcp_interface_info(%args);
##		 refresh_fcp(%args);
##       get_tpg_info(%args);
##
## @status private
##
## @author
##		Afsar syed . afsars@netapp.com
##
## @change
##		4/29/2010 : Afsar : Initial version
## @change
##		3/20/2012 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmFcIscsi;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF );
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    iscsiservice_edit
    iscsiservice
    iscsiservice_refresh
    iscsiinterface_status
    get_iscsi_info
    tpg_create
    tpg_edit
    tpg_delete
    tpg_refresh
    fcp_status
    fcp_edit
    fcp_info
    get_fcp_interface_info
    refresh_fcp
    get_tpg_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::iscsiservice_edit
## @summary
##     Edit service based on parameters provided.
## @description
##     This method can be used to edit service based on parameters provided.
## @synopsis
##     iscsiservice_edit(%args);
## INPUTS TO METHOD CALL 'iscsiservice_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -nodename </td> <td> Name of iSCSI node </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## <tr> <td> -alias </td> <td> Alias of iSCSI node </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub iscsiservice_edit {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$ISCSISERVICE_TAB");
    $self->wait_for_element_present( $ISCSISERVICE_EDIT_BUTTON,
        $GENERIC_WAIT );
    $self->click_button("$ISCSISERVICE_EDIT_BUTTON");
    $self->wait_for_loading();
    if ( exists( $args{'-nodename'} ) ) {
        $Log->comment(
            "*** Edit the name of iSCSI node :$args{'-nodename'}  ****");
        $self->type( "$ISCSISERVICE_EDIT_NODENAME_INPUT",
            $args{'-nodename'} );
    } ## end if ( exists( $args{'-nodename'...}))

    if ( exists( $args{'-alias'} ) ) {
        $Log->comment("Entering the iSCSI alias :$args{'-alias'}");
        $self->type_keys( $ISCSISERVICE_EDIT_ALIAS_INPUT, $args{'-alias'} );
        $self->type( "$ISCSISERVICE_EDIT_ALIAS_INPUT", "$args{'-alias'}" );
        $self->fire_event("$ISCSISERVICE_EDIT_ALIAS_INPUT","blur");
    }
    
    sleep(5);
    
    $self->click_button("$ISCSISERVICE_EDIT_OK_BUTTON");
    
    $self->wait_for_loading();
    if ($self->is_element_present($ISCSISERVICE_EDIT_CONFIRM_OK_BUTTON) ) {
        $Log->comment("Clicking OK button on Confirm modifying Target Node Name dialog");
        $self->click($ISCSISERVICE_EDIT_CONFIRM_OK_BUTTON);
    }
    
    $self->wait_for_element_not_present( $ISCSISERVICE_EDIT_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Edit iSCSI node is successfull");
    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub iscsiservice_edit

################################################################################
## @name ::iscsiservice
## @summary
##     Start and stop the service based on parameters provided.
## @description
##     This method can be used to start and stop the service based on parameters provided.
## @synopsis
##     iscsiservice(%args);
## INPUTS TO METHOD CALL 'iscsiservice' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -service </td> <td> (Required)Start and stop the iSCSI service</td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub iscsiservice {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-service' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$ISCSISERVICE_TAB");
    $self->wait_for_element_present( $ISCSISERVICE_START_BUTTON,
        $GENERIC_WAIT );

    if ( lc( $args{-service} ) eq 'start' ) {
        $self->click_button("$ISCSISERVICE_START_BUTTON");
    }

    if ( lc( $args{-service} ) eq 'stop' ) {
        $self->click_button("$ISCSISERVICE_STOP_BUTTON");
        $self->click_button("$ISCSISERVICE_STOP_CONF_BUTTON");
    }
    $self->check_for_error();
    $self->wait_for_element_not_present( $ISCSISERVICE_STOP_CONF_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("Edit iSCSI service is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub iscsiservice

################################################################################
## @name ::iscsiservice_refresh
## @summary
## 	   Refresh the iSCSI Service Page based on parameters provided.
## @description
##     This method can be used to refresh the iSCSI Service Page based on parameters provided.
## @synopsis
##     iscsiservice_refresh(%args);
## INPUTS TO METHOD CALL 'iscsiservice_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -service </td> <td> Start and stop the iSCSI service </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub iscsiservice_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$ISCSISERVICE_TAB");
    $self->wait_for_element_present( $ISCSISERVICE_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $self->click_button("$ISCSISERVICE_REFRESH_BUTTON");

    $self->wait_for_element_present( $ISCSISERVICE_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("iSCSI Service refresh is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub iscsiservice_refresh

################################################################################
## @name ::iscsiinterface_status
## @summary
## 	   Enable or disable the iSCSI interface based on parameters provided.
## @description
##     This method can be used to enable or disable the iSCSI interface based on parameters provided.
## @synopsis
##     iscsiinterface_status(%args);
## INPUTS TO METHOD CALL 'iscsiinterface_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -interfacename </td> <td> (Required)Name of the iSCSI interface </td> </tr>
## <tr> <td> -status </td> <td> (Required) Enable or disable the iSCSI service   </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub iscsiinterface_status {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-interfacename' => { type => SCALAR },
            "-status"        => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$ISCSISERVICE_TAB");
    $self->wait_for_element_present( $ISCSISERVICE_REFRESH_BUTTON,
        $GENERIC_WAIT );

    if ( exists( $args{'-interfacename'} ) ) {
        $Log->comment("Selecting $args{'-interfacename'} from the grid");
        $self->click_row_with_texts( "$ISCSIINTERFACE_GRID",
            $args{'-interfacename'} );
    }

    if ( lc( $args{'-status'} ) eq 'enable' ) {
        $Log->comment("Enabling the iSCSI interface");
        $self->click("$ISCSIINTERFACE_ENABLE_BUTTON");
        $self->wait_for_element_present( $ISCSIINTERFACE_CONFIRM_CHECKBOX,
            $GENERIC_WAIT );

        $self->set_checkbox( "$ISCSIINTERFACE_CONFIRM_CHECKBOX", "enable" );
        $self->wait_for_element_present( $ISCSIINTERFACE_CONFIRM_BUTTON,
            $GENERIC_WAIT );

        $self->click("$ISCSIINTERFACE_CONFIRM_BUTTON");
        $self->wait_for_element_present( $ISCSIINTERFACE_DISABLE_BUTTON,
            $GENERIC_WAIT );
    } ## end if ( lc( $args{'-status'...}))
    elsif ( lc( $args{'-status'} ) eq 'disable' ) {
        $Log->comment("Disabling the iSCSI interface");
        $self->click("$ISCSIINTERFACE_DISABLE_BUTTON");
        $self->wait_for_element_present( $ISCSIINTERFACE_CONFIRM_CHECKBOX,
            $GENERIC_WAIT );

        $self->set_checkbox( "$ISCSIINTERFACE_CONFIRM_CHECKBOX", "enable" );
        $self->wait_for_element_present( $ISCSIINTERFACE_CONFIRM_BUTTON,
            $GENERIC_WAIT );

        $self->click("$ISCSIINTERFACE_CONFIRM_BUTTON");
        $self->wait_for_element_present( $ISCSIINTERFACE_ENABLE_BUTTON,
            $GENERIC_WAIT );
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Valid inputs for '-status' are enable and disable only");
    }

    $Log->comment("Loading time and commit time are $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("iSCSI Interface status change is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub iscsiinterface_status

################################################################################
## @name ::get_iscsi_info
## @summary
## 	   This function will get iSCSI related information.
## @description
##     This method can be used to get iSCSI related information.
## @synopsis
##     get_iscsi_info(%args);
## INPUTS TO METHOD CALL 'get_iscsi_info ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Iscsi Status Details.

sub get_iscsi_info {
    my ( $self, %args ) = @_;
    my (%values);
    my $iscsi_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$ISCSISERVICE_TAB");
    $self->wait_for_element_present( $ISCSISERVICE_EDIT_BUTTON,
        $GENERIC_WAIT );

    $values{$ISCSISERVICE_INFO_SERVICE_KEY}
        = $self->get_text("$ISCSISERVICE_INFO_SERVICE_LABEL")
        if ( $self->is_element_present("$ISCSISERVICE_INFO_SERVICE_LABEL") );
    $values{$ISCSISERVICE_INFO_NODENAME_KEY}
        = $self->get_text("$ISCSISERVICE_INFO_NODENAME_LABEL")
        if ( $self->is_element_present("$ISCSISERVICE_INFO_NODENAME_LABEL") );
    $values{$ISCSISERVICE_INFO_ALIAS_KEY}
        = $self->get_text("$ISCSISERVICE_INFO_ALIAS_LABEL")
        if ( $self->is_element_present("$ISCSISERVICE_INFO_ALIAS_LABEL") );
    $values{$ISCSISERVICE_INFO_INTERFACE_KEY}
        = $self->get_text("$ISCSISERVICE_INFO_INTERFACE_GRID")
        if ( $self->is_element_present("$ISCSISERVICE_INFO_INTERFACE_GRID") );

    $Log->comment("Collection of iSCSI info is successfull");
    $iscsi_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $iscsi_info;

} ## end sub get_iscsi_info

################################################################################
## @name ::tpg_create
## @summary
## 	   This function will create FC topograpghy.
## @description
##     This method can be used to create FC topograpghy.
## @synopsis
##     tpg_create(%args);
## INPUTS TO METHOD CALL 'tpg_create ' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -tpgname </td> <td>(Required) Name of the topography to be created </td> </tr>
## <tr> <td> -tag </td> <td> Enter the tag number for TPG </td> </tr>
## <tr> <td> -interface </td> <td> Adding one interface for TPG </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub tpg_create {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-tpgname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$TPG_TAB");
    $self->wait_for_loading();

    $self->click_button("$TPG_CREATE_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Enter the name for TPG : $args{'-tpgname'}");
    $self->type( "$TPG_CREATE_NAME_INPUT", $args{'-tpgname'} );
    $self->fire_event( "$TPG_CREATE_NAME_INPUT", "blur" );

    if ( exists( $args{-tag} ) ) {
        $Log->comment("Eneter the tag number for TPG");
        $self->type( "$TPG_CREATE_TAG_INPUT", $args{-tag} );
        $self->fire_event( "$TPG_CREATE_TAG_INPUT", "blur" );
    }
    if ( exists( $args{-interface} ) ) {
        sleep(5);
        $Log->comment("Adding one interface for TPG :$args{-interface}");
        $self->set_checkbox_in_row_with_text( "$TPG_CREATE_INTERFACE_GRID",
            "enable", $args{-interface} );
    }
    $Log->comment("Enter OK for tpg create");
    $self->click_button("$TPG_CREATE_OK_BUTTON");

    $self->check_for_error();
    $self->wait_for_element_present( $TPG_CREATE_CONFORM_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Selecting confirmation checkbox");
    $self->set_checkbox( $TPG_CREATE_CONFORM_CHECKBOX, "enable" );

    $self->check_for_error();
    $self->wait_for_element_present( $TPG_CREATE_CONFORM_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Enter OK for tpg create");
    $self->click($TPG_CREATE_CONFORM_OK_BUTTON);
    $self->check_for_error();
    $self->wait_for_loading();
    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("TPG create is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub tpg_create

################################################################################
## @name ::tpg_edit
## @summary
## 	   This function will edit FC topograpghy parameters.
## @description
##     This method can be used to edit FC topograpghy parameters.
## @synopsis
##    tpg_edit(%args);
## INPUTS TO METHOD CALL 'tpg_edit' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -tpgname </td> <td> (Required) Name of the topography to be edited </td> </tr>
## <tr> <td> -interfaceadd </td> <td> Adding one interface for the FC topography  </td> </tr>
## <tr> <td> -interfaceremove </td> <td> Adding one interface for the FC topography  </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub tpg_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-tpgname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$TPG_TAB");
    $self->click("$TPG_REFRESH_BUTTON");
    $self->wait_for_loading();
    #$self->wait_for_element_present( $TPG_GRID, $GENERIC_WAIT );

    $Log->comment("*** Select TPG :$args{'-tpgname'} ***");
    $self->click_row_with_texts( "$TPG_GRID", $args{'-tpgname'} );

    $self->click_button("$TPG_EDIT_BUTTON");
    $self->wait_for_loading();
#    $self->wait_for_element_present("$TPG_EDIT_INTERFACE_GRID");

    # Remove interface
    if ( exists( $args{-interfaceremove} ) ) {
        $Log->comment("removing interface: $args{-interfaceremove}");
        $self->set_checkbox_in_row_with_text( "$TPG_EDIT_INTERFACE_GRID",
            "disable", $args{-interfaceremove} );
    }

    # Add interface
    if ( exists( $args{-interfaceadd} ) ) {
        $Log->comment("Adding interface: $args{-interfaceadd}");
        $self->set_checkbox_in_row_with_text( "$TPG_EDIT_INTERFACE_GRID",
            "enable", $args{-interfaceadd} );
    }

    $Log->comment("Enter OK for tpg edit");
    $self->click("$TPG_EDIT_OK_BUTTON");
    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_element_present( $TPG_EDIT_CONFORM_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Selecting confirmation checkbox");
    $self->set_checkbox( $TPG_EDIT_CONFORM_CHECKBOX, "enable" );

    $self->check_for_error();
    $self->wait_for_element_present( $TPG_EDIT_CONFORM_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking Edit confirm Button");
    $self->click($TPG_EDIT_CONFORM_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("TPG edit is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub tpg_edit

################################################################################
## @name ::tpg_delete
## @summary
## 	   This function will delete FC topograpghy parameters.
## @description
##     This method can be used to delete FC topograpghy parameters.
## @synopsis
##    tpg_delete(%args);
## INPUTS TO METHOD CALL 'tpg_delete' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -tpgarray </td> <td> (Required) Array of Names of the topograpes to be deleted</td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub tpg_delete {

    my ( $pack, $file, $line, $subname, $hasargs, $wantarray ) = caller(0);

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-tpgarray" => { type => ARRAYREF }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$TPG_TAB");
    $self->click("$TPG_REFRESH_BUTTON");
    $self->wait_for_element_present( $TPG_GRID, $GENERIC_WAIT );

    for my $count ( 0 .. $#{ $args{-tpgarray} } ) {
        my $tpg = $args{-tpgarray}[$count];
        $Log->step("Selecting TPG : $tpg");
        $self->click_row_with_texts( $TPG_GRID, $tpg );
        $self->control_key_down() if ( $count == 0 );
    } ## end for my $count ( 0 .. $#...)
    $self->control_key_up();

    $self->click_button("$TPG_DELETE_BUTTON");
    $self->wait_for_element_present("$TPG_DELETE_OK_BUTTON");

    $Log->comment("Selecting delete confirmation checkbox");
    $self->set_checkbox( $TPG_DELETE_CONFORM_CHECKBOX, "enable" );

    $Log->comment("Selcting oK button to delete TPG");
    $self->click_button("$TPG_DELETE_OK_BUTTON");
    $self->check_for_error();
    $self->wait_for_element_not_present( $TPG_EDIT_OK_BUTTON, $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("TPG delete is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub tpg_delete

################################################################################
## @name ::tpg_refresh
## @summary
## 	   This function will refresh FC topograpghy parameters.
## @description
##     This method can be used to refresh FC topograpghy parameters.
## @synopsis
##    tpg_refresh(%args);
## INPUTS TO METHOD CALL 'tpg_refresh' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub tpg_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $self->click("$TPG_TAB");
    $self->wait_for_element_present( $TPG_GRID, $GENERIC_WAIT );

    $self->click("$TPG_REFRESH_BUTTON");

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("TPG refresh is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub tpg_refresh

################################################################################
## @name ::fcp_status
## @summary
## 	   This function will set the FCP status to start or stop based on the input parameters.
## @description
##     This method can be used to set the FCP status to start or stop based on the input parameters.
## @synopsis
##    fcp_status(%args);
## INPUTS TO METHOD CALL 'fcp_status' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -status </td> <td> (Required) Select the status of FCP service </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub fcp_status {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-status" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>Fcp");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $FCP_TREE );
    $self->wait_for_element_present( $FCP_START_BUTTON, $GENERIC_WAIT );

    if ( lc( $args{-status} ) eq 'start' ) {
        $Log->comment("Selete the status of FCP service :$args{-status} ");
        $self->click_button("$FCP_START_BUTTON");
    }
    if ( lc( $args{-status} ) eq 'stop' ) {
        $Log->comment("Selete the status of FCP service :$args{-status}");
        $self->click_button("$FCP_STOP_BUTTON");

        $Log->comment("Selete the stop button");
        $self->click_button("$FCP_STOP_OK_BUTTON");
    } ## end if ( lc( $args{-status...}))
    $self->check_for_error();
    $self->wait_for_element_not_present( $FCP_STOP_OK_BUTTON, $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("FCP status change is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub fcp_status

################################################################################
## @name ::fcp_edit
## @summary
## 	   This function will edit FCP settings based on the parameters passed.
## @description
##     This method can be used to edit FCP settings based on the parameters passed.
##    fcp_edit(%args);
## INPUTS TO METHOD CALL 'fcp_edit' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -fcpname </td> <td> (Required) Name of the topography to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub fcp_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-fcpname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>Fcp");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $FCP_TREE );

    $self->wait_for_element_present("$FCP_EDIT_BUTTON");
    $self->click_button("$FCP_EDIT_BUTTON");
    $self->wait_for_element_present("$FCP_EDIT_NAME_INPUT");

    if ( exists( $args{-fcpname} ) ) {
        $Log->comment("Enter the given FCP name :$args{-fcpname} ");
        $self->type( "$FCP_EDIT_NAME_INPUT", "" );
        $self->type_keys( "$FCP_EDIT_NAME_INPUT", $args{-fcpname} );
        $self->fire_event( "$FCP_EDIT_NAME_INPUT", "blur" );
    } ## end if ( exists( $args{-fcpname...}))

    $Log->comment("Enter ok for FCP edit");
    $self->click_button("$FCP_EDIT_OK_BUTTON");
    $self->wait_for_loading();
    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_element_not_present( $FCP_EDIT_NAME_INPUT,
        $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("FCP edit is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub fcp_edit

################################################################################
## @name ::fcp_info
## @summary
## 	   This function will get all FCP related information.
## @description
##     This method can be used to get all FCP related information.
##    fcp_info(%args);
## INPUTS TO METHOD CALL 'fcp_info' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the FCP status details.

sub fcp_info {

    my ( $self, %args ) = @_;
    my %values;
    my $fcp_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>Fcp");
    $self->cluster_tree_navigate( $vserverName, $FCP_TREE );

    $self->wait_for_element_present("$FCP_EDIT_BUTTON");

    $Log->comment("Select the value for FC status ");
    $values{$FCP_INFO_STATUS_KEY} = $self->get_text("$FCP_INFO_STATUS_LABEL")
        if ( $self->is_element_present("$FCP_INFO_STATUS_LABEL") );

    $Log->comment("Select the value for WWNN number");
    $values{$FCP_INFO_WWNN_KEY} = $self->get_text("$FCP_INFO_WWNN_LABEL")
        if ( $self->is_element_present("$FCP_INFO_WWNN_LABEL") );

    $Log->comment("Select the value for FC status ");
    $values{$FCP_INFO_INTERFACE_KEY}
        = $self->get_text("$FCP_INFO_INTERFACE_TABLE")
        if ( $self->is_element_present("$FCP_INFO_INTERFACE_TABLE") );

    $Log->comment("Collection of FCP info is successfull");
    $fcp_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $fcp_info;

} ## end sub fcp_info

################################################################################
## @name ::get_fcp_interface_info
## @summary
## 	   This function will get all FCP interface related information.
## @description
##     This method can be used to get all FCP interface related information.
##    get_fcp_interface_info(%args);
## INPUTS TO METHOD CALL 'get_fcp_interface_info' ON APISET OBJECT </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -fc_interface </td> <td> (Required) fc_interface whose info is required </td> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the FCP Interface status Details.

sub get_fcp_interface_info {

    my %temp_fc_interface_info;
    my $fc_interface_info = {};

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-fc_interface" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to configuration>protocol>Fcp");
    $self->cluster_tree_navigate( $vserverName, $FCP_TREE );

    $Log->step("Selecting interface: $args{'-fc_interface'} from grid");
    $self->click_row_with_texts( $FC_INTERFACE_GRID, $args{'-fc_interface'} );

    $Log->step(
        "Getting row data for FC interface $args{'-fc_interface'} from grid");
    my $rowdata = $self->get_row_data_with_text( $FC_INTERFACE_GRID,
        $args{'-fc_interface'} );

    my @rowarray = split( ' ', $rowdata );
    %temp_fc_interface_info = (
        $FC_INFO_INTERFACE_KEY    => $rowarray[0],
        $FC_INFO_WWPN_KEY         => $rowarray[1],
        $FC_INFO_CURRENT_PORT_KEY => $rowarray[2],
        $FC_INFO_STATUS_KEY       => $rowarray[3],
    );

    foreach ( keys %temp_fc_interface_info ) {
        $Log->comment("The Info for $_ => $temp_fc_interface_info{$_}");
    }
    $Log->comment("Collection of FC interface info is successful");
    $fc_interface_info->{parsed_output} = \%temp_fc_interface_info;

    $Log->exit() if $may_exit;

    return $fc_interface_info;

} ## end sub get_fcp_interface_info

################################################################################
## @name ::refresh_fcp
## @summary
## 	   This function will refresh the FCP screen.
## @description
##     This method can be used to refresh the FCP screen.
##    refresh_fcp(%args);
## INPUTS TO METHOD CALL 'refresh_fcp' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of Vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub refresh_fcp {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to protocols>FCP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $FCP_TREE );
    $self->wait_for_element_present( $FCP_REFRESH_BUTTON, $GENERIC_WAIT );
    if ( not $self->is_button_enabled($FCP_REFRESH_BUTTON) ) {
        $self->wait_for_loading();
    }
    my $commit_time
        = $self->click_timecount( $FCP_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub refresh_fcp

################################################################################
# Subroutine Name: get_tpg_info
# Objective:       This function will return information regarding TPG interface.
# Details:         See POD documentation at the end of this file
################################################################################
################################################################################
## @name ::get_tpg_info
## @summary
##     This function will return information regarding TPG interface
## @description
##     This method will fetch tpg related information.
##    get_tpg_info(%args);
## INPUTS TO METHOD CALL 'get_tpg_info' ON APISET OBJECTP </td> </tr>
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -tpgname </td> <td> (Required) Name of TPG </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub get_tpg_info {

    my $self = shift;
    my %args = validate_with(params      => \@_,
                             spec        => {"-tpgname" => {type => SCALAR},},
                             allow_extra => 1);


    $Log->enter() if $may_enter;
    $Log->debug(sub { "Input Hash contains " . Dumper(\%args) });
    
    my $tpg_info;
    my $tpgname = $args{-tpgname};
    
    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'} ;
    }

    $Log->step("Navigating to configuration>protocol>iSCSI");
    $self->cluster_tree_navigate($vserverName,$ISCSI_TREE);

    $self->click("$TPG_TAB");
    $Log->step("Selecting TPG : $tpgname");
    $self->click_row_with_texts( $TPG_GRID, $tpgname );

    $Log->step("Getting row data for Iscsi Target Portal Group $tpgname from grid");
    my $rowdata = $self->get_row_data_with_text($TPG_GRID, $tpgname);

    my @rowarray = split(' ',$rowdata);
    my %temp_tpg_info;
    %temp_tpg_info = (
        $TARGET_PORTAL_GROUP_TPG_KEY  => $rowarray[0],
        $TARGET_PORTAL_GROUP_TYPE_KEY => $rowarray[1],
        $TARGET_PORTAL_GROUP_TAG_KEY  => $rowarray[2],
    );

    $Log->comment("Collection of FC interface info is successful");
    $tpg_info->{parsed_output} = \%temp_tpg_info;

    $Log->exit() if $may_exit;
    
    return $tpg_info;

} ## end sub get_tpg_info

1;
