# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Initiator GUI Functionalities
##
## @description
##       Using this library, user can perform cmode Initiator related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## synopsis:
##     use NACL::APISet::Host::GUI::SysManager::CMode::SmInitiator
##
##      create_igroup(%args);
##      edit_igroup(%args);
##      delete_igroup(%args);
##      get_igroup_info(%args);
##      edit_default_initiator_security(%args);
##      edit_initiator_security(%args);
##      refresh_initiator_security(%args);
##      add_initiator_security(%args);
##      get_initiator_default_security_info(%args);
##      clear_window_sm_initiator(%args);
##
## @status public
##
## @author afsars (afsars@netapp.com)
## @change
##  6/7/2010 : afsars : Initial version
## @change
##  3/20/2012 : vikasp : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmInitiator;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    create_igroup
    edit_igroup
    delete_igroup
    get_igroup_info
    edit_default_initiator_security
    edit_initiator_security
    refresh_initiator_security
    add_initiator_security
    get_initiator_default_security_info
    clear_window_sm_initiator
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::create_igroup
## @summary
##     Creation of igroup using GUI.
## @description
##     This method can be used to create igroup based on paramteres provided using GUI.
## @synopsis
##     create_igroup (%args);
## INPUTS TO METHOD CALL 'create_igroup' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -igroupname </td> <td> (Required)Name of the igroup to be created </td> </tr>
## <tr> <td> -os </td> <td> (Required) OS type to be used </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -type </td> <td> Selecting type based on input parameter </td> </tr>
## <tr> <td> -alua </td> <td> Enabling alua based on input parameter </td> </tr>
## <tr> <td> -portset </td> <td> Enabling alua based on input parameter </td> </tr>
## <tr> <td> -initiators </td> <td> Enabling initiators tab  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub create_igroup {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-igroupname' => { type => SCALAR },
            "-os"         => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on Initiator Groups Tab.");
    my $loading_time
        = $self->click_timecount( $INITIATOR_GROUPS_TAB, "refresh" );

    $Log->step("Clicking on Refresh buton");
    $self->click("$INITIATOR_GROUPS_REFRESH_BUTTON");
    $self->wait_for_loading();
    
    $Log->comment("Clicking igroup Create button");
    $self->click($INITIATOR_GROUPS_CREATE_BUTTON);
    $self->wait_for_loading();
     
    $Log->comment("Clicking General Tab in Create intiator group Wizard");
    $self->click($INITIATOR_GROUPS_CREATE_GENERAL_TAB);

    $Log->comment("Entering igroup name: $args{'-igroupname'}");
    $self->type_keys( $INITIATOR_GROUPS_ADD_NAME_INPUT,
        $args{'-igroupname'} );
    $self->type( $INITIATOR_GROUPS_ADD_NAME_INPUT,
        $args{'-igroupname'} );

    my $os = $args{'-os'};
    if ($args{-os} =~ /aix/i){
        $os = "AIX";
    } elsif ($args{-os} =~ /hp-ux/i){
        $os = "HP-UX";
    } elsif ($args{-os} =~ /hyper-v/i){
        $os = "Hyper-V";
    } elsif ($args{-os} =~ /linux/i){
        $os = "Linux";
    } elsif ($args{-os} =~ /netware/i){
        $os = "NetWare";
    } elsif ($args{-os} =~ /openvms/i){
        $os = "OpenVMS";
    } elsif ($args{-os} =~ /solaris/i){
        $os = "Solaris";
    } elsif ($args{-os} =~ /vmware/i){
        $os = "VMware";
    } elsif ($args{-os} =~ /windows/i){
        $os = "Windows";
    }elsif ($args{-os} =~ /xen/i){
        $os = "Xen";
    }

    $Log->comment("Selecting OS : $os");
    $self->set_combobox_value( $INITIATOR_GROUPS_ADD_OSTYPE_COMBOBOX,
        $os );

    # Selecting type based on input parameter
    if ( exists( $args{'-type'} ) ) {
        $Log->comment("Selecting type: $args{'-type'}");
        if ( lc( $args{'-type'} eq "iscsi" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_ISCSI_OPTION);
        } elsif ( lc( $args{'-type'} eq "fcp" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_FCP_OPTION);

            # Enabling ALUA based on input parameter
            if ( exists( $args{'-alua'} ) ) {
                $Log->comment( $args{'-alua'} . "ing alua" );
                $self->set_checkbox( $INITIATOR_GROUPS_ADD_ALUA_CHECKBOX,
                    $args{'-alua'} );
            }
        } elsif ( lc( $args{'-type'} eq "mixed" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_MIXED_OPTION);
        } else {
            NATE::BaseException->throw(
                "Invalid value for type, it should be either iscsi or fcp or mixed"
            );
        }
    } ## end if ( exists( $args{'-type'...}))

    if ( exists( $args{'-portset'} ) ) {
        $Log->comment("entering portset: $args{'-portset'}");
        $self->type( "$LUN_EDIT_PORTSET_INPUT", $args{-portset} );
    }

    $Log->comment("Clicking Intiators tab In Create Wizard");
    $self->click($INITIATOR_GROUPS_CREATE_INITIATORS_TAB);
    $self->wait_for_loading();
    
    if ( exists( $args{'-initiators'} ) ) {
        for my $count ( 0 .. scalar( @{ $args{-initiators} } ) - 1 ) {
            my $initiator = $args{-initiators}[$count];
            $Log->comment("Clicking ADD button to add initiator: $initiator");
            $self->click($INITIATOR_GROUPS_INIT_ADD_BUTTON);
            $self->wait_for_loading();
            
            $Log->comment("Entering initiator: $initiator");
            $self->type( $INITIATOR_GROUPS_INIT_INPUT, $initiator );
            $Log->comment("Clicking OK button");
            $self->click($INITIATOR_GROUPS_INIT_OK_BUTTON);
            $Log->comment("Initiator added successfully");
        } ## end for my $count ( 0 .. scalar...)
    } ## end if ( exists( $args{'-initiators'...}))

    $Log->comment("Clicking Create button to complete.");
    my $commit_time = $self->click_timecount($INITIATOR_GROUPS_OK_BUTTON);
    
    $self->wait_for_loading(); 
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("igroup created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub create_igroup

################################################################################
## @name ::edit_igroup
## @summary
##     Editing igroup using GUI.
## @description
##     This method can be used to edit igroup based on paramteres provided using GUI.
## @synopsis
##     edit_igroup(%args);
## INPUTS TO METHOD CALL 'edit_igroup' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -igroupname </td> <td> (Required)Name of the igroup to be created </td> </tr>
## <tr> <td> -newigroupname </td> <td> Name of the igroup to be created </td> </tr>
## <tr> <td> -os </td> <td> (Required) OS type to be used </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -type </td> <td> Selecting type based on input parameter </td> </tr>
## <tr> <td> -alua </td> <td> Enabling alua based on input parameter </td> </tr>
## <tr> <td> -addinitiators </td> <td> Adding initiators to the igroup  </td> </tr>
## <tr> <td> -deleteinitiators </td> <td> Deleting initiators from the igroup  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub edit_igroup {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-igroupname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on Initiator Groups Tab.");
    my $loading_time
        = $self->click_timecount( $INITIATOR_GROUPS_TAB, "refresh" );

    $Log->step("Clicking on Refresh buton");
    $self->click("$INITIATOR_GROUPS_REFRESH_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Selecting igroup: $args{'-igroupname'} from Grid");
    $self->click_row_with_texts( $INITIATOR_GROUPS_GRID,
        $args{'-igroupname'} );

    $Log->step("Waiting for Edit button.");
    $self->wait_for_element_present( $INITIATOR_GROUPS_EDIT_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Clicking Edit button.");
    $self->click($INITIATOR_GROUPS_EDIT_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Waiting for iGroup edit Dialog");
   # $self->wait_for_element_present( $INITIATOR_GROUPS_EDIT_NAME_INPUT,
    #    $GENERIC_WAIT );

    if ( exists( $args{'-newigroupname'} ) ) {
        $Log->comment("Entering igroup name: $args{'-newigroupname'}");
        $self->type( $INITIATOR_GROUPS_EDIT_NAME_INPUT,
            $args{'-newigroupname'} );
    }

    if ( exists( $args{'-os'} ) ) {
        my $os = $args{'-os'};
        if ($args{-os} =~ /aix/i){
            $os = "AIX";
        } elsif ($args{-os} =~ /hp-ux/i){
            $os = "HP-UX";
        } elsif ($args{-os} =~ /hyper-v/i){
            $os = "Hyper-V";
        } elsif ($args{-os} =~ /linux/i){
            $os = "Linux";
        } elsif ($args{-os} =~ /netware/i){
            $os = "NetWare";
        } elsif ($args{-os} =~ /openvms/i){
            $os = "OpenVMS";
        } elsif ($args{-os} =~ /solaris/i){
            $os = "Solaris";
        } elsif ($args{-os} =~ /vmware/i){
            $os = "VMware";
        } elsif ($args{-os} =~ /windows/i){
            $os = "Windows";
        }elsif ($args{-os} =~ /xen/i){
            $os = "Xen";
        }
        
        $Log->comment("Selecting OS : $os");
        $self->set_combobox_value( $INITIATOR_GROUPS_EDIT_OSTYPE_COMBOBOX,
            $os );
    }

    if ( exists( $args{'-alua'} ) ) {
        $Log->comment( $args{'-alua'} . "ing alua" );
        $self->set_checkbox( $INITIATOR_GROUPS_EDIT_ALUA_CHECKBOX,
            $args{'-alua'} );
    }

    $Log->comment("Clicking tab - \"Initiators\"");
    $self->click($INITIATOR_GROUPS_CREATE_INITIATORS_TAB);

    if ( exists( $args{'-addinitiators'} ) ) {
        for my $count ( 0 .. $#{ $args{-addinitiators} } ) {
            my $initiator = $args{-addinitiators}[$count];
            $Log->comment("Clicking ADD button to add initiator: $initiator");
            $self->click($INITIATOR_GROUPS_INIT_ADD_BUTTON);
            $self->wait_for_element_present( $INITIATOR_GROUPS_INIT_INPUT,
                $GENERIC_WAIT );
            $Log->comment("Entering initiator: $initiator");
            $self->type( $INITIATOR_GROUPS_INIT_INPUT, $initiator );
            $Log->comment("Clicking OK button");
            $self->click($INITIATOR_GROUPS_INIT_OK_BUTTON);
            $Log->comment("Initiator added successfully");
        } ## end for my $count ( 0 .. $#...)
    } ## end if ( exists( $args{'-addinitiators'...}))

    if ( exists( $args{'-deleteinitiators'} ) ) {
        for my $count ( 0 .. $#{ $args{-deleteinitiators} } ) {
            my $initiator = $args{-deleteinitiators}[$count];
            $Log->comment("Selecting initiator:$initiator from grid");
            if (not $self->does_row_contain_texts(
                    $INITIATOR_GROUPS_INIT_GRID, $initiator
                )
                )
            {
                $Log->comment("Initiator \"$initiator\" is not a Member");
                next;
            } ## end if ( not $self->does_row_contain_texts...)

            $self->click_row_with_texts( $INITIATOR_GROUPS_INIT_GRID,
                $initiator );
            $Log->comment("Clicking Delete button");
            $self->click($INITIATOR_GROUPS_INIT_DEL_BUTTON);
            $Log->comment("Initiator deleted successfully");
        } ## end for my $count ( 0 .. $#...)
    } ## end if ( exists( $args{'-deleteinitiators'...}))

    $Log->comment("Clicking OK button to complete.");
    $self->click($INITIATOR_GROUPS_OK_BUTTON);
    
    $self->wait_for_loading();
    if($self->is_element_present($INITIATOR_GROUPS_MODIFY_OS_OK_BUTTON)) {
        $Log->comment("Clicking on OK button on Modify Initiator Group Operating System dialog");
        $self->click($INITIATOR_GROUPS_MODIFY_OS_OK_BUTTON);
    }
    
    $self->wait_for_loading();

    $Log->comment(
        "Loading time and commit time are $loading_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
    );

    $Log->comment("igroup edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub edit_igroup

################################################################################
## @name ::delete_igroup
## @summary
##     Deleting igroup using GUI.
## @description
##     This method can be used to delete igroup based on paramteres provided using GUI.
## @synopsis
##     delete_igroup(%args);
## INPUTS TO METHOD CALL 'delete_igroup' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -igroupname </td> <td> (Required)Name of the igroup to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub delete_igroup {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-igroupname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on Initiator Groups Tab.");
    my $loading_time
        = $self->click_timecount( $INITIATOR_GROUPS_TAB, "refresh" );

    $Log->step("Clicking on Refresh buton");
    $self->click("$INITIATOR_GROUPS_REFRESH_BUTTON");

#    $Log->comment("Selecting igroup: $args{'-igroupname'} from Grid");
#    $self->click_row_with_texts( $INITIATOR_GROUPS_GRID,
#        $args{'-igroupname'} );

    if( $args{-igroupname} ){
        my @igroups = split( /:/,$args{-igroupname});
        for my $count ( 0 .. $#igroups ) {
            my $igroupname = $igroups[$count];
            $Log->step("Selecting igroup : $igroupname");
            $self->click_row_with_texts( $INITIATOR_GROUPS_GRID, $igroupname );
            $self->control_key_down() if ( $count == 0 );
        } ## end for my $count ( 0 .. $#...)
    }
    $self->control_key_up();

    $Log->step("Clicking Delete button.");
    $self->click($INITIATOR_GROUPS_DELETE_BUTTON);

    $Log->comment("Waiting for igroup Delete confirmation dialog");
    $self->wait_for_element_present(
        $INITIATOR_GROUPS_DELETE_CONFIRM_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Click Delete button");
    my $commit_time
        = $self->click_timecount($INITIATOR_GROUPS_DELETE_CONFIRM_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->step("Igroup Deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub delete_igroup

################################################################################
## @name ::get_igroup_info
## @summary
##     Get information related to igroup using GUI.
## @description
##     This method can be used to get information related to igroup using GUI.
## @synopsis
##     get_igroup_info(%args);
## INPUTS TO METHOD CALL 'get_igroup_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -igroupname </td> <td> (Required)Name of the igroup to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Initiator Group status details.

sub get_igroup_info {

    my %values;
    my $igroup_info = {};

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-igroupname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on Initiator Groups Tab.");
    $self->click($INITIATOR_GROUPS_TAB);
    $self->wait_for_loading();

    $Log->step("Clicking on Refresh buton");
    $self->click("$INITIATOR_GROUPS_REFRESH_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Selecting igroup: $args{'-igroupname'} from Grid");
    $self->click_row_with_texts( $INITIATOR_GROUPS_GRID,
        $args{'-igroupname'} );

    my $rowdata = $self->get_row_data_with_text( "$INITIATOR_GROUPS_GRID", $args{'-igroupname'} );
    my @rowarray = split( ' ', $rowdata );

    if ($rowarray[1] =~ /mixed/i) {
        %values = (
            "name"      => $rowarray[0],
            "type"      => $rowarray[1] . " " . $rowarray[2] . " " . $rowarray[3] . " " . $rowarray[4],
            "os"        => $rowarray[5],
            "portset"   => $rowarray[6],
            "initcount" => $rowarray[7],
        );
    }
    else { 
        %values = (
            "name"      => $rowarray[0],
            "type"      => $rowarray[1],
            "os"        => $rowarray[2],
            "portset"   => $rowarray[3],
            "initcount" => $rowarray[4],
        );
    }
    
    $Log->comment("Fetching initiators list");
    $values{$INITIATOR_GROUPS_INITIATORS_KEY}
        = $self->get_text($INITIATOR_GROUPS_INITIATORS_LIST);

    $Log->comment("Collection of igroup info is successfull");
    $igroup_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    return $igroup_info;

} ## end sub get_igroup_info

################################################################################
## @name ::edit_default_initiator_security
## @summary
##      Edit default initiator security using GUI.
## @description
##     This method can be used to edit default initiator security using GUI.
## @synopsis
##     edit_default_initiator_security(%args);
## INPUTS TO METHOD CALL 'edit_default_initiator_security' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -securitytype </td> <td> Security type for the initiator </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -inboundusername </td> <td> Name of the inbound user </td> </tr>
## <tr> <td> -inboundpassword </td> <td> Password for the inbound user </td> </tr>
## <tr> <td> -inboundconfirmpassword </td> <td> Confirm password for the inbound user </td> </tr>
## <tr> <td> -outboundusername </td> <td> Name of the outbound user </td> </tr>
## <tr> <td> -outboundpassword </td> <td> Password for the outbound user </td> </tr>
## <tr> <td> -outboundconfirmpassword </td> <td> Confirm password for the outbound user </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub edit_default_initiator_security {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $ISCSI_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $Log->step("Clicking on Initiator Security Tab.");
    $self->click($INITIATOR_SECURITY_TAB);
    $self->wait_for_loading();

    $Log->step("Clicking Edit button.");
    $self->click_button($INITIATOR_SECURITY_DEFAULT_SECURITY_EDIT_BUTTON);

    $Log->comment("Waiting for edit default security dialog");
    $self->wait_for_element_present( $INITIATOR_SECURITY_EDIT_NONE_OPTION,
        $GENERIC_WAIT );

    if ( exists( $args{'-securitytype'} ) ) {
        if ( $args{'-securitytype'} eq "none" ) {
            $Log->step("Selecting none option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_NONE_OPTION);
        } elsif ( $args{'-securitytype'} eq "deny" ) {
            $Log->step("Selecting deny option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_DENY_OPTION);
        } elsif ( $args{'-securitytype'} eq "chap" ) {
            $Log->step("Selecting chap option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_CHAP_OPTION);

            if (   exists $args{'-inboundusername'}
                && exists $args{'-inboundpassword'} )
            {
                $Log->step(
                    "Entering Inbound username:$args{'-inboundusername'}");
                $self->type( $INITIATOR_SECURITY_EDIT_INBOUND_UNAME_INPUT,
                    $args{'-inboundusername'} );
                $self->fire_event(
                    "$INITIATOR_SECURITY_EDIT_INBOUND_UNAME_INPUT", "blur" );

                $Log->step(
                    "Entering Inbound Password:$args{'-inboundpassword'} .");
                $self->type( $INITIATOR_SECURITY_EDIT_INBOUND_PASSWORD_INPUT,
                    $args{'-inboundpassword'} );
                $self->fire_event(
                    "$INITIATOR_SECURITY_EDIT_INBOUND_PASSWORD_INPUT",
                    "blur" );

                if ( exists $args{'-inboundconfirmpassword'} ) {
                    $Log->step(
                        "Entering confirm Password:$args{'-inboundconfirmpassword'}"
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_INBOUND_CONFPASS_INPUT,
                        $args{'-inboundconfirmpassword'} );
                    $self->fire_event(
                        "$INITIATOR_SECURITY_EDIT_INBOUND_CONFPASS_INPUT",
                        "blur" );
                } else {
                    $Log->step(
                        "Entering confirm Password:$args{'-inboundpassword'} ."
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_INBOUND_CONFPASS_INPUT,
                        $args{'-inboundpassword'} );
                    $self->fire_event(
                        "$INITIATOR_SECURITY_EDIT_OUTBOUND_UNAME_INPUT",
                        "blur" );
                } ## end else [ if ( exists $args{'-inboundconfirmpassword'...})]

            } else {
                NATE::BaseException->throw(
                    "Inbound Username & Password are mandatory for CHAP");
            }

            if ( exists $args{'-outboundusername'} ) {
                $Log->step("Clicking on Outbound Tab.");
                $self->click($INITIATOR_SECURITY_EDIT_OUTBOUND_TAB);

                $Log->step(
                    "Entering Outbound username:$args{'-outboundusername'}");
                $self->type( $INITIATOR_SECURITY_EDIT_OUTBOUND_UNAME_INPUT,
                    $args{'-outboundusername'} );
                $self->fire_event(
                    "$INITIATOR_SECURITY_EDIT_OUTBOUND_UNAME_INPUT", "blur" );

                if ( exists $args{'-outboundpassword'} ) {

                    $Log->step(
                        "Entering Outbound Password:$args{'-outboundpassword'} ."
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_OUTBOUND_PASSWORD_INPUT,
                        $args{'-outboundpassword'} );
                    $self->fire_event(
                        "$INITIATOR_SECURITY_EDIT_OUTBOUND_PASSWORD_INPUT",
                        "blur" );

                    if ( exists $args{'-outboundconfirmpassword'} ) {
                        $Log->step(
                            "Entering confirm Password:$args{'-outboundconfirmpassword'} "
                        );
                        $self->type(
                            $INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT,
                            $args{'-outboundconfirmpassword'}
                        );
                        $self->fire_event(
                            "$INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT",
                            "blur"
                        );
                    } else {
                        $Log->step(
                            "Entering confirm Password:$args{'-outboundpassword'} "
                        );
                        $self->type(
                            $INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT,
                            $args{'-outboundpassword'}
                        );
                        $self->fire_event(
                            "$INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT",
                            "blur"
                        );
                    } ## end else [ if ( exists $args{'-outboundconfirmpassword'...})]
                } else {
                    NATE::BaseException->throw(
                        "Password missing for outbound CHAP");
                }
            } ## end if ( exists $args{'-outboundusername'...})
        } ## end elsif ( $args{'-securitytype'...})
    } ## end if ( exists( $args{'-securitytype'...}))

    $Log->step("Click OK button to complete");
    $self->click($INITIATOR_SECURITY_EDIT_OK_BUTTON);

    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_element_not_present( $INITIATOR_SECURITY_EDIT_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Initiator default security edited sucessfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub edit_default_initiator_security

################################################################################
## @name ::edit_initiator_security
## @summary
##      Edit initiator security using GUI.
## @description
##     This method can be used to edit initiator security using GUI.
## @synopsis
##     edit_initiator_security(%args);
## INPUTS TO METHOD CALL 'edit_initiator_security' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -initiator </td> <td> (Required) Initiator security to be edit</td> </tr>
## <tr> <td> -securitytype </td> <td> Security type for the initiator </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -inboundusername </td> <td> Name of the inbound user </td> </tr>
## <tr> <td> -inboundpassword </td> <td> Password for the inbound user </td> </tr>
## <tr> <td> -inboundconfirmpassword </td> <td> Confirm password for the inbound user </td> </tr>
## <tr> <td> -outboundusername </td> <td> Name of the outbound user </td> </tr>
## <tr> <td> -outboundpassword </td> <td> Password for the outbound user </td> </tr>
## <tr> <td> -outboundconfirmpassword </td> <td> Confirm password for the outbound user </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub edit_initiator_security {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-initiator" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $ISCSI_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $Log->step("Clicking on Initiator Security Tab.");
    $self->click($INITIATOR_SECURITY_TAB);
    $self->wait_for_loading();

    $Log->comment("Selecting initiator: $args{'-initiator'} from Grid");
    $self->click_row_with_texts( $INITIATOR_SECURITY_GRID, 1, 0,
        $args{'-initiator'} );

    $Log->step("Clicking Edit button.");
    $self->click($INITIATOR_SECURITY_EDIT_BUTTON);

    $self->wait_for_loading();
    $Log->comment("Waiting for edit default security dialog");
    $self->wait_for_element_present( $INITIATOR_SECURITY_EDIT_NONE_OPTION,
        $GENERIC_WAIT );

    if ( exists( $args{'-securitytype'} ) ) {
        if ( $args{'-securitytype'} eq "default" ) {
            $Log->step("Selecting default option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_DEFAULT_OPTION);
        } elsif ( $args{'-securitytype'} eq "none" ) {
            $Log->step("Selecting none option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_NONE_OPTION);
        } elsif ( $args{'-securitytype'} eq "deny" ) {
            $Log->step("Selecting deny option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_DENY_OPTION);
        } elsif ( $args{'-securitytype'} eq "chap" ) {
            $Log->step("Selecting chap option.");
            $self->set_option($INITIATOR_SECURITY_EDIT_CHAP_OPTION);

            if (   exists $args{'-inboundusername'}
                && exists $args{'-inboundpassword'} )
            {
                $Log->step(
                    "Entering Inbound username:$args{'-inboundusername'}");
                $self->type( $INITIATOR_SECURITY_EDIT_INBOUND_UNAME_INPUT,
                    $args{'-inboundusername'} );

                $Log->step(
                    "Entering Inbound Password:$args{'-inboundpassword'} .");
                $self->type( $INITIATOR_SECURITY_EDIT_INBOUND_PASSWORD_INPUT,
                    $args{'-inboundpassword'} );

                if ( exists $args{'-inboundconfirmpassword'} ) {
                    $Log->step(
                        "Entering confirm Password:$args{'-inboundconfirmpassword'} "
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_INBOUND_CONFPASS_INPUT,
                        $args{'-inboundconfirmpassword'} );
                } else {
                    $Log->step(
                        "Entering confirm Password:$args{'-inboundpassword'} ."
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_INBOUND_CONFPASS_INPUT,
                        $args{'-inboundpassword'} );
                } ## end else [ if ( exists $args{'-inboundconfirmpassword'...})]

            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Inbound Username & Password are mandatory for CHAP");
            }

            if ( exists $args{'-outboundusername'} ) {
                $Log->step("Clicking on Outbound Tab.");
                $self->click($INITIATOR_SECURITY_EDIT_OUTBOUND_TAB);
                $self->wait_for_loading();
                $Log->step(
                    "Entering Outbound username:$args{'-outboundusername'}");
                $self->type( $INITIATOR_SECURITY_EDIT_OUTBOUND_UNAME_INPUT,
                    $args{'-outboundusername'} );

                if ( exists $args{'-outboundpassword'} ) {
                    $Log->step(
                        "Entering Outbound Password:$args{'-outboundpassword'} ."
                    );
                    $self->type(
                        $INITIATOR_SECURITY_EDIT_OUTBOUND_PASSWORD_INPUT,
                        $args{'-outboundpassword'} );

                    if ( exists $args{'-outboundconfirmpassword'} ) {
                        $Log->step(
                            "Entering confirm Password:$args{'-outboundconfirmpassword'} "
                        );
                        $self->type(
                            $INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT,
                            $args{'-outboundconfirmpassword'}
                        );
                    } else {
                        $Log->step(
                            "Entering confirm Password:$args{'-outboundpassword'} "
                        );
                        $self->type(
                            $INITIATOR_SECURITY_EDIT_OUTBOUND_CONFPASS_INPUT,
                            $args{'-outboundpassword'}
                        );
                    } ## end else [ if ( exists $args{'-outboundconfirmpassword'...})]
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "Password missing for outbound CHAP");
                }
            } ## end if ( exists $args{'-outboundusername'...})
        } ## end elsif ( $args{'-securitytype'...})
    } ## end if ( exists( $args{'-securitytype'...}))

    $Log->step("Click OK button to complete");
    $self->click($INITIATOR_SECURITY_EDIT_OK_BUTTON);

    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_element_not_present( $INITIATOR_SECURITY_EDIT_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Initiator security edited sucessfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub edit_initiator_security

################################################################################
## @name ::refresh_initiator_security
## @summary
##      Refresh initiator security using GUI.
## @description
##     This method can be used to refresh initiator security using GUI.
## @synopsis
##     refresh_initiator_security(%args);
## INPUTS TO METHOD CALL 'refresh_initiator_security' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub refresh_initiator_security {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $ISCSI_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $Log->step("Clicking on Initiator Security Tab.");
    $self->click($INITIATOR_SECURITY_TAB);
    $self->wait_for_element_present( $INITIATOR_SECURITY_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Clicking on Initiator Security Refresh button");
    $self->click($INITIATOR_SECURITY_REFRESH_BUTTON);
    $self->wait_for_element_present( $INITIATOR_SECURITY_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Initiator security page refreshed sucessfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub refresh_initiator_security

################################################################################
## @name ::add_initiator_security
## @summary
##      Add initiator security using GUI.
## @description
##     This method can be used to add initiator security using GUI.
## @synopsis
##     add_initiator_security(%args);
## INPUTS TO METHOD CALL 'add_initiator_security' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -initiator </td> <td> (Required) Initiator security to be added</td> </tr>
## <tr> <td> -securitytype </td> <td> Security type for the initiator </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -inboundusername </td> <td> Name of the inbound user </td> </tr>
## <tr> <td> -inboundpassword </td> <td> Password for the inbound user </td> </tr>
## <tr> <td> -inboundconfirmpassword </td> <td> Confirm password for the inbound user </td> </tr>
## <tr> <td> -outboundusername </td> <td> Name of the outbound user </td> </tr>
## <tr> <td> -outboundpassword </td> <td> Password for the outbound user </td> </tr>
## <tr> <td> -outboundconfirmpassword </td> <td> Confirm password for the outbound user </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub add_initiator_security {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-initiator" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $ISCSI_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $Log->comment("Clicking on Initiator Security Tab.");
    $self->click($INITIATOR_SECURITY_TAB);
    $self->wait_for_element_present( $INITIATOR_SECURITY_ADD_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on Initiator Security Add button");
    $self->click($INITIATOR_SECURITY_ADD_BUTTON);
    $self->wait_for_element_present( $INITIATOR_SECURITY_INITIATOR_INPUT,
        $GENERIC_WAIT );

    $Log->comment("Entering the initiator: $args{'-initiator'}");
    $self->type("$INITIATOR_SECURITY_INITIATOR_INPUT", $args{'-initiator'});

    if ( exists( $args{-securitytype} ) ) {
        if ( lc( $args{-securitytype} ) eq 'none' ) {
            $Log->comment("Selection None Radio Button");
            $self->set_option( "$INITIATOR_SECURITY_NONE_RADIO", "enable" );
        } elsif ( lc( $args{-securitytype} ) eq 'deny' ) {
            $Log->comment("Selection Deny Radio Button");
            $self->set_option( "$INITIATOR_SECURITY_DENY_RADIO", "enable" );
        } elsif ( lc( $args{-securitytype} ) eq 'chap' ) {
            $Log->comment("Selection Chap Radio Button");
            $self->set_option( "$INITIATOR_SECURITY_CHAP_RADIO", "enable" );

            if ( lc( $args{-chaptype} ) eq 'inbound' ) {
                $Log->comment("Clicking on Inbound Tab");
                $self->click("$INITIATOR_SECURITY_INBOUND_TAB");
                $self->wait_for_element_present(
                    $INITIATOR_SECURITY_INBOUND_USERNAME_INPUT,
                    $GENERIC_WAIT );

                if ( exists( $args{-username} ) ) {
                    $Log->comment("Entering Username");
                    $self->type( "$INITIATOR_SECURITY_INBOUND_USERNAME_INPUT",
                        $args{-username} );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-username is a mandatory parameter");
                }
                if ( exists( $args{-password} ) ) {
                    $Log->comment("Entering Password");
                    $self->type( "$INITIATOR_SECURITY_INBOUND_PASSWORD_INPUT",
                        $args{-password} );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-password is a mandatory parameter");
                }

                if ( exists( $args{-confirmpassword} ) ) {
                    $Log->comment("Entering Confirm Password");
                    $self->type(
                        "$INITIATOR_SECURITY_INBOUND_CONFIRMPASSWORD_INPUT",
                        $args{-confirmpassword} );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-confirmpassword is a mandatory parameter");
                }
            } elsif ( lc( $args{-chaptype} ) eq 'outbound' ) {
                $Log->comment("Clicking on Outbound Tab");
                $self->click("$INITIATOR_SECURITY_OUTBOUND_TAB");
                $self->wait_for_element_present(
                    $INITIATOR_SECURITY_OUTBOUND_USERNAME_INPUT,
                    $GENERIC_WAIT );

                if ( exists( $args{-username} ) ) {
                    $Log->comment("Entering Username");
                    $self->type(
                        "$INITIATOR_SECURITY_OUTBOUND_USERNAME_INPUT",
                        $args{-username} );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-username is a mandatory parameter");
                }
                if ( exists( $args{-password} ) ) {
                    $Log->comment("Entering Password");
                    $self->type(
                        "$INITIATOR_SECURITY_OUTBOUND_PASSWORD_INPUT",
                        $args{-password} );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-password is a mandatory parameter");
                }

                if ( exists( $args{-confirmpassword} ) ) {
                    $Log->comment("Entering Confirm Password");
                    $self->type(
                        "$INITIATOR_SECURITY_OUTBOUND_CONFIRMPASSWORD_INPUT",
                        $args{-confirmpassword}
                    );
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "-confirmpassword is a mandatory parameter");
                }
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Invalid : values for -chaptype can be inbound or outbound"
                );
            } ## end else [ if ( lc( $args{-chaptype...}))]
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid: valid values for '-securitystyle' are none, deny and chap"
            );
        } ## end else [ if ( lc( $args{-securitytype...}))]
    } ## end if ( exists( $args{-securitytype...}))

    $Log->comment("Clicking on the Ok Button");
    $self->click("$INITIATOR_SECURITY_OK_BUTTON");
    $self->wait_for_element_present( $INITIATOR_SECURITY_ADD_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Initiator security added sucessfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub add_initiator_security

################################################################################
## @name ::get_initiator_default_security_info
## @summary
##     Fetch initiator default security details using GUI.
## @description
##     This method can be used to add initiator security using GUI.
## @synopsis
##     add_initiator_security(%args);
## INPUTS TO METHOD CALL 'add_initiator_security' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Initiator Default Status details.

sub get_initiator_default_security_info {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my %values;
    my $security_info = {};

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $ISCSI_TREE");
    $self->cluster_tree_navigate( $vserverName, $ISCSI_TREE );

    $Log->step("Clicking on Initiator Security Tab.");
    $self->click($INITIATOR_SECURITY_TAB);
    $self->wait_for_loading();

    $Log->comment("Fetching Security Type");
    $values{$INITIATOR_SECURITY_DEFAULT_SECURITY_TYPE_KEY}
        = $self->get_text($INITIATOR_SECURITY_DEFAULT_SECURITY_TYPE);

    $Log->comment("Fetching Inbound User Name");
    $values{$INITIATOR_SECURITY_DEFAULT_INBOUND_UNAME_KEY}
        = $self->get_text($INITIATOR_SECURITY_DEFAULT_INBOUND_UNAME);

    $Log->comment("Fetching Outbound User Name");
    $values{$INITIATOR_SECURITY_DEFAULT_OUTBOUND_UNAME_KEY}
        = $self->get_text($INITIATOR_SECURITY_DEFAULT_OUTBOUND_UNAME);

    $Log->comment("Collection of igroup security info is successfull");
    $security_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $security_info;
} ## end sub get_initiator_default_security_info

###############################################################################
## @name ::clear_window_sm_initiator
## @summary
##      Check and close any left over dialog box due to previous testcase failure using GUI.
## @description
##     This method can be used to check and close any left over dialog box due to previous testcase failure using GUI.
## @synopsis
##     clear_window_sm_initiator(%args);
## INPUTS TO METHOD CALL 'clear_window_sm_initiator' ON APISET OBJECT
##

sub clear_window_sm_initiator {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    #Check if any kind of dialog box is present
    while (( $self->is_element_present("$INITIATOR_GROUPS_CANCEL_BUTTON") )
        || ( $self->is_element_present("$INITIATOR_GROUPS_OK_BUTTON") ) )
    {

        #Check if it has a close cross mark on titlebar.
        $Log->comment(
            "Closing dialog box left over by previously failed testcase.");
        if ( $self->is_element_present("$INITIATOR_GROUPS_CANCEL_BUTTON") ) {
            $self->click("$INITIATOR_GROUPS_CANCEL_BUTTON");
        } elsif ( $self->is_element_present("$INITIATOR_GROUPS_OK_BUTTON") ) {
            $self->click("$INITIATOR_GROUPS_OK_BUTTON");
        } else {
            $self->nlog("May be no left over Wizards/Dialogues present");
        }
    } ## end while ( ( $self->is_element_present...))

    #Click the Refresh Button
    if ( $self->is_element_present("$INITIATOR_GROUPS_REFRESH_BUTTON") ) {
        $self->click("$INITIATOR_GROUPS_REFRESH_BUTTON");
    } else {
        $self->nlog("Could not find the Refresh Button");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub clear_window_sm_initiator

1;

