# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode LUN Functionalities
##
## @description
##      Using this library, user can perform Cmode LUN related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmLun;
##
##      lun_create(%args);
##      lun_edit(%args);
##      lun_delete(%args);
##      lun_status(%args);
##      lun_clone(%args);
##      lun_refresh(%args);
##      lun_info(%args);
##
## @status private
##
## @author
##		Afsar syed . afsars@netapp.com
##
## @change
##		4/29/2010 : Afsar : Initial version
## @change
##      3/20/2012 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmLun;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    lun_create
    lun_edit
    lun_delete
    lun_status
    lun_clone
    lun_refresh
    lun_info
    _igroup
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::lun_create
## @summary
##     Creation of Lun using GUI.
## @description
##     This method can be used to create Lun using GUI.
## @synopsis
##     lun_create(%args);
## INPUTS TO METHOD CALL 'lun_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -discrb </td> <td> Descnription of the lun </td> </tr>
## <tr> <td> -type </td> <td> OS type to be used </td> </tr>
## <tr> <td> -size </td> <td> Size of the lun to be created </td> </tr>
## <tr> <td> -sizeunit </td> <td> Sizeunit of the lun to be created </td> </tr>
## <tr> <td> -thinprovisioned </td> <td> Thin provisioning for the lun </td> </tr>
## <tr> <td> -vol </td> <td> New volume to be created for the lun </td> </tr>
## <tr> <td> -path </nf </td> <td> Volumepath for the volume to be created </td> </tr>
## <tr> <td> -ini_new </td> <td> Input for initiators in lun create </td> </tr>
## <tr> <td> -ini_existing  </td> <td> Selecting one initiator for lun creation </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time and Finish Time.

sub lun_create {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-lunname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $self->click("$LUN_TAB");
    $self->wait_for_loading();

    $Log->step("Click create Lun button");
    $self->click("$LUN_CREATE_BUTTON");
    #$self->wait_for_element_present("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();

    $Log->step("Click next in create Lun button");
    $self->click("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();

    $Log->step("Entering LUNNAME");
    $self->type( "$LUN_CREATE_NAME_INPUT", $args{'-lunname'} );

    if ( exists( $args{-discrb} ) ) {
        $Log->step("Entering LUN description");
        $self->type( "$LUN_CREATE_DISCRIB_INPUT", $args{-discrb} );
    }

    if ( exists( $args{-type} ) ) {
        my $type = $args{-type};

        if ( $args{-type} =~ /aix/i ) {
            $type = "AIX";
        }
        elsif ( $args{-type} =~ /hp-ux/i ) {
            $type = "HP-UX";
        }
        elsif ( $args{-type} =~ /hyper-v/i ) {
            $type = "Hyper-V";
        }
        elsif ( $args{-type} =~ /linux/i ) {
            $type = "Linux";
        }
        elsif ( $args{-type} =~ /netware/i ) {
            $type = "NetWare";
        }
        elsif ( $args{-type} =~ /openvms/i ) {
            $type = "OpenVMS";
        }
        elsif ( $args{-type} =~ /solaris/i ) {
            $type = "Solaris";
        }
        elsif ( $args{-type} =~ /solaris efi/i ) {
            $type = "Solaris EFI";
        }
        elsif ( $args{-type} =~ /vmware/i ) {
            $type = "VMware";
        }
        elsif ( $args{-type} =~ /windows|Windows 2003 MBR/i ) {
            $type = "Windows 2003 MBR";
        }
        elsif ( $args{-type} =~ /windows gpt|Windows 2003 GPT/i ) {
            $type = "Windows 2003 GPT";
        }
        elsif ( $args{-type} =~ /windows 2008|Windows 2008 or later/i ) {
            $type = "Windows 2008 or later";
        }
        elsif ( $args{-type} =~ /xen/i ) {
            $type = "Xen";
        }
        
        $Log->step("Entering OS type");
	$self->select_from_dropdown("$LUN_CREATE_TYPE_COMBOBOX", $type, 1);
        $self->wait_for_loading();
    }

    if ( exists( $args{-size} ) ) {
        $Log->step("Entering LUN size");
        $self->type( "$LUN_CREATE_SIZE_INPUT", $args{-size} );
    }

    if ( exists( $args{-sizeunit} ) ) {
        $Log->step("Entering size unit");
	$self->select_from_dropdown("$LUN_CREATE_SIZEUNIT_COMBOBOX", uc($args{-sizeunit}), 1);
        $self->wait_for_loading();
    }

    if ( exists( $args{-spaceguarantee} ) ) {
        if ( lc( $args{-spaceguarantee} eq 'enable' ) ) {
            $Log->step("Selecting space guarantee");
            $self->set_checkbox( "$LUN_CREATE_SPACEGUARANTEE_CHECKBOX",
                "enable" );
        }

        if ( lc( $args{-spaceguarantee} eq 'disable' ) ) {
            $Log->step("Selecting space guarantee");
            $self->set_checkbox( "$LUN_CREATE_SPACEGUARANTEE_CHECKBOX",
                "disable" );
        }
    } ## end if ( exists( $args{-spaceguarantee...}))

    $Log->step("Click next in create Lun button");
    $self->wait_for_loading();
    $self->click("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();
    #$self->wait_for_element_present("$LUN_CREATE_NEW_RADIO");

    if ( exists( $args{-vol} ) ) {
        if ( lc( $args{-vol} eq 'new' ) ) {
            $Log->step(
                "Enter the input for create new volume in Lun create ");
            $self->click("$LUN_CREATE_NEW_RADIO");
            if ( exists( $args{-new_aggrname} ) ) {
                $Log->step(
                    "Select one aggr for new vol : $args{-new_aggrname} ");
                $self->type( "$LUN_CREATE_NEW_AGGRNAME_INPUT",
                    $args{-new_aggrname} );
            } ## end if ( exists( $args{-new_aggrname...}))
            if ( exists( $args{-new_volname} ) ) {
                $Log->step("Select one vol for new Lun :$args{-new_volname}");
                $self->type( "$LUN_CREATE_NEW_VOLNAME_INPUT",
                    $args{-new_volname} );
            }
        } elsif ( lc( $args{-vol} eq 'existing' ) ) {
            $Log->step("Enter the input for existing volume in Lun create ");
            $self->click("$LUN_CREATE_EXISTING_RADIO");

            if ( exists( $args{-path} ) ) {
                $self->type( "$LUN_CREATE_EXISTING_VOLPATH_INPUT",
                    $args{-path} );
            }
        } ## end elsif ( lc( $args{-vol} eq...))
    } ## end if ( exists( $args{-vol...}))

    $Log->step("Click next after vol selection");
    $self->click("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();
    $self->check_for_error();

    if ( exists( $args{-ini_new} ) ) {
        $self->wait_for_element_present("$LUN_CREATE_ADDINITIATOR_BUTTON");
        $Log->step("Enter the input for initiators in Lun create ");
        $self->click("$LUN_CREATE_ADDINITIATOR_BUTTON");
        $self->wait_for_loading();
        my ( $name, $os, $portset, $type ) = @{ $args{-ini_new} };
        $self->_igroup(
            -igroupname => "$name",
            -os         => "$os",
            -type       => "$type",
            -portset    => "$portset",
        );
        $self->wait_for_loading();
        $self->wait_for_element_present("$LUN_CREATE_ADDINITIATOR_BUTTON");
        $self->set_checkbox_in_row_with_text( "$LUN_CREATE_INITIATOR_TABLE",
            "enable", $name );
        if ( exists( $args{-igroupid_new} ) ) {
            $self->type_in_row_with_text( "$LUN_CREATE_INITIATOR_TABLE",
                $args{-igroupid_new}, $name );
        }
    } ## end if ( exists( $args{-ini_new...}))

    if ( exists( $args{-ini_existing} ) ) {
        my $n = 0;
        foreach my $initiatorX ( @{ $args{-ini_existing} } ) {
            $Log->step(
                "Selecting one initiator for Lun creation :$initiatorX");
            
            $self->wait_for_loading();

            $Log->comment("enabling the igroup checkbox"); 
            $self->wait_for_element_present($LUN_CREATE_ALL_IGROUP_CHECKBOX);
            $self->set_checkbox($LUN_CREATE_ALL_IGROUP_CHECKBOX, "enable"); 

            $self->wait_for_loading();

            $self->set_checkbox_in_row_with_text(
                "$LUN_CREATE_INITIATOR_TABLE", "enable", $initiatorX );
            if ( exists( $args{-igroupid_existing} ) ) {
                my @igid = @{ $args{-igroupid_existing} };
                if ( $igid[$n] ) {
                    $self->type_in_row_with_text(
                        "$LUN_CREATE_INITIATOR_TABLE", $igid[$n],
                        $initiatorX );
                } ## end if ( $igid[$n] )
            } ## end if ( exists( $args{-igroupid_existing...}))
            $n++;
        } ## end foreach my $initiatorX ( @{...})
    } ## end if ( exists( $args{-ini_existing...}))

    $Log->step("Clicking next button after input of initiator information ");
	$self->mouse_down("$LUN_CREATE_NEXT_BUTTON");
    $self->click("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->step("Clicking next button after inspection of summary page");
    # my $commit_time = $self->click_timecount( $LUN_CREATE_NEXT_BUTTON, "next",
        # $LUN_CREATE_FINISH_BUTTON );
	$self->click("$LUN_CREATE_NEXT_BUTTON");	
    $self->wait_for_loading();

    $Log->comment("Enter Lun create Finish Button");
    #my $finish_time = $self->click_timecount($LUN_CREATE_FINISH_BUTTON);
	$self->click("$LUN_CREATE_NEXT_BUTTON");
    $self->wait_for_loading();
    
    if($self->is_element_present("$LUN_CREATE_NEXT_BUTTON")){
        $Log->comment("Click on Lun create Finish Button");
        $self->click("$LUN_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();
    }
    
    # $Log->comment(
        # "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    # );
    my %times = (
        loading_time => $loading_time,
        # commit_time  => $commit_time,
        # finish_time  => $finish_time
    );

    $Log->comment("Create Lun is successfull");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_create

################################################################################
## @name ::lun_edit
## @summary
##     Updation of Lun using GUI.
## @description
##     This method can be used to update Lun using GUI.
## @synopsis
##     lun_edit(%args);
## INPUTS TO METHOD CALL 'lun_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -lunrename </td> <td> Renaming the lun </td> </tr>
## <tr> <td> -discrib </td> <td> Desciption of the lun to be used </td> </tr>
## <tr> <td> -size </td> <td> Size of the lun to be created </td> </tr>
## <tr> <td> -sizeunit </td> <td> Sizeunit of the lun to be created </td> </tr>
## <tr> <td> -thinprovisioned </td> <td> Thin provisioning for the lun </td> </tr>
## <tr> <td> -autogrow </td> <td> Thin provisioning for the lun </td> </tr>
## <tr> <td> -igroup </td> <td> Initiator group </td> </tr>
## <tr> <td> -igroupname </td> <td> Name of Initiator group </td> </tr>
## <tr> <td> -igrouprename </nf </td> <td> Renaming the initiator group </td> </tr>
## <tr> <td> -igroupid </td> <td> list of lun ID's respective to existing initiators list </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub lun_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-lunname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );
    $self->click("$LUN_TAB");
    $self->wait_for_loading();

    $Log->step("*** Select LUN :$args{'-lunname'} ***");
    $self->click_row_with_texts( "$LUN_GRID", 1, 0, $args{'-lunname'} );
    $self->wait_for_loading();

    $Log->step("Click Edit Lun button");
    $self->click("$LUN_EDIT_BUTTON");
    $self->wait_for_loading();

    if ( exists( $args{-lunrename} ) ) {
        $Log->step("Renaming LUN to $args{-lunrename}");
        $self->type( "$LUN_EDIT_NAME_INPUT", $args{-lunrename} );
    }
    if ( exists( $args{-discrib} ) ) {
        $Log->step("Click Edit Lun to change describtion");
        $self->type( "$LUN_EDIT_DESCR_INPUT", $args{-discrib} );
    }

    if ( exists( $args{-size} ) ) {
        $Log->step("Click Edit Lun to change size");
        $self->type( "$LUN_EDIT_SIZE_INPUT", $args{-size} );
    }
    if ( exists( $args{-sizeunit} ) ) {
        $Log->step("Click Edit Lun to change size unit");
        $self->set_combobox_value( "$LUN_EDIT_SIZEUNIT_COMBOBOX",
            uc( $args{-sizeunit} ) );

    } ## end if ( exists( $args{-sizeunit...}))
    if ( exists( $args{-spaceguarantee} ) ) {
        $Log->step("Click Edit Lun select space reserve");
        $self->set_checkbox( "$LUN_EDIT_SPACERES_CHECKBOX",
            $args{-spaceguarantee} );

    } ## end if ( exists( $args{-spaceguarantee...}))
    if ( exists( $args{-autogrow} ) ) {
        $Log->step("Click Edit Lun to select lun autogrow");
        $self->set_checkbox( "$LUN_EDIT_AUTOGROW_CHECKBOX",
            $args{-autogrow} );
    }

    if ( exists( $args{-igroup} ) ) {
        $Log->step("Click initiator group tab in Edit Lun ");
        $self->click("$LUN_EDIT_INITIATOR_TAB");
        #$self->wait_for_element_present("$LUN_EDIT_IGROUPADD_BUTTON");
        $self->wait_for_loading();
        $Log->comment("Enabling the Show All Initiator Groups checkbox");
        $self->set_checkbox("$LUN_CREATE_SELECT_IGROUPS_PAGE_SHOW_ALL_CHECKBOX", "enable");

        if ( lc( $args{-igroup} ) eq 'add' ) {
            if ( exists( $args{-igroupname} ) ) {
                foreach my $igroup ( @{ $args{-igroupname} } ) {
                    $Log->step(
                        "Enter the input for initiators in Lun create ");
                    $self->click("$LUN_EDIT_IGROUPADD_BUTTON");
                    $self->wait_for_loading();
                    my @igroup_values = split( /:/, $igroup );
                    $self->wait_for_element_present(
                        "$LUN_EDIT_IGROUPEDIT_NAME_COMBOBOX");
                    $self->set_combobox_value(
                        "$LUN_EDIT_IGROUPEDIT_NAME_COMBOBOX",
                        $igroup_values[0] );
                    if ( exists( $igroup_values[1] ) ) {
                        $Log->step(
                            "Click edit igroup id in Edit Lun : $igroup_values[1] "
                        );
                        $self->type( "$LUN_EDIT_IGROUPEDIT_ID_INPUT",
                            $igroup_values[1] );
                    } ## end if ( exists( $igroup_values...))
                    $self->click("$LUN_EDIT_IGROUPEDIT_OK_INPUT");
                    $self->wait_for_loading();
                } ## end foreach my $igroup ( @{ $args...})
            } ## end if ( exists( $args{-igroupname...}))
        } ## end if ( lc( $args{-igroup...}))
        if ( lc( $args{-igroup} ) eq 'create_new_igroup' ) {
            if ( exists( $args{-igroupname} ) ) {
                foreach my $igroup ( @{ $args{-igroupname} } ) {
                    $Log->step( "Create new igroup" );
                    $self->click($LUN_EDIT_IGROUPADD_BUTTON);

                    $Log->comment("Entering igroup name: $igroup");
                    $self->wait_for_element_present($INITIATOR_GROUPS_ADD_NAME_INPUT);
                    $self->type( $INITIATOR_GROUPS_ADD_NAME_INPUT, $igroup );
                } ## end foreach my $igroup ( @{ $args...})
            } ## end if ( exists( $args{-igroupname...}))

            if ( exists( $args{'-os'} ) ) {
                $Log->comment("Selecting OS : $args{'-os'}");
                $self->set_combobox_value(
                    $INITIATOR_GROUPS_ADD_OSTYPE_COMBOBOX,
                    $args{'-os'} );
            } ## end if ( exists( $args{'-os'...}))

            # Selecting type based on input parameter
            if ( exists( $args{'-type'} ) ) {
                $Log->comment("Selecting type: $args{'-type'}");
                if ( lc( $args{'-type'} eq "iscsi" ) ) {
                    $self->set_option($INITIATOR_GROUPS_ADD_ISCSI_OPTION);
                } elsif ( lc( $args{'-type'} eq "fcp" ) ) {
                    $self->set_option($INITIATOR_GROUPS_ADD_FCP_OPTION);

                    # Enabling ALUA based on input parameter
                    if ( exists( $args{'-alua'} ) ) {
                        $Log->comment( $args{'-alua'} . "ing alua" );
                        $self->set_checkbox(
                            $INITIATOR_GROUPS_ADD_ALUA_CHECKBOX,
                            $args{'-alua'} );
                    } ## end if ( exists( $args{'-alua'...}))
                } else {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw(
                        "Invalid value for type, it should be either iscsi or fcp"
                    );
                } ## end else [ if ( lc( $args{'-type'...}))]
            } ## end if ( exists( $args{'-type'...}))

            if ( exists( $args{'-portset'} ) ) {

                $Log->comment("entering portset: $args{'-portset'}");
                $self->type( "$LUN_EDIT_PORTSET_INPUT", $args{-portset} );
            }

            $Log->step("Click ok for create igroup");
            $self->click("$INITIATOR_GROUPS_OK_BUTTON");
            $self->wait_for_loading();
            if($self->is_element_present($LUN_IGROUP_SUMMARY_OK_BUTTON)) {
                $Log->comment("Clicking on OK button on Initiator-Group summary pop-up");
                $self->click($LUN_IGROUP_SUMMARY_OK_BUTTON);
            }

            if ( exists( $args{-ini_existing} ) ) {
                foreach my $initiatorX ( @{ $args{-ini_existing} } ) {
                    my $n = 0;
                    $Log->step(
                        "Selecting one initiator for Lun creation :$initiatorX"
                    );
                    $self->wait_for_element_present($LUN_CREATE_INITIATOR_TABLE);
                    $self->set_checkbox_in_row_with_text(
                        "$LUN_CREATE_INITIATOR_TABLE", "enable",
                        $initiatorX );
                    if ( exists( $args{-igroupid} ) ) {
                        my @igid = @{ $args{-igroupid} };
                        if ( $igid[$n] ) {
                            $self->type_in_row_with_text(
                                "$LUN_CREATE_INITIATOR_TABLE", $igid[$n],
                                $initiatorX );
                        }
                    } ## end if ( exists( $args{-igroupid...}))
                    $n++;
                } ## end foreach my $initiatorX ( @{...})
            } ## end if ( exists( $args{-ini_existing...}))
        } ## end if ( lc( $args{-igroup...}))
        if ( lc( $args{-igroup} ) eq 'edit' ) {
            foreach my $initiatorX ( @{ $args{-ini_existing} } ) {
                my $n = 0;
                $Log->step(
                    "Selecting one initiator for Lun creation :$initiatorX");
                $self->wait_for_loading();
                $self->set_checkbox_in_row_with_text(
                    "$LUN_CREATE_INITIATOR_TABLE", "enable", $initiatorX );
                eval{
		if ( exists( $args{-igroupid} ) ) {
                    my @igid = @{ $args{-igroupid} };
                    if ( $igid[$n] ) {
                        $self->type_in_row_with_text(
                            "$LUN_CREATE_INITIATOR_TABLE", $igid[$n],
                            $initiatorX );
                    }
                } ## end if ( exists( $args{-igroupid...}))
		};
		if($@){
			$Log->comment("Exception is: $@");
		}
                $n++;
            } ## end foreach my $initiatorX ( @{...})
        } ## end if ( lc( $args{-igroup...}))
    
        $self->wait_for_loading();
        if($self->is_element_present($LUN_IGROUP_SUMMARY_OK_BUTTON)) {
            $Log->comment("Clicking on OK button on Initiator-Group summary pop-up");
            $self->click($LUN_IGROUP_SUMMARY_OK_BUTTON);
        }
    
    } ## end if ( exists( $args{-igroup...}))

    my ( $start, $end, $sec );
    $start = (time)[0];
    $self->wait_for_loading();
    $Log->step("Click ok for edit Lun");
    $self->click_nowait($LUN_EDIT_OK_BUTTON);
    ##Please do not remove this sleep. wait_for_loading() doesnt work here
    sleep 10;
    if ( $self->is_element_present($LUN_EDIT_CONFORM_BUTTON) ) {
        $Log->step("Click ok for conformation edit Lun");
        $self->click_nowait($LUN_EDIT_CONFORM_BUTTON);
    }
    ##Please do not remove this sleep. wait_for_loading() doesnt work here
    sleep 10;
    if ( $self->is_element_present($LUN_EDIT_SHRINK_CHECKBOX)) {
        $Log->step("Enable the shrink checkbox");
        $self->click_nowait($LUN_EDIT_SHRINK_CHECKBOX);
        $Log->step("Click on shrink dialogue ok button");
        $self->click_nowait($LUN_EDIT_SHRINK_OK_BUTTON);
    }
    ##Please do not remove this sleep. wait_for_loading() doesnt work here
    sleep 10;
    if ( $self->is_element_present($LUN_EDIT_RESIZE_OK_BUTTON) ) {
        $Log->step("Close resize warning for edit Lun");
        $self->click($LUN_EDIT_RESIZE_OK_BUTTON);
    }
     
    $self->wait_for_loading();
    $self->check_for_error();

    $end = (time)[0];
    $sec = $end - $start - 20;
    my $commit_time = sprintf( "%.2f", $sec );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->step("Edit Lun is successfull");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_edit

################################################################################
## @name ::lun_delete
## @summary
##     Deletion of Lun using GUI.
## @description
##     This method can be used to delete the Lun using GUI.
## @synopsis
##     lun_delete(%args);
## INPUTS TO METHOD CALL 'lun_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -status </td> <td> Selecting the status of the lun </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub lun_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-lunname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );
    $self->click("$LUN_TAB");
    $self->wait_for_loading();

    if ( lc( $args{-status} ) eq 'offline' ) {

        $Log->step("*** Select LUN :$args{'-lunname'} ***");
        $self->click_row_with_texts( "$LUN_GRID", 1, 0, $args{'-lunname'} );
        $self->wait_for_loading();

        $Log->step("Selete the status of the LUN :$args{-status} ");
        $self->click("$LUN_STATUS_BUTTON");
        $Log->step("Selete the status of the LUN :$args{-status} ");
        $self->click("$LUN_STATUS_OFFLINE_BUTTON");
        $self->wait_for_element_present("$LUN_STATUS_OFFLINE_OK_BUTTON");

        $self->click("$LUN_STATUS_OFFLINE_OK_BUTTON");
    } ## end if ( lc( $args{-status...}))

    $self->click("$LUN_REFRESH_BUTTON");
    $self->wait_for_loading();
    $Log->step("*** Select LUN :$args{'-lunname'} ***");
    $self->click_row_with_texts( "$LUN_GRID", 1, 0, $args{'-lunname'} );
    $self->wait_for_loading();
    
    $Log->comment("Click delete Lun button");
    $self->click("$LUN_DELETE_BUTTON");

    $Log->comment("Waiting for Lun delete conformation");
    #$self->wait_for_element_present("$LUN_DELETE_CONFORM_CHECKBOX");
    $self->wait_for_loading();
    
    sleep(5);
    
    $Log->comment("Click ok for delete Lun");
    $self->set_checkbox( "$LUN_DELETE_CONFORM_CHECKBOX", "enable" );

    if ( exists( $args{'parent_delete'} ) ) {
        $Log->comment("Click ok for deleting parent container");
        $self->set_checkbox( "$LUN_DELETE_CONFORM_CHECKBOX2", "enable" );
    }
    my $commit_time = $self->click_timecount($LUN_DELETE_DELETE_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Delete Lun is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_delete

################################################################################
## @name ::lun_status
## @summary
##     Getting status of Lun using GUI.
## @description
##     This method can be used to get the status of Lun using GUI.
## @synopsis
##     lun_status(%args);
## INPUTS TO METHOD CALL 'lun_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -status </td> <td> Selecting the status of the lun </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub lun_status {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-lunname' => { type => SCALAR },
            "-status"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my ($commit_time);
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $self->click("$LUN_TAB");
    $self->wait_for_loading();
    $Log->step("*** Select LUN :$args{'-lunname'} ***");
    $self->click_row_with_texts( "$LUN_GRID", 1, 0, $args{'-lunname'} );
    $self->wait_for_loading();

    if ( lc( $args{'-status'} ) eq 'online' ) {
        $Log->step("Selete the status of the LUN :$args{-status} ");
        $self->click("$LUN_STATUS_BUTTON");
        $Log->step("Selete the status of the LUN :$args{-status} ");

        $commit_time = $self->click_timecount($LUN_STATUS_ONLINE_BUTTON);
    } ## end if ( lc( $args{'-status'...}))
    if ( lc( $args{'-status'} ) eq 'offline' ) {

        $Log->step("Selete the status of the LUN :$args{-status} ");
        $self->click("$LUN_STATUS_BUTTON");
        $Log->step("Selete the status of the LUN :$args{-status} ");
        $self->click("$LUN_STATUS_OFFLINE_BUTTON");
        $self->wait_for_element_present("$LUN_STATUS_OFFLINE_OK_BUTTON");

        $commit_time = $self->click_timecount($LUN_STATUS_OFFLINE_OK_BUTTON);
    } ## end if ( lc( $args{'-status'...}))

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Lun Status is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_status

################################################################################
## @name ::lun_clone
## @summary
##     Creation of Lun clone using GUI.
## @description
##     This method can be used to create Lun clone using GUI.
## @synopsis
##     lun_clone(%args);
## INPUTS TO METHOD CALL 'lun_clone' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -clonename </td> <td> Selecting the name of lun clone </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub lun_clone {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-lunname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $self->click("$LUN_TAB");
    $self->wait_for_loading();
    $Log->comment("*** Select LUN :$args{'-lunname'} ***");
    $self->click_row_with_texts( $LUN_GRID, 1, 0, $args{'-lunname'} );
    $self->wait_for_loading();
    $Log->comment("*** Select Lun clone button ***");
    $self->click($LUN_CLONE_BUTTON);
    $self->wait_for_element_present($LUN_CLONE_NAME_INPUT);

    if ( exists( $args{-clonename} ) ) {
        $Log->step("Enter the Lun clone name");
        $self->type($LUN_CLONE_NAME_INPUT, "" );
        $self->type($LUN_CLONE_NAME_INPUT, $args{-clonename} );
    }

    $Log->comment("*** Select Lun clone Ok button ***");
    my $commit_time = $self->click_timecount($LUN_CLONE_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Lun Clone is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_clone

################################################################################
## @name ::lun_refresh
## @summary
##     Refresh Lun clone using GUI.
## @description
##     This method can be used to refresh Lun clone using GUI.
## @synopsis
##     lun_refresh(%args);
## INPUTS TO METHOD CALL 'lun_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub lun_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $self->click("$LUN_TAB");

    $Log->comment("*** Select Lun refresh button ***");

    my $commit_time
        = $self->click_timecount( $LUN_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("Lun refresh is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub lun_refresh

################################################################################
## @name ::lun_info
## @summary
##    Get information regarding Lun using GUI.
## @description
##     This method can be used to get information regarding Lun using GUI.
## @synopsis
##     lun_info(%args);
## INPUTS TO METHOD CALL 'lun_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -lunname </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -clonename </td> <td> Selecting the name of lun clone </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the LUN status details.

sub lun_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-lunname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    
    my %values = (
            'name', 'path',      'size',        'status',
            'type', 'serial_no', 'description', 'igroup',
            'initiators', 'thinprovision'
    );
    my $lun_info = {};

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Lun");
    $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $self->click("$LUN_TAB");
    $self->wait_for_loading();
    $Log->step("*** Select LUN :$args{'-lunname'} ***");
    $self->click_row_with_texts( "$LUN_GRID", 1, 0, $args{'-lunname'} );
#    $self->wait_for_element_present("$LUN_INFO_DETAIL_TAB");
    $self->wait_for_loading();

    $Log->comment("Navigating to Storage>Lun Details tab");
    $self->click("$LUN_INFO_DETAIL_TAB");
    #$self->wait_for_loading();

    $Log->step("Getting row data for LUN from grid");
    my $rowdata = $self->get_row_data_with_text( "$LUN_GRID", $args{'-lunname'} );
    my @rowarray = split( ' ', $rowdata );

    %values = (
       "$LUN_INFO_THIN_PROVISION_KEY" => $rowarray[2],
    );
        
    $values{"$LUN_INFO_NAME"} = $self->get_text("$LUN_INFO_DETAIL_NAME_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_NAME_LABEL") );
    $values{"$LUN_INFO_PATH"} = $self->get_text("$LUN_INFO_DETAIL_PATH_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_PATH_LABEL") );
    $values{"$LUN_INFO_SIZE"} = $self->get_text("$LUN_INFO_DETAIL_SIZE_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_SIZE_LABEL") );
    $values{"$LUN_INFO_STATUS"}
        = $self->get_text("$LUN_INFO_DETAIL_STATUS_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_STATUS_LABEL") );
    $values{"$LUN_INFO_TYPE"} = $self->get_text("$LUN_INFO_DETAIL_TYPE_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_TYPE_LABEL") );
    $values{"$LUN_INFO_SERIAL"}
        = $self->get_text("$LUN_INFO_DETAIL_SERIAL_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_SERIAL_LABEL") );
    $values{"$LUN_INFO_DESC"}
        = $self->get_text("$LUN_INFO_DETAIL_DESCR_LABEL")
        if ( $self->is_element_present("$LUN_INFO_DETAIL_DESCR_LABEL") );
    #$self->click("$LUN_INFO_IGROUP_TAB");
    #$values{"$LUN_INFO_IGROUPDET"}
    #    = $self->get_row_data_with_text("$LUN_INFO_IGROUP_GRID")
    #    if ( $self->is_element_present("$LUN_INFO_IGROUP_GRID") );
    #$self->click("$LUN_INFO_INITIAT_TAB");
    #$values{"$LUN_INFO_INITIATORDET"}
    #    = $self->get_row_data_with_text("$LUN_INFO_INITIAT_GRID")
    #    if ( $self->is_element_present("$LUN_INFO_INITIAT_GRID") );

    #$Log->comment("Navigating to Storage>Lun Details tab");
    #$self->click("$LUN_INFO_DETAIL_TAB");
    #$self->wait_for_loading();
    $self->check_for_error();

    foreach ( keys %values ) {
        $Log->comment("The Info for $_ => $values{$_}");
    }
    $Log->comment("Collection of LUN info is successfull");
    $lun_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    return $lun_info;

} ## end sub lun_info

########################################################################
#         P R I V A T E   M E T H O D S
########################################################################
################################################################################
# Subroutine Name: _igroup
# Objective:       Initiator group settings for Lun
# Details:         This method will do the Initiator group settings for
#                  Lun with the given inputs
################################################################################
sub _igroup {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-igroupname' => { type => SCALAR },
            "-os"         => { type => SCALAR },
            "-portset"    => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
#    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Waiting for iGroup Create Dialog");
    $self->wait_for_element_present( $INITIATOR_GROUPS_ADD_NAME_INPUT,
        $GENERIC_WAIT );

    $Log->comment("Entering igroup name: $args{'-igroupname'}");
    $self->type( $INITIATOR_GROUPS_ADD_NAME_INPUT, $args{'-igroupname'} );

    $Log->comment("Selecting OS : $args{'-os'}");
    $self->set_combobox_value( $INITIATOR_GROUPS_ADD_OSTYPE_COMBOBOX,
        $args{'-os'} );

    # Selecting type based on input parameter
    if ( exists( $args{'-type'} ) ) {
        $Log->comment("Selecting type: $args{'-type'}");
        if ( lc( $args{'-type'} eq "iscsi" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_ISCSI_OPTION);
        } elsif ( lc( $args{'-type'} eq "fcp" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_FCP_OPTION);
        } elsif ( lc( $args{'-type'} eq "mixed" ) ) {
            $self->set_option($INITIATOR_GROUPS_ADD_MIXED_OPTION);
        } else {
            NATE::BaseException->throw(
                "Invalid value for type, it should be either iscsi or fcp");
        }
    } ## end if ( exists( $args{'-type'...}))

    $Log->comment("Entering portset: $args{'-portset'}");
    $self->type( $INITIATOR_GROUPS_ADD_PORTSET_INPUT, $args{'-portset'} );

    if ( exists( $args{'-initiators'} ) ) {
        $Log->comment("Clicking on Initiators Tab.");
        $self->click($INITIATOR_GROUPS_INIT_TAB);
        $self->wait_for_element_present( $INITIATOR_GROUPS_INIT_GRID,
            $GENERIC_WAIT );

        for my $count ( 0 .. scalar( @{ $args{-initiators} } ) - 1 ) {
            my $initiator = $args{-initiators}[$count];
            $Log->comment("Clicking ADD button to add initiator: $initiator");
            $self->click($INITIATOR_GROUPS_INIT_ADD_BUTTON);
            $self->wait_for_element_present( $INITIATOR_GROUPS_INIT_INPUT,
                $GENERIC_WAIT );
            $Log->comment("Entering initiator: $initiator");
            $self->type( $INITIATOR_GROUPS_INIT_INPUT, $initiator );
            $Log->comment("Clicking OK button");
            $self->click($INITIATOR_GROUPS_INIT_OK_BUTTON);
            $self->check_for_error();
            $self->wait_for_element_not_present(
                $INITIATOR_GROUPS_INIT_OK_BUTTON,
                $GENERIC_WAIT );
            $Log->comment("Initiator added successfully");
        } ## end for my $count ( 0 .. scalar...)
    } ## end if ( exists( $args{'-initiators'...}))

#    $self->wait_for_element_present( $INITIATOR_GROUPS_CREATE_BUTTON,
#        $GENERIC_WAIT );

    $Log->comment("Clicking Create Button");
    $self->click($LUN_IGROUPS_CREATE_BUTTON);

    $self->wait_for_loading();
    if($self->is_element_present($LUN_IGROUP_SUMMARY_OK_BUTTON)) {
        $Log->comment("Clicking on OK button on Initiator-Group summary pop-up");
        $self->click($LUN_IGROUP_SUMMARY_OK_BUTTON);
    }

    $self->wait_for_loading();    
    $Log->comment("igroup created successfully");
    $Log->exit() if $may_exit;

    return 1;
} ## end sub _igroup

1;
