# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Name Mapping Functionalities
##
## @description
##      Using this library, user can perform Cmode Name Mapping related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmNameMapping
##
##	vserver_namemapping_add(%args);
##	vserver_namemapping_edit(%args);
##	vserver_namemapping_delete(%args);
##	vserver_namemapping_refresh(%args);
##      vserver_namemapping_swap(%args);
##
##
## @author Vinu.Nagarajan@netapp.com
## @change
##  06/23/2011 : vikasp : Initial Version
## @change
##  03/20/2012 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmNameMapping;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    vserver_namemapping_add
    vserver_namemapping_edit
    vserver_namemapping_delete
    vserver_namemapping_refresh
    vserver_namemapping_swap
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::vserver_namemapping_add
## @summary
##     Adding name mapping to vserver using GUI.
## @description
##     This method can be used to add name mapping to vserver using GUI.
## @synopsis
##     vserver_namemapping_add(%args);
## INPUTS TO METHOD CALL 'vserver_namemapping_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -namemapping_direction </td> <td> (Required) Selecting namemapping_direction </td> </tr>
## <tr> <td> -namemapping_position </td> <td> (Required) Selecting namemapping_position </td> </tr>
## <tr> <td> -namemapping_pattern </td> <td> (Required) Selecting namemapping_pattern </td> </tr>
## <tr> <td> -namemapping_replacement </td> <td> (Required) Selecting namemapping_replacement </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_namemapping_add {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-namemapping_direction'   => { type => SCALAR },
            "-namemapping_position"    => { type => SCALAR },
            "-namemapping_pattern"     => { type => SCALAR },
            "-namemapping_replacement" => { type => SCALAR },
        },
        allow_extra => 1
    );

    my ($namemapping_direction, $namemapping_position,
        $namemapping_pattern,   $namemapping_replacement
    );	

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAME_MAPPING_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAME_MAPPING_TREE );

    $Log->comment("Click name mapping add button");
    $self->click($VSERVER_NAME_MAPPING_ADD_BTN);

    $Log->comment("Waiting for name mapping entry add dialog");
    $self->wait_for_element_present( $VSERVER_NAME_MAPPING_DIRECTION_COMBO,
        $GENERIC_WAIT );

    $Log->comment("Select name mapping direction");
    $namemapping_direction = $args{'-namemapping_direction'};
	$self->set_combobox_value(
        $VSERVER_NAME_MAPPING_DIRECTION_COMBO,
        $args{'-namemapping_direction'}
    );

    $namemapping_position = $args{'-namemapping_position'};
	# $Log->comment("Select name mapping position");
	# $self->type(
        # $VSERVER_NAME_MAPPING_POSITION_FIELD,
        # $args{'-namemapping_position'}
    # );
    # $self->fire_event( "$VSERVER_NAME_MAPPING_POSITION_FIELD", "blur" );

    $Log->comment("Entering name mapping pattern");
	$namemapping_pattern = $args{-namemapping_pattern} ;
    $self->type( $VSERVER_NAME_MAPPING_PATTERN_TEXTBOX,
        $args{-namemapping_pattern} );
    $self->fire_event( "$VSERVER_NAME_MAPPING_PATTERN_TEXTBOX", "blur" );

    $Log->comment("Entering name mapping replacement");
    $namemapping_replacement = $args{-namemapping_replacement} ;
	$self->type( $VSERVER_NAME_MAPPING_REPLACEMENT_TEXTBOX,
        $args{-namemapping_replacement} );
    $self->fire_event( "$VSERVER_NAME_MAPPING_REPLACEMENT_TEXTBOX", "blur" );
	
    $Log->comment("Select name mapping position");
    # Temp fix till type type_keys issue is fixed.
    my $gui_val = $self->get_value( "$VSERVER_NAME_MAPPING_POSITION_FIELD" );
	$Log->comment("Gui Value:: " .$gui_val );
    if ($gui_val > $namemapping_position ) {
        my $count = $gui_val - $namemapping_position;
        foreach (1..$count) {
            $Log->comment("Clicking on Spinner Field in if"  );
			$self->click( "$VSERVER_NAME_MAPPING_POSITION_SPINNER_DOWN");
        }
    } elsif ($gui_val < $namemapping_position ) {
        my $count = $namemapping_position - $gui_val;
        foreach (1..$count) {
			$Log->comment("Clicking on Spinner Field in else");
            $self->click( "$VSERVER_NAME_MAPPING_POSITION_SPINNER_UP");
        }
    }	

    $Log->comment("click name mapping entry add ok button");
    my $commit_time = 0;

    $self->click("$VSERVER_NAME_MAPPING_OK_BTN");
    $self->wait_for_loading();
    if ($self->is_element_present(
            "$VSERVER_NAME_MAPPING_POSITION_CHECKBOX")
        )
    {
        $Log->comment("Click checkbox");
        $self->click("$VSERVER_NAME_MAPPING_POSITION_CHECKBOX");
        $Log->comment("Clicking insert button");
        $self->click("$VSERVER_NAME_MAPPING_INSERT_BTN");
		$self->wait_for_loading();
    } ## end if ( $self->is_element_present...)

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Name mapping created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub vserver_namemapping_add

################################################################################
## @name ::vserver_namemapping_edit
## @summary
##     Editing name mapping to vserver using GUI.
## @description
##     This method can be used to edit name mapping to vserver using GUI.
## @synopsis
##     vserver_namemapping_edit(%args);
## INPUTS TO METHOD CALL 'vserver_namemapping_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -namemapping_direction </td> <td> (Required) Selecting namemapping_direction </td> </tr>
## <tr> <td> -namemapping_position </td> <td> (Required) Selecting namemapping_position </td> </tr>
## <tr> <td> -namemapping_pattern </td> <td> Selecting namemapping_pattern </td> </tr>
## <tr> <td> -namemapping_replacement </td> <td> Selecting namemapping_replacement </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_namemapping_edit {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-namemapping_direction' => { type => SCALAR },
            "-namemapping_position"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAME_MAPPING_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAME_MAPPING_TREE );

    $Log->comment("Select namemapping ");
    $self->click_child_row_with_texts(
        $VSERVER_NAME_MAPPING_PAGE_GRID,
        $args{'-namemapping_direction'},
        ["$args{'-namemapping_position'}"]
    );

    $Log->comment("click name mapping edit button");
    $self->click($VSERVER_NAME_MAPPING_EDIT_BTN);

    $Log->comment("Waiting for name mapping entry edit dialog");
    $self->wait_for_element_present( $VSERVER_NAME_MAPPING_PATTERN_TEXTBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-namemapping_pattern'} ) ) {
        $Log->comment("Entering name mapping pattern");
        $self->type( $VSERVER_NAME_MAPPING_PATTERN_TEXTBOX, "" );
        $self->type_keys( $VSERVER_NAME_MAPPING_PATTERN_TEXTBOX,
            $args{-namemapping_pattern} );
    } ## end if ( exists( $args{'-namemapping_pattern'...}))

    if ( exists( $args{'-namemapping_replacement'} ) ) {
        $Log->comment("Entering name mapping replacement");
        $self->type( $VSERVER_NAME_MAPPING_REPLACEMENT_TEXTBOX, "" );
        $self->type_keys( $VSERVER_NAME_MAPPING_REPLACEMENT_TEXTBOX,
            $args{-namemapping_replacement} );
    } ## end if ( exists( $args{'-namemapping_replacement'...}))

    $Log->comment("click edit name mapping entry ok button");
    my $commit_time = $self->click_timecount($VSERVER_NAME_MAPPING_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("vserver name mapping edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub vserver_namemapping_edit

################################################################################
## @name ::vserver_namemapping_delete
## @summary
##     Deleting name mapping to vserver using GUI.
## @description
##     This method can be used to delete name mapping to vserver using GUI.
## @synopsis
##     vserver_namemapping_delete(%args);
## INPUTS TO METHOD CALL 'vserver_namemapping_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -namemapping_direction </td> <td> (Required) Selecting namemapping_direction </td> </tr>
## <tr> <td> -namemapping_position </td> <td> (Required) Selecting namemapping_position </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_namemapping_delete {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-namemapping_direction' => { type => SCALAR },
            "-namemapping_position"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    
    $Log->step("Navigating to $NAME_MAPPING_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NAME_MAPPING_TREE );

    $Log->comment(
        "Selecting namemapping position $args{'-namemapping_position'}");

    $self->click_child_row_with_texts(
        $VSERVER_NAME_MAPPING_PAGE_GRID,
        $args{'-namemapping_direction'},
        ["$args{'-namemapping_position'}"]
    );

    $Log->comment("click name mapping delete button");
    $self->click($VSERVER_NAME_MAPPING_DELETE_BTN);

    $Log->comment("Waiting for namemapping deletion confirmation box");
    $self->wait_for_element_present( $VSERVER_NAME_MAPPING_DELETE_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("click name mapping delete ok button");
    $self->click($VSERVER_NAME_MAPPING_DELETE_CHECKBOX);

    $Log->comment("click name mapping delete button");
    my $commit_time
        = $self->click_timecount($VSERVER_NAME_MAPPING_DELETE_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("vserver name mapping deletion completed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub vserver_namemapping_delete

################################################################################
## @name ::vserver_namemapping_refresh
## @summary
##     Refresh name mapping to vserver using GUI.
## @description
##     This method can be used to refresh name mapping to vserver using GUI.
## @synopsis
##     vserver_namemapping_refresh(%args);
## INPUTS TO METHOD CALL 'vserver_namemapping_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_namemapping_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAME_MAPPING_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAME_MAPPING_TREE );

    $self->wait_for_element_present($VSERVER_NAME_MAPPING_PAGE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $VSERVER_NAME_MAPPING_REFRESH_BTN,
        "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("Name mapping page refreshed successfully");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub vserver_namemapping_refresh

################################################################################
## @name ::vserver_namemapping_swap
## @summary
##     Swap 2 name mappings using GUI.
## @description
##     This method can be used to swap 2 name mapping to vserver using GUI.
## @synopsis
##     vserver_namemapping_swap(%args);
## INPUTS TO METHOD CALL 'vserver_namemapping_swap' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -namemapping_direction </td> <td> (Required) Selecting namemapping_direction </td> </tr>
## <tr> <td> -namemapping_position </td> <td> (Required) Selecting namemapping_position </td> </tr>
## <tr> <td> -swap_name_position </td> <td> (Required) namemapping_position to be swapped with </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_namemapping_swap {

    my $self = shift;
    my %args = validate_with(
            params => \@_,
            spec   => {
            '-namemapping_direction' => { type => SCALAR },
            '-namemapping_position'  => { type => SCALAR },
            '-swap_name_position'    => { type => SCALAR },
            },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $namemapping_direction = $args{'-namemapping_direction'};
    my $namemapping_position  = $args{'-namemapping_position'};
    my $swap_name_position    = $args{'-swap_name_position'};
    
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    
    $Log->step("Navigating to $NAME_MAPPING_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
            $NAME_MAPPING_TREE );

    $Log->comment("Selecting namemapping position $namemapping_position");
    $self->click_child_row_with_texts(
            $VSERVER_NAME_MAPPING_PAGE_GRID,
            $namemapping_direction, ["$namemapping_position"] );
    $self->control_key_down();

    $Log->comment("Selecting namemapping position $swap_name_position");
    $self->click_child_row_with_texts(
            $VSERVER_NAME_MAPPING_PAGE_GRID,
            $namemapping_direction, ["$swap_name_position"] );
    $self->control_key_up();

    $Log->comment("click name mapping swap button");
    $self->click($VSERVER_NAME_MAPPING_SWAP_BTN);

    $Log->comment("Waiting for namemapping swap confirmation box");
    $self->wait_for_element_present(
            $VSERVER_NAME_MAPPING_SWAP_CHECKBOX,
            $GENERIC_WAIT );

    $Log->comment("click name mapping swap ok button");
    $self->set_checkbox($VSERVER_NAME_MAPPING_SWAP_CHECKBOX, "enable");

    $Log->comment("click name mapping swap button");
    my $commit_time = $self->click_timecount(
            $VSERVER_NAME_MAPPING_SWAP_OK_BTN);

    $Log->comment(
            "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
            );
    my %times = (
            loading_time => $loading_time,
            commit_time  => $commit_time
            );
    
    $Log->comment("vserver name mapping swap completed successfully");
    $Log->exit() if $may_exit;
    
    return \%times;
} ## end sub vserver_namemapping_swap

1;

