# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Namespace Functionalities
##
## @description
##      Using this library, user can perform Cmode Namespace related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmNamespace;
##
##		mount(%args);
##		unmount(%args);
##		change_export_policy(%args);
##		refresh_namespace(%args);
##
## @status public
##
## @author subhasri.s@netapp.com
## @change
##  2011-06-07 : subhasri : Initial version
## @change
##  2012-03-20 : vikasp : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmNamespace;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    mount
    unmount
    change_export_policy
    refresh_namespace
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::mount
## @summary
##     Mount the volume using GUI.
## @description
##     This method can be used to mount the volume using GUI.
## @synopsis
##     mount(%args);
## INPUTS TO METHOD CALL 'mount' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required)Name of the volume to be mounted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -junctionname </td> <td> Name of the junction to be used </td> </tr>
## <tr> <td> -junctionpath </td> <td> Selecting junctionpath </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub mount {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-volname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAMESPACE_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAMESPACE_TREE );
    $Log->comment("Clicking mount button");
    $self->click($NAMESPACE_MOUNT_BUTTON);
    $Log->comment("Waiting for Namespace mount Dialog");
    $self->wait_for_loading();

    $Log->comment("Entering Volume name: $args{'-volname'}");
    $self->set_combobox_value( $NAMESPACE_MOUNT_VOLUME_NAME, $args{'-volname'} );

    #Entering junction name
    if ( exists( $args{'-junctionname'} ) ) {
        $Log->comment("Entering junction name : $args{'-junctionname'}");
        $self->type( $NAMESPACE_MOUNT_JUNCTION_NAME, $args{'-junctionname'} );
    }

    #Entering junction path
    if ( exists( $args{'-junctionpath'} ) ) {
        $Log->comment("Entering junction path : $args{'-junctionpath'}");
        $self->type( $NAMESPACE_MOUNT_JUNCTION_PATH, $args{'-junctionpath'} );
    }

    $Log->comment("Selecting OK button to complete.");

    my $commit_time = $self->click_timecount($NAMESPACE_MOUNT_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Mount created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub mount

################################################################################
## @name ::unmount
## @summary
##     Unmount the volume using GUI.
## @description
##     This method can be used to unmount the volume using GUI.
## @synopsis
##     unmount(%args);
## INPUTS TO METHOD CALL 'unmount' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required)Name of the volume to be unmounted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -force </td> <td> Selecting force checkbox </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub unmount {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-volname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAMESPACE_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAMESPACE_TREE );

    $Log->step("Selecting volume: $args{'-volname'}");
    $self->click_row_with_texts_without_ascending(
                         $NAMESPACE_GRID, $args{'-volname'});

    $Log->comment("Clicking unmount button");
    $self->click($NAMESPACE_UNMOUNT_BUTTON);
    $Log->comment("Waiting for Namespace unmount Dialog");
    $self->wait_for_element_present( $NAMESPACE_UNMOUNT_OK_BUTTON,
        $GENERIC_WAIT );

    #Selecting force checkbox
    if ( exists( $args{'-force'} ) ) {
        $Log->comment("Selecting force unmount option");
        $self->click($NAMESPACE_UNMOUNT_FORCE_CHECKBOX);
    }

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($NAMESPACE_UNMOUNT_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Mount created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unmount

################################################################################
## @name ::change_export_policy
## @summary
##      Change export policy using GUI.
## @description
##     This method can be used to change export policy using GUI.
## @synopsis
##     change_export_policy(%args);
## INPUTS TO METHOD CALL 'change_export_policy' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required)Name of the volume for which the export policy has to be added </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -exportpolicy </td> <td> Name of the exportpolicy </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub change_export_policy {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-volname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAMESPACE_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAMESPACE_TREE );

    $Log->step("Selecting volume: $args{'-volname'}");
    $self->click_row_with_texts_without_ascending(
                         $NAMESPACE_GRID, $args{'-volname'});

    $Log->comment("Clicking change export policy button");
    $self->click($NAMESPACE_CHANGE_EXPORT_POLICY_BUTTON);
    $self->wait_for_loading();

    #Selecting export policy
    if ( exists( $args{'-exportpolicy'} ) ) {
        $Log->comment("Selecting export policy");
        $self->set_combobox_value( $NAMESPACE_CHANGE_EXPORT_POLICY_COMBOBOX,
            $args{'-exportpolicy'} );
        $self->wait_for_loading();
    }

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount(
        $NAMESPACE_CHANGE_EXPORT_POLICY_CHANGE_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Mount created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub change_export_policy

################################################################################
## @name ::refresh_namespace
## @summary
##      Refresh namespace page using GUI.
## @description
##     This method can be used to refresh namespace page using GUI.
## @synopsis
##     refresh_namespace(%args);
## INPUTS TO METHOD CALL 'refresh_namespace' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub refresh_namespace {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $NAMESPACE_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NAMESPACE_TREE );

    $self->wait_for_element_present($NAMESPACE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $NAMESPACE_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub refresh_namespace

1;

