# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Networking GUI Functionalities
## @description
##      Using this library, user can perform Cmode Networking related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmNetworking
##
##		create_network_interface(%args)
##		edit_network_interface(%args)
##		delete_network_interface(%args)
##		refresh_network_interface(%args)
##		send_to_home(%args)
##		enable_network_interface(%args)
##		disable_network_interface(%args)
##		get_network_interface_info(%args)
##
## @status private
##
## @author
##		Vikas P. vikas.p@netapp.com
##
## @change
##		6/16/2011 : Vikas : Initial version
## @change
##      3/20/2012 : Vikas : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmNetworking;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    create_network_interface
    edit_network_interface
    delete_network_interface
    refresh_network_interface
    send_to_home
    enable_network_interface
    disable_network_interface
    get_network_interface_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::create_network_interface
## @summary
##     Create Network Interface based on parameters provided using GUI.
## @description
##     This method can be used to create Network Interface using GUI.
## @synopsis
##     create_network_interface(%args);
## INPUTS TO METHOD CALL 'create_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -interfacename </td> <td>(Required) Name of interface to be created </td> </tr>
## <tr> <td> -netmask </td> <td> Enabling dynamic DNS </td> </tr>
## <tr> <td> -homeport </td> <td> (Required)Enabling homeport </td> </tr>
## <tr> <td> -ipaddress </td> <td> Selecting ipaddress </td> </tr>
## <tr> <td> -protocol </td> <td> Selecting protocol </td> </tr>
## <tr> <td> -subprotocol </td> <td> Selecting subprotocol </td> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -ipv6prefix </td> <td> Enabling DNS cache </td> </tr>
## <tr> <td> -gateway </td> <td> Selecting gateway </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time, Commit Time and Finish Time.

sub create_network_interface {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-interfacename" => { type => SCALAR },
            "-homeport"      => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $Log->comment("Waiting for Network Interface Dialog");
    $self->wait_for_element_present( $NETWORK_INTERFACE_CREATE_BUTTON,
        $GENERIC_WAIT );

    $self->click_button("$NETWORK_INTERFACE_CREATE_BUTTON");
    $Log->comment("Waiting for Network Interface create wizard Next Button");
    $self->wait_for_element_present( $NETWORK_INTERFACE_NEXT_BUTTON,
        $GENERIC_WAIT );
    $self->click_button("$NETWORK_INTERFACE_NEXT_BUTTON");
    $self->wait_for_loading();

    $self->type( "$NETWORK_INTERFACE_NAME_INPUT", $args{'-interfacename'} );

    #setting the value for role
    my $role = exists( $args{'-role'} ) ? $args{'-role'} : "data";

    # Selecting Interface Protocols based on the input parameter
    $Log->comment("Selecting Interface role: $args{'-role'}");
    if ( lc( $role eq "data" ) ) {
        $self->set_option($NETWORK_INTERFACE_DATA_RADIO);
    } elsif ( lc( $role eq "management" ) ) {
        $self->set_option($NETWORK_INTERFACE_MANAGEMENT_RADIO);
    } elsif ( lc( $role eq "both" ) ) {
        $self->set_option($NETWORK_INTERFACE_BOTH_RADIO);
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Invalid value for Interface role, valid values are 'data' or 'management' or 'both'"
        );
    } ## end else [ if ( lc( $role eq "data"...))]

    $self->click_button("$NETWORK_INTERFACE_NEXT_BUTTON");
    $self->wait_for_loading();

    # Selecting Interface protocol based on input parameter
    if ( lc( $role eq "data" ) or lc( $role eq "both" ) ) {
        $Log->comment("Waiting for NAS Protocol radio button");
        $self->wait_for_element_present(
            $NETWORK_INTERFACE_NASPROTOCOLS_RADIO,
            $GENERIC_WAIT );
        if ( exists( $args{'-protocol'} ) ) {
            $Log->comment("Selecting Interface Protocol: $args{'-protocol'}");
            if ( lc( $args{'-protocol'} eq "nas" ) ) {
                $self->set_option($NETWORK_INTERFACE_NASPROTOCOLS_RADIO);

                # Selecting Interface sub protocol based on input parameter
                if ( exists( $args{'-subprotocol'} ) ) {
                    $Log->comment(
                        "Selecting Interface sub Protocol: $args{'-subprotocol'}"
                    );
                    if ( lc( $args{'-subprotocol'} eq "cifs" ) ) {
                        $self->set_checkbox( $NETWORK_INTERFACE_CIFS_CHECKBOX,
                            "enable" );
                        $self->set_checkbox( $NETWORK_INTERFACE_NFS_CHECKBOX,
                            "disable" );
                    } elsif ( lc( $args{'-subprotocol'} eq "nfs" ) ) {
                        $self->set_checkbox( $NETWORK_INTERFACE_CIFS_CHECKBOX,
                            "disable" );
                        $self->set_checkbox( $NETWORK_INTERFACE_NFS_CHECKBOX,
                            "enable" );
                    } elsif ( lc( $args{'-subprotocol'} eq "cifsnfs" ) ) {
                        $self->set_checkbox( $NETWORK_INTERFACE_CIFS_CHECKBOX,
                            "enable" );
                        $self->set_checkbox( $NETWORK_INTERFACE_NFS_CHECKBOX,
                            "enable" );
                    } else {
                        $Log->exit() if $may_exit;
                        NATE::BaseException->throw(
                            "Invalid value for Interface sub protocol for nas, valid values are 'cifs' or 'nfs' or 'cifsnfs'"
                        );
                    } ## end else [ if ( lc( $args{'-subprotocol'...}))]
                } ## end if ( exists( $args{'-subprotocol'...}))
            } elsif ( lc( $args{'-protocol'} eq "san" ) ) {
                $self->set_option($NETWORK_INTERFACE_SANPROTOCOLS_RADIO);

                # Selecting Interface sub protocol based on input parameter
                if ( exists( $args{'-subprotocol'} ) ) {
                    $Log->comment(
                        "Selecting Interface sub Protocol: $args{'-subprotocol'}"
                    );
                    if ( lc( $args{'-subprotocol'} eq "iscsi" ) ) {
                        $self->set_option($NETWORK_INTERFACE_ISCSI_RADIO);
                    } elsif ( lc( $role eq "data" ) ) {
                        if ( lc( $args{'-subprotocol'} eq "fc" ) ) {
                            $self->set_option($NETWORK_INTERFACE_FC_RADIO);
                        } elsif ( lc( $args{'-subprotocol'} eq "fcoe" ) ) {
                            $self->set_option($NETWORK_INTERFACE_FC_RADIO);
                        }
                    } else {
                        $Log->exit() if $may_exit;
                        NATE::BaseException->throw(
                            "Invalid value for Interface sub protocol for san, valid values are 'iscsi' or 'fc' or 'fcoe'"
                        );
                    } ## end else [ if ( lc( $args{'-subprotocol'...}))]
                } ## end if ( exists( $args{'-subprotocol'...}))
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Invalid value for Interface protocol, valid values are 'nas' or 'san'"
                );
            } ## end else [ if ( lc( $args{'-protocol'...}))]
        } ## end if ( exists( $args{'-protocol'...}))
        $self->click_button("$NETWORK_INTERFACE_NEXT_BUTTON");
    } ## end if ( lc( $role eq "data"...))

    $Log->comment("Waiting for Home port input");
    $self->wait_for_loading();

    $self->type( "$NETWORK_INTERFACE_HOMEPORT_INPUT", $args{'-homeport'} );

    if (not(   lc( $args{'-subprotocol'} eq "fc" )
            or lc( $args{'-subprotocol'} eq "fcoe" ) )
        )
    {
        if ( exists( $args{'-ipaddress'} ) ) {
            $Log->comment("Entering ip address: $args{'-ipaddress'}");
            $self->type( "$NETWORK_INTERFACE_IPADDRESS_INPUT",
                $args{'-ipaddress'} );
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "$args{'-ipaddress'} is a mandatory parameter");
        }

        if ( exists( $args{'-netmask'} ) ) {
            $Log->comment("Entering netmask: $args{'-netmask'}");
            $self->type( "$NETWORK_INTERFACE_NETMASK_INPUT",
                $args{'-netmask'} );
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "$args{'-netmask'} is a mandatory parameter");
        }

        if ( exists( $args{'-gateway'} ) ) {
            $Log->comment("Entering gateway: $args{'-gateway'}");
            $self->type( "$NETWORK_INTERFACE_GATEWAY_INPUT",
                $args{-gateway} );
        }
    } ## end if ( not( lc( $args{'-subprotocol'...})))

    $self->click_button("$NETWORK_INTERFACE_NEXT_BUTTON");
    $self->wait_for_loading();

    #my $commit_time = $self->click_timecount( $NETWORK_INTERFACE_NEXT_BUTTON,
    #    "next", $NETWORK_INTERFACE_NEXT_BUTTON );
    
    my $commit_time = 0;
    $self->click( $NETWORK_INTERFACE_NEXT_BUTTON );

    $self->wait_for_element_present( $NETWORK_INTERFACE_FINISH_BUTTON,
        $GENERIC_WAIT );

    #my $finish_time
    #    = $self->click_timecount($NETWORK_INTERFACE_FINISH_BUTTON);
    
    my $finish_time = 0;
    $self->click($NETWORK_INTERFACE_FINISH_BUTTON);

    $self->wait_for_element_not_present( $NETWORK_INTERFACE_FINISH_BUTTON,
        $GENERIC_WAIT );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time,
        finish_time  => $finish_time
    );

    $Log->comment("Interface $args{'-interfacename'} created successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub create_network_interface

################################################################################
## @name ::edit_network_interface
## @summary
##     Edit Network Interface based on parameters using GUI.
## @description
##     This method can be used to edit Network Interface using GUI.
## @synopsis
##     edit_network_interface(%args);
## INPUTS TO METHOD CALL 'edit_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -interfacename </td> <td>(Required) Name of interface to be edited </td> </tr>
## <tr> <td> -netmask </td> <td> Enabling dynamic DNS </td> </tr>
## <tr> <td> -homeport </td> <td> (Required)Enabling homeport </td> </tr>
## <tr> <td> -ipaddress </td> <td> Selecting ipaddress </td> </tr>
## <tr> <td> -protocol </td> <td> Selecting protocol </td> </tr>
## <tr> <td> -subprotocol </td> <td> Selecting subprotocol </td> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -gateway </td> <td> Selecting gateway </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub edit_network_interface {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-interfacename" => { type => SCALAR },
            "-homeport"      => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_loading();

    $self->click_button("$NETWORK_INTERFACE_EDIT_BUTTON");
    $Log->comment("Waiting for Network Interface Dialog");
    $self->wait_for_loading();

    if ( exists( $args{'-homeport'} ) ) {
        $Log->comment("Adding homeport: $args{'-homeport'}");
        $self->type( "$NETWORK_INTERFACE_EDIT_HOMEPORT_INPUT",
            $args{'-homeport'} );
        $self->fire_event( "$NETWORK_INTERFACE_EDIT_HOMEPORT_INPUT", "blur" );
    }

    if ( exists( $args{'-ipaddress'} ) ) {
        $Log->comment("Adding ip address: $args{'-ipaddress'}");
        $self->type( "$NETWORK_INTERFACE_EDIT_IPADDRESS_INPUT",
            $args{'-ipaddress'} );
        $self->fire_event( "$NETWORK_INTERFACE_EDIT_IPADDRESS_INPUT",
            "blur" );
    } ## end if ( exists( $args{'-ipaddress'...}))

    if ( exists( $args{'-netmask'} ) ) {
        $Log->comment("Adding netmask: $args{'-netmask'}");
        $self->type( "$NETWORK_INTERFACE_EDIT_NETMASK_INPUT",
            $args{'-netmask'} );
        $self->fire_event( "$NETWORK_INTERFACE_EDIT_NETMASK_INPUT", "blur" );
    } ## end if ( exists( $args{'-netmask'...}))

    if ( exists( $args{'-gateway'} ) ) {
        $Log->comment("Adding gateway: $args{'-gateway'}");
        $self->type( "$NETWORK_INTERFACE_EDIT_GATEWAY_INPUT",
            $args{'-gateway'} );
        $self->fire_event( "$NETWORK_INTERFACE_EDIT_GATEWAY_INPUT", "blur" );
    }

    $self->wait_for_loading();

    my $commit_time
        = $self->click_timecount($NETWORK_INTERFACE_SAVEANDCLOSE_BUTTON);
    $self->wait_for_element_present( $NETWORK_INTERFACE_EDIT_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("Interface edit completed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub edit_network_interface

################################################################################
## @name ::delete_network_interface
## @summary
##     Delete Network Interface based on parameters using GUI.
## @description
##     This method can be used to edit Network Interface using GUI.
## @synopsis
##     delete_network_interface(%args);
## INPUTS TO METHOD CALL 'delete_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -interfacename </td> <td>(Required) Name of interface to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub delete_network_interface {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-interfacename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_element_present( $NETWORK_INTERFACE_DELETE_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_loading();

    $Log->step(
        "Disabling the interface $args{'-interfacename'} before deletion");
    $self->click_button("$NETWORK_INTERFACE_STATUS_BUTTON");

    $self->click_button("$NETWORK_INTERFACE_DISABLE_BUTTON");
    $self->wait_for_element_present( $NETWORK_INTERFACE_CONFIRMDISABLE_BUTTON,
        $GENERIC_WAIT );

    $self->click_button("$NETWORK_INTERFACE_CONFIRMDISABLE_BUTTON");
    $self->wait_for_element_not_present(
        $NETWORK_INTERFACE_CONFIRMDISABLE_BUTTON,
        $GENERIC_WAIT );

    $self->click_button("$NETWORK_INTERFACE_REFRESH_BUTTON");
    $self->wait_for_element_present( $NETWORK_INTERFACE_REFRESH_BUTTON,
            $GENERIC_WAIT );

    $Log->comment("Waiting for Network Interface Delete button");
    $self->wait_for_loading();

    $self->click_button("$NETWORK_INTERFACE_DELETE_BUTTON");
    $Log->comment(
        "Waiting for Network Interface Dialog delete confirm dialog checkbox"
    );
    $self->wait_for_element_present(
        $NETWORK_INTERFACE_CONFIRMDELETE_CHECKBOX,
        $GENERIC_WAIT );

    $self->set_checkbox( "$NETWORK_INTERFACE_CONFIRMDELETE_CHECKBOX",
        "enable" );

    my $commit_time
        = $self->click_timecount($NETWORK_INTERFACE_CONFIRMDELETE_BUTTON);
    $self->wait_for_element_not_present(
        $NETWORK_INTERFACE_CONFIRMDELETE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Interface deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub delete_network_interface

################################################################################
## @name ::refresh_network_interface
## @summary
##     Refresh Network Interface main page based on parameters using GUI.
## @description
##     This method can be used to refresh Network Interface main page using GUI.
## @synopsis
##     refresh_network_interface(%args);
## INPUTS TO METHOD CALL 'refresh_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub refresh_network_interface {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_element_present( $NETWORK_INTERFACE_REFRESH_BUTTON,
        $GENERIC_WAIT );

    my $commit_time
        = $self->click_timecount( $NETWORK_INTERFACE_REFRESH_BUTTON,
        "refresh" );
    $self->wait_for_element_present( $NETWORK_INTERFACE_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Network Interface details refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub refresh_network_interface

################################################################################
## @name ::send_to_home
## @summary
##     Select send_to_home button using GUI.
## @description
##     This method can be used to select send_to_home button using GUI.
## @synopsis
##     send_to_home(%args);
## INPUTS TO METHOD CALL 'send_to_home' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -interfacename </td> <td> (Required) Name of interface to be sent </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub send_to_home {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-interfacename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_element_present( $NETWORK_INTERFACE_CREATE_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_loading();

    my $commit_time
        = $self->click_timecount( $NETWORK_INTERFACE_SENDTOHOME_BUTTON,
        "refresh" );
    $self->wait_for_element_present( $NETWORK_INTERFACE_CREATE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Selected send to home button successfully");
    $Log->enter() if $may_enter;

    return \%times;

} ## end sub send_to_home

################################################################################
## @name ::enable_network_interface
## @summary
##     Enable the Network Interface using GUI.
## @description
##     This method can be used to enable the Network Interface using GUI.
## @synopsis
##     enable_network_interface(%args);
## INPUTS TO METHOD CALL 'enable_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -interfacename </td> <td> (Required) Name of interface to be enabled </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub enable_network_interface {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-interfacename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_element_present( $NETWORK_INTERFACE_STATUS_BUTTON,
        $GENERIC_WAIT );

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_loading();

    $self->click_button("$NETWORK_INTERFACE_STATUS_BUTTON");
    my $commit_time
        = $self->click_timecount($NETWORK_INTERFACE_ENABLE_BUTTON);
    $self->wait_for_element_present( $NETWORK_INTERFACE_STATUS_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Network Interface enabled successfully");
    $Log->enter() if $may_enter;

    return \%times;

} ## end sub enable_network_interface

################################################################################
## @name ::disable_network_interface
## @summary
##    Disable the Network Interface using GUI.
## @description
##     This method can be used to disable the Network Interface using GUI.
## @synopsis
##     enable_network_interface(%args);
## INPUTS TO METHOD CALL 'enable_network_interface' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -interfacename </td> <td> (Required) Name of interface to be disabled </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub disable_network_interface {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-interfacename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
        $NETWORK_INTERFACE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_loading();

    $self->click_button("$NETWORK_INTERFACE_STATUS_BUTTON");
    $self->wait_for_loading();

    $self->click_button("$NETWORK_INTERFACE_DISABLE_BUTTON");
    $self->wait_for_loading();

    my $commit_time
        = $self->click_timecount($NETWORK_INTERFACE_CONFIRMDISABLE_BUTTON);
    $self->wait_for_element_not_present(
        $NETWORK_INTERFACE_CONFIRMDISABLE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("Network Interface disabled successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub disable_network_interface

################################################################################
## @name ::get_network_interface_info
## @summary
##    Get Network Interface details using GUI.
## @description
##     This method can be used to get Network Interface details using GUI.
## @synopsis
##     get_network_interface_info(%args);
## INPUTS TO METHOD CALL 'get_network_interface_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of vserver to be created </td> </tr>
## <tr> <td> -interfacename </td> <td> (Required) Name of interface for which details has to be fetched </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Network Interface Status details.

sub get_network_interface_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-interfacename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

     my %values = (
         'name',           'wwpn',
         'netmask',        'gateway',
         'protocolaccess', 'managementaccess',
         'status',         'homeport',
         'currentport',    'failover',
         'failovergroup',  'failoverstate'
    );
    my $network_interface_info = {};

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Network Interfaces");
    $self->cluster_tree_navigate( $vserverName, $NETWORK_INTERFACE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting network interface: $args{'-interfacename'}");
    $self->click_row_with_texts( $NETWORK_INTERFACE_GRID,
        $args{'-interfacename'} );
    $self->wait_for_element_present( $NETWORK_INTERFACE_INFO_NAME_LABEL,
        $GENERIC_WAIT );

    $values{"$NETWORK_INTERFACE_INFO_NAME"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_NAME_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_NAME_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_WWPN"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_WWPN_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_WWPN_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_NETMASK"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_NETMASK_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_NETMASK_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_GATEWAY"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_GATEWAY_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_GATEWAY_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_PROTOCOLACCESS"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_PROTOCOLACCESS_LABEL")
        if (
        $self->is_element_present(
            "$NETWORK_INTERFACE_INFO_PROTOCOLACCESS_LABEL")
        );

    $values{"$NETWORK_INTERFACE_INFO_MANAGEMENTACCESS"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_MANAGEMENTACCESS_LABEL")
        if (
        $self->is_element_present(
            "$NETWORK_INTERFACE_INFO_MANAGEMENTACCESS_LABEL")
        );

    $values{"$NETWORK_INTERFACE_INFO_STATUS"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_STATUS_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_STATUS_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_HOMEPORT"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_HOMEPORT_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_HOMEPORT_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_CURRENTPORT"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_CURRENTPORT_LABEL")
        if (
        $self->is_element_present(
            "$NETWORK_INTERFACE_INFO_CURRENTPORT_LABEL")
        );

    $values{"$NETWORK_INTERFACE_INFO_FAILOVER"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_FAILOVER_LABEL")
        if (
        $self->is_element_present("$NETWORK_INTERFACE_INFO_FAILOVER_LABEL") );

    $values{"$NETWORK_INTERFACE_INFO_FAILOVERGROUP"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_FAILOVERGROUP_LABEL")
        if (
        $self->is_element_present(
            "$NETWORK_INTERFACE_INFO_FAILOVERGROUP_LABEL")
        );

    $values{"$NETWORK_INTERFACE_INFO_FAILOVERSTATE"}
        = $self->get_text("$NETWORK_INTERFACE_INFO_FAILOVERSTATE_LABEL")
        if (
        $self->is_element_present(
            "$NETWORK_INTERFACE_INFO_FAILOVERSTATE_LABEL")
        );

    $self->check_for_error();

    $Log->comment("Collection of network interface info is successfull");
    $network_interface_info->{parsed_output} = \%values;
    $Log->exit() if $may_exit;

    return $network_interface_info;

} ## end sub get_network_interface_info

1;
