#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
## @summary Routines to handle CMode NFS CIFS related activities
##
## @description
##    This is a library handle CMode NFS CIFS related functionalities.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmNfsCifs
##
## @synopsis
##    use SmNfsCifs;
##
##    setup_cifs(%args);
##    edit_cifs_options(%args);
##    cifs_refresh(%args);
##    cifs_configuration_manage_add(%args)
##    cifs_configuration_manage_delete(%args)
##    cifs_domain_group_policy_enable();
##    cifs_domain_group_policy_disable();
##    cifs_domain_group_policy_reload();
##    cifs_domain_group_policy_update();
##    cifs_domain_group_policy_view(%args);
##    cifs_symlinks_create(%args);
##    cifs_symlinks_edit(%args);
##    cifs_symlinks_delete(%args);
##    cifs_symlink_refresh(%args);
##    nfs_enable(%args);
##    nfs_diasble(%args);
##    nfs_edit(%args);
##    get_nfs_info(%args);
##    nfs_refresh(%args);
##    create_nfs_datastore(%args);
##    cifs_reset_domain(%args);
##
## @status public
##
## @author Vinu Nagarajan  Vinu.Nagarajan@netapp.com
##
## @change
##  6/25/2011 : Vikas : Initial Version
## @change
##  3/20/2012 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmNfsCifs;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    setup_cifs
    edit_cifs_options
    cifs_refresh
    cifs_configuration_manage_add
    cifs_configuration_manage_delete
    cifs_domain_group_policy_enable
    cifs_domain_group_policy_disable
    cifs_domain_group_policy_reload
    cifs_domain_group_policy_update
    cifs_domain_group_policy_view
    cifs_symlinks_create
    cifs_symlinks_edit
    cifs_symlinks_delete
    cifs_symlink_refresh
    nfs_enable
    nfs_disable
    nfs_edit
    nfs_refresh
    get_nfs_info
    edit_nfs_settings
    start_cifs_service
    stop_cifs_service
    edit_cifs_domain_options
    cifs_lookup
    edit_cifs_auditing_options
    cifs_clear_log
    edit_cifs_home_directory
    get_cifs_configuration_info
    clear_window_cifs_main
    create_nfs_datastore
    cifs_reset_domain
    get_cifs_domaincontrollers_info
    get_cifs_symlinks_info
    );

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::setup_cifs
## @summary
##     Setup CIFS using GUI.
## @description
##     This method can be used to  setup CIFS using GUI.
## @synopsis
##     setup_cifs(%args);
## INPUTS TO METHOD CALL 'setup_cifs' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -biosname </td> <td> Entering bios name </td> </tr>
## <tr> <td> -domainname </td> <td> Entering domain name </td> </tr>
## <tr> <td> -adminname </td> <td> Entering admin name </td> </tr>
## <tr> <td> -adminpassword </td> <td> Entering admin password </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub setup_cifs {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>cifs");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );
    $Log->comment("Selecting CIFS configuration button");
    $self->click($CIFS_CONFIGURATION_TAB);

    my $val = $self->get_text($CIFS_DELETE);

    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS setup button");
    $self->click($CIFS_SETUP_BTN);

    if ( $val eq "Configured" ) {
        $Log->comment("Deletion of shares and home directories");
        sleep(5); 
        $self->wait_for_loading();
        $self->set_checkbox( "$CIFS_SETUP_CONFIRMATION", "enable" );
        $self->click("$CIFS_STOP_CONFIRM_BUTTON");
    }

    $self->wait_for_element_present( $CIFS_SETUP_OK_BTN, $GENERIC_WAIT );

    # Entering Domain name, admin name and password
    if ( exists( $args{'-biosname'} ) ) {
        $Log->comment("Entering bios name");
        $self->wait_for_element_present( $CIFS_SETUP_NETBIOS_NAME_TEXTBOX,
            $GENERIC_WAIT );
        $self->type( "$CIFS_SETUP_NETBIOS_NAME_TEXTBOX", $args{'-biosname'} );
    } ## end if ( exists( $args{'-biosname'...}))
    if (   exists( $args{'-biosname'} )
        && exists( $args{'-domainname'} )
        && exists( $args{'-adminname'} )
        && exists( $args{'-adminpassword'} ) )
    {
        $Log->comment("Entering bios name");
        $self->wait_for_element_present( $CIFS_SETUP_NETBIOS_NAME_TEXTBOX,
            $GENERIC_WAIT );
        $self->type( "$CIFS_SETUP_NETBIOS_NAME_TEXTBOX", $args{'-biosname'} );

        $Log->comment("Entering Domain name");
        $self->wait_for_element_present( $CIFS_SETUP_DOMAIN_NAME_TXTBOX,
            $GENERIC_WAIT );
        $self->type( "$CIFS_SETUP_DOMAIN_NAME_TXTBOX", $args{'-domainname'} );

        $Log->comment("Entering Admin name");
        $self->wait_for_element_present( $CIFS_SETUP_DOMAINUSER_NAME_TEXTBOX,
            $GENERIC_WAIT );
        $self->type( "$CIFS_SETUP_DOMAINUSER_NAME_TEXTBOX",
            $args{'-adminname'} );

        $Log->comment("Entering Admin password");
        $self->type( "$CIFS_SETUP_DOMAINUSER_PASSWORD_TEXTBOX",
            $args{'-adminpassword'} );
    } else {
        $Log->comment(
            "options are missing, please provide all the parameter values");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Mandatory parameters are missing");
    } ## end else [ if ( exists( $args{'-biosname'...}))]

    $Log->comment("Selecting Finish button");
    my $commit_time = $self->click_timecount("$CIFS_SETUP_OK_BTN");

    $self->check_for_error();
    #$self->wait_for_element_not_present( $CIFS_SETUP_OK_BTN, $GENERIC_WAIT );
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS settings done\n");
    $Log->exit() if $may_exit;
    return \%times;

} ## end sub setup_cifs

################################################################################
## @name ::edit_cifs_options
## @summary
##     Edit the options for CIFS using GUI.
## @description
##     This method can be used to edit the options for CIFS using GUI.
## @synopsis
##     setup_cifs(%args);
## INPUTS TO METHOD CALL 'setup_cifs' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -defaultunixuser </td> <td> Selecting defaultunixuser </td> </tr>
## <tr> <td> -win_ip_address </td> <td> Selecting win_ip_address </td> </tr>
## <tr> <td> -delwinsip </td> <td> Deleting win server ip </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub edit_cifs_options {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $self->wait_for_loading();
    #$self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS configuration button");
    $self->click("$CIFS_CONFIGURATION_TAB");
    $self->wait_for_loading();

#    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    $Log->comment(
        "Selecting CIFS options edit button $args{'-defaultunixuser'} $args{'-win_ip_address'} "
    );
    $self->click($CIFS_EDIT_OPTION);
    $self->wait_for_loading();
    #$self->wait_for_element_present( $CIFS_IP_ADDRESS_ADD, $GENERIC_WAIT );

    if ( exists( $args{'-defaultunixuser'} ) ) {
        $self->wait_for_loading();
        $Log->comment(
            "Entering default unix user name $args{'-defaultunixuser'}");
        $self->type_keys( $CIFS_EDIT_DEFAULT_USER,
            $args{'-defaultunixuser'} );

        if ( exists( $args{'-win_ip_address'} ) ) {
            $self->wait_for_loading();
            $Log->comment("Entering win ip address: $args{'-win_ip_address'}");
            $self->click($CIFS_IP_ADDRESS_ADD);
            $self->wait_for_loading();
            $self->type_keys( $CIFS_IP_ADDRESS_ADD_TEXT,
                $args{'-win_ip_address'} );
            $self->type( $CIFS_IP_ADDRESS_ADD_TEXT,
                $args{'-win_ip_address'} );
            $self->fire_event( $CIFS_IP_ADDRESS_ADD_TEXT, "blur" );
            $self->wait_for_element_present( $CIFS_WINIP_ADDRESS_SAVE_BTN,
                $GENERIC_WAIT );
            $Log->comment("Saving win ip address");
            $self->click($CIFS_WINIP_ADDRESS_SAVE_BTN);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-win_ip_address'...}))
        if ( exists( $args{'-delwinsip'} ) ) {
            $Log->comment("Selecting the WINS Server from the Grid");
            $self->click_row_with_texts( $CIFS_OPTIONS_WINS_SERVER_GRID,
                $args{'-delwinsip'} );

            $Log->comment("Clicking on the Delete button");
            $self->click($CIFS_OPTIONS_WINS_SERVER_DELETE_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-delwinsip'...}))
    } else {
        $Log->comment(
            "mandatory options are missing, please provide required parameter values"
        );
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("mandatory options are missing");
    } ## end else [ if ( exists( $args{'-defaultunixuser'...}))]

    $self->wait_for_element_present( $CIFS_OPTIONS_SAVE_AND_CLOSE_BTN,
        $GENERIC_WAIT );

    $Log->comment("Selecting Save and Close  button to complete.");
    my $commit_time
        = $self->click_timecount($CIFS_OPTIONS_SAVE_AND_CLOSE_BTN);

    $self->wait_for_element_not_present( $CIFS_OPTIONS_SAVE_AND_CLOSE_BTN,
        $GENERIC_WAIT );
    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("CIFS option settings done\n");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub edit_cifs_options

################################################################################
## @name ::cifs_configuration_manage_add
## @summary
##     Add home directory path for CIFS using GUI.
## @description
##     This method can be used to add home directory path for CIFS GUI.
## @synopsis
##     cifs_configuration_manage_add(%args);
## INPUTS TO METHOD CALL 'cifs_configuration_manage_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -homedirpath </td> <td> Entering home directory path </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_configuration_manage_add {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );
    $self->wait_for_loading();
#    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS configuration button");
    $self->click($CIFS_CONFIGURATION_TAB);
    $self->wait_for_loading();

    #$self->wait_for_element_present( $CIFS_MANAGE, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS manage");
    $self->click($CIFS_MANAGE);
    $self->wait_for_loading();
    #$self->wait_for_element_present( $CIFS_MANAGE_HOMEDIR_ADD,
    #    $GENERIC_WAIT );

    if ( exists( $args{'-homedirpath'} ) ) {
        $Log->comment("Entering home directory path");
        $self->wait_for_loading();
        $self->type_keys( $CIFS_MANAGE_HOME_DIR_TEXT, $args{'-homedirpath'} );
        $self->type( $CIFS_MANAGE_HOME_DIR_TEXT, $args{'-homedirpath'} );
        $self->fire_event( $CIFS_MANAGE_HOME_DIR_TEXT, "blur" );

        $Log->comment("Saving home dir path address");
        $self->click($CIFS_MANAGE_HOMEDIR_ADD);
        $self->wait_for_loading();
        #$self->wait_for_element_present( $CIFS_MANAGE_HOMEDIR_DELETE,
         #   $GENERIC_WAIT );

        $commit_time
            = $self->click_timecount($CIFS_MANAGE_HOMEDIR_SAVE_AND_CLOSE);
    } else {
        $Log->comment(
            "mandatory options are missing, please provide required parameter values"
        );
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "mandatory options are missing, please provide required parameter values"
        );
    } ## end else [ if ( exists( $args{'-homedirpath'...}))]

    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("Addition of CIFS home directory done\n");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cifs_configuration_manage_add

################################################################################
## @name ::cifs_configuration_manage_delete
## @summary
##    Delete home directory path for CIFS using GUI.
## @description
##     This method can be used to delete home directory path for CIFS GUI.
## @synopsis
##     cifs_configuration_manage_delete(%args);
## INPUTS TO METHOD CALL 'cifs_configuration_manage_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -homedirpath </td> <td> Entering home directory path </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_configuration_manage_delete {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );
    $self->wait_for_loading();

    #$self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS configuration button");
    $self->click("$CIFS_CONFIGURATION_TAB");
    $self->wait_for_loading();

    #$self->wait_for_element_present( $CIFS_MANAGE, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS manage");
    $self->click("$CIFS_MANAGE");
    $self->wait_for_loading();
    #$self->wait_for_element_present( $CIFS_MANAGE_HOMEDIR_DELETE,
    #    $GENERIC_WAIT );

    if ( exists( $args{'-homedirpath'} ) ) {
        $Log->comment("Deleting home directory path");
        $self->click_row_with_texts( "$CIFS_EDIT_HOME_DIR_PATH_GRID",
            $args{'-homedirpath'} );

        $self->click("$CIFS_MANAGE_HOMEDIR_DELETE");
        $commit_time
            = $self->click_timecount("$CIFS_MANAGE_HOMEDIR_SAVE_AND_CLOSE");
        $self->wait_for_loading();
    } else {
        $Log->comment(
            "mandatory options are missing, please provide required parameter values"
        );
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "mandatory options are missing, please provide required parameter values"
        );
    } ## end else [ if ( exists( $args{'-homedirpath'...}))]

    $self->wait_for_element_present( $CIFS_MANAGE, $GENERIC_WAIT );
    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS manage home directory done\n");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cifs_configuration_manage_delete

################################################################################
## @name ::cifs_refresh
## @summary
##    Refresh home directory path for CIFS using GUI.
## @description
##     This method can be used to refresh home directory path for CIFS GUI.
## @synopsis
##     cifs_refresh(%args);
## INPUTS TO METHOD CALL 'cifs_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_refresh {
    my ( $self, %args ) = @_;

    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );
    $Log->comment("Selecting CIFS configuration button");
    $self->click("$CIFS_CONFIGURATION_TAB");

    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB, $GENERIC_WAIT );

    #Click the Refresh Button
    if ( $self->is_element_present("$CIFS_REFRESH_BTN") ) {
        $commit_time
            = $self->click_timecount( "$CIFS_REFRESH_BTN", "refresh" );
    } else {
        $self->nlog("Could not find the Refresh Button");
    }
    $self->check_for_error();
    $self->wait_for_element_present( $CIFS_REFRESH_BTN, $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS option settings done\n");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cifs_refresh

################################################################################
## @name ::cifs_domain_group_policy_reload
## @summary
##    Update cifs group policy CIFS using GUI.
## @description
##     This method can be used to refresh home directory path for CIFS GUI.
## @synopsis
##     cifs_domain_group_policy_reload(%args);
## INPUTS TO METHOD CALL 'cifs_domain_group_policy_reload' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_domain_group_policy_reload {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );
    
    $self->wait_for_loading();

    $Log->comment("Selecting CIFS domain tab");
    $self->click($CIFS_DOMAIN_TAB);
    $self->wait_for_loading();

    $Log->comment("Selecting CIFS domain reload");
    $commit_time = $self->click_timecount( $CIFS_DOMAIN_GROUP_POLICY_RELOAD,
        "refresh" );
    $self->wait_for_loading();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS domain reloaded successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_domain_group_policy_reload

################################################################################
## @name ::cifs_domain_group_policy_enable
## @summary
##    Enable group policies for CIFS using GUI.
## @description
##     This method can be used to enable group policies for for CIFS GUI.
## @synopsis
##     cifs_domain_group_policy_enable(%args);
## INPUTS TO METHOD CALL 'cifs_domain_group_policy_enable' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_domain_group_policy_enable {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );
#    $self->wait_for_element_present( $CIFS_DOMAIN_TAB, $GENERIC_WAIT );
    $self->wait_for_loading();        

    $Log->comment("Selecting CIFS domain tab");
    $self->click("$CIFS_DOMAIN_TAB");
    $self->wait_for_loading();        
    #$self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE,
    #    $GENERIC_WAIT );

    if ( $self->get_text($CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE) eq
        "Enable" )
    {
        $commit_time = $self->click_timecount(
            $CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE, "refresh" );
        $self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_UPDATE,
            $GENERIC_WAIT );
        $self->wait_for_loading();        
    } else {
        $Log->comment("FAILED: cifs group policy is already enabled");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("cifs group policy is already enabled");
    }

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS group policy enabled successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_domain_group_policy_enable

################################################################################
## @name ::cifs_domain_group_policy_disable
## @summary
##    Disable group policies for CIFS using GUI.
## @description
##     This method can be used to Disable group policies for CIFS GUI.
## @synopsis
##     cifs_domain_group_policy_disable(%args);
## INPUTS TO METHOD CALL 'cifs_domain_group_policy_disable' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_domain_group_policy_disable {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );
    #$self->wait_for_element_present( $CIFS_DOMAIN_TAB, $GENERIC_WAIT );
    $self->wait_for_loading();

    $Log->comment("Selecting CIFS domain tab");
    $self->click($CIFS_DOMAIN_TAB);
    $Log->comment("Waiting for Disable/Enable Button");
    #$self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE,
    #    $GENERIC_WAIT );
    $self->wait_for_loading();

    if ( $self->get_text($CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE) eq
        "Disable" )
    {
        $Log->comment("Disabling group policy");
        $commit_time = $self->click_timecount(
            "$CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE", "refresh" );
    } else {
        $Log->comment("cifs group policy is already disabled");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("cifs group policy is already disbled");
    }

    $self->wait_for_loading();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS group policy disabled successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_domain_group_policy_disable

################################################################################
## @name ::cifs_domain_group_policy_update
## @summary
##    Update group policies for CIFS Domain using GUI.
## @description
##     This method can be used to update group policies for CIFS Domain using GUI.
## @synopsis
##     cifs_domain_group_policy_disable(%args);
## INPUTS TO METHOD CALL 'cifs_domain_group_policy_disable' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_domain_group_policy_update {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $self->wait_for_element_present( $CIFS_DOMAIN_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS domain tab");
    $self->click("$CIFS_DOMAIN_TAB");
    if ( $self->get_text("$CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE") eq
        "Enable" )
    {
        $Log->comment("Selecting CIFS domain group policy enabling");
        $self->click("$CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE");
        $self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_UPDATE,
            $GENERIC_WAIT );
        $Log->comment("Selecting CIFS domain group policy updating");
        $self->click("$CIFS_DOMAIN_GROUP_POLICY_UPDATE");
    } ## end if ( $self->get_text("$CIFS_DOMAIN_GROUP_POLICY_ENABLE_DISABLE"...))

    $self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_UPDATE,
        $GENERIC_WAIT );
    $commit_time = $self->click_timecount( "$CIFS_DOMAIN_GROUP_POLICY_UPDATE",
        "refresh" );

    $self->wait_for_element_present( $CIFS_DOMAIN_GROUP_POLICY_UPDATE,
        $GENERIC_WAIT );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS group policy updated successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_domain_group_policy_update

################################################################################
## @name ::cifs_domain_group_policy_view
## @summary
##    View group policies for CIFS Domain using GUI.
## @description
##     This method can be used to view group policies for CIFS Domain using GUI.
## @synopsis
##     cifs_domain_group_policy_view(%args);
## INPUTS TO METHOD CALL 'cifs_domain_group_policy_view' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_domain_group_policy_view {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-group_policy" => { type => SCALAR }, },
        allow_extra => 1
    );
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $self->wait_for_element_present( $CIFS_DOMAIN_TAB, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS domain tab");
    $self->click("$CIFS_DOMAIN_TAB");

    $Log->comment("Reloading the CIFS domain screen");
    $self->click("$CIFS_DOMAIN_GROUP_POLICY_RELOAD");
    $self->wait_for_element_present( $CIFS_GROUP_POLICY_GRID, $GENERIC_WAIT );

    $Log->comment("*** Select group policy $args{'-group_policy'} ***");
    $self->click_row_with_texts( "$CIFS_GROUP_POLICY_GRID",
        $args{'-group_policy'} );
    $self->click("$CIFS_DOMAIN_GROUP_POLICY_VIEW");

    if ( $self->is_element_present($CIFS_DOMAIN_GROUP_POLICY_UPDATE_OK_BTN) )
    {
        my $group_view_text
            = $self->get_text($CIFS_DOMAIN_GROUP_POLICY_UPDATE_OK_BTN);
        $Log->comment(
            "Details of $args{'-group_policy'} is $group_view_text");
        $commit_time
            = $self->click_timecount($CIFS_DOMAIN_GROUP_POLICY_UPDATE_OK_BTN);
    } ## end if ( $self->is_element_present...)

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS group policy viewed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_domain_group_policy_view

################################################################################
## @name ::cifs_symlinks_create
## @summary
##    Create symlinks for CIFS Domain using GUI.
## @description
##     This method can be used to create symlinks for CIFS Domain using GUI.
## @synopsis
##     cifs_symlinks_create(%args);
## INPUTS TO METHOD CALL 'cifs_symlinks_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -unixpath </td> <td> Selecting unix path </td> </tr>
## <tr> <td> -cifs_share </td> <td>  Selecting cifs share </td> </tr>
## <tr> <td> -cifs_path </td> <td> Selecting cifs path </td> </tr>
## <tr> <td> -remote_server </td> <td> Selecting cifs path </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_symlinks_create {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $Log->step("Navigating to SYMLINK tab");
    $self->click($CIFS_SYMLINKS_TAB);
    $self->wait_for_loading();
    #$self->wait_for_element_present( $CIFS_SYMLINKS_CREATE, $GENERIC_WAIT );

    $Log->comment("Selecting CIFS symlinks button");
    $self->click($CIFS_SYMLINKS_CREATE);
    $self->wait_for_loading();

    if (   exists( $args{'-unix_path'} )
        && exists( $args{'-cifs_share'} )
        && exists( $args{'-cifs_path'} ) )
    {
        $Log->comment("Entering valid unix path");
        $self->wait_for_element_present( $CIFS_SYMLINKS_UNIX_PATH,
            $GENERIC_WAIT );
        $self->type( $CIFS_SYMLINKS_UNIX_PATH, $args{'-unix_path'} );

        $Log->comment("Entering already created cifs share name");
        $self->wait_for_element_present( $CIFS_SYMLINKS_CIFS_SHARE,
            $GENERIC_WAIT );
        $self->type_keys( $CIFS_SYMLINKS_CIFS_SHARE, $args{'-cifs_share'} );
        $self->type( $CIFS_SYMLINKS_CIFS_SHARE, $args{'-cifs_share'} );

        $Log->comment("Entering cifs path");
        $self->wait_for_element_present( $CIFS_SYMLINKS_CIFS_PATH,
            $GENERIC_WAIT );
        $self->type( $CIFS_SYMLINKS_CIFS_PATH, $args{'-cifs_path'} );

    } else {
        $Log->comment(
            "options are missing, please provide all the parameter values");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "options are missing, please provide all the parameter values");
    } ## end else [ if ( exists( $args{'-unix_path'...}))]

    if ( exists( $args{-remote_server} ) ) {
        $Log->comment("Click to enable remote server ");
        $self->set_checkbox( $CIFS_SYMLINKS_REMOTE_SERVER_CHECK_BOX,
            "enable" );
        $self->wait_for_element_present( $CIFS_SYMLINKS_REMOTE_SERVER_TEXT,
            $GENERIC_WAIT );
        $self->type( $CIFS_SYMLINKS_REMOTE_SERVER_TEXT,
            $args{'-remote_server'} );
    } ## end if ( exists( $args{-remote_server...}))

    $self->wait_for_element_present( $CIFS_SYMLINKS_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Selecting symlink create button");
    $self->click($CIFS_SYMLINKS_OK_BUTTON);
    $self->wait_for_loading();
    $Log->comment("selecting refresh button to reflect created symlink");

    $commit_time = $self->click_timecount( $CIFS_REFRESH_BTN, "refresh" );
    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS symlinks established\n");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_symlinks_create

#######################################################################################
## @name ::cifs_symlinks_edit
## @summary
##    Edit symlinks for CIFS Domain using GUI.
## @description
##     This method can be used to edit symlinks for CIFS Domain using GUI.
## @synopsis
##     cifs_symlinks_edit(%args);
## INPUTS TO METHOD CALL 'cifs_symlinks_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -unixpath </td> <td> (Required) Selecting unix path </td> </tr>
## <tr> <td> -cifs_share </td> <td>  Selecting cifs share </td> </tr>
## <tr> <td> -cifs_path </td> <td> Selecting cifs path </td> </tr>
## <tr> <td> -remote_server </td> <td> Selecting cifs path </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_symlinks_edit {

    my $commit_time;
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-unix_path" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $Log->step("Navigating to SYMLINK tab");
    $self->click($CIFS_SYMLINKS_TAB);
    $self->click_row_with_texts( "$SYMLINK_GRID", $args{'-unix_path'} );

    $Log->comment("Selecting CIFS symlinks edit button");
    $self->wait_for_element_present( $CIFS_SYMLINKS_EDIT, $GENERIC_WAIT );
    $self->click("$CIFS_SYMLINKS_EDIT");
    $self->wait_for_element_present( $CIFS_SYMLINKS_CANCEL_BUTTON,
        $GENERIC_WAIT );

    if (   exists( $args{'-cifs_share'} )
        && exists( $args{'-cifs_path'} ) )
    {
        $Log->comment("Entering cifs share name");
        $self->wait_for_element_present( $CIFS_SYMLINKS_SHARE_TEXT,
            $GENERIC_WAIT );
        $self->type( $CIFS_SYMLINKS_SHARE_TEXT, $args{'-cifs_share'} );

        $Log->comment("Entering cifs path");
        $self->wait_for_element_present( $CIFS_SYMLINKS_CIFS_PATH_TEXT,
            $GENERIC_WAIT );
        $self->type_keys( $CIFS_SYMLINKS_CIFS_PATH_TEXT,
            $args{'-cifs_path'} );
    } else {
        $Log->comment(
            "options are missing, please provide all the parameter values");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "options are missing, please provide all the parameter values");
    } ## end else [ if ( exists( $args{'-cifs_share'...}))]

    if ( exists( $args{-remote_server} ) ) {
        $Log->comment("Click to enable remote server ");
        $self->set_checkbox( $CIFS_SYMLINKS_REMOTE_SERVER_CHECK_BOX,
            lc( $args{-remote_server} ) );
        $self->wait_for_element_present(
            $CIFS_SYMLINKS_EDIT_REMEOTE_SERVER_TEXT,
            $GENERIC_WAIT );
        $self->type( $CIFS_SYMLINKS_EDIT_REMEOTE_SERVER_TEXT,
            $args{'-remote_server'} );
    } ## end if ( exists( $args{-remote_server...}))

    $Log->comment("Selecting symlink save and close button");
    $commit_time = $self->click_timecount($CIFS_SYMLINKS_SAVE_CLOSE_BTN);

    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS symlinks has been edited\n");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_symlinks_edit

################################################################################
## @name ::cifs_symlinks_delete
## @summary
##    Delete symlinks for CIFS Domain using GUI.
## @description
##     This method can be used to delete symlinks for CIFS Domain using GUI.
## @synopsis
##     cifs_symlinks_delete(%args);
## INPUTS TO METHOD CALL 'cifs_symlinks_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_symlinks_delete {

    my $commit_time;
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-unix_path" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");

    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    if ( exists( $args{-unix_path} ) ) {
        $Log->step("Navigating to SYMLINK tab");
        $self->click("$CIFS_SYMLINKS_TAB");
        $self->wait_for_loading();

        $Log->comment("*** Select unix path $args{'-unix_path'}  ***");
        $self->click_row_with_texts( "$SYMLINK_GRID", $args{'-unix_path'} );
        $self->wait_for_element_present( $CIFS_SYMLINK_DELETE,
            $GENERIC_WAIT );

        $Log->comment("Selecting CIFS symlinks delete button");
        $self->click("$CIFS_SYMLINK_DELETE");
        $self->wait_for_loading();
        $Log->comment("Confirming deletion of symlink");

        $commit_time = $self->click_timecount("$CIFS_SYMLINK_CONFIRM_DELETE");
    } ## end if ( exists( $args{-unix_path...}))

    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Deletion of symlink done");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_symlinks_delete

################################################################################
## @name ::cifs_symlink_refresh
## @summary
##    Refresh symlinks for CIFS Domain using GUI.
## @description
##     This method can be used to refresh symlinks for CIFS Domain using GUI.
## @synopsis
##     cifs_symlink_refresh(%args);
## INPUTS TO METHOD CALL 'cifs_symlink_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_symlink_refresh {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $CIFS_TREE );

    $Log->step("Navigating to SYMLINK tab");
    $self->click($CIFS_SYMLINKS_TAB);
    $self->wait_for_element_present( $CIFS_SYMLINK_REFRESH_BTN,
        $GENERIC_WAIT );

    #Click the Refresh Button
    if ( $self->is_element_present("$CIFS_SYMLINK_REFRESH_BTN") ) {
        $commit_time = $self->click_timecount( "$CIFS_SYMLINK_REFRESH_BTN",
            "refresh" );
    } else {
        $self->nlog("Could not find the Refresh Button");
    }

    $self->wait_for_element_present( $CIFS_SYMLINK_REFRESH_BTN,
        $GENERIC_WAIT );
    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("CIFS symlink refresh done\n");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub cifs_symlink_refresh

################################################################################
## @name ::nfs_enable
## @summary
##    Enable nfs for CIFS using GUI.
## @description
##     This method can be used to enable nfs for CIFS using GUI.
## @synopsis
##     nfs_enable(%args);
## INPUTS TO METHOD CALL 'nfs_enable' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub nfs_enable {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>NFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NFS_TREE );

    $Log->step("Click enable nfs button");
    $self->wait_for_element_present( $NFS_ENABLE_BTN, $GENERIC_WAIT );
    if ( $self->is_element_present($NFS_ENABLE_BTN) ) {
        $commit_time = $self->click_timecount( $NFS_ENABLE_BTN, "refresh" );
    } else {
        $self->nlog("Could not find the enable Button");
    }

    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("enable nfs is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub nfs_enable

################################################################################
## @name ::nfs_disable
## @summary
##    Disable nfs for CIFS using GUI.
## @description
##     This method can be used to disable nfs CIFS using GUI.
## @synopsis
##     nfs_disable(%args);
## INPUTS TO METHOD CALL 'nfs_disable' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub nfs_disable {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>NFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NFS_TREE );

    $Log->step("Click disable nfs button");
    $self->wait_for_element_present( $NFS_DISABLE_BTN, $GENERIC_WAIT );

    if ( $self->is_element_present("$NFS_DISABLE_BTN") ) {
        $commit_time = $self->click($NFS_DISABLE_BTN);
    } else {
        $self->nlog("Could not find the disable Button");
    }

    $Log->step("Wait for disable NFS dialog");
    $self->wait_for_element_present( $NFS_DISABLE_OK_BTN, $GENERIC_WAIT );

    $Log->step("click nfs disable ok button");
    $self->click($NFS_DISABLE_OK_BTN);
    $self->wait_for_element_present( $NFS_ENABLE_BTN, $GENERIC_WAIT );

    $self->check_for_error();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("disable nfs is successfull");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub nfs_disable

################################################################################
## @name ::nfs_refresh
## @summary
##   Refresh nfs for CIFS using GUI.
## @description
##     This method can be used to Refresh nfs for CIFS using GUI.
## @synopsis
##     nfs_refresh(%args);
## INPUTS TO METHOD CALL 'nfs_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub nfs_refresh {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>NFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NFS_TREE );
    $self->wait_for_element_present( $NFS_REFRESH_BTN, $GENERIC_WAIT );

    #Click the Refresh Button
    if ( $self->is_element_present("$NFS_REFRESH_BTN") ) {
        $commit_time
            = $self->click_timecount( "$NFS_REFRESH_BTN", "refresh" );
    } else {
        $self->nlog("Could not find the nfs Refresh Button");
    }

    $self->wait_for_element_present( $NFS_REFRESH_BTN, $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("nfs refresh done\n");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub nfs_refresh

################################################################################
## @name ::nfs_edit
## @summary
##   Refresh nfs for CIFS using GUI.
## @description
##     This method can be used to edit nfs using GUI.
## @synopsis
##     nfs_edit(%args);
## INPUTS TO METHOD CALL 'nfs_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -support_version2 </td> <td> Enable Version 2 support </td> </tr>
## <tr> <td> -support_version3 </td> <td> Enable Version 3 support </td> </tr>
## <tr> <td> -support_version4_0 </td> <td> Enable Version 4.0 support </td> </tr>
## <tr> <td> -nfs_acl </td> <td> Enable NFS ACL </td> </tr>
## <tr> <td> -nfs_read_delagation </td> <td> Enable NFS read delegation </td> </tr>
## <tr> <td> -nfs_write_delegation </td> <td> Enable NFS write delegation </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub nfs_edit {

    my ( $self, %args ) = @_;
    my $commit_time;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>NFS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $NFS_TREE );
    $self->wait_for_element_present( $NFS_CONFIG_EDIT_BUTTON, $GENERIC_WAIT );

    $Log->step("Editing NFS configuration");
    $self->click("$NFS_CONFIG_EDIT_BUTTON");

    if ( exists( $args{-support_version2} ) ) {
        $Log->comment("Click to enable support  version 2 ");
        $self->set_checkbox( "$NFS_SUPPORT_VERSION2",
            lc( $args{-support_version2} ) );
    }
    if ( exists( $args{-support_version3} ) ) {
        $Log->comment("Click to enable support  version 3 ");
        $self->set_checkbox( "$NFS_EDIT_SUPPORT_VERSION3",
            lc( $args{-support_version3} ) );
    }
    if ( exists( $args{-support_version4_0} ) ) {
        $Log->comment("Click to enable support  version 4.0 ");
        $self->set_checkbox(
            "$NFS_EDIT_SUPPORT_VERSION4_0",
            lc( $args{-support_version4_0} )
        );

        if (   exists( $args{-nfs_acl} )
            || exists( $args{-nfs_read_delagation} )
            || exists( $args{-nfs_write_delegation} ) )
        {
            $Log->comment("Click to enable nfs acl");
            $self->set_checkbox( "$NFS_EDIT_ACL", lc( $args{-nfs_acl} ) );

            $Log->comment("Click to enable nfs read delegation");
            $self->set_checkbox(
                "$NFS_EDIT_READ_DELEGATE",
                lc( $args{-nfs_read_delagation} )
            );

            $Log->comment("Click to enable nfs write delegation ");
            $self->set_checkbox(
                "$NFS_EDIT_WRITE_DELEGATE",
                lc( $args{-nfs_write_delegation} )
            );

            $Log->comment("Click to enable support  version 4.1 ");
            $self->set_checkbox( "$NFS_EDIT_VERSION4_1",
                lc( $args{-support_version4_1} ) );

            $self->wait_for_element_present( $NFS_EDIT_WINDOWS_USER_TEXT,
                $GENERIC_WAIT );
            $self->type( $NFS_EDIT_WINDOWS_USER_TEXT,
                $args{'-windows_user'} );
        } ## end if ( exists( $args{-nfs_acl...}))
    } else {
        $Log->comment(
            "mandatory options are missing, please provide required parameter values"
        );
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "mandatory options are missing, please provide required parameter values"
        );
    } ## end else [ if ( exists( $args{-support_version4_0...}))]

    $Log->comment("Selecting nfs edit save and close button");
    $commit_time = $self->click_timecount($NFS_EDIT_SAVE_AND_CLOSE);

    $self->wait_for_element_not_present( $NFS_EDIT_SAVE_AND_CLOSE,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("NFS page has been edited\n");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub nfs_edit

################################################################################
## @name ::get_nfs_info
## @summary
##   Fetch nfs Details using GUI.
## @description
##     This method can be used to fetch nfs details using GUI.
## @synopsis
##     get_nfs_info(%args);
## INPUTS TO METHOD CALL 'get_nfs_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the details about NFS.

sub get_nfs_info {

    my ( $self, %args ) = @_;
    my %values;
    my $nfs_info;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
            
    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>NFS");
    my $loading_time = $self->cluster_tree_navigate($vserverName, $NFS_TREE);

    $values{"$NFS_SERVER_STATUS_KEY"}
    = $self->get_text("$NFS_SERVER_STATUS_LABEL")
    if (
    $self->is_element_present("$NFS_SERVER_STATUS_LABEL") );

    $values{"$NFS_CONFIG_VERSION2_KEY"}
    = $self->get_text("$NFS_CONFIG_VERSION2_LABEL")
    if (
    $self->is_element_present("$NFS_CONFIG_VERSION2_LABEL") );

    $values{"$NFS_CONFIG_VERSION3_KEY"}
    = $self->get_text("$NFS_CONFIG_VERSION3_LABEL")
    if (
    $self->is_element_present("$NFS_CONFIG_VERSION3_LABEL") );

    $values{"$NFS_CONFIG_VERSION4_KEY"}
    = $self->get_text("$NFS_CONFIG_VERSION4_LABEL")
    if (
    $self->is_element_present("$NFS_CONFIG_VERSION4_LABEL") );

    $values{"$NFS_CONFIG_VERSION4_1_KEY"}
    = $self->get_text("$NFS_CONFIG_VERSION4_1_LABEL")
    if (
    $self->is_element_present("$NFS_CONFIG_VERSION4_1_LABEL") );

    $Log->comment("Collection of NFS info is successfull");
    $nfs_info->{loading_time} = $loading_time;
    $nfs_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $nfs_info;
} ## end sub get_nfs_info

################################################################################
## @name ::create_nfs_datastore
## @summary
##   Create NFS Datastore for VMWare using GUI.
## @description
##     This method can be used to create NFS Datastore for VMWare using GUI.
## @synopsis
##     create_nfs_datastore(%args);
## INPUTS TO METHOD CALL 'create_nfs_datastore' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -volname </td> <td> Name of the volume </td> </tr>
## <tr> <td> -aggrname </td> <td> Name of the aggregate </td> </tr>
## <tr> <td> -junctionpath </td> <td> Name of the junctionpath </td> </tr>
## <tr> <td> -volsize </td> <td> Size of the volume </td> </tr>
## <tr> <td> -volunit </td> <td>  Unit of the volume </td> </tr>
## <tr> <td> -snapreserve </td> <td> Entering Snapshot Reserve Percentage </td> </tr>
## <tr> <td> -dedupe </td> <td> Seleting dedupe </td> </tr>
## <tr> <td> -addesxserver </td> <td> Entering on the ESX Server Address </td> </tr>
## <tr> <td> -delesxserver </td> <td> Deleting the ESX Server Address </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub create_nfs_datastore {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Storage");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $STORAGE_TREE );

    $Log->comment("Clicking on the Provision Storage for VMWare");
    $self->click("$NFS_DATASTORE_VMWARE_CREATE_BUTTON");
    $self->wait_for_element_present(
        $NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on the Create Wizard Next Button");
    $self->click($NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_element_present(
        $NFS_DATASTORE_VMWARE_CREATE_VOLUME_NAME_TEXTBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-volname'} ) ) {
        $Log->comment("Entering Volume Name");
        $self->type_keys( $NFS_DATASTORE_VMWARE_CREATE_VOLUME_NAME_TEXTBOX,
            $args{'-volname'} );
        $self->type( $NFS_DATASTORE_VMWARE_CREATE_VOLUME_NAME_TEXTBOX,
            $args{'-volname'} );
        $self->fire_event( $NFS_DATASTORE_VMWARE_CREATE_VOLUME_NAME_TEXTBOX,
            "blur" );
    } ## end if ( exists( $args{'-volname'...}))
    if ( exists( $args{'-aggrname'} ) ) {
        $Log->comment("Entering Aggregate Name");
        $self->type_keys( $NFS_DATASTORE_VMWARE_CREATE_AGGREGATE_NAME_TEXTBOX,
            $args{'-aggrname'} );
        $self->type( $NFS_DATASTORE_VMWARE_CREATE_AGGREGATE_NAME_TEXTBOX,
            $args{'-aggrname'} );
        $self->fire_event(
            $NFS_DATASTORE_VMWARE_CREATE_AGGREGATE_NAME_TEXTBOX, "blur" );
    } ## end if ( exists( $args{'-aggrname'...}))
    if ( exists( $args{'-junctionpath'} ) ) {
        $Log->comment("Entering Junction Path");
        $self->type( $NFS_DATASTORE_VMWARE_CREATE_JUNCTION_PATH_TEXTBOX,
            $args{'-junctionpath'} );
    }
    if ( exists( $args{'-volsize'} ) ) {
        $Log->comment("Entering Volume Size");
        $self->type( $NFS_DATASTORE_VMWARE_CREATE_VOLUME_SIZE_TEXTBOX,
            $args{'-volsize'} );
    }
    if ( exists( $args{'-volunit'} ) ) {
	$Log->comment("clicking on input box");
	$self->click($NFS_DATASTORE_VMWARE_CREATE_VOLUME_UNIT_COMBOBOX);
	$self->wait_for_loading();
        eval {
            $Log->comment("Selecting Volume Unit: $args{'-volunit'}");
            $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item') and text()='$args{-volunit}']"
            );
        };
        if ($@) {
            NATE::BaseException->throw(
                "Value: \"$args{-volunit}\" not present in combobox list");
        }
        #$self->set_combobox_value(
        #    $NFS_DATASTORE_VMWARE_CREATE_VOLUME_UNIT_COMBOBOX,
        #    $args{'-volunit'} );
    } ## end if ( exists( $args{'-volunit'...}))
    if ( exists( $args{'-snapreserve'} ) ) {
        $Log->comment("Entering Snapshot Reserve Percentage");
        $self->set_combobox_value(
            $NFS_DATASTORE_VMWARE_CREATE_SNAPSHOT_RESERVE_INPUT,
            $args{'-snapreserve'} );
    } ## end if ( exists( $args{'-snapreserve'...}))

    $Log->comment("Clicking on the Create Wizard Next button");
    $self->click($NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_element_present(
        $NFS_DATASTORE_VMWARE_CREATE_ENABLE_DEDUPE_CHECKBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-dedupe'} ) ) {
        $Log->comment("$args{'-dedupe'}ing the deduplication");
        $self->set_checkbox(
            $NFS_DATASTORE_VMWARE_CREATE_ENABLE_DEDUPE_CHECKBOX,
            $args{'-dedupe'} );
    } ## end if ( exists( $args{'-dedupe'...}))

    $Log->comment("Clicking on the Create Wizard Next button");
    $self->click($NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_element_present(
        $NFS_DATASTORE_VMWARE_CREATE_ESX_SERVER_ADDRESS_INPUT,
        $GENERIC_WAIT );

    if ( exists( $args{'-addesxserver'} ) ) {
        foreach my $esxserver ( @{ $args{-addesxserver} } ) {
            $Log->comment("Entering on the ESX Server Address");
            $self->type(
                $NFS_DATASTORE_VMWARE_CREATE_ESX_SERVER_ADDRESS_INPUT,
                $esxserver );

            $Log->comment("Clicking on the Add Button");
            $self->click(
                $NFS_DATASTORE_VMWARE_CREATE_ESX_SERVER_ADDRESS_ADD_BUTTON);
        } ## end foreach my $esxserver ( @{ ...})
    } ## end if ( exists( $args{'-addesxserver'...}))
    if ( exists( $args{'-delesxserver'} ) ) {
        foreach my $esxserver ( @{ $args{-delesxserver} } ) {
            $Log->comment("Selecting the ESX Server Address");
            $self->click_row_with_texts(
                "$NFS_DATASTORE_VMWARE_CREATE_ESX_SERVER_ADDRESS_GRID",
                $esxserver );

            $Log->comment("Clicking on the Delete Button");
            $self->click(
                $NFS_DATASTORE_VMWARE_CREATE_ESX_SERVER_ADDRESS_DELETE_BUTTON
            );
        } ## end foreach my $esxserver ( @{ ...})
    } ## end if ( exists( $args{'-delesxserver'...}))

    $Log->comment(
        "Clicking on the Create Wizard Next button on ESX Servers NFS Access Page"
    );
    $self->click($NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_element_present(
        $NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment(
        "Clicking on the Create Wizard Next button on Summary Page");
    $self->click($NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Clicking on Finish Button");
    my $commit_time = $self->click_timecount(
        $NFS_DATASTORE_VMWARE_CREATE_WIZARD_NEXT_BUTTON);
    $self->wait_for_loading();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub create_nfs_datastore

################################################################################
## @name ::cifs_reset_domain
## @summary
##     Reset the CIFS domain using GUI.
## @description
##     This method can be used to reset the CIFS domain using GUI.
## @synopsis
##     cifs_reset_domain(%args);
## INPUTS TO METHOD CALL 'cifs_reset_domain' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server </td> </tr>
## </table>
##
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub cifs_reset_domain {
    my ( $self, %args ) = @_;

    my ( $pack, $file, $line, $subname, $hasargs, $wantarray, $commit_time) = caller(0);
    $Log->enter() if $may_enter;
    $self->clear_window();
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};

    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");

    my $loading_time = $self->cluster_tree_navigate($vserverName, $CIFS_TREE);
    $self->wait_for_loading();

    $Log->comment("Selecting CIFS domain tab");
    $self->click("$CIFS_DOMAIN_TAB");
    $self->wait_for_loading();

    $Log->comment("Clicking on Reset button");
    $self->click_button("$CIFS_DOMAIN_RESET_BUTTON");

    $Log->comment("waiting for ok button");
   $self->wait_for_element_present( "$CIFS_DOMAIN_RESET_COMPLETE_BUTTON",
                                      $GENERIC_WAIT );

    $self->click("$CIFS_DOMAIN_RESET_COMPLETE_BUTTON");
    $self->wait_for_loading();
    sleep 5;
    $Log->comment("Loading time and commit time are $loading_time sec ! and $commit_time sec !");
    my %times = ( loading_time =>$loading_time,
                  #commit_time  =>$commit_time
            );

    $Log->comment("****$subname()-Called at line#$line from $file START****");
    return \%times;
} ## end sub cifs_reset_domain

################################################################################
# Subroutine Name: get_cifs_configuration_info
# Objective:       It will give the information related to CIFS configuration
# Details:         See POD documentation at the end of this file
################################################################################
sub get_cifs_configuration_info {

    my ( $self, %args ) = @_;
    my %values;
    my $cifs_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
        
    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time = $self->cluster_tree_navigate($vserverName,$CIFS_TREE);

    $self->wait_for_element_present( $CIFS_CONFIGURATION_TAB,
            $GENERIC_WAIT );

    $Log->comment("Clicking on the Configuration TAB");
    $self->click("$CIFS_CONFIGURATION_TAB");
    $self->wait_for_element_present($CIFS_SETUP_BTN,
            $GENERIC_WAIT );

    $values{systemName}
    = $self->get_text($CIFS_SERVER_SYSTEM_NAME_LABEL)
    if (
        $self->is_element_present($CIFS_SERVER_SYSTEM_NAME_LABEL) );

    $values{authenticationStyle}
    = $self->get_text($CIFS_SERVER_AUTHENTICATION_STYLE_LABEL)
    if (
        $self->is_element_present($CIFS_SERVER_AUTHENTICATION_STYLE_LABEL) );

    $values{addomainname}
    = $self->get_text($CIFS_SERVER_ADDOMAIN_NAME_LABEL)
    if (
        $self->is_element_present($CIFS_SERVER_ADDOMAIN_NAME_LABEL) );

    $self->check_for_error();

    $Log->comment("Collection of CIFS info is successfull");
    $cifs_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $cifs_info;
} ## end sub get_cifs_configuration_info


################################################################################
# Subroutine Name: get_cifs_domaincontrollers_info
# Objective:       It will give the information related to CIFS Domain Controllers
# Details:         See POD documentation at the end of this file
################################################################################
sub get_cifs_domaincontrollers_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-fqdn" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ( %values, %values1 );
    my $cifs_info = {};
    my $fqdn = $args{'-fqdn'};
    my $commit_time;
     
    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time = $self->cluster_tree_navigate($vserverName, $CIFS_TREE);

    $self->wait_for_element_present( $CIFS_DOMAIN_TAB,
            $GENERIC_WAIT );

    $Log->comment("Clicking on the Domain TAB");
    $self->click("$CIFS_DOMAIN_TAB");
    $self->wait_for_element_present($CIFS_DOMAIN_VSERVER_INTERFACE_RESET_BTN,
            $GENERIC_WAIT );

    $Log->step("Getting row data for domain controller $fqdn from discovered Servers grid");
    my $rowdata
        = $self->get_row_data_with_text( $CIFS_DOMAIN_VSERVER_SERVERS_GRID, $fqdn );

    my @rowarray = split( ' ', $rowdata );
    %values = (
        "servers_node"    => $rowarray[0],
        "servers_fqdn"    => $rowarray[1],
        "servers_type"    => $rowarray[2],
        "servers_name"    => $rowarray[3],
        "servers_level"   => $rowarray[4],
        "servers_address" => $rowarray[5],
    );

    $Log->step("Getting row data for domain controller $fqdn from Domain Controllers grid");
    $rowdata
        = $self->get_row_data_with_text( $CIFS_DOMAIN_VSERVER_DC_GRID, $fqdn );

    @rowarray = split( ' ', $rowdata );
    %values1 = (
        "dc_interface"   => $rowarray[0],
        "dc_fqdn"        => $rowarray[1],
        "dc_preferreddc" => $rowarray[2],
    );


    my %cifscoll = ( %values, %values1 );
    $Log->comment("Collection of CIFS info is successfull");
    $cifs_info->{parsed_output} = \%cifscoll;

    $Log->exit() if $may_exit;

    return $cifs_info;
} ## end sub get_cifs_domaincontrollers_info


################################################################################
# Subroutine Name: get_cifs_symlinks_info
# Objective:       It will give the information related to CIFS Symlinks
# Details:         See POD documentation at the end of this file
################################################################################
sub get_cifs_symlinks_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-unixpath" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ( %values, %values1);
    my $cifs_info = {};
    my $unixpath = $args{'-unixpath'};
    
    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to Vserver>Storage>Configuration>Protocols>CIFS");
    my $loading_time = $self->cluster_tree_navigate($vserverName,$CIFS_TREE);

    $self->wait_for_element_present( $CIFS_SYMLINKS_TAB,
            $GENERIC_WAIT );

    $Log->comment("Clicking on the Symlinks TAB");
    $self->click("$CIFS_SYMLINKS_TAB");
    $self->wait_for_element_present($CIFS_SYMLINKS_CREATE,
            $GENERIC_WAIT );

    $Log->step("Getting row data for unix path $unixpath from grid");
    my $rowdata
        = $self->get_row_data_with_text( $SYMLINK_GRID, $unixpath );

    my @rowarray = split( ' ', $rowdata );
    %values = (
        "symlink_unixpath"   => $rowarray[0],
        "symlink_cifsserver" => $rowarray[1],
        "symlink_cifsshare"  => $rowarray[2],
        "symlink_cifspath"   => $rowarray[3],
        "symlink_locality"   => $rowarray[4],
    );


    $Log->comment("Collection of CIFS info is successfull");
    $cifs_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    
    return $cifs_info;
} ## end sub get_cifs_symlinks_info

1;
