# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to perform Portset related functionalities
##
## @description
##      Using this library, user can perform CMode Portset related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmSecurity;
##
##      portset_create(%args);
##      portset_edit(%args);
##      portset_delete(%args);
##      portset_refresh(%args);
##      get_portset_info(%args);
##	portset_bottom_panel(%args);
##
## @status public
##
## @author subhasri.s@netapp.com
## @change
##  2011-06-07 : subhasri : Initial version
## @change
##  2012-03-20 : vikasp : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmPortset;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    portset_create
    portset_edit
    portset_delete
    portset_refresh
    get_portset_info
    portset_bottom_panel
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::portset_create
## @summary
##    Create portset using GUI.
## @description
##     This method can be used to create portset using GUI.
## @synopsis
##    portset_create(%args);
## INPUTS TO METHOD CALL 'portset_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -portsetName </td> <td> (Required) Name of the portset to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -protocol </td> <td> Selecting protocol </td> </tr>
## <tr> <td> -interface </td> <td> Selecting interface </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub portset_create {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-portsetName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);

    $Log->comment("Clicking portset create button");
    $self->click($PORTSET_CREATE_BUTTON);

    $Log->comment("Waiting for portset create Dialog");
    #$self->wait_for_element_present( $PORTSET_NAME_FIELD, $GENERIC_WAIT );
    $self->wait_for_loading();
    
    $Log->comment("Entering portset name: $args{'-portsetName'}");
    $self->type_keys( $PORTSET_NAME_FIELD, $args{'-portsetName'} );

    #Selecting protocol
    if ( exists( $args{-protocol} ) ) {
        if ( lc( $args{-protocol} ) eq 'iscsi' ) {
            $Log->comment("select protocol : $args{-protocol}");
            $self->click("$PORTSET_PROTOCOL_ISCSI");
        } elsif ( lc( $args{-protocol} ) eq 'fcp' ) {
            $Log->comment("select protocol : $args{-protocol}");
            $self->click("$PORTSET_PROTOCOL_FCP");
        } elsif ( lc( $args{-protocol} ) eq 'mixed' ) {
            $Log->comment("select protocol : $args{-protocol}");
            $self->click("$PORTSET_PROTOCOL_MIXED");
        }
    } ## end if ( exists( $args{-protocol...}))

    #Selecting interface
    if ( exists( $args{'-interface'} ) ) {
        $Log->comment("Selecting interface");
        foreach my $int ( @{ $args{-interface} } ) {
            $Log->step("Selecting one interface for portset :$int");
            my @port_values = split( /:/, $int );
            $self->set_checkbox_in_row_with_text(
                "$PORTSET_INTERFACE_CREATE_GRID",
                $port_values[1], $port_values[0] );
        } ## end foreach my $int ( @{ $args{...}})
    } ## end if ( exists( $args{'-interface'...}))

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($PORTSET_CREATE_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("portset created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub portset_create

################################################################################
## @name ::portset_edit
## @summary
##    Edit portset using GUI.
## @description
##     This method can be used to edit portset using GUI.
## @synopsis
##    portset_edit(%args);
## INPUTS TO METHOD CALL 'portset_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -portsetName </td> <td> (Required) Name of the portset to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -interface </td> <td> Selecting interface </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub portset_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-portsetName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);
    $self->wait_for_loading();

    $Log->step("Selecting portset: $args{'-portsetName'}");
    $self->click_row_with_texts( $PORTSET_GRID, $args{'-portsetName'} );

    $Log->comment("Clicking portset edit button");
    $self->click($PORTSET_EDIT_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Waiting for Portset edit Dialog");
    #$self->wait_for_element_present( $PORTSET_INTERFACE_EDIT_GRID,
        #$GENERIC_WAIT );

    #Selecting interface
    if ( exists( $args{'-interface'} ) ) {
        $Log->comment("Selecting interface");
        foreach my $int ( @{ $args{-interface} } ) {
            $Log->step("Selecting one interface for portset :$int");
            my @port_values = split( /:/, $int );
            $self->set_checkbox_in_row_with_text(
                "$PORTSET_INTERFACE_EDIT_GRID",
                $port_values[1], $port_values[0] );

        } ## end foreach my $int ( @{ $args{...}})
    } ## end if ( exists( $args{'-interface'...}))

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($PORTSET_EDIT_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Portset edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub portset_edit

################################################################################
## @name ::portset_delete
## @summary
##    Delete portset using GUI.
## @description
##     This method can be used to delete  portset using GUI.
## @synopsis
##    portset_delete(%args);
## INPUTS TO METHOD CALL 'portset_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -portsetName </td> <td> (Required)Name of the portset to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub portset_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-portsetName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);
    $self->wait_for_loading();

    $Log->step("Selecting portset: $args{'-portsetName'}");
    $self->click_row_with_texts( $PORTSET_GRID, $args{'-portsetName'} );

    $Log->comment("Clicking portset delete button");
    $self->click($PORTSET_DELETE_BUTTON);
    $self->wait_for_loading();
     
    $Log->comment("Waiting for portset delete Dialog");
    $self->wait_for_element_present( $PORTSET_DELETE_OK_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($PORTSET_DELETE_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Portset deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub portset_delete

################################################################################
## @name ::portset_refresh
## @summary
##    Refresh portset using GUI.
## @description
##     This method can be used to refresh portset using GUI.
## @synopsis
##    portset_refresh(%args);
## INPUTS TO METHOD CALL 'portset_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub portset_refresh {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);

    $self->wait_for_element_present($PORTSET_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $PORTSET_REFRESH_BUTTON, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub portset_refresh

################################################################################
# Subroutine Name: get_portset_info
# Objective:       It will return portset information
# Details:         See POD documentation at the end of this file
################################################################################
################################################################################
## @name ::get_portset_info
## @summary
##    Fetch portset details using GUI.
## @description
##     This method can be used to fetch portset details using GUI.
## @synopsis
##    get_portset_info(%args);
## INPUTS TO METHOD CALL 'get_portset_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -portsetName </td> <td> Name of the Portset </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the details about portset.

sub get_portset_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-portsetName" => { type => SCALAR }, },
            allow_extra => 1
            );

    my $portsetName = $args{'-portsetName'};
    my %temp_portset_info;
    my $portset_info = {};
    
    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    
    $Log->step("Navigating to $LUN_TREE");
    my $loading_time = $self->cluster_tree_navigate( $vserverName,
            $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);

    $Log->step("Selecting portset: $portsetName from grid");
    $self->click_row_with_texts( $PORTSET_GRID, $portsetName );

    $Log->step("Getting row data for portset $portsetName from grid");
    my $rowdata
        = $self->get_row_data_with_text( $PORTSET_GRID, $portsetName );

    my @rowarray = split( ' ', $rowdata );
    %temp_portset_info = (
        "name"                  => $rowarray[0],
        "type"                  => $rowarray[1],
        "interface_count"       => $rowarray[2],
        "initiator_group_count" => $rowarray[3],
    );

    $Log->comment("Collection of portset info is successfull");
    $portset_info->{parsed_output} = \%temp_portset_info;

    $Log->exit() if $may_exit;
    
    return $portset_info;
} ## end sub get_portset_info

################################################################################
## @name ::portset_bottom_panel
## @summary
##    portset bottom panel details using GUI.
## @description
##     portset bottom panel details using GUI.
## @synopsis
##    portset_bottom_panel(%args);
## INPUTS TO METHOD CALL 'portset_bottom_panel' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -portsetName </td> <td> (Required) Name of the portset to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -interface </td> <td> Selecting interface </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub portset_bottom_panel {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-portsetName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $LUN_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LUN_TREE );

    $Log->step("Clicking on portset tab");
    $self->click($LUN_PORTSET_TAB);
    $self->wait_for_loading();

    $Log->step("Selecting portset: $args{'-portsetName'}");
    $self->click_row_with_texts( $PORTSET_GRID, $args{'-portsetName'} );

    if ( $args{'-tabName'} =~ /igroup/ ) {
	$Log->step("Clicking on bottom panel initiator group  tab");
    	$self->click("$PORTSET_BOTTOM_IGROUP_TAB");
	$self->wait_for_loading();
    }
    if ( $args{'-tabName'} =~ /interface/ ) {
        $Log->step("Clicking on bottom panel network interfaces tab");
        $self->click("$PORTSET_BOTTOM_INTERFACE_TAB");
        $self->wait_for_loading();
    }
} ## end portset_bottom_panel

1;
