# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##       Routines to parse Cmode Protection related GUI Functionalities
##
## @description
##       Using this library, user can perform Cmode protection related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmProtection;
##
##		snapmirror_create(%args);
##		snapmirror_edit(%args);
##		snapmirror_delete(%args);
##		snapmirror_refresh(%args);
##		snapmirror_operation(%args);
##		get_snapmirror_info(%args);
##
## @status private
##
## @author
##		Afsar syed . afsars@netapp.com
##
## @change
##	4/29/2010 : Afsar : Initial version
## @change
##  3/20/2012 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmProtection;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    snapmirror_create
    snapmirror_edit
    snapmirror_delete
    snapmirror_refresh
    snapmirror_operation
    get_snapmirror_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::snapmirror_create
## @summary
##     Create mirror relationship using GUI.
## @description
##     This method can be used to create mirror relationship using GUI.
## @synopsis
##     snapmirror_create(%args);
## INPUTS TO METHOD CALL 'snapmirror_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -mirrortype </td> <td>(Required) Selecting mirror type to be created </td> </tr>
## <tr> <td> -srcfiler </td> <td> (Required)Selecting source filer </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -srcusername </td> <td> Username for the source </td> </tr>
## <tr> <td> -srcpassword </td> <td> Password for the source </td> </tr>
## <tr> <td> -sourcevsr </td> <td> Source Vserver</td> </tr>
## <tr> <td> -aggrname </td> <td> Name of the aggregate </td> </tr>
## <tr> <td> -dstvserver </td> <td> Source Vserver</td> </tr>
## <tr> <td> -createvolname </td> <td> Enabling DNS cache </td> </tr>
## <tr> <td> -sourcevol </td> <td> Volume of the source </td> </tr>
## <tr> <td> -destvol </td> <td> Volume of the destination </td> </tr>
## <tr> <td> -volsize </td> <td> Volume size  </td> </tr>
## <tr> <td> -volsizeunit </td> <td> Volume sizeunit </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -schedulename </td> <td> Selecting name of schedule </td> </tr>
## <tr> <td> -initialize </td> <td> Selecting initialize mirror </td> </tr>
## <tr> <td> -thinprov </td> <td> Selecting thin provisioning </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time and Finish Time.

sub snapmirror_create {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-mirrortype' => { type => SCALAR },
            "-srcfiler"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );

    $self->wait_for_loading();
    $self->wait_for_element_present("$MIRRORSOURCE_CREATE_BUTTON");

    $Log->comment("Selecting create button");
    $self->click_button("$MIRRORSOURCE_CREATE_BUTTON");
    $self->wait_for_loading();
    
    $Log->comment("clicking Next button1");
    #$self->click_nowait("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
    $self->click("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
    sleep 5;

    if ($self->is_element_present("$MIRRORSOURCE_CREATE_CLUSTER_PEER_BUTTON")) {
        $Log->comment("Click on continue");
        $self->click("$MIRRORSOURCE_CREATE_CLUSTER_PEER_BUTTON");
    }        

    # Selecting mirror type based on input parameter
    $Log->comment("Selecting mirror type: $args{'-mirrortype'}");
    $self->wait_for_loading();
    
    if ( lc( $args{-mirrortype} eq "dest" ) ) {
        $self->click($MIRROR_DEST_RADIO_BUTTON);
        $Log->comment("clicking Next button2");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();
        
        $Log->comment("Selecting source filer as : $args{-srcfiler}");
        $self->set_combobox_value( $MIRROR_DEST_SRC_COMBOBOX,
            $args{-srcfiler} );

        if ( exists( $args{-srcusername} ) ) {
            $Log->comment("Entering username: $args{-srcusername}");
            $self->type( "$MIRROR_DEST_SRC_USERNAME_TEXTBOX",
                $args{-srcusername} );
        }
        if ( exists( $args{-srcpassword} ) ) {
            $Log->comment("Entering password :$args{-srcpassword}");
            $self->type( "$MIRROR_DEST_SRC_PASSWORD_TEXTBOX",
                $args{-srcpassword} );
        }
        $Log->comment("clicking Next button3");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();

        $Log->comment("Entering source vserver : $args{-sourcevsr}");
        $self->type_keys( "$MIRROR_DEST_SRC_VSERVER_TEXTBOX", "" );
        $self->type( "$MIRROR_DEST_SRC_VSERVER_TEXTBOX", $args{-sourcevsr} );

        $Log->comment("Entering source volme : $args{-sourcevol}");
        $self->type_keys( "$MIRROR_SRC_VOL_TEXTBOX", "" );
        $self->type( "$MIRROR_SRC_VOL_TEXTBOX", $args{-sourcevol} );

        $Log->comment("clicking Next button4");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();

    } elsif ( lc( $args{'-mirrortype'} eq "source" ) ) {
        $Log->comment("Click on src radio button");
        $self->click($MIRROR_SRC_RADIO_BUTTON);
        
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();
        
        $Log->comment("Entering source volume : $args{-sourcevol}");
        $self->type( "$MIRROR_SOURCE_VOLUME_TEXT", $args{-sourcevol} );
        $Log->comment("clicking Next button");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();

        $Log->comment("Selecting source filer as : $args{-srcfiler}");
        $self->set_combobox_value( $MIRROR_DEST_SRC_COMBOBOX,
            $args{-srcfiler} );
        $self->wait_for_loading();

        if ( exists( $args{-srcusername} ) ) {
            $Log->comment("Entering username: $args{-srcusername}");
            $self->type( "$MIRROR_DEST_SRC_USERNAME_TEXTBOX",
                $args{-srcusername} );
        }

        if ( exists( $args{-srcpassword} ) ) {
            $Log->comment("Entering password :$args{-srcpassword}");
            $self->type( "$MIRROR_DEST_SRC_PASSWORD_TEXTBOX",
                $args{-srcpassword} );
        }
        $Log->comment("clicking Next button");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Invalid value for mirror type, valid values are 'dest' or 'source'"
        );
    } ## end else [ if ( lc( $args{-mirrortype...}))]

    if ( exists( $args{-dstvserver} ) ) {
        $Log->comment(
            "Entering destination vserver name : $args{-dstvserver}");
        $self->type( "$MIRROR_SOURCE_DST_VSERVER_TEXT", $args{-dstvserver} );
    }

    if ( exists( $args{-optionvol} ) ) {
        if ( lc( $args{-optionvol} ) eq 'select' ) {
            $Log->comment("Select existing volume : $args{-optionvol}");
            $self->click("$MIRROR_DEST_SELECT_VOL_RADIO");

            if ( exists( $args{-createvolname} ) ) {
                $Log->comment(
                    "Entering existing volume name : $args{-createvolname}");
                $self->type( "$MIRROR_DEST_SRC_VOL_TEXTBOX",
                    $args{-createvolname} );
            } ## end if ( exists( $args{-createvolname...}))
        } elsif ( lc( $args{-optionvol} ) eq 'create' ) {
            $Log->comment("Creating new volume : $args{-createvolname}");
            $self->click("$MIRROR_DEST_CREATE_VOL_RADIO");

            if ( exists( $args{-aggrname} ) ) {
                $Log->comment("Entering aggregate name : $args{-aggrname}");
                $self->type( "$MIRROR_DEST_CREATE_VOL_AGGR",
                    $args{-aggrname} );
            }

            if ( exists( $args{-createvolname} ) ) {
                $Log->comment(
                    "Entering new volume name : $args{-createvolname}");
                $self->type( "$MIRROR_DEST_CREATE_VOL_NAME",
                    $args{-createvolname} );
            } ## end if ( exists( $args{-createvolname...}))

            if ( exists( $args{-volsize} ) ) {
                $Log->comment("Entering vol size : $args{-volsize}");
                $self->type( "$MIRROR_DEST_CREATE_VOL_SIZE",
                    $args{-volsize} );
            }

            if ( exists( $args{-volsizeunit} ) ) {
                $Log->comment(
                    "selecting vol size unit : $args{-volsizeunit}");
                $self->set_combobox_value(
                    "$MIRROR_DEST_CREATE_VOL_SIZE_UNIT",
                    $args{-volsizeunit} );
            } ## end if ( exists( $args{-volsizeunit...}))

            if ( exists( $args{-thinprov} ) ) {
                $Log->comment("selecting vol thin prov : $args{-thinprov}");
                $self->click("$MIRROR_DEST_CREATE_VOL_THIN");
            }

        } ## end elsif ( lc( $args{-optionvol...}))

        $Log->comment("clicking Next button");
        $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");
        $self->wait_for_loading();
    } ## end if ( exists( $args{-optionvol...}))

    if ( exists( $args{-schedule} ) ) {
        if ( exists( $args{-schedule} ) eq "demand" ) {
            $Log->comment("selecting schedule : $args{-schedule}");
            $self->click("$MIRROR_CREATE_SCHEDULE_DEMAND");
        } elsif ( exists( $args{-schedule} ) eq "existing" ) {
            $Log->comment("selecting schedule : $args{-schedule}");
            $self->click("$MIRROR_CREATE_SCHEDULE_EXISTING");

            $Log->comment("clicking browse button");
            $self->click("$MIRROR_CREATE_SCHEDULE_EXISTING_BROWSE");

            $Log->comment("selecting schedule name: $args{-schedulename}");
            $self->click_row_with_texts(
                "$MIRROR_CREATE_SCHEDULE_EXISTING_BROWSE_GRID",
                $args{-schedulename} );
        } ## end elsif ( exists( $args{-schedule...}))
        elsif ( exists( $args{-schedule} ) eq "create" ) {
            $Log->comment("selecting schedule : $args{-schedule}");
            $self->click("$MIRROR_CREATE_SCHEDULE_CREATE");

            $Log->comment("clicking create button");
            $self->click("$MIRROR_CREATE_SCHEDULE_CREATE_BUTTON");
            ##### to be written dialog is not loading
        } ## end elsif ( exists( $args{-schedule...}))

    } ## end if ( exists( $args{-schedule...}))

    if ( exists( $args{-initialize} ) ) {
        $self->wait_for_loading();
        $Log->comment("Clicking : initialize mirror relation");
        $self->click("$MIRROR_CREATE_SCHEDULE_INITIALIZE");
    }

    $Log->comment("Clicking Next button");
    $self->click_button("$MIRRORSOURCE_CREATE_NEXT_BUTTON");

    $Log->comment("Clicking Next button in summary page");
    my $commit_time
        = $self->click_timecount( $MIRRORSOURCE_CREATE_NEXT_BUTTON,
        "next", $MIRRORSOURCE_CREATE_NEXT_BUTTON );

    $Log->comment("Enter Finish button");
    my $finish_time
        = $self->click_timecount($MIRRORSOURCE_CREATE_NEXT_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time,
        finish_time  => $finish_time
    );

    $Log->comment("SnapMirror relationship created successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub snapmirror_create

################################################################################
## @name ::snapmirror_edit
## @summary
##     Edit snapmirror settings using GUI.
## @description
##     This method can be used to edit snapmirror settings using GUI.
## @synopsis
##     snapmirror_edit(%args);
## INPUTS TO METHOD CALL 'snapmirror_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sourcename </td> <td> (Required) Volume of the source </td> </tr>
## <tr> <td> -dstname </td> <td> (Required) Volume of the destination </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -mirrortype </td> <td>(Required) Selecting mirror type to be created </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -schedulename </td> <td> Selecting name of schedule </td> </tr>
## <tr> <td> -initialize </td> <td> Selecting initialize mirror </td> </tr>
## <tr> <td> -thinprov </td> <td> Selecting thin provisioning </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snapmirror_edit {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-sourcename' => { type => SCALAR },
            "-dstname"    => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );

    $self->wait_for_loading();
    $self->wait_for_element_present("$MIRRORSOURCE_GRID");

    $Log->comment("Selecting mirror relation");
    $self->click_row_with_texts( "$MIRRORSOURCE_GRID",
        ( $args{'-sourcename'}, $args{'-dstname'} ) );

    $Log->comment("Selecting edit button");
    $self->click_button("$MIRRORSOURCE_EDIT_BUTTON");
    $self->wait_for_element_present("$MIRROR_EDIT_SCHEDULE_DEMAND");

    if ( exists( $args{-schedule} ) ) {
        if ( lc( $args{-schedule} ) eq 'demand' ) {
            $Log->comment("Selecting schedule : $args{-schedule} ");
            $self->click("$MIRROR_EDIT_SCHEDULE_DEMAND");
        }
        if ( lc( $args{-schedule} ) eq 'edit' ) {
            $Log->comment("Clicking on Edit Radio Button");
            $self->click("$MIRROR_EDIT_SCHEDULE_EDIT");
            $Log->comment("Clicking on Browse button");
            $self->click("$MIRROR_EDIT_PROPERTIES_BROWSE_SCHEDULE_BUTTON");
            $self->wait_for_element_present(
                "$MIRROR_EDIT_PROPERTIES_SCHEDULE_BROWSE_GRID");

            $Log->comment("Selecting schedule : $args{-schedulename} ");
            $self->click_row_with_texts(
                "$MIRROR_EDIT_PROPERTIES_SCHEDULE_BROWSE_GRID",
                $args{-schedulename} );
            $self->wait_for_element_present(
                "$MIRROR_EDIT_SCHEDULE_BROWSE_SELECT_BUTTON");

            $Log->comment("Clicking on Select button");
            $self->click("$MIRROR_EDIT_SCHEDULE_BROWSE_SELECT_BUTTON");
            $self->wait_for_element_present(
                "$MIRROR_EDIT_SCHEDULE_EDIT_TEXTBOX");
        } ## end if ( lc( $args{-schedule...}))
    } ## end if ( exists( $args{-schedule...}))

    $Log->comment("Selecting edit ok button");
    $self->wait_for_loading();
    my $commit_time = $self->click_timecount($MIRROR_EDIT_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Successfully edited snapmirror relationship");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapmirror_edit

################################################################################
## @name ::snapmirror_operation
## @summary
##     Execute different snapmirror operations using GUI.
## @description
##     This method can be used to execute different snapmirror operations using GUI.
## @synopsis
##     snapmirror_operation(%args);
## INPUTS TO METHOD CALL 'snapmirror_operation' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sourcename </td> <td> (Required) Volume of the source </td> </tr>
## <tr> <td> -dstname </td> <td> (Required) Volume of the destination </td> </tr>
## <tr> <td> -operation </td> <td>  (Required)Selecting operation  </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snapmirror_operation {

    my $commit_time;
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-sourcename' => { type => SCALAR },
            "-dstname"    => { type => SCALAR },
            "-operation"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );
    $self->wait_for_loading();
    $self->wait_for_element_present("$MIRRORSOURCE_GRID");

    $Log->comment("Selecting mirror relation");
    $self->click_row_with_texts( "$MIRRORSOURCE_GRID",
        ( $args{'-sourcename'}, $args{'-dstname'} ) );
    $self->wait_for_loading();
    
    $Log->comment("Selecting operation button to change settings");
    $self->click_button("$MIRROR_OPERATION_BUTTON");

    if ( lc( $args{-operation} ) eq 'quiesce' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_QUEISE_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_QUEISE_BUTTON");
            $self->wait_for_element_present(
                "$MIRROR_OPERATION_QUEISE_CONFORM_CHECKBOX");

            $Log->comment("Selecting confirm checkbox button");
            $self->click("$MIRROR_OPERATION_QUEISE_CONFORM_CHECKBOX");

            $Log->comment("Selecting quiese ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_QUEISE_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_QUEISE_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'resume' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_RESUME_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_RESUME_BUTTON");
            $Log->comment("Selecting resume ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_RESUME_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_RESUME_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'break' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_BREAK_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_BREAK_BUTTON");

            $Log->comment("Selecting confirm checkbox button");
            $self->click("$MIRROR_OPERATION_BREAK_CONFORM_CHECKBOX");

            $Log->comment("Selecting break ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_BREAK_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_BREAK_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'reverseresync' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_REVERSE_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_REVERSE_BUTTON");

            $Log->comment("Selecting confirm checkbox button");
            $self->click("$MIRROR_OPERATION_REVERSE_CONFORM_CHECKBOX");

            $Log->comment("Selecting reverse sync ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_REVERSE_OK_BUTTON);
        
            $self->click($MIRROR_OPERATION_REVERSE_OK_BUTTON);
            $commit_time = "1.0";
            sleep(10);
            if ($self->is_element_present($MIRROR_OPERARION_REVERSE_INFO_OK_BUTTON)) {
                $Log->comment("VIKAS: Clicking on the Information OK Button");
                $self->click($MIRROR_OPERARION_REVERSE_INFO_OK_BUTTON);
            }

        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'update' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_UPDATE_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_UPDATE_BUTTON");

            $Log->comment("Selecting update ok button");
#            $commit_time
#                = $self->click_timecount($MIRROR_OPERATION_UPDATE_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_UPDATE_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'resync' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_RESYNC_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_RESYNC_BUTTON");

            $Log->comment("Selecting confirm checkbox button");
            $self->click("$MIRROR_OPERATION_RESYNC_CONFORM_CHECKBOX");

            $Log->comment("Selecting resync ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_RESYNC_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_RESYNC_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'initialize' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_INI_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_INI_BUTTON");

            $Log->comment("Selecting initialize ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_INI_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_INI_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    if ( lc( $args{-operation} ) eq 'abort' ) {
        $Log->comment("Selecting the operation button : $args{-operation} ");
        if ( $self->is_button_enabled("$MIRROR_OPERATION_ABORT_BUTTON") ) {
            $self->click_button("$MIRROR_OPERATION_ABORT_BUTTON");

            $Log->comment("Selecting confirm checkbox button");
            $self->click("$MIRROR_OPERATION_ABORT_CONFORM_CHECKBOX");

            $Log->comment("Selecting abort ok button");
            #$commit_time
            #    = $self->click_timecount($MIRROR_OPERATION_ABORT_OK_BUTTON);
            $self->click_timecount($MIRROR_OPERATION_ABORT_OK_BUTTON);
        } ## end if ( $self->is_button_enabled...)
    } ## end if ( lc( $args{-operation...}))

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        #commit_time  => $commit_time
    );

    $Log->comment("Operation for RC is successfull");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub snapmirror_operation

################################################################################
## @name ::snapmirror_delete
## @summary
##     Delete snapmirror relatioship using GUI.
## @description
##     This method can be used to delete snapmirror relatioship using GUI.
## @synopsis
##     snapmirror_delete(%args);
## INPUTS TO METHOD CALL 'snapmirror_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sourcename </td> <td> (Required) Volume of the source </td> </tr>
## <tr> <td> -dstname </td> <td> (Required) Volume of the destination </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snapmirror_delete {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-sourcename' => { type => SCALAR },
            "-dstname"    => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );
    $self->wait_for_loading();
    $self->wait_for_element_present("$MIRRORSOURCE_GRID");

    $Log->comment("Selecting mirror relation");
    $self->click_row_with_texts( "$MIRRORSOURCE_GRID",
        ( $args{'-sourcename'}, $args{'-dstname'} ) );
    sleep 5;

    $Log->comment("Selecting delete button");
    $self->click_button("$MIRROR_DELETE_BUTTON");
    $self->wait_for_element_present("$MIRROR_DELETE_CONFORM_CHECKBOX");

    $Log->comment("Selecting delete confirm button");
    $self->click($MIRROR_DELETE_CONFORM_CHECKBOX);

    $Log->comment("Selecting delete ok button");
    $self->wait_for_loading();
    my $commit_time = $self->click_timecount($MIRROR_DELETE_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Successfully deleted snapmirror relationship");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub snapmirror_delete

################################################################################
## @name ::snapmirror_refresh
## @summary
##     Refresh snapmirror screen using GUI.
## @description
##     This method can be used to refresh snapmirror screen using GUI.
## @synopsis
##     snapmirror_refresh(%args);
## INPUTS TO METHOD CALL 'snapmirror_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snapmirror_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );

    $self->wait_for_loading();
    $self->wait_for_element_present("$MIRRORSOURCE_GRID");

    # Click refresh button
    my $commit_time
        = $self->click_timecount( $MIRROR_REFRESH_BUTTON, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapmirror_refresh

################################################################################
## @name ::get_snapmirror_info
## @summary
##     Get information related to snapmirror using GUI.
## @description
##     This method can be used to get information related to snapmirror using GUI.
## @synopsis
##     get_snapmirror_info(%args);
## INPUTS TO METHOD CALL 'get_snapmirror_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sourcename </td> <td> (Required) Volume of the source </td> </tr>
## <tr> <td> -dstname </td> <td> (Required) Volume of the destination </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the SnapMirror Status Details.

sub get_snapmirror_info {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-sourcename' => { type => SCALAR },
            "-destname"    => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $DATAPROTECTION_TREE");
    $self->cluster_tree_navigate( $vserverName, $DATAPROTECTION_TREE );
    $self->wait_for_element_present("$MIRRORSOURCE_REFRESH_BUTTON");

    $Log->comment("Selecting source container");
    $self->click_row_with_texts( "$MIRRORSOURCE_GRID",
        ( $args{'-sourcename'}, $args{'-destname'} ) );

    $Log->step("Getting row data for snapmirror from grid");
    my $rowdata = $self->get_row_data_with_text( $MIRRORSOURCE_GRID,
        $args{'-sourcename'}, $args{'-destname'} );

    my @rowarray = split( ' ', $rowdata );
    my %values = (
        #"source"      => $rowarray[0],
        #"destination" => $rowarray[1],
        "health"      => $rowarray[2],
        "state"       => $rowarray[3],
        "status"      => $rowarray[4],
    );

    $values{"schedule"} = $self->get_text("$SNAPMIRROR_INFO_SCHEDULE_LABEL")
        if ( $self->is_element_present("$SNAPMIRROR_INFO_SCHEDULE_LABEL") );

    $values{"data_transfer_rate"}
        = $self->get_text("$SNAPMIRROR_INFO_DATATRANSFERRATE_LABEL")
        if (
        $self->is_element_present("$SNAPMIRROR_INFO_DATATRANSFERRATE_LABEL")
        );

    $values{"last_transfer_timestamp"}
        = $self->get_text("$SNAPMIRROR_INFO_LASTTRANSFERTIMESTAMP_LABEL")
        if (
        $self->is_element_present(
            "$SNAPMIRROR_INFO_LASTTRANSFERTIMESTAMP_LABEL")
        );

    $values{"exported_snapshot_copy"}
        = $self->get_text("$SNAPMIRROR_INFO_EXPORTEDSNAPSHOTCOPY_LABEL")
        if (
        $self->is_element_present(
            "$SNAPMIRROR_INFO_EXPORTEDSNAPSHOTCOPY_LABEL")
        );

    $values{"latest_snapshot_copy"}
        = $self->get_text("$SNAPMIRROR_INFO_LATESTSNAPSHOTCOPY_LABEL")
        if (
        $self->is_element_present(
            "$SNAPMIRROR_INFO_LATESTSNAPSHOTCOPY_LABEL")
        );

    $values{"progress"} = $self->get_text("$SNAPMIRROR_INFO_PROGRESS_LABEL")
        if ( $self->is_element_present("$SNAPMIRROR_INFO_PROGRESS_LABEL") );

    $values{"destination_status"}
        = $self->get_text("$SNAPMIRROR_INFO_DESTINATIONSTATUS_LABEL")
        if (
        $self->is_element_present("$SNAPMIRROR_INFO_DESTINATIONSTATUS_LABEL")
        );
    $values{"source_status"}
        = $self->get_text("$SNAPMIRROR_INFO_SOURCESTATUS_LABEL")
        if (
        $self->is_element_present("$SNAPMIRROR_INFO_SOURCESTATUS_LABEL") );

    $Log->comment("Collection of snapmirror info is done");

    my $snapmirror_info = {};
    $snapmirror_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $snapmirror_info;

} ## end sub get_snapmirror_info

1;
