# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Qtree GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode qtree related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::7Mode::SmQtree
##
##      create_qtree(%args);
##      edit_qtree(%args);
##      delete_qtree(%args);
##      refresh_qtree(%args);
##      get_qtree_info(%args);
##
## @status public
##
## @author afsars@netapp.com
## @change
##  2010-03-25 : afsars : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmQtree;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
  create_qtree
  edit_qtree
  delete_qtree
  refresh_qtree
  get_qtree_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::create_qtree
## @summary
##     Creation of Qtree using GUI.
## @description
##     This method can be used to create Qtree based on paramteres provided using GUI.
## @synopsis
##     create_qtree (%args);
## INPUTS TO METHOD CALL 'create_qtree ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required)Name of the Volume to be created </td> </tr>
## <tr> <td> -qtreename </td> <td> (Required)Qtreename Name to be provided </td> </tr>
## <tr> <td> -vservername </td> <td>  Name of the virtual server to be provided </td> </tr>
## <tr> <td> -oplocks </td> <td> Selecting oplocks based on input parameter </td> </tr>
## <tr> <td> -secstyle </td> <td> Selecting security style based on input parameter </td> </tr>
## <tr> <td> -qtreequota </td> <td> Entering qtree quota limit </td> </tr>
## <tr> <td> -qtreespaceusage </td> <td> Entering Qtree Quota Space Usage size	 </td> </tr>
## <tr> <td> -qtreespaceusagemode </td> <td> Entering Qtree Quota Space Usage mode	 </td> </tr>
## <tr> <td> -userquota </td> <td> ntering qtree quota limit </td> </tr>
## <tr> <td> -userspaceusage </td> <td> Entering User Quota Space Usage size </td> </tr>
## <tr> <td> -userspaceusagemode </td> <td> Entering User Quota Space Usage mode </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub create_qtree {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-volname'   => { type => SCALAR },
            "-qtreename" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $QTREE_TREE");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QTREE_TREE );

    $Log->comment("Clicking Qtree Create button");
    $self->click($QTREE_CREATE_BUTTON);

    $Log->comment("Waiting for Qtree Create Dialog");
    $self->wait_for_element_present( $QTREE_VOLNAME_TEXTBOX, $GENERIC_WAIT );

    $Log->comment("Entering Qtree name: $args{'-qtreename'}");
    $self->type( $QTREE_NAME_TEXTBOX, $args{'-qtreename'} );

    $Log->comment("Selecting volume : $args{'-volname'}");
    $self->type( $QTREE_VOLNAME_TEXTBOX, $args{'-volname'} );

    # Selecting oplocks based on input parameter
    if ( exists( $args{'-oplocks'} ) ) {
        $Log->comment( $args{'-oplocks'} . "ing Oplocks" );
        $self->set_checkbox( $QTREE_OPLOCKS_CHECKBOX, $args{'-oplocks'} );
    }

    # Selecting security style based on input parameter
    if ( exists( $args{'-secstyle'} ) ) {
        $Log->comment("Selecting security style: $args{'-secstyle'}");
        if ( lc( $args{'-secstyle'} eq "inherit" ) ) {
            $self->click($QTREE_SECURITY_INHERIT);
        }
        elsif ( lc( $args{'-secstyle'} eq "unix" ) ) {
            $self->click($QTREE_SECURITY_SELECT);
            $self->set_option($QTREE_SECURITY_UNIX_RADIO);
        }
        elsif ( lc( $args{'-secstyle'} eq "ntfs" ) ) {
            $self->click($QTREE_SECURITY_SELECT);
            $self->set_option($QTREE_SECURITY_NTFS_RADIO);
        }
        elsif ( lc( $args{'-secstyle'} eq "mixed" ) ) {
            $self->click($QTREE_SECURITY_SELECT);
            $self->set_option($QTREE_SECURITY_MIXED_RADIO);
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
"Invalid value for security style, valid values are 'unix' or 'ntfs' or 'mixed'"
            );
        } ## end else [ if ( lc( $args{'-secstyle'...}))]
    } ## end if ( exists( $args{'-secstyle'...}))

    $Log->comment("Selecting Quota tab");
    $self->click($QTREE_QUOTA_TAB);
    $self->wait_for_loading();

    $Log->comment("Waiting for Quota panel");
    $self->wait_for_element_present( $QTREE_QTREE_QUOTA_SPACE_TEXTBOX,
        $GENERIC_WAIT );

    if (   exists( $args{'-qtreequota'} )
        && exists( $args{'-qtreespaceusage'} )
        && exists( $args{'-qtreespaceusagemode'} ) )
    {
        $Log->comment( $args{'-qtreequota'} . "ing qtree quota limit" );
        $self->set_checkbox( $QTREE_QTREE_QUOTA_LIMIT_CHECKBOX,
            $args{'-qtreequota'} );
        $self->wait_for_loading();

        $Log->comment(
            "Entering Qtree Quota Space Usage size: $args{'-qtreespaceusage'}"
        );
        $self->type( $QTREE_QTREE_QUOTA_SPACE_TEXTBOX,
            $args{'-qtreespaceusage'} );

        $Log->comment("Setting Unit: $args{'-qtreespaceusagemode'}");
        $self->select_from_dropdown( "$QTREE_QTREE_QUOTA_SPACE_COMBOBOX",
            uc( $args{-qtreespaceusagemode} ), 1 );
    } ## end if ( exists( $args{'-qtreequota'...}))
    if (   exists( $args{'-userquota'} )
        && exists( $args{'-userspaceusage'} )
        && exists( $args{'-userspaceusagemode'} ) )
    {
        $Log->comment( $args{'-userquota'} . "ing qtree quota limit" );
        $self->set_checkbox( $QTREE_USER_QUOTA_LIMIT_CHECKBOX,
            $args{'-userquota'} );
        $self->wait_for_loading();

        $Log->comment(
            "Entering User Quota Space Usage size: $args{'-userspaceusage'}");
        $self->type( $QTREE_USER_QUOTA_SPACE_TEXTBOX,
            $args{'-userspaceusage'} );

        $Log->comment("Setting Unit: $args{'-userspaceusagemode'}");
        $self->select_from_dropdown( "$QTREE_USER_QUOTA_SPACE_COMBOBOX",
            uc( $args{-userspaceusagemode} ), 1 );
    } ## end if ( exists( $args{'-userquota'...}))

    $Log->comment("Selecting OK button to complete.");
    $self->click($QTREE_CREAT_OK_BUTTON);

    if ( $self->is_element_present($QUOTA_EDIT_CONFIRM_CANCEL) ) {
        $Log->comment("Clicking confirmation dialog checkbox");
        $self->set_checkbox( $QUOTA_EDIT_CONFIRM_CHECKBOX, "enable" );
        $Log->comment("Clicking confirmation dialog OK button");
        $self->click($QUOTA_EDIT_CONFIRM_OK);
    }

    $Log->comment("Qtree created successfully");
    $Log->exit() if $may_exit;

} ## end sub create_qtree

################################################################################
## @name ::edit_qtree
## @summary
##     Editing of Qtree using GUI.
## @description
##     This method can be used to edit Qtree based on paramteres provided using GUI.
## @synopsis
##     edit_qtree (%args);
## INPUTS TO METHOD CALL 'edit_qtree ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -qtreename </td> <td> (Required)Qtreename Name to be provided </td> </tr>
## <tr> <td> -vservername </td> <td>  Name of the virtual server to be provided </td> </tr>
## <tr> <td> -oplocks </td> <td> Selecting oplocks based on input parameter </td> </tr>
## <tr> <td> -secstyle </td> <td> Selecting security style based on input parameter </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub edit_qtree {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-qtreename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $QTREE_TREE");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QTREE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting qtree: $args{'-qtreename'}");
    $self->click_row_with_texts( $QTREE_GRID, $args{'-qtreename'} );
    $self->wait_for_loading();

    $Log->comment("Selecting Qtree Edit button");
    $self->click($QTREE_EDIT_BUTTON);

    $Log->comment("Waiting for Qtree Dialog");
    $self->wait_for_element_present( $QTREE_EDIT_OPLOCKS_CHECKBOX,
        $GENERIC_WAIT );

    # Selecting oplocks based on input parameter
    if ( exists( $args{'-oplocks'} ) ) {
        $self->set_checkbox( $QTREE_EDIT_OPLOCKS_CHECKBOX, $args{'-oplocks'} );
    }

    # Selecting security style based on input parameter
    if ( exists( $args{'-secstyle'} ) ) {
        $Log->comment("Selecting security style : $args{'-secstyle'}");
        if ( lc( $args{'-secstyle'} eq "unix" ) ) {
            $self->set_option($QTREE_EDIT_SECURITY_UNIX_RADIO);
        }
        elsif ( lc( $args{'-secstyle'} eq "ntfs" ) ) {
            $self->set_option($QTREE_EDIT_SECURITY_NTFS_RADIO);
        }
        elsif ( lc( $args{'-secstyle'} eq "mixed" ) ) {
            $self->set_option($QTREE_EDIT_SECURITY_MIXED_RADIO);
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
"Invalid value for security style, valid values are 'unix' or 'ntfs' or 'mixed'"
            );
        } ## end else [ if ( lc( $args{'-secstyle'...}))]
    } ## end if ( exists( $args{'-secstyle'...}))

    $Log->comment("Clicking Edit button to complete");
    my $commit_time = $self->click_timecount($QTREE_EDIT_OK_BUTTON);

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Qtree edit completed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub edit_qtree

################################################################################
## @name ::delete_qtree
## @summary
##     Deleting Qtree using GUI.
## @description
##     This method can be used to delete Qtree based on paramteres provided using GUI.
## @synopsis
##     delete_qtree (%args);
## INPUTS TO METHOD CALL 'delete_qtree ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -qtreename </td> <td> (Required)Qtreename Name to be provided </td> </tr>
## <tr> <td> -vservername </td> <td>  Name of the virtual server to be provided </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub delete_qtree {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-qtreename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $QTREE_TREE");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QTREE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting qtree $args{'-qtreename'}");
    $self->click_row_with_texts( $QTREE_GRID, $args{'-qtreename'} );
    $self->wait_for_loading();

    $Log->comment("Selecting Qtree Delete button");
    $self->click($QTREE_DELETE_BUTTON);
    $Log->comment("Waiting for Qtree Delete confirmation dialog");
    $self->wait_for_element_present( $QTREE_DELETE_CONFIRM_CHECKBOX,
        $GENERIC_WAIT );
    $Log->comment("Setting confirmation checkbox");
    $self->set_checkbox( $QTREE_DELETE_CONFIRM_CHECKBOX, "enable" );

    $Log->step("Click Delete button");
    my $commit_time = $self->click_timecount($QTREE_DELETE_OK_BUTTON);

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->step("Delete completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub delete_qtree

################################################################################
## @name ::refresh_qtree
## @summary
##     Refresh Qtree using GUI.
## @description
##     This method can be used to refresh Qtree based on paramteres provided using GUI.
## @synopsis
##     refresh_qtree(%args);
## INPUTS TO METHOD CALL 'refresh_qtree ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>  Name of the virtual server to be provided </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub refresh_qtree {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $QTREE_TREE");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QTREE_TREE );
    $self->wait_for_element_present($QTREE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time =
      $self->click_timecount( $QTREE_REFRESH_BUTTON, "refresh" );

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub refresh_qtree

################################################################################
## @name ::get_qtree_info
## @summary
##     Return information regarding qtree Qtree using GUI.
## @description
##     This method can be used to return information regarding qtree using GUI.
## @synopsis
##     get_qtree_info(%args);
## INPUTS TO METHOD CALL 'get_qtree_info ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr
## <tr> <td> -volname </td> <td> (Required)Name of the Volume to be created </td> </tr>
## <tr> <td> -qtreename </td> <td> (Required)Qtreename Name to be provided </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Qtree Status details.

sub get_qtree_info {

    my %temp_qtree_info;
    my $qtree_info = {};

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-volname'   => { type => SCALAR },
            "-qtreename" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $QTREE_TREE");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QTREE_TREE );
    $self->wait_for_loading();

    $Log->step("Selecting qtree: $args{'-qtreename'} from grid");
    $self->click_row_with_texts( $QTREE_GRID, $args{'-qtreename'},
        $args{'-volname'} );
    $self->wait_for_loading();

    $Log->step("Getting row data for qtree $args{'-qtreename'} from grid");
    my $rowdata =
      $self->get_row_data_with_text( $QTREE_GRID, $args{'-qtreename'},
        $args{'-volname'} );

    my @rowarray = split( ' ', $rowdata );
    %temp_qtree_info = (
        $QTREE_INFO_QTREE_NAME_KEY  => $rowarray[0],
        $QTREE_INFO_VOLUME_NAME_KEY => $rowarray[1],
        $QTREE_INFO_SECSTYLE_KEY    => $rowarray[2],
        $QTREE_INFO_STATUS_KEY      => $rowarray[3],
        $QTREE_INFO_OPLOCKS_KEY     => $rowarray[4],
    );

    $Log->comment("Collection of qtree info is successfull");
    $qtree_info->{parsed_output} = \%temp_qtree_info;

    $Log->exit() if $may_exit;

    return $qtree_info;

} ## end sub get_qtree_info

1;

