# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse cmode Volume Quotas GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode volume quotas related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmQuotas.pm
##
##      quota_create(%args);
##      quota_edit(%args);
##      quota_delete(%args);
##      quota_refresh(%args);
##      quota_manage(%args);
##      quota_manage_refresh(%args);
##	quota_report_edit(%args);
##      quota_report_refresh(%args);
##	quota_report_bottom_panel(%args);
##      get_quota_info(%args);
##      get_quota_detail_info(%args);
##      get_quota_manage_info(%args);
##
## @author
##		Afsar Syed. afsars@netapp.com
##
## @change
##	4/26/2010 : Afsar Syed : Initial version
## @change
##  3/20/2012 : Vikas P : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmQuotas;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    quota_create
    quota_edit
    quota_delete
    quota_refresh
    quota_manage
    quota_manage_refresh
    quota_report_edit
    quota_report_refresh
    quota_report_bottom_panel
    get_quota_info
    get_quota_detail_info
    get_quota_manage_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::quota_create
## @summary
##     Creation of Quota using GUI.
## @description
##     This method can be used to create quota using GUI.
## @synopsis
##     quota_create(%args);
## INPUTS TO METHOD CALL 'quota_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -container </td> <td> (Required) Name of the container </td> </tr>
## <tr> <td> -qtype </td> <td> (Required)Selecting qtree type </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -quotaname</td> <td> Type Quota name </td> </tr>
## <tr> <td> -quser </td> <td> select specific group </td> </tr>
## <tr> <td> -qgroup</td> <td> Percentage of Snapshot Reserve </td> </tr>
## <tr> <td> -spacehardlt </td> <td> Enter Space hard limits </td> </tr>
## <tr> <td> -spacehardltut </td> <td> Enter Space hard limit unit </td> </tr>
## <tr> <td> -spacesoftlt</td> <td> Enter Space soft limits </td> </tr>
## <tr> <td> -spacesoftltut </td> <td> Enter Space soft limit unit </td> </tr>
## <tr> <td> -spacethlt </td> <td> Enter Space threshold limits </td> </tr>
## <tr> <td> -spacethltut </td> <td> Enter Space threshold limit unit </td> </tr>
## <tr> <td> -filehardlt </td> <td> Enter file hard limits </td> </tr>
## <tr> <td> -filehardltut </td> <td> Enter file hard limit unit </td> </tr>
## <tr> <td> -filesoftlt  </td> <td> Enter file soft limits </td> </tr>
## <tr> <td> -filesoftltut </td> <td> Enter file soft limit unit </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time and Finish Time.

sub quota_create {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-container' => { type => SCALAR },
            "-qtype"     => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->click("$QUOTA_TAB");
    $self->wait_for_element_present("$QUOTA_GRID");

    $Log->step("Click Quota create Button");
    $self->click_button("$QUOTA_CREATE_BUTTON");
    $self->wait_for_element_present("$QUOTA_CREATE_NEXT_BUTTON");

    $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");

    $Log->step("Selecting qtree type");
    if ( lc( $args{-qtype} ) eq "qtree" ) {
        $self->set_option( "$QUOTA_CREATE_QTREE_RADIO", "enable" );
    } elsif ( lc( $args{-qtype} ) eq "user" ) {
        $self->set_option( "$QUOTA_CREATE_USER_RADIO", "enable" );
    } elsif ( lc( $args{-qtype} ) eq "group" ) {
        $self->set_option( "$QUOTA_CREATE_GROUP_RADIO", "enable" );
    }

    $Log->step("Type container Name");
    $self->type( "$QUOTA_CREATE_VOL_TEXTBOX", $args{'-container'} );

    $Log->step("Click next Button");
    $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");

    if ( lc( $args{-qtype} ) eq "user" ) {
        $self->wait_for_element_present("$QUOTA_CREATE_ALLUSER_RADIO");
        if ( exists( $args{-quser} ) ) {
            if ( lc( $args{-quser} ) eq "all" ) {
                $Log->step("select all group");
                $self->set_option( "$QUOTA_CREATE_ALLUSER_RADIO", "enable" );

                $Log->step("Click next Button");
                $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
            } elsif ( lc( $args{-quser} ) eq "specific" ) {
                $Log->step("select specific group");
                $self->set_option( "$QUOTA_CREATE_SPECIFICUSER_RADIO",
                    "enable" );

                $Log->step("Click on Add button");
                $self->click("$QUOTA_CREATE_USER_ADD_BUTTON");

                $Log->step("select specific user :$args{-specific} ");
                $self->type( "$QUOTA_CREATE_USER_ADD_INPUT",
                    $args{-specific} );

                $Log->step("Click on save button");
                $self->click("$QUOTA_CREATE_USER_ADD_SAVE_INPUT");
                $self->wait_for_loading();
                $self->check_for_error();

                $Log->step("Click next Button");
                $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
            } ## end elsif ( lc( $args{-quser}...))
        } else {
            $Log->step("Click next Button in default scenario");
            $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
        }
    } elsif ( lc( $args{-qtype} ) eq "group" ) {
        $self->wait_for_element_present("$QUOTA_CREATE_ALLGROUP_RADIO");
        if ( exists( $args{-qgroup} ) ) {
            if ( lc( $args{-qgroup} ) eq "all" ) {
                $Log->step("select all group");
                $self->set_option( "$QUOTA_CREATE_ALLGROUP_RADIO", "enable" );

                $Log->step("Click next Button");
                $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
            } elsif ( lc( $args{-qgroup} ) eq "specific" ) {
                $Log->step("select specific group");
                $self->set_option( "$QUOTA_CREATE_SPECIFICGROUP_RADIO",
                    "enable" );

                $Log->step("Enter the specific group :$args{-specific}");
                $self->type( "$QUOTA_CREATE_GROUP_INPUT", $args{-specific} );

                $Log->step("Click next Button");
                $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
            } ## end elsif ( lc( $args{-qgroup...}))
        } else {
            $Log->step("Click next Button in default scenario");
            $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
        }
    } ## end elsif ( lc( $args{-qtype}...))
    
    $self->wait_for_element_present("$QUOTA_CREATE_SPACE_HARDLIMIT_INPUT");

    if ( exists $args{-spacehardlt} ) {
        $Log->step("Enter Space hard limits");
        $self->type( "$QUOTA_CREATE_SPACE_HARDLIMIT_INPUT",
            $args{-spacehardlt} );
    }
    if ( exists $args{-spacehardltut} ) {
        $Log->step("Enter Space hard limit unit");
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_HARDLIMIT_UNIT",
            uc( $args{-spacehardltut} ) );
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_HARDLIMIT_UNIT",
            uc( $args{-spacehardltut} ) );

    } ## end if ( exists $args{-spacehardltut...})
    if ( exists $args{-spacesoftlt} ) {
        $Log->step("Enter Space soft limits");
        $self->type( "$QUOTA_CREATE_SPACE_SOFTLIMIT_INPUT",
            $args{-spacesoftlt} );

    } ## end if ( exists $args{-spacesoftlt...})
    if ( exists $args{-spacesoftltut} ) {
        $Log->step("Enter Space soft limit unit");
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_SOFTLIMIT_UNIT",
            uc( $args{-spacesoftltut} ) );
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_SOFTLIMIT_UNIT",
            uc( $args{-spacesoftltut} ) );
    }
    if ( exists $args{-spacethlt} ) {
        $Log->step("Enter Space threshold limits");
        $self->type( "$QUOTA_CREATE_SPACE_THRESHLIMIT_INPUT",
            $args{-spacethlt} );

    } ## end if ( exists $args{-spacethlt...})
    if ( exists $args{-spacethltut} ) {
        $Log->step("Enter Space threshold limit unit");
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_THRESHLIMIT_UNIT",
            uc( $args{-spacethltut} ) );
        $self->set_combobox_value( "$QUOTA_CREATE_SPACE_THRESHLIMIT_UNIT",
            uc( $args{-spacethltut} ) );

    } ## end if ( exists $args{-spacethltut...})
    if ( exists $args{-filehardlt} ) {
        $Log->step("Enter file hard limits");
        $self->type( "$QUOTA_CREATE_FILE_HARDLIMIT_INPUT",
            $args{-filehardlt} );

    } ## end if ( exists $args{-filehardlt...})
    if ( exists $args{-filehardltut} ) {
        $Log->step("Enter file hard limit unit");
        $self->set_combobox_value( "$QUOTA_CREATE_FILE_HARDLIMIT_UNIT",
            $args{-filehardltut} );
        $self->set_combobox_value( "$QUOTA_CREATE_FILE_HARDLIMIT_UNIT",
            $args{-filehardltut} );

    } ## end if ( exists $args{-filehardltut...})
    if ( exists $args{-filesoftlt} ) {
        $Log->step("Enter file soft limits");
        $self->type( "$QUOTA_CREATE_FILE_SOFTLIMIT_INPUT",
            $args{-filesoftlt} );

    } ## end if ( exists $args{-filesoftlt...})
    if ( exists $args{-filesoftltut} ) {
        $Log->step("Enter file soft limit unit");
        $self->set_combobox_value( "$QUOTA_CREATE_FILE_SOFTLIMIT_UNIT",
            $args{-filesoftltut} );
        $self->set_combobox_value( "$QUOTA_CREATE_FILE_SOFTLIMIT_UNIT",
            $args{-filesoftltut} );

    } ## end if ( exists $args{-filesoftltut...})

    $self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
    # my $commit_time
        # = $self->click_timecount( $QUOTA_CREATE_NEXT_BUTTON, "next",
        # $QUOTA_CREATE_FINISH_BUTTON );
	$Log->step("Clicking next in summary page");
	$self->click_button("$QUOTA_CREATE_NEXT_BUTTON");
	
    $Log->step("Select finish create quotas");
    $self->check_for_error();
    #my $finish_time = $self->click_timecount($QUOTA_CREATE_FINISH_BUTTON);
	$self->click_button("$QUOTA_CREATE_NEXT_BUTTON");

    # $Log->comment(
        # "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    # );
    my %times = (
        loading_time => $loading_time,
        # commit_time  => $commit_time,
        # finish_time  => $finish_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub quota_create

################################################################################
## @name ::quota_edit
## @summary
##     Editing Quota using GUI.
## @description
##     This method can be used to edit quota using GUI.
## @synopsis
##     quota_edit(%args);
## INPUTS TO METHOD CALL 'quota_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -quotaname</td> <td> (Required) Type Quota name </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -spacehardlt </td> <td> Enter Space hard limits </td> </tr>
## <tr> <td> -spacehardltut </td> <td> Enter Space hard limit unit </td> </tr>
## <tr> <td> -spacesoftlt</td> <td> Enter Space soft limits </td> </tr>
## <tr> <td> -spacesoftltut </td> <td> Enter Space soft limit unit </td> </tr>
## <tr> <td> -spacethlt </td> <td> Enter Space threshold limits </td> </tr>
## <tr> <td> -spacethltut </td> <td> Enter Space threshold limit unit </td> </tr>
## <tr> <td> -filehardlt </td> <td> Enter file hard limits </td> </tr>
## <tr> <td> -filehardltut </td> <td> Enter file hard limit unit </td> </tr>
## <tr> <td> -filesoftlt  </td> <td> Enter file soft limits </td> </tr>
## <tr> <td> -filesoftltut </td> <td> Enter file soft limit unit </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub quota_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-quotaname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->click("$QUOTA_TAB");
    #$self->wait_for_element_present("$QUOTA_GRID");
    $self->wait_for_loading();

    $Log->step("Select quota $args{'-quotaname'}");
    $self->click_row_with_texts( "$QUOTA_GRID", $args{'-quotaname'} );

    $Log->step("Click edit quota Button");
    $self->click_button("$QUOTA_EDIT_BUTTON");

    $Log->step("Enter space tab in edit quota");
    $self->click("$QUOTA_EDIT_DISK_TAB");

    if ( exists $args{-spacehardlt} ) {
        $Log->step("Enter Space hard limits");
        $self->type( "$QUOTA_EDIT_DISK_HARDLIMIT_INPUT",
            $args{-spacehardlt} );
    }

    if ( exists $args{-spacehardltut} ) {
        $Log->step("Enter Space hard limit unit");
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_HARDLIMIT_UNIT",
            uc( $args{-spacehardltut} ) );
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_HARDLIMIT_UNIT",
            uc( $args{-spacehardltut} ) );
    }

    if ( exists $args{-spacesoftlt} ) {
        $Log->step("Enter Space soft limits");
        $self->type( "$QUOTA_EDIT_DISK_SOFTLIMIT_INPUT",
            $args{-spacesoftlt} );
    }

    if ( exists $args{-spacesoftltut} ) {
        $Log->step("Enter Space soft limit unit");
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_SOFTLIMIT_UNIT",
            uc( $args{-spacesoftltut} ) );
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_SOFTLIMIT_UNIT",
            uc( $args{-spacesoftltut} ) );
    }

    if ( exists $args{-spacethlt} ) {
        $Log->step("Enter Space threshold limits");
        $self->type( "$QUOTA_EDIT_DISK_THRESHOLD_INPUT", $args{-spacethlt} );
    }

    if ( exists $args{-spacethltut} ) {
        $Log->step("Enter Space threshold limit unit");
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_THRESHOLD_UNIT",
            uc( $args{-spacethltut} ) );
        $self->set_combobox_value( "$QUOTA_EDIT_DISK_THRESHOLD_UNIT",
            uc( $args{-spacethltut} ) );
    }

    $Log->step("Enter File tab in edit quota");
    $self->click("$QUOTA_EDIT_FILE_TAB");

    if ( exists $args{-filehardlt} ) {
        $Log->step("Enter file hard limits");
        $self->type( "$QUOTA_EDIT_FILE_HARDLIMIT_INPUT", $args{-filehardlt} );
    }

    if ( exists $args{-filehardltut} ) {
        $Log->step("Enter file hard limit unit");
        $self->set_combobox_value( "$QUOTA_EDIT_FILE_HARDLIMIT_UNIT",
            $args{-filehardltut} );
        $self->set_combobox_value( "$QUOTA_EDIT_FILE_HARDLIMIT_UNIT",
            $args{-filehardltut} );
    }

    if ( exists $args{-filesoftlt} ) {
        $Log->step("Enter file soft limits");
        $self->type( "$QUOTA_EDIT_FILE_SOFTLIMIT_INPUT", $args{-filesoftlt} );
    }

    if ( exists $args{-filesoftltut} ) {
        $Log->step("Enter file soft limit unit");
        $self->set_combobox_value( "$QUOTA_EDIT_FILE_SOFTLIMIT_UNIT",
            $args{-filesoftltut} );
        $self->set_combobox_value( "$QUOTA_EDIT_FILE_SOFTLIMIT_UNIT",
            $args{-filesoftltut} );
    }

    $Log->step("Apply the changes to the quota Rule :");
    $self->click($QUOTA_EDIT_OK_BUTTON);

    if ( $self->is_element_present($QUOTA_EDIT_CONFIRM_CANCEL) )
    {
        $Log->comment("Clicking confirmation dialog checkbox");
        $self->set_checkbox( $QUOTA_EDIT_CONFIRM_CHECKBOX,
            "enable" );
        $Log->comment("Clicking confirmation dialog OK button");
        $self->click($QUOTA_EDIT_CONFIRM_OK);
    }

    $self->check_for_error();
    $self->wait_for_loading();

    $Log->exit() if $may_exit;
} ## end sub quota_edit

################################################################################
## @name ::quota_delete
## @summary
##     Deleting Quota using GUI.
## @description
##     This method can be used to delete quota using GUI.
## @synopsis
##     quota_delete (%args);
## INPUTS TO METHOD CALL 'quota_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -quotaname</td> <td> Type Quota name </td> </tr>
## <tr> <td> -vservername</td> <td> Name of the vserver </td> </tr>
## <tr> <td> -multipledel</td> <td> Selecting multipledel </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub quota_delete {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment(
            "Overwrighting the default Vserver for this function to :$args{'-vservername'}"
        );
        $vserverName = $args{'-vservername'};
    } ## end if ( exists( $args{'-vservername'...}))

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $Log->step("Selecting Quota Tab");
    $self->click("$QUOTA_TAB");
    $self->wait_for_element_present($QUOTA_GRID);

    if ( exists $args{-multipledel} ) {
        for my $count ( 0 .. $#{ $args{-multipledel} } ) {
            my $mquotaname = $args{-multipledel}[$count];
            $Log->step("Selecting Quota : $mquotaname");
            $self->click_row_with_texts( $QUOTA_GRID, $mquotaname );
            $self->control_key_down() if ( $count == 0 );
        } ## end for my $count ( 0 .. $#...)
        $self->control_key_up();
    } else {
        $Log->step("Select quota $args{'-quotaname'}");
        $self->click_row_with_texts( $QUOTA_GRID, $args{'-quotaname'} );
    }

    $Log->step("Click delete quota Button");
    $self->click_button($QUOTA_DELETE_BUTTON);

    $Log->comment("Waiting for Qtree Delete confirmation dialog");
    $self->wait_for_element_present( $QUOTA_DELETE_CONFORM_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Setting confirmation checkbox");
    $self->set_checkbox( $QUOTA_DELETE_CONFORM_CHECKBOX, "enable" );

    $Log->step("Click Delete button");
    my $commit_time = $self->click_timecount($QUOTA_DELETE_CONFORM_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    my $i = 0;
    while(not ($self->is_element_present("$QUOTA_DELETE_POPUP_BUTTON")) && $i < 5){
        sleep(1); #waiting for quota delete pop-up
        $i++;
    }

    if ($self->is_element_present("$QUOTA_DELETE_POPUP_BUTTON")) {
        $Log->comment("Found Delete Popup button");
        $self->click($QUOTA_DELETE_POPUP_BUTTON);
    }

    $Log->step("Delete completed");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub quota_delete

################################################################################
## @name ::quota_refresh
## @summary
##     Refresh Quota using GUI.
## @description
##     This method can be used to refresh quota using GUI.
## @synopsis
##     quota_refresh (%args);
## INPUTS TO METHOD CALL 'quota_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername</td> <td> Name of the vserver </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub quota_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );
    $self->click("$QUOTA_TAB");

#    $self->wait_for_element_present("$QUOTA_GRID");
    $self->wait_for_loading();

    $Log->step("Click Refresh Button");
    my $commit_time = $self->click_timecount( $QUOTA_REFRESH, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub quota_refresh

################################################################################
## @name ::quota_manage
## @summary
##     Managing the state of Quota using GUI.
## @description
##     This method can be used to  manage the state of quota  using GUI.
## @synopsis
##     quota_manage(%args);
## INPUTS TO METHOD CALL 'quota_manage' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -container </td> <td> (Required)Type container Name </td> </tr>
## <tr> <td> -vservername</td> <td> Name of the virtual server </td> </tr>
## <tr> <td> -state </td> <td> Selecting state of the quota </td> </tr>
## <tr> <td> -resize </td> <td> Selecting quota resize </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.
##
sub quota_manage {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-container" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};

    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );
    $self->wait_for_element_present("$QUOTA_GRID");

    $Log->step("Enter Manage tab in edit");
    $self->click($QUOTA_MANAGE_TAB);

    $Log->step("Select quota $args{'-container'}");
    $self->click_row_with_texts( "$QUOTA_MANAGE_GRID", 1, 0,
        $args{'-container'} );

    if ( exists( $args{-state} ) ) {
        if ( lc( $args{-state} ) eq 'on' ) {
            $Log->step("Click Manage Refresh Button");
            $self->click("$QUOTA_MANAGE_REFRESH_BUTTON");
            sleep 5;
            $self->click_row_with_texts( "$QUOTA_MANAGE_GRID", 1, 0,
                $args{'-container'} );
            $Log->step("Select quota On");
            $self->click( $QUOTA_MANAGE_ON_BUTTON );
        } elsif ( lc( $args{-state} ) eq 'off' ) {
            $Log->step("Click Manage Refresh Button");
            $self->click("$QUOTA_MANAGE_REFRESH_BUTTON");
            $self->click_row_with_texts( "$QUOTA_MANAGE_GRID", 1, 0,
                $args{'-container'} );
            $Log->step("Select quota OFF");
            sleep 5;
            $self->click_button("$QUOTA_MANAGE_OFF_BUTTON");
            $Log->step("Select confirm OFF button");
            $self->click($QUOTA_MANAGE_OFF_CONFIRM_BUTTON);
        } else {
            $Log->error("Invalid state for Quota");
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Invalid state for Quota");
            return $self->error;
        }
        if ($self->is_element_present($QUOTA_MANAGE_UTILITIES_SHOW_MSG_OK_BUTTON)){
            $self->click_button("$QUOTA_MANAGE_UTILITIES_SHOW_MSG_OK_BUTTON");
            sleep 10;
        }
    }

    if ( exists( $args{-resize} ) and $args{-resize}  =~ /yes|enable/i) {
        $Log->step("Select quota resize");
        $self->click_button("$QUOTA_MANAGE_RESIZE_BUTTON");
        $self->wait_for_loading();
        if ($self->is_element_present($QUOTA_MANAGE_UTILITIES_SHOW_MSG_OK_BUTTON)){
            $self->click_button("$QUOTA_MANAGE_UTILITIES_SHOW_MSG_OK_BUTTON");
        }
        $self->wait_for_loading();
        if($self->is_element_present($QUOTA_MANAGE_RESIZE_CONFIRM_BUTTON)) {
            $self->click_timecount($QUOTA_MANAGE_RESIZE_CONFIRM_BUTTON);
        }
        sleep 1;
    }

    $Log->step("Click Manage Refresh Button");
    $self->click($QUOTA_MANAGE_REFRESH_BUTTON);

    $Log->exit() if $may_exit;

    return {loading_time => $loading_time};
} ## end sub quota_manage

################################################################################
## @name ::quota_manage_refresh
## @summary
##     Managing Quota refresh using GUI.
## @description
##     This method can be used to manage quota refresh using GUI.
## @synopsis
##     quota_manage_refresh(%args);
## INPUTS TO METHOD CALL 'quota_manage_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername</td> <td> Name of the virtual server </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub quota_manage_refresh {

    my ( $self, %args ) = @_;
    my ( %times, $commit_time );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );
    $self->wait_for_element_present("$QUOTA_GRID");

    $Log->step("Enter Manage tab");
    $self->click($QUOTA_MANAGE_TAB);

    $Log->step("Click on Refresh Buttom");
    $commit_time
        = $self->click_timecount( $QUOTA_MANAGE_REFRESH_BUTTON, "refresh" );
    $self->wait_for_element_present("$QUOTA_MANAGE_REFRESH_BUTTON");

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub quota_manage_refresh

################################################################################
## @name ::get_quota_info
## @summary
##    Get information regarding Quota details using GUI.
## @description
##     This method can be used to get information regarding quota details using GUI.
## @synopsis
##    get_quota_info  (%args);
## INPUTS TO METHOD CALL ' get_quota_info  ' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -container </td> <td> (Required)Type container Name </td> </tr>
## <tr> <td> -qtype </td> <td> (Required)Selecting qtree type </td> </tr>
## <tr> <td> -vservername</td> <td> Name of the virtual server </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Quota Status Details.

sub get_quota_info {

    my ( $self, %args ) = @_;

    my %values;
    my $quota_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->wait_for_element_present("$QUOTA_GRID");

    $Log->step("Selecting container: $args{-container}");
    $Log->step("Selecting quota type: $args{-qtype}");
    $Log->step("Selecting Quota refresh button");
    $self->click_button("$QUOTA_REFRESH");
    $self->wait_for_loading();

    $self->click_row_with_texts( "$QUOTA_GRID", 1, 0,
        ( $args{-container}, $args{-qtype} ) );

    $values{"$QUOTA_INFO_VOLUME_KEY"}
        = $self->get_text("$QUOTA_INFO_VOLUME_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_VOLUME_LABEL") );
    $values{"$QUOTA_INFO_TYPE_KEY"}
        = $self->get_text("$QUOTA_INFO_TYPE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_TYPE_LABEL") );
    $values{"$QUOTA_INFO_QTREE_KEY"}
        = $self->get_text("$QUOTA_INFO_QTREE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_QTREE_LABEL") );
    $values{"$QUOTA_INFO_USERS_KEY"}
        = $self->get_text("$QUOTA_INFO_USERS_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_USERS_LABEL") );
    $values{"$QUOTA_INFO_STATUS_KEY"}
        = $self->get_text("$QUOTA_INFO_STATUS_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_STATUS_LABEL") );
    $values{"$QUOTA_INFO_SPACEUSAGE_KEY"}
        = $self->get_text("$QUOTA_INFO_USAGE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_USAGE_LABEL") );
    $values{"$QUOTA_INFO_SPACEHARD_KEY"}
        = $self->get_text("$QUOTA_INFO_HARDLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_HARDLIMIT_LABEL") );
    $values{"$QUOTA_INFO_SPACESOFT_KEY"}
        = $self->get_text("$QUOTA_INFO_SOFTLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_SOFTLIMIT_LABEL") );
    $values{"$QUOTA_INFO_FILEUSAGE_KEY"}
        = $self->get_text("$QUOTA_INFO_FILEUSAGE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILEUSAGE_LABEL") );
    $values{"$QUOTA_INFO_FILEHARD_KEY"}
        = $self->get_text("$QUOTA_INFO_FILEHARDLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILEHARDLIMIT_LABEL") );
    $values{"$QUOTA_INFO_FILESOFT_KEY"}
        = $self->get_text("$QUOTA_INFO_FILESOFTLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILESOFTLIMIT_LABEL") );

    $Log->comment("Collection of quota info is successfull");
    $quota_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $quota_info;

} ## end sub get_quota_info

################################################################################
## @name ::get_quota_detail_info
## @summary
##    Get detail information regarding Quota using GUI.
## @description
##     This method can be used to get detail information regarding quota details using GUI.
## @synopsis
##    get_quota_detail_info(%args);
##
## @return
##     Returns an array reference which contains the Quota details.

sub get_quota_detail_info {

    my ( $self, %args ) = @_;

    my %values;
    my $quota_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->wait_for_element_present("$QUOTA_GRID");
    $Log->step("Select quota $args{'-container'}");
    chomp( $args{'-container'} );
    $self->click_row_with_texts( "$QUOTA_GRID", 1, 0,
        ( $args{'-container'}, $args{'-type'} ) );

    $values{"$QUOTA_INFO_VOLUME_KEY"}
        = $self->get_text("$QUOTA_INFO_VOLUME_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_VOLUME_LABEL") );
    $values{"$QUOTA_INFO_TYPE_KEY"}
        = $self->get_text("$QUOTA_INFO_TYPE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_TYPE_LABEL") );
    $values{"$QUOTA_INFO_QTREE_KEY"}
        = $self->get_text("$QUOTA_INFO_QTREE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_QTREE_LABEL") );
    $values{"$QUOTA_INFO_USERS_KEY"}
        = $self->get_text("$QUOTA_INFO_USERS_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_USERS_LABEL") );
    $values{"$QUOTA_INFO_STATUS_KEY"}
        = $self->get_text("$QUOTA_INFO_STATUS_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_STATUS_LABEL") );
    $values{"$QUOTA_INFO_SPACEUSAGE_KEY"}
        = $self->get_text("$QUOTA_INFO_USAGE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_USAGE_LABEL") );
    $values{"$QUOTA_INFO_SPACEHARD_KEY"}
        = $self->get_text("$QUOTA_INFO_HARDLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_HARDLIMIT_LABEL") );
    $values{"$QUOTA_INFO_SPACESOFT_KEY"}
        = $self->get_text("$QUOTA_INFO_SOFTLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_SOFTLIMIT_LABEL") );
    $values{"$QUOTA_INFO_FILEUSAGE_KEY"}
        = $self->get_text("$QUOTA_INFO_FILEUSAGE_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILEUSAGE_LABEL") );
    $values{"$QUOTA_INFO_FILEHARD_KEY"}
        = $self->get_text("$QUOTA_INFO_FILEHARDLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILEHARDLIMIT_LABEL") );
    $values{"$QUOTA_INFO_FILESOFT_KEY"}
        = $self->get_text("$QUOTA_INFO_FILESOFTLIMIT_LABEL")
        if ( $self->is_element_present("$QUOTA_INFO_FILESOFTLIMIT_LABEL") );

    $Log->comment("Collection of quota info is successfull");
    $quota_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $quota_info;

} ## end sub get_quota_detail_info

################################################################################
## @name ::get_quota_manage_info
## @summary
##    Get manage information regarding Quota using GUI.
## @description
##     This method can be used to get manage information regarding quota details using GUI.
## @synopsis
## 		get_quota_manage_info(%args);
##
## @return
##     Returns an array reference which contains the Quota manage details.

sub get_quota_manage_info {

    my ( $self, %args ) = @_;

    my (%manageInfo);
    my $quota_manage_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Quotas");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );
    $self->wait_for_element_present("$QUOTA_GRID");

    $Log->step("Enter Manage tab in edit");
    $self->click($QUOTA_MANAGE_TAB);

    $Log->step("Select Volume: $args{'-volname'} in Quota Manage Page");
    $self->click_row_with_texts( "$QUOTA_MANAGE_GRID", $args{'-volname'} );

    $Log->step("Fetch error message");
    $manageInfo{$QUOTA_INFO_MANAGE_ERROR_KEY}
        = $self->get_text($QUOTA_INFO_MANAGE_ERROR);

    $Log->step("Fetch warning message");
    $manageInfo{$QUOTA_INFO_MANAGE_WARNING_KEY}
        = $self->get_text($QUOTA_INFO_MANAGE_WARNING);

    $Log->comment("Loading time is $loading_time sec ! ");

    $quota_manage_info->{parsed_output} = \%manageInfo;
    $Log->comment("Collection of quota manage info is successfull");

    $Log->exit() if $may_exit;

    return $quota_manage_info;

} ## end sub get_quota_manage_info

################################################################################
## @name ::quota_report_edit
## @summary
##    Edit Quota Report using GUI.
## @description
##     This method can be used to get edit quota report using GUI.
## @synopsis
##              quota_report_edit(%args);
##
## @return
##     

sub quota_report_edit {
    my ( $self, %args ) = @_;
    my $loading_time;

    $Log->enter() if $may_enter;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->comment("Navigating to Storage>Quotas");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->click("$QUOTA_REPORT_TAB");
    $self->wait_for_element_present("$QUOTA_REPORT_GRID");

    $Log->step("Select quota $args{'-quotaname'}");
    $self->click_row_with_texts( $QUOTA_REPORT_GRID,
        $args{'-quotaname'} );

    $Log->step("Clicking edit button");
    $self->click("$QUOTA_REPORT_EDIT_BUTTON");

    if ( $args{-operation} eq "create" ) {
        if ( $self->is_element_present($QUOTA_REPORT_CREATE_BUTTON)
          ) {
            $Log->step("Clicking create button");
            $self->click("$QUOTA_REPORT_CREATE_BUTTON");
        } ## end if ( $self->is_element_present...)
    } ## end if ( $args{-operation}...)

    $Log->step("Enter Disk tab in edit quota");
    $self->click("$QUOTA_EDIT_DISK_TAB");

    if ( exists $args{-spacehardlt} ) {
        $Log->step("Enter Space hard limits");
        $self->type( "$QUOTA_EDIT_DISK_HARDLIMIT_INPUT",
            $args{-spacehardlt} );
    }

    if ( exists $args{-spacehardltut} ) {
        $Log->step("Enter Space hard limit unit");
        $self->set_combobox_value(
            "$QUOTA_EDIT_DISK_HARDLIMIT_UNIT",
            uc( $args{-spacehardltut} ) );

    } ## end if ( exists $args{-spacehardltut...})

    if ( exists $args{-spacesoftlt} ) {
        $Log->step("Enter Space soft limits");
        $self->type( "$QUOTA_EDIT_DISK_SOFTLIMIT_INPUT",
            $args{-spacesoftlt} );

    } ## end if ( exists $args{-spacesoftlt...})

    if ( exists $args{-spacesoftltut} ) {
        $Log->step("Enter Space soft limit unit");
        $self->set_combobox_value(
            "$QUOTA_EDIT_DISK_SOFTLIMIT_UNIT",
            uc( $args{-spacesoftltut} ) );
    } ## end if ( exists $args{-spacesoftltut...})

    if ( exists $args{-spacethlt} ) {
        $Log->step("Enter Space threshold limits");
        $self->type( "$QUOTA_EDIT_DISK_THRESHOLD_INPUT",
            $args{-spacethlt} );
    }

    if ( exists $args{-spacethltut} ) {
        $Log->step("Enter Space threshold limit unit");
        $self->set_combobox_value(
            "$QUOTA_EDIT_DISK_THRESHOLD_UNIT",
            uc( $args{-spacethltut} ) );
    } ## end if ( exists $args{-spacethltut...})

    $Log->step("Enter File tab in edit quota");
    $self->click("$QUOTA_EDIT_FILE_TAB");

    if ( exists $args{-filehardlt} ) {
        $Log->step("Enter file hard limits");
        $self->type( "$QUOTA_EDIT_FILE_HARDLIMIT_INPUT",
            $args{-filehardlt} );
    }

    if ( exists $args{-filehardltut} ) {
        $Log->step("Enter file hard limit unit");
        $self->set_combobox_value(
            "$QUOTA_EDIT_FILE_HARDLIMIT_UNIT",
            $args{-filehardltut} );

    } ## end if ( exists $args{-filehardltut...})

    if ( exists $args{-filesoftlt} ) {
        $Log->step("Enter file soft limits");
        $self->type( "$QUOTA_EDIT_FILE_SOFTLIMIT_INPUT",
            $args{-filesoftlt} );
    }

    if ( exists $args{-filesoftltut} ) {
        $Log->step("Enter file soft limit unit");
        $self->set_combobox_value(
            "$QUOTA_EDIT_FILE_SOFTLIMIT_UNIT",
            $args{-filesoftltut} );
    } ## end if ( exists $args{-filesoftltut...})

    if ( $args{-operation} eq "create_cancel" ) {
        $Log->step("Cancel the create quota operation from report page :");
        $self->click("$QUOTA_REPORT_CREATE_CANCEL_OK_BUTTON");
    } elsif ( $args{-operation} eq "create" ) {
        $Log->step("Apply the changes to the quota Rule :");
        $self->click_nowait("$QUOTA_REPORT_CREATE_OK_BUTTON");
        sleep(10);
        if (
            $self->is_element_present(
                "$QUOTA_EDIT_CONFIRM_CHECKBOX")
          )
        {
            $Log->comment("Clicking confirmation dialog checkbox");
            $self->click_nowait($QUOTA_EDIT_CONFIRM_CHECKBOX);
            sleep(5);
            $Log->comment("Clicking confirmation dialog OK button");
            $self->click_nowait($QUOTA_EDIT_CONFIRM_OK);
        }
    } elsif ( $args{-operation} eq "edit" ) {
        $Log->comment("Apply the changes to the quota Rule :");
        $self->click_nowait($QUOTA_REPORT_EDIT_OK_BUTTON);
        sleep(10);

        if ( $self->is_element_present( "$QUOTA_EDIT_CONFIRM_CANCEL")) {
            $Log->comment("Clicking confirmation dialog checkbox");
            $self->click_nowait($QUOTA_EDIT_CONFIRM_CHECKBOX);
            sleep(5);
            $Log->comment("Clicking confirmation dialog OK button");
            $self->click_nowait($QUOTA_EDIT_CONFIRM_OK);
        }
    }

    $self->check_for_error();
    $self->wait_for_loading();

    $Log->exit() if $may_exit;

    return $TCD::PASS;
} ## end sub quota_report_edit

################################################################################
## @name ::quota_report_refresh
## @summary
##    Refresh Quota Report using GUI.
## @description
##     This method can be used to refresh quota report using GUI.
## @synopsis
##              quota_report_refresh(%args);
##
## @return
##

sub quota_report_refresh {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->comment("Navigating to Storage>Quotas");
    $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->click("$QUOTA_REPORT_TAB");
    $self->wait_for_element_present("$QUOTA_REPORT_GRID");

    $Log->step("Clicking on refresh button");
    $self->click("$QUOTA_REPORT_REFRESH_BUTTON");
    $self->wait_for_loading();

    $Log->exit() if $may_exit;
    return $TCD::PASS;
} ## end quota_report_refresh

################################################################################
## @name ::quota_report_bottom_panel
## @summary
##    Get Quota Report Bottom Panel Details using GUI.
## @description
##     This method can be used to get quota report bottom panel details using GUI.
## @synopsis
##              quota_report_bottom_panel(%args);
##
## @return
##

sub quota_report_bottom_panel {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->comment("Navigating to Storage>Quotas");
    $self->cluster_tree_navigate( $vserverName, $QUOTAS_TREE );

    $self->click("$QUOTA_REPORT_TAB");
    $self->wait_for_element_present("$QUOTA_REPORT_GRID");

    $Log->step("Select quota $args{'-quotaname'}");
    $self->click_row_with_texts( $QUOTA_REPORT_GRID,
        $args{'-quotaname'} );

    if ( $args{'-tabValue'} =~ /RULES/) {
	$Log->comment("Clicking on Rules Tab");
	$self->click("$QUOTA_REPORT_RULES_BOTTOM_PANEL");
    }    

    if ( $args{'-tabValue'} =~ /REPORT/) {
        $Log->comment("Clicking on Report Tab");
        $self->click("$QUOTA_REPORT_BOTTOM_PANEL");
    }

    $Log->exit() if $may_exit;
    return $TCD::PASS;
} ## end quota_report_bottom_panel

1;
