# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Schedule GUI Functionalities
##
## @description
##       Using this library, user can perform Cmode Schedule related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## synopsis:
##     use NACL::APISet::Host::GUI::SysManager::CMode::SmSchedule
##
##     schedule_create(%args);
##     schedule_edit(%args);
##     schedule_delete(%args);
##     schedule_refresh(%args);
##
## @status public
##
## @author Vinu.Nagarajan@netapp.com
## @change
##  2011-06-07 : vinu : Initial version
## @change
##  2012-03-20 : Vikas : POD modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmSchedule;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF );
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    schedule_create
    schedule_edit
    schedule_delete
    schedule_refresh
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::schedule_create
## @summary
##     Creation of schedule using GUI.
## @description
##     This method can be used to create schedule based on paramteres provided using GUI.
## @synopsis
##     schedule_create (%args);
## INPUTS TO METHOD CALL 'schedule_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -schedule_name </td> <td> (Required)Name of the schedule to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver to be used </td> </tr>
## <tr> <td> -basic </td> <td> Selecting configuration </td> </tr>
## <tr> <td> -weekday </td> <td> Selecting weekday </td> </tr>
## <tr> <td> -run_minute </td> <td> Selecting run_minute  </td> </tr>
## <tr> <td> -run_hour </td> <td> Selecting run_hour  </td> </tr>
## <tr> <td> -run_at_hour </td> <td> Selecting run_at_hour  </td> </tr>
## <tr> <td> -run_at_min </td> <td> Selecting run_at_min  </td> </tr>
## <tr> <td> -run_hour_at_minute </td> <td> Selecting run_hour_at_minute </td> </tr>
## <tr> <td> -interval </td> <td> Selecting interval </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub schedule_create {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-schedule_name' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $SCHEDULES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SCHEDULES_TREE );

    $Log->comment("Clicking schedule create button");
    $self->click($CONFIGURATION_SCHEDULE_CREATE);

    $Log->comment("Waiting for schedule creation");
    $self->wait_for_element_present( $CONFIGURATION_SCHEDULE_CREATE_OK_BTN,
        $GENERIC_WAIT );

    $Log->comment("Entering schedule name $args{'-schedule_name'}");
    $self->type( "$CONFIGURATION_SCHEDULE_NAME", $args{'-schedule_name'} );

    if ( exists( $args{'-basic'} ) ) {
        if ( lc( $args{'-basic'} ) eq 'daily' ) {
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC");
            $Log->comment("Selecting basic daily");
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC_DAILY");
        } elsif ( lc( $args{-basic} ) eq "weekly" ) {
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC_WEEKLY");
            if ( exists( $args{'-weekday'} ) ) {
                my $weekday        = $args{'-weekday'};
                my $weekdaylastidx = $#$weekday;
                for ( my $i = 0; $i <= $weekdaylastidx; $i++ ) {
                    if ( lc( $weekday->[$i] ) eq 'sunday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_SUNDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'monday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_MONDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'tuesday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_TUESDAY",
                            "enable" );

                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'wednesday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_WEDNESDAY",
                            "enable"
                        );

                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'thursday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_THURSDAY",
                            "enable" );

                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'friday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_FRIDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'saturday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_SATURDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                } ## end for ( my $i = 0; $i <= ...)
            } ## end if ( exists( $args{'-weekday'...}))
        } ## end elsif ( lc( $args{-basic}...))

        if ( exists( $args{'-run_minute'} ) ) {
            $Log->comment("*** Select weekday $args{-run_minute}***");
            $self->click( "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_MINUTE" );
            $Log->comment(
                " Selecting Security style as : $args{'-run_minute'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_MINUTE_COMBO",
                ( $args{-run_minute} ) );

        } elsif ( exists( $args{'-run_hour'} ) ) {
            $Log->comment("*** Select weekday $args{-run_hour}***");
            $self->click("$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR");
            $Log->comment(
                " Selecting Security style as : $args{'-run_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_HOUR_COMBO",
                ( $args{-run_hour} ) );

        } elsif ( ( exists( $args{'-run_at_hour'} ) )
            && ( exists( $args{'-run_at_min'} ) ) )
        {
            $Log->comment("*** Select weekday $args{-run_at_hour}***");
            $self->click(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR_MINUTE" );
            $Log->comment(
                " Selecting hour and  minute as : $args{'-run_at_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR_COMBO",
                uc( $args{-run_at_hour} ) );

            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_MINUTE_COMBO",
                uc( $args{-run_at_min} ) );

        } ## end elsif ( ( exists( $args{'-run_at_hour'...})))

        elsif ( exists( $args{'-run_hour_at_minute'} ) ) {
            $Log->comment("*** Select weekday $args{-run_hour_at_minute}***");
            $self->click("$CONFIGURATION_SCHEDULE_BASIC_RUN_HOUR_MINUTE");

            $Log->comment(" provinding hour and  minute  values");
            $self->type( "$CONFIGURATION_SCHEDULE_BASIC_RUN_TEXT_HOUR",
                $args{'-run_hour_text'} );

            $self->type( "$CONFIGURATION_SCHEDULE_BASIC_RUN_TEXT_MINUTE",
                $args{'-run_min_text'} );

        } ## end elsif ( exists( $args{'-run_hour_at_minute'...}))
    } elsif ( exists( $args{'-interval'} ) ) {
        $self->click("$CONFIGURATION_SCHEDULE_SELECT_INTERVAL");
        if ( lc( $args{'-interval'} ) eq 'run_every_day' ) {
            $Log->comment(
                "*** Select interval run every day $args{-run_every_day}***");
            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_DAY",
                "enable" );
            $Log->comment(
                " Selecting run every day as : $args{'-run_every_day'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_DAY",
                $args{-run_every_day} );
        } ## end if ( lc( $args{'-interval'...}))
        if ( lc( $args{'-interval'} ) eq 'run_every_hour' ) {
            $Log->comment(
                "*** Select interval run every hour $args{-run_every_hour}***"
            );
            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_HOUR",
                "enable" );
            $Log->comment(
                " Selecting run every hour as : $args{'-run_every_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_HOUR",
                lc( $args{-run_every_hour} ) );
        } ## end if ( lc( $args{'-interval'...}))
        if ( lc( $args{'-interval'} ) eq 'run_every_min' ) {
            $Log->comment(
                "*** Select interval run every min $args{-run_every_min}***");
            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_MINUTE",
                "enable" );
            $Log->comment(
                " Selecting run every min as : $args{'-run_every_min'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_MINUTE",
                uc( $args{-run_every_min} ) );
        } ## end if ( lc( $args{'-interval'...}))
    } ## end elsif ( exists( $args{'-interval'...}))

    $Log->comment("Selecting OK button to complete creation.");
    my $commit_time
        = $self->click_timecount($CONFIGURATION_SCHEDULE_CREATE_OK_BTN);
    $self->wait_for_loading();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("schedule created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub schedule_create

################################################################################
## @name ::schedule_edit
## @summary
##     Edit schedule using GUI.
## @description
##     This method can be used to edit schedule based on paramteres provided using GUI.
## @synopsis
##     schedule_edit (%args);
## INPUTS TO METHOD CALL 'schedule_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -schedule_name </td> <td> (Required)Name of the schedule to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver to be used </td> </tr>
## <tr> <td> -basic </td> <td> Selecting configuration </td> </tr>
## <tr> <td> -weekday </td> <td> Selecting weekday </td> </tr>
## <tr> <td> -run_minute </td> <td> Selecting run_minute  </td> </tr>
## <tr> <td> -run_hour </td> <td> Selecting run_hour  </td> </tr>
## <tr> <td> -run_at_hour </td> <td> Selecting run_at_hour  </td> </tr>
## <tr> <td> -run_at_min </td> <td> Selecting run_at_min  </td> </tr>
## <tr> <td> -run_hour_at_minute </td> <td> Selecting run_hour_at_minute </td> </tr>
## <tr> <td> -interval </td> <td> Selecting interval </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub schedule_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-schedule_name' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $SCHEDULES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SCHEDULES_TREE );
    $self->wait_for_loading();
        
    $Log->comment("selecting schedule");
    $self->click_row_with_texts( $CONFIGURATION_SCHEDULE_GRID,
        $args{-schedule_name} );
    $self->wait_for_loading();
    
    $Log->comment("Clicking schedule edit button");
    $self->click($CONFIGURATION_SCHEDULE_EDIT);
    $self->wait_for_loading();

    if ( exists( $args{'-basic'} ) ) {
        if ( lc( $args{'-basic'} ) eq 'daily' ) {
            $Log->comment("*** Select basic $args{-basic}***");
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC");
            $Log->comment("Selecting basic daily");
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC_DAILY");
        } elsif ( lc( $args{-basic} ) eq "weekly" ) {
            $Log->comment("*** Click weekly button***");
            $self->click("$CONFIGURATION_SCHEDULE_SELECT_BASIC_WEEKLY");
            if ( exists( $args{'-weekday'} ) ) {
                my $weekday        = $args{'-weekday'};
                my $weekdaylastidx = $#$weekday;
                for ( my $i = 0; $i <= $weekdaylastidx; $i++ ) {
                    if ( lc( $weekday->[$i] ) eq 'sunday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_SUNDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'monday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_MONDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'tuesday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_TUESDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'wednesday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_WEDNESDAY",
                            "enable"
                        );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'thursday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_THURSDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'friday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_FRIDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                    if ( lc( $weekday->[$i] ) eq 'saturday' ) {
                        $Log->comment(
                            "*** Select weekday $args{-weekday}***");
                        $self->set_checkbox(
                            "$CONFIGURATION_SCHEDULE_SELECT_BASIC_SATURDAY",
                            "enable" );
                    } ## end if ( lc( $weekday->[$i...]))
                } ## end for ( my $i = 0; $i <= ...)
            } ## end if ( exists( $args{'-weekday'...}))
        } ## end elsif ( lc( $args{-basic}...))

        if ( exists( $args{'-run_minute'} ) ) {
            $Log->comment("*** Select weekday $args{-run_minute}***");
            $self->click( "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_MINUTE" );
            $Log->comment(
                " Selecting Security style as : $args{'-run_minute'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_MINUTE_COMBO",
                ( $args{-run_minute} ) );
        } elsif ( exists( $args{'-run_hour'} ) ) {
            $Log->comment("*** Select weekday $args{-run_hour}***");
            $self->click("$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR");
            $Log->comment(
                " Selecting Security style as : $args{'-run_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_HOUR_COMBO",
                ( $args{-run_hour} ) );
        } elsif ( ( exists( $args{'-run_at_hour'} ) )
            && ( exists( $args{'-run_at_min'} ) ) )
        {
            $Log->comment("*** Select weekday $args{-run_at_hour}***");
            $self->click(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR_MINUTE" );
            $Log->comment(
                " Selecting hour and  minute as : $args{'-run_at_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_HOUR_COMBO",
                uc( $args{-run_at_hour} ) );

            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_BASIC_RUN_EVERY_MINUTE_COMBO",
                uc( $args{-run_at_min} ) );
        } ## end elsif ( ( exists( $args{'-run_at_hour'...})))
        elsif ( exists( $args{'-run_hour_at_minute'} ) ) {
            $Log->comment("*** Select weekday $args{-run_hour_at_minute}***");
            $self->click("$CONFIGURATION_SCHEDULE_BASIC_RUN_HOUR_MINUTE");

            $Log->comment(" provinding hour and  minute  values");
            $self->type( "$CONFIGURATION_SCHEDULE_BASIC_RUN_TEXT_HOUR",
                $args{'-run_hour_text'} );

            $self->type( "$CONFIGURATION_SCHEDULE_BASIC_RUN_TEXT_MINUTE",
                $args{'-run_min_text'} );
        } ## end elsif ( exists( $args{'-run_hour_at_minute'...}))
    } elsif ( exists( $args{'-interval'} ) ) {
        $self->click("$CONFIGURATION_SCHEDULE_SELECT_INTERVAL");

        if ( lc( $args{'-interval'} ) eq 'run_every_day' ) {
            $Log->comment(
                "*** Select interval run every day $args{-run_every_day}***");
            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_DAY",
                "enable" );
            $self->set_combobox_value(
                    "$CONFIGURATION_SCHEDULE_COMBO_SELECT_DAY",
                    "days");
            sleep 2;
            $Log->comment(
                " Selecting run every day as : $args{'-run_every_day'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_DAY",
                ( $args{-run_every_day} ) );
        } ## end if ( lc( $args{'-interval'...}))
        if ( lc( $args{'-interval'} ) eq 'run_every_hour' ) {
            $Log->comment(
                "*** Select interval run every hour $args{-run_every_hour}***"
            );
            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_HOUR",
                "enable" );
            $Log->comment(
                " Selecting run every hour as : $args{'-run_every_hour'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_HOUR",
                lc( $args{-run_every_hour} ) );
        } ## end if ( lc( $args{'-interval'...}))
        if ( lc( $args{'-interval'} ) eq 'run_every_min' ) {
            $Log->comment(
                "*** Select interval run every min $args{-run_every_min}***");

            $self->set_checkbox( "$CONFIGURATION_SCHEDULE_SELECT_MINUTE",
                "enable" );
            $Log->comment(
                " Selecting run every min as : $args{'-run_every_min'}***");
            $self->set_combobox_value(
                "$CONFIGURATION_SCHEDULE_COMBO_SELECT_MINUTE",
                uc( $args{-run_every_min} ) );
        } ## end if ( lc( $args{'-interval'...}))

    } ## end elsif ( exists( $args{'-interval'...}))

    $Log->comment("Selecting OK button to complete edition.");

#    my $commit_time = $self->click_timecount($CONFIGURATION_SCHEDULE_EDIT_OK);
    my $commit_time = 1; 
    $self->click($CONFIGURATION_SCHEDULE_EDIT_OK);
    $self->wait_for_loading();

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("schedule edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub schedule_edit

################################################################################
## @name ::schedule_delete
## @summary
##     Delete schedule using GUI.
## @description
##     This method can be used to delete schedule based on paramteres provided using GUI.
## @synopsis
##     schedule_delete (%args);
## INPUTS TO METHOD CALL 'schedule_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -schedule_name </td> <td> (Required)Name of the schedule to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub schedule_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-schedule_name' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $SCHEDULES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SCHEDULES_TREE );

    $Log->comment("selecting schedule");
    $self->click_row_with_texts( $CONFIGURATION_SCHEDULE_GRID,
        $args{-schedule_name} );

    $Log->comment("Clicking schedule delete button");
    $self->click($CONFIGURATION_SCHEDULE_DELETE_BTN);

    $Log->comment("Waiting for schedule deletion");
    $self->wait_for_element_present(
        $CONFIGURATION_SCHEDULE_DELETE_CANCEL_BTN,
        $GENERIC_WAIT );

    $Log->comment("clicking delete confirm button");
    $self->set_checkbox( $CONFIGURATION_SCHEDULE_DELETE_CHK_BOX, 'enable' );

    $Log->comment("Selecting OK button to complete deletion.");

    my $commit_time
        = $self->click_timecount($CONFIGURATION_SCHEDULE_DELETE_OK_BTN);
    $self->wait_for_loading();
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("schedule deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub schedule_delete

################################################################################
## @name ::schedule_refresh
## @summary
##     Refresh schedule using GUI.
## @description
##     This method can be used to refresh schedule based on paramteres provided using GUI.
## @synopsis
##     schedule_refresh (%args);
## INPUTS TO METHOD CALL 'schedule_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub schedule_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $SCHEDULES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SCHEDULES_TREE );

    $self->wait_for_element_present($CONFIGURATION_SCHEDULE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $CONFIGURATION_SCHEDULE_GRID, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub schedule_refresh

1;

