# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to handle GUI objects under Configuration>Secuirty like Password, SSH etc
##
## @description
##      Using this library, user can perform Security related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmSecurity;
##
##      security_user_add(%args);
##      security_user_modify(%args);
##      security_user_change_password(%args);
##      user_lock_unlock(%args);
##      user_refresh(%args);
##      security_role_add(%args);
##      security_role_modify(%args);
##      role_refresh(%args);
##      get_role_info(%args);
##      get_role_attribute_info(%args);
##      get_user_info(%args);
##      get_user_login_info(%args);
##
## @status public
##
## @author Vinu.Nagarajan@netapp.com
## @change
##  2011-06-07 : vinu : Initial version
## @change
##  2012-03-20 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmSecurity;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    security_user_add
    security_user_modify
    security_user_change_password
    user_lock_unlock
    user_refresh
    security_role_add
    security_role_modify
    role_refresh
    get_role_info
    get_role_attribute_info
    get_user_info
    get_user_login_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::security_user_add
## @summary
##   Add users using GUI.
## @description
##     This method can be used to add users using GUI.
## @synopsis
##    security_user_add(%args);
## INPUTS TO METHOD CALL 'security_user_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -username </td> <td> (Required) NAme of the user to be added </td> </tr>
## <tr> <td> -workflow </td> <td> (Required) Selecting workflow  </td> </tr>
## <tr> <td> -password </td> <td> (Required) Entering password </td> </tr>
## <tr> <td> -confirm_password </td> <td> (Required) Confirming password </td> </tr>
## <tr> <td> -application </td> <td> (Required) Selecting application </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## <tr> <td> -editapplication </td> <td> Edit the application </td> </tr>
## <tr> <td> -delapplication </td> <td> Deleting the application </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub security_user_add {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-username'         => { type => SCALAR },
            "-workflow"         => { type => SCALAR },
            "-password"         => { type => SCALAR },
            "-confirm_password" => { type => SCALAR },
            "-application"      => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step("Navigating to Vserver>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $Log->comment("Clicking user add button");
    $self->click($SECURITY_USERS_ADD_BTN);

    $Log->comment("Waiting for user add Dialog");
    $self->wait_for_element_present( $SECURITY_USERS_NAME_TEXTBOX,
        $GENERIC_WAIT );

    $Log->comment("Entering username: $args{-username}");
    $self->type_keys( $SECURITY_USERS_NAME_TEXTBOX, $args{-username} );
    $self->type( $SECURITY_USERS_NAME_TEXTBOX, $args{-username} );

    $Log->comment("Entering password: $args{-password}");
    $self->type_keys( $SECURITY_USERS_PASSWORD_TEXTBOX, $args{-password} );
    $self->type( $SECURITY_USERS_PASSWORD_TEXTBOX, $args{-password} );

    $Log->comment("Confirming password");
    $self->type_keys( $SECURITY_USERS_PASSWORD_CONFIRM_TEXTBOX,
        $args{-confirm_password} );
    $self->type( $SECURITY_USERS_PASSWORD_CONFIRM_TEXTBOX,
            $args{-confirm_password} );
    
    foreach my $addapplication ( @{ $args{-application} } ) {
        $Log->comment("clicking add button");
        $self->click($SECURITY_USERS_ADD);
        $self->wait_for_loading();
        my ( $applicationname, $rolename ) = split( /:/, $addapplication );
        $Log->comment("select application : $applicationname");
	$self->select_from_dropdown("$SECURITY_USERS_APPLICATION_COMBO", $applicationname, 1);
        $self->wait_for_loading();

        $Log->comment("select role : $rolename ");
	$self->select_from_dropdown("$SECURITY_USERS_ROLE_COMBO", $rolename, 1);

        $Log->comment("clicking application and roles add button");
        $self->click($SECURITY_USERS_OK_BTN);
        $self->check_for_error();
    } ## end foreach my $addapplication ...

    foreach my $editapplication ( @{ $args{-editapplication} } ) {
        my ( $applicationname, $rolename ) = split( /:/, $editapplication );
        $Log->comment("select application : $applicationname");
        $self->click_row_with_texts( "$SECURITY_USERS_ADD_GRID",
            $applicationname );
        $self->wait_for_loading();
        $Log->comment("clicking edit button");
        $self->click($SECURITY_USERS_EDIT);

        $Log->comment("select role : $rolename ");
        $self->select_from_dropdown( $SECURITY_USERS_ROLE_COMBO, $rolename, 1);

        $Log->comment("clicking application and roles add button");
        $self->click($SECURITY_USERS_OK_BTN);
        $self->check_for_error();
    } ## end foreach my $editapplication...

    foreach my $delapplication ( @{ $args{-delapplication} } ) {
        $Log->comment("Select application from grid:");
        $self->click_row_with_texts( "$SECURITY_USERS_ADD_GRID",
            $delapplication );
        $self->wait_for_loading();
        $Log->comment("deleting application and roles");
        $self->click($SECURITY_USERS_DELETE);
        $self->check_for_error();
    } ## end foreach my $delapplication ...

    $self->wait_for_loading();

    $Log->comment("clicking user add button");
    my $commit_time = $self->click_timecount($SECURITY_USERS_PAGE_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("user added successfully");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub security_user_add

################################################################################
## @name ::security_user_modify
## @summary
##   Modify users using GUI.
## @description
##     This method can be used to modify users using GUI.
## @synopsis
##    security_user_modify(%args);
## INPUTS TO METHOD CALL 'security_user_modify' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -username </td> <td> NAme of the user to be added </td> </tr>
## <tr> <td> -workflow </td> <td> Selecting workflow  </td> </tr>
## <tr> <td> -addapplication </td> <td> Adding the application </td> </tr>
## <tr> <td> -editapplication </td> <td> Edit the application </td> </tr>
## <tr> <td> -delapplication </td> <td> Deleting the application </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub security_user_modify {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-username' => { type => SCALAR },
            "-workflow" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my ( $flag, );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }
        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $USERS_TREE );
    }

    $Log->comment("Select user $args{'-username'}");
    $self->click_row_with_texts( "$SECURITY_USERS_GRID", $args{'-username'} );

    $Log->comment("Clicking user modify button");
    $self->click($SECURITY_USERS_EDIT_BTN);

    $Log->comment("Waiting for user edit Dialog");
    $self->wait_for_element_present( $SECURITY_USERS_ADD_BTN, $GENERIC_WAIT );
    $self->wait_for_loading();

    foreach my $addapplication ( @{ $args{-application} } ) {
        $flag = 1;
        $Log->comment("clicking add button");
        $self->click($SECURITY_USERS_ADD);
        $self->wait_for_loading();
        my ( $applicationname, $rolename ) = split( /:/, $addapplication );
        $Log->comment("select application : $applicationname");
        $self->select_from_dropdown( $SECURITY_USERS_APPLICATION_COMBO,
            $applicationname, 1);
        $self->wait_for_loading();

        $Log->comment("select role : $rolename ");
        $self->select_from_dropdown( $SECURITY_USERS_ROLE_COMBO, $rolename, 1);

        $Log->comment("clicking application and roles add button");
        $self->click($SECURITY_USERS_OK_BTN);
        $self->check_for_error();
    }

    foreach my $editapplication ( @{ $args{-editapplication} } ) {
        $flag = 1;
        my ( $applicationname, $rolename ) = split( /:/, $editapplication );
        $Log->comment("select application : $applicationname");
        $self->click_row_with_texts( "$SECURITY_USERS_ADD_GRID",
            $applicationname );
        $self->wait_for_loading();
        $Log->comment("clicking edit button");
        $self->click($SECURITY_USERS_EDIT);

        $Log->comment("select role : $rolename ");
        $self->select_from_dropdown( $SECURITY_USERS_ROLE_COMBO, $rolename, 1);

        $Log->comment("clicking application and roles add button");
        $self->click($SECURITY_USERS_OK_BTN);
        $self->check_for_error();
    }

    foreach my $delapplication ( @{ $args{-delapplication} } ) {
        $flag = 1;
        $Log->comment("Select application from grid:");
        $self->click_row_with_texts( "$SECURITY_USERS_ADD_GRID",
            $delapplication );
        $self->wait_for_loading();
        $Log->comment("deleting application and roles");
        $self->click($SECURITY_USERS_DELETE);
    }

    if ( $flag != 1 ) {
        NATE::BaseException->throw(
            "Modify Options not specified . Specify either edit, delete or add"
        );
    }

    $Log->comment("Clicking on OK button");
    my $commit_time = $self->click_timecount($SECURITY_USERS_PAGE_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("user modify successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub security_user_modify

################################################################################
## @name ::security_user_change_password
## @summary
##   Change user's password using GUI.
## @description
##     This method can be used to change user's password using GUI.
## @synopsis
##    security_user_change_password(%args);
## INPUTS TO METHOD CALL 'security_user_change_password' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -username </td> <td> NAme of the user to be added </td> </tr>
## <tr> <td> -workflow </td> <td> Selecting workflow  </td> </tr>
## <tr> <td> -password </td> <td> Entering password </td> </tr>
## <tr> <td> -confirm_password </td> <td> Confirming password </td> </tr>
## <tr> <td> -application </td> <td> Selecting application </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub security_user_change_password {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-username'         => { type => SCALAR },
            "-password"         => { type => SCALAR },
            "-confirm_password" => { type => SCALAR },
            "-workflow"         => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $Log->comment("Select user $args{'-username'}");
    $self->click_row_with_texts( "$SECURITY_USERS_GRID", $args{-username} );

    $Log->comment("Clicking password change button");
    $self->click($SECURITY_USERS_PASSWORD_CHANGE_BTN);

    $Log->comment("Waiting for password change Dialog");
    $self->wait_for_element_present(
        $SECURITY_USERS_PASSWORD_CHANGE_NEW_PASSWORD_TEXTBOX,
        $GENERIC_WAIT );

    $Log->comment("Entering password:");
    $self->type_keys( $SECURITY_USERS_PASSWORD_CHANGE_NEW_PASSWORD_TEXTBOX,
        $args{-password} );

    $Log->comment("Confirming password");
    $self->type_keys(
        $SECURITY_USERS_PASSWORD_CHANGE_CONFIRM_PASSWORD_TEXTBOX,
        $args{-confirm_password} );

    $Log->comment("clicking change button");
    $self->click($SECURITY_CHANGE_PASSWORD_OK_BUTTON);
    my $commit_time
        = $self->click_timecount($SECURITY_CHANGE_CONFIRM_PASSWORD_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("user modify successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub security_user_change_password

################################################################################
## @name ::user_lock_unlock
## @summary
##   Lock user page using GUI.
## @description
##     This method can be used to lock user page using GUI.
## @synopsis
##    user_lock_unlock(%args);
## INPUTS TO METHOD CALL 'user_lock_unlock' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -username </td> <td> NAme of the user to be added </td> </tr>
## <tr> <td> -workflow </td> <td> Selecting workflow  </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub user_lock_unlock {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-username' => { type => SCALAR },
            "-workflow" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $self->wait_for_element_present($SECURITY_USERS_GRID);

    $Log->comment("Select user $args{'-username'}");
    $self->click_row_with_texts( "$SECURITY_USERS_GRID", $args{-username} );

    $Log->step("locking or unlocking user $args{'-username'}");
    my $commit_time
        = $self->click_timecount( $SECURITY_USERS_LOCK_UNLOCK, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("user locking and unlocking successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub user_lock_unlock

################################################################################
## @name ::user_refresh
## @summary
##   Refresh user page using GUI.
## @description
##     This method can be used to refresh user page using GUI.
## @synopsis
##    user_refresh(%args);
## INPUTS TO METHOD CALL 'user_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -workflow </td> <td> Selecting workflow  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub user_refresh {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-workflow' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $self->wait_for_element_present($SECURITY_USERS_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $SECURITY_USERS_PAGE_REFRESH, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("User page refresh successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub user_refresh

################################################################################
## @name ::security_role_add
## @summary
##   Add role using GUI.
## @description
##     This method can be used to add role using GUI.
## @synopsis
##    security_role_add(%args);
## INPUTS TO METHOD CALL 'security_role_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -rolename </td> <td> (Required) NAme of the role to be added </td> </tr>
## <tr> <td> -workflow </td> <td> (Required) Selecting workflow  </td> </tr>
## <tr> <td> -addcommand </td> <td> (Required) Adding command </td> </tr>
## <tr> <td> -editcommand </td> <td>  Editing command </td> </tr>
## <tr> <td> -deletecommand </td> <td> Deleting command </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub security_role_add {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-rolename'   => { type => SCALAR },
            "-workflow"   => { type => SCALAR },
            "-addcommand" => { type => ARRAYREF },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $ROLES_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $ROLES_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $ROLES_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $Log->comment("Clicking role add button");
    $self->click($SECURITY_ROLE_ADD_BTN);

    $Log->comment("Waiting for role add Dialog");
    $self->wait_for_element_present( $SECURITY_ROLE_NAME_TEXTBOX,
        $GENERIC_WAIT );

    $Log->comment("Entering role name:$args{-rolename}");
    $self->type_keys( $SECURITY_ROLE_NAME_TEXTBOX, $args{-rolename} );
    $self->type( $SECURITY_ROLE_NAME_TEXTBOX, $args{-rolename} );

    foreach my $addcommand ( @{ $args{'-addcommand'} } ) {
        $Log->comment("clicking add button");
        $self->click($SECURITY_ROLE_ADD);

        my ( $command, $query, $accesslevel ) = split( /:/, $addcommand );
        $Log->comment("Entering command:$command");
        $self->type_keys( $SECURITY_ROLE_COMMAND_TEXTBOX, $command );
        $self->type( $SECURITY_ROLE_COMMAND_TEXTBOX, $command );

        $Log->comment("Entering query:$query");
        $self->type_keys( $SECURITY_ROLE_QUERY_TEXTBOX, $query );

        $Log->comment("select access level : $accesslevel");
        $self->select_from_dropdown( $SECURITY_ROLE_ACCESS_LEVEL_COMBO,
            $accesslevel, 1);

        $Log->comment("clicking application and roles add button");
        $self->click($SECURITY_ROLES_OK_BTN);
        sleep 1;
    } ## end foreach my $addcommand ( @{...})
    foreach my $editcommand ( @{ $args{'-editcommand'} } ) {
        my ( $command, $query, $accesslevel ) = split( /:/, $editcommand );
        $Log->comment("Select command from grid:");
        $self->click_row_with_texts( "$SECURITY_ROLE_ADD_GRID", $command );

        $Log->comment("clicking edit button");
        $self->click($SECURITY_ROLE_EDIT);
        $self->wait_for_element_present($SECURITY_ROLE_QUERY_TEXTBOX);

        $Log->comment("enter query : $query");
        $self->type_keys( $SECURITY_ROLE_QUERY_TEXTBOX, $query );

        $Log->comment("selecting access level : $accesslevel");
        $self->select_from_dropdown( $SECURITY_ROLE_ACCESS_LEVEL_COMBO,
            $accesslevel, 1);

        $Log->comment("clicking role attribute add button");
        $self->click($SECURITY_ROLES_OK_BTN);
        sleep 1;
    } ## end foreach my $editcommand ( @...)
    foreach my $deletecommand ( @{ $args{'-deletecommand'} } ) {
        $Log->comment("Select command from grid:");
        $self->click_row_with_texts( "$SECURITY_ROLE_ADD_GRID",
            $args{$deletecommand} );

        $Log->comment("deleting command and query");
        $self->click($SECURITY_ROLE_DELETE);
    } ## end foreach my $deletecommand (...)

    if ( $self->is_element_present($SECURITY_ROLES_OK_BTN)) {
        $Log->comment("Clicking role add OK button");
        eval {
            $self->click($SECURITY_ROLES_OK_BTN);
        };
        if ($@) { 
            $Log->comment("Attempted click on $SECURITY_ROLES_OK_BTN Failed,"
                        . "Suppressing issue and continuing with execution");
        }
    }

    $Log->comment("Clicking role add button");
    my $commit_time = $self->click_timecount($SECURITY_ROLE_PAGE_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Role $args{-rolename} added successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub security_role_add

################################################################################
## @name ::security_role_modify
## @summary
##   Modify roles using GUI.
## @description
##     This method can be used to modify roles using GUI.
## @synopsis
##    security_role_modify(%args);
## INPUTS TO METHOD CALL 'security_role_modify' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -rolename </td> <td> (Required) NAme of the role to be added </td> </tr>
## <tr> <td> -workflow </td> <td> (Required) Selecting workflow  </td> </tr>
## <tr> <td> -addcommand </td> <td> (Required) Adding command </td> </tr>
## <tr> <td> -editcommand </td> <td>  Editing command </td> </tr>
## <tr> <td> -deletecommand </td> <td> Deleting command </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub security_role_modify {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-rolename' => { type => SCALAR },
            "-workflow" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my ( $flag, );
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $ROLES_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $ROLES_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }
        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $ROLES_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $Log->comment("Select role $args{'-rolename'}");
    $self->click_row_with_texts( "$SECURITY_ROLE_GRID", $args{-rolename} );

    $Log->comment("Clicking role modify button");
    $self->click($SECURITY_ROLE_EDIT_BTN);

    foreach my $addcommand ( @{ $args{'-addcommand'} } ) {
        $flag = 1;
        $Log->comment("clicking add button");
        $self->click($SECURITY_ROLE_ADD);

        my ( $command, $query, $accesslevel ) = split( /:/, $addcommand );
        $Log->comment("Entering command:");
        $self->type_keys( $SECURITY_ROLE_COMMAND_TEXTBOX, $command );

        $Log->comment("Entering query:");
        $self->type_keys( $SECURITY_ROLE_QUERY_TEXTBOX, $query );

        $Log->comment("select access level : $accesslevel");
        $self->select_from_dropdown( $SECURITY_ROLE_ACCESS_LEVEL_COMBO,
            $accesslevel, 1);
    } ## end foreach my $addcommand ( @{...})
    foreach my $editcommand ( @{ $args{'-editcommand'} } ) {
        $flag = 1;
        my ( $command, $query, $accesslevel ) = split( /:/, $editcommand );
        $Log->comment("Select command from grid:");
        $self->click_row_with_texts( "$SECURITY_ROLE_ADD_GRID", $command );

        $Log->comment("clicking edit button");
        $self->click($SECURITY_ROLE_EDIT);
        $self->wait_for_element_present($SECURITY_ROLE_QUERY_TEXTBOX);

        $Log->comment("enter query : $query");
        $self->type_keys( $SECURITY_ROLE_QUERY_TEXTBOX, $query );

        $Log->comment("selecting access level : $accesslevel");
        $self->select_from_dropdown( $SECURITY_ROLE_ACCESS_LEVEL_COMBO,
            $accesslevel, 1);

        $Log->comment("clicking role attribute add button");
        $self->click($SECURITY_ROLES_OK_BTN);
    } ## end foreach my $editcommand ( @...)
    foreach my $deletecommand ( @{ $args{'-deletecommand'} } ) {
        $flag = 1;
        $Log->comment("Select command from grid:");
        $self->click_row_with_texts( "$SECURITY_ROLE_ADD_GRID",
            $args{$deletecommand} );

        $Log->comment("deleting command and query");
        $self->click($SECURITY_ROLE_DELETE);
    } ## end foreach my $deletecommand (...)

    if ( $flag != 1 ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Specify modify options - edit/delete/add");
    }
    my $commit_time = $self->click_timecount($SECURITY_ROLE_PAGE_OK_BTN);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Role $args{'-rolename'} modified successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub security_role_modify

################################################################################
## @name ::role_refresh
## @summary
##    Refresh role page using GUI.
## @description
##     This method can be used to refresh role page using GUI.
## @synopsis
##    role_refresh(%args);
## INPUTS TO METHOD CALL 'role_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -workflow </td> <td> (Required) Selecting workflow  </td> </tr>
## <tr> <td> -vservername </td> <td> NAme of the vserver to be used </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub role_refresh {

    my $self = shift;
    my $loading_time;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-workflow" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $ROLES_TREE");
        $loading_time
            = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
            $ROLES_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
            "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time
            = $self->cluster_tree_navigate( $vserverName, $ROLES_TREE );
    } ## end elsif ( lc( $args{-workflow...}))

    $self->wait_for_element_present($SECURITY_ROLE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time = $self->click_timecount( $SECURITY_ROLE_PAGE_REFRESH_BTN,
        "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("role page refresh successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub role_refresh

################################################################################
# Subroutine Name: get_role_info
# Objective:       It will fetch the role details
# Details:         See POD documentation at the end of this file
################################################################################
sub get_role_info {
    
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { 
            "-workflow" => { type => SCALAR },
            "-rolename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ($options,
        $loading_time,
        $roles_info
    );
    my $rolename = $args{'-rolename'};
    my $workflow = $args{'-workflow'};
    
    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $ROLES_TREE");
        $loading_time = $self->cluster_tree_navigate(
            $self->{$SYSTEM_NAME_KEY}, $ROLES_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment("Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }
        $Log->step("Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time = $self->cluster_tree_navigate( $vserverName,
                $ROLES_TREE );
    } ## end elsif ( lc( $args{-workflow}...))

    $Log->comment("Select role $rolename");
    $self->click_row_with_texts( "$SECURITY_ROLE_GRID",
            $args{-rolename} );

    $Log->step("Getting row data for roles $args{-rolename} from grid");
    my $rowdata
        = $self->get_row_data_with_text( $SECURITY_ROLE_GRID, $args{-rolename} );

    my @rowarray = split( ' ', $rowdata );
    my %temp_roles_info = (
        "role" => $rowarray[0],
    );

    $Log->comment("Collection of roles info is successfull");
    $roles_info->{loading_time} = $loading_time;
    $roles_info->{parsed_output} = \%temp_roles_info;

    $Log->exit() if $may_exit;

    return $roles_info;
} ## end sub get_role_info

################################################################################
# Subroutine Name: get_role_attribute_info
# Objective:       It will fetch the role Attribute details
# Details:         See POD documentation at the end of this file
################################################################################
sub get_role_attribute_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => {
            "-workflow" => { type => SCALAR },
            "-rolename" => { type => SCALAR },
            "-commandname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ($options,
        $loading_time,
        $roles_info
    );
   
    my $rolename    = $args{'-rolename'};
    my $commandname = $args{'-commandname'};
    my $workflow    = $args{'-workflow'};
    
    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $ROLES_TREE");
        $loading_time = $self->cluster_tree_navigate(
            $self->{$SYSTEM_NAME_KEY},
                    $ROLES_TREE );

    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                    "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }
        $Log->step(
                "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time = $self->cluster_tree_navigate( $vserverName,
                $ROLES_TREE );
    } ## end elsif ( lc( $args{-workflow}...))

    $Log->comment("Select role $rolename");
    $self->click_row_with_texts( "$SECURITY_ROLE_GRID",
            $args{-rolename} );

    $Log->comment("Select command $commandname");
    $self->click_row_with_texts( "gwt-debug-CustomGrid",
            $args{-commandname} );

    $Log->step("Getting row data for command $commandname from grid");
    my $rowdata
        = $self->get_row_data_with_text( "gwt-debug-CustomGrid", $args{-commandname} );

    my @rowarray = split( ' ', $rowdata );
    my %temp_roles_info = (
        "query"        => $rowarray[1],
        "access_level" => $rowarray[2],
    );

    $Log->comment("Collection of roles info is successfull");
    $roles_info->{loading_time} = $loading_time;
    $roles_info->{parsed_output} = \%temp_roles_info;

    $Log->exit() if $may_exit;

    return $roles_info;
} ## end sub get_role_attribute_info

################################################################################
# Subroutine Name: get_user_info
# Objective:       it will fetch the user Details
# Details:         See POD documentation at the end of this file
################################################################################
sub get_user_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => {
            "-workflow" => { type => SCALAR },
            "-username" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ( $loading_time, $user_info );
    my $username = $args{'-username'};
    my $workflow = $args{'-workflow'};
    
    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time = $self->cluster_tree_navigate(
            $self->{$SYSTEM_NAME_KEY}, $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                    "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
                "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time = $self->cluster_tree_navigate( $vserverName,
                $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow}...))

    $self->wait_for_element_present($SECURITY_USERS_GRID);

    $Log->comment("Select user $username");
    $self->click_row_with_texts( "$SECURITY_USERS_GRID",
            $args{-username} );

    $Log->step("Getting row data for users $username from grid");
    my $rowdata
        = $self->get_row_data_with_text( $SECURITY_USERS_GRID, $username );

    my @rowarray = split( ' ', $rowdata );
    my %temp_user_info = (
        "account_locked" => $rowarray[1],
    );

    $Log->comment("Collection of user info is successfull");
    $user_info->{loading_time} = $loading_time;
    $user_info->{parsed_output} = \%temp_user_info;

    $Log->exit() if $may_exit;

    return $user_info;
} ## end sub get_user_info

################################################################################
# Subroutine Name: get_user_login_info
# Objective:       it will fetch the user login methods Details
# Details:         See POD documentation at the end of this file
################################################################################
sub get_user_login_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => {
            "-workflow" => { type => SCALAR },
            "-username" => { type => SCALAR },
            "-applicationname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my ( $loading_time, $user_info );
    my $username = $args{'-username'};
    my $applicationname = $args{'-applicationname'};
    my $workflow = $args{'-workflow'};
    
    if ( lc( $args{-workflow} ) eq 'cluster' ) {
        $Log->step("Navigating to $USERS_TREE");
        $loading_time = $self->cluster_tree_navigate(
            $self->{$SYSTEM_NAME_KEY}, $USERS_TREE );
    } elsif ( lc( $args{-workflow} ) eq 'vserver' ) {
        my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
        if ( exists( $args{'-vservername'} ) ) {
            $Log->comment(
                    "Overwrighting the default Vserver for this function");
            $vserverName = $args{'-vservername'};
        }

        $Log->step(
                "Navigating to Vserver>Storage>Configuration>Security>users");
        $loading_time = $self->cluster_tree_navigate( $vserverName,
                $USERS_TREE );
    } ## end elsif ( lc( $args{-workflow}...))

    $self->wait_for_element_present($SECURITY_USERS_GRID);

    $Log->comment("Select user $username");
    $self->click_row_with_texts( "$SECURITY_USERS_GRID",
            $args{-username} );

    $Log->comment("Select application name $applicationname");
    $self->click_row_with_texts( "gwt-debug-CustomGrid",
            $args{-applicationname} );

    $Log->step("Getting row data for application $applicationname from grid");
    my $rowdata
        = $self->get_row_data_with_text( "gwt-debug-CustomGrid", $args{-applicationname} );

    my @rowarray = split( ' ', $rowdata );
    my %temp_user_info = (
        "authentication" => $rowarray[1],
        "role"           => $rowarray[2],
    );

    $Log->comment("Collection of user info is successfull");
    $user_info->{loading_time} = $loading_time;
    $user_info->{parsed_output} = \%temp_user_info;

    $Log->exit() if $may_exit;

    return $user_info;
} ## end sub get_user_login_info

1;
