## Copyright (c) 2009 Network Appliance, Inc.
## All rights reserved.

## @summary
##      Routines to parse Cmode Services related GUI Functionalities
##
## @description
##    	Using this library, user can perform Cmode Services related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmServices.pm
##
##      nis_create(%args);
##      nis_edit(%args);
##      nis_delete(%args);
##      nis_refresh(%args);
##      ldap_add(%args);
##      ldap_edit(%args);
##      ldap_delete(%args);
##      ldap_refresh(%args);
##      vserver_ldap_edit(%args);
##      dns_edit(%args);
##      dns_refresh(%args);
##      kerberos_edit(%args);
##      kerberos_refresh(%args);
##      cluster_kerberos_create(%args);
##      cluster_kerberos_edit(%args);
##      cluster_kerberos_delete(%args);
##      cluster_kerberos_refresh(%args);
##      get_vserver_ldap_info(%args)
##      get_nis_info(%args)
##
##
## @status public
##
## @author vikas.p@netapp.com
##
## @change
##  2011-07-04 : vikasp : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmServices;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF );
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    nis_create
    nis_edit
    nis_delete
    nis_refresh
    ldap_add
    ldap_edit
    ldap_delete
    ldap_refresh
    vserver_ldap_edit
    dns_edit
    dns_refresh
    kerberos_edit
    kerberos_refresh
    cluster_kerberos_create
    cluster_kerberos_edit
    cluster_kerberos_delete
    cluster_kerberos_refresh
    get_vserver_ldap_info
    get_ldap_info
    get_kerberos_info
    get_dns_status_info
    get_nis_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::nis_create
## @summary
##     Create NIS domain with input parameters using GUI.
## @description
##     This method can be used to create NIS domain with input parameters using GUI.
## @synopsis
##    nis_create(%args);
## INPUTS TO METHOD CALL 'nis_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -domainname </td> <td> (Required) Name of the NIS domain </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -addnisserver </td> <td> NIS servers to be added </td> </tr>
## <tr> <td> -delnisserver </td> <td> NIS servers to be deleted </td> </tr>
## <tr> <td> -activedomain </td> <td> Enable active domain </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub nis_create {

    my $disable_flag = 0;
    my $self         = shift;
    my %args         = validate_with(
        params      => \@_,
        spec        => { "-domainname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>NIS");
    my $loading_time;
    eval {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE );
    };
    if ($@) {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE_SPACE );
    }

    $Log->comment("Clicking on NIS Add button");
    $self->click($NIS_ADD_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Entering NIS Domain: $args{'-domainname'}");
    $self->type( $NIS_NIS_DOMAIN_TEXTBOX, $args{'-domainname'} );

    if ( exists( $args{'-addnisserver'} ) ) {
        $Log->comment("Clicking on the NIS Server Add Button");
        $self->click($NIS_SERVER_ADD_BUTTON);
    #    $self->wait_for_element_present( $NIS_SERVER_ADDRESS_TEXTBOX,
     #       $GENERIC_WAIT );
        $self->wait_for_loading();

        $Log->comment("Entering NIS Domain Server: $args{'-addnisserver'}");
        $self->type( $NIS_SERVER_ADDRESS_TEXTBOX, $args{'-addnisserver'} );

        $Log->comment("Clicking on the OK Button");
        $self->click($NIS_SERVER_ADDRESS_OK_BUTTON);
        $self->wait_for_loading();

    } ## end if ( exists( $args{'-addnisserver'...}))

    if ( exists( $args{'-delnisserver'} ) ) {
        $Log->comment(
            "selecting NIS Domain server to delete: $args{'-deldomain'}");
        $self->click_row_with_texts( $NIS_SERVER_ADDRESS_GRID,
            $args{'-delnisserver'} );

        $Log->comment("clicking delete button");
        $self->click($NIS_SERVER_ADDRESS_DELETE_BUTTON);
    } ## end if ( exists( $args{'-delnisserver'...}))

    if ( exists( $args{'-activedomain'} ) ) {
        $Log->comment("Selecting Active domain : $args{'-activedomain'}");
        if ( lc( $args{'-activedomain'} eq "enable" ) ) {
            if ( not( $self->is_checked("$NIS_DOMAIN_ACTIVE_CHECKBOX") ) ) {
                $self->click("$NIS_DOMAIN_ACTIVE_CHECKBOX");
            }
        } elsif ( lc( $args{'-activedomain'} eq "disable" ) ) {
            if ( $self->is_checked("$NIS_DOMAIN_ACTIVE_CHECKBOX") ) {
                $disable_flag = 1;
                $self->click("$NIS_DOMAIN_ACTIVE_CHECKBOX");
            }
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid value for Active domain, valid values are 'enable' or 'disable'"
            );
        } ## end else [ if ( lc( $args{'-activedomain'...}))]
    } ## end if ( exists( $args{'-activedomain'...}))

    $Log->comment("Clicking on Create button");
    my $commit_time = $self->click_timecount($NIS_DOMAIN_CREATE_BUTTON);
    $self->wait_for_loading();

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("NIS created successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub nis_create

################################################################################
## @name ::nis_edit
## @summary
##     Edit NIS domain properties using GUI.
## @description
##     This method can be used to edit NIS domain properties using GUI.
## @synopsis
##    nis_edit(%args);
## INPUTS TO METHOD CALL 'nis_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -domainname </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -addnisserver </td> <td> Path where the export has to be created </td> </tr>
## <tr> <td> -delnisserver </td> <td> Adding new client permission rule </td> </tr>
## <tr> <td> -activedomain </td> <td> Select the permissions to be edited from the grid </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub nis_edit {

    my $disable_flag = 0;
    my $self         = shift;
    my %args         = validate_with(
        params      => \@_,
        spec        => { "-domainname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>NIS");
    my $loading_time;
    eval {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE );
    };
    if ($@) {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE_SPACE );
    }
    $self->wait_for_loading();

    $Log->step("Selecting NIS :$args{'-domainname'}");
    my $dom = $args{'-domainname'};

    $Log->comment("Clicking on the Edit button");
    $self->click($NIS_VSERVER_EDIT_BUTTON);
    #$self->wait_for_element_present( $NIS_SERVER_EDIT_ADD_BUTTON,
     #   $GENERIC_WAIT );

    $self->wait_for_loading();
    
    if ( exists( $args{'-addnisserver'} ) ) {
        $Log->comment("Clicking on the NIS Server Edit Add Button");
        $self->click($NIS_SERVER_EDIT_ADD_BUTTON);
   #     $self->wait_for_element_present( $NIS_SERVER_ADDRESS_EDIT_TEXTBOX,
    #        $GENERIC_WAIT );

        $self->wait_for_loading();

        $Log->comment("Entering NIS Domain Server: $args{'-addnisserver'}");
        $self->type( $NIS_SERVER_ADDRESS_EDIT_TEXTBOX,
            $args{'-addnisserver'} );

        $Log->comment("Clicking on the OK Button");
        $self->click($NIS_SERVER_ADDRESS_EDIT_OK_BUTTON);
        $self->wait_for_loading();
    } ## end if ( exists( $args{'-addnisserver'...}))

    if ( exists( $args{'-delnisserver'} ) ) {
        $Log->comment(
            "selecting NIS Domain server to delete: $args{'-deldomain'}");
        $self->click_row_with_texts( $NIS_SERVER_ADDRESS_EDIT_GRID,
            $args{'-delnisserver'} );

        $Log->comment("Clicking Edit OK button from Add server popup");
        $self->click($NIS_SERVER_ADDRESS_EDIT_OK_BUTTON);
        $self->wait_for_loading();
    } ## end if ( exists( $args{'-delnisserver'...}))

    if ( exists( $args{'-activedomain'} ) ) {
        $Log->comment("Selecting Active domain : $args{'-activedomain'}");
        if ( lc( $args{'-activedomain'} eq "enable" ) ) {
            if ( not( $self->is_checked("$NIS_DOMAIN_ACTIVE_CHECKBOX") ) ) {
                $self->click("$NIS_DOMAIN_ACTIVE_CHECKBOX");
                $self->wait_for_loading();
            }
        } elsif ( lc( $args{'-activedomain'} eq "disable" ) ) {
            $Log->comment(
                "Midhun: disabling the NIS: $args{'-activedomain'}");
            if ( $self->is_checked("$NIS_DOMAIN_ACTIVE_CHECKBOX") ) {
                $Log->comment("Midhun: Before Click");
                $self->click("$NIS_DOMAIN_ACTIVE_CHECKBOX");
                $self->wait_for_loading();
                $disable_flag = 1;
                $Log->comment("Midhun: After Click");
            } ## end if ( $self->is_checked...)
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid value for Active domain, valid values are 'enable' or 'disable'"
            );
        } ## end else [ if ( lc( $args{'-activedomain'...}))]
    } ## end if ( exists( $args{'-activedomain'...}))
    $Log->comment("Clicking Edit OK button from last screen");
    #my $commit_time = $self->click_timecount($NIS_DOMAIN_EDIT_BUTTON, "refresh");
    my $commit_time = 1; 
    $self->click($NIS_DOMAIN_EDIT_BUTTON);
     
    sleep(10);
       
    if ( $disable_flag == 1 ) {
        #$self->wait_for_element_present($NIS_DOMAIN_DISABLE_CONFIRM_BUTTON);
        $Log->comment("Click on confirmation Ok button");
        $self->click($NIS_DOMAIN_DISABLE_CONFIRM_BUTTON);
        $self->wait_for_loading();

    }
    $self->wait_for_loading();
    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("NIS edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub nis_edit

################################################################################
## @name ::nis_delete
## @summary
##     Delete NIS domain using GUI.
## @description
##     This method can be used to delete NIS domain using GUI.
## @synopsis
##    nis_delete(%args);
## INPUTS TO METHOD CALL 'nis_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -domainname </td> <td> (Required) Name of the export to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub nis_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-domainname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>NIS");
    my $loading_time;
    eval {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE );
    };
    if ($@) {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE_SPACE );
    }
    $self->wait_for_loading();

    $Log->step("Selecting NIS :$args{'-domainname'}");
    #$self->click_row_with_texts( "$NIS_GRID", $args{'-domainname'} );
    $self->wait_for_loading();
    $Log->comment("Clicking on the Delete button");
    $self->click($NIS_DELETE_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Clicking on checkbox on NIS delete confirmation");
    $self->set_checkbox( "$NIS_DELETE_CONFIRM_CHECKBOX_BUTTON", "enable" );

    $Log->comment("Clicking on Delete OK button");
    my $commit_time = $self->click_timecount($NIS_DELETE_OK_BUTTON);
    $self->wait_for_loading();
    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("NIS deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub nis_delete

################################################################################
## @name ::nis_refresh
## @summary
##     Refresh NIS page using GUI.
## @description
##     This method can be used to Refresh NIS page using GUI.
## @synopsis
##    nis_refresh(%args);
## INPUTS TO METHOD CALL 'nis_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub nis_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>NIS");
    my $loading_time;
    eval {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE );
    };
    if ($@) {
        $loading_time = $self->cluster_tree_navigate($vserverName, $NIS_TREE_SPACE );
    }

    $Log->comment("Clicking on the NIS Refresh button");
    my $commit_time
        = $self->click_timecount( $NIS_REFRESH_BUTTON, "refresh" );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("NIS page refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub nis_refresh

################################################################################
## @name ::ldap_add
## @summary
##     Create ldap using GUI.
## @description
##     This method can be used to create ldap using GUI.
## @synopsis
##    ldap_add(%args);
## INPUTS TO METHOD CALL 'ldap_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ldapclient </td> <td> (Required) Name of the ldap client to be added </td> </tr>
## <tr> <td> -ldapserver </td> <td> Name of the ldap server on which the client is to be added </td> </tr>
## <tr> <td> -ldapaddomain </td> <td> Entering LDAP Active Directory Domain </td> </tr>
## <tr> <td> -ldapserveradd </td> <td> selecting Active Directory Domain server </td> </tr>
## <tr> <td> -ldapserverdel </td> <td> Deleting the ldap server </td> </tr>
## <tr> <td> -ldapauthlevel </td> <td> Selecting ldap Authentication Level </td> </tr>
## <tr> <td> -ldapbinddn </td> <td> Enter ldap Bind DN </td> </tr>
## <tr> <td> -ldapbindupass </td> <td> Enter ldap Bind user password </td> </tr>
## <tr> <td> -ldapbasedn </td> <td> Enter ldap Bind DN </td> </tr>
## <tr> <td> -ldaptcpport </td> <td> Enter ldap tcp port </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub ldap_add {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ldapclient" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Configuration>Services>LDAP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName,
        $LDAP_TREE );
    $self->wait_for_loading();

    $Log->comment("Clicking on the Add button");
    $self->click($LDAP_ADD_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Clicking on the General Tab");
    $self->click($LDAP_ADD_GENERAL_TAB);
    $self->wait_for_loading();

    $Log->comment("Entering LDAP client config: $args{'-ldapclient'}");
    $self->type_keys( $LDAP_ADD_CLIENTCONFIG_INPUT, $args{'-ldapclient'} );
    $self->type( $LDAP_ADD_CLIENTCONFIG_INPUT, $args{'-ldapclient'} );

    #setting the value for LDAP server
    my $ldapserver
        = exists( $args{'-ldapserver'} ) ? $args{'-ldapserver'} : "add";
    if ( lc($ldapserver) eq 'add' ) {
        $Log->comment("Clicking on the Active Directory Domain radio button");
        $self->click($LDAP_ADD_ADDOMAIN_RADIO);

        if ( exists( $args{'-ldapaddomain'} ) ) {
            $Log->comment(
                "Entering LDAP Active Directory Domain: $args{'-ldapaddomain'}"
            );
            $self->type( $LDAP_ADD_ADDOMAIN_INPUT, $args{'-ldapaddomain'} );
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Active Directory Domain parameter is mandatory");
        }

        if ( exists( $args{'-ldapserveradd'} ) ) {
            $Log->comment(
                "Clicking on the Active Directory Domain Add server button");
            $self->click($LDAP_ADD_ADDOMAIN_ADD_BUTTON);

            $Log->comment(
                "Entering LDAP Active Directory Domain Server: $args{'-ldapserveradd'}"
            );
            $self->type( $LDAP_ADD_ADDOMAIN_SERVER_INPUT,
                $args{'-ldapserveradd'} );

            $Log->comment("Clicking on the Save button");
            $self->click($LDAP_ADD_ADDOMAIN_SERVERSAVE_BUTTON);
        } ## end if ( exists( $args{'-ldapserveradd'...}))

        if ( exists( $args{'-ldapserverdel'} ) ) {
            $Log->comment(
                "selecting Active Directory Domain server: $args{'-ldapserverdel'}"
            );
            $self->click_row_with_texts( $LDAP_ADD_ADDOMAIN_GRID,
                $args{'-ldapserverdel'} );

            $Log->comment("clicking delete button");
            $self->click($LDAP_ADD_ADDOMAIN_DELETE_BUTTON);
        } ## end if ( exists( $args{'-ldapserverdel'...}))
    } elsif ( lc($ldapserver) eq 'ads' ) {
        $Log->comment(
            "Clicking on the Active Directory Servers radio button");
        $self->click($LDAP_ADD_ADSERVERS_RADIO);
        
        if ( exists( $args{'-ldapserveradd'} ) ) {
            $Log->comment(
                "Clicking on the Active Directory Servers Add button");
            $self->click($LDAP_ADD_ADSERVERS_ADD_BUTTON);

            $Log->comment(
                "Entering LDAP Active Directory Server: $args{'-ldapserveradd'}"
            );
            $self->type( $LDAP_ADD_ADSERVERS_SERVER_INPUT,
                $args{'-ldapserveradd'} );

            $Log->comment("Clicking on the Save button");
            $self->click($LDAP_ADD_ADSERVERS_SERVERSAVE_BUTTON);
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Minimum 1 Active Directory Server is mandatory");
        }

        if ( exists( $args{'-ldapserverdel'} ) ) {
            $Log->comment(
                "selecting Active Directory server: $args{'-ldapserverdel'}");
            $self->click_row_with_texts( $LDAP_ADD_ADSERVERS_GRID,
                $args{'-ldapserverdel'} );

            $Log->comment("clicking delete button");
            $self->click($LDAP_ADD_ADSERVERS_DELETE_BUTTON);
        } ## end if ( exists( $args{'-ldapserverdel'...}))
    } ## end elsif ( lc($ldapserver) eq...)

    $Log->comment("Clicking on the Binding Tab");
    $self->click($LDAP_ADD_BINDING_TAB);
    $self->wait_for_loading();

    if ( exists( $args{'-ldapauthlevel'} ) ) {
        $Log->comment(
            "selecting Authentication Level: $args{'-ldapauthlevel'}");
        $self->set_combobox_value( $LDAP_ADD_AUTHENLEVEL_COMBOBOX,
            $args{'-ldapauthlevel'} );
    } ## end if ( exists( $args{'-ldapauthlevel'...}))

    if ( exists( $args{'-ldapbinddn'} ) ) {
        $Log->comment("Enter Bind DN: $args{'-ldapbinddn'}");
        $self->type( $LDAP_ADD_BINDDN_INPUT, $args{'-ldapbinddn'} );
    }

    if ( exists( $args{'-ldapbindupass'} ) ) {
        $Log->comment("Enter Bind user password: $args{'-ldapbindupass'}");
        $self->type( $LDAP_ADD_BINDPASS_INPUT, $args{'-ldapbindupass'} );
    }

    if ( exists( $args{'-ldapbasedn'} ) ) {
        $Log->comment("Enter Base DN: $args{'-ldapbasedn'}");
        $self->type( $LDAP_ADD_BASEDN_INPUT, $args{'-ldapbasedn'} );
    }

    if ( exists( $args{'-ldaptcpport'} ) ) {
        $Log->comment("Enter TCP Port: $args{'-ldaptcpport'}");
        $self->type( $LDAP_ADD_TCPPORT_INPUT, $args{'-ldaptcpport'} );
    }

    $Log->comment("Clicking on the Save and Close Button");
    $self->click($LDAP_ADD_SAVEANDCLOSE_BUTTON);
    $self->wait_for_loading();

    my %times = (
        loading_time => $loading_time,
    );

    $Log->comment("LDAP added successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub ldap_add

################################################################################
## @name ::ldap_edit
## @summary
##    Edit ldap details using GUI.
## @description
##     This method can be used to edit ldap details using GUI.
## @synopsis
##    ldap_edit(%args);
## INPUTS TO METHOD CALL 'ldap_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ldapclient </td> <td> (Required) Name of the ldap client to be added </td> </tr>
## <tr> <td> -ldapserver </td> <td> Name of the ldap server on which the client is to be added </td> </tr>
## <tr> <td> -ldapaddomain </td> <td> Entering LDAP Active Directory Domain </td> </tr>
## <tr> <td> -ldapserveradd </td> <td> selecting Active Directory Domain server </td> </tr>
## <tr> <td> -ldapserverdel </td> <td> Deleting the ldap server </td> </tr>
## <tr> <td> -ldapauthlevel </td> <td> Selecting ldap Authentication Level </td> </tr>
## <tr> <td> -ldapbinddn </td> <td> Enter ldap Bind DN </td> </tr>
## <tr> <td> -ldapbindupass </td> <td> Enter ldap Bind user password </td> </tr>
## <tr> <td> -ldapbasedn </td> <td> Enter ldap Bind DN </td> </tr>
## <tr> <td> -ldaptcpport </td> <td> Enter ldap tcp port </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub ldap_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ldapclient" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Configuration>Services>LDAP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName,
        $LDAP_TREE );
    $self->wait_for_loading();

    $Log->step("Select LDAP :$args{'-ldapclient'}");
    $self->click_row_with_texts( "$LDAP_GRID", $args{'-ldapclient'} );
    $self->wait_for_loading();

    $Log->comment("Clicking on the Edit button");
    $self->click($LDAP_EDIT_BUTTON);
    $self->wait_for_element_present( $LDAP_ADD_GENERAL_TAB, $GENERIC_WAIT );

    $Log->comment("Clicking on the General Tab");
    $self->click($LDAP_ADD_GENERAL_TAB);
    $self->wait_for_element_present( $LDAP_ADD_CLIENTCONFIG_INPUT,
        $GENERIC_WAIT );

    #$Log->comment("Entering LDAP client config: $args{'-ldapclient'}");
    #$self->type_keys($LDAP_ADD_CLIENTCONFIG_INPUT, $args{'-ldapclient'});
    #$self->type( $LDAP_ADD_CLIENTCONFIG_INPUT, $args{'-ldapclient'} );
    #$self->fire_event($LDAP_ADD_CLIENTCONFIG_INPUT, "blur");

    #setting the value for LDAP server
    my $ldapserver
        = exists( $args{'-ldapserver'} ) ? $args{'-ldapserver'} : "add";
    if ( lc($ldapserver) eq 'add' ) {
        $Log->comment("Clicking on the Active Directory Domain radio button");
        $self->click($LDAP_ADD_ADDOMAIN_RADIO);

        if ( exists( $args{'-ldapaddomain'} ) ) {
            $Log->comment(
                "Entering LDAP Active Directory Domain: $args{'-ldapaddomain'}"
            );
            $self->type( $LDAP_ADD_ADDOMAIN_INPUT, $args{'-ldapaddomain'} );
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Active Directory Domain parameter is mandatory");
        }

        if ( exists( $args{'-ldapserveradd'} ) ) {
            $Log->comment(
                "Clicking on the Active Directory Domain Add server button");
            $self->click($LDAP_ADD_ADDOMAIN_ADD_BUTTON);

            $Log->comment(
                "Entering LDAP Active Directory Domain Server: $args{'-ldapserveradd'}"
            );
            $self->type( $LDAP_ADD_ADDOMAIN_SERVER_INPUT,
                $args{'-ldapserveradd'} );

            $Log->comment("Clicking on the Save button");
            $self->click($LDAP_ADD_ADDOMAIN_SERVERSAVE_BUTTON);
        } ## end if ( exists( $args{'-ldapserveradd'...}))

        if ( exists( $args{'-ldapserverdel'} ) ) {
            $Log->comment(
                "selecting Active Directory Domain server: $args{'-ldapserverdel'}"
            );
            $self->click_row_with_texts( $LDAP_ADD_ADDOMAIN_GRID,
                $args{'-ldapserverdel'} );

            $Log->comment("clicking delete button");
            $self->click($LDAP_ADD_ADDOMAIN_DELETE_BUTTON);
        } ## end if ( exists( $args{'-ldapserverdel'...}))
    } elsif ( lc($ldapserver) eq 'ads' ) {
        $Log->comment(
            "Clicking on the Active Directory Servers radio button");
        $self->click($LDAP_ADD_ADSERVERS_RADIO);

        if ( exists( $args{'-ldapserveradd'} ) ) {
            $Log->comment(
                "Clicking on the Active Directory Servers Add button");
            $self->click($LDAP_ADD_ADSERVERS_ADD_BUTTON);

            $Log->comment(
                "Entering LDAP Active Directory Server: $args{'-ldapserveradd'}"
            );
            $self->type( $LDAP_ADD_ADSERVERS_SERVER_INPUT,
                $args{'-ldapserveradd'} );

            $Log->comment("Clicking on the Save button");
            $self->click($LDAP_ADD_ADSERVERS_SERVERSAVE_BUTTON);
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Minimum 1 Active Directory Server is mandatory");
        }

        if ( exists( $args{'-ldapserverdel'} ) ) {
            $Log->comment(
                "selecting Active Directory server: $args{'-ldapserverdel'}");
            $self->click_row_with_texts( $LDAP_ADD_ADSERVERS_GRID,
                $args{'-ldapserverdel'} );

            $Log->comment("clicking delete button");
            $self->click($LDAP_ADD_ADSERVERS_DELETE_BUTTON);
        } ## end if ( exists( $args{'-ldapserverdel'...}))
    } ## end elsif ( lc($ldapserver) eq...)

    $Log->comment("Clicking on the Binding Tab");
    $self->click($LDAP_ADD_BINDING_TAB);
    $self->wait_for_element_present( $LDAP_ADD_AUTHENLEVEL_COMBOBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-ldapauthlevel'} ) ) {
        $Log->comment(
            "selecting Authentication Level: $args{'-ldapauthlevel'}");
        $self->set_combobox_value( $LDAP_ADD_AUTHENLEVEL_COMBOBOX,
            $args{'-ldapauthlevel'} );
    } ## end if ( exists( $args{'-ldapauthlevel'...}))

    if ( exists( $args{'-ldapbinddn'} ) ) {
        $Log->comment("Enter Bind DN: $args{'-ldapbinddn'}");
        $self->type( $LDAP_ADD_BINDDN_INPUT, $args{'-ldapbinddn'} );
    }

    if ( exists( $args{'-ldapbindupass'} ) ) {
        $Log->comment("Enter Bind user password: $args{'-ldapbindupass'}");
        $self->type( $LDAP_ADD_BINDPASS_INPUT, $args{'-ldapbindupass'} );
    }

    if ( exists( $args{'-ldapbasedn'} ) ) {
        $Log->comment("Enter Base DN: $args{'-ldapbasedn'}");
        $self->type( $LDAP_ADD_BASEDN_INPUT, $args{'-ldapbasedn'} );
    }

    if ( exists( $args{'-ldaptcpport'} ) ) {
        $Log->comment("Enter TCP Port: $args{'-ldaptcpport'}");
        $self->type( $LDAP_ADD_TCPPORT_INPUT, $args{'-ldaptcpport'} );
    }

    $Log->comment("Clicking on the Save and Close Button");
    $self->click("$LDAP_ADD_SAVEANDCLOSE_BUTTON");
    $self->wait_for_element_present( $LDAP_EDIT_BUTTON, $GENERIC_WAIT );

    my %times = (
        loading_time => $loading_time,
    );

    $Log->comment("LDAP edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub ldap_edit

################################################################################
## @name ::ldap_delete
## @summary
##    Delete ldap details using GUI.
## @description
##     This method can be used to delete ldap details using GUI.
## @synopsis
##    ldap_delete(%args);
## INPUTS TO METHOD CALL 'ldap_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ldapclient </td> <td> (Required) Name of the ldap client to be added </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub ldap_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ldapclient" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Configuration>Services>LDAP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName,
        $LDAP_TREE );
    $self->wait_for_loading();

    $Log->step("Select LDAP :$args{'-ldapclient'}");
    $self->click_row_with_texts( "$LDAP_GRID", $args{'-ldapclient'} );
    $self->wait_for_loading();

    $Log->comment("Clicking on the Delete button");
    $self->click($LDAP_DELETE_BUTTON);
    $self->wait_for_element_present( $LDAP_DELETE_CONFIRMATION_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Clicking on checkbox on LDAP delete confirmation");
    $self->set_checkbox( "$LDAP_DELETE_CONFIRMATION_CHECKBOX", "enable" );

    $Log->comment("Clicking on delete button on LDAP delete confirmation");
    $self->click("$LDAP_DELETE_CONFIRMATIONDELETE_BUTTON");
    $self->wait_for_element_present( $LDAP_DELETE_BUTTON, $GENERIC_WAIT );

    my %times = (
        loading_time => $loading_time,
    );

    $Log->comment("LDAP deleted successfully");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub ldap_delete

################################################################################
## @name ::ldap_refresh
## @summary
##    Refresh LDAP page using GUI.
## @description
##     This method can be used to refresh LDAP page using GUI.
## @synopsis
##    ldap_refresh(%args);
## INPUTS TO METHOD CALL 'ldap_refresh' ON APISET OBJECT
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub ldap_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Vserver>Configuration>Services>LDAP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName,
        $LDAP_TREE );
    $self->wait_for_loading();

    $Log->comment("Clicking on the Refresh button");
    $self->click("$LDAP_REFRESH_BUTTON");
    $self->wait_for_element_present( $LDAP_REFRESH_BUTTON, $GENERIC_WAIT );

    my %times = (
        loading_time => $loading_time,
    );

    $Log->comment("LDAP refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub ldap_refresh

################################################################################
## @name ::vserver_ldap_edit
## @summary
##    Edit NIS domain properties using GUI.
## @description
##     This method can be used to edit NIS domain properties using GUI.
## @synopsis
##    vserver_ldap_edit(%args);
## INPUTS TO METHOD CALL 'vserver_ldap_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -ldapclientname </td> <td> (Required) Name of the ldap client to be edited </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the client is to be edited  </td> </tr>
## <tr> <td> -enableldapclient </td> <td> Selecting enable LDAP client  </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub vserver_ldap_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-ldapclentname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>LDAP");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $LDAP_TREE );

    $Log->comment("Clicking on the Edit button");
    $self->click($VSERVER_LDAP_EDIT_BUTTON);
#    $self->wait_for_element_present( $VSERVER_LDAP_CLIENT_COMBOBOX,
 #       $GENERIC_WAIT );
    $self->wait_for_loading();

    $Log->comment("selecting LDAP client Name: $args{'-ldapclentname'}");
    $self->set_combobox_value( $VSERVER_LDAP_CLIENT_COMBOBOX,
        $args{'-ldapclentname'} );
    $self->wait_for_loading();
    
    if ( exists( $args{'-enableldapclient'} ) ) {
        $Log->comment(
            "Selecting enable LDAP client : $args{'-enableldapclient'}");
        if ( lc( $args{'-enableldapclient'} eq "enable" ) ) {
            $self->set_checkbox( "$VSERVER_LDAP_CLIENT_ENABLE_CHECKBOX",
                "enable" );

        } elsif ( lc( $args{'-enableldapclient'} eq "disable" ) ) {
            $self->set_checkbox( "$VSERVER_LDAP_CLIENT_ENABLE_CHECKBOX",
                "disable" );
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Invalid value for LDAP client, valid values are 'enable' or 'disable'"
            );
        } ## end else [ if ( lc( $args{'-enableldapclient'...}))]
    } ## end if ( exists( $args{'-enableldapclient'...}))

    $Log->comment("Clicking on OK button");
    my $commit_time = $self->click_timecount($VSERVER_LDAP_EDIT_OK_BUTTON);
    $self->wait_for_loading();

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("LDAP edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_ldap_edit

################################################################################
# Subroutine Name: get_vserver_ldap_info
# Objective:       It will fetch NIS domain property values
# Details:         See POD documentation at the end of this file
################################################################################
# get_vserver_ldap_info(
# -vservername => "VS1",
# );

sub get_vserver_ldap_info {

    my ( $self, %args ) = @_;
    my %values;
    my $ldap_info = {};

    my ( $pack, $file, $line, $subname, $hasargs, $wantarray ) = caller(0);
    $Log->comment("****$subname()-Called at line#$line from $file START****");

    foreach my $k ( keys %args ) {
        print "Hash contains $k => $args{$k}\n";
    }

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }
    
    $Log->step("Navigating to Configuration>Services>LDAP");
    my $loading_time =
        $self->cluster_tree_navigate( $vserverName, $LDAP_TREE );
#    $self->wait_for_element_present( $VSERVER_LDAP_EDIT_BUTTON,
 #           $GlobalConst::GENERIC_WAIT );
        $self->wait_for_loading();

    $values{"$VSERVER_LDAP_CLIENTNAME_KEY"} = $self->get_text(
            "$VSERVER_LDAP_CLIENT_NAME")
        if (
                $self->is_element_present(
                    "$VSERVER_LDAP_CLIENT_NAME")
           );

    $values{"$VSERVER_LDAP_STATE_KEY"} = $self->get_text(
            "$VSERVER_LDAP_STATE_NAME")
        if (
                $self->is_element_present(
                    "$VSERVER_LDAP_STATE_NAME")
           );

    $Log->comment("Collection of snapmirror info is done");
    $ldap_info->{parsed_output} = \%values;

    $Log->comment("****$subname()-Called at line#$line from $file END****");

    return $ldap_info;

} ## end sub get_vserver_ldap_info

################################################################################
## @name ::dns_edit
## @summary
##    Edit DNS details properties using GUI.
## @description
##     This method can be used to edit DNS details using GUI.
## @synopsis
##    dns_edit(%args);
## INPUTS TO METHOD CALL 'dns_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -dnsservice </td> <td> (Required) Setting the DNS Service </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the DNS details are eited </td> </tr>
## <tr> <td> -adddomain </td> <td> Clicking on the Domain Add Button  </td> </tr>
## <tr> <td> -deldomain </td> <td> Clicking on the Domain Delete Button  </td> </tr>
## <tr> <td> -addnameservers </td> <td> Clicking on the Name Servers Add Button  </td> </tr>
## <tr> <td> -delnameservers </td> <td> Clicking on the Name Servers Delete Button  </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub dns_edit {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>DNS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DNS_TREE );
    $self->wait_for_loading();
    $Log->comment("Clicking on the Edit button");
    $self->click($DNS_EDIT_BUTTON);
    $self->wait_for_loading();
    #$self->wait_for_element_present( $DNS_EDIT_ENABLESERVICE_CHECKBOX,
    #    $GENERIC_WAIT );

    #Setting DNS Service Status
    my $dnsservice
        = exists( $args{'-dnsservice'} ) ? $args{'-dnsservice'} : "enable";

    $Log->comment("Setting DNS Service to $dnsservice");
    $self->set_checkbox( "$DNS_EDIT_ENABLESERVICE_CHECKBOX", $dnsservice );
    $self->wait_for_loading();

    if ( lc($dnsservice) eq 'enable' ) {
        if ( exists( $args{'-adddomain'} ) ) {
            $Log->comment("Clicking on the Domain Add Button");
            $self->click($DNS_EDIT_DNSDOMAINS_ADD_BUTTON);
            $self->wait_for_loading();

            $Log->comment("Entering DNS Domain: $args{'-adddomain'}");
            $self->type( $DNS_EDIT_DNSDOMAINS_INPUT, $args{'-adddomain'} );

            $Log->comment("Clicking on the Save Button");
            $self->click($DNS_EDIT_DNSDOMAINS_SAVE_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-adddomain'...}))

        if ( exists( $args{'-deldomain'} ) ) {
            $Log->comment("selecting DNS Domain: $args{'-deldomain'}");
            $self->click_row_with_texts( $DNS_EDIT_DNSDOMAINS_GRID,
                $args{'-deldomain'} );

            $Log->comment("clicking delete button");
            $self->click($DNS_EDIT_DNSDOMAINS_DELETE_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-deldomain'...}))

        if ( exists( $args{'-addnameservers'} ) ) {
            $Log->comment("Clicking on the Name Servers Add Button");
            $self->click($DNS_EDIT_NAMESERVERS_ADD_BUTTON);
            #$self->wait_for_element_present( $DNS_EDIT_NAMESERVERS_INPUT,
            #    $GENERIC_WAIT );
            $self->wait_for_loading();

            $Log->comment(
                "Entering Name Server Ip: $args{'-addnameservers'}");
            $self->type( $DNS_EDIT_NAMESERVERS_INPUT,
                $args{'-addnameservers'} );

            $Log->comment("Clicking on the Save Button");
            $self->click($DNS_EDIT_NAMESERVERS_SAVE_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-addnameservers'...}))

        if ( exists( $args{'-delnameservers'} ) ) {
            $Log->comment("selecting Name Server: $args{'-delnameservers'}");
            $self->click_row_with_texts( $DNS_EDIT_NAMESERVERS_GRID,
                $args{'-delnameservers'} );

            $Log->comment("clicking delete button");
            $self->click($DNS_EDIT_NAMESERVERS_DELETE_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{'-delnameservers'...}))
    } ## end if ( lc($dnsservice) eq...)

    $Log->comment("Clicking on OK button");
    my $commit_time = $self->click_timecount($DNS_EDIT_OK_BUTTON);
    $self->wait_for_loading();
#    $self->wait_for_element_present( $DNS_EDIT_BUTTON, $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("DNS edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub dns_edit

################################################################################
## @name ::dns_refresh
## @summary
##    Refresh DNS page using GUI.
## @description
##     This method can be used to refresh DNS page using GUI.
## @synopsis
##    dns_refresh(%args);
## INPUTS TO METHOD CALL 'dns_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the DNS details are eited </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub dns_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>DNS");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $DNS_TREE );

    $Log->comment("Clicking on the Refresh button");
    my $commit_time
        = $self->click_timecount( $DNS_REFRESH_BUTTON, "refresh" );
    $self->wait_for_element_present( $DNS_REFRESH_BUTTON, $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("DNS refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub dns_refresh

################################################################################
## @name ::kerberos_edit
## @summary
##    Edit Kerberos details using GUI.
## @description
##     This method can be used to edit Kerberos details using GUI.
## @synopsis
##    kerberos_edit(%args);
## INPUTS TO METHOD CALL 'kerberos_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -interfacename </td> <td> Setting the DNS Service </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the DNS details are eited </td> </tr>
## <tr> <td> -enablekerberos </td> <td> Setting Enable Kerberos checkbox  </td> </tr>
## <tr> <td> -kerberosrealm </td> <td> Selecting Kerberos Realm </td> </tr>
## <tr> <td> -servicename </td> <td> Enter Service Principle Name  </td> </tr>
## <tr> <td> -keytaburi </td> <td> Entering Keytab URI </td> </tr>
## <tr> <td> -adminuser </td> <td> Entering Admin User Name  </td> </tr>
## <tr> <td> -adminpass </td> <td> Entering Admin Password  </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub kerberos_edit {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $KERBEROS_NET_INT_TREE);
    $self->wait_for_loading();
    $Log->comment("Clicking on the Edit button");
    $self->click($KERBEROS_EDIT_BUTTON);
    $self->wait_for_loading();

    $self->wait_for_element_present( $KERBEROS_EDIT_INTERFACENAME_COMBOBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-interfacename'} ) ) {
        $Log->comment("selecting Interface Name: $args{'-interfacename'}");
        $self->set_combobox_value( $KERBEROS_EDIT_INTERFACENAME_COMBOBOX,
            $args{'-interfacename'} );
    }

    $self->check_for_error();

    #Setting Kerberos Enable Status
    my $enablekerberos
        = exists( $args{'-enablekerberos'} )
        ? $args{'-enablekerberos'}
        : "enable";

    $Log->comment("Setting Enable Kerberos checkbox to $enablekerberos");

    if ( $enablekerberos eq 'enable' ) {
        $self->click("$KERBEROS_EDIT_ENABLE_CHECKBOX");
    }

    if ( lc($enablekerberos) eq 'enable' ) {
        if ( exists( $args{'-kerberosrealm'} ) ) {
            $Log->comment(
                "selecting Kerberos Realm: $args{'-kerberosrealm'}");
            $self->set_combobox_value( $KERBEROS_EDIT_REALM_COMBOBOX,
                $args{'-kerberosrealm'} );
        } ## end if ( exists( $args{'-kerberosrealm'...}))

        if ( exists( $args{'-servicename'} ) ) {
            $Log->comment(
                "Enter Service Principle Name: $args{'-servicename'}");
            $self->type( $KERBEROS_EDIT_SERVICENAME_INPUT,
                $args{'-servicename'} );
        } ## end if ( exists( $args{'-servicename'...}))

        if ( exists( $args{'-keytaburi'} ) ) {
            $Log->comment("Entering Keytab URI: $args{'-keytaburi'}");
            $self->type( $KERBEROS_EDIT_KEYTABURI_INPUT,
                $args{'-keytaburi'} );
        }

        if ( exists( $args{'-adminuser'} ) ) {
            $Log->comment("Entering Admin User Name: $args{'-adminuser'}");
            $self->type( $KERBEROS_EDIT_ADMINUSER_INPUT,
                $args{'-adminuser'} );
        }

        if ( exists( $args{'-adminpass'} ) ) {
            $Log->comment("Entering Admin Password: $args{'-adminpass'}");
            $self->type( $KERBEROS_EDIT_ADMINPASS_INPUT,
                $args{'-adminpass'} );
        }
    } ## end if ( lc($enablekerberos...))

    $Log->comment("Clicking on OK button");
    my $commit_time = $self->click_timecount($KERBEROS_EDIT_OK_BUTTON);
    $self->wait_for_loading();
    #$self->wait_for_element_present( $KERBEROS_EDIT_BUTTON, $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Kerberos edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub kerberos_edit

################################################################################
## @name ::kerberos_refresh
## @summary
##    Refresh Kerberos page using GUI.
## @description
##     This method can be used to refresh Kerberos page using GUI.
## @synopsis
##    kerberos_refresh(%args);
## INPUTS TO METHOD CALL 'kerberos_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the DNS details are eited </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub kerberos_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to VServer>Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $KERBEROS_NET_INT_TREE);

    $Log->comment("Clicking on the Refresh button");
    my $commit_time
        = $self->click_timecount( $KERBEROS_REFRESH_BUTTON, "refresh" );
    $self->wait_for_element_present( $KERBEROS_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Kerberos refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub kerberos_refresh

################################################################################
## @name ::cluster_kerberos_create
## @summary
##    Create kerberos cluster using GUI.
## @description
##     This method can be used to create kerberos cluster using GUI.
## @synopsis
##    cluster_kerberos_create(%args);
## INPUTS TO METHOD CALL 'cluster_kerberos_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -configname </td> <td> (Required) Entering Configuration Name</td> </tr>
## <tr> <td> -realm </td> <td> (Required)Entering Realm </td> </tr>
## <tr> <td> -kdcipaddr </td> <td> (Required) Entering KDC IP Address </td> </tr>
## <tr> <td> -kdcport </td> <td> Entering KDC Port </td> </tr>
## <tr> <td> -kdcvendor </td> <td> Entering KDC vendor  </td> </tr>
## <tr> <td> -adminserverip </td> <td> Entering Admin Server IP </td> </tr>
## <tr> <td> -pwdserverip </td> <td> Entering Password Server IP </td> </tr>
## <tr> <td> -adservername </td> <td> Entering Active Directory Server  </td> </tr>
## <tr> <td> -adserverip </td> <td> Entering Active Directory Server IP  </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time and Finish time.

sub cluster_kerberos_create {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-configname" => { type => SCALAR },
            "-realm"      => { type => SCALAR },
            "-kdcipaddr"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to Cluster>Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $KERBEROS_REALM_TREE);

    $Log->comment("Clicking on the Create button");
    $self->click($CLUSTER_KERBEROS_CREATE_BUTTON);
    $self->wait_for_element_present(
        $CLUSTER_KERBEROS_CREATEWIZARD_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on the Next button on Create Wizard");
    $self->click($CLUSTER_KERBEROS_CREATEWIZARD_NEXT_BUTTON);
    $self->wait_for_element_present(
        $CLUSTER_KERBEROS_CREATE_CONFIGNAME_INPUT,
        $GENERIC_WAIT );

    $Log->comment("Entering Configuration Name: $args{'-configname'}");
    $self->type( $CLUSTER_KERBEROS_CREATE_CONFIGNAME_INPUT,
        $args{'-configname'} );

    $Log->comment("Entering Realm: $args{'-realm'}");
    $self->type( $CLUSTER_KERBEROS_CREATE_REALM_INPUT, $args{'-realm'} );

    $Log->comment("Entering KDC IP Address: $args{'-kdcipaddr'}");
    $self->type( $CLUSTER_KERBEROS_CREATE_KDCIPADDR_INPUT,
        $args{'-kdcipaddr'} );

    if ( exists( $args{'-kdcport'} ) ) {
        $Log->comment("Entering KDC Port: $args{'-kdcport'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_KDCPORT_INPUT,
            $args{'-kdcport'} );
    }

    #selecting KDC Vendor
    my $kdcvendor
        = exists( $args{'-kdcvendor'} ) ? $args{'-kdcvendor'} : "other";

    if ( lc($kdcvendor) eq 'other' ) {
        $Log->comment("Clicking on the other KDC Venfor radio button");
        $self->click($CLUSTER_KERBEROS_CREATE_KDCVENDOR_OTHER_RADIO);
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->comment("Clicking on the microsoft KDC Venfor radio button");
        $self->click($CLUSTER_KERBEROS_CREATE_KDCVENDOR_MICROSOFT_RADIO);
    }

    $Log->comment("Clicking on the Next button Basic page");
    $self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present(
        $CLUSTER_KERBEROS_CREATE_ADMINSERVERIP_INPUT,
        $GENERIC_WAIT );

    if ( exists( $args{'-adminserverip'} ) ) {
        $Log->comment("Entering Admin Server IP: $args{'-adminserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADMINSERVERIP_INPUT,
            $args{'-adminserverip'} );
    }

    if ( exists( $args{'-pwdserverip'} ) ) {
        $Log->comment("Entering Password Server IP: $args{'-pwdserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_PASSSERVERIP_INPUT,
            $args{'-pwdserverip'} );
    }

    if ( exists( $args{'-adservername'} ) ) {
        $Log->comment(
            "Entering Active Directory Server: $args{'-adservername'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADSERVERNAME_INPUT,
            $args{'-adservername'} );
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Active Directory Server is Mandatory");
    }

    if ( exists( $args{'-adserverip'} ) ) {
        $Log->comment(
            "Entering Active Directory Server IP: $args{'-adserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADSERVERIP_INPUT,
            $args{'-adserverip'} );
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Active Directory Server IP is Mandatory");
    }

    $Log->comment("Clicking on the Next button");
    $self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on the Next button on Summary page");
    # my $commit_time
        # = $self->click_timecount( $CLUSTER_KERBEROS_NEXT_BUTTON, "next",
        # $CLUSTER_KERBEROS_NEXT_BUTTON );
		
    $self->wait_for_element_present( $CLUSTER_KERBEROS_NEXT_BUTTON,
        $GENERIC_WAIT );
	$self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $Log->comment("Clicking on Finish button");
	$self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
	$self->wait_for_loading();
    # my $finish_time = $self->click_timecount($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_CREATE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        # commit_time  => $commit_time,
        # finish_time  => $finish_time
    );

    $Log->comment("Kerberos created successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cluster_kerberos_create

################################################################################
## @name ::cluster_kerberos_edit
## @summary
##    Edit Kerberos details using GUI.
## @description
##     This method can be used to Edit Kerberos details using GUI.
## @synopsis
##    cluster_kerberos_create(%args);
## INPUTS TO METHOD CALL 'cluster_kerberos_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -configname </td> <td> (Required) Entering Configuration Name</td> </tr>
## <tr> <td> -realm </td> <td> Entering Realm </td> </tr>
## <tr> <td> -kdcipaddr </td> <td> Entering KDC IP Address </td> </tr>
## <tr> <td> -kdcport </td> <td> Entering KDC Port </td> </tr>
## <tr> <td> -kdcvendor </td> <td> Entering KDC vendor  </td> </tr>
## <tr> <td> -adminserverip </td> <td> Entering Admin Server IP </td> </tr>
## <tr> <td> -pwdserverip </td> <td> Entering Password Server IP </td> </tr>
## <tr> <td> -adservername </td> <td> Entering Active Directory Server  </td> </tr>
## <tr> <td> -adserverip </td> <td> Entering Active Directory Server IP  </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time and Finish time.

sub cluster_kerberos_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-configname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to Cluster>Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $KERBEROS_REALM_TREE);

    $Log->step("Select Configuration Name :$args{'-configname'}");
    $self->click_row_with_texts( "$CLUSTER_KERBEROS_GRID",
        $args{'-configname'} );

    $Log->comment("Clicking on the Edit button");
    $self->click($CLUSTER_KERBEROS_EDIT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on the Next button on Create Wizard");
    $self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_CREATE_REALM_INPUT,
        $GENERIC_WAIT );

    if ( exists( $args{'-realm'} ) ) {
        $Log->comment("Entering Realm: $args{'-realm'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_REALM_INPUT, $args{'-realm'} );
    }

    if ( exists( $args{'-kdcipaddr'} ) ) {
        $Log->comment("Entering KDC IP Address: $args{'-kdcipaddr'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_KDCIPADDR_INPUT,
            $args{'-kdcipaddr'} );
    }

    if ( exists( $args{'-kdcport'} ) ) {
        $Log->comment("Entering KDC Port: $args{'-kdcport'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_KDCPORT_INPUT,
            $args{'-kdcport'} );
    }

    #selecting KDC Vendor
    my $kdcvendor
        = exists( $args{'-kdcvendor'} ) ? $args{'-kdcvendor'} : "other";

    if ( lc($kdcvendor) eq 'other' ) {
        $Log->comment("Clicking on the other KDC Venfor radio button");
        $self->click($CLUSTER_KERBEROS_CREATE_KDCVENDOR_OTHER_RADIO);
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->comment("Clicking on the microsoft KDC Venfor radio button");
        $self->click($CLUSTER_KERBEROS_CREATE_KDCVENDOR_MICROSOFT_RADIO);
    }

    $Log->comment("Clicking on the Next button Basic page");
    $self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present(
        $CLUSTER_KERBEROS_CREATE_ADMINSERVERIP_INPUT,
        $GENERIC_WAIT );

    if ( exists( $args{'-adminserverip'} ) ) {
        $Log->comment("Entering Admin Server IP: $args{'-adminserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADMINSERVERIP_INPUT,
            $args{'-adminserverip'} );
    }

    if ( exists( $args{'-pwdserverip'} ) ) {
        $Log->comment("Entering Password Server IP: $args{'-pwdserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_PASSSERVERIP_INPUT,
            $args{'-pwdserverip'} );
    }

    if ( exists( $args{'-adservername'} ) ) {
        $Log->comment(
            "Entering Active Directory Server: $args{'-adservername'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADSERVERNAME_INPUT,
            $args{'-adservername'} );
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Active Directory Server is Mandatory");
    }

    if ( exists( $args{'-adserverip'} ) ) {
        $Log->comment(
            "Entering Active Directory Server IP: $args{'-adserverip'}");
        $self->type( $CLUSTER_KERBEROS_CREATE_ADSERVERIP_INPUT,
            $args{'-adserverip'} );
    } elsif ( lc($kdcvendor) eq 'microsoft' ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("Active Directory Server IP is Mandatory");
    }

    $Log->comment("Clicking on the Next button");
    $self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on the Next button on Summary page");
    # my $commit_time
        # = $self->click_timecount( $CLUSTER_KERBEROS_NEXT_BUTTON, "next",
        # $CLUSTER_KERBEROS_NEXT_BUTTON );
	$self->click($CLUSTER_KERBEROS_NEXT_BUTTON);	
    $self->wait_for_element_present( $CLUSTER_KERBEROS_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking on Finish button");
	$self->click($CLUSTER_KERBEROS_NEXT_BUTTON);
	$self->wait_for_loading();
    #my $finish_time = $self->click_timecount($CLUSTER_KERBEROS_NEXT_BUTTON);
    $self->wait_for_element_present( $CLUSTER_KERBEROS_CREATE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        # commit_time  => $commit_time,
        # finish_time  => $finish_time
    );

    $Log->comment("Kerberos edited successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cluster_kerberos_edit

################################################################################
## @name ::cluster_kerberos_delete
## @summary
##    Edit Kerberos details using GUI.
## @description
##     This method can be used to Edit Kerberos details using GUI.
## @synopsis
##    cluster_kerberos_create(%args);
## INPUTS TO METHOD CALL 'cluster_kerberos_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -configname </td> <td> (Required) Entering Configuration Name</td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub cluster_kerberos_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-configname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to Cluster>Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $KERBEROS_REALM_TREE);

    $Log->step("Select Configuration Name :$args{'-configname'}");
    $self->click_row_with_texts( "$CLUSTER_KERBEROS_GRID",1,0,
        $args{'-configname'} );

    $Log->comment("Clicking on the Delete button");
    $self->click($CLUSTER_KERBEROS_DELETE_BUTTON);
    $self->wait_for_element_present(
        $CLUSTER_KERBEROS_DELETE_CONFIRMATION_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Clicking on checkbox on Kerberos delete confirmation");
    $self->set_checkbox( "$CLUSTER_KERBEROS_DELETE_CONFIRMATION_CHECKBOX",
        "enable" );

    $Log->comment(
        "Clicking on delete button on Kerberos delete confirmation");
    #my $commit_time = $self->click_timecount(
    #    $CLUSTER_KERBEROS_DELETE_CONFIRMATIONDELETE_BUTTON);
    $self->click(
        $CLUSTER_KERBEROS_DELETE_CONFIRMATIONDELETE_BUTTON);
    $self->wait_for_loading();
    $self->wait_for_element_present( $CLUSTER_KERBEROS_CREATE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        #commit_time  => $commit_time
    );

    $Log->comment("Kerberos deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cluster_kerberos_delete

################################################################################
## @name ::cluster_kerberos_refresh
## @summary
##    Refresh Kerberos page using GUI.
## @description
##     This method can be used to refresh Kerberos page using GUI.
## @synopsis
##    cluster_kerberos_refresh(%args);
## INPUTS TO METHOD CALL 'cluster_kerberos_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub cluster_kerberos_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->step("Navigating to Cluster>Configuration>Services>Kerberos");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $KERBEROS_REALM_TREE);

    $Log->comment("Clicking on the Refresh button");
    my $commit_time
        = $self->click_timecount( $CLUSTER_KERBEROS_REFRESH_BUTTON,
        "refresh" );
    $self->wait_for_element_present( $CLUSTER_KERBEROS_REFRESH_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Kerberos page refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub cluster_kerberos_refresh

################################################################################
## @name ::get_kerberos_info
## @summary
##    Fetch Kerberos details page using GUI.
## @description
##     This method can be used to fetch Kerberos information using GUI.
## @synopsis
##    get_kerberos_info(%args);
## INPUTS TO METHOD CALL 'get_kerberos_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> configname </td> <td> (Required) Configuration name for which details has to be fetched </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the Kerberos Details.

sub get_kerberos_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-configname" => { type => SCALAR }, },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my %values;
    my $kerberos_info = {};
    my $configname    = $args{'-configname'};

    $Log->step("Navigating to Cluster>Configuration>Services>Kerberos");
        $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
                $KERBEROS_TREE );

    $self->wait_for_element_present("$CLUSTER_KERBEROS_GRID");
    $Log->comment("Select kerberos with config name $configname");
    $self->click_row_with_texts( "$CLUSTER_KERBEROS_GRID", 1, 0,
            $args{'-configname'} );

    $Log->comment("Getting kerberos details");
    $values{"$KERBEROS_DETAIL_CONFIG_NAME_KEY"} =
        $self->get_text("$KERBEROS_INFO_CONFIG_NAME")
        if ( $self->is_element_present("$KERBEROS_INFO_CONFIG_NAME") );

    $values{"$KERBEROS_DETAIL_KDC_IP_ADDRESS_KEY"} =
        $self->get_text("$KERBEROS_INFO_KDC_IP_ADDR")
        if ( $self->is_element_present("$KERBEROS_INFO_KDC_IP_ADDR") );
    $values{"$KERBEROS_DETAIL_KDC_PORT_KEY"} =
        $self->get_text("$KERBEROS_INFO_KDC_PORT")
        if ( $self->is_element_present("$KERBEROS_INFO_KDC_PORT") );

    $values{"$KERBEROS_DETAIL_KDC_VENDOR_KEY"} =
        $self->get_text("$KERBEROS_INFO_KDC_VENDOR")
        if ( $self->is_element_present("$KERBEROS_INFO_KDC_VENDOR") );

    $values{"$KERBEROS_DETAIL_ACTIVE_DIR_SER_KEY"} =
        $self->get_text("$KERBEROS_INFO_AD_SERVER_NAME")
        if ( $self->is_element_present("$KERBEROS_INFO_AD_SERVER_NAME") );

    $values{"$KERBEROS_DETAIL_ACTIVE_DIR_SER_IP_KEY"} =
        $self->get_text("$KERBEROS_INFO_AD_SERVER_IP")
        if ( $self->is_element_present("$KERBEROS_INFO_AD_SERVER_IP") );

    $values{"$KERBEROS_DETAIL_ADMIN_SER_IP_KEY"} =
        $self->get_text("$KERBEROS_INFO_ADMIN_SERVER_IP")
        if ( $self->is_element_present("$KERBEROS_INFO_ADMIN_SERVER_IP") );

    $values{"$KERBEROS_DETAIL_PASSWD_SER_IP_KEY"} =
        $self->get_text("$KERBEROS_INFO_PASSWD_SERVER_IP")
        if ( $self->is_element_present("$KERBEROS_INFO_PASSWD_SERVER_IP") );

    $Log->comment("Collection of kerberos info is successfull");
    $kerberos_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $kerberos_info;
}

################################################################################
## @name ::get_ldap_info
## @summary
##    Fetch LDAP details page using GUI.
## @description
##     This method can be used to fetch LDAP information using GUI.
## @synopsis
##    get_ldap_info(%args);
## INPUTS TO METHOD CALL 'get_ldap_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> ldapclient </td> <td> (Required) LDAP Client name for which details has to be fetched </td> </tr>
## </table>
##
## @return
##    Returns an array reference which contains the LDAP Details.

sub get_ldap_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-ldapclient" => { type => SCALAR }, },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;

    $self->ldap_refresh();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my %values;
    my $ldap_info  = {};
    my $ldapclient = $args{'-ldapclient'};

    $Log->step("Select LDAP :$ldapclient");
    $self->click_row_with_texts( $LDAP_GRID, $ldapclient );

    $Log->step("Getting row data for ldapclient from grid");
    my $rowdata
        = $self->get_row_data_with_text( "$LDAP_GRID",
                $ldapclient );

    my @rowarray = split( ' ', $rowdata );
    %values = (
            "$LDAP_INFO_LDAP_CLIENT_CONFIG_KEY"                    => $rowarray[0],
            "$LDAP_INFO_ACTIVE_DIRECTORY_DOMAIN_KEY"               => $rowarray[1],
            "$LDAP_INFO_ACTIVE_DIRECTORY_SERVERS_KEY"              => $rowarray[2],
            "$LDAP_INFO_PREFERRED_ACTIVE_DIRECTORY_SERVERS_KEY"    => $rowarray[3],
            );

    $self->check_for_error();

    $Log->comment("Collection of ldapclient info is successfull");
    $ldap_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $ldap_info;
} ## end sub get_ldap_info

################################################################################
## @name ::get_dns_status_info
## @summary
##    Fetch DNS details page using GUI.
## @description
##     This method can be used to fetch DNS information using GUI.
## @synopsis
##    get_dns_status_info(%args);
## INPUTS TO METHOD CALL 'get_dns_status_info' ON APISET OBJECT
##
## @return
##    Returns an array reference which contains the DNS Details.

sub get_dns_status_info {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    $self->dns_refresh(%args);

    my %values;
    my $dns_info = {};

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Getting DNS details");
    $values{$DNS_INFO_DNS_SERVICE_KEY} =
        $self->get_text($DNS_INFO_DNS_SERVICE)
        if ( $self->is_element_present($DNS_INFO_DNS_SERVICE) );

    $Log->comment("Collection of dns status is successfull");
    $dns_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $dns_info;
} ## end sub get_dns_status_info

################################################################################
# Subroutine Name: get_nis_info
# Objective:       It will get the details of NIS domain
# Details:         See POD documentation at the end of this file
################################################################################
sub get_nis_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-domainname" => { type => SCALAR }, },
            allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $domainname = $args{-domainname};
    my %values;
    my $nis_info = {};
    
    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Configuration>Services>NIS");
    eval {
        $self->cluster_tree_navigate($vserverName, $NIS_TREE );
    };
    if ($@) {
        $self->cluster_tree_navigate($vserverName, $NIS_TREE_SPACE );
    }


    $Log->step("Selecting NIS :$args{'-domainname'}");
    $self->click_row_with_texts( "$NIS_GRID", 1, 0, $args{'-domainname'} );

    $Log->step("Getting row data for nisdomain from grid");
    my $rowdata
        = $self->get_row_data_with_text( "$NIS_GRID",
                $domainname );

    my @rowarray = split( ' ', $rowdata );
    %values = (
            "$NIS_INFO_DOMAIN_NAME_KEY"    => $rowarray[0],
            "$NIS_INFO_NIS_SERVERS_KEY"    => $rowarray[2],
            );

    $self->check_for_error();

    $Log->comment("Collection of nisdomain info is successfull");
    $nis_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    return $nis_info;

} ## end sub get_nis_info

1;

