# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Shares GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode Shares related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmShares;
##
##      share_create(%args);
##      share_edit(%args);
##      share_stop(%args);
##      clear_window_cifs_share(%args);
##      share_refresh(%args);
##      get_shares_info(%args);
##
## @status private
##
## @author
##		Afsar Syed. afsars@netapp.com
##
## @change
##	4/1/2010 : Afsar Syed : Initial version
## @change
##  3/20/2012 : Vikas P : POD modification
##

package NACL::APISet::Host::GUI::SysManager::CMode::SmShares;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    share_create
    share_create_home_dir
    share_edit
    share_stop
    clear_window_cifs_share
    share_refresh
    get_shares_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

##############################################################################
## @name ::share_create
## @summary
##     Creation of Share using GUI.
## @description
##     This method can be used to create share using GUI.
## @synopsis
##     share_create(%args);
## INPUTS TO METHOD CALL 'share_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sharename </td> <td> Name of the share to be created </td> </tr>
## <tr> <td> -path </td> <td>  Valid path in which share has to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Virtual server to be created </td> </tr>
## <tr> <td> -comment </td> <td> Adding comment to the share </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub share_create {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $SHARES_TREE );

    $self->wait_for_loading();

    $Log->step("Select create shares");
    $self->click("$SHARE_CREATE_BUTTON");

    $self->wait_for_element_present( "$SHARE_CREATE_NAME_INPUT",
        $GENERIC_WAIT );

    $Log->step("Type the valid path:$args{'-path'}");
    $self->type( "$SHARE_CREATE_FOLDER_INPUT", $args{'-path'} );

    $Log->step("Type the Share name: $args{'-sharename'}");
    $self->type( "$SHARE_CREATE_NAME_INPUT", $args{'-sharename'} );

    if ( exists( $args{-comment} ) ) {
        $Log->step("Type the Share name :$args{-comment} ");
        $self->type( "$SHARE_CREATE_COMMENT_INPUT", $args{-comment} );
    }

    sleep(2);
    $Log->step("Select create ok button");
    my $commit_time = $self->click_timecount($SHARE_CREATE_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("share created successfully");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub share_create

##############################################################################
## @name ::share_create_home_dir
## @summary
##     Creation of Share home directory using GUI.
## @description
##     This method can be used to create share home directory using GUI.
## @synopsis
##     share_create_home_dir(%args);
## INPUTS TO METHOD CALL 'share_create_home_dir' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the Virtual server to be created </td> </tr>
## <tr> <td> -comment </td> <td> Adding comment to the share </td> </tr>
## <tr> <td> -addhomedir </td> <td> Adding home directory </td> </tr>
## <tr> <td> -delhomedir </td> <td> Deleting home directory </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub share_create_home_dir {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $SHARES_TREE );

    $self->wait_for_loading();
    $Log->step("Select create home directory");
    $self->click("$SHARE_CREATE_HOME_DIR_BUTTON");
    $self->wait_for_loading();

    $self->wait_for_element_present( "$SHARE_CREATE_HOME_DIR_NAME_TEXTBOX",
        $GENERIC_WAIT );

    $Log->step("Type the home dir name:$args{'-dirname'}");
    $self->type( "$SHARE_CREATE_HOME_DIR_NAME_TEXTBOX", $args{'-dirname'} );

    $Log->step("Type the valid relative path:$args{'-relpath'}");
    $self->type( "$SHARE_CREATE_HOME_DIR_REL_PATH_TEXTBOX",
        $args{'-relpath'} );

    if ( exists( $args{-comment} ) ) {
        $Log->step("Type the comment :$args{-comment} ");
        $self->type( "$SHARE_CREATE_HOME_DIR_COMMENT_TEXTBOX",
            $args{-comment} );
    }

    if ( exists( $args{-addhomedir} ) ) {
        $Log->step("Clicking manage button");
        $self->click("$SHARE_CREATE_HOME_DIR_MANAGE_BUTTON");
        foreach my $val ( @{ $args{-addhomedir} } ) {
            $Log->step("Entering home dir : $val");
            $self->type( "$SHARE_CREATE_HOME_DIR_MANAGE_PATH_TEXTBOX", $val );
            $Log->step("Clicking Add button");
            $self->click("$SHARE_CREATE_HOME_DIR_MANAGE_ADD_BUTTON");
        } ## end foreach my $val ( @{ $args{...}})
    } ## end if ( exists( $args{-addhomedir...}))

    if ( exists( $args{-delhomedir} ) ) {
        $Log->step("Clicking manage button");
        $self->click("$SHARE_CREATE_HOME_DIR_MANAGE_BUTTON");
        foreach my $val ( @{ $args{-delhomedir} } ) {
            $Log->step("Selecting home dir");
            $self->click_row_with_texts(
                "$SHARE_CREATE_HOME_DIR_MANAGE_PATH_GRID", $val );
            $Log->step("Clicking delete button");
            $self->click("$SHARE_CREATE_HOME_DIR_MANAGE_DELETE_BUTTON");
        } ## end foreach my $val ( @{ $args{...}})
    } ## end if ( exists( $args{-delhomedir...}))

    if (   ( exists( $args{-addhomedir} ) )
        || ( exists( $args{-delhomedir} ) ) )
    {
        $Log->step("Clicking save button");
        $self->click("$SHARE_CREATE_HOME_DIR_MANAGE_SAVE_CLOSE_BUTTON");
    } ## end if ( ( exists( $args{-...})))

    $Log->step("Select create ok button");
    my $commit_time
        = $self->click_timecount($SHARE_CREATE_HOME_DIR_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("share create home directory was successful");

    $Log->exit() if $may_exit;

    return \%times;

} ## end sub share_create_home_dir

##############################################################################
## @name ::share_edit
## @summary
##     Editing of Share using GUI.
## @description
##     This method can be used to edit the share using GUI.
## @synopsis
##     share_edit(%args);
## INPUTS TO METHOD CALL 'share_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sharename </td> <td> Name of the share to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Virtual server to be created </td> </tr>
## <tr> <td> -comment </td> <td> Adding comment to the share </td> </tr>
## <tr> <td> -addgrpuser </td> <td> Adding group users who can access the share </td> </tr>
## <tr> <td> -removegrpuser </td> <td> Removing group users who can access the share </td> </tr>
## <tr> <td> -fullcontrol </td> <td> Enabling fullcontrol permission to edit the share </td> </tr>
## <tr> <td> -change </td> <td> Enabling change permission to edit the share </td> </tr>
## <tr> <td> -read </td> <td> Enabling read permission to edit the share </td> </tr>
## <tr> <td> -symlinks </td> <td> Selecting symlinks </td> </tr>
## <tr> <td> -enableoplocks </td> <td> Selecting enableoplocks </td> </tr>
## <tr> <td> -browsable </td> <td> Selecting browsable </td> </tr>
## <tr> <td> -snapshots </td> <td> Selecting snapshots </td> </tr>
## <tr> <td> -notify </td> <td> Selecting notify </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub share_edit {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $SHARES_TREE );
    
    $self->wait_for_loading();
    $Log->step("Select share from grid : $args{'-sharename'}");
    $self->click_row_with_texts( "$SHARE_GRID", $args{'-sharename'} );

    $Log->step("Select edit shares button");
    $self->click("$SHARE_EDIT_BUTTON");
    $self->wait_for_loading();

    sleep(5);
    if($self->is_element_present($SHARE_EDIT_ADMIN_POPUP_OK_BUTTON)) {
        $Log->step("Clicking on the pop-up OK button");
        $self->click($SHARE_EDIT_ADMIN_POPUP_OK_BUTTON);
        sleep(5);
    }
    
    if ( exists( $args{-comment} ) ) {
        $Log->step("Add comment on the share");
        $self->type_keys( "$SHARE_EDIT_COMMENT_INPUT", $args{-comment} );
    }

    if ( exists( $args{-addgrpuser} ) ) {
        $Log->step("Enter permission tab in edit shares");
        $self->click("$SHARE_EDIT_PERMISSION_TAB");
        foreach my $val ( @{ $args{-addgrpuser} } ) {
            $Log->step("Clicking Add button");
            $self->click("$SHARE_EDIT_GROUP_USERS_ADD");
            $Log->step("Entering user/group name : $val");
            $self->type( "$SHARE_EDIT_GROUP_USERS_ADD_TEXTBOX", $val );
            $Log->step("Clicking OK button");
            $self->click("$SHARE_EDIT_GROUP_USERS_OK_BUTTON");
        } ## end foreach my $val ( @{ $args{...}})
    } ## end if ( exists( $args{-addgrpuser...}))

    if ( exists( $args{-removegrpuser} ) ) {
        $Log->step("Enter permission tab in edit shares");
        $self->click("$SHARE_EDIT_PERMISSION_TAB");
        foreach my $val ( @{ $args{-removegrpuser} } ) {
            $Log->step("Selecting group/user name");
            $self->click_row_with_texts( "$SHARE_EDIT_GROUP_USER_GRID",
                $val );
            $Log->step("Clicking remove button");
            $self->click("$SHARE_EDIT_GROUP_USERS_REMOVE");
        } ## end foreach my $val ( @{ $args{...}})
    } ## end if ( exists( $args{-removegrpuser...}))

    if ( exists( $args{-fullcontrol} ) ) {
        $Log->step("Enter permission tab in edit shares");
        $self->click("$SHARE_EDIT_PERMISSION_TAB");
        $Log->step("Selecting full control");

        if ((   not $self->is_element_present(
                    $SHARE_EDIT_PERMISSION_FULL_CONTROL)
            )
            && ( lc( $args{-fullcontrol} ) ) eq "disable"
            )
        {
            $self->mouse_down("$SHARE_EDIT_PERMISSION_FULL_CONTROL");
        } ## end if ( ( not $self->is_element_present...))
    } ## end if ( exists( $args{-fullcontrol...}))

    if ( exists( $args{-change} ) ) {
        $Log->step("Enter permission tab in edit shares");
        $self->click("$SHARE_EDIT_PERMISSION_TAB");
        $Log->step("Selecting full control");
        if ( ( not $self->is_element_present($SHARE_EDIT_PERMISSION_CHANGE) )
            && ( lc( $args{-change} ) ) eq "disable" )
        {
            $self->mouse_down("$SHARE_EDIT_PERMISSION_CHANGE");
        }
    } ## end if ( exists( $args{-change...}))

    if ( exists( $args{-read} ) ) {
        $Log->step("Enter permission tab in edit shares");
        $self->click("$SHARE_EDIT_PERMISSION_TAB");
        $Log->step("Selecting full control");
        if (   ( not $self->is_element_present($SHARE_EDIT_PERMISSION_READ) )
            && ( lc( $args{-change} ) ) eq "disable" )
        {
            $self->mouse_down("$SHARE_EDIT_PERMISSION_READ");
        }
    } ## end if ( exists( $args{-read...}))

    if ( exists( $args{-symlinks} ) ) {
        $Log->step("Clicking options tab in edit shares");
        $self->click("$SHARE_EDIT_OPTIONS_TAB");

        if ( lc( $args{-symlinks} ) eq "hide" ) {
            $Log->step("Clicking hide for symlinks");
            $self->click("$SHARE_EDIT_OPTIONS_SYMLINKS_HIDE");
        }

        elsif ( lc( $args{-symlinks} ) eq "readonly" ) {
            $Log->step("Clicking readonly for symlinks");
            $self->click("$SHARE_EDIT_OPTIONS_SYMLINKS_READONLY");
        }

        elsif ( lc( $args{-symlinks} ) eq "readwrite" ) {
            $Log->step("Clicking readwrite for symlinks");
            $self->click("$SHARE_EDIT_OPTIONS_SYMLINKS_READWRITE");
        }
    } ## end if ( exists( $args{-symlinks...}))

    if ( exists( $args{-enableoplocks} ) ) {
        $Log->step("Clicking $args{-enableoplocks}");
        $self->set_checkbox( "$SHARE_EDIT_OPTIONS_ENABLE_OPLOCKS",
            $args{-enableoplocks} );
    }

    if ( exists( $args{-browsable} ) ) {
        $Log->step("Clicking $args{-browsable}");
        $self->set_checkbox( "$SHARE_EDIT_OPTIONS_BROWSABLE",
            $args{-browsable} );
    }

    if ( exists( $args{-snapshots} ) ) {
        $Log->step("Clicking $args{-snapshots}");
        $self->set_checkbox( "$SHARE_EDIT_OPTIONS_SNAPSHOTS",
            $args{-snapshots} );
    }

    if ( exists( $args{-notify} ) ) {
        $Log->step("Clicking $args{-notify}");
        $self->set_checkbox( "$SHARE_EDIT_OPTIONS_NOTIFY", $args{-notify} );
    }

    $Log->step("click ok for the share edit");
    my $commit_time = $self->click_timecount($SHARE_EDIT_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("share edited successfully");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub share_edit

##############################################################################
## @name ::share_stop
## @summary
##     To stop the share using GUI.
## @description
##     This method can be used to stop the share using GUI.
## @synopsis
##     share_stop(%args);
## INPUTS TO METHOD CALL 'share_stop' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sharename </td> <td> (Required)Name of the share to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server to be created </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub share_stop {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-sharename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $SHARES_TREE );

    $Log->step("Select share from grid : $args{'-sharename'}");
    $self->click_row_with_texts( "$SHARE_GRID", $args{'-sharename'} );

    $Log->step("Select share Stop button");
    $self->click("$SHARE_STOP_BUTTON");

    $Log->step("Select share Stop conformation checkbox");
    $self->click("$SHARE_STOP_CONFORM_CHECKBOX");

    $Log->step("Select share Stop OK button");
    my $commit_time = $self->click_timecount($SHARE_STOP_CONFORM_OK_BUTTON);

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("share stopped successfully");

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub share_stop

##############################################################################
## @name ::share_refresh
## @summary
##     To refresh share using GUI.
## @description
##     This method can be used to refresh share using GUI.
## @synopsis
##     share_refresh(%args);
## INPUTS TO METHOD CALL 'share_refresh' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server to be created </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub share_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $SHARES_TREE );

    my $commit_time
        = $self->click_timecount( $SHARE_REFRESH_BUTTON, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub share_refresh

###############################################################################
## @name ::clear_window_cifs_shares
## @summary
##     To check and close any left over dialog box due to previous testcase failure using GUI.
## @description
##     This method can be used to check and close any left over dialog box due to previous testcase failure using GUI.
## @synopsis
##     clear_window_cifs_shares(%args);
## INPUTS TO METHOD CALL 'clear_window_cifs_shares' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the virtual server to be created </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub clear_window_cifs_shares {

    my ( $self, %args ) = @_;
    my $count = 0;

    #Check if any kind of dialog box is present

    $Log->enter() if $may_enter;
    $self->clear_window();

    while (( $self->is_element_present("$SHARE_CREATE_CANCEL_BUTTON") )
        || ( $self->is_element_present("$SHARE_EDIT_CANCEL_BUTTON") ) )
    {

        #Check if it has a close cross mark on titlebar.
        $Log->comment(
            "Closing dialog box left over by previously failed testcase.");
        if ( $self->is_element_present("$SHARE_CREATE_CANCEL_BUTTON") ) {
            $self->click("$SHARE_CREATE_CANCEL_BUTTON");
        } elsif ( $self->is_element_present("$SHARE_EDIT_CANCEL_BUTTON") ) {
            $self->click("$SHARE_EDIT_CANCEL_BUTTON");
        } else {
            $self->nlog("May be no left over Wizards/Dialogues present");
        }
        last if ( $count++ == 30 );
    } ## end while ( ( $self->is_element_present...))

    #Click the Refresh Button
    if ( $self->is_element_present("$SHARE_REFRESH_BUTTON") ) {
        $self->click("$SHARE_REFRESH_BUTTON");
    } else {
        $self->nlog("Could not find the Refresh Button");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub clear_window_cifs_shares

##############################################################################
## @name ::get_shares_info
## @summary
##     Fetch Share details using GUI.
## @description
##     This method can be used to fetch share information using GUI.
## @synopsis
##     get_shares_info(%args);
## INPUTS TO METHOD CALL 'get_shares_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -sharename </td> <td> (Required) Name of the share for which details has to be fetched </td> </tr>
## </table>
##   @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub get_shares_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-sharename" => { type => SCALAR }, },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $sharename = $args{-sharename};
    my $username = $args{-username};
    my %values;
    my %values1;
    my $shares_info = {};

    my $vserverName=$self->{$SYSTEM_VSERVER_NAME};
    if(exists($args{'-vservername'})){
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName=$args{'-vservername'} ;
    }

    $Log->step("Navigating to $SHARES_TREE");
    my $loading_time = $self->cluster_tree_navigate($vserverName,$SHARES_TREE);

    $Log->step("Select share from grid : $sharename");
    $self->click_row_with_texts("$SHARE_GRID",$sharename);

    $Log->step("Getting row data for shares from grid");
    my $rowdata
        = $self->get_row_data_with_text("$SHARE_GRID", $sharename);

    my @rowarray = split(' ', $rowdata);

    %values = (
            "$SHARE_INFO_SHARE_NAME_KEY"       => $rowarray[0],
            "$SHARE_INFO_PATH_KEY"             => $rowarray[1],
            "$SHARE_INFO_HOME_DIRECTORY_KEY"   => $rowarray[2],
            "$SHARE_INFO_COMMENT_KEY"          => $rowarray[3],
            );

    if ( exists( $args{'-username'} ) ) {
        $Log->step("Select user from grid : $username");
        $self->click_row_with_texts("$SHARE_ACCESS_CONTROL_GRID",$username);

        $Log->step("Getting row data for user from grid");
        my $rowdata1
            = $self->get_row_data_with_text("$SHARE_ACCESS_CONTROL_GRID", $username);

        my @rowarray1 = split( ' ', $rowdata1 );

        %values1 = (
                "$SHARE_INFO_USERNAME_KEY"       => $rowarray1[0],
                "$SHARE_INFO_PERMISSION_KEY"     => $rowarray1[1],
                );
    }

    %values = (%values,%values1);

    $values{"$SHARE_INFO_OPLOCKS_STATUS_KEY"}
    = $self->get_text("$SHARE_INFO_OPLOCKS_STATUS")
        if (
                $self->is_element_present("$SHARE_INFO_OPLOCKS_STATUS")
           );

    $values{"$SHARE_INFO_BROWSABLE_KEY"}
    = $self->get_text("$SHARE_INFO_BROWSABLE")
        if (
                $self->is_element_present("$SHARE_INFO_BROWSABLE")
           );

    $values{"$SHARE_INFO_SHOW_SNAPSHOT_KEY"}
    = $self->get_text("$SHARE_INFO_SHOW_SNAPSHOT")
        if (
                $self->is_element_present("$SHARE_INFO_SHOW_SNAPSHOT")
           );

    $Log->comment("Collection of share info is successfull");
    $shares_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;
    return $shares_info;

} ## end sub get_shares_info

1;
