# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode SnapshotPolicies Functionalities
##
## @description
##      Using this library, user can perform Cmode SnapshotPolicies related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmSnapshotPolicies;
##      snapshot_policy_add(%args);
##      snapshot_policy_edit(%args);
##      snapshot_policy_delete(%args);
##      snapshot_policy_status(%args);
##      snapshot_policy_refresh(%args);
##      get_snapshotpolicy_info(%args);
##
## @status public
##
## @author afsars@netapp.com
## @change
##  2011-06-07 : afsars : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmSnapshotPolicies;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
  snapshot_policy_add
  snapshot_policy_edit
  snapshot_policy_delete
  snapshot_policy_status
  snapshot_policy_refresh
  get_snapshotpolicy_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::snapshot_policy_add
## @summary
##     Creation of snapshot policy using GUI.
## @description
##     This method can be used to create snapshot policy using GUI.
## @synopsis
##     snapshot_policy_add(%args);
## INPUTS TO METHOD CALL 'snapshot_policy_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -maxsnapshot </td> <td> Selecting maxsnapshot </td> </tr>
## <tr> <td> -scheduleedit </td> <td> Selecting scheduleedit </td> </tr>
## <tr> <td> -schedulemodify </td> <td> Selecting schedulemodify </td> </tr>
## <tr> <td> -maxsnapshotedit </td> <td> Selecting maxsnapshotedit </td> </tr>
## <tr> <td> -scheduledel </td> <td> Selecting scheduledel </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time.

sub snapshot_policy_add {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $Log->comment("Clicking snapshot policy add button");
    $self->click($SNAPSHOT_POLICY_ADD_BUTTON);

    $Log->comment("Waiting for snapshot policy add Dialog");
    $self->wait_for_loading();

    $Log->comment("Entering policy name: $args{'-policyName'}");
    $self->type( $SNAPSHOT_POLICY_NAME_TEXTBOX, $args{'-policyName'} );

    if ( exists( $args{-schedule} ) ) {
        $Log->comment("clicking add button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_ADD);

        $Log->comment("selecting schedule from combo: $args{-schedule}");
        $self->click("$SNAPSHOT_POLICY_SCHEDULE_ADD_NAME");
        $self->wait_for_loading();
        eval {
            $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item')]/span[text()='$args{-schedule}']"
            );
        };
        if ($@) {
            NATE::BaseException->throw(
                "Value: \"$args{-schedule}\" not present in combobox list");
        }

        if ( exists( $args{-maxsnapshot} ) ) {
            $Log->comment(
                "Entering the maximum snapshot : $args{-maxsnapshot}");
            $self->type( $SNAPSHOT_POLICY_ADD_MAXSNAPSHOT,
                $args{-maxsnapshot} );
            $Log->comment("Selecting schedule OK button to complete.");
            $self->click($SNAPSHOT_POLICY_SCHEDULE_OK_BUTTON);
        }
    }

    if ( exists( $args{-scheduleedit} ) ) {
        $Log->comment("selecting schedule");
        $self->click_row_with_texts( $SNAPSHOT_SCHEDULE_GRID,
            $args{-scheduleedit} );

        $Log->comment("clicking edit button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_EDIT);

        if ( exists( $args{-schedulemodify} ) ) {
            $Log->comment("select schedule : $args{-schedulemodify}");
            $self->set_combobox_value( $SNAPSHOT_POLICY_SCHEDULE_ADD_NAME,
                $args{-schedulemodify} );
        }

        if ( exists( $args{-maxsnapshotedit} ) ) {
            $Log->comment("select schedule : $args{-maxsnapshotedit}");
            $self->type( $SNAPSHOT_POLICY_ADD_MAXSNAPSHOT, "" );
        } ## end if ( exists( $args{-maxsnapshotedit...}))

        $Log->comment("Selecting OK button to complete.");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_OK_BUTTON);
    } ## end if ( exists( $args{-scheduleedit...}))

    if ( exists( $args{-scheduledel} ) ) {
        $Log->comment("selecting schedule");
        $self->click_row_with_texts( $SNAPSHOT_SCHEDULE_GRID,
            $args{-scheduledel} );

        $Log->comment("clicking delete button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_DELETE);
    } ## end if ( exists( $args{-scheduledel...}))

    $Log->comment("Selecting OK button to complete.");

    my $commit_time = $self->click_timecount($SNAPSHOT_POLICY_ADD_OK_BUTTON);
    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("snapshot policy created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapshot_policy_add

################################################################################
## @name ::snapshot_policy_edit
## @summary
##     Editing snapshot policy using GUI.
## @description
##     This method can be used to edit snapshot policy using GUI.
## @synopsis
##     snapshot_policy_edit(%args);
## INPUTS TO METHOD CALL 'snapshot_policy_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -schedule </td> <td> Selecting schedule </td> </tr>
## <tr> <td> -maxsnapshot </td> <td> Selecting maxsnapshot </td> </tr>
## <tr> <td> -scheduleedit </td> <td> Selecting scheduleedit </td> </tr>
## <tr> <td> -schedulemodify </td> <td> Selecting schedulemodify </td> </tr>
## <tr> <td> -maxsnapshotedit </td> <td> Selecting maxsnapshotedit </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub snapshot_policy_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $SNAPSHOT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking snapshot policy edit button");
    $self->click($SNAPSHOT_POLICY_EDIT_BUTTON);

    $Log->comment("Waiting for snapshot policy edit Dialog");
    $self->wait_for_element_present( $SNAPSHOT_SCHEDULE_GRID, $GENERIC_WAIT );

    if ( exists( $args{-schedule} ) ) {
        $Log->comment("clicking add schedule button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_ADD);
        $self->wait_for_loading();

        $Log->comment("select schedule : $args{'-schedule'}");
        #$self->select_from_dropdown( $SNAPSHOT_POLICY_SCHEDULE_ADD_NAME,
        #    $args{-schedule}, 1 );

        $self->click("$SNAPSHOT_POLICY_SCHEDULE_ADD_NAME");
        $self->wait_for_loading();
        eval {
            $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item')]/span[text()='$args{-schedule}']"
            );
        };
        if ($@) {
            NATE::BaseException->throw(
                "Value: \"$args{-schedule}\" not present in combobox list");
        }

        if ( exists( $args{-maxsnapshot} ) ) {
            $Log->comment("clicking add button");
            $Log->comment("input Max snapshot count : $args{-maxsnapshot}");
            $self->type( $SNAPSHOT_POLICY_ADD_MAXSNAPSHOT,
                $args{-maxsnapshot} );
            $Log->comment("Selecting OK button to complete.");
            $self->click($SNAPSHOT_POLICY_SCHEDULE_OK_BUTTON);
        } ## end if ( exists( $args{-maxsnapshot...}))
    } ## end if ( exists( $args{-schedule...}))

    if ( exists( $args{-scheduleedit} ) ) {
        $Log->comment("selecting schedule");
        $self->click_row_with_texts( $SNAPSHOT_SCHEDULE_GRID,
            $args{'-scheduleedit'} );

        $Log->comment("clicking edit button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_EDIT);
        $self->wait_for_loading();

        if ( exists( $args{-schedulemodify} ) ) {
            $Log->comment("select schedule : $args{'-schedulemodify'}");
            $self->set_combobox_value( $SNAPSHOT_POLICY_SCHEDULE_ADD_NAME,
                $args{-schedulemodify} );
        }

        if ( exists( $args{-maxsnapshotedit} ) ) {
            $Log->comment(
                "Change Max snapshot count : $args{-maxsnapshotedit}");
            $self->type( $SNAPSHOT_POLICY_ADD_MAXSNAPSHOT,
                $args{-maxsnapshotedit} );
        } ## end if ( exists( $args{-maxsnapshotedit...}))

        $Log->comment("Selecting OK button to complete.");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_OK_BUTTON);
        $self->wait_for_loading();

    } ## end if ( exists( $args{-scheduleedit...}))

    if ( exists( $args{-schedule} ) ) {
        $Log->comment("selecting schedule");
        $self->click_row_with_texts( $SNAPSHOT_SCHEDULE_GRID,
            $args{-schedule} );

        $Log->comment("clicking delete button");
        $self->click($SNAPSHOT_POLICY_SCHEDULE_DELETE);
        $self->wait_for_loading();
    } ## end if ( exists( $args{-schedule...}))

    $Log->comment("Selecting OK button to complete.");
    my $commit_time = $self->click_timecount($SNAPSHOT_POLICY_EDIT_OK_BUTTON);

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("policy edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapshot_policy_edit

################################################################################
## @name ::snapshot_policy_delete
## @summary
##     Deleting snapshot policy using GUI.
## @description
##     This method can be used to delete snapshot policy using GUI.
## @synopsis
##     snapshot_policy_delete(%args);
## INPUTS TO METHOD CALL 'snapshot_policy_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -checkbox </td> <td> Selecting checkbox </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub snapshot_policy_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $SNAPSHOT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking snapshot policy delete button");
    $self->click($SNAPSHOT_POLICY_DELETE_BUTTON);

    $Log->comment("Waiting for snapshot policy delete Dialog");
    $self->wait_for_element_present( $SNAPSHOT_POLICY_DELETE_CONFORM_CHECKBOX,
        $GENERIC_WAIT );

    #Selecting confirm checkbox
    if ( exists( $args{-checkbox} ) ) {
        $Log->comment("selecting confirm checkbox");
        $self->click($SNAPSHOT_POLICY_DELETE_CONFORM_CHECKBOX);
    }

    $Log->comment("Selecting OK button to complete.");

    my $commit_time = $self->click_timecount($SNAPSHOT_POLICY_DELETE_OK_BUTTON);

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (

        #loading_time => $loading_time,
        commit_time => $commit_time
    );

    $Log->comment("policy deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapshot_policy_delete

################################################################################
## @name ::snapshot_policy_status
## @summary
##    Get status of snapshot policy using GUI.
## @description
##     This method can be used to get status of snapshot policy using GUI.
## @synopsis
##     snapshot_policy_status(%args);
## INPUTS TO METHOD CALL 'snapshot_policy_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyname </td> <td> (Required)Name of the policy to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## <tr> <td> -status </td> <td> Selecting status </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub snapshot_policy_status {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    my ($commit_time);
    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $SNAPSHOT_POLICY_GRID, $args{'-policyName'} );

    $Log->comment("Clicking snapshot status button");
    $self->click($SNAPSHOT_POLICY_STATUS_BUTTON);

    if ( lc( $args{-status} ) eq "enable" ) {
        if ( $self->is_button_enabled("$SNAPSHOT_POLICY_ENABLE_BUTTON") ) {
            $Log->comment("Click enable Status Button");

            $commit_time =
              $self->click_timecount($SNAPSHOT_POLICY_ENABLE_BUTTON);
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("enable button is not enabled");
        }
    } ## end if ( lc( $args{-status...}))

    elsif ( lc( $args{-status} ) eq "disable" ) {
        if ( $self->is_button_enabled("$SNAPSHOT_POLICY_DISABLE_BUTTON") ) {
            $Log->comment("Click disable Status Button");
            $self->click($SNAPSHOT_POLICY_DISABLE_BUTTON);
            $self->click($SNAPSHOT_POLICY_DISABLE_CHECKBOX);

            $commit_time =
              $self->click_timecount($SNAPSHOT_POLICY_DISABLE_OK_BUTTON);

        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("disable button is not enabled");
        }

    } ## end elsif ( lc( $args{-status...}))

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("policy status changed successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapshot_policy_status

################################################################################
## @name ::snapshot_policy_refresh
## @summary
##    Refresh snapshot policy page using GUI.
## @description
##     This method can be used to get status of snapshot policy using GUI.
## @synopsis
##     snapshot_policy_status(%args);
## INPUTS TO METHOD CALL 'snapshot_policy_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver to be created </td> </tr>
## </table>
## @return
##    Returns an array reference which contains the Loading time and Commit Time

sub snapshot_policy_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $self->wait_for_element_present($SNAPSHOT_POLICY_GRID);

    $Log->step("Click Refresh button");
    my $commit_time =
      $self->click_timecount( $SNAPSHOT_POLICY_REFRESH_BUTTON, "refresh" );

    $Log->comment(
"Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->exit() if $may_exit;

    return \%times;
} ## end sub snapshot_policy_refresh

################################################################################
## @name ::get_snapshotpolicy_info
## @summary
##    Get information regarding snapshot policy using GUI.
## @description
##     This method can be used to get information regarding snapshot policies using GUI.
## @synopsis
##     get_snapshotpolicy_info(%args);
## INPUTS TO METHOD CALL 'lun_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -policyName </td> <td> (Required)Name of the lun to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the snapshot policy status details.
sub get_snapshotpolicy_info {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-policyName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    my $snapshotpolicy_info = {};
    my $vserverName;

    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $SNAPSHOT_POLICIES_TREE");
    my $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNAPSHOT_POLICIES_TREE );

    $Log->step("Selecting policy: $args{'-policyName'}");
    $self->click_row_with_texts( $SNAPSHOT_POLICY_GRID, $args{'-policyName'} );

    $Log->step("Getting row data for snapshot policy from grid");
    my $rowdata =
      $self->get_row_data_with_text( $SNAPSHOT_POLICY_GRID,
        $args{'-policyName'} );

    my @rowarray = split( ' ', $rowdata );
    my %values = (
        "$SNAPSHOT_POLICY_INFO_POLICY_KEY"             => $rowarray[0],
        "$SNAPSHOT_POLICY_INFO_STATUS_KEY"             => $rowarray[1],
        "$SNAPSHOT_POLICY_INFO_MAX_SNAPSHOTCOPIES_KEY" => $rowarray[2],
    );

    $Log->comment("Collection of Export Policy is done");
    $snapshotpolicy_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $snapshotpolicy_info;
} ## end sub get_snapshotpolicy_info

1;
