# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode SystemTools GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode SystemTools related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmSystemTools;
##
##      license_add(%args)
##      license_delete(%args)
##      license_refresh(%args)
##      datetime_edit(%args)
##      datetime_refresh(%args)
##      snmp_edit(%args)
##      snmp_refresh(%args)
##      event_configure(%args)
##      get_snmp_info(%args)
##      get_license_info(%args)
##      get_event_configure_info(%args)
##	job_page(%args)
##
## @author
##        Vinu.Nagarajan@netapp.com
##
## @change
##    3/29/2011 : vinu : Initial version
## @change
##  3/20/2012 : Vikas : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmSystemTools;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    license_add
    license_delete
    license_refresh
    datetime_edit
    datetime_refresh
    snmp_edit
    snmp_refresh
    event_configure
    get_snmp_info
    get_license_info
    get_event_configure_info
    job_page
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::license_add
## @summary
##     Add all the licenses based on the license keys using GUI.
## @description
##     This method can be used to add all the licenses based on the license keys using GUI.
## @synopsis
##     license_add(%args);
## INPUTS TO METHOD CALL 'license_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -licenses </td> <td> (Required)Licenses to be added </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub license_add {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-licenses" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my $commit_time;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>License");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $LICENSES_TREE );
    $self->wait_for_loading();

        my $license = $args{-licenses};
        
        if ( $self->is_element_present($LICENSE_ADD_SN_BUTTON) ) {
            $Log->step("Click Add button");
            $self->click($LICENSE_ADD_SN_BUTTON);
            $self->wait_for_element_present($LICENSE_SN_ADD_FIELD_TEXTBOX);

            $Log->step("Type the license key : $license");
            $self->type($LICENSE_SN_ADD_FIELD_TEXTBOX, $license);
            $self->key_down($LICENSE_SN_ADD_FIELD_TEXTBOX, "\13");
            $self->key_up($LICENSE_SN_ADD_FIELD_TEXTBOX, "\13");
        
            $Log->step("Click Add button");
            $self->click($LICENSE_SN_ADD_NEXT_BUTTON);
            $self->check_for_error();

            $Log->step("Click Add Close button");
            $commit_time = $self->click_timecount($LICENSE_SN_ADD_FINAL_BUTTON);
            $Log->comment("Loading time and commit time are $loading_time sec ! and $commit_time sec !" );

            $self->check_for_error();
            $self->wait_for_element_not_present($LICENSE_SN_ADD_CANCEL_BUTTON, $GENERIC_WAIT);
        } else {
            $Log->step("Click Add button");
            $self->click("$LICENSE_ADD_BUTTON");
            $self->wait_for_loading();
            #$self->wait_for_element_present($LICENSE_ADD_FIELD_TEXTBOX);
        
            $Log->step("Type the license key");
            $self->wait_for_element_present($LICENSE_ADD_FIELD_TEXTBOX);
            $Log->step("Type the license key");
                                                
            $self->type($LICENSE_ADD_FIELD_TEXTBOX, $license );
            $self->key_down($LICENSE_ADD_FIELD_TEXTBOX, "\13");
            $self->key_up($LICENSE_ADD_FIELD_TEXTBOX, "\13");

            $Log->step("Click Add button");
            $self->click("$LICENSE_ADD_FIELD_BUTTON");
            $self->check_for_error();
            $self->wait_for_loading();

            $commit_time
                = $self->click_timecount( $LICENSE_REFRESH_BUTTON, "refresh" );

            $Log->comment(
                "Loading time and commit time are $loading_time sec ! and $commit_time sec !");
        }
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Added Licenses successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub license_add

###################################################################################
## @name ::license_delete
## @summary
##     Delete the licenses based on the license keys using GUI.
## @description
##     This method can be used to delete the licenses based on the license keys using GUI.
## @synopsis
##     license_delete(%args);
## INPUTS TO METHOD CALL 'license_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -licenses </td> <td> (Required)Licenses to be added </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub license_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-licenses" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Storage>License");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $LICENSES_TREE );
    $self->wait_for_loading();

    my $license = $args{-licenses};
        
    $Log->step("Click the License row");
    $self->click_row_with_texts( "$LICENSE_GRID", 1, 0, $license );
    $Log->step("Click Delete button");
    $self->click("$LICENSE_DELETE_BUTTON");
    $self->wait_for_loading();

    $Log->step("Check the confirmation checkbox");
    $self->set_checkbox( "$LICENSE_DELETE_CHECKBOX", 'enable' );
    $Log->step("Clicking Delete confirm button");
    $self->click("$LICENSE_DELETE_CONFIRM_BUTTON");
    $self->check_for_error();
    $self->wait_for_loading();
    
    $Log->step("Click Refresh button");

    my $commit_time
        = $self->click_timecount( $LICENSE_REFRESH_BUTTON, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("Deleted Licenses successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub license_delete

################################################################################
## @name ::license_refresh
## @summary
##     Refresh license page based on the license keys using GUI.
## @description
##     This method can be used to refresh the license page using GUI.
## @synopsis
##     license_refresh(%args);
## INPUTS TO METHOD CALL 'license_refresh' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub license_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->step("Navigating to $LICENSES_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $LICENSES_TREE );

    $self->wait_for_element_present($LICENSE_GRID);

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $LICENSE_REFRESH_BUTTON, "refresh" );

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("license refresh successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub license_refresh

################################################################################
## @name ::datetime_edit
## @summary
##     Handle the dialog box for Edit date time settings using GUI.
## @description
##     This method will editing date time settings using GUI.
## @synopsis
##     datetime_edit(%args);
## INPUTS TO METHOD CALL 'datetime_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -timezone </td> <td> Selecting time zone </td> </tr>
## <tr> <td> -enableNTP </td> <td> Enabling NTP </td> </tr>
## <tr> <td> -addoptions </td> <td> Adding options </td> </tr>
## <tr> <td> -deloptions </td> <td> Deleting options </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub datetime_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-timezone" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $DATE_TIME_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DATE_TIME_TREE );
    $self->wait_for_loading();

    $Log->step("Click edit button");
    $self->click_button("$DATE_TIME_EDIT_BUTTON");
    $self->wait_for_element_present( $DATE_TIME_ADD_BUTTON, $GENERIC_WAIT );

    $Log->comment("Selecting Timezone");
    $self->set_combobox_value( $DATE_TIME_TIMEZONE_COMBOBOX,
        $args{'-timezone'} );

    if ( exists( $args{'-addoptions'} ) ) {
        foreach my $addoptions ( @{ $args{-addoptions} } ) {
            $Log->comment("Entering Time server name:");
            $self->type_keys( $DATE_TIME_NTP_TEXTBOX, $addoptions );
            $self->type( $DATE_TIME_NTP_TEXTBOX, $addoptions );
            $Log->step("Click add button");
            $self->click("$DATE_TIME_NTP_ADD_BUTTON");
        } ## end foreach my $addoptions ( @{...})
    } ## end if ( exists( $args{'-addoptions'...}))
    if ( exists( $args{'-deleteoptions'} ) ) {
        foreach my $deleteoptions ( @{ $args{-deleteoptions} } ) {
            $Log->comment("Select Time server");
            $self->click("//div[text()='$deleteoptions']");
            $Log->step("Click delete button");
            $self->click("$DATE_TIME_NTP_DELETE_BUTTON");
        } ## end foreach my $deleteoptions (...)
    } ## end if ( exists( $args{'-deleteoptions'...}))

    $Log->step("Click edit ok button");
    my $commit_time = $self->click_timecount("$DATE_TIME_ADD_BUTTON");

    if ( $self->is_element_present($DATE_TIME_ADD_CONFIRM_BUTTON) ) {
        $Log->step("Click confirm button");
        my $msg = $self->get_text("$DATE_TIME_ADD_TEXT");
        $self->click("$DATE_TIME_ADD_CONFIRM_BUTTON");
        $self->click("$DATE_TIME_ADD_CANCEL_BUTTON");

        $Log->exit() if $may_exit;
        NATE::BaseException->throw("FAILED : $msg");
    } ## end if ( $self->is_element_present...)

    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("datetime edit successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub datetime_edit

################################################################################
## @name ::datetime_refresh
## @summary
##     To refresh date/time page using GUI.
## @description
##     This method will refresh date/time page using GUI.
## @synopsis
##    datetime_refresh(%args);
## INPUTS TO METHOD CALL 'datetime_refresh' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub datetime_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $DATE_TIME_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $DATE_TIME_TREE );

    $Log->step("Click Refresh button");
    my $commit_time
        = $self->click_timecount( $DATE_TIME_REFRESH_BUTTON, "refresh" );
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("datetime refresh successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub datetime_refresh

################################################################################
## @name ::snmp_edit
## @summary
##     Handle the dialog box for Edit SNMP settings using GUI.
## @description
##     This method will handle the dialog box for Editing SNMP settings using GUI.
## @synopsis
##     snmp_edit(%args);
## INPUTS TO METHOD CALL 'snmp_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -snmp_contact </td> <td> (Required) Selecting snmp_contact </td> </tr>
## <tr> <td> -snmp_location </td> <td> (Required) Selecting snmp_location </td> </tr>
## <tr> <td> -addcommunityname </td> <td> Adding community name </td> </tr>
## <tr> <td> -editcommunityname </td> <td>Editing community name </td> </tr>
## <tr> <td> -deletecommunityname </td> <td> Deleting community name </td> </tr>
## <tr> <td> -addsnmpoptions </td> <td>  Adding smmp options </td> </tr>
## <tr> <td> -editsnmpoptions </td> <td> Editing smmp options </td> </tr>
## <tr> <td> -enabletraphost </td> <td>  Enabling trap host </td> </tr>
## <tr> <td> -addtraphostoptions </td> <td>  Adding trap host options </td> </tr>
## <tr> <td> -editraphostoptions </td> <td>  Editing trap host options </td> </tr>
## <tr> <td> -deletetraphostoptions </td> <td>  Deleting trap host options </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snmp_edit {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-snmp_contact"  => { type => SCALAR },
            "-snmp_location" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Configuration>Storage>SNMP");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNMP_TREE );

    $Log->step("Click Edit button");
    $self->click("$SNMP_EDIT_BUTTON");
    $self->wait_for_element_present( $SNMP_EDIT_GENERAL_TAB, $GENERIC_WAIT );

    $Log->step("Navigating to general tab");
    $self->click("$SNMP_EDIT_GENERAL_TAB");
    $self->wait_for_element_present( $SNMP_EDIT_CONTACT_TEXTBOX,
                $GENERIC_WAIT );
    $Log->step("entering snmp contact");
    $self->type( $SNMP_EDIT_CONTACT_TEXTBOX, $args{'-snmp_contact'} );
    $Log->step(" entering snmp location");
    $self->type( $SNMP_EDIT_LOCATION_TEXTBOX,
    $args{'-snmp_location'} );
    $self->type( $SNMP_EDIT_LOCATION_TEXTBOX, $args{'-snmp_location'} );

    if ( exists( $args{'-addcommunityname'} ) ) {
        foreach my $addcommunityname ( @{ $args{-addcommunityname} } ) {
            $Log->step("Click on add button");
            $self->click($SNMP_COMMUNITY_NAME_ADD_BUTTON);

            $Log->step("entering snmp community");
            $self->type_keys( $SNMP_COMMUNITY_NAME_TEXTBOX,
                $addcommunityname );
            $self->type( $SNMP_COMMUNITY_NAME_TEXTBOX, $addcommunityname );

            $Log->step("Click Ok button");
            $self->click("$SNMP_COMMUNITY_NAME_OK_BUTTON");
            $self->wait_for_element_present( $SNMP_COMMUNITY_NAME_EDIT_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $addcommunityname...
    } ## end if ( exists( $args{'-addcommunityname'...}))

    if ( exists( $args{'-editcommunityname'} ) ) {
        foreach my $editcommunityname ( @{ $args{'-editcommunityname'} } ) {
            my ( $communityname, $communityrename )
                = split( /:/, $editcommunityname );
            $Log->comment("Select community name from grid");
            $self->click_row_with_texts( "$SNMP_COMMUNITY_NAME_GRID",
                $communityname );

            $Log->comment("Click on edit button");
            $self->click("$SNMP_COMMUNITY_NAME_EDIT_BUTTON");

            if ( length($communityrename) ) {
                $Log->step("entering new snmp community name");
                $self->type_keys( "$SNMP_COMMUNITY_NAME_TEXTBOX",
                    $communityrename );
                $self->type( "$SNMP_COMMUNITY_NAME_TEXTBOX",
                    $communityrename );
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Community Rename is a Mandatory Parameter");
            }

            $Log->step("Click Ok button");
            $self->click("$SNMP_COMMUNITY_NAME_OK_BUTTON");
            $self->wait_for_element_present( $SNMP_COMMUNITY_NAME_EDIT_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $editcommunityname...
    } ## end if ( exists( $args{'-editcommunityname'...}))

    if ( exists( $args{'-deletecommunityname'} ) ) {
        foreach my $deletecommunityname ( @{ $args{'-deletecommunityname'} } )
        {
            $Log->comment("Select snmp community name from grid");
            $self->click_row_with_texts( "$SNMP_COMMUNITY_NAME_GRID",
                $deletecommunityname );

            $Log->step("Click Delete button");
            $self->click("$SNMP_COMMUNITY_NAME_DELETE_BUTTON");
            $self->wait_for_element_present( $SNMP_COMMUNITY_NAME_ADD_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $deletecommunityname...
    } ## end if ( exists( $args{'-deletecommunityname'...}))

    $Log->step("Navigating to SNMP v3 tab");
    $self->click("$SNMP_EDIT_SNMP_V3_TAB");
    $self->wait_for_element_present(
        $SNMP_EDIT_SNMP_V3_SECURITY_NAME_ADD_BUTTON,
        $GENERIC_WAIT );

    if ( exists( $args{'-addsnmpoptions'} ) ) {
        foreach my $addsnmpoptions ( @{ $args{'-addsnmpoptions'} } ) {
            $Log->step("Click Add button");
            $self->click("$SNMP_EDIT_SNMP_V3_SECURITY_NAME_ADD_BUTTON");

            my ( $username, $engineid, $authenticationprotocol, $authpass,
                $authconfirmpass, $privproto, $privpass, $privconfirmpass )
                = split( /:/, $addsnmpoptions );
            if ( length($username) ) {
                $Log->step("Entering username");
                $self->type_keys( "$SNMP_EDIT_ADD_SNMPV3_USERNAME_TEXTBOX",
                    $username );
                $self->type( "$SNMP_EDIT_ADD_SNMPV3_USERNAME_TEXTBOX",
                    $username );
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Username is a Mandatory Parameter");
            }

            if ( length($engineid) ) {
                $Log->step("entering engineid");
                $self->type_keys( "$SNMP_EDIT_ADD_SNMPV3_ENGINEID_TEXTBOX",
                    $engineid );
                $self->type( "$SNMP_EDIT_ADD_SNMPV3_ENGINEID_TEXTBOX",
                    $engineid );
            } ## end if ( length($engineid))

            if ( length($authenticationprotocol) ) {
                $Log->step("selecting Authentication protocol");
                $self->set_combobox_value(
                    $SNMP_EDIT_ADD_SNMPV3_AUTH_PROTOCOL_COMBO,
                    $authenticationprotocol );

                if ( not( lc($authenticationprotocol) eq 'none' ) ) {
                    $self->wait_for_element_present(
                        $SNMP_EDIT_ADD_SNMPV3_PASSWORD_TEXTBOX,
                        $GENERIC_WAIT );

                    if ( length($authpass) ) {
                        $Log->step("Entering Authentication Password");
                        $self->type( "$SNMP_EDIT_ADD_SNMPV3_PASSWORD_TEXTBOX",
                            $authpass );
                    } else {
                        $Log->exit() if $may_exit;
                        NATE::BaseException->throw(
                            "Authentication Password is a Mandatory Parameter"
                        );
                    } ## end else [ if ( length($authpass))]

                    if ( length($authconfirmpass) ) {
                        $Log->step(
                            "Entering Authentication Confirmation Password");
                        $self->type(
                            "$SNMP_EDIT_ADD_SNMPV3_CONFIRMPASSWORD_TEXTBOX",
                            $authconfirmpass );
                    } else {
                        $Log->exit() if $may_exit;
                        NATE::BaseException->throw(
                            "Authentication Confrim Password is a Mandatory Parameter"
                        );
                    } ## end else [ if ( length($authconfirmpass...))]

                    if ( length($privproto) ) {
                        $Log->step("selecting Privacy protocol");
                        $self->set_combobox_value(
                            $SNMP_EDIT_ADD_SNMPV3_PRIVACYPROTOCOL_COMBOBOX,
                            $privproto );

                        if ( not( lc($privproto) eq 'none' ) ) {
                            $self->wait_for_element_present(
                                $SNMP_EDIT_ADD_SNMPV3_PRIVACYPASSWORD_TEXTBOX,
                                $GENERIC_WAIT
                            );

                            if ( length($privpass) ) {
                                $Log->step("Entering Privacy Password");
                                $self->type(
                                    "$SNMP_EDIT_ADD_SNMPV3_PRIVACYPASSWORD_TEXTBOX",
                                    $privpass
                                );
                            } else {
                                $Log->exit() if $may_exit;
                                NATE::BaseException->throw(
                                    "Privacy Password is a Mandatory Parameter"
                                );
                            } ## end else [ if ( length($privpass))]

                            if ( length($privconfirmpass) ) {
                                $Log->step(
                                    "Entering Privacy Confirmation Password");
                                $self->type(
                                    "$SNMP_EDIT_ADD_SNMPV3_PRIVACYCONFIRMPASSWORD_TEXTBOX",
                                    $privconfirmpass
                                );
                            } else {
                                $Log->exit() if $may_exit;
                                NATE::BaseException->throw(
                                    "Privacy Confrim Password is a Mandatory Parameter"
                                );
                            } ## end else [ if ( length($privconfirmpass...))]
                        } ## end if ( not( lc($privproto...)))
                    } ## end if ( length($privproto...))
                } ## end if ( not( lc($authenticationprotocol...)))
            } ## end if ( length($authenticationprotocol...))

            $self->click("$SNMP_EDIT_ADD_SNMPV3_OK_BUTTON");
            $self->wait_for_element_not_present(
                $SNMP_EDIT_ADD_SNMPV3_OK_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $addsnmpoptions ...
    } ## end if ( exists( $args{'-addsnmpoptions'...}))

    if ( exists( $args{'-deletesnmpoptions'} ) ) {
        foreach my $deletesnmpoptions ( @{ $args{'-deletesnmpoptions'} } ) {
            $Log->comment("Select security name from grid");
            $self->click_row_with_texts(
                "$SNMP_EDIT_SNMP_V3_SECURITY_NAME_GRID",
                $deletesnmpoptions );

            $Log->step("Click Delete button");
            $self->click( "$SNMP_EDIT_SNMP_V3_SECURITY_NAME_DELETE_BUTTON" );
            $self->wait_for_element_present(
                $SNMP_EDIT_SNMP_V3_SECURITY_NAME_ADD_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $deletesnmpoptions...
    } ## end if ( exists( $args{'-deletesnmpoptions'...}))

    $Log->comment("Navigating to traphost tab");
    $self->click("$SNMP_EDIT_TRAPHOST_TAB");
    $self->wait_for_element_present( $SNMP_EDIT_TRAPHOST_CHECKBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-enabletraphost'} ) ) {
        $Log->comment("Enabling traphost");
        $self->set_checkbox( $SNMP_EDIT_TRAPHOST_CHECKBOX,
            $args{'-enabletraphost'} );
    }

    if ( $args{'-enabletraphost'} and 
         lc( $args{'-enabletraphost'} ) eq 'enable' ) {
        if ( exists( $args{'-addtraphostoptions'} ) ) {
            foreach my $addtraphostoptions ( @{ $args{-addtraphostoptions} } )
            {
                $Log->comment("Click Add button");
                $self->click($SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_BUTTON);

                $Log->comment("Entering ip address");
                $self->type( $SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_TEXTBOX,
                    $addtraphostoptions );

                $Log->comment("Click OK button");
                $self->click($SNMP_EDIT_TRAPHOST_IP_ADDRESS_OK_BUTTON);
                $self->wait_for_element_present(
                    $SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_BUTTON,
                    $GENERIC_WAIT );
            } ## end foreach my $addtraphostoptions...
        } ## end if ( exists( $args{'-addtraphostoptions'...}))

        if ( exists( $args{'-edittraphostoptions'} ) ) {
            foreach
                ( @{ $args{-edittraphostoptions} } )
            {
                my ( $ipaddress, $changeipaddress )
                    = split( /:/, $_ );
                $Log->comment("Select ip address from grid");
                $self->click_row_with_texts(
                    "$SNMP_EDIT_TRAPHOST_IP_ADDRESS_GRID", $ipaddress );

                $Log->comment("Click Edit button");
                $self->click("$SNMP_EDIT_TRAPHOST_EDIT_BUTTON");

                if ( length($changeipaddress) ) {
                    $Log->comment("entering new ip address");
                    $self->type_keys(
                        "$SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_TEXTBOX",
                        $changeipaddress );
                    $self->type( "$SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_TEXTBOX",
                        $changeipaddress );
                } ## end if ( length($changeipaddress...))

                $Log->comment("Click Ok button");
                $self->click("$SNMP_EDIT_TRAPHOST_IP_ADDRESS_OK_BUTTON");
                $self->wait_for_element_present(
                    $SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_BUTTON,
                    $GENERIC_WAIT );
            } ## end foreach my $edittraphostoptions...
        } ## end if ( exists( $args{'-edittraphostoptions'...}))

        if ( exists( $args{'-deletetraphostoptions'} ) ) {
            foreach my $deletetraphostoptions (
                @{ $args{-deletetraphostoptions} } )
            {
                $Log->comment("Select ip address from grid");
                $self->click_row_with_texts(
                    "$SNMP_EDIT_TRAPHOST_IP_ADDRESS_GRID",
                    $deletetraphostoptions );

                $Log->comment("Click Delete button");
                $self->click("$SNMP_EDIT_TRAPHOST_DELETE_BUTTON");
                $self->wait_for_element_present(
                    $SNMP_EDIT_TRAPHOST_IP_ADDRESS_ADD_BUTTON,
                    $GENERIC_WAIT );
            } ## end foreach my $deletetraphostoptions...
        } ## end if ( exists( $args{'-deletetraphostoptions'...}))
    } ## end if ( lc( $args{'-enabletraphost'...}))

    $Log->comment("Click OK button");
    $self->click( "$SNMP_EDIT_OK_BUTTON");
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("snmp parameters edited successfully");
    $Log->exit() if $may_exit;
} ## end sub snmp_edit

################################################################################
## @name ::snmp_refresh
## @summary
##     Refresh snmp page using GUI.
## @description
##     This method will Refresh snmp page using GUI.
## @synopsis
##     snmp_refresh(%args);
## INPUTS TO METHOD CALL 'snmp_refresh' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub snmp_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->step("Navigating to Configuration>Storage>SNMP");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNMP_TREE );

    $Log->step("Click Refresh button");
    $self->click( $SNMP_REFRESH_BUTTON );
    $self->wait_for_loading();

    $Log->comment("snmp page refreshed successfully");
    $Log->exit() if $may_exit;

} ## end sub snmp_refresh

################################################################################
## @name ::event_configure
## @summary
##     Configure events using GUI.
## @description
##     This method will configure events using GUI.
## @synopsis
##     event_configure(%args);
## INPUTS TO METHOD CALL 'event_configure' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -mail_server </td> <td> Selecting mail_server </td> </tr>
## <tr> <td> -mail_from </td> <td> Selecting mail_from </td> </tr>
## <tr> <td> -addoptions </td> <td> Adding options </td> </tr>
## <tr> <td> -editoptions </td> <td> Editing options </td> </tr>
## <tr> <td> -deleteoptions </td> <td> Deleting options </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub event_configure {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-mail_server" => { type => SCALAR },
            "-mail_from"   => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Diagonostics>logs>events");
    my $loading_time;
    eval {
        $loading_time = $self->cluster_tree_navigate(
                $self->{$SYSTEM_NAME_KEY}, $EVENTS_TREE );
    };
    if ($@) {
        $loading_time = $self->cluster_tree_navigate(
                $self->{$SYSTEM_NAME_KEY}, $EVENTS_TREE_SPACE );
    }
    $self->wait_for_loading();
    
    $Log->step("Clicking on the configure button");
    $self->click("$EVENTS_CONFIGURE_BUTTON");
    $self->wait_for_loading();

    $Log->step("selecting mail from");
    $self->type_keys( "$EVENTS_CONFIGURE_MAIL_FROM_TEXTBOX",
        $args{'-mail_from'} );
    $self->type( "$EVENTS_CONFIGURE_MAIL_FROM_TEXTBOX", $args{'-mail_from'} );

    $Log->step("selecting mail server");
    $self->type_keys( "$EVENTS_CONFIGURE_MAIL_SERVER_TEXTBOX",
        $args{'-mail_server'} );
    $self->type( "$EVENTS_CONFIGURE_MAIL_SERVER_TEXTBOX",
        $args{'-mail_server'} );

    if ( exists( $args{'-addoptions'} ) ) {
        foreach my $addoptions ( @{ $args{-addoptions} } ) {
            my ( $receivername, $receivermail ) = split( /:/, $addoptions );
            $Log->step("Click Add button");
            $self->click("$EVENTS_MAIL_RECEIVERS_ADD_BUTTON");
            $self->wait_for_element_present(
                $EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX,
                $GENERIC_WAIT );

            if ( length($receivername) ) {
                $Log->step("Entering receiver name");
                $self->type_keys( "$EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX",
                    $receivername );
                $self->type( "$EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX",
                    $receivername );
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Receiver Name is a Mandatory Parameter");
            }

            if ( length($receivermail) ) {
                $Log->step("Entering receiver mail");
                $self->type_keys( "$EVENTS_MAIL_RECEIVERS_EMAIL_TEXTBOX",
                    $receivermail );
                $self->type( "$EVENTS_MAIL_RECEIVERS_EMAIL_TEXTBOX",
                    $receivermail );
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Receiver Mail is a Mandatory Parameter");
            }

            $Log->step("Clicking on the OK button");
            $self->click("$EVENTS_MAIL_RECEIVERS_OK_BUTTON");
            $self->wait_for_element_present(
                $EVENTS_MAIL_RECEIVERS_ADD_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $addoptions ( @{...})
    } ## end if ( exists( $args{'-addoptions'...}))

    if ( exists( $args{'-editoptions'} ) ) {
        foreach my $editoptions ( @{ $args{-editoptions} } ) {
            my ( $receivername, $changereceivername, $changereceivermail )
                = split( /:/, $editoptions );
            if ( length($receivername) ) {
                $Log->comment("Select receiver name from grid");
                $self->click_row_with_texts( "$EVENTS_MAIL_RECEIVERS_GRID",
                    $receivername );
            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Receiver Name is a Mandatory Parameter");
            }

            $Log->step("Click Edit button");
            $self->click("$EVENTS_MAIL_RECEIVERS_EDIT_BUTTON");
            $self->wait_for_element_present(
                $EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX,
                $GENERIC_WAIT );

            if ( length($changereceivername) ) {
                $Log->step("Changing receiver name");
                $self->type_keys( "$EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX",
                    $changereceivername );
                $self->type( "$EVENTS_MAIL_RECEIVERS_NAME_TEXTBOX",
                    $changereceivername );
            } ## end if ( length($changereceivername...))

            if ( length($changereceivermail) ) {
                $Log->step("Changing receiver mail");
                $self->type_keys( "$EVENTS_MAIL_RECEIVERS_EMAIL_TEXTBOX",
                    $changereceivermail );
                $self->type( "$EVENTS_MAIL_RECEIVERS_EMAIL_TEXTBOX",
                    $changereceivermail );
            } ## end if ( length($changereceivermail...))

            $Log->step("Clicking on the OK button");
            $self->click("$EVENTS_MAIL_RECEIVERS_OK_BUTTON");
            $self->wait_for_element_present(
                $EVENTS_MAIL_RECEIVERS_ADD_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $editoptions ( @...)
    } ## end if ( exists( $args{'-editoptions'...}))

    if ( exists( $args{'-deleteoptions'} ) ) {
        foreach my $deleteoptions ( @{ $args{-deleteoptions} } ) {
            $Log->comment("Select receiver name from grid");
            $self->click_row_with_texts( "$EVENTS_MAIL_RECEIVERS_GRID",
                $deleteoptions );

            $Log->step("Click remove button");
            $self->click("$EVENTS_MAIL_RECEIVERS_REMOVE_BUTTON");
            $self->wait_for_element_present(
                $EVENTS_MAIL_RECEIVERS_ADD_BUTTON,
                $GENERIC_WAIT );
        } ## end foreach my $deleteoptions (...)
    } ## end if ( exists( $args{'-deleteoptions'...}))

    my $commit_time = $self->click_timecount($EVENT_MAIL_OK_BUTTON);
    $Log->comment(
        "Loading time and commit time are $loading_time sec ! and $commit_time sec !"
    );
    my %times = (

        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment("event configured successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub event_configure

################################################################################
## @name ::get_snmp_info
## @summary
##    Give the SNMP Info from the SNMP main page using GUI.
## @description
##     This method will give the SNMP Info from the SNMP main page using GUI.
## @synopsis
##     get_snmp_info(%args);
## INPUTS TO METHOD CALL 'get_snmp_info' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the SNMP status details.

sub get_snmp_info {

    my ( $self, %args ) = @_;
    my %values;
    my $snmp_info = {};

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->step("Navigating to $ SNMP_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $SNMP_TREE );
    $self->wait_for_element_present( $SNMP_REFRESH_BUTTON, $GENERIC_WAIT );

    $Log->comment("Fetching SNMP STATUS");
    if ( $self->is_element_present("$SNMP_INFO_SNMP_STATUS") ) {
        $values{"$SNMP_SNMP_STATUS_KEY"}
            = $self->get_text("$SNMP_INFO_SNMP_STATUS");
    }

    $Log->comment("Fetching TRAP STATUS");
    if ( $self->is_element_present("$SNMP_INFO_TRAP_STATUS") ) {
        $values{"$SNMP_TRAP_STATUS_KEY"}
            = $self->get_text("$SNMP_INFO_TRAP_STATUS");
    }

    $Log->comment("Fetching LOCATION");
    if ( $self->is_element_present("$SNMP_INFO_LOCATION") ) {
        $values{"$SNMP_LOCATION_KEY"}
            = $self->get_text("$SNMP_INFO_LOCATION");
    }

    $Log->comment("Fetching CONTACT");
    if ( $self->is_element_present("$SNMP_INFO_CONTACT") ) {
        $values{"$SNMP_CONTACT_KEY"} = $self->get_text("$SNMP_INFO_CONTACT");
    }

    $Log->comment("Collection of SNMP info is successfull");
    $snmp_info->{parsed_output} = \%values;

    $Log->exit() if $may_exit;

    return $snmp_info;
} ## end sub get_snmp_info

################################################################################
## @name ::get_license_info
## @summary
##     Fetch license details based on the license keys using GUI.
## @description
##     This method can be used to fetch licenses details based on the license keys using GUI.
## @synopsis
##     get_license_info(%args);
## INPUTS TO METHOD CALL 'license_add' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -licenses </td> <td> (Required)Licenses for which details has to be fetched </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the First License Key Details.

sub get_license_info {

    my $self = shift;
    my %args = validate_with(
            params      => \@_,
            spec        => { "-licenses" => { type => SCALAR }, },
            allow_extra => 1
            );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    
    my $license_info = {};
    my $license = $args{-licenses}; 
     
    $Log->comment("Navigating to Storage>License");
    my $loading_time = $self->cluster_tree_navigate(
            $self->{$SYSTEM_NAME_KEY},
            $LICENSES_TREE );

    $Log->step("Click the License row");
    $self->click_row_with_texts("$LICENSE_GRID", 1, 0, $license);

    $Log->step("Getting row data for license $license from grid");
    my $rowdata = $self->get_row_data_with_text("$LICENSE_GRID", $license);

    my @rowarray = split(' ',$rowdata);
    my %temp_license_info;

    %temp_license_info = (
            "name"        => $rowarray[0],
            "license_key" => $rowarray[1],
            "limit"       => $rowarray[2],
            "expires_on"  => $rowarray[3],
    );

    $Log->comment("Collection of license info is successfull");
    $license_info->{parsed_output} = \%temp_license_info;

    $Log->exit() if $may_exit;

    return $license_info;
}

################################################################################
# Subroutine Name: get_event_configure_info
# Objective:       Give the event configuration Info from the events main page
# Details:         See POD documentation at the end of this file
################################################################################
sub get_event_configure_info {
    my($self,%args) = @_;
    my %eventInfo;

    my ($pack, $file, $line, $subname, $hasargs, $wantarray) = caller(0);
    $Log->comment("****$subname()-Called at line#$line from $file START****");

    $Log->comment("ARGUMENTS PASSED:");
    foreach (keys %args)
    {
        $Log->comment( "$_ => $args{$_}");
    }

    $Log->comment("Navigating to Storage>Diagonostics>logs>events");
    my $loading_time
        = $self->cluster_tree_navigate($self->{$SYSTEM_NAME_KEY},
                $EVENTS_TREE );

    $Log->step("Clicking on the configure button");
    $self->click("$EVENTS_CONFIGURE_BUTTON");
    $self->wait_for_loading();

    $Log->comment("fetching mail from");
    $eventInfo{"$EVENT_CONFIGURE_MAILFROM_KEY"} = $self->get_text("$EVENTS_CONFIGURE_MAIL_FROM_TEXTBOX");
    #$eventInfo{"$EVENT_CONFIGURE_MAILFROM_KEY"} = $self->get_text("gwt-debug-manju-cmode1EventConfigDialogMailFromFieldTextField-input");
    
    $Log->comment("fetching mail server");
    $eventInfo{"$EVENT_CONFIGURE_MAILSERVER_KEY"} = $self->get_text("$EVENTS_CONFIGURE_MAIL_SERVER_TEXTBOX");

    $Log->comment("****$subname()-Called at line#$line from $file END****");
    return %eventInfo;
}

################################################################################
## @name ::job_page
## @summary
##     Navigate to job page using GUI.
## @description
##     This method will navigate to job page using GUI.
## @synopsis
##     job_page(%args);
## INPUTS TO METHOD CALL 'job_page' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -mail_server </td> <td> Selecting mail_server </td> </tr>
## <tr> <td> -mail_from </td> <td> Selecting mail_from </td> </tr>
## <tr> <td> -addoptions </td> <td> Adding options </td> </tr>
## <tr> <td> -editoptions </td> <td> Editing options </td> </tr>
## <tr> <td> -deleteoptions </td> <td> Deleting options </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub job_page {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-tabopt" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->comment("Navigating to Diagonostics>Job");
    $self->cluster_tree_navigate(
                $self->{$SYSTEM_NAME_KEY}, $JOB_TREE );
    $self->wait_for_loading();

    if ( $args{'-tabopt'} =~ /current/) {
	$Log->comment("Clicking on Current Tab");
	$self->click("$JOB_CURRENT_TAB");	
	$self->wait_for_loading();

	$Log->comment("Clicking on refresh button in current job page");
        $self->click("$JOB_CURRENT_TAB_REFRESH");
        $self->wait_for_loading();
    }else {
	$Log->comment("Clicking on History Tab");
        $self->click("$JOB_HISTORY_TAB");
	$self->wait_for_loading();

	$Log->comment("Clicking on rferesh button on history job page");
        $self->click("$JOB_HISTORY_TAB_REFRESH");
        $self->wait_for_loading();
    }
}

1;

