# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Unix Users Groups Functionalities
##
## @description
##      Using this library, user can perform Cmode Unix Users Groups related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmUnixUsersGroups
##
##		unix_add_group(%args);
##		unix_edit_group(%args);
##		unix_delete_group(%args);
##		unix_refresh_group(%args);
##		unix_add_users(%args);
##		unix_edit_users(%args);
##		unix_delete_user(%args);
##		unix_refresh_user(%args);
##
##
## @status public
##
## @author afsars@netapp.com
## @change
##  2011-06-07 : afsars : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmUnixUsersGroups;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    unix_add_group
    unix_edit_group
    unix_delete_group
    unix_refresh_group
    unix_add_users
    unix_edit_users
    unix_delete_user
    unix_refresh_user
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::unix_add_group
## @summary
##     Add unix group using GUI.
## @description
##     This method can be used to add unix group using GUI.
## @synopsis
##     unix_add_group(%args);
## INPUTS TO METHOD CALL 'unix_add_group' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -groupname </td> <td> (Required)Name of the group to be added </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -groupID </td> <td> Entering group ID </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_add_group {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-groupName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking on group tab");
    $self->click($GROUP_GROUP_TAB);

    $Log->comment("Clicking group add button");
    $self->click($GROUP_ADD_BUTTON);

    $Log->comment("Waiting for add group Dialog");
    $self->wait_for_element_present( $GROUP_NAME_TEXTBOX, $GENERIC_WAIT );

    $Log->comment("Entering group name: $args{'-groupName'}");
    $self->type( $GROUP_NAME_TEXTBOX, $args{'-groupName'} );
    $self->fire_event( "$GROUP_NAME_TEXTBOX", "blur" );

    if ( exists( $args{-groupID} ) ) {
        $Log->step("Entering group ID: $args{-groupID}");
        $self->type( $GROUP_ID, $args{-groupID} );
        $self->fire_event( "$GROUP_ID", "blur" );

        $Log->comment("Selecting add button to complete.");
        $self->click($GROUP_ADD_OK_BUTTON);

    } ## end if ( exists( $args{-groupID...}))

    $self->check_for_error();
    $self->wait_for_element_not_present( $GROUP_ADD_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("group created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_add_group

################################################################################
## @name ::unix_edit_group
## @summary
##     Edit unix group using GUI.
## @description
##     This method can be used to edit unix group using GUI.
## @synopsis
##     unix_edit_group(%args);
## INPUTS TO METHOD CALL 'unix_edit_group' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -groupname </td> <td> (Required)Name of the group to be edited </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -groupID </td> <td> Entering group ID </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_edit_group {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-groupName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking on group tab");
    $self->click($GROUP_GROUP_TAB);

    $Log->step("Selecting group: $args{'-groupName'}");
    $self->click_row_with_texts( $GROUP_GRID, $args{'-groupName'} );

    $Log->comment("Clicking group edit button");
    $self->click($GROUP_EDIT_BUTTON);

    $Log->comment("Waiting for edit group Dialog");
    $self->wait_for_element_present( $GROUP_ID, $GENERIC_WAIT );

    if ( exists( $args{-groupID} ) ) {
        $Log->step("Entering group ID: $args{-groupID}");
        $self->type_keys( $GROUP_ID, $args{-groupID} );

        $Log->comment("Selecting ok button to complete.");
        $self->click($GROUP_ADD_OK_BUTTON);
    } ## end if ( exists( $args{-groupID...}))

    $self->check_for_error();
    $self->wait_for_element_not_present( $GROUP_ADD_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("group edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_edit_group

################################################################################
## @name ::unix_delete_group
## @summary
##     Delete unix group using GUI.
## @description
##     This method can be used to delete unix group using GUI.
## @synopsis
##     unix_delete_group(%args);
## INPUTS TO METHOD CALL 'unix_delete_group' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -groupname </td> <td> (Required)Name of the group to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -checkbox </td> <td> Selecting checkbox </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_delete_group {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-groupName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking on group tab");
    $self->click($GROUP_GROUP_TAB);

    $Log->step("Selecting group: $args{'-groupName'}");
    $self->click_row_with_texts( $GROUP_GRID, $args{'-groupName'} );

    $Log->comment("Clicking group delete button");
    $self->click($GROUP_DELETE_BUTTON);

    $Log->comment("Waiting for delete group Dialog");
    $self->wait_for_element_present( $GROUP_DELETE_CONFIRM_CHECKBOX,
        $GENERIC_WAIT );

    if ( exists( $args{-checkbox} ) ) {
        $Log->comment("Clicking confirm checkbox");
        $self->click($GROUP_DELETE_CONFIRM_CHECKBOX);

        $Log->comment("Selecting ok button to complete.");
        $self->click($GROUP_DELETE_OK_BUTTON);

    } ## end if ( exists( $args{-checkbox...}))

    $self->check_for_error();
    $self->wait_for_element_not_present( $GROUP_DELETE_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("group deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_delete_group

################################################################################
## @name ::unix_refresh_group
## @summary
##     Refresh unix group page using GUI.
## @description
##     This method can be used to refresh unix group page using GUI.
## @synopsis
##     unix_refresh_group(%args);
## INPUTS TO METHOD CALL 'unix_refresh_group' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_refresh_group {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking on group tab");
    $self->click($GROUP_GROUP_TAB);

    $self->wait_for_element_present($GROUP_GRID);

    $Log->step("Click Refresh button");
    $self->click($GROUP_REFRESH_BUTTON);

    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_refresh_group

################################################################################
## @name ::unix_add_users
## @summary
##     Add unix users using GUI.
## @description
##     This method can be used to add unix users using GUI.
## @synopsis
##     unix_add_users(%args);
## INPUTS TO METHOD CALL 'unix_add_users' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -userName </td> <td> (Required)Name of the user to be added </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -userID </td> <td> Entering user ID </td> </tr>
## <tr> <td> -grpName </td> <td> Entering grpName </td> </tr>
## <tr> <td> -fullName </td> <td> Entering fullName </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_add_users {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-userName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking users tab");
    $self->click($USERS_TAB);

    $Log->comment("Clicking users add button");
    $self->click($USERS_ADD_BUTTON);

    $Log->comment("Waiting for add users Dialog");
    $self->wait_for_element_present( $USERS_NAME_TEXTBOX, $GENERIC_WAIT );

    $Log->comment("Entering user name: $args{'-userName'}");
    $self->type( $USERS_NAME_TEXTBOX, $args{'-userName'} );
    $self->fire_event( "$USERS_NAME_TEXTBOX", "blur" );

    if ( exists( $args{-userID} ) ) {
        $Log->step("Entering group ID: $args{-userID}");
        $self->type( $USERS_ID, $args{-userID} );
        $self->fire_event( "$USERS_ID", "blur" );
    }

    if ( exists( $args{-grpName} ) ) {
        $Log->step("Entering group name: $args{-grpName}");
        $self->set_combobox_value( $USERS_GROUP_NAME_COMBOBOX,
            $args{-grpName} );
    }
    if ( exists( $args{-fullName} ) ) {
        $Log->step("Entering user full name: $args{-fullName}");
        $self->type_keys( $USERS_FULL_NAME_TEXTBOX, $args{-fullName} );
    }

    $Log->comment("Selecting add button to complete.");
    $self->click($USERS_ADD_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $USERS_ADD_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("user created successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_add_users

################################################################################
## @name ::unix_edit_users
## @summary
##     Edit unix users using GUI.
## @description
##     This method can be used to edit unix users using GUI.
## @synopsis
##     unix_edit_users(%args);
## INPUTS TO METHOD CALL 'unix_edit_users' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -userName </td> <td> (Required)Name of the user to be edited </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -userID </td> <td> Entering user ID </td> </tr>
## <tr> <td> -grpName </td> <td> Entering grpName </td> </tr>
## <tr> <td> -fullName </td> <td> Entering fullName </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_edit_users {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-userName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking users tab");
    $self->click($USERS_TAB);

    $Log->step("Selecting user: $args{'-userName'}");
    $self->click_row_with_texts( $USER_GRID, $args{'-userName'} );

    $Log->comment("Clicking user edit button");
    $self->click($USERS_EDIT_BUTTON);

    $Log->comment("Waiting for edit user Dialog");
    $self->wait_for_element_present( $USERS_ID, $GENERIC_WAIT );

    if ( exists( $args{-userID} ) ) {
        $Log->step("Entering group ID: $args{-userID}");
        $self->type( $USERS_ID, $args{-userID} );
        $self->fire_event( "$USERS_ID", "blur" );

    } ## end if ( exists( $args{-userID...}))

    if ( exists( $args{-grpName} ) ) {
        $Log->step("Entering group name: $args{-grpName}");
        $self->set_combobox_value( $USERS_GROUP_NAME_COMBOBOX,
            $args{-grpName} );
    }
    if ( exists( $args{-fullName} ) ) {
        $Log->step("Entering user full name: $args{-fullName}");
        $self->type_keys( $USERS_FULL_NAME_TEXTBOX, $args{-fullName} );
    }

    $Log->comment("Selecting ok button to complete.");
    $self->click($USERS_ADD_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $USERS_ADD_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("user edited successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_edit_users

################################################################################
## @name ::unix_delete_user
## @summary
##     Delete unix users using GUI.
## @description
##     This method can be used to delete unix users using GUI.
## @synopsis
##     unix_delete_user(%args);
## INPUTS TO METHOD CALL 'unix_delete_user' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -userName </td> <td> (Required)Name of the user to be deleted </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## <tr> <td> -checkbox </td> <td> Selecting checkbox </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_delete_user {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-userName" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking users tab");
    $self->click($USERS_TAB);

    $Log->step("Selecting user: $args{'-userName'}");
    $self->click_row_with_texts( $USER_GRID, $args{'-userName'} );

    $Log->comment("Clicking user delete button");
    $self->click($USERS_DELETE_BUTTON);

    $Log->comment("Waiting for delete user Dialog");
    $self->wait_for_element_present( $USERS_DELETE_CONFIRM_CHECKBOX,
        $GENERIC_WAIT );

    if ( exists( $args{-checkbox} ) ) {
        $Log->comment("Clicking confirm checkbox");
        $self->click($USERS_DELETE_CONFIRM_CHECKBOX);

        $Log->comment("Selecting ok button to complete.");
        $self->click($USERS_DELETE_OK_BUTTON);

    } ## end if ( exists( $args{-checkbox...}))

    $self->check_for_error();
    $self->wait_for_element_not_present( $USERS_DELETE_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("user deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_delete_user

################################################################################
## @name ::unix_refresh_user
## @summary
##     Refresh unix user page using GUI.
## @description
##     This method can be used to refresh unix user page using GUI.
## @synopsis
##     unix_refresh_user(%args);
## INPUTS TO METHOD CALL 'unix_refresh_user' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the group is to be added </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub unix_refresh_user {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $UNIX_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $UNIX_TREE );

    $Log->comment("Clicking users tab");
    $self->click($USERS_TAB);

    $self->wait_for_element_present($USER_GRID);

    $Log->step("Click Refresh button");
    $self->click($USERS_REFRESH_BUTTON);

    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->step("Refresh completed");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub unix_refresh_user

1;
