# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##      This modules contains all helper functions for CMode GUI functions.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmUtils;
##
## @status private
##
## @author
##	Afsar Syed afsars@netapp.com
##
## @change
##	4/29/2010 : Afsar Syed : Initial version
## @change
##  3/20/2012 : Vikas P : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmUtils;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NACL::APISet::Host::GUI::SysManager::NWFUtils;
use NACL::APISet;
use NATE::BaseException qw(:try);

use base qw(NACL::APISet::Host::GUI::SysManager::NWFUtils);

use Cwd qw(abs_path);
use NATE::ParamSet;

my $nate_params = NATE::ParamSet->new( global => 1 );
my $logdir      = abs_path( $nate_params->get("LOGDIR_UNIX") );
my $runid       = $nate_params->get("RUNID");

our $screenshot = "$logdir/$runid" . "_captured_screenshot_";
our $s_count    = 0;

################################################################################
# P U B L I C   M E T H O D S
################################################################################

###############################################################################
## @name:: click_button
##
## @summary
##      Click the button with checking pre and post conditions.
## @description
##     This method can be used to click the button with checking pre and post conditions.
## @synopsis
##     click_button(%args);
## INPUTS TO METHOD CALL 'click_button' ON APISET OBJECT.

sub click_button {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    if ( $self->is_button_enabled($debugID) ) {
        $self->check_for_error();
        $self->click($debugID);
        $self->check_for_error();
    } else {
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("The Button \"$debugID\" is not enabled");
    }

    $Log->exit() if $may_exit;
    return 1;

} ## end sub click_button

###############################################################################
## @name:: click_nowait
##
## @summary
##      This will click the button.
## @description
##     This method can be used to click the button.
## @synopsis
##     click_nowait(%args);
## INPUTS TO METHOD CALL 'click_nowait' ON APISET OBJECT.

sub click_nowait {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    $self->click($debugID);
    $Log->exit() if $may_exit;
    return 1;
}

###############################################################################
## @name:: update_system_details
##
## @summary
##      This will update system details
## @description
##      This will update system details.
## @synopsis
##     update_system_details(%args);
## INPUTS TO METHOD CALL 'update_system_details' ON APISET OBJECT.

sub update_system_details {
    my ( $self, %systemNodes ) = @_;
    my $nodeCount = scalar( keys %systemNodes );

    $Log->comment("Updating node count: $nodeCount  in system manager object.");
    $self->{$SYSTEM_NODECOUNT_KEY} = $nodeCount;

    while ( my ( $key, $value ) = each %systemNodes ) {
        $Log->comment("Updating $key = $value  in system manager object.");
        $self->{$key} = $value;
    }

    return 1;
} ## end sub update_system_details

###############################################################################
## @name:: update_active_system
##
## @summary
##      This will update active system details
## @description
##      This will update active system details
## @synopsis
##     update_active_system(%args);
## INPUTS TO METHOD CALL 'update_active_system' ON APISET OBJECT.

sub update_active_system {
    my ( $self, $vservername, $systeminfo ) = @_;
    load_GlobalConst_values();

    # Closing any previously opened system tab
    $Log->comment("Updating system detail in system manager object.");

    # Closing any previously opened system tab
    my $systemTabCloseId =
        "//li[contains(\@id,'$SYSTEM_TAB') and contains(\@class,"
      . " '$SYSTEM_TAB_CLASS_ID')]/a[\@class='$SYSTEM_TAB_CLOSE_CLASS_ID']";
    while ( $self->is_element_present($systemTabCloseId) ) {
        $Log->comment("Closing current active system's manage tab");
        $self->click($systemTabCloseId);
    }
    $Log->comment(
        "Updating Vserver name $vservername in system manager object .");
    $self->{$SYSTEM_VSERVER_NAME} = $vservername;

    # Parsing the string with ip|name|user|password

    my %currentSystem = $self->parse_system_details($systeminfo);

    $self->{$SYSTEM_IP_KEY}       = $currentSystem{$SYSTEM_IP_KEY};
    $self->{$SYSTEM_NAME_KEY}     = $currentSystem{$SYSTEM_NAME_KEY};
    $self->{$SYSTEM_USERNAME_KEY} = $currentSystem{$SYSTEM_USERNAME_KEY};
    $self->{$SYSTEM_PASSWORD_KEY} = $currentSystem{$SYSTEM_PASSWORD_KEY};

    $Log->comment( "Cluster to be managed : "
          . $self->{$SYSTEM_NAME_KEY} . " ( "
          . $self->{$SYSTEM_IP_KEY}
          . " ) " );

    return 1;
} ## end sub update_active_system

###############################################################################
## @name:: manage_system
##
## @summary
##      This function opens a system in new tab and authenticates it.
## @description
##      This function opens a system in new tab and authenticates it.
## @synopsis
##     manage_system(%args);
## INPUTS TO METHOD CALL 'manage_system' ON APISET OBJECT.

sub manage_system {

    my ($self) = @_;
    load_GlobalConst_values();

    my $systemName  = $self->{$SYSTEM_NAME_KEY};

    if ( $self->is_element_present($CLUSTER_TAB)) {
        $Log->comment("Authentication not required. Accessing second time");
    } else {
        eval {
            $Log->comment("Setting browser width and height to :1280, 1280");
            $self->set_window_size(1280,1280);

            # handling Authentication window
            $self->wait_for_element_present($LOGIN_PAGE_USERNAME,
               $GENERIC_WAIT);

            #Code for log popup
            if ( $self->is_element_present(
                    "//button[\@class='gwt-Button' and text()='Minimize']")) {
                $Log->comment("Minimise Log popup .....");
                $self->click(
                    "//button[\@class='gwt-Button' and text()='Minimize']");
            }

            if ($self->is_element_present($LOGIN_PAGE_USERNAME)){

                $Log->comment( "Entering Username : "
                             . $self->{$SYSTEM_USERNAME_KEY} );
                $self->type( $LOGIN_PAGE_USERNAME,
                               $self->{$SYSTEM_USERNAME_KEY} );

                $Log->comment( "Entering Password : "
                             . $self->{$SYSTEM_PASSWORD_KEY} );
                $self->type( $LOGIN_PAGE_PASSWORD,
                               $self->{$SYSTEM_PASSWORD_KEY} );

                # Code to handle Code coverage runs.
                if ( $self->is_element_present($LOGIN_PAGE_SYSTEM_IP) ) {
                    $Log->comment( "Entering Storage System IP: "
                                 . $self->{$SYSTEM_IP_KEY} );
                    $self->type($LOGIN_PAGE_SYSTEM_IP,
                                $self->{$SYSTEM_IP_KEY} );
                }

                $Log->comment("Clicking Connect button");
                $self->click($LOGIN_PAGE_SUBMIT_BTN);

                $self->wait_for_element_present(
                   "//span[\@aria-level=1 and text()='Cluster']",
                   $GENERIC_WAIT );
            } else {
                die "Timed out Waiting for Login dialog";
            }
        };
        if ($@ =~ /Timed out Waiting/ ) {
            # Continue since authentication dialog may not appear for the
            # second time access to the same system
            $Log->comment("Error encountered.Authentication "
                        . "not required. Accessing second time");
        } elsif ($@) {
            $Log->comment(
                "Unkown Error encountered in SmUtils::manage_system() : $@");
        }
    }
    $self->check_for_error();

    #Making sure that system Page is loaded
    #xpath to top tree element node1name/node2name
    my $treeElement =
        "//div[contains(\@id,'navigation-tree')]//span[contains(text(),"
      . " '$systemName' )]";
    $Log->comment("Waiting for system page to load");
    $self->wait_for_element_present( $treeElement, $GENERIC_WAIT );

    my $tabValue = $systemName;
    $TAB = $tabValue;
    $VS  = $self->{$SYSTEM_VSERVER_NAME};
    load_GlobalConst_values();

    $Log->comment("System opened for management.\n");

    return 1;
} ## end sub manage_system

###############################################################################
## @name:: get_disks_details
##
## @summary
##      This function gets the Disk Details like ID,State,Pool,Type,RPM,Size in MB.
## @description
##      This function gets the Disk Details like ID,State,Pool,Type,RPM,Size in MB.
## @synopsis
##     get_disks_details(%args);
## INPUTS TO METHOD CALL 'get_disks_details' ON APISET OBJECT.

sub get_disks_details {
    my ($self) = @_;
    $Log->comment("Creating APIset object for CLI.");
    my $nodeName = $self->{$SYSTEM_NAME_KEY};
    $nodeName =~ s/\..*//;
    my $hostObj     = host($nodeName);
    my $api_Set_Obj = NACL::APISet->new(
        hostobj   => $hostObj,
        category  => "Node",
        interface => "CLI",
        set       => "7Mode"
    );
    my $cmd = "sysconfig -r";
    $Log->comment("Executing $cmd.");
    my $response = $api_Set_Obj->execute_raw_command( command => $cmd );
    my @diskDetails = split( /\n/, $response );
    my @disks;
    my $countDisk = 0;

    foreach my $line (@diskDetails) {
        if ( $line =~ m/\s*(partner|spare|data|parity)\s+(\d.\.\d\d\.\d+)\s+\w+\s+\d+\s+\d+\s+.+:.+\s+(\d+)\s+(.+)\s+(\d+)\s+\d+\/\d+\s+(\d+)\/\d+/) {
            $Log->comment("ID:$2/State:$1/Pool:$3/Type:$4/RPM:$5/Size:$6 MB");
            $disks[$countDisk]{$DISK_STATE_KEY} = $1;
            $disks[$countDisk]{$DISK_ID_KEY}    = $2;
            $disks[$countDisk]{$DISK_POOL_KEY}  = $3;
            $disks[$countDisk]{$DISK_TYPE_KEY}  = $4;
            $disks[$countDisk]{$DISK_RPM_KEY}   = $5;
            $disks[$countDisk]{$DISK_SIZE_KEY}  = $6;
            $countDisk                          = $countDisk + 1;
        } ## end if ($line =~ ...
    } ## end foreach my $line (@diskDetails)
    if ( $countDisk < 1 ) {
        NATE::BaseException->throw("No disks found");
    }
    return @disks;
} ## end sub get_disks_details

###############################################################################
## @name:: vserver_tree_navigate
##
## @summary
##      This function is used to navigate the system tree.
## @description
##      This function is used to navigate the system tree.
## @synopsis
##     system_tree_navigate(%args);
## INPUTS TO METHOD CALL 'system_tree_navigate' ON APISET OBJECT.

sub vserver_tree_navigate {
    my ( $self, $vserverName, $element ) = @_;

    my $menus;

    if (   $element eq "Storage"
        || $element eq "Configuration"
        || $element eq "Policies"
        || $element eq "SnapMirror" ) {
        $menus =
          "Virtual Servers>All Vservers>" . $vserverName . ">" . $element;
    } elsif ($element eq "Volumes"
        || $element eq "LUNs"
        || $element eq "Qtrees"
        || $element eq "Quotas"
        || $element eq "Shares"
        || $element eq "Namespace" ) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Storage>"
          . $element;
    } elsif ($element eq "Export Policies"
        || $element eq "Snapshot Policies"
        || $element eq "Efficiency Policies" ) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Policies>"
          . $element;
    } elsif ($element eq "Network Interfaces"
        || $element eq "Protocols"
        || $element eq "Services"
        || $element eq "Users and Groups" ) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Configuration>"
          . $element;
    } elsif ($element eq "CIFS"
        || $element eq "NFS"
        || $element eq "iSCSI"
        || $element eq "FCP" ) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Configuration>Protocols>"
          . $element;
    } elsif ($element =~ /DNS|LDAP|Kerberos|NIS/) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Configuration>Services>"
          . $element;
    } elsif ( $element eq "Name Mapping" || $element eq "UNIX" ) {
        $menus =
            "Virtual Servers>All Vservers>"
          . $vserverName
          . ">Configuration>Users and Groups>"
          . $element;
    } else {
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw(
            "$element is a wrong Element in $menus.");
    }

    $self->_left_tree_navigate($menus);
    return 1;

} ## end sub vserver_tree_navigate

###############################################################################
## @name:: cluster_tree_navigate
##
## @summary
##      This function navigates to specific tree element in Clusters tab
## @description
##      This function navigates to specific tree element in Clusters
## @synopsis
##     cluster_tree_navigate(%args);
## INPUTS TO METHOD CALL 'cluster_tree_navigate' ON APISET OBJECT.

sub cluster_tree_navigate {
    my ( $self, $scope, $element ) = @_;
    $Log->enter() if $may_enter;
    my ( $menus, $timetaken );

    if ( $scope eq $self->{$SYSTEM_NAME_KEY} ) {
        $Log->comment( "sub cluster_tree_navigate(): Navigating to "
              . "cluster tab : $self->{$SYSTEM_NAME_KEY}" );
        my $cluster = $self->{$SYSTEM_NAME_KEY};
        $self->_tab_navigation("cluster");

        if ( $element eq $cluster ) {
            $menus = $cluster;
        } elsif ($element eq "Storage"
            || $element eq "Configuration"
            || $element eq "Diagnostics"
            || $element eq "Advanced" ) {
            $menus = $cluster . ">" . $element;
        } elsif ( $element eq "Aggregates" || $element eq "Disks" ) {
            $menus = $cluster . ">Storage>" . $element;
        } elsif ($element eq "Services"
            || $element eq "Security"
            || $element eq "System Tools"
            || $element eq "Snapshot Policies"
            || $element eq "Schedules"
            || $element eq "Peers" ) {
            $menus = $cluster . ">Configuration>" . $element;
        } elsif ( $element eq "LDAP"
            || $element eq "Kerberos" ) {
            $menus = $cluster . ">Configuration>Services>" . $element;
        } elsif ( $element eq "Users"
            || $element eq "Roles" ) {
            $menus = $cluster . ">Configuration>Security>" . $element;
        } elsif ($element eq "Licenses"
            || $element eq "Date and Time"
            || $element eq "SNMP" ) {
            $menus = $cluster . ">Configuration>System Tools>" . $element;
        } elsif ($element eq "Job"
            || $element eq "System Health"
            || $element eq "Logs" ) {
            $menus = $cluster . ">Diagnostics>" . $element;
        } elsif ( $element eq "Events"
            || $element eq "Events "
            || $element eq "SnapMirror Log" ) {
            $menus = $cluster . ">Diagnostics>Logs>" . $element;
        } else {
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("$element is a wrong Element in Tree.");
        }
        $timetaken = $self->_left_tree_navigate( $menus );
    } else {
        my $vserverName = $scope;
        $Log->comment( "sub cluster_tree_navigate(): Navigating to"
              . " Vserver :$vserverName tab" );
        my $clusterName = $self->{$SYSTEM_NAME_KEY};
        $self->_tab_navigation("vserver");

        if ( $element eq $clusterName ) {
            $menus = $clusterName;
        } elsif ( $element eq $vserverName ) {
            $menus = $clusterName . ">" . $vserverName;
        } elsif ($element eq "Storage"
            || $element eq "Configuration"
            || $element eq "Policies"
            || $element eq "SnapMirror" ) {
            $menus = $clusterName . ">" . $vserverName . ">" . $element;
        } elsif ($element eq "Volumes"
            || $element eq "LUNs"
            || $element eq "Qtrees"
            || $element eq "Quotas"
            || $element eq "Shares"
            || $element eq "Namespace" ) {
            $menus = $clusterName . ">" . $vserverName . ">Storage>" . $element;
        } elsif ($element eq "Export Policies"
            || $element eq "Snapshot Policies"
            || $element eq "Efficiency Policies" ) {
            $menus =
              $clusterName . ">" . $vserverName . ">Policies>" . $element;
        } elsif ($element eq "Network Interfaces"
            || $element eq "Protocols"
            || $element eq "Security"
            || $element eq "Services"
            || $element eq "Users and Groups" ) {
            $menus =
              $clusterName . ">" . $vserverName . ">Configuration>" . $element;
        } elsif ($element eq "CIFS"
            || $element eq "NFS"
            || $element eq "iSCSI"
            || $element eq "FC/FCoE" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Protocols>"
              . $element;
        } elsif ($element =~ /DNS|LDAP Client|Kerberos|NIS/) { 
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Services>"
              . $element;
        } elsif ( $element eq "Name Mapping"
            || $element eq "UNIX" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Users and Groups>"
              . $element;
        } elsif ( $element eq "Users" || $element eq "Roles" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Security>"
              . $element;
        } else {
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("$element is a wrong Element.");
        }

        $timetaken = $self->_left_tree_navigate( $menus );
    } ## end else

    $Log->comment("*** Navigation Completed : Time taken: $timetaken");
    $Log->exit() if $may_exit;
    return $timetaken;
} ## end sub cluster_tree_navigate

###############################################################################
## @name:: clear_window
##
## @summary
##      This function will check and close any left over dialog box due to previous testcase failure.
## @description
##      This function will check and close any left over dialog box due to previous testcase failure.
## @synopsis
##     clear_window(%args);
## INPUTS TO METHOD CALL 'clear_window' ON APISET OBJECT.

sub clear_window {

    my ($self) = @_;
    my $loop_count = 5;

    while (  $self->is_element_present($GENERIC_DIALOG_TITLE_BAR)
          or $self->is_element_present($GENERIC_DIALOG_TITLE_BAR1) ) {
        $Log->comment(
            "Closing dialog boxes left over by previously failed testcase.");
        while ($self->is_element_present($GENERIC_DIALOG_CLOSE)) {
             $self->click($GENERIC_DIALOG_CLOSE);
             $self->wait_for_loading();
        }
        while ($self->is_element_present($GENERIC_DIALOG2_OK_BUTTON)) {
             $self->click($GENERIC_DIALOG2_OK_BUTTON);
             $self->wait_for_loading();
        }
        while ($self->is_element_present($GENERIC_DIALOG_OK_BUTTON)) {
             $self->click($GENERIC_DIALOG_OK_BUTTON);
             $self->wait_for_loading();
        }
        if ($loop_count++ > 5) {
            NATE::BaseException->throw("clear_window() exceeded loop count");
        }
    } ## end while ($self->is_element_present...

    return 1;
} ## end sub clear_window

###############################################################################
## @name:: collapse_tree
##
## @summary
##      This function will collapse the left system tree.
## @description
##      This function is used to collape the opened view.
## @synopsis
##     collapse_tree(%args);
## INPUTS TO METHOD CALL 'collapse_tree' ON APISET OBJECT.

sub collapse_tree {
    my ($self) = @_;

    $Log->comment("Collapsing left tree.....");
    if ( $self->is_element_present(
            "//span[text()='Volumes' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Storage' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Syslog' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='System Tools' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='SSH/SSL' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Security' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='NFS' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Protocols' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Interfaces' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Networking' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Users' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Users and Groups' and $TREE_ELEMENT_CLASS_ID]"
        );
    }
    if ( $self->is_element_present(
            "//span[text()='Networking' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Configuration' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Session' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Diagnostics' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Storage' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='$self->{$SYSTEM_NAME_KEY}' and $TREE_ELEMENT_CLASS_ID]"
        );
    }
    return 1;
} ## end sub collapse_tree

###############################################################################
## @name:: check_for_error
##
## @summary
##      This function will check for all kinds of possible errors after completing a wizard.
## @description
##      This function will check for all kinds of possible errors after completing a wizard.
## @synopsis
##     check_for_error(%args);
## INPUTS TO METHOD CALL 'check_for_error' ON APISET OBJECT.

sub check_for_error {
    my ($self) = @_;
    $Log->comment("Checking to see if any error fields/pop-ups are present");
    $self->_check_error_field();
    $self->_check_for_errordialog();
    return 1;
}

###############################################################################
## @name:: wait_for_loading
##
## @summary
##      This function waits till the loading message disapears.
## @description
##      This function This function waits till the loading message disapears.
## @synopsis
##     check_for_error(%args);
## INPUTS TO METHOD CALL 'wait_for_loading' ON APISET OBJECT.

sub wait_for_loading {
    my ( $self, $timeout ) = @_;

    $Log->comment("Waiting for Page to load......");
    sleep("1");
    my $sleep_count;

    if ( defined $timeout ) {
        $sleep_count = $timeout / 1000;
    } else {
        $sleep_count = $GENERIC_WAIT / 1000;
    }

    while ( $self->is_element_present($GENERIC_PLEASE_WAIT) ) {
        $Log->comment("Waiting for Page to load......");
        sleep("1");
        $sleep_count = $sleep_count - 1;
        if ( $sleep_count == 0 ) {
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("Page load timeout.");
        }
    }
} ## end sub wait_for_loading

###############################################################################
## @name:: parse_system_details
## @summary
##      This function opens a system in new tab and authenticates it.
## @description
##      This function opens a system in new tab and authenticates it.
## @synopsis
##     parse_system_details(%args);
## INPUTS TO METHOD CALL 'parse_system_details' ON APISET OBJECT.

sub parse_system_details {
    my ( $self, $systemDetails ) = @_;

    # Splitting the string with | as delemiter
    my @sysDetail = split( /\|/, $systemDetails );

    #4 inputs expected -  IP|name|Username|Password
    if ( scalar(@sysDetail) != 4 ) {
        my $cnt = scalar(@sysDetail);
        NATE::BaseException->throw(
            "Incorrect System Details.\n$cnt\n@sysDetail\n");
    }

    # set IP/name/Username/Password
    return (
        $SYSTEM_IP_KEY       => $sysDetail[0],
        $SYSTEM_NAME_KEY     => $sysDetail[1],
        $SYSTEM_USERNAME_KEY => $sysDetail[2],
        $SYSTEM_PASSWORD_KEY => $sysDetail[3]
    );

} ## end sub parse_system_details

###############################################################################
## @name:: click_timecount
## @summary
##      This function perform click action and return time required to complete the action.
## @description
##      This function Perform click action and return time required to complete the action.
## @synopsis
##     click_timecount(%args);
## INPUTS TO METHOD CALL 'click_timecount' ON APISET OBJECT.

sub click_timecount {
    my ( $self, $curentID, $type, $nextID ) = @_;
    my $time_out = 300000;
    my ( $sec, $end, $start, $time_sec, $final_values );

    #Check for loading times
    if ( $curentID =~ /gwt-debug/ and
            !($self->is_button_enabled($curentID)) ) {
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw(
           "The Button \"$curentID\" is not enabled");
    }
    my $sleep_count = 300;

    $start = (time)[0];
    $self->click($curentID);
    sleep 1;
    if ( $type and $type eq 'refresh' ) {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
    } elsif ( $type and $type eq 'next' ) {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
        $self->wait_for_element_present( $nextID, $time_out );
        while ( not $self->is_button_enabled($nextID) ) {
            $Log->comment("Waiting for button $nextID to enable.");
            sleep 1;
        }
    } else {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
        while ( $self->is_element_present($curentID) ) {
            $Log->comment("Waiting for commit ......");
            sleep 1;
            $self->check_for_error();
            $sleep_count = $sleep_count - 1;
            if ( $sleep_count == 0 ) {
                $self->capture_screenshot($screenshot.$s_count++.".png");
                NATE::BaseException->throw("Page load timeout.");
            }
        }

        $self->wait_for_element_not_present( $curentID, $time_out );
    } ## end else [ if ($type eq 'refresh')
    $end = (time)[0];

    #Timings
    $sec = $end - $start;
    $final_values = sprintf( "%.2f", $sec );
    $self->clear_window();
    return $final_values;
} ## end sub click_timecount

###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Method:    _check_error_field
# Objective: This function checks error fields due to invalid entries.
# Details:   This function checks error fields due to invalid entries.
###############################################################################
sub _check_error_field {

    my ($self) = @_;
    $Log->enter() if $may_enter;
    #Check if invalid field in red color is displayed
    if ( $self->is_element_present($GENERIC_INVALID_FIELD) ) {

        #Getting debugID of the invalid field
        my $debugID = $self->get_attribute( $GENERIC_INVALID_FIELD, 'id' );

        #Removing the -input part from ID
        $debugID =~ s/-input//;

        #Appending it with x-form-el to form the upper element's ID
        my $topElementID = "x-form-el-" . $debugID;

        #Check if a red color Error Icon is displayed
        if ( $self->is_element_present("//div[\@id='$topElementID']/img") ) {
            #If red color Error Icon is displayed then do mouse over it
            $self->mouse_over("//div[\@id='$topElementID']/img");
        } elsif ( $self->is_element_present($GENERIC_INVALID_ICON) ) {
            #If any other Error Icon is displayed then do mouse over it
            $self->mouse_over($GENERIC_INVALID_ICON);
        } else {
            # If no Error Icon is displayed then do mouse over on the error textbox
            $self->mouse_over($debugID);
        }

        #Check if tooltip is displayed
        if ( $self->is_element_present($GENERIC_TOOLTIP) ) {

            # Fetch the tooltip message
            $Log->comment("Invalid input in $debugID \n ToolTip Message:");
            my $tooltipMsg = $self->get_text($GENERIC_TOOLTIP);
            #Die with the tooltip message
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("\"$tooltipMsg\"");
        }

        #Check for error icon is displayed
        if ( $self->is_element_present($GENERIC_ERROR_ICON) ) {

            # Fetch the error message along with error icon
            $Log->comment("Invalid Operation Error Message:");
            my $Vserver_ID = "//table/tbody[tr/td/div/div/table/tbody/"
                            ."tr/td/img[\@src='images/Delete.png]]/tr/td/div/span";

            #ID for Vserver dialog
            my $errorMsg = $self->get_text($GENERIC_ERROR_ICON);

            #Die with error message
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("\"$errorMsg\"");
        } ## end if ($self->is_element_present...

        # If not tooltip then throw Generic error message
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("Invalid input in $debugID");
    } ## end if ($self->is_element_present...

    $Log->exit() if $may_exit;
    return 1;
} ## end sub _check_error_field
###############################################################################
# Method:    _check_for_errordialog
# Objective: check if error dialog has popup and throw error message from error dialog.
# Details:   This Method checks if error dialog has popup and throw error message from error dialoge.
###############################################################################
sub _check_for_errordialog {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my $errorMessage;
    sleep 1;

    #Check if Error Dialog is present
    if ( $self->is_element_present($GENERIC_ERROR_DIALOG) ) {
        #Fetching the error message from Error dialog using Xpath
        $errorMessage = $self->get_text(
                "//div[\@id='$GENERIC_ERROR_DIALOG']/div/div/div/div/div/"
              . "div/div/div/div/table/tbody/tr/td/div" );
        $Log->comment("Error occurred: Dialog message is $errorMessage");
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    # Checking for alternate error dialog
    if ( $self->is_element_present($GENERIC_ERROR_DIALOG1) ) {
        #Fetching the error message from Error dialog using Xpath
        $errorMessage = $self->get_text(
                "//div[\@id='$GENERIC_ERROR_DIALOG1']/div/div/div/div/div/div/"
              . "div/div/div/table/tbody/tr/td/div" );
        $Log->comment("Error occurred: Dialog message is $errorMessage");
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    # Checking for error icon
    if ( $self->is_element_present($POPUP_ERROR_ICON) ) {
        $errorMessage = $self->get_text($POPUP_ERROR_MESSAGE);
        $Log->comment("Error occurred: Pop-up error message is $errorMessage");
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    # Checking for error window
    if ( $self->is_element_present($GENERIC_WARNING_DIALOG) ) {
        $errorMessage = $self->get_text($GENERIC_WARNING_TEXT);
        $Log->comment("Error occurred: Pop-up error message is $errorMessage");
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    $Log->exit() if $may_exit;
    return 1;
} ## end sub _check_for_errordialog

###############################################################################
# Method:    _left_tree_navigate
# Objective: This function navigates to specific tree element in left tree
# Details:   This function navigates to specific tree element in left tree.
###############################################################################
sub _left_tree_navigate {
    my ( $self, $menus ) = @_;
    my ( $sec, $end, $start, $final_values );

    $Log->comment("Navigating to $menus");

    # Splitting the menu string with > as delimiter
    my @menu = split( />/, $menus );
    $Log->comment("menu list : @menu");

    my $menu_count = scalar(@menu);

    my ( $count, $currMenuID, $prvMenuID );

    #by default the aria-level will be 1
    my $ariaCount = 1;

    my $ariaLevel = $ariaCount;

    # Xpath to first element in the tree. i.e System Name
    $prvMenuID =
      "//div[div[\@aria-level='$ariaLevel']//span[text()='$menu[0]']]";
    $Log->debug("Root Menu ID : $prvMenuID");

    if ($menu_count eq 1) {
        $self->wait_for_loading($GENERIC_WAIT);
        $currMenuID =
            "//div[contains(\@id,'navigation-tree__navigation-tree')"
          . " and \@aria-level=1]//span[text()='$menu[$menu_count-1]'"
          . " and $TREE_ELEMENT_CLASS_ID]";
    } else {
        for ( $count = 1 ; $count < $menu_count ; $count++ ) {
            # aria-level is an attribute in tree objects
            $ariaLevel = $count + $ariaCount;

            $Log->comment("Checking if ( $menu[$count] ) menu is present");
            $currMenuID = $prvMenuID
                        . "//div[div[\@aria-level='$ariaLevel']"
                        . "//span[text()='$menu[$count]']]";

            if ( $count eq 1 ) {
                $self->wait_for_element_present( $currMenuID,
                $GENERIC_WAIT);
            }

            unless ( $self->is_element_present($currMenuID) ) {
                # Since its not present double click on previous menu item
                $self->wait_for_element_present( $prvMenuID, $GENERIC_WAIT);
                $Log->comment(
                    "-----> No, So Clicking on ( $menu[$count-1] ) in tree");
                $self->_double_click($prvMenuID);
                $Log->comment("Waiting for ( $menu[$count] ) menu");
                # Waiting for subtree to expand
                $self->wait_for_loading($GENERIC_WAIT);
            }
            $Log->comment("-----> OK ( $menu[$count] ) menu is present");
            $prvMenuID = $currMenuID;
        } ## end for ( $count = 1; $count...)
    }
    
    $Log->comment("Clicking on final menu : ( $menu[$menu_count-1] )"
                 . "in the navigation tree");
    $self->wait_for_element_present( $currMenuID, $GENERIC_WAIT);
    $self->click($currMenuID);

    $start = (time)[0];
    $self->wait_for_loading($GENERIC_WAIT+200000);
    $end = (time)[0];

    #Timings
    $sec = $end - $start;
    $final_values = sprintf( "%.2f", $sec );
    $Log->comment( "Time measurement :Loading time for page"
                 . " $menus is $final_values Seconds" );

    return $final_values;

} ## end sub _left_tree_navigate

###############################################################################
# Method:    _cluster_tab_navigation
# Objective: This function will navigate accross the available tabs in Cluster view.
# Details:   This function will navigate accross the available tabs in Cluster view.
###############################################################################

sub _tab_navigation {
    my ( $self, $tablevel ) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Navigating across TAB : $tablevel");
    if ( $tablevel eq 'cluster' ) {
        $Log->comment("Check if Cluster Tab is present");
        if ( $self->is_element_present( $CLUSTER_TAB_COLLAPSED ) ) {
            $Log->comment(
                "-----> No, So Clicking Cluster Tab to expand for navigation");
            $self->click($CLUSTER_TAB_COLLAPSED);
            $self->wait_for_loading();
            $Log->comment("Cluster Tab is available for navigation");
        } ## end if ($self->is_element_present...

    } ## end if ($tablevel eq 'cluster')
    if ( $tablevel eq 'vserver' ) {
        my $elem = $VSERVER_TAB;
        $Log->comment("Check if Vserver Tab is present");
        if ( $self->is_element_present( $VSERVER_TAB )) {
            $Log->comment("Clicking Vserver Tab to expand for navigation");
            # The below click command sometimes fails in certain scenarios,
            # but can safely be bypassed and error if present will be caught in
            # subsequent $self->wait_for_element_present call.
            eval {
                $self->click( $VSERVER_TAB );
            };
            $self->wait_for_loading();
            $self->wait_for_element_present($CLUSTER_VSERVER, $GENERIC_WAIT);
            $Log->comment("Vserver Tab is available for navigation");
        } ## end if ($self->is_element_present...
    } ## end if ($tablevel eq 'vserver')
    $Log->exit() if $may_exit;
    return 1;
} ## end sub _tab_navigation

###############################################################################
# Method:    config_info
# Objective: This function will return data related to ONTAP version and config related information
# Details:   See POD documentation at the end of this file.
###############################################################################
sub config_info {
    my ($self) = @_;

    if ( $self->is_element_present($LOGGING_POPUP) ) {
        $Log->comment("Move the logging pop-up to center of menu bar");

        my $elem1 = $self->find_element($LOGGING_POPUP);
        my $elem2 = $self->find_element($MENU_BAR);

        $self->mouse_move_to_location( 'element' => $elem1 );
        $self->button_down;

        $self->mouse_move_to_location( 'element' => $elem2 );
        $self->button_up;
    }

    $Log->comment("Clicking help button");
    $self->click("$HELP_BUTTON");

    $Log->comment("Clicking Netapp About");
    $self->click("$ABOUT_NETAPP_DROPDOWN");

    $Log->comment("Clicking Configuration button");
    $self->click("$CONFIG_BUTTON");

    $Log->comment("Reading Netapp About data");
    my %value;
    my $elem_obj = $self->find_element($CONFIG_TEXTAREA);
    $value{"config"} = $elem_obj->get_value();

    $Log->comment("Collection of Config Information completed");

    my $temp_value = {};
    $temp_value->{parsed_output} = \%value;

    return $temp_value;
}

1;
