# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Volume GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode volume related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use SmVolume;
##
##      volume_create(%args);
##      volume_edit(%args);
##      clone_volume(%args);
##      clone_split(%args);
##      volume_status(%args);
##      volume_resize(%args);
##      is_clone_enabled(%args);
##      is_split_enabled(%args);
##      is_delete_enabled(%args);
##      modify_volume_status(%args);
##      delete_volume(%args);
##      create_volsnap(%args);
##      manage_volsnap(%args);
##      volume_dedupe(%args);
##      get_volume_info(%args);
##      is_snapshot_created(%args);
##      vol_storageeff_info(%args);
##      refresh_volume(%args);
##      get_snapshot_info(%args);
##
## @status public
##
## @author      Arun William. arunw@netapp.com
##
## @change
##	4/29/2010 : Arun William : Initial version
## @change
##  2/15/2012 : Vikas : POD Modifications

package NACL::APISet::Host::GUI::SysManager::CMode::SmVolume;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF );
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
  volume_create
  volume_edit
  clone_volume
  clone_split
  volume_status
  volume_resize
  is_clone_enabled
  is_split_enabled
  is_delete_enabled
  modify_volume_status
  delete_volume
  create_volsnap
  manage_volsnap
  volume_dedupe
  get_volume_info
  is_snapshot_created
  vol_storageeff_info
  refresh_volume
  get_snapshot_info
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::volume_create
## @summary
##     Creation of Volume using GUI.
## @description
##     This method can be used to create volume using GUI.
## @synopsis
##     volume_create(%args);
## INPUTS TO METHOD CALL 'volume_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver on which the volume is to be created </td> </tr>
## <tr> <td> -volsize </td> <td> Total Size of the Volume to be created </td> </tr>
## <tr> <td> -voltype </td> <td> Storage Type of the Volume to be created </td> </tr>
## <tr> <td> -volunit </td> <td> Unit of the Volume to be created </td> </tr>
## <tr> <td> -aggrname </td> <td> Aggregate Name in which the Volume has to be created </td> </tr>
## <tr> <td> -volSnapres </td> <td> Percentage of Snapshot Reserve </td> </tr>
## <tr> <td> -volSpacereser </td> <td> Enable Thin Provisioning for this Volume </td> </tr>
## <tr> <td> -voldedup </td> <td> Enable Storage Efficiency for this Volume </td> </tr>
## <tr> <td> -compression </td> <td> Enable Compression for this Volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub volume_create {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-volname'  => { type => SCALAR },
            "-aggrname" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );

    $Log->comment("***Click Create Button***");
    $self->click_button("$VOLUME_CREATE_BUTTON");
    $self->wait_for_loading();

    $Log->comment("***Type Volume Name :$args{'-volname'} ****");
    $self->type( "$VOLUME_NAME_INPUT", $args{'-volname'} );
    $self->fire_event( "$VOLUME_NAME_INPUT", "blur" );

    $Log->comment("***Select aggregate to Create Vol:$args{-aggrname} ***");
    $self->type( $VOLUME_AGGREGATE_INPUT, $args{-aggrname} );
    $self->fire_event( "$VOLUME_AGGREGATE_INPUT", "blur" );

    if ( exists( $args{'-voltype'} ) ) {
        if ( lc( $args{'-voltype'} ) eq 'nas' ) {
            $Log->comment("*** Click in Vol type $args{-voltype}***");
            $self->click("$VOLUME_TYPENAS_RADIO");
        }
        elsif ( lc( $args{-voltype} ) eq "san" ) {
            $Log->comment("*** Click in Vol type $args{-voltype}***");
            $self->click("$VOLUME_TYPESAN_RADIO");
        }
    } ## end if ( exists( $args{'-voltype'...}))

    if ( exists( $args{-volsize} ) ) {
        $Log->comment("*** Enter the Vol size:$args{-volsize} ***");
        $self->type( "$VOLUME_SIZE_INPUT", "" );
        $self->type_keys( "$VOLUME_SIZE_INPUT", $args{-volsize} );
        $self->type( "$VOLUME_SIZE_INPUT", $args{-volsize} );
        $self->fire_event( "$VOLUME_SIZE_INPUT", "blur" );
    }
    if ( exists( $args{-volunit} ) ) {
        if ( uc( $args{-volunit} ) =~ /[GB|TB|PB|MB]/ ) {
            $Log->comment("*** Click to enter vol unit :uc$args{-volunit} ***");
            $self->set_combobox_value( "$VOLUME_SIZEUNIT_COMBOBOX",
                uc( $args{-volunit} ) );
            $self->set_combobox_value( "$VOLUME_SIZEUNIT_COMBOBOX",
                uc( $args{-volunit} ) );
        } ## end if ( uc( $args{-volunit...}))
    } ## end if ( exists( $args{-volunit...}))

    if ( exists( $args{-volSnapres} ) ) {
        $Log->comment(
            "*** Enter the snap reserve input :$args{-volSnapres} ***");

        $self->type( "$VOLUME_SNAPSHOTRESERVE_INPUT", "" );
        $self->type_keys( "$VOLUME_SNAPSHOTRESERVE_INPUT", $args{-volSnapres} );
        $self->type( "$VOLUME_SNAPSHOTRESERVE_INPUT", $args{-volSnapres} );
        $self->fire_event( "$VOLUME_SNAPSHOTRESERVE_INPUT", "blur" );
    } ## end if ( exists( $args{-volSnapres...}))

    if ( exists( $args{-volSpacereser} ) ) {
        $Log->comment("Click to select Thin provision ");
        $self->set_checkbox( "$VOLUME_SPACE_GUARANTEE_INPUT",
            lc( $args{-volSpacereser} ) );
    }

    if ( exists( $args{-voldedup} ) ) {
        $self->click("$VOLUME_OPTIONS_TAB");
        if ( lc( $args{-voldedup} ) =~ m/enable/ ) {
            $Log->comment("*** Click to select Vol dedupe  : enable ***");
            $self->click( $VOLUME_DEDUPE_CHECKBOX, "enable" );
            if (
                $self->is_element_present($VOLUME_DEDUPE_COMPRESSION_CHECKBOX) )
            {
                if ( lc( $args{-compression} ) =~ m/enable/ ) {
                    $Log->comment(
                        "*** Click to select Vol dedupe  : enable ***");
                    $self->set_checkbox( $VOLUME_DEDUPE_COMPRESSION_CHECKBOX,
                        "enable" );
                }
                elsif ( lc( $args{-compression} ) =~ m/disable/ ) {
                    $Log->comment(
                        "*** Click to select Vol dedupe  : enable ***");
                    $self->set_checkbox( $VOLUME_DEDUPE_COMPRESSION_CHECKBOX,
                        "disable" );
                } ## end elsif ( lc( $args{-compression...}))
            } ## end if ( $self->is_element_present...)
        }
        elsif ( lc( $args{-voldedup} ) =~ m/disable/ ) {
            $Log->comment("*** Click to select Vol dedupe  : enable ***");
            $self->set_checkbox( $VOLUME_DEDUPE_CHECKBOX, "disable" );
        }
    } ## end if ( exists( $args{-voldedup...}))

    $Log->comment("*** Click OK to create ***");

    #$self->mouse_down($VOLUME_CREATE_CREATE_BUTTON);
    $self->click($VOLUME_CREATE_CREATE_BUTTON);
    $self->check_for_error();
    my %times = ( loading_time => $loading_time );
    $self->wait_for_loading();

    $Log->comment("Volume $args{'-volname'} created successfully");
    $Log->exit() if $may_exit;
    return \%times;

} ## end sub volume_create

################################################################################
## @name ::volume_edit
## @summary
##     Edit Volume using GUI.
## @description
##     This method can be used to edit volume using GUI.
## @synopsis
##     volume_edit(%args);
## INPUTS TO METHOD CALL 'volume_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the volume to be edit </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume to be edited resides </td> </tr>
## <tr> <td> -volrename </td> <td> Volume Name to be Rename </td> </tr>
## <tr> <td> -volstyle </td> <td> Security Style of this volume </td> </tr>
## <tr> <td> -volSpacereser </td> <td> Enable/Disable Thin Provisioning for this volume </td> </tr>
## <tr> <td> -volgrow </td> <td> Enable/Disable Automatically grow option for this volume </td> </tr>
## <tr> <td> -volautomaxgrow </td> <td> Maximum Size for Automatically grow option for this volume </td> </tr>
## <tr> <td> -maxunit </td> <td> Maximum Size Unit for Automatically grow option for this volume </td> </tr>
## <tr> <td> -volsnapdel </td> <td> Enable/Disable to delete older Snapshot Copies Automatically for this volume </td> </tr>
## <tr> <td> -volunicode </td> <td> Enable/Disable Support Unicode Format for directories </td> </tr>
## <tr> <td> -volacctime </td> <td> Enable/Disable Update access time when a file is read </td> </tr>
## <tr> <td> -dedup </td> <td> Enable/Disable Storage Efficiency for this Volume </td> </tr>
## <tr> <td> -voldedup </td> <td> Type of deduplication for this volume </td> </tr>
## <tr> <td> -volcompression </td> <td> Enable/Disable Compression for this Volume </td> </tr>
## <tr> <td> -volcompressioninline </td> <td> Enable/Disable Compress Inline for this Volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub volume_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { '-volname' => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");
    $self->wait_for_loading();
    $Log->comment("*** Select volume $args{'-volname'}:$args{'-volname'} ***");
    $self->click_row_with_texts( "$VOLUME_GRID", 1, 0, $args{'-volname'} );
    $self->wait_for_loading();
    sleep 10;

    $Log->comment("*** Click Edit Button ***");
    $self->click_button("$VOLUME_EDIT_BUTTON");
    $self->wait_for_loading();

    $Log->comment("*** Select volume $args{'-volname'}:$args{'-volname'} ***");
    if ( exists( $args{-volrename} ) ) {
        $Log->comment(" Renaming Volume name to:$args{-volrename} ***");
        $self->wait_for_element_present( $VOLUME_EDIT_NAME_INPUT,
            $args{-volrename} );
        $self->type( $VOLUME_EDIT_NAME_INPUT, $args{-volrename} );
    }

    if ( exists( $args{-volstyle} ) ) {
        $Log->comment(" Selecting Security style as :$args{-volstyle} ***");
        $self->set_combobox_value( "$VOLUME_EDIT_STYLE_INPUT",
            ( $args{-volstyle} ) );
    }

    if ( exists( $args{-volSpacereser} ) ) {
        $Log->comment(" Selecting Thin provision :$args{-volSpacereser} ***");
        $self->set_checkbox( "$VOLUME_EDIT_SPACEGUARA_INPUT",
            $args{-volSpacereser} );
    }

    if ( exists( $args{-volgrow} ) ) {
        $self->click("$VOLUME_EDIT_OPTION_TAB");
        $self->wait_for_loading();
        $Log->comment("Click on Option tab ");

        if ( lc( $args{-volgrow} ) =~ /enable/ ) {
            $Log->comment("*** Selecting vol Autogrow ***");
            $self->set_checkbox( "$VOLUME_EDIT_AUTOGROW_CHECKBOX", "enable" );
            if ( exists( $args{-volautomaxgrow} ) ) {
                $Log->comment(
"*** Reset max six for vol auto del to $args{-volautomaxgrow} ***"
                );
                $self->type( "$VOLUME_EDIT_AUTOGROW_MAX_INPUT",
                    $args{-volautomaxgrow} );
            } ## end if ( exists( $args{-volautomaxgrow...}))
            if ( uc( $args{-maxunit} ) =~ /[GB|TB|PB|MB]/ ) {
                $Log->comment(
                    "*** Click to enter vol unit :uc$args{-maxunit} ***");
                $self->set_combobox_value(
                    "$VOLUME_EDIT_AUTOGROW_MAXUNIT_COMBOBOX",
                    uc( $args{-maxunit} ) );
            } ## end if ( uc( $args{-maxunit...}))

        }
        elsif ( lc( $args{-volgrow} ) =~ /disable/ ) {
            $Log->comment("*** Deselecting vol Autogrow ***");
            $self->set_checkbox( "$VOLUME_EDIT_AUTOGROW_CHECKBOX", "disable" );
        }
    } ## end if ( exists( $args{-volgrow...}))

    if ( exists( $args{-volsnapdel} ) ) {

        $self->click("$VOLUME_EDIT_OPTION_TAB");
        $self->wait_for_loading();
        $Log->comment("Click on Option tab ");
        if ( lc( $args{-volsnapdel} ) =~ /enable/ ) {
            $Log->comment("*** Selecting vol Auto delete snapshot ***");
            $self->set_checkbox( "$VOLUME_EDIT_AUTODELETE_CHECKBOX", "enable" );
        }
        elsif ( lc( $args{-volsnapdel} ) =~ /disable/ ) {
            $Log->comment("*** Deselecting vol Auto delete snapshot ***");
            $self->set_checkbox( "$VOLUME_EDIT_AUTODELETE_CHECKBOX",
                "disable" );
        }
    } ## end if ( exists( $args{-volsnapdel...}))

    if ( exists( $args{-volunicode} ) ) {
        $self->click("$VOLUME_EDIT_OPTION_TAB");
        $self->wait_for_loading();
        $Log->comment("Click on Option tab ");
        if ( lc( $args{-volunicode} ) =~ /enable/ ) {
            $Log->comment("*** Selecting Unicode ***");
            $self->set_checkbox( "$VOLUME_EDIT_UNICODE_CHECKBOX", "enable" );
        }
        elsif ( lc( $args{-volunicode} ) =~ /disable/ ) {
            $Log->comment("*** Deselecting unicode ***");
            $self->set_checkbox( "$VOLUME_EDIT_UNICODE_CHECKBOX", "disable" );
        }
    } ## end if ( exists( $args{-volunicode...}))

    if ( exists( $args{-volacctime} ) ) {
        $self->click("$VOLUME_EDIT_OPTION_TAB");
        $self->wait_for_loading();
        $Log->comment("Click on Option tab ");
        if ( lc( $args{-volacctime} ) =~ /enable/ ) {
            $Log->comment("*** Selecting vol access time update ***");
            $self->set_checkbox( "$VOLUME_EDIT_NOACCESSTIME_CHECKBOX",
                "enable" );
        }
        elsif ( lc( $args{-volacctime} ) =~ /disable/ ) {
            $Log->comment("*** Deselecting vol access time update ***");
            $self->set_checkbox( "$VOLUME_EDIT_NOACCESSTIME_CHECKBOX",
                "disable" );
        }
    } ## end if ( exists( $args{-volacctime...}))

    if ( lc( $args{-dedup} ) eq 'enable' ) {
        $Log->comment("Selecting storage efficiency tab");
        $self->click("$VOLUME_EDIT_EFFICIENCY_TAB");
        $self->wait_for_loading();

        $Log->comment("*** Click to select Vol dedupe $args{-dedup} ***");
        $self->set_checkbox( $VOLUME_EDIT_DEDUPE_CHECKBOX, "enable" );

        if ( lc( $args{-voldedup} ) eq 'manually' ) {
            $Log->comment("Selecting manual dedupe : $args{-voldedup}");
            $self->set_option( "$VOLUME_EDIT_MANUAL_RADIOBUTTON", "enable" );
        }
        elsif ( lc( $args{-voldedup} ) eq 'automatic' ) {
            $Log->comment("Selecting automatic dedupe : $args{-voldedup}");
            $self->set_option( "$VOLUME_EDIT_AUTOMATIC_RADIOBUTTON", "enable" );
        }
        elsif ( lc( $args{-voldedup} ) eq 'schedule' ) {
            $Log->comment(
                "Selecting default efficiency policy : $args{-voldedup}");
            $self->set_option( "$VOLUME_EDIT_SCHEDULE_RADIOBUTTON", "enable" );

            $Log->comment("Clicking Schedule choose button");
            $self->click("$VOLUME_DEDUPE_MANUAL_SCHEDULE_BUTTON");
            $self->wait_for_loading();

            $Log->comment("Clicking efficiency default policy OK button");
            $self->click("$VOLUME_EDIT_SCHEDULE_EFF_POLICY_OK_BUTTON");
        }
        elsif ( lc( $args{-voldedup} ) eq 'customschedule' ) {
            $Log->comment(
                "Selecting custom efficiency policy : $args{-voldedup}");
            $self->set_option( "$VOLUME_EDIT_SCHEDULE_RADIOBUTTON", "enable" );

            $Log->comment("Clicking Schedule choose button");
            $self->click("$VOLUME_DEDUPE_MANUAL_SCHEDULE_BUTTON");
            $self->wait_for_loading();
            $Log->comment(
                "Selecting custom efficiency policy: $args{-effpolicy}");
            $self->set_combobox_value(
                "$VOLUME_EDIT_SCHEDULE_EFF_POLICY_COMBOBOX",
                $args{-effpolicy} );

            $Log->comment("Clicking efficiency policy OK button");
            $self->click("$VOLUME_EDIT_SCHEDULE_EFF_POLICY_OK_BUTTON");
        } ## end elsif ( lc( $args{-voldedup...}))

        if ( exists( $args{'-volcompression'} ) ) {
            if (   ( lc( $args{-volcompression} ) eq 'enable' )
                || ( lc( $args{-volcompression} ) eq 'disable' ) )
            {
                $Log->comment(
                    "Selecting compression : $args{'-volcompression'}");
                $self->set_checkbox(
                    "$VOLUME_EDIT_COMPRESSION_CHECKBOX",
                    lc( $args{-volcompression} )
                );
                if ( exists( $args{'-volcompressioninline'} ) ) {
                    if (   ( lc( $args{-volcompressioninline} ) eq 'enable' )
                        || ( lc( $args{-volcompressioninline} ) eq 'disable' ) )
                    {
                        $Log->comment(
"Selecting compression inline : $args{'-volcompressioninline'}"
                        );
                        $self->click(
                            "$VOLUME_EDIT_COMPRESSION_INLINE_CHECKBOX");
                    }
                    else {
                        NATE::BaseException->throw(
"Invalid value for Compression Inline, valid values are 'enable' or 'disable'"
                        );
                    }
                } ## end if ( exists( $args{'-volcompressioninline'...}))
            }
            else {
                NATE::BaseException->throw(
"Invalid value for Compression, valid values are 'enable' or 'disable'"
                );
            }
        } ## end if ( exists( $args{'-volcompression'...}))
    }
    elsif ( lc( $args{-dedup} ) eq 'disable' ) {
        $self->click("$VOLUME_EDIT_EFFICIENCY_TAB");
        $self->wait_for_loading();
        $Log->comment("Click on storage efficiency tab ");
        $Log->comment("*** Click to select Vol dedupe $args{-dedup} ***");
        $self->set_checkbox( $VOLUME_EDIT_DEDUPE_CHECKBOX, "disable" );
    } ## end elsif ( lc( $args{-dedup}...))

    $Log->comment("*** Click OK button ***");
    my $commit_time = $self->click_timecount($VOLUME_EDIT_OK_BUTTON);
    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . " and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Volume $args{'-volname'} edited successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub volume_edit

################################################################################
## @name ::clone_volume
## @summary
##     Clone a Volume using GUI.
## @description
##     This method can be used to Clone a volume using GUI.
## @synopsis
##     clone_volume(%args);
## INPUTS TO METHOD CALL 'clone_volume' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the volume to be Cloned </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume to be cloned is a part of </td> </tr>
## <tr> <td> -clonename </td> <td> Name of the Clone </td> </tr>
## <tr> <td> -type </td> <td> Type of Clone to be Created </td> </tr>
## <tr> <td> -colnespaceres </td> <td> Type of Space Reserve for this Clone </td> </tr>
## <tr> <td> -parentcopy </td> <td> Type of Parent Copy for this Clone </td> </tr>
## <tr> <td> -parentname </td> <td> Parent Name </td> </tr>
## <tr> <td> -filepath </td> <td> Selected Path for Clone </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub clone_volume {

    my ($commit_time);
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-volname' => { type => SCALAR },
            "-type"    => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );

    $self->wait_for_element_present("$VOLUME_GRID");
    $self->wait_for_loading();
    $Log->comment("*** Select volume $args{-volname} ***");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{-volname} );
    $Log->comment("Click Clone Button");
    $self->click_button("$VOLUME_CLONE_BUTTON");
    $self->wait_for_element_present("$VOLUME_CREATECLONE_BUTTON");

    if ( lc( $args{-type} ) eq 'volume' ) {
        $self->mouse_over($VOLUME_CREATECLONE_BUTTON);
        $self->mouse_over($VOLUME_CLONE_CREATE_VOLUME_BUTTON);
        $self->click_button($VOLUME_CLONE_CREATE_VOLUME_BUTTON);
        $self->wait_for_loading();

        $Log->comment("*** Enter clone name ***");

        $self->type( "$VOLUME_CLONE_CREATE_NAME_INPUT", $args{-clonename} );
        if ( exists( $args{-colnespaceres} ) ) {
            if ( lc( $args{-colnespaceres} ) =~ /none/ ) {
                $Log->comment("*** Selecting space guarantee for clone ***");
                $self->click("$VOLUME_CLONE_CREATE_TYPE_INPUT");
            }

        } ## end if ( exists( $args{-colnespaceres...}))

        if ( exists( $args{-parentcopy} ) ) {
            if ( lc( $args{-parentcopy} ) eq 'new' ) {
                $Log->comment("Selecting new snapshot for clone ");
                $self->set_option( "$VOLUME_CLONE_CREATE_NEW_RADIO", "enable" );

            }
            elsif ( lc( $args{-parentcopy} ) eq 'existing' ) {
                $Log->comment("*** Selecting existing snapshot for clone ***");
                $self->set_option( "$VOLUME_CLONE_CREATE_EXISTING_RADIO",
                    "enable" );
                $self->click_row_with_texts( "$VOLUME_CLONE_GRID",
                    $args{-parentname} );
            } ## end elsif ( lc( $args{-parentcopy...}))
        } ## end if ( exists( $args{-parentcopy...}))
        $Log->comment("*** Click ok for clone creation ***");

        $self->click($VOLUME_CLONE_OK_BUTTON);
        if ( $self->is_element_present($VOLUME_CLONE_CONFIRMATION_BUTTON) ) {
            $Log->comment("Click the confirmation dialog button");
            $self->click($VOLUME_CLONE_CONFIRMATION_BUTTON);
        }
    } ## end if ( lc( $args{-type} ...))

    if ( lc( $args{-type} ) eq 'file' ) {
        $self->mouse_over("$VOLUME_CREATECLONE_BUTTON");
        $self->mouse_over($VOLUME_CLONE_CREATE_FILE_BUTTON);
        $self->click_button("$VOLUME_CLONE_CREATE_FILE_BUTTON");
        $self->wait_for_element_present("$VOLUME_CLONE_CREATE_FILE_NAME_INPUT");
        if ( exists( $args{-filepath} ) ) {
            $Log->comment("*** Input the value for filepath ***");
            $self->type( "$VOLUME_CLONE_CREATE_FILE_PATH_INPUT",
                $args{-filepath} );
        }
        if ( exists( $args{-clonename} ) ) {
            $Log->comment("*** Enter the clone name:$args{-clonename} ***");
            $self->type( "$VOLUME_CLONE_CREATE_FILE_NAME_INPUT",
                $args{-clonename} );
        }
        $Log->comment("*** Enter ok for create clone ***");
        $commit_time =
          $self->click_timecount($VOLUME_CLONE_CREATE_FILE_OK_BUTTON);

    } ## end if ( lc( $args{-type} ...))

    my %times = ( loading_time => $loading_time );

    $Log->comment("Clone of volume created successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub clone_volume

################################################################################
## @name ::clone_split
## @summary
##     Split Clone using GUI.
## @description
##     This method can be used to Split clone using GUI.
## @synopsis
##     clone_split(%args);
## INPUTS TO METHOD CALL 'clone_split' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver </td> </tr>
## <tr> <td> -clonename </td> <td> Name of the Clone to be Split </td> </tr>
## <tr> <td> -clonesplit </td> <td> Type of the Clone Split </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub clone_split {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-clonename" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $clonename = $args{'-clonename'};
    my $commit_time;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_element_present($VOLUME_GRID);
    $Log->comment("*** Select volume $clonename ***");
    $self->click_row_with_texts( $VOLUME_GRID, $clonename );
    $self->wait_for_element_present("$VOLUME_CLONE_BUTTON");

    $Log->comment("*** Click Clone Split Button ***");
    $self->click($VOLUME_CLONE_BUTTON);

    #$self->wait_for_element_present($VOLUME_SPLIT_CLONE_BUTTON);

    sleep(3);
    if ( not $self->is_button_enabled($VOLUME_SPLIT_CLONE_BUTTON) ) {
        NATE::BaseException->throw("Split button is not enabled");
    }

    $self->click("$VOLUME_SPLIT_CLONE_BUTTON");
    $self->check_for_error();

    #$self->wait_for_loading();

    #wait_for_loading doesn't work here so using sleep.
    sleep(10);

    if ( exists( $args{-clonesplit} ) ) {
        if ( lc( $args{-clonesplit} ) eq 'split' ) {
            $Log->comment("Selecting split confirm for $clonename");
            $self->wait_for_element_present("$VOLUME_SPLIT_CONFIRM_CHECKBOX");
            $self->set_checkbox( "$VOLUME_SPLIT_CONFIRM_CHECKBOX", "enable" );
            $commit_time =
              $self->click_timecount("$VOLUME_SPLIT_CLONE_OK_BUTTON");
        }
        elsif ( lc( $args{-clonesplit} ) eq 'no' ) {
            $Log->comment("Selecting split cancel for $clonename");
            $self->click_button("$VOLUME_SPLIT_CLONE_CANCEL_BUTTON");
        }
    } ## end if ( exists( $args{-clonesplit...}))

    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . " and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_element_not_present( $VOLUME_SPLIT_CLONE_OK_BUTTON,
        $GENERIC_WAIT );
    $Log->comment("Clone split done successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub clone_split

################################################################################
## @name ::volume_status
## @summary
##     Set Volume Status using GUI.
## @description
##     This method can be used to Online/Offline/Restrict the volume using GUI.
## @synopsis
##     volume_status(%args);
## INPUTS TO METHOD CALL 'volume_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which Status has to be set </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume is part of </td> </tr>
## <tr> <td> -status </td> <td> Type of the Status to be set to the Volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub volume_status {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            '-volname' => { type => SCALAR },
            "-status"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $commit_time;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_element_present("$VOLUME_GRID");
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( $VOLUME_GRID, $args{'-volname'} );
    $self->wait_for_loading();

    if ( exists( $args{-status} ) ) {
        $Log->comment("Click Status Button");
        $self->click("$VOLUME_STATUS_BUTTON");
        $self->wait_for_loading();
        if ( lc( $args{-status} ) eq "online" ) {
            if ( $self->is_button_enabled("$VOLUME_ONLINE_STATUS_BUTTON") ) {
                $Log->comment("Click online Status Button");
                $commit_time =
                  $self->click_timecount("$VOLUME_ONLINE_STATUS_BUTTON");
            }
            else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw("online button is not enabled");
            }
        }
        elsif ( lc( $args{-status} ) eq "offline" ) {
            #if ( $self->is_button_enabled("$VOLUME_OFFLINE_STATUS_BUTTON") ) {
                $self->click("$VOLUME_OFFLINE_STATUS_BUTTON");
                $self->wait_for_element_present( "$VOLUME_STATUS_OK_BUTTON",
                    $GENERIC_WAIT );
                $Log->comment("Click confirm OK Button");
                $commit_time = $self->click_timecount("$VOLUME_STATUS_OK_BUTTON");
            #}
            #else {
            #    $Log->exit() if $may_exit;
            #    NATE::BaseException->throw("offline button is not enabled");
            #}
        }
        elsif ( lc( $args{-status} ) eq "restrict" ) {
            if ( $self->is_button_enabled("$VOLUME_RESTRICT_STATUS_BUTTON") ) {
                $Log->comment("Click restrict Status Button");
                $self->click("$VOLUME_RESTRICT_STATUS_BUTTON");
                $self->wait_for_element_present( "$VOLUME_STATUS_OK_BUTTON",
                    $GENERIC_WAIT );
                $Log->comment("Click conform Button");
                $commit_time = $self->click_timecount("$VOLUME_STATUS_OK_BUTTON");
            }
            else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw("restrict button is not enabled");
            }
        } ## end elsif ( lc( $args{-status...}))
    } ## end if ( exists( $args{-status...}))
    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . " and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("Volume status displayed successfully");
    return \%times;
} ## end sub volume_status

################################################################################
## @name ::volume_resize
## @summary
##     Resize the Volume Size using GUI.
## @description
##     This method can be used to Resize the volume size using GUI.
## @synopsis
##     volume_resize(%args);
## INPUTS TO METHOD CALL 'volume_resize' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume which has to be resized </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume resides </td> </tr>
## <tr> <td> -resize </td> <td> The Resize value to be set for the Volume </td> </tr>
## <tr> <td> -sizeunit </td> <td> The Resize Unit to be set for the Volume </td> </tr>
## <tr> <td> -volreserv </td> <td> The Value to be set for Snapshot Reserve for this Volume </td> </tr>
## <tr> <td> -snapshot </td> <td> Type of Snapshot for this Volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub volume_resize {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my ($commit_time);
    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};

    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );

    $Log->comment("Select resize wizard");
    $self->click_button("$VOLUME_RESIZE_BUTTON");

    $self->wait_for_element_present( $VOLUME_RESIZE_NEXT_BUTTON,
        $GENERIC_WAIT );
    $self->click_button("$VOLUME_RESIZE_NEXT_BUTTON");
    $self->wait_for_element_present( $VOLUME_RESIZE_CAPACITY_INPUT,
        $GENERIC_WAIT );

    if ( exists( $args{-resize} ) ) {
        $Log->comment("Entering new size : $args{-resize}");
        $self->type( "$VOLUME_RESIZE_CAPACITY_INPUT", $args{-resize} );
        $self->fire_event( "$VOLUME_RESIZE_CAPACITY_INPUT", "blur" );
    }

    if ( uc( $args{-sizeunit} ) =~ /[GB|TB|PB|MB]/ ) {
        $Log->comment("Entering size unit :$args{-sizeunit}");
        $self->set_combobox_value( "$VOLUME_RESIZE_SIZEUNIT_COMBOBOX",
            uc( $args{-sizeunit} ) );
    }

    if ( exists( $args{-volreserv} ) ) {
        $Log->comment("Selecting space guarantee for vol: $args{-volreserv}");
        $self->type( "$VOLUME_RESIZE_RESERVE_BUTTON", $args{-volreserv} );
    }

    $self->click_button("$VOLUME_RESIZE_NEXT_BUTTON");

    if ( lc( $args{-snapshot} ) eq 'selectall' ) {
        $Log->comment("Select all the snapshot for delete");
        $self->click("$VOLUME_RESIZE_SNAPHOT_SELECTALL_BUTTON");
    }
    elsif ( lc( $args{-snapshot} ) eq 'deselectall' ) {
        $Log->comment("Deselect all the snapshot from delete");
        $self->click("$VOLUME_RESIZE_SNAPHOT_DESELECTALL_BUTTON");
    }

    $self->click_button("$VOLUME_RESIZE_NEXT_BUTTON");

    $Log->comment("Selecting next button");
    $self->click_button("$VOLUME_RESIZE_NEXT_BUTTON");
    $self->wait_for_loading();
    $Log->comment("Selecting finish button");
    $self->click_button("$VOLUME_RESIZE_FINISH_BUTTON");

    $self->wait_for_element_not_present( $VOLUME_RESIZE_FINISH_BUTTON,
        $GENERIC_WAIT );
    $self->wait_for_loading();
    $self->check_for_error();

    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . "and $commit_time sec !" );
    my %times = ( loading_time => $loading_time );

    $Log->comment("Volume resize done successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub volume_resize

################################################################################
## @name ::is_clone_enabled
## @summary
##     To verify Clone Enable on Volume using GUI.
## @description
##     This method can be used to Verify the Clone Enabled on Volume or not using GUI.
## @synopsis
##     is_clone_enabled(%args);
## INPUTS TO METHOD CALL 'is_clone_enabled' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume on which Clone enabling has to be verified </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume resides </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub is_clone_enabled {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );

    $Log->comment("Click Clone Button");
    $self->click_button("$VOLUME_CLONE_BUTTON");
    $self->wait_for_element_present("$VOLUME_CREATECLONE_BUTTON");
    $self->mouse_over("$VOLUME_CREATECLONE_BUTTON");

    $Log->comment("Check clone menu item is enabled or not");
    if ( $self->is_button_enabled("$VOLUME_CREATECLONE_BUTTON") ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Clone Button should not be enabled for offline volume");
    }
    else {
        $Log->comment("PASSED: Clone button is disabled for offline volume");
    }

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;
    return \%times;

} ## end sub is_clone_enabled

################################################################################
## @name ::is_split_enabled
## @summary
##     To verify Split Enable on Volume using GUI.
## @description
##     This method can be used to Verify the Split Enabled on Volume or not using GUI.
## @synopsis
##     is_split_enabled(%args);
## INPUTS TO METHOD CALL 'is_split_enabled' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume on which Split enabling has to be verified </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume resides </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub is_split_enabled {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );

    $Log->comment("Click Clone Button");
    $self->click_button("$VOLUME_CLONE_BUTTON");
    $self->wait_for_element_present("$VOLUME_SPLIT_CLONE_BUTTON");
    $self->mouse_over("$VOLUME_SPLIT_CLONE_BUTTON");

    $Log->comment("Check split menu item is enabled or not");
    if ( $self->is_button_enabled("$VOLUME_SPLIT_CLONE_BUTTON") ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Split Button should not be enabled for normal volume");
    }
    else {
        $Log->comment("PASSED: Split button is disabled for normal volume");
    }
    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub is_split_enabled

################################################################################
## @name ::is_delete_enabled
## @summary
##     To verify Delete Enable on Volume using GUI.
## @description
##     This method can be used to Verify the Delete Enabled on Volume or not using GUI.
## @synopsis
##     is_delete_enabled(%args);
## INPUTS TO METHOD CALL 'is_delete_enabled' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume on which Delete enabling has to be verified </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume resides </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub is_delete_enabled {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );

    $Log->comment("Check delete button is enabled or not");
    if ( $self->is_button_enabled("$VOLUME_DELETE_BUTTON") ) {
        $Log->comment(
            "FAILED: Delete button is enabled for volume which has clone");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "Delete Button should not be enabled for volume which has clone");
    }
    else {
        $Log->comment("PASSED: Delete button is disabled on volume with clone");
    }
    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub is_delete_enabled

################################################################################
## @name ::modify_volume_status
## @summary
##     To Modify Status of Multiple Volumes using GUI.
## @description
##     This method can be used to Modify Status of Multiple Volumes using GUI.
## @synopsis
##     modify_volume_status(%args);
## INPUTS TO METHOD CALL 'modify_volume_status' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volnames </td> <td> Name of the Volumes for which Status has to be set </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver in which the volume resides </td> </tr>
## <tr> <td> -status </td> <td> The Status that has to be set for all the Volumes </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub modify_volume_status {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volnames" => { type => ARRAYREF }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();

    $Log->comment("Selecting volumes");
    for my $count ( 0 .. $#{ $args{-volnames} } ) {
        my $volname = $args{-volnames}[$count];
        $Log->step("Selecting volume : $volname");
        $self->click_row_with_texts( $VOLUME_GRID, $volname );
        $self->control_key_down() if ( $count == 0 );
    } ## end for my $count ( 0 .. $#...)
    $self->control_key_up();

    if ( lc( $args{-status} ) eq 'online' ) {
        $Log->comment("Click Status Button");
        $self->click("$VOLUME_STATUS_BUTTON");
        if ( $self->is_button_enabled("$VOLUME_ONLINE_STATUS_BUTTON") ) {
            $Log->comment("Click online Status Button");
            $self->click("$VOLUME_ONLINE_STATUS_BUTTON");
            $self->wait_for_element_present( $VOLUME_STATUS_OK_BUTTON,
                $GENERIC_WAIT );
            $Log->comment("Click confirm Button");
            $self->click("$VOLUME_STATUS_OK_BUTTON");
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("online button is not enabled");
        }
    } ## end if ( lc( $args{-status...}))

    if ( lc( $args{-status} ) eq 'offline' ) {
        $Log->comment("Click Status Button");
        $self->click("$VOLUME_STATUS_BUTTON");
        if ( $self->is_button_enabled("$VOLUME_OFFLINE_STATUS_BUTTON") ) {
            $Log->comment("Click offline Status Button");
            $self->click("$VOLUME_OFFLINE_STATUS_BUTTON");
            $self->wait_for_element_present( $VOLUME_STATUS_OK_BUTTON,
                $GENERIC_WAIT );
            $Log->comment("Click confirm Button");
            $self->click("$VOLUME_STATUS_OK_BUTTON");
        }
        else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("offline button is not enabled");
        }
    } ## end if ( lc( $args{-status...}))

    $Log->comment("SUCCESS: Updated volume status");
    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub modify_volume_status

################################################################################
## @name ::delete_volume
## @summary
##     To verify Delete Volume using GUI.
## @description
##     This method can be used to Delete Volume using GUI.
## @synopsis
##     delete_volume(%args);
## INPUTS TO METHOD CALL 'delete_volume' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volnames </td> <td> Name of the Volumes to be deleted </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time

sub delete_volume {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volnames" => { type => ARRAYREF }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    my ( %times, $commit_time, $loading_time );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    $loading_time = $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();

    $Log->comment("Selecting volumes");
    for my $count ( 0 .. $#{ $args{-volnames} } ) {
        my $volname = $args{-volnames}[$count];
        $Log->step("Selecting volume : $volname");
        $self->click_row_with_texts( $VOLUME_GRID, $volname );

        #        $self->click_row_with_texts( $VOLUME_GRID, 1,0,$volname );
        $self->control_key_down() if ( $count == 0 );
    } ## end for my $count ( 0 .. $#...)
    $self->control_key_up();

    $Log->comment("Click Delete button");
    $self->click("$VOLUME_DELETE_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Selecting Confirm Checkbox");
    $self->set_checkbox( $VOLUME_DELETE_CONFIRM_CHECKBOX, "enable" );

    $Log->comment("Selecting Delete button");

    $commit_time = $self->click_timecount($VOLUME_DELETE_DELETE_BUTTON);
    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . "and $commit_time sec !" );
    %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->comment("Deletion of volume(s) done");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub delete_volume

################################################################################
## @name ::create_volsnap
## @summary
##     To Create Volume Snapshot using GUI.
## @description
##     This method can be used to Create Volume Snapshot using GUI.
## @synopsis
##     create_volsnap(%args);
## INPUTS TO METHOD CALL 'create_volsnap' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which Snapshot has to be created </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver which has the volume </td> </tr>
## <tr> <td> -snapname </td> <td> Snapshot Name that has to be created </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time

sub create_volsnap {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );
    $self->wait_for_loading();
    $Log->comment("***Click Create Snapshot Button***");
    $self->click_button("$VOLUME_SNAPSHOT_BUTTON");
    $self->click("$VOLUME_CREATE_SNAPSHOT_BUTTON");
    $self->wait_for_element_present( "$VOLUME_CREATE_SNAPSHOT_NAME_INPUT",
        $GENERIC_WAIT );

    if ( exists( $args{-snapname} ) ) {
        $Log->comment("*** Type snapshot Name :$args{-snapname} ****");
        $self->type( "$VOLUME_CREATE_SNAPSHOT_NAME_INPUT", $args{-snapname} );
    }
    $Log->comment("***Click Create OK Button***");

    my $commit_time =
      $self->click_timecount($VOLUME_CREATE_SNAPSHOT_NAMECREATE_BUTTON);
    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . " and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("volume snapshot created successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub create_volsnap

################################################################################
## @name ::manage_volsnap
## @summary
##     To manage Volume Snapshot using GUI.
## @description
##     This method can be used to manage Volume Snapshot using GUI.
## @synopsis
##     manage_volsnap(%args);
## INPUTS TO METHOD CALL 'manage_volsnap' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which Snapshot has to be managed </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the vserver which has the volume </td> </tr>
## <tr> <td> -option </td> <td> Type of option to manage the Snapshot </td> </tr>
## <tr> <td> -snapres </td> <td> Value for Snapshot Reserve </td> </tr>
## <tr> <td> -dirshow </td> <td> Enable/Disable Directory Show </td> </tr>
## <tr> <td> -snapschedule </td> <td> Snapshot Schedule Value </td> </tr>
## <tr> <td> -policy </td> <td> Snapshot policy name </td> </tr>
## <tr> <td> -snapname </td> <td> Name of Snapshot </td> </tr>
## <tr> <td> -snaprename </td> <td> Rename of Snapshot </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time

sub manage_volsnap {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my ( $commit_time, $loading_time );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    $loading_time = $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );
    $self->wait_for_loading();

    if ( lc( $args{-option} ) eq 'config' ) {
        $Log->comment("***Click Create config Button***");
        $self->click_button("$VOLUME_SNAPSHOT_BUTTON");
        $self->click_button("$VOLUME_CONFIGURE_SNAPSHOT_BUTTON");
        $self->wait_for_loading();

        if ( exists( $args{-snapres} ) ) {
            $Log->comment("*** Enter the value for sanpshot reserve ***");
            $self->type( "$VOLUME_CONFIGURE_SNAPSHOT_SNAPRES_COMBOBOX",
                $args{-snapres} );
        }

        if ( exists( $args{-dirshow} ) ) {
            $Log->comment("*** Select to show snapshot dir ***");
            $self->set_checkbox(
                "$VOLUME_CONFIGURE_SNAPSHOT_DIRVISIBLE_CHECKBOX",
                $args{-dirshow} );
        } ## end if ( exists( $args{-dirshow...}))
        if ( exists( $args{-snapschedule} ) ) {
            $Log->comment("*** Enter the value for sanpshot schedule ***");
            $self->set_checkbox( "$VOLUME_CONFIGURE_SNAPSHOT_SCHEDULE_CHECKBOX",
                $args{-snapschedule} );

            if ( exists( $args{-policy} ) ) {
                $Log->comment("Selecting snapshot policy :$args{-policy}");
                $self->select_from_dropdown(
                    "$VOLUME_CONFIGURE_SNAPSHOT_SCHEDULE_POLICY_COMBOBOX",
                    $args{-policy}, 0 );
            } ## end if ( exists( $args{-policy...}))
        } ## end if ( exists( $args{-snapschedule...}))

        $Log->comment("*** Select configure button to save settings ***");
        $commit_time =
          $self->click_timecount($VOLUME_CONFIGURE_SNAPSHOT_OK_BUTTON);
    } ## end if ( lc( $args{-option...}))

    if ( lc( $args{-option} ) eq 'restore' ) {
        $Log->comment("***Click snapshot restore in schedule snapshot***");
        $self->click_button("$VOLUME_SNAPSHOT_BUTTON");
        $self->wait_for_loading();
        $self->click_button("$VOLUME_RESTORE_SNAPSHOT_BUTTON");
        sleep 90;
        $self->wait_for_loading();

        if ( exists( $args{-snapname} ) ) {
            $Log->comment(
                "***Select the snapshot from grid:$args{-snapname} ***");
            $self->click_row_with_texts( "$VOLUME_RESTORE_SNAPSHOT_GRID",
                $args{-snapname} );
            $self->wait_for_loading();
            $Log->comment("***Select the snapshot restore ok button ***");
            $self->click_button("$VOLUME_RESTORE_SNAPSHOT_OK_BUTTON");
            $self->wait_for_loading();

            $Log->comment(
                "***Select the snapshot restore conform checkbox ***");
            $self->set_checkbox( "$VOLUME_RESTORE_SNAPSHOT_CONF_CHECKBOX",
                "enable" );
            $Log->comment(
                "***Select the snapshot restore conform ok button ***");
            $commit_time =
              $self->click_timecount($VOLUME_RESTORE_SNAPSHOT_CONF_OK_BUTTON);
        }
    }
    if ( lc( $args{-option} ) eq 'rename' ) {
        $Log->comment("Renaming snapshot");
        $self->click("$VOLUME_SNAPSHOT_TAB");
        $self->wait_for_loading();
        $self->click_row_with_texts( "$VOLUME_SNAPSHOT_TAB_GRID",
            $args{'-snapname'} );
        $self->wait_for_loading();
        $Log->comment("***Click snapshot rename Button***");
        $self->click_button("$VOLUME_SNAPSHOT_RENAME_BUTTON");
        $self->wait_for_element_present( "$VOLUME_SNAPSHOT_RENAME_NAME_INPUT",
            $GENERIC_WAIT );
        $self->type( "$VOLUME_SNAPSHOT_RENAME_NAME_INPUT", $args{-snaprename} );
        $Log->comment("***Click Snapshot rename OK Button***");
        $commit_time =
          $self->click_timecount($VOLUME_SNAPSHOT_RENAME_OK_BUTTON);
        $Log->comment("***Click details tab Button***");
        $self->click("$VOLUME_DETAIL_TAB");

    }
    if ( lc( $args{-option} ) eq 'delete' ) {
        $Log->comment("Deleting snapshot");
        $self->click("$VOLUME_SNAPSHOT_TAB");
        $self->wait_for_loading();
        $self->click_row_with_texts( "$VOLUME_SNAPSHOT_TAB_GRID",
            $args{'-snapname'} );
        $self->wait_for_loading();
        $Log->comment("***Click delete Button***");
        $self->click_button("$VOLUME_SNAPSHOT_DELETE_BUTTON");
        $self->wait_for_element_present( "$VOLUME_SNAPSHOT_DELETE_CONF_BUTTON",
            $GENERIC_WAIT );
        $self->set_checkbox( "$VOLUME_SNAPSHOT_DELETE_CONF_BUTTON", "enable" );
        $Log->comment("***Click Snapshot delete OK Button***");
        $commit_time =
          $self->click_timecount($VOLUME_SNAPSHOT_DELETE_OK_BUTTON);
        $Log->comment("***Click details tab Button***");
        $self->click("$VOLUME_DETAIL_TAB");
    }

    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . "and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("snapshot updation done successfully");
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub manage_volsnap

################################################################################
## @name ::volume_dedupe
## @summary
##     To Create Volume Deduplication using GUI.
## @description
##     This method can be used to Create Volume Deduplication using GUI.
## @synopsis
##     volume_dedupe(%args);
## INPUTS TO METHOD CALL 'volume_dedupe' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which Deduplication has to be created </td> </tr>
## <tr> <td> -vserver </td> <td> Name of the vserver which has volume </td> </tr>
## <tr> <td> -state </td> <td> State of Deduplication </td> </tr>
## <tr> <td> -type </td> <td> Type of Deduplication </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time

sub volume_dedupe {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");
    $self->wait_for_loading();

    $Log->comment("Select volume $args{'-volname'}");
    $self->wait_for_element_present( "$VOLUME_GRID", $GENERIC_WAIT );
    $self->click_row_with_texts( "$VOLUME_GRID", $args{-volname} );
    if ( exists( $args{-state} ) ) {
        if ( lc( $args{-state} ) eq 'start' ) {
            $Log->comment("Selecting Volume dudupe: Start");
            if ( lc( $args{-type} ) eq 'partial' ) {
                $self->click_button("$VOLUME_DEDUPE_BUTTON");
                $Log->comment("Selecting Scan partial volume");
                $self->set_checkbox( $VOLUME_DEDUPE_SCAN_CHECKBOX, "disable" );
                $self->click("$VOLUME_DEDUPE_START_BUTTON");
            }
            elsif ( lc( $args{-type} ) eq 'full' ) {
                $self->click_button("$VOLUME_DEDUPE_BUTTON");
                $self->wait_for_loading();
                $Log->comment("Selecting Scan entire volume");
                $self->set_checkbox( $VOLUME_DEDUPE_SCAN_CHECKBOX, "enable" );
                $self->click("$VOLUME_DEDUPE_START_BUTTON");
                $self->wait_for_loading();
            } ## end elsif ( lc( $args{-type} ...))
        }
        elsif ( lc( $args{-state} ) eq 'abort' ) {
            $Log->comment("Deselecting Volume dudupe");
            $self->click_button("$VOLUME_DEDUPE_BUTTON");
            $self->wait_for_element_present( $VOLUME_DEDUPE_ABORT_BUTTON,
                $GENERIC_WAIT );
            $self->click_button("$VOLUME_DEDUPE_ABORT_BUTTON");
        } ## end elsif ( lc( $args{-state}...))
    } ## end if ( exists( $args{-state...}))

    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub volume_dedupe

################################################################################
## @name ::get_volume_info
## @summary
##     To Fetch Volume Details from GUI.
## @description
##     This method can be used to Fetch Volume Details using GUI.
## @synopsis
##     get_volume_info(%args);
## INPUTS TO METHOD CALL 'get_volume_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which Details has to be fetched </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver which has the volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Volume status details.

sub get_volume_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );
    my (%values);
    my $volume_info = {};

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );

    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");

    $self->wait_for_element_present("$VOLUME_GRID");
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{-volname} );
    $self->wait_for_loading();

    $Log->step("Getting row data for volume from grid");
    my $rowdata =
      $self->get_row_data_with_text( "$VOLUME_GRID", $args{-volname} );

    my @rowarray = split( ' ', $rowdata );

    %values = (
        "$VOLUME_INFO_AGGR_KEY"           => $rowarray[1],
        "$VOLUME_INFO_THINPROVISION_KEY"  => $rowarray[3],
        "$VOLUME_INFO_PERCENTAGEUSED_KEY" => $rowarray[4],
    );
    $Log->comment("***Click details tab Button***");
    $self->click("$VOLUME_PANEL_DETAILS_TAB");
    $self->wait_for_loading();
    sleep(10);
    $Log->comment("Getting volume details");
    $values{"$VOLUME_DETAIL_STATUS_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_STATUS")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_STATUS") );
    $values{"$VOLUME_DETAIL_MAXFILE_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_MAXFILE")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_MAXFILE") );
    $values{"$VOLUME_DETAIL_CURFILE_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_CURFILE")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_CURFILE") );

    $values{"$VOLUME_DETAIL_CREATEUNICODE_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_CREATEUNICODE")
      if ( $self->is_element_present($VOLUME_DETAILTAB_CREATEUNICODE) );

    $values{"$VOLUME_DETAIL_CLONE_PARENT_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_CLONE_PARENT")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_CLONE_PARENT") );

    $values{"$VOLUME_DETAIL_CLONE_CHILD_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_CLONE_CHILD")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_CLONE_CHILD") );

    $values{"$VOLUME_DETAIL_SPLITSTATUS_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_SPLITSTATUS")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_SPLITSTATUS") );

    $values{"$VOLUME_DETAIL_AUTOMAXIMUM_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_AUTOMAXIMUM")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_AUTOMAXIMUM") );

    $values{"$VOLUME_DETAIL_AUTOINCREM_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_AUTOINCREM")
      if ( $self->is_element_present($VOLUME_DETAILTAB_AUTOINCREM) );

    $values{"$VOLUME_DETAIL_SNAP_AUTODEL_KEY"} =
      $self->get_text("$VOLUME_DETAILTAB_SNAP_AUTODEL")
      if ( $self->is_element_present("$VOLUME_DETAILTAB_SNAP_AUTODEL") );

    $Log->comment("Clicking in the Space allocation tab");
    $self->click("$VOLUME_SPACEALLOC_TAB");

    $values{"$VOLUME_SPACEALLOC_TOTAL_SPACE_KEY"} =
      $self->get_text("$VOLUME_SPACEALLOC_TOTAL_SPACE")
      if ( $self->is_element_present("$VOLUME_SPACEALLOC_TOTAL_SPACE") );

    $values{"$VOLUME_SPACEALLOC_SNAPSHOT_KEY"} =
      $self->get_text("$VOLUME_SPACEALLOC_SNAPSHOT")
      if ( $self->is_element_present("$VOLUME_SPACEALLOC_SNAPSHOT") );

    $Log->comment("Collection of volume info is successfull");
    $volume_info->{parsed_output} = \%values;
    $volume_info->{loading_time}  = $loading_time;

    $Log->exit() if $may_exit;

    return $volume_info;
} ## end sub get_volume_info

################################################################################
## @name ::is_snapshot_created
## @summary
##     Verify Snapshot Creation from GUI.
## @description
##     This method can be used to Verify Snapshot Creation using GUI.
## @synopsis
##     is_snapshot_created(%args);
## INPUTS TO METHOD CALL 'is_snapshot_created' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which particular snapshot has to be Verified </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver which has the volume </td> </tr>
## <tr> <td> -snapname </td> <td> Name of the Snapshot for which creation has to be Verified </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub is_snapshot_created {
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-volname"  => { type => SCALAR },
            "-snapname" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    my $snapname    = $args{-snapname};
    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");
    $self->wait_for_element_present( $VOLUME_GRID, $GENERIC_WAIT );
    $Log->comment("Select volume $args{'-volname'}");
    $self->click_row_with_texts( "$VOLUME_GRID", $args{'-volname'} );
    $self->click("$VOLUME_SNAPSHOT_TAB");
    $self->wait_for_loading();

    if ( $self->does_row_contain_texts(
            $VOLUME_SNAPSHOT_TAB_GRID, $snapname ) ) {
        $Log->comment("SUCCESS: $snapname is created.");
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("$snapname has not been created");
    }

    $Log->comment("Loading time is $loading_time sec !");
    my %times = ( loading_time => $loading_time );
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub is_snapshot_created

################################################################################
## @name ::vol_storageeff_info
## @summary
##     To Fetch Volume Storage Efficiency Details from GUI.
## @description
##     This method can be used to Fetch Volume Storage Efficiency Details using GUI.
## @synopsis
##     vol_storageeff_info(%args);
## INPUTS TO METHOD CALL 'vol_storageeff_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which info has to be fetched </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver which has the volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Volume storage efficiency details.

sub vol_storageeff_info {
    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-volname" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my %values;
    my $efficiency_info = {};
    my $vserverName     = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );

    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");
    $self->wait_for_element_present( $VOLUME_GRID, $GENERIC_WAIT );
    $Log->comment("*** Select volume $args{'-volname'} ***");
    $self->click_row_with_texts( $VOLUME_GRID, $args{'-volname'} );

    $Log->comment("Navigating to Storage>volume>Stotage efficiency tab");
    $self->click("$VOLUME_EFFITAB_TAB");

    $self->wait_for_loading();

    $values{"$VOLUME_EFFI_ENABLE_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_ENABLE")
      if ( $self->is_element_present("$VOLUME_EFFITAB_ENABLE") );

    $values{"$VOLUME_EFFI_MODE_KEY"} = $self->get_text("$VOLUME_EFFITAB_MODE")
      if ( $self->is_element_present("$VOLUME_EFFITAB_MODE") );

    $values{"$VOLUME_EFFI_STATUS_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_STATUS")
      if ( $self->is_element_present("$VOLUME_EFFITAB_STATUS") );

    $values{"$VOLUME_EFFI_TYPE_KEY"} = $self->get_text("$VOLUME_EFFITAB_TYPE")
      if ( $self->is_element_present("$VOLUME_EFFITAB_TYPE") );

    $values{"$VOLUME_EFFI_SPACESAVED_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_SPACESAVED")
      if ( $self->is_element_present("$VOLUME_EFFITAB_SPACESAVED") );

    $values{"$VOLUME_EFFI_SAVEDPERCENT_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_SAVEDPERCENT")
      if ( $self->is_element_present("$VOLUME_EFFITAB_SAVEDPERCENT") );

    $values{"$VOLUME_EFFI_STARTTIME_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_STARTTIME")
      if ( $self->is_element_present("$VOLUME_EFFITAB_STARTTIME") );

    $values{"$VOLUME_EFFI_ENDTIME_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_ENDTIME")
      if ( $self->is_element_present("$VOLUME_EFFITAB_ENDTIME") );
    $values{"$VOLUME_EFFI_COMPRESSION_KEY"} =
      $self->get_text("$VOLUME_EFFITAB_COMPRESSION")
      if ( $self->is_element_present("$VOLUME_EFFITAB_COMPRESSION") );

    $self->check_for_error();

    $Log->comment("Collection of storage efficiency info is successfull");
    $efficiency_info->{parsed_output} = \%values;
    $efficiency_info->{loading_time}  = $loading_time;
    $Log->exit() if $may_exit;
    return $efficiency_info;

} ## end sub vol_storageeff_info

################################################################################
## @name ::refresh_volume
## @summary
##     To Refresh Volume Page in GUI.
## @description
##     This method can be used to Refresh Volume page using GUI.
## @synopsis
##     vol_storageeff_info(%args);
## INPUTS TO METHOD CALL 'vol_storageeff_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume for which info has to be fetched </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver which has the volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub refresh_volume {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $self->wait_for_loading();
    unless ( $self->is_button_enabled($VOLUME_REFRESH_BUTTON) ) {
        sleep(10);
    }
    $Log->comment("*** Refresh Volume Table ******");
    my $commit_time =
      $self->click_timecount( $VOLUME_REFRESH_BUTTON, "refresh" );
    $Log->comment( "Loading time and commit time are $loading_time sec !"
          . "and $commit_time sec !" );
    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );
    $Log->exit() if $may_exit;
    return \%times;
} ## end sub refresh_volume

################################################################################
## @name ::get_snapshot_info
## @summary
##     To Fetch Volume Snapshot Details using GUI.
## @description
##     This method can be used to Fetch Volume Snapshot Detials using GUI.
## @synopsis
##     get_snapshot_info(%args);
## INPUTS TO METHOD CALL 'get_snapshot_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> Name of the Volume </td> </tr>
## <tr> <td> -snapname </td> <td> Name of the SnapShot for which Info has to be fetched </td> </tr>
## <tr> <td> -vservername </td> <td> Name of the Vserver which has the volume </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub get_snapshot_info {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-volname"  => { type => SCALAR },
            "-snapname" => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $volname  = $args{'-volname'};
    my $snapname = $args{'-snapname'};
    my $snapshot_info;

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwrighting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to Storage>Volume");
    my $loading_time =
      $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );
    $Log->comment("Refreshing volume screen");
    $self->click("$VOLUME_REFRESH_BUTTON");
    $self->wait_for_element_present( $VOLUME_GRID, $GENERIC_WAIT );
    $Log->comment("Select volume $volname");
    $self->click_row_with_texts( "$VOLUME_GRID", $volname );

    $Log->comment("Clicking in the Snapshot Copies Tab");
    $self->click("$VOLUME_SNAPSHOT_TAB");
    $self->wait_for_loading();

    $Log->step("Selecting snapshot $snapname");
    $self->click_row_with_texts( "$VOLUME_SNAPSHOT_TAB_GRID", $snapname );

    $Log->step("Getting row data for Snapshot $snapname from grid");
    my $rowdata =
      $self->get_row_data_with_text( $VOLUME_SNAPSHOT_TAB_GRID, $snapname );

    my @rowarray = split( ' ', $rowdata );
    my %temp_snapshot_info;

    %temp_snapshot_info = (
        "name"                   => $rowarray[0],
        "date_time"              => $rowarray[1],
        "total_size"             => $rowarray[2],
        "cumulative_total_size"  => $rowarray[3],
        "status"                 => $rowarray[4],
        "application_dependency" => $rowarray[5],
    );

    $Log->comment("Collection of snapshot info is successfull");
    $snapshot_info->{loading_time}  = $loading_time;
    $snapshot_info->{parsed_output} = \%temp_snapshot_info;

    $Log->exit() if $may_exit;

    return $snapshot_info;
} ## end sub get_snapshot_info

###############################################################################
#                               Private Methods
###############################################################################
################################################################################
# Subroutine Name: _verifyvalues
# Objective:       This function will verify key
# Details:         See POD documentation at the end of this file
################################################################################
sub _verifyvalues {
    $Log->enter() if $may_enter;
    my ( $self, $key, $value, $id ) = @_;

    my $flag = 0;
    my $type = $self->get_attribute( $id, "text" );
    if ( lc($value) eq "$type" ) {
        $flag = 1;
        $Log->comment("The value given for $key exist");
    }
    else {
        $Log->comment("The invalid $key is given for verification");
    }
    $Log->comment("$key name is verified");
    $Log->exit() if $may_exit;

    return $flag;
} ## end sub _verifyvalues

################################################################################
# Subroutine Name: _spacefind
# Objective:       This function will return percentage
# Details:         See POD documentation at the end of this file
################################################################################
sub _spacefind {
    $Log->enter() if $may_enter;
    my ( $self, $id, $value ) = @_;
    my $test     = $self->get_text($id);
    my $newvalue = $1 if $test =~ /\s[\d+\.\d+]\s/;
    my $percent  = ( $newvalue / 100 ) * $value;
    $Log->exit() if $may_exit;
    return $percent;
} ## end sub _spacefind

1;
