# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to handle Volume move GUI objects.
##
## @description
##      Using this library, user can perform Volume move related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmVolumeMove;
##
##      volume_move(%args);
##      volume_move_aggr(%args);
##
## @status public
##
## @author afsars@netapp.com
## @change
##  2011-06-07 : afsars : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmVolumeMove;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    volume_move
    volume_move_aggr
);

###############################################################################
# P U B L I C   M E T H O D S
###############################################################################

###############################################################################
## @name ::volume_move
## @summary
##    Move the volume using GUI.
## @description
##     This method can be used to move the volume using GUI.
## @synopsis
##    volume_move(%args);
## INPUTS TO METHOD CALL 'volume_move' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required) Name of volume to be moved </td> </tr>
## <tr> <td> -destaggr </td> <td>(Required) Name of destination aggregate </td> </tr>
## <tr> <td> -srcaggr </td> <td> (Required) Name of source aggregate </td> </tr>
## <tr> <td> -vservername </td> <td> Name of volume to be moved </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit time.

sub volume_move {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-volName"  => { type => SCALAR },
            "-destaggr" => { type => SCALAR },
            "-srcaggr"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    my ($commit_time);
    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $VOLUME_TREE");
    my $loading_time
        = $self->cluster_tree_navigate( $vserverName, $VOLUME_TREE );

    $Log->step("Selecting volume: $args{'-volName'}");
    $self->click_row_with_texts( $VOLUME_GRID, $args{'-volName'});

    $Log->comment("Clicking volume move button");
    $self->click($VOLUME_MOVE_BUTTON);

    $Log->comment("Waiting for volume move Dialog");
    $self->wait_for_element_present( $VOLUME_DEST_AGGR_GRID, $GENERIC_WAIT );

    if ( exists( $args{-destaggr} ) ) {
        $self->wait_for_loading();
        $Log->step("Selecting destination aggregate: $args{'-destaggr'}");
        $self->click_row_with_texts( $VOLUME_DEST_AGGR_GRID,
            $args{'-destaggr'} );

        $Log->comment("Selecting move button to complete.");
        $self->click($VOLUME_MOVE_OK_BUTTON);

        if ( $self->is_element_present($VOLUME_MOVE_CONFIRM_CHECKBOX) ) {
            $Log->comment("Check conformation checkbox for volume move.");
            $self->set_checkbox( $VOLUME_MOVE_CONFIRM_CHECKBOX, "enable" );
        }
    } ## end if ( exists( $args{-destaggr...}))
    $Log->comment("click ok confirmation for volume move.");
    $self->click($VOLUME_MOVE_CONFIRM_OK_BUTTON);

    #$self->wait_for_element_present($VOLUME_MOVE_CONFIRM_OK_OK_BUTTON);
    $self->wait_for_loading();
    sleep 5;
    
    $Log->comment("Selecting ok button after completion.");
    $self->click($VOLUME_MOVE_CONFIRM_OK_OK_BUTTON);

    $self->check_for_error();
    $self->wait_for_element_not_present( $VOLUME_MOVE_CONFIRM_OK_OK_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    my %times = ( loading_time => $loading_time, );

    #commit_time  => $commit_time);

    $Log->comment("volume moved successfully");
    $Log->exit() if $may_exit;

    #return \%times;
    return 1;
} ## end sub volume_move

################################################################################
## @name ::volume_move_aggr
## @summary
##    Move the volume using GUI.
## @description
##     This method can be used to move the volume using GUI.
## @synopsis
##    volume_move_aggr(%args);
## INPUTS TO METHOD CALL 'volume_move_aggr' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -volname </td> <td> (Required) Name of volume to be moved </td> </tr>
## <tr> <td> -destaggr </td> <td> (Required) Name of destination aggregate </td> </tr>
## <tr> <td> -srcaggr </td> <td> (Required) Name of source aggregate </td> </tr>
## <tr> <td> -vservername </td> <td> Name of volume to be moved </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time.

sub volume_move_aggr {
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-volName"  => { type => SCALAR },
            "-destaggr" => { type => SCALAR },
            "-srcaggr"  => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    my $vserverName = $self->{$SYSTEM_VSERVER_NAME};
    if ( exists( $args{'-vservername'} ) ) {
        $Log->comment("Overwriting the default Vserver for this function");
        $vserverName = $args{'-vservername'};
    }

    $Log->step("Navigating to $AGGR_TREE");
    my $loading_time = 0;
    $loading_time = $self->cluster_tree_navigate( $self->{$SYSTEM_NAME_KEY},
        $AGGR_TREE );
    $self->wait_for_loading();
    $Log->step("Selecting aggregate: $args{'-srcaggr'}");
    $self->click_row_with_texts( $AGGR_GRID, $args{'-srcaggr'} );

    $Log->comment("Clicking volume move button");
    $self->click($VOLUME_MOVE_AGGR_BUTTON);
    $self->wait_for_loading();

    $Log->comment("Waiting for volume move Dialog");
    $self->wait_for_element_present( $VOLUME_MOVE_AGGR_NEXT_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Clicking next button");
    $self->click($VOLUME_MOVE_AGGR_NEXT_BUTTON);
    $self->wait_for_loading();
    
    if ( exists( $args{-volName} ) ) {

        $Log->step("Selecting volume: $args{-volName}");
        $self->click_row_with_texts( $VOLUME_MOVE_AGGR_VOLUME_GRID,
            $args{'-volName'} );
        $self->wait_for_loading();

        $Log->comment("Clicking next button");
        $self->click($VOLUME_MOVE_AGGR_NEXT_BUTTON);
        $self->wait_for_loading();

        if ( exists( $args{-destaggr} ) ) {
            $Log->step("Selecting destination aggregate: $args{'-destaggr'}");
            $self->click_row_with_texts( $VOLUME_MOVE_DEST_AGGR_GRID,
                $args{'-destaggr'} );
            sleep 5;
            $Log->comment("Clicking next button");
            $self->click($VOLUME_MOVE_AGGR_NEXT_BUTTON);
            $self->wait_for_loading();
        } ## end if ( exists( $args{-destaggr...}))

        $Log->comment("Clicking next button");
        $self->click($VOLUME_MOVE_AGGR_NEXT_BUTTON);
        $self->wait_for_loading();

        $Log->comment("Clicking finish button");
        $self->click($VOLUME_MOVE_AGGR_NEXT_BUTTON);
    } ## end if ( exists( $args{-volName...}))

    $self->check_for_error();
    $self->wait_for_element_not_present( $VOLUME_MOVE_AGGR_NEXT_BUTTON,
        $GENERIC_WAIT );
    $self->check_for_error();

    $Log->comment("Loading time is $loading_time sec!");
    my %times = ( loading_time => $loading_time );

    $Log->comment("volume moved successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub volume_move_aggr

1;
