# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary
##      Routines to parse Cmode Vserver GUI Functionalities
##
## @description
##      Using this library, user can perform Cmode Vserver related tasks.
##      The function name will be the same as the functionality that can be
##      performed using GUI.
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::CMode::SmVserver;
##		vserver_create(%args)
##		vserver_edit(%args)
##		vserver_delete(%args)
##		vserver_stop(%args)
##		vserver_start(%args)
##		vserver_refresh(%args)
##		get_vserver_info(%args)
##		clear_vserver_window(%args)
## @status public
##
## @author vikas.p@netapp.com
## @change
##  2011-06-27 : vikasp : Initial version
## @change
##  2012-03-20 : vikasp : POD modification

package NACL::APISet::Host::GUI::SysManager::CMode::SmVserver;

## Compiler directives.
use strict;
use warnings;

## Module import.
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Host::GUI::SysManager::CMode::GlobalConst;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use Exporter 'import';

our @EXPORT = qw (
    vserver_create
    vserver_edit
    vserver_delete
    vserver_stop
    vserver_start
    vserver_refresh
    get_vserver_info
    clear_vserver_window
);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
## @name ::vserver_create
## @summary
##     Create vserver using GUI.
## @description
##     This method can be used to create vserver using GUI.
## @synopsis
##     vserver_create(%args);
## INPUTS TO METHOD CALL 'vserver_create' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td> (Required)Vserver name to be created</td> </tr>
## <tr> <td> -aggrname </td> <td> (Required)Aggregate name to be created</td> </tr>
## <tr> <td> -lang </td> <td> (Required)Language to be added </td> </tr>
## <tr> <td> -protocol </td> <td> (Required)Protocols to be added ('',nfs,cifs,fcp,iscsi,fcoe)</td> </tr>
## <tr> <td> -security </td> <td> (Required)Security style (UNIX|NTFS)</td> </tr>
## <tr> <td> -ipSpace </td> <td> ipSpace to be selected </td> </tr>
## <tr> <td> -voltype </td> <td> Type of volume: (flexvol|infinite) Default:flexvol</td> </tr>
## <tr> <td> -dnsDomain </td> <td> Name of the domain </td> </tr>
## <tr> <td> -dnsServer </td> <td> Adding name server </td> </tr>
## <tr> <td> -subnet </td> <td> Subnet to select </td> </tr>
## <tr> <td> -node </td> <td> Node on which subnet needs to be configured</td> </tr>
## <tr> <td> -port </td> <td> port to assign for lif creation</td> </tr>
## <tr> <td> -cifsServer </td> <td> cifs server</td> </tr>
## <tr> <td> -cifsDomain </td> <td> cifs domain name</td> </tr>
## <tr> <td> -orgUnit </td> <td> org unit to select for Cifs</td> </tr>
## <tr> <td> -cifsUserName </td> <td> cifs username</td> </tr>
## <tr> <td> -cifsPassword </td> <td> cifs password</td> </tr>
## <tr> <td> -shareName </td> <td> Cifs share name</td> </tr>
## <tr> <td> -shareUser </td> <td> User to give cifs share permision</td> </tr>

## <tr> <td> -interface </td> <td> Selecting interface </td> </tr>
## <tr> <td> -interfacename </td> <td> Name of the interface to be added </td> </tr>
## <tr> <td> -homeport </td> <td> Selecting homeport </td> </tr>
## <tr> <td> -ipaddress  </td> <td> Selecting ipaddress </td> </tr>
## <tr> <td> -netmask </td> <td> Selecting netmask </td> </tr>
## <tr> <td> -gateway </td> <td> Selecting gateway </td> </tr>
## <tr> <td> -pridomainname </td> <td> Name of the pridomain </td> </tr>
## <tr> <td> -addsearchdomainname </td> <td> Adding search domain name</td> </tr>
## <tr> <td> -delsearchdomainname </td> <td> Deleting search domain name </td> </tr>
## <tr> <td> -addnameserver </td> <td> Adding name server </td> </tr>
## <tr> <td> -delnameserver </td> <td> Delete name server </td> </tr>
## <tr> <td> -addnfsinterfacename </td> <td> Adding nfs interface name </td> </tr>
## <tr> <td> -nfshomeport </td> <td> Selecting nfshomeport </td> </tr>
## <tr> <td> -nfsipaddress </td> <td>Selecting nfs ip address </td> </tr>
## <tr> <td> -nfsnetmask </td> <td> Selecting nfs netmask </td> </tr>
## <tr> <td> -nfsgateway </td> <td> Selecting nfs gateway </td> </tr>
## <tr> <td> -nfsallowmgmt </td> <td> Selecting nfs data interface management </td> </tr>
## <tr> <td> -delnfsinterfacename </td> <td> Deleting nfs interfacename </td> </tr>
## <tr> <td> -nfsversion </td> <td> Selecting nfsversion </td> </tr>
## <tr> <td> -nameservers </td> <td> Selecting nameservers </td> </tr>
## <tr> <td> -nisprimarydomain </td> <td> Selecting nis primary domain </td> </tr>
## <tr> <td> -nisserver </td> <td> Selecting nisserver </td> </tr>
## <tr> <td> -addsecondarydomains </td> <td> Adding secondary domains </td> </tr>
## <tr> <td> -addsecondarynisserver </td> <td> Adding secondary nis server </td> </tr>
## <tr> <td> -delsecondarydomains </td> <td> Deleting secondary domains </td> </tr>
## <tr> <td> -localusersgroupname </td> <td> Selecting local users groupname </td> </tr>
## <tr> <td> -localusersusername </td> <td> Selecting local users username </td> </tr>
## <tr> <td> -localusersuserid </td> <td> Selecting local users user id </td> </tr>
## <tr> <td> -localusersfullname </td> <td> Selecting local users fullname </td> </tr>
## <tr> <td> -addmapping </td> <td> Selecting Add Name Mapping button </td> </tr>
## <tr> <td> -mappingdirection </td> <td> Selecting mapping direction </td> </tr>
## <tr> <td> -mappingposition </td> <td> Selecting mapping position </td> </tr>
## <tr> <td> -mappingpattern </td> <td>  Selecting mapping pattern </td> </tr>
## <tr> <td> -mappingreplacement </td> <td> Selecting mapping replacement </td> </tr>
## <tr> <td> -editmapping </td> <td> Selecting edit mapping </td> </tr>
## <tr> <td> -editmappingpattern </td> <td> Selecting edit mapping pattern </td> </tr>
## <tr> <td> -editmappingreplacement </td> <td> Selecting edit mapping replacement</td> </tr>
## <tr> <td> -delmapping </td> <td> Selecting delmapping </td> </tr>
## <tr> <td> -dnsconfig </td> <td> Selecting dnsconfig </td> </tr>
## <tr> <td> -iscsitargetnodename </td> <td> Selecting iscsi target nodename </td> </tr>
## <tr> <td> -iscsitargetalias </td> <td> Selecting iscsi target alias </td> </tr>
## <tr> <td> -iscsicontrollername </td> <td> Selecting iscsi controller name </td> </tr>
## <tr> <td> -iscsiinterfacepercntrl </td> <td> Selecting iscsi interface per cntrl </td> </tr>
## <tr> <td> -iscsistartingip </td> <td> Selecting iscsi starting ip </td> </tr>
## <tr> <td> -iscsinetmask </td> <td> Selecting iscsi netmask </td> </tr>
## <tr> <td> -iscsigateway </td> <td> Selecting iscsi gateway </td> </tr>
## <tr> <td> -iscsiportset </td> <td> Selecting iscsi portset </td> </tr>
## <tr> <td> -createinterfacefor </td> <td> Selecting create interface for </td> </tr>
## <tr> <td> -addcifsinterfacename </td> <td> Adding cifs interface name </td> </tr>
## <tr> <td> -cifshomeport </td> <td>  Selecting cifs homeport </td> </tr>
## <tr> <td> -cifsnetmask </td> <td> Selecting cifs netmask </td> </tr>
## <tr> <td> -cifsgateway </td> <td> Selecting cifs gateway </td> </tr>
## <tr> <td> -delcifsinterfacename </td> <td> Deleting cifs interface name </td> </tr>
## <tr> <td> -cifsservername </td> <td> Selecting cifs server name </td> </tr>
## <tr> <td> -cifsdomainname </td> <td> Selecting cifs domain name </td> </tr>
## <tr> <td> -cifsadminname </td> <td> Selecting cifs admin name </td> </tr>
## <tr> <td> -ldapclient </td> <td> Selecting ldapclient </td> </tr>
## <tr> <td> -ldapserver </td> <td> Selecting ldapserver </td> </tr>
## <tr> <td> -ldapaddomain </td> <td> Selecting ldapaddomain </td> </tr>
## <tr> <td> -ldapaddserveradd </td> <td> Selecting Active Directory Domain add server button </td> </tr>
## <tr> <td> -ldapaddserverdel </td> <td> Selecting Active Directory Domain delete server button </td> </tr>
## <tr> <td> -ldapadsserveradd </td> <td> Selecting Active Directory Server Add button </td> </tr>
## <tr> <td> -ldapadsserverdel </td> <td> Selecting Active Directory Server delete button </td> </tr>
## <tr> <td> -ldapauthlevel </td> <td> Selecting ldap authentication level</td> </tr>
## <tr> <td> -ldapbinddn </td> <td> Selecting ldap bind dn </td> </tr>
## <tr> <td> -ldapbindupass </td> <td> Selecting bind user password </td> </tr>
## <tr> <td> -ldapbasedn </td> <td> Selecting ldap base dn </td> </tr>
## <tr> <td> -ldaptcpport </td> <td> Selecting ldap tcp port </td> </tr>
## <tr> <td> -fcnodename </td> <td> Selecting fc nodename </td> </tr>
## <tr> <td> -fccontrollersel </td> <td> Selecting fc controller grid</td> </tr>
## <tr> <td> -fcdatainterfacepercontroller </td> <td> Selecting fc data interface per controller </td> </tr>
## <tr> <td> -fcportsetnumber </td> <td> Selecting fc portsetnumber </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub vserver_create {

    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            "-vservername"  => { type => SCALAR },
            "-aggrname"     => { type => SCALAR },
            "-lang"         => { type => SCALAR },
            "-protocol"     => { type => SCALAR },
        },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'}
                                                   , $self->{$SYSTEM_NAME_KEY});
    my $TAB = $self->{$SYSTEM_NAME_KEY};
    
    $Log->comment("Clicking vserver create button");
    $self->click($VSERVER_CREATE_BUTTON);
    $self->wait_for_loading();

    ###################################

    $Log->comment("Entering vserver Name: " . $args{"-vservername"});
    $self->type($VSERVER_CREATE_VSERVERNAME_INPUT, $args{"-vservername"});

    $Log->comment("Selecting the language: " . $args{"-lang"});
    $self->select_from_dropdown($VSERVER_CREATE_LANG_COMBOBOX,
                              $args{"-lang"}, 1);
    
    $Log->comment("Selecting the Aggregate: " . $args{"-aggrname"});
    $self->click($VSERVER_CREATE_ROOT_AGGR_COMBO);
    $self->click("//table/tbody/tr/td\[contains(\@title,'"
               . $args{'-aggrname'} . "')\]");
    
    if (exists($args{"-ipSpace"})) {
        $Log->comment("Selecting ipSpace from drop down: " . $args{"-ipSpace"});
        $self->select_from_dropdown($VSERVER_CREATE_IPSPACE_INPUT,
                                    $args{"-ipSpace"});
    }
    if (exists($args{"-voltype"})) {
        if ($args{"-voltype"} =~ /flexvol/i) {
            $Log->comment("Selecting Volume Type: " . $args{"-voltype"});
            $self->click($VSERVER_CREATE_VOLTYPE_RADIO_FLEX_VOL);
        } elsif ($args{"-voltype"} =~ /infinite/i) {
            $self->click($VSERVER_CREATE_VOLTYPE_RADIO_IV_VOL);
        } else {
            #Do nothing: Metro cluster does not throw this option at all
        }
    }
    if ($args{"-protocol"} =~ /cifs/i ) {
        $Log->comment( "Selecting the protocol CIFS");
        $self->click($VSERVER_CREATE_PROTOCOL_CHECKBOX_CIFS);
    }

    if ($args{"-protocol"} =~ /nfs/i ) {
        $Log->comment( "Selecting the protocol NFS");
        $self->click($VSERVER_CREATE_PROTOCOL_CHECKBOX_NFS);
    }

    if ($args{"-protocol"} =~ /iscsi/i) {
        $Log->comment( "Selecting the protocol iSCSI");
        $self->click($VSERVER_CREATE_PROTOCOL_CHECKBOX_ISCSI);
    }

    if ($args{"-protocol"} =~ /fcp/i || $args{"-protocol"} =~ /fcoe/i ) {
        $Log->comment( "Selecting the protocol FCP/FCoE");
        $self->click($VSERVER_CREATE_PROTOCOL_CHECKBOX_FC);
    }
    
    if (exists($args{"-security"})) {
        $Log->comment( "Selecting the Security Style: ". $args{"-security"});
        $self->select_from_dropdown($VSERVER_CREATE_SECURITY_STYLE_COMBO,
                              $args{"-security"}, 1);
    }    
    if (exists($args{"-dnsDomain"}) ) {
        $Log->comment("Entering dns domain: " . $args{"-dnsDomain"});
        $self->type($VSERVER_CREATE_DNS_DOMAIN_INPUT, "");
        $self->type($VSERVER_CREATE_DNS_DOMAIN_INPUT, $args{"-dnsDomain"} );
    }
    if (exists($args{"-dnsServer"}) ) {
        $Log->comment("Entering dns server: " . $args{"-dnsServer"});
        $self->type($VSERVER_CREATE_DNS_SERVER_INPUT, "");
        $self->type($VSERVER_CREATE_DNS_SERVER_INPUT, $args{"-dnsServer"});
    }  

    # Click submit and continue
    $self->click($VSERVER_SETUPWIZARD_SUBMIT);

    # Check fo any errors on page    
    $self->check_for_error();
    $self->wait_for_loading();
    
    if ($args{"-protocol"} =~ /cifs/i ) {
        $Log->comment("Configuring CIFS Protocol");
        $self->wait_for_element_present($VSERVER_CREATE_DATA_INT_FIELD);
        
        # TODO : convert following lines into lib
        if ( exists($args{"-subnet"})) {
            $Log->comment("Clicking on Subnet text box");
            $self->_find_element("//form/table/tbody/tr/td[3]/div/input")->click();
            $Log->comment( "Selecting Subnet: " . $args{"-subnet"} );
            $self->_find_element("//div[contains(\@class,'ellipsis') and text()='" . $args{"-subnet"} . "']")->click();
        }
        if ( exists($args{"-node"}) and exists($args{"-port"}) ) {
            $Log->comment("Select port for LIF configuration");
            $self->click($VSERVER_CREATE_PORT_BROWSE_BTN);
            $self->wait_for_loading();
            $Log->comment( "Select Node containing port: ". $args{"-node"});
            $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, 1,1, $args{'node'});
            $self->wait_for_loading();
            $Log->comment( "Clicking on the port " . $args{"port"});
            $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, $args{'port'});
            
            #$self->_find_element("//div[contains(\@class,'x-tree3-el')]/span[contains(\@class,'x-tree3-node-text') and text()='". $args{'port'} . "']")->click();
    
            $Log->comment( "Clicking on ok");
            $self->click($GENERIC_DIALOG2_OK_BUTTON);
        }
        if ( exists($args{"-cifsServer"})
          or exists($args{"-cifsDomain"})
          or exists($args{"-orgUnit"})
          or exists($args{"-cifsUserName"})
          or exists($args{"-cifsPassword"}) ) {
            $Log->comment( "Entering cifsServer: " . $args{"-cifsServer"});
            $self->type($VSERVER_CREATE_CIFS_SERVER_NAME,"");
            $self->type($VSERVER_CREATE_CIFS_SERVER_NAME,$args{"-cifsServer"});

            $Log->comment( "Entering cifsServer: " . $args{"-cifsDomain"});
            $self->type($VSERVER_CREATE_CIFS_AD_DOMAIN,"");
            $self->type($VSERVER_CREATE_CIFS_AD_DOMAIN,$args{"-cifsDomain"});

            $Log->comment( "Entering cifsServer: ". $args{"-orgUnit"});
            $self->type($VSERVER_CREATE_CIFS_ORG_UNIT,"");
            $self->type($VSERVER_CREATE_CIFS_ORG_UNIT,$args{"-orgUnit"});

            $Log->comment( "Entering cifsServer: " . $args{"-cifsUserName"});
            $self->type($VSERVER_CREATE_CIFS_ADMIN_NAME,$args{"-cifsUserName"});

            $Log->comment( "Entering cifsServer: " . $args{"-cifsPassword"});
            $self->type($VSERVER_CREATE_CIFS_ADMIN_PASS,$args{"-cifsPassword"});
        }

        if (exists($args{"-shareName"})) {

            $Log->comment( "Entering Share Name: " . $args{"-shareName"});
            $self->type($VSERVER_CREATE_CIFS_SHARE_NAME,$args{"-shareName"});

            $Log->comment( "Entering Share Size: ". $args{"-shareSize"});
            $self->type($VSERVER_CREATE_CIFS_SHARE_SIZE_TEXT,$args{"-shareSize"});

            $Log->comment("Selecting Share Size Unit: "
                          . $args{"-shareSizeUnit"});
            unless ( $args{"-shareSizeUnit"} ) {
                $args{"-shareSizeUnit"}  = 'MB';
            }
            $self->set_combobox_value($VSERVER_CREATE_CIFS_SHARE_SIZE_UNIT,
                                      $args{"-shareSizeUnit"},'partial');

            if (exists($args{"-shareUser"}) ) {
                if ( $self->get_text($VSERVER_CREATE_CIFS_PERM_TEXTAREA)
                    !~ /$args{'-shareUser'}/) {
                    $Log->comment("Clicking On Change Button");
                    $self->click($VSERVER_CREATE_CIFS_PERM_CHANGE_BUTTON);
                
                    if ( !$self->is_element_present(
                            $VSERVER_CREATE_CIFS_SHARE_PERM_TAB_LIST
                          . "/div[contains(text(),'$args{'-shareUser'}')]")
                       ) {
                        $Log->comment("Clicking On User/Group ADD Button");
                        $self->click($VSERVER_CREATE_CIFS_SHARE_PERM_TAB_ADD_BTN);
        
                        $Log->comment("Entering User/Group: ". $args{"-shareUser"});
                        $self->type($VSERVER_CREATE_CIFS_SHARE_ADDUSER_INPUT,$args{"-shareUser"});
        
                        $Log->comment("Clicking On User/Group ADD OK Button");
                        $self->click($VSERVER_CREATE_CIFS_SHARE_ADDUSER_OK_BTN);
                        sleep(2);
                    }
                    $self->click($VSERVER_CREATE_CIFS_SHARE_PERM_TAB_LIST
                          . "/div[contains(text(),'$args{'-shareUser'}')]" );
                    # click permissisons
                    $Log->comment("Clicking On User/Group ADD Save Button");
                    $self->click($VSERVER_CREATE_CIFS_SHARE_PERM_TAB_SAVE_CLOSE_BTN);
                }
            }
        }
    }

    if ($args{"-protocol"} =~ /nfs/i) {
        if ($args{"-protocol"} !~ /cifs/i) {
            # TODO : convert following lines into lib
            if ( exists($args{"-subnet"})) {
                $Log->comment("Clicking on Subnet text box");
                $self->_find_element("//form/table/tbody/tr/td[3]/div/input")->click();
                $Log->comment( "Selecting Subnet: " . $args{"-subnet"} );
                $self->_find_element("//div[contains(\@class,'ellipsis') and text()='" . $args{"-subnet"} . "']")->click();
            }
            if ( exists($args{"-node"}) and exists($args{"-port"}) ) {
                $Log->comment("Select port for LIF configuration");
                $self->click($VSERVER_CREATE_PORT_BROWSE_BTN);
                $self->wait_for_loading();
                $Log->comment( "Select Node containing port: ". $args{"-node"});
                $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, 1,1, $args{'node'});
                $self->wait_for_loading();
                $Log->comment( "Clicking on the port " . $args{"port"});
                $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, $args{'port'});
                
                #$self->_find_element("//div[contains(\@class,'x-tree3-el')]/span[contains(\@class,'x-tree3-node-text') and text()='". $args{'port'} . "']")->click();
        
                $Log->comment( "Clicking on ok");
                $self->click($GENERIC_DIALOG2_OK_BUTTON);
            }
        } elsif (exists($args{"noRetain"})) {
            $Log->comment("Clicking Retain CheckBox as NFS and CIFS config is different");
            $self->_find_element("//div[\@id='gwt-debug-"
                                . $TAB
                                . "VserverSetupNasDataInterfacePanelDataIntfCheckboxCheckBox']/input")->click();
            $Log->comment("Clicking on Subnet text box");
            $self->_find_element(
                            "//td[2]/div/table/tbody/tr[2]/td/div/div[2]/div/form/table/tbody/tr/td[3]/div/input")->click();
            $Log->comment( "Selecting Subnet: ". $args{"-subnet"});
            $self->_find_element(
                            "//div[contains(\@class,'ellipsis') and text()='". $args{"-subnet"} . "']")->click();

            $Log->comment( "Clicking on browse");
            $self->_find_element(
                "//div/table/tbody/tr/td[2]/table/tbody/tr[2]/td[2]/em/button")->click();
            $self->wait_for_element_not_present("LOADING");

            $Log->comment( "clicking on node: ". $args{"-node"});
            $self->_find_element("//div[contains(\@id,'gwt-debug-"
                               . $TAB
                               . "NetworkPortInterfaceGrid')]/div/span[contains(\@class,'x-tree3-node-text') and text()='"
                               . $args{"-node"} . "']")->_double_click();
            $self->wait_for_element_not_present("LOADING");

            $Log->comment( "Clicking on the port " . $args{"port"});
            $self->_find_element(
                            "//div[contains(\@class,'x-tree3-el')]/span[contains(\@class,'x-tree3-node-text') and text()='"
                                            . $args{"port"} . "']")->click();

            $Log->comment( "Clicking on ok");
            $self->_find_element(
                            "//td[2]/table/tbody/tr/td/table/tbody/tr/td/table/tbody/tr[2]/td[2]/em/button")->click();
        }
        if (exists($args{"-nisConfig"})) {
            $Log->comment("NIS configuration details");
            if (!$self->is_element_present("gwt-debug-" . $TAB . "VserverSetupNasConfigNisPanelDomainNameFieldTextArea-input")) {
                $Log->comment("Opening NIS Config Details Panel");
                $self->_find_element("//fieldset[\@id='gwt-debug-". $TAB . "VserverSetupNasConfigNisPanelNisConfigFieldSet']/legend/div")->click();
            }
            if (exists($args{"-nisDomain"})) {
                $Log->comment( "Entering NIS Details: ". $args{"-nisDomain"});
                $self->_find_element("gwt-debug-" . $TAB . "VserverSetupNasConfigNisPanelDomainNameFieldTextArea-input")->send_keys("");
                $self->_find_element("gwt-debug-". $TAB. "VserverSetupNasConfigNisPanelDomainNameFieldTextArea-input")->send_keys($args{"-nisDomain"});
            }
            if (exists($args{"-nisServer"})) {
                $Log->comment( "Entering NIS Details: ". $args{"-nisServer"});
                $self->_find_element("gwt-debug-". $TAB. "VserverSetupNasConfigNisPanelIpAddressesFieldTextArea-input")->send_keys("");
                $self->_find_element("gwt-debug-". $TAB . "VserverSetupNasConfigNisPanelIpAddressesFieldTextArea-input")->send_keys($args{"-nisServer"});
            }
        }
        if (exists($args{"-exportName"})) {
            $Log->comment("Entering Export Name: " . $args{"-exportName"});
            $self->_find_element("gwt-debug-". $TAB. "VserverSetupNasConfigNisPanelNfsExportAsFieldTextField-input")->send_keys($args{"-exportName"});
        }
        if (exists($args{"-exportSize"})) {
            $Log->comment("Entering Export Size: ". $args{"-exportSize"});
            $self->_find_element("gwt-debug-". $TAB. "VserverSetupNasConfigNisPanelNfsVolumeSizeFieldTextField-input")->send_keys($args{"-exportSize"});
        }
        if (exists($args{"-exportSizeUnit"})) {
            $Log->comment("Selecting Export Size Unit: " . $args{"-exportSizeUnit"});
            $self->_find_element("//td/fieldset/div/table/tbody/tr/td[2]/div/table/tbody/tr[3]/td[2]/div/table/tbody/tr/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='". $args{"-exportSizeUnit"} . "']")->click();
        }
        if (exists($args{"-exportPermission"})) {
            $Log->comment("Clicking On Change Button");
            $self->_find_element("//td/fieldset/div/table/tbody/tr/td[2]/div/table/tbody/tr[4]/td[2]/div/table/tbody/tr/td[2]/div/span")->click();
            $self->wait_for_element_not_present("LOADING");
            if (exists($args{"-clientSpec"})) {
                $Log->comment("Entering Client Specification: " . $args{"-clientSpec"});
                $self->_find_element("gwt-debug-". $TAB . "CreateModifyRuleDialogClientSpec-input")->send_keys("");
                $self->_find_element("gwt-debug-". $TAB . "CreateModifyRuleDialogClientSpec-input")->send_keys($args{"-clientSpec"});
            }
            if (exists($args{"-accessProtocols"})) {
                $Log->comment("Spliting the Access Protocols ");
                my @protocols = split(":",$args{"-accessProtocols"});
                for (my $i = 0; $i < @protocols ; $i++) {
                    if ($protocols[$i] =~ ("CIFS")) {
                        $Log->comment("Disabling CIFS");
                        $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialogCifsChkBox']/input")->click();
                    } elsif ($protocols[$i] =~ /NFSv3/ ) {
                        $Log->comment("Disabling NFSv3");
                        $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialognfsv3CheckBOx']/input")->click();
                    } elsif ($protocols[$i] =~ /NFSv4/ ) {
                        $Log->comment("Disabling NFSv4");
                        $self->_find_element("//div[\@id='gwt-debug-" . $TAB. "CreateModifyRuleDialognfsv4CheckBOx']/input")->click();
                    } elsif ($protocols[$i] =~ /NFS/) {
                       $Log->comment("Disabling NFS");
                       $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialognfsCheckBOx']/input")->click();
                    } elsif ($protocols[$i] =~ /Flexcache/) {
                       $Log->comment("Disabling Flexcache");
                       $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialogFlexcacheChkBox']/input")->click();
                    }
                }
            }

            if (exists($args{"-accessDetailsRO"}) ) {
                $Log->comment("Enabiling Read only Check Box");
                $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialogReadOnlyCheckBox']/input")->click();
                $Log->comment("Spliting the Access Details RO");
                my @accessRo = split(",",$args{"-accessDetailsRO"} );
                for (my $i = 0; $i < @accessRo; $i++ ) {
                    if ($accessRo[$i] =~ /UNIX/i) {
                        $Log->comment("Disabling UNIX");
                        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "CreateModifyRuleDialogRoRuleUnixChkBox']/input")->click();
                    } elsif ($accessRo[$i] =~ /Kerberos5i/i ) {
                        $Log->comment("Disabling Kerberos 5i");
                        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "CreateModifyRuleDialogRoRuleKerberos5iChkBox']/input")->click();
                    } elsif ($accessRo[$i] =~ /Kerberos5/i) {
                        $Log->comment("Disabling Kerberos 5");
                        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "CreateModifyRuleDialogRoRuleKerberos5ChkBox']/input")->click();
                    } elsif ($accessRo[$i] =~ /NTLM/i) {
                        $Log->comment("Disabling NTLM");
                        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "CreateModifyRuleDialogRoRuleNtlmChkBox']/input")->click();
                    }
                }
            }

            if (exists($args{"-accessDetailsRW"}) ) {
                if ($args{"-accessDetailsRW"} =~ /disable/i) {
                    $Log->comment("Disabling Read Write Check Box");
                    $self->_find_element("//div[\@id='gwt-debug-". $TAB. "CreateModifyRuleDialogReadWriteCheckBox']/input")->click();
                } else {
                    $Log->comment("Spliting the Access Details RW");
                    my @accessRw = split(",",$args{"-accessDetailsRW"} );
                    for (my $i = 0; $i < @accessRw; $i++ ) {
                        if ($accessRw[$i] =~ /UNIX/i ) {
                            $Log->comment("Disabling UNIX");
                            $self->_find_element("//div[\@id='gwt-debug-"
                                . $TAB
                                . "CreateModifyRuleDialogRoRuleUnixChkBox']/input")->click();
                        } elsif ($accessRw[$i] =~ /Kerberos5/i) {
                            $Log->comment("Disabling Kerberos 5");
                            $self->_find_element("//div[\@id='gwt-debug-"
                                . $TAB
                                . "CreateModifyRuleDialogRoRuleKerberos5ChkBox']/input")->click();
                        } elsif ($accessRw[$i] =~ /Kerberos5i/i) {
                            $Log->comment("Disabling Kerberos 5i");
                            $self->_find_element("//div[\@id='gwt-debug-"
                                        . $TAB
                                        . "CreateModifyRuleDialogRoRuleKerberos5iChkBox']/input")->click();
                        } elsif ($accessRw[$i] =~ /NTLM/i) {
                            $Log->comment("Disabling NTLM");
                            $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "CreateModifyRuleDialogRoRuleNtlmChkBox']/input")->click();
                        }
                    }
                }
            }
            if (exists($args{"-superUser"})) {
                $Log->comment("Enabiling SuperUser Access");
                $self->_find_element("//div[\@id='gwt-debug-". $TAB . "CreateModifyRuleDialogRuleLabelSuperUserCheckBox']/input")->click();
            }
            
            $Log->comment("Clicking on Modify Export Rule Dialog OK Button");
            $self->_find_element("gwt-debug-" . $TAB . "CreateModifyRuleDialogOkBtn")->click();
        }
    }
    if ($args{"-protocol"} =~ /iscsi/i) {
        $Log->comment( "Configuring iScsi Protocol");
        if (exists($args{"-targetAlias"})) {
            $Log->comment("Entering -targetAlias: " . $args{"-targetAlias"});
            $self->_find_element("gwt-debug-" . $TAB . "IscsiPageTargetAliasTextField-input")->send_keys($args{"-targetAlias"});
        }
        if (exists($args{"-lifNode"})) {
            $Log->comment("Entering -lifNode: " . $args{"-lifNode"});
            $self->_find_element("gwt-debug-" . $TAB . "IscsiPageIntfPerNodeField-input")->send_keys("");
            $self->_find_element("gwt-debug-" . $TAB . "IscsiPageIntfPerNodeField-input")->send_keys($args{"-lifNode"});
        }
        
        $Log->comment( "Clicking on Vertical Panel");
        $self->_find_element("gwt-debug-" . $TAB . "iscsiVerticalPanel")->click();
        $self->wait_for_element_not_present("RETRIEVING");
        
        if (exists($args{"-lunProv"})) {
            $Log->comment("Entering lun prov: " . $args{"-lunProv"});
            $self->_find_element("//tr[2]/td[2]/div/table/tbody/tr/td/div/input")->send_keys($args{"-lunProv"});
        }
        if (exists($args{"-lunSizeUnit"})) {
            $Log->comment( "Selecting Lun size Unit: " . $args{"-lunSizeUnit"});
            $self->_find_element("//tr[2]/td[2]/div/table/tbody/tr/td[2]/div/img")->click();
            $self->_find_element( "//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunSizeUnit"} . "']")->click();
        }
        if (exists($args{"-lunOS"})) {
            $Log->comment( "Selecting Lun OS Type: ". $args{"-lunOS"});
            $self->_find_element("//td[2]/div/table/tbody/tr[3]/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunOS"} . "']")->click();
        }
        if (exists($args{"-lunInitiator"})) {
            $Log->comment("Entering initiator: " . $args{"-lunInitiator"});
            $self->_find_element("//td[2]/div/table/tbody/tr[4]/td[2]/div/input")->send_keys($args{"-lunInitiator"});
        }
        if (exists($args{"-subnet"})) {
            $Log->comment( "Clicking on Subnet text box");
            $self->_find_element("gwt-debug-" . $TAB . "IscsiPageSubnetCombobox-input")->click();
            $Log->comment("Selecting Subnet: " . $args{"-subnet"});
            $self->_find_element("//div[contains(\@class,'ellipsis') and text()='" . $args{"-subnet"} . "']")->click();
        }

        if (exists($args{"-iscsiAdvanced"}) ) {
            $Log->comment("Clicking on Advanced Setting CheckBox");
            $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "IscsiPageAdvancedSettingsCheckbox']/input")->click();
            if (exists($args{"-portSets"}) ) {
                $Log->comment( "Entering number of portsets: " . $args{"-portSets"});
                $self->_find_element("gwt-debug-" . $TAB . "-iscsiPortsetSpinner-spinnerField-input")->send_keys("");
                $self->_find_element("gwt-debug-" . $TAB . "-iscsiPortsetSpinner-spinnerField-input")->send_keys($args{"-portSets"});
            }
            if (exists($args{"-iscsiInterfaceName"}) ) {
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->click();
                $Log->comment("Double Clicking the interface row");
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->_double_click();
                $self->wait_for_element_not_present("LOADING");
                $Log->comment("Entering interface name: " . $args{"-iscsiInterfaceName"});
                $self->_find_element("gwt-debug-" . $TAB . "-iscsiInterfaceNameCol-input")->send_keys("");
                $self->_find_element("gwt-debug-" . $TAB . "-iscsiInterfaceNameCol-input")->send_keys($args{"-iscsiInterfaceName"});
            }
            if (exists($args{"-iscsiPort"}) ) {
                $Log->comment("Clicking on port selection");
                $self->_find_element("gwt-debug-" . $TAB . "iscsiHomeportCol-input")->click();
                $Log->comment("Selecting port: " . $args{"-iscsiPort"});
                $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-iscsiPort"} . "']")->click();
            }
            $Log->comment( "Clicking On Save Button");
            $self->_find_element("//div/table/tbody/tr/td/table/tbody/tr[2]/td[2]/em/button")->click();
        }
    }   
    if ($args{"-protocol"} =~ /fcp/i) {
        $Log->comment("Configuring FC Page Config CheckBox");
        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "fcPageConfigCheckbox']/input")->click();
        if (exists($args{"-fcLifNode"})) {
            $Log->comment( "Entering FC Lif Per Node: " . $args{"-fcLifNode"});
            $self->_find_element( "gwt-debug-" . $TAB . "fcPageIntfPerNodeField-input")->send_keys("");
            $self->_find_element( "gwt-debug-" . $TAB . "fcPageIntfPerNodeField-input")->send_keys($args{"-fcLifNode"});
        }
        if (exists($args{"-lunProvFC"})) {
            $Log->comment( "Entering lun prov: " . $args{"-lunProvFC"});
            $self->_find_element( "//td[3]/div/table/tbody/tr[2]/td[2]/div/table/tbody/tr/td/div/input")->send_keys($args{"-lunProvFC"});
        }
        if (exists($args{"-lunSizeUnitFC"})) {
            $Log->comment( "Selecting Lun size Unit: " . $args{"-lunSizeUnitFC"});
            $self->_find_element("//td[3]/div/table/tbody/tr[2]/td[2]/div/table/tbody/tr/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunSizeUnitFC"} . "']")->click();
        }
        if (exists($args{"-lunOSFC"})) {
            $Log->comment( "Selecting Lun OS Type: " . $args{"-lunOSFC"});
            $self->_find_element("//td[3]/div/table/tbody/tr[3]/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunOSFC"} . "']")->click();
        }
        if (exists($args{"-lunInitiatorFC"})) {
            $Log->comment( "Entering initiator: " . $args{"-lunInitiatorFC"});
            $self->_find_element( "//td[3]/div/table/tbody/tr[4]/td[2]/div/input")->send_keys($args{"-lunInitiatorFC"});
        }
        
        if (exists($args{"-fcpAdvanced"})) {
            $Log->comment("Clicking on Advanced Setting CheckBox");
            $self->_find_element("//div[\@id='gwt-debug-". $TAB . "fcPageAdvancedSettingsCheckbox']/input")->click();
            if (exists($args{"--fcp-portSets"}) ) {
                $Log->comment("Entering number of portsets: " . $args{"--fcp-portSets"});
                $self->_find_element("gwt-debug-" . $TAB . "fcPortsetSpinner-spinnerField-input")->send_keys("");
                $self->_find_element("gwt-debug-" . $TAB . "fcPortsetSpinner-spinnerField-input")->send_keys($args{"--fcp-portSets"});
            }
            if (exists($args{"-fcpInterfaceName"}) ) {
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->click();
                sleep(5);
                $Log->comment("Double Clicking the interface row");
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->_double_click();
                sleep(5);
                $Log->comment("Entering interface name: " . $args{"-fcpInterfaceName"});
                $self->_find_element( "gwt-debug-" . $TAB . "fcInterfaceNameCol-input")->send_keys("");
                $self->_find_element( "gwt-debug-" . $TAB . "fcInterfaceNameCol-input")->send_keys($args{"-fcpInterfaceName"});
            }
            if (exists($args{"-fcpPort"}) ) {
                $Log->comment("Clicking on port selection");
                $self->_find_element("gwt-debug-" . $TAB . "fcHomeportCol-input")->click();
    
                $Log->comment( "Selecting port: " . $args{"-fcpPort"});
                $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-fcpPort"} . "']")->click();
            }
            $Log->comment("Clicking On Save Button");
            $self->_find_element("gwt-debug-saveButton")->click();
        }
    }
        
    if ($args{"-protocol"} =~ /fcoe/i ) {
        $Log->comment( "Clicking on FCoE Config CheckBox");
        $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "fcoePageConfigCheckbox']/input")->click();
        if (exists($args{"-fcoeLifNode"}) ) {
            $Log->comment("Entering fcoe lif per node: " . $args{"-fcoeLifNode"});
            $self->_find_element( "gwt-debug-" . $TAB . "fcoePageIntfPerNodeField-input")->send_keys("");
            $self->_find_element("gwt-debug-" . $TAB . "fcoePageIntfPerNodeField-input")->send_keys($args{"-fcoeLifNode"});
        }
        $Log->comment( "Clicking on Text Box");
        $self->_find_element("//tr[2]/td/div/div[2]/div/form/div[3]/table/tbody/tr/td/div/table/tbody/tr/td/div/table/tbody/tr/td[3]/div/table/tbody/tr[2]/td[2]/div/table/tbody/tr/td/div/input")->click();
        $self->wait_for_element_not_present("LOADING");
        if (exists($args{"-lunProvFC"}) ) {
            $Log->comment( "Entering lun prov: " . $args{"-lunProvFC"});
            $self->_find_element("//tr[2]/td/div/div[2]/div/form/div[3]/table/tbody/tr/td/div/table/tbody/tr/td/div/table/tbody/tr/td[3]/div/table/tbody/tr[2]/td[2]/div/table/tbody/tr/td/div/input")->send_keys($args{"-lunProvFC"});
        }
        if (exists($args{"-lunSizeUnitFC"})) {
            $Log->comment("Selecting Lun size Unit: " . $args{"-lunSizeUnitFC"});
            $self->_find_element("//tr[2]/td/div/div[2]/div/form/div[3]/table/tbody/tr/td/div/table/tbody/tr/td/div/table/tbody/tr/td[3]/div/table/tbody/tr[2]/td[2]/div/table/tbody/tr/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunSizeUnitFC"} . "']")->click();
        }
        if (exists($args{"-lunOSFC"})) {
            $Log->comment("Selecting Lun OS Type: " . $args{"-lunOSFC"});
            $self->_find_element("//tr[2]/td/div/div[2]/div/form/div[3]/table/tbody/tr/td/div/table/tbody/tr/td/div/table/tbody/tr/td[3]/div/table/tbody/tr[3]/td[2]/div/img")->click();
            $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-lunOSFC"} . "']")->click();
        }
        if (exists($args{"-lunInitiatorFC"})) {
            $Log->comment( "Entering initiator: " . $args{"-lunInitiatorFC"});
            $self->_find_element("//tr[2]/td/div/div[2]/div/form/div[3]/table/tbody/tr/td/div/table/tbody/tr/td/div/table/tbody/tr/td[3]/div/table/tbody/tr[4]/td[2]/div/input")->send_keys($args{"-lunInitiatorFC"});
        }

        if (exists($args{"-fcoeAdvanced"}) ) {
            $Log->comment("Clicking on Advanced Setting CheckBox");
            $self->_find_element("//div[\@id='gwt-debug-" . $TAB . "fcoePageAdvancedSettingsCheckbox']/input")->click();
            if (exists($args{"-fcp-portSets"}) ) {
                $Log->comment("Entering number of portsets: " . $args{"fcoe-portSets"});
                $self->_find_element("gwt-debug-" . $TAB . "-fcoePortsetSpinner-spinnerField-input")->send_keys("");
                $self->_find_element("gwt-debug-". $TAB . "fcPortsetSpinner-spinnerField-input")->send_keys($args{"fcoe-portSets"});
            }
            if (exists($args{"-fcoeInterfaceName"}) ) {
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->click();
                $Log->comment("Double Clicking the interface row");
                $self->_find_element("//div[contains(\@class,'x-grid3-col-interface-name')]/div")->_double_click();
                $self->wait_for_element_not_present("LOADING");
                $Log->comment("Entering interface name: " . $args{"-fcoeInterfaceName"});
                $self->_find_element("gwt-debug-" . $TAB . "-fcoeInterfaceNameCol-input")->send_keys("");
                $self->_find_element("gwt-debug-" . $TAB . "-fcoeInterfaceNameCol-input")->send_keys($args{"-fcoeInterfaceName"});
            }
            if (exists($args{"-fcoePort"}) ) {
                $Log->comment("Clicking on port selection");
                $self->_find_element("gwt-debug-" . $TAB . "fcoeHomeportCol-input")->click();
    
                $Log->comment("Selecting port: " . $args{"-fcoePort"});
                $self->_find_element("//div[contains(\@class,'x-combo-list-item') and text()='" . $args{"-fcoePort"}. "']")->click();
            }
            $Log->comment("Clicking On Save Button");
            $self->_find_element("//div/table/tbody/tr/td/table/tbody/tr[2]/td[2]/em/button")->click()
        }
    }

    if (exists($args{"-vsAdmin"}) and $args{"-vsAdmin"} !~ /skip/) {
        $Log->comment( "Entering VsAdmin Password: " . $args{"-vsAdminPasswd"});
        $self->_find_element("gwt-debug-". $TAB . "VserverSetupAdminPageVserverSetupAdminPageNameAdminPasswordTextField-input")->send_keys($args{"-vsAdminPasswd"});
        $Log->comment( "Confirming Password" );
        $self->_find_element("gwt-debug-" . $TAB . "VserverSetupAdminPageVserverSetupAdminPageConfirmAdminPasswordTextField-input")->send_keys($args{"-vsAdminPasswd"});
        
        if (exists($args{"-vsMgtLif"}) and $args{"-vsMgtLif"} =~ /enable/i ) {
            my $checkbox = "//div[\@id='gwt-debug-" . $TAB . "VserverSetupAdminPageVserverSetupAdminPageNewDataLifCheckBox']/input";
            $self->set_checkbox($checkbox,$args{"-vsMgtLif"});
            # TODO : convert following lines into lib
            if ( exists($args{"-subnet"})) {
                $Log->comment("Clicking on Subnet text box");
                $self->_find_element("//form/table/tbody/tr/td[3]/div/input")->click();
                $Log->comment( "Selecting Subnet: " . $args{"-subnet"} );
                $self->_find_element("//div[contains(\@class,'ellipsis') and text()='" . $args{"-subnet"} . "']")->click();
            }
            if ( exists($args{"-node"}) and exists($args{"-port"}) ) {
                $Log->comment("Select port for LIF configuration");
                $self->click($VSERVER_CREATE_PORT_BROWSE_BTN);
                $self->wait_for_loading();
                $Log->comment( "Select Node containing port: ". $args{"-node"});
                $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, 1,1, $args{'node'});
                $self->wait_for_loading();
                $Log->comment( "Clicking on the port " . $args{"port"});
                $self->click_row_with_texts($VSERVER_CREATE_NET_PORT_INT_GRID, $args{'port'});
                
                #$self->_find_element("//div[contains(\@class,'x-tree3-el')]/span[contains(\@class,'x-tree3-node-text') and text()='". $args{'port'} . "']")->click();
        
                $Log->comment( "Clicking on ok");
                $self->click($GENERIC_DIALOG2_OK_BUTTON);
            }
        }
        if ($self->is_element_present("gwt-debug-" . $TAB . "VserverSetupDialogSubmitButton")) {
            $Log->comment( "Clicking On Submit Button");
            $self->_find_element("gwt-debug-" . $TAB . "VserverSetupDialogSubmitButton")->click();
        }
        $self->wait_for_element_not_present("CREATING");
        $self->wait_for_element_not_present("LOADING");
    } else {
        $Log->comment( "Clicking On Skip Button");
        $self->click($VSERVER_SETUPWIZARD_SKIP_BUTTON);
    }
    
    ###################################

    $self->wait_for_loading();
    $Log->comment( "Clicking on ok");
    $self->click($GENERIC_DIALOG2_OK_BUTTON, $GENERIC_WAIT );
    $self->wait_for_loading();
    $self->check_for_error();
    $self->wait_for_loading();
    my %times = ( loading_time => $loading_time );

    $Log->comment("vserver created and configured successfully");
    $Log->exit() if $may_exit;

    return \%times;
} ## end sub vserver_create

################################################################################
## @name ::vserver_edit
## @summary
##     Edit vserver details using GUI.
## @description
##     This method will edit vserver details using GUI.
## @synopsis
##     vserver_edit(%args);
## INPUTS TO METHOD CALL 'vserver_edit' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>(Required) Name of the Vserver  </td> </tr>
## <tr> <td> -changename </td> <td> Change name for the Vserver </td> </tr>
## <tr> <td> -nssnis </td> <td> Selecting Name Server Switch nis </td> </tr>
## <tr> <td> -nssfile </td> <td> Selecting Name Server Switch file </td> </tr>
## <tr> <td> -nssldap </td> <td> Selecting Name Server Switch ldap </td> </tr>
## <tr> <td> -nmsfile </td> <td> Selecting Name Mapping Switch file </td> </tr>
## <tr> <td> -nmsldap </td> <td> Selecting Name Mapping Switch file </td> </tr>
## <tr> <td> -cifs </td> <td> Selecting CIFS Protocol </td> </tr>
## <tr> <td> -nfs </td> <td> Selecting NFS Protocol </td> </tr>
## <tr> <td> -iscsi </td> <td> Selecting ISCSI Protocol </td> </tr>
## <tr> <td> -fc </td> <td> Selecting FC Protocol</td> </tr>
## <tr> <td> -volcreation </td> <td> Selecting Volume Creation </td> </tr>
## <tr> <td> -delegateaggrname </td> <td> Selecting Aggregate </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub vserver_edit {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-vservername" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'},
        $self->{$SYSTEM_NAME_KEY} );

    $Log->step("Selecting Vserver: $args{'-vservername'}");
    $self->click_row_with_texts( $VSERVER_GRID, $args{'-vservername'} );
    $self->wait_for_element_present( $VSERVER_EDIT_BUTTON, $GENERIC_WAIT );

    $Log->comment("Clicking vserver edit button");
    $self->click($VSERVER_EDIT_BUTTON);
    $self->wait_for_element_present( $VSERVER_EDIT_NAME_INPUT,
        $GENERIC_WAIT );

    my $confirmappears = 0;

    if ( exists( $args{'-changename'} ) ) {
        $Log->comment("Entering vserver name: $args{'-changename'}");
        $self->type( $VSERVER_EDIT_NAME_INPUT, $args{'-changename'} );
    }

    if ( exists( $args{'-nssnis'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting Name Server Switch nis");
        $self->set_checkbox_in_row_with_text( "$VSERVER_EDIT_NSS_GRID",
            $args{'-nssnis'}, "nis" );
    } ## end if ( exists( $args{'-nssnis'...}))

    if ( exists( $args{'-nssfile'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting Name Server Switch file");
        $self->set_checkbox_in_row_with_text( "$VSERVER_EDIT_NSS_GRID",
            $args{'-nssfile'}, "file" );
    } ## end if ( exists( $args{'-nssfile'...}))

    if ( exists( $args{'-nssldap'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting Name Server Switch ldap");
        $self->set_checkbox_in_row_with_text( "$VSERVER_EDIT_NSS_GRID",
            $args{'-nssldap'}, "ldap" );
    } ## end if ( exists( $args{'-nssldap'...}))

    if ( exists( $args{'-nmsfile'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting Name Mapping Switch file");
        $self->set_checkbox_in_row_with_text( "$VSERVER_EDIT_NMS_GRID",
            $args{'-nmsfile'}, "file" );
    } ## end if ( exists( $args{'-nmsfile'...}))

    if ( exists( $args{'-nmsldap'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting Name Mapping Switch ldap");
        $self->set_checkbox_in_row_with_text( "$VSERVER_EDIT_NMS_GRID",
            $args{'-nmsfile'}, "ldap" );
    } ## end if ( exists( $args{'-nmsldap'...}))

    $Log->comment("Clicking Protocols Tab");
    $self->click($VSERVER_EDIT_PROTOCOLS_TAB);
    $self->wait_for_element_present( $VSERVER_EDIT_PROTOCOLCIFS_CHECKBOX,
        $GENERIC_WAIT );

    if ( exists( $args{'-cifs'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting CIFS Protocol");
        $self->set_checkbox( "$VSERVER_EDIT_PROTOCOLCIFS_CHECKBOX",
            $args{'-cifs'} );
    } ## end if ( exists( $args{'-cifs'...}))

    if ( exists( $args{'-nfs'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting NFS Protocol");
        $self->set_checkbox( "$VSERVER_EDIT_PROTOCOLNFS_CHECKBOX",
            $args{'-nfs'} );
    } ## end if ( exists( $args{'-nfs'...}))

    if ( exists( $args{'-iscsi'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting iSCSI Protocol");
        $self->set_checkbox( "$VSERVER_EDIT_PROTOCOLISCSI_CHECKBOX",
            $args{'-iscsi'} );
    } ## end if ( exists( $args{'-iscsi'...}))

    if ( exists( $args{'-fc'} ) ) {
        $confirmappears = 1;
        $Log->comment("Selecting FC/FCoE Protocol");
        $self->set_checkbox( "$VSERVER_EDIT_PROTOCOLFC_CHECKBOX",
            $args{'-fc'} );
    } ## end if ( exists( $args{'-fc'...}))

    $Log->comment("Clicking Resource Allocation Tab");
    $self->click($VSERVER_EDIT_RESOURCEALLOCATION_TAB);

    if ( exists( $args{'-volcreation'} ) ) {
        if ( lc( $args{'-volcreation'} ) eq 'delegate' ) {
            $Log->comment(
                "selecting Volume Creation : $args{'-volcreation'}");
            $self->click("$VSERVER_EDIT_DELEGATE_RADIO");

            if ( exists( $args{'-delegateaggrname'} ) ) {
                #$Log->comment(
                #    "selecting Volume Creation : $args{'-volcreation'}");
                #$self->click("$VSERVER_EDIT_DELEGATE_RADIO");
                sleep(10);
                my @delegateaggrsplit = split( /:/, $args{'-delegateaggrname'} );
                foreach (@delegateaggrsplit) {
                    my $convertstr = ucfirst($_);
                    $Log->comment("Selecting aggregate: $convertstr");
                    $self->click("//div[contains(\@id,'$convertstr')]/input");
                }
                #$Log->comment(
                #    "selecting Aggregate : $args{'-delegateaggrname'}");
                #$self->set_checkbox_in_row_with_text(
                #    "$VSERVER_EDIT_DELEGATE_GRID", "enable",
                #    $args{'-delegateaggrname'} );

            } else {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "Aggregate is mandatory for delegated volume creation");
            }
        } elsif ( lc( $args{'-volcreation'} ) eq 'nodelegate' ) {
            $Log->comment(
                "selecting Volume Creation : $args{'-volcreation'}");
            $self->click("$VSERVER_EDIT_NODELEGATE_RADIO");
        }
    } ## end if ( exists( $args{'-volcreation'...}))

    $Log->comment("Clicking Save and Close Button");
    my $commit_time;
    if ( not($confirmappears) ) {
        $commit_time
            = $self->click_timecount($VSERVER_EDIT_SAVEANDCLOSE_BUTTON);
    } elsif ($confirmappears) {
        $self->click($VSERVER_EDIT_SAVEANDCLOSE_BUTTON);
        $Log->comment("Clicking Confirmation OK Button");
        $commit_time = $self->click_timecount($VSERVER_EDIT_CONFIRMOK_BUTTON);

    } ## end elsif ($confirmappears)
    $self->wait_for_element_not_present( $VSERVER_EDIT_SAVEANDCLOSE_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Vserver edit operation completed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_edit

################################################################################
## @name ::vserver_delete
## @summary
##     Delete vserver details using GUI.
## @description
##     This method will delete vserver details using GUI.
## @synopsis
##     vserver_delete(%args);
## INPUTS TO METHOD CALL 'vserver_delete' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>(Required) Name of the Vserver  </td> </tr>
## <tr> <td> -stop </td> <td> Selecting Stop Button  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub vserver_delete {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-vservername" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'vserverame'},
        $self->{$SYSTEM_NAME_KEY} );

    $Log->step("Selecting Vserver: $args{'-vservername'}");
    $self->click_row_with_texts( $VSERVER_GRID, $args{'-vservername'} );
    $self->wait_for_element_present( $VSERVER_STOP_BUTTON, $GENERIC_WAIT );

    if ( exists $args{'-stop'} ) {
        $Log->comment("Click Stop Button");
        $self->click_button("$VSERVER_STOP_BUTTON");
        $self->wait_for_element_present( $VSERVER_STOP_CONFIRMATION_CHECKBOX,
            $GENERIC_WAIT );

        $Log->comment("Select Stop confirmation checkbox");
        $self->set_checkbox( "$VSERVER_STOP_CONFIRMATION_CHECKBOX",
            "enable" );
        $self->wait_for_element_present( $VSERVER_STOP_CONFIRMATION_BUTTON,
            $GENERIC_WAIT );

        $Log->comment("Click Stop Button on confirmation");
        $self->click_button("$VSERVER_STOP_CONFIRMATION_BUTTON");
        $self->wait_for_element_present( $VSERVER_DELETE_BUTTON,
            $GENERIC_WAIT );
    } ## end if ( exists $args{'-stop'...})

    $Log->comment("Click Delete Button");
    $self->click_button("$VSERVER_DELETE_BUTTON");
    $self->wait_for_element_present( $VSERVER_DELETE_CONFIRMATION_CHECKBOX,
        $GENERIC_WAIT );

    $Log->comment("Select Delete confirmation checkbox");
    $self->set_checkbox( "$VSERVER_DELETE_CONFIRMATION_CHECKBOX", "enable" );
    $self->wait_for_element_present( $VSERVER_DELETE_CONFIRMATION_BUTTON,
        $GENERIC_WAIT );

    $Log->comment("Click Delete Button on confirmation");
    my $commit_time
        = $self->click_timecount("$VSERVER_DELETE_CONFIRMATION_BUTTON");
    $self->wait_for_element_not_present( $VSERVER_DELETE_CONFIRMATION_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Vserver deleted successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_delete

################################################################################
## @name ::vserver_stop
## @summary
##     Stop vserver using GUI.
## @description
##     This method will stop vserver details using GUI.
## @synopsis
##     vserver_stop(%args);
## INPUTS TO METHOD CALL 'vserver_stop' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>(Required) Name of the Vserver  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.
sub vserver_stop {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-vservername" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'},
        $self->{$SYSTEM_NAME_KEY} );
    $self->wait_for_loading();

    $Log->step("Selecting Vserver: $args{'-vservername'}");
    $self->click_row_with_texts( $VSERVER_GRID, $args{'-vservername'} );
    $self->wait_for_loading();

    sleep 5;
    $Log->comment("Click Stop Button");
    $self->click_button("$VSERVER_STOP_BUTTON");

    $self->wait_for_loading();
    $Log->comment("Select Stop confirmation checkbox");
    $self->set_checkbox( "$VSERVER_STOP_CONFIRMATION_CHECKBOX", "enable" );
    $self->wait_for_loading();

    $Log->comment("Click Stop Button on confirmation");
    my $commit_time
        = $self->click_timecount("$VSERVER_STOP_CONFIRMATION_BUTTON");
    $self->wait_for_element_not_present( $VSERVER_STOP_CONFIRMATION_BUTTON,
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("Vserver stopped successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_stop

################################################################################
## @name ::vserver_start
## @summary
##     Start vserver using GUI.
## @description
##     This method will start vserver details using GUI.
## @synopsis
##     vserver_start(%args);
## INPUTS TO METHOD CALL 'vserver_start' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>(Required) Name of the Vserver  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub vserver_start {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-vservername" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'},
        $self->{$SYSTEM_NAME_KEY} );
    $self->wait_for_loading();

    $Log->step("Selecting Vserver: $args{'-vservername'}");
    $self->click_row_with_texts( $VSERVER_GRID, $args{'-vservername'} );
    $self->wait_for_element_present( $VSERVER_START_BUTTON, $GENERIC_WAIT );
    sleep 5;

    $Log->comment("Click Start Button");
    my $commit_time
        = $self->click_timecount( "$VSERVER_START_BUTTON", "refresh" );
    $self->wait_for_element_present( $VSERVER_REFRESH_BUTTON, $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $self->wait_for_loading();
    $Log->comment("Vserver started successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_start

################################################################################
## @name ::vserver_refresh
## @summary
##     Refresh vserver using GUI.
## @description
##     This method will refresh vserver details using GUI.
## @synopsis
##     vserver_refresh(%args);
## INPUTS TO METHOD CALL 'vserver_refresh' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.

sub vserver_refresh {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'},
        $self->{$SYSTEM_NAME_KEY} );

    $Log->comment("Click Refresh Button");
    my $commit_time
        = $self->click_timecount( "$VSERVER_REFRESH_BUTTON", "refresh" );
    $self->wait_for_element_present( "$VSERVER_REFRESH_BUTTON",
        $GENERIC_WAIT );

    $self->check_for_error();

    my %times = (
        loading_time => $loading_time,
        commit_time  => $commit_time
    );

    $Log->comment("Vserver Page Refreshed successfully");
    $Log->exit() if $may_exit;

    return \%times;

} ## end sub vserver_refresh

################################################################################
## @name ::get_vserver_info
## @summary
##    Return the details about the Vserver using GUI.
## @description
##     This method will return the details about the Vserver using GUI.
## @synopsis
##     get_vserver_info(%args);
## INPUTS TO METHOD CALL 'get_vserver_info' ON APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> -vservername </td> <td>(Required) Name of the Vserver  </td> </tr>
## </table>
##
## @return
##     Returns an array reference which contains the VServer Status Details.

sub get_vserver_info {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        spec        => { "-vservername" => { type => SCALAR }, },
        allow_extra => 1
    );

    $Log->enter() if $may_enter;
    $self->clear_window();
    my %values;

    $Log->debug( sub { "Input Hash contains " . Dumper( \%args ) } );

    $Log->step("Navigating to $self->{$SYSTEM_NAME_KEY}");
    my $loading_time = $self->cluster_tree_navigate( $args{'-vservername'},
        $self->{$SYSTEM_NAME_KEY} );

    $Log->step("Selecting Vserver: $args{'-vservername'}");
    $self->click_row_with_texts( $VSERVER_GRID, $args{'-vservername'} );
    $self->wait_for_element_present( "$VSERVER_REFRESH_BUTTON",
        $GENERIC_WAIT );

    #$values{"$VSERVER_DETAILSTAB_NAME_KEY"}
    #    = $self->get_text("$VSERVER_DETAILSTAB_NAME")
    #    if ( $self->is_element_present("$VSERVER_DETAILSTAB_NAME") );

    $values{"$VSERVER_DETAILSTAB_NSS_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_NSS")
        if ( $self->is_element_present("$VSERVER_DETAILSTAB_NSS") );

    $values{"$VSERVER_DETAILSTAB_NMS_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_NMS")
        if ( $self->is_element_present("$VSERVER_DETAILSTAB_NMS") );

    $values{"$VSERVER_DETAILSTAB_SNAPSHOTPOLICY_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_SNAPSHOTPOLICY")
        if (
        $self->is_element_present("$VSERVER_DETAILSTAB_SNAPSHOTPOLICY") );

    $values{"$VSERVER_DETAILSTAB_NISDOMAIN_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_NISDOMAIN")
        if ( $self->is_element_present("$VSERVER_DETAILSTAB_NISDOMAIN") );

    $values{"$VSERVER_DETAILSTAB_LDAPCLIENT_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_LDAPCLIENT")
        if ( $self->is_element_present("$VSERVER_DETAILSTAB_LDAPCLIENT") );

    $values{"$VSERVER_DETAILSTAB_LANGUAGUE_KEY"}
        = $self->get_text("$VSERVER_DETAILSTAB_LANGUAGE")
        if ( $self->is_element_present("$VSERVER_DETAILSTAB_LANGUAGE") );

    my $return_hash = {};
    $return_hash->{parsed_output} = \%values;
    $return_hash->{loading_time}  = $loading_time;

    $Log->exit() if $may_exit;
    return $return_hash;

} ## end sub get_vserver_info

################################################################################
## @name ::clear_vserver_window
## @summary
##   Clear vserver window using GUI.
## @description
##     This method will clear vserver window using GUI.
## @synopsis
##     clear_vserver_window(%args);
## INPUTS TO METHOD CALL 'clear_vserver_window' ON APISET OBJECT
##
## @return
##     Returns an array reference which contains the Loading time and Commit Time.
sub clear_vserver_window {

    my ($self) = @_;

    $Log->enter() if $may_enter;
    $self->clear_window();

    $Log->comment("Click Cancel Button");
    $self->click("$VSERVER_CREATE_CLOSE_BUTTON");

    $Log->comment("Click Close Button");
    $self->click("$VSERVER_CREATE_CLOSE_BUTTON");

    $Log->comment("Click Confirmation dialog Box ");
    $self->click("$VSERVER_CREATE_EXIT_BUTTON");

    $Log->comment("Vserver Window Cleared successfully");

    return 1;
} ## end sub clear_vserver_window

1;

