#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets SOAP commands for Host
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::SOAP;

#
# Compiler Directives
#
use strict;
use warnings;
use Params::Validate qw(:all);

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#
# Exceptions imported
#
use NATE::BaseException qw(:try);

# Base class
use base qw(NACL::APISet::Host);

our $AUTOLOAD;

# This is what most constructors of Host APISet does
# - just bless the object
sub new {
    $Log->enter() if $may_enter;
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    $Log->exit() if $may_exit;
    return $self;
}

###############################################################################
# Method Name: get_host
# Objective:  Return stored host name
# Details:    Refer to POD documentation at the end of the file
###############################################################################

sub get_host {
    my $self = shift;
    my $host;
    if ( $self->hostobj ) {
        $host = $self->{hostobj}->hostname();
    } else {
        $host = $self->{hostname};
    }
    return $host;
} ## end sub get_host

###############################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call invoke
#             If the API call is prefixed with an "xlate_" prefix, it should
#             only be translated and not invoked, so we should call
#             _only_translate_method
###############################################################################

sub AUTOLOAD {
    return if our $AUTOLOAD =~ /::DESTROY$/;

    my ( $self, %opts ) = @_;
    $Log->enter() if $may_enter;

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion

    # Set params for the call
    $opts{command} = $method;

    # Call invoke with the arguments given to execute the command
    my $response = $self->{session}->send_command( %opts );

    $Log->exit() if $may_exit;
    return $response;
} ## end sub AUTOLOAD

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Host API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path.
##############################################################################

sub _init_API {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    # There is nothing to init, and there is no need to call
    # parent class method for SOAP interfaces

    $Log->exit() if $may_exit;
} ## end sub _init_API

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Host Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec( \%args );

    my $interface = $self->{interface} // 'SOAP';

    # Establish the session if not provided
    # Determine the type of the session based on the set.
    my $session_type = $self->{set};

    # Establish the session if not provided
    my $session_pkg
        = "NACL::APISet::Session::" . $interface . "::" . $session_type;

    # Create a Session object. This takes care of connecting to the service
    # and preparing it for use.
    $self->{session} = $session_pkg->new(%args);

    $Log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   execute command
# Details:     This method invokes the command along with its options and returns
#              its output. It does this by invoking the command on the server
#              using the Session object.
###############################################################################

sub _invoke {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $return_response_object = delete $args{'return_response_object'} // 1;

    # execute command
    my $result;
    if ( $return_response_object ) {
        $result = $self->{session}->send_command(
            command      => $args{command},
            params       => $args{params},
            timeout      => $args{timeout},
        );
    } else {
        $self->{session}->send_command(
            command      => $args{command},
            params       => $args{params},
            timeout      => $args{timeout},
        );
    }

    $Log->exit() if $may_exit;
    return $result;
} ## end sub _invoke

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::SOAP - Base for API Sets using SOAP interfaces

=head1 SYNOPSIS

 use NACL::APISet::Host::SOAP;

=head1 DESCRIPTION

A C<NACL::APISet::Host::SOAP> is used to connect to SOAP appliance and send SOAP commands. This
is further subclassed to send SOAP commands to specific SOAP appliances.

=head1 METHODS

NOTE: No cdef's or schemas are used for SOAP interfaces. All calls to the APISet invoke the AUTOLOAD method.

=head2 get_host

This method is used to return the stored hostname.

=over

=item Synopsis

$hostname = $Api_Set_Obj->get_host();

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
