#
# Copyright (c) 2012-* NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Initialize Xen APISet Session object creation and call Xen object method on it
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::XAPI;


# NATE imports
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

# Module import - APISets
use base qw(NACL::APISet::Host);
use NACL::APISet::Session::XAPI::Xen;
use NACL::APISet::Response::XAPI::Xen;

# Imports CPAN libs
use RPC::XML::Client;

# Import Exceptions
use NACL::APISet::Exceptions::ResponseException qw(:try);

our $AUTOLOAD;
#####################################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call the
#             method on Xen object which is the value of $self->{session}->{connection}.
#             This method also constructs and returns a response object
#
######################################################################################

sub AUTOLOAD {
    $Log->enter() if $may_enter;

    my ($self, @args) = @_;

    my $method = $AUTOLOAD;

    $method =~ s/.*://;    # strip fully-qualified portion
    if ($method eq "DESTROY") {
        $Log->exit() if $may_exit;
        return;
    }
    
    my $session = $self->xen_session();
    
    # RPC client object only accepts unnamed parameters
    # First two arguments (request, $session) are fixed
    # Rest of the arguments keeps changing depending upon
    # command requested. Since $session is fixed for a
    # particular Xen host, It is not suppose to be passed
    # to APISet object because it will be provide here
    
    # Pushing session object at index position 1.
    splice(@args, 1, 0, $session);
    
    $Log->comment("Executing command: " . $method . "('" . $args[0] . "')");
    $Log->debug(sub { "Parameters passed to command are" . Dumper(\@args)});
    my $time_start = time();
    my $ret = $self->rpc_clientobj()->$method(@args);
    
    $Log->debug(Dumper($ret));

    my $time_end = time();
    my $execution_time = $time_end - $time_start;

    my ( $output, $response_obj);
    
    if ($ret->{Status} eq 'Failure') {
        $output = $ret->{ErrorDescription};
    } else {
        $output = $ret->{Value};
    }

    # Sometime output is return as a scalar
    # Making it as array ref
    if (! ref ($output)) {
        $output = [ $output ];
    }
    
    $response_obj = NACL::APISet::Response::XAPI::Xen->new(
        parsed_output   => $output,
        output          => $output,
        method_name     => $method,
        execution_time  => $execution_time,
    );

    if ($ret->{Status} eq 'Failure') {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ResponseException
                    ->throw(text => $output->[0], response_object => $response_obj);
    }

    $Log->exit() if $may_exit;
    return $response_obj;
}

sub _init_Session {
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;
    my $interface = $self->{interface};
    my $session_type = $self->{set};

    my $session_pkg = "NACL::APISet::Session::" . $interface . "::" . $session_type;
    $self->{session} = $session_pkg->new(%args, hostobj => $self->hostobj());

    $Log->exit() if $may_exit;
}


=head1 NAME

NACL::APISet::Host::XAPI - Initialize Xen APISet Session object creation and call
Xen object method on it

=head1 SYNOPSIS

 use NACL::APISet::Host::XAPI;

=head1 DESCRIPTION

A C<NACL::APISet::Host::XAPI> is used to construct the Xen APISet session object.
This is subclassed as an additional Host interface

=head1 EXCEPTIONS
NACL::APISet::Exceptions::ResponseException

The error output from the Xen object method call is passed on through
NACL::APISet::Exceptions::ResponseException

=cut

1;
