#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets ZAPI commands
## @author tanzim@netapp.com, dl-sm-nacl-dev@netapp.com, nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::ZAPI;

#
# Compiler Directives
#
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);

# Module import - APISets
use NACL::APISet::Schema::ZapiSchema;

use NACL::APISet::_Mixins::ZAPI qw(:all);

#
# Exceptions imported
#
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Base class
use base qw(NACL::APISet::Host);

my $FALSE = 1;

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw ZAPI command on the DFM host and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    $Log->enter() if $may_enter;

    my $self = shift();

    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            "command"                => { type => SCALAR, optional => 0 },
            'zapi-timeout'           => { type => SCALAR, optional => 1 },
            'return-response-object' => { type => SCALAR, optional => 1 },
            'log_level'              => { type => SCALAR, optional => 1 },
            'input_log_level'        => { type => SCALAR, optional => 1 },
            'output_log_level'       => { type => SCALAR, optional => 1 },
        }
    );

    # If users had requested for a response object, then return a response
    # object even for the direct XML inputs.
    my $need_resp_obj;
    if ( defined $args{'return-response-object'} ) {
        $need_resp_obj = $args{'return-response-object'};
    }
    elsif ( defined $self->{'return-response-object'} ) {
        $need_resp_obj = $self->{'return-response-object'};
    }
    $need_resp_obj = ( defined $need_resp_obj ) ? $need_resp_obj : 0;
    $Log->debug("Response object is requested") if ($need_resp_obj);

    my $output =
      $self->{session}
      ->execute_raw_ZAPI( %args, 'return-response-object' => $need_resp_obj );

    $Log->exit() if $may_exit;
    return $output;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the DFM API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path.
##############################################################################

sub _init_API {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $cdef_path = $self->{cdef_path};
    my $api_set   = $self->{set};

    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path.
    $self->{schema} = NACL::APISet::Schema::ZapiSchema->new();
    $self->{schema}->{searched_cdef_path} = $cdef_path;
    $self->{schema}->load( path => $cdef_path );

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the DFM Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    $self->_get_connectrec_or_hostrec( \%args );

    $args{set} = $self->{actual_set};
    my $interface = $self->{interface};

    # Establish the session if not provided

    # Setting the type of the session .
    my $session_type = $self->{set};

    my $session_pkg =
      "NACL::APISet::Session::" . $interface . "::" . $session_type;

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $Log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into input xml and execute
# Details:     This method invokes the ZAPI along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual xml to be invoked on the
#              service, and then invoking that on the host using the Session
#              object.
###############################################################################

sub _invoke {
    my ( $self, %args ) = @_;

    my $cdef_method = $args{cdef_method};
    my $raw_data    = $FALSE;

    $Log->enter() if $may_enter;

    my $timeout = undef;
    my ( $log_level, $input_log_level, $output_log_level );

    # process parameters
    while ( my ( $key, $value ) = each(%args) ) {
        if ( $key =~ /-?cdef$/ ) {
            if ( !length($value) ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }
        }
        elsif ( $key =~ /-?debug_level$/ ) {
            if ( !length($value) || $value !~ /\d+/ ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
        elsif ( $key =~ /^-?zapi-timeout$/ ) {

            # Timeout override
            if ( !defined( $timeout = $value ) ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
        elsif ( $key =~ /^log_level$/ ) {

            # log_level override
            if ( !defined( $log_level = $value ) ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
        elsif ( $key =~ /^input_log_level$/ ) {

            # log_level override
            if ( !defined( $input_log_level = $value ) ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
        elsif ( $key =~ /^output_log_level$/ ) {

            # log_level override
            if ( !defined( $output_log_level = $value ) ) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value );
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
    }

    # verify mandatory cdef argument
    unless ( defined $args{cdef_method} and length $args{cdef_method} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Mandatory argument \"cdef_method\" not specified ");
    }

    my $schema_obj = $self->get_schema_obj();

    # translate cdef_method schema into command
    my $cmd = $schema_obj->translate( cdef_method => $cdef_method, %args );
    my $xmlStr = $schema_obj->getSimpleXML( logxml => $cmd );

    my $result_obj = $self->{session}->send_command(
        command          => $xmlStr,
        "zapi-timeout"   => $timeout,
        log_level        => $log_level,
        input_log_level  => $input_log_level,
        output_log_level => $output_log_level
    );

    $Log->exit() if $may_exit;
    return $result_obj;

} ## end sub _invoke

###############################################################################
# Method Name: _only_translate_method
# Objective:   Translate the API call into the command invoked on the service
#              and return this without executing it.
# Details:     When the API is prefixed by an "xlate_", it must only translate
#              the API call without executing it. This will come in handy
#              while developing new APISets and for cases where undoing
#              API calls can be cumbersome.
#              This method strips the leading xlate_ prefix, translates
#              the API and returns the command
###############################################################################
sub _only_translate_method {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $cdef_method = $args{method};
    delete( $args{method} );
    $cdef_method =~ s/^xlate_//;
    $cdef_method =~ s/\_/\-/g;
    my $schema_obj = $self->get_schema_obj();
    my $command = $schema_obj->translate( cdef_method => $cdef_method, %args );
    my $xmlStr = $schema_obj->getSimpleXML( logxml => $command );
    $Log->exit() if $may_exit;
    return $xmlStr;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::ZAPI - Base for API Sets for ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Host::ZAPI;

=head1 DESCRIPTION

A C<NACL::APISet::Host::ZAPI> is used to connect to and send ZAPI commands. This
is further subclassed to send ZAPI commands specifically to hosts like DFM etc.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument 

=back

=cut

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw ZAPI commands on the host . The input xml to
be executed should be provided by the user. This will be invoked and the raw 
xml output will be returned as a string directly without any processing. 
This is useful for cases where we want to invoke a command without parsing the 
output or if the users want to invoke any ZAPIs whose CDEF files(.xml files) are
not yet generated.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(
command => "<aggr-add><aggregate>tempAggr</aggregate></aggr-add>");

=item Arguments

=over

=item command

(Mandatory)
The command (input xml) to be invoked.

=item zapi-timeout

(Optional)
Override timeout value for ZAPI execution.

=item return-response-object

(Optional)
Boolean option. If set, a response object will be returned else raw XML output
will be returned. By default, this option will be set to 0.

=back

=item Returns

Raw output or Response object depending on 'return-response-object' option.

=back

=head2 set_port

This method sets the port number to which the ZAPI input requests are to be
sent.

=over

=item Synopsis

$Api_Set_Obj->set_port('port' => "82");

=item Arguments

=over

=item port

Required
Port number

=back

=item Returns

Nothing.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
