#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for DFM ZAPI commands
## @author tanzim@netapp.com, dl-sm-nacl-dev@netapp.com, nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::ZAPI::DFM;

#
# Compiler Directives
#
use strict;
use warnings;

# Module import - APISets
use NACL::APISet::Session::ZAPI::DFM;

# Exceptions imported
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Host::ZAPI);
use NACL::APISet::_Mixins::CLI qw(_load_version_xml);

# global variables
use constant DFM => "DFM";

# Constructor
sub new {
    $Log->enter() if $may_enter;

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my $class = shift;
    my %args  = @_;
    my $self  = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            version => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the DFM API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
# to make the API Set ready for work.  This is accomplished by calling the
# parents' init_API method and performing work specific to this class afterwards.
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file( $self->{cdef_path} );

    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }

    $self->SUPER::_init_API( %args, set => $self->{actual_set} );

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the DFM Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.	
##############################################################################

sub _init_Session {
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;
   
    $self->SUPER::_init_Session( %args, set => $self->{actual_set} );   
    $self->{session}->set_server_type( "server-type" => DFM );
    $Log->exit();   
}

###############################################################################
# Method:    find_cdef_directory
# Objective: To find the cdef directory for DFM based on various version related
#            attributes obtained from the service.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub find_cdef_directory {
    $Log->enter() if $may_enter;
    my $self = shift();

    # Load CDEFs from the CDEF directories which we maintain based on branch
    # and release information.
    my %required_attributes = ( "DFM" => ["release"], );

    my $cdef_dir;
    my $attr_info;
    my $type = $self->{set};

    if ( defined( $self->{version_manager} ) ) {
        $attr_info = $self->_find_cdef_directory(
            required_attributes => $required_attributes{$type} );
    }

    my $release = $attr_info->{release};
    if ( defined $release ) {

        # release should be normalized in lowercase so that its case
        # insensitive when we try to map it to its codename.
        $release = lc($release);
        try {
            $release = $self->{version_manager}
                ->map_release_to_codename( release => $release );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Caught an exception while trying to map $release "
                    . "to its codename: "
                    . $ex->text() );

            # Emit a warning, set the normalized release name to undef
            # and then proceed.
            $Log->warn( "Versioning hit the error: "
                    . $ex->text()
                    . "\nRelease mapping may be missing for release $release."
                    . "\nPlease contact dl-sm-nacl-dev about this warning."
                    . "\n Loading the default CDEF Directory." );
            $release = undef;
        };
    }

    # Load version.xml file
    my $libxml_obj = $self->_load_version_xml();
    
    # Discover the CDEF path
    $cdef_dir = $libxml_obj->find("/version/host/zapi/dfm/default_cdef_dir")
    ->string_value;

    unless ( ( defined $release ) ) {
        $Log->debug("Release information could not be obtained");
        $Log->exit() if $may_exit;
        return $cdef_dir;
    }

    # Discover the CDEF path for specific release
    my $release_cdef_dir = $libxml_obj->find("/version/host/zapi/dfm/release[\@value='" .
                    $release . "']/release_cdef_dir")->string_value();
    if (! defined $release_cdef_dir or $release_cdef_dir eq "") {
        $Log->debug("CDEF directory is not yet supported for $release");

        $Log->warn( "CDEF directory corresponding to release '$release' "
                . "doesn't exist. Please contact dl-sm-nacl-dev for this warning."
                . " Loading CDEFs from default CDEF directory defined for the"
                . " set '$self->{set}'." );
    } else {
        $Log->debug(
            "A release specific CDEF directory is found for " . $release );
        $cdef_dir = $release_cdef_dir;
    }

    $Log->exit() if $may_exit;
    return $cdef_dir;
}

=head1 NAME

NACL::APISet::Host::ZAPI::DFM - API Set for DFM ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Host::ZAPI::DFM;

=head1 DESCRIPTION

A C<NACL::APISet::Host::ZAPI::DFM> is used to connect to and send ZAPI commands to the
DFM host.

=cut

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::ZAPI::DFM>.

=head2 find_cdef_directory

This method gets the information of various version related attributes and uses
them to autodiscover the CDEF path. The data about the CDEF directory to
be loaded is maintained in a separate XML file from which it finds the CDEF path
based on the discovered version attributes. The version attributes to be
discovered will vary depending on the category and set.

=over

=item Synopsis

my $cdef_dir = $Api_Set_Obj->find_cdef_directory();

=over

=item Arguments

Nothing.

=item Returns

Returns the relative path to CDEF directory for DFM.

=back

=item Synopsis

 $api_set_obj = NACL::APISet::Host::ZAPI::DFM->new( %args );

=over 

=item Arguments

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and NACL::APISet::Host::ZAPI for details.

=item Returns

A reference to a new API Set for DFM.

=back

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
