#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets CLI commands
## @author pressley@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::CLI;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
use Data::Dumper;
use Params::Validate qw(validate_with SCALAR HASHREF);

#
# Module import - APISets
#
use NACL::APISet::Schema;
use NACL::APISet::_Mixins::CLI qw(:all);
use NACL::APISet::_Mixins::CLIZAPI qw(:baseline_opts);
use NATE::ParamSet qw( param_global);

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::TimeoutException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MgwdCrashedException;

# Base class
use base qw(NACL::APISet::Node);

our $AUTOLOAD;

use Class::MethodMaker [
    scalar => [
        { -default_ctor => sub { $_[0]->_get_highest_possible_privilege() }
        },
        'highest_possible_privilege'
    ],
];

###############################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call invoke
#             If the API call is prefixed with an "xlate_" prefix, it should
#             only be translated and not invoked, so we should call
#             _only_translate_method
###############################################################################

sub AUTOLOAD {
    my $self = shift;
    $Log->enter();

    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion

    my $spec = {};
    if ($method ne 'execute_command') {
        # extended_query is not applicable for execute_command
        $spec = { extended_query => { type => HASHREF, default => {} } };
    }
    my %args = $self->validate_params(
        params => \@_,
        spec   => $spec,
        allow_extra => 1
    );
    $Log->debug( sub { "Options to '$method' are:\n" . Dumper( \%args ) } );

    # convert the space separated value(s) string which does NOT contain
    # the switch between the values to one where the switch is present before
    # each of those values. If multiple values are specified and the switch is
    # already specified before each value after the first one, nothing is
    # done to the string

    if ( $method =~ /^(xlate_)?vfiler_/ ) {
        $self->_vfiler_option_handler( \%args );
    } elsif ( $method =~ /^(xlate_)?vol_move_/ ) {

        # manual-cutover is defined as -M for vol copy
        # manual-cut-over is defined as -m for vol move
        # using either key name is allowed for vol move
        $args{'manual-cut-over'} = delete $args{'manual-cutover'}
            if defined $args{'manual-cutover'};
    }
    # To solve burt 638911
    delete $args{'nodescope-node-name'} if ($self->{actual_set} eq "7Mode");

    if ( $method =~ /^xlate_/ ) {
        if ( $method =~ /^xlate_execute(_raw)?_command/ ) {
            my $name = $1 || "";

            # Explicitly prevent xlate_ being used with execute_raw_command
            $Log->exit();
            NATE::BaseException->throw(
                "xlate cannot be used " . "with execute$name" . "_command" );
        } else {
            $Log->exit();
            return $self->_only_translate_method( method => $method, %args );
        }
    } else {
        my $_invoke;
        my $pre_method  = '_'.$method . '_pre';
        my $post_method = '_'.$method . '_post';
        my $result_hash = {};
        if ($self->can($pre_method)) {
            $result_hash = $self->$pre_method( args => \%args);
        }
        try {
            # Call invoke with the arguments given to execute the command
            $_invoke = $self->_invoke( method => $method, %args );
         }
        catch NATE::BaseException with {
            my $exception = shift;
            if ( $exception->text() =~ /rsh server returned error:/ ) {
                NACL::APISet::Exceptions::ConnectionFailedException->throw(
                    $exception->text() );
            } else {
                $exception->throw();
            }
        };
        if ($self->can($post_method)) {
            $self->$post_method(
                args => \%args,
                pre_result => $result_hash,
                resp_obj   => $_invoke,
            );
        }
        $Log->exit();
        return $_invoke;
    }
} ## end sub AUTOLOAD

sub clear_privilege {
    $Log->enter();
    my ($self) = @_;

    # Set to the highest possible privilege level
    my $highest_priv = $self->highest_possible_privilege();
    try {
       $self->set_privilege( 'privilege-level' => $highest_priv );
    }
    catch NACL::APISet::Exceptions::CommandNotFoundException with {
        my $e = shift;
        if($e =~ /"set" is not a recognized command/ && $self->{'no_set_command'} == 1 ) {
           #supressing set is not a recognized command error
           #For reference view burt 901243
        }
        else {
           $e->throw();
        }
    }
    $Log->exit();
}

sub maximum_privilege {
    $Log->enter();
    $Log->exit();
    return 'test';
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Node API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path.
##############################################################################

sub _init_API {
    my ( $self, %args ) = @_;
    $Log->enter();

    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path. Load CDEFS after the check for
    # shell_type is done so that we need not unnessarily load CDEFs even in
    # case of invalid shell_type.
    my $cdef_path = $self->{cdef_path};
    my $is_cmode_cli = $self->{set} eq 'CMode' ? 1 : 0;
    $self->{schema} = NACL::APISet::Schema->new(is_cmode_cli => $is_cmode_cli);
	$self->{schema}->{searched_cdef_path} = $cdef_path;

   # Get the list of th CDEF files to be loaded. The implementation of getting
   # the CDEF files list will go into the specific APISet class.
    my @files;
    if (   ( defined $self->{version_info}->{cdefs_from_build} )
        && ( $self->{version_info}->{cdefs_from_build} ) )
    {
        @files = $self->_get_cdef_files_list( buildroot => $cdef_path );
    } else {

        # Handle linux, windows and HTTP paths as well.
        @files
            = @{ $self->{schema}->read_cdef_dir( directory => $cdef_path ) };
    }

    # if files list is empty, cdef files have not been consolidated
    if ( scalar(@files) == 0) {
        $Log->exit();
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "No cdef data files found in: $cdef_path ");
    }

    # Load the cdef files.
    # option cdefs_from_build is needed to load *.aliases files
    my $cdefs_from_build = $self->{version_info}->{cdefs_from_build} ? $self->{version_info}->{cdefs_from_build} : 0;
    $self->{schema}->load( files            => \@files,
                           path             => $cdef_path,
                           cdefs_from_build => $cdefs_from_build,
                         );

    $Log->exit();
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the Node Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {
    my ( $self, %args ) = @_;
    $Log->enter();

    $self->_get_connectrec_or_hostrec(\%args);
    my $interface = $self->{interface};
    # Establish the session if not provided
    # First check whether a valid shell_host (hostobj) is passed
    # for the context
    $self->_check_session_type();

    # Determine the type of the session based on the shell_type and context.
    my $session_type = $self->{shell_type};
    if ( $self->{shell_type} eq "Vfiler" ) {
        $session_type = "7Mode";
    } elsif ( ( $self->{shell_type} eq "CMode" )
        && ( $self->{actual_set} eq "Nodescope" ) )
    {

        # User had asked for Nodescope commands from a cluster shell
        $session_type .= "::Nodescope";
    } elsif ( $self->is_nodescopeNGSH() )
    {
        $session_type .= "::NodescopeNGSH";
    } elsif ( $self->{shell_type} eq "Vserver" ) {
        $session_type = "CMode::" . $session_type;
    }
    my $session_pkg
        = "NACL::APISet::Session::" . $interface . "::" . $session_type;

    $self->_record_connection_into_keyval();

    # Create a Session object. The Session object automatically takes care of
    # connecting to the service and preparing the service for use
    #################  Try Catch Block for Burt - 993649 ########################
    ###	check_if_xinetd_process_is_up ###
    my $retry_connect=0;
    SESSION_RETRY:
    try{
        $self->{session} = $session_pkg->new(%args);
    }
    catch NACL::APISet::Exceptions::ConnectionFailedException with {
	    my $exception = shift;
        if ((defined param_global->get('NACL_RETRY_SSH_FAILURE')) & (!$retry_connect)) {
            $retry_connect=1;
            if (!$self->_check_xinetd_process_is_up()) {
                NACL::APISet::Exceptions::ConnectionFailedException->throw("Looks like xinetd is not up and running even after 60 seconds");
            }else{
                $Log->comment("Retrying Connection...");
                goto SESSION_RETRY;
            }
        }
        else {
            NACL::APISet::Exceptions::ConnectionFailedException->throw($exception->text());
        }
    };
    $Log->exit();
}

##############################################################################
# Method:    _check_xinetd_process_is_up
# Objective: Check if the xinetd process is up.
# Details:   This method login on console and then transit to system shell to
#            run ssh coomands to check if xinetd process is running.
##############################################################################
sub _check_xinetd_process_is_up {
    $Log->enter();
    my ( $self, %args ) = @_;
    # Local variables

    my $success = 0;
    my $retry = 6; #attempts
    my ($output,$pid,$apiset);

    # Commands to execute
    my $ps_cmd = "ps -ax | grep xinetd | grep -v grep ";

    $Log->comment('create console connection');
    my $hostrec = $self->{hostobj};
    my $connection = $hostrec->connect( template => "console" , username => $hostrec->default_username() || 'admin' , password => $hostrec->default_password
            || 'netapp1!' );
    $Log->comment("Transit filer to systemshell if it's not there.");
    my $transit = NACL::Transit->new(name => $hostrec->{hostname});
    $transit->change_state(to => 'SYSTEMSHELL');

    my $api_set_obj = NACL::APISet->new(
        connection => $connection,
        category   => "Node",
        interface  => "CLI",
        set        => "Systemshell"
    );
    $Log->comment("Checking if xinetd process is up and running...");
    while ($retry > 0) {
        $output = $api_set_obj->execute_raw_command(
            command => $ps_cmd
        );

        if ($output =~ /xinetd/i) {
            $Log->comment("SUCCESS: xinetd is up and running.");
            $success = 1;
            last;
        } else {
            $Log->comment("Xinetd process not up. Will sleep and retry");
            Tharn::snooze(10);
            $retry--;
        }
    }
   return $success;
}

###############################################################################
# Method Name: _check_session_type
# Objective:   Check whether the shell_type passed by the user is appropriate
#              for the shell_host
###############################################################################
sub _check_session_type {
    $Log->enter();
    my ( $self, %args ) = @_;

    # Valid combinations for shell_type and set passed by the user
    my %valid_hash = (

        #actual_set         shell_type
        "7Mode"           => "7Mode",
        "Vfiler"          => "7Mode Vfiler",
        "CMode"           => "CMode",
        "Nodescope"       => "CMode",
        "NodescopeNGSH"   => "CMode",
        "Vserver"         => "CMode Vserver",
        "Firmware"        => "Firmware",
        "Maintenance"     => "Maintenance",
        "SP"              => "SP",
        "SPShell"         => "SPShell",

     );
    if ( $valid_hash{ $self->{actual_set} } !~ /$self->{shell_type}/ ) {
        $Log->exit();
        NATE::Exceptions::Argument->throw( "Invalid hostobj or shell_type "
                . "is sent for set '$self->{actual_set}'"
                . " and the shell_type obtained is "
                . $self->{shell_type} );
    }

    $Log->exit();
}

###############################################################################
# Method Name: _validate_and_get_overrides
# Objective:   Validate and get overrides provided in the APISet method call
# Details:     This method validates the overrides and deletes them from the
#              argument list of the APISet method call and finally returns them
#              in a separate hash to the caller
###############################################################################

sub _validate_and_get_overrides {
    my ( $self, %opts ) = (@_);
    validate_with(
        params      => \%opts,
        spec        => { args_ptr => { type => HASHREF } },
        allow_extra => 1
    );
    my $args_ptr = $opts{args_ptr};

    if (!defined $args_ptr->{method}) {
        NATE::Exceptions::Argument->throw("Mandatory argument 'method' "
            . "not passed to '_validate_and_get_overrides'");
    }

    my ( $vserver, $vfiler, $privilege, $partner, %connectrec );

    # Delete the option only for 7M APISet which understands this
    # option in a different way by supporting the context
    # With this change, the code present in
    # _handle_command_line_modification to check whether vfiler-name
    # input is specified in the method call on a CMode APISet
    # object will be a non-reachable code.
    if ($self->{set} eq '7Mode' && exists $args_ptr->{'vfiler-name'}) {
        $vfiler = delete $args_ptr->{'vfiler-name'};
    }

    if (defined $args_ptr->{'privilege-level'}) {
        $privilege = delete $args_ptr->{'privilege-level'};
    }

    if (defined $args_ptr->{partner} && $self->{set} eq '7Mode'
        && $args_ptr->{partner} =~ /^(0|1)$/) {
        # Partner context only valid for 7Mode
        # Also, if the value is not 0/1, then it's probably an option
        # to the command itself (example: ifconfig), so we leave the
        # option alone in that case.
        $partner = delete $args_ptr->{partner};
    }

    my %nodescope_hash;
    if (defined $args_ptr->{'nodescope-node-name'}
        && $self->{actual_set} eq 'Nodescope') {
        $nodescope_hash{'nodescope-node-name'} =
            delete $args_ptr->{'nodescope-node-name'};
    }


    if (defined $args_ptr->{'vserver-context-name'} && $self->{set} eq 'CMode') {
        $vserver = delete $args_ptr->{'vserver-context-name'};
    }

    $self->_move_connectrec_options(
        source => $args_ptr,
        target => \%connectrec
    );

    if ($args_ptr->{method} eq 'execute_command') {
        # For execute_command, all options (except "method") are considered
        # to be connectrec options
        %connectrec = ( %connectrec, %{$args_ptr} );
        delete $connectrec{method};
    }

    return (
        vfiler     => $vfiler,
        privilege  => $privilege,
        partner    => $partner,
        connectrec => \%connectrec,
        vserver    => $vserver,
        %nodescope_hash
    );
}

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into actual command and execute
# Details:     This method invokes the command along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual command to be invoked on the
#              service, and then invoking that on the filer using the Session
#              object.
###############################################################################

sub _invoke {
    $Log->enter();
    my ($self, %args) = @_;
    my $cdef_obj = delete $args{cdef_obj};
    my %baseline_args = $self->_move_baseline_args(source => \%args);
    $self->api_reconnect_session()
        if $self->get_connection()->is_disconnected( verify => 0 );

    my $method = $args{method};
    my $actual_command;
    my $nacl_refresh_stale_connections_retry = 0;
    my $retry_value = param_global->get('NACL_REFRESH_STALE_CONNECTIONS_RETRY');
    if(defined $retry_value) {
        $nacl_refresh_stale_connections_retry = $retry_value if($retry_value == 1);
    }

    my $raw_output_is_like;

    my %overrides = $self->_validate_and_get_overrides( args_ptr => \%args );

    my $connectrec = delete $overrides{connectrec};

    if ($self->is_nodescopeNGSH()) {
        $connectrec->{raw_output_is_like} = delete $args{raw_output_is_like} || $self->{raw_output_is_like} || "7Mode";
    }

    my ( $command, %compat_mirror, $file );
    $command = $connectrec->{command};

    if ( $method ne 'execute_command' ) {

        # retrieve schema handle
        my $schema_obj = $self->get_schema_obj();

        # Check for compatibility
        my $is_cmode = 0;
        $is_cmode = 1 if ( $self->{"set"} eq "CMode" );

        %compat_mirror = ();
        if ( $is_cmode && $self->{version_info}->{compatibility} ) {
            $Log->debug("Compatibility will be performed");
            my $output = $self->_convert_method_and_options(
                command_params => \%args );
            $Log->debug(
                      sub {"Results after performing compatibility on method name "
                    . "and options:\n"
                    . Dumper($output)} );
            %args = %{ $output->{command_params} }
                if ( defined $output->{command_params} );
            $method = $args{method} = $output->{method};
            %compat_mirror = %{ $output->{output_compat_mirror} };
        }

      # Check whether the cdef method exists and appropriately throw exception
      # text if we found it to be an alias command.
		try {
			$cdef_obj //= $schema_obj->get_command_definition(command => $method);
		} catch NACL::APISet::Exceptions::MethodNotFoundException with {
			my $exception = shift;
			my $org_error_text = $exception->text();
			if ($self->{set} eq 'CMode') {
				my $error_text;
				my $cmd_regex = $method;

				# To match part words
				$cmd_regex =~ s/_/\([^_]*\)_/g;
				$cmd_regex .= "([^_])*\$";
				$cmd_regex = "(^$cmd_regex|_$cmd_regex)";
				my @potential_cmds = grep( /$cmd_regex/, keys %{$schema_obj} );
				my $cons_error_text;
				if ( scalar(@potential_cmds) ) {
					my $potential_matches = join ', ', @potential_cmds;
					$cons_error_text = "CDEF entry not found for '$method'. ";
					$cons_error_text
						.= "The command '$method' might be an alias command"
						. " for which CDEF is not supported. \nThe potential matches "
						. "which we found are: $potential_matches. \nIf it is not an"
						. " alias command and supported by the service, then contact "
						. "dl-nacl-dev";
				} else {
					$cons_error_text
						= "CDEF entry not found for '$method'.\n If you find "
						. "the command to be supported by your service, then probably "
						. "CDEF is missing and please contact dl-nacl-dev about this";
				}
				my $mod_error_text = $org_error_text . "\n" . $cons_error_text;
				$exception->set_text($mod_error_text);
			}
			$exception->throw();
		};

        # translate cdef schema into command
        $command = $schema_obj->translate(%args, cdef_obj => $cdef_obj);
        $Log->debug("The command translated from schema is: '$command'");
        $actual_command = $command;

       # Determine the CDEF file which was read to construct the command
       # Then change the extension from .cdefs to .pm, so that the appropriate
       # parser file gets loaded.
        $file = $cdef_obj->get_file();
        $file =~ s/cdefs$/pm/;
    }

    # Use the overrides to modify the constructed command
    $command = $self->_handle_command_line_modification(
        command => $command,
        %overrides
    );

    $connectrec->{on_timeout}
        = $self->_throw_timeout_exception(
        before => $connectrec->{on_timeout}
            || delete $connectrec->{on_timeout}, timeout => $connectrec->{timeout} );
    $connectrec->{on_idle}
        = $self->_throw_timeout_exception(
        before => $connectrec->{on_idle}
            || delete $connectrec->{on_idle}, timeout => $connectrec->{timeout} );

    # execute command
    my $result;

    # While most overrides can be used to modify the constructed command,
    # for Nodescope part of the command is constructed by the Session
    # class. For this reason, we need to pass on the nodescope node override
    # and let the override be used there.
    my %nodescope_hash;
    $self->_obtain_nodescope_overrides(
        overrides_hash => \%overrides,
        nodescope_hash => \%nodescope_hash
    );

    # Intercept the exception and update it to mention possible reasons
    # for the error
    my $construct_error = sub {
        my %opts = validate_with(
            params => \@_,
            spec   => {
                exception => 1,
                type      => { type => SCALAR },
            },
        );
        my $exception = $opts{exception};
        my $type      = $opts{type};
        my $current_privilege = $self->get_privilege();
        my $additional_text =
            "This could be because of any the following reasons:\n"
            . "1) The $type is invalid\n"
            . "2) The $type is not available on this build\n";

        if ( defined $current_privilege ) {
            if ($current_privilege ne $self->maximum_privilege()) {
                $additional_text .=
                    "3) The $type is available at a higher "
                    . 'privilege level (current privilege level is '
                    . "'$current_privilege')\n";
            }
        }

        $exception->set_text($exception->text() . "\n" .
            NATE::Util::indent(4, $additional_text));
        $Log->exit();
        $exception->throw();
    };

    my %additional_send_cmd_opts = %nodescope_hash;
    if ($self->{set} eq 'CMode') {
        %additional_send_cmd_opts = (%additional_send_cmd_opts, %baseline_args);
    }
    # Retry send_command maximum 3 times
    my $redo = 3;
    # Attempt send_command
    SEND_CMND: {
    try {
        if ( $method eq 'execute_command' ) {
            $result = $self->{session}->send_command(
                connectrec => $connectrec,
                command => $command,
                %additional_send_cmd_opts
            );
        } else {
            try {
                $result = $self->{session}->send_command(
                    actual_command => $actual_command,
                    command        => $command,
                    method_name    => $method,
                    cdef_path      => $self->{cdef_path},
                    connectrec     => $connectrec,
                    parser_file          => $file,
                    output_compat_mirror => \%compat_mirror,
                    %additional_send_cmd_opts,
                );
            } catch NACL::APISet::Exceptions::CommandNotFoundException with {
                $construct_error->(exception => $_[0], type => 'command');
            } catch NACL::APISet::Exceptions::InvalidParamException with {
                $construct_error->(exception => $_[0], type => 'option');
            } catch NACL::APISet::Exceptions::MgwdCrashedException with {
                my $exception = shift;
                $Log->debug(sub {"MgwdCrashedException caught!. ".
                                 "Resetting the privilege level and".
                                 "preparing the service for use"} );
                $self->{session}->_reconnect();

                ## as per burt923196 resyncing the connection
                $self->{"session"}->{"connection"}->resync();

                $self->{session}->prep_service();
                $self->clear_privilege();
                $exception->throw();
            };
	    $self->{"session"}->{"connection"}->resync() if ( $command =~ /system power off|cycle/ || $command =~ /system reset/ );
        } # end else
     } catch NATE::BaseException with {
         my $exception = shift;
         if ($redo && ($exception->text() =~ /can not write/i) && $nacl_refresh_stale_connections_retry) {
             # Connection failed, wait and retry
             sleep(5);
             $redo--;
             $self->{session}->_reconnect();
             $self->{"session"}->{"connection"}->resync();
             $self->{session}->prep_service();
             $self->clear_privilege();
             # If failure is because of connectivity, re-run SEND_CMND.
             redo SEND_CMND ;
        }
        # Check if exception is expected because of Automatic Dedupe Scheduling (ADS)
        elsif ( $self->_check_expected_failure( message => $exception->text(), method => $method, args => \%args ) ) {
             $Log->warn("Ignoring error for method $method because Automatic Dedupe Scheduling \(ADS\) is enabled on the Node");
             if( (! defined $result) or (!$result) ){
                 #Lets forge result in the format expected by higher layers
                 $result = bless({'output' => 'DUMMY', parsed_output => [ {'message' => $exception->text()} ]}, 'NACL::APISet::Response::CLI::' . $self->{'set'});
             }
        }
         else {
             $exception->throw();
         }
     }; # end catch
    } # End SEND_CMND

    $Log->exit();
    return $result;

} ## end sub _invoke

sub _check_expected_failure{
    $Log->enter();
    my ($self, %opts) = @_;
    validate_with(
        params => \%opts,
        spec   => {
            message => { type => SCALAR },
            method  => { type => SCALAR },
            args    => { type => HASHREF },
        }
    );
    my $result = 1;

    #Expected failures for commands
    my %expected_failure_lists = (
        volume_efficiency_on    => ['Operation has already been enabled','Another sis operation is currently active'],
        volume_efficiency_start => ['Another sis operation is currently active'],
    );

    $Log->debug("_check_expected_failure:: message is " . $opts{'message'});
    $Log->debug("_check_expected_failure:: method is " . $opts{'method'});

    if(! defined $expected_failure_lists{ $opts{'method'} } ){
        $Log->debug("_check_expected_failure:: Failed command is not one we are expecting. So bailing out.");
        $result = 0;
        goto MY_END;
    }else{
        my $found=0;
        foreach my $msg ( @{ $expected_failure_lists{ $opts{'method'} } } ){
            if( $opts{'message'} =~ /$msg/ ){
                $found++;
                last;
            }
        }
        unless( $found ){
            $Log->debug("_check_expected_failure:: Failure message is not we are expecting. So bailing out.");
            $result = 0;
            goto MY_END;
        }
    }
    if(! $self->_is_ads_enable() ){
        $Log->debug("_check_expected_failure:: Node is not ADS capabale. So bailing out.");
        $result = 0;
        goto MY_END;
    }
    if(! $self->_is_vol_eff_pol_auto( args => $opts{'args'} ) ){
        $Log->debug("_check_expected_failure:: Volume efficiency policy is not \"auto\". So bailing out.");
        $result = 0;
        goto MY_END;
    }

MY_END:
    $Log->exit();
    return $result;
}

sub _is_ads_enable{
    $Log->enter();
    my ($self) = (@_);
    my $result = 1;

    my $output = ($self->system_node_show( fields => 'is-all-flash-optimized', 'node' => $self->{hostobj}->{hostname} ))->get_parsed_output();
    my $all_flashed_optimized = $output->[0]->{'is-all-flash-optimized'};
    $Log->debug("_is_ads_enable :: \$all_flash_optimized is " . Dumper( $all_flashed_optimized));

    $output = $self->_invoke(
        'method'          => 'execute_command',
        'command'         => "set test\; run local bootargs print se_default_enable",
    );
    my $se_default_enable = $self->_parse_output ($output);
    $Log->debug("_is_ads_enable :: \$se_default_enable is \"$se_default_enable\"");
    unless( ( $se_default_enable eq 'true' ) || ($all_flashed_optimized eq 'true') ){
        $result = 0;
    }

MY_END:
    $Log->exit();
    return $result;
} ## end sub _is_aff_or_se_default_env

sub _is_vol_eff_pol_auto{
    $Log->enter();
    my ($self, %opts) = (@_);
    my $result = 1;
    my %args;
    if( $opts{'args'}->{'path'} ){
        $args{'path'} = $opts{'args'}->{'path'};
    }elsif( $opts{'args'}->{'volume'} ){
        $args{'volume'} = $opts{'args'}->{'volume'};
    }else{
        $Log->debug("_is_vol_eff_pol_auto: Neither path nor volume is available");
        $result = 0;
        goto MY_END;
    }
    my $output = ($self->volume_efficiency_show(vserver => $opts{'args'}->{'vserver'}, fields => 'policy', %args))->get_parsed_output();
    my $eff_policy = $output->[0]->{'policy'};
    $Log->debug("_is_vol_eff_pol_auto :: \$eff_policy is " . Dumper( $eff_policy));
    unless ( $eff_policy eq 'auto' ){
        $result = 0;
    }

MY_END:
    $Log->exit();
    return $result;
}

sub _parse_output
{
    my ($self, $output) = @_;
    chomp $output;
    my $value;
    foreach my $line ( split ( /\n/, $output ) ) {
        if ( $line =~ /==/ ) {
            ($value) = $line =~ /\w+\s+\=\=\s+(\w+)/;
            last;
        }
    } ## end foreach my $line ( split ( ...))
    return $value;
} ## end sub _parse_output


# If the nodescope override is present in the overrides hash, copy
# it into the nodescope hash. This method updates the reference to the
# nodescope hash passed in as argument.
# Requires "overrides_hash" and "nodescope_hash" as arguments.
sub _obtain_nodescope_overrides {
    $Log->enter();
    my ($self, %opts) = @_;
    validate_with(
        params => \%opts,
        spec   => {
            overrides_hash => { type => HASHREF },
            nodescope_hash => { type => HASHREF }
        }
    );
    my $nodescope_hash = $opts{nodescope_hash};
    my $overrides      = $opts{overrides_hash};
    if (exists $overrides->{'nodescope-node-name'}) {
        $nodescope_hash->{'nodescope-node-name'} =
            $overrides->{'nodescope-node-name'};
    }

    $Log->exit();
}

###############################################################################
# Method Name: _handle_command_line_modification
# Objective:   Get the updated command string after prefixing and appending the
#              command line
# Details:     Return the updated command line after checking for privilege,
#              partner and vfiler settings
###############################################################################

sub _handle_command_line_modification {
    my ( $self, %args ) = (@_);
    my ( $command, $vfiler, $privilege, $partner, $vserver ) = (
        $args{command}, $args{vfiler}, $args{privilege},
        $args{partner}, $args{vserver}
    );

    # Check for overrides also and prefix the command appropriately
    my $is_cmode = 0;
    $is_cmode = 1 if ( $self->{"set"} eq "CMode" );
    my $vfiler_context  = 0;
    my $vserver_context = 0;
    if ($is_cmode) {
        if ( defined($vfiler) ) {
            NATE::Exceptions::Argument->throw(
                      "\"vfiler-name\" input argument"
                    . "should not be passed for CMode APISet method" );
        }
        if ( defined($partner) ) {
            NATE::Exceptions::Argument->throw( "\"partner\" input argument"
                    . "should not be passed for CMode APISet method" );
        }
        if ( defined($vserver) ) {
            if ( !length($vserver) ) {
                NATE::Exceptions::Argument->throw(
                          "\"context-vserver-name\" input argument"
                        . "if passed, should not be empty" );
            } else {
                $vserver_context = 1;
            }
        } else {
            $vserver = $self->get_vserver();
            $vserver_context = 1 if ( defined($vserver) && length($vserver) );
        }
    }

    if ( $self->{set} eq '7Mode' ) {
        if ( !defined($vfiler) ) {

            # No override is provided. so look for the values provided
            # by the user using the accessor methods.
            $vfiler = $self->get_vfiler();
        }

       # When vfiler is ""(default value), then vfiler prefix is not required.
        if ( length($vfiler) ) {
            $vfiler_context = 1;
            $command        = "vfiler run -q $vfiler " . $command;
        }
        if ( defined($vserver) ) {
            NATE::Exceptions::Argument->throw(
                "vserver context cannot be used in" . "non-cmode set" );
        }
        # Switch used to specify packetsize in RR is -s, in SN it
        # is changed to -d.
        # Making appropriate changes in libs so that existing script
        # will work for SN also.
        # See burt660890 for more info.
        if ($command =~ /ping6/ and $command =~ /-s/) {
            if ($self->get_version_manager->is_version_ge(release => 8.2)) {
                $command =~ s/-s/-d/;
            }
        }
    }

    # We are not validating the value of privilege in both CMode and 7Mode.
    # whatever the user passes, we just pass it to the filer. But in cmode
    # sending empty string will result in error.
    my $is_nodescope = 0;
    $is_nodescope = 1 if ( $self->{"actual_set"} eq "Nodescope" );

    if ( defined($privilege) ) {

        # override is supplied for privilege on a per command
        # invocation basis.
        my $old_privilege = $self->get_privilege();
        if ($is_cmode) {
            $command
                = "set -privilege $privilege; "
                . $command
                . "; set -privilege $old_privilege";
        } else {
            $command
                = "priv set -q $privilege; "
                . $command
                . "; result -l; priv set -q $old_privilege";
        }
    } else {
        if ($is_nodescope) {

            # In case of nodescope privilege-level must be prefixed to command
            my $priv_in_self = $self->get_privilege();
            if (   ( $command !~ /priv\s+set\s+-q/ )
                && ( $priv_in_self !~ /admin/ )
                && ( $priv_in_self ne "" ) )
            {

                # For 7Mode peray connection, prefix needn't be
                # prepended if the privilege level supplied by the user in
                # set_privilege() is "admin" or "" since for persay, the
                # connection is established in admin privilege level by
                # default.
                $command = "priv set -q $priv_in_self; " . $command;
            }
        } else {

            # No override is provided. so look for the values provided
            # by the user using the accessor methods.
            my $connection = $self->{"session"}->{"connection"};

            #If persay, prepend the privilege string for each command invoked
            # if the requested privilege is not the default privilege
            # Else (i.e for persistent connections), no need of any prefixing.
            # In set_privilege() itself, the user supplied privilege is set on
            # the filer/vsim
            if ( $connection->is_persay() ) {

                # This is a persay connection
                my $priv_in_self = $self->get_privilege();
                if ( !$is_cmode ) {
                    if (   ( $command !~ /priv\s+set\s+-q/ )
                        && defined ($priv_in_self)
                        && ( $priv_in_self !~ /admin/ )
                        && ( $priv_in_self ne "" ) )
                    {

                    # For 7Mode peray connection, prefix needn't be
                    # prepended if the privilege level supplied by the user in
                    # set_privilege() is "admin" or "" since for persay, the
                    # connection is established in admin privilege level by
                    # default.
                        $command = "priv set -q $priv_in_self; " . $command;
                    }
                } else {

                    # For CMode, we are not supporting persay connections.
                    # This is just a place holder incase if we support it
                    # in future.
                    # $command = "set -privilege $priv_in_self; " . $command;
                }
            }
        }
    }

  # Whenever partner is passed as override, it becomes the responsibility of
  # the users to override other values appropriately orelse by default it will
  # take the values stored in $self if none is passed for other values.
  # We wont reset the other values because overrides are basically used to
  # change the prefix on a per-invocation basis
    my $partner_context = 0;
    unless ($is_cmode) {
        if ( $partner || $self->{partner} ) {
            $command         = "partner " . $command;
            $partner_context = 1;
        }
    }

  # To get execution status (for 7Mode), append "result -l" except for the
  # case when a override for privilege is supplied since we would have already
  # appended "result -l" in that case appropriately after the actual command
  # invoked.
  # Though "result -l" is not a supported command in a direct connection to
  # vfiler shell, we always append "result -l" and later if the service
  # respond as "result not found", we will set the execution status to "" so
  # that we are deciding the execution status after executing the command on
  # the service.
    if (( $command !~ /result -l/ )
        && (   $self->{"actual_set"} eq "7Mode"
            || $self->{"actual_set"} eq "Nodescope" )
        )
    {
        if ($vfiler_context) {
            $command .= "; vfiler run -q $vfiler result -l";
        } else {
            $command .= "; result -l";
        }
    }

    if ( $vserver_context && $is_cmode ) {
        $command = $command . " -vserver $vserver";
    }

 # If the command is required to run in partner node under certain privilege,
 # then the word partner needs to be prepended once for the previlege and
 # again before the actual comamnd invoked by the user.
 # This case has to be considered only for overrides and persay connection.
 #
 # For eg, if partner =1, privilege-level = diag, and command = "qtree status"
 # then full command should be
 # "partner priv set -q diag; partner qtree status"
    if ( ( !$is_cmode ) && $partner_context ) {
        $command =~ s/;/; partner/g;
    }
    return $command;
}

###############################################################################
# Method Name: _only_translate_method
# Objective:   Translate the API call into the command invoked on the service
#              and return this without executing it.
# Details:     When the API is prefixed by an "xlate_", it must only translate
#              the API call without executing it. This will come in handy
#              while developing new APISets and for cases where undoing
#              API calls can be cumbersome.
#              This method strips the leading xlate_ prefix, translates
#              the API and returns the command
###############################################################################
sub _only_translate_method {
    my ( $self, %args ) = @_;
    $Log->enter();

    my $method = delete $args{method};
    $method =~ s/^xlate_//;
    $args{method} = $method;

    my $schema_obj = $self->get_schema_obj();

    if ($method eq 'options'
        && (   $self->{"actual_set"} eq "7Mode"
            || $self->{"actual_set"} eq "Nodescope" )
        )
    {
        return $self->options( %args, 'translate-only' => 1 );
    } elsif ( ( $self->{set} eq "CMode" )
        && $self->{version_info}->{compatibility} )
    {
        $Log->debug("Compatibility will be performed");
        my $output
            = $self->_convert_method_and_options( command_params => \%args );
        $Log->debug( "Results after performing compatibility on method name "
                . "and options:\n"
                . Dumper($output) );
        %args = %{ $output->{command_params} }
            if ( defined $output->{command_params} );
        $args{method} = $output->{method};
    }

    my $command = $schema_obj->translate(%args);
    $Log->exit();
    return $command;
}

sub _get_highest_possible_privilege {
    $Log->enter();
    my $self = $_[0];
    my $highest_level  = $self->maximum_privilege();
    my $fallback_level = $self->fallback_privilege();
    my $priv;

    try {
        my $is_promoted = $self->get_version_manager()->is_promoted();
        $priv = $is_promoted ? $fallback_level : $highest_level;
    } otherwise {
        my $exception = shift;
        $priv = $fallback_level;
        $Log->warn("Call to 'is_promoted' failed. Error:\n"
            . $exception->text()
            . "\nSince it could not be determined whether a promoted "
            . 'build is being run or not, the highest possible privilege '
            . "level is being set to '$priv'");
    };
    $Log->trace("Highest possible privilege level determined to be '$priv'");
    $Log->exit();
    return $priv;
}

sub is_nodescopeNGSH {
    $Log->enter();
    my $self = $_[0];
    $Log->exit();
    return ($self->{actual_set} eq 'NodescopeNGSH') ? 1 : 0;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::CLI - Base for API Sets for CLI commands

=head1 SYNOPSIS

 use NACL::APISet::Node::CLI;

=head1 DESCRIPTION

A C<NACL::APISet::Node::CLI> is used to connect to and send CLI commands. This
is further subclassed to send CLI commands specifically to CMode, 7Mode etc.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::TimeoutException

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 ATTRIBUTES

In addition to the attribute defined by the base class, all Node-CLI
APISet objects have the following attributes.

=head2 highest_possible_privilege

The highest possible privilege level for this APISet object. If a
promoted ONTAP build is being run then the value of this is "diag" (for 7Mode)
or "diagnostic" (for CMode). The value can be accessed using the accessor
method.

    my $highest_priv = $apiset->highest_possible_privilege();

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw CLI commands on the filer. The entire command
line is to be provided exactly as it would be on the filer. This command is
executed and the raw output is returned as a scalar. This is useful for cases
where we want to invoke a command without parsing the output, or for cases
where the particular parser has not been defined yet but we want to see the
raw output.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(command => "system node show -node myvsim1");

=item Arguments

execute_raw_command accepts all arguments accepted by Connectrec->execute().
Of these, only command is mandatory.

=over

=item command

(Mandatory)
The command to be invoked. This is the full command line as it will be entered on the filer.

=back

=item Returns

(Scalar)
Raw output

=back

=head2 get_privilege

This methods returns the current privilege level at which commands are being run

=over

=item Synopsis

$privilege_level = $Api_Set_Obj->get_privilege();

=item Arguments

None.

=item Returns

(Scalar)
Privilege level

=back

=head2 get_allowed_options_for_command

See
L<NACL::APISet::_Mixins::CLI::get_allowed_options_for_command|lib-NACL-APISet-_Mixins-CLI-pm/get_allowed_options_for_command>.

=head2 maximum_privilege

This method returns the maximum privilege level on Node-CLI APISets
(this is currently "test")

=over

=item Synopsis

    my $max_priv = $Api_Set_Obj->maximum_privilege();

=item Arguments

None

=item Returns

(Scalar)
Maximum Privilege

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI.pm#6 $
 Last revised $Date: 2013/01/30 $ by $Author: kjp $

=over

=item NACL_REFRESH_STALE_CONNECTIONS_RETRY

  parameter NACL_REFRESH_STALE_CONNECTIONS_RETRY controls if retry on failure of connection is required
  possible values 0/1
  It will retry if NACL_REFRESH_STALE_CONNECTIONS_RETRY is 1

=back

=cut

1;

