#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for 7mode CLI commands
## @author pressley@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::CLI::7Mode;

#
# Compiler Directives
#
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Module import - APISets
use NACL::APISet::Session::CLI::7Mode;
use NACL::APISet::Session::CLI::Maintenance;
use NACL::APISet::Session::CLI::CMode::Nodescope;
use NACL::APISet::Session::CLI::CMode::NodescopeNGSH;
use NATE::Exceptions::Argument qw(:try);
use Tie::IxHash;

#
#Exceptions
#
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::InconsistentVolume qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALAR BOOLEAN validate_with);
use Data::Dumper;

sub new {
    $Log->enter() if $may_enter;

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;
    my $self = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            version => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
# Method:    set_vfiler
# Objective: To set the current vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_vfiler {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "vfiler-name" => { type => SCALAR } }
    );
    $self->{"vfiler"} = $args{"vfiler-name"};

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    set_partner
# Objective: To set the execution context to partner node
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_partner {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    $self->{"partner"} = 1;

    # When entering into partner context, the privilege level stored in
    # $self->{"privilege"} should be set in the partner node also for
    # persistent connections basically
    # vfiler context is overridden to "" so that the priv set command
    # can work
    $self->priv_set(
        "quiet"       => 1,
        "level"       => $self->{"privilege-level"},
        "vfiler-name" => ""
    );

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    get_vfiler
# Objective: Get the current vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_vfiler {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"vfiler"};
}

##############################################################################
# Method:    get_partner
# Objective: Get the current vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_partner {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"partner"};
}

##############################################################################
# Method:    clear_vfiler
# Objective: To set the default vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_vfiler {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    # Empty string is taken as default value so that users are allowed to
    # explicitly prefix their commands with vfiler0
    $self->set_vfiler( "vfiler-name" => "" );

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    clear_partner
# Objective: To reset the exceution context to taken over node.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_partner {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    $self->{"partner"} = 0;

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    set_privilege
# Objective: Set the current privilege level
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_privilege {
    $Log->enter() if $may_enter;

    my $self = shift;
    my %args = $self->validate_params(
        params => \@_,
        spec   => { 'privilege-level' => { type => SCALAR } }
    );

    my $set_on_object = sub {
        $self->{'privilege-level'} = $args{'privilege-level'};
    };

    if ( $self->is_nodescopeNGSH() ) {
        my $priv = $args{'privilege-level'};
        $self->execute_command( command => "set -privilege $priv" );
        $Log->exit() if $may_exit;
        return;
    }

    # In the case of Nodescope/per-say connections it is not necessary to
    # run the command "priv set", so we just set the value on the object
    # and return.
    if (   ( $self->is_nodescope() )
        || ( $self->get_connection()->is_persay() ) )
    {
        $set_on_object->();
        $Log->exit() if $may_exit;
        return;
    }

    # If any unrecognized privilege level is supplied by the user,
    # then an exception will be thrown at this point itself and we
    # need not wait till the execution of next command as we are directly
    # invoking the "priv set" here itself.
    # vfiler context is overridden to "" so that the priv set command
    # can work
    $self->priv_set(
        "quiet"       => 1,
        "level"       => $args{"privilege-level"},
        "vfiler-name" => ""
    );
    $set_on_object->();

    # Also if partner context is set, then appropriately change the privilege
    # level in the other node in cluster.
    # This should be done to handle persistent connections.
    if ( $self->get_partner() ) {

        # Partner context is set.
        # This will change the privilege level in the other node in cluster
        # vfiler context is overridden to "" so that the priv set command
        # can work
        $self->priv_set(
            "quiet"       => "",
            "level"       => $self->{"privilege-level"},
            "partner"     => "0",
            "vfiler-name" => ""
        );
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    clear_all_prefixes
# Objective: To set the defaults for all 7Mode prefixing commands like vfiler,
#            privilege and partner.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_all_prefixes {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    # This order matters.
    $self->clear_vfiler();
    $self->clear_privilege();
    $self->clear_partner();

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    cifs_setup
# Objective: Overridden method for invocation of cifs setup for 7Mode
##############################################################################

sub cifs_setup {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $prompts_answers = $args{'connectrec-match_table'};
    my $connectrec;

    # Store the connection object because we are going to modify it to be
    # using console if not specified by the user
    my $connection_old = $self->get_connection();

# Cifs Setup is an interactive command and works only with console/ssh persist
# connection
    if ( $connection_old->{connid} !~ /(7-login|console)/ ) {
        my $host_name = $connection_old->hostp();
        my $host      = Tharn::host($host_name);
        $connectrec = $host->connect( template => "7-login" );
        $self->{session}->set_connection( connection => $connectrec );
    }

    # This table is a list of errors that can appear during
    # the interactive prompts
    my $prompt_errors = [
        '(Only \\(y\\)es or \\(n\\)o allowed. Ctrl-C to quit)|
        (Only values from \\d+ to \\d+ allowed)|
        (CIFS Setup cannot find a necessary DNS service)|
        (You must supply a value)|
        (Please enter one of)|
        (is not a legal)' => sub {
            $self->{session}->{connection}->send_intr();
            NATE::Exceptions::Argument->throw(
                "Invalid Argument(s) passed during cifs setup");
        },
    ];

    push( @$prompts_answers, @$prompt_errors );

    my $result_obj;
    try {
        $result_obj = $self->_custom_handler( method => "cifs_setup", %args );
    }
    finally {

        # Set the old connection back
        $self->{session}->set_connection( connection => $connection_old );
    };

    $Log->exit() if $may_exit;
    return $result_obj;
}
##############################################################################
# Method:    options
# Objective: To handle the options method call on the APISet object
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub options {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $show_options = delete( $args{'show-options'} );
    $args{$show_options} = "" if $show_options;
    $Log->exit() if $may_exit;
    return $self->_custom_handler( method => "options", %args );
}

##############################################################################
# Method:    wrfile
# Objective: To handle the wrfile method call on the APISet object
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub wrfile {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    if ((      exists $args{content} 
            && defined $args{content}
            && (!exists $args{append}
                || ( exists $args{append}
                    && $args{append} =~ /0|false/i )
            )
        )
        )
    {

        $Log->exit() if $may_exit;
        return $self->_custom_handler(
            method => "wrfile",
            %args, wrfile_without_append => 1
        );
    } else {

        $Log->exit() if $may_exit;
        return $self->_custom_handler(
            method => "wrfile",
            %args, wrfile_without_append => 0
        );

    }
}

sub get_nodescope_node_name {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->_check_if_nodescope( caller => 'get_nodescope_node_name' );
    $Log->exit() if $may_exit;
    return $self->{session}->node_name();
}

sub set_nodescope_node_name {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $self->_check_if_nodescope( caller => 'set_nodescope_node_name' );
    validate_with(
        params => \%args,
        spec   => { node_name => { type => SCALAR } }
    );
    $self->{session}->node_name( $args{node_name} );
    $Log->exit() if $may_exit;
}

sub clear_nodescope_node_name {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->_check_if_nodescope( caller => 'clear_nodescope_node_name' );
    $self->{session}->node_name_reset();
    $Log->exit() if $may_exit;
}

sub execute_raw_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $self->_move_connectrec_options(
        source => \%args,
        target => \%args
    );

    # If set is NodescopeNGSH, delete raw_output_is_like
    # from %args.
    if ( $self->is_nodescopeNGSH() ) {
        delete $args{raw_output_is_like};
    }

    if ((   $self->is_nodescope() && $args{command} !~ /system\s*node\s*run.*/
        )
        || ( $self->is_nodescopeNGSH() )
        )
    {
        $Log->warn( 'Usage of execute_raw_command is not recommended with '
                . 'Nodescope/NodescopeNGSH APISets.since it sends commands to ngsh '
                . 'Please use execute_command instead since it '
                . 'automatically constructs the appropriate '
                . 'command to run on the DBlade' );
    }

    my $output = $self->SUPER::execute_raw_command(%args);
    $Log->exit() if $may_exit;
    return $output;
}

sub fallback_privilege {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return 'diag';
}

sub is_nodescope {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $Log->exit() if $may_exit;
    return ( $self->{actual_set} eq 'Nodescope' ) ? 1 : 0;
}

sub parse_batched_free_log_file {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    my $var = $opts{output};
    my @lines          = split /\r*\n+/, $var;
    my $vol_dump = {};
    foreach my $line (@lines) {
        if ($line =~ /^\(\s+(\d)\,\s+(\d+)\)\s+\[\s+(\d+)\]\:\s+(\w+)\,\s+(\w+)/) {
            push @{ $vol_dump->{chunk_id}}, $1;
            push @{ $vol_dump->{chunk_bn}}, $2;
            push @{ $vol_dump->{fbn}}, $3;
            push @{ $vol_dump->{PVBN}}, $4;
            push @{ $vol_dump->{VVBN}}, $5;
        }

    }
    $Log->exit() if $may_exit;
    return $vol_dump;
}


##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _simple_translate
# Objective: Return a command line constructed from the keys and values given
# Details:   Translate key names and value names as is into a space separated
#            string with the option-name and option-value separated by the
#            specified separator.
##############################################################################

sub _simple_translate {
    my ( $self, %args ) = (@_);
    $Log->enter() if $may_enter;
    my $command      = delete $args{method};
    my $pass_through = delete $args{pass_through} || "";
    my $sep          = delete $args{command_sep} || " ";
    my $cmd_line     = "";
    foreach ( keys %args ) {
        $cmd_line .= " " . "$_" . $sep . $args{$_};
    }

    $command .= $cmd_line . " " . $pass_through;
    $command =~ s/\s+$//;
    $Log->exit() if $may_exit;
    return $command;
}

##############################################################################
# Method:    _custom_handler
# Objective: Handle most of the functionality for overriden methods
# Details:   In overriden methods, a custom translation mechanism is required
#            The rest of the processing is similar to that done for other
#            commands. This handler is intended to handle the options and
#            license commands.
##############################################################################

sub _custom_handler {
    my ( $self, %args ) = (@_);
    $Log->enter() if $may_enter;
    my $translate_only = delete $args{'translate-only'};
    $Log->enter() if $may_enter;

    my $method = $args{method};
    my %overrides = $self->_validate_and_get_overrides( args_ptr => \%args );

    my $connectrec = delete $overrides{connectrec};
    my $file;
    if ( $method =~ /cifs_setup/ ) {
        $args{method} = 'cifs setup';
        $file = $self->{cdef_path} . "/cifs" . '.pm';
    } else {
        $file = $self->{cdef_path} . "/$method" . '.pm';
    }

    if ( $self->is_nodescopeNGSH ) {
        $connectrec->{raw_output_is_like} 
            = delete $args{raw_output_is_like}
            || $self->{raw_output_is_like}
            || "7Mode";
    }

    my $command;
    my $wrfile_without_append_flag;
    my $wrfile_content;
    if ( $method =~ /wrfile/ ) {
        $wrfile_without_append_flag = delete $args{wrfile_without_append};
        $wrfile_content             = delete $args{content}
            if !exists $args{append}
                or $args{append} =~ /0|false/i;
        $command = $self->SUPER::_only_translate_method(%args);
    } else {
        $command = $self->_simple_translate(%args);
    }
    return $command if $translate_only;

    my $actual_command = $command;

    $command = $self->_handle_command_line_modification(
        command => $command,
        %overrides
    );

    my %nodescope_hash;
    $self->_obtain_nodescope_overrides(
        overrides_hash => \%overrides,
        nodescope_hash => \%nodescope_hash
    );

    $connectrec->{on_timeout} = $self->_throw_timeout_exception(
        before  => $connectrec->{on_timeout},
        timeout => $args{timeout}
    );
    $connectrec->{on_idle} = $self->_throw_timeout_exception(
        before  => $connectrec->{on_idle},
        timeout => $args{timeout}
    );

    my %prep_args;
    $prep_args{actual_command}       = $actual_command;
    $prep_args{command}              = $command;
    $prep_args{method_name}          = $method;
    $prep_args{cdef_path}            = $self->{cdef_path};
    $prep_args{connectrec}           = $connectrec;
    $prep_args{parser_file}          = $file;
    $prep_args{output_compat_mirror} = undef;

    # execute command
    my $result_obj;
    if ($wrfile_without_append_flag) {
        $result_obj = $self->{session}->send_command(
            %prep_args,
            wrfile_content => $wrfile_content,
            %nodescope_hash,
            %args,
        );
    } else {
        $result_obj
            = $self->{session}->send_command( %prep_args, %nodescope_hash );
    }

    $Log->exit() if $may_exit;
    return $result_obj;
}

##############################################################################
# Method:    _init_API
# Objective: Initialize the 7Mode API Set
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file( $self->{cdef_path} );

    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason:\n$@");
    }

    $self->SUPER::_init_API(%args);

    # Initialize defaults for prefixing member variables.
    # Calling clear_all_prefixes() will actually set the privilege level by
    # running "priv set" command which should not be done when the connection
    # is established directly to a vfiler.
    $self->{"vfiler"}  = "";
    $self->{"partner"} = 0;

    if ( $self->{'shell_type'} ne 'Vfiler' ) {
        if ( defined $self->{'privilege-level'} ) {
            $self->set_privilege(
                'privilege-level' => $self->{'privilege-level'} );
        } else {
            $self->clear_privilege();
        }
    }

   # Set the vfiler name provided through context_definition hash irrespective
   # of whether we had directly connected to the vfiler or through the node.
   # Thus incase of direct connection to vfiler, if the user overrides the
   # vfiler name with other vfiler name or even the same vfiler name, we will
   # prepend the command with this name and allowing the service to respond to
   # it. Thus using the context_definition hash incase of direct connection
   # will cause an exception to be thrown based on how the service reacts.
    if ( $self->{actual_set} eq "Vfiler" ) {
        my $vfiler_name
            = defined( $self->{context_definition}->{name} )
            ? $self->{context_definition}->{name}
            : (
            defined $self->{context_definition}->{schema_hostobj}
            ? $self->{context_definition}->{schema_hostobj}->hostname()
            : ""
            );
        $self->set_vfiler( "vfiler-name" => $vfiler_name )
            if ( length $vfiler_name );
    }

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _vfiler_option_handler
# Objective: Method to insert the right switch before each of the space
#            separated values.
# Details:   For certain command line arguments, the vfiler commands take
#            multiple values where each value is specified after the
#            corresponding switch multiple times. Since we can't have a
#            programmatic key name for specifying each of these multiple values,
#            they can be specified as a space separated string of those values
#            and this method will convert it to the required form
#            switch<separator>value [switch<separator>value ...]
##############################################################################

sub _vfiler_option_handler {
    $Log->enter() if $may_enter;
    my ( $self, $args ) = @_;

    _convert( value => \$args->{'ip-addresses'}, switch => '-i ' );
    if ( $args->{'protocol'} ) {
        $Log->warn( "Use of the key 'protocol' is deprecated, please use "
                . "'protocols' instead" );
        _convert( value => \$args->{'protocol'}, switch => 'proto=' );
    }
    _convert( value => \$args->{'protocols'}, switch => 'proto=' );

    if ( $args->{'remote-login-name'} && $args->{'remote-login-password'} ) {
        $args->{'remote-login-name'}
            .= ":" . $args->{'remote-login-password'};
        delete $args->{'remote-login-password'};
    }

    if ( $args->{'alternate-source'} && $args->{'alternate-destination'} ) {
        $args->{'alternate-source'} .= ":" . $args->{'alternate-destination'};
        delete $args->{'alternate-destination'};
    }

    if ( $args->{'dns-server'} ) {
        _convert( value => \$args->{'dns-server'}, switch => ':' );
        $args->{'dns-server'} =~ s/://;
        $args->{'dns-server'} =~ s/\s+//g;
    }

    if ( $args->{'nis-server'} ) {
        _convert( value => \$args->{'nis-server'}, switch => ':' );
        $args->{'nis-server'} =~ s/://;
        $args->{'nis-server'} =~ s/\s+//g;
    }

    if ( $args->{'remote-vfiler'} && $args->{'remote-filer'} ) {
        $args->{'remote-filer'}
            = $args->{'remote-vfiler'} . "@" . $args->{'remote-filer'};
        delete $args->{'remote-vfiler'};
    }

    $Log->exit() if $may_exit;
}

sub _convert {
    $Log->enter() if $may_enter;
    my %args  = @_;
    my $value = $args{'value'};
    return unless $$value;
    my $switch = $args{'switch'};
    $$value = ' ' . $$value;
    $$value =~ s/ $switch/ /g;
    $$value =~ s/([^\s]+)/$switch$1/g;
    $Log->exit() if $may_exit;
}

# Check if this a Nodescope APISet object.
# We take caller as an argument rather than use Perl's caller() so that
# any later refactoring of code does not result in the error message
# mentioning the wrong subroutine name.
sub _check_if_nodescope {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    validate_with(
        params => \%args,
        spec   => { caller => { type => SCALAR } }
    );
    if ( !$self->is_nodescope() ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "Call to method '$args{caller}' can "
                . 'only be done for Nodescope APISet objects. This is an '
                . $self->{actual_set}
                . 'APISet object' );
    }
    $Log->exit() if $may_exit;
}

#
# This method is called for pre-processing of connectrec-match_table
# valuea, which is required to findout, if volume is marked inconsistent.
# In case of inconsistent volume, a prompt is thrown by the filer for user
# confirmation. After matching the prompt, a flag is set to 1 indicating
# the inconsistent volume. Such a volume will not be deleted, if user
# inputs 'no' and an exception will be thrown by _vol_destroy_post()
# method after evaluating the flag.
#
sub _vol_destroy_pre {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my %opts = %{ $args{args} };
    my $flag = 0;
    my $sub;
    my $expected_prompt = 'This volume is marked corrupt and could be useful'
        . ' for debugging purposes, are you sure you want to proceed? ';

    if ( $opts{'connectrec-match_table'} ) {
        my %match_table = @{ $opts{'connectrec-match_table'} };
        foreach my $regex ( keys %match_table ) {
            if ( $expected_prompt =~ /$regex/i ) {
                my $value = $match_table{$regex};

                # This is the prompt which we are looking for
                if ( !ref($value) ) {

                    # $value is a scalar
                    if ( $value =~ /^(n(o)?)$/i ) {
                        $sub = sub { $flag = 1; return $value; };
                        $match_table{$regex} = $sub;
                    }
                }
                last;
            }
        }

        # update the original input hash
        $args{args}->{'connectrec-match_table'} = [%match_table];
    }
    $Log->exit() if $may_exit;
    return { flag => \$flag };
}

#
# This method is called after actual vol_destroy() method is invoked.
# This method evaluates the flag, which is set by _vol_destroy_pre()
# method to indicate the inconsistent volume. If flag is set, an
# exception 'NACL::APISet::Exceptions::InconsistentVolume' will be
# thrown.
#
sub _vol_destroy_post {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $flag_ref = $args{pre_result}->{flag};
    my $resp_obj = $args{resp_obj};
    my $vol      = $args{args}->{volume};
    if ($$flag_ref) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::InconsistentVolume->throw(
            text            => "$vol volume is marked inconsistent",
            response_object => $resp_obj,
        );
    }
    $Log->exit() if $may_exit;
}

#
# This method is called for pre-processing of snapvault release for
# is-primary. In case this command needs to be invoked on secondary,
# is-primary option must be set to secondary. This would
# then flip the order of primary-path and secondary-path. This is required
# because of the following different usage of the command:
# usage:\n
# On a snapvault primary:\n
# snapvault release <primary_path> <secondary_filer>:<secondary_path>\n
# On a snapvault secondary:\n
# snapvault release <secondary_path> <primary_system_path>\n
# where <primary_system_path> is <primary_system>:<restored_path>\n
#
sub _snapvault_release_pre {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my %opts = %{ $args{args} };
    %opts = NACL::APISet->validate_params(
        params => \%opts,
        spec   => { "is-primary" => { type => BOOLEAN, default => 1 }, }

    );
    if ( !$opts{'is-primary'} ) {
        ( $opts{'primary-path'}, $opts{'secondary-path'} )
            = ( $opts{'secondary-path'}, $opts{'primary-path'} );
    }
    delete $opts{'is-primary'};
    %{ $args{args} } = %opts;
    $Log->exit() if $may_exit;
}

#
# This method is called for pre-processing of snapvault restore command
# to flip the order of primary-location and secondary-path.
# As the the source path and destination path for the restore command
# will be secondary-path and primary-location respectively.

sub _snapvault_restore_pre {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my %opts = %{ $args{args} };
    ( $opts{'primary-location'}, $opts{'secondary-path'} )
        = ( $opts{'secondary-path'}, $opts{'primary-location'} );
    %{ $args{args} } = %opts;
    $Log->exit() if $may_exit;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::CLI::7Mode - API Set for 7mode CLI commands

=head1 SYNOPSIS

use NACL::APISet::Node::CLI::7Mode;

=head1 DESCRIPTION

A C<NACL::APISet::Node::CLI::7Mode> is used to connect to and send CLI commands
to the 7Mode Node.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::CLI::7Mode>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Node::CLI::7Mode->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new CLI 7Mode API Set.

=back

=head1 METHODS

=head2 set_vfiler

Specify which vfiler in which to run the commands following the call

=over

=item Synopsis

$Api_Set_Obj->set_vfiler('vfiler-name' => $vfiler_name);

=item Arguments

=over

=item vfiler-name

Name of the vfiler

=back

=item Returns

Nothing.

=back

=head2 set_partner

Set execution context to partner mode.

=over

=item Synopsis

$Api_Set_Obj->set_partner();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 get_vfiler

Get current vfiler context in which we are executing commands

=over

=item Synopsis

$Api_Set_Obj->get_vfiler();

=item Arguments

None.

=item Returns

(String)
Current vfiler context

=back

=head2 get_partner

Returns whether commands are currently being executed in partner mode or not

=over

=item Synopsis

$Api_Set_Obj->get_partner();

=item Arguments

None.

=item Returns

1 if commands are currently being executed in partner context, 0 if not

=back

=head2 clear_vfiler

Reset to executing commands in default vfiler context

=over

=item Synopsis

$Api_Set_Obj->clear_vfiler();

=item Arguments

None.

=item Returns

None.

=back

=head2 clear_partner

Set execution context to taken over node

=over

=item Synopsis

$Api_Set_Obj->clear_partner();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 set_privilege

Set privilege level in which commands are to be executed

=over

=item Synopsis

$Api_Set_Obj->set_privilege('privilege-level' => $privilege);

=item Arguments

=over

=item privilege-level

The privilege level in which further commands need to be executed.
Possible values are admin, advanced and diag.

=back

=item Returns

Nothing.

=back

=head2 clear_privilege

Reset to privilege level to default (diag)

=over

=item Synopsis

$Api_Set_Obj->clear_privilege();

=item Arguments

=item Returns

None.

=back

=head2 options

Method for handling the 7Mode options command

=over

=item Synopsis

$Api_Set_Obj->options();

=item Arguments

=over

=item list of key value pairs

keys and their values to be sent to the options command

=item show-options

option-name prefix which is used to display all option names beginning with that
prefix and their values

=item Overrides

Overrides such as privilege, partner settings, vfiler and connection overrides

=back

=item Returns

Response object or translated command line

=back

=head2 wrfile

Method for handling the 7Mode options command

=over

=item Synopsis

$Api_Set_Obj->wrfile();

=item Arguments

=over

=item list of key value pairs

keys and their values to be sent to the wrfile command

=item content

option-name prefix which is used to specify the contents to be inserted into
the specified file

=item Overrides

Overrides such as privilege, partner settings, vfiler and connection overrides

=back

=item Returns

Response object or translated command line

=back

=head2 get_nodescope_node_name

(Applicable only for Nodescope APISet objects)
Returns the node name on which the Nodescope APISet object issues
commands to.

=over

=item Synopsis

    my $node_name = $Api_Set_Obj->get_nodescope_node_name();

=item Arguments

None.

=item Returns

Node name.

=back

=head2 set_nodescope_node_name

(Applicable only for Nodescope APISet objects)
Sets the node name on which the Nodescope APISet object issues
commands to. (i.e. from that point onwards, commands will be issued
to the new node value)

=over

=item Synopsis

    $Api_Set_Obj->set_nodescope_node_name(node_name => 'name1');
    $Api_Set_Obj->aggr_status(); # This gets run on node "name1"

=item Arguments

=over

=item node_name

The new node name to which commands should be issue to.

=back

=item Returns

Nothing.

=back

=head2 clear_nodescope_node_name

(Applicable only for Nodescope APISet objects)
Resets the node name back to the default ("local")

=over

=item Synopsis

    $Api_Set_Obj->clear_nodescope_node_name();

=item Arguments

None.

=item Returns

Nothing.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI/7Mode.pm#5 $
 Last revised $Date: 2013/02/21 $ by $Author: veerabha $

=cut
