#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/acpadmin.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode acpadmin commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode acpadmin commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "acpadmin online" will be acpadmin_online()
##
## @synopsis
##    use acpadmin;
##
##    acpadmin_configure();
##    acpadmin_list_all();
##    acpadmin_stats();
##    acpadmin_post_data(%args);
##    acpadmin_expander_reset(%args);
##    acpadmin_expander_power_cycle(%args);
##    acpadmin_voltage_status(%args)
##    acpadmin_shelf_power(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::acpadmin;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args            = @_;
    my $output          = $args{raw_output};
    my $response_object = $args{response_object};
    my $error_level = $response_object->get_execution_status();
    my ( $error, $error_message );
    if ( $output
        =~ /(Missing shelf information|Missing module information)/
        ) {
        $error          = "MissingRequiredParam";
        $error_message = $1;
    } elsif ( $output =~ /(specify valid module name|Invalid shelf id|illegal argument passed in)/ ) {
        $error          = "InvalidParam";
        $error_message = $1;
    } elsif ( $output =~ /(Unable to find ACPP for|Module id can only be 'A' or 'B')/ ) {
        $error          = "InvalidParamValue";
        $error_message = $1;
    } elsif ( $output =~ /[Uu]sage|Command failed:/i ) {
        $error          = "CommandFailed";
        $error_message = "Command failed because of dependencies";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }
} ## end sub _check_error

## @name ::acpadmin_list_all
##
## @summary Parses the raw output from the "acpadmin list_all" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_list_all with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_list_all(%args);
## INPUTS TO METHOD CALL 'acpadmin_list_all' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin list_all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IP              MAC                Reset  Last Contact  Protocol   Assigner    Shelf             Current Inband   IOM
##&nbsp;Address         Address            Cnt    (seconds ago) Version    ACPA ID     S/N               State   ID       Type
##&nbsp;----------------------------------------------------------------------------------------------------------------------
##&nbsp;192.168.1.1     00:50:cc:72:89:00  002      303         1.1.1.31   1786426465   SHU0954292N0HAG   0x5    0a.02.B  IOM3
##&nbsp;192.168.1.19    00:50:cc:72:89:12  000      203         1.1.1.31   1786426465   SHU0954292N0HAG   0x5    0a.02.A  IOM3
##&nbsp;192.168.1.67    00:a0:98:14:05:42  000      228         1.2.2.6    1786426465   6000007163;DS22   0x5    0a.01.A  IOM6
##&nbsp;192.168.1.69    00:50:cc:72:89:45  000      260         1.1.1.31   1786426465   SHU0954292N0GLA   0x5    0a.03.A  IOM3
##&nbsp;192.168.1.189   00:a0:98:18:d5:bc  003      341         2.1.1.0    1786426465   SHJMS000000048F   0x5    0a.00.B IOM6E
##&nbsp;192.168.2.35    00:50:cc:72:8a:23  000      259         1.1.1.31   1786426465   SHU0954292N0GLA   0x5    0a.03.B  IOM3
##&nbsp;192.168.2.78    00:a0:98:18:e2:4c  003      260         2.1.1.10   1786426465   SHJMS000000048F   0x5    0a.00.A IOM6E
##&nbsp;192.168.2.77    00:a0:98:14:06:4c  000      239         1.2.2.6    1786426465   6000007163;DS22   0x5    0a.01.B  IOM6
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reset_cnt' => '002',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0HAG',
##&nbsp;            'protocol_version' => '1.1.1.31',
##&nbsp;            'inband_id' => '0a.02.B',
##&nbsp;            'last_contact_seconds_ago' => '303',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:50:cc:72:89:00',
##&nbsp;            'iom_type' => 'IOM3',
##&nbsp;            'ip_address' => '192.168.1.1',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '000',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0HAG',
##&nbsp;            'protocol_version' => '1.1.1.31',
##&nbsp;            'inband_id' => '0a.02.A',
##&nbsp;            'last_contact_seconds_ago' => '203',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:50:cc:72:89:12',
##&nbsp;            'iom_type' => 'IOM3',
##&nbsp;            'ip_address' => '192.168.1.19',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '000',
##&nbsp;            'shelf_serial_num' => '6000007163;DS22',
##&nbsp;            'protocol_version' => '1.2.2.6',
##&nbsp;            'inband_id' => '0a.01.A',
##&nbsp;            'last_contact_seconds_ago' => '228',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:a0:98:14:05:42',
##&nbsp;            'iom_type' => 'IOM6',
##&nbsp;            'ip_address' => '192.168.1.67',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '000',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0GLA',
##&nbsp;            'protocol_version' => '1.1.1.31',
##&nbsp;            'inband_id' => '0a.03.A',
##&nbsp;            'last_contact_seconds_ago' => '260',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:50:cc:72:89:45',
##&nbsp;            'iom_type' => 'IOM3',
##&nbsp;            'ip_address' => '192.168.1.69',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '003',
##&nbsp;            'shelf_serial_num' => 'SHJMS000000048F',
##&nbsp;            'protocol_version' => '2.1.1.0',
##&nbsp;            'inband_id' => '0a.00.B',
##&nbsp;            'last_contact_seconds_ago' => '341',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:a0:98:18:d5:bc',
##&nbsp;            'iom_type' => 'IOM6E',
##&nbsp;            'ip_address' => '192.168.1.189',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '000',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0GLA',
##&nbsp;            'protocol_version' => '1.1.1.31',
##&nbsp;            'inband_id' => '0a.03.B',
##&nbsp;            'last_contact_seconds_ago' => '259',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:50:cc:72:8a:23',
##&nbsp;            'iom_type' => 'IOM3',
##&nbsp;            'ip_address' => '192.168.2.35',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '003',
##&nbsp;            'shelf_serial_num' => 'SHJMS000000048F',
##&nbsp;            'protocol_version' => '2.1.1.10',
##&nbsp;            'inband_id' => '0a.00.A',
##&nbsp;            'last_contact_seconds_ago' => '260',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:a0:98:18:e2:4c',
##&nbsp;            'iom_type' => 'IOM6E',
##&nbsp;            'ip_address' => '192.168.2.78',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'reset_cnt' => '000',
##&nbsp;            'shelf_serial_num' => '6000007163;DS22',
##&nbsp;            'protocol_version' => '1.2.2.6',
##&nbsp;            'inband_id' => '0a.01.B',
##&nbsp;            'last_contact_seconds_ago' => '239',
##&nbsp;            'current_state' => '0x5',
##&nbsp;            'mac_address' => '00:a0:98:14:06:4c',
##&nbsp;            'iom_type' => 'IOM6',
##&nbsp;            'ip_address' => '192.168.2.77',
##&nbsp;            'assigner_acpa_id' => '1786426465'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub acpadmin_list_all {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @lines  = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, @all_fields, $i, $k );
    $lines[0] =~ s/Last Contact/last_contact/g;
    $lines[1] =~ s/ACPA ID/acpa_id/g;
    $lines[1] =~ s/S\/N/serial_num/g;
    my @fields1 = split( /\s+/, $lines[0] );
    if ( $lines[1] =~ /\(\w+\s+\w+\)/ ) {
        $lines[1] =~ s/\((\w+)\s+(\w+)\)/$1_$2/g;
    }
    my @fields2 = split( /\s+/, $lines[1] );
    for ( $i = 0; $i <= $#fields1; $i++ ) {
        $all_fields[$i] = $fields1[$i] . "_" . $fields2[$i];
    }
    $k = 0;
    for ( my $j = 3; $j <= $#lines; $j++ ) {
        my @temp = split( /\s+/, $lines[$j] );
        for ( $i = 0; $i <= $#all_fields; $i++ ) {
            $result_array[$k]->{ to_key_name( 'item' => $all_fields[$i] ) }
                = $temp[$i];
        }
        $k++;
    }

    return \@result_array;
} ## end sub acpadmin_list_all

## @name ::acpadmin_stats
##
## @summary Parses the raw output from the "acpadmin stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_stats with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_stats(%args);
## INPUTS TO METHOD CALL 'acpadmin_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IP                  Inband     Shelf            Pwr-Cycl  Pwr-Off   Pwr-On    Exp-Rst        Exp-PwrCycl   Shelf Power
##&nbsp;Address             ID         S/N              Cnt       Cnt       Cnt       Cnt            Cnt           State
##&nbsp;------------------- ---------- ---------------- --------- --------- --------- -------------- ------------- ------------
##&nbsp;192.168.1.1          0a.02.B   SHU0954292N0HAG  002       000       000       002            000                 on
##&nbsp;192.168.1.19         0a.02.A   SHU0954292N0HAG  002       000       000       000            000                 on
##&nbsp;192.168.1.67         0a.01.A   6000007163;DS22  005       002       003       000            000                 on
##&nbsp;192.168.1.69         0a.03.A   SHU0954292N0GLA  000       000       000       000            000                 on
##&nbsp;192.168.1.189        0a.00.B   SHJMS000000048F  000       000       000       001            002                 on
##&nbsp;192.168.2.35         0a.03.B   SHU0954292N0GLA  000       000       000       000            000                 on
##&nbsp;192.168.2.78         0a.00.A   SHJMS000000048F  000       000       000       003            000                 on
##&nbsp;192.168.2.77         0a.01.B   6000007163;DS22  005       002       003       000            000                 on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0HAG',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.02.B',
##&nbsp;            'exp_rst_cnt' => '002',
##&nbsp;            'ip_address' => '192.168.1.1',
##&nbsp;            'pwr_cycl_cnt' => '002'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0HAG',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.02.A',
##&nbsp;            'exp_rst_cnt' => '000',
##&nbsp;            'ip_address' => '192.168.1.19',
##&nbsp;            'pwr_cycl_cnt' => '002'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '003',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => '6000007163;DS22',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '002',
##&nbsp;            'inband_id' => '0a.01.A',
##&nbsp;            'exp_rst_cnt' => '000',
##&nbsp;            'ip_address' => '192.168.1.67',
##&nbsp;            'pwr_cycl_cnt' => '005'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0GLA',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.03.A',
##&nbsp;            'exp_rst_cnt' => '000',
##&nbsp;            'ip_address' => '192.168.1.69',
##&nbsp;            'pwr_cycl_cnt' => '000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHJMS000000048F',
##&nbsp;            'exp_pwrcycl_cnt' => '002',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.00.B',
##&nbsp;            'exp_rst_cnt' => '001',
##&nbsp;            'ip_address' => '192.168.1.189',
##&nbsp;            'pwr_cycl_cnt' => '000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHU0954292N0GLA',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.03.B',
##&nbsp;            'exp_rst_cnt' => '000',
##&nbsp;            'ip_address' => '192.168.2.35',
##&nbsp;            'pwr_cycl_cnt' => '000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '000',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => 'SHJMS000000048F',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '000',
##&nbsp;            'inband_id' => '0a.00.A',
##&nbsp;            'exp_rst_cnt' => '003',
##&nbsp;            'ip_address' => '192.168.2.78',
##&nbsp;            'pwr_cycl_cnt' => '000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pwr_on_cnt' => '003',
##&nbsp;            'shelf_power_state' => 'on',
##&nbsp;            'shelf_serial_num' => '6000007163;DS22',
##&nbsp;            'exp_pwrcycl_cnt' => '000',
##&nbsp;            'pwr_off_cnt' => '002',
##&nbsp;            'inband_id' => '0a.01.B',
##&nbsp;            'exp_rst_cnt' => '000',
##&nbsp;            'ip_address' => '192.168.2.77',
##&nbsp;            'pwr_cycl_cnt' => '005'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub acpadmin_stats {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $first_line = 'IP\s+Inband';
    my @lines  = get_lines( raw_output => $output, first_line => $first_line,response_object => $resp, can_throw => 1 );
    my ( @result_array, @all_fields, $i, $k );

    $k = -1;
    my (@fields1, @fields2, @temp);
    for ( my $j = 0; $j <= $#lines; $j++ ) {
        if( $lines[$j] =~ /(IP\s+Inband.*)/) {
            $lines[$j] =~ $1;
            $lines[$j] =~ s/Shelf Power/shelf_power/g;
            $lines[$j] =~ s/-/_/g;
            @fields1 = split( /\s+/, $lines[$j] );
        }
        if( $lines[$j] =~ /^Address\s+/) {
            $lines[$j] =~ s/S\/N/serial_num/g;
            @fields2 = split( /\s+/, $lines[$j] );
        }

        if($fields2[0]) {
            for(my $head_count = 0; $head_count <= $#fields1; $head_count++) {
                $all_fields[$head_count] = $fields1[$head_count] . "_" . $fields2[$head_count];
            }
        }
        if($lines[$j] =~ /^((\d{1,3})\.){3}\d{1,3}/) {
            $k++;
            @temp = split( /\s+/, $lines[$j] );
            for ( $i = 0; $i <= $#all_fields; $i++ ) {
                $result_array[$k]->{ to_key_name( 'item' => $all_fields[$i] ) }
                = $temp[$i];
            }
        }
    }
    return \@result_array;
} ## end sub acpadmin_stats

## @name ::acpadmin_shelf_power
##
## @summary Parses the raw output from the "acpadmin shelf_power" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from acpadmin_shelf_power with any of its options as described in the 
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_shelf_power(%args);
## INPUTS TO METHOD CALL 'acpadmin_shelf_power' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'power-type' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'adapter-name' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'adapter-shelf-id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'shelf-serial-num' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; acpadmin shelf_power cycle -f 0a.01; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[acpadmin shelf_power cycle] Issuing power_cycle command to shelf 0a.01
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'shelf' => '0a.01',
##&nbsp;            'status' => 'power_cycle'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; acpadmin shelf_power off -f 0a.01; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[acpadmin shelf_power off] Issuing off command to shelf 0a.01
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'shelf' => '0a.01',
##&nbsp;            'status' => 'off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; acpadmin shelf_power on -f 0a.01; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[acpadmin shelf_power on] Issuing on command to shelf 0a.01
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'shelf' => '0a.01',
##&nbsp;            'status' => 'on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub acpadmin_shelf_power {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ($status, @result_array, $shelf);
    if($output =~ /\[acpadmin shelf_power \w+\]\s+/) { 
        $status = $';
    }
    if($status =~ /Issuing\s+(\w+)\s+command\s+to\s+shelf\s+/) {
        $status = $1;
        $shelf = $';
        chomp($shelf);
        $result_array[0]->{shelf} = $shelf;
    }
    $result_array[0]->{status} = $status;
    return \@result_array;
} ## end sub acpadmin_shelf_power


## @name ::acpadmin_configure
##
## @summary Parses the raw output from the "acpadmin configure" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_configure with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_configure(%args);
## INPUTS TO METHOD CALL 'acpadmin_configure' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples


##&nbsp;Command = priv set -q test; acpadmin configure; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub acpadmin_configure {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::acpadmin_expander_reset
##
## @summary Parses the raw output from the "acpadmin expander_reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_expander_reset with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_expander_reset(%args);
## INPUTS TO METHOD CALL 'acpadmin_expander_reset' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin expander_reset 0a.00.A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub acpadmin_expander_reset {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub acpadmin_expander_reset

## @name ::acpadmin_expander_power_cycle
##
## @summary Parses the raw output from the "acpadmin expander_power_cycle" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_expander_power_cycle with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_expander_power_cycle(%args);
## INPUTS TO METHOD CALL 'acpadmin_expander_power_cycle' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin expander_power_cycle 0a.00.B; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub acpadmin_expander_power_cycle {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub acpadmin_expander_power_cycle

## @name ::acpadmin_voltage_status
##
## @summary Parses the raw output from the "acpadmin voltage_status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_voltage_status with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_voltage_status(%args);
## INPUTS TO METHOD CALL 'acpadmin_voltage_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin voltage_status 0a.00.A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Voltage Status Received from ACP module: 0a.0.A
##&nbsp;
##&nbsp;Module Voltage ACP Status
##&nbsp;ACP 0.9v:Good
##&nbsp;ACP 1.2v:Good
##&nbsp;ACP 1.8v:Good
##&nbsp;ACP 2.5v:Good
##&nbsp;ACP 3.3v:Good
##&nbsp;ACP 5.0v:Good
##&nbsp;ACP Hotswap:Good
##&nbsp;
##&nbsp;Module Voltage EBOD Status
##&nbsp;EBOD 1.0v:Good
##&nbsp;EBOD 1.2v:Good
##&nbsp;EBOD 3.3v:Good
##&nbsp;EBOD 4.0v:Good
##&nbsp;EBOD 5.0v:Good
##&nbsp;EBOD Hotswap:Good
##&nbsp;
##&nbsp;Module Voltage ACP Latched Status
##&nbsp;ACP 0.9v:Good
##&nbsp;ACP 1.2v:Good
##&nbsp;ACP 1.8v:Good
##&nbsp;ACP 2.5v:Good
##&nbsp;ACP 3.3v:Good
##&nbsp;ACP 5.0v:Good
##&nbsp;ACP Hotswap:Good
##&nbsp;
##&nbsp;Module Voltage EBOD Latched Status
##&nbsp;EBOD 1.0v:Good
##&nbsp;EBOD 1.2v:Good
##&nbsp;EBOD 3.3v:Good
##&nbsp;EBOD 4.0v:Good
##&nbsp;EBOD 5.0v:Good
##&nbsp;EBOD Hotswap:Good
##&nbsp;
##&nbsp;Module PC Voltage Status
##&nbsp;PC 12.0v:Good
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'voltage_status_received_from_acp_module' => '0a.0.A',
##&nbsp;            'module_voltage_ebod_latched_status' => {
##&nbsp;                                                      'ebod_4.0v' => 'Good',
##&nbsp;                                                      'ebod_5.0v' => 'Good',
##&nbsp;                                                      'ebod_hotswap' => 'Good',
##&nbsp;                                                      'ebod_1.0v' => 'Good',
##&nbsp;                                                      'ebod_3.3v' => 'Good',
##&nbsp;                                                      'ebod_1.2v' => 'Good'
##&nbsp;                                                    },
##&nbsp;            'module_voltage_acp_latched_status' => {
##&nbsp;                                                     'acp_5.0v' => 'Good',
##&nbsp;                                                     'acp_1.8v' => 'Good',
##&nbsp;                                                     'acp_0.9v' => 'Good',
##&nbsp;                                                     'acp_2.5v' => 'Good',
##&nbsp;                                                     'acp_1.2v' => 'Good',
##&nbsp;                                                     'acp_hotswap' => 'Good',
##&nbsp;                                                     'acp_3.3v' => 'Good'
##&nbsp;                                                   },
##&nbsp;            'module_voltage_ebod_status' => {
##&nbsp;                                              'ebod_4.0v' => 'Good',
##&nbsp;                                              'ebod_5.0v' => 'Good',
##&nbsp;                                              'ebod_hotswap' => 'Good',
##&nbsp;                                              'ebod_1.0v' => 'Good',
##&nbsp;                                              'ebod_3.3v' => 'Good',
##&nbsp;                                              'ebod_1.2v' => 'Good'
##&nbsp;                                            },
##&nbsp;            'module_voltage_acp_status' => {
##&nbsp;                                             'acp_5.0v' => 'Good',
##&nbsp;                                             'acp_1.8v' => 'Good',
##&nbsp;                                             'acp_0.9v' => 'Good',
##&nbsp;                                             'acp_2.5v' => 'Good',
##&nbsp;                                             'acp_1.2v' => 'Good',
##&nbsp;                                             'acp_hotswap' => 'Good',
##&nbsp;                                             'acp_3.3v' => 'Good'
##&nbsp;                                           },
##&nbsp;            'module_pc_voltage_status' => {
##&nbsp;                                            'pc_12.0v' => 'Good'
##&nbsp;                                          }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub acpadmin_voltage_status {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, $register );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        my @temp = split( /:\s*/, $line ) if ( $line =~ /:\s*/ );
        if ( $line =~ /(\w+) Voltage (.*) Status|Module PC Voltage Status/ )
        {
            $register = $line;
            $register = to_key_name( item => $register );
            next;
        }
        if ($register) {
            $result_array[0]->{$register}->{ to_key_name( item => $temp[0] ) }
                = $temp[1];
        } else {
            $result_array[0]->{ to_key_name( item => $temp[0] ) } = $temp[1];
        }
    }
    return \@result_array;
} ## end sub acpadmin_voltage_status

## @name ::acpadmin_post_data
##
## @summary Parses the raw output from the "acpadmin post_data" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from acpadmin_post_data with any of its options as described in the
##    corresponding cdef file 'acpadmin.cdefs'
## @synopsis
##      acpadmin_post_data(%args);
## INPUTS TO METHOD CALL 'acpadmin_post_data' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; acpadmin post_data 0a.00.A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Post Data Received from ACP module: 0a.0.A
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;EBOD Post Status Register 1:
##&nbsp; Post Test Completed:PASSED
##&nbsp; EBOD Active Boot Image:A
##&nbsp; EBOD System Monitor Status:PASSED
##&nbsp; EBOD Internal Memory Test:PASSED
##&nbsp; EBOD Memory Test:PASSED
##&nbsp; EBOD Topology:NO CHANGE
##&nbsp;
##&nbsp;EBOD Post Status Register 2:
##&nbsp; HAL Init:PASSED
##&nbsp; VPD Compatibility Test:PASSED
##&nbsp; I2C Test:PASSED
##&nbsp; Lockdown Mode:OFF
##&nbsp; MidPlane CPLD Test:PASSED
##&nbsp; EBOD CPLD Test:PASSED
##&nbsp;
##&nbsp;
##&nbsp;ACP CPLD Version:1.7
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ebod_post_status_register_2' => {
##&nbsp;                                               'ebod_cpld_test' => 'PASSED',
##&nbsp;                                               'lockdown_mode' => 'OFF',
##&nbsp;                                               'i2c_test' => 'PASSED',
##&nbsp;                                               'hal_init' => 'PASSED',
##&nbsp;                                               'midplane_cpld_test' => 'PASSED',
##&nbsp;                                               'vpd_compatibility_test' => 'PASSED'
##&nbsp;                                             },
##&nbsp;            'ebod_post_status_register_1' => {
##&nbsp;                                               'ebod_memory_test' => 'PASSED',
##&nbsp;                                               'ebod_system_monitor_status' => 'PASSED',
##&nbsp;                                               'ebod_active_boot_image' => 'A',
##&nbsp;                                               'post_test_completed' => 'PASSED',
##&nbsp;                                               'ebod_topology' => 'NO CHANGE',
##&nbsp;                                               'ebod_internal_memory_test' => 'PASSED'
##&nbsp;                                             },
##&nbsp;            'post_data_received_from_acp_module' => '0a.0.A',
##&nbsp;            'acp_cpld_version' => '1.7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub acpadmin_post_data {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, $register );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        my @temp = split( /:\s*/, $line );
        $temp[0] =~ s/-/_/g;
        $temp[0] =~ s/'s//g;
        $temp[0] =~ s/\//_or_/g;
        if ( $temp[0] =~ /Post Status Register/ || !$temp[1]) {
            $register = $temp[0];
            $register = to_key_name( item => $register );
            next;
        }
        if ( $register && $temp[0] !~ /ACP CPLD/ ) {
            $result_array[0]->{$register}->{ to_key_name( item => $temp[0] ) }
                = $temp[1];
        } else {
            $result_array[0]->{ to_key_name( item => $temp[0] ) } = $temp[1];
        }
    }
    return \@result_array;
} ## end sub acpadmin_post_data
1;
