#$Id$
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode aggr commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode aggr commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "aggr status" will be aggr_status()
##
## @synopsis
##    use aggr;
##
##    aggr_create(%args);
##    aggr_status(%args);
##    aggr_online(%args);
##    aggr_offline(%args);
##    aggr_destroy(%args);
##    aggr_options(%args);
##    aggr_rename(%args);
##    aggr_media_scrub_status(%args);
##    aggr_scrub_status(%args);
##    aggr_verify_status(%args);
##    aggr_scrub_start(%args);
##    aggr_scrub_stop(%args);
##    aggr_scrub_resume(%args);
##    aggr_scrub_suspend(%args);
##    aggr_verify_start(%args);
##    aggr_verify_stop(%args);
##    aggr_verify_resume(%args);
##    aggr_verify_suspend(%args);
##    aggr_restrict(%args)
##    aggr_add(%args)
##    aggr_wafliron_status(%args)
##    aggr_wafliron_start(%args)
##    aggr_wafliron_stop(%args)
##    aggr_64bit_upgrade_start(%args)
##    aggr_64bit_upgrade_status(%args)
##    aggr_64bit_upgrade_scan_start(%args)
##    aggr_64bit_upgrade_scan_stop(%args)
##    aggr_copy_start(%args);
##    aggr_copy_status(%args);
##    aggr_copy_abort(%args);
##    aggr_copy_throttle(%args);
##    aggr_split(%args);
##    aggr_show_space(%args);
##    aggr_mirror(%args);
##    aggr_read_fsid(%args);
##    aggr_rewrite_fsid(%args);
##    aggr_reserve_show(%args);
##    aggr_reserve_default_show(%args);
##    aggr_reserve_modify(%args);
##    aggr_reserve_default_modify(%args);
##    aggr_wafliron_commit(%args);
##    aggr_wafliron_reject(%args);
##
## @author dl-nacl-dev@netapp.com, govinda1@netapp.com, benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::aggr;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::MissingValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException qw(:try);

## @name ::aggr_status
##
## @summary Parses the raw output from the "aggr status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_status(%args);
## INPUTS TO METHOD CALL 'aggr_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'vbn-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'maintenance-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'broken-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'light' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'device' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'checksum' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'volumes' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'show-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'spare-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'raid-disk' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; aggr status; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'options' => [
##&nbsp;                           {
##&nbsp;                             'root' => 1
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'status' => 'raid_dp, aggr, 64-bit',
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'state' => 'online'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; aggr status -r; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'spare_disks' => [
##&nbsp;                               {
##&nbsp;                                 'pool1' => []
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'pool0' => [
##&nbsp;                                              {
##&nbsp;                                                'bay' => '?',
##&nbsp;                                                'ha' => 'v4',
##&nbsp;                                                'phys(MB/blks)' => '1027/2104448',
##&nbsp;                                                'device' => 'v4.19',
##&nbsp;                                                'rpm' => '15000',
##&nbsp;                                                'shelf' => '?',
##&nbsp;                                                'used(MB/blks)' => '1020/2089984',
##&nbsp;                                                'pool' => '0',
##&nbsp;                                                'chan' => 'FC:B',
##&nbsp;                                                'type' => 'FCAL',
##&nbsp;                                                'raid_disk' => 'spare',
##&nbsp;                                                'is_zeroed' => 'not zeroed'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'bay' => '?',
##&nbsp;                                                'ha' => 'v5',
##&nbsp;                                                'phys(MB/blks)' => '1027/2104448',
##&nbsp;                                                'device' => 'v5.32',
##&nbsp;                                                'rpm' => '15000',
##&nbsp;                                                'shelf' => '?',
##&nbsp;                                                'used(MB/blks)' => '1020/2089984',
##&nbsp;                                                'pool' => '0',
##&nbsp;                                                'chan' => 'FC:B',
##&nbsp;                                                'type' => 'FCAL',
##&nbsp;                                                'raid_disk' => 'spare',
##&nbsp;                                                'is_zeroed' => ''
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'aggregate' => [
##&nbsp;                             {
##&nbsp;                               'plex' => [
##&nbsp;                                           {
##&nbsp;                                             'status' => 'online, normal, active, pool0',
##&nbsp;                                             'plex_name' => '/aggr0/plex0',
##&nbsp;                                             'raid_group' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'disks' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'bay' => '?',
##&nbsp;                                                                                'ha' => 'v4',
##&nbsp;                                                                                'used_mb/blks' => '1020/2089984',
##&nbsp;                                                                                'device' => 'v4.16',
##&nbsp;                                                                                'rpm' => '15000',
##&nbsp;                                                                                'shelf' => '?',
##&nbsp;                                                                                'pool' => '0',
##&nbsp;                                                                                'chan' => 'FC:B',
##&nbsp;                                                                                'type' => 'FCAL',
##&nbsp;                                                                                'phys_mb/blks' => '1027/2104448',
##&nbsp;                                                                                'raid_disk' => 'dparity'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'bay' => '?',
##&nbsp;                                                                                'ha' => 'v4',
##&nbsp;                                                                                'used_mb/blks' => '1020/2089984',
##&nbsp;                                                                                'device' => 'v4.18',
##&nbsp;                                                                                'rpm' => '15000',
##&nbsp;                                                                                'shelf' => '?',
##&nbsp;                                                                                'pool' => '0',
##&nbsp;                                                                                'chan' => 'FC:B',
##&nbsp;                                                                                'type' => 'FCAL',
##&nbsp;                                                                                'phys_mb/blks' => '1027/2104448',
##&nbsp;                                                                                'raid_disk' => 'data'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                                 'status' => 'normal, block checksums',
##&nbsp;                                                                 'raid_group_name' => '/aggr0/plex0/rg0'
##&nbsp;                                                               }
##&nbsp;                                                             ]
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               'status' => 'raid_dp',
##&nbsp;                               'checksums' => 'block',
##&nbsp;                               'aggregate' => 'aggr0',
##&nbsp;                               'state' => 'online'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; aggr status -v; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'plex' => [
##&nbsp;                        {
##&nbsp;                          'status' => 'online, normal, active',
##&nbsp;                          'raid_group' => [
##&nbsp;                                            {
##&nbsp;                                              'status' => 'normal, block checksums',
##&nbsp;                                              'name' => '/aggr0/plex0/rg0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                          'name' => '/aggr0/plex0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'volumes' => 'vol0',
##&nbsp;            'options' => [
##&nbsp;                           {
##&nbsp;                             'raid_lost_write' => 'on',
##&nbsp;                             'ignore_inconsistent' => 'off',
##&nbsp;                             'ha_policy' => 'cfo',
##&nbsp;                             'free_space_realloc' => 'off',
##&nbsp;                             'root' => 1,
##&nbsp;                             'raid_cv' => 'on',
##&nbsp;                             'lost_write_protect' => 'on',
##&nbsp;                             'resyncsnaptime' => '60',
##&nbsp;                             'fs_size_fixed' => 'off',
##&nbsp;                             'raidtype' => 'raid_dp',
##&nbsp;                             'snapshot_autodelete' => 'on',
##&nbsp;                             'thorough_scrub' => 'off',
##&nbsp;                             'hybrid_enabled' => 'off',
##&nbsp;                             'nosnap' => 'off',
##&nbsp;                             'snapmirrored' => 'off',
##&nbsp;                             'raidsize' => '16',
##&nbsp;                             'percent_snapshot_space' => '0%',
##&nbsp;                             'diskroot' => 1
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'status' => 'raid_dp, aggr, 64-bit, rlw_on',
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'state' => 'online'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; aggr status; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'options' => [
##&nbsp;                           {
##&nbsp;                             'root' => 1
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'status' => 'raid_dp, aggr, 64-bit',
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'state' => 'online'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; aggr status -r; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'spare_disks' => [
##&nbsp;                               {
##&nbsp;                                 'pool1' => []
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'pool0' => [
##&nbsp;                                              {
##&nbsp;                                                'bay' => '?',
##&nbsp;                                                'ha' => 'v4',
##&nbsp;                                                'phys(MB/blks)' => '1027/2104448',
##&nbsp;                                                'device' => 'v4.19',
##&nbsp;                                                'rpm' => '15000',
##&nbsp;                                                'shelf' => '?',
##&nbsp;                                                'used(MB/blks)' => '1020/2089984',
##&nbsp;                                                'pool' => '0',
##&nbsp;                                                'chan' => 'FC:B',
##&nbsp;                                                'type' => 'FCAL',
##&nbsp;                                                'raid_disk' => 'spare',
##&nbsp;                                                'is_zeroed' => 'not zeroed'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'bay' => '?',
##&nbsp;                                                'ha' => 'v4',
##&nbsp;                                                'phys(MB/blks)' => '1027/2104448',
##&nbsp;                                                'device' => 'v4.20',
##&nbsp;                                                'rpm' => '15000',
##&nbsp;                                                'shelf' => '?',
##&nbsp;                                                'used(MB/blks)' => '1020/2089984',
##&nbsp;                                                'pool' => '0',
##&nbsp;                                                'chan' => 'FC:B',
##&nbsp;                                                'type' => 'FCAL',
##&nbsp;                                                'raid_disk' => 'spare',
##&nbsp;                                                'is_zeroed' => 'not zeroed'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'bay' => '?',
##&nbsp;                                                'ha' => 'v5',
##&nbsp;                                                'phys(MB/blks)' => '1027/2104448',
##&nbsp;                                                'device' => 'v5.32',
##&nbsp;                                                'rpm' => '15000',
##&nbsp;                                                'shelf' => '?',
##&nbsp;                                                'used(MB/blks)' => '1020/2089984',
##&nbsp;                                                'pool' => '0',
##&nbsp;                                                'chan' => 'FC:B',
##&nbsp;                                                'type' => 'FCAL',
##&nbsp;                                                'raid_disk' => 'spare',
##&nbsp;                                                'is_zeroed' => ''
##&nbsp;                                              }
##&nbsp;                                            ]
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'aggregate' => [
##&nbsp;                             {
##&nbsp;                               'plex' => [
##&nbsp;                                           {
##&nbsp;                                             'status' => 'online, normal, active, pool0',
##&nbsp;                                             'plex_name' => '/aggr0/plex0',
##&nbsp;                                             'raid_group' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'disks' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'bay' => '?',
##&nbsp;                                                                                'ha' => 'v4',
##&nbsp;                                                                                'used_mb/blks' => '1020/2089984',
##&nbsp;                                                                                'device' => 'v4.16',
##&nbsp;                                                                                'rpm' => '15000',
##&nbsp;                                                                                'shelf' => '?',
##&nbsp;                                                                                'pool' => '0',
##&nbsp;                                                                                'chan' => 'FC:B',
##&nbsp;                                                                                'type' => 'FCAL',
##&nbsp;                                                                                'phys_mb/blks' => '1027/2104448',
##&nbsp;                                                                                'raid_disk' => 'dparity'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'bay' => '?',
##&nbsp;                                                                                'ha' => 'v4',
##&nbsp;                                                                                'used_mb/blks' => '1020/2089984',
##&nbsp;                                                                                'device' => 'v4.18',
##&nbsp;                                                                                'rpm' => '15000',
##&nbsp;                                                                                'shelf' => '?',
##&nbsp;                                                                                'pool' => '0',
##&nbsp;                                                                                'chan' => 'FC:B',
##&nbsp;                                                                                'type' => 'FCAL',
##&nbsp;                                                                                'phys_mb/blks' => '1027/2104448',
##&nbsp;                                                                                'raid_disk' => 'data'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                                 'status' => 'normal, block checksums',
##&nbsp;                                                                 'raid_group_name' => '/aggr0/plex0/rg0'
##&nbsp;                                                               }
##&nbsp;                                                             ]
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               'status' => 'raid_dp',
##&nbsp;                               'checksums' => 'block',
##&nbsp;                               'aggregate' => 'aggr0',
##&nbsp;                               'state' => 'online'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; aggr status -v; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'plex' => [
##&nbsp;                        {
##&nbsp;                          'status' => 'online, normal, active',
##&nbsp;                          'raid_group' => [
##&nbsp;                                            {
##&nbsp;                                              'status' => 'normal, block checksums',
##&nbsp;                                              'name' => '/aggr0/plex0/rg0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                          'name' => '/aggr0/plex0'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'volumes' => 'vol0',
##&nbsp;            'options' => [
##&nbsp;                           {
##&nbsp;                             'raid_lost_write' => 'on',
##&nbsp;                             'ignore_inconsistent' => 'off',
##&nbsp;                             'ha_policy' => 'cfo',
##&nbsp;                             'free_space_realloc' => 'off',
##&nbsp;                             'root' => 1,
##&nbsp;                             'raid_cv' => 'on',
##&nbsp;                             'lost_write_protect' => 'on',
##&nbsp;                             'resyncsnaptime' => '60',
##&nbsp;                             'fs_size_fixed' => 'off',
##&nbsp;                             'raidtype' => 'raid_dp',
##&nbsp;                             'snapshot_autodelete' => 'on',
##&nbsp;                             'thorough_scrub' => 'off',
##&nbsp;                             'hybrid_enabled' => 'off',
##&nbsp;                             'nosnap' => 'off',
##&nbsp;                             'snapmirrored' => 'off',
##&nbsp;                             'raidsize' => '16',
##&nbsp;                             'percent_snapshot_space' => '0%',
##&nbsp;                             'diskroot' => 1
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'status' => 'raid_dp, aggr, 64-bit, rlw_on',
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'state' => 'online'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = aggr status -S
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregate_metadata' => '636KB',
##&nbsp;            'aggregate_metadata_percent' => '0%',
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'volume_footprints' => '533GB',
##&nbsp;            'total_percent' => '95%',
##&nbsp;            'volume_footprints_percent' => '95%',
##&nbsp;            'snapshot_reserve' => '28.0GB',
##&nbsp;            'snapshot_reserve_percent' => '5%',
##&nbsp;            'volume_footprints_percent' => '95%',
##&nbsp;            'total' => '533GB'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; aggr status -R; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Aggregate temp_aggr (online, raid_dp, parity uninit'd!) (block checksums)
##&nbsp;  Plex /temp_aggr/plex0 (online, normal, active)
##&nbsp;    RAID group /temp_aggr/plex0/rg0 (normal, block checksums)
##&nbsp;
##&nbsp;      RAID Disk	Device    Model Number     Serial Number        VBN Start          VBN End
##&nbsp;      ---------	------    ------------     -------------        ---------          -------
##&nbsp;      dparity   0b.3      Virtualdisk      6000c292f37d2b0a2a111893d8030e2f -                  -
##&nbsp;      parity    0b.10     Virtualdisk      6000c29a045e72e4a0167fd9ef6fb9ad -                  -
##&nbsp;      data      0b.6      Virtualdisk      6000c292aab369ca5740c19e0d8129da 0                  15935
##&nbsp;
##&nbsp;Aggregate aggr0 (online, raid_dp, parity uninit'd!) (block checksums)
##&nbsp;  Plex /aggr0/plex0 (online, normal, active)
##&nbsp;    RAID group /aggr0/plex0/rg0 (normal, block checksums)
##&nbsp;
##&nbsp;      RAID Disk	Device    Model Number     Serial Number        VBN Start          VBN End
##&nbsp;      ---------	------    ------------     -------------        ---------          -------
##&nbsp;      dparity   0b.0      Virtualdisk      6000c2990173dbda5cb137365abc2a54 -                  -
##&nbsp;      parity    0b.1      Virtualdisk      6000c2959a68b46387295310b16ae4f4 -                  -
##&nbsp;      data      0b.2      Virtualdisk      6000c29fbce54d9fc25ae3ac08cf1431 0                  925247
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregate' => [
##&nbsp;                             {
##&nbsp;                               'plex' => [
##&nbsp;                                           {
##&nbsp;                                             'status' => 'online, normal, active',
##&nbsp;                                             'plex_name' => '/temp_aggr/plex0',
##&nbsp;                                             'raid_group' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'disks' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c292f37d2b0a2a111893d8030e2f',
##&nbsp;                                                                                'vbn_start' => '-',
##&nbsp;                                                                                'device' => '0b.3',
##&nbsp;                                                                                'raid_disk' => 'dparity',
##&nbsp;                                                                                'vbn_end' => '-',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c29a045e72e4a0167fd9ef6fb9ad',
##&nbsp;                                                                                'vbn_start' => '-',
##&nbsp;                                                                                'device' => '0b.10',
##&nbsp;                                                                                'raid_disk' => 'parity',
##&nbsp;                                                                                'vbn_end' => '-',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c292aab369ca5740c19e0d8129da',
##&nbsp;                                                                                'vbn_start' => '0',
##&nbsp;                                                                                'device' => '0b.6',
##&nbsp;                                                                                'raid_disk' => 'data',
##&nbsp;                                                                                'vbn_end' => '15935',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                                 'status' => 'normal, block checksums',
##&nbsp;                                                                 'raid_group_name' => '/temp_aggr/plex0/rg0'
##&nbsp;                                                               }
##&nbsp;                                                             ]
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               'status' => 'raid_dp, parity uninit\'d!',
##&nbsp;                               'checksums' => 'block',
##&nbsp;                               'aggregate' => 'temp_aggr',
##&nbsp;                               'state' => 'online'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'plex' => [
##&nbsp;                                           {
##&nbsp;                                             'status' => 'online, normal, active',
##&nbsp;                                             'plex_name' => '/aggr0/plex0',
##&nbsp;                                             'raid_group' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'disks' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c2990173dbda5cb137365abc2a54',
##&nbsp;                                                                                'vbn_start' => '-',
##&nbsp;                                                                                'device' => '0b.0',
##&nbsp;                                                                                'raid_disk' => 'dparity',
##&nbsp;                                                                                'vbn_end' => '-',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c2959a68b46387295310b16ae4f4',
##&nbsp;                                                                                'vbn_start' => '-',
##&nbsp;                                                                                'device' => '0b.1',
##&nbsp;                                                                                'raid_disk' => 'parity',
##&nbsp;                                                                                'vbn_end' => '-',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              },
##&nbsp;                                                                              {
##&nbsp;                                                                                'serial_number' => '6000c29fbce54d9fc25ae3ac08cf1431',
##&nbsp;                                                                                'vbn_start' => '0',
##&nbsp;                                                                                'device' => '0b.2',
##&nbsp;                                                                                'raid_disk' => 'data',
##&nbsp;                                                                                'vbn_end' => '925247',
##&nbsp;                                                                                'model_number' => 'Virtualdisk'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                                 'status' => 'normal, block checksums',
##&nbsp;                                                                 'raid_group_name' => '/aggr0/plex0/rg0'
##&nbsp;                                                               }
##&nbsp;                                                             ]
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                               'status' => 'raid_dp, parity uninit\'d!',
##&nbsp;                               'checksums' => 'block',
##&nbsp;                               'aggregate' => 'aggr0',
##&nbsp;                               'state' => 'online'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub aggr_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"        => $output,
        "command"           => "aggr status",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _aggr_status(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} ## end sub aggr_status

sub _aggr_status {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @want_array   = ();

    # For aggr_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    # Extracting the command option.
    my $option = $command_line;
    $option =~ s/aggr\sstatus\s?//;
    if ( $option !~ /-\w/ ) {
        $option = "";
    }

    # Possible states a aggregate can be in
    my @aggr_states = (
        "online",            "offline",
        "creating",          "aggr",
        "copying",           "degraded",
        "foreign",           "growing",
        "initializing",      "invalid",
        "ironing",           "mirror degraded",
        "mirrored",          "needs check",
        "partial",           "raid0",
        "raid4",             "raid_dp",
        "raid_tec",          "reconstruct",
        "resyncing",         "snapmirrored",
        "trad",              "verifying",
        "wafl inconsistent", "restricted",
        "mixed_raid_type",   "failed",
    );
    my $some_state = join( "|", @aggr_states );

    # Based on options sent to aggr_status command, the parsing method will be
    # decided
    if ( ( $option eq "" ) || ( $option =~ /-l/ ) || ( $option =~ /-i/ ) ) {
        my $first_line = 'Aggr\s+State\s+Status\s+(Options|Volumes)';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        @arr = grep {! /\[\S+:\S+:\S+]/} @arr;
        shift(@arr);    #Remove Heading line

        my $if_specific = 0;
        my ( @aggr_array, @parsed );

        # The aggr status command has been executed for one specific aggrume
        if ( grep( /Volumes:/, @arr ) ) {
            $if_specific = 1;
        }
        if ( $if_specific == 1 ) {
            @aggr_array = ( \@arr );
            @parsed     = _parse_aggr_data(
                "aggregate_array_ref" => \@aggr_array,
                "command_option"      => $option
            );
            return \@parsed;
        } else {
            @aggr_array = ();
            my @temp_array = ( $arr[0] );    #Push the first line always

            # Now process the rest of the lines
            for ( my $line = 1; $line < @arr; $line++ ) {
                next if ( $arr[$line] eq "" );
                if ( $arr[$line]
                    =~ /($some_state)\s+\b(raid\d+|raid_dp|raid_tec|mixed_raid_type),?/
                    ) {
                    push( @aggr_array, [@temp_array] );
                    @temp_array = ( $arr[$line] );
                } else {
                    push( @temp_array, $arr[$line] );
                }
            } ## end for ( my $line = 1; $line...
            push( @aggr_array, [@temp_array] )
                ;    #For pushing the last aggr data collected
            @parsed = _parse_aggr_data(
                "aggregate_array_ref" => \@aggr_array,
                "command_option"      => $option
            );
            return \@parsed;
        } ## end else [ if ( $if_specific == 1)
    } elsif ( $option =~ /-b/ || $option =~ /-d/ ) {
        my ( @heading, $first_line );
        if ( $option =~ /-b/ ) {
            $first_line
                = 'Aggr\s+Block\sSize\s\(bytes\)\s+Aggr\sSize\s\(blocks\)\s+FS\sSize\s\(blocks\)';
            @heading = (
                "aggr",              "block_size(bytes)",
                "aggr_size(blocks)", "fs_size(blocks)"
            );
        } else {
            $first_line
                = 'Device\s+HA\s+SHELF\s+BAY\s+CHAN\s+Disk\sVital\sProduct\sInformation';
            @heading = (
                "device", "ha", "shelf", "bay", "chan",
                "disk_vital_product_information"
            );
        } ## end else [ if ( $option =~ /-b/ )
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # The first two lines in the @arr contains heading and underline
        # Hence we remove those two lines before parsing
        splice( @arr, 0, 2 );

        for ( my $count = 0; $count < @arr; $count++ ) {
            my %output = ();
            my $index  = 0;
            my @values = split( /\s+/, $arr[$count] );
            # On mcc-ip the channel remains blank throwing off the parser. This step adds in a -
            # BURT 1087841
            if ( scalar(@heading) - scalar(@values) == 1 && scalar(@values) == 5) {
                splice @values, 4, 0, '-';
            }
            foreach (@values) {
                $output{ $heading[$index] } = $_;
                $index++;
            }
            push( @want_array, \%output );
        } ## end for ( my $count = 0; $count...
        return \@want_array;
    } elsif ( $option =~ /-c/ ) {
        my $first_line = 'Aggr\s+Checksum\sstyle\s+Checksum\sstatus';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @heading = ( "aggr", "checksum_style", "checksum_status" );

        # Remove the heading line from the array
        shift(@arr);
        for ( my $count = 0; $count < @arr; $count++ ) {
            my %output = ();
            my $index  = 0;
            my @values = split( /\s+/, $arr[$count] );
            $values[2] = $values[2] . " " . $values[3];
            pop(@values);
            foreach (@values) {
                $output{ $heading[$index] } = $_;
                $index++;
            }
            push( @want_array, \%output );
        } ## end for ( my $count = 0; $count...
        return \@want_array;
    } elsif ( $option =~ /-f/ ) {
        my $first_line = '(Broken)?(Local broken)?\sdisks\s?(\(?empty\)?)?';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        if ( grep( /Broken\sdisks\s?\(empty\)/, @arr ) ) {
            my @want_array;
            return \@want_array;
        } else {
            shift(@arr);    #First line is an empty line
            my $arr_ref     = \@arr;
            my $disk_output = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $arr_ref );
            return $disk_output;
        } ## end else [ if ( grep( /Broken\sdisks\s?\(empty\)/...
    } elsif ( $option =~ /-r|-R/ ) {
        my $first_line = '((Volume\s\w+\s.*)|(Aggregate\s\w+\s.*))';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # necessary data structure to logically separate the data
        my ( @Volume, @Aggregate, @Pool_spare_disks, @Spare_disks,
            $Broken_disks, $Partner_disks, $Maintenance_disks );
        my ( $Volume_index, $Aggr_index, $Pool_spare_index, $Spare_index )
            = ( 0, 0, 0, 0 );
        my $temp_ref;
        my %output = ();
        for ( my $i = 0; $i < @arr; $i++ ) {
            if ( $arr[$i]
                =~ /^(Volume\s.*|Aggregate\s.*|Pool\d\sspare\sdisks\s?(\(empty\))?|Spare\sdisks\s?(\(empty\))?|Broken\sdisks|Partner\sdisks|Maintenance\sdisks)$/
                ) {

                # The next logical block has started
                my $matched = $1;
                if ( $matched =~ /^Volume/ ) {
                    $Volume[$Volume_index] = [];
                    $temp_ref = $Volume[ $Volume_index++ ];
                } elsif ( $matched =~ /^Aggregate/ ) {
                    $Aggregate[$Aggr_index] = [];
                    $temp_ref = $Aggregate[ $Aggr_index++ ];
                } elsif ( $matched =~ /Pool\d\sspare\sdisks/ ) {
                    $Pool_spare_disks[$Pool_spare_index] = [];
                    $temp_ref = $Pool_spare_disks[ $Pool_spare_index++ ];
                } elsif ( $matched eq "Spare disks" ) {
                    $Spare_disks[$Spare_index] = [];
                    $temp_ref = $Spare_disks[ $Spare_index++ ];
                } elsif ( $matched eq "Broken disks" ) {
                    $Broken_disks = [];
                    $temp_ref     = $Broken_disks;
                } elsif ( $matched eq "Maintenance disks" ) {
                    $Maintenance_disks = [];
                    $temp_ref          = $Maintenance_disks;
                } else {
                    $Partner_disks = [];
                    $temp_ref      = $Partner_disks;
                }
                push( @$temp_ref, $arr[$i] );
            } else {
                push( @$temp_ref, $arr[$i] );
            }
        } ## end for ( my $i = 0; $i < @arr...
        if (@Volume) {
            $output{"volume"} = _get_vol_parsed(
                "volume_ref"      => \@Volume,
                "response_object" => $response
            );
        } ## end if (@Volume)
        if (@Aggregate) {
            $output{"aggregate"} = _get_vol_parsed(
                "volume_ref"      => \@Aggregate,
                "response_object" => $response
            );
        } ## end if (@Aggregate)
        if (@Pool_spare_disks) {
            $output{"spare_disks"}
                = _get_spare_disks( "spare_disks_ref" => \@Pool_spare_disks );
        }
        if (@Spare_disks) {
            $output{"spare_disks"}
                = _get_spare_disks( "spare_disks_ref" => \@Spare_disks );
        }
        if ( defined($Broken_disks) ) {
            $output{"broken_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Broken_disks );
        }
        if ( defined($Partner_disks) ) {
            $output{"partner_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Partner_disks );
        }
        if ( defined($Maintenance_disks) ) {
            $output{"maintenance_disks"} = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $Maintenance_disks );
        }
        push( @want_array, \%output );
        return \@want_array;
    } elsif ( $option =~ /-s/ ) {
        my $first_line
            = '([Pp]ool[01]\sspare\sdisks\s?(\(empty\))?|[Ss]pare\sdisks\s?(\(empty\))?)';
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my ( @Pool_spare_disks, @Spare_disks );
        my ( $Pool_spare_index, $Spare_index ) = ( 0, 0 );
        my $temp_ref;
        my $output_ref;

        for ( my $i = 0; $i < @arr; $i++ ) {
            next if ( $arr[$i] eq "" );
            if ( $arr[$i]
                =~ /^(Pool\d\sspare\sdisks\s?(\(empty\))?|Spare\sdisks\s?(\(empty\))?)$/
                ) {
                my $matched = $1;
                if ( $matched =~ /Pool\d\sspare\sdisks/ ) {
                    $Pool_spare_disks[$Pool_spare_index] = [];
                    $temp_ref = $Pool_spare_disks[ $Pool_spare_index++ ];
                } else {
                    $Spare_disks[$Spare_index] = [];
                    $temp_ref = $Spare_disks[ $Spare_index++ ];
                }
                push( @$temp_ref, $arr[$i] );
            } else {
                push( @$temp_ref, $arr[$i] );
            }
        } ## end for ( my $i = 0; $i < @arr...
        if (@Pool_spare_disks) {
            $output_ref
                = _get_spare_disks( "spare_disks_ref" => \@Pool_spare_disks );
        }
        if (@Spare_disks) {
            $output_ref
                = _get_spare_disks( "spare_disks_ref" => \@Spare_disks );
        }
        return $output_ref;
    } elsif ( $option =~ /-v/ ) {
        my $first_line = 'Aggr\s+State\s+Status\s+Options';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );

        # Remove the heading line
        shift(@arr);

        # Storing the data displayed for each of the aggr in an anonymous
        # array and pushing the reference into @aggr_array for later
        # processing of individual fields
        #
        my @aggr_array = ();
        my @temp_array = ( $arr[0] );    #Push the first line always

        # Now process the rest of the lines
        for ( my $line = 1; $line < @arr; $line++ ) {
            if ( $arr[$line]
                =~ /($some_state)\s+\b(raid\d+|raid_dp|raid_tec|mixed_raid_type),?/ ) {
                push( @aggr_array, [@temp_array] );
                @temp_array = ( $arr[$line] );
            } else {
                push( @temp_array, $arr[$line] );
            }
        } ## end for ( my $line = 1; $line...
        push( @aggr_array, [@temp_array] )
            ;    #For pushing the last aggregate data  collected
        my @parsed = _parse_aggr_data(
            "aggregate_array_ref" => \@aggr_array,
            "command_option"      => "-v"
        );
        return \@parsed;
    } elsif ( $option =~ /-m/ ) {
        my $first_line = 'Maintenance\sdisks\s(\(?empty\)?)?';
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @want_array;
        if ( grep( /Maintenance\sdisks\s?\(empty\)/, @arr ) ) {
            push @want_array;
            return \@want_array;
        } else {
            shift(@arr);    #First line is an empty line
            my $arr_ref     = \@arr;
            my $disk_output = _get_broken_or_partner_disks(
                "broken_or_partner_disks_ref" => $arr_ref );
            return $disk_output;
        } ## end else [ if ( grep( /Maintenance\sdisks\s?\(empty\)/...
    } elsif ( $option =~ /-S/ ) {

        # No first line to ignore
        my $first_line = '';
        my @lines      = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @aggr_array = ();
        my $aggr       = "";
        my %aggr_hash;
        foreach my $line (@lines) {
            if ( $line =~ /Aggregate\s*:\s*(\w+)/i ) {
                if ( $aggr ne "" ) {

                    # Copy hash contents into the aggr array
                    push( @aggr_array, {%aggr_hash} );
                    %aggr_hash = ();
                }
                $aggr = $1;
                $aggr_hash{aggregate} = $aggr;
            } elsif ( $line =~ /Volume Footprints\s+([\d\.]+\w?B)\s+(\d+%)/i )
            {
                $aggr_hash{volume_footprints}         = $1;
                $aggr_hash{volume_footprints_percent} = $2;
            } elsif (
                $line =~ /Aggregate Metadata\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $aggr_hash{aggregate_metadata}         = $1;
                $aggr_hash{aggregate_metadata_percent} = $2;
            } elsif ( $line =~ /Snapshot Reserve\s+([\d\.]+\w?B)\s+(\d+%)/i )
            {
                $aggr_hash{snapshot_reserve}         = $1;
                $aggr_hash{snapshot_reserve_percent} = $2;
            } elsif ( $line =~ /Total Used\s+([\d\.]+\w?B)\s+(\d+%)/i ) {
                $aggr_hash{total}         = $1;
                $aggr_hash{total_percent} = $2;
            }

            # The rest of the lines are table headers and can be ignored
        }
        if ( $aggr ne "" ) {
            push( @aggr_array, \%aggr_hash );
        }
        return \@aggr_array;
    } else {
        NACL::APISet::Exceptions::InvalidParamException->throw(
            text            => "Invalid argument is sent",
            response_object => $response
        );
    } ## end else [ if ( ( $option eq "" )...
} ## end sub _aggr_status

sub _parse_aggr_data {
    my %args           = @_;
    my $aggr_array_ref = $args{"aggregate_array_ref"};
    my $option         = $args{"command_option"};
    my @aggr_array     = @$aggr_array_ref;
    my ( @want_array, %aggr_hash );
    foreach my $array_ref (@aggr_array) {
        my @aggr_data = @$array_ref;
        my %aggr_info = ();
        my ( $temp, $snapshot_settings, $autosize_settings );

        # Process the first line
        $aggr_data[0]
            =~ /(\S+)\s+(\w+)\s+(mixed_raid_type|raid\d+|raid_dp|raid_tec),?\s?(\w+)?\s*(.*)/;
        $aggr_info{"aggregate"} = $1;
        $aggr_info{"state"}     = $2;
        $aggr_info{"status"}    = $3 . ", " . $4;
        $temp                   = $5;

        my $current_plex;
        my $plex_hash_ref = {};
        my $raid_hash_ref = {};
        my %plex_hash;
        my %raid_hash;
        my $reached_settings = 0;
        for ( my $line = 1; $line < @aggr_data; $line++ ) {
            next if ( $aggr_data[$line] eq "" );
            if ( $aggr_data[$line] =~ /([Pp]lex|RAID)/ ) {
                if ( $aggr_data[$line] =~ /Plex\s+(.*):\s(.*)$/ ) {
                    $plex_hash_ref = {} if exists( $plex_hash_ref->{"name"} );
                    $plex_hash_ref->{"name"}   = $1;
                    $plex_hash_ref->{"status"} = $2;
                    push( @{ $aggr_info{"plex"} }, $plex_hash_ref );
                    next;
                } else {
                    $aggr_data[$line] =~ /RAID\sgroup\s(.*):\s(.*)$/;
                    $raid_hash_ref = {} if exists( $raid_hash_ref->{"name"} );
                    $raid_hash_ref->{"name"}   = $1;
                    $raid_hash_ref->{"status"} = $2;
                    push(
                        @{ $plex_hash_ref->{"raid_group"} },
                        $raid_hash_ref
                    );
                    next;
                } ## end else [ if ( $aggr_data[$line]...
            } ## end if ( $aggr_data[$line]...
            if ( $aggr_data[$line] =~ /(.*):\s+(.*)/ ) {
                my $key        = lc($1);
                my $total_vols = $2;
                $key =~ s/\s+/_/g;
                $total_vols = "" if ( $total_vols eq "<none>" );
                ++$line;
                while ( $aggr_data[ $line ] && ( $aggr_data[ $line ] ) !~ /Plex.*/ ) {
                    $total_vols = "$total_vols" . "$aggr_data[$line]";
                    ++$line;
                }
                $aggr_info{$key} = $total_vols;
                --$line;
                next;
            } ## end if ( $aggr_data[$line]...
            if ( $reached_settings == 0 ) {
                if ( grep( /\s{2,}/, $aggr_data[$line] ) ) {
                    $aggr_data[$line] =~ /(.{18})(.*)/;
                    $aggr_info{"status"} .= ", " . $1;
                    my $second = $2;
                    if ( $option =~ /-i/ ) {
                        $temp .= ", " . $second;
                    } else {
                        $temp .= $second;
                    }
                    $aggr_info{"status"} =~ s/\s+$//;
                    next;
                } else {
                    if ( $aggr_data[$line] =~ /=/ ) {
                        $temp .= $aggr_data[$line];
                        next;
                    } else {
                        $aggr_info{"status"} .= ", " . $aggr_data[$line];
                        $aggr_info{"status"} =~ s/\s+$//;
                    }
                } ## end else [ if ( grep( /\s{2,}/, $aggr_data...
            } ## end if ( $reached_settings...
        } ## end for ( my $line = 1; $line...

        #Process data in $temp
        if ( ( $option =~ /-i/ ) && $temp ) {
            $temp =~ s/,\s/,/g;
            $aggr_info{"volumes"} = $temp;
        } elsif ($temp) {
            my %temp_hash;
            $temp =~ s/,\s*/,/g;
            foreach ( split( ",", $temp ) ) {
                if ( $_ =~ /(.*)=(.*)/ ) {
                    my $key   = lc($1);
                    my $value = $2;
                    $key =~ s/\s+/_/g;
                    $temp_hash{$key} = $value;
                } else {
                    my $key = lc($_);
                    $key =~ s/\s+/_/g;
                    $temp_hash{$key} = 1;
                }
            } ## end foreach ( split( ",", $temp...
            $aggr_info{"options"} = [ \%temp_hash ];
        } ## end elsif ($temp)
        push( @want_array, \%aggr_info );
    } ## end foreach my $array_ref (@aggr_array)
    return @want_array;
} ## end sub _parse_aggr_data

sub _get_spare_disks {
    my %args           = @_;
    my $pool_array_ref = $args{"spare_disks_ref"};
    my @pool_array     = @$pool_array_ref;
    my @pool_info      = ();
    my $key;
    my @heading = (
        "raid_disk", "device", "ha", "shelf", "bay", "chan", "pool", "type",
        "rpm", "used(MB/blks)", "phys(MB/blks)"
    );
    foreach my $array_ref (@pool_array) {
        my @pool_data = @$array_ref;
        my %temp_pool_hash;
        my $no_pool = 0;
        if ( grep( /Spare\sdisks\s\(empty\)/i, @pool_data ) ) {
            $pool_data[0] =~ /(Pool\d)\sspare\sdisks\s\(empty\)/;
            $key = to_key_name( item => $1 );
            $temp_pool_hash{$key} = [];
            push( @pool_info, \%temp_pool_hash );
        } else {
            if ( $pool_data[0] =~ /^Spare\sdisks/ ) {
                $no_pool = 1;
            } else {
                $pool_data[0] =~ /^(Pool\d)/;
                $key = lc($1);
                $temp_pool_hash{$key} = ();
            }

            # Remove the Pool sparse disks line, heading line and underline
            # line and the additional line coming in output which need not be
            # parsed
            splice( @pool_data, 0, 3 );
            my %spare_disks_hash;
            for ( my $count = 0; $count < @pool_data; $count++ ) {
                next if ( $pool_data[$count] =~ /^Spare\sdisks\sfor/ );
                my %disk  = ();
                my $index = 0;
                if ( $pool_data[$count] =~ s/\((.*)\)$// ) {
                    $disk{"is_zeroed"} = $1;
                } else {
                    $disk{"is_zeroed"} = "";
                }
                my @values = split( /\s+/, $pool_data[$count] );
                if ( $#heading != $#values ) {
                     splice( @values, 5, 0, "-" );
                }

                foreach (@values) {
                    $disk{ $heading[$index] } = $_;
                    $index++;
                }
                if ($no_pool) {
                    push( @pool_info, \%disk );
                } else {
                    push( @{ $temp_pool_hash{$key} }, \%disk );
                }

            } ## end for ( my $count = 0; $count...
            push( @pool_info, \%temp_pool_hash ) unless ($no_pool);
        } ## end else [ if ( grep( /Spare\sdisks\s\(empty\)/i...
    } ## end foreach my $array_ref (@pool_array)
    return \@pool_info;
} ## end sub _get_spare_disks

sub _get_vol_parsed {
    my %args        = @_;
    my $all_vol_ref = $args{"volume_ref"};
    my $response    = $args{"response_object"};
    my @vols        = @$all_vol_ref;
    my @vol_output;
    my ( $temp, $current_plex, $current_raid, @headings );

    foreach my $vol_ref (@vols) {
        my %each_vol = ();
        my @vol_data = @$vol_ref;
        my %disk_hash;

        # Process first line
        my $first_line = shift(@vol_data);
        $first_line =~ /(\w+)\s(\w+)\s\((.*)\)\s\((.*)\)/;
        my $key_name = $1;
        $key_name = to_key_name( item => $key_name );
        $each_vol{$key_name} = $2;
        my $temp  = $3;
        my $temp1 = $4;
        my @words = split( /,\s/, $temp );
        $each_vol{"state"} = shift(@words);
        $each_vol{"status"} = join( ", ", @words );
        $temp1 =~ /(.*)\schecksums/;
        $each_vol{"checksums"} = $1;
        my $plex_hash_ref = {};
        my $raid_hash_ref = {};

    NEXT: while ( my $line = shift(@vol_data) ) {
            if ( $line =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/ ) {
                my $heading_line = $line;
                @headings = split( /\s+/, $heading_line );
                for ( my $i = 0; $i < scalar(@headings); $i++ ) {
                    if ( $headings[$i] =~ /^\(/ ) {
                        $headings[ $i - 1 ] .= " " . $headings[$i];
                        splice( @headings, $i, 1 );
                        $i--;
                    }
                } ## end for ( my $i = 0; $i < scalar...
                if ( scalar(@headings) == 12 ) {
                    my $first_element = shift(@headings);
                    $headings[0] = $first_element . " " . $headings[0];
                }
                foreach my $temp_heading (@headings) {
                    $temp_heading = to_key_name( "item" => $temp_heading );
                }

                #skip the ---- line
                shift(@vol_data);
            } elsif ($line =~ /(RAID\sDisk)\s+(Device)\s+(Model\s+Number)\s+(Serial\s+Number)\s+(VBN\s+Start)\s+(VBN\s+End)/ ) {
                @headings = ($1,$2,$3,$4,$5,$6);
                foreach my $temp_heading (@headings) {
                    $temp_heading = to_key_name( "item" => $temp_heading );
                }
                shift(@vol_data);
            } elsif ( $line =~ /([Pp]lex|RAID)/ ) {
                if ( $line =~ /Plex\s+(.*)\s+\((.*)\)/ ) {
                    $plex_hash_ref = {}
                        if exists( $plex_hash_ref->{"plex_name"} );
                    $plex_hash_ref->{"plex_name"} = $1;
                    $plex_hash_ref->{"status"}    = $2;
                    push( @{ $each_vol{"plex"} }, $plex_hash_ref );
                } else {
                    $line =~ /RAID\sgroup\s(.*)\s\((.*)\)/;
                    $raid_hash_ref = {}
                        if ( ( exists( $raid_hash_ref->{"raid_group_name"} ) )
                        || ( exists( $raid_hash_ref->{"info"} ) ) );
                    $raid_hash_ref->{"raid_group_name"} = $1;
                    $raid_hash_ref->{"status"}          = $2;
                    push(
                        @{ $plex_hash_ref->{"raid_group"} },
                        $raid_hash_ref
                    );
                } ## end else [ if ( $line =~ /Plex\s+(.*)\s+\((.*)\)/)
            } elsif ( $line
                =~ /(Targeted to traditional volume or aggregate but not yet assigned to a raid group)|(Unassimilated .+ disks)/
                ) {
                $raid_hash_ref = {};
                $raid_hash_ref->{"info"} = $line;
                push( @{ $plex_hash_ref->{"raid_group"} }, $raid_hash_ref );
            } else {
                do {
                    if (( $line =~ /[Pp]lex|RAID/ )
                        || ( $line
                            =~ /(Targeted to traditional volume or aggregate but not yet assigned to a raid group)|(Unassimilated .+ disks)/
                        )
                        ) {
                        unshift( @vol_data, $line );
                        next NEXT;
                    } ## end if ( ( $line =~ /[Pp]lex|RAID/...

                    if ( $line =~ /Raid group is missing (.*) disk/ ) {
                        $raid_hash_ref->{"no_of_missing_disks"} = $1;
                        next;
                    }

                    next
                        if ( $line =~ /FAILED/ )
                        ; # For failed disks. This information is not required

                    my @values = split( /\s+/, $line );
                    if ( scalar(@values) != scalar(@headings) ) {
                        for ( my $i = 0; $i < scalar(@values); $i++ ) {
                            if ( $values[$i] =~ /^\(/ ) {
                                my $j = 0;
                                do {
                                    $values[ $i - 1 ]
                                        .= " " . $values[ $i + $j ];
                                } while ( $values[ $i + $j++ ] !~ /\)$/ );
                                splice( @values, $i, $j );
                                $i -= $j;
                            } ## end if ( $values[$i] =~ /^\(/)
                        } ## end for ( my $i = 0; $i < scalar...
                        if ( ( scalar(@values) - scalar(@headings) ) == 1 ) {
                            my $first_element = shift(@values);
                            $values[0] = $first_element . " " . $values[0];
                        }
                        # On mcc-ip the channel remains blank throwing off the parser. This step adds in a -
                        # value[5] is the disk pool. BURT 1087841
                        if ( scalar(@headings) - scalar(@values) == 1 && scalar(@values) > 5 && $values[5] == 1) {
                            splice @values, 5, 0, '-';
                        }
                    } ## end if ( scalar(@values) !=...
                    if ( scalar(@values) != scalar(@headings) ) {
                        if ( $line !~ /LUN|Local\s+spares/ ) {
                            my $error
                                = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                            $error->throw(
                                text => "unexpected format of output",
                                response_object => $response
                            );
                        } ## end if ( $line !~ /LUN/ )

                    } ## end if ( scalar(@values) !=...
                    my $index = 0;
                    my %per_disk;
                    if ( $line =~ /LUN/
                        && scalar(@values) != scalar(@headings) ) {

          # Case to handle output for LUNs and the fields contain empty value.
          # For this case the "CHAN" column is always empty.
                        foreach my $current_value (@values) {
                            $per_disk{ $headings[ $index++ ] }
                                = $current_value;
                            $per_disk{ $headings[ $index++ ] } = ""
                                if ( $headings[$index] =~ "chan" );
                        } ## end foreach my $current_value (...
                    } else {
                        foreach my $current_value (@values) {
                            $per_disk{ $headings[ $index++ ] }
                                = $current_value;
                        }
                    } ## end else [ if ( $line =~ /LUN/ &&...
                    push( @{ $raid_hash_ref->{"disks"} }, \%per_disk );
                } while ( $line = shift(@vol_data) );
            } ## end else [ if ( $line =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/)
        } ## end while ( my $line = shift(...
        push( @vol_output, \%each_vol );
    } ## end foreach my $vol_ref (@vols)
    return \@vol_output;
} ## end sub _get_vol_parsed

sub _get_broken_or_partner_disks {
    my %args          = @_;
    my $disk_data_ref = $args{"broken_or_partner_disks_ref"};
    my @disk_data     = @$disk_data_ref;
    my @headings;
    my @disk_output;

   #    #Remove the first line which either says Broken disks or Partner disks
    if ( $disk_data[0] =~ /^(Broken disks|Maintenance disks)$/ ) {
        shift(@disk_data);
    }
    for ( my $line = 0; $line < @disk_data; $line++ ) {
        next if ( $disk_data[$line] eq "" );
        if ( $disk_data[$line] =~ /RAID\s+Disk\s+Device\s+HA\s+SHELF\s+BAY/ )
        {
            @headings = (
                "raid_disk", "device", "ha", "shelf", "bay", "chan", "pool",
                "type", "rpm", "used(MB/blks)", "phys(MB/blks)"
            );

            # Remove the heading and underline lines
            splice( @disk_data, $line, 2 );
            $line--;
            next;
        } ## end if ( $disk_data[$line]...
        my @values = split( /\s+/, $disk_data[$line] );
        if ( $#headings != $#values ) {
              splice( @values, 5, 0, "-" );
        }
        if ( scalar(@values) - scalar(@headings) == 1 ) {

       #Takes care of case where raid_disk name is something like admin_failed
            my $first_value = shift(@values);
            $values[0] = $first_value . " " . $values[0];
        } ## end if ( scalar(@values) -...
        my $index = 0;
        my %per_disk;
        foreach (@values) {
            $per_disk{ $headings[ $index++ ] } = $_;
        }
        push( @disk_output, \%per_disk );
    } ## end for ( my $line = 0; $line...
    return \@disk_output;
} ## end sub _get_broken_or_partner_disks

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error;

    if ( $raw_output
        =~ /(No aggregate named .+ was found)|(aggr options: percent_snapshot_space must be between 0 and 100)|(No aggregate named .+ exists)|(must be between)/
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
        return $error;
    } elsif ( $raw_output
        =~ /([Uu]sage:\naggr\s+split\s+(.*))|(Unrecognized option -s)/ ) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
        return $error;
    } elsif ( $raw_output
        =~ /(64-bit upgrade is not currently in progress .+ its contained volumes)|(Invalid operation on 64-bit format\s+(volume|aggregate) '.+'.)|(Cannot mirror aggregate .+ because the aggregate snapshot reserve could not be increased)|(Couldn't mirror volume: Disk '.+' does not match type of other disks)|(Couldn't mirror volume: Disk .+ does not exist)
           |(Couldn't mirror volume: Operation requires a syncmirror_local license.)|(Aggregate '.+' is already mirrored.)|(Couldn't mirror volume: Not enough disks specified.)|(plex .* cannot be taken offline)/s
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
        return $error;
    } elsif ( $raw_output
        =~ /aggr wafliron stop.*wafliron is not currently active|copy throttle \[operation_number\] throttle_value|aggr split:\s+(\'.*\' is not mirrored.|Destination aggregate .* already exists.|Invalid operation on offline volume .*|Plex to split does not contain a valid filesystem.|That is the only good plex and therefore can't be removed.)|aggr.*Wafliron is finishing. Abort ignored.*|[Ss]tarting a new raid(?:4|_dp) RAID group/
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
        return $error;
    } elsif ( $raw_output
        =~ /Restrict it before copying to it|Neither aggregate local|Could not connect to filer|Cannot Init Input, aborting|Read Socket received EOF|Aggregate is not active|Cannot retreive the IPv6 address for the filer|Unknown resolver error|Interrupted|Destination aggregate.+is too small|No operations in progress|copy abort \[ all \| operation_number\]/
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
        return $error;
    } elsif ( $raw_output =~ /Missing aggregate name|Must specify aggregate name/ ) {
        $$error_message = $raw_output;
        $error          = 'MissingRequiredParam';
        return $error;
    } elsif ( $raw_output =~ /Must specify value/i ) {
        $$error_message = $raw_output;
        $error          = 'MissingValue';
        return $error;
    } elsif ( $raw_output
        =~ /option "-p inet6" is given with IPv4 address|Illegal session value|Illegal throttle value|Aggregate doesn't exist/
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
        return $error;
    } elsif ( $raw_output =~ /(This operation .+ admin or advanced mode)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
        return $error;
    } elsif ( $raw_output
        =~ /aggr options:\s+(Operation not permitted on 32-bit aggregates)/ )
    {
        $$error_message = $1;
        $error          = "CommandFailed";
        return $error;
    }
    if ( $raw_output =~ /$command(?:[^:]*):(.*\n.*)/ ) {
        $$error_message = $1;
        if ( $$error_message
            =~ /(No aggregate name supplied)|(Missing)|(required after)|(Two aggregate names required)|(Neither a count)/
            ) {
            $error = "MissingRequiredParam";
        } elsif ( $$error_message
            =~ /([Ee]xtra arguments)|([Uu]nrecognized)|(is a volume)|(used only when)/
            ) {
            $error = "InvalidParam";
        } elsif ( $$error_message
            =~ /([Ii]nvalid)|(No aggregate named)|(cannot be destroyed)|(Value must be)|(not a legal)/
            ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message
            =~ /(already)|([cC]annot)|(Operation requires a)|(atleast)|(but only)|([Cc]ouldn't create)/
            ) {
            $error = "CommandFailed";
        } elsif ( $$error_message
            =~ /(No aggregate .* exists|must be greater than|must be a multiple)/
            ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message
            =~ /(wafliron is currently active|but no matching disks are available|Unable to)/
            ) {
            $error = "CommandFailed";
        } elsif (
            $$error_message =~ /(Aggregate is in use by other operations)/ ) {
            $error = "CommandFailed";
        } elsif ( $$error_message
            =~ /Please offline the aggregate .* before running wafliron in optional commit mode/
            ) {
            $error = "CommandFailed";
        } elsif (
            (   $$error_message
                =~ /prev_cp can only be run in optional commit mode/
            )
            || ( $$error_message
                =~ /Please offline the Aggregate .* before running prev_cp/ )
            || ( $$error_message
                =~ /.+get a volume reference.+volume is unavailable/ )
            || ( $$error_message
                =~ /The specified operation could not be completed as the aggregate is currently busy/
            )
            ) {
            $error = "CommandFailed";
        } elsif ( $$error_message
            =~ /Disk type does not match type of disks in the aggregate./i ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message
            =~ /No aggregate or traditional volume named .* exists./i ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message
            =~ /Neither a count of disks nor a list of disks to add.*/i ) {
            $error = "InvalidParamValue";
        } elsif (
            $$error_message =~ /\'\w+\' is a volume; use 'vol read_fsid'/i ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message =~ /No aggregate named .+ exists/i ) {
            $error = "InvalidParamValue";
        } elsif ($raw_output =~ /Use "snaplock clock initialize" to set Compliance Clock/) {
            $error = 'CommandFailed';
            $error_message = $raw_output;
        }
        
        return $error;
    } elsif ( $raw_output =~ /([Uu]sage:)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to dependencies or wrong usage\n$raw_output";
        return $error;
    }
    if ( $raw_output
        =~ /Cannot mirror aggregate (.*) because the aggregate snapshot reserve could not be increased/
        ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
        return $error;
    } elsif ( $raw_output
        =~ /Couldn't mirror volume: There are not enough spare disks in the pool/
        ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
        return $error;
    } elsif ( $raw_output =~ /aggr wafliron: No aggregate.*/){
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
        return $error;
    } 
    return undef;
} ## end sub _check_error
## @name ::aggr_wafliron_status
##
## @summary Parses the raw output from the "aggr wafliron status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_status(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'status-for-flexvols' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr wafliron status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'aggrtry'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'component_test_aggr'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'volnew'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'aggr0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'component_mirrored_aggr'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'mount_phase_info' => [
##&nbsp;                                    {
##&nbsp;                                      'snap_inofiles' => '0ms',
##&nbsp;                                      'activemap' => '200ms',
##&nbsp;                                      'snapmaps' => '0ms',
##&nbsp;                                      'snapdir' => '29ms',
##&nbsp;                                      'metadir' => '55ms',
##&nbsp;                                      'flex_vols' => '0ms',
##&nbsp;                                      'rootdir_for_iron' => '27ms',
##&nbsp;                                      'refcnt' => '0ms',
##&nbsp;                                      'rootdir' => '20ms',
##&nbsp;                                      'summary_map' => '0ms',
##&nbsp;                                      'snap_selfcover' => '0ms'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'status' => 'Cleaning up',
##&nbsp;            'status_file_info' => [
##&nbsp;                                    {
##&nbsp;                                      'blks_total' => '104',
##&nbsp;                                      'volume' => 'vol0',
##&nbsp;                                      'files' => '6',
##&nbsp;                                      'blks_used' => '24'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'aggregate' => 'try'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = aggr wafliron status try
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status_of_previous_attempt' => 'Successful',
##&nbsp;            'status' => 'not currently active',
##&nbsp;            'aggregate' => 'try'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub aggr_wafliron_status {
    my %args = @_;
    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $can_throw = 0;

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr wafliron",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines
        = get_lines( 'raw_output' => $output, 'can_throw' => $can_throw );

    return (
        _aggr_wafliron_status(
            "lines"           => \@lines,
            "response_object" => $resp
        )
    );
} ## end sub aggr_wafliron_status

## @name ::aggr_wafliron_review
##
## @summary Parses the raw output from the "aggr wafliron review" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_review with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_review(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_review' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp; Command = aggr wafliron review dst_aggr
##&nbsp; $VAR1 = [
##&nbsp;          {
##&nbsp;            'change_list' => [
##&nbsp;                               'Adjusting blocks used from 5678 to 65721',
##&nbsp;                               'Total blocks used corrected.'
##&nbsp;                             ],
##&nbsp;            'volume' => 'testvol_second'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'change_list' => [
##&nbsp;                              'Adjusting blocks used from 5678 to 65694',
##&nbsp;                               'Total blocks used corrected.'
##&nbsp;                             ],
##&nbsp;            'volume' => 'testvol_third'
##&nbsp;          }
##&nbsp;        ];
##&nbsp; Command = aggr wafliron review src_aggr
##&nbsp; $VAR1 = [];

sub aggr_wafliron_review {
    my %args = @_;
    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $can_throw = 0;

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr wafliron",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines
        = get_lines( 'raw_output' => $output, 'can_throw' => $can_throw );

    return (
        _aggr_wafliron_review(
            "lines"           => \@lines,
            "response_object" => $resp
        )
    );
} ## end sub aggr_wafliron_review

sub _aggr_wafliron_review {
    my %args  = @_;
    my @lines = sort ( @{ $args{"lines"} } );
    my $resp  = $args{"response_object"};
    my @result_array;
    my $line;
    my $aggregate_name;
    my $i = -1;
    while ( $line = shift(@lines) ) {

        if ( $line =~ /:/ ) {
            my ( $temp_vol, $temp_change ) = split( /\s*:\s*/, $line );
            $i++
                if ( ( $i < 0 )
                || ( $result_array[$i]->{'volume'} ne $temp_vol ) );
            push( @{ $result_array[$i]->{'change_list'} }, $temp_change );
            $result_array[$i]->{'volume'} = $temp_vol
                if ( $result_array[$i]->{'volume'} ne $temp_vol );
        }
    }
    return ( \@result_array );
} ## end sub _aggr_wafliron_review

sub _aggr_wafliron_status {
    my %args  = @_;
    my @lines = @{ $args{"lines"} };
    my $resp  = $args{"response_object"};
    my @result_array;
    my @unparsed_lines;
    my %info_hash;
    my $line;
    my $aggregate_name;

    while ( $line = shift(@lines) ) {
        if ( $line =~ /:\s+no\s+waflirons\s+in\s+progress/ ) {
            my $error
                = 'NACL::APISet::Exceptions::NoMatchingEntriesException';
            $error->throw( text => " $line", response_object => $resp );
        } elsif ( $line =~ /^Status\sfiles\sredirected\sto\svolume\s(.*)/ ) {

            #Information regarding status files
            my %hash;
            my @temp_array;
            $hash{'volume'} = $1;
            $line = shift(@lines);
            my @fields = split( /,\s*/, $line );
            foreach my $field (@fields) {
                my ( $value, $key ) = split( /\s+/, $field, 2 );
                trim( 'string_ptr' => \$key );
                $key = to_key_name( 'item' => $key );
                $hash{$key} = $value;
            } ## end foreach my $field (@fields)
            push( @temp_array, \%hash );
            $info_hash{'status_file_info'} = \@temp_array;
        } elsif ( $line =~ /^(.*) mount phase of aggregate .* took (.*)\.$/ )
        {
            my %hash;
            my $i     = 0;
            my $key   = $1;
            my $value = $2;
            trim( 'string_ptr' => \$key );
            $key = to_key_name( 'item' => $key );
            $line = shift(@lines);
            while ( $line =~ /^(.*) mount phase of aggregate .* took (.*)\./ )
            {
                $i++;
                my $temp_key = $1;
                trim( 'string_ptr' => \$temp_key );
                $temp_key = to_key_name( 'item' => $temp_key );
                $hash{$temp_key} = $2;
                $line = shift(@lines);
            } ## end while ( $line =~ ...
            unshift( @lines, $line );
            if ($i) {
                $hash{$key} = $value;
                my @temp_array;
                push( @temp_array, \%hash );
                $info_hash{'mount_phase_info'} = \@temp_array;
            } else {
                $info_hash{$key} = $value;
            }
        } elsif ( $line =~ /Wafliron\sscan\sstats\sfor\saggregate:\s(.*)$/ ) {

            #Information regarding scan stats
            my %hash;
            my @temp_array;
            $hash{'aggregate'} = $1;
            $line = shift(@lines);
            while ( $line !~ /:/ ) {
                if ( $line =~ /^(\d*)\s(.*)\sdone$/ ) {
                    my $key   = $2 . " done";
                    my $value = $1;
                    trim( 'string_ptr' => \$key );
                    $key = to_key_name( 'item' => $key );
                    $hash{$key} = $value;
                } elsif ( $line =~ /Scanning\s(.*)\sdone/ ) {
                    $hash{'scanning'} = $1;
                } elsif ( $line =~ /Summary\s+scan\s+(.*)\s+done/ ) {
                    $hash{'summary_scan'} = $1;
                }
                $line = shift(@lines);
            } ## end while ( $line !~ /:/ )
            unshift( @lines, $line );
            push( @temp_array, \%hash );
            $info_hash{'scan_stats_info'} = \@temp_array;
        } elsif (
            $line =~ /Wafliron\sis\sactive\son\s(?:aggregate|volume)\s(.+)\s:\s(.*)$/ ) {
            $info_hash{'aggregate'} = $1;
            $info_hash{'status'}    = $2;
        } elsif ( $line
            =~ /Mounting\sfinished\ssteps\s(\d+\sof\s\d+)\s.*\smount\sphase/ )
        {
            $info_hash{'steps_finished'} = $1;
        } elsif ( $line
            =~ /Status\sof\sthe\sprevious\sattempt\sto\sstart\swafliron\son\s(.+):\s(.*)\./
            ) {
            $aggregate_name                          = $1;
            $info_hash{'aggregate'}                  = $aggregate_name;
            $info_hash{'status_of_previous_attempt'} = $2;
        } elsif (
            $line =~ /(\S+)\s*:\s*wafliron\s+is\s+(not\s+currently\s+active)/
            || $line
            =~ /aggr\s+wafliron\s+status\s+(.+)\s*:\s*wafliron\s+in\s+(optional\s+commit\s+mode\s+has\s+completed)/
            ) {
            if ( $aggregate_name ne $1 ) {
                my %aggr_info = ( "aggregate" => $1, "status" => $2 );
                push @result_array, \%aggr_info;
            } else {
                $info_hash{'status'} = $2;
            }
        } elsif ( $line =~ /Optional commit mode is (.+)/ ) {
            $info_hash{'optional_commit_mode'} = $1;
        } else {
            push @unparsed_lines, $line;
        }
    } ## end while ( $line = shift(@lines...
    if ( ( scalar @unparsed_lines > 0 ) && ( scalar keys(%info_hash) == 0 ) )
    {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $resp
        );
    } ## end if ( ( scalar @unparsed_lines...
    $info_hash{unparsed_lines} = \@unparsed_lines
        if ( scalar @unparsed_lines );
    push( @result_array, \%info_hash ) if ( scalar keys(%info_hash) );
    return ( \@result_array );
} ## end sub _aggr_wafliron_status
## @name ::aggr_wafliron_start
##
## @summary Parses the raw output from the "aggr wafliron start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_start(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume-to-iron-first' </td> </tr>
## <tr> <td> '-t' </td> <td> 'triage' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-c' </td> <td> 'check-compressed-data-blocks' </td> </tr>
## <tr> <td> '-o' </td> <td> 'optional-commit' </td> </tr>
## <tr> <td> '-prev_cp' </td> <td> 'prev_cp' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inject-errors' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-backup' </td> <td> 'backup' </td> </tr>
## <tr> <td> '-use_backup' </td> <td> 'use-backup' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_wafliron_start {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $first_line = "aggr wafliron start";
    my $can_throw  = 0;

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr wafliron start",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    my $line = shift(@lines);
    if ( $line =~ /aggr wafliron start (.*): \./ ) {
        my %hash;
        my @result_array;
        $hash{$1} = "aggr wafliron started";
        push( @result_array, \%hash );
        return ( \@result_array );
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    } ## end else [ if ( $line =~ /aggr wafliron start (.*): \./)
} ## end sub aggr_wafliron_start
## @name ::aggr_wafliron_stop
##
## @summary Parses the raw output from the "aggr wafliron stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_stop with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_stop(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_wafliron_stop {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $first_line = "aggr wafliron stop";
    my $can_throw  = 0;
    my $error      = _check_error(
        raw_output        => $output,
        command           => "aggr wafliron stop",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) ) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    } else {
        my %hash;
        my @result_array;
        $hash{'result'} = "aggr wafliron stopped";
        push( @result_array, \%hash );
        return ( \@result_array );
    } ## end else [ if ( scalar(@lines) )
} ## end sub aggr_wafliron_stop

## @name ::aggr_create
##
## @summary Parses the raw output from the "aggr create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_create with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_create(%args);
## INPUTS TO METHOD CALL 'aggr_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'checksum-style' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'disk-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'trad-volume' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'rpm' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'language-code' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'is-mirrored' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'raid-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'mirror-disks' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'snaplock-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'block-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'raid-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force-spare-pool' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_create {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr create",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $regex
        = 'create\s+([^\s]+)\s+(-f\s+)?(-L\s+(\w+\s+)?)?(-B\s+(\d+)\s+)?'
        . '(-m\s+)?(-r\s+(\w+)\s+)?(-R\s+(\w+)\s+)?(-T\s+(\w+)\s+)?'
        . '(-t\s+(\w+)\s+)?(-v\s+)?(-l\s+(\w+)\s+)?-d\s+([^-\r\n]+)'
        . '(-d\s+([^\r\n]+))?';

    if ( $output =~ /$regex/ ) {    # for -n option
        my %hash;
        $hash{'aggregate'}     = $1;
        $hash{'force'}         = 1 if ($2);
        $hash{'snaplock-type'} = $4 ? trim_string( item => $4 ) : "" if ($3);
        $hash{'block-type'}    = $6 if ($5);
        $hash{'is-mirrored'}   = 1 if ($7);
        $hash{'raid-size'}     = $9 if ($8);
        $hash{'rpm'}           = $11 if ($10);
        $hash{'disk-type'}     = $13 if ($12);
        $hash{'raid-type'}     = $15 if ($14);
        $hash{'language-code'} = $18 if $17;
        my @disks = split( /\s+/, $19 );

        if ($20) {
            my @mirror_disks = split( /\s+/, $21 );
            $hash{'mirror-disks'} = \@mirror_disks;
        }
        $hash{'disks'} = \@disks;
        $output =~ /(aggr create\s.*-d[^\r\n]*)/;
        $hash{'command'} = $1;
        @result_array = ( \%hash );
    } elsif ( $output
        =~ /Creation\sof\san\saggregate\swith\s(\d+)\sdisks\shas\s(.*)\./ ) {
        @result_array = ( { 'result' => $2, 'disks' => $1 } );
    } elsif ( $output
        =~ /Creation\sof\san\saggregate\shas\s(.*)\.\s+(\d+)\sdisks\sneed\sto\sbe\szeroed/
        ) {
        @result_array = ( { 'result' => $1, 'disks' => $2 } );
    } elsif ( $output
        =~ /[Aa]ggregate has been created with (\d+) .* added to the aggregate.*(\d+).* need/
        ) {
        @result_array = (
            {   'result'          => 'initiated',
                'disks-completed' => $1,
                'disks-remaining' => $2
            }
        );
    } elsif ( $output
        =~ /Creation of (\s*an\s*)?aggregate .*has been (.*). (\d+) disks? needs? to be zeroed/
        ) {
        @result_array = ( { 'result' => $2, 'disks' => $3 } );
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    } ## end else [ if ( $output =~ /$regex/)
    return \@result_array;
} ## end sub aggr_create

## @name ::aggr_rename
##
## @summary Parses the raw output from the "aggr rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_rename with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_rename(%args);
## INPUTS TO METHOD CALL 'aggr_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-aggregate-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_rename {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output        => $output,
        command           => "aggr rename",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /\'(.*)\'\srenamed\sto\s\'(.*)\'/ ) {
        @result_array
            = ( { 'oldname' => $1, 'result' => 'renamed', 'newname' => $2 } );
        return \@result_array;
    }

    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_rename

## @name ::aggr_destroy
##
## @summary Parses the raw output from the "aggr destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_destroy with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_destroy(%args);
## INPUTS TO METHOD CALL 'aggr_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_destroy {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr destroy",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /.*\s\'(.*)\'\sdestroyed/ ) {
        @result_array = ( { 'name' => $1, 'result' => 'destroyed' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_destroy

## @name ::aggr_undestroy
##
## @summary Parses the raw output from the "aggr undestroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_undestroy with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_undestroy(%args);
## INPUTS TO METHOD CALL 'aggr_undestroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_undestroy {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr undestroy",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /.*\s\'(.*)\'\sundestroyed/ ) {
        @result_array = ( { 'name' => $1, 'result' => 'destroyed' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_undestroy

## @name ::aggr_offline
##
## @summary Parses the raw output from the "aggr offline" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_offline with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_offline(%args);
## INPUTS TO METHOD CALL 'aggr_offline' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'cifs-delay-seconds' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'aggr-with-vols' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_offline {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr offline",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /.*\s\'(.*)\'\s.*\snow\soffline/ ) {
        @result_array = ( { 'name' => $1, 'result' => 'offline' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_offline

## @name ::aggr_online
##
## @summary Parses the raw output from the "aggr online" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_online with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_online(%args);
## INPUTS TO METHOD CALL 'aggr_online' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_online {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr online",
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my %hash;

    if ( $output =~ /.*\s\'(.*)\'\sis now online/ ) {
        @result_array = ( { 'name' => $1, 'result' => 'online' } );
        return \@result_array;
    }
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_online

## @name ::aggr_options
##
## @summary Parses the raw output from the "aggr options" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_options with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_options(%args);
## INPUTS TO METHOD CALL 'aggr_options' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'option-value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'option-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_options {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr options",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    $output =~ s/\r|\n//g;

    if (( $command =~ /aggr options\s+[^\s]+\s+[^\s]+\s+[^\s]+\s*/ )
        && !snip_only_required_output(
            raw_output => $output,
            delimiter  => ',.*,'
        )
        ) {
        return [];
    } ## end if ( ( $command =~ /aggr options\s+[^\s]+\s+[^\s]+\s+[^\s]+\s*/...

    my @items = split( /,\s*/, $output );
    my %hash;

    foreach my $item (@items) {
        if ( $item !~ /=/ ) {
            $hash{$item} = '';
            next;
        }
        my ( $name, $value ) = split( /=/, $item );
        $hash{$name} = $value;
    } ## end foreach my $item (@items)

    @result_array = ( \%hash );
    return \@result_array;
} ## end sub aggr_options

## @name ::_is_pushable
##
## @summary Determines whether a new output section has begun
##
## @description
##    This method is used to find out if a new section of status output has
##    begun for aggr (scrub, media_scrub or verify) status commands
##
## @synopsis
##    is_pushable(%args);
##
## @arg %args Required: The current line to be checked
## line => a line of output
##
## @return Returns 1 or undef depending on whether a new section should begin

sub _is_pushable {
    my %args = @_;
    my $line = $args{line};
    return 1 if ( $line =~ /aggr ((media_)?scrub|verify)/ );
    return undef;
} ## end sub _is_pushable

## @name ::_generic_parser_status
##
## @summary Parses the raw output for  "aggr media_scrub status,
##    aggr scrub status and aggr verify status" commands
##
## @description
##    This method is a generic parser for "aggr media_scrub status,
##    aggr scrub status and aggr verify status" commands
##
## @synopsis
##    _generic_parser_status(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array which contains the parsed output data. This
## array will contain one hash for each section of status output.

sub _generic_parser_status {  #for aggr (scrub, media_scrub and verify) status
    my %args = @_;

    my $error_line_expected = '(?:aggr (media_)?scrub status|aggr verify)';

    my $error_message;
    my @result_array = ();
    my $resp         = $args{response_object};
    my $output       = $resp->get_processed_output();
    my $command      = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $error_line_expected,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $is_can_throw = 1;

    # 'aggr verify status' command sometimes returns no output.
    # This is observed especially when the mirror is degraded.
    # component_flex_aggr online          raid4, aggr
    #                           degraded
    #                           mirrored
    #                            64-bit
    # my-vsim1> aggr verify status component_flex_aggr
    # my-vsim1>
    $is_can_throw = 0 if ( $command =~ /aggr verify status/ );

    my $first_line = '(aggr (media_)?scrub.*|aggr verify .*)';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $is_can_throw,
        response_object => $resp
    );

    my ( %hash, $verbose );

    if ( $command =~ /\s*-v\s*/ ) {
        $verbose = 1;
    }

    foreach my $line (@lines) {
        if ( _is_pushable( line => $line ) ) {
            my %hash2 = %hash;
            push( @result_array, \%hash2 ) if keys(%hash2);
            if ($verbose) {
                if ( $line =~ /status of (.*) :/ ) {
                    %hash = ( 'name' => $1 );
                }
            } elsif ( $line =~ /aggr verify ([^ ]*) .*is (.*)/ ) {
                %hash = ( 'name' => $1, 'status' => $2 );
                $hash{'status'} =~ s/\.//g;
                if ( $hash{'status'} =~ /(\d+)\%/ ) {
                    $hash{'percentage_completed'} = $1;
                }
                if ( $hash{'status'} =~ /suspended/i ) {
                    $hash{'is_suspended'} = 'true';
                } else {
                    $hash{'is_suspended'} = 'false';
                }
            } elsif ( $line =~ /scrub ([^ ]*).*is (.*)/ ) {
                %hash = ( 'name' => $1, 'status' => $2 );
                $hash{'status'} =~ s/\.//g;
            } elsif ( ( $line =~ /scrub ([^ ]*)\s*:\s*(is\s*)?(.*)/ ) ) {
                %hash = ( 'name' => $1, 'status' => $3 );
                $hash{'status'} =~ s/\.//g;
            }
        } else {    #other verbose mode lines
            if ( $line =~ /Current .* is (.*) complete/ ) {    # %complete
                $hash{'current_scrub'} = $1;
            } elsif ( $line =~ /Media scrub ([^\.]*)/ ) { #started / suspended
                $hash{'state'} = $1;
            } elsif ( $line =~ /Last full .*scrub completed: (.*)/ )
            {                                             #last complete
                $hash{'last_full_scrub_completed'} = $1;
            } elsif ( $line =~ /Scrub is not active/ ) {    # if not active
                $hash{'current_scrub'} = 'not active';
            }
        } ## end else [ if ( _is_pushable( line...
    } ## end foreach my $line (@lines)
    push( @result_array, \%hash ) if keys(%hash);           #last entry
    return \@result_array;
} ## end sub _generic_parser_status

## @name ::aggr_media_scrub_status
##
## @summary Parses the raw output from the "aggr media_scrub status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_media_scrub_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_media_scrub_status(%args);
## INPUTS TO METHOD CALL 'aggr_media_scrub_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'rg-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'disk-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_media_scrub_status {
    return ( _generic_parser_status(@_) );
}

## @name ::aggr_scrub_status
##
## @summary Parses the raw output from the "aggr scrub status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_scrub_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_scrub_status(%args);
## INPUTS TO METHOD CALL 'aggr_scrub_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_scrub_status {
    return ( _generic_parser_status(@_) );
}

## @name ::aggr_verify_scrub_status
##
## @summary Parses the raw output from "aggr verify status" command
##
## @description
##    This method can be used for parsing the output from "aggr verify status"
##    command called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/default/aggr.cdefs
##
## @synopsis
##    aggr_verify_status(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array reference which contains the parsed output data.
## This array will contain one hash for each section of status output.

## @name ::aggr_verify_status
##
## @summary Parses the raw output from the "aggr verify status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_verify_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_verify_status(%args);
## INPUTS TO METHOD CALL 'aggr_verify_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_verify_status {
    return ( _generic_parser_status(@_) );
}

## @name ::_generic_parser_scrub
##
## @summary Parses the raw output for all aggr scrub sub commands except
##    aggr scrub status
##
##
## @description
##    This method is a generic parser for aggr scrub start, aggr scrub stop,
##    aggr scrub resume, aggr scrub suspend
##
## @synopsis
##    _generic_parser_scrub(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array which contains the parsed output data. This
## array will contain one hash for each section of output.

sub _generic_parser_scrub {    #for scrub commands other than status
    my %args = @_;

    my $can_throw = 1;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my $error   = _check_error(
        raw_output        => $output,
        command           => 'aggr scrub (?:resume|start|stop|suspend)',
        ptr_error_message => \$error_message
    );
    if ($error) {

        if (   $command =~ /aggr scrub \w+\s*$/
            && $error eq 'CommandFailed'
            && ( $output =~ /already \w+/ || $output =~ /not currently \w+/ )
            ) {
            $error = undef;
        } else {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
        $can_throw = 0;
    } ## end if ($error)

    my $first_line = 'aggr scrub.*';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $can_throw,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line =~ /aggr scrub: (.*) scrub of (.*)/ ) {    #specific
            %hash = ( 'result' => $1, 'name' => $2 );
            push( @result_array, \%hash );
        } elsif ( $line =~ /aggr scrub .*: (.*) scrubbing in (.*)\./ ) {  #all
            %hash = ( 'result' => $1, 'items' => $2 );
            push( @result_array, \%hash );
        } elsif (
            $line =~ /aggr scrub .*: ([^ ]*) not currently being scrubbed/ ) {
            %hash = (
                'result' => 'not currently being scrubbed',
                'name'   => $1
            );
            $hash{'name'} =~ s/'//g;
            push( @result_array, \%hash );
        } ## end elsif ( $line =~ ...
    } ## end foreach my $line (@lines)

    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@result_array );
    }
    return \@result_array;
} ## end sub _generic_parser_scrub

## @name ::aggr_scrub_start
##
## @summary Parses the raw output from the "aggr scrub start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_scrub_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_scrub_start(%args);
## INPUTS TO METHOD CALL 'aggr_scrub_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_scrub_start {
    return ( _generic_parser_scrub(@_) );
}

## @name ::aggr_scrub_stop
##
## @summary Parses the raw output from the "aggr scrub stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_scrub_stop with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_scrub_stop(%args);
## INPUTS TO METHOD CALL 'aggr_scrub_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_scrub_stop {
    return ( _generic_parser_scrub(@_) );
}

## @name ::aggr_scrub_suspend
##
## @summary Parses the raw output from the "aggr scrub suspend" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_scrub_suspend with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_scrub_suspend(%args);
## INPUTS TO METHOD CALL 'aggr_scrub_suspend' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_scrub_suspend {
    return ( _generic_parser_scrub(@_) );
}

## @name ::aggr_scrub_resume
##
## @summary Parses the raw output from the "aggr scrub resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_scrub_resume with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_scrub_resume(%args);
## INPUTS TO METHOD CALL 'aggr_scrub_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_scrub_resume {
    return ( _generic_parser_scrub(@_) );
}

## @name ::_generic_parser_verify
##
## @summary Parses the raw output for all aggr verify sub commands except
##    aggr verify status
##
##
## @description
##    This method is a generic parser for aggr verify start, aggr verify stop,
##    aggr verify resume, aggr verify suspend
##
## @synopsis
##    _generic_parser_verify(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array which contains the parsed output data. This
## array will contain one hash for each section of output.

sub _generic_parser_verify {    #for aggr verify commands other than status
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => 'aggr verify',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }

    my $first_line = 'aggr verify .*';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ($line =~ /aggr verify .*: No verify running on aggregate (.*)\./ )
        {
            %hash = ( 'result' => 'not running', 'name' => $1 );
            $hash{'name'} =~ s/'//g;
            push( @result_array, \%hash );
        } elsif ( $line =~ /aggr verify .*: verify (.*) on (.*)\./ ) {
            %hash = ( 'result' => $1, 'items' => $2 );
            push( @result_array, \%hash );
        }
    } ## end foreach my $line (@lines)

    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@result_array );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub _generic_parser_verify

## @name ::aggr_verify_start
##
## @summary Parses the raw output from the "aggr verify start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_verify_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_verify_start(%args);
## INPUTS TO METHOD CALL 'aggr_verify_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'plex-number' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_verify_start {
    return ( _generic_parser_verify(@_) );
}

## @name ::aggr_verify_stop
##
## @summary Parses the raw output from the "aggr verify stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_verify_stop with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_verify_stop(%args);
## INPUTS TO METHOD CALL 'aggr_verify_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_verify_stop {
    return ( _generic_parser_verify(@_) );
}

## @name ::aggr_verify_resume
##
## @summary Parses the raw output from the "aggr verify resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_verify_resume with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_verify_resume(%args);
## INPUTS TO METHOD CALL 'aggr_verify_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_verify_resume {
    return ( _generic_parser_verify(@_) );
}

## @name ::aggr_verify_suspend
##
## @summary Parses the raw output from the "aggr verify suspend" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_verify_suspend with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_verify_suspend(%args);
## INPUTS TO METHOD CALL 'aggr_verify_suspend' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub aggr_verify_suspend {
    return ( _generic_parser_verify(@_) );
}

## @name ::aggr_restrict
##
## @summary Parses the raw output from the "aggr restrict" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_restrict with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_restrict(%args);
## INPUTS TO METHOD CALL 'aggr_restrict' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'cifs-delay-seconds' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'aggr-with-vols' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_restrict {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output        => $output,
        command           => "aggr restrict",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    if ( $output =~ /Aggregate '(.*)' is now restricted/ ) {
        @result_array = ( { 'name' => $1, 'result' => 'restricted' } );
        return \@result_array;
    }

    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "Output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_restrict

## @name ::aggr_add
##
## @summary Parses the raw output from the "aggr add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_add with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_add(%args);
## INPUTS TO METHOD CALL 'aggr_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'checksum-style' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'disk-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'raid-group' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> '-64bit-upgrade' </td> <td> '64bit-upgrade' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-count' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr add aggr0 -T FCAL 1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'initiated'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = aggr add hya_aggr -T FCAL 2
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'initiated'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub aggr_add {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output        => $output,
        command           => "aggr add",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $command =~ /-64bit-upgrade/ ) {
        return aggr_64bit_upgrade_start( response_object => $resp );
    }

    my $regex_check
        = 'The\s+disks?\s+needs?\s+to\s+be\s+zeroed\s+before\s+(it|they)\s+can'
        . '\s+be\s+added\s+to\s+the\s+aggregate\.\s+The\s+process\s+'
        . 'has\s+been\s+initiated';

    # for -n option
    ###########################-name-####force##########rg#######--disks-####
    if ( $output
        =~ /aggr add\s+([^\s]+)\s+(-f\s+)?(-g\s+(.*)\s+)?-d\s+([^\r\n]*)/ ) {
        my %hash = ( 'aggregate' => $1 );
        $hash{'force'}      = 1  if $2;
        $hash{'raid-group'} = $4 if $3;
        my @disks = split( /\s+/, $5 );
        $hash{'disks'} = \@disks;
        $output =~ /(aggr add\s.*-d[^\r\n]*)/;
        $hash{'command'} = $1;
        @result_array = ( \%hash );
    } elsif ( $output
        =~ /($regex_check)|(^\d+ disk(s)? \w+ to be zeroed before \w+ can be added to the aggregate.*)/
        ) {
        @result_array = ( { 'result' => 'initiated' } );
    } elsif ( $output
        =~ /Addition of (\d+) disk.* to the aggregate has been initiated/ ) {
        @result_array = ( { 'disks' => $1, 'result' => 'initiated' } );
    } elsif ( $output
        =~ /Addition of (\d+) disk.* to the aggregate has completed.*((\d+).* need)?/
        ) {
        my %hash;
        if ($2) {
            %hash = (
                'result'          => 'initiated',
                'disks-completed' => $1,
                'disks-remaining' => $3
            );
        } else {
            %hash = ( 'disks' => $1, 'result' => 'completed' );
        }
        @result_array = ( \%hash );
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    } ## end else [ if ( $output =~ ...

    return \@result_array;
} ## end sub aggr_add

## @name ::aggr_64bit_upgrade_start
##
## @summary Parses the raw output from the "aggr 64bit-upgrade start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_64bit_upgrade_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_64bit_upgrade_start(%args);
## INPUTS TO METHOD CALL 'aggr_64bit_upgrade_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-mode' </td> <td> 'mode' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##   aggr_64bit_upgrade_start(aggregate => api_new_aggr, mode => check);
##
##   Output:
##
##      [ { 'aggregate' => 'api_new_aggr' } ];
##
##      OR
##
##        [
##          {
##              'volume' => 'api_test_vol_2',
##              'used' => '1600KB',
##              'aggregate' => 'api_new_aggr',
##              'capacity' => '6%',
##              'total' => '25MB',
##              'available' => '23MB'
##          },
##          {
##              'volume' => 'api_test_vol_3',
##              'used' => '1.00GB',
##              'aggregate' => 'api_new_aggr',
##              'capacity' => '100%',
##              'total' => '1GB',
##              'available' => '0B',
##              'additional_space_required_for_upgrade' => '488KB',
##              'filesys_size_fixed' => 'true,
##          }
##        ];
##
##   aggr_64bit_upgrade_start(aggregate => api_new_aggr, mode => {normal | grow-reserved | grow-all});
##
##   Output:
##
##    [   {   'status'    => 'Started 64-bit upgrade',
##            'aggregate' => 'api_new_aggr'
##        }
##    ];

sub aggr_64bit_upgrade_start {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $command,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ((   $output
            =~ /.+To initiate the 64-bit upgrade of aggregate (.+), run .+/s
        )
        || ( $output
            =~ /.+To initiate the upgrade of aggregate (.+) to 64-bit, run .+/s
        )
        )    # Output of '... -mode check'
    {
        my $aggrName = $1;
        if ( $output
            =~ /Upgrading a volume to 64-bit consumes additional free space in the volume/
            ) {
            my @result_array;
            #######################################################################
       # Get the output from the headings line onwards, get the headings line
       # (this is the first line), and then split on two or more spaces
       # to get the heading names.
       # Since the output in CMode is the same as 7Mode except for there being
       # an additional "Vserver" field, we handle both in this parser. In
       # 7Mode the headings line starts with "Volume Name", in CMode it
       # starts with "Vserver"
            #######################################################################
            my ( $output_at_end, $output_from_header );
            my $output_from_header_till_end = snip_only_required_output(
                raw_output => $output,
                delimiter  => '(Volume Name|Vserver)'
            );

            # Split this output into two sections. The first section will
            # have a table and is expected always when -mode check is run.
            # while the second section may or may not be useful depending on
            # whether the volumes are 100% full or not.
            if ( $output_from_header_till_end =~ /((.|\s)+%)((.|\s)+)/ ) {
                $output_from_header = $1;
                $output_at_end      = $3;
            }

            # Parse the first section.
            my @header_lines = split /\r*\n+/, $output_from_header;
            my $header_line  = $header_lines[0];
            my @fields       = split /\s{2,}/, $header_line;

            #######################################################################
          # Get the substring from the last occurence of "--" till the "%" and
          # then split on one or more spaces to get the values
            #######################################################################
            my $index_of_last_hyphens = rindex $output_from_header, '--';
            $index_of_last_hyphens += 3;    # that is '--\n '
                                            #              ^ this location

            my $tempOutput = $output_from_header;
            while ( $tempOutput =~ /%/g ) {

                my $index_of_percent = index $tempOutput, '%';
                my $length_to_extract
                    = $index_of_percent - $index_of_last_hyphens + 1;
                my $string = substr $tempOutput, $index_of_last_hyphens,
                    $length_to_extract;
                $string =~ s/^\s+//g;

                my @values = split /\s+/, $string;

                # If the number of fields and values do not match, throw an
                # UnexpectedOutputException
                if ( @fields != @values ) {
                    my $text
                        = "Unexpected output format!\n "
                        . "Number of fields does not match number of values.\n"
                        . "Fields are:\n"
                        . Dumper( \@fields )
                        . "Values are:\n"
                        . Dumper( \@values );
                    NACL::APISet::Exceptions::UnexpectedOutputException
                        ->throw(
                        text            => $text,
                        response_object => $resp
                        );
                } ## end if ( @fields != @values)

                my %hash;
                my @tempFields = @fields;
                while (@tempFields) {
                    $hash{ to_key_name( item => shift @tempFields ) }
                        = shift @values;
                }

                #####################################################################
          # The earlier version of the parser stored the volume name in a key
          # called "volume", but the above logic would end up storing it under
          # a "volume_name" key. For backwards compatibility, we rename this
          # key to "volume".
                #####################################################################
                if ( exists $hash{volume_name} ) {
                    $hash{volume} = delete $hash{volume_name};
                }

                $hash{aggregate} = $aggrName;
                push @result_array, \%hash;

                my $next = $index_of_percent + 2;
                if ( $next >= length($tempOutput) ) {
                    last;
                } else {

                    $tempOutput = substr $tempOutput, $next;
                    $index_of_last_hyphens = 0;
                }
            } ## end while ( $tempOutput =~ /%/g)

            $output_at_end = snip_only_required_output(
                raw_output => $output_at_end,
                delimiter  => '(Volume Name|Vserver)'
            );

          # Only when one or more volumes are 100% full,we get this additional
          # useful data. Only if this output is available, proceed to parse
          # this part of the output.

            # Replace "see below" string with "dummy0B" string so that the
            # existing parser handles the special case listed in burt 502244
            $output_at_end =~ s/see below/dummy0B/g;

         # search for the last character of 'B'. The part till this is useful.
            if ( $output_at_end =~ /((.|\s)*\d+.?B)/ ) {
                $output_at_end = $1;
                my @header_lines          = split /\r*\n+/, $output_at_end;
                my $header_line           = $header_lines[0];
                my @fields                = split /\s{2,}/, $header_line;
                my $index_of_last_hyphens = rindex $output_at_end, '--';
                $index_of_last_hyphens += 3;    # that is '--\n '
                                                #              ^ this location

                my $temp_end_output = $output_at_end;
                while ( $temp_end_output =~ /B/g ) {
                    my $index_of_B = index $temp_end_output, 'B';
                    my $length_to_extract
                        = $index_of_B - $index_of_last_hyphens + 1;
                    my $string = substr $temp_end_output,
                        $index_of_last_hyphens,
                        $length_to_extract;
                    $string =~ s/^\s+//g;

                    my @values = split /\s+/, $string;

                   # If the number of fields and values do not match, throw an
                   # UnexpectedOutputException
                    if ( @fields != @values ) {
                        my $text
                            = "Unexpected output format!\n "
                            . "Number of fields does not match number of values.\n"
                            . "Fields are:\n"
                            . Dumper( \@fields )
                            . "Values are:\n"
                            . Dumper( \@values );
                        NACL::APISet::Exceptions::UnexpectedOutputException
                            ->throw(
                            text            => $text,
                            response_object => $resp
                            );
                    } ## end if ( @fields != @values)
                    my %hash;
                    my @tempFields = @fields;
                    while (@tempFields) {
                        $hash{ to_key_name( item => shift @tempFields ) }
                            = shift @values;
                    }

                    #####################################################################
          # The earlier version of the parser stored the volume name in a key
          # called "volume", but the above logic would end up storing it under
          # a "volume_name" key. For backwards compatibility, we rename this
          # key to "volume".
                    #####################################################################
                    if ( exists $hash{volume_name} ) {
                        $hash{volume} = delete $hash{volume_name};
                    }
                    my $filesys_value;
                    if ( $hash{volume} =~ /(.*)\((\*)\)(\(\*\*\))?/ ) {
                        $hash{volume} = $1;
                        $hash{"filesys_size_fixed"} = "true";
                        if ($3) {
                            $hash{"space_reserved_files"} = "true";
                            delete
                                $hash{additional_space_required_for_upgrade};
                        }
                    } ## end if ( $hash{volume} =~ ...
                    foreach my $result_hash (@result_array) {
                        if ( $result_hash->{volume} eq $hash{volume} ) {
                            if ( not exists $result_hash->{vserver} ) {
                                $result_hash = { %$result_hash, %hash };
                            } elsif (
                                $result_hash->{vserver} eq $hash{vserver} ) {
                                $result_hash = { %$result_hash, %hash };
                            }
                        } ## end if ( $result_hash->{volume...
                    } ## end foreach my $result_hash (@result_array)
                    my $next = $index_of_B + 2;
                    if ( $next >= length($temp_end_output) ) {
                        last;
                    } else {
                        $temp_end_output = substr $temp_end_output, $next;
                        $index_of_last_hyphens = 0;
                    }
                } ## end while ( $temp_end_output ...
            } ## end if ( $output_at_end =~...
            return \@result_array;
        } else {    # No flex vols in this aggr
            return [ { 'aggregate' => "$aggrName" } ];
        }
    } elsif (
        $output =~ /(Started 64-bit upgrade) on aggregate (.+) and all .+/
        ) # Output of 'vol 64bit-upgrade start aggrName [-mode {normal | grow-reserved | grow-all}]'
    {
        return [
            {   'aggregate' => "$2",
                'status'    => "$1"
            }
        ];
    } ## end elsif ( $output =~ ...
} ## end sub aggr_64bit_upgrade_start

## @name ::aggr_64bit_upgrade_status
##
## @summary Parses the raw output from the "aggr 64bit-upgrade status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_64bit_upgrade_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_64bit_upgrade_status(%args);
## INPUTS TO METHOD CALL 'aggr_64bit_upgrade_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-all' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##   aggr_64bit_upgrade_status(aggregate => api_new_aggr);
##
##   Output:
##
## [
##  {
##  'time_to_completion' => 'N/A',
##   'percent_completed' => '0',
##    'scanner_status' => 'running',
##     'format' => 'upgrading',
##      'volume' => 'api_new_aggr',
##       'progress' => 'fbn 0, inofile, public'
##  },
##  {
##  'scanner_status' => 'not running',
##   'format' => 'upgrading',
##    'volume' => 'api_test_vol_2'
##     },
##      {
##       'scanner_status' => 'not running',
##        'format' => 'upgrading',
## 'volume' => 'api_test_vol_3'
##  }
## ];

sub aggr_64bit_upgrade_status {
    my %args = @_;

    my ( $error_message, $aggrName, @lines, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $command,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output =~ /Volume\s+Format\s+Scanner Status.+/ ) {
        my $first_line
            = 'Volume\s+Format\s+Scanner Status\s+% Completed\s+Time to Completion\s+Progress';

        @lines = get_lines(
            raw_output      => $output,
            first_line      => $first_line,
            can_throw       => 1,
            response_object => $resp
        );

        shift @lines;    # Get rid of the headings

        foreach my $elem (@lines) {
            my %hash = ();
            my @tokens;
            @tokens = split ' ', $elem;

            $hash{'volume'} = $tokens[0];
            $hash{'format'} = $tokens[1];

            # Aggrs and vols that are still being upgraded
            if ( $tokens[1] !~ /64-bit/ ) {
                if ( $tokens[2] =~ /not/ && $tokens[3] =~ /running/ ) {
                    $hash{'scanner_status'} = 'not running';
                } else {
                    my $progress_string
                        = join( ' ', @tokens[ 5 .. $#tokens ] );

                    $hash{'scanner_status'}     = $tokens[2];
                    $hash{'percent_completed'}  = $tokens[3];
                    $hash{'time_to_completion'} = $tokens[4];
                    $hash{'progress'}           = "$progress_string";
                } ## end else [ if ( $tokens[2] =~ /not/...
            } ## end if ( $tokens[1] !~ /64-bit/)
            push @result_array, \%hash;
        } ## end foreach my $elem (@lines)
        return \@result_array;
    } else {
        return [];
    }
} ## end sub aggr_64bit_upgrade_status

## @name ::aggr_64bit_upgrade_scan_start
##
## @summary Parses the raw output from the "aggr 64bit-upgrade scan-start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_64bit_upgrade_scan_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_64bit_upgrade_scan_start(%args);
## INPUTS TO METHOD CALL 'aggr_64bit_upgrade_scan_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##   aggr_64bit_upgrade_scan_start(aggregate => api_new_aggr);
##
##   Output:
##
##    [   {   'status'    => 'Started the 64-bit upgrade scanner',
##            'aggregate' => 'api_new_aggr'
##        }
##    ];

sub aggr_64bit_upgrade_scan_start {
    my %args = @_;

    my ( $error_message, $aggrName );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $command,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output
        =~ /(Started the 64-bit upgrade scanner on aggregate)\s(.+)\s(?:and.+)/
        ) {
        my $myStatus = $1;
        $aggrName = $2;

        $myStatus =~ s/ on aggregate//g;
        return [
            {   'aggregate' => "$aggrName",
                'status'    => "$myStatus"
            }
        ];
    } ## end if ( $output =~ ...
} ## end sub aggr_64bit_upgrade_scan_start

## @name ::aggr_64bit_upgrade_scan_stop
##
## @summary Parses the raw output from the "aggr 64bit-upgrade scan-stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_64bit_upgrade_scan_stop with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_64bit_upgrade_scan_stop(%args);
## INPUTS TO METHOD CALL 'aggr_64bit_upgrade_scan_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##   aggr_64bit_upgrade_scan_stop(aggregate => api_new_aggr);
##
##   Output:
##
##    [   {   'status'    => 'Stopped the 64-bit upgrade scanner',
##            'aggregate' => 'api_new_aggr'
##        }
##    ];

sub aggr_64bit_upgrade_scan_stop {
    my %args = @_;

    my ( $error_message, $aggrName );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output        => $output,
        command           => $command,
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output
        =~ /(Stopped the 64-bit upgrade scanner on aggregate)\s(.+)\s(?:and.+)/
        ) {
        my $myStatus = $1;
        $aggrName = $2;

        $myStatus =~ s/ on aggregate//g;
        return [
            {   'aggregate' => "$aggrName",
                'status'    => "$myStatus"
            }
        ];
    } ## end if ( $output =~ ...
} ## end sub aggr_64bit_upgrade_scan_stop

## @name ::aggr_copy_start
##
## @summary Parses the raw output from the "aggr copy start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_copy_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_copy_start(%args);
## INPUTS TO METHOD CALL 'aggr_copy_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'free-space-defragmentation' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'all-snapshots' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_copy_start {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr copy start",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return undef;
} ## end sub aggr_copy_start

## @name ::aggr_copy_status
##
## @summary Parses the raw output from the "aggr copy status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_copy_status with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_copy_status(%args);
## INPUTS TO METHOD CALL 'aggr_copy_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr copy status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'begun, 0 MB to be copied',
##&nbsp;            'operation_no' => '0'
##&nbsp;          }
##&nbsp;        ];

sub aggr_copy_status {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr copy status",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = qr/(aggr copy restore)|(aggr copy dump)/;
    my $can_throw  = 1;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp,
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line
            =~ /aggr copy restore (\d+) : (begun, .+ to be copied|\d+% done, .+ copied.)/
            ) {
            $hash{'operation_no'} = $1;
            $hash{'status'}       = $2;
            push( @result_array, \%hash );
        } elsif ( $line
            =~ /aggr copy dump (\d+) : (\d+\s*% done). Estimate (\d+) minutes remaining./
            ) {
            $hash{'operation_no'}                   = $1;
            $hash{'status'}                         = $2;
            $hash{'estimate_time_remaining_in_min'} = $3;
            push( @result_array, \%hash );
        } elsif ( $line
            =~ /aggr copy restore (\d+) : (\d+\s*% done). Estimate (\d+) minutes remaining/
            ) {
            $hash{'operation_no'}                   = $1;
            $hash{'status'}                         = $2;
            $hash{'estimate_time_remaining_in_min'} = $3;
            push( @result_array, \%hash );
        } ## end elsif ( $line =~ ...
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub aggr_copy_status

## @name ::aggr_copy_abort
##
## @summary Parses the raw output from the "aggr copy abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_copy_abort with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_copy_abort(%args);
## INPUTS TO METHOD CALL 'aggr_copy_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all-operations' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr copy abort
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'status' => 'aborted',
##&nbsp;            'operation_no' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'aborted',
##&nbsp;            'operation_no' => '1'
##&nbsp;          }
##&nbsp;        ];

sub aggr_copy_abort {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr copy abort",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "image operation";
    my $can_throw  = 0;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );

    foreach my $line (@lines) {
        my %hash;
        if ( $line
            =~ /image operation (\d+) (is now being aborted|already being aborted)/
            ) {
            $hash{'operation_no'} = $1;
            $hash{'status'}       = "aborted";
            push( @result_array, \%hash );
        } ## end if ( $line =~ ...
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub aggr_copy_abort

## @name ::aggr_copy_throttle
##
## @summary Parses the raw output from the "aggr copy throttle" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_copy_throttle with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_copy_throttle(%args);
## INPUTS TO METHOD CALL 'aggr_copy_throttle' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'throttle-value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'operation-number' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr copy throttle
##&nbsp;$VAR1 =   [
##&nbsp;          {
##&nbsp;            'operation_no' => '0',
##&nbsp;            'throttle_adjusted_to' => '40%'
##&nbsp;          }
##&nbsp;        ];

sub aggr_copy_throttle {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my @result_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr copy throttle",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "copy operation";
    my $can_throw  = 1;
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );

    foreach my $line (@lines) {
        if ( $line
            =~ /copy operation (\d+): Throttle adjusted from \d+% to (\d+%) of full speed/
            ) {
            my %hash;
            $hash{'operation_no'}         = $1;
            $hash{'throttle_adjusted_to'} = $2;
            push( @result_array, \%hash );
        } ## end if ( $line =~ ...
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub aggr_copy_throttle

## @name ::aggr_split
##
## @summary Parses the raw output from the "aggr split" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_split with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_split(%args);
## INPUTS TO METHOD CALL 'aggr_split' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-aggregate-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr split /aggr0/plex3 aggr1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'old_aggregate' => 'aggr0',
##&nbsp;             'new_aggregate' => 'aggr1',
##&nbsp;             'result' => 'completed'
##&nbsp;           }
##&nbsp;        ];
sub aggr_split {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        raw_output        => $raw_output,
        command           => "aggr split",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $raw_output
        =~ /Split\sof\sformerly-mirrored\saggregate\s(.*)\scompleted,\snew\saggregate\s(.*)\screated\./
        ) {
        @result_array = (
            {   'old_aggregate' => $1,
                'new_aggregate' => $2,
                'result'        => 'completed'
            }
        );
        return \@result_array;
    } ## end if ( $raw_output =~ ...
    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text            => "output is not in the expected format",
        response_object => $resp
    );
} ## end sub aggr_split

## @name ::aggr_show_space
##
## @summary Parses the raw output from the "aggr show_space" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_show_space with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_show_space(%args);
## INPUTS TO METHOD CALL 'aggr_show_space' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-k' </td> <td> 'kilo-size' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mega-size' </td> </tr>
## <tr> <td> '-t' </td> <td> 'tera-size' </td> </tr>
## <tr> <td> '-g' </td> <td> 'giga-size' </td> </tr>
## <tr> <td> '-b' </td> <td> 'byte-size' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-h' </td> <td> 'actual-size' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr show_space -g
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => [
##&nbsp;                         {
##&nbsp;                           'wafl_reserve' => '0GB',
##&nbsp;                           'volume' => [
##&nbsp;                                         {
##&nbsp;                                           'allocated' => '0GB',
##&nbsp;                                           'guarantee' => 'volume',
##&nbsp;                                           'volume' => 'vol1',
##&nbsp;                                           'used' => '0GB'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'allocated' => '0GB',
##&nbsp;                                           'guarantee' => 'volume',
##&nbsp;                                           'volume' => 'vol3',
##&nbsp;                                           'used' => '0GB'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;                           'snap_reserve' => '0GB',
##&nbsp;                           'smtape' => '0GB',
##&nbsp;                           'a_sis' => '0GB',
##&nbsp;                           'total_space' => '0GB',
##&nbsp;                           'usable_space' => '0GB',
##&nbsp;                           'bsr_nvlog' => '0GB',
##&nbsp;                           'aggregate' => [
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'Total space',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'Snap reserve',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'WAFL reserve',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            }
##&nbsp;                                          ]
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;             'aggregate' => 'aggr1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'info' => [
##&nbsp;                         {
##&nbsp;                           'wafl_reserve' => '0GB',
##&nbsp;                           'total_space' => '0GB',
##&nbsp;                           'usable_space' => '0GB',
##&nbsp;                           'snap_reserve' => '0GB',
##&nbsp;                           'bsr_nvlog' => '0GB',
##&nbsp;                           'aggregate' => [
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'Total space',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'Snap reserve',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'allocated' => '0GB',
##&nbsp;                                              'aggregate_property' => 'WAFL reserve',
##&nbsp;                                              'used' => '0GB',
##&nbsp;                                              'avail' => '0GB'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                           'smtape' => '0GB',
##&nbsp;                           'a-sis' => '0GB'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;             'aggregate' => 'aggr2'
##&nbsp;           },
##&nbsp;        ]

sub aggr_show_space {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"        => $output,
        "command"           => "aggr show_space",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _aggr_show_space(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} ## end sub aggr_show_space

sub _aggr_show_space {
    my %args = @_;
    my ( $first_line, $record, $aggr_name ) = ();
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};

    # For aggr_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    # Extracting the command option.
    my $option = $command_line;
    $option =~ s/aggr\sshow_space\s?//;
    if ( $option !~ /-\w/ ) {
        $option = "";
    }
    my @aggr_info = ();

    $first_line = 'Aggregate\s\'*\'';
    $raw_output = snip_only_required_output(
        'raw_output' => $raw_output,
        'delimiter'  => $first_line
    );
    my @records = split( /Aggregate\s\'*\'/, $raw_output );
    shift(@records);    # Remove the first empty element

    foreach $record (@records) {
        if (   ( $option eq "" )
            || ( $option =~ /-h/ )
            || ( $option =~ /-k/ )
            || ( $option =~ /-m/ )
            || ( $option =~ /-g/ )
            || ( $option =~ /-t/ )
            || ( $option =~ /-b/ ) ) {
            $first_line = ".*\'";
            my @arr = get_lines(
                "raw_output"      => $record,
                "first_line"      => $first_line,
                "can_throw"       => $can_throw,
                "response_object" => $response
            );
            $arr[0] =~ s/\'$//;    # Remove the single quote at the end
            $aggr_name = $arr[0];
            my %temp_aggr_hash;
            my %total_space = ();
            my @heading     = (
                "total_space",  "wafl_reserve",
                "snap_reserve", "usable_space",
                "bsr_nvlog",    "a_sis",
                "smtape"
            );
            $temp_aggr_hash{'aggregate'} = $aggr_name;
            $temp_aggr_hash{'info'}      = ();
            shift(@arr);           # Remove line "Aggregate 'aggr'"
            $first_line
                = 'Total\sspace\s+WAFL\sreserve\s+Snap\sreserve\s+Usable\sspace\s+BSR\sNVLOG\s+A-SIS';
            @arr = get_lines(
                "raw_output"      => $record,
                "first_line"      => $first_line,
                "can_throw"       => $can_throw,
                "response_object" => $response
            );
            shift(@arr)
                ;    # Remove heading line "Total space    WAFL reserve ..."
            my $reserve = shift(@arr);
            my $index   = 0;
            my @values  = split( /\s+/, $reserve );

            foreach (@values) {
                $total_space{ $heading[$index] } = $_;
                $index++;
            }
            shift(@arr)
                ;  # Remove line "Space allocated to volumes in the aggregate"
            foreach my $volume (@arr) {
                my ( %vol_hash, %aggr_hash ) = ();
                if ( $volume =~ /^Volume\s+/ ) {
                    $total_space{'volume'} = ();
                    next;
                }
                if ( $volume =~ /^Aggregate\s+/ ) {
                    $total_space{'aggregate'} = ();
                    next;
                }
                if ( $volume =~ /^(\w+)\s+(\w+)\s+(\w+)\s+(\w+)$/ ) {
                    $vol_hash{'volume'}    = $1;
                    $vol_hash{'allocated'} = $2;
                    $vol_hash{'used'}      = $3;
                    $vol_hash{'guarantee'} = $4;
                    push( @{ $total_space{'volume'} }, \%vol_hash );
                } elsif ( $volume
                    =~ /(Total\sspace|Snap\sreserve|WAFL\sreserve)\s+(\w+)\s+(\w+)\s+(\w+)/
                    ) {
                    $aggr_hash{'aggregate_property'} = $1;
                    $aggr_hash{'allocated'}          = $2;
                    $aggr_hash{'used'}               = $3;
                    $aggr_hash{'avail'}              = $4;
                    push( @{ $total_space{'aggregate'} }, \%aggr_hash );
                } ## end elsif ( $volume =~ ...
            } ## end foreach my $volume (@arr)
            push( @{ $temp_aggr_hash{'info'} }, \%total_space );
            push( @aggr_info,                   \%temp_aggr_hash );
        } ## end if ( ( $option eq "" )...
    } ## end foreach $record (@records)
    return \@aggr_info;
} ## end sub _aggr_show_space

## @name ::aggr_mirror
##
## @summary Parses the raw output from the "aggr mirror" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_mirror with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_mirror(%args);
## INPUTS TO METHOD CALL 'aggr_mirror' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> </tr>
## <tr> <td> '-v' </td> <td> 'victim-aggr' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-n' </td> <td> 'is-display-command' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr mirror apiset_aggr1 -f -d v4.28 v5.21
##&nbsp;$VAR1 =  [
##&nbsp;            {
##&nbsp;              'disks' => '2',
##&nbsp;              'status' => 'initiated'
##&nbsp;            }
##&nbsp;         ];

sub aggr_mirror {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        raw_output        => $raw_output,
        command           => "aggr mirror",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # If we have reached here, we assume we have covered all the
    # error cases.
    if ( $raw_output
        =~ /Creation of a mirror plex has been initiated.\s+(\d+) disk(s)? need(s)? to be/
        ) {
        @result_array = ( { 'disks' => $1, 'status' => 'initiated' } );
    } elsif ( $raw_output
        =~ /Creation of a mirror plex with (\d+) disks has completed./ ) {
        @result_array = ( { 'disks' => $1, 'result' => 'completed' } );
    }

    return \@result_array;
} ## end sub aggr_mirror

## @name ::aggr_media_scrub_start
##
## @summary Parses the raw output from the "aggr media_scrub start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_media_scrub_start with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_media_scrub_start(%args);
## INPUTS TO METHOD CALL 'aggr_media_scrub_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'rg-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> </tr>
## <tr> <td> '-s' </td> <td> 'disk-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_media_scrub_start {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr media_scrub start",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return undef;
} ## end sub aggr_media_scrub_start

## @name ::aggr_media_scrub_stop
##
## @summary Parses the raw output from the "aggr media_scrub stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_media_scrub_stop with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_media_scrub_stop(%args);
## INPUTS TO METHOD CALL 'aggr_media_scrub_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'rg-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> </tr>
## <tr> <td> '-s' </td> <td> 'disk-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub aggr_media_scrub_stop {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr media_scrub stop",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return undef;
} ## end sub aggr_media_scrub_stop

## @name ::aggr_read_fsid
##
## @summary Parses the raw output from the "aggr read_fsid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_read_fsid with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_read_fsid(%args);
## INPUTS TO METHOD CALL 'aggr_read_fsid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr read_fsid component_flex_aggr
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregate' => 'component_flex_aggr',
##&nbsp;            'fsid' => '0x5abf3b07'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub aggr_read_fsid {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr read_fsid",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result;
    my %result_hash;

    if ( $raw_output =~ /Aggregate (\w+) has an FSID of (\w+)/i ) {
        $result_hash{'aggregate'} = $1;
        $result_hash{'fsid'}      = $2;
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub aggr_read_fsid

## @name ::aggr_rewrite_fsid
##
## @summary Parses the raw output from the "aggr rewrite_fsid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_rewrite_fsid with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_rewrite_fsid(%args);
## INPUTS TO METHOD CALL 'aggr_rewrite_fsid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr rewrite_fsid component_flex_aggr
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregate' => 'component_flex_aggr',
##&nbsp;            'old_fsid'  => '0x5abf3b07'
##&nbsp;            'new_fsid'  => '0x11a073e9',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub aggr_rewrite_fsid {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr rewrite_fsid",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #my $first_line = "copy operation";
    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => 1,
        "response_object" => $resp,
    );

    my @result;
    my %result_hash;

    foreach my $line (@lines) {
        if ( $line
            =~ /Updating the old fsid (\d+) with new fsid (\d+): start time (\d+)/i
            ) {

#First line may contains the fsid(old and new) in decimal format and start time.
            $result_hash{'old_fsid_decimal'} = $1;
            $result_hash{'new_fsid_decimal'} = $2;
            $result_hash{'start_time'}       = $3;
        } elsif ( $line
            =~ /Aggregate \'(\w+)\' has had its FSID changed from (\w+) to (\w+)/i
            ) {
            $result_hash{'aggregate'} = $1;
            $result_hash{'old_fsid'}  = $2;
            $result_hash{'new_fsid'}  = $3;
        }
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub aggr_rewrite_fsid

## @name ::aggr_reserve_show
##
## @summary Parses the raw output from the "aggr reserve show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_reserve show with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_reserve show(%args);
## INPUTS TO METHOD CALL 'aggr reserve show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'atv-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr reserve show
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'factory_default' => '0',
##&nbsp;            'current_default' => '0',
##&nbsp;            'assignments_remaining' => '0',
##&nbsp;            'aggregates' => [
##&nbsp;              {
##&nbsp;                'aggregate' => 'testaggr',
##&nbsp;                'reserve'   => '0',
##&nbsp;              },
##&nbsp;              {
##&nbsp;                'aggregate' => 'testaggr_2',
##&nbsp;                'reserve'   => '5',
##&nbsp;              }
##&nbsp;            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = aggr reserve show [-p] testaggr
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregates' => [
##&nbsp;              {
##&nbsp;                'aggregate' => 'testaggr',
##&nbsp;                'reserve'   => '0',
##&nbsp;              },
##&nbsp;            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub aggr_reserve_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr read_fsid",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result;
    my %result_hash;

    # Extracting the command option.
    my $option = $command;
    $option =~ s/aggr\sreserve\sshow\s*//;
    my %aggr_hash;

    # If there are any options, then we know we have the simple case to parse
    if ($option eq "") {
        my $first_line = '';
        my @lines      = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => 1,
            "response_object" => $resp,
        );
        my $aggr = "";
        foreach my $line (@lines) {
            if ($line =~ /Factory Default:\s+(\d+)%/i) {
                $result_hash{factory_default} = $1;
            } elsif ($line =~ /Current Default:\s+(\d+)%/i) {
                $result_hash{current_default} = $1;
            } elsif ($line =~ /Assignments \((\d+) Remaining\)/i) {
                $result_hash{assignments_remaining} = $1;
            } elsif ($line =~ /(\w+)\s+(\d+)%/i) {
                $aggr_hash{aggregate} = $1;
                $aggr_hash{reserve}   = $2;
                push @{$result_hash{aggregates}}, {%aggr_hash};
            }
        }
    } else {
        if ($raw_output =~ /Aggregate:\s(\w+)/i) {
            $aggr_hash{aggregate} = $1;
        }
        if ($raw_output =~ /Reserve:\s(\d+)%/i) {
            $aggr_hash{reserve} = $1;
        }
        push @{$result_hash{aggregates}}, \%aggr_hash;
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub aggr_reserve_show

## @name ::aggr_reserve_default_show
##
## @summary Parses the raw output from the "aggr reserve default show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_reserve_default_show with any of its options as
##    described in the corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_reserve_default_show(%args);
## INPUTS TO METHOD CALL 'aggr_reserve_default_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = aggr reserve default show
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'factory_default' => '0',
##&nbsp;            'current_default' => '0'
##&nbsp;            'assignments_remaining' => '4'
##&nbsp;            'partner_assignments_remaining' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub aggr_reserve_default_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr reserve default show",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result;
    my %result_hash;

    # No first line to ignore
    my $first_line = '';
    my @lines      = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => 1,
        "response_object" => $resp,
    );
    my @aggr_array = ();
    my $aggr       = "";
    my %aggr_hash;
    foreach my $line (@lines) {
        if ($line =~ /Factory Default:\s+(\d+)%/i) {
            $result_hash{factory_default} = $1;
        } elsif ($line =~ /Current Default:\s+(\d+)%/i) {
            $result_hash{current_default} = $1;
        } elsif ($line =~ /Assignments Remaining:\s+(\d+)/i) {
            $result_hash{assignments_remaining} = $1;
        } elsif (
            $line =~ /Assignments Remaining for Partner Aggregates:\s+(\d+)/i)
        {
            $result_hash{partner_assignments_remaining} = $1;
        }
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub aggr_reserve_default_show

## @name ::aggr_reserve_default_modify
##
## @summary Parses the raw output from the "aggr reserve default modify" command
##
## @description
##    This method can be used to check for errors in the output of
##    aggr_reserve_default_modify with any of its options as
##    described in the corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_reserve_default_modify(%args);
## INPUTS TO METHOD CALL 'aggr_reserve_default_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty response object -- this merely checks for errors

sub aggr_reserve_default_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr reserve default modify",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @empty_array = ();
    return @empty_array;
} ## end sub aggr_reserve_default_modify

## @name ::aggr_reserve_modify
##
## @summary Parses the raw output from the "aggr reserve modify" command
##
## @description
##    This method can be used to check for errors in the output of
##    aggr_reserve_modify with any of its options as
##    described in the corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_reserve_modify(%args);
## INPUTS TO METHOD CALL 'aggr_reserve_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty response object -- this merely checks for errors

sub aggr_reserve_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "aggr reserve modify",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @empty_array = ();
    return @empty_array;
} ## end sub aggr_reserve_modify

## @name ::aggr_wafliron_commit
##
## @summary Parses the raw output from the "aggr wafliron commit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_commit with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_commit(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_commit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub aggr_wafliron_commit {
    my %args = @_;
    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(
        raw_output        => $output,
        "command"       => $command,
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub aggr_wafliron_commit

## @name ::aggr_wafliron_reject
##
## @summary Parses the raw output from the "aggr wafliron reject" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aggr_wafliron_reject with any of its options as described in the
##    corresponding cdef file 'aggr.cdefs'
## @synopsis
##      aggr_wafliron_reject(%args);
## INPUTS TO METHOD CALL 'aggr_wafliron_reject' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub aggr_wafliron_reject {
    my %args = @_;
    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(
        raw_output        => $output,
        "command"       => $command,
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub aggr_wafliron_reject

1;
