#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode autosupport command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode
##    autosupport command.
##
## @synopsis
##    use autosupport;
##
##    autosupport_budget_modify(%args);
##    autosupport_budget_show(%args);
##    autosupport_destinations_show(%args);
##    autosupport_history_show(%args);
##    autosupport_history_retransmit(%args);
##    autosupport_trigger_modify(%args);
##    autosupport_manifest_show(%args);
##
## @author kathar@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::autosupport;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error         = undef;
    if ($output =~ /Error:\s+(Missing\s+value\s+for\s+.*)/) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ($output =~
            /Error:\s+command\s+failed:\s+((.*)\s+is\s+a\s+required\s+field)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~
          /Error:\s+("\w+"\s+is\s+an\s+invalid\s+value)\s+for\s+field\s+(.*)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~
        /Error:\s+('\w+'\s+was\s+not\s+expected.\s+Please\s+specify\s+-fieldname\s+first)/
      ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /[Uu]sage:/) {
        $error          = "CommandFailed";
        $$error_message = "Invalid switch or Invalid Param value has been sent";
        return $error;
    } elsif ($output =~ /Error:\s+(invalid\s+argument\s+".*")/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif (
        $output =~ /Error:\s+Field\s+((.*)\s+was\s+specified\s+twice\s+(.*))/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /Error:\s+(Unexpected\s+argument\s+(.*))/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~
        /Error:\s+command\s+failed:\s+(Retransmit\s+request\s+already\s+in\s+progress)/
      ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~ /Error:\s+command\s+failed:\s+(URI\s+is\s+invalid)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~
        /Error:\s+-(fields\s+takes\s+a\s+list\s+of\s+field\s+names\s+without\s+a\s+leading\s+(.*))/
      ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~
        /Error:\s+command\s+failed:\s+The\s+mandatory\s+subsystem\s+is\s+(already\s+being\s+collected\s+in\s+the\s+troubleshooting\s+subsystem\s+list)/
      ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~ /Error:\s+(cannot\s+specify\s+.*\s+together)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~ /Error:\s+command failed:\s(entry\s+doesn't\s+exist)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($output =~ /Error:\s+(Missing\s+value\s+for\s+.*)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~
            /Error:\s+You\s+need\s+to\s+provide\s+some\s+values\s+to\s+apply/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~
        /Error:\s+Missing\s+value\s+for\s+(.*)\s+If\s+you\s+want\s+to\s+only\s+display\s+certain\s+fields, please\s+use\s+-fields\s+<list of fields>/
      ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::autosupport_budget_modify
##
## @summary Parses the raw output from the "autosupport budget modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_budget_modify with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_budget_modify(%args);
## INPUTS TO METHOD CALL 'autosupport_budget_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-size-limit' </td> <td> 'size-limit' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> </tr>
## <tr> <td> '-time-limit' </td> <td> 'time-limit' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport budget modify -subsystem performance -time-limit 3m; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub autosupport_budget_modify {
    my %args = @_;
    my @result_array1;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub autosupport_budget_modify

## @name ::autosupport_budget_show
##
## @summary Parses the raw output from the "autosupport budget show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_budget_show with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_budget_show(%args);
## INPUTS TO METHOD CALL 'autosupport_budget_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-size-limit' </td> <td> 'size-limit' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-time-limit' </td> <td> 'time-limit' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport budget show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'subsystem' => 'platform',
##&nbsp;            'size_budget' => '2MB',
##&nbsp;            'time_budget' => '1m'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subsystem' => 'secd',
##&nbsp;            'size_budget' => '2MB',
##&nbsp;            'time_budget' => '1m'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subsystem' => 'metrocluster',
##&nbsp;            'size_budget' => '2MB',
##&nbsp;            'time_budget' => '1m'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'subsystem' => 'mandatory',
##&nbsp;            'size_budget' => '0B',
##&nbsp;            'time_budget' => '0h'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub autosupport_budget_show {
    my %args = @_;
    my @result_array1;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($output =~ /There are no entries matching your query/) {
        my %info;
        $info{status} = "no matching entries found";
        push(@result_array, \%info);
    } else {
        my $option = $command;
        $option =~ s/autosupport budget show\s?//;
        if (   $option =~ /-fields/
            || $option eq ""
            || $option =~ /(-time-limit|-size-limit)/) {
            my @lines = get_lines(raw_output      => $output,
                                  response_object => $resp,
                                  can_throw       => 1);
            my @headings = split(/\s+/, shift(@lines));
            if (($headings[$#headings] eq 'Budget')) {
                $headings[$#headings - 1] .= ' ' . pop(@headings);
            }
            if ($output =~ /Subsystem\s+Time\s+Budget\s+Size\s+Budget/) {
                pop(@lines);
                splice(@headings, 1, 2, ("time_budget"));
            }
            @headings = map(to_key_name(item => $_), @headings);
            my (@values, $i);
            shift(@lines);
            foreach my $line (@lines) {
                my %hash;
                @values = split(/\s+/, $line);
                $i = 0;
                foreach (@headings) {
                    $hash{$_} = $values[$i];
                    $i++;
                }
                push(@result_array, \%hash);
            } ## end foreach my $line (@lines)
        } elsif ($option =~ /-instance/ || $option =~ /-subsystem/) {
            my @lines = get_lines(raw_output      => $output,
                                  response_object => $resp,
                                  can_throw       => 1);
            my $i = -1;
            foreach my $line (@lines) {
                if ($line =~ /Subsystem:\s+(.*)/) {
                    $i++;
                }
                if (($line =~ /(.*):\s+(.*)/) && ($line !~ /entries/)) {
                    my $val1 = $1;
                    to_key_name(item => $val1);
                    my $val2 = $2;
                    to_key_name(item => $val2);
                    $result_array[$i]->{$val1} = $val2;
                }
            } ## end foreach my $line (@lines)
        } ## end elsif ($option =~ /-instance/...
    } ## end else [ if ($output =~ /There are no entries matching your query/)
    return \@result_array;
} ## end sub autosupport_budget_show

## @name ::autosupport_destinations_show
##
## @summary Parses the raw output from the "autosupport destinations show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_destinations_show with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_destinations_show(%args);
## INPUTS TO METHOD CALL 'autosupport_destinations_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport destinations show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'destination_1' => 'https://testbed.corp.netapp.com/put/AsupPut',
##&nbsp;            'destination_2' => 'https://testbed.corp.netapp.com/asupprod/post/1.0/postAsup'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub autosupport_destinations_show {
    my %args = @_;
    my @result_array1;
    my ($error_message, @result_array);
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "---------------";
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          response_object => $resp,
                          can_throw       => 1);
    shift(@lines);
    my %info;
    my $i = 1;
    foreach my $line (@lines) {
        if ($line =~ /(.*)/) {
            my $value = $1;
            my $key   = "destination_" . $i;
            $info{$key} = $value;
            $i++;
        }
    }
    push(@result_array, \%info);
    return \@result_array;
} ## end sub autosupport_destinations_show

## @name ::autosupport_history_show
##
## @summary Parses the raw output from the "autosupport history show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_history_show with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_history_show(%args);
## INPUTS TO METHOD CALL 'autosupport_history_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-trigger' </td> <td> 'trigger' </td> </tr>
## <tr> <td> '-subject' </td> <td> 'subject' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> </tr>
## <tr> <td> '-destination' </td> <td> 'destination' </td> </tr>
## <tr> <td> '-delivery' </td> <td> 'delivery' </td> </tr>
## <tr> <td> '-last-update' </td> <td> 'last-update' </td> </tr>
## <tr> <td> '-attempt-count' </td> <td> 'attempt-count' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-uri' </td> <td> 'uri' </td> </tr>
## <tr> <td> '-detail' </td> <td> 'detail' </td> </tr>
## <tr> <td> '-seq-num' </td> <td> 'seq-num' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-error' </td> <td> 'error' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport history show -fields trigger -seq-num 39; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'trigger' => 'callhome.management.log',
##&nbsp;            'seq-num' => '39',
##&nbsp;            'seq_num' => '39',
##&nbsp;            'destination' => 'smtp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'trigger' => 'callhome.management.log',
##&nbsp;            'seq-num' => '39',
##&nbsp;            'destination' => 'http'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'trigger' => 'callhome.management.log',
##&nbsp;            'seq-num' => '39',
##&nbsp;            'seq_num' => '39',
##&nbsp;            'destination' => 'noteto'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'trigger' => 'callhome.management.log',
##&nbsp;            'seq-num' => '39',
##&nbsp;            'seq_num' => '39',
##&nbsp;            'destination' => 'retransmit'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub autosupport_history_show {
    my %args = @_;
    my ($i, $j, $value);
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line;
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($output =~ /There are no entries matching your query/) {
        my %info;
        $info{status} = "no matching entries found";
        push(@result_array, \%info);
    } else {
        my $option = $command;
        my $seq_no;
        $option =~ s/autosupport history show\s?//;
        if ($option =~ /-delivery/ || $option =~ /detail/) {
            my @lines = get_lines(raw_output      => $output,
                                  response_object => $resp,
                                  can_throw       => 1);
            shift(@lines);
            my $i = -1;
            my $j;
            my $flag;
            foreach my $line (@lines) {
                if ($line =~ /Num\s+Trigger\s+Subject/) {
                    $flag = 1;
                } elsif (   ($line =~ /([a-z\.]+)\s+([A-Z\s+\_]+)/)
                         && ($line !~ /entries/)
                         && ($flag == 1)
                         && ($line !~ /^(\d+)/)) {
                    $result_array[$i]->{statistics}->[$j]->{trigger} = $1;
                    $result_array[$i]->{statistics}->[$j]->{subject} = $2;
                    $j++;
                } elsif (   ($line =~ /(\w+)\s+(.*)/)
                         && ($line !~ /entries/)
                         && ($line !~ /^(\d+)/)) {
                    $result_array[$i]->{statistics}->[$j]->{dest_type}   = $1;
                    $result_array[$i]->{statistics}->[$j]->{delivery_to} = $2;
                    $j++;
                } elsif (($line =~ /(\d+)/) && ($line !~ /entries/)) {
                    $i++;
                    $j = 0;
                    $result_array[$i]->{seq_num} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ($option =~ /-instance/) {
            my @lines = get_lines(raw_output      => $output,
                                  response_object => $resp,
                                  can_throw       => 1);
            my $i = -1;
            my ($val1, $val2);
            foreach my $line (@lines) {
                if ($line =~ /AutoSupport\s+Sequence\s+Number:\s+(\d+)/) {
                    $i++;
                }
                if ($line =~ /(.*):\s+(.*)/) {
                    $val1                      = $1;
                    $val2                      = $2;
                    $val1                      = to_key_name(item => $val1);
                    $result_array[$i]->{$val1} = $val2;
                }
            } ## end foreach my $line (@lines)
        } elsif ($option =~ /-fields/) {
            my $first_line = 'seq-num';
            my @lines = get_lines(raw_output      => $output,
                                  first_line      => $first_line,
                                  response_object => $resp);
            my (@head_arr, @value_arr, $k, $count);
            my $i = 0;
            my $m = 0;
            foreach my $line (@lines) {
                if ($line =~ /^seq-num.*/) {
                    @head_arr = split(/\s+/, $line);
                    next;
                } elsif ($line =~ /---------.*/) {
                    next;
                } elsif (($line =~ /\d+\s+\w+\s+.*/) && ($line !~ /entries/)) {
                    @value_arr = split(//, $line);
                    $count = 0;
                    foreach my $k (@value_arr) {
                        if ($k =~ /\"/) {
                            $count = $count + 1;
                        }
                        if ($count % 2 == 1) {
                            $k =~ s/\s+/####/;
                        }
                    }
                    my $var1 = join(/./, @value_arr);
                    @value_arr = split(/\s+/, $var1);
                    for (my $m = 0 ; $m <= $#head_arr ; $m++) {
                        $value_arr[$m] =~ s/####/ /g;
                        $value_arr[$m] =~ s/\"//g;
                        $result_array[$i]->{$head_arr[$m]} = $value_arr[$m];
                    }
                } ## end elsif (($line =~ /\d+\s+\w+\s+.*/...
                $i++;
            } ## end foreach my $line (@lines)
        } ## end elsif ($option =~ /-fields/)
        if (($option eq "")
            || ($option =~
                /(-seq-num|-destination|-trigger|-last-update|-status|-attempt-count|-subject|-uri|-error)/
               )
          ) {
            my @lines = get_lines(raw_output      => $output,
                                  response_object => $resp,
                                  can_throw       => 1);
            my $i = -1;
            my $j;
            my @result_array2;
            foreach my $line (@lines) {
                if ($line =~
                    /(?:--------.*)|(?:Seq\s+Attempt\s+Last)|(?:Num\s+Destination\s+Status\s+Count\s+Update)/
                  ) {
                    next;
                } elsif (($line =~ /^(\d+)/) && ($line !~ /entries/)) {
                    $i++;
                    $j = 0;
                    $result_array[$i]->{seq_num} = $1;
                }
                elsif (
                    ($line =~
                     /(\w+)\s+(.*)\s+([\d+]+|-)\s+(\d+\/\d+\/\d+\s+\d+:\d+:\d+)/
                    )
                    && ($line !~ /entries/)
                  ) {
                    my $val1 = $2;
                    $val1 =~ tr/\t //d;

                    #$val1 = to_key_name(item => $val1);
                    my $val2 = $3;
                    $val2 =~ tr/\t //d;

                    #$val1 = to_key_name(item => $val1);
                    $result_array[$i]->{statistics}->[$j]->{destination} = $1;
                    $result_array[$i]->{statistics}->[$j]->{status} = $val1;
                    $result_array[$i]->{statistics}->[$j]->{attemp_count} =
                      $val2;
                    $result_array[$i]->{statistics}->[$j]->{last_update} = $4;
                    $j++;
                } ## end elsif (($line =~ ...
            } ## end foreach my $line (@lines)
        } ## end if (($option eq "") ||...
        return \@result_array;
    } ## end else [ if ($output =~ /There are no entries matching your query/)
} ## end sub autosupport_history_show

## @name ::autosupport_history_retransmit
##
## @summary Parses the raw output from the "autosupport history retransmit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_history_retransmit with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_history_retransmit(%args);
## INPUTS TO METHOD CALL 'autosupport_history_retransmit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-seq-num' </td> <td> 'seq-num' </td> </tr>
## <tr> <td> '-size-limit' </td> <td> 'size-limit' </td> </tr>
## <tr> <td> '-uri' </td> <td> 'uri' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport history retransmit -seq-num 83 -uri https://testbed.corp.netapp.com/put/AsupPut; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub autosupport_history_retransmit {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub autosupport_history_retransmit

## @name ::autosupport_trigger_modify
##
## @summary Parses the raw output from the "autosupport trigger modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_trigger_modify with any of its options as
##    described in the corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_trigger_modify(%args);
## INPUTS TO METHOD CALL 'autosupport_trigger_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-noteto' </td> <td> 'noteto' </td> </tr>
## <tr> <td> '-to' </td> <td> 'to' </td> </tr>
## <tr> <td> '-troubleshooting-additional' </td> <td> 'troubleshooting-additional' </td> </tr>
## <tr> <td> '-autosupport-message' </td> <td> 'autosupport-message' </td> </tr>
## <tr> <td> '-basic-additional' </td> <td> 'basic-additional' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport trigger modify -autosupport-message zombie.upg -to enabled; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub autosupport_trigger_modify {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub autosupport_trigger_modify

## @name ::autosupport_manifest_show
##
## @summary Parses the raw output from the "autosupport manifest show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_manifest_show with any of its options as described in the
##    corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_manifest_show(%args);
## INPUTS TO METHOD CALL 'autosupport_manifest_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-cmd' </td> <td> 'cmd' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-query' </td> <td> 'query' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsys' </td> <td> 'subsys' </td> <td> '0' </td> </tr>
## <tr> <td> '-content' </td> <td> 'content' </td> <td> '0' </td> </tr>
## <tr> <td> '-body-file' </td> <td> 'body-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-content-type' </td> <td> 'content-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-time-collected' </td> <td> 'time-collected' </td> <td> '0' </td> </tr>
## <tr> <td> '-seq-num' </td> <td> 'seq-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-prio-num' </td> <td> 'prio-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-cmd-tgt' </td> <td> 'cmd-tgt' </td> <td> '0' </td> </tr>
## <tr> <td> '-error' </td> <td> 'error' </td> <td> '0' </td> </tr>
## <tr> <td> '-size-collected' </td> <td> 'size-collected' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; autosupport manifest show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'cmd' => 'sysconfig -a',
##&nbsp;             'body_file' => 'sysconfig-a.txt',
##&nbsp;             'seq_num' => '647',
##&nbsp;             'status' => 'completed',
##&nbsp;             'size_collected' => '8.29KB'
##&nbsp;           },
##&nbsp;]
##&nbsp;Command = priv set -q diag; autosupport manifest show -fields seq-num, prio-num,subsys,cmd-tgt,body-file,cmd,query,size-collected,time-collected,status,error,content-type; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'cmd' => 'sysconfig -a',
##&nbsp;             'status' => 'completed',
##&nbsp;             'query' => '',
##&nbsp;             'subsys' => 'mandatory',
##&nbsp;             'body-file' => 'SYSCONFIG-A.txt',
##&nbsp;             'content-type' => 'basic',
##&nbsp;             'seq-num' => '62',
##&nbsp;             'prio-num' => '0',
##&nbsp;             'time-collected' => '90',
##&nbsp;             'cmd-tgt' => 'dblade',
##&nbsp;             'error' => '',
##&nbsp;             'size-collected' => '1.62KB'
##&nbsp;           },
##&nbsp;];
##&nbsp;
##&nbsp;Command = priv set -q diag; autosupport manifest show -seq-num 598;  result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'cmd' => 'sysconfig -a',
##&nbsp;             'body_file' => 'web-diag.gz',
##&nbsp;             'seq_num' => '598',
##&nbsp;             'status' => 'content-not-collected-mode',
##&nbsp;             'size_collected' => '-'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'body_file' => 'web-diag-partner.gz',
##&nbsp;             'seq_num' => '598',
##&nbsp;             'status' => 'content-not-collected-mode',
##&nbsp;             'size_collected' => '-'
##&nbsp;           }
##&nbsp;];
##&nbsp;Command = priv set -q diag; autosupport manifest show -seq-num 598 -body-file autosupport.xml; result -l
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'body_file' => 'autosupport.xml',
##&nbsp;             'seq_num' => '598',
##&nbsp;             'status' => 'completed',
##&nbsp;             'size_collected' => '11.87KB'
##&nbsp;           }
##&nbsp;];
sub autosupport_manifest_show {
    my %args = @_;
    my @result_array1;
    my ($i, $j, $key1, $key2, $value, $value2);
    my ($error_message, @result_array);
    my %hash2;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error =
      _check_error(raw_output => $output, error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my $option = $command;
    $option =~ s/autosupport\smanifest\sshow\s?//;
    if ($option =~ /-instance/) {
        my @lines = get_lines(raw_output      => $output,
                              response_object => $resp,
                              can_throw       => 1);
        my $i = -1;
        my $j = 0;
        foreach my $line (@lines) {
            if ($line =~ /^(AutoSupport\s+Sequence\s+Number):\s+(\d+)$/) {
                $i++;
                my $val1 = to_key_name(item => $1);
                my $val2 = to_key_name(item => $2);
                $result_array[$i]->{$val1} = $val2;
            } elsif ($line =~ /^([A-Za-z\s+\(\)]+):([A-Za-z\s+\-\.\d+]+)?$/) {
                my $val1 = to_key_name(item => $1);
                my $val2 = to_key_name(item => $2);
                $result_array[$i]->{$val1} = $val2;
            } elsif ($line =~
                /^(The\sactual\sdata\sbeing\scollected:\sD-blade\sCLI\sOR\sBSD\scommand\sOR\sXML\stable\sOR\sfilename):([\s\w+\-]+)?$/
              ) {
                my $val1 = to_key_name(item => $1);
                my $val2 = to_key_name(item => $2);
                $result_array[$i]->{$val1} = $val2;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    } elsif ($option =~ /-content/) {
        my @lines = get_lines(raw_output      => $output,
                              response_object => $resp,
                              can_throw       => 1);
        my $i = -1;
        my $j = 0;
        foreach my $line (@lines) {
            if ($line =~ /^(Sequence)\s:\s(\d+)$/) {
                $i++;
                my $val1 = to_key_name(item => $1);
                my $val2 = to_key_name(item => $2);
                $result_array[$i]->{$val1} = $val2;
            } elsif ($line =~ /^([\w+\s+]+):\s([\w+\-\.\_\s]+)$/) {
                my $val1 = to_key_name(item => $1);
                my $val2 = to_key_name(item => $2);
                $result_array[$i]->{$val1} = $val2;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    }
    elsif (
        ($option eq "")
        || ($option =~
            /(-seq-num|-prio-num|-subsys|-cmd-tgt|-body-file|-cmd|-query|-size-collected|-time-collected|-status|-error|-content-type)/
           )
      ) {
        my $first_line = "---------.* ";
        my @lines = get_lines(raw_output      => $output,
                              "first_line"    => $first_line,
                              response_object => $resp,
                              can_throw       => 1);
        my $i = -1;
        my $val1;
        my @value_arr = ();
        foreach my $line (@lines) {
            if ($line =~ /^(\d+)$/) {
                $val1 = $1;
                next;
            } elsif ($line =~ /---------.*/) {
                next;
            } elsif ($line =~
                /^([\w+\-\.\_]+)\s+([0-9A-Z\.\-]+)\s+([\w+\-]+)(\s+([\w+0-9\(\)\s+\-]+))?/
              ) {
                $i++;
                $result_array[$i]->{"seq_num"}        = $val1;
                $result_array[$i]->{"body_file"}      = to_key_name(item => $1);
                $result_array[$i]->{"size_collected"} = $2;
                $result_array[$i]->{"status"}         = to_key_name(item => $3);
                $result_array[$i]->{"error"} = to_key_name(item => $4) if ($4);
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    } elsif ($option =~ /-fields/) {
        my $first_line = 'seq-num\sprio-num';
        my @lines = get_lines(raw_output      => $output,
                              "first_line"    => $first_line,
                              response_object => $resp);
        my (@head_arr, @value_arr, $k, $count);
        my $i = 0;
        my $m = 0;
        foreach my $line (@lines) {
            if ($line =~ /^seq-num\sprio-num.*/) {
                @head_arr = split(/\s+/, $line);
                next;
            } elsif ($line =~ /---------.*/) {
                next;
            } elsif ($line =~ /\d+\s+\d+\s+.*/) {

                @value_arr = split(//, $line);
                $count = 0;
                foreach my $k (@value_arr) {
                    if ($k =~ /\"/) {
                        $count = $count + 1;
                    }
                    if ($count % 2 == 1) {
                        $k =~ s/\s+/####/;
                    }
                }
                my $var1 = join("", @value_arr);
                @value_arr = split(/\s+/, $var1);
                for (my $m = 0 ; $m <= $#head_arr ; $m++) {
                    $value_arr[$m] =~ s/####/ /g;
                    $value_arr[$m] =~ s/\"//g;
                    $result_array[$i]->{$head_arr[$m]} = $value_arr[$m];
                }
            }
            $i++;
        } ## end foreach my $line (@lines)
        return \@result_array;
    } ## end elsif ($option =~ /-fields/)
} ## end sub autosupport_manifest_show

## @name ::autosupport_trigger_show
##
## @summary Parses the raw output from the "autosupport trigger show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from autosupport_trigger_show with any of its options as described in the
##    corresponding cdef file 'autosupport.cdefs'
## @synopsis
##      autosupport_trigger_show(%args);
## INPUTS TO METHOD CALL 'autosupport_trigger_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-noteto' </td> <td> 'noteto' </td> </tr>
## <tr> <td> '-basic' </td> <td> 'basic' </td> </tr>
## <tr> <td> '-basic-default' </td> <td> 'basic-default' </td> </tr>
## <tr> <td> '-troubleshooting-additional' </td> <td> 'troubleshooting-additional' </td> </tr>
## <tr> <td> '-autosupport-message' </td> <td> 'autosupport-message' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-additional-content' </td> <td> 'additional-content' </td> </tr>
## <tr> <td> '-troubleshooting' </td> <td> 'troubleshooting' </td> </tr>
## <tr> <td> '-troubleshooting-default' </td> <td> 'troubleshooting-default' </td> </tr>
## <tr> <td> '-to' </td> <td> 'to' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-basic-additional' </td> <td> 'basic-additional' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; autosupport trigger show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'to' => 'enabled',
##&nbsp;             'autosupport-message' => 'wafliron.chg.pending',
##&nbsp;             'note-to' => 'enabled',
##&nbsp;             'additional-content' => '-'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'to' => 'disabled',
##&nbsp;             'autosupport-message' => 'weekly',
##&nbsp;             'note-to' => 'disabled',
##&nbsp;             'additional-content' => '-'
##&nbsp;           }
##&nbsp;];
##&nbsp;Command = priv set -q test; autosupport trigger show -fields autosupport-message,to,noteto,basic-default,troubleshooting-default,additional-content,basic-additional,troubleshooting-additional; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'noteto' => 'disabled',
##&nbsp;             'basic-default' => [
##&nbsp;                                  'mandatory',
##&nbsp;                                  'platform',
##&nbsp;                                  'mhost',
##&nbsp;                                  'performance',
##&nbsp;                                  'san',
##&nbsp;                                  'snapvault',
##&nbsp;                                  'snapmirror',
##&nbsp;                                  'dedupe',
##&nbsp;                                  'nfs',
##&nbsp;                                  'wafl',
##&nbsp;                                  'mot',
##&nbsp;                                  'ha',
##&nbsp;                                  'networking',
##&nbsp;                                  'cifs',
##&nbsp;                                  'multistore',
##&nbsp;                                  'raid',
##&nbsp;                                  'storage',
##&nbsp;                                  'asup_ems',
##&nbsp;                                  'kernel',
##&nbsp;                                  'secd'
##&nbsp;                                ],
##&nbsp;             'troubleshooting-additional' => '-',
##&nbsp;             'autosupport-message' => 'weekly',
##&nbsp;             'additional-content' => '-',
##&nbsp;             'troubleshooting-default' => [
##&nbsp;                                            'mandatory',
##&nbsp;                                            'networking'
##&nbsp;                                          ],
##&nbsp;             'to' => 'disabled',
##&nbsp;             'basic-additional' => '-'
##&nbsp;           }
##&nbsp;];
##&nbsp;Command = priv set -q test; autosupport trigger show -autosupport-message weekly; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'noteto' => 'disabled',
##&nbsp;            'basic-default' => [
##&nbsp;                                 'mandatory',
##&nbsp;                                 'platform',
##&nbsp;                                 'mhost',
##&nbsp;                                 'performance',
##&nbsp;                                 'san',
##&nbsp;                                 'snapvault',
##&nbsp;                                 'snapmirror',
##&nbsp;                                 'dedupe',
##&nbsp;                                 'nfs',
##&nbsp;                                 'wafl',
##&nbsp;                                 'mot',
##&nbsp;                                 'ha',
##&nbsp;                                 'networking',
##&nbsp;                                 'cifs',
##&nbsp;                                 'multistore',
##&nbsp;                                 'raid',
##&nbsp;                                 'storage',
##&nbsp;                                 'asup_ems',
##&nbsp;                                 'kernel',
##&nbsp;                                 'secd'
##&nbsp;                               ],
##&nbsp;            'troubleshooting-additional' => '-',
##&nbsp;            'autosupport-message' => 'weekly',
##&nbsp;            'time-limit' => '-',
##&nbsp;            'additional-content' => '-',
##&nbsp;            'troubleshooting-default' => [
##&nbsp;                                           'mandatory',
##&nbsp;                                           'networking'
##&nbsp;                                         ],
##&nbsp;            'to' => 'disabled',
##&nbsp;            'basic-additional' => undef
##&nbsp;          }
##&nbsp;];


sub autosupport_trigger_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'autosupport trigger show',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array;
    my ($entry_trigger, $entry_basic, $first_line_var ,$basic_def_next_line , $ts_def_next_line , $states );
    $first_line_var = 0;
    $entry_trigger = 0;
    $entry_basic = 0;
    $basic_def_next_line = 0;
    $ts_def_next_line = 0;
    my @name;
    my $opt = $command;
    $opt =~ s/autosupport trigger show (-fields)?//g;
    if(($opt =~ /instance/) || ($opt =~
            /(-autosupport-message|-to|-noteto|-basic-default|-troubleshooting-default|-additional-content|-basic-additional|-troubleshooting-additional|-time-limit)/
           )) {

        my @lines = get_lines( raw_output => $raw_output,
                        can_throw => 0, response_object => $resp );
        my $basic_default = 0;
        my $troubleshooting_default = 0;
        foreach my $line (@lines) {
            if($line =~ /^EMS Message: (.+)/) {
                $result_array[$entry_trigger] -> {'autosupport-message'} = $1;
            } elsif($line =~ /^Whether to deliver to options autosupport\.to addresses: (.+)/) {
                $result_array[$entry_trigger] -> {'to'} = $1;
            } elsif($line =~ /^Whether to deliver to options autosupport\.noteto addresses: (\w+)/) {
                $result_array[$entry_trigger] -> {'noteto'} = $1;
            } elsif($line =~ /^Additional content flag:\s+([0-9a-zA-Z\-]+)/) {
                $result_array[$entry_trigger] -> {'additional-content'} = $1;
            } elsif($line =~ /^Additional subsystems reporting basic info:\s+([0-9a-zA-Z\-]+)/) {
                $result_array[$entry_trigger] -> {'basic-additional'} = $2;
            } elsif($line =~ /^Additional subsystems reporting troubleshooting info:\s+([0-9a-zA-Z\-]+)/) {
                $result_array[$entry_trigger] -> {'troubleshooting-additional'} = $1;
            } elsif($line =~ /^The data collection time budget for this trigger \(in seconds\):\s+([0-9a-zA-Z\-]+)/) {
                $result_array[$entry_trigger] -> {'time-limit'} = $1;
            } elsif($line =~ /^Default subsystems reporting basic info: (.*)/) {
                $states = $1;
                $basic_default = 1;
                foreach my $state (split(/,/,$states)) {
                    $state =~ s/\s+//g;
                    $result_array[$entry_trigger] -> {'basic-default'} -> [$entry_basic] = $state;
                    $entry_basic++;
                    $basic_def_next_line = $entry_basic;
                }
                $entry_basic = 0;
            } elsif($line =~ /^Default subsystems reporting troubleshooting info: (.*)/)  {
                $states = $1;
                $troubleshooting_default = 1;
                foreach my $state (split(/,/,$states)) {
                    $state =~ s/\s+//g;
                    $result_array[$entry_trigger] -> {'troubleshooting-default'} -> [$entry_basic] = $state;
                    $entry_basic++;
                    $ts_def_next_line = $entry_basic;
                }
                $entry_basic = 0;
            } elsif(($line =~ /^([a-zA-Z\_\,\s+]+)$/) && ($basic_default == 1)){
                 $states = $1;
                 foreach my $state (split(/,/,$states)) {
                    $state =~ s/\s+//g;
                    $result_array[$entry_trigger] -> {'basic-default'} -> [$basic_def_next_line] = $state;
                    $basic_def_next_line++;
                }
            } elsif(($line =~ /^([a-zA-Z\_\,\s+]+)$/) && ($troubleshooting_default == 1)){
                 $states = $1;
                 foreach my $state (split(/,/,$states)) {
                    $state =~ s/\s+//g;
                    $result_array[$entry_trigger] -> {'troubleshooting-default'} -> [$ts_def_next_line] = $state;
                    $ts_def_next_line++;
                }
            }
        }
    } elsif($opt =~ /^basic$|^troubleshooting$/) {
        my $firstline = "\-+";
        my @lines = get_lines( raw_output => $raw_output, first_line => $firstline,
                           can_throw => 1, response_object => $resp );
        $name[0] = $opt;
        foreach my $line (@lines) {
            if($line =~ /^([a-zA-Z\.]+)\s+([a-zA-Z\,\_\s+]+)\s+([0-9a-zA-Z\-]+)/) {
            $states = $2;
            $entry_basic = 0;
            $result_array[$entry_trigger] -> {'autosupport-message'} = $1;
            my @hash = split(/,/,$states);
            foreach my $state (@hash) {
                $state =~ s/\s+//g;
                $result_array[$entry_trigger] -> {$name[0]."-"."default"} -> [$entry_basic]  = $state;
                 $entry_basic++;
            }
            $result_array[$entry_trigger] -> {$name[0]."-"."additional"} = $3;
            }
            $entry_trigger++;
        }
        return \@result_array;
    } else {
        my @lines = get_lines( raw_output => $raw_output,
                               can_throw => 0, response_object => $resp );
        my $head;
        foreach my $line (@lines) {
            if($line =~ /^autosupport-message\s+([0-9a-zA-Z\-\s+]+)$/) {
                $head = $1;
                foreach my $op (split(/\s+/,$head)) {
                $name[$first_line_var] = $op;
                $first_line_var++;
                }
            } elsif($line =~ /^(\d+) entries were displayed.$/) {
                $result_array[$entry_trigger] -> {'total-entries'} = $1;
                $entry_trigger++;
                last;
            } elsif($line =~ /^([0-9a-zA-Z\.]+)\s+(\w+)\s+(\w+)\s+([0-9a-zA-Z\-]+)$/) {
                $result_array[$entry_trigger] -> {'autosupport-message'} = $1;
                $result_array[$entry_trigger] -> {'to'} = $2;
                $result_array[$entry_trigger] -> {'note-to'} = $3;
                $result_array[$entry_trigger] -> {'additional-content'} = $4;
                $entry_trigger++;
            } elsif($line =~ /^([a-zA-Z0-9\.]+)$/) {
                $result_array[$entry_trigger] -> {'autosupport-message'} = $1;
                $entry_trigger++;
            } elsif($line =~ /^([a-zA-Z0-9\.\_]+)\s+([0-9\.a-zA-Z\,\_\s+\-]+)$/) {
                $states = $2;
                $result_array[$entry_trigger] -> {'autosupport-message'} = $1;
                my @arr = split(/\s+/,$states);
                for(my $k = 0;$arr[$k] ne "";$k++) {
                    if($arr[$k] =~ /,/) {
                        my @hash = split(/,/,$arr[$k]);
                        $entry_basic = 0;
                        foreach my $state (@hash) {
                            $state =~ s/\s+//g;
                            $name[$k] =~ s/\s+//g;
                            $result_array[$entry_trigger] -> {$name[$k]} -> [$entry_basic] = $state;
                            $entry_basic++;
                        }
                    } else {
                        $result_array[$entry_trigger] -> {$name[$k]} = $arr[$k];
                    }
                }
                $entry_trigger++;
            }
        }
    }
    return \@result_array;
} ## end sub autosupport_trigger_show

1;
