#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode bootargs commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode bootargs commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "bootargs set" will be bootargs_set()
##
## @synopsis
##    use bootargs;
##
##    bootargs_set(%args);
##    bootargs_get(%args);
##    bootargs_unset(%args);
##    bootargs_print(%args);
##    bootargs_dump(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::bootargs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::bootargs_set
##
## @summary Parses the raw output from the "bootargs set" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootargs_set with any of its options as described in the 
##    corresponding cdef file 'bootargs.cdefs'
## @synopsis
##      bootargs_set(%args);
## INPUTS TO METHOD CALL 'bootargs_set' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'arg' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub bootargs_set {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    my ($line) = ($raw_output =~ /^(.+\s==\s.+)$/m);
    unless (defined $line) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response
        );
    }
    $result = _parse_output(
        raw_output      => $line,
        command         => $command_line,
        response_object => $response
    );
    return $result;
}

## @name ::bootargs_unset
##
## @summary Parses the raw output from the "bootargs unset" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootargs_unset with any of its options as described in the 
##    corresponding cdef file 'bootargs.cdefs'
## @synopsis
##      bootargs_unset(%args);
## INPUTS TO METHOD CALL 'bootargs_unset' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'arg' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub bootargs_unset {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # This command gives no output and hence return empty array reference
    # if there is no error in command execution.
    return [];
}

## @name ::bootargs_print
##
## @summary Parses the raw output from the "bootargs print" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootargs_print with any of its options as described in the 
##    corresponding cdef file 'bootargs.cdefs'
## @synopsis
##      bootargs_print(%args);
## INPUTS TO METHOD CALL 'bootargs_print' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'arg' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub bootargs_print {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    my ($line) = ($raw_output =~ /^(.+\s==\s.+)$/m);
    unless (defined $line) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response
        );
    }
    $result = _parse_output(
        raw_output      => $line,
        command         => $command_line,
        response_object => $response
    );
    return $result;
}

## @name ::bootargs_get
##
## @summary Parses the raw output from the "bootargs get" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootargs_get with any of its options as described in the 
##    corresponding cdef file 'bootargs.cdefs'
## @synopsis
##      bootargs_get(%args);
## INPUTS TO METHOD CALL 'bootargs_get' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'arg' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub bootargs_get {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    my ($line) = ($raw_output =~ /^(.+\s==\s.+)$/m);
    unless (defined $line) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response
        );
    }
    $result = _parse_output(
        raw_output      => $line,
        command         => $command_line,
        response_object => $response
    );
    return $result;
}

## @name ::bootargs_dump
##
## @summary Parses the raw output from the "bootargs dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootargs_dump with any of its options as described in the 
##    corresponding cdef file 'bootargs.cdefs'
## @synopsis
##      bootargs_dump(%args);
## INPUTS TO METHOD CALL 'bootargs_dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub bootargs_dump {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _parse_output(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
}

# Common routine which can parse the output of bootargs commands.
sub _parse_output {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    
    my @arr = ();
    if ($command =~ /^bootargs dump/) {
        @arr  = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => 'Printing boot options',
            "can_throw"       => 1,
            "response_object" => $response
        );
    } else {
        # For bootargs set, print and get, we would have already processed
        # and will be sending the required one line to be parsed in 
        # 'raw_output' field.
        $arr[0] = $raw_output;
    }
    
    my %hash;
    # Skip the first line if it says "Printing boot options:".
    $arr[0] = trim_string(string => $arr[0]);
    shift (@arr) if ($arr[0] =~ /^Printing boot options:$/);
    # Now we can parse
    foreach my $line (@arr) {
        $line = trim_string(string => $line);
        if ($line =~ /^(.+)\s==\s(.+)$/) {
            my ($key, $value) = ($1, $2);
            $hash{$key} = $value;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text => "Unexpected Output line while parsing:\n$line",
                response_object => $response
            );
        }           
    }
    return [\%hash];
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if ( $raw_output =~ /USAGE:/mi ) {
        $error = "CommandFailed";
        $$error_message
            = "Command failed due to dependencies or wrong usage";
        return $error;
    }
    return undef;
}

1;
