#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/cdpd.pm# $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode cdpd command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode cdpd command.
##
## @synopsis
##    use cdpd;
##
##
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::cdpd;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /[Uu]sage:/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    }
    return $error;
}
## @name ::cdpd
##
## @summary Parses the raw output from the "cdpd" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cdpd with any of its options as described in the
##    corresponding cdef file 'cdpd.cdefs'
## @synopsis
##      cdpd(%args);
## INPUTS TO METHOD CALL 'cdpd' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> 'zero-stats' </td> <td> 'zero-stats' </td> <td> '1' </td> </tr>
## <tr> <td> 'show-stats' </td> <td> 'show-stats' </td> <td> '1' </td> </tr>
## <tr> <td> 'show-neighbors' </td> <td> 'show-neighbors' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; cdpd show-stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'receive' => [
##&nbsp;                           {
##&nbsp;                             'malformed' => '0',
##&nbsp;                             'packets' => '28',
##&nbsp;                             'mem_alloc_fails' => '0',
##&nbsp;                             'csum_errors' => '0',
##&nbsp;                             'invalid_length' => '0',
##&nbsp;                             'cache_overflow' => '0',
##&nbsp;                             'other_errors' => '0',
##&nbsp;                             'unsupported_vers' => '10',
##&nbsp;                             'missing_tlvs' => '0'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'other' => [
##&nbsp;                         {
##&nbsp;                           'initfailures' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'transmit' => [
##&nbsp;                            {
##&nbsp;                              'mem_alloc_fails' => '0',
##&nbsp;                              'no_hostname' => '0',
##&nbsp;                              'xmit_fails' => '0',
##&nbsp;                              'packets' => '8',
##&nbsp;                              'other_errors' => '0',
##&nbsp;                              'packet_truncated' => '0'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; cdpd show-neighbors -v; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => [
##&nbsp;                           'CiscoInternetworkOperatingSystemSoftware'
##&nbsp;                         ],
##&nbsp;            'copyright' => '1986-2006 by cisco Systems, Inc.',
##&nbsp;            'local_port' => 'e0b',
##&nbsp;            'remote_port' => 'GigabitEthernet0/7',
##&nbsp;            'hold_time' => '132',
##&nbsp;            'entry_address(es)' => '192.168.70.128',
##&nbsp;            'compiled' => 'Fri 01-Dec-06 17:59 by weiliu',
##&nbsp;            'capabilities' => [
##&nbsp;                                'Switch',
##&nbsp;                                'IGMP'
##&nbsp;                              ],
##&nbsp;            'platform' => 'cisco OS-CIGESM-18TT-EBU',
##&nbsp;            'remote_device' => 'VSIMBC-11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'technical_support' => 'http://www.cisco.com/techsupport',
##&nbsp;            'version' => [
##&nbsp;                           'CiscoIOSSoftware',
##&nbsp;                           'Catalyst4500L3SwitchSoftware(cat4500-ENTSERVICES-M)',
##&nbsp;                           'Version12.2(53)SG3',
##&nbsp;                           'RELEASESOFTWARE(fc1)'
##&nbsp;                         ],
##&nbsp;            'copyright' => '1986-2010 by Cisco Systems, Inc.',
##&nbsp;            'local_port' => 'e0c',
##&nbsp;            'remote_port' => 'GigabitEthernet1/2',
##&nbsp;            'hold_time' => '175',
##&nbsp;            'entry_address(es)' => '10.72.88.56',
##&nbsp;            'compiled' => 'Fri 13-Aug-10 21:30 by prod_rel_team',
##&nbsp;            'capabilities' => [
##&nbsp;                                'Switch',
##&nbsp;                                'IGMP'
##&nbsp;                              ],
##&nbsp;            'platform' => 'cisco WS-C4948',
##&nbsp;            'remote_device' => 'NBDC-EGL-4F-F20-AC05'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; cdpd show-neighbors; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'remote_platform' => 'cisco OS-CIGESM.',
##&nbsp;            'remote_interface' => 'GigabitEthernet0/7',
##&nbsp;            'remote_capability' => 'SI',
##&nbsp;            'local_port' => 'e0b',
##&nbsp;            'hold_time' => '132',
##&nbsp;            'remote_device' => 'VSIMBC-11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'remote_platform' => 'cisco WS-C4948',
##&nbsp;            'remote_interface' => 'GigabitEthernet1/2',
##&nbsp;            'remote_capability' => 'SI',
##&nbsp;            'local_port' => 'e0c',
##&nbsp;            'hold_time' => '174',
##&nbsp;            'remote_device' => 'NBDC-EGL-4F-F2.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; cdpd zero-stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub cdpd {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'cdpd',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array = ();
    my ($i, $j, $name);
    my ($key1,$key2,$key3,$value1,$value2,$value3,$states);
    $i = 0;
    $j = 0;
    if($command =~ /show-stats/) {
        my @lines = get_lines( raw_output => $raw_output,
	                       can_throw => 0, response_object => $resp );
        foreach my $line (@lines) {
            $name = "receive" if($line =~ /^RECEIVE$/);
            $name = "transmit" if($line =~ /^TRANSMIT$/);
            $name = "other" if($line =~ /^OTHER$/);
            if($line =~ /^([a-zA-Z\s0-9]+)\:\s+(\w+)\s+\|\s+([a-zA-Z\s0-9]+)\:\s+(\w+)\s+\|\s+([a-zA-Z\s0-9]+)\:(\s+)?(\w+)$/) {
                $key1 = $1;
                $key2 = $3;
                $key3 = $5;
                $value1 = $2;
                $value2 = $4;
                $value3 = $7;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
            } elsif($line =~ /^([a-zA-Z\s0-9]+)\:\s+(\w+)$/) {
                $key1 = $1;
                $value1 = $2;
                $key1 =~ s/([(]|[)]|\s+)//g;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
            } elsif($line =~ /^([a-zA-Z\s0-9]+)\:\s+(\w+)\s+.\s([a-zA-Z\s0-9]+)\:\s+(\w+)$/) {
                $key1 = $1;
                $key2 = $3;
                $value1 = $2;
                $value2 = $4;
                $key1 =~ s/([(]|[)]|\s+)//g;
                $key2 =~ s/([(]|[)]|\s+)//g;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
                $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
            }
        }
    } elsif($command =~ /show-neighbors -v$/) {
        my @lines = get_lines( raw_output => $raw_output,
	                       can_throw => 0, response_object => $resp );
        $i = -1;
        foreach my $line (@lines) {
            if($line =~ /^([0-9\.]+)$/) {
                $result_array[$i] -> {"entry_address(es)"} = $1;
            } elsif(($line =~ /^(Capabilities):\s+(.*)$/) || ($line =~ /^(Version):\s+(.*)$/)){
                $states = $2;
				$key1 = $1;
                $j = 0;
                my @hash = split(/,/,$states);
			    foreach my $state (@hash) {
                    $state =~ s/\s+//g;
                    $result_array[$i] -> {to_key_name(item => $key1)} -> [$j]  = $state;
                    $j++;
			    }
            } elsif ($line =~ /^(.*):\s+(.*)/) {
                $key1 = $1;
                $value1 = $2;
                $i++ if ($key1 =~/Remote Device/);
                $result_array[$i] -> {to_key_name(item => $key1)}   = $value1;
            } elsif($line =~ /^Compiled\s+(.*)$/) {
                $result_array[$i] -> {"compiled"} = $1;
            }
        }
    } elsif($command =~ /show-neighbors$/) {
        my $firstline = "\-+";
        my @lines = get_lines( raw_output => $raw_output, first_line => $firstline,
	                       can_throw => 1, response_object => $resp );
        foreach my $line (@lines) {
            if($line =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\w+\s?[\w\-\.]+)\s+(\S+)\s+(\S+)/) {
                $result_array[$i] -> {"local_port"} = $1;
                $result_array[$i] -> {"remote_device"} = $2;
                $result_array[$i] -> {"remote_interface"} = $3;
                $result_array[$i] -> {"remote_platform"} = $4;
                $result_array[$i] -> {"hold_time"} = $5;
                $result_array[$i] -> {"remote_capability"} = $6;
                $i++;
            }
        }
    }
   return \@result_array;
}
1;
