# $Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode cf commands
##
## @description
##    This is a library to parse the raw output of 7Mode cf commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "cf enable" will be cf_enable()
##
## @synopsis
##    use cf;
##
##    cf_enable(%args);
##    cf_disable(%args);
##    cf_monitor(%args);
##    cf_partner(%args);
##    cf_status(%args);
##    cf_timers(%args);
##    cf_stat_times(%args);
##    cf_giveback(%args);
##    cf_takeover(%args);
##    cf_checkstate(%args);
##    cf_forcegiveback(%args);
##    cf_forcetakeover(%args);
##    cf_hw_assist_test(%args);
##    cf_hw_assist_stats_clear(%args);
##    cf_hw_assist_stats(%args);
##    cf_hw_assist_status(%args);
##    cf_readfw(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::cf;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors {
    my %args         = @_;
    my $raw_output   = $args{raw_output};
    my $command      = $args{command};
    my $errormsg_ptr = $args{'errormsg_ptr'};

    if ( $raw_output =~ /cf: not initialized yet!/ ) {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /cf: takeover cannot be performed/ ) {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /cf: not in takeover state!/ ) {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /Unable to read from partner, giveback cancelled/ ) {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    elsif ( $raw_output =~
        /cf: use of non-disruptive takeover option is not needed/ )
    {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /usage/i ) {
        $$errormsg_ptr = $command . " Invalid Parameter Passed";
        return "InvalidParam";
    }
    elsif ( $raw_output =~ /forcetakeover: Invalid option/ ) {
        $$errormsg_ptr = $raw_output;
        return "InvalidParam";
    }
    return undef;
} ## end sub _parse_for_errors

sub _license(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my @returnarray;
    my $resp = $args{'response_object'};
    while ( my $line = shift(@lines) ) {
        if ( $line =~ /^(\S*) (.*)$/ ) {
            my %hash;
            $hash{$1} = $2;
            $hash{"service"} = $1;
            $hash{"is_licensed"} =
              ( $line =~ /not licensed/ ) ? "false" : "true";
            $hash{"is_site"} = ( $line =~ /site/i ) ? "true" : "false";
            $hash{"is_demo"} = ( $line =~ /demo/i ) ? "true" : "false";
            if ( ( $hash{"is_licensed"} eq "false" ) || ( $line =~ /ENABLED/ ) )
            {
                $hash{"license_code"} = "";
            }
            else {
                $line =~ /(\w+)$/;
                $hash{"license_code"} = $1;
            }
            push( @returnarray, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $resp
            );
        }
    } ## end while (my $line = shift(@lines...
    return \@returnarray;
} ## end sub _license(%)
## @name ::cf_enable
##
## @summary Parses the raw output from the "cf enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_enable with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_enable(%args);
## INPUTS TO METHOD CALL 'cf_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub cf_enable(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line;
    my $can_throw = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    #end of funtion cf_enable()

## @name ::cf_disable
##
## @summary Parses the raw output from the "cf disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_disable with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_disable(%args);
## INPUTS TO METHOD CALL 'cf_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub cf_disable(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line;
    my $can_throw = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    #end of funtion cf_disable()

## @name ::cf_timers
##
## @summary Parses the raw output from the "cf timers" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_timers with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_timers(%args);
## INPUTS TO METHOD CALL 'cf_timers' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cf timers
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'timer(secs)' => 'takeover timeout (disk)',
##&nbsp;            'when(GMT)' => '',
##&nbsp;            'longest_observed' => '0',
##&nbsp;            'setting' => '10'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'timer(secs)' => 'takeover timeout (IC)         15',
##&nbsp;            'when(GMT)' => 'Jan1970 00:00:00',
##&nbsp;            'longest_observed' => '01',
##&nbsp;            'setting' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'timer(secs)' => 'reboot timeout',
##&nbsp;            'when(GMT)' => '',
##&nbsp;            'longest_observed' => '0',
##&nbsp;            'setting' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'timer(secs)' => 'coredump timeout',
##&nbsp;            'when(GMT)' => '',
##&nbsp;            'longest_observed' => '0',
##&nbsp;            'setting' => '60'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'timer(secs)' => 'operator response timeout',
##&nbsp;            'when(GMT)' => '',
##&nbsp;            'longest_observed' => '0',
##&nbsp;            'setting' => '600'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_timers(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = 'timer \(secs\)';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return _cf_timers( 'lines' => \@lines, 'response_object' => $resp );
    }
}    #end of funtion cf_timers()

sub _cf_timers(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my @returnarray;
    my $resp = $args{'response_object'};
    shift(@lines);    #remove the heading
    while ( my $line = shift(@lines) ) {
        print $line. "\n";
        if ( $line =~ /(.*)\s+(\d+)\s+(\d+)(?:\s+)?(.*)?/ ) {
            my %hash;
            $hash{'timer(secs)'} = $1;
            $hash{'timer(secs)'} =
              trim_string( 'string' => $hash{'timer(secs)'} );
            $hash{'setting'}          = $2;
            $hash{'longest_observed'} = $3;
            $hash{'when(GMT)'}        = $4;
            push( @returnarray, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $resp
            );
        }
    } ## end while (my $line = shift(@lines...
    return \@returnarray;
} ## end sub _cf_timers(%)

## @name ::cf_status
##
## @summary Parses the raw output from the "cf status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_status with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_status(%args);
## INPUTS TO METHOD CALL 'cf_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'time-master-slave' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; cf status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Controller Failover enabled, csiqa-2040-1b is up.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'partner_status' => 'up',
##&nbsp;            'cluster_monitor' => 'enabled',
##&nbsp;            'partner' => 'csiqa-2040-1b'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_status(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = '';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        my %hash;
        my @reason;
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /^Controller Failover (disabled)\.$/ ) {
                $hash{'cluster_monitor'} = $1;
            }
            elsif (( $line =~ /Interconnect status\:\s*(.+)\./ )
                || ( $line =~ /VIA Interconnect is (\S+).*/ ) )
            {
                $hash{'interconnect_status'} = $1;
            }
            elsif (( $line =~ /cluster monitor (.*)\./ )
                || ( $line =~ /Controller Failover (.*)\, (.*) is (\w*)[.]?/ ) )
            {
                $hash{'cluster_monitor'} = $1;
                $hash{'partner'}         = $2;
                $hash{'partner_status'}  = $3;
            }
            elsif ( $line =~ /Partner is (.*)/ ) {
                $hash{'partner'} = $1;
            }
            elsif ( $line =~
/(\S+)\s([is]+?|[may be]+?)\s(\S+)\,\stakeover\s(\S+).*reason\s\((.*)\)/
              )
            {
                push( @reason, $5 );
                $hash{'reason'}         = \@reason;
                $hash{'partner'}        = $1;
                $hash{'partner_status'} = $3;
            }
            elsif (( $line =~ /\S+\shas been taken over by\s(.*)./ )
                || ( $line =~ /\S+\shas taken over\s(.*)./ ) )
            {
                $hash{partner} = $1;
                $hash{'possible'} = "true";
            }
            elsif ( $line =~ /.*(operator initiated cf takeover).*/ ) {
                $hash{'possible'} = "true";
                push( @reason, $1 );
                $hash{'reason'} = \@reason;
            }
            elsif ( $line =~ /\S+\shas disabled takeover by (\S+)\s\((.*)\)/ ) {
                $hash{'possible'} = "false";
                $hash{partner} = $1;
                push( @reason, $2 );
                $hash{'reason'} = \@reason;
            }
        }
        push( @returnarray, \%hash );
        return \@returnarray;
    } ## end else [ if ($error)
}    #end of funtion cf_status()

## @name ::cf_monitor
##
## @summary Parses the raw output from the "cf monitor" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_monitor with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_monitor(%args);
## INPUTS TO METHOD CALL 'cf_monitor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; cf monitor; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  current time: 20Feb2012 06:32:47
##&nbsp;  UP 50+12:18:39, partner 'csiqa-2040-16b', cluster monitor disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cluster_monitor' => 'disabled',
##&nbsp;            'current_time' => '20Feb2012 06:32:47',
##&nbsp;            'state' => 'UP',
##&nbsp;            'partner' => 'csiqa-2040-16b'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; cf monitor all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;cf: Current monitor status (20Feb2012 06:32:48):
##&nbsp;partner 'csiqa-2040-16b', VIA Interconnect is up (link up)
##&nbsp;HA Interconnect Device is Infiniband ( Mellanox Sinai )
##&nbsp;state UP, current time 4364320752, current sk_cycles 121769243594200, event CHECK_FSM, elem ChkMbValid (12)
##&nbsp;mirrorConsistencyRequired TRUE
##&nbsp;takeoverByPartner 0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>
##&nbsp;mirrorEnabled TRUE, lowMemory FALSE, memio UNINIT, killPackets TRUE
##&nbsp;degraded FALSE, reservePolicy ALWAYS_AFTER_TAKEOVER, resetDisks TRUE
##&nbsp;hw_assist status:
##&nbsp;	hw_assist Inactive on csiqa-2040-16a: csiqa-2040-16a not monitoring alerts from partner(csiqa-2040-16b)
##&nbsp;	hw_assist Inactive on csiqa-2040-16b: csiqa-2040-16b not monitoring alerts from partner csiqa-2040-16a
##&nbsp;timeouts:
##&nbsp;    fast 1000 last scheduled (in sk_msecs) 297870825384574976 (in sk_cycles) 11009726871737729025, slow 28351 last scheduled (in sk_msecs) 2500 (in sk_cycles) 4364320752, mailbox -1731577262, connect 28351
##&nbsp;    operator 600000, firmware 0 recvd (in sk_msecs) 15000 (in sk_cycles) 4364320752, dumpcore -1731447262
##&nbsp;    booting 300000 (recvd 0)
##&nbsp;    transit timer enabled TRUE, transit 600000 (last 18431481125739167744)
##&nbsp;Disk inventory mismatch:
##&nbsp;Mismatched disks on host(csiqa-2040-16a): 0a.23
##&nbsp;Mismatched disks on host(csiqa-2040-16b): <none>
##&nbsp;mailbox disks:
##&nbsp;Disk 0a.16 is a local mailbox disk
##&nbsp;Disk 0a.17 is a local mailbox disk
##&nbsp;Disk 0b.16 is a partner mailbox disk
##&nbsp;Disk 0b.17 is a partner mailbox disk
##&nbsp;primary state:
##&nbsp;	version 2, senderSysid 135092892
##&nbsp;	hbt major seq 1329719474 minor seq 127, node_status TAKEOVER_DISABLED
##&nbsp;	info 0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>
##&nbsp;	flags 0x0 <>
##&nbsp;	channel CHANNEL_MAILBOX, abs_time 1329719568, sk_time 4364320752, sk_cycles 121769181208440
##&nbsp;	channel_status 0
##&nbsp;	channel CHANNEL_IC, abs_time 1329719568, sk_time 4364320752, sk_cycles 121769181190870
##&nbsp;	channel_status 0
##&nbsp;	channel CHANNEL_NETWORK, abs_time 0, sk_time 0, sk_cycles 0
##&nbsp;	channel_status -1
##&nbsp;backup state:
##&nbsp;	version 2, senderSysid 135093024
##&nbsp;	hbt major seq 1329719474 minor seq 99292, node_status TAKEOVER_DISABLED
##&nbsp;	info 0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>
##&nbsp;	flags 0x0 <>
##&nbsp;	channel CHANNEL_MAILBOX, abs_time 1329719566, sk_time 4364318275, sk_cycles 121765053537370
##&nbsp;	channel_status 0
##&nbsp;	Channel Read Ctx:
##&nbsp;	version 2, senderSysid 135093024
##&nbsp;	hbt major seq 1329719474 minor seq 99290, node_status TAKEOVER_DISABLED
##&nbsp;	info 0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>
##&nbsp;	flags 0x0 <>
##&nbsp;	channel CHANNEL_IC, abs_time 1329719568, sk_time 4364320752, sk_cycles 121769181148450
##&nbsp;	channel_status 0
##&nbsp;	Channel Read Ctx:
##&nbsp;	version 2, senderSysid 135093024
##&nbsp;	hbt major seq 1329719474 minor seq 99292, node_status TAKEOVER_DISABLED
##&nbsp;	info 0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>
##&nbsp;	flags 0x0 <>
##&nbsp;	channel CHANNEL_NETWORK, abs_time 0, sk_time 0, sk_cycles 0
##&nbsp;	channel_status -1
##&nbsp;	Channel Read Ctx:
##&nbsp;	version 2, senderSysid 0
##&nbsp;	hbt major seq 0 minor seq 0, node_status UNKNOWN
##&nbsp;	info 0x0 <>
##&nbsp;	flags 0x0 <>
##&nbsp;takeoverState FT_NONE, takeoverString 'No takeover information'
##&nbsp;givebackState FT_NONE, givebackString 'No giveback information'
##&nbsp;givebackRetries 0, givebackRequested FALSE
##&nbsp;autoGivebackEnabled FALSE, autoGivebackWasDone FALSE, autoGivebackCifsStopping FALSE
##&nbsp;autoGivebackLastVetoCheck 0, autoGivebackAttemptsExceeded FALSE
##&nbsp;Maximum primary disk mailbox io times: normal = 1926, transition = 0
##&nbsp;Maximum backup disk mailbox io times: normal = 298, transition = 0
##&nbsp;Num times logs unsynced : 0
##&nbsp;Total system uptime: 72953522 msec
##&nbsp;  Sync state total time : 93927 msec
##&nbsp;  Sync state  Max  time : 92146 msec
##&nbsp;  Sync state  Min  time : 1781 msec
##&nbsp;HA enabled.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reserve_policy' => 'ALWAYS_AFTER_TAKEOVER',
##&nbsp;            'sync_state_max_time' => '92146 msec',
##&nbsp;            'mismatched_disks_on_host_csiqa_2040_16a' => '0a.23',
##&nbsp;            'state' => 'UP',
##&nbsp;            'primary_state' => [
##&nbsp;                                 {
##&nbsp;                                   'info' => '0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>',
##&nbsp;                                   'flags' => '0x0 <>',
##&nbsp;                                   'sender_sysid' => '135092892',
##&nbsp;                                   'node_status' => 'TAKEOVER_DISABLED',
##&nbsp;                                   'version' => '2',
##&nbsp;                                   'channel' => [
##&nbsp;                                                  {
##&nbsp;                                                    'channel_name' => 'CHANNEL_MAILBOX',
##&nbsp;                                                    'sk_cycles' => '121769181208440',
##&nbsp;                                                    'abs_time' => '1329719568',
##&nbsp;                                                    'channel_status' => '0',
##&nbsp;                                                    'sk_time' => '4364320752'
##&nbsp;                                                  },
##&nbsp;                                                  {
##&nbsp;                                                    'channel_name' => 'CHANNEL_IC',
##&nbsp;                                                    'sk_cycles' => '121769181190870',
##&nbsp;                                                    'abs_time' => '1329719568',
##&nbsp;                                                    'channel_status' => '0',
##&nbsp;                                                    'sk_time' => '4364320752'
##&nbsp;                                                  },
##&nbsp;                                                  {
##&nbsp;                                                    'channel_name' => 'CHANNEL_NETWORK',
##&nbsp;                                                    'sk_cycles' => '0',
##&nbsp;                                                    'abs_time' => '0',
##&nbsp;                                                    'channel_status' => '-1',
##&nbsp;                                                    'sk_time' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                                   'hbt_minor_seq' => '127',
##&nbsp;                                   'hbt_major_seq' => '1329719474'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'mirror_enabled' => 'TRUE',
##&nbsp;            'sync_state_total_time' => '93927 msec',
##&nbsp;            'ha_interconnect_device' => 'Infiniband ( Mellanox Sinai )',
##&nbsp;            'sync_state_min_time' => '1781 msec',
##&nbsp;            'giveback_retries' => '0',
##&nbsp;            'giveback_requested' => 'FALSE',
##&nbsp;            'maximum_primary_disk_mailbox_io_times' => [
##&nbsp;                                                         {
##&nbsp;                                                           'transition' => '0',
##&nbsp;                                                           'normal' => '1926'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'auto_giveback_was_done' => 'FALSE',
##&nbsp;            'maximum_backup_disk_mailbox_io_times' => [
##&nbsp;                                                        {
##&nbsp;                                                          'transition' => '0',
##&nbsp;                                                          'normal' => '298'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;            'event' => 'CHECK_FSM',
##&nbsp;            'giveback_state' => 'FT_NONE',
##&nbsp;            'mailbox_disks' => [
##&nbsp;                                 {
##&nbsp;                                   'disk' => '0a.16'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'disk' => '0a.17'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'disk' => '0b.16'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'disk' => '0b.17'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'current_monitor_status' => '20Feb2012 06:32:48',
##&nbsp;            'auto_giveback_attempts_exceeded' => 'FALSE',
##&nbsp;            'degraded' => 'FALSE',
##&nbsp;            'elem_chkmbvalid' => '12',
##&nbsp;            'partner' => 'csiqa-2040-16b',
##&nbsp;            'current_sk_cycles' => '121769243594200',
##&nbsp;            'backup_state' => [
##&nbsp;                                {
##&nbsp;                                  'info' => '0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>',
##&nbsp;                                  'flags' => '0x0 <>',
##&nbsp;                                  'sender_sysid' => '135093024',
##&nbsp;                                  'node_status' => 'TAKEOVER_DISABLED',
##&nbsp;                                  'version' => '2',
##&nbsp;                                  'sk_cycles' => '121765053537370',
##&nbsp;                                  'channel' => 'CHANNEL_MAILBOX',
##&nbsp;                                  'abs_time' => '1329719566',
##&nbsp;                                  'hbt_minor_seq' => '99292',
##&nbsp;                                  'channel_status' => '0',
##&nbsp;                                  'sk_time' => '4364318275',
##&nbsp;                                  'hbt_major_seq' => '1329719474',
##&nbsp;                                  'channel_read_ctx' => ''
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'info' => '0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>',
##&nbsp;                                  'flags' => '0x0 <>',
##&nbsp;                                  'sender_sysid' => '135093024',
##&nbsp;                                  'node_status' => 'TAKEOVER_DISABLED',
##&nbsp;                                  'version' => '2',
##&nbsp;                                  'sk_cycles' => '121769181148450',
##&nbsp;                                  'channel' => 'CHANNEL_IC',
##&nbsp;                                  'abs_time' => '1329719568',
##&nbsp;                                  'hbt_minor_seq' => '99290',
##&nbsp;                                  'channel_status' => '0',
##&nbsp;                                  'sk_time' => '4364320752',
##&nbsp;                                  'hbt_major_seq' => '1329719474',
##&nbsp;                                  'channel_read_ctx' => ''
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'info' => '0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>',
##&nbsp;                                  'flags' => '0x0 <>',
##&nbsp;                                  'sender_sysid' => '135093024',
##&nbsp;                                  'node_status' => 'TAKEOVER_DISABLED',
##&nbsp;                                  'version' => '2',
##&nbsp;                                  'sk_cycles' => '0',
##&nbsp;                                  'channel' => 'CHANNEL_NETWORK',
##&nbsp;                                  'abs_time' => '0',
##&nbsp;                                  'hbt_minor_seq' => '99292',
##&nbsp;                                  'channel_status' => '-1',
##&nbsp;                                  'sk_time' => '0',
##&nbsp;                                  'hbt_major_seq' => '1329719474',
##&nbsp;                                  'channel_read_ctx' => ''
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'info' => '0x0 <>',
##&nbsp;                                  'flags' => '0x0 <>',
##&nbsp;                                  'sender_sysid' => '0',
##&nbsp;                                  'node_status' => 'UNKNOWN',
##&nbsp;                                  'version' => '2',
##&nbsp;                                  'hbt_minor_seq' => '0',
##&nbsp;                                  'hbt_major_seq' => '0'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'total_system_uptime' => '72953522 msec',
##&nbsp;            'auto_giveback_cifs_stopping' => 'FALSE',
##&nbsp;            'current_time' => '4364320752',
##&nbsp;            'ha' => 'enabled',
##&nbsp;            'low_memory' => 'FALSE',
##&nbsp;            'hw_assist_status' => [
##&nbsp;                                    {
##&nbsp;                                      'hw_assist' => 'Inactive on csiqa-2040-16a'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'hw_assist' => 'Inactive on csiqa-2040-16b'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'timeouts' => [
##&nbsp;                            {
##&nbsp;                              'recvd_in_sk_msecs' => '11009726871737729025',
##&nbsp;                              'recvd_in_sk_cycles' => '28351',
##&nbsp;                              'firmware' => '297870825384574976',
##&nbsp;                              'operator' => '1000',
##&nbsp;                              'booting' => '300000',
##&nbsp;                              'transit' => '600000',
##&nbsp;                              'dumpcore' => '2500',
##&nbsp;                              'transit_timer_enabled' => 'TRUE',
##&nbsp;                              'mailbox' => '-1731577262',
##&nbsp;                              'fast' => [
##&nbsp;                                          {
##&nbsp;                                            'last_scheduled_in_sk_msecs' => '297870825384574976',
##&nbsp;                                            'last_scheduled_in_sk_cycles' => '11009726871737729025',
##&nbsp;                                            'fast' => '1000'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'slow' => [
##&nbsp;                                          {
##&nbsp;                                            'last_scheduled_in_sk_msecs' => '2500',
##&nbsp;                                            'last_scheduled_in_sk_cycles' => '4364320752',
##&nbsp;                                            'slow' => '28351'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'connect' => '28351'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'kill_packets' => 'TRUE',
##&nbsp;            'takeover_state' => 'FT_NONE',
##&nbsp;            'takeover_by_partner' => '0x2041 <NVRAM_DOWN,CLUSTER_DISABLE,TAKEOVER_ON_PANIC>',
##&nbsp;            'mismatched_disks_on_host_csiqa_2040_16b' => '<none>',
##&nbsp;            'reset_disks' => 'TRUE',
##&nbsp;            'takeover_string' => 'No takeover information',
##&nbsp;            'giveback_string' => 'No giveback information',
##&nbsp;            'mirror_consistency_required' => 'TRUE',
##&nbsp;            'auto_giveback_last_veto_check' => '0',
##&nbsp;            'num_times_logs_unsynced' => '0',
##&nbsp;            'memio' => 'UNINIT',
##&nbsp;            'via_interconnect' => 'up',
##&nbsp;            'auto_giveback_enabled' => 'FALSE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_monitor(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = 'current time';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    if ( $command =~ /all/ ) {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'can_throw'  => $can_throw
        );
        my @result;
        my %hash;
        my ( $key, $val, $i, $j );

        for ( $i = 0 ; $i <= $#lines ; $i++ ) {
            if ( $lines[$i] =~ /cf:.*\((.*)\)/ ) {
                $hash{current_monitor_status} = $1;
                $i++;
                $lines[$i] =~ /'(.*)'.*is\s+(\S+)/;
                $hash{partner}          = $1;
                $hash{via_interconnect} = $2;
                $i++;
                $lines[$i] =~ /is\s+(.*)/;
                $hash{ha_interconnect_device} = $1;
                $i++;
                $lines[$i] =~
/state\s+(.*),\s+current time\s(.*),\s+current sk_cycles\s+(.*),\s+event\s+(.*),\s+elem ChkMbValid\s+\((.*)\)/;
                $hash{state}             = $1;
                $hash{current_time}      = $2;
                $hash{current_sk_cycles} = $3;
                $hash{event}             = $4;
                $hash{elem_chkmbvalid}   = $5;
                $i++;
                $lines[$i] =~ /mirrorConsistencyRequired\s+(.*)/;
                $hash{mirror_consistency_required} = $1;
                $i++;
                $lines[$i] =~ /takeoverByPartner\s+(.*)/;
                $hash{takeover_by_partner} = $1;
                $i++;
                $lines[$i] =~
/mirrorEnabled\s+(.*),\s+lowMemory\s+(.*),\s+memio\s+(.*),\s+killPackets\s+(.*)/;
                $hash{mirror_enabled} = $1;
                $hash{low_memory}     = $2;
                $hash{memio}          = $3;
                $hash{kill_packets}   = $4;
                $i++;
                $lines[$i] =~
/degraded\s+(.*),\s+reservePolicy\s+(.*),\s+resetDisks\s+(.*)/;
                $hash{degraded}       = $1;
                $hash{reserve_policy} = $2;
                $hash{reset_disks}    = $3;
                $i                    = $i + 2;
                $lines[$i] =~ /hw_assist\s+(.*):/;
                $hash{hw_assist_status}->[0]->{hw_assist} = $1;
                $i++;
                $lines[$i] =~ /hw_assist\s+(.*):/;
                $hash{hw_assist_status}->[1]->{hw_assist} = $1;
                $i = $i + 2;
                $lines[$i] =~
/fast\s+(.*)\s+last scheduled \(in sk_msecs\)\s+(.*)\s+\(in sk_cycles\)\s+(.*),\s+slow\s+(.*)\s+last scheduled \(in sk_msecs\)\s+(.*)\s+\(in sk_cycles\)\s+(.*),\s+mailbox\s+(.*),\s+connect\s+(.*)/;
                $hash{timeouts}->[0]->{fast}->[0]->{fast} = $1;
                $hash{timeouts}->[0]->{fast}->[0]
                  ->{last_scheduled_in_sk_msecs} = $2;
                $hash{timeouts}->[0]->{fast}->[0]
                  ->{last_scheduled_in_sk_cycles} = $3;
                $hash{timeouts}->[0]->{slow}->[0]->{slow} = $4;
                $hash{timeouts}->[0]->{slow}->[0]
                  ->{last_scheduled_in_sk_msecs} = $5;
                $hash{timeouts}->[0]->{slow}->[0]
                  ->{last_scheduled_in_sk_cycles} = $6;
                $hash{timeouts}->[0]->{mailbox} = $7;
                $hash{timeouts}->[0]->{connect} = $8;
                $i++;
                $lines[$i] =~
/operator\s+(.*),\s+firmware\s(.*)\s+\(recvd in sk_msecs\)\s+(.*)\(in sk_cycles\)\s+(.*),\s+dumpcore\s+(.*)/;
                $hash{timeouts}->[0]->{operator}           = $1;
                $hash{timeouts}->[0]->{firmware}           = $2;
                $hash{timeouts}->[0]->{recvd_in_sk_msecs}  = $3;
                $hash{timeouts}->[0]->{recvd_in_sk_cycles} = $4;
                $hash{timeouts}->[0]->{dumpcore}           = $5;
                $i++;
                $lines[$i] =~ /booting\s+(\d+)/;
                $hash{timeouts}->[0]->{booting} = $1;
                $i++;
                $lines[$i] =~ /transit timer enabled\s(\w+),\stransit\s+(\d+)/;
                $hash{timeouts}->[0]->{transit_timer_enabled} = $1;
                $hash{timeouts}->[0]->{transit}               = $2;
            }
            elsif ( $lines[$i] =~ /(Mismatched disks.*):\s(.*)/ ) {
                $key = $1;
                $key =~ tr/-(/_ /;
                $hash{ to_key_name( item => $key ) } = $2;
            }
            elsif ( $lines[$i] =~ /(mailbox disks):/ ) {
                $j = 0;
                $i++;
                while (1) {
                    last if ( $lines[$i] =~ /(primary state):/ );
                    $lines[$i] =~ /Disk\s+(.*) is a .*/;
                    $hash{mailbox_disks}->[$j]->{disk} = $1;
                    $j++;
                    $i++;
                }
                if ( $lines[$i] =~ /(primary state):/ ) {
                    $i++;
                    $lines[$i] =~ /version\s+(\d+),\s+senderSysid\s+(.*)/;
                    $hash{primary_state}->[0]->{version}      = $1;
                    $hash{primary_state}->[0]->{sender_sysid} = $2;
                    $i++;
                    $lines[$i] =~
                      /hbt major seq (.*)\s+minor seq (.*), node_status (.*)/;
                    $hash{primary_state}->[0]->{hbt_major_seq} = $1;
                    $hash{primary_state}->[0]->{hbt_minor_seq} = $2;
                    $hash{primary_state}->[0]->{node_status}   = $3;
                    $i++;
                    $lines[$i] =~ /info\s+(.*)/;
                    $hash{primary_state}->[0]->{info} = $1;
                    $i++;
                    $lines[$i] =~ /flags\s+(.*)/;
                    $hash{primary_state}->[0]->{flags} = $1;
                    $i++;
                    if($lines[$i] =~ /headswap state (.*), old partner sysid (\d+), old local sysid (\d+)/) {
                        $hash{primary_state}->[0]->{headswap_state} = $1;
                        $hash{primary_state}->[0]->{old_partner_sysid} = $2;
                        $hash{primary_state}->[0]->{old_local_sysid} = $3;
                        $i++;
                    }
                    $j = 0;

                    while (1) {
                        last if ( $lines[$i] =~ /backup state:/ );
                        $lines[$i] =~
/channel\s+(\w+),\s+abs_time\s+(\d+),\s+sk_time\s+(\d+),\s+sk_cycles\s+(.*)/;
                        $hash{primary_state}->[0]->{channel}->[$j]
                          ->{channel_name} = $1;
                        $hash{primary_state}->[0]->{channel}->[$j]->{abs_time} =
                          $2;
                        $hash{primary_state}->[0]->{channel}->[$j]->{sk_time} =
                          $3;
                        $hash{primary_state}->[0]->{channel}->[$j]
                          ->{sk_cycles} = $4;
                        $i++;
                        $lines[$i] =~ /channel_status\s+(.*)/;
                        $hash{primary_state}->[0]->{channel}->[$j]
                          ->{channel_status} = $1;
                        $j++;
                        $i++;
                    }
                    if ( $lines[$i] =~ /backup state:/ ) {
                        $i++;
                        $j = 0;
                        while (1) {
                            last if ( $lines[$i] =~ /takeoverState.*/ );
                            $lines[$i] =~
                              /version\s+(\d+),\s+senderSysid\s+(.*)/;
                            $hash{backup_state}->[$j]->{version}      = $1;
                            $hash{backup_state}->[$j]->{sender_sysid} = $2;
                            $i++;
                            $lines[$i] =~
/hbt major seq (.*)\s+minor seq (.*), node_status (.*)/;
                            $hash{backup_state}->[$j]->{hbt_major_seq} = $1;
                            $hash{backup_state}->[$j]->{hbt_minor_seq} = $2;
                            $hash{backup_state}->[$j]->{node_status}   = $3;
                            $i++;
                            $lines[$i] =~ /info\s+(.*)/;
                            $hash{backup_state}->[$j]->{info} = $1;
                            $i++;
                            $lines[$i] =~ /flags\s+(.*)/;
                            $hash{backup_state}->[$j]->{flags} = $1;
                            $i++;
                            if($lines[$i] =~ /headswap state (.*), old partner sysid (\d+), old local sysid (\d+)/) {
                                $hash{backup_state}->[$j]->{headswap_state} = $1;
                                $hash{backup_state}->[$j]->{old_partner_sysid} = $2;
                                $hash{backup_state}->[$j]->{old_local_sysid} = $3;
                                $i++;
                            }
                            if ( $lines[$i] =~
/channel\s+(\w+),\s+abs_time\s+(\d+),\s+sk_time\s+(\d+),\s+sk_cycles\s+(.*)/
                              )
                            {
                                $hash{backup_state}->[$j]->{channel}   = $1;
                                $hash{backup_state}->[$j]->{abs_time}  = $2;
                                $hash{backup_state}->[$j]->{sk_time}   = $3;
                                $hash{backup_state}->[$j]->{sk_cycles} = $4;
                                $i++;
                            }
                            if ( $lines[$i] =~ /channel_status\s+(.*)/ ) {
                                $hash{backup_state}->[$j]->{channel_status} =
                                  $1;
                                $i++;
                            }
                            if ( $lines[$i] =~ /Channel Read Ctx:(.*)/ ) {
                                $hash{backup_state}->[$j]->{channel_read_ctx} =
                                  $1;
                                $i++;
                            }
                            $j++;
                        }
                        $i--;
                    }
                }
            }
            elsif (
                $lines[$i] =~ /takeoverState\s+(.*),\s+takeoverString\s'(.*)'/ )
            {
                $hash{takeover_state}  = $1;
                $hash{takeover_string} = $2;
            }
            elsif (
                $lines[$i] =~ /givebackState\s+(.*),\s+givebackString\s'(.*)'/ )
            {
                $hash{giveback_state}  = $1;
                $hash{giveback_string} = $2;
            }
            elsif ( $lines[$i] =~
                /givebackRetries\s+(.*),\s+givebackRequested\s+(.*)/ )
            {
                $hash{giveback_retries}   = $1;
                $hash{giveback_requested} = $2;
            }
            elsif ( $lines[$i] =~
/autoGivebackEnabled\s+(.*),\s+autoGivebackWasDone\s+(.*),\s+autoGivebackCifsStopping\s+(.*)/
              )
            {
                $hash{auto_giveback_enabled}       = $1;
                $hash{auto_giveback_was_done}      = $2;
                $hash{auto_giveback_cifs_stopping} = $3;
            }
            elsif ( $lines[$i] =~
/autoGivebackLastVetoCheck\s+(.*),\s+autoGivebackAttemptsExceeded\s+(.*)/
              )
            {
                $hash{auto_giveback_last_veto_check}   = $1;
                $hash{auto_giveback_attempts_exceeded} = $2;
            }
            elsif ( $lines[$i] =~
/(Maximum .* disk mailbox io times):\s+normal\s+=\s+(.*),\s+transition\s+=\s+(.*)/
              )
            {
                $key = to_key_name( item => $1 );
                $hash{$key}->[0]->{normal}     = $2;
                $hash{$key}->[0]->{transition} = $3;
                if ( $1 =~ /Maximum backup disk mailbox io times/ ) {
                    $i++;
                    while (shift(@lines)) {
                        last if ( $lines[$i] =~ /HA/ );
                        $lines[$i] =~ /(.*):\s(.*)/;
                        $key = $1;
                        $val = $2;
                        $key =~ s/\s+$//g;
                        $key =~ s/^\s+//g;
                        $key = to_key_name( item => $key );
                        $hash{$key} = $val;
                    }
                }
                if ( $lines[$i] =~ /HA\s+(.*)./ ) {
                    $hash{ha} = $1;
                }
            }
        }
        push( @result, \%hash );
        return \@result;
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        my %hash;
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /current time: (.*)/ ) {

                $hash{'current_time'} = $1;
            }
            elsif (
                $line =~ /(\S+).*partner\s['](.*)['].*monitor (.*)/ )
            {
                $hash{'partner'}         = $2;
                $hash{'cluster_monitor'} = $3;
                $hash{state}             = $1;
            }
        }
        push( @returnarray, \%hash );
        return \@returnarray;
    } ## end else [ if ($error)
}    #end of funtion cf_monitor()

## @name ::cf_partner
##
## @summary Parses the raw output from the "cf partner" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_partner with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_partner(%args);
## INPUTS TO METHOD CALL 'cf_partner' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cf partner
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'partner' => 'partner'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_partner(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = '';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        my %hash;
        if ( scalar(@lines) == 1 ) {
            $hash{'partner'} = $lines[0];
        }
        push( @returnarray, \%hash );
        return \@returnarray;
    } ## end else [ if ($error)
}    #end of funtion cf_monitor()

## @name ::cf_stat_times
##
## @summary Parses the raw output from the "cf stat times" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_stat_times with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_stat_times(%args);
## INPUTS TO METHOD CALL 'cf_stat_times' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cf stat times
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'giveback_stats' => [
##&nbsp;                                  {
##&nbsp;                                    'Number of Failures' => '0',
##&nbsp;                                    'Total number of operations' => '0'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'takeover_stats' => [
##&nbsp;                                  {
##&nbsp;                                    'Number of Failures' => '0',
##&nbsp;                                    'Total number of operations' => '0'
##&nbsp;                                  }
##&nbsp;                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_stat_times(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = 'TAKEOVER STATS';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return _cf_stat_times( 'lines' => \@lines, 'response_object' => $resp );
    }
}    #end of funtion cf_stat_times()

sub _cf_stat_times(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my @returnarray;
    my $resp = $args{'response_object'};
    my %temp_hash;
    while ( my $line = shift(@lines) ) {
        if ( $line =~ /TAKEOVER STATS/ ) {
            my %hash;
            shift(@lines);    #remove the dotted separator
            $line = shift(@lines);
            my ( $key, $value ) = split( /:/, $line );
            $value = trim_string( 'string' => $value );
            $hash{$key} = $value;
            $line = shift(@lines);
            ( $key, $value ) = split( /:/, $line );
            $value = trim_string( 'string' => $value );
            $hash{$key} = $value;
            push( @{ $temp_hash{'takeover_stats'} }, \%hash );
        }
        elsif ( $line =~ /GIVEBACK STATS/ ) {
            my %hash;
            shift(@lines);    #remove the dotted separator
            $line = shift(@lines);
            my ( $key, $value ) = split( /:/, $line );
            $value = trim_string( 'string' => $value );
            $hash{$key} = $value;
            $line = shift(@lines);
            ( $key, $value ) = split( /:/, $line );
            $value = trim_string( 'string' => $value );
            $hash{$key} = $value;
            push( @{ $temp_hash{'giveback_stats'} }, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $resp
            );
        }
    } ## end while (my $line = shift(@lines...
    push( @returnarray, \%temp_hash );
    return \@returnarray;
} ## end sub _cf_stat_times(%)

## @name ::cf_takeover
##
## @summary Parses the raw output from the "cf takeover" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_takeover with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_takeover(%args);
## INPUTS TO METHOD CALL 'cf_takeover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-n' </td> <td> 'version-mismatch' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples

sub cf_takeover(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}    #end of funtion cf_takeover()

## @name ::cf_giveback
##
## @summary Parses the raw output from the "cf giveback" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_giveback with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_giveback(%args);
## INPUTS TO METHOD CALL 'cf_giveback' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples

sub cf_giveback(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}    #end of funtion cf_giveback()

## @name ::cf_checkstate
##
## @summary Parses the raw output from the "cf checkstate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_checkstate with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_checkstate(%args);
## INPUTS TO METHOD CALL 'cf_checkstate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'expected-state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cf checkstate expectedstate
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cf_state' => 'unexpected',
##&nbsp;            'expectedstate' => 'REBOOTING'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cf checkstate REBOOTING
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cf_state' => 'expected'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_checkstate(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );

    my @result_array;
    my %info = ();
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    if ( $raw_output =~
        /.*(unexpected)\s+cf\s+state;\s+expected\s+\w+\s+\w+\s+(\w+)/i )
    {
        $info{"cf_state"}      = $1;
        $info{"expectedstate"} = $2;
    }
    elsif ( $raw_output eq "" ) {
        $info{"cf_state"} = "expected";
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }
    push( @result_array, \%info );

    return \@result_array;
}    #end of funtion cf_checkstate()

## @name ::cf_forcegiveback
##
## @summary Parses the raw output from the "cf forcegiveback" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_giveback with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_forcegiveback(%args);
## INPUTS TO METHOD CALL 'cf_forcegiveback' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples
##
##&nbsp;Command = cf forcegiveback -f
##&nbsp;$VAR1 = [];
##&nbsp;
sub cf_forcegiveback(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}    #end of funtion cf_forcegiveback()

## @name ::cf_forcetakeover
##
## @summary Parses the raw output from the "cf forcetakeover" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_giveback with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_forcetakeover(%args);
## INPUTS TO METHOD CALL 'cf_forcetakeover' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'danger' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples
##
##&nbsp;Command = cf forcetakeover -f
##&nbsp;$VAR1 = [];
##&nbsp;
sub cf_forcetakeover(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}    #end of funtion cf_forcetakeover()

## @name ::cf_hw_assist_test
##
## @summary Parses the raw output from the "cf_hw_assist_test" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_enable with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_enable(%args);
## INPUTS TO METHOD CALL 'cf_hwassist_test' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub cf_hw_assist_test(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line;
    my $can_throw = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    #end of funtion cf_hw_assist_test()

## @name ::cf_hw_assist_stats_clear
##
## @summary Parses the raw output from the "cf_hw_assist_stats_clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_enable with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_enable(%args);
## INPUTS TO METHOD CALL 'cf_hw_assist_stats_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub cf_hw_assist_stats_clear(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line;
    my $can_throw = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    #end of funtion cf_hw_assist_stats_clear()

## @name ::cf_hw_assist_stats
##
## @summary Parses the raw output from the "cf hw_assist stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_hw_assist_stats with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_hw_assist_stats(%args);
## INPUTS TO METHOD CALL 'cf_hw_assist_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; cf hw_assist stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'count' => '6',
##&nbsp;                          'alert_type' => 'system_down',
##&nbsp;                          'last_received' => 'Tue Aug  2 19:14:07 IST 2011',
##&nbsp;                          'takeover' => 'Yes',
##&nbsp;                          'alert_event' => 'power_cycle_via_rlm'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'count' => '28',
##&nbsp;                          'alert_type' => 'system_down',
##&nbsp;                          'last_received' => 'Tue Aug  2 19:46:30 IST 2011',
##&nbsp;                          'takeover' => 'No',
##&nbsp;                          'alert_event' => 'abnormal_reboot'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'count' => '8199',
##&nbsp;                          'alert_type' => 'keep_alive',
##&nbsp;                          'last_received' => 'Thu Aug  4 12:48:53 IST 2011',
##&nbsp;                          'takeover' => 'No',
##&nbsp;                          'alert_event' => 'periodic_message'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'partner' => 'f3170-cte3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_hw_assist_stats(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    my @lines = get_lines( 'raw_output' => $raw_output, 'can_throw' => 1 );
    my @result_array;
    my @heading;
    my $i = -1;
    foreach my $line (@lines) {
        if ( $line =~ /hw_assist alerts received from partner[(](.*)[)]/ ) {
            $result_array[0]->{"partner"} = $1;
        }
        elsif ( $line =~
            /alert\s+type\s+alert\s+event\s+count\s+takeover\s+last\s+received/
          )
        {
            @heading = split( /\s{2,}/, $line );
            next;
        }
        elsif ( $line =~ /----------.*/ ) {
            next;
        }
        elsif ( $line =~ /^(\w+)\s+(\w+)\s+(\d+)\s+(\w+)\s+(.*)$/ ) {
            $i++;
            $result_array[0]->{info}->[$i]
              ->{ to_key_name( item => $heading[0] ) } = $1;
            $result_array[0]->{info}->[$i]
              ->{ to_key_name( item => $heading[1] ) } = $2;
            $result_array[0]->{info}->[$i]
              ->{ to_key_name( item => $heading[2] ) } = $3;
            $result_array[0]->{info}->[$i]
              ->{ to_key_name( item => $heading[3] ) } = $4;
            $result_array[0]->{info}->[$i]
              ->{ to_key_name( item => $heading[4] ) } = $5;
        }
        else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "Output is not in the expected format",
                response_object => $resp
            );
        }
    }

    return \@result_array;
}    #end of funtion cf_hw_assist_stats()

## @name ::cf_hw_assist_status
##
## @summary Parses the raw output from the "cf hw_assist status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_hw_assist_status with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_hw_assist_status(%args);
## INPUTS TO METHOD CALL 'cf_hw_assist_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cf hw_assist status
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'alerts_from' => 'f3170-cte3',
##&nbsp;             'alerts_status' => 'monitoring',
##&nbsp;             'node_name' => 'f3170-cte4',
##&nbsp;             'status' => 'Active',
##&nbsp;             'ip_address' => '10.72.245.78',
##&nbsp;             'port' => '4444',
##&nbsp;             'node' => 'Local'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub cf_hw_assist_status(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $can_throw = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines =
      get_lines( 'raw_output' => $raw_output, 'can_throw' => $can_throw );
    my %hash;
    my @temp;
    my $type;
    my $name;
    my $i = -1;
    while ( my $line = shift(@lines) ) {

        if ( $line =~ /(\w+)\s+\w+\((\w+-\w+|\w+-\w+-\w+)\)\s+\w+:/ ) {
            $i++;
            $type = $1;
            $type =~ tr/[A-Z]/[a-z]/;
            $name          = "$type" . "_node";
            $hash{"$name"} = $2;
            $name          = "$type" . "_node_info";
        }
        elsif ( $line =~ /(\w+):\s+\S+\s+((\w+\s+)?monitoring).*[(](\S+)[)]/ ) {
            $hash{"$name"}->[$i]->{'status'}        = $1;
            $hash{"$name"}->[$i]->{'alerts_status'} = $2;
            $hash{"$name"}->[$i]->{'alerts_from'}   = $4;
        }
        elsif ( $line =~ /\D+(\d+)\D+(\S+)/ ) {
            $hash{"$name"}->[$i]->{'port'}       = $1;
            $hash{"$name"}->[$i]->{'ip_address'} = $2;
        }
        elsif ( $line =~ /(Can not read partner state)/ ) {
            $hash{"$name"}->[$i]->{'status'} = $1;
        }
        elsif ( $line =~ /(Interconnect is down between nodes)/ ) {
            $hash{"$name"}->[$i]->{'interconnect_state'} = 'down';
        }
        elsif ( $line =~ /(Throttling hw_assist alerts.)/ ) {
            $hash{"$name"}->[$i]->{'alerts'} = 'throttling';
        }
    }
    push( @temp, \%hash );
    return \@temp;

}    #end of funtion cf_hw_assist_status()
## @name ::cf_readfw
##
## @summary Parses the raw output from the "cf readfw" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cf_readfw with any of its options as described in the
##    corresponding cdef file 'cf.cdefs'
## @synopsis
##      cf_readfw(%args);
## INPUTS TO METHOD CALL 'cf_readfw' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cf readfw
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'local_state' => 'SF_UP',
##&nbsp;            'partner_state' => 'SF_UP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cf_readfw(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = '';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @returnarray;

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        my @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        my %hash;
        while ( my $line = shift(@lines) ) {
            if (
                (
                    $line =~
/cf: LOCAL: state (.*), progress \d+, cf: PARTNER: state (.*), progress \d+,/
                )
                || ( $line =~ /cf: LOCAL: state (.*), progress \d+, cf: (.*)/ )
              )
            {
                $hash{'local_state'}   = $1;
                $hash{'partner_state'} = $2;
            }
        }
        push( @returnarray, \%hash );
        return \@returnarray;
    } ## end else [ if ($error)
}    #end of funtion cf_readfw()

1;
