#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode cifs commands
##
## @description
##    This is a library to parse the raw output of 7Mode cifs commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "cifs terminate" will be cifs_terminate()
##
## @synopsis
##    use cifs;
##
##    cifs_terminate(%args);
##    cifs_restart (%args);
##    cifs_shares (%args);
##    cifs_setup (%args);
##    cifs_prefdc_print (%args);
##    cifs_prefdc_add (%args);
##    cifs_prefdc_delete (%args);
##    cifs_cifs_gpupdate (%args);
##    cifs_cifs_gpresult (%args);
##    cifs_access (%args);
##    cifs_changefilerpwd (%args);
##    cifs_resetdc (%args);
##    cifs_domaininfo(%args);
##    cifs_comment(%args);
##    cifs_nbalias(%args);
##    cifs_homedir(%args);
##    cifs_branchcache_hash_stat(%args);
##    cifs_branchcache_set_key(%args);
##    cifs_stat(%args);
##
##
## @author makshay@netapp.com, bhaskarm@netapp.com, kathar.hidayath@netapp.com,
##         skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::cifs;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output
        =~ /(CIFS is already running|CIFS not running|CIFS is not licensed|CIFS Setup can not be run while CIFS is active)/i
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /CIFS Setup is not supported from RSH|The system is not configured to run this command/
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /(is not connected)|(not find|There is already a share named)/i ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /(Multiple workstations specified)|(does not exist)|(No share is matching (that|the) name)|(Illegal IP address.*)|(.*is not a member of a domain.)|(No preferred Domain Controllers configured.)|(Unknown Unix group.*|Unknown user\/group.*|Invalid rights.*)/i
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /[Uu]sage:\s|Invalid arguments to CIFS Setup/ ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(no CIFS home directory|no NetBIOS aliases)/i )
    {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif (
        $raw_output =~ /(No\smatching\sclients\sfound.*)|(Invalid\suser.*)/i )
    {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /(There is no share named.*)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    }
    return $error;
} ## end sub _check_error

## @name ::cifs_setup
##
## @summary Parses the raw output from the "cifs setup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_setup. This command is an interactive command,
##    and hence is just for error checking.
## @synopsis
##      cifs_setup();
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples

##&nbsp;Command = cifs setup
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_setup {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = undef;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cifs_setup

## @name ::cifs_terminate
##
## @summary Parses the raw output from the "cifs terminate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_terminate with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_terminate(%args);
## INPUTS TO METHOD CALL 'cifs_terminate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'wsname' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ipaddress' </td> </tr>
## <tr> <td> '-t' </td> <td> 'interval' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs terminate
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_terminate {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cifs_terminate

## @name ::cifs_restart
##
## @summary Parses the raw output from the "cifs restart" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_restart with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_restart(%args);
## INPUTS TO METHOD CALL 'cifs_restart' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs restart
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_restart {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cifs_restart

## @name ::cifs_shares
##
## @summary Parses the raw output from the "cifs shares" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_shares with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_shares(%args);
## INPUTS TO METHOD CALL 'cifs_shares' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-noaccessbasedenum' </td> <td> 'noaccessbasedenum' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'cifs-shares-defined' </td> <td> '1' </td> </tr>
## <tr> <td> '-novscan' </td> <td> 'novscan' </td> <td> '1' </td> </tr>
## <tr> <td> '-nocomment' </td> <td> 'nocomment' </td> <td> '1' </td> </tr>
## <tr> <td> '-nofile_umask' </td> <td> 'nofile-umask' </td> <td> '0' </td> </tr>
## <tr> <td> '-change' </td> <td> 'change' </td> <td> '1' </td> </tr>
## <tr> <td> '-maxusers' </td> <td> 'maxusers' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'sharename' </td> <td> '0' </td> </tr>
## <tr> <td> '-manual_caching' </td> <td> 'manual-caching' </td> <td> '1' </td> </tr>
## <tr> <td> '-vscan' </td> <td> 'vscan' </td> <td> '1' </td> </tr>
## <tr> <td> '-namespace_caching' </td> <td> 'namespace-caching' </td> <td> '1' </td> </tr>
## <tr> <td> '-auto_program_caching' </td> <td> 'auto-program-caching' </td> <td> '1' </td> </tr>
## <tr> <td> '-no_caching' </td> <td> 'no-caching' </td> <td> '1' </td> </tr>
## <tr> <td> '-delete' </td> <td> 'delete' </td> <td> '1' </td> </tr>
## <tr> <td> '-accessbasedenum' </td> <td> 'accessbasedenum' </td> <td> '1' </td> </tr>
## <tr> <td> '-browse' </td> <td> 'browse' </td> <td> '1' </td> </tr>
## <tr> <td> '-add' </td> <td> 'add' </td> <td> '1' </td> </tr>
## <tr> <td> '-noscanread' </td> <td> 'novscanread' </td> <td> '1' </td> </tr>
## <tr> <td> '-file_umask' </td> <td> 'file-umask' </td> <td> '0' </td> </tr>
## <tr> <td> '-noumask' </td> <td> 'noumask' </td> <td> '1' </td> </tr>
## <tr> <td> '-nosymlink_strict_security' </td> <td> 'nosymlink-strict-security' </td> <td> '1' </td> </tr>
## <tr> <td> '-noforcegroup' </td> <td> 'noforcegroup' </td> <td> '1' </td> </tr>
## <tr> <td> '-widelink' </td> <td> 'widelink' </td> <td> '1' </td> </tr>
## <tr> <td> '-symlink_strict_security' </td> <td> 'symlink-strict-security' </td> <td> '1' </td> </tr>
## <tr> <td> '-nobrowse' </td> <td> 'nobrowse' </td> <td> '1' </td> </tr>
## <tr> <td> '-dir_umask' </td> <td> 'dir-umask' </td> <td> '0' </td> </tr>
## <tr> <td> '-nodir_umask' </td> <td> 'nodir-umask' </td> <td> '1' </td> </tr>
## <tr> <td> '-no_namespace_caching' </td> <td> 'no-namespace-caching' </td> <td> '0' </td> </tr>
## <tr> <td> '-auto_document_caching' </td> <td> 'auto-document-caching' </td> <td> '1' </td> </tr>
## <tr> <td> '-nowidelink' </td> <td> 'nowidelink' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-nomaxusers' </td> <td> 'nomaxusers' </td> <td> '1' </td> </tr>
## <tr> <td> '-comment' </td> <td> 'comment' </td> <td> '0' </td> </tr>
## <tr> <td> '-forcegroup' </td> <td> 'forcegroup' </td> <td> '0' </td> </tr>
## <tr> <td> '-umask' </td> <td> 'umask' </td> <td> '0' </td> </tr>
## <tr> <td> '-vscanread' </td> <td> 'vscanread' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs shares
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'Mount Point' => '/etc',
##&nbsp;            'Name' => 'ETC$',
##&nbsp;            'Description' => ' Remote Administration',
##&nbsp;            'other_settings' => [
##&nbsp;                                  'BUILTIN\\Administrators / Full Control'
##&nbsp;                                ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Mount Point' => '/vol/vol0/home',
##&nbsp;            'Name' => 'HOME',
##&nbsp;            'Description' => ' Default Share',
##&nbsp;            'other_settings' => [
##&nbsp;                                  'everyone / Full Control'
##&nbsp;                                ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Mount Point' => '/',
##&nbsp;            'Name' => 'C$',
##&nbsp;            'Description' => ' Remote Administration',
##&nbsp;            'other_settings' => [
##&nbsp;                                  'BUILTIN\\Administrators / Full Control'
##&nbsp;                                ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'Mount Point' => '/vol/vol0/testqtree1',
##&nbsp;            'Description' => undef,
##&nbsp;            'Name' => 'testqtree1',
##&nbsp;            'other_settings' => [
##&nbsp;                                  'user limit=5',
##&nbsp;                                  'widelinks supported',
##&nbsp;                                  'not visible to browsers',
##&nbsp;                                  'strict symlink security disabled',
##&nbsp;                                  'everyone / Full Control'
##&nbsp;                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = cifs shares -t
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'Number of CIFS shares defined' => ' 4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cifs_shares {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $command =~ /-t/ ) {
        if ( $output =~ /(.+):(.+)/ ) {
            my $hash = { $1 => $2 };
            push @result_array, $hash;
        }
    } elsif ( $command !~ /(-add)|(-change)|(-delete)/ ) {
        my $first_line = '(?i)name\s*mount\s*point\s*description';
        my @lines      = get_lines(
            raw_output      => $output,
            first_line      => $first_line,
            response_object => $resp
        );

        @result_array = @{ _table_parser( \@lines ) };
    }
    return \@result_array;

} ## end sub cifs_shares

sub _table_parser {

    # from heading line
    my @lines = @{ $_[0] };

    # get the column names splitting on two or more spaces
    my @headings = split /\s\s+/, shift @lines;

    # remove the line that contains only underscores.
    shift @lines;

    # the array that we got previously contains some elements which have more
    # than one column name in them.
    # These must become separate headings - stored in @new_headings

    # Now, process the remaining lines (rows)
    my %hash;
    my %prev_hash;
    my ( @other_settings_arr, @arr );
    foreach my $line (@lines) {
        my @values = split /\s+/, $line;
        if ( $line =~ /(Full Control)|(Read)|(Change)|(No Access)/ ) {
            push( @other_settings_arr, $line );
        } elsif ( $line =~ /\.\.\.\s*(.+)/ ) {
            push( @other_settings_arr, $1 );
        } else {
            my $i = 0;

           # This line indicates the end of the previous hash. So populate the
           # prev hash with @other_settings and then start the new hash.
            my %prev_hash = %hash;
            my @set_arr   = @other_settings_arr;
            $prev_hash{other_settings} = \@set_arr
                if ( scalar @other_settings_arr );
            push @arr, \%prev_hash if ( scalar( keys %hash ) );
            %hash               = ();
            @other_settings_arr = ();
            my $description_value;

            foreach my $value (@values) {
                if ( $i >= 2 ) {
                    $description_value .= " " . $value;
                    next;
                }
                $hash{ $headings[$i] } = $value;
                $i++;
            }
            $hash{ $headings[2] } = $description_value;
        }
    }
    $hash{other_settings} = \@other_settings_arr
        if ( scalar @other_settings_arr );
    push @arr, \%hash if ( scalar( keys %hash ) );
    return \@arr;
} ## end sub _table_parser

## @name ::cifs_prefdc_print
##
## @summary Parses the raw output from the "cifs prefdc print" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_prefdc_print with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_prefdc_print(%args);
## INPUTS TO METHOD CALL 'cifs_prefdc_print' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; cifs prefdc print; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'domain' => 'GOOGLE',
##&nbsp;            'preferred_dc' => '10.24.23.52'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; cifs prefdc print GOOGLE; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'domain' => 'GOOGLE',
##&nbsp;            'preferred_dc' => '10.24.23.52'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cifs_prefdc_print {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $value;
    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @array_lines
        = get_lines( raw_output => $output, response_object => $resp );
    my $i = -1;
    my $k = 0;
    my $j = 0;
    my $val;
    foreach $value (@array_lines) {
        if ( $value =~ /Preferred DC ordering per domain:/ ) {
            next;
        } elsif ( $value =~ /([A-Za-z0-9\.]+):/ ) {
            $i++;
            $result_array[$i]->{"domain"} = $1;
            if ( $k == 1 ) {
                $val = "";
            }
        } elsif ( $value =~ /\d+.\s+([0-9\.]+)/ ) {
            my $val2 = $1;
            $k   = 1;
            $val = $val2 . "," . $val;
            $val =~ s/\,$//g;
            $result_array[$i]->{"preferred_dc"} = $val;
        }
    } ## end foreach $value (@array_lines)
    return \@result_array;
} ## end sub cifs_prefdc_print

## @name ::cifs_prefdc_add
##
## @summary Parses the raw output from the "cifs prefdc add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_prefdc_add with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_prefdc_add(%args);
## INPUTS TO METHOD CALL 'cifs_prefdc_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'preferred-dc' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs prefdc add
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_prefdc_add {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_prefdc_add

## @name ::cifs_prefdc_delete
##
## @summary Parses the raw output from the "cifs prefdc delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_prefdc_delete with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_prefdc_delete(%args);
## INPUTS TO METHOD CALL 'cifs_prefdc_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs prefdc delete
##&nbsp;$VAR1 = [];
##&nbsp;

sub cifs_prefdc_delete {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_prefdc_delete

## @name ::cifs_gpupdate
##
## @summary Parses the raw output from the "cifs gpupdate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_gpupdate with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_gpupdate(%args);
## INPUTS TO METHOD CALL 'cifs_gpupdate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs gpupdate
##&nbsp;$VAR1 = [];
##&nbsp;

sub cifs_gpupdate {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_gpupdate

## @name ::cifs_gpresult
##
## @summary Parses the raw output from the "cifs gpresult" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_gpresult with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_gpresult(%args);
## INPUTS TO METHOD CALL 'cifs_gpresult' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-r' </td> <td> 'resultant-group-policy' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dump' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose-version' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs gpresult
##&nbsp;$VAR1 = [];
##&nbsp;

sub cifs_gpresult {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_gpresult
## @name ::cifs_access
##
## @summary Parses the raw output from the "cifs access" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_access with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_access(%args);
## INPUTS TO METHOD CALL 'cifs_access' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-m' </td> <td> 'windows-access' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'permissions' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'user-or-group' </td> <td> '0' </td> </tr>
## <tr> <td> '-delete' </td> <td> 'delete' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'sharename' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs access
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_access {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_access
## @name ::cifs_changefilerpwd
##
## @summary Parses the raw output from the "cifs changefilerpwd" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_changefilerpwd with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_changefilerpwd(%args);
## INPUTS TO METHOD CALL 'cifs_changefilerpwd' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs changefilerpwd
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_changefilerpwd {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_changefilerpwd
## @name ::cifs_resetdc
##
## @summary Parses the raw output from the "cifs resetdc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_resetdc with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_resetdc(%args);
## INPUTS TO METHOD CALL 'cifs_resetdc' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs resetdc
##&nbsp;$VAR1 = [];
##&nbsp;

sub cifs_resetdc {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub cifs_resetdc

## @name ::cifs_comment
##
## @summary Parses the raw output from the "cifs comment" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_comment with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_comment(%args);
## INPUTS TO METHOD CALL 'cifs_comment' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-comment' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs comment
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'comment' => 'display or change CIFS server description',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cifs_comment {
    my %args = @_;

    my ($error_message);
    my $resp         = $args{response_object};
    my $output       = $resp->get_processed_output();
    my $command      = $resp->get_actual_command();
    my @result_array = ();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    chomp($output);
    $result_array[0]->{"comment"} = $output;
    return \@result_array;
} ## end sub cifs_comment

## @name ::cifs_nbalias
##
## @summary Parses the raw output from the "cifs nbalias" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_nbalias with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_nbalias(%args);
## INPUTS TO METHOD CALL 'cifs_nbalias' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> load </td> <td> 'load' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs nbalias
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_nbalias {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $output !~ /No NetBIOS aliases/i ) {
        chomp($output);
        my @result = split( /,/, $output );
        $result_array[0]->{"netbios_alias"} = \@result if ($output);
    }
    return \@result_array;
} ## end sub cifs_nbalias

## @name ::cifs_homedir
##
## @summary Parses the raw output from the "cifs homedir" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_homedir with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_homedir(%args);
## INPUTS TO METHOD CALL 'cifs_homedir' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> load </td> <td> 'load' </td> </tr>
## <tr> <td> -f </td> <td> 'force' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'show-user' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cifs homedir
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_homedir {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @array_lines
        = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@array_lines) {
        my %hash;
        if ( $line =~ /^(\/\w+)+$/ ) {
            %hash = ( 'homedir' => "$&" );
        } elsif ( $line =~ /^(\w+)\s+((\/\w+)+)$/ ) {
            %hash = ( 'user_name' => "$1", 'homedir' => "$2" );
        }
        push( @result_array, \%hash );
    }
    return \@result_array;
} ## end sub cifs_homedir

## @name ::cifs_domaininfo
##
## @summary Parses the raw output from the "cifs domaininfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_domaininfo with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_domaininfo(%args);
## INPUTS TO METHOD CALL 'cifs_domaininfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domain' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; cifs domaininfo csiqa.rtp.netapp.com; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'not_currently_connected_to_any_dcs' => [
##&nbsp;                                                      {
##&nbsp;                                                        'preferred_addresses' => [],
##&nbsp;                                                        'favored_addresses' => [],
##&nbsp;                                                        'other_addresses' => []
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;            'windows_2003_domain_name' => 'csiqa.rtp.netapp.com',
##&nbsp;            'netbios_domain' => 'CSIQA',
##&nbsp;            'filer_ad_site' => 'Default-First-Site-Name',
##&nbsp;            'type' => 'Windows 2003',
##&nbsp;            'connected_ad_ldap_server_info' => [
##&nbsp;                                                 {
##&nbsp;                                                   'preferred_addresses' => [],
##&nbsp;                                                   'favored_addresses' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              'domain_name' => 'csiqa-ad3.csiqa.rtp.netapp.com',
##&nbsp;                                                                              'ip_address' => '10.227.64.23'
##&nbsp;                                                                            },
##&nbsp;                                                                            {
##&nbsp;                                                                              'domain_name' => 'csiqa-ad4.csiqa.rtp.netapp.com',
##&nbsp;                                                                              'ip_address' => '10.227.64.24'
##&nbsp;                                                                            },
##&nbsp;                                                                            {
##&nbsp;                                                                              'domain_name' => 'csiqa-ad1.csiqa.rtp.netapp.com',
##&nbsp;                                                                              'ip_address' => '10.227.64.21'
##&nbsp;                                                                            },
##&nbsp;                                                                            {
##&nbsp;                                                                              'domain_name' => 'csiqa-ad2.csiqa.rtp.netapp.com',
##&nbsp;                                                                              'ip_address' => '10.227.64.22'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                   'other_addresses' => []
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'connected_ad_ldap_server' => '\\\\csiqa-ad3.csiqa.rtp.netapp.com'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cifs_domaininfo {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my @result_array;
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $i = 0;
    my ( $head, $head1, $head2, $head11, $head21, $flag, $flag1, $sflag );
    foreach my $line (@lines) {
        if ( $line =~ /([\w\s\d]+):\s+([\.\-\s\w]+)/ ) {
            my $key = $1;
            my $value = $2;
            # The 'if' block below is necessary to fix burt650962
            # (http://burtweb-prd.eng.netapp.com/burt/burt-bin/start?id=650962&fz=279&btn=view)
            if ($key =~ /^(windows)\s+\d+(\s+.*)/i){
                $result_array[0]->{ to_key_name( item => $1.$2 ) } = $value;
            }
            $result_array[0]->{ to_key_name( item => $key ) } = $value;
            next;
        } elsif ( $line =~ /(Not currently connected to any DCs)/ ) {
            $head1 = $1;
            $head1 = to_key_name( item => $1 );
            $flag  = 1;
            $i     = 0;
            next;
        } elsif ( $line =~ /(Connected AD LDAP Server):\s+(.*)/ ) {
            $head2                     = $1;
            $head2                     = to_key_name( item => $1 );
            $flag                      = 2;
            $i                         = 0;
            $result_array[0]->{$head2} = $2;
            next;
        }
        if ( $flag == 1 ) {
            if ( $line =~ /^(\w+\s+Addresses:)/ ) {
                $head11 = $1;
                $head11 = to_key_name( item => $head11 );
                $sflag  = 1;
                next;
            }
            if ( $sflag == 1 ) {
                if ( $line =~ /((?:[\d]+\.){3}\d+)/ ) {
                    $result_array[0]->{$head1}->[0]->{$head11}->[$i]
                        ->{"ip_address"} = $1;
                } elsif ( $line !~ /((?:[\d]+\.){3}\d+)/ ) {
                    if ( $line =~ /(.*)/ ) {
                        if ( $line =~ /None/ ) {
                            $result_array[0]->{$head1}->[0]->{$head11} = [];
                        } else {
                            $result_array[0]->{$head1}->[0]->{$head11}->[$i]
                                ->{"domain_name"} = $1;
                            $i++;
                        }
                    }
                }
            }
        } elsif ( $flag == 2 ) {
            if ( $line =~ /^(\w+\s+Addresses:)/ ) {
                $head21 = $1;
                $head21 = to_key_name( item => $head21 );
                $sflag  = 2;
                next;
            }
            if ( $sflag == 2 ) {
                if ( $line =~ /((?:[\d]+\.){3}\d+)/ ) {
                    $result_array[0]->{ $head2 . "_info" }->[0]->{$head21}
                        ->[$i]->{"ip_address"} = $1;
                } elsif ( $line !~ /((?:[\d]+\.){3}\d+)/ ) {
                    if ( $line =~ /(.*)/ ) {
                        if ( $line =~ /None/ ) {
                            $result_array[0]->{ $head2 . "_info" }->[0]
                                ->{$head21} = [];
                        } else {
                            $result_array[0]->{ $head2 . "_info" }->[0]
                                ->{$head21}->[$i]->{"domain_name"} = $1;
                            $i++;
                        }
                    }
                }
            }
        }
    }
    return \@result_array;
} ## end sub cifs_domaininfo

## @name ::cifs_branchcache_hash_stat
##
## @summary Parses the raw output from the "cifs branchcache hash stat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_branchcache_hash_stat with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_branchcache_hash_stat(%args);
## INPUTS TO METHOD CALL 'cifs_branchcache_hash_stat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-flush' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> '-size' </td> <td> 'size' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; cifs branchcache hash stat; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Number of hashes flushed in duration of 600 seconds = 0
##&nbsp;Number of hashes flushed in duration of 900 seconds = 0
##&nbsp;Number of hashes flushed in duration of 1200 seconds = 0
##&nbsp;Number of hashes flushed in duration of 1500 seconds = 0
##&nbsp;Number of hashes flushed in duration of more than 1800 seconds = 0
##&nbsp;Number of files (size <= 10KB) for which hashes were asked for = 0
##&nbsp;Number of files (10KB < size <= 100KB) for which hashes were asked for = 0
##&nbsp;Number of files (100KB < size <= 250k) for which hashes were asked for = 0
##&nbsp;Number of files (250KB < size <= 1MB) for which hashes were asked for = 0
##&nbsp;Number of files (1MB < size <= 10MB) for which hashes were asked for = 0
##&nbsp;Number of files (10MB < size <= 100MB) for which hashes were asked for = 0
##&nbsp;Number of files (size > 100MB) for which hashes were asked for = 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'files' => [
##&nbsp;                         {
##&nbsp;                           'size_specification' => 'size <= 10KB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => '10KB < size <= 100KB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => '100KB < size <= 250k',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => '250KB < size <= 1MB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => '1MB < size <= 10MB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => '10MB < size <= 100MB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'size_specification' => 'size > 100MB',
##&nbsp;                           'no_of_files' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'hashes' => [
##&nbsp;                          {
##&nbsp;                            'no_of_hashes_flushed' => '0',
##&nbsp;                            'duration_in_seconds' => '600'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'no_of_hashes_flushed' => '0',
##&nbsp;                            'duration_in_seconds' => '900'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'no_of_hashes_flushed' => '0',
##&nbsp;                            'duration_in_seconds' => '1200'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'no_of_hashes_flushed' => '0',
##&nbsp;                            'duration_in_seconds' => '1500'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'no_of_hashes_flushed' => '0',
##&nbsp;                            'duration_in_seconds' => 'more than 1800'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cifs_branchcache_hash_stat {
    my %args = @_;

    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ( $i, $j ) = ( 0, 0 );
    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @array_lines
        = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@array_lines) {
        if ( $line
            =~ /Number\s+of\s+hashes\s+flushed\s+in\s+duration\s+of\s+(\d+)\s+seconds\s+=\s+(\d+)/
            ) {
            $result_array[0]->{'hashes'}->[$i]->{'duration_in_seconds'}
                = "$1";
            $result_array[0]->{'hashes'}->[ $i++ ]->{'no_of_hashes_flushed'}
                = "$2";
        } elsif ( $line
            =~ /Number\s+of\s+hashes\s+flushed\s+in\s+duration\s+of\s+(more\s+than\s+\d+)\s+seconds\s+=\s+(\d+)/
            ) {
            $result_array[0]->{'hashes'}->[$i]->{'duration_in_seconds'}
                = "$1";
            $result_array[0]->{'hashes'}->[ $i++ ]->{'no_of_hashes_flushed'}
                = "$2";
        } elsif ( $line
            =~ /Number\s+of\s+files\s+\((.*)\)\s+for\s+which\s+hashes\s+were\s+asked\s+for\s+=\s+(\d+)/
            ) {
            $result_array[0]->{'files'}->[$j]->{'size_specification'} = "$1";
            $result_array[0]->{'files'}->[ $j++ ]->{'no_of_files'} = "$2";
        }
    }
    return \@result_array;
} ## end sub cifs_branchcache_hash_stat

## @name ::cifs_branchcache_set_key
##
## @summary Parses the raw output from the "cifs branchcache set key" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_branchcache_set_key with any of its options as described in the
##    corresponding cdef file 'cifs.cdefs'
## @synopsis
##      cifs_branchcache_set_key(%args);
## INPUTS TO METHOD CALL 'cifs_branchcache_set_key' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'key' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; cifs branchcache set key key1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The server secret key is updated successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'The server secret key is updated successfully.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cifs_branchcache_set_key {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array = ();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    chomp($output);
    if ( $output
        =~ /(The\s+server\s+secret\s+key\s+is\s+updated\s+successfully)/ ) {
        $result_array[0]->{"status"} = "$1";
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );

    }
    return \@result_array;
} ## end sub cifs_branchcache_set_key

## @name ::cifs_stat
##
## @summary Parses the raw output from the "cifs_stat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cifs_stat with any of its options as described in the
##    corresponding cdef file 'cifs_stat.cdefs'
## @synopsis
##      cifs_stat(%args);
## INPUTS TO METHOD CALL 'cifs_stat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'preceding-interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'host' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'user' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'counts' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zeroes' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cifs stat -z
##&nbsp;$VAR1 = [];
##&nbsp;
sub cifs_stat {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $key   = 'blocking';
    my $i     = 0;
    my $error = _check_error(
        "raw_output"  => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines
        = get_lines( raw_output => $output, response_object => $resp );
    foreach my $lines (@arraylines) {
        if ( $lines =~ /CIFS\snon-blocking\srequest\sstatistics:/ ) {
            $key = 'non_blocking';
        }
        if ( $lines
            =~ /(\S+\s+\S+):\s+\S+\s+=\s+(\d+),\s+\S+\s+=\s+(\d+),\s+\S+\s+=\s+(\d+)/
            ) {
            $result_array[0]->{$key}->[0]->{ to_key_name( item => $1 ) }
                ->{"builtins"} = $2;
            $result_array[0]->{$key}->[0]->{ to_key_name( item => $1 ) }
                ->{"user_defined"} = $3;
            $result_array[0]->{$key}->[0]->{ to_key_name( item => $1 ) }
                ->{"sids"} = $4;
        } elsif ( $lines
            =~ /(.*)\s+=\s+(\d+),?\s+(.*)\s+=\s+(\d+),?\s+(.*)\s+=\s+(\d+)/ )
        {
            $result_array[$i]->{$key}[0]{ to_key_name( item => $1 ) } = $2;
            $result_array[$i]->{$key}[0]{ to_key_name( item => $3 ) } = $4;
            $result_array[$i]->{$key}[0]{ to_key_name( item => $5 ) } = $6;
        } elsif ( $lines =~ /(.*)\s+=\s+(\d+),?\s+(.*)\s+=\s+(\d+)/ ) {
            $result_array[$i]->{$key}[0]{ to_key_name( item => $1 ) } = $2;
            $result_array[$i]->{$key}[0]{ to_key_name( item => $3 ) } = $4;
        } elsif ( $lines =~ /(.*)\s+=\s+(.*)/ ) {
            my $key1 = $1;
            my $val  = $2;
            $key1 =~ s/\s+$//g;
            $result_array[$i]->{$key}[0]{ to_key_name( item => $key1 ) }
                = $val;
        } elsif (
            $lines =~ /(\S+\s+\S+\s+\S+|\S+\s+\S+|\S+)\s+(\d+)\s+(\d+)%/ ) {
            $result_array[0]->{$key}->[0]->{ to_key_name( item => $1 ) }
                ->{"count"} = $2;
            $result_array[0]->{$key}->[0]->{ to_key_name( item => $1 ) }
                ->{"percentage"} = $3;
        }
    }
    return \@result_array;
} ## end sub cifs_stat
1;

