#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode clone command
##
## @description
##    This is a library to parse the raw output of 7Mode clone  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use clone ;
##
##    clone_list(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::clone;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub clone_start {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'clone start',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array = ();
    my $i            = 0;
    if ( $raw_output
        =~ /Clone operation started successfully\.\s+(\w+)\:\s+(\d+)\./ ) {
        $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => 'Unexpected output',
            response_object => $resp
        );
    }
    return \@result_array;
} ## end sub clone_start

sub clone_list {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'clone list',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my ( $i, $j, $k, $key, $key1, $value, $value1, $name );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    $i = 0;
    $j = 0;
    $k = 0;
    my $count = -1;
    foreach my $line (@lines) {
        $name = "clone_map_file_header"
            if ( $line =~ /clone map file header/ );
        if ( $line =~ /^([a-z0-9A-Z\_]+)\:\s*([0-9a-zA-Z]+)?$/ ) {
             $result_array[$i]->{"$name"}->[$j]->{$1} = $2;
        } elsif ( $line =~ /(snapid)\s+(\d+)\s+\((\w+)\s+\=\s+(\d+)\)\:\s+(\d+)/ ) {
             $count++;
             $result_array[$i]->{$name}->[0]->{snaprefs}->[$count]->{$1} = $2;
             $result_array[$i]->{$name}->[0]->{snaprefs}->[$count]->{$3} = $4;
             $result_array[$i]->{$name}->[0]->{snaprefs}->[$count]->{refcount} =
             $5;
        } elsif (
            $line =~ /Printing clone_map\s+(\w+)\s+(\d+)\s+(\w+)\s+(\d+)/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{$1} = $2;
            $result_array[$i]->{"clone_map"}->[$j]->{$3} = $4;
        } elsif ( $line =~ /^([0-9a-zA-Z\_\ ]+)\s+\=\s+([0-9a-z]+)$/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{$1} = $2;
        } elsif ( $line =~ /^([0-9a-zA-Z\_\ ]+)\s+([0-9a-z]+)$/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{$1} = $2;
        } elsif ( $line
            =~ /^Clone-inum\s+\=\s+([a-zA-Z0-9\(\) ]+),\s+start\-fbn\s+\=\s+(\d+)\sblk\-count\s\=\s(\d+)/
            ) {
            $k++;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"clone_inum"} = $1;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"start_fbn"} = $2;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"blk_count"} = $3;
        } elsif ( $line
            =~ /^Parent inum\s+\=\s+([a-zA-Z0-9\(\) ]+),\s+start\-fbn\s+\=\s+(\d+)/
            ) {
            $k++;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"parent_inum"} = $1;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"start_fbn"} = $2;
        }
    }
    return \@result_array;
} ## end sub clone_list

sub clone_rlist {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'clone rlist',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my ( $i, $j, $k, $key, $key1, $value, $value1, $name );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    $i = 0;
    $j = 0;
    $k = 0;
    foreach my $line (@lines) {
        $name = "clone_refs_file_header"
            if ( $line =~ /clone refs file header/ );
        if ( $line =~ /^([a-z0-9A-Z\_]+)\:(\s+([0-9a-zA-Z]+))?$/ ) {
            $key = $1;
            $key =~ s/\s+//g;
            $value = $2;
            $value =~ s/\s+//g;
            $result_array[$i]->{"$name"}->[$j]->{$key} = $value;
        } elsif (
            $line =~ /Printing clone_refs\s+(\w+)\s+(\d+)\s+(\w+)\s+(\d+)/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{$1} = $2;
            $result_array[$i]->{"clone_map"}->[$j]->{$3} = $4;
        } elsif ( $line =~ /^([0-9a-zA-Z\_\ ]+)\s+\=\s+([0-9a-z]+)$/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{$1} = $2;
        } elsif ( $line =~ /^([0-9a-zA-Z\_\ ]+)\s+([0-9a-z]+)$/ ) {
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{$1} = $2;
        } elsif ( $line
            =~ /^Parent-inum =\s+([0-9a-zA-Z\(\) ]+), start-fbn =\s+(\d+) blk-count =\s+(\d+) refs=(\d+)$/
            ) {
            $k++;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"parent_inum"} = $1;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"start_fbn"} = $2;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"blk_count"} = $3;
            $result_array[$i]->{"clone_map"}->[$j]->{"Bplus_header_fields"}
                ->[$k]->{"refs"} = $4;
        }
    }
    return \@result_array;
} ## end sub clone_rlist

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    my $invalidParamRegex
        = '(inum not in use|No entries found|Cannot get clone_map)';
    if ( $raw_output =~ /(Unable to find volume.*)|(Invalid snapshot.*)|(SIS clone not supported on this kind of volume)/ ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /$invalidParamRegex/ ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /(clone start: Source file does not exist..)|(clone start: No space..)|(clone start: Volume does not exist..)/
        ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
    }
    return $error;
} ## end sub _check_error
1;
