#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse cna command output
##
## @description
##    This is a library to parse the raw outputs of 7Mode cna commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "cna" will be cna dump()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::cna;
##
## cna_dump(%args);
## cna_reset(%args);
##
## @author abehera@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::cna;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(usage:\s)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /(Illegal option.*)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /(Converged Network Adapter \S+ not found.*)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /(Too few arguments.*)/i ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

## @name ::cna_dump
##
## @summary Parses the raw output from the "cna_dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cna_dump with any of its options as described in the
##    corresponding cdef file 'cna.cdefs'
## @synopsis
##      cna_dump(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; cna dump -f 3b; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Dumping and resetting Converged Network Adapter 3b...
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;
sub cna_dump {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub cna_dump

## @name ::cna_reset
##
## @summary Parses the raw output from the "cna_reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cna_reset with any of its options as described in the
##    corresponding cdef file 'cna.cdefs'
## @synopsis
##      cna_reset(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; cna reset -f 3b; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Dumping and resetting Converged Network Adapter 3b...
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;
sub cna_reset {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub cna_reset
1;
