#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode coredump segment  commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode coredump segment  commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "coredump_segment_start" will be coredump segment start()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::coredump_segment;
##
##coredump_segment_start(%args);
##coredump_segment_stop(%args);
##coredump_segment_show(%args);
##coredump_segment_status(%args);
##coredump_segment_delete(%args);
##coredump_segment_delete_all(%args);
##coredump_segment_config_modify(%args);
##coredump_segment_config_show(%args);
##
##
## @author gupendra@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::coredump;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

## @name ::coredump_segment_start
##
## @summary Parses the raw output from the "coredump segment start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_start with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_start(%args);
## INPUTS TO METHOD CALL 'coredump_segment_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-core-name' </td> <td> 'core-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-delete-core' </td> <td> 'delete-core' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment start -core-name core.118042691.2012-01-11.04_37_17.nz; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub coredump_segment_start {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @raw_output;
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }

    return [];
}    # end sub coredump_segment_start

## @name ::coredump_segment_stop
##
## @summary Parses the raw output from the "coredump segment stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_stop with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_stop(%args);
## INPUTS TO METHOD CALL 'coredump_segment_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-job-id' </td> <td> 'job-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment stop -job-id 137; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub coredump_segment_stop {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    return [];

} ## end sub coredump_segment_stop

## @name ::coredump_segment_delete
##
## @summary Parses the raw output from the "coredump segment delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_delete with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_delete(%args);
## INPUTS TO METHOD CALL 'coredump_segment_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-segment' </td> <td> 'segment' </td> <td> '0' </td> </tr>
## <tr> <td> '-owner-node' </td> <td> 'owner-node' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment delete -segment core.118042691.2012-01-11.04_37_17; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub coredump_segment_delete {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    return [];

} ## end sub coredump_segment_delete

## @name ::coredump_segment_delete_all
##
## @summary Parses the raw output from the "coredump segment delete-all" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_delete_all with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_delete_all(%args);
## INPUTS TO METHOD CALL 'coredump_segment_delete_all' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub coredump_segment_delete_all {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    return [];

} ## end sub coredump_segment_delete_all

## @name ::coredump_segment_config_modify
##
## @summary Parses the raw output from the "coredump segment config modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_config_modify with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_config_modify(%args);
## INPUTS TO METHOD CALL 'coredump_segment_config_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-auto-segment' </td> <td> 'auto-segment' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-auto-delete' </td> <td> 'auto-delete' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment config modify -auto-delete true; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub coredump_segment_config_modify {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    return [];

} ## end sub coredump_segment_config_modify

## @name ::coredump_segment_show
##
## @summary Parses the raw output from the "coredump segment show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_show with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_show(%args);
## INPUTS TO METHOD CALL 'coredump_segment_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-segment' </td> <td> 'segment' </td> <td> '0' </td> </tr>
## <tr> <td> '-owner-node' </td> <td> 'owner-node' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## <tr> <td> '-panic-system-id' </td> <td> 'panic-system-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-total-segment-count' </td> <td> 'total-segment-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-panic-time' </td> <td> 'panic-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-segment-name' </td> <td> 'segment-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-md5-data-chksum' </td> <td> 'md5-data-chksum' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment show -fields node; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'segment' => 'core.118042691.2012-01-11.04_37_17.nvram.nz',
##&nbsp;            'owner-node' => 'csiqa-6070-41',
##&nbsp;            'node' => 'csiqa-6070-41'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'segment' => 'core.118042691.2012-01-11.04_37_17.ontap.nz',
##&nbsp;            'owner-node' => 'csiqa-6070-41',
##&nbsp;            'node' => 'csiqa-6070-41'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub coredump_segment_show {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    my ( @Res,       $value );
    my ( @value_arr, @head_arr,$value1,$value2 );
    my $i      = 0;
    my $x      = 0;
    my $k      = 0;
    my $option = $command;
    $option =~ s/coredump segment show\s?//;

    if ( $option eq "" ) {
        my $i     = -1;
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
        );
        foreach $value (@lines) {
    	    if ( $value =~ m/(Node)\:\s+(.*)/ ) {
                $value2=$2; 
                $value2=~s/\s$//;
                $Res[ ++$i ]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ m/(Segment Directory)\:\s+(.*)/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ m/(Panic Time)\:\s+(.*)/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ m/(Number.*)\:\s+(.*)/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ m/(core.*)/ ) {
                $Res[$i]->{"segment_file_names"}->[$x] = $1;
                $x++;
            }
            elsif ( $value =~ m/(\d+)\s+entries.*/ ) {
                $Res[$i]->{"no_of_entries_displayed"} = $1;
            }
        }    ##foreach(..
    }    ##if($option eq ""

    if ( $option =~ /-fields/ ) {
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
            'first_line'      => "node",
        );

        foreach $value (@lines) {
            if ( $value =~ /node\s+.*/ ) {
                @head_arr = split( /\s+/, $value );
                next;
            }
            elsif ( $value =~ /---------.*/ ) {
                next;
            }
            elsif ( $value =~ /\w+.*.nz/ ) {
                @value_arr = split( /\s+/, $value );
                for ( $k = 0 ; $k <= $#head_arr ; $k++ ) {
                    my $v =$head_arr[$k];
                    $v =~ s/\s$//;
                    $v =~ s/-/_/g;
                    $Res[$i]->{ to_key_name( item =>($v))} = $value_arr[$k];
                }
            }
            elsif ( $value =~ m/(\d+)\s+entries.*/ ) {
                $Res[$i]->{"no_of_entries_displayed"} = $1;
            }
	    $i++;
        }
    }
    elsif ( $option =~ /-instance/ ) {
        my ( $val1, $val2 );
        my $i     = -1;
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,

        );
        foreach $value (@lines) {
            $i++ if ( $value =~ /Node:\S*/ );
            if ( $value =~ m/(.*):\s+(.*)/ ) {
                $val1 = $1;
                $val2 = $2;
                $val1 =~ s/\s$//;
                $Res[$i]->{ to_key_name( item => $val1 ) } = $val2;
            }

        }
    }    ##elsif option

    return \@Res;
} ## end sub coredump_segment_show

## @name ::coredump_segment_status
##
## @summary Parses the raw output from the "coredump segment status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_status with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_status(%args);
## INPUTS TO METHOD CALL 'coredump_segment_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-percent-completed' </td> <td> 'percent-completed' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-core-name' </td> <td> 'core-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-job-id' </td> <td> 'job-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-owner-node' </td> <td> 'owner-node' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment status -fields node,job-id; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'core-name' => 'core.118042691.2012-01-11.04_37_17.nz',
##&nbsp;            'job-id' => '137',
##&nbsp;            'owner-node' => 'csiqa-6070-41',
##&nbsp;            'node' => 'csiqa-6070-41'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub coredump_segment_status {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    my ( @Res, $value );
    my $i = 0;
    my ( @value_arr, @head_arr );
    my $k      = 0;
    my $option = $command;
    $option =~ s/coredump segment status\s?//;
    if ( $option eq "" ) {
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
            'first_line'      => "Node",
        );
        foreach $value (@lines) {

            if ( $value =~ /----.*$/ ) {
                next;
            }
            elsif ( $value =~ /^([\w\d\S]+)$/ ) {
                $Res[$i]->{"node"} = $1;
            }
            elsif ( $value =~ /\w+\s+\entries\s+\w+\s+displayed.$/ ) {
                next;
            }
            elsif ( $value =~ /\s*(\d+)\s+([\w\d\S]+)\s+(\w+)\s+(\d+)%/ ) {
                $Res[$i]->{"id"}        = $1;
                $Res[$i]->{"core_name"} = $2;
                $Res[$i]->{"status"}    = $3;
                $Res[$i]->{"percent"}   = $4;
                $i++;
            }
        }
    }
    if ( $option =~ /-fields/ ) {
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
            'first_line'      => "node",
        );

        foreach $value (@lines) {
            if ( $value =~ /node\s+.*/ ) {
                @head_arr = split( /\s+/, $value );
                next;
            }
            elsif ( $value =~ /---------.*/ ) {
                next;
            }
            elsif ( $value =~ /\w+\s+\entries\s+\w+\s+displayed.$/ ) {
                next;

            }
            elsif ( $value =~ /(.*)/ && $value !~ /entries/ ) {
                @value_arr = split( /\s+/, $value );
                for ( $k = 0 ; $k <= $#head_arr ; $k++ ) {
                    my $v =$head_arr[$k];
                    $v =~ s/\s$//;
                    $v =~ s/-/_/g;
                    $Res[$i]->{ to_key_name( item =>($v))} = $value_arr[$k];
                }
            }
            $i++;
        }
    }    ##if option
    elsif ( $option =~ /-instance/ ) {
        my ( $val1, $val2 );
        my $i     = 0;
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
        );
        foreach $value (@lines) {
            if ( $value =~ m/(.*):\s+(.*)/ ) {
                $val1 = $1;
                $val2 = $2;
                $val1 =~ s/\s$//;
                $Res[$i]->{ to_key_name( item => $val1 ) } = $val2;
            }
        }
        $i++;
    }    ##elsif option
    return \@Res;
} ## end sub coredump_segment_status

## @name ::coredump_segment_config_show
##
## @summary Parses the raw output from the "coredump segment config show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from coredump_segment_config_show with any of its options as described in the
##    corresponding cdef file 'coredump.cdefs'
## @synopsis
##      coredump_segment_config_show(%args);
## INPUTS TO METHOD CALL 'coredump_segment_config_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-auto-segment' </td> <td> 'auto-segment' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-auto-delete' </td> <td> 'auto-delete' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; coredump segment config show -fields auto-delete; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'auto_delete' => 'true',
##&nbsp;            'node' => 'csiqa-6070-41'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub coredump_segment_config_show {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    my ( @Res, $value );
    my $i = 0;
    my ( @value_arr, @head_arr );
    my $k      = 0;
    my $option = $command;
    $option =~ s/coredump segment config show\s?//;
    if ( $option eq "" ) {
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
        );
        foreach $value (@lines) {
            if ( $value =~ /^Node\:\s+(.*)$/ ) {
                $Res[$i]->{"node"} = $1;
            }
            elsif ( $value =~
/^Enable\s+Deletion\s+of\s+Full\s+Core\s+After\s+Automatic\s+Core\s+Segmenting\:\s+(\w+)$/
              )
            {
                $Res[$i]->{"auto_delete"} = $1;
            }
            elsif ( $value =~
/^Enable\s+Automatic\s+Core\s+Segmenting\s+After\s+Saving\s+of\s+a\s+Full\s+Core\:\s+(\w+)$/
              )
            {
                $Res[$i]->{"auto_segment"} = $1;
            }
        }
        $i++;
    }    ##if($option eq "")

    if ( $option =~ /-fields/ ) {
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
            'first_line'      => "node",
        );

        foreach $value (@lines) {
            if ( $value =~ /node\s+.*/ ) {
                @head_arr = split( /\s+/, $value );
                next;
            }
            elsif ( $value =~ /---------.*/ ) {
                next;
            }
            elsif ( $value =~ /(.*)/ ) {
                @value_arr = split( /\s+/, $value );
                for ( $k = 0 ; $k <= $#head_arr ; $k++ ) {     
                    my $v =$head_arr[$k];
                    $v =~ s/\s$//; 
                    $v =~ s/-/_/g;
                    $Res[$i]->{ to_key_name( item =>($v))} = $value_arr[$k];
                }
            }
            $i++;
        }
    }    ##if option

    elsif ( $option =~ /-instance/ ) {
        my ( $val1, $val2 );
        my $i     = 0;
        my @lines = get_lines(
            "raw_output"      => $raw_output,
            "response_object" => $resp,
        );
        foreach $value (@lines) {
            if ( $value =~ m/(.*):\s+(.*)/ ) {
                $val1 = $1;
                $val2 = $2;
                $val1 =~ s/\s$//;
                $val1 =~ s/-/_/g;
                $Res[$i]->{ to_key_name( item => $val1 ) } = $val2;

            }
        }
        $i++;
    }    ##elsif option

    return \@Res;

} ## end sub coredump_segment_config_show

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{response_object};
    my $error_message;
    my $error;
    if ( $output =~ //si ) {
        $error         = "InvalidParam";
        $error_message = "Invalid options are used in the command";
    }
    elsif ( $output =~ /Error: command failed: This command is not supported for this platform/i ) {
        $error         = "CommandFailed";
        $error_message = $output;
    }
    elsif ( $output =~ /Error\:\s+.*/si ) {
        $error         = "InvalidParamValue";
        $error_message = $output;
    }
    elsif ( $output =~ /\s+\[\s+\-instance.*/si ) {
        $error         = "InvalidParam";
        $error_message = $output;
    }
    elsif ( $output =~ /\s+\[\[\-core-name.*/si ) {
        $error         = "InvalidParamValue";
        $error_message = $output;
    }
    elsif ( $output =~ /There\s+are\s+no.*/si ) {
        $error         = "NoMatchingEntries";
        $error_message = $output;
    }
    elsif ( $output =~ /Invalid\s+command.*/si ) {
        $error         = "InvalidParam";
        $error_message = $output;
    }
    elsif ( $output =~ //si ) {
        $error         = "InvalidParamValue";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return $error;
} ## end sub _check_error
1;
