#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode date command output
##
## @description
##    This is a library to parse the raw output of 7Mode date command.
##
## @synopsis
##    use date;
##
##    date(%args);
##
## @author govinda1@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::date;
use strict;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command       = $args{date};
    my $error;
    if ($raw_output =~ /[Uu]sage:/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif($raw_output =~ /date: cannot set date when NTP is running./) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
}


## @name ::date
##
## @summary Parses the raw output from the "date" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from date with any of its options as described in the
##    corresponding cdef file 'date.cdefs'
## @synopsis
##      date(%args);
## INPUTS TO METHOD CALL 'date' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-date' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'universal-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = date -u
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_current_time' => '11:21:00 GMT',
##&nbsp;            'system_year' => '2011',
##&nbsp;            'system_day_date' => 'Thu May 26'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = date
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_current_time' => '11:21:01 GMT',
##&nbsp;            'system_current_time_without_timezone' => '11:21:01',
##&nbsp;            'timezone' => 'GMT',
##&nbsp;            'system_year' => '2011',
##&nbsp;            'system_day_date' => 'Thu May 26'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub date {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (%process_output, @final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => 'date',
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } elsif($output =~ /(\w+\s+\w+\s+\d+)\s(\d+:\d+:\d+\s\w+)\s(\d+)/){
        $process_output{system_day_date} = $1;
        $process_output{system_current_time} = $2;
        my @timezone = split(/\s+/, $process_output{system_current_time});
        $process_output{'system_current_time_without_timezone'} = $timezone[0];
        $process_output{'timezone'} = $timezone[1];
        $process_output{system_year} = $3;
        push(@final,\%process_output);
    }

    return (\@final);
}

1;

