#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode dcb commands
##
## @description
##     This is a library to parse the raw output of 7Mode dcb commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use dcb;
##
##     dcb(%args);
##
## @author sheethal@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::dcb;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args            = @_;
    my $raw_output      = $args{raw_output};
    my $command         = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ($raw_output =~ /(Usage:)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    } elsif ($raw_output =~ /(dcb: No such interface.*)/) {
        $error_message = $1;
        $error         = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }

} ## end sub _check_error

## @name ::dcb_show
##
## @summary Parses the raw output from the "dcb show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dcb_show with any of its options as described in the
##    corresponding cdef file 'dcb.cdefs'
## @synopsis
##      dcb_show(%args);
## INPUTS TO METHOD CALL 'dcb_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pgid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; dcb show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Interface  PGID  Priority           Application             Bandwidth
##&nbsp;---------  ----  ----------         ------------         ------------
##&nbsp;e7a
##&nbsp;           0     0 1 2 3 4 5 6 7    unassigned                    0%
##&nbsp;e7b
##&nbsp;           0     0 1 2 3 4 5 6 7    unassigned                    0%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '0',
##&nbsp;                            '1',
##&nbsp;                            '2',
##&nbsp;                            '3',
##&nbsp;                            '4',
##&nbsp;                            '5',
##&nbsp;                            '6',
##&nbsp;                            '7'
##&nbsp;                          ],
##&nbsp;            'bandwidth_in_percentage' => '0',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '0',
##&nbsp;                            '1',
##&nbsp;                            '2',
##&nbsp;                            '3',
##&nbsp;                            '4',
##&nbsp;                            '5',
##&nbsp;                            '6',
##&nbsp;                            '7'
##&nbsp;                          ],
##&nbsp;            'bandwidth_in_percentage' => '0',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dcb_show {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );

    # Now we can parse the output
    my @lines
        = get_lines(raw_output => $raw_output, response_object => $resp);
    my @result_array;
    my @heads;
    my @values;
    my $interface;
    my $count = 0;
    my $i     = 0;
    @heads = qw(interface pgid priority application bandwidth_in_percentage);

    for ($i = 2; $i <= $#lines; $i++) {
        if ($lines[$i] =~ /^\S+$/) {
            $interface = $lines[$i];
        } elsif ($lines[$i] =~ /^(\d)\s\s+([\d\s]+)\s+(\S+)\s+(\d+)\%$/) {
            my $priority    = $2;
            my $pgid        = $1;
            my $application = $3;
            my $bandwidth   = $4;
            my @priority    = split(/\s+/, $priority);
            $result_array[$count]->{$heads[0]} = $interface;
            $result_array[$count]->{$heads[1]} = $pgid;
            $result_array[$count]->{$heads[2]} = \@priority;
            $result_array[$count]->{$heads[3]} = $application;
            $result_array[$count]->{$heads[4]} = $bandwidth;
            $count++;
        }
    }
    return \@result_array;
} ## end sub dcb_show

## @name ::dcb_priority_show
##
## @summary Parses the raw output from the "dcb priority show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dcb_priority_show with any of its options as described in the
##    corresponding cdef file 'dcb.cdefs'
## @synopsis
##      dcb_priority_show(%args);
## INPUTS TO METHOD CALL 'dcb_priority_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'priority' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; dcb priority show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Interface  Priority PGID Flow Control        Application
##&nbsp;---------  -------- ---- ------------       ------------
##&nbsp;e7a
##&nbsp;           0        0    disabled             unassigned
##&nbsp;           1        0    disabled             unassigned
##&nbsp;           2        0    disabled             unassigned
##&nbsp;           3        0    disabled             unassigned
##&nbsp;           4        0    disabled             unassigned
##&nbsp;           5        0    disabled             unassigned
##&nbsp;           6        0    disabled             unassigned
##&nbsp;           7        0    disabled             unassigned
##&nbsp;e7b
##&nbsp;           0        0    disabled             unassigned
##&nbsp;           1        0    disabled             unassigned
##&nbsp;           2        0    disabled             unassigned
##&nbsp;           3        0    disabled             unassigned
##&nbsp;           4        0    disabled             unassigned
##&nbsp;           5        0    disabled             unassigned
##&nbsp;           6        0    disabled             unassigned
##&nbsp;           7        0    disabled             unassigned
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '0'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '1'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '2'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '3'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '4'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '5'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '6'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '7'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7a',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '0'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '1'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '2'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '3'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '4'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '5'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '6'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'priority' => [
##&nbsp;                            '7'
##&nbsp;                          ],
##&nbsp;            'flow_control' => 'disabled',
##&nbsp;            'application' => 'unassigned',
##&nbsp;            'interface' => 'e7b',
##&nbsp;            'pgid' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dcb_priority_show {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );

    # Now we can parse the output
    my @lines
        = get_lines(raw_output => $raw_output, response_object => $resp);
    my @result_array;
    my @heads;
    my @values;
    my $interface;
    my $count = 0;
    my $i     = 0;
    @heads = qw(interface priority pgid flow_control application);

    for ($i = 2; $i <= $#lines; $i++) {
        if ($lines[$i] =~ /^\S+$/) {
            $interface = $lines[$i];
        } elsif ($lines[$i] =~ /^(\d+)\s+(\d+)\s+(\S+)\s+(\S+)/) {
            my $pgid         = $2;
            my $flow_control = $3;
            my $application  = $4;
            my @priority;
            push(@priority, $1);
            $result_array[$count]->{$heads[0]} = $interface;
            $result_array[$count]->{$heads[1]} = \@priority;
            $result_array[$count]->{$heads[2]} = $pgid;
            $result_array[$count]->{$heads[3]} = $flow_control;
            $result_array[$count]->{$heads[4]} = $application;
            $count++;
        }
    }
    return \@result_array;
} ## end sub dcb_priority_show
1;
