#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode df command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode df command.
##
## @synopsis
##    use df;
##
##    df(%args);
##
## @author anerudh@netapp.com, udaykk@netapp.com, pbala@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::df;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /$command(?:[^:]*):(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message =~ /Missing/ ) {
            $error = "MissingRequiredParam";
        }
        elsif ( $$error_message =~ /No such file/ ) {
            $error = "InvalidParamValue";
        }
        elsif ( $$error_message =~
            /([uU]nrecognized option|[Ee]xtra arguments|[Ii]nvalid)/ )
        {
            $error = "InvalidParam";
        }
        return $error;
    }
    elsif ( $raw_output =~ /[Uu]sage:/ ) {
        $error          = "CommandFailed";
        $$error_message = "Invalid switch or Invalid Param value has been sent";
        return $error;
    }
    elsif ( $raw_output =~ /(No volumes found.)/ ) {
        $error          = "InvalidParam";
        $$error_message = $1;
        return $error;
    }
    return undef;
}

## @name ::df
##
## @summary Parses the raw output from the "df" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from df with any of its options as described in the
##    corresponding cdef file 'df.cdefs'
## @synopsis
##      df(%args);
## INPUTS TO METHOD CALL 'df' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-h' </td> <td> 'scale-units' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode-usage' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'scale-gigabytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'scale-terabytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'supress-snapshot-lines' </td> <td> '1' </td> </tr>
## <tr> <td> '-AH' </td> <td> 'aggregate-units' </td> <td> '1' </td> </tr>
## <tr> <td> '-AC' </td> <td> 'aggregate-composite-units' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'scale-kilobytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'scale-megabytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-hybrid' </td> <td> 'hybrid' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reserved-space' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'flexcache-volume' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'block-sharing-and-compression-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'block-sharing-statistics' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; df; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kbytes' => '828324',
##&nbsp;            'filesystem' => '/vol/vol0/',
##&nbsp;            'mounted_on' => '/vol/vol0/',
##&nbsp;            'used' => '291852',
##&nbsp;            'capacity' => '35%',
##&nbsp;            'avail' => '536472'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '43592',
##&nbsp;            'filesystem' => '/vol/vol0/.snapshot',
##&nbsp;            'mounted_on' => '/vol/vol0/.snapshot',
##&nbsp;            'used' => '92320',
##&nbsp;            'capacity' => '212%',
##&nbsp;            'avail' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '19456',
##&nbsp;            'filesystem' => '/vol/vol1/',
##&nbsp;            'mounted_on' => '/vol/vol1/',
##&nbsp;            'used' => '124',
##&nbsp;            'capacity' => '1%',
##&nbsp;            'avail' => '19332'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'kbytes' => '1024',
##&nbsp;            'filesystem' => '/vol/vol1/.snapshot',
##&nbsp;            'mounted_on' => '/vol/vol1/.snapshot',
##&nbsp;            'used' => '588',
##&nbsp;            'capacity' => '57%',
##&nbsp;            'avail' => '436'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub df {
    my %args = @_;
    my @result_array1;
    my @result_array2;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "df",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/df\s?//;
    if ( $option !~ /-\w+/ ) {
        $option = "";
    }
    if ( $option =~ /-(AH|AC)/ ) {
        my @array_lines =
          get_lines( raw_output => $output, response_object => $resp );
        my @headlines = split( /\s+/, shift(@array_lines) );
        foreach my $lin (@array_lines) {
            my @values = split( /\s+/, $lin );
            if ( $option =~ /-AC/ ) {
                if ( $values[1] eq 'Perf' ) {
                    my $removed_item = splice (@values, 2, 1);
                    $values[1] = $values[1] .' '. $removed_item;
                }
            }
            my %info;
            for ( my $i = 0 ; $i <= $#values ; $i++ ) {
                $info{ to_key_name( item => $headlines[$i] ) } = $values[$i];
            }
            push( @result_array1, \%info );
        }
        return \@result_array1;
    }
    if ( $option =~ /-A\s+-hybrid/ ) {
        my @array_lines =
          get_lines( raw_output => $output, response_object => $resp );
        my @headlines = split( /\s+/, shift(@array_lines) );
        foreach my $lin1 (@array_lines) {
            my @values = split( /\s+/, $lin1 );
            my %info;
            for ( my $i = 0 ; $i <= $#values ; $i++ ) {
                $info{ to_key_name( item => $headlines[$i] ) } = $values[$i];
            }
            push( @result_array2, \%info );
        }
        return \@result_array2;
    }

    #end of changes
    my $first_line = '(Filesystem|Aggregate)  ';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp,
        can_throw       => 1
    );
    my @headings = split( /\s+/, shift(@lines) );
    if ( ( $headings[$#headings] eq 'on' ) ) {
        $headings[ $#headings - 1 ] .= ' ' . pop(@headings);
    }
    @headings = map( to_key_name( item => $_ ), @headings );
    my ( @values, $i );
    foreach my $line (@lines) {
        my %hash;
        @values = split( /\s+/, $line );
        $i = 0;
        foreach (@headings) {
            $hash{$_} = $values[$i];
            $i++;
        }
        push( @result_array, \%hash );
    }
    return \@result_array;
}

1;
