# $Id: //depot/prod/test/nacl/lib/NACL/APISet/Node/CLI/7Mode/default/disk_fw_update.pm $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode disk_fw_update.pm command
##
## @description
##    This is a library to parse the raw output of 7Mode disk_fw_update command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "disk_fw_update" will be disk_fw_update()
##
## @synopsis
##    use disk_fw_update;
##
##    disk_fw_update(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::disk_fw_update;

# use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;

    if ( $raw_output =~ /([Uu]sage:\s)/ ) {
        $error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output
        =~ /(Disk firmware could not be downloaded)|(Disk firmware update already in progress.)/
        ) {
        $error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /(disk_fw_update: *)|([E|e]rror reading firmware file)/ ) {
        $error_message = $1;
        $error          = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::disk_fw_update
##
## @summary Parses the raw output from the "disk_fw_update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_fw_update with any of its options as described in the
##    corresponding cdef file 'disk_fw_update.cdefs'
## @synopsis
##      disk_fw_update(%args);
## INPUTS TO METHOD CALL 'disk_fw_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'stop' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'firmware-file' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'revert' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'qual-test' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk_fw_update -f; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Refreshing information on all the disks.......
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;Downloading disk firmware can be disruptive to the system.
##&nbsp;It involves spinning down and then spinning up all
##&nbsp;affected disks, and suspending disk I/O to disks being
##&nbsp;downloaded. This delay may cause clients to stop responding,
##&nbsp;and may cause CIFS clients to drop sessions.
##&nbsp;
##&nbsp;NOTE: Updating firmware on some Fibre Channel disks
##&nbsp;may create an open loop condition which can only be cleared
##&nbsp;by power cycling the affected shelf or shelves.
##&nbsp;
##&nbsp;
##&nbsp;(You can ignore disk error messages for disks being updated until this
##&nbsp; update completes)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; disk_fw_update -f -Q; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;WARNING: This -Q option interface to BACKGROUND disk_fw_update is for development
##&nbsp;use only. Firmware will be downloaded CONTINUOUSLY to all disks.
##&nbsp;Test will run repetitively until a disk_fw_update -S command is issued or
##&nbsp;until the system is rebooted.
##&nbsp;
##&nbsp;Ready to start BACKGROUND download Qualification test...
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; disk_fw_update -f -S; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;WARNING: This -S option interface to disk_fw_update is for development
##&nbsp;use only. This terminates a -Q firmware download qualification test.
##&nbsp;
##&nbsp;Ready to stop download Qualification test...
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; disk_fw_update -f -R; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;WARNING: This -R option interface to disk_fw_update is support
##&nbsp;use only. Firmware will be re-downloaded to all specified disks.
##&nbsp;
##&nbsp;Ready to start revert of firmware...
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub disk_fw_update {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    my %result_array;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }

    if ( $output =~ /(No disks .* for download)/ ) {
        $result_array{'status'} = $1;
    }
    return [\%result_array];
}    # end sub disk_fw_update;
1;
