#$Id: $
#
#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode disk_qual commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode disk_qual commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##
##    disk_qual_start(%args);
##    disk_qual_stop(%args);
##    disk_qual_status(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::disk_qual;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $resp          = $args{response_object};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(Invalid\s+disk_qual\s+command.*)|(Error invalid test ID!)|(Unrecognized\soption.*)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    } elsif ( $output =~ /(Disk\s+\w+\s+does\s+not\s+exist.)|(Error\s+invalid\s+disk\s+name)/){
        $error_message = $1;
        $error          = "InvalidParamValue";
    } elsif ( $output =~ /(disk name is required)/){
        $error_message = $1;
        $error         = "MissingRequiredParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

## @name ::disk_qual_start
##
## @summary Parses the raw output from the "disk_qual start" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from disk_qual_start with any of its options as described in the 
##    corresponding cdef file 'disk_qual.cdefs'
## @synopsis
##      disk_qual_start(%args);
## INPUTS TO METHOD CALL 'disk_qual_start' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-u' </td> <td> 'lun' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'disk-list' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'first-sector-to-test' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'test-number' </td> <td> '0' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'pass-count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'last-sector-to-test' </td> <td> '0' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'adapter-list' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk_qual start -t 12 -d 0b.13.2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;[0b.13.2]: Starting PRINT LOG SENSE DATA test. 
##&nbsp;[0b.13.2]: Write ECC OTF     : 0 
##&nbsp;[0b.13.2]: Write ECC         : 671717 
##&nbsp;[0b.13.2]: Write Retries     : 265 
##&nbsp;[0b.13.2]: Total Write errors: 0 
##&nbsp;[0b.13.2]: Total times tried : 135834 
##&nbsp;[0b.13.2]: Total bytes proces: 1420858548200 
##&nbsp;[0b.13.2]: Unrecoverable     : 0 
##&nbsp;[0b.13.2]: Read ECC OTF      : 0 
##&nbsp;[0b.13.2]: Read ECC          : 194956 
##&nbsp;[0b.13.2]: Read Retries      : 448 
##&nbsp;[0b.13.2]: Total Read errors : 0 
##&nbsp;[0b.13.2]: Total times tried : 1785114 
##&nbsp;[0b.13.2]: Total bytes proces: 40778987970760 
##&nbsp;[0b.13.2]: Unrecoverable     : 0 
##&nbsp;[0b.13.2]: Non-medium Error Count: 3076 
##&nbsp;[0b.13.2]: Power on time (minutes): 745971 
##&nbsp;
##&nbsp;[0b.13.2]: PRINT LOG SENSE DATA Passed
##&nbsp;[0b.13.2]: Started  Thu Apr 16 06:07:26 GMT 2015
##&nbsp;[0b.13.2]: Finished Thu Apr 16 06:07:27 GMT 2015
##&nbsp;[0b.13.2]: Disk [NETAPP   X421_HCOBE450A10 NA00] Serial Number =         KMH7RK2F
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            '0b.13.2' => {
##&nbsp;                           'serial_number' => 'KMH7RK2F',
##&nbsp;                           'disk' => '[NETAPP   X421_HCOBE450A10 NA00]',
##&nbsp;                           'non-medium_error_count' => '3076',
##&nbsp;                           'write_ecc' => '671717',
##&nbsp;                           'power_on_time_minutes' => '745971',
##&nbsp;                           'total_times_tried' => '1785114',
##&nbsp;                           'total_read_errors' => '0',
##&nbsp;                           'total_write_errors' => '0',
##&nbsp;                           'total_bytes_proces' => '40778987970760',
##&nbsp;                           'read_ecc' => '194956',
##&nbsp;                           'write_retries' => '265',
##&nbsp;                           'read_ecc_otf' => '0',
##&nbsp;                           'write_ecc_otf' => '0',
##&nbsp;                           'started' => 'Thu Apr 16 06:07:26 GMT 2015',
##&nbsp;                           'read_retries' => '448',
##&nbsp;                           'finished' => 'Thu Apr 16 06:07:27 GMT 2015',
##&nbsp;                           'unrecoverable' => '0'
##&nbsp;                         }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_qual_start {
    $Log->enter() if $may_enter;
    my %args = @_;

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $error   = _check_error(raw_output => $output, response_object => $resp );
    my @lines   = get_lines( raw_output => $output, response_object => $resp );

    my ($diskname,@result) ;
    my $k = -1 ;

    my $line = shift(@lines);
    while(@lines || $line){
        if ($line =~ /\[(\d+\w+.*)\].*Starting PRINT LOG SENSE DATA test/){
            $diskname = $1 ;
            $line = shift(@lines);
            $k++ ;
            next ;
        }elsif ( $line =~ /\[(\d+\w+.*)\]/) {
            if($line =~ /PRINT LOG SENSE DATA Passed/) {
                ## skipping this line
            }
            elsif($line =~ /(Started|Finished)(.*)/){
                my $key   = $1;
                my $value = $2;
                $value = trim_string( 'string' => $value );
                $result[$k]->{$diskname}->{to_key_name(item => $key)} = $value ;
            }elsif($line =~ /(Disk)(.*)(Serial\s+Number.*)/) {
                 $result[$k]->{$diskname}->{disk} = trim_string( 'string' => $2 );
                 my @arr = split(/=/,$3);
                 $result[$k]->{$diskname}->{'serial_number'} = trim_string( 'string' => $arr[1] ) ;
            }else {
                 my @arr = split(/:\s+/,$line);
                 my $key = to_key_name(item => $arr[1]);
                 $key =~ s/_$//;
                 $result[$k]->{$diskname}->{$key} = $arr[2] ;
            }
        }
        $line = shift(@lines);
    }
    $Log->exit() if $may_exit;
    return \@result;
} ## end sub disk_qual_start


## @name ::disk_qual_stop
##
## @summary Parses the raw output from the "disk_qual stop" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from disk_qual_stop with any of its options as described in the 
##    corresponding cdef file 'disk_qual.cdefs'
## @synopsis
##      disk_qual_stop(%args);
## INPUTS TO METHOD CALL 'disk_qual_stop' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-u' </td> <td> 'lun' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'disk-list' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'adapter-list' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk_qual stop -d 0b.13.2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub disk_qual_stop {
    my %args = @_;

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::disk_qual_status
##
## @summary Parses the raw output from the "disk_qual status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from disk_qual_status with any of its options as described in the 
##    corresponding cdef file 'disk_qual.cdefs'
## @synopsis
##      disk_qual_status(%args);
## INPUTS TO METHOD CALL 'disk_qual_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk_qual status
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;diskqual is idle
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'idle'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_qual_status {
    my %args = @_;

    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines   = get_lines( raw_output => $output, response_object => $resp );
    my ($line,@result);
    while(@lines){
        $line = shift(@lines);
        if($line =~ /diskqual is running/){
           $result[0]->{status} = "running";
           $line = shift(@lines);
           if($line =~ /(\d+)\s+disks active/) {
              $result[0]->{'active_disk'} = $1;
           }
        }elsif($line =~ /diskqual is idle/){
            $result[0]->{status} = 'idle';
        }
    }
    return \@result;
}


1;
