#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode dns commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode dns commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "dns info" will be dns_info()
##
## @synopsis
##    use dns;
##
##    dns_info(%args);
##    dns_add(%args);
##    dns_delete(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::dns;

use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /[Uu]sage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }
    return $error;
} ## end sub _parse_for_errors
## @name ::dns_info
##
## @summary Parses the raw output from the "dns info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dns_info with any of its options as described in the
##    corresponding cdef file 'dns.cdefs'
## @synopsis
##      dns_info(%args);
## INPUTS TO METHOD CALL 'dns_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub dns_info(%) {
    my %args = @_;

    my @result_array;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _parse_for_errors( raw_output => $output );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => 'Incorrect usage', response_object => $resp );
    }

    my $first_line = 'DNS is .*bled';
    my @lines      = get_lines(
        raw_output => $output,
        first_line => $first_line,
        can_throw  => 1
    );
    my ( %hash, $i, $key, $value, $j );
    $i     = 0;
    $key   = '';
    $j     = 0;
    $value = '';

    if ( $lines[$i] =~ /DNS is (en|dis)abled/ ) {
        $hash{'status'} = $1 . 'abled';
        shift(@lines);
        if ( $hash{'status'} eq 'disabled' ) {
            @result_array = ( \%hash );
            return \@result_array;
        }
    }
    if ( $lines[$i] =~ /DNS caching is (en|dis)abled/ ) {
        $hash{'caching'} = $1 . 'abled';
        shift(@lines);

        if ( $hash{'caching'} eq 'enabled' ) {
            while ( $key !~ /cache_replacement/ && scalar(@lines) > 0) {
                $lines[$i] =~ /(\d+)\s(.*)/;
                $value      = $1;
                $key        = $2;
                $key        = to_key_name( item => $key );
                $hash{$key} = $value;
                shift(@lines);
            } ## end while ($key ne 'cache_replacements')
        } ## end if ($hash{'caching'} eq...
    } ## end if ($lines[$i] =~ /DNS caching is (en|dis)abled/)
    my @keys_array =
      split( /\s\s+/, $lines[$i] );    # get the keys splitting on 2+ spaces

    my %valid_keys = (
        'ip_address'  => 1,
        'state'       => 1,
        'last_polled' => 1,
        'avg_rtt'     => 1,
        'calls'       => 1,
        'errs'        => 1
    );
    for $j ( 0 .. @keys_array - 1 ) {
        $keys_array[$j] = to_key_name( item => $keys_array[$j] );
        if ( $keys_array[$j] eq 'avg_rtt_calls' ) {
            $keys_array[$j] = 'avg_rtt';
        }
        elsif ( $keys_array[$j] eq 'errs' ) {
            $keys_array[$j] = 'calls';
        }
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Invalid key --$keys_array[$j]-- found in @keys_array",
            response_object => $resp
        ) if ( !$valid_keys{ $keys_array[$j] } );
    } ## end for $j (0 .. @keys_array...

    $keys_array[@keys_array] = 'errs';

    shift(@lines);
    shift(@lines);

    $hash{'table'} = [];

    while ( $lines[$i] !~ 'Default domain' ) {
        my @values = split( /\s\s+/, $lines[$i] );
        $j = 0;
        my %internal_hash;
        foreach my $item (@keys_array) {
            $internal_hash{$item} = defined( $values[$j] ) ? $values[$j] : '';
            if ( $values[$j] eq 'NO INFO' ) {
                $values[$j] = '';
                $j--;
            }
            $j++;
        }
        push( @{ $hash{'table'} }, \%internal_hash );
        shift(@lines);
    } ## end while ($lines[$i] !~ 'Default domain')
    while (@lines) {
        ( $key, $value ) = split( /:\s/, $lines[$i] );
        $key = to_key_name( item => $key );
        if ( $value =~ /\s/ ) {
            $hash{$key} = [];
            $j = 0;
            foreach ( split( /\s+/, $value ) ) {
                $hash{$key}->[$j]->{'domain'} = $_;
                $j++;
            }
        }
        else {
            $hash{$key} = $value;
        }
        shift(@lines);
        last if $key =~ /search_domains/;
    } ## end while (@lines)
    @result_array = ( \%hash );
    return \@result_array;
} ## end sub dns_info(%)
## @name ::dns_add
##
## @summary Parses the raw output from the "dns add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dns_add with any of its options as described in the
##    corresponding cdef file 'dns.cdefs'
## @synopsis
##      dns_add(%args);
## INPUTS TO METHOD CALL 'dns_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-address' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'domainname' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub dns_add {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub dns_add
## @name ::dns_delete
##
## @summary Parses the raw output from the "dns delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dns_delete with any of its options as described in the
##    corresponding cdef file 'dns.cdefs'
## @synopsis
##      dns_delete(%args);
## INPUTS TO METHOD CALL 'dns_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-address' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'domainname' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; dns delete rtp.netapp.com; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub dns_delete {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
} ## end sub dns_delete

1;
