#$Id: //depot/prod/test/main/lib/NACL/APISet/Node/CLI/7Mode/default/dump.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode dump commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode dump commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::dump;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::dump
##
## @summary Parses the raw output from the "dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dump with any of its options as described in the 
##    corresponding cdef file 'dump.cdefs'
## @synopsis
##      dump(%args);
## INPUTS TO METHOD CALL 'dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tree' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'blocksize' </td> </tr> 
## <tr> <td> 'Q' </td> <td> 'ignore-qtree-files-directories' </td> </tr> 
## <tr> <td> 'n' </td> <td> 'multi-subtree-dump-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'dumpname' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'exclude-filelist' </td> </tr> 
## <tr> <td> 'f' </td> <td> 'backup-specific-files' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'blocking-factor-size' </td> </tr> 
## <tr> <td> 'X' </td> <td> 'exclude-files' </td> </tr> 
## <tr> <td> 'u' </td> <td> 'update-dumpdates' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tape-name' </td> </tr> 
## <tr> <td> 'l' </td> <td> 'multi-subtree-dump' </td> </tr> 
## <tr> <td> 'B' </td> <td> 'dump-file-block-size' </td> </tr> 
## <tr> <td> 'A' </td> <td> 'ignore-acls' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'dump-level' </td> </tr> 
## <tr> <td> 'b' </td> <td> 'tape-blocking-factor' </td> </tr> 
## <tr> <td> 'R' </td> <td> 'restart-failed-dump' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub dump {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "dump",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

# regex for all output for dump command using case sensitive 'DUMP:' as the marker
    my @dump_output = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => '(?m)^DUMP:',
        "can_throw"       => 1,
        "response_object" => $response
    );

    my $return_hash;

    for ( my $index = 0; $index <= $#dump_output; $index++ ) {
        my ( $key, $value );

      # All regex comparisons are case-sesitive as 'DUMP:' is the only pattern
      # available for many output lines such as size, type etc

        # snapshot name
        if ( $dump_output[$index]
            =~ /(?m)((^DUMP:\s+creating\s+\")(.+?)(\" snapshot\.))/ )
        {
            $key   = 'snapshot';
        } elsif ( $dump_output[$index] =~ /(?m)((^DUMP:\s+\")(.+?))/ ) {

     # There is no pattern for dump type as it can be:
     # DUMP: Using Full Volume dump or  DUMP: Dumping all files not in qtrees.
            $key   = 'type';
        } elsif ( $dump_output[$index]
            =~ /(?m)((^DUMP:\s+Date of this level \d+ dump: )(.+?)(\.))/ )
        {
            # dump date
            $key   = 'date';
        } elsif ( $dump_output[$index]
            =~ /(?m)((^DUMP:\s+Dumping\s+)(.+)(\s+to\s+)(.+))/ )
        {
            # filer tree & tape name
            $key                 = 'tape';
            $value               = $5;
            $key                 = to_key_name( "item" => $key );
            $return_hash->{$key} = $value;

            $key   = 'tree';
        } elsif ( $dump_output[$index]
            =~ /(?m)((^DUMP:\s+)(\d+\s+(K|k|M|m|G|g|T|t)(B|b)))/ )
        {
            # dump size
            $key   = 'size';
        } elsif (
            $dump_output[$index] =~ /(?m)((^DUMP:\s+DUMP IS\s+)(\w+))/ )
        {
            # dump status
            $key   = 'status';
        } ## end elsif ( $dump_output[$index...

        $value = $3;
        if ( defined($key) ) {
            $key = to_key_name( "item" => $key );
            $return_hash->{$key} = $value;
        }
    } ## end for ( my $index = 0; $index...

    return [$return_hash];
} ## end sub dump

# Private sub-routine to check for commonly encountered error in dump operation
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};

    # regular expression strings to match
    my $missing_param
        = qr{(?m)((^usage:\s+dump)|(^DUMP: Unknown arguments to dump:))};

     my $incorrect_param
        = qr{(?m)((^DUMP: Should specify only one dump level between 0 and 9)|(^DUMP: Invalid tape drive.+)|(^DUMP: Could not create file.+)|(^DUMP: No volume:.+)|(^DUMP: cannot open.+)|(^DUMP: Cannot do an incremental dump of all non-qtree files)|(^DUMP: Failed to locate the specified restartable dump)|(^DUMP: Must specify pathname to dump.+))};

    my $error = undef;

    if ( $raw_output =~ $missing_param ) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ $incorrect_param ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }

    return $error;
} ## end sub _check_error

1;
