#$Id: //depot/prod/test/main/lib/NACL/APISet/Node/CLI/7Mode/default/dumpblock.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode dumpblock commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode dumpblock commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "dumpblock info" will be dumpblock_info()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::dumpblock;
##
##    dumpblock_info(%args);
##
## @author dl-nacl-dev@netapp.com, swati.bajaj@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::dumpblock;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::dumpblock_info
##
## @summary Parses the raw output from the "dumpblock info" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dumpblock_info with any of its options as described in the 
##    corresponding cdef file 'dumpblock.cdefs'
## @synopsis
##      dumpblock_info(%args);
## INPUTS TO METHOD CALL 'dumpblock_info' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'level' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'blockno' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'fileid' </td> </tr> 
## <tr> <td> '-private' </td> <td> 'private' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'snapid' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = dumpblock info -V vol0 100 0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cksum_dbn' => '13475',
##&nbsp;            'pvbn' => '8227',
##&nbsp;            'pvbn-of-inofile' => '89356',
##&nbsp;            'fbn-of-inofile' => '4',
##&nbsp;            'dbn' => '13475',
##&nbsp;            'disk' => 'v5.18',
##&nbsp;            'ckse-fbn' => '195847',
##&nbsp;            'pvbn-of-inofile-from-container' => '89356',
##&nbsp;            'vvbn-of-inofile' => '178901',
##&nbsp;            'fbn-of-container-l1' => '178688',
##&nbsp;            'vvbn' => '195847',
##&nbsp;            'ckse-cp-count' => '4',
##&nbsp;            'pvbn-from-parent' => '8227',
##&nbsp;            'fileid-of-container-l1' => '24697',
##&nbsp;            'pvbn-from-container' => '8227',
##&nbsp;            'ckse-notflexvol-flag' => '0',
##&nbsp;            'ckse-checksum' => '0xfe469a01',
##&nbsp;            'fileid-of_container-l1-is-private' => '1',
##&nbsp;            'pvbn-of-inofile-from-parent' => '89356',
##&nbsp;            'ckse-buftreeid' => '0x607c',
##&nbsp;            'ckse-dbn' => '13475'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

##&nbsp;For a VBN_ZERO block:
##&nbsp;Command = dumpblock info -V vol1 0x000060 0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'is_vbn_zero' => '1',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dumpblock_info {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $line;
    my %hash;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    # If this is a VBN_ZERO block, then this is the only line of output; there
    # are no other fields of output.
    if ($output =~ /This is a VBN ZERO block/) {
        return [{is_vbn_zero => 1}];
    }
    
    my @lines=get_lines('raw_output'=>$output);

    
    shift(@lines) if ($lines[0] =~/^ok$/);
    
    while ($line = shift(@lines)) {
        if($line =~ /(\S+)\s+(\S+)/) {
            my $key = to_key_name('item'=>$1);
            $hash{$key} = $2;
        } else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
    }
            
    return [ \%hash ];
}

## @name ::dumpblock_ifile
##
## @summary Parses the raw output from the "dumpblock ifile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dumpblock_ifile with any of its options as described in the 
##    corresponding cdef file 'dumpblock.cdefs'
## @synopsis
##      dumpblock_ifile(%args);
## INPUTS TO METHOD CALL 'dumpblock_ifile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'fileid' </td> </tr> 
## <tr> <td> '-private' </td> <td> 'private' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'block' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'snapid' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.

sub dumpblock_ifile {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
            
    return undef;
}

## @name ::dumpblock_file
##
## @summary Parses the raw output from the "dumpblock file" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dumpblock_file with any of its options as described in the 
##    corresponding cdef file 'dumpblock.cdefs'
## @synopsis
##      dumpblock_file(%args);
## INPUTS TO METHOD CALL 'dumpblock_file' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'blockno' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'fileid' </td> </tr> 
## <tr> <td> '-private' </td> <td> 'private' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.

sub dumpblock_file {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
            
    return undef;
}

## @name ::dumpblock_vbn
##
## @summary Parses the raw output from the "dumpblock vbn" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dumpblock_vbn with any of its options as described in the 
##    corresponding cdef file 'dumpblock.cdefs'
## @synopsis
##      dumpblock_vbn(%args);
## INPUTS TO METHOD CALL 'dumpblock_vbn' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'vbn' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.

sub dumpblock_vbn {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
            
    return undef;
}

## @name ::dumpblock_dbn
##
## @summary Parses the raw output from the "dumpblock dbn" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dumpblock_dbn with any of its options as described in the 
##    corresponding cdef file 'dumpblock.cdefs'
## @synopsis
##      dumpblock_dbn(%args);
## INPUTS TO METHOD CALL 'dumpblock_dbn' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'checksum' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'dbn' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'disk-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.

sub dumpblock_dbn {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
            
    return undef;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command          = $args{command};
    my $error;
    if ( $raw_output =~ /(usage:)|(block not found .+)|(too many arguments)/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(volume name .+ is invalid)|(out of range)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $command." contains invalid value(s)";
    } elsif ( $raw_output =~ /expected (<fileid>|<blockno>|<level>|<snapid>|<inum>|<vol_name>|<fbn>|<vbn>|<disk_name>)/) {
        $error          = "MissingRequiredParam";
        $$error_message = $raw_output;
    }
    return $error;
}

1;
