#$Id: $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode echo command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode echo command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "echo" will be echo()
##
## @synopsis
##    use echo;
##
##    echo(%args);
##
## @author tanil@netaap.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::echo;

use strict;

## @name ::echo
##
## @summary Parses the raw output from the "echo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from echo with any of its options as described in the
##    corresponding cdef file 'echo.cdefs'
## @synopsis
##      echo(%args);
## INPUTS TO METHOD CALL 'echo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'text' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = echo
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'echo_output' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub echo {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _echo(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} ## end sub echo

sub _echo {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # The input string to the echo command will be used to create the output
    # hash
    chomp $raw_output;
    @result_array = ( { 'echo_output' => $raw_output } );
    return \@result_array;
} ## end sub _echo

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error;

    # No exception possible here
    return undef;
} ## end sub _check_error
1;
