# $Id$
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode ems commands
##
## @description
##     This is a library to parse the raw output of 7Mode ems commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use ems;
##
##    ems_event_status(%args);
##    ems_log_dump(%args);
##    ems_log_status(%args);
##    ems_rotate(%args);
##    ems_invoke(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ems;

use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _parse_for_errors {
    my %args         = @_;
    my $raw_output   = $args{raw_output};
    my $command      = $args{command};
    my $errormsg_ptr = $args{'errormsg_ptr'};

    if ( $raw_output =~ /Usage:/ ) {
        $$errormsg_ptr = $command . " command failed";
        return "CommandFailed";
    }
    return undef;
} ## end sub _parse_for_errors

sub _ems_event_status(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my @oupt_array;
    my $emslog_entry;
    my $parsed_event;
    my $i;
    for ( $i = 0; $i <= $#lines; $i = $i + 2 ) {
        if ( $lines[$i] =~ /Event/ ) {
            next;
        }
        my %log_events;
        my @line_one = split /\s{2,}/, $lines[$i];
        my @line_two = split /\s+/,    $lines[ $i + 1 ];
        my ( $event, $priority ) = split /:/, $line_one[0];
        $log_events{eventname}          = $event;
        $log_events{priority}           = $priority;
        $log_events{last_time}          = $line_one[1];
        $log_events{indications}        = $line_two[0];
        $log_events{drops}              = $line_two[1];
        $log_events{'dup-suppressed'}   = $line_two[2];
        $log_events{'timer-suppressed'} = $line_two[3];
        $log_events{'autosuppressed'}   = $line_two[4];
        push @oupt_array, \%log_events;
    } ## end for ( $i = 0; $i <= $#lines...)

    #return the events
    return \@oupt_array;
} ## end sub _ems_event_status(%)

sub _ems_log_dump(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $log_events;
    my $emslog_entry;
    my $parsed_event;

    #my $line = shift(@lines);
    foreach my $line (@lines) {
        #Line begins with NR, so call NR parse and continue
        if ( $line =~ /^<NR/ ) {
            $emslog_entry = $line;
        }

        #Line starts with LR, so continue reading the line
        elsif ( $line =~ /^<LR/ ) {
            $emslog_entry = $line;
            next;
        }

#We've reached the end of the LR entry, so lets add the ending and process the message
        elsif ( $line =~ /^<\/LR/ ) {
            $emslog_entry .= $line;
        }

  #The line is part of the LR content, add to the message and continue reading
        else {
            $emslog_entry .= " $line";
            next;
        }

        #Now parse the full LR line
        $parsed_event = _parseLR_emslog_line( $emslog_entry, $args{'resp'} );

        #add the line to the event array
        push( @$log_events, ($parsed_event) );

        #reset the emslog_entry value
        $emslog_entry = "";
    } ## end foreach my $line (@lines)

    #return the events
    return $log_events;
} ## end sub _ems_log_dump(%)

sub _parseLR_emslog_line {

    my ( $line, $resp ) = @_;
    my ( $header, $event, $id, $i );
    my ( @headerlist, @eventlist );
    my ( $type,       $version );
    my $emslog_event_by_id;
    #Process the line, separate the header and event body
    if ( $line =~ /^\s*<LR (.*)>\s*<(.*)\/>\s*<\/LR>\s*$/ ) {
        $header = "$1";
        $event  = $2;
    } 
    elsif( $line =~ /^\s*<NR (.*)\/>\s*$/ ){
        $emslog_event_by_id->{header}->{'is-nr'} = 1;
        $header = "$1";
    }else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "emslog_process_line: mismatch\n",
            response_object => $resp
        );
    } ## end else [ if ( $line =~ /^\s*<LR (.*)>\s*<(.*)\/>\s*<\/LR>\s*$/)]

    # Get the id.
    if ( $header =~ /\s*id=\"([^\"]+)\"/ ) {
        $id = $1;
        $emslog_event_by_id->{id} = $id;
    }

    # Parse the header (LR) section of the event record
    while ( $header ne "" ) {
        if ( $header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/ ) {
            if ( $1 eq "d" ) {
                $emslog_event_by_id->{header}->{date} = $2;
            } elsif ( $1 eq "n" ) {
                $emslog_event_by_id->{header}->{'node-name'} = $2;
            } elsif ( $1 eq "pn" ) {
                $emslog_event_by_id->{header}->{'partner-node'} = $2;
            } elsif ( $1 eq "t" ) {
                $emslog_event_by_id->{header}->{timestamp} = $2;
            } elsif ( $1 eq "id" ) {
                $emslog_event_by_id->{header}->{'event-id'} = $2;
            } elsif ( $1 eq "seq" ) {
                $emslog_event_by_id->{header}->{'sequence-number'} = $2;
            } elsif ( $1 eq "p" ) {
                $emslog_event_by_id->{header}->{priority} = $2;
            } elsif ( $1 eq "s" ) {
                $emslog_event_by_id->{header}->{'cluster-status'} = $2;
            } elsif ( $1 eq "o" ) {
                $emslog_event_by_id->{header}->{'thread-name'} = $2;
            } elsif ( $1 eq "vf" ) {
                $emslog_event_by_id->{header}->{'vfiler-name'} = $2;
            } elsif ( $1 eq "replay" ) {
                $emslog_event_by_id->{header}->{'nvram-replay'} = $2;
            } elsif ( $1 eq "inv" ) {
                $emslog_event_by_id->{header}->{'event-invoked'} = $2;
            } elsif ( $1 eq "type" ) {
                $emslog_event_by_id->{header}->{'type'} = $2;
            } elsif ( $1 eq "supp" ) {
                $emslog_event_by_id->{header}->{'suppressed'} = $2;
            } elsif ( $1 eq "v" ) {
                $emslog_event_by_id->{header}->{'version'} = $2;
            } elsif ( $1 eq "tz" ) {
                $emslog_event_by_id->{header}->{'timezone'} = $2;
            } elsif ( $1 eq "nv" ) {
                $emslog_event_by_id->{header}->{'nv'} = $2;
            }

            $header = $3;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "No HEADER",
                response_object => $resp
            );
            $header = "";
        } ## end else [ if ( $header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/)]
    } ## end while ( $header ne "" )

    #Parse for the event and type
    if ( $event =~ /^\s*(\S+)\s*(.*)/ ) {
        $type  = $1;
        $event = $2;
    }

    if ( $type =~ /^(\S+)_([0-9]+)$/ ) {
        $type = $1;
        $type =~ s/_/\./g;
    }
    if ( defined $event){
    $emslog_event_by_id->{event} = $type;
    }

    # Parse the event parameters
    while ( $event ne "" ) {
        if ( $event =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/ ) {
            $emslog_event_by_id->{parameters}{$1} = $2;
            if ( $emslog_event_by_id->{attribute_list} eq "" ) {
                $emslog_event_by_id->{attribute_list} = $1;
            } else {
                $emslog_event_by_id->{attribute_list} .= ",$1";
            }
            $event = $3;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "No EVENT",
                response_object => $resp
            );
            $event = "";
        } ## end else [ if ( $event =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/)]
    } ## end while ( $event ne "" )

    #return the formatted event line
    return $emslog_event_by_id;
} ## end sub _parseLR_emslog_line

sub _ems_log_status {

    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $emslog_status;
    my @ems_array;
    foreach my $line (@lines) {
        if ( $line !~ /last update/ ) {
            @ems_array = split /,/, $line;
        } else {
            @ems_array = $line;
        }

        foreach my $val (@ems_array) {

            if ( $val =~ /enabled/ ) {
                $emslog_status->{enabled} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /save/ ) {
                $emslog_status->{save} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /rotate/ ) {
                $emslog_status->{rotate} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /size/ ) {
                $emslog_status->{size} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /file/ ) {
                $emslog_status->{file} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /format/ ) {
                $emslog_status->{format} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /level/ ) {
                $emslog_status->{level} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /indications/ ) {
                $emslog_status->{indications} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /drops/ ) {
                $emslog_status->{drops} = _trim_output( $val, '\s+' );
            } elsif ( $val =~ /last update/ ) {
                $val =~ s/:/|/;
                $emslog_status->{last_update} = _trim_output( $val, '\|' );
            }
        } ## end foreach my $val (@ems_array)
    } ## end foreach my $line (@lines)
    return $emslog_status;
} ## end sub _ems_log_status

sub _trim_output {
    my ( $line, $sperator ) = @_;
    my $value;
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    $value = ( split /$sperator/, $line )[1];
    $value =~ s/^\s+//;
    $value =~ s/\s+$//;
    return $value;
} ## end sub _trim_output


## @name ::ems_event_status
##
## @summary Parses the raw output from the "ems event status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ems_event_status with any of its options as described in the 
##    corresponding cdef file 'ems.cdefs'
## @synopsis
##      ems_event_status(%args);
## INPUTS TO METHOD CALL 'ems_event_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ems event status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'priority' => 'WARN',
##&nbsp;            'last_time' => '3/9/2011 00:47:41',
##&nbsp;            'eventname' => 'api.input.invalidSchema',
##&nbsp;            'timer-suppressed' => '0',
##&nbsp;            'autosuppressed' => '0',
##&nbsp;            'dup-suppressed' => '0',
##&nbsp;            'indications' => '2',
##&nbsp;            'drops' => '0'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub ems_event_status (%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $error;
    my $errormsg;
    my $first_line = "Event:Priority";
    my $can_throw  = 1;
    if ($error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    } ## end if ( $error = _parse_for_errors...)
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    return ( _ems_event_status( 'lines' => \@lines ) );
} ## end sub ems_event_status (%)


## @name ::ems_log_dump
##
## @summary Parses the raw output from the "ems log dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ems_log_dump with any of its options as described in the 
##    corresponding cdef file 'ems.cdefs'
## @synopsis
##      ems_log_dump(%args);
## INPUTS TO METHOD CALL 'ems_log_dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'time-interval' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ems_log_dump (%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $error;
    my $errormsg;
    if ($error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    } ## end if ( $error = _parse_for_errors...)
    my @lines = get_lines(
        'raw_output' => $raw_output,
    );

    return ( _ems_log_dump( 'lines' => \@lines, 'resp' => $resp ) );
} ## end sub ems_log_dump (%)


## @name ::ems_log_status
##
## @summary Parses the raw output from the "ems log status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ems_log_status with any of its options as described in the 
##    corresponding cdef file 'ems.cdefs'
## @synopsis
##      ems_log_status(%args);
## INPUTS TO METHOD CALL 'ems_log_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ems log status
##&nbsp;$VAR1 = {
##&nbsp;          'last_update' => 'Wed, 23 Mar 2011 03:23:33 GMT',
##&nbsp;          'file' => '/etc/log/ems',
##&nbsp;          'size' => '4187190',
##&nbsp;          'save' => '5',
##&nbsp;          'rotate' => 'weekly',
##&nbsp;          'level' => 'debug',
##&nbsp;          'format' => 'xml',
##&nbsp;          'indications' => '68924',
##&nbsp;          'drops' => '7921',
##&nbsp;          'enabled' => 'on'
##&nbsp;        };
##&nbsp;
sub ems_log_status (%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $error;
    my $errormsg;
    my $first_line = "enabled";
    my $can_throw  = 1;
    if ($error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    } ## end if ( $error = _parse_for_errors...)
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    return ( _ems_log_status( 'lines' => \@lines, 'resp' => $resp ) );
} ## end sub ems_log_status (%)


## @name ::ems_invoke
##
## @summary Parses the raw output from the "ems invoke" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ems_invoke with any of its options as described in the 
##    corresponding cdef file 'ems.cdefs'
## @synopsis
##      ems_invoke(%args);
## INPUTS TO METHOD CALL 'ems_invoke' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'event-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'values' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ems_invoke (%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $error;
    my $errormsg;
    if ($error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    } ## end if ( $error = _parse_for_errors...)

    return [];
} ##


## @name ::ems_rotate
##
## @summary Parses the raw output from the "ems rotate" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ems_rotate with any of its options as described in the 
##    corresponding cdef file 'ems.cdefs'
## @synopsis
##      ems_rotate(%args);
## INPUTS TO METHOD CALL 'ems_rotate' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ems_rotate (%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $error;
    my $errormsg;
    if ($error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    } ## end if ( $error = _parse_for_errors...)

    return [];
} ##
1;
