# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/environ.pm $
#
# -*- perl -*-
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode environment status commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode environment commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "environment status shelf" will be environment_status_shelf()
##
## @synopsis
##    use environ;
##
##    environment_status_shelf(%args);
##
## @author nagajyot@netapp.com,dl-nacl-dev,xdl-storage-automation,dl-storsub-qa
##
## @status To_be_reviewed
package NACL::APISet::Node::CLI::7Mode::default::environ;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};

    my $error;
    if ($output =~ /(Illegal option)|(Too many arguments)/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~ /(Invalid shelf name)|(Invalid expander name)|( Shelf .* not found)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /[Uu]sage/i) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
    }
    return $error;
} ## end sub _check_error

## @name ::environment_status_shelf
##
## @summary Parses the raw output from the "environment status shelf" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from environment_status_shelf with any of its options as described in the 
##    corresponding cdef file 'environ.cdefs'
## @synopsis
##      environment_status_shelf(%args);
## INPUTS TO METHOD CALL 'environment_status_shelf' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'all' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; environment status shelf; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_shelves_monitored' => '2',
##&nbsp;            'environment_for_channel' => '1a',
##&nbsp;            'environmental_failure_on_shelves_on_this_channel' => 'no',
##&nbsp;            'enabled' => 'yes'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_path_port' => '1a.16',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2',
##&nbsp;            'product_identification' => 'DS14-Mk4-FC',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'power_control_element_status' => 'ok',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'shelf_bay_list' => '13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'ses_configuration_access' => 'via loop',
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '34',
##&nbsp;                                                         'power_element_serial_number' => 'PMN441920002667  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620564149  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'embedded_switching_hub_installed_element_with_error' => 'none',
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'ims698133122045  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS69813319C08C  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '1',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '57',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'embedded_switching_hub_installed_element_list' => '1, 2',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'vendor_identification' => 'XYRATEX',
##&nbsp;            'status_reads_attempted' => '3831',
##&nbsp;            'channel' => '1a',
##&nbsp;            'product_revision_level' => '1414',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '22 C (71 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '36 C (96 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'OPS6773421CBD4B',
##&nbsp;            'ses_configuration_shelf_id' => '1',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'logical_identifier' => '0x50050cc0021cbd4b',
##&nbsp;            'ses_configuration_source_id' => '16',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'optional_settings' => '0x00',
##&nbsp;            'module_type' => 'ESH4',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_path_port' => '1a.33',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2',
##&nbsp;            'product_identification' => 'DS14-Mk4-FC',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'power_control_element_status' => 'ok',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'shelf_bay_list' => '13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'ses_configuration_access' => 'via loop',
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620564543  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620445306  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'embedded_switching_hub_installed_element_with_error' => 'none',
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS69813318A541  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS698133192046  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '2',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '57',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'embedded_switching_hub_installed_element_list' => '1, 2',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'vendor_identification' => 'XYRATEX',
##&nbsp;            'status_reads_attempted' => '17970',
##&nbsp;            'channel' => '1a',
##&nbsp;            'product_revision_level' => '1414',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '24 C (75 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '32 C (89 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'OPS6773421CD4BE',
##&nbsp;            'ses_configuration_shelf_id' => '2',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'logical_identifier' => '0x50050cc0021cd4be',
##&nbsp;            'ses_configuration_source_id' => '33',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'optional_settings' => '0x00',
##&nbsp;            'module_type' => 'ESH4',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'number_of_shelves_monitored' => '2',
##&nbsp;            'environment_for_channel' => '0a',
##&nbsp;            'environmental_failure_on_shelves_on_this_channel' => 'no',
##&nbsp;            'enabled' => 'yes'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'sas_cable_information_by_element' => [
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00176+20',
##&nbsp;                                                      'sas_serial_number' => '020409-083  ',
##&nbsp;                                                      'sas_type' => 'QSFP copper 0.5m  ',
##&nbsp;                                                      'sas_element_number' => '1',
##&nbsp;                                                      'sas_id' => '00  ',
##&nbsp;                                                      'sas_vendor' => 'WL GORE',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '<N/A>',
##&nbsp;                                                      'sas_serial_number' => '<N/A>  ',
##&nbsp;                                                      'sas_type' => '<N/A> <N/A>  <N/A>  ',
##&nbsp;                                                      'sas_element_number' => '2',
##&nbsp;                                                      'sas_id' => '<N/A>  ',
##&nbsp;                                                      'sas_vendor' => '<N/A>',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00242+30',
##&nbsp;                                                      'sas_serial_number' => '93A110110008  ',
##&nbsp;                                                      'sas_type' => 'QSFP active copper 10m  ',
##&nbsp;                                                      'sas_element_number' => '3',
##&nbsp;                                                      'sas_id' => '01  ',
##&nbsp;                                                      'sas_vendor' => 'Molex Inc.',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00176+A0',
##&nbsp;                                                      'sas_serial_number' => '051710-029  ',
##&nbsp;                                                      'sas_type' => 'QSFP copper 0.5m  ',
##&nbsp;                                                      'sas_element_number' => '4',
##&nbsp;                                                      'sas_id' => '00  ',
##&nbsp;                                                      'sas_vendor' => 'WL GORE',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;            'current_sensor_installed_element_list' => '1, 2, 3, 4',
##&nbsp;            'device_path_port' => '0a.00.99',
##&nbsp;            'sas_expander_module_installed_element_list' => '1, 2',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2, 3, 4',
##&nbsp;            'product_identification' => 'DS2246',
##&nbsp;            'sas_connector_attached_element_list' => '1, 2, 3, 4',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'acp_information_by_element' => [
##&nbsp;                                              {
##&nbsp;                                                'acp_element_number' => '1',
##&nbsp;                                                'acp_element_mac_address' => '00:A0:98:14:0A:16'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'acp_element_number' => '2',
##&nbsp;                                                'acp_element_mac_address' => '00:A0:98:14:08:90'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'shelf_bay_list' => '11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'shelf_voltages_by_element' => [
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '1',
##&nbsp;                                               'voltage_element_volts' => '12.18'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '2',
##&nbsp;                                               'voltage_element_volts' => '5.07'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '3',
##&nbsp;                                               'voltage_element_volts' => '12.18'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '4',
##&nbsp;                                               'voltage_element_volts' => '5.07'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '114-00065+A0',
##&nbsp;                                                         'power_element_firmware_version' => '020F  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'XXT103302881  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '114-00065+A0',
##&nbsp;                                                         'power_element_firmware_version' => '020F  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'XXT103302868  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '111-00690+A1',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '23  ',
##&nbsp;                                                           'es_element_serial_number' => '8000100264  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '111-00690+A1',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '23  ',
##&nbsp;                                                           'es_element_serial_number' => '8000101294  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '0',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '56',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '5 C (41 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '42 C (107 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '4',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '5',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '6',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '7',
##&nbsp;                                                       'temparature_element_high_critical' => '60 C (140 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '8',
##&nbsp;                                                       'temparature_element_high_critical' => '60 C (140 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'sas_connector_attached_element_with_error' => 'none',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'acp_installed_element_with_error' => 'none',
##&nbsp;            'acp_installed_element_list' => '1, 2',
##&nbsp;            'vendor_identification' => 'NETAPP',
##&nbsp;            'status_reads_attempted' => '17970',
##&nbsp;            'channel' => '0a',
##&nbsp;            'product_revision_level' => '0111',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '21 C (69 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '22 C (71 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '21 C (69 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '4',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '30 C (86 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '5',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '24 C (75 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '6',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '40 C (104 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '7',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '22 C (71 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '8',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '25 C (77 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'shelf_currents_by_element' => [
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '1',
##&nbsp;                                               'current_element_current' => '0'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '2',
##&nbsp;                                               'current_element_current' => '0'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '3',
##&nbsp;                                               'current_element_current' => '0'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '4',
##&nbsp;                                               'current_element_current' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => '6000005153',
##&nbsp;            'ses_configuration_shelf_id' => '0',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'voltage_sensor_installed_with_error' => 'none',
##&nbsp;            'sas_expander_module_installed_element_with_error' => 'none',
##&nbsp;            'logical_identifier' => '0x500a09800006b977',
##&nbsp;            'current_sensor_installed_with_error' => 'none',
##&nbsp;            'voltage_sensor_installed_element_list' => '1, 2, 3, 4',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3, 4, 5, 6, 7, 8',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'cooling_units_by_element' => [
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '3020',
##&nbsp;                                              'cooling_element_number' => '1'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2970',
##&nbsp;                                              'cooling_element_number' => '2'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '3020',
##&nbsp;                                              'cooling_element_number' => '3'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '3000',
##&nbsp;                                              'cooling_element_number' => '4'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'module_type' => 'IOM6',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'sas_cable_information_by_element' => [
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00243+30',
##&nbsp;                                                      'sas_serial_number' => '93A110210019  ',
##&nbsp;                                                      'sas_type' => 'QSFP active copper 20m  ',
##&nbsp;                                                      'sas_element_number' => '1',
##&nbsp;                                                      'sas_id' => '01  ',
##&nbsp;                                                      'sas_vendor' => 'Molex Inc.',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00176+20',
##&nbsp;                                                      'sas_serial_number' => '020409-083  ',
##&nbsp;                                                      'sas_type' => 'QSFP copper 0.5m  ',
##&nbsp;                                                      'sas_element_number' => '2',
##&nbsp;                                                      'sas_id' => '01  ',
##&nbsp;                                                      'sas_vendor' => 'WL GORE',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00176+A0',
##&nbsp;                                                      'sas_serial_number' => '051710-029  ',
##&nbsp;                                                      'sas_type' => 'QSFP copper 0.5m  ',
##&nbsp;                                                      'sas_element_number' => '3',
##&nbsp;                                                      'sas_id' => '01  ',
##&nbsp;                                                      'sas_vendor' => 'WL GORE',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'sas_part_number' => '112-00177+A0',
##&nbsp;                                                      'sas_serial_number' => '009920177  ',
##&nbsp;                                                      'sas_type' => 'QSFP copper 2m  ',
##&nbsp;                                                      'sas_element_number' => '4',
##&nbsp;                                                      'sas_id' => '00  ',
##&nbsp;                                                      'sas_vendor' => 'Molex Inc.',
##&nbsp;                                                      'sas_swaps' => '0'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;            'current_sensor_installed_element_list' => '1, 2, 3, 4, 5, 6, 7, 8',
##&nbsp;            'device_path_port' => '0b.02.99',
##&nbsp;            'sas_expander_module_installed_element_list' => '1, 2',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2, 3, 4, 5, 6, 7, 8',
##&nbsp;            'product_identification' => 'DS4243',
##&nbsp;            'sas_connector_attached_element_list' => '1, 2, 3, 4',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'acp_information_by_element' => [
##&nbsp;                                              {
##&nbsp;                                                'acp_element_number' => '1',
##&nbsp;                                                'acp_element_mac_address' => '00:50:CC:62:63:E5'
##&nbsp;                                              },
##&nbsp;                                              {
##&nbsp;                                                'acp_element_number' => '2',
##&nbsp;                                                'acp_element_mac_address' => '00:50:CC:62:64:07'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'shelf_bay_list' => '23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4',
##&nbsp;            'shelf_voltages_by_element' => [
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '1',
##&nbsp;                                               'voltage_element_volts' => '5.11'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '2',
##&nbsp;                                               'voltage_element_volts' => '12.26'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '3',
##&nbsp;                                               'voltage_element_volts' => '5.11'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '4',
##&nbsp;                                               'voltage_element_volts' => '12.30'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '5',
##&nbsp;                                               'voltage_element_volts' => '5.07'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '6',
##&nbsp;                                               'voltage_element_volts' => '12.18'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '7',
##&nbsp;                                               'voltage_element_volts' => '5.03'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'voltage_element_range' => 'normal voltage range',
##&nbsp;                                               'voltage_element_number' => '8',
##&nbsp;                                               'voltage_element_volts' => '12.14'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '0082562-08',
##&nbsp;                                                         'power_element_firmware_version' => '0209  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'PMW8256200059D0  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '0082562-10',
##&nbsp;                                                         'power_element_firmware_version' => '0305  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'PMW825620005FA8  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '0082562-10',
##&nbsp;                                                         'power_element_firmware_version' => '0305  ',
##&nbsp;                                                         'power_element_number' => '3',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'PMW825620005FC2  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '0082562-08',
##&nbsp;                                                         'power_element_firmware_version' => '0209  ',
##&nbsp;                                                         'power_element_number' => '4',
##&nbsp;                                                         'power_element_type' => '9C',
##&nbsp;                                                         'power_element_serial_number' => 'PMW825620005827  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '0094214-05',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '14  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS9421400003B3  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '0094214-05',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '14  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS9421400003E4  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '2',
##&nbsp;            'power_supply_installed_element_list' => '1, 2, 3, 4',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '56',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '5 C (41 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '42 C (107 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '4',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '5',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '6',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '7',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '8',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '9',
##&nbsp;                                                       'temparature_element_high_critical' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '10',
##&nbsp;                                                       'temparature_element_high_critical' => '70 C (158 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '65 C (149 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '11',
##&nbsp;                                                       'temparature_element_high_critical' => '60 C (140 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '12',
##&nbsp;                                                       'temparature_element_high_critical' => '60 C (140 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '55 C (131 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '5C (41 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'sas_connector_attached_element_with_error' => 'none',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'acp_installed_element_with_error' => 'none',
##&nbsp;            'acp_installed_element_list' => '1, 2',
##&nbsp;            'vendor_identification' => 'NETAPP',
##&nbsp;            'status_reads_attempted' => '17970',
##&nbsp;            'channel' => '0a',
##&nbsp;            'product_revision_level' => '0131',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '23 C (73 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '29 C (84 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '33 C (91 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '4',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '46 C (114 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '5',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '31 C (87 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '6',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '46 C (114 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '7',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '37 C (98 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '8',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '50 C (122 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '9',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '10',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '51 C (123 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '11',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '12',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '38 C (100 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'shelf_currents_by_element' => [
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '1',
##&nbsp;                                               'current_element_current' => '1600'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '2',
##&nbsp;                                               'current_element_current' => '2890'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '3',
##&nbsp;                                               'current_element_current' => '3630'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '4',
##&nbsp;                                               'current_element_current' => '2810'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '5',
##&nbsp;                                               'current_element_current' => '1400'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '6',
##&nbsp;                                               'current_element_current' => '2890'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '7',
##&nbsp;                                               'current_element_current' => '1990'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'current_element_range' => 'normal current range',
##&nbsp;                                               'current_element_number' => '8',
##&nbsp;                                               'current_element_current' => '4100'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'SHX0931422G009S',
##&nbsp;            'ses_configuration_shelf_id' => '2',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'voltage_sensor_installed_with_error' => 'none',
##&nbsp;            'sas_expander_module_installed_element_with_error' => 'none',
##&nbsp;            'logical_identifier' => '0x50050cc102000aec',
##&nbsp;            'current_sensor_installed_with_error' => 'none',
##&nbsp;            'voltage_sensor_installed_element_list' => '1, 2, 3, 4, 5, 6, 7, 8',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'cooling_units_by_element' => [
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '3000',
##&nbsp;                                              'cooling_element_number' => '1'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2020',
##&nbsp;                                              'cooling_element_number' => '2'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '3000',
##&nbsp;                                              'cooling_element_number' => '3'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2770',
##&nbsp;                                              'cooling_element_number' => '4'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2920',
##&nbsp;                                              'cooling_element_number' => '5'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2550',
##&nbsp;                                              'cooling_element_number' => '6'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2920',
##&nbsp;                                              'cooling_element_number' => '7'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'cooling_element_rpm' => '2550',
##&nbsp;                                              'cooling_element_number' => '8'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'module_type' => 'IOM3',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; environment status shelf 0c.1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_shelves_monitored' => '2',
##&nbsp;            'environment_for_channel' => '0c',
##&nbsp;            'environmental_failure_on_shelves_on_this_channel' => 'no',
##&nbsp;            'enabled' => 'yes'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_path_port' => '1a.16',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2',
##&nbsp;            'product_identification' => 'DS14-Mk4-FC',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'power_control_element_status' => 'ok',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'shelf_bay_list' => '13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'ses_configuration_access' => 'via loop',
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '34',
##&nbsp;                                                         'power_element_serial_number' => 'PMN441920002667  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620564149  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'embedded_switching_hub_installed_element_with_error' => 'none',
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'ims698133122045  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS69813319C08C  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '1',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '57',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'embedded_switching_hub_installed_element_list' => '1, 2',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'vendor_identification' => 'XYRATEX',
##&nbsp;            'status_reads_attempted' => '3831',
##&nbsp;            'channel' => '0c',
##&nbsp;            'product_revision_level' => '1414',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '22 C (71 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '36 C (96 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'OPS6773421CBD4B',
##&nbsp;            'ses_configuration_shelf_id' => '1',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'logical_identifier' => '0x50050cc0021cbd4b',
##&nbsp;            'ses_configuration_source_id' => '16',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'optional_settings' => '0x00',
##&nbsp;            'module_type' => 'ESH4',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; environment status shelf 0c; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_shelves_monitored' => '2',
##&nbsp;            'environment_for_channel' => '0c',
##&nbsp;            'environmental_failure_on_shelves_on_this_channel' => 'no',
##&nbsp;            'enabled' => 'yes'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_path_port' => '1a.16',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2',
##&nbsp;            'product_identification' => 'DS14-Mk4-FC',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'power_control_element_status' => 'ok',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'shelf_bay_list' => '13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'ses_configuration_access' => 'via loop',
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '34',
##&nbsp;                                                         'power_element_serial_number' => 'PMN441920002667  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620564149  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'embedded_switching_hub_installed_element_with_error' => 'none',
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'ims698133122045  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS69813319C08C  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '1',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '57',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'embedded_switching_hub_installed_element_list' => '1, 2',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'vendor_identification' => 'XYRATEX',
##&nbsp;            'status_reads_attempted' => '3831',
##&nbsp;            'channel' => '0c',
##&nbsp;            'product_revision_level' => '1414',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '22 C (71 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '36 C (96 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'OPS6773421CBD4B',
##&nbsp;            'ses_configuration_shelf_id' => '1',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'logical_identifier' => '0x50050cc0021cbd4b',
##&nbsp;            'ses_configuration_source_id' => '16',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'optional_settings' => '0x00',
##&nbsp;            'module_type' => 'ESH4',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_path_port' => '1a.33',
##&nbsp;            'cooling_unit_installed_element_list' => '1, 2',
##&nbsp;            'product_identification' => 'DS14-Mk4-FC',
##&nbsp;            'shelf_bay_list_with_error' => 'none',
##&nbsp;            'power_control_element_status' => 'ok',
##&nbsp;            'es_electronics_installed_element_list' => '1, 2',
##&nbsp;            'shelf_bay_list' => '13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0',
##&nbsp;            'ses_configuration_access' => 'via loop',
##&nbsp;            'power_supply_information_by_element' => [
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '1',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620564543  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'power_element_part_number' => '<N/A>',
##&nbsp;                                                         'power_element_firmware_version' => '<N/A>  ',
##&nbsp;                                                         'power_element_number' => '2',
##&nbsp;                                                         'power_element_type' => '5c',
##&nbsp;                                                         'power_element_serial_number' => 'PMC643620445306  ',
##&nbsp;                                                         'power_element_swaps' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;            'embedded_switching_hub_installed_element_with_error' => 'none',
##&nbsp;            'shelf_status' => 'normal condition',
##&nbsp;            'es_electronics_information_by_element' => [
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '1',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS69813318A541  '
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'es_element_number' => '2',
##&nbsp;                                                           'es_element_part_number' => '<N/A>',
##&nbsp;                                                           'es_element_swaps' => '0',
##&nbsp;                                                           'es_element_cpld_version' => '<N/A>  ',
##&nbsp;                                                           'es_element_serial_number' => 'IMS698133192046  '
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'monitoring' => 'active',
##&nbsp;            'shelf' => '2',
##&nbsp;            'power_supply_installed_element_list' => '1, 2',
##&nbsp;            'control_writes_failed' => '0',
##&nbsp;            'control_writes_attempted' => '57',
##&nbsp;            'temperature_thresholds_by_element' => [
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '1',
##&nbsp;                                                       'temparature_element_high_critical' => '50 C (122 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '40 C (104 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '2',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     },
##&nbsp;                                                     {
##&nbsp;                                                       'temparature_element_low_warning' => '10 C (50 F)',
##&nbsp;                                                       'temparature_element_number' => '3',
##&nbsp;                                                       'temparature_element_high_critical' => '63 C (145 F)',
##&nbsp;                                                       'temparature_element_high_warning' => '53 C (127 F)',
##&nbsp;                                                       'temparature_element_low_critical' => '0C (32 F)'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;            'embedded_switching_hub_installed_element_list' => '1, 2',
##&nbsp;            'es_electronics_installed_element_with_error' => 'none',
##&nbsp;            'vendor_identification' => 'XYRATEX',
##&nbsp;            'status_reads_attempted' => '17970',
##&nbsp;            'channel' => '0c',
##&nbsp;            'product_revision_level' => '1414',
##&nbsp;            'shelf_temperatures_by_element' => [
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '1',
##&nbsp;                                                   'shelf_element_range' => '(ambient)  Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '24 C (75 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '2',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '34 C (93 F)'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'shelf_temperature_element_number' => '3',
##&nbsp;                                                   'shelf_element_range' => ' Normal temperature range',
##&nbsp;                                                   'shelf_element_temparature' => '32 C (89 F)'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'device_path' => 'local access',
##&nbsp;            'product_serial_number' => 'OPS6773421CD4BE',
##&nbsp;            'ses_configuration_shelf_id' => '2',
##&nbsp;            'status_reads_failed' => '0',
##&nbsp;            'logical_identifier' => '0x50050cc0021cd4be',
##&nbsp;            'ses_configuration_source_id' => '33',
##&nbsp;            'power_supply_installed_with_error' => 'none',
##&nbsp;            'temperature_sensor_installed_element_list' => '1, 2, 3',
##&nbsp;            'cooling_unit_installed_element_with_error' => 'none',
##&nbsp;            'optional_settings' => '0x00',
##&nbsp;            'module_type' => 'ESH4',
##&nbsp;            'temperature_sensor_installed_element_with_error' => 'none',
##&nbsp;            'es_electronics_reporting_element' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub environment_status_shelf {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(raw_output    => $output,
                             command       => "environment status shelf",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    
    my $first_line = "Environment for channel";
    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          can_throw       => 1,
                          response_object => $resp);
    
    my @result_array = ();
    my @power_array = ();
    my @voltage_array = ();
    my @current_array = ();
    my @cooling_array = ();
    my @temparature_array = ();
    my @es_array = ();
    my @sas_array = ();
    my @acp_array = ();
    my @shelf_temp_array = ();
    my %cool_hash = ();
    my %current_hash = ();
    my %temparature_hash = ();
    my %voltage_hash = ();
    my %power_hash = ();
    my %es_hash = ();
    my %sas_hash = ();
    my %acp_hash = ();
    my %temperature_elements = ();
    
    # Here are the data elements requried to extract the temperature data.
    my $temperature_sensors_flag = 0;
    my $cooling_flag = 0;
    my $acp_flag = 0;
    my $sas_flag = 0;
    my $power_flag = 0;
    my $es_flag = 0;
    my $voltage_flag = 0;
    my $current_flag = 0;
    my $last_temperature_sensor_read = 0;
    my $highest_sensor_number = 0;
    my $temperature_thresholds_flag = 0;
    my $current_temperatures_flag = 0;
    my $power_number = 0;
    my $es_number = 0;
    my $element;
    my $last_acp_element;
    my $last_sas_element;
    my $last_es_element;
    my $last_voltage_element;
    my $last_power_element;
    my $last_shelf_element;
    my $last_cooling_element;
    my $last_temparature_element;
    my $last_current_element;
    my $cool_number;
    my $current_number;
    my $sas_number;
    
    foreach my $line (@lines) {

        # skip blank lines
        if ( $line eq '' ) { next; }
        if ( $line =~ /Environment for channel\s(.*)/ ) {
           my %temp_hash = ();
           push(@result_array, \%temp_hash);
           $result_array[$#result_array]->{"environment_for_channel"} = $1;
        }elsif($line =~ /Number of shelves monitored:\s(.*)\tenabled:\s(.*)/){ # Number of shelves monitored: 1\tenabled: yes\
           $result_array[$#result_array]->{"number_of_shelves_monitored"} = $1;
           $result_array[$#result_array]->{"enabled"} = $2;
        }elsif($line =~ /Environmental failure on shelves on this channel\?\s(.*)/){
           $result_array[$#result_array]->{"environmental_failure_on_shelves_on_this_channel"} = $1;
        }elsif ($line =~ /Channel:/){
           my %temp_hash = ();
           push(@result_array, \%temp_hash);
           my @words = split(/:\s+/, $line);
           $result_array[$#result_array]->{"channel"} = $words[1];
        } elsif($line =~ /Shelf:/){
           my @words = split(/:\s+/, $line);
           $result_array[$#result_array]->{"shelf"} = $words[1];
        } elsif($line =~ /SES\sdevice path:(.*\s.*):\s(.*)/){ # SES device path: local access: 0c.00.99
           $result_array[$#result_array]->{"device_path_port"} = $2;
           $result_array[$#result_array]->{"device_path"} = $1;
        } elsif($line =~ /Module type:\s(.*);\smonitoring\sis\s(.*)/){
           $result_array[$#result_array]->{"module_type"} = $1;
           $result_array[$#result_array]->{"monitoring"} = $2;
        } elsif($line =~ /Shelf\sstatus:\s(.*)/){ #Shelf status: normal condition
           $result_array[$#result_array]->{"shelf_status"} = $1;
        } elsif($line =~ /SES Configuration, shelf\s(\d*):/){ #SES Configuration, shelf 0:
           $result_array[$#result_array]->{"ses_configuration_shelf_id"} = $1;
        } elsif($line =~ /SES Configuration,\s(via\sloop)\sid\s(\d*)\sin\sshelf\s(\d*):/){ #SES Configuration, via loop id 95 in shelf 6:
           $result_array[$#result_array]->{"ses_configuration_shelf_id"} = $3;
           $result_array[$#result_array]->{"ses_configuration_source_id"} = $2;
           my $ses_access = $1;
           $result_array[$#result_array]->{"ses_configuration_access"} = $1; # via loop
        } elsif($line =~ /logical identifier/){  # logical identifier
           my @words = split(/=/, $line);
           $result_array[$#result_array]->{"logical_identifier"} = $words[1];
        } elsif($line =~ /vendor identification/){ #  vendor identification
           my @words = split(/=/, $line);
           $result_array[$#result_array]->{"vendor_identification"} = $words[1];
        } elsif($line =~ /product identification/){ #  product identification=DS14-Mk2-AT
           my @words = split(/=/, $line);
           $result_array[$#result_array]->{"product_identification"} = $words[1];
        } elsif($line =~ /product revision level/){ # product revision level=0505
           my @words = split(/=/, $line);
           $result_array[$#result_array]->{"product_revision_level"} = $words[1];
        } elsif($line =~ /Product Serial Number:\s(.*)/){ # Product Serial Number: NTAPLGH-10020D25DA
           $result_array[$#result_array]->{"product_serial_number"} = $1;           
        } elsif($line =~ /Optional Settings:\s(.*)/){
           $result_array[$#result_array]->{"optional_settings"} = $1;        
        } elsif($line =~ /Status reads attempted:\s(.*);\sfailed:\s(.*)/){ # Status reads attempted: 20121; failed: 0
           $result_array[$#result_array]->{"status_reads_attempted"} = $1;
           $result_array[$#result_array]->{"status_reads_failed"} = $2;
        } elsif($line =~ /Control writes attempted:\s(.*);\sfailed:\s(.*)/){ # Control writes attempted: 62; failed: 0
           $result_array[$#result_array]->{"control_writes_attempted"} = $1;
           $result_array[$#result_array]->{"control_writes_failed"} = $2;
        } elsif($line =~ /^(\d*),\s(\d*)/){ # 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0
           my $shelf_bay_list = $line;
           $result_array[$#result_array]->{"shelf_bay_list"} = $shelf_bay_list;
        } elsif($line =~ /^(with)\serror:\s(.*)/){ #  with error: none
           $result_array[$#result_array]->{"shelf_bay_list_with_error"} = $2;
        } elsif($line =~ /Power Supply installed element list:\s(.*);\swith error:\s(.*)/){ # Power Supply installed element list: 1, 2; with error: none
           my $temp = $1;
           my @temp_array = split(/,/,$temp);
           $last_power_element = $temp_array[$#temp_array];
           $result_array[$#result_array]->{"power_supply_installed_element_list"} = $1;
           $result_array[$#result_array]->{"power_supply_installed_with_error"} = $2;
        } elsif($line =~ /Power Supply information by element:/){ #Power Supply information by element:
           $power_flag = 1;
        } elsif(($power_flag) && ($line =~ /\s*\[(\d*)\]\s*Serial number:\s*(.*)\s*Part number:\s*(.*)/)){ #   [1] Serial number: XXT103302881  Part number: 114-00065+A0\n
           $power_number = $1;
           $power_hash{"power_element_number"} = $power_number;
           $power_hash{"power_element_serial_number"} = $2;
           $power_hash{"power_element_part_number"} = $3;
        } elsif(($power_flag) && ($line =~ /Type:\s*(.*)/)) { # Type: 9C
           $power_hash{"power_element_type"} = $1;
        } elsif(($power_flag) && ($line =~ /Firmware version:\s*(.*)\s*Swaps:\s*(.*)/)){ # Firmware version: 020F  Swaps: 0
           $power_hash{"power_element_firmware_version"} = $1;
           $power_hash{"power_element_swaps"} = $2;
           my %temp_hash = %power_hash;
           push(@power_array,\%temp_hash);
           my @temp_array = @power_array;
           $result_array[$#result_array]->{"power_supply_information_by_element"} = \@temp_array;
           if($last_power_element == $power_number){
             $power_flag = 0;
             @power_array = ();
           }
        } elsif($line =~ /Voltage Sensor installed element list:\s(.*);\swith\serror:\s(.*)/){ #tVoltage Sensor installed element list: 1, 2, 3, 4, 5, 6, 7, 8; with error: none
           my $temp = $1;
           my @temp_array = split(/,/,$temp);
           $last_voltage_element = $temp_array[$#temp_array];
           $result_array[$#result_array]->{"voltage_sensor_installed_element_list"} = $1;
           $result_array[$#result_array]->{"voltage_sensor_installed_with_error"} = $2;
        } elsif($line =~ /Shelf voltages by element:/){
           $voltage_flag = 1;
        } elsif(($voltage_flag) && ($line =~ /\s*\[(\d*)\]\s(\d*\.\d*)\sVolts\s(.*\svoltage\srange)/)){#  [1] 5.07 Volts Normal voltage range
             my $number = $1;
             $voltage_hash{"voltage_element_number"} = $number;
             $voltage_hash{"voltage_element_volts"} = $2;
             $voltage_hash{"voltage_element_range"} = $3;
             my %temp_hash = %voltage_hash;
             push(@voltage_array,\%temp_hash);
             my @temp_array = @voltage_array;
             $result_array[$#result_array]->{"shelf_voltages_by_element"} = \@temp_array;
             if($last_voltage_element == $number){
               $voltage_flag = 0;
               @voltage_array = ();
             }
        } elsif($line =~ /Current Sensor installed element list:\s(.*);\swith\serror:\s(.*)/){ # Current Sensor installed element list: 1, 2, 3, 4, 5, 6, 7, 8; with error: none\n
           my $temp = $1;
           my @temp_array = split(/,/,$temp);
           $last_current_element = $temp_array[$#temp_array];
           $result_array[$#result_array]->{"current_sensor_installed_element_list"} = $1;
           $result_array[$#result_array]->{"current_sensor_installed_with_error"} = $2;
        } elsif($line =~ /Shelf currents by element:/){
           $current_flag = 1;
        } elsif(($current_flag) && ($line =~ /\s*\[(\d*)\]\s(\d*)\smA\s(.*\scurrent\srange)/)) { # [1] 1480 mA Normal current range
            $current_number = $1;
            $current_hash{"current_element_number"} = $current_number;
            $current_hash{"current_element_current"} = $2;
            $current_hash{"current_element_range"} = $3;
            my %temp_hash = %current_hash;
            push(@current_array,\%temp_hash);
            my @temp_array = @current_array;
            $result_array[$#result_array]->{"shelf_currents_by_element"} = \@temp_array;
            if($last_current_element == $current_number){
               $current_flag = 0;
               @current_array = ();
            }
        } elsif($line =~ /Auto power reset enabled:\s(.*)/){ #  Auto power reset enabled: no
           $result_array[$#result_array]->{"auto_power_reset_enabled"} = $1;
        } elsif($line =~ /Power control element status:\s(.*)/){ # Power control element status: ok
          $result_array[$#result_array]->{"power_control_element_status"} = $1;
        } elsif($line =~ /Cooling Unit installed element list:\s(.*);\swith error:\s(.*)/){ # Cooling Unit installed element list: 1, 2; with error: none
          $result_array[$#result_array]->{"cooling_unit_installed_element_list"} = $1;
          my $list = $1; 
          my @list_array = split(/,/,$list);
          #print "list array is @list_array";
          $last_cooling_element = $list_array[$#list_array];
          #print "last cooling element is $last_cooling_element";
          $result_array[$#result_array]->{"cooling_unit_installed_element_with_error"} = $2;
        } elsif($line =~ /Cooling Units by element:/){
          $cooling_flag = 1;
        } elsif(($cooling_flag) && ($line =~ /\s*\[(\d*)\]\s*(\d*)\s*RPM/)){ # [1] 5510 RPM
           $cool_number = $1;
           $cool_hash{"cooling_element_number"} = $cool_number;
           $cool_hash{"cooling_element_rpm"} = $2;
           my %temp_hash = %cool_hash;
           push(@cooling_array,\%temp_hash);
           my @temp_array = @cooling_array;
           $result_array[$#result_array]->{"cooling_units_by_element"} = \@temp_array;
           if($last_cooling_element == $cool_number){
              $cooling_flag = 0;
              @cooling_array = ();
           }
        } elsif($line =~ /Temperature Sensor installed element list:\s(.*);\swith error:\s(.*)/){ # Temperature Sensor installed element list: 1, 2, 3; with error: none
           my $temp = $1;
           my @temp_array = split(/,/,$temp);
           $last_temparature_element = $temp_array[$#temp_array];
           $result_array[$#result_array]->{"temperature_sensor_installed_element_list"} = $1;
           $result_array[$#result_array]->{"temperature_sensor_installed_element_with_error"} = $2;
        } elsif ($line =~ /Temperature thresholds by element:/) {
           # Clear $current_temparetures and set
           # $temperature_thresholds_flag which needs to be a global
           # also
           $temperature_thresholds_flag = 1;
       } elsif(($temperature_thresholds_flag) && ($line =~ /\s*\[(\d*)\]\sHigh critical:\s*(\d*\sC\s\(\d*\sF\));\s*high warning\s*(\d*\sC\s\(\d*\sF\))/)) {
           # [1] High critical: 42 C (107 F); high warning 40 C (104 F)\n
           # will match first line of threshold for each item
           # need to save $1 as $last_temperature_sensor_read for
           # second line processing

           $last_temperature_sensor_read = $1;
           
           # add high critical temps and high warning temps to the
           # elements
           my $temp = $1;
           $temparature_hash{"temparature_element_number"} = $temp;
           $temparature_hash{"temparature_element_high_critical"} = $2;
           $temparature_hash{"temparature_element_high_warning"} = $3;
       } elsif (($temperature_thresholds_flag) && ($line =~ /\s*Low critical:\s*(\d*\s*C\s\(\d*\sF\));\s*low warning\s*(\d*\sC\s\(\d*\sF\))/)) {
           # use $last_temperature_sensore_read as key
           $temparature_hash{"temparature_element_low_critical"} = $1;
           $temparature_hash{"temparature_element_low_warning"} = $2;
           my %temp_hash = %temparature_hash;
           push(@temparature_array,\%temp_hash);
           my @temp_array = @temparature_array;
           $result_array[$#result_array]->{"temperature_thresholds_by_element"} = \@temp_array; 
           if($last_temparature_element == $last_temperature_sensor_read){
              $temperature_thresholds_flag = 0;
              @temparature_array = ();
           }
       } elsif($line =~ /Shelf temperatures by element:/) {
           # identifies that the next things seen will be current
           # temperatures set a flag to track.
           # this needs to be added with the $temperature_sensors_flag.
           $current_temperatures_flag = 1;
       } elsif(($current_temperatures_flag) && ($line =~ /\s*\[(\d*)\]\s(\d*\sC\s\(\d*\sF\))\s(.*)/)){ # [2] 24 C (75 F)  Normal temperature range
            # this line will give key for $temperature_elements as $1
            # C and F temperatures in $2
            $element = $1;
            $temperature_elements{"shelf_temperature_element_number"} = $element;
            $temperature_elements{"shelf_element_temparature"} = $2;
            $temperature_elements{"shelf_element_range"} = $3;
            my %temp_hash = %temperature_elements;
            push(@shelf_temp_array,\%temp_hash);
            my @temp_array = @shelf_temp_array;
            $result_array[$#result_array]->{"shelf_temperatures_by_element"} = \@temp_array;
            if($last_temparature_element == $element){
               $current_temperatures_flag = 0;
               @shelf_temp_array = ();
            }
        } elsif($line =~ /SAS connector attached element list:\s(.*);\swith error:\s(.*)/) {# SAS connector attached element list: 2, 4; with error: none
          my $temp = $1;
          my @temp_array = split(/,/,$temp);
          $last_sas_element = $temp_array[$#temp_array];
          $result_array[$#result_array]->{"sas_connector_attached_element_list"} = $1;
          $result_array[$#result_array]->{"sas_connector_attached_element_with_error"} = $2;
        } elsif($line =~ /SAS cable information by element:/) {
          $sas_flag = 1;        
        } elsif(($sas_flag) && ($line =~ /\s*\[(\d*)\]\s*Vendor:\s*(.*)/)){ # [1] Vendor: Molex Inc.\n
          $sas_number = $1;
          $sas_hash{"sas_element_number"} = $sas_number;
          $sas_hash{"sas_vendor"} = $2;
        } elsif(($sas_flag) && ($line =~ /\s*Type:\s*(.*)ID:\s*(.*)\s*Swaps:\s*(.*)/)){ # Type: QSFP active copper 20m  ID: 01  Swaps: 0
          $sas_hash{"sas_type"} = $1;
          $sas_hash{"sas_id"} = $2;
          $sas_hash{"sas_swaps"} = $3;
        } elsif(($sas_flag) && ($line =~ /\s*Serial number:\s*(.*)\s*Part number:\s*(.*)/)) { # Serial number: 009920177  Part number: 112-00177+A0
          $sas_hash{"sas_serial_number"} = $1;
          $sas_hash{"sas_part_number"} = $2;
          my %temp_hash = %sas_hash;
          push(@sas_array,\%temp_hash);
          my @temp_array = @sas_array;
          $result_array[$#result_array]->{"sas_cable_information_by_element"} = \@temp_array;
          if($last_sas_element == $sas_number){
               $sas_flag = 0;
               @sas_array = ();
            }
        } elsif($line =~ /ACP installed element list:\s(.*);\swith error:\s(.*)/) {# ACP installed element list: 1, 2; with error: none
          my $temp = $1;
          my @temp_array = split(/,/,$temp);
          $last_acp_element = $temp_array[$#temp_array];
          $result_array[$#result_array]->{"acp_installed_element_list"} = $1;
          $result_array[$#result_array]->{"acp_installed_element_with_error"} = $2;
        } elsif($line =~ /ACP information by element:/) {
          $acp_flag = 1;        
        } elsif(($acp_flag) && ($line =~ /\s*\[(\d*)\]\s*MAC address:\s*(.*)/)){ # [1] MAC address: 00:50:CC:62:63:E5
          my $acp_number = $1;
          $acp_hash{"acp_element_number"} = $acp_number;
          $acp_hash{"acp_element_mac_address"} = $2;
           my %temp_hash = %acp_hash;
           push(@acp_array,\%temp_hash);
           my @temp_array = @acp_array;
          $result_array[$#result_array]->{"acp_information_by_element"} = \@temp_array;
          if($last_acp_element == $acp_number){
               $acp_flag = 0;
               @acp_array = ();
            }
        } elsif($line =~ /SAS Expander Module installed element list:\s(.*);\swith error:\s(.*)/) {# SAS Expander Module installed element list: 1, 2; with error: none
          $result_array[$#result_array]->{"sas_expander_module_installed_element_list"} = $1;
          $result_array[$#result_array]->{"sas_expander_module_installed_element_with_error"} = $2;
        } elsif($line =~ /ES Electronics installed element list:\s(.*);\swith error:\s(.*)/){ # ES Electronics installed element list: 1, 2; with error: none
          my $temp = $1;
          my @temp_array = split(/,/,$temp);
          $last_es_element = $temp_array[$#temp_array];
          $result_array[$#result_array]->{"es_electronics_installed_element_list"} = $1;
          $result_array[$#result_array]->{"es_electronics_installed_element_with_error"} = $2;
        } elsif($line =~ /ES Electronics reporting element:\s*(.*)/){ # ES Electronics reporting element: 1\
          $result_array[$#result_array]->{"es_electronics_reporting_element"} = $1;
        } elsif($line =~ /ES Electronics information by element:/){ # ES Electronics information by element:
          $power_flag = 0;
          $es_flag = 1;
		} elsif(($es_flag) && ($line =~ /\s*\[(\d*)\]\s*Serial number:\s*(.*)\s*Part number:\s*(.*)/)){ # [1] Serial number: 8000100264  Part number: 111-00690+A1\n [1] Serial number: IMS65228003068F  Part number: <N/A>\n
          $es_number = $1;
          $es_hash{"es_element_number"} = $es_number;
          $es_hash{"es_element_serial_number"} = $2;
          $es_hash{"es_element_part_number"} = $3;
		} elsif(($es_flag) && ($line =~ /\s*CPLD version:\s*(.*)\s*Swaps:\s*(.*)/)){ #CPLD version: 23  Swaps: 0 CPLD version: <N/A>  Swaps: 0\n
          $es_hash{"es_element_cpld_version"} = $1;
          $es_hash{"es_element_swaps"} = $2;
          my %temp_hash = %es_hash;
		  push(@es_array,\%temp_hash);
          my @temp_array = @es_array;
          $result_array[$#result_array]->{"es_electronics_information_by_element"} = \@temp_array;
          if($last_es_element == $es_number){
             $es_flag = 0;
             @es_array = ();
          }
        } elsif($line =~ /Embedded Switching Hub installed element list:\s(.*);\swith error:\s(.*)/){ # Embedded Switching Hub installed element list: 1, 2; with error: none
          $result_array[$#result_array]->{"embedded_switching_hub_installed_element_list"} = $1;
          $result_array[$#result_array]->{"embedded_switching_hub_installed_element_with_error"} = $2;
        }
    } # end foreach
    return \@result_array;
}

## @name ::environment_status_shelf_log
##
## @summary Parses the raw output from the "environment status shelf_log" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from environment_status_shelf_log with any of its options as described in the
##    corresponding cdef file 'environ.cdefs'
## @synopsis
##      environment_status_shelf_log(%args);
## INPUTS TO METHOD CALL 'environment_status_shelf_log' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; environment status shelf_log; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub environment_status_shelf_log {
    my $error_message;
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(raw_output    => $raw_output,
                             command       => "environment status shelf_log",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no error, then it means that the command had
    # executed successfully
    return [];
}

## @name ::environment_status_shelf_power_status
##
## @summary Parses the raw output from the "environment status shelf_power_status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from environment_status_shelf_power_status with any of its options as described in the
##    corresponding cdef file 'environ.cdefs'
## @synopsis
##      environment_status_shelf_power_status(%args);
## INPUTS TO METHOD CALL 'environment_status_shelf_power_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; environment status shelf_power_status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub environment_status_shelf_power_status {
    my $error_message;
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(raw_output    => $raw_output,
                             command       => "environment status shelf_power_status",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no error, then it means that the command had
    # executed successfully
    return [];
}
## @name ::environment_status_shelf_stats
##
## @summary Parses the raw output from the "environment status shelf_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from environment_status_shelf_stats with any of its options as described in the
##    corresponding cdef file 'environ.cdefs'
## @synopsis
##      environment_status_shelf_stats(%args);
## INPUTS TO METHOD CALL 'environment_status_shelf_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; environment status shelf_stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;--------------------------------------------------------------
##&nbsp;Channel: 0a Shelf: 1 Module type: AT-FCX Firmware rev: 38
##&nbsp;Shelf Serial Number: OPS8248721FDC2A
##&nbsp;Module A Serial Number: IMS828990069B2B
##&nbsp;Timestamp: Sat Jan 28 20:48:28 GMT 2012
##&nbsp;--------------------------------------------------------------
##&nbsp;
##&nbsp; ATA          Loss of   Invalid   Invalid   Invalid
##&nbsp;Disk           signal    TX/CRC    RX/CRC   TX/Word
##&nbsp;  ID            count     count     count     count
##&nbsp;0a.16               0         0         0         0
##&nbsp;0a.17               0         0         0         0
##&nbsp;0a.18               0         0         0         0
##&nbsp;0a.19               0         0         0         0
##&nbsp;0a.20               0         0         0         0
##&nbsp;0a.21               0         0         0         0
##&nbsp;0a.22               0         0         0         0
##&nbsp;0a.23               0         0         0         0
##&nbsp;0a.24               0         0         0         0
##&nbsp;0a.25               0         0         0         0
##&nbsp;0a.26               0         0         0         0
##&nbsp;0a.27               0         0         0         0
##&nbsp;0a.28               0         0         0         0
##&nbsp;0a.29               0         0         0         0
##&nbsp;
##&nbsp;Internal         Link   Loss of     Loss of   Primitive   Invalid   Invalid
##&nbsp;    Link      Failure      sync      signal    Protocol   TX/Word    RX/CRC
##&nbsp;  Errors        count     count       count   Err count     count     count
##&nbsp;HI/HO               0       151           0         0      1219         0
##&nbsp;A->B                1         1           0         0         5         0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            '0a' => [
##&nbsp;                      {
##&nbsp;                        'module_A' => [
##&nbsp;                                        {
##&nbsp;                                          'disk_details' => [
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.16'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.17'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.18'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.19'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.20'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.21'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.22'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.23'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.24'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.25'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.26'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.27'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.28'
##&nbsp;                                                              },
##&nbsp;                                                              {
##&nbsp;                                                                'invalid_tx_word_count' => '0',
##&nbsp;                                                                'invalid_tx_crc_count' => '0',
##&nbsp;                                                                'invalid_rx_crc_count' => '0',
##&nbsp;                                                                'loss_of_signal_count' => '0',
##&nbsp;                                                                'ata_disk_id' => '0a.29'
##&nbsp;                                                              }
##&nbsp;                                                            ],
##&nbsp;                                          'error_details' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'invalid_tx_word_count' => '0',
##&nbsp;                                                                 'invalid_tx_crc_count' => '1219',
##&nbsp;                                                                 'loss_of_sync_count' => '151',
##&nbsp;                                                                 'primitive_protocol_err_count' => '0',
##&nbsp;                                                                 'link_failure_count' => '0',
##&nbsp;                                                                 'loss_of_signal_count' => '0',
##&nbsp;                                                                 'internal_link_errors' => 'HI/HO'
##&nbsp;                                                               }
##&nbsp;                                                             ]
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;                        'channel_details' => [
##&nbsp;                                               {
##&nbsp;                                                 'module_b_serial_number' => 'IMS828990069B65',
##&nbsp;                                                 'timestamp' => 'Sat Jan 28 20:48:28 GMT 2012',
##&nbsp;                                                 'shelf' => '1',
##&nbsp;                                                 'shelf_serial_number' => 'OPS8248721FDC2A',
##&nbsp;                                                 'module_type' => 'AT-FCX',
##&nbsp;                                                 'channel' => '0a',
##&nbsp;                                                 'module_a_serial_number' => 'IMS828990069B2B',
##&nbsp;                                                 'firmware_rev' => '38'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                      }
##&nbsp;                    ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub  environment_status_shelf_stats{
    my $error_message;
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(raw_output    => $raw_output,
                             command       => "environment status shelf_stats",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    my @lines = get_lines( raw_output => $raw_output, response_object => $response );
    my $i=0;
    my $j=0;
    my $key="";
    my @result_array;
    my $key2= "";

    foreach my $line (@lines) {
       if($line =~/[\-]+/ )
       {
           if($line =~ /^(Channel):\s(\S+)\s(\w+):\s(\d+)\s(\w+\s\w+):\s(\S+)\s(\w+\s\w+):\s(\d+)/){
               $key= "channel_".$2;
               $i=0;
               $j=0;
               $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{"channel"}=$2;
               $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{"shelf"}=$4;
               $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{"module_type"}=$6;
               $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{"firmware_rev"}=$8;
           }
       }elsif($line =~ /((Module)\s(\S+)[\s\w]+):\s(.*)/){
          if($2 eq "Module"){
               $key2 ="module_".$3;
               $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{to_key_name( item => $1 )}=$4;
          }
       }elsif ($line =~ /^(Shelf\sSerial\s\w+|Timestamp|Request\s\w+):\s(.*)/){
                $result_array[0]->{$key}->[0]->{"channel_details"}->[0]->{to_key_name( item => $1 )}=$2;
       }elsif ($line =~ /(\w+\.\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/){
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"disk_details"}->[$i]->{"ata_disk_id"}=$1;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"disk_details"}->[$i]->{"loss_of_signal_count"}=$2;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"disk_details"}->[$i]->{"invalid_tx_crc_count"}=$3;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"disk_details"}->[$i]->{"invalid_rx_crc_count"}=$4;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"disk_details"}->[$i]->{"invalid_tx_word_count"}=$5;
                $i++;
       }elsif($line =~ /^(\w+\/\w+|\w->\w)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/){
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"internal_link_errors"}=$1;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"link_failure_count"}=$2;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"loss_of_sync_count"}=$3;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"loss_of_signal_count"}=$4;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"primitive_protocol_err_count"}=$5;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"invalid_tx_crc_count"}=$6;
                $result_array[0]->{$key}->[0]->{$key2}->[0]->{"error_details"}->[$j]->{"invalid_tx_word_count"}=$7;
                $j++;
       }

   }
return \@result_array;
}

## @name ::environment_status_chassis
##
## @summary Parses the raw output from the "environment status chassis" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from environment_status_chassis with any of its options as described in the
##    corresponding cdef file 'environ.cdefs'
## @synopsis
##      environment_status_chassis(%args);
## INPUTS TO METHOD CALL 'environment_status_chassis' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'list-sensors' </td> <td> 'list-sensors' </td> <td> '1' </td> </tr>
## <tr> <td> '"SYS FAN"' </td> <td> 'sys-fan' </td> <td> '1' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> 'Voltage' </td> <td> 'voltage' </td> <td> '1' </td> </tr>
## <tr> <td> '"PSU 1"' </td> <td> 'psu-1' </td> <td> '1' </td> </tr>
## <tr> <td> 'NVRAM7-temperature-0' </td> <td> 'nvram7-temperature-0' </td> <td> '1' </td> </tr>
## <tr> <td> 'NVRAM7-battery-0' </td> <td> 'nvram7-battery-0' </td> <td> '1' </td> </tr>
## <tr> <td> '"PSU 2"' </td> <td> 'psu-2' </td> <td> '1' </td> </tr>
## <tr> <td> 'Temperature' </td> <td> 'temperature' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; environment status chassis all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Voltage ok
##&nbsp;PSU 1 ok
##&nbsp;PSU 2 ok
##&nbsp;Temperature ok
##&nbsp;System_Fan ok
##&nbsp;CF_status ok
##&nbsp;NVRAM6-temperature-1 ok
##&nbsp;NVRAM6-battery-1 OK
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nvram6_temperature_1' => 'ok',
##&nbsp;            'psu_1' => 'ok',
##&nbsp;            'temperature' => 'ok',
##&nbsp;            'system_fan' => 'ok',
##&nbsp;            'voltage' => 'ok',
##&nbsp;            'psu_2' => 'ok',
##&nbsp;            'cf_status' => 'ok',
##&nbsp;            'nvram6_battery_1' => 'OK'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; environment status chassis "PSU 1"; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PSU 1 ok
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'psu_1' => 'ok'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; environment status chassis list-sensors; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Sensor Name              State          Current    Critical     Warning     Warning    Critical
##&nbsp;                                        Reading       Low         Low         High       High
##&nbsp;-------------------------------------------------------------------------------------------------
##&nbsp;CPU3.3V Active           normal          3339 mV       --        3036 mV     3564 mV       --
##&nbsp;CPU5V Active             normal          5005 mV       --        4500 mV     5500 mV       --
##&nbsp;CPU12V                   normal         12000 mV       --       10800 mV    13200 mV       --
##&nbsp;CPU3.3V Standby          normal          3356 mV       --        3036 mV     3564 mV       --
##&nbsp;CPU5V Standby            normal          5013 mV       --        4500 mV     5500 mV       --
##&nbsp;RTC Battery              normal          2805 mV       --        2800 mV     3500 mV       --
##&nbsp;IOB 3.3V Standby         normal          3339 mV       --        3036 mV     3564 mV       --
##&nbsp;IOB +12V                 normal         12062 mV       --       10800 mV    13200 mV       --
##&nbsp;IOB 5V                   normal          5039 mV       --        4500 mV     5500 mV       --
##&nbsp;IOB 3.3V Active          normal          3373 mV       --        3036 mV     3564 mV       --
##&nbsp;IOB HTLE PCI-EX          normal          1175 mV       --        1104 mV     1296 mV       --
##&nbsp;IOB VLDT                 normal          1194 mV       --        1104 mV     1296 mV       --
##&nbsp;IOB HTLE Core            normal          1194 mV       --        1104 mV     1296 mV       --
##&nbsp;CPU0 Core Voltage        normal          1419 mV       --        1308 mV     1492 mV       --
##&nbsp;CPU1 Core Voltage        normal          1417 mV       --        1308 mV     1492 mV       --
##&nbsp;CPU0&1 VDD25             normal          2578 mV       --        2325 mV     2675 mV       --
##&nbsp;CPU0&1 VTT               normal          1289 mV       --        1162 mV     1338 mV       --
##&nbsp;RLM Standby Voltage      normal          3316 mV       --        3036 mV     3564 mV       --
##&nbsp;HTLE SB1 Core Voltage    normal          1216 mV       --        1104 mV     1296 mV       --
##&nbsp;HTLE VLDT Voltage        normal          1190 mV       --        1104 mV     1296 mV       --
##&nbsp;HTLE PCI-Ex Voltage      normal          1193 mV       --        1104 mV     1296 mV       --
##&nbsp;FC Core Voltage          normal          1787 mV       --        1656 mV     1944 mV       --
##&nbsp;FC Bridge Core Voltage   normal          1510 mV       --        1380 mV     1620 mV       --
##&nbsp;FC Bridge Ref Voltage    normal          2482 mV       --        2300 mV     2700 mV       --
##&nbsp;FC 3.3 Voltage           normal          3309 mV       --        3036 mV     3564 mV       --
##&nbsp;PSU1 present             normal           good
##&nbsp;PSU1 AC status           normal           good
##&nbsp;PSU 1 12V                normal         12000 mV       --       10800 mV    13200 mV    15900 mV
##&nbsp;PSU 1 5V                 normal          5248 mV       --        4500 mV     5836 mV       --
##&nbsp;PSU 1 AC Voltage         normal         223000 mV       --          --          --          --
##&nbsp;PSU 1 Temperature        normal            32 C        --          --          55 C        75 C
##&nbsp;PSU 1 FAN 1              normal          4560 RPM    2800 RPM    3000 RPM      --          --
##&nbsp;PSU 1 FAN 2              normal          4354 RPM    2800 RPM    3000 RPM      --          --
##&nbsp;PSU2 present             normal           good
##&nbsp;PSU2 AC status           normal           good
##&nbsp;PSU 2 12V                normal         12000 mV       --       10800 mV    13200 mV    15900 mV
##&nbsp;PSU 2 5V                 normal          5221 mV       --        4500 mV     5836 mV       --
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'list_sensors' => [
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3339 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => 'CPU3.3V Active'
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '4500 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '5005 mV',
##&nbsp;                                  'warning_high' => '5500 mV',
##&nbsp;                                  'sensor_name' => 'CPU5V Active',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '10800 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '12000 mV',
##&nbsp;                                  'warning_high' => '13200 mV',
##&nbsp;                                  'sensor_name' => 'CPU12V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3356 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => '3V Standby',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '4500 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '5013 mV',
##&nbsp;                                  'warning_high' => '5500 mV',
##&nbsp;                                  'sensor_name' => 'CPU5V Standby',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '2800 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '2805 mV',
##&nbsp;                                  'warning_high' => '3500 mV',
##&nbsp;                                  'sensor_name' => 'RTC Battery',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3339 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => '3V Standby',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '10800 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '12062 mV',
##&nbsp;                                  'warning_high' => '13200 mV',
##&nbsp;                                  'sensor_name' => '+12V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '4500 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '5039 mV',
##&nbsp;                                  'warning_high' => '5500 mV',
##&nbsp;                                  'sensor_name' => 'IOB 5V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3373 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => '3V Active',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1175 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'PCI-EX',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1194 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'IOB VLDT',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1194 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'HTLE Core',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1308 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1419 mV',
##&nbsp;                                  'warning_high' => '1492 mV',
##&nbsp;                                  'sensor_name' => 'Core Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1308 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1417 mV',
##&nbsp;                                  'warning_high' => '1492 mV',
##&nbsp;                                  'sensor_name' => 'Core Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '2325 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '2578 mV',
##&nbsp;                                  'warning_high' => '2675 mV',
##&nbsp;                                  'sensor_name' => '1 VDD25',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1162 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1289 mV',
##&nbsp;                                  'warning_high' => '1338 mV',
##&nbsp;                                  'sensor_name' => '1 VTT',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3316 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => 'Standby Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1216 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'Core Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1190 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'VLDT Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1104 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1193 mV',
##&nbsp;                                  'warning_high' => '1296 mV',
##&nbsp;                                  'sensor_name' => 'Ex Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1656 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1787 mV',
##&nbsp;                                  'warning_high' => '1944 mV',
##&nbsp;                                  'sensor_name' => 'Core Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '1380 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '1510 mV',
##&nbsp;                                  'warning_high' => '1620 mV',
##&nbsp;                                  'sensor_name' => 'Core Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '2300 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '2482 mV',
##&nbsp;                                  'warning_high' => '2700 mV',
##&nbsp;                                  'sensor_name' => 'Ref Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3036 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '3309 mV',
##&nbsp;                                  'warning_high' => '3564 mV',
##&nbsp;                                  'sensor_name' => '3 Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'current_reading' => 'good',
##&nbsp;                                  'sensor_name' => 'PSU1 present',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'current_reading' => 'good',
##&nbsp;                                  'sensor_name' => 'PSU 1 AC status',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '10800 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '15900 mV',
##&nbsp;                                  'current_reading' => '12000 mV',
##&nbsp;                                  'warning_high' => '13200 mV',
##&nbsp;                                  'sensor_name' => 'PSU 1 12V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '4500 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '5248 mV',
##&nbsp;                                  'warning_high' => '5836 mV',
##&nbsp;                                  'sensor_name' => 'PSU 1 5V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '--',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '223000 mV',
##&nbsp;                                  'warning_high' => '--',
##&nbsp;                                  'sensor_name' => 'PSU 1 AC Voltage',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '--',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '75 C',
##&nbsp;                                  'current_reading' => '32 C',
##&nbsp;                                  'warning_high' => '55 C',
##&nbsp;                                  'sensor_name' => 'PSU 1 Temperature',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3000 RPM',
##&nbsp;                                  'critical_low' => '2800 RPM',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '4560 RPM',
##&nbsp;                                  'warning_high' => '--',
##&nbsp;                                  'sensor_name' => 'PSU 1 FAN 1',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '3000 RPM',
##&nbsp;                                  'critical_low' => '2800 RPM',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '4354 RPM',
##&nbsp;                                  'warning_high' => '--',
##&nbsp;                                  'sensor_name' => 'PSU 1 FAN 2',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'current_reading' => 'good',
##&nbsp;                                  'sensor_name' => 'PSU2 present',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'current_reading' => 'good',
##&nbsp;                                  'sensor_name' => 'PSU 2 AC status',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '10800 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '15900 mV',
##&nbsp;                                  'current_reading' => '12000 mV',
##&nbsp;                                  'warning_high' => '13200 mV',
##&nbsp;                                  'sensor_name' => 'PSU 2 12V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'warning_low' => '4500 mV',
##&nbsp;                                  'critical_low' => '--',
##&nbsp;                                  'critical_high' => '--',
##&nbsp;                                  'current_reading' => '5221 mV',
##&nbsp;                                  'warning_high' => '5836 mV',
##&nbsp;                                  'sensor_name' => 'PSU 2 5V',
##&nbsp;                                  'state' => 'normal'
##&nbsp;                                },
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub  environment_status_chassis{
    my $error_message;
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my @result_array;
    my $i=0;

    my $error = _check_error(raw_output    => $raw_output,
                             command       => "environment status chassis",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    my @lines = get_lines( raw_output => $raw_output, response_object => $response );
    foreach my $line (@lines) {
		if($line =~ /(.*)\s(OK|GOOD|PRESENT|NOT_PRESENT|IPMI_HB_OK)$/i){
			my $key = $1;
			my $value = $2;
			$key =~ s/\s+$//;
			$result_array[0]->{to_key_name( item => $key )} = $value;
		} elsif($line =~ /(.+?)\s+(normal|critlow|warnhigh|crithigh|warnlow|extrlow|extrhigh|)\s+(\S+\s\S+|good|OK\n|PRESENT\n)\s*((\S+\s\S+|[\-]+)\s+(\S+\s\S+|[\-]+)\s+(\S+\s\S+|[\-]+)\s+(\S+\s\S+|[\-]+)|)/){
			if($3 eq "good"){
				$result_array[0]->{list_sensors}->[$i]->{sensor_name} = $1;
				$result_array[0]->{list_sensors}->[$i]->{state} = $2;
				$result_array[0]->{list_sensors}->[$i]->{current_reading} = $3;
		   } else{
				$result_array[0]->{list_sensors}->[$i]->{sensor_name} = $1;
				$result_array[0]->{list_sensors}->[$i]->{state} = $2;
				$result_array[0]->{list_sensors}->[$i]->{current_reading} = $3;
				$result_array[0]->{list_sensors}->[$i]->{critical_low} = $5;
				$result_array[0]->{list_sensors}->[$i]->{warning_low} = $6;
				$result_array[0]->{list_sensors}->[$i]->{warning_high} = $7;
				$result_array[0]->{list_sensors}->[$i]->{critical_high} = $8;
			}
			$i++;
		}
    }
    return \@result_array;
}

1;
