#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/exportfs.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode exportfs command output
##
## @description
##    This is a library to parse the raw output of 7Mode exportfs command.
##
## @synopsis
##    use exportfs;
##
##    exportfs(%args);
##
## @author dl-nacl-dev@netapp.com, swati.bajaj@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::exportfs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command          = $args{command};
    my $error;
    if ( $raw_output =~ /no such directory|Invalid argument|Not currently exported|Bad subnet entry|Invalid access check|There is no storage associated with .*|trying to overwrite (.+) which is a directory/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /option requires an argument|Invalid option/ ) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    # } elsif ($raw_output =~ /NFS not licensed/) {
        # $error = "FeatureNotLicensed";
        # $$error_message = $raw_output;
    } elsif ( $raw_output =~ /[Uu]sage:\s(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage or dependencies";
    }
    return $error;
}

## @name ::exportfs
##
## @summary Parses the raw output from the "exportfs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from exportfs with any of its options as described in the
##    corresponding cdef file 'exportfs.cdefs'
## @synopsis
##      exportfs(%args);
## INPUTS TO METHOD CALL 'exportfs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'permissions' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> 'allhosts' </td> <td> 'allhosts' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ignore' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zap' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'persistent' </td> <td> '1' </td> </tr>
## <tr> <td> 'save' </td> <td> 'save' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reload' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'clientid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'clientaddr' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'check' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> 'nosave' </td> <td> 'nosave' </td> <td> '1' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'write' </td> <td> '1' </td> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'query' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'persistent-options' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'fence' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secflavor' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'unexport' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'revert' </td> <td> '1' </td> </tr>
## <tr> <td> 'allpaths' </td> <td> 'allpaths' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'storage' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = exportfs -q /vol/vol1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'export_options' => '-sec=sys,(ruleid=0),rw,nosuid',
##&nbsp;            'path' => 'vol/vol1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; exportfs -q /vol/vol2; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'export_options' => '-sec=sys,(ruleid=2),rw',
##&nbsp;            'path' => 'vol/vol2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub exportfs {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output =~ /^\/(.*)\s+(.*)/m ) {
    # parser for exportfs (without options)
        my $first_line = qr/^\/(.*)\s+(.*)/m;
        my $can_throw = 1;
        my @lines = get_lines(raw_output => $output, response_object => $resp, 'first_line' => $first_line, 'can_throw'  => $can_throw);
        foreach my $line (@lines) {
            my %hash = ();
            if ($line =~ /^\/(.*)\s+(.*)/m) {
                $hash{path} = $1;
                $hash{export_options} = $2;
                push (@result_array, \%hash);
            } else {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Output is not in the expected format",
                    response_object => $resp
                );
            }
        }
        return \@result_array;
    } elsif ( $output =~ /The storage of (.+)\s+is\s+(.+)/) {
        #Parser for exportfs -s
        my %hash;
        $hash{path} = $1;
        $hash{storage} = $2;
        push (@result_array, \%hash);
        return \@result_array;
    } else {
        # No parser support for exportfs with options (Is it required?)
        return \@result_array;
    }
}
1;
