#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode fcadmin commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode fcadmin commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "fcadmin online" will be fcadmin_online()
##
## @synopsis
##    use fcadmin;
##
##    fcadmin_online(%args);
##    fcadmin_offline(%args);
##    fcadmin_link_state(%args);
##    fcadmin_config(%args);
##    fcadmin_channels(%args);
##
## @author nagajyot@netapp.com,dl-nacl-dev,xdl-storage-automation,dl-storsub-qa
## @author veerap@netapp.com
##
## @status To_be_reviewed

package NACL::APISet::Node::CLI::7Mode::default::fcadmin;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};

    my $error;
    if ($output =~ /(No such channel)/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~ /[Uu]sage/i) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
    } elsif ($output =~ /(Error: adapter \w+ is not configurable|Error: Invalid type argument:.*|Error: <adapter_name> argument required)/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~ /fcadmin\s+online:\s+Channel\s+(\w+\s+is\s+not\s+OFFLINE)/){
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif (
        $output =~ /fcadmin\s+offline:\s+Channel\s+(\w+\s+is\s+already\s+OFFLINE)/){
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /[Nn]o\s+drives\s+were\s+programmed/){
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /Cannot offline adapter as one or more devices on host adapter.*is currently being sanitized|fcadmin\s+bridge:\s+failed.|[Rr]equest\s+to\s+power.*|[Ss]helf\s+(\S+)\s+does\s+not\s+have\s+power.*|[nN]o\s+disks\s+match.*/){
        $error          = "CommandFailed";
        $$error_message = $output;
    }
    return $error;
} ## end sub _check_error

## @name ::fcadmin_online
##
## @summary Parses the raw output from the "fcadmin online" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_online with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_online(%args);
## INPUTS TO METHOD CALL 'fcadmin_online' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcadmin online 0a
##&nbsp;$VAR1 = [];
##&nbsp;
sub fcadmin_online {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    my $error = _check_error(raw_output    => $output,
                             command       => "fcadmin online",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub fcadmin_online

## @name ::fcadmin_offline
##
## @summary Parses the raw output from the "fcadmin offline" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_offline with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_offline(%args);
## INPUTS TO METHOD CALL 'fcadmin_offline' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcadmin offline 0a
##&nbsp;$VAR1 = [];
##&nbsp;
sub fcadmin_offline {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    my $error = _check_error(raw_output    => $output,
                             command       => "fcadmin offline",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub fcadmin_offline

## @name ::fcadmin_link_state
##
## @summary Parses the raw output from the "fcadmin link_state" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_link_state with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_link_state(%args);
## INPUTS TO METHOD CALL 'fcadmin_link_state' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = fcadmin link_state
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link-state' => 'offline-logical',
##&nbsp;            'adapter-name' => '0a'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port-type' => 'L-port',
##&nbsp;            'link-state' => 'UP',
##&nbsp;            'adapter-name' => '0b'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fcadmin link_state 0a
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link-state' => 'offline-logical',
##&nbsp;            'adapter-name' => '0a'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fcadmin_link_state {
    my %args = @_;

    my ($error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(raw_output    => $output,
                             command       => "fcadmin link_state",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my $first_line = "";
    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          can_throw       => 1,
                          response_object => $resp);

    foreach my $line (@lines) {
            my %hash = ();
            if($line =~ /.+\s(.+):\s(.+)\s\-\s(.+)/){
              $hash{'adapter-name'} = $1;
              $hash{'link-state'} = $2;
              $hash{'port-type'} = $3;
            } elsif($line =~ /.+\s(.+):\s(.+)\s\((.+)\)/){
              $hash{'adapter-name'} = $1;
              $hash{'link-state'} = lc($2)."-".lc($3);
            }
            push(@result_array, \%hash);
    }
    return \@result_array;
} ## end sub fcadmin_link_state

## @name ::fcadmin_config
##
## @summary Parses the raw output from the "fcadmin config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_config with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_config(%args);
## INPUTS TO METHOD CALL 'fcadmin_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = fcadmin config
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'local_state' => 'CONFIGURED.',
##&nbsp;            'type' => 'initiator',
##&nbsp;            'adapter-name' => '0a'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'online',
##&nbsp;            'local_state' => 'CONFIGURED.',
##&nbsp;            'type' => 'initiator',
##&nbsp;            'adapter-name' => '0b'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fcadmin config 0a
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'local_state' => 'CONFIGURED.',
##&nbsp;            'type' => 'initiator',
##&nbsp;            'adapter-name' => '0a'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub fcadmin_config {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "fcadmin config",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $command =~ /-t|-d|-e/ ) {
        return [];
    }
    else {
        my $first_line = "Adapter";
        my @lines      = get_lines(
            raw_output      => $output,
            first_line      => $first_line,
            can_throw       => 1,
            response_object => $resp
        );
        splice( @lines, 0, 2 );
        foreach my $line (@lines) {
            my @words = split( /\s+/, $line );
            my %hash = ();
            $hash{'adapter-name'} = $words[0];
            $hash{'type'}         = $words[1];
            $hash{'local_state'}  = $words[2];
            $hash{'status'}       = $words[3];
            push( @result_array, \%hash );
        }
        return \@result_array;
    }
} ## end sub fcadmin_config

## @name ::fcadmin_channels
##
## @summary Parses the raw output from the "fcadmin channels" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_channels with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_channels(%args);
## INPUTS TO METHOD CALL 'fcadmin_channels' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; fcadmin channels; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'gihaid' => '0',
##&nbsp;            'channel' => 'v0',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '11cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '1',
##&nbsp;            'channel' => 'v1',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '12cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '2',
##&nbsp;            'channel' => 'v2',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '13cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '3',
##&nbsp;            'channel' => 'v3',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '14cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '4',
##&nbsp;            'channel' => 'v4',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '15cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '5',
##&nbsp;            'channel' => 'v5',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '16cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '6',
##&nbsp;            'channel' => 'v6',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '17cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '7',
##&nbsp;            'channel' => 'v7',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '18cc2df200000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'gihaid' => '8',
##&nbsp;            'channel' => 'v8',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '19cc2df200000000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; fcadmin channels v6; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'gihaid' => '6',
##&nbsp;            'channel' => 'v6',
##&nbsp;            'port' => 'L-port ',
##&nbsp;            'fc_node_name' => '17cc2df200000000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub fcadmin_channels {
    my %args = @_;
    my ($error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(raw_output    => $output,
                             command       => "fcadmin channels",
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(raw_output      => $output,
                          response_object => $resp);
    foreach my $line (@lines) {
        my %hash = ();
        if($line =~ /(\w+)\s+(\w+)\s+(.*)\s+\((\w+:\s+(\d+))\)/){
            $hash{'fc_node_name'} = $1 ;
            $hash{'channel'} = $2;
            $hash{'port'} = $3;
            $hash{'gihaid'} = $5;
            push(@result_array, \%hash);
        }
    }
    return \@result_array;
} ## end sub fcadmin_channels

1;
