#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/fcadmin_bridge.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode fcadmin bridge commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode fcadmin bridge commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use fcadmin_bridge;
##
##    fcadmin_bridge(%args);
##
## @author jais@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::fcadmin_bridge;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};

    my $error;
    if ($output =~ /[Uu]sage/i) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
    } elsif ( $output =~ /fcadmin\s+bridge:\s+[Ii]nvalid\s+device.*|fcadmin\s+bridge:\s+[Ii]nvalid\s+loop\s+[iI][dD].*|[Nn]o\s+drives\s+were\s+programmed/){
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /fcadmin\s+bridge:\s+failed.|[Rr]equest\s+to\s+power.*|[Ss]helf\s+(\S+)\s+does\s+not\s+have\s+power.*|[nN]o\s+disks\s+match.*|fcadmin\s+bridge:\s+[Ii]nvalid\s+WWN.*/){
        $error          = "CommandFailed";
        $$error_message = $output;
    }
    return $error;
} ## end sub _check_error

## @name ::fcadmin_bridge
##
## @summary Parses the raw output from the "fcadmin bridge" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcadmin_bridge with any of its options as described in the
##    corresponding cdef file 'fcadmin.cdefs'
## @synopsis
##      fcadmin_bridge(%args);
## INPUTS TO METHOD CALL 'fcadmin_bridge' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'access-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vendor-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cmdstring' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disk-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'octet-number' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'display' </td> <td> '1' </td> </tr>
## <tr> <td> '-cid' </td> <td> 'customer-id' </td> <td> '0' </td> </tr>
## <tr> <td> 'cmd_cemi' </td> <td> 'cmd-cemi' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'ddb-reset' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> 'cmd_sr1216' </td> <td> 'cmd-sr1216' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; fcadmin bridge cmd_sr1216 -d 0a.16 -c ddb_read -s; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fcadmin bridge: DDB read from drive 0a.16 is successful.
##&nbsp;All values are in hexadecimal format, unless otherwise mentioned explicitly.
##&nbsp;
##&nbsp;DDBRev             : 01
##&nbsp;AdditionalLength   : 2e
##&nbsp;WCE                : 00
##&nbsp;QAM                : 00
##&nbsp;PLPB_RHA           : 00
##&nbsp;CID_Upper          : 02
##&nbsp;CID_Lower          : bc
##&nbsp;SerialNum          : 0c d8 9c
##&nbsp;ActLEDSpunDownOff  : 3c
##&nbsp;ActLEDSpunDownOn   : 08
##&nbsp;ActLEDSpunUpOff    : 0a
##&nbsp;ActLEDSpunUpOn     : 0a
##&nbsp;ActLEDSpinUpCycles : 05
##&nbsp;AFDL               : 01
##&nbsp;FLIO_FLDO          : 02
##&nbsp;PathSwitchHyst     : 14
##&nbsp;CTA                : 00
##&nbsp;ComTOV             : 0c
##&nbsp;DSU_DPU            : 00
##&nbsp;MRIE_LogErr_EExcpt : 0e
##&nbsp;IE_IntervalTimer   : 00000000
##&nbsp;dpf_validation     : cc
##&nbsp;dpf_CID            : bc
##&nbsp;dpf_type_interface : 01
##&nbsp;dpf_type_vendor    : 04
##&nbsp;dpf_features       : 07 00 00 11
##&nbsp;dpf_RBS            : 00000080
##&nbsp;dpf_WBS            : 00000080
##&nbsp;dpf_RTOV           : 03e8
##&nbsp;dpf_PWRC           : 55
##&nbsp;dpf_MO             : 00
##&nbsp;dpf_ML             : 0c
##&nbsp;CustomerVersion    : 24
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'additionallength' => '2e',
##&nbsp;            'ie_intervaltimer' => '00000000',
##&nbsp;            'dpf_type_vendor' => '04',
##&nbsp;            'customerversion' => '24',
##&nbsp;            'dpf_rtov' => '03e8',
##&nbsp;            'mrie_logerr_eexcpt' => '0e',
##&nbsp;            'dpf_ml' => '0c',
##&nbsp;            'plpb_rha' => '00',
##&nbsp;            'actledspunupon' => '0a',
##&nbsp;            'serialnum' => '0c d8 9c',
##&nbsp;            'dpf_mo' => '00',
##&nbsp;            'cid_lower' => 'bc',
##&nbsp;            'cta' => '00',
##&nbsp;            'dsu_dpu' => '00',
##&nbsp;            'actledspundownoff' => '3c',
##&nbsp;            'dpf_wbs' => '00000080',
##&nbsp;            'afdl' => '01',
##&nbsp;            'qam' => '00',
##&nbsp;            'comtov' => '0c',
##&nbsp;            'dpf_cid' => 'bc',
##&nbsp;            'ddbrev' => '01',
##&nbsp;            'actledspunupoff' => '0a',
##&nbsp;            'pathswitchhyst' => '14',
##&nbsp;            'dpf_validation' => 'cc',
##&nbsp;            'dpf_pwrc' => '55',
##&nbsp;            'wce' => '00',
##&nbsp;            'flio_fldo' => '02',
##&nbsp;            'actledspinupcycles' => '05',
##&nbsp;            'dpf_features' => '07 00 00 11',
##&nbsp;            'dpf_rbs' => '00000080',
##&nbsp;            'actledspundownon' => '08',
##&nbsp;            'cid_upper' => '02',
##&nbsp;            'dpf_type_interface' => '01'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fcadmin_bridge {
    my %args = @_;
    my $error_message;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(raw_output    => $output,
                             error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(raw_output      => $output,
                          response_object => $resp);
    my $i=0;
    my $j=0;
    foreach my $line (@lines)  {
        $line =~ s/-/_/;
        if ($line =~ /(.*[lL]evel)\s*:\s+(\w+)/ || $line =~ /(\S+)\s+:\s+(.*)/ || $line =~/([dD|sS|fF]\w+\s+\w+):\s+(\d+)/){
            $result_array[$i]->{to_key_name(item => $1)}=$2;
        } elsif ($line =~ /(\d+\w+.\d+)\s+([uU]pdate\s+\S+)()/){
              $result_array[$i]->{update_status}[$j]->{disk}=$1;
              $result_array[$i]->{update_status}[$j]->{status}=$2;
              $result_array[$i]->{update_status}[$j]->{error}=$3;
              $j++;
        } elsif( $line =~ /(\w+)=(\S+)\s+(\w+)=(\S+)/) {
              $result_array[$i]->{to_key_name(item => $1)}=$2;
              $result_array[$i]->{to_key_name(item => $3)}=$4;
              $i++;
        } elsif ($line =~/([Tt]otal\s+[Ll][Uu][Nn]s.*reported).=\s+(\d+).([Tt]otal\s+[Ll][Uu][Nn]s.*programmed).=\s+(\d+)/) {
              $result_array[$i]->{to_key_name(item => $1)}=$2;
              $result_array[$i]->{to_key_name(item =>$3)}=$4;
        } elsif($line =~ /(\d+\w+.\d+)\s+(\d+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(.\w+:\w+:\w+.)/){
              $result_array[$i]->{program_statistics}[$j]->{'devicename'}=$1;
              $result_array[$i]->{program_statistics}[$j]->{'lun'}=$2;
              $result_array[$i]->{program_statistics}[$j]->{'path'}=$3;
              $result_array[$i]->{program_statistics}[$j]->{'no_of_attempts'}=$4;
              $result_array[$i]->{program_statistics}[$j]->{'no_of_failures'}=$5;
              $result_array[$i]->{program_statistics}[$j]->{'wwn'}=$6;
              $j++;
        } elsif($line =~ /fcadmin\s+bridge:\s+DDB\s+update.*drive\s+(\d+\w+.\d+)\s+\w+\s+(\w+)/){
              $result_array[$i]->{'drive'}=$1;
              $result_array[$i]->{'status'}=$2;
        }
    }
    return \@result_array;
} ## end sub fcadmin_bridge

1;
