#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/fcmon.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode fcmon commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode fcmon  commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "fcmon show" will be fcmon_show()
##
## @synopsis
##    use fcmon;
##
##    fcmon_show(%args);
##    fcmon_show_counter_names(%args);
##    fcmon_show_counter_attrib(%args);
##    fcmon_show_config_names(%args);
##    fcmon_show_counter(%args);
##    fcmon_start(%args);
##    fcmon_stop(%args);
##    fcmon_modify_config(%args);
##    fcmon_modify_counter(%args);
##    fcmon_list_counter(%args);
##    fcmon_dump(%args);
##    fcmon_test_sdm_addition(%args);
##    fcmon_test_sdm_deletion(%args);
##    fcmon_test_check(%args);
##
## @author sreedhar@netapp.com;dl-nacl-dev@netapp.com
##
## @status To_be_reviewed

package NACL::APISet::Node::CLI::7Mode::default::fcmon;

use Data::Dumper;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /((Missing.*)|(at\sleast\sone\s.*\sargument\srequired))/ ) {
        $error = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ( $output =~
/((Unrecognized\soption.*)|(Did not recognize option.*) | (Has extra arguments.*) |(Unknown argument.*) |(Invalid argument.*))/
      )
    {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif ( $output =~ /((Could\snot\sfind.*))/ ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /^usage:/mi ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
    } elsif ( ( $command eq "fcmon start" )
        && ( $output !~ /FCMON has started/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    } elsif ( ( $command eq "fcmon stop" )
        && ( $output !~ /FCMON has stopped/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    } elsif ( ( $command =~/fcmon test sdm addition/ )
        && ( $output =~ /failed/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    } elsif ( ( $command =~/fcmon test sdm deletion/ )
        && ( $output =~/(failed)/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    } elsif ( ( $command eq "fcmon test check" )
        && ( $output !~ /FCMON: No errors found/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    } elsif ( ( $command =~/fcmon dump -file/ )
        && ( $output =~ /FCMON dump failed/ ) )
    {
        $error          = "CommandFailed";
        $$error_message = $output;
    }
    return $error;
} ## end sub _check_error

## @name ::fcmon_show
##
## @summary Parses the raw output from the "fcmon show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_show with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_show(%args);
## INPUTS TO METHOD CALL 'fcmon_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_show(%) {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "fcmon  show",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = 'is_enabled';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    my %hash = ();
    foreach my $line (@lines) {
        my @tok = split( /:|\s+/, $line );
        my $key = $tok[0];
        $key =~ s/-/_/g;
        $key = to_key_name( item => $key );
        $hash{$key} = $tok[2];
    }
    push( @result_array, \%hash );
    return \@result_array;
} ## end sub fcmon_show

## @name ::fcmon_show_counter_names
##
## @summary Parses the raw output from the "fcmon show counter-names" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_show_counter_names with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_show_counter_names(%args);
## INPUTS TO METHOD CALL 'fcmon_show_counter_names' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_show_counter_names(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "fcmon show -counter-names",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "";

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    my %hash = ();
    foreach my $line (@lines) {
        my @tok = split( /\s+-\s+/, $line );
        my $key = $tok[0];
        $key =~ s/-/_/g;
        $key = to_key_name( item => $key );
        $hash{$key} = $tok[1];
    }
    push( @result_array, \%hash );
    return \@result_array;
} ## end sub fcmon_show_counter_names

## @name ::fcmon_show_counter_attrib
##
## @summary Parses the raw output from the "fcmon show counter-attrib" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_show_counter_attrib with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_show_counter_attrib(%args);
## INPUTS TO METHOD CALL 'fcmon_show_counter_attrib' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_show_counter_attrib(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "fcmon show -counter-attrib",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "";

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    my %hash = ();
    foreach my $line (@lines) {
        if ( $line =~ /([\w-]+)\s+-\s+([\S\s]+)/ ) {
            my $key = $1;
            my $val = $2;
            $key =~ s/-/_/g;
            $key = to_key_name( item => $key );
            $hash{$key} = $val;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
} ## end sub fcmon_show_counter_attrib

## @name ::fcmon_show_config_names
##
## @summary Parses the raw output from the "fcmon show config-names" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_show_config_names with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_show_config_names(%args);
## INPUTS TO METHOD CALL 'fcmon_show_config_names' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_show_config_names(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "fcmon show -config-names",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "";

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    my %hash = ();
    foreach my $line (@lines) {
        my @tok = split( /\s+-\s+/, $line );
        my $key = $tok[0];
        $key =~ s/-/_/g;
        $key = to_key_name( item => $key );
        $hash{$key} = $tok[1];
    }
    push( @result_array, \%hash );
    return \@result_array;
} ## end fcmon_show_config_names

## @name ::fcmon_show_counter
##
## @summary Parses the raw output from the "fcmon show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_show_counter with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_show_counter(%args);
## INPUTS TO METHOD CALL 'fcmon_show_counter' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_show_counter(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon show -counter",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "counter name";

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    foreach my $line (@lines) {
        if ( $line =~
/([\w-]+)\s+[|](0x\w+)[|]\s+(0x\w+)[|]\s+(\w+)\s+[|]\s+(0x\w+)[|]\s+(\w+)\s+[|]\s+(\w+)\s+[|]\s+(\d+)\s+[|]/
          )
        {
            my %hash = ();
            $hash{'counter_name'}           = $1;
            $hash{'overflow_range'}         = $2;
            $hash{'burst_error_level_cic'}  = $3;
            $hash{'burst_action'}           = $4;
            $hash{'threshold_error_level'}  = $5;
            $hash{'threshold_error_action'} = $6;
            $hash{'counter_type'}           = $7;
            $hash{'counter_sigbits'}        = $8;
            push( @result_array, \%hash );
        }
    }
    return \@result_array;
} ## end sub fcmon_show_counter

## @name ::fcmon_start
##
## @summary Parses the raw output from the "fcmon start" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_start with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_start(%args);
## INPUTS TO METHOD CALL 'fcmon_start' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_start(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon start",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end fcmon_start

## @name ::fcmon_stop
##
## @summary Parses the raw output from the "fcmon stop" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_stop with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_stop(%args);
## INPUTS TO METHOD CALL 'fcmon_stop' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_stop(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon stop",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end fcmon_stop

## @name ::fcmon_modify_config
##
## @summary Parses the raw output from the "fcmon modify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_modify_config with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_modify_config(%args);
## INPUTS TO METHOD CALL 'fcmon_modify_config' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'config-val' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_modify_config(%) {
    my %args = @_;
    my %hash = ();
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon modify -config",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end fcmon_modify_config

## @name ::fcmon_modify_counter
##
## @summary Parses the raw output from the "fcmon modify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_modify_counter with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_modify_counter(%args);
## INPUTS TO METHOD CALL 'fcmon_modify_counter' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-attrib' </td> <td> 'counter-attrib' </td> </tr> 
## <tr> <td> '-value' </td> <td> 'counter-attrib-val' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_modify_counter(%) {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon modify -counter",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end fcmon_modify_counter

## @name ::fcmon_list_counter
##
## @summary Parses the raw output from the "fcmon list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_list_counter with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_list_counter(%args);
## INPUTS TO METHOD CALL 'fcmon_list_counter' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_list_counter(%) {
    my %args = @_;
    my $key;
    my ( $cnt,           $counter );
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon list -counter",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = /\s+(\w+[-\w+]+):\s+/;

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    # to track  counter attribute values
    $cnt = 0;
    my %hash;
    foreach my $line (@lines) {
        if ( ( $line =~ /(\w+[-\w]+):/ ) && ( $cnt == 0 ) ) {
            $key = $1;
            $key =~ s/-/_/g;
            $key     = to_key_name( item => $key );
            $counter = $key;
            $cnt     = 1;
            $hash{counter_name}=$counter;
            next;
        } else {
            my @tok = split( /\s+:/, $line );
            $key = $tok[0];
            $key =~ s/-/_/g;
            $key = to_key_name( item => $key );
            my @tok1 = split( /\s+/, $tok[1] );
            $tok1[1] =~ s/\s+//g;
            $hash{$key} = $tok1[1];
            $cnt++;

            if ( $cnt == 8 ) {
                $cnt = 0;
                my %hash1 = %hash;
                push( @result_array, \%hash1 );
            }
            next;
        }
    }
    return ( \@result_array );
} ## end sub fcmon_list_counter

## @name ::fcmon_dump
##
## @summary Parses the raw output from the "fcmon dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_dump with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_dump(%args);
## INPUTS TO METHOD CALL 'fcmon_dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_dump(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon dump -file",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub fcmon_dump

## @name ::fcmon_test_adm_addition
##
## @summary add SDM device to FCMON
##
## @description
##    This method can be used adding a new deivice to FCMON
##    called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/default/fcmon.cdefs
##
## @synopsis
## fcmon_test_sdm_addition(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
##
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##

## @name ::fcmon_test_sdm_addition
##
## @summary Parses the raw output from the "fcmon test sdm addition" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_test_sdm_addition with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_test_sdm_addition(%args);
## INPUTS TO METHOD CALL 'fcmon_test_sdm_addition' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_test_sdm_addition(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon test sdm addition",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];

} ## end sub fcmon_test_sdm_addition

## @name ::fcmon_test_adm_deletion
##
## @summary delete  SDM device from  FCMON
##
## @description
##    This method can be used deleting  a  deivice to FCMON
##    called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/default/fcmon.cdefs
##
## @synopsis
## fcmon_test_sdm_deletion(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
##
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##

## @name ::fcmon_test_sdm_deletion
##
## @summary Parses the raw output from the "fcmon test sdm deletion" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_test_sdm_deletion with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_test_sdm_deletion(%args);
## INPUTS TO METHOD CALL 'fcmon_test_sdm_deletion' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_test_sdm_deletion(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon test sdm deletion",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub fcmon_test_sdm_deletion

## @name ::fcmon_test_check
##
## @summary Parses the raw output from the "fcmon test check" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcmon_test_check with any of its options as described in the 
##    corresponding cdef file 'fcmon.cdefs'
## @synopsis
##      fcmon_test_check(%args);
## INPUTS TO METHOD CALL 'fcmon_test_check' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-config' </td> <td> 'config' </td> </tr> 
## <tr> <td> '-counter' </td> <td> 'counter' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub fcmon_test_check(%) {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "fcmon test check",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub fcmon_test_check
1;
