#$Id$
#
#Copyright (c) 2011-2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse fcp command
##
## @description
##    This is a library to parse the raw output of the fcp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "fcp show adapter" will be fcp_show_adapter
##
## @synopsis
##    use fcp;
##
##    fcp_show_adapter(%args);
##    fcp_show_adapters(%args);
##    fcp_portname_show(%args);
##    fcp_wwpn_alias_show(%args);
##    fcp_status(%args);
##    fcp_portname_set(%args);
##    fcp_show_cfmode(%args);
##    fcp_show_initiator(%args);
##    fcp_stats(%args);
##    fcp_wwpn_alias_set(%args);
##    fcp_wwpn_alias_remove(%args);
##    fcp_start(%args);
##    fcp_stop(%args);
##    fcp_config(%args);
##
## @author  dl-nacl-dev@netapp.com,rroshan@netapp.com
##
## @status public
#
################################################################################

package NACL::APISet::Node::CLI::7Mode::default::fcp;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);
use NACL::APISet::Exceptions::MissingValueException;
use Data::Dumper;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use TCD;
## @name ::fcp_show_adapter
##
## @summary Parses the raw output from the "fcp show adapter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_show_adapter with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_show_adapter(%args);
## INPUTS TO METHOD CALL 'fcp_show_adapter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> </tr>
## <tr> <td> '-v' </td> <td> 'valid-info' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcp show adapter -v 0d
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'fabric_established' => 'no',
##&nbsp;             'pci_clock_speed' => '33_mhz',
##&nbsp;             'status' => 'offlined_by_user/system',
##&nbsp;             'external_gbic' => 'no',
##&nbsp;             'host_port_address' => '0x000000',
##&nbsp;             'slot' => '0d',
##&nbsp;             'firmware_rev' => '5.3.0',
##&nbsp;             'fc_nodename' => '50:0a:09:80:89:5b:4d:c1 (500a0980895b4dc1)',
##&nbsp;             'sram_parity' => 'yes',
##&nbsp;             'fc_packet_size' => '2048',
##&nbsp;             'standby' => 'no',
##&nbsp;             'data_link_rate' => '4_gbit',
##&nbsp;             'fc_portname' => '50:0a:09:82:89:5b:4d:c1 (500a0982895b4dc1)',
##&nbsp;             'partner_adapter' => 'none',
##&nbsp;             'description' => 'fibre_channel_target_adapter_0d_dual-channel,_qlogic_2432_2462_rev._2',
##&nbsp;             'switch_port' => 'unknown',
##&nbsp;             'cacheline_size' => '16',
##&nbsp;             'mediatype' => 'auto',
##&nbsp;             'connection_established' => 'ptp',
##&nbsp;             'target_port_id' => '0x2',
##&nbsp;             'adapter_type' => 'local',
##&nbsp;             'pci_bus_width' => '64-bit'
##&nbsp;           }
##&nbsp;         ];
sub fcp_show_adapter {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my @lines = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my ($key1,$key2,@Res,%Output,%Hash_Flag_Out,@Arr_2,$Count,$Flag,$value);
    my $i = -1;
    my $j = 0;
    foreach $value(@lines){
        if($value =~/^(Slot):\s+(\d\w)$/){
            $Flag = 1;
            $i++;
            $Res[$i]->{to_key_name(item=>$1)} = $2;
            next;
        }
        if ($Flag == 1) {
            if ($value =~/^(FC Portname|FC Nodename):\s+([0-9a-z\:]+\s+[0-9a-z\(\)]+)$/){
                $Res[$i]->{to_key_name(item=>$1)} = $2;
                next;
            }
            elsif($value =~/^(\w+.*):\s+([0-9a-z\:\(\)\-A-Z\s+\,\.\/]+)$/){
                $Res[$i]->{to_key_name(item=>$1)} = to_key_name(item=>$2);
            }
        }
    }
    return \@Res;
}
## @name ::fcp_show_adapters
##
## @summary Parses the raw output from the "fcp show adapters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_show_adapters with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_show_adapters(%args);
## INPUTS TO METHOD CALL 'fcp_show_adapters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> </tr>
## <tr> <td> '-v' </td> <td> 'valid-info' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcp show adapters -v 0d
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'fabric_established' => 'no',
##&nbsp;             'pci_clock_speed' => '33_mhz',
##&nbsp;             'status' => 'offlined_by_user/system',
##&nbsp;             'external_gbic' => 'no',
##&nbsp;             'host_port_address' => '0x000000',
##&nbsp;             'slot' => '0d',
##&nbsp;             'firmware_rev' => '5.3.0',
##&nbsp;             'fc_nodename' => '50:0a:09:80:89:5b:4d:c1 (500a0980895b4dc1)',
##&nbsp;             'sram_parity' => 'yes',
##&nbsp;             'fc_packet_size' => '2048',
##&nbsp;             'standby' => 'no',
##&nbsp;             'data_link_rate' => '4_gbit',
##&nbsp;             'fc_portname' => '50:0a:09:82:89:5b:4d:c1 (500a0982895b4dc1)',
##&nbsp;             'partner_adapter' => 'none',
##&nbsp;             'description' => 'fibre_channel_target_adapter_0d_dual-channel,_qlogic_2432_2462_rev._2',
##&nbsp;             'switch_port' => 'unknown',
##&nbsp;             'cacheline_size' => '16',
##&nbsp;             'mediatype' => 'auto',
##&nbsp;             'connection_established' => 'ptp',
##&nbsp;             'target_port_id' => '0x2',
##&nbsp;             'adapter_type' => 'local',
##&nbsp;             'pci_bus_width' => '64-bit'
##&nbsp;           }
##&nbsp;         ];

sub fcp_show_adapters {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my @lines = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my ($key1,$key2,@Res,%Output,%Hash_Flag_Out,@Arr_2,$Count,$Flag,$value);
    my $i = -1;
    my $j = 0;
    foreach $value(@lines){
        if($value =~/^(Slot):\s+(\d\w)$/){
            $Flag = 1;
            $i++;
            $Res[$i]->{to_key_name(item=>$1)} = $2;
            next;
        }
        if ($Flag == 1) {
            if ($value =~/^(FC Portname|FC Nodename):\s+([0-9a-z\:]+\s+[0-9a-z\(\)]+)$/){
                $Res[$i]->{to_key_name(item=>$1)} = $2;
                next;
            }
            elsif($value =~/^(\w+.*):\s+([0-9a-z\:\(\)\-A-Z\s+\,\.\/]+)$/){
                $Res[$i]->{to_key_name(item=>$1)} = to_key_name(item=>$2);
            }
        }
    }
    return \@Res;
}
## @name ::fcp_portname_show
##
## @summary Parses the raw output from the "fcp portname show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_portname_show with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_portname_show(%args);
## INPUTS TO METHOD CALL 'fcp_portname_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-v' </td> <td> 'valid-info' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcp portname show f
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'Portname' => '50:0a:09:82:89:5b:4d:c1',
##&nbsp;             'Adatpter' => '0d'
##&nbsp;           },
##&nbsp;          {
##&nbsp;             'Portname' => '50:0a:09:85:89:5b:4d:c1',
##&nbsp;             'Adatpter' => '0c'
##&nbsp;           }
##&nbsp;         ];
sub fcp_portname_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my @lines = get_lines("raw_output"=>$output,"response_object"=>$resp);
    my (@Res,$value);
    my $i = 0;
    foreach $value(@lines){
        if($value =~/^([0-9a-z\:]+)\s+([\w+|\d+]\w+)$/){
            $Res[$i]->{"portname"} = $1;
            $Res[$i]->{"adapter"} = $2;
            $i++;
        }
    }
    return \@Res;
}
## @name ::fcp_status
##
## @summary Parses the raw output from the "fcp status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_status with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_status(%args);
## INPUTS TO METHOD CALL 'fcp_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcp status
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'status' => 'Running'
##&nbsp;           }
##&nbsp;         ];

sub fcp_status {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my (@Res,$i);
	$i = 0;
    if ($output =~/FCP service is (running|not running)./) {
        $Res[$i]->{"status"} = $1;
    }
    return \@Res;
}
## @name ::fcp_wwpn_alias_show
##
## @summary Parses the raw output from the "fcp wwpn-alias show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_wwpn_alias_show with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_wwpn_alias_show(%args);
## INPUTS TO METHOD CALL 'fcp_wwpn_alias_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-w' </td> <td> 'wwpn' </td> </tr>
## <tr> <td> '-a' </td> <td> 'alias' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fcp wwpn-alias show
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'wwpn' => '50:0a:09:80:89:5b:4d:c1',
##&nbsp;             'alias' => 'test'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'wwpn' => '50:0a:09:80:89:5b:4d:c1',
##&nbsp;             'alias' => 'test1'
##&nbsp;           }
##&nbsp;         ];

sub fcp_wwpn_alias_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my @lines = get_lines("raw_output"=>$output,"response_object"=>$resp);
    my (@Res,$value,@head_arr);
    my $i = 0;
    foreach $value (@lines){
        if($value =~/^WWPN\s+Alias$/) {
            @head_arr = split(/\s+/,$value);
            next;
        } elsif($value =~/^---- .*/){
            next;
        } elsif($value =~/^([0-9a-z\:]+)\s+([\w+|\d+]\w+)$/){
            $Res[$i]->{to_key_name(item => $head_arr[0])} = $1;
            $Res[$i]->{to_key_name(item => $head_arr[1])} = $2;
            $i++;
        }
    }
    return \@Res;
}

## @name ::fcp_portname_set
##
## @summary Parses the raw output from the "fcp portname set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_portname_set with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_portname_set(%args);
## INPUTS TO METHOD CALL 'fcp_portname_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'wwpn' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_portname_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_portname_set

## @name ::fcp_show_cfmode
##
## @summary Parses the raw output from the "fcp show cfmode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_show_cfmode with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_show_cfmode(%args);
## INPUTS TO METHOD CALL 'fcp_show_cfmode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_show_cfmode {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_show_cfmode

## @name ::fcp_show_initiator
##
## @summary Parses the raw output from the "fcp show initiator" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_show_initiator with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_show_initiator(%args);
## INPUTS TO METHOD CALL 'fcp_show_initiator' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_show_initiator {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_show_initiator

## @name ::fcp_stats
##
## @summary Parses the raw output from the "fcp stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_stats with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_stats(%args);
## INPUTS TO METHOD CALL 'fcp_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-a' </td> <td> 'list-of-all-hba-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zeros-all-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_stats {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_stats

## @name ::fcp_wwpn_alias_set
##
## @summary Parses the raw output from the "fcp wwpn-alias set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_wwpn_alias_set with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_wwpn_alias_set(%args);
## INPUTS TO METHOD CALL 'fcp_wwpn_alias_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-wwpn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-alias' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_wwpn_alias_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_wwpn_alias_set

## @name ::fcp_wwpn_alias_remove
##
## @summary Parses the raw output from the "fcp wwpn-alias remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_wwpn_alias_remove with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_wwpn_alias_remove(%args);
## INPUTS TO METHOD CALL 'fcp_wwpn_alias_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-w' </td> <td> 'wwpn-alias' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'alias' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_wwpn_alias_remove {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_wwpn_alias_remove

## @name ::fcp_start
##
## @summary Parses the raw output from the "fcp start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_start with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_start(%args);
## INPUTS TO METHOD CALL 'fcp_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_start {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_start

## @name ::fcp_stop
##
## @summary Parses the raw output from the "fcp stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_stop with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_stop(%args);
## INPUTS TO METHOD CALL 'fcp_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub fcp_stop {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}## end sub fcp_stop

## @name ::fcp_config
##
## @summary Parses the raw output from the "fcp config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_config with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_config(%args);
## INPUTS TO METHOD CALL 'fcp_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'media-type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'speed' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

## @name ::fcp_config
##
## @summary Parses the raw output from the "fcp config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fcp_config with any of its options as described in the
##    corresponding cdef file 'fcp.cdefs'
## @synopsis
##      fcp_config(%args);
## INPUTS TO METHOD CALL 'fcp_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'media-type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'speed' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'status' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =  fcp config;
##&nbsp;-------- Raw Output ---------
##&nbsp;3a:   LINK NOT CONNECTED <ADAPTER UP>
##&nbsp;        host address 000000
##&nbsp;        portname 50:0a:09:81:99:fb:61:a3  nodename 50:0a:09:80:89:fb:61:a3
##&nbsp;        mediatype auto speed auto
##&nbsp;
##&nbsp;3b:   LINK NOT CONNECTED <ADAPTER UP>
##&nbsp;        host address 000000
##&nbsp;        portname 50:0a:09:82:99:fb:61:a3  nodename 50:0a:09:80:89:fb:61:a3
##&nbsp;        mediatype auto speed auto
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'host_address' => '000000',
##&nbsp;            'mediatype' => [
##&nbsp;                             'auto',
##&nbsp;                             'speed',
##&nbsp;                             'auto'
##&nbsp;                           ],
##&nbsp;            'nodename' => ' 50:0a:09:80:89:fb:61:a3',
##&nbsp;            'adapter' => '3a',
##&nbsp;            'portname' => '50:0a:09:81:99:fb:61:a3',
##&nbsp;            'link_status' => 'LINK NOT CONNECTED',
##&nbsp;            'adapter_state' => 'UP'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'host_address' => '000000',
##&nbsp;            'mediatype' => [
##&nbsp;                             'auto',
##&nbsp;                             'speed',
##&nbsp;                             'auto'
##&nbsp;                           ],
##&nbsp;            'nodename' => ' 50:0a:09:80:89:fb:61:a3',
##&nbsp;            'adapter' => '3b',
##&nbsp;            'portname' => '50:0a:09:82:99:fb:61:a3',
##&nbsp;            'link_status' => 'LINK NOT CONNECTED',
##&nbsp;            'adapter_state' => 'UP'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub fcp_config {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = -1;
    my @result_array;
    foreach my $line (@lines) {
        if($line =~ /(\S+)\:\s+(.*)\s+\<ADAPTER\s+(.*)\>/) {
            $count++;
            $result_array[$count]->{adapter} = $1;
            $result_array[$count]->{link_status} = $2;
            $result_array[$count]->{adapter_state} = $3;
        } elsif($line =~ /(host\s+address)\s+(.*)/) {
            $result_array[$count]->{to_key_name( item =>$1)} = $2;
        } elsif($line =~ /(portname)\s+(.*\S+)\s+(nodename)(.*)/) {
            $result_array[$count]->{$1} = $2;
            $result_array[$count]->{$3} = $4;
        } elsif($line =~ /mediatype\s+(.*)/) {
            my @values = split(/\s+/,$1);
            $result_array[$count]->{mediatype} = \@values;
        }
    }
    return \@result_array;
}## end sub fcp_config

sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif($output =~ /^(fcp show adapter:(.*)not found.)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp portname show: Illegal.*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp wwpn-alias: Invalid argumen .*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp wwpn-alias: Too few arguments.)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp wwpn-alias: Illegal option .*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp wwpn-alias: Alias .* not found.)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp wwpn-alias: Invalid portname .*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(Alias \".*\" not found.)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /^(fcp: No FCP Target Adapters.*|FCP service is not running.*)/si) {
        $error = "CommandFailed";
        $$error_message = $output;
    } elsif($output =~ /^(fcp: FCP is not licensed.*)/si){
        $error = "FeatureNotLicensed";
        $$error_message = $output;
    } elsif($output =~ /No\s+mediatype\s+given|No\s+speed\s+given/i ) {
        $error = "MissingValue";
        $$error_message = $output;
    } elsif($output =~ /Fibre\s+Channel\s+Target\s+Adapter.*not\s+found|Unknown\s+mediatype\s+given|Unrecognized\s+speed/i) {
        $error = "InvalidParamValue";
        $$error_message = $output;
    } elsif($output =~ /The\s+FCP\s+port\s+is\s+already.*|Requested\s+speed\s+is\s+not\s+supported\s+on\s+this\s+target\s+adapter/i) {
        $error = "CommandFailed";
        $$error_message = $output;
    }
    return $error;
}

1;
