# $Id$
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode fiji commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode fiji commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "fiji filter" will be fiji_filter()
##
## @synopsis
##    use fiji;
##
##    fiji_filter(%args);
##    fiji_clear(%args);
##    fiji_show(%args);
##    fiji_nvfilter(%args);
##    fiji_nvclear(%args);
##    fiji_test_is_true(%args);
##    fiji_dumpfilter(%args);
##    fiji_test(%args);
##    fiji_dumpnvram(%args);
##    fiji_dump(%args);
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::fiji;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::UnexpectedOutputException;
use NACL::APISet::Exceptions::NoMatchingEntriesException;

## @name ::fiji_filter
##
## @summary Parses the raw output from the "fiji filter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_filter with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_filter(%args);
## INPUTS TO METHOD CALL 'fiji_filter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filter' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji filter "(fiji.test (eq @str dog) (echo @str))"
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '3',
##&nbsp;            'filter_status' => 'set'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fiji filter "(fiji.test (eq @str dog) (echo @str))"
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '0',
##&nbsp;            'filter_status' => 'set'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_filter {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my @fiji;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji filter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Below regex is matched with a sample string mentioned
    # fiji: filter set (slot 4)

    if ( $output =~ /\w+\:\s+\w+\s+(\w+)\s+\(\w+\s+(\d+)\)/ ) {
        $info{"status"}      = $1;
        $info{"slot_number"} = $2;
        push( @fiji, \%info );
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "unexpected format of output",
            response_object => $resp
        );
    } ## end else [ if ( $output =~ /\w+\:\s+\w+\s+(\w+)\s+\(\w+\s+(\d+)\)/)
    return \@fiji;
} ## end sub fiji_filter

## @name ::fiji_clear
##
## @summary Parses the raw output from the "fiji clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_clear with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_clear(%args);
## INPUTS TO METHOD CALL 'fiji_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'slot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji clear all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '1',
##&nbsp;            'status' => 'cleared'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '2',
##&nbsp;            'status' => 'cleared'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '3',
##&nbsp;            'status' => 'cleared'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '4',
##&nbsp;            'status' => 'cleared',
##&nbsp;            'nvram' => 'true'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_clear {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji filter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Extracting the command option.
    $command =~ /fiji\s+clear\s+(\w+)/;
    my $option = $1;
    my @fiji;
    my $value;

    if ( $option =~ /all/ ) {
        my @arr = get_lines( "raw_output" => $output,
            "response_object" => $resp );
        foreach $value (@arr) {
            my %info = ();
            if ( $value
                =~ /\w+\:\s+\w+\s+(\w+)\s+(\((\w+)?\)\s+)?\(\w+\s+(\d+)\)/ )
            {
                $info{"status"}      = $1;
                $info{"slot_number"} = $4;
                if ($3) {
                    $info{"nvram"} = "true";
                }
            } ## end if ( $value =~ ...
            push( @fiji, \%info );
        } ## end foreach $value (@arr)
    } elsif ( $option =~ /\d+/ ) {
        my %info = ();
        if ( $output
            =~ /\w+\:\s+\w+\s+(\w+)\s+(\((\w+)?\)\s+)?\(\w+\s+(\d+)\)/ )
        {
            $info{"filter_status"} = $1;
            $info{"slot_number"}   = $4;
            if ($3) {
                $info{"nvram"} = "true";
            }
        } ## end if ( $output =~ ...
        push( @fiji, \%info );
    } elsif ( $option eq "" ) {
        my %info = ();
        if ( $value =~ /\w+\:\s+\w+\s+(\w+)\s+\(\w+\)/ ) {
            $info{"filter_status"} = $1;
            $info{"slot_number"}   = $2;
            if ($3) {
                $info{"nvram"} = "true";
            }
        } ## end if ( $value =~ /\w+\:\s+\w+\s+(\w+)\s+\(\w+\)/)
        push( @fiji, \%info );
    } ## end elsif ( $option eq "" )
    return \@fiji;
} ## end sub fiji_clear

## @name ::fiji_show
##
## @summary Parses the raw output from the "fiji show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_show with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_show(%args);
## INPUTS TO METHOD CALL 'fiji_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji show
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filter' => '"(fiji.test (eq @str dog) (echo @str))"',
##&nbsp;            'slot' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filter' => '"(fiji.test (eq @str dog) (echo @str))"',
##&nbsp;            'slot' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filter' => '"(fiji.test (eq @str dog) (echo @str))\\"',
##&nbsp;            'slot' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'filter' => '"(fiji.test (eq @str xxx) (echo @str))"',
##&nbsp;            'nvram' => 'true',
##&nbsp;            'slot' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_show {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji filter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @fiji;

    # Below regex is matched with a sample string mentioned
    # 1: "(fiji.test (eq @str dog) (echo @str))"
    # fiji: no filter set

    my @arr
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my $value;
    foreach $value (@arr) {
        my %info = ();
        if ( $value =~ /(\w+)(\s+\((\w+)?\))?:\s+(.*)/ ) {

            $info{"filter"} = $4;
            $info{"slot"}   = $1;

            if ($3) {
                $info{"nvram"} = "true";
            }
        } ## end if ( $value =~ /(\w+)(\s+\((\w+)?\))?:\s+(.*)/)

        push( @fiji, \%info );
    } ## end foreach $value (@arr)
    return \@fiji;
} ## end sub fiji_show

## @name ::fiji_nvfilter
##
## @summary Parses the raw output from the "fiji nvfilter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_nvfilter with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_nvfilter(%args);
## INPUTS TO METHOD CALL 'fiji_nvfilter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filter' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji nvfilter "(fiji.test (eq @str xxx) (echo @str))"
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '4',
##&nbsp;            'nvram' => 'true',
##&nbsp;            'filter_status' => 'set'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_nvfilter {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji nvfilter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @fiji;

    # Below regex is matched with a sample string mentioned
    # fiji: filter set (nvram) (slot 1)

    if ( $output =~ /\w+\:\s+\w+\s+(\w+)\s+\((\w+)\)\s+\(\w+\s+(\d+)\)/ ) {
        $info{"filter_status"} = $1;
        $info{"slot_number"}   = $3;
        if ($3) {
            $info{"nvram"} = "true";
        }
    } ## end if ( $output =~ ...
    push( @fiji, \%info );
    return \@fiji;
} ## end sub fiji_nvfilter

## @name ::fiji_nvclear
##
## @summary Parses the raw output from the "fiji nvclear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_nvclear with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_nvclear(%args);
## INPUTS TO METHOD CALL 'fiji_nvclear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji nvclear
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'cleared',
##&nbsp;            'nvram' => 'true'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fiji nvclear  all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'cleared',
##&nbsp;            'nvram' => 'true'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_nvclear {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji filter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @fiji;

    # Below regex is matched with a sample string mentioned
    # fiji: filter cleared (nvram only)

    if ( $output =~ /\w+\:\s+\w+\s+(\w+)\s+\((\w+)\s+\w+\)/ ) {
        $info{"status"} = $1;
        if ($2) {
            $info{"nvram"} = "true";
        }
        push( @fiji, \%info );
    } ## end if ( $output =~ /\w+\:\s+\w+\s+(\w+)\s+\((\w+)\s+\w+\)/)
    return \@fiji;
} ## end sub fiji_nvclear

## @name ::fiji_test_is_true
##
## @summary Parses the raw output from the "fiji test_is_true" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_test_is_true with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_test_is_true(%args);
## INPUTS TO METHOD CALL 'fiji_test_is_true' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; fiji test_is_true; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'value' => 0
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_test_is_true {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my %result;
    my @result_array;
    my $found = 0;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "fiji",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $cmd = $resp->get_actual_command();

    if ( $raw_output =~ /FIJI_test_is_true: True/ ) {
        $result{'value'} = 1;
    } elsif ( $raw_output =~ /FIJI_test_is_true: False/ ) {
        $result{'value'} = 0;
    } else {
        $error         = "UnexpectedOutput";
        $error_message = "fiji test_is_true, unrecognized output";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @result_array = ( \%result );
    return \@result_array;
} ## end sub fiji_test_is_true

## @name ::fiji_dumpfilter
##
## @summary Parses the raw output from the "fiji dumpfilter" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_dumpfilter with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_dumpfilter(%args);
## INPUTS TO METHOD CALL 'fiji_dumpfilter' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = fiji dumpfilter
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'value' => '(eval (fiji.test (true )(return "1" )))',
##&nbsp;            'slot_number' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_dumpfilter {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji dumpfilter",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my $value;
    my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my $i = 0;
    my %info;
    foreach $value (@arr) {
        if($value =~ /(\d+.*):/) {
            $result_array[$i]->{"slot_number"} = $1;
        } elsif($value =~ /\(\w+.*/) {
            $result_array[$i]->{"value"}      = $&;
            $i++;
        } elsif($value =~ /\w+:\s+(.*)/) {
            my %info = ();
            $info{"status"} = $1;
            push( @result_array,\%info );
        }
    }
    return \@result_array;

}

## @name ::fiji_test
##
## @summary Parses the raw output from the "fiji test" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_test with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_test(%args);
## INPUTS TO METHOD CALL 'fiji_test' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'test-point' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji test tests
##&nbsp;$VAR1 = [];
##&nbsp;
sub fiji_test {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "fiji test",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array;

    # Below regex is matched with a sample string mentioned
    # FIJI: [fiji.test]: dog

    my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my $value;
        foreach $value (@arr)  {
            my %info = ();
            if($value =~ /\w+:\s(.\w+.\w+.):\s(\w+)/) {
                $info{$1}     = $2;
            }
            push( @result_array, \%info );
        }
    return \@result_array;
}

## @name ::fiji_dumpnvram
##
## @summary Parses the raw output from the "fiji dumpnvram" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fiji_dumpnvram with any of its options as described in the
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_dumpnvram(%args);
## INPUTS TO METHOD CALL 'fiji_dumpnvram' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fiji dumpnvram
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'fiji_nvramHeader_version' => '0x1',
##&nbsp;             'hp' => '0xa35c1800',
##&nbsp;             'lnv_toc_magic' => '0xaeacdaed',
##&nbsp;             'fiji_nvramHeader_eid' => '0x0',
##&nbsp;             'nv_start' => '0xa3026000',
##&nbsp;             'lnv_toc_blocks[NV_TESTPOINT_BLOCK].start' => '0x59b000',
##&nbsp;             'lnv_toc_blocks[NV_DEBUG_BLOCK].start' => '0x599000',
##&nbsp;             'lnv_toc_version' => '0x17',
##&nbsp;             'fiji_nvram' => '0xa35c1800',
##&nbsp;             'lnv_toc_blocks[NV_DEBUG_BLOCK].len' => '0x2000',
##&nbsp;             'lnv_toc_blocks[NV_TESTPOINT_BLOCK].len' => '0x1000'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub fiji_dumpnvram {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $val ;
    my @arr = get_lines(raw_output=>$output,response_object=>$resp);
    my @fiji_dumpnvram;
    foreach $val (@arr) {
        if($val =~ /(\S+)=(\S+),\s+(\S+)=(\S+)?/) {
            my $key1 = $1;
            my $key2 = $3;
            my $value1 = $2;
            my $value2 = $4;
            $key1 =~ s/->/_/g;
            $key2 =~ s/->/_/g;
            $fiji_dumpnvram[0]->{"$key1"} = $value1;
            $fiji_dumpnvram[0]->{"$key2"} = $value2;
        } elsif($val =~ /(\S+)=(\S+)/) {
             $fiji_dumpnvram[0]->{$1} = $2;
        }
    }

    return \@fiji_dumpnvram;
 }

## @name ::fiji_dump
##
## @summary Parses the raw output from the "fiji dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fiji_dump with any of its options as described in the 
##    corresponding cdef file 'fiji.cdefs'
## @synopsis
##      fiji_dump(%args);
## INPUTS TO METHOD CALL 'fiji_dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; fiji dump; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;0:
##&nbsp;    [0] OPERAND, id 1, operand eval, nsubs 1
##&nbsp;        [0] ITERATOR, id 2, name repeat, maxCount 50, count 50
##&nbsp;            [0] EVENT_TYPE, id 4, event raid.disk.write.done
##&nbsp;                [0] OPERAND, id 5, operand true, nsubs 0
##&nbsp;                [1] ACTION, id 6, name echo, nsubs 3
##&nbsp;                    [0] VARIABLE, id 7, value $<P|v˪0}v}Z%}Z%
##&nbsp;                    [1] STRING, id 8, value " "
##&nbsp;                    [2] REFERENCE, id 9, value @disk
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nsubs' => '1',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'operand' => 'eval',
##&nbsp;            'id' => '1',
##&nbsp;            'node' => '[0] OPERAND'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'count' => '50',
##&nbsp;            'maxcount' => '50',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'name' => 'repeat',
##&nbsp;            'id' => '2',
##&nbsp;            'node' => '[0] ITERATOR'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '0',
##&nbsp;            'status' => 'fiji hit',
##&nbsp;            'id' => '4',
##&nbsp;            'event' => 'raid.disk.write.done',
##&nbsp;            'node' => '[0] EVENT_TYPE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'nsubs' => '0',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'operand' => 'true',
##&nbsp;            'id' => '5',
##&nbsp;            'node' => '[0] OPERAND'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'nsubs' => '3',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'name' => 'echo',
##&nbsp;            'id' => '6',
##&nbsp;            'node' => '[1] ACTION'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'value' => '$<P|v˪0}v}Z%}Z%',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'id' => '7',
##&nbsp;            'node' => '[0] VARIABLE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'value' => '" "',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'id' => '8',
##&nbsp;            'node' => '[1] STRING'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'value' => '@disk',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'id' => '9',
##&nbsp;            'node' => '[2] REFERENCE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fiji_dump {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output = $resp->get_processed_output();
    my @lines = get_lines(response_object => $resp, raw_output => $output);
    my (@result_array, $slot, $status, @temp);
	$status = undef ;
    my $count = 0;
    for(my $i = 0; $i <= $#lines; $i++) {
        if($lines[$i] =~ /^(\d+.*):$/) {
           $slot = $1;

        }

	## fiji status value can be determined by following 2 ways
	## 1. repeat fiji - check for the count value of ITERATOR
        ## if (count > 0) fiji hit else in progress
	## 2. without repeat fiji - check the ---> in EVENT_TYPE
	## if --> then in progress otherwise fiji hit

	if($lines[$i] =~ /EVENT_TYPE/ && $lines[$i] =~ /-+>\s+/) {
            $lines[$i] =~ s/-+>\s+//g;
            $status = "in progress" if(!defined $status);
        }

        @temp = split(/[,]\s+/, $lines[$i]);
        my @values;
        if($#temp > 1) {
            $result_array[$count]->{slot_number} = $slot;
            foreach my $var (@temp) {
                @values = split(/\s+/, $var);
                if($var =~ /value\s+(.*)/) {
                    $values[0] = "value";
                    $values[1] = $1;
                }
                if($values[1] =~ /^[A-Z]+.*[A-Z]$/ && $values[0] =~ /\[\d+\]/) {
                    $values[0] = "node";
                    $values[1] = $var;
                }
                chomp($values[1]);
                $result_array[$count]->{to_key_name( item => $values[0])} = $values[1];
            }
           if($lines[$i] =~ /ITERATOR/) {
	       $status = ($result_array[$count]->{'count'}) ? "fiji hit" : "in progress" ;
	   }
           if(defined $status && $lines[$i] =~ /EVENT_TYPE/){
	       $result_array[$count]->{status} = $status;
	   }elsif(!defined $status && $lines[$i] =~ /EVENT_TYPE/){
               $result_array[$count]->{status} = "fiji hit";
	   }
           $count++;
        }
    }
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /unable\s+to\s+create\s+filter/i ) {
        if ( $raw_output =~ /Missing\s+conditional/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Missing condition in the filter";
        } elsif ( $raw_output =~ /Should\s+be\s+an\s+action/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Missing Action in the filter";
        } elsif ( $raw_output =~ /Missing\s+right\s+paren/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Missing Fault Pointer in the filter";
        } elsif ( $raw_output =~ /Illegal\s+filter\s+operand\s+type/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Illegal parameter is passed";
        } else {
            $error          = "UnexpectedOutput";
            $$error_message = "Unexpected format of output";
        }
    } elsif ( $raw_output =~ /Documentation\s+can\s+be\s+found\s+at/i ) {
        $error          = "InvalidParam";
        $$error_message = "command failed due to wrong dependicies usage";
    } elsif ( $raw_output =~ /no filter set/i ) {
        $error          = "NoMatchingEntries";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error
1;

