#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/file.pm#1 $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode file command output
##
## @description
##    This is a library to parse the raw output of 7Mode file command.
##
## @synopsis
##    use file;
##
##    file_reservation(%args);
##    file_punchhole(%args);
##    file_fingerprint(%args);
##    file_clear_error(%args);
##    file_kireeti(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::file;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::_Mixins::ZAPI qw(_xml2perl);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command       = $args{command};
    my $error;
    if ( $raw_output =~ /^[Uu]sage:\s(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage or dependencies";
    }elsif ( $raw_output =~ /^file\s*:\s*(.*)/m ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }elsif ( $raw_output =~ /offset and length must be multiple of 4096/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }elsif ( $raw_output =~ /Offset + Length exceeds number/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }elsif ( $raw_output =~ /error opening file/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }elsif ( $raw_output =~ /Invalid algorithm:.*|file fingerprint:.*|File '.*' doesn't exist|Volume '.*' doesn't exist/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::file_reservation
##
## @summary Parses the raw output from the "file reservation" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from file_reservation with any of its options as described in the
##    corresponding cdef file 'file.cdefs'
## @synopsis
##      file_reservation(%args);
## INPUTS TO METHOD CALL 'file_reservation' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub file_reservation {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    my %hash;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Any errors will be caught in the previous block. If it has reached
    # here, then the command has successfully executed.
    if ($output =~ /space reservations for file (.+?)\s*:\s*(.+)/) {
        $hash{file} = $1;
        $hash{status} = $2;
        return [\%hash];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }
}
## @name ::file_punchhole
##
## @summary Parses the raw output from the "file punchhole" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from file_punchhole with any of its options as described in the
##    corresponding cdef file 'file.cdefs'
## @synopsis
##      file_punchhole(%args);
## INPUTS TO METHOD CALL 'file_punchhole' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'length' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'offset' </td> </tr>
## <tr> <td> -b </td> <td> 'punch-bad' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## $VAR = [];
##
sub file_punchhole {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error( raw_output    => $output,
                                error_message => \$error_message,
                                command       => $command );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::file_fingerprint
##
## @summary Parses the raw output from the "file fingerprint" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from file_fingerprint with any of its options as described in the
##    corresponding cdef file 'file.cdefs'
## @synopsis
##      file_fingerprint(%args);
## INPUTS TO METHOD CALL 'file_fingerprint' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'data-scope' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'metadata-scope' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'digest-algorithm' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'xml-output' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub file_fingerprint {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error( raw_output    => $output,
                                error_message => \$error_message,
                                command       => $command );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    @result_array = _xml2perl(undef, xmlString => $output, ForceArray => 1);
    return \@result_array;
} #end sub file_fingerprint

## @name ::file_clear_error
##
## @summary Parses the raw output from the "file clear_error" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from file_clear_error with any of its options as described in the
##    corresponding cdef file 'file.cdefs'
## @synopsis
##      file_clear_error(%args);
## INPUTS TO METHOD CALL 'file_clear_error' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-vol' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-fbns' </td> <td> 'fbns' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system node run -node local -command "priv set -q test; file clear_error /vol/source(1)/file_1  -inaccessible;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Output will be saved to file '/etc/log/file_clear_error.source(1).96.20140505061617'
##&nbsp;A scanner has been successfully started
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'A scanner has been successfully started',
##&nbsp;            'log_file' => '/etc/log/file_clear_error.source(1).96.20140505061617'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub file_clear_error {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error( raw_output    => $output,
                                error_message => \$error_message,
                                command       => $command );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ($output =~ m/Output\s+will\s+be\s+saved\s+to\s+file\s+\'(.*)\'.*(A\s+scanner\s+has\s+been\s+successfully\s+started)/is) {
        return [
            {
                status => $2,
                log_file => $1,
            }
        ];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $resp
        );
    }

} #end sub file_clear_error

## @name ::file_kireeti
##
## @summary Parses the raw output from the "file kireeti" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from file_kireeti with any of its options as described in the
##    corresponding cdef file 'file.cdefs'
## @synopsis
##      file_kireeti(%args);
## INPUTS TO METHOD CALL 'file_kireeti' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-l' </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-private' </td> <td> 'private' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'fbn' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapid' </td> <td> '0' </td> </tr>
## <tr> <td> '-force' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub file_kireeti {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error( raw_output    => $output,
                                error_message => \$error_message,
                                command       => $command );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} #end sub file_kireeti

1;
