#$Id: //depot/prod/test/main/lib/NACL/APISet/Node/CLI/7Mode/default/filersio.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode filersio commands output
##
## @description
##    This is a library to parse the raw output of 7Mode filersio commands.
##
## @synopsis
##    use filersio;
##
##    filersio_asyncio_active(%args);
##    filersio_asyncio_pause(%args);
##    filersio_status(%args);
##    filersio_stop(%args);
##    filersio_spc1(%args);
##
## @author udaykk@netapp.com, tanil@netapp.com, jais@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::filersio;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /SYNOPSIS\s+.*\s.*\s.*\s+.*\s(filersio):(.*)/ ) {
            $$error_message = $1.$2;
            $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /filersio:\s+failed\s+to\s+start/ ) {
        if ($raw_output =~ /filersio:\s+(unknown\s+operand\s+.*)filersio:.*/ ) {
            $$error_message = $1;
            $error          = "MissingRequiredParam";
        } elsif ( $raw_output =~ /.*filersio:\s+(unknown\s+option).*|SYNOPSIS/ ) {
            $$error_message = $raw_output;
            $error          = "InvalidParam";
        } elsif ( $raw_output
            =~ /(.*filersio:\s+Unable\s+to\s+locate\s+volume\s+for\s+file|file\s+.*\s+does\s+not\s+exist.*)/ ) {
			$$error_message = $1;
            $error          = "InvalidParamValue";
        } elsif ($raw_output
            =~ /filersio: \"pause\" should be 10ms or greater|filersio: \"read_percentage\" should be between 0-100/){
		    $$error_message = $raw_output;
            $error          = "InvalidParamValue";
		} else {
            $$error_message = "Command Failed";
            $error          = "CommandFailed";
        }
    } elsif ( $raw_output =~ /filersio:\s+currently running workload/) {
	$$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /filersio: rand_percentage should be between 0-100|filersio: valid instance range is 0 - 63/){
	$$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif($raw_output =~ /(filersio: instance \d+ not currently running)|(filersio: no instances currently running)/){
	$$error_message = $raw_output;
        $error          = "CommandFailed";
    } elsif($raw_output =~ /(filersio:\s+cannot\s+open\s+cfg\s+file\s+.*)/){
        $$error_message = $1;
        $error          = "InvalidParamValue";
    } elsif($raw_output =~ /(SYNOPSIS\s+filersio spc1)/){
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::filersio_asyncio_active
##
## @summary Parses the raw output from the "filersio asyncio_active" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_asyncio_active with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##      filersio_asyncio_active(%args);
## INPUTS TO METHOD CALL 'filersio_asyncio_active' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-create' </td> <td> 'create' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-active' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rand-percentage' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'read-percentage' </td> </tr>
## <tr> <td> '-verifyonce' </td> <td> 'verify-pattern' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'run-time' </td> </tr>
## <tr> <td> '-print_stats' </td> <td> 'interval' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'blk-size' </td> </tr>
## <tr> <td> '-fillonce' </td> <td> 'fill-pattern' </td> </tr>
## <tr> <td> '-runonce' </td> <td> 'runonce' </td> </tr>
## <tr> <td> '-e' </td> <td> 'exp-constant' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> </tr>
## <tr> <td> '-synchronous' </td> <td> 'synchronous' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-byte' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-byte' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = filersio asyncio_active 100 -r 0 64k 0 1M 7 128 /vol/vol0/testfile1 -create
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'filersio_status' => 'workload initiated asynchronously'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = filersio asyncio_active 100 -r 0 64k 0 1M 7 128 /vol/vol0/testfile1 -synchronous -create
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'max_read_latency_ms' => '10',
##&nbsp;                                'write_ios' => '0',
##&nbsp;                                'total_read_latency_ms' => '896896',
##&nbsp;                                'max_write_latency_ms' => '0',
##&nbsp;                                'run_times' => '7',
##&nbsp;                                'instance' => '0',
##&nbsp;                                'total_write_latency_ms' => '0',
##&nbsp;                                'avg_write_iops' => '0',
##&nbsp;                                'avg_write_latency_ms' => '0',
##&nbsp;                                'read_ios' => '319600',
##&nbsp;                                'avg_read_latency_ms' => '2',
##&nbsp;                                'filersio_status' => 'workload completed successfully',
##&nbsp;                                'avg_read_iops' => '45657'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = filersio asyncio_active 100 -r 0 64k 0 1M 7 128 /vol/vol0/testfile1 -synchronous -print_stats 2 -create
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'max_read_latency_ms' => '20',
##&nbsp;                                'write_ios' => '0',
##&nbsp;                                'total_read_latency_ms' => '896512',
##&nbsp;                                'max_write_latency_ms' => '0',
##&nbsp;                                'run_times' => '7',
##&nbsp;                                'instance' => '0',
##&nbsp;                                'total_write_latency_ms' => '0',
##&nbsp;                                'avg_write_iops' => '0',
##&nbsp;                                'avg_write_latency_ms' => '0',
##&nbsp;                                'read_ios' => '320800',
##&nbsp;                                'avg_read_latency_ms' => '2',
##&nbsp;                                'filersio_status' => 'workload completed successfully',
##&nbsp;                                'avg_read_iops' => '45828'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'print_stats' => [
##&nbsp;                               {
##&nbsp;                                 'max_read_latency_ms' => '10',
##&nbsp;                                 'avg_write_latency_ms' => '0',
##&nbsp;                                 'write_ios' => '0',
##&nbsp;                                 'max_write_latency_ms' => '0',
##&nbsp;                                 'read_ios' => '86285',
##&nbsp;                                 'avg_read_latency_ms' => '2'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'max_read_latency_ms' => '20',
##&nbsp;                                 'avg_write_latency_ms' => '0',
##&nbsp;                                 'write_ios' => '0',
##&nbsp;                                 'max_write_latency_ms' => '0',
##&nbsp;                                 'read_ios' => '85706',
##&nbsp;                                 'avg_read_latency_ms' => '3'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'max_read_latency_ms' => '10',
##&nbsp;                                 'avg_write_latency_ms' => '0',
##&nbsp;                                 'write_ios' => '0',
##&nbsp;                                 'max_write_latency_ms' => '0',
##&nbsp;                                 'read_ios' => '98517',
##&nbsp;                                 'avg_read_latency_ms' => '2'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub filersio_asyncio_active {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _filersio_asyncio_active(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} # end sub filersio_asyncio_active

sub _filersio_asyncio_active {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # Extracting the command option.
    my $option = $command_line;
    $option =~ s/filersio\sasyncio_active\s?//;
    if ( $option !~ /-\w/ ) {
        $option = "";
    }
    if ( $option !~ /-synchronous/ ) {

        # Workload initiated asynchronously
        if ( $raw_output
            =~ /filersio:\s+(workload\s+initiated\s+asynchronously).*/ ) {
            my @asynchronous = ( { 'filersio_status' => $1 } );
            @result_array = ( { 'statistics' => \@asynchronous } );
            return \@result_array;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $response
            );
        } # end else [ if ( $raw_output =~ ...
    } else {

        # For filersio asyncio_active, some output is always expected.
        # So we are making $can_throw as 1 so that if the
        # lines array got finally in _get_lines is empty,
        # UnexpectedOutputException can be thrown there.
        my $can_throw = 1;
        my $first_line
            = 'filersio:\s+starting\s+workload\s+asyncio_active,\s+instance\s+\d+';
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @headings_prints_stats;
        my @headings_statistics;
        my @print_stats_output;
        my @statistics_output;

        if ( $arr[0]
            =~ /^filersio:\s+starting\s+workload\s+asyncio_active,\s+instance\s+\d+$/ ) {
            shift(@arr);
        } # end if ( $arr[0] =~ ...
        for ( my $line = 0; $line < @arr; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line]
                =~ /Read\s+I\/Os\s+Avg\.\s+read\s+Max\.\s+read\s+Write\s+I\/Os\s+Avg\.\s+write\s+Max\s+write/ ) {
                @headings_prints_stats
                    = qw(read_ios avg_read_latency_ms max_read_latency_ms write_ios avg_write_latency_ms max_write_latency_ms);

                # Remove the heading and underline lines
                splice( @arr, $line, 2 );
                $line--;
                next;
            } ## end if ( $arr[$line] =~ ...
            if ( $arr[$line] =~ /^\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+$/ ) {
                my @values = split( /\s+/, $arr[$line] );
                my $index = 0;
                my %print_stats;
                foreach (@values) {
                    $print_stats{ $headings_prints_stats[ $index++ ] } = $_;
                }
                push( @print_stats_output, \%print_stats );
            } elsif ( $arr[$line]
                =~ /^Statistics\s+for\s+active_active\s+model,\s+instance\s+\d+$/ ) {
                $first_line
                    = 'Statistics\s+for\s+active_active\s+model,\s+instance\s+\d+';
                @arr = get_lines(
                    "raw_output"      => $raw_output,
                    "first_line"      => $first_line,
                    "can_throw"       => $can_throw,
                    "response_object" => $response
                );
                last;
            }
        }
        @headings_statistics
            = qw(instance run_times total_read_latency_ms read_ios avg_read_iops avg_read_latency_ms max_read_latency_ms total_write_latency_ms write_ios avg_write_iops avg_write_latency_ms max_write_latency_ms filersio_status);
        my %statistics;
        for ( my $line = 0; $line < @arr; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line]
                =~ /Statistics\s+for\s+active_active\s+model,\s+instance\s+(\d+)/ ) {
                $statistics{ $headings_statistics[0] } = $1;
            } elsif ( $arr[$line] =~ /Running\s+for\s+(\d+)s/ ) {
                $statistics{ $headings_statistics[1] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[2] } = $1;
            } elsif ( $arr[$line] =~ /Read\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[3] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[4] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[5] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[6] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[7] } = $1;
            } elsif ( $arr[$line] =~ /Write\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[8] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[9] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[10] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[11] } = $1;
            } elsif ( $arr[$line]
                =~ /filersio:\s+instance\s+\d+:\s+(workload\s+completed\s+successfully)/ ) {
                $statistics{ $headings_statistics[12] } = $1;
            } else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "unexpected format of output",
                    response_object => $response
                );
            }
        }
        push( @statistics_output, \%statistics );
        @result_array = ( { 'statistics' => \@statistics_output } );
        if (@print_stats_output) {
            $result_array[0]->{'print_stats'} = \@print_stats_output;
        }

        return \@result_array;
    }
} # end sub _filersio_asyncio_active

sub _filersio_asyncio_pause {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # Extracting the command option.
    my $option = $command_line;
    $option =~ s/filersio\sasyncio_pause\s?//;
    #if ( $option !~ /-\w/ ) {
     #   $option = "";
    #}
    if ( $option !~ /-synchronous/ ) {
        # Workload initiated asynchronously
        if ( $raw_output
            =~ /filersio:\s+(workload\s+initiated\s+asynchronously).*/ ) {
            my @asynchronous = ( { 'filersio_status' => $1 } );
            @result_array = ( { 'statistics' => \@asynchronous } );
            return \@result_array;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "unexpected format of output",
                response_object => $response
            );
        } # end else [ if ( $raw_output =~ ...
    } else {
        # For filersio asyncio_active, some output is always expected.
        # So we are making $can_throw as 1 so that if the
        # lines array got finally in _get_lines is empty,
        # UnexpectedOutputException can be thrown there.
        my $can_throw = 1;
        my $first_line
            = 'filersio:\s+starting\s+workload\s+asyncio_pause';
        my @arr = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my @headings_prints_stats;
        my @headings_statistics;
        my @print_stats_output;
        my @statistics_output;
        if ( $arr[0]
            =~ /^filersio:\s+starting\s+workload\s+asyncio_pause$/ ) {
            shift(@arr);
        } # end if ( $arr[0] =~ ...
        for ( my $line = 0; $line < @arr; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line]
                =~ /Read\s+I\/Os\s+Avg\.\s+read\s+Max\.\s+read\s+Write\s+I\/Os\s+Avg\.\s+write\s+Max\s+write/ ) {
                @headings_prints_stats
                    = qw(read_ios avg_read_latency_ms max_read_latency_ms write_ios avg_write_latency_ms max_write_latency_ms);
                # Remove the heading and underline lines
                splice( @arr, $line, 2 );
                $line--;
                next;
            } ## end if ( $arr[$line] =~ ...
            if ( $arr[$line] =~ /^\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+$/ ) {
                my @values = split( /\s+/, $arr[$line] );
                my $index = 0;
                my %print_stats;
                foreach (@values) {
                    $print_stats{ $headings_prints_stats[ $index++ ] } = $_;
                }
                push( @print_stats_output, \%print_stats );
            } elsif ( $arr[$line]
                =~ /^Statistics\s+for\s+asyncio_pause\s+model,\s+instance\s+\d+$/ ) {
                $first_line
                    = 'Statistics\s+for\s+asyncio_pause\s+model,\s+instance\s+\d+';
                @arr = get_lines(
                    "raw_output"      => $raw_output,
                    "first_line"      => $first_line,
                    "can_throw"       => $can_throw,
                    "response_object" => $response
                );
                last;
            }
        }
        @headings_statistics
            = qw(instance run_times total_read_latency_ms read_ios avg_read_iops avg_read_latency_ms max_read_latency_ms total_write_latency_ms write_ios avg_write_iops avg_write_latency_ms max_write_latency_ms total_iops target_iops percent_of_requested_throughput_achieved filersio_status);
        my %statistics;
        for ( my $line = 0; $line < @arr; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line]
                =~ /Statistics\s+for\s+asyncio_pause\s+model,\s+instance\s+(\d+)/ ) {
                $statistics{ $headings_statistics[0] } = $1;
            } elsif ( $arr[$line] =~ /Running\s+for\s+(\d+)s/ ) {
                $statistics{ $headings_statistics[1] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[2] } = $1;
            } elsif ( $arr[$line] =~ /Read\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[3] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[4] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[5] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[6] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[7] } = $1;
            } elsif ( $arr[$line] =~ /Write\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[8] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[9] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[10] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[11] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[12] } = $1;
            } elsif ( $arr[$line] =~ /Target\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[13] } = $1;
            } elsif ( $arr[$line] =~ /Percent\s+of\s+requested\s+throughput\s+achieved:\s+([\d\.\%]+)/ ) {
                $statistics{ $headings_statistics[14] } = $1;
            } elsif ( $arr[$line]
                =~ /filersio:\s+(workload\s+completed\s+successfully)/ ) {
                $statistics{ $headings_statistics[15] } = $1;
            } else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "unexpected format of output",
                    response_object => $response
                );
            }
        }
        push( @statistics_output, \%statistics );
        @result_array = ( { 'statistics' => \@statistics_output } );
        if (@print_stats_output) {
            $result_array[0]->{'print_stats'} = \@print_stats_output;
        }

        return \@result_array;
    }
} # end sub _filersio_asyncio_pause

## @name ::filersio_asyncio_pause
##
## @summary Parses the raw output from the "filersio asyncio_pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_asyncio_pause with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##      filersio_asyncio_pause(%args);
## INPUTS TO METHOD CALL 'filersio_asyncio_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-create' </td> <td> 'create' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'count' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rand-percentage' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'read-percentage' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'run-time' </td> </tr>
## <tr> <td> '-print_stats' </td> <td> 'interval' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'blk-size' </td> </tr>
## <tr> <td> '-max_ios' </td> <td> 'maxio' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pause' </td> </tr>
## <tr> <td> '-e' </td> <td> 'exp-constant' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> </tr>
## <tr> <td> '-synchronous' </td> <td> 'synchronous' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-byte' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-byte' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; filersio asyncio_pause 12 -r 10  16k 0 20g 10 10 2 /vol/vol0/testfile1 -synchronous -create; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'max_read_latency_ms' => '38',
##&nbsp;                                'write_ios' => '1786',
##&nbsp;                                'total_read_latency_ms' => '3564',
##&nbsp;                                'max_write_latency_ms' => '48',
##&nbsp;                                'run_times' => '18',
##&nbsp;                                'instance' => '0',
##&nbsp;                                'total_write_latency_ms' => '29280',
##&nbsp;                                'avg_write_iops' => '99',
##&nbsp;                                'avg_write_latency_ms' => '16',
##&nbsp;                                'read_ios' => '214',
##&nbsp;                                'avg_read_latency_ms' => '16',
##&nbsp;                                'filersio_status' => 'workload completed successfully',
##&nbsp;                                'avg_read_iops' => '11'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; filersio asyncio_pause 1 -r 0  16k 0 10m 10 10 2 /vol/vol0/testfile1 -synchronous -create -print_stats 10; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'max_read_latency_ms' => '29',
##&nbsp;                                'write_ios' => '1974',
##&nbsp;                                'total_read_latency_ms' => '440',
##&nbsp;                                'max_write_latency_ms' => '37',
##&nbsp;                                'run_times' => '18',
##&nbsp;                                'instance' => '0',
##&nbsp;                                'total_write_latency_ms' => '32172',
##&nbsp;                                'avg_write_iops' => '109',
##&nbsp;                                'avg_write_latency_ms' => '16',
##&nbsp;                                'read_ios' => '26',
##&nbsp;                                'avg_read_latency_ms' => '16',
##&nbsp;                                'filersio_status' => 'workload completed successfully',
##&nbsp;                                'avg_read_iops' => '1'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'print_stats' => [
##&nbsp;                               {
##&nbsp;                                 'max_read_latency_ms' => '29',
##&nbsp;                                 'avg_write_latency_ms' => '15',
##&nbsp;                                 'write_ios' => '1058',
##&nbsp;                                 'max_write_latency_ms' => '29',
##&nbsp;                                 'read_ios' => '16',
##&nbsp;                                 'avg_read_latency_ms' => '18'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; filersio asyncio_pause 100 -r 0 64k 0 1M 7 10 5 /vol/vol0/testfile1 -print_stats 2 -max_ios 512; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'filersio_status' => 'workload initiated asynchronously'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub filersio_asyncio_pause {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _filersio_asyncio_pause(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} # end sub filersio_asyncio_pause

## @name ::filersio_status
##
## @summary Parses the raw output from the "filersio status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_status with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##      filersio_status(%args);
## INPUTS TO METHOD CALL 'filersio_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'instance-id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; filersio status 0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'max_read_latency_ms' => '19',
##&nbsp;                                'write_ios' => '503',
##&nbsp;                                'total_read_latency_ms' => '19',
##&nbsp;                                'max_write_latency_ms' => '28',
##&nbsp;                                'run_times' => '4',
##&nbsp;                                'instance' => '0',
##&nbsp;                                'total_write_latency_ms' => '9375',
##&nbsp;                                'avg_write_iops' => '125',
##&nbsp;                                'avg_write_latency_ms' => '18',
##&nbsp;                                'read_ios' => '1',
##&nbsp;                                'avg_read_latency_ms' => '19',
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub filersio_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
	my @result_array;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %statistics;
    my @headings_statistics;
	my @statistics_output;
    if($output =~ /filersio:\s+instance\s+(\d+):\s+(.*)/i) {
	    @headings_statistics
            = qw(instance filersio_status);
		$statistics{ $headings_statistics[0] } = $1;
		$statistics{ $headings_statistics[1] } = $2;
    } else {
	@headings_statistics
            = qw(instance run_times total_read_latency_ms read_ios avg_read_iops avg_read_latency_ms max_read_latency_ms total_write_latency_ms write_ios avg_write_iops avg_write_latency_ms max_write_latency_ms missed_ios total_iops target_iops percent_of_requested_throughput_achieved );
        my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
        for ( my $line = 0; $line < @arr; $line++ ) {
            next if ( $arr[$line] eq "" );
            if ( $arr[$line]
                =~ /Statistics\s+for\s+.*\s+model,\s+instance\s+(\d+)/ ) {
                $statistics{ $headings_statistics[0] } = $1;
            } elsif ( $arr[$line] =~ /Running\s+for\s+(\d+)s/ ) {
                $statistics{ $headings_statistics[1] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[2] } = $1;
            } elsif ( $arr[$line] =~ /Read\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[3] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[4] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[5] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+read\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[6] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[7] } = $1;
            } elsif ( $arr[$line] =~ /Write\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[8] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[9] } = $1;
            } elsif ( $arr[$line] =~ /Avg\.\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[10] } = $1;
            } elsif ( $arr[$line] =~ /Max\s+write\s+latency\(ms\)\s+(\d+)/ ) {
                $statistics{ $headings_statistics[11] } = $1;
            } elsif ( $arr[$line] =~ /Missed\s+I\/Os\s+(\d+)/ ) {
                $statistics{ $headings_statistics[12] } = $1;
            } elsif ( $arr[$line] =~ /Total\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[13] } = $1;
            } elsif ( $arr[$line] =~ /Target\s+IOPS\s+(\d+)/ ) {
                $statistics{ $headings_statistics[14] } = $1;
            } elsif ( $arr[$line] =~ /Percent\s+of\s+requested\s+throughput\s+achieved:\s+([\d\.\%]+)/ ) {
                $statistics{ $headings_statistics[15] } = $1;
            }
             else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "unexpected format of output",
                    response_object => $resp
                );
			}
		}
		push( @statistics_output, \%statistics );
        @result_array = ( { 'statistics' => \@statistics_output } );
		return \@result_array;
	}
} # end sub filersio_status

## @name ::filersio_stop
##
## @summary Parses the raw output from the "filersio stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_stop with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##      filersio_stop(%args);
## INPUTS TO METHOD CALL 'filersio_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'instance-id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; filersio stop 0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'filersio_status' => 'aborting'
##&nbsp;                                'instance' => '0'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub filersio_stop {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
	my @result_array;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %statistics;
	my @statistics_output;
    if($output =~ /filersio:\s+(\w+)\s+instance\s+(\d+)(:\s+.*)?/i) {
       my @headings_statistics
            = qw(instance filersio_status);
		$statistics{ $headings_statistics[0] } = $2;
		$statistics{ $headings_statistics[1] } = $1;
	}
	push( @statistics_output, \%statistics );
    @result_array = ( { 'statistics' => \@statistics_output } );
	return \@result_array;
} # end sub filersio_status

## @name ::filersio_spc1
##
## @summary Parses the raw output from the "filersio spc1" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_spc1 with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##      filersio_spc1(%args);
## INPUTS TO METHOD CALL 'filersio_spc1' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-max_ios' </td> <td> 'maxio' </td> <td> '0' </td> </tr>
## <tr> <td> '-synchronous' </td> <td> 'synchronous' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'cfg-file' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ramp-time' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'run-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-print_stats' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-bsu' </td> <td> '0' </td> </tr>
## <tr> <td> '-hostspcgen_like_stats' </td> <td> 'hostspcgen-like-stats' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub filersio_spc1{
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];

} # end sub filersio_spc1


## @name filersio_ereg
##
## @summary Parses the raw output from the "filersio ereg" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from filersio_ereg with any of its options as described in the
##    corresponding cdef file 'filersio.cdefs'
## @synopsis
##    use filersio;
##
##    filersio_ereg(%args);
## @author chenp@netapp.com, dl-metawafl-qa
##
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.

my %method_hash = (

    filersio_ereg => {
        cmd_pass_string => /^filersio: (starting workload ereg|workload initiated asynchronously)/,
    },

);

parser_method_builder(__PACKAGE__, %method_hash);

1;

