#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/flexcache.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode flexcache command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode flexcache command.
##
## @synopsis
##    use flexcache;
##
##    flexcache_fstat(%args);
##    flexcache_stats(%args);
##    flexcache_wb_dbg(%args);
##    flexcache_eject(%args);
##    flexcache_delegations(%args);
##    flexcache_monitor(%args);
##    flexcache_hist(%args);
##    flexcache_wb(%args);
##    flexcache_wb_maint(%args);
##    flexcache_wb_snap(%args);
##    flexcache_blockstat(%args);
##    flexcache_eject_blocks(%args);
##    flexcache_sparseify(%args);
##    flexcache_panic_origin(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::flexcache;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException ;

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error;
    my $error_message = undef;
    my $error_level   = $resp->get_execution_status();
	if ( $raw_output =~ /doesn't\s+host\s+any\s+cache\s+volumes|writeback_mode\s+is\s+not\s+enabled|FlexCache\s+is\s+not\s+enabled/ ){
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /[Uu]sage:|flexcache stats:|flexcache fstat:|Object does not exist|No volumes found|Invalid file name/
        ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::flexcache_fstat
##
## @summary Parses the raw output from the "flexcache fstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_fstat with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_fstat(%args);
## INPUTS TO METHOD CALL 'flexcache_fstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'inode-vol' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inode-file' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; flexcache fstat /vol/vol1/testfile; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Properties:
##&nbsp;    Filename: /vol/vol1/testfile
##&nbsp;    Attributes:
##&nbsp;        Type: file
##&nbsp;        Mode: 0644
##&nbsp;        Nlinks: 1
##&nbsp;        UID: 0
##&nbsp;        GID: 0
##&nbsp;        Size: 16
##&nbsp;        Blocks: 0
##&nbsp;        Indirect Blocks: 0
##&nbsp;        Fsid: 0xf9c1f469
##&nbsp;        Fileid: 96
##&nbsp;        Xinode: 0 (gen 0)
##&nbsp;        Sinode: 0 (gen 0)
##&nbsp;        AV version: 0
##&nbsp;        Security: 0 (qtree=0 sec=0)
##&nbsp;        Last access time: Wed, 11 Jul 2012 09:13:56 (+304342 uSec) GMT
##&nbsp;        Last modified time: Wed, 11 Jul 2012 09:13:59 (+244393 uSec) GMT
##&nbsp;        Last change time: Wed, 11 Jul 2012 09:13:59 (+244393 uSec) GMT
##&nbsp;        Creation time: Wed, 11 Jul 2012 09:13:56 (+304342 uSec) GMT
##&nbsp;        Inode Generation #: 1688579
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'inode_generation' => '1688579',
##&nbsp;            'xinode' => '0 (gen 0)',
##&nbsp;            'fileid' => '96',
##&nbsp;            'mode' => '0644',
##&nbsp;            'sinode' => '0 (gen 0)',
##&nbsp;            'last_change_time' => 'Wed, 11 Jul 2012 09:13:59 (+244393 uSec) GMT',
##&nbsp;            'size' => '16',
##&nbsp;            'last_access_time' => 'Wed, 11 Jul 2012 09:13:56 (+304342 uSec) GMT',
##&nbsp;            'filename' => '/vol/vol1/testfile',
##&nbsp;            'security' => '0 (qtree=0 sec=0)',
##&nbsp;            'creation_time' => 'Wed, 11 Jul 2012 09:13:56 (+304342 uSec) GMT',
##&nbsp;            'type' => 'file',
##&nbsp;            'last_modified_time' => 'Wed, 11 Jul 2012 09:13:59 (+244393 uSec) GMT',
##&nbsp;            'nlinks' => '1',
##&nbsp;            'fsid' => '0xf9c1f469'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub flexcache_fstat {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        my @values = split( /:\s+/, $line );
        $values[0] =~ s/\s+#//;
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1]
            if ( $values[1] );
    }
    return \@result_array;
} ## end sub flexcache_fstat

## @name ::flexcache_stats
##
## @summary Parses the raw output from the "flexcache stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_stats with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_stats(%args);
## INPUTS TO METHOD CALL 'flexcache_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-z' </td> <td> 'zero-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'each-client' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'caching' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'clientname' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'server-stats' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; flexcache stats -S -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Volume                                                                 Total Ops
##&nbsp;vol2                                                                 5160 ( 96%)
##&nbsp;
##&nbsp;   Client anupamap-vsim-1.sim.eng.btc.netapp.in
##&nbsp;          Volume Ops     In (MB)   In (KB/s)    Out (MB)  Out (KB/s)
##&nbsp;         5160 (100%)           0           0           0           0
##&nbsp;
##&nbsp;Volume                                                                 Total Ops
##&nbsp;(other)                                                               166 (  3%)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'in_mb' => '0',
##&nbsp;            'total_ops_percentage' => '96',
##&nbsp;            'total_ops' => '5160',
##&nbsp;            'volume' => 'vol2',
##&nbsp;            'volume_ops' => '5160',
##&nbsp;            'volume_ops_percentage' => '100',
##&nbsp;            'client' => 'anupamap-vsim-1.sim.eng.btc.netapp.in',
##&nbsp;            'in_kb_per_s' => '0',
##&nbsp;            'out_mb' => '0',
##&nbsp;            'out_kb_per_s' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_ops_percentage' => '3',
##&nbsp;            'total_ops' => '166',
##&nbsp;            'volume' => 'other'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; flexcache stats -C; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;          Volume  Total Ops    Hit     Hit-    Miss    Miss-   Proxy   Bandwidth
##&nbsp;                                     Verify           Verify    -ops     Savings
##&nbsp;            vol3       2407     5%      95%      0%       0%      0%          0%
##&nbsp;
##&nbsp;          Volume   Read Ops    Hit     Hit-    Miss    Miss-   Proxy
##&nbsp;                                     Verify           Verify    -ops
##&nbsp;            vol3       2353     2%      97%      0%       0%      0%
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_ops_info' => {
##&nbsp;                                  'hit_verify_percentage' => '95',
##&nbsp;                                  'proxy_ops_percentage' => '0',
##&nbsp;                                  'total_ops' => '2407',
##&nbsp;                                  'bandwidth_savings_percentage' => '0',
##&nbsp;                                  'miss_percentage' => '0',
##&nbsp;                                  'miss_verify_percentage' => '0',
##&nbsp;                                  'hit_percentage' => '5'
##&nbsp;                                },
##&nbsp;            'volume' => 'vol3',
##&nbsp;            'read_ops_info' => {
##&nbsp;                                 'hit_verify_percentage' => '97',
##&nbsp;                                 'proxy_ops_percentage' => '0',
##&nbsp;                                 'miss_percentage' => '0',
##&nbsp;                                 'miss_verify_percentage' => '0',
##&nbsp;                                 'hit_percentage' => '2',
##&nbsp;                                 'read_ops' => '2353'
##&nbsp;                               }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub flexcache_stats {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $c = -1;
    my ( @head, @values, %hash_pos );
    my $temp_head;

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /Volume/ ) {
            $lines[$i] =~ s/Hit-/hit_verify_percentage/;
            $lines[$i] =~ s/Miss-/miss_verify_percentage/;
            $lines[$i] =~ s/Hit/hit_percentage/;
            $lines[$i] =~ s/Miss/miss_percentage/;
            $lines[$i] =~ s/Proxy/proxy_ops_percentage/;
            $lines[$i] =~ s/Bandwidth/bandwidth_savings_percentage/;
            @head = split( /\s\s+/, $lines[$i] );
            $c = 0 if ( $cmd =~ /-C/ );
            $c++ if ( $cmd =~ /-S/ && $lines[$i] =~ /Volume\s\s+/ );
        } elsif ( $lines[$i] =~ /\w+\s+\d+\s+|^\(other\)/ ) {
            @values = split( /\s\s\s+/, $lines[$i] );
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $head[$j] =~ s/\//_per_/;
                $values[$j] =~ s/%// if($head[$j] =~ /percentage/);
                $values[$j] =~ s/\(other\)/other/;
                if ( $head[$j] =~ /Ops/ ) {
                    $temp_head = $head[$j] . " " . "info";
                    $temp_head = to_key_name( item => $temp_head );
                } elsif ( $head[$j] =~ /Volume/ ) {
                    $temp_head = undef;
                }
                ## The Total Ops & Read Ops are performed for same volume
                ## So each volume has two records.
                ## So the records(Total Ops & Read Ops) are combined under a volume
                ## When the first Ops is found for a volume, the position is got.
                ## The rest of the Ops for that volume are put into the same position.
                if ( $cmd =~ /-C/ ) {
                    if ( not defined $hash_pos{ $values[0] } ) {
                        $hash_pos{ $values[0] } = $c;
                    }
                    foreach my $key ( keys(%hash_pos) ) {
                        if ( $key ne $values[0] ) {
                            $hash_pos{ $values[0] } = $c;
                            $c++;
                        } else {
                            $c = $hash_pos{ $values[0] };
                        }
                    }
                }
                if ( $temp_head && ( $cmd !~ /-S/ ) ) {
                    $result_array[$c]->{$temp_head}
                        ->{ to_key_name( item => $head[$j] ) } = $values[$j];
                } else {
                    # Example : The Volume Ops has Ops number & percentage
                    # So splitting the number & percentage
                    if($head[$j] =~ /Ops/ && $values[$j] =~ /(\d+)\s+\(\s*(.*)%\s*\)/) {
                       $values[$j] = $1;
                       $result_array[$c]->{ to_key_name( item => "$head[$j]_percentage")} = $2;
                    }
                    $result_array[$c]->{ to_key_name( item => $head[$j] ) }
                        = $values[$j];
                }
            }

        } elsif ( $lines[$i] =~ /(Client)\s+/ ) {
            $result_array[$c]->{ to_key_name( item => $1 ) } = $';
        }
    }
    return \@result_array;
} ## end sub flexcache_stats

## @name ::flexcache_wb_dbg
##
## @summary Parses the raw output from the "flexcache wb_dbg" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_wb_dbg with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_wb_dbg(%args);
## INPUTS TO METHOD CALL 'flexcache_wb_dbg' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'feature' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; flexcache wb_dbg op_to on; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Setting op_to to 20 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'op_to' => '20 ms'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub flexcache_wb_dbg {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    my @result_array;
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    if ( $output =~ /Setting (.*) to (.*)/ ) {
        $result_array[0]->{ to_key_name( item => $1 ) } = $2;
    }
    return \@result_array;
} ## end sub flexcache_wb_dbg

## @name ::flexcache_eject
##
## @summary Parses the raw output from the "flexcache eject" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_eject with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_eject(%args);
## INPUTS TO METHOD CALL 'flexcache_eject' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'data-blocks' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode-vol' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; flexcache eject /vol/volc1 -f -t; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/vol/volc1: ejected
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub flexcache_eject {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_eject

## @name ::flexcache_delegations
##
## @summary Parses the raw output from the "flexcache delegations" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_delegations with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_delegations(%args);
## INPUTS TO METHOD CALL 'flexcache_delegations' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_delegations {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_delegations

## @name ::flexcache_monitor
##
## @summary Parses the raw output from the "flexcache monitor" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_monitor with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_monitor(%args);
## INPUTS TO METHOD CALL 'flexcache_monitor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'cache' </td> <td> 'cache' </td> <td> '1' </td> </tr>
## <tr> <td> 'origin' </td> <td> 'origin' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_monitor {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_monitor

## @name ::flexcache_hist
##
## @summary Parses the raw output from the "flexcache hist" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_hist with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_hist(%args);
## INPUTS TO METHOD CALL 'flexcache_hist' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-z' </td> <td> 'zero-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'server-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'caching' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_hist {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_hist

## @name ::flexcache_wb
##
## @summary Parses the raw output from the "flexcache wb" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_wb with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_wb(%args);
## INPUTS TO METHOD CALL 'flexcache_wb' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'log' </td> <td> 'log' </td> <td> '1' </td> </tr>
## <tr> <td> 'state' </td> <td> 'state' </td> <td> '1' </td> </tr>
## <tr> <td> 'hist' </td> <td> 'hist' </td> <td> '1' </td> </tr>
## <tr> <td> 'clear_stats' </td> <td> 'clear-stats' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_wb {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_wb

## @name ::flexcache_wb_maint
##
## @summary Parses the raw output from the "flexcache wb_maint" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_wb_maint with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_wb_maint(%args);
## INPUTS TO METHOD CALL 'flexcache_wb_maint' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'flush' </td> <td> 'flush' </td> <td> '1' </td> </tr>
## <tr> <td> 'clear' </td> <td> 'clear' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_wb_maint {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_wb_maint

## @name ::flexcache_wb_snap
##
## @summary Parses the raw output from the "flexcache wb_snap" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_wb_snap with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_wb_snap(%args);
## INPUTS TO METHOD CALL 'flexcache_wb_snap' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-disable' </td> <td> 'disable' </td> <td> '1' </td> </tr>
## <tr> <td> '-int' </td> <td> 'interval-in-sec' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-creds' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_wb_snap {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_wb_snap

## @name ::flexcache_blockstat
##
## @summary Parses the raw output from the "flexcache blockstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_blockstat with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_blockstat(%args);
## INPUTS TO METHOD CALL 'flexcache_blockstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'fbn' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bitmap' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_blockstat {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_blockstat

## @name ::flexcache_eject_blocks
##
## @summary Parses the raw output from the "flexcache eject_blocks" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_eject_blocks with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_eject_blocks(%args);
## INPUTS TO METHOD CALL 'flexcache_eject_blocks' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'length' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-offset-fbn' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_eject_blocks {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_eject_blocks

## @name ::flexcache_sparseify
##
## @summary Parses the raw output from the "flexcache sparseify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_sparseify with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_sparseify(%args);
## INPUTS TO METHOD CALL 'flexcache_sparseify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'blocks' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_sparseify {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_sparseify

## @name ::flexcache_panic_origin
##
## @summary Parses the raw output from the "flexcache panic_origin" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flexcache_panic_origin with any of its options as described in the
##    corresponding cdef file 'flexcache.cdefs'
## @synopsis
##      flexcache_panic_origin(%args);
## INPUTS TO METHOD CALL 'flexcache_panic_origin' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub flexcache_panic_origin {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_raw_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    return [];
} ## end sub flexcache_panic_origin

1;
