#
#Copyright (c) 2014 NetApp, Inc.
#All rights reserved.
### @summary
##    Parse 7Mode fmm commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode fmm
##    commands. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "fmm summary" will be fmm_summary().
##
## @synopsis
##    use fmm;
##    fmm_summary(%args);
##    fmm_stats(%args);
##    fmm_status(%args);
##    fmm_info(%args);
##    fmm_syncstats(%args);
##    fmm_zerostats(%args);
##    fmm_state(%args);
##    fmm_thresholds(%args);
##    fmm_syncstats(%args);
##
## @author rahula@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::fmm;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ($output =~ /usage.+/si) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif (($output =~ /(.* is not a valid.*)/si)
        || ($output =~ /(FMM returned: unknown.*)/)) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }

    return $error;
} ## end sub _check_error

## @name ::fmm_summary
##
## @summary Parses the raw output from the "fmm summary" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_summary with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_summary(%args);
## INPUTS TO METHOD CALL 'fmm_summary' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm summary card 6
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot6' => [
##&nbsp;                          {
##&nbsp;                            'board_assembly_number' => '111-00708',
##&nbsp;                            'erase_cycles_avg' => '11',
##&nbsp;                            'erase_cycles_low' => '11(iface 3, bank 7)',
##&nbsp;                            'memory_status' => [],
##&nbsp;                            'card_status' => 'online',
##&nbsp;                            'bad_block_count' => '2798',
##&nbsp;                            'erase_cycles_high' => '12(iface 3, bank 1)',
##&nbsp;                            'corrected_errors' => '7950479'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fmm_summary {
    my %args = @_;
    my ($error_message, $i, $key, $k, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm summary",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @line = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );
    $i = -1;
    for (my $j = 0; $j <= $#line; $j++) {
        if ($line[$j] =~ m/Summary information for slot:\s(.*)/) {
            $i++;
            $key = "slot" . $1;
        } elsif ($line[$j] =~ m/Board Assembly Number:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]->{'board_assembly_number'} = $1;
        } elsif ($line[$j] =~ m/Status of the card:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'card_status'} = $1;
        } elsif ($line[$j] =~ m/Total Bad Block Count for Board:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'bad_block_count'} = $1;
        } elsif ($line[$j] =~ m/Corrected errors:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'corrected_errors'} = $1;
        } elsif ($line[$j] =~ m/Lowest Number of erase cycles:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'erase_cycles_low'} = $1;
        } elsif ($line[$j] =~ m/Highest Number of erase cycles:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'erase_cycles_high'} = $1;
        } elsif ($line[$j] =~ m/Average Number of erase cycles:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'erase_cycles_avg'} = $1;
        } elsif ($line[$j] =~ /^\+(-)+\+$/) {
            if ((   $line[++$j]
                    =~ /(\s*Address\s*\|\s*Bad_blocks\s*\|\s*Avg_erases_per_blk\s*\|\s*Status)/
                )
                && ($line[++$j] =~ /^\+(-)+\+$/)
                ) {
                $j++;
                $k = 0;
                my @hash;
                while ($line[$j] !~ /^\+(-)+\+$/) {
                    $line[$j]
                        =~ /((\d+\/)+\d+)\s*\|\s*(\d+)\s*\|\s*(\d+)\s*\|\s*(.*)/;
                    $hash[$k]->{address}             = $1;
                    $hash[$k]->{bad_blocks}          = $3;
                    $hash[$k]->{avg_erases_per_blks} = $4;
                    $hash[$k]->{status}              = $5;
                    $j++;
                    $k++;
                }
                $result_array[$i]->{$key}->[0]{'memory_status'} = \@hash;
            }
        }
    }
    return \@result_array;
} ## end sub fmm_summary

## @name ::fmm_info
##
## @summary Parses the raw output from the "fmm info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_info with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_info(%args);
## INPUTS TO METHOD CALL 'fmm_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm info card 6
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot6' => [
##&nbsp;                          {
##&nbsp;                            'block_per_core' => '4148',
##&nbsp;                            'cores_per_bank' => '9',
##&nbsp;                            'firewire_revision' => '2:2',
##&nbsp;                            'banks_per_iface' => '8',
##&nbsp;                            'model_name' => 'X1971A-R5',
##&nbsp;                            'part_number' => '111-00708',
##&nbsp;                            'interfaces' => '4',
##&nbsp;                            'erase_cycles_high' => '512 GB'
##&nbsp;                            'capacity' => '512 GB'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fmm_info {
    my %args = @_;
    my @head;
    my ($error_message, $i, $j, $key, $result_array, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm info",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @line = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );
    @line = grep (!(/^(---.*)/), @line);
    @line = grep (!(/^(===.*)/), @line);
    @line = grep(!(/^(INVENTORY INFORMATION.*)/), @line);
    @line = grep(!(/^(\n)/), @line);

    $i = -1;
    for (my $j = 0; $j < @line; $j++) {
        if ($line[$j] =~ m/Inventory information for Slot:.*/) {
            $i++;
        } elsif ($line[$j] =~ m/FMM ID:\s+(.*)\s+Flash Cache in slot\s(.*)/) {
            $key = "slot" . $2;
        } elsif ($line[$j] =~ m/Model name:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'model_name'} = $1;
        } elsif ($line[$j] =~ m/Part number:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'part_number'} = $1;
        } elsif ($line[$j] =~ m/Firmware Revision:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'firewire_revision'} = $1;
            $result_array[$i]->{$key}->[0]{'firmware_revision'} = $1;
        } elsif ($line[$j] =~ m/Global Config reg\s+(.*)/) {
            @head = split(/:\s+/, $line[$j], 2);
            $result_array[$i]->{$key}->[0]{'global_config_reg'} = $head[1];
        } elsif ($line[$j] =~ m/Capacity:\s+(.*)/) {
            @head = split(/:\s+/, $line[$j], 2);
            $result_array[$i]->{$key}->[0]{'erase_cycles_high'} = $head[1];
            $result_array[$i]->{$key}->[0]{'capacity'} = $head[1];
        } elsif ($line[$j] =~ m/Interfaces:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'interfaces'} = $1;
        } elsif ($line[$j] =~ m/Banks per iface:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'banks_per_iface'} = $1;
        } elsif ($line[$j] =~ m/Cores per bank:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'cores_per_bank'} = $1;
        } elsif ($line[$j] =~ m/Blocks per core:\s+(.*)/) {
            $result_array[$i]->{$key}->[0]{'block_per_core'} = $1;
        } else {

            my @head = split(/:\s+/, $line[$j], 2);
            $result_array[$i]->{$key}->[0]{to_key_name(item => $head[0])}
                = $head[1];

        }

    }
    return \@result_array;
} ## end sub fmm_info

## @name ::fmm_zerostats
##
## @summary Parses the raw output from the "fmm zerostats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_zerostats with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_zerostats(%args);
## INPUTS TO METHOD CALL 'fmm_zerostats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm zerostats card 6
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;		    'status' => 'OK'
##&nbsp;		  }
##&nbsp;        ];
##&nbsp;
sub fmm_zerostats {
    my %args = @_;
    my ($error_message, $i, %result_array, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm zerostats",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($output =~ /FMM returned:(.*)/) {
        $result_array{'status'} = $1;
    }
    push(@result_array, \%result_array);
    return \@result_array;
} ## end sub fmm_zerostats

## @name ::fmm_syncstats
##
## @summary Parses the raw output from the "fmm syncstats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_syncstats with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_syncstats(%args);
## INPUTS TO METHOD CALL 'fmm_syncstats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm syncstats card 6
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;		    'status' => 'OK'
##&nbsp;		  }
##&nbsp;        ];
##&nbsp;
sub fmm_syncstats {
    my %args = @_;
    my ($error_message, $i, %result_array, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm syncstats",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    if ($output =~ /FMM returned:(.*)/) {
        $result_array{'status'} = $1;
    }
    push(@result_array, \%result_array);
    return \@result_array;
} ## end sub fmm_syncstats

## @name ::fmm_senderror
##
## @summary Parses the raw output from the "fmm senderror" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_senderror with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_senderror(%args);
## INPUTS TO METHOD CALL 'fmm_senderror' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'nblocks' </td> </tr>
## <tr> <td> 'chip' </td> <td> 'chip' </td> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'error' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'eraseblock' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm senderror card 6 1
##&nbsp;$VAR1 = [];
##&nbsp;
sub fmm_senderror {
    my %args = @_;
    my ($error_message, $i, $result_array, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm senderror",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub fmm_senderror

## @name ::fmm_state
##
## @summary Parses the raw output from the "fmm state" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_state with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_state(%args);
## INPUTS TO METHOD CALL 'fmm_state' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm state on
##&nbsp;$VAR1 = [];
##&nbsp;
sub fmm_state {
    my %args = @_;
    my ($error_message, $i, $result_array, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm state",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub fmm_state

## @name ::fmm_thresholds
##
## @summary Parses the raw output from the "fmm thresholds" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_thresholds with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_thresholds(%args);
## INPUTS TO METHOD CALL 'fmm_thresholds' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm thresholds
##&nbsp;$VAR1 = {
##&nbsp;          'flash-threshold-profile' => [
##&nbsp;                                         {
##&nbsp;                                          'threshold_profile' => 'Version 0',
##&nbsp;                                           'flash-threshold' => [
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'lun',
##&nbsp;                                                                    'action' => 'log an EMS event, and disable the lun If no other LUNs in the containing bank failed ...',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'bad_blocks' => '3 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'lun',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'action' => 'log an EMS event, and disable the lun',
##&nbsp;                                                                    'bad_blocks' => '9 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'bank',
##&nbsp;                                                                    'luns_offline' => '10 percent',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'bank',
##&nbsp;                                                                    'luns_offline' => '20 percent',
##&nbsp;                                                                    'action' => 'log an EMS event, and disable the bank',
##&nbsp;                                                                    'metric' => 'percentage'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event, and send an AutoSupport message',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '10 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '15 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '20 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '25 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event, send an AutoSupport message, and disable the card',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '30 percent'
##&nbsp;                                                                  }
##&nbsp;                                                                ]
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'threshold_profile' => 'Version 1',
##&nbsp;                                           'flash-threshold' => [
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'lun',
##&nbsp;                                                                    'action' => 'log an EMS event, and disable the lun',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'bad_blocks' => '13 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'bank',
##&nbsp;                                                                   'luns_offline' => '1 percent',
##&nbsp;                                                                    'action' => 'log an EMS event, and disable the bank',
##&nbsp;                                                                    'metric' => 'percentage'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event, and send an AutoSupport message',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '25 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                   'banks_offline' => '30 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '35 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '40 percent'
##&nbsp;                                                                  },
##&nbsp;                                                                  {
##&nbsp;                                                                    'domain' => 'card',
##&nbsp;                                                                    'action' => 'log an EMS event, send an AutoSupport message, and disable the card',
##&nbsp;                                                                    'metric' => 'percentage',
##&nbsp;                                                                    'banks_offline' => '50 percent'
##&nbsp;                                                                  }
##&nbsp;                                                               ]
##&nbsp;                                         }
##&nbsp;                                      ]
##&nbsp;        };
##&nbsp;
sub fmm_thresholds {
    my ($key, $i, $error_message, $temp, @result_array);
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => "fmm thresholds",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @line = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );

    my ( $k, $action, $key_2, %threshold );
    $key  = "flash-threshold-profile";
    $key_2 = "flash-threshold";
    $i = $k = -1;
    for (my $j = 0; $j <= $#line; $j++) {
        if ($line[$j] =~ /(.* profile)\s*:\s*(Version\s*\d+)/ ) {
            $i++;
            $k=-1;
            $threshold{$key}->[$i]->{to_key_name(item => $1)} = $2;
        } elsif ( $line[$j] =~ /^(Domain)\s+:\s*(\w*)/ ) {
            $k++;
            $threshold{$key}->[$i]->{$key_2}->[$k]->{to_key_name(item => $1)} = $2;
        } elsif ( $line[$j] =~ /^(\w+\s*?\w*)\s+:\s+(.*)$/) {
             $threshold{$key}->[$i]->{$key_2}->[$k]->{to_key_name(item => $1)} = $2;
        } elsif ( $line[$j] =~ /^(If no other LUN.*)/ ) {
            $action = $threshold{$key}->[$i]->{$key_2}->[$k]->{'action'};
            $threshold{$key}->[$i]->{$key_2}->[$k]->{'action'} .= " $1" ;
        } elsif ($line[$j] eq "else") {
            $k++;
            %{$threshold{$key}->[$i]->{$key_2}->[$k]} = %{$threshold{$key}->[$i]->{$key_2}->[$k-1]} ;
            $threshold{$key}->[$i]->{$key_2}->[$k]->{'action'} = $action;
        }
    }
    push ( @result_array, \%threshold );
    return \@result_array;
} ## end sub fmm_thresholds

## @name ::fmm_status
##
## @summary Parses the raw output from the "fmm status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_status with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_status(%args);
## INPUTS TO METHOD CALL 'fmm_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-S' </td> <td> 'short' </td> </tr>
## <tr> <td> 'bank' </td> <td> 'bank' </td> </tr>
## <tr> <td> 'chip' </td> <td> 'chip' </td> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm status chip 7/3/15/8
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'online',
##&nbsp;            'bad_blks' => '7',
##&nbsp;            'last_change_time' => 'N/A',
##&nbsp;            'total_blks' => '4096'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fmm_status {
    my %args = @_;

    my ($error_message,
        $all,  $i,    $level, $j,    $slot, $var1,
        $var2, $var3, $var4,  $var5, $bank, $chip,
        $c, $itr, @heading, @value, $line
    );
    my %hash;
    my $resp         = $args{response_object};
    my $output       = $resp->get_processed_output();
    my $command      = $resp->get_actual_command();
    my @result_array = ();
    my $error        = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    $all = $bank = $chip = 0;
    if ($command =~ /(.*-a.*)|(.*card.*)|(.*card.*-S.*)/) {
        $all = 1;
    } elsif ($command =~ /(.*bank.*)|(.*bank.*-S.*)/) {
        $bank = 1;
    } elsif ($command =~ /(.*chip.*)/) {
        $chip = 1;
    }

    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );

    for ($i = 0; $i < @lines; $i++) {
        if ($all) {
             while(@lines){
                 $line = shift(@lines);
                 if($line =~ /Status information for slot:\s(\d+)/i){
                     $slot =  "slot" . $1;
                     $line = shift(@lines);
                     $line = shift(@lines) if($line =~ /^[--]/);
                     while (($line !~ /Bank level status/) && (@lines)){
                         @value = split(/\s*[:]\s+/,$line);
                         $hash{$slot}->{to_key_name(item =>$value[0])} = $value[1];
                         $line = shift(@lines);
                     }
                 }
                 if($line =~ /(.*):$/ ){
                    $level = $1;
                    $level =~ tr/A-Z/a-z/;
                    $line = shift(@lines);
                    $line = shift(@lines) if($line =~ /^\(/);
                    $line = shift(@lines) if($line =~ /^[+--]/);
                    @heading = split(/\s+[|]\s+/,$line);
                    $line = shift(@lines) ;
                    $line = shift(@lines) ;
                    $c = 0;
                    while($line !~ /^[+---]/){
                        @value = split(/\s+[|]\s+/,$line);
                        $itr = 0;
                        foreach my $val (@value){
                            $heading[$itr] = 'capacity_percent' if ($heading[$itr] =~ /(.+)[%]+/);
                            $hash{$slot}->{to_key_name(item =>$level)}->[$c]{to_key_name(item => $heading[$itr])} = $val;
                            $itr++;
                         }
                    $c++;
                    $line = shift(@lines);
                    }
                }
           }
        } elsif ($bank) {
            if ($lines[$i] =~ /(.*) level status:/) {
                $level = $1 . "_level_status";
                $j     = 0;
                $level =~ tr/A-Z/a-z/;
            } elsif ($lines[$i]
                =~ /(\S+)\s\|\s+(\d+)\s\|\s+(\d+)\s\|\s+(\w+)\s\|\s+(.*)/) {
                $var1                          = $1;
                $var2                          = $2;
                $var3                          = $3;
                $var4                          = $4;
                $var5                          = $5;
                $hash{$level}->[$j]{'address'} = $var1;
                if ($level =~ /bank.*/) {
                    $hash{$level}->[$j]{'capacity_percent'} = $var2;
                    $hash{$level}->[$j]{'chips_off'}        = $var3;
                } elsif ($level =~ /core.*/) {
                    $hash{$level}->[$j]{'bad_blocks'}     = $var2;
                    $hash{$level}->[$j]{'avg_erases/blk'} = $var3;
                }
                $hash{$level}->[$j]{'status'} = $var4;
                $hash{$level}->[$j]{'time'}   = $var5;
                $j++;
            }
        } elsif ($chip) {
            if ($lines[$i] =~ /Status\s+:\s(\w+)/) {
                $hash{'status'} = $1;
            } elsif ($lines[$i] =~ /Total Blocks\s+:\s(\d+)/) {
                $hash{'total_blks'} = $1;
            } elsif ($lines[$i] =~ /Bad blocks\s+:\s(\d+)/) {
                $hash{'bad_blks'} = $1;
            } elsif ($lines[$i] =~ /Time of last change\s:\s(.*)/) {
                $hash{'last_change_time'} = $1;
            }
        }
    }

    push(@result_array, \%hash);
    return \@result_array;
} ## end sub fmm_status

## @name ::fmm_stats
##
## @summary Parses the raw output from the "fmm stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fmm_status with any of its options as described in the
##    corresponding cdef file 'fmm.cdefs'
## @synopsis
##      fmm_stats(%args);
## INPUTS TO METHOD CALL 'fmm_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-S' </td> <td> 'short' </td> </tr>
## <tr> <td> 'bank' </td> <td> 'bank' </td> </tr>
## <tr> <td> 'card' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fmm stats card 6 -S
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot6' => {
##&nbsp;                         'reads' => '190342641',
##&nbsp;                         'read_latency' => '1247',
##&nbsp;                         'capacity_online_%' => '100',
##&nbsp;                         'write_latency' => '2826',
##&nbsp;                         'Erasures' => '12127996962',
##&nbsp;                         'erase_latency' => '0',
##&nbsp;                         'bad_blks' => '2798',
##&nbsp;                         'writes' => '184109198',
##&nbsp;                         'corrected_errors' => '7950479',
##&nbsp;                         'bad_cores' => '0',
##&nbsp;                         'uncorrected_errors' => '0'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fmm_stats {
    my %args = @_;

    my ($error_message,
        $all,  $i,    $level, $j,    $slot, $var1,
        $var2, $var3, $var4,  $var5, $var6, $bank
    );
    my %hash;
    my $resp         = $args{response_object};
    my $output       = $resp->get_processed_output();
    my $command      = $resp->get_actual_command();
    my @result_array = ();
    my $error        = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );

    if ($command =~ (/(.*-a.*)|(.*card.*)|(.*card.*-S.*)|(.*bank.*)/)) {
        $all = 1;
    }

    for ($i = 0; $i < @lines; $i++) {

        #for option fmm status -a
        if ($all) {
            if (($lines[$i] =~ /Stats information for slot:\s(\d+)/)) {
                $slot = "slot" . $1;
            } elsif ($lines[$i] =~ /Reads:\s+(\d+)/) {
                my $reads = $1;
                if ($command =~ /.*card\s+(\d+).*/) {
                    $slot = "slot" . $1;
                } elsif ($command =~ /.*bank.*(\d+\/\d+\/\d+).*/) {
                    $slot = $1;
                }
                $hash{$slot}{'reads'} = $reads;
            } elsif ($lines[$i] =~ /Read latency \(usecs\):\s+(\d+)/) {
                $hash{$slot}->{'read_latency'} = $1;
            } elsif ($lines[$i] =~ /Writes:\s+(\d+)/) {
                $hash{$slot}->{'writes'} = $1;
            } elsif ($lines[$i] =~ /Write latency \(usecs\):\s+(\d+)/) {
                $hash{$slot}->{'write_latency'} = $1;
            } elsif ($lines[$i] =~ /Erasures:\s+(\d+)/) {
                $hash{$slot}->{'Erasures'} = $1;
            } elsif ($lines[$i] =~ /Erase latency \(usecs\):\s(\d+)/) {
                $hash{$slot}->{'erase_latency'} = $1;
            } elsif ($lines[$i] =~ /Corrected errors:\s+(\d+)/) {
                $hash{$slot}->{'corrected_errors'} = $1;
            } elsif ($lines[$i] =~ /Uncorrected errors:\s+(\d+)/) {
                $hash{$slot}->{'uncorrected_errors'} = $1;
            } elsif ($lines[$i] =~ /Bad cores:\s+(\d+)/) {
                $hash{$slot}->{'bad_cores'} = $1;
            } elsif ($lines[$i] =~ /Bad blocks:\s+(\d+)/) {
                $hash{$slot}->{'bad_blks'} = $1;
            } elsif ($lines[$i] =~ /Capacity online:\s+(\d+).*/) {
                $hash{$slot}->{'capacity_online_%'} = $1;
            } elsif ($lines[$i] =~ /(.*) level stats:/) {
                $level = $1 . "_level_status";
                $level =~ tr/A-Z/a-z/;
                $j = 0;
            } elsif ($lines[$i] =~ /(.*):\s+(\d+)/) {
                $hash{$slot}->{to_key_name(item => $1)} = $2;
            } elsif ($lines[$i]
                =~ /(\S+)\s\|\s+(\d+)\s\|\s+(\d+)\s\|\s+(\d+)\s\|\s+(\d+)\s\|\s+(\d+)/
                ) {
                $var1                                              = $1;
                $var2                                              = $2;
                $var3                                              = $3;
                $var4                                              = $4;
                $var5                                              = $5;
                $var6                                              = $6;
                $hash{$slot}->{$level}->[$j]{'address'}            = $var1;
                $hash{$slot}->{$level}->[$j]{'online'}             = $var2;
                $hash{$slot}->{$level}->[$j]{'corrected_errors'}   = $var3;
                $hash{$slot}->{$level}->[$j]{'uncorrected_errors'} = $var4;
                $hash{$slot}->{$level}->[$j]{'bad_cores'}          = $var5;
                $hash{$slot}->{$level}->[$j]{'bad_blks'}           = $var6;
                $j++;
            } elsif (
                $lines[$i] =~ /(\S+)\s\|\s+(\d+)\s\|\s+(\d+)\s\|\s+(\d+)/) {
                $var1                                    = $1;
                $var2                                    = $2;
                $var3                                    = $3;
                $var4                                    = $4;
                $hash{$slot}->{$level}->[$j]{'address'}  = $var1;
                $hash{$slot}->{$level}->[$j]{'reads'}    = $var2;
                $hash{$slot}->{$level}->[$j]{'writes'}   = $var3;
                $hash{$slot}->{$level}->[$j]{'erasures'} = $var4;
                $j++;
            }
        }
    }
    push(@result_array, \%hash);
    return \@result_array;
} ## end sub fmm_stats

1;
