package NACL::APISet::Node::CLI::7Mode::default::fru_led;

use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::fru_led
##
## @summary Parses the raw output from the "fru_led" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fru_led with any of its options as described in the 
##    corresponding cdef file 'fru_led.cdefs'
## @synopsis
##      fru_led(%args);

## @name ::fru_led
##
## @summary Parses the raw output from the "fru_led" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fru_led with any of its options as described in the 
##    corresponding cdef file 'fru_led.cdefs'
## @synopsis
##      fru_led(%args);
## INPUTS TO METHOD CALL 'fru_led' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'on' </td> <td> 'on' </td> <td> '1' </td> </tr> 
## <tr> <td> 'status' </td> <td> 'status' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'led-index' </td> <td> '0' </td> </tr> 
## <tr> <td> 'off' </td> <td> 'off' </td> <td> '1' </td> </tr> 
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'fru-type' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; fru_led status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Controller slot B: Off
##&nbsp;
##&nbsp;DIMM slot 1:       Off
##&nbsp;DIMM slot 2:       Off
##&nbsp;DIMM slot 3:       Off
##&nbsp;DIMM-NV slot 1:    Off
##&nbsp;PCI slot 1:        Off
##&nbsp;PCI slot 2:        Off
##&nbsp;NV battery 1:      Off
##&nbsp;Boot device 1:     Off
##&nbsp;Coin Cell 1:       Off
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pci_slot_2' => 'Off',
##&nbsp;            'dimm_slot_1' => 'Off',
##&nbsp;            'nv_battery_1' => 'Off',
##&nbsp;            'dimm_slot_2' => 'Off',
##&nbsp;            'boot_device_1' => 'Off',
##&nbsp;            'coin_cell_1' => 'Off',
##&nbsp;            'dimm-nv_slot_1' => 'Off',
##&nbsp;            'pci_slot_1' => 'Off',
##&nbsp;            'controller_slot_b' => 'Off',
##&nbsp;            'dimm_slot_3' => 'Off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; fru_led on dimm 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q test; fru_led on all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub fru_led {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines   = get_lines(
        raw_output      => $output,
        response_object => $resp
    );
    my @result_array;
    if ( $command =~ /status/ ) {
        for ( my $j = 0 ; $j <= $#lines ; $j++ ) {
            my @values = split( ':\s+', $lines[$j] );
			$values[1]=~ tr/)(//d;
            $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
        }
        
        return \@result_array;
    } else {
        return [];
    }
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $result_arrayp = $args{response_object};
    my $error_level   = $result_arrayp->get_execution_status();
    my ( $error, $error_message );
    if ( $raw_output =~ /Unknown.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    elsif( $raw_output =~ /Usage:\s+.*/i|$raw_output=~/Missing.*/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $result_arrayp
        );

    }

}
1;

