#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/get_vol_config_attrs.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode get_vol_config_attrs command output
##
## @description
##    This is a library to parse the raw output of 7Mode get_vol_config_attrs command.
##
## @synopsis
##
##    get_vol_config_attrs(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::get_vol_config_attrs;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::get_vol_config_attrs
##
## @summary Parses the raw output from the "get_vol_config_attrs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_vol_config_attrs with any of its options as described in the
##    corresponding cdef file 'get_vol_config_attrs.cdefs'
## @synopsis
##      get_vol_config_attrs(%args);
## INPUTS TO METHOD CALL 'get_vol_config_attrs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsid' </td> <td> '0' </td> </tr>
## <tr> <td> '-tf' </td> <td> 'total' </td> <td> '1' </td> </tr>
## <tr> <td> '-pid' </td> <td> 'provenance-id' </td> <td> '1' </td> </tr>
## <tr> <td> '-fileshim-offset' </td> <td> 'fileshim-offset' </td> <td> '1' </td> </tr>
## <tr> <td> '-fileshim-suffix' </td> <td> 'fileshim-suffix' </td> <td> '1' </td> </tr>
## <tr> <td> '-nvfail' </td> <td> 'nvfail' </td> <td> '1' </td> </tr>
## <tr> <td> '-in-nvfailed-state' </td> <td> 'in-nvfailed-state' </td> <td> '1' </td> </tr>
## <tr> <td> '-has-i2p' </td> <td> 'has-i2p' </td> <td> '1' </td> </tr>
## <tr> <td> '-xfer-size' </td> <td> 'transfer-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-snapid' </td> <td> 'snapid' </td> <td> '0' </td> </tr>
## <tr> <td> '-mds' </td> <td> 'maxdirsize' </td> <td> '1' </td> </tr>
## <tr> <td> '=' </td> <td> 'match' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get_vol_config_attrs -f 0xd843fded
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Success
##&nbsp;maxdirsize=41861
##&nbsp;maxfiles=93686
##&nbsp;total=31142
##&nbsp;provenance-id=e91dee38-b3c6-11e1-bf50-123478563412
##&nbsp;nvfail=off
##&nbsp;in-nvfailed-state=false
##&nbsp;has-i2p=true
##&nbsp;fileshim-offset=0
##&nbsp;fileshim-suffix=
##&nbsp;transfer size=2773962752
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'transfer_size' => '2773962752',
##&nbsp;            'fileshim_suffix' => '',
##&nbsp;            'has_i2p' => 'true',
##&nbsp;            'fileshim_offset' => '0',
##&nbsp;            'provenance_id' => 'e91dee38-b3c6-11e1-bf50-123478563412',
##&nbsp;            'status' => 'Success',
##&nbsp;            'maxdirsize' => '41861',
##&nbsp;            'total' => '31142',
##&nbsp;            'in_nvfailed_state' => 'false',
##&nbsp;            'maxfiles' => '93686',
##&nbsp;            'nvfail' => 'off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = get_vol_config_attrs -f 0xd843fded -nvfail=off
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Success
##&nbsp;maxdirsize=41861
##&nbsp;maxfiles=93686
##&nbsp;total=31142
##&nbsp;provenance-id=e91dee38-b3c6-11e1-bf50-123478563412
##&nbsp;nvfail=off
##&nbsp;in-nvfailed-state=false
##&nbsp;has-i2p=true
##&nbsp;fileshim-offset=0
##&nbsp;fileshim-suffix=
##&nbsp;transfer size=2773962752
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'transfer_size' => '2773962752',
##&nbsp;            'fileshim_suffix' => '',
##&nbsp;            'has_i2p' => 'true',
##&nbsp;            'fileshim_offset' => '0',
##&nbsp;            'provenance_id' => 'e91dee38-b3c6-11e1-bf50-123478563412',
##&nbsp;            'status' => 'Success',
##&nbsp;            'maxdirsize' => '41861',
##&nbsp;            'total' => '31142',
##&nbsp;            'in_nvfailed_state' => 'false',
##&nbsp;            'maxfiles' => '93686',
##&nbsp;            'nvfail' => 'off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub get_vol_config_attrs {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    $result_array[0]->{status} = $lines[0];
    shift @lines;
    foreach my $line (@lines){
        if ( $line =~ /(.*\S+)[=](\S*)/ ) {
            my $head  = $1;
            my $value = $2;
            $head =~ s/-/_/g;
            $result_array[0]->{ to_key_name( item => $head ) } = $value;
        }
    }
    return \@result_array;
} ## end sub get_vol_config_attrs

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~
/Undefined\s+error:.*|Volume\s+is\s+offline.*|Turn\s+on\s+wafl_paloma_enable\s+using\s+setflag.*/
      )
    {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /No\s+such\s+file\s+or\s+directory.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /usage:.*|Invalid\s+argument.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

