# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode granset command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode granset command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::granset;
##
##    granset_metafile_create(%args);
##    granset_metafile_delete(%args);
##    granset_metafile_dump(%args);
##    granset_metafile_load(%args);
##    granset_create(%args);
##    granset_modify(%args);
##    granset_delete(%args);
##    granset_show(%args);
##    granset_item_add(%args);
##    granset_item_delete(%args);
##    granset_item_is_dp(%args);
##    granset_lookup(%args);
##
## @author roopeshv
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::granset;

use strict;
use warnings;
use English qw($LAST_PAREN_MATCH %LAST_PAREN_MATCH);
use NATE::BaseException ':try';
use NACL::APISet::Response::CLI::ParserUtils
    qw(parser_method_builder get_lines);
use NATE::Log 'log_global';
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

my %method_hash = (
    granset_create => {
        cmd_pass_string      => 'Success',
        output_parser_method => 'granset_create_parser',
    },
    granset_show => {
        cmd_pass_string      => 'Success',
        output_parser_method => 'granset_show_parser',
    },
    granset_item_add => {
        cmd_pass_string => 'Success',
        cmd_fail_string => 'Invalid argument',
    },
    granset_item_is_dp => {
        cmd_pass_string      => 'Success',
        output_parser_method => 'granset_item_is_dp_parser',
    },
    granset_delete      => {cmd_pass_string => 'Success',},
    granset_item_delete => {cmd_pass_string => 'Success'},
    granset_modify      => {
        cmd_pass_string => 'Success',
        cmd_fail_string => 'Invalid argument',
    }
);

# parser_method_builder with custom output parser can still do the error
# validation, and after that error handling is done, it can pass the control
# to output_parser_method. The custom parser can handle the output at this
# point without worrying about the error handling of the command output
__PACKAGE__->parser_method_builder(%method_hash);

sub granset_create_parser {
    $Log->enter() if $may_enter;
    my $params     = shift;
    my %args       = %{$params};
    my $raw_output = $args{response_object}->get_processed_output;
    my %parsed_output;

    if ($raw_output
        =~ /Granset\s+created\s+with\s+GSID\s*:\s*(?P<granset_id>\d+)/xs) {
        $parsed_output{granset_id} = $LAST_PAREN_MATCH{granset_id};
    } else {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ResponseException->throw(
            "cannot find line with *generation id number* in output.\n"
                . $raw_output);
    }

    return \%parsed_output;
}

sub granset_show_parser {
    $Log->enter() if $may_enter;
    my $params        = shift;
    my %args          = %{$params};
    my $raw_output    = $args{response_object}->get_processed_output();
    my @lines         = get_lines(raw_output => $raw_output);
    my %parsed_output = ();
    my @items;
    foreach my $line (@lines) {

        if ($line =~ /Item\s+(?P<item>\d+)\s+:\s+
                  fileid\s+(?P<fileid>\d+)\s+
                  generation\s+(?P<generation>\d+)/x
            ) {
            my %matches = %LAST_PAREN_MATCH;
            push @items, \%matches;
        } elsif ($line =~ /Total number of items: (?P<item_count>\d+)/) {
            $parsed_output{item_count} = $LAST_PAREN_MATCH{item_count};
        } else {
            my ($key, $value) = split(/\s*:\s*/, $line, 2);
            if ($key && $value) {
                $key =~ s/ /_/g; # space->underscore
                $value =~ s/^\s+|\s+$//g; # trim whitespace from beginning & end
                $parsed_output{$key} = $value;
            }
        }
    }

    $parsed_output{items} = \@items;

    $Log->exit() if $may_exit;
    return \%parsed_output;
}

sub granset_item_is_dp_parser {
    $Log->enter() if $may_enter;
    my $params        = shift;
    my %args          = %{$params};
    my $raw_output    = $args{response_object}->get_processed_output();
    my @lines         = get_lines(raw_output => $raw_output);
    my %parsed_output = (granset_type => undef, granset_id => undef);
    if ($raw_output
        =~ /Item\s+belongs\s+to\s+(?P<granset_type>.*)\s+with\s+GSID\s*:\s*(?P<granset_id>\d+)/xs
        ) {
        %parsed_output = %LAST_PAREN_MATCH;
    }

    $Log->exit() if $may_exit;
    return \%parsed_output;
}

1;

##
## @name granset_metafile_create
## @synopsis
##   granset_metafile_create(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
## </table>
##
## @name granset_metafile_delete
## @synopsis
##   granset_metafile_elete(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
## </table>
##
## @name granset_metafile_dump
## @synopsis
##   granset_metafile_dump(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
## </table>
##
## @name granset_metafile_load
## @synopsis
##   granset_metafile_load(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
## </table>
##
## @name granset_create
## @synopsis
##   granset_create(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-u</td><td>gs_uuid</td><td>UUID for granset creation</td></tr>
##   <tr><td>-a</td><td>gs_access</td><td></td></tr>
##   <tr><td>-n</td><td>item_count</td><td></td></tr>
##   <tr><td>-f</td><td>file_ids</td><td>CSV of file inums</td></tr>
##   <tr><td>-g</td><td>granset_ids</td><td>CSV of generation ids for each file</td></tr>
## </table>
## @returns
## nss3220-02a-cm::> system node run -node local -command "priv set -q test; granset create -u 7290551A-C68B-11E4-893D-1D4CD03178EF -a 2 -n 1 -f 96 -g 91350004 testvol; result -l"
## Granset created with GSID : 1
## Success
## $resp = bless( {
##                  'processed_output' => 'Granset created with GSID : 1
## Success
## ',
##                  'execution_time' => '0.12427282333374',
##                  'execution_status' => '0',
##                  'method_name' => 'granset_create',
##                  'actual_command' => 'granset create -u 7290551A-C68B-11E4-893D-1D4CD03178EF -a 2 -n 1 -f 96 -g 91350004 testvol',
##                  'output' => 'Granset created with GSID : 1
## Success
## ',
##                  'cdef_path' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default',
##                  'parser_file' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default/granset.pm',
##                  'parsed_output' => {
##                                       'granset_id' => '1'
##                                     },
##                  'command' => 'priv set -q test; granset create -u 7290551A-C68B-11E4-893D-1D4CD03178EF -a 2 -n 1 -f 96 -g 91350004 testvol; result -l'
##                }, 'NACL::APISet::Response::CLI::7Mode' );
## my $gsid = $resp->get_parsed_output->{granset_id}
##
## @name granset_modify
## @synopsis
##   granset_modify(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-i</td><td>gsid</td><td>generation id of the granset to be modified</td></tr>
##   <tr><td>-a</td><td>gs_access</td><td></td></tr>
##   <tr><td>-s</td><td>gs_state</td><td></td></tr>
##   <tr><td>-r</td><td>gs_redirect_snapid</td><td></td></tr>
## </table>
##
## @name granset_delete
## @synopsis
##   granset_delete(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-i</td><td>gsid</td><td>generation id of the granset to be modified</td></tr>
## </table>
##
## @name granset_show
## @synopsis
##   granset_show(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-i</td><td>gsid</td><td>generation id of the granset to be modified</td></tr>
## </table>
## @returns
##  nss3220-02a-cm::> system node run -node local -command "priv set -q test; granset show -i 1 testvol; result -l"
##  granset id: 1 uuid: 7290551a-c68b-11e4-893d-1d4cd03178ef type: 2 state: 1 redirect snapid: 0
##  Item 1 : fileid 96 generation 91350004
##
##  Total number of items: 1
##  Success
##  result=0
##
##  $resp = $Apiset->granset_show(%args);
##  $resp = bless( {
##                   'processed_output' => 'granset id: 1 uuid: 7290551a-c68b-11e4-893d-1d4cd03178ef type: 2 state: 1 redirect snapid: 0
##  Item 1 : fileid 96 generation 91350004
##
##  Total number of items: 1
##  Success
##  ',
##                   'execution_time' => '0.118458032608032',
##                   'execution_status' => '0',
##                   'method_name' => 'granset_show',
##                   'actual_command' => 'granset show -i 1 testvol',
##                   'output' => 'granset id: 1 uuid: 7290551a-c68b-11e4-893d-1d4cd03178ef type: 2 state: 1 redirect snapid: 0
##  Item 1 : fileid 96 generation 91350004
##
##  Total number of items: 1
##  Success
##  ',
##                   'cdef_path' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default',
##                   'parser_file' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default/granset.pm',
##                   'parsed_output' => {
##                                        'item_count' => '1',
##                                        'redirect_snapid' => '0',
##                                        'type' => '2',
##                                        'uuid' => '7290551a-c68b-11e4-893d-1d4cd03178ef',
##                                        'granset_id' => '1',
##                                        'items' => [
##                                                     {
##                                                       'generation' => '91350004',
##                                                       'fileid' => '96',
##                                                       'item' => '1'
##                                                     }
##                                                   ],
##                                        'state' => '1'
##                                      },
##                   'command' => 'priv set -q test; granset show -i 1 testvol; result -l'
##                 }, 'NACL::APISet::Response::CLI::7Mode' );
##
## @name granset_item_add
## @synopsis
##   granset_item_add(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-i</td><td>gsid</td><td>generation id of the granset to be modified</td></tr>
##   <tr><td>-n</td><td>item_count</td><td></td></tr>
##   <tr><td>-f</td><td>file_ids or fileid</td><td>choose either name for the param</td></tr>
##   <tr><td>-g</td><td>generation_ids or generation</td><td></td></tr>
## </table>
## @returns
## $resp = $Apiset->granset_item_add(%args);
##
## system node run -node local -command "priv set -q test; granset item add -i 1 -n 1 -f 97 -g 91350192 testvol; result -l"
## Success
## result=0
##
## nss3220-02a-cm::>
## $resp = bless( {
##                  'processed_output' => 'Success
## ',
##                  'execution_time' => '0.126044034957886',
##                  'execution_status' => '0',
##                  'method_name' => 'granset_item_add',
##                  'actual_command' => 'granset item add -i 1 -n 1 -f 97 -g 91350192 testvol',
##                  'output' => 'Success
## ',
##                  'cdef_path' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default',
##                  'parser_file' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default/granset.pm',
##                  'parsed_output' => {},
##                  'command' => 'priv set -q test; granset item add -i 1 -n 1 -f 97 -g 91350192 testvol; result -l'
##                }, 'NACL::APISet::Response::CLI::7Mode' );
##
## @name granset_item_delete
## @synopsis
##   granset_item_delete(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-i</td><td>gsid</td><td>generation id of the granset to be modified</td></tr>
##   <tr><td>-n</td><td>item_count</td><td></td></tr>
##   <tr><td>-f</td><td>file_ids or fileid</td><td>choose either name for the param</td></tr>
##   <tr><td>-g</td><td>generation_ids or generation</td><td></td></tr>
## </table>
##
## @name granset_item_is_dp
## @synopsis
##   granset_item_is_dp(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-f</td><td>fileid</td><td></td></tr>
##   <tr><td>-g</td><td>generation</td><td></td></tr>
## </table>
## @returns
## $resp = $Apiset->granset_item_is_dp(%args);
##
## system node run -node local -command "priv set -q test; granset item is_dp -f 96 -g 91350004 testvol; result -l"
## Item belongs to non-DP GranSet with GSID: 1
## Success
## result=0
##
## nss3220-02a-cm::>
## $resp = bless( {
##                  'processed_output' => 'Item belongs to non-DP GranSet with GSID: 1
## Success
## ',
##                  'execution_time' => '0.124003887176514',
##                  'execution_status' => '0',
##                  'method_name' => 'granset_item_is_dp',
##                  'actual_command' => 'granset item is_dp -f 96 -g 91350004 testvol',
##                  'output' => 'Item belongs to non-DP GranSet with GSID: 1
## Success
## ',
##                  'cdef_path' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default',
##                  'parser_file' => '/x/eng/bbsvl/users/roopeshv/dev_granset_libs_2743521_1503061629/test/lib/NACL/APISet/Node/CLI/7Mode/default/granset.pm',
##                  'parsed_output' => {
##                                       'granset_type' => 'non-DP GranSet',
##                                       'granset_id' => '1'
##                                     },
##                  'command' => 'priv set -q test; granset item is_dp -f 96 -g 91350004 testvol; result -l'
##                }, 'NACL::APISet::Response::CLI::7Mode' );
##
## @name granset_lookup
## @synopsis
##   granset_lookup(%args)
## The command takes the following inputs
## <table border=1>
##   <tr><th>OPTION</th><th>PROGRAMMATIC NAME</th><th>Description</th></tr>
##   <tr><td>No switch for this key</td><td>volume</td><td>Name of the volume</td></tr>
##   <tr><td>-u</td><td>gs_uuid</td><td>UUID of the granset</td></tr>
## </table>
