#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ha_config commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ha_config commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ha_config show" will be ha_config_show()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::ha_config;
##
##    ha_config_modify(%args);
##    ha_config_show(%args);
##
## @author pbala@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::7Mode::default::ha_config;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $error_message;
    my $error;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /[Uu]sage:/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::ha_config_modify
##
## @summary Parses the raw output from the "ha-config modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ha_config_modify with any of its options as described in the
##    corresponding cdef file 'ha_config.cdefs'
## @synopsis
##      ha_config_modify(%args);
## INPUTS TO METHOD CALL 'ha_config_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'controller' </td> <td> 'controller' </td> <td> '1' </td> </tr>
## <tr> <td> 'non-ha' </td> <td> 'non-ha' </td> <td> '1' </td> </tr>
## <tr> <td> 'ha' </td> <td> 'ha' </td> <td> '1' </td> </tr>
## <tr> <td> 'default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## <tr> <td> 'chassis' </td> <td> 'chassis' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ha-config modify controller ha; result -l;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Controller configuration will change from ha to ha.
##&nbsp;Warning: Incorrect configuration may lead to data loss. Do you want to continue?? [no]
##&nbsp;y
##&nbsp;Controller FRU PROM successfully modified to ha
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub ha_config_modify {
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    # If there is no error, then it means that the command had
    # executed successfully
    return [];
}

## @name ::ha_config_show
##
## @summary Parses the raw output from the "ha-config show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ha_config_show with any of its options as described in the
##    corresponding cdef file 'ha_config.cdefs'
## @synopsis
##      ha_config_show(%args);
## INPUTS TO METHOD CALL 'ha_config_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ha-config show; result -l;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Chassis HA configuration: ha
##&nbsp;Controller HA configuration: ha
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'controller_ha_configuration' => 'ha',
##&nbsp;            'chassis_ha_configuration' => 'ha'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub ha_config_show {
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );
    my @result_array;
    my @lines = get_lines( 'raw_output' => $raw_output, "response_object" => $response );
    foreach my $line (@lines) {
        if ( $line =~ /(Controller.*|Chassis.*):\s(.*)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
}
1;
