#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode hammer commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode hammer commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "hammer status" will be hammer_status()
##
## @synopsis
##    use hammer;
##
##    hammer_status(%args);
##    hammer(%args);
##    hammer_abort(%args);
##    hammer_pause(%args);
##    hammer_restart(%args);
##
## @author udaykk@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::hammer;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;


## @name ::hammer_status
##
## @summary Parses the raw output from the "hammer status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from hammer_status with any of its options as described in the
##    corresponding cdef file 'hammer.cdefs'
## @synopsis
##      hammer_status(%args);
## INPUTS TO METHOD CALL 'hammer_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = hammer status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'size_written' => '400 KB',
##&nbsp;            'iterations_done' => '5432',
##&nbsp;            'file_name' => '/vol/vol0/hammer1',
##&nbsp;            'status' => 'Writing',
##&nbsp;            'total_iterations' => '-1',
##&nbsp;            'write_size' => '4096 bytes',
##&nbsp;            'total_size' => '400 KB',
##&nbsp;            'hammer_id' => 'blacksmith #0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub hammer_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @output =
      get_lines( "raw_output" => $output, "response_object" => $resp );

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "hammer status",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @hammer;
    my $value;
    foreach $value (@output) {
        my %info    = ();

# Below regex is matched with a sample string above mentioned
# blacksmith #0: file /vol/vol1/hammer1, 13572 KB of 40000 KB writesize 4096 bytes,
# iteration 9 of -1 - Writing
        if (
            $value =~ /(\w+\s\#\d)\:\s+\w+\s+(\/\w+\/\w+\/\w+)\,\s+(\d+\s[K|k][B|b])\s+\w+\s+(\d+\s[K|k][B|b])\s+\w+\s+(\d+\s+\w+)\,\s+\w+\s(\d+)\s+\w+\s+([-]?\d+)\s+\-\s+(\w+)/
          ) {
	    $info{"hammer_id"}        = $1;
            $info{"file_name"}        = $2;
            $info{"size_written"}     = $3;
            $info{"total_size"}       = $4;
            $info{"write_size"}       = $5;
            $info{"iterations_done"}  = $6;
            $info{"total_iterations"} = $7;
            $info{"status"}           = $8;
            push( @hammer, \%info );
	}
    }
    return \@hammer;
}


## @name ::hammer
##
## @summary Parses the raw output from the "hammer" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from hammer with any of its options as described in the
##    corresponding cdef file 'hammer.cdefs'
## @synopsis
##      hammer(%args);
## INPUTS TO METHOD CALL 'hammer' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-runs' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-blocks' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = hammer -f -1 /vol/vol0/hammer1 100
##&nbsp;$VAR1 = [];
##&nbsp;
sub hammer {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "hammer",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}


## @name ::hammer_abort
##
## @summary Parses the raw output from the "hammer abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from hammer_abort with any of its options as described in the
##    corresponding cdef file 'hammer.cdefs'
## @synopsis
##      hammer_abort(%args);
## INPUTS TO METHOD CALL 'hammer_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = hammer abort
##&nbsp;$VAR1 = [];
##&nbsp;
sub hammer_abort {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "hammer abort",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::hammer_pause
##
## @summary Parses the raw output from the "hammer pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from hammer_pause with any of its options as described in the
##    corresponding cdef file 'hammer.cdefs'
## @synopsis
##      hammer_pause(%args);
## INPUTS TO METHOD CALL 'hammer_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = hammer pause
##&nbsp;$VAR1 = [];
##&nbsp;
sub hammer_pause {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "hammer pause",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::hammer_restart
##
## @summary Parses the raw output from the "hammer restart" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from hammer_restart with any of its options as described in the
##    corresponding cdef file 'hammer.cdefs'
## @synopsis
##      hammer_restart(%args);
## INPUTS TO METHOD CALL 'hammer_restart' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = hammer restart
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_name' => '/vol/vol0/hammer1',
##&nbsp;            'num_blocks' => '100',
##&nbsp;            'num_runs' => '4294967295',
##&nbsp;            'hammer_id' => 'blacksmith #0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub hammer_restart {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @hammer;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "hammer restart",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $value;
    my @arr = get_lines("raw_output"=>$output, "response_object"=>$resp);
        foreach $value (@arr)  {
            my %info    = ();
            if($value =~ /(\w+\s\#\d)\:\s+\w+\s+(\d+)\s+\w+\s+(\/\w+\/\w+\/\w+)\s+\w+\s+(\d+)/) {
                 $info{"hammer_id"}        = $1;
                 $info{"num_runs"}         = $2;
                 $info{"file_name"}        = $3;
                 $info{"num_blocks"}       = $4;
                 push( @hammer,\%info );
            }
        }
    return \@hammer;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /usage/i ) {
        $error          = "InvalidParam";
        $$error_message = "command failed due to wrong dependicies usage";
    }
    return $error;
}
1;
