#
# Copyright (c) 2012-13 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode ic command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ic commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ic" will be ic stats error()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::ic;
##
## ic_stats_error(%args);
## ic_status(%args);
## ic_stats_performance(%args);
## ic_stats_histogram(%args);
## ic_dumpq(%args);
## ic_nvthruput(%args);
## ic_dump_hw_stats(%args);
## ic_ifthruput(%args);
## ic_reset(%args);
## ic_config_show(%args);
## ic_link(%args);
## ic_ood_stats(%args);
## ic_ood_stats_error_show(%args);
## ic_ood_stats_performance_show(%args);
## ic_ood_diag_send(%args);
## ic_flint(%args);
## ic_ood_status(%args);
##
##
## @author Akshay Mohit(makshay@netapp.com), rahula@netapp.com, tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::ic;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /([Uu]sage:\s)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(Invalid channel.*)/i ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /(Missing required.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /(Throughput.*)/i ) {
        $$error_message = $1;
        $error          = "CommandNotFound";
    } elsif ( $raw_output =~ /(The diag option is not valid.*)/i ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(The command is only valid.*)/i ) {
        $$error_message = $1;
        $error          = "CommandNotFound";
    } elsif ( $raw_output =~ /(Invalid Transfer.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /(HA interconnect not found.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    return $error;
} ## end sub _check_error

## @name ::ic_stats_error
##
## @summary Parses the raw output from the "ic stats error" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_stats_error with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_stats_error(%args);
## INPUTS TO METHOD CALL 'ic_stats_error' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ic stats error -v; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_local_wq_invalid_requests' => '0',
##&nbsp;            'number_of_ib_events' => '5',
##&nbsp;            'h/w_error_stats' => [
##&nbsp;                                   {
##&nbsp;                                     'number_of_remote_invalid_rd_request_errors' => '0',
##&nbsp;                                     'number_of_uc_discarded_packets' => '0',
##&nbsp;                                     'number_of_rnr_nak_retries_exceeded_errors' => '0',
##&nbsp;                                     'requester_number_of_wr_flushed_errors' => '0',
##&nbsp;                                     'excessive_buffer_overrun_error' => '0',
##&nbsp;                                     'responder_number_of_remote_access_errors' => '0',
##&nbsp;                                     'responder_number_of_remote_invalid_req_errors' => '0',
##&nbsp;                                     'responder_number_of_local_length_errors' => '0',
##&nbsp;                                     'link_error_recovery_counter' => '0',
##&nbsp;                                     'responder_number_of_local_protection_errors' => '0',
##&nbsp;                                     'number_of_bad_doorbells' => '0',
##&nbsp;                                     'local_link_integrity_error' => '0',
##&nbsp;                                     'responder_number_of_resync_operations' => '0',
##&nbsp;                                     'requester_number_of_resync_operations' => '0',
##&nbsp;                                     'responder_number_local_ee_operation_error' => '0',
##&nbsp;                                     'requester_number_of_memory_window_bind_errors' => '0',
##&nbsp;                                     'requester_number_of_local_protection_errors' => '0',
##&nbsp;                                     'port_transmit_constraint_error' => '0',
##&nbsp;                                     'number_of_invalid_ee_context_errors' => '0',
##&nbsp;                                     'requester_number_of_remote_operation_errors' => '0',
##&nbsp;                                     'number_of_invalid_ee_context_state_errors' => '0',
##&nbsp;                                     'port_transmit_packets' => '6963731',
##&nbsp;                                     'vl_15_dropped' => '0',
##&nbsp;                                     'port_receive_error' => '0',
##&nbsp;                                     'link_downed_counter' => '0',
##&nbsp;                                     'responder_number_of_wr_flushed_errors' => '0',
##&nbsp;                                     'port_transmit_data' => '1065345489',
##&nbsp;                                     'number_of_out_of_sequence_requests_received' => '0',
##&nbsp;                                     'requester_number_local_qp_operation_error' => '0',
##&nbsp;                                     'port_receive_packets' => '6328824',
##&nbsp;                                     'number_of_eq_overflows' => '0',
##&nbsp;                                     'number_of_cq_overflows' => '0',
##&nbsp;                                     'number_of_transport_retries_exceeded_errors' => '0',
##&nbsp;                                     'requester_number_local_ee_operation_error' => '0',
##&nbsp;                                     'requester_number_of_remote_invalid_req_errors' => '0',
##&nbsp;                                     'port_transmit_discard' => '0',
##&nbsp;                                     'responder_number_of_remote_operation_errors' => '0',
##&nbsp;                                     'port_receive_switch_relay_error' => '0',
##&nbsp;                                     'port_receive_data' => '875149810',
##&nbsp;                                     'responder_number_of_local_access_errors' => '0',
##&nbsp;                                     'requester_number_of_remote_access_errors' => '0',
##&nbsp;                                     'number_of_local_rdd_violation_errors' => '0',
##&nbsp;                                     'responder_number_of_local_qp_operation_errors' => '0',
##&nbsp;                                     'port_receive_constraint_error' => '0',
##&nbsp;                                     'number_of_out_of_sequence_naks_received' => '0',
##&nbsp;                                     'requester_number_of_local_length_errors' => '0',
##&nbsp;                                     'number_of_remote_aborted_errors' => '0',
##&nbsp;                                     'requester_number_of_bad_response_errors' => '0',
##&nbsp;                                     'number_of_bad_multicast_packets_received' => '0',
##&nbsp;                                     'port_receive_remote_physical_error' => '0',
##&nbsp;                                     'number_of_ud_discarded_packets' => '9',
##&nbsp;                                     'symbol_error_counter' => '0'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'number_of_nic_resets' => '2',
##&nbsp;            'number_of_mad_recv' => '0x7',
##&nbsp;            'destroy_cq_failures_' => '0',
##&nbsp;            'broken_cable_or_gbic_' => '0',
##&nbsp;            'number_of_mlnx_events' => '0',
##&nbsp;            'enable_ofw_status_reads_' => 'TRUE FALSE',
##&nbsp;            'enable_nic_reset_for_unsynced_logs_' => 'TRUE',
##&nbsp;            'destroy_vi_failures_' => '0',
##&nbsp;            'max_xig_threshold' => '12',
##&nbsp;            'peer_connection_failures' => '0',
##&nbsp;            'port_mtu' => '0x3',
##&nbsp;            'vi_errors_reported_to_rv_' => '0',
##&nbsp;            'number_of_catastrophic_error_events' => '0',
##&nbsp;            'operator_issued_nic_resets_' => '0',
##&nbsp;            'number_of_local_wq_catastropical_error' => '0',
##&nbsp;            'number_of_fatal_master_abort_on_ecr_[0,1]' => '0',
##&nbsp;            'memory_registration_failures_' => '0',
##&nbsp;            'descriptor_allocation_failures_' => '0',
##&nbsp;            'number_of_times_switched_nics_' => '0',
##&nbsp;            'h/w_performance_stats' => [
##&nbsp;                                         {
##&nbsp;                                           'number_of_tpt_cache_access' => '51180359'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;            'vi_max_connection_time_' => '239 msec',
##&nbsp;            'kstat_receive_time_outs' => '0',
##&nbsp;            'rv_connection_attempts_' => '3',
##&nbsp;            'number_of_interrupts' => '4965789',
##&nbsp;            'number_of_send_queue_drained' => '0',
##&nbsp;            'connection_setup_failures_' => '0',
##&nbsp;            'number_of_path_migrations_failed' => '0',
##&nbsp;            'nic_resets_for_unsynced_log_' => '0',
##&nbsp;            'number_of_local_catastrophic_errors' => '0',
##&nbsp;            'number_of_communication_established' => '0',
##&nbsp;            'number_of_fatal_errors' => '0',
##&nbsp;            'number_of_local_wq_access_violations' => '0',
##&nbsp;            'rv_max_connection_time_' => '239 msec',
##&nbsp;            'port_state' => 'PORT ACTIVE',
##&nbsp;            'cfo_rv_connection_state_' => 'CONNECTED',
##&nbsp;            'descriptor_xid_mismatches_' => '0',
##&nbsp;            'rv_avg_connection_time_' => '225 msec',
##&nbsp;            'nic_resets_for_rv_peer-connect_timeout_' => '0',
##&nbsp;            'create_cq_failures_' => '0',
##&nbsp;            'number_of_command_if_events' => '389',
##&nbsp;            'number_of_mad_sent' => '0x9',
##&nbsp;            'number_of_recv_desc' => '0x1',
##&nbsp;            'receive_descriptor_errors_' => '0',
##&nbsp;            'rv_connections_made_' => '2',
##&nbsp;            'cfo_rv_nic_used_' => '0',
##&nbsp;            'send_descriptor_timeouts_' => '0',
##&nbsp;            'create_vi_failures_' => '0',
##&nbsp;            'operator_issued_link_resets_' => '0',
##&nbsp;            'number_of_local_eec_catastrophic_errors' => '0',
##&nbsp;            'number_of_port_errors' => '5',
##&nbsp;            'software_resets_' => '2',
##&nbsp;            'send_descriptor_errors_' => '0',
##&nbsp;            'link_state' => 'UP',
##&nbsp;            'number_of_path_migrations' => '0',
##&nbsp;            'number_of_overrun\'s' => '0',
##&nbsp;            'number_of_cq_completion_events' => '156166',
##&nbsp;            'rv_connections_lost' => '2',
##&nbsp;            'confirmed_delivery_timeouts_' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; ic stats error; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_interrupts' => '4965843',
##&nbsp;            'number_of_local_wq_invalid_requests' => '0',
##&nbsp;            'number_of_send_queue_drained' => '0',
##&nbsp;            'number_of_ib_events' => '5',
##&nbsp;            'number_of_path_migrations_failed' => '0',
##&nbsp;            'number_of_local_catastrophic_errors' => '0',
##&nbsp;            'number_of_communication_established' => '0',
##&nbsp;            'number_of_fatal_errors' => '0',
##&nbsp;            'number_of_nic_resets' => '2',
##&nbsp;            'number_of_local_wq_access_violations' => '0',
##&nbsp;            'number_of_mlnx_events' => '0',
##&nbsp;            'cfo_rv_connection_state_' => 'CONNECTED',
##&nbsp;            'number_of_command_if_events' => '392',
##&nbsp;            'number_of_catastrophic_error_events' => '0',
##&nbsp;            'number_of_local_wq_catastropical_error' => '0',
##&nbsp;            'number_of_fatal_master_abort_on_ecr_[0,1]' => '0',
##&nbsp;            'cfo_rv_nic_used_' => '0',
##&nbsp;            'rv_connections_made_' => '2',
##&nbsp;            'number_of_local_eec_catastrophic_errors' => '0',
##&nbsp;            'number_of_port_errors' => '5',
##&nbsp;            'software_resets_' => '2',
##&nbsp;            'link_state' => 'UP',
##&nbsp;            'number_of_times_switched_nics_' => '0',
##&nbsp;            'number_of_path_migrations' => '0',
##&nbsp;            'number_of_cq_completion_events' => '156167',
##&nbsp;            'number_of_overrun\'s' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_stats_error {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _ic_stats_error(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
}    # end sub ic_stats_error

sub _ic_stats_error {
    my %args = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'ic stats error', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );
    my $flag = 0;
    my ( $head1, $head2, $value, $value2 );

    foreach my $line (@arr) {
        if ( $line =~ m/.*(H\/W Error Stats).*/ ) {
            $flag = 1;
            $head1 = to_key_name( item => $1 );
            next;
        } elsif ( $line =~ m/.*(H\/W Performance Stats).*/ ) {
            $flag = 2;
            $head2 = to_key_name( item => $1 );
            next;
        }
        if ( $flag == 1 ) {
            if ( $line =~ m/(.*):\s+(.*)/ ) {
                $result_array[0]->{$head1}->[0]->{ to_key_name( item => $1 ) }
                    = $2;
            }
        } elsif ( $flag == 2 ) {
            if ( $line =~ m/(.*):\s+(.*)/ ) {
                $result_array[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }
                    = $2;
            }
        } else {
            if ( $line =~ m/(.*):\s+(.*)/ ) {
                $value  = $1;
                $value2 = $2;
                $value =~ s/\s$//;
                $result_array[0]->{ to_key_name( item => $value ) } = $value2;
            }
        }
    } ## end foreach my $line (@arr)

    return \@result_array;
} ## end sub _ic_stats_error

## @name ::ic_status
##
## @summary Parses the raw output from the "ic status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_status with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_status(%args);
## INPUTS TO METHOD CALL 'ic_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ic status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Link 0: up
##&nbsp;Link 1: up
##&nbsp;cfo_rv connection state :                       CONNECTED
##&nbsp;cfo_rv nic used :                               0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'link_1' => 'up',
##&nbsp;             'cfo_rv_nic_used' => '0',
##&nbsp;             'link_0' => 'up',
##&nbsp;             'cfo_rv_connection_state' => 'CONNECTED'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub ic_status {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $i = 0;
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines
        = get_lines( raw_output => $output, response_object => $resp );
    foreach my $lines (@arraylines) {
        if ( $lines =~ /(.*):\s+(\S+\s+\S+|\S+)/ ) {
            my $key = $1;
            $key = trim_string( 'string' => $key );
            $result_array[$i]->{ to_key_name( item => $key ) } = $2;
        }
    }
    return \@result_array;
}


## @name ::ic_stats_performance
##
## @summary Parses the raw output from the "ic stats performance" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_stats_performance with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_stats_performance(%args);
## INPUTS TO METHOD CALL 'ic_stats_performance' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic stats performance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Elapsed time : 783 secs
##&nbsp;ic_qmax_waits: 56
##&nbsp;avg qmax_wait time: 0 usecs
##&nbsp;NV connection failover time: 12049 msecs
##&nbsp;ic_preempt_timeout: 0
##&nbsp;ic_non_preempt_timeout: 0
##&nbsp;ic_notify_timeout: 0
##&nbsp;Total WAFL Data IOs: 2565482
##&nbsp;Total WAFL MetaData IOs: 1421361
##&nbsp;Total RAID Data IOs: 71654
##&nbsp;Total RAID MetaData IOs: 54361
##&nbsp;Total MISC Data IOs: 5
##&nbsp;Total MISC MetaData IOs: 9140
##&nbsp;ic_waits : 437
##&nbsp;ic_isdones: 5082
##&nbsp;ic_isdones done: 5070
##&nbsp;ic_isdones not-done: 12
##&nbsp;RDMA reads: 792
##&nbsp;Confirm delivery poll counter: 0
##&nbsp;Descriptor Transfer Id Queue 0 count: 0
##&nbsp;Descriptor Transfer Id Queue 1 count: 0
##&nbsp;Descriptor Transfer Id Queue 2 count: 0
##&nbsp;avg ic_waitdone RDMA-READ time: 13 us
##&nbsp;avg ic_waitdone time : 23 us
##&nbsp;avg time between rnv msgs: 73935 us
##&nbsp;rnv transfers: 31075
##&nbsp;avg rnv transfer size: 622
##&nbsp;avg time between rnv transfers: 25198 us
##&nbsp;rnv msgs done: 10584
##&nbsp;rnv queue total waittime: 256439 us
##&nbsp;rnv completion wakeups: 31
##&nbsp;rnv completion waitdone: 2
##&nbsp;Avg MB/s : 0.1164
##&nbsp;avg bytes per xfer: 432
##&nbsp;total xfers: 70047
##&nbsp;avg nv vi q lengths: 0 0 2
##&nbsp;Completion Intr rate: 0 per sec
##&nbsp;nvlog Avg time to sync: 577 msec
##&nbsp;nvlog Max time to sync: 630 msec
##&nbsp;ic_small_writes: 33746
##&nbsp;ic_4k_writes: 613
##&nbsp;ic_8k_writes: 159
##&nbsp;ic_16K+_writes: 467
##&nbsp;SG entries 1: Total count 9314, Total size 3093280
##&nbsp;SG entries 2: Total count 404, Total size 205962
##&nbsp;SG entries 8: Total count 3072, Total size 12577688
##&nbsp;ic_recvq_waits: 0
##&nbsp;VI IF statistics
##&nbsp;================
##&nbsp;Channel 0:
##&nbsp;==========
##&nbsp;Send credit : 126
##&nbsp;Send Queue length : 0
##&nbsp;Send Queue full : 22867
##&nbsp;Sent Queue length : 0
##&nbsp;Sent data : 8192 kbytes
##&nbsp;Receive Queue length : 128
##&nbsp;Received data : 8192 kbytes
##&nbsp;Explicit credit updates received: 16510
##&nbsp;Explicit credit updates sent: 127
##&nbsp;Piggyback credit updates received: 0
##&nbsp;Piggyback credit updates sent: 0
##&nbsp;Sent mbufs: 16384
##&nbsp;Recv mbufs freed by vi_if: 130
##&nbsp;Recv mbufs freed by client: 0
##&nbsp;Send fails: 15
##&nbsp;Channel 1:
##&nbsp;==========
##&nbsp;Send credit : 125
##&nbsp;Send Queue length : 0
##&nbsp;Send Queue full : 0
##&nbsp;Sent Queue length : 0
##&nbsp;Sent data : 5829 kbytes
##&nbsp;Receive Queue length : 128
##&nbsp;Received data : 5829 kbytes
##&nbsp;Explicit credit updates received: 127
##&nbsp;Explicit credit updates sent: 127
##&nbsp;Piggyback credit updates received: 6738
##&nbsp;Piggyback credit updates sent: 6740
##&nbsp;Sent mbufs: 6740
##&nbsp;Recv mbufs freed by vi_if: 6741
##&nbsp;Recv mbufs freed by client: 0
##&nbsp;Send fails: 15
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ic_16k+_writes' => '467',
##&nbsp;            'ic_isdones_done' => '5070',
##&nbsp;            'ic_non_preempt_timeout' => '0',
##&nbsp;            'ic_small_writes' => '33746',
##&nbsp;            'ic_preempt_timeout' => '0',
##&nbsp;            'ic_notify_timeout' => '0',
##&nbsp;            'avg_bytes_per_xfer' => '432',
##&nbsp;            'ic_isdones_not-done' => '12',
##&nbsp;            'ic_waits' => '437',
##&nbsp;            'nvlog_avg_time_to_sync' => '577 msec',
##&nbsp;            'total_raid_data_ios' => '71654',
##&nbsp;            'ic_isdones' => '5082',
##&nbsp;            'nv_connection_failover_time' => '12049 msecs',
##&nbsp;            'channel0_info' => {
##&nbsp;                                 'sent_mbufs' => '16384',
##&nbsp;                                 'send_fails' => '15',
##&nbsp;                                 'recv_mbufs_freed_by_client' => '0',
##&nbsp;                                 'recv_mbufs_freed_by_vi_if' => '130',
##&nbsp;                                 'received_data' => '8192 kbytes',
##&nbsp;                                 'sent_data' => '8192 kbytes',
##&nbsp;                                 'receive_queue_length' => '128',
##&nbsp;                                 'send_queue_length' => '0',
##&nbsp;                                 'channel_number' => '0',
##&nbsp;                                 'sent_queue_length' => '0',
##&nbsp;                                 'piggyback_credit_updates_received' => '0',
##&nbsp;                                 'piggyback_credit_updates_sent' => '0',
##&nbsp;                                 'send_credit' => '126',
##&nbsp;                                 'explicit_credit_updates_received' => '16510',
##&nbsp;                                 'explicit_credit_updates_sent' => '127',
##&nbsp;                                 'send_queue_full' => '22867'
##&nbsp;                               },
##&nbsp;            'descriptor_transfer_id_queue_0_count' => '0',
##&nbsp;            'rnv_completion_waitdone' => '2',
##&nbsp;            'rnv_completion_wakeups' => '31',
##&nbsp;            'total_raid_metadata_ios' => '54361',
##&nbsp;            'nvlog_max_time_to_sync' => '630 msec',
##&nbsp;            'avg_nv_vi_q_lengths' => '0 0 2',
##&nbsp;            'avg_ic_waitdone_time' => '23 us',
##&nbsp;            'avg_mb/s' => '0.1164',
##&nbsp;            'channel1_info' => {
##&nbsp;                                 'sent_mbufs' => '6740',
##&nbsp;                                 'send_fails' => '15',
##&nbsp;                                 'recv_mbufs_freed_by_client' => '0',
##&nbsp;                                 'recv_mbufs_freed_by_vi_if' => '6741',
##&nbsp;                                 'received_data' => '5829 kbytes',
##&nbsp;                                 'sent_data' => '5829 kbytes',
##&nbsp;                                 'receive_queue_length' => '128',
##&nbsp;                                 'send_queue_length' => '0',
##&nbsp;                                 'channel_number' => '1',
##&nbsp;                                 'sent_queue_length' => '0',
##&nbsp;                                 'piggyback_credit_updates_received' => '6738',
##&nbsp;                                 'piggyback_credit_updates_sent' => '6740',
##&nbsp;                                 'send_credit' => '125',
##&nbsp;                                 'explicit_credit_updates_received' => '127',
##&nbsp;                                 'explicit_credit_updates_sent' => '127',
##&nbsp;                                 'send_queue_full' => '0'
##&nbsp;                               },
##&nbsp;            'ic_recvq_waits' => '0',
##&nbsp;            'avg_time_between_rnv_transfers' => '25198 us',
##&nbsp;            'avg_ic_waitdone_rdma-read_time' => '13 us',
##&nbsp;            'descriptor_transfer_id_queue_1_count' => '0',
##&nbsp;            'total_misc_metadata_ios' => '9140',
##&nbsp;            'rnv_msgs_done' => '10584',
##&nbsp;            'completion_intr_rate' => '0 per sec',
##&nbsp;            'total_xfers' => '70047',
##&nbsp;            'rnv_queue_total_waittime' => '256439 us',
##&nbsp;            'avg_qmax_wait_time' => '0 usecs',
##&nbsp;            'confirm_delivery_poll_counter' => '0',
##&nbsp;            'descriptor_transfer_id_queue_2_count' => '0',
##&nbsp;            'avg_rnv_transfer_size' => '622',
##&nbsp;            'ic_4k_writes' => '613',
##&nbsp;            'rdma_reads' => '792',
##&nbsp;            'total_wafl_metadata_ios' => '1421361',
##&nbsp;            'elapsed_time' => '783 secs',
##&nbsp;            'total_wafl_data_ios' => '2565482',
##&nbsp;            'ic_qmax_waits' => '56',
##&nbsp;            'avg_time_between_rnv_msgs' => '73935 us',
##&nbsp;            'rnv_transfers' => '31075',
##&nbsp;            'ic_8k_writes' => '159',
##&nbsp;            'total_misc_data_ios' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_stats_performance {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    while (my $line = shift @lines ) {
        if ($line =~ /Channel\s+(\d+)/){
            my %hash;
            my $key = join('_','channel'.$1,'info');
            $hash{'channel_number'} = $1;
            while ($lines[0]!~/Channel\s+\d+/ and ( scalar (@lines) > 0)) {
                if ($lines[0] =~ /^(\w+.*)\:\s+(\d+.*)/){
                    my $chankey = trim_string ('string' => $1);
                    my $chanval = $2;
                    $hash{to_key_name(item => $chankey)} = $chanval;
                }
                shift @lines;
            }
            $result_array[0]->{$key}= \%hash;
        } elsif ($line =~ /^(\w+.*)\:\s+(\d+.*)/){
            my $key = trim_string('string' => $1);
            my $value = $2;
            $result_array[0]->{to_key_name(item => $key)} = $value;
        }
    }
    return \@result_array;
}

## @name ::ic_stats_histogram
##
## @summary Parses the raw output from the "ic stats histogram" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_stats_histogram with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_stats_histogram(%args);
## INPUTS TO METHOD CALL 'ic_stats_histogram' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ic_stats_histogram {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::ic_dumpq
##
## @summary Parses the raw output from the "ic dumpq" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_dumpq with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_dumpq(%args);
## INPUTS TO METHOD CALL 'ic_dumpq' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vi-num' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic dumpq 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;No such QP handle
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'No such QP handle'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_dumpq {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my (@result_array);
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line =~ /.*/ ) {
            push( @result_array, { status => $line } );
            last;
        }
    }
    return \@result_array;
}

# @name ::ic_nvthruput
##
## @summary Parses the raw output from the "ic nvthruput" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_nvthruput with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_nvthruput(%args);
## INPUTS TO METHOD CALL 'ic_nvthruput' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'ic-nvthruput' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ic_nvthruput {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}



## @name ::ic_dump_hw_stats
##
## @summary Parses the raw output from the "ic dump hw_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_dump_hw_stats with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_dump_hw_stats(%args);
## INPUTS TO METHOD CALL 'ic_dump_hw_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic dump hw_stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;	Connection Error :               0
##&nbsp;	SQ Overrun Error :               56
##&nbsp;	Allocate PD Error :              0
##&nbsp;	Deallocate PD Error :            0
##&nbsp;	Create CQ Error :                0
##&nbsp;	Destroy CQ Error :               0
##&nbsp;	Create QP Error :                0
##&nbsp;	Destroy QP Error :               0
##&nbsp;	Register Memory Error :          0
##&nbsp;	Deregister Memory Error :        0
##&nbsp;	Post Send Error :                0
##&nbsp;	Post Receive Error :             0
##&nbsp;	Poll CQ Error :                  0
##&nbsp;	SQ Timeout Error :               0
##&nbsp;	Multicast Route ADD Error :      0
##&nbsp;	NIC Flush Error :                0
##&nbsp;	NIC reset count :                0
##&nbsp;	NIC reset error T3 :             0
##&nbsp;	NIC reset error IWCH :           0
##&nbsp;	NIC reset error IF :             0
##&nbsp;	NIC reset error ULP :            0
##&nbsp;	NIC reset error NV :             0
##&nbsp;	NIC reset memory timeout :       0
##&nbsp;	NIC reset memory max :           0
##&nbsp;	Early connect fail return code : 0
##&nbsp;
##&nbsp;	---- H/W Error Stats ----
##&nbsp;	Responder Number of local length Errors: 	0
##&nbsp;	Requester Number of local length Errors: 	0
##&nbsp;	Responder Number of local QP operation Errors: 	0
##&nbsp;	Requester Number local QP operation Error: 	0
##&nbsp;	Responder Number of local protection Errors: 	0
##&nbsp;	Requester Number of local protection Errors: 	0
##&nbsp;	Responder Number of WR flushed Errors: 	1544
##&nbsp;	Requester Number of WR flushed Errors: 	131
##&nbsp;	Requester Number of memory window bind Errors: 	0
##&nbsp;	Requester Number of bad response Errors: 	0
##&nbsp;	Responder Number of local access Errors: 	0
##&nbsp;	Responder Number of remote invalid req Errors: 	0
##&nbsp;	Requester Number of remote invalid req Errors: 	0
##&nbsp;	Responder Number of remote access Errors:		0
##&nbsp;	Requester Number of remote access Errors:		0
##&nbsp;	Requester Number of transport retries exceeded Errors: 	0
##&nbsp;	Requester Number of RNR nak retries exceeded Errors: 	0
##&nbsp;	Responder Number of RNR naks  sent: 	0
##&nbsp;	Requester Number of RNR naks  received: 	0
##&nbsp;	Responder Number of out of sequence requests received: 	0
##&nbsp;	Requester Number of out of sequence Naks received: 	0
##&nbsp;	Responder Number of bad multicast packets received: 	0
##&nbsp;	Number of UD discarded packets: 	0
##&nbsp;	Number of UC discarded packets: 	0
##&nbsp;	Number of CQ overflows: 	0
##&nbsp;	Number of EQ overflows: 	0
##&nbsp;	Number of bad doorbells: 	0
##&nbsp;	Responder Number local EE operation Error: 	0
##&nbsp;	Requester Number local EE operation Error: 	0
##&nbsp;	Number of remote operation Errors: 	0
##&nbsp;	Number of remote aborted Errors: 	0
##&nbsp;	Number of invalid EE context Errors: 	0
##&nbsp;	Number of invalid EE context state Errors: 	0
##&nbsp;	Responder Number of RESYNC operations: 	0
##&nbsp;	Requester Number of RESYNC operations: 	0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nic_reset_memory_timeout' => '0',
##&nbsp;            'nic_reset_error_if' => '0',
##&nbsp;            'connection_error' => '0',
##&nbsp;            'post_send_error' => '0',
##&nbsp;            'nic_flush_error' => '0',
##&nbsp;            'destroy_cq_error' => '0',
##&nbsp;            'h/w_error_stats' => [
##&nbsp;                                   {
##&nbsp;                                     'number_of_uc_discarded_packets' => '0',
##&nbsp;                                     'requester_number_of_transport_retries_exceeded_errors' => '0',
##&nbsp;                                     'requester_number_of_wr_flushed_errors' => '131',
##&nbsp;                                     'responder_number_of_remote_access_errors' => '0',
##&nbsp;                                     'responder_number_of_remote_invalid_req_errors' => '0',
##&nbsp;                                     'responder_number_of_local_length_errors' => '0',
##&nbsp;                                     'requester_number_of_out_of_sequence_naks_received' => '0',
##&nbsp;                                     'responder_number_of_out_of_sequence_requests_received' => '0',
##&nbsp;                                     'responder_number_of_local_protection_errors' => '0',
##&nbsp;                                     'number_of_bad_doorbells' => '0',
##&nbsp;                                     'responder_number_of_resync_operations' => '0',
##&nbsp;                                     'requester_number_of_resync_operations' => '0',
##&nbsp;                                     'responder_number_local_ee_operation_error' => '0',
##&nbsp;                                     'requester_number_of_memory_window_bind_errors' => '0',
##&nbsp;                                     'requester_number_of_local_protection_errors' => '0',
##&nbsp;                                     'number_of_invalid_ee_context_errors' => '0',
##&nbsp;                                     'number_of_invalid_ee_context_state_errors' => '0',
##&nbsp;                                     'responder_number_of_rnr_naks_sent' => '0',
##&nbsp;                                     'requester_number_of_rnr_naks_received' => '0',
##&nbsp;                                     'responder_number_of_wr_flushed_errors' => '1544',
##&nbsp;                                     'requester_number_local_qp_operation_error' => '0',
##&nbsp;                                     'number_of_eq_overflows' => '0',
##&nbsp;                                     'number_of_cq_overflows' => '0',
##&nbsp;                                     'requester_number_of_remote_invalid_req_errors' => '0',
##&nbsp;                                     'requester_number_local_ee_operation_error' => '0',
##&nbsp;                                     'number_of_remote_operation_errors' => '0',
##&nbsp;                                     'responder_number_of_bad_multicast_packets_received' => '0',
##&nbsp;                                     'responder_number_of_local_access_errors' => '0',
##&nbsp;                                     'requester_number_of_remote_access_errors' => '0',
##&nbsp;                                     'requester_number_of_rnr_nak_retries_exceeded_errors' => '0',
##&nbsp;                                     'responder_number_of_local_qp_operation_errors' => '0',
##&nbsp;                                     'requester_number_of_local_length_errors' => '0',
##&nbsp;                                     'number_of_remote_aborted_errors' => '0',
##&nbsp;                                     'requester_number_of_bad_response_errors' => '0',
##&nbsp;                                     'number_of_ud_discarded_packets' => '0'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'nic_reset_error_iwch' => '0',
##&nbsp;            'register_memory_error' => '0',
##&nbsp;            'multicast_route_add_error' => '0',
##&nbsp;            'destroy_qp_error' => '0',
##&nbsp;            'post_receive_error' => '0',
##&nbsp;            'early_connect_fail_return_code' => '0',
##&nbsp;            'create_qp_error' => '0',
##&nbsp;            'create_cq_error' => '0',
##&nbsp;            'deregister_memory_error' => '0',
##&nbsp;            'allocate_pd_error' => '0',
##&nbsp;            'deallocate_pd_error' => '0',
##&nbsp;            'nic_reset_error_t3' => '0',
##&nbsp;            'poll_cq_error' => '0',
##&nbsp;            'sq_overrun_error' => '56',
##&nbsp;            'sq_timeout_error' => '0',
##&nbsp;            'nic_reset_error_nv' => '0',
##&nbsp;            'nic_reset_memory_max' => '0',
##&nbsp;            'nic_reset_error_ulp' => '0',
##&nbsp;            'nic_reset_count' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_dump_hw_stats {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $result = _ic_stats_error(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $result;
}

## @name ::ic_ifthruput
##
## @summary Parses the raw output from the "ic ifthruput" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ifthruput with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ifthruput(%args);
## INPUTS TO METHOD CALL 'ic_ifthruput' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'channel-no' </td> <td> '0' </td></tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic ifthruput 5; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;1677721 messages of 5 bytes sent in 15108614 usecs
##&nbsp;111044 IOPS, 0 Mbytes/s
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_messages' => '1677721',
##&nbsp;            'speed_in_mbytes' => '0',
##&nbsp;            'size_in_bytes' => '5',
##&nbsp;            'iops' => '111044',
##&nbsp;            'sent_in_usecs' => '15108614'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_ifthruput {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ($line =~ /(\d+)\s+messages\s+\w+\s+(\d+)\s+.*\s+(\d+).*/){
            $result_array[0]->{'total_messages'} = $1;
            $result_array[0]->{'size_in_bytes'} = $2;
            $result_array[0]->{'sent_in_usecs'} = $3;
        } elsif ($line =~ /(\d+)\s+IOPS\,\s+(\d+)\s+\w+.*/) {
            $result_array[0]->{'iops'} = $1;
            $result_array[0]->{'speed_in_mbytes'} = $2;
        }
    }
    return \@result_array;
}

## @name ::ic_reset
##
## @summary Parses the raw output from the "ic reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_reset with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_reset(%args);
## INPUTS TO METHOD CALL 'ic_reset' ON APISET OBJECT
## <tr> <td> No switch for this key </td> <td> 'link' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'rv' </td> </tr>
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ic reset link
##&nbsp;$VAR1 = [];
##&nbsp;
sub ic_reset {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::ic_config_show
##
## @summary Parses the raw output from the "ic config show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_config_show with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_config_show(%args);
## INPUTS TO METHOD CALL 'ic_config_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic config show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;   Interconnect Type: iWARP (Chelsio T3)
##&nbsp;     Local System ID: 1573989891
##&nbsp;   Partner System ID: 1573989966
##&nbsp;Connection Initiator: Partner
##&nbsp;c0a: flags=0x1f0e867<UP,BROADCAST,RUNNING,MULTICAST,TCPCKSUM,LINK_UP,UDPCKSUM> mtu 9000 PRIVATE
##&nbsp;        inet 192.168.1.157 netmask 0xffffff00 broadcast 192.168.1.255   Vserver ID: 0
##&nbsp;        ether 00:a0:98:16:5d:5a (auto-10g_twinax-fd-up) flowcontrol full
##&nbsp;c0b: flags=0x1f0e867<UP,BROADCAST,RUNNING,MULTICAST,TCPCKSUM,LINK_UP,UDPCKSUM> mtu 9000 PRIVATE
##&nbsp;        inet 192.168.2.110 netmask 0xffffff00 broadcast 192.168.2.255   Vserver ID: 0
##&nbsp;        ether 00:a0:98:16:5d:5b (auto-10g_twinax-fd-up) flowcontrol full
##&nbsp;           Interface: External
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = {
##&nbsp;           'connection_initiator' => 'Partner',
##&nbsp;           'local_system_id' => '1573989891',
##&nbsp;           'interfaces' => [
##&nbsp;                             {
##&nbsp;                               'flags' => [
##&nbsp;                                            {
##&nbsp;                                              'tcpcksum' => 1,
##&nbsp;                                              'link_up' => 1,
##&nbsp;                                              'multicast' => 1,
##&nbsp;                                              'running' => 1,
##&nbsp;                                              'up' => 1,
##&nbsp;                                              'broadcast' => 1,
##&nbsp;                                              'udpcksum' => 1,
##&nbsp;                                              'hex' => '0x1f0e867'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               'flowcontrol' => 'full',
##&nbsp;                               'interface' => 'c0a',
##&nbsp;                               'media_spec' => [
##&nbsp;                                                 {
##&nbsp;                                                   'link' => 'up',
##&nbsp;                                                   'mediatype' => 'auto',
##&nbsp;                                                   'cable' => '10g_twinax',
##&nbsp;                                                   'duplex' => 'fd'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                               'ipv4_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'broadcast' => '192.168.1.255',
##&nbsp;                                                  'vserver_id' => '0',
##&nbsp;                                                  'ip' => '192.168.1.157',
##&nbsp;                                                  'netmask' => '0xffffff00'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                               'mac' => '00:a0:98:16:5d:5a',
##&nbsp;                               'mtu' => '9000'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'flags' => [
##&nbsp;                                            {
##&nbsp;                                              'tcpcksum' => 1,
##&nbsp;                                              'link_up' => 1,
##&nbsp;                                              'multicast' => 1,
##&nbsp;                                              'running' => 1,
##&nbsp;                                              'up' => 1,
##&nbsp;                                              'broadcast' => 1,
##&nbsp;                                              'udpcksum' => 1,
##&nbsp;                                              'hex' => '0x1f0e867'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                               'flowcontrol' => 'full',
##&nbsp;                               'interface' => 'c0b',
##&nbsp;                               'media_spec' => [
##&nbsp;                                                 {
##&nbsp;                                                   'link' => 'up',
##&nbsp;                                                   'mediatype' => 'auto',
##&nbsp;                                                   'cable' => '10g_twinax',
##&nbsp;                                                   'duplex' => 'fd'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                               'ipv4_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'broadcast' => '192.168.2.255',
##&nbsp;                                                  'vserver_id' => '0',
##&nbsp;                                                  'ip' => '192.168.2.110',
##&nbsp;                                                  'netmask' => '0xffffff00'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                               'mac' => '00:a0:98:16:5d:5b',
##&nbsp;                               'mtu' => '9000'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;           'partner_system_id' => '1573989966',
##&nbsp;           'interconnect_type' => 'iWARP (Chelsio T3)',
##&nbsp;           'interface' => 'External'
##&nbsp;         };
##&nbsp;
sub ic_config_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array;
    my $key;
    my $i = 0;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line
            =~ /(Interconnect Type|Local System ID|Partner System ID|Connection Initiator|Interface|Node role)/i
            )
        {
            my @values = split( ':\s+', $line );
            $values[1] =~ s/(\d+)\(\*\)/$1/;
            $result_array[0]->{ to_key_name( item => $values[0] ) }
                = $values[1];
        } elsif ( $line =~ /^(\w+)\:\s+flags=(.+)<(.+)>.*/ ) {
            my %flags;
            $result_array[0]->{'interfaces'}->[$i]->{'interface'} = $1;
            $flags{'hex'} = $2;
            my $states = $3;
            foreach my $state ( split( /,/, $states ) ) {
                $flags{ to_key_name( item => $state ) } = 1;
            }
            push(
                @{ $result_array[0]->{'interfaces'}->[$i]->{'flags'} },
                \%flags
            );
            if ( $line =~ /mtu\s+(\d+)/ ) {
                $result_array[0]->{'interfaces'}->[$i]->{'mtu'} = $1;
            }
            if ( $line =~ /dad_attempts\s+(\d+)/ ) {
                $result_array[0]->{'interfaces'}->[$i]->{'dad_attempts'} = $1;
            }
        } elsif ( $line
            =~ /inet\s(\S+)\snetmask\s(\S+)\sbroadcast\s(\S+)\s+(\w+\s\w+)\:\s+(\d+)/i
            )
        {
            my %info;
            $info{'ip'}        = $1;
            $info{'netmask'}   = $2;
            $info{'broadcast'} = $3;
            $info{ to_key_name( item => $4 ) } = $5;
            push(
                @{ $result_array[0]->{'interfaces'}->[$i]->{'ipv4_info'} },
                \%info
            );
        } elsif ( $line =~ /ether\s(\S+)\s\((\w+.*)\)\s\w+\s(\w+)/i ) {
            my %cable_info;
            $result_array[0]->{'interfaces'}->[$i]->{'mac'} = $1;
            $result_array[0]->{'interfaces'}->[$i]->{'flowcontrol'}
                = $3;
            my @info = split( /-/, $2 );
            $cable_info{'mediatype'} = $info[0];
            $cable_info{'cable'}     = $info[1];
            $cable_info{'duplex'}    = $info[2];
            $cable_info{'link'}      = $info[3];
            push(
                @{  $result_array[0]->{'interfaces'}->[$i]->{'media_spec'}
                    },
                \%cable_info
            );
            $i++;
        }
          elsif ( $line =~ /^(\w+)\:\s+IP\:\s+(\S+)/ ) {
            $result_array[0]->{'interfaces'}->[$i]->{'interface'} = $1;
            $result_array[0]->{'interfaces'}->[$i]->{'ip'} = $2;
            $i++;
        }
    }
    return \@result_array;

}    #end sub ic_config_show

## @name ::ic_link
##
## @summary Parses the raw output from the "ic link" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_link with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_link(%args);
## INPUTS TO METHOD CALL 'ic_link' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'on' </td> <td> 'on' </td> <td> '0' </td> </tr>
## <tr> <td> 'off' </td> <td> 'off' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub ic_link {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}    #end sub ic_link

## @name ::ic_ood_stats
##
## @summary Parses the raw output from the "ic ood stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ood_stats with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ood_stats(%args);
## INPUTS TO METHOD CALL 'ic_ood_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'on' </td> <td> 'stats-on' </td> <td> '1' </td> </tr>
## <tr> <td> 'clear' </td> <td> 'stats-clear' </td> <td> '1' </td> </tr>
## <tr> <td> 'off' </td> <td> 'stats-off' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub ic_ood_stats {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}    #end sub ic_ood_stats

## @name ::ic_ood_stats_error_show
##
## @summary Parses the raw output from the "ic ood stats error show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ood_stats_error_show with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ood_stats_error_show(%args);
## INPUTS TO METHOD CALL 'ic_ood_stats_error_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic ood stats error show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Coalescing cut error: 0
##&nbsp;Coalescing memory force freed: 0
##&nbsp;Coalescing Memory Full: 0
##&nbsp;Dispath create failed: 0
##&nbsp;Glist entries error: 0
##&nbsp;OOD request sent failed: 0
##&nbsp;OOD Mempool usedq slot allocation failed: 0
##&nbsp;Req was accessed while VI is not in connected state: 0
##&nbsp;Request memory force freed: 0
##&nbsp;Request memory full: 0
##&nbsp;Request missing when completed: 0
##&nbsp;Total number of out of order requests on receive side: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'coalescing_cut_error' => '0',
##&nbsp;            'coalescing_memory_force_freed' => '0',
##&nbsp;            'coalescing_memory_full' => '0',
##&nbsp;            'request_memory_force_freed' => '0',
##&nbsp;            'request_missing_when_completed' => '0',
##&nbsp;            'total_number_of_out_of_order_requests_on_receive_side' => '0',
##&nbsp;            'dispath_create_failed' => '0',
##&nbsp;            'ood_mempool_usedq_slot_allocation_failed' => '0',
##&nbsp;            'request_memory_full' => '0',
##&nbsp;            'req_was_accessed_while_vi_is_not_in_connected_state' => '0',
##&nbsp;            'glist_entries_error' => '0',
##&nbsp;            'ood_request_sent_failed' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_ood_stats_error_show {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    foreach my $line (@lines) {
        my @values = split( /:\s+/, $line );
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
    }
    return \@result_array;
}    #end sub ic_ood_stats_error_show

## @name ::ic_ood_stats_performance_show
##
## @summary Parses the raw output from the "ic ood stats performance show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ood_stats_performance_show with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ood_stats_performance_show(%args);
## INPUTS TO METHOD CALL 'ic_ood_stats_performance_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'channel-no' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q test; ic ood stats performance show  -v; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NIC 0 Statistics
##&nbsp;--------------------
##&nbsp;OOD Status Enabled
##&nbsp;Coalescing Enabled
##&nbsp;
##&nbsp;OOD VI 6 Statistics (RDMA Write VI)
##&nbsp;-------------------------
##&nbsp;# of IOs: 475668
##&nbsp;# of IC XOrder IOs: 160005
##&nbsp;bucket of xorder qdepth on new xorder req
##&nbsp;0 = 144887, 1 = 14394, 2 = 710, 3 = 10, 4 = 4, 5 = 0, 6 = 0, 7 = 0, 8 = 0, 9 = 0,
##&nbsp;Total=15860 Count=160006 Avg=0 Max=4
##&nbsp;
##&nbsp;bucket of normal reqs qdepth
##&nbsp;0 = 148347, 1 = 154853, 2 = 146306, 3 = 14842, 4 = 7102, 5 = 3961, 6 = 206, 7 = 28, 8 = 19, 9 = 10,
##&nbsp;Total=541914 Count=475674 Avg=1 Max=17
##&nbsp;
##&nbsp;bucket count of xorder reqs coalsced
##&nbsp;0 = 0, 1 = 159949, 2 = 27, 3 = 1, 4 = 0, 5 = 0, 6 = 0, 7 = 0, 8 = 0, 9 = 0,
##&nbsp;Total=160006 Count=159977 Avg=1 Max=3
##&nbsp;
##&nbsp;bucket of % reqs were cut from coal chain by timer
##&nbsp;0-9 = 0, 10-19 = 0, 20-29 = 0, 30-39 = 2, 40-49 = 0, 50-59 = 6666, 60-69 = 25, 70-79 = 1, 80-89 = 0, 90-99 = 0,
##&nbsp;Total=335091 Count=6694 Avg=50 Max=75
##&nbsp;
##&nbsp;bucket of time for normal reqs
##&nbsp;0-199 = 236675, 200-399 = 69753, 400-599 = 7149, 600-799 = 1015, 800-999 = 334, 1000-1199 = 169, 1200-1399 = 146, 1400-1599 = 98, 1600-1799 = 181, 1800-1999 = 667,
##&nbsp;Total=56598419 Count=316187 Avg=179 Max=49764
##&nbsp;
##&nbsp;bucket of time for xorder reqs
##&nbsp;0-199 = 58294, 200-399 = 78716, 400-599 = 15477, 600-799 = 3148, 800-999 = 1022, 1000-1199 = 537, 1200-1399 = 356, 1400-1599 = 236, 1600-1799 = 315, 1800-1999 = 1357,
##&nbsp;Total=59526560 Count=159458 Avg=373 Max=260282
##&nbsp;
##&nbsp;
##&nbsp;IO Timings:
##&nbsp;-------------
##&nbsp;Total time                           116145639
##&nbsp;Total IOs                               475674
##&nbsp;Min IO time                                 20
##&nbsp;Max IO time                             260282
##&nbsp;Total Sample Rtrip time                  79313
##&nbsp;Sample Reqs                                100
##&nbsp;
##&nbsp;
##&nbsp;OOD VI 7 Statistics (RDMA Write VI)
##&nbsp;-------------------------
##&nbsp;# of IOs: 7541
##&nbsp;# of IC XOrder IOs: 3468
##&nbsp;bucket of xorder qdepth on new xorder req
##&nbsp;0 = 3193, 1 = 47, 2 = 13, 3 = 9, 4 = 5, 5 = 5, 6 = 5, 7 = 5, 8 = 5, 9 = 181,
##&nbsp;Total=5394 Count=3468 Avg=1 Max=50
##&nbsp;
##&nbsp;bucket of normal reqs qdepth
##&nbsp;0 = 5376, 1 = 1258, 2 = 159, 3 = 105, 4 = 71, 5 = 48, 6 = 39, 7 = 30, 8 = 22, 9 = 433,
##&nbsp;Total=18924 Count=7541 Avg=2 Max=92
##&nbsp;
##&nbsp;bucket count of xorder reqs coalsced
##&nbsp;0 = 0, 1 = 3468, 2 = 0, 3 = 0, 4 = 0, 5 = 0, 6 = 0, 7 = 0, 8 = 0, 9 = 0,
##&nbsp;Total=3468 Count=3468 Avg=1 Max=1
##&nbsp;
##&nbsp;bucket of % reqs were cut from coal chain by timer
##&nbsp;0-9 = 0, 10-19 = 0, 20-29 = 0, 30-39 = 0, 40-49 = 0, 50-59 = 0, 60-69 = 0, 70-79 = 0, 80-89 = 0, 90-99 = 0,
##&nbsp;Total=0 Count=0 Avg=0 Max=0
##&nbsp;
##&nbsp;bucket of time for normal reqs
##&nbsp;0-199 = 1558, 200-399 = 1555, 400-599 = 629, 600-799 = 451, 800-999 = 357, 1000-1199 = 198, 1200-1399 = 156, 1400-1599 = 106, 1600-1799 = 99, 1800-1999 = 1700,
##&nbsp;Total=18608744 Count=6809 Avg=2732 Max=24876
##&nbsp;
##&nbsp;bucket of time for xorder reqs
##&nbsp;0-199 = 77, 200-399 = 113, 400-599 = 28, 600-799 = 47, 800-999 = 45, 1000-1199 = 23, 1200-1399 = 16, 1400-1599 = 9, 1600-1799 = 11, 1800-1999 = 363,
##&nbsp;Total=11483820 Count=732 Avg=15688 Max=324490
##&nbsp;
##&nbsp;
##&nbsp;IO Timings:
##&nbsp;-------------
##&nbsp;Total time                            30092564
##&nbsp;Total IOs                                 7541
##&nbsp;Min IO time                                 21
##&nbsp;Max IO time                             324490
##&nbsp;Total Sample Rtrip time                 615203
##&nbsp;Sample Reqs                                100
##&nbsp;
##&nbsp;
##&nbsp;OOD VI 8 Statistics (RDMA Write VI)
##&nbsp;-------------------------
##&nbsp;# of IOs: 2636
##&nbsp;# of IC XOrder IOs: 2636
##&nbsp;bucket of xorder qdepth on new xorder req
##&nbsp;0 = 296, 1 = 151, 2 = 152, 3 = 154, 4 = 90, 5 = 92, 6 = 95, 7 = 103, 8 = 102, 9 = 1401,
##&nbsp;Total=205859 Count=2636 Avg=78 Max=401
##&nbsp;
##&nbsp;bucket of normal reqs qdepth
##&nbsp;0 = 148, 1 = 152, 2 = 152, 3 = 155, 4 = 159, 5 = 92, 6 = 95, 7 = 103, 8 = 102, 9 = 1478,
##&nbsp;Total=206857 Count=2636 Avg=78 Max=401
##&nbsp;
##&nbsp;bucket count of xorder reqs coalsced
##&nbsp;0 = 0, 1 = 2627, 2 = 0, 3 = 0, 4 = 0, 5 = 0, 6 = 0, 7 = 0, 8 = 0, 9 = 0,
##&nbsp;Total=2627 Count=2627 Avg=1 Max=1
##&nbsp;
##&nbsp;bucket of % reqs were cut from coal chain by timer
##&nbsp;0-9 = 0, 10-19 = 0, 20-29 = 0, 30-39 = 0, 40-49 = 0, 50-59 = 0, 60-69 = 0, 70-79 = 0, 80-89 = 0, 90-99 = 0,
##&nbsp;Total=0 Count=0 Avg=0 Max=0
##&nbsp;
##&nbsp;bucket of time for normal reqs
##&nbsp;0-199 = 33, 200-399 = 18, 400-599 = 19, 600-799 = 18, 800-999 = 15, 1000-1199 = 9, 1200-1399 = 6, 1400-1599 = 7, 1600-1799 = 10, 1800-1999 = 12,
##&nbsp;Total=141646 Count=147 Avg=963 Max=18023
##&nbsp;
##&nbsp;bucket of time for xorder reqs
##&nbsp;0-199 = 1, 200-399 = 3, 400-599 = 10, 600-799 = 11, 800-999 = 15, 1000-1199 = 13, 1200-1399 = 17, 1400-1599 = 17, 1600-1799 = 26, 1800-1999 = 2366,
##&nbsp;Total=383656927 Count=2479 Avg=154762 Max=850491
##&nbsp;
##&nbsp;
##&nbsp;IO Timings:
##&nbsp;-------------
##&nbsp;Total time                           383798573
##&nbsp;Total IOs                                 2626
##&nbsp;Min IO time                                 37
##&nbsp;Max IO time                             850491
##&nbsp;Total Sample Rtrip time                 101934
##&nbsp;Sample Reqs                                100
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nic_info' => [
##&nbsp;                            {
##&nbsp;                              'ood_status' => 'Enabled',
##&nbsp;                              'nic' => '0',
##&nbsp;                              'coalescing' => 'Enabled'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'stats_info' => [
##&nbsp;                              {
##&nbsp;                                'time_for_normal_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '316187',
##&nbsp;                                                              'bucket200_399' => '69753',
##&nbsp;                                                              'avg' => '179',
##&nbsp;                                                              'bucket1400_1599' => '98',
##&nbsp;                                                              'bucket1200_1399' => '146',
##&nbsp;                                                              'max' => '49764',
##&nbsp;                                                              'bucket400_599' => '7149',
##&nbsp;                                                              'total' => '56598419',
##&nbsp;                                                              'bucket1800_1999' => '667,',
##&nbsp;                                                              'bucket1600_1799' => '181',
##&nbsp;                                                              'bucket800_999' => '334',
##&nbsp;                                                              'bucket600_799' => '1015',
##&nbsp;                                                              'bucket1000_1199' => '169',
##&nbsp;                                                              'bucket0_199' => '236675'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'reqs_were_cut_from_coal_chain_by_timer' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'bucket0_9' => '0',
##&nbsp;                                                                                'count' => '6694',
##&nbsp;                                                                                'bucket70_79' => '1',
##&nbsp;                                                                                'avg' => '50',
##&nbsp;                                                                                'max' => '75',
##&nbsp;                                                                                'bucket80_89' => '0',
##&nbsp;                                                                                'bucket50_59' => '6666',
##&nbsp;                                                                                'bucket30_39' => '2',
##&nbsp;                                                                                'total' => '335091',
##&nbsp;                                                                                'bucket40_49' => '0',
##&nbsp;                                                                                'bucket10_19' => '0',
##&nbsp;                                                                                'bucket60_69' => '25',
##&nbsp;                                                                                'bucket20_29' => '0',
##&nbsp;                                                                                'bucket90_99' => '0,'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                'count_of_xorder_reqs_coalsced' => [
##&nbsp;                                                                     {
##&nbsp;                                                                       'count' => '159977',
##&nbsp;                                                                       'avg' => '1',
##&nbsp;                                                                       'bucket1' => '159949',
##&nbsp;                                                                       'max' => '3',
##&nbsp;                                                                       'bucket8' => '0',
##&nbsp;                                                                       'bucket7' => '0',
##&nbsp;                                                                       'bucket2' => '27',
##&nbsp;                                                                       'total' => '160006',
##&nbsp;                                                                       'bucket0' => '0',
##&nbsp;                                                                       'bucket6' => '0',
##&nbsp;                                                                       'bucket9' => '0,',
##&nbsp;                                                                       'bucket4' => '0',
##&nbsp;                                                                       'bucket5' => '0',
##&nbsp;                                                                       'bucket3' => '1'
##&nbsp;                                                                     }
##&nbsp;                                                                   ],
##&nbsp;                                'ood_stats' => [
##&nbsp;                                                 {
##&nbsp;                                                   'ood' => 'VI 6',
##&nbsp;                                                   'type' => 'RDMA Write VI',
##&nbsp;                                                   'ic_xorder_ios' => '160005',
##&nbsp;                                                   'ios' => '475668'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                                'normal_reqs_qdepth' => [
##&nbsp;                                                          {
##&nbsp;                                                            'count' => '475674',
##&nbsp;                                                            'avg' => '1',
##&nbsp;                                                            'bucket1' => '154853',
##&nbsp;                                                            'max' => '17',
##&nbsp;                                                            'bucket8' => '19',
##&nbsp;                                                            'bucket7' => '28',
##&nbsp;                                                            'bucket2' => '146306',
##&nbsp;                                                            'total' => '541914',
##&nbsp;                                                            'bucket0' => '148347',
##&nbsp;                                                            'bucket6' => '206',
##&nbsp;                                                            'bucket9' => '10,',
##&nbsp;                                                            'bucket4' => '7102',
##&nbsp;                                                            'bucket5' => '3961',
##&nbsp;                                                            'bucket3' => '14842'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                'time_for_xorder_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '159458',
##&nbsp;                                                              'bucket200_399' => '78716',
##&nbsp;                                                              'avg' => '373',
##&nbsp;                                                              'bucket1400_1599' => '236',
##&nbsp;                                                              'bucket1200_1399' => '356',
##&nbsp;                                                              'max' => '260282',
##&nbsp;                                                              'bucket400_599' => '15477',
##&nbsp;                                                              'total' => '59526560',
##&nbsp;                                                              'bucket1800_1999' => '1357,',
##&nbsp;                                                              'bucket1600_1799' => '315',
##&nbsp;                                                              'bucket800_999' => '1022',
##&nbsp;                                                              'bucket600_799' => '3148',
##&nbsp;                                                              'bucket1000_1199' => '537',
##&nbsp;                                                              'bucket0_199' => '58294'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'xorder_qdepth_on_new_xorder_req' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'count' => '160006',
##&nbsp;                                                                         'avg' => '0',
##&nbsp;                                                                         'bucket1' => '14394',
##&nbsp;                                                                         'max' => '4',
##&nbsp;                                                                         'bucket8' => '0',
##&nbsp;                                                                         'bucket7' => '0',
##&nbsp;                                                                         'bucket2' => '710',
##&nbsp;                                                                         'total' => '15860',
##&nbsp;                                                                         'bucket0' => '144887',
##&nbsp;                                                                         'bucket6' => '0',
##&nbsp;                                                                         'bucket9' => '0,',
##&nbsp;                                                                         'bucket4' => '4',
##&nbsp;                                                                         'bucket5' => '0',
##&nbsp;                                                                         'bucket3' => '10'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                'io_timings' => [
##&nbsp;                                                  {
##&nbsp;                                                    'max_io_time' => '260282',
##&nbsp;                                                    'min_io_time' => '20',
##&nbsp;                                                    'sample_reqs' => '100',
##&nbsp;                                                    'total_sample_rtrip_time' => '79313',
##&nbsp;                                                    'total_ios' => '475674',
##&nbsp;                                                    'total_time' => '116145639'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'time_for_normal_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '6809',
##&nbsp;                                                              'bucket200_399' => '1555',
##&nbsp;                                                              'avg' => '2732',
##&nbsp;                                                              'bucket1400_1599' => '106',
##&nbsp;                                                              'bucket1200_1399' => '156',
##&nbsp;                                                              'max' => '24876',
##&nbsp;                                                              'bucket400_599' => '629',
##&nbsp;                                                              'total' => '18608744',
##&nbsp;                                                              'bucket1800_1999' => '1700,',
##&nbsp;                                                              'bucket1600_1799' => '99',
##&nbsp;                                                              'bucket800_999' => '357',
##&nbsp;                                                              'bucket600_799' => '451',
##&nbsp;                                                              'bucket1000_1199' => '198',
##&nbsp;                                                              'bucket0_199' => '1558'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'reqs_were_cut_from_coal_chain_by_timer' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'bucket0_9' => '0',
##&nbsp;                                                                                'count' => '0',
##&nbsp;                                                                                'bucket70_79' => '0',
##&nbsp;                                                                                'avg' => '0',
##&nbsp;                                                                                'max' => '0',
##&nbsp;                                                                                'bucket80_89' => '0',
##&nbsp;                                                                                'bucket50_59' => '0',
##&nbsp;                                                                                'bucket30_39' => '0',
##&nbsp;                                                                                'total' => '0',
##&nbsp;                                                                                'bucket40_49' => '0',
##&nbsp;                                                                                'bucket10_19' => '0',
##&nbsp;                                                                                'bucket60_69' => '0',
##&nbsp;                                                                                'bucket20_29' => '0',
##&nbsp;                                                                                'bucket90_99' => '0,'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                'count_of_xorder_reqs_coalsced' => [
##&nbsp;                                                                     {
##&nbsp;                                                                       'count' => '3468',
##&nbsp;                                                                       'avg' => '1',
##&nbsp;                                                                       'bucket1' => '3468',
##&nbsp;                                                                       'max' => '1',
##&nbsp;                                                                       'bucket8' => '0',
##&nbsp;                                                                       'bucket7' => '0',
##&nbsp;                                                                       'bucket2' => '0',
##&nbsp;                                                                       'total' => '3468',
##&nbsp;                                                                       'bucket0' => '0',
##&nbsp;                                                                       'bucket6' => '0',
##&nbsp;                                                                       'bucket9' => '0,',
##&nbsp;                                                                       'bucket4' => '0',
##&nbsp;                                                                       'bucket5' => '0',
##&nbsp;                                                                       'bucket3' => '0'
##&nbsp;                                                                     }
##&nbsp;                                                                   ],
##&nbsp;                                'ood_stats' => [
##&nbsp;                                                 {
##&nbsp;                                                   'ood' => 'VI 7',
##&nbsp;                                                   'type' => 'RDMA Write VI',
##&nbsp;                                                   'ic_xorder_ios' => '3468',
##&nbsp;                                                   'ios' => '7541'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                                'normal_reqs_qdepth' => [
##&nbsp;                                                          {
##&nbsp;                                                            'count' => '7541',
##&nbsp;                                                            'avg' => '2',
##&nbsp;                                                            'bucket1' => '1258',
##&nbsp;                                                            'max' => '92',
##&nbsp;                                                            'bucket8' => '22',
##&nbsp;                                                            'bucket7' => '30',
##&nbsp;                                                            'bucket2' => '159',
##&nbsp;                                                            'total' => '18924',
##&nbsp;                                                            'bucket0' => '5376',
##&nbsp;                                                            'bucket6' => '39',
##&nbsp;                                                            'bucket9' => '433,',
##&nbsp;                                                            'bucket4' => '71',
##&nbsp;                                                            'bucket5' => '48',
##&nbsp;                                                            'bucket3' => '105'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                'time_for_xorder_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '732',
##&nbsp;                                                              'bucket200_399' => '113',
##&nbsp;                                                              'avg' => '15688',
##&nbsp;                                                              'bucket1400_1599' => '9',
##&nbsp;                                                              'bucket1200_1399' => '16',
##&nbsp;                                                              'max' => '324490',
##&nbsp;                                                              'bucket400_599' => '28',
##&nbsp;                                                              'total' => '11483820',
##&nbsp;                                                              'bucket1800_1999' => '363,',
##&nbsp;                                                              'bucket1600_1799' => '11',
##&nbsp;                                                              'bucket800_999' => '45',
##&nbsp;                                                              'bucket600_799' => '47',
##&nbsp;                                                              'bucket1000_1199' => '23',
##&nbsp;                                                              'bucket0_199' => '77'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'xorder_qdepth_on_new_xorder_req' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'count' => '3468',
##&nbsp;                                                                         'avg' => '1',
##&nbsp;                                                                         'bucket1' => '47',
##&nbsp;                                                                         'max' => '50',
##&nbsp;                                                                         'bucket8' => '5',
##&nbsp;                                                                         'bucket7' => '5',
##&nbsp;                                                                         'bucket2' => '13',
##&nbsp;                                                                         'total' => '5394',
##&nbsp;                                                                         'bucket0' => '3193',
##&nbsp;                                                                         'bucket6' => '5',
##&nbsp;                                                                         'bucket9' => '181,',
##&nbsp;                                                                         'bucket4' => '5',
##&nbsp;                                                                         'bucket5' => '5',
##&nbsp;                                                                         'bucket3' => '9'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                'io_timings' => [
##&nbsp;                                                  {
##&nbsp;                                                    'max_io_time' => '324490',
##&nbsp;                                                    'min_io_time' => '21',
##&nbsp;                                                    'sample_reqs' => '100',
##&nbsp;                                                    'total_sample_rtrip_time' => '615203',
##&nbsp;                                                    'total_ios' => '7541',
##&nbsp;                                                    'total_time' => '30092564'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'time_for_normal_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '147',
##&nbsp;                                                              'bucket200_399' => '18',
##&nbsp;                                                              'avg' => '963',
##&nbsp;                                                              'bucket1400_1599' => '7',
##&nbsp;                                                              'bucket1200_1399' => '6',
##&nbsp;                                                              'max' => '18023',
##&nbsp;                                                              'bucket400_599' => '19',
##&nbsp;                                                              'total' => '141646',
##&nbsp;                                                              'bucket1800_1999' => '12,',
##&nbsp;                                                              'bucket1600_1799' => '10',
##&nbsp;                                                              'bucket800_999' => '15',
##&nbsp;                                                              'bucket600_799' => '18',
##&nbsp;                                                              'bucket1000_1199' => '9',
##&nbsp;                                                              'bucket0_199' => '33'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'reqs_were_cut_from_coal_chain_by_timer' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'bucket0_9' => '0',
##&nbsp;                                                                                'count' => '0',
##&nbsp;                                                                                'bucket70_79' => '0',
##&nbsp;                                                                                'avg' => '0',
##&nbsp;                                                                                'max' => '0',
##&nbsp;                                                                                'bucket80_89' => '0',
##&nbsp;                                                                                'bucket50_59' => '0',
##&nbsp;                                                                                'bucket30_39' => '0',
##&nbsp;                                                                                'total' => '0',
##&nbsp;                                                                                'bucket40_49' => '0',
##&nbsp;                                                                                'bucket10_19' => '0',
##&nbsp;                                                                                'bucket60_69' => '0',
##&nbsp;                                                                                'bucket20_29' => '0',
##&nbsp;                                                                                'bucket90_99' => '0,'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                'count_of_xorder_reqs_coalsced' => [
##&nbsp;                                                                     {
##&nbsp;                                                                       'count' => '2627',
##&nbsp;                                                                       'avg' => '1',
##&nbsp;                                                                       'bucket1' => '2627',
##&nbsp;                                                                       'max' => '1',
##&nbsp;                                                                       'bucket8' => '0',
##&nbsp;                                                                       'bucket7' => '0',
##&nbsp;                                                                       'bucket2' => '0',
##&nbsp;                                                                       'total' => '2627',
##&nbsp;                                                                       'bucket0' => '0',
##&nbsp;                                                                       'bucket6' => '0',
##&nbsp;                                                                       'bucket9' => '0,',
##&nbsp;                                                                       'bucket4' => '0',
##&nbsp;                                                                       'bucket5' => '0',
##&nbsp;                                                                       'bucket3' => '0'
##&nbsp;                                                                     }
##&nbsp;                                                                   ],
##&nbsp;                                'ood_stats' => [
##&nbsp;                                                 {
##&nbsp;                                                   'ood' => 'VI 8',
##&nbsp;                                                   'type' => 'RDMA Write VI',
##&nbsp;                                                   'ic_xorder_ios' => '2636',
##&nbsp;                                                   'ios' => '2636'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                                'normal_reqs_qdepth' => [
##&nbsp;                                                          {
##&nbsp;                                                            'count' => '2636',
##&nbsp;                                                            'avg' => '78',
##&nbsp;                                                            'bucket1' => '152',
##&nbsp;                                                            'max' => '401',
##&nbsp;                                                            'bucket8' => '102',
##&nbsp;                                                            'bucket7' => '103',
##&nbsp;                                                            'bucket2' => '152',
##&nbsp;                                                            'total' => '206857',
##&nbsp;                                                            'bucket0' => '148',
##&nbsp;                                                            'bucket6' => '95',
##&nbsp;                                                            'bucket9' => '1478,',
##&nbsp;                                                            'bucket4' => '159',
##&nbsp;                                                            'bucket5' => '92',
##&nbsp;                                                            'bucket3' => '155'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                'time_for_xorder_reqs' => [
##&nbsp;                                                            {
##&nbsp;                                                              'count' => '2479',
##&nbsp;                                                              'bucket200_399' => '3',
##&nbsp;                                                              'avg' => '154762',
##&nbsp;                                                              'bucket1400_1599' => '17',
##&nbsp;                                                              'bucket1200_1399' => '17',
##&nbsp;                                                              'max' => '850491',
##&nbsp;                                                              'bucket400_599' => '10',
##&nbsp;                                                              'total' => '383656927',
##&nbsp;                                                              'bucket1800_1999' => '2366,',
##&nbsp;                                                              'bucket1600_1799' => '26',
##&nbsp;                                                              'bucket800_999' => '15',
##&nbsp;                                                              'bucket600_799' => '11',
##&nbsp;                                                              'bucket1000_1199' => '13',
##&nbsp;                                                              'bucket0_199' => '1'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                'xorder_qdepth_on_new_xorder_req' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'count' => '2636',
##&nbsp;                                                                         'avg' => '78',
##&nbsp;                                                                         'bucket1' => '151',
##&nbsp;                                                                         'max' => '401',
##&nbsp;                                                                         'bucket8' => '102',
##&nbsp;                                                                         'bucket7' => '103',
##&nbsp;                                                                         'bucket2' => '152',
##&nbsp;                                                                         'total' => '205859',
##&nbsp;                                                                         'bucket0' => '296',
##&nbsp;                                                                         'bucket6' => '95',
##&nbsp;                                                                         'bucket9' => '1401,',
##&nbsp;                                                                         'bucket4' => '90',
##&nbsp;                                                                         'bucket5' => '92',
##&nbsp;                                                                         'bucket3' => '154'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                'io_timings' => [
##&nbsp;                                                  {
##&nbsp;                                                    'max_io_time' => '850491',
##&nbsp;                                                    'min_io_time' => '37',
##&nbsp;                                                    'sample_reqs' => '100',
##&nbsp;                                                    'total_sample_rtrip_time' => '101934',
##&nbsp;                                                    'total_ios' => '2626',
##&nbsp;                                                    'total_time' => '383798573'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_ood_stats_performance_show {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count  = -1;
    my $limit  = -1;
    my $count1 = -1;
    my $limit1 = -1;
    my $key;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    foreach my $line (@lines) {
        if ( $line =~ /NIC\s+(\d+)\s+Statistics/ ) {
            $count++;
            $count1++;
            $limit = -1;

            $result_array[$count1]->{nic_info}[$count]->{nic} = $1;
        } elsif ( $line =~ /(OOD\s+Status|Coalescing)\s+(.*)$/ ) {
            $result_array[$count1]->{nic_info}[$count]
                ->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /OOD\s+(VI\s+\d+).*\s+\((.*)\)/ ) {
            $limit++;
            $result_array[$count1]->{stats_info}[$limit]->{ood_stats}[0]
                ->{ood} = $1;
            $result_array[$count1]->{stats_info}[$limit]->{ood_stats}[0]
                ->{type} = $2;
        } elsif ( $line =~ /#\s+of\s+(.*)[:]\s+(\d+)/ ) {
            $result_array[$count1]->{stats_info}[$limit]->{ood_stats}[0]
                ->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /^(Total|Min|Max|Sample)(.*\w+)\s+(\d+)$/ ) {
            $result_array[$count1]->{stats_info}[$limit]->{io_timings}[0]
                ->{ to_key_name( item => $1 . "" . $2 ) } = $3;
        } elsif ( $line =~ /^bucket\s+[of]*\s*(.*)/ ) {
            $key = $1;
            $key =~ s/%\s*//;
            $limit1++;
        } elsif ( $line =~ /^\d+.*,$/ ) {
            my @values = split( /,\s+/, $line );
            for ( my $j = 0; $j <= $#values; $j++ ) {
                my @values1 = split( /\s+=\s+/, $values[$j] );
                $values1[0] =~ s/-/_/;
                $result_array[$count1]->{stats_info}[$limit]
                    ->{ to_key_name( item => $key ) }[$limit1]
                    ->{ 'bucket' . $values1[0] } = $values1[1];
            }
        } elsif ( $line =~ /^Total.*/ ) {
            $count  = -1;
            $limit1 = -1;
            my @values = split( /\s+/, $line );
            for ( my $j = 0; $j <= $#values; $j++ ) {
                my @values1 = split( /=/, $values[$j] );
                $result_array[$count1]->{stats_info}[$limit]
                    ->{ to_key_name( item => $key ) }[$limit1]
                    ->{ to_key_name( item => $values1[0] ) } = $values1[1];
            }
        }
    }
    return \@result_array;
}    #end sub ic_ood_stats_performance_show

## @name ::ic_ood_diag_send
##
## @summary Parses the raw output from the "ic ood diag send" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ood_diag_send with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ood_diag_send(%args);
## INPUTS TO METHOD CALL 'ic_ood_diag_send' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub ic_ood_diag_send {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}    #end sub ic_ood_diag_send

## @name ::ic_flint
##
## @summary Parses the raw output from the "ic flint" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_flint with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_flint(%args);
## INPUTS TO METHOD CALL 'ic_flint' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'verify' </td> <td> 'verify' </td> <td> '1' </td> </tr>
## <tr> <td> 'query' </td> <td> 'query' </td> <td> '1' </td> </tr>
## <tr> <td> 'image' </td> <td> 'image' </td> <td> '1' </td> </tr>
## <tr> <td> 'flash' </td> <td> 'flash' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; ic flint query image; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Image type:      Failsafe
##&nbsp;FW Version:      1.0.800
##&nbsp;I.S. Version:    1
##&nbsp;Device ID:       25204
##&nbsp;HW Access Key:   Enabled
##&nbsp;Chip Revision:   A0
##&nbsp;Description:     Node             Port1            Sys image
##&nbsp;GUIDs:           0002c9000100d050 0002c9000100d051 0002c9000100d050
##&nbsp;Board ID:        n/a (MT_0230000002)
##&nbsp;VSD:             n/a
##&nbsp;PSID:            MT_0230000002
##&nbsp;Firmware Image Primary Pointer Sector Signature:      5a445a44
##&nbsp;Firmware Image Secondary Pointer Sector Signature:      80004800
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'firmware_image_primary_pointer_sector_signature' => '5a445a44',
##&nbsp;            'guid_description' => [
##&nbsp;                                    {
##&nbsp;                                      'sys_image' => '0002c9000100d050',
##&nbsp;                                      'node' => '0002c9000100d050',
##&nbsp;                                      'port1' => '0002c9000100d051'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'hw_access_key' => 'Enabled',
##&nbsp;            'image_type' => 'Failsafe',
##&nbsp;            'firmware_image_secondary_pointer_sector_signature' => '80004800',
##&nbsp;            'device_id' => '25204',
##&nbsp;            'fw_version' => '1.0.800',
##&nbsp;            'chip_revision' => 'A0',
##&nbsp;            'psid' => 'MT_0230000002',
##&nbsp;            'vsd' => 'n/a',
##&nbsp;            'is_version' => '1',
##&nbsp;            'board_id' => 'n/a (MT_0230000002)'
##&nbsp;          }
##&nbsp;        ];

sub ic_flint {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count = -1;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    shift @lines if ($lines[0] =~ /\w+\s+firmware image on .*/);
    if ( $command =~ /query/ ) {
        my ( @key, @val );
        foreach my $line (@lines) {
            if ( $line =~ /(Image\s+type)\:\s+(.*)/ ) {
                $count++;
                $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /Description\:\s+(.*)/ ) {
                @key = split( /\s\s+/, $1 );
            } elsif ( $line =~ /GUIDs\:\s+(.*)/ ) {
                @val = split( /\s+/, $1 );
                for ( my $i = 0; $i <= $#val; $i++ ) {
                    $result_array[$count]->{guid_description}[0]
                        ->{ to_key_name( item => $key[$i] ) } = $val[$i];
                }
            } else {
                my @values = split( /:\s+/, $line );
                $values[0] =~ s/\.//g;
                $result_array[$count]->{ to_key_name( item => $values[0] ) }
                    = $values[1];
            }
        }
    } elsif ( $command =~ /verify/ ) {
        my ( $key, $head );
        my ( @values, @values1, @values2 );
        for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
            if ( $lines[$i] =~ /(Failsafe\s+image)/ ) {
                $key = to_key_name( item => $1 );
                $count++;
            } elsif ( $lines[$i] =~ /(Invariant)\s+(.*)/ ) {
                $head = to_key_name( item => $1 );
                push( @values, $2 );
                while ( $lines[ $i + 1 ] !~ /(Primary\s+Image)/ ) {
                    push( @values, $lines[ $i + 1 ] );
                    $i++;
                }
                $result_array[$count]->{$key}->{$head} = \@values;
            } elsif ( $lines[$i] =~ /(Primary\s+Image)\s+(.*)/ ) {
                $head = to_key_name( item => $1 );
                push( @values1, $2 );
                while ( $lines[ $i + 1 ] !~ /(Secondary\s+Image)/ ) {
                    push( @values1, $lines[ $i + 1 ] );
                    $i++;
                }
                $result_array[$count]->{$key}->{$head} = \@values1;
            } elsif ( $lines[$i] =~ /(Secondary\s+Image)\s+(.*)/ ) {
                $head = to_key_name( item => $1 );
                push( @values2, $2 );
                while ( $lines[ $i + 1 ] =~ /(\/.*)/ ) {
                    push( @values2, $lines[ $i + 1 ] );
                    $i++;
                }
                $result_array[$count]->{$key}->{$head} = \@values2;
            }
        }
    }
    return \@result_array;
}    #end sub ic_flint

## @name ::ic_ood_status
##
## @summary Parses the raw output from the "ic ood status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_ood_status with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_ood_status(%args);
## INPUTS TO METHOD CALL 'ic_ood_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic ood status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NIC 0 Statistics (Source NIC)
##&nbsp;--------------------
##&nbsp;OOD Status Enabled
##&nbsp;Coalescing Enabled
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ood_status' => 'Enabled',
##&nbsp;            'coalescing' => 'Enabled',
##&nbsp;            'nic' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_ood_status {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count = -1;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    foreach my $line (@lines) {
        if ( $line =~ /NIC\s+(\d+)\s+Statistics/ ) {
            $count++;
            $result_array[$count]->{nic} = $1;
        } elsif ( $line =~ /(.*)\s+(.*)$/ ) {
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
}    #end sub ic_ood_status

## @name ::ic_show_port
##
## @summary Parses the raw output from the "ic show port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_show_port with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_show_port(%args);
## INPUTS TO METHOD CALL 'ic_show_port' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic show port; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Degraded Link Detection: Enabled
##&nbsp;Description                                      Port 1
##&nbsp;---------------------------                      --------------------
##&nbsp;Physical Layer State                             LinkUp
##&nbsp;Link Layer State                                 Active
##&nbsp;Physical Link Up Counter                         9
##&nbsp;Physical Link Down Counter                       8
##&nbsp;Current Active Port                              Yes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'degraded_link_detection' => 'Enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'description' => 'Physical Layer State',
##&nbsp;            'port_1_stats' => 'LinkUp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'description' => 'Link Layer State',
##&nbsp;            'port_1_stats' => 'Active'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'description' => 'Physical Link Up Counter',
##&nbsp;            'port_1_stats' => '9'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'description' => 'Physical Link Down Counter',
##&nbsp;            'port_1_stats' => '8'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'description' => 'Current Active Port',
##&nbsp;            'port_1_stats' => 'Yes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_show_port {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my (@result_array, @key);
    my $count = 0;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    foreach my $line (@lines) {
        if ( $line =~ /(Degraded Link Detection):\s+(\w+)/ ) {
            $result_array[$count]->{to_key_name(item => $1)} = $2;
            $count++;
        } elsif ( $line =~ /(Description)\s\s+(\w+\s\d+)/ && $line !~ /(Description)\s\s+(\w+\s\d+)\s\s+(\w+\s\d+)/ ) {
            @key = ($1, $2.' '.'stats');
        } elsif ( $line =~ /(\w+\s+\w+.*)\s\s+(\w+)/ && $line !~ /(\w+\s+\w+.*)\s\s+(\w+)\s\s+(\w+)/ && $line !~ /^(Description)/  ) {
            $result_array[$count]->{to_key_name(item => $key[0])} = trim_string('string' =>$1);
            $result_array[$count]->{to_key_name(item => $key[1])} = trim_string('string' =>$2);
            $count++;
        } elsif ( $line =~ /(Description)\s\s+(\w+\s\d+)\s\s+(\w+\s\d+)/ ) {
            @key = ($1, $2. ' ' .'stats', $3. ' ' .'stats');
        } elsif ( $line =~ /(\w+\s+\w+.*)\s\s+(\w+)\s\s+(\w+)/ ) {
            $result_array[$count]->{to_key_name(item => $key[0])} = trim_string('string' =>$1);
            $result_array[$count]->{to_key_name(item => $key[1])} = trim_string('string' =>$2);
            $result_array[$count]->{to_key_name(item => $key[2])} = trim_string('string' =>$3);
            $count++;
        }
    }
    return \@result_array;
}    #end sub ic_show_port

## @name ::ic_stats_port
##
## @summary Parses the raw output from the "ic stats port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ic_stats_port with any of its options as described in the
##    corresponding cdef file 'ic.cdefs'
## @synopsis
##      ic_stats_port(%args);
## INPUTS TO METHOD CALL 'ic_stats_port' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ic stats port; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Performance Counters                         Port 0               Port 1
##&nbsp;------------------------------ -------------------- --------------------
##&nbsp;Packets Tx                                        0             20477329
##&nbsp;Packets Rx                                        0             20367848
##&nbsp;Data Tx                                           0           2244348426
##&nbsp;Data Rx                                           0           2169921345
##&nbsp;Wait Tx                                           0               348904
##&nbsp;Symbol Error                                      0                    0
##&nbsp;Link Error Recovery                               0                    0
##&nbsp;Link Down                                         0                    3
##&nbsp;Error Rx                                          0                    0
##&nbsp;Remote Physical Error Rx                          0                    0
##&nbsp;Switch Relay Error Rx                             0                    0
##&nbsp;Discard Tx                                        0                  195
##&nbsp;Constraint Error Tx                               0                    0
##&nbsp;Constraint Error Rx                               0                    0
##&nbsp;Local Link Integrity Error                        0                    0
##&nbsp;Excessive Buffer Overrun Error                    0                    0
##&nbsp;VL 15 Dropped                                     0                    0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Packets Tx',
##&nbsp;            'port_1_stats' => '20477329'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Packets Rx',
##&nbsp;            'port_1_stats' => '20367848'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Data Tx',
##&nbsp;            'port_1_stats' => '2244348426'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Data Rx',
##&nbsp;            'port_1_stats' => '2169921345'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Wait Tx',
##&nbsp;            'port_1_stats' => '348904'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Symbol Error',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Link Error Recovery',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Link Down',
##&nbsp;            'port_1_stats' => '3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Error Rx',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Remote Physical Error Rx',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Switch Relay Error Rx',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Discard Tx',
##&nbsp;            'port_1_stats' => '195'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Constraint Error Tx',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Constraint Error Rx',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Local Link Integrity Error',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'Excessive Buffer Overrun Error',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port_0_stats' => '0',
##&nbsp;            'performance_counters' => 'VL 15 Dropped',
##&nbsp;            'port_1_stats' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ic_stats_port {
    my %args = @_;
    my ( $error_message, $error );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my (@result_array, @key);
    my $count = 0;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    foreach my $line (@lines) {
        if ( $line =~ /(Performance Counters)\s+(\w+\s\d+)/ && $line !~ /(Performance Counters)\s+(\w+\s\d+)\s+(\w+\s\d+)/) {
            @key = ($1, $2. ' ' .'stats');
        } elsif ( ($line =~ /(\w+\s+\w+.*)\s\s+(\d+)/ && $line !~ /(\w+\s+\w+.*)\s\s+(\d+)\s\s+(\d+)/) ) {
            $result_array[$count]->{to_key_name(item => $key[0])} = trim_string('string' =>$1);
            $result_array[$count]->{to_key_name(item => $key[1])} = trim_string('string' =>$2);
            $count++;
        } elsif ( $line =~ /(Performance Counters)\s\s+(\w+\s\d+)\s\s+(\w+\s\d+)/) {
            @key = ($1, $2. ' ' .'stats', $3. ' ' .'stats');
        } elsif ( $line =~ /(\w+\s+\w+.*)\s\s+(\d+)\s\s+(\d+)/ ) {
            $result_array[$count]->{to_key_name(item => $key[0])} = trim_string('string' =>$1);
            $result_array[$count]->{to_key_name(item => $key[1])} = trim_string('string' =>$2);
            $result_array[$count]->{to_key_name(item => $key[2])} = trim_string('string' =>$3);
            $count++;
        }

    }
    return \@result_array;
}    #end sub ic_stats_port


1;
