#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode if_addr_filter_info commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode if_addr_filter_info commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use if_addr_filter_info;
##
##    if_addr_filter_info(%args);
##
## @auhor anupamap@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::if_addr_filter_info;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $resp        = $args{response_object};
    my $command       = $args{command};
    my $error_message;
    my $error         = undef;
    if ( $raw_output =~ /<cmd>.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = "missing required parameter";
    }
        if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
        }
} ## end sub _check_error
## @name ::if_addr_filter_info
##
## @summary Parses the raw output from the "if_addr_filter_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from if_addr_filter_info with any of its options as described in the
##    corresponding cdef file 'if_addr_filter_info.cdefs'
## @synopsis
##      if_addr_filter_info(%args);
## INPUTS TO METHOD CALL 'if_addr_filter_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; if_addr_filter_info e0d; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Total number of address filter entries 1328
##&nbsp;Number of used address filter entries: 5
##&nbsp;The used address filter entries:
##&nbsp;U 0 50 56 94 52 bd
##&nbsp;M 1 0 5e 0 0 1
##&nbsp;M 1 0 5e 0 0 9
##&nbsp;M 33 33 ff 94 52 bd
##&nbsp;M 33 33 0 0 0 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'the_used_address_filter_entries' =>[
##&nbsp;                                                 {
##&nbsp;                                                   'multicast' => [
##&nbsp;                                                                    'M 1 0 5e 0 0 1',
##&nbsp;                                                                    'M 1 0 5e 0 0 9',
##&nbsp;                                                                    'M 33 33 ff 94 52 bd',
##&nbsp;                                                                    'M 33 33 0 0 0 1'
##&nbsp;                                                                  ],
##&nbsp;                                                   'unicast' => [
##&nbsp;                                                                  'U 0 50 56 94 52 bd'
##&nbsp;                                                                ]
##&nbsp;                                                 }
##&nbsp;                                                ],
##&nbsp;            'total_number_of_address_filter_entries' => '1328',
##&nbsp;            'number_of_used_address_filter_entries' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub if_addr_filter_info {
    my %args = @_;
    my ( @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    my @lines = get_lines( raw_output => $output);
    my $k=0;
    my $i=0;
    my $x=0;
    foreach my $line (@lines) {
        if ( $line =~ /^(Total)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+)$/ ){
            $result_array[$x]{'total_number_of_address_filter_entries'} = $7;
        }
        if($line=~/(Number\s+of\s+used\s+address\s+filter\s+entries:)\s*(\d+)/ || $line=~ /(Free\s+number\s+of\s+address\s+filter:)\s*(\d+)/){
            $result_array[$x]{to_key_name(item => $1)}=$2;
        }
        if ( $line =~ /(U.*)/ ) {
            $result_array[$x]{'the_used_address_filter_entries'}[0]->{unicast}[ $k++ ] = $1;
        }
        if ( $line =~ /(M.*)/ ) {
            $result_array[$x]{'the_used_address_filter_entries'}[0]->{multicast}[ $i++ ] = $1;
        }
    }
    return \@result_array;    #to be updated to return a reference
} ## end sub if_addr_filter_info

1;


