#$Id: $
#
#Copyright (c) 2010-2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ifconfig command
##
## @description
##    This is a library to parse the raw output of 7Mode ifconfig command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ifconfig" will be ifconfig()
##
## @synopsis
##    use ifconfig;
##
##    ifconfig(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ifconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::ifconfig
##
## @summary Parses the raw output from the "ifconfig " command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifconfig with any of its options as described in the
##    corresponding cdef file 'ifconfig.cdefs'
## @synopsis
##      ifconfig(%args);
## INPUTS TO METHOD CALL 'ifconfig' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'up_private' </td> <td> 'up_private' </td> <td> '1' </td> </tr>
## <tr> <td> 'inet6' </td> <td> 'inet6' </td> <td> '1' </td> </tr>
## <tr> <td> 'prefixlen' </td> <td> 'prefixlen' </td> <td> '0' </td> </tr>
## <tr> <td> 'nfo' </td> <td> 'nfo' </td> <td> '1' </td> </tr>
## <tr> <td> '-partner' </td> <td> 'no_partner' </td> <td> '1' </td> </tr>
## <tr> <td> '-wins' </td> <td> 'no_wins' </td> <td> '1' </td> </tr>
## <tr> <td> 'up' </td> <td> 'up' </td> <td> '1' </td> </tr>
## <tr> <td> 'mtusize' </td> <td> 'mtusize' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> 'wins' </td> <td> 'wins' </td> <td> '1' </td> </tr>
## <tr> <td> 'broadcast' </td> <td> 'broadcast' </td> <td> '0' </td> </tr>
## <tr> <td> 'inet' </td> <td> 'inet' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> 'down' </td> <td> 'down' </td> <td> '1' </td> </tr>
## <tr> <td> 'untrusted' </td> <td> 'untrusted' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> 'netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-alias' </td> <td> 'unalias' </td> <td> '0' </td> </tr>
## <tr> <td> 'flowcontrol' </td> <td> 'flowcontrol' </td> <td> '0' </td> </tr>
## <tr> <td> 'trusted' </td> <td> 'trusted' </td> <td> '1' </td> </tr>
## <tr> <td> '-nfo' </td> <td> 'no_nfo' </td> <td> '1' </td> </tr>
## <tr> <td> 'no_ddns' </td> <td> 'no_ddns' </td> <td> '1' </td> </tr>
## <tr> <td> 'mediatype' </td> <td> 'mediatype' </td> <td> '0' </td> </tr>
## <tr> <td> 'dad_attempts' </td> <td> 'dad_attempts' </td> <td> '0' </td> </tr>
## <tr> <td> 'alias' </td> <td> 'alias' </td> <td> '0' </td> </tr>
## <tr> <td> 'partner' </td> <td> 'partner' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ifconfig  -a
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        'fe80::2a0:98ff:fe24:6e20'
##&nbsp;                      ],
##&nbsp;            'ipv4' => [
##&nbsp;                        '10.228.216.180'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x2f4c867',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'name' => 'e0a',
##&nbsp;            'ipv4_info' => [
##&nbsp;                             {
##&nbsp;                               'broadcast' => '10.228.216.255',
##&nbsp;                               'ip' => '10.228.216.180',
##&nbsp;                               'mask' => '0xffffff00'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'auto-1000t-fd-up' => 1,
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'scopeid' => '0x6',
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fe80::2a0:98ff:fe24:6e20',
##&nbsp;                               'link_local' => 1,
##&nbsp;                               'autoconf' => 1
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'broadcast' => '10.228.216.255',
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'mac' => '00:a0:98:24:6e:20',
##&nbsp;            'partner' => '10.228.216.182',
##&nbsp;            'mtu' => '1500',
##&nbsp;            'mask' => '0xffffff00',
##&nbsp;            'link_local_ipv6' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        'fe80::2a0:98ff:fe24:6e21',
##&nbsp;                        'fd20:8b1e:b255:8318:2a0:98ff:fe24:6e21'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x2f4c867',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'name' => 'e0b',
##&nbsp;            'auto-1000t-fd-up' => 1,
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'scopeid' => '0x7',
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fe80::2a0:98ff:fe24:6e21',
##&nbsp;                               'link_local' => 1,
##&nbsp;                               'autoconf' => 1
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fd20:8b1e:b255:8318:2a0:98ff:fe24:6e21',
##&nbsp;                               'autoconf' => 1
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'mac' => '00:a0:98:24:6e:21',
##&nbsp;            'mtu' => '1500',
##&nbsp;            'link_local_ipv6' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        'fd20:8b1e:b255:8318:215:17ff:fed3:2778',
##&nbsp;                        'fd20:8b1e:b255:8318::1000',
##&nbsp;                        'fe80::215:17ff:fed3:2778'
##&nbsp;                      ],
##&nbsp;            'ipv4' => [
##&nbsp;                        '4.1.52.0'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x2f4c867',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'name' => 'e1a',
##&nbsp;            'ipv4_info' => [
##&nbsp;                             {
##&nbsp;                               'broadcast' => '4.255.255.255',
##&nbsp;                               'ip' => '4.1.52.0',
##&nbsp;                               'mask' => '0xff000000'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'auto-1000t-fd-up' => 1,
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fd20:8b1e:b255:8318:215:17ff:fed3:2778',
##&nbsp;                               'autoconf' => 1
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fd20:8b1e:b255:8318::1000'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'scopeid' => '0x2',
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fe80::215:17ff:fed3:2778',
##&nbsp;                               'link_local' => 1,
##&nbsp;                               'autoconf' => 1
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'broadcast' => '4.255.255.255',
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'mac' => '00:15:17:d3:27:78',
##&nbsp;            'link_local_ipv6' => undef,
##&nbsp;            'mtu' => '1500',
##&nbsp;            'mask' => '0xff000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x270c866',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'name' => 'e1b',
##&nbsp;            'mac' => '00:15:17:d3:27:79',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x270c866',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'name' => 'e1c',
##&nbsp;            'mac' => '00:15:17:d3:27:7a',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x270c866',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'name' => 'e1d',
##&nbsp;            'mac' => '00:15:17:d3:27:7b',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        'fe80::2a0:98ff:fe24:6e22'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'BROADCAST' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x2b4c867',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'name' => 'e0M',
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'scopeid' => '0x1',
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fe80::2a0:98ff:fe24:6e22',
##&nbsp;                               'link_local' => 1,
##&nbsp;                               'autoconf' => 1
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'dad_attempts' => '2',
##&nbsp;            'mac' => '00:a0:98:24:6e:22',
##&nbsp;            'mtu' => '1500',
##&nbsp;            'link_local_ipv6' => undef
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        '::1',
##&nbsp;                        'fe80::1'
##&nbsp;                      ],
##&nbsp;            'ipv4' => [
##&nbsp;                        '127.0.0.1'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'LOOPBACK' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'TCPCKSUM' => 1,
##&nbsp;                           'MULTIHOST' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x1b68049',
##&nbsp;                           'MULTICAST' => 1
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'lo',
##&nbsp;            'ipv4_info' => [
##&nbsp;                             {
##&nbsp;                               'broadcast' => '127.0.0.1',
##&nbsp;                               'ip' => '127.0.0.1',
##&nbsp;                               'mask' => '0xff000000'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'prefixlen' => '128',
##&nbsp;                               'ip' => '::1'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'scopeid' => '0x8',
##&nbsp;                               'prefixlen' => '64',
##&nbsp;                               'ip' => 'fe80::1',
##&nbsp;                               'link_local' => 1,
##&nbsp;                               'autoconf' => 1
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'broadcast' => '127.0.0.1',
##&nbsp;            'mac' => '00:00:00:00:00:00',
##&nbsp;            'link_local_ipv6' => undef,
##&nbsp;            'mtu' => '8160',
##&nbsp;            'mask' => '0xff000000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ipv6' => [
##&nbsp;                        '::2001'
##&nbsp;                      ],
##&nbsp;            'ipv4' => [
##&nbsp;                        '127.0.20.1'
##&nbsp;                      ],
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'LOOPBACK' => 1,
##&nbsp;                           'UP' => 1,
##&nbsp;                           'RUNNING' => 1,
##&nbsp;                           'hex' => '0x40a400c9'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'losk',
##&nbsp;            'ipv4_info' => [
##&nbsp;                             {
##&nbsp;                               'broadcast' => '127.0.20.1',
##&nbsp;                               'ip' => '127.0.20.1',
##&nbsp;                               'mask' => '0xff000000'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'ipv6_info' => [
##&nbsp;                             {
##&nbsp;                               'prefixlen' => '128',
##&nbsp;                               'ip' => '::2001'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'broadcast' => '127.0.20.1',
##&nbsp;            'mtu' => '9188',
##&nbsp;            'mask' => '0xff000000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifconfig {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'ifconfig',
                             "ptr_error_message" => \$error_message
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # Now we can parse the output
    $result = _ifconfig(raw_output      => $raw_output,
                        command         => $command,
                        response_object => $resp
                        );
    return $result;
}

sub _ifconfig {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my @result_array;

    my $can_throw = 0;

    #e0M: flags=0x2b0c866<BROADCAST,RUNNING,MULTICAST,TCPCKSUM> mtu 1500
    my $first_line = qr/^(\S+)\s+flags/m;
    my @arr = get_lines("raw_output"      => $raw_output,
                        "first_line"      => $first_line,
                        "can_throw"       => $can_throw,
                        "response_object" => $response
                        );

    my %hash;
    my $interface;
    my @interfaces;
    foreach my $line (@arr) {
	next if ( $line =~ /Default IPSpace|ANYVSERVER IPSpace|Blackhole IPSpace|Cluster IPSpace|IPSpace|ACP IPSpace/ );
        #e0b: flags=0x2f4c867<UP,BROADCAST,RUNNING,MULTICAST,TCPCKSUM> mtu 1500
        #inet 172.16.2.253 netmask 0xfffe0000 broadcast 172.17.255.255
        #ether 00:a0:98:24:6e:21 (auto-1000t-fd-up) flowcontrol full
        #inet6 fd20:8b1e:b255:8318:215:17ff:fed3:2778 prefixlen 64 autoconf

        if ($line =~ /^(\S+)\s+flags=(.+)<(.+)>/) {
            $interface = $1;
            my $hex    = $2;
            my $states = $3;
            $interface =~ s/:$//;
            push(@interfaces, $interface);
            $hash{$interface}{'name'} = $interface;

            my %flags;
            $flags{'hex'} = $hex;
            foreach my $state (split(/,/, $states)) {
                $flags{$state} = 1;
            }
            push(@{$hash{$interface}{'flags'}}, \%flags);

            if ($line =~ /mtu\s+(\d+)/) {
                $hash{$interface}{'mtu'} = $1;
            }
            if ($line =~ /dad_attempts\s+(\d+)/) {
                $hash{$interface}{'dad_attempts'} = $1;
            }
        }

        if ($interface) {
            if ($line =~ /.+inet\s+addr:(.+)\s+Bcast:(\S+)\s+Mask:(\S+)/) {

                # Old data structure
                push(@{$hash{$interface}{'ipv4'}}, $1);
                $hash{$interface}{'broadcast'} = $2;

                # New data structure
                my %info;
                $info{'ip'}        = $1;
                $info{'broadcast'} = $2;
                $info{'mask'}      = $3;
                push(@{$hash{$interface}{'ipv4_info'}}, \%info);
            } elsif ($line =~
                /inet\s+(\S+)\s+(?:netmask|netmask-or-prefix)\s+(\S+)\s+broadcast\s+(\S+)/
              ) {

                # Old data structure
                push(@{$hash{$interface}{'ipv4'}}, $1);
                $hash{$interface}{'mask'}      = $2;
                $hash{$interface}{'broadcast'} = $3;

                # New data structure
                my %info;
                $info{'ip'}        = $1;
                $info{'mask'}      = $2;
                $info{'broadcast'} = $3;
                if ($line =~ /vFiler ID:\s*(\d*)/ ) {
                    $info{'vfiler_id'} = $1;
                }
                push(@{$hash{$interface}{'ipv4_info'}}, \%info);
            } elsif ($line =~ /.+inet6\s+addr:\s+(\S+)\/\d+\sScope:Link/) {

                # Old data structure
                $hash{$interface}{'link_local_ipv6'} = $1;

                # New data structure
                my %info;
                $info{'ip'}         = $1;
                $info{'link_local'} = 1;
                push(@{$hash{$interface}{'ipv6_info'}}, \%info);
            } elsif ($line =~ /.+inet6\s+addr:\s+(\S+)\/\d+\sScope:Global/) {

                # Old data structure
                push(@{$hash{$interface}{'ipv6'}}, $1);

                # New data structure
                my %info;
                $info{'ip'} = $1;
                push(@{$hash{$interface}{'ipv6_info'}}, \%info);
            } elsif ($line =~ /inet6\s+(\S+)/) {

                # Old data structure
                push(@{$hash{$interface}{'ipv6'}}, $1);

                # New data structure
                my %info;
                $info{'ip'} = $1;
                if ($line =~ /fe80/) {
                    $info{'link_local'} = 1;

                    # Old data structure
                    $hash{$interface}{'link_local_ipv6'} = $1;
                }
                if ($line =~ /autoconf/) {
                    $info{'autoconf'} = 1;
                }
                if ($line =~ /prefixlen\s+(\S+)/) {
                    $info{'prefixlen'} = $1;
                }
                if ($line =~ /scopeid\s+(\S+)/) {
                    $info{'scopeid'} = $1;
                }
                if ($line =~ /vFiler ID:\s*(\d*)/ ) {
                    $info{'vfiler_id'} = $1;
                }
                push(@{$hash{$interface}{'ipv6_info'}}, \%info);
            } elsif ($line =~ /ether\s+(\S+)/) {
                $hash{$interface}{'mac'} = $1;
                if ( $line =~ /\((auto-[1|4]0\w+-fd-\w+|[1|4]0\w+-fd-\w+|[1|4]0\w+-\w+|auto-unknown-\w+)\)/ ) {
                    my @if = split(/-/, $1);
                    $hash{$interface}{'status'} = pop(@if);
                    $hash{$interface}{'mediatype'} = join("-", @if);
                }
                if ($line =~ /flowcontrol\s+(\S+)/) {
                    $hash{$interface}{'flowcontrol'} = $1;
                }
            } elsif ($line =~ /(?:partner|partner inet)\s+(\S+)\s+\((.+)\)/) {
                $hash{$interface}{'partner'} = $1;
                $hash{$interface}{'partner_port'} = $2;
            } elsif ($line =~ /nfo\s+(\w+)/) {
                $hash{$interface}{'nfo'} = $1;
            }
        }
    }

    foreach my $interface (@interfaces) {
        push(@result_array, $hash{$interface});
    }
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    my $invalidParamRegex =
      '(bad address|no such interface|no such flow control type|interface.+does not exist)';
    if ($raw_output =~ /usage:/) {
        $$error_message = "Required parameter missing";
        $error          = "MissingRequiredParam";
        return $error;
    } elsif ($raw_output =~ /$invalidParamRegex/) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
        return $error;
    }
    return undef;
}

1;
