#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
### @summary Routines to parse 7Mode ifgrp commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ifgrp
##    commands. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "ifgrp status" will be ifgrp_status().
##
## @synopsis
##    use ifgrp;
##    ifgrp_create(%args);
##    ifgrp_add(%args);
##    ifgrp_delete(%args);
##    ifgrp_destroy(%args);
##    ifgrp_favor(%args);
##    ifgrp_nofavor(%args);
##    ifgrp_status(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::ifgrp;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif($output =~ /(.* is not a valid.*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*cannot.*up)/si) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(.* is already attached to an ifgrp)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.* unable to attach \w+: interface not found)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*is already in use)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*no such interface)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(.*Cannot create a single mode 802.3ad ifgrp)/is) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif($output =~ /(.*Cannot combine Load Balancing with single mode ifgrp)/is) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif($output =~ /(.*is not linked to.*ifgrp)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(can't find interface.*)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(bad ifgrp name given)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(No configured ifgrps present)/is) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($output =~ /(is not a member of ifgrp)/is) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }

    return $error;
}
## @name ::ifgrp_status
##
## @summary Parses the raw output from the "ifgrp status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_status with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_status(%args);
## INPUTS TO METHOD CALL 'ifgrp_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifgrp-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp status; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ifgrp_type' => 'multi_mode',
##&nbsp;            'fail' => 'log',
##&nbsp;            'transmit' => 'IP Load balancing',
##&nbsp;            'ifgrp_name' => 'default'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ifgrp_status' => 'Up',
##&nbsp;            'ifgrp_type' => 'single_mode',
##&nbsp;            'interfaces' => [
##&nbsp;                              {
##&nbsp;                                'time' => '08:07:00 (00:05:55)',
##&nbsp;                                'date' => '21Jun2011',
##&nbsp;                                'brokenindications' => '0',
##&nbsp;                                'inputprobepackets' => '0',
##&nbsp;                                'state' => 'up',
##&nbsp;                                'inputbytes' => '262980',
##&nbsp;                                'indication_date' => '21Jun2011',
##&nbsp;                                'transitions' => '1',
##&nbsp;                                'consecutive' => '334',
##&nbsp;                                'interface_name' => 'e0b',
##&nbsp;                                'outputprobepackets' => '0',
##&nbsp;                                'flags' => 'enabled unfavored',
##&nbsp;                                'inputpackets' => '4383',
##&nbsp;                                'outputbytes' => '1218',
##&nbsp;                                'total_strike_count' => '10',
##&nbsp;                                'upindications' => '1',
##&nbsp;                                'dropslink' => '0',
##&nbsp;                                'outputpackets' => '15',
##&nbsp;                                'mediatype' => 'auto-1000t-fd-up',
##&nbsp;                                'dropsif' => '0',
##&nbsp;                                'indication' => 'up',
##&nbsp;                                'indication_time' => '08:07:00',
##&nbsp;                                'current_strike_count' => '0'
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'time' => '08:07:01 (00:05:54)',
##&nbsp;                                'date' => '21Jun2011',
##&nbsp;                                'brokenindications' => '0',
##&nbsp;                                'inputprobepackets' => '0',
##&nbsp;                                'state' => 'down',
##&nbsp;                                'inputbytes' => '202800',
##&nbsp;                                'indication_date' => '21Jun2011',
##&nbsp;                                'transitions' => '1',
##&nbsp;                                'consecutive' => '333',
##&nbsp;                                'interface_name' => 'e0c',
##&nbsp;                                'outputprobepackets' => '0',
##&nbsp;                                'flags' => 'enabled',
##&nbsp;                                'inputpackets' => '3380',
##&nbsp;                                'outputbytes' => '0',
##&nbsp;                                'total_strike_count' => '10',
##&nbsp;                                'upindications' => '1',
##&nbsp;                                'dropslink' => '0',
##&nbsp;                                'outputpackets' => '0',
##&nbsp;                                'mediatype' => 'auto-1000t-fd-up',
##&nbsp;                                'dropsif' => '0',
##&nbsp;                                'indication' => 'up',
##&nbsp;                                'indication_time' => '08:07:01',
##&nbsp;                                'current_strike_count' => '0'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'fail' => 'default',
##&nbsp;            'transmit' => 'none',
##&nbsp;            'links' => '1',
##&nbsp;            'ifgrp_name' => 'vif1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifgrp_status {
    my %args = @_;
    my ($error_message, @result_array, $i, $j, $key1, $key2, $value1, $value2);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "ifgrp status",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines( raw_output => $output, can_throw => 1,
                           response_object => $resp );
    $i = -1;
    foreach my $line (@lines) {
        #default: transmit 'IP Load balancing', Ifgrp Type 'multi_mode', fail 'log'
        if($line =~ m/(\w+):\s+\w+\s+'(.*)',.*'(.*)',.*\s'(\w+)'/) {
            $i++;
            $result_array[$i] -> {'ifgrp_name'} = $1;
            $result_array[$i] -> {'transmit'} = $2;
            $result_array[$i] -> {'ifgrp_type'} = $3;
            $result_array[$i] -> {'fail'} = $4;
        } elsif  ($line =~ m/(\w+):\s+(\d+)\s+\w+,\s+\w+\s+'(.*)',.*'(\w+)'\s+\w+\s+'(\w+)'/) {
            #vif3: 0 links, transmit 'IP Load balancing', Ifgrp Type 'lacp' fail 'default'
            $i++;
            $j = -1;
            $result_array[$i] -> {'ifgrp_name'} = $1;
            $result_array[$i] -> {'links'} = $2;
            $result_array[$i] -> {'transmit'} = $3;
            $result_array[$i] -> {'ifgrp_type'} = $4;
            $result_array[$i] -> {'fail'} = $5;
        } elsif ($line =~ /ifgrp status\s+(\w+)?\s+addr_set(.*)?/i) {
            #Ifgrp Status   Up      Addr_set
            $result_array[$i] -> {'ifgrp_status'} = $1 if($1);
            $result_array[$i] -> {'addr_set'} = $2 if($2);
        } elsif ($line =~ /trunked:\s+(.*)/i) {
            #trunked: ifgrp3
            $result_array[$i] -> {'trunked'} = $1 if($1);
        } elsif ($line =~ /(\w+):\s+\w+\s+(\w+),\s+\w+\s+(\w+)\s+(.*)/i) {
            #e0a: state lag_inactive, since 17Jun2011 07:50:33 (00:00:51)
            $j++;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {'interface_name'} = $1;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {'state'} = $2;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {'date'} = $3;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {'time'} = $4 ;
        } elsif ($line =~ /(\w+):\s(\S+\s?\S+)$/i) {
            #mediatype: auto-1000t-fd-up
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"$1"} = $2;
        } elsif ($line =~ /(\S+\s?\S+\s?\S+)\s+(\d+),\s+(\S+\s?\S+\s?\S+)\s+(\d+)$/i) {
            #input lacp packets 0, output lacp packets 52
            $key1 = $1;
            $key2 = $3;
            $value1 = $2;
            $value2 = $4;
            $key1 =~ s/([(]|[)]|\s+)//g;
            $key2 =~ s/([(]|[)]|\s+)//g;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"$key1"} = $value1;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"$key2"} = $value2;
        } elsif ($line =~ /strike.*:\s+(\d+)\s+of\s+(\d+)/i) {
            #strike count: 0 of 10
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"current_strike_count"} = $1;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"total_strike_count"} = $2;
        } elsif ($line =~ /\w+:\s+(\w+)\s+at\s+(\w+)\s+(.*)/i) {
            #indication: up at 17Jun2011 07:51:20
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"indication"} = $1;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"indication_date"} = $2;
            $result_array[$i] -> {'interfaces'} -> [$j] -> {"indication_time"} = $3;
        }
    }
    return \@result_array;
}
## @name ::ifgrp_create
##
## @summary Parses the raw output from the "ifgrp create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_create with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_create(%args);
## INPUTS TO METHOD CALL 'ifgrp_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'lacp' </td> <td> 'lacp' </td> </tr>
## <tr> <td> 'single' </td> <td> 'single' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifgrp-name' </td> </tr>
## <tr> <td> '-b' </td> <td> 'load-balance' </td> </tr>
## <tr> <td> 'multi' </td> <td> 'multi' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp create single vif1 e0b; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_create {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp create",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
## @name ::ifgrp_favor
##
## @summary Parses the raw output from the "ifgrp favor" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_favor with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_favor(%args);
## INPUTS TO METHOD CALL 'ifgrp_favor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp favor e0b; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_favor {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp favor",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
## @name ::ifgrp_nofavor
##
## @summary Parses the raw output from the "ifgrp nofavor" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_nofavor with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_nofavor(%args);
## INPUTS TO METHOD CALL 'ifgrp_nofavor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp nofavor e0b; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_nofavor {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp nofavor",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}

## @name ::ifgrp_destroy
##
## @summary Parses the raw output from the "ifgrp destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_destroy with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_destroy(%args);
## INPUTS TO METHOD CALL 'ifgrp_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifgrp-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp destroy vif1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_destroy {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp destroy",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}

## @name ::ifgrp_delete
##
## @summary Parses the raw output from the "ifgrp delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_delete with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_delete(%args);
## INPUTS TO METHOD CALL 'ifgrp_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifgrp-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp delete vif1 e0c; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_delete {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp delete",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
## @name ::ifgrp_add
##
## @summary Parses the raw output from the "ifgrp add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifgrp_add with any of its options as described in the
##    corresponding cdef file 'ifgrp.cdefs'
## @synopsis
##      ifgrp_add(%args);
## INPUTS TO METHOD CALL 'ifgrp_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifgrp-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifgrp add vif1 e0c; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifgrp_add {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "ifgrp add",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
1;
