#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode ifinfo commands
##
## @description
##     This is a library to parse the raw output of 7Mode ifinfo commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use ifinfo;
##
##     ifinfo(%args);
##
## @author tanil@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ifinfo;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ( $raw_output =~ /([Uu]sage: ifinfo.*)/ ) {
        $error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(ifinfo: No such interface.*)/ ) {
        $error_message = $1;
        $error          = "InvalidParamValue";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response_object );
    }

}

## @name ::ifinfo
##
## @summary Parses the raw output from the "ifinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifinfo with any of its options as described in the
##    corresponding cdef file 'ifinfo.cdefs'
## @synopsis
##      ifinfo(%args);
## INPUTS TO METHOD CALL 'ifinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ifinfo -a; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-- interface  e0a --
##&nbsp;
##&nbsp;MAC address: 00:50:56:94:3f:7f, mediatype: auto-1000t-fd-up
##&nbsp;DBG: if_flags=00e48867, if_d_flags=00000001
##&nbsp;
##&nbsp;Driver name: Gb_Enet-V
##&nbsp;Unicast addresses: 1
##&nbsp;Unicast slots: 15
##&nbsp;Status 0000cb83
##&nbsp;PCI
##&nbsp;PHY 1000T Status 7c00
##&nbsp;PHY Spec Ctrl 0b68
##&nbsp;RxDescriptorPhysical 00003db3a000 - 0x000000003db40000
##&nbsp;TxDescriptorPhysical 00003db37000 - 0x000000003db39000
##&nbsp;
##&nbsp;EEPROM:
##&nbsp;00: 5000 9456 7f3f 0420
##&nbsp;04: ffff ffff ffff ffff
##&nbsp;08: 0000 0000 460b 0750
##&nbsp;0c: 15ad 100f 8086 01e8
##&nbsp;10: ffff ffff ffff ffff
##&nbsp;14: ffff ffff ffff ffff
##&nbsp;18: ffff ffff ffff ffff
##&nbsp;1c: ffff ffff ffff ffff
##&nbsp;20: dfde 7861 280c 0000
##&nbsp;24: 00c8 ffff ffff ffff
##&nbsp;28: ffff ffff ffff ffff
##&nbsp;2c: ffff ffff ffff 0602
##&nbsp;30: ffff ffff ffff ffff
##&nbsp;34: ffff ffff ffff ffff
##&nbsp;38: ffff ffff ffff ffff
##&nbsp;3c: ffff ffff ffff d698
##&nbsp;
##&nbsp;
##&nbsp;-- interface  e0b --
##&nbsp;
##&nbsp;MAC address: 00:50:56:94:3f:80, mediatype: auto-1000t-fd-up
##&nbsp;DBG: if_flags=00e48867, if_d_flags=00000001
##&nbsp;
##&nbsp;Driver name: Gb_Enet-V
##&nbsp;Unicast addresses: 1
##&nbsp;Unicast slots: 15
##&nbsp;Status 0000cb83
##&nbsp;PCI
##&nbsp;PHY 1000T Status 7c00
##&nbsp;PHY Spec Ctrl 0b68
##&nbsp;RxDescriptorPhysical 00003db47000 - 0x000000003db4d000
##&nbsp;TxDescriptorPhysical 00003db44000 - 0x000000003db46000
##&nbsp;
##&nbsp;EEPROM:
##&nbsp;00: 5000 9456 803f 0420
##&nbsp;04: ffff ffff ffff ffff
##&nbsp;08: 0000 0000 460b 0750
##&nbsp;0c: 15ad 100f 8086 01e8
##&nbsp;10: ffff ffff ffff ffff
##&nbsp;14: ffff ffff ffff ffff
##&nbsp;18: ffff ffff ffff ffff
##&nbsp;1c: ffff ffff ffff ffff
##&nbsp;20: dfde 7861 280c 0000
##&nbsp;24: 00c8 ffff ffff ffff
##&nbsp;28: ffff ffff ffff ffff
##&nbsp;2c: ffff ffff ffff 0602
##&nbsp;30: ffff ffff ffff ffff
##&nbsp;34: ffff ffff ffff ffff
##&nbsp;38: ffff ffff ffff ffff
##&nbsp;3c: ffff ffff ffff d598
##&nbsp;
##&nbsp;
##&nbsp;-- interface  e0c --
##&nbsp;
##&nbsp;MAC address: 00:50:56:94:3f:81, mediatype: auto-1000t-fd-up
##&nbsp;DBG: if_flags=00e48867, if_d_flags=00000001
##&nbsp;
##&nbsp;Driver name: Gb_Enet-V
##&nbsp;Unicast addresses: 1
##&nbsp;Unicast slots: 15
##&nbsp;Status 0000cb83
##&nbsp;PCI
##&nbsp;PHY 1000T Status 7c00
##&nbsp;PHY Spec Ctrl 0b68
##&nbsp;RxDescriptorPhysical 00003db54000 - 0x000000003db5a000
##&nbsp;TxDescriptorPhysical 00003db51000 - 0x000000003db53000
##&nbsp;
##&nbsp;EEPROM:
##&nbsp;00: 5000 9456 813f 0420
##&nbsp;04: ffff ffff ffff ffff
##&nbsp;08: 0000 0000 460b 0750
##&nbsp;0c: 15ad 100f 8086 01e8
##&nbsp;10: ffff ffff ffff ffff
##&nbsp;14: ffff ffff ffff ffff
##&nbsp;18: ffff ffff ffff ffff
##&nbsp;1c: ffff ffff ffff ffff
##&nbsp;20: dfde 7861 280c 0000
##&nbsp;24: 00c8 ffff ffff ffff
##&nbsp;28: ffff ffff ffff ffff
##&nbsp;2c: ffff ffff ffff 0602
##&nbsp;30: ffff ffff ffff ffff
##&nbsp;34: ffff ffff ffff ffff
##&nbsp;38: ffff ffff ffff ffff
##&nbsp;3c: ffff ffff ffff d498
##&nbsp;
##&nbsp;
##&nbsp;-- interface  e0d --
##&nbsp;
##&nbsp;MAC address: 00:50:56:94:3f:82, mediatype: auto-1000t-fd-up
##&nbsp;DBG: if_flags=00e48867, if_d_flags=00000001
##&nbsp;
##&nbsp;Driver name: Gb_Enet-V
##&nbsp;Unicast addresses: 1
##&nbsp;Unicast slots: 15
##&nbsp;Status 0000cb83
##&nbsp;PCI
##&nbsp;PHY 1000T Status 7c00
##&nbsp;PHY Spec Ctrl 0b68
##&nbsp;RxDescriptorPhysical 00003db61000 - 0x000000003db67000
##&nbsp;TxDescriptorPhysical 00003db5e000 - 0x000000003db60000
##&nbsp;
##&nbsp;EEPROM:
##&nbsp;00: 5000 9456 823f 0420
##&nbsp;04: ffff ffff ffff ffff
##&nbsp;08: 0000 0000 460b 0750
##&nbsp;0c: 15ad 100f 8086 01e8
##&nbsp;10: ffff ffff ffff ffff
##&nbsp;14: ffff ffff ffff ffff
##&nbsp;18: ffff ffff ffff ffff
##&nbsp;1c: ffff ffff ffff ffff
##&nbsp;20: dfde 7861 280c 0000
##&nbsp;24: 00c8 ffff ffff ffff
##&nbsp;28: ffff ffff ffff ffff
##&nbsp;2c: ffff ffff ffff 0602
##&nbsp;30: ffff ffff ffff ffff
##&nbsp;34: ffff ffff ffff ffff
##&nbsp;38: ffff ffff ffff ffff
##&nbsp;3c: ffff ffff ffff d398
##&nbsp;
##&nbsp;
##&nbsp;-- interface  lo --
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-- interface  vh --
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-- interface  losk --
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'driver_name' => ' Gb_Enet-V',
##&nbsp;            'unicast_addresses' => ' 1',
##&nbsp;            'mac_address' => ' 00:50:56:94:3f:7f',
##&nbsp;            'mediatype' => ' auto-1000t-fd-up',
##&nbsp;            'if_flags' => '00e48867',
##&nbsp;            'interface' => 'e0a',
##&nbsp;            'if_d_flags' => '00000001',
##&nbsp;            'unicast_slots' => ' 15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'driver_name' => ' Gb_Enet-V',
##&nbsp;            'unicast_addresses' => ' 1',
##&nbsp;            'mac_address' => ' 00:50:56:94:3f:80',
##&nbsp;            'mediatype' => ' auto-1000t-fd-up',
##&nbsp;            'if_flags' => '00e48867',
##&nbsp;            'interface' => 'e0b',
##&nbsp;            'if_d_flags' => '00000001',
##&nbsp;            'unicast_slots' => ' 15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'driver_name' => ' Gb_Enet-V',
##&nbsp;            'unicast_addresses' => ' 1',
##&nbsp;            'mac_address' => ' 00:50:56:94:3f:81',
##&nbsp;            'mediatype' => ' auto-1000t-fd-up',
##&nbsp;            'if_flags' => '00e48867',
##&nbsp;            'interface' => 'e0c',
##&nbsp;            'if_d_flags' => '00000001',
##&nbsp;            'unicast_slots' => ' 15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'driver_name' => ' Gb_Enet-V',
##&nbsp;            'unicast_addresses' => ' 1',
##&nbsp;            'mac_address' => ' 00:50:56:94:3f:82',
##&nbsp;            'mediatype' => ' auto-1000t-fd-up',
##&nbsp;            'if_flags' => '00e48867',
##&nbsp;            'interface' => 'e0d',
##&nbsp;            'if_d_flags' => '00000001',
##&nbsp;            'unicast_slots' => ' 15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'interface' => 'lo'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'interface' => 'vh'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'interface' => 'losk'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifinfo {
	my %args = @_;
	my $resp= $args{response_object};
	my $raw_output = $resp->get_processed_output();
	my $command_line = $resp->get_actual_command();
	my $result;
	_check_error(
			"raw_output"        => $raw_output,
			"command"           => $command_line,
			"response_object"   => $resp
		    );
# Now we can parse the output
	my @array = get_lines(raw_output => $raw_output, response_object => $resp);
	my @output;
	my $i = -1;
	my $xgmac_seen;
	my @head;
	my @values;
	@array = grep ( !(/==.*/), @array);
	@array = grep (!(/EEPROM/),@array);
	foreach my $input (@array) {
		if($input =~ /interface\s+(.*)\s+-/) {
			$i++;
			$output[$i]->{interface} = $1;
		} elsif($input =~ /MAC address:\s+(.*),\s+mediatype:\s+(.*)/) {
			$output[$i]->{mac_address} = $1;
			$output[$i]->{mediatype} = $2;
		} elsif($input =~ /if_flags=(.*),\s+if_d_flags=(.*)/) {
			$output[$i]->{if_flags} = $1;
			$output[$i]->{if_d_flags} = $2;
		} elsif($input =~ /rar used count(.*)/) {
			my $match = $1;
			$match =~ s/[\s+:]//g;
			$output[$i]->{rar_used_count} = $match;
		} elsif($input =~ /MTA used count\s+(.*)/) {
			$output[$i]->{mta_used_count} = $1;
		} elsif($input =~ /number of multicast address in the filter\s+(.*)/) {
			$output[$i]->{number_of_multicast_address_in_the_filter} = $1;
		} elsif ( $input =~ /T3\sadapter\:\s+adapter_index\=(\d+)/i ) {
			$output[$i]->{T3_adapter}->{adapter_index} = $1;
		} elsif ($input  =~ /(.*)\s*\:\s*(.*)/) {
			if(defined $xgmac_seen){
				$output[$i]->{XGMAC}->[$xgmac_seen]->{to_key_name(item => $1)} = $2;
			} else {
				if($input =~/^qla/)
				{
					@head = split(/:\s+/, $input, 2);
					@values = split(/\s+/, $head[1], 2);
					$output[$i]->{to_key_name(item => $head[0]."_".$values[0])} = $values[1];
				}
				@head = split(/:\s*/, $input, 2);
				$output[$i]->{to_key_name(item => $head[0])} = $head[1];
			}
		} elsif ($input  =~ /(.*)\s*\=\s*(.*)/) {
			$output[$i]->{to_key_name(item => $1)} = $2;
		} elsif ($input =~ /XGMAC\[(\d+)\]/){
			$xgmac_seen = $1;
			$output[$i]->{XGMAC}->[$xgmac_seen]->{xgmac_id} = $xgmac_seen;
		} else {
			if ($input=~ /^saved/ || $input=~ /^msix/|| $input=~/^sfp/ || $input =~ /^Status/ ||  $input =~/^PHY/ )
			{
				@head = split(/\s+/, $input, 2);
				$output[$i]->{to_key_name(item => $head[0])} = $head[1];
			} elsif($input=~ /^RxDescriptorPhysical/ || $input=~ /^TxDescriptorPhysical/) {
				@head = split(/ /, $input, 2);
				$output[$i]->{to_key_name(item => $head[0])} = $head[1];
			} elsif($input=~ /^link speed/ ) {
				if($input=~ /(\S+)\s+(\S+)\s+(.*)/){
					$output[$i]->{to_key_name(item => $1."_".$2)} = $3;
				}
			} elsif($input=~ /^cable/){
				@head = split(/\s+/, $input, 2);
				$output[$i]->{to_key_name(item => $head[0])} = $head[1];
			}elsif($input=~ /^(Module Type)\s+(\S+)/) {
				$output[$i]->{to_key_name(item => $1)} =$2;
			}
		}
	}
	return \@output;
}
1;
