#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/ifstat.pm#4  $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ifstat command
##
## @description
##    This is a library to parse the raw output of 7Mode ifstat  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use ifstat ;
##
##    ifstat_ifstat(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ifstat;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::ifstat
##
## @summary Parses the raw output from the "ifstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifstat with any of its options as described in the
##    corresponding cdef file 'ifstat.cdefs'
## @synopsis
##      ifstat(%args);
## INPUTS TO METHOD CALL 'ifstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'clear' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifstat -a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'RSS' => {
##&nbsp;                        'hash_errors' => '0',
##&nbsp;                        'itable_updates' => '0'
##&nbsp;                      },
##&nbsp;             'interface' => 'e5a',
##&nbsp;             'hours' => '16',
##&nbsp;             'TRANSMIT' => {
##&nbsp;                             'mcast_pkts' => '0',
##&nbsp;                             'tso_bytes' => '0',
##&nbsp;                             'queue_overflows' => '62',
##&nbsp;                             'frames_queued' => '0',
##&nbsp;                             'multi_broadcast' => '0',
##&nbsp;                             'jumbo_frames' => '0',
##&nbsp;                             'no_buffers' => '0',
##&nbsp;                             'bus_underruns' => '0',
##&nbsp;                             'errors_minute' => '0',
##&nbsp;                             'hw_udp_cksums' => '0',
##&nbsp;                             'ucast_pkts' => '0',
##&nbsp;                             'total_errors' => '0',
##&nbsp;                             'discards_minute' => '0',
##&nbsp;                             'pause_frames' => '0',
##&nbsp;                             'mac_octets' => '0',
##&nbsp;                             'bytes_second' => '0',
##&nbsp;                             'total_frames' => '0',
##&nbsp;                             'tso_segments' => '0',
##&nbsp;                             'total_bytes' => '0',
##&nbsp;                             'total_discards' => '62',
##&nbsp;                             'hw_tcp_cksums' => '0',
##&nbsp;                             'buffer_coalesces' => '0',
##&nbsp;                             'frames_second' => '0',
##&nbsp;                             'bcast_pkts' => '0',
##&nbsp;                             'mtus_too_big' => '0'
##&nbsp;                           },
##&nbsp;             'RECEIVE' => {
##&nbsp;                            'long_frames' => '0',
##&nbsp;                            'tag_drop' => '0',
##&nbsp;                            'multi_broadcast' => '0',
##&nbsp;                            'jumbo_frames' => '0',
##&nbsp;                            'vlan_unicasts' => '0',
##&nbsp;                            'good_udp_cksum' => '0',
##&nbsp;                            'non-primary_u_c' => '0',
##&nbsp;                            'mac_octets' => '0',
##&nbsp;                            'bad_tcp_cksum' => '0',
##&nbsp;                            'total_frames' => '2',
##&nbsp;                            'total_bytes' => '96',
##&nbsp;                            'total_discards' => '0',
##&nbsp;                            'vlan_broadcasts' => '0',
##&nbsp;                            'bad_tcp6_cksum' => '0',
##&nbsp;                            'redo_udp_cksum' => '0',
##&nbsp;                            'congest_drops' => '0',
##&nbsp;                            'mcast_pkts' => '0',
##&nbsp;                            'good_tcp6_cksum' => '0',
##&nbsp;                            'no_buffers' => '0',
##&nbsp;                            'vlan_forwards' => '0',
##&nbsp;                            'vlan_tag_drop' => '0',
##&nbsp;                            'runt_frames' => '0',
##&nbsp;                            'errors_minute' => '0',
##&nbsp;                            'ucast_pkts' => '0',
##&nbsp;                            'symbol_errors' => '0',
##&nbsp;                            'total_errors' => '0',
##&nbsp;                            'crc_errors' => '0',
##&nbsp;                            'discards_minute' => '0',
##&nbsp;                            'pause_frames' => '0',
##&nbsp;                            'good_tcp_cksum' => '0',
##&nbsp;                            'bytes_second' => '0',
##&nbsp;                            'vlan_untag_drop' => '0',
##&nbsp;                            'redo_tcp_cksum' => '0',
##&nbsp;                            'frames_second' => '0',
##&nbsp;                            'bad_udp_cksum' => '0',
##&nbsp;                            'bcast_pkts' => '0',
##&nbsp;                            'jabber' => '0',
##&nbsp;                            'alignment_errors' => '0',
##&nbsp;                            'bus_overrun' => '0',
##&nbsp;                            'redo_tcp6_cksum' => '0'
##&nbsp;                          },
##&nbsp;             'minutes' => '27',
##&nbsp;             'LINK_INFO' => {
##&nbsp;                              'current_state' => 'down',
##&nbsp;                              'flowcontrol' => 'none',
##&nbsp;                              'speed' => '10000k',
##&nbsp;                              'up_to_downs' => '3',
##&nbsp;                              'duplex' => 'full'
##&nbsp;                            },
##&nbsp;             'days' => '21',
##&nbsp;             'seconds' => '11',
##&nbsp;             'queues' => [
##&nbsp;                           {
##&nbsp;                             'rx_bytes' => '0',
##&nbsp;                             'tx_bytes' => '0',
##&nbsp;                             'queue_id' => '0',
##&nbsp;                             'requeued' => '0',
##&nbsp;                             'tx_frames' => '0',
##&nbsp;                             'rx_frames' => '0'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'rx_bytes' => '0',
##&nbsp;                             'tx_bytes' => '0',
##&nbsp;                             'queue_id' => '1',
##&nbsp;                             'requeued' => '0',
##&nbsp;                             'tx_frames' => '0',
##&nbsp;                             'rx_frames' => '0'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'rx_bytes' => '0',
##&nbsp;                             'tx_bytes' => '0',
##&nbsp;                             'queue_id' => '2',
##&nbsp;                             'requeued' => '0',
##&nbsp;                             'tx_frames' => '0',
##&nbsp;                             'rx_frames' => '0'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'rx_bytes' => '0',
##&nbsp;                             'tx_bytes' => '0',
##&nbsp;                             'queue_id' => '3',
##&nbsp;                             'requeued' => '0',
##&nbsp;                             'tx_frames' => '0',
##&nbsp;                             'rx_frames' => '0'
##&nbsp;                           }
##&nbsp;                         ]
##&nbsp;           }
##&nbsp;];
##&nbsp;
sub ifstat {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'ifstat',
        "ptr_error_message" => \$error_message,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my ( $i, $j, $k, $name, $key, $value );
    my @fields;
    my @lines = get_lines(
        raw_output      => $raw_output,
        first_line      => '\-\-\s+interface\s+\w+',
        can_throw       => 1,
        response_object => $resp
    );
    $i = -1;

    foreach my $line (@lines) {
        if ( $line =~ /^(RECEIVE|TRANSMIT|LINK_INFO|RSS|DEVICE)$/ ) {
            $name = $1;
            next;
        }

        if ( $line =~ /^Queue\s+\d+$/ ) {
            $name = $&;
            $name =~ tr/[A-Z]/[a-z]/;
            $name =~ s/\s+/_/g;
            last;
        }

        if ( $line
            =~ /^...\w+\s+(\S+)\s+.(\d+)\s\w+.\s(\d+)\s\w+.\s(\d+)\s\w+.\s(\d+)\s\w+..../
            ) {
            $i++;
            $result_array[$i]->{'interface'} = $1;
            $result_array[$i]->{'days'}      = $2;
            $result_array[$i]->{'hours'}     = $3;
            $result_array[$i]->{'minutes'}   = $4;
            $result_array[$i]->{'seconds'}   = $5;
        } elsif ( $line
            =~ /^...\w+\s+(\S+)\s+.(\d+)\s\w+.\s(\d+)\s\w+.\s(\d+)\s\w+..../ )
        {
            $i++;
            $result_array[$i]->{'interface'} = $1;
            $result_array[$i]->{'hours'}     = $2;
            $result_array[$i]->{'minutes'}   = $3;
            $result_array[$i]->{'seconds'}   = $4;
        } elsif ( $line =~ /^...\w+\s+(\S+)\s+.(\d+)\s\w+.\s(\d+)\s\w+..../ )
        {
            $i++;
            $result_array[$i]->{'interface'} = $1;
            $result_array[$i]->{'minutes'}   = $2;
            $result_array[$i]->{'seconds'}   = $3;
        } elsif ( $line =~ /^...\w+\s+(\S+)\s+.(\d+)\s\w+..../ ) {
            $i++;
            $result_array[$i]->{'interface'} = $1;
            $result_array[$i]->{'seconds'}   = $2;
        } else {
            $i++ if ( $i == -1 );
            @fields = split( /\|/, $line );
            foreach (@fields) {
                $_ =~ s/\s+$//;
                $_ =~ s/^\s+//;
                $_ =~ /^(.*)\s*:\s*(.*)$/;
                $key = to_key_name( item => $1 );
                $value = $2;
                $result_array[$i]->{"$name"}->{"$key"} = $value;
            }
        }
    }
    @lines = get_lines(
        raw_output      => $raw_output,
        first_line      => "Queue 0",
        can_throw       => 0,
        response_object => $resp
    );
    $j = -1;
    foreach my $line (@lines) {
        if ( $line =~ /^Queue\s+(\d+)$/ ) {
            $j++;
            $result_array[$i]->{"queues"}->[$j]->{"queue_id"} = $1;
        } else {
            @fields = split(/\|/, $line);
            foreach (@fields) {
                $_ =~ s/\s+$//;
                $_ =~ s/^\s+//;
                $_ =~ /^(.*):\s+(.*)$/;
                $key = $1;
                $value = $2;
                $key =~ tr/\/ /__/;
                $key =~ tr/[A-Z]/[a-z]/;
                $result_array[$i]->{"queues"}->[$j]->{"$key"} = $value;
            }
        }
    }
    return \@result_array;
} ## end sub ifstat

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if ( $raw_output =~ /[Uu]sage:/ ) {
        $$error_message = "Required parameter missing";
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /([Nn]o such interface)/ ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    }
    return $error;
} ## end sub _check_error
1;
