#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ifswitch command output
##
## @description
##    This is a library to parse the raw output of 7Mode ifswitch command.
##
## @synopsis
##    use ifswitch;
##
##    ifswitch_stat(%args);
##    ifswitch_mode(%args);
##    ifswitch_read(%args);
##    ifswitch_write(%args);
##    ifswitch_dump(%args);
##
## @author sshaik@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ifswitch;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $resp          = $args{response_object};
    my $error_message;
    my $error;
    if ( $raw_output =~ /ifswitch|[Uu]sage(.*)/ ) {
        $error_message = "Command failed because of invalid usage";
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /Command not supported on this platform./ ) {
        $error_message = $raw_output;
        $error          = "CommandFailed";
      }
      elsif ($raw_output =~ /ERROR : Failed to open.*/i ) {
        $error_message = "Command failed because of invalid Path";
        $error          = "InvalidParam";
    }
    if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
    }
}
## @name ::ifswitch_stat
##
## @summary Parses the raw output from the "ifswitch stat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch_stat with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch_stat(%args);
## INPUTS TO METHOD CALL 'ifswitch_stat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifswitch stat; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;--interface sw-RJ45--
##&nbsp;
##&nbsp;Rx Good Frames	0
##&nbsp;Tx Frames	0
##&nbsp;Rx Discards	0
##&nbsp;link		down
##&nbsp;media		auto-100tx-fd
##&nbsp;
##&nbsp;--interface sw-RLM--
##&nbsp;
##&nbsp;Rx Good Frames	930
##&nbsp;Tx Frames	1865
##&nbsp;Rx Discards	0
##&nbsp;link		up
##&nbsp;media		auto-100tx-fd
##&nbsp;
##&nbsp;--interface sw-PartnerSwitch--
##&nbsp;
##&nbsp;Rx Good Frames	926
##&nbsp;Tx Frames	1869
##&nbsp;Rx Discards	0
##&nbsp;link		up
##&nbsp;media		auto-100tx-fd
##&nbsp;
##&nbsp;--interface sw-e0M--
##&nbsp;
##&nbsp;Rx Good Frames	939
##&nbsp;Tx Frames	1856
##&nbsp;Rx Discards	0
##&nbsp;link		up
##&nbsp;media		auto-100tx-fd
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link' => 'down',
##&nbsp;            'rx_good_frames' => '0',
##&nbsp;            'media' => 'auto-100tx-fd',
##&nbsp;            'interface' => 'sw-RJ45',
##&nbsp;            'rx_discards' => '0',
##&nbsp;            'tx_frames' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'rx_good_frames' => '930',
##&nbsp;            'media' => 'auto-100tx-fd',
##&nbsp;            'interface' => 'sw-RLM',
##&nbsp;            'rx_discards' => '0',
##&nbsp;            'tx_frames' => '1865'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'rx_good_frames' => '926',
##&nbsp;            'media' => 'auto-100tx-fd',
##&nbsp;            'interface' => 'sw-PartnerSwitch',
##&nbsp;            'rx_discards' => '0',
##&nbsp;            'tx_frames' => '1869'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'rx_good_frames' => '939',
##&nbsp;            'media' => 'auto-100tx-fd',
##&nbsp;            'interface' => 'sw-e0M',
##&nbsp;            'rx_discards' => '0',
##&nbsp;            'tx_frames' => '1856'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifswitch_stat {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    my @lines = get_lines( "raw_output" => $output, "response_object" => $resp );
    my (@Res,%hash);
	my $i = -1;
    foreach my $value (@lines) {
	if ($value =~ /.*interface\s+(\S+)--/) {
		$i++;
		$Res[$i]->{'interface'} = $1;
	} elsif ($value =~/(\w+((\s+\w+){1,2})?)\s+(.*)/){
		$Res[$i]->{to_key_name( item => $1 )} = $4;
	}
    }
    return  \@Res;
}
## @name ::ifswitch_mode
##
## @summary Parses the raw output from the "ifswitch mode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch_mode with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch_mode(%args);
## INPUTS TO METHOD CALL 'ifswitch_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'normal' </td> <td> 'normal' </td> <td> '1' </td> </tr>
## <tr> <td> 'error' </td> <td> 'error' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ifswitch mode normal; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ifswitch_mode {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    } else {
    	return [];
    }
}
## @name ::ifswitch_read
##
## @summary Parses the raw output from the "ifswitch read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch_read with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch_read(%args);
## INPUTS TO METHOD CALL 'ifswitch_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'register' </td> <td> '0' </td> </tr>
## <tr> <td> 'port' </td> <td> 'port-register' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-PartnerSwitch' </td> <td> 'sw-partnerswitch' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-RJ45' </td> <td> 'sw-rj45' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-RLM' </td> <td> 'sw-rlm' </td> <td> '1' </td> </tr>
## <tr> <td> 'global1' </td> <td> 'global1-register' </td> <td> '1' </td> </tr>
## <tr> <td> 'phy' </td> <td> 'phy-register' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-e0M' </td> <td> 'sw-e0m' </td> <td> '1' </td> </tr>
## <tr> <td> 'global2' </td> <td> 'global2-register' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ifswitch_read {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    my @lines = get_lines( "raw_output" => $output, "response_object" => $resp );
    my (@Res,%hash);
    foreach my $value (@lines) {
        if ($value =~/\w+\s+(\S+)\s+(\w+)\s+\w+\s+(\S+)\s+=\s+(\S+)/) {
            $Res[0]->{device} = $1;
            $Res[0]->{register_type} = $2;
            $Res[0]->{register} = $3;
            $Res[0]->{value} = $4;
    	} elsif ($value =~/Switch\s+(\w+)\D+(\S+)\s+=\s+(\S+)/) {
            $Res[0]->{register_type} = $1;
            $Res[0]->{register} = $3;
            $Res[0]->{value} = $4;
      }
    }
    return \@Res;
}
## @name ::ifswitch_write
##
## @summary Parses the raw output from the "ifswitch write" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch_write with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch_write(%args);
## INPUTS TO METHOD CALL 'ifswitch_write' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'register' </td> <td> '0' </td> </tr>
## <tr> <td> 'port' </td> <td> 'port-register' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-PartnerSwitch' </td> <td> 'sw-partnerswitch' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-RJ45' </td> <td> 'sw-rj45' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-RLM' </td> <td> 'sw-rlm' </td> <td> '1' </td> </tr>
## <tr> <td> 'global1' </td> <td> 'global1-register' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'data' </td> <td> '0' </td> </tr>
## <tr> <td> 'phy' </td> <td> 'phy-register' </td> <td> '1' </td> </tr>
## <tr> <td> 'sw-e0M' </td> <td> 'sw-e0m' </td> <td> '1' </td> </tr>
## <tr> <td> 'global2' </td> <td> 'global2-register' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ifswitch write sw-RJ45 phy 0x00 0x7b00; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Wrote 0x7b00 to device sw-RJ45 phy register 0x00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'register' => '0x00',
##&nbsp;            'register_type' => 'phy',
##&nbsp;            'register_value' => '0x7b00',
##&nbsp;            'device' => 'sw-RJ45'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifswitch_write {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    my @lines= get_lines( "raw_output" => $output, "response_object" => $resp );
    my (@Res,%hash);
    foreach my $value (@lines) {
        if ($value =~/\w+\s+(\S+)\s+\w+\s+(device|switch)\s+(s\S+)?\s?(\w+)\s+\w+\s+(\S+)/) {
            $Res[0]->{register_value} = $1;
            $Res[0]->{device} = $3 if ($3);
            $Res[0]->{register_type} = $4;
            $Res[0]->{register} = $5;
    	}
    }
    return \@Res;
}
## @name ::ifswitch_dump
##
## @summary Parses the raw output from the "ifswitch dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch_dump with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch_dump(%args);
## INPUTS TO METHOD CALL 'ifswitch_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ifswitch dump; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Marvell 88E6061 PHY registers ...
##&nbsp;	sw-	sw-	sw-		sw-
##&nbsp;	RJ45	RLM	PartnerSwitch	e0M	Unused
##&nbsp;========================================================
##&nbsp;0x0	0xfb00	0x3000	0x3000		0x3000	0x3000
##&nbsp;0x1	0x78c9	0x78ed	0x78ed		0x78ed	0x78c9
##&nbsp;0x2	0x141	0x141	0x141		0x141	0x141
##&nbsp;0x3	0xc89	0xc89	0xc89		0xc89	0xc89
##&nbsp;0x4	0x5e1	0x5e1	0x5e1		0x5e1	0x5e1
##&nbsp;0x5	0x0	0x41e1	0x45e1		0xcde1	0x81
##&nbsp;0x6	0x4	0x5	0x5		0xd	0x4
##&nbsp;0x7	0x2801	0x2801	0x2801		0x2801	0x2001
##&nbsp;0x8	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x9	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xa	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xb	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xc	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xd	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xe	0x0	0x0	0x0		0x0	0x0
##&nbsp;0xf	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x10	0x4130	0x4130	0x4130		0x4130	0x4130
##&nbsp;0x11	0x6000	0x6c40	0x6c00		0x6c40	0x40
##&nbsp;0x12	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x13	0x0	0x0	0x0		0x0	0xf00
##&nbsp;0x14	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x15	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x16	0x4584	0x4584	0x4584		0x4584	0x4584
##&nbsp;0x17	0x30cc	0x30cc	0x30cc		0x30cc	0x30cc
##&nbsp;0x18	0x42bf	0x42bf	0x42bf		0x42bf	0x42bf
##&nbsp;0x19	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x1a	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x1b	0x0	0x0	0x0		0x0	0x0
##&nbsp;0x1c	0x2	0x2	0x2		0x2	0x2
##&nbsp;0x1d	0x8	0x8	0x8		0x8	0x8
##&nbsp;0x1e	0x802a	0x802a	0x802a		0x802a	0x802a
##&nbsp;0x1f	0x0	0x0	0x0		0x0	0x0
##&nbsp;
##&nbsp;
##&nbsp;Marvell 88E6061 Ports/Global registers ...
##&nbsp;	sw-	sw-	sw-		sw-
##&nbsp;	RJ45	RLM	PartnerSwitch	e0M	Unused	Unused	Global2	Global1
##&nbsp;================================================================================
##&nbsp;0x0	0x4b00	0x7b00	0xfb00		0xfb00	0x4800	0xf00	0x0	0x3800
##&nbsp;0x1	0x3	0x3	0x3		0x3	0x3	0x3	0x0	0x0
##&nbsp;0x2	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0x3	0x610	0x610	0x610		0x610	0x610	0x610	0x0	0x0
##&nbsp;0x4	0x7f	0x7f	0x7f		0x7f	0x7f	0x7f	0x0	0x8001
##&nbsp;0x5	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0x6	0x3a	0x3d	0x3a		0x37	0x2f	0x1f	0x0	0x0
##&nbsp;0x7	0x1	0x1	0x1		0x1	0x1	0x1	0x3f	0x0
##&nbsp;0x8	0x80	0x80	0x80		0x80	0x80	0x80	0x0	0x0
##&nbsp;0x9	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0xa	0x8000	0x8000	0x8000		0x8000	0x8000	0x8000	0x0	0x2161
##&nbsp;0xb	0x1	0x2	0x4		0x8	0x10	0x20	0x0	0x0
##&nbsp;0xc	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0xd	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0xe	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0xf	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x0
##&nbsp;0x10	0x0	0x50	0x4e		0x4d	0x0	0x0	0x0	0x0
##&nbsp;0x11	0x0	0x9b	0x9d		0x9e	0x0	0x0	0x0	0x0
##&nbsp;0x12	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x5555
##&nbsp;0x13	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0x5555
##&nbsp;0x14	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0xaaaa
##&nbsp;0x15	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0xaaaa
##&nbsp;0x16	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0xffff
##&nbsp;0x17	0x0	0x0	0x0		0x0	0x0	0x0	0x0	0xffff
##&nbsp;0x18	0x3210	0x3210	0x3210		0x3210	0x3210	0x3210	0x0	0xfa41
##&nbsp;0x19	0x7654	0x7654	0x7654		0x7654	0x7654	0x7654	0x0	0x0
##&nbsp;0x1a	0x9100	0x9100	0x9100		0x9100	0x9100	0x9100	0x0	0x7
##&nbsp;0x1b	0x0	0x1	0x21		0x21	0x0	0x0	0x0	0x106d
##&nbsp;0x1c	0xc8eb	0xc8eb	0xc8eb		0xc8eb	0xc8eb	0xc8eb	0x0	0x0
##&nbsp;0x1d	0x300	0x300	0x300		0x300	0x300	0x300	0x0	0xc00
##&nbsp;0x1e	0x8740	0xc0	0x250		0x20	0x0	0x0	0x0	0x0
##&nbsp;0x1f	0x103f	0x0	0x2000		0xc000	0x1fff	0x80f1	0x0	0x0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'marvell_88E6061_ports/global_registers' => [
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x4b00',
##&nbsp;                                                            'sw_rlm' => '0x7b00',
##&nbsp;                                                            'global1' => '0x3800',
##&nbsp;                                                            'sw_partnerswitch' => '0xfb00',
##&nbsp;                                                            'sequence_number' => '0x0',
##&nbsp;                                                            'sw_e0m' => '0xfb00',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x4800',
##&nbsp;                                                            'global1_unused' => '0xf00'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x3',
##&nbsp;                                                            'sw_rlm' => '0x3',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x3',
##&nbsp;                                                            'sequence_number' => '0x1',
##&nbsp;                                                            'sw_e0m' => '0x3',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x3',
##&nbsp;                                                            'global1_unused' => '0x3'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x2',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x610',
##&nbsp;                                                            'sw_rlm' => '0x610',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x610',
##&nbsp;                                                            'sequence_number' => '0x3',
##&nbsp;                                                            'sw_e0m' => '0x610',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x610',
##&nbsp;                                                            'global1_unused' => '0x610'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x7f',
##&nbsp;                                                            'sw_rlm' => '0x7f',
##&nbsp;                                                            'global1' => '0x8001',
##&nbsp;                                                            'sw_partnerswitch' => '0x7f',
##&nbsp;                                                            'sequence_number' => '0x4',
##&nbsp;                                                            'sw_e0m' => '0x7f',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x7f',
##&nbsp;                                                            'global1_unused' => '0x7f'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x5',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x3a',
##&nbsp;                                                            'sw_rlm' => '0x3d',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x3a',
##&nbsp;                                                            'sequence_number' => '0x6',
##&nbsp;                                                            'sw_e0m' => '0x37',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x2f',
##&nbsp;                                                            'global1_unused' => '0x1f'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x1',
##&nbsp;                                                            'sw_rlm' => '0x1',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x1',
##&nbsp;                                                            'sequence_number' => '0x7',
##&nbsp;                                                            'sw_e0m' => '0x1',
##&nbsp;                                                            'global2' => '0x3f',
##&nbsp;                                                            'global2_unused' => '0x1',
##&nbsp;                                                            'global1_unused' => '0x1'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x80',
##&nbsp;                                                            'sw_rlm' => '0x80',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x80',
##&nbsp;                                                            'sequence_number' => '0x8',
##&nbsp;                                                            'sw_e0m' => '0x80',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x80',
##&nbsp;                                                            'global1_unused' => '0x80'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x9',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x8000',
##&nbsp;                                                            'sw_rlm' => '0x8000',
##&nbsp;                                                            'global1' => '0x2161',
##&nbsp;                                                            'sw_partnerswitch' => '0x8000',
##&nbsp;                                                            'sequence_number' => '0xa',
##&nbsp;                                                            'sw_e0m' => '0x8000',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x8000',
##&nbsp;                                                            'global1_unused' => '0x8000'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x1',
##&nbsp;                                                            'sw_rlm' => '0x2',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x4',
##&nbsp;                                                            'sequence_number' => '0xb',
##&nbsp;                                                            'sw_e0m' => '0x8',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x10',
##&nbsp;                                                            'global1_unused' => '0x20'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0xc',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0xd',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0xe',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0xf',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x50',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x4e',
##&nbsp;                                                            'sequence_number' => '0x10',
##&nbsp;                                                            'sw_e0m' => '0x4d',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x9b',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x9d',
##&nbsp;                                                            'sequence_number' => '0x11',
##&nbsp;                                                            'sw_e0m' => '0x9e',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x5555',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x12',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x5555',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x13',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0xaaaa',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x14',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0xaaaa',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x15',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0xffff',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x16',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0xffff',
##&nbsp;                                                            'sw_partnerswitch' => '0x0',
##&nbsp;                                                            'sequence_number' => '0x17',
##&nbsp;                                                            'sw_e0m' => '0x0',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x3210',
##&nbsp;                                                            'sw_rlm' => '0x3210',
##&nbsp;                                                            'global1' => '0xfa41',
##&nbsp;                                                            'sw_partnerswitch' => '0x3210',
##&nbsp;                                                            'sequence_number' => '0x18',
##&nbsp;                                                            'sw_e0m' => '0x3210',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x3210',
##&nbsp;                                                            'global1_unused' => '0x3210'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x7654',
##&nbsp;                                                            'sw_rlm' => '0x7654',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x7654',
##&nbsp;                                                            'sequence_number' => '0x19',
##&nbsp;                                                            'sw_e0m' => '0x7654',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x7654',
##&nbsp;                                                            'global1_unused' => '0x7654'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x9100',
##&nbsp;                                                            'sw_rlm' => '0x9100',
##&nbsp;                                                            'global1' => '0x7',
##&nbsp;                                                            'sw_partnerswitch' => '0x9100',
##&nbsp;                                                            'sequence_number' => '0x1a',
##&nbsp;                                                            'sw_e0m' => '0x9100',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x9100',
##&nbsp;                                                            'global1_unused' => '0x9100'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x0',
##&nbsp;                                                            'sw_rlm' => '0x1',
##&nbsp;                                                            'global1' => '0x106d',
##&nbsp;                                                            'sw_partnerswitch' => '0x21',
##&nbsp;                                                            'sequence_number' => '0x1b',
##&nbsp;                                                            'sw_e0m' => '0x21',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0xc8eb',
##&nbsp;                                                            'sw_rlm' => '0xc8eb',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0xc8eb',
##&nbsp;                                                            'sequence_number' => '0x1c',
##&nbsp;                                                            'sw_e0m' => '0xc8eb',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0xc8eb',
##&nbsp;                                                            'global1_unused' => '0xc8eb'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x300',
##&nbsp;                                                            'sw_rlm' => '0x300',
##&nbsp;                                                            'global1' => '0xc00',
##&nbsp;                                                            'sw_partnerswitch' => '0x300',
##&nbsp;                                                            'sequence_number' => '0x1d',
##&nbsp;                                                            'sw_e0m' => '0x300',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x300',
##&nbsp;                                                            'global1_unused' => '0x300'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x8740',
##&nbsp;                                                            'sw_rlm' => '0xc0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x250',
##&nbsp;                                                            'sequence_number' => '0x1e',
##&nbsp;                                                            'sw_e0m' => '0x20',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x0',
##&nbsp;                                                            'global1_unused' => '0x0'
##&nbsp;                                                          },
##&nbsp;                                                          {
##&nbsp;                                                            'sw_rj45' => '0x103f',
##&nbsp;                                                            'sw_rlm' => '0x0',
##&nbsp;                                                            'global1' => '0x0',
##&nbsp;                                                            'sw_partnerswitch' => '0x2000',
##&nbsp;                                                            'sequence_number' => '0x1f',
##&nbsp;                                                            'sw_e0m' => '0xc000',
##&nbsp;                                                            'global2' => '0x0',
##&nbsp;                                                            'global2_unused' => '0x1fff',
##&nbsp;                                                            'global1_unused' => '0x80f1'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;            'marvell_88E6061_phy_registers' => [
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x3000',
##&nbsp;                                                   'sequence_number' => '0x0',
##&nbsp;                                                   'sw_rj45' => '0xfb00',
##&nbsp;                                                   'sw_rlm' => '0x3000',
##&nbsp;                                                   'unused' => '0x3000',
##&nbsp;                                                   'sw_partnerswitch' => '0x3000'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x78ed',
##&nbsp;                                                   'sequence_number' => '0x1',
##&nbsp;                                                   'sw_rj45' => '0x78c9',
##&nbsp;                                                   'sw_rlm' => '0x78ed',
##&nbsp;                                                   'unused' => '0x78c9',
##&nbsp;                                                   'sw_partnerswitch' => '0x78ed'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x141',
##&nbsp;                                                   'sequence_number' => '0x2',
##&nbsp;                                                   'sw_rj45' => '0x141',
##&nbsp;                                                   'sw_rlm' => '0x141',
##&nbsp;                                                   'unused' => '0x141',
##&nbsp;                                                   'sw_partnerswitch' => '0x141'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0xc89',
##&nbsp;                                                   'sequence_number' => '0x3',
##&nbsp;                                                   'sw_rj45' => '0xc89',
##&nbsp;                                                   'sw_rlm' => '0xc89',
##&nbsp;                                                   'unused' => '0xc89',
##&nbsp;                                                   'sw_partnerswitch' => '0xc89'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x5e1',
##&nbsp;                                                   'sequence_number' => '0x4',
##&nbsp;                                                   'sw_rj45' => '0x5e1',
##&nbsp;                                                   'sw_rlm' => '0x5e1',
##&nbsp;                                                   'unused' => '0x5e1',
##&nbsp;                                                   'sw_partnerswitch' => '0x5e1'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0xcde1',
##&nbsp;                                                   'sequence_number' => '0x5',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x41e1',
##&nbsp;                                                   'unused' => '0x81',
##&nbsp;                                                   'sw_partnerswitch' => '0x45e1'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0xd',
##&nbsp;                                                   'sequence_number' => '0x6',
##&nbsp;                                                   'sw_rj45' => '0x4',
##&nbsp;                                                   'sw_rlm' => '0x5',
##&nbsp;                                                   'unused' => '0x4',
##&nbsp;                                                   'sw_partnerswitch' => '0x5'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x2801',
##&nbsp;                                                   'sequence_number' => '0x7',
##&nbsp;                                                   'sw_rj45' => '0x2801',
##&nbsp;                                                   'sw_rlm' => '0x2801',
##&nbsp;                                                   'unused' => '0x2001',
##&nbsp;                                                   'sw_partnerswitch' => '0x2801'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x8',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x9',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xa',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xb',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xc',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xd',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xe',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0xf',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x4130',
##&nbsp;                                                   'sequence_number' => '0x10',
##&nbsp;                                                   'sw_rj45' => '0x4130',
##&nbsp;                                                   'sw_rlm' => '0x4130',
##&nbsp;                                                   'unused' => '0x4130',
##&nbsp;                                                   'sw_partnerswitch' => '0x4130'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x6c40',
##&nbsp;                                                   'sequence_number' => '0x11',
##&nbsp;                                                   'sw_rj45' => '0x6000',
##&nbsp;                                                   'sw_rlm' => '0x6c40',
##&nbsp;                                                   'unused' => '0x40',
##&nbsp;                                                   'sw_partnerswitch' => '0x6c00'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x12',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x13',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0xf00',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x14',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x15',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x4584',
##&nbsp;                                                   'sequence_number' => '0x16',
##&nbsp;                                                   'sw_rj45' => '0x4584',
##&nbsp;                                                   'sw_rlm' => '0x4584',
##&nbsp;                                                   'unused' => '0x4584',
##&nbsp;                                                   'sw_partnerswitch' => '0x4584'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x30cc',
##&nbsp;                                                   'sequence_number' => '0x17',
##&nbsp;                                                   'sw_rj45' => '0x30cc',
##&nbsp;                                                   'sw_rlm' => '0x30cc',
##&nbsp;                                                   'unused' => '0x30cc',
##&nbsp;                                                   'sw_partnerswitch' => '0x30cc'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x42bf',
##&nbsp;                                                   'sequence_number' => '0x18',
##&nbsp;                                                   'sw_rj45' => '0x42bf',
##&nbsp;                                                   'sw_rlm' => '0x42bf',
##&nbsp;                                                   'unused' => '0x42bf',
##&nbsp;                                                   'sw_partnerswitch' => '0x42bf'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x19',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x1a',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x1b',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x2',
##&nbsp;                                                   'sequence_number' => '0x1c',
##&nbsp;                                                   'sw_rj45' => '0x2',
##&nbsp;                                                   'sw_rlm' => '0x2',
##&nbsp;                                                   'unused' => '0x2',
##&nbsp;                                                   'sw_partnerswitch' => '0x2'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x8',
##&nbsp;                                                   'sequence_number' => '0x1d',
##&nbsp;                                                   'sw_rj45' => '0x8',
##&nbsp;                                                   'sw_rlm' => '0x8',
##&nbsp;                                                   'unused' => '0x8',
##&nbsp;                                                   'sw_partnerswitch' => '0x8'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x802a',
##&nbsp;                                                   'sequence_number' => '0x1e',
##&nbsp;                                                   'sw_rj45' => '0x802a',
##&nbsp;                                                   'sw_rlm' => '0x802a',
##&nbsp;                                                   'unused' => '0x802a',
##&nbsp;                                                   'sw_partnerswitch' => '0x802a'
##&nbsp;                                                 },
##&nbsp;                                                 {
##&nbsp;                                                   'sw_e0m' => '0x0',
##&nbsp;                                                   'sequence_number' => '0x1f',
##&nbsp;                                                   'sw_rj45' => '0x0',
##&nbsp;                                                   'sw_rlm' => '0x0',
##&nbsp;                                                   'unused' => '0x0',
##&nbsp;                                                   'sw_partnerswitch' => '0x0'
##&nbsp;                                                 }
##&nbsp;                                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifswitch_dump {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    my @lines= get_lines( "raw_output" => $output, "response_object" => $resp );
    my @Res;
    my $i=0;
    my $j=0;
    foreach my $value (@lines) {
        if ($value =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)$/) {
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'sequence_number'} = $1;
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'sw_rj45'} = $2;
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'sw_rlm'} = $3;
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'sw_partnerswitch'} = $4;
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'sw_e0m'} = $5;
            $Res[0]->{'marvell_88E6061_phy_registers'}->[$i]->{'unused'} = $6;
            $i++;
        } elsif ($value =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)$/) {
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'sequence_number'} = $1;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'sw_rj45'} = $2;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'sw_rlm'} = $3;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'sw_partnerswitch'} = $4;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'sw_e0m'} = $5;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'global2_unused'} = $6;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'global1_unused'} = $7;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'global2'} = $8;
            $Res[0]->{'marvell_88E6061_ports/global_registers'}->[$j]->{'global1'} = $9;
            $j++;
        }
    }
    return \@Res;
}

## @name ::ifswitch
##
## @summary Parses the raw output from the "ifswitch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ifswitch with any of its options as described in the
##    corresponding cdef file 'ifswitch.cdefs'
## @synopsis
##      ifswitch(%args);
## INPUTS TO METHOD CALL 'ifswitch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'eeprom' </td> <td> 'eeprom' </td> <td> '1' </td> </tr>
## <tr> <td> 'dump' </td> <td> 'dump' </td> <td> '1' </td> </tr>
## <tr> <td> 'version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> 'up' </td> <td> 'up' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> 'program' </td> <td> 'program' </td> <td> '0' </td> </tr>
## <tr> <td> 'stat' </td> <td> 'stat' </td> <td> '1' </td> </tr>
## <tr> <td> 'reset' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> 'mediatype' </td> <td> 'mediatype' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'switch' </td> <td> '0' </td> </tr>
## <tr> <td> 'down' </td> <td> 'down' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ifswitch -a; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Data update in progress...............
##&nbsp;
##&nbsp;-- switch port 0/0: sw-locked-wrench --
##&nbsp;     link             down
##&nbsp;     speed            10
##&nbsp;     duplex           half
##&nbsp;     flowcontrol      full
##&nbsp;     autonegotiation  incomplete
##&nbsp;
##&nbsp;-- switch port 0/1: sw-wrench --
##&nbsp;     link             up
##&nbsp;     speed            1000
##&nbsp;     duplex           full
##&nbsp;     flowcontrol      full
##&nbsp;     autonegotiation  complete
##&nbsp;
##&nbsp;-- switch port 0/2: sw-e0P --
##&nbsp;     link             down
##&nbsp;     speed            10
##&nbsp;     duplex           half
##&nbsp;     flowcontrol      full
##&nbsp;     autonegotiation  incomplete
##&nbsp;
##&nbsp;-- switch port 0/3: sw-e0M --
##&nbsp;     link             up
##&nbsp;     speed            1000
##&nbsp;     duplex           full
##&nbsp;     flowcontrol      full
##&nbsp;     autonegotiation  complete
##&nbsp;
##&nbsp;-- switch port 0/4: sw-unused --
##&nbsp;     link             down
##&nbsp;     speed            10
##&nbsp;     duplex           half
##&nbsp;     flowcontrol      full
##&nbsp;     autonegotiation  incomplete
##&nbsp;
##&nbsp;-- switch port 0/5: sw-sp-1 --
##&nbsp;     link             up
##&nbsp;     speed            100
##&nbsp;     duplex           full
##&nbsp;     flowcontrol      none
##&nbsp;     autonegotiation  unknown
##&nbsp;
##&nbsp;-- switch port 0/6: sw-sp-2 --
##&nbsp;     link             up
##&nbsp;     speed            100
##&nbsp;     duplex           full
##&nbsp;     flowcontrol      none
##&nbsp;     autonegotiation  unknown
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link' => 'down',
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'speed' => '10',
##&nbsp;            'name' => 'sw-locked-wrench',
##&nbsp;            'port' => '0/0',
##&nbsp;            'autonegotiation' => 'incomplete',
##&nbsp;            'duplex' => 'half'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'speed' => '1000',
##&nbsp;            'name' => 'sw-wrench',
##&nbsp;            'port' => '0/1',
##&nbsp;            'autonegotiation' => 'complete',
##&nbsp;            'duplex' => 'full'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'down',
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'speed' => '10',
##&nbsp;            'name' => 'sw-e0P',
##&nbsp;            'port' => '0/2',
##&nbsp;            'autonegotiation' => 'incomplete',
##&nbsp;            'duplex' => 'half'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'speed' => '1000',
##&nbsp;            'name' => 'sw-e0M',
##&nbsp;            'port' => '0/3',
##&nbsp;            'autonegotiation' => 'complete',
##&nbsp;            'duplex' => 'full'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'down',
##&nbsp;            'flowcontrol' => 'full',
##&nbsp;            'speed' => '10',
##&nbsp;            'name' => 'sw-unused',
##&nbsp;            'port' => '0/4',
##&nbsp;            'autonegotiation' => 'incomplete',
##&nbsp;            'duplex' => 'half'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'flowcontrol' => 'none',
##&nbsp;            'speed' => '100',
##&nbsp;            'name' => 'sw-sp-1',
##&nbsp;            'port' => '0/5',
##&nbsp;            'autonegotiation' => 'unknown',
##&nbsp;            'duplex' => 'full'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'up',
##&nbsp;            'flowcontrol' => 'none',
##&nbsp;            'speed' => '100',
##&nbsp;            'name' => 'sw-sp-2',
##&nbsp;            'port' => '0/6',
##&nbsp;            'autonegotiation' => 'unknown',
##&nbsp;            'duplex' => 'full'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; ifswitch 0/1 stat; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-- switch port 0/1: sw-wrench --
##&nbsp;Data update in progress...............
##&nbsp;     1024ToMaxOctets           0
##&nbsp;     128To255Octets            5981
##&nbsp;     256To511Octets            8960
##&nbsp;     512To1023Octets           18157
##&nbsp;     64Octets                  94387
##&nbsp;     65To127Octets             214
##&nbsp;     InBadOctets               0
##&nbsp;     InBroadcasts              18407
##&nbsp;     InFCSErr                  0
##&nbsp;     InFragments               0
##&nbsp;     InGoodOctetsHi            0
##&nbsp;     InGoodOctetsLo            20069341
##&nbsp;     InJabber                  0
##&nbsp;     InMulticasts              101999
##&nbsp;     InOversize                0
##&nbsp;     InPasue                   0
##&nbsp;     InRxErr                   0
##&nbsp;     InUndersize               0
##&nbsp;     InUnicast                 64
##&nbsp;     OutBroadcasts             4145
##&nbsp;     OutCollisions             0
##&nbsp;     OutExcessive              0
##&nbsp;     OutFCSErr                 0
##&nbsp;     OutMulticasts             2964
##&nbsp;     OutMultiple               0
##&nbsp;     OutOctetsHi               0
##&nbsp;     OutOctetsLo               736356
##&nbsp;     OutPause                  0
##&nbsp;     OutSingle                 0
##&nbsp;     OutUnicast                120
##&nbsp;     Outdeferred               0
##&nbsp;     Outlate                   0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'outmulticasts' => '2964',
##&nbsp;            '256to511octets' => '8960',
##&nbsp;            'outunicast' => '120',
##&nbsp;            '64octets' => '94387',
##&nbsp;            'outexcessive' => '0',
##&nbsp;            'infragments' => '0',
##&nbsp;            '128to255octets' => '5981',
##&nbsp;            'outcollisions' => '0',
##&nbsp;            'outbroadcasts' => '4145',
##&nbsp;            'outlate' => '0',
##&nbsp;            'outdeferred' => '0',
##&nbsp;            'outsingle' => '0',
##&nbsp;            'inundersize' => '0',
##&nbsp;            'inbroadcasts' => '18407',
##&nbsp;            'ingoodoctetslo' => '20069341',
##&nbsp;            'injabber' => '0',
##&nbsp;            'inbadoctets' => '0',
##&nbsp;            'ingoodoctetshi' => '0',
##&nbsp;            'outfcserr' => '0',
##&nbsp;            'outmultiple' => '0',
##&nbsp;            'inoversize' => '0',
##&nbsp;            '65to127octets' => '214',
##&nbsp;            'outoctetshi' => '0',
##&nbsp;            'inrxerr' => '0',
##&nbsp;            'inunicast' => '64',
##&nbsp;            'outpause' => '0',
##&nbsp;            'inmulticasts' => '101999',
##&nbsp;            '1024tomaxoctets' => '0',
##&nbsp;            'infcserr' => '0',
##&nbsp;            'outoctetslo' => '736356',
##&nbsp;            'inpasue' => '0',
##&nbsp;            '512to1023octets' => '18157'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; ifswitch 0 eeprom version; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;switch config revision: Version2 SB_XX Switch0 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'switch_identifier' => 'Switch0',
##&nbsp;            'version' => 'Version2',
##&nbsp;            'board_identifier' => 'SB_XX'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ifswitch {
    my %args = @_;
    my ( $error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my (@result_array, @lines);
    my $i=-1;

    if ($error_level) {
        $error = _check_error(
            raw_output    => $output,
            response_object => $resp,
        );
    }
    @lines= get_lines( "raw_output" => $output, "response_object" => $resp );
    if ($command =~ /-a/){
        foreach my $line (@lines) {
            if ($line =~ /switch port\s+(\d+\/\d+)\:\s+(\S+).*/) {
                $i++;
                $result_array[$i]->{'port'} = $1;
                $result_array[$i]->{'name'} = $2;
            }
            elsif ($line =~ /^(\w+)\s+(\w+)$/){
                $result_array[$i]->{$1} = $2;
            }
        }
    }
    elsif ($command =~ /stat/) {
        foreach my $line (@lines) {
            if ($line =~ /^(\w+)\s+(\w+)$/) {
            $result_array[0]->{to_key_name(item => $1)} = $2;
            }
        }
    }
    elsif ($command =~ /up|down|mediatype|eeprom program|reset/){
        return [];
    }
    elsif ($command =~ /eeprom version/) {
        if ($output =~ /switch config revision\:\s+(\w+)\s+(\S+)\s+(\w+)/){
            $result_array[0]->{'version'} = $1;
            $result_array[0]->{'board_identifier'} = $2;
            $result_array[0]->{'switch_identifier'} = $3;
        }
    }
    elsif ($command =~ /eeprom dump/) {
        foreach my $line(@lines) {
            if($line =~ /\d+/){
               $result_array[0]->{'hex_values'} = $line;
            }
        }
    }
    return \@result_array;
} ## end sub ifswitch

1;
