#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/igroup.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#g
## @summary Routines to parse 7Mode igroup commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode igroup commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "igroup create" will be igroup_create()
##
## @synopsis
##    use igroup;
##
##    igroup_create(%args);
##    igroup_show(%args);
##    igroup_add(%args);
##    igroup_rename(%args);
##    igroup_remove(%args);
##    igroup_set(%args);
##    igroup_destroy(%args);
##    igroup_bind(%args);
##    igroup_unbind(%args);
##
## @author govinda1@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#package NACL::APISet::Node::CLI::7Mode::default::lun;
package NACL::APISet::Node::CLI::7Mode::default::igroup;

use strict;
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::igroup_create
##
## @summary Parses the raw output from the "igroup create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_create with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_create(%args);
## INPUTS TO METHOD CALL 'igroup_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-a' </td> <td> 'port-set' </td> </tr>
## <tr> <td> '-t' </td> <td> 'os-type' </td> </tr>
## <tr> <td> '-i' </td> <td> 'initiator-type-iscsi' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> '-f' </td> <td> 'initiator-type-fcp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'node' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup create -f -t linux igroup1 01:02:03:04:05:06:07:08; result -l
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_create {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my %temp;
    my @return_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "igroup create",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } elsif($raw_output eq "") {
         @return_array = ();
         return \@return_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_show
##
## @summary Parses the raw output from the "igroup show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_show with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_show(%args);
## INPUTS TO METHOD CALL 'igroup_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> </tr>
## <tr> <td> '-a' </td> <td> 'display-stat' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interval' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose-listing' </td> </tr>
## <tr> <td> '-t' </td> <td> 'info-throttles' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup show -v igroup1; result -l
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'initiator_group_os' => 'linux',
##&nbsp;             'report_scsi_name_inquiry_descriptor' => 'Yes',
##&nbsp;             'initiator_group_type' => 'FCP',
##&nbsp;             'initiator_group_uuid' => '44f09d4b-ae2a-11e0-ba4e-123478563412',
##&nbsp;             'initiator_group_member' => [
##&nbsp;                                           '01:02:03:04:05:06:07:08'
##&nbsp;                                         ],
##&nbsp;             'initiator_group' => 'igroup1'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub igroup_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my (@temp, $i);
    my @return_array;

    # Now check for any error in command execution
    my $error = _check_error(
        raw_output        => $raw_output,
        command           => "igroup show",
        ptr_error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(raw_output      => $raw_output,
                          can_throw       => 0,
                          response_object => $resp);
     $i = -1;
     my $j;
     if (    $command =~ /igroup\s+show/
        and $command !~ /igroup\s+show\s+-t/
        and $command !~ /igroup\s+show\s+-v/)
     {
        foreach my $line (@lines) {
            if ($line =~ /(\S+)\s+\((\w+)\)\s+\(\w+:\s+(\w+)\):/) {
                $i++;
                $j = 0;
                $temp[$i]->{initiator_group} = $1;
                $temp[$i]->{initiator_group_type} = $2;
                $temp[$i]->{initiator_group_os} = $3;
                next;
            }
            if ($line =~ /(.*)\s+\(.*/) {
                $temp[$i]->{initiator_group_node}->[$j++] = $1;
            } elsif ($line =~ /Pset:\s+(\w+)/) {
                $temp[$i]->{port_set} = $1;
	        }
        } ## foreach my $line (@lines)...
    } elsif ($command =~ /igroup\s+show\s+-v/){
        $i = -1;
        foreach my $line (@lines) {
            if ($line =~ /(\S+)\s+\((\w+)\):/) {
                $i++;
                $j = 0;
                $temp[$i]->{initiator_group} = $1;
                $temp[$i]->{initiator_group_type} = $2;
                } elsif ($line =~ /OS Type:\s+(\w+)/) {
                    $temp[$i]->{initiator_group_os} = $1;
                } elsif ($line =~ /UUID:\s+(.*)/) {
                    $temp[$i]->{initiator_group_uuid} = $1;
                } elsif ($line =~ /Member:\s+(.*)\s+\(.*/) {
                    $temp[$i]->{initiator_group_member}->[$j++] = $1;
                } elsif ($line =~ /Report SCSI Name in Inquiry Descriptor: (\w+)/){
                    $temp[$i]->{report_scsi_name_inquiry_descriptor} = $1;
                } elsif ($line =~ /Pset:\s+(\w+)/) {
				    $temp[$i]->{port_set} = $1;
			    }
        } ## end of foreach my $line (@lines)...
    } elsif($command =~ /igroup\s+show\s+-t/){
        $i = -1;
        foreach my $line (@lines) {
            if ($line =~ /(\S+)\s+(\d+%)/) {
                $i++;
                $temp[$i]->{initiator_group} = $1;
                $temp[$i]->{initiator_group_reserved} = $2;
            }
        } ## foreach my $line (@lines)..
    }

    return \@temp;

} ## end of sub igroup_show...

## @name ::igroup_add
##
## @summary Parses the raw output from the "igroup add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_add with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_add(%args);
## INPUTS TO METHOD CALL 'igroup_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'node' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup add igroup1 02:03:44:04:05:06:07:08; result -l
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_add {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    $error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup add",
            ptr_error_message => \$error_message
        );
    if ($error){
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_set
##
## @summary Parses the raw output from the "igroup set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_set with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_set(%args);
## INPUTS TO METHOD CALL 'igroup_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'attribute' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'attribute-value' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup set igroup1 throttle_borrow yes
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_set {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup set",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_remove
##
## @summary Parses the raw output from the "igroup remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_remove with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_remove(%args);
## INPUTS TO METHOD CALL 'igroup_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'node' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup remove -f igroup1 02:03:44:04:05:06:07:08
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_remove {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup remove",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_rename
##
## @summary Parses the raw output from the "igroup rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_rename with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_rename(%args);
## INPUTS TO METHOD CALL 'igroup_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-igroup-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup rename igroup1 newname
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_rename {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup rename",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_destroy
##
## @summary Parses the raw output from the "igroup destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_destroy with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_destroy(%args);
## INPUTS TO METHOD CALL 'igroup_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup destroy newname
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_destroy {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup destroy",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_bind
##
## @summary Parses the raw output from the "igroup bind" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_bind with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_bind(%args);
## INPUTS TO METHOD CALL 'igroup_bind' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup bind ig ps
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_bind {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup bind",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}

## @name ::igroup_unbind
##
## @summary Parses the raw output from the "igroup unbind" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from igroup_unbind with any of its options as described in the
##    corresponding cdef file 'igroup.cdefs'
## @synopsis
##      igroup_unbind(%args);
## INPUTS TO METHOD CALL 'igroup_unbind' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = igroup unbind ig
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub igroup_unbind {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @result_array;
    if ($error = _check_error(
            raw_output   => $raw_output,
            command      => "igroup unbind",
            ptr_error_message => \$error_message
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  elsif($raw_output eq "") {
         @result_array = ();
         return \@result_array;
    }
       NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp

    );
}
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error;
    if ( $raw_output =~ /igroup\s+create\:/i ) {
        if($raw_output =~ /mutually\s+exclusive/i){
            $error          = "InvalidParamValue";
            $$error_message = "Invalid options";
        } elsif( $raw_output =~ /unrecognized\s+command/i ) {
            $error          = "CommandError";
            $$error_message = "Unrecognized command";
        } elsif( $raw_output =~ /Initiator\s+group\s+already\s+exists/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Initiator group already exists";
        } elsif( $raw_output =~ /Invalid\s+argument\s+with\s+\-t\s+option\:\s+.*/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Invalid Argument is passed";
        } elsif( $raw_output =~ /Illegal\s+iSCSI\s+node\s+name\:\s+.*/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Invalid name format";
        } elsif( $raw_output =~ /Illegal\s+\world-wide\s+name\s+\(WWN\)\:\s+.*/i ) {
            $error          = "InvalidParamValue";
            $$error_message = "Invalid WWN format is passed";
        } else {
            $error          = "UnexpectedOutput";
            $$error_message = "Unexpected format of output";
        }
    } elsif ( $raw_output =~ /[Uu]sage:/i){
        $error          = "InvalidParam";
        $$error_message = "command failed due to wrong dependicies usage";
    } elsif ( $raw_output =~ /igroup\s+show:\s+No\s+such\s+initiator\s+group\s+exists/) {
	    $error          = "CommandFailed";
	    $$error_message = "igroup show: No such initiator group exists ";
    } elsif ( $raw_output =~ /igroup\s+show:\s+Invalid\s+option/) {
	    $error          = "InvalidParamValue";
	    $$error_message = "Invalid option";
    } elsif ($raw_output =~ /igroup\s+show:\s+Initiator\s+group\s+is\s+not\s+throttled/ig) {
	    $error          = "CommandFailed";
	    $$error_message = "Initiator group is not throttled";
    } elsif ($raw_output =~ /igroup\s+add:\s+Not a valid/){
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /igroup\s+add:\s+((Initiator group already contains node)|((.*) No such initiator group exists))/){
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /igroup\s+set:\s+((Invalid initiator group attribute value for ostype)|(Invalid argument)|(Throttle reserve exceeds available resources))|(option\s+(.*):)/){
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /igroup\s+set:\s+((iSCSI initiator groups cannot be throttled)|(Initiator group is not throttled)|(ALUA is not supported on FCP igroups when not clustered))/){
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /igroup\s+remove:\s+(.*) Not a valid/){
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /igroup\s+remove:\s+(.*) Initiator group does not contain node/){
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /((igroup\s+rename:)|(igroup\s+destroy:)|(igroup\s+bind:)|(igroup\s+unbind:)\s+(.*) No such initiator group exists)/){
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(igroup bind: \w+: No such initiator group exists)|(igroup bind: \w+: No such portset exists)/){
        $error          = "CommandFailed";
        $$error_message = $&;
    } elsif ($raw_output =~ /(igroup unbind: Initiator group is not bound to a portset)|(igroup unbind: \w+: No such initiator group exists)/){
        $error          = "CommandFailed";
        $$error_message = $&;
    }
    return $error;
}
1;
