#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routines to parse 7Mode inodepath commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode inodepath
##    commands. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "inodepath" will be inodepath().
##
## @synopsis
##    use inodepath;
##    inodepath(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::inodepath;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /(Could not find the specified volume)/s) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(Inode number \d+ is a metafile id|Inode type does not support i2p)/s) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(This option is a placeholder for a future feature)/s) {
        $error = "UnsupportedOption";
        $$error_message = $1;
    } elsif($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }
    return $error;
}

## @name ::inodepath
##
## @summary Parses the raw output from the "inodepath" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from inodepath with any of its options as described in the
##    corresponding cdef file 'inodepath.cdefs'
## @synopsis
##      inodepath(%args);
## INPUTS TO METHOD CALL 'inodepath' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-a' </td> <td> 'all-possible-names' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inum' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'nametype' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'snapid' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapname' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'parent-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'list-file-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'list-path-name' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = inodepath -v vol0 -a -i -n 0x000040
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'list-of-names' => [
##&nbsp;                                  {
##&nbsp;                                    'primary-name' => 'vol0',
##&nbsp;                                    'parent-inode' => '64',
##&nbsp;                                    'parent-dir-cookie' => '0',
##&nbsp;                                    'pathname' => '/vol/vol0'
##&nbsp;                                  },
##&nbsp;                                  {
##&nbsp;                                    'name' => 'volume0',
##&nbsp;                                    'parent-inode' => '64',
##&nbsp;                                    'parent-dir-cookie' => '1',
##&nbsp;                                    'pathname' => '/vol/vol0/volume0'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;             'volume-uuid' => 'b9a72370-5880-11e0-a7af-000c296b2e1c',
##&nbsp;             'volume' => 'vol0',
##&nbsp;             'no-of-names' => '1',
##&nbsp;             'inode' => '64',
##&nbsp;             'fsid' => '0x2baf3dd3'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub inodepath {
    my %args = @_;
    my ($error_message, $i, @result_array, @temp);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "inodepath",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my $firstline = 'Inode\s+\d+\s+.*';
    my @lines = get_lines(raw_output => $output, first_line => $firstline,
                    can_throw => 1, response_object => $resp);
    $i = 0;
    foreach my $line (@lines) {
        if ($line =~
        /Inode\s+(\d+)\s+\w+\s+\w+\s+(\w+)\s+[(]\w+\s+(\w+)[)]\s+\w+\s+(\d+)/) {
            $result_array[0] -> {'inode' }      = $1;
            $result_array[0] -> {'volume'}      = $2;
            $result_array[0] -> {'fsid'  }      = $3;
            $result_array[0] -> {'no-of-names'} = $4;
        } elsif ($line =~ /Volume \w+\s+\w+:\s+(.*)/i) {
            $result_array[0] -> {'volume-uuid'} = $1;
        } elsif ($line =~ /\[\s+(\d+)\]\s+\w+\s+\w+\s+=\s+(.*)/i) {
            $temp[$i]->{'primary-name'} = $2;
        } elsif ($line =~ /\[\s+(\d+)\]\s+name\s+=\s+(.*)/i) {
            $i++;
            $temp[$i]->{'name'} = $2;
        } elsif ($line =~ /pathname\s+=\s+(.*)/i) {
            $i++;
            $temp[$i]->{'pathname'} = $1;
        } elsif ($line =~ /\w+\s+=\s+\w+\s+\w+\s+=\s+(\d+)/i) {
            $temp[$i]->{'parent-inode'} = $1;
        } elsif ($line =~ /\w+\s+\w+\s+\w+\s+=\s+(\d+)/i) {
            $temp[$i]->{'parent-dir-cookie'} = $1;
        }
    }
    $result_array[0] -> {'list-of-names'} = \@temp;
    return \@result_array;
}
1;
