#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/iofence.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode iofence command output
##
## @description
##    This is a library to parse the raw output of 7Mode iofence command.
##
## @synopsis
##
##    iofence_set(%args);
##    iofence_get(%args);
##    iofence_clear(%args);
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::iofence;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::iofence_set
##
## @summary Parses the raw output from the "iofence set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iofence_set with any of its options as described in the
##    corresponding cdef file 'iofence.cdefs'
## @synopsis
##      iofence_set(%args);
## INPUTS TO METHOD CALL 'iofence_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'inode-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'generation-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iofence set -i 96 -g 261194042 -f 1060
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Success
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'status' => 'Success'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub iofence_set {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    $result_array[0]->{status} = $lines[0];
    return \@result_array;
} ## end sub iofence_set

## @name ::iofence_get
##
## @summary Parses the raw output from the "iofence get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iofence_get with any of its options as described in the
##    corresponding cdef file 'iofence.cdefs'
## @synopsis
##      iofence_get(%args);
## INPUTS TO METHOD CALL 'iofence_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'inode-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'generation-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iofence get -i 96 -g 261194042 -f 1060
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Type=WIOF_TYPE_ONDISK_RW Proto_fmode=1 Refcnt=0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'proto_fmode' => '1',
##&nbsp;             'refcnt' => '0',
##&nbsp;             'type' => 'WIOF_TYPE_ONDISK_RW'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub iofence_get {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    foreach my $line (@lines) {
        my @head = split( /\s+/, $line );
        for ( my $i = 0 ; $i <= $#head ; $i++ ) {
            my @values = split( /=/, $head[$i] );
            $result_array[$count]->{ to_key_name( item =>$values[0]) }
                = $values[1];
        }
        $count++;
    }
    return \@result_array;
} ## end sub iofence_get

## @name ::iofence_clear
##
## @summary Parses the raw output from the "iofence clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iofence_clear with any of its options as described in the
##    corresponding cdef file 'iofence.cdefs'
## @synopsis
##      iofence_clear(%args);
## INPUTS TO METHOD CALL 'iofence_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'inode-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'generation-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fsid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iofence clear -i 96 -g 261194042 -f 1060
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Success
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'status' => 'Success'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub iofence_clear {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    $result_array[0]->{status} = $lines[0];
    return \@result_array;
} ## end sub iofence_clear

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /No\s+active\s+fence|Volume\s+is\s+offline/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Invalid\s+argument|Must\s+specify\s+a\s+valid\s+number/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Usage:\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

