#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routines to parse 7Mode ipspace commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ipspace commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ipspace create" will be ipspace_create()
##
## @synopsis
##    use ipspace;
##
##    ipspace_create(%args);
##    ipspace_destroy(%args);
##    ipspace_assign(%args);
##    ipspace_list();
##
## @author dl-nacl-dev@netapp.com
##
## @status public
package NACL::APISet::Node::CLI::7Mode::default::ipspace;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /(Ipspace.+already exists)/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(interface .+ does not exist)/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(Ipspace name cannot .+)/) {
        $error = "InvalidParamValue";
    } elsif ($output =~ /(ipspace .+ does not exist)/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(interface .+ has IP addresses: cannot change ipspace)/) {
        $error = "CommandFailed";
    } elsif ($output =~ /(Cannot destroy.+)/si) {
        $error = "CommandFailed";
    } elsif($output =~ /(usage.+)/si) {
        $error = "CommandFailed";
        $$error_message = $1;
    }
    $$error_message = $1;
    return $error;
}

## @name ::ipspace_create
##
## @summary Parses the raw output from the "ipspace create" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipspace_create with any of its options as described in the 
##    corresponding cdef file 'ipspace.cdefs'
## @synopsis
##      ipspace_create(%args);
## INPUTS TO METHOD CALL 'ipspace_create' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ipspacename' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'interface-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ipspace create new
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'created',
##&nbsp;            'ipspace-name' => 'new'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ipspace_create {
    my %args = @_;
    my ($error_message, $name, $status, $ipspace, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(raw_output=>$output,command=>"ipspace create",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $firstline = 'Ipspace\s+\"\w+\"\s+\w+';
    my @lines = get_lines(raw_output=>$output, first_line=>$firstline,
                    can_throw=>1, response_object=>$resp);

    if ($lines[0] =~ /\w+\s+\"(.*)\"\s+(\w+)/){
        $name = $1;
    }

    $ipspace->{'ipspace-name'} = $name;
    $ipspace->{'status'} = "created";
    push (@result_array, $ipspace);
    return \@result_array;
}

## @name ::ipspace_destroy
##
## @summary Parses the raw output from the "ipspace destroy" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipspace_destroy with any of its options as described in the 
##    corresponding cdef file 'ipspace.cdefs'
## @synopsis
##      ipspace_destroy(%args);
## INPUTS TO METHOD CALL 'ipspace_destroy' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ipspacename' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ipspace destroy new
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'destroyed',
##&nbsp;            'ipspace-name' => 'new'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ipspace_destroy {
    my %args = @_;
    my ($error_message, $name, $status, $ipspace, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(raw_output=>$output,command=>"ipspace destroy",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $firstline = 'Ipspace\s+\"\w+\"\s+\w+';
    my @lines = get_lines(raw_output=>$output, first_line=>$firstline,
                    can_throw=>1, response_object=>$resp);

    if ($lines[0] =~ /\w+\s+\"(.*)\"\s+(\w+)/){
        $name = $1;
    }

    $ipspace->{'ipspace-name'} = $name;
    $ipspace->{'status'} = "destroyed";

    push (@result_array, $ipspace);
    return \@result_array;
}

## @name ::ipspace_assign
##
## @summary Parses the raw output from the "ipspace assign" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipspace_assign with any of its options as described in the 
##    corresponding cdef file 'ipspace.cdefs'
## @synopsis
##      ipspace_assign(%args);
## INPUTS TO METHOD CALL 'ipspace_assign' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ipspacename' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'interface-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ipspace assign new e0a e0b
##&nbsp;$VAR1 = [];
##&nbsp;
sub ipspace_assign {
    my %args = @_;
    my ($error_message, $ipspace);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(raw_output=>$output,command=>"ipspace assign",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];
}

## @name ::ipspace_list
##
## @summary Parses the raw output from the "ipspace list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ipspace_list with any of its options as described in the 
##    corresponding cdef file 'ipspace.cdefs'
## @synopsis
##      ipspace_list(%args);
## INPUTS TO METHOD CALL 'ipspace_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ipspace list
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'interfaces' => [
##&nbsp;                              'e0a',
##&nbsp;                              'e0b',
##&nbsp;                              'e0c',
##&nbsp;                              'e0d',
##&nbsp;                              'losk',
##&nbsp;                            ],
##&nbsp;            'name' => 'default-ipspace'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'interfaces' => [],
##&nbsp;            'name' => 'new'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ipspace_list {
    my %args = @_;
    my ($error_message, $num, $i, $j, @new2, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(raw_output=>$output, command=>"ipspace list",
                            error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $firstline = 'Number of ipspaces configured:\s+\d+';

    my  @response = get_lines(raw_output=>$output, first_line=>$firstline,
                             can_throw=>1, response_object=>$resp);

    # if ($response[0] =~ /Number of ipspaces configured\:\s+(\d+)/) {
        # $num = $1;
    # }

    my $count = $#response;
    for ($i=1; $i <= $count; $i++) {
        push (@new2, (split(/\(/, $response[$i])));
    }

    my $count1 = $#new2;
    #$ipspace->{'Number of ipspaces configured'} = $num;

    $i = 1;
    $j = 0;
    while ($i <= $count1) {
        my $ipspace;
        $new2[$j] =~ s/\s+$//;
        $ipspace->{'name'} = $new2[$j];
        $new2[$i] =~ s/\)$//;
        $ipspace->{'interfaces'} = [];
        $ipspace->{'interfaces'} = [split (/\s+/ , $new2[$i])] unless $new2[$i] =~ 'no interfaces';
        $i = $i+2;
        $j = $j+2;
        push (@result_array, $ipspace);
    }
    return \@result_array;
}
1;
