# $Id$
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode iscsi commands
##
## @description
##     This is a library to parse the raw output of 7Mode iscsi commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##    use iscsi;
##
##    iscsi_alias(%args);
##    iscsi_initiator_show(%args);
##    iscsi_interface_disable(%args);
##    iscsi_interface_enable(%args);
##    iscsi_interface_show(%args);
##    iscsi_isns_config(%args);
##    iscsi_isns_show(%args);
##    iscsi_isns_start(%args);
##    iscsi_isns_stop(%args);
##    iscsi_isns_update(%args);
##    iscsi_nodename(%args);
##    iscsi_portal_show(%args);
##    iscsi_security_add(%args);
##    iscsi_security_default(%args);
##    iscsi_security_delete(%args);
##    iscsi_security_generate(%args);
##    iscsi_security_show(%args);
##    iscsi_session_show(%args);
##    iscsi_show_initiator(%args);
##    iscsi_start(%args);
##    iscsi_stats(%args);
##    iscsi_status(%args);
##    iscsi_stop(%args);
##    iscsi_tpgroup_add(%args);
##    iscsi_tpgroup_alua_set(%args);
##    iscsi_tpgroup_alua_show(%args);
##    iscsi_tpgroup_create(%args);
##    iscsi_tpgroup_destroy(%args);
##    iscsi_tpgroup_remove(%args);
##    iscsi_tpgroup_show(%args);
##    iscsi_interface_accesslist_show
##    iscsi_interface_accesslist_add
##    iscsi_interface_accesslist_remove
##    iscsi_help(%args);
##    iscsi_interface_help(%args);
##    iscsi_isns_help(%args);
##    iscsi_tpgroup_help(%args);
##    iscsi_portal_help(%args);
##    iscsi_connection_help(%args);
##    iscsi_initiator_help(%args);
##    iscsi_security_help(%args);
##    iscsi_session_help(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::iscsi;

use strict;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub AUTOLOAD(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my %hash;
    my @return_array;
    $hash{'help_output'} = help_output( 'raw_output' => $raw_output );
    push( @return_array, \%hash );
    return \@return_array;
}

sub _iscsi_initiator_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);
    my @return_array;
    if ( $line =~ /^iscsi show initiator:(.*)$/ ) {
        my %warning_hash;
        $line = shift(@lines);
        $warning_hash{'warning'} = $1 . " " . $line;
        push( @return_array, \%warning_hash );
        $line = shift(@lines);
    }
    elsif ( $line =~ /No initiators connected/ ) {
        my %hash;
        $hash{'result'} = $line;
        push( @return_array, \%hash );
        return \@return_array;
    }
    elsif ( $line =~ /^Initiators connected:$/ ) {
        $line = shift(@lines);
        $line = trim_string( string => $line );
        my @keys      = split( / +/, $line );
        my @keys_more = split( /\//, $keys[$#keys] );
        pop(@keys);
        push( @keys, @keys_more );
        foreach my $key (@keys) {
            $key = to_key_name( 'item' => $key );
        }
        while ( $line = shift(@lines) ) {
            my %new_record;
            my @values;
            my $client;
            my $initiator;
            my $i;

            $line =~ s/\)//;
            $line =~ s/\(//;
            if ( $line =~ /(\w+)\s+(\w+)\s+(.*)\/(.*)\/(.*)/ ) {
                $values[0] = trim_string( 'string' => "$1" );
                $values[1] = trim_string( 'string' => "$2" );
                $values[2] = trim_string( 'string' => "$3" );
                $values[3] = trim_string( 'string' => "$4" );
                $values[4] = trim_string( 'string' => "$5" );
            }
            if ( $values[2] =~ /((\w+-?)+)\s+(.*)/ ) {
                $client    = $1;
                $initiator = $3;
            }
            $values[2] = $initiator;
            for ( $i = 0 ; $i <= $#values ; $i++ ) {
                $new_record{ $keys[$i] } = $values[$i];
            }
            my %temphash;
            $temphash{$client} = \%new_record;
            push( @return_array, \%temphash );
        }
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
    return \@return_array;
}
## @name ::iscsi_show_initiator
##
## @summary Parses the raw output from the "iscsi show initiator" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_show_initiator with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_show_initiator(%args);
## INPUTS TO METHOD CALL 'iscsi_show_initiator' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub iscsi_show_initiator {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $command    = $args{command};
    my $resp       = $args{'response_object'};
    my $first_line = "(Initiators|No|iscsi)";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_initiator_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}
## @name ::iscsi_initiator_show
##
## @summary Parses the raw output from the "iscsi initiator show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_initiator_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_initiator_show(%args);
## INPUTS TO METHOD CALL 'iscsi_initiator_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; iscsi initiator show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Initiators connected:
##&nbsp;  TSIH  TPGroup  Initiator/ISID/IGroup
##&nbsp;    18    1003   lnx219-44 (iqn.1994-05.com.redhat:lnx219-44 / 00:02:3d:76:00:00 / lingrp2; lingrp1)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lnx219-44' => {
##&nbsp;                             'igroup' => 'lingrp2; lingrp1',
##&nbsp;                             'initiator' => 'iqn.1994-05.com.redhat:lnx219-44',
##&nbsp;                             'tsih' => '18',
##&nbsp;                             'tpgroup' => '1003',
##&nbsp;                             'isid' => '00:02:3d:76:00:00'
##&nbsp;                           }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iscsi_initiator_show {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(Initiators|No)";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_initiator_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_status(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);
    my @return_array;
    my %hash;
    if ( $line =~ /^iSCSI service is running$/ ) {
        $hash{'status'} = "running";
        push( @return_array, \%hash );
        return \@return_array;
    }
    elsif ( $line =~ /^iSCSI service is not running$/ ) {
        $hash{'status'} = "not running";
        push( @return_array, \%hash );
        return \@return_array;
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
}
## @name ::iscsi_status
##
## @summary Parses the raw output from the "iscsi status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_status with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_status(%args);
## INPUTS TO METHOD CALL 'iscsi_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_status(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iSCSI";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return ( _iscsi_status( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_portal_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line;
    my @return_array;
    while ( $line = shift(@lines) ) {
        if ( $line =~ /^Network portals:$/ ) {
            my %hash;
            my $fields = shift(@lines);
            my @columns = split( /  +/, $fields );
            foreach my $column_name (@columns) {
                $column_name = to_key_name( 'item' => $column_name );
            }
            my $value_line = shift(@lines);
            my @values = split( / +/, $value_line );
            if ( $#columns == $#values ) {
                for ( my $i = 0 ; $i <= $#columns ; $i++ ) {
                    $hash{ $columns[$i] } = $values[$i];
                }
                push( @return_array, \%hash );
            }
            else {
                my $error =
                  'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "Unexpected form of output",
                    response_object => $resp
                );
            }
        }
    }
    return \@return_array;
}
## @name ::iscsi_portal_show
##
## @summary Parses the raw output from the "iscsi portal show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_portal_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_portal_show(%args);
## INPUTS TO METHOD CALL 'iscsi_portal_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_portal_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = "Network";
    my $can_throw  = 0;
    my @return_array;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_portal_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_stats(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my %returnhash;
    my $line       = shift(@lines);
    my $returnflag = 0;
    my @return_array;
    do {

        if ( $line =~ /^iSCSI/ ) {
            my %hash;
            trim( 'string_ptr' => \$line );
            $hash{'statistics_name'} = $line;
            $line = shift(@lines);
            my $data_lines;
            while ( $line !~ /iSCSI/ ) {
                $data_lines .= $line;
                $data_lines .= "|";
                last if ( scalar(@lines) == 0 );
                $line = shift(@lines);
            }
            unshift( @lines, $line ) if ( scalar(@lines) > 0 );
            my @units = split( /\|/, $data_lines );
            foreach my $field (@units) {
                my @field_value = split( /:/, $field );

                trim( 'string_ptr' => \$field_value[0] );
                trim( 'string_ptr' => \$field_value[1] );
                $field_value[0] = to_key_name( 'item' => $field_value[0] );
                $hash{ $field_value[0] } = $field_value[1];
            }
            push( @return_array, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected form of output",
                response_object => $resp
            );
        }
    } while ( $line = shift(@lines) );
    return ( \@return_array );
}
## @name ::iscsi_stats
##
## @summary Parses the raw output from the "iscsi stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_stats with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_stats(%args);
## INPUTS TO METHOD CALL 'iscsi_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_stats (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iSCSI";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return ( _iscsi_stats( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_nodename(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);
    my %hash;
    my @return_array;
    if ( $line =~ /^iSCSI target nodename: (.*)$/ ) {
        $hash{'nodename'} = $1;
        push( @return_array, \%hash );
        return ( \@return_array );
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
}

## @name ::iscsi_interface_disable
##
## @summary Parses the raw output from the "iscsi interface disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_interface_disable with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_interface_disable(%args);
## INPUTS TO METHOD CALL 'iscsi_interface_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_interface_disable(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(Disabling|Interface)";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    my @return_array;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my $line;

    while ( $line = shift(@lines) ) {
        if ( $line =~ /Disabling interface (.*)/ ) {
            my %hash;
            $hash{$1} = "disabled";
            push( @return_array, \%hash );
        }
    }
    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@return_array );
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return ( \@return_array );
}
## @name ::iscsi_interface_enable
##
## @summary Parses the raw output from the "iscsi interface enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_interface_enable with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_interface_enable(%args);
## INPUTS TO METHOD CALL 'iscsi_interface_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_interface_enable(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(Enabling|Interface)";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    my @return_array;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my $line;

    while ( $line = shift(@lines) ) {
        if ( $line =~ /Enabling interface (.*)/ ) {
            my %hash;
            $hash{$1} = "enabled";
            push( @return_array, \%hash );
        }
    }
    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@return_array );
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return ( \@return_array );
    }
}

sub _iscsi_session_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);
    my (@return_array,$key ,$session);
    my $count = -1;

    if ( $line =~ /No active sessions/ ) {
        my %hash;
        $hash{'result'} = "no active sessions";
        push( @return_array, \%hash );
        return ( \@return_array );
    }
    else {
        unshift( @lines, $line );
        while (@lines){
            $line  = shift(@lines);
            if ( $line =~ /^Session\s+(\d+)/ ) {
                $session = $1 ;
		$count++;
		$return_array[$count]->{'session'}  = $1 ;
	    }else {
	        if($line =~ /(Initiator\s+Information)/){
	            $key = 'initiator_information';
	            $line = shift(@lines);
		    while($line =~ /(.*):\s+(.*)/){
		        $return_array[$count]->{$key}->[0]->{to_key_name( 'item' => $1)} = $2 ;
		        $line = shift(@lines);
		    }
		}
		if($line =~ /Session\s+Parameters/){
		    $key = 'session_parameters';
	            $line = shift(@lines);
		    while($line =~ /(.*)\=(.*)/) {
                        $return_array[$count]->{$key}->[0]->{to_key_name( 'item' => $1)} = $2 ;
		        $line = shift(@lines);
		    }
		}
	        if($line =~ /Connection\s+Information/){
                    $key = 'connection_information';
                    $line = shift(@lines);
                    if($line =~ /Connection\s+(\d+)/){
		        $return_array[$count]->{$key}->[0]->{connection} = $1 ;
			$line = shift(@lines);
		    }
	            while($line =~ /(.*):\s+(.*)/){
		        $return_array[$count]->{$key}->[0]->{to_key_name( 'item' => $1)} = $2 ;
		        $line = shift(@lines);
	            }
		}
		if($line =~ /Command\s+Information/){
		    $key = 'command_information';
                    my $counter = 0;
		    $line = shift(@lines);
		    while($line =~ /(.*)\=(.*)/){
		        $counter = 1 ;
		        $return_array[$count]->{$key}->[0]->{to_key_name( 'item' => $1)} = $2 ;
		        $line = shift(@lines);
	            }
		    if($counter == 0){
		        $return_array[$count]->{'command_information'} = $line ;
		    }
		}
		unshift @lines,$line if ( $line =~ /^Session\s+(\d+)/ );
	    }
        }
    }
    return \@return_array;
}
## @name ::iscsi_session_show
##
## @summary Parses the raw output from the "iscsi session show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_session_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_session_show(%args);
## INPUTS TO METHOD CALL 'iscsi_session_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_session_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(Session|No)";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_session_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}
## @name ::iscsi_isns_start
##
## @summary Parses the raw output from the "iscsi isns start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_start with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_start(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_isns_start(%) {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $command    = $args{command};
    my $resp       = $args{'response_object'};
    my $first_line = "iscsi";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my %hash;
    my @return_array;
    $hash{'result'} = "service started";
    push( @return_array, \%hash );
    return ( \@return_array );
}
## @name ::iscsi_isns_stop
##
## @summary Parses the raw output from the "iscsi isns stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_stop with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_stop(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_isns_stop(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iscsi";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my %hash;
    my @return_array;
    $hash{'result'} = "service stopped";
    push( @return_array, \%hash );
    return ( \@return_array );
}

sub _iscsi_isns_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my @return_array;
    foreach my $line (@lines) {
        if ( $line =~ /:/ ) {
            my @fields = split( /:/, $line );
            trim( 'string_ptr' => \$fields[0] );
            trim( 'string_ptr' => \$fields[1] );
            $fields[0] = to_key_name( 'item' => $fields[0] );
            my %hash;
            $hash{ $fields[0] } = $fields[1];
            push( @return_array, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected form of output",
                response_object => $resp
            );
        }
    }
    return \@return_array;
}
## @name ::iscsi_isns_show
##
## @summary Parses the raw output from the "iscsi isns show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_show(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_isns_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iSNS";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return ( _iscsi_isns_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}

## @name ::iscsi_isns_update
##
## @summary Parses the raw output from the "iscsi isns update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_update with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_update(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_isns_update(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}
## @name ::iscsi_security_generate
##
## @summary Parses the raw output from the "iscsi security generate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_generate with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_generate(%args);
## INPUTS TO METHOD CALL 'iscsi_security_generate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_security_generate(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "Generated";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my $line = shift(@lines);
    if ( $line =~ /:/ ) {
        my @fields = split( /:/, $line );
        trim( 'string_ptr' => \$fields[0] );
        trim( 'string_ptr' => \$fields[1] );
        $fields[0] = to_key_name( 'item' => $fields[0] );
        my %hash;
        my @return_array;
        $hash{ $fields[0] } = $fields[1];
        push( @return_array, \%hash );
        return ( \@return_array );
    }
    else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
}
## @name ::iscsi_isns_config
##
## @summary Parses the raw output from the "iscsi isns config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_config with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_config(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_isns_config(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}

sub _parse_for_errors(%) {
    my %args         = @_;
    my $raw_output   = $args{'raw_output'};
    my $command      = $args{'command'};
    my $errormsg_ptr = $args{'errormsg_ptr'};
    if ( $raw_output =~ /iscsi: service is not licensed/ ) {
        $$errormsg_ptr = "iscsi service is not licensed";
        return "FeatureNotLicensed";
    }
    elsif (( $raw_output =~ /iSCSI service is not running/ )
        || ( $raw_output =~ /iscsi isns stop: iSNS service already stopped/ ) )
    {
        $$errormsg_ptr = "service already stopped";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /Interface .* already disabled/ ) {
        $$errormsg_ptr = $command . " contains already disabled interface";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /Interface .* already enabled/ ) {
        $$errormsg_ptr = $command . " contains already enabled interface";
        return "CommandFailed";
    }
    elsif (( $raw_output =~ /iSCSI service is already running/ )
        || ( $raw_output =~ /iscsi isns start: iSNS service already started/ ) )
    {
        $$errormsg_ptr = "service is already running";
        return "CommandFailed";
    }
    elsif (( $raw_output =~ /iscsi stats: unexpected parameter/ )
        || ( $raw_output =~ /iscsi isns update: iSNS service not running/ ) )
    {
        $$errormsg_ptr = $command . " contains unexpected parameter";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /iscsi nodename: (.*)/ ) {
        $$errormsg_ptr = $1;
        return "InvalidParamValue";
    }
    elsif (
        ( $raw_output =~ /interface .* unknown/ )
        || ( $raw_output =~
/iscsi isns config: could not configure iSNS server address, Invalid IP address or hostname not found/
        )
        || ( $raw_output =~ /iscsi tpgroup .*: tpgroup .* does not exist/ )
        || ( $raw_output =~ /iscsi tpgroup add: interface .* unknown/ )
      )
    {
        $$errormsg_ptr = $command . " contains invalid parameter value(s).";
        return "InvalidParamValue";
    }
    elsif (
        ( $raw_output =~ /iscsi tpgroup .*: no network interfaces specified/ )
        || ( $raw_output =~ /iscsi tpgroup .*: no tpgroup_name specified/ ) )
    {
        $$errormsg_ptr = $command . " is missing required param(s)";
        return "MissingRequiredParam";
    }
    elsif ( $raw_output =~ /iscsi tpgroup create: invalid tpgtag / ) {
        $$errormsg_ptr = $1;
        return "InvalidParamValue";
    }
    elsif (
           ( $raw_output =~ /iscsi tpgroup create: tpgtag (.*)/ )
        || ( $raw_output =~ /tpgroup .* already exists/ )
        || ( $raw_output =~
            /network interface .* is not a member of tpgroup .*/ )
        || ( $raw_output =~
            /iscsi tpgroup destroy: cannot destroy system default tpgroup/ )
        || ( $raw_output =~
/iscsi tpgroup alua set: ALUA setting .* already established for tpgroup/
        )
      )
    {
        $$errormsg_ptr = $command . " Command Failed.";
        return "CommandFailed";
    }
    elsif (
        ( $raw_output =~ /iscsi isns config: unexpected parameter/ )
        || ( $raw_output =~
/iscsi tpgroup add: network interface .* already a member of tpgroup/
        )
      )
    {
        $$errormsg_ptr = $command . " command falied";
        return "CommandFailed";
    }
    elsif (( $raw_output =~ /iscsi security .*: unexpected parameter/ )
        || ( $raw_output =~ /iscsi isns update: iSCSI service is not running/ )
      )
    {
        $$errormsg_ptr = $command . " command falied";
        return "CommandFailed";
    }
    elsif ( $raw_output =~ /iscsi security .*: invalid/ ) {
        $$errormsg_ptr = $command . " conatins invalid param value(s)";
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~ /iscsi.*: unrecognized command/ ) {
        $$errormsg_ptr = $command . " conatins invalid param value(s)";
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~ /iscsi.*: invalid initiator nodename/ ) {
        $$errormsg_ptr = $command . " conatins invalid param value(s)";
        return "InvalidParamValue";
    }
        elsif ( $raw_output =~
        /iscsi interface accesslist add: (no initiator specified)/ )
    {
        $$errormsg_ptr = $command . $1;
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~
        /iscsi interface accesslist remove: (no initiator specified)/ )
    {
        $$errormsg_ptr = $command . $1;
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~
        /iscsi interface accesslist show: ( invalid initiator nodename .*)/ )
    {
        $$errormsg_ptr = $command . $1;
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~/(Command not found)/)
    {
        $$errormsg_ptr = $command . $1;
        return "CommandNotFound";
    }
    return undef;
}

sub _iscsi_interface_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line;
    my @return_array;
    while ( $line = shift(@lines) ) {
        if ( $line =~ /Interface (.*) (.*)/ ) {
            my %hash;
            $hash{$1} = $2;
            push( @return_array, \%hash );
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected form of output",
                response_object => $resp
            );
        }
    }
    return \@return_array;
}
## @name ::iscsi_stop
##
## @summary Parses the raw output from the "iscsi stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_stop with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_stop(%args);
## INPUTS TO METHOD CALL 'iscsi_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_stop(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iSCSI";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}
## @name ::iscsi_start
##
## @summary Parses the raw output from the "iscsi start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_start with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_start(%args);
## INPUTS TO METHOD CALL 'iscsi_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_start(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "iSCSI";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}
## @name ::iscsi_interface_show
##
## @summary Parses the raw output from the "iscsi interface show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_interface_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_interface_show(%args);
## INPUTS TO METHOD CALL 'iscsi_interface_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_interface_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "Interface";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_interface_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}
## @name ::iscsi_nodename
##
## @summary Parses the raw output from the "iscsi nodename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_nodename with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_nodename(%args);
## INPUTS TO METHOD CALL 'iscsi_nodename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-node-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_nodename (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(iSCSI|iscsi)";
    my $errormsg;
    my $error;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    if ( !$raw_output ) {
        return [];
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => 0
    );
    return ( _iscsi_nodename( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_alias(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);
    my %hash;
    my @returnarray;

    if ( $line =~ /^iSCSI target alias: (.*)$/ ) {
        $hash{'alias'} = $1;
        push( @returnarray, \%hash );
        return ( \@returnarray );
    }
    elsif (( $line =~ /New iSCSI target alias:/ )
        || ( $line =~ /^Clearing iSCSI target alias$/ ) )
    {
        $hash{'result'} = "successfull";
        push( @returnarray, \%hash );
        return ( \@returnarray );
    }
    elsif ( $line =~ /No iSCSI target alias assigned/ ) {
        $hash{'alias'} = "No target alias";
        push( @returnarray, \%hash );
        return ( \@returnarray );
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
}

## @name ::iscsi_alias
##
## @summary Parses the raw output from the "iscsi alias" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_alias with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_alias(%args);
## INPUTS TO METHOD CALL 'iscsi_alias' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'clear' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-alias' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_alias(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "(iSCSI|Clearing|New|No)";
    my $errormsg;
    my $error;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => 0
    );
    return ( _iscsi_alias( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _iscsi_tpgroup_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my $line  = shift(@lines);

    if ( $line =~ /^TPGTag/ ) {
        my @return_array;
        my @keys = split( /  +/, $line );
        foreach my $key (@keys) {
            $key = to_key_name( 'item' => $key );
        }
        foreach my $data_line (@lines) {
            my %hash;
            my @fields = split( /\s{2,}/, $data_line );
            $fields[2] =~ s/[()]//g;
            my @interfaces = split( /[,]\s+/, $fields[2] );
            $hash{ $keys[0] } = $fields[0];
            $hash{ $keys[1] } = $fields[1];
            $hash{ $keys[2] } = [@interfaces];
            push( @return_array, \%hash );
        }
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_tpgroup_show
##
## @summary Parses the raw output from the "iscsi tpgroup show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_show(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iscsi tpgroup show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'member_interfaces' => [
##&nbsp;                                     'e0a',
##&nbsp;                                     'e0b',
##&nbsp;                                     'e0c'
##&nbsp;                                   ],
##&nbsp;            'name' => 'test_group',
##&nbsp;            'tpgtag' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'member_interfaces' => [
##&nbsp;                                     'none'
##&nbsp;                                   ],
##&nbsp;            'name' => 'e0a_default',
##&nbsp;            'tpgtag' => '1000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'member_interfaces' => [
##&nbsp;                                     'none'
##&nbsp;                                   ],
##&nbsp;            'name' => 'e0b_default',
##&nbsp;            'tpgtag' => '1001'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'member_interfaces' => [
##&nbsp;                                     'none'
##&nbsp;                                   ],
##&nbsp;            'name' => 'e0c_default',
##&nbsp;            'tpgtag' => '1002'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'member_interfaces' => [
##&nbsp;                                     'e0d'
##&nbsp;                                   ],
##&nbsp;            'name' => 'e0d_default',
##&nbsp;            'tpgtag' => '1003'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iscsi_tpgroup_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "TPGTag";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_tpgroup_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}
## @name ::iscsi_tpgroup_add
##
## @summary Parses the raw output from the "iscsi tpgroup add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_add with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_add(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_add(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_tpgroup_create
##
## @summary Parses the raw output from the "iscsi tpgroup create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_create with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_create(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-t' </td> <td> 'tpgroup-tag' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_create(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_tpgroup_remove
##
## @summary Parses the raw output from the "iscsi tpgroup remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_remove with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_remove(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_remove(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_tpgroup_destroy
##
## @summary Parses the raw output from the "iscsi tpgroup destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_destroy with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_destroy(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_destroy(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}

sub _iscsi_tpgroup_alua_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $line  = shift(@lines);
    my $resp  = $args{'response_obj'};
    if ( $line =~ /^TPGTag/ ) {
        my @return_array;
        my @keys = split( /  +/, $line );
        foreach my $key (@keys) {
            $key = to_key_name( 'item' => $key );
        }
        foreach my $data_line (@lines) {
            my %hash;
            my @fields = split( / +/, $data_line, 3 );
            if ( scalar(@keys) == scalar(@fields) ) {
                for ( my $i = 0 ; $i < scalar(@keys) ; $i++ ) {
                    $hash{ $keys[$i] } = $fields[$i];
                }
                push( @return_array, \%hash );
            }
        }
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_tpgroup_alua_show
##
## @summary Parses the raw output from the "iscsi tpgroup alua show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_alua_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_alua_show(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_alua_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_alua_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "TPGTag";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_tpgroup_alua_show(
            'lines'        => \@lines,
            'response_obj' => $resp
        )
    );
}
## @name ::iscsi_tpgroup_alua_set
##
## @summary Parses the raw output from the "iscsi tpgroup alua set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_alua_set with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_alua_set(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_alua_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'preference' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-alua-state' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tpgroup-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_tpgroup_alua_set(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "Interface";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}

sub _iscsi_security_show(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my @return_array;
    my %hash;
    my $line = shift(@lines);

    if ( $line =~ /^Default sec is None$/ ) {
        $hash{'default_sec'} = "None";
        push( @return_array, \%hash );
    }
    elsif ( $line =~
/^Default sec is CHAP Local Inbound password:(.*)Inbound username:(.*)Outbound password:(.*)Outbound username:(.*)$/
      )
    {
        $hash{'default_sec'}            = "CHAP";
        $hash{'local_inbound_password'} = $1;
        $hash{'inbound_username'}       = $2;
        $hash{'outbound_password'}      = $3;
        $hash{'outbound_username'}      = $4;
        foreach my $key ( keys(%hash) ) {
            trim( 'string_ptr' => \$hash{$key} );
        }
        push( @return_array, \%hash );
    }
    elsif ( $line =~
/Default sec is CHAP RADIUS Outbound password:(.*) Outbound username:(.*)$/
      )
    {
        $hash{'default_sec'}       = "CHAP";
        $hash{'protocol'}          = "RADIUS";
        $hash{'outbound_password'} = $1;
        $hash{'outbound_username'} = $2;
        foreach my $key ( keys(%hash) ) {
            trim( 'string_ptr' => \$hash{$key} );
        }
        push( @return_array, \%hash );
    }
    elsif ( $line =~ /^Default sec is Deny$/ ) {
        $hash{'default_sec'} = "Deny";
        push( @return_array, \%hash );
    }
    else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
    return ( \@return_array );
}
## @name ::iscsi_security_show
##
## @summary Parses the raw output from the "iscsi security show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_show(%args);
## INPUTS TO METHOD CALL 'iscsi_security_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_security_show(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "Default";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return (
        _iscsi_security_show( 'lines' => \@lines, 'response_obj' => $resp ) );
}
## @name ::iscsi_security_default
##
## @summary Parses the raw output from the "iscsi security default" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_default with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_default(%args);
## INPUTS TO METHOD CALL 'iscsi_security_default' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-p' </td> <td> 'password' </td> </tr>
## <tr> <td> '-s' </td> <td> 'auth-type' </td> </tr>
## <tr> <td> '-m' </td> <td> 'outbound-user-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'radius' </td> </tr>
## <tr> <td> '-o' </td> <td> 'outbound-password' </td> </tr>
## <tr> <td> '-n' </td> <td> 'user-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_security_default(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_security_add
##
## @summary Parses the raw output from the "iscsi security add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_add with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_add(%args);
## INPUTS TO METHOD CALL 'iscsi_security_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-i' </td> <td> 'initiator' </td> </tr>
## <tr> <td> '-p' </td> <td> 'password' </td> </tr>
## <tr> <td> '-s' </td> <td> 'auth-type' </td> </tr>
## <tr> <td> '-m' </td> <td> 'outbound-user-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'radius' </td> </tr>
## <tr> <td> '-o' </td> <td> 'outbound-password' </td> </tr>
## <tr> <td> '-n' </td> <td> 'user-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_security_add(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}
## @name ::iscsi_security_delete
##
## @summary Parses the raw output from the "iscsi security delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_delete with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_delete(%args);
## INPUTS TO METHOD CALL 'iscsi_security_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-i' </td> <td> 'initiator' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub iscsi_security_delete(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(
        text            => "Unexpected form of output",
        response_object => $resp
    );
}

## @name ::iscsi_interface_accesslist_show
##
## @summary Parses the raw output from the "iscsi interface accesslist show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_interface_accesslist_show with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_interface_accesslist_show(%args);
## INPUTS TO METHOD CALL 'iscsi_interface_accesslist_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iscsi interface accesslist show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'access_list' => [
##&nbsp;                               'e0a',
##&nbsp;                               'e0b',
##&nbsp;                               'e0c'
##&nbsp;                             ],
##&nbsp;            'initiator_name' => 'iqn.1986-03.com.sun:01:0003ba29c029.4898d235  '
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'access_list' => [
##&nbsp;                               'e0a'
##&nbsp;                             ],
##&nbsp;            'initiator_name' => 'iqn.1994-05.com.redhat:a135db776c5  '
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iscsi interface accesslist show -a; result -l
##&nbsp;
sub iscsi_interface_accesslist_show {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _iscsi_interface_accesslist_show(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
}    # end sub iscsi_interface_accesslist_show

sub _iscsi_interface_accesslist_show {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    if ( $raw_output =~ /No\s+interface\s+access\s+lists/ ) {
        return \@result_array;
    }

# For 'iscsi interface accesslist show', some output is always expected. So we are making
# $can_throw as 1 so that if the lines array got finally in _get_lines is
# empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );
    my $key1;
    my $key2;

    foreach my $line (@lines) {
        my %output;
        if ( $line =~ /(Initiator Name)\s+(Access List)/ ) {
            $key1 = to_key_name( 'item' => $1 );
            $key2 = to_key_name( 'item' => $2 );
            next;
        }
        elsif ( $line =~ /^(iqn.*)\s{2,}(.*)$/ ) {
            $output{$key1} = $1;
            my @arr1 = split( /[,]\s+/, $2 );
            $output{$key2} = [@arr1];
        }
        push @result_array, \%output;
    }

    return \@result_array;
}

sub iscsi_interface_accesslist_add {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @res;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@res;

}    # end sub iscsi_interface_accesslist_add

sub iscsi_interface_accesslist_remove {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @res;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@res;

}    # end sub iscsi_interface_accesslist_remove

## @name ::iscsi_help
##
## @summary Parses the raw output from the "iscsi help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_help(%args);
## INPUTS TO METHOD CALL 'iscsi_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_help

## @name ::iscsi_interface_help
##
## @summary Parses the raw output from the "iscsi interface help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_interface_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_interface_help(%args);
## INPUTS TO METHOD CALL 'iscsi_interface_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_interface_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_interface_help

## @name ::iscsi_isns_help
##
## @summary Parses the raw output from the "iscsi isns help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_isns_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_isns_help(%args);
## INPUTS TO METHOD CALL 'iscsi_isns_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_isns_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_isns_help

## @name ::iscsi_tpgroup_help
##
## @summary Parses the raw output from the "iscsi tpgroup help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_tpgroup_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_tpgroup_help(%args);
## INPUTS TO METHOD CALL 'iscsi_tpgroup_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_tpgroup_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_tpgroup_help

## @name ::iscsi_portal_help
##
## @summary Parses the raw output from the "iscsi portal help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_portal_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_portal_help(%args);
## INPUTS TO METHOD CALL 'iscsi_portal_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_portal_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_portal_help

## @name ::iscsi_connection_help
##
## @summary Parses the raw output from the "iscsi connection help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_connection_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_connection_help(%args);
## INPUTS TO METHOD CALL 'iscsi_connection_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_connection_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_connection_help

## @name ::iscsi_initiator_help
##
## @summary Parses the raw output from the "iscsi initiator help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_initiator_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_initiator_help(%args);
## INPUTS TO METHOD CALL 'iscsi_initiator_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_initiator_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_initiator_help

## @name ::iscsi_security_help
##
## @summary Parses the raw output from the "iscsi security help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_security_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_security_help(%args);
## INPUTS TO METHOD CALL 'iscsi_security_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iscsi_security_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
} ## end sub iscsi_security_help

## @name ::iscsi_session_help
##
## @summary Parses the raw output from the "iscsi session help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iscsi_session_help with any of its options as described in the
##    corresponding cdef file 'iscsi.cdefs'
## @synopsis
##      iscsi_session_help(%args);
## INPUTS TO METHOD CALL 'iscsi_session_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.


sub iscsi_session_help(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    return [];
}## end sub iscsi_session_help

1;
