#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode label command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode label command.
##
## @synopsis
##    use label;
##    label_examine(%args);
##    label_edit(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::label;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~ /[Uu]sage:/) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ($raw_output =~
            /(Disk \S+ does not exist)|(Disk \w+ is not in maintenance mode)/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::label_examine
##
## @summary Parses the raw output from the "label examine" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from label_examine with any of its options as described in the
##    corresponding cdef file 'label.cdefs'
## @synopsis
##      label_examine(%args);
## INPUTS TO METHOD CALL 'label_examine' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'overall' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = label examine v5.17 -a
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mirror_basesnap.timeval' => '0',
##&nbsp;            'vol_nvgen' => '1396511745',
##&nbsp;            'plex_clean_shutdown' => '0',
##&nbsp;            'mirror_childmap.2.onilne' => '0',
##&nbsp;            'striping_flag' => 'unknown',
##&nbsp;            'mirror_childmap.0.state' => '2',
##&nbsp;            'vol_objversion' => '4',
##&nbsp;            'group_rbnmap.0.rbnstart' => '0',
##&nbsp;            'plex_childMap.0.childid' => '0',
##&nbsp;            'disk_objsize' => '344',
##&nbsp;            'group_nextcid' => '2',
##&nbsp;            'mirror_childmap.2.childID' => '0',
##&nbsp;            'group_rbnmap.1.state' => 'parity',
##&nbsp;            'plex_ctime' => '1316683562',
##&nbsp;            'mirror_childmap.2.state' => '0',
##&nbsp;            'mirror_objsize' => '672',
##&nbsp;            'tricky_heritage' => 'false',
##&nbsp;            'group_nchildren' => '2',
##&nbsp;            'mirror_flags' => '0',
##&nbsp;            'group_ndisks' => '0',
##&nbsp;            'mirror_childmap.1.state' => '0',
##&nbsp;            'tree_resyncsnaptime' => '60',
##&nbsp;            'tree_ndisks' => '2',
##&nbsp;            'tree_rtid' => '0x37471000-11e0e68f-c005ab1-d57bba29',
##&nbsp;            'mirror_objid' => '0',
##&nbsp;            'group_grouptype' => 'RAID4',
##&nbsp;            'plex_childMap.0.state' => '1',
##&nbsp;            'mirror_childName.1.name' => undef,
##&nbsp;            'vol_flags2' => '0',
##&nbsp;            'plex_nchildren' => '1',
##&nbsp;            'disk_drivesize' => '0',
##&nbsp;            'group_rbnmap.0.length' => '256000',
##&nbsp;            'disk_state' => '4',
##&nbsp;            'group_rbnmap.1.objstart' => '0',
##&nbsp;            'label_crc' => '0x1b31b612',
##&nbsp;            'tree_creator' => '-230839287',
##&nbsp;            'vol_waflsize' => '256000',
##&nbsp;            'mirror_childmap.1.onilne' => '0',
##&nbsp;            'mirror_nextsnap.snapid' => '0',
##&nbsp;            'vol_flags' => '6e0381',
##&nbsp;            'vol_raidsize' => '256000',
##&nbsp;            'label_magic' => '0x52414944',
##&nbsp;            'tree_type' => 'RAID4',
##&nbsp;            'disk_blocksize' => '4096',
##&nbsp;            'group_resyncoff' => '-1',
##&nbsp;            'group_rbnmap.0.state' => 'data',
##&nbsp;            'tree_objsize' => '408',
##&nbsp;            'plex_objversion' => '1',
##&nbsp;            'group_fixErr' => '0',
##&nbsp;            'plex_nextcid' => '1',
##&nbsp;            'plex_objsize' => '1320',
##&nbsp;            'group_reconstructOffset' => '0',
##&nbsp;            'group_reconstructDisk' => '-2',
##&nbsp;            'disk_zeroprogress' => '0',
##&nbsp;            'label_flags' => '0x0',
##&nbsp;            'mirror_basesnap.snapid' => '0',
##&nbsp;            'mirror_nextsnap.timeval' => '0',
##&nbsp;            'group_objid' => '0',
##&nbsp;            'tree_objversion' => '2',
##&nbsp;            'ha_policy' => 'cfo',
##&nbsp;            'mirror_childmap.1.childID' => '0',
##&nbsp;            'group_rbnmap.1.childid' => '0',
##&nbsp;            'mirror_childmap.0.onilne' => '1',
##&nbsp;            'disk_uniqueid' => '0xb05e978e1a4fc17b9cf680ad3d353c35',
##&nbsp;            'tree_ordinal_claim' => '1318321099',
##&nbsp;            'tree_name' => 'aggr0',
##&nbsp;            'mirror_nchildren' => '1',
##&nbsp;            'group_rbnmap.1.rbnstart' => '-1',
##&nbsp;            'tree_owner' => '-230839287',
##&nbsp;            'group_rbnmap.1.length' => '256000',
##&nbsp;            'group_rbnmap.0.childid' => '1',
##&nbsp;            'vol_nchildren' => '1',
##&nbsp;            'plex_gen' => '6834',
##&nbsp;            'group_objversion' => '3',
##&nbsp;            'tree_serialno' => '4064128009',
##&nbsp;            'mirror_nextcid' => '2',
##&nbsp;            'tree_createtime' => '1316683562',
##&nbsp;            'group_rbnmap.0.objstart' => '0',
##&nbsp;            'tree_maxgroupsize' => '8',
##&nbsp;            'label_version' => '10',
##&nbsp;            'vol_objid' => '1',
##&nbsp;            'disk_objversion' => '2',
##&nbsp;            'disk_type' => 'FILESYSTEM',
##&nbsp;            'mirror_childName.0.name' => undef,
##&nbsp;            'group_objsize' => '1920',
##&nbsp;            'vol_objsize' => '144',
##&nbsp;            'label_size' => '5504',
##&nbsp;            'plex_time' => '1318321101',
##&nbsp;            'mirror_objversion' => '1',
##&nbsp;            'plex_objid' => '0',
##&nbsp;            'mirror_childName.0.id' => '0',
##&nbsp;            'vol_pseudo_bad_block_count' => '0',
##&nbsp;            'disk_objid' => '1',
##&nbsp;            'mirror_childmap.0.childID' => '0',
##&nbsp;            'mirror_childName.1.id' => '0',
##&nbsp;            'vol_fsid0' => '1320725299',
##&nbsp;            'vol_blocksize' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub label_examine {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'label examine',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array = ();
    my @array;
    my @lines = get_lines(raw_output      => $raw_output,
                          can_throw       => 0,
                          response_object => $resp);
    foreach my $line (@lines) {
        @array = split(/\,/, $line);
    }
    foreach my $val (@array) {
        if ($val =~ /([\_\.\w]+)\=([\w\-\_]+)?/) {
            $result_array[0]->{$1} = $2;
        }
    }
    return \@result_array;
} ## end sub label_examine

## @name ::label_edit
##
## @summary Parses the raw output from the "label edit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from label_edit with any of its options as described in the
##    corresponding cdef file 'label.cdefs'
## @synopsis
##      label_edit(%args);
## INPUTS TO METHOD CALL 'label_edit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'magic' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = label edit \n \n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub label_edit {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'label edit',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub label_edit

## @name ::label_vuinfo
##
## @summary Parses the raw output from the "label vuinfo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from label_vuinfo with any of its options as described in the
##    corresponding cdef file 'label.cdefs'
## @synopsis
##      label_vuinfo(%args);
## INPUTS TO METHOD CALL 'label_vuinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; label vuinfo 0b.1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Disk 0b.1: magic 0x55545259 version 4 label-version 12 time 1337788182 disk id:75:54:a6:35:68:ee:f9:03:fe:bc:c1:0a:48:41:8b:92:
##&nbsp;Raidtree aggr0, ID 0xe596276d-11e1a4ee-34125f88-12345678 type 5 creation time 1337788175 ndisks 3
##&nbsp;ha_policy 1, striping 0 max groupsize 16 cache max groupsize 23 tricky_heritage 0
##&nbsp;cksum type 0x6e0000 nplex 1 plexid 0 ngroup 1 rgid 0
##&nbsp;ndisk 3 rgdn 1 global id 65537 vol_flag 6e0381 3 group_raid_lw_scrub 0x3 cxt_parity 1
##&nbsp;lw_disk_signature 0x0 disk_cksum_type 0x3 cksum_zone_size 0x0 cksum_offset 0x0 vr_start -1 size 925248
##&nbsp;	child ID 2	state 1		objStart 0	vr_start 0	size 925248
##&nbsp;	child ID 1	state 2		objStart 0	vr_start -1	size 925248
##&nbsp;	child ID 0	state 11		objStart 0	vr_start -2	size 925248
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'child_id' => '2',
##&nbsp;            'vr_start' => '0',
##&nbsp;            'size' => '925248',
##&nbsp;            'objstart' => '0',
##&nbsp;            'state' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'child_id' => '1',
##&nbsp;            'vr_start' => '-1',
##&nbsp;            'size' => '925248',
##&nbsp;            'objstart' => '0',
##&nbsp;            'state' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'child_id' => '0',
##&nbsp;            'vr_start' => '-2',
##&nbsp;            'size' => '925248',
##&nbsp;            'objstart' => '0',
##&nbsp;            'state' => '11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lw_disk_signature' => '0x0',
##&nbsp;            'ndisk' => '3',
##&nbsp;            'disk' => '0b.1',
##&nbsp;            'cksum_offset' => '0x0',
##&nbsp;            'striping' => '0',
##&nbsp;            'time' => '1337788182',
##&nbsp;            'group_raid_lw_scrub' => '0x3',
##&nbsp;            'cksum_type' => '0x6e0000',
##&nbsp;            'vr_start' => '-1',
##&nbsp;            'rgdn' => '1',
##&nbsp;            'size' => '925248',
##&nbsp;            'ha_policy' => '1',
##&nbsp;            'rgid' => '0',
##&nbsp;            'plexid' => '0',
##&nbsp;            'disk_id' => '75:54:a6:35:68:ee:f9:03:fe:bc:c1:0a:48:41:8b:92:',
##&nbsp;            'magic' => '0x55545259',
##&nbsp;            'version' => '4',
##&nbsp;            'cksum_zone_size' => '0x0',
##&nbsp;            'cxt_parity' => '1',
##&nbsp;            'tricky_heritage' => '0',
##&nbsp;            'vol_flag' => '6e0381 3',
##&nbsp;            'nplex' => '1',
##&nbsp;            'disk_cksum_type' => '0x3',
##&nbsp;            'global_id' => '65537',
##&nbsp;            'label_version' => '12',
##&nbsp;            'ngroup' => '1',
##&nbsp;            'cache_max_groupsize' => '23',
##&nbsp;            'max_groupsize' => '16'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub label_vuinfo {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'label vuinfo',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result;
    my %hash;
    my ($i, $key, $val) = 0;
    my @lines = get_lines(raw_output      => $raw_output,
                          can_throw       => 0,
                          response_object => $resp);
    for ($i = 0; $i<=$#lines; $i++) {
	if($lines[$i] =~ /(Disk)\s+(\S+):\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(disk\s+id:)(.*)/
	|| $lines[$i] =~ /(ndisk)\s+(\S+)\s+(\S+)\s+(\S+)\s+(global\s+id)\s+(\S+)\s+(\S+)\s+(\S+\s+\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/
	|| $lines[$i] =~ /(lw_disk_signature)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/) {
	    $hash{to_key_name(item => $1)} = $2;
	    $hash{to_key_name(item => $3)} = $4;
	    $hash{to_key_name(item => $5)} = $6;
	    $hash{to_key_name(item => $9)} = $10;
	    $hash{to_key_name(item => $11)} = $12;
            $key = $7;
            $val = $8;
            $key =~ s/-/_/;
	    $hash{$key} = $val;
	} elsif($lines[$i] =~ /(Raidtree)\s+(.*),\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(creation\s+time)\s+(\S+)\s+(\S+)\s+(.*)/
	|| $lines[$i] =~ /(ha_policy)\s+(.*),\s+(\S+)\s+(\S+)\s+(max\s+groupsize)\s+(\S+)\s(\S+\s+\S+\s+\S+)\s+(\S+)\s+(\S+)\s(.*)/
	|| $lines[$i] =~ /(cksum\s+type)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
	    $hash{to_key_name(item => $1)} = $2;
	    $hash{to_key_name(item => $3)} = $4;
	    $hash{to_key_name(item => $5)} = $6;
	    $hash{to_key_name(item => $7)} = $8;
	    $hash{to_key_name(item => $9)} = $10;
	} elsif($lines[$i] =~ /(child\s+ID)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/) {
	    my %temp;
	    $temp{to_key_name(item => $1)} = $2;
	    $temp{to_key_name(item => $3)} = $4;
	    $temp{to_key_name(item => $5)} = $6;
	    $temp{to_key_name(item => $7)} = $8;
	    $temp{to_key_name(item => $9)} = $10;
	    push (@result, \%temp);
	}
    }
    push (@result, \%hash);
    return \@result;
} ## end sub label_vuinfo

1;
