#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode labelmaint commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode labelmaint commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "labelmaint isolate" will be labelmaint_isolate()
##
## @synopsis
##    use labelmaint;
##
##    labelmaint_isolate(%args);
##    labelmaint_list(%args);
##    labelmaint_unisolate(%args);
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::labelmaint;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::labelmaint_isolate
##
## @summary Parses the raw output from the "labelmaint isolate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from labelmaint_isolate with any of its options as described in the
##    corresponding cdef file 'labelmaint.cdefs'
## @synopsis
##      labelmaint_isolate(%args);
## INPUTS TO METHOD CALL 'labelmaint_isolate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr-list' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = labelmaint isolate v4.22 aggr_label
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'labelmaint_isolate_status' => 'done',
##&nbsp;            'isolated_disk' => [
##&nbsp;                                 'v4.22'
##&nbsp;                               ],
##&nbsp;            'num_disk_and_aggr' => '2',
##&nbsp;            'isolated_aggr' => [
##&nbsp;                                 'aggr_label'
##&nbsp;                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub labelmaint_isolate {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $can_throw = 1;
    my @labelmaint;
    my %info = ();
    my $error = _check_error(raw_output=>$output,command=>"labelmaint isolate",
                             error_message=>\$error_message);

    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $can_throw = 0;
    }
    my $value;
    my @arr = get_lines(raw_output=>$output, can_throw=>$can_throw,
                         response_object=>$resp);
    my $i=0;
    my $j=0;
    my @iso_arr;
    my @disk;
    my @aggr;
    foreach $value (@arr) {
        if($value =~ /^[Ii]solated\s+(.*)/) {
            my $regex = $1;
            if ( $regex =~ /(\w+\.\d+)/) {
                $disk[$i] = $regex;
                $i++;
            } else {
                $aggr[$j] = $regex;
                $j++;
            }
        } elsif($value =~ /^labelmaint\s+isolate\s+\w+\s+(\d+)\s+.*\s+(\w+)/) {
            $info{"labelmaint_isolate_status"} = $2;
            $info{"num_disk_and_aggr"}      = $1;
        } elsif(($value =~ /^Disk\s+.*/) | ($value =~ /^Aggregate\s+.*/)) {
            next;
        } elsif($value =~ /^Only\s+non\-filesystem\s+disks\s+or\s+offline.*/i) {
            next;
        } 
    }
    if(@disk > 0) {
        $info{"isolated_disk"} = \@disk;
    }
    if(@aggr > 0) {
        $info{"isolated_aggr"} = \@aggr;
    }
    push(@labelmaint,\%info);

    if($error) {
        $resp->set_parsed_output(parsed_output=>\@labelmaint);
        $error->throw(text=>$error_message,response_object => $resp);
    }
    return \@labelmaint;
}

## @name ::labelmaint_list
##
## @summary Parses the raw output from the "labelmaint list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from labelmaint_list with any of its options as described in the
##    corresponding cdef file 'labelmaint.cdefs'
## @synopsis
##      labelmaint_list(%args);
## INPUTS TO METHOD CALL 'labelmaint_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = labelmaint list
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'num_disks_in_maintenance' => '4',
##&nbsp;            'disks_in_maintenance_mode' => [
##&nbsp;                                             'v5.18',
##&nbsp;                                             'v5.19',
##&nbsp;                                             'v5.20',
##&nbsp;                                             'v5.22'
##&nbsp;                                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub labelmaint_list {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @labelmaint;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "labelmaint list",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @mode;
    my $value;
    my $i=0;
    my %info = ();
    if($output =~ /([Nn]o\s+disks\s+in\s+online\s+.*)/){
        $info{"num_disk_in_maintenance"} = $1;
    } elsif ($output =~ /Disks\s+in\s+Online\s+Maintenance\s+Mode:/) {
        my $first_line = 'Disks\s+in\s+Online\s+Maintenance\s+Mode:';
        my @arr = get_lines("raw_output"=>$output, "first_line"=>$first_line,
                            "response_object"=>$resp);
        shift(@arr); #Remove Heading line
        foreach $value (@arr) {
            if($value =~ /(\w+\.\d+)/) {
                $mode[$i] = $1;
                $i++;
            } elsif($value =~ /(\d+)\s+disks\s+in\s+.*/) {
                $info{"num_disks_in_maintenance"} = $1;
            }
        }
        $info{"disks_in_maintenance_mode"} = \@mode;
    }
    push(@labelmaint,\%info);
    return \@labelmaint;
}

## @name ::labelmaint_unisolate
##
## @summary Parses the raw output from the "labelmaint unisolate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from labelmaint_unisolate with any of its options as described in the
##    corresponding cdef file 'labelmaint.cdefs'
## @synopsis
##      labelmaint_unisolate(%args);
## INPUTS TO METHOD CALL 'labelmaint_unisolate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = labelmaint unisolate
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'labelmaint_unisolate_status' => 'done'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub labelmaint_unisolate {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my @labelmaint;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "labelmaint unisolate",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ($output =~ /^[Ll]abelmaint\s+unisolate\s+(\w+)/) {
        $info{"labelmaint_unisolate_status"}     = $1;
    }
    push( @labelmaint, \%info );
    return \@labelmaint;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ($raw_output =~ /Usage/i) {
        $error          = "InvalidParam";
        $$error_message = "Invalid Parameter Passed";
    } elsif ($raw_output =~ /(^No\s+disk\s+or\s+aggregate.*)/i) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(^Disk\s+.*) | (^Aggregate\s+.*)/i) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}
1;
