#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode license commands
##
## @description
##    This is a library to parse the raw output of 7Mode license commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "license add" will be license_add()
##
## @synopsis
##    use license;
##
##    license(%args);
##    license_add(%args);
##    license_delete(%args);
##    license_help(%args);
##    license_show(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com, pbala@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::license;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command=$args{command};
    my $errormsg_ptr=$args{'errormsg_ptr'};

    if($raw_output=~/license delete: invalid service .*, does not match any known service./) {
        $$errormsg_ptr=$command." contains invalid param value(s)";
        return "InvalidParamValue";
    } elsif($raw_output=~/There is no need to install a Demo License/i) {
        $$errormsg_ptr=$raw_output;
        return "InvalidParamValue";
    } elsif($raw_output=~/license add: invalid license code .*, license codes must be 7 letters/) {
        $$errormsg_ptr=$command." contains invalid param value(s)";
        return "InvalidParamValue";
    } elsif ($raw_output=~/invalid platform/){
        $$errormsg_ptr=$command." command failed as it is invalid platform";
        return "CommandFailed";
    } elsif ($raw_output=~/is an included feature and cannot be deleted/){
        $$errormsg_ptr=$command." command failed as it is an included feature and cannot be deleted";
        return "CommandFailed";
    }elsif ($raw_output=~/is included and no license code is necessary/){
        $$errormsg_ptr=$command." command failed as it is included and no license code is necessary";
        return "CommandFailed";
    }elsif ($raw_output=~/license: syntax error./){
        $$errormsg_ptr=$command." contains invalid param value(s)";
        return "InvalidParamValue";
    } elsif ($raw_output=~/license: service for license code: .*, does not match specified service name: .*/){
        $$errormsg_ptr=$command." contains invalid param value(s)";
        return "InvalidParamValue";
    } elsif ($raw_output=~/license:\s+the\s+'license\s+<service>=<code>'\s+syntax\s+is\s+no\s+longer\s+supported/){
        $$errormsg_ptr=$command." command failed";
        return "CommandFailed";
    } elsif ($raw_output =~ /(license [Uu]sage.*|.*invalid license key.*|.*unknown license or package.*)/) {
        $$errormsg_ptr =$command.$1;
#       BURT 672391
#       Product output:
#       license add: option "-license-code" is unnecessary, skipping.
#       license add: invalid license key "GCKYSWDUSAFDAAAAAAAAAAAAAAAA", reason: "License for package "FlexClone" is already installed. There is no need to install a Site License.", skipping.
        return undef if ($raw_output =~ /is already installed|no need to install a Site License/);
        return "InvalidParamValue";
    } elsif ($raw_output=~/.*failed to delete.*(reason:\s+\"entry doesn't exist\")/){
        $$errormsg_ptr=$command." $1";
        return "CommandFailed";
    }

    return undef;
}
sub _license(%){
    my %args = @_;
    my @lines =@{$args{'lines'}};
    my @returnarray;
    my $resp=$args{'response_object'};
    while(my $line = shift(@lines)){
        if($line=~/^(\S*) (.*)$/){
            my %hash;
            $hash{$1}=$2;
            $hash{"service"} = $1;
            $hash{"is_licensed"} = ($line=~/not licensed/) ? "false" : "true";
            $hash{"is_site"} = ($line=~/site/i) ? "true" : "false";
            $hash{"is_demo"} = ($line=~/demo/i) ? "true" : "false";
            if (($hash{"is_licensed"} eq "false") || ($line=~/ENABLED/)) {
                $hash{"license_code"} = "";
            } else {
                $line=~/([A-Z]+)/;
                $hash{"license_code"} = $1;
            }
            push(@returnarray,\%hash);
        }else{
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"unexpected format of output",response_object => $resp);
        }
    }
    return \@returnarray;
}
sub _license_v2(%){
    my %args = @_;
    my @lines =@{$args{'lines'}};
    my @returnarray;
    my $i=0;
    my $resp=$args{'response_object'};
    while(my $line = shift(@lines)){
        if($line =~ /(\w+\s\w+|\w+):\s+(.*)/){
            $returnarray[0]->{to_key_name(item=>$1)}=$2;
        }elsif($line =~ /(\w+\s\w+\s\w+|\w+\s\w+|\w+)\s+(license|site|demo)\s+(\w+\s\w+\s\w+|\w+\s\w+|\w+)\s+(\S+\s\S+|\S+)/){
            $returnarray[0]->{"licenses"}->[$i]->{"package"}= $1;
            $returnarray[0]->{"licenses"}->[$i]->{"type"}= $2;
            $returnarray[0]->{"licenses"}->[$i]->{"description"}= $3;
            $returnarray[0]->{"licenses"}->[$i]->{"expiration"}= $4;
            $i++;
        }
    }
    return \@returnarray;
}
## @name ::license
##
## @summary Parses the raw output from the "license" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from license with any of its options as described in the
##    corresponding cdef file 'license.cdefs'
## @synopsis
##      license(%args);
## INPUTS TO METHOD CALL 'license' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'snapdrive_windows' </td> <td> 'snapdrive_windows' </td> <td> '0' </td> </tr>
## <tr> <td> 'disk_sanitization' </td> <td> 'disk_sanitization' </td> <td> '0' </td> </tr>
## <tr> <td> 'http' </td> <td> 'http' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmanagerexchange' </td> <td> 'snapmanagerexchange' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_windows_pri' </td> <td> 'sv_windows_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'iscsi' </td> <td> 'iscsi' </td> <td> '0' </td> </tr>
## <tr> <td> 'vld' </td> <td> 'vld' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_exchange_pri' </td> <td> 'sv_exchange_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmover' </td> <td> 'snapmover' </td> <td> '0' </td> </tr>
## <tr> <td> 'smdomino' </td> <td> 'smdomino' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmanager_sharepoint' </td> <td> 'snapmanager_sharepoint' </td> <td> '0' </td> </tr>
## <tr> <td> 'smsql' </td> <td> 'smsql' </td> <td> '0' </td> </tr>
## <tr> <td> 'a_sis' </td> <td> 'a_sis' </td> <td> '0' </td> </tr>
## <tr> <td> 'fcp' </td> <td> 'fcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'cf' </td> <td> 'cf' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmanager_hyperv' </td> <td> 'snapmanager_hyperv' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmanager_oracle' </td> <td> 'snapmanager_oracle' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_linux_pri' </td> <td> 'sv_linux_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'provisioning_manager' </td> <td> 'provisioning_manager' </td> <td> '0' </td> </tr>
## <tr> <td> 'operations_manager' </td> <td> 'operations_manager' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmanager_sap' </td> <td> 'snapmanager_sap' </td> <td> '0' </td> </tr>
## <tr> <td> 'flex_scale' </td> <td> 'flex_scale' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_unix_pri' </td> <td> 'sv_unix_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_sql_pri' </td> <td> 'sv_sql_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'nearstore_option' </td> <td> 'nearstore_option' </td> <td> '0' </td> </tr>
## <tr> <td> 'nfs' </td> <td> 'nfs' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_oracle_pri' </td> <td> 'sv_oracle_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'snaplock_enterprise' </td> <td> 'snaplock_enterprise' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_ontap_pri' </td> <td> 'sv_ontap_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'flexcache_nfs' </td> <td> 'flexcache_nfs' </td> <td> '0' </td> </tr>
## <tr> <td> 'cifs' </td> <td> 'cifs' </td> <td> '0' </td> </tr>
## <tr> <td> 'compression' </td> <td> 'compression' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapdrive_unix' </td> <td> 'snapdrive_unix' </td> <td> '0' </td> </tr>
## <tr> <td> 'v-series' </td> <td> 'v-series' </td> <td> '0' </td> </tr>
## <tr> <td> 'syncmirror_local' </td> <td> 'syncmirror_local' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_applications_pri' </td> <td> 'sv_applications_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_sharepoint_pri' </td> <td> 'sv_sharepoint_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_marketing_pri' </td> <td> 'sv_marketing_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'multistore' </td> <td> 'multistore' </td> <td> '0' </td> </tr>
## <tr> <td> 'pamii' </td> <td> 'pamii' </td> <td> '0' </td> </tr>
## <tr> <td> 'flex_clone' </td> <td> 'flex_clone' </td> <td> '0' </td> </tr>
## <tr> <td> 'snaplock' </td> <td> 'snaplock' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_ontap_sec' </td> <td> 'sv_ontap_sec' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_vmware_pri' </td> <td> 'sv_vmware_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmirror_sync' </td> <td> 'snapmirror_sync' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_windows_ofm_pri' </td> <td> 'sv_windows_ofm_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'cf_remote' </td> <td> 'cf_remote' </td> <td> '0' </td> </tr>
## <tr> <td> 'sv_vi_pri' </td> <td> 'sv_vi_pri' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapvalidator' </td> <td> 'snapvalidator' </td> <td> '0' </td> </tr>
## <tr> <td> 'protection_manager' </td> <td> 'protection_manager' </td> <td> '0' </td> </tr>
## <tr> <td> 'snaprestore' </td> <td> 'snaprestore' </td> <td> '0' </td> </tr>
## <tr> <td> 'snapmirror' </td> <td> 'snapmirror' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; license; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Serial Number: 4061502-49-2
##&nbsp;Owner: anupamap-vsim-2
##&nbsp;Package           Type    Description           Expiration
##&nbsp;----------------- ------- --------------------- --------------------
##&nbsp;iSCSI             site    iSCSI License         -
##&nbsp;FCP               site    FCP License           -
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'owner' => 'anupamap-vsim-2',
##&nbsp;            'serial_number' => '4061502-49-2',
##&nbsp;            'licenses' => [
##&nbsp;                            {
##&nbsp;                              'expiration' => '-',
##&nbsp;                              'type' => 'site',
##&nbsp;                              'description' => 'iSCSI License',
##&nbsp;                              'package' => 'iSCSI'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'expiration' => '-',
##&nbsp;                              'type' => 'site',
##&nbsp;                              'description' => 'FCP License',
##&nbsp;                              'package' => 'FCP'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; license; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                 a_sis ENABLED
##&nbsp;                    cf site KDQLGBN
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'is_demo' => 'false',
##&nbsp;            'license_code' => '',
##&nbsp;            'is_site' => 'false',
##&nbsp;            'a_sis' => 'ENABLED',
##&nbsp;            'is_licensed' => 'true',
##&nbsp;            'service' => 'a_sis'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'is_demo' => 'false',
##&nbsp;            'license_code' => 'KDQLGBN',
##&nbsp;            'is_site' => 'true',
##&nbsp;            'is_licensed' => 'true',
##&nbsp;            'cf' => 'site KDQLGBN',
##&nbsp;            'service' => 'cf'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub license(%){
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my @returnarray;
    my $first_line;
    my $can_throw=0;
    $error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg);
    if ($command=~/^license$/){
        if ($raw_output =~ /Serial\s+Number:.*/){
        my @lines=get_lines('raw_output'=>$raw_output);
        return (_license_v2('lines' => \@lines));
        }
        $first_line="a_sis";
        my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
        return (_license('lines'=>\@lines));
    }else{
        $first_line="";
        my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
        my $warning_string;
        while(my $line=shift(@lines)){
            if($line=~/the 'license <service>=<code>' syntax has been deprecated/){
                $warning_string=$line;
            }elsif($line=~/A (.*) site license has been installed./){
                my %hash;
                $hash{$1}="installed";
                $hash{'warning'}=$warning_string if (length($warning_string));
                push(@returnarray,\%hash);
            }
        }
        if($error){
             $resp->set_parsed_output('parsed_output'=>\@returnarray);
             $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
              $error->throw(text=>$errormsg,response_object => $resp);
        }else {
           return \@returnarray;
        }
    }
}#end of funtion license()
## @name ::license_add
##
## @summary Parses the raw output from the "license add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from license_add with any of its options as described in the
##    corresponding cdef file 'license.cdefs'
## @synopsis
##      license_add(%args);
## INPUTS TO METHOD CALL 'license_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'license-v2-code' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'code' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; license add APZXOWDUSAFDAAAAAAAAAAAAAAAA; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;license add: successfully added license key "APZXOWDUSAFDAAAAAAAAAAAAAAAA".
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'license_key' => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
##&nbsp;            'status' => 'successfully added'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; license add CFZRXFF; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;A smdomino site license has been installed.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'smdomino' => 'installed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub license_add (%){
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line="";
    my $can_throw=0;
    my @returnarray;
    $error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg);
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    while(my $line=shift(@lines)){
        if($line=~/A (.*) site license has been installed./){
            my %hash;
            $hash{$1}="installed";
            push(@returnarray,\%hash);
        }elsif($line=~/license\sadd:\s(\S+\s\S+)\s\S+\s\S+\s\"(\S+)\"./){
            $returnarray[0]->{to_key_name(item=>"license_key")}=$2;
            $returnarray[0]->{to_key_name(item=>"status")}=$1;
        } elsif ($line=~/license\sadd:.*invalid license key\s\"(\S+)\".*(is already installed)/) {
#           BURT 672391
#           Product output:
#           license add: option "-license-code" is unnecessary, skipping.
#           license add: invalid license key "GCKYSWDUSAFDAAAAAAAAAAAAAAAA", reason: "License for package "FlexClone" is already installed. There is no need to install a Site License.", skipping.
            $returnarray[0]->{to_key_name(item=>"license_key")}=$1;
            $returnarray[0]->{to_key_name(item=>"status")}=$2;

            #execution_status is set to 3, reset it to null (irregular product output)
            $resp->{execution_status} = "";
        }
    }
    if($error){
        $resp->set_parsed_output('parsed_output'=>\@returnarray);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
         $error->throw(text=>$errormsg,response_object => $resp);
    }else {
        return \@returnarray;    
    }
}#end of funtion license_add()

## @name ::license_delete
##
## @summary Parses the raw output from the "license delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from license_delete with any of its options as described in the
##    corresponding cdef file 'license.cdefs'
## @synopsis
##      license_delete(%args);
## INPUTS TO METHOD CALL 'license_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'package-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'service' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; license delete NFS; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;license delete: successfully deleted "NFS".
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'license_package' => 'NFS',
##&nbsp;            'status' => 'successfully deleted'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; license delete smdomino; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;unlicensed smdomino.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'smdomino' => 'unlicensed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub license_delete (%){
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line="";
    my $can_throw=0;
    my @returnarray;
    $error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg);
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    while(my $line=shift(@lines)){
        if($line=~/unlicensed (.*)./){
            my %hash;
            $hash{$1}="unlicensed";
            push(@returnarray,\%hash);
        }elsif($line=~/license\sdelete:\s(\S+\s\S+)\s\"(\S+)\"/){
            $returnarray[0]->{to_key_name(item=>"license_package")}=$2;
            $returnarray[0]->{to_key_name(item=>"status")}=$1;
        }
    }
    if($error){
        $resp->set_parsed_output('parsed_output'=>\@returnarray);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
         $error->throw(text=>$errormsg,response_object => $resp);
    }else {
        return \@returnarray;    
    }
}

## @name ::license_help
##
## @summary Parses the raw output from the "license help" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from license_help with any of its options as described in the 
##    corresponding cdef file 'license.cdefs'
## @synopsis
##      license_help(%args);
## INPUTS TO METHOD CALL 'license_help' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub license_help(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    
    my %hash;
    my @return_array;
    $hash{'help_output'}=help_output('raw_output'=>$raw_output);
    push(@return_array,\%hash);
    return \@return_array;
}

## @name ::license_show
##
## @summary Parses the raw output from the "license show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from license_show with any of its options as described in the
##    corresponding cdef file 'license.cdefs'
## @synopsis
##      license_show(%args);
## INPUTS TO METHOD CALL 'license_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'yes' </td> <td> 'yes' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'field-name' </td> <td> '0' </td> </tr>
## <tr> <td> 'no' </td> <td> 'no' </td> <td> '0' </td> </tr>
## <tr> <td> '-description' </td> <td> 'description' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## <tr> <td> '-customer-id' </td> <td> 'customer-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'license-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-legacy' </td> <td> 'legacy' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'text' </td> <td> '0' </td> </tr>
## <tr> <td> '-expiration' </td> <td> 'expiration' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; license show -type site; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Serial Number: 4061502-48-9
##&nbsp;Owner: anupamap-vsim-1
##&nbsp;Package           Type    Description           Expiration
##&nbsp;----------------- ------- --------------------- --------------------
##&nbsp;SnapMirror        site    SnapMirror License    -
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'owner' => 'anupamap-vsim-1',
##&nbsp;            'serial_number' => '4061502-48-9',
##&nbsp;            'licenses' => [
##&nbsp;                            {
##&nbsp;                              'expiration' => '-',
##&nbsp;                              'type' => 'site',
##&nbsp;                              'description' => 'SnapMirror License',
##&nbsp;                              'package' => 'SnapMirror'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub license_show{
    my %args    = @_;
    my $resp    = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $errormsg;
    my $error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg);
    my $i=0;
    my @result_array;
    my @lines = get_lines( 'raw_output' => $raw_output, "response_object" => $resp );
    foreach my $line (@lines) {
       if($line =~ /(\w+\s\w+|\w+):\s+(.*)/){
          $result_array[0]->{to_key_name(item=>$1)}=$2;
       }elsif($line =~ /(\w+\s\w+\s\w+|\w+\s\w+|\w+)\s+(license|site|demo)\s+(\w+\s\w+\s\w+|\w+\s\w+|\w+)\s+(\S+\s\S+|\S+)/){
          $result_array[0]->{"licenses"}->[$i]->{"package"}= $1;
          $result_array[0]->{"licenses"}->[$i]->{"type"}= $2;
          $result_array[0]->{"licenses"}->[$i]->{"description"}= $3;
          $result_array[0]->{"licenses"}->[$i]->{"expiration"}= $4;
          $i++;
       }
    }
    if($error){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
         $error->throw(text=>$errormsg,response_object => $resp);
    }
    return \@result_array;
}

1;
