#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse the 7Mode ls command
##
## @description
##    This is a library to parse the raw output of 7Mode ls command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::ls
##
##    ls(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ls;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::ls
##
## @summary Parses the raw output from the "ls" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ls with any of its options as described in the 
##    corresponding cdef file 'ls.cdefs'
## @synopsis
##      ls(%args);
## INPUTS TO METHOD CALL 'ls' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ls / /home
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'files' => [
##&nbsp;                         '.',
##&nbsp;                         '..',
##&nbsp;                         'etc',
##&nbsp;                         'home',
##&nbsp;                         '.ha',
##&nbsp;                         '.',
##&nbsp;                         '..',
##&nbsp;                         'http'
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ls {
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my (@result_array, $error);
    my @lines = get_lines(raw_output=>$raw_output, first_line=>'');
    
    my %hash = (files => []);
    
    # error checking is done while parsing.
    # Partial success is possible for ls and we do not want the line indicating 
    # the error to be in the parsed output. Since we are avoiding that line 
    # while parsing, we can use that for error checking as well
    
    foreach my $line ( @lines ) {
        if ( $line =~ /(.*: Can't open directory: .*)/ && !$error) {
            $error = 'NACL::APISet::Exceptions::CommandFailedException';
        } else {
            chomp($line);
            my @files = split(/\s+/, $line);
            push @{$hash{files}}, @files;
        }
    }
    
    push( @result_array, \%hash );
    
    if ($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error->throw(text => $1, response_object => $resp);
    }
    
    return \@result_array;
}

1;
