#
#Copyright (c) 2011-2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode lun commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode lun commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lun show" will be lun_show()
##
## @synopsis
##    use lun;
##
##    lun_show(%args);
##    lun_create(%args);
##    lun_destroy(%args);
##    lun_clone_create(%args);
##    lun_clone_split_start(%args);
##    lun_clone_split_stop(%args);
##    lun_clone_split_status(%args);
##    lun_resize(%args);
##    lun_share(%args);
##    lun_set_reservation(%args);
##    lun_map(%args);
##    lun_unmap(%args);
##    lun_move(%args);
##    lun_maxsize(%args);
##    lun_persistent_resv_clear(%args);
##    lun_persistent_resv_show(%args);
##    lun_rescan(%args);
##    lun_stats(%args);
##    lun_online(%args);
##    lun_offline(%args);
##    lun_geometry(%args);
##    lun_setup(%args);
##    lun_attribute_list(%args);
##    lun_attribute_set(%args);
##    lun_attribute_unset(%args);
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::lun;

use strict;
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
## @name ::lun_show
##
## @summary Parses the raw output from the "lun show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_show with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_show(%args);
## INPUTS TO METHOD CALL 'lun_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mapping' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'node' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> 'staging' </td> <td> 'staging' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'online' </td> <td> '1' </td> </tr>
## <tr> <td> 'unmapped' </td> <td> 'unmapped' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'offline' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'mapped-to' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cloning' </td> <td> '1' </td> </tr>
## <tr> <td> 'mapped' </td> <td> 'mapped' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; lun show -m; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;LUN path                            Mapped to          LUN ID  Protocol
##&nbsp;-----------------------------------------------------------------------
##&nbsp;/vol/vol0/test                      nacl_stask_ig6291        0       FCP
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'protocol' => 'FCP',
##&nbsp;            'lun_id' => '0',
##&nbsp;            'igroup' => 'nacl_stask_ig6291',
##&nbsp;            'lun_path' => '/vol/vol0/test'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = priv set -q test; lun show -i; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;***STEP 6 : SUCCESS
##&nbsp;Lun Number              : 4112
##&nbsp;UUID                    : b1e943a8-0fc7-11e3-a3e3-123478563412
##&nbsp;Vdisk Size              : 106896384
##&nbsp;space_alloc             : no
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'step_6' => 'success',
##&nbsp;            'lun_number' => '4112',
##&nbsp;            'uuid' => 'b1e943a8-0fc7-11e3-a3e3-123478563412',
##&nbsp;            'vdisk' => '106896384',
##&nbsp;            'space_alloc' => 'no',
##&nbsp;          }
##&nbsp;        ]
##&nbsp;

sub lun_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun show",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array = ();

    # We're currently not handling parsing for the options "-c"
    if ( $command =~ /lun\sshow\s((-c)\s)+/ ) {
        return undef;
    }

    # No output indicates that there are no luns of that type
    return [] if ( !$raw_output || $raw_output =~ /^LUN\s+path.+Protocol(\s+-+)?$/i);

    my $first_line = '\/vol\/.*';
    my @lines      = get_lines(
        raw_output      => $raw_output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    # This hash contains the mappings between the headings shown in the output
    # and the keys of the parsed output hash
    my %mapping_hash = (
        'Comment'            => 'comment',
        'Backed by'          => 'backed-by',
        'Serial#'            => 'serial',
        'Share'              => 'share',
        'Space Reservation'  => 'space-reserve',
        'Multiprotocol Type' => 'protocol',
        'Maps'               => 'maps',
        'Occupied Size'      => 'occupied-size',
        'Prefix Size'        => 'prefix-size',
        'Suffix Size'        => 'suffix-size',
        'Extent Boundary'    => 'extent-boundary',
        'Occupied Size'      => 'occupied-size',
        'Creation Time'      => 'creation-timestamp',
        'Alignment'          => 'alignment',
        'Cluster Shared Volume Information' =>
            'cluster-shared-volume-information',
        'Space_alloc'        => 'space_alloc',
        'report-physical-size' => 'report-physical-size',
        'Read-Only'          => 'read-only'
    );
    foreach my $line (@lines) {
        my ( $lun_path, $lun_path1, $size, $permission, $state, $mapped,%temp );
        if ( $line =~ /^(.*?)\s+(?:.*?)\s\((\d+)\)\s+\((.*?),\s(.*?)\)/i ) {

            # This line indicates we're displaying details of another lun
            # Create one more entry in the result_array
            $lun_path   = $1;
            $size       = $2;
            $permission = $3;
            $state      = $4;
 
            #The following is required when the lun is mapped
            #to an igroup , the $state would online, mapped
            if ( $state =~ /(^.*?),\s(.*$)/ ) {
                $state = $1;
                $mapped = $2;
            }
            push @result_array,
                {
                'lun-path'   => $lun_path,
                'size'       => $size,
                'permission' => $permission,
                'state'      => $state,
                 mapped      => $mapped
                };
        } elsif ($command =~ /lun\sshow\s(-m).*/) {
            #Skip the headers in the output
            if ($line =~ /lun path/i
                || $line =~ /^-+$/ ) {
                 next;
            } elsif ($line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i) {
                  $lun_path1 = $1;
                  $temp{'lun_path'} = $lun_path1;
                  $temp{'igroup'} = $2;
                  $temp{'lun_id'} = $3;
                  $temp{'protocol'} = $4;
                  push @result_array,\%temp;

              } elsif($line =~ /^(\S+)\s+(\S+)\s+(\S+)$/i && ($line !~ /^\/vol/)) {
                    $temp{'lun_path'} = $lun_path1;
                    $temp{'igroup'}  = $1;
                    $temp{'lun_id'}  = $2;
                    $temp{'protocol'} = $3;
                    push @result_array,\%temp;

                } else {
                   @result_array = [];
            }
        } elsif ($command =~ /lun\s+show\s+(-i).*/) {
           if ($line =~ /(\s?\w.*):(.*)/i) {
               $result_array[0]->{to_key_name(item => trim_string('string' => $1))}
                = trim_string('string' => $2);
           }
        } else {
                # The other lines not in that format are futher details of the
                # same LUN.
                trim( \$line );
                my ( $key, $value ) = split /:\s+/, $line;
                $result_array[-1]->{ $mapping_hash{$key} } = $value;

              }

        }

    return \@result_array;
} ## end sub lun_show

## @name ::lun_create
##
## @summary Parses the raw output from the "lun create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_create with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_create(%args);
## INPUTS TO METHOD CALL 'lun_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-e space_alloc' </td> <td> 'space-alloc' </td> </tr>
## <tr> <td> '-t' </td> <td> 'ostype' </td> </tr>
## <tr> <td> '-o noreserve' </td> <td> 'noreserve' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> '-P' </td> <td> 'prefix-size' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub lun_create {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun create",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_create

## @name ::lun_destroy
##
## @summary Parses the raw output from the "lun destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_destroy with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_destroy(%args);
## INPUTS TO METHOD CALL 'lun_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub lun_destroy {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun destroy",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_destroy

## @name ::lun_clone_create
##
## @summary Parses the raw output from the "lun clone create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_clone_create with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_clone_create(%args);
## INPUTS TO METHOD CALL 'lun_clone_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o noreserve' </td> <td> 'noreserve' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'clone-lunpath' </td> </tr>
## <tr> <td> '-b' </td> <td> 'parent-lunpath' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'parent-snap' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub lun_clone_create {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun clone create",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_clone_create

## @name ::lun_clone_split_start
##
## @summary Parses the raw output from the "lun clone split start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_clone_split_start with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_clone_split_start(%args);
## INPUTS TO METHOD CALL 'lun_clone_split_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disable-space-efficient-splitting' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun clone split start /vol/vol0/snap2
##&nbsp;$VAR1 = [];
##&nbsp;

sub lun_clone_split_start {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun clone split start",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_clone_split_start

## @name ::lun_clone_split_stop
##
## @summary Parses the raw output from the "lun clone split stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_clone_split_stop with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_clone_split_stop(%args);
## INPUTS TO METHOD CALL 'lun_clone_split_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub lun_clone_split_stop {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun clone split stop",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_clone_split_stop

## @name ::lun_clone_split_status
##
## @summary Parses the raw output from the "lun clone split status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_clone_split_status with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_clone_split_status(%args);
## INPUTS TO METHOD CALL 'lun_clone_split_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun clone split status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'percent-completed' => '52%',
##&nbsp;            'lun-path' => '/vol/vol0/snap2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lun clone split status /vol/vol0/snap2
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'percent-completed' => '76%',
##&nbsp;            'total-blocks' => '1024000',
##&nbsp;            'blocks-completed' => '785703'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub lun_clone_split_status {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun clone split status",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $result_array = [];
    if ( $raw_output
        =~ /[Dd]one\s+(\d+)\s+of\s+(\d+)\s*[a-zA-Z]*\s*\((\d+%)\s*complete\)/
        ) {
        my %result_hash;
        %result_hash = (
            "blocks-completed"  => $1,
            "total-blocks"      => $2,
            "percent-completed" => $3,
        );
        $result_array = [ \%result_hash ];
    } elsif ( $raw_output =~ /\(\s*\d+%\s*complete\s*/ ) {
        my @arr = get_lines( "raw_output" => $raw_output );
        foreach my $line (@arr) {
            if ( $line =~ /\s*(\S+)\s*\((\d+%)\s*complete\)/ ) {
                my $line_hash
                    = { "lun-path" => $1, "percent-completed" => $2 };
                push @$result_array, $line_hash;
            }
        }
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw( text => $raw_output, response_object => $resp );
    }
    return $result_array;
} ## end sub lun_clone_split_status

## @name ::lun_resize
##
## @summary Parses the raw output from the "lun resize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_resize with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_resize(%args);
## INPUTS TO METHOD CALL 'lun_resize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-size' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lun resize -f /vol/vol0/lun1 +2m
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'size' => '34m (35651584)     '
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lun_resize {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun resize",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $result_array = [];
    if ( $raw_output =~ /resized\s+to\s*:\s*(.+)\s+$/ ) {
        my %result_hash;
        %result_hash = ( "size" => $1 );
        $result_array = [ \%result_hash ];
    }
    return $result_array;
} ## end sub lun_resize

## @name ::lun_share
##
## @summary Parses the raw output from the "lun share" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_share with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_share(%args);
## INPUTS TO METHOD CALL 'lun_share' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-access' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lun share /vol/vol0/lun1 none
##&nbsp;$VAR1 = [];
##&nbsp;
sub lun_share {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun share",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_share

## @name ::lun_set_reservation
##
## @summary Parses the raw output from the "lun set reservation" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_set_reservation with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_set_reservation(%args);
## INPUTS TO METHOD CALL 'lun_set_reservation' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lun set reservation /vol/vol0/vol_lun
##&nbsp;$VAR1 =  [
##&nbsp;                    {
##&nbsp;                        'status' => 'enabled',
##&nbsp;                        'inode' => '3024',
##&nbsp;                        'lun_path' => '/vol/vol0/vol_lun'
##&nbsp;                    }
##&nbsp;         ],
##&nbsp;

sub lun_set_reservation {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun set reservation",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $result_array = [];
    if ( $raw_output
        =~ /Space Reservation for LUN (.+) \(inode (\d+)\): (disabled|enabled)/
        ) {
        my %result_hash;
        $result_hash{lun_path} = $1;
        $result_hash{inode}    = $2;
        $result_hash{status}   = $3;
        $result_array          = [ \%result_hash ];
    }
    return $result_array;
} ## end sub lun_set_reservation

## @name ::lun_map
##
## @summary Parses the raw output from the "lun map" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_map with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_map(%args);
## INPUTS TO METHOD CALL 'lun_map' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> '-f' </td> <td> 'no-conflict-checks' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun map -f /vol/vol0/lun1 igrp 3
##&nbsp;$VAR1 = [];
##&nbsp;

sub lun_map {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun map",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_map

## @name ::lun_unmap
##
## @summary Parses the raw output from the "lun unmap" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_unmap with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_unmap(%args);
## INPUTS TO METHOD CALL 'lun_unmap' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'initiator-group' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun unmap /vol/vol0/lun1 igrp
##&nbsp;$VAR1 = [];
##&nbsp;

sub lun_unmap {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $result;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun unmap",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # As there is no output given by the command, parsed output will be null
    my @result_array = ();

    return \@result_array;
} ## end sub lun_unmap

## @name ::lun_maxsize
##
## @summary Parses the raw output from the "lun maxsize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_maxsize with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_maxsize(%args);
## INPUTS TO METHOD CALL 'lun_maxsize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun maxsize /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ostype' => [
##&nbsp;                          'solaris',
##&nbsp;                          ' aix',
##&nbsp;                          ' hpux',
##&nbsp;                          ' linux',
##&nbsp;                          ' netware',
##&nbsp;                          ' vmware',
##&nbsp;                          ' openvms',
##&nbsp;                          ' xen',
##&nbsp;                          ' solaris_efi or image'
##&nbsp;                        ],
##&nbsp;            'without_snapshot_reserve' => '13m (13631488)',
##&nbsp;            'with_snapshot_reserve' => '6m (6291456)',
##&nbsp;            'with_complete_snapshot_reserve' => '6m (6291456)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ostype' => [
##&nbsp;                          ' windows',
##&nbsp;                          ' windows_gpt',
##&nbsp;                          ' windows_2008 or hyper_v'
##&nbsp;                        ],
##&nbsp;            'without_snapshot_reserve' => 'no space',
##&nbsp;            'with_snapshot_reserve' => 'no space',
##&nbsp;            'with_complete_snapshot_reserve' => 'no space'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub lun_maxsize {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @result;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun maxsize",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $response
    );
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line =~ /Space available for a LUN of type: (.*)/ ) {
            my $ostypes = $1;
            my @ostype = split( ', ', $ostypes );
            $result[$i]->{'ostype'} = \@ostype;
        } elsif ( $line =~ /Without snapshot reserve: (.*)/ ) {
            $result[$i]->{'without_snapshot_reserve'} = $1;
        } elsif ( $line =~ /With snapshot reserve: (.*)/ ) {
            $result[$i]->{'with_snapshot_reserve'} = $1;
        } elsif ( $line =~ /With complete snapshot reserve: (.*)/ ) {
            $result[$i]->{'with_complete_snapshot_reserve'} = $1;
            $i++;
        }
    }
    return \@result;
} ## end sub lun_maxsize

## @name ::lun_move
##
## @summary Parses the raw output from the "lun move" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_move with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_move(%args);
## INPUTS TO METHOD CALL 'lun_move' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'to-lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun move /vol/vol0/lun1 /vol/vol0/lun2
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub lun_move {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun move",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_move

## @name ::lun_persistent_resv_clear
##
## @summary Parses the raw output from the "lun persistent_resv clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_persistent_resv_clear with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_persistent_resv_clear(%args);
## INPUTS TO METHOD CALL 'lun_persistent_resv_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun persistent_resv clear /vol/vol0/lun1
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub lun_persistent_resv_clear {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun persistent_resv clear",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_persistent_resv_clear

## @name ::lun_online
##
## @summary Parses the raw output from the "lun online" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_online with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_online(%args);
## INPUTS TO METHOD CALL 'lun_online' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'no-conflict-checks' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun online /vol/vol0/lun1
##&nbsp;$VAR1 = [ ];
##&nbsp;
sub lun_online {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun online",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_online

## @name ::lun_offline
##
## @summary Parses the raw output from the "lun offline" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_offline with any of its options as described in the
##    corresponding cdef file 'lun_offline.cdefs'
## @synopsis
##      lun_offline(%args);
## INPUTS TO METHOD CALL 'lun_offline' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun offline /vol/vol0/lun1
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub lun_offline {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun offline",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_offline

## @name ::lun_persistent_resv_show
##
## @summary Parses the raw output from the "lun lun_persistent resv show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_persistent_resv_show with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_persistent_resv_show(%args);
## INPUTS TO METHOD CALL 'lun_persistent_resv_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun persistent resv show /vol/vol0/lun1
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub lun_persistent_resv_show {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun persistent_resv show",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_persistent_resv_show

## @name ::lun_rescan
##
## @summary Parses the raw output from the "lun rescan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_rescan with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_rescan(%args);
## INPUTS TO METHOD CALL 'lun_rescan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; lun rescan vol0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = lun rescan /vol/vol0/lun1
##&nbsp;$VAR1 = [ ];
##&nbsp;

sub lun_rescan {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun rescan",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
} ## end sub lun_rescan

## @name ::lun_stats
##
## @summary Parses the raw output from the "lun stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_stats with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_stats(%args);
## INPUTS TO METHOD CALL 'lun_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-a' </td> <td> 'lun-zero-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'lun-interval-in-seconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'lun-extended-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'lun-count-of-iterations' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'resets-statics' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun stats /vol/vol0/lun1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'last_zeroed' => '0:0:1',
##&nbsp;            'path' => '/vol/apiset_volume/api_new_lun'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lun_stats {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun stats",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $response
    );
    my $i    = 0;
    my $flag = 0;
    foreach my $line (@lines) {
        if ($line =~ /(\S+)\s+\((\d+)\s+\w+,\s+(\d+)\s+\w+,\s+(\d+)\s+\w+\)/ )
        {
            $result[$i]->{'path'}        = $1;
            $result[$i]->{'last_zeroed'} = $2 . ":" . $3 . ":" . $4;
        } elsif ( $line
            =~ /Read \(kbytes\)   Write \(kbytes\)  Read Ops  Write Ops  Other Ops  QFulls  Partner Ops Partner KBytes/
            ) {
            $flag = 1;
            next;
        } elsif ( $flag == 1
            && $line
            =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
            ) {
            $result[$i]->{'read_kbyte'}    = $1;
            $result[$i]->{'write_kbyte'}   = $2;
            $result[$i]->{'read_ops'}      = $3;
            $result[$i]->{'write_ops'}     = $4;
            $result[$i]->{'other_ops'}     = $5;
            $result[$i]->{'qfulls'}        = $6;
            $result[$i]->{'partner_ops'}   = $7;
            $result[$i]->{'partner_kbyte'} = $8;
        }
    }
    return \@result;
} ## end sub lun_stats

## @name ::lun_geometry
##
## @summary Parses the raw output from the "lun geometry" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_geometry with any of its options as described in the
##    corresponding cdef file 'lun_unmap.cdefs'
## @synopsis
##      lun_geometry(%args);
## INPUTS TO METHOD CALL 'lun_geometry' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun geometry /vol/vol0/lun1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cylinder_size' => '1048576',
##&nbsp;            'cylinders' => '10',
##&nbsp;            'bytes_per_sector' => '512',
##&nbsp;            'sectors_per_cylinder' => '2048',
##&nbsp;            'sectors_per_track' => '128',
##&nbsp;            'tracks_per_cylinder' => '16',
##&nbsp;            'max_resize_size' => '68718428160',
##&nbsp;            'device_size' => '10485760',
##&nbsp;            'sectors' => '20480'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub lun_geometry {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "lun geometry",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $response
    );
    my $i    = 0;
    my $flag = 0;
    foreach my $line (@lines) {
        if ( $line =~ /(\d+)\s+bytes\/sector/ ) {
            $result[$i]->{'bytes_per_sector'} = $1;
        } elsif ( $line =~ /(\d+)\s+sectors\/track/ ) {
            $result[$i]->{'sectors_per_track'} = $1;
        } elsif ( $line =~ /(\d+)\s+tracks\/cylinder/ ) {
            $result[$i]->{'tracks_per_cylinder'} = $1;
        } elsif ( $line =~ /(\d+)\s+sectors\/cylinder/ ) {
            $result[$i]->{'sectors_per_cylinder'} = $1;
        } elsif ( $line =~ /(\d+)\s+cylinders/ ) {
            $result[$i]->{'cylinders'} = $1;
        } elsif ( $line =~ /(\d+)\s+sectors$/ ) {
            $result[$i]->{'sectors'} = $1;
        } elsif ( $line =~ /(\d+)\s+cylinder\s+size/ ) {
            $result[$i]->{'cylinder_size'} = $1;
        } elsif ( $line =~ /(\d+)\s+device\s+size/ ) {
            $result[$i]->{'device_size'} = $1;
        } elsif ( $line =~ /(\d+)\s+max\s+resize\s+size/ ) {
            $result[$i]->{'max_resize_size'} = $1;
        }

    }
    return \@result;
} ## end sub lun_geometry

## @name ::lun_setup
##
## @summary Parses the raw output from the "lun setup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_setup. This command is an interactive command,
##    and hence is just for error checking.
## @synopsis
##      lun_setup();
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples

##&nbsp;Command = lun setup
##&nbsp;$VAR1 = [];
##&nbsp;
sub lun_setup {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub lun_setup

## @name ::lun_attribute_list
##
## @summary Parses the raw output from the "lun attribute list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_attribute_list with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_attribute_list(%args);
## INPUTS TO METHOD CALL 'lun_attribute_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = lun attribute list /vol/apiset_forking_disturber_1378279407_vol/api_new_lun
##&nbsp;-------- Raw Output ---------
##&nbsp;path_last = "api_new_lun"
##&nbsp;enabled = "1"
##&nbsp;type = "linux"
##&nbsp;cylinder_size = "1048576"
##&nbsp;custom = "no"
##&nbsp;extent_size = "0"
##&nbsp;read_only = "no"
##&nbsp;space_alloc = "no"
##&nbsp;serial = "Be/gH?DKsVkI"
##&nbsp;host_stamp = "4071189578, -582518255, 0"
##&nbsp;@Anvil@FSConsistent = "off"
##&nbsp;test_attribute = "yes"
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'anvil_fsconsistent' => 'off',
##&nbsp;            'host_stamp' => '4071189578, -582518255, 0',
##&nbsp;            'serial' => 'Be/gH?DKsVkI',
##&nbsp;            'cylinder_size' => '1048576',
##&nbsp;            'custom' => 'no',
##&nbsp;            'path_last' => 'api_new_lun',
##&nbsp;            'space_alloc' => 'no',
##&nbsp;            'test_attribute' => 'yes'
##&nbsp;            'extent_size' => '0',
##&nbsp;            'read_only' => 'no',
##&nbsp;            'type' => 'linux',
##&nbsp;            'enabled' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub lun_attribute_list {
    my %args = @_;

    my ($error_message);
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @result;
    my $error = _check_error(
        raw_output        => $raw_output,
        command           => 'lun attribute list',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $response
    );
    foreach my $line (@lines) {
        my @values = split(/\s+=\s+/,$line);
        $values[0] =~ s/\@Anvil\@FSConsistent/anvil_fsconsistent/g;
        $values[1] =~ s/"//g;
        $result[0]->{to_key_name( item => $values[0] )} = $values[1];
    }
    return \@result;
}

## @name ::lun_attribute_set
##
## @summary Parses the raw output from the "lun attribute set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_attribute_set with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_attribute_set(%args);
## INPUTS TO METHOD CALL 'lun_attribute_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub lun_attribute_set {
    my %args = @_;

    my ($error_message);
    my $response    = $args{response_object};
    my $raw_output  = $response->get_processed_output();
    my $error = _check_error(
        raw_output        => $raw_output,
        command           => 'lun attribute set',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
}

## @name ::lun_attribute_unset
##
## @summary Parses the raw output from the "lun attribute unset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lun_attribute_unset with any of its options as described in the
##    corresponding cdef file 'lun.cdefs'
## @synopsis
##      lun_attribute_unset(%args);
## INPUTS TO METHOD CALL 'lun_attribute_unset' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'lun-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub lun_attribute_unset {
    my %args = @_;

    my ($error_message);
    my $response    = $args{response_object};
    my $raw_output  = $response->get_processed_output();
    my $error = _check_error(
        raw_output        => $raw_output,
        command           => 'lun attribute unset',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = "CommandFailed";
    if ( $raw_output =~ /$command(?:[^:]*):(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message
            =~ /exactly one of|missing|option is required|must be supplied/ )
        {
            $error = "MissingRequiredParam";
        } elsif ( $$error_message
            =~ m{No such volume|Invalid type|scsi-lun value out of range|
            No such LUN exists|LUN is not mapped to this group|
            Requested LUN size|lun size must be|minimum size|
            lun pathname should start with|No\s+such\s+attribute|
            No such initiator group exists}
            ) {
            $error = "InvalidParamValue";
        } elsif (
            $$error_message =~ /usage:|lun pathname should start with / ) {
            $error = "InvalidParam";
        } elsif ( $$error_message
            =~ /Path\s+\S+\s+is invalid: No such file or directory/ ) {
            $error = "InvalidParam";
        } elsif (
            $$error_message =~ /\S+: path is not a volume or qtree root/ ) {
            $error = "InvalidParam";
        } elsif ( $$error_message =~ /A LUN exists at the path/ ) {
            $error = "InvalidParam";
        } elsif ( $$error_message =~ /Both paths must be in the same volume/ )
        {
            $error = "InvalidParam";
        } elsif ( $$error_message =~ /[Dd]one\s+\d+\s+of\s+\d+/ ) {
            $error = undef;
        } elsif ( $$error_message =~ /resized\s+to/ ) {
            $error = undef;
        } elsif ( $$error_message =~ /Missing/ ) {
            $error = "MissingRequiredParam";
        } elsif ( $$error_message =~ /.*No such (LUN|initiator).*/ ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message
            =~ /(.*lun pathname should start.* | .*LUN is not mapped.* | .*path is not a.*)/
            ) {
            $error = "InvalidParam";
        } elsif ( $raw_output =~ /LUN already mapped to this group/ ) {
            $error          = "CommandFailed";
            $$error_message = "$&";
            return $error;
        } elsif ( $$error_message =~ /created a LUN of size/ ) {
            $error = undef;
        }
        return $error;
    } elsif ( $raw_output =~ /[Uu]sage:/ ) {
        $error = "CommandFailed";
        $$error_message
            = "Invalid switch or Invalid Param value has been sent";
        return $error;
    }
    return undef;
} ## end sub _check_error

1;
