#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode maxfiles commands
##
## @description
##    This is a library to parse the raw outputs of 7Mode maxfiles commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "maxfiles" will be maxfiles()
##
## @synopsis
##    use maxfiles;
##    maxfiles(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::maxfiles;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::maxfiles
##
## @summary Parses the raw output from the "maxfiles" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from maxfiles with any of its options as described in the
##    corresponding cdef file 'maxfiles.cdefs'
## @synopsis
##      maxfiles(%args);
## INPUTS TO METHOD CALL 'maxfiles' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resize' </td> <td> 'resize' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'max' </td> <td> '0' </td> </tr>
## <tr> <td> '-stats' </td> <td> 'stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-private' </td> <td> 'private' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-capacity' </td> <td> 'capacity' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = maxfiles
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'used' => '5564',
##&nbsp;            'max_private_files' => '49995'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub maxfiles {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "maxfiles",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @return_array;
    my @output =
      get_lines( "raw_output" => $output, "response_object" => $resp );
    my $i = -1;
    foreach my $line (@output) {
    #Volume vol0: maximum number of private files is currently 25876 (518 used).
        if ( $line =~ /\w+\s+(\w+):\s+.*private files.*\s+(\d+)\s+[(](\d+)/) {
            $i++;
            $return_array[$i]->{'volume'}            = $1;
            $return_array[$i]->{'max_private_files'} = $2;
            $return_array[$i]->{'used'}              = $3;
    #Volume vol0: maximum number of files is currently 25876 (518 used).
        } elsif ( $line =~ /\w+\s+(\w+):\s+.*of files.*\s+(\d+)\s+[(](\d+)/) {
            $i++;
            $return_array[$i]->{'volume'}    = $1;
            $return_array[$i]->{'max_files'} = $2;
            $return_array[$i]->{'used'}      = $3;
    # private inode file capacity is currently 25876
        } elsif($line =~ /private inode file capacity.*\s+(\d+)/) {
            $return_array[$i]->{'private_inode_capacity'} = $1;
    # inode file capacity is currently 25876
        } elsif($line =~ /^\s+inode file capacity.*\s+(\d+)/) {
            $return_array[$i]->{'inode_capacity'} = $1;
    # Volume vol0: private inodes used:5925 reserve:1361 total:27237 maximum:52626
        } elsif($line =~ /\w+\s+(\w+):\s+\w+\s+\w+\s+\w+:(\d+)\s+\w+:(\d+)\w+\s+\w+:(\d+)\s+\w+:(\d+)/) {
            $i++;
            $return_array[$i]->{'volume'} = $1;
            $return_array[$i]->{'private_inode_used'} = $2;
            $return_array[$i]->{'reserve'} = $3;
            $return_array[$i]->{'total'} = $4;
            $return_array[$i]->{'maximum'} = $5;
    # Volume vol0: inodes used:5925 reserve:1361 total:27237 maximum:52626
        } elsif($line =~ /\w+\s+(\w+):\s+\w+\s+\w+:(\d+)\s+\w+:(\d+)\s+\w+:(\d+)\s+\w+:(\d+)/) {
            $i++;
            $return_array[$i]->{'volume'} = $1;
            $return_array[$i]->{'inode_used'} = $2;
            $return_array[$i]->{'reserve'} = $3;
            $return_array[$i]->{'total'} = $4;
            $return_array[$i]->{'maximum'} = $5;
        } elsif($line =~ /new maximum number of files.*\s+(\d+)/) {
            $return_array[0]->{'max_number_rounded'} = $1;
        }
    }
    return \@return_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /.*\s+[(]\d+[)]\s+is invalid/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /Unknown volume.*/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /max must be a number/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif (( $raw_output =~ /(-resize|-stats|-capacity|-private)/i ) &&
            ( $raw_output =~ /cannot be/i )) {
        $error          = "InvalidParam";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /.*New count must be larger than.*/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /usage:.*/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid Parameter Passed";
    } elsif ( $raw_output =~ /New size too small/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /Inofile already at maximum size/i ) {
        $error          = "InvalidParamValue";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /volume required/i ) {
        $error          = "InvalidParam";
        $$error_message = "$&";
    } elsif ( $raw_output =~ /new size required/i ) {
        $error          = "InvalidParam";
        $$error_message = "$&";
    }
    return $error;
}
1;
