#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/mbstat.pm$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode mbstat command
##
## @description
##    This is a library to parse the raw output of 7Mode mbstat  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use mbstat ;
##
##    mbstat(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mbstat;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::mbstat
##
## @summary Parses the raw output from the "mbstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mbstat with any of its options as described in the
##    corresponding cdef file 'mbstat.cdefs'
## @synopsis
##      mbstat(%args);
## INPUTS TO METHOD CALL 'mbstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; mbstat; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'old_style_mbuf' => [
##&nbsp;                                  {
##&nbsp;                                    'mallocs' => '0',
##&nbsp;                                    'drops' => '0',
##&nbsp;                                    'total' => '50'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'nfs' => [
##&nbsp;                       {
##&nbsp;                         'request' => '0'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'ip_reassembly' => [
##&nbsp;                                 {
##&nbsp;                                   'frags' => '0',
##&nbsp;                                   'pkts' => '0'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'total_bufs' => [
##&nbsp;                              {
##&nbsp;                                'MT_FREE' => '45518',
##&nbsp;                                'MT_DATA' => '6203',
##&nbsp;                                'total_mbufs' => '51721'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'descriptor_mbuf' => [
##&nbsp;                                   {
##&nbsp;                                     'mallocs' => '0',
##&nbsp;                                     'drops' => '0',
##&nbsp;                                     'total' => '38144'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'system_pool_large_bufs' => [
##&nbsp;                                          {
##&nbsp;                                            'current_number_of_large_bufs_in_the_system' => '8279',
##&nbsp;                                            'initial_pool_size' => '6656',
##&nbsp;                                            'free_mbufs' => [
##&nbsp;                                                              {
##&nbsp;                                                                'common_pool' => '512',
##&nbsp;                                                                'all_per-cpu_pools' => '1564'
##&nbsp;                                                              }
##&nbsp;                                                            ],
##&nbsp;                                            'mallocs' => '1623',
##&nbsp;                                            'drops' => '0',
##&nbsp;                                            'size_of_each_mbuf' => '2048',
##&nbsp;                                            'drains' => [
##&nbsp;                                                          {
##&nbsp;                                                            'success' => '0',
##&nbsp;                                                            'total_drains' => '0'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                            'borrowed' => '0',
##&nbsp;                                            'waits' => '0',
##&nbsp;                                            'maximum_pool_size' => '69457'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'system_pool_small_bufs' => [
##&nbsp;                                          {
##&nbsp;                                            'initial_pool_size' => '5248',
##&nbsp;                                            'free_mbufs' => [
##&nbsp;                                                              {
##&nbsp;                                                                'common_pool' => '3712',
##&nbsp;                                                                'all_per-cpu_pools' => '1536'
##&nbsp;                                                              }
##&nbsp;                                                            ],
##&nbsp;                                            'mallocs' => '0',
##&nbsp;                                            'drops' => '0',
##&nbsp;                                            'size_of_each_mbuf' => '192',
##&nbsp;                                            'drains' => [
##&nbsp;                                                          {
##&nbsp;                                                            'success' => '0',
##&nbsp;                                                            'total_drains' => '0'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                            'borrowed' => '0',
##&nbsp;                                            'waits' => '0',
##&nbsp;                                            'current_number_of_small_bufs_in_the_system' => '5248',
##&nbsp;                                            'maximum_pool_size' => '14848'
##&nbsp;                                          }
##&nbsp;                                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub mbstat {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'mbstat',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array;
    my ($i,$j,$k,$m,$key1,$key2,$key3,$key4,$key5,$key6,$name1,$name2,$name);
    my ($value1, $value2, $value3, $value4, $value5, $value6, $hist,$count);
    my @lines = get_lines( raw_output => $raw_output, can_throw => 0,
                           response_object => $resp );
    $i = 0;
    $j = 0;
    $k = 0;
    $m = -1;
    $count = 1;
    foreach my $line (@lines) {
        $name = "system_pool_small_bufs"  if ($line =~ /^System\spool\ssmall\sbufs:$/);
        $name = "system_pool_large_bufs"  if ($line =~ /^System\spool\slarge\sbufs:$/);
        $hist = "block_length";
        $hist = "amount" if($count%2 == 0);
    if($line =~ /^Size of each mbuf\s(\w+)./){
        $value1 = $1;
        $result_array[$i] -> {"$name"} -> [$j] -> {"size_of_each_mbuf"} = $value1;
    } elsif ($line =~ /^(\w+\s\w+\s\w+)\s(\w+)\,\s(\w+\s\w+\s\w+)\s(\w+)\,\s(\w+\s\w+\s\w+\s\w+\s\w+\s\w+\s\w+\s\w+)\s(\w+)\,/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $key1 =~ s/\s+/_/g;
        $key2 =~ s/\s+/_/g;
        $key3 =~ s/\s+/_/g;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key1"} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key2"} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key3"} = $value3;
    } elsif ($line =~ /^\w+\s\w+\s.(\w+\s\w+)\s(\w+)\s..(\w+\s\w+.\w+\s\w+)\s(\w+)./) {
        $key1 = $1;
        $key2 = $3;
        $value1 = $2;
        $value2 = $4;
        $key1 =~ s/\s+/_/g;
        $key2 =~ s/\s+/_/g;
        $result_array[$i] -> {"$name"} -> [$j] -> {"free_mbufs"} -> [$k] ->
        {"$key1"} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {"free_mbufs"} -> [$k] ->
        {"$key2"} = $value2;
    } elsif ($line =~ /^(\w+)\s(\w+).\s(\w+)\s(\w+).\s(\w+)\s(\w+)\s.(\w+)\s(\w+)..\s(\w+)\s(\w+).\s(\w+)\s(\w+)/){
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $key4 = $7;
        $key5 = $9;
        $key6 = $11;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $value4 = $8;
        $value5 = $10;
        $value6 = $12;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key1"} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key2"} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {"drains"} -> [$k] ->
        {"$key4"} = $value4;
        $result_array[$i] -> {"$name"} -> [$j] -> {"drains"} -> [$k] ->
        {"total_drains"} = $value3;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key5"} = $value5;
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key6"} = $value6;
    } elsif ($line =~ /^CPU(\w+)\s(\w+)\sCPU(\w+)\s(\w+)$/) {
        $key1 = $1;
        $key2 = $3;
        $key1 = "CPU".$key1;
        $key2 = "CPU".$key2;
        $value1 = $2;
        $value2 = $4;
        $result_array[$i] -> {"$name"} -> [$j] -> {"per-cpu_free_pool_size"} ->
        [$k] -> {"$key1"} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {"per-cpu_free_pool_size"} ->
        [$k] -> {"$key2"} = $value2;
    } elsif ($line =~ /^(\w+\s\w+)\s(initial sizing|currently using)\s(\w+\s)?roughly\s(\w+\%) of the total memory\s(\w+)/) {
        $key1 = $1;
        $key2 = $2;
        $value1 = $4;
        $value2 = $5;
        $key1 =~ s/\s+/_/g;
        $key2 =~ s/\s+/_/g;
        $key1 = "$key1"."_"."$key2";
        $value1 = "roughly"."_"."$value1"."_"."of_the_total_memory"."_"."$value2"
                   ."\(not_including_mem_allocated_to_FreeBSD_kernel\)";
        $result_array[$i] -> {"$name"} -> [$j] -> {"$key1"} = $value1;
    } elsif ($line =~ /^(\w+)\s((\w+_\w+)|total mbufs)$/) {
        $value1 = $1;
        $key1 = $2;
        $key1 =~ s/\s+/_/g;
        $key1 =~ tr/A-Z/a-z/;
        $result_array[$i] -> {"total_bufs"} -> [$j] -> {"$key1"} = $value1;
    } elsif ($line =~ /^Descriptor mbuf\:\s(\w+)\s(\w+)\,\s(\w+)\s(\w+)\,\s(\w+)\s(\w+)$/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $result_array[$i] -> {"descriptor_mbuf"} -> [$j] -> {"$key1"} = $value1;
        $result_array[$i] -> {"descriptor_mbuf"} -> [$j] -> {"$key2"} = $value2;
        $result_array[$i] -> {"descriptor_mbuf"} -> [$j] -> {"$key3"} = $value3;
    } elsif ($line =~ /^Old style mbuf\:\s(\w+)\s(\w+)\,\s(\w+)\s(\w+)\,\s(\w+)\s(\w+)$/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $result_array[$i] -> {"old_style_mbuf"} -> [$j] -> {"$key1"} = $value1;
        $result_array[$i] -> {"old_style_mbuf"} -> [$j] -> {"$key2"} = $value2;
        $result_array[$i] -> {"old_style_mbuf"} -> [$j] -> {"$key3"} = $value3;
    } elsif ($line =~ /^nfs\:\s(\w+)\s(\w+)$/) {
        $key1 = $1;
        $value1 = $2;
        $result_array[$i] -> {"nfs"} -> [$j] -> {"$key1"} = $value1;
    } elsif($line =~ /^ip reassembly\:\s+(\w+)\s(\w+)\,\s(\w+)\s(\w+)$/) {
        $key1 = $2;
        $key2 = $4;
        $value1 = $1;
        $value2 = $3;
        $result_array[$i] -> {"ip_reassembly"} -> [$j] -> {"$key1"} = $value1;
        $result_array[$i] -> {"ip_reassembly"} -> [$j] -> {"$key2"} = $value2;
    } elsif($line =~ /^Interface\s+(\w+)$/) {
        $value1 = $1;
        $m++;
        $k = 0;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"} -> [$k] -> {"interface_name"} = $value1;
    } elsif(($line =~ /^(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/)
            || ($line =~ /^\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+(UNUSED)$/)
            || ($line =~ /^\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\<(\d+)\s+\>(\d+)$/)) {
        $value1 = $1;
        $value2 = $2;
        $value3 = $3;
        $value4 = $4;
        $value5 = $5;
        $value6 = $6;
        $key1 = $7;
        $key2 = $8;
        my $c = $k;
        $k++;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value1;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value2;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value3;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value4;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value5;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$value6;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k++] -> {"$hist"} =$key1;
        $result_array[$i] -> {"driver_receive_histograms"} -> [$m] -> {"interface"}-> [$k] -> {"$hist"} =$key2;
        $count++;
        $k = $c if($count%2 == 0);
    }
    }
     return \@result_array;
}
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    return undef;
}
1;
