#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode mcc command output
##
## @description
##    This is a library to parse the raw output of 7Mode mcc command.
##
## @synopsis
##
##    mcc(%args);
##
## @author dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mcc;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::mcc_test_set_dump_apis
##
## @summary Parses the raw output from the "mcc test set_dump_apis" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mcc_test_set_dump_apis with any of its options as described in the
##    corresponding cdef file 'mcc_test_set_dump_apis.cdefs'
## @synopsis
##      mcc_test_set_dump_apis(%args);
## INPUTS TO METHOD CALL 'mcc_test_set_dump_apis' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system node run -node local -command "priv set -q test; mcc test set_dump_apis; result -l"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;test_id 1 -- flags 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => '1',
##&nbsp;            'test_id' => '1'
##&nbsp;          }
##&nbsp;        ];

sub mcc_test_set_dump_apis {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count = -1;
    foreach my $line (@lines) {
        if ( $line =~ /(\w+)\s+(\d+)\s+\S+\s+(\w+)\s+(\d+)/ ) {
            $count++;
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
            $result_array[$count]->{ to_key_name( item => $3 ) } = $4;
        }
    }
    return \@result_array;
}

## @name ::mcc_test_clear
##
## @summary Parses the raw output from the "mcc test clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mcc_test_clear with any of its options as described in the
##    corresponding cdef file 'mcc_test_clear.cdefs'
## @synopsis
##      mcc_test_clear(%args);
## INPUTS TO METHOD CALL 'mcc_test_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub mcc_test_clear {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::mcc_test_dump_apis_now
##
## @summary Parses the raw output from the "mcc test dump_apis_now" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mcc_test_dump_apis_now with any of its options as described in the
##    corresponding cdef file 'mcc_test_dump_apis_now.cdefs'
## @synopsis
##      mcc_test_dump_apis_now(%args);
## INPUTS TO METHOD CALL 'mcc_test_dump_apis_now' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system node run -node local -command "priv set -q test; mcc test dump_apis_now; result -l"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DRSOM TEST API DUMP:
##&nbsp;                                      drsom_host : DR-PRIMARY
##&nbsp;                        drsom_initialization_done: true
##&nbsp;
##&nbsp;                  drsom_is_operation_in_progress : false
##&nbsp;        drsom_is_[nso]_source_commit_in_progress : false
##&nbsp;     drsom_is_[nso]_source_vetocheck_in_progress : false
##&nbsp;     drsom_is_[nso]_source_commit_complete : false
##&nbsp;      drsom_is_node_in_switchback_recovery : false
##&nbsp;
##&nbsp;         drsom_is_primary_switchover_in_progress : false
##&nbsp;       drsom_is_auxiliary_switchover_in_progress : false
##&nbsp;                 drsom_is_switchover_in_progress : false
##&nbsp;   drsom_is_primary_switchover_retry_in_progress : false
##&nbsp; drsom_is_auxiliary_switchover_retry_in_progress : false
##&nbsp;           drsom_is_switchover_retry_in_progress : false
##&nbsp;
##&nbsp;   drsom_is_primary_switchover_abort_in_progress : false
##&nbsp;   drsom_is_primary_switchover_abort_in_progress : false
##&nbsp; drsom_is_auxiliary_switchover_abort_in_progress : false
##&nbsp;           drsom_is_switchover_abort_in_progress : false
##&nbsp;
##&nbsp;           drsom_is_primary_switchover_scheduled : false
##&nbsp;         drsom_is_auxiliary_switchover_scheduled : false
##&nbsp;                   drsom_is_switchover_scheduled : false
##&nbsp;
##&nbsp;      drsom_is_primary_state_switchover_complete : false
##&nbsp;    drsom_is_auxiliary_state_switchover_complete : false
##&nbsp;
##&nbsp;                  drsom_has_switchover_been_done : false
##&nbsp;                  drsom_is_primary_switched_over : false
##&nbsp;                drsom_is_auxiliary_switched_over : false
##&nbsp;
##&nbsp;         drsom_is_primary_heal_aggrs_in_progress : false
##&nbsp;       drsom_is_auxiliary_heal_aggrs_in_progress : false
##&nbsp;
##&nbsp;      drsom_is_primary_state_heal_aggrs_complete : false
##&nbsp;    drsom_is_auxiliary_state_heal_aggrs_complete : false
##&nbsp;
##&nbsp;         drsom_is_primary_heal_roots_in_progress : false
##&nbsp;       drsom_is_auxiliary_heal_roots_in_progress : false
##&nbsp;
##&nbsp;      drsom_is_primary_state_heal_roots_complete : false
##&nbsp;    drsom_is_auxiliary_state_heal_roots_complete : false
##&nbsp;
##&nbsp;                 drsom_is_switchback_in_progress : false
##&nbsp;          drsom_is_switchback_in_progress_normal : false
##&nbsp;     drsom_is_switchback_in_progress_left_behind : false
##&nbsp;         drsom_is_sb_in_progress_due_to_so_panic : false
##&nbsp;           drsom_is_switchback_retry_in_progress : false
##&nbsp;  drsom_is_sb_in_progress_due_to_so_retry_reboot : false
##&nbsp;  drsom_is_sb_in_progress_due_to_so_abort_reboot : false
##&nbsp;      drsom_is_sb_in_progress_due_to_so_failures : false
##&nbsp;
##&nbsp;                   drsom_is_switchback_scheduled : false
##&nbsp;      drsom_is_sb_scheduled_due_to_so_failures   : false
##&nbsp;      drsom_is_sb_scheduled_due_to_sb_failure    : false
##&nbsp;                   drsom_early_handling_required : false
##&nbsp;        drsom_switchover_abort_handling_required : false
##&nbsp;             drsom_switchback_vetocheck_bypassed : false
##&nbsp;                    drsom_get_wafl_nvinfo_online : 0
##&nbsp;               drsom_get_force_nvfail_all_option : false
##&nbsp;                             remote_cserver_uuid : bdf5a48f-ef64-11e2-8145-123478563412
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'drsom_switchover_abort_handling_required' => 'false',
##&nbsp;            'drsom_is_primary_state_switchover_complete' => 'false',
##&nbsp;            'drsom_is_primary_state_heal_roots_complete' => 'false',
##&nbsp;            'drsom_is_primary_switchover_in_progress' => 'false',
##&nbsp;            'drsom_is_sb_scheduled_due_to_sb_failure' => 'false',
##&nbsp;            'drsom_switchback_vetocheck_bypassed' => 'false',
##&nbsp;            'drsom_is_auxiliary_state_heal_roots_complete' => 'false',
##&nbsp;            'drsom_is_auxiliary_switchover_scheduled' => 'false',
##&nbsp;            'drsom_is_switchback_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_heal_aggrs_in_progress' => 'false',
##&nbsp;            'drsom_has_switchover_been_done' => 'false',
##&nbsp;            'drsom_is_primary_switchover_scheduled' => 'false',
##&nbsp;            'drsom_is_node_in_switchback_recovery' => 'false',
##&nbsp;            'drsom_is_operation_in_progress' => 'false',
##&nbsp;            'drsom_is_switchback_in_progress_left_behind' => 'false',
##&nbsp;            'drsom_is_switchover_retry_in_progress' => 'false',
##&nbsp;            'drsom_is_primary_switchover_retry_in_progress' => 'false',
##&nbsp;            'drsom_early_handling_required' => 'false',
##&nbsp;            'drsom_is_auxiliary_switchover_abort_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_switchover_retry_in_progress' => 'false',
##&nbsp;            'drsom_is_switchover_scheduled' => 'false',
##&nbsp;            'drsom_is_primary_switchover_abort_in_progress' => 'false',
##&nbsp;            'drsom_is_sb_scheduled_due_to_so_failures' => 'false',
##&nbsp;            'drsom_is_switchover_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_switched_over' => 'false',
##&nbsp;            'drsom_host' => 'DR-PRIMARY',
##&nbsp;            'drsom_is_sb_in_progress_due_to_so_failures' => 'false',
##&nbsp;            'drsom_is_sb_in_progress_due_to_so_panic' => 'false',
##&nbsp;            'drsom_is_nso_source_commit_complete' => 'false',
##&nbsp;            'drsom_is_auxiliary_switchover_in_progress' => 'false',
##&nbsp;            'drsom_is_sb_in_progress_due_to_so_abort_reboot' => 'false',
##&nbsp;            'drsom_get_force_nvfail_all_option' => 'false',
##&nbsp;            'drsom_is_primary_heal_roots_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_state_heal_aggrs_complete' => 'false',
##&nbsp;            'drsom_is_switchback_in_progress_normal' => 'false',
##&nbsp;            'drsom_is_primary_heal_aggrs_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_heal_roots_in_progress' => 'false',
##&nbsp;            'remote_cserver_uuid' => 'bdf5a48f-ef64-11e2-8145-123478563412',
##&nbsp;            'drsom_get_wafl_nvinfo_online' => '0',
##&nbsp;            'drsom_is_switchback_retry_in_progress' => 'false',
##&nbsp;            'drsom_is_nso_source_commit_in_progress' => 'false',
##&nbsp;            'drsom_is_nso_source_vetocheck_in_progress' => 'false',
##&nbsp;            'drsom_is_auxiliary_state_switchover_complete' => 'false',
##&nbsp;            'drsom_is_switchback_scheduled' => 'false',
##&nbsp;            'drsom_is_sb_in_progress_due_to_so_retry_reboot' => 'false',
##&nbsp;            'drsom_initialization_done' => 'true',
##&nbsp;            'drsom_is_primary_state_heal_aggrs_complete' => 'false',
##&nbsp;            'drsom_is_switchover_abort_in_progress' => 'false',
##&nbsp;            'drsom_is_primary_switched_over' => 'false'
##&nbsp;         }
##&nbsp;      ];

sub mcc_test_dump_apis_now {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    @lines = grep( !/DRSOM\s+TEST\s+API\s+DUMP/, @lines );
    my @result_array;
    foreach my $line (@lines) {
        my @values = split( /\s*:\s+/, $line );
        $values[0] =~ s/\[|\]//g;
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
    }
    return \@result_array;
}

## @name ::mcc_nvram
##
## @summary Parses the raw output from the "mcc nvram" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mcc_nvram with any of its options as described in the
##    corresponding cdef file 'mcc.cdefs'
## @synopsis
##      mcc_nvram(%args);
## INPUTS TO METHOD CALL 'mcc_nvram' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system node run -node local -command "priv set -q test; mcc nvram; result -l"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;mcc_magic                   : -855978306
##&nbsp;mcc_version                 : 1
##&nbsp;wafl_info_online            : false
##&nbsp;nvcopy_done                 : false
##&nbsp;force_nvfail_all            : false
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wafl_info_online' => 'false',
##&nbsp;            'force_nvfail_all' => 'false',
##&nbsp;            'nvcopy_done' => 'false',
##&nbsp;            'mcc_version' => '1',
##&nbsp;            'mcc_magic' => '-855978306'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub mcc_nvram {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my $rew= raw_output => $output;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    foreach my $line (@lines) {
        my @values = split( /\s*:\s+/, $line );
        $values[0] =~ s/\[|\]//g;
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
    }
    return \@result_array;
}

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~
        /mcc\s+commands\s+not\s+supported\s+on\s+non-mcc\s+setups/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

