#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/mem_stats.pm#  $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode mem_stats command
##
## @description
##    This is a library to parse the raw output of 7Mode mem_stats  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use mem_stats ;
##
##    mem_stats(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mem_stats;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::mem_stats
##
## @summary Parses the raw output from the "mem_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mem_stats with any of its options as described in the
##    corresponding cdef file 'mem_stats.cdefs'
## @synopsis
##      mem_stats(%args);
## INPUTS TO METHOD CALL 'mem_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'clean-up' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'dump' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapshot' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'process' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'positive-changes' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; mem_stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes' => '2064',
##&nbsp;            'location' => 'maytag.ko::wafl_init_disks+40',
##&nbsp;            'allocs' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bytes' => '192',
##&nbsp;            'location' => 'maytag.ko::vdb_entry_fsid_create+54',
##&nbsp;            'allocs' => '6'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bytes' => '7240',
##&nbsp;            'location' => 'maytag.ko::fct_shdmem_init+26',
##&nbsp;            'allocs' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bytes' => '63552',
##&nbsp;            'location' => 'nbladekmod.ko::_ZN10NitroProxy11_bipConnectEP10RouteEntryjjtt+149',
##&nbsp;            'allocs' => '66'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bytes' => '24',
##&nbsp;            'location' => 'maytag.ko::add_subtree+26',
##&nbsp;            'allocs' => '1'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub mem_stats {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'mem_stats',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array = ();
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    # No output indicates that those are internal operations
    return [] if ( !$raw_output );

    foreach my $line (@lines) {
        my %hash = ();
        my ($count1,$count2,$count3,$value1,$value2,$value3);
        my ($count ,$byte ,$loc);
        my $re_hex_word    = "[0-9A-Za-z\$\.\_\+\/\-]+";
        if($line =~ /^(\d+)\s+(\d+)\s+($re_hex_word(\:\:$re_hex_word))$/) {
            $count = $1;
            $byte  = $2;
            $loc   = $3;
            $hash{'allocs'} = $count;
            $hash{'bytes'}  = $byte;
            $hash{'location'} = $loc;
        } elsif($line =~ /^(\d+)\s+($re_hex_word)$/) {
            $byte = $1;
            $loc   = $2;
            $hash{'bytes'}  = $byte;
            $hash{'location'} = $loc;
        }
       push( @result_array, \%hash );
    }
    return \@result_array;
}
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if($raw_output =~ /(Invalid option)/) {
        $$error_message = $1;
        $error          = "InvalidParam";
    } elsif($raw_output =~ /(No saved copy of memory statistics)/) {
        $$error_message = $1;
        $error = "CommandFailed";
    }
    return $error;
}
1;
