#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Maint mode mini_disk command output
##
## @description
##    This is a library to parse the raw output of Maint mode mini_disk command.
##
## @synopsis
##    use mini_disk;
##
##    mini_disk_list(%args);
##    mini_disk_create(%args);
##    mini_disk_delete(%args);
##    mini_disk_zero(%args);
##
## @author bhaskarm@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mini_disk;
use strict;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command       = $args{command};
    my $error;
    if ($raw_output =~ /[Uu]sage:/) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage";
    } elsif ($raw_output =~ /Invalid mini_disk size|does not exist/) {
        $error  = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /$command:/) {
        $error = "CommandFailed";
        $$error_message = $raw_output;
    }        
    return $error;
}

## @name ::mini_disk_list
##
## @summary Parses the raw output from the "mini_disk list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mini_disk_list with any of its options as described in the 
##    corresponding cdef file 'mini_disk.cdefs'
## @synopsis
##      mini_disk_list(%args);
## INPUTS TO METHOD CALL 'mini_disk_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mini_disk list
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'v4.16',
##&nbsp;            'type' => 'NORMAL disk'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'v4.29',
##&nbsp;            'type' => 'NORMAL disk'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'v4.32',
##&nbsp;            'type' => 'NORMAL disk'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'v5.22',
##&nbsp;            'type' => 'minidisk',
##&nbsp;            'size' => '600MB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'disk' => 'v5.24',
##&nbsp;            'type' => 'NORMAL disk'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub mini_disk_list {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw = 0;
    my (@final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => 'mini_disk',
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = 'Disk';
    my @lines=get_lines('raw_output'=>$output,'can_throw'=>$can_throw,'first_line'=>$first_line);
    foreach my $line (@lines) {
        if($line =~ /Disk\s+(\S+):\s+is configured as a\s+(\S+)\s+(\S+)/) {
            my $hash = {};
            $hash->{disk} = $1;
            my $size = $2;
            my $type = $3;
            if ($type =~ /^disk$/){
                $hash->{type} = $size." ".$type;
            } else {
                $hash->{type} = $type;
                $hash->{size} = $size;
            }
            push @final,$hash;
        }
    }  
    return (\@final);
}

## @name ::mini_disk_create
##
## @summary Parses the raw output from the "mini_disk create" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mini_disk_create with any of its options as described in the 
##    corresponding cdef file 'mini_disk.cdefs'
## @synopsis
##      mini_disk_create(%args);
## INPUTS TO METHOD CALL 'mini_disk_create' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'size' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mini_disk create v5.23
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub mini_disk_create {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw = 0;
    my (@final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => 'mini_disk',
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return (\@final);
}

## @name ::mini_disk_delete
##
## @summary Parses the raw output from the "mini_disk delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mini_disk_delete with any of its options as described in the 
##    corresponding cdef file 'mini_disk.cdefs'
## @synopsis
##      mini_disk_delete(%args);
## INPUTS TO METHOD CALL 'mini_disk_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mini_disk delete v5.23
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub mini_disk_delete {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw = 0;
    my (@final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => 'mini_disk',
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return (\@final);
}

## @name ::mini_disk_zero
##
## @summary Parses the raw output from the "mini_disk zero" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mini_disk_zero with any of its options as described in the 
##    corresponding cdef file 'mini_disk.cdefs'
## @synopsis
##      mini_disk_zero(%args);
## INPUTS TO METHOD CALL 'mini_disk_zero' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = mini_disk zero all
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 =  [
##&nbsp;          {
##&nbsp;            'status' => 'done',
##&nbsp;          },
##&nbsp;         ]
##&nbsp;

sub mini_disk_zero {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw = 0;
    my (@final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => 'mini_disk',
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if($output =~ /mini disk zeroing operation is done/) {
        my $hash = {};
        $hash->{status} = "done";
        push @final,$hash;
    }
    return (\@final);
}

1;

