# Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/mt.pm#2 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode mt commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode mt
##    commands.
##
## @synopsis
##    use mt;
##
##    mt(%args);
##
## @author dl-dpg-nb-automation;dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mt;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);


## @name ::mt
##
## @summary Parses the raw output from the "mt" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mt with any of its options as described in the 
##    corresponding cdef file 'mt.cdefs'
## @synopsis
##      mt(%args);
## INPUTS TO METHOD CALL 'mt' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'weof' </td> <td> 'write-end-of-file' </td> </tr> 
## <tr> <td> 'status' </td> <td> 'status' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'tape-device-switch2' </td> </tr> 
## <tr> <td> 'diag' </td> <td> 'diagnostic-messages' </td> </tr> 
## <tr> <td> 'eof' </td> <td> 'end-of-file' </td> </tr> 
## <tr> <td> 'eom' </td> <td> 'end-of-data' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'tape-device-switch1' </td> </tr> 
## <tr> <td> 'fsf' </td> <td> 'forward-over-filemarks' </td> </tr> 
## <tr> <td> 'bsf' </td> <td> 'backward-over-filemarks' </td> </tr> 
## <tr> <td> 'offline' </td> <td> 'offline' </td> </tr> 
## <tr> <td> 'erase' </td> <td> 'erase' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'count' </td> </tr> 
## <tr> <td> 'bsr' </td> <td> 'backward-over-records' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tape-name' </td> </tr> 
## <tr> <td> 'fsr' </td> <td> 'forward-over-records' </td> </tr> 
## <tr> <td> 'rewind' </td> <td> 'rewind' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub mt {
    my %args  = @_;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my (%res_hash,$error_message);


    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>$command_line,
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    if ($command_line =~ /status/i) {
        if ($raw_output =~ /Tape drive:/i) {
            ($res_hash{"tape_drive"}) = ($raw_output =~ /Tape drive:\s+(.+)/i);
            $res_hash{"tape_drive"} =~ s/\s\s+/ /g;
            $res_hash{"tape_drive"} =~ s/\s$//;
            ($res_hash{"status"}) = ($raw_output =~ /Status:\s+(.+)/i);
            ($res_hash{"format"}) = ($raw_output =~ /Format:\s+(.+)/i);
            $res_hash{"format"} =~ s/\s+$//;
            ($res_hash{"fileno"}, $res_hash{"blockno"}, $res_hash{"resid"}) =
            ($raw_output =~ /fileno\s+?=\s+?(.+?)\s+?blockno\s+?=\s+?(.+?)\s+?resid\s+?=\s+?(.+?).*/i);
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                          text =>"Output is not in the expected format",
                                           response_object => $response);
        }
        return [\%res_hash];
    } else {
        # If there is no exception thrown and this part of the code is reached,
        # then it implies that the command had succeeded.
        #Return Empty array reference implies that command has succeeded
        return []; 
    }
}

sub _check_error {
    my %args = @_;
    my $raw_output = $args{"raw_output"};
    my $command = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    if( ($raw_output =~ /(mt.+?\sfailed.*)/i) or
        ($raw_output =~ /(mt.+?\scurrently in use \.Access denied.*)/i) or
        ($raw_output =~ /(mt.+?Tape device.+?is not a valid device name.*)/i) or
        ($raw_output =~ /(mt.+?No media loaded for tape device.*)/i) or
        ($raw_output =~ /(mt.+?Additional commands on same line not supported.*)/i) ) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif( ($raw_output =~ /(mt:\sUnrecognized command.*)/i) or
              ( $raw_output =~ /(mt:\sMissing count argument.*)/i) or
              ( $raw_output =~ /(mt:.*\r*\n*usage:\smt.*)/i) or 
              ( $raw_output =~ /(mt:\sInvalid argument.*)/i) ) {

        $error = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(mt.+Illegal count.*)/i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } else {
        return undef;
    }

    # Remove ctrl-M from error message which is taken from raw output.
    $$error_message =~ s/\cM//g;
    $$error_message =~ s/^$command:\s//;

    return $error;
}

1;
