#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode mv commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode mv commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mv" will be mv()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::mv;
##
##    mv(%args);
##
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mv;

use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if ( $raw_output =~ /No such file or directory/) {
        $$error_message = $raw_output;
        $error = "CommandFailed";
    } elsif ($raw_output =~ /[Uu]sage\:/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::mv
##
## @summary Parses the raw output from the "mv" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mv with any of its options as described in the
##    corresponding cdef file 'mv.cdefs'
## @synopsis
##      mv(%args);
## INPUTS TO METHOD CALL 'mv' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-file' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-file' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = mv
##&nbsp;$VAR1 = [];
##&nbsp;
sub mv {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                            "command"=>"mv",
                            "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
}
1;
