#
# Copyright (c) 2014 NetApp.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw_mfile commands
##
## @description
##    This is a library to parse the raw output of node scope metawafl mw_mfile commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw_mfile file_create" will be mw_mfile_file_create()
##
## @synopsis
##    use mw_mfile;
##
##    mw_mfile_file_create(%args);
##    mw_mfile_file_load(%args);
##    mw_mfile_file_unload(%args);
##    mw_mfile_file_delete(%args);
##    mw_mfile_key_insert(%args);
##    mw_mfile_key_delete(%args);
##    mw_mfile_key_lookup(%args);
##    mw_mfile_key_prefetch(%args);
##
## @author chenp@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mw_mfile;

use strict;
use warnings;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (
    mw_mfile_file_create => {
        cmd_pass_string => 'created successfully',
    },

    mw_mfile_file_load => {
        cmd_pass_string => 'loaded successfully',
    },

    mw_mfile_file_unload => {
        cmd_pass_string => 'unloaded successfully',
    },

    mw_mfile_file_delete => {
        cmd_pass_string => 'deleted successfully',
    },

    mw_mfile_file_grow => {
        cmd_pass_string => 'grown successfully',
    },

    mw_mfile_key_insert   => {
        cmd_pass_string => 'inserted successfully',
    },

    mw_mfile_key_delete   => {
        cmd_pass_string => 'deleted successfully',
    },

    mw_mfile_key_lookup   => {
        canned_output_parser => 'key_colon_value',
        remove_pattern_from_all_lines => 'Key\s|Value\s',
    },

    mw_mfile_key_prefetch => {
        cmd_pass_string => 'prefetched successfully',
    },

);

parser_method_builder(__PACKAGE__, %method_hash);

1;

## @name ::mw_mfile_file_create
##
## @summary Parses the raw output from the "mw_mfile file_create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_file_create with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_file_create(%args);
## INPUTS TO METHOD CALL 'mw_mfile_file_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION          </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr'         </td> <td> 'aggregate'    </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-shape'        </td> <td> 'shape'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-size'         </td> <td> 'size'         </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-plsize'       </td> <td> 'plsize'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-l0data'       </td> <td> 'l0data'       </td> <td> 1 </td> </tr>
## <tr> <td> '-fixed_fileid' </td> <td> 'fixed_fileid' </td> <td> 1 </td> </tr>
## <tr> <td> '-ndx0'         </td> <td> 'ndx0'         </td> <td> 1 </td> </tr>
## <tr> <td> '-l0bpp'         </td> <td> 'ndx0'         </td> <td> 1 </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_file_load
##
## @summary Parses the raw output from the "mw_mfile file_load" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_file_load with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_file_load(%args);
## INPUTS TO METHOD CALL 'mw_mfile_file_load' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION          </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr'         </td> <td> 'aggregate'    </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-shape'        </td> <td> 'shape'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-size'         </td> <td> 'size'         </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-plsize'       </td> <td> 'plsize'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-l0data'       </td> <td> 'l0data'       </td> <td> 1 </td> </tr>
## <tr> <td> '-fixed_fileid' </td> <td> 'fixed_fileid' </td> <td> 1 </td> </tr>
## <tr> <td> '-ndx0'         </td> <td> 'ndx0'         </td> <td> 1 </td> </tr>
## <tr> <td> '-l0bpp'       </td> <td> 'l0data'       </td> <td> 1 </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_file_unload
##
## @summary Parses the raw output from the "mw_mfile file_unload" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_file_unload with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_file_unload(%args);
## INPUTS TO METHOD CALL 'mw_mfile_file_unload' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION  </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr' </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_file_delete
##
## @summary Parses the raw output from the "mw_mfile file_delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_file_delete with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_file_delete(%args);
## INPUTS TO METHOD CALL 'mw_mfile_file_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION  </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr' </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_key_insert
##
## @summary Parses the raw output from the "mw_mfile key_insert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_key_insert with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_key_insert(%args);
## INPUTS TO METHOD CALL 'mw_mfile_key_insert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION      </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION       </th> </tr>
## <tr> <td> '-aggr'     </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-num_keys' </td> <td> 'num_keys'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-keys'     </td> <td> 'keys'            </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-values'   </td> <td> 'values'          </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_key_delete
##
## @summary Parses the raw output from the "mw_mfile key_delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_key_delete with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_key_delete(%args);
## INPUTS TO METHOD CALL 'mw_mfile_key_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION      </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION       </th> </tr>
## <tr> <td> '-aggr'     </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-num_keys' </td> <td> 'num_keys'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-keys'     </td> <td> 'keys'            </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

## @name ::mw_mfile_key_lookup
##
## @summary Parses the raw output from the "mw_mfile key_lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_key_lookup with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_key_lookup(%args);
## INPUTS TO METHOD CALL 'mw_mfile_key_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION      </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION       </th> </tr>
## <tr> <td> '-aggr'     </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-num_keys' </td> <td> 'num_keys'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-keys'     </td> <td> 'keys'            </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @example
## mw_mfile file_get_wip -aggr mfile_aggr
##
## Key 1: Value 100
##
## Parsed output: $VAR1 = [
##           {
##             'value' => '100',
##             'key' => '1'
##           }
##         ];

## @name ::mw_mfile_key_prefetch
##
## @summary Parses the raw output from the "mw_mfile key_prefetch" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_mfile_key_prefetch with any of its options as described in the
##    corresponding cdef file 'mw_mfile.cdefs'
## @synopsis
##      mw_mfile_key_prefetch(%args);
## INPUTS TO METHOD CALL 'mw_mfile_key_prefetch' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION      </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION       </th> </tr>
## <tr> <td> '-aggr'     </td> <td> 'aggregate'       </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-num_keys' </td> <td> 'num_keys'        </td> <td> Not a boolean option </td> </tr>
## <tr> <td> '-keys'     </td> <td> 'keys'            </td> <td> Not a boolean option </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value

