#
# Copyright (c) 2015 NetApp.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw_stage commands
##
## @description
##    This is a library to parse the raw output of node scope metawafl mw_stage commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw_stage_test" will be mw_stage_test()
##
## @synopsis
##    use mw_stage;
##
##    mw_stage_test(%args);
##    mw_stage_stat(%args);
##    mw_stage_zerostat(%args);
##
## @author chenp@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mw_stage;

use strict;
use warnings;

use Data::Dumper;

use List::MoreUtils qw(natatime);

use NATE::BaseException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (

    mw_stage_test => {
        cmd_pass_string      => 'Warning: Onlining or Creating Vvols on this Aggr during the test may result in inconsistency',
    },

    mw_stage_stat => {
        output_parser_method => '_stage_stat_parser',
    },

    mw_stage_zerostat => {},

);

parser_method_builder(__PACKAGE__, %method_hash);

=head
Sample output for the parser
Original output:
        Op   Level   Total        Empty        Stages       Eids         Stages       Eids
                     Stages       Stages frd   added        added        committed    committed
                     acquired     on Susp      in frntend   in frntend   in Bckend    in Bckend
        ---- ------- ------------ ------------ ------------ ------------ ------------ ------------
        Decr 0       468          143          325          10415        325          10415
        Decr 1       848          213          635          20788        635          20788
        Decr 2       175          43           132          4431         132          4431
        Decr 3       517          124          393          12759        393          12759
        Decr 4       918          210          708          24314        708          24314
        Decr 5       1433         306          1127         37848        1127         37848
        Decr 6       2094         427          1667         55379        1667         55379
        Decr 7       2715         466          2249         75278        2249         75278
        Decr 8       3602         531          3071         104137       3071         104137
        Decr 9       3823         428          3395         112835       3395         112835
        Decr 10      586          68           518          17385        518          17385
        Decr 11      1321         142          1179         39374        1179         39374
        Decr 12      2199         249          1950         65720        1950         65720
        Decr 13      3132         294          2838         94902        2838         94902
        Decr 14      4129         319          3810         126606       3810         126606
        Decr 15      5347         290          5057         166938       5057         166938
        Decr 16      6601         178          6423         216244       6423         216244
        Decr 17      3334         3            3331         213048       3331         213048
        Decr 18      0            0            0            0            0            0
        Incr 0       462          112          350          11445        350          11445
        Incr 1       906          213          693          22947        693          22947
        Incr 2       203          51           152          4831         152          4831
        Incr 3       543          129          414          14032        414          14032
        Incr 4       937          221          716          23973        716          23973
        Incr 5       1429         294          1135         38231        1135         38231
        Incr 6       2109         381          1728         58316        1728         58316
        Incr 7       2878         450          2428         80477        2428         80477
        Incr 8       3761         489          3272         110434       3272         110434
        Incr 9       3963         388          3575         118801       3575         118801
        Incr 10      612          61           551          18407        551          18407
        Incr 11      1345         129          1216         41355        1216         41355
        Incr 12      2128         206          1922         64582        1922         64582
        Incr 13      3031         242          2789         92884        2789         92884
        Incr 14      4234         325          3909         130045       3909         130045
        Incr 15      5353         295          5058         169324       5058         169324
        Incr 16      6730         229          6501         217628       6501         217628
        Incr 17      6482         23           6459         216578       6459         216578
        Incr 18      0            0            0            0            0            0

Parsed output:
[
        {   'stages_committed_in_bckend' => '325',
            'total_stages_acquired'      => '468',
            'eids_added_in_frntend'      => '10415',
            'level'                      => '0',
            'empty_stages_frd_on_susp'   => '143',
            'stages_added_in_frntend'    => '325',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '10415'
        },
        {   'stages_committed_in_bckend' => '635',
            'total_stages_acquired'      => '848',
            'eids_added_in_frntend'      => '20788',
            'level'                      => '1',
            'empty_stages_frd_on_susp'   => '213',
            'stages_added_in_frntend'    => '635',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '20788'
        },
        {   'stages_committed_in_bckend' => '132',
            'total_stages_acquired'      => '175',
            'eids_added_in_frntend'      => '4431',
            'level'                      => '2',
            'empty_stages_frd_on_susp'   => '43',
            'stages_added_in_frntend'    => '132',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '4431'
        },
        {   'stages_committed_in_bckend' => '393',
            'total_stages_acquired'      => '517',
            'eids_added_in_frntend'      => '12759',
            'level'                      => '3',
            'empty_stages_frd_on_susp'   => '124',
            'stages_added_in_frntend'    => '393',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '12759'
        },
        {   'stages_committed_in_bckend' => '708',
            'total_stages_acquired'      => '918',
            'eids_added_in_frntend'      => '24314',
            'level'                      => '4',
            'empty_stages_frd_on_susp'   => '210',
            'stages_added_in_frntend'    => '708',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '24314'
        },
        {   'stages_committed_in_bckend' => '1127',
            'total_stages_acquired'      => '1433',
            'eids_added_in_frntend'      => '37848',
            'level'                      => '5',
            'empty_stages_frd_on_susp'   => '306',
            'stages_added_in_frntend'    => '1127',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '37848'
        },
        {   'stages_committed_in_bckend' => '1667',
            'total_stages_acquired'      => '2094',
            'eids_added_in_frntend'      => '55379',
            'level'                      => '6',
            'empty_stages_frd_on_susp'   => '427',
            'stages_added_in_frntend'    => '1667',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '55379'
        },
        {   'stages_committed_in_bckend' => '2249',
            'total_stages_acquired'      => '2715',
            'eids_added_in_frntend'      => '75278',
            'level'                      => '7',
            'empty_stages_frd_on_susp'   => '466',
            'stages_added_in_frntend'    => '2249',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '75278'
        },
        {   'stages_committed_in_bckend' => '3071',
            'total_stages_acquired'      => '3602',
            'eids_added_in_frntend'      => '104137',
            'level'                      => '8',
            'empty_stages_frd_on_susp'   => '531',
            'stages_added_in_frntend'    => '3071',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '104137'
        },
        {   'stages_committed_in_bckend' => '3395',
            'total_stages_acquired'      => '3823',
            'eids_added_in_frntend'      => '112835',
            'level'                      => '9',
            'empty_stages_frd_on_susp'   => '428',
            'stages_added_in_frntend'    => '3395',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '112835'
        },
        {   'stages_committed_in_bckend' => '518',
            'total_stages_acquired'      => '586',
            'eids_added_in_frntend'      => '17385',
            'level'                      => '10',
            'empty_stages_frd_on_susp'   => '68',
            'stages_added_in_frntend'    => '518',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '17385'
        },
        {   'stages_committed_in_bckend' => '1179',
            'total_stages_acquired'      => '1321',
            'eids_added_in_frntend'      => '39374',
            'level'                      => '11',
            'empty_stages_frd_on_susp'   => '142',
            'stages_added_in_frntend'    => '1179',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '39374'
        },
        {   'stages_committed_in_bckend' => '1950',
            'total_stages_acquired'      => '2199',
            'eids_added_in_frntend'      => '65720',
            'level'                      => '12',
            'empty_stages_frd_on_susp'   => '249',
            'stages_added_in_frntend'    => '1950',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '65720'
        },
        {   'stages_committed_in_bckend' => '2838',
            'total_stages_acquired'      => '3132',
            'eids_added_in_frntend'      => '94902',
            'level'                      => '13',
            'empty_stages_frd_on_susp'   => '294',
            'stages_added_in_frntend'    => '2838',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '94902'
        },
        {   'stages_committed_in_bckend' => '3810',
            'total_stages_acquired'      => '4129',
            'eids_added_in_frntend'      => '126606',
            'level'                      => '14',
            'empty_stages_frd_on_susp'   => '319',
            'stages_added_in_frntend'    => '3810',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '126606'
        },
        {   'stages_committed_in_bckend' => '5057',
            'total_stages_acquired'      => '5347',
            'eids_added_in_frntend'      => '166938',
            'level'                      => '15',
            'empty_stages_frd_on_susp'   => '290',
            'stages_added_in_frntend'    => '5057',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '166938'
        },
        {   'stages_committed_in_bckend' => '6423',
            'total_stages_acquired'      => '6601',
            'eids_added_in_frntend'      => '216244',
            'level'                      => '16',
            'empty_stages_frd_on_susp'   => '178',
            'stages_added_in_frntend'    => '6423',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '216244'
        },
        {   'stages_committed_in_bckend' => '3331',
            'total_stages_acquired'      => '3334',
            'eids_added_in_frntend'      => '213048',
            'level'                      => '17',
            'empty_stages_frd_on_susp'   => '3',
            'stages_added_in_frntend'    => '3331',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '213048'
        },
        {   'stages_committed_in_bckend' => '0',
            'total_stages_acquired'      => '0',
            'eids_added_in_frntend'      => '0',
            'level'                      => '18',
            'empty_stages_frd_on_susp'   => '0',
            'stages_added_in_frntend'    => '0',
            'op'                         => 'Decr',
            'eids_committed_in_bckend'   => '0'
        },
        {   'stages_committed_in_bckend' => '350',
            'total_stages_acquired'      => '462',
            'eids_added_in_frntend'      => '11445',
            'level'                      => '0',
            'empty_stages_frd_on_susp'   => '112',
            'stages_added_in_frntend'    => '350',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '11445'
        },
        {   'stages_committed_in_bckend' => '693',
            'total_stages_acquired'      => '906',
            'eids_added_in_frntend'      => '22947',
            'level'                      => '1',
            'empty_stages_frd_on_susp'   => '213',
            'stages_added_in_frntend'    => '693',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '22947'
        },
        {   'stages_committed_in_bckend' => '152',
            'total_stages_acquired'      => '203',
            'eids_added_in_frntend'      => '4831',
            'level'                      => '2',
            'empty_stages_frd_on_susp'   => '51',
            'stages_added_in_frntend'    => '152',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '4831'
        },
        {   'stages_committed_in_bckend' => '414',
            'total_stages_acquired'      => '543',
            'eids_added_in_frntend'      => '14032',
            'level'                      => '3',
            'empty_stages_frd_on_susp'   => '129',
            'stages_added_in_frntend'    => '414',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '14032'
        },
        {   'stages_committed_in_bckend' => '716',
            'total_stages_acquired'      => '937',
            'eids_added_in_frntend'      => '23973',
            'level'                      => '4',
            'empty_stages_frd_on_susp'   => '221',
            'stages_added_in_frntend'    => '716',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '23973'
        },
        {   'stages_committed_in_bckend' => '1135',
            'total_stages_acquired'      => '1429',
            'eids_added_in_frntend'      => '38231',
            'level'                      => '5',
            'empty_stages_frd_on_susp'   => '294',
            'stages_added_in_frntend'    => '1135',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '38231'
        },
        {   'stages_committed_in_bckend' => '1728',
            'total_stages_acquired'      => '2109',
            'eids_added_in_frntend'      => '58316',
            'level'                      => '6',
            'empty_stages_frd_on_susp'   => '381',
            'stages_added_in_frntend'    => '1728',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '58316'
        },
        {   'stages_committed_in_bckend' => '2428',
            'total_stages_acquired'      => '2878',
            'eids_added_in_frntend'      => '80477',
            'level'                      => '7',
            'empty_stages_frd_on_susp'   => '450',
            'stages_added_in_frntend'    => '2428',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '80477'
        },
        {   'stages_committed_in_bckend' => '3272',
            'total_stages_acquired'      => '3761',
            'eids_added_in_frntend'      => '110434',
            'level'                      => '8',
            'empty_stages_frd_on_susp'   => '489',
            'stages_added_in_frntend'    => '3272',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '110434'
        },
        {   'stages_committed_in_bckend' => '3575',
            'total_stages_acquired'      => '3963',
            'eids_added_in_frntend'      => '118801',
            'level'                      => '9',
            'empty_stages_frd_on_susp'   => '388',
            'stages_added_in_frntend'    => '3575',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '118801'
        },
        {   'stages_committed_in_bckend' => '551',
            'total_stages_acquired'      => '612',
            'eids_added_in_frntend'      => '18407',
            'level'                      => '10',
            'empty_stages_frd_on_susp'   => '61',
            'stages_added_in_frntend'    => '551',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '18407'
        },
        {   'stages_committed_in_bckend' => '1216',
            'total_stages_acquired'      => '1345',
            'eids_added_in_frntend'      => '41355',
            'level'                      => '11',
            'empty_stages_frd_on_susp'   => '129',
            'stages_added_in_frntend'    => '1216',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '41355'
        },
        {   'stages_committed_in_bckend' => '1922',
            'total_stages_acquired'      => '2128',
            'eids_added_in_frntend'      => '64582',
            'level'                      => '12',
            'empty_stages_frd_on_susp'   => '206',
            'stages_added_in_frntend'    => '1922',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '64582'
        },
        {   'stages_committed_in_bckend' => '2789',
            'total_stages_acquired'      => '3031',
            'eids_added_in_frntend'      => '92884',
            'level'                      => '13',
            'empty_stages_frd_on_susp'   => '242',
            'stages_added_in_frntend'    => '2789',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '92884'
        },
        {   'stages_committed_in_bckend' => '3909',
            'total_stages_acquired'      => '4234',
            'eids_added_in_frntend'      => '130045',
            'level'                      => '14',
            'empty_stages_frd_on_susp'   => '325',
            'stages_added_in_frntend'    => '3909',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '130045'
        },
        {   'stages_committed_in_bckend' => '5058',
            'total_stages_acquired'      => '5353',
            'eids_added_in_frntend'      => '169324',
            'level'                      => '15',
            'empty_stages_frd_on_susp'   => '295',
            'stages_added_in_frntend'    => '5058',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '169324'
        },
        {   'stages_committed_in_bckend' => '6501',
            'total_stages_acquired'      => '6730',
            'eids_added_in_frntend'      => '217628',
            'level'                      => '16',
            'empty_stages_frd_on_susp'   => '229',
            'stages_added_in_frntend'    => '6501',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '217628'
        },
        {   'stages_committed_in_bckend' => '6459',
            'total_stages_acquired'      => '6482',
            'eids_added_in_frntend'      => '216578',
            'level'                      => '17',
            'empty_stages_frd_on_susp'   => '23',
            'stages_added_in_frntend'    => '6459',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '216578'
        },
        {   'stages_committed_in_bckend' => '0',
            'total_stages_acquired'      => '0',
            'eids_added_in_frntend'      => '0',
            'level'                      => '18',
            'empty_stages_frd_on_susp'   => '0',
            'stages_added_in_frntend'    => '0',
            'op'                         => 'Incr',
            'eids_committed_in_bckend'   => '0'
        }

]
=cut
sub _stage_stat_parser {

    my $args       = shift;
    my $raw_output = $args->{response_object}->get_processed_output;

    my @lines = get_lines(
        'raw_output' => $raw_output,
        'can_throw'  => 1,
    );

    my @array;
    my @keys = @lines[1 .. 3];
    my @values = @lines[5 .. scalar(@lines)-1];

    my @keys_head = split(/\s{2,}/, $keys[0]);
    my @keys_mid  = ('', '', split(/\s{2,}/, $keys[1]));
    my @keys_last = ('', '', split(/\s{2,}/, $keys[2]));

    my @keys_final = map {
        to_key_name(item => "$keys_head[$_] $keys_mid[$_] $keys_last[$_]")}
     (0 .. scalar(@keys_head)-1);

    for my $value (@values){
        my @each_values = split(/\s+/, $value);
        my %hash = map {$keys_final[$_] => $each_values[$_] } (0 .. scalar(@each_values)-1);

        push @array, \%hash;
    } ## for (@values)

    return \@array;
} ## end _stage_stat_parser

1;
