#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw teh commands
##
## @description
##    This is a library to parse the raw output of dblade metawafl 'mw teh' commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw teh init" will be mw_teh_init()
##
## @synopsis
##    use mw_teh;
##
##    mw_teh_init();
##    mw_teh_insert(%args);
##    mw_teh_delete(%args);
##    mw_teh_lookup(%args);
##    mw_teh_stats();
##
## @author Abhijith.Jain@netapp.com,dl-metawafl-qa@netapp.com

## @status public

package NACL::APISet::Node::CLI::7Mode::default::mw_teh;

use strict;
use warnings;
use NATE::BaseException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (
    mw_teh_init   => {},
    mw_teh_insert => {cmd_pass_string => 'key\(s\) inserted',},
    mw_teh_delete => {cmd_pass_string => 'key\(s\) removed',},
    mw_teh_lookup => {cmd_pass_string => 'key\(s\) found',},
    mw_teh_stats  => {
        canned_output_parser => 'key_colon_value',
        first_line           => 'Total:',
    },
);

parser_method_builder(__PACKAGE__, %method_hash);

1;

## @name ::mw_teh_init
##
## @summary Parses the raw output from the "mw teh init" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_teh_init
## @synopsis
##      mw_teh_init();
## INPUTS TO METHOD CALL 'mw_teh_init' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg no args required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
#
## @name ::mw_teh_insert
##
## @summary Parses the raw output from the "mw teh insert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_teh_insert with any of its options as described in the
##    corresponding cdef file 'teh.cdefs'
## @synopsis
##      mw_teh_insert(%args);
## INPUTS TO METHOD CALL 'mw_teh_insert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr' </td> <td> 'aggregate' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-key' </td> <td> 'key' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-count' </td> <td> 'count' </td><td> Not a boolean option </td></tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
#
## @name ::mw_teh_delete
##
## @summary Parses the raw output from the "mw teh delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_teh_delete with any of its options as described in the
##    corresponding cdef file 'mw_teh.cdefs'
## @synopsis
##      mw_teh_delete(%args);
## INPUTS TO METHOD CALL 'mw_teh_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr' </td> <td> 'aggregate' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-key' </td> <td> 'key' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-count' </td> <td> 'count' </td><td> '1' </td></tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
#
## @name ::mw_teh_lookup
##
## @summary Parses the raw output from the "mw teh lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_teh_lookup with any of its options as described in the
##    corresponding cdef file 'mw_teh.cdefs'
## @synopsis
##      mw_teh_lookup(%args);
## INPUTS TO METHOD CALL 'mw_teh_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-aggr' </td> <td> 'aggregate' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-key' </td> <td> 'key' </td><td> Not a boolean option </td></tr>
## <tr> <td> '-count' </td> <td> 'count' </td><td> '1' </td></tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
#
## @name ::mw_teh_stats
##
## @summary Parses the raw output from the "mw teh stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mw_teh_stats with any of its options as described in the
##    corresponding cdef file 'mw_teh.cdefs'
## @synopsis
##      mw_teh_stats(%args);
## INPUTS TO METHOD CALL 'mw_teh_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg no args required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = priv set -q diag; mw teh stats ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Ehash stats:
##&nbsp; Total:          1
##&nbsp; Inserted:       4
##&nbsp; Deleted:        3
##&nbsp; Lookup Success: 7
##&nbsp; Lookup Failed:  0
##&nbsp; Lock:           14
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;  $VAR1 = [
##&nbsp;  {
##&nbsp;  'lookup_failed' => '0',
##&nbsp;  'lock' => '14',
#&nbsp;  'inserted' => '4',
##&nbsp;  'lookup_success' => '7',
##&nbsp;  'deleted' => '3',
##&nbsp;  'total' => '1'
##&nbsp;  }
##&nbsp;  ];
