#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ndmpd commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ndmpd
##    commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ndmpd on" will be ndmpd_on()
##
## @synopsis
##    use ndmpd;
##
##    ndmpd_on(%args);
##    ndmpd_off(%args);
##    ndmpd_status(%args);
##    ndmpd kill(%args);
##    ndmpd_killall(%args);
##    ndmpd_password(%args);
##    ndmpd_debug(%args);
##    ndmpd_version(%args);
##    ndmpd_probe(%args);
##
## @author dl-dpg-nb-automation; dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ndmpd;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::ndmpd_on
##
## @summary Parses the raw output from the "ndmpd on" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_on with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_on(%args);
## INPUTS TO METHOD CALL 'ndmpd_on' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_on {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd on",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    #  Returning empty array of hash implies, command is succeeded.
    return [];
}

## @name ::ndmpd_off
##
## @summary Parses the raw output from the "ndmpd off" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_off with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_off(%args);
## INPUTS TO METHOD CALL 'ndmpd_off' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_off {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd off",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    #  Returning empty array of hash implies, command is succeeded.
    return [];
}


## @name ::ndmpd_status
##
## @summary Parses the raw output from the "ndmpd status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_status with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_status(%args);
## INPUTS TO METHOD CALL 'ndmpd_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_status {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd status",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpd_status(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

## @name ::ndmpd_debug
##
## @summary Parses the raw output from the "ndmpd debug" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_debug with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_debug(%args);
## INPUTS TO METHOD CALL 'ndmpd_debug' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'stack' </td> <td> 'debug-level-stack' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'debug-level' </td> </tr> 
## <tr> <td> 'screen' </td> <td> 'debug-level-screen' </td> </tr> 
## <tr> <td> 'file' </td> <td> 'debug-level-file' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_debug {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd debug",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpd_debug(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

## @name ::ndmpd_kill
##
## @summary Parses the raw output from the "ndmpd kill" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_kill with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_kill(%args);
## INPUTS TO METHOD CALL 'ndmpd_kill' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'session-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_kill {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>$command_line,
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    $result = [];

    return $result;
}

## @name ::ndmpd_killall
##
## @summary Parses the raw output from the "ndmpd killall" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_killall with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_killall(%args);
## INPUTS TO METHOD CALL 'ndmpd_killall' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_killall {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd killall",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = [];

    return $result;
}

## @name ::ndmpd_password
##
## @summary Parses the raw output from the "ndmpd password" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_password with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_password(%args);
## INPUTS TO METHOD CALL 'ndmpd_password' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'password-for-non-root-user-account' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_password {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd password",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpd_password(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

## @name ::ndmpd_version
##
## @summary Parses the raw output from the "ndmpd version" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_version with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_version(%args);
## INPUTS TO METHOD CALL 'ndmpd_version' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ndmpd-max-version' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_version {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd version",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpd_version(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

## @name ::ndmpd_probe
##
## @summary Parses the raw output from the "ndmpd probe" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpd_probe with any of its options as described in the 
##    corresponding cdef file 'ndmpd.cdefs'
## @synopsis
##      ndmpd_probe(%args);
## INPUTS TO METHOD CALL 'ndmpd_probe' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'session-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpd_probe {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"ndmpd probe",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        #$error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error = "NACL::APISet::Exceptions::${error}Exception"; 
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpd_probe(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

sub _ndmpd_status {

    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array,$mystring,@final_array, @empty_array);

    my $can_throw =0;

    #Converting the keys to standard format, similar to other output lines
    $raw_output =~ s/mover state/mover.state/g;
    $raw_output =~ s/data state/data.state/g;
    $raw_output =~ s/data operation/data.operation/g;

    my $first_line = 'ndmpd .+';
    # Convert the raw_output into lines before parsing
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    # Always the first line is about the status of ndmpd
    $lines[0] =~ /ndmpd (ON|OFF)/;
    $res_hash{"ndmpd_status"} = $1;

    # Remove the first line which is processed already
    shift(@lines);

    if((scalar(@lines) == 1) ||($lines[0] =~ /No ndmpd sessions active/)) {
       # Currently there is no active ndmpd session active
       # Returing empty array of hash implies that no session is active

        $res_hash{"ndmpd_session_info"} = [];
        push(@result_array, \%res_hash);
        return \@result_array;

    } else {

    my (@TempArray, $index, $temp_reference);
    # First split the data into logical blocks
    for(my $i=0;$i<@lines;$i++) {

        if($lines[$i] =~ /^Session:/) {
                # The next logical block has started
                $TempArray[$index] = [];
                $temp_reference=$TempArray[$index++];
                push(@$temp_reference, $lines[$i]);

        } else {
                push(@$temp_reference, $lines[$i]);
        }
    }
        my @entries;
        # Now process the data in each block
        foreach my $ref (@TempArray) {
            my @data = @$ref;
            my %hash;
            for(my $j=0; $j<@data; $j++) {
                if ($data[$j] =~ /(.+):\s+(.+)/i) {
                    my $key = $1;
                    my $value= $2;
                    $key = to_key_name("item" => $key);
                    $hash{$key} = $value;
                } else {
                    $mystring = $data[$j];
                    $mystring =~ m/(\w+)/;
                    my $key = to_key_name ("item" => $1);
                    $hash{$key} = $data[$j];
                }
            }
            push(@entries, \%hash);
        }
        $res_hash{"ndmpd_session_info"} = \@entries;
        push(@final_array, \%res_hash);
        return \@final_array;
    }
}

sub _ndmpd_debug {

    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array,$mystring);
    my ( $index, $temp_reference, %hash);
    my $can_throw =0;

    my $first_line = 'ndmpd debug .+';
    # Convert the raw_output into lines before parsing
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    my @entries;
    #Reading the raw_output line by line
    foreach my $line(@lines) {
        my $data = scalar (@lines);

        for(my $j=0; $j<$data; $j++) {

            #Based on semicolon, we are extracting Keys and value
            if ($lines[$j] =~ /(.+):\s+(.+)/i) {
                my $key = $1;
                my $value= $2;

                $key = to_key_name("item" => $key);
                $hash{$key} = $value;
            }
        }
    }
        push(@entries, \%hash);
        return \@entries;


}

sub _ndmpd_password {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array);

    my $can_throw = 0;
    my $first_line = '.*User.*|.*NDMP.*|.*password.*';

    # Convert the raw_output into lines before parsing
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    # the user has ndmp login capability and hence password is returned, which we parse
    $lines[0] =~ m|^password (.+)$|i;
    $res_hash{"password"} = $1;

    push(@result_array, \%res_hash);
    return \@result_array;
}

sub _ndmpd_version {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array);

    $raw_output =~ /ndmpd (highest ){0,1}version set to: (\d+)/i;
    $res_hash{"version"} = $2;
    if($1)
    {
        $res_hash{"action"} = "display"
    } else {
        $res_hash{"action"} = "set"
    }

    push(@result_array, \%res_hash);
    return \@result_array;
}

sub _ndmpd_probe {

    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, %session_hash, @lines, $line, $session_id, $mystring, @final_array);

    my $can_throw =1;

    #Converting the keys to standard format, similar to other output lines
    $raw_output =~ s/protocol version/protocol.version/g;
    $raw_output =~ s/client addr/client.addr/g;
    $raw_output =~ s/mover state/mover.state/g;

    my $first_line = 'ndmpd .+';
    # Convert the raw_output into lines before parsing
    @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    # Always the first line is about the status of ndmpd
    $lines[0] =~ /ndmpd (ON|OFF)/;
    $res_hash{"ndmpd_status"} = $1;

    shift (@lines);

    if((scalar(@lines) == 1) ||($raw_output =~ /No ndmpd sessions active/)) {
        # Currently there is no active ndmpd session active
        # Returing empty array of hash implies that no session is active
        $res_hash{"ndmp_session_info"} = [];
    } else {
        my (@TempArray, $index, $temp_reference);
        # First split the data into logical blocks
        for(my $i=0;$i<@lines;$i++) {

           if($lines[$i] =~ /^Session:/) {
                # The next logical block has started
                $TempArray[$index] = [];
                $temp_reference=$TempArray[$index++];
                push(@$temp_reference, $lines[$i]);

           } else {
                push(@$temp_reference, $lines[$i]);
           }
        }
        my @entries;
        # Now process the data in each block
        foreach my $ref (@TempArray) {
            my @data = @$ref;
            my %hash;
            for(my $j=0; $j<@data; $j++) {

                if ($data[$j] =~ /([^\s]+):?\s+(.+)/i) {
                    my $key = $1;
                    my $value= $2;
                    $key = to_key_name("item" => $key);
                    $hash{$key} = $value;
                } else {
                    $mystring = $data[$j];
                    $mystring =~ m/(\w+)/;
                    $hash{"$1"} = $data[$j];
                }
            }
            push(@entries, \%hash);
        }
        $res_hash{"ndmpd_session_info"} = \@entries;
    }
    return [\%res_hash];
}

sub _check_error {
    my %args =@_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if($raw_output =~ /.*?usage.+/i ) {
        $error = "MissingRequiredParam";
        $$error_message = "missing required parameters";
    } elsif ($raw_output =~ /(Ndmpd session .+ not active)\./i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Unable to retrieve session information)/i ) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Attempt to access invalid session \d+)/i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(No such user .*)/i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(?:.|\n)*?(Cannot generate NDMP password)\./i ) {
        $error = "InvalidParamValue";
        $$error_message = "$1 : User has no 'login-ndmp' capability";
    } elsif ($raw_output =~ /(?:.|\n)*?(No NDMP specific password available for .+)/i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(ndmpd invalid version number: \d+)/i ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } else {
        return undef;
    }

    # Remove ctrl-M from error message which is taken from raw output.
    $$error_message =~ s/\cM//g;
    $$error_message =~ s/^$command:\s//;
    return $error;
}

1;
