#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ndp commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ndp commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ndp" will be ndp()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::ndp;
##
##    ndp(%agrs);
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::7Mode::default::ndp;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $error_message;
    my $error;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /(.+ does not exist)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $1;
    } elsif ( $raw_output =~ /usage:/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(.+ Domain not found)/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::ndp
##
## @summary Parses the raw output from the "ndp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ndp with any of its options as described in the
##    corresponding cdef file 'ndp.cdefs'
## @synopsis
##      ndp(%args);
## INPUTS TO METHOD CALL 'ndp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'parse-file' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'default-interface' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hostname' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'register-ndp-entry' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'harmonize-consistency' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delete-ndp-entry' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'print-timestamp' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'no-truncate' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'flags' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'wait-repeat' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'dump-existing-ndp-entries' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'show-prefix-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'flush-prefix-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'show-default-router-list' </td> <td> '1' </td> </tr>
## <tr> <td> 'temp' </td> <td> 'temp' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ether_addr' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'erase-all-ndp-entries' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-resolve' </td> <td> '1' </td> </tr>
## <tr> <td> 'proxy' </td> <td> 'proxy' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'flush-default-router-list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ndp fe80::215:c6ff:fe20:bd00%e0c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Neighbor                        Linklayer Address  Netif Expire    St Flgs Prbs
##&nbsp;fe80::215:c6ff:fe20:bd00%e0c    0:15:c6:20:bd:0      e0c expired   S  R
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flag_probes' => 'R',
##&nbsp;            'expiry_status' => 'expired',
##&nbsp;            'network_interface' => 'e0c',
##&nbsp;            'linklayer_address' => '0:15:c6:20:bd:0',
##&nbsp;            'neighbour' => 'fe80::215:c6ff:fe20:bd00%e0c',
##&nbsp;            'state' => 'S'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = priv set -q diag; ndp -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fe80::215:c6ff:fe20:bd00%e0c (fe80::215:c6ff:fe20:bd00%e0c) deleted
##&nbsp;fe80::215:c6ff:fe20:c1c0%e0c (fe80::215:c6ff:fe20:c1c0%e0c) deleted
##&nbsp;fe80::215:c6ff:fe20:bd00%e0d (fe80::215:c6ff:fe20:bd00%e0d) deleted
##&nbsp;fe80::215:c6ff:fe20:c1c0%e0d (fe80::215:c6ff:fe20:c1c0%e0d) deleted
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'deleted',
##&nbsp;            'neighbour' => 'fe80::215:c6ff:fe20:bd00%e0c'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'deleted',
##&nbsp;            'neighbour' => 'fe80::215:c6ff:fe20:c1c0%e0c'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'deleted',
##&nbsp;            'neighbour' => 'fe80::215:c6ff:fe20:bd00%e0d'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'deleted',
##&nbsp;            'neighbour' => 'fe80::215:c6ff:fe20:c1c0%e0d'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = priv set -q test; ndp -p; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fd20:8b1e:b255:4071::/64 if=e0c
##&nbsp;  flags=LA vltime=2592000, pltime=604800, expire=29d23h59m12s, origin=RA
##&nbsp;  advertised by
##&nbsp;    fe80::5:73ff:fea0:20%e0c (reachable)
##&nbsp;fd20:8b1e:b255:4071::/64 if=e0d
##&nbsp;  flags=LA vltime=2592000, pltime=604800, expire=29d23h59m12s, origin=RA
##&nbsp;  advertised by
##&nbsp;    fe80::5:73ff:fea0:20%e0d (reachable)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => 'LA',
##&nbsp;            'status' => 'reachable',
##&nbsp;            'origin' => 'RA',
##&nbsp;            'advertised_by' => 'fe80::5:73ff:fea0:20%e0c',
##&nbsp;            'pltime' => '604800',
##&nbsp;            'neighbour' => 'fd20:8b1e:b255:4071::/64',
##&nbsp;            'network_interface' => 'e0c',
##&nbsp;            'vltime' => '2592000',
##&nbsp;            'expire' => '29d23h59m12s'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => 'LA',
##&nbsp;            'status' => 'reachable',
##&nbsp;            'origin' => 'RA',
##&nbsp;            'advertised_by' => 'fe80::5:73ff:fea0:20%e0d',
##&nbsp;            'pltime' => '604800',
##&nbsp;            'neighbour' => 'fd20:8b1e:b255:4071::/64',
##&nbsp;            'network_interface' => 'e0d',
##&nbsp;            'vltime' => '2592000',
##&nbsp;            'expire' => '29d23h59m12s'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; ndp -r; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fe80::5:73ff:fea0:20 if=e0c, flags=, expire=29m12s
##&nbsp;fe80::5:73ff:fea0:20 if=e0d, flags=, expire=29m12s
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => '',
##&nbsp;            'network_interface' => 'e0c',
##&nbsp;            'neighbour' => 'fe80::5:73ff:fea0:20',
##&nbsp;            'expire' => '29m12s'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'flags' => '',
##&nbsp;            'network_interface' => 'e0d',
##&nbsp;            'neighbour' => 'fe80::5:73ff:fea0:20',
##&nbsp;            'expire' => '29m12s'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ndp {
    my %args         = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my @result;
    my $count = 0;
    my $error = _check_error(
        "raw_output"      => $raw_output,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $raw_output,
        response_object => $response );

    foreach my $line (@lines) {
        my %hash;

        if ($line =~ /(\w+::?\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\w)\s*(\w|\s*)/ )
        {
            $hash{'neighbour'}         = $1;
            $hash{'linklayer_address'} = $2;
            $hash{'network_interface'} = $3;
            $hash{'expiry_status'}     = $4;
            $hash{'state'}             = $5;
            $hash{'flag_probes'}       = $6;

            push @result, \%hash;
        } elsif ( $line =~ /(\S+)\s\S+\s(deleted)/ ) {
            $hash{'neighbour'} = $1;
            $hash{'status'}    = $2;

            push @result, \%hash;
        } elsif ( $line =~ /(\w+.*\d+)\s+\w+=(\w+),\s+\w+=(.*)\s+\w+=(.*)/ ) {
            $hash{'neighbour'}         = $1;
            $hash{'network_interface'} = $2;
            $hash{'expire'}            = $4;
            if ( $3 =~ /\S/ ) {
                $hash{'flags'} = '';
            } else {
                $hash{'flags'} = $3;
            }
            push @result, \%hash;
        } elsif ( $line =~ /(\w+.*::\/\d+)\s+\w+=(\w+)/ ) {
            while (@lines) {
                $line = shift(@lines);
                if ( $line =~ /(\w+.*::\/\d+)\s+\w+=(\w+)/ ) {
                    $result[$count]->{'neighbour'}         = $1;
                    $result[$count]->{'network_interface'} = $2;
                }
                $line = shift(@lines);
                if ( $line =~ /\w+=(\w+)\s\w+=(\d+),\s\w+=(\d+),\s\w+=(.*),\s\w+=(\w+)/)
                {
                    $result[$count]->{'flags'}  = $1;
                    $result[$count]->{'vltime'} = $2;
                    $result[$count]->{'pltime'} = $3;
                    $result[$count]->{'expire'} = $4;
                    $result[$count]->{'origin'} = $5;
                }
                shift(@lines);
                $line = shift(@lines);
                if ( $line =~ /(\w+::.*:\d+%\w+)\s+\((\w+)\)/ ) {
                    $result[$count]->{'advertised_by'} = $1;
                    $result[$count]->{'status'}        = $2;
                }
                $count++;
            }
        }
    }
    return \@result;
}    # end of ndp

1;
