#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/netdiag.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode netdiag command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode netdiag command.
##
## @synopsis
##    use netdiag;
##    netdiag(%args);
##
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::netdiag;

use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /[Uu]sage:/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    } elsif($raw_output =~ /No such interface .*/) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::netdiag
##
## @summary Parses the raw output from the "netdiag" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from netdiag with any of its options as described in the
##    corresponding cdef file 'netdiag.cdefs'
## @synopsis
##      netdiag(%args);
## INPUTS TO METHOD CALL 'netdiag' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'summary' </td> <td> 'summary' </td> </tr>
## <tr> <td> 'verbose' </td> <td> 'verbose' </td> </tr>
## <tr> <td> 'debug' </td> <td> 'debug' </td> </tr>
## <tr> <td> 'interface-layer' </td> <td> 'interface-layer' </td> </tr>
## <tr> <td> 'interface' </td> <td> 'interface' </td> </tr>
## <tr> <td> 'network-layer' </td> <td> 'network-layer' </td> </tr>
## <tr> <td> 'transport-layer' </td> <td> 'transport-layer' </td> </tr>
## <tr> <td> 'error-codes' </td> <td> 'error-codes' </td> </tr>
## <tr> <td> 'overall' </td> <td> 'overall' </td> </tr>
## <tr> <td> 'all-layers' </td> <td> 'all-layers' </td> </tr>
## <tr> <td> 'non-interactive' </td> <td> 'non-interactive' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; netdiag -d; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'physical_layer_diagnostics' => [
##&nbsp;                                               'Checking interface e0a',
##&nbsp;                                               'Checking interface e0a for hardware status',
##&nbsp;                                               'Checking interface e0a for link status',
##&nbsp;                                               'The interface e0a appears to be configured UP',
##&nbsp;                                               'media-type is auto-1000t-fd-up',
##&nbsp;                                               'Gigabit Ethernet interface,skipping duplex mismatch test',
##&nbsp;                                               'Interface e0a OK',
##&nbsp;                                               'Checking interface e0b',
##&nbsp;                                               'Checking interface e0b for hardware status',
##&nbsp;                                               'Checking interface e0b for link status',
##&nbsp;                                               'The interface e0b appears to be configured UP',
##&nbsp;                                               'media-type is auto-1000t-fd-up',
##&nbsp;                                               'Gigabit Ethernet interface,skipping duplex mismatch test',
##&nbsp;                                               'Interface e0b OK',
##&nbsp;                                               'Checking interface e0c',
##&nbsp;                                               'Checking interface e0c for hardware status',
##&nbsp;                                               'Checking interface e0c for link status',
##&nbsp;                                               'The interface e0c appears to be configured UP',
##&nbsp;                                               'media-type is auto-1000t-fd-up',
##&nbsp;                                               'Gigabit Ethernet interface,skipping duplex mismatch test',
##&nbsp;                                               'Interface e0c OK',
##&nbsp;                                               'Checking interface e0d',
##&nbsp;                                               'Checking interface e0d for hardware status',
##&nbsp;                                               'Checking interface e0d for link status',
##&nbsp;                                               'The interface e0d appears to be configured UP',
##&nbsp;                                               'media-type is auto-1000t-fd-up',
##&nbsp;                                               'Gigabit Ethernet interface,skipping duplex mismatch test',
##&nbsp;                                               'Interface e0d OK',
##&nbsp;                                               'Checking interface e0a to see if it is wedged',
##&nbsp;                                               'Checking interface e0a for misc errors',
##&nbsp;                                               'Checking interface e0a for capacity problems',
##&nbsp;                                               'Checking interface e0b to see if it is wedged',
##&nbsp;                                               'Checking interface e0b for misc errors',
##&nbsp;                                               'Checking interface e0b for capacity problems',
##&nbsp;                                               'Checking interface e0c to see if it is wedged',
##&nbsp;                                               'Checking interface e0c for misc errors',
##&nbsp;                                               'Checking interface e0c for capacity problems',
##&nbsp;                                               'Checking interface e0d to see if it is wedged',
##&nbsp;                                               'Checking interface e0d for misc errors',
##&nbsp;                                               'Checking interface e0d for capacity problems'
##&nbsp;                                             ],
##&nbsp;             'network_layer_diagnostics' => [
##&nbsp;                                              'Checking for excessive IP reassembly errors',
##&nbsp;                                              'IP is OK',
##&nbsp;                                              'Checking for excessive IPv6 reassembly errors',
##&nbsp;                                              'IPv6 is OK'
##&nbsp;                                            ],
##&nbsp;             'transport_layer_diagnostics' => [
##&nbsp;                                                'Checking for problems with current TCP connections',
##&nbsp;                                                'Checking for problems with recent TCP connections',
##&nbsp;                                                'Checking for listen queue overflow',
##&nbsp;                                                'Checking for filtered connection rejects',
##&nbsp;                                                'Checking for excessive embryonic connection drops',
##&nbsp;                                                'Checking for excessive TCP checksum errors ',
##&nbsp;                                                'Checking for excessive TCP bad received packets ',
##&nbsp;                                                'Checking for excessive UDP checksum errors',
##&nbsp;                                                'TCP is OK',
##&nbsp;                                                'UDP is OK'
##&nbsp;                                              ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = priv set -q diag; netdiag -nbat; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'diagnostics' => [
##&nbsp;                                'Performing network layer diagnostics.....OK',
##&nbsp;                                'Performing transport layer diagnostics.....OK'
##&nbsp;                              ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = priv set -q diag; netdiag -s; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'physical_layer_diagnostics_summary' => [
##&nbsp;                                                       {
##&nbsp;                                                         'link' => 'Y',
##&nbsp;                                                         'speed-mismatch' => 'N',
##&nbsp;                                                         'duplex-mismatch' => '-',
##&nbsp;                                                         'configured-up' => 'Y',
##&nbsp;                                                         'autoneg-mismatch' => '-',
##&nbsp;                                                         'hw-status' => 'OK',
##&nbsp;                                                         'interface' => 'e0a'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'link' => 'Y',
##&nbsp;                                                         'speed-mismatch' => 'N',
##&nbsp;                                                         'duplex-mismatch' => '-',
##&nbsp;                                                         'configured-up' => 'Y',
##&nbsp;                                                         'autoneg-mismatch' => '-',
##&nbsp;                                                         'hw-status' => 'OK',
##&nbsp;                                                         'interface' => 'e0b'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'link' => 'Y',
##&nbsp;                                                         'speed-mismatch' => 'N',
##&nbsp;                                                         'duplex-mismatch' => '-',
##&nbsp;                                                         'configured-up' => 'Y',
##&nbsp;                                                         'autoneg-mismatch' => '-',
##&nbsp;                                                         'hw-status' => 'OK',
##&nbsp;                                                         'interface' => 'e0c'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'link' => 'Y',
##&nbsp;                                                         'speed-mismatch' => 'N',
##&nbsp;                                                         'duplex-mismatch' => '-',
##&nbsp;                                                         'configured-up' => 'Y',
##&nbsp;                                                         'autoneg-mismatch' => '-',
##&nbsp;                                                         'hw-status' => 'OK',
##&nbsp;                                                         'interface' => 'e0d'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;             'network_layer_diagnostics_summary' => [
##&nbsp;                                                      {
##&nbsp;                                                         'protocol' => 'IP',
##&nbsp;                                                        'status' => 'OK'
##&nbsp;                                                      },
##&nbsp;                                                      {
##&nbsp;                                                        'protocol' => 'IPv6',
##&nbsp;                                                        'status' => 'OK'
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;             'transport_layer_diagnostics_summary' => [
##&nbsp;                                                        {
##&nbsp;                                                          'protocol' => 'TCP',
##&nbsp;                                                          'status' => 'OK'
##&nbsp;                                                        },
##&nbsp;                                                        {
##&nbsp;                                                          'protocol' => 'UDP',
##&nbsp;                                                          'status' => 'OK'
##&nbsp;                                                        }
##&nbsp;                                                      ]
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; netdiag -s -p -I e0a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'physical_layer_diagnostics_summary' => [
##&nbsp;                                                       {
##&nbsp;                                                         'link' => 'Y',
##&nbsp;                                                         'speed-mismatch' => 'N',
##&nbsp;                                                         'duplex-mismatch' => '-',
##&nbsp;                                                         'configured-up' => 'Y',
##&nbsp;                                                         'autoneg-mismatch' => '-',
##&nbsp;                                                         'hw-status' => 'OK',
##&nbsp;                                                         'interface' => 'e0a'
##&nbsp;                                                       }
##&nbsp;                                                     ]
##&nbsp;           }
##&nbsp;         ];


sub netdiag {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'netdiag',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array = ();
    my ($i, $j, $name, $m, $k, $p,$value);
    $i = 0;
    $k = 0;
    $p = 0;
    my @lines = get_lines( raw_output => $raw_output,
	                       can_throw => 0, response_object => $resp );
    foreach my $line (@lines) {
        if($line =~ /Physical Layer Diagnostics Summary:$/) {
            $name = "physical_layer_diagnostics_summary";
            $j = 0;
        } elsif($line =~ /Performing physical layer diagnostics.....$/) {
            $name = "physical_layer_diagnostics";
            $m = 0;
        } elsif($line =~ /Network Layer Diagnostics Summary:$/) {
            $name = "network_layer_diagnostics_summary";
            $j= 0;
        } elsif($line =~ /Performing network layer diagnostics.....$/) {
            $name = "network_layer_diagnostics";
            $m = 0;
        } elsif($line =~ /Transport Layer Diagnostics Summary:$/) {
            $name = "transport_layer_diagnostics_summary";
            $j = 0;
        } elsif($line =~ /Performing transport layer diagnostics.....$/) {
            $name = "transport_layer_diagnostics";
            $m = 0;
        } elsif($line =~ /(Interface\s+H\/W\s+Link\s+Configured\s+Speed\s+Duplex\s+AutoNeg)|(\s+Status\s+UP\s+Mismatch\s+Mismatch\s+Mismatch)/) {
            next;
        } elsif($line =~ /^([A-Za-z0-9\-]+)\s+(\w+)\s+([A-Za-z\-]+)\s+([A-Za-z\-]+)\s+([A-Za-z\-]+)\s+([A-Za-z\-]+)\s+([A-Za-z\-]+)$/) {
            $result_array[$i] -> {"$name"} -> [$j] -> {"interface"} = $1;
            $result_array[$i] -> {"$name"} -> [$j] -> {"hw-status"} = $2;
            $result_array[$i] -> {"$name"} -> [$j] -> {"link"} = $3;
            $result_array[$i] -> {"$name"} -> [$j] -> {"configured-up"} = $4;
            $result_array[$i] -> {"$name"} -> [$j] -> {"speed-mismatch"} = $5;
            $result_array[$i] -> {"$name"} -> [$j] -> {"duplex-mismatch"} = $6;
            $result_array[$i] -> {"$name"} -> [$j] -> {"autoneg-mismatch"} = $7;
            $j++;
        } elsif($line =~ /^(Protocol)\s+(Status)$/) {
            next;
        } elsif($line =~ /^(\w+)\s+(\w+)$/) {
            $result_array[$i] -> {"$name"} -> [$j] -> {"protocol"} = $1;
            $result_array[$i] -> {"$name"} -> [$j] -> {"status"} = $2;
            $j++;
        } elsif($line =~ /^Checking\s+interface\s+([0-9A-Za-z\.\s+]+)$/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "Checking interface ".$value;
        } elsif($line =~ /^The\s+interface\s+([0-9A-Za-z\s+]+)/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "The interface ".$value;
        } elsif($line =~ /^The\s+pseudo\s+interface\s+([0-9A-Za-z\s+]+)/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "The pseudo interface ".$value;
        } elsif($line =~ /^Error\s+(\d+)$/) {
            $value = $1;
            $result_array[$k] -> {"$name"} -> [$m++] = "Error ".$value;
        } elsif($line =~ /^Interface\s+([0-9A-Za-z\s+]+)/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "Interface ".$value;
        } elsif($line =~ /(IP is OK)|(TCP is OK)|(UDP is OK)|(IPv6 is OK)/) {
            $value = $&;
            $result_array[$k] -> {"$name"} -> [$m++] = $value;
        } elsif($line =~ /^Checking\s+for\s+([a-zA-Z0-9\.\s+]+)$/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "Checking for ".$value;
        } elsif($line =~ /^Mediatype:\s+([a-z\-0-9A-Z]+)/) {
            $result_array[$k] -> {"$name"} -> [$m++] = "media-type is ". $1;
        } elsif($line =~ /Gigabit Ethernet interface,\s+([0-9A-Za-z\s+\.]+)$/) {
            $value = $1;
            $value =~ s/\.//g;
            $result_array[$k] -> {"$name"} -> [$m++] = "Gigabit Ethernet interface,".$value;
        } elsif($line =~ /(Performing physical layer diagnostics.....OK$)|(Performing network layer diagnostics.....OK$)|(Performing transport layer diagnostics.....OK$)/) {
            $value = $&;
            $result_array[$k] -> {"diagnostics"} -> [$p++] = $value;
        } elsif($line =~ /^Atleast\s+([a-zA-Z\s0-9\.]+)$/) {
            $result_array[$i] = "Atleast ".$1;
            $i++;
        }
    }
	return \@result_array;
}
1;
