#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse the 7Mode netstat command
##
## @description
##    This is a library to parse the raw outputs of the 7Mode netstat command
##    sent with any of its options.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::netstat;
##
##    netstat(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::netstat;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $c             = $command . ':\s*';
    my $error;
    if ($output =~ /$c(option requires an argument.+)/) {
        $error = "MissingRequiredParam";
    } elsif ($output =~ /$c(Invalid option.+)|(illegal option).+/) {
        $error = "InvalidParam";
    } elsif ($output =~ /$c(.+(unknown or|No such).+)/) {
        $error = "InvalidParamValue";
    }
    $$error_message = $1;
    return $error;
} ## end sub _check_error

## @name ::netstat
##
## @summary Parses the raw output from the "netstat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from netstat with any of its options as described in the 
##    corresponding cdef file 'netstat.cdefs'
## @synopsis
##      netstat(%args);
## INPUTS TO METHOD CALL 'netstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-p' </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'extended-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'per-protocol-stats' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'socket-bytes' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-network-addresses' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'network-context' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-sockets' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'configured-interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'routing-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dropped-packets' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'memory-management-stats' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; netstat; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active TCP connections
##&nbsp;Local Address         Remote Address         Swind Send-Q  Rwind Recv-Q State       TOE?
##&nbsp;f3270-142-125.514     10.72.8.241.33905       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1021    10.72.8.241.41373       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.514     10.72.8.241.49789       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1022    10.72.8.241.36956       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.514     10.72.8.241.46828       9216      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1023    10.72.8.241.38782       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1020    10.72.8.241.52303       5792      0   8760      0 ESTABLISHED  - 
##&nbsp;f3270-142-125.514     10.72.8.241.35373       6144      1   8760      0 ESTABLISHED  - 
##&nbsp;localhost-20.951      localhost-10.38605     65216      0   8760      0 ESTABLISHED  - 
##&nbsp;192.168.2.85.6767     192.168.2.169.18389   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48900   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.32424   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.24275   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.15971   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.45938   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.7079    262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.31125   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48950   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.41239   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.33059   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.38764   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;localhost-20.16231    localhost-10.7951      66560      0   8760      0 ESTABLISHED  - 
##&nbsp;localhost-20.5939     *.*                        0      0      0      0 LISTEN       - 
##&nbsp;192.168.2.85.6767     *.*                        0      0      0      0 LISTEN      TOE
##&nbsp;192.168.1.116.6767    *.*                        0      0      0      0 LISTEN      TOE
##&nbsp;
##&nbsp;Active UDP sockets
##&nbsp;Local Address         Remote Address        Send-Q Recv-Q
##&nbsp;f3270-142-125.4444    *.*                        0      0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.33905',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.41373',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1021',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.49789',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.36956',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1022',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '9216',
##&nbsp;                           'remote_address' => '10.72.8.241.46828',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.38782',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1023',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '5792',
##&nbsp;                           'remote_address' => '10.72.8.241.52303',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1020',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '1',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.35373',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '65216',
##&nbsp;                           'remote_address' => 'localhost-10.38605',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'localhost-20.951',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.18389',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.48900',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.32424',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.24275',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.15971',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.45938',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.7079',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.31125',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.48950',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.41239',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.33059',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.38764',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '66560',
##&nbsp;                           'remote_address' => 'localhost-10.7951',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'localhost-20.16231',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => 'localhost-20.5939',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.1.116.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_tcp_connections'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => 'f3270-142-125.4444'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_udp_sockets'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -a; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active TCP connections (including servers)
##&nbsp;Local Address         Remote Address         Swind Send-Q  Rwind Recv-Q State       TOE?
##&nbsp;f3270-142-125.514     10.72.8.241.35373       8192      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1020    10.72.8.241.52303       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.514     10.72.8.241.33905       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1021    10.72.8.241.41373       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.514     10.72.8.241.49789       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1022    10.72.8.241.36956       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.514     10.72.8.241.46828       9216      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1023    10.72.8.241.38782       6144      0   8760      0 TIME_WAIT    - 
##&nbsp;f3270-142-125.1019    10.72.8.241.57136       5792      0   8760      0 ESTABLISHED  - 
##&nbsp;f3270-142-125.514     10.72.8.241.59402       6144      1   8760      0 ESTABLISHED  - 
##&nbsp;localhost-20.951      localhost-10.38605     65216      0   8760      0 ESTABLISHED  - 
##&nbsp;192.168.2.85.6767     192.168.2.169.18389   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48900   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.32424   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.24275   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.15971   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.45938   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.7079    262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.31125   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48950   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.41239   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.33059   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.38764   262144      0 262144      0 ESTABLISHED TOE
##&nbsp;localhost-20.16231    localhost-10.7951      66560      0   8760      0 ESTABLISHED  - 
##&nbsp;*.4047                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.4045                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.4046                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.2049                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.111                 *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.443                 *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.22                  *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.10569               *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.10568               *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.10567               *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.8514                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.514                 *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.8023                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.23                  *.*                        0      0      0      0 LISTEN       - 
##&nbsp;*.951                 *.*                        0      0      0      0 LISTEN       - 
##&nbsp;localhost-20.5939     *.*                        0      0      0      0 LISTEN       - 
##&nbsp;192.168.2.85.6767     *.*                        0      0      0      0 LISTEN      TOE
##&nbsp;192.168.1.116.6767    *.*                        0      0      0      0 LISTEN      TOE
##&nbsp;*.2049                *.*                        0      0      0      0 LISTEN       - 
##&nbsp;
##&nbsp;Active UDP sockets (including servers)
##&nbsp;Local Address         Remote Address        Send-Q Recv-Q
##&nbsp;*.4049                *.*                        0      0
##&nbsp;*.4047                *.*                        0      0
##&nbsp;*.4045                *.*                        0      0
##&nbsp;*.4046                *.*                        0      0
##&nbsp;*.2049                *.*                        0      0
##&nbsp;*.111                 *.*                        0      0
##&nbsp;*.10570               *.*                        0      0
##&nbsp;*.69                  *.*                        0      0
##&nbsp;f3270-142-125.4444    *.*                        0      0
##&nbsp;*.161                 *.*                        0      0
##&nbsp;*.520                 *.*                        0      0
##&nbsp;*.*                   *.*                        0      0
##&nbsp;*.28442               *.*                        0      0
##&nbsp;*.28442               *.*                        0      0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '8192',
##&nbsp;                           'remote_address' => '10.72.8.241.35373',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.52303',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1020',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.33905',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.41373',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1021',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.49789',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.36956',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1022',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '9216',
##&nbsp;                           'remote_address' => '10.72.8.241.46828',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.38782',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1023',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '5792',
##&nbsp;                           'remote_address' => '10.72.8.241.57136',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1019',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '1',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.59402',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '65216',
##&nbsp;                           'remote_address' => 'localhost-10.38605',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'localhost-20.951',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.18389',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.48900',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.32424',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.24275',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.15971',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.45938',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.7079',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.31125',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.48950',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.41239',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.33059',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'remote_address' => '192.168.2.169.38764',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '66560',
##&nbsp;                           'remote_address' => 'localhost-10.7951',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'localhost-20.16231',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'ESTABLISHED'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.4047',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.4045',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.4046',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.2049',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.111',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.443',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.22',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.10569',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.10568',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.10567',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.8514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.514',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.8023',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.23',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.951',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => 'localhost-20.5939',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.1.116.6767',
##&nbsp;                           'toe' => 'TOE',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '*.2049',
##&nbsp;                           'toe' => '-',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_tcp_connections_including_servers'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.4049'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.4047'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.4045'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.4046'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.2049'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.111'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.10570'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.69'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => 'f3270-142-125.4444'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.161'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.520'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.*'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.28442'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => '*.28442'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_udp_sockets_including_servers'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -i; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Name   Mtu   Network             Address             Ipkts  Ierrs  Opkts  Oerrs  Collis  Queue
##&nbsp;e0a*    1500 none                none         67253      0   1453      0       0    431
##&nbsp;e0b*    1500 none                none          9357      0   1453      0       0    431
##&nbsp;e1a*    1500 none                none             0      0      0      0       0  131071
##&nbsp;e1b*    1500 none                none             0      0      0      0       0  131071
##&nbsp;e2a*    1500 none                none             0      0      0      0       0  131071
##&nbsp;e2b*    1500 none                none             0      0      0      0       0  131071
##&nbsp;c0a     9000 192.168.1           192.168.1.11 625711      0      2      0       0  131071
##&nbsp;c0b     9000 192.168.2           192.168.2.85 174178      0  174272      0       0    100
##&nbsp;e0M     1500 10.238.142/23       f3270-142-12 135640      0   2878      0       0     90
##&nbsp;e0P*    1500 none                none             0      0      0      0       0  131071
##&nbsp;lo      9188 127                 localhost        0      0      0      0       0      0
##&nbsp;losk    9188 127                 localhost-20     0      0  49754      0       0      0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e0a*',
##&nbsp;            'opkts' => '1453',
##&nbsp;            'ipkts' => '67253',
##&nbsp;            'queue' => '431',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e0b*',
##&nbsp;            'opkts' => '1453',
##&nbsp;            'ipkts' => '9357',
##&nbsp;            'queue' => '431',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e1a*',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e1b*',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e2a*',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e2b*',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '192.168.1',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'c0a',
##&nbsp;            'opkts' => '2',
##&nbsp;            'ipkts' => '625711',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => '192.168.1.11',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '9000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '192.168.2',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'c0b',
##&nbsp;            'opkts' => '174272',
##&nbsp;            'ipkts' => '174178',
##&nbsp;            'queue' => '100',
##&nbsp;            'address' => '192.168.2.85',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '9000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '10.238.142/23',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e0M',
##&nbsp;            'opkts' => '2878',
##&nbsp;            'ipkts' => '135640',
##&nbsp;            'queue' => '90',
##&nbsp;            'address' => 'f3270-142-12',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => 'none',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'e0P*',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '131071',
##&nbsp;            'address' => 'none',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '1500'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '127',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'lo',
##&nbsp;            'opkts' => '0',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '0',
##&nbsp;            'address' => 'localhost',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '9188'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'oerrs' => '0',
##&nbsp;            'network' => '127',
##&nbsp;            'ierrs' => '0',
##&nbsp;            'name' => 'losk',
##&nbsp;            'opkts' => '49754',
##&nbsp;            'ipkts' => '0',
##&nbsp;            'queue' => '0',
##&nbsp;            'address' => 'localhost-20',
##&nbsp;            'collis' => '0',
##&nbsp;            'mtu' => '9188'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -m; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;80256 mbufs out of 98866 in use:
##&nbsp; 76875 out of 78539 large buffers in use
##&nbsp; 3381 out of 20277 small buffers in use
##&nbsp; 0 out of 50 non-data mbufs in use
##&nbsp;258726 Kbytes allocated to network (95% in use)
##&nbsp;0 requests for memory denied
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_small_buffers' => '20277',
##&nbsp;            'total_non-data_mbufs' => '50',
##&nbsp;            'kbytes_allocated' => '258726',
##&nbsp;            'total_mbufs' => '98866',
##&nbsp;            'memory_requests_denied' => '0',
##&nbsp;            'non-data_mbufs_in_use' => '0',
##&nbsp;            'percentage_used' => '95',
##&nbsp;            'mbufs_in_use' => '80256',
##&nbsp;            'large_buffers_in_use' => '76875',
##&nbsp;            'total_large_buffers' => '78539',
##&nbsp;            'small_buffers_in_use' => '3381'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -s; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;tcp:
##&nbsp;	50495 packets sent
##&nbsp;		46474 data packets (44316145 bytes)
##&nbsp;		0 data packets (0 bytes) retransmitted
##&nbsp;		2587 ack-only packets (499 delayed)
##&nbsp;		0 URG only packets
##&nbsp;		0 window probe packets
##&nbsp;		194 window update packets
##&nbsp;		1240 control packets
##&nbsp;	51698 packets received
##&nbsp;		26267 acks (for 44318018 bytes)
##&nbsp;		668 duplicate acks
##&nbsp;		0 acks for unsent data
##&nbsp;		23104 packets (3732229 bytes) received in-sequence
##&nbsp;		0 completely duplicate packets (0 bytes)
##&nbsp;		0 old duplicate packets
##&nbsp;		0 packets with some duplicate data (0 bytes duped)
##&nbsp;		0 out-of-order packets (0 bytes)
##&nbsp;		0 packets (0 bytes) of data after window
##&nbsp;		0 window probes
##&nbsp;		2374 window update packets
##&nbsp;		0 packets received after close
##&nbsp;		0 discarded for bad checksums
##&nbsp;		0 discarded for bad header offset fields
##&nbsp;		0 discarded because packet too short
##&nbsp;		0 discarded because reassembly queue overflow
##&nbsp;	0 packets in reass queue
##&nbsp;	274 connection requests
##&nbsp;	783 connection accepts
##&nbsp;	0 bad connection attempts
##&nbsp;	0 listen queue overflows
##&nbsp;	997 connections established (including accepts)
##&nbsp;	1368 connections closed (including 1 drop)
##&nbsp;	60 embryonic connections dropped
##&nbsp;	26483 segments updated rtt (of 16265 attempts)
##&nbsp;	15 retransmit timeouts
##&nbsp;		15 abrupt retransmit timeouts 
##&nbsp;		0 subsequent retransmit timeouts 
##&nbsp;		0 connections dropped by rexmit timeout
##&nbsp;	0 fast retransmits
##&nbsp;	0 dupacks that did not ultimately trigger fast retransmit 
##&nbsp;	0 persist timeouts
##&nbsp;	0 connections timed out in persist
##&nbsp;	195 keepalive timeouts
##&nbsp;		195 keepalive probes sent
##&nbsp;		0 connections dropped by keepalive
##&nbsp;	30 correct ACK header predictions
##&nbsp;	12122 correct data packet header predictions
##&nbsp;	0 segments dropped at untrusted interface
##&nbsp;	0 connection requests closed at filter
##&nbsp;	0 SACK options sent
##&nbsp;	997 SACK options received
##&nbsp;	0 RST segments not sent (stream.c defense)
##&nbsp;	0 resends due to MTU discovery
##&nbsp;	0 segments sent using TSO
##&nbsp;	0 bytes sent using TSO
##&nbsp;	0 TSO segments truncated
##&nbsp;	0 TSO wrapped sequence space segments
##&nbsp;	0 segments sent using TSO6
##&nbsp;	0 bytes sent using TSO6
##&nbsp;	0 TSO6 segments truncated
##&nbsp;	0 TSO6 wrapped sequence space segments
##&nbsp;	0 segments received using LRO
##&nbsp;	0 bytes received using LRO
##&nbsp;	1 socket flushes due to timer
##&nbsp;	50611 socket flushes due to TCP flags
##&nbsp;	198 mbufs batched
##&nbsp;	0 tcp send window based extreme flowcontrol 
##&nbsp;		0 zero window increases, 0 send buffer size increases
##&nbsp;	0 connection resets in extreme flowcontrol (of 0 attempts)
##&nbsp;	0 sends, 0 receives max reset threshold reached for extreme flowcontrol
##&nbsp;	8467 times one or more connections was idle 
##&nbsp;	0 packets -- 0 bytes (window probes) received above buffer high-water
##&nbsp;	0 packets -- 0 bytes (data) received above buffer high-water
##&nbsp;	0 sockets with receive buffer overflow
##&nbsp;	0 window advertisements above receive high-water
##&nbsp;	13230 window advertisements greater than available window size
##&nbsp;	0 window advertisements greater than available window size and above receive buffer high-water
##&nbsp;	0 negative receive window advertisements
##&nbsp;	20 packets transferred to FreeBSD successfully
##&nbsp;	0 packets failed to transfer to FreeBSD
##&nbsp;	0 times socket receive size set less than current size of contents
##&nbsp;	0 times tcp socket input overflowed buffer capacity
##&nbsp;	0 times tcp socket input made with negative space available
##&nbsp;	0 times tcp socket input made with negative buffer character count
##&nbsp;	0 times tcp socket input made with negative reserve count
##&nbsp;	0 times tcp socket output made with negative send window
##&nbsp;	0 times tcp socket output made with negative congestion window
##&nbsp;	0 times tcp socket output made with  negative send-length (value not -1)
##&nbsp;	0 times tcp socket output made with send-length > 2.1GB
##&nbsp;udp:
##&nbsp;	96111 datagrams received
##&nbsp;	0 with incomplete header
##&nbsp;	0 with bad data length field
##&nbsp;	0 with bad checksum
##&nbsp;	0 dropped due to no socket
##&nbsp;	4617 filtered packets dropped
##&nbsp;	3406 broadcast/multicast datagrams dropped due to no socket
##&nbsp;	0 dropped due to full socket buffers
##&nbsp;	88088 delivered
##&nbsp;	87058 datagrams output
##&nbsp;ip:
##&nbsp;	151779 total packets received
##&nbsp;	0 bad header checksums
##&nbsp;	0 with size smaller than minimum
##&nbsp;	0 with size larger than maximum
##&nbsp;	0 with data size < data length
##&nbsp;	0 with header length < data size
##&nbsp;	0 with data length < header length
##&nbsp;	0 with bad options
##&nbsp;	0 with incorrect version number
##&nbsp;	0 packets with spoofed source address
##&nbsp;	0 packets arrived on wrong port
##&nbsp;	0 fragments received
##&nbsp;	the last 2 src addr that send fragments:
##&nbsp;		0.0.0.0 0.0.0.0
##&nbsp;	0 fragments dropped (dup or out of space)
##&nbsp;	0 malformed fragments dropped
##&nbsp;	0 overlapping fragments discarded
##&nbsp;	0 fragments dropped after timeout
##&nbsp;	the last 2 src addr that have fragment time out:
##&nbsp;		0.0.0.0 0.0.0.0
##&nbsp;	0 packets dropped, too many fragments
##&nbsp;	the last 2 src addr that sent too many fragments:
##&nbsp;		0.0.0.0 0.0.0.0
##&nbsp;	0 packets dropped, reassembly queue overflow
##&nbsp;	0 packets reassembled ok
##&nbsp;	147824 packets for this host
##&nbsp;	0 packets for unknown/unsupported protocol
##&nbsp;	0 packets forwarded
##&nbsp;	3945 packets not forwardable
##&nbsp;	0 redirects sent
##&nbsp;	137776 packets sent from this host
##&nbsp;	0 packets sent with fabricated ip header
##&nbsp;	0 output packets dropped due to no bufs, etc.
##&nbsp;	0 output packets discarded due to no route
##&nbsp;	0 output datagrams fragmented
##&nbsp;	0 fragments created
##&nbsp;	0 datagrams that can't be fragmented
##&nbsp;	0 filtered packets dropped by ipfw
##&nbsp;	0 filtered packets dropped
##&nbsp;	0 packets dropped due to vfiler mismatch
##&nbsp;	0 packets dropped during the cutover state of online vfiler unit migration
##&nbsp;	0 packets forwarded by source interface
##&nbsp;	Classifier hash queue counts:
##&nbsp;		queue 0:	32768
##&nbsp;		queue 1:	0
##&nbsp;		queue 2:	0
##&nbsp;		queue 3:	0
##&nbsp;icmp:
##&nbsp;	0 calls to icmp_error
##&nbsp;	0 errors not generated because old message was icmp
##&nbsp;	0 messages with bad code fields
##&nbsp;	0 messages < minimum length
##&nbsp;	0 bad checksums
##&nbsp;	0 messages with bad length
##&nbsp;	Input histogram:
##&nbsp;		echo reply: 7
##&nbsp;	0 pings dropped due to throttling
##&nbsp;	0 ping replies dropped due to throttling
##&nbsp;	0 redirects ignored
##&nbsp;	0 message responses generated
##&nbsp;igmp:
##&nbsp;	0 total IGMP messages received
##&nbsp;		0 received with too few bytes
##&nbsp;		0 received with bad checksums
##&nbsp;		0 membership queries received
##&nbsp;		0 invalid queries received
##&nbsp;		0 membership reports received
##&nbsp;		0 invalid reports received
##&nbsp;		0 received reports for our group
##&nbsp;	6 membership reports sent
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'times_socket_receive_size_set_less_than_current_size_of_contents' => '0',
##&nbsp;                           'window_update_packets' => '2374',
##&nbsp;                           'packets_--_0_bytes_data_received_above_buffer_high-water' => '0',
##&nbsp;                           'socket_flushes_due_to_timer' => '1',
##&nbsp;                           'times_tcp_socket_input_made_with_negative_reserve_count' => '0',
##&nbsp;                           'times_tcp_socket_input_made_with_negative_buffer_character_count' => '0',
##&nbsp;                           'resends_due_to_mtu_discovery' => '0',
##&nbsp;                           'sack_options_received' => '997',
##&nbsp;                           'packets_of_data_after_window' => '0',
##&nbsp;                           'times_tcp_socket_output_made_with_send-length_>_2.1gb' => '0',
##&nbsp;                           'urg_only_packets' => '0',
##&nbsp;                           'packets_failed_to_transfer_to_freebsd' => '0',
##&nbsp;                           'bytes_sent_using_tso6' => '0',
##&nbsp;                           'bytes_of_data_after_window' => '0',
##&nbsp;                           'bytes_sent_using_tso' => '0',
##&nbsp;                           'bytes_received_in-sequence' => '3732229',
##&nbsp;                           'correct_ack_header_predictions' => '30',
##&nbsp;                           'connections_timed_out_in_persist' => '0',
##&nbsp;                           'dupacks_that_did_not_ultimately_trigger_fast_retransmit' => '0',
##&nbsp;                           'ack-only_packets' => '2587',
##&nbsp;                           'packets_with_some_duplicate_data' => '0',
##&nbsp;                           'retransmit_timeouts' => '15',
##&nbsp;                           'fast_retransmits' => '0',
##&nbsp;                           'bytes_duped' => '0',
##&nbsp;                           'window_advertisements_greater_than_available_window_size_and_above_receive_buffer_high-water' => '0',
##&nbsp;                           'segments_received_using_lro' => '0',
##&nbsp;                           'abrupt_retransmit_timeouts' => '15',
##&nbsp;                           'segments_sent_using_tso' => '0',
##&nbsp;                           'tso6_segments_truncated' => '0',
##&nbsp;                           'listen_queue_overflows' => '0',
##&nbsp;                           'negative_receive_window_advertisements' => '0',
##&nbsp;                           'keepalive_timeouts' => '195',
##&nbsp;                           'data_bytes' => '44316145',
##&nbsp;                           'packets_received' => '51698',
##&nbsp;                           'completely_duplicate_packets_bytes' => '0',
##&nbsp;                           'zero_window_increases,_0_send_buffer_size_increases' => '0',
##&nbsp;                           'times_tcp_socket_input_overflowed_buffer_capacity' => '0',
##&nbsp;                           'tcp_send_window_based_extreme_flowcontrol' => '0',
##&nbsp;                           'ack-only_packets_delayed' => '499',
##&nbsp;                           'segments_updated_rtt' => '26483',
##&nbsp;                           'persist_timeouts' => '0',
##&nbsp;                           'times_tcp_socket_output_made_with_negative_send-length_value_not_-1' => '0',
##&nbsp;                           'data_packets_retransmitted' => '0',
##&nbsp;                           'segments_updated_rtt_attempts' => '16265',
##&nbsp;                           'old_duplicate_packets' => '0',
##&nbsp;                           'window_advertisements_above_receive_high-water' => '0',
##&nbsp;                           'rst_segments_not_sent_stream.c_defense' => '0',
##&nbsp;                           'connections_established_including_accepts' => '997',
##&nbsp;                           'packets_transferred_to_freebsd_successfully' => '20',
##&nbsp;                           'connections_closed_including_1_drop' => '1368',
##&nbsp;                           'sockets_with_receive_buffer_overflow' => '0',
##&nbsp;                           'embryonic_connections_dropped' => '60',
##&nbsp;                           'duplicate_acks' => '668',
##&nbsp;                           'packets_--_0_bytes_window_probes_received_above_buffer_high-water' => '0',
##&nbsp;                           'discarded_for_bad_checksums' => '0',
##&nbsp;                           'discarded_for_bad_header_offset_fields' => '0',
##&nbsp;                           'connections_dropped_by_keepalive' => '0',
##&nbsp;                           'tso_wrapped_sequence_space_segments' => '0',
##&nbsp;                           'tso_segments_truncated' => '0',
##&nbsp;                           'connection_accepts' => '783',
##&nbsp;                           'keepalive_probes_sent' => '195',
##&nbsp;                           'connections_dropped_by_rexmit_timeout' => '0',
##&nbsp;                           'out-of-order_bytes' => '0',
##&nbsp;                           'times_tcp_socket_output_made_with_negative_send_window' => '0',
##&nbsp;                           'bytes_received_using_lro' => '0',
##&nbsp;                           'segments_sent_using_tso6' => '0',
##&nbsp;                           'packets_sent' => '50495',
##&nbsp;                           'window_probe_packets' => '0',
##&nbsp;                           'window_probes' => '0',
##&nbsp;                           'times_one_or_more_connections_was_idle' => '8467',
##&nbsp;                           'discarded_because_packet_too_short' => '0',
##&nbsp;                           'tso6_wrapped_sequence_space_segments' => '0',
##&nbsp;                           'bytes_retransmitted' => '0',
##&nbsp;                           'times_tcp_socket_output_made_with_negative_congestion_window' => '0',
##&nbsp;                           'window_advertisements_greater_than_available_window_size' => '13230',
##&nbsp;                           'subsequent_retransmit_timeouts' => '0',
##&nbsp;                           'acks' => '26267',
##&nbsp;                           'sack_options_sent' => '0',
##&nbsp;                           'packets_received_after_close' => '0',
##&nbsp;                           'control_packets' => '1240',
##&nbsp;                           'mbufs_batched' => '198',
##&nbsp;                           'sends,_0_receives_max_reset_threshold_reached_for_extreme_flowcontrol' => '0',
##&nbsp;                           'connection_requests' => '274',
##&nbsp;                           'acks_bytes' => '44318018',
##&nbsp;                           'socket_flushes_due_to_tcp_flags' => '50611',
##&nbsp;                           'data_packets' => '46474',
##&nbsp;                           'discarded_because_reassembly_queue_overflow' => '0',
##&nbsp;                           'times_tcp_socket_input_made_with_negative_space_available' => '0',
##&nbsp;                           'acks_for_unsent_data' => '0',
##&nbsp;                           'packets_in_reass_queue' => '0',
##&nbsp;                           'connection_requests_closed_at_filter' => '0',
##&nbsp;                           'bad_connection_attempts' => '0',
##&nbsp;                           'out-of-order_packets' => '0',
##&nbsp;                           'segments_dropped_at_untrusted_interface' => '0',
##&nbsp;                           'completely_duplicate_packets' => '0',
##&nbsp;                           'correct_data_packet_header_predictions' => '12122',
##&nbsp;                           'connection_resets_in_extreme_flowcontrol_of_0_attempts' => '0',
##&nbsp;                           'packets_received_in-sequence' => '23104'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'tcp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'filtered_packets_dropped' => '4617',
##&nbsp;                           'with_bad_checksum' => '0',
##&nbsp;                           'with_incomplete_header' => '0',
##&nbsp;                           'dropped_due_to_full_socket_buffers' => '0',
##&nbsp;                           'datagrams_output' => '87058',
##&nbsp;                           'broadcast/multicast_datagrams_dropped_due_to_no_socket' => '3406',
##&nbsp;                           'with_bad_data_length_field' => '0',
##&nbsp;                           'dropped_due_to_no_socket' => '0',
##&nbsp;                           'delivered' => '88088',
##&nbsp;                           'datagrams_received' => '96111'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'udp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'with_size_larger_than_maximum' => '0',
##&nbsp;                           'packets_dropped,_too_many_fragments' => '0',
##&nbsp;                           'packets_for_this_host' => '147824',
##&nbsp;                           'with_data_length_<_header_length' => '0',
##&nbsp;                           'packets_forwarded_by_source_interface' => '0',
##&nbsp;                           'overlapping_fragments_discarded' => '0',
##&nbsp;                           'fragments_created' => '0',
##&nbsp;                           'output_datagrams_fragmented' => '0',
##&nbsp;                           'the_last_2_src_addr_that_sent_too_many_fragments' => '0.0.0.0 0.0.0.0',
##&nbsp;                           'packets_dropped_due_to_vfiler_mismatch' => '0',
##&nbsp;                           'packets_for_unknown/unsupported_protocol' => '0',
##&nbsp;                           'filtered_packets_dropped' => '0',
##&nbsp;                           'fragments_received' => '0',
##&nbsp;                           'packets_not_forwardable' => '3945',
##&nbsp;                           'packets_dropped,_reassembly_queue_overflow' => '0',
##&nbsp;                           'output_packets_dropped_due_to_no_bufs,_etc.' => '0',
##&nbsp;                           'packets_sent_with_fabricated_ip_header' => '0',
##&nbsp;                           'packets_dropped_during_the_cutover_state_of_online_vfiler_unit_migration' => '0',
##&nbsp;                           'filtered_packets_dropped_by_ipfw' => '0',
##&nbsp;                           'with_data_size_<_data_length' => '0',
##&nbsp;                           'malformed_fragments_dropped' => '0',
##&nbsp;                           'packets_sent_from_this_host' => '137776',
##&nbsp;                           'with_size_smaller_than_minimum' => '0',
##&nbsp;                           'output_packets_discarded_due_to_no_route' => '0',
##&nbsp;                           'packets_with_spoofed_source_address' => '0',
##&nbsp;                           'packets_arrived_on_wrong_port' => '0',
##&nbsp;                           'classifier_hash_queue_counts' => [
##&nbsp;                                                               {
##&nbsp;                                                                 'queue_0' => '32768',
##&nbsp;                                                                 'queue_3' => '0',
##&nbsp;                                                                 'queue_2' => '0',
##&nbsp;                                                                 'queue_1' => '0'
##&nbsp;                                                               }
##&nbsp;                                                             ],
##&nbsp;                           'redirects_sent' => '0',
##&nbsp;                           'the_last_2_src_addr_that_have_fragment_time_out' => '0.0.0.0 0.0.0.0',
##&nbsp;                           'total_packets_received' => '151779',
##&nbsp;                           'the_last_2_src_addr_that_send_fragments' => '0.0.0.0 0.0.0.0',
##&nbsp;                           'fragments_dropped_after_timeout' => '0',
##&nbsp;                           'datagrams_that_can\'t_be_fragmented' => '0',
##&nbsp;                           'packets_forwarded' => '0',
##&nbsp;                           'with_bad_options' => '0',
##&nbsp;                           'packets_reassembled_ok' => '0',
##&nbsp;                           'fragments_dropped_dup_or_out_of_space' => '0',
##&nbsp;                           'with_incorrect_version_number' => '0',
##&nbsp;                           'with_header_length_<_data_size' => '0',
##&nbsp;                           'bad_header_checksums' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'ip'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'calls_to_icmp_error' => '0',
##&nbsp;                           'messages_with_bad_length' => '0',
##&nbsp;                           'message_responses_generated' => '0',
##&nbsp;                           'ping_replies_dropped_due_to_throttling' => '0',
##&nbsp;                           'errors_not_generated_because_old_message_was_icmp' => '0',
##&nbsp;                           'input_histogram' => [
##&nbsp;                                                  {
##&nbsp;                                                    'echo_reply' => '7'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                           'pings_dropped_due_to_throttling' => '0',
##&nbsp;                           'messages_<_minimum_length' => '0',
##&nbsp;                           'redirects_ignored' => '0',
##&nbsp;                           'messages_with_bad_code_fields' => '0',
##&nbsp;                           'bad_checksums' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'icmp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'invalid_queries_received' => '0',
##&nbsp;                           'membership_queries_received' => '0',
##&nbsp;                           'received_with_bad_checksums' => '0',
##&nbsp;                           'received_with_too_few_bytes' => '0',
##&nbsp;                           'invalid_reports_received' => '0',
##&nbsp;                           'received_reports_for_our_group' => '0',
##&nbsp;                           'membership_reports_received' => '0',
##&nbsp;                           'membership_reports_sent' => '6',
##&nbsp;                           'total_igmp_messages_received' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'igmp'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -rs; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;routing:
##&nbsp;	0 bad routing redirects
##&nbsp;	0 dynamically created routes
##&nbsp;	0 new gateways due to redirects
##&nbsp;	267 destinations found unreachable
##&nbsp;	0 uses of a wildcard route
##&nbsp;	0 times searched a route in all routing tables
##&nbsp;	0 route update failures
##&nbsp;	0 freebsd route add failures
##&nbsp;	0 freebsd route delete failures
##&nbsp;	0 freebsd route update failures
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'freebsd_route_update_failures' => '0',
##&nbsp;            'freebsd_route_add_failures' => '0',
##&nbsp;            'new_gateways_due_to_redirects' => '0',
##&nbsp;            'destinations_found_unreachable' => '267',
##&nbsp;            'route_update_failures' => '0',
##&nbsp;            'dynamically_created_routes' => '0',
##&nbsp;            'uses_of_a_wildcard_route' => '0',
##&nbsp;            'times_searched_a_route_in_all_routing_tables' => '0',
##&nbsp;            'freebsd_route_delete_failures' => '0',
##&nbsp;            'bad_routing_redirects' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; netstat -x; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Active TCP connections
##&nbsp;Local Address         Remote Address         Swind Send-Q  Rwind Recv-Q State       FastPath On? TOE?
##&nbsp;f3270-142-125.514     10.72.8.241.44237       7168      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1015    10.72.8.241.35720       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.50939      24576      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1016    10.72.8.241.51799       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.58512       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1017    10.72.8.241.60029       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.45254       8192      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1018    10.72.8.241.60670       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.59402      17920      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1019    10.72.8.241.57136       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.35373       8192      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1020    10.72.8.241.52303       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.33905       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1021    10.72.8.241.41373       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.49789       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1022    10.72.8.241.36956       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.514     10.72.8.241.46828       9216      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1023    10.72.8.241.38782       6144      0   8760      0 TIME_WAIT                 - 
##&nbsp;f3270-142-125.1014    10.72.8.241.57840       5792      0   8760      0 ESTABLISHED          Yes  - 
##&nbsp;f3270-142-125.514     10.72.8.241.46956       6144      1   8760      0 ESTABLISHED          Yes  - 
##&nbsp;localhost-20.951      localhost-10.38605     65216      0   8760      0 ESTABLISHED          Yes  - 
##&nbsp;192.168.2.85.6767     192.168.2.169.18389   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48900   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.32424   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.24275   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.15971   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.45938   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.7079    262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.31125   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.48950   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.41239   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.33059   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;192.168.2.85.6767     192.168.2.169.38764   262144      0 262144      0 ESTABLISHED          Yes TOE
##&nbsp;localhost-20.16231    localhost-10.7951      66560      0   8760      0 ESTABLISHED          Yes  - 
##&nbsp;localhost-20.5939     *.*                        0      0      0      0 LISTEN                    - 
##&nbsp;192.168.2.85.6767     *.*                        0      0      0      0 LISTEN                   TOE
##&nbsp;192.168.1.116.6767    *.*                        0      0      0      0 LISTEN                   TOE
##&nbsp;
##&nbsp;Active UDP sockets
##&nbsp;Local Address         Remote Address        Send-Q Recv-Q
##&nbsp;f3270-142-125.4444    *.*                        0      0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '7168',
##&nbsp;                           'remote_address' => '10.72.8.241.44237',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.35720',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1015',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '24576',
##&nbsp;                           'remote_address' => '10.72.8.241.50939',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.51799',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1016',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.58512',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.60029',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1017',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '8192',
##&nbsp;                           'remote_address' => '10.72.8.241.45254',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.60670',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1018',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '17920',
##&nbsp;                           'remote_address' => '10.72.8.241.59402',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.57136',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1019',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '8192',
##&nbsp;                           'remote_address' => '10.72.8.241.35373',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.52303',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1020',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.33905',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.41373',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1021',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.49789',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.36956',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1022',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '9216',
##&nbsp;                           'remote_address' => '10.72.8.241.46828',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.514',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'remote_address' => '10.72.8.241.38782',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'local_address' => 'f3270-142-125.1023',
##&nbsp;                           'state' => 'TIME_WAIT'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => '-',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '10.72.8.241.57840',
##&nbsp;                           'swind' => '5792',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => 'f3270-142-125.1014'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => '-',
##&nbsp;                           'send-q' => '1',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '10.72.8.241.46956',
##&nbsp;                           'swind' => '6144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => 'f3270-142-125.514'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => '-',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => 'localhost-10.38605',
##&nbsp;                           'swind' => '65216',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => 'localhost-20.951'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.18389',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.48900',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.32424',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.24275',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.15971',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.45938',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.7079',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.31125',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.48950',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.41239',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.33059',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => 'TOE',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '262144',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => '192.168.2.169.38764',
##&nbsp;                           'swind' => '262144',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => '192.168.2.85.6767'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           '' => '-',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'rwind' => '8760',
##&nbsp;                           'state' => 'ESTABLISHED',
##&nbsp;                           'remote_address' => 'localhost-10.7951',
##&nbsp;                           'swind' => '66560',
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'fastpath_on_toe?' => 'Yes',
##&nbsp;                           'local_address' => 'localhost-20.16231'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'fastpath_on_toe?' => '-',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => 'localhost-20.5939',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'fastpath_on_toe?' => 'TOE',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.2.85.6767',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'swind' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'fastpath_on_toe?' => 'TOE',
##&nbsp;                           'rwind' => '0',
##&nbsp;                           'local_address' => '192.168.1.116.6767',
##&nbsp;                           'state' => 'LISTEN'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_tcp_connections'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'recv-q' => '0',
##&nbsp;                           'send-q' => '0',
##&nbsp;                           'remote_address' => '*.*',
##&nbsp;                           'local_address' => 'f3270-142-125.4444'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'name' => 'active_udp_sockets'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub netstat {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(raw_output    => $output,
                             command       => "netstat",
                             error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my $first_line =
        '(Active|Name\s+Mtu|.+mbufs out of|routing|Routing tables|(igmp|icmp|ip|udp|tcp|ip6|icmp6):)';
    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          can_throw       => 1,
                          response_object => $resp);

    if ($lines[0] =~ /Active\s+TCP/) {
        my $sections = _get_sections(lines_ref   => \@lines,
                                     strings_ref => ['Active TCP', 'Active UDP']
                                    );
        foreach (@$sections) {
            push @result_array,
              { name  => to_key_name(item => shift @{$_}),
                stats => _table_parser($_)};
        }
    } elsif ($lines[0] =~ /Active\s+Internet/i) {
	@result_array = @{_parser(\@lines)};
    } elsif ($lines[0] =~ /Name\s+Mtu/) {
        @result_array = @{_table_parser(\@lines)};
    } elsif ($lines[0] =~ /.+mbufs out of/) {
        @result_array = @{_m_parser(\@lines)};
    } elsif ($lines[0] =~ /Routing tables/) {
        my $sections =
          _get_sections(lines_ref => \@lines, strings_ref => ['Internet']);
        foreach ($$sections[1]) {
            push @result_array,
              { name  => to_key_name(item => shift @{$_}),
                stats => _table_parser($_)};
        }
    } elsif ($lines[0] =~ /routing/) {
        shift @lines;
        @result_array = @{_rs_parser(\@lines)};
    } elsif ($lines[0] =~ /(igmp|icmp|ip|udp|tcp|ip6|icmp6):/) {
        my $sections = _get_sections(
             lines_ref   => \@lines,
             strings_ref => ['tcp', 'udp', 'ip', 'icmp', 'igmp', 'ip6', 'icmp6']
        );
        foreach (@$sections) {
            push @result_array,
              { name  => to_key_name(item => shift @{$_}),
                stats => _s_and_p_parser($_)}
              if ($_->[0]);
        }
    } ## end elsif (@lines[0] =~ /(igmp|icmp|ip|udp|tcp|ip6|icmp6):/)
    return \@result_array;
} ## end sub netstat

sub _m_parser {
    my @lines = @{$_[0]};
    my (%hash, $string, $in_use, $total);
    foreach (@lines) {
        if ($_ =~ /(\d+) mbufs out of (\d+) in use:/) {
            $hash{mbufs_in_use} = $1;
            $hash{total_mbufs}  = $2;
        } elsif ($_ =~ /(\d+) out of (\d+)\s+(.+)\s+in use/) {
            ($in_use, $total) = ($1, $2);
            $string = to_key_name(item => $3);
            $hash{$string . '_in_use'} = $in_use;
            $hash{'total_' . $string} = $total;
        } elsif ($_ =~ /(\d+) Kbytes allocated to network \((\d+)\% in use\)/) {
            $hash{kbytes_allocated} = $1;
            $hash{percentage_used}  = $2;
        } elsif ($_ =~ /(\d+) requests for memory denied/) {
            $hash{'memory_requests_denied'} = $1;
        }
    } ## end foreach (@lines)
    return [\%hash];
} ## end sub _m_parser

sub _s_and_p_parser {
    my @lines = @{$_[0]};
    my (%hash, $name, $value, $temp, $flag_array);
    foreach (@lines) {
        if ($name) {
            if ($flag_array == 1) {
                if ($_ =~ /^(.*):\s+(.*)$/) {
                    $hash{$name}->[0]->{to_key_name(item => $1)} = $2;
                    next;
                } else {
                    $flag_array = 0;
                }
            } else {
                $_ =~ /(.+)/;
                $hash{$name} = $1;
                $name = '';
                next;
            }
        } ## end if ($name)
        if ($_ =~ /^(\d+) (.+)/) {
            ($value, $temp) = ($1, $2);
            $name = to_key_name(item => $temp);
            if ($name =~ /data_packets_(\d+)_bytes$/) {
                $hash{data_bytes} = $1;
                $name = 'data_packets';

            } elsif ($name =~ /data_packets_(\d+)_bytes_retransmitted/) {
                $hash{bytes_retransmitted} = $1;
                $name = 'data_packets_retransmitted';

            } elsif ($name =~ /ack-only_packets_(\d+)_delayed/) {
                $hash{'ack-only_packets_delayed'} = $1;
                $name = 'ack-only_packets';

            } elsif ($name =~ /acks_for_(\d+)_bytes/) {
                $hash{acks_bytes} = $1;
                $name = 'acks';

            } elsif ($name =~ /packets_(\d+)_bytes_received_in-sequence/) {
                $hash{'bytes_received_in-sequence'} = $1;
                $name = 'packets_received_in-sequence';

            } elsif ($name =~ /completely_duplicate_packets_(\d+)_bytes/) {
                $hash{'completely_duplicate_packets_bytes'} = $1;
                $name = 'completely_duplicate_packets';

            } elsif (
                $name =~ /packets_with_some_duplicate_data_(\d+)_bytes_duped/) {
                $hash{'bytes_duped'} = $1;
                $name = 'packets_with_some_duplicate_data';

            } elsif ($name =~ /out-of-order_packets_(\d+)_bytes/) {
                $hash{'out-of-order_bytes'} = $1;
                $name = 'out-of-order_packets';

            } elsif ($name =~ /packets_(\d+)_bytes_of_data_after_window/) {
                $hash{'bytes_of_data_after_window'} = $1;
                $name = 'packets_of_data_after_window';

            } elsif ($name =~ /connections_closed_including_(\d+)_drops/) {
                $hash{'connection_drops'} = $1;
                $name = 'connections_closed';

            } elsif ($name =~ /segments_updated_rtt_of_(\d+)_attempts/) {
                $hash{'segments_updated_rtt_attempts'} = $1;
                $name = 'segments_updated_rtt';

            }
            $hash{$name} = $value;
            $name = '';
        } elsif ($_ =~ /(.+):$/) {
            $name = to_key_name(item => $1);
            if ($_ =~
                /Classifier hash queue counts|Input\s+histogram|Output\s+histogram/i
              ) {
                $flag_array = 1;
            }
        }
    } ## end foreach (@lines)
    return [\%hash];
} ## end sub _s_and_p_parser

sub _rs_parser {
    my @lines = @{$_[0]};
    my (%hash, $name, $value);
    foreach (@lines) {
        if ($_ =~ /^(\d+) (.+)/) {
            ($value, $name) = ($1, $2);
            $hash{to_key_name(item => $name)} = $value;
        }
    }
    return [\%hash];
} ## end sub _rs_parser

sub _table_parser {

    # from heading line
    my @lines = @{$_[0]};

    # get the column names splitting on two or more spaces
    my @temp_headings = split /\s\s+/, shift @lines;

    # the array that we got previously contains some elements which have more
    # than one column name in them.
    # These must become separate headings - stored in @new_headings

    my ($heading, @new_headings, @temp);
    foreach (@temp_headings) {
        $heading = to_key_name(item => $_);
        $heading =~ s/\?//;
        if ($heading =~ /-Q/i) {
            @temp = split /_/, $heading;
        } else {
            @temp = ($heading);
        }
        push @new_headings, @temp;
    } ## end foreach (@temp_headings)

    my (@values, @arr, $i);

    # Now, process the remaining lines (rows)
    foreach (@lines) {
        my %hash;
        @values = split /\s+/, $_;
        $i = 0;
        foreach (@values) {
            $hash{$new_headings[$i]} = $_;
            $i++;
        }
        push @arr, \%hash;
    } ## end foreach (@lines)

    return \@arr;
} ## end sub _table_parser

sub _parser {

    my @lines = @{$_[0]};
    my ($line,@result_arr,%hash);
    while ( $line = shift(@lines) ) {
	if ( $line =~ /Active Internet/ ) {
	    my $head = $line;
	    my (@array,@key,%hash_new);
	    for ( my $count = 0 ; $count < @lines ; $count++ ) {
		if ( $lines[$count] =~ /^(Proto)/i ) {
		    $lines[$count] =~ s/ /  /g;
                    $lines[$count] =~ s/  Address/ Address/g;
		    @key = split(/\s\s+/, $lines[$count]);
		    shift @lines;
		}
		last if ( $lines[$count] =~ /Active UNIX/ );
		next if ( $lines[$count] =~ /Default IPSpace|ANYVSERVER IPSpace|Blackhole IPSpace|Cluster IPSpace|IPSpace|ACP IPSpace/ );
		my @values = split( /\s+/, $lines[$count]);
		for ( my $i=0 ; $i < @key ; $i++ ){
		    $array[$count]->{to_key_name(item => $key[$i])} = $values[$i];
		}
	    }
	    push @{$hash{to_key_name(item => $head)}},\@array ;
	} elsif ( $line =~ /Active UNIX/ ) {
	    my $head = $line;
	    my (@array,@key,%hash_new);
            for ( my $count = 0 ; $count < @lines ; $count++ ) {
                if ( $lines[$count] =~ /^(Address)/ ) {
		    @key = split(/\s+/, $lines[$count]);
		    shift @lines;
		}
		last if ( $lines[$count] =~ /Active Internet/ );
		next if ( $lines[$count] =~ /Default IPSpace|ANYVSERVER IPSpace|Blackhole IPSpace|Cluster IPSpace|IPSpace|ACP IPSpace/ );
                my @values = split( /\s+/, $lines[$count]);
                for ( my $i=0 ; $i < @key ; $i++ ){
                    $array[$count]->{to_key_name(item => $key[$i])} = $values[$i];
                }
            }
            push @{$hash{to_key_name(item => $head)}},\@array ;
        }
    }
    push( @result_arr,\%hash ); 
    return \@result_arr;

} ## end sub _parser

sub _get_sections {

    # returns separate sections of output each having multiple lines based on
    # key words (used to perform the split)
    # each section begins with the line used to perform the split
    my %args    = @_;
    my @lines   = @{$args{'lines_ref'}};
    my @strings = @{$args{'strings_ref'}};

    my ($line, @sections, @part);

    while (@strings && @lines) {
        $line = $lines[0];
        if ($line =~ /$strings[0]/) {
            my @push_this = @part;
            push(@sections, \@push_this);
            @part = ();
            shift(@strings);
        }
        push(@part, $line);
        shift(@lines);
    } ## end while (@strings && @lines)

    # ---------for the last part---------------
    push(@part,     @lines);
    push(@sections, \@part);

    # -----------------------------------------
    shift(@sections) if (!@{$sections[0]});
    return \@sections;
} ## end sub _get_sections

1;
