#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode nfs commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode nfs commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "nfs stat" will be nfs_stat()
##
## @synopsis
##    use nfs;
##
##    nfs_on(%args);
##    nfs_off(%args);
##    nfs_status(%args);
##    nfs_stat(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::nfs;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};

    my $error;
    if($output =~ /(NFS service is not licensed)/) { #for nfs on, off, status
        $error = "CommandFailed";
        $$error_message = $1;
        return $error; 
    } elsif($output =~ /$command(?:[^:]*):(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /must be a positive/) {
            $error = "InvalidParamValue";
        } elsif($$error_message =~ /([Ii]nvalid option)|([Ee]xtra arguments)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /currently not being/) {
            $error = "CommandFailed";
        }
        return $error;
    } elsif ($output =~ /[Uu]sage:/) {
        $error = "CommandFailed";
        $$error_message = "Invalid switch or Invalid Param value has been sent";
        return $error;
    }
    return undef;
}

sub _generic_parser {
    my %args = @_;
    
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    
    my $error = _check_error(raw_output=>$output,command=>'',
                             error_message=>\$error_message);

    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    } elsif($output =~ /NFS\sserver\sis\s(.*)\./) {
        my @result_array = ({'status'=>$1});
        return \@result_array;
    }

    NACL::APISet::Exceptions::UnexpectedOutputException->throw(
        text=>"-- $output -- could not be parsed",response_object => $resp);
}

## @name ::nfs_status
##
## @summary Parses the raw output from the "nfs status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nfs_status with any of its options as described in the 
##    corresponding cdef file 'nfs.cdefs'
## @synopsis
##      nfs_status(%args);
## INPUTS TO METHOD CALL 'nfs_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub nfs_status {
    return (_generic_parser(@_));
}


## @name ::nfs_on
##
## @summary Parses the raw output from the "nfs on" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nfs_on with any of its options as described in the 
##    corresponding cdef file 'nfs.cdefs'
## @synopsis
##      nfs_on(%args);
## INPUTS TO METHOD CALL 'nfs_on' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub nfs_on () {
    return (_generic_parser(@_));
}


## @name ::nfs_off
##
## @summary Parses the raw output from the "nfs off" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nfs_off with any of its options as described in the 
##    corresponding cdef file 'nfs.cdefs'
## @synopsis
##      nfs_off(%args);
## INPUTS TO METHOD CALL 'nfs_off' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub nfs_off () {
    return (_generic_parser(@_));
}

## @name ::nfs_stat
##
## @summary Parses the raw output from the "nfs stat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nfs_stat with any of its options as described in the 
##    corresponding cdef file 'nfs.cdefs'
## @synopsis
##      nfs_stat(%args);
## INPUTS TO METHOD CALL 'nfs_stat' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-C' </td> <td> 'flex-cache' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'since-boot' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'include-reply-cache-statistics' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'count' </td> </tr> 
## <tr> <td> '-z' </td> <td> 'zero' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'interval' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub nfs_stat {
    my %args = @_;
    
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
        
    my $error = _check_error(raw_output=>$output,command=>"nfs stat",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }

    my ($first_line, @lines, $line, $item, $i);

    #separate parsing logic for this(shown in regex) form of command invocation
    if($command =~ /nfs\s+stat\s+\d+\s*\d*\s*/) {
        $first_line = 'getattr\s+lookup\s+readlink\s+read\s+write\s+create\s+remove';
        @lines = get_lines(raw_output=>$output, first_line=>$first_line,
                    can_throw=>1, response_object=>$resp);
        my @keys_array = split(/\s+/,$lines[0]); # get the keys
        shift (@lines); # remove keys from output lines

        foreach $line(@lines) {
            my %hash = ();
            my @values = split(/\s\s+/,$line);
            $i = 0;
            foreach $item (@keys_array) {
                $hash{$item} = defined($values[$i])?$values[$i]:'';
                $i = $i+1;
            }
            push (@result_array, \%hash);
        }
    } else { #parser for all other forms
        if($command =~ /nfs\s+stat\s+-C\s*/) {
            $first_line = 'Cache nfs:';
        } elsif($command =~ /nfs\s+stat\s+-z\s*/) {
            return @result_array;
        } else  { #if($command =~ /nfs\sstat\s*$/)
            $first_line = 'Server rpc:';
        }

        my %main_keys = ('write_request_stats'=>1,
            'read_request_stats'=>1, 'server_nfs'=>1,
            'tcp'=>1, 'udp'=>1, 'cache_nfs'=>1, 'server_rpc'=>1,
            'nfs_reply_cache_statistics'=>1, #its usually starts like nfs_v2_reply
            'nfs_non-blocking_request_statistics'=>1);

        @lines = get_lines(raw_output=>$output, first_line=>$first_line,
                    can_throw=>1, response_object=>$resp);

        my $flag = 1;
        my (@keys_array, @values, $name, $hash_ref, $jump, %hash, $name2);

        foreach my $line (@lines) {
            if($flag) {
                $name = to_key_name(item=>$line);
                $name2 = $name;
                $name2 =~ s/_(version_\d+|v\d+)//g;
                if($main_keys{$name2}) {
                    if($name eq 'tcp') {
                        $hash{'stats'} = [];
                        $hash{'stats'}->[0] = {};
                        $hash{'stats'}->[0]->{'name'} = $name;
                        $hash_ref = \%{$hash{'stats'}->[0]};
                    } elsif($name eq 'udp') {
                        $hash{'stats'}->[1] = {};
                        $hash{'stats'}->[1]->{'name'} = $name;
                        $hash_ref = \%{$hash{'stats'}->[1]};
                    } else {
                        my %hash2 = %hash;
                        push(@result_array, \%hash2) if keys(%hash);
                        %hash = (); # clear
                        if($line =~ /(\d+) calls/) {
                            $hash{'calls'} = $1;
                        }
                        if($line =~ /(\d+) ops/) {
                            $hash{'ops'} = $1; 
                        }
                        $hash{'name'} = $name;    # ready hash for the next section
                        $hash_ref = \%hash;
                    }
                } else { #for sub-keys
                    @keys_array = split(/\s\s+/,$line);
                    $flag = 0; # get values
                }
            } else {
                @values = split(/\s\s+/,$line);
                $i = 0;
                foreach $item (@keys_array) {
                    $jump = 1;
                    if($item =~ /([^\s]*)\s(.*)/) {
                        my $item2 = $2;
                        $item2 = to_key_name(item=>$item2);
                        $hash_ref->{$item2} = defined($values[$i+$jump])?$values[$i+$jump]:'';
                        $item = $1;
                        $jump = 2;
                    }
                    $item = to_key_name(item=>$item);
                    $hash_ref->{$item} = defined($values[$i])?$values[$i]:'';
                    $i = $i+$jump;
                }
                $flag = 1;
            }
        }
        push(@result_array, \%hash);
    }
    return \@result_array;
}

1;
