#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode options commands
##
## @description
##     This is a library to parse the raw output of 7Mode options commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use options;
##
##    options(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::options;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException ();
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _options(%) {
    my %args       = @_;
    my @temp_lines = @{ $args{'lines'} };
    my $resp       = $args{'response_obj'};
    my $command    = $resp->get_actual_command();
    my @returnarray;
    my %hash;
    my @lines;
    while ( scalar(@temp_lines) ) {
        my $line = shift(@temp_lines);
        next if ( $line =~ /Command line input: the command is 'options'/ );
        push( @lines, $line );
    }
    if ( scalar(@lines) == 0 ) {
        return [];
    }
    foreach my $line (@lines) {

        #next if($line =~ /Setting invalid option/);
        if ( $command !~ /^options\s+\S+\s+\w+/ or $command =~ / node / ) {
            if ( $line =~ /\S+\s+\S+\s+\(value.*/ ) {
                my ( $key, $value ) = split( / +/, $line );
                $hash{$key} = $value;
            }
            elsif ( $line =~ /\S+\s+\(value.*/ ) {
                my ($key) = split( / +/, $line );
                $hash{$key} = '';
            }
            else {
                my ( $key, $value ) = split( / +/, $line );
                $hash{$key} = $value;
            }
        } else {
            return [];
        }
    }
    push( @returnarray, \%hash );
    return \@returnarray;
}

sub _parse_for_errors(%) {
    my %args         = @_;
    my $raw_output   = $args{'raw_output'};
    my $command      = $args{'command'};
    my $errormsg_ptr = $args{'errormsg_ptr'};
    my @commandarg   = split( / +/, $command );
    my $optionname   = $commandarg[1];
    if (   ( $raw_output =~ /No such option .*/ )
        || ( $raw_output =~ /option .*: Value must be .*/ )
        || ( $raw_output =~ /option .*: Invalid value.*/ )
        || ( $raw_output =~ /$optionname: Invalid/ )
        || ( $raw_output =~ /Setting invalid option/ ) )
    {
        $$errormsg_ptr = $command . " contains invalid param value(s).";
        return "InvalidParamValue";
    }
    elsif ( $raw_output =~ /not licensed/ ) {
        $$errormsg_ptr = $raw_output;
        return 'FeatureNotLicensed';
    }
    return undef;
}
## @name ::options
##
## @summary Parses the raw output from "options" command
##
## @description
##    To set an option to a particular value, specify the option name as the key and the value to set to as the value.
##    To show the value of a single option, use 'show-options' as key and the option name as the value.
##
## @synopsis
##    options(%args); The Key-value is provided as argument programatically. Multiple
##                    keys can be given.
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref which contains hash refs that contain parsed
##    output of various sections that appear in the output

sub options(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 1;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines =
      get_lines( 'raw_output' => $raw_output, 'first_line' => $first_line );

    my $result_array_ptr =
      _options( 'lines' => \@lines, 'response_obj' => $resp );
    return $result_array_ptr;
}    # end of function options

1;
