#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/partner.pm#  $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode partner command
##
## @description
##    This is a library to parse the raw output of 7Mode partner  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use partner ;
##
##    partner_ifconfig(%args);
##    partner_ifstat(%args);
##    partner_options_ip(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::partner;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);


## @name ::partner_ifconfig
##
## @summary Parses the raw output from the "partner ifconfig" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from partner_ifconfig with any of its options as described in the
##    corresponding cdef file 'partner_ifconfig.cdefs'
## @synopsis
##      partner_ifconfig(%args);
## INPUTS TO METHOD CALL 'partner_ifconfig' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'nfo' </td> <td> 'nfo' </td> </tr>
## <tr> <td> '-wins' </td> <td> 'no_wins' </td> </tr>
## <tr> <td> '-partner' </td> <td> 'no_partner' </td> </tr>
## <tr> <td> 'mtusize' </td> <td> 'mtusize' </td> </tr>
## <tr> <td> 'up' </td> <td> 'up' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## <tr> <td> 'wins' </td> <td> 'wins' </td> </tr>
## <tr> <td> 'broadcast' </td> <td> 'broadcast' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> 'netmask' </td> <td> 'netmask' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'address' </td> </tr>
## <tr> <td> 'untrusted' </td> <td> 'untrusted' </td> </tr>
## <tr> <td> 'down' </td> <td> 'down' </td> </tr>
## <tr> <td> '-alias' </td> <td> 'unalias' </td> </tr>
## <tr> <td> 'flowcontrol' </td> <td> 'flowcontrol' </td> </tr>
## <tr> <td> 'trusted' </td> <td> 'trusted' </td> </tr>
## <tr> <td> '-nfo' </td> <td> 'no_nfo' </td> </tr>
## <tr> <td> 'no_ddns' </td> <td> 'no_ddns' </td> </tr>
## <tr> <td> 'mediatype' </td> <td> 'mediatype' </td> </tr>
## <tr> <td> 'alias' </td> <td> 'alias' </td> </tr>
## <tr> <td> 'partner' </td> <td> 'partner' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; partner ifconfig -a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'broadcast' => '127.0.0.1',
##&nbsp;            'ether' => '00:00:00:00:00:00(VIAProvider)',
##&nbsp;            'flags' => [
##&nbsp;                         {
##&nbsp;                           'RUNNING',
##&nbsp;                           'LOOPBACK',
##&nbsp;                           'UP',
##&nbsp;                           'PARTNER_UP',
##&nbsp;                           'TCPCKSUM',
##&nbsp;                           'MULTIHOST',
##&nbsp;                           '0x1be8049',
##&nbsp;                           'MULTICAST'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'mode' => 'takeovermode(lo)',
##&nbsp;            'inet' => '127.0.0.1',
##&nbsp;            'name' => 'lo',
##&nbsp;            'netmask' => '0xff000000',
##&nbsp;            'mtu' => '8160'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub partner_ifconfig {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'partner ifconfig',
                             "ptr_error_message" => \$error_message
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # Now we can parse the output
    my @result_array = ();
    my $key;
    my @lines = get_lines("raw_output"      => $raw_output,
                        "can_throw"       => 0,
                        "response_object" => $resp
                        );

    my $interface;
    my $i = -1;
    my $m = 0;
    foreach my $line (@lines) {
        if ($line =~ /^([a-z0-9\-]+)\:\s+flags=(\w+)\<(.+)\>/) {
            $interface = $1;
            my $hex    = $2;
            my $states = $3;
            $interface =~ s/:$//;
            $i++;
            $result_array[$i] -> {'name'} = $interface;
            $result_array[$i] -> {'flags'} -> [$m++] = $hex;
            foreach my $state (split(/,/, $states)) {
                $result_array[$i] -> {'flags'} -> [$m++] = $state;
            }
            if ($line =~ /mtu\s+(\d+)/) {
                $result_array[$i] -> {'mtu'} = $1;
            }
            if ($line =~ /dad_attempts\s+(\d+)/) {
                $result_array[$i] -> {'dad_attempts'} = $1;
            }
        } elsif ($line =~ /inet\s+(\S+)\s+(?:netmask|netmask-or-prefix)\s+(\S+)\s+broadcast\s+(\S+)/) {
                $result_array[$i] -> {'inet'} = $1;
                $result_array[$i] -> {'netmask'} = $2;
                $result_array[$i] -> {'broadcast'} = $3;
                if($line =~ /(noddns)/) {
                $result_array[$i] -> [$m++] = $1 if $4;
                }
                if($line =~ /vFiler ID:\s+(\d+)/) {
                $result_array[$i] -> {'vfiler-id'} = $6 if $5;
                }
        } elsif ($line =~ /ether\s+(\S+\s+[0-9a-z-()A-Z ]+)/) {
                $key = $1;
                $key =~ s/\s+//g;
                $result_array[$i] -> {'ether'} = $key;
                if ($line =~ /flowcontrol\s+(\S+)/) {
                    $result_array[$i] -> {'flowcontrol'} = $1;
                }
        } elsif($line =~ /^(\w+\s+\w+\s+\(\w+\))$/) {
                $key = $1;
                $key =~ s/\s+//g;
                $result_array[$i] -> {'mode'} = $key;
        }
    }
    return \@result_array;
}

## @name ::partner_ifstat
##
## @summary Parses the raw output from the "partner ifstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from partner_ifstat with any of its options as described in the
##    corresponding cdef file 'partner_ifstat.cdefs'
## @synopsis
##      partner_ifstat(%args);
## INPUTS TO METHOD CALL 'partner_ifstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## <tr> <td> '-z' </td> <td> 'clearence' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; partner ifstat -a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;            {
##&nbsp;             'receive' => [
##&nbsp;                            {
##&nbsp;                              'Packets' => '62109',
##&nbsp;                              'Queuefull' => '0',
##&nbsp;                              'Errors' => '0',
##&nbsp;                              'Bytes' => '6011k'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;             'seconds' => '34',
##&nbsp;             'interface_name' => 'lo',
##&nbsp;             'hours' => '4',
##&nbsp;             'minutes' => '8',
##&nbsp;             'transmit' => [
##&nbsp;                             {
##&nbsp;                               'Packets' => '62109',
##&nbsp;                               'Collisions' => '0',
##&nbsp;                               'Errors' => '0',
##&nbsp;                               'Bytes' => '6011k'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;             'days' => '146'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'receive' => [
##&nbsp;                            {
##&nbsp;                              'Packets' => '0',
##&nbsp;                              'Queuefull' => '0',
##&nbsp;                              'Errors' => '0',
##&nbsp;                              'Bytes' => '0'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;             'seconds' => '34',
##&nbsp;             'interface_name' => 'vh',
##&nbsp;             'hours' => '4',
##&nbsp;             'minutes' => '8',
##&nbsp;             'transmit' => [
##&nbsp;                             {
##&nbsp;                               'Packets' => '0',
##&nbsp;                               'Collisions' => '0',
##&nbsp;                               'Errors' => '0',
##&nbsp;                               'Bytes' => '0'
##&nbsp;                                                         }
##&nbsp;                           ],
##&nbsp;             'days' => '146'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;

sub partner_ifstat {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'partner ifstat',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array;
    my ($i, $j, $k, $key1, $key2, $key3, $name);
    my ($value1, $value2, $value3);
    my @lines = get_lines( raw_output => $raw_output, can_throw => 1,
                           response_object => $resp );
    $i = -1;
    $j = 0;

    foreach my $line (@lines) {

        $name = "receive" if($line =~ /^RECEIVE$/);
        $name = "transmit" if($line =~ /^TRANSMIT$/);
        $name = "link_info" if($line =~ /^LINK_INFO$/);

    if($line =~ /^...\w+\s+(\S+)\s+.(\w+)\s\w+.\s(\w+)\s\w+.\s(\w+)\s\w+.\s(\w+)\s\w+..../) {
        $i++;
        $result_array[$i] -> {'interface_name'} = $1;
        $result_array[$i] -> {'days'} = $2;
        $result_array[$i] -> {'hours'} = $3;
        $result_array[$i] -> {'minutes'} = $4;
        $result_array[$i] -> {'seconds'} = $5;
    } elsif($line =~ /^...\w+\s+(\w+)\s+.(\w+)\s\w+.\s(\w+)\s\w+.\s(\w+)\s\w+..../) {
        $i++;
        $result_array[$i] -> {'interface_name'} = $1;
        $result_array[$i] -> {'hours'} = $2;
        $result_array[$i] -> {'minutes'} = $3;
        $result_array[$i] -> {'seconds'} = $4;
    } elsif($line =~ /^...\w+\s+(\w+)\s+.(\w+)\s\w+.\s(\w+)\s\w+..../) {
        $i++;
        $result_array[$i] -> {'interface_name'} = $1;
        $result_array[$i] -> {'minutes'} = $2;
        $result_array[$i] -> {'seconds'} = $3;
    } elsif($line =~ /^...\w+\s+(\w+)\s+.(\w+)\s\w+..../) {
        $i++;
        $result_array[$i] -> {'interface_name'} = $1;
        $result_array[$i] -> {'seconds'} = $2;
    } elsif($line =~ /^(\w+.\w+).\s+(\w+)\s+.\s(\w+.\w+).\s+(\w+)\s+.\s(\w+.\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
    } elsif($line =~ /^(\w+.\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $key2 =~ s/([(]|[)]|\s+)//g;
        $key3 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
    } elsif($line =~ /^(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+.\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $key1 =~ s/([(]|[)]|\s+)//g;
        $key2 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
    } elsif($line =~ /^(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+.\w+\s?\w+.\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $key1 =~ s/([(]|[)]|\s+)//g;
        $key3 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
    } elsif($line =~ /^(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $key3 = $5;
        $value1 = $2;
        $value2 = $4;
        $value3 = $6;
        $key1 =~ s/([(]|[)]|\s+)//g;
        $key2 =~ s/([(]|[)]|\s+)//g;
        $key3 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key3)} = $value3;
    } elsif($line =~ /^(\w+\s?\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $value1 = $2;
        $key1 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
    } elsif($line =~ /^(\w+\s?\w+\s?\w+).\s+(\w+)\s+.\s(\w+\s?\w+\s?\w+).\s+(\w+)/) {
        $key1 = $1;
        $key2 = $3;
        $value1 = $2;
        $value2 = $4;
        $key1 =~ s/([(]|[)]|\s+)//g;
        $key2 =~ s/([(]|[)]|\s+)//g;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key1)} = $value1;
        $result_array[$i] -> {"$name"} -> [$j] -> {to_key_name(item=>$key2)} = $value2;
     }
   }
    return \@result_array;
}

## @name ::partner_options_ip
##
## @summary Parses the raw output from the "partner options ip" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from partner_options_ip with any of its options as described in the
##    corresponding cdef file 'partner.cdefs'
## @synopsis
##      partner_options_ip(%args);
## INPUTS TO METHOD CALL 'partner_options_ip' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; partner options ip; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip.tcp.sack.enable' => 'on',
##&nbsp;            'ip.icmp_ignore_redirect.enable' => 'off',
##&nbsp;            'ip.ping_throttle.alarm_interval' => '0',
##&nbsp;            'ip.match_any_ifaddr' => 'on',
##&nbsp;            'ip.ipsec.enable' => 'off',
##&nbsp;            'ip.tcp.newreno.enable' => 'on',
##&nbsp;            'ip.v6.enable' => 'off',
##&nbsp;            'ip.tcp.abc.enable' => 'on',
##&nbsp;            'ip.ping_throttle.drop_level' => '150',
##&nbsp;            'ip.v6.ra_enable' => 'on',
##&nbsp;            'ip.tcp.batching.enable' => 'on',
##&nbsp;            'ip.tcp.rfc3390.enable' => 'on',
##&nbsp;            'ip.fastpath.enable' => 'on',
##&nbsp;            'ip.tcp.abc.l_limit' => '2',
##&nbsp;            'ip.path_mtu_discovery.enable' => 'on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub partner_options_ip{
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error(raw_output=>$output, command=>
                    'partner options ip',error_message=>\$error_message);
    my %info;
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if ($output =~ /ip\.\w+.*/) {
        my @arr = get_lines("raw_output"=>$output,
        "response_object"=>$resp);
        foreach my $value (@arr){
            if ($value =~ /(^ip\..*?)\s+(\w+)/) {
                $info{$1} = $2;
            }
        }
        push(@result_array, \%info);
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(text=>"unexpected format of output",response_object=>$resp);
    }
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    my $invalidParamRegex =
      '(bad address|[Nn]o such interface|no such flow control type|Invalid option)';
    if($raw_output =~ /ifconfig \[ -a \| \[ <interface>|ifstat [-z]  -a |
                      <interface>/) {
        $$error_message = "Required parameter missing";
        $error          = "InvalidParam";
    } elsif($raw_output =~ /$invalidParamRegex/) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    }
    return $error;
}
1;
