#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ping command output
##
## @description
##    This is a library to parse the raw output of 7Mode ping command.
##
## @synopsis
##    use ping;
##
##    ping(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ping;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(ping|[Uu]sage):\s(.*)/ ) {
        $$error_message = $raw_output;
        $error          = "CommandFailed";
        if ( $$error_message =~ /([Ii]nvalid|[Uu]nrecognized) option/ ) {
            $error = "InvalidParam";
        } elsif (( $$error_message !~ /unknown host/i ) && ($$error_message !~ /No route to host/i) && ($$error_message =~ /illegal/)){
            $$error_message
                = "Command failed due to wrong usage or dependencies";
        } 
        
    }elsif ( $raw_output =~ /(no\s+answer\s+from\s+(.*))/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /(ping6: Domain not found)/) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } 
    return $error;
}

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'datagram-every-second' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'bypass-routing-tables' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'record-route' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'packet-size' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interface' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ping -c 3 10.72.219.44; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;10.72.219.44 is alive
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'state' => 'alive',
##&nbsp;            'host' => '10.72.219.44'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; ping -c 3 ::1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING6(56=40+8+8 bytes) ::1 --> ::1
##&nbsp;16 bytes from ::1, icmp_seq=0 hlim=64 time=0 ms
##&nbsp;16 bytes from ::1, icmp_seq=1 hlim=64 time=0 ms
##&nbsp;16 bytes from ::1, icmp_seq=2 hlim=64 time=0 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'state' => 'alive',
##&nbsp;            'host' => '::1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my %hash;
    unless ( $command =~ /\s+-s\s*/ ) {
        if ( $output =~ /(.*)\s+is\s+alive\s/ ) {
            $hash{host}  = $1;
            $hash{state} = "alive";
        } elsif ( $output =~ /(.*)\s+bytes\s+from\s+(.*),\s/ ) {
            $hash{host}  = $2;
            $hash{state} = "alive";
        }
    }
    @result_array = ( \%hash );
    return \@result_array;
}

1;
