#
#Copyright (c) 2011-13 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ping6 command output
##
## @description
##    This is a library to parse the raw output of 7Mode ping6 command.
##
## @synopsis
##    use ping6;
##
##    ping6(%args);
##
## @author benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ping6;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    $$error_message   = $raw_output;
    my $error;
    if ( $raw_output =~ /([Uu]sage:\s.*)|(no\s+answer\s+from\s+.*)|(ping\s+encountered\s+an\s+error.*)/i ) {
        $error = "CommandFailed";
    } elsif ( $raw_output
        =~ /(patterns must be specified as hex digits)|(.*Device not configured)|(illegal number of packets.*)|(illegal timing interval.*)|(illegal datalen value.*)/i
        ) {
        $error = "InvalidParamValue";
    } elsif ( $raw_output
        =~ /(Domain not found)|(No address associated with hostname)/i ) {
        $error = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::ping6
##
## @summary Parses the raw output from the "ping6" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ping6 with any of its options as described in the
##    corresponding cdef file 'ping6.cdefs'
## @synopsis
##      ping6(%args);
## INPUTS TO METHOD CALL 'ping6' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'probe-node-information-multicast-group' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'reverse-lookup' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hops' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'packetsize' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'sourceaddr' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'addrtype' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet-output' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'generate-icmpv6-node-info-fqdn-query' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'wait' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bufsize' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'set-so-debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'hoplimit' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'preload' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-output-only' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = ping6 -v -c 3 fd20:8b1e:b255:4219::14
##&nbsp;-----------------------------------------------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source' => 'fd20:8b1e:b255:8305:2a0:98ff:fe23:d0a1',
##&nbsp;            'destination' => 'fd20:8b1e:b255:4219::14',
##&nbsp;            'hops' => [
##&nbsp;                        {
##&nbsp;                          'bytes' => '16',
##&nbsp;                          'time' => '233 ms',
##&nbsp;                          'destination' => 'fd20:8b1e:b255:8305:2a0:98ff:fe23:d0a1',
##&nbsp;                          'icmp_seq' => '0',
##&nbsp;                          'hop_limit' => '59',
##&nbsp;                          'node' => 'fd20:8b1e:b255:4219::14'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '16',
##&nbsp;                          'time' => '231 ms',
##&nbsp;                          'destination' => 'fd20:8b1e:b255:8305:2a0:98ff:fe23:d0a1',
##&nbsp;                          'icmp_seq' => '1',
##&nbsp;                          'hop_limit' => '59',
##&nbsp;                          'node' => 'fd20:8b1e:b255:4219::14'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '16',
##&nbsp;                          'time' => '231 ms',
##&nbsp;                          'destination' => 'fd20:8b1e:b255:8305:2a0:98ff:fe23:d0a1',
##&nbsp;                          'icmp_seq' => '2',
##&nbsp;                          'hop_limit' => '59',
##&nbsp;                          'node' => 'fd20:8b1e:b255:4219::14'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'packetsize' => '56=40+8+8 bytes'
##&nbsp;          }
##&nbsp;];
##&nbsp;
sub ping6 {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    #parsing header information
    my $first_line = $lines[0];
    if ( $first_line =~ /PING6\((\S+\s\w+)\)\s+(\S+)\s+-->\s+(\S+)/ ) {
        $result_array[0]->{'packetsize'}  = $1;
        $result_array[0]->{'source'}      = $2;
        $result_array[0]->{'destination'} = $3;
    }
    if ( $first_line =~ /(.*)\s+is\s+(alive)/ ) {
        $result_array[0]->{host}  = $1;
        $result_array[0]->{state}  = $2;
    }

    #removing header information
    shift @lines;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line
            =~ /(\d+)\s+bytes\sfrom\s(\S+),\sicmp_seq=(\d+)\shlim=(\d+)\stime=(\d+\s\w+)/
            ) {
            $result_array[0]->{'hops'}->[$i]->{'bytes'}     = $1;
            $result_array[0]->{'hops'}->[$i]->{'node'}      = $2;
            $result_array[0]->{'hops'}->[$i]->{'icmp_seq'}  = $3;
            $result_array[0]->{'hops'}->[$i]->{'hop_limit'} = $4;
            $result_array[0]->{'hops'}->[$i]->{'time'}      = $5;
            $i++;
        } elsif ( $line
            =~ /(\d+)\s+bytes\sfrom\s(\S+),\sicmp_seq=(\d+)\shlim=(\d+)\sdst=(\S+)\stime=(\d+\s\w+)/
            ) {
            $result_array[0]->{'hops'}->[$i]->{'bytes'}       = $1;
            $result_array[0]->{'hops'}->[$i]->{'node'}        = $2;
            $result_array[0]->{'hops'}->[$i]->{'icmp_seq'}    = $3;
            $result_array[0]->{'hops'}->[$i]->{'hop_limit'}   = $4;
            $result_array[0]->{'hops'}->[$i]->{'destination'} = $5;
            $result_array[0]->{'hops'}->[$i]->{'time'}        = $6;
            $i++;
        } elsif ( $line
            =~ /(\d+)\spackets transmitted,\s(\d+)\spackets received,\s(\S+%)\spacket loss/
            ) {
            $result_array[0]->{'statistics'}->[0]->{'packets_transmitted'}
                = $1;
            $result_array[0]->{'statistics'}->[0]->{'packets_received'} = $2;
            $result_array[0]->{'statistics'}->[0]->{'packets_loss'}     = $3;
        #If received packets are true then the state will be set to alive.
            $result_array[0]->{state}                                   = "alive" if($2);
        } elsif ( $line
            =~ /round-trip\smin\/avg\/max\s=\s(\d+)\/(\d+)\/(\d+)\s(\w+)/ ) {
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_min'}
                = $1 . ' ' . $4;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_avg'}
                = $2 . ' ' . $4;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_max'}
                = $3 . ' ' . $4;
        }    

    }
    return \@result_array;
} ## end sub ping6
1;
