#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode pktt commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode pktt commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "pktt start" will be pktt_start()
##
## @synopsis
##    use pktt;
##
##    pktt_start(%args);
##    pktt_pause(%args);
##    pktt_dump(%args);
##    pktt_stop(%args);
##    pktt_status(%args);
##    pktt_delete(%args);
##    pktt_list(%args);
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::pktt;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::pktt_start
##
## @summary Parses the raw output from the "pktt start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pktt_start with any of its options as described in the
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_start(%args);
## INPUTS TO METHOD CALL 'pktt_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-b' </td> <td> 'buffer-size' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directory-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-m' </td> <td> 'packet-length' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ipaddress' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pktt start e0d /vol/vol0/ 1792 793 10.123.123.32
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'e0d' => 'Already tracing'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pktt start all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'losk' => 'Already tracing',
##&nbsp;            'lo' => 'Already tracing',
##&nbsp;            'e0d' => 'Already tracing'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_start {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt start",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $value (@lines) {
        if($value =~ /(\w+):(\s+)?(.*)/) {
            $temp{$1} = $3;
        } elsif($value =~ /^Already\s+tracing\s+(\w+)/i) {
            $temp{$1} = "Already tracing";
        } elsif($value =~ /Buffer\s+size\s+(changed\s+to.*)/i) {
            $temp{"buffer_size"} = $1;
        }
    }
    push(@return_array, \%temp);

    return \@return_array;
}

## @name ::pktt_pause
##
## @summary Parses the raw output from the "pktt pause" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pktt_pause with any of its options as described in the
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_pause(%args);
## INPUTS TO METHOD CALL 'pktt_pause' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pktt pause e0d
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'e0d' => 'Tracing suspended'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pktt pause all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'losk' => 'Tracing suspended',
##&nbsp;            'lo' => 'Tracing suspended',
##&nbsp;            'e0d' => 'tracing already suspended'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_pause {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt pause",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $value (@lines) {
        if($value =~ /(^pktt\s+.*)\s+on\s+interface\s+(\w+)/) {
            $temp{$2} = $1;
        } elsif ($value =~ /(\w+):\s+(.*)/) {
            $temp{$1} = $2;
        }
    }
    push(@return_array, \%temp);

    return \@return_array;
}

## @name ::pktt_dump
##
## @summary Parses the raw output from the "pktt dump" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from pktt_dump with any of its options as described in the 
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_dump(%args);
## INPUTS TO METHOD CALL 'pktt_dump' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'file-path' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'directory-path' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pktt dump e0d -d /vol/vol0/
##&nbsp;$VAR1 = [];
##&nbsp;
sub pktt_dump {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt dump",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $value (@lines) {
        if($value =~ /^pktt.*:\s*(\w+\s+\w+\s+\w+)\s+\w+\s+\w+\s+(\w+)/) {
            $temp{$2} = $1;
        } elsif($value =~ /^pktt_dump:\s+\(if\s+(\w+)\)\s+(already\s+writing\s+to\s+file\s+.*.trc)/i) {
            $temp{$1} = $2;
        }
        push(@return_array, \%temp); 
    }
    #push(@return_array, \%temp);

    return \@return_array;
}

## @name ::pktt_status
##
## @summary Parses the raw output from the "pktt status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pktt_status with any of its options as described in the
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_status(%args);
## INPUTS TO METHOD CALL 'pktt_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## <tr> <td> '-v' </td> <td> 'show-status' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pktt status all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '259',
##&nbsp;            'total_bytes_seen' => '20218',
##&nbsp;            'interface' => 'e0d',
##&nbsp;            'trace_buffer_utilization' => '18% of 130816 bytes, 259 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '0',
##&nbsp;            'total_bytes_seen' => '0',
##&nbsp;            'interface' => 'lo',
##&nbsp;            'trace_buffer_utilization' => '0% of 130816 bytes, 0 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '0',
##&nbsp;            'total_bytes_seen' => '0',
##&nbsp;            'interface' => 'losk',
##&nbsp;            'trace_buffer_utilization' => '0% of 130816 bytes, 0 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pktt status -v
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '1094',
##&nbsp;            'total_bytes_seen' => '84172',
##&nbsp;            'interface' => 'e0d',
##&nbsp;            'trace_buffer_utilization' => '77% of 130816 bytes, 1094 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'oldest_packet_in_buffer' => '69.481975 sec old',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '0',
##&nbsp;            'total_bytes_seen' => '0',
##&nbsp;            'interface' => 'lo',
##&nbsp;            'trace_buffer_utilization' => '0% of 130816 bytes, 0 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'packets_dropped' => '0',
##&nbsp;            'packets_seen' => '0',
##&nbsp;            'total_bytes_seen' => '0',
##&nbsp;            'interface' => 'losk',
##&nbsp;            'trace_buffer_utilization' => '0% of 130816 bytes, 0 packets',
##&nbsp;            'packets_truncated' => '1514 bytes',
##&nbsp;            'packet_tracing' => 'enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt status",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $var;
    my @array1 = split(/\n\n/,$output);
    for(my $i=0; $i<=$#array1; $i++) {
        my @lines = get_lines('raw_output' => $array1[$i],"response_object" => $resp);
        my %info =();
        foreach $value (@lines) {
            if($value =~ /(\w+):\s+Packet\s+tracing\s+(\w+)\;\s+packets\s+truncated\s+at\s+(\d+\s+\w+)/i) {
                $info{interface} = $1;
                $info{packet_tracing} = $2;
                $info{packets_truncated} = $3;
            } elsif ($value =~ /(\w+):\s+Trace\s+buffer\s+utilization\s+=\s+(.*)/i) {
                $info{trace_buffer_utilization} = $2;
            } elsif ($value =~ /(\w+):\s+(\d+)\s+packets\s+seen;\s+(\d+)\s+packets\s+dropped;\s+(\d+)\s+total\s+bytes\s+seen/i) {
                $info{packets_seen} = $2;
                $info{packets_dropped} = $3;
                $info{total_bytes_seen} = $4;
            } elsif ($value =~ /(\w+):\s+oldest\s+packet\s+in\s+buffer\s+is\s+(.*)/i) {
                $info{oldest_packet_in_buffer} = $2;
            }
        }
        push(@return_array, \%info);
    }
    return \@return_array;
}

## @name ::pktt_stop
##
## @summary Parses the raw output from the "pktt stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pktt_stop with any of its options as described in the
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_stop(%args);
## INPUTS TO METHOD CALL 'pktt_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pktt stop e0d
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'e0d' => 'Tracing stopped and packet trace buffers released.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pktt stop all
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'losk' => 'Tracing stopped and packet trace buffers released.',
##&nbsp;            'lo' => 'Tracing stopped and packet trace buffers released.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_stop {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt stop",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $value (@lines) {
        if($value =~ /(\w+):(\s+)?(.*)/) {
            $temp{$1} = $3;
        }
    }
    push(@return_array, \%temp);

    return \@return_array;
}

## @name ::pktt_delete
##
## @summary Parses the raw output from the "pktt delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from pktt_delete with any of its options as described in the 
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_delete(%args);
## INPUTS TO METHOD CALL 'pktt_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pktt delete e0b_20110630_135505.trc
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'deleting_trace_file' => 'e0b_20110630_135505.trc'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_delete {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt delete",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    foreach $value (@lines) {
        if($value =~ /Deleting\s+trace\s+file\s+(.*)/i) {
            $temp{deleting_trace_file} = $1;
        }
    }
    push(@return_array, \%temp);

    return \@return_array;
}

## @name ::pktt_list
##
## @summary Parses the raw output from the "pktt list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pktt_list with any of its options as described in the
##    corresponding cdef file 'pktt.cdefs'
## @synopsis
##      pktt_list(%args);
## INPUTS TO METHOD CALL 'pktt_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = pktt list
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_list' => [
##&nbsp;                             'e0d_20110707_155447.trc',
##&nbsp;                             'e0d_20110705_141730.trc',
##&nbsp;                             'lo_20110707_155447.trc',
##&nbsp;                             'losk_20110707_155447.trc',
##&nbsp;                             'e0d_20110707_160638.trc',
##&nbsp;                             'e0d_20110707_160800.trc',
##&nbsp;                             'e0d_20110707_170348.trc',
##&nbsp;                             'lo_20110707_170348.trc',
##&nbsp;                             'losk_20110707_170348.trc'
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pktt_list {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %temp    = ();
    my @return_array;
    my $value;
    my @arr;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "pktt list",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines('raw_output' => $output,"response_object" => $resp);
    my $temp1;
    foreach $value (@lines) {
        if($value =~ /(.*.trc)/i) {
            $temp1 = $1;
            push(@arr, $1);
        }
    }
    $temp{"file_list"} = \@arr;
    push(@return_array, \%temp);

    return \@return_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /(Max packet size (\d+) too big for buffer size (\d+).*)|(No such interface.*)|(Invalid IP addr.*)/i){
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif( $raw_output =~ /(Usage.*)/i ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif( $raw_output =~ /(Unable to open.*)|(.*(if \w+) can't create a log file.*)|(Cannot\s+perform\s+pktt\s+on\s+trunked\s+link.*)/i ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}
1;

