#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/portset.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode portset command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode portset command.
##
## @synopsis
##    use portset;
##
##    portset(%args);
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::portset;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /$command(.*)/) {
        $$error_message = $1;
        if($raw_output =~ /Missing/) {
            $error = "MissingRequiredParam";
        } elsif($raw_output =~ /(No such .*)|(.* already exists.*)|(.* does not exist)|(usage:)/) {
            $error = "InvalidParamValue";
        } elsif($raw_output =~ /([uU]nrecognized option|[Ee]xtra arguments|[Ii]nvalid)/) {
            $error = "InvalidParam";
        }
        return $error;
    }
    return undef;
}

## @name ::portset_create
##
## @summary Parses the raw output from the "portset create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portset_create with any of its options as described in the
##    corresponding cdef file 'portset.cdefs'
## @synopsis
##      portset_create(%args);
## INPUTS TO METHOD CALL 'portset_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filer-port' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fcp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset-name' </td> </tr>
## <tr> <td> '-i' </td> <td> 'iscsi' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =  portset create -f port_2
##&nbsp;$VAR1 = [];
##&nbsp;

sub portset_create {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
}


## @name ::portset_add
##
## @summary Parses the raw output from the "portset add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portset_add with any of its options as described in the
##    corresponding cdef file 'portset.cdefs'
## @synopsis
##      portset_add(%args);
## INPUTS TO METHOD CALL 'portset_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filer-port' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = portset add  port_2 0d
##&nbsp;$VAR1 = [];
##&nbsp;

sub portset_add {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
}
## @name ::portset_show
##
## @summary Parses the raw output from the "portset show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portset_show with any of its options as described in the
##    corresponding cdef file 'portset.cdefs'
## @synopsis
##      portset_show(%args);
## INPUTS TO METHOD CALL 'portset_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = portset show port_1
##&nbsp; Output $VAR1 = [
##&nbsp;          {
##&nbsp;            'ports' => [
##&nbsp;                         'ssan-3210-8a',
##&nbsp;                         '0c',
##&nbsp;                         '1b',
##&nbsp;                         'ssan-3210-8b',
##&nbsp;                         '0c'
##&nbsp;                       ],
##&nbsp;            'igroups' => [
##&nbsp;                           'mike2',
##&nbsp;                           'mikefcp2'
##&nbsp;                         ],
##&nbsp;            'port-details' => [
##&nbsp;                                {
##&nbsp;                                  'ports' => [
##&nbsp;                                               '0c',
##&nbsp;                                               '1b'
##&nbsp;                                             ],
##&nbsp;                                  'node' => 'ssan-3210-8a'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'ports' => [
##&nbsp;                                               '0c'
##&nbsp;                                             ],
##&nbsp;                                  'node' => 'ssan-3210-8b'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'portset-name' => 'mikefcp2'
##&nbsp;          }
##&nbsp;        ];
sub portset_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $line;
    my @arr;
    my @result_array;
    my @result_array1;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $i = 0;
    my $j = 0;
    @arr = get_lines( raw_output => $output, response_object => $resp );
    while ( $arr[$i] =~ /(\w+)\s+\(FCP\):/ ) {
        $result_array[$j]->{"portset-name"} = $1;
        $i++;
        if ( $arr[$i] =~ /ports:/ ) {
            $i++;
            my $pd = 0;
            my @node_port_list;
            my @all_ports;
            while ( $arr[$i] !~ /igroups:/ && $arr[$i] !~ /(\w+)\s+\(FCP\):/) {
                @node_port_list = split( /\s+/, $arr[$i] );
                push ( @all_ports,@node_port_list );
                $result_array[$j]->{ports} = \@all_ports;
                my ($node,@arr) =  @node_port_list;
                $result_array[$j]->{'port-details'}[$pd]{node} = $node;
                $result_array[$j]->{'port-details'}[$pd]{ports} = \@arr;
                $i++;
                $pd++;
            }
        }
        if ( $arr[$i] =~ /igroups:/ && $arr[$i] !~ /(\w+)\s+\(FCP\):/ ) {
            $i++;
            my @igroups = split /\s+/, $arr[$i];
            $result_array[$j]->{igroups} = \@igroups;
            $i++;
        }
        $j++;
    }
    return \@result_array;
} ## end sub portset_show

## @name ::portset_destroy
##
## @summary Parses the raw output from the "portset destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portset_destroy with any of its options as described in the
##    corresponding cdef file 'portset.cdefs'
## @synopsis
##      portset_destroy(%args);
## INPUTS TO METHOD CALL 'portset_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'fcp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = portset destroy port_2
##&nbsp;$VAR1 = [];
##&nbsp;

sub portset_destroy {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
}

## @name ::portset_remove
##
## @summary Parses the raw output from the "portset remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portset_remove with any of its options as described in the
##    corresponding cdef file 'portset.cdefs'
## @synopsis
##      portset_remove(%args);
## INPUTS TO METHOD CALL 'portset_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filer-port' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'portset-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = portset remove port_2 0d
##&nbsp;$VAR1 = [];
##&nbsp;
sub portset_remove {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @Res;
    return \@Res;
}

1;
