#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode printflag command
##
## @description
##    This is a library to parse the raw outputs of 7Mode printflag command.
##
## @synopsis
##    use printflag;
##
##    printflag(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::printflag;

use strict;
use Tie::IxHash;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /(no such flag:.*)/) {
        $$error_message = $1;
        $error = "InvalidParamValue";
        return $error;
    } elsif ($raw_output =~ /(([Uu]sage:|$command(?:[^:]*):).*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
        return $error;
    }
    return undef;
}

## @name ::printflag
##
## @summary Parses the raw output from the "printflag" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from printflag with any of its options as described in the 
##    corresponding cdef file 'printflag.cdefs'
## @synopsis
##      printflag(%args);
## INPUTS TO METHOD CALL 'printflag' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'flags' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub printflag {
    my %args = @_;
    

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $can_throw = 1;
    
    my $error = _check_error(raw_output=>$output, 
                             command=>"printflag",
                             error_message=>\$error_message);

    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $can_throw = 0;
    }

    my $result;
    my $first_line = '\w+\s+=\s+-?\w+';
    my @lines = get_lines(raw_output=>$output, first_line=>$first_line, 
                    can_throw=>$can_throw, response_object=>$resp);

    my %hash;
    tie %hash, 'Tie::IxHash';
    foreach my $line(@lines) {
        if ($line =~ /^(\w+)\s+=\s+([^\s]+)$/) {
             $hash{$1} = $2;
        } else {
            $Log->comment("Skipping line: \"$line\"");
        }
        
    }
    
    push (@result_array, \%hash);
    
    if($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error->throw(text=>$error_message,response_object => $resp);
    }
    return \@result_array;
}

1;
