# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/priority.pm# $
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode priority commands
##
## @description
##    This is a library to parse the raw output of 7Mode priority commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "priority hybrid-cache show"
##    will be priority_hybrid_cache_show ()
##
## @synopsis
##    use priority;
## @author kathar.hidayath@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::priority;
use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::priority_hybrid_cache_show
##
## @summary Parses the raw output from the "priority hybrid-cache show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from priority_hybrid_cache_show with any of its options as
##    described in the corresponding cdef file 'priority.cdefs'
## @synopsis
##      priority_hybrid_cache_show(%args);
## INPUTS TO METHOD CALL 'priority_hybrid_cache_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; priority hybrid-cache show vol2; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'read_cache_policy' => 'all',
##&nbsp;            'write_cache_policy' => 'none',
##&nbsp;            'volume' => 'vol2',
##&nbsp;            'status' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub priority_hybrid_cache_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my @arr1;
    my %temp;
    my $line;
    my $can_throw;
    my $first_line;
    my $key;
    my $value;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error){
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($output =~ /Aggregate/){
    $first_line = "Aggregate";
    }elsif($output =~ /Volume/){
    $first_line = "Volume";
    }
    my @lines      = get_lines(
        'raw_output' => $output,'first_line'=>$first_line,'can_throw'=>1,
    );
    if ($first_line eq "Volume"){
        foreach $line(@lines){
            if($line =~ /(.*):\s+(\w+.*)/){
                $key = $1;
                $value = $2;
                $key =~ tr/A-Z/a-z/;
                $key =~ s/\s+/_/g;
                $value =~ tr/A-Z/a-z/;
                $value =~ s/\s+/_/g;
                $temp{$key} = $value;
            }
        }
        push(@return_array,\%temp);
    }elsif($first_line eq "Aggregate"){
        foreach my $line (@lines) {
			my %temp2;
            if ($line =~ /(\w+):(\w+)/ ) {
				$temp{ aggregate } = $2 ;
            }elsif($line =~/(\w+):\s+(\w+)/){
				$temp{ status } = $2 ;
            }elsif($line =~ /(-+\s+-+\s+-+)|(Read\s+policy\s+Write\s+policy\s+Volume)/){
				next;
            }elsif ($line =~ /(\w+[-|\w]*)\s+(\w+[-|\w]*)\s+(\w+)/ ) {
				$temp2{volume} = $3;
                $temp2{read_policy} = $1;
                $temp2{write_policy} = $2;
                push @arr1, \%temp2;
            }
		}
        $temp{volumes} = \@arr1;
        push @return_array ,\%temp ;
    }
    return \@return_array;
}    # end priority_hybrid_cache_show ;

## @name ::priority_hybrid_cache_default
##
## @summary Parses the raw output from the "priority hybrid-cache default" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from priority_hybrid_cache_default with any of its options as
##    described in the corresponding cdef file 'priority.cdefs'
## @synopsis
##      priority_hybrid_cache_default(%args);
## INPUTS TO METHOD CALL 'priority_hybrid_cache_default' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; priority hybrid-cache default; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'default_write_cache_policy' => 'random-write',
##&nbsp;            'default_read_cache_policy' => 'all'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; priority hybrid-cache default; result -l
##&nbsp;

sub priority_hybrid_cache_default {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my %temp;
    my $line;
    my $key;
    my $value;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines      = get_lines(
        'raw_output' => $output,
    );
    foreach $line(@lines){
        if($line =~ /(.*):\s+(.*)/){
            $key = $1;
            $value = $2;
            $key =~ tr/A-Z/a-z/;
            $key =~ s/\s+/_/g;
            $value =~ tr/A-Z/a-z/;
            $value =~ s/\s+/_/g;
            $temp{$key} = $value;
        }
    }
    push(@return_array,\%temp);
    return \@return_array;
} # end priority_hybrid_cache_default;

## @name ::priority_hybrid_cache_enable
##
## @summary Parses the raw output from the "priority hybrid-cache enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from priority_hybrid_cache_enable with any of its options
##    as described in the corresponding cdef file 'priority.cdefs'
## @synopsis
##      priority_hybrid_cache_enable(%args);
## INPUTS TO METHOD CALL 'priority_hybrid_cache_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; priority hybrid-cache enable hyA; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub priority_hybrid_cache_enable {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my %temp;
    my $line;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    # Now we can parse the output
    return [];
}  # end priority_hybrid_cache_enable

## @name ::priority_hybrid_cache_disable
##
## @summary Parses the raw output from the "priority hybrid-cache disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from priority_hybrid_cache_disable with any of its options as
##    described in the corresponding cdef file 'priority.cdefs'
## @synopsis
##      priority_hybrid_cache_disable(%args);
## INPUTS TO METHOD CALL 'priority_hybrid_cache_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; priority hybrid-cache disable hyA; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub priority_hybrid_cache_disable {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my %temp;
    my $line;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error){
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}  # end priority_hybrid_cache_disable;

## @name ::priority_hybrid_cache_set
##
## @summary Parses the raw output from the "priority hybrid-cache set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from priority_hybrid_cache_set with any of its options as
##    described in the corresponding cdef file 'priority.cdefs'
## @synopsis
##      priority_hybrid_cache_set(%args);
## INPUTS TO METHOD CALL 'priority_hybrid_cache_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'read-cache' </td> <td> 'read-cache' </td> </tr>
## <tr> <td> 'write-cache' </td> <td> 'write-cache' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priority_hybrid_cache_set hyA read-cache=all write-cache=none; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub priority_hybrid_cache_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @return_array;
    my %temp;
    my $line;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($output eq ""){
        return [];
    }
} # end priority_hybrid_cache_set;

sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error=undef;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }elsif($output =~ /(could not find.*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }elsif($output =~ /\w+\snot\sconfigured\son\volume\s(\w+)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Invalid\sparameters/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Missing\s+(\w+)\s+name/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Invalid\s+volume\s+name(.*)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Hybrid\s+cache\s+is\s+not\s+configured\s+on\s+(\w+)\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Invalid\s+aggregate\s+name\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /(\w+)\s+policy\s+options/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Invalid\s+value\s+for\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid parameter";
    }elsif($output =~ /Invalid\s+hybrid-cache\s+options\s+(.*)missing\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message =" invalid volume name parameter";
    }elsif($output =~ /Cannot\s+enable\s+hybrid.*purge_cache\s+is\s+active/){
        $error = "Response";
        $$error_message =" cannot enable hybrid cache when wafl hya "
                       . "purge_cache is active";
    }
    return $error;
}
1;
