#
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode priv commands
##
## @description
##    This is a library to parse the raw output of 7Mode priv commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "priv set" will be priv_set()
##
## @synopsis
##    use priv;
##
##    priv_set(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::priv;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::priv
##
## @summary Parses the raw output from the "priv" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from priv with any of its options as described in the 
##    corresponding cdef file 'priv.cdefs'
## @synopsis
##      priv(%args);
## INPUTS TO METHOD CALL 'priv' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub priv {
    my %args = @_;
    my $result;
    
    my $error_message;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    
    # First action is to remove command echo if any
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"priv",
                             "ptr_error_message"=>\$error_message);
    if($error)
    {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object=>$resp);
    }
    
    # Now we can parse the output
    if($raw_output =~ /(admin|advanced|diag|test)/) {
        $result = $1;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text =>"Output is not in the expected format",
            response_object => $resp);
    }
    return $result;  
}

## @name ::priv_set
##
## @summary Parses the raw output from the "priv set" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from priv_set with any of its options as described in the 
##    corresponding cdef file 'priv.cdefs'
## @synopsis
##      priv_set(%args);
## INPUTS TO METHOD CALL 'priv_set' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'level' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub priv_set {
    my %args         = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"priv",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
}

sub _check_error {
    my %args =@_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if($raw_output =~ /$command(?:[^:]*):(.*)/) {
        $$error_message = $1;
        if ($$error_message =~ /unrecognized privilege/) {
            $error = "InvalidParamValue";
        } elsif ($$error_message =~ /(too many parameters)/) {
            $error = "InvalidParam";
        } elsif ($$error_message =~ /(unrecognized subcommand)/) {
            $error = "CommandFailed";
        }
    } elsif($raw_output =~ /(Invalid option .+)/) {
        $error = "InvalidParam";
        $$error_message = $1;
    } else {            
        return undef;
    }
    $$error_message =~ s/\cM//g;
    return $error;
}
1;
