#$id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/ps.pm#6 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ps command
##
## @description
##    This is a library to parse the raw outputs of 7Mode ps command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
## @author govindaraj s (govinda1@netapp.com),dl-nacl-dev@netapp.com
#
package NACL::APISet::Node::CLI::7Mode::default::ps;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use vars qw(@ISA %EXPORT_TAGS @EXPORT @EXPORT_OK);
require Exporter;

## @name ::ps
##
## @summary Parses the raw output from the "ps" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ps with any of its options as described in the
##    corresponding cdef file 'ps.cdefs'
## @synopsis
##      ps(%args);
## INPUTS TO METHOD CALL 'ps' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-p' </td> <td> 'skip-threads-no-profileinfo' </td> </tr>
## <tr> <td> '-s' </td> <td> 'use-scheduletime' </td> </tr>
## <tr> <td> '-h' </td> <td> 'cutoff-equivalent' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cutoff' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long-listing' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pids' </td> </tr>
## <tr> <td> '-z' </td> <td> 'clear-time-proc' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ps -l -h
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_statistics_time' => '8822.777 seconds',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'sched_count' => '1063912',
##&nbsp;                           'percentage_stack_used' => '27%',
##&nbsp;                           'percentage_cpu_usage' => '104%',
##&nbsp;                           'name' => 'idle_thread0',
##&nbsp;                           'd' => 'i',
##&nbsp;                           'stack_used' => '1120',
##&nbsp;                           'state' => 'RR',
##&nbsp;                           'pid' => '0x8308ca14',
##&nbsp;                           'stack_base' => '0xb7402000',
##&nbsp;                           'slot_num' => '3',
##&nbsp;                           'sched_in_msecs' => '9139854',
##&nbsp;                           'stack_size' => '4092'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'sched_count' => '972426',
##&nbsp;                           'percentage_stack_used' => '27%',
##&nbsp;                           'percentage_cpu_usage' => '104%',
##&nbsp;                           'name' => 'idle_thread1',
##&nbsp;                           'd' => 'i',
##&nbsp;                           'stack_used' => '1120',
##&nbsp;                           'state' => 'RR',
##&nbsp;                           'pid' => '0x8308cd70',
##&nbsp;                           'stack_base' => '0xb7404000',
##&nbsp;                           'slot_num' => '4',
##&nbsp;                           'sched_in_msecs' => '9184656',
##&nbsp;                           'stack_size' => '4092'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -h
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_statistics_time' => '8822.900 seconds',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'domain' => 'i',
##&nbsp;                           'percentage_stack_used' => '27%',
##&nbsp;                           'percentage_cpu_usage' => '104%',
##&nbsp;                           'name' => 'idle_thread0',
##&nbsp;                           'id' => '3',
##&nbsp;                           'stack_used' => '1120',
##&nbsp;                           'state' => 'RR'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'domain' => 'i',
##&nbsp;                           'percentage_stack_used' => '27%',
##&nbsp;                           'percentage_cpu_usage' => '104%',
##&nbsp;                           'name' => 'idle_thread1',
##&nbsp;                           'id' => '4',
##&nbsp;                           'stack_used' => '1120',
##&nbsp;                           'state' => 'RR'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ps -s -c 1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process_statistics_time' => '8823.023 seconds',
##&nbsp;            'stats' => [
##&nbsp;                         {
##&nbsp;                           'run_max' => '90238',
##&nbsp;                           'name' => 'idle_thread0',
##&nbsp;                           'onq_ave' => '0',
##&nbsp;                           'd' => 'i',
##&nbsp;                           'run_ave' => '8590',
##&nbsp;                           'onq_max' => '0',
##&nbsp;                           'id' => '3',
##&nbsp;                           'run_count' => '1063941',
##&nbsp;                           'pri' => '100'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'run_max' => '410837',
##&nbsp;                           'name' => 'idle_thread1',
##&nbsp;                           'onq_ave' => '0',
##&nbsp;                           'd' => 'i',
##&nbsp;                           'run_ave' => '9445',
##&nbsp;                           'onq_max' => '0',
##&nbsp;                           'id' => '4',
##&nbsp;                           'run_count' => '972455',
##&nbsp;                           'pri' => '100'
##&nbsp;                         },
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ps {
    my %args = @_;

    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error("raw_output"=>$output,
                             "command"=>"ps",
                             "error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    # Now we can parse the output
    $output = _ps(command=>$command, raw_output=>$output,
                              response_object=> $resp);
    return $output;
  }

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ($raw_output =~ /(Bad\s+percentage\s+parameter)|(0:\s+slot\s+out\s+of\s+range)/) {
        $$error_message = $1;
        $error = "InvalidParamValue";
    } elsif ($raw_output =~ /([Uu]sage:)/ ) {
        $error = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
  }
sub _ps {
    my %args= @_;
    my $raw_output= $args{"raw_output"};
    my $command_line =$args{"command"};
    my $response =$args{"response_object"};
    my (@stats,@final,%process_values);
    my $can_throw =1;
    if ($raw_output =~ /Slot\s+PID\s+St\s+D\s+SchedMsecs\s+%CPU\s+SchedCount\s+StackBase\s+StackSize\s+StackUsed\s+%Stk\s+Name/) {
        my $first_line = 'Slot\s+PID\s+St\s+D\s+SchedMsecs\s+%CPU\s+SchedCount\s+StackBase\s+StackSize\s+StackUsed\s+%Stk\s+Name';
        my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
        shift(@arr); #Remove Heading line
        my @heading =("slot_num","pid","state","d","sched_in_msecs","percentage_cpu_usage","sched_count","stack_base","stack_size","stack_used","percentage_stack_used","name");
        my $i = 0;
        foreach (@arr){
            my %temp=();
            my @values = split(/\s+/,$arr[$i++]);
            my $index = 0;
            foreach (@values){
                $temp{$heading[$index]}=$_;
                $index++;
            }
            push(@stats, \%temp);
        }
        $process_values{stats}=\@stats;
        $raw_output =~ /Process statistics over (.* seconds).../;
        $process_values{process_statistics_time}=$1;
        push(@final, \%process_values);
        return(\@final);
    } elsif( $raw_output =~ /ID\s+State\s+Domain\s+%CPU\s+StackUsed\s+%StackUsed\s+Name/){
        my $first_line = 'ID\s+State\s+Domain\s+%CPU\s+StackUsed\s+%StackUsed\s+Name';
        my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
        shift(@arr); #Remove Heading line
        my @heading =("id","state","domain","percentage_cpu_usage","stack_used","percentage_stack_used","name");
        my $i = 0;
        foreach (@arr){
            my %temp=();
            my @values = split(/\s+/,$arr[$i++]);
            my $index = 0;
            foreach (@values){
                $temp{$heading[$index]}=$_;
                $index++;
            }
            push(@stats, \%temp);
        }
        $process_values{stats}=\@stats;
        $raw_output =~ /Process statistics over (.* seconds).../;
        $process_values{process_statistics_time}=$1;
        push(@final, \%process_values);
        return(\@final);
    } elsif( $raw_output =~ /ID\s+RunCount\s+RunMax\s+RunAve\s+OnQMax\s+OnQAve\s+Pri\s+D\s+Name/){
        my $first_line = 'ID\s+RunCount\s+RunMax\s+RunAve\s+OnQMax\s+OnQAve\s+Pri\s+D\s+Name';
        my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
        shift(@arr); #Remove Heading line
        my @heading =("id","run_count","run_max","run_ave","onq_max","onq_ave","pri","d","name");
        my $i = 0;
        foreach (@arr){
            my %temp=();
            my @values = split(/\s+/,$arr[$i++]);
            my $index = 0;
            foreach (@values){
                $temp{$heading[$index]}=$_;
                $index++;
            }
            push(@stats, \%temp);
        }
        $process_values{stats}=\@stats;
        $raw_output =~ /Process statistics over (.* seconds).../;
        $process_values{process_statistics_time}=$1;
        push(@final, \%process_values);
        return(\@final);
    }
    return(\@final);
}

1;
