#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode qtree commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode qtree commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "qtree create" will be qtree_create()
##
## @synopsis
##    use qtree;
##
##    qtree(%args);
##    qtree_create(%args);
##    qtree_delete(%args);
##    qtree_rename(%args);
##    qtree_security(%args);
##    qtree_stats(%args);
##    qtree_status(%args);
##    qtree_oplocks(%args);
##
## @author dl-nacl-dev@netapp.com, ksmahesh@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::qtree;

use Data::Dumper;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _qtree_help_command(%) {
    my %args=@_;
    my $command=$args{'command'};
    my $raw_output= $args{'raw_output'};
    my @words=split(/\s+/,$command);
    my $first_line="";
    my $can_throw=0;
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if (($words[$#words] =~/^-\?$/) ||($words[$#words] =~/^help$/)) {
        if((scalar(@lines)==6)&&($lines[0]=~/qtree/)) {
            return 1;
        }
    }
    return 0;
}
sub _parse_for_errors(%){
    my %args=@_;
    my $raw_output=$args{'raw_output'};
    my $command=$args{'command'};
    my $errormsg_ptr=$args{'errormsg_ptr'};
    $$errormsg_ptr=$raw_output;

    if($raw_output=~/qtree: invalid volume path /){
        return "InvalidParamValue";
    } elsif (($raw_output=~/qtree create: invalid qtree mode /)||($raw_output=~/ Volume  does not exist./)){
        return "InvalidParamValue";
    } elsif($raw_output=~/qtree create: Can't create qtree (.* exists)./){
        return "CommandFailed";
    }elsif(($raw_output=~/qtree: dst is not in src's volume/) || ($raw_output =~ /couldn't delete qtree/)){
        return "CommandFailed";
    } elsif($raw_output=~/qtree create: A qtree may only be created in a volume. Volume .* does not exist./){
        return "InvalidParamValue";
    } elsif ($raw_output=~/qtree create: missing qtree name./) {
        return "MissingRequiredParam";
    } elsif ($raw_output=~/qtree: must supply .* path/) {
        return "MissingRequiredParam";
    } elsif($raw_output=~/qtree: can't find path/){
        return "InvalidParamValue";
    } elsif ($raw_output=~/qtree create: No \'\/\' allowed in qtree name. Expected \/vol\/<vol>\/<qtree>./) {
        return "InvalidParamValue";
    }
    return undef;
}

sub _qtree_status(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my $line=shift(@lines);
    my @return_array;
    if($line=~/^qtree:(.*)$/){
        my %warning_hash;
        $warning_hash{'warning'}=$1;
        push(@return_array,\%warning_hash);
        $line=shift(@lines);
    }
    my @keys=split(/\s+/,$line,6);
    if (defined($keys[5])&& ($keys[5]!~ /^(Owning vfiler|ID)$/)) {
    	#both ID and owning vfiler is seen
    	splice(@keys,5,1,("ID","Owning vfiler"));
    }
    foreach my $key(@keys){
        $key=to_key_name('item'=>$key);
    }

    shift(@lines);
    foreach my $data_line (@lines){
        my %hash;
        my @fields=split(/\s+/,$data_line);
        # The only field which has been seen empty so far is qtree name
        #so insert null value in array fields where qtree name is expected
        # which is at the index 1
        if(scalar(@keys)- scalar(@fields)==1){
            my $nullelement="";
            splice(@fields,1,0,$nullelement);
        } elsif (scalar(@fields) > scalar(@keys)) {
        	@fields=();
        	my $i=scalar(@keys) - 2;
        	$data_line=~ s/\s*((\w+\s*){$i})$//;
        	my $temp = $1;
        	$data_line=~/(\w+)\s+(.*)/;
            push(@fields,$1,$2,split(/\s+/,$temp));
        }
        if(scalar(@keys)==scalar(@fields)){
            for(my $i=0;$i<scalar(@keys);$i++){
                $hash{$keys[$i]}=$fields[$i];
            }
            push(@return_array,\%hash);
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
    }
    return \@return_array;
}

sub _qtree_stats(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my @return_array;

    LOOP: while (scalar(@lines)){
        my $line=shift(@lines);
        if ($line=~/No qtrees are in use in Volume (.*)$/){
            my %hash;
            my $vol_name=$1;
            $hash{'volume'}=$1;
            $hash{'message'}="No qtrees are in use";
            push (@return_array,\%hash);
            next;
        }
        my @keys=split(/\t/,$line);
        foreach my $key(@keys){
            trim('string_ptr'=>\$key);
            $key=to_key_name('item'=>$key);
        }
        shift(@lines);
        $line=shift(@lines);
        while (($line!~/^Volume/)&&($line!~/No qtrees are in use in Volume/)){
            my %hash;
            my @values=split(/\t/,$line);
            foreach my $value(@values){
                trim('string_ptr'=>\$value);
            }
            if(scalar(@keys)==scalar(@values)){
                for(my $i=0;$i<scalar(@keys);$i++){
                    $hash{$keys[$i]}=$values[$i];
                }
                push (@return_array,\%hash);
            } else {
                my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(text=>"Unexpected form of output",response_object => $resp);
            }
            last LOOP if (scalar(@lines)==0);
            $line=shift(@lines);
        }
        unshift(@lines,$line);
    }
    return \@return_array;
}

## @name ::qtree_status
##
## @summary Parses the raw output from the "qtree status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_status with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_status(%args);
## INPUTS TO METHOD CALL 'qtree_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol' </td> </tr>
## <tr> <td> '-i' </td> <td> 'id' </td> </tr>
## <tr> <td> '-v' </td> <td> 'owning-vfiler' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_status(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw=0;
    my $error;
    my $errormsg;
    my $first_line='Volume\s+Tree';
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_qtree_status('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::qtree
##
## @summary Parses the raw output from the "qtree" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree(%args);
## INPUTS TO METHOD CALL 'qtree' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw=0;
    my $error;
    my $errormsg;
    my $first_line="qtree:";
    #handling the case of help seen only in 'qtree help'
    if(_qtree_help_command('raw_output'=>$raw_output,'command'=>$command)){
        my %hash;
        my @return_array;
        $hash{'help_output'}=help_output('raw_output'=>$raw_output);
        push(@return_array,\%hash);
        return \@return_array;
    }
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_qtree_status('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::qtree_stats
##
## @summary Parses the raw output from the "qtree stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_stats with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_stats(%args);
## INPUTS TO METHOD CALL 'qtree_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'id' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zero' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub qtree_stats(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw=0;
    my $error;
    my $errormsg;
    my $first_line='Volume\s+Tree\s+NFS ops\s+CIFS ops';
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_qtree_stats('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::qtree_rename
##
## @summary Parses the raw output from the "qtree rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_rename with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_rename(%args);
## INPUTS TO METHOD CALL 'qtree_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'dstpath' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'srcpath' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_rename(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,
    									'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if(scalar(@lines)==0){
       return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

## @name ::qtree_delete
##
## @summary Parses the raw output from the "qtree delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_delete with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_delete(%args);
## INPUTS TO METHOD CALL 'qtree_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_delete(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if(scalar(@lines)==0){
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

## @name ::qtree_create
##
## @summary Parses the raw output from the "qtree create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_create with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_create(%args);
## INPUTS TO METHOD CALL 'qtree_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_create(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if(scalar(@lines)==0){
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

## @name ::qtree_oplocks
##
## @summary Parses the raw output from the "qtree oplocks" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_oplocks with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_oplocks(%args);
## INPUTS TO METHOD CALL 'qtree_oplocks' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_oplocks(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if(scalar(@lines)==0){
        return [];
    }
    $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

## @name ::qtree_security
##
## @summary Parses the raw output from the "qtree security" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qtree_security with any of its options as described in the
##    corresponding cdef file 'qtree.cdefs'
## @synopsis
##      qtree_security(%args);
## INPUTS TO METHOD CALL 'qtree_security' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'style' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub qtree_security(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="qtree:";
    my $can_throw=0;
    my $error;
    my $errormsg;
    my $warning_string;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if(scalar(@lines)){
        foreach my $line(@lines){
            $warning_string.=" ".$line;
        }
    }else {
        undef $warning_string;
    }
    my %hash;
    my @return_array;
    $hash{'warning'}=$warning_string if (defined($warning_string));
    push(@return_array,\%hash);
    return \@return_array;
}

1;
