# $Id$
#
# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode quota commands
##
## @description
##     This is a library to parse the raw output of 7Mode quota commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use quota;
##
##    quota_status(%args);
##    quota_report(%args);
##    quota_on(%args);
##    quota_off(%args);
##    quota_err_inject(%args);
##    quota_resize(%args);
##    quota_allow(%args);
##    quota_disallow(%);
##    quota_logmsg_on(%);
##    quota_logmsg_off(%);
##    quota_scramble_conf(%);
##
## @author ksmahesh@netapp.com, Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::quota;

use strict;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors(%){
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{errormsg_ptr};
    my $error;

    if ($raw_output =~ /((unrecognized volume name.+)|(.+contains no entries.+)|(no limits allowed.+)|(.+already on)|(.+already off))/) {
        $$error_message = $1;
        $error = 'CommandFailed';
    } elsif ($raw_output =~ /usage:/) {
        $$error_message = $raw_output;
        $error = 'InvalidParamValue';
    } elsif ($raw_output =~ /quota:\s+unrecognized\s+volume\s+name.*/) {
        $$error_message = $raw_output;
        $error = 'InvalidParamValue';
	} elsif ($raw_output =~ /(quota:\s+quotas\s+are\s+already\s+enabled\s+on\s+volume.*)/) {
        $$error_message = $1;
        $error = 'CommandFailed';
    } elsif ($raw_output =~ /(quota:\s+quotas\s+are\s+already\s+disabled\s+on\s+volume.*)/) {
        $$error_message = $1;
        $error = 'CommandFailed';
    } elsif ($raw_output =~ /(quota:.*:\s+quotas\s+not\s+on.)/) {
        $$error_message = $1;
        $error = 'CommandFailed';
    } elsif ($raw_output =~ /(user name .+ not found|Unexpected end of file.+|no quota for file|can't stat file)/) {
        $$error_message = $1;
        $error = 'CommandFailed';
    } elsif ($raw_output =~ /(no available volumes)/) {
        $$error_message = $1;
        $error = 'MissingRequiredParam';
    } elsif ($raw_output =~ /(invalid quota type.+)/) {
        $$error_message = $1;
        $error = 'InvalidParamValue';
    } elsif ($raw_output =~ /usage: quota { /) {
        $$error_message = $raw_output;
        $error = 'InvalidParam';
    }elsif ($raw_output =~ /quota:\s+(.*:\s+quotas\s+not\s+on;\s+err\s+inject\s+not\s+done)/) {
        $$error_message = $1;
        $error = 'InvalidParamValue';
    }elsif ($raw_output =~ /quota:\s+(.*:\s+quotas\s+not\s+on;\s+resize\s+not\s+done)/) {
        $$error_message = $1;
        $error = 'InvalidParamValue';
    }
    return $error;
}

sub _quota_report {
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my @return_array;
    my $resp=$args{'response_obj'};
    my @keys;
    my $line;
    shift(@lines);
    push(@keys,"type","id","volume","tree","k_bytes_used","disk_limit");
    $line=shift(@lines);
    push(@keys,"s_disk_limit") if ($line =~ /S-Limit/ );
    push(@keys,"t_hold") if ($line =~ /T-hold/ );
    push(@keys,"files_used");
    push(@keys,"file_limit");
    push(@keys,"s_file_limit") if ($line =~ /S-Limit/ );
    push(@keys,"vfiler") if ($line =~ /Vfiler/ );
    push(@keys,"quota_specifier");
    my $regex = '(\w+)\s+(\S+)\s+(\w+)\s+(.*)\s+(\d+)\s+(-|\d+)';
    $regex .= '\s+(-|\d+)' if ($line =~ /S-Limit/ );
    $regex .= '\s+(-|\d+)' if ($line =~ /T-hold/ );
    $regex .= '\s+(\d+)'; #regex for files_used
    $regex .= '\s+(-|\d+)'; #regex for limit
    $regex .= '\s+(-|\d+)' if ($line =~ /S-Limit/ );
    $regex .= '\s+(\w+)' if ($line =~ /Vfiler/ );
    $regex .= '\s*(.*)$'; #regex for quota specifier
    shift(@lines);
    while ($line = shift (@lines)) {
        my %temp_hash;
        if ($line =~ /$regex/) {
            my @values;
            for(my $i=1; $i<14;$i++) {
                push (@values,eval "\$$i" )if defined (eval "\$$i");
            }
            #for tree .* is used in regex, so it will have space characters
            #appended so remove them after obtaining the values
            $values[3] =~ s/\s+$//;
            if (scalar(@keys) == scalar (@values)) {
                for(my $i=0; $i<scalar(@keys);$i++) {
                    $temp_hash{$keys[$i]}=$values[$i];
                }
            } else {
                my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(text=>"Unexpected form of output",response_object => $resp);
            }
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output. Parsing error at ".
                                "line: '$line' ",response_object => $resp);
        }
        push (@return_array,\%temp_hash);
    }
    return \@return_array;
}

sub _quota_on(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my %result;
    my $resp=$args{'response_obj'};

    foreach my $line (@lines) {
        if ($line =~ /(Starting quota initialization)/) {
            $result{status} = $1;
        } elsif ($line =~ /^\[quota: warning\]:\s*(.+)/) {
            push (@{$result{warnings}}, $1);
        }
    }
    unless (defined $result{status}) {
        # The expected output of 'Starting quota initialization' is not
        # observed.
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(text=>"The expected string of ".
            "'Starting quota initialization' is not observed",
            response_object => $resp);
    }
    return [\%result];
}

sub _quota_status(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my @return_array;
    my $resp=$args{'response_obj'};

    while (my $line = shift (@lines)) {
        if ($line =~ /(.*): quotas are (.*)./) {
            my %info_hash;
            $info_hash{'volume'} = $1;
            $info_hash{'status'} = $2;
            push (@return_array,\%info_hash);
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
    }
    return \@return_array;
}

## @name ::quota_status
##
## @summary Parses the raw output from the "quota status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_status with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_status(%args);
## INPUTS TO METHOD CALL 'quota_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub quota_status(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line='.+quotas are';
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_quota_status('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::quota_on
##
## @summary Parses the raw output from the "quota on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_on with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_on(%args);
## INPUTS TO METHOD CALL 'quota_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## <tr> <td> '-w' </td> <td> 'foreground' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub quota_on(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # Parse the output
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>'',
        'can_throw'=>1);
    my $output = _quota_on('lines'=>\@lines,'response_obj'=>$resp);
    return $output;
}

## @name ::quota_off
##
## @summary Parses the raw output from the "quota off" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_off with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_off(%args);
## INPUTS TO METHOD CALL 'quota_off' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub quota_off(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}

## @name ::quota_report
##
## @summary Parses the raw output from the "quota report" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_report with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_report(%args);
## INPUTS TO METHOD CALL 'quota_report' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-x' </td> <td> 'all-id-single-line' </td> </tr>
## <tr> <td> '-u' </td> <td> 'all-id-multiple-line' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vfiler' </td> </tr>
## <tr> <td> '-s' </td> <td> 'soft-limit' </td> </tr>
## <tr> <td> '-q' </td> <td> 'numeric-id' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-t' </td> <td> 'warning-threshold' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub quota_report(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line='K-Bytes';
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_quota_report('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::quota_err_inject
##
## @summary Parses the raw output from the "quota err_inject" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_err_inject with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_err_inject(%args);
## INPUTS TO METHOD CALL 'quota_err_inject' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; quota err_inject apisetvol; result -l; priv set -q diag
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub quota_err_inject(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}

## @name ::quota_resize
##
## @summary Parses the raw output from the "quota resize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_resize with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_resize(%args);
## INPUTS TO METHOD CALL 'quota_resize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; quota resize apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub quota_resize(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
## @name ::quota_allow
##
## @summary Parses the raw output from the "quota allow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_allow with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_allow(%args);
## INPUTS TO METHOD CALL 'quota_allow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; quota allow apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub quota_allow(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
## @name ::quota_disallow
##
## @summary Parses the raw output from the "quota disallow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_disallow with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_disallow(%args);
## INPUTS TO METHOD CALL 'quota_disallow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; quota disallow apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub quota_disallow(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
## @name ::quota_logmsg_on
##
## @summary Parses the raw output from the "quota logmsg on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_logmsg_on with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_logmsg_on(%args);
## INPUTS TO METHOD CALL 'quota_logmsg_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interval' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; quota logmsg on apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub quota_logmsg_on(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
## @name ::quota_logmsg_off
##
## @summary Parses the raw output from the "quota logmsg off" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_logmsg_off with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_logmsg_off(%args);
## INPUTS TO METHOD CALL 'quota_logmsg_off' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; quota logmsg off apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub quota_logmsg_off(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
## @name ::quota_scramble_conf
##
## @summary Parses the raw output from the "quota scramble_conf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from quota_scramble_conf with any of its options as described in the
##    corresponding cdef file 'quota.cdefs'
## @synopsis
##      quota_scramble_conf(%args);
## INPUTS TO METHOD CALL 'quota_scramble_conf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; quota scramble_conf apisetvol; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub quota_scramble_conf(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $errormsg;
    my $error = _parse_for_errors('raw_output'=>$raw_output,
        'errormsg_ptr'=>\$errormsg);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg, response_object => $resp);
    }
    # If there is no error, then we assume that the execution was successful.
    return [];
}
1;
