#$Id: $
#
#Copyright (c) 2010-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode raid_config command
##
## @description
##    This is a library to parse the raw outputs of any of the raid_config commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "raid_config" will be raid_config()
##
## @synopsis
##    use raid_config;
##
##    raid_config(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::raid_config;

use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /([Uu]sage:\s(.*))|(raid_config\s+\w+:\s+Mirror\s+Vote\s+Update\s+not\s+allowed)/i) {
        $error = "CommandFailed";
        $$error_message = $output;
        return $error;
    } elsif ($output =~ /\w+\s+not found/){
        $error = "InvalidParamValue";
        $$error_message = $output;
        return $error;
    } 
    $$error_message = $1;
    return $error;
}

## @name ::raid_config
##
## @summary Parses the raw output from the "raid_config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from raid_config with any of its options as described in the
##    corresponding cdef file 'raid_config.cdefs'
## @synopsis
##      raid_config(%args);
## INPUTS TO METHOD CALL 'raid_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'resource-manager-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> 'reconfig' </td> <td> 'reconfig' </td> <td> '0' </td> </tr>
## <tr> <td> 'info' </td> <td> 'info' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'checksum-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'plex-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'xor-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-xz' </td> <td> 'reset-xor-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'wafl-buffer-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggr-name' </td> <td> '0' </td> </tr>
## </table>
## Note:
## When the aggregate or volume name is specified in the value to the 'info'
## option, '-name' should not be used.
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; raid_config info listplex /aggr0/plex0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;id=63,parent_id=61,type=plex,tree=aggr0,fsm_state=NORMAL,raid_size=256000,raid_name=/aggr0/plex0,plexid=0,ctime=1331048567,online=1,resyncing=0,outofdate=0,resync_level=0,resync_progress=-1,plex_gen=992,plex_time=1331269283,cls_complete=1,cls_label=1,parent_state_resync=0,plex_rightsize=261248
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tree' => 'aggr0',
##&nbsp;            'raid_size' => '256000',
##&nbsp;            'raid_name' => '/aggr0/plex0',
##&nbsp;            'resync_level' => '0',
##&nbsp;            'parent_id' => '61',
##&nbsp;            'plexid' => '0',
##&nbsp;            'cls_complete' => '1',
##&nbsp;            'id' => '63',
##&nbsp;            'ctime' => '1331048567',
##&nbsp;            'outofdate' => '0',
##&nbsp;            'plex_time' => '1331269283',
##&nbsp;            'fsm_state' => 'NORMAL',
##&nbsp;            'cls_label' => '1',
##&nbsp;            'resyncing' => '0',
##&nbsp;            'online' => '1',
##&nbsp;            'plex_gen' => '992',
##&nbsp;            'parent_state_resync' => '0',
##&nbsp;            'plex_rightsize' => '261248',
##&nbsp;            'type' => 'plex',
##&nbsp;            'resync_progress' => '-1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub raid_config(%) {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(raw_output=>$output,error_message=>\$error_message);

    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    # these are the possible lines(parts) that can appear at the beginning of
    # the output
    my $first_line ='(maximum disks possible|alloc_bufs|Disk readerr raidarea|'.
                    'Scrub\/Reconstruct\/Resync|Scrub\/Verify|id=|version=|vendor=|'.
                    'Resource Pool Statistic:)';
    my @lines = get_lines(raw_output=>$output, first_line=>$first_line,
                          response_object=>$resp);

    if($command =~ /\s+-xz\s*/) {
        push (@result_array, {'result'=>'zeroed'});
    } elsif($command =~ /\s+-x\s*/) {
        # for -x option, special processing is needed as there are distinct sections
        # of output each having more than one line and having different key names
        my @strings = ('Scrub/Verify:','Recons/Resync:','Consistency Point:');
        my @sections = @{_get_sections(lines_ref=>\@lines,
                                       strings_ref=>\@strings)};

        foreach my $section(@sections) {
            # for each section, the first word is the value for the key 'name'
            my $name = shift(@{$section});
            my %hash;
            # get the rest of the keys
            %hash = _populate_hash(lines_ref=>$section);
            $hash{'name'} = to_key_name(item=>$name);
            push(@result_array, \%hash);
        }
    } elsif($command =~ /\s+-r\s*/) {
        my @strings = ('Resource Pool Statistic:','Buffers Statistic:',
                       'Lock Statistic', 'NVRAM Statistic');
        my @sections = @{_get_sections(lines_ref=>\@lines,
                                       strings_ref=>\@strings)};
        foreach my $section(@sections) {
            # for each section, the first word is the value for the key 'name'
            my $name = shift(@{$section});
            my %hash;
            $hash{'name'} = to_key_name(item=>$name);
            if($name =~ /Resource Pool Statistic/i) {
                my @beginnings = map('\['.sprintf("%2d",$_).'\]', (1..33));
                my @sections = @{_get_sections(lines_ref=>$section,
                                       strings_ref=>\@beginnings)};
                foreach my $section(@sections) {
                    my %internal_hash;
                    if($$section[0] =~ /\[\s?\d+\]\s([^\s]+)\s+:\s+\(Count:\s(\d+), Maxactive:\s(\d+), Memory:\s(\d+)\)/) {
                        %internal_hash = ('name'=>$1, 'count'=>$2,
                                             'maxactive'=>$3, 'memory'=>$4);
                    }
                    if($$section[1] =~ /Total:\salloc=(\d+)\s\/\sfree=(\d+)/) {
                        $internal_hash{'total_alloc'}=$1;
                        $internal_hash{'total_free'}=$2;
                    }
                    if($$section[2] =~ /Current:\salloc=(\d+)\s\/\sfree=(\d+),\shigh\swater\s=\s(\d+)/) {
                        $internal_hash{'current_alloc'}=$1;
                        $internal_hash{'current_free'}=$1;
                        $internal_hash{'high_water'}=$3;
                    }
                    push(@{$hash{'stats'}}, \%internal_hash);
                }
                (undef,$hash{'total_memory'}) = split(/=\s/,
                                                   ${$sections[$#sections]}[3]);
            } elsif($name =~ /Buffers Statistic/i) {
                foreach(@{$section}) {
                    %hash = (%hash, _populate_hash_line(line=>$_));
                }
            } elsif($name =~ /Lock Statistic/i) {
                foreach(@{$section}) {
                    if($_ =~ /(.+):\stotal\s=\s(\d+),\swaits\s=\s(\d+)/) {
                        my $name = to_key_name(item=>$1);
                        $hash{$name.'_total'} = $2;
                        $hash{$name.'_waits'} = $3;
                    }
                }
            } elsif($name =~ /NVRAM Statistic/i) {
                my @headings = map(to_key_name(item=>$_),split(/\s+/,
                                                             shift(@$section)));
                my $first_key = shift(@headings);
                my $i;
                foreach my $line (@$section) {
                    my %internal_hash;
                    if($line =~ s/(.+)\://) {
                        $internal_hash{$first_key} = $1;
                    }
                    my @values;
                    (undef, @values) = split(/\s\s+/, $line);
                    foreach(@headings) {
                        $internal_hash{$_} = $values[$i];
                        $i++;
                    }
                    $i = 0;
                    push(@{$hash{'stats'}}, \%internal_hash);
                }
            }
            push(@result_array, \%hash);
        }
    } elsif ($command =~ /\s+info\s+/) {
        # conditional processing for info option
        if ( $command =~ /raid_config info listplex|raid_config info listaggr/ ) {
            foreach my $line (@lines) {
                my %hash;
                my @array = split(",", $line);
                foreach my $words (@array){
                    $words =~ /(\S+)=(\S+)/;
                    $hash{$1}=$2;
                }
                push (@result_array,\%hash);
            }
        } else {
            my $can_push=1;

            # special processing needed for showfdr since one section has more than
            # one line of raw output, for all other cases it is one line per section
            my $flag = 1 if ($command =~ /showfdr/);

            my %full_hash;
            foreach my $line (@lines) {
               if($flag) {
               # checking whether it is the end of a section (only for showfdr)
               $can_push = ($line =~ /, device=/);
               }

               # hash formed from current line
               my %next_hash = _populate_hash_line(line=>$line);

               # hash obtained after appending current hash and
               # previous ones (if present)
               %full_hash = (%next_hash, %full_hash);

               if ($can_push) {
                   my %hash = %full_hash;
                   push(@result_array, \%hash);
                   %full_hash = ();
               }
               $can_push = 1;
            }
        }
    } else {
        # for all other cases, the result array has only one hash
        # containing all the values
        my %hash;
        %hash = _populate_hash(lines_ref=>\@lines);
        push (@result_array, \%hash);
    }
    return \@result_array;
}


sub _populate_hash_line {
    # returns a hash constructed from comma separated name-value pairs
    # present in a string

    my %args = @_;
    my $line = $args{'line'};
    my %hash;
    my @items = split(/\s*,\s*/,$line);
    my ($name,$value);
    foreach my $item (@items) {
        ($name,$value) = split(/\s?=\s?/, $item);
        $name = 'vendor' if($name =~ /vendor/);
        $name = to_key_name(item=>$name);
        $hash{$name} = $value;
    }
    return %hash;
}


sub _populate_hash {
    # returns a hash constructed from lines which are in a format where the
    # last word is the value and the preceeding words form the key name

    my %args = @_;
    my %hash ;
    my @lines = @{$args{'lines_ref'}};

    my ($key, $value, $key2);
    foreach my $line(@lines) {
        my @words = split(/\s+/,$line);
        unless ($key2) {
            $value = pop(@words);
            $key = join('_',@words);
            $key = to_key_name(item=>$key);
            $key =~ s/(_=|[\.\/])//g;
        } else {
            $key = $key2;
            $value = $line;
            $key2 = '';
        }
        if($key eq 'total_batch') {
            $key2 = 'total_batch_histogram';
            next;
        }
        $hash{$key} = $value;
    }
    return %hash;
}

sub _get_sections {
    # returns separate sections of output each having multiple lines based on
    # key words (used to perform the split)
    # each section begins with the line used to perform the split

    my %args = @_;
    my @lines = @{$args{'lines_ref'}};
    my @strings = @{$args{'strings_ref'}};

    my ($line, @sections, @part);

    while(@strings) {
        $line = $lines[0];
        if($line =~ /$strings[0]/) {
            my @push_this = @part;
            push (@sections, \@push_this);
            @part = ();
            shift(@strings);
        }
        push(@part, $line);
        shift(@lines);
    }
    # ---------for the last part---------------
    push(@part, @lines);
    push(@sections,\@part);
    # -----------------------------------------
    shift(@sections) if(!@{$sections[0]});
    return \@sections;
}

1;
