#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse rastrace command output
##
## @description
##    This is a library to parse the raw outputs of 7Mode rastrace commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rastrace list" will be rastrace_list()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::rastrace;
##
## rastrace_list(%args);
## rastrace_show(%args);
##
## @author abehera@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::rastrace;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(option requires.*)/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(Invalid option.*)/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(invalid module.*)/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return undef;
} ## end sub _check_error

## @name ::rastrace_list
##
## @summary Parses the raw output from the "rastrace_list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rastrace_list with any of its options as described in the
##    corresponding cdef file 'rastrace.cdefs'
## @synopsis
##      rastrace_list(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; rastrace list -m 0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    FCT_3a, id =  2, inst = 0,     ACTIVE
##&nbsp;    FCT_3b, id =  2, inst = 1,     ACTIVE
##&nbsp;      SCSI, id =  3, inst = 0,     ACTIVE
##&nbsp;     VDISK, id =  5, inst = 0,     ACTIVE
##&nbsp;     VDISK, id =  5, inst = 1,     ACTIVE
##&nbsp;     VDISK, id =  5, inst = 2,     ACTIVE
##&nbsp;     VDISK, id =  5, inst = 3,     ACTIVE
##&nbsp;     iSCSI, id =  6, inst = 0,     ACTIVE
##&nbsp;     iSCSI, id =  6, inst = 2,     ACTIVE
##&nbsp;FCT_EXT_3a, id = 13, inst = 0,     ACTIVE
##&nbsp;FCT_EXT_3b, id = 13, inst = 1,     ACTIVE
##&nbsp; ISPCNA_3a, id = 14, inst = 0,     ACTIVE
##&nbsp; ISPCNA_3b, id = 14, inst = 1,     ACTIVE
##&nbsp;      VDOM, id = 17, inst = 0,     ACTIVE
##&nbsp;      VDOM, id = 17, inst = 1,     ACTIVE
##&nbsp;       BDB, id = 19, inst = 0,     ACTIVE
##&nbsp; ICS_SCSIT, id = 20, inst = 0,     ACTIVE
##&nbsp; ICS_SCSIT, id = 20, inst = 1,     ACTIVE
##&nbsp; ICS_SCSIT, id = 20, inst = 2,     ACTIVE
##&nbsp; ICS_SCSIT, id = 20, inst = 3,     ACTIVE
##&nbsp;   ICS_BII, id = 21, inst = 0,     ACTIVE
##&nbsp;   ICS_BII, id = 21, inst = 1,     ACTIVE
##&nbsp;   ICS_BII, id = 21, inst = 2,     ACTIVE
##&nbsp;   ICS_BII, id = 21, inst = 3,     ACTIVE
##&nbsp;   ICS_BII, id = 21, inst = 3,     ACTIVE
##&nbsp;ICS_BCOMKA_LOW, id = 22, inst = 0,     ACTIVE
##&nbsp;ICS_BCOMKA_LOW, id = 22, inst = 1,     ACTIVE
##&nbsp;ICS_BCOMKA_LOW, id = 22, inst = 2,     ACTIVE
##&nbsp;ICS_BCOMKA_LOW, id = 22, inst = 3,     ACTIVE
##&nbsp;   ICS_ICS, id = 23, inst = 0,     ACTIVE
##&nbsp;   ICS_ICS, id = 23, inst = 1,     ACTIVE
##&nbsp;   ICS_ICS, id = 23, inst = 2,     ACTIVE
##&nbsp;   ICS_ICS, id = 23, inst = 3,     ACTIVE
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;
sub rastrace_list {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub rastrace_list

## @name ::rastrace_show
##
## @summary Parses the raw output from the "rastrace_show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rastrace_show with any of its options as described in the
##    corresponding cdef file 'rastrace.cdefs'
## @synopsis
##      rastrace_show(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; rastrace show -m 14 -i 1 -n 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;================================================================
##&nbsp;[Build: NetApp///x/eng/rlse70/DOT/nightly/RsierranevadaN_130411_2330/]
##&nbsp;ISPCNA_4b, 2 traces, starting at 245 (total 247), num_fills 0
##&nbsp;last_fill_time: 00:00:00.000.000
##&nbsp;min_fill_time:  00:00:00.000.000
##&nbsp;max_fill_time:  00:00:00.000.000
##&nbsp;avg_fill_time:  00:00:00.000.000
##&nbsp;================================================================
##&nbsp;TRACE #245 20130430_02:34:12.307.968PDT RAS_CAT:INFO RTT_COUNT:0
##&nbsp;QL8_SIGNAL_MPI_EVT_EXT
##&nbsp;QL8_SIGNAL_RECEIVED: MPI_EVT_EXT_SIGNAL received for card0 port1 init_state 0xe fw_state 0x8400
##&nbsp;---------------------------------------------------------------
##&nbsp;TRACE #246 20130430_02:34:12.308.003PDT RAS_CAT:INFO RTT_COUNT:0
##&nbsp;QL8_MPI_EVT_AEN_IDC_CMPLT
##&nbsp;QL8_MPI_EVT: Evt AEN IDC_CMPLT received card0 port1 mbox_out[1]=0x1200111 mbox_out[2]=0x80008 mbox_out[3]=0x0 mbox_out[4]=0x0
##&nbsp;---------------------------------------------------------------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;

sub rastrace_show {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub rastrace_show
1;
