#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routine to parse 7Mode rdate command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode rdate
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "rdate" will be rdate().
##
## @synopsis
##    use rdate;
##    rdate(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::rdate;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /cannot set date when NTP is running/si) {
        $error = "CommandFailed";
        $$error_message = "$&";
    } elsif($output =~ /no response.*/si) {
        $error = "CommandFailed";
        $$error_message = "$&";
    } elsif($output =~ /Connection refused/si) {
        $error = "CommandFailed";
        $$error_message = "$&";
    } elsif($output =~ /unknown host.*/si) {
        $error = "InvalidParamValue";
        $$error_message = "$&";
    } elsif($output =~ /usage:/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid Parameter Passed";
    }
    return $error;
}
## @name ::rdate
##
## @summary Parses the raw output from the "rdate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rdate with any of its options as described in the
##    corresponding cdef file 'rdate.cdefs'
## @synopsis
##      rdate(%args);
## INPUTS TO METHOD CALL 'rdate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_current_time' => '11:21:01 GMT',
##&nbsp;            'system_year' => '2011',
##&nbsp;            'system_day_date' => 'Wed Jun 15'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rdate {
    my %args = @_;
    my ($error_message, @result);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "rdate",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /(\w+\s+\w+\s+\d+)\s+(\d+.\d+.\d+\s+\w+)\s+(\d+)/){
        $result[0]->{'system_day_date'} = $1;
        $result[0]->{'system_current_time'} = $2;
        $result[0]->{'system_year'} = $3;
    }
    return \@result;
}
1;
