#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/reallocate.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routines to parse 7Mode reallocate commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode reallocate
##    commands. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "reallocate status" will be reallocate_status().
##
## @synopsis
##    use reallocate;
##    reallocate_status(%args);
##    reallocate_start(%args);
##    reallocate_measure(%args);
##    reallocate_on();
##    reallocate_off();
##    reallocate_stop(%args);
##    reallocate_restart(%args);
##    reallocate_schedule(%args);
##    reallocate_quiesce(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::reallocate;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif($output =~ /(No reallocation status was found for.*)/s) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($output =~ /(Unable to \w+ reallocation.*)/is) {
        $error = "CommandFailed";
        $$error_message = $&;
    } elsif($output =~ /(missing|invalid).*specification/is) {
        $error = "InvalidParamValue";
        $$error_message = $&;
    } elsif($output =~ /(invalid reallocate|invalid threshold).*/is) {
        $error = "InvalidParamValue";
        $$error_message = $&;
    } elsif($output =~ /.*mutually exclusive/is) {
        $error = "InvalidParam";
        $$error_message = $&;
    } elsif($output =~ /(.*scan already exists for ['].*[']),/is) {
        $error = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}
## @name ::reallocate_status
##
## @summary Parses the raw output from the "reallocate status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_status with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_status(%args);
## INPUTS TO METHOD CALL 'reallocate_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate status -v
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;                 'optimization' => '2',
##&nbsp;                 'threshold' => '4',
##&nbsp;                 'flags' => 'whole_vol,maybe_realloc,repeat',
##&nbsp;                 'state' => 'Idle'
##&nbsp;                 'next_run' => 'Fri May 13 09:17:40 GMT 2011',
##&nbsp;                 'interval' => '1 day',
##&nbsp;                 'path' => '/vol/vol0',
##&nbsp;                 'schedule' => 'n/a'
##&nbsp;                 'reallocation_scans' => 'on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_status {
    my %args = @_;
    my ($error_message, %hash, @result_array, $i, @temp);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "reallocate status",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my $firstline = 'Reallocation\s+scans\s+are\s+o\w\w?';
    my @lines = get_lines(raw_output => $output, first_line => $firstline,
                    can_throw => 1, response_object => $resp);
    $i = 0;
    if ($lines[0] =~ /Reallocation\s+scans\s+are\s+(o\w\w?)/) {
            $temp[$i] -> {'reallocation_scans'} = $1;
    }
    shift @lines;
    if($lines[0] =~ /No reallocation status/) {
            $temp[$i] -> {'state'} = $&;
    } else {
        if($lines[0] =~ m/((\/\w+){2,})[:]/) {
            $temp[$i] -> {'path'} = $1;
        } elsif  ($lines[0] =~ m/^(\w+)[:]$/) {
            $temp[$i] -> {'aggregate'} = $1;
        }
        shift @lines;
        foreach my $line (@lines) {
            if($line =~ m/((\/\w+){2,})[:]/) {
                $i++;
                $temp[$i] -> {'path'} = $1;
                $temp[$i] -> {'reallocation_scans'} = $temp[0] -> {'reallocation_scans'};
            } elsif  ($line =~ m/^(\w+)[:]$/) {
                $i++;
                $temp[$i] -> {'aggregate'} = $1;
                $temp[$i] -> {'reallocation_scans'} = $temp[0] -> {'reallocation_scans'};
            } elsif ($line =~ /State:\s+(\w+):?(.*)/i) {
                $temp[$i] -> {'state'} = $1;
                if($2) {
                    my $temp_data = $2;
                    #condition check for public inode 27074 of 32781, block 11985 of 17177
                    if($temp_data =~ /([a-zA-Z]+)?\s+[a-zA-Z]+\s+(\d+)\s+of\s+(\d+),\s+[a-zA-Z]+\s+(\d+)\s+of\s+(\d+)/) {
                        $temp[$i] -> {'scope'}         = $1 ;
                        $temp[$i] -> {'current_inode'} = $2 ;
                        $temp[$i] -> {'total_inode'}   = $3 ;
                        $temp[$i] -> {'current_block'} = $4 ;
                        $temp[$i] -> {'total_block'}   = $5 ;
                    #condition check for public inode 27074 of 32781, block 0
                    } elsif ($temp_data =~ /([a-zA-Z]+)?\s+[a-zA-Z]+\s+(\d+)\s+of\s+(\d+),\s+[a-zA-Z]+\s+(\d+)/) {
                        $temp[$i] -> {'scope' }        = $1 ;
                        $temp[$i] -> {'current_inode'} = $2 ;
                        $temp[$i] -> {'total_inode'}   = $3 ;
                        $temp[$i] -> {'current_block'} = $4 ;
                        $temp[$i] -> {'total_block'}   = $4 ;
                    #condition check for public inode 32781, block 11985 of 17177
                    } elsif ($temp_data =~ /([a-zA-Z]+)?\s+[a-zA-Z]+\s+(\d+),\s+[a-zA-Z]+\s+(\d+)\s+of\s+(\d+)/) {
                        $temp[$i] -> {'scope'  }       = $1 ;
                        $temp[$i] -> {'current_inode'} = $2 ;
                        $temp[$i] -> {'total_inode'}   = $2 ;
                        $temp[$i] -> {'current_block'} = $3 ;
                        $temp[$i] -> {'total_block'}   = $4 ;
                    #condition check for public inode 32781, block 0
                    } elsif ($temp_data =~ /([a-zA-Z]+)?\s+[a-zA-Z]+\s+(\d+),\s+[a-zA-Z]+\s+(\d+)/) {
                        $temp[$i] -> {'scope'}         = $1 ;
                        $temp[$i] -> {'current_inode'} = $2 ;
                        $temp[$i] -> {'total_inode'}   = $2 ;
                        $temp[$i] -> {'current_block'} = $3 ;
                        $temp[$i] -> {'total_block'}   = $3 ;
                    #condition check for Reallocating: block 239408 of 256000
                    } elsif ($temp_data =~ /[a-zA-Z]+\s+(\d+)\s+of\s+(\d+)/) {
                        $temp[$i] -> {'current_block'} = $1 ;
                        $temp[$i] -> {'total_block'}   = $2 ;
                    }
                }
            } elsif ($line =~ /Flags:\s+(.*)/i) {
                $temp[$i] -> { 'flags'} = $1 ;
            } elsif ($line =~ /Threshold:\s+(.*)/i) {
                $temp[$i] -> { 'threshold'} = $1 ;
            } elsif ($line =~ /Schedule:\s+(.*)/i) {
                $temp[$i] -> { 'schedule'} = $1 ;
            } elsif ($line =~ /Interval:\s+(.*)/i) {
                $temp[$i] -> { 'interval'} = $1 ;
            } elsif ($line =~ /NextRun:\s+(.*)/i) {
                $temp[$i] -> { 'next_run'} = $1 ;
            } elsif ($line =~ /optimization:\s+(.*)/i) {
                $temp[$i] -> { 'optimization'} = $1 ;
            } elsif ($line =~ /Measure Log:\s+(.*)/i) {
                $temp[$i] -> { 'measure_log'} = $1 ;
            }
        }
    }
    return \@temp;
}
## @name ::reallocate_on
##
## @summary Parses the raw output from the "reallocate on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_on with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_on(%args);
## INPUTS TO METHOD CALL 'reallocate_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate on
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation_scans' => 'on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_on {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate on",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation scans.*\s+(\w+)\.?$/) {
        $result[0]->{ 'reallocation_scans' } = $1;
    }
    return \@result;
}
## @name ::reallocate_off
##
## @summary Parses the raw output from the "reallocate off" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_off with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_off(%args);
## INPUTS TO METHOD CALL 'reallocate_off' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate off
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation_scans' => 'off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_off {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate off",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation scans.*\s+(\w+)\.?$/) {
        $result[0]->{ 'reallocation_scans' } = $1;
    }
    return \@result;
}
## @name ::reallocate_stop
##
## @summary Parses the raw output from the "reallocate stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_stop with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_stop(%args);
## INPUTS TO METHOD CALL 'reallocate_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate stop /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation' => 'stopped'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_stop {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate stop",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation will be stopped/s) {
        $result[0]->{ 'reallocation' } = 'stopped';
    }
    return \@result;
}
## @name ::reallocate_quiesce
##
## @summary Parses the raw output from the "reallocate quiesce" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_quiesce with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_quiesce(%args);
## INPUTS TO METHOD CALL 'reallocate_quiesce' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate quiesce /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation' => 'quiesced'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_quiesce {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate quiesce",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation will be quiesced/s) {
        $result[0]->{ 'reallocation' } = 'quiesced';
    }
    return \@result;
}
## @name ::reallocate_restart
##
## @summary Parses the raw output from the "reallocate restart" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_restart with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_restart(%args);
## INPUTS TO METHOD CALL 'reallocate_restart' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ignore-checkpoints' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate restart /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation' => 'restarted'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_restart {
    my %args = @_;
    my ($error_message, @result);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate restart",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation will be restarted/s) {
        $result[0]->{ 'reallocation' } = 'restarted';
    }
    return \@result;
}
## @name ::reallocate_schedule
##
## @summary Parses the raw output from the "reallocate schedule" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_schedule with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_schedule(%args);
## INPUTS TO METHOD CALL 'reallocate_schedule' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'delete-schedule' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-s' </td> <td> 'schedule' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate schedule -s "12 2 2 4" /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => '/vol/vol0',
##&nbsp;            'schedule' => '12 2 2 4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = reallocate schedule -s "1 2 3 4" aggr1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'aggregate' => 'aggr1',
##&nbsp;            'schedule' => '1 2 3 4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_schedule {
    my %args = @_;
    my ($error_message, @result, $temp);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                              command       => "reallocate schedule",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /Reallocation\s+for\s+'(.*)[']\s+.*for\s+'(.*)'/) {
        $result[0]->{'schedule'} = $2;
        $temp = $1;
        if($temp =~ m/((\/\w+){2,})/) {
            $result[0]->{'path'} = $temp;
        } else {
            $result[0]->{'aggregate'} = $temp;
        }
    } elsif($output =~ /['](.*)['].*removed.$/s) {
        $temp = $1;
        if($temp =~ m/((\/\w+){2,})/) {
            $result[0]->{'path'} = $temp;
        } else {
            $result[0]->{'aggregate'} = $temp;
        }
        $result[0]->{ 'schedule' } = 'removed';
    }
    return \@result;
}
## @name ::reallocate_start
##
## @summary Parses the raw output from the "reallocate start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_start with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_start(%args);
## INPUTS TO METHOD CALL 'reallocate_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o' </td> <td> 'once' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-check' </td> </tr>
## <tr> <td> '-t' </td> <td> 'threshold' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interval' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-p' </td> <td> 'physical-blocks-alone' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate-level' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = reallocate start /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation_scan' => 'started',
##&nbsp;            'path' => '/vol/vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_start {
    my %args = @_;
    my ($error_message, @result, $temp);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                               command       => "reallocate start",
                               error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /started on ['](.*)[']/is) {
        $temp = $1;
        if($temp =~ m/((\/\w+){2,})/) {
            $result[0]->{'path'} = $temp;
        } else {
            $result[0]->{'aggregate'} = $temp;
        }
        $result[0]->{ 'reallocation_scan' } = 'started';
    }
    return \@result;
}
## @name ::reallocate_measure
##
## @summary Parses the raw output from the "reallocate measure" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reallocate_measure with any of its options as described in the
##    corresponding cdef file 'reallocate.cdefs'
## @synopsis
##      reallocate_measure(%args);
## INPUTS TO METHOD CALL 'reallocate_measure' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o' </td> <td> 'once' </td> </tr>
## <tr> <td> '-t' </td> <td> 'threshold' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interval' </td> </tr>
## <tr> <td> '-l' </td> <td> 'logfile' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = reallocate measure -l logfile -t 4 /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'reallocation_scan' => 'started',
##&nbsp;            'path' => '/vol/vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub reallocate_measure {
    my %args = @_;
    my ( $error_message, @result );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error( raw_output    => $output,
                               command       => "reallocate measure",
                               error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /started on ['](.*)[']/is) {
        $result[0]->{'path'} = $1;
        $result[0]->{ 'reallocation_scan' } = 'started';
    }
    return \@result;
}
1;
