#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode registry commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode registry commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "registry get" will be registry_get()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::registry;
##
##    registry_get(%args);
##    registry_set(%args);
##    registry_delete(%args);
##    registry_walk(%args);
##    registry_commit(%args);
##    registry_deltree(%args);
##    registry_export(%args);
##    registry_import(%args);
##    registry_restore(%args);
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::7Mode::default::registry;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $error_message;
    my $error;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /(.+ does not exist)/ ) {
        $error = "InvalidParamValue";
        $error_message = $1;
    } elsif ( $raw_output =~ /usage:/ ) {
        $error = "MissingRequiredParams";
        $error_message = $raw_output;
    } elsif ( $raw_output =~
        /(An error occurred while trying to commit the registry)/ ) {
        $error = "CommandFailed";
        $error_message = $1;
    } elsif ( $raw_output =~ /Registry couldn't delete tree/) {
        $error = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
}

## @name ::registry_get
##
## @summary Parses the raw output from the "registry get" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_get with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_get(%args);
## INPUTS TO METHOD CALL 'registry_get' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'key' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; registry get options.rsh.enable; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;options.rsh.enable=on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'options.rsh.enable' => 'on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_get {
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my %result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "response_object"   => $response
    );
    # Now we can parse the output
    if ($raw_output =~ /^(.+)=/m) {
        my $key = $1;
        my $regex = qr($key=);
        $raw_output =~ s/$regex//;
        chomp($raw_output);
        $result{$key} = $raw_output;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $response);
    }
    return [\%result];
}

## @name ::registry_set
##
## @summary Parses the raw output from the "registry set" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_set with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_set(%args);
## INPUTS TO METHOD CALL 'registry_set' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'key' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; registry set abc.demonstrate.key 123; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub registry_set {
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "response_object"   => $response
    );

    # If there is no error, then it means that the command had
    # executed successfully
    return [];
}

## @name ::registry_delete
##
## @summary Parses the raw output from the "registry delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_delete with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_delete(%args);
## INPUTS TO METHOD CALL 'registry_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'key' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; registry delete abc.demonstrate.key; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;abc.demonstrate.key deleted.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'key' => 'abc.demonstrate.key'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_delete {
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "response_object"   => $response 
    );

    # Parse the output
    if ($raw_output =~ /(.+) deleted/) {
        return [{status=> $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $response);
    }
}

## @name ::registry_walk
##
## @summary Parses the raw output from the "registry walk" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_walk with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_walk(%args);
## INPUTS TO METHOD CALL 'registry_walk' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; registry walk etc.lang.codepage; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;etc.lang.codepage.C=ascii
##&nbsp;etc.lang.codepage.ar=cp1256
##&nbsp;etc.lang.codepage.cs=cp1250
##&nbsp;etc.lang.codepage.da=cp850
##&nbsp;etc.lang.codepage.de=cp850
##&nbsp;etc.lang.codepage.en=cp850
##&nbsp;etc.lang.codepage.en_US=cp437
##&nbsp;etc.lang.codepage.es=cp850
##&nbsp;etc.lang.codepage.fi=cp850
##&nbsp;etc.lang.codepage.fr=cp850
##&nbsp;etc.lang.codepage.he=cp862
##&nbsp;etc.lang.codepage.hr=cp1250
##&nbsp;etc.lang.codepage.hu=cp1250
##&nbsp;etc.lang.codepage.it=cp850
##&nbsp;etc.lang.codepage.ja=cp932
##&nbsp;etc.lang.codepage.ja_v1=cp932_v1
##&nbsp;etc.lang.codepage.ja_JP.PCK=cp932
##&nbsp;etc.lang.codepage.ja_JP.932=cp932
##&nbsp;etc.lang.codepage.ja_JP.PCK_v2=cp932_v1
##&nbsp;etc.lang.codepage.ko=cp949
##&nbsp;etc.lang.codepage.no=cp850
##&nbsp;etc.lang.codepage.nl=cp850
##&nbsp;etc.lang.codepage.pl=cp1250
##&nbsp;etc.lang.codepage.pt=cp860
##&nbsp;etc.lang.codepage.ro=cp1250
##&nbsp;etc.lang.codepage.ru=cp866
##&nbsp;etc.lang.codepage.sk=cp1250
##&nbsp;etc.lang.codepage.sl=cp852
##&nbsp;etc.lang.codepage.sv=cp850
##&nbsp;etc.lang.codepage.tr=cp857
##&nbsp;etc.lang.codepage.zh=cp936
##&nbsp;etc.lang.codepage.zh.GBK=cp936
##&nbsp;etc.lang.codepage.zh_TW=cp950
##&nbsp;etc.lang.codepage.zh_TW.BIG5=cp950
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'etc.lang.codepage.zh_TW' => 'cp950',
##&nbsp;            'etc.lang.codepage.sv' => 'cp850',
##&nbsp;            'etc.lang.codepage.ro' => 'cp1250',
##&nbsp;            'etc.lang.codepage.ja_JP.PCK_v2' => 'cp932_v1',
##&nbsp;            'etc.lang.codepage.es' => 'cp850',
##&nbsp;            'etc.lang.codepage.da' => 'cp850',
##&nbsp;            'etc.lang.codepage.ar' => 'cp1256',
##&nbsp;            'etc.lang.codepage.fi' => 'cp850',
##&nbsp;            'etc.lang.codepage.pl' => 'cp1250',
##&nbsp;            'etc.lang.codepage.it' => 'cp850',
##&nbsp;            'etc.lang.codepage.ja_JP.PCK' => 'cp932',
##&nbsp;            'etc.lang.codepage.zh' => 'cp936',
##&nbsp;            'etc.lang.codepage.pt' => 'cp860',
##&nbsp;            'etc.lang.codepage.zh_TW.BIG5' => 'cp950',
##&nbsp;            'etc.lang.codepage.no' => 'cp850',
##&nbsp;            'etc.lang.codepage.hu' => 'cp1250',
##&nbsp;            'etc.lang.codepage.en' => 'cp850',
##&nbsp;            'etc.lang.codepage.sk' => 'cp1250',
##&nbsp;            'etc.lang.codepage.he' => 'cp862',
##&nbsp;            'etc.lang.codepage.ru' => 'cp866',
##&nbsp;            'etc.lang.codepage.sl' => 'cp852',
##&nbsp;            'etc.lang.codepage.ja_v1' => 'cp932_v1',
##&nbsp;            'etc.lang.codepage.C' => 'ascii',
##&nbsp;            'etc.lang.codepage.zh.GBK' => 'cp936',
##&nbsp;            'etc.lang.codepage.ja_JP.932' => 'cp932',
##&nbsp;            'etc.lang.codepage.en_US' => 'cp437',
##&nbsp;            'etc.lang.codepage.ko' => 'cp949',
##&nbsp;            'etc.lang.codepage.fr' => 'cp850',
##&nbsp;            'etc.lang.codepage.nl' => 'cp850',
##&nbsp;            'etc.lang.codepage.hr' => 'cp1250',
##&nbsp;            'etc.lang.codepage.cs' => 'cp1250',
##&nbsp;            'etc.lang.codepage.de' => 'cp850',
##&nbsp;            'etc.lang.codepage.ja' => 'cp932',
##&nbsp;            'etc.lang.codepage.tr' => 'cp857'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_walk {
    my %args = @_;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my %result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "response_object"   => $response
    );

    # Now we can parse the output
    $raw_output      = snip_only_required_output(
        'raw_output' => $raw_output,
        'delimiter' => qr/^(.+)=/m
    );
    my @lines = split(/\r*\n/, $raw_output);

    return [{}] unless (scalar @lines);
    my $current_key;
    foreach my $line (@lines) {
        if ($line =~ /^(.+)=/) {
            $current_key = $1;
            $line =~ s/^(.+)=//;
            $result{$current_key} = $line;
        } else {
            $result{$current_key} .= "\n".$line;
        }
    }
    return [\%result];
}

## @name ::registry_commit
##
## @summary Parses the raw output from the "registry commit" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_commit with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_commit(%args);
## INPUTS TO METHOD CALL 'registry_commit' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; registry commit; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Registry has been committed.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Registry has been committed.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_commit {
    my %args = @_;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'registry commit',
                             "response_object"   => $resp,
                             );

    # Parse the output
    if ($raw_output =~ /(Registry has been committed.)/) {
        return [{status => $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

## @name ::registry_deltree
##
## @summary Parses the raw output from the "registry deltree" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_deltree with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_deltree(%args);
## INPUTS TO METHOD CALL 'registry_deltree' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; registry deltree asd; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Everything under asd is deleted.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Everything under asd is deleted.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_deltree {
    my %args = @_;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'registry deltree',
                             "response_object"   => $resp,
                             );

    # Parse the output
    if ($raw_output =~ /(Everything under .+ is deleted.)/) {
        return [{status => $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

## @name ::registry_export
##
## @summary Parses the raw output from the "registry export" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_export with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_export(%args);
## INPUTS TO METHOD CALL 'registry_export' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'password' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; registry export asd /etc/hosts; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Registry has been dumped to /etc/hosts.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Registry has been dumped to /etc/hosts.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_export {
    my %args = @_;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'registry export',
                             "response_object"   => $resp,
                             );

    # Parse the output
    if ($raw_output =~ /(Registry has been dumped to .+)/) {
        return [{status => $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

## @name ::registry_import
##
## @summary Parses the raw output from the "registry import" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_import with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_import(%args);
## INPUTS TO METHOD CALL 'registry_import' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'password' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; registry import /etc/hosts; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Registry has been imported from /etc/hosts.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Registry has been imported from /etc/hosts.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_import {
    my %args = @_;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'registry import',
			     "response_object"   => $resp, 
                             );

    # Parse the output
    if ($raw_output =~ /(Registry has been imported from .+)/) {
        return [{status => $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

## @name ::registry_restore
##
## @summary Parses the raw output from the "registry restore" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from registry_restore with any of its options as described in the 
##    corresponding cdef file 'registry.cdefs'
## @synopsis
##      registry_restore(%args);
## INPUTS TO METHOD CALL 'registry_restore' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'password' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; registry restore /etc/hosts; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Registry has been restored from /etc/hosts.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Registry has been restored from /etc/hosts.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub registry_restore {
    my %args = @_;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'registry restore',
			     "response_object"   => $resp,
                             );

    # Parse the output
    if ($raw_output =~ /(Registry has been restored from .+)/) {
        return [{status => $1}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

1;
