#$Id: //depot/prod/test/main/lib/NACL/APISet/Node/CLI/7Mode/default/restore.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode restore commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode restore commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @status public
##
## @author dl-dpg-nb-automation; dl-nacl-dev@netapp.com

package NACL::APISet::Node::CLI::7Mode::default::restore;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::restore
##
## @summary Parses the raw output from the "restore" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from restore with any of its options as described in the 
##    corresponding cdef file 'restore.cdefs'
## @synopsis
##      restore(%args);
## INPUTS TO METHOD CALL 'restore' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'block-size' </td> </tr> 
## <tr> <td> 'Q' </td> <td> 'ignore-qtree-information' </td> </tr> 
## <tr> <td> 'b' </td> <td> 'restore-block-size' </td> </tr> 
## <tr> <td> 's' </td> <td> 'restore-file-by-number' </td> </tr> 
## <tr> <td> 'D' </td> <td> 'restore-target' </td> </tr> 
## <tr> <td> 'f' </td> <td> 'next-restore-file' </td> </tr> 
## <tr> <td> 'x' </td> <td> 'extract-files' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'target-location' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tape-name' </td> </tr> 
## <tr> <td> 'T' </td> <td> 'list-qtree-root-files' </td> </tr> 
## <tr> <td> 'R' </td> <td> 'restart-restore-all' </td> </tr> 
## <tr> <td> 'r' </td> <td> 'restore-all' </td> </tr> 
## <tr> <td> 'F' </td> <td> 'force-continue' </td> </tr> 
## <tr> <td> 'y' </td> <td> 'confirm-prompt' </td> </tr> 
## <tr> <td> 't' </td> <td> 'list-backup-file' </td> </tr> 
## <tr> <td> 'A' </td> <td> 'ignore-acls' </td> </tr> 
## <tr> <td> 'N' </td> <td> 'verify-only' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub restore {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "restore",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

# regex for all output for restore command using case sensitive 'RESTORE:' as the marker
    my @restore_output = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => '(?m)^RESTORE:',
        "can_throw"       => 1,
        "response_object" => $response
    );

    my $return_hash;

    for ( my $index = 0; $index <= $#restore_output; $index++ ) {
        my ( $key, $value );

   # All regex comparisons are case-sesitive as 'RESTORE:' is the only pattern
        if ( $restore_output[$index]
            =~ /(?m)((^RESTORE: tape file.+device\s+)(.+)(\.))/ )
        {
            $key = 'tapefile';
        } elsif ( $restore_output[$index]
            =~ /(?m)((^RESTORE:\s+Dump\s+date:\s+)(.+))/ )
        {
            $key = 'dumpdate';
        } elsif ( $restore_output[$index]
            =~ /(?m)((^RESTORE:\s+)(Level\s+\d)(\s+dump of )(.+)( on )(.+))/ )
        {
            $key                 = 'dumptree';
            $value               = $5;
            $key                 = to_key_name( "item" => $key );
            $return_hash->{$key} = $value;

            $key                 = 'dumpsource';
            $value               = $7;
            $key                 = to_key_name( "item" => $key );
            $return_hash->{$key} = $value;

            $key = 'dumplevel';
        } elsif ( $restore_output[$index]
            =~ /(?m)((^RESTORE:\s+)(.+)(: Begin level 0 restore))/ )
        {
            $key = 'restoredate';
        } elsif ( $restore_output[$index]
            =~ /(?m)((^RESTORE: RESTORE IS\s+)(\w+))/ )
        {
            $key = 'status';
        }

        $value = $3;
        if ( defined($key) ) {
            $key = to_key_name( "item" => $key );
            $return_hash->{$key} = $value;
        }
    } ## end for ( my $index = 0; $index...

    return [$return_hash];
} ## end sub restore

# Private sub-routine to check for commonly encountered error in restore operations
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};

    # regular expression strings to match
    my $missing_param = qr{(?mx)((^usage:\s+restore)
                        | (^RESTORE:\s+Dump\s+number\s+ must\s+ be\s+ a\s+ positive\s+ integer)
                        | (^RESTORE:\s+Bad\s+ key\s+ character))};

    my $incorrect_param
        = qr{(?mx)((^RESTORE:\s+ Could\s+ not\s+ access\s+ volume\s+ in\s+ path)
             |(^RESTORE:\s+ Unexpected\s+ arguments)
             |(^RESTORE:\s+ Missing\s+ restore\s+ directory\s+ specifier)
             )};

    my $command_failed
        = qr{(?mx)((^RESTORE:\s+ Already\s+ at\s+ the\s+ end\s+ of\s+ tape)
                                | (^RESTORE:\s+ Could\s+ not\s+ initialize\s+ media))};
    my $error = undef;

    if ( $raw_output =~ $missing_param ) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ $incorrect_param ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output =~ $command_failed ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }

    return $error;
} ## end sub _check_error

1;
