# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/route.pm#  $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode route commands
##
## @description
##    This is a library to parse the raw output of 7Mode route commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "route add" will be root_add()
##
## @synopsis
##    use route;
##
##    route_add(%args);
##    route_delete(%args);
##    route_add_inet6(%args);
##    route_delete_inet6(%args);
##    route_show(%args);
##
## @author kathar.hidayath@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::route;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::route_add
##
## @summary Parses the raw output from the "route add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_add with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_add(%args);
## INPUTS TO METHOD CALL 'route_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> 'inet' </td> <td> 'inet' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'metric' </td> <td> '0' </td> </tr>
## <tr> <td> '&' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> 'default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## <tr> <td> '/' </td> <td> 'prefix' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; route add inet net 1.2.3.4/32 2.3.4.5 3; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------ $VAR1 = [
##&nbsp;  $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => '2.3.4.5',
##&nbsp;             'net' => '1.2.3.4/32'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub route_add(%){

    my %args=@_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @return_array;
    my %temp;
   $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
    }
    if($output =~ /add\s+(\w+)\s+default:\s+(\w+)\s+(.*)/){
        $temp{default_gateway} = $3;
    }elsif($output =~ /add\s+(\w+)\s+(.*):\s+gateway\s+(.*)/){
          $temp{$1} = $2;
        $temp{gateway} =$3;
    }
    push(@return_array,\%temp);
    return \@return_array;

} # end route_add subroutine.

## @name ::route_delete
##
## @summary Parses the raw output from the "route delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_delete with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_delete(%args);
## INPUTS TO METHOD CALL 'route_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> 'inet' </td> <td> 'inet' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'metric' </td> <td> '0' </td> </tr>
## <tr> <td> 'default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; route delete 1.2.3.4; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'host' => '1.2.3.4'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub route_delete(%){

    my %args=@_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @return_array;
    my %temp;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
    }
    if($output =~ /(delete\s+net\s+default)/){
        $temp{status} = $1;
    }elsif($output =~ /delete\s+(\w+)\s+(.*)/){
        $temp{$1} = $2;
    }
    push(@return_array,\%temp);
    return \@return_array;

} # end route_delete subroutine.

## @name ::route_show
##
## @summary Parses the raw output from the "route -s" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_show with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_show(%args);
## INPUTS TO METHOD CALL 'route_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'list-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'num-mode' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; route -s; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'internet' => [
##&nbsp;                            {
##&nbsp;                              'use' => '8161',
##&nbsp;                              'refs' => '6',
##&nbsp;                              'flags' => 'UGS',
##&nbsp;                              'gateway' => 'NBDC-EGL-4F-E17-DS',
##&nbsp;                              'destination' => 'default',
##&nbsp;                              'interface' => 'e0d',
##&nbsp;                              'pmtu' => '1500'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'use' => '0',
##&nbsp;                              'refs' => '0',
##&nbsp;                              'flags' => 'UGHS',
##&nbsp;                              'gateway' => '10.4.5.6',
##&nbsp;                              'destination' => '10.2.3.4',
##&nbsp;                              'interface' => 'e0d',
##&nbsp;                              'pmtu' => '1500'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'internet_v6' => [
##&nbsp;                               {
##&nbsp;                                 'use' => '4',
##&nbsp;                                 'refs' => '0',
##&nbsp;                                 'flags' => 'UHL',
##&nbsp;                                 'gateway' => 'e4:1f:13:4b:b6:46',
##&nbsp;                                 'destination' => 'fe80::%e0d',
##&nbsp;                                 'interface' => 'e0d',
##&nbsp;                                 'pmtu' => '1500'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'use' => '0',
##&nbsp;                                 'refs' => '0',
##&nbsp;                                 'flags' => 'UC',
##&nbsp;                                 'gateway' => 'link#4',
##&nbsp;                                 'destination' => 'fe80::%e0d',
##&nbsp;                                 'interface' => 'e0d',
##&nbsp;                                 'pmtu' => '1500'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub route_show(%){

    my %args=@_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @return_array;
    my %temp;
	my $routing_group;
    my $routing_group_id;
	my @arr1;
    my @arr2;
    my $type;
    my $flag;
    my @headings;
    my $route;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $output,
    );
    foreach my $line(@lines){
        my %temp2;
        my (@values, $i);
	if ( $line =~ /Routing group:\s+(.+)$/ ) {
		$routing_group = $1;
	}
	if ( $line =~ /Routing group-id:\s+(.+)$/ ) {
		$routing_group_id = $1;
	}
        if($line =~ /Internet v6:/){
            $flag = 1;
            next;
        }
        if($line =~ /Internet:/){
            $flag = 2;
            next;
        }elsif($line =~ /Destination\s+Gateway\s+Flags\s+Refs\s+Use\s+Interface\s+PMTU/){
            @headings = split(/\s+/,$line);
            @headings = map(to_key_name(item=>$_), @headings);
        }elsif($line =~     /(.*)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\w+)\s+(\w+)|(.*)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\w+)\s+=>\s+(\w+)/){
            my %hash;
            $line =~ s/=>//g;
            @values = split(/\s+/, $line);
            $i = 0;
	    foreach(@headings) {
		    $hash{$_} = $values[$i];
			    $i++;
	    }
	    if($routing_group)
	    {
		    $hash{'routing_group'}=$routing_group;
	    }
	    if($routing_group_id)
	    {
		    $hash{'routing_group_id'} = $routing_group_id;
	    }
	    if ($flag == 1){
		    push (@arr1, \%hash);
	    }elsif($flag ==2){
		    push (@arr2, \%hash);
	    }
	}
        if ($flag == 1){
            $temp { internet_v6 } = \@arr1 ;
        }elsif($flag == 2){
            $temp { internet } = \@arr2 ;
        }
    }
    push @return_array ,\%temp ;
	    return \@return_array;
} # end route_show subroutine.

## @name ::route_delete_inet6
##
## @summary Parses the raw output from the "route delete inet6" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_delete_inet6 with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_delete_inet6(%args);
## INPUTS TO METHOD CALL 'route_delete_inet6' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-type' </td> <td> '0' </td> </tr>
## <tr> <td> 'prefixlen' </td> <td> 'prefixlen' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'metric' </td> <td> '0' </td> </tr>
## <tr> <td> 'default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; route delete inet6 host ee80::21c:29ff:fe3d:3925; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'host' => 'ee80::21c:29ff:fe3d:3925'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub route_delete_inet6(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @return_array;
    my %temp;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
    }
    if($output =~ /(delete\s+net\s+default)/){
        $temp{status} = $1;
    }elsif($output =~ /delete\s+(\w+)\s+(.*)/){
        $temp{$1} = $2;
    }
    push(@return_array,\%temp);
    return \@return_array;

} # end route_delete_inet6 subroutine.

## @name ::route_add_inet6
##
## @summary Parses the raw output from the "route add inet6" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_add_inet6 with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_add_inet6(%args);
## INPUTS TO METHOD CALL 'route_add_inet6' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-type' </td> <td> '0' </td> </tr>
## <tr> <td> 'prefixlen' </td> <td> 'prefixlen' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'metric' </td> <td> '0' </td> </tr>
## <tr> <td> 'default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; route add inet6 host ee80::21c:29ff:fe3d:3925 fe80::1%lo 1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'gateway' => 'fe80::1%lo',
##&nbsp;             'host' => 'ee80::21c:29ff:fe3d:3925'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub route_add_inet6(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $error_message;
    my @return_array;
    my %temp;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
            $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
            $error->throw( text => $error_message, response_object => $resp );
    }
    if($output =~ /add\s+(\w+)\s+default:\s+(\w+)\s+(.*)/){
        $temp{default_gateway} = $3;
    }elsif($output =~ /add\s+(\w+)\s+(.*):\s+gateway\s+(.*)/){
          $temp{$1} = $2;
        $temp{gateway} =$3;
    }
    push(@return_array,\%temp);
    return \@return_array;

} # end route_add_inet6 subroutine.

sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error=undef;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }elsif($output =~ /(could not find.*)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /\s(\w+)\snot\sfound/) {
        $error = "InvalidParamValue";
        $$error_message = "not found";
    } elsif ($output =~ /add\s+(\w+)\s+(.*):\s+gateway\s+(.*):\s+(network\s+unreachable)/) {
        $error = "InvalidParamValue";
        $$error_message = $3;
    } elsif ($output =~ /add\s+(.*):\s+gateway\s+(.*):\s+(entry\s+already\s+exists)/) {
        $error = "InvalidParamValue";
        $$error_message = $3;
    } elsif ($output eq "") {
        $error = "InvalidParamValue";
        $$error_message = "undef";
    } elsif ($output =~ /route:\s+(Invalid\s+parameter)\s+(.*)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /route:\s+(Invalid\s+host)\/netmask\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /route:\s+(Can't\s+find\s+address)\s+for\s+host\s+(.*)/){
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~ /route:\s+(Can't\s+specify\s+netmask)\s+on\s+host\s+route:\s+(.*)/){
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif ($output =~  /delete\s+(\w+)\s+(.*):\s+(not\s+in\s+table)/){
        $error = "InvalidParamValue";
        $$error_message = $3;
    } elsif ($output =~ /(route\s+\w+:\s+expected\s+a\s+metric\s+number),\s+got\s+(.*)/){
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /(Domain\s+not\s+found):\s+bad\s+value/){
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /(No\s+address\s+associated\s+with\s+hostname):\s+bad\s+value/){
        $error = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
}
1;
