#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/routed.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode routed commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode routed commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "routed status" will be routed_status()
##
## @synopsis
##    use routed;
##
##    routed(%args);
##    
## @author govinda1@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::routed;

use strict;
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Include exceptions which will be thrown
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::routed
##
## @summary Parses the raw output from the "routed" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from routed with any of its options as described in the 
##    corresponding cdef file 'routed.cdefs'
## @synopsis
##      routed(%args);
## INPUTS TO METHOD CALL 'routed' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'status' </td> <td> 'display-status' </td> </tr> 
## <tr> <td> 'off' </td> <td> 'routed-off' </td> </tr> 
## <tr> <td> 'on' </td> <td> 'routed-on' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'status-numeric' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; routed; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; routed status; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_free_gateway_entries' => '0',
##&nbsp;            'time_last_heard' => 'Tue Jul 19 14:35:11 GMT 2011',
##&nbsp;            'gateway' => '10.72.88.1',
##&nbsp;            'metric' => '1',
##&nbsp;            'rip_snooping_status' => 'off',
##&nbsp;            'total_used_gateways' => '1',
##&nbsp;            'state' => 'ALIVE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub routed() {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my %temp;
    my @return_array;

    # Now check for any error in command execution
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "igroup create",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } elsif($raw_output eq "") {
         @return_array = ();
         return \@return_array;
    } elsif($raw_output =~ /RIP\s+snooping\s+is\s+\w+/) {
        my @lines = get_lines(raw_output  => $raw_output,
                          can_throw       => 1,
                          response_object => $resp);
        my @headings = ("gateway", "metric", "state", "time_last_heard");
        foreach my $line (@lines) {
           if($line =~ /RIP\s+snooping\s+is\s+(\w+)/){
              $temp{'rip_snooping_status'} = $1;
           } elsif ($line =~ /(\d+.\d+.\d+.\d+)\s+\t+(\w+)\t+(\w+)\t+(.*)/) {
              $temp{$headings[0]} = $1;
              $temp{$headings[1]} = $2;
              $temp{$headings[2]} = $3;
              $temp{$headings[3]} = $4;
           } elsif ($line =~ /(\d+)\s+free gateway entries,\s+(\d+) used/) {
              $temp{total_free_gateway_entries} = $1;
              $temp{total_used_gateways} = $2;
           }
        }
        push (@return_array, \%temp);
        return \@return_array;
    }
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
          text            => "Unexpected output",
          response_object => $resp
          );

 }

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error;
    if ( $raw_output =~ /[Uu]sage:/i){
        $error          = "InvalidParam";
        $$error_message = "command failed due to wrong dependicies usage";
    }
    return $error;
}
1;
