#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode rshkill commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode rshkill commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rshkill" will be rshkill()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::rshkill;
##
##    rshkill(%args);
##
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::rshkill;

use strict;
use Data::Dumper;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if ( $raw_output =~ /Usage/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    } elsif ($raw_output =~ /rsh session number can be determined by running 'rshstat -a' command.*/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /rshkill: Unable to close rsh session 1.*/s) {
        $error = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::rshkill
##
## @summary Parses the raw output from the "rshkill" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rshkill with any of its options as described in the 
##    corresponding cdef file 'rshkill.cdefs'
## @synopsis
##      rshkill(%args);
## INPUTS TO METHOD CALL 'rshkill' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'session-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rshkill 0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'rsh session 0 was closed successfully'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rshkill {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @return_array;

    my $error = _check_error("raw_output"=>$raw_output,
                            "command"=>"rshkill",
                            "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    my $value;
    my @output = get_lines(raw_output=>$raw_output, response_object=>$response);

    my %info = ();
    my $line;
    foreach $line (@output) {
        if ($line =~ /rshkill:\s+(.* successfully)./) {
            $info{"status"} = $1;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "unexpected format of output",
                    response_object => $response
                );
        }
    }
    push(@return_array,\%info);
    return \@return_array;
}
1;
