#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/rtag.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode rtag command
##
## @description
##    This is a library to parse the raw output of 7Mode rtag  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use rtag ;
##
##    rtag(%args);
##
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::rtag;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::rtag
##
## @summary Parses the raw output from the "rtag" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rtag with any of its options as described in the
##    corresponding cdef file 'rtag.cdefs'
## @synopsis
##      rtag(%args);
## INPUTS TO METHOD CALL 'rtag' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'mbuf' </td> <td> 'mbuf' </td> </tr>
## <tr> <td> 'sk' </td> <td> 'sk' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'module' </td> </tr>
## <tr> <td> '-l' </td> <td> 'lost' </td> </tr>
## <tr> <td> 'malloc' </td> <td> 'malloc' </td> </tr>
## <tr> <td> '-a' </td> <td> 'list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; rtag -a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'module_id' => 'ClCifs.i',
##&nbsp;            'type' => 'mbuf',
##&nbsp;            'description' => 'ClMode_CIFS_input'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_id' => 'ClCns.i',
##&nbsp;            'type' => 'mbuf',
##&nbsp;            'description' => 'ClMode_CIFS_NS_input'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_id' => 'ClCt.i',
##&nbsp;            'type' => 'mbuf',
##&nbsp;            'description' => 'ClMode_CT_input'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_id' => 'ClLoop.i',
##&nbsp;            'type' => 'mbuf',
##&nbsp;            'description' => 'ClMode_Loopback_input'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'module_id' => 'ClMount.i',
##&nbsp;            'type' => 'mbuf',
##&nbsp;            'description' => 'ClMode_Mount_input'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
sub rtag {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'rtag',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array;
    my @lines = get_lines( raw_output      => $raw_output,
                           can_throw       => 0,
                           response_object => $resp
                         );
    my $i = 0;
    my $j = 0;
    foreach my $line (@lines) {
        my ($value1,$value2,$value3,$value4,$value5,$value6);
        my $re_hex_word    = "[0-9A-Za-z\$\.\_\+\/\-]+";
        if($line =~ /^(\d+)\s+(\d+)\s+($re_hex_word\:\:$re_hex_word)$/) {
            $value1 = $1;
            $value2 = $2;
            $value3 = $3;
            $result_array[$i] -> {"allocs"} = $value1;
            $result_array[$i] -> {"bytes"}  = $value2;
            $result_array[$i] -> {"location"} = $value3;
            $i++;
        } elsif($line =~ /^(\w+(\.\w+)?)\s+(\w+)\s+([0-9a-zA-Z_\-\(\)\/\ ]+)\s+\|(\s+(\w+(\.\w+)?)\s+(\w+)\s+([0-9a-zA-Z\_\-\(\)\/\ ]+))?$/) {
            $value1 = $1;
            $value2 = $3;
            $value3 = $4;
            $value4 = $6;
            $value5 = $8;
            $value6 = $9;
            $value3 =~ s/\s+/_/g;
            $value6 =~ s/\s+/_/g;
            chop($value3);
            $result_array[$i] -> {"module_id"} = $value1;
            $result_array[$i] -> {"type"} = $value2;
            $result_array[$i] -> {"description"} = $value3;
            $i++;
            $result_array[$i] -> {"module_id"} = $value4 if $value4;
            $result_array[$i] -> {"type"} = $value5 if $value5;
            $result_array[$i] -> {"description"} = $value6 if $value6;
            $i++ if $value4;
        } elsif($line =~ /^Total:\s+(\d+)\s(\w+)\,\s+(\d+)\s+(\w+)\s+\((\d+)\s+(\w+)\s+(\w+)\)/) {
            $value1 = $1;
            $value2 = $2;
            $value3 = $3;
            $value4 = $4;
            $value5 = $5;
            $value6 = "$6"."_"."$7";
            $result_array[$i] -> {"total"} -> [$j] -> {"$value2"} = $value1;
            $result_array[$i] -> {"total"} -> [$j] -> {"$value4"} = $value3;
            $result_array[$i] -> {"total"} -> [$j] -> {"$value6"} = $value5;
        } elsif($line =~ /^(\d+)\s+(\w+\s\w+)\,\s(\d+)\s(\w+)/) {
            $value1 = $1;
            $value2 = $2;
            $value3 = $3;
            $value4 = $4;
            $value2 =~ s/\s+/_/g;
            $result_array[$i] -> {"total"} -> [$j] -> {"$value2"} = $value1;
            $result_array[$i] -> {"total"} -> [$j] -> {"$value4"} = $value3;
        } elsif($line =~ /^Total:\s+(\d+)\s(mbufs-in-use)$/) {
            $value1 = $1;
            $result_array[$i] -> {"total-mbufs-in-use"} = $value1;
        } elsif($line =~ /^([0-9a-zA-Z\_\-\(\)\/\.]+)\s+mbufs-in-use\:\s+(\d+)$/) {
            $value1 = $1;
            $value2 = $2;
            $result_array[$i] -> {"module_id"} = $value1;
            $result_array[$i] -> {"mbufs-in-use"} = $value2;
            $i++;
        } elsif($line =~ /([0-9a-zA-Z\.\_]+)\s+(\d+)\s+([0-9a-zA-Z\_\-\(\)\/ ]+)/) {
            $value1 = $1;
            $value2 = $2;
            $value3 = $3;
            $result_array[$i] -> {"module"} = $value1;
            $result_array[$i] -> {"mbufs-in-use"} = $value2;
            $result_array[$i] -> {"description"} = $value3;
            $i++;
        } elsif($line =~ /^None$/) {
            $result_array[$i] -> {"lost_rtags"} = "none";
        }
    }
    return \@result_array;
}
sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if($raw_output =~ /(unknown rtag|undefined type )/) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    } elsif($raw_output =~ /(No saved copy of memory statistics)/) {
        $$error_message = $1;
        $error = "CommandFailed";
    }
    return $error;
}
1;
