#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/sasadmin.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode sasadmin commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode sasadmin commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sasadmin channels" will be sasadmin_channels
##
## @synopsis
##    use sasadmin;
##    sasadmin_dev_stats(%args);
##    sasadmin_channels(%args);
##    sasadmin_dev_phy_reset(%args);
##    sasadmin_dev_phy_disable(%args);
##    sasadmin_adapter_state(%args);
##    sasadmin_expander_cli(%args);
##
## @author gurus@netapp.com,dl-nacl-dev,xdl-storage-automation,dl-storsub-qa
##
## @status To_be_reviewed

package NACL::APISet::Node::CLI::7Mode::default::sasadmin;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use strict;

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ($output =~
        /((No such adapter.*)|(Not a SAS adapter.*)|((?:\:\s*).+ is not valid or does not exist)|(Invalid option specified))/
      ) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($output =~ /[Uu]sage:/) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
        return $error;
    } elsif ($output =~ /sasadmin expander_cli:.*/) {
        $error          = "MissingRequiredParam";
        $$error_message = $output;
    }
    return $error;
} ## end sub _check_error
## @name ::sasadmin_channels
##
## @summary Parses the raw output from the "sasadmin channels" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasadmin_channels with any of its options as described in the 
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_channels(%args);
## INPUTS TO METHOD CALL 'sasadmin_channels' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sasadmin_channels {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line ='\w{16}\s+\w{2}\s+SAS\-port\s+\(GIHAID\:\s+\d+\)';
    my $error = _check_error("raw_output"        => $output,
                             "command"           => "sasadmin channels",
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # Now we can parse the output
	my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
			              can_throw       => 1,
                          response_object => $resp);

    foreach my $line (@lines) {

        my @words = split(/\s+/, $line);
        my %hash;
        $hash{wwn}          = $words[0];
        $hash{channel}      = $words[1];
        $hash{adapter_type} = $words[2];
        chop($words[4]);
        $hash{gihaid} = $words[4];
        push(@result_array, \%hash);

    } ## end foreach my $line (@lines)

    return \@result_array;
} ## end sub sasadmin_channels

## @name ::sasadmin_dev_stats
##
## @summary Parses the raw output from the "sasadmin dev_stats" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasadmin_dev_stats with any of its options as described in the 
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_dev_stats(%args);
## INPUTS TO METHOD CALL 'sasadmin_dev_stats' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sasadmin_dev_stats {
    my %args = @_;
    my ($error_message);
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = 'Device\s+stats\s+on\s+channel';
    my $error = _check_error(raw_output    => $output,
                             command       => "sasadmin dev_stats",
                             error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          can_throw       => 1,
                          response_object => $resp);

    my (@result_array, $adapter);
    foreach my $line (@lines) {
        if ($line =~ /Device\s+stats\s+on\s+channel\s+(\S*):/) {
            $adapter = $1;
            my %hash;
	   $hash{channel}=$adapter;
		    push(@result_array, \%hash);
        } elsif ($line =~ /(^$adapter\.\S+)\s+/) {

            my @words = split(/\s+/, $line);
            my %temp;

            $temp{device}             = $words[0];
            $temp{cmds_cmplt_good}    = $words[1];
            $temp{cmds_cmplt_error}   = $words[2];
            $temp{frames_in}          = $words[3];
            $temp{frames_out}         = $words[4];
            $temp{vctm_abrt}          = $words[5];
            $temp{qsce_cnt}           = $words[6];
            $temp{udrn_cnt}           = $words[7];
            $temp{ovrn_cnt}           = $words[8];
            $temp{trns_err_cnt}       = $words[9];
            $temp{scsi_ptcl_err_cnt}  = $words[10];
            $temp{crc_err_cnt}        = $words[11];
            $temp{open_err_cnt}       = $words[12];
            $temp{invd_cnt}           = $words[13];
            $temp{dlay_cnt}           = $words[14];
            $temp{sata_init_fail_cnt} = $words[15];
            $temp{to_1_ct}            = $words[16];
            $temp{to_2_ct}            = $words[17];
            $temp{to_3_ct}            = $words[18];
            $temp{to_4_ct}            = $words[19];

            push(@{$result_array[$#result_array]->{stats}},\%temp);
        } ## end elsif ($line =~ /(^$adapter\.\S+)\s+/)

    } ## end foreach my $line (@lines)

#    push(@result_array, \%hash);

    return \@result_array;
} ## end sub sasadmin_dev_stats

## @name ::sasadmin dev_phy_reset
##
## @summary Parses the raw output from "sasadmin dev_phy_reset" command
##
## @description
##    This method can be used for parsing the output from "sasadmin dev_phy_reset" command
##    called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/0.0_20091111/sasadmin.cdefs
##
## @synopsis
##    sasadmin_dev_phy_reset(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an empty array ref on success.
## @name ::sasadmin_dev_phy_reset
##
## @summary Parses the raw output from the "sasadmin dev_phy_reset" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasadmin_dev_phy_reset with any of its options as described in the 
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_dev_phy_reset(%args);
## INPUTS TO METHOD CALL 'sasadmin_dev_phy_reset' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sasadmin_dev_phy_reset {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $result;
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "sasadmin dev_phy_reset",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub sasadmin_dev_phy_reset
## @name ::sasadmin sasadmin dev_phy_disable
##
## @summary Parses the raw output from "sasadmin dev_phy_disable" command
##
## @description
##    This method can be used for parsing the output from "sasadmin dev_phy_disable" command
##    called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/0.0_20091111/sasadmin.cdefs
##
## @synopsis
##    sasadmin_dev_phy_disable(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an empty array ref on success.
## @name ::sasadmin_dev_phy_disable
##
## @summary Parses the raw output from the "sasadmin dev_phy_disable" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasadmin_dev_phy_disable with any of its options as described in the 
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_dev_phy_disable(%args);
## INPUTS TO METHOD CALL 'sasadmin_dev_phy_disable' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sasadmin_dev_phy_disable {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $result;
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "sasadmin dev_phy_disable",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub sasadmin_dev_phy_disable
## @name ::sasadmin sasadmin adapter_state
##
## @summary Parses the raw output from "sasadmin adapter_state" command
##
## @description
##    This method can be used for parsing the output from "sasadmin_adapter_state " command
##    called with any of its options as described in the cdef file at
##    lib/NACL/APISet/Node/CLI/7Mode/0.0_20091111/sasadmin.cdefs
##
## @synopsis
##    sasadmin_adapter_state(%args);
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and the full command sent to the filer/vsim.
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array ref which contains hash-refs where the keys are the
##    heading labels appearing at the top of the output and the value of each
##    key is the value that appears in the column corresponding to that key
##    for any given row.
## @name ::sasadmin_adapter_state
##
## @summary Parses the raw output from the "sasadmin adapter_state" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasadmin_adapter_state with any of its options as described in the 
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_adapter_state(%args);
## INPUTS TO METHOD CALL 'sasadmin_adapter_state' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sasadmin_adapter_state {
    my %args = @_;
    my ($error_message);
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = 'State\s+for\s+adapter\s+channel';
    my $error = _check_error(raw_output    => $output,
                             command       => "sasadmin adapter_state",
                             error_message => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @lines = get_lines(raw_output      => $output,
                          first_line      => $first_line,
                          can_throw       => 1,
                          response_object => $resp);

    #In the actual hash for all the keys spaces will be replaced with '-'
    my @possible_keys =
      ("Frames In", "Frames Out", "Commands Completed successfully", "Commands Completed in error");
    my @result_array = ();

    foreach my $line (@lines) {

        my @words = split(/:\s+/, $line);
        my @temp = map($words[0] =~ /^\s*$_\s*/ ? $_ : (), @possible_keys);

        if ($line =~ /State\s+for\s+adapter\s+channel\s+(\S+):\s+(\S+)/) {

            #start a new hash for the new shelf
            my (%temp_hash, @temp_array);
            push(@result_array, \%temp_hash);
            chomp($words[1]);
            $result_array[$#result_array]->{channel}       = $1;
            $result_array[$#result_array]->{state}         = $2;
            $result_array[$#result_array]->{other_details} = \@temp_array;
        } elsif (@temp) {
            $temp[0] =~ s/(\s+)/_/g;
            my $key = lc($temp[0]);
            chomp($words[1]);
            $result_array[$#result_array]->{$key} = $words[1];
        } elsif ($line =~ /^\s*(\d+)\s+(\S+\s+Gb\/s)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s*$/) {

            #Store the other details in a hash and push this hash in to an array, reference of this array must be stored as
            #the values to the key 'Other_details'.
            my %temp_hash;
            $temp_hash{phy}                = $1;
            $temp_hash{state}              = $2;
            $temp_hash{Invald_dword_count} = $3;
            $temp_hash{disprt_error_count} = $4;
            $temp_hash{loss_sync_count}    = $5;
            $temp_hash{phy_reset_prob}     = $6;
            $temp_hash{phy_change_count}   = $7;
            push(@{$result_array[$#result_array]->{other_details}}, \%temp_hash);
        } ## end elsif ($line =~ ...
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub sasadmin_adapter_state

## @name ::sasadmin_expander_cli
##
## @summary Parses the raw output from the "sasadmin expander_cli" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sasadmin_expander_cli with any of its options as described in the
##    corresponding cdef file 'sasadmin.cdefs'
## @synopsis
##      sasadmin_expander_cli(%args);
## INPUTS TO METHOD CALL 'sasadmin_expander_cli' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'shelf-module-id' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; sasadmin expander_cli 4a.shelf3.A "ver"; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Canister firmware      : 1.11
##&nbsp;Canister firmware date : Jan 12 2011 15:14:22
##&nbsp;Canister bootloader    : 6.33
##&nbsp;Canister config CRC    : Not present
##&nbsp;Canister VPD structure : 0x09
##&nbsp;Canister VPD CRC       : 0xA277A1C3
##&nbsp;Canister CPLD          : 0x23
##&nbsp;Canister chip          : 0x80050002
##&nbsp;Canister SDK           : 3.06.01+ (B028)
##&nbsp;Midplane VPD structure : 0x0A
##&nbsp;Midplane VPD CRC       : 0x224D8D87
##&nbsp;Midplane CPLD          : 0x0F
##&nbsp;PCM 1 firmware         : 2.0E
##&nbsp;PCM 2 firmware         : 2.0A
##&nbsp;PCM 1 VPD structure    : 0x03
##&nbsp;PCM 2 VPD structure    : 0x03
##&nbsp;PCM 1 VPD CRC          : 0x40E06E13
##&nbsp;PCM 2 VPD CRC          : 0x40E06E13
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];
##&nbsp;

sub sasadmin_expander_cli {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my @result_array;
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "sasadmin expander_cli",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub sasadmin_expander_cli

1;
