#$Id$
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode savecore  commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode savecore  commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::savecore;
##
##
## @author  gupendra@netapp.com, dl-nacl-dev@netapp.com
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::savecore;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

## @name ::savecore
##
## @summary Parses the raw output from the "savecore" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from savecore with any of its options as described in the 
##    corresponding cdef file 'savecore.cdefs'
## @synopsis
##      savecore(%args);
## INPUTS TO METHOD CALL 'savecore' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'display-info' </td> <td> '1' </td> </tr> 
## <tr> <td> '-k' </td> <td> 'invalidate' </td> <td> '1' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'synchronous' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'core-id' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'display-progress' </td> <td> '1' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; savecore -l; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'os_version' => '/x/eng/nrlse67/DOT/nightly/RrollingrockN_120112_2045/: Thu Jan 12 21:24:37 PST 2012',
##&nbsp;            'panic_string' => 'reboot -d panic in SK process idle_thread0 on release RrollingrockN_120112_2045',
##&nbsp;            'core_name' => 'core.118042691.2012-02-19.18_45_09.nz',
##&nbsp;            'panic_time' => ' Sun Feb 19 13:45:09 EST 2012'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'os_version' => '/x/eng/nrlse67/DOT/nightly/RrollingrockN_120112_2045/: Thu Jan 12 21:24:37 PST 2012',
##&nbsp;            'panic_string' => 'reboot -d panic in SK process idle_thread0 on release RrollingrockN_120112_2045',
##&nbsp;            'core_name' => 'core.118042691.2012-02-19.20_13_14.nz',
##&nbsp;            'panic_time' => ' Sun Feb 19 15:13:14 EST 2012'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'os_version' => '/x/eng/nrlse67/DOT/nightly/RrollingrockN_120112_2045/: Thu Jan 12 21:24:37 PST 2012',
##&nbsp;            'panic_string' => 'reboot -d panic in SK process idle_thread0 on release RrollingrockN_120112_2045',
##&nbsp;            'core_name' => 'core.118042691.2012-02-19.21_19_32.nz',
##&nbsp;            'panic_time' => ' Sun Feb 19 16:19:32 EST 2012'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = priv set -q diag; savecore -s; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'total_blocks' => '2829492',
##&nbsp;             'saved_blocks' => '2198066',
##&nbsp;             'saved_percent' => '77',
##&nbsp;             'core_name' => 'core.118042691.2012-02-19.21_19_32.nz',
##&nbsp;             'state' => 'Saving'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;
##&nbsp;Command = priv set -q test; savecore -i; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;7874160 blocks are available on 28 disks that support sprayed dumpcore.
##&nbsp;The sprayed dumpcore feature will be used.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'type' => 'sprayed',
##&nbsp;            'spray_dumpcore_disks' => '28',
##&nbsp;            'spray_dumpcore_blocks' => '7874160'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub savecore {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $raw_output,
            response_object => $resp,
        );
    }
    my $option = $command;
    my ( @Res, $value );
    my $i = -1;
    if ( $option =~ /-l\s?/ ) {
        my @lines =
          get_lines( "raw_output" => $raw_output, "response_object" => $resp );
        foreach $value (@lines) {
            if ( $value =~ /^(core.*.nz)/ ) {
                $i++;
                $Res[$i]->{"core_name"} = $1;
            }
            elsif ( $value =~ /^(Panic\s+string)\:\s*(\w+.*)$/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ /^(Panic\s+time)\:\s*(.*)$/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
            elsif ( $value =~ /^(OS\s+version)\:\s*(.*)$/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }    ##if ( $option =~ /-l\s?//)
    elsif ( $option =~ /-s\s?/ ) {
        my @lines =
          get_lines( "raw_output" => $raw_output, "response_object" => $resp );
        my $j = 0;
        foreach $value (@lines) {
            if ( $value =~
                /^(Saving)\s+(core.*.nz)\:\s+(\d+)\%\s+\((\d+)\s+of\s+(\d+).*/ )
            {
                $Res[$j]->{"state"}         = $1;
                $Res[$j]->{"core_name"}     = $2;
                $Res[$j]->{"saved_percent"} = $3;
                $Res[$j]->{"saved_blocks"}  = $4;
                $Res[$j]->{"total_blocks"}  = $5;
                $j++;
            }

        }
    }
    elsif ( $option =~ /-i\s?/ ) {
        my @lines =
          get_lines( "raw_output" => $raw_output, "response_object" => $resp );
        my %hash;
        foreach $value (@lines) {
            if ( $value =~
                /The (.*) dumpcore feature will be used./ )
            {
                $hash{type} = $1;
            }
            if ( $value =~
                /(\d+) blocks are available on (\d+) disks that support sprayed dumpcore./ )
            {
                $hash{spray_dumpcore_blocks} = $1;
                $hash{spray_dumpcore_disks}  = $2;
            }
            if( $value =~ /^\s*(\d+) unsaved core is occupying (\d+) disk./) {
                $hash{unsaved_core}             = $1;
                $hash{unsavedcore_disk_count}   = $2;
            }
        }
        push (@Res , \%hash);
    }
    return \@Res;
}

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{response_object};
    my $error_message;
    my $error;
    if ( $output =~ //si ) {
        $error         = "InvalidParam";
        $error_message = "Invalid options are used in the command";
    }
    elsif ( $output =~ /^savecore:\s+Invalid.*/si ) {
        $error         = "InvalidParamValue";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return $error;
} ## end sub _check_error
1;

