# $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI/7Mode/default/scramble.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode scramble command
##
## @description
##    This is a library to parse the raw output of 7Mode scramble command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "scramble path" will be scramble_path()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::scramble;
##
##    scramble_path(%args);
##
## @author Veerabhadraswamy.P@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::scramble;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::scramble_path
##
## @summary Parses the raw output from the "scramble path" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from scramble_path with any of its options as described in the
##    corresponding cdef file 'scramble.cdefs'
## @synopsis
##      scramble_path(%args);
## INPUTS TO METHOD CALL 'scramble_path' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'blocks' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'ext_cache' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'start' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; scramble path /vol/compo_flex_vol/test_file; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 2 blocks at level 1 starting at fbn 0 in inode 96 in snapid 0 vol compo_flex_vol with: use_pattern=0 pattern=0x0 verbose=0
##&nbsp;reallocated 2 blocks for inode 96 in flex vol compo_flex_vol
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'reallocating' => '2',
##&nbsp;            'inode' => '96',
##&nbsp;            'snapid' => '0',
##&nbsp;            'started_fbn' => '0',
##&nbsp;            'level' => '1',
##&nbsp;            'use_pattern' => '0',
##&nbsp;            'pattern' => '0x0',
##&nbsp;            'reallocated_blocks' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; scramble path -v /vol/compo_flex_vol/test_file; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 2 blocks at level 1 starting at fbn 0 in inode 96 in snapid 0 vol compo_flex_vol with: use_pattern=0 pattern=0x0 verbose=1
##&nbsp;reallocated 0 blocks for inode 96 in flex vol compo_flex_vol
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'reallocating' => '2',
##&nbsp;            'inode' => '96',
##&nbsp;            'snapid' => '0',
##&nbsp;            'started_fbn' => '0',
##&nbsp;            'level' => '1',
##&nbsp;            'use_pattern' => '0',
##&nbsp;            'pattern' => '0x0',
##&nbsp;            'reallocated_blocks' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; scramble path -v -n 10 /vol/compo_flex_vol/test_file; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 10 blocks at level 1 starting at fbn 0 in inode 96 in snapid 0 vol compo_flex_vol with: use_pattern=0 pattern=0x0 verbose=1
##&nbsp;reallocated 5 blocks for inode 96 in flex vol compo_flex_vol
##&nbsp;moved vvbn:14 fbn:0 pvbn:80214 to pvbn:16477
##&nbsp;moved vvbn:15 fbn:1 pvbn:80215 to pvbn:16478
##&nbsp;moved vvbn:16 fbn:2 pvbn:64268 to pvbn:16479
##&nbsp;moved vvbn:17 fbn:3 pvbn:64269 to pvbn:16480
##&nbsp;moved vvbn:18 fbn:4 pvbn:64270 to pvbn:16481
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'moved_blocks' => [
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '16477',
##&nbsp;                                  'vvbn' => '14',
##&nbsp;                                  'fbn' => '0',
##&nbsp;                                  'pvbn_from' => '80214'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '16478',
##&nbsp;                                  'vvbn' => '15',
##&nbsp;                                  'fbn' => '1',
##&nbsp;                                  'pvbn_from' => '80215'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '16479',
##&nbsp;                                  'vvbn' => '16',
##&nbsp;                                  'fbn' => '2',
##&nbsp;                                  'pvbn_from' => '64268'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '16480',
##&nbsp;                                  'vvbn' => '17',
##&nbsp;                                  'fbn' => '3',
##&nbsp;                                  'pvbn_from' => '64269'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '16481',
##&nbsp;                                  'vvbn' => '18',
##&nbsp;                                  'fbn' => '4',
##&nbsp;                                  'pvbn_from' => '64270'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'reallocating' => '10',
##&nbsp;            'inode' => '96',
##&nbsp;            'snapid' => '0',
##&nbsp;            'started_fbn' => '0',
##&nbsp;            'level' => '1',
##&nbsp;            'use_pattern' => '0',
##&nbsp;            'pattern' => '0x0',
##&nbsp;            'reallocated_blocks' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; scramble path -v -s 1 -n 10 -p 0x228 -a /vol/compo_flex_vol/test_file; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 10 blocks at level 1 starting at fbn 1 in inode 96 in snapid 0 vol compo_flex_vol with: use_pattern=1 pattern=0x228 verbose=1
##&nbsp;reallocated 4 blocks for inode 96 in flex vol compo_flex_vol
##&nbsp;moved vvbn:15 fbn:1 pvbn:16478 to pvbn:32418
##&nbsp;moved vvbn:16 fbn:2 pvbn:16479 to pvbn:32419
##&nbsp;moved vvbn:17 fbn:3 pvbn:16480 to pvbn:32420
##&nbsp;moved vvbn:18 fbn:4 pvbn:16481 to pvbn:32421
##&nbsp;overwrote 0 blocks with blocks (stride=1) containing pattern 0x00000228
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'moved_blocks' => [
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '32418',
##&nbsp;                                  'vvbn' => '15',
##&nbsp;                                  'fbn' => '1',
##&nbsp;                                  'pvbn_from' => '16478'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '32419',
##&nbsp;                                  'vvbn' => '16',
##&nbsp;                                  'fbn' => '2',
##&nbsp;                                  'pvbn_from' => '16479'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '32420',
##&nbsp;                                  'vvbn' => '17',
##&nbsp;                                  'fbn' => '3',
##&nbsp;                                  'pvbn_from' => '16480'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'pvbn_to' => '32421',
##&nbsp;                                  'vvbn' => '18',
##&nbsp;                                  'fbn' => '4',
##&nbsp;                                  'pvbn_from' => '16481'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'reallocating' => '10',
##&nbsp;            'overwrote_blocks' => '0',
##&nbsp;            'inode' => '96',
##&nbsp;            'snapid' => '0',
##&nbsp;            'started_fbn' => '1',
##&nbsp;            'level' => '1',
##&nbsp;            'use_pattern' => '1',
##&nbsp;            'pattern' => '0x00000228',
##&nbsp;            'reallocated_blocks' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub scramble_path {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my ( @result_array, $line );

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    while ( $line = shift(@lines) ) {
        if ( $line
            =~ /^Reallocating\s+(\d+).*\s(\d+).*\s(\d+).*\s(\d+).*\s(\d+)\s\w+\s(\w+).*:\s(\w+)=(\d)\s+(\w+)=(\w+).*/i
            )
        {
            $result_array[0]->{'reallocating'} = $1;
            $result_array[0]->{'level'}        = $2;
            $result_array[0]->{'started_fbn'}  = $3;
            $result_array[0]->{'inode'}        = $4;
            $result_array[0]->{'snapid'}       = $5;
            $result_array[0]->{'vol'}          = $6;
            $result_array[0]->{$7}             = $8;
            $result_array[0]->{$9}             = $10;
        } elsif ( $line =~ /reallocated (\d+)/ ) {
            $result_array[0]->{'reallocated_blocks'} = $1;
        } elsif ( $line
            =~ /moved\s(\w+):(\d+)\s+(\w+):(\d+)\s\w+:(\d+)\s+\w+\s+\w+:(\d+)/i
            )
        {
            my %internal_hash;
            $internal_hash{$1}          = $2;
            $internal_hash{$3}          = $4;
            $internal_hash{'pvbn_from'} = $5;
            $internal_hash{'pvbn_to'}   = $6;
            push( @{ $result_array[0]->{'moved_blocks'} }, \%internal_hash );
        } elsif ( $line =~ /overwrote (\d+).*\spattern\s(\d\w\d+)/i ) {
            $result_array[0]->{'overwrote_blocks'} = $1;
            $result_array[0]->{'pattern'}          = $2;
        } elsif ( $line =~ /placed (\d+).*ext_cache.*\s(\w+).*/i ) {
            $result_array[0]->{'ext_cache_blocks'} = $1;
            $result_array[0]->{'pattern'}          = $2;
        }
    }

    return \@result_array;
}

sub _scramble_blocks {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my ( @result_array, $line );
    while ( $line = shift(@lines) ) {
        if ( $line
            =~ /Reallocating (\d+).*\sin\s\w+\s(\w+)\s\w+:\s+(\w+)=(\d)\s+(\w+)=(\w+).*/i
            )
        {
            $result_array[0]->{reallocating}  = $1;
            $result_array[0]->{ $args{key1} } = $2;
            $result_array[0]->{$3}            = $4;
            $result_array[0]->{$5}            = $6;
        } elsif ( $line =~ /reallocated (\d+)/i ) {
            $result_array[0]->{ $args{key2} } = $1;
        } elsif (
            $line =~ /moved\s+(\w+):(\d+)\s+\w+:(\d+)\s+\w+\s+\w+:(\d+)/i )
        {
            my %internal_hash;
            $internal_hash{$1}          = $2;
            $internal_hash{'pvbn_from'} = $3;
            $internal_hash{'pvbn_to'}   = $4;
            push( @{ $result_array[0]->{'moved_blocks'} }, \%internal_hash );
        } elsif ( $line =~ /overwrote (\d+).*\spattern\s(\w+)/i ) {
            $result_array[0]->{'overwrote_blocks'} = $1;
            $result_array[0]->{'pattern'}          = $2;
        }
    }
    return \@result_array;
}

## @name ::scramble_pvbn
##
## @summary Parses the raw output from the "scramble pvbn" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from scramble_pvbn with any of its options as described in the
##    corresponding cdef file 'scramble.cdefs'
## @synopsis
##      scramble_pvbn(%args);
## INPUTS TO METHOD CALL 'scramble_pvbn' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pvbn_list' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; scramble pvbn compo_flex_aggr 24155 9015; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 2 pvbns in aggr compo_flex_aggr with:  use_pattern=0 pattern=0x0 verbose=0
##&nbsp;reallocated 0 pvbns in aggregate compo_flex_aggr
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pattern' => '0x0',
##&nbsp;            'use_pattern' => '0',
##&nbsp;            'reallocating' => '2',
##&nbsp;            'reallocated_pvbns' => '0',
##&nbsp;            'aggregate' => 'compo_flex_aggr'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; scramble pvbn -v -p 0x228 compo_flex_aggr 24155 9015; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 2 pvbns in aggr compo_flex_aggr with:  use_pattern=1 pattern=0x228 verbose=1
##&nbsp;reallocated 0 pvbns in aggregate compo_flex_aggr
##&nbsp;overwrote 0 blocks with blocks (stride=1) containing pattern 0x00000228
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pattern' => '0x00000228',
##&nbsp;            'use_pattern' => '1',
##&nbsp;            'reallocating' => '2',
##&nbsp;            'overwrote_blocks' => '0',
##&nbsp;            'reallocated_pvbns' => '0',
##&nbsp;            'aggregate' => 'compo_flex_aggr'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub scramble_pvbn {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my ( @result_array, $line );

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    _scramble_blocks(
        'lines' => \@lines,
        key1    => "aggregate",
        key2    => "reallocated_pvbns"
    );

}

## @name ::scramble_vvbn
##
## @summary Parses the raw output from the "scramble vvbn" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from scramble_vvbn with any of its options as described in the
##    corresponding cdef file 'scramble.cdefs'
## @synopsis
##      scramble_vvbn(%args);
## INPUTS TO METHOD CALL 'scramble_vvbn' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vvbn_list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; scramble vvbn compo_flex_vol 14 15 16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 3 vvbns in vol compo_flex_vol with: use_pattern=0 pattern=0x0 verbose=0
##&nbsp;Reallocated 3 vvbns in vol compo_flex_vol
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pattern' => '0x0',
##&nbsp;            'use_pattern' => '0',
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'reallocating' => '3',
##&nbsp;            'reallocated_vvbns' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; scramble vvbn -v -p 0x228 compo_flex_vol 14 15 16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Reallocating 3 vvbns in vol compo_flex_vol with: use_pattern=1 pattern=0x228 verbose=1
##&nbsp;Reallocated 0 vvbns in vol compo_flex_vol
##&nbsp;overwrote 0 blocks with blocks (stride=1) containing pattern 0x00000228
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pattern' => '0x00000228',
##&nbsp;            'use_pattern' => '1',
##&nbsp;            'vol' => 'compo_flex_vol',
##&nbsp;            'reallocating' => '3',
##&nbsp;            'overwrote_blocks' => '0',
##&nbsp;            'reallocated_vvbns' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub scramble_vvbn {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @lines  = get_lines( raw_output => $output, response_object => $resp );
    my $result;
    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    $result = _scramble_blocks(
        'lines' => \@lines,
        key1    => "vol",
        key2    => "reallocated_vvbns"
    );

    return $result;
}

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /scramble path: invalid\s+.*/i
        | $raw_output =~ /failed to reallocate\s+.*/ )
    {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /scramble .* required argument .*/i
        | $raw_output =~ /scramble\s\w+\S\s+Missing.*/i )
    {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /scramble .* not a mounted .* /i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

1;

