#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode scsi commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode scsi commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "scsi start" will be scsi_start()
##
## @synopsis
##    use scsi;
##
## @author amal@netapp.com
##
## @changes 2013/04/25 nandeesh@netapp.com : Added parser for command "scsi diskinfo"
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::scsi;

use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);


use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /(.*name of scsi device to use)/){
        $error = "MissingRequiredParam";
        $$error_message = $1;
        return $error;
    }elsif($output =~ /(.*not found)/){
        $error = 'InvalidParamValue';
        $$error_message = $1;
        return $error;
    }
    $$error_message = $1;
    return $error;

}


## @name ::scsi_inquiry
##
## @summary Parses the raw output from the "scsi inquiry" command
##
## @description
##    This method can be used for inquiring scsi status on a particular drive
## @synopsis
##      scsi_inquiry(%args);
## INPUTS TO METHOD CALL 'scsi_inquiry' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## <tr> <td> '-d' </td> <td> 'device-name' </td> </tr>
## <tr> <td> '-s' </td> <td> 'serial' </td> </tr>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## my $output = $Api_Set_Obj->scsi_inquiry('serial'=>'1', 'disk' => '0b.21');
##

sub scsi_inquiry{
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error   = _check_error(raw_output=>$output,command=>"scsi inquiry",
                             error_message=>\$error_message);

    if($error){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
    $error->throw(text=>$error_message,response_object => $resp);
    }
    my $final_output = [];

    if($command =~ m/inquiry -s/){
        #serial #: 'HU104514FJ'
        $output =~ m/serial #:(.*)/;
        $final_output->[0]->{'serial'} = $1;
    }elsif($command =~ m/inquiry/){
        my @personal = split(/\n/, $output);
        #Peripheral Qualifier/Type: 0x1  (Sequential-access device is connected on this logical unit)
        $personal[0] =~ m/.*:(.*)/;
        $final_output->[0]->{'type'} = $1;
        #RMB: 0x80 Version: 0x3 (device complies with SPC standard)
        $personal[1] =~ m/RMB:(.*)\sVersion:(.*)/;
        $final_output->[0]->{'rmb'}     = $1;
        $final_output->[0]->{'version'} = $2;
        #NormACA: 0 HiSup: 0 RDF: 0x2
        $personal[2] =~ m/NormACA: (.*) HiSup: (.*) RDF: (.*)/;
        $final_output->[0]->{'normaca'} = $1;
        $final_output->[0]->{'hisup'}   = $2;
        $final_output->[0]->{'rdf'}     = $3;
        #Additional Length: 0x5b (91)
        $personal[3] =~ m/Additional Length: (.*)/;
        $final_output->[0]->{'length'}  = $1;
        #t10_vendor_id: "HP      "
        $personal[6] =~ m/t10_vendor_id: "(.*?)\s+"/;
        $final_output->[0]->{'vendor'}  = $1;
        #product_id: "Ultrium 3-SCSI  "
        $personal[7] =~ m/product_id: "(.*)"/;
        $final_output->[0]->{'product_id'}  = $1;
        #revision: "L22D"
        $personal[8] =~ m/revision: "(.*)"/;
        $final_output->[0]->{'revision'} = $1;
        #vendor unique ""
        $personal[9] =~ m/vendor unique "(.*)"/;
        $final_output->[0]->{'vendor_unique'} = $1;
    }
    return $final_output;
}



## @name ::scsi_start
##
## @summary Parses the raw output from the "scsi start" command
##
## @description
##    This method can be used for starting scsi on a particular drive
## @synopsis
##      scsi_start(%args);
## INPUTS TO METHOD CALL 'scsi_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## <tr> <td> '-d' </td> <td> 'device-name' </td> </tr>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## my $output = $Api_Set_Obj->scsi_start('disk' => '0b.21');
##

sub scsi_start{
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error   = _check_error(raw_output=>$output,command=>"scsi start",
                             error_message=>\$error_message);

    if($error){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
    $error->throw(text=>$error_message,response_object => $resp);
    }

    my $final_output = [];

    if($command =~ m/start/){
        #scsi start: completed in 0 seconds
        if($output =~ m/scsi start: completed in (.*) seconds/)
        {
        $final_output->[0]->{'status'} = 'completed';
        $final_output->[0]->{'time'}   =  $1;
    }
     }
    return $final_output;

}


## @name ::scsi_diskinfo
##
## @summary Parses the raw output from the "scsi diskinfo" command
##
## @description
##    This method can be used for starting scsi on a particular drive
## @synopsis
##      scsi_diskinfo(%args);
## INPUTS TO METHOD CALL 'scsi_diskinfo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## <tr> <td> '-d' </td> <td> 'device-name' </td> </tr>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## my $output = $Api_Set_Obj->scsi_diskinfo('device-name' => '0b.21');
##

sub scsi_diskinfo{
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(raw_output=>$output,command=>"scsi diskinfo",
                             error_message=>\$error_message);

    if($error){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
    $error->throw(text=>$error_message,response_object => $resp);
    }

    my $final_output = [];
    my $final;
    if($command =~ m/diskinfo/){
        my @personal  = split(/(diskinfo for disk|Configurational Options|Disk Driver Information|Disk Qualification Details|RAID Information|DBN\'s of interest|ERROR INJECTION constants|Partition Info):?/, $output);
        shift @personal;
        my %disk_hash=@personal;
        my @temp;
        while( my ($key, $value) = each(%disk_hash)){
            if($key =~ /Partition Info/i){
                @temp = split(/Partition Table:\n/, $value);
                foreach my $tmp (@temp){
                    while($tmp =~ /\s*(.*)\s*:\s*(.*)\s*\n?/g) {
                        my $tmp1 = $1;
                        my $tmp2 = $2;
                        if($tmp !~ /valid/){
                            $final->{$key}->{$tmp1}=$tmp2 ;
                        } elsif($tmp1 =~ /Edit Time:\s+(.*)/){
                            $final->{$key}->{'Edit Time'} = "$1".':'."$tmp2" ;
                        } else {
                            $final->{$key}->{'Partition Table'}->{$tmp1}=$tmp2 ;
                        }
                    }
                }
                while ($temp[1] =~ /(\d+)\s+(\d\d\d+)/g) {
                    $final->{$key}->{'Partition Table'}->{partition}->{$1} = $2 ;
                }
            } elsif( $key =~ /diskinfo for disk/i){
                my $disk = $1 if($value =~ /(\S+):/g) ;
                while($value =~ /(\S+)\s+(\S+)/g){
                    $final->{$key}->{$disk}->{$1} = $2;
                }
            } else {
                my $value1 = $value;
                while($value =~ /\s*(.*)\s*:\s*(.*)\s*\n/g){
                    my $tmp3 = $1;
                    my $tmp4 = $2;
                    $final->{$key}->{$tmp3} = $tmp4 ;
                }
                while($value1 =~ /\s*(\S+)\s*(.*)\s*\n/g){
                    my $tmp3 = $1;
                    my $tmp4 = $2;
                    if(($tmp3!~ /:/) && ($tmp4 !~ /:/)){
                        $final->{$key}->{$tmp3} = $tmp4 ;
                    }
                }
            }
        }
    }
    $final_output->[0] = $final;
    return $final_output;
}

1;

