# $Id: //depot/prod/test/nacl/lib/NACL/APISet/Node/CLI/7Mode/default/secureadmin.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode secure admin commands
##
## @description
##    This is a library to parse the raw output of 7Mode secureadmin commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use secureadmin;
##
##    secureadmin_addcert_ssl(%args);
##    secureadmin_status (%args);
##
##
## @author anupamap@netapp.com,
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::secureadmin;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /[Uu]sage:\s/) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    }
    return $error;
}

## @name ::secureadmin_status
##
## @summary Parses the raw output from the "secureadmin status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from secureadmin_status with any of its options as described in the
##    corresponding cdef file 'secureadmin.cdefs'
## @synopsis
##      secureadmin_status(%args);
## INPUTS TO METHOD CALL 'secureadmin_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## @examples
##&nbsp;Command = secureadmin status
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'service' => 'ssh2',
##&nbsp;            'state' => 'active'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'service' => 'ssh1',
##&nbsp;            'state' => 'active'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'service' => 'ssl',
##&nbsp;            'state' => 'active'
##&nbsp;          }
##&nbsp;        ];

sub secureadmin_status {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $lin;
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines=get_lines('raw_output'=>$output,response_object=>$resp);
    foreach $lin (@lines){
    my %info;
    if($lin =~ /(\w+)\s+-\s+(\w+)/){
        $info{"service"} = $1;
        $info{"state"} = $2;
    }
       push (@result_array, \%info);
    }
    return \@result_array;
}

## @name ::secureadmin_addcert_ssl
##
## @summary Parses the raw output from the "secureadmin addcert ssl" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from secureadmin_addcert_ssl with any of its options as described in the
##    corresponding cdef file 'secureadmin.cdefs'
## @synopsis
##      secureadmin_addcert_ssl(%args);
## INPUTS TO METHOD CALL 'secureadmin_addcert_ssl' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'certificate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = secureadmin addcert ssl /etc/keymgr/cert/secureadmin.pem
##&nbsp;$VAR1 = [];
##&nbsp;

sub secureadmin_addcert_ssl {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $lin;
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];

}
1;
