# $Id$
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to execute 7Mode setral command
##
## @descriptiona
## This APISet allows for the setting of setral options, reseting all setral
## options and showing currently configured setral options as well as setting
## or more setral options
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::setral;
##
##    setral(%args);
##
## @author bellizzi@netapp.com, anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::setral;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use constant rd => '_random_deny';
use constant pc => '_percentage_consumed';

## @name ::setral
##
## @summary Parses the raw output from the "setral" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from setral with any of its options as described in the 
##    corresponding cdef file 'setral.cdefs'
## @synopsis
##      setral(%args);
## INPUTS TO METHOD CALL 'setral' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'page' </td> <td> 'page' </td> </tr> 
## <tr> <td> 'excl' </td> <td> 'excl' </td> </tr> 
## <tr> <td> 'spinhimem' </td> <td> 'nblhostrpc' </td> </tr> 
## <tr> <td> 'reset' </td> <td> 'reset' </td> </tr> 
## <tr> <td> 'nwdrop' </td> <td> 'nwdrop' </td> </tr> 
## <tr> <td> 'hseg' </td> <td> 'hseg' </td> </tr> 
## <tr> <td> 'stall' </td> <td> 'stall' </td> </tr> 
## <tr> <td> 'nscorrupt' </td> <td> 'nscorrupt' </td> </tr> 
## <tr> <td> 'gc' </td> <td> 'gc' </td> </tr> 
## <tr> <td> 'spinhimem' </td> <td> 'spinhimem' </td> </tr> 
## <tr> <td> 'fd' </td> <td> 'fd' </td> </tr> 
## <tr> <td> 'incl' </td> <td> 'incl' </td> </tr> 
## <tr> <td> 'show' </td> <td> 'show' </td> </tr> 
## <tr> <td> 'cpu' </td> <td> 'cpu' </td> </tr> 
## <tr> <td> 'nwcorrupt' </td> <td> 'nwcorrupt' </td> </tr> 
## <tr> <td> 'java' </td> <td> 'java' </td> </tr> 
## <tr> <td> 'kma_sleep' </td> <td> 'kma_sleep' </td> </tr> 
## <tr> <td> 'socket' </td> <td> 'socket' </td> </tr> 
## <tr> <td> 'pBlk' </td> <td> 'pBlk' </td> </tr> 
## <tr> <td> 'mbuf' </td> <td> 'mbuf' </td> </tr> 
## <tr> <td> 'mbuf' </td> <td> 'waflbuf' </td> </tr> 
## <tr> <td> 'raidrm' </td> <td> 'raidrm' </td> </tr> 
## <tr> <td> 'spinhicsm' </td> <td> 'spinhicsm' </td> </tr> 
## <tr> <td> 'malloc' </td> <td> 'malloc' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub setral {
    my %args = @_;

    my $error_message;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command      = $resp->get_actual_command();
    my @result_array = ( { stats => [] } );

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'setral',
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    if( $command =~ /setral\s+stall/) {
        return [];
    }

    my $first_line
        = "(setral: agitation started|setral: agitation stopped|setral settings:)";
    my @lines = get_lines(
        raw_output      => $raw_output,
        first_line      => $first_line,
        response_object => $resp,
        can_throw       => 1
    );
    if ( $lines[0] =~ /agitation\s+(.+)/ ) {
        return [ { 'agitation' => $1 } ];
    }

    my ( $line, $value, $name );
    my %settings_keys = (
         'cpu'                 => [ 'cpu' . pc ],
         'malloc'              => [ 'malloc' . pc, 'malloc' . rd ],
         'mbuf'                => [ 'mbuf' . pc, 'mbuf' . rd ],
         'waflbuf'             => [ 'waflbuf' . pc, 'waflbuf' . rd ],
         'waflbdata'           => [ 'waflbdata' . pc, 'waflbdata' .rd ],
         'fd'                  => [ 'fd' . pc, 'fd' . rd ],
         'pBlk'                => [ 'pblk' . pc, 'pblk' . rd ],
         'raidrm'              => [ 'raidrm' . pc, 'raidrm' . rd ],
         'socket deny'         => [ 'socket' . rd ],
         'nwdata drop'         => [ 'nwdrop' . rd ],
         'corrupt'             => ['nwcorrupt_random_corrupt'],
         'ns corrupt'          => ['nscorrupt_random_corrupt'],
         'java deny'           => [ 'java' . rd ],
         'java gc'             => ['random_garbage_collect'],
         'page deny'           => [ 'page' . rd ],
         'hseg deny'           => [ 'hseg' . rd ],
         'kma_sleep'           => [ 'random_sleep', 'sleep_time' ],
         'spinhi csm deny'     => [ 'spinhicsm' . rd ],
         'mem deny'            => [ 'spinhimem' . rd ],
         'nblade hostrpc deny' => [ 'nblhostrpc' . rd ],
         'delay_max_usecs'     => ['delay_max_usecs'],
         'delay_hz'            => ['delay_hz'],
         'delay_domain'        => ['delay_domain'],
        );
    $lines[0] =~ s/setral settings:\s+//;

    # Section 1 of output
    while ( $lines[0] !~ /active/ ) {
        #                   name       value(s)
        while ( $lines[0] =~ s/([A-Za-z _]+)((0[xX][0-9a-fA-F]+)|([^A-Za-z]+))// ) {
            $name = trim_string( string => $1 );
			$value = $2;
			trim( string_ptr => \$value );
            foreach ( split /\s+/, $value ) {
			    my $key = shift @{ $settings_keys{$name} };
			    $result_array[0]->{ $key } = $_;
            }

        }
        # now ready $line for the next iteration
        shift @lines;
	}

    ( $result_array[0]->{'agitation'} ) = ( $lines[0] =~ /\((.*active)\)/ );
    shift @lines;

    # Section 2 of output
    # Now we have to deal with the statistics section present only for show
    if ( $command =~ /setral\s+show/ ) {
        $lines[0] =~ s/.+Consumed/Consumed/ if @lines;
        my @keys = split( /\s+/, $lines[0] );

        # we fill in the inner hash in reverse order since some values in the
        # beginning after the name can be missing from each row of statistics
        # output
        @keys = reverse @keys;

        # remove the heading line, so we are only left with the value lines
        shift(@lines);

        # the last part of the output - included/excluded rtags section
        while ( $lines[-1] =~ /(.+cluded rtags):\s+(.+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) }
                = [ split( /\s+/, $2 ) ];
            pop @lines;
        }

        # logic to handle each row of statistics output
        foreach $line (@lines) {
            my %inner_hash;
            my @values = split( /\s+/, $line );
            $inner_hash{'name'} = shift @values;

            foreach (@keys) {
                $inner_hash{ to_key_name( item => $_ ) } = pop @values;
            }
            push @{ $result_array[0]->{stats} }, \%inner_hash;
        }
    }
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    my $invalidParamRegex = '(unknown|not allowed)';
    if ( $raw_output =~ /usage:|parameter missing/ ) {
        $$error_message = "Required parameter missing";
        $error          = "MissingRequiredParam";
        return $error;
    } elsif ( $raw_output =~ /$command:(.+($invalidParamRegex))/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
        return $error;
    } elsif ( $raw_output =~ /$command:(.+(bad|too large|too small|value must be))/ ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
        return $error;
    }
    return undef;
}

1;
