#
# Copyright (c) 2002-13 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode sis commands
##
## @description
##     This is a library to parse the raw output of 7Mode sis commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use sis;
##
##    sis_config(%args);
##    sis_help(%args);
##    sis_off(%args);
##    sis_on(%args);
##    sis_revert_to(%args);
##    sis_start(%args);
##    sis_status(%args);
##    sis_stop(%args);
##    sis_check(%args);
##    sis_dump(%args);
##    sis_ls(%args);
##    sis_reset(%args);
##    sis_stat(%args);
##    sis_undo(%args);
##    sis_file_idd(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::sis;

use Data::Dumper;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors(%){
    my %args=@_;
    my $raw_output=$args{'raw_output'};
    my $command=$args{'command'};
    my $errormsg_ptr=$args{'errormsg_ptr'};

    if($raw_output=~/SIS is not licensed:/){
        $$errormsg_ptr="sis service is not licensed";
        return "FeatureNotLicensed";
    } elsif(($raw_output=~/Invalid path specification:/)||($raw_output=~/Volume does not exist:/)||($raw_output=~/Invalid location specification/)){
        $$errormsg_ptr=$command." contains invalid value(s)\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }elsif($raw_output=~/Path must be supplied as \/vol\/volname/){
        $$errormsg_ptr=$command." is missing argument(s)\nActual error:\n".$raw_output;
        return "MissingRequiredParam";
    }elsif($raw_output=~/Operation has already been enabled:/){
        $$errormsg_ptr="operation already enabled";
        return "CommandFailed";
    }elsif($raw_output=~/Operation has already been disabled:/){
        $$errormsg_ptr="operation already disabled";
        return "CommandFailed";
    }elsif($raw_output=~/Operation is currently active:/i){
        $$errormsg_ptr="operation currently active";
        return "CommandFailed";
    }elsif($raw_output=~/Operation is currently idle:/){
        $$errormsg_ptr="operation currently idle";
        return "CommandFailed";
    }elsif($raw_output=~/Invalid schedule /){
        $$errormsg_ptr=$command." has invalid parameter value(s)\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }elsif($raw_output=~/minimum blocks shared value: .* not in the range: 1-64/){
        $$errormsg_ptr=$command." has invalid parameter value(s).\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }elsif($raw_output=~/version should be specified/i){
        #ignoring the case due to UI changes burt: 444128
        $$errormsg_ptr=$command." is missing argument(s).\nActual error:\n".$raw_output;
        return "MissingRequiredParam";
    }elsif($raw_output=~/(Volume does not exist|Volume type has to be Flexvol)/){
        $$errormsg_ptr= $command." has invalid parameter value(s).\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }elsif($raw_output=~/(sis: unrecognized command|Cannot find directory|Invalid file argument|Invalid dir argument|Invalid offset values)/){
        $$errormsg_ptr= $command." has invalid parameter value(s). \nActual error:\n".$raw_output;
        return "InvalidParamValue";
    } elsif($raw_output=~/(Failed to calculate size of dense metafile data|sis dump options -file and -dir cannot be used together|The -offset option must be used with the -file option|Volume .+ does not have enough free space|Volume .+ is not sis enabled)/){
        $$errormsg_ptr=$1;
        return "CommandFailed";
    }elsif($raw_output=~/(Volume is not SIS enabled|No compressed data in the volume|Operation is not disabled|Operation is not enabled|Operation is not configured)/){
        $$errormsg_ptr=$1;
        return "CommandFailed";
    }elsif($raw_output=~/(Option '-d' should be used along with '-t wrong' semantics only|option 'v' should be used along with option 'c' only|option 'd' should not be used along with other options)/){
        $$errormsg_ptr= $command." has invalid parameter(s).\nActual error: ".$raw_output;
        return "InvalidParam";
    }elsif($raw_output=~/sis reset <path>/){
        $$errormsg_ptr= $command." is missing argument(s).\nActual error:\n".$raw_output;
        return "MissingRequiredParam";
    }
    elsif($raw_output=~/Invalid file path No such file or directory/) {
        $$errormsg_ptr= $command." has invalid file path.\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }
    elsif($raw_output=~/sis file_idd Failed on file/) {
        $$errormsg_ptr= $command." has invalid filename.\nActual error:\n".$raw_output;
        return "InvalidParamValue";
    }
    elsif($raw_output=~/Invalid number of arguments/) {
        $$errormsg_ptr=$command." is missing argument(s).\nActual error:\n".$raw_output;
        return "MissingRequiredParam";
    }
    elsif($raw_output=~/The -Q option requires a true\/false argument/) {
        $$errormsg_ptr= $command." has invalid argument.\nActual error:\n".$raw_output;
        return "InvalidParam";
    }
    elsif($raw_output=~/Invalid file size/) {
        $$errormsg_ptr= $command." has invalid file size.\nActual error:\n".$raw_output;
        return "InvalidParam";
    }elsif ( $raw_output =~ /Volume is offline|Volume is restricted|Volume is in use by other operations/i) {
	$$errormsg_ptr= $raw_output;
        return "CommandFailed";
    }
}

sub _sis_config(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $line=shift(@lines);
    my $resp=$args{'response_obj'};
	my @return_array;

    if($line=~/^Path/){
        my @keys=split(/  +/,$line);
        foreach my $key(@keys){
            $key=to_key_name('item'=>$key);
        }
        foreach my $data_line (@lines){
            my %hash;
            my @fields=split(/ +/,$data_line);
            if(scalar(@keys)==scalar(@fields)){
                for(my $i=0;$i<scalar(@keys);$i++){
                    $hash{$keys[$i]}=$fields[$i];
                }
            push (@return_array,\%hash);
            }
        }
    }
	return \@return_array;
}

sub _get_record(%){
    my %args=@_;
    my $lines=$args{'lines'};
    my %hash ;
    my $line=shift(@$lines);
    do{
        my ($key,$value)=split(/:  +/,$line);
        $key=to_key_name('item'=>$key);
        $hash{$key}=$value;
        if(($$lines[0]=~/^Path:/)||(scalar(@$lines)<1)){
            return %hash;
        } else {
            $line=shift(@$lines);
        }
    } while (1);
}
sub _sis_status(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $line=$lines[0];
    my @return_array;
    my $resp=$args{'response_obj'};

    if ($line=~/No status|statistics entry found./) {
        return \@return_array;
    } elsif ($line=~/^Path:/) {
        while (scalar(@lines)){
            my %hash=_get_record('lines'=>\@lines);
            push (@return_array,\%hash);
        }
    } elsif ($line=~/^Path/) {
        my @keys=split(/  +/,$line);
        foreach my $key(@keys){
            $key=to_key_name('item'=>$key);
        }
        shift(@lines);
        foreach my $data_line (@lines){
            my %hash;
            my @temp_fields=split(/ +/,$data_line);
            my (@fields, @progress_fields, $progress);

            for(my $j=0;$j<3; $j++) {
               $fields[$j] =shift @temp_fields;
            }
            for(my $k=0;$k<scalar(@temp_fields);$k++){
                push(@progress_fields, $temp_fields[$k]);
            }
            $progress = join(' ', @progress_fields);
            push(@fields, $progress);

            if(scalar(@keys)==scalar(@fields)){
                for(my $i=0;$i<scalar(@keys);$i++){
                    $hash{$keys[$i]}=$fields[$i];
                }
            push (@return_array,\%hash);
            }else{
                my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(text=>"Unexpected form of output",response_object => $resp);
            }
        }
    } else{
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(text=>"Unexpected form of output",response_object => $resp);
    }
    return \@return_array;
}
sub _sis_stop(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my $line=shift(@lines);

    if($line=~/^The operation on ".*" is being stopped./){
        my %hash;
        my @return_array;
        $hash{'result'}="successful";
        push (@return_array,\%hash);
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

sub _sis_start(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
	for (my $j = 0; $j <= $#lines; $j++) {
        if ($lines[$j] =~ /The SIS operation for \".*\" is started./) {
            my %hash;
            my @return_array;
            $hash{'result'} = "successful";
            push(@return_array, \%hash);
            return \@return_array;
        }
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

sub _sis_off(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};

    my $line=shift(@lines);

    if($line=~/^SIS for ".*" is disabled./){
        my %hash;
        my @return_array;
        $hash{'result'}="successful";
        push (@return_array,\%hash);
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}
sub _sis_on(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp = $args{'response_obj'};

    my $line=shift(@lines);

    if($line=~/^SIS for .* is enabled./){
        my %hash;
        my @return_array;
        $hash{'result'}="successful";
        push (@return_array,\%hash);
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

sub _sis_dump(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my $line=shift(@lines);
    if($line =~ /Free Space Required:\s(\d+)\sBytes./){
        my %hash;
        my @return_array;
        $hash{'space_required'} = $1;
        $line = shift(@lines);
        if ($line =~ /Saved metafiles in (.+)./) {
            $hash{'metafiles_saved_in'}=$1;
        }
        push (@return_array,\%hash);
        return \@return_array;
    }
    my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
    $error->throw(text=>"Unexpected form of output",response_object => $resp);
}

sub _sis_ls(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp = $args{'response_obj'};

    my @return_array;
    while ( my $line = shift(@lines) ) {
        my $info_hash = {};
        my $directory;
        if($line=~/^SIS metadata for flexible volume (.+) in aggregate (.+):$/) {
            $info_hash->{'volume'} = $1;
            $info_hash->{'aggregate'} = $2;
            shift(@lines); #Skip the headings
            shift(@lines); #Skip the dashes
            $line = shift(@lines);
            while($line !~ /Total files:/) {
                my $table_hash ={};
                if ($line =~ /(\S+)(?:\s+)\(Directory\)/) {
                    $directory = $1;
                    $line = shift(@lines);
                }
                if ($line =~ /(\S+)\s\s+(\S+)\s\s+(\S+)/) {
                    $table_hash->{'filename'} = $1;
                    $table_hash->{'size'} = $2;
                    $table_hash->{'blocks_used'} = $3;
                    $table_hash->{'directory'} = $directory if ($table_hash->{'filename'} =~ /bucket/);
                }
                push(@{ $info_hash->{'file_info'} }, $table_hash);
                $line = shift(@lines);
            }
            if ($line =~ /Total files: (\d+)\. Total blocks used: (\d+)/) {
                $info_hash->{'total_files_in_aggr'} = $1;
                $info_hash->{'total_blocks_used_in_aggr'} = $2;
            }
            $line = shift(@lines);
            if($line =~ /SIS metadata in flexible volume (.+):/) {
                shift(@lines); #skip headings
                shift(@lines); #skip dashes
                $line = shift(@lines);
                while($line !~ /Total files:/) {
                    my $table_hash ={};
                    if ($line =~ /(\S+)(?:\s+)\(Directory\)/) {
                        $directory = $1;
                        $line = shift(@lines);
                    }
                    if ($line =~ /(\S+)\s\s+(\S+)\s\s+(\S+)/) {
                        $table_hash->{'filename'} = $1;
                        $table_hash->{'size'} = $2;
                        $table_hash->{'blocks_used'} = $3;
                        $table_hash->{'directory'} = $directory if ($table_hash->{'filename'} =~ /bucket/);
                    }
                    push(@{ $info_hash->{'volume_file_info'} }, $table_hash);
                    $line = shift(@lines);
                }
                if ($line =~ /Total files: (\d+)\. Total blocks used: (\d+)/) {
                    $info_hash->{'total_files_in_volume'} = $1;
                    $info_hash->{'total_blocks_used_in_volume'} = $2;
                }
            }
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected form of output",response_object => $resp);
        }
        push (@return_array,$info_hash);
    }

    return \@return_array;
}

## @name ::sis_on
##
## @summary Parses the raw output from the "sis on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_on with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_on(%args);
## INPUTS TO METHOD CALL 'sis_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sis_on(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="SIS";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_on('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_off
##
## @summary Parses the raw output from the "sis off" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_off with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_off(%args);
## INPUTS TO METHOD CALL 'sis_off' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sis_off(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="SIS";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_off('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_status
##
## @summary Parses the raw output from the "sis status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_status with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_status(%args);
## INPUTS TO METHOD CALL 'sis_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-l' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; sis status /vol/vol_on_the_fly_20111108122307; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Path                           State      Status     Progress
##&nbsp;/vol/vol_on_the_fly_20111108122307 Enabled    Idle       Idle for 02:43:52
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'progress' => 'Idle for 02:43:52',
##&nbsp;            'status' => 'Idle',
##&nbsp;            'path' => '/vol/vol_on_the_fly_20111108122307',
##&nbsp;            'state' => 'Enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sis_status(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="(Path|No)";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_status('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_start
##
## @summary Parses the raw output from the "sis start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_start with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_start(%args);
## INPUTS TO METHOD CALL 'sis_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-D' </td> <td> 'dedupe' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'snapshot-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delete-checkpoint' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'compression' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'shared-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'scan' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'enqueue' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'use-checkpoint' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sis_start(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="The";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_start('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_stop
##
## @summary Parses the raw output from the "sis stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_stop with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_stop(%args);
## INPUTS TO METHOD CALL 'sis_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sis_stop(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="The";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_stop('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_config
##
## @summary Parses the raw output from the "sis config" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sis_config with any of its options as described in the 
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_config(%args);
## INPUTS TO METHOD CALL 'sis_config' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-C' </td> <td> 'enable-compression' </td> <td> '0' </td> </tr> 
## <tr> <td> '-Q' </td> <td> 'idd' </td> <td> '0' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'verify-trigger-rate' </td> <td> '0' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'minimum-blocks-shared' </td> <td> '0' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'inline-compression' </td> <td> '0' </td> </tr> 
## <tr> <td> '-z' </td> <td> 'quick-check-fsize' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'schedule' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; sis config  -Q true /vol/c_testvol; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q test; sis config  -z 524288000 /vol/c_testvol; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sis_config(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    if($command=~/-[sm]/){
        my $first_line="";
        my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
        if (scalar(@lines)==0){
            return [];
        }
    }
    my $first_line="Path";
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_config('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_revert_to
##
## @summary Parses the raw output from the "sis revert_to" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_revert_to with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_revert_to(%args);
## INPUTS TO METHOD CALL 'sis_revert_to' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-delete' </td> <td> 'delete' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-cleanup' </td> <td> 'cleanup' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sis_revert_to(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="The";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    return [];
}
## @name ::sis_help
##
## @summary Parses the raw output from the "sis help" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_help with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_help(%args);
## INPUTS TO METHOD CALL 'sis_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub sis_help(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my %hash;
    my @return_array;
    $hash{'help_output'}=help_output('raw_output'=>$raw_output);
    push(@return_array,\%hash);
    return \@return_array;
}

## @name ::sis_stat
##
## @summary Parses the raw output from the "sis stat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_stat with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_stat(%args);
## INPUTS TO METHOD CALL 'sis_stat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-l' </td> <td> 'verbose' </td> </tr>
## <tr> <td> '-b' </td> <td> 'blocks' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose-mode' </td> </tr>
## <tr> <td> '-g' </td> <td> 'node-statistics' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sis stat /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'allocated' => '112 MB',
##&nbsp;            'saving' => '0 KB',
##&nbsp;            '%saved' => '0 %',
##&nbsp;            'path' => '/vol/vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = sis stat -l /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mismatch_by_overwrite' => '0',
##&nbsp;            'succeeded_check_op' => '0',
##&nbsp;            '%saved' => '0 %',
##&nbsp;            'same_fbn_location' => '0',
##&nbsp;            'total_verify_time' => '-',
##&nbsp;            'succeeded_op' => '0',
##&nbsp;            'shared' => '0 KB',
##&nbsp;            'staled_donor' => '0',
##&nbsp;            'uncompressed_walloced' => '0',
##&nbsp;            'partial_cg_modifies' => '0',
##&nbsp;            'file_too_small' => '0',
##&nbsp;            'total_process_time' => '-',
##&nbsp;            'background_compressed' => '0',
##&nbsp;            'avg_compress_time' => '0',
##&nbsp;            'total_sorted_blocks' => '0',
##&nbsp;            'allocated' => '115364 KB',
##&nbsp;            'compressed_walloced' => '0',
##&nbsp;            'compression_saved' => '0',
##&nbsp;            'sis_files' => '2',
##&nbsp;            'snapshot_blocks_skipped' => undef,
##&nbsp;            'suspended_check_op' => '0',
##&nbsp;            'out_of_space' => '0',
##&nbsp;            'avg_decompress_time' => '0',
##&nbsp;            'additional_sharing_messages' => undef,
##&nbsp;            'inline_compressed' => '0',
##&nbsp;            'unaligned_compression_blocks' => undef,
##&nbsp;            'fp_false_match' => '0',
##&nbsp;            'un-flushed_changelogs' => undef,
##&nbsp;            'staled_recipient' => '0',
##&nbsp;            'failed_check_op' => '0',
##&nbsp;            'overlapped_blocks' => '0',
##&nbsp;            'failed_op' => '0',
##&nbsp;            'shared_blocks_skipped' => '0',
##&nbsp;            'deferred_op' => '0',
##&nbsp;            'cg_decompressed' => '0',
##&nbsp;            'same_vbn' => '0',
##&nbsp;            'total_processed' => '0 KB',
##&nbsp;            'compress_low_savings' => '0',
##&nbsp;            'decompress_disk_bad' => '0',
##&nbsp;            'mismatched_data' => '0',
##&nbsp;            'saving' => '0 KB',
##&nbsp;            'new_partial_cg_writes' => '0',
##&nbsp;            'compress_failed' => '0',
##&nbsp;            'total_fp_deleted' => '0',
##&nbsp;            'path' => '/vol/vol0',
##&nbsp;            'same_data' => '0',
##&nbsp;            'decompress_sw_bad' => '0',
##&nbsp;            'stopped_op' => '0',
##&nbsp;            'max_reference_hits' => '0',
##&nbsp;            'compress_attempts' => '0',
##&nbsp;            'extra_cp_reads' => '0',
##&nbsp;            'max_refcount' => '32767',
##&nbsp;            'same_fingerprint' => '0',
##&nbsp;            'started_op' => '0',
##&nbsp;            'delino_records' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sis_stat(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line=qr/(Path|No|(.+?):\s+\d+)/;
    my $can_throw=1;
    my $error;
    my $errormsg;
    my %hash;
    my @result;
    my $i=0;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if ($command =~ /-g/) {
        while (my $line = shift @lines) {
            my ($key,$value)=split(/:\s*|\s\s+/,$line);
            $key=to_key_name('item'=>$key);
            $hash{$key}=$value;
        }
        return [\%hash];
    } elsif ($command =~ /-b/ || $command =~ /^sis stat$/ || $command =~ /^sis stat \/.*/) {
        while (my $line = shift @lines) {
            if($line =~ /(\S+)\s+(\d+\s+\w+)\s+(\d+\s+\w+)\s+(\d+\s+\S+)/ ){
                $result[$i]->{"path"} = $1;
                $result[$i]->{"allocated"} = $2;
                $result[$i]->{"saving"} = $3;
                $result[$i]->{"%saved"} = $4;
                $i++;
            }
        }
        return \@result;
    } else {
        # reusing _sis_status method, because it does the same parsing as required for sis_stat
        return (_sis_status('lines'=>\@lines,'response_obj'=>$resp));
    }
}

## @name ::sis_dump
##
## @summary Parses the raw output from the "sis dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_dump with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_dump(%args);
## INPUTS TO METHOD CALL 'sis_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-to' </td> <td> 'to' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose-mode' </td> </tr>
## <tr> <td> '-file' </td> <td> 'file' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-dir' </td> <td> 'dir' </td> </tr>
## <tr> <td> '-offset' </td> <td> 'offset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sis dump /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'metafiles_saved_in' => '/vol/vol0/sisdump',
##&nbsp;            'space_required' => '944264'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sis_dump(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="Free Space Required";
    my $can_throw=1;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_dump('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_ls
##
## @summary Parses the raw output from the "sis ls" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_ls with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_ls(%args);
## INPUTS TO METHOD CALL 'sis_ls' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
# @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sis ls /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_files_in_volume' => '4',
##&nbsp;            'total_blocks_used_in_aggr' => '0',
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'volume_file_info' => [
##&nbsp;                                    {
##&nbsp;                                      'blocks_used' => '1',
##&nbsp;                                      'filename' => 'stale_fbns',
##&nbsp;                                      'size' => '4KB'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'blocks_used' => '1',
##&nbsp;                                      'filename' => 'bucket0',
##&nbsp;                                      'directory' => 'clogdir.0',
##&nbsp;                                      'size' => '4KB'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'blocks_used' => '1',
##&nbsp;                                      'filename' => 'bucket0',
##&nbsp;                                      'directory' => 'clogdir.1',
##&nbsp;                                      'size' => '4KB'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'blocks_used' => '1',
##&nbsp;                                      'filename' => 'dense_state_bin',
##&nbsp;                                      'size' => '0KB'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'aggregate' => 'aggr0',
##&nbsp;            'total_blocks_used_in_volume' => '4',
##&nbsp;            'total_files_in_aggr' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sis_ls(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="SIS metadata for flexible";
    my $can_throw=1;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_sis_ls('lines'=>\@lines,'response_obj'=>$resp));
}

## @name ::sis_undo
##
## @summary Parses the raw output from the "sis undo" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_undo with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_undo(%args);
## INPUTS TO METHOD CALL 'sis_undo' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-C' </td> <td> 'compression' </td> </tr>
## <tr> <td> '-D' </td> <td> 'dedupe' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> '-d' </td> <td> 'log' </td> </tr>
## <tr> <td> '-t' </td> <td> 'undo-type' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sis undo /vol/vol0
##&nbsp;$VAR1 = [];
##&nbsp;
sub sis_undo(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }

    # If we have reached this place, then the command has executed successfully.
    # If there are any errors we would have thrown exceptions already.
    return [];
}

## @name ::sis_reset
##
## @summary Parses the raw output from the "sis reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_reset with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_reset(%args);
## INPUTS TO METHOD CALL 'sis_reset' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'location' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'relative-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sis reset  /vol/vol0 vol0 0
##&nbsp;$VAR1 = [];
##&nbsp;
sub sis_reset(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }

    # If we have reached this place, then the command has executed successfully.
    # If there are any errors we would have thrown exceptions already.
    return [];
}

## @name ::sis_check
##
## @summary Parses the raw output from the "sis check" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sis_check with any of its options as described in the
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_check(%args);
## INPUTS TO METHOD CALL 'sis_check' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'changelog' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose-mode' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delete-checkpoint' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sis check /vol/vol0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => '/vol/vol0',
##&nbsp;            'status' => 'Verification started'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; sis check -c -v /vol/component_source_volume; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Checking fingerprint  ...  11 records
##&nbsp; Number of Segments: 2
##&nbsp; Number of Records:         4,        7
##&nbsp; Contents of Segment: 0 (number of entries: 4)
##&nbsp; 200000100000000        197      14215
##&nbsp;de93130009000000        196      14215
##&nbsp;fbb8668d45000000         35      14215
##&nbsp;fc50587945000000        205      14215
##&nbsp;
##&nbsp; Contents of Segment: 1 (number of entries: 7)
##&nbsp;fbb8668d45000000         76      15883
##&nbsp;fbb8668d45000000         91      15883
##&nbsp;fbb8668d45000000        111      15883
##&nbsp;fbb8668d45000000        188      15883
##&nbsp;fbb8668d45000000        190      15883
##&nbsp;fbb8668d45000000        192      15883
##&nbsp;fbb8668d45000000        194      15883
##&nbsp;
##&nbsp;Checking fingerprint.vvol  ...  11 records
##&nbsp;fbb8668d45000000         35      14215
##&nbsp;fbb8668d45000000         76      15883
##&nbsp;fbb8668d45000000         91      15883
##&nbsp;fbb8668d45000000        111      15883
##&nbsp;fbb8668d45000000        188      15883
##&nbsp;fbb8668d45000000        190      15883
##&nbsp;fbb8668d45000000        192      15883
##&nbsp;fbb8668d45000000        194      15883
##&nbsp;de93130009000000        196      14215
##&nbsp; 200000100000000        197      14215
##&nbsp;fc50587945000000        205      14215
##&nbsp;
##&nbsp;Checking clogdir.0  ...  0 records
##&nbsp; Number of changelog buckets: 1
##&nbsp;
##&nbsp;Checking clogdir.1  ...  0 records
##&nbsp; Number of changelog buckets: 1
##&nbsp;
##&nbsp;Checking fingerprint.stale.sorted  ...  22 records
##&nbsp; 200000100000000        198       14215
##&nbsp; 200000100000000        199       14215
##&nbsp; 200000100000000        200       14215
##&nbsp; 200000100000000        201       14215
##&nbsp; 200000100000000        202       14215
##&nbsp; 200000100000000        203       14215
##&nbsp; 200000100000000        204       14215
##&nbsp;fbb8668d45000000         75       14215
##&nbsp;fbb8668d45000000         76       14215
##&nbsp;fbb8668d45000000         88       14215
##&nbsp;fbb8668d45000000         91       14215
##&nbsp;fbb8668d45000000         95       14215
##&nbsp;fbb8668d45000000        111       14215
##&nbsp;fbb8668d45000000        182       14215
##&nbsp;fbb8668d45000000        188       14215
##&nbsp;fbb8668d45000000        189       14215
##&nbsp;fbb8668d45000000        190       14215
##&nbsp;fbb8668d45000000        191       14215
##&nbsp;fbb8668d45000000        192       14215
##&nbsp;fbb8668d45000000        193       14215
##&nbsp;fbb8668d45000000        194       14215
##&nbsp;fbb8668d45000000        195       14215
##&nbsp;
##&nbsp;Checking Semi Upgraded temporary metafiles (8.0)
##&nbsp;
##&nbsp;Checking Semi Upgraded temporary metafiles (8.1)
##&nbsp;
##&nbsp;Checking Downgraded temporary metafiles (8.1)
##&nbsp;
##&nbsp;Checking Background compression metafiles
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'metafile' => 'fingerprint',
##&nbsp;            'number_of_segments' => '2',
##&nbsp;            'segments' => [
##&nbsp;                            {
##&nbsp;                              'info' => [
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '14215',
##&nbsp;                                            'vvbn' => '197',
##&nbsp;                                            'fingerprint' => '200000100000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '14215',
##&nbsp;                                            'vvbn' => '196',
##&nbsp;                                            'fingerprint' => 'de93130009000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '14215',
##&nbsp;                                            'vvbn' => '35',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '14215',
##&nbsp;                                            'vvbn' => '205',
##&nbsp;                                            'fingerprint' => 'fc50587945000000'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'number_of_entries' => '4',
##&nbsp;                              'segment_id' => '0'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'info' => [
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '76',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '91',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '111',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '188',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '190',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '192',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'cp_count' => '15883',
##&nbsp;                                            'vvbn' => '194',
##&nbsp;                                            'fingerprint' => 'fbb8668d45000000'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'number_of_entries' => '7',
##&nbsp;                              'segment_id' => '1'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '35',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '76',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '91',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '111',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '188',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '190',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '192',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '15883',
##&nbsp;                          'vvbn' => '194',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '196',
##&nbsp;                          'fingerprint' => 'de93130009000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '197',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '205',
##&nbsp;                          'fingerprint' => 'fc50587945000000'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'metafile' => 'fingerprint.vvol'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'metafile' => 'clogdir.0',
##&nbsp;            'number_of_changelog_buckets' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'metafile' => 'clogdir.1',
##&nbsp;            'number_of_changelog_buckets' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '198',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '199',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '200',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '201',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '202',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '203',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '204',
##&nbsp;                          'fingerprint' => '200000100000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '75',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '76',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '88',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '91',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '95',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '111',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '182',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '188',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '189',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '190',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '191',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '192',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '193',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '194',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'cp_count' => '14215',
##&nbsp;                          'vvbn' => '195',
##&nbsp;                          'fingerprint' => 'fbb8668d45000000'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'metafile' => 'fingerprint.stale.sorted'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'metafile' => 'BR'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'metafile' => 'RR'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'metafile' => 'RR'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sis_check(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my @result_array;
    my $i = -1;
    my $j;
    my $k    = -1;
    my $flag = -1;
    my $metafile;
    my ( $limit, $count,$records);
    my $segment = 0;
    my $bucket  = 0;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    if ( $raw_output =~ /Fingerprints of \"(.+)\" are being verified/ ) {
        $i++;
        $result_array[$i]->{'volume'} = $1;
        $result_array[$i]->{'status'} = "Verification started";
    }
    else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            response_object => $resp,
            can_throw       => 1
        );
        foreach my $line (@lines) {

            if ( $line =~ /Checking\s?([\w\.?]*)\s?(.*)\s?records/ ) {
		$i++;
                $j        = 0;
                $count    = 0;
                $limit    = -1;
                $segment  = 0;
                $bucket   = 0;
                $metafile = $1;
		$records  = $2;
		$records =~ s/\.\.\.*//;
		trim( 'string_ptr' => \$records );
		$result_array[$i]->{metafile} = $metafile;
                $result_array[$i]->{No_of_records} = $records;
            } elsif ( $line =~
/Contents\s+of\s+Segment:\s*(\d+)\s+\(number\s+of\s+entries:\s*(\d+)\)/ ) {
                $k++;
                $result_array[$i]->{segments}->[$k]->{number_of_entries} = $2;
                $result_array[$i]->{segments}->[$k]->{segment_id}        = $1;
                $segment                                                 = 1;
                $j                                                       = 0;
                $count                                                   = 0;
            } elsif ($line =~ /^(\w+)\s+(\d+)\s+(\d+)$/
                && $segment == 1 ) {
                $result_array[$i]->{segments}->[$k]->{info}->[$count]
                  ->{fingerprint} = $1;
                $result_array[$i]->{segments}->[$k]->{info}->[$count]->{vvbn} =
                  $2;
                $result_array[$i]->{segments}->[$k]->{info}->[$count]
                  ->{cp_count} = $3;
                $count++;
            } elsif ( $line =~ /^(\w+)\s+(\d+)\s+(\d+)$/ ) {
                $result_array[$i]->{info}->[$count]->{fingerprint} = $1;
                $result_array[$i]->{info}->[$count]->{vvbn}        = $2;
                $result_array[$i]->{info}->[$count]->{cp_count}    = $3;
                $count++;
            } elsif ($line =~ /\s*(.+):\s*(.+)/
                && $line !~ /\(number\s+of\s+entries:\s+\d+\)/
                && $line !~ /Number\s+of\sRecords:\s+.*/ ) {
                $result_array[$i]->{ to_key_name( 'item' => $1 ) } = $2;
            } elsif ( $line =~
/contents\s+of\s+bucket:\s+(\d+)\s+\(number\s+of\s+entries:\s+(\d+)\)/ ) {
                $flag++;
                $result_array[$i]->{buckets}->[$flag]->{number_of_entries} = $2;
                $result_array[$i]->{buckets}->[$flag]->{bucket_id}         = $1;
                $limit                                                     = 0;
                $bucket                                                    = 1;
            } elsif ( $line =~ /\(number\s+of\s+entries:\s+(\d+)\)/ ) {
                $result_array[$i]->{no_of_entries} = $1;
			} elsif ( $line =~
                /(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
                && $bucket == 1 ) {
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{fingerprint} = $1;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{inode} = $2;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]->{fbn} =
                  $3;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{vvbn} = $4;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{generation_no} = $5;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{cp_count} = $6;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{compression_attempted} = $7;
                $result_array[$i]->{buckets}->[$flag]->{info}->[$limit]
                  ->{unprocessed} = $8;
                $limit++;
            } elsif ($line =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
                && $segment == 1 ) {
                $result_array[$i]->{segments}->[$k]->{info}->[$j]
                  ->{fingerprint} = $1;
                $result_array[$i]->{segments}->[$k]->{info}->[$j]->{inode} = $2;
                $result_array[$i]->{segments}->[$k]->{info}->[$j]->{fbn}   = $3;
                $result_array[$i]->{segments}->[$k]->{info}->[$j]
                  ->{generation_no} = $4;
                $result_array[$i]->{segments}->[$k]->{info}->[$j]->{cp_count} =
                  $5;
                $result_array[$i]->{segments}->[$k]->{info}->[$j]
                  ->{compression_attempted} = $6;
                $j++;
            } elsif ( $line =~
                /(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
                && $segment == 0 ) {
                $result_array[$i]->{info}->[$j]->{fingerprint}           = $1;
                $result_array[$i]->{info}->[$j]->{inode}                 = $2;
                $result_array[$i]->{info}->[$j]->{fbn}                   = $3;
                $result_array[$i]->{info}->[$j]->{vvbn}                  = $4;
                $result_array[$i]->{info}->[$j]->{generation_no}         = $5;
                $result_array[$i]->{info}->[$j]->{cp_count}              = $6;
                $result_array[$i]->{info}->[$j]->{compression_attempted} = $7;
                $result_array[$i]->{info}->[$j]->{unprocessed}           = $8;
                $j++;
            } elsif (
                $line =~ /Checking\s?(.*)\s?temporary\s?metafiles\s?\((.*)\)/ ) {
                $i++;
                my $ontap_version = $2;
                if ( $ontap_version == "7.2" || $ontap_version == "7.3" ) {
                    $result_array[$i]->{metafile} = "IC";
                } elsif ( $ontap_version == "8.0" ) {
                    $result_array[$i]->{metafile} = "BR";
                } elsif ( $ontap_version == "8.1" ) {
                    $result_array[$i]->{metafile} = "RR";
                } elsif ( $ontap_version == "8.2" ) {
                    $result_array[$i]->{metafile} = "SN";
                } elsif ( $ontap_version == "8.3" ) {
                    $result_array[$i]->{metafile} = "FS";
                }
            }
        }
    }
    return \@result_array;
  }


## @name ::sis_file_idd
##
## @summary Parses the raw output from the "sis file_idd" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sis_file_idd with any of its options as described in the 
##    corresponding cdef file 'sis.cdefs'
## @synopsis
##      sis_file_idd(%args);
## INPUTS TO METHOD CALL 'sis_file_idd' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'clear' </td> <td> '1' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'get' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'set' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; sis file_idd /vol/c_testvol/filename -g; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                File name			 IDD flag 
##&nbsp;  /vol/c_testvol/filename			 set
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_idd' => 'set',
##&nbsp;            'filepath' => '/vol/c_testvol/filename'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; sis file_idd /vol/c_testvol/filename -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Clearing sis file_idd Succeeded on file /vol/c_testvol/filename 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q test; sis file_idd /vol/c_testvol/filename -s; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Setting sis file_idd Succeeded on file /vol/c_testvol/filename 
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sis_file_idd(%) {
    my %args = @_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $i =0 ;
    my @result_array;

    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines = get_lines(  raw_output      => $raw_output,
                            response_object => $resp,
                            can_throw       => 1
                         );
   if ( $command =~ /-g/ ) {
       foreach my $line (@lines) {
           next if ( $line =~ /^File\s*name.*/ );
           if ( $line =~ /(^\/vol\/\S+)?\s+(\S+\s*\S+)?$/) {
               $result_array[$i]->{'filepath'} = $1;
               $result_array[$i]->{'file_idd'} = $2;
           }
       }
   }
   return  \@result_array;
}

1;
