#
#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode sldiag commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode sldiag commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sldiag" will be sldiag()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::sldiag
##
##    sldiag(%agrs);
##
## @author dl-nacl-dev@netapp.com,Veerabhadraswamy.P@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::7Mode::default::sldiag;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::sldiag_version_show
##
## @summary Parses the raw output from the "sldiag version show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_version_show with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_version_show(%args);
## INPUTS TO METHOD CALL 'sldiag_version_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag version show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;System Level Diagnostic Version A.01.00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'version' => 'A.01.00'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_version_show {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my (@result_array);

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /System Level Diagnostic Version (\S+)/i ) {
            $result_array[0]->{'version'} = $1;
        }
    }
    return \@result_array;
}    #end of sub sldiag_version_show

## @name ::sldiag_device_show
##
## @summary Parses the raw output from the "sldiag device show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_show with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_show(%args);
## INPUTS TO METHOD CALL 'sldiag_device_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DEV              NAME             SELECTION
##&nbsp;---------------- ---------------- ----------------
##&nbsp;nvmem            nvmem            enabled
##&nbsp;nic              e0a              enabled
##&nbsp;nic              e0b              enabled
##&nbsp;nic              e1a              enabled
##&nbsp;nic              e1b              enabled
##&nbsp;nic              e2a              enabled
##&nbsp;nic              e2b              enabled
##&nbsp;nic              e0M              enabled
##&nbsp;acp              acpview          enabled
##&nbsp;acpm             0a.1.A           enabled
##&nbsp;acpm             0a.1.B           enabled
##&nbsp;acpm             0a.2.A           enabled
##&nbsp;acpm             0a.2.B           enabled
##&nbsp;acpm             0a.3.A           enabled
##&nbsp;acpm             0a.3.B           enabled
##&nbsp;acpm             0a.4.A           enabled
##&nbsp;acpm             0a.4.B           enabled
##&nbsp;acpm             0b.1.A           enabled
##&nbsp;acpm             0b.1.B           enabled
##&nbsp;acpm             0b.2.A           enabled
##&nbsp;acpm             0b.2.B           enabled
##&nbsp;acpm             0b.3.A           enabled
##&nbsp;acpm             0b.3.B           enabled
##&nbsp;acpm             0b.4.A           enabled
##&nbsp;acpm             0b.4.B           enabled
##&nbsp;fcal             0c               enabled
##&nbsp;fcal             0d               enabled
##&nbsp;mem              mem              enabled
##&nbsp;storage          0a               enabled
##&nbsp;storage          0b               enabled
##&nbsp;bootmedia        bootmedia        enabled
##&nbsp;environmentals   environmentals   enabled
##&nbsp;serviceProcessor serviceProcessor enabled
##&nbsp;pci              pci              disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'nvmem',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nvmem'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e0a',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e0b',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e1a',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e1b',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e2a',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e2b',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'e0M',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'nic'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'acpview',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acp'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.1.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.1.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.2.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.2.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.3.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.3.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.4.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a.4.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.1.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.1.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.2.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.2.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.3.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.3.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.4.A',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b.4.B',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'acpm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0c',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'fcal'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0d',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'fcal'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'mem',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'mem'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0a',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'storage'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => '0b',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'storage'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'bootmedia',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'bootmedia'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'environmentals',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'environmentals'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'serviceProcessor',
##&nbsp;            'selection' => 'enabled',
##&nbsp;            'device' => 'serviceProcessor'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'name' => 'pci',
##&nbsp;            'selection' => 'disabled',
##&nbsp;            'device' => 'pci'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_device_show {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my ( @result_array, @values );
    my $count = 0;

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
        first_line      => '--------------'
    );
    shift @lines;
    foreach my $line (@lines) {
        if ( $line =~ /(\w+)\s+(\S+)\s+(\w+)/ ) {
            $result_array[$count]->{'device'}    = $1;
            $result_array[$count]->{'name'}      = $2;
            $result_array[$count]->{'selection'} = $3;
            $count++;
        }
    }
    return \@result_array;
}    #end of sub sldiag_device_show

## @name ::sldiag_device_types
##
## @summary Parses the raw output from the "sldiag device types" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_types with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_types(%args);
## INPUTS TO METHOD CALL 'sldiag_device_types' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device types
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;nvmem
##&nbsp;nic
##&nbsp;acp
##&nbsp;acpm
##&nbsp;fcal
##&nbsp;mem
##&nbsp;storage
##&nbsp;bootmedia
##&nbsp;environmentals
##&nbsp;serviceProcessor
##&nbsp;pci
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_types' => [
##&nbsp;                                'nvmem',
##&nbsp;                                'nic',
##&nbsp;                                'acp',
##&nbsp;                                'acpm',
##&nbsp;                                'fcal',
##&nbsp;                                'mem',
##&nbsp;                                'storage',
##&nbsp;                                'bootmedia',
##&nbsp;                                'environmentals',
##&nbsp;                                'serviceProcessor',
##&nbsp;                                'pci'
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_device_types {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my ( @result_array, $line );
    my %internal_hash;

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /Initializing/ ) {
            next;
        } else {
            push @{ $internal_hash{'device_types'} }, $line;
        }
    }
    push( @result_array, \%internal_hash );
    return \@result_array;
}    #end of sub sldiag_device_types

## @name ::sldiag_device_status
##
## @summary Parses the raw output from the "sldiag device status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_status with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_status(%args);
## INPUTS TO METHOD CALL 'sldiag_device_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-mfg' </td> <td> 'mfg' </td> <td> '1' </td> </tr>
## <tr> <td> '-long' </td> <td> 'long' </td> <td> '1' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-default' </td> <td> 'default' </td> <td> '1' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> '-state' </td> <td> 'state' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device status
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;To see details use: sldiag device status -long
##&nbsp;Device       TestName         StartDate           EndDate             Status     LOOP
##&nbsp;----------   ---------------- ------------------- ------------------- ---------- --------
##&nbsp;nvmem        NVMEM details    Thu Mar 28 07:51:05 Thu Mar 28 07:51:05 PASSED     1/1
##&nbsp;nvmem        Read test        Thu Mar 28 07:51:06 Thu Mar 28 07:51:07 PASSED     1/1
##&nbsp;nic          NIC Detection    Thu Mar 28 07:51:09 Thu Mar 28 07:51:09 Completed  1/1
##&nbsp;acpview      ACP show         Thu Mar 28 07:51:09 Thu Mar 28 07:51:09 Completed  1/1
##&nbsp;nic          NIC Packet Test  Thu Mar 28 07:51:10 Thu Mar 28 07:59:03 PASSED     1/1
##&nbsp;acpm         ACP post data    Thu Mar 28 07:51:13 Thu Mar 28 07:51:25 ERROR      1/1
##&nbsp;0c           Fcal Loopback Test
##&nbsp;                              Thu Mar 28 07:51:13 Thu Mar 28 07:54:00 PASSED     1/1
##&nbsp;0d           Fcal Loopback Test
##&nbsp;                              Thu Mar 28 07:51:13 Thu Mar 28 07:54:00 PASSED     1/1
##&nbsp;mem          Random Address Mem Test
##&nbsp;                              Thu Mar 28 07:51:15 Thu Mar 28 08:05:03 PASSED     1/1
##&nbsp;bootmedia    Media Info       Thu Mar 28 07:51:16 Thu Mar 28 07:51:16 PASSED     1/1
##&nbsp;environmentals
##&nbsp;             Sensor Test      Thu Mar 28 07:51:16 Thu Mar 28 07:51:18 PASSED     1/1
##&nbsp;serviceProcessor
##&nbsp;             Service Processor
##&nbsp;                              Thu Mar 28 07:51:16 Thu Mar 28 07:51:16 PASSED     1/1
##&nbsp;0a           Disk Seq Read test
##&nbsp;                              Thu Mar 28 08:05:04 Thu Mar 28 08:08:09 PASSED     1/1
##&nbsp;0b           Disk Seq Read test
##&nbsp;                              Thu Mar 28 08:08:10 Thu Mar 28 08:11:31 ERROR      1/1
##&nbsp;mem          Random Data Mem Test
##&nbsp;                              Thu Mar 28 08:08:10 Thu Mar 28 08:10:42 PASSED     1/1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:05',
##&nbsp;            'test_name' => 'NVMEM details',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:05',
##&nbsp;            'device' => 'nvmem',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:07',
##&nbsp;            'test_name' => 'Read test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:06',
##&nbsp;            'device' => 'nvmem',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'Completed',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:09',
##&nbsp;            'test_name' => 'NIC Detection',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:09',
##&nbsp;            'device' => 'nic',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'Completed',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:09',
##&nbsp;            'test_name' => 'ACP show',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:09',
##&nbsp;            'device' => 'acpview',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:59:03',
##&nbsp;            'test_name' => 'NIC Packet Test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:10',
##&nbsp;            'device' => 'nic',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'ERROR',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:25',
##&nbsp;            'test_name' => 'ACP post data',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:13',
##&nbsp;            'device' => 'acpm',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:54:00',
##&nbsp;            'test_name' => 'Fcal Loopback Test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:13',
##&nbsp;            'device' => '0c',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:54:00',
##&nbsp;            'test_name' => 'Fcal Loopback Test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:13',
##&nbsp;            'device' => '0d',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 08:05:03',
##&nbsp;            'test_name' => 'Random Address Mem Test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:15',
##&nbsp;            'device' => 'mem',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:16',
##&nbsp;            'test_name' => 'Media Info',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:16',
##&nbsp;            'device' => 'bootmedia',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:18',
##&nbsp;            'test_name' => 'Sensor Test',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:16',
##&nbsp;            'device' => 'environmentals',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 07:51:16',
##&nbsp;            'test_name' => 'Service Processor',
##&nbsp;            'start_date' => 'Thu Mar 28 07:51:16',
##&nbsp;            'device' => 'serviceProcessor',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 08:08:09',
##&nbsp;            'test_name' => 'Disk Seq Read test',
##&nbsp;            'start_date' => 'Thu Mar 28 08:05:04',
##&nbsp;            'device' => '0a',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'ERROR',
##&nbsp;            'end_date' => 'Thu Mar 28 08:11:31',
##&nbsp;            'test_name' => 'Disk Seq Read test',
##&nbsp;            'start_date' => 'Thu Mar 28 08:08:10',
##&nbsp;            'device' => '0b',
##&nbsp;            'loop' => '1/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'PASSED',
##&nbsp;            'end_date' => 'Thu Mar 28 08:10:42',
##&nbsp;            'test_name' => 'Random Data Mem Test',
##&nbsp;            'start_date' => 'Thu Mar 28 08:08:10',
##&nbsp;            'device' => 'mem',
##&nbsp;            'loop' => '1/1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_device_status {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my ( @result_array, @finalarray );
    my %test_info;
    my ( $key, $sub_key );
    my $secondline;
    my $i = 0;
    my $j = 0;
    my $k = 0;

    _check_error( raw_output => $output, response_object => $resp );
    if ( $command =~ /^sldiag device status .*-mfg/ ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
        );
        foreach my $line (@lines) {
            if ( $line
                =~ /(^DEVTYPE\:|^NAME\:|^START DATE\:|^STATUS\:|^END DATE\:|^LOOP\:)/i
                )
            {
                $key = $1;
                if ( $line =~ /(\w+\s+\w+|\w+)\:\s+(.*)/ ) {
                    $result_array[$i]->{ to_key_name( item => $key ) } = $2;
                }
                $sub_key = 'interconnect_devtest' . $i;
            } elsif ( $line
                =~ /^STARTING:\s+(Interconnect).*\s+(test\s+\w+)\s+(\w+)\:/ )
            {
                $result_array[$i]->{'interface'} = $3;
            } elsif ( $line
                =~ /(Basic.*)\:\s+(\w+)\s+(.*generator)\:\s+(\w+)\s+(offset)\:\s+(\w+)\s+(Errors)\:\s+(\w+)/i
                )
            {
                $test_info{ to_key_name( item => $1 ) } = $2;
                $test_info{ to_key_name( item => $3 ) } = $4;
                $test_info{ to_key_name( item => $5 ) } = $6;
                $test_info{ to_key_name( item => $7 ) } = $8;
            } elsif ( $line
                =~ /(^\w.*):\s+.*\s+\-\s+(Basic Pattern Algorithm)\s+(\w+)/ )
            {
                $test_info{'test_status'} = $1;
                $test_info{ to_key_name( item => $2 ) } = $3;
                push( @{ $result_array[$i]->{$sub_key} }, {%test_info} );
                %test_info = ();
            } elsif ( $line =~ /^(\w+)\:\s+Cannot.*/ ) {
                $test_info{'test_status'} = $1;
                push( @{ $result_array[$i]->{$sub_key} }, {%test_info} );
                %test_info = ();
            } elsif ( $line
                =~ /(^\w+)\s+mtu\s+(\w+)\s+ether\s+(\S+)\s+(\w+)\s+(\w+)\s+\w+\s+(\S+)/
                )
            {
                $result_array[$i]->{$sub_key}->[$j]->{'interface'} = $1;
                $result_array[$i]->{$sub_key}->[$j]->{'mtu'}       = $2;
                $result_array[$i]->{$sub_key}->[$j]->{'mac'}       = $3;
                $result_array[$i]->{$sub_key}->[$j]->{$4}          = $5;
                my @info = split( /-/, $6 );
                my %cable_info;
                $cable_info{'mediatype'} = $info[0];
                $cable_info{'cable'}     = $info[1];
                $cable_info{'duplex'}    = $info[2];
                $cable_info{'link'}      = $info[3];
                push(
                    @{  $result_array[$i]->{$sub_key}->[$j]->{'media_spec'}
                        },
                    \%cable_info
                );
                $j++;
            } elsif ( $line =~ /Interface\s+(\w+)\s+\w+\s+(\w+)\s+\!/ ) {
                $result_array[$i]->{$sub_key}->[$j]->{'interface'} = $1;
                $result_array[$i]->{$sub_key}->[$j]->{'network'}   = $2;
            } elsif ( $line
                =~ /TEST END --------------------------------------------/ )
            {
                $i++;
                $j = 0;
            }
        }
    } elsif ( $command =~ /^sldiag device status.*-long/ ) {
        my @lines
            = get_lines( raw_output => $output, response_object => $resp );
        foreach my $line (@lines) {
            if ( $line
                =~ /(^DEVTYPE\:|^NAME\:|^START DATE\:|^STATUS\:|^END DATE\:|^LOOP\:)/i
                )
            {
                $key = $1;
                if ( $line =~ /(\w+\s+\w+|\w+)\:\s+(.*)/ ) {
                    $result_array[$i]->{ to_key_name( item => $key ) } = $2;
                }
            } elsif ( $line
                =~ /(\w+)\s+(\w+)\s+(\d+)\s+\w+\s+(\S+)\s+(\w+)\s+(\w+)\s+\w+\s+(\S+)/
                )
            {
                $result_array[$i]->{'interfaces'}->[$j]->{'interface'} = $1;
                $result_array[$i]->{'interfaces'}->[$j]->{$2}          = $3;
                $result_array[$i]->{'interfaces'}->[$j]->{'mac'}       = $4;
                $result_array[$i]->{'interfaces'}->[$j]->{$5}          = $6;
                my @info = split( /-/, $7 );
                my %cable_info;
                $cable_info{'mediatype'} = $info[0];
                $cable_info{'cable'}     = $info[1];

                if ( $info[2] eq 'down' || $info[$2] eq 'down' ) {
                    $cable_info{'duplex'} = 'undef';
                    $cable_info{'link'}   = $info[2];
                } else {
                    $cable_info{'duplex'} = $info[2];
                    $cable_info{'link'}   = $info[3];
                }
                push(
                    @{  $result_array[$i]->{'interfaces'}->[$j]
                            ->{'media_spec'}
                        },
                    \%cable_info
                );
                $j++;
            } elsif ( $line =~ /\w+\s+(\w+)\s+\w+\s+(\w+)\s+\!/ ) {
                $result_array[$i]->{'interfaces'}->[$j]->{'interface'} = $1;
                $result_array[$i]->{'interfaces'}->[$j]->{'network'}   = $2;
                $j++;
            } elsif ( $line =~ /(\w+)\s+\w+\s+(\w+)\s+\w+\s+partners/ ) {
                $result_array[$i]->{'partners'} = [ $1, $2 ];
            } elsif ( $line
                =~ /^(\w+)\s+\:\s+\w+\s+\w+\s+\:(\d+)\s+\w+\s+\w+\s+\:(\d+)/ )
            {
                $result_array[$i]->{'interfaces'}->[$k]->{'interface'} = $1;
                $result_array[$i]->{'interfaces'}->[$k]->{'input_packets'}
                    = $2;
                $result_array[$i]->{'interfaces'}->[$k]->{'output_packets'}
                    = $3;
            } elsif (
                $line =~ /^\w+\:\s+(\d+)\s+\w+\s+\w+\,\s+(\d+)\s+\w+\s+\w+/ )
            {
                $result_array[$i]->{'interfaces'}->[$k]->{'input_errors'}
                    = $1;
                $result_array[$i]->{'interfaces'}->[$k]->{'output_errors'}
                    = $2;
                $k++;
            } elsif ( $line
                =~ /TEST END --------------------------------------------/ )
            {
                $i++;
                $j = 0;
                $k = 0;
            }
        }
    } elsif ( $command
        =~ /^sldiag device status$|^sldiag device status.*-state|sldiag device status.*-default/
        )
    {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
            first_line => 'To see details use: sldiag device status -long'
        );    #For converting the multiple output lines single output lines
        while ( my $line = shift(@lines) ) {
            if ( $line !~ /\d+\/\d+/ ) {
                $secondline = shift @lines;
                $line = join( ' ', $line, $secondline );
                if ( $line !~ /\d+\/\d+/ ) {
                    $secondline = shift @lines;
                    $line = join( ' ', $line, $secondline );
                }
                push @finalarray, $line;
            } else {
                push @finalarray, $line;
            }
        }
        for ( my $i = 0; $i < @finalarray; $i++ ) {
            my %hash_status;
            if ( $finalarray[$i]
                =~ /^(\w+)\s+(\w+\s.*\w+)\s+(Sun|Mon|Tue|Wed|Thu|Fri|Sat)\s+(\w+\s+\d+\s+\d+\:\d+\:\d+)\s+(Sun|Mon|Tue|Wed|Thu|Fri|Sat)\s+(\w+\s+\d+\s+\d+\:\d+\:\d+)\s+(\w+)\s+(\d+\/\d+)$/
                )
            {
                $hash_status{'device'}     = $1;
                $hash_status{'test_name'}  = $2;
                $hash_status{'start_date'} = join( ' ', $3, $4 );
                $hash_status{'end_date'}   = join( ' ', $5, $6 );
                $hash_status{'status'}     = $7;
                $hash_status{'loop'}       = $8;
                push( @result_array, \%hash_status );
            }
        }
    }

    return \@result_array;
}    #end of sub sldiag_device_status

## @name ::sldiag_test_show
##
## @summary Parses the raw output from the "sldiag test show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_test_show with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_test_show(%args);
## INPUTS TO METHOD CALL 'sldiag_test_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag test show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;INDEX DEVTYPE         TEST NAME                 SELECTION
##&nbsp;----- --------------- ------------------------- ---------
##&nbsp;0     nvmem           NVMEM details             enabled
##&nbsp;1     nvmem           Read test                 enabled
##&nbsp;2     nvmem           Chkrbrd test              disabled
##&nbsp;3     nvmem           Rndm data test            disabled
##&nbsp;4     nvmem           User data test            disabled
##&nbsp;5     nvmem           Ramp data test            disabled
##&nbsp;6     nvmem           Battery toggle            disabled
##&nbsp;15    nic             NIC Detection             enabled
##&nbsp;16    nic             NIC Packet Test           enabled
##&nbsp;24    acp             ACP show                  enabled
##&nbsp;25    acpm            ACP post data             enabled
##&nbsp;26    acpm            ACP voltage_stat          enabled
##&nbsp;27    acpm            ACP exp. reset            enabled
##&nbsp;28    acpm            ACP exp.pwr cycle         disabled
##&nbsp;53    fcal            Fcal Loopback Test        enabled
##&nbsp;55    mem             Memory Details            disabled
##&nbsp;56    mem             Random Address Mem Test   enabled
##&nbsp;57    mem             Random Data Mem Test      enabled
##&nbsp;58    mem             Checker Board Mem Test    disabled
##&nbsp;59    mem             User Data Mem Test        disabled
##&nbsp;60    mem             Scrubber Mem Test         disabled
##&nbsp;61    mem             Ramp Data Mem Test        disabled
##&nbsp;62    mem             Ramp Byte Mem Test        disabled
##&nbsp;63    mem             Ramp Step Data Mem Test   disabled
##&nbsp;64    mem             Stuck Fault Mem Test      disabled
##&nbsp;65    storage         Disk RCA test             disabled
##&nbsp;66    storage         Disk Seq Read test        enabled
##&nbsp;67    storage         Disk Seq Write test       disabled
##&nbsp;68    storage         Disk Seq Alternating test disabled
##&nbsp;69    storage         Disk Seq Write Verify test disabled
##&nbsp;228   bootmedia       Media Info                enabled
##&nbsp;229   environmentals  Sensor Test               enabled
##&nbsp;230   serviceProcessor Service Processor         enabled
##&nbsp;231   pci             PCI Card test             disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'index' => '0',
##&nbsp;            'testname' => 'NVMEM details',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '1',
##&nbsp;            'testname' => 'Read test',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '2',
##&nbsp;            'testname' => 'Chkrbrd test',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '3',
##&nbsp;            'testname' => 'Rndm data test',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '4',
##&nbsp;            'testname' => 'User data test',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '5',
##&nbsp;            'testname' => 'Ramp data test',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '6',
##&nbsp;            'testname' => 'Battery toggle',
##&nbsp;            'devtype' => 'nvmem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '15',
##&nbsp;            'testname' => 'NIC Detection',
##&nbsp;            'devtype' => 'nic',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '16',
##&nbsp;            'testname' => 'NIC Packet Test',
##&nbsp;            'devtype' => 'nic',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '24',
##&nbsp;            'testname' => 'ACP show',
##&nbsp;            'devtype' => 'acp',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '25',
##&nbsp;            'testname' => 'ACP post data',
##&nbsp;            'devtype' => 'acpm',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '26',
##&nbsp;            'testname' => 'ACP voltage_stat',
##&nbsp;            'devtype' => 'acpm',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '27',
##&nbsp;            'testname' => 'ACP exp. reset',
##&nbsp;            'devtype' => 'acpm',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '28',
##&nbsp;            'testname' => 'ACP exp.pwr cycle',
##&nbsp;            'devtype' => 'acpm',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '53',
##&nbsp;            'testname' => 'Fcal Loopback Test',
##&nbsp;            'devtype' => 'fcal',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '55',
##&nbsp;            'testname' => 'Memory Details',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '56',
##&nbsp;            'testname' => 'Random Address Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '57',
##&nbsp;            'testname' => 'Random Data Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '58',
##&nbsp;            'testname' => 'Checker Board Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '59',
##&nbsp;            'testname' => 'User Data Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '60',
##&nbsp;            'testname' => 'Scrubber Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '61',
##&nbsp;            'testname' => 'Ramp Data Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '62',
##&nbsp;            'testname' => 'Ramp Byte Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '63',
##&nbsp;            'testname' => 'Ramp Step Data Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '64',
##&nbsp;            'testname' => 'Stuck Fault Mem Test',
##&nbsp;            'devtype' => 'mem',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '65',
##&nbsp;            'testname' => 'Disk RCA test',
##&nbsp;            'devtype' => 'storage',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '66',
##&nbsp;            'testname' => 'Disk Seq Read test',
##&nbsp;            'devtype' => 'storage',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '67',
##&nbsp;            'testname' => 'Disk Seq Write test',
##&nbsp;            'devtype' => 'storage',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '68',
##&nbsp;            'testname' => 'Disk Seq Alternating test',
##&nbsp;            'devtype' => 'storage',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '69',
##&nbsp;            'testname' => 'Disk Seq Write Verify test',
##&nbsp;            'devtype' => 'storage',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '228',
##&nbsp;            'testname' => 'Media Info',
##&nbsp;            'devtype' => 'bootmedia',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '229',
##&nbsp;            'testname' => 'Sensor Test',
##&nbsp;            'devtype' => 'environmentals',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '230',
##&nbsp;            'testname' => 'Service Processor',
##&nbsp;            'devtype' => 'serviceProcessor',
##&nbsp;            'selection' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'index' => '231',
##&nbsp;            'testname' => 'PCI Card test',
##&nbsp;            'devtype' => 'pci',
##&nbsp;            'selection' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_test_show {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my (@result_array);
    my $count = 0;

    _check_error(
        raw_output      => $output,
        response_object => $resp,

    );

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
        first_line      => '--------------'
    );
    shift @lines;
    foreach my $line (@lines) {
        if ( $line =~ /(\d+)\s+(\w+)\s+(\w+.*\w)\s+(enabled|disabled)/ ) {
            $result_array[$count]->{'index'}     = $1;
            $result_array[$count]->{'devtype'}   = $2;
            $result_array[$count]->{'testname'}  = $3;
            $result_array[$count]->{'selection'} = $4;
        }
        $count++;
    }
    return \@result_array;
}    #end of sub sldiag_test_show

## @name ::sldiag_option_show
##
## @summary Parses the raw output from the "sldiag option show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_option_show with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_option_show(%args);
## INPUTS TO METHOD CALL 'sldiag_option_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'option' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag option show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Type sldiag option help <name> for a description:
##&nbsp;OPTION NAME     MODIFIER        VALUE(s)
##&nbsp;--------------  --------------- ---------------
##&nbsp;display-style                   default
##&nbsp;display-dest                    log
##&nbsp;errstop                         enabled
##&nbsp;loop                            1
##&nbsp;signaldone                      enabled
##&nbsp;seed                            unset
##&nbsp;errstopall                      disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'option_name' => 'display-style',
##&nbsp;            'values' => 'default',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'display-dest',
##&nbsp;            'values' => 'log',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'errstop',
##&nbsp;            'values' => 'enabled',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'loop',
##&nbsp;            'values' => '1',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'signaldone',
##&nbsp;            'values' => 'enabled',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'seed',
##&nbsp;            'values' => 'unset',
##&nbsp;            'modifier' => ''
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'option_name' => 'errstopall',
##&nbsp;            'values' => 'disabled',
##&nbsp;            'modifier' => ''
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sldiag_option_show {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my ( @result_array, @values );
    my $count = 0;

    _check_error(
        raw_output      => $output,
        response_object => $resp,

    );

    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
        first_line      => '--------------'
    );
    shift @lines;
    for ( my $i = 0; $i < @lines; $i++ ) {
        @values = split( '\s+', $lines[$i] );
        $result_array[$count]->{'option_name'} = $values[0];
        if ( $values[2] eq undef ) {
            $result_array[$count]->{'modifier'} = '';
            $result_array[$count]->{'values'}   = $values[1];
        } else {
            $result_array[$count]->{'modifier'} = $values[1];
            $result_array[$count]->{'values'}   = $values[2];
        }
        $count++;
    }
    return \@result_array;
}    # end of sldiag_option_show

## @name ::sldiag_option_modify
##
## @summary Parses the raw output from the "sldiag option modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_option_modify with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_option_modify(%args);
## INPUTS TO METHOD CALL 'sldiag_option_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'option' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag option modify signaldone enable
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_option_modify {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    # end of sldiag_option_modify

## @name ::sldiag_device_run
##
## @summary Parses the raw output from the "sldiag device run" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_run with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_run(%args);
## INPUTS TO METHOD CALL 'sldiag_device_run' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sldiag device run
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = sldiag device run
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_device_run {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    # end of sldiag_device_run

## @name ::sldiag_device_clearstatus
##
## @summary Parses the raw output from the "sldiag device clearstatus" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_clearstatus with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_clearstatus(%args);
## INPUTS TO METHOD CALL 'sldiag_device_clearstatus' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device clearstatus
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_device_clearstatus {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    # end of sldiag_device_clearstatus

## @name ::sldiag_device_stop
##
## @summary Parses the raw output from the "sldiag device stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_stop with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_stop(%args);
## INPUTS TO METHOD CALL 'sldiag_device_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device stop
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_device_stop {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    # end of sldiag_device_stop

## @name ::sldiag_device_modify
##
## @summary Parses the raw output from the "sldiag device modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_modify with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_modify(%args);
## INPUTS TO METHOD CALL 'sldiag_device_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-selection' </td> <td> 'selection' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device modify -selection disable -dev nic -name e1a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_device_modify {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    #end of sldiag_device_modify

## @name ::sldiag_test_modify
##
## @summary Parses the raw output from the "sldiag test modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_test_modify with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_test_modify(%args);
## INPUTS TO METHOD CALL 'sldiag_test_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-index' </td> <td> 'index' </td> <td> '0' </td> </tr>
## <tr> <td> '-selection' </td> <td> 'selection' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag test modify -dev nic -selection disable -index 18
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;sldiag device run
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_test_modify {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    #end of sldiag_test_modify

## @name ::sldiag_device_rescan
##
## @summary Parses the raw output from the "sldiag device rescan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_device_rescan with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_device_rescan(%args);
## INPUTS TO METHOD CALL 'sldiag_device_rescan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag device rescan
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Initializing . . .
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_device_rescan {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    #end of sldiag_device_rescan

## @name ::sldiag_test_reorder
##
## @summary Parses the raw output from the "sldiag test reorder" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sldiag_test_reorder with any of its options as described in the
##    corresponding cdef file 'sldiag.cdefs'
## @synopsis
##      sldiag_test_reorder(%args);
## INPUTS TO METHOD CALL 'sldiag_test_reorder' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-index' </td> <td> 'index' </td> <td> '0' </td> </tr>
## <tr> <td> '-selection' </td> <td> 'selection' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev' </td> <td> 'device' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sldiag test reorder -dev nic -index 16,15 -selection enable
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub sldiag_test_reorder {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
}    #end of sldiag_test_reorder

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /SLDIAG: Cannot .*running/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /sldiag: missing required .*|sldiag: invalid command format.*/i )
    {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /sldiag: invalid .*|ERROR: specified option could not .*|ERROR:.*test index.*/i
        )
    {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
