#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/slist.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode slist command output
##
## @description
##    This is a library to parse the raw output of 7Mode slist command.
##
## @synopsis
##    use slist;
##
##    slist(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::slist;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command       = $args{command};
    my $error;
    if ( $raw_output =~ /[Uu]sage:\s(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage or dependencies";
    } elsif ( $raw_output =~ /(.*): Invalid path/ ) {
        $error          = "InvalidParamValue";
        $$error_message = "Command failed due to an invalid path specified";
    }
    return $error;
}

sub _check_for_undefined {
    my ( $inode, $file_size, $name, $resp ) = @_;
    if (    $inode =~ /\d+/
        and $file_size =~ /\d+/
        and $name !~ /Inode|undef/i )
    {
        return 1;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "One of Inode#, File size or Name is not defined",
            response_object => $resp
        );
    }

}
## @name ::slist
##
## @summary Parses the raw output from the "slist" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from slist with any of its options as described in the
##    corresponding cdef file 'slist.cdefs'
## @synopsis
##      slist(%args);
## INPUTS TO METHOD CALL 'slist' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; slist /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Inode#        File size      Name
##&nbsp;-----------  ---------------   ----
##&nbsp;        64              4096   /vol/vol0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_size' => '4096',
##&nbsp;            'name' => '/vol/vol0',
##&nbsp;            'inode' => '64'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; slist /vol/vol0 /vol/vol1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Inode#        File size      Name
##&nbsp;-----------  ---------------   ----
##&nbsp;        64              4096   /vol/vol0
##&nbsp;
##&nbsp;  Inode#        File size      Name
##&nbsp;-----------  ---------------   ----
##&nbsp;        64              4096   /vol/vol1
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_size' => '4096',
##&nbsp;            'name' => '/vol/vol0',
##&nbsp;            'inode' => '64'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'file_size' => '4096',
##&nbsp;            'name' => '/vol/vol1',
##&nbsp;            'inode' => '64'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; slist /vol/vol1/lun; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Inode#        File size      Name
##&nbsp;-----------  ---------------   ----
##&nbsp;        96           4194304   /vol/vol1/lun
##&nbsp;        98               206   	attributes
##&nbsp;        99                 0   	lunmap
##&nbsp;       100                 0   	prefix
##&nbsp;       101                 0   	suffix
##&nbsp;       102                 0   	persistent.reservations
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'extended-inode-info' => [
##&nbsp;                                       {
##&nbsp;                                         'file_size' => '206',
##&nbsp;                                         'name' => 'attributes',
##&nbsp;                                         'inode' => '98'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'file_size' => '0',
##&nbsp;                                         'name' => 'lunmap',
##&nbsp;                                         'inode' => '99'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'file_size' => '0',
##&nbsp;                                         'name' => 'prefix',
##&nbsp;                                         'inode' => '100'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'file_size' => '0',
##&nbsp;                                         'name' => 'suffix',
##&nbsp;                                         'inode' => '101'
##&nbsp;                                       },
##&nbsp;                                       {
##&nbsp;                                         'file_size' => '0',
##&nbsp;                                         'name' => 'persistent.reservations',
##&nbsp;                                         'inode' => '102'
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;            'file_size' => '4194304',
##&nbsp;            'name' => '/vol/vol1/lun',
##&nbsp;            'inode' => '96'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub slist {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = 'Inode#\s+File size\s+Name';

    if ( $output =~ /$first_line/ ) {
        $output =~ s/[^a-zA-Z0-9\s+\/\.\-\_]//g;
        my @arr_2 = split( /\s+/, $output );
        my $len = @arr_2;
        my $i;
        my $l = 0;
        for ( $i = 1; $i < $len; $i++ ) {

            my %hash1 = ();
            my @xinode_arr;
            while ( $arr_2[$i] !~ /Inode|File|size|Name|----/
                and $i < $len - 1 )
            {
                my $inode     = $arr_2[ $i++ ];
                my $file_size = $arr_2[$i];
                if ( $arr_2[ ++$i ] =~ /\// ) {
                    $hash1{inode}     = $inode;
                    $hash1{file_size} = $file_size;
                    $hash1{name}      = $arr_2[$i];
                    &_check_for_undefined( $hash1{inode}, $hash1{file_size},
                        $hash1{name}, $resp );
                } else {
                    my %hash2 = ();
                    $hash2{inode}     = $inode;
                    $hash2{file_size} = $file_size;
                    $hash2{name}      = $arr_2[$i];
                    push( @xinode_arr, \%hash2 );
                    $l++;
                    &_check_for_undefined( $hash2{inode}, $hash2{file_size},
                        $hash2{name}, $resp );
                }
                $i++;
                if (@xinode_arr) {
                    $hash1{'extended-inode-info'} = \@xinode_arr;
                }
            }
            if (%hash1) {
                push( @result_array, \%hash1 );
            }
        }
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $resp
        );
    }

    return \@result_array;
}

1;
