#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/smf.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode smf command output
##
## @description
##    This is a library to parse the raw output of 7Mode smf command.
##
## @synopsis
##
##    smf(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::smf;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException;

## @name ::smf_show
##
## @summary Parses the raw output from the "smf show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from smf_show with any of its options as described in the
##    corresponding cdef file 'smf.cdefs'
## @synopsis
##      smf_show(%args);
## INPUTS TO METHOD CALL 'smf_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'table' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'abort' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'list' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bulk-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'max-rows' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'rpc-timeout' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'rpc-connect-timeout' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = smf show iomem_fru_driver_2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;slot flash-vendor channels-per-board packages-per-channel lun-per-package lun-per-chip-enable blocks-per-lun pages-per-block page-size planes-per-lun flex-planes bch-sector-size bch-checksum-size lun-interleaving write-interrupt random-cache-read channel-speed erase-power-cost write-data-power-cost page-program-power-cost page-read-power-cost read-data-power-cost front-bisection-power-limit back-bisection-power-limit flash-package-power-limit flash-endurance-rating flash-id          \r\n
##&nbsp;---- ------------ ------------------ -------------------- --------------- ------------------- -------------- --------------- --------- -------------- ----------- --------------- ----------------- ---------------- --------------- ----------------- ------------- ---------------- --------------------- ----------------------- -------------------- -------------------- --------------------------- -------------------------- ------------------------- ---------------------- ----------------- \r\n
##&nbsp;1    Micron       32                 2                    2               1                   4096           256             8192      2              TRUE        1040            40                TRUE             TRUE            TRUE              200           7                9                     9                       11                   11                   375                         375                        87                        10000                  2c:88:04:4b:a9:01
##&nbsp;1 entries were displayed
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lun_per_chip_enable' => '1',
##&nbsp;            'page_read_power_cost' => '11',
##&nbsp;            'lun_per_package' => '2',
##&nbsp;            'channels_per_board' => '32',
##&nbsp;            'flash_id' => '2c:88:04:4b:a9:01',
##&nbsp;            'bch_sector_size' => '1040',
##&nbsp;            'flex_planes' => 'TRUE',
##&nbsp;            'planes_per_lun' => '2',
##&nbsp;            'blocks_per_lun' => '4096',
##&nbsp;            'flash_endurance_rating' => '10000',
##&nbsp;            'back_bisection_power_limit' => '375',
##&nbsp;            'slot' => '1',
##&nbsp;            'bch_checksum_size' => '40',
##&nbsp;            'page_program_power_cost' => '9',
##&nbsp;            'flash_vendor' => 'Micron',
##&nbsp;            'write_data_power_cost' => '9',
##&nbsp;            'channel_speed' => '200',
##&nbsp;            'flash_package_power_limit' => '87',
##&nbsp;            'packages_per_channel' => '2',
##&nbsp;            'pages_per_block' => '256',
##&nbsp;            'page_size' => '8192',
##&nbsp;            'write_interrupt' => 'TRUE',
##&nbsp;            'random_cache_read' => 'TRUE',
##&nbsp;            'lun_interleaving' => 'TRUE',
##&nbsp;            'read_data_power_cost' => '11',
##&nbsp;            'front_bisection_power_limit' => '375',
##&nbsp;            'erase_power_cost' => '7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub smf_show {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;

    @head = split( /\s+/, $lines[0] );
    for ( my $i = 2 ; $i <= $#lines ; $i++ ) {
        @values = split( /\s+/, $lines[$i] );
        if ( $lines[$i] !~ /entries were displayed/ ) {
            for ( my $j = 0 ; $j <= $#head ; $j++ ) {
                $head[$j] =~ s/-/_/g;
                $result_array[$count]->{ to_key_name( item => $head[$j] ) } =
                  $values[$j];
            }
            $count++;
        }
    }
    return \@result_array;
} ## end sub smf

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /Table\s+.*\s+does\s+not\s+exist/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /This table is currently empty.*/ ) {
        $error         = "NoMatchingEntries";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

