#
#Copyright (c) 2014 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse smnadmin command output
##
## @description
##    This is a library to parse the raw outputs of 7Mode smnadmin commands.
##    The function name will be the same as command name.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::smnadmin;
##
## smnadmin_initiate_selection();
## smnadmin_master_name();
## smnadmin_master_dblade();
## smnadmin_status();
##
## @author dsubra@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::smnadmin;

use strict;
use warnings;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(.*unavailable.*)/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return undef;
} ## end sub _check_error

## @name ::smnadmin_initiate_selection
##
## @summary Parses the raw output from the "smnadmin_initiate_selection" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from smnadmin_initiate_selection.
## @synopsis
##      smnadmin_initiate_selection(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; smnadmin initiate_selection; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Re-initiating selection logic for local node
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;
sub smnadmin_initiate_selection {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub smnadmin_initiate_selection

## @name ::smnadmin_master_name
##
## @summary Parses the raw output from the "smnadmin_master_name" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from smnadmin master_name.
## @synopsis
##      rastrace_show(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; smnadmin master_name; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Storage master node is stor-3270-1
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;

sub smnadmin_master_name {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub smnadmin_master_name

## @name ::smnadmin_master_dblade
##
## @summary Parses the raw output from the "smnadmin_master_dblade" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from smnadmin master_dblade.
## @synopsis
##      smnadmin_master_dblade(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; smnadmin master_dblade; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Storage master node is stor-3270-1
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;

sub smnadmin_master_dblade {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub smnadmin_master_dblade

## @name ::smnadmin_status
##
## @summary Parses the raw output from the "smnadmin_status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from smnadmin master_status.
## @synopsis
##      smnadmin_master_status(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; smnadmin status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Storage master node is stor-3270-1
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [ ]
##&nbsp;

sub smnadmin_status {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    return [];
} ## end sub smnadmin_status
1;
