#$Id: //depot/prod/test/main/lib/NACL/APISet/Node/CLI/7Mode/default/smtape.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode smtape commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode smtape
##    commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "smtape backup" will be smtape_backup()
##
## @synopsis
##    use smtape;
##
##    smtape_backup(%args);
##    smtape_restore(%args);
##    smtape_status(%args);
##    smtape_continue(%args);
##    smtape_abort(%args);
##
## @author dl-dpg-nb-automation; dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::smtape;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Node::CLI);
# Include exceptions which will be thrown

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException qw(:try);

## @name ::smtape_backup
##
## @summary Parses the raw output from the "smtape backup" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from smtape_backup with any of its options as described in the 
##    corresponding cdef file 'smtape.cdefs'
## @synopsis
##      smtape_backup(%args);
## INPUTS TO METHOD CALL 'smtape_backup' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-b' </td> <td> 'block-size' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'snapshot-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tape-device' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'volume-geometry' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub smtape_backup {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command_line,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    # Now we can parse the output
    $result = _smtape_backup(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );

    return $result;
}

sub _smtape_backup {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array );

    if ( $raw_output =~ /Job (\d+) started/i ) {
    $res_hash{'job_id'} = $1;
    } else {
         NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                      text =>"Output is not in the expected format",
                                       response_object => $response);
    }

    push( @result_array, \%res_hash );
    return \@result_array;
}

## @name ::smtape_restore
##
## @summary Parses the raw output from the "smtape restore" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from smtape_restore with any of its options as described in the 
##    corresponding cdef file 'smtape.cdefs'
## @synopsis
##      smtape_restore(%args);
## INPUTS TO METHOD CALL 'smtape_restore' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-b' </td> <td> 'block-size' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'backup-image' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'volume-path' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'tape-device' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub smtape_restore {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command_line,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _smtape_restore(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );

    return $result;
}

sub _smtape_restore {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array );

    if ( $raw_output =~ /Job (\d+) started/i ) {
        $res_hash{'job_id'} = $1;
    } elsif ( $raw_output =~ /To optimize smtape restore.*?\n-g (.+)/i ) {
        $res_hash{'volume_geometry'} = $1;
    } elsif ( $command =~ /-h\s?/ ) {
        my ( @lines,       @snapshots,     $key );
        my ( $snapshot_id, $snapshot_time, $snapshot_name );
        my $can_throw = 1;

        #specifying the first line of the output
        my $first_line = '[A-Z].+?\s*:\s*.+';

        # Convert the raw_output into lines before parsing
        @lines = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        my $i;
        for ( $i = 0 ; $i < @lines && $lines[$i] !~ /^Snapshot/i ; $i++ ) {
            unless ( $lines[$i] =~ /(.+?)\s*:\s*(.+)/ ) {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text =>"Output is not in the expected format",
                    response_object => $response);
            }
            $key = to_key_name( "item" => $1 );
            $res_hash{$key} = $2;
        }
        while ( $i < @lines ) {
            unless ( $lines[$i] =~ /.+?:\s*(.+)/ ) {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text =>"Output is not in the expected format",
                    response_object => $response);
            }
            $snapshot_id = $1;
            unless ( $lines[ $i + 1 ] =~ /.+?:\s*(.+)/ ) {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text =>"Output is not in the expected format",
                    response_object => $response);
            }
            $snapshot_time = $1;
            unless ( $lines[ $i + 2 ] =~ /.+?:\s*(.+)/ ) {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text =>"Output is not in the expected format",
                    response_object => $response);
            }
            $snapshot_name = $1;
            push @snapshots,
              {
                'snapshot_id'   => $snapshot_id,
                'snapshot_time' => $snapshot_time,
                'snapshot_name' => $snapshot_name
              };
            $i += 3;
        }
        $res_hash{'snapshots'} = \@snapshots;
    } else {
         NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                      text =>"Output is not in the expected format",
                                       response_object => $response);
    }

    push( @result_array, \%res_hash );
    return \@result_array;
}

## @name ::smtape_status
##
## @summary Parses the raw output from the "smtape status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from smtape_status with any of its options as described in the 
##    corresponding cdef file 'smtape.cdefs'
## @synopsis
##      smtape_status(%args);
## INPUTS TO METHOD CALL 'smtape_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-l' </td> <td> 'long-status' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'status-path' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'job-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub smtape_status {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>$command_line,
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = "NACL::APISet::Exceptions::${error}Exception";
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _smtape_status(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

sub _smtape_status {

    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array, @entries);
    my $can_throw =1;

    # Extracting the command option.
    $command =~ s/(smtape status)\s?//;

    if ($command !~ /-\w/) {
        $command = "";
    }

    #specifying the first line of the output
    my $first_line = '[J|j]ob .+';

    # Convert the raw_output into lines before parsing
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    if ($command eq ""  || $command =~ /(-p)\s?/) {

        shift (@lines);
        foreach my $line(@lines) {
            my @values = split(/\s+/,$line);
            my %hash;
            $hash{"job_id"}        = $values[0];
            $hash{"sequence_no"}   = $values[1];
            $hash{"type"}          = $values[2];
            $hash{"status"}        = $values[3];
            $hash{"path"}          = $values[4];
            $hash{"device"}        = $values[5];
            $hash{"progress"}      = $values[6].$values[7];

            push (@entries, \%hash);
        }
        return \@entries;
    }
    my (@TempArray, $index, $temp_reference);
    if ( $command =~ /(-l)\s?/) {

        # First split the data into logical blocks
        for(my $i=0;$i<@lines;$i++) {

            if($lines[$i] =~ /^Job(\s)+ID/) {
                # The next logical block has started
                $TempArray[$index] = [];
                $temp_reference=$TempArray[$index++];
                push(@$temp_reference, $lines[$i]);

            } else {
                push(@$temp_reference, $lines[$i]);
            }
        }

        # Now process the data in each block
        foreach my $ref (@TempArray) {
            my @data = @$ref;
            my %hash;
            for(my $j=0; $j<@data; $j++) {
                if ($data[$j] =~ /(.+):\s+(.+)/) {
                    my $key = $1;
                    my $value= $2;
                    $key = to_key_name("item" => $key);
                    $hash{$key} = $value;
                }
            }
            push(@entries, \%hash);
        }

        return \@entries;
    }
}

## @name ::smtape_continue
##
## @summary Parses the raw output from the "smtape continue" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from smtape_continue with any of its options as described in the 
##    corresponding cdef file 'smtape.cdefs'
## @synopsis
##      smtape_continue(%args);
## INPUTS TO METHOD CALL 'smtape_continue' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'job-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub smtape_continue {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command_line,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    # Now we can parse the output
    $result = _smtape_continue(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );

    return $result;
}

sub _smtape_continue {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array );

    if ($raw_output =~ /Job (\d+) continued/i) {
        $res_hash{'job_id'} = $1;
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                     text =>"Output is not in the expected format",
                                      response_object => $response);
    }

    push( @result_array, \%res_hash );
    return \@result_array;
}

## @name ::smtape_abort
##
## @summary Parses the raw output from the "smtape abort" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from smtape_abort with any of its options as described in the 
##    corresponding cdef file 'smtape.cdefs'
## @synopsis
##      smtape_abort(%args);
## INPUTS TO METHOD CALL 'smtape_abort' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'job-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub smtape_abort {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command_line,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    # Now we can parse the output
    $result = _smtape_abort(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );

    return $result;
}

sub _smtape_abort {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array );

    if ($raw_output =~ /Job (\d+) aborted/i) {
        $res_hash{'job_id'} = $1;
    }
    else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                     text =>"Output is not in the expected format",
                                      response_object => $response);
    }

    push( @result_array, \%res_hash );
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    # Command missing required parameters
    if ( $raw_output =~ /(.*?required.*)/i ) {

        $error          = "MissingRequiredParam";
        $$error_message = $1;

    } elsif ( $raw_output =~ /(.*?Tape record size.*)|(.*?Invalid.*)/i
                or $raw_output =~ /(.*?geometry option is only .*)/i
                or $raw_output =~ /(.*?Destination volume .+? not found.*)/i
                or $raw_output =~ /(.*?Job ID out of range.*)/i ) {

        $error          = "InvalidParamValue";
        $$error_message = $1;

    } elsif ($raw_output =~ m/.*?(job not found)/i) {

       $error = "NoMatchingEntries";
       $$error_message = $1;

    } elsif ( $raw_output =~ /(.*?not found.*)|(.*?too .*)|(.*?required.*)/i ) {

        $error          = "InvalidParamValue";
        $$error_message = $1;

    } elsif ( $raw_output =~/(.*?failed.*)|(.*?out of order.*)|(.*?missing.*)/i
                or $raw_output =~ /(.*?no such command.*?)|(.*?not supported.*)/i
                or $raw_output =~ /(.*?volume .*)/
                or $raw_output =~ /(.*?job already active.*?)/i ) {

        $error          = "CommandFailed";
        $$error_message = $1;
    }
    # Some error messages are displayed later after the prompt returns, not
    # before the prompt(as desired). Burts 404134, 404108 have been raised
    # to that effect and the changes will be reflected in SN. Changes must
    # be made here when the burt is resolved.
    else {
        return undef;
    }

    $$error_message =~ s/^$command:\s//;
    return $error;
}

1;
