#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode snap commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode snap commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "snap create" will be snap_create()
##
## @synopsis
##    use snap;
##
##    snap_create(%args);
##    snap_delete(%args);
##    snap_rename(%args);
##    snap_reserve(%args);
##    snap_list(%args);
##    snap_status(%args);
##    snap_autodelete(%args);
##    snap_restore(%args);
##    snap_sched(%args);
##    snap_reclaimable(%args);
##
## @author dl-nacl-dev@netapp.com
## @change 2011-05-25 govinda1@netapp.com :
##               Added APIset support for command "snap restore"
## @change 2011-05-30 kathar.hidayath@netapp.com :
##               Added APIset support for command "snap sched"
## @change 2011-11-22 skotagir@netapp.com
## @change 2012-04-10 Akshay Mohit(makshay@netapp.com):
##               Added APIset support for command "snap sched"(Minutely schedule).
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::snap;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::snap_create
##
## @summary Parses the raw output from the "snap create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_create with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_create(%args);
## INPUTS TO METHOD CALL 'snap_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snapshot' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_create {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap create",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_create(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_create

## @name ::snap_delete
##
## @summary Parses the raw output from the "snap delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_delete with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_delete(%args);
## INPUTS TO METHOD CALL 'snap_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snapshot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_delete {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap delete",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_delete(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_delete

## @name ::snap_rename
##
## @summary Parses the raw output from the "snap rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_rename with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_rename(%args);
## INPUTS TO METHOD CALL 'snap_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'current-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_rename {
    my %args = @_;

    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap rename",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_rename(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_rename

## @name ::snap_reserve
##
## @summary Parses the raw output from the "snap reserve" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_reserve with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_reserve(%args);
## INPUTS TO METHOD CALL 'snap_reserve' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'percentage' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_reserve {
    my %args = @_;

    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap reserve",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_reserve(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_reserve
## @name ::snap_status
##
## @summary Parses the raw output from the "snap status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_status with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_status(%args);
## INPUTS TO METHOD CALL 'snap_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> '-I' </td> <td> 'volume-logical-snapid' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_status {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap status",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_status(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_status

## @name ::snap_list
##
## @summary Parses the raw output from the "snap list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_list with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_list(%args);
## INPUTS TO METHOD CALL 'snap_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-snapshot-space-consumption' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'busy' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'snap-lock' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'snapshot-list-uuid' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; snap list vol0; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Volume vol0
##&nbsp;working...
##&nbsp;
##&nbsp;  %/used       %/total  date          name
##&nbsp;----------  ----------  ------------  --------
##&nbsp; 11% (11%)    2% ( 2%)  Jan 29 00:00  nightly.0 (busy, vclone)
##&nbsp; 25% (17%)    5% ( 3%)  Jan 28 00:00  nightly.1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'vol0',
##&nbsp;            'snapshot_info' => [
##&nbsp;                                 {
##&nbsp;                                   '%/total' => '2% ( 2%)',
##&nbsp;                                   'date' => 'Jan 29 00:00',
##&nbsp;                                   'name' => 'nightly.0',
##&nbsp;                                   '%/used' => '11% (11%)',
##&nbsp;                                   'snapshot_name' => 'nightly.0',
##&nbsp;                                   'snapshot_status' => [ 'busy' ,
##&nbsp;                                                      'vclone']
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   '%/total' => '5% ( 3%)',
##&nbsp;                                   'date' => 'Jan 28 00:00',
##&nbsp;                                   'name' => 'nightly.1',
##&nbsp;                                   '%/used' => '25% (17%)'
##&nbsp;                                   'snapshot_name' => 'nightly.1',
##&nbsp;                                 },
##&nbsp;                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; snap list -l vol2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Volume vol2
##&nbsp;working...
##&nbsp;
##&nbsp;snapshot date               retention date              name
##&nbsp;--------------------------  --------------------------  --------
##&nbsp;Jan 29 12:31:03 2012 +0000  -                           snap2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume' => 'vol2',
##&nbsp;            'snapshot_info' => [
##&nbsp;                                 {
##&nbsp;                                   'retention_date' => '-',
##&nbsp;                                   'name' => 'snap2',
##&nbsp;                                   'snapshot_date' => 'Jan 29 12:31:03 2012 +0000',
##&nbsp;                                   'snapshot_name' => 'snap2',
##&nbsp;                                 }
##&nbsp;                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snap_list {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap list",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_list(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_list

## @name ::snap_autodelete
##
## @summary Parses the raw output from the "snap autodelete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_autodelete with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_autodelete(%args);
## INPUTS TO METHOD CALL 'snap_autodelete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'delete_order' </td> <td> 'delete-order' </td> </tr>
## <tr> <td> 'trigger' </td> <td> 'trigger' </td> </tr>
## <tr> <td> 'destroy_list' </td> <td> 'destroy-list' </td> </tr>
## <tr> <td> 'target_free_space' </td> <td> 'target-free-space' </td> </tr>
## <tr> <td> 'commitment' </td> <td> 'commitment' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> 'defer_delete' </td> <td> 'defer-delete' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> 'prefix' </td> <td> 'prefix' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snap_autodelete {
    my %args = @_;

    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap autodelete",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snap_autodelete(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub snap_autodelete

sub _snap_create {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    # If there are any errors in command execution, then an exception would
    # have been thrown and we wouldn't have reached this function at all.
    # So we believe that the command had run successfully.

    return [];
} ## end sub _snap_create

sub _snap_delete {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my @output;

    if ( $raw_output =~ /^Deleted\s\w+\ssnapshot\s(.*)\./ ) {
        my $first_line = 'Deleted\s\w+\ssnapshot\s(.*)\.';
        my $can_throw  = 1;
        my @arr        = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        foreach (@arr) {
            if ( $_ =~ /^Deleted\s\w+\ssnapshot\s(.*)\./ ) {
                my %hash;
                $hash{"snapshot"} = $1;
                $hash{"status"}   = "deleted";
                push( @output, \%hash );
            } else {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Unexpected Output while parsing",
                    response_object => $response
                );
            }
        }
        return \@output;
    } else {

        # If there are any errors, we would have thrown exceptions already.
        # If this command is for multiple deletion of snapshots, then we would
        # have returned the output in the previous if loop.
        # Since none of these happened, we believe that the command had run
        # successfully with no output. This is the case when a single snapshot
        # is deleted.
        return [];
    }
} ## end sub _snap_delete

sub _snap_rename {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    # If there are any errors in command execution, then an exception would
    # have been thrown and we wouldn't have reached this function at all.
    # So we believe that the command had run successfully.

    return [];
} ## end sub _snap_rename

## @name ::snap_reclaimable
##
## @summary Parses the raw output from the "snap reclaimable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_reclaimable with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_reclaimable(%args);
## INPUTS TO METHOD CALL 'snap_reclaimable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snapshot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'bytes_freed' => '60 Kbytes'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub snap_reclaimable {
    my %args       = @_;
    my $response   = $args{"response_object"};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $error_message;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap reclaimable",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @return_array;
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 1,
        response_object => $response
    );
    foreach my $line (@lines) {
        if ( $line =~ /Approximately\s+(\d+\s+\w+)\s+would be freed/i ) {
            $return_array[0]->{'bytes_freed'} = $1;
        }
    }
    return \@return_array;
} ## end sub snap_reclaimable

sub _snap_reserve {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my @output;

    if ( $raw_output
        =~ /current\ssnapshot\sreserve\sis\s(\d+)%\sor\s(\d+)\s(.+)/ ) {
        my $first_line
            = '(Volume|Aggregate)\s\w+:\scurrent\ssnapshot\sreserve\sis\s\d+%\sor\s\d+\s.+';

  # Always some output is expected when snap reserve is invoked in get context
        my $can_throw = 1;
        my @arr       = get_lines(
            "raw_output"      => $raw_output,
            "first_line"      => $first_line,
            "can_throw"       => $can_throw,
            "response_object" => $response
        );
        for ( my $i = 0; $i < @arr; $i++ ) {
            if ( $arr[$i]
                =~ /(Volume|Aggregate)\s(\w+):\scurrent\ssnapshot\sreserve\sis\s(\d+)%\sor\s(\d+)\s(.+)/
                ) {
                my %hash;
                my $name = $1 . "_name";
                $name            = lc($name);
                $hash{$name}     = $2;
                $hash{"reserve"} = $3 . '%';
                $hash{"size"}    = $4 . " " . $5;
                push( @output, \%hash );
            } else {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Unexpected Output while parsing",
                    response_object => $response
                );
            }
        }
        return \@output;
    } else {

        # If there are any errors, we would have thrown exceptions already.
        # If this command is for getting snap reserve details, then we would
        # have returned the output in the previous if loop.
        # Since none of these happened, we believe that the command had been
        # used in set context and run successfully with no output.

        return [];
    }
} ## end sub _snap_reserve

sub _snap_status {
    my %args       = @_;
    my $unit       = undef;
    my $key1       = undef;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    #executing snap status for aggregate
    if ( $command =~ /-A/ ) {
        $unit = "Aggregate";
        $key1 = "aggregate";
    } else {
        $unit = "Volume";
        $key1 = "volume";
    }
    my $pattern    = '^' . $unit . '\s+(\w+)(?:\s+)?(?:\(.*\))?$';
    my $first_line = "$unit";
    my $can_throw  = 0;
    my @lines      = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my @return_array;

    while ( my $line = shift(@lines) ) {
        my $info_hash = {};
        if ( $line =~ /$pattern/ ) {
            $info_hash->{$key1} = $1;
            $line = shift(@lines);
            if ( $line =~ /No snapshots exist./ ) {
                $info_hash->{'snapshot_status'} = [];
                push( @return_array, $info_hash );
                next;
            }
            my @keys;
            my $vol_log_flag = 0;
            if ( $line =~ /Volume.*Logical/ ) {
                $vol_log_flag = 1;
                @keys         = (
                    'volume_snapid', 'logical_snapid',
                    'status',        'date',
                    'ownblks',       'release',
                    'fsrev',         'name',
                    'snapshot_name', 'progress',
                );
                shift(@lines)
                    ; #shifting the line which has snapid status and such column names
            } else {
                @keys = (
                    'snapid',  'status', 'date', 'ownblks',
                    'release', 'fsrev',  'name',
                    'snapshot_name', 'progress',
                );
            }
            shift(@lines);    #skipping the dashes
            $line = shift(@lines);
            while ( $line !~ /$unit/ ) {
                my $table_hash = {};
                my @values;
                my $date_format = '\w{3}\s+\d{1,2}\s+\d{2}:\d{2}';
                if ($vol_log_flag) {
                    $line
                        =~ /(\d+)(?:\s+)?(\d+)(?:\s+)?(\w+)?(?:\s+)?($date_format)(?:\s+)?(\d+)(?:\s+)?(\d+\.\d+)(?:\s+)(\d+)(?:\s+)(.*)$/;
                    push( @values, $1, $2, $3, $4, $5, $6, $7, $8 );
                    my $snap_name = $7;
                    if ( $snap_name =~ /(\w+)\s+(.*)/ ) {
                        push(@values, $1, $2);
                    } else {
                        push(@values, $snap_name, '-');
                    }
                } else {
                    $line
                        =~ /(\d+)(?:\s+)?(\w+)?(?:\s+)?($date_format)(?:\s+)?(\d+)(?:\s+)?(\d+\.\d+)(?:\s+)(\d+)(?:\s+)(.*)$/;
                    push( @values, $1, $2, $3, $4, $5, $6, $7 );
                    my $snap_name = $7;
                    if ( $snap_name =~ /(\w+)\s+(.*)/ ) {
                        push(@values, $1, $2);
                    } else {
                        push(@values, $snap_name, '-');
                    }
                }
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Unexpected Output while parsing",
                    response_object => $response
                ) unless ( scalar(@keys) == scalar(@values) );

                for ( my $i = 0; $i < scalar(@keys); $i++ ) {
                    $table_hash->{ $keys[$i] } = $values[$i];
                }
                push( @{ $info_hash->{'snapshot_status'} }, $table_hash );
                last unless ( scalar(@lines) );
                $line = shift(@lines);
            }
            unshift( @lines, $line ) if ( scalar(@lines) );
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "Unexpected Output while parsing",
                response_object => $response
            );
        }
        push( @return_array, $info_hash );
    }

    return \@return_array;
} ## end sub _snap_status

sub _snap_list {
    my %args       = @_;
    my $unit       = undef;
    my $key1       = undef;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    #executing snap status for aggregate
    if ( $command =~ /-A/ ) {
        $unit = "Aggregate";
        $key1 = "aggregate";
    } else {
        $unit = "Volume";
        $key1 = "volume";
    }
    my $pattern    = '^' . $unit . '\s+(\w+)$';
    my $first_line = "$unit";
    my $can_throw  = 0;
    my @lines      = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my @return_array;
    while ( my $line = shift(@lines) ) {
        my $info_hash = {};
        if ( $line =~ /$pattern/ ) {
            $info_hash->{$key1} = $1;
            shift(@lines) if ( $lines[0] =~ /^working(\.)+$/ );
            $line = shift(@lines);
            if ( $line =~ /No snapshots exist./ ) {
                $info_hash->{'snapshot_info'} = [];
                push( @return_array, $info_hash );
                next;
            }
            if ( $line =~ /date\s+name/ ) {
                my $used_column_presence = 0;
                $used_column_presence = 1
                    if ( $line =~ /%\/used\s+%\/total\s+/ );
                $info_hash->{'snapshot_info'} = [];
                shift(@lines)
                    if $lines[0] =~ /^[-\s]+$/;    #skip the line of dashes
                $line = shift(@lines);
                while ( defined $line && ( $line !~ /$unit/ ) ) {
                    my $table_hash        = {};
                    my $percentage_format = '\d+%\s\(\s?\d+%\)';
                    my $date_format       = '\w{3}\s+\d{1,2}\s+\d{2}:\d{2}';
                    my $opt_l_date_format
                        = '\w{3}\s+\d{2}\s+\d{2}:\d{2}:\d{2}\s+\d{4}\s+[-|\+]\d{4}';
                    if ( $line
                        =~ /($percentage_format|$opt_l_date_format)?(?:\s+)?($percentage_format|$opt_l_date_format|-)?(?:\s+)?($date_format)?(?:\s+)?(.*)$/
                        ) {
                        if ($used_column_presence) {
                            $table_hash->{'%/used'}  = $1;
                            $table_hash->{'%/total'} = $2;
                            $table_hash->{'date'}    = $3;
                        }
                        $table_hash->{'name'} = $4;
                        unless ($used_column_presence) {
                            $table_hash->{'snapshot_date'}  = $1;
                            $table_hash->{'retention_date'} = $2;
                        }
                        if ( $table_hash->{'name'} =~ /(.+)\s+(?:\((.+)\))$/ )
                        {
                            $table_hash->{'snapshot_name'}   = $1;
                            $table_hash->{'snapshot_status'} = $2;
                            $table_hash->{'snapshot_name'} =~ s/\s*$//;
                            $table_hash->{'snapshot_status'} = [
                                split(
                                    /\s*,\s*/,
                                    $table_hash->{'snapshot_status'}
                                )
                            ];
                        } else {
                            $table_hash->{'snapshot_name'}
                                = $table_hash->{'name'};
                        }

                    } else {
                        NACL::APISet::Exceptions::UnexpectedOutputException
                            ->throw(
                            text => "Unexpected Output while parsing",
                            response_object => $response
                            );
                    }
                    push( @{ $info_hash->{'snapshot_info'} }, $table_hash );
                    last unless ( scalar(@lines) );
                    $line = shift(@lines);
                }
                unshift( @lines, $line ) if ( scalar(@lines) );
            } elsif ( $line =~ /^name\s+owners$/ ) {
                shift(@lines) if ( $lines[0] =~ /-*\s*-*/ );
                $line = shift(@lines);
                while ( $line !~ /$unit/ ) {
                    my $table_hash = {};
                    if ( $line =~ /(\S*)\s*(\w+)?/ ) {
                        my $name = $1;
                        $name =~ s/\s+$//;
                        $table_hash->{'name'} = $name;
                        if ( defined($2) ) {
                            $table_hash->{'owners'} = $2;
                        } else {
                            $table_hash->{'owners'} = '';
                        }
                        $table_hash->{'snapshot_name'}
                            = $table_hash->{'name'};
                    } else {
                        NACL::APISet::Exceptions::UnexpectedOutputException
                            ->throw(
                            text => "Unexpected Output while parsing",
                            response_object => $response
                            );
                    }
                    push( @{ $info_hash->{'snapshot_info'} }, $table_hash );
                    last unless ( scalar(@lines) );
                    $line = shift(@lines);
                }
                unshift( @lines, $line ) if ( scalar(@lines) );
            } else {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Unexpected Output while parsing",
                    response_object => $response
                );
            }
            push( @return_array, $info_hash );
        }
    }
    return \@return_array;
} ## end sub _snap_list

sub _snap_autodelete {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    my $first_line = '(snapshot autodelete|snap autodelete)';
    my $can_throw  = 1;
    my @lines      = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my @return_array;
    my %hash;

    if ( $raw_output =~ /snapshot autodelete settings for (.+)?:/ ) {
        $hash{'volume'} = $1;
        shift @lines;
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /(.+?)\s*:\s*(.*)?/ ) {
                $hash{$1} = $2;
            } else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "Unexpected output",
                    response_object => $response
                );
            }
        }
        push( @return_array, \%hash );
    } elsif ( $raw_output
        =~ /snap autodelete: snap autodelete configuration options set/ ) {
        $hash{'status'} = "configuration set";
        push( @return_array, \%hash );
    } elsif ( $raw_output =~ /snap autodelete: snap autodelete (.+)/ ) {

# If there are any errors, we would have thrown exceptions already.
# If this command is for getting snap autodelete show details, then we would
# have populated the output in the first if loop. If this comand is for
# setting configuration options, then we have set the status in the second
# if loop. Since none of these happened, we believe that some action has taken
# place like on, off etc and command run successfully.
        $hash{'status'} = $1;
        push( @return_array, \%hash );
    }
    return \@return_array;
} ## end sub _snap_autodelete

## @name ::snap_restore
##
## @summary Parses the raw output from the "snap restore" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_restore with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_restore(%args);
## INPUTS TO METHOD CALL 'snap_restore' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'restore-from-path' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-r' </td> <td> 'restore-as-path' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snap restore -t
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = snap restore -s
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = snap restore -r
##&nbsp;$VAR1 = [];
##&nbsp;
sub snap_restore {
    my %args       = @_;
    my $response   = $args{"response_object"};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $error_message;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap restore",
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @return_array;
    my %hash;

    if ( $raw_output =~ /Volume\s+(.*):\s+revert\s+successful.*/ ) {
        $hash{'volume'}        = $1;
        $hash{'revert_status'} = 'successful';
        push( @return_array, \%hash );
    }

    return \@return_array;
} ## end sub snap_restore

# XXX To be documented with the DocGenerator
sub snap_restore_aggr {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $error    = _check_error(
        raw_output        => $response->get_raw_output(),
        command           => 'snap restore',
        ptr_error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
} ## end sub snap_restore_aggr

## @name ::snap_sched
##
## @summary Parses the raw output from the "snap sched" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snap_sched with any of its options as described in the
##    corresponding cdef file 'snap.cdefs'
## @synopsis
##      snap_sched(%args);
## INPUTS TO METHOD CALL 'snap_sched' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hours' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hourly-snapshot-num' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hour-list' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'minutes' </td> </tr>
## <tr> <td> '-V' </td> <td> 'volume' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'aggregate-name' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'days' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'weeks' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snap sched -V vol0
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'hours' => '1',
##&nbsp;             'volume' => 'vol0',
##&nbsp;             'minutes' => '2',
##&nbsp;             'days' => '0',
##&nbsp;             'weeks' => '0'
##&nbsp;           },
##&nbsp;        ];
##&nbsp;
sub snap_sched {
    my %args       = @_;
    my $resp       = $args{"response_object"};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error_message;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "snap sched",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @return_array;
    my @output = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $resp
    );
    my $i = -1;
    foreach my $line (@output) {
        if ( $line
            =~ /(\w+)\s+(\w+):\s+(\d+)\s+(\d+)\s*(\d+@?[\d+,?]{1,}|\d+@|\d+)\s+(\d+)/
            || $line
            =~ /(\w+)\s+(\w+):\s+(\d+)\s+(\d+)\s*(\d+@?[\d+,?]{1,}|\d+@|\d+)/
            ) {
            $i++;
            my $temp = $1;
            $temp =~ tr/[A-Z]/[a-z]/;
            $return_array[$i]->{"$temp"} = $2;
            $return_array[$i]->{'weeks'} = $3;
            $return_array[$i]->{'days'}  = $4;
            $return_array[$i]->{'hours'} = $5;
            if ( defined $6 ) {
                $return_array[$i]->{'minutes'} = $6;
            }
        }
    }
    return \@return_array;
} ## end sub snap_sched

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if ( $raw_output =~ /$command(?:[^:]*):(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message =~ /((:?Volume|Aggregate) name not specified)/ )
        {
            $error = "MissingRequiredParam";
        } elsif ( $raw_output
            =~ /snap delete: \-f option must be used with \-a when using rsh/
            ) {
            $$error_message = $raw_output;
            $error          = "MissingRequiredParam";
        } elsif ( $raw_output
            =~ /(snap\s+restore:\s+Volume\s+is\s+in\s+use\s+by\s+other\s+operations.*|Maximum\s+single-file\s+restores\s+reached.*)/
            ) {
            $$error_message = $raw_output;
            $error          = "CommandFailed";
        } elsif ( $$error_message
            =~ /((aggregate|volume)\sname\shas\sto\sbe\sspecified.)|(already exists)|(percent must be a number between 0 and 100)|(does not exist or is not online)|(invalid value of option)|(-t\sfile\srequires\sa\spathname\sof\sthe\sfile\sbeing\srestored.)|(Incorrect\scombination\sof\s-t\svol\sor\sfile\swith\svolname\sor\srestore_from_path.)|((aggregate|volume)\sname\shas\sto\sbe\sspecified.)|(volume\sname\sor\spath\shave\sto\sbe\sspecified.)/
            ) {
            $error = "InvalidParamValue";
        } elsif (
            $$error_message
            =~ /([Ii]nvalid option)|([uU]nrecognized option)|(Too many
        arguments)|(Has\sextra\sarguments.)|(-t\svol\sand\s.*\sare\sincompatible\soptions.)|(Unrecognized\s+option\s+.*)/
            ) {
            $error = "InvalidParam";
        } elsif ( $$error_message
            =~ /(Volume\s.*\sdoes\snot\shave\sa\ssnapshot\snamed\s.*)|(File\sdoes\snot\sexist\sin\ssnapshot.)|(aggregate\sor\straditional\svolume\s.*\sdoes\snot\sexist\sor\sis\snot\sonline.)|(volname\sor\spathname\s.*\sdoes\snot\sexist\sor\sis\snot\sonline)|((.+)?Unexpected error)|(Volume\s.*\sdoes\snot\shave\sa\ssnapshot\snamed\s.*)|(restore_as_path\sand\srestore_from_path\shave\sto\sbe\sspecified\sin\sthe\ssame\svolume.)/
            ) {
            $error = "CommandFailed";
        } elsif ( $$error_message =~ /(invalid schedule)/s ) {
            $error = "InvalidParamValue";
        }
    } elsif ( $raw_output
        =~ /snap restore:\s(vol or file is required for -t option.)|(does not have a snapshot)/
        ) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ( $raw_output
        =~ /(Snapshot already exists|\w+ is not the root of a qtree|No such snapshot)/
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(\w+ \w+ does not exist or is not online)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = "$1";
    } elsif ( $raw_output =~ /snap .*: .* is an invalid option./ ) {
        $error          = "InvalidParamValue";
        $$error_message = "$1";
    } elsif ( $raw_output =~ /.* does not exist or is not online./ ) {
        $error          = "InvalidParamValue";
        $$error_message = "$1";
    } elsif ( $raw_output =~ /usage:/si ) {
        $error = "InvalidParam";
        $$error_message
            = "Invalid switch or Invalid Param value has been sent";
    } elsif ( $raw_output
        =~ /(No space left on device|No\ssnapshots\sexist\sfor\saggregate\s.*|.*is not licensed|No more snapshots available)/
        ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output
        =~ /(one or more of the listed snapshots are not valid.)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif (
        $raw_output =~ /(volume \w+ does not exist or is not online.)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(No snapshots found in the volume)/si ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

1;
