#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Node snaplock command
##
## @description
##    This is a library to parse the raw output of Node snaplock command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
## use snaplock;
##
## snaplock(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::snaplock_clock;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::snaplock_clock_initialize
##
## @summary Parses the raw output from the "snaplock_clock_initialize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snaplock_clock with any of its options as described in the
##    corresponding cdef file 'snaplock_clock.cdefs'
## @synopsis
##      snaplock_clock(%args);
## INPUTS TO METHOD CALL 'snaplock_clock_initialize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snaplock clock initialize
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Compliance Clock already initialized.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'compliance_clock' => 'initialized'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snaplock_clock_initialize {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line =~ /[Cc]ompliance\s+[Cc]lock\s+already\s+(initialized).*/ ) {
            $result_array[0]->{to_key_name(item => 'compliance_clock')} = $1;
        }
    }
    return \@result_array;
} ## end sub snaplock_clock_initialize

## @name ::snaplock_clock_status
##
## @summary Parses the raw output from the "snaplock_clock_status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snaplock_clock with any of its options as described in the
##    corresponding cdef file 'snaplock_clock.cdefs'
## @synopsis
##      snaplock_clock(%args);
## INPUTS TO METHOD CALL 'snaplock_clock_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'verbose' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snaplock clock status -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; System Compliance Clock: 2014-08-14:10:26:22 GMT
##&nbsp; System Clock: 2014-08-14:10:26:29 GMT
##&nbsp; System Compliance Clock Skew: 7
##&nbsp; Node ID: -223779755
##&nbsp; SCC ID: 0x147aed54927
##&nbsp;                      In-Core Volume      On-disk Volume      On-disk System
##&nbsp; Volume             Compliance Clock    Compliance Clock    Compliance Clock          SCC id   Node id      VCC skew
##&nbsp;                               (GMT)               (GMT)               (GMT)
##&nbsp; --------------- ------------------- ------------------- ------------------- --------------- --------- -------------
##&nbsp; vol0            2014-08-14:10:26:22 2014-08-14:10:26:20 2014-08-14:10:26:20 0x147aed54927 -223779755             0
##&nbsp; newvol          2014-08-14:10:26:22 2014-08-14:05:18:47 2014-08-14:05:18:47 0x147aed54927 -223779755             0
##&nbsp; Alice_codings_1407837404_vol 2014-08-14:10:26:22 2014-08-14:05:18:47 2014-08-14:05:18:47 0x147aed54927 -223779755             0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'system_compliance_clock_skew' => '6',
##&nbsp;             'volume' => [
##&nbsp;                           {
##&nbsp;                             'on_disk_system_compliance_clock' => '2014-08-21:12:15:20',
##&nbsp;                             'node_id' => '-223779755',
##&nbsp;                             'volume' => 'vol0',
##&nbsp;                             'vcc_skew' => '0',
##&nbsp;                             'in_core_volume_compliance_clock' => '2014-08-21:12:15:21',
##&nbsp;                             'scc_id' => '0x147aed54927',
##&nbsp;                             'on_disk_volume_compliance_clock' => '2014-08-21:12:15:20'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'on_disk_system_compliance_clock' => '2014-08-21:12:00:01',
##&nbsp;                             'node_id' => '-223779755',
##&nbsp;                             'volume' => 'newvol',
##&nbsp;                             'vcc_skew' => '0',
##&nbsp;                             'in_core_volume_compliance_clock' => '2014-08-21:12:15:21',
##&nbsp;                             'scc_id' => '0x147aed54927',
##&nbsp;                             'on_disk_volume_compliance_clock' => '2014-08-21:12:00:01'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'on_disk_system_compliance_clock' => '2014-08-21:12:00:01',
##&nbsp;                             'node_id' => '-223779755',
##&nbsp;                             'volume' => 'Alice_codings_1407837404_vol',
##&nbsp;                             'vcc_skew' => '0',
##&nbsp;                             'in_core_volume_compliance_clock' => '2014-08-21:12:15:21',
##&nbsp;                             'scc_id' => '0x147aed54927',
##&nbsp;                             'on_disk_volume_compliance_clock' => '2014-08-21:12:00:01'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;             'node_id' => '-223779755',
##&nbsp;             'system_clock' => '2014-08-21:12:15:27 GMT',
##&nbsp;             'scc_id' => '0x147aed54927',
##&nbsp;             'system_compliance_clock' => '2014-08-21:12:15:21 GMT'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub snaplock_clock_status {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my (@heads, @values, @space_heads);

    if ($command =~ /-v/ ) {
        my $count = -1;
        foreach my $line (@lines) {
            my %hash;
            if ( $line =~ /(.*):\s+(.*)/ ) {
                $result_array[0]->{to_key_name(item => $1)} = $2;
            } elsif ( $line =~ /^\s*(\w+-\w+\s+\w+)\s+(\w+-\w+\s+\w+)\s+(\w+-\w+\s+\w+)/) {
                @space_heads = ($1,$2,$3);
                my $s = 0;
                foreach (@space_heads) {
                    $_ =~ s/-/_/g;
                }
            } elsif ( $line =~ /Volume.*/) {
                @heads = split(/\s+/, $line);
            } elsif ($line =~ /(\w+)\s+(.*)\s+(.*)\s+(.*)\s+(.*)\s+(-\d+)\s+(\d+)/) {
                my ($first_val,$second_val,$third_val,$fourth_val,$fifth_val,$sixth_val,$seventh_val) = ($1,$2,$3,$4,$5,$6,$7);
                $count++;
                $hash{ to_key_name( item => $heads[0])} = $first_val;
                $hash{(to_key_name( item => $space_heads[0]."_". $heads[1]) )."_".
                      (to_key_name( item => $heads[2]) )} = $second_val;
                $hash{(to_key_name( item => $space_heads[1]."_". $heads[3]) )."_".
                      (to_key_name( item => $heads[4]) )} = $third_val;
                $hash{(to_key_name( item => $space_heads[2]."_". $heads[5]) )."_".
                      (to_key_name( item => $heads[6]) )} = $fourth_val;
                $hash{(to_key_name( item => $heads[7]) )."_".
                      (to_key_name( item => $heads[8]) )} = $fifth_val;
                $hash{(to_key_name( item => $heads[9]) )."_".
                      (to_key_name( item => $heads[10]) )} = $sixth_val;
                $hash{(to_key_name( item => $heads[11]) )."_".
                      (to_key_name( item => $heads[12]) )} = $seventh_val;
                $result_array[0]->{volumes}[$count] = \%hash;
            }
        }
    } else {
        my $i = -1;
        foreach my $line (@lines) {
            my %hash;
            if ($line =~ /(.*):\s+(.*)/) {
                $result_array[0]->{to_key_name (item => $1) } = $2;
            } elsif ($line =~ /^Volume.*/) {
                @heads = split(/\s+/,$line);
            } elsif ($line =~ /^(\w+)\s+(.*)/) {
	        my ($first_value,$second_value) = ($1,$2);
                $i++;
                $hash{to_key_name(item => $heads[0])} = $first_value;
                $hash{(to_key_name( item => $heads[1]) )."_".
                      (to_key_name( item => $heads[2]) )} = $second_value;
                $result_array[0]->{volumes}[$i] = \%hash;
            }
        }
    }
    return (\@result_array);
}  ## end sub snaplock_clock_status

1;
