# Hey emacs, this is basically -*- perl -*-
#$Id$
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode snapmirror commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode snapmirror
##    commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "snapmirror on" will be snapmirror_on()
##
## @synopsis
##    use snapmirror;
##
##    snapmirror_initialize(%args);
##    snapmirror_on(%args);
##    snapmirror_off(%args);
##    snapmirror_status(%args);
##    snapmirror_quiesce(%args);
##    snapmirror_break(%args);
##    snapmirror_update(%args);
##    snapmirror_resume(%args);
##    snapmirror_destinations(%args);
##    snapmirror_abort(%args);
##    snapmirror_throttle(%args);
##    snapmirror_release(%args);
##    snapmirror_check_start(%args);
##    snapmirror_resync(%args);
##    snapmirror_check_logs_show(%args);
##    snapmirror_migrate(%args);
##    snapmirror_check_abort(%args);
##    snapmirror_check_logs_delete(%args);
##    snapmirror_nvlogs(%args);
##    snapmirror_trace_dump(%args);
##    snapmirror_trace_dump_autodump(%args);
##    snapmirror_trace_status(%args);
##    snapmirror_trace_set(%args);
##    snapmirror_convert(%args);
##    snapmirror_help(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::snapmirror;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);

## @name ::snapmirror_initialize
##
## @summary Parses the raw output from the "snapmirror initialize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_initialize with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_initialize(%args);
## INPUTS TO METHOD CALL 'snapmirror_initialize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-S' </td> <td> 'source-location' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'source-snapshot' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-location' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'destination-snapshot' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub snapmirror_initialize {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $result;
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror initialize",
                             "ptr_error_message" => \$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    if ($raw_output =~ /Transfer successful/) {
        $result = [{"status" => "Transfer successful"}];
    } elsif ($raw_output =~ /Transfer started/) {
        $result = [{'status' => 'Transfer started'}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
    }
    return $result;
} ## end sub snapmirror_initialize

## @name ::snapmirror_on
##
## @summary Parses the raw output from the "snapmirror on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_on with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_on(%args);
## INPUTS TO METHOD CALL 'snapmirror_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_on {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror on",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub snapmirror_on

## @name ::snapmirror_off
##
## @summary Parses the raw output from the "snapmirror off" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_off with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_off(%args);
## INPUTS TO METHOD CALL 'snapmirror_off' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_off {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror off",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub snapmirror_off

## @name ::snapmirror_status
##
## @summary Parses the raw output from the "snapmirror status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_status with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_status(%args);
## INPUTS TO METHOD CALL 'snapmirror_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'location' </td> </tr>
## <tr> <td> '-t' </td> <td> 'transfer' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiesce' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_status {
    my %args = @_;

    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror status",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    $result = _snapmirror_status(raw_output      => $raw_output,
                                 command         => $command_line,
                                 response_object => $response);

    return $result;
} ## end sub snapmirror_status

## @name ::snapmirror_quiesce
##
## @summary Parses the raw output from the "snapmirror quiesce" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_quiesce with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_quiesce(%args);
## INPUTS TO METHOD CALL 'snapmirror_quiesce' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_quiesce {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror quiesce",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    if ($raw_output =~
        /snapmirror\squiesce:\s(.+)\s:\s+(Successfully\squiesced)/) {
        $result = [{"destination" => $1, "status" => $2}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
    }

    return $result;
} ## end sub snapmirror_quiesce

## @name ::snapmirror_break
##
## @summary Parses the raw output from the "snapmirror break" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_break with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_break(%args);
## INPUTS TO METHOD CALL 'snapmirror_break' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_break {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror break",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    if ($raw_output =~ /Destination\s(.+)\sis\snow\swritable/) {
        $result =
          [{"destination" => $1, "status" => "Destination is writable"}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
    }

    return $result;
} ## end sub snapmirror_break

## @name ::snapmirror_update
##
## @summary Parses the raw output from the "snapmirror update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_update with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_update(%args);
## INPUTS TO METHOD CALL 'snapmirror_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-S' </td> <td> 'source-location' </td> </tr>
## <tr> <td> '-s' </td> <td> 'source-snapshot' </td> </tr>
## <tr> <td> '-w' </td> <td> 'destination-location' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> </tr>
## <tr> <td> '-c' </td> <td> 'destination-snapshot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_update {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror update",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    if ($raw_output =~ /Transfer successful/) {
        $result = [{"status" => "Transfer successful"}];
    } elsif ($raw_output =~ /Transfer started/) {
        $result = [{'status' => 'Transfer started'}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
    }

    return $result;
} ## end sub snapmirror_update

## @name ::snapmirror_resume
##
## @summary Parses the raw output from the "snapmirror resume" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_resume with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_resume(%args);
## INPUTS TO METHOD CALL 'snapmirror_resume' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_resume {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror resume",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    if ($raw_output =~
        /snapmirror\sresume:\s?(.+)\s?:\s?(Successfully\sresumed)/) {
        $result = [{"destination" => $1, "status" => $2}];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
    }

    return $result;
} ## end sub snapmirror_resume

## @name ::snapmirror_destinations
##
## @summary Parses the raw output from the "snapmirror destinations" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_destinations with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_destinations(%args);
## INPUTS TO METHOD CALL 'snapmirror_destinations' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> </tr>
## <tr> <td> '-s' </td> <td> 'list-snapshot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_destinations {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror destinations?",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # Now we can parse the output
    $result = _snapmirror_destinations(raw_output      => $raw_output,
                                       command         => $command_line,
                                       response_object => $response);

    return $result;
} ## end sub snapmirror_destinations

## @name ::snapmirror_abort
##
## @summary Parses the raw output from the "snapmirror abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_abort with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_abort(%args);
## INPUTS TO METHOD CALL 'snapmirror_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## <tr> <td> '-h' </td> <td> 'clear-checkpoint' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_abort {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    # This command takes all the words coming after "snapmirror abort"
    # as destinations.
    $command_line =~ s/snapmirror abort( -h)?\s*//;
    my @dest = split(/\s+/, $command_line);
    my @result_array;
    foreach (@dest) {
        my %hash;
        $hash{"destination"} = $_;
        $hash{"status"}      = "Aborted the specified transfer";
        push(@result_array, \%hash);
    }

    # Now check for any error in command execution
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror abort",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        if (   ($error_message !~ "couldn't find destination")
            && ($error_message !~ "already idle")) {
            $error->throw(text => $error_message, response_object => $response);
        } else {

            # Now look for partial success i.e., for eg take transfer to one
            # destination is aborted while for the other one it is not able to
            # locate the destination.
            foreach (@result_array) {
                my $des_hash = $_;
                my $des      = $des_hash->{"destination"};
                if ($raw_output =~ /$des.+is already idle/) {
                    $des_hash->{"status"} = "Destination is already idle";
                } elsif ($raw_output =~ /couldn't find.+$des/) {
                    $des_hash->{"status"} = "Couldn't find destination";
                } else {
                    $des_hash->{"status"} = "Aborted the specified transfer";
                }
            } ## end foreach (@result_array)
            $response->set_parsed_output(parsed_output => \@result_array);
            $error->throw(text => $error_message, response_object => $response);
        } ## end else [ if (($error_message !~...
    } ## end if ($error)

    # There is no output for this command if it aborts transfers to all
    # specified destinations. Hence if there is no exception thrown, then
    # it implies that the command has got succeeded.

    return \@result_array;
} ## end sub snapmirror_abort

## @name ::snapmirror_throttle
##
## @summary Parses the raw output from the "snapmirror throttle" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_throttle with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_throttle(%args);
## INPUTS TO METHOD CALL 'snapmirror_throttle' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'transfer-rate' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_throttle {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror throttle",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub snapmirror_throttle

## @name ::snapmirror_release
##
## @summary Parses the raw output from the "snapmirror release" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_release with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_release(%args);
## INPUTS TO METHOD CALL 'snapmirror_release' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapmirror_release {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => "snapmirror release",
                             "ptr_error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $response);
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    return [];
} ## end sub snapmirror_release

sub _snapmirror_status {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array);

    # For snapmirror_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    # Extracting the command option. since snapmirror status cannot be
    # invoked with more than one option, this simple logic will work.
    $command =~ s/snapmirror\sstatus\s?//;
    if ($command !~ /-\w/) {
        $command = "";
    }

    # Convert the raw_output into lines before parsing
    my $first_line = 'Snapmirror is .+';
    my @lines = get_lines("raw_output"      => $raw_output,
                          "first_line"      => $first_line,
                          "can_throw"       => $can_throw,
                          "response_object" => $response);

    # Always the first line is about the status of snapmirror
    $lines[0] =~ /Snapmirror is (on|off)/;
    $res_hash{"snapmirror_status"} = $1;
    shift(@lines);    # Remove the first line which is processed already
    if ((scalar(@lines) == 1) || ($lines[1] =~ /There are no active transfers/))
    {

        # Currently Snapmirror status is not throwing any error
        # on specifying invalid volume name or qtree path
        # The output will just contain whether snapmirror is turned off
        # or on as the first line.
        # On supplying a qtree path or volume name which hasn't acted as
        # a snapmirror source, we will get details about status of
        # snapmirror only as the previous case.
        # Hence its not able to differentiate the error condition.
        $res_hash{"snapmirror_entries"} = [];
        push(@result_array, \%res_hash);
        return \@result_array;
    } elsif ($lines[1] =~ /None .+ is quiescing or quiesced/) {
        $res_hash{"quiesced_entries"} = [];
        push(@result_array, \%res_hash);
        return \@result_array;
    }

    # Based on options sent to snapmirror_status command, the parsing method
    # will be decided

    # First condition is to check if the command:
    # 1) is a simple "snapmirror status" (no options)
    # 2) "snapmirror status <vol_or_qtree_path>"
    # 3) "snapmirror status -t"

    # To check that the command contains a particular option, check that it
    # begins with the option. Simply checking for its presence anywhere
    # in the string could lead to false matches, such as if the
    # volume/qtree name/path contains that string
    if (($command eq "") || ($command !~ /^-/) || ($command =~ /^-t/)) {
        my @keys_array = split(/\s+/, $lines[0]);    # get the keys
        my $item;
        for (my $i = 0 ; $i < @keys_array ; $i++) {
            $keys_array[$i] = to_key_name("item" => $keys_array[$i]);
        }
        shift(@lines);                               # Remove the heading line
        my @Mirrored_entries;
        foreach my $line (@lines) {
            my %hash;
            my @values = split(/\s+/, $line);
            my $i = 0;
            foreach $item (@keys_array) {
                $hash{$item} = $values[$i] || "";
                $i = $i + 1;
            }
            if (defined($values[$i])) {
                $hash{"data_transferred"} = $values[$i];
                $hash{"data_transferred"} =~ s/\((.+)\)/$1/;
            }
            push(@Mirrored_entries, \%hash);
        } ## end foreach my $line (@lines)
        $res_hash{"snapmirror_entries"} = \@Mirrored_entries;
    } elsif ($command =~ /^-l/) {

        # First split the data into logical blocks
        my (@Entry, $index, $temp_ref);
        for (my $i = 0 ; $i < @lines ; $i++) {
            next if ($lines[$i] eq "");
            if ($lines[$i] =~ /^Source:/) {

                # The next logical block has started
                $Entry[$index] = [];
                $temp_ref = $Entry[$index++];
                push(@$temp_ref, $lines[$i]);
            } else {
                push(@$temp_ref, $lines[$i]);
            }
        } ## end for (my $i = 0 ; $i < @lines...

        # Now process the data in each block
        my @Mirrored_entries;
        foreach my $ref (@Entry) {
            my @data = @$ref;
            my %hash;
            for (my $j = 0 ; $j < @data ; $j++) {
                if ($data[$j] =~ /(.+):\s+(.+)/) {
                    my $key   = $1;
                    my $value = $2;
                    $key = to_key_name("item" => $key);
                    $hash{$key} = $value;
                }
            }
            push(@Mirrored_entries, \%hash);
        } ## end foreach my $ref (@Entry)
        $res_hash{"snapmirror_entries"} = \@Mirrored_entries;
    } elsif ($command =~ /-q/) {
        my @vols   = ();
        my @qtrees = ();
        foreach my $line (@lines) {
            if ($line =~ /(\/vol\/.+) is quiesced/) {
                push(@qtrees, $1);
            } elsif ($line =~ /(\w+) is quiesced/) {
                push(@vols, $1);
            } else {
                next if ($line =~ /\w+ has quiescing or quiesced qtree/);
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                                 text => "Output is not in the expected format",
                                 response_object => $response);
            }
        } ## end foreach my $line (@lines)

        # TODO:  Check how a simple array converted into xml object will look
        # like and appropriately change the structure if required  - isshwar
        $res_hash{"quiesced_volumes"} = \@vols;
        $res_hash{"quiesced_qtrees"}  = \@qtrees;
    } else {
        NACL::APISet::Exceptions::InvalidParamException->throw(
                                                    text => "Invalid argument",
                                                    response_object => $response
        );
    }
    push(@result_array, \%res_hash);
    return \@result_array;
} ## end sub _snapmirror_status

sub _snapmirror_destinations {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array);

    # For snapmirror_destinations command, some output is always expected.
    # So we are making $can_throw as 1 so that if the lines array got finally
    # in _get_lines is empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    # Convert the raw_output into lines before parsing
    my $first_line = '(Path\s+Destination|Path\s+Snapshot\s+Destination)';
    my @lines = get_lines("raw_output"      => $raw_output,
                          "first_line"      => $first_line,
                          "can_throw"       => $can_throw,
                          "response_object" => $response);

    my @keys_array = split(/\s+/, $lines[0]);    # get the keys
    for (my $i = 0 ; $i < @keys_array ; $i++) {
        $keys_array[$i] = to_key_name("item" => $keys_array[$i]);
    }
    shift(@lines);                               # Remove the heading line
    my @Destinations;
    foreach my $line (@lines) {
        my %hash;
        my @values = split(/\s+/, $line);
        my $i = 0;
        foreach my $item (@keys_array) {
            $hash{$item} = $values[$i] || "";
            $i = $i + 1;
        }
        push(@Destinations, \%hash);
    } ## end foreach my $line (@lines)
    return \@Destinations;
} ## end sub _snapmirror_destinations

## @name ::snapmirror_check_start
##
## @summary Parses the raw output from the "snapmirror check start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_check_start with any of its options as described in the
##    corresponding cdef file 'snapmirror_check.cdefs'
## @synopsis
##      snapmirror_check_start(%args);
## INPUTS TO METHOD CALL 'snapmirror_check_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-S' </td> <td> 'source' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kilobytes' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr>
## <tr> <td> '-h' </td> <td> 'fix-option' </td> </tr>
## <tr> <td> '-F' </td> <td> 'fix' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-L' </td> <td> 'check' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'quick-mode' </td> </tr>
## <tr> <td> '-d' </td> <td> 'snap' </td> </tr>
## <tr> <td> '-w' </td> <td> 'control' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapmirror check start -w -S 10.227.71.48:/vol/vol_test1/qtree123  csiqa-2040-14a:/vol/vol_test1/qtree1;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'snapmirror_check' => 'started',
##&nbsp;             'snapmirror_check_status' => 'completed_successfully'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapmirror_check_start {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my %info       = ();
    my $error = _check_error("raw_output"    => $raw_output,
                             "command"       => "snapmirror check start",
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines =
      get_lines("raw_output" => $raw_output, "response_object" => $resp);
    my ($line, $key, @res_ar);
    my $i = 0;
    foreach $line (@lines) {

        # since we are checking start and commpletion of snamirror check start command  we are
        # neglecting the line "Monitor progress with 'snapmirror status -l' or the snapmirror log."
        # from the output

        if ($line =~ /^snapmirror\s+check\s+(started).$/) {
            $res_ar[$i]->{"snapmirror_check"} = to_key_name(item => $1);
        } elsif ($line =~
            /(Monitor\s+progress\s+with\s+'snapmirror\s+status\s+-l'\s+or\s+the\s+snapmirror\s+log.)/
          ) {
            next;
        } elsif ($line =~ /^snapmirror\s+check\s+(completed\s+successfully).$/)
        {
            $res_ar[$i]->{"snapmirror_check_status"} = to_key_name(item => $1);
        }
    } ## end foreach $line (@lines)
    return \@res_ar;
} ## end sub snapmirror_check_start

## @name ::snapmirror_resync
##
## @summary Parses the raw output from the "snapmirror resync" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_resync with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_resync(%args);
## INPUTS TO METHOD CALL 'snapmirror_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-S' </td> <td> 'source-location' </td> </tr>
## <tr> <td> '-s' </td> <td> 'source-snapshot' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-location' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> </tr>
## <tr> <td> '-c' </td> <td> 'destination-snapshot' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-n' </td> <td> 'test-run' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snapmirror resync -S 10.227.71.52:/vol/vol_test1/qtree123  -w csiqa-2040-16b:/vol/vol_test1/qtree; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'transfer_status' => 'successful'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapmirror_resync {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error = _check_error("raw_output"    => $raw_output,
                             "command"       => "snapmirror_resync",
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines =
      get_lines("raw_output" => $raw_output, "response_object" => $resp);
    my ($line, $key, @res_ar,$status);
    foreach $line (@lines) {
        if ($line =~ /^(Transfer)\s+(started).$/) {
          $status = $2;
          $res_ar[0]->{transfer_status} = $status;
        } elsif ($line =~
            /(Monitor\sprogress\swith\s'snapmirror status'\sor\sthe\ssnapmirror\slog.)/
          ) {
            next;
        } elsif ($line =~ /^(Transfer)\s(successful).$/) {
          $status = $2;
          $res_ar[0]->{transfer_status} = $status;
        } else {
            return [];
        }
      }## end foreach $line (@lines)
    return \@res_ar;
} ## end sub snapmirror_resync

## @name ::snapmirror_check_logs_show
##
## @summary Parses the raw output from the "snapmirror check logs show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_check_logs_show with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_check_logs_show(%args);
## INPUTS TO METHOD CALL 'snapmirror_check_logs_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'verify' </td> </tr>
## <tr> <td> '-a' </td> <td> 'verify-all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;'priv set -q diag; snapmirror check logs show -a dst; result -l'
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;         {
##&nbsp;            'verifier_log_header' => [
##&nbsp;                                       {
##&nbsp;                                         'number_of_mismatches_info' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'mismatch_0' => [
##&nbsp;                                                                                              {
##&nbsp;                                                                                                'source_cksum' => '1448297299',
##&nbsp;                                                                                                'destination_cksum' => '1345624814',
##&nbsp;                                                                                                'type' => 'content mismatch',
##&nbsp;                                                                                                'vbn' => '4'
##&nbsp;                                                                                              }
##&nbsp;                                                                                            ]
##&nbsp;                                                                          },
##&nbsp;                                                                          {
##&nbsp;                                                                            'mismatch_1' => [
##&nbsp;                                                                                              {
##&nbsp;                                                                                                'source_cksum' => '1448297299',
##&nbsp;                                                                                                'destination_cksum' => '1129225753',
##&nbsp;                                                                                                'type' => 'content mismatch',
##&nbsp;                                                                                                'vbn' => '8'
##&nbsp;                                                                                              }
##&nbsp;                                                                                            ]
##&nbsp;                                                                          },
##&nbsp;                                                                          {
##&nbsp;                                                                            'mismatch_2' => [
##&nbsp;                                                                                              {
##&nbsp;                                                                                                'source_cksum' => '1448297299',
##&nbsp;                                                                                                'destination_cksum' => '2064421736',
##&nbsp;                                                                                                'type' => 'content mismatch',
##&nbsp;                                                                                                'vbn' => '28'
##&nbsp;                                                                                              }
##&nbsp;                                                                                            ]
##&nbsp;                                                                          },
##&nbsp;                                                                          {
##&nbsp;                                                                            'mismatch_3' => [
##&nbsp;                                                                                              {
##&nbsp;                                                                                                'source_cksum' => '1448297299',
##&nbsp;                                                                                                'destination_cksum' => '1717471177',
##&nbsp;                                                                                                'type' => 'content mismatch',
##&nbsp;                                                                                                'vbn' => '32'
##&nbsp;                                                                                              }
##&nbsp;                                                                                            ]
##&nbsp;                                                                          },
##&nbsp;                                                                          {
##&nbsp;                                                                            'mismatch_4' => [
##&nbsp;                                                                                              {
##&nbsp;                                                                                                'source_cksum' => '1448297299',
##&nbsp;                                                                                                'destination_cksum' => '-2105574367',
##&nbsp;                                                                                                'type' => 'content mismatch',
##&nbsp;                                                                                                'vbn' => '36'
##&nbsp;                                                                                              }
##&nbsp;                                                                                            ]
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                         'number_of_mismatches' => '5',
##&nbsp;                                         'version' => '2',
##&nbsp;                                         'magic_number' => 'feedbac',
##&nbsp;                                         'snapshot_[0]' => [
##&nbsp;                                                             {
##&nbsp;                                                               'tv_usec' => '807135',
##&nbsp;                                                               'cpcount' => '4',
##&nbsp;                                                               'tv_sec' => '1318849932',
##&nbsp;                                                               'snapid' => '1'
##&nbsp;                                                             }
##&nbsp;                                                           ],
##&nbsp;                                         'block_header' => [
##&nbsp;                                                             {
##&nbsp;                                                               'source_volume' => 'src',
##&nbsp;                                                               'destination_volume' => 'dst',
##&nbsp;                                                               'version' => '2',
##&nbsp;                                                               'destination_filter' => 'csiqa-3170-10a',
##&nbsp;                                                               'isdata' => '0'
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                       }
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'indirect_list_log_header' => [
##&nbsp;                                            {
##&nbsp;                                              'number_of_indirect_blocks_info' => [
##&nbsp;                                                                                    {
##&nbsp;                                                                                      'vvbn[5]' => '29',
##&nbsp;                                                                                      'kireeti[4]' => '176',
##&nbsp;                                                                                      'vvbn[2]' => '26',
##&nbsp;                                                                                      'kireeti[3]' => '176',
##&nbsp;                                                                                      'vvbn[6]' => '30',
##&nbsp;                                                                                      'vvbn[1]' => '11',
##&nbsp;                                                                                      'kireeti[8]' => '192',
##&nbsp;                                                                                      'kireeti[5]' => '176',
##&nbsp;                                                                                      'kireeti[6]' => '176',
##&nbsp;                                                                                      'kireeti[2]' => '176',
##&nbsp;                                                                                      'vvbn[8]' => '32',
##&nbsp;                                                                                      'vvbn[4]' => '28',
##&nbsp;                                                                                      'kireeti[1]' => '185',
##&nbsp;                                                                                      'vvbn[7]' => '31',
##&nbsp;                                                                                      'kireeti[7]' => '177',
##&nbsp;                                                                                      'kireeti[0]' => '184',
##&nbsp;                                                                                      'vvbn[3]' => '27',
##&nbsp;                                                                                      'vvbn[0]' => '10'
##&nbsp;                                                                                    }
##&nbsp;                                                                                  ],
##&nbsp;                                              'version' => '2',
##&nbsp;                                              'magic_number' => 'feedbac',
##&nbsp;                                              'number_of_indirect_blocks' => '9'
##&nbsp;                                            }
##&nbsp;                                          ]
##&nbsp;          }
##&nbsp;        ];
sub snapmirror_check_logs_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error = _check_error("raw_output"    => $raw_output,
                             "command"       => $command,
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines =
      get_lines("raw_output" => $raw_output, "response_object" => $resp);
    my ($line,  $key,   @res_ar, $version_value, $head1,
        $head2, $head3, $head4,  $headn,         $flag1,
        $flag2, $flag3, $flagn);
    my $i = -1;
    my $j = 0;
    my $k = 0;
    my $n = 0;
    foreach $line (@lines) {

        if ($line =~ /^\|\s+(.*)\s+\|$/) {
            $head1 = $1;
            $i++;
            $k = 0;
            $head1 = to_key_name(item => $head1);
            next;
        } elsif ($line =~ /^(-)+$/) {
            next;
        } elsif ($line =~ /^Version:\s+(.*)$/) {
            $version_value = $1;
            if (!$k) {
                $res_ar[$i]->{$head1}->[0]->{"version"} = $version_value;
            } else {
                $res_ar[$i]->{$head1}->[0]->{$head3}->[0]->{"version"} =
                  $version_value;
            }
            $k++;
            next;
        } elsif ($line =~ /^(Magic\s+Number)\s+(.*)$/) {
            $res_ar[$i]->{$head1}->[0]->{to_key_name(item => $1)} = $2;
        } elsif ($line =~ /^(Number\sof\smismatches)\s(\d+)$/) {
            $head2                               = $1;
            $flag1                               = 1;
            $head2                               = to_key_name(item => $head2);
            $res_ar[$i]->{$head1}->[0]->{$head2} = $2;
            next;
        } elsif ($line =~ /^(Number\s+of\s+indirect\s+blocks)\s+(\d+)$/) {
            $headn                               = $1;
            $flagn                               = 1;
            $headn                               = to_key_name(item => $headn);
            $res_ar[$i]->{$head1}->[0]->{$headn} = $2;
            next;
        } elsif ($line =~ /^(Block\s+\S+)$/) {
            $head3 = $1;
            $flag2 = 1;
            $head3 = to_key_name(item => $head3);
            next;
        } elsif ($line =~ /Snapshot(\[\d+\])/) {
            $head4 = $1;
            $flag3 = 1;
            next;
        }
        if ($flag1 == 1) {
            if ($line =~
                /^(\d+):(\S+)\s+(\d+)\s+(Type):\s+(\S+\s+\S+)\s+(Source cksum):\s+(\d+)\s+(Destination cksum):\s+([\d+\-]+)$/
              ) {
                $res_ar[$i]->{$head1}->[0]->{$head2 . "_info"}->[$j]
                  ->{"mismatch_" . $1}->[0]->{to_key_name(item => $2)} = $3;
                $res_ar[$i]->{$head1}->[0]->{$head2 . "_info"}->[$j]
                  ->{"mismatch_" . $1}->[0]->{to_key_name(item => $4)} = $5;
                $res_ar[$i]->{$head1}->[0]->{$head2 . "_info"}->[$j]
                  ->{"mismatch_" . $1}->[0]->{to_key_name(item => $6)} = $7;
                $res_ar[$i]->{$head1}->[0]->{$head2 . "_info"}->[$j]
                  ->{"mismatch_" . $1}->[0]->{to_key_name(item => $8)} = $9;
                $j++;
            } ## end if ($line =~ ...
        } ## end if ($flag1 == 1)
        if ($flag2 == 1) {
            if ($line =~ /^([\w\s]+):\s+([\w\-]+)$/) {
                $res_ar[$i]->{$head1}->[0]->{$head3}->[0]
                  ->{to_key_name(item => $1)} = $2;
            }
        }
        if ($flag3 == 1) {
            if ($line =~ /^(\w+)=(\d+)\s(\w+)=(\d+)$/) {
                $res_ar[$i]->{$head1}->[0]->{"snapshot_" . $head4}->[0]->{$1} =
                  $2;
                $res_ar[$i]->{$head1}->[0]->{"snapshot_" . $head4}->[0]->{$3} =
                  $4;
            }
        }
        if ($flagn == 1) {
            if ($line =~ /^(\w+\[\d+\])=(\d+)\s+(\w+\[\d+\])=(\d+)$/) {
                $res_ar[$i]->{$head1}->[0]->{$headn . "_info"}->[0]->{$1} = $2;
                $res_ar[$i]->{$head1}->[0]->{$headn . "_info"}->[0]->{$3} = $4;
            }
        }
    } ## end foreach $line (@lines)
    return \@res_ar;
} ## end sub snapmirror_check_logs_show

## @name ::snapmirror_migrate
##
## @summary Parses the raw output from the "snapmirror migrate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_migrate with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_migrate(%args);
## INPUTS TO METHOD CALL 'snapmirror_migrate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'test-run' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_migrate {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_migrate

## @name ::snapmirror_check_abort
##
## @summary Parses the raw output from the "priv set -q diag;snapmirror check abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_check_abort with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_check_abort(%args);
## INPUTS TO METHOD CALL 'snapmirror_check_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_check_abort {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_check_abort

## @name ::snapmirror_check_logs_delete
##
## @summary Parses the raw output from the "snapmirror check logs delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_check_logs_delete with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_check_logs_delete(%args);
## INPUTS TO METHOD CALL 'snapmirror_check_logs_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_check_logs_delete {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_check_logs_delete

## @name ::snapmirror_nvlogs
##
## @summary Parses the raw output from the "snapmirror nvlogs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_nvlogs with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_nvlogs(%args);
## INPUTS TO METHOD CALL 'snapmirror_nvlogs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'delete-nvlog' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_nvlogs {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_nvlogs

## @name ::snapmirror_trace_dump
##
## @summary Parses the raw output from the "snapmirror trace dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_trace_dump with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_trace_dump(%args);
## INPUTS TO METHOD CALL 'snapmirror_trace_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_trace_dump {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_trace_dump

## @name ::snapmirror_trace_dump_autodump
##
## @summary Parses the raw output from the "snapmirror trace dump autodump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_trace_dump_autodump with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_trace_dump_autodump(%args);
## INPUTS TO METHOD CALL 'snapmirror_trace_dump_autodump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'autodump-enable' </td> <td> '0' </td> </tr>
## <tr> <td> '1' </td> <td> 'autodump-disable' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_trace_dump_autodump {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_trace_dump_autodump

## @name ::snapmirror_trace_status
##
## @summary Parses the raw output from the "snapmirror trace status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_trace_status with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_trace_status(%args);
## INPUTS TO METHOD CALL 'snapmirror_trace_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_trace_status {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_trace_status

## @name ::snapmirror_trace_set
##
## @summary Parses the raw output from the "snapmirror trace set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_trace_set with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_trace_set(%args);
## INPUTS TO METHOD CALL 'snapmirror_trace_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'trace-module' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_trace_set {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_trace_set

## @name ::snapmirror_convert
##
## @summary Parses the raw output from the "snapmirror convert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_convert with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_convert(%args);
## INPUTS TO METHOD CALL 'snapmirror_convert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_convert {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_convert

## @name ::snapmirror_help
##
## @summary Parses the raw output from the "help snapmirror" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapmirror_help with any of its options as described in the
##    corresponding cdef file 'snapmirror.cdefs'
## @synopsis
##      snapmirror_help(%args);
## INPUTS TO METHOD CALL 'snapmirror_help' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapmirror_help {
    my %args = @_;
    my $error_message;
    my $response   = $args{response_object};
    my $command    = $response->get_actual_command();
    my $raw_output = $response->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapmirror_help


sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if ($raw_output =~ /$command(?:[^:]*):(.*)/) {
        $$error_message = $1;
        if ($$error_message =~ /(Invalid option|only one option)/) {
            $error = "InvalidParam";
        } elsif ($$error_message =~ /(Unrecognized option|Duplicate option)/) {
            $error = "InvalidParam";
        } elsif ($$error_message =~ /(not a snapmirror qtree)/) {
            $error = "CommandFailed";
        } elsif (($raw_output !~ /Current Transfer Error:/)
              && ($$error_message =~ /(offline, is restricted|already idle)/)) {
            $error = "CommandFailed";
        } elsif ($$error_message =~ /(destination (is|was) not quiesced)/) {
            $error = "CommandFailed";
        } elsif ($$error_message =~ /(no known destinations)/) {
            $error = "CommandFailed";
        } elsif ($$error_message =~
                 /(couldn't find destination with active transfer)/) {
            $error = "CommandFailed";
        } elsif ($$error_message =~ /(couldn't find destination)/) {
            $error = "InvalidParamValue";
        } elsif (
              $$error_message =~ /(Invalid throttle value|path must include)/) {
            $error = "InvalidParamValue";
        } elsif ($$error_message =~ /(argument must be)/) {
            $error = "InvalidParamValue";
        } elsif ($$error_message =~ /(No release-able destination)/) {
            $error = "InvalidParamValue";
        } elsif (
              $$error_message =~ /is currently involved in another operation/) {
            $error = 'CommandFailed';
        } elsif ($$error_message =~
            /(transfer attempted from busy source)|(volume is not online; cannot execute operation)/
          ) {
            $error = 'CommandFailed';
        }elsif ( $raw_output =~ /(snapmirror migrate: Bad source argument)/i ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
        } elsif ( $raw_output
        =~ /(snapmirror migrate: Source and destination arguments required)/i
        ){
        $error          = "MissingRequiredParam";
        $$error_message = $1;
        }elsif ( $raw_output =~ /(snapmirror nvlogs: Destination must be volumes)/i ) {
             $error          = "InvalidParamValue";
             $$error_message = $1;
        } elsif ( $raw_output =~ /(snapmirror nvlogs: destination is offline, is restricted, or does not exist)/i ) {
             $error          = "CommandFailed";
             $$error_message = $1;
        } elsif ( $raw_output =~ /(snapmirror: unrecognized command)/i ) {
             $error          = "InvalidParam";
             $$error_message = $1;
        } elsif ( $raw_output =~ /(snapmirror check logs: Volume a does not exist)/i ) {
             $error          = "InvalidParamValue";
             $$error_message = $1;
        }
        $$error_message =~ s/\cM//g;
        return $error if (length($error));
    } ## end if ($raw_output =~ /$command(?:[^:]*):(.*)/)
    if ($raw_output =~ /(Source not set.+)/) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Destination path specification required)/) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Invalid argument to the -k.+)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.+not of the same type|Invalid destination.+)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Illegal destination filer.+)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(couldn't find destination.+|.+not a valid.+)/) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /(automatically created snapshot.+)/) {
        $error          = "InvalidParamValue";
        $$error_message = "Cannot transfer a snapshot with a name in the
            format of an $1";
    } elsif ($raw_output =~ /(Cannot specify a snapshot.+|Too many arguments)/)
    {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Invalid option.+|option requires an argument.+)/)
    {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Unrecognized option.+|Duplicate option.*)/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Extra Arguments specified)/) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Duplicate snapshot.+|.+is turned off)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.+not a snapmirror qtree.+)/) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif (   ($raw_output !~ /Current Transfer Error:/)
             && ($raw_output =~ /(.+offline, is restricted.+)/)) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif (   ($raw_output !~ /Current Transfer Error:/)
             && ($raw_output =~ /(.+not in snapmirrored state)/)) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(Transfer aborted: .+)/) {
        $$error_message = $1;
        my $msg = $$error_message;
        if ($msg =~ /(does not exist)/) {
            $error = "InvalidParamValue";
        } else {
            $error = "CommandFailed";
        }
    } elsif ($raw_output =~ /(Transfer failed: .+)/) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ($raw_output =~ /(no such command.+|snapmirror is not licensced)/) {
        $error          = "FeatureNotLicensed";
        $$error_message = $1;
    } elsif ($raw_output =~ /usage:/) {
        $error          = "CommandFailed";
        $$error_message = "Failure may be because of missing required params
            or invalid param or invalid param value";
    } elsif ($raw_output =~ /usage.+/si) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ($raw_output =~ /^(Too many arguments.)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif (
        $raw_output =~ /^(Source not set on command line or in conf file.)/si) {
        $error          = "MissingRequiredParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Destination path specification required.)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Invalid destination.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(snapmirror check aborted.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Cannot specify quick mode .*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Invalid comparison level specification.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Incompatible options.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Invalid argument .*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Transfer aborted.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(Illegal destination filer specification.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ($raw_output =~ /^(qtree is not a snapmirror qtree replica.*)/si) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif (($raw_output =~
        /^(Snapmirror resynchronization of.*: no common snapshot to use as the base for resynchronization)/si
      )||($raw_output =~
        /^(Snapmirror resynchronization of.*: destination is offline, is restricted, or does not exist)/si
      )||($raw_output =~
        /^(Snapmirror resynchronization of.*: transfer aborted because of network error)/si
      )||($raw_output =~
         /^(Snapmirror resynchronization of.*: destination volume is root)/si
      )||($raw_output =~
        /^(Snapmirror resynchronization of.*: could not register source qtree information)/si
      )||($raw_output =~
        /^(Snapmirror resynchronization of.*: destination requested snapshot that does not exist on the source)/si
      )||($raw_output =~
             /^(snapmirror check logs: Verifier log does not exist.*)/si
	  )||(
         $raw_output =~ /^(snapmirror check logs: Volume.*does not exist.)/si)) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output =~ /No trace records to dump/i ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(type "snapmirror help <command>")/i ) {
             $error          = "InvalidParam";
             $$error_message = $1;
    } else {
        return undef;
    }

    # Remove ctrl-M from error message which is taken from raw output.
    $$error_message =~ s/\cM//g;
    $$error_message =~ s/^$command:\s//;
    return $error;
} ## end sub _check_error

1;
