# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/snapvault.pm# $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary
## Routines to parse 7Mode snapvault commands
##
## @description
##  This is a library to parse the raw outputs of any of the 7Mode snapvault
##  commands.

## @synopsis
##    use snapvault;
##
##    snapvault_start(%args);
##    snapvault_update(%args);
##    snapvault_status(%args);
##    snapvault_abort(%args);
##    snapvault_modify(%args);
##    snapvault_destinations(%args);
##    snapvault_snap_preservations(%args);
##    snapvault_stop(%args);
##    snapvault_snap_preserve(%args);
##    snapvault_snap_unpreserve(%args);
##    snapvault_snap_sched(%args);
##    snapvault_snap_unsched(%args);
##    snapvault_prerevert(%args);
##    snapvault_restore(%args);
##    snapvault_snap_cretae(%args);
##    snapvault_release(%args);
##    snapvault_check_abort(%args);
##    snapvault_convert(%args);
##    snapvault_check_start(%args);
##    snapvault_snap_retain(%args);
##
## @author udaykk@netapp.com, dl-dpg-nb-automation; dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::snapvault;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);

## @name ::snapvault_start
##
## @summary Parses the raw output from the "snapvault start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_start with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_start(%args);
## INPUTS TO METHOD CALL 'snapvault_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> </tr>
## <tr> <td> '-t' </td> <td> 'try-count' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> </tr>
## <tr> <td> '-r' </td> <td> 'restart-backup' </td> </tr>
## <tr> <td> '-S' </td> <td> 'primary-location' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapvault_start {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault start",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
}

## @name ::snapvault_update
##
## @summary Parses the raw output from the "snapvault update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_update with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_update(%args);
## INPUTS TO METHOD CALL 'snapvault_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapvault_update {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault update",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
}

## @name ::snapvault_status
##
## @summary Parses the raw output from the "snapvault status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_status with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_status(%args);
## INPUTS TO METHOD CALL 'snapvault_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-m' </td> <td> 'success-status' </td> </tr>
## <tr> <td> '-q' </td> <td> 'qtree' </td> </tr>
## <tr> <td> '-t' </td> <td> 'active-transfers' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long' </td> </tr>
## <tr> <td> '-c' </td> <td> 'configuration' </td> </tr>
## <tr> <td> ''   </td> <td> 'system-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snapvault_status {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault status",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _snapvault_status(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

sub _snapvault_status {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array);

    # For snapvault_status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw =1;

    # Extracting the command option. since snapvault status cannot be
    # invoked with more than one option, this simple logic will work.
    $command =~ s/snapvault\sstatus//;
    
    # If there are no options, default to an empty string
    if ($command =~ /\s(-\w)/) {
        $command = $1;
    } else {
        $command = '';
    }

    # Convert the raw_output into lines before parsing
    my $first_line = 'Snapvault .+';
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    # Always the first line is about the status of snapvault
    if ($lines[0] =~ /Snapvault primary is (on|off)/i){
        $res_hash{"snapvault_status"} = $1;
    } elsif ($lines[0] =~ /Snapvault is (on|off)/i) {
        $res_hash{"snapvault_status"} = $1;
    }
    shift(@lines); # Remove the first line which is processed already
    if((scalar(@lines) == 1) ||($lines[0] =~ /There are no active transfers/i)) {
        # Currently Snapvault status is not throwing any error
        # on specifying invalid qtree path
        # The output will just contain whether snapvault is turned off
        # or on as the first line.
        # On supplying a qtree path or volume name which hasn't acted as
        # a snapvault source, we will get details about status of
        # snapvault only as the previous case.
        # Hence its not able to differentiate the error condition.
        $res_hash{"snapvault_entries"} = [];
        push(@result_array, \%res_hash);
        return \@result_array;
    } elsif (($command eq "") || ($command =~ /-t/) || ($command =~ /-q/)) {
        my @keys_array = split(/\s+/,$lines[0]); # get the keys
        my $item;
        for (my $i=0; $i<@keys_array; $i++) {
            $keys_array[$i] = to_key_name("item" => $keys_array[$i]);
        }
        shift (@lines); # Remove the heading line
        my @vaulted_entries;
        foreach my $line(@lines) {
            my %hash;
            my @values = split(/\s+/,$line);
            my $i = 0;
            foreach $item (@keys_array) {
                $hash{$item} = $values[$i] || "";
                $i = $i+1;
            }
            if(defined($values[$i]) && $command !~ /-q/) {
                ($hash{"data_transferred"}) = ($values[$i] =~ /\((\d+)/i);
            }
            push (@vaulted_entries, \%hash);
        }
        $res_hash{"snapvault_entries"} = \@vaulted_entries;
    } elsif($command =~ /-l/ || $command =~ /-m/) {
        # First split the data into logical blocks
        my (@Entry, $index, $temp_ref);
        for(my $i=0;$i<@lines;$i++) {
            next if ($lines[$i] eq "");
            if($lines[$i] =~ /^Source:/) {
                # The next logical block has started
                $Entry[$index] = [];
                $temp_ref=$Entry[$index++];
                push(@$temp_ref, $lines[$i]);
            } else {
                push(@$temp_ref, $lines[$i]);
            }
        }
        # Now process the data in each block
        my @vaulted_entries;
        foreach my $ref (@Entry) {
            my @data = @$ref;
            my %hash;
            for(my $j=0; $j<@data; $j++) {
                if ($data[$j] =~ /(.+):\s+(.+)/) {
                    my $key = $1;
                    my $value= $2;
                    $key = to_key_name("item" => $key);
                    $hash{$key} = $value;
                }
            }
            push(@vaulted_entries, \%hash);
        }
        $res_hash{"snapvault_entries"} = \@vaulted_entries;
    } elsif ($command =~ /-c/) {
        my @entries;
        my $item;
        my @vaulted_entries;
        foreach my $line(@lines) {
            my (%hash, $key, $val);
            my @values = split(/[\s,]/,$line);
            $hash{"destination"} = $values[0];
            shift(@values);
            foreach $item (@values) {
                ($key, $val) = split (/=/, $item, 2);
                $key = to_key_name("item" => $key);
                $hash{$key} = $val;
            }
            push (@vaulted_entries, \%hash);
        }
        $res_hash{"snapvault_entries"} = \@vaulted_entries;
    } else {
         NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                      text =>"Output is not in the expected format",
                                       response_object => $response);
    }
    push(@result_array, \%res_hash);
    return \@result_array;
}

## @name ::snapvault_abort
##
## @summary Parses the raw output from the "snapvault abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_abort with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_abort(%args);
## INPUTS TO METHOD CALL 'snapvault_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'clear-checkpoint' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapvault abort -f -h /vol/vol0/qtreetest.12291
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'already idle',
##&nbsp;            'destination' => '/vol/vol0/qtreetest.12291'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_abort {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault abort",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my %info;
    my @lines = get_lines(raw_output=>$output, response_object=>$response);
    foreach $value (@lines) {
        if($value =~ /This\s+may\s+take\s+a\s+long\s+time.*/i) {
            ##Do nothing
        } elsif($value =~ /destination\s+is\s+((\w+\s+)?\w+):(\s+)?(.*)/i) {
            $info{"destination"} = $4;
            $info{"status"} = $1;
            push(@result_array, \%info);
        }
    }

    return \@result_array;
}

## @name ::snapvault_modify
##
## @summary Parses the raw output from the "snapvault modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_modify with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_modify(%args);
## INPUTS TO METHOD CALL 'snapvault_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'transfer' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'write-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'primary-location' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapvault modify -k 20 -S fas3170-210-09:/vol/vol0 /vol/vol0/qtreetest.17581
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tries' => '2',
##&nbsp;            'source' => 'fas3170-210-09:/vol/vol0',
##&nbsp;            'utf8_primary_path' => 'off',
##&nbsp;            'connection_mode' => 'unspec',
##&nbsp;            'wsize' => '0',
##&nbsp;            'qtree' => '/vol/vol0/qtreetest.17581',
##&nbsp;            'back_up_open_files' => 'on',
##&nbsp;            'kbs' => '20',
##&nbsp;            'ignore_atime' => 'off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_modify {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault modify",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my %info;
    my @lines = get_lines(raw_output=>$raw_output, response_object=>$response);
    foreach $value (@lines) {
        if($value =~ /(No\s+changes\s+in\s+the\s+configuration.*)/i) {
            $info{"info"} = $1;
        } elsif($value =~ /Configuration\s+for\s+qtree.*/i) {
            ##do nothing
        } elsif($value =~ /(.*)\s+source=(.*)\s+kbs=(\d+)\s+tries=(\d+)\s+connection_mode=(\w+)\s+wsize=(\d+)\s+back_up_open_files=(\w+),ignore_atime=(\w+),utf8_primary_path=(\w+)/i) {
            $info{"qtree"} = $1;
            $info{"source"} = $2;
            $info{"kbs"} = $3;
            $info{"tries"} = $4;
            $info{"connection_mode"} = $5;
            $info{"wsize"} = $6;
            $info{"back_up_open_files"} = $7;
            $info{"ignore_atime"} = $8;
            $info{"utf8_primary_path"} = $9;
        }
    }
    push(@result_array, \%info);

    return \@result_array;
}

## @name ::snapvault_destinations
##
## @summary Parses the raw output from the "snapvault destinations" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_destinations with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_destinations(%args);
## INPUTS TO METHOD CALL 'snapvault_destinations' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'primary-path' </td> </tr>
## <tr> <td> '-s' </td> <td> 'list-snapshot' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapvault destinations
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'destination' => 'fas3170-210-10:/vol/vol0/qtree',
##&nbsp;            'path' => '/vol/vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = snapvault destinations -s
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'destination' => 'fas3170-210-10:/vol/api/qtree2',
##&nbsp;            'snapshot' => 'fas3170-210-10(0151758368)_api_qtree2-src.0',
##&nbsp;            'path' => '/vol/api/qtree2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_destinations {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault destinations",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    if ($output =~ /snapvault\s+destinations:\s+no\s+known\s+destinations/i){
        my %info;
        $info{"status"} = "no known destinations";
        push(@result_array, \%info);
    } elsif($output =~ /Path\s+Destination.*/i) {
        my @lines = get_lines(raw_output=>$output, response_object=>$response);
        foreach $value (@lines) {
            my %info = ();
            if($value =~ /((\/\w+)+).+\s+(.*)/i) {
                $info{"path"} = $1;
                $info{"destination"} = $3;
                push(@result_array, \%info);
            }
        }
    } elsif($output =~ /Path\s+Snapshot\s+Destination/i) {
        my @lines = get_lines(raw_output=>$output, response_object=>$response);
        foreach $value (@lines) {
            my %info = ();
            if($value =~ /((\/\w+(\d+)?)+)\s+(.*)\s+(.*)/i) {
                $info{"path"} = $1;
		$info{"snapshot"} = $4;
                $info{"destination"} = $5;
                push(@result_array, \%info);
            }
        }
    }
    return \@result_array;
}

## @name ::snapvault_snap_preservations
##
## @summary Parses the raw output from the "snapvault snap preservations" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_preservations with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_preservations(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_preservations' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault snap preservations vol0 fas3170-210-09(0151758294)_vol0_qtree-dst.0
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tag_name' => 'tag0',
##&nbsp;            'type' => 'cli'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_snap_preservations {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap preservations",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    if ($output =~ /(No\s+preservations\s+on\s+snapshot\s+.*)/i){
        my %info;
        $info{"status"} = $1;
        push(@result_array, \%info);
    } elsif($output =~ /type\s+tagname.*/i) {
        my @lines = get_lines(raw_output=>$output, response_object=>$response);
        shift @lines;
        shift @lines;
        foreach $value (@lines) {
            my %info = ();
            if($value =~ /(\w+)(\s+(\w+))?/i) {
                $info{"type"} = $1;
                if($3) {
                    $info{"tag_name"} = $3;
                }
                push(@result_array, \%info);
            }
        }
    } elsif($output =~ /snapid\s+snapname.*/i) {
        my @lines = get_lines(raw_output=>$output, response_object=>$response);
        shift @lines;
        shift @lines;
        foreach $value (@lines) {
            my %info = ();
            if($value =~ /(\d+)\s+(.*)/i) {
                $info{"snap_id"} = $1;
                $info{"snap_name"} = $2;
                push(@result_array, \%info);
            }
        }
    }
    return \@result_array;
}

## @name ::snapvault_stop
##
## @summary Parses the raw output from the "snapvault stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_stop with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_stop(%args);
## INPUTS TO METHOD CALL 'snapvault_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault stop -f /vol/vol0/qtreetest.232131
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'configuration for the qtree has been deleted'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_stop {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault stop",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my %info;
    my @lines = get_lines(raw_output=>$output, response_object=>$response);
    foreach $value (@lines) {
        if($value =~ /Snapvault\s+configuration\s+for\s+the\s+qtree\s+has\s+been\s+deleted./i) {
            $info{"status"} = "configuration for the qtree has been deleted";
        } elsif($value =~ /(Protection\s+to\s+(\/\w+)*\s+.*)/i) {
            $info{"info"} = $1;
        }
    }
    push(@result_array, \%info);

    return \@result_array;
}

## @name ::snapvault_snap_preserve
##
## @summary Parses the raw output from the "snapvault snap preserve" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_preserve with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_preserve(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_preserve' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tag-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault snap preserve vol0 fas3170-210-09(0151758294)_vol0_qtreetest.197971-dst.0
##&nbsp;$VAR1 = [];
##&nbsp;
sub snapvault_snap_preserve {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap preserve",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::snapvault_snap_unpreserve
##
## @summary Parses the raw output from the "snapvault snap unpreserve" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_unpreserve with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_unpreserve(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_unpreserve' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-all' </td> <td> 'preserve-all' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'tag-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapvault snap unpreserve api fas3170-210-10(0151758368)_api-base.1.0
##&nbsp;$VAR1 = [];
##&nbsp;
sub snapvault_snap_unpreserve {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap unpreserve",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::snapvault_snap_sched
##
## @summary Parses the raw output from the "snapvault snap sched" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_sched with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_sched(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_sched' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'schedule' </td> </tr>
## <tr> <td> '-x' </td> <td> 'start-transfer' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault snap sched -f -x vol0 fas3170-210-10(0151758368)_vol0_qtreename1-dst.0 10@0-45/23
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = snapvault snap sched
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume_name' => "vol0",
##&nbsp;            'snap_name' => "fas3170-210-10(0151758368)_vol0_qtreename1-dst.0",
##&nbsp;            'schedule' => "0@-",
##&nbsp;            'status' => "create",
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_snap_sched {
    my %args         = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $raw_output = $resp->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap sched",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $value;
    my %info;
    my @lines = get_lines(raw_output=>$output, response_object=>$resp);
    foreach $value (@lines) {
        if($value =~ /(\w+)\s+(\w+)\s+(.*)\s+(.*)/) {
            $info{"volume_name"} = $2;
            $info{"status"} = $1;
	    $info{"snap_name"} = $3;
	    $info{"schedule"} = $4;
            push(@result_array, \%info);
        }
    }

    return \@result_array;
}

## @name ::snapvault_snap_unsched
##
## @summary Parses the raw output from the "snapvault snap unsched" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_unsched with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_unsched(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_unsched' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault snap unsched -f vol0 fas3170-210-10(0151758368)_vol0_qtreename1-dst.0
##&nbsp;$VAR1 = [];
##&nbsp;
sub snapvault_snap_unsched {
    my %args         = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $raw_output = $resp->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap unsched",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snapvault_prerevert
##
## @summary Parses the raw output from the "snapvault prerevert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_prerevert with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_prerevert(%args);
## INPUTS TO METHOD CALL 'snapvault_prerevert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault prerevert -f
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => "OSSV is ready for revert on all online volumes"
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_prerevert {
    my %args         = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $raw_output = $resp->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault prerevert",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $value;
    my %info;
    if($output =~ /(OSSV\s+is\s+ready\s+for\s+revert\s+on\s+all\s+online\s+volumes).*/i) {
		$info{"status"} = $1;
		push(@result_array,\%info);
    }

    return \@result_array;
}

## @name ::snapvault_restore
##
## @summary Parses the raw output from the "snapvault restore" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_restore with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_restore(%args);
## INPUTS TO METHOD CALL 'snapvault_restore' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'incremental-restore' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'write-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'snapshot' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'primary-location' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = snapvault restore -r -f -S fas3170-210-10:/vol/api/qtree2 fas3170-210-09:/vol/api/qtree2
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => "transfer started"
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_restore {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $raw_output = $resp->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault restore",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info;
    if($output =~ /.*Transfer started.*/i) {
        $info{"status"} = "transfer started";
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(text=>"unexpected format of output",response_object=>$resp);
    }
    push(@result_array, \%info);

    return \@result_array;
}

## @name ::snapvault_snap_create
##
## @summary Parses the raw output from the "snapvault snap create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_create with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_create(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> </tr>
## <tr> <td> '-o' </td> <td> 'options' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault snap create api fas3170-210-10(0151758368)_api-base.1
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'snapshot creation started'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub snapvault_snap_create {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output = $response->get_processed_output();
    my $raw_output = $response->get_processed_output();
    my @result_array;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap create",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my %info;
    if($output =~ /.*Snapshot\s+creation\s+started.*/i) {
	$info{"status"} = "snapshot creation started";
	push(@result_array,\%info);
    } else {
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(text=>"unexpected format of output",response_object=>$response);
    }

    return \@result_array;
}

## @name ::snapvault_release
##
## @summary Parses the raw output from the "snapvault release" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_release with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_release(%args);
## INPUTS TO METHOD CALL 'snapvault_release' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'primary-path' </td> </tr> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> </tr> '0' </td> </tr>
## <tr> <td> This option tells whether the snapvault release is run on a primary or secondary. 
## Default is 1, i.e primary </td> <td> 'is-primary' </td> </tr> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault release /vol/api/qtree2 fas3170-210-10:/vol/api/qtree2
##&nbsp;$VAR1 = [];
##&nbsp;
sub snapvault_release {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault release",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
}

## @name ::snapvault_check_abort
##
## @summary Parses the raw output from the "snapvault check abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_check_abort with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_check_abort(%args);
## INPUTS TO METHOD CALL 'snapvault_check_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snapvault check abort fas3170-210-10:/vol/ap1/qtree
##&nbsp;$VAR1 = [];
##&nbsp;
sub snapvault_check_abort {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault check abort",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # If there is no exception thrown and this part of the code is reached,
    # then it implies that the command had succeeded.
    # Returning empty array of hash implies, command is succeeded.
    return [];
}

## @name ::snapvault_convert
##
## @summary Parses the raw output from the "snapvault convert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_convert with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_convert(%args);
## INPUTS TO METHOD CALL 'snapvault_convert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'qtree-path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapvault_convert {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault convert",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapvault_convert

## @name ::snapvault_check_start
##
## @summary Parses the raw output from the "snapvault check start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_check_start with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_check_start(%args);
## INPUTS TO METHOD CALL 'snapvault_check_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'ctime' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'secondary-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-F' </td> <td> 'fix' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'max-transfer-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'primary-location' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'check' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'snapname' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'quick-mode' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapvault_check_start {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault check start",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapvault_check_start

## @name ::snapvault_snap_retain
##
## @summary Parses the raw output from the "snapvault snap retain" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snapvault_snap_retain with any of its options as described in the
##    corresponding cdef file 'snapvault.cdefs'
## @synopsis
##      snapvault_snap_retain(%args);
## INPUTS TO METHOD CALL 'snapvault_snap_retain' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'snap-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vol-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub snapvault_snap_retain {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"snapvault snap retain",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    return [];
} ## end sub snapvault_snap_retain

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if (($raw_output =~ /.*?(Unrecognized option).+/i) or
        ($raw_output =~ /.*?(Too few arguments).+/i) or
        ($raw_output =~ /.*?(Destination path specification required).+/i) or
        ($raw_output =~ /.*?(Both volume and snapshot name must be specified).+/i) or
        ($raw_output =~ /.*?(No volume specified).+/i) ){
        $error = "MissingRequiredParam";
        $$error_message = $1;
    } elsif (( $raw_output =~ /.*?(source qtree does not exist.+)/i) or
        ($raw_output =~ /.*?(Secondary path.+?already configured and initialized.+)/i) or
        ($raw_output =~ /.*?(The path.+?is in a non-existent volume.+)/i) or
        ($raw_output =~ /.*?(Source filer not specified.+)/i) or
        ($raw_output =~ /.*?(Transfer aborted:.+)/i)  or
        ($raw_output =~ /.*?(No matching snapshot schedule found.*.+)/i) or
        ($raw_output =~ /.*?(snapshot target not configured+)/i)  or
        ($raw_output =~ /.*?(Transfer failed: replication destination write failed.+)/i) or
        ($raw_output =~ /.*?(initialization is not complete.+)/i) or
        ($raw_output =~ /.*?(snapshot target not configured.*)/i) or
        ($raw_output =~ /.*?(Snapvault must be turned off to convert.+)/i)  or
        ($raw_output =~ /.*?(couldn't find destination.+)/i) or
        ($raw_output =~ /.*?(destination is not in snapmirrored state.+)/i) or
        ($raw_output =~ /.*?(The qtree \/.* is not configured in snapvault.)/i) or
        ($raw_output =~ /.*?(Release must be done on an idle or broken-off relationship)/i) or
        ($raw_output =~ /.*?(Use the 'snapvault start' command to configure the qtree.+)/i) or
        ($raw_output =~ /.*?(Error locating specified volume:\s+couldn't find destination.+)/i) or
        ($raw_output =~ /.*?(Could not delete qtree: destination qtree does not exist.+)/i) or
        ($raw_output =~ /.*?(Could not unconfigure.+)/i) or
        ($raw_output =~ /.*?((Invalid path:.+\n)?(usage).+\n(This command is available only on a snapvault secondary.+))/i)) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(couldn't\s+find\s+destination.*)|(If specified, filer must match local host.*)|(Volume .* does not exist or is not online.*)|(snapshot .* does not exist.*)|(.*Improper value \w+ specified for option ignore_atime.*)|(.*Invalid path:.*)|(.*Illegal destination filer specification.*)|(.*Transfer aborted: source is not a valid name or qtree path.*)|(.*Invalid schedule argument.*)|(The volume \w+ does not exist.*)|(No matching snapshot schedule found.*)|(snapshot .* does not exist.*)|(Volume \w+ does not exist or is not online.*)|(.*No release-able destination found that matches those parameters.*)|(.*is not a valid primary path.*)|(.*Improper (source|destination) specification.*)|(.*path is not a qtree.*)|(.*Invalid qtree path.*)|(.*qtree is not a snapmirror qtree replica.*)/i) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif( $raw_output =~ /Snapvault secondary is not licensed/){
        $error ="FeatureNotLicensed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid option|usage/i) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(.*path must include filer, and volume or qtree.*)|(.*If specified, filer must match local host.*)|(.*Improper path specification:.*)|(.*The volume \w+ does not exist.*)|(Invalid argument.*)|(.*Too many arguments.*)|(.*No preserved snapshots.*)|(.*No preservations on snapshot.*)/i) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    } else {
        return undef;
    }
    $$error_message =~ s/^$command:\s//;
    return $error;
}

1;
