#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/snmp.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode 'snmp' commands
##
## @description
##    This is a library to parse the raw output of certain 7Mode 'snmp ' commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "snmp community add" will be snmp_community_add()
##
## @synopsis
##    use snmp;
##
##    snmp_community(%args);
##    snmp_community_add(%args);
##    snmp_community_delete(%args);
##    snmp();
##    snmp_init(%args);
##    snmp_contact(%args);
##    snmp_location(%args);
##    snmp_authtrap(%args);
##    snmp_traphost_add(%args);
##    snmp_traphost_delete(%args);
##
## @author kathar@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::snmp;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::_check_error
##
## @summary Parses the raw output from "snmp community *" commands and
##          returns the appropriate exception name, if an error was
##          encountered.
##
## @description
##    This method can be used to return whether the "snmp community *" commands
##    were initiated successfully or not. The cdef file is at
##    lib/NACL/APISet/Node/CLI/7Mode/default/snmp.cdefs
##
## @synopsis
##    _check_error(%args);
##
## @status private
##
## @arg %args Required: Hash containing the data for raw output to be parsed
## and a ref to an empty error message string.
##
## @return Returns the exception name, if an error was encountered.  Also
## sets a ref to the error message string that was detected in the raw
## output.

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(snmp:\sNo\spermission\sargument\sgiven.\s\sUse\sro.)/ )
    {
        $error = "MissingRequiredParam";
    }
    elsif ( $output
        =~ /((Action\s\".+\"\sisn\'t\svalid.\sUse\sadd\sor\sdelete)|(Permissions\s\".+\"\saren\'t\svalid.\sOnly\s\".+\"\ssupported.))/
        )
    {
        $error = "InvalidParamValue";
    }
    elsif ( $output
        =~ /((No more space for new communities.*)|(Did not find community \".+\" to delete))/
        )
    {
        $error = "CommandFailed";
    }
    elsif ( $output =~ /(.*illegal value passed to snmp init)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $output =~ /(Did not recognize option .*)/ ) {
        $error = "InvalidParam";
    }
    elsif ( $output =~ /(.* already exists in list of trap hosts)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $output =~ /(Unknown host .*)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $output =~ /(Specify a host name or ipaddress)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $output =~ /(Did not recognize .*)/ ) {
        $error = "InvalidParam";
    }
    elsif ( $output =~ /(Did not find the host name .* for deletion.)/ ) {
        $error = "InvalidParamValue";
    }
    elsif ( $output =~ /(illegal value passed.*)/ ) {
        $error = "InvalidParamValue";
    }

    $$error_message = $1;
    return $error;
}

## @name ::snmp_community
##
## @summary Parses the raw output from the "snmp community" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_community(%args);
## INPUTS TO METHOD CALL 'snmp_community' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snmp_community {
    my %args = @_;

    my ( $error_message, @lines, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $output =~ /ro\s.+/ ) {
        my $first_line = '\s+ro\s.+';

        @lines = get_lines(
            raw_output      => $output,
            first_line      => $first_line,
            can_throw       => 1,
            response_object => $resp
        );

        foreach my $elem (@lines) {
            my ( $permissions, $community );
            ( $permissions, $community ) = split /\s/, $elem;
            push @result_array,
                { permissions => $permissions, community => $community };
        }
        return \@result_array;
    }
    else {
        return [];
    }
}

## @name ::snmp_community_add
##
## @summary Parses the raw output from the "snmp community add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_community_add(%args);
## INPUTS TO METHOD CALL 'snmp_community_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'community' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'permission' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp community add ro public ; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub snmp_community_add {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::snmp_community_delete
##
## @summary Parses the raw output from the "snmp community delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_delete with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_community_delete(%args);
## INPUTS TO METHOD CALL 'snmp_community_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'community' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'permission' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub snmp_community_delete {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_init
##
## @summary Parses the raw output from the "snmp init" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_init(%args);
## INPUTS TO METHOD CALL 'snmp_init' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'init' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp init 1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_init {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_contact
##
## @summary Parses the raw output from the "snmp contact" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp_contact' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'contact' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp contact public; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_contact {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_authtrap
##
## @summary Parses the raw output from the "snmp authtrap" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp_authtrap' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'authtrap' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp authtrap 1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_authtrap {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_location
##
## @summary Parses the raw output from the "snmp location" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp_location' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'location' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp location nb; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_location {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_traphost_add
##
## @summary Parses the raw output from the "snmp traphost add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp_traphost_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp traphost add 10.72.219.44; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_traphost_add {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

## @name ::snmp_traphost_delete
##
## @summary Parses the raw output from the "snmp traphost delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp_traphost_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp traphost delete 10.72.219.44; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub snmp_traphost_delete {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::snmp_traphost_delete
##
## @summary Parses the raw output from the "snmp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_community_add with any of its options as described in the
##    corresponding cdef file 'snmp.cdefs'
## @synopsis
##      snmp_contact(%args);
## INPUTS TO METHOD CALL 'snmp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; snmp; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'community' => [
##&nbsp;                            'ro public'
##&nbsp;                            ],
##&nbsp;            'authtrap' => '1',
##&nbsp;            'location' => 'nb',
##&nbsp;            'contact' => 'public',
##&nbsp;            'init' => '0'
##&nbsp;          };
##&nbsp;         ]
##&nbsp;

sub snmp {
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @result;
    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );

    my %hash;
    my $key;
    my $flag;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line
            =~ /(contact)|(location)|(authtrap)|(init)|(traphosts)|(community)/
            )
        {
            $key = $&;
            if ( $key =~ /(traphosts)|(community)/ ) {
                $flag = 1;
            }
            else {
                $flag = 0;
            }
        }
        elsif ( $flag == 1 ) {
            $line =~ s/^(\s+)//;
            $hash{$key}->[$i] = $line;
            $i++;
        }
        else {
            $line =~ s/\s+//;
            $hash{$key} = $line;
        }
    }
    push @result, \%hash;
    return \@result;
}
1;
