#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/software.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode software commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode software commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "software update" will be software_update()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::software;
##
##    software_update(%args);
##    software_list(%args);
##
## @author dl-nacl-dev@netapp.com, swati.bajaj@netapp.com, srikanth.guruvayurappan@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::software;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::software_update
##
## @summary Parses the raw output from the "software update" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from software_update with any of its options as described in the 
##    corresponding cdef file 'software.cdefs'
## @synopsis
##      software_update(%args);
## INPUTS TO METHOD CALL 'software_update' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-r' </td> <td> 'disable-immediate-reboot' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'disable-automatic-installation' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination-file' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'overwrite-destination-file' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'request-automatic-reboot' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'url' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.

sub software_update {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
            
    return undef;
}

## @name ::software_list
##
## @summary Parses the raw output from the "software list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from software_list with any of its options as described in the 
##    corresponding cdef file 'software.cdefs'
## @synopsis
##      software_list(%args);
## INPUTS TO METHOD CALL 'software_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##   software_list();
##
##   Output:
##
##    [
##        {
##            'software_image' => [
##                                   'imag1.zip',
##                                   'image.zip.downloading'
##                                ]
##        }
##    ];


sub software_list {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
     
    my @lines = get_lines('raw_output' => $output,);
    my @images;
    #Parsing the output
    foreach my $line (@lines) {
        if ( $line =~ /^(\S+)$/ ) {
            push (@images, $line);
        } 
    }
    my %hash;
    $hash{'software_images'} = \@images;
 
    return [ \%hash ];
  
}

## @name ::software_get
##
## @summary Parses the raw output from the "software get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from software_get with any of its options as described in the
##    corresponding cdef file 'software.cdefs'
## @synopsis
##      software_get(%args);
## INPUTS TO METHOD CALL 'software_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef.
sub software_get {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command          = $args{command};
    my $error;
    if ( $raw_output =~ /(software: Unknown option)/ ) {
        $error          = "InvalidParam";
        $$error_message = $command." contains invalid param(s)";
    } elsif ( $raw_output =~ /Too few parameters/ ) {
        $error          = "MissingRequiredParam";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(No package name in URL)|(File .+ not found.)|(software: The URL must begin with http.*)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(Parameters not allowed after local package)|(software: Usage)|(installation of .+ was not successful)|(.+ already exists; use -f flag for overwrite)|(software: Unable to open connection to.*)|(Usage: software get.*)|(.*unable to fetch file from the specified location.)/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } 
    return $error;
}

1;
