#
# Copyright (c) 2011-2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    use sp;
##
##    sp_status(%args);
##    sp_setup(%args);
##    sp_update(%args);
##    sp_reboot(%args);
##    sp_test(%args);
##    sp_heartbeat(%args);
##    sp_userlist(%args);
##    sp_notify(%args);
##    sp_debugmode(%args);
##    sp_test(%args);
##    sp_ems(%args);
##    sp_stmgr(%args);
##    sp_getfile(%args);
##    sp_sendfile(%args);
##    sp_symlink(%args);
##    sp_log_stats(%args);
##
## @summary Base for API Sets CLI commands
## @author Abhishek Kumar(kabhishe@netapp.com),skotagir@netapp.com,veerap@netapp.com
##         dl-nacl-dev@netapp.com


package NACL::APISet::Node::CLI::7Mode::default::sp;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

## @name ::sp_status
##
## @summary Parses the raw output from the "sp status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_status with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_status(%args);
## INPUTS TO METHOD CALL 'sp_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; sp status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;	Service Processor           Status: Online
##&nbsp;		Firmware Version:   2.1X6
##&nbsp;		Mgmt MAC Address:   00:A0:98:15:79:4E
##&nbsp;		Ethernet Link:      up, 100Mb, full duplex, auto-neg complete
##&nbsp;		Using DHCP:         no
##&nbsp;	IPv4 configuration:
##&nbsp;		IP Address:         172.22.201.184
##&nbsp;		Netmask:            255.255.254.0
##&nbsp;		Gateway:            172.22.200.1
##&nbsp;	IPv6 configuration:         Disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ipv6_configuration' => 'Disabled',
##&nbsp;            'service_processor_status' => 'Online',
##&nbsp;            'ethernet_link' => 'up, 100Mb, full duplex, auto-neg complete',
##&nbsp;            'firmware_version' => '2.1X6',
##&nbsp;            'ipv4_configuration' => [
##&nbsp;                                      {
##&nbsp;                                        'gateway' => '172.22.200.1',
##&nbsp;                                        'netmask' => '255.255.254.0',
##&nbsp;                                        'ip_address' => '172.22.201.184'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'using_dhcp' => 'no',
##&nbsp;            'mgmt_mac_address' => '00:A0:98:15:79:4E'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub sp_status {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    #Now we can parse the output
    my @array_lines = get_lines("raw_output"=>$output, "response_object"=>$resp);
    my @result;
    my %hash;
    my ($line, $key, $value);

    for ($line = 0; $line<=$#array_lines; $line++) {
	if ( $array_lines[$line] =~ /(Service\s+Processor\s+Status):\s+(.*)/
            || $array_lines[$line] =~ /(IPv4\s+\S+)/
            || $array_lines[$line] =~ /(IPv6\s+\S+)/ ) {
	    $key = $1;
            $value = $2;
            $key = to_key_name(item => $key);
	    if($key =~ /service.*/) {
	        $hash{$key} = $value;
	    }
	    $line++;
            if($array_lines[$line]) {        #skipping blank lines in output
	        while($array_lines[$line] =~ /\s*(.*):\s+(.*)/) {
		    if( $2 eq "Disabled") {
		        $hash{to_key_name(item => $1)} = $2;
		        last;
		    } elsif($key eq "service_processor_status") {
		        $hash{to_key_name(item => $1)} = $2;
		    } else {
		        $hash{$key}->[0]->{to_key_name(item => $1)} = $2;
		    }
	            $line++;
	        }
            }
	    $line--;
	}

    }
    push (@result, \%hash);
    return \@result;
}

## @name ::sp_setup
##
## @summary Parses the raw output from the "sp setup" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp_setup with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_setup(%args);
## INPUTS TO METHOD CALL 'sp_setup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sp_setup {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
}

## @name ::sp_update
##
## @summary Parses the raw output from the "sp update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp_update with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_update(%args);
## INPUTS TO METHOD CALL 'sp_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'force-logs' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'url' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; sp update; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_update {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
}

## @name ::sp_reboot
##
## @summary Parses the raw output from the "sp reboot" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp_reboot with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_reboot(%args);
## INPUTS TO METHOD CALL 'sp_reboot' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'primary' </td> <td> 'primary' </td> <td> '1' </td> </tr>
## <tr> <td> 'backup' </td> <td> 'backup' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sp_reboot {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
}

## @name ::sp_test
##
## @summary Parses the raw output from the "sp test" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp_test with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_test(%args);
## INPUTS TO METHOD CALL 'sp_test' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'snmp' </td> <td> 'snmp' </td> <td> '1' </td> </tr>
## <tr> <td> 'autosupport' </td> <td> 'autosupport' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; sp test autosupport; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_test {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
}

sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /sp:\s+(unrecognized\s+command)\s*(.*)/) {
        $error = "CommandNotFound",
        $$error_message = $1;
    } elsif ( $output =~ /sp\s*:\s*ignoring\s*(.*)/) {
        $error = "InvalidParam",
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+test\s+autosupport/ && $output =~ /sp\s+test\s+autosupport/ ){
        $error = "MissingRequiredParam",
        $$error_message = "incomplete command";
    } elsif($output =~ /sp\s+:\s+ignoring\s+.*/) {
        $error = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+notify\s+<command>.*/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif($output =~ /Unknown EMS.*/) {
        $error = "CommandFailed";
        $$error_message = $output;
    } elsif ( $output =~ /Insufficient\s+number\s+of\s+arguments/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+(sendfile|getfile)\s+<srcFile>\s+<dstFile>.*/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif ( $output =~ /The\s+following\s+commands\s+are\s+available.*/ ){
       $error = "MissingRequiredParam",
       $$error_message = $output;
    } elsif ( $output =~ /There\s+is\s+no\s+SP.*/ ){
       $error = "CommandNotFound",
       $$error_message = $output;
    }
    return $error;
}

## @name ::sp_heartbeat
##
## @summary Parses the raw output from the "sp heartbeat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_heartbeat with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_heartbeat(%args);
## INPUTS TO METHOD CALL 'sp_heartbeat' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'sp-status' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; sp heartbeat status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; system node run -node local -command "priv set -q diag; sp heartbeat status; result -l"
##&nbsp;sp status
##&nbsp;  - Display status for a Service Processor.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_heartbeat {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_userlist
##
## @summary Parses the raw output from the "sp userlist" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_userlist with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_userlist(%args);
## INPUTS TO METHOD CALL 'sp_userlist' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp userlist; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_userlist {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_notify
##
## @summary Parses the raw output from the "sp notify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_notify with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_notify(%args);
## INPUTS TO METHOD CALL 'sp_notify' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'data-bytes' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp notify; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_notify {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_debugmode
##
## @summary Parses the raw output from the "sp debugmode" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_debugmode with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_debugmode(%args);
## INPUTS TO METHOD CALL 'sp_debugmode' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp debugmode; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_debugmode {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_ems
##
## @summary Parses the raw output from the "sp ems" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_ems with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_ems(%args);
## INPUTS TO METHOD CALL 'sp_ems' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ems-id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp ems driver.hourly.stats; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_ems {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_stmgr_state_set
##
## @summary Parses the raw output from the "sp stmgr state set" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_stmgr_state_set with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_stmgr_state_set(%args);
## INPUTS TO METHOD CALL 'sp_stmgr_state_set' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'recovery-state' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'reboot-state' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'ipmi-state' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'state' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'packet-state' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp stmgr state set online online normal normal normal; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_stmgr_state_set {
    my %args = @_;
    my $error_message;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
 return [];
}

## @name ::sp_stmgr_state_show
##
## @summary Parses the raw output from the "sp stmgr state show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_stmgr_state_show with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_stmgr_state_show(%args);
## INPUTS TO METHOD CALL 'sp_stmgr_state_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp stmgr state show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;(    online     online     normal     normal       normal)    
##&nbsp;       
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;               'recovery-state' => 'normal',
##&nbsp;               'reboot_state' => 'normal',
##&nbsp;               'ipmi_state ' => 'online',
##&nbsp;               'packet_state' => 'online',
##&nbsp;               'overall-state' => 'normal'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;

sub sp_stmgr_state_show {
    my %args = @_;
    my $error_message;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output);
    foreach my $line (@lines) { 
        if ( $line =~ /\(\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\w+)\)/ ){
            $result_array[0]->{'ipmi_state '}=$1;
            $result_array[0]->{'packet_state'}=$2;
            $result_array[0]->{'reboot_state'}=$3;
            $result_array[0]->{'recovery-state'}=$4;
            $result_array[0]->{'overall-state'}=$5;
       }
    } 
   return \@result_array;
}

## @name ::sp_stmgr_event
##
## @summary Parses the raw output from the "sp stmgr event" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_stmgr_event with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_stmgr_event(%args);
## INPUTS TO METHOD CALL 'sp_stmgr_event' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'event-id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp stmgr event ipmi_hb; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_stmgr_event{
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_stmgr_debug
##
## @summary Parses the raw output from the "sp stmgr debug" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_stmgr_debug with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_stmgr_debug(%args);
## INPUTS TO METHOD CALL 'sp_stmgr_debug' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'debug-arg' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp stmgr debug accept-events; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_stmgr_debug{
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_sendfile
##
## @summary Parses the raw output from the "sp sendfile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_sendfile with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_sendfile(%args);
## INPUTS TO METHOD CALL 'sp_sendfile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp sendfile /vol/file1 /SP; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_sendfile {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_getfile
##
## @summary Parses the raw output from the "sp getfile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_getfile with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_getfile(%args);
## INPUTS TO METHOD CALL 'sp_getfile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp getfile /SP/file/ /vol/vol0/ ; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_getfile {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_symlink
##
## @summary Parses the raw output from the "sp symlink" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sp_symlink with any of its options as described in the 
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_symlink(%args);
## INPUTS TO METHOD CALL 'sp_symlink' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'old-path' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'new-path' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; sp symlink /SP/sp1 /SP/sp2; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub sp_symlink {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::sp_log_stats
##
## @summary Parses the raw output from the "sp log stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp_log_stats with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sp_log_stats(%args);
## INPUTS TO METHOD CALL 'sp_log_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; sp log stats; result -l; priv set -q diag
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;======SPLOG Total Statistics=====
##&nbsp;        ===============================
##&nbsp;SPLOG stats saved at /etc/log/sp/splogd_stats daily.
##&nbsp;SPLOG State:
##&nbsp;         Splogd initalized (yes), enabled(testing only): yes
##&nbsp;         SP FW support splog function: yes
##&nbsp;         SP FW current state: online
##&nbsp;         Splogd has sent callhome.reboot.XXXX:yes
##&nbsp;         Splog downfiler is  available at /etc/log/sp/downfiler.
##&nbsp;         Splog latest is available at /etc/log/sp/latest
##&nbsp;         Splogd scan happens daily around 5AM.
##&nbsp;SPLOG Events:
##&nbsp;         Event Boot_Msg: CNT 2 last at Fri Mar  9 12:28:41 EST 2012
##&nbsp;         Event Daily_Scan: CNT 3 last at Mon Mar 12 05:05:00 EDT 2012
##&nbsp;         Event Daily_Stats: CNT 3 last at Mon Mar 12 05:30:00 EDT 2012
##&nbsp;         Event Downfiler: CNT 0 last at --
##&nbsp;         Event Latest: CNT 0 last at --
##&nbsp;         Event ASychnorized-Timeout Msg: CNT 0 last at --
##&nbsp;         Event sychnorized Msg: CNT 2 last at Fri Mar  9 14:03:53 EST 2012
##&nbsp;         Event Retry: CNT 0 last at --
##&nbsp;SPLOG Downfiler Transfer:
##&nbsp;         Total Success: CNT 1 last at Fri Mar  9 12:28:44 EST 2012
##&nbsp;         Total Failure: CNT 1 last at Fri Mar  9 12:28:17 EST 2012
##&nbsp;         Total Failure due to Checksum: CNT 0 last at --
##&nbsp;         Total Failure due to timeout: CNT 1 last at Fri Mar  9 12:28:17 EST 2012
##&nbsp;         Total failure due to other misc reason: CNT 0 last at --
##&nbsp;SPLOG Latest Transfer:
##&nbsp;         Total Success: CNT 5 last at Mon Mar 12 05:05:14 EDT 2012
##&nbsp;         Total Failure: CNT 0 last at --
##&nbsp;         Total Failure due to Checksum: CNT 0 last at --
##&nbsp;         Total Failure due to timeout: CNT 0 last at --
##&nbsp;         Total failure due to other misc reason: CNT 0 last at --
##&nbsp;SPLOG untar to Downfiler:
##&nbsp;         Total Success: CNT 1 last at Fri Mar  9 12:28:45 EST 2012
##&nbsp;        Total Failure: CNT 0 last at --
##&nbsp;SPLOG untar to Latest:
##&nbsp;        Total Success: CNT 5 last at Mon Mar 12 05:05:14 EDT 2012
##&nbsp;        Total Failure: CNT 0 last at --
##&nbsp;SPLOG untar to preReboot:
##&nbsp;        Total Success: CNT 2 last at Fri Mar  9 14:03:53 EST 2012
##&nbsp;        Total Failure: CNT 0 last at --
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'splog_untar_to_latest' => [
##&nbsp;                                          {
##&nbsp;                                            'total_success' => ' CNT 5 last at Mon Mar 12 05:05:14 EDT 2012',
##&nbsp;                                            'total_failure' => ' CNT 0 last at --'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;             'splog_latest_transfer' => [
##&nbsp;                                          {
##&nbsp;                                            'total_success' => ' CNT 5 last at Mon Mar 12 05:05:14 EDT 2012',
##&nbsp;                                            'total_failure_due_to_other_misc_reason' => ' CNT 0 last at --',
##&nbsp;                                            'total_failure_due_to_checksum' => ' CNT 0 last at --',
##&nbsp;                                            'total_failure' => ' CNT 0 last at --',
##&nbsp;                                            'total_failure_due_to_timeout' => ' CNT 0 last at --'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;             'splog_stats_saved_at' => [
##&nbsp;                                         {
##&nbsp;                                           'occurence' => 'daily',
##&nbsp;                                           'path' => '/etc/log/sp/splogd_stats'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;             'splog_events' => [
##&nbsp;                                 {
##&nbsp;                                   'event_asychnorized_timeout_msg' => ' CNT 0 last at --',
##&nbsp;                                   'event_daily_scan' => ' CNT 3 last at Mon Mar 12 05:05:00 EDT 2012',
##&nbsp;                                   'event_retry' => ' CNT 0 last at --',
##&nbsp;                                   'event_boot_msg' => ' CNT 2 last at Fri Mar  9 12:28:41 EST 2012',
##&nbsp;                                   'event_sychnorized_msg' => ' CNT 2 last at Fri Mar  9 14:03:53 EST 2012',
##&nbsp;                                   'event_downfiler' => ' CNT 0 last at --',
##&nbsp;                                   'event_daily_stats' => ' CNT 3 last at Mon Mar 12 05:30:00 EDT 2012',
##&nbsp;                                   'event_latest' => ' CNT 0 last at --'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;             'splog_downfiler_transfer' => [
##&nbsp;                                             {
##&nbsp;                                               'total_success' => ' CNT 1 last at Fri Mar  9 12:28:44 EST 2012',
##&nbsp;                                               'total_failure_due_to_other_misc_reason' => ' CNT 0 last at --',
##&nbsp;                                               'total_failure_due_to_checksum' => ' CNT 0 last at --',
##&nbsp;                                               'total_failure' => ' CNT 1 last at Fri Mar  9 12:28:17 EST 2012',
##&nbsp;                                               'total_failure_due_to_timeout' => ' CNT 1 last at Fri Mar  9 12:28:17 EST 2012'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;             'splog_state' => [
##&nbsp;                                {
##&nbsp;                                  'sp_fw_current_state' => ' online',
##&nbsp;                                  'splogd_initalized' => 'yes',
##&nbsp;                                  'splogd_has_sent_callhome_reboot_xxxx' => 'yes',
##&nbsp;                                  'splog_latest_is_available_at' => '/etc/log/sp/latest',
##&nbsp;                                  'splog_downfiler_is_available_at' => '/etc/log/sp/downfiler',
##&nbsp;                                  'sp_fw_support_splog_function' => ' yes',
##&nbsp;                                  'splogd_scan_happens_daily_around' => '5AM',
##&nbsp;                                  'enabled' => 'testing only',
##&nbsp;                                  'splogd_initalized_yes_enabledtesting_only' => 'yes'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;             'splog_untar_to_downfiler' => [
##&nbsp;                                             {
##&nbsp;                                               'total_success' => ' CNT 1 last at Fri Mar  9 12:28:45 EST 2012',
##&nbsp;                                               'total_failure' => ' CNT 0 last at --'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;             'splog_untar_to_prereboot' => [
##&nbsp;                                             {
##&nbsp;                                               'total_success' => ' CNT 2 last at Fri Mar  9 14:03:53 EST 2012',
##&nbsp;                                               'total_failure' => ' CNT 0 last at --'
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;           }
##&nbsp;         ];

sub sp_log_stats {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output );
    my @result_array;
    my $key = "";
    foreach my $line (@lines) {
        $line =~ s/-Timeout/_timeout/;
        if ( $line =~ /(SPLOG\s+stats\s+\S+\s+\w+)\s+(\S+)\s+(\w+)./ ) {
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{path} = $2;
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{occurence}
                = $3;
        } elsif ( $line
            =~ /(Splogd\s+initalized)\s+.(\S+).,\s+(enabled).(\S+\s+\w+).:\s+(\S+)/
            ) {
            $result_array[0]->{splog_state}->[0]
                ->{ to_key_name( item => $1 ) } = $2;
            $result_array[0]->{splog_state}->[0]
                ->{ to_key_name( item => $3 ) } = $4;
            $line
                =~ /(Splogd\s+initalized\s+\(\S+\),\s+\w+\(\w+\s+\w+\)):\s+(\w+)/;
            my $value1 = $1;
            my $value2 = $2;
            $value1 =~ s/,//g;
            $result_array[0]->{splog_state}->[0]
                ->{ to_key_name( item => $value1 ) } = $value2;
        } elsif ( $line =~ /^(SPLOG.*):$/ ) {
            $key = $1;
        } elsif ( $line =~ /\s*([a-zA-Z\s\(\)\,_\.]*):(.*)/
            || $line
            =~ /^\s*(\w+\s+\w+\s+\w+\s+\w+\s+\w+)\s+(.*\/\w+|\w+).?/ ) {
            my $value1 = $1;
            my $value2 = $2;
            $value1 =~ s/\./_/g;
            $result_array[0]->{ to_key_name( item => $key ) }->[0]
                ->{ to_key_name( item => $value1 ) } = $value2;
        }
    }
    return (\@result_array);
} ## end sub sp_log_stats
1;
