#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode spinhi_stats command output
##
## @description
##    This is a library to parse the raw output of 7Mode spinhi_stats command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::spinhi_stats;
##
##    spinhi_stats(%args);
##
## @author anupamap@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::spinhi_stats;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $command    = $args{command};
    my $error;
    my $error_message;
    my $resp = $args{"response_object"};
    if ( $raw_output =~ /.*not found.*/i ) {
        $error         = "CommandNotFound";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}
## @name ::spinhi_stats
##
## @summary Parses the raw output from the "spinhi_stats" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from spinhi_stats with any of its options as described in the 
##    corresponding cdef file 'spinhi_stats.cdefs'
## @synopsis
##      spinhi_stats(%args);
## INPUTS TO METHOD CALL 'spinhi_stats' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; spinhi_stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;fileop request pool:
##&nbsp;	size=160 total_req=1682493 cur_req=0 max_req=96 total_resp=1682493 total_replay_resp=0
##&nbsp;	defer_req=0 cur_defer=0 max_defer=0 hipri=1426 unmarshal_errs=0 marshal_errs=0 fastpath_null_resps=0
##&nbsp;	Access: count=152564 hipri=0 errs=0 elapsed: max=70011.51 avg=833.11
##&nbsp;		spinhi cpu: max=10866.56 avg=22.95
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Create: count=929 hipri=0 errs=0 elapsed: max=477818.78 avg=1737.73
##&nbsp;		spinhi cpu: max=103.41 avg=25.14
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	GetAttr: count=938731 hipri=0 errs=0 elapsed: max=132461.90 avg=762.11
##&nbsp;		spinhi cpu: max=17170.77 avg=23.58
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	GetRootFh: count=1 hipri=0 errs=0 elapsed: max=166.09 avg=166.09
##&nbsp;		spinhi cpu: max=41.22 avg=41.22
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Link: count=3 hipri=0 errs=0 elapsed: max=1584.21 avg=643.76
##&nbsp;		spinhi cpu: max=22.85 avg=20.99
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Lookup: count=344326 hipri=0 errs=283171 elapsed: max=1896676.53 avg=896.67
##&nbsp;		spinhi cpu: max=11165.96 avg=21.09
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;		errs: 2[283171]
##&nbsp;	Read: count=96221 hipri=0 errs=0 elapsed: max=2938402.79 avg=2283.41
##&nbsp;		spinhi cpu: max=6341.15 avg=27.44
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Readdir: count=5381 hipri=0 errs=0 elapsed: max=1723614.76 avg=2921.52
##&nbsp;		spinhi cpu: max=3922.76 avg=26.99
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Readlink: count=189 hipri=0 errs=0 elapsed: max=6484.21 avg=902.51
##&nbsp;		spinhi cpu: max=43.29 avg=22.10
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Rename: count=13 hipri=0 errs=0 elapsed: max=1792.99 avg=832.16
##&nbsp;		spinhi cpu: max=31.13 avg=22.15
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Setattr: count=960 hipri=0 errs=0 elapsed: max=11137.12 avg=945.06
##&nbsp;		spinhi cpu: max=52.12 avg=21.56
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Unlink: count=1411 hipri=0 errs=0 elapsed: max=10235.40 avg=888.21
##&nbsp;		spinhi cpu: max=41.85 avg=20.37
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;	Write: count=141764 hipri=0 errs=0 elapsed: max=11953288.12 avg=1168.89
##&nbsp;		spinhi cpu: max=13257.92 avg=26.11
##&nbsp;		wafl cpu: max=0.00 avg=0.00 restarts=0
##&nbsp;filecb request pool:
##&nbsp;	size=97 total_req=0 cur_req=0 max_req=0 total_resp=0 total_replay_resp=0
##&nbsp;	defer_req=0 cur_defer=0 max_defer=0 hipri=0 unmarshal_errs=0 marshal_errs=0 fastpath_null_resps=0
##&nbsp;coralop request pool:
##&nbsp;	size=128 total_req=0 cur_req=0 max_req=0 total_resp=0 total_replay_resp=0
##&nbsp;	defer_req=0 cur_defer=0 max_defer=0 hipri=0 unmarshal_errs=0 marshal_errs=0 fastpath_null_resps=0
##&nbsp;spinhi memory: 0 total 2k blocks
##&nbsp;    0 (2k):
##&nbsp;	#allocs=354 (189 demand) #frees=354 #togbl=189 #fromgbl=0 #vmpress=283 target=0
##&nbsp;	allocated=0 inuse=0 max_period=[C:0 P:0 P2:0] max_ever=1
##&nbsp;	free_counts: gbl=0 cpu0=0 cpu1=0 #uses: cpu0=78 cpu1=111
##&nbsp;
##&nbsp;LocalBypass: inSkip=0 outSync=0, outASyncSkipped=0 (failed 0)
##&nbsp;LocalBypass: maxReqs=0(0+0) numCurrReqs=0 highwaterReqs=0 extFrees=0
##&nbsp;Total 7G requests started: 1682493
##&nbsp;Total 7G requests completed: 1682493
##&nbsp;Total 7G requests completed with errors: 283171
##&nbsp;Total 7G bytes read: 1562485819
##&nbsp;Total 7G bytes written: 89217605
##&nbsp;Op stats: 	Null: 0 ops 0 errors
##&nbsp;	Access: 152564 ops 0 errors
##&nbsp;	Close: 0 ops 0 errors
##&nbsp;	Create: 929 ops 0 errors
##&nbsp;	DiscardCred: 0 ops 0 errors
##&nbsp;	Downgrade: 0 ops 0 errors
##&nbsp;	FhToPath: 0 ops 0 errors
##&nbsp;	FoldFile: 0 ops 0 errors
##&nbsp;	GetAttr: 938731 ops 0 errors
##&nbsp;	GetAttrBulk: 0 ops 0 errors
##&nbsp;	GetCred: 0 ops 0 errors
##&nbsp;	GetParent: 0 ops 0 errors
##&nbsp;	GetRootFh: 1 ops 0 errors
##&nbsp;	Link: 3 ops 0 errors
##&nbsp;	Lock: 0 ops 0 errors
##&nbsp;	LockGrantedAck: 0 ops 0 errors
##&nbsp;	Lookup: 344326 ops 283171 errors
##&nbsp;	MoveLocks: 0 ops 0 errors
##&nbsp;	Open: 0 ops 0 errors
##&nbsp;	Prefetch: 0 ops 0 errors
##&nbsp;	Read: 96221 ops 0 errors
##&nbsp;	Readdir: 5381 ops 0 errors
##&nbsp;	Readlink: 189 ops 0 errors
##&nbsp;	ReclaimLocksAck: 0 ops 0 errors
##&nbsp;	Remove: 0 ops 0 errors
##&nbsp;	Rename: 13 ops 0 errors
##&nbsp;	ScanMatchingLocks: 0 ops 0 errors
##&nbsp;	ScanFile: 0 ops 0 errors
##&nbsp;	Setattr: 960 ops 0 errors
##&nbsp;	SetCred: 0 ops 0 errors
##&nbsp;	Share: 0 ops 0 errors
##&nbsp;	Unlink: 1411 ops 0 errors
##&nbsp;	Unshare: 0 ops 0 errors
##&nbsp;	Watch: 0 ops 0 errors
##&nbsp;	Write: 141764 ops 0 errors
##&nbsp;	NrvGetAttrSnapInofile: 0 ops 0 errors
##&nbsp;	FhToPath2: 0 ops 0 errors
##&nbsp;	ReadLink2: 0 ops 0 errors
##&nbsp;	Create2: 0 ops 0 errors
##&nbsp;	ChangeFileType: 0 ops 0 errors
##&nbsp;	ReplayFindBin: 0 ops 0 errors
##&nbsp;	ReplayGetOpInfo: 0 ops 0 errors
##&nbsp;	ReplayReleaseBin: 0 ops 0 errors
##&nbsp;	Audit: 0 ops 0 errors
##&nbsp;	GetVolumeLanguage: 0 ops 0 errors
##&nbsp;	GetFsAttrs: 0 ops 0 errors
##&nbsp;	CancelFileop: 0 ops 0 errors
##&nbsp;	GetQuota: 0 ops 0 errors
##&nbsp;	FreeBlocks: 0 ops 0 errors
##&nbsp;	WriteZeros: 0 ops 0 errors
##&nbsp;	VdiskSetAttr: 0 ops 0 errors
##&nbsp;	VdiskGetAttr: 0 ops 0 errors
##&nbsp;	VdiskDestroy: 0 ops 0 errors
##&nbsp;	SetLuMetadata: 0 ops 0 errors
##&nbsp;	GetLuMetadata: 0 ops 0 errors
##&nbsp;	Disconnect: 0 ops 0 errors
##&nbsp;	Reconnect: 0 ops 0 errors
##&nbsp;	MakeResilient: 0 ops 0 errors
##&nbsp;	OpenToName: 0 ops 0 errors
##&nbsp;	CopyNotify: 0 ops 0 errors
##&nbsp;	CopyRevoke: 0 ops 0 errors
##&nbsp;	Copy: 0 ops 0 errors
##&nbsp;	CopyStatus: 0 ops 0 errors
##&nbsp;	CopyAbort: 0 ops 0 errors
##&nbsp;	CopyAuthcheck: 0 ops 0 errors
##&nbsp;
##&nbsp;Response gbuf: reserved=0 allocated=0  max=16
##&nbsp;    globals: use=0 fail=0 req=0 rel=0
##&nbsp;    per-cpu: reserved=0 success=0 fail=0 cancel=0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fileop_request_pool' => [
##&nbsp;                                       {
##&nbsp;                                         'getattr' => [
##&nbsp;                                                        {
##&nbsp;                                                          'wafl_cpu' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'restarts' => '0',
##&nbsp;                                                                            'avg' => '0.00',
##&nbsp;                                                                            'max' => '0.00'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                          'count' => '938731',
##&nbsp;                                                          'hipri' => '0',
##&nbsp;                                                          'spinhi_cpu' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              'avg' => '23.58',
##&nbsp;                                                                              'max' => '17170.77'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                          'elapsed' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'avg' => '762.11',
##&nbsp;                                                                           'max' => '132461.90'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                          'errs' => '0'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;                                         'max_defer' => '0',
##&nbsp;                                         'setattr' => [
##&nbsp;                                                        {
##&nbsp;                                                          'wafl_cpu' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'restarts' => '0',
##&nbsp;                                                                            'avg' => '0.00',
##&nbsp;                                                                            'max' => '0.00'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                          'count' => '960',
##&nbsp;                                                          'hipri' => '0',
##&nbsp;                                                          'spinhi_cpu' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              'avg' => '21.56',
##&nbsp;                                                                              'max' => '52.12'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                          'elapsed' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'avg' => '945.06',
##&nbsp;                                                                           'max' => '11137.12'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                          'errs' => '0'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;                                         'cur_defer' => '0',
##&nbsp;                                         'size' => '160',
##&nbsp;                                         'unlink' => [
##&nbsp;                                                       {
##&nbsp;                                                         'wafl_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'restarts' => '0',
##&nbsp;                                                                           'avg' => '0.00',
##&nbsp;                                                                           'max' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                         'count' => '1411',
##&nbsp;                                                         'hipri' => '0',
##&nbsp;                                                         'spinhi_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '20.37',
##&nbsp;                                                                             'max' => '41.85'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                         'elapsed' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'avg' => '888.21',
##&nbsp;                                                                          'max' => '10235.40'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                         'errs' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                         'rename' => [
##&nbsp;                                                       {
##&nbsp;                                                         'wafl_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'restarts' => '0',
##&nbsp;                                                                           'avg' => '0.00',
##&nbsp;                                                                           'max' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                         'count' => '13',
##&nbsp;                                                         'hipri' => '0',
##&nbsp;                                                         'spinhi_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '22.15',
##&nbsp;                                                                             'max' => '31.13'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                         'elapsed' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'avg' => '832.16',
##&nbsp;                                                                          'max' => '1792.99'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                         'errs' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                         'readdir' => [
##&nbsp;                                                        {
##&nbsp;                                                          'wafl_cpu' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'restarts' => '0',
##&nbsp;                                                                            'avg' => '0.00',
##&nbsp;                                                                            'max' => '0.00'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                          'count' => '5381',
##&nbsp;                                                          'hipri' => '0',
##&nbsp;                                                          'spinhi_cpu' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              'avg' => '26.99',
##&nbsp;                                                                              'max' => '3922.76'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                          'elapsed' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'avg' => '2921.52',
##&nbsp;                                                                           'max' => '1723614.76'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                          'errs' => '0'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;                                         'marshal_errs' => '0',
##&nbsp;                                         'total_replay_resp' => '0',
##&nbsp;                                         'getrootfh' => [
##&nbsp;                                                          {
##&nbsp;                                                            'wafl_cpu' => [
##&nbsp;                                                                            {
##&nbsp;                                                                              'restarts' => '0',
##&nbsp;                                                                              'avg' => '0.00',
##&nbsp;                                                                              'max' => '0.00'
##&nbsp;                                                                            }
##&nbsp;                                                                          ],
##&nbsp;                                                            'count' => '1',
##&nbsp;                                                            'hipri' => '0',
##&nbsp;                                                            'spinhi_cpu' => [
##&nbsp;                                                                              {
##&nbsp;                                                                                'avg' => '41.22',
##&nbsp;                                                                                'max' => '41.22'
##&nbsp;                                                                              }
##&nbsp;                                                                            ],
##&nbsp;                                                            'elapsed' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '166.09',
##&nbsp;                                                                             'max' => '166.09'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                            'errs' => '0'
##&nbsp;                                                          }
##&nbsp;                                                        ],
##&nbsp;                                         'read' => [
##&nbsp;                                                     {
##&nbsp;                                                       'wafl_cpu' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'restarts' => '0',
##&nbsp;                                                                         'avg' => '0.00',
##&nbsp;                                                                         'max' => '0.00'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                                       'count' => '96221',
##&nbsp;                                                       'hipri' => '0',
##&nbsp;                                                       'spinhi_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'avg' => '27.44',
##&nbsp;                                                                           'max' => '6341.15'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                       'elapsed' => [
##&nbsp;                                                                      {
##&nbsp;                                                                        'avg' => '2283.41',
##&nbsp;                                                                        'max' => '2938402.79'
##&nbsp;                                                                      }
##&nbsp;                                                                    ],
##&nbsp;                                                       'errs' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                         'fastpath_null_resps' => '0',
##&nbsp;                                         'write' => [
##&nbsp;                                                      {
##&nbsp;                                                        'wafl_cpu' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'restarts' => '0',
##&nbsp;                                                                          'avg' => '0.00',
##&nbsp;                                                                          'max' => '0.00'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                        'count' => '141764',
##&nbsp;                                                        'hipri' => '0',
##&nbsp;                                                        'spinhi_cpu' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'avg' => '26.11',
##&nbsp;                                                                            'max' => '13257.92'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                        'elapsed' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'avg' => '1168.89',
##&nbsp;                                                                         'max' => '11953288.12'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                                        'errs' => '0'
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;                                         'max_req' => '96',
##&nbsp;                                         'total_resp' => '1682493',
##&nbsp;                                         'unmarshal_errs' => '0',
##&nbsp;                                         'create' => [
##&nbsp;                                                       {
##&nbsp;                                                         'wafl_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'restarts' => '0',
##&nbsp;                                                                           'avg' => '0.00',
##&nbsp;                                                                           'max' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                         'count' => '929',
##&nbsp;                                                         'hipri' => '0',
##&nbsp;                                                         'spinhi_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '25.14',
##&nbsp;                                                                             'max' => '103.41'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                         'elapsed' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'avg' => '1737.73',
##&nbsp;                                                                          'max' => '477818.78'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                         'errs' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                         'link' => [
##&nbsp;                                                     {
##&nbsp;                                                       'wafl_cpu' => [
##&nbsp;                                                                       {
##&nbsp;                                                                         'restarts' => '0',
##&nbsp;                                                                         'avg' => '0.00',
##&nbsp;                                                                         'max' => '0.00'
##&nbsp;                                                                       }
##&nbsp;                                                                     ],
##&nbsp;                                                       'count' => '3',
##&nbsp;                                                       'hipri' => '0',
##&nbsp;                                                       'spinhi_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'avg' => '20.99',
##&nbsp;                                                                           'max' => '22.85'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                       'elapsed' => [
##&nbsp;                                                                      {
##&nbsp;                                                                        'avg' => '643.76',
##&nbsp;                                                                        'max' => '1584.21'
##&nbsp;                                                                      }
##&nbsp;                                                                    ],
##&nbsp;                                                       'errs' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                         'access' => [
##&nbsp;                                                       {
##&nbsp;                                                         'wafl_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'restarts' => '0',
##&nbsp;                                                                           'avg' => '0.00',
##&nbsp;                                                                           'max' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                         'count' => '152564',
##&nbsp;                                                         'hipri' => '0',
##&nbsp;                                                         'spinhi_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '22.95',
##&nbsp;                                                                             'max' => '10866.56'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                         'elapsed' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'avg' => '833.11',
##&nbsp;                                                                          'max' => '70011.51'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                         'errs' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                         'readlink' => [
##&nbsp;                                                         {
##&nbsp;                                                           'wafl_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'restarts' => '0',
##&nbsp;                                                                             'avg' => '0.00',
##&nbsp;                                                                             'max' => '0.00'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                           'count' => '189',
##&nbsp;                                                           'hipri' => '0',
##&nbsp;                                                           'spinhi_cpu' => [
##&nbsp;                                                                             {
##&nbsp;                                                                               'avg' => '22.10',
##&nbsp;                                                                               'max' => '43.29'
##&nbsp;                                                                             }
##&nbsp;                                                                           ],
##&nbsp;                                                           'elapsed' => [
##&nbsp;                                                                          {
##&nbsp;                                                                            'avg' => '902.51',
##&nbsp;                                                                            'max' => '6484.21'
##&nbsp;                                                                          }
##&nbsp;                                                                        ],
##&nbsp;                                                           'errs' => '0'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;                                         'defer_req' => '0',
##&nbsp;                                         'lookup' => [
##&nbsp;                                                       {
##&nbsp;                                                         'error_count' => '2[283171]',
##&nbsp;                                                         'wafl_cpu' => [
##&nbsp;                                                                         {
##&nbsp;                                                                           'restarts' => '0',
##&nbsp;                                                                           'avg' => '0.00',
##&nbsp;                                                                           'max' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                                       ],
##&nbsp;                                                         'count' => '344326',
##&nbsp;                                                         'hipri' => '0',
##&nbsp;                                                         'spinhi_cpu' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'avg' => '21.09',
##&nbsp;                                                                             'max' => '11165.96'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                                         'elapsed' => [
##&nbsp;                                                                        {
##&nbsp;                                                                          'avg' => '896.67',
##&nbsp;                                                                          'max' => '1896676.53'
##&nbsp;                                                                        }
##&nbsp;                                                                      ],
##&nbsp;                                                         'errs' => '283171'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                         'total_req' => '1682493',
##&nbsp;                                         'hipri' => '1426',
##&nbsp;                                         'cur_req' => '0'
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;            'response_gbuf' => [
##&nbsp;                                 {
##&nbsp;                                   'allocated' => '0',
##&nbsp;                                   'per_cpu' => [
##&nbsp;                                                  {
##&nbsp;                                                    'success' => '0',
##&nbsp;                                                    'fail' => '0',
##&nbsp;                                                    'cancel' => '0',
##&nbsp;                                                    'reserved' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                                   'globals' => [
##&nbsp;                                                  {
##&nbsp;                                                    'rel' => '0',
##&nbsp;                                                    'req' => '0',
##&nbsp;                                                    'fail' => '0',
##&nbsp;                                                    'use' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                                   'max' => '16',
##&nbsp;                                   'reserved' => '0'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'spinhi_memory' => [
##&nbsp;                                 {
##&nbsp;                                   'inuse' => '0',
##&nbsp;                                   'blocks' => '2k',
##&nbsp;                                   'fromgbl' => '0',
##&nbsp;                                   'allocs' => '354',
##&nbsp;                                   'total' => '0',
##&nbsp;                                   'allocated' => '0',
##&nbsp;                                   'vmpress' => '283',
##&nbsp;                                   'uses' => [
##&nbsp;                                               {
##&nbsp;                                                 'cpu1' => '111',
##&nbsp;                                                 'cpu0' => '78'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                                   'free_counts' => [
##&nbsp;                                                      {
##&nbsp;                                                        'cpu1' => '0',
##&nbsp;                                                        'gbl' => '0',
##&nbsp;                                                        'cpu0' => '0'
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;                                   'togbl' => '189',
##&nbsp;                                   'target' => '0',
##&nbsp;                                   'max_ever' => '1',
##&nbsp;                                   'max_period' => [
##&nbsp;                                                     {
##&nbsp;                                                       'P2' => '0',
##&nbsp;                                                       'C' => '0',
##&nbsp;                                                       'P' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                                   'demand' => '189',
##&nbsp;                                   'frees' => '354'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'total_7g_bytes_read' => '1562485819',
##&nbsp;            'localbypass' => [
##&nbsp;                               {
##&nbsp;                                 'extFrees' => '0',
##&nbsp;                                 'inSkip' => '0',
##&nbsp;                                 'highwaterReqs' => '0',
##&nbsp;                                 'failed' => '0',
##&nbsp;                                 'outSync' => '0',
##&nbsp;                                 'maxReqs' => '0',
##&nbsp;                                 'numCurrReqs' => '0',
##&nbsp;                                 'outASyncSkipped' => '0'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'total_7g_requests_completed_with_errors' => '283171',
##&nbsp;            'op_stats' => [
##&nbsp;                            {
##&nbsp;                              'scanmatchinglocks' => [
##&nbsp;                                                       {
##&nbsp;                                                         'errors' => '0',
##&nbsp;                                                         'ops' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                              'readlink2' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'movelocks' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'create2' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'rename' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '0',
##&nbsp;                                              'ops' => '13'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'makeresilient' => [
##&nbsp;                                                   {
##&nbsp;                                                     'errors' => '0',
##&nbsp;                                                     'ops' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;                              'vdiskgetattr' => [
##&nbsp;                                                  {
##&nbsp;                                                    'errors' => '0',
##&nbsp;                                                    'ops' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                              'cancelfileop' => [
##&nbsp;                                                  {
##&nbsp;                                                    'errors' => '0',
##&nbsp;                                                    'ops' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                              'getfsattrs' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'replayfindbin' => [
##&nbsp;                                                   {
##&nbsp;                                                     'errors' => '0',
##&nbsp;                                                     'ops' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;                              'reconnect' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'audit' => [
##&nbsp;                                           {
##&nbsp;                                             'errors' => '0',
##&nbsp;                                             'ops' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                              'open' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'getlumetadata' => [
##&nbsp;                                                   {
##&nbsp;                                                     'errors' => '0',
##&nbsp;                                                     'ops' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;                              'vdisksetattr' => [
##&nbsp;                                                  {
##&nbsp;                                                    'errors' => '0',
##&nbsp;                                                    'ops' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                              'replaygetopinfo' => [
##&nbsp;                                                     {
##&nbsp;                                                       'errors' => '0',
##&nbsp;                                                       'ops' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                              'lockgrantedack' => [
##&nbsp;                                                    {
##&nbsp;                                                      'errors' => '0',
##&nbsp;                                                      'ops' => '0'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;                              'freeblocks' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'watch' => [
##&nbsp;                                           {
##&nbsp;                                             'errors' => '0',
##&nbsp;                                             'ops' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                              'getrootfh' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '1'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'foldfile' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '0'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'copynotify' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'create' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '0',
##&nbsp;                                              'ops' => '929'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'replayreleasebin' => [
##&nbsp;                                                      {
##&nbsp;                                                        'errors' => '0',
##&nbsp;                                                        'ops' => '0'
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;                              'copyrevoke' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'access' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '0',
##&nbsp;                                              'ops' => '152564'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'getquota' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '0'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'setcred' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'lookup' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '283171',
##&nbsp;                                              'ops' => '344326'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'writezeros' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'getattr' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '938731'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'getcred' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'remove' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '0',
##&nbsp;                                              'ops' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'lock' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'opentoname' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'disconnect' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'readdir' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '5381'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'unlink' => [
##&nbsp;                                            {
##&nbsp;                                              'errors' => '0',
##&nbsp;                                              'ops' => '1411'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                              'copy' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'downgrade' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'getattrbulk' => [
##&nbsp;                                                 {
##&nbsp;                                                   'errors' => '0',
##&nbsp;                                                   'ops' => '0'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                              'read' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '96221'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'fhtopath2' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'vdiskdestroy' => [
##&nbsp;                                                  {
##&nbsp;                                                    'errors' => '0',
##&nbsp;                                                    'ops' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ],
##&nbsp;                              'getparent' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'write' => [
##&nbsp;                                           {
##&nbsp;                                             'errors' => '0',
##&nbsp;                                             'ops' => '141764'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                              'readlink' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '189'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'null' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'changefiletype' => [
##&nbsp;                                                    {
##&nbsp;                                                      'errors' => '0',
##&nbsp;                                                      'ops' => '0'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;                              'nrvgetattrsnapinofile' => [
##&nbsp;                                                           {
##&nbsp;                                                             'errors' => '0',
##&nbsp;                                                             'ops' => '0'
##&nbsp;                                                           }
##&nbsp;                                                         ],
##&nbsp;                              'unshare' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'setattr' => [
##&nbsp;                                             {
##&nbsp;                                               'errors' => '0',
##&nbsp;                                               'ops' => '960'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                              'copyabort' => [
##&nbsp;                                               {
##&nbsp;                                                 'errors' => '0',
##&nbsp;                                                 'ops' => '0'
##&nbsp;                                               }
##&nbsp;                                             ],
##&nbsp;                              'reclaimlocksack' => [
##&nbsp;                                                     {
##&nbsp;                                                       'errors' => '0',
##&nbsp;                                                       'ops' => '0'
##&nbsp;                                                     }
##&nbsp;                                                   ],
##&nbsp;                              'copyauthcheck' => [
##&nbsp;                                                   {
##&nbsp;                                                     'errors' => '0',
##&nbsp;                                                     'ops' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;                              'close' => [
##&nbsp;                                           {
##&nbsp;                                             'errors' => '0',
##&nbsp;                                             'ops' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                              'share' => [
##&nbsp;                                           {
##&nbsp;                                             'errors' => '0',
##&nbsp;                                             'ops' => '0'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;                              'scanfile' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '0'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'fhtopath' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '0'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'copystatus' => [
##&nbsp;                                                {
##&nbsp;                                                  'errors' => '0',
##&nbsp;                                                  'ops' => '0'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'link' => [
##&nbsp;                                          {
##&nbsp;                                            'errors' => '0',
##&nbsp;                                            'ops' => '3'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                              'prefetch' => [
##&nbsp;                                              {
##&nbsp;                                                'errors' => '0',
##&nbsp;                                                'ops' => '0'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;                              'getvolumelanguage' => [
##&nbsp;                                                       {
##&nbsp;                                                         'errors' => '0',
##&nbsp;                                                         'ops' => '0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                              'discardcred' => [
##&nbsp;                                                 {
##&nbsp;                                                   'errors' => '0',
##&nbsp;                                                   'ops' => '0'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;                              'setlumetadata' => [
##&nbsp;                                                   {
##&nbsp;                                                     'errors' => '0',
##&nbsp;                                                     'ops' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ]
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'coralop_request_pool' => [
##&nbsp;                                        {
##&nbsp;                                          'unmarshal_errs' => '0',
##&nbsp;                                          'max_defer' => '0',
##&nbsp;                                          'cur_defer' => '0',
##&nbsp;                                          'defer_req' => '0',
##&nbsp;                                          'size' => '128',
##&nbsp;                                          'marshal_errs' => '0',
##&nbsp;                                          'total_req' => '0',
##&nbsp;                                          'total_replay_resp' => '0',
##&nbsp;                                          'cur_req' => '0',
##&nbsp;                                          'hipri' => '0',
##&nbsp;                                          'fastpath_null_resps' => '0',
##&nbsp;                                          'max_req' => '0',
##&nbsp;                                          'total_resp' => '0'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'total_7g_requests_started' => '1682493',
##&nbsp;            'total_7g_requests_completed' => '1682493',
##&nbsp;            'total_7g_bytes_written' => '89217605',
##&nbsp;            'filecb_request_pool' => [
##&nbsp;                                       {
##&nbsp;                                         'unmarshal_errs' => '0',
##&nbsp;                                         'max_defer' => '0',
##&nbsp;                                         'cur_defer' => '0',
##&nbsp;                                         'defer_req' => '0',
##&nbsp;                                         'size' => '97',
##&nbsp;                                         'marshal_errs' => '0',
##&nbsp;                                         'total_req' => '0',
##&nbsp;                                         'total_replay_resp' => '0',
##&nbsp;                                         'cur_req' => '0',
##&nbsp;                                         'hipri' => '0',
##&nbsp;                                         'fastpath_null_resps' => '0',
##&nbsp;                                         'max_req' => '0',
##&nbsp;                                         'total_resp' => '0'
##&nbsp;                                       }
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub spinhi_stats {
    my %args = @_;
    my @result_array;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $resp,
    );
    my @lines =
      get_lines( raw_output => $raw_output, response_object => $resp );
    my $head1; 
    my $head2;
    my $flag1 = 0;
    my $flag2 = 0;
    my $flag3 = 0;
    my $flag4 = 0;

    foreach my $line (@lines) {
        if ( $line =~ /(\w+\s\w+\s\w+):$/ ) {
            $head1 = $1;
            $flag1 = 1;
            $head1 = to_key_name( item => $head1 );
            next;
        }
        elsif ( $line =~
/(\w+):\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+):\s(\w+)=(\d+\.\d+)\s(\w+)=(\d+\.\d+)/
          )
        {
            $head2 = $1;
            $flag2 = 1;
            $head2 = to_key_name( item => $head2 );
            $result_array[0]->{$head1}->[0]->{$head2}->[0]->{$2} = $3;
            $result_array[0]->{$head1}->[0]->{$head2}->[0]->{$4} = $5;
            $result_array[0]->{$head1}->[0]->{$head2}->[0]->{$6} = $7;
            $result_array[0]->{$head1}->[0]->{$head2}->[0]->{$8}->[0]->{$9} =
              $10;
            $result_array[0]->{$head1}->[0]->{$head2}->[0]->{$8}->[0]->{$11} =
              $12;
        }
        elsif ( $line =~ /(Total.*):\s(\d+)/ ) {
            $head1                     = $1;
            $head1                     = to_key_name( item => $head1 );
            $result_array[0]->{$head1} = $2;
        }
        elsif ( $line =~ /(Op.*):\s+(\w+):\s(\d+)\s(\w+)\s(\d+)\s(\w+)/ ) {
            $head1 = $1;
            $head1 = to_key_name( item => $head1 );
            $result_array[0]->{$head1}->[0]->{ to_key_name( item => $2 ) }->[0]
              ->{$4} = $3;
            $result_array[0]->{$head1}->[0]->{ to_key_name( item => $2 ) }->[0]
              ->{$6} = $5;
        }
        elsif (
            $line =~ /(Response.*):\s(\w+)=(\d+)\s(\w+)=(\d+)\s+(\w+)=(\d+)/ )
        {
            $head1 = $1;
            $head1 = to_key_name( item => $head1 );
            $result_array[0]->{$head1}->[0]->{$2} = $3;
            $result_array[0]->{$head1}->[0]->{$4} = $5;
            $result_array[0]->{$head1}->[0]->{$6} = $7;
            $flag3                                = 1;
        }
        elsif (
            (
                $line =~
/(LocalBypass):\s(\w+)=(\d+)\s(\w+)=(\d+),\s(\w+)=(\d+)\s\((\w+)\s(\d)\)/
            )
            || ( $line =~
/(LocalBypass):\s(\w+)=(\d+)\S+\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)/
            )
          )
        {
            $head1 = $1;
            $head1 = to_key_name( item => $head1 );
            $result_array[0]->{$head1}->[0]->{$2} = $3;
            $result_array[0]->{$head1}->[0]->{$4} = $5;
            $result_array[0]->{$head1}->[0]->{$6} = $7;
            $result_array[0]->{$head1}->[0]->{$8} = $9;
        }
        elsif ( $line =~ /(spinhi memory):\s(\d+)\s(\w+)\s(\w+)\s(\w+)/ ) {
            $head1 = $1;
            $head1 = to_key_name( item => $head1 );
            $result_array[0]->{$head1}->[0]->{$3} = $2;
            $result_array[0]->{$head1}->[0]->{$5} = $4;
            $flag4                                = 1;
        }
        elsif ( $line =~ /(\w+):\s(\d+)\s(\w+)\s(\d+)\s(\w+)/ ) {
            $result_array[0]->{$head1}->[0]->{ to_key_name( item => $1 ) }->[0]
              ->{$3} = $2;
            $result_array[0]->{$head1}->[0]->{ to_key_name( item => $1 ) }->[0]
              ->{$5} = $4;
        }
        elsif ( $line =~ /""/ ) {
            next;
        }
        if ( $flag1 == 1 ) {
            if ( $line =~
/(size)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}  = $2;
                $result_array[0]->{$head1}->[0]->{$3}  = $4;
                $result_array[0]->{$head1}->[0]->{$5}  = $6;
                $result_array[0]->{$head1}->[0]->{$7}  = $8;
                $result_array[0]->{$head1}->[0]->{$9}  = $10;
                $result_array[0]->{$head1}->[0]->{$11} = $12;
            }
            elsif ( $line =~
/(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}  = $2;
                $result_array[0]->{$head1}->[0]->{$3}  = $4;
                $result_array[0]->{$head1}->[0]->{$5}  = $6;
                $result_array[0]->{$head1}->[0]->{$7}  = $8;
                $result_array[0]->{$head1}->[0]->{$9}  = $10;
                $result_array[0]->{$head1}->[0]->{$11} = $12;
                $result_array[0]->{$head1}->[0]->{$13} = $14;
            }
        }
        if ( $flag2 == 1 ) {
            if ( $line =~ /(spinhi.*):\s(\w+)=(\d+\.\d+)\s(\w+)=(\d+\.\d+)/ ) {
                $result_array[0]->{$head1}->[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }->[0]
                  ->{$2} = $3;
                $result_array[0]->{$head1}->[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }->[0]
                  ->{$4} = $5;

            }
            elsif ( $line =~
                /(wafl.*):\s(\w+)=(\d+\.\d+)\s(\w+)=(\d+\.\d+)\s(\w+)=(\d+)/ )
            {
                $result_array[0]->{$head1}->[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }->[0]
                  ->{$2} = $3;
                $result_array[0]->{$head1}->[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }->[0]
                  ->{$4} = $5;
                $result_array[0]->{$head1}->[0]->{$head2}->[0]->{ to_key_name( item => $1 ) }->[0]
                  ->{$6} = $7;
            }
            elsif ( $line =~ /(errs):\s(.*)/ ) {
                $result_array[0]->{$head1}->[0]->{$head2}->[0]
                  ->{"error_count"} = $2;
            }
        }
        if ( $flag3 == 1 ) {
            if ( $line =~
/(globals):\s(\w+)=(\d+)\s(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$2} = $3;
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$4} = $5;
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$6} = $7;
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$8} = $9;
            }
            elsif ( $line =~
/(per-cpu):\s(\w+)=(\d+)\s(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{"per_cpu"}->[0]->{$2} = $3;
                $result_array[0]->{$head1}->[0]->{"per_cpu"}->[0]->{$4} = $5;
                $result_array[0]->{$head1}->[0]->{"per_cpu"}->[0]->{$6} = $7;
                $result_array[0]->{$head1}->[0]->{"per_cpu"}->[0]->{$8} = $9;
            }
        }
        if ( $flag4 == 1 ) {
            if ( $line =~
/\#(\w+)=(\d+)\s\((\d+)\s(\w+)\)\s\#(\w+)=(\d+)\s\#(\w+)=(\d+)\s\#(\w+)=(\d+)\s\#(\w+)=(\d+)\s(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}  = $2;
                $result_array[0]->{$head1}->[0]->{$4}  = $3;
                $result_array[0]->{$head1}->[0]->{$5}  = $6;
                $result_array[0]->{$head1}->[0]->{$7}  = $8;
                $result_array[0]->{$head1}->[0]->{$9}  = $10;
                $result_array[0]->{$head1}->[0]->{$11} = $12;
                $result_array[0]->{$head1}->[0]->{$13} = $14;
            }
            elsif ( $line =~
/(allocated)=(\d+)\s(\w+)=(\d+)\s(\S+)=\[(\w):(\d)\s(\w):(\d)\s(\w+):(\d)\]\s(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}             = $2;
                $result_array[0]->{$head1}->[0]->{$3}             = $4;
                $result_array[0]->{$head1}->[0]->{$5}->[0]->{$6}  = $7;
                $result_array[0]->{$head1}->[0]->{$5}->[0]->{$8}  = $9;
                $result_array[0]->{$head1}->[0]->{$5}->[0]->{$10} = $11;
                $result_array[0]->{$head1}->[0]->{$12}            = $13;
            }
            elsif ( $line =~
/(free_counts):\s(\w+)=(\d+)\s(\w+)=(\d+)\s(\w+)=(\d+)\s\#(\w+):\s(\w+)=(\d+)\s(\w+)=(\d+)/
              )
            {
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$2}  = $3;
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$4}  = $5;
                $result_array[0]->{$head1}->[0]->{$1}->[0]->{$6}  = $7;
                $result_array[0]->{$head1}->[0]->{$8}->[0]->{$9}  = $10;
                $result_array[0]->{$head1}->[0]->{$8}->[0]->{$11} = $12;
            }
        }
    }
    return \@result_array;
}

1;
