#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode statit command output
##
## @description
##    This is a library to parse the raw output of 7Mode statit command.
##
## @synopsis
##    use statit;
##
##    statit(%args);
##
## @author alokik@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::statit;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(statit: expected)\s+(.*)/ ) {
        $$error_message = $2;
        $error          = "CommandFailed";
        if ( $$error_message =~ /([Ii]nvalid|[Uu]nrecognized) option/ ) {
            $error = "InvalidParam";
        } else {
            $$error_message
                = "Command failed due to wrong usage or dependencies";
        }
    } elsif($raw_output =~ /usage: statit/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::statit
##
## @summary Parses the raw output from the "statit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from statit with any of its options as described in the
##    corresponding cdef file 'statit.cdefs'
## @synopsis
##      statit(%args);
## INPUTS TO METHOD CALL 'statit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-r' </td> <td> 'report' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'beginning' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'nfs' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'ending' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; statit -e; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Hostname: csiqa-2040-16a  ID: 0135092892  Memory: 2812 MB
##&nbsp;  NetApp Release RrollingrockN_120202_2045 7-Mode: Thu Feb  2 21:06:24 PST 2012
##&nbsp;    <5d>
##&nbsp;  Start time: Mon Feb 20 05:11:45 GMT 2012
##&nbsp;
##&nbsp;                       CPU Statistics
##&nbsp;      10.087663 time (seconds)       100 %
##&nbsp;       0.438846 system time            4 %
##&nbsp;       0.110192 rupt time              1 %   (41269 rupts x 3 usec/rupt)
##&nbsp;       0.328654 non-rupt system time   3 %
##&nbsp;      19.736478 idle time            196 %
##&nbsp;
##&nbsp;       0.086035 time in CP             1 %   100 %
##&nbsp;       0.001037 rupt time in CP                1 %   (356 rupts x 3 usec/rupt)
##&nbsp;
##&nbsp;                       Multiprocessor Statistics (per second)
##&nbsp;                          cpu0       cpu1      total
##&nbsp;sk switches            2340.98    2392.23    4733.21
##&nbsp;hard switches          1237.45    1279.98    2517.43
##&nbsp;domain switches          75.44      95.26     170.70
##&nbsp;CP rupts                 17.45      17.84      35.29
##&nbsp;nonCP rupts            2040.61    2015.13    4055.75
##&nbsp;IPI rupts                 0.00       0.00       0.00
##&nbsp;grab kahuna               0.00       0.00       0.00
##&nbsp;
##&nbsp;grab kahuna usec          0.00       0.00       0.00
##&nbsp;
##&nbsp;CP rupt usec             84.86      17.84     102.80
##&nbsp;nonCP rupt usec        9243.87    1576.68   10820.64
##&nbsp;idle                 975544.09  980952.58 1956496.66
##&nbsp;kahuna                 2231.54    6936.99    9168.63
##&nbsp;storage                1200.48    1573.41    2773.88
##&nbsp;exempt                  915.97    1146.35    2062.32
##&nbsp;raid                    934.01     809.40    1743.42
##&nbsp;target                   21.91      10.81      32.71
##&nbsp;dnscache                  0.00       9.12       9.12
##&nbsp;cifs                      6.25       7.04      13.28
##&nbsp;wafl_exempt               0.00       0.00       0.00
##&nbsp;wafl_xcleaner             0.00       0.00       0.00
##&nbsp;sm_exempt                67.61      45.80     113.41
##&nbsp;cluster                   0.00       0.00       0.00
##&nbsp;protocol                  0.00       0.00       0.00
##&nbsp;nwk_exclusive             0.00       0.00       0.00
##&nbsp;nwk_exempt                0.00       0.00       0.00
##&nbsp;nwk_legacy             5030.30    2012.95    7043.36
##&nbsp;hostOS                 4718.54    4900.34    9618.98
##&nbsp;
##&nbsp;       0.214958 seconds with one or more CPUs active   (  2%)
##&nbsp;
##&nbsp;       0.188354 seconds with one CPU active            (  2%)
##&nbsp;       0.026603 seconds with both CPUs active          (  0%)
##&nbsp;
##&nbsp;                       Domain Utilization of Shared Domains (per second)
##&nbsp;      0.00 idle                              0.00 kahuna
##&nbsp;      0.00 storage                           0.00 exempt
##&nbsp;      0.00 raid                              0.00 target
##&nbsp;      0.00 dnscache                          0.00 cifs
##&nbsp;      0.00 wafl_exempt                       0.00 wafl_xcleaner
##&nbsp;      0.00 sm_exempt                         0.00 cluster
##&nbsp;      0.00 protocol                          0.00 nwk_exclusive
##&nbsp;      0.00 nwk_exempt                        0.00 nwk_legacy
##&nbsp;      0.00 hostOS
##&nbsp;
##&nbsp;                       Miscellaneous Statistics (per second)
##&nbsp;   2517.43 hard context switches             0.00 NFS operations
##&nbsp;      0.00 CIFS operations                   0.00 HTTP operations
##&nbsp;      3.17 network KB received               0.30 network KB transmitted
##&nbsp;      3.97 disk KB read                     14.27 disk KB written
##&nbsp;      0.20 NVRAM KB written                  0.00 nolog KB written
##&nbsp;      0.00 WAFL bufs given to clients        0.00 checksum cache hits  (   0%)
##&nbsp;      0.00 no checksum - partial buffer      0.00 FCP operations
##&nbsp;      0.00 iSCSI operations
##&nbsp;
##&nbsp;                       WAFL Statistics (per second)
##&nbsp;      4.86 name cache hits      (  88%)      0.69 name cache misses    (  13%)
##&nbsp;    400.39 buf hash hits        ( 100%)      0.79 buf hash misses      (   0%)
##&nbsp;     16.55 inode cache hits     ( 100%)      0.00 inode cache misses   (   0%)
##&nbsp;     41.64 buf cache hits       ( 100%)      0.00 buf cache misses     (   0%)
##&nbsp;      0.00 blocks read                       0.00 blocks read-ahead
##&nbsp;      0.00 chains read-ahead                 0.00 dummy reads
##&nbsp;      0.00 blocks speculative read-ahead      0.00 blocks written
##&nbsp;      0.00 stripes written                   0.00 blocks page flipped
##&nbsp;      0.00 blocks over-written               0.10 wafl_timer generated CP
##&nbsp;      0.00 snapshot generated CP             0.00 wafl_avail_bufs generated CP
##&nbsp;      0.00 dirty_blk_cnt generated CP        0.00 full NV-log generated CP
##&nbsp;      0.00 back-to-back CP                   0.00 flush generated CP
##&nbsp;      0.00 sync generated CP                 0.00 deferred back-to-back CP
##&nbsp;      0.00 container-indirect-pin CP         0.00 low mbufs generated CP
##&nbsp;      0.00 low datavecs generated CP        68.50 non-restart messages
##&nbsp;      0.00 IOWAIT suspends                   0.00 next nvlog nearly full msecs
##&nbsp;      0.00 dirty buffer susp msecs           0.00 nvlog full susp msecs
##&nbsp;    415028 buffers
##&nbsp;
##&nbsp;                       RAID Statistics (per second)
##&nbsp;      0.20 xors                              0.00 long dispatches [0]
##&nbsp;      0.00 long consumed [0]                 0.00 long consumed hipri [0]
##&nbsp;      0.00 long low priority [0]             0.00 long high priority [0]
##&nbsp;      0.00 long monitor tics [0]             0.00 long monitor clears [0]
##&nbsp;      0.00 long dispatches [1]               0.00 long consumed [1]
##&nbsp;      0.00 long consumed hipri [1]           0.00 long low priority [1]
##&nbsp;      0.00 long high priority [1]            0.00 long monitor tics [1]
##&nbsp;      0.00 long monitor clears [1]             17 max batch
##&nbsp;      0.20 blocked mode xor                  0.00 timed mode xor
##&nbsp;      0.00 fast adjustments                  0.00 slow adjustments
##&nbsp;         0 avg batch start                      0 avg stripe/msec
##&nbsp;      0.00 checksum dispatches               0.00 checksum consumed
##&nbsp;      0.30 tetrises written                  0.00 master tetrises
##&nbsp;      0.00 slave tetrises                    2.38 stripes written
##&nbsp;      0.00 partial stripes                   2.38 full stripes
##&nbsp;      2.38 blocks written                    0.00 blocks read
##&nbsp;      2.38 1 blocks per stripe size 1
##&nbsp;
##&nbsp;                       Network Interface Statistics (per second)
##&nbsp;iface    side      bytes    packets multicasts     errors collisions  pkt drops
##&nbsp;e0a      recv       0.00       0.00       0.00       0.00                  0.00
##&nbsp;         xmit       0.00       0.00       0.00       0.00       0.00
##&nbsp;e0b      recv       0.00       0.00       0.00       0.00                  0.00
##&nbsp;         xmit       0.00       0.00       0.00       0.00       0.00
##&nbsp;e0c      recv       0.00       0.00       0.00       0.00                  0.00
##&nbsp;         xmit       0.00       0.00       0.00       0.00       0.00
##&nbsp;e0d      recv    3254.37      47.68      44.61       0.00                  0.00
##&nbsp;         xmit     339.03       2.87       0.00       0.00       0.00
##&nbsp;e0P      recv       0.00       0.00       0.00       0.00                  0.00
##&nbsp;         xmit       0.00       0.00       0.00       0.00       0.00
##&nbsp;vh       recv       0.00       0.00       0.00       0.00                  0.00
##&nbsp;         xmit       0.00       0.00       0.00       0.00       0.00
##&nbsp;
##&nbsp;                       Disk Statistics (per second)
##&nbsp;	ut% is the percent of time the disk was busy.
##&nbsp;	xfers is the number of data-transfer commands issued per second.
##&nbsp;	xfers = ureads + writes + cpreads + greads + gwrites
##&nbsp;	chain is the average number of 4K blocks per command.
##&nbsp;	usecs is the average disk round-trip time per 4K block.
##&nbsp;
##&nbsp;disk             ut%  xfers  ureads--chain-usecs writes--chain-usecs cpreads-chain-usecs greads--chain-usecs gwrites-chain-usecs
##&nbsp;/test_aggr/plex0/rg0:
##&nbsp;0a.22              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.28              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.24              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;/aggr0/plex0/rg0:
##&nbsp;0a.16              5   1.98    0.40   1.00  4500   1.59   1.00  3188   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.17              5   2.08    0.40   1.00  3000   1.69   1.06  2500   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.18              1   0.40    0.20   1.00     0   0.20   1.00 25500   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;/test_vol1/plex0/rg0:
##&nbsp;0a.25              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.26              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0a.27              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;/test_aggr1/plex0/rg0:
##&nbsp;0c.00.0            0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0c.00.1            0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0c.00.2            0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;
##&nbsp;Aggregate statistics:
##&nbsp;Minimum            0   0.00    0.00                0.00                0.00                0.00                0.00
##&nbsp;Mean               1   0.30    0.00                0.20                0.00                0.00                0.00
##&nbsp;Maximum            5   2.08    0.40                1.69                0.00                0.00                0.00
##&nbsp;
##&nbsp;Spares and other disks:
##&nbsp;0b.18              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;0b.20              0   0.00    0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .   0.00   ....     .
##&nbsp;
##&nbsp;                       FCP Statistics (per second)
##&nbsp;      0.00 FCP Bytes recv                    0.00 FCP Bytes sent
##&nbsp;      0.00 FCP ops
##&nbsp;
##&nbsp;                       iSCSI Statistics (per second)
##&nbsp;      0.00 iSCSI Bytes recv                  0.00 iSCSI Bytes xmit
##&nbsp;      0.00 iSCSI ops
##&nbsp;
##&nbsp;                       Interrupt Statistics (per second)
##&nbsp;     36.78 int_1                            22.11 int_2
##&nbsp;      2.08 int_3                            19.13 Gigabit Ethernet (IRQ 4)
##&nbsp;      5.55 Gigabit Ethernet (IRQ 6)          5.55 Gigabit Ethernet (IRQ 7)
##&nbsp;     36.78 int_8                            36.78 int_9
##&nbsp;      0.00 RTC                               0.00 IPI
##&nbsp;   1001.82 Msec Clock                     1166.57 total
##&nbsp;
##&nbsp;                       NVRAM Statistics (per second)
##&nbsp;      0.00 total dma transfer KB             0.00 wafl write req data KB
##&nbsp;      0.00 dma transactions                  0.00 dma destriptors
##&nbsp;      0.00 waitdone preempts                 0.00 waitdone delays
##&nbsp;      0.00 transactions not queued           0.59 transactions queued
##&nbsp;      0.59 transactions done                 0.00 total waittime (MS)
##&nbsp;      3.07 completion wakeups                0.00 nvdma completion wakeups
##&nbsp;      0.00 nvdma completion waitdone         0.30 total nvlog KB
##&nbsp;      0.00 nvlog shadow header array full      0.00 channel1 dma transfer KB
##&nbsp;      0.00 channel1 dma transactions         0.00 channel1 dma descriptors
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            '/test_aggr/plex0/rg0' => [
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.22',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.28',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.24',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'wafl_statistics' => [
##&nbsp;                                   {
##&nbsp;                                     'buffers' => '415028',
##&nbsp;                                     'stripes_written' => '0.00',
##&nbsp;                                     'blocks_readahead' => '0.00',
##&nbsp;                                     'low_mbufs_generated' => '0.00',
##&nbsp;                                     'wafl_timer_generated' => '0.10',
##&nbsp;                                     'backtoback' => '0.00',
##&nbsp;                                     'nvlog_full_susp_msecs' => '0.00',
##&nbsp;                                     'wafl_avail_bufs_generated' => '0.00',
##&nbsp;                                     'snapshot_generated' => '0.00',
##&nbsp;                                     'flush_generated' => '0.00',
##&nbsp;                                     'deferred_backtoback' => '0.00',
##&nbsp;                                     'name_cache_hits' => '4.86',
##&nbsp;                                     'low_datavecs_generated' => '0.00',
##&nbsp;                                     'containerindirectpin' => '0.00',
##&nbsp;                                     'blocks_speculative_readahead' => '0.00',
##&nbsp;                                     'full_log_generated' => '0.00',
##&nbsp;                                     'inode_cache_hits' => '16.55',
##&nbsp;                                     'dummy_reads' => '0.00',
##&nbsp;                                     'buf_hash_misses' => '0.79',
##&nbsp;                                     'dirty_buffer_susp_msecs' => '0.00',
##&nbsp;                                     'name_cache_misses' => '0.69',
##&nbsp;                                     'buf_cache_hits' => '41.64',
##&nbsp;                                     'blocks_overwritten' => '0.00',
##&nbsp;                                     'nonrestart_messages' => '68.50',
##&nbsp;                                     'blocks_page_flipped' => '0.00',
##&nbsp;                                     'sync_generated' => '0.00',
##&nbsp;                                     'next_nvlog_nearly_full_msecs' => '0.00',
##&nbsp;                                     'dirty_blk_cnt_generated' => '0.00',
##&nbsp;                                     'suspends' => '0.00',
##&nbsp;                                     'buf_cache_misses' => '0.00',
##&nbsp;                                     'chains_readahead' => '0.00',
##&nbsp;                                     'blocks_read' => '0.00',
##&nbsp;                                     'inode_cache_misses' => '0.00',
##&nbsp;                                     'blocks_written' => '0.00',
##&nbsp;                                     'buf_hash_hits' => '400.39'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'date' => 'Thu Feb  2 21:06:24 PST 2012',
##&nbsp;            'netapp_release' => 'RrollingrockN_120202_2045 7-Mode',
##&nbsp;            'hostname' => 'csiqa-2040-16a',
##&nbsp;            'miscellaneous_statistics' => [
##&nbsp;                                            {
##&nbsp;                                              'network_transmitted' => '0.30',
##&nbsp;                                              'written' => '0.20',
##&nbsp;                                              'bufs_given_to_clients' => '0.00',
##&nbsp;                                              'disk_written' => '14.27',
##&nbsp;                                              'disk_read' => '3.97',
##&nbsp;                                              'nolog_written' => '0.00',
##&nbsp;                                              'network_received' => '3.17',
##&nbsp;                                              'iscsi_operations' => '0.00',
##&nbsp;                                              'checksum_cache_hits' => '0.00',
##&nbsp;                                              'operations' => '0.00',
##&nbsp;                                              'hard_context_switches' => '2517.43',
##&nbsp;                                              'no_checksum_partial_buffer' => '0.00'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            '/test_aggr1/plex0/rg0' => [
##&nbsp;                                         {
##&nbsp;                                           'disk' => '0c.00.0',
##&nbsp;                                           'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'xfers' => '0.00',
##&nbsp;                                           'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ut%' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'disk' => '0c.00.1',
##&nbsp;                                           'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'xfers' => '0.00',
##&nbsp;                                           'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ut%' => '0'
##&nbsp;                                         },
##&nbsp;                                         {
##&nbsp;                                           'disk' => '0c.00.2',
##&nbsp;                                           'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'xfers' => '0.00',
##&nbsp;                                           'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                           'ut%' => '0'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;            '/test_vol1/plex0/rg0' => [
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.25',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.26',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        },
##&nbsp;                                        {
##&nbsp;                                          'disk' => '0a.27',
##&nbsp;                                          'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'xfers' => '0.00',
##&nbsp;                                          'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                          'ut%' => '0'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'id' => '0135092892',
##&nbsp;            'raid_statistics' => [
##&nbsp;                                   {
##&nbsp;                                     'tetrises_written' => '0.30',
##&nbsp;                                     'max_batch' => '17',
##&nbsp;                                     'avg_batch_start' => '0',
##&nbsp;                                     'long_monitor_tics' => '0.00',
##&nbsp;                                     'partial_stripes' => '0.00',
##&nbsp;                                     'stripes_written' => '2.38',
##&nbsp;                                     'blocked_mode_xor' => '0.20',
##&nbsp;                                     'long_consumed_hipri' => '0.00',
##&nbsp;                                     'long_monitor_clears' => '0.00',
##&nbsp;                                     'checksum_dispatches' => '0.00',
##&nbsp;                                     'fast_adjustments' => '0.00',
##&nbsp;                                     'long_consumed' => '0.00',
##&nbsp;                                     'long_low_priority' => '0.00',
##&nbsp;                                     'checksum_consumed' => '0.00',
##&nbsp;                                     'long_dispatches' => '0.00',
##&nbsp;                                     'long_high_priority' => '0.00',
##&nbsp;                                     'full_stripes' => '2.38',
##&nbsp;                                     'timed_mode_xor' => '0.00',
##&nbsp;                                     'blocks_read' => '0.00',
##&nbsp;                                     'xors' => '0.20',
##&nbsp;                                     '1_blocks_per_stripe_size_1' => '2.38',
##&nbsp;                                     'blocks_written' => '2.38',
##&nbsp;                                     'slave_tetrises' => '0.00',
##&nbsp;                                     'avg_stripemsec' => '0',
##&nbsp;                                     'slow_adjustments' => '0.00',
##&nbsp;                                     'master_tetrises' => '0.00'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'domain_utilization_of_shared_domains' => [
##&nbsp;                                                        {
##&nbsp;                                                          'protocol' => '0.00',
##&nbsp;                                                          'cluster' => '0.00',
##&nbsp;                                                          'sm_exempt' => '0.00',
##&nbsp;                                                          'kahuna' => '0.00',
##&nbsp;                                                          'dnscache' => '0.00',
##&nbsp;                                                          'nwk_exclusive' => '0.00',
##&nbsp;                                                          'exempt' => '0.00',
##&nbsp;                                                          'idle' => '0.00',
##&nbsp;                                                          'cifs' => '0.00',
##&nbsp;                                                          'raid' => '0.00',
##&nbsp;                                                          'target' => '0.00',
##&nbsp;                                                          'nwk_legacy' => '0.00',
##&nbsp;                                                          'nwk_exempt' => '0.00',
##&nbsp;                                                          'wafl_xcleaner' => '0.00',
##&nbsp;                                                          'storage' => '0.00',
##&nbsp;                                                          'wafl_exempt' => '0.00'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;            'iscsi_statistics' => [
##&nbsp;                                    {
##&nbsp;                                      'iscsi_bytes_xmit' => '0.00',
##&nbsp;                                      'iscsi_bytes_recv' => '0.00',
##&nbsp;                                      'iscsi_ops' => '0.00'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'multiprocessor_statistics' => [
##&nbsp;                                             {
##&nbsp;                                               'cpu1' => [
##&nbsp;                                                           {
##&nbsp;                                                             'protocol' => '0.00',
##&nbsp;                                                             'sm_exempt' => '45.80',
##&nbsp;                                                             'cp_rupt_usec' => '17.84',
##&nbsp;                                                             'domain_switches' => '95.26',
##&nbsp;                                                             'kahuna' => '6936.99',
##&nbsp;                                                             'dnscache' => '9.12',
##&nbsp;                                                             'nwk_exclusive' => '0.00',
##&nbsp;                                                             'hostos' => '4900.34',
##&nbsp;                                                             'idle' => '980952.58',
##&nbsp;                                                             'sk_switches' => '2392.23',
##&nbsp;                                                             'cifs' => '7.04',
##&nbsp;                                                             'raid' => '809.40',
##&nbsp;                                                             'target' => '10.81',
##&nbsp;                                                             'nwk_exempt' => '0.00',
##&nbsp;                                                             'grab_kahuna_usec' => '0.00',
##&nbsp;                                                             'cluster' => '0.00',
##&nbsp;                                                             'noncp_rupt_usec' => '1576.68',
##&nbsp;                                                             'ipi_rupts' => '0.00',
##&nbsp;                                                             'exempt' => '1146.35',
##&nbsp;                                                             'hard_switches' => '1279.98',
##&nbsp;                                                             'nwk_legacy' => '2012.95',
##&nbsp;                                                             'grab_kahuna' => '0.00',
##&nbsp;                                                             'cp_rupts' => '17.84',
##&nbsp;                                                             'wafl_xcleaner' => '0.00',
##&nbsp;                                                             'noncp_rupts' => '2015.13',
##&nbsp;                                                             'storage' => '1573.41',
##&nbsp;                                                             'wafl_exempt' => '0.00'
##&nbsp;                                                           }
##&nbsp;                                                         ],
##&nbsp;                                               'cpu0' => [
##&nbsp;                                                           {
##&nbsp;                                                             'protocol' => '0.00',
##&nbsp;                                                             'sm_exempt' => '67.61',
##&nbsp;                                                             'cp_rupt_usec' => '84.86',
##&nbsp;                                                             'domain_switches' => '75.44',
##&nbsp;                                                             'kahuna' => '2231.54',
##&nbsp;                                                             'dnscache' => '0.00',
##&nbsp;                                                             'nwk_exclusive' => '0.00',
##&nbsp;                                                             'hostos' => '4718.54',
##&nbsp;                                                             'idle' => '975544.09',
##&nbsp;                                                             'sk_switches' => '2340.98',
##&nbsp;                                                             'cifs' => '6.25',
##&nbsp;                                                             'raid' => '934.01',
##&nbsp;                                                             'target' => '21.91',
##&nbsp;                                                             'nwk_exempt' => '0.00',
##&nbsp;                                                             'grab_kahuna_usec' => '0.00',
##&nbsp;                                                             'cluster' => '0.00',
##&nbsp;                                                             'noncp_rupt_usec' => '9243.87',
##&nbsp;                                                             'ipi_rupts' => '0.00',
##&nbsp;                                                             'exempt' => '915.97',
##&nbsp;                                                             'hard_switches' => '1237.45',
##&nbsp;                                                             'nwk_legacy' => '5030.30',
##&nbsp;                                                             'grab_kahuna' => '0.00',
##&nbsp;                                                             'cp_rupts' => '17.45',
##&nbsp;                                                             'wafl_xcleaner' => '0.00',
##&nbsp;                                                             'noncp_rupts' => '2040.61',
##&nbsp;                                                             'storage' => '1200.48',
##&nbsp;                                                             'wafl_exempt' => '0.00'
##&nbsp;                                                           }
##&nbsp;                                                         ],
##&nbsp;                                               'total' => [
##&nbsp;                                                            {
##&nbsp;                                                              'protocol' => '0.00',
##&nbsp;                                                              'sm_exempt' => '113.41',
##&nbsp;                                                              'cp_rupt_usec' => '102.80',
##&nbsp;                                                              'domain_switches' => '170.70',
##&nbsp;                                                              'kahuna' => '9168.63',
##&nbsp;                                                              'dnscache' => '9.12',
##&nbsp;                                                              'nwk_exclusive' => '0.00',
##&nbsp;                                                              'hostos' => '9618.98',
##&nbsp;                                                              'idle' => '1956496.66',
##&nbsp;                                                              'sk_switches' => '4733.21',
##&nbsp;                                                              'cifs' => '13.28',
##&nbsp;                                                              'raid' => '1743.42',
##&nbsp;                                                              'target' => '32.71',
##&nbsp;                                                              'nwk_exempt' => '0.00',
##&nbsp;                                                              'grab_kahuna_usec' => '0.00',
##&nbsp;                                                              'cluster' => '0.00',
##&nbsp;                                                              'noncp_rupt_usec' => '10820.64',
##&nbsp;                                                              'ipi_rupts' => '0.00',
##&nbsp;                                                              'exempt' => '2062.32',
##&nbsp;                                                              'hard_switches' => '2517.43',
##&nbsp;                                                              'nwk_legacy' => '7043.36',
##&nbsp;                                                              'grab_kahuna' => '0.00',
##&nbsp;                                                              'cp_rupts' => '35.29',
##&nbsp;                                                              'wafl_xcleaner' => '0.00',
##&nbsp;                                                              'noncp_rupts' => '4055.75',
##&nbsp;                                                              'storage' => '2773.88',
##&nbsp;                                                              'wafl_exempt' => '0.00'
##&nbsp;                                                            }
##&nbsp;                                                          ]
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'fcp_statistics' => [
##&nbsp;                                  {
##&nbsp;                                    'fcp_ops' => '0.00',
##&nbsp;                                    'fcp_bytes_recv' => '0.00',
##&nbsp;                                    'fcp_bytes_sent' => '0.00'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'aggregate_statistics' => [
##&nbsp;                                        {
##&nbsp;                                          'minimum' => [
##&nbsp;                                                         {
##&nbsp;                                                           'writes_chain_usecs' => '0.00',
##&nbsp;                                                           'cpreads_chain_usecs' => '0.00',
##&nbsp;                                                           'greads_chain_usecs' => '0.00',
##&nbsp;                                                           'ureads_chain_usecs' => '0.00',
##&nbsp;                                                           'gwrites_chain_usecs' => '0.00',
##&nbsp;                                                           'ut%' => '0',
##&nbsp;                                                           'xfers' => '0.00'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;                                          'maximum' => [
##&nbsp;                                                         {
##&nbsp;                                                           'writes_chain_usecs' => '1.69',
##&nbsp;                                                           'cpreads_chain_usecs' => '0.00',
##&nbsp;                                                           'greads_chain_usecs' => '0.00',
##&nbsp;                                                           'ureads_chain_usecs' => '0.40',
##&nbsp;                                                           'gwrites_chain_usecs' => '0.00',
##&nbsp;                                                           'ut%' => '5',
##&nbsp;                                                           'xfers' => '2.08'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;                                          'mean' => [
##&nbsp;                                                      {
##&nbsp;                                                        'writes_chain_usecs' => '0.20',
##&nbsp;                                                        'cpreads_chain_usecs' => '0.00',
##&nbsp;                                                        'greads_chain_usecs' => '0.00',
##&nbsp;                                                        'ureads_chain_usecs' => '0.00',
##&nbsp;                                                        'gwrites_chain_usecs' => '0.00',
##&nbsp;                                                        'ut%' => '1',
##&nbsp;                                                        'xfers' => '0.30'
##&nbsp;                                                      }
##&nbsp;                                                    ]
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'start_time' => 'Mon Feb 20 05:11:45 GMT 2012',
##&nbsp;            '/aggr0/plex0/rg0' => [
##&nbsp;                                    {
##&nbsp;                                      'disk' => '0a.16',
##&nbsp;                                      'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ureads_chain_usecs' => '0.40   1.00  4500',
##&nbsp;                                      'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'xfers' => '1.98',
##&nbsp;                                      'writes_chain_usecs' => '1.59   1.00  3188',
##&nbsp;                                      'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ut%' => '5'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'disk' => '0a.17',
##&nbsp;                                      'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ureads_chain_usecs' => '0.40   1.00  3000',
##&nbsp;                                      'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'xfers' => '2.08',
##&nbsp;                                      'writes_chain_usecs' => '1.69   1.06  2500',
##&nbsp;                                      'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ut%' => '5'
##&nbsp;                                    },
##&nbsp;                                    {
##&nbsp;                                      'disk' => '0a.18',
##&nbsp;                                      'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ureads_chain_usecs' => '0.20   1.00     0',
##&nbsp;                                      'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'xfers' => '0.40',
##&nbsp;                                      'writes_chain_usecs' => '0.20   1.00 25500',
##&nbsp;                                      'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                      'ut%' => '1'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'memory' => '2812 MB',
##&nbsp;            'spares_and_other_disks' => [
##&nbsp;                                          {
##&nbsp;                                            'disk' => '0b.18',
##&nbsp;                                            'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'xfers' => '0.00',
##&nbsp;                                            'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'ut%' => '0'
##&nbsp;                                          },
##&nbsp;                                          {
##&nbsp;                                            'disk' => '0b.20',
##&nbsp;                                            'greads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'ureads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'gwrites_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'xfers' => '0.00',
##&nbsp;                                            'writes_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'cpreads_chain_usecs' => '0.00   ....     .',
##&nbsp;                                            'ut%' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'network_interface_statistics' => [
##&nbsp;                                                {
##&nbsp;                                                  'e0a' => [
##&nbsp;                                                             {
##&nbsp;                                                               'recv' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'packets' => '0.00',
##&nbsp;                                                                           'packet_drops' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             },
##&nbsp;                                                             {
##&nbsp;                                                               'xmit' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'collisions' => '0.00',
##&nbsp;                                                                           'packets' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'e0b' => [
##&nbsp;                                                             {
##&nbsp;                                                               'recv' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'packets' => '0.00',
##&nbsp;                                                                           'packet_drops' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             },
##&nbsp;                                                             {
##&nbsp;                                                               'xmit' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'collisions' => '0.00',
##&nbsp;                                                                           'packets' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'e0c' => [
##&nbsp;                                                             {
##&nbsp;                                                               'recv' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'packets' => '0.00',
##&nbsp;                                                                           'packet_drops' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             },
##&nbsp;                                                             {
##&nbsp;                                                               'xmit' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'collisions' => '0.00',
##&nbsp;                                                                           'packets' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'e0d' => [
##&nbsp;                                                             {
##&nbsp;                                                               'recv' => {
##&nbsp;                                                                           'bytes' => '3254.37',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '44.61',
##&nbsp;                                                                           'packets' => '47.68',
##&nbsp;                                                                           'packet_drops' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             },
##&nbsp;                                                             {
##&nbsp;                                                               'xmit' => {
##&nbsp;                                                                           'bytes' => '339.03',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'collisions' => '0.00',
##&nbsp;                                                                           'packets' => '2.87'
##&nbsp;                                                                         }
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'e0p' => [
##&nbsp;                                                             {
##&nbsp;                                                               'recv' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'packets' => '0.00',
##&nbsp;                                                                           'packet_drops' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             },
##&nbsp;                                                             {
##&nbsp;                                                               'xmit' => {
##&nbsp;                                                                           'bytes' => '0.00',
##&nbsp;                                                                           'errors' => '0.00',
##&nbsp;                                                                           'multicast' => '0.00',
##&nbsp;                                                                           'collisions' => '0.00',
##&nbsp;                                                                           'packets' => '0.00'
##&nbsp;                                                                         }
##&nbsp;                                                             }
##&nbsp;                                                           ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'vh' => [
##&nbsp;                                                            {
##&nbsp;                                                              'recv' => {
##&nbsp;                                                                          'bytes' => '0.00',
##&nbsp;                                                                          'errors' => '0.00',
##&nbsp;                                                                          'multicast' => '0.00',
##&nbsp;                                                                          'packets' => '0.00',
##&nbsp;                                                                          'packet_drops' => '0.00'
##&nbsp;                                                                        }
##&nbsp;                                                            },
##&nbsp;                                                            {
##&nbsp;                                                              'xmit' => {
##&nbsp;                                                                          'bytes' => '0.00',
##&nbsp;                                                                          'errors' => '0.00',
##&nbsp;                                                                          'multicast' => '0.00',
##&nbsp;                                                                          'collisions' => '0.00',
##&nbsp;                                                                          'packets' => '0.00'
##&nbsp;                                                                        }
##&nbsp;                                                            }
##&nbsp;                                                          ]
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;            'cpu_statistics' => [
##&nbsp;                                  {
##&nbsp;                                    'system_time' => '0.438846',
##&nbsp;                                    'non-rupt_system_time' => '0.328654',
##&nbsp;                                    'idle_time' => '19.736478',
##&nbsp;                                    'rupt_time' => '0.110192',
##&nbsp;                                    'time' => '10.087663',
##&nbsp;                                    'rupt_time_in_cp' => '0.001037',
##&nbsp;                                    'time_in_cp' => '0.086035'
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'interrupt_statistics' => [
##&nbsp;                                        {
##&nbsp;                                          'msec_clock' => '1001.82',
##&nbsp;                                          'rtc' => '0.00',
##&nbsp;                                          'gigabit_ethernet_irq' => '5.55',
##&nbsp;                                          'int_' => '36.78',
##&nbsp;                                          'total' => '1166.57',
##&nbsp;                                          'ipi' => '0.00'
##&nbsp;                                        }
##&nbsp;                                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub statit {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ( $command =~ / -b/){
    return [];
    }
    my @lines  = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );
    my @result;
    my @temp;
    my %hash;
    my ($key, $k1,$k2,$v1,$v2,$i,$j);

    for ($i = 0; $i<=$#lines; $i++)
    {
        if($lines[$i] =~ /Hostname:\s+(\S+)\s+ID:\s+(\S+)\s+Memory:\s+(.*)/) {
	    $hash{hostname} = $1;
            $hash{id} = $2;
            $hash{memory} = $3;
            $i++;
            $lines[$i] =~ /NetApp\s+Release\s+(\S+\s+[\d\w-]+):\s+(.*)/;
            $hash{netapp_release} = $1;
            $hash{date} = $2;
            $i = $i+2;
            $lines[$i] =~ /Start\s+time:\s+(.*)/;
            $hash{start_time} = $1;
        }
        if($lines[$i] =~ /CPU Statistics/) {
            $i++;
            while(1) {
                last if ($lines[$i] =~ /Multiprocessor Statistics/);
                if($lines[$i] =~ /(\d+\.\d+|\d+)+\s+(\S+\s\w+\s\w+\s\w+|\S+\s\w+\s\w+|\w+\s\w+|\w+)/) {
                    $hash{cpu_statistics}->[0]->{to_key_name(item => $2)} = $1;
                }
	        $i++;
	    }
        }
	if($lines[$i] =~ /Multiprocessor Statistics/) {
		$i = $i +1;
		my @heads;
		my @values;
		while(1) {
			last if ($lines[$i] =~ /seconds with one or more CPUs active/);
			if($lines[$i] =~ /^(cpu.*)\s+(cpu.*)\s+(.*)/)
			{
				@heads = split(/\s+/,$lines[$i]);
				$i = $i +1;
			}
			@values = split(/\s+/,$lines[$i]);
			my $k=1;
			for(my $j=0; $j<scalar(@heads);$j++){
				$hash{multiprocessor_statistics}->[0]->{to_key_name(item => $heads[$j])}->[0]->{to_key_name(item => $values[0])} = $values[$k];
				$k++;
				}
			$i++;
		}
	}
	if($lines[$i] =~ /Domain Utilization of Shared.*/) {
	    $i++;
	    $j=0;
	    while($lines[$i] =~ /([\d+\.\d+]+)\s+(\w+|\w+\s+\w+|\w+\s+\w+\s+\w+|\w+\s+\w+\s+\w+\s+\w+)\s+([\d+\.\d+]+)\s+(\w+|\w+\s+\w+|\w+\s+\w+\s+\w+|\w+\s+\w+\s+\w+\s+\w+)/) {
	        $hash{domain_utilization_of_shared_domains}->[0]->{to_key_name(item => $2)} = $1;
		$hash{domain_utilization_of_shared_domains}->[0]->{to_key_name(item => $4)} = $3;
		$i++;
            }
	}
        if($lines[$i] =~ /(Miscellaneous Statistics|WAFL Statistics|RAID Statistics)/){
            $key=$1;
	    $i++;
            while($lines[$i] =~ /(\d+\.\d+|\d+)\s(.*)\s+(\d+\.\d+|\d+)\s(.*)/){
	        $k1 = $2;
		$k2 = $4;
		$v1 = $1;
		$v2 = $3;
		$k1 =~ s/[\[\(\)\]\d+\.\%-\\]//g;
		$k2 =~ s/[\[\(\)\]\d+\.\%-\\]//g;
		$k1 =~ s/\s+$//g;
		$k2 =~ s/\s+$//g;
		$k1 =~ s/^\s+//g;
		$k2 =~ s/^\s+//g;
                $hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $k1)} = $v1;
		$hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $k2)} = $v2;
		$i++;
            }
            if($lines[$i] =~ /(\d+\.\d+|\d+)\s(1\sblocks\sper.*|buffers|iSCSI\soperations)/) {
                $hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $2)} = $1;
            }
	}
	if($lines[$i] =~ /Network Interface Statistics.*/){
            $i = $i+2;
            while(1){
	        last if ($lines[$i] =~ /Disk Statistics.*/);
	        if($lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
		    $key = $1;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[0]->{recv}->{bytes} = $3;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[0]->{recv}->{packets} = $4;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[0]->{recv}->{multicast} = $5;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[0]->{recv}->{errors} = $6;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[0]->{recv}->{packet_drops} = $7;
		    $i++;
		    $lines[$i] =~ /xmit\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[1]->{xmit}->{bytes} = $1;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[1]->{xmit}->{packets} = $2;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[1]->{xmit}->{multicast} = $3;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[1]->{xmit}->{errors} = $4;
		    $hash{network_interface_statistics}->[to_key_name(item => $j)]->{to_key_name(item => $key)}->[1]->{xmit}->{collisions} = $5;
	        }
		$i++;
		$j++;
	    }
	}
        if($lines[$i] =~ /disk\s+ut%.*/) {
	    $i++;
	    while(1) {
                last if ($lines[$i] =~ /Aggregate statistics.*/);
		if($lines[$i] =~ /(\/\w+\/\w+.*):/) {
		    $key = $1;
		    $j=0;
		} elsif ($lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)/) {
		    $hash{$key}->[to_key_name(item => $j)]->{'disk'} = $1;
		    $hash{$key}->[to_key_name(item => $j)]->{'ut%'} = $2;
		    $hash{$key}->[to_key_name(item => $j)]->{'xfers'} = $3;
		    $hash{$key}->[to_key_name(item => $j)]->{'ureads_chain_usecs'} = $4;
		    $hash{$key}->[to_key_name(item => $j)]->{'writes_chain_usecs'} = $5;
		    $hash{$key}->[to_key_name(item => $j)]->{'cpreads_chain_usecs'} = $6;
		    $hash{$key}->[to_key_name(item => $j)]->{'greads_chain_usecs'} = $7;
		    $hash{$key}->[to_key_name(item => $j)]->{'gwrites_chain_usecs'} = $8;
		    $j++;
		}
		$i++;
	    }
	}
	if($lines[$i] =~ /Aggregate statistics.*/) {
	    $i++;
	    my @keys = ( "minimum", "mean", "maximum");
	    $j=0;
	    while($lines[$i] =~ /\S+\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
	        $hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'ut%'} = $1;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'xfers'} = $2;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'ureads_chain_usecs'} = $3;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'writes_chain_usecs'} = $4;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'cpreads_chain_usecs'} = $5;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'greads_chain_usecs'} = $6;
		$hash{aggregate_statistics}->[0]->{to_key_name(item => $keys[$j])}->[0]->{'gwrites_chain_usecs'} = $7;
		$j++;
		$i++;
	    }
	}
	if($lines[$i] =~ /Spares and other.*/) {
	    $i++;
	    $j=0;
            while(1) {
	        last if ($lines[$i] =~ /FCP Statistics.*/);
		if ($lines[$i] =~ /(\S+)\s+(\d+)\s+(\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)\s+(\S+\s+\S+\s+\S+)/) {
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'disk'} = $1;
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'ut%'} = $2;
	            $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'xfers'} = $3;
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'ureads_chain_usecs'} = $4;
	            $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'writes_chain_usecs'} = $5;
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'cpreads_chain_usecs'} = $6;
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'greads_chain_usecs'} = $7;
		    $hash{"spares_and_other_disks"}->[to_key_name(item => $j)]->{'gwrites_chain_usecs'} = $8;
		    $j++;
		}
		$i++;
	    }
	}
	if($lines[$i] =~ /FCP Statistics.*/) {
	    $i++;
	    $key = "fcp_statistics";
	    $lines[$i] =~ /(\d+.\d+|\d+)\s+FCP Bytes recv\s+(\d+.\d+|\d+)\s+FCP Bytes sent/;
	    $hash{$key}->[0]->{fcp_bytes_recv} = $1;
	    $hash{$key}->[0]->{fcp_bytes_sent} = $2;
	    $i++;
	    $lines[$i] =~ /(\d+.\d+|\d+)\s+(FCP ops)/;
	    $hash{$key}->[0]->{to_key_name(item => $2)} = $1;
	}
	if($lines[$i] =~ /iSCSI Statistics.*/) {
	    $i++;
	    $key = "iscsi_statistics";
	    $lines[$i] =~ /(\d+.\d+|\d+)\s+iSCSI Bytes recv\s+(\d+.\d+|\d+)\s+iSCSI Bytes xmit/;
	    $hash{$key}->[0]->{iscsi_bytes_recv} = $1;
	    $hash{$key}->[0]->{iscsi_bytes_xmit} = $2;
	    $i++;
	    $lines[$i] =~ /(\d+.\d+|\d+)\s+(iSCSI ops)/;
	    $hash{$key}->[0]->{to_key_name(item => $2)} = $1;
	}
	if($lines[$i] =~ /Tape Statistics.*/) {
	    $i = $i+3;
	    $j=0;
	    while(1) {
	        last if ($lines[$i] =~ /Interrupt Statistics.*/);
		if($lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
		    $hash{"tape_statistics"}->[to_key_name(item => $j)]->{tape} = $1;
		    $hash{"tape_statistics"}->[to_key_name(item => $j)]->{write_bytes} = $2;
		    $hash{"tape_statistics"}->[to_key_name(item => $j)]->{blocks} = $3;
	            $hash{"tape_statistics"}->[to_key_name(item => $j)]->{read_bytes} = $4;
		    $hash{"tape_statistics"}->[to_key_name(item => $j)]->{blocks} = $5;
		    $j++;
		}
		$i++;
	    }
	}
        if($lines[$i] =~ /(Interrupt Statistics)/){
            $key=$1;
	    $i++;
            while($lines[$i] =~ /(\d+\.\d+|\d+)\s(.*)\s+(\d+\.\d+|\d+)\s(.*)/){
                $k1 = $2;
		$k2 = $4;
		$v1 = $1;
		$v2 = $3;
		$k1 =~ s/[\[\(\)\]\d+\.\%-]//g;
		$k2 =~ s/[\[\(\)\]\d+\.\%-]//g;
		$k1 =~ s/\s+$//g;
		$k2 =~ s/\s+$//g;
                $hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $k1)} = $v1;
		$hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $k2)} = $v2;
		$i++;
	    }
	    if($lines[$i] =~ /(\d+\.\d+|\d+)\s(1\sblocks\sper.*|buffers|iSCSI\soperations|nvlog shadow.*)/) {
                $hash{to_key_name(item => $key)}->[0]->{to_key_name(item => $2)} = $1;
            }
	}
	if($lines[$i] =~ /Server rpc/) {
	    $i++;
	    $j=-1;
	    while(1) {
		last if ($lines[$i] =~ /Server nfs:/);
		if($lines[$i] =~ /(TCP|UDP|IPv4|IPv6)/) {
                    $key = to_key_name(item => $1);
		    $i = $i + 2;
		    $j++;
		    $lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
	            $hash{server_rpc}->[$j]->{$key}->[0]->{calls} = $1;
	            $hash{server_rpc}->[$j]->{$key}->[0]->{badcalls} = $2;
		    $hash{server_rpc}->[$j]->{$key}->[0]->{nullrecv} = $3;
		    $hash{server_rpc}->[$j]->{$key}->[0]->{badlen} = $4;
		    $hash{server_rpc}->[$j]->{$key}->[0]->{xdrcall} = $5;
		}
		$i++;
	    }
        }
	if($lines[$i] =~ /Server nfs:/) {
	    $i = $i + 2;
	    $lines[$i] =~ /(\S+)\s+(\S+)/;
	    $hash{server_nfs}->[0]->{calls} = $1;
	    $hash{server_nfs}->[0]->{badcalls} = $2;
        }
	if($lines[$i] =~ /(Server nfs V2|NFS V2 non-blocking request statistics)/) {
	    $i = $i + 2;
            $key = to_key_name(item => $1);
	    while(1) {
	        last if ($lines[$i] =~ /Read request stats \(version 2\)/ | $lines[$i] =~/NFS V3 non-blocking request statistics:/);
	        $lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{null} = $1;
		$hash{$key}->[0]->{getattr} = $2;
		$hash{$key}->[0]->{setattr} = $3;
		$hash{$key}->[0]->{root} = $4;
		$hash{$key}->[0]->{lookup} = $5;
		$hash{$key}->[0]->{readlink} = $6;
		$hash{$key}->[0]->{read} = $7;
                $i = $i + 2;
	        $lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{wrcache} = $1;
		$hash{$key}->[0]->{write} = $2;
		$hash{$key}->[0]->{create} = $3;
		$hash{$key}->[0]->{remove} = $4;
		$hash{$key}->[0]->{rename} = $5;
		$hash{$key}->[0]->{link} = $6;
		$hash{$key}->[0]->{symlink} = $7;
                $i = $i + 2;
		$lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{mkdir} = $1;
		$hash{$key}->[0]->{rmdir} = $2;
		$hash{$key}->[0]->{readdir} = $3;
		$hash{$key}->[0]->{statfs} = $4;
                $i = $i + 1;
	    }
        }
	if($lines[$i] =~ /(Read request stats \(version 2\)|Read request stats \(version 3\))/) {
	    $i = $i + 2;
            $key = to_key_name(item => $1);
	    while(1) {
	        last if ($lines[$i] =~ /Server nfs|Misaligned Read request stats/);
		if ($lines[$i] =~ /(Write request stats \(version 2\)|Write request stats \(version 3\))/) {
                    $key = to_key_name(item => $1);
	        }
	        $lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
                $hash{$key}->[0]->{'0-511'} = $1;
	        $hash{$key}->[0]->{'512-1023'} = $2;
	        $hash{$key}->[0]->{'1k-2047'} = $3;
	        $hash{$key}->[0]->{'2k-4095'} = $4;
	        $hash{$key}->[0]->{'4k-8191'} = $5;
	        $hash{$key}->[0]->{'8k-16383'} = $6;
	        $hash{$key}->[0]->{'16k-32767'} = $7;
	        $hash{$key}->[0]->{'32k-65535'} = $8;
	        $hash{$key}->[0]->{'64k-131071'} = $9;
	        $hash{$key}->[0]->{'>_131071'} = $10;
	        $i++;
	    }
        }
	if($lines[$i] =~ /(Misaligned Read request stats)/) {
	    $i = $i + 2;
	    $j = 0;
            $key = to_key_name(item => $1);
	    while($j < 2) {
	        if($lines[$i] =~ /(Misaligned Write request stats)/) {
                    $key = to_key_name(item => $1);
		    $i = $i + 2;
		}
		$lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S)/;
		$hash{$key}->[0]->{bin_0} = $1;
		$hash{$key}->[0]->{bin_1} = $2;
		$hash{$key}->[0]->{bin_2} = $3;
		$hash{$key}->[0]->{bin_3} = $4;
		$hash{$key}->[0]->{bin_4} = $5;
		$hash{$key}->[0]->{bin_5} = $6;
		$hash{$key}->[0]->{bin_6} = $7;
		$hash{$key}->[0]->{bin_7} = $8;
		$i++;
		$j++;
	    }
            $i--;
        }
	if($lines[$i] =~ /(Server nfs V3|NFS V3 non-blocking request statistics)/) {
	    $i = $i + 2;
            $key = to_key_name(item => $1);
	    while(1) {
	        if ($lines[$i] =~ /Read request stats|TCP:/) {
	            $i--;
		    last;
	        }
		$lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{null} = $1;
		$hash{$key}->[0]->{getattr} = $2;
		$hash{$key}->[0]->{setattr} = $3;
		$hash{$key}->[0]->{lookup} = $4;
		$hash{$key}->[0]->{access} = $5;
		$hash{$key}->[0]->{readlink} = $6;
		$hash{$key}->[0]->{read} = $7;
                $i = $i + 2;
		$lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{write} = $1;
		$hash{$key}->[0]->{create} = $2;
		$hash{$key}->[0]->{mkdir} = $3;
		$hash{$key}->[0]->{symlink} = $4;
		$hash{$key}->[0]->{mknod} = $5;
		$hash{$key}->[0]->{remove} = $6;
		$hash{$key}->[0]->{rmdir} = $7;
                $i = $i + 2;
		$lines[$i] =~ /(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)\s+(\d+\s+\d+%)/;
		$hash{$key}->[0]->{rename} = $1;
		$hash{$key}->[0]->{link} = $2;
		$hash{$key}->[0]->{readdir} = $3;
		$hash{$key}->[0]->{'readdir+'} = $4;
		$hash{$key}->[0]->{fsstat} = $5;
		$hash{$key}->[0]->{fsinfo} = $6;
		$hash{$key}->[0]->{pathconf} = $7;
                $i++;
                if ($lines[$i] =~ /commit/) {
		    $i++;
		    $lines[$i] =~ /(\d+\s+\d+%)/;
                    $hash{$key}->[0]->{commit} = $1;
                }
                $i = $i + 1;
	    }
        }
	if($lines[$i] =~ /(NFS reply cache statistics)/) {
	    $i = $i + 3;
	    $j = 0;
	    $key = "tcp";
	    while($j < 2) {
	        $lines[$i] =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
		$hash{nfs_reply_cache_statistics}->[0]->{$key}->[0]->{inprog_hits} = $1;
		$hash{nfs_reply_cache_statistics}->[0]->{$key}->[0]->{misses} = $2;
		$hash{nfs_reply_cache_statistics}->[0]->{$key}->[0]->{cache_hits} = $3;
		$hash{nfs_reply_cache_statistics}->[0]->{$key}->[0]->{false_hits} = $4;
		$i++;
		if( $lines[$i] =~ /(UDP)/) {
		    $key = "udp";
		    $i = $i + 2;
		}
		$j++;
	    }
        }

    }
    push (@result, \%hash);
    return \@result;
}

1;
