#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse stats command
##
## @description
##    This is a library to parse the raw output of the stats command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "stats show" will be stats_show
##
## @synopsis
##    use stats;
##
##    stats_show(%args);
##    stats_explain_counters(%args);
##    stats_start(%args);
##    stats_stop(%args);
##    stats_list_presets(%args);
##    stats_list_objects(%args);
##    stats_list_instances(%args);
##    stats_list_counters(%agrs);
##
## @author Eric Osgood (osgood@netapp.com), dl-nacl-dev@netapp.com,govinda1@netapp.com
##
## @status public
#
################################################################################

package NACL::APISet::Node::CLI::7Mode::default::stats;

###################
# Everytime
use strict;
use warnings;

use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use Data::Dumper;
sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    my $re_hex_word    = "[0-9A-F]{8}";
    my $re_uuid        = join( ":", ($re_hex_word) x 10 );
    my $re_simple_term = "[^:]+";
    my $re_term        = "(?:$re_uuid|$re_simple_term)";

# if the output is not in this form, we are going to assume there was an error
    if ( !( $output =~ /($re_term:$re_term:$re_term):(.*)|Stats identifier name is.*|Stats Presets:|Objects:|Instances for object name:\s+(\w+)|Counters for object name:\s+\w+/ ) ) {

        # we only need to the first line of output for the error message
        $$error_message = ( split( /\n/, $output ) )[0];

        if ( $$error_message =~ /[Uu]nknown|[Nn]o\s(instance|counter)/ ) {
            $error = 'InvalidParamValue';
        } elsif($output =~ /^Unknown\/invalid object name.*/){
        $error = "CommandFailed";
        $$error_message = $output;
        } elsif($output =~ /^show: option requires an argument.*/){
            $error = "CommandFailed";
            $$error_message = $output;
        } elsif($output =~ /^[Uu]sage:/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^Specify objects\/counters with.*/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^No background stats are currently running/){
            $error = 'CommandFailed';
            $$error_message = $output;
        } elsif($output =~/^start: option requires an argument.*/){
            $error = 'InvalidParamValue';
            $$error_message = $output;
        } elsif($output =~/^show: Invalid option.*/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^No instances for object.*/){
            $error = 'CommandFailed';
            $$error_message = $output;
        } elsif($output =~/^Unable to open preset file.*: No such file or directory/){
            $error = 'InvalidParamValue';
            $$error_message = $output;
        } elsif($output =~/^Extra arguments for 'list.*'/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^Unknown subcommand:'.*'/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^stats: No such command.*/){
            $error = 'InvalidParam';
            $$error_message = $output;
        } elsif($output =~/^objects: option requires an argument.*/){
            $error = 'InvalidParamValue';
            $$error_message = $output;
        } elsif($output =~/^Only one of -p and object_name may be used./){
            $error = 'InvalidParam';
            $$error_message = $output;
        } else {
                $error = "CommandFailed";
            }
    }
    return $error;
}

## @name ::stats_show
##
## @summary Parses the raw output from the "stats show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_show with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_show(%args);
## INPUTS TO METHOD CALL 'stats_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-O' </td> <td> 'options' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'num' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rows' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interval' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'pathname' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'obj_inst_cnt' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'regex' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'columns' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delimiter' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'raw' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub stats_show {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "stats show",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = "";

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my %hash = ();

# since disk has disk id's made up of 10 8 digit hex words separated by ':'
# and a line of stat output will have 4 columns separated by ':' so, in order to uniformly
# parse this data into a name and value per the 7Mode way, it is necessary to check for the
# presence of one of these disk id's as one of the 4 columns and then parse the first three
# columns as the name (object:instance:counter) and value
# example rows    -------------------------DISK ID--------------------------------------------------------
# disk row: disk:'4E455441:50502020:56442D31:3030304D:422D465A:2D353230:31343436:33383030:00000000:00000000':disk_busy:0%
# maps to: name=>'disk:4E455441:50502020:56442D31:3030304D:422D465A:2D353230:31343436:33383030:00000000:00000000:disk_busy'
#          value=>'0%'
# regular row: fcp:fcp:instance_name:fcp
# maps to: name=>'fcp:fcp:instance_name'
#          value=>'fcp'
        my $re_hex_word    = "[0-9A-F]{8}";
        my $re_uuid        = join( ":", ($re_hex_word) x 10 );
        my $re_simple_term = "[^:]+";

        # one of the 4 columns is either a word without a ':' or a disk-id
        my $re_term = "(?:$re_uuid|$re_simple_term)";

        # Parse only lines that are of the object:instance:counter form
        # Ignore all other lines
        if ( $line =~ /($re_term:$re_term:$re_term):(.*)/ ) {
            my ( $name, $value )
                = ( $line =~ /($re_term:$re_term:$re_term):(.*)/ );

            $hash{value} = $value;
            $hash{name}  = $name;
            push( @result_array, \%hash );
        }
    }
    return \@result_array;
}

## @name ::stats_explain_counters
##
## @summary Parses the raw output from the "stats explain counters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_explain_counters with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_explain_counters(%args);
## INPUTS TO METHOD CALL 'stats_explain_counters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'object_name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'counter_name' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; stats explain counters volume ; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'counter_info' => [
##&nbsp;                                {
##&nbsp;                                  'unit' => 'none',
##&nbsp;                                  'description' => 'Source has SPEC vbns',
##&nbsp;                                  'counter_name' => 'clone_inline_split_source_spec_vbn',
##&nbsp;                                  'properties' => 'raw'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'unit' => 'none',
##&nbsp;                                  'description' => 'Not enough space on the vol',
##&nbsp;                                  'counter_name' => 'clone_inline_split_enospc',
##&nbsp;                                  'properties' => 'raw'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'unit' => 'none',
##&nbsp;                                  'description' => 'Not enogh quota for the user',
##&nbsp;                                  'counter_name' => 'clone_inline_split_edquot',
##&nbsp;                                  'properties' => 'raw'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'unit' => 'none',
##&nbsp;                                  'description' => 'Number of times blks_vvol_dbys_df_cnt counter being not zero, at the end persistent delayed free merge scores.',
##&nbsp;                                  'counter_name' => 'nonzero_dbys_cnt',
##&nbsp;                                  'properties' => 'raw'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;            'object_name' => 'volume'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; stats explain counters volume write_ops; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'counter_info' => [
##&nbsp;                                {
##&nbsp;                                  'unit' => 'per_sec',
##&nbsp;                                  'description' => 'Number of writes per second to the volume',
##&nbsp;                                  'counter_name' => 'write_ops',
##&nbsp;                                  'properties' => 'rate'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'object_name' => 'volume'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stats_explain_counters {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => "stats explain counters",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my (%hash,%hash1,@counter,@final);
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp
    );
    my $i = -1;
    my $j;
    foreach my $line (@lines) {
        if ($line =~ /Counters for object name: (\w+)/){
            $i++;
            $j = -1;
            $final[$i]->{object_name} = $1;
        } elsif ($line =~ /Name:\s+(.*)/){
            $j++;
            $final[$i]->{counter_info}->[$j]->{counter_name} = $1;
        } elsif ($line =~ /Description:\s+(.*)/){
            $final[$i]->{counter_info}->[$j]->{description} = $1;
        } elsif ($line =~ /Properties:\s+(.*)/){
            $final[$i]->{counter_info}->[$j]->{properties} = $1;
        } elsif ($line =~ /Unit:\s+(.*)/){
            $final[$i]->{counter_info}->[$j]->{unit} = $1;
        }
    }
    return \@final;
 }

## @name ::stats_list_presets
##
## @summary Parses the raw output from the "stats list presets" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_list_presets with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_list_presets(%args);
## INPUTS TO METHOD CALL 'stats_list_presets' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =  stats list presets;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'stats_presets' => [
##&nbsp;                                 'flexscale-access',
##&nbsp;                                 'flexscale-pcs',
##&nbsp;                                 'flexscale',
##&nbsp;                                 'spcres_aggr',
##&nbsp;                                 'spcres_vol'
##&nbsp;                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stats_list_presets {
    my %args = @_;
    my ( $error_message, @result_array, $name, $value,$i,$j);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "stats list presets",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    $i = 0;
    $j = 0;
    foreach my $line (@lines) {
        if ( $line =~ /^(Stats Presets):/ ) {
            $name = $1;
            next;
        } elsif ($line =~/^([\w+\-\_]+)/){
            $value = $1;
            $result_array[$i]-> {to_key_name(item=>$name)}->[$j++] = $value;
        }
    }
    return \@result_array;

}
## @name ::stats_list_objects
##
## @summary Parses the raw output from the "stats list objects " command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_list_objects with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_list_objects(%args);
## INPUTS TO METHOD CALL 'stats_list_objects' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =  stats list objects  -p flexscale-pcs;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'objects' => [
##&nbsp;                           'ext_cache_obj'
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stats_list_objects {
    my %args = @_;
    my ( $error_message, @result_array, $name, $value,$i,$j);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    $i = 0;

    foreach my $line (@lines) {
        if ( $line =~ /(?:No .*)/) {
            next;
        } elsif ( $line =~ /^(Objects):/ ) {
            $name = $1;
            $j = -1;
            next;
        } elsif ($line =~/^([\w+\-\_]+)/){
            $j++;
            $value = $1;
            $result_array[$i]-> {to_key_name(item=>$name)}->[$j] = $value;
        }
    }
    return \@result_array;

}

## @name ::stats_list_instances
##
## @summary Parses the raw output from the "stats list instances" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_list_instances with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_list_instances(%args);
## INPUTS TO METHOD CALL 'stats_list_instances' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'object_name' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =   stats list instances -p spcres_aggr;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;
##&nbsp;             'instance_info' => [
##&nbsp;                                 'aggr0',
##&nbsp;                                 'component_flex_aggr'
##&nbsp;                                ],
##&nbsp;             'instances_for_object_name' => 'aggregate'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stats_list_instances {
    my %args = @_;
    my ( $error_message, @result_array, $name, $value,$i,$j);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    $i = -1;

    foreach my $line (@lines) {
        if ( $line =~ /(?:No .*)/) {
            next;
        } elsif ( $line =~ /^(Instances for object name):\s+(\w+)/ ) {
            $name = $1;
            $value = $2;
            $i++;
            $j = -1;
            $result_array[$i]-> {to_key_name(item=>$name)} = $value;
         } else{
            $j++;
            $i++ if $i<0;
            $result_array[$i]-> {"instance_info"}->[$j] = $line;
        }
    }
    return \@result_array;

}

## @name ::stats_list_counters
##
## @summary Parses the raw output from the "stats list counters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_list_counters with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_list_counters(%args);
## INPUTS TO METHOD CALL 'stats_list_counters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'object_name' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =   stats list counters -p spcres_vol;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'counters_for_object_name' => 'volume',
##&nbsp;             'counter_info' => [
##&nbsp;                                 'wv_fsid',
##&nbsp;                                 'wv_vol_type',
##&nbsp;                                 'wv_fsinfo_fs_version',
##&nbsp;                                 'wv_volinfo_fs_options',
##&nbsp;                                 'wv_volinfo_fs_flags',
##&nbsp;                                 'wv_fsinfo_blks_total',
##&nbsp;                                 'wv_fsinfo_blks_reserve',
##&nbsp;                                 'wv_fsinfo_blks_used',
##&nbsp;                                 'wv_fsinfo_blks_used_by_plane0',
##&nbsp;                                 'wv_fsinfo_blks_blks_rsrv_holes_cifs',
##&nbsp;                                 'wv_fsinfo_blks_blks_rsrv_holes',
##&nbsp;                                 'wv_fsinfo_blks_blks_rsrv_overwrite',
##&nbsp;                                 'wv_fsinfo_blks_rsrv_absents',
##&nbsp;                                 'wv_fsinfo_blks_snap_reserve_pct',
##&nbsp;                                 'wv_fsinfo_blks_res_state',
##&nbsp;                                 'wv_fsinfo_blks_overwrite_slider_pct',
##&nbsp;                                 'wvblk_past_eof',
##&nbsp;                                 'wvblk_snap_reserve',
##&nbsp;                                 'wvblk_zombie_blks',
##&nbsp;                                 'wv_fsinfo_blks_rsrv_parent',
##&nbsp;                                 'wvblk_spcres_in_parent',
##&nbsp;                                 'wvblk_rsrv_parent_holes',
##&nbsp;                                 'wvblk_rsrv_parent_overwrite',
##&nbsp;                                 'wvblk_rsrv_parent_overwrite_always',
##&nbsp;                                 'wvbd_active_frees',
##&nbsp;                                 'wvbd_whole_frees',
##&nbsp;                                 'wvbd_active_frees_y',
##&nbsp;                                 'wvbd_owner_changed_y',
##&nbsp;                                 'wvbd_whole_frees_o',
##&nbsp;                                 'wvblk_delalloc',
##&nbsp;                                 'wvblk_rsrv_delalloc',
##&nbsp;                                 'wvblk_rsrv_holes_cifs',
##&nbsp;                                 'wvblk_rsrv_holes',
##&nbsp;                                 'wvblk_rsrv_overwrite',
##&nbsp;                                 'wvblk_rsrv_absents',
##&nbsp;                                 'extent_size',
##&nbsp;                                 'wvblk_parent_to_be_reclaimed'
##&nbsp;                                ]
 ##&nbsp;          }
 ##&nbsp;        ];
##&nbsp;

sub stats_list_counters {
    my %args = @_;
    my ( $error_message, @result_array, $name, $value,$i,$j);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    $i = -1;

    foreach my $line (@lines) {
        if ( $line =~ /(?:No .*)/) {
            next;
        } elsif ( $line =~ /^([\w+\s+]+):\s+(\w+)/ ) {
            $name = $1;
            $value = $2;
            $i++;
            $j = -1;
            $result_array[$i]-> {to_key_name(item=>$name)} = $value;
            next;
        } elsif ($line =~/^([\w+\-\/\.\ \:\]]+)/){
            $j++;
            $value = $1;
            $result_array[$i]-> {"counter_info"}->[$j] = $value;
        }
    }
    return \@result_array;

}


 ## @name ::stats_start
##
## @summary Parses the raw output from the "stats start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_start with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_start(%args);
## INPUTS TO METHOD CALL 'stats_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'object_name' </td> </tr>
## <tr> <td> '-I' </td> <td> 'id' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; stats start; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stats_identifier_name' => 'Ind0xcf239b88'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub stats_start {
    my %args = @_;
    my %hash = ();
    my ( $error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "stats start",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output.
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    foreach my $line (@lines) {
        if ( $line =~ /(?:No .*)/) {
            next;
        }elsif ( $line =~ /^Stats identifier name is '.*'/ ) {
            my ( $name, $value )
            = ( $line =~ /^(Stats identifier name) is '(.*)'/ );
            $hash{to_key_name(item=>$name)} = $value;
            push( @result_array, \%hash );
        }
    }
    return \@result_array;
}



 ## @name ::stats_stop
##
## @summary Parses the raw output from the "stats stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from stats_stop with any of its options as described in the
##    corresponding cdef file 'stats.cdefs'
## @synopsis
##      stats_stop(%args);
## INPUTS TO METHOD CALL 'stats_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-O' </td> <td> 'options' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'obj_inst_cnt' </td> </tr>
## <tr> <td> '-c' </td> <td> 'columns' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delimiter' </td> </tr>
## <tr> <td> '-I' </td> <td> 'id' </td> </tr>
## <tr> <td> '-o' </td> <td> 'pathname' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rows' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preset' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; stats stop; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statisticsid' => 'ind0xcf239b88'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'blkr_redirect_blocks_invalid' => '0',
##&nbsp;            'delete_log.purge_blk_cnt' => '0',
##&nbsp;            'wv_fsinfo_fs_version' => '0',
##&nbsp;            'blkr_async_no_msg' => '0',
##&nbsp;            'wv_fsinfo_blks_blks_rsrv_holes_cifs' => '0',
##&nbsp;            'wv_fsinfo_blks_used_by_plane0' => '0',
##&nbsp;            'delete_log.overflow_send' => '0',
##&nbsp;            'wvblk_rsrv_holes64' => '0',
##&nbsp;            'blkr_blocks_postfiltered.buftreeid_undefined' => '0',
##&nbsp;            'user_write_blocks' => '0/s',
##&nbsp;            'delete_log.purge_send' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.aa_target_score' => '0',
##&nbsp;            'delete_log.dlog_overflow' => '0',
##&nbsp;            'wv_fsinfo_blkr_cp' => '0',
##&nbsp;            'blkr_blocks_redirected' => '0',
##&nbsp;            'wvblk_saved_private_fsinfo_inos_used' => '0',
##&nbsp;            'delete_log.overflow_commits' => '0',
##&nbsp;            'delete_log.overflow_throttle' => '0',
##&nbsp;            'delete_log.purge_vbns' => '0',
##&nbsp;            'blkr_csc_buf_suspended' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.aa_target' => '0',
##&nbsp;            'wv_fsinfo_containment_version_spare2' => '0',
##&nbsp;            'delete_log.dlog_hole_cnt' => '0',
##&nbsp;            'wv_fsinfo_blks64_blks_rsrv_holes_cifs' => '0',
##&nbsp;            'wvbd_owner_changed_y' => '0',
##&nbsp;            'wv_fsinfo_blks_res_state' => '0',
##&nbsp;            'delete_log.purge_ind_blk_cnt' => '0',
##&nbsp;            'delete_log.trunc_overflow_of_append' => '0',
##&nbsp;            'wv_fsinfo_containment_version_compression' => '0',
##&nbsp;            'blkr_redirect_blocks_updated' => '0',
##&nbsp;            'user_writes' => '0/s',
##&nbsp;            'wv_fsinfo_blks_reserve' => '0',
##&nbsp;            'wvblk_child_pledge_percent' => '0',
##&nbsp;            'wvbd_active_frees' => '0',
##&nbsp;            'wvblk_saved_public_fsinfo_inos_total' => '0',
##&nbsp;            'delete_log.overflow_frees' => '0',
##&nbsp;            'blkr_blocks_redirected_noio' => '0',
##&nbsp;            'wv_fsinfo_private_inos_used' => '0',
##&nbsp;            'wv_fsinfo_blks_overwrite_slider_pct' => '0%',
##&nbsp;            'wvip_private_inofile_wi_blk_cnt' => '0',
##&nbsp;            'blkr_redirect_kireetis_scanned' => '0',
##&nbsp;            'blkr_async_completed' => '0',
##&nbsp;            'blkr_redirect_indirects_inspected.demand' => '0',
##&nbsp;            'blkr_blocks_read' => '0',
##&nbsp;            'wvbd_whole_frees' => '0',
##&nbsp;            'delete_log.pre_split_failed' => '0',
##&nbsp;            'wvblk_saved_public_fsinfo_inos_reserve' => '0',
##&nbsp;            'wv_volinfo_fs_flags' => '',
##&nbsp;            'instance_name' => 'component_flex_aggr',
##&nbsp;            'wv_volinfo_fs_options' => '',
##&nbsp;            'blkr_blocks_redirected_maybe' => '0',
##&nbsp;            'blkr_csc_aa_requested' => '0',
##&nbsp;            'delete_log.purge_overflow' => '0',
##&nbsp;            'delete_log.dlog_keys' => '0',
##&nbsp;            'blkr_blocks_postfiltered.readonly' => '0',
##&nbsp;            'delete_log.dlog_vbn_clears' => '0',
##&nbsp;            'blkr_blocks_postfiltered.buftreeid_nullid' => '0',
##&nbsp;            'wv_fsinfo_containment_version_sle' => '0',
##&nbsp;            'delete_log.overflow1_blk_cnt' => '0',
##&nbsp;            'delete_log.overflow0_blk_cnt' => '0',
##&nbsp;            'blkr_blocks_redirected_noverify' => '0',
##&nbsp;            'blkr_blocks_overwritten' => '0',
##&nbsp;            'instance_uuid' => '',
##&nbsp;            'wvblk_past_eof' => '0',
##&nbsp;            'blkr_blocks_postfiltered.buftreeid_superblk' => '0',
##&nbsp;            'blkr_redirect_blocks_ok' => '0',
##&nbsp;            'blkr_redirect_indirects_updated.scanned' => '0',
##&nbsp;            'wvblk_snap_reserve' => '0',
##&nbsp;            'wvblk_zombie_blks' => '0',
##&nbsp;            'wv_fsinfo_containment_version_highest_spare2' => '0',
##&nbsp;            'wvblk_rsrv_child_overwrite' => '0',
##&nbsp;            'blkr_super_blocks_scanned' => '0',
##&nbsp;            'wv_vol_type' => '',
##&nbsp;            'wv_fsinfo_public_inos_total' => '0',
##&nbsp;            'wv_fsinfo_blks_blks_rsrv_holes' => '0',
##&nbsp;            'blkr_redirect_indirects_ok.scanned' => '0',
##&nbsp;            'blkr_redirect_demand_drop' => '0',
##&nbsp;            'wvblk_child_indirect_blk_cnt' => '0',
##&nbsp;            'blkr_blocks_postfiltered.superblock' => '0',
##&nbsp;            'blkr_blocks_postfiltered.buftreeid_collide' => '0',
##&nbsp;            'wvblk_rsrv_delalloc' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.score' => '0',
##&nbsp;            'wv_fsinfo_public_inos_used' => '0',
##&nbsp;            'blkr_redirect_indirects_ok.demand' => '0',
##&nbsp;            'wvbd_active_frees_y' => '0',
##&nbsp;            'aggregate' => 'component_flex_aggr',
##&nbsp;            'wv_fsinfo_containment_version_spare1' => '0',
##&nbsp;            'delete_log.overflow_in_use' => '0',
##&nbsp;            'cp_read_blocks' => '0/s',
##&nbsp;            'wvblk_saved_private_fsinfo_inos_reserve' => '0',
##&nbsp;            'delete_log.pre_bufs_loaded' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.aa' => '0',
##&nbsp;            'wvblk_rsrv_child_holes' => '0',
##&nbsp;            'wv_fsinfo_blks_total' => '0',
##&nbsp;            'wv_fsinfo_containment_version_highest_compression' => '0',
##&nbsp;            'wvblk_rsrv_overwrite' => '0',
##&nbsp;            'delete_log.overflow_frees_ind' => '0',
##&nbsp;            'blkr_async_launched' => '0',
##&nbsp;            'wvblk_rsrv_holes' => '0',
##&nbsp;            'blkr_blocks_postfiltered.fbn_too_big' => '0',
##&nbsp;            'wvbd_whole_frees_o' => '0',
##&nbsp;            'blkr_redirect_susps' => '0',
##&nbsp;            'wv_fsinfo_blks_used' => '0',
##&nbsp;            'delete_log.dlog_cap' => '0',
##&nbsp;            'wv_fsid' => '',
##&nbsp;            'wvblk_delalloc' => '0',
##&nbsp;            'delete_log.del_log' => '0',
##&nbsp;            'parent_host' => 'csiqa-2040-16b',
##&nbsp;            'delete_log.overflow_vbns' => '0',
##&nbsp;            'blkr_blocks_postfiltered.invalid' => '0',
##&nbsp;            'blkr_redirect_indirects_inspected.scanned' => '0',
##&nbsp;            'delete_log.pre_overflow' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.target' => '0',
##&nbsp;            'blkr_blocks_postfiltered.notflexvol' => '0',
##&nbsp;            'wvblk_ind_delalloc' => '0',
##&nbsp;            'delete_log.dlog_blk_cnt' => '0',
##&nbsp;            'delete_log.purge_block' => '0',
##&nbsp;            'total_transfers' => '0/s',
##&nbsp;            'blkr_blocks_redirected_nol1' => '0',
##&nbsp;            'blkr_csc_aa_miss' => '0',
##&nbsp;            'blkr_redirect_ra_l1' => '0',
##&nbsp;            'delete_log.purge_overflow_susp' => '0',
##&nbsp;            'blkr_redirect_ra_map' => '0',
##&nbsp;            'wvblk_rsrv_holes_cifs64' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.aa_score' => '0',
##&nbsp;            'wvip_public_inofile_wi_blk_cnt' => '0',
##&nbsp;            'blkr_free_blocks_scanned' => '0',
##&nbsp;            'blkr_blocks_dummy_read' => '0',
##&nbsp;            'wv_fsinfo_containment_version_slc' => '0',
##&nbsp;            'delete_log.dlog_frees' => '0',
##&nbsp;            'wvblk_past_eof64' => '0',
##&nbsp;            'blkr_reads_launched' => '0',
##&nbsp;            'blkr_rejected_segments_before_scan' => '0',
##&nbsp;            'blkr_empty_segments_scanned' => '0',
##&nbsp;            'blkr_csc_aa_used' => '0',
##&nbsp;            'blkr_blocks_reallocated' => '0',
##&nbsp;            'blkr_blocks_postfiltered.pvbn_mismatched' => '0',
##&nbsp;            'delete_log.act_dirty' => '0',
##&nbsp;            'user_reads' => '0/s',
##&nbsp;            'wvdf_max_frees' => '0',
##&nbsp;            'wvzmb_num_zmsgs_inuse' => '0',
##&nbsp;            'wv_fsinfo_blks_blks_rsrv_overwrite' => '0',
##&nbsp;            'blkr_redirect_demand_rereq' => '0',
##&nbsp;            'blkr_aggrsnap_blocks_scanned' => '0',
##&nbsp;            'delete_log.overflow_susp' => '0',
##&nbsp;            'wv_fsinfo_blks_rsrv_absents' => '0',
##&nbsp;            'wv_fsinfo_containment_version_highest_spare1' => '0',
##&nbsp;            'blkr_aa_blocks_scanned' => '0',
##&nbsp;            'blkr_blocks_postfiltered.aggr_unavailable' => '0',
##&nbsp;            'blkr_async_no_mem' => '0',
##&nbsp;            'blkr_blocks_scanned' => '0',
##&nbsp;            'wvblk_rsrv_holes_cifs' => '0',
##&nbsp;            'wv_fsinfo_public_inos_reserve' => '0',
##&nbsp;            'node_name' => '',
##&nbsp;            'wvblk_child_to_be_reclaimed' => '0',
##&nbsp;            'blkr_blocks_redirected_reread' => '0',
##&nbsp;            'cp_reads' => '0/s',
##&nbsp;            'delete_log.dlog_ind_blk_cnt' => '0',
##&nbsp;            'wv_fsinfo_blks64_blks_rsrv_holes' => '0',
##&nbsp;            'delete_log.dlog_overuse_mem' => '0',
##&nbsp;            'wvblk_saved_private_fsinfo_inos_total' => '0',
##&nbsp;            'delete_log.purge_overflow_no_split' => '0',
##&nbsp;            'blkr_rejected_segments_scanned' => '0',
##&nbsp;            'delete_log.dlog_in_use' => '0',
##&nbsp;            'wvblk_saved_public_fsinfo_inos_used' => '0',
##&nbsp;            'wvblk_lev0_over_nominal' => '0',
##&nbsp;            'wvblk_space_tax' => '0',
##&nbsp;            'wv_fsinfo_containment_version_highest_slc' => '0',
##&nbsp;            'disk_type' => '0',
##&nbsp;            'blkr_csc_aa_inventory' => '0',
##&nbsp;            'user_read_blocks' => '0/s',
##&nbsp;            'blkr_blocks_postfiltered.no_vvi' => '0',
##&nbsp;            'blkr_async_offline' => '0',
##&nbsp;            'wv_fsinfo_containment_version_highest_sle' => '0',
##&nbsp;            'blkr_blocks_redirected_noread' => '0',
##&nbsp;            'wvblk_child_delalloc' => '0',
##&nbsp;            'wvblk_rsrv_child_overwrite_always' => '0',
##&nbsp;            'wv_fsinfo_blks_snap_reserve_pct' => '0%',
##&nbsp;            'blkr_csc_aa_cleaned' => '0',
##&nbsp;            'blkr_segments_scanned' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.target_score' => '0',
##&nbsp;            'wv_fsinfo_private_inos_reserve' => '0',
##&nbsp;            'blkr_redirect_indirects_updated.demand' => '0',
##&nbsp;            'wv_fsinfo_private_inos_total' => '0',
##&nbsp;            'blkr_rejected_blocks_scanned' => '0',
##&nbsp;            'blkr_policy1_reject_reasons.full' => '0',
##&nbsp;            'wvblk_child_rsrv_delalloc' => '0',
##&nbsp;            'blkr_full_segments_scanned' => '0',
##&nbsp;            'blkr_redirect_demand_req' => '0',
##&nbsp;            'delete_log.overflow_total' => '0',
##&nbsp;            'blkr_redirect_ra_l0' => '0'
##&nbsp;          },
##&nbsp;          .........................
##&nbsp;          .........................
##&nbsp;          .........................
##&nbsp;          .........................
##&nbsp;        ];
##&nbsp;
sub stats_stop {
    my %args = @_;
    my ( $error_message, @result_array,$i,$obj_key );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "stats stop",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $first_line = "";
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    $i = 0;
    $obj_key = "";
    foreach my $line (@lines) {
        if ( $line =~ /^(StatisticsID):\s+(.*)$/) {
            $result_array[$i]->{to_key_name(item=>$1)} = to_key_name(item=>$2);
            next;
        } elsif (( $line =~/^(\w+):([\w\.\-\/]+):([\w\.\-\<\d\s\>=%]+):([\d\-\w\/\%\.]+|)$/)
                ||( $line =~/^(disk):([A-Za-z\d\:]+):([\w\-\>\<\.\s]+):(.*)$/)
                ||( $line =~/^(wafltop):(\w+::?[\w\s\/]+:?[\w\s\/]+)::([\w+\.]+):([\:\s\w\%\/]+?)$/)) {
                    $i++ if($obj_key ne $2);
                    $result_array[$i]->{$1} = $2;
                    $result_array[$i]->{$3} = $4;
                    $obj_key = $2;
        }
    }
    return \@result_array;
}
1;
