#
# Copyright (c) 2014-2016 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode sysconfig commands
##
## @description
##     This is a library to parse the raw output of 7Mode sysconfig commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use sysconfig;
##
##    sysconfig(%args);
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::sysconfig;

use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use Data::Dumper;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Node::CLI::7Mode::default::aggr;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _parse_for_errors(%) {
    my %args         = @_;
    my $error;
    my $raw_output   = $args{'raw_output'};
    my $resp       = $args{response_object};
    if ($raw_output =~/usage:.*/si) {
       $error       = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
}

sub _slot_parser (%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_object'};
    my $line;
    my @return_array;
    my @config_status;
    my @devices;
    my @config_errors;

    # The following lines comes if system is in takeover mode.
    # Get rid of unwanted lines
    @lines = grep (!(/taken\s+over\s+by\s+\S+|No\s+hardware\s+device\s+information\s+is\s+available/i), @lines);
    while ( $line = shift(@lines) ) {
        if ( $line =~ /slot\s+(\d+|\d+[-]\d+):\s*(.*)/ ) {
            my %temp_hash;
            my $temp_hash;
            my $ky;
            $temp_hash{'slot_number'} = $1;
            $temp_hash{'device'} = $2;
            $line                     = shift(@lines);
            my @arr;
            my @io_information;
            my @disk_info;
            while ( $line !~ /slot\s+(\d+|\d+[-]\d+):/ ) {
                if ( $line =~ /(\S+)\s+\w+\s+\w+:\s+(\S+)\s+\((\S+)\)/ ) {
                    my %hash;
                    $hash{port_number} = $1;
                    $hash{mac_address} = $2;
                    $hash{status} = $3;
                    push(@arr,\%hash);
                } elsif ($line =~ /(.*I\/O base)\s+(\S+),\ssize\s(\S+)/ ||
                    $line =~ /(prefetchable\s+memory\s+base)\s+(\S+),\s+size\s+(\S+)/i) {
                    my %io_information;
                    my $key = to_key_name('item' => $1);
                    my $io_base = $2;
                    my $size = $3;
                    $key =~ s/\///;
                    $io_information{$key} = $io_base;
                    $io_information{'size'} = $size;
                    push (@io_information, \%io_information);
                } elsif (($line =~ /^(\d+)\s+:\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)?\s+\S+/ ) ||
			        ($line =~ /^\d+\.(\d+):\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)?\s+\S+/) ||
			        ($line =~ /^\d+\.(\d+)\s+:\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)?\s+\S+/)) {
                    my %slot_disk_info;
                    $slot_disk_info{id} = $1;
                    $slot_disk_info{vendor} = $2;
                    $slot_disk_info{model} = $3;
                    $slot_disk_info{firmware_version} = $4;
                    $slot_disk_info{size} = $5;
                    # Bytes per sector is seen for sysconfig -a command only.
                    # To prevent wrong data from appearing in the parsed output
                    # the following check is made.
                    $slot_disk_info{'bytes_per_sector'} = $6 if $6 !~ /^\(/;
                    push(@disk_info,\%slot_disk_info);
                } elsif ($line =~ /Interconnect\s+Port\s+\d+:/i) {
                    my %temp_hash;
                    $line = shift(@lines);
                    while ( $line =~ /^([\w\s]+):\s+(.+)/ ) {
                        my $key = to_key_name('item' => $1);
                        $temp_hash{$key} = $2;
                        last if ( $lines[0] !~ /^[\w\s]+:\s+(\S+)/ || $lines[0] =~ /^slot\s+\d+(:|[-]\d+:)/i);
                        $line = shift(@lines);
                    }
                    push(@arr,\%temp_hash);
                } else {
                    my ( $key, $value ) = split( /:/, $line, 2 ) if ($line =~ /:/);
                    $key = to_key_name( 'item' => $key );
                    trim( 'string_ptr' => \$value );
                    $key =~ s/^\_//;
                    $key =  "ioxm_" . $key if  (exists $temp_hash{$key} && $key =~ /^serial_number|^part_number/)  ;
                    $key =  "local_ip_" . $key if  (exists $temp_hash{$key} && $key =~ /prefix_length/)  ;
                    my $val = $line if ($line !~ /:/);
                    trim( 'string_ptr' => \$line ) if ($line !~ /:/);
                    if ( $line =~ /:/) {
                        $temp_hash{$key} = $value if ($key);
                        $ky = $key if $key;
                    } else {
                        if (ref($temp_hash{$ky}) ne 'ARRAY') {
                            trim( 'string_ptr' => \$line ) if ($line !~ /:/);
                            $temp_hash{'device'}= [$temp_hash{'device'}, $val] if ($val =~ /boot0/ );
                            if($val =~ /^Boot Media/)
                            {
                                my @value= split( / /, $val, 4);
                                $temp_hash{'boot_media'}=$value[3];
                            }
                            $temp_hash{$ky} = [$temp_hash{$ky}, $val] if (defined $val && defined $ky && ($val !~ /ID\s+Vendor\s+Model\s+FW\s+Size$/i));
                        } else {
                            push @{$temp_hash{$ky}}, $val if (defined $ky && defined $val && ($val !~ /ID\s+Vendor\s+Model\s+FW\s+Size$/i));
                        }
                    }
                }
                last unless ( scalar(@lines) );
                $line = shift(@lines);
            }

            $temp_hash{'io_information'} = \@io_information if @io_information;
            $temp_hash{'disk_information'} = \@disk_info if @disk_info;
            $temp_hash{ports} = \@arr;
            unshift( @lines, $line ) if ( scalar(@lines) );
			push( @return_array, \%temp_hash );
	    } elsif($line =~ /^(\S+:)+\s(\S+)\s+(\S+)\s+(\S+)\s+(\S+)$/){
		my @variables;
		my %slot_disk_info;
		push (@variables,$1,$2,$3,$4,$5);
		$variables[0] =~ s/.$//;
		$slot_disk_info{'bridge'}   = $variables[0];
		$slot_disk_info{'vendor'}   = $variables[1];
		$slot_disk_info{'model'}    = $variables[2];
		$slot_disk_info{'rev'}      = $variables[3];
		$slot_disk_info{'serial'}   = $variables[4];
		@variables = undef;
		push(@return_array,\%slot_disk_info);
	   }elsif (($line =~ /There are no configuration errors/)||($line =~ /System configuration meets block protocol requirements/)||($line =~ /No shelf configuration errors detected/)){
            push( @config_status, $line);
	} elsif ( $line =~ /slot\s+(\d+)\s+(\w+):\s?(.*):\s?(.*)/ ){
		    my %temp_hash;
            $temp_hash{'slot_number'} = $1;
            $temp_hash{'status'}      = $2;
            $temp_hash{'device_part_number'} = $3;
            $temp_hash{'device'} = $4;
            push( @devices, \%temp_hash );
        } else {
            push( @config_errors, $line);
        }
    }
    my %hash;
    if (@config_errors) {
        $hash{'configuration_errors'} = \@config_errors;
    }
    if (@config_status) {
        $hash{'configuration_status'} = \@config_status;
    }
    if(@devices) {
        $hash{'devices'} = \@devices;
    }
    if (@config_errors||@devices||@config_status){
        push(@return_array, \%hash);
    }
    return ( \@return_array );
}

sub _disk_parser (%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_object'};
    my @return_array;

    my $line = shift(@lines);
    my @headings = split( /\s+/, $line, 6 );

    #skip the --- line
    $line = shift(@lines);
    while ( $line = shift(@lines) ) {
        my %temp_hash;
        my @values = split( /\s+/, $line );
        for ( my $i = 0; $i < scalar(@headings); $i++ ) {
            $temp_hash{ $headings[$i] } = $values[$i];
        }
        push( @return_array, \%temp_hash );
    }
    return \@return_array;
}

sub _tape_parser(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_object'};
    my $line;
    my @return_array;

    if (   ( ( scalar(@lines) == 1) || ( scalar(@lines) == 2) )
        && ( $lines[0] =~ /No\s+tape\s+drives\s+present\s+on\s+system./ ) )
    {
        return \@return_array;
    } else {
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /Tape\s+drive\s+\((.*)\.(.*)\)\s+(.*)/ ) {
                my $hash_ptr = {};
                $hash_ptr->{'slot'}        = $1;
                $hash_ptr->{'scsi_id'}     = $2;
                $hash_ptr->{'name'}        = $3;
                trim( 'string_ptr' => \$hash_ptr->{'name'} );
                if ($hash_ptr->{'name'} =~ /\(Non-qualified\s+tape\s+drive\)/) {
                    $hash_ptr->{'name'} =~ s/^(.*)\s+\(.*\)/$1/;
                    trim( 'string_ptr' => \$hash_ptr->{'name'} );
                    $hash_ptr->{'non_qualified'} = "true";
                    push( @return_array, $hash_ptr );
                    next;
                } else {
                    $hash_ptr->{'device_info'} = [];
                }
                $line                      = shift(@lines);
                while ( $line !~ /Tape\s+drive\s\(.*\..*\)\s+.*/ and scalar(@lines)) {
                    my $temp_hash_ptr = {};
                    if ( $line
                        =~ /(.*)\s+-\s+(.*)\s+device,\s+format\s+is:(.*)/ )
                    {
                        $temp_hash_ptr->{'name'}        = $1;
                        trim( 'string_ptr' => \$temp_hash_ptr->{'name'} );
                        $temp_hash_ptr->{'device_type'} = $2;
                        trim( 'string_ptr' => \$temp_hash_ptr->{'device_type'} );
                        $temp_hash_ptr->{'format'}      = $3;
                        trim( 'string_ptr' => \$temp_hash_ptr->{'format'} );
                    } else {
                        my $error
                            = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                        $error->throw(
                            text            => "Unexpected form of output",
                            response_object => $resp
                        );
                    }
                    push( @{ $hash_ptr->{'device_info'} }, $temp_hash_ptr );
                    last unless ( scalar(@lines) );
                    $line = shift(@lines);
                }
                unshift( @lines, $line ) if ( scalar(@lines) );
                push( @return_array, $hash_ptr );
            } else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "Unexpected form of output",
                    response_object => $resp
                );
            }
        }
        return \@return_array;
    }
}

sub _sysconfig(%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $can_throw  = 0;
    my $option     = $command;
    my @lines      = get_lines(
            'raw_output' => $raw_output,
            'can_throw'  => $can_throw
        );

    $option =~ s/sysconfig\s?//;
    if ($lines[0] =~ /^slot \d+(:|[-]\d+:)/) {
        return (
            _slot_parser( 'lines' => \@lines, 'response_object' => $resp ) );
    } elsif ( $option =~ /-t/ ) {
        my @return_array;
        my $first_line = '(Tape\s+drive|No)';
        my @lines      = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return (
            _tape_parser( 'lines' => \@lines, 'response_object' => $resp ) );
    } elsif ( $option =~ /-r/ ) {
        return (
            NACL::APISet::Node::CLI::7Mode::default::aggr::aggr_status(
                "raw_output"      => $raw_output,
                "command"         => $command,
                "response_object" => $resp
            )
        );
    } elsif ( $option =~ /-d/ ) {
        my @return_array;
        my $first_line = 'Device';
        my @lines      = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return (
            _disk_parser( 'lines' => \@lines, 'response_object' => $resp ) );
    } elsif ( $option =~ /-c/){
	    my @lines      = get_lines(
            'raw_output' => $raw_output,
            'can_throw'  => $can_throw
        );
        return (
            _slot_parser( 'lines' => \@lines, 'response_object' => $resp ) );
	} elsif ( $option eq ""
        || $option =~ /-h/
        || $option =~ /-av$/
        || $option =~ /-v/
        || $option =~ /^-a$/)
    {
        my @return_array;
        my @lines      = get_lines(
            'raw_output' => $raw_output,
            'can_throw'  => $can_throw
        );
        my $line;
        my %temp_hash;

        # The following lines comes if system is in takeover mode.
        # Get rid of unwanted lines
        @lines = grep (!(/taken\s+over\s+by\s+\S+|No\s+hardware\s+device\s+information\s+is\s+available/i), @lines);
	#first line is always version info
	$line                      = shift(@lines);
	$line                      =~ s/,$//;
        $temp_hash{'version_info'} = $line;
        $line                      = shift(@lines);
	while ( $line !~ /slot\s+\d+:/ and scalar(@lines) && $line !~ /^(\S+:)+\s(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
		if ($line =~ /;\s+/){
			my @id_line = split(/;\s+/,$line);
			unshift(@lines,@id_line);
		}
		my ( $key, $value ) = split( /:/, $line );
		$key = to_key_name( 'item' => $key );
		trim( 'string_ptr' => \$value );
		$temp_hash{$key} = $value;
		$line = shift(@lines);
	}
        unshift( @lines, $line ) if ($line =~ /slot\s+\d+:/ || $line =~ /^(\S+:)+\s(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ || $line =~ /\s+(\S+)\s+(\S+)\s+(\S+)/);
        my $slot_parsed_array_ptr
            = _slot_parser( 'lines' => \@lines, 'response_object' => $resp );
        $temp_hash{'slot_information'} = $slot_parsed_array_ptr;
        push( @return_array, \%temp_hash );
        return ( \@return_array );
    } elsif ( $option =~ /-M/ ) {
        my $first_line = '\!System\s+Serial\s+Number\!';
        my @lines      = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        my @return_array;
        my $temp_hash_ptr;
        while ( scalar(@lines) ) {
            my $line = shift(@lines);
            my ( $dummy, $key, $value ) = split( /!/, $line, 3 );
            $key = to_key_name( 'item' => $key );
            $value =~ s/\!$//;
            if ( ( exists( $temp_hash_ptr->{$key} ) ) ) {
                if ( ref( $temp_hash_ptr->{$key} ) ne 'ARRAY' ) {
                    my $temp = $temp_hash_ptr->{$key};
                    $temp_hash_ptr->{$key} = ();
                    push( @{ $temp_hash_ptr->{$key} }, $temp, $value );
                } else {
                    push( @{ $temp_hash_ptr->{$key} }, $value )
                }
            } else {
                $temp_hash_ptr->{$key} = $value;
            }
        }
        push( @return_array, $temp_hash_ptr );
        return ( \@return_array );
    } elsif ( ( $option =~ /-av\s+\d+/ ) || ( $option !~ /-/ ) ) {

        #then they hav pased the slot number
        my $first_line = 'slot';
        my @lines      = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return (
            _slot_parser( 'lines' => \@lines, 'response_object' => $resp ) );
    } elsif ( $option =~ /-ac/ ){
	    my $first_line = 'slot';
	    my @lines      = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
        return (
            _slot_parser( 'lines' => \@lines, 'response_object' => $resp ) );
    } elsif ( $option =~ /-P/ ) {
        my $line;
        my $count = 0;
        my @return_arr;
        my $sub_key;
        my $sub_val;
        while( $line = shift(@lines)){
            my %hash ;
            my $key ;
            if( $line =~ /on controller/i ){
                $key = "pci device";
                $return_arr[$count]->{to_key_name( item => $key )} = $line ;
                $line = shift(@lines);
            }
            while ( $line !~ /on controller/i ) {
                if ( $line =~ /^(\w+)\(/ ) {
		    if ( $line =~ /linkcap/i ) {
                        $sub_key = "link capacity";
		    }elsif ( $line =~ /linkstatus/i ) {
			$sub_key = "link status";
		    }
                    $line =~ s/$1//g;
                    $sub_val = $line;
                    $return_arr[$count]->{to_key_name( item => $sub_key )} = $sub_val ;
                }
            $line = shift @lines ;
	    last if ( !($line) );
            }
            unshift @lines, $line ;
            $count++ ;
        }
        return \@return_arr;
    } elsif ( $option =~ /-A|-m|-p|-V|-D|-l|-M/ ) {
        return [];
    }
}

## @name ::sysconfig
##
## @summary Parses the raw output from the "sysconfig" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sysconfig with any of its options as described in the
##    corresponding cdef file 'sysconfig.cdefs'
## @synopsis
##      sysconfig(%args);
## INPUTS TO METHOD CALL 'sysconfig' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'disk' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-readable' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'tape-library-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'raid-configuration' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'detailed-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'pool-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'check-expansion-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-M' </td> <td> 'model-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'reports' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disk-information' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'aggr-config-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'tape-drive' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'host-machine-info' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; sysconfig -M; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;!System Serial Number!4071187-45-7!
##&nbsp;!Model Name!SIMBOX!
##&nbsp;!System ID!4071187457!
##&nbsp;!Kernel version!NetApp Release daemon_base 7-Mode: Thu Feb 23 19:08:17 EST 2012 !
##&nbsp;!System Rev!NOT SET!
##&nbsp;!Firmware release!NOT SET!
##&nbsp;!Mother Board!999999!NOT SET!NOT SET!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_version' => 'NetApp Release daemon_base 7-Mode: Thu Feb 23 19:08:17 EST 2012 ',
##&nbsp;            'firmware_release' => 'NOT SET',
##&nbsp;            'system_id' => '4071187457',
##&nbsp;            'mother_board' => '999999!NOT SET!NOT SET',
##&nbsp;            'system_rev' => 'NOT SET',
##&nbsp;            'system_serial_number' => '4071187-45-7',
##&nbsp;            'model_name' => 'SIMBOX'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; sysconfig -a -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;sysconfig: slot 1 OK: X1049A: PCI-E Quad 10/100/1000 Ethernet G20
##&nbsp;sysconfig: Unless directed by NetApp Global Services volumes newvol2, newvol, vol0, test_vol1, and test_vol2 should have the volume option create_ucode set to On.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'configuration_errors' => [
##&nbsp;                                        'sysconfig: Unless directed by NetApp Global Services volumes newvol2, newvol, vol0, test_vol1, and test_vol2 should have the volume option create_ucode set to On.'
##&nbsp;                                      ],
##&nbsp;            'devices' => [
##&nbsp;                           {
##&nbsp;                             'status' => 'OK',
##&nbsp;                             'slot_number' => '1',
##&nbsp;                             'device' => ' PCI-E Quad 10/100/1000 Ethernet G20',
##&nbsp;                             'device_part_number' => ' X1049A'
##&nbsp;                           }
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sysconfig(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    _parse_for_errors(
            'raw_output'   => $raw_output,
            response_object => $resp,
    );
    return ( _sysconfig( 'response_object' => $resp ) );
}
1;
