# $Id$
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode syslog commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode syslog commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "syslog rotate_syslog" will be syslog_rotate_syslog()
##
## @synopsis
##    use syslog;
##
##    syslog_rotate_syslog(%args);
##    syslog_reset_syslog(%args);
##    syslog(%args);
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::syslog;
use strict;

## @name ::syslog_rotate_syslog
##
## @summary Parses the raw output from the "syslog rotate_syslog" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from syslog_rotate_syslog with any of its options as described in the
##    corresponding cdef file 'syslog.cdefs'
## @synopsis
##      syslog_rotate_syslog(%args);
## INPUTS TO METHOD CALL 'syslog_rotate_syslog' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = syslog rotate_syslog
##&nbsp;$VAR1 = [];
##&nbsp;
sub syslog_rotate_syslog {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "syslog rotate_syslog",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::syslog_reset_syslog
##
## @summary Parses the raw output from the "syslog rotate_syslog" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from syslog_reset_syslog with any of its options as described in the
##    corresponding cdef file 'syslog.cdefs'
## @synopsis
##      syslog_reset_syslog(%args);
## INPUTS TO METHOD CALL 'syslog_reset_syslog' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = syslog rotate_syslog
##&nbsp;$VAR1 = [];
##&nbsp;
sub syslog_reset_syslog {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "syslog reset_syslog",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::syslog
##
## @summary Parses the raw output from the "syslog" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from syslog with any of its options as described in the
##    corresponding cdef file 'syslog.cdefs'
## @synopsis
##      syslog(%args);
## INPUTS TO METHOD CALL 'syslog' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'priority' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'repeat-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-1' </td> <td> 'toggle' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'message' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'facility' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; syslog 1 crit auth test_msg; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub syslog {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    if ( $raw_output =~ /Usage\s+1:syslog.*/ ) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /syslog:\s+Unknown.*/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;

} ## end sub _check_error

1;
