#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode sysstat command output
##
## @description
##    This is a library to parse the raw output of 7Mode sysstat command.
##
## @synopsis
##    use sysstat;
##
##    sysstat(%args);
##
## @author bhaskarm@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::sysstat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(sysstat|[Uu]sage):\s(.*)/ ) {
        $$error_message = $2;
        $error          = "CommandFailed";
        if ( $$error_message =~ /([Ii]nvalid|[Uu]nrecognized) option/ ) {
            $error = "InvalidParam";
        } else {
            $$error_message
                = "Command failed due to wrong usage or dependencies";
        }
    }
    return $error;
}

## @name ::sysstat
##
## @summary Parses the raw output from the "sysstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sysstat with any of its options as described in the
##    corresponding cdef file 'sysstat.cdefs'
## @synopsis
##      sysstat(%args);
## INPUTS TO METHOD CALL 'sysstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th>                                   </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interval' </td> <td> '0' </td> <                                  /tr>
## <tr> <td> '-m' </td> <td> 'multiprocessor-statistics' </td> <td> '0' </td> </                                  tr>
## <tr> <td> '-u' </td> <td> 'print-utilization' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'print-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'different-disk-types' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'iscsi-target-statistics' </td> <td> '1' </td> </tr                                  >
## <tr> <td> '-s' </td> <td> 'print-summary' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'FCP-target-statistics' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'SAN-statistics' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; sysstat -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; CPU     NFS    CIFS    HTTP     Net   kB/s    Disk   kB/s    Tape   kB/                                  s  Cache
##&nbsp;                                  in    out    read  write    read  writ                                  e    age
##&nbsp;  3%       0       0       0       0      0       5     16       0                                        0   >60
##&nbsp;  4%       0       0       0       0      0      11      0       0                                        0   >60
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stat' => [
##&nbsp;                        {
##&nbsp;                          'HTTP' => '0',
##&nbsp;                          'Tape-read' => '0',
##&nbsp;                          'CIFS' => '0',
##&nbsp;                          'CPU' => '3%',
##&nbsp;                          'NFS' => '0',
##&nbsp;                          'Tape-kB/s-write' => '0',
##&nbsp;                          'Net-in' => '0',
##&nbsp;                          'Disk-kB/s-write' => '16',
##&nbsp;                          'Cache-age' => '>60',
##&nbsp;                          'Net-kB/s-out' => '0',
##&nbsp;                          'Disk-read' => '5'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'HTTP' => '0',
##&nbsp;                          'Tape-read' => '0',
##&nbsp;                          'CIFS' => '0',
##&nbsp;                          'CPU' => '4%',
##&nbsp;                          'NFS' => '0',
##&nbsp;                          'Tape-kB/s-write' => '0',
##&nbsp;                          'Net-in' => '0',
##&nbsp;                          'Disk-kB/s-write' => '0',
##&nbsp;                          'Cache-age' => '>60',
##&nbsp;                          'Net-kB/s-out' => '0',
##&nbsp;                          'Disk-read' => '11'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub sysstat {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $summary_output;
    $summary_output = $1 if ( $output =~ /Summary Statistics.*\n((.|\n)*)/ );
	my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        first_line      => '(ANY\s+AVG|CPU\s+)',
        response_object => $resp
    );
    my $line1 = shift(@lines);
    my @keys1 = split( /\s+/, $line1 );
    my $counter = @keys1;
    #the for loop is making sure that there are no duplicate entries which get over written in the parsed output.
    for ( my $i = 0; $i <= $counter; $i++){
        if ( $keys1[$i] =~ /kB\/s/){
            $keys1[$i]=$keys1[$i-1]."-".$keys1[$i];
        }
    }
    if ( $line1 =~ /CPU\s+/i ) {
        my $line2 = shift(@lines);
        if ( $command =~ /^(.*)-x(.*)-H/ ){
            $line2 =~ s/util\s+util/util util - - - / ;
        } elsif  ( $command =~ /-x/ ){
            $line2 =~ s/util\s/util - - - / ;
        }
        my @keys2 = split( /\s+/, $line2 );
        my $keys1_last_index = $#keys1;
        while ( $keys2[0] ) {
            my $last = pop(@keys2);
            $keys1[$keys1_last_index] .= "-" . $last
                if ( $last !~ /\s*-\s*/ );
            $keys1_last_index--;
        }
    }
    my @stat_array;
    foreach my $line (@lines) {
        last if ( $line eq "--" );
        my @data = split( /\s+/, $line );
        my %hash;
        for ( my $i = 0; $i <= $#keys1; $i++ ) {
            $hash{ $keys1[$i] } = $data[$i];
        }
        push( @stat_array, \%hash );
    }
    my %result_hash;
    $result_hash{stat} = \@stat_array;
    if ($summary_output) {
        my @sum_lines = get_lines(
            raw_output      => $summary_output,
            can_throw       => 1,
            first_line      => '(ANY\s+AVG|CPU\s+)',
            response_object => $resp
        );
        my $sum_line1 = shift(@sum_lines);
        shift(@sum_lines) if ( $sum_line1 =~ /CPU\s+/i );
        for ( my $i = 0; $i <= 2; $i++ ) {
            my $name = shift(@sum_lines);
            my %hash;
            my @data = split( /\s+/, shift(@sum_lines) );
            for ( my $j = 0; $j <= $#keys1; $j++ ) {
                $hash{ $keys1[$j] } = $data[$j];
            }
            $hash{name} = $name;
            $result_hash{$name} = \%hash;
        }
    }
    return [\%result_hash];
}

1;
