#
#Copyright (c) 2012 NetApp
#All rights reserved.
#
## @summary Routines to parse 7Mode tapetest commands
##
## @description
##    This is a library to parse the raw outputs of any of the tape commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "tapetest write perf" will be tapetest_write_perf()
##
## @synopsis
##    use tapetest;
##
## @author amal@netapp.com
##
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::tapetest;

use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use strict;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /(.*argument required.*)/ ) {
        $error = "MissingRequiredParam";
	#$$error_message = "There is a parameter missing in the issued command";
	$$error_message = $1;
	return $error;
    }
    $$error_message = $1;
    return $error;

} ## end sub _check_error

## @name ::tapetest_read_perf
##
## @summary Parses the raw output from the "tapetest read_perf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tapetest_read_perf with any of its options as described in the
##    corresponding cdef file 'tapetest.cdefs'
## @synopsis
##      tapetest_read_perf(%args);
## INPUTS TO METHOD CALL 'tapetest_read_perf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'display-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## my $output = $Api_Set_Obj->tapetest_read_perf('count' => '20000', 'size' => '256', 'display_rate' => 0);
##
sub tapetest_read_perf {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "tapetest read_perf",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return _parse_read_write_perf($output);

} ## end sub tapetest_read_perf

## @name ::tapetest_write_perf
##
## @summary Parses the raw output from the "tapetest write_perf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tapetest_write_perf with any of its options as described in the
##    corresponding cdef file 'tapetest.cdefs'
## @synopsis
##      tapetest_write_perf(%args);
## INPUTS TO METHOD CALL 'tapetest_write_perf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'display-rate' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'device-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## my $output = $Api_Set_Obj->tapetest_write_perf('count' => '20000', 'size' => '256', 'display_rate' => 0);
##
sub tapetest_write_perf {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "tapetest write_perf",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return _parse_read_write_perf($output);

} ## end sub tapetest_write_perf

sub _parse_read_write_perf {

    my $output = $_[0];
    my $final_output;

    my @personal = split( /\n/, $output );

    #tapetest write_perf: starting at Fri Jan 13 18:28:36 EST 2012
    $personal[1] =~ m/tapetest.*: starting at (.*)/;
    $final_output->[0]->{'date_started'} = $1;

    #transfer size = 262144 count = 20000 display rate = 0 total size = 5242880000
    $personal[3]
        =~ m/transfer size = (.*) count = (.*) display rate = (.*) total size = (.*)/;
    $final_output->[0]->{'transfer_size'} = $1;
    $final_output->[0]->{'count'}         = $2;
    $final_output->[0]->{'display_rate'}  = $3;
    $final_output->[0]->{'total_size'}    = $4;

    #transferred 5242880000 bytes in 71 seconds (73722911 bytes/s)
    $personal[5] =~ m/transferred (.*) bytes in (.*) seconds \((.*)\)/;
    $final_output->[0]->{'transferred_bytes'} = $1;
    $final_output->[0]->{'seconds'}           = $2;
    $final_output->[0]->{'rate'}              = $3;

    #tapetest write_perf: completed OK
    $personal[6] =~ m/.*:(.*)/;
    $final_output->[0]->{'status'} = $1;
    $personal[8] =~ m/stopped at (.*)/;
    $final_output->[0]->{'date_completed'} = $1;

    return $final_output;

} ## end sub _parse_read_write_perf

1;
