#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode testsetattr command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode testsetattr command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::testsetattr;
##
##    testsetattr(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::testsetattr;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /$command(?:[^:]*):(.*)/ ) {
        $$error_message = $1;

        # if a value is to be provided as a space separated string of
        # smaller values and the entire set of smaller values isn't provided
        # then the value provided is incomplete and we consider it an
        # invalid value for that option(key/switch).
        # For example specifying only the offset for write-offset-and-length
        if ( $$error_message =~ /Invalid|Specify.+\s+and\s+/ ) {
            $error = "InvalidParamValue";
        } elsif ( $$error_message =~ /[Ss]pecify/ ) {
            $error = "MissingRequiredParam";
        } elsif ( $$error_message =~ /([uU]nknown option)/ ) {
            $error = "InvalidParam";
        } else {
            $error = "CommandFailed";
        }
        return $error;
    } elsif ( $raw_output =~ /[Uu]sage:/ ) {
        $error = "CommandFailed";
        $$error_message
            = "Invalid switch or Invalid Param value has been sent";
        return $error;
    }
    return undef;
}

## @name ::testsetattr
##
## @summary Parses the raw output from the "testsetattr" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from testsetattr with any of its options as described in the
##    corresponding cdef file 'testsetattr.cdefs'
## @synopsis
##      testsetattr(%args);
## INPUTS TO METHOD CALL 'testsetattr' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-W' </td> <td> 'write-offset-and-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'bypass-snapmirror' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'permissions' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'get-attributes' <td> '1' </td> </td> </tr>
## <tr> <td> '-r' </td> <td> 'restore' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'file-space-reservations' </td>  <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'close' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inherit-base-inode-reservation' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'read-offset-and-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'stream-name' </td> <td> '0' </td> </tr>
## <tr> <td> '--su' </td> <td> 'user' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'file-allocation-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'file-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'time-settings' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'open' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'set-cifs-sparse-flag' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'gid' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'block-size' </td> <td> '0' </td> </tr>
## <tr> <td> '--eat-reserve' </td> <td> 'eat-wafl-reserve' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub testsetattr {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "testsetattr: error",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $command !~ /\s+-l/ ) {
        return [];
    }

    my $first_line = '\w+_type:';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        response_object => $resp,
        can_throw       => 1
    );
    my ( $key, $value, %hash );
    foreach my $line (@lines) {
        if ( ( $key, $value ) = split( /:\s+/, $line ) ) {
            $hash{ to_key_name( item => $key ) } = $value;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "Output is not in the expected format",
                response_object => $resp
            );
        }
    }
    return [\%hash];
}

1;
