#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/tp.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode tp command output
##
## @description
##    This is a library to parse the raw output of 7Mode tp command.
##
## @synopsis
##
##    tp_enable(%args);
##    tp_list(%args);
##    tp_disable(%args);
##    tp_clear(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::tp;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::tp_enable
##
## @summary Parses the raw output from the "tp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tp_enable with any of its options as described in the
##    corresponding cdef file 'tp.cdefs'
## @synopsis
##      tp_enable(%args);
## INPUTS TO METHOD CALL 'tp_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hitcount' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'parm1' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'enable-type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'parm2' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'module-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'module-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub tp_enable {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub tp_enable

## @name ::tp_list
##
## @summary Parses the raw output from the "tp list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tp_list with any of its options as described in the
##    corresponding cdef file 'tp.cdefs'
## @synopsis
##      tp_list(%args);
## INPUTS TO METHOD CALL 'tp_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; tp list; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;tps enabled 2
##&nbsp;[0]: Module rdmalib id max_send_wrid count 2 action 4 parm1 1 parm2 0
##&nbsp;[1]: Module rdmalib1 id max_send_wrid1 count 3 action 1 parm1 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tp_info' => [
##&nbsp;                            {
##&nbsp;                              'count' => '2',
##&nbsp;                              'parm1' => '1',
##&nbsp;                              'action' => '4',
##&nbsp;                              'parm2' => '0',
##&nbsp;                              'id' => 'max_send_wrid',
##&nbsp;                              'module' => 'rdmalib',
##&nbsp;                              'tp_id' => '0'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'count' => '3',
##&nbsp;                              'parm1' => '2',
##&nbsp;                              'action' => '1',
##&nbsp;                              'id' => 'max_send_wrid1',
##&nbsp;                              'module' => 'rdmalib1',
##&nbsp;                              'tp_id' => '1'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'tps_enabled' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub tp_list {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head, @values );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = 0;
    foreach my $line (@lines) {

        if ( $line =~ /^(\w+\s+\w+)\s+(\d+)$/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            $count = 0;
        }
        elsif ( $line =~
/\[(\d+)\][:]\s+(\w+)\s+(\S+)\s+(\w+)\s+(\S+)\s+(\w+)\s+(\d+)\s+(\w+)\s+(\d+)\s+(\w+)\s+(\w+)\s*(\w*)\s*(\d*)/
          )
        {
            my %hash = (
                "tp_id" => "$1",
                "$2"    => "$3",
                "$4"    => "$5",
                "$6"    => "$7",
                "$8"    => "$9",
                "$10"   => "$11",
                "$12"   => "$13"
            );
            while ( my ( $key, $value ) = each(%hash) ) {
                $result_array[0]->{tps_info}->[$count]
                  ->{ to_key_name( item => $key ) } = $value
                  if ($key);
            }
            $count++;
        }
    }
    return \@result_array;
} ## end sub tp_list

## @name ::tp_disable
##
## @summary Parses the raw output from the "tp disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tp_disable with any of its options as described in the
##    corresponding cdef file 'tp.cdefs'
## @synopsis
##      tp_disable(%args);
## INPUTS TO METHOD CALL 'tp_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'module-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'module-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub tp_disable {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub tp_disable

## @name ::tp_clear
##
## @summary Parses the raw output from the "tp clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tp_clear with any of its options as described in the
##    corresponding cdef file 'tp.cdefs'
## @synopsis
##      tp_clear(%args);
## INPUTS TO METHOD CALL 'tp_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub tp_clear {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub tp_clear

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~
        /No\s+testpoints\s+enabled.*|can't\s+enable\/disable\s+tp/ )
    {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /wrong\s+#args.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Usage:\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

