#$Id: $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode traceroute command output
##
## @description
##    This is a library to parse the raw output of 7Mode traceroute command.
##
## @synopsis
##    use traceroute;
##
##    traceroute(%args);
##
## @author kathar@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::traceroute;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /Usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif (
        $output =~ /(wait must be > 1 sec.*)|(max ttl must be > 1.*)|
                     (tos must be 0 to 255.*)|(packet size must be 0 <=.*)/i
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /(.*No route to host)/i ) {
        $error          = "CommandFailed";
        $$error_message = $&;
    } elsif ( $output =~ /.*(!N)|.*(!P)|.*(!S)|.*(!A)/ ) {
        $error          = "CommandFailed";
        $$error_message = $& . "Destination  Unreachable";
    }
    return $error;
} ## end sub _check_error

## @name ::traceroute
##
## @summary Parses the raw output from the "traceroute" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from traceroute with any of its options as described in the
##    corresponding cdef file 'traceroute.cdefs'
## @synopsis
##      traceroute(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hop_count' => '4',
##&nbsp;            'pkt_size' => '40',
##&nbsp;            'host_name' => '10.72.219.1',
##&nbsp;            'host_ip' => '10.72.219.1',
##&nbsp;            'hops' => [
##&nbsp;                        {
##&nbsp;                          'round_trip_time' => [
##&nbsp;                                                 '10.067 ms',
##&nbsp;                                                 '0.000 ms',
##&nbsp;                                                 '0.000 ms',
##&nbsp;                                                 '10.000 ms'
##&nbsp;                                               ],
##&nbsp;                          'node_ip' => '10.72.32.2',
##&nbsp;                          'hop_count' => '1',
##&nbsp;                          'node_name' => '10.72.32.2'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'round_trip_time' => [
##&nbsp;                                                 '0.000 ms',
##&nbsp;                                                 '0.000 ms'
##&nbsp;                                               ],
##&nbsp;                          'node_ip' => '10.72.127.2',
##&nbsp;                          'hop_count' => '2',
##&nbsp;                          'node_name' => '10.72.127.2'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##
sub traceroute {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ($error_message, $error,     $i,
        $j,             $count,     $temp,
        @lines,         $firstline, @result
    );

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    $firstline = "traceroute to .*";
    @lines     = get_lines(
        raw_output      => $output,
        first_line      => $firstline,
        can_throw       => 1,
        response_object => $resp
    );
    $i         = 0;
    $j         = 0;
    $firstline = $lines[0];
    if ( $firstline
        =~ /(\S+)\s+\((\S+)\),\s+(\d+) hops max, (\d+) byte packets/ ) {
        $result[$i]->{'host_name'} = $1;
        $result[$i]->{'host_ip'}   = $2;
        $result[$i]->{'hop_count'} = $3;
        $result[$i]->{'pkt_size'}  = $4;
    } elsif ( $firstline
        =~ /(\S+)\s+\((\S+)\) from (\S+),\s+(\d+) hops max, (\d+) byte packets/
        ) {
        $result[$i]->{'host_name'} = $1;
        $result[$i]->{'host_ip'}   = $2;
        $result[$i]->{'src_addr'}  = $3;
        $result[$i]->{'hop_count'} = $4;
        $result[$i]->{'pkt_size'}  = $5;
    }

    ## removing the header information
    shift @lines;
    foreach $temp (@lines) {
        if ( $temp =~ /^(\d+)\s+(\S+)\s+\((\S+)\)(.*)/ ) {
            $result[$i]->{'hops'}->[$j]->{'hop_count'} = $1;
            $result[$i]->{'hops'}->[$j]->{'node_name'} = $2;
            $result[$i]->{'hops'}->[$j]->{'node_ip'}   = $3;
            my $rtt = $4;
            my @round_trip_time;
            $count = 0;
            while ( $rtt =~ /(\d+\.\d+ ms)/g ) {
                $round_trip_time[$count] = $1;
                $count++;
            }
            $result[$i]->{'hops'}->[$j]->{'round_trip_time'}
                = \@round_trip_time;
            $j++;
        }
    }
    return \@result;
} ## end sub traceroute
1;
