# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/traceroute6.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode traceroute6 command output
##
## @description
##    This is a library to parse the raw output of 7Mode traceroute6 command.
##
## @synopsis
##    use traceroute6;
##
##    traceroute6(%args);
##
## @author benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::traceroute6;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /Usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ( $output =~ /(Unknown error.*)/i ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ( $output =~ /(Invalid argument.*)|(Invalid option.*)/i ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $output
        =~ /(.*wait must be >1 sec)|(min hoplimit must be <= 1)|(packet size must be 0 <= s <.*)/i
        ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $output =~ /(.*No route to host)/i ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ( $output
        =~ /(.*!N)|(.*!P)|(.*!S)|(.*!A)|(Can't assign requested address on bind)/
        ) {
        $error          = "CommandFailed";
        $$error_message = $& . "Destination  Unreachable";
    }
    return $error;
} ## end sub _check_error

## @name ::traceroute6
##
## @summary Parses the raw output from the "traceroute6" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from traceroute6 with any of its options as described in the
##    corresponding cdef file 'traceroute6.cdefs'
## @synopsis
##      traceroute6(%args);
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'src_addr' => '6ffe::14f2',
##&nbsp;            'hop_count' => '4',
##&nbsp;            'pkt_size' => '12',
##&nbsp;            'host_name' => '6ffe::14f2',
##&nbsp;            'host_ip' => '6ffe::14f2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'round_trip_time' => [
##&nbsp;                                   '0.000 ms',
##&nbsp;                                   '0.000 ms',
##&nbsp;                                   '0.000 ms',
##&nbsp;                                   '0.000 ms',
##&nbsp;                                   '0.000 ms',
##&nbsp;                                   '0.000 ms'
##&nbsp;                                 ],
##&nbsp;            'node_ip' => '6ffe::14f2',
##&nbsp;            'hop_count' => '1',
##&nbsp;            'pkt_size' => '68',
##&nbsp;            'src_ip' => '6ffe::14f2'
##&nbsp;          }
##&nbsp;];
##&nbsp;
sub traceroute6 {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ( $error_message, $error, $i, $temp, @lines, $firstline, @result );

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    $firstline = "traceroute6 to .*";
    @lines     = get_lines(
        raw_output      => $output,
        first_line      => $firstline,
        can_throw       => 1,
        response_object => $resp
    );
    $i         = 0;
    $firstline = $lines[0];
    if ( $firstline
        =~ /(\S+)\s+\((\S+)\) from (\S+),\s+(\d+) hops max, (\d+) byte packets/
        ) {
        $result[$i]->{'host_name'} = $1;
        $result[$i]->{'host_ip'}   = $2;
        $result[$i]->{'src_addr'}  = $3;
        $result[$i]->{'hop_count'} = $4;
        $result[$i]->{'pkt_size'}  = $5;
    }
    $i++;
    shift @lines;
    foreach $temp (@lines) {
        if ( $temp
            =~ /^(\d+)\s+(\S+)\s+(\d+)\s+bytes of data to\s+(\S+)\s+(.*)/ ) {
            $result[$i]->{'hop_count'} = $1;
            $result[$i]->{'node_ip'}   = $2;
            $result[$i]->{'pkt_size'}  = $3;
            $result[$i]->{'src_ip'}    = $4;
            my $rtt = $5;
            my @round_trip_time;
            my $j = 0;
            while ( $rtt =~ /(\d+\.\d+ ms)/g ) {
                $round_trip_time[$j] = $1;
                $j++;
            }
            $result[$i]->{'round_trip_time'} = \@round_trip_time;
            $i++;
        } elsif ( $temp =~ /^(\d+)\s+(\S+)\s+(.*)/ ) {
            $result[$i]->{'hop_count'} = $1;
            $result[$i]->{'node_ip'}   = $2;
            my $rtt = $3;
            my @round_trip_time;
            my $j = 0;
            while ( $rtt =~ /(\d+\.\d+ ms)/g ) {
                $round_trip_time[$j] = $1;
                $j++;
            }
            $result[$i]->{'round_trip_time'} = \@round_trip_time;
            $i++;
        }
    }
    return \@result;
} ## end sub traceroute6
1;
