#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode treecompare commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode treecompare
##    commands.

## @synopsis
##    use treecompare;
##
##    treecompare(%args);
##
## @author dl-dpg-nb-automation; dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::treecompare;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::treecompare
##
## @summary Parses the raw output from "treecompare" command
##
## @description
## This method can be used for parsing the output from "treecompare"
## command called with any of its options as described in the cdef file at
## lib/NACL/APISet/Node/CLI/7Mode/default/treecompare.cdefs
##
## @synopsis
## treecompare(%args);
##
## response_object => used to get the raw output, actual command line invoked,
##                    to populate the response_object
##                    with the parsed output incase of partial success in the
##                    command execution and to throw exceptions of type
##                    ResponseException in which the exception object will be
##                    populated with the response object
##
## @return Returns an array reference which contains the parsed output data.
#
## @name ::treecompare_start
##
## @summary Parses the raw output from the "treecompare start" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from treecompare_start with any of its options as described in the 
##    corresponding cdef file 'treecompare.cdefs'
## @synopsis
##      treecompare_start(%args);
## INPUTS TO METHOD CALL 'treecompare_start' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'treepath1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'dc-ctime' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'dc-ACLs' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'treepath2' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'c-inode-generation-treeid' </td> </tr> 
## <tr> <td> '-D' </td> <td> 'dc-DOS-bits' </td> </tr> 
## <tr> <td> '-U' </td> <td> 'dc-cifs-bit' </td> </tr> 
## <tr> <td> '-L' </td> <td> 's-check-level' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'dc-atime' </td> </tr> 
## <tr> <td> '-V' </td> <td> 'dc-vdisk-metadata' </td> </tr> 
## <tr> <td> '-Q' </td> <td> 'quick-mode' </td> </tr> 
## <tr> <td> '-k' </td> <td> 's-throttle-value' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'dc-crtime' </td> </tr> 
## <tr> <td> '-P' </td> <td> 'dc-permissions-uid' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub treecompare_start {
    my %args = @_;
    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"treecompare",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = "NACL::APISet::Exceptions::${error}Exception"; 
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _treecompare_start(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);

    return $result;
}

sub _treecompare_start {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array );
    my $can_throw =1;

    # specifying the first line of the output.
    # mismatch output will vary with snapshots we are comparing hence we will 
    # parse only mismatch number, tree2_only, tree1_only , not the
    # entries / activities.

    my $first_line = '(TREECMP: Done.) .+';
    my @lines = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);

    #regex to parse the o/p in the format [tree1_only = X, tree2_only = X, mismatch = XXXX]
    if ( $lines[1] =~  /(\w+)\s*=\s*(\d+)\s*\,\s*(\w+)\s*=\s*(\d+)\s*\,\s*(\w+)\s*=\s*(\d+)\s*/) {
        my %hash;
        my $key1 = $1;
        my $key3 = $3;
        my $key5 = $5;

        $key1 = to_key_name("item" => $key1);
        $key3 = to_key_name("item" => $key3);
        $key5 = to_key_name("item" => $key5);

        $hash{"$key1"}   = $2;
        $hash{"$key3"}   = $4;
        $hash{"$key5"}   = $6;
        push(@result_array, \%hash);
    }  else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                        text =>"Output is not in the expected format",
                                       response_object => $response);
    }

    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if ( $raw_output =~ /.*?(treecompare failed).+/i ) {
        $error          = "CommandFailed";
        $$error_message = "$1";
    } else {
        return undef;
    }
    $$error_message =~ s/^$command:\s//;
    return $error;
}

1;
