#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ttcp command
##
## @description
##    This is a library to parse the raw output of 7Mode ttcp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ttcp" will be ttcp()
##
## @synopsis
##    use ttcp;
##
##    ttcp(%args);
##
## @author anupamap@netapp.com, dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ttcp;

#use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(usage:\s)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    }
    elsif ( $raw_output =~ /(supports only sink mode.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(bind error.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(bad hostname.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(ttcp not found.*)/i ) {
        $$error_message = $1;
        $error          = "CommandNotFoundException";
    }
    elsif ( $raw_output =~ /(socket error.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(connect error.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(Timeout has occured.*)/i ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    return $error;
}

## @name ::ttcp
##
## @summary Parses the raw output from the "ttcp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ttcp with any of its options as described in the
##    corresponding cdef file 'ttcp.cdefs'
## @synopsis
##      ttcp(%args);
## INPUTS TO METHOD CALL 'ttcp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'source' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'input-buffer-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'misalign' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'address-family' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'number-of-source-bufs' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sink' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'receive-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'tcp-window-scale' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'snapvault-destination' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'udp' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'transmit-mode' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-number' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ignore-congestion-flag' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'full-blocks' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface-ipv6' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-address-v6' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'multicast-group' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'port' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'length-of-bufs-read' </td> <td> '0' </td> </tr>
## <tr> <td> '-S' </td> <td> 'output-buffer-size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; ttcp -r -s 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ttcp -r : nbuf=1024, buflen=1024, sock_buf=262144 port=5001\n
##&nbsp;ttcp-r: socket\n
##&nbsp;ttcp-r: accept\n
##&nbsp;ttcp-r:\n
##&nbsp;ttcp-r: 1048576 bytes processed\n
##&nbsp;ttcp-r: CPU% 0, busy_tic 0, cput 3 \n
##&nbsp;ttcp-r:         0 CPU sec  =    102400 KB/cpu sec,     819200 Kbits/cpu sec\n
##&nbsp;ttcp-r:         0 real sec =     34133 KB/real sec,    273066 Kbits/sec\n
##
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;               'cpu' => '0',
##&nbsp;               'KB_per_real_sec' => '34133',
##&nbsp;               'KB_per_cpu_sec' => '102400',
##&nbsp;               'busy_tic' => '0',
##&nbsp;               'connection_status' => 'accept',
##&nbsp;               'port' => '5001',
##&nbsp;               'sock_buf' => '262144',
##&nbsp;               'Kbits_per_real_sec' => '273066',
##&nbsp;               'buflen' => '1024',
##&nbsp;               'nbuf' => '1024',
##&nbsp;               'cput' => '3',
##&nbsp;               'bytes_processed' => '1048576',
##&nbsp;               'Kbits_per_cpu_sec' => '819200'
##&nbsp;            }
##&nbsp;        ];
##&nbsp;
sub ttcp {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my $i = 0;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @arraylines =
      get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arraylines) {
        my %hash;
            if ( $line =~ /^\w+.\w:\s+(\d+)\s+\w+\s+\w+$/ ) {
                $result_array[0]->{"bytes_processed"} = $1;
            }
            elsif (
                $line =~ /^\w+.\w:\s+\S+\s(\d+).\s+\S+\s(\d+).\s+\S+\s(\d+)/ )
            {
                $result_array[0]->{"cpu"}      = $1;
                $result_array[0]->{"busy_tic"} = $2;
                $result_array[0]->{"cput"}     = $3;
            }
            elsif ( $line =~
/^\w+\s+.\w+\s+:\s+\S+=(\d+).\s+\S+=(\d+).\s+\S+=(\d+)\s+\S+=(\d+)/
              )
            {
                $result_array[0]->{"nbuf"}     = $1;
                $result_array[0]->{"buflen"}   = $2;
                $result_array[0]->{"sock_buf"} = $3;
                $result_array[0]->{"port"}     = $4;
            }
            elsif ( $line =~ /^\w+.\w:\s+(\w+)$/ ) {
                $result_array[0]->{"connection_status"} = $1;
            }
            elsif ( $line =~
/^\w+.\w:\s+.\s+\w+\s\S+\s+=\s+(\d+)\s\S+\s+\S+\,\s+(\d+)\s+\S+\s\S+$/
              )
            {
                $result_array[0]->{"KB_per_cpu_sec"}    = $1;
                $result_array[0]->{"Kbits_per_cpu_sec"} = $2;
            }
            elsif ( $line =~
/^\w+.\w:\s+.\s+\S+\s+\S+\s+=\s+(\d+)\s\S+\s+\w+\,\s+(\d+)\s+\S+$/
              )
            {
                $result_array[0]->{"KB_per_real_sec"}    = $1;
                $result_array[0]->{"Kbits_per_real_sec"} = $2;
            }
    }
    return \@result_array;
}    # end sub ttcp;
1;
