#
#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ucadmin commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ucadmin commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ucadmin" will be ucadmin()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::ucadmin
##
##    ucadmin(%agrs);
##
## @author dl-nacl-dev@netapp.com,Veerabhadraswamy.P@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::7Mode::default::ucadmin;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::ucadmin_show
##
## @summary Parses the raw output from the "ucadmin show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ucadmin_show with any of its options as described in the
##    corresponding cdef file 'ucadmin.cdefs'
## @synopsis
##      ucadmin_show(%args);
## INPUTS TO METHOD CALL 'ucadmin_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'capability' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; ucadmin show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;         Current  Current    Pending  Pending
##&nbsp;Adapter  Mode     Type       Mode     Type       Status
##&nbsp;-------  -------  ---------  -------  ---------  -------
##&nbsp;  0a     fc       initiator  -        -          online
##&nbsp;  0b     fc       initiator  -        -          online
##&nbsp;  0c     fc       initiator  -        -          offline
##&nbsp;  0d     fc       initiator  -        -          offline
##&nbsp;  3a     cna      target     -        -          offline
##&nbsp;  3b     cna      target     -        -          offline
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'online',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '0a',
##&nbsp;            'current_mode' => 'fc',
##&nbsp;            'current_type' => 'initiator'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'online',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '0b',
##&nbsp;            'current_mode' => 'fc',
##&nbsp;            'current_type' => 'initiator'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '0c',
##&nbsp;            'current_mode' => 'fc',
##&nbsp;            'current_type' => 'initiator'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '0d',
##&nbsp;            'current_mode' => 'fc',
##&nbsp;            'current_type' => 'initiator'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '3a',
##&nbsp;            'current_mode' => 'cna',
##&nbsp;            'current_type' => 'target'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => 'offline',
##&nbsp;            'pending_type' => '-',
##&nbsp;            'pending_mode' => '-',
##&nbsp;            'adapter_name' => '3b',
##&nbsp;            'current_mode' => 'cna',
##&nbsp;            'current_type' => 'target'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; ucadmin show -c; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Adapter  Mode    FC Supported Types     CNA Supported Types
##&nbsp;-------  ------  ---------------------  ---------------------
##&nbsp;  0a     fc      initiator,target
##&nbsp;  0b     fc      initiator,target
##&nbsp;  0c     fc      initiator,target
##&nbsp;  0d     fc      initiator,target
##&nbsp;  3a     cna                            target
##&nbsp;  3b     cna                            target
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      'initiator',
##&nbsp;                                      'target'
##&nbsp;                                    ],
##&nbsp;            'adapter' => '0a',
##&nbsp;            'mode' => 'fc',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       ''
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      'initiator',
##&nbsp;                                      'target'
##&nbsp;                                    ],
##&nbsp;            'adapter' => '0b',
##&nbsp;            'mode' => 'fc',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       ''
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      'initiator',
##&nbsp;                                      'target'
##&nbsp;                                    ],
##&nbsp;            'adapter' => '0c',
##&nbsp;            'mode' => 'fc',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       ''
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      'initiator',
##&nbsp;                                      'target'
##&nbsp;                                    ],
##&nbsp;            'adapter' => '0d',
##&nbsp;            'mode' => 'fc',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       ''
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      ''
##&nbsp;                                    ],
##&nbsp;            'adapter' => '3a',
##&nbsp;            'mode' => 'cna',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       'target'
##&nbsp;                                     ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fc_supported_types' => [
##&nbsp;                                      ''
##&nbsp;                                    ],
##&nbsp;            'adapter' => '3b',
##&nbsp;            'mode' => 'cna',
##&nbsp;            'cna_supported_types' => [
##&nbsp;                                       'target'
##&nbsp;                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ucadmin_show {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    my (@result_array);
    my $count = 0;

    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp, first_line => "-------" );
    #to get rid of the line ------ ------- -------
    shift @lines;
    if ( $command =~ /^ucadmin show -c/) {
        foreach my $line (@lines) {
            if ($line=~/^(\w+)\s+(\w+)\s+(\s{20,})\s+(\S+)/|$line=~/(\w+)\s+(\w+)\s+(\w+)\,(\w+).*/) {
                $result_array[$count]->{'adapter'} = $1;
                $result_array[$count]->{'mode'} = $2;
                my $value=$4;
                if($3=~/\s+/){
                    push (@{$result_array[$count]->{'fc_supported_types'}},'');
                    push (@{$result_array[$count]->{'cna_supported_types'}},$value);
                }
                else {
                    push (@{$result_array[$count]->{'fc_supported_types'}},$3,$4);
                    push (@{$result_array[$count]->{'cna_supported_types'}},'');
                }
                $count++;
            }
        }
    }
    else {
        foreach my $line(@lines){
            my @values = split ('\s+', $line);
            $result_array[$count]->{'adapter_name'} = $values[0];
            $result_array[$count]->{'current_mode'} = $values[1];
            $result_array[$count]->{'current_type'} = $values[2];
            $result_array[$count]->{'pending_mode'} = $values[3];
            $result_array[$count]->{'pending_type'} = $values[4];
            $result_array[$count]->{'status'} = $values[5];
            $count++;
        }
    }
    return \@result_array;
}

## @name ::ucadmin_modify
##
## @summary Parses the raw output from the "ucadmin modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ucadmin_modify with any of its options as described in the
##    corresponding cdef file 'ucadmin.cdefs'
## @synopsis
##      ucadmin_modify(%args);
## INPUTS TO METHOD CALL 'ucadmin_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub ucadmin_modify {
    my %args = @_;
    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();

    _check_error( raw_output => $output, response_object => $resp );

	return [];
} ## end sub ucadmin_modify

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /ucadmin modify:.*/i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error
1;
