#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/unsparse.pm#4 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode unsparse commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode unsparse commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "unsparse" will be unsparse()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::unsparse;
##
##    unsparse(%args);
##
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::unsparse;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error = undef;
    if ( $raw_output =~ /Usage/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    } elsif ($raw_output =~ /(file creation failed)|(creat\/open .*)|(.*Volume in path not found.*)|(Invalid argument)/) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /File.*already exists/) {
        $error = "CommandFailed";
        $$error_message = $raw_output;
    }

    return $error;
}

## @name ::unsparse
##
## @summary Parses the raw output from the "unsparse" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from unsparse with any of its options as described in the
##    corresponding cdef file 'unsparse.cdefs'
## @synopsis
##      unsparse(%args);
## INPUTS TO METHOD CALL 'unsparse' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'length' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'range' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directory' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; unsparse -v -d /vol/vol0 4 ... 7; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;unsparse: /vol/vol0/4
##&nbsp;unsparse: /vol/vol0/5
##&nbsp;unsparse: /vol/vol0/6
##&nbsp;unsparse: /vol/vol0/7
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'unsparsed_files' => [
##&nbsp;                                   '/vol/vol0/4',
##&nbsp;                                   '/vol/vol0/5',
##&nbsp;                                   '/vol/vol0/6',
##&nbsp;                                   '/vol/vol0/7'
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub unsparse {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my @return_array;

    my $error = _check_error("raw_output"   => $raw_output,
                            "command"       => "unsparse",
                            "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    if(($raw_output =~ //)|| ($raw_output =~ /unsparse is not allowed on aggregates with reallocated blocks/)) {
        return [];
    }
    my @output = get_lines(raw_output=>$raw_output, response_object=>$response);

    my $line;
    my @file_name;
    my %info = ();
    foreach $line (@output) {
        if ($line =~ /unsparse: (.*)/) {
           push (@file_name , $1);
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "unexpected format of output",
                    response_object => $response
                );
        }
    }
    $info{"unsparsed_files"} = \@file_name if(scalar(@file_name)); 
    push(@return_array,\%info);
    return \@return_array;
}
1;
