#
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode uptime command
##
## @description
##    This is a library to parse the raw output of 7Mode uptime command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "uptime" will be uptime()
##
## @synopsis
##    use uptime;
##
##    uptime(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::uptime;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::uptime
##
## @summary Parses the raw output from the "uptime" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from uptime with any of its options as described in the 
##    corresponding cdef file 'uptime.cdefs'
## @synopsis
##      uptime(%args);
## INPUTS TO METHOD CALL 'uptime' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub uptime {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $result;
    
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>'uptime',
                             "ptr_error_message"=>\$error_message);
                             
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object=>$resp);
    }
    
    # Now we can parse the output
    $result = _uptime(raw_output=>$raw_output, command=>$command, 
                      response_object=>$resp);
    return $result;  
}


sub _uptime {
    my %args= @_;
    my $raw_output= $args{"raw_output"};
    my $command =$args{"command"};
    my $response =$args{"response_object"};
    my @result_array ;
    
    # For uptime, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw =1;
    
    my $first_line = 
        '(\d+:\d+)(am|pm)\s+(\w+)\s+(.+)\s+(\d+)\sNFS\sops,\s(\d+)\sCIFS\sops,\s(\d+)\sHTTP\sops,\s(\d+)\sFCP\sops,\s(\d+)\siSCSI\sops';
    my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
                        "can_throw"=>$can_throw, "response_object"=>$response);
    if (@arr >1) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text=>"Output is not in the expected format", 
            response_object=>$response);
    }
    my %hash;
    $arr[0] =~ 
        /(\d+:\d+)(am|pm)\s+(\w+)\s+(.+)\s+(\d+)\sNFS\sops,\s(\d+)\sCIFS\sops,\s(\d+)\sHTTP\sops,\s(\d+)\sFCP\sops,\s(\d+)\siSCSI\sops/;
    $hash{current_time}=$1.$2;
    $hash{state}=$3;
    $hash{uptime_length}=$4;
    $hash{nfs_ops}=$5;
    $hash{cifs_ops}=$6;
    $hash{http_ops}=$7;
    $hash{fcp_ops}=$8;
    $hash{icsi_ops}=$9;
    
    $hash{uptime_length} =~ s/,$//;
    push (@result_array, \%hash);
    return \@result_array;
}

sub _check_error {
    my %args =@_;
    my $raw_output = $args{"raw_output"};
    my $command = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if($raw_output =~ /uptime takes no arguments/) {
        $$error_message = "uptime takes no arguments";
        $error = "InvalidParam";
        return $error;
    }
    return undef;
}
1;
