# Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/useradmin.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode useradmin commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode useradmin
##    commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "useradmin user list" will be useradmin_user_list()
##
## @synopsis
##    use useradmin;
##
##    useradmin_whoami(%args);
##    useradmin_user_add(%args);
##    useradmin_user_modify(%args);
##    useradmin_user_delete(%args);
##    useradmin_user_list(%args);
##    useradmin_group_add(%args);
##    useradmin_group_modify(%args);
##    useradmin_group_delete(%args);
##    useradmin_group_list(%args);
##    useradmin_role_add(%args);
##    useradmin_role_modify(%args);
##    useradmin_role_delete(%args);
##    useradmin_role_list(%args);
##    useradmin_domainuser_add(%args);
##    useradmin_domainuser_delete(%args);
##    useradmin_domainuser_list(%args);
##    useradmin_domainuser_load(%args);
##    useradmin_diaguser_lock(%args);
##    useradmin_diaguser_unlock(%args);
##    useradmin_diaguser_password(%args);
##    useradmin_diaguser_show(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::useradmin;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::useradmin_whoami
##
## @summary Parses the raw output from the "useradmin whoami" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_whoami with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_whoami(%args);
## INPUTS TO METHOD CALL 'useradmin_whoami' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_whoami {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $can_throw  = 0;
    my $first_line = "";
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    #Parsing the output
    if ( $lines[0] =~ /(.+)/ ) {
        my %hash;
        $hash{'user_logged_in'} = $1;
        return [ \%hash ];
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $response
        );
    }
}

## @name ::useradmin_user_add
##
## @summary Parses the raw output from the "useradmin user add" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_user_add with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_user_add(%args);
## INPUTS TO METHOD CALL 'useradmin_user_add' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-p' </td> <td> 'password' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'password-min-age' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'full-name' </td> </tr> 
## <tr> <td> '-M' </td> <td> 'password-max-age' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'login-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_user_add {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
 
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    if ( $raw_output =~ /User <?(.+?)>? added/ ) {
        my %hash;
        $hash{'login_name'} = $1;
        $hash{'status'}     = "added";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_user_modify
##
## @summary Parses the raw output from the "useradmin user modify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_user_modify with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_user_modify(%args);
## INPUTS TO METHOD CALL 'useradmin_user_modify' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'password-min-age' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'full-name' </td> </tr> 
## <tr> <td> '-M' </td> <td> 'password-max-age' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'login-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_user_modify {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /User <?(.+?)>? modified/ ) {
        my %hash;
        $hash{'login_name'} = $1;
        $hash{'status'}     = "modified";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_user_delete
##
## @summary Parses the raw output from the "useradmin user delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_user_delete with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_user_delete(%args);
## INPUTS TO METHOD CALL 'useradmin_user_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'login-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_user_delete {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /User <?(.+?)>? deleted/ ) {
        my %hash;
        $hash{'login_name'} = $1;
        $hash{'status'}     = "deleted";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_user_list
##
## @summary Parses the raw output from the "useradmin user list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_user_list with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_user_list(%args);
## INPUTS TO METHOD CALL 'useradmin_user_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'extended-format' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'login-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_user_list {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #Parsing the output
    $result = _parse_list(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $response
    );

    return $result;
}

## @name ::useradmin_group_add
##
## @summary Parses the raw output from the "useradmin group add" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_group_add with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_group_add(%args);
## INPUTS TO METHOD CALL 'useradmin_group_add' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group-name' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'role' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_group_add {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Group <?(.+?)>? added/ ) {
        my %hash;
        $hash{'group_name'} = $1;
        $hash{'status'}     = "added";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_group_modify
##
## @summary Parses the raw output from the "useradmin group modify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_group_modify with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_group_modify(%args);
## INPUTS TO METHOD CALL 'useradmin_group_modify' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'group-name' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'role' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_group_modify {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Group <?(.+?)>? modified/ ) {
        my %hash;
        $hash{'group_name'} = $1;
        $hash{'status'}     = "modified";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_group_delete
##
## @summary Parses the raw output from the "useradmin group delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_group_delete with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_group_delete(%args);
## INPUTS TO METHOD CALL 'useradmin_group_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_group_delete {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Group <?(.+?)>? deleted/ ) {
        my %hash;
        $hash{'group_name'} = $1;
        $hash{'status'}     = "deleted";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_group_list
##
## @summary Parses the raw output from the "useradmin group list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_group_list with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_group_list(%args);
## INPUTS TO METHOD CALL 'useradmin_group_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group-name' </td> </tr> 
## <tr> <td> '-u' </td> <td> 'user-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_group_list {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #Parsing the output
    $result = _parse_list(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $response
    );

    return $result;
}

## @name ::useradmin_role_add
##
## @summary Parses the raw output from the "useradmin role add" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_role_add with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_role_add(%args);
## INPUTS TO METHOD CALL 'useradmin_role_add' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'role-name' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'capability' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_role_add {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Role <?(.+?)>? added/ ) {
        my %hash;
        $hash{'role_name'} = $1;
        $hash{'status'}    = "added";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_role_modify
##
## @summary Parses the raw output from the "useradmin role modify" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_role_modify with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_role_modify(%args);
## INPUTS TO METHOD CALL 'useradmin_role_modify' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'role-name' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'comments' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'capability' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_role_modify {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Role <?(.+?)>? modified/ ) {
        my %hash;
        $hash{'role_name'} = $1;
        $hash{'status'}    = "modified";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_role_delete
##
## @summary Parses the raw output from the "useradmin role delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_role_delete with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_role_delete(%args);
## INPUTS TO METHOD CALL 'useradmin_role_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'role-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_role_delete {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    
    if ( $raw_output =~ /Role <?(.+?)>? deleted/ ) {
        my %hash;
        $hash{'role_name'} = $1;
        $hash{'status'}    = "deleted";
        return [ \%hash ];
    } else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected output",
            response_object => $response
        );
    }
}

## @name ::useradmin_role_list
##
## @summary Parses the raw output from the "useradmin role list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_role_list with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_role_list(%args);
## INPUTS TO METHOD CALL 'useradmin_role_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'role-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_role_list {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $result;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #Parsing the output
    $result = _parse_list(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $response
    );

    return $result;
}

## @name ::useradmin_domainuser_add
##
## @summary Parses the raw output from the "useradmin domainuser add" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_domainuser_add with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_domainuser_add(%args);
## INPUTS TO METHOD CALL 'useradmin_domainuser_add' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domainuser-name' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_domainuser_add {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $can_throw  = 0;
    my $first_line = "SID\s+=";
    my %hash;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    foreach my $line (@lines) {
        if ( $line =~ /SID = (.+)/ ) {
            $hash{'sid'} = $1;
        } elsif ( $line =~ /Domain User <?(.+?)>? successfully .+/ ) {
            $hash{'domainuser_name'} = $1;
            $hash{'status'}          = "added";
        } else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected output",
                response_object => $response
            );
        }
    }

    return [ \%hash ];
}

## @name ::useradmin_domainuser_delete
##
## @summary Parses the raw output from the "useradmin domainuser delete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_domainuser_delete with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_domainuser_delete(%args);
## INPUTS TO METHOD CALL 'useradmin_domainuser_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'domainuser-name' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_domainuser_delete {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $can_throw  = 0;
    my $first_line = "SID\s+=";
    my %hash;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    foreach my $line (@lines) {
        if ( $line =~ /SID = (.+)/ ) {
            $hash{'sid'} = $1;
        } elsif ( $line =~ /Domain User <?(.+?)>? successfully .+/ ) {
            $hash{'domainuser_name'} = $1;
            $hash{'status'}          = "deleted";
        } else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected output",
                response_object => $response
            );
        }
    }

    return [ \%hash ];
}

## @name ::useradmin_domainuser_list
##
## @summary Parses the raw output from the "useradmin domainuser list" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_domainuser_list with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_domainuser_list(%args);
## INPUTS TO METHOD CALL 'useradmin_domainuser_list' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'group' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_domainuser_list {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $can_throw  = 0;
    my $first_line = "";
    my %hash;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    shift(@lines);
    my @sid;
    foreach my $line (@lines) {
        next if ( $line =~ /For more information .+/ );
        if ( $line =~ /(.+)/ ) {
            push( @sid, $1 );
        } else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected output",
                response_object => $response
            );
        }
    }
    $hash{'sid'} = \@sid;

    return [ \%hash ];
}

## @name ::useradmin_domainuser_load
##
## @summary Parses the raw output from the "useradmin domainuser load" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_domainuser_load with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_domainuser_load(%args);
## INPUTS TO METHOD CALL 'useradmin_domainuser_load' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_domainuser_load {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();

    my @resultarray;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #If it has reached this place then the command has successfully executed.
    #Return an empty array

    return \@resultarray;
}

## @name ::useradmin_diaguser_lock
##
## @summary Parses the raw output from the "useradmin diaguser lock" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_diaguser_lock with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_diaguser_lock(%args);
## INPUTS TO METHOD CALL 'useradmin_diaguser_lock' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_diaguser_lock(%args) {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();

    my @resultarray;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #If it has reached this place then the command has successfully executed.
    #Return an empty array

    return \@resultarray;
}

## @name ::useradmin_diaguser_unlock
##
## @summary Parses the raw output from the "useradmin diaguser unlock" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_diaguser_unlock with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_diaguser_unlock(%args);
## INPUTS TO METHOD CALL 'useradmin_diaguser_unlock' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_diaguser_unlock(%args) {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();

    my @resultarray;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #If it has reached this place then the command has successfully executed.
    #Return an empty array

    return \@resultarray;
}

## @name ::useradmin_diaguser_password
##
## @summary Parses the raw output from the "useradmin diaguser password" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_diaguser_password with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_diaguser_password(%args);
## INPUTS TO METHOD CALL 'useradmin_diaguser_password' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_diaguser_password(%args) {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();

    my @resultarray;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    #If it has reached this place then the command has successfully executed.
    #Return an empty array

    return \@resultarray;
}

## @name ::useradmin_diaguser_show
##
## @summary Parses the raw output from the "useradmin diaguser show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from useradmin_diaguser_show with any of its options as described in the 
##    corresponding cdef file 'useradmin.cdefs'
## @synopsis
##      useradmin_diaguser_show(%args);
## INPUTS TO METHOD CALL 'useradmin_diaguser_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub useradmin_diaguser_show(%args) {
    my %args = @_;

    my $error_message;
    my $response   = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command    = $response->get_actual_command();
    my $can_throw  = 0;
    my $first_line = "";
    my %hash;
    my @resultarray;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command,
        "ptr_error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    foreach my $line (@lines) {
        if ( $line =~ /(.+?):\s*(.*)?/ ) {
            $hash{$1} = $2;
        } else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected output",
                response_object => $response
            );
        }
    }

    push( @resultarray, \%hash );
    return \@resultarray;
}

sub _parse_list {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my $can_throw  = 0;
    my $first_line = "";
    my @returnarray;

    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );

    my ( @Output, $temp_ref, $index );
    foreach my $line (@lines) {
        if ( $line =~ /^Name:/ ) {
            $Output[$index] = [];
            $temp_ref = $Output[ $index++ ];
            push( @$temp_ref, $line );
        } else {
            push( @$temp_ref, $line );
        }
    }

    foreach my $ref (@Output) {
        my @data = @$ref;
        my %hash;
        for ( my $j = 0; $j < @data; $j++ ) {
            if ( $data[$j] =~ /(.+?):\s*(.*)?/ ) {
                $hash{$1} = $2;
            } else {
                my $error
                    = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                $error->throw(
                    text            => "Unexpected output",
                    response_object => $response
                );
            }
        }
        push( @returnarray, \%hash );
    }

    return \@returnarray;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    if (   ( $raw_output =~ /(Error: (User|Group|Role) does not exist)/ )
        || ( $raw_output =~ /(Error: Invalid parameter)/ )
        || ( $raw_output =~ /(Error: User is already present)/ )
        || ( $raw_output =~ /(Error: (Group|Role) already exists)/ )
        || ( $raw_output =~ /(Error: Invalid (Group|role) name)/ )
        || ( $raw_output =~ /(Error: Invalid capability)/ )
        || ( $raw_output
            =~ /(Error: User name contains an invalid character)/ ) )
    {
        $$error_message = $1;
        return "InvalidParamValue";
    } elsif ( ( $raw_output =~ /(Error: CIFS is not started)/ )
        || ( $raw_output =~ /(Could not find file .+)/ )
        || ( $raw_output =~ /(No changes are made to .+)/ )
        || ( $raw_output =~ /(Error: Group is not empty)/ )
        || ( $raw_output =~ /(Error: A group contains this role)/ )
        || ( $raw_output =~ /(Error: Cannot delete .+)/ )
        || ( $raw_output =~ /((Error|ERROR): .+)/ ) )
    {
        $$error_message = $command . " failed. " . $1;
        return "CommandFailed";
    } elsif ( ( $raw_output =~ /(ERROR: New password must be different .+)/ )
        || ( $raw_output =~ /(ERROR: Passwords didn't match)/ )
        || ( $raw_output =~ /(Invalid password .+)/ ) )
    {
        $$error_message = $command . " failed. " . $1;
        return "CommandFailed";
    } elsif ( $raw_output =~ /(U|u)sage:/ ) {
        $$error_message = "Failure may be because of missing required params 
            or invalid param or invalid param value";
        return "CommandFailed";
    } else {
        return undef;
    }
}

1;

