# Hey emacs, this is basically -*- perl -*-
#$Id$
#
#Copyright (c) 2011 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode utest commands
##
## @description
##    This is a library to parse the raw output of 7Mode utest commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "utest run" will be utest_run()
##
## @synopsis
##    use utest;
##
##    utest_run(%args);
##
## @author laurentn@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::utest;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::utest
##
## @summary Parses the raw output from the "utest" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from utest with any of its options as described in the 
##    corresponding cdef file 'utest.cdefs'
## @synopsis
##      utest(%args);
## INPUTS TO METHOD CALL 'utest' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub utest {
    my %args = @_;
    my @result_array;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "utest",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    # Nothing to parse
    return \@result_array;

} ## end sub utest

## @name ::utest_init
##
## @summary Parses the raw output from the "utest init" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from utest_init with any of its options as described in the 
##    corresponding cdef file 'utest.cdefs'
## @synopsis
##      utest_init(%args);
## INPUTS TO METHOD CALL 'utest_init' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub utest_init {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my %result;
    my @result_array;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "utest",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $raw_output =~ /Inited: ([0-9]),\s*sk_pid: (.*)/ ) {
        $result{'inited'} = $1;
        $result{'sk_pid'} = $2;
    } else {
        $error         = "UnexpectedOutput";
        $error_message = "utest init, unrecognized output";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @result_array = ( \%result );
    return \@result_array;

} ## end sub utest_show
## @name ::utest_show
##
## @summary Parses the raw output from the "utest show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from utest_show with any of its options as described in the 
##    corresponding cdef file 'utest.cdefs'
## @synopsis
##      utest_show(%args);
## INPUTS TO METHOD CALL 'utest_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub utest_show {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my %result;
    my @result_array;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "utest",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ( $raw_output =~ /Registered function pointer: (0x[0-9a-f]+)/ ) {
        $result{'ptr'} = $1;
    } else {
        $error         = "UnexpectedOutput";
        $error_message = "utest show, unrecognized output";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @result_array = ( \%result );
    return \@result_array;

} ## end sub utest_show

## @name ::utest_run
##
## @summary Parses the raw output from the "utest run" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from utest_run with any of its options as described in the 
##    corresponding cdef file 'utest.cdefs'
## @synopsis
##      utest_run(%args);
## INPUTS TO METHOD CALL 'utest_run' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr> 
## <tr> <td> '--help' </td> <td> 'help' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug' </td> </tr> 
## <tr> <td> '--help-tests' </td> <td> 'help-tests' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'testsuite' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'testcase' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub utest_run {
    my %args = @_;

    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();

    my %result;
    my @result_array;
    my $found = 0;

    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => "utest",
        "ptr_error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $cmd = $resp->get_actual_command();

    if ( $cmd =~ /--help-tests/ ) { return \@result_array; }
    if ( $cmd =~ /--help/ )       { return \@result_array; }

    if ( $raw_output =~ /Running (\d+) test/ ) {
        $result{'running'} = $1;
        $found += 1;
    }

    if ( $raw_output =~ /Failed (\d+) of (\d+) test/ ) {
        $result{'failed'} = $1;
        $result{'total'}  = $2;
    }

    if ( $raw_output =~ /Success rate: (\d+)%/ ) {
        $result{'rate'} = $1;
        $found += 1;
    } elsif ( $raw_output =~ /OK!\s/ ) {
        $result{'rate'} = 100;
        $found += 1;
    }

    if ( $found != 2 ) {
        $error         = "UnexpectedOutput";
        $error_message = "utest run, unrecognized output";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @result_array = ( \%result );
    return \@result_array;
} ## end sub utest_run

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    if ( $raw_output =~ /$command:\s+(.*)/ ) {
        $$error_message = $1;
        if ( $$error_message =~ /unknown option/ ) {
            $error = "InvalidParam";
        }
    } elsif ( $raw_output =~ /ERROR: (unknown option '.*')/ ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ /ERROR: (unknown suite '.*')/ ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ /ERROR: (unknown test '.*')/ ) {
        $error          = "InvalidParam";
        $$error_message = $1;
    } elsif ( $raw_output =~ /($command .*not found)/ ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } else {
        return undef;
    }
    $$error_message =~ s/\cM//g;
    return $error;
} ## end sub _check_error
1;
