#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/vdom.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode vdom command output
##
## @description
##    This is a library to parse the raw output of 7Mode vdom command.
##
## @synopsis
##
##    vdom_dump(%args);
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::vdom;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::vdom_dump
##
## @summary Parses the raw output from the "vdom dump" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vdom_dump with any of its options as described in the
##    corresponding cdef file 'vdom.cdefs'
## @synopsis
##      vdom_dump(%args);
## INPUTS TO METHOD CALL 'vdom_dump' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-metafile' </td> <td> 'metafile' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'qtree-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-attr-stream' </td> <td> 'attr-stream' </td> <td> '1' </td> </tr>
## <tr> <td> '-lock' </td> <td> 'lock' </td> <td> '1' </td> </tr>
## <tr> <td> '-help' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dsid' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'header' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'lun' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'attributes' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'short-version' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vdom dump -lock
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;DSID = 1110, lockers = 0, waiters = 0, aggr = 0, vol = 1
##&nbsp;DSID = 1112, lockers = 0, waiters = 0, aggr = 1, vol = 1
##&nbsp;DSID = 1113, lockers = 0, waiters = 0, aggr = 1, vol = 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lockers' => '0',
##&nbsp;            'vol' => '1',
##&nbsp;            'waiters' => '0',
##&nbsp;            'dsid' => '1110',
##&nbsp;            'aggr' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lockers' => '0',
##&nbsp;            'vol' => '1',
##&nbsp;            'waiters' => '0',
##&nbsp;            'dsid' => '1112',
##&nbsp;            'aggr' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lockers' => '0',
##&nbsp;            'vol' => '1',
##&nbsp;            'waiters' => '0',
##&nbsp;            'dsid' => '1113',
##&nbsp;            'aggr' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vdom dump -attr_stream -d 1148 -n lun1
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;System attributes for LUN lun1:
##&nbsp;Attribute Version: 1
##&nbsp;UUID: 6601e8ff-6823-11e2-bd88-123478563412
##&nbsp;Cylinder Size(bytes): 1048576
##&nbsp;Extent Size(bytes): 0
##&nbsp;Bytes per sector (logical block size): 512
##&nbsp;Serial Number: BeaR3$BRqP4J
##&nbsp;OS Type: linux
##&nbsp;Administrative state: Online
##&nbsp;Space Alloc: 0
##&nbsp;NVFAIL: 0
##&nbsp;Comment:
##&nbsp;Prev serial number:
##&nbsp;Select Flag: 0
##&nbsp;Class: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'extent_size_bytes' => '0',
##&nbsp;            'nvfail' => '0',
##&nbsp;            'serial_number' => 'BeaR3$BRqP4J',
##&nbsp;            'cylinder_size_bytes' => '1048576',
##&nbsp;            'comment' => undef,
##&nbsp;            'bytes_per_sector_logical_block_size' => '512',
##&nbsp;            'system_attributes_for_lun' => 'lun1',
##&nbsp;            'space_alloc' => '0',
##&nbsp;            'attribute_version' => '1',
##&nbsp;            'class' => '0',
##&nbsp;            'select_flag' => '0',
##&nbsp;            'uuid' => '6601e8ff-6823-11e2-bd88-123478563412',
##&nbsp;            'prev_serial_number' => undef,
##&nbsp;            'administrative_state' => 'Online',
##&nbsp;            'os_type' => 'linux'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vdom dump -metafile  vtoc -d 1147
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;LUN Name: lun_refilled_campaign_1362545726\r\n
##&nbsp;Base fileid: 96\r\n
##&nbsp;Record Contents:\r\n
##&nbsp;      Base generation: 179112767   \r\n
##&nbsp;      Clone parent fileid: 0 \r\n
##&nbsp;      Clone parent generation: 0 \r\n
##&nbsp;      Clone snapid: 0 \r\n
##&nbsp;      Flags: 0\r\n
##&nbsp;\r\n
##&nbsp;LUN Name: lun_insurers_fielders_1362545726\r\n
##&nbsp;Base fileid: 101\r\n
##&nbsp;Record Contents:\r\n
##&nbsp;      Base generation: 179112772   \r\n
##&nbsp;      Clone parent fileid: 0 \r\n
##&nbsp;      Clone parent generation: 0 \r\n
##&nbsp;      Clone snapid: 0 \r\n
##&nbsp;      Flags: 0\r\n
##&nbsp;\r\n
##&nbsp;vdom dump vtoc: found 2 records in 0 msecs\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;  {
##&nbsp;    'record_contents' => [
##&nbsp;                           {
##&nbsp;                             'clone_snapid' => '0',
##&nbsp;                             'clone_parent_generation' => '0',
##&nbsp;                             'base_generation' => '179112767',
##&nbsp;                             'flags' => '0',
##&nbsp;                             'clone_parent_fileid' => '0'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;    'lun_name' => 'lun_refilled_campaign_1362545726',
##&nbsp;    'base_fileid' => ' 96'
##&nbsp;  },
##&nbsp;  {
##&nbsp;    'record_contents' => [
##&nbsp;                           {
##&nbsp;                             'clone_snapid' => '0',
##&nbsp;                             'clone_parent_generation' => '0',
##&nbsp;                             'base_generation' => '179112772',
##&nbsp;                             'flags' => '0',
##&nbsp;                             'clone_parent_fileid' => '0'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;    'lun_name' => 'lun_insurers_fielders_1362545726',
##&nbsp;    'base_fileid' => ' 101'
##&nbsp;  }
##&nbsp;];
##&nbsp;
##&nbsp;Command = vdom dump -metafile  vtoc -d 1147 -h 1
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;VTOC Header Contents
##&nbsp;---------------------
##&nbsp;Magic Number:   1668248694
##&nbsp;Major Version:  3
##&nbsp;Minor Version:  0
##&nbsp;LUN Name: lun1
##&nbsp;Base fileid: 96
##&nbsp;Record Contents:
##&nbsp;Base generation: 293174713
##&nbsp;Clone parent fileid: 0
##&nbsp;Clone parent generation: 0
##&nbsp;Clone snapid: 0
##&nbsp;Flags: 0
##&nbsp;LUN Name: lun_insurers_fielders_1362545726\r\n
##&nbsp;Base fileid: 101\r\n
##&nbsp;Record Contents:\r\n
##&nbsp;      Base generation: 179112772   \r\n
##&nbsp;      Clone parent fileid: 0 \r\n
##&nbsp;      Clone parent generation: 0 \r\n
##&nbsp;      Clone snapid: 0 \r\n
##&nbsp;      Flags: 0\r\n
##&nbsp;vdom dump vtoc: found 1 records in 19 msecs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lun_name' => 'lun1',
##&nbsp;            'record_contents' => [
##&nbsp;                                   {
##&nbsp;                                     'clone_snapid' => '0',
##&nbsp;                                     'flags' => '0',
##&nbsp;                                     'clone_parent_generation' => '0',
##&nbsp;                                     'base_generation' => '293174713',
##&nbsp;                                     'clone_parent_fileid' => '0'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'vtoc_header_contents' => [
##&nbsp;                                        {
##&nbsp;                                          'minor_version' => '0',
##&nbsp;                                          'major_version' => '3',
##&nbsp;                                          'magic_number' => '1668248694'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'base_fileid' => '96',
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vtoc_header_contents' => [
##&nbsp;                              $VAR1->[0]{'vtoc_header_contents'}[0]
##&nbsp;                            ],
##&nbsp;            'record_contents' => [
##&nbsp;                         {
##&nbsp;                           'clone_snapid' => '0',
##&nbsp;                           'clone_parent_generation' => '0',
##&nbsp;                           'base_generation' => '179112772',
##&nbsp;                           'flags' => '0',
##&nbsp;                           'clone_parent_fileid' => '0'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'lun_name' => 'lun_insurers_fielders_1362545726',
##&nbsp;            'base_fileid' => ' 101'
##&nbsp;        }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vdom dump -metafile  vtoc -d 1147 -s 1 -h 1
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;VTOC Header Contents
##&nbsp;---------------------
##&nbsp;Magic Number:   1668248694
##&nbsp;Major Version:  3
##&nbsp;Minor Version:  0
##&nbsp;lun1:
##&nbsp;Base (fileid, generation): (96, 293174713) clone_parent (fileid, generation): (0 0) clone snapid 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'clone_snapid' => '0',
##&nbsp;            'base' => [
##&nbsp;                        {
##&nbsp;                          'generation' => '293174713',
##&nbsp;                          'fileid' => '96'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'lun' => 'lun1',
##&nbsp;            'clone_parent' => [
##&nbsp;                                {
##&nbsp;                                  'generation' => '0',
##&nbsp;                                  'fileid' => '0'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vdom dump -metafile oovc -d 1147 -h 1
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;OOVC Header Contents
##&nbsp;---------------------
##&nbsp;Magic Number:   1668706159
##&nbsp;Major Version:  1
##&nbsp;Minor Version:  0
##&nbsp;Volume Context: 0
##&nbsp;Volume Content Change#: 0
##&nbsp;Volume Context flags:   0
##&nbsp;LUN Name: lun1
##&nbsp;Base fileid: 96
##&nbsp;OOVC Contents:
##&nbsp;Base generation: 293174713
##&nbsp;Serial number: BeaR3$BRqP4I
##&nbsp;Administrative state: Online
##&nbsp;Metadb offset: 4096
##&nbsp;vdom dump oovc: found 1 records in 0 msecs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lun_name' => 'lun1',
##&nbsp;            'base_fileid' => '96',
##&nbsp;            'oovc_contents' => [
##&nbsp;                                 {
##&nbsp;                                  'metadb_offset' => '4096',
##&nbsp;                                  'base_generation' => '293174713',
##&nbsp;                                  'administrative_state' => 'Online',
##&nbsp;                                  'serial_number' => 'BeaR3$BRqP4I'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'oovc_header_contents' => [
##&nbsp;                                        {
##&nbsp;                                         'volume_context' => '0',
##&nbsp;                                         'volume_context_flags' => '0',
##&nbsp;                                         'volume_content_change' => '0',
##&nbsp;                                         'minor_version' => '0',
##&nbsp;                                         'major_version' => '1',
##&nbsp;                                         'magic_number' => '1668706159'
##&nbsp;                                        },
##&nbsp;                                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vdom dump metadb -d 1147 -h
##&nbsp;
##&nbsp;-------- Raw Output --------- ;
##&nbsp;Bitmask Header contents
##&nbsp;-----------------------
##&nbsp;Magic Number: 1635018093
##&nbsp;Major Version: 1
##&nbsp;Minor Version: 0
##&nbsp;Bitmask
##&nbsp;Bitmask[0]: 1
##&nbsp;Metadb chunk header contents
##&nbsp;----------------------------
##&nbsp;(Base fileid, Base generation): (96, 293174713) metadb offset:  4096
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bitmask_header_contents' => [
##&nbsp;                                           {
##&nbsp;                                             'bitmask' => [
##&nbsp;                                                            {
##&nbsp;                                                              'bitmask0' => '1'
##&nbsp;                                                            }
##&nbsp;                                                          ],
##&nbsp;                                             'minor_version' => '0',
##&nbsp;                                             'major_version' => '1',
##&nbsp;                                             'magic_number' => '1635018093'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'metadb_chunk_header_contents' => [
##&nbsp;                                                {
##&nbsp;                                                  'metadb_offset' => '4096'
##&nbsp;                                                  'base_generation' => '293174713',
##&nbsp;                                                  'base_fileid' => '96'
##&nbsp;                                                }
##&nbsp;                                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub vdom_dump {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    if ( $cmd =~ /-lock/ ) {
        my ( @head, @values );
        my $count = 0;
        foreach my $line (@lines) {
            my @head = split( /,\s+/, $line );
            for ( my $i = 0 ; $i <= $#head ; $i++ ) {
                my @values = split( /=/, $head[$i] );
                trim( 'string_ptr' => \$values[0] );
                $result_array[$count]->{ to_key_name( item => $values[0] ) } =
                  $values[1];
            }
            $count++;
        }
    } elsif ( $cmd =~ /-attr_stream/ ) {
        my @values;
        my $count = -1;
        foreach my $line (@lines) {
            if ( $line =~ /(System\s+attributes\s+for\s+LUN)\s+(.*)[:]/ ) {
                $count++;
                $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
            } else {
                @values = split( /:\s*/, $line );
                for ( my $i = 0 ; $i <= $#values ; $i = $i + 2 ) {
                    $values[$i] =~ s/\(|\s\(/_/;
                    $values[$i] =~ s/\)//;
                    $result_array[$count]
                      ->{ to_key_name( item => $values[$i] ) } =
                      $values[ $i + 1 ];
                }
            }
        }
    } elsif ( $cmd =~ /-metafile/ ) {
        my ( @values, $head1, $head2, $head );
        my $count  = -1;
        my $count1 = -1;
        my %hash;
        if ( $cmd =~ /vtoc|oovc/ ) {
            foreach my $line (@lines) {
                if ( $line =~
                    /(VTOC\s+Header\s+Contents|OOVC\s+Header\s+Contents)/ ){
                    $count1++;
                    $head = $1;
                } elsif ( $line =~ /Magic.*|Major.*|Minor.*|Volume.*/ ) {
                    @values = split( /:\s*/, $line );
                    $values[0] =~ s/#//;
                    $hash{ to_key_name( item =>$values[0])} = $values[1];
                } elsif ( $line =~ /^(\w+):$/ ) {
                    $count++;
                    $result_array[$count]->{lun} = $1;
                    $result_array[$count]->{ to_key_name( item =>$head) }->[$count1] = \%hash if( %hash );
                } elsif ( $line =~
/Base\s+\((fileid)\,\s+(generation)\)\:\s+\((\d+)\,\s+(\d+)\)\s+clone_parent\s+\((fileid)\,\s+(generation)\)\:\s+\((\d+)\s+(\d+)\)\s+(clone\s+snapid)\s+(\d+)/){
                    $result_array[$count]->{base}->[0]
                      ->{ to_key_name( item => $1 ) } = $3;
                    $result_array[$count]->{base}->[0]
                      ->{ to_key_name( item => $2 ) } = $4;
                    $result_array[$count]->{clone_parent}->[0]
                      ->{ to_key_name( item => $5 ) } = $7;
                    $result_array[$count]->{clone_parent}->[0]
                      ->{ to_key_name( item => $6 ) } = $8;
                    $result_array[$count]->{ to_key_name( item => $9 ) } = $10;
                } elsif ( $line =~
/LUN\s+Name\:\s+(\w+)\s+\((Base\s+fileid)\,\s+(Base\s+generation)\)\:\s+\((\d+)\,\s+(\d+)\)/){
                    $result_array[0]->{ to_key_name( item => $1 ) }[0]
                      ->{ to_key_name( item => $2 ) } = $4;
                    $result_array[0]->{ to_key_name( item => $1 ) }[0]
                      ->{ to_key_name( item => $3 ) } = $5;
                } elsif ( $line =~ /LUN\s+Name.*/ ) {
				    $count++;
                    @values = split( /:\s*/, $line );
                    $result_array[$count]
                      ->{ to_key_name( item => $values[0] ) } = $values[1];
                    $result_array[$count]->{ to_key_name( item =>$head) }->[$count1] = \%hash if( %hash );
                } elsif ( $line =~ /(Record\s+Contents|OOVC\s+Contents)[:]/ ) {
                    $head2 = $1;
                } elsif ( $line =~ /(Base\s+fileid):(.*)/ ) {
                    $result_array[$count]->{ to_key_name( item =>$1) } = $2;
                } elsif ( $line !~ /-----------------.*/ && $line !~ /found.*records\s+in.*msecs/ ) {
                    @values = split( /:\s*/, $line );
                    for ( my $i = 0 ; $i <= $#values ; $i = $i + 2 ) {
                        $result_array[$count]->{ to_key_name( item => $head2 ) }
                          ->[0]->{ to_key_name( item => $values[$i] ) } =
                          $values[ $i + 1 ];
                    }
                }
            }
        } elsif ( $cmd =~ /metadb/ ) {
            my $limit  = -1;
            my $count1 = -1;
            my $count = 0;
            foreach my $line (@lines) {
                if ( $line =~
/(Bitmask\s+Header\s+contents|Metadb\s+chunk\s+header\s+contents)/){
                    $limit++;
                    $head = $1;
                } elsif ( $line =~ /Magic.*|Major.*|Minor.*/ ) {
                    @values = split( /:\s*/, $line );
                    $result_array[0]->{ to_key_name( item => $head ) }
                      ->[$limit]->{ to_key_name( item => $values[0] ) } =
                      $values[1];
                } elsif ( $line =~ /^(Bitmask)$/ ) {
                    $count1++;
                    $head1 = $1;
                } elsif ( $line =~ /(Bitmask)\[(\d+)\][:]\s+(\d+)/ ) {
                    $result_array[0]->{ to_key_name( item => $head ) }
                      ->[$limit]->{ to_key_name( item => $head1 ) }[ $count1++ ]
                      ->{ to_key_name( item => ( $1 . "" . $2 ) ) } = $3;
                } elsif ( $line =~
/(Base\s+fileid)\,\s+(Base\s+generation)\)\:\s+\((\d+)\,\s+(\d+)\)\s+(metadb\s+offset)[:]\s+(\d+)/){
                    $result_array[0]->{ to_key_name( item => $head ) }[$count]
                      ->{ to_key_name( item => $1 ) } = $3;
                    $result_array[0]->{ to_key_name( item => $head ) }[$count]
                      ->{ to_key_name( item => $2 ) } = $4;
                    $result_array[0]->{ to_key_name( item => $head ) }[$count]
                      ->{ to_key_name( item => $5 ) } = $6;
                    $count++;
                }
            }
        }
    }
    return \@result_array;
} ## end sub vdom_status

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /Volume\s+not\s+found|No\s+such\s+LUN\s+exists/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~
/Dumps\s+contents\s+of\s+Virtual\s+Disk\s+Object\s+Manager|option\s+requires\s+an\s+argument.*/
      )
    {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Usage:.*|Invalid\s+option/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

