#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode version commands
##
## @description
##     This is a library to parse the raw output of 7Mode version commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use version;
##
##    version(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::version;

use Data::Dumper;
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors(%) {
    my %args         = @_;
    my $raw_output   = $args{'raw_output'};
    my $command      = $args{'command'};
    my $errormsg_ptr = $args{'errormsg_ptr'};

    if ( $raw_output =~ /Usage: illegal argument:/ ) {
        $$errormsg_ptr = $command . " contains invalid param value(s).";
        return "InvalidParamValue";
    }
    return undef;
}

## @name ::version
##
## @summary Parses the raw output from the "version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from version with any of its options as described in the
##    corresponding cdef file 'version.cdefs'
## @synopsis
##      version(%args);
## INPUTS TO METHOD CALL 'version' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'boot' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; version; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NetApp Release main_ib_bibN_120821_1600 7-Mode: Tue Aug 21 16:28:28 PDT 2012   <1>
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ontap_version' => 'NetApp Release main_ib_bibN_120821_1600 7-Mode',
##&nbsp;            'date_created' => 'Tue Aug 21 16:28:28 PDT 2012',
##&nbsp;            'build_info' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; version -b; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;/cfcard/x86_64/freebsd/image1/kernel: OS /x/eng/rlse63/DOT/nightly/main_ib_bibN_120821_1600/
##&nbsp;/cfcard/x86_64/freebsd/image2/kernel: OS /x/eng/rlse63/DOT/nightly/main_ib_bibN_120821_1600/
##&nbsp;/cfcard/x86_64/diag/diag.krn:  5.7
##&nbsp;/cfcard/x86_64/firmware/excelsio/firmware.img: Firmware 1.10X3
##&nbsp;/cfcard/x86_64/firmware/DrWho/firmware.img: Firmware 2.6X3
##&nbsp;/cfcard/x86_64/firmware/SB_XV/firmware.img: Firmware 4.5X3
##&nbsp;/cfcard/x86_64/firmware/SB_XVI/firmware.img: Firmware 5.2X4
##&nbsp;/cfcard/x86_64/firmware/SB_XVIII/firmware.img: Firmware 7.2X3
##&nbsp;/cfcard/x86_64/firmware/SB_XIX/firmware.img: Firmware 8.1.0
##&nbsp;/cfcard/common/firmware/zdi/zdi_fw.zpk: Flash Cache 2 CPLD 0.0 (Build 0xd)
##&nbsp;/cfcard/common/firmware/zdi/zdi_fw.zpk: Flash Cache 2 FPGA 3.0 (Build 0x201207182256)
##&nbsp;/cfcard/common/firmware/zdi/zdi_fw.zpk: Flash Cache Firmware 2.3 (Build 0x201204251259)
##&nbsp;/cfcard/common/firmware/zdi/zdi_fw.zpk: PAM II Firmware 1.11 (Build 0x201204251243)
##&nbsp;/cfcard/common/firmware/zdi/zdi_fw.zpk: X1936A FPGA Configuration PROM 1.0 (Build 0x200706131558)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/freebsd/image1/kernel',
##&nbsp;            'ontap_version' => 'OS /x/eng/rlse63/DOT/nightly/main_ib_bibN_120821_1600/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/freebsd/image2/kernel',
##&nbsp;            'ontap_version' => 'OS /x/eng/rlse63/DOT/nightly/main_ib_bibN_120821_1600/'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/diag/diag.krn',
##&nbsp;            'diag_version' => '5.7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/excelsio/firmware.img',
##&nbsp;            'firmware_version' => '1.10X3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/DrWho/firmware.img',
##&nbsp;            'firmware_version' => '2.6X3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/SB_XV/firmware.img',
##&nbsp;            'firmware_version' => '4.5X3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/SB_XVI/firmware.img',
##&nbsp;            'firmware_version' => '5.2X4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/SB_XVIII/firmware.img',
##&nbsp;            'firmware_version' => '7.2X3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/x86_64/firmware/SB_XIX/firmware.img',
##&nbsp;            'firmware_version' => '8.1.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/common/firmware/zdi/zdi_fw.zpk',
##&nbsp;            'firmware_version' => 'Flash Cache 2 CPLD 0.0 (Build 0xd)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/common/firmware/zdi/zdi_fw.zpk',
##&nbsp;            'firmware_version' => 'Flash Cache 2 FPGA 3.0 (Build 0x201207182256)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/common/firmware/zdi/zdi_fw.zpk',
##&nbsp;            'firmware_version' => 'Flash Cache Firmware 2.3 (Build 0x201204251259)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/common/firmware/zdi/zdi_fw.zpk',
##&nbsp;            'firmware_version' => 'PAM II Firmware 1.11 (Build 0x201204251243)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'info' => '/cfcard/common/firmware/zdi/zdi_fw.zpk',
##&nbsp;            'firmware_version' => 'X1936A FPGA Configuration PROM 1.0 (Build 0x200706131558)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub version(%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( @return_array, @values );
    my $count     = 0;
    my $can_throw = 0;
    my $error;
    my $errormsg;
    my $first_line = "";

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( ( $command =~ /^version$/ ) || ( $command =~ /-v$/ ) ) {
        my $line = shift(@lines);
        if ( $line =~ /:/ ) {
            my @fields = split( /:/, $line, 2 );
            my %hash;
            trim( 'string_ptr' => \$fields[0] );
            $fields[1] =~ s/<(.*)>//;
            $hash{'build_info'} = $1;
            trim( 'string_ptr' => \$fields[1] );
            $hash{'ontap_version'} = $fields[0];
            $hash{'date_created'}  = $fields[1];
            push( @return_array, \%hash );
            return ( \@return_array );
        }
        else {
            $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected form of output",
                response_object => $resp
            );
        }
    }
    elsif ( $command =~ /-b$/ ) {
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /image1/ ) {
                @values = split( ':\s+', $line );
                $return_array[$count]->{'info'}          = $values[0];
                $return_array[$count]->{'ontap_version'} = $values[1];
                $count++;
            }
            elsif ( $line =~ /image2/ ) {
                @values = split( ':\s+', $line );
                $return_array[$count]->{info}          = $values[0];
                $return_array[$count]->{ontap_version} = $values[1];
                $count++;
            }
            elsif ( $line =~ /diag/ ) {
                @values = split( ':\s+', $line );
                $return_array[$count]->{info}         = $values[0];
                $return_array[$count]->{diag_version} = $values[1];
                $count++;
            }
            elsif ( $line =~ /firmware/ ) {
                unshift( @lines, $line );
                while ( $line = shift(@lines) ) {
                    @values = split( ':\s+', $line );
                    $return_array[$count]->{info}             = $values[0];
                    if ( $values[1] =~ /Firmware (\d+.*)/ ) {
                         $return_array[$count]->{firmware_version} = $1;
                         $count++;
                    }
                    else {
                        $return_array[$count]->{firmware_version} = $values[1];
                        $count++;
                    }
                }
            }
        }

        return ( \@return_array );
    }
    else {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected form of output",
            response_object => $resp
        );
    }
}

1;
