#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode vfiler commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode vfiler commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vfiler create" will be vfiler_create()
##
## @synopsis
##    use vfiler;
##
##    vfiler_create(%args);
##    vfiler_status(%args);
##    vfiler_limit(%args);
##    vfiler_allow(%args);
##    vfiler_disallow(%args);
##    vfiler_rename(%args);
##    vfiler_start(%args);
##    vfiler_stop(%args);
##    vfiler_destroy(%args);
##    vfiler_add(%args);
##    vfiler_remove(%args);
##    vfiler_move(%args);
##    vfiler_dr_status(%args);
##    vfiler_migrate_status(%args);
##    vfiler_run(%args);
##    vfiler_migrate_complete(%args);
##    vfiler_migrate_cancel(%args);
##    vfiler_migrate_start(%args);
##    vfiler_migrate(%args);
##    vfiler_dr_resync(%args);
##    vfiler_dr_activate(%args);
##    vfiler_dr_delete(%args);
##    vfiler_dr_configure(%args);
##    vfiler_context(%args);
##
## @author tanil@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::vfiler;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);

## @name ::vfiler_create
##
## @summary Parses the raw output from the "vfiler create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_create with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_create(%args);
## INPUTS TO METHOD CALL 'vfiler_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'storage-units' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'bypass-setup' </td><td> '1' </td></tr>
## <tr> <td> '-b' </td> <td> 'old-vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'recreate-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'ipspace' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-addresses' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_create {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_create(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;
}

## @name ::vfiler_status
##
## @summary Parses the raw output from the "vfiler status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_status with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_status(%args);
## INPUTS TO METHOD CALL 'vfiler_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all-details' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'ip-and-storage-details' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_status {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_status(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;
}

## @name ::vfiler_limit
##
## @summary Parses the raw output from the "vfiler limit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_limit with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_limit(%args);
## INPUTS TO METHOD CALL 'vfiler_limit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'maxvfilers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_limit {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_limit(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;
}

## @name ::vfiler_allow
##
## @summary Parses the raw output from the "vfiler allow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_allow with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_allow(%args);
## INPUTS TO METHOD CALL 'vfiler_allow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'protocols' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_allow {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_allow_or_disallow(raw_output=>$raw_output,
                        command=>$command_line, response_object=>$response);
    return $result;
}

## @name ::vfiler_disallow
##
## @summary Parses the raw output from the "vfiler disallow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_disallow with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_disallow(%args);
## INPUTS TO METHOD CALL 'vfiler_disallow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'protocol' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'protocols' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_disallow {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_allow_or_disallow(raw_output=>$raw_output,
                            command=>$command_line, response_object=>$response);
    return $result;
}

## @name ::vfiler_rename
##
## @summary Parses the raw output from the "vfiler rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_rename with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_rename(%args);
## INPUTS TO METHOD CALL 'vfiler_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-vfiler-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_rename {
    my %args         = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_rename(raw_output=>$raw_output,
                            command=>$command_line, response_object=>$response);
    return $result;
}

## @name ::vfiler_start
##
## @summary Parses the raw output from the "vfiler start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_start with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_start(%args);
## INPUTS TO METHOD CALL 'vfiler_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_start {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_start(raw_output=>$raw_output, command=>$command_line,
                            response_object=>$response);
    return $result;
}

## @name ::vfiler_stop
##
## @summary Parses the raw output from the "vfiler stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_stop with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_stop(%args);
## INPUTS TO METHOD CALL 'vfiler_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_stop {
    my %args         = @_;
    my $error_message;

    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_stop(raw_output=>$raw_output, command=>$command_line,
                            response_object=>$response);
    return $result;
}

## @name ::vfiler_destroy
##
## @summary Parses the raw output from the "vfiler destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_destroy with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_destroy(%args);
## INPUTS TO METHOD CALL 'vfiler_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_destroy {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_destroy(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;
}

## @name ::vfiler_add
##
## @summary Parses the raw output from the "vfiler add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_add with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_add(%args);
## INPUTS TO METHOD CALL 'vfiler_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'storage-units' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-addresses' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_add {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);

    # Though we observed cases with partial success in vfiler add (For eg,
    # Trying to add a valid IP address together with an invalid storage path
    # results in IP address getting added together but an error for invalid
    # path), its not coded to populate the exception object with the partial
    # parsed output because for addition of IP address, there will be no
    # visible output returned by the command.
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_add(raw_output=>$raw_output, command=>$command_line,
                            response_object=>$response);
    return $result;
}

## @name ::vfiler_remove
##
## @summary Parses the raw output from the "vfiler remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_remove with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_remove(%args);
## INPUTS TO METHOD CALL 'vfiler_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'storage-units' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-addresses' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_remove {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;
    my $throw = 1;
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);
    # Look for error messages which are observed on partial success
    # If any such cases noticed, we populate the response object with the
    # parsed output before throwing it.
    if (defined($error_message) && ($error_message=~ /Check path/)) {
        $throw = 0;
    }
    if($error && $throw) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object=>$response);
    }
    # Now we can parse the output
    $result = _vfiler_remove_or_move(raw_output=>$raw_output,
                            command=>$command_line, response_object=>$response);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $response->set_parsed_output( parsed_output => $result );
        $error->throw( text => $error_message, response_object => $response );
    }
    return $result;
}

## @name ::vfiler_move
##
## @summary Parses the raw output from the "vfiler move" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_move with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_move(%args);
## INPUTS TO METHOD CALL 'vfiler_move' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'storage-units' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler-to' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ip-addresses' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vfiler_move {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;
    my $throw = 1;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler",
                             "ptr_error_message"=>\$error_message);

    # Look for error messages which are observed on partial success
    # If any such cases noticed, we populate the response object with the
    # parsed output before throwing it.
    if (defined($error_message) && ($error_message=~ /Check path/)) {
        $throw = 0;
    }
    if($error && $throw) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object=>$response);
    }
    # Now we can parse the output
    $result = _vfiler_remove_or_move( %args );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $response->set_parsed_output( parsed_output => $result );
        $error->throw( text => $error_message, response_object => $response );
    }
    return $result;
}


## @name ::vfiler_dr_status
##
## @summary Parses the raw output from the "vfiler dr status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_dr_status with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_dr_status(%args);
## INPUTS TO METHOD CALL 'vfiler_dr_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_dr_status {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler dr status",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_dr_status(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;
}

## @name ::vfiler_migrate_status
##
## @summary Parses the raw output from the "vfiler migrate status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_migrate_status with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_migrate_status(%args);
## INPUTS TO METHOD CALL 'vfiler_migrate_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples


sub vfiler_migrate_status {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler migrate status",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_dr_status(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;

}
## @name ::vfiler_owner
##
## @summary Parses the raw output from the "vfiler owner" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_owner with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_owner(%args);
## INPUTS TO METHOD CALL 'vfiler_owner' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; vfiler owner /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'path' => ' /vol/vol0',
##&nbsp;            'filer' => 'vfiler0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vfiler_owner {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler owner",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output

    $result = _vfiler_owner(raw_output=>$raw_output, command=>$command_line,
                                response_object=>$response);
    return $result;

}


## @name ::vfiler_run
##
## @summary Parses the raw output from the "vfiler run" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_run with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_run(%args);
## INPUTS TO METHOD CALL 'vfiler_run' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-q' </td> <td> 'remove-seperator' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'command-args' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_run {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler run",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

sub _vfiler_owner {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (@result_array);
    my $can_throw = 1;
    my @arr = get_lines("raw_output"=>$raw_output,
        "can_throw"=>$can_throw, "response_object"=>$response);
	my %hash;
    if ($arr[0] =~ /(.+):(.+)/) {
		$hash{'filer'} = $1;
        $hash{'path'} = $2;
    }
    push @result_array, \%hash;
    return \@result_array;
}

sub _vfiler_dr_status {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (@result_array);
    my ($key,$value);
    my $can_throw = 1;

    my $first_line = 'Storage';
    my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
    foreach (@arr) {
    	my %hash;
        if ( $_ =~ /(Storage\sPath)\s+(Status)/ ) {
            $key   = to_key_name( item => $1 );
            $value = lc($2);
#            $key =~ s/\s/_/g;
            next;
        }
        ($hash{$key},$hash{$value}) = split /\s+/,$_;
        push @result_array , \%hash;
    }
    return \@result_array;
}


sub _vfiler_create {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my @result_array;

    if ( $raw_output =~ /ipspace:/ ) {

        # This vfiler is recreated from the old one.

        # For vfiler create -r form, some output is always expected.
        # So we are making $can_throw as 1 so that if the
        # lines array got finally in _get_lines is empty,
        # UnexpectedOutputException can be thrown there.
        my $can_throw = 1;

        my $some_state = _get_vfiler_states();
        my $first_line = "[\\w-]+\\s+($some_state)(,\\s($some_state))*";
        my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
            "can_throw"=>$can_throw, "response_object"=>$response);

        # Get the first line for later processing
        my $line = shift(@arr);

        my @Vfiler = ( \@arr );
        my $parsed_array = _parse_vfiler_output(@Vfiler);
        my $parsed = @{$parsed_array}[0];

        $line =~ /([\w-]+)\s+(.+)/;
        $parsed->{"vfiler"} = $1;
        $parsed->{"state"} = $2;
        $parsed->{"status"} = "created";

        $raw_output =~ /The etc configuration directory for vfiler \"[\w-]+\" is (\/\w+\/\w+\/\w+.*)/;
        $parsed->{"configuration_directory"} = $1;
        $parsed->{"configuration_directory"} =~ s/\.\r*\n*$//;
        push(@result_array, $parsed);
        return \@result_array;
    } else {
        # This vfiler is created with user supplied configuration
        if ( $raw_output =~ /The etc configuration directory for vfiler \"([\w-]+)\" is (\/\w+\/\w+\/\w+.*)/) {
            my %hash;
            $hash{"vfiler"} = $1;
            $hash{"configuration_directory"} = $2;
            $hash{"configuration_directory"} =~ s/\.\r*\n*$//;
            $hash{"status"} = "created";
            push(@result_array, \%hash);
            return \@result_array;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text => "Output is not in the expected format",
                response_object => $response );
        }
    }
}

sub _vfiler_status {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    # For vfiler status, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my $some_state = _get_vfiler_states();
    my $first_line = "[\\w-]+\\s+($some_state)(,\\s($some_state))*";
    my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
    my @command_words = split( /\s+/, $command );
    if ( ( grep( "-a", @command_words ) ) || ( grep( "-r", @command_words ) ) ) {
        my $Vfiler_index = 0;
        my ( @Vfiler, $temp_ref );
        for ( my $i = 0 ; $i < @arr ; $i++ ) {
            next if ( $arr[$i] eq "" );
            if ( $arr[$i] =~ /^[\w-]+\s+$some_state(,\s$some_state)*/ ) {
                # The next logical block has started
                $Vfiler[$Vfiler_index] = [];
                $temp_ref = $Vfiler[ $Vfiler_index++ ];
                push( @$temp_ref, $arr[$i] );
            } else {
                push( @$temp_ref, $arr[$i] );
            }
        }
        my $parsed = _parse_vfiler_output(@Vfiler);
        return $parsed;
    }
    else {

        # vfiler status is called with no options
        my @result_array;
        foreach (@arr) {
            if ( $_ =~ /[\w-]+\s+$some_state(,\s$some_state)*/ ) {
                my %hash;
                $_ =~ /([\w-]+)\s+(.+)/;
                $hash{"vfiler"} = $1;
                $hash{"status"} = $2;
                push( @result_array, \%hash );
            } else {
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text => "Output is not in the expected format",
                    response_object => $response );
            }
        }
        return \@result_array;
    }
}

sub _vfiler_limit {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%hash, @result_array);
    if ( $raw_output =~ /(Increasing|Decreasing)\sthe\slimit\sfor\smaximum/ ) {
        # The command had been run in set context.
        $hash{"status"} = $1;
        $raw_output =~ /this\shost\sfrom\s(\d+)\sto\s(\d+)/;
        $hash{"old_limit"} = $1;
        $hash{"new_limit"} = $2;
        $hash{"status"} =~ s/ing$/ed the limit/;
        push(@result_array, \%hash);
        return \@result_array;
    }

    # The command has got run in get context

    # For vfiler limit, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my $first_line = 'Current\slimit:';
    my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
        "can_throw"=>$can_throw, "response_object"=>$response);
    foreach (@arr) {
        if ( $_ =~ /(.+):\s+(.+)/ ) {
            my $key   = lc($1);
            my $value = $2;
            $key =~ s/\s/_/g;
            if ( $_ =~ /^Current\slimit:\s+(\d+)\s?(\(?(\d+)\safter\sreboot\)?)?/ ) {
                $hash{"current_limit"} = $1;
                if (defined $2) {
                    $hash{"limit_after_reboot"} = $3;
                }
                next;
            }
            $hash{$key} = $value;
        }
        else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text => "Output is not in the expected format",
                response_object => $response );
        }
    }
    push(@result_array, \%hash);
    return \@result_array;
}

sub _vfiler_allow_or_disallow {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my @result_array;

    if ( $raw_output =~ /Protocols\s(allowed|disallowed):/ ) {
     # For vfiler allow/disallow in get context, some output is always expected.
     # So we are making $can_throw as 1 so that if the lines array got finally
     # in _get_lines is empty, UnexpectedOutputException can be thrown there.
        my $can_throw = 1;

        my $first_line = '[\w-]+\s*\r*\n*Protocols\s(allowed|disallowed):';
        my @arr = get_lines("raw_output"=>$raw_output, "first_line"=>$first_line,
            "can_throw"=>$can_throw, "response_object"=>$response);
        my $Vfiler_index = 0;
        my ( @Vfiler, $temp_ref );
        for ( my $i = 0 ; $i < @arr ; $i++ ) {
            next if ( $arr[$i] eq "" );
            if ( $arr[$i] =~ /^[\w-]+$/ ) {
                # The next logical block has started
                $Vfiler[$Vfiler_index] = [];
                $temp_ref = $Vfiler[ $Vfiler_index++ ];
                push( @$temp_ref, $arr[$i] );
            } else {
                push( @$temp_ref, $arr[$i] );
            }
        }
        my $parsed = _parse_vfiler_output(@Vfiler);
        return $parsed;
    } else {
    # Command is invoked in set context
        # No output will be got in set context and since no error condition is
        # detected, it can be taken as the command has ran successfully
        # allowing or disallowing some protocols.
        my %hash;
        $hash{"status"} = "Allowed the protocol(s)" if ( $command =~ /^vfiler allow/ );
        $hash{"status"} = "Disallowed the protocol(s)" if ( $command =~ /^vfiler disallow/ );
        push(@result_array, \%hash);
        return \@result_array;
    }
}

sub _vfiler_rename {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my $result;

    if ( $raw_output =~ /vfiler rename complete/ ) {
        $result = [{"status" => "Renamed"}];
        return $result;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response );
    }
}

sub _vfiler_start {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%hash, @result_array);

    if ( $raw_output =~ /([\w-]+)\s+running/ ) {
        $hash{"vfiler"} = $1;
        $hash{"status"} = "Running";
        push(@result_array, \%hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response );
    }
}

sub _vfiler_stop {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%hash, @result_array);

    if ( $raw_output =~ /([\w-]+)\s+stopped/ ) {
        $hash{"vfiler"} = $1;
        $hash{"status"} = "Stopped";
        push(@result_array, \%hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response );
    }
}

sub _vfiler_destroy {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%hash,@result_array);

    if ( $raw_output =~ /Vfiler ([\w-]+) removed/ ) {
        $hash{"vfiler"} = $1;
        $hash{"status"} = "Removed";
        push(@result_array, \%hash);
        return \@result_array;
    } elsif ( $raw_output =~ /Resources for vfiler ([\w-]+) moved to hosting filer/ ) {
        $hash{"vfiler"} = $1;
        $hash{"status"} =
            "Destroyed and the resources are moved to hosting filer";
        push(@result_array, \%hash);
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "Output is not in the expected format",
            response_object => $response );
    }
}

sub _vfiler_add {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};

    # TODO: Currently vfiler add command is not giving error on
    # MissingRequiredParams like this
    # isshwar-vsim2> vfiler add vfiler_name -i
    # isshwar-vsim2>
    # This should be fixed by the vfiler team to make the following
    # status true all the times.

    # For addition of IP addresses, no output is observed.
    # For addition of storage, Warning message will be given if -f
    #     option is not given.
    # For addition of both IP address and storage, only warning for
    #     addition of storage is given.
    # From the warning, we cannot differentiate whether its addition of
    # storage alone or addition of both.

    # Hence status tells simply that resources are added assuming that
    # the command had succeeded since no exceptions are thrown by
    # _check_error().
    return [];
}

sub _vfiler_remove_or_move {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%hash, @result_array);

    # TODO: Currently vfiler remove/move command is not giving error on
    # MissingRequiredParams like this
    # isshwar-vsim2> vfiler remove vfiler_name -i
    # isshwar-vsim2>
    # This should be fixed by the vfiler team to make the following
    # status true all the times.
    if ( $raw_output =~ /WARNING: reassigning storage/ ) {
        $hash{"status"} = "Reassigned storage";
    }
    if ( $raw_output =~ /WARNING: removing IP addresses/ ) {
        if ( $hash{"status"} ) {
            $hash{"status"} .= " and " . "Removed IP addresses";
        } else {
            $hash{"status"} = "Removed IP addresses";
        }
    }
    if ( $raw_output !~ /WARNING: (reassigning storage|removing IP addresses)/ ) {
        $hash{"status"} = "Removed the resource(s)";
    }
    push(@result_array, \%hash);
    return \@result_array;
}

sub _check_error {
    my %args =@_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;
    if(($raw_output =~ /(.+ not a valid IP address)/)||($raw_output =~ /(.+ does not exist)/)||($raw_output =~ /(Check path .*)/)){
	$error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /$command(?:[^:]*):(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /([Ii]ncorrect usage)/) {
            # Since Incorrect usage is displayed both for invalid param and
            # invalid param value, it is classified as CommandFailed Exception
            $error = "CommandFailed";
        }
    } elsif ($raw_output =~ /(Missing\s+path\.)/) {
#made changes here..
        $error ="MissingRequiredParam";
		$$error_message = $1;
	} elsif ($raw_output =~ /Error: (.+)/) {
        $$error_message = $1;
        if($$error_message =~ /specify at least one IP and one unit of storage/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /already configured|appears more than once/) {
            $error = "InvalidParamValue";
        }
    } elsif ($raw_output =~ /Not\s+found\.(.+)/ ) {
	    $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /((Could not import|Root of root volume).+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /((Can't access|Storage path).+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /((Maximum allowed|Specified IPspace).+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(Vfiler not found: .+|vfiler .+ not found)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /([vV]filer \S+ already exists)/) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif($raw_output =~ /(Use of vfilers requires a multistore license)/) {
        $error = "FeatureNotLicensed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Unable to recreate vfiler .+|Value too large\..+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(Cannot limit vfilers to 0.+|Invalid argument.+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(cannot rename .+|Cannot destroy vfiler0)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(Cannot stop\/start vfiler0)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(The default vfiler cannot be stopped or started)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /vfiler rename: (invalid name .+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(Vfiler [\w-]+ could not be started .*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Vfiler [\w-]+ is still running.*)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Volume .+ has one or more vfilers sharing .+)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Operation would (leave|remove).+)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(.+ assigned to an interface that is UP)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Vfiler ([\w-]+) already stopped)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Vfiler ([\w-]+) is already running)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(New limit matches previous limit|Unable to save.+)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(Cannot directly (add|remove) resources (to|from) vfiler0)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(.+ active command in progress)/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /(.+ is already assigned .+|Invalid path .+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(.+ is not part of vfiler [\w-]+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /(.+ not in use by vfiler [\w-]+)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /Vfiler ([\w-]+) is not being mirrored from/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /Vfiler ([\w-]+) is not a DR backup/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /Unable to get UUID of remote vfiler/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /Login incorrect/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /Unable to resolve remote filer name/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /Cannot (add|remove) resources from ([\w-]+)/) {
        $error = "CommandFailed";
        $$error_message = "Cannot $1 resources from $2.
            Vfiler is part of the DR configuration for a remote vfiler";
    } elsif($raw_output =~ /Remote vfiler ([\w-]+) does not exist/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif($raw_output =~ /[vV]filer is being run from another connection.+/){
	$error = "CommandFailed";
	$$error_message = "vfiler is being run from another connection; try again later";
    } else {
        return undef;
    }
    # Remove ctrl-M from error message which is taken from raw output.
    $$error_message =~ s/\cM//g;
    return $error;
}

sub _parse_vfiler_output {
    my @Vfiler = @_;
    my @result_array;
    foreach my $ref (@Vfiler) {
        my @arr = @$ref;
        my ( @allowed_protocols, @disallowed_protocols, @ip_addresses );
        my ( @paths, %hash );
        if ( $arr[0] =~ /([\w-]+)\s+(.+)/ ) {
            $hash{"vfiler"} = $1;
            $hash{"status"} = $2;
        } elsif ( $arr[0] =~ /([\w-]+)/ ) {
            $hash{"vfiler"} = $1;
        }
        foreach (@arr) {
            if ( $_ =~ /(.+):\s(.+)/ ) {
                my $key   = lc($1);
                my $value = $2;
                $key =~ s/\s/_/;
                if ( $key eq "ip_address" ) {
                    my %ip_hash;
                    $value =~ /(.+)\s\[(.+)\]/;
                    $ip_hash{"ip_address"} = $1;
                    $ip_hash{"interface"}  = $2;
                    push( @ip_addresses, \%ip_hash );
                } elsif ( $key eq "path" ) {
                    if ( $value =~ /(.+)\s\[\/etc\]/ ) {
                        $hash{"configuration_directory"} = "/etc" if ( $1 eq "/" );
                        $hash{"configuration_directory"} = $1 . '/etc' if ( $1 ne "/" );
                        push( @paths, $1 );
                    } else {
                        # TODO:  Check how a simple array converted into xml
                        # object will look like and appropriately change the
                        # structure if required  - isshwar
                        push( @paths, $value );
                    }
                } elsif ( $key eq "allowed" ) {
                    $value =~ s/proto=//;
                    # TODO:  Check how a simple array converted into xml
                    # object will look like and appropriately change the
                    # structure if required  - isshwar
                    push( @allowed_protocols, $value );
                } elsif ( $key eq "disallowed" ) {
                    $value =~ s/proto=//;
                    # TODO:  Check how a simple array converted into xml
                    # object will look like and appropriately change the
                    # structure if required  - isshwar
                    push( @disallowed_protocols, $value );
                } else {
                    $hash{$key} = $value;
                }
            }
        }
        if (@allowed_protocols) {
            $hash{"allowed"} = \@allowed_protocols;
        }
        if (@disallowed_protocols) {
            $hash{"disallowed"} = \@disallowed_protocols;
        }
        if (@ip_addresses) {
            $hash{"ip_address_configuration"} = \@ip_addresses;
        }
        if (@paths) {
            $hash{"path"} = \@paths;
        }
        push( @result_array, \%hash );
    }
    return \@result_array;
}

## @name ::vfiler_dr_configure
##
## @summary Parses the raw output from the "vfiler dr configure" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_dr_configure with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_dr_configure(%args);
## INPUTS TO METHOD CALL 'vfiler_dr_configure' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-a' </td> <td> 'alternate-source' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'skip-snapmirror-init' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'nis-server' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-login-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'alternate-destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'remote-login-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sync-snapmirror' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'interface-details' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dns-server' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_dr_configure {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler dr configure",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::vfiler_dr_delete
##
## @summary Parses the raw output from the "vfiler dr delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_dr_delete with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_dr_delete(%args);
## INPUTS TO METHOD CALL 'vfiler_dr_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_dr_delete {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler dr delete",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];

}


sub vfiler_dr_activate {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler dr activate",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::vfiler_dr_resync
##
## @summary Parses the raw output from the "vfiler dr resync" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_dr_resync with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_dr_resync(%args);
## INPUTS TO METHOD CALL 'vfiler_dr_resync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-a' </td> <td> 'alternate-source' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-login-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'alternate-destination' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'remote-login-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sync-snapmirror' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_dr_resync {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler dr resync",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::vfiler_migrate
##
## @summary Parses the raw output from the "vfiler migrate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_migrate with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_migrate(%args);
## INPUTS TO METHOD CALL 'vfiler_migrate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-login-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'remote-login-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'method' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'interface-details' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_migrate {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler migrate",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

## @name ::vfiler_migrate_start
##
## @summary Parses the raw output from the "vfiler migrate start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_migrate_start with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_migrate_start(%args);
## INPUTS TO METHOD CALL 'vfiler_migrate_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-login-password' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'remote-login-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'interface-details' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'method' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_migrate_start {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler migrate start",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];

}

## @name ::vfiler_migrate_cancel
##
## @summary Parses the raw output from the "vfiler migrate cancel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_migrate_cancel with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_migrate_cancel(%args);
## INPUTS TO METHOD CALL 'vfiler_migrate_cancel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_migrate_cancel {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler migrate cancel",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

##
## @summary Parses the raw output from the "vfiler migrate complete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_migrate_complete with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_migrate_complete(%args);
## INPUTS TO METHOD CALL 'vfiler_migrate_complete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'secure' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-login-password' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'remote-login-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-vfiler' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'remote-filer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vfiler_migrate_complete {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler migrate complete",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    return [];
}

sub _get_vfiler_states {
    my @vfiler_states = ( "running", "stopped", "inconsistent", "defunct" );
    my $some_state = join( "|", @vfiler_states );
    return $some_state;
}

## @name ::vfiler_context
##
## @summary Parses the raw output from the "vfiler context" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vfiler_context with any of its options as described in the
##    corresponding cdef file 'vfiler.cdefs'
## @synopsis
##      vfiler_context(%args);
## INPUTS TO METHOD CALL 'vfiler_context' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vfiler' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; vfiler context vfiler1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub vfiler_context {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();
    my $result;

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"vfiler context",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _vfiler_context(raw_output=>$raw_output, command=>$command_line,
                              response_object=>$response);
    return $result;
}

sub _vfiler_context {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
        my @parsed = ();
        my %hash = ();
        my @array  = get_lines(raw_output => $raw_output, response_object => $response, can_throw => 0);
        foreach my $line (@array) {
            if($line =~ /(stopped).*'(.*)'/) {
                $hash{vfiler_status}=$1;
                $hash{vfiler_name}=$2;
                push(@parsed,\%hash);
                return \@parsed;
           } else {
               return [];
           }
        }
}
1;
