#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode vlan commands
##
## @description
##     This is a library to parse the raw output of 7Mode vlan commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##    use vlan;
##
##    vlan_add(%args);
##    vlan_create(%args);
##    vlan_delete(%args);
##    vlan_modify(%args);
##    vlan_stat(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::vlan;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _parse_for_errors (%) {
    my %args         = @_;
    my $raw_output   = $args{'raw_output'};
    my $command      = $args{'command'};
    my $errormsg_ptr = $args{'errormsg_ptr'};
    if (
           ( $raw_output =~ /vlan: (.*): no such interface/i )
        || ( $raw_output =~ /gvrp option must be followed by "on" or "off"./ )
        || ( $raw_output =~
            /invalid vlan id: .*s.Valid vlan id is from 1 to 4094./ )
        || ( $raw_output =~ /vlan: no such vlan id .* on interface/ )
      )
    {
        $$errormsg_ptr = $command . " contains invalid param value(s)";
        $$errormsg_ptr .= ". Error is,\n$raw_output";
        return "InvalidParamValue";
    }
    elsif (( $raw_output =~ /vlan:(.*): Interface is a VLAN interface./ )
        || ( $raw_output =~ /gvrp option must be followed by "on" or "off"./ )
        || ( $raw_output =~ /Usage:/ ) )
    {
        $$errormsg_ptr = $command . " contains invalid param value(s)";
        $$errormsg_ptr .= ". Error is,\n$raw_output";
        return "InvalidParamValue";
    }
    elsif (
        (
            $raw_output =~
            /vlan: Add failed. Vlan (ID|index) is already configured/
        )
        || ( $raw_output =~ /vlan: Modify failed/ )
        || ( $raw_output =~ /GVRP has already been (.*) on this interface./ )
        || ( $raw_output =~ /vlan: Display stat failed./ )
      )
    {
        $$errormsg_ptr = $command . " failed";
        $$errormsg_ptr .= ". Error is,\n$raw_output";
        return "CommandFailed";
    }
    elsif (( $raw_output =~ /^vlan:\s+command failed. Interface .*/ )
        || ( $raw_output =~ /^vlan:\s+\w+:\s+\w+.*interface/ ) )
    {
        $$errormsg_ptr = $command . " failed";
        $$errormsg_ptr .= ". Error is,\n$raw_output";
        return "CommandFailed";
    }
    return undef;
}

## @name ::vlan_create
##
## @summary Parses the raw output from the "vlan create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vlan_create with any of its options as described in the
##    corresponding cdef file 'vlan.cdefs'
## @synopsis
##      vlan_create(%args);
## INPUTS TO METHOD CALL 'vlan_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'gvrp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifname' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vlanid-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vlan_create(%) {
    my %args = @_;

    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();

    my $first_line = "vlan";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my $line;
    my @returnarray;

    while ( $line = shift(@lines) ) {
        my %hash;
        if ( $line =~ /vlan: (.*) has been created/ ) {
            $hash{$1} = "created";
            push( @returnarray, \%hash );
        }
    }    #end of while loop
    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@returnarray );
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    #end of sub vlan create

## @name ::vlan_add
##
## @summary Parses the raw output from the "vlan add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vlan_add with any of its options as described in the
##    corresponding cdef file 'vlan.cdefs'
## @synopsis
##      vlan_add(%args);
## INPUTS TO METHOD CALL 'vlan_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'gvrp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifname' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vlanid-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vlan_add (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "vlan";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    $error = _parse_for_errors(
        'raw_output'   => $raw_output,
        'command'      => $command,
        'errormsg_ptr' => \$errormsg
    );
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    my $line;
    my @returnarray;

    while ( $line = shift(@lines) ) {
        my %hash;
        if ( $line =~ /vlan: (.*) has been created/ ) {
            $hash{$1} = "created";
            push( @returnarray, \%hash );
        }
    }    #end of while loop
    if ($error) {
        $resp->set_parsed_output( 'parsed_output' => \@returnarray );
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    else {
        return \@returnarray;
    }
}    # end of sub vlan_add

## @name ::vlan_delete
##
## @summary Parses the raw output from the "vlan delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vlan_delete with any of its options as described in the
##    corresponding cdef file 'vlan.cdefs'
## @synopsis
##      vlan_delete(%args);
## INPUTS TO METHOD CALL 'vlan_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifname' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vlanid-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-q' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub vlan_delete (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    my %hash;
    my $error_level = $resp->get_execution_status();
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
	return [];
}    #end of vlan_delete

## @name ::vlan_modify
##
## @summary Parses the raw output from the "vlan modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vlan_modify with any of its options as described in the
##    corresponding cdef file 'vlan.cdefs'
## @synopsis
##      vlan_modify(%args);
## INPUTS TO METHOD CALL 'vlan_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-g' </td> <td> 'gvrp' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifname' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vlan_modify (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "";
    my $can_throw  = 0;
    my $error;
    my $errormsg;
    my %hash;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    if ( scalar(@lines) ) {
        $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw( text => "Unexpected output", response_object => $resp );
    }
    return [];
}    #end of vlan modify

## @name ::vlan_stat
##
## @summary Parses the raw output from the "vlan stat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vlan_stat with any of its options as described in the
##    corresponding cdef file 'vlan.cdefs'
## @synopsis
##      vlan_stat(%args);
## INPUTS TO METHOD CALL 'vlan_stat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ifname' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vlanid-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub vlan_stat (%) {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $first_line = "Vlan";
    my $can_throw  = 0;
    my $error;
    my $errormsg;

    if (
        $error = _parse_for_errors(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'errormsg_ptr' => \$errormsg
        )
      )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'first_line' => $first_line,
        'can_throw'  => $can_throw
    );
    return ( _vlan_stat( 'lines' => \@lines, 'response_obj' => $resp ) );
}

sub _vlan_stat(%) {
    my %args  = @_;
    my @lines = @{ $args{'lines'} };
    my $resp  = $args{'response_obj'};
    my @returnarray;
    while ( scalar(@lines) ) {
        if ( $lines[0] =~ /Vlan .*Interface: +(.*)  / ) {
            push @returnarray,
              { name => $1, %{ get_record( 'lines' => \@lines ) } };
        }
        else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(
                text            => "Unexpected output",
                response_object => $resp
            );
        }
    }
    return \@returnarray;
}

sub get_record {
    my %args      = @_;
    my $lines_ptr = $args{'lines'};
    my %returnhash;
    my $line;
    $line = shift(@$lines_ptr);
    my $returnflag;
    $returnflag = 0;
    do {

        if ( $line =~ /Vlan (.*)Interface: +.*  / ) {
            my @fields = split( /:/, $line );
            if ( $1 =~ /Physical/ ) {
                $returnhash{'interface'} = "physical";
            }
            else {
                $returnhash{'interface'} = "vlan";
            }
        }
        elsif (( $line =~ /IDs?: +(.*)/ )
            || ( $line =~ /GVRP: +(.*)/ )
            || ( $line =~ /MAC Address: (.*)/ ) )
        {
            my @fields = split( /:/, $line );
            $fields[0] = to_key_name( 'item' => $fields[0] );
            $returnhash{ $fields[0] } = $1;
        }
        elsif ( $line =~ /(.*) STATISTICS/ ) {
            $line = shift(@$lines_ptr);
            my $data_lines;
            my %hash_data;
            my $statistics_name = $1 . " statistics";
            $returnflag = 1 if ( $1 =~ /TRANSMIT/ );
            $statistics_name = lc($statistics_name);
            $statistics_name =~ s/ +/_/g;
            $statistics_name = to_key_name( 'item' => $statistics_name );

            while (( $line !~ /STATISTICS/ )
                && ( $line !~ /Vlan .*Interface: +(.*)  / ) )
            {
                $data_lines .= $line;
                $data_lines .= "|";
                last if ( scalar(@$lines_ptr) == 0 );
                $line = shift(@$lines_ptr);
            }
            unshift( @$lines_ptr, $line ) if ( scalar(@$lines_ptr) > 0 );
            my @units = split( /\|/, $data_lines );
            foreach my $field (@units) {
                my @field_value = split( /:/, $field );
                foreach my $temp_field (@field_value) {
                    trim( 'string_ptr' => \$temp_field );
                }
                $field_value[0] = to_key_name( 'item' => $field_value[0] );
                $hash_data{ $field_value[0] } = $field_value[1];
            }
            $returnhash{$statistics_name} = [ \%hash_data ];
        }
        if ($returnflag) {
            return \%returnhash;
        }
    } while ( $line = shift(@$lines_ptr) );
}
1;
