#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode vstorage command output
##
## @description
##    This is a library to parse the raw output of 7Mode vstorage command.
##
## @synopsis
##    use vstorage;
##
##    vstorage(%args);
##
## @author anupamap@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::vstorage;

use strict;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $command    = $args{command};
    my $error;
    my $error_message;
    my $resp = $args{"response_object"};
    if ( ( $raw_output =~ /Invalid usage/i ) | ( $raw_output =~ /Unknown/i ) ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /(.*for more information)/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::vstorage_show
##
## @summary Parses the raw output from the "vstorage show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vstorage_show with any of its options as described in the
##    corresponding cdef file 'vstorage.cdefs'
## @synopsis
##      vstorage_show(%args);
## INPUTS TO METHOD CALL 'vstorage_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = priv set -q diag; vstorage show ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VAAI Traffic priority = LOWER
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'traffic_priority' => 'LOWER'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vstorage_show {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output      => $output,
        command         => $command,
        response_object => $resp,
    );
    if ( $output =~ /(.*)\s+Traffic priority =\s+(.*)/m ) {
        my %hash;
        $hash{traffic_priority} = $2;
        push( @result_array, \%hash );
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }

}

## @name ::vstorage_set_priority
##
## @summary Parses the raw output from the "vstorage set_priority" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vstorage_set_priority with any of its options as described in the
##    corresponding cdef file 'vstorage.cdefs'
## @synopsis
##      vstorage_set_priority(%args);
## INPUTS TO METHOD CALL 'vstorage_set_priority' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'priority' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = priv set -q diag; vstorage set_priority LOWER; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;vStorage traffic priority is now LOWER.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'traffic_priority' => 'LOWER.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub vstorage_set_priority {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output      => $output,
        command         => $command,
        response_object => $resp,
    );
    if ( $output =~ /vStorage traffic priority is now\s+(.*)/m ) {
        my %hash;
        $hash{traffic_priority} = $1;
        push( @result_array, \%hash );
        return \@result_array;
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => 'Unexpected output', response_object => $resp);
    }
}

## @name ::vstorage_zero_counters
##
## @summary Parses the raw output from the "vstorage zero_counters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vstorage_zero_counters with any of its options as described in the
##    corresponding cdef file 'vstorage.cdefs'
## @synopsis
##      vstorage_zero_counters(%args);
## INPUTS TO METHOD CALL 'vstorage_zero_counters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##
##&nbsp;Command = priv set -q diag; vstorage zero_counters; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub vstorage_zero_counters {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _check_error(
        raw_output      => $output,
        command         => $command,
        response_object => $resp,
    );
    return [];
}
1;
