#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/wreck.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode wreck command output
##
## @description
##    This is a library to parse the raw output of 7Mode wreck command.
##
## @synopsis
##    use wreck;
##
##    wreck(%args);
##
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::wreck;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::VolumeNotRestrictedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /(Missing.*argument)/ ) {
        $$error_message = $1;
        $error          = "MissingRequiredParam";
    } elsif ( $raw_output =~ /((Can\'t find inject file.*)|(Too many arguments)|(Can\'t find volume.*)|(Can\'t find aggregate.*)|(Unknown injection command))/ ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /((Failed to resolve.*)|(Failed to read.*)|(can't wreck.*need parity protected disks to set pseudo_bad or unverified))/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    } elsif ( $raw_output =~ /((vol restrict\: Volume is in use by other operations)|(Volume.*is not restricted))/i ) {
	$$error_message = $1;
        $error          = "VolumeNotRestricted";
    }
    return $error;
} ## end sub _check_error

## @name ::wreck
##
## @summary Parses the raw output from the "wreck" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wreck with any of its options as described in the
##    corresponding cdef file 'wreck.cdefs'
## @synopsis
##      wreck(%args);
## INPUTS TO METHOD CALL 'wreck' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'inject-file' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'inject-command' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume' </td> </tr>
## <tr> <td> '-unverified' </td> <td> 'unverified' </td> </tr>
## <tr> <td> '-pseudo_bad' </td> <td> 'pseudo-bad' </td> </tr>
## <tr> <td> '-pseudo_bad_ssd_cached_block' </td> <td> 'pseudo_bad_ssd_cached_block' </td> </tr>
## <tr> <td> '-A' </td> <td> 'aggregate' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = wreck vol1 'volinfo_snap_delete_mask bits=1 index=0'
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'wreck_status' => 'Finished',
##&nbsp;            'wreck_corruption' => 'Wrote vbn 1469859 vvbn 1'
##&nbsp;            'inj_info' => 'FSINFO change SSD blocks used for write caching from 0 to 123456',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub wreck {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _wreck(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );
    return $output;
} # end sub wreck

sub _wreck {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For wreck, some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in _get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;
    my $first_line = 'Remounting\s+(aggregate|volume)\s+\'.*\'\s+read-only\.\.\.';

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => $first_line,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );

    my %wreck_output;
    my $other_info;
    for ( my $line = 0; $line < @arr; $line++ ) {
        chomp $line;
        next if ( $arr[$line] eq "" );
        if ( $arr[$line] =~ m/((Skipping.*)|(Wrote.*))/ ) {
            $wreck_output{'wreck_corruption'} = $1;
        } elsif ( $arr[$line] =~ m/(Finished)\./ ) {
            $wreck_output{'wreck_status'} = $1;
        } elsif ( $arr[$line] =~ /INJ\s*:\s*(.*)/ ) {
            $wreck_output{'inj_info'} = $1;
        } else {
            $other_info = $other_info."\n".$arr[$line];
        }        
    }
    $wreck_output{'other_info'} = $other_info if ($other_info);
    unless ( defined $wreck_output{'wreck_corruption'} and defined $wreck_output{'wreck_status'}){
        my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
        $error->throw(
            text            => "Unexpected format of output",
            response_object => $response
        );
    }
    @result_array = (\%wreck_output);

    return \@result_array;
} # end sub _wreck

1;
