#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for cluster mode CLI commands
## @author pressley@netapp.com, skotagir@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::CLI::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

# Module import - APISets
use NACL::APISet::Session::CLI::CMode;
use NACL::APISet::Session::CLI::CMode::Vserver;
use NACL::APISet::_Mixins::CLI qw(_move_connectrec_options);

#
#Exceptions
#
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

#
# NATE imports
#
use Tie::IxHash;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Node::CLI);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;
    my $self = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            version => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;
    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method:    find_cdef_directory
# Objective: To find the cdef directory based on various version related
#            attributes obtained from the service.
# Details:   Refer to NACL::APISet::Node for POD documentation.
###############################################################################

sub find_cdef_directory {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;

    # Load CDEFs directly from the build root if specified by the user
    if (   ( defined $self->{version_info}->{cdefs_from_build} )
        && ( $self->{version_info}->{cdefs_from_build} ) )
    {
        my $vm = $self->{version_manager};
        NATE::Exceptions::Argument->throw(
                  "Version manager could not be found"
                . ". Hence build root cannot be autodiscovered to load CDEFs."
        ) unless ( defined $vm );
        my $build_path;
        unless ( $self->_is_vserver_version_manager($vm) ) {
            $build_path
                = $vm->get_version_attribute( attribute => "build_path" );
        } else {
            NATE::Exceptions::Argument->throw(
                "Build root cannot be discovered when the version manager"
                    . " is of type "
                    . ref($vm) );
        }
        # If the buildpath obtained from versioning does not exist, we 
        # will try searching the build under /x/eng/rlse/DOT, because most
        # of the builds move to this location after a couple of days.
        unless ( -d $build_path ) {
            my $old_path = $build_path;
            if ($build_path =~ s/.+\/(\w+_\d+_\d+\/?)$/$1/) {
                $build_path = '/x/eng/rlse/DOT/' . $build_path; 
            
                $Log->comment("The build path $old_path does not exist. " . 
                    "Trying to use $build_path to load the cdefs");
		    }
        }
        $Log->exit() if $may_exit;
        return $build_path;
    }
    my $ret = $self->SUPER::find_cdef_directory(%opts);
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub find_cdef_directory

##############################################################################
# Method:    set_vserver
# Objective: To set the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_vserver {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "vserver-context-name" => { type => SCALAR } }
    );
    $self->{"vserver"} = $args{"vserver-context-name"};

    $Log->exit() if $may_exit;
} ## end sub set_vserver

##############################################################################
# Method:    get_vserver
# Objective: get the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_vserver {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"vserver"};
} ## end sub get_vserver

##############################################################################
# Method:    clear_vserver
# Objective: clear the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_vserver {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $self->{"vserver"} = "";

    $Log->exit() if $may_exit;
} ## end sub clear_vserver

##############################################################################
# Method:    set_privilege
# Objective: Set the current privilege level
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
# Setting privilege for CMode is done directly by using set() CLI command and
# this will work only for persistent connections and at the moment CMode
# supports only persistent connections.

sub set_privilege {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "privilege-level" => { type => SCALAR } }
    );
    $self->set( privilege => $args{"privilege-level"} );
    $self->{"privilege-level"} = $args{"privilege-level"};

    $Log->exit() if $may_exit;
} ## end sub set_privilege

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw CLI command on the filer and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $conn = $self->get_connection();

    $self->api_reconnect_session()
        if $conn->is_disconnected( verify => 0 );

    $self->_move_connectrec_options(
        source => \%args,
        target => \%args
    );

    $self->timeout_scaling_factor(
        Tharn::param("APISET_TIMEOUT_SCALING_FACTOR"))
        if defined Tharn::param("APISET_TIMEOUT_SCALING_FACTOR");

    $args{"on_timeout"}
        = $self->_throw_timeout_exception( before => $args{"on_timeout"},
            timeout => $args{timeout});
    $args{"on_idle"}
        = $self->_throw_timeout_exception( before => $args{"on_idle"},
            timeout => $args{timeout} );

    my $output = $self->{session}->send_command( connectrec => \%args );

    $Log->exit() if $may_exit;
    return $output;
} ## end sub execute_raw_command

sub fallback_privilege {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return 'diagnostic';
}

### Command overrides ###

##############################################################################
# Method:    volume_explore
# Objective: Overriden method for when the "volume_explore" method is invoked
# Details:   The command "volume explore" takes multiple options.
#            Unfortunately each option produces output in a different format.
#            It's unfeasible to write parsers for each option, and option
#            combination, so while invoking the command the "-dump all"
#            option is added automatically (except for specific values of
#            format or when -set is specified) - which results in all output
#            fields being displayed and in a standard key=value format.
##############################################################################
sub volume_explore {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    if (   !exists $args{dump}
        && !exists $args{set}
        && $args{format} !~ /^(?:sdir|raw|syntax|examples|fields|time)$/i )
    {
        $args{dump} = "all";
    }
    if ( !exists $args{verbose} && $args{format} =~ /^sdir$/i ) {
        $args{verbose} = "";
    }

    $Log->exit() if $may_exit;
    return $self->_invoke( method => "volume_explore", %args );
} ## end sub volume_explore


##############################################################################
# Method:    debug_smdb_table_show
# Objective: This method invokes the brute force parser for the
#            "debug smdb table show" command
##############################################################################
sub debug_smdb_table_show {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    $args{action} = "show";
    my $resp = $self->_debug_smdb( %args );
    $Log->exit() if $may_exit;
    return $resp;
} ## end sub debug_smdb_table_show

##############################################################################
# Method:    debug_smdb_table_run
# Objective: This method invokes the brute force parser for the
#            "debug smdb table run" command
##############################################################################
sub debug_smdb_table_run {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    $args{action} = "run";
    my $resp = $self->_debug_smdb( %args );
    $Log->exit() if $may_exit;
    return $resp;
} ## end sub debug_smdb_table_run

##############################################################################
# Method:    debug_smdb_table_create
# Objective: This method invokes the brute force parser for the
#            "debug smdb table create" command
##############################################################################
sub debug_smdb_table_create {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    $args{action} = "create";
    my $resp = $self->_debug_smdb( %args );
    $Log->exit() if $may_exit;
    return $resp;
} ## end sub debug_smdb_table_create

##############################################################################
# Method:    debug_smdb_table_delete
# Objective: This method invokes the brute force parser for the
#            "debug smdb table delete" command
##############################################################################
sub debug_smdb_table_delete {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    $args{action} = "delete";
    my $resp = $self->_debug_smdb( %args );
    $Log->exit() if $may_exit;
    return $resp;
} ## end sub debug_smdb_table_delete

##############################################################################
# Method:    debug_smdb_table_modify
# Objective: This method invokes the brute force parser for the
#            "debug smdb table modify" command
##############################################################################
sub debug_smdb_table_modify {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    $args{action} = "modify";
    my $resp = $self->_debug_smdb( %args );
    $Log->exit() if $may_exit;
    return $resp;
} ## end sub debug_smdb_table_modify

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the CMode API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API, assigning a default CDEF path
#            and resetting the privilege level. The privilege-level
#            constructor override for Vserver APISets would try to set the
#            specified privilege level and if that fails (because we are
#            running on a older build), then that would be ignored
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file( $self->{cdef_path} );
    
    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }

    $self->SUPER::_init_API(%args);

    my $vm;
    try {
        $vm = $self->get_version_manager();
    }
    catch NATE::BaseException with {
        my $ex = shift;
        $Log->debug( 'version manager is not available' . $ex->text );
    };

    # Invoke compatibility
    if ( $self->{version_info}->{compatibility} ) {
        if ( defined $vm ) {
            $Log->debug("Initializing compatibility engine");
            try {
                unless ( $self->_is_vserver_version_manager($vm) ) {
                    $vm->setup_compatibility( interface => "CLI" );
                } else {
                    $Log->trace(
                        "Compatibility will be disabled as the version manager "
                            . "is of type 'NACL::APISet::Version::Node::Vserver' and "
                            . "uichanges information cannot be discovered when directly"
                            . " connected to vserver shell" );

                    # Disable compatibility
                    $self->{version_info}->{compatibility} = 0;
                }
            }
            catch NATE::BaseException with {

                # The possible errors are
                # 1)unable to find uichange strings
                # 2)unable to find CompatibilityCli.pm
                # 3)error in loading CompatibilityCli.pm
                # where the third one is most unlikely.
                my $exception = shift;
                $Log->warn( "Failed while trying to setup compatibility "
                        . "and the error is: "
                        . $exception->text()
                        . "\n Compatibility will be disabled" );

                # Disable compatibility
                $self->{version_info}->{compatibility} = 0;
            };
        } else {

            # version manager is not available. so compatibility cannot be set
            $Log->warn( "Version manager is not available. "
                    . "Compatibility will be disabled" );
        }
    }

    # Initialize defaults for prefixing member variables
    if ( $self->{"shell_type"} ne "Vserver" ) {
        if ( defined $self->{'privilege-level'} ) {
            $self->set_privilege(
                'privilege-level' => $self->{'privilege-level'} );
        } else {
            $self->clear_privilege();
        }
    } elsif ( ( $self->{"shell_type"} eq "Vserver" ) ) {
        if ( defined $vm && ( $vm->is_version_ge( release => "8.2.0" ) ) ) {
            try {
                if ( defined $self->{'privilege-level'} ) {
                    $self->set_privilege(
                        'privilege-level' => $self->{'privilege-level'} );
                } else {
                    $self->clear_privilege();
                }
            }
            catch NACL::APISet::Exceptions::CommandFailedException with {
                my $ex = shift;
                $ex->throw
                    unless (
                    $ex->text() =~ /not authorized for that command/ );
            };
        } elsif ( !defined $vm ) {
            $Log->warn(
                'Version manager is not available, so default privilege level'
                    . ' for Vserver CLI APISet is left in the admin level' );
        }
    }

 # Set the vserver name provided through context_definition hash irrespective
 # of whether we had directly connected to the vserver or through the node.
 # Thus incase of direct connection to vserver, if the user overrides the
 # vserver name with other vserver name or even the same vserver name, we will
 # prepend the command with this name and allowing the service to respond to
 # it. Thus using the context_definition hash incase of direct connection
 # will cause an exception to be thrown based on how the service reacts.
    if ( $self->{actual_set} eq "Vserver" ) {
        my $vserver_name
            = defined( $self->{context_definition}->{name} )
            ? $self->{context_definition}->{name}
            : (
            defined $self->{context_definition}->{schema_hostobj}
            ? $self->{context_definition}->{schema_hostobj}->hostname()
            : ""
            );
        $self->set_vserver( "vserver-context-name" => $vserver_name )
            if ( length $vserver_name );
    }

    $Log->exit() if $may_exit;
} ## end sub _init_API

##############################################################################
# Method:    _get_cdef_files_list
# Objective: To consolidate the cdef files present in a build directory
# Details:   This method returns back the list of cdef paths for the cdef
# files distributed in various sub-directories under build root.
##############################################################################

sub _get_cdef_files_list {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;
    my $build_dir = $args{buildroot};
    # Update this array when new command creating SMF files are added to a new
    # directory. Refer burt 454335 for the newly added locations to search the
    # cdefs from.
    my @target_dirs = (
        "\/final\/bedrock\/export\/common\/cdefsprod",
    );
    my @files;
    for my $dir (@target_dirs) {
        my $full_dir = $build_dir . $dir;
        if (-d $full_dir) {
            require File::Spec::Functions;
            opendir DH, $full_dir or do {
                $Log->warn("Could not open directory '$full_dir: $!");
                next;
            };
            while (my $file = readdir DH) {
                next if ($file !~ /\.cdefs$/);
                push @files, File::Spec::Functions::catfile($full_dir, $file);
            }
        }
    }
    $Log->exit() if $may_exit;

    return @files;
} ## end sub _get_cdef_files_list

##############################################################################
# Method:    _convert_method_and_options
# Objective: To update the method name and option names as expected by the
# service by performing forward compatibility
# Details:  This method iterates over the various entries in the compatibility
# hash and convert the method name and options name. Any cascading changes
# will also be handled. This method returns the command name, updated command
# options and compatibility hash for doing compatibility in output side.
##############################################################################

sub _convert_method_and_options {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "command_params" => { type => HASHREF, optional => 0 } }
    );
    my $method          = delete $args{command_params}{method};
    my %command_params  = %{ $args{command_params} };
    my $version_manager = $self->get_version_manager();
    my %compatibility   = %{ $version_manager->{compatibility} };
    my @compat_keys     = keys %{ $version_manager->{compatibility} };
    $Log->debug( sub {"compat_keys: " . Dumper( \@compat_keys )});

    my ( $oldCommand, $command );
    $oldCommand = $command = $method;

    # The conversion hash is order preserved to prevent any infinite
    # loop conditions which can be introduced by cyclic changes such as
    # A=>B, B=>C, C=>A. Now if the order is preserved, we just have to
    # traverse through the hash only once in forward direction while
    # updating the option names.
    my %conversion_hash = ();
    tie %conversion_hash, 'Tie::IxHash';

    # Convert the method name to the form expected by the underlying service.
    # Check to see if an exact match exists. This will reduce the looping time
    # as for an exact matches we need not check other entries as any other ui
    # changes happened after that on the same command would have been already
    # merged when constructing the compatibility hash dynamically.
    my $change_flag = 0;
    foreach my $key (@compat_keys) {

        # Partial match should not happen in full command pattern
        # For Eg: protocol_show should not match with protocol_show_all
        # hence skip the full command pattern entries
        next if ( $key eq "_ALL_COMMANDS" );
        
        # Looking for pattern $command in the entry $key
        if (   ( ( $key !~ /_$/ ) && ( $command eq $key ) )
            || ( ( $key =~ /_$/ ) && ( $command =~ /^$key/ ) ) )
        {
            $change_flag = 1;

            # Update the command name if new_command is defined. But
            # irrespective of this, conversion hash should be collected if
            # there are any option name changes.
            $Log->debug("Compatibility match on: $key");
            if ( defined $compatibility{$key}{new_command} ) {

                # define a temp variable and make the conversion on it
                my $temp_command = $command;
                $temp_command =~ s/$key/$compatibility{$key}{new_command}/;

                # We shouldn't check whether the loaded schema has an entry
                # for temp_command and stop with that iteration. Always
                # loop through the entire hash once because we would be
                # keeping CDEFs for both old and new command syntax incase
                # if the command name had undergone modification.
                # Hence for cascading changes, the old command might exist
                # in schema before we could reach the latest method name
                # expected by the service.
                $Log->debug( "The command name is updated from $command to "
                        . $temp_command );
                $command    = $temp_command;
                $oldCommand = $key;
            }

            # Collect the conversion hash
            # Even if there are no input options, still we need to get the
            # conversion hash for doing output compatibility
            my %new;
            tie %new, 'Tie::IxHash';
            %new
                = ( defined $compatibility{$key}{conversion} )
                ? %{ $compatibility{$key}{conversion} }
                : ();
            %conversion_hash = ( %conversion_hash, %new );
        }
    }

    # The method name would have been converted to the form as expected by the
    # service. We don't check whether the cdef exists for this method name at
    # this point. An exception will be thrown by schema later if cdef for the
    # updated method name doesn't exist.
    $Log->debug(
        "Updated method name after compatibility translation: " . $command );
    $Log->debug(
        sub {"The conversion hash obtained is \n" . Dumper( \%conversion_hash )});
    $Log->debug("This command '$command' hasn't undergone any uichanges")
        unless ($change_flag);

    # Get the _ALL_COMMANDS conversion hash. We maintain order of changes
    # for this change too.
    my %all_commands_conversion = ();
    tie %all_commands_conversion, 'Tie::IxHash';
    if (   ( exists $compatibility{_ALL_COMMANDS}->{conversion} )
        && ( defined $compatibility{_ALL_COMMANDS}->{conversion} ) )
    {
        %all_commands_conversion
            = %{ $compatibility{_ALL_COMMANDS}->{conversion} };
    }
    $Log->debug( sub {"The conversion hash for _ALL_COMMANDS change\n"
            . Dumper( \%all_commands_conversion ) });

    # Update the option names
    %command_params = $self->_convert_params(
        command_params          => \%command_params,
        conversion              => \%conversion_hash,
        all_commands_conversion => \%all_commands_conversion,
    );

    # Output conversion params list. Conversion hash entries should override
    # _ALL_COMMANDS conversion hash entries as command specific entries gain
    # higher precedence than _ALL_COMMANDS entry.
    %conversion_hash = ( %all_commands_conversion, %conversion_hash );
    my %output_compat_mirror = ();

    # Output compatibility doesn't require the order of the conversion
    # hash to be maintained.
    while ( my ( $old, $new ) = each %conversion_hash ) {
        $output_compat_mirror{$new} = $old;
    }
    $Log->debug( sub {"The compat mirror for doing output compatibility is: "
            . Dumper( \%output_compat_mirror )} );

    # Return the result
    my $output = {
        method               => $command,
        command_params       => \%command_params,
        output_compat_mirror => \%output_compat_mirror
    };
    $Log->exit() if $may_exit;
    return $output;
} ## end sub _convert_method_and_options

##############################################################################
# Method:    _convert_params
# Objective: To convert the option names specified by the user
# Details:   This method will convert the keys in the command_params to the
# mapping specified in the conversion hash
##############################################################################

sub _convert_params {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            "command_params"          => { type => HASHREF, optional => 0 },
            "conversion"              => { type => HASHREF, optional => 0 },
            "all_commands_conversion" => { type => HASHREF, optional => 0 },
        }
    );
    my %conversion              = %{ $args{conversion} };
    my %all_commands_conversion = %{ $args{all_commands_conversion} };
    my %command_params          = %{ $args{command_params} };
    my %params                  = ();

    # Both coversion hash and _ALL_COMMANDS conversion hash are empty.
    # No option updation required
    if (   ( scalar( keys %conversion ) == 0 )
        && ( scalar( keys %all_commands_conversion ) == 0 ) )
    {
        $Log->debug( "Both conversion hash and _ALL_COMMANDS converison hash "
                . "are empty. Returning command_params without any change" );
        $Log->exit() if $may_exit;
        return %command_params;
    }

    # There are 'show' type commands which usually takes -fields and -instance
    # as options (these options are not defined in the CDEFs). -fields is an
    # array option whose value is a comma separated values of column names in
    # the output table. If the column name changes, then compatibility should
    # update the value passed -fields option as well.
    # Eg: mycommand show -fields foo -foo value1
    # option (or column) 'foo' is renamed to 'abc'
    # This command should be translated as,
    # 'mycommand show -fields abc -abc value1'
    my %fields           = ();
    my @processed_fields = ();
    if ( exists $command_params{fields} ) {
        $Log->debug( "-fields option is passed with the value of "
                . "'$command_params{fields}'" );
        my @fields_arr = split( /\s*,\s*/, $command_params{fields} );
        foreach (@fields_arr) {
            $fields{$_} = 1;
        }
    }

    # Subroutine which handles the updation of options by reading only
    # ALL_COMMANDS conversion hash
    # Input: Option name specified by the user
    # Output: Processed option name
    my $update_option_for_ALL_COMMANDS_conversion = sub {
        my $input = shift;
        while ( defined $all_commands_conversion{$input} ) {
            $Log->debug( "Input conversion: $input => "
                    . $all_commands_conversion{$input} );
            $input = $all_commands_conversion{$input};
        }
        return $input;
    };

    # Subroutine which handles the updation of options by reading
    # ALL_COMMANDS and the command specific conversion hash
    # Input: Option name specified by the user
    # Output: Processed option name
    my $update_option_for_both_conv_hash_and_ALL_COMMANDS = sub {
        my $input      = shift;
        my $curr_index = 0;

        while (1) {
            my $do_next = 1;
            my $newname = "";
            if ( defined $conversion{$input} ) {
                $do_next = 0;
                $newname = $conversion{$input};

                # Get the index
                my $index = -1;
                my $i     = 0;
                map { $index = $i if ( $_ eq $input ); $i++ }
                    keys %{ $args{conversion} };
                if ( ( $index <= $curr_index ) && ( $curr_index != 0 ) ) {

                    # This case will be hit when we have cyclic changes.
                    # In that case, this entry of the conversion hash is
                    # already traversed and should be ignored.
                    $Log->debug( "Cyclic change detected and hence ignoring "
                            . "this match. curr_index = $curr_index, new_index = "
                            . $index );
                    $do_next = 1;
                    $newname = "";
                } else {
                    $curr_index = $index;
                }
            }
            if ( ( defined $all_commands_conversion{$input} ) && ($do_next) )
            {
                $newname = $all_commands_conversion{$input};
            }
            if ( $newname ne "" ) {
                $Log->debug("Input conversion: $input => $newname");
                $input = $newname;
                next;
            }

           # A match is not hit in conversion hash and all_commands conversion
           # hash. So iteration should be stopped now.
            last;
        }
        return $input;
    };

    # Only _ALL_COMMANDS conversion hash has got entries. We believe cyclic
    # changes won't happen in _ALL_COMMANDS and hence the logic for
    # option updation is simple: Keep looping untill we don't find an entry
    # for the updated option name in conversion hash.
    if ( scalar( keys %conversion ) == 0 ) {
        $Log->debug( "Conversion hash is empty. Hence only _ALL_COMMANDS"
                . " conversion hash will be processed" );
        while ( my ( $input, $value ) = each %{ $args{command_params} } ) {
            $Log->debug("starting params updation for $input => $value") if (defined $value);
            my $processed_input
                = $update_option_for_ALL_COMMANDS_conversion->($input);
            $params{$processed_input} = $value;

            # Update the values (column names) passed to 'fields' option.
            # This optimizes the processing by using the translation done for
            # the options (-foo, -goo) passed for the command to be resued in
            # the translation for -fields foo,goo as well.
            if ( exists $fields{$input} ) {

                # Delete $input from %fields to skip re-processing of these
                # fields in the following second pass.
                delete $fields{$input};
                push( @processed_fields, $processed_input );
            }
        }

        # The options specified to -fields which haven't undergone processing
        # in the previous pass would be updated now
        if ( scalar( keys %fields ) ) {
            foreach my $input ( keys %fields ) {
                my $processed_input
                    = $update_option_for_ALL_COMMANDS_conversion->($input);

                # Deletion of the key $input from %fields is not needed
                push( @processed_fields, $processed_input );
            }
        }
    } else {

      # Both conversion hash and _ALL_COMMANDS conversion hash are present.
      # This logic handles any cyclic changes (A=>B, B=>C, C=>A) and cases
      # where some change happened in a command, then followed by _ALL_COMMAND
      # change which will also affect the previous command and then again a
      #  specific change in the same command itself.
        $Log->debug(
                  "Input params will be updated for both conversion hash and"
                . " _ALL_COMMANDS conversion hash" );

        while ( my ( $input, $value ) = each %{ $args{command_params} } ) {
            $Log->debug("starting params updation for $input => $value") if (defined $value);
            my $processed_input
                = $update_option_for_both_conv_hash_and_ALL_COMMANDS->(
                $input);

            # Now the input option name would have been converted to the form
            # expected by the service.
            $params{$processed_input} = $value;

            # Update values passed to -fields option as well
            if ( exists $fields{$input} ) {
                delete $fields{$input};
                push( @processed_fields, $processed_input );
            }
        }

        # The options specified to -fields which haven't undergone processing
        # in the previous pass would be updated now
        if ( scalar( keys %fields ) ) {
            foreach my $input ( keys %fields ) {
                my $processed_input
                    = $update_option_for_both_conv_hash_and_ALL_COMMANDS->(
                    $input);

                # Deletion of the key $input from %fields is not needed
                push( @processed_fields, $processed_input );
            }
        }
    }

    # If @processed_fields is non-empty, then it implies that user has
    # specified -fields option, hence we should add the processed value of
    # -fields back to %params hash.
    if ( scalar @processed_fields ) {
        $params{'fields'} = join( ',', @processed_fields );
    }
    $Log->debug(
        sub {"\%args specified by the user: " . Dumper( \%command_params )} );
    $Log->debug( sub {"Updated \%args by compatibility: " . Dumper( \%params )} );
    $Log->exit() if $may_exit;
    return %params;
} ## end sub _convert_params

##############################################################################
# Method:    cli_command_helper
# Objective: This method walks the UI for a give api command and returns an
#            hashref containing the options,its datatype and description,
#            for more details refer to the POD
##############################################################################

sub cli_command_helper {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %alias_map;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            'privilege-level' => { type => SCALAR | UNDEF, optional => 1 },
            api               => { type => SCALAR },
            cdef_lookup       => { type => BOOLEAN,        default  => 1 }
        },
    );
    my $cdef_lookup = delete $opts{cdef_lookup};
    my $fields      = {};
    my $cmd         = $opts{api};
    if ($cdef_lookup) {
        my @params
            = $self->get_allowed_options_for_command( command => $cmd );
        foreach my $param (@params) {
            next if ( length($param) == 1 );
            $fields->{$param}->{field}    = $param;
            $fields->{$param}->{is_array} = 0;
        }
    }
    my %priv_args = ();
    if ( defined $opts{'privilege-level'} ) {
        $priv_args{'privilege-level'} = $opts{'privilege-level'};
    }
    my $response = $self->$cmd( pass_through => "?", %priv_args );
    my $ds = $response->generate_ds( fields => $fields );

    my $cdef = $self->get_command_definition(command => $opts{api});
    my $alias_map_ref = $cdef->get_alias_map();

    if ($alias_map_ref) {
       %alias_map = %{ $alias_map_ref };
    }
    while (my ($old, $new) = each (%alias_map)) {
        if ( $fields->{$new} ) {
            my %convert_hash = %{ $fields->{$new} };
            $convert_hash{field} = $old;        
            $fields->{$old} = \%convert_hash;
        }
    }

    $Log->exit() if $may_exit;
    return $ds;
}

sub get_valid_values {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            api               => { type => SCALAR },
            option            => { type => SCALAR }
        },
    );
    my $cmd         = $opts{api};
    my $option      = $opts{option};
    my $output = $self->cli_command_helper(api => $cmd); 
    my @valid_values = @{$output->{fields}{$option}{valid_values}};
    $Log->exit() if $may_exit;
    return \@valid_values;
}

##############################################################################
# Method:    is_command_present
# Objective: To check if the command is supported in the build
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub is_command_present {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            'privilege-level' => { type => SCALAR | UNDEF, optional => 1 },
            api               => { type => SCALAR },
        },
        allow_extra => 0
    );
    my $api       = $opts{api};
    my $ret       = 0;
    my %priv_args = ();
    if ( defined $opts{'privilege-level'} ) {
        $priv_args{'privilege-level'} = $opts{'privilege-level'};
    }
    try {
        $self->$api( pass_through => '?', %priv_args );
        $ret = 1;
    }
    catch NACL::APISet::Exceptions::CommandNotFoundException with    {}
        catch NACL::APISet::Exceptions::MethodNotFoundException with {};
    $Log->debug("Is present $opts{api}: $ret");
    $Log->exit() if $may_exit;
    return $ret;
}

##############################################################################
# Method:    is_option_present
# Objective: To check if the command has got a specific option
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub is_option_present {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            'privilege-level' => { type => SCALAR | UNDEF, optional => 1 },
            api               => { type => SCALAR },
            field             => { type => SCALAR },
        },
        allow_extra => 0
    );
    my $field        = delete $opts{field};
    my $command_info = $self->cli_command_helper( %opts, cdef_lookup => 0 );
    my $ret          = 0;
    $ret = 1 if ( exists $command_info->{fields}->{$field} );
    $Log->debug("Option $field is present in $opts{api}: $ret");
    $Log->exit() if $may_exit;
    return $ret;
}

sub get_allowed_options_for_command {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $compat_hashref;

    # compatibility() is not defined in the vserver context. Hence,
    # we need to check if the method exists.
    if ($self->get_version_manager()->can('compatibility')) {
        $compat_hashref = $self->get_version_manager()->compatibility();
    }
    my @allowed_options = $self->SUPER::get_allowed_options_for_command(@_,
            compatibility => $compat_hashref);
    $Log->exit() if $may_exit;
    return @allowed_options;
}

sub _debug_smdb {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args = $self->validate_params(
        params => \@opts,
        spec   => {
            table               => { type => SCALAR },
            action              => { type => SCALAR },
        },
        allow_extra => 1
    );
    my $table;
    if ( exists $args{table} ) {
        $table = delete( $args{table} );
    }
    my $action = delete( $args{action} );

    # instantiate cdef object
    my $cmd      = "debug smdb table $table $action";
    my $api_cmd = "debug_smdb_table_" . $table . "_" . $action;
    try {
        $self->{schema}->get_command_definition( command => $api_cmd );
    } catch NACL::APISet::Exceptions::MethodNotFoundException with {
        # per cdef object, store it in schema object
        my $cdef_obj = NACL::APISet::Schema::Cdef->new(
            command       => $cmd,
            alias_params  => {},
            option_order  => [],
            file          => 'CLI/CMode.pm',
            concat_limits => [],
            options_sep   => {},
            switch_identifier => '-',
        );
        $self->{schema}->set_command_definition($api_cmd => $cdef_obj);
    };
    if (defined($args{'nacl_table_filter'}))
    {
     $args{table} = delete $args{nacl_table_filter};
    }
    my $resp = $self->_invoke( method => $api_cmd, %args );
    $Log->exit;
    return $resp;
}

sub translate_api_to_command {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = validate(@args, {api => {type => SCALAR}});
    my $cdef = $self->get_command_definition(command => $opts{api});
    my $cmd = $cdef->{command};
    $Log->exit() if $may_exit;
    return $cmd;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::CLI::CMode - API Set for cluster mode CLI commands

=head1 SYNOPSIS

 use NACL::APISet::Node::CLI::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Node::CLI::CMode> is used to connect to and send CLI commands to the
cluster interface of a node.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Node::CLI::CMode>.

=over 

=item Synopsis

 $api_set_obj = NACL::APISet::Node::CLI::CMode->new( %args );

=item Arguments

=over 

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new API Set for Cluster Mode.

=back

=head1 METHODS

=head2 set_privilege

This method is to be used to set the privilege level

=over

=item Synopsis

$Api_Set_Obj->set_privilege('privilege-level' => $privilege);

=item Arguments

=over

=item privilege-level

(Mandatory)
The privilege level to be set to. Valid values are diagnostic, advanced and admin

=back

=item Returns

Nothing.

=back

=head2 clear_privilege

This method is to be used to reset the privilege level to its default value
(diagnostic)

=over

=item Synopsis

$Api_Set_Obj->clear_privilege();

=item Arguments

None

=item Returns

Nothing.

=back

=head2 set_vserver

This method is to be used to set the vserver context

=over

=item Synopsis

$Api_Set_Obj->set_vserver('vserver-context-name' => $vserver);

=item Arguments

=over

=item vserver-context-name

(Mandatory)
The vserver name in the context of which the commands are to be executed

=back

=item Returns

Nothing.

=back

=head2 get_vserver

This method is to be used to get the vserver context

=over

=item Synopsis

$vserver_context=$Api_Set_Obj->get_vserver();

=item Arguments

None

=item Returns

Name of vserver in whose context the command should be run.

=back

=head2 clear_vserver

This method is to be used to clear the vserver context.

=over

=item Synopsis

$Api_Set_Obj->clear_server();

=item Arguments

None

=item Returns

Nothing.

=back

=head2 get_valid_values

This method gets all the possible values which a particulr option in a command
can take.

=over

=item Synopsis

$Api_Set_Obj->get_valid_values('api' => 'lun_create', option => 'ostype');

=item Arguments

=over

=item api

(Mandatory)
The name of the api command

=item option

(Mandatory)
The name of the option whose valid values need to be found

=back

=item Returns

It returns an array reference of the valid values

        [
          'vmware',
          'hyper_v',
          'windows_2008',
          'windows_gpt',
          'windows',
          'linux',
          'xen',
          'solaris',
          'solaris_efi',
          'hpux',
          'aix',
          'netware',
          'openvms'
        ];

=back

=head2 cli_command_helper

This method walks the UI by issuing ? for a given command ,parses the output
returns back a hashref which contains the options of the command , its option 
datatype and its description. It also discovers whether the command is a show
command or not

=over

=item Synopsis

$Api_Set_Obj->cli_command_helper('api' => 'lun_create');

=item Arguments

=over

=item api

(Mandatory)
The name of the api command

=item privilege-level

(Optional)
The privilege level at which to fetch the details about the command.
Default is whatever the current privilege level of apiset object.

=back

=item Returns

It returns a hash reference 
Sample hash structure for lun_create command: 

                       'is_CS' => 0,
                       'fields' => {
                                     'extent-size' => {
                                                        'description' => [
                                                                           '*Extent Size'
                                                                         ],
                                                        'is_array' => 0,
                                                        'field' => 'extent-size'
                                                      },
                                     'file-path' => {
                                                      'description' => [
                                                                         'File Path'
                                                                       ],
                                                      'is_array' => 0,
                                                      'field' => 'file-path'
                                                    },
                                     'volume' => {
                                                   'is_array' => 0,
                                                   'field' => 'volume'
                                                 },
                                     'vserver' => {
                                                    'description' => [
                                                                       'Vserver Name'
                                                                     ],
                                                    'is_array' => 0,
                                                    'field' => 'vserver'
                                                  },
                                     'suffix-size' => {
                                                        'description' => [
                                                                           '*Suffix Size'
                                                                         ],
                                                        'is_array' => 0,
                                                        'field' => 'suffix-size'
                                                       }
                           }

is_CS => 1 , incase of commands which take -fields options

Sample hash structure for antivirus_show command:

                             'is_singleton' => 1,
                             'is_CS' => 1,
                             'fields' => {
                                           'last-update-reason' => {
                                                                     'is_array' => 0,
                                                                     'field' => 'last-update-reason'
                                                                   },
                                           'internal-version' => {
                                                                   'is_array' => 0,
                                                                   'field' => 'internal-version'
                                                                 },
                                           'target-nodes' => {
                                                               'is_array' => 0,
                                                               'field' => 'target-nodes'
                                                             },
                                           'version' => {
                                                          'is_array' => 0,
                                                          'field' => 'version'
                                                        },
                                           'last-update' => {
                                                              'is_array' => 0,
                                                              'field' => 'last-update'
                                                            },
                                           'auto-update' => {
                                                              'is_array' => 0,
                                                              'field' => 'auto-update'
                                                            },
                                           'current-nodes' => {
                                                                'is_array' => 0,
                                                                'field' => 'current-nodes'
                                                              }
                                         },


=back

=head2 is_command_present

This method can be used to check if the specified command is supported in the 
build or not.

=over

=item Synopsis

my $is_present_boolean = 
$Api_Set_Obj->is_command_present('api' => 'snapmirror_break');

=item Arguments

=over

=item api

(Mandatory)
The command to be verified specified as method name.
Note the use of underscores instead of spaces.

=item privilege-level

(Optional)
The privilege level at which to verify the presence of the command.
Default is whatever the current privilege level of apiset object.

=back

=item Returns

1 or 0. 1 if the command is present, 0 if it's not present.

=back

=head2 is_option_present

This method can be used to check if the specified command supports a specific
option or not.

=over

=item Synopsis

my $is_present_boolean = $Api_Set_Obj->is_option_present(
    'api' => 'snapmirror_update', 
    field => 'source-snapshot'
);

=item Arguments

=over

=item api

(Mandatory)
The command to be verified specified as method name.
Note the use of underscores instead of spaces.

=item privilege-level

(Optional)
The privilege level at which to verify the presence of the command.
Default is whatever the current privilege level of apiset object.

=item field
(Mandatory)
Name of the field

=back

=item Returns

1 or 0. 1 if the option is present, 0 if it's not present.

=back

=head2 debug_smdb_table_show

This method is used to perform debug smdb table show.

=over

=item Synopsis

$Api_Set_Obj->debug_smdb_table_show('table' => $table_name , 'node'=> $node, 'nacl_table_filter' => $table );

=item Arguments

=over

=item table

(Mandatory)
Name of the smdb table on which show operation has to be done.

=item node

(Optional)
Name of the smdb node on which show operation has to be done.

=item nacl_table_filter

(Optional)
Scalar that contains table filter.


# if the command accepts "-table" as an option, then that can be specified through "nacl_table_filter"
# For example, issuing "debug smdb table bcomka_cache_summary show -table vdisk" can be done like this:
# $Api_Set_Obj->debug_smdb_table_show(table => 'bcomka_cache_summary', nacl_table_filter => 'vdisk');

=back

=item Returns

Returns the response object after invoking api command.

=back

=head2 debug_smdb_table_run

This method is used to perform debug smdb table run.

=over

=item Synopsis

$Api_Set_Obj->debug_smdb_table_run('table' => $table_name , 'node'=> $node, %other_opts);

=item Arguments

=over

=item table

(Mandatory)
Name of the smdb table on which run operation has to be done.

=item node

(Optional)
Name of the smdb node on which run operation has to be done.

=back

=item Returns

Returns the response object after invoking api command.

=back

=head2 debug_smdb_table_modify

This method is used to modify debug smdb table.

=over

=item Synopsis

$Api_Set_Obj->debug_smdb_table_modify('table' => $table_name , 'node'=> $node, %other_opts);

=item Arguments

=over

=item table

(Mandatory)
Name of the smdb table which needs to be modified.

=item node

(Optional)
Name of the smdb node which needs to be modified.

=back

=item Returns

Returns the response object after invoking api command.

=back

=head2 debug_smdb_table_create

This method is used to create debug smdb table.

=over

=item Synopsis

$Api_Set_Obj->debug_smdb_table_create('table' => $table_name , 'node'=> $node, %other_opts);

=item Arguments

=over

=item table

(Mandatory)
Name of the smdb table which is to be created.

=item node

(Optional)
Name of the smdb node which is to be created.

=back

=item Returns

Returns the response object after invoking api command.

=back

=head2 debug_smdb_table_delete

This method is used to delete a  debug smdb table.

=over

=item Synopsis

$Api_Set_Obj->debug_smdb_table_delete('table' => $table_name , 'node'=> $node, %other_opts);

=item Arguments

=over

=item table

(Mandatory)
Name of the smdb table which is to be deleted.

=item node

(Optional)
Name of the smdb node which is to be deleted.

=back

=item Returns

Returns the response object after invoking api command.

=back

=head2 translate_api_to_command

    my $cmd = $apiset->translate_api_to_command(api => $api);

Returns the command name associated with a particular API. This is done by
looking-up the Schema object. For example, if the API is
"vserver_export_policy_create", then command returned would be
"vserver export-policy create".

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
