#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for CLI Firmware commands
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::CLI::Firmware;

#
# Compiler Directives
#
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Module import - APISets
use NACL::APISet::Session::CLI::Firmware;

#
#Exceptions
#
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
use Params::Validate qw(SCALAR);
use Data::Dumper;


sub new
{
    $Log->enter();
    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;
    my $self = {};


    $self = NACL::APISet::validate_params (
        $class,
        params => \%args,
        spec   => {
            version   => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit();
    return $self;
}

##############################################################################
# Method:    set_privilege
# Objective: Set the current privilege level
##############################################################################
# Setting privilege for Firmware is done directly by using execute_command

sub set_privilege {
    $Log->enter();

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "privilege-level" => { type => SCALAR } }
    );
    my $priv = $args{'privilege-level'};

    #Keep "privilege-level" value as empty since "priv set" is sufficient to set admin privilege at the LOADER prompt
    $priv = "" if ($priv eq 'admin');

    $self->execute_command( command => "priv set $priv");

    $self->{"privilege-level"} = $args{"privilege-level"};

    $Log->exit();
} ## end sub set_privilege

##############################################################################
# Method:    clear_privilege
# Objective: Set the default  privilege level
##############################################################################
sub clear_privilege {
    $Log->enter();
    my ($self) = @_;
    $self->set_privilege( 'privilege-level' => 'test' );
    $Log->exit();
}


##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Firmware API Set
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub _init_API
{
    $Log->enter();

    my ($self, %args) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file($self->{cdef_path});

    unless (defined $self->{cdef_path}) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason:\n$@");
    }

    $self->SUPER::_init_API(%args);

    if ( defined $self->{'privilege-level'} ) {
        $self->set_privilege('privilege-level' => $self->{'privilege-level'} );
    }else {
        $self->clear_privilege();
    }

    $Log->exit();
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::CLI::Firmware - API Set for Firmware CLI commands

=head1 SYNOPSIS

use NACL::APISet::Node::CLI::Firmware;

=head1 DESCRIPTION

A C<NACL::APISet::Node::CLI::Firmware> is used to connect to and send Firmware CLI
commands to the 7Mode/CMode Node.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::CLI::Firmware>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Node::CLI::Firmware->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new CLI Firmware API Set.

=back

=head1 METHODS

=head2 set_privilege

This method is to be used to set the privilege level

=over

=head2 clear_privilege

This method is to be used to reset the privilege level to its default value
(test)

=over

=cut
