#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for Maintenance mode CLI commands
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::CLI::Maintenance;

#
# Compiler Directives
#
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::Node::CLI);

# Module import - APISets
use NACL::APISet::Session::CLI::Maintenance;
use Tie::IxHash;

#
#Exceptions
#
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALAR);
use Data::Dumper;

sub new {
    $Log->enter() if $may_enter;

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;
    my $self = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            version => { type => SCALAR, optional => 1, default => 99.99 },
        }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
}


##############################################################################
# Method:    _init_API
# Objective: Initialize the Maintenance API Set
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub _init_API {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file( $self->{cdef_path} );

    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason:\n$@");
    }

    $self->SUPER::_init_API(%args);
    
    $Log->exit() if $may_exit;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::CLI::Maintenance - API Set for Maintenance mode CLI commands

=head1 SYNOPSIS

use NACL::APISet::Node::CLI::Maintenance;

=head1 DESCRIPTION

A C<NACL::APISet::Node::CLI::Maintenance> is used to connect to and send CLI commands
to the Node.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::CLI::Maintenance>.

=over 

=item Synopsis

 $api_set_obj = NACL::APISet::Node::CLI::Maintenance->new( %args );

=item Arguments

=over 

=item version

(Optional)
Version string 

=back

=item Returns

A reference to a new CLI Maintenance mode API Set.

=back

=head1 METHODS

=head2 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id $

=cut
