# Copyright (c) 2012 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse SP date command
##
## @description
##    This is a library to parse the raw output of SP date command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "date" will be date()
##
## @synopsis
##    use NACL::APISet::NODE::CLI::SP::default::date;
##
##    date(%args);
##
## @author sheethal@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
package NACL::APISet::Node::CLI::SP::default::date;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;


## @name ::date
##
## @summary Parses the raw output from the "date" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from date with any of its options as described in the 
##    corresponding cdef file 'date.cdefs'
## @synopsis
##      date(%args);
## INPUTS TO METHOD CALL 'date' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = date
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Mon Jan 14 10:02:35 GMT 2013
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'date' => 'Mon Jan 14 10:02:35 GMT 2013'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub date {

    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my (@result_array);
    my @lines = get_lines( raw_output => $output, response_object => $resp );
                    push( @result_array, { date => $lines[$0] } );
    return \@result_array;
}
 ## end sub date

sub _check_error {

    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /date:\s+no\s+support.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /([Uu]sage:|date).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error
1;
