#
# $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI/SP/default/ping.pm#1 $
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    use ping;
##
##    ping(%args);
##
## @summary Routines to parse ping command output
## @author dl-nacl-dev

package NACL::APISet::Node::CLI::SP::default::ping;

###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{"response_object"};
    my $error;
    my $error_message;
    if ( $output =~ /ping[:]+\s+(Domain not found|bad address|unknown host)/i ) {
	$error_message = $output;
        $error = "CommandFailed";
    }elsif ( $output =~ /(usage|Invalid\s+command\s+usage)/i ) {
        $error         = "InvalidParam";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping with any of its options as described in the 
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'iterface' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'size' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ping -c 6 10.224.203.68
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING 10.224.203.68 (10.224.203.68): 56 data bytes
##&nbsp;64 bytes from 10.224.203.68: seq=0 ttl=255 time=1.711 ms
##&nbsp;64 bytes from 10.224.203.68: seq=1 ttl=255 time=0.611 ms
##&nbsp;64 bytes from 10.224.203.68: seq=2 ttl=255 time=0.600 ms
##&nbsp;64 bytes from 10.224.203.68: seq=3 ttl=255 time=0.582 ms
##&nbsp;64 bytes from 10.224.203.68: seq=4 ttl=255 time=0.639 ms
##&nbsp;64 bytes from 10.224.203.68: seq=5 ttl=255 time=0.618 ms
##&nbsp;
##&nbsp;--- 10.224.203.68 ping statistics ---
##&nbsp;6 packets transmitted, 6 packets received, 0% packet loss
##&nbsp;round-trip min/avg/max = 0.582/0.793/1.711 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'source' => '10.224.203.68 ',
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'roundtrip_avg' => '0.793 ms',
##&nbsp;                                'roundtrip_min' => '0.582 ms',
##&nbsp;                                'packets_transmitted' => '6',
##&nbsp;                                'packets_loss' => '0%',
##&nbsp;                                'packets_received' => '6',
##&nbsp;                                'roundtrip_max' => '1.711 ms'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'hops' => [
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '1',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '0',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '1',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '2',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '3',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '4',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '255',
##&nbsp;                          'seq' => '5',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'packetsize' => '56'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping -c 6 10.224.203.68
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING 10.224.203.68 (10.224.203.68) 56(84) bytes of data.
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=1 ttl=252 time=0.568 ms
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=2 ttl=252 time=0.558 ms
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=3 ttl=252 time=0.584 ms
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=4 ttl=252 time=0.558 ms
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=5 ttl=252 time=0.564 ms
##&nbsp;64 bytes from 10.224.203.68: icmp_seq=6 ttl=252 time=0.565 ms
##&nbsp;
##&nbsp;--- 10.224.203.68 ping statistics ---
##&nbsp;6 packets transmitted, 6 received, 0% packet loss, time 5009ms
##&nbsp;rtt min/avg/max/mdev = 0.558/0.566/0.584/0.016 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'roundtrip_avg' => '0.566 ms',
##&nbsp;                                'packets_loss' => '0%',
##&nbsp;                                'packets_received' => '6',
##&nbsp;                                'roundtrip_mdev' => '0.016 ms',
##&nbsp;                                'packets_transmitted' => '6',
##&nbsp;                                'roundtrip_min' => '0.558 ms',
##&nbsp;                                'total_time' => '5009ms',
##&nbsp;                                'roundtrip_max' => '0.584 ms'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'hops' => [
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '1',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '2',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '3',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '4',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '5',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '252',
##&nbsp;                          'seq' => '6',
##&nbsp;                          'node' => '10.224.203.68'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'host' => '10.224.203.68',
##&nbsp;            'packetsize' => '56(84) bytes of data.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    _check_error(
        "raw_output"      => $output,
        "response_object" => $resp
    );

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $first_line = $lines[0];
	if ( $first_line =~ /PING\s*(.*)\s+\((.*)\)\s+(.*\(.*\).*)/ )
    {
        $result_array[0]->{'packetsize'} = $3;
        $result_array[0]->{'host'}       = $1;
    }
    shift @lines;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line
            =~ /(\d*)\s*bytes\s*from\s*(.*)\s*[:]\s*(seq|icmp_seq)=(\d*)\s*ttl=(\d*)\s*time=(\d*\s*\w*)/
            )
        {
            $result_array[0]->{'hops'}->[$i]->{'bytes'} = $1;
            $result_array[0]->{'hops'}->[$i]->{'node'}  = $2;
            $result_array[0]->{'hops'}->[$i]->{'seq'}   = $4;
            $result_array[0]->{'hops'}->[$i]->{'limit'} = $5;
            $result_array[0]->{'hops'}->[$i]->{'time'}  = $6;
            $i++;
        } elsif ( $line
            =~ /(\d+)\spackets transmitted,\s(\d+)\s(packets received|received),\s(\S+%)\spacket loss(,\s+time\s+(.+))*/
            )
        {
            $result_array[0]->{'statistics'}->[0]->{'packets_transmitted'} = $1;
            $result_array[0]->{'statistics'}->[0]->{'packets_received'}    = $2;
            $result_array[0]->{'statistics'}->[0]->{'packets_loss'}        = $4;
	    my $time_value = $6;
	    if($time_value){
	        $result_array[0]->{'statistics'}->[0]->{'total_time'} = $time_value;
	    }
        } elsif ( $line
            =~ /(round-trip|rtt)\s*min\/*avg\/*max(\/mdev)*\s*=\s*(\d+\.?\d+)\/(\d+\.?\d*)\/(\d+\.?\d*)(\/(\d+\.?\d*))*\s*(\w*)/
            )
        {
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_min'}
                = $3 . ' ' . $8;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_avg'}
                = $4 . ' ' . $8;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_max'}
                = $5 . ' ' . $8;
            my $mdev_value = $7;
	    if($mdev_value){
	       $result_array[0]->{'statistics'}->[0]->{'roundtrip_mdev'} = $mdev_value.' '.$8;
	    }
        }

    }
    return \@result_array;
}

1;
