#
# $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI/SP/default/ping6.pm#1 $
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    use ping6;
##
##    ping6(%args);
##
## @summary Routines to parse ping6 command output
## @author dl-nacl-dev

package NACL::APISet::Node::CLI::SP::default::ping6;

###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{"response_object"};
    my $error;
    my $error_message;
    if ( $output =~ /ping6[:]+\s+(Domain not found|bad address|unknown host)/i ) {
        $error_message = $output;
        $error = "CommandFailed";
    }elsif ( $output =~ /(usage|Invalid\s+command\s+usage)/i ) {
        $error         = "InvalidParam";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::ping6
##
## @summary Parses the raw output from the "ping6" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ping6 with any of its options as described in the 
##    corresponding cdef file 'ping6.cdefs'
## @synopsis
##      ping6(%args);
## INPUTS TO METHOD CALL 'ping6' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'count' </td> <td> '0' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'iterface' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'size' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ping6 -c 5 fe80::2a0:98ff:fe13:d0d4
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING fe80::2a0:98ff:fe13:d0d4 (fe80::2a0:98ff:fe13:d0d4): 56 data bytes
##&nbsp;64 bytes from fe80::2a0:98ff:fe13:d0d4: seq=0 ttl=64 time=0.706 ms
##&nbsp;64 bytes from fe80::2a0:98ff:fe13:d0d4: seq=1 ttl=64 time=0.621 ms
##&nbsp;64 bytes from fe80::2a0:98ff:fe13:d0d4: seq=2 ttl=64 time=0.642 ms
##&nbsp;64 bytes from fe80::2a0:98ff:fe13:d0d4: seq=3 ttl=64 time=0.639 ms
##&nbsp;64 bytes from fe80::2a0:98ff:fe13:d0d4: seq=4 ttl=64 time=0.638 ms
##&nbsp;
##&nbsp;--- fe80::2a0:98ff:fe13:d0d4 ping statistics ---
##&nbsp;5 packets transmitted, 5 packets received, 0% packet loss
##&nbsp;round-trip min/avg/max = 0.621/0.649/0.706 ms
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'host' => 'fe80::2a0:98ff:fe13:d0d4',
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'roundtrip_avg' => '0.649 ms',
##&nbsp;                                'roundtrip_min' => '0.621 ms',
##&nbsp;                                'roundtrip_max' => '0.706 ms'
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'packets_transmitted' => '5',
##&nbsp;            'packets_loss' => '0%',
##&nbsp;            'packets_received' => '5',
##&nbsp;            'hops' => [
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '64',
##&nbsp;                          'seq' => '0',
##&nbsp;                          'node' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '64',
##&nbsp;                          'seq' => '1',
##&nbsp;                          'node' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '64',
##&nbsp;                          'seq' => '2',
##&nbsp;                          'node' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '64',
##&nbsp;                          'seq' => '3',
##&nbsp;                          'node' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'bytes' => '64',
##&nbsp;                          'time' => '0',
##&nbsp;                          'limit' => '64',
##&nbsp;                          'seq' => '4',
##&nbsp;                          'node' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'packetsize' => '56 data bytes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = ping6 -c 5 fe80::2a0:98ff:fe13:d0d4
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;PING fe80::2a0:98ff:fe13:d0d4 (fe80::2a0:98ff:fe13:d0d4): 56 data bytes
##&nbsp;ping6: sendto: Network is unreachable
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'Network is unreachable',
##&nbsp;            'packetsize' => '56 data bytes',
##&nbsp;            'host' => 'fe80::2a0:98ff:fe13:d0d4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping6 {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    _check_error(
        "raw_output"      => $output,
        "response_object" => $resp
    );

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $first_line = $lines[0];

    if ( $first_line =~ /PING\s*(.*)\s+\((.*)\)\s*[:]+\s+(.+)/ )
    {
        $result_array[0]->{'host'}     = $1;
        $result_array[0]->{'packetsize'} = $3;
    }
    shift @lines;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line
            =~ /(\d*)\s*bytes\s*from\s*(.*)\s*[:]\s*seq=(\d*)\s*ttl=(\d*)\s*time=(\d*\s*\w*)/
            )
        {
            $result_array[0]->{'hops'}->[$i]->{'bytes'} = $1;
            $result_array[0]->{'hops'}->[$i]->{'node'}  = $2;
            $result_array[0]->{'hops'}->[$i]->{'seq'}   = $3;
            $result_array[0]->{'hops'}->[$i]->{'limit'} = $4;
            $result_array[0]->{'hops'}->[$i]->{'time'}  = $5;
            $i++;
        } elsif ( $line
            =~ /(\d+)\spackets transmitted,\s(\d+)\spackets received,\s(\S+%)\spacket loss/
            )
        {
            $result_array[0]->{'packets_transmitted'} = $1;
            $result_array[0]->{'packets_received'}    = $2;
            $result_array[0]->{'packets_loss'}        = $3;
        } elsif ( $line
            =~ /round-trip\s*min\/*avg\/*max\s*=\s*(\d+\.?\d+)\/(\d+\.?\d*)\/(\d+\.?\d*)\s*(\w*)/
            )
        {
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_min'}
                = $1 . ' ' . $4;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_avg'}
                = $2 . ' ' . $4;
            $result_array[0]->{'statistics'}->[0]->{'roundtrip_max'}
                = $3 . ' ' . $4;
        }elsif( $line =~ /ping6[:]+\s+sendto[:]+\s+(.+)/){
		    $result_array[0]->{'result'} = $1;
		}

    }
    return \@result_array;
}

1;
