#
#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse SP rsa command
##
## @description
##    This is a library to parse the raw output of sp rsa subcommands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rsa setup" will be rsa_setup()
##
## @synopsis
##    use NACL::APISet::Node::CLI::SP:default::rsa
##
##    rsa_setup(%agrs);
##
## @author dl-nacl-dev@netapp.com,Veerabhadraswamy.P@netapp.com
##
## @status public
##
package NACL::APISet::Node::CLI::SP::default::rsa;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::rsa_setup
##
## @summary Parses the raw output from the "rsa setup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rsa_setup with any of its options as described in the
##    corresponding cdef file 'rsa.cdefs'
## @synopsis
##      rsa_setup(%args);
## INPUTS TO METHOD CALL 'rsa_setup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-password' </td> <td> 'password' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr>
## <tr> <td> '-ssl' </td> <td> 'ssl' </td> <td> '0' </td> </tr>
## <tr> <td> '-hostname' </td> <td> 'hostname' </td> <td> '0' </td> </tr>
## <tr> <td> '-port' </td> <td> 'port' </td> <td> '0' </td> </tr>
## <tr> <td> '-username' </td> <td> 'username' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub rsa_setup {
    my %args = @_;
    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );

    return [];
} ## end of sub rsa_setup

## @name ::rsa_status
##
## @summary Parses the raw output from the "rsa status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rsa_status with any of its options as described in the
##    corresponding cdef file 'rsa.cdefs'
## @synopsis
##      rsa_status(%args);
## INPUTS TO METHOD CALL 'rsa_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rsa status -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Remote Support Agent is enabled.
##&nbsp;
##&nbsp;Connection status:
##&nbsp;HTTP:
##&nbsp;Status              : failed
##&nbsp;Last checked        : 11:22 Jun 27 2013
##&nbsp;Last successful at  : not available
##&nbsp;Failed since        : 06:18 Jun 27 2013
##&nbsp;Failure reason      : HTTP health check interface busy
##&nbsp;
##&nbsp;RSE:
##&nbsp;Status              : ok
##&nbsp;Last checked        : 11:22 Jun 27 2013
##&nbsp;
##&nbsp;
##&nbsp;Support Information:
##&nbsp;System ID           : 1785944316
##&nbsp;Heartbeat check     : every 5 minutes
##&nbsp;
##&nbsp;Recent activity:
##&nbsp;No recent activity found
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'recent_activity' => 'No recent activity found',
##&nbsp;            'support_information' => {
##&nbsp;                                       'heartbeat_check' => 'every 5 minutes',
##&nbsp;                                       'system_id' => '1785944316'
##&nbsp;                                     },
##&nbsp;            'remote_support_agent' => 'enabled',
##&nbsp;            'connection_status' => {
##&nbsp;                                     'http' => {
##&nbsp;                                                 'last_successful_at' => 'not available',
##&nbsp;                                                 'failure_reason' => 'HTTP health check interface busy',
##&nbsp;                                                 'last_checked' => '11:22 Jun 27 2013',
##&nbsp;                                                 'failed_since' => '06:18 Jun 27 2013',
##&nbsp;                                                 'status' => 'failed'
##&nbsp;                                               },
##&nbsp;                                     'rse' => {
##&nbsp;                                                'last_checked' => '11:22 Jun 27 2013',
##&nbsp;                                                'status' => 'ok'
##&nbsp;                                              }
##&nbsp;                                   }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rsa_status {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my ( $head_key, $sub_key );

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /(Remote Support Agent) is (\w+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line
            =~ /(Connection status|Support Information|Recent activity)/ )
        {
            $head_key = to_key_name( item => trim_string( 'string' => $1 ) );
        } elsif ( $line =~ /(HTTP:$|RSE:$|Directory listing:)/ ) {
            $sub_key = to_key_name( item => trim_string( 'string' => $1 ) );
        } elsif ( $line =~ /^(\w.*)\s+:\s+(.*)/
            and $head_key =~ /connection_status|recent_activity/
            and $command  =~ /-v/ )
        {
            my ( $key, $value ) = split( '\s+:\s+', $line );
            $result_array[0]->{$head_key}->{$sub_key}
                ->{ to_key_name( item => trim_string( 'string' => $key ) ) }
                = $value;
        } elsif ( $line =~ /(No recent activity found)/ ) {
            $result_array[0]->{'recent_activity'} = $1;
        } elsif ( $line =~ /^(\w.*)\s+:\s+(.*)/ ) {
            $result_array[0]->{$head_key}
                ->{ to_key_name( item => trim_string( 'string' => $1 ) ) }
                = $2;
        }
    }

    return \@result_array;
} ## end of sub rsa_status

## @name ::rsa_show
##
## @summary Parses the raw output from the "rsa show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rsa_show with any of its options as described in the
##    corresponding cdef file 'rsa.cdefs'
## @synopsis
##      rsa_show(%args);
## INPUTS TO METHOD CALL 'rsa_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rsa show
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Remote Support Agent is enabled.
##&nbsp;
##&nbsp;Remote Support Enterprise URL: https://remotesupportagent.netapp.com/eMessage
##&nbsp;
##&nbsp;Use proxy: no
##&nbsp;
##&nbsp;Cluster management LIF: 10.226.28.117
##&nbsp;HTTP with SSL enabled: no
##&nbsp;Storage controller HTTP port: 80
##&nbsp;Storage controller HTTP username: root
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'remote_support_agent' => 'enabled',
##&nbsp;            'http_with_ssl_enabled' => 'no',
##&nbsp;            'storage_controller_http_port' => '80',
##&nbsp;            'cluster_management_lif' => '10.226.28.117',
##&nbsp;            'storage_controller_http_username' => 'root',
##&nbsp;            'use_proxy' => 'no',
##&nbsp;            'remote_support_enterprise_url' => 'https://remotesupportagent.netapp.com/eMessage'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rsa_show {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /(Remote Support Agent) is (\w+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /^(\w.*):\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
} ## end of sub rsa_show

## @name ::rsa_test
##
## @summary Parses the raw output from the "rsa test" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rsa_test with any of its options as described in the
##    corresponding cdef file 'rsa.cdefs'
## @synopsis
##      rsa_test(%args);
## INPUTS TO METHOD CALL 'rsa_test' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'group' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rsa test
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Testing cluster management LIF HTTP connection ..................... failed
##&nbsp;HTTP health check interface busy. Please try again later
##&nbsp;Testing Remote Support Enterprise connection ....................... ok
##&nbsp;One or more configuration tests failed.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'testing_remote_support_enterprise_connection' => 'ok',
##&nbsp;            'testing_cluster_management_lif_http_connection' => 'failed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rsa_test {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /(Testing.*)\s+..*\s+(\w+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
} ## end of sub rsa_test

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /USAGE:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error
1;
