#
#Copyright (c) 2016 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse default_fw_program command output
##
## @description
##    This is a library to parse the raw output of default_fw_program command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::SPShell::default::default_fw_program;
##
##  default_fw_program(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::SPShell::default::default_fw_program;

use NACL::APISet::Exceptions::InvalidParamException;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;

    if ( $raw_output =~ /([Uu]sage:\s+default_fw_program.*)/ ) {
        $error         = "InvalidParam";
        $error_message = $1;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::default_fw_program
##
## @summary Parses the raw output from the "/usr/local/bin/default_fw_program.sh" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from default_fw_program with any of its options as described in the 
##    corresponding cdef file 'default_fw_program.cdefs'
## @synopsis
##      default_fw_program(%args);
## INPUTS TO METHOD CALL 'default_fw_program' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'all' </td> <td> 'all-device' </td> <td> '1' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = /usr/local/bin/default_fw_program.sh -v all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;battery:
##&nbsp;
##&nbsp;0x0012
##&nbsp;
##&nbsp;eth_switch:
##&nbsp;mb: Version2 SB_XX Switch0  
##&nbsp;
##&nbsp;support_fpga:
##&nbsp;Retrieving version of support FPGA
##&nbsp;1.1
##&nbsp;
##&nbsp;seq_cpld:
##&nbsp;Retrieving version of sequencing CPLD
##&nbsp;1.1
##&nbsp;
##&nbsp;nvram_cpld:
##&nbsp;Retrieving version of NVRAM CPLD
##&nbsp;0.18
##&nbsp;
##&nbsp;nvram_fpga:
##&nbsp;Retrieving version of NVRAM FPGA
##&nbsp;1.3.0
##&nbsp;
##&nbsp;pcie_switch:
##&nbsp;id0: 3.02
##&nbsp;Done
##&nbsp;
##&nbsp;ddr_eeprom:
##&nbsp;Version A0 SB_XX SP DDR2        
##&nbsp;
##&nbsp;fan:
##&nbsp; Fan_A1:
##&nbsp;    Front: Version: FC1.7
##&nbsp;    Back: Version: FC1.7
##&nbsp; Fan_A2:
##&nbsp;    Front: Version: FC1.7
##&nbsp;    Back: Version: FC1.7
##&nbsp; Fan_A3:
##&nbsp;    Front: Version: FC1.7
##&nbsp;    Back: Version: FC1.7
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'battery' => '0x0012'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'eth_switch' => 'mb: Version2 SB_XX Switch0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'support_fpga' => '1.1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'seq_cpld' => '1.1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'nvram_cpld' => '0.18'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'nvram_fpga' => '1.3.0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'pcie_switch' => 'id0: 3.02'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ddr_eeprom' => 'Version A0 SB_XX SP DDR2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fan' => {
##&nbsp;                       'Fan_A1' => {
##&nbsp;                                     'Back' => ' Version: FC1.7',
##&nbsp;                                     'Front' => ' Version: FC1.7'
##&nbsp;                                   },
##&nbsp;                       'Fan_A2' => {
##&nbsp;                                     'Back' => ' Version: FC1.7',
##&nbsp;                                     'Front' => ' Version: FC1.7'
##&nbsp;                                   },
##&nbsp;                       'Fan_A3' => {
##&nbsp;                                     'Back' => ' Version: FC1.7',
##&nbsp;                                     'Front' => ' Version: FC1.7'
##&nbsp;                                   }
##&nbsp;                     }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub default_fw_program {
    my %args = @_;
    my (@result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $option  = $command;
    $option =~ s/\/usr\/local\/bin\/default_fw_program.sh//;

    _check_error( "raw_output" => $output, "response_object" => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    if ( $option =~ /-v\s+all/ ) {
        my %fan_parent_hash;

        for ( my $i = 0; $i <= $#lines; $i++ ) {
            my %hash;
            my ( $key, $value ) = split( /:/, $lines[$i], 2 ) if ( $lines[$i] =~ /:/ );

            if ( defined($key) && ( !$value ) ) {
                if ( $key =~ /fan/ ) {
                    my ( $sub_key_old, %fan_hash );
                    while ( $i <= $#lines + 1 ) {
                        my ( $sub_key, $sub_value ) = split( /:/, $lines[$i], 2 ) if ( $lines[ ++$i ] =~ /:/ );

                        if ( ( $sub_key =~ /Fan_*/ ) || ( $i eq $#lines + 1 ) ) {
                            if ( (%fan_hash) && defined($sub_key_old) ) {
                                $fan_parent_hash{$sub_key_old} = {%fan_hash};
                                %fan_hash = ();
                            }
                            $sub_key_old = $sub_key;
                        } else {
                            $fan_hash{$sub_key} = $sub_value;
                        }
                    }
                    push( @result_array, { $key => \%fan_parent_hash } );
                } else {
                    $value = $lines[ ++$i ];
                    $value = $lines[ ++$i ] if ( $value =~ /Retrieving version of/i );
                    $hash{$key} = $value if ( defined $value );
                }
            } else {
                $hash{$key} = $value if ( defined $key );
            }
            push( @result_array, \%hash ) if (%hash);
        }
    }
    return \@result_array;
} ## end sub default_fw_program
1;
