#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/SPShell/default/i2c_test.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##  i2c_test(%args);
##
## @summary Routines to parse i2c-test command output
## @author sshaik(sshaik@netapp.com), dl-nacl-dev@netapp.com

package NACL::APISet::Node::CLI::SPShell::default::i2c_test;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

## @name ::i2c_test
##
## @summary Parses the raw output from the "i2c-test" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from i2c-test with any of its options as described in the
##    corresponding cdef file 'i2c_test.cdefs'
## @synopsis
##      i2c_test(%args);
## INPUTS TO METHOD CALL 'i2c_test' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-b' </td> <td> 'bus-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'slave' </td> <td> '0' </td> </tr>
## <tr> <td> '-scan' </td> <td> 'scan' </td> <td> '1' </td> </tr>
## <tr> <td> '-rc' </td> <td> 'count' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-mc' </td> <td> 'mux-channel' </td> <td> '0' </td> </tr>
## <tr> <td> '-ms' </td> <td> 'mux-slave' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'forever-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-delay' </td> <td> 'delay' </td> <td> '0' </td> </tr>
## <tr> <td> '-sbd' </td> <td> 'sbd' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'read' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'write' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = i2c-test -b 0 -s a0 -rc 16 -m 1 -d 0 0;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Bytes read: 16
##&nbsp;01 00 00 00 01 00 00 fe 01 0d 00 cc 4e 65 74 41
##&nbsp;Bytes written:   2
##&nbsp;00 00
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'bytes' => [
##&nbsp;                          '01',
##&nbsp;                          '00',
##&nbsp;                          '00',
##&nbsp;                          '00',
##&nbsp;                          '01',
##&nbsp;                          '00',
##&nbsp;                          '00',
##&nbsp;                          'fe',
##&nbsp;                          '01',
##&nbsp;                          '0d',
##&nbsp;                          '00',
##&nbsp;                          'cc',
##&nbsp;                          '4e',
##&nbsp;                          '65',
##&nbsp;                          '74',
##&nbsp;                          '41'
##&nbsp;                        ],
##&nbsp;             'bytes_read' => '16'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'bytes_written' => '2',
##&nbsp;             'bytes' => [
##&nbsp;                          '00',
##&nbsp;                          '00'
##&nbsp;                        ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = i2c-test -b 1 -scan -mc 70;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Found device at address 0x90 on bus 1
##&nbsp;Found device at address 0x92 on bus 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'device_address' => '0x90',
##&nbsp;             'bus' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'device_address' => '0x92',
##&nbsp;             'bus' => '1'
##&nbsp;           },
##&nbsp;         ]
##&nbsp;
##&nbsp;Command = i2c-test -b 3 -scan -ms ee -mc 7f;
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Found device at address 0x9c on bus 3 mux slave addr 0xee, mux channel 0x01 (segment 0)
##&nbsp;Found device at address 0x2e on bus 3 mux slave addr 0xee, mux channel 0x02 (segment 1)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'device_address' => '0x9c',
##&nbsp;             'mux_slave_addr' => '0xee',
##&nbsp;             'bus' => '3',
##&nbsp;             'segment' => '0',
##&nbsp;             'mux_channel' => '0x01'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'device_address' => '0x2e',
##&nbsp;             'mux_slave_addr' => '0xee',
##&nbsp;             'bus' => '3',
##&nbsp;             'segment' => '1',
##&nbsp;             'mux_channel' => '0x02'
##&nbsp;           },
##&nbsp;         ]
##&nbsp;

sub i2c_test {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count = 0;
    for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
        if ( $lines[$i] =~ /Found\s+device\s+at\s+address\s+(\S+)\s+on\s+bus\s+(\d+)\s*(.*)/) {
            $result_array[$count]->{device_address} = $1;
            $result_array[$count]->{bus}            = $2;
            my $temp = $3 if ($3);
            if($temp =~ /.*\s+(\S+)\,.*\s+(\S+)\s+\(\w+\s+(\d+)\)/){
                $result_array[$count]->{mux_slave_addr} = $1;
                $result_array[$count]->{mux_channel}    = $2;
                $result_array[$count]->{segment}        = $3;
            }
        } elsif ( $lines[$i] =~ /(Bytes\s+\w+):\s+(\d+)/ ) {
            my %bytes;
            my @values = split( /\s/, $lines[ $i + 1 ] );
            push( @{ $bytes{'bytes'} }, @values );
            push( @result_array,        \%bytes );
            $result_array[$count-1]->{to_key_name( item =>$1)} = $2;
        }
        $count++;
    }
    return \@result_array;

} ## end sub i2c_test

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{"response_object"};
    my ( $error, $error_message ) = undef;
    if ( $output =~ /[Uu]sage.*|Invalid\s+argument|^Error.*/ ) {
        $error         = "InvalidParam";
        $error_message = $output;
    } elsif ( $output =~ /(Missing argument.*)/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

1;
