#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse sel command output
##
## @description
##    This is a library to parse the raw output of sel command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::SPShell::default::sel;
##
##  sel(%args);
##
##  sel_clear(%args);
##  sel_rd_all(%args);
##
## @author anupamap@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::SPShell::default::sel;

use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

## @name ::sel_clear
##
## @summary Parses the raw output from the "sel clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp shell write with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sel_clear(%args);
## INPUTS TO METHOD CALL 'sel_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sel clear; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;priv set diag
##&nbsp;Warning: These diagnostic commands are for use by support personnel only.
##&nbsp;
##&nbsp;sel clear
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];

sub sel_clear {
    my %args = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _parse_for_errors(
        "raw_output"      => $output,
        "response_object" => "$resp"
    );

    return [];
}

## @name ::sel_rd_all
##
## @summary Parses the raw output from the "sel rd_all" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sp shell write with any of its options as described in the
##    corresponding cdef file 'sp.cdefs'
## @synopsis
##      sel_rd_all(%args);
## INPUTS TO METHOD CALL 'sel_rd_all' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sel rd_all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;priv set diag
##&nbsp;Warning: These diagnostic commands are for use by support personnel only.
##&nbsp;
##&nbsp;sel rd_all
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [];

sub sel_rd_all {
    my %args = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _parse_for_errors(
        "raw_output"      => $output,
        "response_object" => "$resp"
    );

    return [];
}


## @name ::sel_rd_first
##
## @summary Parses the raw output from the "sel rd_first" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sel_rd_first with any of its options as described in the
##    corresponding cdef file 'sel.cdefs'
## @synopsis
##      sel_rd_first(%args);
## INPUTS TO METHOD CALL 'sel_rd_first' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-recs' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sel_rd_first {
    my %args = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _parse_for_errors(
        "raw_output"      => $output,
        "response_object" => "$resp"
    );

    return [];
}

## @name ::sel_rd_last
##
## @summary Parses the raw output from the "sel rd_last" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sel_rd_last with any of its options as described in the
##    corresponding cdef file 'sel.cdefs'
## @synopsis
##      sel_rd_last(%args);
## INPUTS TO METHOD CALL 'sel_rd_last' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'num-recs' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples


sub sel_rd_last {
    my %args = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _parse_for_errors(
        "raw_output"      => $output,
        "response_object" => $resp
    );

    return [];
}

## @name ::sel_info
##
## @summary Parses the raw output from the "sel info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sel_info with any of its options as described in the
##    corresponding cdef file 'sel.cdefs'
## @synopsis
##      sel_info(%args);
## INPUTS TO METHOD CALL 'sel_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sel info
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;SEL version: 5
##&nbsp;Maximum number of records: 4000
##&nbsp;
##&nbsp;Number of records: 8
##&nbsp;Record size: 64 Bytes
##&nbsp;Oldest record's id: 1
##&nbsp;Latest record's id: 8
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'maximum_number_of_records' => '4000',
##&nbsp;            'oldest_records_id => '1',
##&nbsp;            'record_size' => '64 Bytes',
##&nbsp;            'number_of_records' => '8',
##&nbsp;            'latest_records_id' => '8',
##&nbsp;            'sel_version' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub sel_info {
    my %args = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _parse_for_errors(
        "raw_output"      => $output,
        "response_object" => "$resp"
    );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;

    foreach my $line (@lines) {
        $line =~ s/\'//;
        if ($line =~ /(.*):\s*(\d+\s*\w*)/){
            $result_array[0]{to_key_name(item => $1)}=$2;
        }

    }
    return \@result_array ;
}

sub _parse_for_errors {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{response_object};

    my ( $error, $error_message );
    if ( $output =~ /[Uu]sage.*|Available\s+options\:.*/i ) {
        $error         = "InvalidParam";
        $error_message = $output;

    }
    elsif ( $output =~ /(.*not supported.*)/i ) {
        $error_message = $1;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

}

1;
