#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/SPShell/default/switcheprm.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##  switcheprm(%args);
##
## @summary Routines to parse switcheprm command output
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com

package NACL::APISet::Node::CLI::SPShell::default::switcheprm;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();



## @name ::switcheprm
##
## @summary Parses the raw output from the "switcheprm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from switcheprm with any of its options as described in the
##    corresponding cdef file 'switcheprm.cdefs'
## @synopsis
##      switcheprm(%args);
## INPUTS TO METHOD CALL 'switcheprm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'compare_eeprom_vs_cfg_file' </td> <td> 'compare-eeprom-vs-cfg-file' </td> <td> '1' </td> </tr>
## <tr> <td> 'version_from_cfg_file' </td> <td> 'version-from-cfg-file' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> 'version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'offset-addr' </td> <td> '0' </td> </tr>
## <tr> <td> 'lib_write' </td> <td> 'lib-write' </td> <td> '1' </td> </tr>
## <tr> <td> 'read_cfg_file' </td> <td> 'read-cfg-file' </td> <td> '1' </td> </tr>
## <tr> <td> 'program_rd' </td> <td> 'program-read' </td> <td> '1' </td> </tr>
## <tr> <td> 'program_wr' </td> <td> 'program-write' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'length' </td> <td> '0' </td> </tr>
## <tr> <td> 'reset' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> 'reset_enet_switch' </td> <td> 'reset-enet-switch' </td> <td> '1' </td> </tr>
## <tr> <td> 'read_offset' </td> <td> 'read-offset' </td> <td> '1' </td> </tr>
## <tr> <td> 'write_one_word' </td> <td> 'write-one-word' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = switcheprm write_one_word 0x0000 0x8046
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Start = 0x0 , data = 0x8046
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'data' => '0x8046',
##&nbsp;            'start' => '0x0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = switcheprm read_offset 0x0000 4
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;   Ethernet switch configuration read from EEPROM
##&nbsp;
##&nbsp;   0x0000(000) :  0x8046  0x0004  0x8046  0x0028
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'switch_info' => [
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x8046',
##&nbsp;                                               '0x0004',
##&nbsp;                                               '0x8046',
##&nbsp;                                               '0x0028'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0000(000)'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = switcheprm read_cfg_file
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;   Ethernet switch configuration present in (/firmware/Buell/switch/controller_switch_img.txt)
##&nbsp;
##&nbsp;   0x0000(000) :  0x8026  0x0004  0x8046  0x0028  0x8086  0x0001  0x8106  0x0022
##&nbsp;   0x0008(008) :  0x8206  0x0000  0x8406  0x000a  0x8806  0x0000  0x8409  0x000a
##&nbsp;   0x0010(016) :  0x840a  0x80fa  0x8044  0x007c  0x7fe9  0x9000  0x7fea  0x0004
##&nbsp;   0x0018(024) :  0x7fe9  0xbf01  0x7fea  0x0004  0x7fe9  0xb110  0x7fea  0x0019
##&nbsp;   0x0020(032) :  0x7fe9  0xb111  0x7fea  0x0020  0x7fe9  0xb112  0x7fea  0x4c88
##&nbsp;   0x0028(040) :  0x7fe9  0xb113  0x7fea  0x0119  0x7fe9  0xb114  0x7fea  0xfff0
##&nbsp;   0x0030(048) :  0x7fe9  0xb115  0x7fea  0x00ff  0x7fe9  0xb116  0x7fea  0x1000
##&nbsp;   0x0038(056) :  0x7fe9  0xb117  0x8044  0x007f  0x8036  0x8088  0x8056  0x8088
##&nbsp;   0x0040(064) :  0x7ff9  0x0c00  0x7ff8  0x9409  0x7ff9  0x1340  0x7ff8  0x9400
##&nbsp;   0x0048(072) :  0x7ff9  0x0c00  0x7ff8  0x9449  0x7ff9  0x1340  0x7ff8  0x9440
##&nbsp;   0x0050(080) :  0xffff  0x5665  0x7273  0x696f  0x6e20  0x3200  0x0000  0x5342
##&nbsp;   0x0058(088) :  0x5f58  0x5800  0x0000  0x5377  0x6974  0x6368  0x2030  0x0000
##&nbsp;   0x0060(096) :  0xffff  0xffff  0x0000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'switch_info' => [
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x8026',
##&nbsp;                                               '0x0004',
##&nbsp;                                               '0x8046',
##&nbsp;                                               '0x0028',
##&nbsp;                                               '0x8086',
##&nbsp;                                               '0x0001',
##&nbsp;                                               '0x8106',
##&nbsp;                                               '0x0022'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0000(000)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x8206',
##&nbsp;                                               '0x0000',
##&nbsp;                                               '0x8406',
##&nbsp;                                               '0x000a',
##&nbsp;                                               '0x8806',
##&nbsp;                                               '0x0000',
##&nbsp;                                               '0x8409',
##&nbsp;                                               '0x000a'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0008(008)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x840a',
##&nbsp;                                               '0x80fa',
##&nbsp;                                               '0x8044',
##&nbsp;                                               '0x007c',
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0x9000',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x0004'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0010(016)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xbf01',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x0004',
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb110',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x0019'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0018(024)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb111',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x0020',
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb112',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x4c88'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0020(032)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb113',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x0119',
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb114',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0xfff0'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0028(040)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb115',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x00ff',
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb116',
##&nbsp;                                               '0x7fea',
##&nbsp;                                               '0x1000'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0030(048)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7fe9',
##&nbsp;                                               '0xb117',
##&nbsp;                                               '0x8044',
##&nbsp;                                               '0x007f',
##&nbsp;                                               '0x8036',
##&nbsp;                                               '0x8088',
##&nbsp;                                               '0x8056',
##&nbsp;                                               '0x8088'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0038(056)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7ff9',
##&nbsp;                                               '0x0c00',
##&nbsp;                                               '0x7ff8',
##&nbsp;                                               '0x9409',
##&nbsp;                                               '0x7ff9',
##&nbsp;                                               '0x1340',
##&nbsp;                                               '0x7ff8',
##&nbsp;                                               '0x9400'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0040(064)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x7ff9',
##&nbsp;                                               '0x0c00',
##&nbsp;                                               '0x7ff8',
##&nbsp;                                               '0x9449',
##&nbsp;                                               '0x7ff9',
##&nbsp;                                               '0x1340',
##&nbsp;                                               '0x7ff8',
##&nbsp;                                               '0x9440'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0048(072)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0xffff',
##&nbsp;                                               '0x5665',
##&nbsp;                                               '0x7273',
##&nbsp;                                               '0x696f',
##&nbsp;                                               '0x6e20',
##&nbsp;                                               '0x3200',
##&nbsp;                                               '0x0000',
##&nbsp;                                               '0x5342'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0050(080)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0x5f58',
##&nbsp;                                               '0x5800',
##&nbsp;                                               '0x0000',
##&nbsp;                                               '0x5377',
##&nbsp;                                               '0x6974',
##&nbsp;                                               '0x6368',
##&nbsp;                                               '0x2030',
##&nbsp;                                               '0x0000'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0058(088)'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'values' => [
##&nbsp;                                               '0xffff',
##&nbsp;                                               '0xffff',
##&nbsp;                                               '0x0000'
##&nbsp;                                             ],
##&nbsp;                                 'offset_addr' => '0x0060(096)'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'config_file' => '/firmware/Buell/switch/controller_switch_img.txt'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub switcheprm {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my ($count,$k,$j,$temp_value);
    $k = $count = $j = $temp_value = 0;
    my $c = -1;
    for ( my $i = 0 ; $i <= $#lines ; $i++ ) {
        if ( $lines[$i] =~
            /Ethernet switch configuration\s+(read from|present in)\s+\(?([A-Za-z0-9.\/_-]+)\)?/ ) {
            my $phrase = $1;
            $result_array[0]->{config_file} = $2 if ($phrase !~/read from/);
        } elsif ( $lines[$i] =~ /Version\s+(\S+)\s+(\S+)/ ) {
            $result_array[0]->{'version'} = $1;
            $result_array[0]->{'switch'}  = $2;
        } elsif ($lines[$i] =~ /(start)\s+=\s+(\S+)\s+,\s+(data)\s+=\s+(\S+)/i) {
            $result_array[0]->{to_key_name( item => $1)} = $2;
            $result_array[0]->{to_key_name( item => $3)} = $4;
        } elsif (($lines[$i] !~ /Mismatched values/) && ($lines[$i] =~ /(ED|FD)?\s*(\S+)\s+:\s+(.*)/)) {
            my ($offset_addr,$temp,$type,$key);
            if ($3) {
               $offset_addr = $2;
               $type = $1;
               $temp = $3;
            } else {
               $offset_addr = $1;
               $temp = $2;
            }
            my @values = split(/\s+/, $temp);
            if ($type) {
                if ($type =~ /ED/) {
                    $key = "ed";
                    $c++;
                    $temp_value = $j;
                } elsif($type =~ /FD/) {
                   $j = $temp_value;
                   $key = "fd";
                }
                $result_array[0]->{$key."_info"}->[$c]->{'offset_addr'} = $offset_addr;
                $result_array[0]->{$key."_info"}->[$c]->{'values'} = \@values;
                for (my $i = 0; $i <= $#values; $i++) {
                    if ($values[$i] =~ /\S+\*$/) {
                        $result_array[0]->{'mismatched_values'}->[$j]->{'offset_addr'} = $offset_addr;
                        $result_array[0]->{'mismatched_values'}->[$j]->{$key."_value"} = $values[$i];
                         $j++;
                    }
                }
            } else {
                $result_array[0]->{'switch_info'}->[$k]->{'offset_addr'} = $offset_addr;
                $result_array[0]->{'switch_info'}->[$k]->{'values'} = \@values;
                $k++;
            }
        }
    }
    return \@result_array;

} ## end sub switcheprm

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{"response_object"};
    my ( $error, $error_message ) = undef;
    if ( $output =~ /Examples|^Error.*/ ) {
        $error         = "InvalidParam";
        $error_message = $output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

1;
