# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets ZAPI commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::ZAPI;

#
# Compiler Directives
#
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use Params::Validate qw(:all);

# Module import - APISets
use NACL::APISet::Schema::ZapiSchema;

use NACL::APISet::_Mixins::ZAPI qw(:all);
use NACL::APISet::_Mixins::CLIZAPI qw(:baseline_opts);
use NACL::Global;

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);


# Base class
use base qw(NACL::APISet::Node);

# global variables
use constant FILER => "filer";

my $FALSE = 1;

use Class::MethodMaker [
    scalar => [
        {
            -default_ctor => sub {
                $Log->enter() if $may_enter;
                my $self = $_[0];
                my $is_ge_lb;
                try {
                    $is_ge_lb = $self->{version_manager}->is_version_ge(release =>
						$NACL::Global::ONTAP_Release_Longboard_0);

                    # TEMPORARY workaround until LB.0 is ONTAP 9.0.0 and the "beer-1" build
                    # is LB.0 (9.0.0) for cit-ndu and other affected CITs.
                    $is_ge_lb = $is_ge_lb || $self->{version_manager}->is_version_ge(release => '8.4');
                } otherwise {
                    # Versioning call failed. (One possible reason is SSH is disabled,
                    # such as if run pre-cluster) Let's just assume it's running LB
                    # or later
                    $is_ge_lb = 1;
                };

                # For LB onwards, default transport_type to HTTPS. See burt 947157
                my $default_transport_type = $is_ge_lb ? 'HTTPS' : 'HTTP';
                $Log->exit() if $may_exit;
                return $default_transport_type;
            }
        },
        '_default_transport_type',
    ],
];

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw ZAPI command on the filer and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    $Log->enter() if $may_enter;

    my $self = shift();

    my %args = NACL::APISet::validate_params
        ($self,
         params => \@_,
         spec   => {
            # ZAPI command to be executed
                    "command"                => {type=>SCALAR, optional => 0},
            # vfiler, vserver, ontapi version are sent as part of input ZAPI
            # XML. So user has to explictly mention these parameters in the
            # call to execute_raw_command().
                    "vfiler-name"            => {type=>SCALAR, optional => 1},
                    "vserver-override-name"  => {type=>SCALAR, optional => 1},
                    'ontapi-version'         => {type=>SCALAR, optional => 1},
            # CLI supports overriding the timeout in the call to execute_raw_
            # command. Hence this option is supported in parity for ZAPI also.
                    'zapi-timeout'           => {type=>SCALAR, optional => 1},
            # For RFE burt434468.
                    'return-response-object' => {type=>SCALAR, optional => 1},
            # For burt483671
                    'log_level'              => {type=>SCALAR, optional => 1},
                    'input_log_level'        => {type=>SCALAR, optional => 1},
                    'output_log_level'       => {type=>SCALAR, optional => 1},
                   }
        );
    # If users had requested for a response object, then return a response
    # object even for the direct XML inputs.
    my $need_resp_obj;
    if (defined $args{'return-response-object'}) {
        $need_resp_obj = $args{'return-response-object'};
    } elsif (defined $self->{'return-response-object'}) {
        $need_resp_obj = $self->{'return-response-object'};
    }
    $need_resp_obj = (defined $need_resp_obj)?$need_resp_obj:0;
    $Log->debug("Response object is requested") if ($need_resp_obj);

    if ((defined($args{"vfiler-name"})) && (defined($args{"vserver-override-name"}))) {
        NATE::Exceptions::Argument->throw("Provide either 'vfiler-name' or" .
            " 'vserver-override-name'");
    }
    my $override;
    if (defined $args{"vfiler-name"}) {
        $override = delete ($args{"vfiler-name"});
    } elsif (defined $args{"vserver-override-name"}) {
        $override = delete ($args{"vserver-override-name"});
    }
    # This output can be either raw XML output or response object
    my $output = $self->{session}->execute_raw_ZAPI(%args,
        'server-name-override' => $override,
        'return-response-object'=>$need_resp_obj);

    $Log->exit() if $may_exit;
    return $output;
}

##############################################################################
# Method:    set_ontapi_version
# Objective: To set the ontapi major version and minor version
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_ontapi_version {
    $Log->enter() if $may_enter;

    my $self = shift();

    my %args = NACL::APISet::validate_params
        ($self,
         params => \@_,
         spec   => {"major-version" => {type => SCALAR, optional => 0},
                    "minor-version" => {type=>SCALAR, optional => 0},
                   }
        );
    $self->{session}->{major_version} = $args{"major-version"};
    $self->{session}->{minor_version} = $args{"minor-version"};

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    get_ontapi_version
# Objective: To get the ontapi major version and minor version
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_ontapi_version {
    $Log->enter() if $may_enter;

    my $self = shift();

    my $major = $self->{session}->{major_version};
    my $minor = $self->{session}->{minor_version};
    my $ontapi_v = "";
    if((defined $major) && (defined $minor)) {
        $ontapi_v = $major . "." . $minor;
    }

    $Log->exit() if $may_exit;
    return $ontapi_v;
}

##############################################################################
# Method:    clear_ontapi_version
# Objective: To clear the ontapi major version and minor version
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_ontapi_version {
    $Log->enter() if $may_enter;

    my $self = shift();

    $self->{session}->{major_version} = undef;
    $self->{session}->{minor_version} = undef;

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    set_target_cluster_uuid
# Objective: Sets target cluster uuid on ZAPI APISet object
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub set_target_cluster_uuid {
    $Log->enter() if $may_enter;
    my ($self,@args) = @_;

    my %args = validate_with(
        params => \@args,
        spec   => { target_cluster_uuid => { type => SCALAR } },
    );

    $self->{session}->target_cluster_uuid( $args{target_cluster_uuid} );

    $Log->exit() if $may_exit;
} ## end sub set_target_cluster_uuid

##############################################################################
# Method:    set_target_vserver_name
# Objective: Sets target cluster uuid on ZAPI APISet object
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub set_target_vserver_name {
    $Log->enter() if $may_enter;
    my ($self,@args) = @_;

    my %args = validate_with(
        params => \@args,
        spec   => { target_vserver_name => { type => SCALAR } },
    );

    $self->{session}->target_vserver_name( $args{target_vserver_name} );

    $Log->exit() if $may_exit;
} ## end sub set_target_vserver_name

##############################################################################
# Method:    set_target_vserver_uuid
# Objective: Sets target vserver uuid on ZAPI APISet object
##############################################################################
sub set_target_vserver_uuid {
    $Log->enter() if $may_enter;
    my ($self,@args) = @_;

    my %args = validate_with(
            params => \@args,
            spec   => { target_vserver_uuid => { type => SCALAR } },
    );

    $self->{session}->target_vserver_uuid( $args{target_vserver_uuid});
    $Log->exit() if $may_exit;
} ## end sub set_target_vserver_uuid

##############################################################################
# Method:    set_app_header
# Objective: Sets X-Dot-Client-App HTTP header value on ZAPI APISet object
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub set_app_header {
    $Log->enter() if $may_enter;
    my ($self,@args) = @_;

    my %args = validate_with(
        params => \@args,
        spec   => { app_header => { type => SCALAR } },
    );

    $self->{session}->app_header( $args{app_header} );

    $Log->exit() if $may_exit;
} ## end sub set_app_header

##############################################################################
# Method:    set_zapi_header
# Objective: Sets the zapi_header from which the ONTAP APIs are generated
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub set_zapi_header {
    $Log->enter() if $may_enter;
    my ($self,@args) = @_;

    my %args = validate_with(
        params => \@args,
        spec   => { zapi_header => { type => HASHREF } },
    );

    $self->{session}->zapi_header( $args{zapi_header} );

    $Log->exit() if $may_exit;
} ## end sub set_zapi_header

##############################################################################
# Method:    get_target_cluster_uuid
# Objective: Returns target cluster uuid
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub get_target_cluster_uuid {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->exit() if $may_exit;

    return $self->{session}->target_cluster_uuid;
} ## end sub get_target_cluster_uuid

##############################################################################
# Method:    get_target_vserver_name
# Objective: Returns target vserver name
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub get_target_vserver_name {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->exit() if $may_exit;

    return $self->{session}->target_vserver_name;
} ## end sub get_target_vserver_name

##############################################################################
# Method:    get_target_vserver_uuid
# Objective: Returns target vserver uuid
##############################################################################
sub get_target_vserver_uuid {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->exit() if $may_exit;

    return $self->{session}->target_vserver_uuid;
} ## end sub get_target_vserver_uuid

##############################################################################
# Method:    get_app_header
# Objective: Returns value set for X-Dot-Client-App header
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub get_app_header {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->exit() if $may_exit;

    return $self->{session}->app_header;
} ## end sub get_app_header

##############################################################################
# Method:    get_zapi_header
# Objective: Returns value set for the ONTAP APIs generated
# Details:   Refer to POD documentation at the end of the file.
##############################################################################
sub get_zapi_header {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->exit() if $may_exit;

    return $self->{session}->zapi_header;
} ## end sub get_zapi_header

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the CMode API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            creating a new schema object and loading it with CDEFs from the
#            appropriate path
##############################################################################

sub _init_API
{
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path. Load CDEFS after the check for
    # shell_type is done so that we need not unnessarily load CDEFs even in
    # case of invalid shell_type.

    my $cdef_path = $self->{cdef_path};
    $self->{schema} = NACL::APISet::Schema::ZapiSchema->new();
    $self->{schema}->{searched_cdef_path} = $cdef_path;

    # Load the cdefs from the build if the option is set.

    if (( defined $self->{version_info}->{cdefs_from_build} )
        && ( $self->{version_info}->{cdefs_from_build} ) )
    {
        my $dir = '/final/bedrock/export/common/zapiprod/cdefs/';
        $cdef_path = $cdef_path . $dir . lc $self->{set};
    }
    $self->{schema}->load(path => $cdef_path);

    # This section is for burt 947157
    if (!$self->{session}->transport_type_isset()) {
        # If this attribute is set, it means that an override was
        # provided on the command-line which should be used. That's why
        # we should first check if isset() instead of just blindly setting
        # to HTTPS
        my $default_transport_type = $self->_default_transport_type();
        $self->{session}->transport_type($default_transport_type);
    }

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the ZAPI Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $api_set = $self->{set};

    $self->_get_connectrec_or_hostrec(\%args);
    my $interface = $self->{interface};
    $args{set} = $self->{actual_set};
    # Establish the session if not provided
    # First check whether a valid shell_host (hostobj) is passed
    # for the context
    $self->_check_session_type();

    # Determine the type of the session based on the shell_type and context.
    my $session_type = $self->{shell_type};
    if ($self->{shell_type} eq "Vfiler") {
        $session_type = "7Mode";
    } elsif ($self->{shell_type} eq "Vserver") {
        $session_type = "CMode";
    } elsif ( $self->{actual_set} eq "Nodescope" ) {
        # User had asked for Nodescope commands from a cluster shell
        $session_type = "CMode";
    }
    my $session_pkg = "NACL::APISet::Session::" . $interface . "::"
        . $session_type;

    # We havent technically connected to the node yet
    # but it's imminent. Doing it here avoids having to do work
    # on every ZAPI call
    $self->_record_connection_into_keyval();

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    # Set the server-type here. For category being "Node", the only possible
    # server-type can be "filer".
    $self->{session}->set_server_type("server-type" => FILER);

    $Log->exit() if $may_exit;

}

###############################################################################
# Method Name: _check_session_type
# Objective:   Check whether the shell_type passed by the user is appropriate
#              for the shell_host
###############################################################################
sub _check_session_type {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

    # Valid combinations for shell_type and set passed by the user
    my %valid_hash = (
        #actual_set    shell_type
        "7Mode"     => "7Mode",
        "Vfiler"    => "7Mode Vfiler",
        "CMode"     => "CMode",
        "Vserver"   => "CMode Vserver",
        "Nodescope" => "CMode",
        "DBlade"    => "CMode 7Mode"
    );
    if($valid_hash{$self->{actual_set}}!~ /$self->{shell_type}/) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Invalid hostobj or shell_type "
            . "is sent for set '$self->{actual_set}'"
            . " and the shell_type obtained is ".$self->{shell_type});
    }

    $Log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into input xml and execute
# Details:     This method invokes the ZAPI along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual xml to be invoked on the
#              service, and then invoking that on the filer using the Session
#              object.
###############################################################################

sub _invoke {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    my %baseline_args = $self->_move_baseline_args(source => \%args);

    my $cdef_method = $args{cdef_method};
    my $raw_data    = $FALSE;
    my $vfiler       = undef;
    my $vserver       = undef;
    my $timeout       = undef;
    my ($log_level, $input_log_level, $output_log_level);
    my ($target_cluster_uuid, $target_vserver_name, $target_vserver_uuid);
    my $app_header = delete $args{app_header};
    my $zapi_header = delete $args{zapi_header};

    # process parameters
    while (my ($key, $value) =  each(%args)) {
        if ($key =~ /-?cdef$/) {
            if (! length($value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
        } elsif ($key =~ /-?debug_level$/) {
            if (! length($value) || $value !~ /\d+/) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^-?vfiler-name$/) {
            # For vfiler, sending empty string is allowed and hence we
            # are not checking for the length.
            # If vfiler-name=>"" is provided in overrides, then the command will
            # be invoked in default vfiler vfiler0 and hence empty
            # string is supported.

            if (! defined($vfiler = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^-?vserver-override-name$/) {
            # For vserver, sending empty string is allowed and hence we
            # are not checking for the length.
            # If vserver-override-name=>"" is provided in overrides, then the command will
            # be invoked in default vserver and hence empty
            # string is supported.

            if (! defined($vserver = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^-?zapi-timeout$/) {
            # Timeout override
            if (! defined($timeout = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }


            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^log_level$/) {
            # log_level override
            if (! defined($log_level = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^input_log_level$/) {
            # log_level override
            if (! defined($input_log_level = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^target_cluster_uuid$/) {
            # log_level override
            if (! defined($target_cluster_uuid = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^target_vserver_name$/) {
            # log_level override
            if (! defined($target_vserver_name = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^output_log_level$/) {
            # log_level override
            if (! defined($output_log_level = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /^target_vserver_uuid$/) {
            # log_level override
            if (! defined($target_vserver_uuid  = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
            # delete args that is not needed for translation
            delete $args{$key};
            next;
        }
    }

    # verify mandatory cdef argument
    unless (defined $args{cdef_method} and length $args{cdef_method}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Mandatory argument \"cdef_method\" not specified ");
    }

    my $schema_obj = $self->get_schema_obj();

    # translate cdef_method schema into command
    my $cmd = $schema_obj->translate(cdef_method => $cdef_method ,%args);
    ### Burt979349 : The below regex find the xml tags which has only space characters.
    ### 1. Data between tags should not have and character/digit/newline.
    ### 2. Starting and closing tags should match. For this (\S+) and \1 has been used.
    my $str = $1 if( $cmd =~ /<(\S+)>((?:(?![\n\r])\s)+)<\/\1>/);
    my $xmlStr = $schema_obj->getSimpleXML(logxml=>$cmd);
    $xmlStr =~ s/<$str><\/$str>/<$str> <\/$str>/ if (defined $str);

    # Check for overrides also and prefix the command appropriately
    my $is_cmode = 0;
    $is_cmode = 1 if($self->{"set"} eq "CMode");

    if ($is_cmode && defined($vfiler)) {
        NATE::Exceptions::Argument->throw("\"vfiler-name\" input argument"
                . "should not be passed for CMode APISet method");
    }
    if (!$is_cmode && defined($vserver)) {
        NATE::Exceptions::Argument->throw("\"vserver-override-name\" input argument"
                . "should not be passed for 7Mode APISet method");
    }
    if ($is_cmode) {
        if (defined($vserver)) {
           $self->{session}->{server_name_override} = $vserver;
        }
    } else {
        if (defined($vfiler)) {
            $self->{session}->{server_name_override} = $vfiler;
        }
    }

    my %additional_send_cmd_opts;
    %additional_send_cmd_opts = (%baseline_args) if ($is_cmode);

    my $result_obj = $self->{session}->send_command(
        command          => $xmlStr,
        "zapi-timeout"   => $timeout,
        log_level        => $log_level,
        input_log_level  => $input_log_level,
        output_log_level => $output_log_level,
        target_cluster_uuid => $target_cluster_uuid,
        target_vserver_name => $target_vserver_name,
        target_vserver_uuid => $target_vserver_uuid,
        app_header       => $app_header,
	zapi_header      => $zapi_header,
        %additional_send_cmd_opts,
    );
    $Log->exit() if $may_exit;
    return $result_obj;

} ## end sub _invoke

###############################################################################
# Method Name: _only_translate_method
# Objective:   Translate the API call into the command invoked on the service
#              and return this without executing it.
# Details:     When the API is prefixed by an "xlate_", it must only translate
#              the API call without executing it. This will come in handy
#              while developing new APISets and for cases where undoing
#              API calls can be cumbersome.
#              This method strips the leading xlate_ prefix, translates
#              the API and returns the command
###############################################################################
sub _only_translate_method {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my $cdef_method = $args{method};
    delete($args{method});
    $cdef_method =~ s/^xlate_//;
    $cdef_method =~ s/\_/\-/g;
    my $schema_obj = $self->get_schema_obj();
    my $command = $schema_obj->translate(cdef_method => $cdef_method, %args);
    my $xmlStr = $schema_obj->getSimpleXML(logxml=>$command);
    $Log->exit() if $may_exit;
    return $xmlStr;
}

###############################################################################
# Method:    find_cdef_directory
# Objective: To find the cdef directory based on various version related
#            attributes obtained from the service.
# Details:   Refer to NACL::APISet::Node for POD documentation.
###############################################################################

sub find_cdef_directory {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    # Load CDEFs directly from the build root if specified by the user
    if (   ( defined $self->{version_info}->{cdefs_from_build} )
        && ( $self->{version_info}->{cdefs_from_build} ) )
    {
        my $vm = $self->get_version_manager();
        NATE::Exceptions::Argument->throw(
            "Version manager could not be found"
            . ". Hence build root cannot be autodiscovered to load CDEFs."
        ) unless ( defined $vm );
        my $build_path;
        unless ($self->_is_vserver_version_manager($vm)) {
            $build_path = $vm->get_version_attribute(
                attribute => "build_path");
        } else {
            NATE::Exceptions::Argument->throw(
                  "Build root cannot be discovered when the version manager".
                  " is of type ". ref($vm));
        }
        # If the buildpath obtained from versioning does not exist, we
        # will try searching the build under /x/eng/rlse/DOT, because most
        # of the builds move to this location after a couple of days.
        unless ( -d $build_path ) {
            my $old_path = $build_path;
            if ($build_path =~ s/.+\/(\w+_\d+_\d+\/?)$/$1/) {
                $build_path = '/x/eng/rlse/DOT/' . $build_path;

                $Log->comment("The build path $old_path does not exist. " .
                    "Trying to use $build_path to load the cdefs");
            }
        }
        $Log->exit() if $may_exit;
        return $build_path;

    }
    my $ret = $self->SUPER::find_cdef_directory(%opts);
    $Log->exit() if $may_exit;
    return $ret;
}


1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::ZAPI - Base for API Sets for ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Node::ZAPI;

=head1 DESCRIPTION

A C<NACL::APISet::Node::ZAPI> is used to connect to and send ZAPI commands. This
is further subclassed to send ZAPI commands specifically to CMode, 7Mode etc.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument

=back

=cut

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw ZAPI commands on the filer. The input xml to
be executed should be provided by the user. This will be invoked and the raw
xml output will be returned as a string directly without any processing.
This is useful for cases where we want to invoke a command without parsing the
output or if the users want to invoke any ZAPIs whose CDEF files(.xml files) are
not yet generated. If you want to set the vfiler/vserver context,
ontapi-version in the input XML, they should be explicitly specified in the
call to execute_raw_command. These values set in the APIset object through the
corresponding accessor methods will not be effected for execute_raw_command.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(
command => "<volume-get-language><volume>vol0</volume></volume-get-language>");

=item Arguments

=over

=item command

(Mandatory)
The command (input xml) to be invoked.

=item vfiler-name

(Optional)
Name of the vfiler in whose context to execute the ZAPI.

=item vserver-override-name

(Optional)
Name of the vserver in whose context to execute the ZAPI.

=item ontapi-version

(Optional)
Ontapi version to be set in the header of ZAPI input XML.
If not specified, defaults will be assigned by the library itself.

=item zapi-timeout

(Optional)
Override timeout value for ZAPI execution.

=item return-response-object

(Optional)
Boolean option. If set, a response object will be returned else raw XML output
will be returned. By default, this option will be set to 0.

=back

=item Returns

Raw output or Response object depending on 'return-response-object' option.

=back

=head2 set_ontapi_version

This method sets the major verison and minor version of ontapi version
to user specified value

=over

=item Synopsis

$Api_Set_Obj->set_ontapi_version('major-version' => $major,
'minor-version' => $minor);

=item Arguments

=over

=item major-version

Required
Major version of ontapi version

=item minor-version

Required
Minor version of ontapi version

=back

=item Returns

Nothing.

=back

=head2 get_ontapi_version

This method returns the ontapi version stored in the APISet object.

=over

=item Synopsis

my $version = $Api_Set_Obj->get_ontapi_version();

=item Arguments

Nothing.

=item Returns

This method returns the ontapi version as a string "X.Y" format where X
corresponds to major version and Y to minor version. If not already set or
cleared, then it will return empty string.

=back

=head2 clear_ontapi_version

This method clears the ontapi version stored in the APISet object.

=over

=item Synopsis

$Api_Set_Obj->clear_ontapi_version();

=item Arguments

Nothing.

=item Returns

Nothing.

=back

=head2 set_port

This method sets the port number to which the ZAPI input requests are to be
sent.

=over

=item Synopsis

$Api_Set_Obj->set_port('port' => "82");

=item Arguments

=over

=item port

Required
Port number

=back

=item Returns

Nothing.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut


1;
