#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for cluster mode ZAPI commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::ZAPI::7Mode;

#
# Compiler Directives
#
use strict;
use warnings;

# Module import - APISets
use NACL::APISet::Session::ZAPI::7Mode;
use NACL::APISet::Session::ZAPI::CMode;

# Exceptions imported
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Node::ZAPI);

# Constructor
sub new {
    $Log->enter();

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my $class = shift;
    my %args  = @_;
    my $self  = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   =>
          { version => { type => SCALAR, optional => 1, default => 99.99 }, }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit();
    return $self;
}

##############################################################################
# Method:    set_vfiler
# Objective: To set the current vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_vfiler 
{
    $Log->enter();
    my $self = shift();
    my %args = NACL::APISet::validate_params
      ($self,
       params => \@_, 
       spec   => {"vfiler-name" => {type => SCALAR}}
      );
    $self->{session}->set_vfiler_or_vserver(server_name => $args{"vfiler-name"});    
    $Log->exit();
}

##############################################################################
# Method:    get_vfiler
# Objective: Get the current vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_vfiler 
{
    $Log->enter();
    my ($self, %args) = @_;
    $Log->exit();
    return $self->{session}->get_vfiler_or_vserver();
}

##############################################################################
# Method:    clear_vfiler
# Objective: To set the default vfiler context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_vfiler 
{
    $Log->enter();
    my ($self, %args) = @_;
    $self->{session}->clear_vfiler_or_vserver();
    $Log->exit();
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the 7Mode API Set
# Details:   Initializes the API Set.  This method drives the workflow needed 
# to make the API Set ready for work.  This is accomplished by calling the 
# parents' init_API method and performing work specific to this class afterwards.
##############################################################################

sub _init_API {
    $Log->enter();

    my ( $self, %args ) = @_;
    
    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file($self->{cdef_path});

    unless (defined $self->{cdef_path}) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }
    
    $self->SUPER::_init_API( %args, set => $self->{actual_set});
    
    # Set the vfiler name provided through context_definition hash irrespective
    # of whether we had directly connected to the vfiler or through the node.
    # Thus incase of direct connection to vfiler, if the user overrides the
    # vfiler name with other vfiler name or even the same vfiler name, we will
    # prepend the command with this name and allowing the service to respond to
    # it. Thus using the context_definition hash incase of direct connection 
    # will cause an exception to be thrown based on how the service reacts.
    if ($self->{actual_set} eq "Vfiler" || $self->{actual_set} eq "Nodescope") {
        my $vfiler_name = defined ($self->{context_definition}->{name})? 
            $self->{context_definition}->{name}:
            (defined $self->{context_definition}->{schema_hostobj}?
            $self->{context_definition}->{schema_hostobj}->hostname(): "");
        $self->set_vfiler("vfiler-name" => $vfiler_name) 
            if(length $vfiler_name);
    }    
    
    $Log->exit();
}


=head1 NAME

NACL::APISet::Node::ZAPI::7Mode - API Set for 7Mode ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Node::ZAPI::7Mode;

=head1 DESCRIPTION

A C<NACL::APISet::Node::ZAPI::7Mode> is used to connect to and send ZAPI 
commands to the 7Mode node.

=cut

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Node::ZAPI::7Mode>.

=over 

=item Synopsis

 $api_set_obj = NACL::APISet::Node::ZAPI::7Mode->new( %args );

=item Arguments

=over 

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and NACL::APISet::Node::ZAPI for details.

=back

=item Returns

A reference to a new API Set for Cluster Mode.

=back

=cut

=head1 METHODS

=head2 set_vfiler

Specify which vfiler in which to run the commands following the call

=over

=item Synopsis

$Api_Set_Obj->set_vfiler('vfiler-name' => $vfiler_name);

=item Arguments

=over

=item vfiler-name

Name of the vfiler

=back

=item Returns

Nothing.

=back

=head2 get_vfiler

Get current vfiler context in which we are executing commands

=over

=item Synopsis

$Api_Set_Obj->get_vfiler();

=item Arguments

None.

=item Returns

(String)
Current vfiler context

=back

=head2 clear_vfiler

Reset to executing commands in default vfiler context

=over

=item Synopsis

$Api_Set_Obj->clear_vfiler();

=item Arguments

None.

=item Returns

None.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
