#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for DBlade ZAPI commands
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::ZAPI::DBlade;

#
# Compiler Directives
#
use strict;
use warnings;

# Module import - APISets
use NACL::APISet::Response::ZAPI::DBlade;
use NACL::APISet::Schema::ZapiSchema::DBlade;
use NACL::APISet::Session::CLI::CMode::Shell;
use NACL::APISet::Session::CLI::7Mode::Shell;
use NACL::APISet::_Mixins::ZAPI qw(_xml2perl);
# Exceptions imported
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Base class
use base qw(NACL::APISet::Node::ZAPI);

# We define the default timeout value to be 60 seconds
use constant DEFAULT_TIMEOUT => 60;

# Default log level for command execution is 'comment'.
use constant DEFAULT_LOG_LEVEL => 'comment';

# When ZAPI execution fails, we log the output XML at comment level by default.
# This will help the users to debug the failures without having to enable
# trace level logs.
use constant DEFAULT_LOG_LEVEL_ON_ERROR => 'comment';

# Constructor
sub new {
    $Log->enter();

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my $class = shift;
    my %args  = @_;
    my $self  = {};

    $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   =>
          { version => { type => SCALAR, optional => 1, default => 99.99 }, }
    );

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit();
    return $self;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the DBlade API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
# to make the API Set ready for work.  This is accomplished by calling the
# parents' init_API method and performing work specific to this class afterwards.
##############################################################################

sub _init_API {
    $Log->enter();

    my ( $self, %args ) = @_;

    my $path = $self->{cdef_path};
    $self->{cdef_path} = NATE::Inc::find_file($self->{cdef_path});

    unless (defined $self->{cdef_path}) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directory $path and the reason: \n$@");
    }

    # Create the schema object and load the CDEFs from CDEF path discovered by
    # versioning or the user specified path. Load CDEFS after the check for
    # shell_type is done so that we need not unnessarily load CDEFs even in
    # case of invalid shell_type.

    my $cdef_path = $self->{cdef_path};
    $self->{schema} = NACL::APISet::Schema::ZapiSchema::DBlade->new();

    # Load the cdefs from the build if the option is set.
    if (( defined $self->{version_info}->{cdefs_from_build} )
        && ( $self->{version_info}->{cdefs_from_build} ) )
    {
        my $dir = '/final/bedrock/export/common/zapiprod/cdefs/';
        $cdef_path = $cdef_path . $dir . lc $self->{set};
    }

    $self->{schema}->load(path => $cdef_path);

    $Log->exit();
}

##############################################################################
# Method:    _init_Session
# Objective: Initialize the DBlade Session object
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {
    my ( $self, %args ) = @_;
    $Log->enter();

    $self->_get_connectrec_or_hostrec(\%args);

    $args{set} = $self->{actual_set};

    my $interface = $self->{interface};

    my $session_pkg = "NACL::APISet::Session::CLI::"
        . $self->{shell_type} . "::Shell";

    # Check for shell type
    $self->_check_session_type();

    # We havent technically connected to the node yet
    # but it's imminent. Doing it here avoids having to do work
    # on every ZAPI call
    $self->_record_connection_into_keyval();

    # Create a Session object
    $self->{session} = $session_pkg->new(%args);

    $Log->exit();
}
###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into input xml and execute
# Details:     This method invokes the ZAPI along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual xml to be invoked on the
#              service, and then invoking that on the filer using the Session
#              object.
###############################################################################

sub _invoke
{
    my ($self, %args) = @_;

    my $cdef_method = $args{cdef_method};
    my $timeout       = undef;
    my ($log_level, $input_log_level, $output_log_level);

    $Log->enter();

    # process parameters
    %args = NACL::APISet::validate_params
        ($self,
         params => \%args,
         spec   => {
                    'cdef_method'            => {type=>SCALAR, optional => 0},
                    'zapi-timeout'           => {type=>SCALAR, optional => 1},
                    'log_level'              => {type=>SCALAR, optional => 1},
                    'input_log_level'        => {type=>SCALAR, optional => 1},
                    'output_log_level'       => {type=>SCALAR, optional => 1},
                   }
    );

    my $schema_obj = $self->get_schema_obj();

    # translate cdef_method schema into command
    my $cmd = $schema_obj->translate(cdef_method => $cdef_method ,%args);

    my $result_obj = $self->_execute_zsmcli(
                                    command          => $cmd,
                                    actual_command   => $cmd,
                                    %args,
                                );

    $Log->exit();
    return $result_obj;

} ## end sub _invoke

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw ZAPI command on the filer and return the raw
#            output
# Details:   POD documentation is in Base Class.
##############################################################################
sub execute_raw_command {
    $Log->enter();

    my $self = shift();

    my %args = NACL::APISet::validate_params
        ($self,
         params => \@_,
         spec   => {
            # ZAPI command to be executed
                    "command"                => {type=>SCALAR, optional => 0},
                    'zapi-timeout'           => {type=>SCALAR, optional => 1},
                    'return-response-object' => {type=>SCALAR, optional => 1},
                    'log_level'              => {type=>SCALAR, optional => 1},
                    'input_log_level'        => {type=>SCALAR, optional => 1},
                    'output_log_level'       => {type=>SCALAR, optional => 1},
                   }
        );
    # If users had requested for a response object, then return a response
    # object even for the direct XML inputs.
    my $need_resp_obj;
    if (defined $args{'return-response-object'}) {
        $need_resp_obj = $args{'return-response-object'};
    } elsif (defined $self->{'return-response-object'}) {
        $need_resp_obj = $self->{'return-response-object'};
    }
    $need_resp_obj = (defined $need_resp_obj)?$need_resp_obj:0;
    $Log->debug("Response object is requested") if ($need_resp_obj);

    delete $args{'return-response-object'};

    my ($output);
    # This output can be either raw XML output or response object

    $output = $self->_execute_zsmcli(%args,
                                     'need-resp-obj' => $need_resp_obj,
                                    );

    $Log->exit();
    return $output;
}

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw ZSMCLI command on the filer and return the output.
# It will also logs the results according to the log level
##############################################################################
sub _execute_zsmcli {

    $Log->enter();
    my ($self, %opts) = @_;

    # Get the log levels
    my ($input_log_level_on_success, $output_log_level_on_success,
        $input_log_level_on_error,   $output_log_level_on_error
    );
    $input_log_level_on_success
        = $opts{input_log_level}
        || $opts{log_level}
        || DEFAULT_LOG_LEVEL;
    $output_log_level_on_success
        = $opts{output_log_level}
        || $opts{log_level}
        || DEFAULT_LOG_LEVEL;
    $input_log_level_on_error
        = $opts{input_log_level}
        || $opts{log_level}
        || DEFAULT_LOG_LEVEL_ON_ERROR;
    $output_log_level_on_error
        = $opts{output_log_level}
        || $opts{log_level}
        || DEFAULT_LOG_LEVEL_ON_ERROR;

    my $timeout = $opts{'zapi-timeout'} || DEFAULT_TIMEOUT;

    my $inputstr = "Zsmcli input command is:" . $opts{command};
    my ($output, $execution_time, $execution_status);

    try {
        ($output, $execution_time, $execution_status)  = $self->{session}->send_command(
                                                        command          => $opts{command},
                                                        "timeout"        => $timeout,
                                                        log_level        => "debug",
                                                        );
    } catch NACL::APISet::Exceptions::TimeoutException with {

        # Timeout occured.
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_error
        );
        $Log->exit();
        my $exception = shift;
        $exception->throw();
    };

    my $outputstr = "\nOutput xml is:\n$output\n";

    my ($status, $errReason, $errNo);
    if ($execution_status == 0) {
        my $printXML = $self->_xml2perl(xmlString => $output, KeepRoot => 1, KeyAttr => []);
        $status = $printXML->{results}->{status};
        $errReason = (defined $printXML->{results}->{reason}) ? $printXML->{results}->{reason} : "";
        $errNo = (defined $printXML->{results}->{errno}) ? $printXML->{results}->{errno} : "";
    } else {
        $status = "failed";
        $errNo = "";
        $errReason = "";
    }

    # Log the input and output XML at a desired log level based on whether the
    # ZAPI invocation succeeded or not.
    if ( $status eq 'passed' ) {
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_success
        );
        $self->_log_data(
            data      => $outputstr,
            log_level => $output_log_level_on_success
        );
    } else {
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_error
        );
        $self->_log_data(
            data      => $outputstr,
            log_level => $output_log_level_on_error
        );
    }

    # We need to make sure that "$opts{'need-resp-obj'}" is defined
    # because $opts{'need-resp-obj'} will not be defined in case where
    # execute_raw_command is not invoked
    if (defined $opts{'need-resp-obj'} and $opts{'need-resp-obj'} == 0) {
        $Log->exit();
        return $output;
    }

    my $result_obj;
    $result_obj = NACL::APISet::Response::ZAPI::DBlade->new(
                        command             => $opts{command},
                        output              => $output,
                        execution_status    => $status,
                        execution_time      => $execution_time,
                        error_number        => $errNo,
                        error_reason        => $errReason,
                    );

    $Log->exit();
    return $result_obj;
}

###############################################################################
# Method :    _log_data
# Objective:  Log the data in the specified log level
# Details:    This method uses Tharn log object ($Log) to log the specified
#             data in the given log level. Throws exception if the specified
#             log level is invalid
###############################################################################
sub _log_data {
    $Log->enter();
    my $self      = shift();
    my %args      = @_;
    my $log_level = $args{log_level};
    try {
        $Log->$log_level( $args{data} );
    }
    otherwise {

        # NATE will throw script error which will be promoted to BaseException
        my $ex = shift;
        if ( $ex->text() =~ /Can't locate object method/ ) {
            NATE::Exceptions::Argument->throw(
                'Invalid log level: ' . $log_level );
        } else {
            $ex->throw();
        }
    };
    $Log->exit;
}

1;

=head1 NAME

NACL::APISet::Node::ZAPI::DBlade - API Set for DBlade ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Node::ZAPI::DBlade;

=head1 DESCRIPTION

A C<NACL::APISet::Node::ZAPI::DBlade> is used to connect to and send ZAPI commands to the
cluser interface of a node.

=cut

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Node::ZAPI::DBlade>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Node::ZAPI::DBlade->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and NACL::APISet::Node::ZAPI for details.

=back

=item Returns

A reference to a new API Set for DBlade ZAPI.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
