package NACL::APISet::Response;

use strict;
use Data::Dumper;
use Storable qw(dclone);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::ResponseException;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

##############################################################################
# Method:    get_raw_output
# Objective: Returns the raw command output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_raw_output {
    $Log->enter() if $may_enter;
    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->{output};
}

##############################################################################
# Method:    get_execution_time
# Objective: Return the time taken for execution of the command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_execution_time {
    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug( "Execution time is " . $self->{execution_time} );
    $Log->exit() if $may_exit;
    return $self->{execution_time};
}

##############################################################################
# Method:    get_execution_status
# Objective: Returns the execution status of the command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_execution_status {
    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug( "Execution status is " . $self->{execution_status} );
    $Log->exit() if $may_exit;
    $self->{execution_status} = ""
        if ( !defined( $self->{execution_status} ) );
    return $self->{execution_status};
}

##############################################################################
# Method:    get_parsed_output
# Objective: Returns the parsed output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_parsed_output {
    $Log->enter() if $may_enter;
    my $self = shift;
    NACL::APISet::Exceptions::ResponseException->throw(
        response_object => $self,
        text =>
            "get_parsed_output was called when the parsed output is not available"
    ) if !defined $self->{parsed_output};
    $Log->exit() if $may_exit;
    return $self->{parsed_output};
}

##############################################################################
# Method:    set_parsed_output
# Objective: Sets parsed output to the value sent in
# Details:   See POD documentation at the end of this file.
##############################################################################
sub set_parsed_output {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    $self->{parsed_output} = $args{parsed_output};
}

##############################################################################
# Method:    get_command
# Objective: Returns the command invoked on the service
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_command {
    my $self = shift;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{command};
}
##############################################################################
# Method:    search
# Objective: Search for key value pair in parsed output.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub search {
    my ( $obj, %args ) = @_;

    # Validate parameters and populate member variables

    %args = NACL::APISet::validate_params(
        $obj,
        params => \%args,
        spec   => {
            '__path__'              => { type => SCALAR,   optional => 1 },
            '__match-case__'        => { type => SCALAR,   optional => 1 },
            '__deep-search-items__' => { type => ARRAYREF, optional => 1 },
            '__search-array-ref__'  => { type => ARRAYREF, optional => 1 },
            '__accessor-string__'   => { type => SCALAR,   optional => 1 },
	    '__is-regex__'   => { type => SCALAR, default => 1 }
        },
    );

    my @path_parts = split( '/', delete $args{'__path__'} );
    shift(@path_parts) if ( !$path_parts[0] );

    # choose either the array ref specified as '__search-array-ref__' or the
    # parsed output structure by default.
    # Also, deep clone the parsed output structure so that no changes are made
    # to the original structure

    my $array_ref = delete( $args{'__search-array-ref__'} )
        || $obj->{parsed_output};
    $array_ref = dclone($array_ref);

    my $case = delete $args{'__match-case__'} ? '' : 'i';

   # construct a hash from the array elements specified as 'deep-search-items'
    my %deep_search_items_hash
        = map { $_ => 1 } @{ $args{'__deep-search-items__'} };
    my $deep_search_items = \%deep_search_items_hash;

    # delete this argument since we are going to be passing it differently
    my @deep_search_item_keys = @{ delete $args{'__deep-search-items__'} };

    my $is_regex = delete $args{'__is-regex__'};
    # now %args contains only the search parameters

    # if there are no parameters, then throw an exception
    if ( !scalar( keys(%args) ) ) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(
            text            => 'No search parameters provided',
            response_object => $obj
        );
    }

    # if deep-search-items contains something not mentioned in %args,
    # throw an InvalidParamValueException
    foreach (@deep_search_item_keys) {
        unless ( exists( $args{$_} ) ) {
            NACL::APISet::Exceptions::InvalidParamValueException->throw(
                text =>
                    "Item '$_' mentioned in __deep-search-items__ is not one"
                    . " of the search parameters",
                response_object => $obj
            );
        }
    }

    my @answers;    # for holding the search results

    # call internal method _finder passing %args (untouched arguments) along
    # with the ones that are explicitly mentioned since they are not present
    # in the call to this function(search) and hence are not part of %args
    # This function will populate @answers

    _finder(
        '__path-parts__'        => \@path_parts,
        '__array__'             => $array_ref,
        '__case__'              => $case,
        '__deep-search-items__' => $deep_search_items,
        '__results__'           => \@answers,
	'__is-regex__'		=> $is_regex,
        %args
    );

    my @results;

  # if there were any deep-search-items mentioned, check each item in @answers
  # for the presence of each of those keys. We can't check in @answers as a
  # whole directly since we won't know which answer to discard when more than
  # one result is obtained.
  # Also, This has to be done separately for each key because the
  # corresponding value can be present at any level.

    foreach my $item (@deep_search_item_keys) {
        @answers = grep {
            if (   ref( $_->{$item} ) eq 'ARRAY'
                && ref( $_->{$item}->[0] ) eq 'HASH' )
            {
                @results = ();
                _finder(
                    '__array__'   => $_->{$item},
                    '__case__'    => $case,
                    '__results__' => \@results,
                    '__any-key__' => $args{$item},
		    '__is-regex__' => $is_regex,
                );
            } else {
                @results = (1);
            }
            @results;
        } @answers;
        last unless @answers;
    }
    return @answers;
}
##############################################################################
# Method:    _finder
# Objective: Recursive function which searches at various depths of output.
##############################################################################

sub _finder {
    my %args = @_;

    # the list of keys that indicate which (sub)level to seach in
    my $path_parts = delete $args{'__path-parts__'};

    my $arr_ref = delete $args{'__array__'};    # the parsed output structure
    my $case
        = delete $args{'__case__'};    # for case sensitive/insensitive search

    # the string indicating how to access the search result in the
    # parsed output structure
    my $accessor_string = delete $args{'__accessor-string__'};

    # hashref of items that are considered for deep search
    #(key is item and value is 1)
    my $deep_search_items = delete $args{'__deep-search-items__'};

    # $matches is an array reference that holds the successful search results
    my $matches = delete $args{'__results__'};

    my $is_regex =delete $args{'__is-regex__'};	# for regex/non-regex context search

    # now %args contains only the search parameters

   # in a recursive call, it indicates part of the key name that describes the
   # path at the current level (in a nested structure)
    my $part = shift( @{$path_parts} );

    my $i = -1;

    foreach my $hashref (@$arr_ref) {
        $i++;

        # if a path was specified and that path doesn't exist in the current
        # hash, then move on to the next iteration.
        next if ( $part && !( $hashref->{$part} ) );

        my $count;    # to indicate the number of matches at the current level

       # for each item in this hash, call _finder recursively if the value is
       # an array of hashes.
       # Then check if that item is what we're looking for (search parameters)

        foreach my $item ( keys(%$hashref) ) {
            if (   ref( $hashref->{$item} ) eq 'ARRAY'
                && ref( $hashref->{$item}->[0] ) eq 'HASH' )
            {
                _finder(
                    '__path-parts__'      => $path_parts,
                    '__array__'           => $hashref->{$item},
                    '__accessor-string__' => $accessor_string
                        . "->[$i]"
                        . "->{'$item'}",
                    '__case__'              => $case,
                    '__deep-search-items__' => $deep_search_items,
                    '__results__'           => $matches,
		    '__is-regex__'	    => $is_regex,
                    %args
                );
                $count++ if ( $$deep_search_items{$item} );
            }

            next
                if (
                ( $item eq '__accessor-string__' )
                || (   !exists( $args{$item} )
                    && !exists( $args{'__any-key__'} ) )
                );

            $count++
                if (
                (   exists( $args{$item} ) && _is_found(
                        'search-query' => $args{$item},
                        'value'        => $hashref->{$item},
                        'case'         => $case,
			'is-regex'     => $is_regex
                    )
                )
                || (exists( $args{'__any-key__'} )
                    && _is_found(
                        'search-query' => $args{'__any-key__'},
                        'value'        => $hashref->{$item},
                        'case'         => $case,
                        'is-regex'     => $is_regex
                    )
                )
                );
        }

        next if ( $count < scalar( keys(%args) ) );

        # comes here only when search is successful
        if ( !$part ) {
            $hashref->{'__accessor-string__'} = $accessor_string . "->[$i]";
            push( @$matches, $hashref );
        }
    }
}

##############################################################################
# Method:    _is_found
# Objective: Function that indicates whether the given item satisfies the
#            criteria
##############################################################################

sub _is_found {

    my %args         = (@_);
    my $search_query = $args{'search-query'};    # the search parameter

    #replace trailing \ with blank #burt975626
    if ( $search_query =~ /\\$/) {
       $search_query =~ s/\\$//;
    }

    # the value that is checked for the existance of the search parameter
    # this is the value is the parsed output structure
    my $value = $args{'value'};

    my $case = $args{'case'};    # for case sensitive/insensitive search

    my $is_regex = $args{'is-regex'};	# for regex/non-regex context search
    $is_regex = 1 if (! defined $is_regex);

    $search_query =~ s/\\/\\\\/g if (!$is_regex);	#append escape character before '\' #burt1011571

    my $flag = 0;                # flag gets set only if search is successful

  # if the value is not a simple string but an array of simple strings and not
  # an array of hashes

    if (   ref($value) eq 'ARRAY'
        && !ref( $value->[0] )
        && ref($search_query) eq 'ARRAY' )
    {
        $flag = 1
            if (
            _find_in_array(
                'search-array' => $search_query,
                'array'        => $value,
		'is-regex'     => $is_regex
            )
            );
    } elsif ( !ref($value) && $value =~ /(?$case)$search_query/ ) {
        $flag = 1;
    }
    return $flag;
}

##############################################################################
# Method:    _find_in_array
# Objective: Function that checks whether the given array contains all items
#            present in the array specified as the search parameter
##############################################################################

sub _find_in_array {
    my %args = (@_);

    my $search_array_ref = $args{'search-array'};    # the search parameter
    my $array_ref = $args{'array'}; # the array reference in the parsed output
    my $case      = $args{'case'};  # for case sensitive/insensitive search
    my $is_regex = $args{'is-regex'};   # for regex/non-regex context search
    $is_regex = 1 if (! defined $is_regex);

    # flag remains set only if all elements in the search parameter(array) are
    # contained in the parsed output
    my $flag = 1;

    foreach my $temp (@$search_array_ref) {
	$temp =~ s/\\/\\\\/g if (!$is_regex);       #append escape character before '\' #burt1011571
        unless ( grep { $_ =~ /(?$case)$temp/ } @$array_ref ) {
            $flag = 0;
            last;
        }
    }
    return $flag;
}

# Storable blesses the object (during thaw) with the appropriate class
# name and load the library if we place storable hooks. Hence the foll
# hooks are added, otherwise there is no customization needed in freeze
# and thaw operations.

use Storable qw(freeze thaw);

sub STORABLE_freeze {
    my ( $self, $cloning ) = @_;
    return if $cloning;    # Regular default serialization
    my $data  = {%$self};
    my $dummy = "";
    return ( $dummy, $data );
} ## end sub STORABLE_freeze

sub STORABLE_thaw {
    my ( $self, $cloning, $dummy, $data ) = @_;
    %$self = %$data;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response - Response base class

=head1 SYNOPSIS

 use NACL::APISet::Response;

=head1 DESCRIPTION

A C<NACL::APISet::Response> contains data and methods common to all Response
classes. This includes functionality to access the execution time, execution
status, parsed output and to set the parsed output.

=head1 EXCEPTIONS

None.

=head1 CONSTRUCTOR

None. This class is present only to provide common functionality, an object of
this class should never be instantiated. We should only instantiate objects
of Response sub-classes.

=head1 Methods

=head2 get_raw_output

This method is used to return the raw command output.

=over

=item Synopsis

$raw_output = $response_object->get_raw_output();

=item Arguments

None.

=item Returns

(Scalar)
Raw command output.

=back

=head2 get_execution_time

This method returns the time taken (in seconds) to execute the command on the
resource

=over

=item Synopsis

$execution_time = $Api_Set_Object->get_execution_time();

=item Arguments

None.

=item Returns

(Scalar)
Execution time in seconds

=back

=head2 get_execution_status

This method returns the exit status of the executed command

=over

=item Synopsis

$execution_status = $Api_Set_Obj->get_execution_status();

=item Arguments

None.

=item Returns

(Scalar)
Exit status of executed command

=back

=head2 get_command

This method returns the command invoked in the service

=over

=item Synopsis

$command = $Api_Set_Object->get_command();

=item Arguments

None.

=item Returns

(Scalar)
Command invoked on the service

=back

=head2 get_parsed_output

This method returns the parsed output.

=over

=item Synopsis

$parsed_output = $Api_Set_Object->get_parsed_output();

=item Arguments

None.

=item Returns

(Reference to an array of hashes)
Parsed command output

=back

=head2 set_parsed_output

This method sets the parsed output of the Response object.

=over

=item Synopsis

$Api_Set_Object->set_parsed_output(parsed_output => $parsed_output);

=item Arguments

=over

=item parsed_output

(Reference to an array of hashes)
The parsed output.

=back

=item Returns

Nothing.

=back

=head2 search

This method returns an array of search results  for a search performed on the
response object

=over

=item Synopsis

$command = $Api_Set_Object->search('__search-array-ref__'=>$arrar_ref,
                                   '__path__'=>$unix_like_path,
                                   '_deep-search-items__'=>[...],
                                   '__match-case__'=>$boolean,
                                   '__accessor-string__'=>$accessor_string,
				   '__is-regex__'=>$search_in_regex_context,
                                   key=>value ...);

=item Arguments

=over

=item __match-case__

(optional)
1 or 0 value indicating whether the search should be case-sensitive
Default value is 0 ( case-insensitive search )

=item __is-regex__

(optional)
1 or 0 value indicating whether the search should be in a regex context
Default value is 1 ( regex-context search )

=item __search-array-ref__

(optional)
array refrence to the array on which search must be performed
Entire parsed output structure of the response object is taken by default.

=item __deep-search-items__

(optional)
array refrence to the array which holds the list of key names for which a deep
search must be performed

=item __accessor-string__

(optional)
initial value of string which will be appended with the perl statement
that indicates how to access the found element(hashref)

=item __path__

(optional)
unix like path to indicate where the search must begin
Default value is / in which case searches are performed from the topmost level

=item keys and values

(required)
key value pairs which indicate the search parameters. value can be in regex format.


=back

=item Returns

(Array)
Search results (each result is a hash containing all the key-value pairs in the
hash where the match occured)

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
