#
# TBD - add full documentation
#
package NACL::APISet::Response::CLI;

use strict;
use warnings;

use base  qw(NACL::APISet::Response);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Schema;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::ParserNotLoadedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

### Parse the provided input buffer into a standard
# Response class.
#
sub _parse_output {
    $Log->enter() if $may_enter;

    my ($self, %params) = @_;
    no strict 'refs';

    my $output = $self->get_raw_output();
    my $parser_file_path = $self->{parser_file};

    # set parsed output field to undef if unable to locate parser file
    # instead of throwing exception. This is helpful for cases where the cdefs
    # have been created but not the parsers.

    unless (NATE::Inc::find_file($parser_file_path, register_for_s2t => 0)) {
        $self->{parsed_output} = undef;

        # for now this is just for 7Mode
        $self->_check_error() if (ref $self eq 'NACL::APISet::Response::CLI::7Mode');
        return;
    }
    # Now remove the command echo from raw output before sending it to parser.
    # This cannot be done in parser since the command line sent to the parser
    # will not be a full command line.
    remove_echo("output_ptr"=>\$output,"command"=>$self->{command});
    #Remove lines "Trying Kerberos rsh..." from the processed output. Refer burt490247 for more details.
    $output =~ s/connect.*:\sconnection\srefused\r*\n*trying krb4 rsh.*\r*\n*connect.*:\sconnection\srefused\r*\n*trying normal.*\r*\n*//i;
    #set the processed_output field
    $self->_set_processed_output('processed_output' => $output);

    $Log->debug($parser_file_path."\n");

    my $parsed_output = undef;

    # Check whether the command is a help command. Commands with a '?'
    # or '-?' as a single word at the end will be treated as
    # help commands. If its a help command, we need not call the parser method
    # to parse the raw output.

    if (is_help_command("command"=>$self->{actual_command},
                                    "raw_output"=>$output)) {
        my $outp = help_output("raw_output"=>$output);
        $parsed_output = [{help_output=> $outp}];
    } else {

        my $parser_file_package = $parser_file_path;
        my $string = NATE::Inc::get_file($parser_file_path, register_for_s2t => 0);
        $parser_file_package = $1 if ($string =~ /^\s*package\s+(.+);/m);

        $Log->debug("Parser file package name constructed: ".
                    "$parser_file_package\n");

        # Switch the context to package $parser_file_package and try to execute
        # the parser file's contents using require to avoid redefining
        # subroutines if already defined.
        my $function_name = $parser_file_package."::".$self->{method_name};
        if( ! defined(&$function_name)){
            eval "require $parser_file_package ;";

            if ($@) {
                # This exception will be thrown if the parser is not loaded
                # properly (some syntax errors in the parser or if dependent
                # packages are not found / have errors in them)
                NACL::APISet::Exceptions::ParserNotLoadedException->throw(
                                      text => "Error in loading parser: $@",
                                      response_object =>$self);
            }

        }
        if(defined(&$function_name)) {
            try {
                $parsed_output = &$function_name(response_object=>$self);
            } catch NATE::BaseException with {
                # NATE::Baseexception is used instead of ResponseException
                # because do_mount.pm parser throws timeout exception which
                # is not a subclass of ResponseException.
                my $ex = shift;
                # Remove ctrl-M from error message which is taken from raw
                # output.
                $ex->{'-text'} =~ s/\cM//g;
                $ex->throw();
            };
        }
    }
    $Log->debug(Dumper($parsed_output));
    $self->{parsed_output} = $parsed_output;
}

##############################################################################
# Method:    get_row
# Objective: Returns the row of parsed output corresponding to the row index
#            provided
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_row {
    my ($self, %args) = @_;
    my $index=$args{index};
    my @output = @{$self->{parsed_output}};

    my $num_rows = scalar(@output);

    if (($index=~/^[0-9]+$/) and ($index<$num_rows)) {
        return $output[$index];
    } else {
        NACL::APISet::Exceptions::RowNotFoundException->throw(
        text => "Invalid Index number passed to get_row\n" .
        "Index number of last row present in output is " . ($num_rows-1) . ", ".
        "index passed in was $index",
        response_object => $self);
    }
}

##############################################################################
# Method:    get_actual_command
# Objective: Returns the actual translated command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_actual_command {
    my $self = shift;
    return $self->{actual_command};
}

#
##Internal method to set the processed output to the given value
#

sub _set_processed_output {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    $self->{processed_output} = $args{processed_output};
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    get_processed_output
# Objective: Returns the processed output after removing echo
# Details:   See POD documentation at the end of this file.
##############################################################################

sub get_processed_output {
    my $self = shift;
    return $self->{processed_output};
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI - API Set Response base class for CLIs

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI> contains data and methods common to all API Set
Response CLI classes. This includes accessors for raw output, actual command invoked,
getting one row of output and logic to parse output for APISets which will have
multiple parsers divided into different packages.

Inherit from this class when creating new API Set Response classes
for CLI.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::RowNotFoundException

=item NACL::APISet::Exceptions::ParserNotLoadedException

=back

=head1 CONSTRUCTOR

None. This class is present only to provide common functionality, an object of
this class should never be instantiated. We should only instantiate objects
of Session sub-classes.

=head1 Methods

=head2 get_row

This method is used to return the row of parser output
corresponding to the row index provided.

=over

=item Synopsis

%row = $response_object->get_row();

=item Arguments

None.

=item Returns

(Hash-ref)
Hash corresponding to the row of output

=back

=head2 get_actual_command

This method is used to return the command translated by the schema. This could
differ from the command invoked on the service.
For example, if we want to run a command with privilege level diag on a vfiler
vfiler1 and then reset the privilege to the previous value after running the
command ran, the command invoked on the service is
priv -set diag; vfiler run -q vfiler1; disk show -v; result -l; priv -set $oldvalue
The "actual command" translated by the schema is "disk show -v"

=over

=item Synopsis

$actual_command = $response_object->get_actual_command();

=item Arguments

None.

=item Returns

The actual command translated by the schema.

=back

=head2 get_processed_output

This method is used to return the processed raw output (after removing command
echo).

=over

=item Synopsis

$processed_output = $response_object->get_processed_output();

=item Arguments

None.

=item Returns

The processed raw output

=back
