#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Resonse class for all DFM API Sets
## @author tanzim.rahmathullah, dl-sm-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::CLI::DFM;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Response::CLI);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# Exceptions imported
#
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Exceptions::CapabilityException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);
use NACL::APISet::Exceptions::ServiceException qw(:try);

##############################################################################
# Method:  _check_for_errors
# Purpose: Check for errors and throw a relevant exception if an error is
#          found.
# Input:   None.
# Output:  None.
# Notes:   If an error occurs while running a command on a DFM system,
#          the output will mostly be in the form:
#
#             "ERROR: <error message>"
#
#	   The error strings are intuitive enough to be classified under
#	   related generic exception types.When encountered in the command
#	   output, the corresponding exception type is thrown
#
#	      There maybe a few instances where the output is not preceded
#	      by the "Error" string.These instances should be handled
#	      and corresponding patterns added to their respective
#	      exception types in the function whenever
#	      they are encountered.
#
# Exceptions: The following are the exceptions that this method could throw:
#             1) MissingRequiredParamException
#             2) InvalidParamException
#             3) InvalidParamValueException
#             4) UnsupportedOptionException
#	      5) CapabilityException
#	      6) FeatureNotLicensedException
#	      7) ServiceException
#             8) ResponseException
##############################################################################

sub _check_for_errors {
    $Log->enter() if $may_enter;

    my $self = shift;
    my $output = $self->{processed_output} || '';

    my $exception;

    #my $pattern_hash;
    $Log->debug( "Raw output is " . $output );

    # The substitution below is not of significant importance per se.
    # DFM errors are not structurally consistent , they are not always
    # preceded by an "Error:" or "Usage:" string.The substitution below
    #is to bring consistence to the pattern matching strings.
    if ( $output =~ /ERROR:\s/i ) {
        $output =~ s/ERROR:\s//;
    }

    # A default exception type cannot be thrown due to the same reason as
    # mentioned above , inconsistency in the DFM errors.There is no fixed
    # error string common across all DFM errors which makes pattern matching
    # every error string mandatory.

    if (   ( $output =~ /Required\s.*\soption\sis\smissing.*/i )
        or ( $output =~ /.*?option\srequires\san\sargument/i ) )
    {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(
            text            => "$output",
            response_object => $self
        );
    } elsif ( ( $output =~ /The.*?command\stakes.*?arguments;\syou\sgave.*/i )
        or ( $output =~ /Illegal\s+option.*/i )
        or ( $output =~ /.*?is\snot\sa\svalid\soption.*/i ) )
    {
        NACL::APISet::Exceptions::InvalidParamException->throw(
            text            => "$output",
            response_object => $self
        );
    } elsif (
        ( $output    =~ /There\sis\sno\sobject\snamed.*/i )
        or ( $output =~ /Could\snot\screate.*/i )
        or ( $output =~ /Could\snot\sdelete.*/i )
        or ( $output =~ /Output\sformat.*?is\snot\svalid.*/i )
        or ( $output
            =~ /There\sis\sno\shost,\svolume,\sresource\sgroup,\sor\saggregate\snamed\.*/i
        )
        or ( $output =~ /Option.*?requires\sa\svalue.*/i )
        or ( $output =~ /.*already\sexists.*/i )
        or ( $output
            =~ /.Only\shttp\sservice\sconfiguration\scan\sbe\schanged./ )
        )
    {
        NACL::APISet::Exceptions::InvalidParamValueException->throw(
            text            => "$output",
            response_object => $self
        );
    } elsif ( ( $output =~ /DFM\sDatabase\sWrite\spermission\sdenied.*/i )
        or ( $output =~ /You\smust\sbe\sroot\/administrator/i ) )
    {
        NACL::APISet::Exceptions::CapabilityException->throw(
            text            => "$output",
            response_object => $self
        );
    } elsif (
        ( $output =~ /your\slicense\sdoes\snot\senable\sthis\scommand.*/i )
        or ( $output
            =~ /license-key:\sError:\sis\snot\sa\svalid\slicense\scode.*/i )
        )
    {
        NACL::APISet::Exceptions::FeatureNotLicensedException->throw(
            text            => "$output",
            response_object => $self
        );
    } elsif (
        (   $output
            =~ /The\sDataFabric\sManager\sdatabase\sis\stemporarily\sunavailable.*/i
        )
        or ( $output =~ /Cannot\schange\sthe\sservice\sconfiguration.*/i )
        )
    {
        NACL::APISet::Exceptions::ServiceException->throw(
            text            => "$output",
            response_object => $self
        );
    } else {
        NACL::APISet::Exceptions::ResponseException->throw(
            text            => "Response exception: $output",
            response_object => $self
        );
    }

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:  _is_help_command
# Purpose: Used to determine whether the user wants to view help information
#          of the command
# Inputs:  None.
# Outputs: 1 if it is a help command, 0 if not.
# Notes:   If a help is present at the end of the constructed command, then the
#          user wants help for the command.With the structure of the DFM help
#          commands , it is relatively easy to parse the output into a
#          hash with the following Keys : NAME , SYNOPSIS , DESCRIPTION
#
##############################################################################

sub _is_help_command {
    $Log->enter() if $may_enter;

    my $self = shift;
    my $help_hash;
    if ( $self->{command} =~ /help$/i ) {
        my $output = $self->{processed_output};
        $help_hash = $self->_process_help_output();
        $self->{parsed_output} = [$help_hash];

        #$self->{parsed_output} = [{'help_output'=>$output}];
        $Log->debug(
            "Help parsed output: " . Dumper( $self->{parsed_output} ) );

        $Log->exit() if $may_exit;
        return 1;
    }
    $Log->exit() if $may_exit;
    return 0;
}

##############################################################################
# Method:  _process_help_output
# Purpose: Used to parse the  output of DFM help commands
#          of the command
# Inputs:  None.
# Outputs:  hash reference with the parsed output.
# Notes:
#
##############################################################################

sub _process_help_output {

    $Log->enter() if $may_enter;

    my $self     = shift;
    my %args     = @_;
    my $help_out = $self->{processed_output};

    my $help_hash;
    my $key;

    $help_out =~ s/^\s+//smg;
    $help_out =~ s/\s+$//smg;

    my @help_arr = split( /([A-Z]{3,})/, $help_out );
    shift @help_arr;

    %$help_hash = @help_arr;

    $Log->exit() if $may_exit;

    return $help_hash;
}

##############################################################################
# Method:  _remove_echo
# Purpose: Remove the command echo from output
# Inputs:  Output from which command echo is to be stripped
# Outputs: Output with command echo stripped
# Notes:   This will probably need to be removed once we migrate to the new
#          connections which automatically takes care of removing echo.
##############################################################################

sub _remove_echo {

    $Log->enter() if $may_enter;
    my $self   = shift;
    my $output = $self->{processed_output};
    if ($output) {
        my $command = $self->{command};
        my @lines = split( /\n/, $output );
        shift @lines if ( $lines[0] =~ /$command/ );

        # Not implementing the line below as it modifies the output which
        # affects parser functionality
        # just skip blank lines
        #@lines = grep {/\w/} @lines;
        $output = undef;
        foreach (@lines) {
            $output .= $_ . "\n";
        }
    }
    $Log->debug( "Output stripped of echo is: " . $output );

    $self->{processed_output} = $output;
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:  get_rows
# Purpose: Retrieve a subset of the parsed output
# Inputs:  Field , Value
# Outputs: Returns the rows(as reference to array of hashes)containing
#	   specified field and value
# Notes:   Method taken from Response/CLI/CMode.pm
#
##############################################################################

sub get_rows {
    my ( $self, %args ) = @_;
    my $field  = $args{field};
    my $value  = $args{value};
    my @output = @{ $self->{parsed_output} };

    my @result = ();
    foreach my $hash (@output) {
        foreach my $key ( keys( %{$hash} ) ) {
            if ( $key eq $field ) {
                if ( $hash->{$key} eq $value ) {
                    my %outhash = %{$hash};
                    push( @result, \%outhash );
                }
            }    # end of if key=field
        }
    }
    return \@result;
}

##############################################################################
# Method:    get_raw_execution_time
# Objective: Return the raw time taken for execution of the command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_raw_execution_time {
    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug( "Raw execution time is " . $self->{raw_execution_time} );
    $Log->exit() if $may_exit;
    return $self->{raw_execution_time};
}

###############################################################################
# Function     : _parse_field_output()
# Input        : The raw output of a command
# Return value : Returns the key-value output in the form of an array of hashes
# Summary      : _parse_field_output() is called by the command specific parsers to
#                parse the output and return the output in the form of an array
#                of hashes containing the object name and object Id
###############################################################################

sub _parse_field_output {

    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my @arr_hashes      = ();
    my $result          = $self->{processed_output};
    my @res_array_lines = split( /\n/, $result );
    my $line;
    foreach $line (@res_array_lines) {
        if ( $line =~ /\s(\S+)\s\((\d{1,})\)/ ) {
            my $hash;
            $hash->{"ObjectName"} = $1;
            $hash->{"ObjectID"}   = $2;
            push( @arr_hashes, $hash );
        } else {
            my $hash_output;
            $hash_output->{'output'} = $result;
            push( @arr_hashes, $hash_output );
            last;
        }
    }

    $self->{parsed_output} = \@arr_hashes;
    $Log->exit() if $may_exit;
}

###############################################################################
# Function     : _parse_get_output()
# Input        : The raw output of a command
# Return value : Returns the key-value output in the form of an array of hashes
# Summary      : _parse_get_output() is called by the command specific parsers to
#                parse the output and return the output in the form of an array
#                of hashes containing the object name and object Id
###############################################################################

sub _parse_get_output {

    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $result = $self->{processed_output};
    my ($delim);
    my ( @temp1_array, @arr_hashes ) = ();
    my ( $counteq, $countsp, $countcl );
    $counteq = $countsp = $countcl = 0;

    #split details of each host/object
    my @res_array = split( /\n\n/, $result );

    #to get the delimiter from the command output
    foreach (@res_array) {
        my @array_lines = split( /\n/, $_ );

        foreach my $line_sep (@array_lines) {

            if ( $line_sep =~ /(\s\s+)|(=)|(:)/ ) {
                if ( $& eq "\=" ) {
                    ++$counteq;
                    if ( ( $counteq > 2 ) or ( $counteq >= @array_lines ) ) {
                        $delim = "=";
                        last;
                    }
                }
                if ( $& eq "\:" ) {
                    ++$countcl;
                    if ( ( $countcl > 2 ) or ( $countcl >= @array_lines ) ) {
                        $delim = ":";
                        last;
                    }
                }
                if ( $& =~ /\s\s+/ ) {
                    ++$countsp;
                    if ( ( $countsp > 2 ) or ( $countsp >= @array_lines ) ) {
                        $delim = "\\s\\s+";
                        last;
                    }
                }
            }
        }
    }

#for the case when output is not in desired format, i.e. no delimiter is obtained

    if ( not defined $delim ) {
        $self->{parsed_output} = [ { 'output' => $result } ];
        $Log->exit() if $may_exit;
        return;
    }

    #otherwise, form the array of hash splitting at the $delim

    foreach (@res_array) {
        my @array_line = split( /\n/, $_ );
        my $hash;
        foreach my $line (@array_line) {
	    $line =~ (s/[\r\n]+//) if defined $line;  
            $line =~ (s/^\s*//) if defined $line;
            my @split_output = split( $delim, $line );

            for ( my $i = 1; $i < $#split_output; $i++ ) {
                $split_output[1] = join( "$delim",
                    $split_output[1], $split_output[ $i + 1 ] );
            }
            $split_output[1] =~ (s/^\s*//) if defined $split_output[1];
            $hash->{ $split_output[0] } = $split_output[1];
        }
        push( @arr_hashes, $hash );
    }

    #return \@arr_hashes;
    $self->{parsed_output} = \@arr_hashes;
    $Log->exit() if $may_exit;
}

###############################################################################
# Function     : _parse_table_output()
# Input        : The raw output of a command
# Return value : Returns the tabular output in the form of an array of hashes
# Summary      : _parse_table_output() is called by the command specific parsers to
#                parse the output
###############################################################################

sub _parse_table_output {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my ( @result_array, @arr_hashes, $no_of_fields, @str_dashes, $line, $i );
    my ( @arr, @headings, $temp_hash, $temp, $key, $hash, $temp2 );
    my $result = $self->{processed_output};
    @result_array = split( "\n", $result );

 #   In case the output does not contain a table , return the output as it is.
    if ( $result !~ /-+/ ) {
        $self->{parsed_output} = [ { 'output' => $result } ];
        return;
    }

    #   Split result into multiple lines to parse the output
    @str_dashes   = split( " ", $result_array[1] );
    $no_of_fields = scalar(@str_dashes);
    $line         = $result_array[0];

    #   Get all the attributes(column headings).
    foreach $i ( 0 .. $no_of_fields - 1 ) {
        $arr[$i] = length( $str_dashes[$i] );
        $temp = substr( $line, 0, $arr[$i] + 1 );
        $temp2 = quotemeta($temp);
        $line =~ s/$temp2//;
        $headings[$i] = $self->_remove_spaces( 'string' => $temp );
    }
    shift(@result_array);
    shift(@result_array);

    #   Process each row and store each row data in a hash
    foreach $line (@result_array) {
        $temp_hash = {};

        # In reports where there exists a total sum for some fields, it has
        # to be removed.
        last if ( $line =~ /^Totals(\s*)\d+/ );

        foreach $i ( 0 .. $no_of_fields - 1 ) {

            if ( $i == ( $no_of_fields - 1 ) ) {
                $temp = $line;
            } else {
                $temp = substr( $line, 0, $arr[$i] );
            }

            if ( $temp =~ /([^\s]+)(\s\s+)(\s)([^\s]+)/ ) {
                $temp = $1 . $2;
            }

            $temp2 = quotemeta($temp);
            $line =~ s/$temp2//;

            if ( !( $line =~ /^\s/ ) ) {
                my $match = "";
                $line =~ /(.+?\s)/;
                $match .= $1 if ($1);
                $temp .= $match;
                $match = quotemeta($match);
                $line =~ s/$match//;
            } else {
                $line =~ s/^\s//;
            }
            $temp_hash->{ $headings[$i] }
                = $self->_remove_spaces( 'string' => $temp );
        }
        push( @arr_hashes, $temp_hash );
    }

    $self->{parsed_output} = \@arr_hashes;
    $Log->exit() if $may_exit;
}

###############################################################################
# Function     : _process_service_output()
# Input        : The raw output of a command
# Return value : Returns the multiline output in the form of an array of hashes
# Summary      : _process_service_output() is called by the command specific
#                parsers to parse the output
###############################################################################

sub _process_service_output {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $mod_output  = " ";
    my $result      = $self->{processed_output};
    my @line_output = split( /\n/, $result );

    #condense the output and insert colons as delimiters so that it can be
    #parsed by _parse_get_output

    for ( my $i = 0; $i <= $#line_output; $i++ ) {
        if ( $line_output[$i] =~ /Service:\s+/ ) {
            $line_output[$i] =~ (s/Service:\s+(.+?)(\s)/$1:/)
                if defined $line_output[$i];
            $mod_output = $mod_output . $line_output[$i] . "\n";
        }
    }
    chomp($mod_output);
    $self->{processed_output} = $mod_output;
    $self->_parse_get_output();

    #$self->{output} = $result;
    $Log->exit() if $may_exit;
}

###############################################################################
# Function     : _remove_spaces()
# Input        : String
# Return value : Returns the modified string
# Summary      : _remove_spaces is called to trim spaces from the front and back
#                of a string
###############################################################################

sub _remove_spaces (%) {

    my $self = shift;
    $Log->enter() if $may_enter;

    my %args = @_;
    my $str  = $args{'string'};
    $str =~ s/\s+$//;
    $str =~ s/^\s*//;

    $Log->exit() if $may_exit;
    return $str;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::DFM - DFM CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::DFM;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::DFM> object is used to receive, parse and hold 
the output from DFM CLI API methods.

=head1 ATTRIBUTES

=over

=item output

Raw command output

=item parsed_output

Output parsed into a standardized array of hashes structure

=item not_parsed

Lines of output not parsed - these lines would include info lines

=item command

Entire command line invoked on the DFM filer

=item execution_time

Time taken to execute the command (in seconds)

=item method_name

The base command invoked

=back

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ResponseException

=item NACL::APISet::Exceptions::MissingRequiredParamException

=item NACL::APISet::Exceptions::InvalidParamException

=item NACL::APISet::Exceptions::InvalidParamValueException

=item NACL::APISet::Exceptions::UnsupportedOptionException 

=back

=item 

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::DFM>.  

=over 

=item Synopsis

 $response = NACL::APISet::Response::CLI::DFM->new( %args );

=item Arguments

=over 

=item output

The raw output from the command.  This can be parsed into an array of key value pairs.

=item CLI_separator

The ngsh string used to separate the fields of the output.

=item command

This is the full command line invoked

=item execution_time

The time taken to execute the command (in seconds)

=item method_name

The name of the method invoked.
E.g: system_node_show

=item parsed_output

A reference to parsed output - which is an array of key-value pairs.

=item not parsed

Reference to an array of lines not parsed

=back

=back

=head1 METHODS

=head2 get_raw_output

Return the output as it was originally returned from the node, stripped of 
command echo.

=over 

=item Synopsis

$raw_output = $response->get_raw_output();

=back

=head2 get_raw_execution_time

Return the raw execution time taken i.e., only the time taken for the command
to finish executing on the DFM server without including the NATE induced time lags

=over 

=item Synopsis

$raw_time = $response->get_raw_execution_time();

=back

=item 

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-sm-nacl-dev@netapp.com)

=back

=cut

