# $Id$

#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Resonse class for all DFM API Sets
## @author tanzim.rahmathullah, dl-sm-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::CLI::DFM::DFM3;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Response::CLI::DFM);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# Exceptions imported
#
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

#use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);
use NACL::APISet::Exceptions::CapabilityException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);
use NACL::APISet::Exceptions::ServiceException qw(:try);

sub new {
    $Log->enter() if $may_enter;
    my ( $class, %args ) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            output           => { type => SCALAR },
            command          => { type => SCALAR },
            execution_time   => { type => SCALAR },
            execution_status => { type => SCALAR },
            method_name      => { type => SCALAR },
        }
    );

    # This tags the hash pointed to by $self as a
    # type of DFM
    bless $self, $class;

    # Creating a new key "processed_output" for methods that need to modify
    # the command output while retaining the original output in the "output"
    # key
    my @output_array = split( "\n", $self->{output} );

    # Removing warning messages from the output before copying
    for ( my $i = 0; $i < @output_array; $i++ ) {

        if ( $output_array[$i] =~ /^#Warning/ ) {
            $Log->debug(
                "Removing from the output, line: " . shift(@output_array) );
            $i--;
        } else {
            last;
        }
    }
    $self->{processed_output} = join( "\n", @output_array );

    # Strip command echo from raw output.
    $self->SUPER::_remove_echo();

    # Check the raw output for errors. In the case of an error, an exception
    # is thrown.
    $self->SUPER::_check_for_errors() if ( $self->{execution_status} );

# DFM contains a couple of generic parsers, which are capable of parsing
# most commands' output. However, they cannot parse (or incorrectly)
# parse output of certain commands. Command-Specific parsers for those
# commands have been written here (whose method names are the same as the
# name of the command). If it does not find an implementation for the
# method, it means that the generic parsers can be used for the command. In
# such case, it hits AUTOLOAD, which in turn calls corresponding parser methods
# based on the command
#

    my $method_name = "_" . $self->{method_name};

    # Do not parse if we determine it's a help command. If it's a help command
    # then the parsed output should be a hash with a single key (help_output)
    # containing the entire output  verbatim. Hence, parsing should not be
    # done if it's a help command.

   # The above guideline is currently followed for CMode. However , the output
   # of the help command in DFM is structured and can be parsed into a
   # meaningful data structure facilitating easy access of relevant fields
   # in the help.

    $self->$method_name() unless $self->SUPER::_is_help_command();

    $Log->debug( Dumper($self) );

    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
# Method:    AUTOLOAD
# Objective: Call corresponding parser methods based on the command type
#            if a command specific parser is not found
#
##############################################################################
sub AUTOLOAD {
    my $self    = shift;
    my $command = $self->{command};
    my $output  = $self->{processed_output};
    $Log->enter() if $may_enter;

# On the basis of the "type" of a command , a corresponding parser is invoked.
# By default , the command output is returned as the parsed output with the
# corresponding key "output" .

    if ( $command =~ /list|view|status/i ) {
        ( $command =~ /list.*-l/ )
            ? $self->SUPER::_parse_get_output()
            : $self->SUPER::_parse_table_output();

        #$self->_parse_table_output();
    } elsif ( $command =~ /detail|get|summary/i ) {
        $self->SUPER::_parse_get_output();
    } elsif (
        $command =~ /add|delete|discover|set|create|modify|pull|import/i )
    {
        $self->SUPER::_parse_field_output();
    } elsif ( $command =~ /set$/i ) {

        #remove the first line of output

        my @line_output = split( /\n/, $output, 2 );
        $self->{processed_output} = $line_output[1];
        $self->SUPER::_parse_get_output();

        #$self->{output} = join( "\n", @line_output );
    } elsif ( $command =~ /service/i ) {
        $self->SUPER::_process_service_output();
    } else {

        #$self->_default_parser();
        $self->{parsed_output} = [ { 'output' => $output } ];
    }

    $Log->exit() if $may_exit;
}

# This is to prevent scripts from failing with ugly error messages like this:
# Method 'NACL::APISet::Response::CLI::DFM:DESTROY' not defined on instance
#'NACL::APISet::Response::CLI::DFM=HASH(0x3c36760)' at ....
# The issue is essentially that if an AUTOLOAD is written, it will end up
# catching DESTROY (which is Perl's built-in implicitly called destructor).
# So, at runtime, when the class tries to define itself completely, including
# DESTROY, it fails.
# Note that this is a runtime error and occurs intermittently (basically as and
# when it pleases) so this error could be difficult to replicate. But,
# for any class with an AUTOLOAD, this error _could_ happen, so it's best
# we follow this convention of defining an empty DESTROY for such classes

# Note that an alternative mechanism of handling this is to have a condition
# in the AUTOLOAD like
# if ($AUTOLOAD eq "DESTROY") {
#     return; }

sub DESTROY { }

##############################################################################
#                               Private Methods
##############################################################################

# Command specifc parsers come here
# These are for commands whose outputs cannot be parsed using the generic
# parsers

##############################################################################
# Method:    _dfm_event_ack
# Objective: Command specific parser for "dfm event ack" command.
# Details:   This is the parser for the 'dfm event set' command.
#	     The generic parsers cannot parse the output as it is neither
#	     in the form of a table nor  key - value based.The output
#            is a single line output which gives the status of
#	     acknowledging an event.
#
#
# Note : A parser is not required for this command.The intention behind
#	 behind implementing this is to get review comments for the design,
#	 if any, which can be used as guidelines to implement command
#	 specific parsers in the future
#
##############################################################################

sub _dfm_event_ack {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;

    if ( $output
        = /Acknowledged\s+\d+\s+event\s+out\s+of\s+\d+\s+requested/i )
    {
        $self->{parsed_output} = [ { 'status' => "Acknowledged" } ];

    } else {

        $self->{parsed_output} = [ { 'status' => "UnAcknowledged" } ];
    }

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_database_get
# Objective: Command specific parser for "dfm databas get" command.
# Details:   This is the parser for the 'dfm database get' command.
#            This command contains the term  "get" but its output is a "table"
#            type output. Hence for this particular case _parse_table_output
#            method would be called.
#
#
##############################################################################

sub _dfm_database_get {
    my $self = shift;

    #my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_database_get");
    $self->_parse_table_output();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_alarm_list
# Objective: Command specific parser for "dfm alarm list" command.
# Details:   This is the parser for the 'dfm alarm list' command.
#            This command contains the term  "list" but its output is key-value
#            based similar to "get" type commands. Hence for this particular case
#            _parse_get_output method would be called.
#
#
##############################################################################

sub _dfm_alarm_list {
    my $self = shift;

    #my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_alarm_list");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_detail
# Objective: Command specific parser for "dfm detail" command.
# Details:   This is the parser for the 'dfm detail' command.
#            It calls the _parse_get_output method to parse the output.
#
#
############################################################################

sub _dfm_detail {
    my $self   = shift;
    my $output = $self->{processed_output};
    $output =~ s/\n\n/\n/g;
    $self->{processed_output} = $output;
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_detail");
    $self->_parse_get_output();
}
##############################################################################
# Method:    _dfm_data_export_schedule_status
# Objective: Command specific parser for "dfm data export schedule status" command.
# Details:   This is the parser for the 'dfm data export schedule status' command.
#            It calls the _parse_get_output method to parse the output.
#
#
############################################################################

sub _dfm_data_export_schedule_status {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_data_export_schedule_status");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _dfm_run_status
# Objective: Command specific parser for "dfm run status" command.
# Details:   This is the parser for the 'dfm run status' command.
#            It calls the _parse_get_output method to parse the output.
#
#
############################################################################

sub _dfm_run_status {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_run_status");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_config_get
# Objective: Command specific parser for "dfm config get" command.
# Details:   This is the parser for the 'dfm config get' command.
#	     This command contains the term  "get" but doesn't require parsing
#	     given in _parse_get_output
#
#
##############################################################################

sub _dfm_config_get {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_config_get");
    $self->{parsed_output} = [ { 'output' => $output } ];
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_service_list
# Objective: Command specific parser for "dfm service list" command.
# Details:   This is the parser for the 'dfm service list' command.
#	     This command contains the term  "list" but its output is key-value
#	     based similar to "get" type commands. Hence for this particular
#            case _parse_get_output method would be called.
#
#
##############################################################################

sub _dfm_service_list {
    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_service_list");
    $self->SUPER::_parse_get_output();
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _dfdrm_mirror_list
# Objective: Command specific parser for "dfdrm mirror list" command.
# Details:   This is the parser for the 'dfdrm mirror list' command.
#	     This command contains the term  "list" but its output is key-value
#	     based similar to "get" type commands. Hence for this particular
#            case _parse_get_output method would be called.
#
#
##############################################################################

sub _dfdrm_mirror_list {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfdrm_mirror_list");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _dfdrm_policy_list
# Objective: Command specific parser for "dfdrm policy list" command.
# Details:   This is the parser for the 'dfdrm policy list' command.
#	     This command contains the term  "list" but its output is key-value
#	     based similar to "get" type commands. Hence for this particular
#            case _parse_get_output method would be called.
#
#
##############################################################################

sub _dfdrm_policy_list {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfdrm_policy_list");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _dfdrm_job_list
# Objective: Command specific parser for "dfdrm job list" command.
# Details:   This is the parser for the 'dfdrm job list' command.
#	     This command contains the term  "list" but its output is key-value
#	     based similar to "get" type commands. Hence for this particular
#            case _parse_get_output method would be called.
#
#
##############################################################################

sub _dfdrm_job_list {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfdrm_job_list");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_schedule_detail
# Objective: Command specific parser for "dfm schedule detail" command.
# Details:   This is the parser for the 'dfm schedule detail' command.
#            It modifies the output to handle a corner case and then
#            calls the _parse_get_output method to parse the output.
#
############################################################################

sub _dfm_schedule_detail {
    $Log->enter() if $may_enter;
    my $self   = shift;
    my $output = $self->{processed_output};
    $output =~ s/Schedule\sDescription\s/Schedule Description  /g;
    $self->{processed_output} = $output;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_schedule_detail");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_role_list
# Objective: Command specific parser for "dfm role list" command.
# Details:   This is the parser for the 'dfm role list' command.
#	         This command gives a tabular output by default; but on specifying the
#            role-name at the end, it gives the output as a partial key-value pair, partila list
#            and partial table output
#
##############################################################################

sub _dfm_role_list {
    $Log->enter() if $may_enter;
    my $self    = shift;
    my $command = $self->{command};
    $Log->comment("In Response/CLI/DFM/DFM3/_dfm_role_list");

    $command =~ /dfm role list(.*)/;

    if ( $1 =~ /\-x/ ) {

        $Log->debug("Command has -x option");
        $self->_parse_table_output();

    } elsif ( $1 =~ /[a-zA-Z0-9]+/ ) {

        #Parse
        $Log->debug("Has something after list");

        $self->_parse_role_list_type_output(

            'list_output_type'  => ["Inherited Roles"],
            'table_output_type' => ["Capabilities"],
        );

    } else {

        #Since nothing there
        $self->_parse_table_output();
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_role_operation_list
# Objective: Command specific parser for "dfm role operation list" command.
# Details:   This is the parser for the 'dfm role list' command.
#	         This command gives a tabular output by default; but on specifying the
#            -x option at the end, it gives the output as sort of list with 3 lines for
#            each operation: name, synopsis and description.
#            This is parsed as a table output with
#
##############################################################################

sub _dfm_role_operation_list {

    $Log->enter() if $may_enter;
    my $self    = shift;
    my $command = $self->{command};
    $Log->comment("In Response/CLI/DFM/DFM3/_dfm_role_operation_list");

    if ( $command =~ /\-x/ ) {

        #Parse differently

        my ( $output, $i, $temp_hash );
        my ( @output_array, @hash_array );

        $output = $self->{'processed_output'};
        @output_array = split( "\n", $output );

        for ( $i = 0; $i < @output_array; $i++ ) {

            $temp_hash = {};

            if ( $output_array[$i] ) {

                $temp_hash->{'Operation Name'} = $output_array[$i];
                $temp_hash->{'Synopsis'}       = $output_array[ $i + 1 ];
                $temp_hash->{'Description'}    = $output_array[ $i + 2 ];

                $i += 3;
            }

            push( @hash_array, $temp_hash );
        }

        $self->{'parsed_output'} = \@hash_array;

    } else {

        $self->_parse_table_output();

    }
    $Log->exit() if $may_exit;

}

#####################################################################################################
# Method:    _parse_role_list_type_output
# Objective: Command specific parser for output of type "dfm role list <role-name>" command.
# Details:   The output contains partial get type output, partial list type, partial table output,
# Input:   Two array references:
#          'list_output_type' contains keys which are followed by list type of output
#          'table_output_type' contains keys which are followed by table type of output
#
#####################################################################################################

sub _parse_role_list_type_output {

    $Log->enter() if $may_enter;

    my $self = shift;
    my %args = @_;

    my ( $output, $temp, $i, $ins_pos );
    my ( @output_array, @hash_array, @temp_array1, @temp_array2 ) = ();
    my $temp_hash = {};

    $output = $self->{'processed_output'};
    @output_array = split( /\n/, $output );

    my @list_output_sections  = @{ $args{'list_output_type'} };
    my @table_output_sections = @{ $args{'table_output_type'} };

    for ( $i = 0; $i < @output_array; $i++ ) {

        $temp = $output_array[$i];
        @temp_array1 = split( ':', $temp );

        unless ( grep ( /$temp_array1[0]/, @table_output_sections ) ) {

            if ( grep ( /$temp_array1[0]/, @list_output_sections ) ) {

                unless ( defined $temp_array1[1] ) {

                    # Make hash with the list values as keys
                    my $temp_hash2 = {};

                    for (
                        $i++;
                        ( $output_array[$i] !~ /(.*):(.*)/ )
                            and $i < @output_array;
                        $i++
                        )
                    {
                        $temp_hash2->{ $output_array[$i] } = "";
                    }
                    $i--;
                    $temp_hash->{ $temp_array1[0] } = $temp_hash2;
                } else {

                    # Make list
                    @temp_array1 = split( ':', $temp );
                    $temp_hash->{ $temp_array1[0] } = $temp_array1[1];

                }
            } else {
                $temp_array1[1] =~ s/^\s*(.*)\s*$/$1/
                    ;    # Remove leading and trailing spaces
                $temp_hash->{ $temp_array1[0] } = $temp_array1[1];
            }
        } else {
            @temp_array2 = ();
            for (
                $i++;
                ( $output_array[$i] !~ /(.*):(.*)/ ) and $i < @output_array;
                $i++
                )
            {
                push( @temp_array2, $output_array[$i] );

            }
            $self->{processed_output} = join( "\n", @temp_array2 );
            $self->_parse_table_output();

            $temp_hash->{ $temp_array1[0] } = $self->{parsed_output};
            $i--;
        }
    }

    push( @hash_array, $temp_hash );

    $self->{parsed_output} = \@hash_array;
    $Log->exit() if $may_exit;

}
##############################################################################
# Method:    _dfdrm_job_detail
# Objective: Command specific parser for "dfdrm job detail" command.
# Details:   This is the parser for the 'dfdrm job detail' command.
#
#
##############################################################################

sub _dfdrm_job_detail {

    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my ( $output, $flag, $var, $dupvar, $temp1, $temp2, $temp3 );
    my ( @temp_array1, @hash_array, @temp_array2, @temp_array3 ) = ();
    my $temp_hash  = {};
    my $temp_hash2 = {};

    $Log->debug("In Response/CLI/DFM/DFM3/_dfdrm_job_detail");
    $output = $self->{processed_output};

    if ( $output
        =~ /Job Summary:\s*(.*)\s*Job Progress:\s*(.*)\s*Job Log:\s*(.*)\s*/si
        )
    {
        $temp1 = $1;
        $temp2 = $2;
        $temp3 = $3;
    }

    @temp_array1 = split( "\n", $temp1 );
    for ( my $i = 0; $i < @temp_array1; $i++ ) {
        my $temp = $temp_array1[$i];
        if ( $temp =~ /(job.*)\s{2,}(.*)/ ) {
            my $var    = $1;
            my $dupvar = $2;
            $temp_hash->{$var} = $dupvar;
        } else {
            next;
        }
    }
    $temp_hash2->{"Job Summary"} = $temp_hash;

    @temp_array2 = split( "\n", $temp2 );
    $var = "";
    for ( my $i = 0; $i < @temp_array2; $i++ ) {
        my $temp = $temp_array2[$i];
        if ( $temp =~ /-{3,}/ ) {
            next;
        } else {
            $var = $var . "\n" . $temp;
        }
    }
    $temp_hash2->{"Job Progress"} = $var;

    @temp_array3 = split( "\n", $temp3 );
    $dupvar = "";
    for ( my $i = 0; $i < @temp_array3; $i++ ) {
        my $temp = $temp_array3[$i];
        if ( $temp =~ /-{3,}/ ) {
            next;
        } else {
            $dupvar = $dupvar . "\n" . $temp;
        }
    }
    $temp_hash2->{"Job Log"} = $dupvar;

    push( @hash_array, $temp_hash2 );
    print Dumper( \@hash_array );
    $self->{parsed_output} = \@hash_array;
    $Log->exit() if $may_exit;

}

##############################################################################
# Method:    _dfdrm_event_list
# Objective: Command specific parser for "dfdrm event list" command.
# Details:   This is the parser for the 'dfdrm event list' command.
#
#
##############################################################################

sub _dfdrm_event_list {
    my $self   = shift;
    my $output = $self->{processed_output};
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfdrm_event_list");
    $self->_parse_get_output();
    $Log->exit() if $may_exit;
}
##############################################################################
# Method:    _dfm_about
# Objective: Command specific parser for "dfm about" command.
# Details:   This is the parser for the 'dfm about' command.
#
#
##############################################################################

sub _dfm_about {

    my $self = shift;
    $Log->enter() if $may_enter;

    my ( $output, $temp, $i );
    my ( @output_array, @hash_array, @temp_array ) = ();
    my $temp_hash = {};

    $Log->debug("In Response/CLI/DFM/_dfm_about");

    $output = $self->{processed_output};
    @output_array = split( "\n", $output );

    for ( $i = 0; $i < @output_array; $i++ ) {

        $temp = $output_array[$i];

        if ( $temp =~ /Licensed Features\s+(.*)/ ) {

            my $temp_hash2 = {};
            $temp = $1;
            $temp =~ /(.*)\s*:\s*(.*)/;
            $temp_hash2->{$1} = $2;

            for ( $i++; $output_array[$i] =~ /:/; $i++ ) {
                $temp = $output_array[$i];
                $temp =~ /\s+(.*)\s*:\s*(.*)/;
                $temp_hash2->{$1} = $2;
            }
            $temp_hash->{'Licensed Features'} = $temp_hash2;

        } elsif ( $temp =~ /Installed Plugins/ ) {

            my $temp_hash2 = {};

            for ( ; $i < @output_array and $output_array[$i]; $i++ ) {

                if ( $output_array[$i] =~ /.* (\d.*) [(](\d.*)[)] .*/ ) {
                    $temp_hash2->{$1} = "";
                    $temp_hash2->{$2} = "";
                }
            }
            $temp_hash->{'Installed Plugins'} = $temp_hash2;

        } else {

            if (   ( $temp =~ /(Version)\s+(.*)/ )
                or ( $temp =~ /(Serial Number)\s+(.*)/ )
                or ( $temp =~ /(Administrator Name)\s+(.*)/ )
                or ( $temp =~ /(Host Name)\s+(.*)/ )
                or ( $temp =~ /(Host IP Address)\s+(.*)/ )
                or ( $temp =~ /(Host Full Name)\s+(.*)/ )
                or ( $temp =~ /(Operations Manager Node limit)\s+(.*)/ )
                or ( $temp =~ /(Provisioning Manager Node Limit)\s+(.*)/ )
                or ( $temp =~ /(Protection Manager Node Limit)\s+(.*)/ )
                or ( $temp =~ /(Operating System)\s+(.*)/ )
                or ( $temp =~ /(CPU Count)\s+(.*)/ )
                or ( $temp =~ /(System Memory)\s+(.*)/ )
                or ( $temp =~ /(Installation Directory)\s+(.*)/ )
                or ( $temp =~ /(Perf Data Directory)\s+(.*)/ )
                or ( $temp =~ /(Data Export Directory)\s+(.*)/ )
                or ( $temp =~ /(Database Backup Directory)\s+(.*)/ )
                or ( $temp =~ /(Reports Archival Directory)\s+(.*)/ ) )
            {

                $temp_hash->{$1} = $2;
            }
        }

    }

    push( @hash_array, $temp_hash );
    $self->{parsed_output} = \@hash_array;

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    dfm_ssl_server_show
# Objective: Command specific parser for "dfm ssl server show"
# Description: Calls the _parse_dfm_ssl_show parser.
#
##############################################################################

sub _dfm_ssl_server_show {

    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_ssl_server_show");
    $self->_parse_dfm_ssl_show();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    dfm_ssl_self_show
# Objective: Command specific parser for "dfm ssl self show"
# Description: Calls the _parse_dfm_ssl_show parser.
#
##############################################################################

sub _dfm_ssl_self_show {

    my $self = shift;
    $Log->enter() if $may_enter;
    $Log->debug("In Response/CLI/DFM/DFM3/_dfm_ssl_self_show");
    $self->_parse_dfm_ssl_show();
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _parse_dfm_ssl_show
# Objective: Command specific parser for "dfm ssl server show" and "dfm ssl self show" commands
#
#
##############################################################################

sub _parse_dfm_ssl_show {

    my $self = shift;
    $Log->enter() if $may_enter;

    $Log->debug("In Response/CLI/DFM/DFM3/_parse_dfm_ssl_show");

    my ( $output, $temp, $i, $key, $value );
    my ( @output_array, @hash_array, @temp_array ) = ();
    my $temp_hash = {};

    $output = $self->{processed_output};
    @output_array = split( "\n", $output );

    for ( $i = 0; $i < @output_array; $i++ ) {

        if ( $output_array[$i] =~ /\-+BEGIN ([a-zA-Z ]*)\-+/ ) {

            $key   = $1;
            $value = "";

        } elsif ( $output_array[$i] =~ /\-+END $key\-+/ ) {

            $temp_hash->{$key} = $value;
        } else {
            for ( ; $output_array[$i] !~ /\-+/; $i++ ) {
                $value = $value . $output_array[$i];
            }
            $i--;
        }

    }
    push( @hash_array, $temp_hash );
    $self->{parsed_output} = \@hash_array;

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _dfm_database_query_run
# Objective: Command specific parser for "dfm database query run"
# Description: Parses the output of "dfm database query run <select-query>"
#              If the output format is mentioned, then the parsed output would contain the raw output only.
#
##############################################################################

sub _dfm_database_query_run {

    my $self = shift;
    $Log->enter() if $may_enter;

    my ( $result, $row, $temp_hash_ref );
    my ( @result_array, @headers, @row_values, @array_of_hash );

    $result = $self->{'processed_output'};

    if ( $self->{command} =~ /-F / ) {

        $self->{parsed_output} = [ { 'output' => $result } ];
        $Log->debug(
            "Sending the raw output for parsed output because of the presence of -F option"
        );
        $Log->exit() if $may_exit;
        return;
    }

    @result_array = split( "\n", $result );

    @headers = eval( $result_array[0] );
    shift(@result_array);

    @array_of_hash = ();

    foreach $row (@result_array) {

        $temp_hash_ref = {};
        $row =~ s/\$/\\\$/;
        @row_values = eval($row);

        for ( my $i = 0; $i < @row_values; $i++ ) {

            $temp_hash_ref->{ $headers[$i] } = $row_values[$i];
        }
        push( @array_of_hash, $temp_hash_ref );

    }

    $self->{parsed_output} = \@array_of_hash;

    $Log->exit() if $may_exit;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::DFM::DFM3 - DFM3 CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::DFM;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::DFM::DFM3> object is used to receive, parse and hold 
the output from DFM CLI API methods.

=head1 ATTRIBUTES

=over

=item output

Raw command output

=item parsed_output

Output parsed into a standardized array of hashes structure

=item not_parsed

Lines of output not parsed - these lines would include info lines

=item command

Entire command line invoked on the DFM filer

=item execution_time

Time taken to execute the command (in seconds)

=item method_name

The base command invoked

=back

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ResponseException

=item NACL::APISet::Exceptions::MissingRequiredParamException

=item NACL::APISet::Exceptions::InvalidParamException

=item NACL::APISet::Exceptions::InvalidParamValueException

=item NACL::APISet::Exceptions::UnsupportedOptionException 

=back

=item 

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::DFM::DFM3>.  

=over 

=item Synopsis

 $response = NACL::APISet::Response::CLI::DFM::DFM3->new( %args );

=item Arguments

=over 

=item output

The raw output from the command.  This can be parsed into an array of key value pairs.

=item CLI_separator

The ngsh string used to separate the fields of the output.

=item command

This is the full command line invoked

=item execution_time

The time taken to execute the command (in seconds)

=item method_name

The name of the method invoked.
E.g: system_node_show

=item parsed_output

A reference to parsed output - which is an array of key-value pairs.

=item not parsed

Reference to an array of lines not parsed

=back

=back

=head1 METHODS



=item Synopsis

$raw_output = $response->get_raw_output();

=back



=item Synopsis

$raw_time = $response->get_raw_execution_time();

=back

=item 

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-sm-nacl-dev@netapp.com)

=back

 $Id$
 Last revised $Date$ by $Author$

=cut

