# Hey emacs, this is basically -*- perl -*-
#$Id$
#
# Copyright (c) 2009 Network Appliance, Inc. as an unpublished work.
# All rights reserved.

## @summary Utility methods used for parsing output
## @author dl-nacl-dev
## @pod here

package NACL::APISet::Response::CLI::ParserUtils;
use vars qw(@ISA %EXPORT_TAGS @EXPORT @EXPORT_OK);
require Exporter;
use NACL::APISet::Response::CLI::CannedParsers;

@ISA = qw(Exporter);

%EXPORT_TAGS = ('all' => [qw(trim trim_string remove_echo
                             snip_only_required_output get_lines to_key_name
                             is_help_command help_output parser_method_builder)]);

@EXPORT = (@{ $EXPORT_TAGS{'all'} });

###############################################################################
# Method:    trim
# Objective: To remove leading and trailing white spaces from a string
# Details:   See POD documentation at the end of this file.
###############################################################################

sub trim(%) {
    my %args=@_;
    my $item = $args{'string_ptr'};
    $$item =~ s/^\s+//;
    $$item =~ s/\s+$//;
}

###############################################################################
# Method:    trim_string
# Objective: To remove leading and trailing white spaces from a word
# Details:   See POD documentation at the end of this file.
###############################################################################

sub trim_string(%) {
    my %args=@_;
    my $item = $args{'string'};
    $item =~ s/^\s+//;
    $item =~ s/\s+$//;
    return $item;
}

###############################################################################
# Method:    remove_echo
# Objective: To remove command echo if present in the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################


sub remove_echo(%) {
    my %args=@_;
    my $output_ptr = $args{'output_ptr'};
    my $command_line = $args{'command'};
    $command_line =~ s/([\^\$\(\)\<\[\{\\\|\>\.\*\+\?\]\}])/\\$1/g;
    $$output_ptr =~ s/^\s*$command_line\s*(;\s*echo __status__\=(\$\?|\!ERRORLEVEL\!))?//;
}

###############################################################################
# Method:    snip_only_required_output
# Objective: to remove all characters present in the output till the
#            beginning of the output
# Details:   See POD documentation at the end of this file.
###############################################################################


sub snip_only_required_output(%) {
    my %args=@_;
    my $raw_output = $args{'raw_output'};
    my $delimiter = $args{'delimiter'};
    my $output;

    if($raw_output =~ /($delimiter(.|\n)*)/) {
        $output = $1;
        return $output;
    }
    return undef;
}

###############################################################################
# Method:    get_lines
# Objective: To return an array of lines from the beginning of output excluding
#            any blank lines
# Details:   See POD documentation at the end of this file.
###############################################################################


sub get_lines(%) {
    my %args=@_;
    my $raw_output = $args{'raw_output'};
    my $first_line = $args{'first_line'};
    my $can_throw = $args{'can_throw'};
    my @lines;

    if($raw_output = snip_only_required_output('raw_output'=>$raw_output, 'delimiter'=>$first_line)){
        @lines = split(/\r*\n+/, $raw_output);
        map(s/^\s+//,@lines);
        map(s/\s+$//,@lines);
        map(s/\cM//g,@lines);
        @lines = grep (/\S+/,@lines);
    } elsif($can_throw){
        my $response_object = $args{'response_object'};
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text => "output is not in the expected format",
            response_object => $response_object);
    }
    return @lines;
}

###############################################################################
# Method:    to_key_name
# Objective: To convert a string into a form that can be used as the hash key
#            name
# Details:   See POD documentation at the end of this file.
###############################################################################


sub to_key_name(%) {
    my %args=@_;
    my $item = $args{'item'};
    $item = lc($item);
    $item =~ s/^\s+|\s+$//g;
    $item =~ s/\s+/_/g;
    $item =~ s/\(|\)//g;
    $item =~ s/:.*//g;
    $item =~ s/\cM//g;
    $item =~ s/\[|\]//g;
    return $item;
}

###############################################################################
# Method:    is_help_command
# Objective: To determine whether the command was invoked as a help command
# Details:   See POD documentation at the end of this file.
###############################################################################


sub is_help_command(%) {
    my %args=@_;
    my $command=$args{'command'};
    my $raw_output= $args{'raw_output'};
    my @words=split(/\s+/,$command);
    if (($words[$#words] =~/^\?$/) || ($words[$#words] =~/^-\?$/) ||
        ($words[$#words] =~/^help$/)||($words[$#words] =~/^--help$/)) {
        # A command will be treated as a help command only if the raw output
        # contains "Usage:" or "The following commands.....".
        # Most of the commands have the word "Usage:" in it raw output to
        # qualify it as a help command.
        # We noticed that for certain commands, the message "The following
        # commands..." comes when help is appended to the base command through
        # pass_through option.
        # Another case is appending the word "help" to a command in vfiler
        # context. If the particular command is not available in vfiler context,
        # still we can treat it as a help command if the output has "usage:" or
        # "The following commands..." because in such cases it is obvious from
        # these error messages that the filer has treated them as a help
        # command.

        if(($raw_output =~ /The following commands are available;/) ||
            ($raw_output =~ /[Uu]sage:\s?/)) {
            return 1;
        }
    }
    return 0;
}

###############################################################################
# Method:    help_output
# Objective: To return the help output (usage) as a scalar
# Details:   See POD documentation at the end of this file.
###############################################################################


sub help_output(%) {
    my %args=@_;
    my $raw_output = $args{'raw_output'};
    # Convert the scalar input into array for processing
    my @arr= split(/\r*\n+/,$raw_output);

    # Formatting the lines- Remove control characters if any present
    for (my $i=0; $i<@arr; $i++) {
        $arr[$i]=~ s/\cM//g;
        chomp($arr[$i]);
    }
    my $outp = join ("\n",@arr);
    return $outp;
}

###############################################################################
# Method:    parser_method_builder
# Objective: To build parser method for APISet commands based on hash
# Details:   See POD documentation at the end of this file.
###############################################################################

sub parser_method_builder {
    my ($pkg, %method_hash) = @_;
    ## Build methods based on method name
    while( my ($method, $other_args) = each(%method_hash) ) {
        no strict 'refs';
        *{"${pkg}::$method"} = sub {
            my %args = (@_, %$other_args);

            my $cmd_pass_string = delete $args{cmd_pass_string};
            my $cmd_fail_string = delete $args{cmd_fail_string};
            check_error_and_throw_exception(
                cmd_pass_string => $cmd_pass_string,
                cmd_fail_string => $cmd_fail_string,
                response_object => $args{response_object},
            );

            if($args{output_parser_method}) {
                my $parser_method = "${pkg}::$args{output_parser_method}";
                return $parser_method->(\%args);
            } elsif($args{canned_output_parser}) {
                my $parser_method = "NACL::APISet::Response::CLI::CannedParsers::$args{canned_output_parser}";
                return $parser_method->(\%args);
            }
            return [];
        }
    }
}


###############################################################################
# Method:    check_error_and_throw_exception
# Objective: To check for standard errors and throw an exception
# Details:   See POD documentation at the end of this file.
###############################################################################

sub check_error_and_throw_exception {
    my %args            = @_;
    my $cmd_pass_string = $args{cmd_pass_string};
    my $cmd_fail_string = $args{cmd_fail_string};
    my $resp            = $args{response_object};
    my $raw_output      = $resp->get_processed_output();
    my $error;

    if ( $raw_output =~ /error|not\sready|not\sfound|SKSIM_PANIC/i ) {
        $error = 'CommandFailed';
    }
    elsif ( $raw_output =~ /usage|incorrect/i ) {
        $error = 'InvalidParamValue';
    }

    if (( $cmd_pass_string && $raw_output !~ /$cmd_pass_string/i ) ||
        ( $cmd_fail_string && $raw_output =~ /$cmd_fail_string/i)){
        $error = 'CommandFailed';
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
} ## end sub check_error_and_throw_exception

1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::ParserUtils - Utilities for all parsers

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::ParserUtils;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::ParserUtils> contains functions that can be
used by all parser methods for performing some useful tasks

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::UnexpectedOutputException

=back

=head1 Methods

=head2 trim

This method is used to remove all white spaces at the beginning and end of the
passed scalar value

=over

=item Synopsis

trim(string_ptr=>\$string)

=item Arguments

=over

=item string_ptr

(mandatory)
pointer to a string

=back

=item Returns

Nothing. The string whose pointer was sent will now have trailing and leading
whitespaces removed.

=back

=head2 trim_string

This method is used to remove all white spaces at the beginning and end of the
passed scalar value and return the trimmed string

=over

=item Synopsis

trim_string(string=>$string)

=item Arguments

=over

=item string

(mandatory)
the string that is to be trimmed

=back

=item Returns

The trimmed string (having no leading and trailing white spaces)

=back


=head2 remove_echo

This method is used to remove command echo if present in the raw output

=over

=item Synopsis

remove_echo(output_ptr=>\$raw_output, command=>$command_line)

=item Arguments

=over

=item output_ptr

(mandatory)
pointer to the raw output

=item command

(mandatory)
command line sent

=back

=item Returns

Nothing. The raw output whose pointer was sent will be free of the command line
if it was present.

=back

=head2 snip_only_required_output

This method is used to return the output after removing all characters till the
beginning of the output

=over

=item Synopsis

snip_only_required_output(raw_output=>$raw_output, delimiter=>$beginning)

=item Arguments

=over

=item raw_output

(mandatory)
raw output of the command

=item delimiter

(mandatory)
a regex describing the beginning of the actual output

=back

=item Returns

The raw output after removing all characters till the beginning of output

=back

=head2 get_lines

This method is used to return an array of lines from the beginning of the
output after removing all blank lines. It makes use of
snip_only_required_output internally.

=over

=item Synopsis

get_lines(raw_output=>$raw_output, first_line=>$first_line,
can_throw=>$can_throw, response_object=>$resp)

=item Arguments

=over

=item raw_output

(mandatory)
raw output of the command

=item first_line

(mandatory)
a regex describing the beginning of the first expected line

=item can_throw

(optional)
a value (true or false) specifying whether this function can throw an exception
if the first expected does not appear

=item response_object

(optional)
the response object that should be passed when can_throw is true so that it can
be thrown with the exception if it occurs.

=back

=item Returns

Array of lines

=back

=head2 to_key_name

To convert a string into a form that can be used as the hash key
name while parsing the output

=over

=item Synopsis

to_key_name(item=>$value)

=item Arguments

=over

=item item

(mandatory)
a string that has to be converted to the necessary form

=back

=item Returns

the converted form of the string

=back

=head2 is_help_command

This method is used to determine if the command was invoked as a help command

=over

=item Synopsis

is_help_command(command=>$command_line, raw_output=>$raw_output);

=item Arguments

=over

=item raw_output

(mandatory)
raw output of the command

=item command

(mandatory)
the command line sent

=back

=item Returns

TRUE or FALSE

=back


=head2 help_output

This method is used to return the command usage as a scalar

=over

=item Synopsis

help_output(raw_output=>$raw_output);

=item Arguments

=over

=item raw_output

(mandatory)
raw output of the command

=back

=item Returns

command usage as a scalar

=back

=head2 parser_method_builder

This method to build parser methods based on a hash structure with keys as
APISet command methods. This method builds a method that invokes
check_error_and_throw_exception and also calls the output_parser method which
is set as the value for the key output_parser_method

=over

=item Synopsis

my %method_hash = (
    teh_init   => {},
    teh_insert => {
        cmd_pass_string => 'key\(s\) inserted',
    },
    teh_delete => {
        cmd_pass_string => 'key\(s\) removed',
    },
    teh_lookup => {
        cmd_pass_string => 'key\(s\) found',
    },
    teh_stats  => {
        output_parser_method => _output_parser_1,
    },
    teh_find => {
        output_parser_method => _output_parser_2,
    },
    ## _output_parser_1 and _output_parser_2 should be defined in the caller class
);

parser_method_builder(__PACKAGE__, %method_hash);

=item Arguments

=over

=item package name

(mandatory)
package name of the caller class

=item method hash

(mandatory)
Method hash contining APISet method names as keys and additional args that are
supposed to be passed to check_error_and_throw_exception and output_parser_method
The method set as the value for output_parser_method should be defined in the
caller class.

=back

=item Returns

Parsed output if output_parser_method flag was set else nothing.

=back

=head2 check_error_and_throw_exception

This method checks for general errors and throws an exception if an error is
encountered. It also accepts a command pass string which should be output if
the command execution succeeded.

=over

=item Synopsis

check_error_and_throw_exception(
    cmd_pass_string => $cmd_pass_string,
    response_object => $response_object,
);

=item Arguments

=over

=item response_object

(mandatory)
Response object passed to the parser.

=item cmd_pass_string

(optional)
This is the pattern thats expected to be output if the commands succeeds.
This is an optional param. Not passing this param will skip doing a regex match
for command pass string.

=back

=item Returns

nothing

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

