# $Id: //depot/prod/test/main/lib/NACL/APISet/Response/CLI/SQL.pm#7 $

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Resonse class for all SQL API Sets
## @author dl-nacl-dev@netapp.com,Shilpi.Nagpal@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::CLI::SQL;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Response::CLI);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# Exceptions imported
#

use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;

    # Fill the member vars for this instance
    my $self =
      NACL::APISet::validate_params(
                         $class,
                         params => \%args,
                         spec   => {
                             output         => {type => SCALAR, required => 1,},
                             command        => {type => SCALAR, required => 1,},
                             execution_time => {type => SCALAR, required => 1,},
                                 });

    bless $self, $class;
    $self->_check_for_errors();
    $Log->debug(Dumper($self));
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new
##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:  _check_for_errors
# Purpose: Check for errors and throw a relevant exception if an error is
#          found.
# Input:   None.
# Output:  None.
# Notes:   If an error occurs while running a SQL command,

#          The error strings are intuitive enough to be classified under
#          related generic exception types.When encountered in the command
#          output, the corresponding exception type is thrown
#
#             There maybe a few instances where the output is not preceded
#             by the "Error" string.These instances should be handled
#             and corresponding patterns added to their respective
#             exception types in the function whenever
#             they are encountered.
#
# Exceptions: The following are the exceptions that this method could throw:
#             1)CommandNotFoundException
#             2)CommandFailedException
#             3)ResponseException
##############################################################################

sub _check_for_errors {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    my $output = $self->get_raw_output();
    if ($output =~ /ERROR/) {
        if ($output =~ /(ORA\-00942: table or view does not exist)/) {
            NACL::APISet::Exceptions::CommandFailedException->throw(
                                                        text            => $1,
                                                        response_object => $self
            );
        } elsif ($output =~ /(ORA-00904: "(.*)": invalid identifier)/) {
            NACL::APISet::Exceptions::CommandFailedException->throw(
                                                        text            => $1,
                                                        response_object => $self
            );
        } else {
            NACL::APISet::Exceptions::ResponseException->throw(
                                                        text => "$output",
                                                        response_object => $self
            );
        }
    } ## end if ($output =~ /ERROR/)
    if ($output =~ /unknown command/) {
        NACL::APISet::Exceptions::CommandNotFoundException->throw(
                                                        text => $output,
                                                        response_object => $self
        );
    }

    $Log->exit() if $may_exit;

} ## end sub _check_for_errors
1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::SQL - SQL CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::SQL;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::SQL> object is used to receive, parse and hold
the output from SQL CLI API methods.

=head1 ATTRIBUTES

=over

=item output

Raw command output

=back

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CommandFailedException

=item NACL::APISet::Exceptions::CommandNotFoundException

=item NACL::APISet::Exceptions::ResponseException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::SQL>.

=over

=item Synopsis

 $response = NACL::APISet::Response::CLI::SQL->new( %args );

=item Arguments

=over

=item output

The raw output from the command.

=item command

The command invoked on the service

=item execution_time

Time taken to execute the command

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/main/lib/NACL/APISet/Response/CLI/SQL.pm

=cut

