#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Response class for Symbolapi APISet
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::Symbolapi;

use strict;
use warnings;

use base  qw(NACL::APISet::Response);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Schema;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Data::Dumper;
use Params::Validate qw(:all);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            output               => { type => SCALAR, },
            command              => { type => ARRAYREF, },
            method_name              => { type => SCALAR, },
            execution_time       => { type => SCALAR, },
        }
    );

    # This tags the hash pointed to by $object as a
    # type of CMode
    bless $self, $class;

    # Strip command echo from raw output
    my $output = $self->get_raw_output();
    $self->{"processed_output"} = $output;
    $self->{complete_raw_output} = $output;

    $self->_parse_output();
    
    # Unlike other APISets we are checking for errors
    # using the parsed output for convenience. In the case of an 
    # error, an exception is thrown. Only cases where we have a returnCode 
    # as part of the response object, can the error checking be done.
    $self->_check_for_errors() if (exists $self->{parsed_output}->{returnCode});

    $Log->debug( Dumper($self) );
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

##############################################################################
# Method:  _check_for_errors
# Purpose: Check for errors and throw a CommandFailed exception with the 
#          relevant error type if an error is found.
##############################################################################

sub _check_for_errors {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $output = $self->{'parsed_output'} || '';
    
    my $value = $self->{parsed_output}->{returnCode}{value};
    if ( $value != $self->{parsed_output}->{returnCode}{RETCODE_OK}) {
         
         # Find the type of error which has been thrown by Eseries
         foreach my $key (keys %{$self->{parsed_output}->{returnCode}}) {
             if ($self->{parsed_output}->{returnCode}{$key} eq $value) {
                $Log->exit(); 
                NACL::APISet::Exceptions::CommandFailedException->throw(
                    text            => "$self->{actual_command} failed with $key",
                    response_object => $self
                );
             }
         }
    }
    
    $Log->exit();        
} ## end sub _check_for_errors

##############################################################################
# Method:    _parse_output
# Objective: To parse the raw xml output
# Details:  This function will be invoked in the constructor call of the
#           response object. This parses the raw xml output into perl data 
#           structure similar to the parsed output obtained from using CLI
#           interface.
##############################################################################

sub _parse_output {
    $Log->enter() if $may_enter;

    my ($self, %params) = @_;
    
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    
    my $raw_output;
    my %processed_output;
    
    $raw_output = $self->{output};

    $raw_output =~ s/^\s*{//g;
    $raw_output =~ s/\s*},$//g;
    
    # Form a hash structure from the output string, and eval it
    # to get the parsed output
    my $processed_out = "%processed_output = (" . $raw_output . ");";
    
    no strict "subs";
    eval $processed_out;
    if ($@) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ResponseException->throw(
                    text            => "Error in doing an eval of the output: $@",
                    response_object => $self
        );
    }
    
    use strict "subs";
 
    $self->{complete_parsed_output} = \%processed_output;
    $self->{parsed_output} = \%processed_output;
    $Log->exit(); 
}

##############################################################################
# Method:    get_complete_raw_output
# Objective: Returns complete raw command output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_complete_raw_output {
    $Log->enter() if $may_enter;
    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->{complete_raw_output};
}

##############################################################################
# Method:    get_complete_parsed_output
# Objective: Returns the complete parsed output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_complete_parsed_output {
    $Log->enter() if $may_enter;
    my $self = shift;
    
    $Log->exit() if $may_exit;
    return $self->{complete_parsed_output};
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::Symbolapi - Symbolapi Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::Symbolapi;

=head1 DESCRIPTION

A C<NACL::APISet::Response::Symbolapi> object is used to provide parsing and other
methods for Symbolapi class. This includes accessors for raw output, 
parsed output

=head1 EXCEPTIONS

=over 

=item NACL::APISet::Exceptions::ResponseException

=item NACL::APISet::Exceptions::CommandFailedException

=back 

=head1 CONSTRUCTOR

Returns a reference to a newly created C<NACL::APISet::Response::Symbolapi>.

=head1 METHODS

=head2 get_complete_parsed_output

Returns the complete parsed output.

=over 

=item Synopsis

 $complete_parsed_output = $response->get_complete_parsed_output();
 
=item Returns

The complete parsed output of invoked Symbolapi procedure call

=back 

=head2 get_complete_raw_output

Returns the complete raw output. Or else it will be undef.

=over 

=item Synopsis

 $complete_raw_output = $response->get_complete_raw_output();
 
=item Returns

The complete raw output of invoked Symbolapi procedure call.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut


