#
# Copyright (c) 2014,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Response class for Webapi APISet
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::Webapi;

use strict;
use warnings;

use base qw(NACL::APISet::Response);

# NATE imports
use NATE::Log qw(log_global);
my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

# Exceptions
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

# CPAN stuff
use Data::Dumper;
use Params::Validate qw(:all);
use JSON;

sub new {
    $log->enter() if $may_enter;

    my ( $class, %args ) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            output         => { type => SCALAR|HASHREF, },
            execution_time => { type => SCALAR, },
        }
    );
    bless $self, $class;

    my $output = $self->get_raw_output();
    $self->{processed_output} = $output;
    $self->{complete_raw_output} = $output;

    # check for errors - will die if there is an error
    $self->_check_for_errors();

    # parse the output - will be returned in Perl data structure
    $self->_parse_output();

    $log->exit() if $may_exit;
    return $self;
} ## end sub new

##############################################################################
# Method:  _check_for_errors
# Purpose: Check for errors and throw a CommandFailed exception with the 
#          relevant error type if an error is found.
##############################################################################

sub _check_for_errors {
    $log->enter() if $may_enter;
    my $self = shift;
    my $output = $self->{complete_raw_output} || '';

    $log->debug( sub { "Raw output is:\n" . Dumper( $output ) } );
    $log->debug( sub { "Response content is:\n" . Dumper( $output->responseContent ) } );
    $log->debug( sub { "Response code is:\n" . Dumper( $output->responseCode ) } );

    my $resp_content = $output->responseContent();
    my $resp_code = $output->responseCode();
    if (($resp_code == 200) || ($resp_code == 201) || ($resp_code == 202) || ($resp_code == 203)|| ($resp_code == 204)) {
        $log->debug('response looks ok');
    } else {
        $log->comment("ERROR RESPONSE=$resp_code. CONTENT=".Dumper($resp_content));
        $log->exit(); 
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => "failed with ERROR RESPONSE=$resp_code",
            response_object => $self
        );
    }
    $log->exit();        
} ## end sub _check_for_errors

##############################################################################
# Method:    _parse_output
# Objective: To decode the output which is in JSON format
# Details:  This function will be invoked in the constructor call of the
#           response object. This parses the JSON output into perl data 
#           structure.
##############################################################################

sub _parse_output {
    $log->enter() if $may_enter;

    my ($self, %params) = @_;
    
    my $raw_output = $self->{output};
    my $processed_output;
    my $resp_content = $raw_output->responseContent();
    my $resp_code = $raw_output->responseCode();
    if ($resp_content eq "\"ok\"" || $resp_content eq "\"background\"") {
        $log->debug('Command Completed Successfully');
        $processed_output=$resp_content;
    } else {
        if ($resp_content) {
            $log->debug("DECODE JSON $resp_content");
            $processed_output = decode_json($resp_content);
            $log->debug('Decoded output is '.Dumper($processed_output));
        } else {
            $log->debug('No content to decode');
            $processed_output=$resp_content;
        }
    }

    use strict "subs";
    $self->{complete_parsed_output} = $processed_output;
    $self->{parsed_output} = $processed_output;
    $log->exit(); 
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::Webapi - Webapi Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::Webapi;

=head1 DESCRIPTION

A C<NACL::APISet::Response::Webapi> object is used to provide parsing and other
methods for Webapi class.

=head1 EXCEPTIONS

=over 

=item NACL::APISet::Exceptions::ResponseException

=item NACL::APISet::Exceptions::CommandFailedException

=back 

=head1 CONSTRUCTOR

Returns a reference to a newly created C<NACL::APISet::Response::Webapi>.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut


