# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Response class for ZAPI CMode APISet
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::ZAPI::CMode;

use strict;
use warnings;

use base qw(NACL::APISet::Response::ZAPI);

use NACL::APISet::Exceptions::NoMatchingEntriesException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::UnsupportedOptionException;
use NACL::APISet::Exceptions::VserverLockedRecordingBaselineTransfer;
use NACL::APISet::Exceptions::InternalMCCOrVDRError;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

use NACL::APISet::_Mixins::CLIZAPI qw(
    _vserver_locked_recording_baseline_transfer_regex
    _internal_mcc_or_vdr_error_regex
);


##############################################################################
# Method:  get_job_ids
# Purpose: Return Job IDs in the parsed output (if present)
# Input:   None.
# Output:  None.
# Notes:   Refer to POD below.
##############################################################################

sub get_job_ids {
    $Log->enter() if $may_enter;

    my $self = shift;
    my (@ids, $flag);
    if(wantarray) {
        $Log->exit() if $may_exit;
        map { push(@ids, $_->{'result-jobid'}) if(exists $_->{'result-jobid'} ) }
                                                @{$self->{parsed_output}};
        return @ids;
    } else {
        $Log->exit() if $may_exit;
        return $self->{parsed_output}->[0]->{'result-jobid'} if exists
                                        ($self->{parsed_output}->[0]->{'result-jobid'});
    }
}

##############################################################################
# Method:  _classify_error
# Purpose: Classify the error if possible or else invoke the base class's
#          method.
# Input:   None.
# Output:  None.
# Exceptions: The following are the exceptions that this method could throw:
#             1) NACL::APISet::Exceptions::NoMatchingEntriesException
#             2) NACL::APISet::Exceptions::MissingRequiredParamException
#             3) NACL::APISet::Exceptions::InvalidParamValueException
#             4) NACL::APISet::Exceptions::InvalidParamException
#             5) NACL::APISet::Exceptions::UnsupportedOptionException
##############################################################################
sub _classify_error {
    $Log->enter() if $may_enter;

    my $self = shift;
    my $error = $self->get_error_reason();
    if ($error =~ /Missing required input|Missing input|Missing value for zapi field/i ){
        $Log->exit() if $may_exit;
        return 'MissingRequiredParamException';
    } elsif ($error =~ /Invalid\s*\w*\s*value/i ){
        $Log->exit() if $may_exit;
        return 'InvalidParamValueException';
    } elsif ($error =~ /Extra input/i){
        $Log->exit() if $may_exit;
        return 'InvalidParamException';
    } elsif ($error =~ /operation\s+is\s+not\s+supported/i ){
        $Log->exit() if $may_exit;
        return 'UnsupportedOptionException';
    } elsif ($error =~ /does not exist|doesn't exist/) {
        $Log->exit() if $may_exit;
        return 'NoMatchingEntriesException';
    } elsif ($error =~ $self->_vserver_locked_recording_baseline_transfer_regex()) {
        $Log->exit() if $may_exit;
        return 'VserverLockedRecordingBaselineTransfer';
    } elsif ($error =~ $self->_internal_mcc_or_vdr_error_regex()) {
        $Log->exit() if $may_exit;
        return 'InternalMCCOrVDRError';
    } else {
        my $exception = $self->SUPER::_classify_error();
        $Log->exit() if $may_exit;
        return $exception;
    }
}

1;

=head1 NAME

NACL::APISet::Response::ZAPI::CMode - Cluster mode ZAPI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::ZAPI::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Response::ZAPI::CMode> object is used to recieve, parse and hold 
the output from cluster mode ZAPI API methods.

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::ZAPI::CMode>.  

=over 

=item Synopsis

 $response = NACL::APISet::Response::ZAPI::CMode->new( %args );

=item Arguments

=over 

=item output

The raw xml output from the command.  This can be parsed into an array of 
key value pairs.

=item command

The ZAPI command which has been sent to the filer.

=item execution_time

The time taken to execute the ZAPI command.

=item execution_status

The status returned after the execution of ZAPI command.
Possible values are passed and failed.

=item error_number

The error number returned on the execution of ZAPI

=item error_reason

The reason for error in case if any

=back

=back

=head2 get_job_ids

Return the job IDs from the parsed ouptut (if present). Can be invoked in
either list or scalar context. Returns the first Job ID in scalar context.

=over

=item Synopsis

@IDs = $response->get_job_ids();
$ID = $response->get_job_ids();

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

