# $Id$
#
# Copyright (c) 2007-2017 NetApp Inc.
# All rights reserved.
#
## @summary Provides a mechanism to load, retrieve, and translate schema of
##          command definitions into actual command for execution
##
##          Note original version written by Korry Nguyen (lib/San/Schema.pm)
##
## @author pressley@netapp.com, dl-nacl-dev
## @status public

package NACL::APISet::Schema;

# compiler directives
use strict;

# NATE imports
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);

# Module imports - APISets
use NACL::APISet::Schema::Cdef;
use NACL::APISet::Schema::Cdef::CModeCLI;

## Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

my $TRUE = 1;

use Class::MethodMaker [
    new    => ['-hash', 'new'],
    scalar => [{-default => 0}, 'is_windows'],
    scalar => [{-default => 0}, 'is_cmode_cli'],
    scalar => 'path',
];

#
# Schema caching Burt:475008
# ----------------------------
# Cache the cdef objects.
# If the same CDEF dir needs to be loaded, then we can point to the same
# memory location instead of loading another copy. With this caching,a loaded
# CDEF directory would continue to occupy the memory even after the APISet
# object which loaded it ceased to exist. In future, we can try to load CDEFs
# on demand if we hit any memory issues with this design.

our %nacl_apiset_cli_cdef_hash = ();

###############################################################################
# Method:    read_cdef_dir()
# Objective: Read the CDEF directory and get the list of files
# Details:   See POD documentation at the end of this file.
###############################################################################
sub read_cdef_dir {
    $Log->enter() if $may_enter;
    my $self = shift();

    my %args = validate_with(
        params => \@_,
        spec   => { "directory" => { type => SCALAR, optional => 0 }, }
    );
    my @files = ();

    # Validate cdef directory path
    my @cdef_paths = split( /\;/, $args{directory} );
    foreach my $cdef_path (@cdef_paths) {
        if ( -d $cdef_path ) {

            # retrieve cdef files. non-http path
            opendir( DIR, $cdef_path ) or do {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                    "Can not open dir: $cdef_path ");
            };
            my $sep = ( $cdef_path =~ /^\// ) ? "/" : "\\";
            foreach my $file ( readdir(DIR) ) {

                # skip files that does not have cdefs extension
                next if ( $file !~ /\.cdefs$/ );
                push @files, $cdef_path . $sep . $file;
            }
            closedir(DIR);
        } else {

            # To handle special paths like http paths
            # Read the directory to get the list of cdef files
            my $dirs = NATE::Inc::get_file($cdef_path, register_for_s2t => 0);
            unless ($dirs) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                    "Could not open $cdef_path: $@");
            }

           # For HTTP paths, the entries are returned in HTML so just parse it
            foreach my $dir_entries ( split( /\n/, $dirs ) ) {
                if ( $dir_entries =~ /a href=\"(.+\.cdefs)\"/ ) {
                    my $file = $1;
                    push @files, "$cdef_path/$file";
                }
            }
        }
    }

    $Log->exit() if $may_exit;
    return \@files;

} ## end sub read_cdef_dir

###############################################################################
# Method:    load()
# Objective: Load command definitions
# Details:   See POD documentation at the end of this file.
###############################################################################
sub load {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    %args = validate_with(
        params => \%args,
        spec   => { 
            "path"             => { type => SCALAR, },
            "files"            => { type => ARRAYREF, optional => 0 },
            "cdefs_from_build" => { type => SCALAR,   optional => 1 },
        },
    );

    my $path = $args{path};
    $path = $path.'/' if ($path !~ /\/$/);
    $self->path($path);
    my $nacl_db;
    if ( defined $nacl_apiset_cli_cdef_hash{$path} ) {
        # Already loaded
        $Log->debug("CDEF dir is already loaded for '$path' and cached");

        $Log->exit() if $may_exit;
        return;
    } else {
        require NACL::Database;
        $nacl_db = NACL::Database->new();
        if ($nacl_db) {
            my $cdefs = $nacl_db->_query_cdefs_table(path => $path);
            if ($cdefs && keys %$cdefs) {
                $Log->debug('CDEFs loaded from the DB');
                $nacl_apiset_cli_cdef_hash{$path} = $cdefs;
                $Log->exit() if $may_exit;
                return;
            }
        }
    }

    my @files = @{ $args{files} };
    
    my $cdef_pkg = 'NACL::APISet::Schema::Cdef';
    if ($self->is_cmode_cli()) {
        $cdef_pkg .= '::CModeCLI';
    }

    # get command definition from each file and create cdef object per command
    foreach my $file (@files) {
        $Log->debug( "Loading cdef: " . $file );

        # initialize command definition hash per cdef file
        my %CommandDefinitions;

        # Obtain Boolean Options from cdef file
        my @Boolean_Options;

        # get command definitions info from cdef file in order to update
        # attribute/value pairs in CommandDefinition hash
        my $content = NATE::Inc::get_file($file, register_for_s2t => 0)
            or NATE::BaseException->throw("Error reading file '$file': $@");

        eval $content;
        NATE::BaseException->throw(
            "Could not load CDEF file : $file\nReason: $@\n")
            if $@;

        my $switch_identifier_char = '-';
        if ($self->is_windows()) {
            $switch_identifier_char = '/';
        }
        
        # create cdef object for each command
        # and build command with its params
        foreach my $cdef ( keys %CommandDefinitions ) {
            # If the CDEF is already loaded, need not overwrite.
            next if ( exists $self->{$cdef} );

            # get params of each type (required, optionals, and aliases)
            my $additional_options
                = ( defined $CommandDefinitions{$cdef}{additional_options} )
                ? $CommandDefinitions{$cdef}{additional_options}
                : {};

            my $options
                = ( defined $CommandDefinitions{$cdef}{options} )
                ? $CommandDefinitions{$cdef}{options}
                : {};

            my $option_order
                = ( defined $CommandDefinitions{$cdef}{option_order} )
                ? $CommandDefinitions{$cdef}{option_order}
                : [];

            my $repeated_options
                = ( defined $CommandDefinitions{$cdef}{repeated_options} )
                ? $CommandDefinitions{$cdef}{repeated_options}
                : [];

            my $command_sep = $CommandDefinitions{$cdef}{command_sep} || " ";

            my $options_sep
                = ( defined $CommandDefinitions{$cdef}{options_sep} )
                ? $CommandDefinitions{$cdef}{options_sep}
                : {};

            my $boolean_options
                = ( scalar @Boolean_Options ) ? \@Boolean_Options : [];

            my $required_options
                = ( defined $CommandDefinitions{$cdef}{required_options} )
                ? $CommandDefinitions{$cdef}{required_options}
                : [];

            my $optional_options
                = ( defined $CommandDefinitions{$cdef}{optional_options} )
                ? $CommandDefinitions{$cdef}{optional_options}
                : [];

            my $concat_limits = $CommandDefinitions{$cdef}{concat_limits}
                || [];

            my $special_properties
                = $CommandDefinitions{$cdef}{special_properties} || {};

            my $switch_identifier
                = (defined $CommandDefinitions{$cdef}{switch_identifier}) ? $CommandDefinitions{$cdef}{switch_identifier} : $switch_identifier_char;

            my $alias_params = {};
            my %all_options_hash;
            if (@$option_order) {
                map { $alias_params->{$_} = $options->{$_} } @$option_order;
            } elsif ( exists $CommandDefinitions{$cdef}{required_options} ) {
                map { $all_options_hash{$_} = $options->{$_} }
                    @$required_options;
                map { $all_options_hash{$_} = $options->{$_} }
                    @$optional_options;
                %all_options_hash
                    = ( %all_options_hash, %$additional_options );
                $alias_params = {%all_options_hash} || {};
            }
            my $alias_map = $CommandDefinitions{$cdef}{alias_map};
            if (defined $alias_map)
            {
                my @keys = keys %{$alias_map};
                map { $all_options_hash{$_} = $options->{$alias_map->{$_}} }
                    keys %{$alias_map};
                $alias_params = {%all_options_hash} || {};
            }
            # instantiate cdef object
            my $cdef_obj = $cdef_pkg->new(
                command             => $CommandDefinitions{$cdef}{command},
                alias_params        => $alias_params,
                option_order        => $option_order,
                file                => $file,
                command_sep         => $command_sep,
                options_sep         => $options_sep,
                boolean_options     => $boolean_options,
                repeated_options    => $repeated_options,
                is_windows          => $self->is_windows(),
                concat_limits       => $concat_limits,
                special_properties  => $special_properties,
                switch_identifier   => $switch_identifier,
                alias_map           => $alias_map,
            );

            # per cdef object, store it in schema object
            $self->set_command_definition( $cdef => $cdef_obj );
        }
    }

    # Add support for cdef aliases. This code will
    # get executed only in case of set=CMode and interface=CLI

    if ( $args{cdefs_from_build} ) {

        my $cli_aliases_path = $path
            . "final\/bedrock\/export\/common\/cdefsprod\/cli_aliases";
        # Read Aliases. It is assumed that alias files will be kept in
        # "$path\cli_aliases" dir. HTTP paths are not supported if cdefs
        # are taken from build root. Just using unix path
        my $aliases = $self->_read_cdef_aliases_dir(alias_path => $cli_aliases_path);
        $self->_load_cdef_aliases( aliases => $aliases );
    }

    # Update the DB with these CDEFs
    if ($nacl_db) {
        my $row = {
            ds   => $nacl_apiset_cli_cdef_hash{$path},
            path => $path,
        };
        
        $nacl_db->_insert_table(
            row        => $row,
            table_name => 'cdefs'
        );
    }

    $Log->exit() if $may_exit;
}
 ## end sub load

###############################################################################
# Method:    translate()
# Objective: return cmd syntax for requested system operation
# Details:   See POD documentation at the end of this file.
###############################################################################
sub translate {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args = validate_with(
        params => \@opts,
        spec   => {
            method => {type => SCALAR},
            cdef_obj => {
                type     => OBJECT,
                isa      => 'NACL::APISet::Schema::Cdef',
                optional => 1,
            },
        },
        allow_extra => 1,
    );
    my $method   = delete $args{method};
    my $cdef_obj = delete $args{cdef_obj}
        // $self->get_command_definition(command => $method);

    # translate definition
    my $cmd = $cdef_obj->translate(%args);

    $Log->exit() if $may_exit;
    return $cmd;
} ## end sub translate

##############################################################################
# Method:    get_command_definition
# Objective: To obtain the command definition object of a particular command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_command_definition {
    my $self = shift @_;
    $Log->enter() if $may_enter;

    my $cdef_path = $self->{searched_cdef_path};

    my %args = NACL::APISet->validate_params(
        params => \@_,
        spec   => { "command" => { type => SCALAR, }, }
    );

    my $command = $args{command};
    $Log->debug( "Command is: " . $command );
    
    my $cdef_hash = $nacl_apiset_cli_cdef_hash{$self->path()};
    my $cdef = $cdef_hash->{$command};

    # If cdef_obj doesn't exists for the command.
    # Get cdef_obj for aliased command and return it
    if ( !defined $cdef ) {
        if ( defined( $cdef_hash->{_aliases} ) )
        {
            my $cdef_aliases = $cdef_hash->{_aliases};
            $Log->debug("Trying to load aliases from the build");
            my $alias_for_command = $cdef_aliases->{$command};
            if ( defined( $alias_for_command )) {
                $cdef = $self->get_command_definition(
                    command => $alias_for_command );
            } else {
                # The entry in the alias hash might just be for the directory,
                # for example an entry of
                # "export-policy" => "vserver export-policy",
                # would mean that any export_policy_* call should get
                # translated to vserver_export_policy_*. Here we will walk
                # through each entry in the alias hash to see if the command's
                # directory matches
                
                # NOTE: Need to make a deep-copy here. Do NOT do:
                #     each %$cdef_aliases
                # This is because $cdef_aliases is a reference, and each()
                # sets a location in the reference, so the next call to each()
                # will continue from there. So a subsequent call to
                # get_command_definition will not start iterating from the
                # beginning of the hash, but will continue from that point.
                my %cdef_aliases_copy = %$cdef_aliases;
                while (my ($old_cmd, $new_cmd) = each %cdef_aliases_copy) {
                    # The trailing underscore is VERY important to prevent
                    # going into an infinite loop, since it could lead to
                    # partial matches in the middle of a command.
                    if ($command =~ /^${old_cmd}_/) {
                        my $new_full_command = $command;
                        $new_full_command =~ s/$old_cmd/$new_cmd/;
                        my $found_alias;
                        try {
                            $cdef = $self->get_command_definition(
                                command => $new_full_command);
                            $found_alias = 1;
                        } otherwise {};
                        last if ($found_alias);
                    }
                }
            }
        }
    }

    # Throw an exception if we don't have a command definition for the
    # given command.
    NACL::APISet::Exceptions::MethodNotFoundException->throw(
              "Failed to get command definition for "
            . "command $args{command} (CDEF Path: $cdef_path) " )
        unless ($cdef);

    $Log->exit() if $may_exit;
    return $cdef;
}
 ## end sub get_command_definition

##############################################################################
# Method:    get_allowed_options_for_command
# Objective: To obtain the input options allowed for a particular command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_allowed_options_for_command {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args = validate_with(
        params => \@opts,
        spec   => {
            compatibility => {
                type    => HASHREF|UNDEF,
                default => {}
            },
            command => {type => SCALAR},
            cdef_obj => {
                type     => OBJECT,
                isa      => 'NACL::APISet::Schema::Cdef',
                optional => 1,
            },
        },
    );
    my $compat_hashref  = delete $args{compatibility};
    my $cdef            = delete $args{cdef_obj}
        // $self->get_command_definition(%args);
    my @allowed_options = keys %{$cdef->{alias_params}};
    my @compat_keys_for_command;
    my @matching_patterns = grep { $args{command} =~ /^$_/; }
        keys %{$compat_hashref};

    if (exists $compat_hashref->{_ALL_COMMANDS}->{conversion}
        and defined $compat_hashref->{_ALL_COMMANDS}->{conversion})
    {
        push(@compat_keys_for_command,
            keys %{$compat_hashref->{_ALL_COMMANDS}->{conversion}});
    }
    foreach my $matching_pattern (@matching_patterns) {
        if (exists $compat_hashref->{$matching_pattern}->{conversion}
            and defined $compat_hashref->{$matching_pattern}->{conversion})
        {
            push(@compat_keys_for_command,
                keys %{$compat_hashref->{$matching_pattern}->{conversion}});
        }

    }
    my @allowed_opts_union_compat_keys
        = _find_union(\@allowed_options, \@compat_keys_for_command);

    $Log->exit() if $may_exit;
    return @allowed_opts_union_compat_keys;
}


##############################################################################
# Method:    get_all_cdef_objects 
# Objective: To obtain the all the cdef objects on a schema
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_all_cdef_objects {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $Log->exit() if $may_exit;
    return $nacl_apiset_cli_cdef_hash{$self->{path}};

}

#############################################################################
#############################################################################
######################         PRIVATE METHODS        #######################
#############################################################################
#############################################################################

##############################################################################
# Method:    _read_cdef_aliases_dir
# Objective: Will read cdef alias directory and return all alias files
##############################################################################
sub _read_cdef_aliases_dir {
    $Log->enter() if $may_enter;

	my ($self,%args) = @_;
    my $alias_path = $args{alias_path};

	my @aliases;
    if ( -d $alias_path ) {
        # retrieve alias files. non-http path
        opendir( DIR, $alias_path ) or do {
            $Log->warn("Can not open aliased dir: $alias_path");
            $Log->exit() if $may_exit;
            return \@aliases;
        };
        foreach my $file ( readdir(DIR) ) {
            # skip files that does not have aliases extension
            next if ( $file !~ /\.aliases$/ );
            push @aliases, $alias_path . "/" . $file;
        }
        closedir(DIR);
    }

    $Log->exit() if $may_exit;
    return \@aliases;
}

##############################################################################
# Method:    _load_cdef_aliases
# Objective: Will load cdef aliases and store it in self
##############################################################################
sub _load_cdef_aliases {
    $Log->enter() if $may_enter;
    my ($self,%args) = @_;

    my @aliases = @{$args{aliases}};

	my %CommandAliases;
    foreach my $alias (@aliases) {
        my %aliases;
        my $content = NATE::Inc::get_file($alias, register_for_s2t => 0) or 
            do {
                $Log->warn("Unable to read file '$alias': $@");
                next;
            };

        eval $content;
        if ($@) {   
            $Log->warn("Could not load CDEF alias file : $alias\nReason: $@.\n" .
                       " Please submit a BURT against ONTAP regarding this.");
            next;
        }
        foreach my $key (keys %aliases) {
            my $value = $aliases{$key};

            # Replace spaces and hyphens by underscores
            $value =~ s/\s/_/g;
            $value =~ s/-/_/g;
            $key =~ s/\s/_/g;
            $key =~ s/-/_/g;

            $CommandAliases{$key} = $value;
        }
    }
    $self->set_aliases(aliases => \%CommandAliases);
    $Log->exit() if $may_exit;
}

sub set_command_definition {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my ($cmd, $cdef_obj) = validate_pos (@args, {type => SCALAR},
        {type => OBJECT, isa => 'NACL::APISet::Schema::Cdef'});

    # Add the entry to the global cache.
    $nacl_apiset_cli_cdef_hash{$self->path()}->{$cmd} = $cdef_obj;
    $Log->exit() if $may_exit;
}

sub set_aliases {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {aliases => {type => HASHREF}},
    );
    
    # Add to the CDEF hash. The leading underscore is to show that it's not
    # a command, and also prevents the possibility of clashing if a command
    # named "aliases" is introduced
    $nacl_apiset_cli_cdef_hash{$self->path()}->{_aliases} = $opts{aliases};
    $Log->exit() if $may_exit;
}

sub _find_union {
    $Log->enter() if $may_enter;
    my ($arrRef1, $arrRef2) = @_;
    my %hash;
    @hash{@$arrRef1} = undef;
    @hash{@$arrRef2} = undef;
    $Log->exit() if $may_exit;
    return keys %hash;
}

#############################################################################
#                             POD Documentation
#############################################################################
## @pod here

=pod

=head1 Package Name

Schema.pm

=head1 Author

dl-nacl-dev

=head1 Description

Schema.pm provides a mechanism to load, retrieve, and translate schema of
command definitions (cdefs) into actual commands for execution on the resource.

=head1 B<Public Methods>

=head2 B<new()>

Create a schema object.

=over

=item Inputs

=over

=item is_windows

(Optional)
Boolean option. For Windows clients, set it to 1.

=back

=item Outputs

Returns a schema object

=item Examples

 use NACL::APISet::Schema;

 # Create schema object
 #
 my $schema_obj = NACL::APISet::Schema->new();
 unless (defined($schema_obj)) {
     NATE::Exceptions::Argument->throw("can not create schema object");
 }

=back

=head2 B<load(%args)>

Load cdef schema for the list of cdef files specified

=over

=item Inputs

=item path => $cdef_path

Path to CDEF directory. This is required to cache the loaded schema.

=item files => \@files

Reference to fully qualified paths of cdef files to be loaded.

=item Outputs

None

=item Examples

 # load the cdef schema
 #
 try {
    $schema_obj->load(path => $cdef_path, files => \@files);
 } catch NATE::BaseException with {
    # If any action to be done in case of error
    # in loading
 };

=back

=head2 B<translate(%args)>

Convert command definition into actual command

=over

=item Inputs

=over

=item method

Name of the method (ie. volume_transition, lun_create, ...)

=item %args

Other attributes based on each command definition.
(See cdef files for additional attributes per command definition)

=back

=item Outputs

Return formated command that is ready to be executed on the resource

=item Examples

 # Convert cdef into formated command that can be executed on a resource
 #
 my $cmd = $schema_obj->translate(method => $method,
                                  %args);

=back

=head2 B<read_cdef_dir(%args)>

Get the list of files present in the cdef directory. The path to cdef
directory should be an absolute path and it can be a linux, windows or
HTTP path.

=over

=item Inputs

=item directory => $cdef_path

Absolute path to CDEF directory from which the CDEFs should be loaded.

=item Outputs

Reference to an array of fully qualified paths of cdef files present in
the specified directory

=item Examples

 try {
    my @files = $schema_obj->read_cdef_dir(directory => $cdef_path);
 } catch NACL::APISet::Exceptions::CDEFNotFoundException with {
    # If any action to be done in case of error
    # in reading the files
 };

=back

=head2 get_command_definition

This method is used to get the command definition object of a particular command.

=over

=item Synopsis

my $cdef_obj = $schema_obj->get_command_definition('command' => "volume_show");

=item Arguments

=over

=item command

Required
The command for which the command definition is required.

=back

=item Returns

The command definition structure of the given command of type L<NACL::APISet::Schema::Cdef|lib-NACL-APISet-Schema-Cdef-pm>.

Example:

	    $VAR1 = bless( {
                   'concat_limits' => [],
                   'boolean_options' => [],
                   'file' => '/u/swatib/nacldev/lib/NACL/APISet/Node/CLI/CMode/Rollingrock_1/Rrollingrock/network_interface.cdefs',
                   'alias_params' => {
                                       'failover-group' => '-failover-group',
                                       'home-port' => '-home-port',
                                       'failover-policy' => '-failover-policy',
                                       'firewall-policy' => '-firewall-policy',
                                       ..
									   ..
									   ..
                                       'is-home' => '-is-home',
                                       'netmask-length' => '-netmask-length',
                                       'numeric-id' => '-numeric-id',
                                       'curr-node-str' => '-curr-node-str'
                                     },
                   'is_windows' => undef,
                   'options_sep' => {},
                   'command_sep' => ' ',
                   'option_order' => [],
                   'command' => 'network interface show',
                   'repeated_options' => []
                 }, 'NACL::APISet::Schema::Cdef' );

=back

=head2 get_allowed_options_for_command

This method returns an array of option names which are accepted by the command.

=over

=item Synopsis

    my @allowed_options = $schema_obj->get_allowed_options_for_command(command => $command);

=item Arguments

=over

=item compatibility

(This option applies to CMode CLI only)
A hashref containing a mapping of the old option name with the
changed option name. Default value is an empty hashref.
The value can be obtained by using $version_manager->compatibility().

=item command

Required
The command for which the command definition is required.

=back

=item Returns

An array of option names accepted as input by that command.

=back

=head2 get_all_cdef_objects

This method returns a hashref containing cdef objects for all the commands in a schema.

=over

=item Synopsis

    my $cdef_objects = $schema_obj->get_all_cdef_objects();

=item Returns

A hashref containing cdef objects keyed by method name.

{
  volume_show => $cdef_obj_for_volume_show,
  ...
}

=back

=cut

1;
