#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Schema for SNMP APISet
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Schema::SnmpSchema;

# compiler directives
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);
use File::Basename;

# Exceptions imported
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandTranslationFailed qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NATE::BaseException qw(:try);

#
# Schema caching Burt:475008
# ----------------------------
# Cache the cdef objects.
# If the same CDEF dir needs to be loaded, then we can point to the same
# memory location instead of loading another copy. With this caching,a loaded
# CDEF directory would continue to occupy the memory even after the APISet
# object which loaded it ceased to exist. In future, we can try to load CDEFs
# on demand if we hit any memory issues with this design.

our %nacl_apiset_snmp_cdef_hash = ();

#-----------------------------------------------------------------------------------------------------------------
# Method:    new()
# Objective: To Create schema object for command translation
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub new {
    my ($class, %args) = @_;
    $Log->enter() if $may_enter;
	
	my $self = {};
    
    # bless instance into class
    my $schema = bless $self, $class;

    $self = $schema;
    
    $Log->exit() if $may_exit;
    return $schema;
} ## end sub new

#-----------------------------------------------------------------------------------------------------------------
# Method:    load()
# Objective: Load command definitions
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub load {
    my $self = shift();
    $Log->enter() if $may_enter;
    
	my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "path" => { type => SCALAR, optional => 0 } }
    );
    NATE::Exceptions::Argument->throw("'path' cannot be empty string")
        unless($args{path});

    my @files = ();
    my $path = $args{path};
    $self->{path} = $path;
	
    
    # my $path = $self->{cdef_path};
	
    if (defined $nacl_apiset_snmp_cdef_hash{$path}) {
        # Already loaded
        $Log->debug("CDEF dir is already loaded for '$path' and cached");
        $Log->exit() if $may_exit;
        return;
    }    
       
    # check for valid cdef path
    if (-d $path) {

        # retrieve cdef files. non HTTP path
        opendir(DIR, $path) or do {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                "Cannot open dir: $path ");
        };
        my $sep = ($path =~ /^\//)? "/": "\\";
        foreach my $file (readdir(DIR)) {
            # skip files that does not have cdefs extension
            next if ($file !~ /\.cdefs$/);
            push @files, ($path. $sep .$file);
        }
        closedir(DIR);
    } else {
        # To handle special paths like http paths       
        # Read the directory to get the list of cdef files
        my $dirs = NATE::Inc::get_file($path, register_for_s2t => 0);
        unless ($dirs) {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                "Could not open $path: $@");
        }
        
        # For HTTP paths, the entries are returned in HTML so just parse it
        foreach my $dir_entries (split(/\n/, $dirs)) {
            if ($dir_entries =~ /a href=\"(.+\.cdefs)\"/) {
                my $file = $1;
                push @files, "$path/$file";
            }
        }
    }

    # if files list is empty, cdef files have not been consolidated
    if (scalar(@files) == 0) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "No cdef data files found in: $path ");
    }
    
    # Save a pointer to the list of cdef file names which will be used later
    # in translate()
    $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{files}
        = $self->_arrange_files( files => \@files );
    # Now load the cdef files
    foreach my $cdef_file (@files) {
        $Log->debug("Loading cdef: " . $cdef_file);
        my $content = NATE::Inc::get_file($cdef_file, register_for_s2t => 0) or
            NATE::BaseException->throw("Error reading file '$cdef_file': $@");
        $content =~ s/\{/\$commandDefinition = \{/;
        my $commandDefinition;
        eval $content;
        NATE::BaseException->throw(
            "Could not load CDEF file : $cdef_file\nReason: $@\n")
            if $@;
        $cdef_file =~ /([\w-]+)\.cdefs$/;
        my $mib = $1;
        $self->set_command_definition($mib => $commandDefinition);
    }
    # Insert netapp.mib, iscsi*.mib as the first two entries so that search
    # would be faster.    
    $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mib_files}
        = $self->_arrange_files(
        files => [
            keys %{ $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mibs} }
        ]
        );
    # Insert the array indexe information in each object node
    # so that we can get easily get the next or previous element easily.
    foreach my $mib (
        @{ $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mib_files} } )
    {
        $Log->debug("Indexing CDEF file: ${mib}.cdefs");
        my @keys
         = keys %{ $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mibs}->{$mib} };

        my $mibName = $keys[0];
        # If there are no nodes, then skip to next CDEF file.
        next
            unless (
            defined $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mibs}
            ->{$mib}->{$mibName}->{nodes} );
        my @nodes
            = @{ $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mibs}->{$mib}
                ->{$mibName}->{nodes} };
        my $count = 0;
        map {$_->{array_index} = $count++;$_->{mib}= $mib;} @nodes;
    }
    
    $Log->exit();    
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    _arrange_files()
# Objective: Arrange the list of CDEF files in a optimized way so that searches
#            for specific OIDs become faster.
# Details:   Normally customers would be more interested in the OIDs from
#            netapp.mib and iscsi.mib and hence we push these files to the 
#            beginning of the list so that translation would be faster.
#-----------------------------------------------------------------------------------------------------------------

sub _arrange_files {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my @list = @{$args{files}};
    my @new_list = ();
    my @most_used = ();
    map {
        my $file = $_;
        $file =~ /([\w-]+)(\.cdefs)?$/;
        my $name = $1;
        if ($name =~ /^(netapp|iscsi)/) {
            push (@most_used, $file);
        } else {
            push (@new_list, $file);
        }
    } @list;
    # We want netapp.cdefs as the first entry
    @most_used = reverse(@most_used);
    @new_list = (@most_used, @new_list);
    $Log->exit() if $may_exit;
    return \@new_list; 
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    translate()
# Objective: Translate object identifiers represented in string format to their
#            respective numeric OIDs.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub translate {
    my $self = shift();
    $Log->enter() if $may_enter;
    
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "varbind" => { type => SCALAR|ARRAYREF, optional => 0 }
                  }
    );
    my $varbind = $args{varbind};
    my $typeinfo = defined $args{typeinfo} ? $args{typeinfo} : 0;
    my %OIDhash;
    if (! ref($varbind)) {
        $Log->debug('varbind is a simple string');
        my $res = $self->convert_to_numericOID(object => $varbind);
        $OIDhash{oid}{$varbind} = $res->{oid};  
        $OIDhash{node}{$varbind} = $res->{node};    
    } else {
        $Log->debug('varbind is an array');
        map {my $res = $self->convert_to_numericOID(object => $_);
            $OIDhash{oid}{$_} = $res->{oid};
            $OIDhash{node}{$_} = $res->{node};
        } @$varbind;
    }
    $Log->debug("Translated varbind is\n". Dumper(\%OIDhash));
    return \%OIDhash;
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    convert_to_numericOID()
# Objective: Return numeric OID for the specified symbolic OID name.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub convert_to_numericOID {
    my $self = shift();
    $Log->enter() if $may_enter;
    
	my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "object" => { type => SCALAR, optional => 0 } }
    );
    # obtain the index if specified and search the numeric OID only for the
    # object identifier specified.
    my ($index, $object, %cdefNode);
    $object = $args{object};
    if ($object =~ /[^.]+\.(.+)/) {
        $index = $1;
        $object =~ s/([^.]+)\..+/$1/;
    }
    
    my @oids = split('_', $object);
    my @files = @{$nacl_apiset_snmp_cdef_hash{$self->{path}}->{files}};
	my $cdef_path = $self->{searched_cdef_path};
	
    my @translateCommand = ();
    
    foreach my $file (@files) {
        $file =~ /([\w-]+\.cdefs)$/;
        my $mib = $1;
        $Log->debug("$mib is looked up for object ".$object);           
        # STEP 1:
        # -------
        # To optimize the implementation, we first ensure whether each of
        # the node in the partial OID tree path is present in the cdef file
        # representing a MIB. Only then the MIB file qualifies for the next
        # step of finding numeric OID. Here we just ensure whether the name
        # of each of nodes in the specified OID path exists in the cdefs file. 
        # Sometimes this might lead to false match because the specified node
        # name might not exist as a object but it might be present as a
        # string in description of an OID which will also match our search.
        # Thus this first step is just for optimization and a cdef file
        # qualified in this step need not have the object defined in it.
        
        # The MIB files are already read once during load() and hence we
        # need not check for error in reading the file once again now.
        my $content = NATE::Inc::get_file($file);
        my @matches = grep {$_ if ($content =~ $_)} @oids;
        # If the specified partial path of object nodes doesn't exist, then
        # the MIB file need not be looked further.
        if (scalar(@oids) != scalar(@matches)) {
            $Log->debug("$mib is not qualified for further searching of ".
                "numeric OID");
            next;
        }
        $Log->debug("$mib is qualified for further searching of numeric OID");
        # STEP 2:
        # -------
        # Translate the object name to numeric OID
        my %numericOID;
        $mib =~ s/\.cdefs$//;
        my $mibs_global_hash = $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mibs};
        my @keys = keys %{$mibs_global_hash->{$mib}};
        # The outer hash in the cdefs file has got only one key which is
        # same as the MIB name.
        my $mibName = $keys[0];
        unless (defined $mibs_global_hash->{$mib}->{$mibName}->{nodes}) {
            $Log->debug("There are no nodes defined in $mib\.cdefs. ".
                "Skip to next");
            next;
        }
        my @nodes = @{$mibs_global_hash->{$mib}->{$mibName}->{nodes}};
        # Obtain the numeric OIDs for each of the object nodes specified
        # in the OID tree path.
        foreach my $oidNode (@nodes) {
            if (grep {$oidNode->{name} eq $_} @oids) {
                push(@{$numericOID{$oidNode->{name}}}, $oidNode->{oid});
                # Reference to matched nodes are stored to obtain the 
                # datatype later.
                push(@{$cdefNode{$oidNode->{name}}}, $oidNode);
            }
        }
        $Log->debug("Numeric OIDs for each of the node\n". 
            Dumper \%numericOID);
        # When the CDEF file has got the name of the node, say in its 
        # description, then the file would be qualified for further search
        # But in the second step, there won't be any numeric OID matches.
        # Hence skip this CDEF and process the next one.
        if(scalar (keys %numericOID) == 0) {
            $Log->debug("$mib.cdefs lacks description for the symbolic OID ".
                "path. Hence skipping this CDEF file");
            next;
        }
        # Ignore the invalid object paths.
        for(my $i=($#oids-1); $i>=0; $i--) {
            my @newArr = ();
            foreach my $baseOID (@{$numericOID{$oids[$i]}}) {
                my $qbaseOID = quotemeta($baseOID);
                grep {
                    push(@newArr, $baseOID) if($_ =~ /^$qbaseOID\.(\d+)$/)} 
                    @{$numericOID{$oids[$i+1]}};
            }
            
            $numericOID{$oids[$i]} = \@newArr;
        }
        $Log->debug("Numeric OIDs after ignoring invalid paths\n". 
            Dumper\%numericOID);
        # Obtain the numeric OID
        foreach my $possibleOID (@{$numericOID{$oids[$#oids]}}) {
            my $nOID = $possibleOID;
            my $fail = 0;
            for(my $i=($#oids-1); $i>=0; $i--) {
                my $currentOID;                 
                unless (grep {my $qbase = quotemeta($_); 
                    $currentOID = $_ if($nOID =~ /^$qbase\.(\d+)$/)} 
                    @{$numericOID{$oids[$i]}}) {
                    $fail =1;
                    last;
                }
                $nOID = $currentOID;
            }
            push (@translateCommand, $possibleOID) unless ($fail);
        }           
    } 
    # Invalid object specified. 
    NACL::APISet::Exceptions::MethodNotFoundException->throw(
        "CDEF entry not found for ". $object . "(CDEF Path: ". $cdef_path .")") 
        unless (scalar @translateCommand);
    # Remove the duplicate matches resulting from the same object being defined
    # in different MIBs.
    my %duplicateRemoval;
    map {$duplicateRemoval{$_} = 1} @translateCommand;
    @translateCommand = keys %duplicateRemoval;
    # If there are more than one numeric OIDs obtained, then the specified OID
    # path is not unique enough to represent a unique object and hence throw
    # an exception.
    if (scalar (@translateCommand) > 1) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandTranslationFailed->throw(
            "The specified OID path is not unique among the MIBs supported by".
            " the service. Command translation resulted in multiple OID ".
            "matches.\n" . join(", ", @translateCommand));
    }
    
    # If index had specified, append it to the translated OID.
    my $node;
    foreach my $object (keys %cdefNode){
        foreach my $nodeobj (@{$cdefNode{$object}}) {
            if ($nodeobj->{oid} eq $translateCommand[0]) {
                $node = $nodeobj;
                last;
            }
        }
    }
    my $res = $translateCommand[0];
    $res .= '.' . $index if (defined $index);    
        
    $Log->debug("oid ". $args{object}. " is translated to $res");
    $Log->exit() if $may_exit;
    return {oid => $res, node =>$node};
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    convert_numericOID_to_name()
# Objective: Return the object name corresponding to the specified numeric OID.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub convert_numericOID_to_name {
    my $self = shift();
    $Log->enter() if $may_enter;
    
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "OID" => { type => SCALAR, optional => 0 }}
    );
    my @names;
    my $oid = $args{OID};
    # If the oid starts with a leading dot, remove it.
    $oid =~ s/^\.//;
    my $loaded_mibs = $nacl_apiset_snmp_cdef_hash{$self->{path}}->{mibs};
    foreach my $mib (
        @{ $nacl_apiset_snmp_cdef_hash{ $self->{path} }->{mib_files} } )
    {
        $Log->debug("Searching for matching nodes in $mib");
        my @keys = keys %{$loaded_mibs->{$mib}};
        # The outer hash in the cdefs file has got only one key which is
        # same as the MIB name.
        my $mibName = $keys[0];
        unless (defined $loaded_mibs->{$mib}->{$mibName}->{nodes}) {
            $Log->debug("There are no nodes defined in $mib\.cdefs. ".
                "Skip to next");
            next;
        }
        my @nodes = @{$loaded_mibs->{$mib}->{$mibName}->{nodes}};
        my @relevantNodes;
        # Collect only the relevant nodes for doing refined search later
        foreach my $node (@nodes) {
            my $oid_to_compare = quotemeta($node->{oid});
            push(@relevantNodes, $node)
              if (($oid =~ /^$oid_to_compare\./) || ($oid eq $node->{oid}));
        }
        # If there are no matching nodes, then do the processing for the next
        # MIB then.
        unless(scalar @relevantNodes) {
            $Log->debug("No matching nodes found for OID $oid in $mibName");
            next;
        }
        $Log->debug("The matching nodes in $mibName are:\n". 
            Dumper(\@relevantNodes));
        # Select the node whose OID matches better with the specified numeric
        # OID.
        my $aptNode = shift(@relevantNodes);
        foreach my $node (@relevantNodes) {
            my $apt_node_oid = quotemeta($aptNode->{oid});
            if ($node->{oid}=~ /^$apt_node_oid\./) {
                $aptNode = $node;
            }
        }
        my %hash;
        $hash{node} = $aptNode;
        $hash{mibname} = $mibName;      
        my $temp_oid = $oid;
        my $aptNode_oid = quotemeta($aptNode->{oid});
        $temp_oid =~ s/^$aptNode_oid//;
        $hash{remains} = $temp_oid;
        push(@names, \%hash);       
    }
    NATE::BaseException->throw("OID $oid doesn't exist in the loaded MIBs") 
        unless (scalar @names);
    $Log->debug("The most relevant matches from various CDEF files are:\n".
        Dumper(\@names));
    my $select = $names[0];
    map {$select = $_ if(length($select->{remains}) > length($_->{remains}))} @names;
    my $name = $select->{mibname}.'::'.$select->{node}->{name};
    if(length $select->{remains}) {
        if (($select->{node}->{type} eq 'scalar') || 
            ($select->{node}->{type} eq 'column')) {
            $name .= $select->{remains} ;
        } else {
            NATE::BaseException->throw(
                "OID $oid doesn't exist in the loaded MIBs");
        }
    }
    $Log->debug("The OID $oid is translated to $name");
    $Log->exit() if $may_exit;
    return {object => $name, node => $select->{node}};
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    get_node()
# Objective: Return the CDEF entry of the node matching the specified numeric 
#            oid.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------

sub get_node {
    my $self = shift();
    $Log->enter() if $may_enter;
    
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "oid" => { type => SCALAR, optional => 0 },
                    "module" => { type => SCALAR|UNDEF, optional => 1 }
        }
    );
    
    my $oid = $args{oid};
    my $loaded_mibs = $nacl_apiset_snmp_cdef_hash{$self->{path}}->{mibs};
    my @module_array = ($args{module});
    my @modules = (defined $args{module})? @module_array :
        @{$nacl_apiset_snmp_cdef_hash{$self->{path}}->{mib_files}};
    my $return_node;
    foreach my $mib (@modules) {
        my @keys = keys %{$loaded_mibs->{$mib}};
        # The outer hash in the cdefs file has got only one key which is
        # same as the MIB name.
        my $mibName = $keys[0];
        unless (defined $loaded_mibs->{$mib}->{$mibName}->{nodes}) {
            $Log->debug("There are no nodes defined in $mib\.cdefs. ".
                "Skip to next");
            next;
        }
        my @nodes = @{$loaded_mibs->{$mib}->{$mibName}->{nodes}};
        foreach my $node (@nodes) {
            if ($node->{oid} eq $oid) {
                $return_node = $node;
                last;
            }
        }
        last if (defined $return_node);
    }
    
    NATE::BaseException->throw("Couldn't find a cdef_entry for '$oid'") 
        unless (defined $return_node);
    $Log->exit() if $may_exit;
    return $return_node;
}

#-----------------------------------------------------------------------------------------------------------------
# Method:    read_column_entries()
# Objective: Find all the column nodes of the specified table or row object.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------

sub read_column_entries {
    my $self = shift();
    $Log->enter() if $may_enter;
    
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "baseoid" => { type => SCALAR, optional => 0 },
                    "module" => { type => SCALAR, optional => 1 }
        }
    );
    
    my $baseoid = $args{baseoid};
    my $mibs_global_hash = $nacl_apiset_snmp_cdef_hash{$self->{path}}->{mibs};
    my $base_node = $self->get_node(oid => $baseoid, module => $args{module});    
    my @return_nodes;    
    my $start;
    if ($base_node->{type} eq 'table') {
        $start = $base_node->{array_index}+2;
    } elsif ($base_node->{type} eq 'row') {
        $start = $base_node->{array_index}+1;
    } else {
        NATE::Exceptions::Argument->throw("Invalid baseoid $baseoid of type ".
            "'$base_node->{type}' specified");
    }
    my @keys = keys %{$mibs_global_hash->{$base_node->{mib}}};
    my $mibName = $keys[0];
    my @nodes = @{$mibs_global_hash->{$base_node->{mib}}->{$mibName}->{nodes}};
    my $oid_to_compare = quotemeta($baseoid);
    for (my $i = $start; ; $i++) {
        if ($nodes[$i]->{oid} =~ /^$oid_to_compare\.(.+)/) {
            push (@return_nodes, $nodes[$i]);
        } else {
            last;
        }
    }
    $Log->exit() if $may_exit;
    return \@return_nodes;
}
1;

##############################################################################
# Method:    get_command_definition
# Objective: To obtain the command definition object of a particular command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_command_definition {
    my $self = shift @_;
    $Log->enter() if $may_enter;
    
    my %args = NACL::APISet->validate_params(
        params => \@_,
        spec   => { "command" => { type => SCALAR, optional => 0 } }
    );
	
	my $cdef_path = $self->{searched_cdef_path};
	
	my $cdef = $self->convert_to_numericOID( object => $args{command} );

    # Throw an exception if we don't have a command definition for the
    # given command.
    NACL::APISet::Exceptions::MethodNotFoundException->throw("Failed to get command definition for ".
        "command $args{command} (CDEF Path: ". $cdef_path .")") unless ( $cdef );

    $Log->exit() if $may_exit;
    return $cdef->{node};
} ## end sub get_command_definition

sub set_command_definition {
    my ($self, $mib, $cdef) = @_;
    $Log->enter() if $may_enter;
     if (!$mib || !($cdef) ) {
        NATE::Exceptions::Argument->throw("Could not set command definition");
    } else {
        # Add the entry to the global cache.
        $nacl_apiset_snmp_cdef_hash{$self->{path}}->{mibs}->{$mib} = $cdef;
    }
    $Log->exit() if $may_exit;
}
#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 NAME

NACL::APISet::Schema::SnmpSchema - interface to translate object names to
numeric OIDs and viceversa for SNMP

=head1 Synopsis

use NACL::APISet::Schema::SnmpSchema;

=head1 Description

Provides a mechanism to load, retrieve, and translate object names to numeric
OIDs for SNMP APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::CDEFNotFoundException

=item NACL::APISet::Exceptions::MethodNotFoundException

=item NACL::APISet::Exceptions::CommandTranslationFailed

=item NATE::BaseException

=back

=head1 CONSTRUCTOR

=head2 new

Creates a SnmpSchema object.

=over

=item Synopsis

$schema_obj = NACL::APISet::Schema::SnmpSchema->new();

=item Arguments

No input

=item Returns

A reference to the created SnmpSchema object.

=back

=head1 METHODS

=head2 load

Loads all the CDEF files present in the specified 'path' input into
the schema object itself.

=over

=item Synopsis

$SnmpSchema_obj->load(path=>$path);

=item Arguments

=over

=item path

(Required)
The path to CDEF files

=back

=item Returns

Nothing

=back

=head2 translate

Translate object identifiers represented in string format to their
respective numeric OIDs.

=over

=item Synopsis

$command = $SnmpSchema_obj->translate(varbind => \@varbind);

=item Arguments

=over

=item varbind

(Required)
It takes either a scalar or array of object names present in the MIB.The object
names can be partially qualified paths as well or just the name of the object
alone without the details of its parent nodes. When more than one object
needs translation, it can be specified as an array reference.

=back

=item Returns

Returns a hash reference having two keys oid and node, the first holding the
numeric OIDs which are translated from the specifed object names and the other
to hold the corresponding cdef entries. The values for oid and node keys are
hash references again whose keys are specified object names in varbind and 
values are oids and cdef entry of that object respectively.

=back

=head2 convert_to_numericOID

Translate the specified symbolic object name to its numeric OID.

=over

=item Synopsis

$output = $SnmpSchema_obj->convert_to_numericOID(object => $object);

=item Arguments

=over

=item object

(Required)
The object name can be partially qualified path or just the name of the object
alone without the details of its parent nodes.
Eg: aggrName or aggrTable_aggrEntry_aggrName

=back

=item Returns

Returns a hash reference having two keys oid and node. oid holds the numeric
OID information for the specifed object and node holding the cdef_entry for
that object.

=back

=head2 convert_numericOID_to_name

Translate the specified numeric OID to its object name 
(MIBName::ObjectName.Instance)

=over

=item Synopsis

$output = $SnmpSchema_obj->convert_numericOID_to_name(OID => $oid);

=item Arguments

=over

=item OID

(Required)
The numeric OID which needs to be translated to its object name.

=back

=item Returns

Returns a hash reference having two keys object and node. object holds the
object name of the specified numeric OID and node holding the cdef_entry for
that object.

=back

=head2 get_node

This method finds the cdef entry of the node having the specified
numeric oid either in the specified module if specified or in the list of
all modules (loaded mibs). NATE::BaseException will be thrown if it couldn't
find the cdef entry

=over

=item Synopsis

$node = $SnmpSchema_obj->get_node(oid => $oid);

=item Arguments

=over

=item oid

(Required)
The numeric OID whose cdef entry is required

=item module

(Optional)
The module in which the search for cdef entry should be performed

=back

=item Returns

Returns the matching cdef entry (hash ref).

=back

=head2 get_command_definition

This method is used to get the command definition object of a particular command.

=over

=item Synopsis

$Api_Set_Obj->get_command_definition('command' => "volTable_volEntry_volName");

=item Arguments

=over

=item command

Required
The command for which the command definition is required.

=back

=item Returns

The command definition structure of the given command.
Example:
  $VAR1 = {
            'array_index' => 905,
            'access' => 'readonly',
            'status' => 'current',
            'oid' => '1.3.6.1.4.1.789.1.5.8.1.2',
            'name' => 'volName',
            'mib' => 'netapp',
            'description' => 'Name of the volume.  This includes no /vol/ prefix.',
            'type' => 'column',
            'syntax' => {
                          'type' => {
                                      'name' => 'DisplayString',
                                      'module' => 'SNMPv2-TC'
                                    }
                        },
            'module' => 'NETAPP-MIB'
          };

=back

=head2 read_column_entries

This method returns the cdef entries of all the column nodes present under the
specified table or row object. NATE::Exceptions::Argument will be thrown if
the baseoid is not of type table or row.

=over

=item Synopsis

$nodes_array = $SnmpSchema_obj->read_column_entries(baseoid => $oid);

=item Arguments

=over

=item oid

(Required)
The numeric OID of the table or row node.

=item module

(Optional)
The module in which the search for cdef entries should be performed

=back

=item Returns

Returns an array reference having the cdef entries of all the column nodes.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
