## $Id$
#
# Copyright (c) 2007-2010 NetApp Inc.
# All rights reserved.
#

package NACL::APISet::Schema::ZapiSchema;

# compiler directives
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);
use NACL::APISet::_Mixins::ZAPI qw(_get_xpathcontext _xml2perl _perl2xml);
use NACL::APISet::Schema::Cdef::ZapiCdef;
use NACL::APISet::Response::CLI::ParserUtils qw(trim_string);
use NACL::Database;

# Exceptions imported

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException;
use NACL::APISet::Exceptions::MethodNotFoundException;
use NACL::APISet::Exceptions::InvalidCDEFException;
use NACL::APISet::Exceptions::InvalidXMLException;


# global variables
# TBD - get from more generic location
my $TRUE  = 1;
my $FALSE = 0;

#
# Schema caching Burt:475008
# ----------------------------
# Cache the cdef objects.
# If the same CDEF dir needs to be loaded, then we can point to the same
# memory location instead of loading another copy. With this caching,a loaded
# CDEF directory would continue to occupy the memory even after the APISet
# object which loaded it ceased to exist. In future, we can try to load CDEFs
# on demand if we hit any memory issues with this design.

our %nacl_apiset_zapi_cdef_hash = ();

#-----------------------------------------------------------------------------------------------------------------
# Method:    new()
# Objective: To Create schema object for command translation
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub new {
    my ($class, %args) = @_;
    $Log->enter() if $may_enter;

	my $self = {};

    # bless instance into class
    my $schema = bless $self, $class;

    $self = $schema;

    $Log->exit() if $may_exit;
    return $schema;
} ## end sub new


#-----------------------------------------------------------------------------------------------------------------
# Method:    load()
# Objective: Load command definitions
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub load {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $path;
    my @files = ();
    $path = $args{path};
    $self->{path} = $path;
    my $nacl_db;
    if (defined $nacl_apiset_zapi_cdef_hash{$path}) {
        # Already loaded in the global hash
        $Log->debug("CDEF dir is already loaded for '$path' and cached");
        $Log->exit() if $may_exit;
        return;
    } else {
        # Check if it's present in the NACL DB
        $nacl_db = NACL::Database->new();
        if ($nacl_db) {
            my $cdefs = $nacl_db->_query_zapi_cdefs_table(path => $path);
            if (defined $cdefs) {
                $Log->debug("CDEF dir for '$path' read from DB");
                $nacl_apiset_zapi_cdef_hash{$path} = $cdefs;
                $Log->exit() if $may_exit;
                return;
            }
        }
    }

    # process parameters
    while (my ($key, $value) =  each(%args)) {
        if ($key =~ /path$/) {
            if (! length($path = $value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Invalid value for \"$key\": " . $value);
            }
        }
    }

    unless (defined $path and length $path) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Mandatory argument \"path\" not specified ");
    }

    # check for valid cdef path
    if (-d $path) {

        # retrieve cdef files. non HTTP path
        opendir(DIR, $path) or do {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                "Cannot open dir: $path ");
        };
        my $sep = ($path =~ /^\//)? "/": "\\";
        foreach my $file (readdir(DIR)) {
            # skip files that does not have cdefs extension
            next if ($file !~ /\.xml$/);
            push @files, ($path. $sep .$file);
        }
        closedir(DIR);
    } else {
        # To handle special paths like http paths
        # Read the directory to get the list of cdef files
        my $dirs = NATE::Inc::get_file($path, register_for_s2t => 0);
        unless ($dirs) {
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                "Could not open $path: $@");
        }

        # For HTTP paths, the entries are returned in HTML so just parse it
        foreach my $dir_entries (split(/\n/, $dirs)) {
            if ($dir_entries =~ /a href=\"(.+\.xml)\"/) {
                my $file = $1;
                push @files, "$path/$file";
            }
        }
    }

    # if files list is empty, cdef files have not been consolidated
    if (scalar(@files) == 0) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "No cdef data files found in: $path ");
    }

    # Load each of the XML file and get an XPATH object for each of the
    # ZAPI and store them directly in the name of ZAPI in schema object
    foreach my $cdef_file (sort @files) {

        $Log->debug("Loading cdef: " . $cdef_file);
        $cdef_file =~ /([\w-]+)\.xml$/;
        my $file = $1;
        my $content = NATE::Inc::get_file($cdef_file, register_for_s2t => 0) or
            NATE::BaseException->throw("Error reading file '$cdef_file': $@");
        
        my $xpc = $self->_get_xpathcontext(xmlString => $content);
        
        # If there any options whose name is "command", even that will be
        # picked up. So ignore processing in that case
        # The 'command' tag which we are interested in will have no
        # attributes and there are no child nodes under it.
        # Its always of the form
        #           <command>zapi_name</command>
        # This is the basis for our check, which can be done by
        # including '[not(@*)]' in XPath Expression
        my @nodes = $xpc->findnodes('//command[not(@*)]');
        foreach my $node (@nodes) {
            my $command = $node->string_value;
            my @commands = $xpc->findnodes("/$file/$command");
            if (scalar (@commands) != 1) {
                $Log->exit() if $may_exit;
                NACL::APISet::Exceptions::InvalidCDEFException->throw(
                    "The ZAPI '$command' should be defined only once in the ".
                    "XML CDEFs but its defined ". scalar (@commands) .
                    " times");
            }
            
            my ($zapi_command_node) = $xpc->findnodes("/$file/$command/$command");

            # Populate the hash with all the necessary information about the
            # command for later usage. The file name from which the ZAPI
            # definition is loaded will be used by ZAPI CDEF updation tool.
            my $obj = NACL::APISet::Schema::Cdef::ZapiCdef->new(
                file    => $file . '.xml',
                command => $command,
                zapi_command_str  => $zapi_command_node->toString(),
            );
            $self->set_command_definition($command => $obj);
        }
    }

    if ($nacl_db) {
        $nacl_db->_insert_table(
            table_name => 'zapi_cdefs',
            row         => {
                path => $path,
                ds   => $nacl_apiset_zapi_cdef_hash{$path},
            },
        );
    }
    $Log->exit() if $may_exit;
} ## end sub load

#-----------------------------------------------------------------------------------------------------------------
# Method:    translate()
# Objective: Return ZAPI XML input command for the invoked ZAPI
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub translate {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    
    # Delete it so that we wont end up in translation of it
    my $cdef_method = delete $opts{cdef_method};

    my %hashOfScalar = ();
    my %hashOfArrays = ();
    my %hashOfHashes = ();
    my %hashOfArrayOfHashes = ();
    
    # Segregate the input options
    while (my ($key, $val) = each %opts) {
        my $ref = ref $val;
        my $trimmed_key = trim_string(string => $key);
        if (!$ref) {
            $hashOfScalar{$trimmed_key} = [$val];
        } elsif ($ref eq 'ARRAY' && !ref $val->[0]) {
            $hashOfArrays{$trimmed_key} = $val;
        } elsif ($ref eq 'HASH') {
            # Always embed in array so that we need can handle typedefs and
            # typedef array in a similar way
            $hashOfHashes{$trimmed_key} = [$val];
        } else {
            $hashOfArrayOfHashes{$trimmed_key} = $val;
        }
    }

    my $zapi_command_str = $self->get_command_definition(command => $cdef_method)
        ->zapi_command_str();
    my $cmd = $self->_expandXML(zapiCommandStr=> $zapi_command_str,
        hashOfArrays=>\%hashOfArrays, hashOfScalar=>\%hashOfScalar,
        hashOfHashes=>\%hashOfHashes,
        hashOfArrayOfHashes=>\%hashOfArrayOfHashes);
    
    $Log->exit() if $may_exit;
    return $cmd;
} ## end sub translate

##############################################################################
# Method:    get_command_definition
# Objective: To obtain the command definition object of a particular command
# Details:   See POD documentation at the end of this file.
##############################################################################
sub get_command_definition {
    my $self = shift @_;
    $Log->enter() if $may_enter;

	my $cdef_path = $self->{searched_cdef_path};
	
    my %args = NACL::APISet->validate_params(
        params => \@_,
        spec   => { "command" => { type => SCALAR, } }
    );

    my $cdef = $nacl_apiset_zapi_cdef_hash{$self->{path}}->{$args{command}};

    # Throw an exception if we don't have a command definition for the
    # given command.
    NACL::APISet::Exceptions::MethodNotFoundException->throw("Failed to get command definition for ".
        "command $args{command} (CDEF Path: ". $cdef_path .")" ) unless ( $cdef );

    $Log->exit() if $may_exit;
    return $cdef;
} ## end sub get_command_definition

#-----------------------------------------------------------------------------------------------------------------
# Method:    _encode()
# Objective: Encode all the special characters that can screw up the XML
# Details:   This method converts the special characters in the XML to a
#            form accepted by the XML processing libraries.
# @arg $value Required: string to be encoded for special characters
# Returns: Encoded string
#-----------------------------------------------------------------------------------------------------------------
sub _encode {
    my ($self, %args) = @_;
    my $value = $args{'value'};
    # '&' should be processed first
    if ($value) {
        $value =~ s/&/\&amp;/g;
        $value =~ s/</\&lt;/g;
        $value =~ s/>/\&gt;/g;
        $value =~ s/'/\&apos;/g;
        $value =~ s/"/\&quot;/g;
    }
    return $value;
} ## end sub _encode

#-----------------------------------------------------------------------------------------------------------------
# Method:        _expandxml()
# Objective: Expands a given zapinode with variables expanded in multilines.
# @arg $zapiCommandStr Required:  Xml string for the ZAPI
# @arg \%hashOfScalar Optional: reference to hash of scalar items
# @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
# @arg \%hashOfHashes Optional: reference to hash of hashes
# @arg \%hashOfArrayOfHashes Optional: reference to hash of arrays having hash items
# Returns:     Expanded zapinode on success.  On failue throws appropriate exception.
# Usage:
#              my %hashOfScalar =("key1"=>"value1", "key2"=>"value2");
#              my %hashOfArrays =("arg1key"=>["arg1value1", "arg1value2",..."arg1valuen"],....
#                         "argnkey"=>["argnvalue1", "argnvalue2",..."argnvaluen"]);
#              my %hashOfHashes =("key1"=>{"subkey1"=> "value1", "subkey2"=>"value2"},
#                                 "key2"=>{"subkey1"=> "value1", "subkey2"=>"value2");
#              my %hashOfArrayOfHashes =("key1"=>[{"subkey1"=> "value1", "subkey2"=>"value2"},
#                                 {"subkey1"=> "value1", "subkey2"=>"value2"],
#                                 key2=> [{...},...]);
#
#              $outputNode=_expandXML(zapinode => $zapiCommandNode,
#                                    hashOfScalar=>\%hashOfScalar,
#                                    hashOfArrays =>\%hashOfArrays,
#                                    hashOfHashes => \%hashOfHashes,
#                                    hashOfArrayOfHashes => \%hashOfArrayOfHashes);
#-----------------------------------------------------------------------------------------------------------------
sub _expandXML {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    my $xml = $opts{zapiCommandStr};
    # Expand scalar typedef
    if (scalar keys %{$opts{hashOfHashes}}) {
        $xml=$self->_expandTypedef ( xmlString=>$xml,
            hashOfTypedef=>$opts{hashOfHashes},
            attrToExpand => "typedef");
    }

    # Expand array typedef
    if (scalar keys %{$opts{hashOfArrayOfHashes}}) {
        $xml=$self->_expandTypedef ( xmlString=>$xml,
            hashOfTypedef=>$opts{hashOfArrayOfHashes},
            attrToExpand => "typedefarray");
    }

    # Expand scalars as arrays having one element
    if (scalar keys %{$opts{hashOfScalar}}) {
        $xml=$self->_expandArrays( xmlString=>$xml,
            hashOfArrays=>$opts{hashOfScalar},
            attrToExpand => "scalar");
    }

    # Expand arrays
    if (scalar keys %{$opts{hashOfArrays}}) {
        $xml=$self->_expandArrays ( xmlString=>$xml,
            hashOfArrays=>$opts{hashOfArrays},
            attrToExpand => "array");
    }

    # Cleanup unused inputs
    $xml = $self->_cleanup(xmlString => $xml);
    $Log->exit() if $may_exit;
    return $xml;
} ## end sub _expandXML

#-----------------------------------------------------------------------------------------------------------------
# Method:    _cleanup()
# Objective: All the unused tags for which inputs are not expanded will be
#            cleaned up.
# @arg $xmlString Required: Xml as a string
# Returns:   Xml string cleaned up for unused tags
#-----------------------------------------------------------------------------------------------------------------
sub _cleanup {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # # Inputs
    my $xml = $opts{xmlString};
    my $xpc =  $self->_get_xpathcontext(xmlString => $xml);;

    # Get the zapi command name from the xmlString
    $xml =~ /<([\w-]+?)>/;
    my $zapiCommand = $1;
    $zapiCommand =~ s:\s*/\s*:: if (defined $zapiCommand);

    # Remove the XML templates for which the inputs are not specified
    my @attributes = qw(typedef typedefarray scalar array);
    foreach my $attrToExpand (@attributes) {
        my @nodeSet = $xpc->findnodes('//@'.$attrToExpand);
        foreach my $attrNode (@nodeSet) {
            my $optionName = $attrNode->nodeValue();
            $optionName =~ s/^(\$|\@|\%)(.+)(\$|\@|\%)$/$2/;
            my @nodes = $xpc->findnodes("/$zapiCommand/$optionName");
            # There can be more than one matching nodes if the CDEF dir
            # is an updated one using CDEF updation tool.
            my $attrValue = $attrToExpand. '="'.$attrNode->getValue().'"';
            $attrValue = quotemeta($attrValue);
            foreach my $n (@nodes) {
                if ($n->toString() =~ $attrValue) {
                    my $parent = $n->parentNode();
                    $parent->removeChild($n);
                }
            }
        }
    }

    # Return the XML string
    my @nodeSet = $xpc->findnodes('*');
    my $finalXml = $nodeSet[0]->toString;

    $Log->debug("XML template after cleanup:\n ". $finalXml);
    $Log->exit() if $may_exit;
    return $finalXml;
} ## end sub _cleanup

#-----------------------------------------------------------------------------------------------------------------
# Method:    _expandTypedef()
# Objective: Expands a given xml string with hashOfTypedef variables expanded in multilines
#            internaly called by _expandXML
# @arg $xmlString Required: Xml as a string
# @arg $attrToExpand Required: Attribute to expand. Possible values are typedef and typedefarray
# @arg \%hashOfTypedef Required: Reference to hash of arrays having typedef items
# Returns:   Expanded xmlstring
#-----------------------------------------------------------------------------------------------------------------
sub _expandTypedef {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # Inputs
    my $xml = $opts{xmlString};
    my %hashOfTypedef = %{$opts{hashOfTypedef}};
    my $attrToExpand = $opts{attrToExpand};

    # Get the zapi command name from the xmlString
    $xml =~ /<([\w-]+?)>/;
    my $zapiCommand = $1;
    $zapiCommand =~ s:\s*/\s*:: if (defined $zapiCommand);

    # XPath Element
    my $xpc = $self->_get_xpathcontext(xmlString => $xml);

    my @nodeSet;
    foreach my $option (keys %hashOfTypedef) {
        # Get the XML template for the required option
        my $xmlTemplate;
        @nodeSet = $xpc->findnodes("/$zapiCommand/$option");
        # There can be either 0 nodes (unrecognized input option) or 1.
        # It cannot be more than 1 as an option name of a specific data type
        # is unique and we'll always get one matching node if it is a
        # defined option in CDEF.
        my $invalidOption = 0;
        my $currentNode;
        if (scalar @nodeSet) {
            # Defined option. Now get the option definition corresponding
            # to $attrToExpand datatype. Say if a option's datatype is changed
            # from typedef to typedefarray, then there exists two definitions
            # for the same option. This will happen if the selected CDEF dir is
            # the one which has been updated by CDEF updation tool
            @nodeSet = $xpc->findnodes("/$zapiCommand/$option/child::*".
            '[@'.$attrToExpand.']');
            unless (scalar @nodeSet) {
                # Option exists but the data type is incorrect. For this, we
                # are throwing an exception instead of silently handling it
                # because we suspect it to be a coding error. Also throwing
                # a ResponseException later will not help users in realizing
                # the fact that the issue hails from the wrong input XML
                # resulted from invalid option value.
                NATE::Exceptions::Argument->throw(
                    "Invalid $attrToExpand input type is sent for '$option'");
            }
            # The above query for findnodes would have returned the XML from
            # the tag type tag. We have to obtain its parent for the complete
            # definition of that option.
            $currentNode = $nodeSet[0]->parentNode();
            $xmlTemplate =  $currentNode->toString();
        } else {
            # Undefined option.
            $invalidOption = 1;
            $xmlTemplate = "";
            # For undefined options, there won't be any XML template to
            # insert before it. So get the last node in the XML definition
            # and insert at the end.
            my @nodes = $xpc->findnodes('*');
            # To insert it at the end
            $currentNode = undef;
        }
        # Get the root zapi command node. Not using the nodeSet[0] to find the
        # parent node because the user might have given an unknown option also
        @nodeSet = $xpc->findnodes("/$zapiCommand");
        my $parent = $nodeSet[0];
        # Frame the input XML
        my $returnString = $self->_expandTypedefOption(
            xmlString => $xmlTemplate, typedefOption => $option,
            typedefValue => $hashOfTypedef{$option},
            attrToExpand => $attrToExpand
        );

        # Insert the constructed XML
        # This is to handle unknown typedef array inputs
        my @commands = (ref($returnString) eq "ARRAY")?
            @{$returnString}:($returnString);
        foreach my $returnXml (@commands) {
            my $tmpXP=  $self->_get_xpathcontext(xmlString => $returnXml);
            foreach my $tNode ($tmpXP->findnodes('*')){
                $parent->insertBefore($tNode, $currentNode);
            }
        }
        # Remove the XML template
        $parent->removeChild($currentNode) unless ($invalidOption);
    }

    # Return the XML string expanded for the specified attribute
    @nodeSet = $xpc->findnodes('*');
    my $finalXml = $nodeSet[0]->toString;
    
    $Log->debug("XML template after expanding for $attrToExpand:\n ".
        $finalXml);
    $Log->exit() if $may_exit;
    return $finalXml;
} ## end sub _expandTypedef

#-----------------------------------------------------------------------------------------------------------------
# Method:    _expandTypedefOption()
# Objective: Expands the specified typedef option for its element values and
#            internally called by _expandTypedef or recursively called by
#            _expandTypedefElement
# @arg $xmlString Required: Xml as a string
# @arg $attrToExpand Required: Attribute to expand. Possible values are typedef
#                              and typedefarray
# @arg $typedefOption Required: Name of the typedef option
# @arg \@typedefValue Required: Corresponding value for the typedef option. For
#      both typedef and typedefarray attributes, value should be passed as an
#      array reference.
# @arg $rec Optional: Recurring prefix for the multilevel typedef options
# Returns:   Expanded xmlstring for the specified typedefOption
#-----------------------------------------------------------------------------------------------------------------
sub _expandTypedefOption {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # Inputs
    my $xml = $opts{xmlString};
    my $arg = $opts{typedefOption};
    my @typedefValue = @{$opts{typedefValue}};
    my $attrToExpand = $opts{attrToExpand};
    my $rec = (defined $opts{rec})? $opts{rec} : "";

    my $xp =  $self->_get_xpathcontext(xmlString => $xml);
    $rec = $rec. "_" if($rec ne "");

    my $formXml = "";
    my @nodeSet = ();
    # Trying to perform findnodes on an XPath element created from empty
    # XML string will throw error.
    @nodeSet = $xp->findnodes("/$arg") if ($xml ne "");
    # There can be either 0 nodes (unrecognized input option) or 1
    # It cannot be more than 1 as option names are unique and we'll always
    # get one matching node.
    if (scalar @nodeSet == 0) {
        # User had specified an input which is not part of
        # CDEF template. Still construct it and pass it to the service

        # Encoding of special characters will automatically taken care by
        # XML::Simple
        my @commands = ();
        # For typedef array input type, if we return the XML as one single
        # string, then while adding it to XPath element, XPath will complain
        # about junk value because there is no root element to hold the entire
        # xml. So it should be returned as an array so that we can insert each
        # of the hash value in typedef array one by one.
        foreach my $hashValue (@{$opts{typedefValue}}) {
            my $hashToXml = $self->_perl2xml(
                perldata => {$arg => $hashValue},
                KeepRoot => 1,
                NoAttr   => 1,
                KeyAttr  => [],
            );
            push(@commands, $hashToXml);
        }
        $Log->exit() if $may_exit;
        return \@commands;
    }
    # Input option is defined
    my $xmlTemplate = $nodeSet[0]->toString();
    my $attribute = '%'.$rec.$arg. '%';
    my $search_str = "/$arg/*".'[@'.$attrToExpand."='".$attribute."'".']';
    @nodeSet = $xp->findnodes($search_str);
    if (scalar @nodeSet == 0) {
        # If no matching nodes found, then user had passed a wrong input data
        # type and hence throw an exception. For Eg: Passing typedef value for
        # a known scalar input argument
        $attribute =~ s/^\%(.+)\%$/$1/;
        $attribute =~ s/_/->/g;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Invalid $attrToExpand input type is sent for '$attribute'");
    }
    # There should be only one match as an option can be typedef of only
    # one type at this point.
    my $tag_type = $nodeSet[0]->toString();
    my $str = '<(.+)\s+'.$attrToExpand.'=.+>';
    $tag_type =~ $str;
    $tag_type = $1;
    # When option name and tag type name are same, then we need not put the
    # type tag for scalar typedefs
    my $is_required_type = ($arg ne $tag_type)?1:0;
    $formXml .= "\n<$arg>";
    # Form the XML for each of the hash value in typedef array
    foreach my $hash (@typedefValue) {
        $formXml .= "\n<$tag_type>" if($is_required_type);
        $formXml .= $self->_expandTypedefElement(xpathObj => $xp,
            typedefHashInput => $hash, rec => $rec.$arg, typedefOption => $arg);
        $formXml .= "\n</$tag_type>" if($is_required_type);
    }
    $formXml .= "\n</$arg>";

    $Log->exit() if $may_exit;
    return $formXml;
} ## end _expandTypedefOption

#-----------------------------------------------------------------------------------------------------------------
# Method:    _expandTypedefElement()
# Objective: Expands the specified typedef option for its element values and
#            internally called by _expandTypedefOption or recursively by itself
# @arg $xpathObj Required: The input to be expanded as a XPath object
# @arg $typedefOption Required: Name of the typedef option
# @arg \%typedefHashInput Required: Reference to hash value for typedef input
#      type or each of the individual hash element in the typedef array
# @arg $rec Required: Recurring prefix for the multilevel typedef options
# Returns:   Expanded xmlstring for the specified typedefOption
#-----------------------------------------------------------------------------------------------------------------
sub _expandTypedefElement {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # Inputs
    my $xp = $opts{xpathObj};
    my %input_hash = %{$opts{typedefHashInput}};
    my $rec = $opts{rec};
    my $option = $opts{typedefOption};

    my @nodeSet = $xp->findnodes("/$option");
    my $xml = $nodeSet[0]->toString();
    my $form_xml = '';

    foreach my $hash_key (keys %input_hash) {
        my @elementNode = $xp->findnodes("/$option/child::*/$hash_key");
        my $xmlTemplate = (scalar @elementNode)? $elementNode[0]->toString():"";
        my $option_in_cdef = $rec . "_".$hash_key;
        $form_xml .= "\n";
        if ( ! ref($input_hash{$hash_key})) {
            # Scalar input
            if (scalar @elementNode == 0) {
                # Unknown scalar input
                my $temp = $self->_encode(value => $input_hash{$hash_key});
                if($temp) {
                     $form_xml.= "<$hash_key>".$temp."</$hash_key>";
                } else {
                     $form_xml.= "<$hash_key>"."</$hash_key>";
                }
            } else {
                my $xmlOption;
                my $str = 'scalar="$'.$option_in_cdef.'$"';
                $str = quotemeta($str);
                # There might be many matching nodes if the option had
                # undergone a datatype change. Select the cdef node describing
                # the option to be scalar. If no definition exists, then user
                # had passed a wrong datatype, hence throw an exception.
                foreach my $node (@elementNode) {
                    if ($node->toString() =~ $str) {
                        $xmlOption = $node->toString();
                        last;
                    }
                }
                unless (defined $xmlOption) {
                    $Log->exit() if $may_exit;
                    $option_in_cdef =~ s/_/->/g;
                    NATE::Exceptions::Argument->throw(
                        "Invalid scalar input type is sent for '$option_in_cdef'");
                }
                # This is a known scalar input
                my $refCommands = $self->_expandArrayOption(
                    xmlString => $xmlOption,
                    arrayOption => $hash_key,
                    arrayValue  => [$input_hash{$hash_key}],
                    attrToExpand => "scalar",
                    optionInCdef => $option_in_cdef
                    );
                foreach my $command (@$refCommands) {
                    $form_xml .= $command. "\n";
                }
            }
        } elsif ((ref($input_hash{$hash_key}) eq "ARRAY") &&
            (!ref($input_hash{$hash_key}->[0]))) {
            # Array input
            if (scalar @elementNode == 0) {
                # Unknown array input
                foreach my $array_element (@{$input_hash{$hash_key}}) {
                    $form_xml.= "<$hash_key>".
                    $self->_encode(value =>$array_element)."</$hash_key>"."\n";
                }
            } else {
                my $xmlOption;
                my $str = 'array="@'.$option_in_cdef.'@"';
                $str = quotemeta($str);
                foreach my $node (@elementNode) {
                    if ($node->toString() =~ $str) {
                        $xmlOption = $node->toString();
                        last;
                    }
                }
                unless (defined $xmlOption) {
                    $Log->exit() if $may_exit;
                    $option_in_cdef =~ s/_/->/g;
                    NATE::Exceptions::Argument->throw(
                        "Invalid array input type is sent for '$option_in_cdef'");
                }

                # This is a known array input
                my $refCommands = $self->_expandArrayOption(
                    xmlString =>$xmlOption,
                    arrayOption => $hash_key,
                    arrayValue  => $input_hash{$hash_key},
                    attrToExpand => "array",
                    optionInCdef => $option_in_cdef);
                foreach my $command (@$refCommands) {
                    $form_xml .= $command. "\n";
                }
            }
        } elsif (ref($input_hash{$hash_key}) eq "HASH") {
            # Typedef input
            my @typedefNode = $xp->findnodes(
                "/$option/child::*/$hash_key/child::*".'[@typedef]');
            if ((scalar @typedefNode == 0) && (scalar @elementNode != 0)) {
                # The element node ($hash_key) exists but user had passed a
                # wrong datatype
                $Log->exit() if $may_exit;
                $option_in_cdef =~ s/_/->/g;
                NATE::Exceptions::Argument->throw(
                    "Invalid typedef input type is sent for '$option_in_cdef'");
            }
            # If the element node doesn't exist, then pass xmlString as "" so
            # that in the next recursive iteration, the XML would be constructed
            # for this unknown option. Else we would send the XML string
            # denoting the respective node
            my $xmlStr = (scalar @elementNode == 0)? "" :
                $typedefNode[0]->parentNode()->toString();
            my $ret = $self->_expandTypedefOption(
                xmlString => $xmlStr,
                typedefOption => $hash_key,
                typedefValue => [$input_hash{$hash_key}],
                attrToExpand => "typedef",
                rec => $rec);
            # We will get either array reference (incase of undefined inputs) or
            # a XML string (for defined typedefarray input). The reason for
            # this approach can be referred in _expandTypedefOption()
            $ret = join("\n", @$ret) if(ref($ret) eq "ARRAY");
            # $ret is a string now for both cases
            $form_xml .= $ret;
        } else {
            # Typedef Array input
            my @typedefArrayNode = $xp->findnodes(
                "/$option/child::*/$hash_key/child::*".'[@typedefarray]');
            if ((scalar @typedefArrayNode == 0) &&(scalar @elementNode != 0)) {
                # The element node ($hash_key) exists but user had passed a
                # wrong datatype
                $Log->exit() if $may_exit;
                $option_in_cdef =~ s/_/->/g;
                NATE::Exceptions::Argument->throw(
                    "Invalid typedefarray input type is sent for ".
                    "'$option_in_cdef'");
            }
            # If the element node doesn't exist, then pass xmlString as "" so
            # that in the next recursive iteration, the XML would be constructed
            # for this unknown option. Else we would send the XML string
            # denoting the respective node
            my $xmlStr = (scalar @elementNode == 0)? "" :
                $typedefArrayNode[0]->parentNode()->toString();
            my $ret = $self->_expandTypedefOption(
                xmlString => $xmlStr,
                typedefOption => $hash_key,
                typedefValue => $input_hash{$hash_key},
                attrToExpand => "typedefarray",
                rec => $rec);
            $ret = join("\n", @$ret) if(ref($ret) eq "ARRAY");
            $form_xml .= $ret;
        }
    }

    $Log->exit() if $may_exit;
    return $form_xml;
} ## end _expandTypedefElement

#-----------------------------------------------------------------------------------------------------------------
# Method:    _expandArrays()
# Objective: Expands a given xml string with hashOfArrays expanded in multi line
#            internally called by _expandXML
# @arg $xmlString Required: Xml as a string
# @arg $attrToExpand Required: Attribute to expand. Possible values are array and scalar
# @arg \%hashOfArrays Required: Reference to hash of arrays having scalar items
# Returns:   Expanded xmlstring
#-----------------------------------------------------------------------------------------------------------------
sub _expandArrays {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # Inputs
    my $xml = $opts{xmlString};
    my %hashOfArrays = %{$opts{hashOfArrays}};
    my $attrToExpand = $opts{attrToExpand};

    # Get the zapi command name from the xmlString
    $xml =~ /<([\w-]+)>/;
    my $zapiCommand = $1;
    $zapiCommand =~ s:\s*/\s*:: if (defined $zapiCommand);

    my $xpc =  $self->_get_xpathcontext(xmlString => $xml);
    my @nodeSet;
    my $char = ($attrToExpand eq "scalar")? '$': '@';
    foreach my $option (keys %hashOfArrays) {
        # Get the XML template for the required option
        my $xmlTemplate;
        @nodeSet = $xpc->findnodes("//$zapiCommand/$option");
        my $invalidOption = 0;
        my $currentNode;
        if (scalar @nodeSet) {
            # Defined option. Now get the option definition corresponding
            # to $attrToExpand datatype. We have used the regexp logic here
            # rather than direct findnodes() because we accept a scalar value
            # for simple scalar input and for a typedef having only one element
            # The XML template differs between these two and findnodes cannot
            # be used to account both patterns.
            my $str = $attrToExpand.'="'.$char.$option.$char.'"';
            $str = quotemeta($str);
            foreach my $node (@nodeSet) {
                if ($node->toString() =~ $str) {
                    $xmlTemplate = $node->toString();
                    $currentNode = $node;
                    last;
                }
            }
            unless (defined $xmlTemplate) {
                # Option exists but the data type passed by the user is incorrect.
                NATE::Exceptions::Argument->throw(
                    "Invalid $attrToExpand input type is sent for '$option'");
            }
        } else {
            # Undefined option.
            $invalidOption = 1;
            $xmlTemplate = "";
            # For undefined options, there won't be any XML template to
            # insert before it. So get the last node in the XML definition
            # and insert at the end.
            my @nodes = $xpc->findnodes('*');
            # To insert it at the end
            $currentNode = undef;
        }
        # Get the root zapi command node. Not using the nodeSet[0] to find the
        # parent node because the user might have given an unknown option also
        @nodeSet = $xpc->findnodes("/$zapiCommand");
        my $parent = $nodeSet[0];
        # Frame the input XML
        my $refCommand = $self->_expandArrayOption(
            xmlString => $xmlTemplate, arrayOption => $option,
            arrayValue => $hashOfArrays{$option},
            attrToExpand => $attrToExpand
        );
        
        # Insert the constructed XML
        foreach my $returnXml (@{$refCommand}) {
            my $tmpXP = $self->_get_xpathcontext(xmlString => $returnXml);
            foreach my $tNode ($tmpXP->findnodes('*')) {
                no warnings 'uninitialized';
                $parent->insertBefore($tNode, $currentNode);
            }
        }
        # Remove the XML template
        $parent->removeChild($currentNode) unless ($invalidOption);
    }

    # Return the XML string expanded for the specified attribute
    @nodeSet = $xpc->findnodes('*');
    my $finalXml = $nodeSet[0]->toString;

    $Log->debug("XML template after expanding for $attrToExpand:\n ".
        $finalXml);
    $Log->exit() if $may_exit;
    return $finalXml;
} ## end sub _expandArrays

#-----------------------------------------------------------------------------------------------------------------
# Method:    _expandArrayOption()
# Objective: Expands the specified array or scalar option for its element values
#            and internally called by _expandArray
# @arg $xmlString Required: Xml as a string
# @arg $attrToExpand Required: Attribute to expand. Possible values are array
#                              and scalar
# @arg $arrayOption Required: Name of the array option
# @arg \@arrayValue Required: Corresponding value for the array/scalar option. For
#      both array and scalar options, value should be passed as an
#      array reference.
# @arg $optionInCdef Optional: Name of the option as given in the value
#                              dereferencing part of CDEFs
# Returns:   Reference to an array having the expanded commands
#-----------------------------------------------------------------------------------------------------------------
sub _expandArrayOption {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;

    # Inputs
    my $xml = $opts{xmlString};
    my $arg = $opts{arrayOption};
    my @arrayValue = @{$opts{arrayValue}};
    my $attrToExpand = $opts{attrToExpand};
    my $optionInCdef = (defined ($opts{optionInCdef}))? $opts{optionInCdef}:
        $arg;

    # XPath Element
    my $node;
    my $xpc = $self->_get_xpathcontext(xmlString => $xml);

    my @commands;
    my @nodeSet = ();
    # Trying to perform findnodes on an XPath element created from empty
    # XML string will throw error.
    @nodeSet = $xpc->findnodes("/$arg") if ($xml ne "");
    # There can be either 0 nodes (unrecognized input option) or 1
    # It cannot be more than 1 as option names are unique and we'll always
    # get one matching node.
    my $invalidOption = (scalar @nodeSet)? 0:1;
    if ($invalidOption) {
        # User had specified an input which is not part of
        # CDEF template. Still construct it and pass it to the service
        foreach my $value (@arrayValue) {
            push (@commands, "<$arg>$value</$arg>");
        }
        $Log->exit() if $may_exit;
        return \@commands;
    }
    # Input option is defined
    my $xmlTemplate = $nodeSet[0]->toString();
    my $char = ($attrToExpand eq "scalar")? '$':'@';
    my $search_str = '[@'.$attrToExpand."='".$char.$optionInCdef.$char."'".']';
    my @attrNodes = $xpc->findnodes('//*'.$search_str);
    if (scalar @attrNodes == 0) {
        # If no matching nodes found, then user had passed a wrong input data
        # type and hence throw an exception. For Eg: Passing scalar value for
        # a known typedef input argument
        $Log->exit() if $may_exit;
        $optionInCdef =~ s/_/->/g;
        NATE::Exceptions::Argument->throw(
            "Invalid $attrToExpand input type is sent for '$optionInCdef'");
    } else {
        # There should be only one match as there cannot be duplicate
        # option names
        # For simple scalar and simple array (string[]), directly return the
        # commands array. No processing required. For other types of scalar
        # and array, we need to insert into the typedef structure properly
        # as in _expandArrays, we are just going to insert the expanded
        # command replacing the XML template.
        my @simpleNodes = $xpc->findnodes("/$arg".$search_str);
        my $insertRequired = (scalar @simpleNodes)? 0 : 1;
        my $currentNode=$attrNodes[0];
        foreach my $attrnode ($currentNode->attributes()) {
            $currentNode->removeAttribute($attrToExpand);
        }
        my $refCommands = $self->_generateCommands(
            command      => $currentNode->toString(),
            hashOfScalar => {
                $optionInCdef => \@arrayValue,
            },
        );
        @commands = @$refCommands;
        if ($insertRequired) {
            my $parent=$currentNode->parentNode();
            # Insert the constructed XML
            foreach my $returnXml (@commands) {
                my $tmpXP = $self->_get_xpathcontext(xmlString => $returnXml);
                foreach my $tNode ($tmpXP->findnodes('*')){
                    $parent->insertBefore($tNode, $currentNode);
                }
            }
            # Remove the template
            $parent->removeChild($currentNode);
            my $tmpXP = $self->_get_xpathcontext(xmlString => $parent->toString());
            # Get the commands array after insertion
            @nodeSet = $tmpXP->findnodes('*');
            @commands = ();
            foreach my $returnNode (@nodeSet) {
                push(@commands, $returnNode->toString());
            }
        }
    }

    $Log->exit() if $may_exit;
    return \@commands;
} ## end sub _expandArrayOption

#-------------------------------------------------------------------------------------------
# Method:    _generateCommands()
# Objective: Expands a scalar or array input option with the values specified.
# @arg $command Required: command to be expanded.
# @arg \%hashOfScalar Required: Hash of array values.
# Returns:   Reference to an array whose each element corresponds to the
#            expanded XML for each of the array element
#-------------------------------------------------------------------------------------------
sub _generateCommands {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    # Inputs
    my %hashOfScalar = %{$opts{hashOfScalar}};
    my $command = $opts{command};

    my @commands;
    # Iterate for every key in hashOfScalar
    foreach my $hKey (keys %hashOfScalar) {
        # For each of the input sent, expand the respective tag
        # with the corresponding values
        my $oldCmd = $command;  # Used to preserve the old value of $command
        foreach my $aVal (@{$hashOfScalar{$hKey}}) {
            $aVal = $self->_encode(value => $aVal);
            if ($command =~/\[(\$|\@)\s*$hKey\s*(\$|\@)\]/) {
                $command =~ s/\[(\$|\@)\s*$hKey\s*(\$|\@)\]/$aVal/g;
                push(@commands, $command);
                $command = $oldCmd;
            }
        }
    }
    $Log->exit() if $may_exit;
    return \@commands;
} ## end sub _generateCommands

#-----------------------------------------------------------------------------------------------------------------
# Method: getSimpleXML()
# Objective: Returns formatted xml string.
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub getSimpleXML {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    
    my $perldata = $self->_xml2perl(
        xmlString  => $opts{logxml},
        KeepRoot   => 1,
        ForceArray => 1,
        KeyAttr    => [],
    );
    
    my $output = $self->_perl2xml(
        perldata => $perldata,
        KeepRoot => 1,
        KeyAttr  => [],
        NoAttr   => 1
    ) ;

    $Log->exit() if $may_exit;
    return $output;
} ## end sub getSimpleXML

sub set_command_definition {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my ($cmd, $cdef) = validate_pos(@opts, {type => SCALAR},
        {type => OBJECT, isa => 'NACL::APISet::Schema::Cdef::ZapiCdef'});
    
    # Add the entry to the global cache.
    #if (exists $nacl_apiset_zapi_cdef_hash{$self->{path}}->{$cmd}) {
        # TODO (Madhav): I've discovered some duplicate entries for the same
        # ZAPIs. Should investigate why this is the case and remove the
        # unnecessary entries. Once that's done, this bit of code should
        # probably throw an exception.
    #    Tharn::logcomment($cmd);
    #}
    $nacl_apiset_zapi_cdef_hash{$self->{path}}->{$cmd} = $cdef;
    $Log->exit() if $may_exit;
}

1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 NAME

NACL::APISet::Schema::ZapiSchema - interface to get ZAPI command definition

=head1 Synopsis

use NACL::APISet::Schema::ZapiSchema;

=head1 Description

Provides a mechanism to load, retrieve, and translate schema of
ZAPI command definitions into actual XML for execution

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::CDEFNotFoundException

=item NACL::APISet::Exceptions::MethodNotFoundException

=item NACL::APISet::Exceptions::InvalidCDEFException

=item NACL::APISet::Exceptions::InvalidXMLException

=back

=head1 CONSTRUCTOR

=head2 new

Creates a ZapiSchema object for the given command.

=over

=item Synopsis

$cdef_obj = NACL::APISet::Schema::ZapiSchema->new();

=item Arguments

No input

=item Returns

A reference to the created ZapiSchema object.

=back

=head1 METHODS

=head2 load

Loads all the CDEF files present in the specified 'path' input into
the schema object itself.

=over

=item Synopsis

$ZapiSchema_obj->load(path=>$path);

=item Arguments

=over

=item path

(Required)
The path to CDEF files

=back

=item Returns

Nothing

=back

=head2 get_command_definition

This method is used to get the command definition object of a particular command.

=over

=item Synopsis

$Api_Set_Obj->get_command_definition('command' => "aggr-get-iter");

=item Arguments

=over

=item command

Required
The command for which the command definition is required.

=back

=item Returns

The command definition structure of the given command.
Example:
$VAR1 = {
            'object' => bless( do{\(my $o = bless( [
                                          undef,
                                          '31',
                                          346925,
                                          '',
                                          [
                                            bless( do{\(my $o = bless( [
                                                              ${$VAR1->{'object'}},
                                                              '0',
                                                              346930,
                                                              '
      '
                                                            ], 'XML::XPath::Node::TextImpl' ))}, 'XML::XPath::Node::Text' ),
                                            ..
											..
											..
                                            bless( do{\(my $o = bless( [
                                                              ${$VAR1->{'object'}},
                                                              '5',
                                                              346960,
                                                              '',
                                                              [
                                                                bless( do{\(my $o = bless( [
                                                                                  ${${$VAR1->{'object'}}->[4]->[5]},
                                                                                  '0',
                                                                                  346965,
                                                                                  '
      '
                                                                                ], 'XML::XPath::Node::TextImpl' ))}, 'XML::XPath::Node::Text' )
                                                              ],
                                                              'system-get-version',
                                                              []
                                                            ], 'XML::XPath::Node::ElementImpl' ))}, 'XML::XPath::Node::Element' ),
                                            bless( do{\(my $o = bless( [
                                                              ${$VAR1->{'object'}},
                                                              '6',
                                                              346970,
                                                              '
    '
                                                            ], 'XML::XPath::Node::TextImpl' ))}, 'XML::XPath::Node::Text' )
                                          ],
                                          'system-get-version',
                                          []
                                        ], 'XML::XPath::Node::ElementImpl' ))}, 'XML::XPath::Node::Element' ),
            'file' => 'system.xml'
          };

=back

=head2 translate

Translate the ZAPI CDEF into input XML to be invoked

=over

=item Synopsis

$command = $ZapiSchema_obj->translate(cdef_method=>volume_create,%opts);

=item Arguments

=over

=item cdef_method

(Required)
Zapi method name to be invoked

=item %opts

(Optional)
Input arguments to the ZAPI command as a hash

=back

=item Returns

Returns input xml on success orelse it will throw an appropriate exception

=back

=head2 getSimpleXML

Format the ZAPI input XML returned by translate()

=over

=item Synopsis

$xml_command = $ZapiSchema_obj->getSimpleXML(logxml=>$cmd);

=item Arguments

=over

=item logxml

(Required)
XML which has to be formatted

=back

=item Returns

Returns the formatted XML on success

=back

=head1 AUTHOR/MAINTAINER

=over

=item nsesha@netapp.com, Isshwarya.M@netapp.com, NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id$
 Last revised $Date$ by $Author$

=cut


1;

