#
# Copyright (c) 2011 NetApp Inc.
# All rights reserved.
#

package NACL::APISet::Schema::ZapiSchema::DBlade;

# Base class
use base qw(NACL::APISet::Schema::ZapiSchema);

# compiler directives
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);
use NACL::APISet::_Mixins::ZAPI qw(_xml2perl);

#-----------------------------------------------------------------------------------------------------------------
# Method:    translate()
# Objective: Return ZAPI XML input command for the invoked ZAPI
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub translate {
    my ($self, %opts) = @_;
    $Log->enter() if $may_enter;
    my $cdef_method = $opts{cdef_method};
    
    my $cmd = $self->SUPER::translate(%opts);

    my $printXML = $self->_xml2perl(xmlString => $cmd, KeepRoot => 1, KeyAttr=>[]);
    my $str = "";
    
    foreach my $key (keys %{$printXML->{$cdef_method}}) {
        my $option = $printXML->{$cdef_method}->{$key};
        if (ref($option) eq "HASH") {
            $str .= " $key='[";
            $self->_xmlToZsmcli(option => $option,
                        str => \$str,
                       );
            $str .= "]' ";
        } else {
            $str .= " $key=$option";
        }
    }
    $cmd ="zsmcli " . $cdef_method . " $str";
    
    $Log->exit() if $may_exit;
    return $cmd;
} ## end sub translate

#-----------------------------------------------------------------------------------------------------------------
# Method:    _xmlToZsmcli()
# Objective: Will convert ZAPI xml into CLI that is understood by zsmcli tool
# Details:   See POD documentation at the end of this file.
#-----------------------------------------------------------------------------------------------------------------
sub _xmlToZsmcli {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    my $option = $args{option};
    my $str =  $args{str};
    foreach my $key (keys %{$option}) {
        
        if (ref($option->{$key}) eq "ARRAY") {
            foreach my $ele (@{$option->{$key}}) {
                $$str .= " $key=[";
                foreach my $key1 (keys %{$ele}) {
                    if (ref($ele->{$key1}) eq "HASH") {
                        $$str .= "$key1=[";
                        $self->_xmlToZsmcli(option => $ele->{$key1}, str => $str);
                        $$str .="] ";
                    } else{
                        my $value = $ele->{$key1};
                        $$str .= "$key1=$value]";
                    }
                }
            }
        } elsif (ref($option->{$key}) eq "HASH") {
            $$str .= "$key=[";
            $self->_xmlToZsmcli(option => $option->{$key}, str => $str);
            $$str .= "] ";
        } else {
            $$str .= " $key=$option->{$key}";
        }
    }
    
    $Log->exit() if $may_exit;
}

#-----------------------------------------------------------------------------------------------------------------
# Method: getSimpleXML()
# Objective: Base class method getSimpleXML override. Will return whatever is the input.
#-----------------------------------------------------------------------------------------------------------------
sub getSimpleXML {
    my ($self, %opts) = @_;
    
    return $opts{logxml};
} ## end sub getSimpleXML

1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 NAME

NACL::APISet::Schema::ZapiSchema::DBlade - interface to get ZAPI command definition

=head1 Synopsis

use NACL::APISet::Schema::ZapiSchema::DBlade;

=head1 Description

Provides a mechanism to load, retrieve, and translate schema of
ZAPI command definitions into actual XML for execution

=head1 CONSTRUCTOR

=head2 new

Creates a DBlade object for the given command.

=over

=item Synopsis

$cdef_obj = NACL::APISet::Schema::ZapiSchema::DBlade->new();

=item Arguments

No input

=item Returns

A reference to the created DBlade object.

=back

=head1 METHODS

=head2 translate

Translate the ZAPI CDEF into input XML to be invoked

=over

=item Synopsis

$command = $DBlade_obj->translate(cdef_method=>volume_create,%opts);

=item Arguments

=over

=item cdef_method

(Required)
Zapi method name to be invoked

=item %opts

(Optional)
Input arguments to the ZAPI command as a hash

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item rawat@netapp.com, NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;

