#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for 7Mode Sessions
## @author olalde@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::7Mode;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

# Module import - APISets
use NACL::APISet::Response::CLI::7Mode;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;
use Time::HiRes qw(time);  # This module is for measuring time in high precision

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;
    
    $Log->debug( sub { Dumper(\%args) } );

    my $self = $class->SUPER::new(
        %args,
        additional_spec => { connid => { type => SCALAR, default => '7-exec' } }
    );

    $Log->exit() if $may_exit;
    return $self;
}

###############################################################################
# Method:    send_command
# Objective: Execute the command on a 7Mode filer and gather the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter() if $may_enter;

    my ( $self, %opts ) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { connectrec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );

    my %connectrec = %{ delete $args{connectrec} };
    $self->_get_scaled_timeout_value( timeout => \$connectrec{timeout} );
    my $parser_file = $args{parser_file};
    $Log->debug( sub { "Session/CLI/7mode:send_command: " . Dumper( \%args ) }
    );

    unless ( $args{method_name} ) {

        # Caller is execute_command, all the options passed
        # should be considered as connectrec options
        %connectrec = ( %connectrec, %args );
        $args{command} ||= $connectrec{command};
        delete $connectrec{command};
    }

    $Log->debug(
        sub {
            "Arguments that are being passed to execute(): "
              . "command = '"
              . $args{command} . "'\n"
              . Dumper( \%connectrec );
        }
    );

    my $time_start = time();

    my $output;
    if (   $args{command} !~ /wrfile -a/
        && exists $args{wrfile_content}
        && defined $args{wrfile_content} )
    {
        $self->{connection}->execute_start( command => $args{command} );
        $self->{connection}->send( $args{wrfile_content} );
        $self->{connection}->send_EOF();
        $output = $self->{connection}->execute_expect();
    }
    else {
        $output = $self->{connection}->execute(
            command => $args{command},
            %connectrec
        );
    }

    my $time_end = time();

    my $execution_time = $time_end - $time_start;

    my $execution_status;

    # If the actual command sent itself doesn't have the "result -l",
    # then execution status need not be found.
    if ( $args{command} !~ /result -l/ ) {
        $execution_status = "";
    }
    else {
        $execution_status = $self->_find_execution_status(
            "output_ptr" => \$output,
            "command"    => $args{'actual_command'}
        );
    }

    # for execute_raw_command and execute_command just return the raw output
    if ( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }

    # With a connection to 7mode shell, the context of the commands can
    # be node, vfiler or systemshell. If the context is node or vfiler
    # then response object should be of type 7Mode.
    # If it is systemshell context, then we would have created a session
    # of type Session/CLI/7Mode/Shell and hence from this session class,
    # the response object returned will always be of type 7Mode.

    my $response_object = NACL::APISet::Response::CLI::7Mode->new(
        output           => $output,
        actual_command   => $args{actual_command},
        command          => $args{command},
        cdef_path        => $args{cdef_path},
        method_name      => $args{method_name},
        execution_status => $execution_status,
        execution_time   => $execution_time,
        parser_file      => $parser_file
    );

    $Log->exit() if $may_exit;
    return $response_object;
}

##############################################################################
# Method:    _find_execution_status
# Objective: Returns the execution status of the command
# Details:   For 7Mode, the execution status is determined by appending a
#            "result -l" to the command invoked. Due to this the output will
#            have an extra line of the form "result=<value>". We need to
#            retrieve this value AND remove this line from the raw output to
#            ensure parsing is done correctly.
##############################################################################
sub _find_execution_status {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;
    my $output_ptr = $args{"output_ptr"};
    $$output_ptr =~ s/result=(-?\d+).*\n*//g;
    my $status = $1;
    if ( !defined($status) ) {

        # If status couldn't be obtained because of error in prefixing,
        # then leave it to the response class which will later throw
        # an appropriate exception. Hence set the execution status to
        # empty string here.
        my @error_list = (
            '[Vv]filer not found.+',
            '.+requires a multistore license',
            'Not in takeover mode',
            'controller failover is not licensed',
            'partner:.*',
            'command not supported.*',
            'result not found'
        );
        foreach my $error_msg (@error_list) {
            if ( $$output_ptr =~ /$error_msg/ ) {
                $status = "";
                last;
            }
        }

        # Remove the error message from the raw output. This is required incase
        # of direct connection to vfiler shell where "result -l" is not a
        # supported command.
        $$output_ptr =~
          s/result not found\.\s+Type \'\?\' for a list of commands//;
    }

    # If status is still not defined, then there is some problem in establishing
    # the connection itself.

    NACL::APISet::Exceptions::ConnectionFailedException->throw(
"Command output didn't contain a result status because of the failure in establishing the connection.\nCommand:$args{command}\nOutput: ${$output_ptr}"
    ) if ( !defined($status) );

    $Log->exit() if $may_exit;

    return $status;
}
1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::7Mode - 7Mode CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::7Mode;
 
=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::7Mode> object is used to establish a connection
and send commands for the 7Mode CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::7Mode>.  

=over 

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::7Mode->new( %args );

=item Arguments

=over 

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new 7Mode CLI Session class.

=back

=head1 METHODS

=head2 send_command

Send the command to the node.  The output of the command is returned in a NACL::APISet::Response::CLI::7Mode
object.

=over 

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over 

=item command

The text of the command to send.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

