#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for Armadillo sessions
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::Armadillo;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::Armadillo;
use Connectrec qw();

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper(@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    if ( ( defined $args{conntype} ) and ( not defined $args{connid} ) ) {
        NATE::Exceptions::Argument->throw(
            "Conntype provided but connid not provided");
    }

#This check is necessary because the bridge accepts only console and telnet connections
    if ( ( defined $args{conntype} ) or ( defined $args{connid} ) ) {
        unless (
            (   $args{conntype} =~ /^rconsole/ and $args{connid} =~ /^console/
            )
            || ( $args{connid} =~ /^telnet/ )
            )
        {
            NATE::Exceptions::Argument->throw(
                "Wrong value provided for conntype or connid. Possible combinations are\n(conntype = rconsole and connid = console) or (connid = telnet)"
            );
        }
    }    # Fill the member vars for this instance
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj => { isa => "Hostrec", optional => 1 },
            username => { type => SCALAR, optional => 1, default => "root" },
            password =>
                { type => SCALAR, optional => 1, password => "password" },
            conntype =>
                { type => SCALAR, optional => 1, default => "rconsole" },
            connid => { type => SCALAR, optional => 1, default => "console" },
            ipaddress  => { type => SCALAR,       optional => 1 },
            connection => { isa  => "Connectrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of Armadillo
    bless $self, $class;

    # Connect to the service
    $self->connect();

    # Prepare the service for use
    $self->prep_service();

    # And give the caller the referece to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method:    send_command
# Objective: Constructs the Armadillo response object.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub send_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->debug( "Session/CLI/Armadillo:send_command: " . Dumper( \%args ) );

    my %connectrec = %{delete $args{connectrec}};
    $self->_get_scaled_timeout_value(timeout => \$connectrec{timeout});
    my $parser_file = $args{parser_file};
    
    my $execution_status;

    my $time_start = time();
    my $output;
    $output = $self->{connection}->execute(
        command => $args{command},
        %connectrec
    );
    my $time_end = time();

    my $execution_time = $time_end - $time_start;

    if ( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }

    my $response_object = NACL::APISet::Response::CLI::Armadillo->new(
        output         => $output,
        command        => $args{command},
        cdef_path      => $args{cdef_path},
        method_name    => $args{method_name},
        execution_time => $execution_time,
        parser_file    => $parser_file
    );

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::Armadillo - Armadillo CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::Armadillo;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::Armadillo> object is used to establish a connection
and send commands for the Armadillo CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::Armadillo>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::Armadillo->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item connid

The connection ID (possible values- console,telnet)

=item conntype

Connection type with which to connect (possible values rconsole when connid is console)

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new Armadillo CLI Session class.

=back

=head1 METHODS

=head2 send_command

Send the command to the host.

=over

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over

=item command

The text of the command to send.

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Returns

Return the array reference containing the execution_time,execution_status,parser_file and output.

=back

=head2 send_command

Return the Armadillo response object

=over

=item Arguments

=over

=item command

The text of the command to send.

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Synopsis

$response = $session->send_command(command => "raw command string");

=item Returns

Returns a Armadillo response object

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
