#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for CMode Sessions used
## for sending Nodescoep commands
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::CMode::Nodescope;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI::CMode);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

# Module import - APISets
use NACL::APISet::Response::CLI::7Mode;
use NACL::GeneralUtils qw(_log_profiling_data);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;
use Time::HiRes qw(time); # This module is for measuring time in high precision

# APISet code, say hello to MethodMaker
use Class::MethodMaker [
    scalar => [
        { -default => 'local' },
        'node_name'
    ],
];

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);

# COnstructor
sub new {
    $Log->enter();

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;
    
    $Log->debug(sub { 'Arguments to ' . __PACKAGE__ . "->new are:\n"
        . Dumper(\%args) });
    
    #
    # The conntype parameter should be specified ONLY if connid is also
    # provided
    #
    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
        "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance

    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj            => { isa => "Hostrec", optional => 1 },
            username           => { type => SCALAR, optional => 1 },
            password           => { type => SCALAR, optional => 1 },
            context_definition => { type => HASHREF, optional => 1 },
            conntype           => { type => SCALAR, optional => 1 },
            ipaddress          => { type => SCALAR, optional => 1 },
            connid             => { type => SCALAR,
                                    optional => 1,
                                    default => "c-login" },
            connection         => { isa => "Connectrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of 7Mode
    bless $self, $class;

    # Connect to the service
    $self->connect();
    
    # Get the node name in whose context the nodescope commands needs to be 
    # executed
    if(defined($self->{context_definition}->{name})) {
        $self->node_name($self->{context_definition}->{name});
    } elsif(defined($self->{context_definition}->{schema_hostobj})) {
        $self->node_name($self->{context_definition}->{schema_hostobj}->hostname());
            
    }
    
    # And give the caller the referece to the bless object.
    $Log->exit();
    return $self;
}

###############################################################################
# Method:    send_command
# Objective: Execute the command and gather the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter();

    my ($self, %opts) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { connectrec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    
    my %connectrec = %{delete $args{connectrec}};
    my $parser_file = $args{parser_file};
    $Log->debug( sub { "Session/CLI/CMode/Nodescope:send_command: " . Dumper( \%args ) });

    # Surely "nodescope-node-name" won't clash with anything
    # in the future, right?
    my $node = delete $args{'nodescope-node-name'} ||
        $self->node_name();
    
    unless ($args{method_name}) {
        # Caller is execute_(raw)_command, all the options passed
        # should be considered as connectrec options
        %connectrec = ( %connectrec, %args );
    }

    my $time_start = time();
    
    my $command;
    my $temp_command = $args{command};
    if ( !$args{method_name} ) {
        $temp_command = delete $connectrec{command};
    }

    my ($output,$msg);
    
    $Log->debug( sub { "Arguments that are being passed to execute(): ". "command = '".
                $args{command}. "'\n". Dumper(\%connectrec) });
    
    # There are two cases in which we cannot invoke the command from the
    # NGSH, but need to actually log into the dblade shell. The first is
    # for interactive commands (i.e. ones which return a prompt that
    # needs answering). This change was added to FS and SN.1 (see burt 685060).
    # To answer prompts, a match_table value has to be provided, so this
    # is our cue for determining that this is an interactive command.
    # The second case is if the command itself has double quotes or a ?.
    # This is because at the NGSH we need to construct the command within
    # double quotes (example: -command "vol status"). A double quote in the
    # command itself would be wrongly understood to be the end of the command
    # (there's also no way to escape it). The question mark in the command
    # would end up getting read by the NGSH instead of being understood
    # to be a part of the Nodescope command.
    
    # For reference, the error message that shows up when we attempt to
    # run an interactive Nodescope command from NGSH is:
    # “<command you tried to run> is not allowed through RSH, use run local shell.”
    my $is_interactive = exists $connectrec{match_table};
    my $has_special_chrs = ($temp_command =~ /("|\?)/);
    
    my $try_count;
    if ($is_interactive || $has_special_chrs) {
        if ($is_interactive) {
            $msg = 'Since this is an interactive command, we need to log '
                   . 'into the dblade shell to execute the command (see '
                   . 'burt 685060 which prevents interactive Nodescope '
                   . 'commands from being run from the NGSH). ';
        } else {
            $msg = "Since the command contains either '\"' or '?' "
                   . 'we cannot execute the command from the NGSH but need '
                   . 'to log into the dblade shell to execute the command. ';
        } 
        $output = $self->_execute_in_nodescope( connectrec   => \%connectrec,
                                                node         => $node,
                                                temp_command => $temp_command,
                                                msg          => $msg,
                                                %args );
    } else {
        my $run_in_node = 'system node run -node ' . $node .' -command "';
        $command = $run_in_node . $temp_command. '"';
        $output = $self->{connection}->execute(command => $command,
                            %connectrec);
        # Refer Burt 808913
        if ($output =~ /This command must be executed from the nodeshell/) {
            $msg = "This command cannot be executed directly on "
                   . "ngshell. Need to drop down to Nodescope. Refer "
                   . "to burt 808913 for more details";
            $output = $self->_execute_in_nodescope( connectrec   => \%connectrec,
                                                    node         => $node,
                                                    temp_command => $temp_command,
                                                    msg          => $msg,
                                                    %args );
        }
    }
    my $time_end = time();
    
    my $execution_time = $time_end - $time_start;
    my $execution_status = $self->_find_execution_status("output_ptr" => \$output);
    
    # for  execute_command just return the raw output we obtained
    if(!$args{method_name}) {
        $Log->exit();
        return $output;
    }
    if ( Tharn::param('PROFILING_ENABLED') ) {
        my $cmd = $command || $temp_command;
        $self->_log_profiling_data(
            node           => $self->{connection}->hostp()->hostname(),
            start_time     => $time_start,
            end_time       => $time_end,
            elapsed_time   => $execution_time,
            command        => $cmd,
            interface      => 'cli'
        );
    }
    # Since this session class is to run Nodescope commands from CMode shell,
    # the repsonse object returned will always be of type Response/CLI/7Mode.

    my $response_object =  NACL::APISet::Response::CLI::7Mode->new(
            output            => $output,
            actual_command    => $args{actual_command},
            command           => $args{command},
            cdef_path         => $args{cdef_path},
            method_name       => $args{method_name},
            execution_status  => $execution_status,
            execution_time    => $execution_time,
            parser_file       => $parser_file   
        );
        
    $Log->exit();
    return $response_object;

}

sub _execute_in_nodescope {
    $Log->enter();
    my ($self) = shift;
    my %opts = @_;
    my ($done, $try_count, $output);
    my %connectrec = %{$opts{connectrec}};
    my $node = $opts{node};
    my $temp_command = $opts{temp_command};
    my $conn = $self->{connection};
    my $msg = $opts{msg};
        
    until ($done) {
        my $node_enter_output = $conn->execute(
        command => "system node run -node $node", %connectrec );

        # Only one user is allowed to log into the node shell at once.
        # We'd get an error like this if we attempt to log in while
        # another user is logged in:
        # Too many users logged in!  Please try again later.
        # If the command fails for this reason, then we try up-to 3
        # times (since it's possible for parallel processes to be
        # attempting to use the node shell at the same time).
        if ($node_enter_output =~ /Too\s+many\s+users\s+/i ) {
            # At some point we could probably make it possible for the
            # user to specify how many times they'd like to try, but
            # let's start with 3 attempts for now.
            if (++$try_count < 3) {
                Tharn::snooze(10);
                next;
            }

            # Remove the extra \n at the end.
            chomp $node_enter_output;
            my $error = $node_enter_output
                . 'Could not log into the dblade shell after '
                . "$try_count attempts.\n";
            $error .= $msg;

            # Once Nodescope commands are ported to NGSH (OneShell), then we
            # should probably update this to mention that using that is
            # another possibility.
            $error .= 'If there exists a ZAPI to achieve this functionality, '
                   . 'then it would be preferable to use that since it would '
                   . 'not suffer from this limitation.';
            NACL::APISet::Exceptions::ConnectionFailedException->throw($error);
       } else {
            $done = 1;
            $output = $conn->execute(command => $temp_command, %connectrec);
            $conn->execute(command => 'exit', %connectrec);
       }
    }
    return $output;
}

##############################################################################
# Method:    _find_execution_status
# Objective: Returns the execution status of the command
# Details:   For 7Mode, the execution status is determined by appending a
#            "result -l" to the command invoked. Due to this the output will
#            have an extra line of the form "result=<value>". We need to 
#            retrieve this value AND remove this line from the raw output to 
#            ensure parsing is done correctly.
##############################################################################
sub _find_execution_status {
    $Log->enter();

    my ( $self, %args ) = @_;
    my $output_ptr = $args{"output_ptr"};
    $$output_ptr =~ s/result=(\d+).*\n*//g;
    my $status = $1;
    if(!defined($status)) {
        # When giving an invalid node name(Nodescope),execution 
        # status will be undef and this will cause an Argument Exception to
        # be thrown but actually an exception for sending invalid value 
        # (InvalidParamValue) should have been thrown by the repsone class.
        # Hence setting the status to nothing
        $status = "";
    }
     
    $Log->exit();
    
    return $status;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::CMode::Nodescope - CMode CLI Session class for 
running Nodescope commands

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::CMode::Nodescope;
 
=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::CMode::Nodescope> object is used to establish 
a connection and send Nodescope commands from the CMode shell.

=head1 ATTRIBUTES

=head2 node_name

The name of the node to which commands will be issued. This value can
be obtained/manipulated through the accessor methods.

    # Get value stored in the object
    my $node_name = $session->node_name();
    
    # Set value stored in object to $name
    $session->node_name($name);
    
    # Reset to the default, i.e. "local"
    $session->node_reset();

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=back

=head1 METHODS

=head2 send_command

Send the nodescope command to the specified schema_host specified in 
"context_definition".If none is specified, then it will run the nodescope
commands in the context of local node.  
The output of the command is returned in a NACL::APISet::Response::CLI::7Mode
object.

=over 

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over 

=item command

The text of the command to send.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

