#
# Copyright (c) 2012-13 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for CMode Sessions used
## for sending Nodescop commands
## @author Vishwanath.Rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::CMode::NodescopeNGSH;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI::CMode);

use NACL::GeneralUtils qw(_log_profiling_data);
use NACL::APISet::Response::CLI::CMode;
use NACL::APISet::Response::CLI::7Mode;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;
use Time::HiRes qw(time); # This module is for measuring time in high precision

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);

###############################################################################
# Method:    send_command
# Objective: Execute the command and gather the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter();

    my ($self, %opts) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { connectrec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    
    my %connectrec = %{delete $args{connectrec}};
    my $parser_file = $args{parser_file};
    $Log->debug( sub { "Session/CLI/CMode/NodescopeNGSH:send_command: " . Dumper( \%args ) });
    
    my $command = $args{command};
    my $raw_output_is_like;
    
    # This code is to handle "options" command
    if (exists $args{method_name} and $args{method_name} =~ /options/) {
        $command =~ s/ node / -node /;
        $command =~ s/raw_output_is_like\s+(\w+)//;
    }
        
    $raw_output_is_like = delete $connectrec{raw_output_is_like};
    
    unless ($args{method_name}) {
        # Caller is execute_command, all the options passed
        # should be considered as connectrec options
        %connectrec = ( %connectrec, %args );
    }

    my $time_start = time();
    $command = $self->_cmode_to_7mode_command_conversion(command => $command);
    my $output;
    if ( !$args{method_name} ) {
        $command = delete $connectrec{command};
    }
    
    $Log->debug( sub { "Arguments that are being passed to execute(): ". "command = '".
                $args{command} . "'\n" . Dumper(\%connectrec) });
    
    $output = $self->{connection}->execute(command => $command,
                                           %connectrec);

    my $time_end = time();
    my $execution_time = $time_end - $time_start;
    
    # for execute_command just return the raw output we obtained
    if(!$args{method_name}) {
        $Log->exit();
        return $output;
    }
    if ( Tharn::param('PROFILING_ENABLED') ) {
        my $cmd = $command;
        $self->_log_profiling_data(
            node           => $self->{connection}->hostp()->hostname(),
            start_time     => $time_start,
            end_time       => $time_end,
            elapsed_time   => $execution_time,
            command        => $cmd,
            interface      => 'cli'
        );
    }
    # Since this session class is to run 7Mode commands from CMode shell,
    # the repsonse object returned will always be of type Response/CLI/7Mode.

    
    # CMode doesnt return execution status of the command.
    # execution_status => 0, is passed above just to get rid
    # of warnings/failure thrown by response class while validating params
    my $execution_status = 0;

    # If ported 7mode command is executed on ngsh, the following line will come
    # in output. Gettting rid of it.
    $output =~ s/This command will not support tab-completion and help for options//;
    
    my $resp_package = "NACL::APISet::Response::CLI::" . $raw_output_is_like;
    
    my $response_object =  $resp_package->new(
        output               => $output,
        actual_command       => $args{actual_command},
        command              => $args{command},
        cdef_path            => $args{cdef_path},
        method_name          => $args{method_name},
        execution_status     => $execution_status,
        execution_time       => $execution_time,
        parser_file          => $parser_file,
        output_compat_mirror => {},
        CLI_separator        => $self->{CLI_separator},            
    );
    
    # Setting execution_status to undef so that user doesn't have wrong idea that
    # the execution status is ZERO
    $response_object->{execution_status} = undef;
    
    $Log->exit();
    return $response_object;

}

sub _cmode_to_7mode_command_conversion {
    $Log->enter();
    my ($self,%opts) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { command => { type => SCALAR } },
    );
    my $command = $args{command};
    my %conversion = $self->_get_cmode_to_7mode_conversion_hash();
    foreach my $key (keys %conversion) {
        if ($command =~ /^$key/) {
            $command =~ s/$key/$conversion{$key}/;
        }
    }
    $Log->exit();
    
    return $command;
}

sub _get_cmode_to_7mode_conversion_hash {
    $Log->enter();
    $Log->exit();
 
    return ( "options" => "system node options",);
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::CMode::NodescopeNGSH - CMode CLI Session class for 
running NodescopeNGSH commands

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::CMode::NodescopeNGSH;
 
=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::CMode::NodescopeNGSH> object is used to establish 
a connection and send NodescopeNGSH commands from the CMode shell.

=head1 METHODS

=head2 send_command

Send the NodescopeNGSH command to the specified schema_host specified in 
"context_definition".If none is specified, then it will run the NodescopeNGSH
commands in the context of local node.  
The output of the command is returned in a NACL::APISet::Response::CLI::7Mode
object.

=over 

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over 

=item command

The text of the command to send.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Session/CLI/CMode/NodescopeNGSH.pm#1 $
 
=cut

