#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for vserver session
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::CMode::Vserver;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI::CMode);

# Module import - APISets
use NACL::APISet::Response::CLI::CMode;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter();
    $Log->debug( Dumper(@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;

    #
    # The conntype parameter should be specified ONLY if connid is also
    # provided
    #
    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
        "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance

    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            username => { type => SCALAR, optional => 1},
            password => { type => SCALAR, optional => 1 },
            conntype => { type => SCALAR, optional => 1 },
            ipaddress => { type => SCALAR, optional => 1 },
            connid => { type => SCALAR, optional => 1, default => "c-login" },
            CLI_separator =>
              { type => SCALAR, optional => 1, default => "X;X" },
            connection => { isa => "Connectrec", optional => 1 },
            hostobj => {isa => "Hostrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.


    # This tags the hash pointed to by $object as a
    # type of CMode/Vserver
    bless $self, $class;

    # Connect to the service
    $self->connect();

	#prep service
	$self->prep_service();
    # And give the caller the referece to the bless object.
    $Log->exit();
    return $self;
}

##############################################################################
# Method:    prep_service
# Objective: Prepare the service for commands
# Details:   This may be an initial stanza needed to establish a session or some
# up front command needed to prep the session, whatever.  In Cluster Mode
# this is the act of turning off prompts and requesting all fields.
##############################################################################
sub prep_service {
    $Log->enter();

    my ($self, %args) = @_;

    my $CLI_separator = $self->{CLI_separator};
    
    my $connection = $self->{"connection"};
    $connection->execute( command => "set -confirmations off" );
    $connection->execute(
       	command => "set -showallfields true -showseparator \"$CLI_separator\" "
        	. "; rows 0"
    );
    $Log->exit();
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::CMode::Vserver - CMode Vserver CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::CMode::Vserver;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::CMode::Vserver> object is used to establish a
connection to the vserver directly and send commands.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::CMode::Vserver>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::CMode::Vserver->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item prompt_username

Username with which to log into the vserver

=item prompt_password

Password with which to log into the vserver

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new CMode CLI Session class.

=back

=head1 METHODS

=head2 prep_service

After connecting, initialize the connection.  In the cluster mode CLI case, this means 
preparing the ngsh session the show all fields and separate fields in a predictable way.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

