#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for Maintenance Sessions
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::Maintenance;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::7Mode;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;
use Time::HiRes qw(time); # This module is for measuring time in high precision

#
# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper(@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;
    
    #
    # The conntype parameter should be specified ONLY if connid is also
    # provided
    #
    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
        "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance

    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj   => { isa => "Hostrec", optional => 1 },
            conntype => { type => SCALAR, optional => 1 },
            connid   => { type => SCALAR, optional => 1, default => "console" },
            connection => { isa => "Connectrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of Maintenance
    bless $self, $class;
    
    # Pass the Maintenance prompt as an additional parameter to connect() in 
    # Session/CLI.pm but only if the hostobj is defined (when user does not 
    # pass in Connectrec object directly).
    
    my %additional_parameters;
    
    if ( ref ($self->{hostobj}->{prompt_fw}) eq 'ARRAY'  && 
         ref($self->{hostobj}->{prompt_norm}) eq 'ARRAY' ) {
        %additional_parameters = ( prompts => 
                               [@{ $self->{hostobj}->{prompt_norm}} ,'^\*>\s']);
    }

    # Connect to the service
    
    $self->connect(%additional_parameters);
    
    # If the user has passed in a Connection object, then we have to manually 
    # add the Maint mode prompt also to the list of prompts in the Connectrec object.
    # We do this by extracting it from the Hostrec object within the Connectrec 
    # object. Adding a duplicate entry for this prompt is not an issue.
    # So we do this irrespective of how the Connectrec object was created.
    
    $self->{connection}->prompts($self->{connection}->prompts(), '^\*>\s');
    
    # Prepare the service for use
    $self->prep_service();

    # And give the caller the referece to the bless object.
    $Log->exit() if $may_exit;
    return $self;
}

###############################################################################
# Method:    send_command
# Objective: Execute the Maintenance command to the filer and gather the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter() if $may_enter;

    my ($self, %opts) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { connectrec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    
    my %connectrec = %{delete $args{connectrec}};
    $self->_get_scaled_timeout_value(timeout => \$connectrec{timeout});
    my $parser_file = $args{parser_file};
    $Log->debug( "Session/CLI/Maintenance:send_command: " . Dumper( \%args ) );

    unless ($args{method_name}) {
        # Caller is execute_command, all the options passed
        # should be considered as connectrec options
        %connectrec = ( %connectrec, %args );
        $args{command} ||= $connectrec{command};
        delete $connectrec{command};
    }
    
    $Log->debug("Arguments that are being passed to execute(): ". "command = '".
                $args{command}. "'\n". Dumper(\%connectrec));
                
    my $time_start = time();
    my $output = $self->{connection}->execute(command => $args{command},
                            %connectrec);
    my $time_end = time();
    
    my $execution_time = $time_end - $time_start;
    
    my $execution_status = "";

    # The output is always in the 7Mode format.
    # So we always return a 7Mode response object
    
    # for execute_raw_command and execute_command just return the raw output
    if(!$args{method_name}) {
        $Log->exit() if $may_exit;
        return $output;
    }

    my $response_object =  NACL::APISet::Response::CLI::7Mode->new(
            output            => $output,
            actual_command    => $args{actual_command},
            command           => $args{command},
            cdef_path         => $args{cdef_path},
            method_name       => $args{method_name},
            execution_status  => $execution_status,
            execution_time    => $execution_time,
            parser_file       => $parser_file   
        );
        
    $Log->exit() if $may_exit;
    return $response_object;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::Maintenance - Maintenance CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::Maintenance;
 
=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::Maintenance> object is used to establish a connection
and send commands for the Maintenance CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::Maintenance>.

=over 

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::Maintenance->new( %args );

=item Arguments

=over 

=item hostobj

The hostrec object of the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new Maintenance CLI Session class.

=back

=head1 METHODS

=head2 send_command

Send the command to the node.  The output of the command is returned in a NACL::APISet::Response::CLI::Maintenance
object.

=over 

=item Synopsis

$response = $session->send_command( command => "raw command string", %other_connection_overrides );

=item Arguments

=over 

=item command

The text of the command to send.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
