#
# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for ONTAPSelectDeploy Sessions
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::ONTAPSelectDeploy;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::ONTAPSelectDeploy;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper(@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    if ( ( defined $args{conntype} ) and ( not defined $args{connid} ) ) {
        NATE::Exceptions::Argument->throw(
            "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj  => { isa  => "Hostrec", optional => 1 },
            username => { type => SCALAR,    optional => 1 },
            password => { type => SCALAR,    optional => 1 },
            conntype => { type => SCALAR,    optional => 1 },
            connid => { type => SCALAR, optional => 1, default => "login" },
            ipaddress  => { type => SCALAR,       optional => 1 },
            connection => { isa  => "Connectrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of ONTAPSelectDeploy
    bless $self, $class;

    # Connect to the service
    $self->connect();

    # Prepare the service for use
    $self->prep_service();

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method:    process_command
# Objective: Execute the command to the host pass the results to send_command.
# Details:   See POD documentation at the end of this file.
###############################################################################

### Execute the command and gather the raw output
sub process_command {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $do_not_answer;    # for answering the password prompt
                          #this match table is used only during sudo
    my @password_match_table = (
        '.*[Pp]assword.*:\s?' => sub {
            return undef if $do_not_answer;
            $do_not_answer = 1;
            return $self->{connection}->{password};
        }
    ) if delete $args{answer_password};

    my %connectrec = %{ delete $args{connectrec} };
    $self->_get_scaled_timeout_value( timeout => \$connectrec{timeout} );
    my $parser_file = $args{parser_file};

    my $execution_status = 0;

    $connectrec{'match_table'} = [
        @password_match_table,
        exists( $connectrec{'match_table'} )
        ? @{ $connectrec{match_table} }
        : ()
    ];

    my $time_start = time();

    my $output = '';
    # If shell_type is "none", connection may be for flashray
    # ignore get_status for such case
    $connectrec{get_status} = \$execution_status if ( $self->{connection}->{shell_type} ne "none" );
    $output = $self->{connection}->execute(
        command          => $args{command},
        command_username => $args{command_username},
        %connectrec
    );

    my $time_end = time();

    my $execution_time = $time_end - $time_start;

    if ( !defined $execution_status ) {

        # Execution status will be undef only if there is some problem with
        # establishing the connection itself.
        NACL::APISet::Exceptions::ConnectionFailedException->throw($output);
    }

    $output =~ s/.*[Pp]assword.*//
        if $do_not_answer;    # if password was answered
    $output =~ s/__status__=(\d+)\s+//;

    if ( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }

    my @vars = ( $output, $execution_time, $execution_status, $parser_file );
    $Log->debug( Dumper( \@vars ) );
    $Log->exit() if $may_exit;
    return \@vars;
} ## end sub process_command

###############################################################################
# Method:    send_command
# Objective: Constructs the ONTAPSelectDeploy response object.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub send_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $result = $self->process_command(%args);
    $Log->debug( "Session/CLI/ONTAPSelectDeploy:send_command: " . Dumper( \%args ) );

    if ( !$args{method_name} ) {

        # when only raw output is returned
        $Log->exit() if $may_exit;
        return $result;
    }

    my $response_object = NACL::APISet::Response::CLI::ONTAPSelectDeploy->new(
        output           => $result->[0],
        actual_command   => $args{actual_command},
        command          => $args{command},
        cdef_path        => $args{cdef_path},
        method_name      => $args{method_name},
        execution_status => $result->[2],
        execution_time   => $result->[1],
        parser_file      => $result->[3]
    );

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::ONTAPSelectDeploy - ONTAPSelectDeploy CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::ONTAPSelectDeploy;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::ONTAPSelectDeploy> object is used to establish a connection
and send commands for the ONTAPSelectDeploy CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::ONTAPSelectDeploy>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::ONTAPSelectDeploy->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new ONTAPSelectDeploy CLI Session class.

=back

=head1 METHODS

=head2 process_command

Send the command to the host.

=over

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over

=item command

The text of the command to send.

=item command_username

For commands to be run as that user

=item answer_password

Boolean value indicating whether password prompt must be answered or not.

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Returns

Return the array reference  containing the execution_time,execution_status,parser_file and output.

=back

=head2 send_command

Return the ONTAPSelectDeploy response object

=over

=item Arguments

=over

=item command

The text of the command to send.

=item command_username

For commands to be run as that user

=item answer_password

Boolean value indicating whether password prompt must be answered or not.

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Synopsis

$response = $session->send_command(command => "raw command string");

=item Returns

Returns a ONTAPSelectDeploy response object

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
