# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Session/CLI/PowerShell.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for PowerShell Sessions
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::PowerShell;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::PowerShell;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException;

###############################################################################
## Method:    new
## Objective: Creates PowerShell session class object. This also handles the
##            arguments to be passed to Connectrec::new which determines how
##            the connection should be started
###############################################################################

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper( \@_ ) ) if ( $Log->may_debug() );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    if ( ( defined $args{conntype} ) and ( not defined $args{connid} ) ) {
        NATE::Exceptions::Argument->throw(
            "Conntype provided but connid is not provided");
    }

    # Fill the member vars for this instance
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj   => { isa  => "Hostrec", optional => 1 },
            username  => { type => SCALAR,    optional => 1 },
            password  => { type => SCALAR,    optional => 1 },
            ipaddress => { type => SCALAR,    optional => 1 },
            conntype  => { type => SCALAR,    optional => 1 },
            connid    => { type => SCALAR,    default  => "exec" },
        },
    );

    if ( $args{connection} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Specifying a connection object to PowerShell APISet is not "
                . "supported. Pass hostobj option instead" );
    }

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of PowerShell
    bless $self, $class;

    $self->connect_and_prep();

    # And give the caller the referece to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

sub connect_and_prep {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    # Overrides for initial_command
    my %overrides;
    if ( $self->{connid} eq 'login' ) {

        # initial command has to be overriden only for persist connections
        $overrides{initial_command} = "echo ps## && powershell -Command -";
    }

    # shell_type need to be set to 'none' for NATE not to start cmd.exe by
    # seeing this host is running Windows
    $overrides{shell_type} = "none";
    $overrides{prompts}    = ['^\s*ps##'];

    # Connect to the service
    $self->connect(%overrides);

    # Prepare the service
    $self->prep_service();

    $Log->exit() if $may_exit;
}

###############################################################################
## Method:    prep_service
## Objective: Prepare the service for use. Mimic prompts so that completion
##            of command execution can be understood
###############################################################################

sub prep_service {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $connection = $self->{connection};
    $Log->debug(
        sub { "Prompts are set to\n" . Dumper( $connection->prompts ) } );

    # NOTES:
    # In case of persistent connections, powershell is very quiet and
    # so we are generating the prompts synthetically.

    if ( $connection->is_persist() ) {

        # This will instruct the PowerShell not to log errors in stderr and
        # later we can dump the error using 'echo' will be sent through stdout
        # channel so that the issues of indeterminate order of output coming
        # from these two channels in case of 'rsh' is handled.
        $connection->execute( command =>
                '$ErrorActionPreference = "silentlycontinue"; echo ps##' );
        $self->{prefix} = '$error.clear();';
        $self->{suffix} = '; echo $error; echo ps##';
    } else {    # this is only for persay
        $self->{prefix} = 'echo \\n | powershell "';
        $self->{suffix} = '"';
    }
    $Log->debug( "Prefix is " . $self->{prefix} );
    $Log->debug( "Suffix is " . $self->{suffix} );
    $Log->exit() if $may_exit;
} ## end sub prep_service

###############################################################################
## Method:    process_command
## Objective: Execute the command adding the necessary prefix and suffix
##            which facilitate the execution of command in PowerShell. It
##            returns back raw output, execution time, execution status,
##            command executed (includes prefix and suffix added)
###############################################################################

sub process_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my %connectrec = %{ delete $args{connectrec} };
    $self->_get_scaled_timeout_value( timeout => \$connectrec{timeout} );
    my $execution_status;
    # We cannot expect NATE to get the execution status as we are
    # performing prefixing and suffixing on our own in case
    # of persay connections and NATE can fetch exe status only from
    # cmd.exe which will not work for persay.
    # Also exe status is obtained even for execute_raw_command calls
    # to find out if the session is  established properly for persay
    my $command = $args{command} || delete $connectrec{command};
    $command
        = $self->{prefix} 
        . $command
        . ' ; echo __status__=__$?__'
        . $self->{suffix};

    my $time_start = time();

    my $output = $self->{connection}->execute(
        %connectrec,
        command          => $command,
        command_username => $args{command_username},
    );
    my $time_end = time();

    my $execution_time = $time_end - $time_start;

    $output =~ s/__status__=__(True|False)__\s*//;
    $execution_status = $1;

    #strip command echo
    my $cmd = quotemeta($command);
    $output =~ s/^(\s+)//s;
    my $cmdline = qr/\A$cmd\r*\n\r*/;

    if ( $output =~ /$cmdline/ ) {
        $output =~ s/$cmdline//;
    }

    if ( !defined $execution_status ) {

        # Execution status will be undef only if there is some problem with
        # establishing the connection itself.
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ConnectionFailedException->throw($output);
    }
    if ( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }
    my @vars = ( $output, $execution_time, $execution_status, $command );
    $Log->debug( Dumper( \@vars ) );
    $Log->exit() if $may_exit;
    return \@vars;
} ## end sub process_command

###############################################################################
## Method:    send_command
## Objective: Invoke the command in PowerShell using process_command() and
##             returns back a response object which can be returned back to
##            the caller
###############################################################################

sub send_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $result = $self->process_command(%args);
    $Log->debug(
        sub { "Session/CLI/PowerShell:send_command: " . Dumper( \%args ) } );
    if ( !$args{method_name} ) {

        # when only raw output is returned
        $Log->exit() if $may_exit;
        return $result;
    }

    my $response_object = NACL::APISet::Response::CLI::PowerShell->new(
        output           => $result->[0],
        actual_command   => $args{actual_command},
        command          => $result->[3],
        cdef_path        => $args{cdef_path},
        method_name      => $args{method_name},
        execution_status => $result->[2],
        execution_time   => $result->[1],
    );

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::PowerShell - PowerShell CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::PowerShell;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::PowerShell> object is used to establish a connection
and send commands for the PowerShell CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::PowerShell>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::PowerShell->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item ipaddress

ip address of the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=back

=item Returns

A reference to a new PowerShell CLI Session class.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
