# $Id: //depot/prod/test/main/lib/NACL/APISet/Session/CLI/SMO.pm#6 $

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for SMO Sessions
## @author dl-nacl-dev@netapp.com Shilpi.Nagapal@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::SMO;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::SMO;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug(Dumper(@_));

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;

    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
                                   "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance
    my $self =
      NACL::APISet->validate_params(
            params => \%args,
            spec   => {
                hostobj  => {isa  => "Hostrec", optional => 1},
                username => {type => SCALAR,    optional => 1},
                password => {type => SCALAR,    optional => 1},
                conntype => {type => SCALAR, default => "login", optional => 1},
                connid     => {type => SCALAR,       optional => 1},
                ipaddress  => {type => SCALAR,       optional => 1},
                connection => {isa  => "Connectrec", optional => 1},},);

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of SMO
    my $type;
    my $hostrec = defined ($args{hostobj})? $args{hostobj} : $args{connection}->hostp();
    my $name    = $hostrec->hosttype();

    # Calling constructor of Windows and Linux depends upon the operating system
    # Since session handling remains same for hpux,aix,solaris and linux,
    #Calling Linux session constructor for the rest of OS.

    if ($name =~ m/(unix-solaris)|(unix-linux)|(unix-hpux)|(unix-aix)/) {
        eval "use base NACL::APISet::Session::CLI::Linux";
        $type = "NACL::APISet::Session::CLI::Linux";
    } elsif ($name =~ m/(windows)/) {
        eval "use base NACL::APISet::Session::CLI::Windows";
        $type = "NACL::APISet::Session::CLI::Windows";
    }
    $self = $type->new(%args);
    bless $self, $class;

    # And give the caller the referece to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

### Execute the command and gather the raw output

sub send_command {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    $Log->debug("Session/CLI/SMO:send_command: " . Dumper(\%args));
    
    my $result = $self->process_command(%args);
    
    if( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $result;
    }

    my $response_object =
      NACL::APISet::Response::CLI::SMO->new(output      => $result->[0],
                                            command     => $args{command},
                                            method_name => $args{method_name},
                                            execution_status => $result->[2],
                                            execution_time   => $result->[1],);

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::SMO - Linux CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::SMO;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::SMO> object is used to establish a connection
and send commands for the SMO CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::SMO>.

=over

=item Synopsis

$self->{session} = NACL::APISet::Session::CLI::SMO->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new SMO CLI Session class.

=back

=head1 METHODS

=head2 send_command

Send the command to the host.  The output of the command is returned in a NACL::APISet::Response::CLI::SMO
object.

=over

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=over

=item command

Raw command  to the host

=item Returns

Returns the SMO session object.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/main/lib/NACL/APISet/Session/CLI/SMO.pm

=cut

1;
