#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Class to hold all connection information for SPShell sessions
## @author gsrikant@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here
# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.
package NACL::APISet::Session::CLI::SPShell;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);
use NACL::APISet::Response::CLI::SPShell;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Exceptions imported
#
use NATE::Exceptions::Argument qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper(@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    #
    # The conntype parameter should be specified ONLY if connid is also
    # provided
    #
    if ( ( defined $args{conntype} ) and ( not defined $args{connid} ) ) {
        NATE::Exceptions::Argument->throw(
            "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance

    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            hostobj => { isa => "Hostrec", optional => 1 },
            username => { type => SCALAR, optional => 1, default => 'root' },
            password =>
                { type => SCALAR, optional => 1, default => 'superuser' },
            context_definition => { type => HASHREF,        optional => 1 },
            prompt_username    => { type => SCALAR | UNDEF, optional => 1 },
            prompt_password    => { type => SCALAR | UNDEF, optional => 1 },
            conntype           => { type => SCALAR,         optional => 1 },
            ipaddress          => { type => SCALAR,         optional => 1 },
            connid             => {
                type     => SCALAR,
                optional => 1,
                default  => 'sp_login',
            },
            connection => { isa => "Connectrec", optional => 1 },
        },
    );

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of SPShell
    bless $self, $class;
    
    #prepare the service for use
    $self->prep_service();
    
    # Connect to the service
    $self->connect();
            
    # And give the caller a reference to the blessed object.
    $Log->exit() if $may_exit;
    return $self;
}

###############################################################################
# Method:    prep_service
# Objective: Prepares the service to login to the SPShell
# Details:   See POD documentation at the end of this file.
###############################################################################
sub prep_service {
    $Log->enter() if $may_enter;
    my ($self) = shift;

    # If we are using connectrec, all the steps like setting the privilege level
    # to test and adding prompts need to be done in the test script itself.
    # Hence, we are just returning if the connection object is passed to the
    # APISet constructor.
    if ($self->{connection} ) {
        $Log->exit() if $may_exit;
        return;
    }

    # Since NATE does not support the SPShell prompts we need to handle this in
    # our code explicitly. Please refer burt577427.
    my $p      = $self->{hostobj}->prompt_norm();
    my @prompt = @{$p};
    push( @prompt, '^#\s*$' );
    $self->{hostobj}->prompt_norm( \@prompt );
    my $conn;
    if ( $self->{hostobj}->is_cmode() ) {
        $conn = $self->{hostobj}->connect( template => "c-login" );
        $conn->execute(
                  command     => "set test",
                  match_table => [ ("Do you want to continue.*\? ") => "y", ],
        );
        $conn->execute( command => "system service-processor test debugmode -node local" );

    } else {
        $conn = $self->{hostobj}->connect(template => "7-exec" );
        $conn->execute( command => "priv set test" );
        $conn->execute( command => "sp debugmode" );

    }
    $conn->destroy();

    $Log->exit() if $may_exit;
}

###############################################################################
# Method:    send_command
# Objective: Execute the command on a SP Shell and gather the raw output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter() if $may_enter;

    my ( $self, %opts ) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        => { connectrec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );

    my %connectrec = %{ delete $args{connectrec} };
    $self->_get_scaled_timeout_value(timeout => \$connectrec{timeout});
    my $parser_file = $args{parser_file};
    $Log->debug( sub { "Session/CLI/SP:send_command: " . Dumper( \%args ) } );

    unless ( $args{method_name} ) {

        # Caller is execute_command, all the options passed
        # should be considered as connectrec options
        %connectrec = ( %connectrec, %args );
        $args{command} ||= $connectrec{command};
        delete $connectrec{command};
    }

    $Log->debug(
        sub {
            "Arguments that are being passed to execute(): "
                . "command = '"
                . $args{command} . "'\n"
                . Dumper( \%connectrec );
        }
    );

    my $time_start = time();
    my $output     = $self->{connection}->execute(
        command => $args{command},
        %connectrec
    );
    my $time_end = time();

    # Find total Execution time
    my $execution_time = $time_end - $time_start;

    # for execute_raw_command and execute_command just return the raw output
    if ( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }

    my $response_object = NACL::APISet::Response::CLI::SPShell->new(
        output         => $output,
        actual_command => $args{actual_command},
        command        => $args{command},
        cdef_path      => $args{cdef_path},
        method_name    => $args{method_name},
        execution_time => $execution_time,
        parser_file    => $parser_file
    );

    $Log->exit() if $may_exit;
    return $response_object;
}

1;

##############################################################################
#                                    POD here
##############################################################################


=head1 NAME

NACL::APISet::Session::CLI::SPShell - SP shell CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::7Mode::Shell;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::SPShell> object is used to establish a
connection and send commands to the SP Shell.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::SPShell>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::SPShell->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item prompt_username

Username with which to log into the SPShell. This value is 'root' by default.

=item prompt_password

Username with which to log into the SPShell. This value is 'superuser' by default.

=item connid

The connection ID. The default value is 'sp_login'.

=item conntype

Connection type with which to connect.We don't recommend 
overriding default conntype until its necessary. sp_login template has libssh-persist as default conntype


=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new SP Shell CLI Session class.

=back

=head1 METHODS

=head2 prep_service

This routine sets the privilege-level to 'test' and runs 'sp debugmode' or run 
'run local sp debugmode' depending on 7Mode or CMode filer/vsim, which are 
required before going into the SPShell by establishing a new connection to the 
filer or vsim. Once these steps are completed, the connection is destroyed.

=head2 send_command

Send the command to the SP Shell.  The output of the command is returned in a NACL::APISet::Response::CLI::SPShell
object.

=over 

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over

=item command

The text of the command to send.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut


