#$Id: //depot/prod/test/main/lib/NACL/APISet/Session/CLI/SQL.pm 

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for SQL Sessions
## @author Shilpi.Nagpal@netapp.com,dl-nacl-dev@netapp.com
## @status shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::SQL;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::SQL;


use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug(Dumper(@_));

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;

    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
                                   "Conntype provided but connid not provided");
    }

    # Fill the member vars for this instance
    my $self =
      NACL::APISet->validate_params(
               params => \%args,
               spec   => {
                   hostobj  => {isa  => "Hostrec", optional => 1},
                   username => {type => SCALAR,    optional => 1},
                   password => {type => SCALAR,    optional => 1},
                   conntype => {type => SCALAR,    optional => 1},
                   connid => {type => SCALAR, optional => 1, default => "exec"},
                   ipaddress  => {type => SCALAR,       optional => 1},
                   connection => {isa  => "Connectrec", optional => 1},},);

    # Add more variables that are sub-class specific.
    # This tags the hash pointed to by $object as a
    # type of SQL
    bless $self, $class;
    my $path = Tharn::param("ORACLE_HOME");

    #Constructing the command line for exporting the ORACLE_HOME,ORACLE_SID and then executing
    #sqlplus command "sqlplus " /as sysdba". Currently "as sysdba" is hardcoded
    #and will be generalized in the future whenever the need arises for future references.

    if ($args{hostobj}) {
        my $hostrec = $args{hostobj};
        my $name    = $hostrec->hosttype();
        if ($name =~ m/(unix-solaris)|(unix-linux)|(unix-hpux)|(unix-aix)/) {
            $self->{cmdline} =
              NATE::Util::quote_argv_sh(
                                   "exec",
                                   "env",
                                   "ORACLE_HOME=" . Tharn::param("ORACLE_HOME"),
                                   "ORACLE_SID=" . Tharn::param("TARGET_SID"),
                                   "$path/bin/sqlplus",
                                   "/",
                                   "as",
                                   "sysdba");
        } else {
            $self->{cmdline} = "set "
              . NATE::Util::quote_argv_cmd(
                                   "ORACLE_HOME=" . Tharn::param("ORACLE_HOME"))
              . "& set "
              . NATE::Util::quote_argv_cmd(
                                     "oracle_sid=" . Tharn::param("TARGET_SID"))
              . "& "
              . NATE::Util::quote_argv_cmd("$path/bin/sqlplus")
              . " / as sysdba";
        } ## end else [ if ($name =~ ...

        my $cmdline = $self->{cmdline};
        $Log->debug("Value of command line is $cmdline");

        #Handling for persay connections.

        if (   ($args{connid} eq "exec")
            || ($args{conntype} =~ /^(rsh-socket|(rsh|(lib|)ssh|staf)-persay)$/)
          ) {
            $self->connect(initial_command => $cmdline,
                           prompts         => ['SQL>'],
                           shell_type      => "none");
        } else {
            $self->connect();
            $self->prep_service();
        }
    } else {
        $self->connect();
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method:    prep_service
# Objective: Prepare the service for use.
#            Makes the session ready with SQL prompt.
#
# Details:   See POD documentation at the end of this file.
################################################################################

sub prep_service {
    my ($self, %args) = @_;
    my $connection = $self->{"connection"};
    my $cmdline    = $self->{cmdline};
    $connection->prompts("SQL>");
    $connection->execute(command => "$cmdline");
}

###############################################################################
# Method:    send_command
# Objective:Executes the SQL command on host and returns the raw output.
#
# Details:   See POD documentation at the end of this file.
################################################################################

sub send_command {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    my %connectrec = %{delete $args{connectrec}};
    $self->_get_scaled_timeout_value(timeout => \$connectrec{timeout});
    delete $connectrec{answer_password};

    $Log->debug(  "Arguments that are being passed to execute(): "
                . "command = '"
                . $args{command} . "'\n"
                . Dumper(%connectrec)
               );

    my $time_start = time();
    my $output =
      $self->{connection}->execute(command => $args{command},
                                   %connectrec);

    my $time_end = time();
    my $execution_time = $time_end - $time_start;

    $Log->exit() if $may_exit;
    return $output unless ( $args{"method_name"} );
    
    #There is no concept of execution status in SQL shell

    my $response_object =
      NACL::APISet::Response::CLI::SQL->new(output         => $output,
                                            command        => $args{command},
                                            execution_time => $execution_time,);

    return $response_object;
} ## end sub send_command
1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::SQL - SQL Session

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::SQL;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::SQL> object is used to establish a connection
and send commands for the SQL CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::SQL>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::SQL->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item  test parameters

=over

=item ORACLE_HOME

location where oracle home resides

=item TARGET_SID

SID (service identifier) of the oracle database

=item Returns

A reference to a new SQL CLI Session class.

=back

=back

=head1 METHODS

=head2 prep_service

Prepares the service to use.Makes the SQL shell ready.

=over

=item Synopsis

$session->prep_service();

=item Arguments

None

=item Returns

Nothing


=back

=head2 send_command

Send the command to the node.  The output of the command is returned in a NACL::APISet::Response::CLI::SQL
object.

=over

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

None

=item Returns

returns the response object if return-response-object is set to true or else return raw output.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/main/lib/NACL/APISet/Session/CLI/SQL.pm#6 $ 

=cut

1;
