#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for Windows Sessions
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::CLI::Windows;

use strict;
use warnings;

use base qw(NACL::APISet::Session::CLI);

use NACL::APISet::Response::CLI::Windows;


use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Time::HiRes qw(time);
use Params::Validate qw(:all);
use Data::Dumper;

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);

sub new {
    $Log->enter() if $may_enter;
    $Log->debug(Dumper(@_));

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;

    if ((defined $args{conntype}) and (not defined $args{connid})) {
        NATE::Exceptions::Argument->throw(
                              "Conntype provided but connid " . "not provided");
    }

    # Fill the member vars for this instance
    my $self =
      NACL::APISet->validate_params(
              params => \%args,
              spec   => {
                  hostobj   => {isa  => "Hostrec", optional => 1},
                  username  => {type => SCALAR,    optional => 1},
                  password  => {type => SCALAR,    optional => 1},
                  ipaddress => {type => SCALAR,    optional => 1},
                  conntype  => {type => SCALAR,    optional => 1},
                  connid => {type => SCALAR, optional => 1, default => "login"},
                  connection => {isa => "Connectrec", optional => 1},},);

    # Add more variables that are sub-class specific.

    # This tags the hash pointed to by $object as a
    # type of Windows
    bless $self, $class;

    # Connect to the service
    $self->connect();
    #As per burt#983358, avoid rsh session to land on drives other than the C:
    $self->{connection}->execute( command => "C:" );
    # And give the caller the referece to the bless object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

### Execute the command and gather the raw output

sub process_command {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    my %connectrec = %{delete $args{connectrec}};
    $self->_get_scaled_timeout_value(timeout => \$connectrec{timeout});
    my $parser_file = $args{parser_file};

    if( !$args{method_name} ) {
        $args{command} = delete $connectrec{command};
        delete $connectrec{answer_password};
    }

    my $execution_status;

    my $time_start = time();

    my $output =
      $self->{connection}->execute(command          => $args{command},
                                   command_username => $args{command_username},
								   get_status => \$execution_status,
                                   %connectrec);

    my $time_end = time();
    my $execution_time = $time_end - $time_start;

    if (!defined $execution_status) {

        # Execution status will be undef only if there is some problem with
        # establishing the connection itself.
        NACL::APISet::Exceptions::ConnectionFailedException->throw($output);
    }

    if( !$args{method_name} ) {
        $Log->exit() if $may_exit;
        return $output;
    }

    my @vars = ($output, $execution_time, $execution_status, $parser_file);
    $Log->debug(Dumper(\@vars));
    $Log->exit() if $may_exit;
    return \@vars;
} ## end sub process_command

sub send_command {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;

	my $result = $self->process_command(%args);

    if (!$args{method_name} ) {
        # when only raw output is returned
        $Log->exit() if $may_exit;
        return $result;
    }

    $Log->debug("Session/CLI/Windows:send_command: " . Dumper(\%args));
    my $response_object =
      NACL::APISet::Response::CLI::Windows->new(
                                        output         => $result->[0],
                                        actual_command => $args{actual_command},
                                        command        => $args{command},
                                        cdef_path      => $args{cdef_path},
                                        method_name    => $args{method_name},
                                        execution_status => $result->[2],
                                        execution_time   => $result->[1],
                                        parser_file      => $result->[3]);

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::CLI::Windows - Windows CLI Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::CLI::Windows;

=head1 DESCRIPTION

A C<NACL::APISet::Session::CLI::Windows> object is used to establish a connection
and send commands for the Windows CLI APISet.

=head1 EXCEPTIONS

=over

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::CLI::Windows>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::CLI::Windows->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource

=item username

Username with which to log into the resource

=item password

Password with which to log into the resource

=item ipaddress

ip address of the resource

=item connid

The connection ID

=item conntype

Connection type with which to connect

=item connection

connectrec object for an existing connection to the resource

=back

=item Returns

A reference to a new Windows CLI Session class.

=back

=head1 METHODS

=head2 prep_service

This method is used to take all the steps needed to prepare the service
for use. The base class method has been overriden here. We turn on delayed
environment variable expansion and to contruct the prefix and suffix portion
of each command that is to be executed depending on the connection.

=over

=item Synopsis

$session->prep_service();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 process_command

Sends the command to the host and returns the execution results.

=over

=item Synopsis

$response = $session->send_command( command => "raw command string" );

=item Arguments

=over

=item command

The text of the command to send

=item command_username

For commands to be run as that user

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Returns

Return the array reference  containing the execution_time,execution_status,parser_file and output.

=back


=head2 send_command

Collects the execution results and return Windows response object

=over

=item Arguments

=over

=item command

The text of the command to send.

=item command_username

For commands to be run as that user

=item parser_file

The parser file to be loaded for parsing the raw output

=back

=item Synopsis

$response = $session->send_command(command => "raw command string");

=item Returns

Returns a Windows response object

=back


=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
