# $Id$

## @summary Class to hold all selenium GUI connection information.
## @author madhav.shekar@netapp.com, sindhe@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::GUI::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

# Module Imports
use Data::Dumper;
#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NATE::Process;
use NATE::ProcManager;

#
# Library imports
#
use NACL::APISet::Node::GUI::CMode::SmVserver;

use base qw( NACL::APISet::Session::GUI
  NACL::APISet::Node::GUI::CMode::GUI_Utils
);

###############################################################################
# Method:    new
# Objective: Creates a new object with selenium connection.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub new {
    my ($self);
    my $class = shift;

    $Log->enter() if $may_enter;

    my %args = validate_with(
                    params => \@_,
                    spec   => {
                        remote_server_addr => { type => SCALAR },
                        systemIp           => { type => UNDEF|SCALAR, optional => 1 },
                        systemName         => { type => UNDEF|SCALAR, optional => 1 },
                        systemUsername     => { type => SCALAR, optional => 1 },
                        systemPassword     => { type => UNDEF|SCALAR, optional => 1 },
                        port               => { type => UNDEF|SCALAR, optional => 1,
                                                default => "4444"},
                        browser_url        => { type => UNDEF|SCALAR, optional => 1 },
                        browser_name       => { type => UNDEF|SCALAR, optional => 1,
                                                default => "firefox"}},
                    allow_extra => 1);

    $self = {};
    $Log->debug("Arguments Passed to "
         . "NACL::APISet::Session::GUI::CMode->new: "
         . Dumper(\%args));

    $Log->comment("Initializing Selenium Remote Driver");
    eval {
        $self = create_selenium_session(%args);
    };
    if ($@ =~ /RemoteConnection.pm|Selenium server did not return proper status/) {
        $Log->comment("Error recieved during initializing Selenium : $@");
        $Log->comment("Selenium server may not have been "
            . "configured on the client.\n"
            . "Installing client prequistes before retrying connection");
        # Client pre-requisites have to be installed.
        install_client_prereqs();
        $Log->comment("Client prequistes installed");
        eval {
            $Log->comment("Retrying : Initializing Selenium Remote Driver");
            $self = create_selenium_session(%args);
        };
        if ($@) {
            NACL::APISet::Exceptions::ConnectionFailedException->throw(
                "Failed to create Session Object : ". Dumper($@));
        }
    } elsif ($@) {
        NACL::APISet::Exceptions::ConnectionFailedException->throw(
            "Failed to create Session Object : " . Dumper($@));
    }

    bless $self, $class
      or NACL::APISet::Exceptions::ConnectionFailedException->throw(
           "Failed to create Session Object : Can't bless $class : $!");

    $self->{'systemIp'}       = $args{'systemIp'};
    $self->{'systemName'}     = $args{'systemName'};
    $self->{'systemUsername'} = $args{'systemUsername'};
    $self->{'systemPassword'} = $args{'systemPassword'};
    $self->{'apiset_cli'}     = $args{'apiset_cli'};

    if( $args{vserver} ) {
        $self->{'vserver'} = $args{vserver};
    }
    $self->{timeout} = 30;

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

sub create_selenium_session {
    my %args = @_;
    my $self = Selenium::Remote::Driver->new(
                remote_server_addr => $args{'remote_server_addr'},
                port               => $args{'port'},
                browser_name       => $args{'browser_name'} );

    $Log->comment("Opening Browser");
    $self->get($args{browser_url});
    return $self;
}

sub install_client_prereqs {

    # Creating a code spec to setup the client environment
    my $client_setup = 'sysmanager/Sysmgr_Client_Setup.thpl';

    # create a new NATE::Process for the reboot scenario
    my $proc = NATE::Process->new(
        codespec => $client_setup,
        runid    => "client_setup_$$",
        inherit  => 1,
        onexit   => \&NATE::Process::on_exit_propagate_worst_result
    );
    $proc->start;
    $proc->wait;
    $proc->destroy;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::GUI::CMode::RFullsteam - API Set Session sub class for CMode GUIs

=head1 SYNOPSIS

 use NACL::APISet::Session::GUI::CMode::RFullsteam;

=head1 DESCRIPTION

Creates a new GUI object with selenium connection.

=head1 EXCEPTIONS

None.

=head1 CONSTRUCTORS

=head2 new

Returns a object contaning all methods of inherited perl modules

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Session::GUI::CMode::RFullsteam->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new GUI CMode API Set.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development madhav.shekar@netapp.com, neelkant@netapp.com, dl-nacl-dev@netapp.com

=back

 $Id$
 Last revised $Date: 2012/04/05 $ by $Author: vikasp $

=cut
