#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all selenium GUI connection information.

## @author rohit4@netapp.com, dl-nacl-dev@netapp.com

## @status Review
## @pod here

package NACL::APISet::Session::GUI::SMTT::V1_4;

#
# Compiler Directives
#
use strict;
use warnings;

# Module Imports
use Data::Dumper;
use NACL::APISet::Host::GUI::SMTT::V1_4::GlobalConst;
use NACL::APISet::Host::GUI::SMTT::GConst;
use NACL::APISet::Host::GUI::SMTT::V1_4::Transition;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#
# Library imports
#
use base qw( NACL::APISet::Session::GUI::SMTT
  NACL::APISet::Host::GUI::SMTT::V1_4::SMTTUtils
);

###############################################################################
# Method:    new
# Objective: Creates a new object with selenium connection.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub new {
    my $self;
    my $class = shift;

    $Log->enter();

    my %args = validate_with(
        params => \@_,
        spec   => {
            remote_server_addr => {type => SCALAR, default => "localhost"},
            port               => {type => SCALAR, default => "4444"},
            browser_name       => {type => SCALAR, default => "firefox"},
            browser_url        => {type => SCALAR, default => "https://127.0.0.1:8443/transition/"}
        },
        allow_extra => 1
    );

    eval {
        $self = {};
        $Log->debug("Arguments Passed to "
             . "NACL::APISet::Session::GUI::SMTT::V1_4->new: "
             . Dumper(\%args));

        $Log->comment("Initializing Selenium.");
        $self = new Selenium::Remote::Driver(
            remote_server_addr => $args{'remote_server_addr'},
            port               => $args{'port'},
            browser_name       => $args{'browser_name'}
        );
        $self->{username}            = $args{'username'};
        $self->{password}            = $args{'password'};

        $self->debug_on if ($SEL_DEBUG_LOGS);
	$Log->comment("Opening Browser");
        my $browser_url = delete ($args{browser_url});
        $self->get($browser_url);

        $Log->comment("Waiting for page to load. Value is $GENERIC_WAIT ms");
        sleep $GENERIC_WAIT/1000;

        $self->window_maximize();
        $Log->comment("Window maximised.");

        bless $self, $class
          or NATE::BaseException->throw("Can't bless $class: $!");

        # In secs
        $self->{timeout} = 100;

        $self->_login();
    };
    if ($@) {
        $Log->comment("Failed to create Session Object : $@");
        NATE::BaseException->throw("$@");
    }

    $Log->exit();
    return $self;
} ## end sub new

###############################################################################
# Method:    DESTROY
# Objective: Destroy method called when the last reference to an object goes away
# Details:   This method takes a screenshot before destroying the object to capture
#              last state of the object
###############################################################################

sub DESTROY{
    my $self=shift;
    $Log->comment("Taking a base-64 encoded screenshot before destroying session "
        ."object.. Use freeformatter to decode"
    );
    $Log->comment($self->screenshot());
    my $handles = $self->get_window_handles;
    foreach (0..@{$handles}-1){
        $self->switch_to_window($handles->[$_]);
        $self->close;
    }
    $self->quit();
} ## end sub DESTROY

##############################################################################
# Method:    get_timeout
# Objective: Retrieve the current timeout value
# Details:   Used to retrieve the timeout value for the session
##############################################################################
sub get_timeout {
    my $self = shift;
    $Log->enter();
    $Log->exit();

    return $self->{timeout};
} ## end sub get_timeout

###############################################################################
# Method:    set_timeout
# Objective: Set the timeout to a new value
# Details:   Used to set the timeout value for the session
###############################################################################
sub set_timeout {
    my $self = shift;
    $Log->enter();
    my %args = validate_with(
        params => \@_,
        spec   => {
            'method-timeout' => {type => SCALAR, optional => 1},
            timeout          => {type => SCALAR, optional => 1},
        }
    );
    my $timeout;
    if (defined $args{'method-timeout'}) {
        $timeout = $args{'method-timeout'};
    } elsif (defined $args{timeout}) {
        $timeout = $args{timeout};
    } else {
        NATE::Exceptions::Argument->throw('Either of the parameters '
            . "'method-timeout' or 'timeout' should have been "
            . 'passed but neither were'
        );
    }
    $self->{timeout} = $timeout;

    # Type casting from string to int
    $timeout *= 1;

    # Setting the GUI selenium timeout to 5000 ms
    $self->set_implicit_wait_timeout(5000);

    $Log->debug('Timeout set to ' . $self->{timeout});
    $Log->exit();
} ## end sub set_timeout

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::GUI::SMTT::V1_4 - Class to hold session information for V1.4

=head1 SYNOPSIS

use NACL::APISet::Session::GUI::SMTT::V1_4;

=head1 DESCRIPTION

Contains session information for V1.3 version of 7MTT

=head1 METHODS

=head2 new

This method is used to create a NACL::APISet::Session::GUI::SMTT::V1_4 object.

=over

=item Synopsis

$output = $GuiObj->new(%args);;

=item Arguments

new function accepts all arguments.

=item Returns

(Scalar)
GUI object

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (rohit4@netapp.com, dl-nacl-dev@netapp.com)

=back

=cut
