#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all SOAP connection information
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::SOAP;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use NATE::SOAP;
use NATE::SOAP::Data;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate qw(:all);

#
# Exceptions imported
#
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::TimeoutException;

use base qw(NACL::APISet::Session);

use Class::MethodMaker [
    new     => ['-hash', '-init'],
    scalar  => ['port'],
    scalar  => ['hostname'],
    scalar  => ['username'],
    scalar  => ['password'],
    scalar  => ['timeout'],
    scalar  => ['url'],
    scalar  => ['base_path'],
    scalar  => ['use_async_calls'],
    scalar  => [ { -type => 'Hostrec' }, 'hostobj'],
    scalar  => [ { -type => 'NATE::SOAP' }, 'connection'],
];

sub new {
    my ($class, @args) = @_;

    my $self = {};
    bless $self, $class;

    $self->init(@args);

    return $self;
}


sub init {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;

    my %params = validate_with(
        params => \@opts,
        spec   => { additional_spec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    my $additional_spec = delete $params{additional_spec} if (defined($params{additional_spec}));

    my %args = validate_with(
        params => \%params,
        spec   => {
            hostname               => { type => SCALAR, optional => 1 },
            username               => { type => SCALAR, optional => 1 },
            password               => { type => SCALAR, optional => 1 },
            timeout                => { type => SCALAR, optional => 1 },
            default_timeout        => { type => SCALAR, optional => 1 },
            url                    => { type => SCALAR, optional => 1 },
            base_path              => { type => SCALAR, optional => 1 },
            hostobj                => { type => OBJECT | UNDEF, isa => 'Hostrec', optional => 1 },
            %$additional_spec,
        },
        allow_extra => 1
    );

    # Set default values to self
    $self->_set_args_to_self(%args);

    # Pull parameters from the hostrec object and values to self
    $self->_set_hostrec_values_to_self() if ($self->hostobj);

    # Connect to the service
    $self->connect();

    $Log->exit() if $may_exit;
} ## end sub init

###############################################################################
# Method:    connect
# Objective: Connect to the service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub connect {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my %connect_args;
    # Build the url if we have not overridden the base_path
    if ($self->url and !$self->base_path) {
       $connect_args{url}            = $self->url;
       $connect_args{timeout}        = $self->timeout;
    } else {
       $connect_args{hostname}       = $self->hostname;
       $connect_args{port}           = $self->port;
       $connect_args{timeout}        = $self->timeout;
       $connect_args{base_path}      = $self->base_path;
    }

    # Create the NATE::SOAP object connection
    my $nate_soap_conn = NATE::SOAP->new(%connect_args);

    $self->set_connection(soap_connection => $nate_soap_conn);

    $Log->exit() if $may_exit;
} ## end sub connect

###############################################################################
# Method:    send_command
# Objective: sends the soap command to the soap server.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub send_command {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;
    my $execution_status;
    my $execution_time  = time();
    my $hostname        = defined($self->hostname) ? $self->hostname : $self->hostobj->hostname();
    my $command         = $args{command};
    my $params          = $args{params};
    my $timeout         = $args{timeout};

    my $soap_response;
    try {
        $soap_response = $self->connection->call(command => $command,
                                                 params  => $params,
                                                 timeout => $timeout);
    } otherwise {
        my $e = shift;
        if ($e->text() =~ /500 read timeout/i) {
            NACL::APISet::Exceptions::TimeoutException->throw($e->text(),
                                                              timeout => $timeout,
                                                              output => $e->text());
        } else {
            $e->throw();
        }
    };
    $execution_time = time() - $execution_time;

    # Set Execution Status
    $execution_status = $soap_response->fault ? $soap_response->faultcode : 0;
    my $response_object = NACL::APISet::Response::SOAP->new(
        output           => $soap_response,
        command          => $command,
        execution_status => $execution_status,
        execution_time   => $execution_time,
    );

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

###############################################################################
# Method:    set_connection
# Objective: Set the Session object to internally hold the connection object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_connection {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    $self->connection($args{soap_connection});

    $Log->exit() if $may_exit;
} ## end sub set_connection

###############################################################################
# Method:    prep_service
# Objective: Prepare the service for use. We define it to default to not
#            do anything.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub prep_service {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    $Log->exit() if $may_exit;
} ## end sub prep_service

###############################################################################
# Method:    set_timeout
# Objective: Set the timeout to a new value
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_timeout {
    my $self = shift;
    $Log->enter() if $may_enter;

    my %args = validate_with(
        params => \@_,
        spec   => {
            timeout => { type => SCALAR, optional => 1 },
        }
    );
    my $timeout = $self->timeout;

    my $connection = $self->connection;

    $self->connection->connection_object->timeout($timeout);

    $Log->debug( 'Timeout set to ' . $self->{timeout} ) if ($Log->may_debug());
    $Log->exit() if $may_exit;
} ## end sub set_timeout

###############################################################################
# Method:    clear_timeout
# Objective: Set the timeout to the default value
# Details:   See POD documentation at the end of this file.
###############################################################################
sub clear_timeout {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $timeout = $self->default_timeout;

    my $connection = $self->connection;

    $self->connection->connection_object->timeout($timeout);

    $Log->debug( "Timeout has been reset to " . $timeout ) if ($Log->may_debug());

    $Log->exit() if $may_exit;
} ## end sub clear_timeout

###############################################################################
############################### Private Methods ###############################
###############################################################################

###############################################################################
# Method:    _create_nate_soap_data_object
# Objective: Constructs SOAP::Data objects using a hash of parameters passed to the call.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _create_nate_soap_data_object {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    my $nate_soap_data_object = NATE::SOAP::Data->new(%args);

    $Log->exit() if $may_exit;

    return $nate_soap_data_object;
}

###############################################################################
# Method:    _set_hostrec_values_to_self
# Objective: Copies the hostrec values to this object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _set_hostrec_values_to_self {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;
    my $connectrec = $self->hostobj->connect(template => 'soap');

    if ($self->hostobj) {
        $self->port($connectrec->connport);
        $self->hostname($connectrec->connhost);
        $self->username($connectrec->username);
        $self->password($connectrec->password);

        $self->url($connectrec->url);

        # Connectrec defaults both url and base_path
        # We will only update the 'base_path' attribute if
        # it differs from the Connectrec refault value.
        # This ensures that we attempt to build the url for session
        # connection only if we have specified an override for
        # the base_path, and that the override differs from the
        # Connectrec default.
        if (!$self->base_path) {
            $self->base_path($connectrec->base_path);
        }
    } else {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text   => "No host object has been defined. Cannot set " .
                                                                                 "the values of the host object onto the " .
                                                                                 "NACL::APISet::Session::SOAP object.",
                                                                       object => $self);
    }

    $Log->exit() if $may_exit;
} ## end sub _set_hostrec_values_to_self

###############################################################################
# Method:    _set_args_to_self
# Objective: Copies the key/value args to this object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _set_args_to_self {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    # Setting the args as object attributes
    foreach my $arg (keys(%args)) {
        if (defined($self->can("$arg"))) {
            $self->$arg($args{$arg}) if (defined $args{$arg});
        }
    }

    $Log->exit() if $may_exit;
} ## _set_args_to_self

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::SOAP - API Set Session base class for SOAP

=head1 SYNOPSIS

 use NACL::APISet::Session::SOAP;

=head1 DESCRIPTION

A C<NACL::APISet::Session::SOAP> contains data and methods common to all API Set
Session SOAP classes. This includes implementations for connect(), set_connection()
and prep_service().
Inherit from this class when creating new API Set Session classes
for SOAP. This class provides functionality common to all Session classes for SOAP.

=head1 EXCEPTIONS

=over

=item  NACL::APISet::Exceptions::MissingRequiredParamException;

=back

=head1 CONSTRUCTOR

None. This class is present only to provide common functionality, an object of
this class should never be instantiated. We should only instantiate objects
of Session sub-classes.

=head1 Methods

=head2 connect

This method is used to connect to the service. It does this by either using the
connection object provided, or by constructing a full connection spec and
connecting using this full connection spec.

If the connection cannot be established, a ConnectionFailedException is thrown.

=over

=item Synopsis

$self->{session}->connect();

=item Arguments

=over

=item %Additional_Connection_Parameters

A hash containing any additional arguments to Hostrec->connect()

=back

=item Returns

Nothing.

=back

=head2 set_connection

This method is used to set the Session object to internally hold the connection
object created.

=over

=item Synopsis

$self->{session}->set_connection(connection => $conn_object);

=item Arguments

=over

=item connection

This is the connection object created.

=back

=item Returns

Nothing.

=back

=head2 prep_service

This method is used to take all the steps needed to prepare the service
for use. It is defined here to do nothing, since for most services nothing
needs to be done. For any commands that do have preparatory steps, we should
override this method in the appropriate Session sub-class.

=over

=item Synopsis

$self->{session}->prep_service();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 clear_timeout

This method is used to set the timeout to the default value.
Default = Timeout value that was present when the connectrec object was created

=over

=item Synopsis

$timeout = $self->{session}->clear_timeout();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 set_timeout

This method is used to set the timeout to a new value.

=over

=item Synopsis

    $self->{session}->set_timeout(timeout => $new_timeout);

=item Arguments

=over

=item timeout

The new timeout value (in seconds)

=back

=item Returns

Nothing.

=back
