#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for VPserver Sessions
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::SOAP::VPserver;

use strict;
use warnings;

use base qw(NACL::APISet::Session::SOAP);

use NACL::APISet::Response::SOAP::VPserver;
use NATE::SOAP::Data;

use Time::HiRes qw(time);
use Params::Validate qw(:all);

use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ServiceException;
use NACL::APISet::Exceptions::InvalidParamValueException;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    scalar  => ['port'],
    scalar  => ['hostname'],
    scalar  => ['username'],
    scalar  => ['password'],
    scalar  => ['timeout'],
    scalar  => ['use_async_calls'],
    scalar  => ['execution_time'],
    scalar  => ['url'],
    scalar  => ['include_credential'],
    scalar  => [ { -type => 'Hostrec' }, 'hostobj'],
    scalar  => [ { -type => 'Connectrec' }, 'connectobj'],
    scalar  => [ { -type => 'NATE::SOAP' }, 'connection'],
    scalar  => [ { -type => 'NATE::SOAP::Data' }, 'credential'],
];

sub new {
    $Log->enter() if $may_enter;
    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    my $include_credential = delete $args{include_credential} // '1';
    my $self = $class->SUPER::new(%args);

    # handle any overrides that are vpserver specific
    $self->_handle_overrides();

    $self->include_credential($include_credential);

    # Create the SOAP Credential for the VPServer
    if (!$self->credential) {
        $self->_create_credential();
    }

    # Connect to the service
    $self->connect();

    # And give the caller the referece to the blessed object.
    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

###############################################################################
# Method:    send_command
# Objective: Constructs the VPserver response object.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub send_command {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    my $hostname        = defined($self->hostname) ? $self->hostname : $self->hostobj->hostname();
    my $command         = $args{command};
    my $params          = $args{params};
    my $timeout         = $args{timeout};
    my $credential      = $args{credential};

    if ($credential and !($credential->isa('NATE::SOAP::Data'))) {
        NACL::APISet::Exceptions::InvalidParamValueException->throw(text   => "The value of the 'credential' parameter " .
                                                                              "was not of type: NACL::SOAP::Data.",
                                                                    object => $self);
    }

    if ($self->include_credential) {
        if ($credential) {
            unshift(@{$params}, $credential);
        } else {
            unshift(@{$params}, $self->credential);
        }
    }

    my $soap_response_obj = $self->SUPER::send_command(command => $command,
                                                       params  => $params,
                                                       timeout => $timeout);
    my $response_object = NACL::APISet::Response::SOAP::VPserver->new(
        output           => $soap_response_obj->output,
        command          => $soap_response_obj->command,
        execution_status => $soap_response_obj->execution_status,
        execution_time   => $soap_response_obj->execution_time,
    );

    $Log->exit() if $may_exit;
    return $response_object;
} ## end sub send_command

###############################################################################
############################### Private Methods ###############################
###############################################################################

###############################################################################
# Method:    _create_credential
# Objective: Create the credential soap data object for the VPserver
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _create_credential {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    my $credential = $self->SUPER::_create_nate_soap_data_object('credential' => {'VPUsername' => $self->hostobj->default_username(),
                                                                                  'VPPassword' => $self->hostobj->default_password(),
                                                                                  'applicationName' => 'nacl_soap_vpserver',
                                                                                 }
                                                                );
    $self->credential($credential);

    $Log->exit() if $may_exit;
} ## end sub _create_credential

###############################################################################
# Method:    _handle_overrides
# Objective: Handle special object overrides for the VPserver session attributes
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _handle_overrides {
    $Log->enter() if $may_enter;

    my ( $self, %args) = @_;;
    my $connectrec = $self->hostobj->connect(template => 'soap');

    # Connectrec defaults both url and base_path
    # We will only update the 'base_path' attribute if
    # it differs from the Connectrec refault value.
    # This ensures that we attempt to build the url for session
    # connection only if we have specified an override for
    # the base_path, and that the override differs from the
    # Connectrec default.
    if ($connectrec->base_path ne 'services/vasa-public_latest') {
        $self->base_path($connectrec->base_path);
    }
}


1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::SOAP::VPserver - VPserver SOAP Session class

=head1 SYNOPSIS

 use NACL::APISet::Session::SOAP::VPserver;

=head1 DESCRIPTION

A C<NACL::APISet::Session::SOAP::VPserver> object is used to establish a connection
and send commands for the VPserver SOAP APISet.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ServiceException

=item NACL::APISet::Exceptions::InvalidParamValueException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::SOAP::VPserver>.

=over

=item Synopsis

 $self->{session} = NACL::APISet::Session::SOAP::VPserver->new( %args );

=item Arguments

=over

=item hostobj

The hostrec object of the resource.
(Optional)

=item username

Username with which to log into the resource.
(Optional)

=item password

Password with which to log into the resource.
(Optional)

=item port

The port for the connection.
(Optional)

=item timeout

The timeout for the connection.
(Optional)

=item use_async_calls

Boolean value to determine if calls for this session should be done asynchronously. Defaulted to: 0.
(Optional)

=item application_name

The name of the application you are running on the SOAP server.
(Optional)

=back

=item Returns

A reference to a new VPserver SOAP Session class.

=back

=head1 METHODS

=head2 send_command

Return the VPserver response object

=over

=item Arguments

=over

=item command

The text of the command to send.

=item params

(ArrayRef)
An array of parameters to be used for the specified command. These values can be an array of hashes, or an Array of NATE::SOAP::Data objects.

=item timeout

The timeout value for the command. Defaulted to 60.
(Optional)

=item credential

This is the credential object, of type NATE::SOAP::Data, that will override any preexisting credntial object for this method call.
(Optional)

=back

=item Synopsis

=item Example1 - Sending Command's with using an array of hash values as params

    my %convert_to_nate_soap_data = ("param1    " => {key1 => 'value1',
                                                      key2 => 'value2'
                                                     }
                                    );

    my $response = $session->send_command(command => "clusterList",
                                          params  => \%convert_to_nate_soap_data,
                                         );


=item Example2 - Sending Command's with using an array of hash values as params

    my @array_of_hash_params({"credential" => {VPPassword => 'netapp11',
                                               VPUsername => 'vpserver',
                                               applicationName => 'foo'
                                               }
                             },
                            );

    my $response = $session->send_command(command => "clusterList",
                                          params  => \@array_of_hash_params,
                                         );

=item Returns

Returns a VPserver response object

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
