# Copyright (c) 2014-* NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

## @summary Construction of Webapi Session Object to the specified REST server
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::Webapi;

# Compiler Directives
use strict;
use warnings;

# CPAN stuff
use Params::Validate qw(:all);
use Data::Dumper;
use JSON;

# NATE imports
use NATE::REST;
use NATE::Log qw(log_global);
my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

# Exceptions imported
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::ConnectionFailedException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

use NACL::APISet::Response::Webapi;

use base qw(NACL::APISet::Session);

sub new {
    $log->enter() if $may_enter;

    my ($class, @opts) = @_;
    my %args = validate_with(
        params => \@opts,
        spec   => { additional_spec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    my $additional_spec = delete $args{additional_spec};

    # Fill the member vars for this instance
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            username => { type => SCALAR, optional => 1 },
            password => { type => SCALAR, optional => 1 },
            ipaddress => { type => SCALAR, optional => 1 },
            connection => { type => OBJECT, optional => 1 },
            hostobj => { type => OBJECT, isa => 'Hostrec', optional => 0 },
            restobj => { type => OBJECT, isa => 'Hostrec', optional => 0 },
            %$additional_spec
        },
    );
    bless $self, $class;
    
    # Connect to the service
    $self->connect();

    # Prepare it for use - register the eseries controller
    $self->_register_controller();

    # Give the caller a reference to the blessed object
    $log->exit() if $may_exit;
    return $self;
}

###############################################################################
# Method:    connect
# Objective: Connect to REST server
# Details:   See POD documentation at the end of this file.
###############################################################################
sub connect {
    $log->enter() if $may_enter;

    # %args is usually empty. But the individual Session classes can send in
    # something that has to be passed to Hostrec->connect() additionally
    my ( $self, %args ) = @_;

    # connect to the REST server
    my $rest_host_obj = $self->{restobj};
    my $base = 'devmgr/v2';
    my $rest_obj = NATE::REST->new(
                        hostname => $rest_host_obj->default_ip,
                        username => $rest_host_obj->default_username,
                        password => $rest_host_obj->default_password,
                        secure => 0,
                        port => 8080,
                        timeout => 60,
                        base => $base,
                    );

    # save the connection details in object
    $self->set_connection(connection => $rest_obj);
    $log->exit() if $may_exit;
}

###############################################################################
# Method:    _register_controller
# Objective: Register the ESeries controller with REST server 
#            and set the system_id in the object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _register_controller {
    $log->enter() if $may_enter;
    my ( $self, %args ) = @_;

    my $eseries_host_obj = $self->{hostobj};
    my $eseries_default_ip = $eseries_host_obj->{default_ip};
    my $rest_host_obj = $self->{restobj};
    my $raw_payload;
    if ($eseries_host_obj->{part_host}) {
        my $eseries_part_host = $eseries_host_obj->{part_host};
        $log->comment("Register controller $eseries_default_ip and $eseries_part_host with REST server $rest_host_obj->{hostname}");
        $raw_payload->{controllerAddresses} = [$eseries_default_ip,$eseries_part_host];
    } else {
        $log->comment("Register controller $eseries_default_ip with REST server $rest_host_obj->{hostname}");
        $raw_payload->{controllerAddresses} = $eseries_default_ip;
    }
    if ($eseries_host_obj->{default_password} eq '') {
        $log->comment("Password not found for E-Series Array.");
    } else {
        $log->comment("Password found for E-Series Array.");
        $raw_payload->{password} = $eseries_host_obj->{default_password};
    }
    my $response = $self->send_command('url'=>'storage-systems', payload=>$raw_payload);

    my $output = $response->{complete_raw_output} || '';
    $log->debug( sub { "Raw output is:\n" . Dumper( $output ) } );
    $log->debug( sub { "Response content is:\n" . Dumper( $output->responseContent ) } );
    $log->debug( sub { "Response code is:\n" . Dumper( $output->responseCode ) } );

    my $resp_content = $output->responseContent();
    my $resp_code = $output->responseCode();
    if (($resp_code == 200) || ($resp_code == 201) || ($resp_code == 202) || ($resp_code == 203)|| ($resp_code == 204)) {
        $log->debug('response looks ok');
    } else {
        $log->comment("ERROR RESPONSE=$resp_code. CONTENT=".Dumper($resp_content));
        $log->exit() if $may_exit;
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => "failed with ERROR RESPONSE=$resp_code",
            response_object => $self
        );
    }

    # obtain the system id and store it in the APISet obj
    $log->debug("ID is : $response->{complete_parsed_output}{id}");
    my $system_id = $response->{complete_parsed_output}{id};
    $self->set_system_id(system_id => $system_id);
    $log->debug("Created Webapi obj . System ID is $system_id");
    $log->exit() if $may_exit;
}

###############################################################################
# Method:    set_system_id
# Objective: Set the system id
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_system_id {
    my ($self, @opts) = @_;
    $log->enter() if $may_enter;
    my %args = validate_with(
        params => \@opts,
        spec   => { system_id => { type => SCALAR, optional => 0 }, }
    );

    $self->{system_id} = $args{system_id};
    $log->debug("Set system_id to $args{system_id}");
    $log->exit() if $may_exit;
}

###############################################################################
# Method:    set_timeout
# Objective: Set the timeout to a new value
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_timeout {
    my ($self, @opts) = @_;
    $log->enter() if $may_enter;
    my %args = validate_with(
        params => \@opts,
        spec   => { timeout => { type => SCALAR, default => 60 }, }
    );

    $self->{connection}->setTimeout($args{timeout}); # fall down to the REST::Client
    $self->{timeout} = $args{timeout};
    $log->comment('Timeout set to '.$self->{timeout});
    $log->exit() if $may_exit;
}

###############################################################################
# Method:    set_connection
# Objective: Set the Session object to internally hold the connection object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_connection {
    $log->enter() if $may_enter;
     my ( $self, %args ) = @_;
    $self->{connection} = $args{connection};
    $log->exit() if $may_exit;
}


##############################################################################
# Method:    send_command
# Objective: Invoke the command on the Eseries host
# Details:   See POD documentation at the end of this file.
##############################################################################
sub send_command {
    $log->enter() if $may_enter;
    my $symbol = 0;
    my ($self, %opts) = @_;
    my %args = validate_with(
        params      => \%opts,
        spec        =>  {
                        payload => { type => SCALAR|HASHREF|ARRAYREF|UNDEF, optional => 1 },
                        delete => { type => SCALAR, default => 0, optional => 1 },
                        skip_encode => { type => SCALAR, default => 0, optional => 1 },
                        url => { type => SCALAR, optional => 1, default => 0 },
                        path => { type => SCALAR, optional => 1 },
                        cmd => { type => SCALAR, optional => 1 },
                        'object-id' => { type => SCALAR, optional => 1 },
                        query => { type => HASHREF|UNDEF, optional=>1 },
                        restapi => { type => SCALAR, optional=>1 }, # used for the raw
                        },
        allow_extra => 1
    );
    $log->debug("ARGS are ".Dumper(%args));

    # Determine endpoint - basically construct the URL
    # note : base = 'devmgr/v2'
    my $url = $args{url};
    if (!$url) {
        my $cmd = $args{cmd};
        if (!$cmd) {
            $log->comment("'cmd' param was empty");
            $log->exit() if $may_exit;
            NACL::APISet::Exceptions::MissingRequiredParamException->throw(
                                text => "'cmd' param was empty",
                                );
        }
        my $system_id = $self->{system_id};
        # if cmd does not have '-' or any uppercase then its a symbol endpoint
        # else v2 endpoint
        if ((lc($cmd) eq $cmd) || ($cmd =~ /-/)) {
            # v2 endpoint
            $url = 'storage-systems/'.$system_id;
            $url .= '/'.$args{path} if ($args{path});
            $url .= '/'.$cmd;
            $url .= '/'.$args{'object-id'} if ($args{'object-id'});
        } else {
            # symbol endpoint
            $url = 'storage-systems/'.$system_id .'/symbol/'.$cmd;
            $symbol=1;
        }
    }
    # strip off excess / in the URL - sometimes folks may have made a mistake
    $url =~ s/\/\//\//g;    # replace all '//' with '/'

    # decide REST API, if its a GET or POST or DELETE
    # and process raw payload
    my $raw_payload = $args{payload};
    my $restapi = $args{restapi};
    if (!$restapi) { 
        if ($args{delete}) {
            $restapi = 'DELETE';
        } else {
            if ($raw_payload || $symbol == 1) {
                $restapi = 'POST';
            } else {
                $restapi = 'GET';
            }
        }
    }
    my %rest_opts;
    if ($raw_payload) {
        if ($args{skip_encode}) {
            $log->debug('Skipping the encoding to JSON');
            $rest_opts{content} = $raw_payload;
        } else {
            $log->debug('RAW PAYLOAD '.Dumper($raw_payload));
            my $payload = encode_json($raw_payload);
            $log->debug('ENCODED JSON '.Dumper($payload));
            $rest_opts{content} = $payload;
        }
    }

    $rest_opts{query} = $args{query} if ($args{query});
    $rest_opts{headers} = {'Accept' => 'application/json',
                            'Content-Type' => 'application/json',
                            };
    $log->debug("Issue rest API: $restapi URL: $url");
    $rest_opts{url} = $url;
    # invoke the command on the connection made to the REST server
    my $time_start = time();
    my $rest_connection = $self->{connection};
    $log->debug('REST OPTS are '.Dumper(%rest_opts));
    my $output = $rest_connection->$restapi(%rest_opts);
    my $time_end = time();
    my $execution_time = $time_end - $time_start;
    $log->comment("Execution time=$execution_time");
    $log->debug('Got response '.Dumper($output));
    
    # handle the response
    my $resp = NACL::APISet::Response::Webapi->new(
                output         => $output,
                command        => $args{command},
                execution_time => $execution_time,
                actual_command => $args{command},
                );

    $log->exit() if $may_exit;
    return $resp;
}


###############################################################################
# Method:    execute_raw_command
# Objective: execute the raw REST API and return back processed output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute_raw_command {
    $log->enter() if $may_enter;
    my ($self, %args) = @_;

    if (!$args{url}) {
        my $text = "'url' param was empty";
        $log->comment($text);
        $log->exit() if $may_exit;
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => $text);
    };

    if ($args{restapi} !~ /^(GET|POST|DELETE)$/) {
        my $text = "Invalid restapi param value specified: '$args{restapi}'. ".
                "Supported restapi's are GET, POST, DELETE.";
        $log->comment($text);
        $log->exit() if $may_exit;
        NACL::APISet::Exceptions::InvalidParamValueException->throw(text => $text);
    }

    if ($args{restapi} =~ /POST/ && !$args{payload}) {
        my $text = "'payload' param is empty.". 
                   "Expecting it to be set when restapi='POST'";
        $log->comment($text);
        $log->exit() if $may_exit;
        NACL::APISet::Exceptions::MissingRequiredParamException->throw(text => $text);
    }
   
    my $result = $self->send_command(%args);

    $log->exit() if $may_exit;
    return $result;
}

=head1 NAME

NACL::APISet::Session::Webapi - Construction of Webapi session object connected
to the specified REST server

=head1 SYNOPSIS

 use NACL::APISet::Session::Webapi;

=head1 DESCRIPTION

A C<NACL::APISet::Session::Webapi> is used to construct the Webapi session object
connected to the specified REST server and include it as a value to the
APISet session connection parameter

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ConnectionFailedException

=back

=head1 Methods

=over

=back

=head2 connect

Connects to the REST server and saves the connection details in the object.

=over

=item Synopsis

$self->connect();

where $self is a NACL::APISet::Session::Webapi object

=item Arguments

Accepts all the arguments passed on to it from the APISet constructor.

=back

=head2 _register_controller

Will register the EF-Series contoller with the REST server, verify the response and
on success will extract the system's id and save it in the object.
If the registeration fails it will throw a ConnectionFailedException.

=over

=item Synopsis

$self->_register_controller();

where $self is a NACL::APISet::Session::Webapi object

=item Arguments

Accepts all the arguments passed on to it from the APISet constructor.

=back

=head2 set_system_id

This method is used to set the Session object to internally hold the system-id.

=over

=item Synopsis

$self->set_system_id(system_id => $system_id); 

=item Arguments

=item system_id

The system's id is obtained when the EF-Series controller is registered with the REST server.

=back

=head2 set_timeout

This method is used to set the timeout that is used.

=over

=item Synopsis

set_timeout(timeout=>60);

=item Arguments

=item timeout

Timeout in secs. Defaults to 60.

=back

=head2 set_connection

This method is used to set the Session object to internally hold the 
NATE::REST connection information.

=over

=item Synopsis

$self->set_connection(connection => $rest_obj);

=item Arguments

=item connection

A NATE::REST object.

=back

=head2 send_command

This method is used to send the Webapi command.
It will construct the endpooint URL and determine the REST API to be used.

=over

=item Synopsis

$self->send_command(...);

=item Arguments

=item payload

Input to the Webapi endpoint. Optional.

=item delete

If set, denotes a DELETE REST API. Optional.

=item skip_encode

In some cases the Webapi does not confirm to JSON. If set JSON encoding will be skipped. Optional.

=item url

The relative endpoint URL from base 'devmgr/v2'.

=item path

Directory path for 'v2' endpoints. Optional.

=item cmd

'symbol' or 'v2' Webapi endpoint command.

=item 'object-id'

Optional 'object-id' used in some Webapi endpoints.

=item query

Optional query to be added to endpoint URL.

=item restapi

'GET','POST', or 'DELETE'.

=back

=head2 execute_raw_command

Method to allow user to send specific GET, POST, DELETE with constructed URL.
This will return processed output.

=over

=item Synopsis

$self->execute_raw_command(...)

=item Arguments

Same as send_command method.

=back

=cut

1;
