#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base Session class for ZAPI APISet
## @author isshwar@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::ZAPI;

use strict;
use warnings;
use Params::Validate qw(:all);
use Data::Dumper;
use base qw(NACL::APISet::Session);
use Time::HiRes qw(time);
use NACL::GeneralUtils qw(_log_profiling_data);

#Importing Exceptions
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use NACL::APISet::Exceptions::TimeoutException qw(:try);
use NACL::APISet::Exceptions::InvalidXMLException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException qw(:try);

use NACL::APISet::_Mixins::ZAPI qw(_xml2perl _perl2xml);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NaServer;
require NACL::APISet;

# We define the default timeout value to be 150 seconds
use constant DEFAULT_TIMEOUT => 150;

# Default log level for command execution is 'comment'.
use constant DEFAULT_LOG_LEVEL => 'comment';

# When ZAPI execution fails, we log the output XML at comment level by default.
# This will help the users to debug the failures without having to enable
# trace level logs.
use constant DEFAULT_LOG_LEVEL_ON_ERROR => 'comment';

# Global Variables

use Class::MethodMaker [
    scalar => 'transport_type',
    scalar => 'port',
    scalar => 'cert_file',
    scalar => 'key_file',
    scalar => 'ca_cert_file',
    scalar => 'key_password',
    scalar => 'ssl_cert_verification',
    scalar => 'originator_id',
    scalar => 'hostname_verification',
    scalar => 'trace_threshold',
    scalar => 'server_cert_verify',
    scalar => 'target_cluster_uuid',
    scalar => 'target_vserver_name',
    scalar => 'target_vserver_uuid',
    scalar => [ 
        {   -default => 'NACL',
        },
        'app_header',
    ],
    scalar => 'zapi_log_level',
    scalar => 'zapi_log_level_on_error',
    scalar => 'zapi_header',
];

sub new {
    $Log->enter() if $may_enter;
    $Log->debug( Dumper(\@_) );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my $class = shift;

    # Fill the member vars for this instance

    my $self = NACL::APISet->validate_params(
        params => \@_,
        spec   => {
            hostobj    => { isa  => "Hostrec",    optional => 1 },
            ipaddress  => { type => SCALAR,       optional => 1 },
            username   => { type => SCALAR,       optional => 1 },
            password   => { type => SCALAR,       optional => 1 },
            connection => { isa  => "Connectrec", optional => 1 },
            set        => { type => SCALAR,       optional => 1 },
        },
    );
    if ( $self->{hostobj} ) {
        my $hostobj = $self->{hostobj};
        unless (defined $self->{ipaddress}) {
            $Log->trace("Obtained ipaddress from hostrec object");
            $self->{ipaddress} =  $hostobj->default_ip();
        }

        my $str = 'use Data::Validate::IP qw(is_ipv4 is_ipv6);';
        # Temporary fix for burt629110 
        eval $str;
        unless ($@) {
            # Fix for burt 602737
            # default_ip returns either the ipaddress or the hostid,
            # hence validating the ipaddress.    
            unless (is_ipv4($self->{ipaddress}) or is_ipv6($self->{ipaddress})) {
                $Log->warn( "Neither ipaddress is specified in the tharnhost file (or hostspec),".
                    " nor the host id could be resolved into ipaddress. ".
                    "Provide the ipaddress info in tharnhost file (or hostspec) explicitly. ".
                    "The ip address obtained is '$self->{ipaddress}'");     
            }
        }
        unless (defined $self->{username}) {
            $Log->trace("Obtained username from hostrec object");
            $self->{username}  = $hostobj->default_username();
        }
        unless ( defined $self->{password} ) {
            $Log->trace("Obtained password from hostrec object");
            $self->{password} = $hostobj->default_password();
        }
        $self->{hostid} = $hostobj->id();
    } else {
        my $connection = $self->{connection};
        $self->{ipaddress} = $connection->connhost();
        $self->{username}  = $connection->username();
        $self->{password}  = $connection->password();
        $self->{hostid}    = $connection->hostp()->id();
        $Log->trace('connection details are obtained from connectrec object');
    }
    
    bless $self, $class;

    # set the defaults
    $self->{"server_name"} = "";
    $self->clear_timeout();

    #Set test parameters
    $self->set_test_parameters();

    $Log->debug(sub {'ZAPI session object is '. Dumper ($self)});

    # And give the caller the reference to the bless object.				 
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
# Method:    set_server_type
# Objective: Set the current server type.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_server_type {
    $Log->enter() if $may_enter;

    my $self = shift();

    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "server-type" => { type => SCALAR } }
    );
    $self->{"server-type"} = $args{"server-type"};
    $Log->debug( "Server type in ZAPI is set to " . $self->{"server-type"} );
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    get_server_type
# Objective: Get the current server type
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_server_type {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"server-type"};
}

##############################################################################
# Method:    set_test_parameters
# Objective: Set the test parameters globally
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_test_parameters {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    
    require NATE::ParamSet;
    my $paramset = NATE::ParamSet->new(global => 1);
    
    my $partial_param = 'NACL_ZAPI_' . uc $self->{set} . '_';
    
    my @params_matching_method_name = (qw(TRANSPORT_TYPE TARGET_CLUSTER_UUID
        TARGET_VSERVER_NAME TARGET_VSERVER_UUID APP_HEADER PORT KEY_FILE KEY_PASSWORD CERT_FILE
        CA_CERT_FILE ORIGINATOR_ID TRACE_THRESHOLD SERVER_CERT_VERIFY ZAPI_HEADER)
    );
    my %params_list = (
        'USE_SSL_AUTH'                 => 'ssl_cert_verification',
        'ENABLE_HOSTNAME_VERIFICATION' => 'hostname_verification',
    );
    map {$params_list{$_} = lc $_} @params_matching_method_name;
    
    my $set_param = sub {
        my ($param, $method) = @_;
        my $param_value = $paramset->get($param);
        if (defined $param_value) {
            $self->$method($param_value);
            $Log->debug("Test parameter override provided: $param='$param_value'");
        }
    };
    
    while (my ($param, $method) = each %params_list) {
        my $full_param  = $partial_param . $param;
        $set_param->($full_param, $method);
    }
    
    # These are not of the form NACL_ZAPI_<SET>_<PARAM> (the <SET> section
    # is not applicable here), so list these separately.
    my %other_params = (
        'NACL_ZAPI_LOG_LEVEL'          => 'zapi_log_level',
        'NACL_ZAPI_LOG_LEVEL_ON_ERROR' => 'zapi_log_level_on_error',
    );
    while (my ($param, $method) = each %other_params) {
        $set_param->($param, $method);
    }

    # Read Log in DB parameter
    my $log_in_db = $partial_param . 'LOG_IN_DB';
    my $value = $paramset->get($log_in_db);

    if ( defined  $value) {
        unless (($value == 0) || ($value == 1)) {
            $Log->exit;
            NATE::Exceptions::Argument->throw("$log_in_db accepts either 0 or ".
                "1 but the value '$value' was specified");
        }
        # Set the value for LOG_ZAPI which decides whether logging into the NateDB
        # should be done or not
        # Ideally we should be using ParamSet->set() here, but the log_zapi
        # library is using Tharn::param(), which does not appear to be picking
        # up the value from ParamSet->set().
        Tharn::param("LOG_ZAPI", '-set', $value);
        
    }

    if ($paramset->get('PROFILING_ENABLED')) {
        $Log->debug(sub {"Profiling is enabled"});
        $self->{profiling_enabled} = 1;
    }
    $Log->exit() if $may_exit;

}

##############################################################################
# Method:    execute_raw_ZAPI
# Objective: Execute the raw ZAPI input XML sent by the user
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub execute_raw_ZAPI {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            command             => { type => SCALAR,         optional => 0 },
            'zapi-timeout'      => { type => SCALAR | UNDEF, optional => 1 },
            'ontapi-version'    => { type => SCALAR | UNDEF, optional => 1 },
            'server-name-override' =>
                { type => SCALAR | UNDEF, optional => 1 },
            'return-response-object' => { type => SCALAR, optional => 0 },
            'log_level'         => { type => SCALAR | UNDEF, optional => 1 },
            'input_log_level'   => { type => SCALAR | UNDEF, optional => 1 },
            'output_log_level'  => { type => SCALAR | UNDEF, optional => 1 },
        }
    );

    # Any overrides set in the APIset object which will not go as part of
    # input command like timeout, port no, transport type will
    # be considered for execute_raw_command as well if these overrides are
    # not explictly mentioned in the call to execute_raw_command.
    # All other overrides like vfiler name, vserver name, ontapi version
    # which are set in the headers of ZAPI input XML will not be considered
    # from APIset object for execute_raw_command. User should pass these
    # values explicitly in the call to execute_raw_command.

    my $server      = delete $args{'server-name-override'} || "";
    my $command     = $args{command};
    my $is_response = delete $args{'return-response-object'};

    my $output;
    $self->{server_name_override} = $server;
    $self->{ontapi_version_override} = delete $args{'ontapi-version'} || "";
    if ($is_response) {
        $output = $self->send_command(%args);
    } else {
        ( $output, undef, undef, undef ) = $self->executeZAPI(%args);
    }
    $Log->exit() if $may_exit;
    return $output;
}

##############################################################################
# Method:    executeZAPI
# Objective: Execute the input Zapi XML
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub executeZAPI {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => {
            command            => { type => SCALAR,         optional => 0 },
            'zapi-timeout'     => { type => SCALAR | UNDEF, optional => 1 },
            'log_level'        => { type => SCALAR | UNDEF, optional => 1 },
            'input_log_level'  => { type => SCALAR | UNDEF, optional => 1 },
            'output_log_level' => { type => SCALAR | UNDEF, optional => 1 },
            'target_cluster_uuid'  => { type => SCALAR | UNDEF, optional => 1 },
            'target_vserver_name'  => { type => SCALAR | UNDEF, optional => 1 },
            'target_vserver_uuid'  => { type => SCALAR | UNDEF, optional => 1 },
            'app_header'         => { type => SCALAR | UNDEF, optional => 1 },
	    'zapi_header'       => { type => HASHREF | UNDEF, optional => 1 },
        }
    );
    my $timeout_scaling_factor = $self->timeout_scaling_factor()||1.0;
    my $commandInput = $opts{command};
    my $username     = $self->{username};
    my $password     = $self->{password};
    my $Filer_ip     = $self->{ipaddress};
    my $timeout      = ( defined $opts{'zapi-timeout'} )
        ? $opts{'zapi-timeout'}*$timeout_scaling_factor   
        : $self->{timeout}*$timeout_scaling_factor;
    # Get params needed to enable SSL Authentication
    my $cert_file = $self->cert_file;
    my $ca_cert_file = $self->ca_cert_file;
    my $key_file = $self->key_file;
    my $use_ssl_cert_verification = $self->ssl_cert_verification ? 1 : 0;
    my $enable_hostname_verification = $self->hostname_verification;
    my $key_passwd = $self->key_password;
    # Verify required params is passed
    unless ( defined $self->{ipaddress} ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($self->{ipaddress} . " cannot be undefined");
    }

    # Username and password is needed if ssl_cert_verification is disabled
    if ($use_ssl_cert_verification == 0) {
        if (not (defined $self->{username} and defined $self->{password})) {
            NATE::Exceptions::Argument->throw("If SSL based verification is not being requested " .
                                              "then both fields username and password must be defined");
        }
    }

    my $NaServerObj;
    my $major_default = 1;
    my $minor_default = 0;
    eval {

        # Override gets the precedence
        my $server_context
            = ( defined $self->{server_name_override} )
            ? $self->{server_name_override}
            : $self->get_vfiler_or_vserver();
        my $v = $self->{ontapi_version_override};

       # Immediately undef the override field so that even if some exceptions
       # are thrown, the effect of overrides wont affect the subsequent calls.
        $self->{server_name_override}    = undef;
        $self->{ontapi_version_override} = undef;

        if ( length($server_context) ) {

            # To set either vfiler context (for 7Mode) or vserver context
            # (for CMode).  Vfiler tunneling requires version 1.7 or above
            # So set the major and minor version to 1 and 7 respectively
            $major_default = 1;
            $minor_default = 7;
        }

        # Now set the major and minor version overrides specified by the user.
        # This will override the defaults (1.0 normally and 1.7 for vfiler
        # context)
        my ( $major, $minor );
        if ( defined $v ) {

            # execute_raw_command overrides. Per method overrides are not
            # supported for method calls as the need is not observed.
            $v = $v . ".0" if ( $v !~ /\./ );

            # If $v is just an empty string, then $major and $minor will be
            # assigned undef and that will end up executing ZAPI with the
            # default ontapi version as expected.
            ( $major, $minor ) = ( $v =~ /(\d+)\.(\d+)/ );
        } else {

            # For APIset method calls - ontapi version can be set only using
            # accessor methods.
            $major = $self->{major_version};
            $minor = $self->{minor_version};
        }
        if ( ( defined $major ) && ( defined $minor ) ) {
            $Log->debug("setting the ontapi version to $major.$minor");
            $NaServerObj
                = new NaServer( $Filer_ip,
                $major, $minor );
        } else {
            $Log->debug(
                "setting the ontapi version to $major_default.$minor_default"
            );
            $NaServerObj
                = new NaServer( $Filer_ip,
                $major_default, $minor_default );
            if ($use_ssl_cert_verification == 1) {
                $NaServerObj->set_style("CERTIFICATE");
            }
        }

        # Set the transport type
        if ( defined( $self->{transport_type} ) ) {
            $NaServerObj->set_transport_type( uc( $self->{transport_type} ) );
        }
        my $transport_type = $NaServerObj->get_transport_type();
        
        my $comment_str = "Connection details: ipaddress='$self->{ipaddress}', hostname='$self->{hostid}'"
            . ", transport_type='$transport_type'";
        if ($use_ssl_cert_verification == 1) {

            # Enable/disable hostname cert verification
            $NaServerObj->set_hostname_verification($enable_hostname_verification);

            # Set Client certificate and key
            $NaServerObj->set_client_cert_and_key($cert_file, $key_file, $key_passwd);

            # Set Autority certificates
            $NaServerObj->set_ca_certs($ca_cert_file);
            $Log->comment( $comment_str );
        } else {
            $NaServerObj->set_admin_user( $username, $password );
            $comment_str = $comment_str . ", username='$self->{username}', password='$self->{password}'";
            $comment_str = $comment_str . ", execution_context='$server_context'" if (length $server_context);
            $Log->comment( $comment_str );
        }

        # Set originator ID
        if (defined($self->originator_id)) {
            $NaServerObj->set_originator_id($self->originator_id);
        }

        # Set trace threshold
        if (defined $self->trace_threshold) {
            $NaServerObj->set_trace_threshold($self->trace_threshold);
        }

        if (defined $self->trace_threshold) {
            $NaServerObj->set_trace_threshold($self->trace_threshold);
        }

        if (defined $self->server_cert_verify()) {
            $Log->debug(
                sub {
                    "Server cert verify set to " . $self->server_cert_verify();
                }
            );
            $NaServerObj->set_server_cert_verification(
                $self->server_cert_verify());
        }

        $Log->debug(
            "setting the server type to" . uc( $self->get_server_type() ) );
        $NaServerObj->set_server_type( uc( $self->get_server_type() ) );

       # Set the port no. This should be invoked after setting the server_type
       # This value should override the default port no set by set_server_type
        $NaServerObj->set_port( $self->{port} ) if ( defined $self->{port} );
        $Log->debug( "Port is set to" . uc( $NaServerObj->get_port() ) );

        $NaServerObj->set_vfiler($server_context)
            if ( length($server_context) );
                
        # Set the target cluster uuid
        my $target_cluster_uuid = $opts{target_cluster_uuid} || $self->target_cluster_uuid;
        $NaServerObj->set_target_cluster_uuid($target_cluster_uuid) if (defined $target_cluster_uuid);
        
        my $target_vserver_name = $opts{target_vserver_name} || $self->target_vserver_name;

        # Set the target vserver name
        if (defined $target_vserver_name) {
            $Log->debug("target vserver name is sent for remote call via ZAPI");
            $NaServerObj->set_target_vserver_name($target_vserver_name);
        }

        my $target_vserver_uuid = $opts{target_vserver_uuid} || $self->target_vserver_uuid;
        
        # Set the target vserver uuid
        if (defined $target_vserver_uuid) {
            $Log->debug("target vserver uuid is sent for remote call via ZAPI");
            $NaServerObj->set_target_vserver_uuid($target_vserver_uuid);
        }

        # Set the application name header to the supplied value
        my $app_header = $opts{app_header} || $self->app_header;
        $NaServerObj->set_app_header($app_header);

	# Set the zapi header
	my $zapi_header = $opts{zapi_header} || $self->zapi_header;
        $NaServerObj->set_zapi_header($zapi_header);
    };
    if ($@) {
        $Log->exit() if $may_exit;
        NACL::APISet::Exceptions::ConnectionFailedException->throw($@);
    }

    my $inputstr = "\nZAPI input XML:\n$commandInput";
    # Get the log levels
    my ($input_log_level_on_success, $output_log_level_on_success,
        $input_log_level_on_error,   $output_log_level_on_error
    );
		
    $input_log_level_on_success
        = $opts{input_log_level}
        || $opts{log_level}
		|| $self->{zapi_log_level}
        || DEFAULT_LOG_LEVEL;
    $output_log_level_on_success
        = $opts{output_log_level}
        || $opts{log_level}
		|| $self->{zapi_log_level}
        || DEFAULT_LOG_LEVEL;
    $input_log_level_on_error
        = $opts{input_log_level}
        || $opts{log_level}
		|| $self->{zapi_log_level_on_error}
        || $self->{zapi_log_level}
		|| DEFAULT_LOG_LEVEL_ON_ERROR;
    $output_log_level_on_error
        = $opts{output_log_level}
        || $opts{log_level}
		|| $self->{zapi_log_level_on_error}
		|| $self->{zapi_log_level}
        || DEFAULT_LOG_LEVEL_ON_ERROR;

    # Invoke the ZAPI
    my ($start_mes, $end_mes, $output_with_dtd);
    eval {
        $Log->debug( "Timeout scaling factor is $timeout_scaling_factor and "
                . "the timeout value is $timeout" );
        ## w.r.t burt991486 sometime the zapi connections
        ## takes much longer than expected before reporting
        ## timeout error so adding alarm for SIGINT
        local $SIG{ALRM} = sub { die "ZAPI Timed Out" };
        # Setting the alarm with $timeout value
        alarm $timeout;

        $NaServerObj->set_timeout($timeout);
        $start_mes = time();
        ($output_with_dtd) = $NaServerObj->invoke_xmlin_xmlout($commandInput);
        $end_mes = time();

        # Reset the alarm to 0 (i.e. no alarm)
        alarm 0;
    };# end of eval
    if ($@ || $@ =~ /ZAPI Timed Out/) {
       my $err_msg = $@;
       $end_mes = time();
       $Log->exit() if $may_exit;
       NATE::BaseException->throw("ZAPI invocation failed because,\n$err_msg" . "\n" . $NaServerObj->get_diag_msgs());
    }
    my $execution_time = $end_mes - $start_mes;
    $inputstr .= 'Execution time (in sec): '. $execution_time;
    if ( !defined ($output_with_dtd) ) {

        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_error
        );
        my $msg
            = "Internal error: NaServer library used by ZAPI APISets didn't return a NaElement object. "
            . "Please raise a burt with type as NACL and subtype as nacl_core";
        $Log->debug("Failure in invoking zapi: zapi call failed!");
        $Log->exit() if $may_exit;
        NATE::BaseException->throw($msg);
    }
    
    if (ref $output_with_dtd) {
        # invoke_xmlin_xmlout is supposed to return an XML string. However,
        # if there is an NaServer failure, then it returns an NaElement
        # object with details of the failure.
        # We log the input and output and throw an exception immediately here
        
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_error
        );

        my $outputstr = $self->_get_encoded_xml_string(output_object => $output_with_dtd);
        $self->_log_data(
            data      => $outputstr,
            log_level => $output_log_level_on_error
        );

        my $results_reason = $output_with_dtd->results_reason();
        
        # NaServer returns "Timeout. Could not read API response." when a ZAPI call
        # has timed out.
        if (defined $results_reason && $results_reason =~ /Timeout\.\s*Could.*/i) {
            my $msg = $output_with_dtd->results_reason()." The timeout value was $timeout";
            $msg .= "\n" . $NaServerObj->get_diag_msgs();
            $Log->exit() if $may_exit;
            
            NACL::APISet::Exceptions::TimeoutException->throw(
                $msg,
                timeout => $timeout,
                output  => $msg
            );
        } else {
            my $msg = 'Failure from NaServer: ' . $output_with_dtd->results_reason()
                . "\n" . $NaServerObj->get_diag_msgs();
            
            # Construct a CommandFailedException and create a mock Response
            # object
            my $response_obj = bless {
                command        => $commandInput,
                output         => $outputstr,
                execution_time => $execution_time,
                execute_status => 'failed',
                error_number   => $output_with_dtd->results_errno(),
                error_reason   => $results_reason,
                complete_raw_output => $outputstr
            }, 'NACL::APISet::Response::ZAPI';
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            => $msg,
                response_object => $response_obj,
            );
        }
    }
    
    my $complete_parsed_output_with_root = $self->_xml2perl(
        xmlString  => $output_with_dtd,
        ForceArray => 1,
        KeyAttr    => [],
        KeepRoot   => 1,
    );
    
    my ($root) = keys %{$complete_parsed_output_with_root};
    if ($root ne 'netapp') {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw('Complete XML doesnt contain "netapp" tag.'
            . "Root tag was '$root\n"
            . " Complete XML is:\n$output_with_dtd\n"
            . "NaServer disagnostic messages:\n"
            . $NaServerObj->get_diag_msgs()
        );
    }
    my $complete_parsed_output = $complete_parsed_output_with_root->{netapp}[0];
    
    my $complete_raw_output_encoded = $self->_perl2xml(
        perldata => $complete_parsed_output_with_root,
        KeepRoot => 1,
        KeyAttr  => [],
    );

    # Use the encoded form for logging, because that is what is actually
    # returned by the ZAPI
    my $outputstr = "\nZAPI output XML:\n$complete_raw_output_encoded\n";

    my $results = $complete_parsed_output->{results}[0];

    my $errNo  = $results->{errno} // 0;
    my $reason = $results->{reason} // '';
    my $status = $results->{status};
    
    # The keys of the ZAPI output could contain "status" and "reason".
    # For example, for ZAPI output of (this is from quota-status):
    #
    # <results status="passed">
    #     <reason>skipping new definition for bare (type:user target:bare qtree:&quot;&quot;).
    #     </reason>
    #     <status>on</status>
    #     <substatus>none</substatus>
    # </results>
    #
    # it would result in the parsed output having:
    # results => {
    #     status => ['passed', 'off']
    # }
    # In this case, the "status" of the ZAPI call would be the first element
    $status    = $status->[0] if (ref $status);
    $reason    = ($status eq 'passed') ? '' : $reason->[0] if (ref $reason);

    # The "raw output" is the complete output with the netapp tags stripped out
    my $raw_output_encoded = $complete_raw_output_encoded;
    $raw_output_encoded =~ s/^\s*<netapp.*?>//;
    $raw_output_encoded =~ s/<\/netapp>\s*$//;
    $raw_output_encoded =~ s/^\s*//;

    if ($self->{profiling_enabled}) {
        my $zapi_name = $commandInput;
        $zapi_name =~ s/\n//g;
        $self->_log_profiling_data(
            node           => $self->{hostid},
            start_time     => $start_mes,
            end_time       => $end_mes,
            elapsed_time   => $execution_time,
            command        => $zapi_name,
            interface      => 'zapi'
        );

    }

    # Log the input and output XML at a desired log level based on whether the
    # ZAPI invocation succeeded or not.
    my ($complete_raw_output_to_return, $raw_output_to_return);
    if ($status eq 'passed') {
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_success
        );
        $self->_log_data(
            data      => $outputstr,
            log_level => $output_log_level_on_success
        );

        # On success, return the encoded form (i.e has entity references like
        # "&gt;")
        $complete_raw_output_to_return = $complete_raw_output_encoded;
        $raw_output_to_return = $raw_output_encoded;
    } else {
        $self->_log_data(
            data      => $inputstr,
            log_level => $input_log_level_on_error
        );
        $self->_log_data(
            data      => $outputstr,
            log_level => $output_log_level_on_error
        );
        
        # On error, return the decoded form, since scripts rely on the
        # decoded form being returned on error.
        $complete_raw_output_to_return = $self->_decode_sp_chrs($complete_raw_output_encoded);
        $raw_output_to_return = $self->_decode_sp_chrs($raw_output_encoded);
    }

    $Log->exit() if $may_exit;
    return ($raw_output_to_return, $errNo, $reason, $status,
        $complete_raw_output_to_return, $execution_time, $complete_parsed_output
    );
} ## end sub executeZAPI

# Convert entity references back to the normal string form (such as &amp; -> &)
sub _decode_sp_chrs {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my ($xml) = validate_pos(@opts, {type => SCALAR});
    $xml =~ s/&lt;/</g;
    $xml =~ s/&gt;/>/g;
    $xml =~ s/&pos;/'/g;
    $xml =~ s/&quot;/"/g;
    $xml =~ s/&amp;/&/g;
    $Log->exit() if $may_exit;
    return $xml;
}

###############################################################################
# Method :    _log_data
# Objective:  Log the data in the specified log level
# Details:    This method uses Tharn log object ($Log) to log the specified
#             data in the given log level. Throws exception if the specified
#             log level is invalid
###############################################################################
sub _log_data {
    $Log->enter() if $may_enter;
    my $self      = shift();
    my %args      = @_;
    my $log_level = $args{log_level};
    try {
        $Log->$log_level( $args{data} );
    }
    otherwise {

        # NATE will throw script error which will be promoted to BaseException
        my $ex = shift;
        if ( $ex->text() =~ /Can't locate object method/ ) {
            NATE::Exceptions::Argument->throw(
                'Invalid log level: ' . $log_level );
        } else {
            $ex->throw();
        }
    };
    $Log->exit;
}

##############################################################################
# Method:    _get_encoded_xml_string_no_check
# Objective: To get the formatted (indented) and encoded XML string from
#            NaElement object
# Details: Recusrive code from _get_encoded_xml_string method is moved to this
#          method. It is done because there is no need of param validation
#          in every recursive call. For more details see burt778829
# See also: _get_encoded_xml_string
##############################################################################
sub _get_encoded_xml_string_no_check {
    my ($self, %args) = @_;
    my $indent    = $args{indent};
    my $naelement = $args{output_object};

    if ( !defined($indent) ) {
        $indent = "";
    }
    my $n = $naelement->{name};

    my $s = "$indent<$n";

    my $keys = $naelement->{attrkeys};
    my $vals = $naelement->{attrvals};
    for ( my $i = 0; $i <= $#$keys; $i++ ) {
        $s .= " " . $keys->[$i] . "=\"" . $vals->[$i] . "\"";
    }
    $s .= ">";

    my $i;
    my $children = $naelement->{children};
    if ( $#$children >= 0 ) {
        $s .= "\n";
    }

    for ( $i = 0; $i <= $#$children; $i++ ) {
        my $c = $children->[$i];
        $s .= $self->_get_encoded_xml_string_no_check(
            indent        => "$indent\t",
            output_object => $c
        );
    }
    my $cont = $naelement->{content};

    # The raw XML output returned from the service has all special
    # characters of XML represented as entity references which is required for
    # the XML output to be valid.
    # The NaElement output object returned by the zapi library has the entity
    # references converted to its corresponding Unicode character as it uses
    # XML::Parser XML library to obtain the NaElement object. When we return
    # back the stringified XML to the users, those special XML characters like
    # &,<,>,'," should be converted back to its corresponding entity reference
    # so that users can use the raw XML output with any XML processing library
    # One such user of raw XML output is get_parsed_output(). (We ourself!)
    $cont =~ s/&/\&amp;/g;    # it's important that & be substituted first
    $cont =~ s/</\&lt;/g;
    $cont =~ s/>/\&gt;/g;
    $cont =~ s/'/\&apos;/g;
    $cont =~ s/"/\&quot;/g;

    $s .= $cont;
    if ( $#$children >= 0 ) {
        $s .= "$indent";
    }
    $s .= "</$n>\n";
    
    return $s;
}

##############################################################################
# Method:    _get_encoded_xml_string
# Objective: To get the formatted (indented) and encoded XML string from
#            NaElement object
# Details: This method returns the xml string which is formatted for the indent
#          specified by the user and it also converts any special characters in
#          the xml content into its corresponding entity reference so that the
#          raw XML output is returned as a valid XML.
# See also:   toEncodedString of NaElement.pm, but the only difference is that
#             the encoding of special characters of XML like (&, <, > , ', ")
#             are not done by toEncodedString
##############################################################################
sub _get_encoded_xml_string {
    $Log->enter() if $may_enter;
    my $self = shift();
    my %args = validate_with(
        params => \@_,
        spec   => {
            "output_object" => { isa => "NaElement" },
            "indent" => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1,
    );

    my $s = $self->_get_encoded_xml_string_no_check(%args);
    $Log->debug("The XML obtained from NaElement object is $s");

    $Log->exit() if $may_exit;
    return $s;
}

sub send_command {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    
    my $response_type = delete $args{_response_type};
    my ($output,$errNo,$errReason, $execution_status, $completRawOutput,
        $execution_time, $complete_parsed_output) = $self->executeZAPI(%args);

    my $response_pkg = "NACL::APISet::Response::ZAPI::$response_type";
    my $response = $response_pkg->new(
        output           => $output,
        command          => $args{command},
        execution_time   => $execution_time,
        error_number     => $errNo,
        error_reason     => $errReason,
        execution_status => $execution_status,
        complete_raw_output => $completRawOutput,
        complete_parsed_output => $complete_parsed_output,
    );
    $Log->exit() if $may_exit;
    return $response;
}

##############################################################################
# Method:    set_vfiler_or_vserver
# Objective: To set the current vfiler or vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_vfiler_or_vserver {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "server_name" => { type => SCALAR } }
    );
    $self->{"server_name"} = $args{"server_name"};

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    get_vfiler_or_vserver
# Objective: Get the current vfiler or vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_vfiler_or_vserver {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"server_name"};
}

##############################################################################
# Method:    clear_vfiler_or_vserver
# Objective: To set the default vfiler or vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_vfiler_or_vserver {
    $Log->enter() if $may_enter;

    my ( $self, %args ) = @_;

    # Empty string is taken as default value so that users are allowed to
    # explicitly prefix their commands with vfiler0
    $self->set_vfiler_or_vserver( "server_name" => "" );

    $Log->exit() if $may_exit;
}

###############################################################################
# Method:    clear_timeout
# Objective: Set the timeout to the default value
# Details:   See POD documentation at the end of this file.
###############################################################################
sub clear_timeout {
    my $self = shift;
    $Log->enter() if $may_enter;

    $self->{timeout} = DEFAULT_TIMEOUT;

    $Log->exit() if $may_exit;
}

###############################################################################
# Method:    set_timeout
# Objective: Set the timeout to a new value
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_timeout {
    my $self = shift;
    $Log->enter() if $may_enter;
    my %args = validate_with(
        params => \@_,
        spec   => {
            'zapi-timeout' => { type => SCALAR, optional => 1 },
            timeout        => { type => SCALAR, optional => 1 },
        }
    );
    my $timeout;
    if ( defined $args{'zapi-timeout'} ) {
        $timeout = $args{'zapi-timeout'};
    } elsif ( defined $args{timeout} ) {
        $timeout = $args{timeout};
    } else {
        NATE::Exceptions::Argument->throw( 'Either of the parameters '
                . "'zapi-timeout' or 'timeout' should have been "
                . 'passed but neither were' );
    }
    $self->{timeout} = $timeout;
    $Log->debug( 'Timeout set to ' . $self->{timeout} );
    $Log->exit() if $may_exit;
}

##############################################################################
#                                    POD here
##############################################################################



=head1 NAME

NACL::APISet::Session::ZAPI - Base session class for ZAPI APISet

=head1 SYNOPSIS

 use NACL::APISet::Session::ZAPI;

=head1 DESCRIPTION

A C<NACL::APISet::Session::ZAPI> is used to establish a connection with the
filer and invoke zapi commands. This is further subclassed to execute ZAPI commands
specifically to CMode, 7Mode etc.

=cut



=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::ZAPI>.

=over

=item Synopsis

 Refer subclasses L<new|lib-NACL-APISet-Session-ZAPI-DFM-pm/initialize>,
 L<new|lib-NACL-APISet-Session-ZAPI-7Mode-pm/initialize>,
 L<new|lib-NACL-APISet-Session-ZAPI-CMode-pm/initialize>,
 L<new|lib-NACL-APISet-Session-ZAPI-Windows-pm/initialize>

=item Arguments

=over

=item hostobj

(Optional)
The hostrec object from which contains essential details like default_ipaddress,
default_username and default_password will be obtained to connect to the
host.

=item ipaddress

(Optional)
ip-address of the host to which we want to connect to.This will be
ignored if a connection object is passed. If no value is provided, the
connection layer will get this value from the host record.

=item username

(Optional)
Username used to log into the resource.This will be ignored if a connection
object is passed. If no value is provided, the connection layer will get
this value from the host record.

=item password

(Optional)
Password used to log into the resource. This will be ignored if a connection
object is passed. If no value is provided, the connection layer will get
this value from the host record.

=item connection

(Optional)
An established Connectrec object from which details like ip-address,
username and password can be obtained.

=item set

(Mandatory)
For POD refer L<set|lib-NACL-APISet-pm/initialize>

=back

=back

=cut



=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ConnectionFailedException

=item NACL::APISet::Exceptions::InvalidXMLException

=item NATE::Exceptions::Argument

=back

=cut



=head1 METHODS

=head2 executeZAPI

This method will establish a http/https connection with the service and sends
the input XML to the selected port number for execution. It also
looks for vfiler/vserver override and ontapi version overrides stored in
session object and considers these values appropriately by setting these
overrides in the ZAPI object created for execution.
This function throws NACL::APISet::Exceptions::TimeoutException on timeout.

=over

=item Synopsis

($output, $error_no, $error_reason, $status) =
$session->executeZAPI( command => "<system-get-version/>" );

=item Arguments

=over

=item command

(Mandatory)
The input raw xml to be sent

=item zapi-timeout

(Optional)
Override zapi timeout in seconds if any

=back

=item Returns

This method will returns the stringified raw xml output, error number,
error reason and execution status in the order specified.

=back

=head2 execute_raw_ZAPI

This method will be used by execute_raw_command to execute the input
XML sent by the user.

=over

=item Synopsis

$output = $session->execute_raw_ZAPI( command => "<system-get-version/>",
'zapi-timeout'=>$timeout, 'server-override-name' =>$server,
'ontapi-version'=>$ontapi_version,
'return-response-object'=><0|1> );

=item Arguments

=over

=item command

(Mandatory)
The input raw xml to be sent

=item server-override-name

(Optional)
Override for vfiler or vserver context

=item ontapi-version

(Optional)
Ontapi-version to be set in the input XML

=item zapi-timeout

(Optional)
Override zapi timeout in seconds if any

=item return-response-object

(Mandatory)
If set, a response object will be returned else raw XML
output will be returned

=back

=item Returns

This method will return a response object or raw XML output depending
on 'return-response-object' is set or unset.

=back

=head2 set_server_type

This method is used to set the server type. This is equivalent to -t option
in ontapi. Possible values are filer, dfm, agent and netcache. Based on the
server type, the appropriate DTD file (external declaration of DTD) will be
determined. Default value will be "filer".

=over

=item Synopsis

$Api_Set_Obj->set_server_type(type=>"netcache");

=item Arguments

=over

=item type

(Mandatory)
The server type- filer, dfm, agent, netcache.

=back

=item Returns
Nothing

=back

=head2 get_server_type

This method returns the current server type.

=over

=item Synopsis

$server_type = $Api_Set_Obj->get_server_type();

=item Arguments
No Inputs

=item Returns

(Scalar)
Server type

=back

=head2 set_vfiler_or_vserver

To set the name of the vfiler (for 7Mode) or vserver (for CMode) in
whose context commands has to be executed.

=over

=item Synopsis

$Api_Set_Obj->set_vfiler_or_vserver('server_name' => $name);

=item Arguments

=over

=item server_name

Name of the vfiler or vserver

=back

=item Returns

Nothing.

=back

=head2 get_vfiler_or_vserver

Get current vfiler (for 7Mode) or vserver (for CMode)context
in which we are executing commands

=over

=item Synopsis

$Api_Set_Obj->get_vfiler_or_vserver();

=item Arguments

None.

=item Returns

(String)
Current vfiler context or vserver context

=back

=head2 clear_vfiler_or_vserver

Reset to executing commands in default vfiler context (vfiler0 or the 7Mode
node) for 7Mode or default vserver context (to the context of whose mgmt-ip we
are connected with. Possibilities can be admin-vserver if we are connected
via cluster lif or the node-vserver if we are connected via node mgmt ip)
for CMode.

=over

=item Synopsis

$Api_Set_Obj->clear_vfiler_or_vserver();

=item Arguments

None.

=item Returns

None.

=back

=head2 set_test_parameters

To set the test parameters globally

=head3 Parameters

=over

=item NACL_ZAPI_<SET>_LOG_IN_DB

Takes a boolean value (0/1) and defaults to 1.
This parameter decides whether logging of ZAPI name, its inputs and outputs
into NATE DB should happen or not. This logged information into NATE
DB would later be used by QC.

=item NACL_ZAPI_<SET>_TRANSPORT_TYPE

This parameter sets the transport type for ZAPI execution.
The possible values are "HTTP" (default transport type) 
and "HTTPS". The values are case-insensitive.

=item NACL_ZAPI_<SET>_PORT

This parameter can be used to specify port for different sets.

=item NACL_ZAPI_<SET>_USE_SSL_AUTH

Takes a boolean value (0/1) and defaults to 1.
This parameter decides whether or not to enable 
certificate based authentication while executing ZAPI on the filer. 

=item NACL_ZAPI_<SET>_KEY_FILE

The secret key in a Public Key Cryptography system,
used to decrypt incoming messages and sign outgoing ones.

=item NACL_ZAPI_<SET>_KEY_PASSWORD

Password required to decrypt host private key 
file incase key is encrypted.

=item NACL_ZAPI_<SET>_CERT_FILE

Host certificate file. This is used by certificate
authority to certify host.

=item NACL_ZAPI_<SET>_CA_CERT_FILE

Certificate authority certificate file. 
It is used by host(user) to certify filer(server).

=item NACL_ZAPI_<SET>_ENABLE_HOSTNAME_VERIFICATION

Takes a boolean value (0/1) and defaults to 1.
Whether or not to verify the hostname while doing 
certificate based authentication.

=item NACL_ZAPI_<SET>_ORIGINATOR_ID

ONTAP audit is a facility to log administrative 
actions in a protected log. Each administrative action is
audited with when it was performed, who performed, 
type of request and request it self. The originator id
(Originator-Id)is an username or UID of an administrator who 
logged in to the management stations to administer filers.
ZAPI passes Originator-Id in the XML request such that 
ONTAP correctly audits this information.

(See 
http://wikid.netapp.com/w/Mot-eng/Auditing/Zapi_Originator_Id 
for detailed information on the usage of this parameter)

=item NACL_ZAPI_<SET>_TRACE_THRESHOLD

Takes a boolean value (0/1) and defaults to 1.
To enable ZAPI smftrace trigger to the http header of a ZAPI request.

=back

=over

=item Synopsis

$Api_Set_Obj->set_test_parameters();

=item Returns

Nothing.

=back

=head2 clear_timeout

This method is used to set the timeout to the default value.

=over

=item Synopsis

$timeout = $self->{session}->clear_timeout();

=item Arguments

None.

=item Returns

Nothing.

=back

=head2 set_timeout

This method is used to set the timeout to a new value.

=over

=item Synopsis

    $self->{session}->set_timeout('zapi-timeout' => $new_timeout);

                          or

    $self->{session}->set_timeout(timeout => $new_timeout);

=item Arguments

Either of timeout or zapi-timeout should be provided. If both are
provided, then zapi-timeout takes precedence.

=over

=item zapi-timeout

The new timeout value in seconds

=item timeout

The new timeout value in seconds

=back

=item Returns

Nothing.

=back


=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;

