# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Armadillo bridge commands
##
## @description
##    This is a library to parse the raw output of Armadillo bridge commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::Armadillo::default::get
##
##  get_bridgemodel(%args)
##  get_bridgename (%args)
##  get_date(%args)
##  get_fcporterrors(%args)
##  get_fcconnmode(%args)
##  get_fcdatarate (%args)
##  get_fwwwname (%args)
##  get_ipaddress(%args)
##  get_ipgateway (%args)
##  get_ipsubnetmask(%args)
##  get_serialnumber (%args)
##  get_ipdhcp(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::Armadillo::default::get;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::UnexpectedOutputException;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Wrong\/Missing Parameters/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}
## @name ::get_bridgemodel
##
## @summary Parses the raw output from the "get bridgemodel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_bridgemodel with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_bridgemodel(%args);
## INPUTS TO METHOD CALL 'get_bridgemodel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get bridgemodel
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;"ATTO FibreBridge 6500N"
##&nbsp;(c) 2002 - 2011 ATTO Technology, Incorporated
##&nbsp;Firmware version A64P release date "Oct  7 2011", 13:08:59 Build A64P
##&nbsp;Base version    51.01
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'release_date' => 'Oct  7 2011,13:08:59',
##&nbsp;            'base_version' => '51.01',
##&nbsp;            'model' => 'ATTO FibreBridge 6500N',
##&nbsp;            'build' => 'A64P',
##&nbsp;            'firmaware_version' => 'A64P'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_bridgemodel {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output
        =~ /\"(.*)\"\n*.*\n*Firmware\sversion\s*(\S+)\s*release\sdate\s\"([\w\s\d]+)\"\D+([\d:]+)\s*Build\s(\w+)\r*\n*Base\s*version\s*(\d+\.\d+)/i
        )
    {
        $hash{'model'}            = $1;
        $hash{'firmware_version'} = $2;
        $hash{'release_date'}     = $3 . ',' . $4;
        $hash{'build'}            = $5;
        $hash{'base_version'}     = $6;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_bridgename
##
## @summary Parses the raw output from the "get bridgename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_bridgename with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_bridgename(%args);
## INPUTS TO METHOD CALL 'get_bridgename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get bridgename
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;BridgeName = "fcs01"
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bridge_name' => 'fcs01'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_bridgename {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /bridgename\s*=\s*\"(.*)\"/i ) {
        $hash{'bridge_name'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_date
##
## @summary Parses the raw output from the "get date" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_date with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_date(%args);
## INPUTS TO METHOD CALL 'get_date' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get date
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Date = 02/02/2012
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'date' => '02/02/2012'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_date {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /Date\s=\s(.*)/i ) {
        $hash{'date'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_serialnumber
##
## @summary Parses the raw output from the "get serialnumber" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_serialnumber with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_serialnumber(%args);
## INPUTS TO METHOD CALL 'get_serialnumber' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get serialnumber
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Part Identifier: FB6500N Sequence Number: 100133
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sequence_number' => '100133',
##&nbsp;            'part_identifier' => 'FB6500N'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_serialnumber {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /Part\s*identifier:\s*(.*)\ssequence\snumber:\s*(.*)/i ) {
        $hash{'part_identifier'} = $1;
        $hash{'sequence_number'} = $2;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_ipdnsserver
##
## @summary Parses the raw output from the "get ipdnsserver" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_ipdnsserver with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_ipdnsserver(%args);
## INPUTS TO METHOD CALL 'get_ipdnsserver' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get ipdnsserver
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPDNSServer = 0.0.0.0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_dnsserver' => '0.0.0.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_ipdnsserver {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /IPDNSServer\s=\s(.*)/i ) {
        $hash{'ip_dnsserver'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_ipaddress
##
## @summary Parses the raw output from the "get ipaddress" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_ipaddress with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_ipaddress(%args);
## INPUTS TO METHOD CALL 'get_ipaddress' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get ipaddress mp1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPAddress = 10.228.213.94
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_address' => '10.228.213.94'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_ipaddress {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /IPAddress\s=\s(.*)/i ) {
        $hash{'ip_address'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_ipgateway
##
## @summary Parses the raw output from the "get ipgateway" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_ipgateway with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_ipgateway(%args);
## INPUTS TO METHOD CALL 'get_ipgateway' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get ipgateway mp1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPGateway = 10.228.213.1
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_gateway' => '10.228.213.1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_ipgateway {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /IPGateway\s=\s(.*)/i ) {
        $hash{'ip_gateway'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_ipsubnetmask
##
## @summary Parses the raw output from the "get ipsubnetmask" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_ipsubnetmask with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_ipsubnetmask(%args);
## INPUTS TO METHOD CALL 'get_ipsubnetmask' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get ipsubnetmask mp1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPSubnetMask = 255.255.255.0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_subnetmask' => '255.255.255.0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_ipsubnetmask {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /IPSubnetMask\s=\s(.*)/i ) {
        $hash{'ip_subnetmask'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_ipdhcp
##
## @summary Parses the raw output from the "get ipdhcp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_ipdhcp with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_ipdhcp(%args);
## INPUTS TO METHOD CALL 'get_ipdhcp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get ipdhcp mp1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IPDHCP = disabled
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_dhcp' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_ipdhcp {
    my %args = @_;
    my @result_array;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error = _check_error( raw_output      => $output,
                              response_object => $resp );
    my %hash;
    if ( $output =~ /IPDHCP\s=\s(.*)/i ) {
        $hash{'ip_dhcp'} = $1;
        push( @result_array, \%hash );
    }
    return \@result_array;
}
## @name ::get_fcporterrors
##
## @summary Parses the raw output from the "get fcporterrors" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_fcporterrors with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_fcporterrors(%args);
## INPUTS TO METHOD CALL 'get_fcporterrors' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'fp' </td> <td> 'fp' </td> <td> '1' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get fcporterrors all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;5
##&nbsp;; Fibre Channel Error Counts
##&nbsp;; Port | Link Failures | Sync Loss | Signal Loss | Invalid Tx | Invalid CRC
##&nbsp;;==========================================================================
##&nbsp;   1                24           0             0          323             0
##&nbsp;   2                 1           1             1      1065135             0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'signal_loss' => '0',
##&nbsp;            'invalid_crc' => '0',
##&nbsp;            'invalid_tx' => '323',
##&nbsp;            'sync_loss' => '0',
##&nbsp;            'link_failures' => '24',
##&nbsp;            'port' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'signal_loss' => '1',
##&nbsp;            'invalid_crc' => '0',
##&nbsp;            'invalid_tx' => '1065135',
##&nbsp;            'sync_loss' => '1',
##&nbsp;            'link_failures' => '1',
##&nbsp;            'port' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_fcporterrors {
    my %args = @_;
    my @result_array;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_raw_output();
    my $error = _check_error( raw_output      => $raw_output,
                              response_object => $resp );
    my %hash;

    if ( $output =~ /^(\d)/ ) {
        my $first_line = $1;
        my @arr_lines = get_lines( "raw_output"      => $raw_output,
                                   "first_line"      => $first_line,
                                   "can_throw"       => 1,
                                   "response_object" => $resp
        );
        my @header;
        foreach my $lines (@arr_lines) {
            my %hash = ();
            if ( $lines =~ /^;.*\|/ ) {
                @header = split( /\|/, $lines );
            }
            next if ( $lines =~ /^;|^\d$/ );
            my @values = split( /\s+/, $lines );
            for ( my $i = 0; $i < scalar(@values) - 1; $i++ ) {
                my $key = lc $header[$i];
                $key =~ s/;//;
                $key = trim_string( string => $key );
                $key =~ s/\s/_/;
                $hash{$key} = $values[$i];
            }
            push( @result_array, \%hash ) if %hash;
        }
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                            'text' => "Output was not in the expected format",
                            'response_object' => $resp );
    }
    return \@result_array;
}
## @name ::get_fcconnmode
##
## @summary Parses the raw output from the "get fcconnmode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_fcconnmode with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_fcconnmode(%args);
## INPUTS TO METHOD CALL 'get_fcconnmode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'fp' </td> <td> 'fp' </td> <td> '1' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get fcconnmode all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4
##&nbsp;; Port    Conn Mode
##&nbsp;;==================
##&nbsp;   1      ptp
##&nbsp;   2      ptp
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'conn_mode' => 'ptp',
##&nbsp;            'port' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'conn_mode' => 'ptp',
##&nbsp;            'port' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_fcconnmode {
    my %args = @_;
    my @result_array;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_raw_output();
    my $error = _check_error( raw_output      => $raw_output,
                              response_object => $resp );
    my %hash;

    if ( $output =~ /^(\d)/ ) {
        my $first_line = $1;
        my @arr_lines = get_lines( "raw_output"      => $raw_output,
                                   "first_line"      => $first_line,
                                   "can_throw"       => 1,
                                   "response_object" => $resp
        );
        foreach my $lines (@arr_lines) {
            my %hash = ();
            next if ( $lines =~ /^;/ );
            if ( $lines =~ /(\d+)\s+(\w+)/ ) {
                $hash{'port'}      = $1;
                $hash{'conn_mode'} = $2;
            }
            push( @result_array, \%hash ) if %hash;
        }
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                            'text' => "Output was not in the expected format",
                            'response_object' => $resp );
    }
    return \@result_array;
}
## @name ::get_fcdatarate
##
## @summary Parses the raw output from the "get fcdatarate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_fcdatarate with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_fcdatarate(%args);
## INPUTS TO METHOD CALL 'get_fcdatarate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'fp' </td> <td> 'fp' </td> <td> '1' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get fcdatarate all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4
##&nbsp;; Port    Data Rate
##&nbsp;;==================
##&nbsp;   1      8Gb
##&nbsp;   2      8Gb
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'port' => '1',
##&nbsp;            'data_rate' => '8Gb'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port' => '2',
##&nbsp;            'data_rate' => '8Gb'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub get_fcdatarate {
    my %args = @_;
    my @result_array;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_raw_output();
    my $error = _check_error( raw_output      => $raw_output,
                              response_object => $resp );
    my %hash;

    if ( $output =~ /^(\d)/ ) {
        my $first_line = $1;
        my @arr_lines = get_lines( "raw_output"      => $raw_output,
                                   "first_line"      => $first_line,
                                   "can_throw"       => 1,
                                   "response_object" => $resp
        );
        foreach my $lines (@arr_lines) {
            my %hash = ();
            next if ( $lines =~ /^;/ );
            if ( $lines =~ /(\d+)\s+(.+)/ ) {
                $hash{'port'}      = $1;
                $hash{'data_rate'} = $2;
            }
            push( @result_array, \%hash ) if %hash;
        }
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                            'text' => "Output was not in the expected format",
                            'response_object' => $resp );
    }
    return \@result_array;
}
## @name ::get_fcwwname
##
## @summary Parses the raw output from the "get fcwwname" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from get_fcwwname with any of its options as described in the
##    corresponding cdef file 'get.cdefs'
## @synopsis
##      get_fcwwname(%args);
## INPUTS TO METHOD CALL 'get_fcwwname' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = get fcwwname all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4
##&nbsp;; Port    FCWWName
##&nbsp;;================================
##&nbsp;   1      20 00 00 10 86 60 02 3C
##&nbsp;   2      20 00 00 10 86 60 02 3C
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'port' => '1',
##&nbsp;            'fcwwname' => '20 00 00 10 86 60 02 3C'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'port' => '2',
##&nbsp;            'fcwwname' => '20 00 00 10 86 60 02 3C'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = get fcwwname all
##&nbsp;
sub get_fcwwname {
    my %args = @_;
    my @result_array;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_raw_output();
    my $error = _check_error( raw_output      => $raw_output,
                              response_object => $resp );
    my %hash;

    if ( $output =~ /^(\d)/ ) {
        my $first_line = $1;
        my @arr_lines = get_lines( "raw_output"      => $raw_output,
                                   "first_line"      => $first_line,
                                   "can_throw"       => 1,
                                   "response_object" => $resp
        );
        foreach my $lines (@arr_lines) {
            my %hash = ();
            next if ( $lines =~ /^;/ );
            if ( $lines =~ /(\d+)\s+(.+)/ ) {
                $hash{'port'}     = $1;
                $hash{'fcwwname'} = $2;
            }
            push( @result_array, \%hash ) if %hash;
        }
    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                            'text' => "Output was not in the expected format",
                            'response_object' => $resp );
    }
    return \@result_array;
}
1;
