#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/aptpolicy.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC aptpolicy command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch aptpolicy command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::aptpolicy
##
##  aptpolicy(%args)
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::aptpolicy;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::aptpolicy
##
## @summary Parses the raw output from the "aptpolicy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from aptpolicy with any of its options as described in the
##    corresponding cdef file 'aptpolicy.cdefs'
## @synopsis
##      aptpolicy(%args);
## INPUTS TO METHOD CALL 'aptpolicy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'policy' </td> </tr>
## <tr> <td> '-ap' </td> <td> 'ap_policy' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = aptpolicy
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Current Policy: 1 1(ap)
##&nbsp;
##&nbsp; 3 0(ap): Default Policy
##&nbsp; 1: Port Based Routing Policy
##&nbsp; 3: Exchange Based Routing Policy
##&nbsp;         0: AP Shared Link Policy
##&nbsp;         1: AP Dedicated Link Policy
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'current_policy' => '1 1(ap)',
##&nbsp;            'available_policy_options' => [
##&nbsp;                                            {
##&nbsp;                                              'ap_dedicated_link_policy' => '1',
##&nbsp;                                              'port_based_routing_policy' => '1',
##&nbsp;                                              'ap_shared_link_policy' => '0',
##&nbsp;                                              'exchange_based_routing_policy' => '3'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'default_policy' => '3 0(ap)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub aptpolicy {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _aptpolicy(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub aptpolicy

sub _aptpolicy {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'aptpolicy', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );

    foreach my $line (@arr) {
        my $key1;
        if ($line =~ /(Policy updated successfully)/ ) {
            $result_array[0]->{status} = $1;
            last;
        }
        if ($line =~ /(Current Policy)\: (.*)/ ) {
            $key1 = to_key_name(item=>$1);
            $result_array[0]->{$key1} = $2;
        } elsif ($line =~ /(.*)\: (Default Policy)/ ) {
            $key1 = to_key_name(item=>$2);
            $result_array[0]->{$key1} = $1;
        } elsif ($line =~ /(1): (Port Based Routing Policy)/ ) {
            $key1 = to_key_name(item=>$2);
            $result_array[0]->{available_policy_options}->[0]->{$key1} = $1;
        } elsif ($line =~ /(3): (Exchange Based Routing Policy)/ ) {
            $key1 = to_key_name(item=>$2);
            $result_array[0]->{available_policy_options}->[0]->{$key1} = $1;
        } elsif ($line =~ /(0): (AP Shared Link Policy)/ ) {
            $key1 = to_key_name(item=>$2);
            $result_array[0]->{available_policy_options}->[0]->{$key1} = $1;
        } elsif ($line =~ /(1): (AP Dedicated Link Policy)/ ) {
            $key1 = to_key_name(item=>$2);
            $result_array[0]->{available_policy_options}->[0]->{$key1} = $1;
        }
    }

    return \@result_array;
}

sub _parse_for_errors {
    my %args = @_;

    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $command = $args{command};
    my $error;
    if($output =~ /Invalid usage/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif($output =~ /(Invalid AP policy)|(External policy must be numeric)|(Issue "switchDisable" before changing the policy)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }

    return $error;
}

1;
