#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC cfgactvshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch cfgactvshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::cfgactvshow
##
##  cfgactvshow(%args)
##
## @author dl-nacl-dev@netapp.com,veerr@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::cfgactvshow;

use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::cfgactvshow
##
## @summary Parses the raw output from the "cfgactvshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cfgactvshow with any of its options as described in the
##    corresponding cdef file 'cfgactvshow.cdefs'
## @synopsis
##      cfgactvshow(%args);
## INPUTS TO METHOD CALL 'cfgactvshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cfgactvshow
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'effective_configuration' => [
##&nbsp;                                           {
##&nbsp;                                             'cfg_name' => 'camaro',
##&nbsp;                                             'zone' => [
##&nbsp;                                                         {
##&nbsp;                                                           'members' => [
##&nbsp;                                                                          '50:0a:09:80:00:14:25:90',
##&nbsp;                                                                          '50:06:01:68:30:22:7a:7f'
##&nbsp;                                                                        ],
##&nbsp;                                                           'zone_name' => 'camaro_87a_0c_cx5a_fe0'
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'members' => [
##&nbsp;                                                                          '50:0a:09:80:00:14:1b:a0',
##&nbsp;                                                                          '50:06:01:68:30:22:7a:7f'
##&nbsp;                                                                        ],
##&nbsp;                                                           'zone_name' => 'camaro_87b_0d_cx5b_fe0'
##&nbsp;                                                         }
##&nbsp;                                                       ]
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cfgactvshow {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'cfgactvshow',
                             "ptr_error_message" => \$error_message,
                             );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @result_array;
    my ($k,$p);
    my @lines = get_lines( raw_output => $raw_output, can_throw => 0,
                           response_object => $resp );
    $k = -1;
    foreach my $line (@lines) {
        if($line =~ /^vgbr200es2:admin/) {
            next;
        } elsif($line =~ /^cfg:\s+([0-9a-zA-Z\_]+)/) {
            $result_array[0] -> {"effective_configuration"} -> [0] ->{"cfg_name"} = $1;
        } elsif($line =~ /^zone:\s+([0-9a-zA-Z\_]+)$/) {
            $p = 0;
            $k++;
            $result_array[0] -> {"effective_configuration"} -> [0] -> {"zone"} -> [$k] -> {"zone_name"} = $1;
        } elsif($line =~ /^([0-9\:a-zA-Z]+)$/) {
            $result_array[0] -> {"effective_configuration"} -> [0] -> {"zone"} -> [$k] -> {"members"} -> [$p]= $1;
            $p++;
        }
    }
    return \@result_array;
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if($raw_output =~ /[Uu]sage:/) {
        $$error_message = $raw_output;
        $error = "CommandFailed";
    }
    return $error;
}
1;
