#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC cfgshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch cfgshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::cfgshow
##  
##  cfgshow(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::cfgshow;

use NACL::APISet::Switch::CLI::BrocadeFC::default::zoneshow;

## @name ::cfgshow
##
## @summary Parses the raw output from the "cfgshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cfgshow with any of its options as described in the
##    corresponding cdef file 'cfgshow.cdefs'
## @synopsis
##      cfgshow(%args);
## INPUTS TO METHOD CALL 'cfgshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> </tr>
## <tr> <td> '--sort' </td> <td> 'sort' </td> </tr>
## <tr> <td> ',' </td> <td> 'mode' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cfgshow
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Defined configuration:
##&nbsp; cfg:   SANLinux
##&nbsp;                Raj_178_24_25_1; Raj_178_24_25_2; Raj_179_24_25_1;
##&nbsp;                Raj_179_24_25_2; SAC_105_0a; SAC_105_0b; SAC_107_0a;
##&nbsp;                SAC_107_0b; SAC_113_0a; SAC_113_0b; SAC_114_0a; SAC_114_0b;
##&nbsp;                SAC_143_0a; SAC_143_0b; SAC_144_0a; SAC_144_0b; SAC_174_0a;
##&nbsp;                SAC_174_0b; sub_149_a; sub_149_b
##&nbsp; zone:  Raj_178_24_25_1
##&nbsp;                10:00:00:00:c9:d0:88:ba; 50:0a:09:81:87:59:82:bb;
##&nbsp;                50:0a:09:81:97:59:82:bb; 50:0a:09:85:87:59:82:bb;
##&nbsp;                50:0a:09:85:97:59:82:bb
##&nbsp;Effective configuration:
##&nbsp; cfg:   SANLinux
##&nbsp; zone:  Raj_178_24_25_1
##&nbsp;                10:00:00:00:c9:d0:88:ba
##&nbsp;                50:0a:09:81:87:59:82:bb
##&nbsp;                50:0a:09:81:97:59:82:bb
##&nbsp;                50:0a:09:85:87:59:82:bb
##&nbsp;                50:0a:09:85:97:59:82:bb
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'defined_cfg' => [
##&nbsp;                               {
##&nbsp;                                 'name' => 'SANLinux',
##&nbsp;                                 'zones' => [
##&nbsp;                                              'Raj_178_24_25_1',
##&nbsp;                                              'Raj_178_24_25_2',
##&nbsp;                                              'Raj_179_24_25_2',
##&nbsp;                                              'SAC_105_0a',
##&nbsp;                                              'SAC_105_0b',
##&nbsp;                                              'SAC_107_0b',
##&nbsp;                                              'SAC_113_0a',
##&nbsp;                                              'SAC_113_0b',
##&nbsp;                                              'SAC_114_0a',
##&nbsp;                                              'SAC_143_0a',
##&nbsp;                                              'SAC_143_0b',
##&nbsp;                                              'SAC_144_0a',
##&nbsp;                                              'SAC_144_0b',
##&nbsp;                                              'SAC_174_0b',
##&nbsp;                                              'sub_149_a'
##&nbsp;                                            ]
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'effective_cfg' => [
##&nbsp;                                 {
##&nbsp;                                   'name' => 'SANLinux',
##&nbsp;                                   'zones' => [
##&nbsp;                                                'Raj_178_24_25_1',
##&nbsp;                                                'Raj_178_24_25_2',
##&nbsp;                                                'Raj_179_24_25_1',
##&nbsp;                                                'Raj_179_24_25_2',
##&nbsp;                                                'SAC_105_0a',
##&nbsp;                                                'SAC_105_0b',
##&nbsp;                                                'SAC_107_0a',
##&nbsp;                                                'SAC_107_0b',
##&nbsp;                                                'SAC_113_0a',
##&nbsp;                                                'SAC_113_0b',
##&nbsp;                                                'SAC_114_0a',
##&nbsp;                                                'SAC_114_0b',
##&nbsp;                                                'SAC_143_0a',
##&nbsp;                                                'SAC_143_0b',
##&nbsp;                                                'SAC_144_0a',
##&nbsp;                                                'SAC_144_0b',
##&nbsp;                                                'SAC_174_0a',
##&nbsp;                                                'SAC_174_0b',
##&nbsp;                                                'sub_149_a',
##&nbsp;                                                'sub_149_b'
##&nbsp;                                              ]
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'members' => [
##&nbsp;                                          '10:00:00:00:c9:d0:88:ba',
##&nbsp;                                          '50:0a:09:81:97:59:82:bb',
##&nbsp;                                          '50:0a:09:85:97:59:82:bb'
##&nbsp;                                        ],
##&nbsp;                           'name' => 'Raj_178_24_25_1'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cfgshow {
    my %args = @_;

    my @result_array = NACL::APISet::Switch::CLI::BrocadeFC::default::zoneshow::zoneshow(%args);

    return \@result_array;
}

1;
