#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/configshow.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC configshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch configshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::configshow
##
##  configshow(%args)
##
## @author anupamap(anupamap@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::configshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error );
    if ( $raw_output =~ /unrecognized option/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /option.*requires an argument/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~
/local file.*not found|USB device either not present or not enabled|unknown fabric ID/i
      )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::configshow
##
## @summary Parses the raw output from the "configshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from configshow with any of its options as described in the
##    corresponding cdef file 'configshow.cdefs'
## @synopsis
##      configshow(%args);
## INPUTS TO METHOD CALL 'configshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-USB' </td> <td> 'usb' </td> <td> '0' </td> </tr>
## <tr> <td> '-local' </td> <td> 'local' </td> <td> '0' </td> </tr>
## <tr> <td> '-all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-pattern' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-switch' </td> <td> 'switch' </td> <td> '1' </td> </tr>
## <tr> <td> '-fid' </td> <td> 'fid' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-chassis' </td> <td> 'chassis' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = configshow -all -pattern 'passwdcfg'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;passwdcfg.adminlockout:0
##&nbsp;passwdcfg.digits:0
##&nbsp;passwdcfg.history:1
##&nbsp;passwdcfg.lockoutduration:30
##&nbsp;passwdcfg.lockoutthreshold:0
##&nbsp;passwdcfg.lowercase:0
##&nbsp;passwdcfg.maxpasswordage:0
##&nbsp;passwdcfg.minlength:8
##&nbsp;passwdcfg.minpasswordage:0
##&nbsp;passwdcfg.punctuation:0
##&nbsp;passwdcfg.repeat:1
##&nbsp;passwdcfg.sequence:1
##&nbsp;passwdcfg.status:0
##&nbsp;passwdcfg.uppercase:0
##&nbsp;passwdcfg.warning:0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'passwdcfg_minlength' => '8',
##&nbsp;            'passwdcfg_status' => '0',
##&nbsp;            'passwdcfg_lockoutduration' => '30',
##&nbsp;            'passwdcfg_history' => '1',
##&nbsp;            'passwdcfg_warning' => '0',
##&nbsp;            'passwdcfg_lowercase' => '0',
##&nbsp;            'passwdcfg_sequence' => '1',
##&nbsp;            'passwdcfg_uppercase' => '0',
##&nbsp;            'passwdcfg_punctuation' => '0',
##&nbsp;            'passwdcfg_minpasswordage' => '0',
##&nbsp;            'passwdcfg_repeat' => '1',
##&nbsp;            'passwdcfg_digits' => '0',
##&nbsp;            'passwdcfg_maxpasswordage' => '0',
##&nbsp;            'passwdcfg_adminlockout' => '0',
##&nbsp;            'passwdcfg_lockoutthreshold' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = configshow -all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;[Configuration upload Information]
##&nbsp;Configuration Format = 3.0
##&nbsp;Minimum Compatible Format = 3.0
##&nbsp;Excluding Format = 0.0
##&nbsp;date = Mon Jul 30 12:34:30 2012
##&nbsp;FOS version = v7.0.0.0
##&nbsp;Number of LS = 1
##&nbsp;[Chassis Configuration Begin]
##&nbsp;
##&nbsp;[fcRouting]
##&nbsp;fcRoute.backboneFabricId:128
##&nbsp;fcRoute.fcrState:2
##&nbsp;fcRouteParam.maxLsanCount:3000
##&nbsp;fcRoute.xlate.persistxdState:1
##&nbsp;fcRouteParam.lsan.tagCnt:0
##&nbsp;
##&nbsp;[Chassis Configuration]
##&nbsp;webtools.fabricevents.enabled:1
##&nbsp;webtools.initiatorports:none
##&nbsp;webtools.licensecheck.enabled:1
##&nbsp;webtools.max.session:50
##&nbsp;webtools.max.zonetxn:50
##&nbsp;webtools.session.ttl:7200
##&nbsp;webtools.targetports:none
##&nbsp;webtools.upfrontlogin.enabled:0
##&nbsp;webtools.zonetxn.ttl:600
##&nbsp;
##&nbsp;[FCOE_CH_CONF]
##&nbsp;
##&nbsp;[UDROLE_CONF]
##&nbsp;
##&nbsp;[LicensesDB]
##&nbsp;
##&nbsp;[DMM_WWN]
##&nbsp;
##&nbsp;[Licenses]
##&nbsp;
##&nbsp;[AGWWN_MAPPING_CONF]
##&nbsp;
##&nbsp;[LicensesLservrc]
##&nbsp;
##&nbsp;[GE Blade Mode]
##&nbsp;
##&nbsp;[FWD CHASSIS CFG]
##&nbsp;
##&nbsp;[FRAME LOG]
##&nbsp;framelog.enable:1
##&nbsp;
##&nbsp;[DMM_TB]
##&nbsp;
##&nbsp;[MOTD]
##&nbsp;chassis.motd:
##&nbsp;[Chassis Configuration End]
##&nbsp;[Switch Configuration Begin : 0]
##&nbsp;SwitchName = dpg-mcc-brcd6510-01-B-fabY
##&nbsp;Fabric ID = 128
##&nbsp;
##&nbsp;[Boot Parameters]
##&nbsp;boot.name:dpg-mcc-brcd6510-01-B-fabY
##&nbsp;boot.ipa:10.228.160.157
##&nbsp;boot.mac:10:00:00:05:33:8a:21:65
##&nbsp;boot.device:eth0
##&nbsp;boot.gateway.ipa:10.228.160.1
##&nbsp;
##&nbsp;[Configuration]
##&nbsp;thresh.alarmsfilter:1
##&nbsp;ts.clockServer:LOCL
##&nbsp;ts.clockServerList:LOCL
##&nbsp;xlativeModeDisable:0
##&nbsp;zoning.check.nodeNameDisabled:0
##&nbsp;zoning.standardMode:0
##&nbsp;
##&nbsp;[Bottleneck Configuration]
##&nbsp;bottleneck.enabled:0
##&nbsp;
##&nbsp;[FCOE_SW_CONF]
##&nbsp;
##&nbsp;[Zoning]
##&nbsp;cfg.fcvi_storage_cfg_fabY:FCVI1;FCVI2;storage_zone_fabY
##&nbsp;zone.FCVI1:2,0;2,8
##&nbsp;zone.FCVI2:4,0;4,8
##&nbsp;zone.storage_zone_fabY:2,1;2,2;2,9;2,10;4,1;4,2;4,9;4,10;2,7;4,7
##&nbsp;defzone:allaccess
##&nbsp;enable:fcvi_storage_cfg_fabY
##&nbsp;
##&nbsp;[Defined Security policies]
##&nbsp;
##&nbsp;[Active Security policies]
##&nbsp;
##&nbsp;[iSCSI]
##&nbsp;
##&nbsp;[cryptoDev]
##&nbsp;
##&nbsp;[FICU SAVED FILES]
##&nbsp;
##&nbsp;[VS_SW_CONF]
##&nbsp;vs.hdr:01abcdef000000010000000000000068
##&nbsp;vs.num_entries:0
##&nbsp;
##&nbsp;[Banner]
##&nbsp;
##&nbsp;[End]
##&nbsp;[Switch Configuration End : 0]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;            'configuration_upload_info' => [
##&nbsp;                                             {
##&nbsp;                                               'fos_version' => 'v7.0.0.0',
##&nbsp;                                               'date' => 'Thu Aug  2 07:13:12 2012',
##&nbsp;                                               'excluding_format' => '0.0',
##&nbsp;                                               'number_of_ls' => '1',
##&nbsp;                                               'minimum_compatible_format' => '3.0',
##&nbsp;                                               'configuration_format' => '3.0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'chassis_configuration_info' => [
##&nbsp;                                              {
##&nbsp;                                                'frame_log' => [
##&nbsp;                                                                 {
##&nbsp;                                                                   'framelog_enable' => '1'
##&nbsp;                                                                 }
##&nbsp;                                                               ],
##&nbsp;                                                'chassis_configuration' => [
##&nbsp;                                                                             {
##&nbsp;                                                                               'webtools_upfrontlogin_enabled' => '0',
##&nbsp;                                                                               'webtools_initiatorports' => 'none',
##&nbsp;                                                                               'webtools_zonetxn_ttl' => '600',
##&nbsp;                                                                               'webtools_targetports' => 'none',
##&nbsp;                                                                               'passwdcfg_lockoutthreshold' => '0',
##&nbsp;						                               'webtools_session_ttl' => '7200',
##&nbsp;							                       'webtools_max_zonetxn' => '50',
##&nbsp;									       'webtools_max_session' => '50',
##&nbsp;                                                                             }
##&nbsp;                                                                           ],
##&nbsp;                                                'fcrouting' => [
##&nbsp;                                                                 {
##&nbsp;                                                                   'fcrouteparam_maxlsancount' => '3000',
##&nbsp;                                                                   'fcrouteparam_lsan_tagcnt' => '0',
##&nbsp;                                                                   'fcroute_fcrstate' => '2',
##&nbsp;                                                                   'fcroute_backbonefabricid' => '128',
##&nbsp;                                                                   'fcroute_xlate_persistxdstate' => '1'
##&nbsp;                                                                 }
##&nbsp;                                                               ]
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'switch_configuration_info' => [
##&nbsp;                                             {
##&nbsp;                                               'switch_configuration' => [
##&nbsp;                                                                           {
##&nbsp;                                                                             'switchname' => 'dpg-mcc-brcd6510-01-A-fabY',
##&nbsp;                                                                             'fabric_id' => '128'
##&nbsp;                                                                           }
##&nbsp;                                                                         ],
##&nbsp;                                               'vs_sw_conf' => [
##&nbsp;                                                                 {
##&nbsp;                                                                   'vs_hdr' => '01abcdef000000010000000000000068',
##&nbsp;                                                                   'vs_num_entries' => '0'
##&nbsp;                                                                 }
##&nbsp;                                                               ],
##&nbsp;                                               'boot_parameters' => [
##&nbsp;                                                                      {
##&nbsp;                                                                        'boot_gateway_ipa' => '10.228.160.1',
##&nbsp;                                                                        'boot_device' => 'eth0',
##&nbsp;                                                                        'boot_mac' => '8e',
##&nbsp;                                                                        'boot_ipa' => '10.228.160.155',
##&nbsp;                                                                        'boot_name' => 'dpg-mcc-brcd6510-01-A-fabY'
##&nbsp;                                                                      }
##&nbsp;                                                                    ],
##&nbsp;                                               'zoning' => [
##&nbsp;                                                             {
##&nbsp;                                                               'zone_fcvi1' => '2,0;2,8',
##&nbsp;                                                               'enable' => 'fcvi_storage_cfg_fabY',
##&nbsp;                                                               'defzone' => 'allaccess',
##&nbsp;                                                               'zone_fcvi2' => '4,0;4,8',
##&nbsp;                                                               'cfg_fcvi_storage_cfg_faby' => 'FCVI1;FCVI2;storage_zone_fabY',
##&nbsp;                                                               'zone_storage_zone_faby' => '2,1;2,2;2,9;2,10;4,1;4,2;4,9;4,10;2,7;4,7'
##&nbsp;                                                             }
##&nbsp;                                                           ],
##&nbsp;                                               'configuration' => [
##&nbsp;                                                                    {
##&nbsp;							              'ts_clockserverlist' => 'LOCL',
##&nbsp;							              'xlativemodedisable' => '0',
##&nbsp;                                                                      'ag_port_nsfporttopo_4' => '0x0',
##&nbsp;					                              'ag_port_nsfporttopo_45' => '0x0',
##&nbsp;                                                                      'ts_clockserver' => 'LOCL',
##&nbsp;                                                                      'switch_status_policy_percentage_errorports_down' => '0.00',
##&nbsp;                                                                    }
##&nbsp;                                                                  ],
##&nbsp;                                               'bottleneck_configuration' => [
##&nbsp;                                                                               {
##&nbsp;                                                                                 'bottleneck_enabled' => '0'
##&nbsp;                                                                               }
##&nbsp;                                                                             ]
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub configshow {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my (@result_array);
    my ( $header, $temp_header, $sub_header, $key, $value );
    my ( $begin, $conf_count ) = 0;

    for ( my $i = 1 ; $i <= $#lines ; $i++ ) {
        if ( $cmd =~ /pattern/ ) {
            if ( $lines[$i] =~ /(\S+):(\S+)/ ) {
                my ( $temp1, $temp2 ) = ( $1, $2 );
                $temp1 =~ s/\./_/g;
                $result_array[0]->{ to_key_name( item => $temp1 ) } = $temp2;
            }
        }
        else {
            if (
                $lines[$i] =~ /(\S+):(\S+)?/ | $lines[$i] =~ /(.*)\s+=\s+(.*)/ )
            {
                $key   = $1;
                $value = $2;
                $key =~ s/\./_/g;
            }
            if ( $lines[$i] =~ /\[(.*)\s+Begin/ ) {
                $temp_header = $1;
                if ( $lines[$i] =~ /\[(.*)\s+Begin\s+:\s+(\d+)/ ) {
                    $conf_count = $2;
                }
                $begin  = 1;
                $header = $temp_header . "_info";
            }
            if ( $lines[$i] =~ /\[(.*)\s+End/ ) {
                if ( $temp_header eq $1 ) {
                    $begin = 0;
                }
            }
            if ( $lines[$i] =~ /\[(.*)\]/ ) {
                $sub_header = $1;
                $sub_header =~ s/\s+Begin\s+:\s+\d+//;
                next
                  if ( $lines[ $i + 1 ] !~ /(\S+):(\S+)?/ |
                    $lines[ $i + 1 ] !~ /(.*)\s+=\s+(.*)/ );
            }
            if ($begin) {
                $result_array[0]->{ to_key_name( item => $header ) }
                  ->[$conf_count]->{ to_key_name( item => $sub_header ) }->[0]
                  ->{ to_key_name( item => $key ) } = $value
                  if ( defined $value );
            }
            else {
                $result_array[0]->{configuration_upload_info}->[0]
                  ->{ to_key_name( item => $key ) } = $value;
            }
        }
    }
    return \@result_array;
} ## end sub configshow

1;
