#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC portcfgshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch portcfgshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::portcfg
##
##  portcfg(%args)
##
## @author udaykk@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::portcfgshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /Invalid port/i) {
        $error = "InvalidParam";
    } elsif($raw_output =~ /Usage/i) {
        $error = "InvalidParamValue";
    }
    return $error;
}

## @name ::portcfgshow
##
## @summary Parses the raw output from the "portcfgshow" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from portcfgshow with any of its options as described in the 
##    corresponding cdef file 'portcfg.cdefs'
## @synopsis
##      portcfgshow(%args);
## INPUTS TO METHOD CALL 'portcfgshow' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'slot-number' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'port-number' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = portcfgshow 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Area Number:              1
##&nbsp;Speed Level:              2G
##&nbsp;Fill Word:                0(Idle-Idle)
##&nbsp;AL_PA Offset 13:          OFF
##&nbsp;Trunk Port                ON
##&nbsp;Long Distance             OFF
##&nbsp;VC Link Init              OFF
##&nbsp;Locked L_Port             OFF
##&nbsp;Locked G_Port             OFF
##&nbsp;Disabled E_Port           OFF
##&nbsp;ISL R_RDY Mode            OFF
##&nbsp;RSCN Suppressed           OFF
##&nbsp;Persistent Disable        OFF
##&nbsp;NPIV capability           ON
##&nbsp;QOS E_Port                OFF
##&nbsp;Port Auto Disable:        OFF
##&nbsp;F_Port Buffers            OFF
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'npiv_capability' => 'ON',
##&nbsp;            'speed_level' => '2G',
##&nbsp;            'al_pa_offset_13' => 'OFF',
##&nbsp;            'rscn_suppressed' => 'OFF',
##&nbsp;            'vc_link_init' => 'OFF',
##&nbsp;            'area_number' => '1',
##&nbsp;            'fill_word' => '0(Idle-Idle)',
##&nbsp;            'disabled_e_port' => 'OFF',
##&nbsp;            'trunk_port' => 'ON',
##&nbsp;            'isl_r_rdy_mode' => 'OFF',
##&nbsp;            'long_distance' => 'OFF',
##&nbsp;            'locked_l_port' => 'OFF',
##&nbsp;            'qos_e_port' => 'OFF',
##&nbsp;            'f_port_buffers' => 'OFF',
##&nbsp;            'locked_g_port' => 'OFF',
##&nbsp;            'persistent_disable' => 'OFF',
##&nbsp;            'port_auto_disable' => 'OFF'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub portcfgshow {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command = $response->get_command();
    my $error_level = $response->get_execution_status();
    my @result_array = ();

    my $error = _check_error("raw_output"=>$raw_output,
                                "command"=>"portcfgshow",
                                "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my ($line, $count, $ports, $value);
    my (@abcd, @array2, @data, @key, @temp);
    my @output = get_lines(raw_output=>$raw_output, response_object=>$response);
    my $i = -1;
    my %info;
    my $s = 0;
    foreach $line (@output){
        if ($line =~ /Ports of Slot (\d+)/) {
            $array2[$s] = $1;
            @abcd = split(/\s+/, $line);
            $ports = @abcd;
            $ports -= 4;
            $s++;
            $i++;
        } elsif($line =~ /^([-+]+)?$/) {
            #discard blank and unwanted lines
        } elsif ($line =~ /Area Number.*/i) {
            my %info1;
            my @array2;
            my $val1;
            foreach $val1 (@output) {
                if ($val1 =~ /(\w+(\s\w+)*(\s\d+)*):?\s+((\d+)?(\w+)?(\(Idle-Idle\))?)/ ) {
                    my @key = split(/ /,$1);
                    my $key = join("_", @key);
                    $key =~ tr/[A-Z]/[a-z]/;
                    $info1{$key} = $4;
                }
            }
            push(@array2,\%info1);
            return \@array2;
        } else {
            my @data1 = split(/\s+/, $line);
            my $length = @data1;
            $a = $length - $ports;
            @data = @data1[$a..$length];
            @key = splice(@data1,0,$a);
            my $key = join("_", @key);
            if($line =~ /Persistent Disable((\w|[.])+)/) {
                $key = "persistent_disable";
                $data[0] = $1;
            }
            for(my $j = 0; $j < $ports; $j++) {
                $key =~ tr/[A-Z]/[a-z]/;
                $temp[$i]->[$j]->{$key} = $data[$j];
                $count++;
            }
        }
    }
    #my @x =  keys %{$temp[0]->[1]};
    my @array1;
    my $val;
    foreach $val (@array2) {
        my $k = 0;
        for(my $j=0;$j<=15;$j++) {
            my %hash;
            $hash{ports_of_slots} = $val;
            $hash{area_number} = $j;
            $hash{speed} = $temp[$k]->[$j]->{speed};
            $hash{fill_word} = $temp[$k]->[$j]->{fill_word};
            $hash{al_pa_offset} = $temp[$k]->[$j]->{al_pa_offset_13};
            $hash{trunk_port} = $temp[$k]->[$j]->{trunk_port};
            $hash{long_distance} = $temp[$k]->[$j]->{long_distance};
            $hash{vc_link_init} = $temp[$k]->[$j]->{vc_link_init};
            $hash{locked_l_port} = $temp[$k]->[$j]->{locked_l_port};
            $hash{locked_g_port} = $temp[$k]->[$j]->{locked_g_port};
            $hash{disabled_e_port} = $temp[$k]->[$j]->{disabled_e_port};
            $hash{isl_r_rdy_mode} = $temp[$k]->[$j]->{isl_r_rdy_mode};
            $hash{rscn_suppressed} = $temp[$k]->[$j]->{rscn_suppressed};
            $hash{persistent_disable} = $temp[$k]->[$j]->{persistent_disable};
            $hash{npiv_capability} = $temp[$k]->[$j]->{npiv_capability};
            $hash{qos_e_port} = $temp[$k]->[$j]->{qos_e_port};
            $hash{fport_buffers} = $temp[$k]->[$j]->{fport_buffers};
            $hash{port_auto_disable} = $temp[$k]->[$j]->{port_auto_disable};
            push(@array1,\%hash);
        }
        $k++;
    }
   return  \@array1;
}
1;
