#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/portloginshow.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC  portloginshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch  portloginshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::portloginshow
##
##   portloginshow(%args)
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::portloginshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my ( $error_message, $error );
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /Usage:|Invalid input parameters/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::portloginshow
##
## @summary Parses the raw output from the "portloginshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portloginshow with any of its options as described in the
##    corresponding cdef file 'portloginshow.cdefs'
## @synopsis
##      portloginshow(%args);
## INPUTS TO METHOD CALL 'portloginshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = portloginshow 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;Type  PID     World Wide Name        credit df_sz cos
##&nbsp;=====================================================
##&nbsp;  fe  040000 21:00:00:24:ff:32:06:92     5  2048   8  scr=0x3
##&nbsp;  ff  040000  21:00:00:24:ff:32:06:92     0     0   8  d_id=FFFFFA
##&nbsp;  ff  040000  21:00:00:24:ff:32:06:92     0     0   8  d_id=FFFFFC
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'world_wide_name' => '21:00:00:24:ff:32:06:92',
##&nbsp;            'state_change_registration' => '0x3',
##&nbsp;            'pid' => '040000',
##&nbsp;            'credit' => '5',
##&nbsp;            'class_of_service' => '8',
##&nbsp;            'type' => 'fe',
##&nbsp;            'default_frame_size' => '2048'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'world_wide_name' => '21:00:00:24:ff:32:06:92',
##&nbsp;            'pid' => '040000',
##&nbsp;            'credit' => '0',
##&nbsp;            'class_of_service' => '8',
##&nbsp;            'type' => 'ff',
##&nbsp;            'domain_id' => 'FFFFFA',
##&nbsp;            'default_frame_size' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'world_wide_name' => '21:00:00:24:ff:32:06:92',
##&nbsp;            'pid' => '040000',
##&nbsp;            'credit' => '0',
##&nbsp;            'class_of_service' => '8',
##&nbsp;            'type' => 'ff',
##&nbsp;            'domain_id' => 'FFFFFC',
##&nbsp;            'default_frame_size' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub portloginshow {
    my %args = @_;
    my (@result_array);
    my $resp        = $args{response_object};
    my $output      = $resp->get_raw_output();
    my $error_level = $resp->get_execution_status();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my $count = -1;
    my ( @head, @values );

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /^Type/ ) {
            $lines[$i] =~ s/World Wide Name/world_wide_name/;
            $lines[$i] =~ s/df_sz/default_frame_size/;
            $lines[$i] =~ s/cos/class_of_service/;
            @head = split( /\s+/, $lines[$i] );
        }
        if ( $lines[$i] =~ /^\w+\s+\d+\s+\d/ ) {
            @values = split( /\s+/, $lines[$i] );
            $count++;
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$count]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
            if ( $#head != $#values ) {
                my @temp = split( "=", $values[-1] );
                $temp[0] =~ s/scr/state_change_registration/;
                $temp[0] =~ s/d_id/domain_id/;
                $result_array[$count]->{ to_key_name( item => $temp[0] ) } = $temp[1];
            }
        }
    }
    return \@result_array;
} ## end sub portloginshow

1;
