#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/portstatsshow.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC portstatsshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch portstatsshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::portstatsshow
##
##  portstatsshow(%args)
##
## @author udaykk@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::portstatsshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error 
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(Invalid input options)|(Valid Port Range 0-47)/i ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::portstatsshow
##
## @summary Parses the raw output from the "portstatsshow" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from portstatsshow with any of its options as described in the 
##    corresponding cdef file 'portstatsshow.cdefs'
## @synopsis
##      portstatsshow(%args);
## INPUTS TO METHOD CALL 'portstatsshow' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-number' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = portstatsshow 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tim' => [
##&nbsp;                       {
##&nbsp;                         'status' => 'Time R_RDY high priority',
##&nbsp;                         'port' => 'tim_rdy_pri',
##&nbsp;                         'num_bytes_transferred' => '0'
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'status' => 'Time TX Credit Zero (2.5Us ticks)',
##&nbsp;                         'port' => 'tim_txcrd_z',
##&nbsp;                         'num_bytes_transferred' => '61660230'
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'tim_txcrd_z_vc_0__3' => [
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '61660230'
##&nbsp;                                                  ]
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'tim_txcrd_z_vc_4__7' => [
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '0'
##&nbsp;                                                  ]
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'tim_txcrd_z_vc_8_11' => [
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '0',
##&nbsp;                                                    '0'
##&nbsp;                                                  ]
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'tim_txcrd_z_vc_12_15' => [
##&nbsp;                                                     '0',
##&nbsp;                                                     '0',
##&nbsp;                                                     '0',
##&nbsp;                                                     '0'
##&nbsp;                                                   ]
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'er' => [
##&nbsp;                      {
##&nbsp;                        'status' => 'Encoding errors inside of frames',
##&nbsp;                        'port' => 'er_enc_in',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames with CRC errors',
##&nbsp;                        'port' => 'er_crc',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames shorter than minimum',
##&nbsp;                        'port' => 'er_trunc',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames longer than maximum',
##&nbsp;                        'port' => 'er_toolong',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames with bad end-of-frame',
##&nbsp;                        'port' => 'er_bad_eof',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Encoding error outside of frames',
##&nbsp;                        'port' => 'er_enc_out',
##&nbsp;                        'num_bytes_transferred' => '23046'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Invalid ordered set',
##&nbsp;                        'port' => 'er_bad_os',
##&nbsp;                        'num_bytes_transferred' => '12269'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Class 3 receive frames discarded due to timeout',
##&nbsp;                        'port' => 'er_rx_c3_timeout',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Class 3 transmit frames discarded due to timeout',
##&nbsp;                        'port' => 'er_tx_c3_timeout',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames that are unroutable',
##&nbsp;                        'port' => 'er_unroutable',
##&nbsp;                        'num_bytes_transferred' => '158'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Frames with unreachable destination',
##&nbsp;                        'port' => 'er_unreachable',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Other discards',
##&nbsp;                        'port' => 'er_other_discard',
##&nbsp;                        'num_bytes_transferred' => '158'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'frames with FTB type 1 miss',
##&nbsp;                        'port' => 'er_type1_miss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'frames with FTB type 2 miss',
##&nbsp;                        'port' => 'er_type2_miss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'frames with FTB type 6 miss',
##&nbsp;                        'port' => 'er_type6_miss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'frames with hard zoning miss',
##&nbsp;                        'port' => 'er_zone_miss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'frames with LUN zoning miss',
##&nbsp;                        'port' => 'er_lun_zone_miss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Crc error with good eof',
##&nbsp;                        'port' => 'er_crc_good_eof',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Invalid ARB',
##&nbsp;                        'port' => 'er_inv_arb',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Single vcrdy/frame loss on link',
##&nbsp;                        'port' => 'er_single_credit_loss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      },
##&nbsp;                      {
##&nbsp;                        'status' => 'Multiple vcrdy/frame loss on link',
##&nbsp;                        'port' => 'er_multi_credit_loss',
##&nbsp;                        'num_bytes_transferred' => '0'
##&nbsp;                      }
##&nbsp;                    ],
##&nbsp;            'stat' => [
##&nbsp;                        {
##&nbsp;                          'status' => '4-byte words transmitted',
##&nbsp;                          'port' => 'stat_wtx',
##&nbsp;                          'num_bytes_transferred' => '1151128252'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => '4-byte words received',
##&nbsp;                          'port' => 'stat_wrx',
##&nbsp;                          'num_bytes_transferred' => '2520176991'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Frames transmitted',
##&nbsp;                          'port' => 'stat_ftx',
##&nbsp;                          'num_bytes_transferred' => '3664314116'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Frames received',
##&nbsp;                          'port' => 'stat_frx',
##&nbsp;                          'num_bytes_transferred' => '37866810'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Class 2 frames received',
##&nbsp;                          'port' => 'stat_c2_frx',
##&nbsp;                          'num_bytes_transferred' => '0'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Class 3 frames received',
##&nbsp;                          'port' => 'stat_c3_frx',
##&nbsp;                          'num_bytes_transferred' => '37866810'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Link control frames received',
##&nbsp;                          'port' => 'stat_lc_rx',
##&nbsp;                          'num_bytes_transferred' => '0'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Multicast frames received',
##&nbsp;                          'port' => 'stat_mc_rx',
##&nbsp;                          'num_bytes_transferred' => '0'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Multicast timeouts',
##&nbsp;                          'port' => 'stat_mc_to',
##&nbsp;                          'num_bytes_transferred' => '0'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'status' => 'Multicast frames transmitted',
##&nbsp;                          'port' => 'stat_mc_tx',
##&nbsp;                          'num_bytes_transferred' => '0'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub portstatsshow {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => "portstatsshow",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines(raw_output=>$output);

    my (@stat, @tim, @er);
    foreach my $line (@lines) {
        my %info;
        if ($line =~ /(tim.*)(\s+)?:\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/) {
            my @array;
            my $var1 = $1;
            push(@array, $3,$4,$5,$6);
            $var1 =~ s/\s+/_/g;
            $var1 =~ s/-/_/g;
            $info{$var1} = \@array;
            push(@tim, \%info);
        } elsif($line =~ /(stat\w+)\s+(\d+)\s+(.*)/) {
            $info{"port"} = $1;
            $info{"num_bytes_transferred"} = $2;
            $info{"status"} = $3;
            push(@stat, \%info);
        } elsif($line =~ /(er\w+)\s+(\d+)\s+(.*)/) {
            $info{"port"} = $1;
            $info{"num_bytes_transferred"} = $2;
            $info{"status"} = $3;
            push(@er, \%info);
        } elsif($line =~ /(tim\w+)\s+(\d+)\s+(.*)/) {
            $info{"port"} = $1;
            $info{"num_bytes_transferred"} = $2;
            $info{"status"} = $3;
            push(@tim, \%info);
        }
    }

    my %info1;
    $info1{"stat"} = \@stat;
    $info1{"er"} = \@er;
    $info1{"tim"} = \@tim;
    push(@result_array, \%info1);
    return \@result_array;
} ## end sub portstatsshow
1;
