#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/sfpshow.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC sfpshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch sfpshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::sfpshow
##
##  sfpshow(%args)
##
## @author udaykk@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::sfpshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ($raw_output =~ /Usage:(.*)/) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::sfpshow
##
## @summary Parses the raw output from the "sfpshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sfpshow with any of its options as described in the
##    corresponding cdef file 'sfpshow.cdefs'
## @synopsis
##      sfpshow(%args);
## INPUTS TO METHOD CALL 'sfpshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-number' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = sfpshow 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;Identifier:  3    SFP
##&nbsp;Connector:   7    LC
##&nbsp;Transceiver: 7004404000000000 4,8,16_Gbps M5 sw Short_dist
##&nbsp;Encoding:    6    64B66B
##&nbsp;Baud Rate:   140  (units 100 megabaud)
##&nbsp;Length 9u:   0    (units km)
##&nbsp;Length 9u:   0    (units 100 meters)
##&nbsp;Length 50u:  3    (units 10 meters)
##&nbsp;Length 62.5u:0    (units 10 meters)
##&nbsp;Length Cu:   0    (units 1 meter)
##&nbsp;Vendor Name: BROCADE
##&nbsp;Vendor OUI:  00:05:1e
##&nbsp;Vendor PN:   57-0000088-01
##&nbsp;Vendor Rev:  A
##&nbsp;Wavelength:  850  (units nm)
##&nbsp;Options:     003a Loss_of_Sig,Tx_Fault,Tx_Disable
##&nbsp;BR Max:      0
##&nbsp;BR Min:      0
##&nbsp;Serial No:   HAF11130000042A
##&nbsp;Date Code:   110717
##&nbsp;DD Type:     0x68
##&nbsp;Enh Options: 0xfa
##&nbsp;Status/Ctrl: 0xb0
##&nbsp;Pwr On Time: 0.62 years (5392 hours)
##&nbsp;E-Wrap Control: 0
##&nbsp;O-Wrap Control: 0
##&nbsp;Alarm flags[0,1] = 0x5, 0x0
##&nbsp;Warn Flags[0,1] = 0x5, 0x0
##&nbsp;                                          Alarm                  Warn
##&nbsp;                                   low        high       low         high
##&nbsp;Temperature: 46      Centigrade    -5          85         0           75
##&nbsp;Current:     8.230   mAmps          2.500      12.000     2.000       11.500
##&nbsp;Voltage:     3287.3  mVolts         3000.0      3600.0    3100.0       3500.0
##&nbsp;RX Power:    -3.7    dBm (423.0uW) 31.6   uW 1258.9 uW   31.6   uW  794.3  uW
##&nbsp;TX Power:    -2.9    dBm (517.2 uW)125.9  uW   1258.9 uW  251.2  uW   794.3  uW
##&nbsp;
##&nbsp;State transitions: 24
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'alarm_flags_0_1' => '0x5, 0x0',
##&nbsp;            'vendor_name' => 'BROCADE',
##&nbsp;            'pwr_on_time' => '0.62 years (5392 hours)',
##&nbsp;            'date_code' => '110717',
##&nbsp;            'identifier' => '3    SFP',
##&nbsp;            'wavelength' => '850  (units nm)',
##&nbsp;            'o_wrap_control' => '0',
##&nbsp;            'status_ctrl' => '0xb0',
##&nbsp;            'encoding' => '6    64B66B',
##&nbsp;            'length_50u' => '3    (units 10 meters)',
##&nbsp;            'warn_flags_0_1' => '0x5, 0x0',
##&nbsp;            'transceiver' => '7004404000000000 4,8,16_Gbps M5 sw Short_dist',
##&nbsp;            'voltage' => [
##&nbsp;                           {
##&nbsp;                             'voltage' => '3287.3 mVolts',
##&nbsp;                             'warn_low' => '3100.0',
##&nbsp;                             'alarm_low' => '3000.0',
##&nbsp;                             'warn_high' => '3500.0',
##&nbsp;                             'alarm_high' => '3600.0'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'enh_options' => '0xfa',
##&nbsp;            'length_cu' => '0    (units 1 meter)',
##&nbsp;            'br_max' => '0',
##&nbsp;            'baud_rate' => '140  (units 100 megabaud)',
##&nbsp;            'vendor_rev' => 'A',
##&nbsp;            'options' => '003a Loss_of_Sig,Tx_Fault,Tx_Disable',
##&nbsp;            'e_wrap_control' => '0',
##&nbsp;            'serial_no' => 'HAF11130000042A',
##&nbsp;            'length_9u' => '0    (units 100 meters)',
##&nbsp;            'vendor_pn' => '57-0000088-01',
##&nbsp;            'tx_power' => [
##&nbsp;                            {
##&nbsp;                              'warn_low' => '251.2 uW',
##&nbsp;                              'alarm_low' => '125.9 uW',
##&nbsp;                              'warn_high' => '794.3 uW',
##&nbsp;                              'tx_power' => '-2.9    dBm (517.2 uW)',
##&nbsp;                              'alarm_high' => '1258.9uW'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'vendor_oui' => '1e',
##&nbsp;            'current' => [
##&nbsp;                           {
##&nbsp;                             'warn_low' => '2.000',
##&nbsp;                             'current' => '8.230 mAmps',
##&nbsp;                             'alarm_low' => '2.500',
##&nbsp;                             'warn_high' => '11.500',
##&nbsp;                             'alarm_high' => '12.000'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'connector' => '7    LC',
##&nbsp;            'dd_type' => '0x68',
##&nbsp;            'state_transitions' => '24',
##&nbsp;            'length_62.5u' => '0    (units 10 meters)',
##&nbsp;            'br_min' => '0',
##&nbsp;            'rx_power' => [
##&nbsp;                            {
##&nbsp;                              'rx_power' => '-3.7    dBm (423.0uW)',
##&nbsp;                              'warn_low' => '794.3 uW',
##&nbsp;                              'alarm_low' => '1258.9uW',
##&nbsp;                              'warn_high' => 'uW',
##&nbsp;                              'alarm_high' => '31.6  uW'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'temparature' => [
##&nbsp;                               {
##&nbsp;                                 'warn_low' => '0',
##&nbsp;                                 'alarm_low' => '-5',
##&nbsp;                                 'warn_high' => '75',
##&nbsp;                                 'alarm_high' => '85',
##&nbsp;                                 'temperature' => '46 Centigrade'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sfpshow {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output    => $output,
                          command       => "sfpshow",
                          error_message => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @lines = get_lines(raw_output => $output);
    shift @lines;
    my %info1;
    foreach my $line (@lines) {
        my @array;
        my %info;
        if ($line =~
            /Temperature:\s+(.?\d+)\s+(\w+)\s+(.?\d+)\s+(.?\d+)\s+(.?\d+)\s+(.?\d+)/
          ) {
            $info{"temperature"} = $1 . " " . $2;
            $info{"alarm_low"}   = $3;
            $info{"alarm_high"}  = $4;
            $info{"warn_low"}    = $5;
            $info{"warn_high"}   = $6;
            push(@array, \%info);
            $info1{"temparature"} = \@array;
        } elsif ($line =~
            /Current:\s+(\d+.?\d+)\s+(\w+)\s+(\d+.?\d+)\s+(\d+.?\d+)\s+(\d+.?\d+)\s+(\d+.?\d+)/
          ) {
            $info{"current"}    = $1 . " " . $2;
            $info{"alarm_low"}  = $3;
            $info{"alarm_high"} = $4;
            $info{"warn_low"}   = $5;
            $info{"warn_high"}  = $6;
            push(@array, \%info);
            $info1{"current"} = \@array;
        } elsif ($line =~
            /Voltage:\s+(\d+.?\d+)\s+(\w+)\s+(\d+.?\d+)\s+(\d+.?\d+)\s+(\d+.?\d+)\s+(\d+.?\d+)/
          ) {
            $info{"voltage"}    = $1 . " " . $2;
            $info{"alarm_low"}  = $3;
            $info{"alarm_high"} = $4;
            $info{"warn_low"}   = $5;
            $info{"warn_high"}  = $6;
            push(@array, \%info);
            $info1{"voltage"} = \@array;
        } elsif ($line =~
            /RX Power:\s+(.*\s+dBm\s+\(.*\))\s+(.*)\s+uW\s+(.*)\s+uW\s+(.*)\s+uW\s+(.*)\s+uW/
          ) {
            $info{"rx_power"}   = $1;
            $info{"alarm_low"}  = $3 . "uW";
            $info{"alarm_high"} = $4 . "uW";
            $info{"warn_low"}   = $5 . "uW";
            $info{"warn_high"}  = $6 . "uW";
            push(@array, \%info);
            $info1{"rx_power"} = \@array;
        } elsif ($line =~
            /TX Power:\s+(.*\s+dBm\s+\(.*\)(\s+)?)(.*)\s+uW\s+(.*)\s+uW\s+(.*)\s+uW\s+(.*)\s+uW/
          ) {
            $info{"tx_power"}   = $1;
            $info{"alarm_low"}  = $3 . "uW";
            $info{"alarm_high"} = $4 . "uW";
            $info{"warn_low"}   = $5 . "uW";
            $info{"warn_high"}  = $6 . "uW";
            push(@array, \%info);
            $info1{"tx_power"} = \@array;
        } elsif ($line =~
            /Port\s+(\d+):\s+id\s+\(sw\)\s+Vendor:\s+(.*)\s+Serial\s+No:\s+(.*)\s+Speed:\s+(.*)/
          ) {
            $info{"port_number"}   = $1;
            $info{"vendor"}        = $2;
            $info{"serial_number"} = $3;
            $info{"speed"}         = $4;
            push(@array, \%info);
            $info1{"port"} = \@array;
        } elsif ($line =~ /(.*)[:|=](.*)/) {
            my $var1 = $1;
            my $var2 = $2;
            $var1 =~ s/\]//;
            if ($var1 =~ /\s$/) {
                chop($var1);
            }
            $var1 =~ s/[\/\[\,\-]/_/g;
            $var1 = to_key_name(item => $var1);
            $var2 =~ s/^\s+//g;
            $info1{$var1} = $var2;
        } ## end elsif ($line =~ /(.*)[:|=](.*)/)
    } ## end foreach my $line (@lines)
    push(@result_array, \%info1);
    return \@result_array;
} ## end sub sfpshow
1;
