#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC tsTimeZone command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch tsTimeZone command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::tsTimeZone
##
##  tsTimeZone(%args)
##
## @author udaykk@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::tsTimeZone;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /(Invalid timezone)/) {
        $error = "InvalidParam";
    }
    return $error;
}

## @name ::tsTimeZone
##
## @summary Parses the raw output from the "tsTimeZone" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tsTimeZone with any of its options as described in the
##    corresponding cdef file 'tsTimeZone.cdefs'
## @synopsis
##      tsTimeZone(%args);
## INPUTS TO METHOD CALL 'tsTimeZone' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'timezonename' </td> </tr>
## <tr> <td> '--interactive' </td> <td> 'interactive' </td> </tr>
## <tr> <td> '--old' </td> <td> 'old' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = tsTimeZone
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_zone' => 'Africa/Algiers'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub tsTimeZone {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command = $response->get_command();
    my $error_level = $response->get_execution_status();
    my @result_array = ();

    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>"tsTimeZone",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    my $value;
    my @lines = get_lines(raw_output=>$raw_output, response_object=>$response);
    my %info;
    foreach $value (@lines) {
        if($value =~ /(System\s+Time\s+Zone\s+change\s+will\s+take\s+effect\s+at\s+next\s+reboot.*)|(This command now requires a timezone name, rather than an offset.*)/i) {
            $info{"status"} = $1;
        } elsif($value =~ /Time\s+Zone\s+Hour\s+Offset:\s+(.?\d+).*/i) {
            $info{"time_zone_hour_offset"} = $1;
        } elsif($value =~ /Time\s+Zone\s+Minute\s+Offset:\s+(.?\d+).*/i) {
            $info{"time_zone_minute_offset"} = $1;
        } elsif($value =~ /(\w+\/\w+)/i) {
            $info{"time_zone"} = $1;
        }
    }
    push(@result_array, \%info) if(%info) ;
    return \@result_array;
}

1;
