#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC zoneshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch zoneshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::zoneshow
##
##  zoneshow(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::zoneshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if (($raw_output =~ /$command.*:\s(.*)/) or ($raw_output =~ /([Ii]nvalid\s.*)$/) or ($raw_output =~ /(does not exist)/) or ($raw_output =~ /(error: Usage:)/i)) {
        $$error_message = $1;
        if($$error_message =~ /([Ii]nvalid option)|([Uu]nrecognized option)|([Ii]nvalid input)|(does not exist)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(can't find)/) {
            $error = "InvalidParamValue";
        } elsif($$error_message =~ /(command not found)|(error: Usage:)/i) {
            $error = "CommandFailed";
        }
    }
    return $error;
}

## @name ::zoneshow
##
## @summary Parses the raw output from the "zoneshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zoneshow with any of its options as described in the
##    corresponding cdef file 'zoneshow.cdefs'
## @synopsis
##      zoneshow(%args);
## INPUTS TO METHOD CALL 'zoneshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'zone-name' </td> </tr>
## <tr> <td> '--sort' </td> <td> 'sort' </td> </tr>
## <tr> <td> ',' </td> <td> 'mode' </td> </tr>
## <tr> <td> '--help' </td> <td> 'help' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = zoneshow
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Defined configuration:
##&nbsp; cfg:	SANLinux
##&nbsp;		Raj_178_24_25_1; Raj_178_24_25_2; Raj_179_24_25_1;
##&nbsp;		Raj_179_24_25_2; SAC_105_0a; SAC_105_0b; SAC_107_0a;
##&nbsp;		SAC_107_0b; SAC_113_0a; SAC_113_0b; SAC_114_0a; SAC_114_0b;
##&nbsp;		SAC_143_0a; SAC_143_0b; SAC_144_0a; SAC_144_0b; SAC_174_0a;
##&nbsp;		SAC_174_0b; sub_149_a; sub_149_b
##&nbsp; zone:	Raj_178_24_25_1
##&nbsp;		10:00:00:00:c9:d0:88:ba; 50:0a:09:81:87:59:82:bb;
##&nbsp;		50:0a:09:81:97:59:82:bb; 50:0a:09:85:87:59:82:bb;
##&nbsp;		50:0a:09:85:97:59:82:bb
##&nbsp;Effective configuration:
##&nbsp; cfg:	SANLinux
##&nbsp; zone:	Raj_178_24_25_1
##&nbsp;		10:00:00:00:c9:d0:88:ba
##&nbsp;		50:0a:09:81:87:59:82:bb
##&nbsp;		50:0a:09:81:97:59:82:bb
##&nbsp;		50:0a:09:85:87:59:82:bb
##&nbsp;		50:0a:09:85:97:59:82:bb
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'defined_cfg' => [
##&nbsp;                               {
##&nbsp;                                 'name' => 'SANLinux',
##&nbsp;                                 'zones' => [
##&nbsp;                                              'Raj_178_24_25_1',
##&nbsp;                                              'Raj_178_24_25_2',
##&nbsp;                                              'Raj_179_24_25_2',
##&nbsp;                                              'SAC_105_0a',
##&nbsp;                                              'SAC_105_0b',
##&nbsp;                                              'SAC_107_0b',
##&nbsp;                                              'SAC_113_0a',
##&nbsp;                                              'SAC_113_0b',
##&nbsp;                                              'SAC_114_0a',
##&nbsp;                                              'SAC_143_0a',
##&nbsp;                                              'SAC_143_0b',
##&nbsp;                                              'SAC_144_0a',
##&nbsp;                                              'SAC_144_0b',
##&nbsp;                                              'SAC_174_0b',
##&nbsp;                                              'sub_149_a'
##&nbsp;                                            ]
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'effective_cfg' => [
##&nbsp;                                 {
##&nbsp;                                   'name' => 'SANLinux',
##&nbsp;                                   'zones' => [
##&nbsp;                                                'Raj_178_24_25_1',
##&nbsp;                                                'Raj_178_24_25_2',
##&nbsp;                                                'Raj_179_24_25_1',
##&nbsp;                                                'Raj_179_24_25_2',
##&nbsp;                                                'SAC_105_0a',
##&nbsp;                                                'SAC_105_0b',
##&nbsp;                                                'SAC_107_0a',
##&nbsp;                                                'SAC_107_0b',
##&nbsp;                                                'SAC_113_0a',
##&nbsp;                                                'SAC_113_0b',
##&nbsp;                                                'SAC_114_0a',
##&nbsp;                                                'SAC_114_0b',
##&nbsp;                                                'SAC_143_0a',
##&nbsp;                                                'SAC_143_0b',
##&nbsp;                                                'SAC_144_0a',
##&nbsp;                                                'SAC_144_0b',
##&nbsp;                                                'SAC_174_0a',
##&nbsp;                                                'SAC_174_0b',
##&nbsp;                                                'sub_149_a',
##&nbsp;                                                'sub_149_b'
##&nbsp;                                              ]
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'members' => [
##&nbsp;                                          '10:00:00:00:c9:d0:88:ba',
##&nbsp;                                          '50:0a:09:81:97:59:82:bb',
##&nbsp;                                          '50:0a:09:85:97:59:82:bb'
##&nbsp;                                        ],
##&nbsp;                           'name' => 'Raj_178_24_25_1'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub zoneshow {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output=>$output,command=>"zoneshow",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    my %hash;
    if ($command =~ /^zoneshow$/ or $command =~ /^zoneshow\s+--sort$/ or $command =~ /^cfgshow$/) {
        my @lines = get_lines(raw_output=>$output,first_line=>'Defined\s+configuration:');
        return [] if !@lines;

        # get rid of first line
        shift(@lines);

        # Reading defined configuration
        my (@cfg_members, $config, @defined_conf_arr);
        my $line = shift(@lines);
        while ($line !~ /zone:/ and (scalar(@lines) > 0)) {
            if ($line =~ /cfg:\s+/) {
                if (scalar(@cfg_members) > 0) {
                    my %cfg;
                    push(@{$cfg{'zones'}}, @cfg_members);
                    $cfg{'name'} = $config;
                    push(@defined_conf_arr, \%cfg);

                    @cfg_members = ();
                }
                my @tmp_arr = split (/\s+/, $line);
                $config = $tmp_arr[1];
                if ( scalar(@tmp_arr) > 2 ) {
                    if ($tmp_arr[2] =~ /\;/) {
                        @cfg_members = split(/\;\s*/, $tmp_arr[2]);
                    } else {
                        push(@cfg_members, $tmp_arr[2]);
                    }
                }
            } else {
                if ($line =~ /\;/) {
                    my @tmp_cfg_members = split(/\;\s*/, $line);
                    push(@cfg_members, @tmp_cfg_members);
                } else {
                    push(@cfg_members, $line);
                }
            }
            $line = shift(@lines);;
        }
        my %tmp_cfg;
        $tmp_cfg{'name'} = $config;
        push(@{$tmp_cfg{'zones'}}, @cfg_members);
        push(@defined_conf_arr, \%tmp_cfg);
        $hash{'defined_cfg'} = \@defined_conf_arr;

        # Reading zones info
        my (@zones, @zone_members, $zone_name);
        while ($line !~ /alias:/ and (scalar(@lines) > 0)){
            if ($line =~ /zone:\s+/) {
                if (scalar(@zone_members) > 0) {
                    my %zone;
                    $zone{'name'} = $zone_name;
                    push(@{$zone{'members'}}, @zone_members);
                    push(@zones, \%zone);
                    @zone_members = ();
                }
                my @tmp_arr = split(/\s+/, $line);
                $zone_name = $tmp_arr[1];
                if ( scalar(@tmp_arr) > 2 ) {
                    for (my $mem = 2; $mem < scalar(@tmp_arr); $mem++) {
                        if ($tmp_arr[$mem] =~ /\;/) {
                            push(@zone_members, split(/\;\s*/, $tmp_arr[$mem]));
                        } else {
                            push(@zone_members, $tmp_arr[$mem]);
                        }
                    }
                }
            } elsif ($line =~ /\;/) {
                my @tmp_zone_members = split(/\;\s*/, $line);
                push(@zone_members, @tmp_zone_members);
            } else {
                push(@zone_members, $line);
            }
            $line = shift(@lines);
        }
        my %tmp_zone;
        $tmp_zone{'name'} = $zone_name;
        push(@{$tmp_zone{'members'}}, @zone_members);
        push(@zones, \%tmp_zone);
        $hash{'zones'} = \@zones;

        # Reading alias info
        my (@aliases, @alias_members, $alias_name);
        while ($line !~ /Effective configuration:/i and (scalar(@lines) > 0)){
            if ($line =~ /alias:\s+/) {
                if (scalar(@alias_members) > 0) {
                    my %alias;
                    $alias{'name'} = $alias_name;
                    push(@{$alias{'members'}}, @alias_members);
                    push(@aliases, \%alias);
                    @alias_members = ();
                }
                my @tmp_arr = split(/\s+/, $line);
                $alias_name = $tmp_arr[1];
                if ( scalar(@tmp_arr) > 2 ) {
                    for (my $mem = 2; $mem < scalar(@tmp_arr); $mem++) {
                        if ($tmp_arr[$mem] =~ /\;/) {
                            push(@alias_members, split(/\;\s*/, $tmp_arr[$mem]));
                        } else {
                            push(@alias_members, $tmp_arr[$mem]);
                        }
                    }
                }
            } elsif ($line =~ /\;/) {
                my @tmp_alias_members = split(/\;\s*/, $line);
                push(@alias_members, @tmp_alias_members);
            } else {
                push(@alias_members, $line);
            }
            $line = shift(@lines);
        }
        my %tmp_alias;
        $tmp_alias{'name'} = $alias_name;
        push(@{$tmp_alias{'members'}}, @alias_members);
        push(@aliases, \%tmp_alias);
        $hash{'aliases'} = \@aliases;

        # Reading effective configuration
        @cfg_members = ();
        my (@effective_conf_arr);
        while(@lines) {
            my $line = shift(@lines);
            if ($line =~ /cfg:\s+(\S+)/) {
                if (scalar(@cfg_members) > 0 ) {
                    my %cfg;
                    $cfg{'name'} = $config;
                    push(@{$cfg{'zones'}}, @cfg_members);
                    push(@effective_conf_arr, \%cfg);
                    @cfg_members = ();
                }
                $config = $1;
            } elsif ($line =~ /zone:\s+(\S+)/) {
                push(@cfg_members, $1);
            }
        }
        my %tmp_cfg2;
        $tmp_cfg2{'name'} = $config;
        push(@{$tmp_cfg2{'zones'}}, @cfg_members);
        push(@effective_conf_arr, \%tmp_cfg2);
        $hash{'effective_cfg'} = \@effective_conf_arr;

    } else { # if pattern is passed to the command

        my @lines = get_lines(raw_output=>$output,first_line=>'zone:');
        my $pattern_type = "zone";
        if (!@lines) {
            @lines = get_lines(raw_output=>$output,first_line=>'cfg:');
            $pattern_type = "cfg";
        }

        return [] if !@lines;

        my (@patterns, @members, $pattern);
        my $line = shift(@lines);
        while (defined $line){
            if ($line =~ /$pattern_type:\s+/) {
                if (scalar(@members) > 0) {
                    my %pattern;
                    $pattern{'name'} = $pattern;
                    push(@{$pattern{'members'}}, @members);
                    push(@patterns, \%pattern);
                    @members = ();
                }
                my @tmp_arr = split(/\s+/, $line);
                $pattern = $tmp_arr[1];
                if ( scalar(@tmp_arr) > 2 ) {
                    for (my $mem = 2; $mem < scalar(@tmp_arr); $mem++) {
                        if ($tmp_arr[$mem] =~ /\;/) {
                            push(@members, split(/\;\s*/, $tmp_arr[$mem]));
                        } else {
                            push(@members, $tmp_arr[$mem]);
                        }
                    }
                }
            } elsif ($line =~ /\;/) {
                my @tmp_members = split(/\;\s*/, $line);
                push(@members, @tmp_members);
            } else {
                push(@members, $line);
            }
            $line = shift(@lines);
        }
        my %tmp_pattern;
        $tmp_pattern{'name'} = $pattern;
        push(@{$tmp_pattern{'members'}}, @members);
        push(@patterns, \%tmp_pattern);
        my $patterns = $pattern_type . "s";
        $hash{$patterns} = \@patterns;
    } # end if ($command =~ ...
    push(@result_array, \%hash);

    return wantarray ? @result_array : \@result_array;
}

1;
