#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC interface commands
##
## @description
##    This is a library to parse the raw output of interface commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::interface
##
##  interface(%args)
##
## @author jais@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::interface;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp = $args{"response_object"};
    my $error_message;
    my $error;

    # Check for the errors
    if ($raw_output =~ /[iI]nvalid command/) {
        $error = "InvalidParamValue";
        $error_message = "Invalid command option";
    } elsif ($raw_output =~ /Invalid\s+interface\s+format/) {
         $error = "InvalidParamValue";
         $error_message = "Invalid command option";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
}

## @name ::interface_fc
##
## @summary Parses the raw output from the "interface fc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from interface_fc with any of its options as described in the
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_fc(%args);
## INPUTS TO METHOD CALL 'interface_fc' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = interface fc 1/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_fc {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

## @name ::interface_eth
##
## @summary Parses the raw output from the "interface eth" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from interface_eth with any of its options as described in the 
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_eth(%args);
## INPUTS TO METHOD CALL 'interface_eth' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = interface eth 1/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_eth {
    my %args = @_;

    my $resp          = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

## @name ::interface_vfc
##
## @summary Parses the raw output from the "interface" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from interface_vfc with any of its options as described in the 
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_vfc(%args);
## INPUTS TO METHOD CALL 'interface_vfc' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = interface vfc3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_vfc {
    my %args = @_;

    my $resp          = $args{response_object};
    my $raw_output    = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

## @name ::interface_mgmt
##
## @summary Parses the raw output from the "interface mgmt" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from interface_mgmt with any of its options as described in the
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_mgmt(%args);
## INPUTS TO METHOD CALL 'interface_mgmt' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mgmt-interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = interface mgmt 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_mgmt {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

## @name ::interface_port_channel
##
## @summary Parses the raw output from the "interface port-channel" command
#
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from interface_port_channel with any of its options as described in the
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_port_channel(%args);
## INPUTS TO METHOD CALL 'interface_port_channel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-channel' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = interface port-channel 256
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_port_channel {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

## @name ::interface_vsan
##
## @summary Parses the raw output from the "interface vsan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from interface_vsan with any of its options as described in the
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface_vsan(%args);
## INPUTS TO METHOD CALL 'interface_vsan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsan' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = interface vsan 33
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface_vsan {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

1;
