#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC switch "show device" commands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch "show device" commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_device
##
##  show_device-alias_database(%args)
##
## @author rroshan@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_device;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    #Check for the error
    if ($raw_output =~ /(There are no entries.* )/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.*Invalid.* )/) {
        $error = "InvalidParam";
        $$error_message = $1;
    }
    return $error;
}

## @name ::show_device_alias_database
##
## @summary Parses the raw output from the "show device-alias database" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show device-alias database with any of its options as described in the
##    corresponding cdef file 'show_device.cdefs'
## @synopsis
##      show device-alias database(%args);
## INPUTS TO METHOD CALL 'show_device_alias_database' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = show device-alias database
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_entries' => '2',
##&nbsp;            'device_alias_names' => [
##&nbsp;                                      {
##&nbsp;                                        'pwwn' => '50:0a:09:80:00:02:52:1f',
##&nbsp;                                        'device-alias_name' => 'dev2'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'pwwn' => '50:0a:09:80:00:02:51:1f',
##&nbsp;                                        'device-alias_name' => 'tech7'
##&nbsp;                                      }
##&nbsp;                                    ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_device_alias_database {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output=>$output,command=>$command,
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $i=0;
    my $j=-1; 
    my @arr = get_lines(raw_output=>$output,response_object=>$resp);
    foreach my $line (@arr){
        if ($line =~ /(^device-alias name)\s(\w+)\s(pwwn)\s([0-9a-z\:]+)/) {
            $j++;
            $key = to_key_name(item=>"device_alias_names");
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>$1)} = $2;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>$3)} = $4;
        } elsif ($line =~ /Total.*\s\=\s(\d+)/) {
            $result_array[$i]->{to_key_name(item=>"Total Entries")} = $1;
        }
    }
    return \@result_array;
}
1;
