#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC switch show fc commands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch show fc commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_fc
##
##  show_fcalias(%args)
##
## @author rroshan@netapp.com
##
## @status
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_fc;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    #Check for the error
    if($raw_output =~ /(Alias not.* )/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.*Invalid.* )/) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.*No pending.* )/) {
        $error = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}

## @name ::show_fcalias
##
## @summary Parses the raw output from the "show fcalias" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show device-alias database with any of its options as described in the
##    corresponding cdef file 'show_device.cdefs'
## @synopsis
##      show fcalias(%args);
## INPUTS TO METHOD CALL 'show_fcalias' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = show fcalias
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tech2' => [
##&nbsp;                         {
##&nbsp;                           'pwwn' => '10:00:00:23:45:67:89:ab',
##&nbsp;                           'fwwn' => '10:01:10:01:10:ab:cd:ef',
##&nbsp;                           'fcid' => '0x222222'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'vsan' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'alias1' => [
##&nbsp;                          {
##&nbsp;                            'pwwn' => '50:0a:09:81:00:02:37:2a'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'vsan' => '3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'tech4' => [
##&nbsp;                         {
##&nbsp;                           'pwwn' => '10:00:00:23:45:67:89:ab',
##&nbsp;                           'fcid' => '0x222222'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'vsan' => '4'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = show fcalias name tech2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tech2' => [
##&nbsp;                         {
##&nbsp;                           'pwwn' => '10:00:00:23:45:67:89:ab',
##&nbsp;                           'fwwn' => '10:01:10:01:10:ab:cd:ef',
##&nbsp;                           'fcid' => '0x222222'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'vsan' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = show fcalias vsan 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tech2' => [
##&nbsp;                         {
##&nbsp;                           'pwwn' => '10:00:00:23:45:67:89:ab',
##&nbsp;                           'fwwn' => '10:01:10:01:10:ab:cd:ef',
##&nbsp;                           'fcid' => '0x222222'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'vsan' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_fcalias {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output=>$output,command=>$command,
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $i=-1;
    my @arr = get_lines(raw_output=>$output,response_object=>$resp);
    foreach my $line (@arr){
        if ($line =~ /^fcalias name\s(\w+)\s(vsan)\s(\d+)/){
            $i++;
            $result_array[$i]->{to_key_name(item=>$2)} = $3;
            $key = to_key_name(item=>$1);
            next;
        } elsif ($line =~ /^(fcid)\s+(\w+)/) {
            $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
        } elsif ($line =~ /^(fwwn)\s+([0-9a-z\:]+)/) {
            $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
        } elsif ($line =~ /^(pwwn)\s+([0-9a-z\:]+)/) {
            $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
        }
    }
    return \@result_array;
};
1;
