#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_fcdomain_domain_list.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse CiscoNetwork show_fcdomain_domain_list command
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch show_fcdomain_domain_list command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_fcdomain_domain_list
##
##  show_fcdomain_domain_list(%args)
##
## @author skotagir@netapp.com ,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_fcdomain_domain_list;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my ( $error, $error_message );
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(Incomplete\s+command)/i ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif (
        $raw_output =~ /(Invalid\s+number|Error:\s+VSAN.*not\s+available)/i )
    {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Invalid\s+command/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::show_fcdomain_domain_list
##
## @summary Parses the raw output from the "show fcdomain domain-list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_fcdomain_domain_list with any of its options as described in the
##    corresponding cdef file 'show_fcdomain_domain_list.cdefs'
## @synopsis
##      show_fcdomain_domain_list(%args);
## INPUTS TO METHOD CALL 'show_fcdomain_domain_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show fcdomain domain-list
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VSAN 22
##&nbsp;Number of domains: 2
##&nbsp;Domain ID              WWN
##&nbsp;---------    -----------------------
##&nbsp;0x6c(108)    20:16:00:05:73:fc:fa:69 [Local] [Principal]
##&nbsp;0x9a(154)    20:16:54:7f:ee:06:22:d1
##&nbsp;
##&nbsp;VSAN 98
##&nbsp;Number of domains: 1
##&nbsp;Domain ID              WWN
##&nbsp;---------    -----------------------
##&nbsp;0x8d(141)    20:62:00:05:73:fc:fa:69 [Local] [Principal]
##&nbsp;
##&nbsp;VSAN 101
##&nbsp;Number of domains: 1
##&nbsp;Domain ID              WWN
##&nbsp;---------    -----------------------
##&nbsp; 0x2b(43)    20:65:00:05:73:fc:fa:69 [Local] [Principal]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'domain_info' => [
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 1,
##&nbsp;                                 'domain_id' => '0x6c(108)',
##&nbsp;                                 'is_local' => 1,
##&nbsp;                                 'wwn' => '20:16:00:05:73:fc:fa:69'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 0,
##&nbsp;                                 'domain_id' => '0x9a(154)',
##&nbsp;                                 'is_local' => 0,
##&nbsp;                                 'wwn' => '20:16:54:7f:ee:06:22:d1'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'number_of_domains' => '2',
##&nbsp;            'vsan_id' => '22'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'domain_info' => [
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 1,
##&nbsp;                                 'domain_id' => '0x8d(141)',
##&nbsp;                                 'is_local' => 1,
##&nbsp;                                 'wwn' => '20:62:00:05:73:fc:fa:69'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'number_of_domains' => '1',
##&nbsp;            'vsan_id' => '98'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'domain_info' => [
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 1,
##&nbsp;                                 'domain_id' => '0x2b(43)',
##&nbsp;                                 'is_local' => 1,
##&nbsp;                                 'wwn' => '20:65:00:05:73:fc:fa:69'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'number_of_domains' => '1',
##&nbsp;            'vsan_id' => '101'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show fcdomain domain-list vsan 22
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Number of domains: 2
##&nbsp;Domain ID              WWN
##&nbsp;---------    -----------------------
##&nbsp;0x6c(108)    20:16:00:05:73:fc:fa:69 [Local] [Principal]
##&nbsp;0x9a(154)    20:16:54:7f:ee:06:22:d1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'domain_info' => [
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 1,
##&nbsp;                                 'domain_id' => '0x6c(108)',
##&nbsp;                                 'is_local' => 1,
##&nbsp;                                 'wwn' => '20:16:00:05:73:fc:fa:69'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'is_principal' => 0,
##&nbsp;                                 'domain_id' => '0x9a(154)',
##&nbsp;                                 'is_local' => 0,
##&nbsp;                                 'wwn' => '20:16:54:7f:ee:06:22:d1'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'number_of_domains' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_fcdomain_domain_list {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( $count, $limit ) = 0;

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /(VSAN)\s+(\d+)/ ) {
            $result_array[$count]->{vsan_id} = $2;
        } elsif ( $lines[$i] =~ /(.*)[:]\s+(\d+)/ ) {
            $result_array[$count]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $lines[$i] =~ /.*WWN/ ) {
            @head = split( /\s\s+/, $lines[$i] );
        } elsif ( $lines[$i]
            =~ /(\w+\S+)\s+(\w+[:].*\w+[:]\w+)\s*(\[Local])?\s*(\[Principal])?/ ) {
            $result_array[$count]->{domain_info}->[$limit]
                ->{ to_key_name( item => $head[0] ) } = $1;
            $result_array[$count]->{domain_info}->[$limit]
                ->{ to_key_name( item => $head[1] ) } = $2;
            $result_array[$count]->{domain_info}->[$limit]->{is_local}
                = $3 ? 1 : 0;
            $result_array[$count]->{domain_info}->[$limit]->{is_principal}
                = $4 ? 1 : 0;
            $limit++;
            if ( $lines[ $i + 1 ] =~ /(VSAN)\s+(\d+)/ ) {
                $count++;
                $limit = 0;
            }
        }
    }
    return \@result_array;
} ## end sub show_fcdomain_domain_list
1;
