#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC switch show_fcns_database_local command
##
## @description
##    This is a library to parse the raw output of CiscoFC switch show_fcns_database_local command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_fcns_database_local
##
##  show_fcns_database_local(%args)
##
## @author rroshan@netapp.com
##
## @status
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_fcns_database_local;

use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    #Check for the error
    if($raw_output =~ /(vsan not.* )/) {
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.*Invalid.* )/) {
        $error = "InvalidParam";
        $$error_message = $1;
    } elsif ($raw_output =~ /(.*Incomplete.* )/) {
        $error = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}

## @name ::show_fcns_database_local
##
## @summary Parses the raw output from the "show fcns database local" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show device-alias database with any of its options as described in the
##    corresponding cdef file 'show_fcns_database_local.cdefs'
## @synopsis
##      show fcns database local(%args);
## INPUTS TO METHOD CALL 'show_fcns_database_local' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = show fcns database local
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan_1' => [
##&nbsp;                          {
##&nbsp;                            'fc_type' => 'scsi-fcp:init',
##&nbsp;                            'pwwn' => '21:00:00:e0:8b:8f:69:9e',
##&nbsp;                            'type' => 'N',
##&nbsp;                            'vendor' => 'Qlogic',
##&nbsp;                            'fcid' => '0x1b0b00'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'fc_type' => 'scsi-fcp 244',
##&nbsp;                            'pwwn' => '50:00:1f:e1:50:01:81:ec',
##&nbsp;                            'type' => 'N',
##&nbsp;                            'vendor' => 'HP',
##&nbsp;                            'fcid' => '0x1b0c00'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_entries' => '17'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;Command = show fcns database local vsan 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan_1' => [
##&nbsp;                          {
##&nbsp;                            'fc_type' => 'scsi-fcp:init',
##&nbsp;                            'pwwn' => '21:00:00:e0:8b:8f:69:9e',
##&nbsp;                            'type' => 'N',
##&nbsp;                            'vendor' => 'Qlogic',
##&nbsp;                            'fcid' => '0x1b0b00'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'fc_type' => 'scsi-fcp 244',
##&nbsp;                            'pwwn' => '50:00:1f:e1:50:01:81:ec',
##&nbsp;                            'type' => 'N',
##&nbsp;                            'vendor' => 'HP',
##&nbsp;                            'fcid' => '0x1b0c00'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          },
##&nbsp;        ];
##&nbsp;Command = show fcns database detail
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fcid_info' => [
##&nbsp;                             {
##&nbsp;                               'hard_addr' => '0x000000',
##&nbsp;                               'port_type' => 'N',
##&nbsp;                               'symbolic_node_name' => 'QLA2342 FW:v3.03.06 DVR:v9.0.0.2 (w32 IP)',
##&nbsp;                               'symbolic_port_name' => '',
##&nbsp;                               'node_wwn' => '20:00:00:e0:8b:8f:69:9e',
##&nbsp;                               'connected_interface' => 'fc1/21',
##&nbsp;                               'port_ip_addr' => '0.0.0.0',
##&nbsp;                               'ipa' => 'ff ff ff ff ff ff ff ff',
##&nbsp;                               'node_ip_addr' => '0.0.0.0',
##&nbsp;                               'switch_name_ip_address' => 'vgci9124s38 (10.227.195.58)',
##&nbsp;                               'port_wwn_vendor' => '21:00:00:e0:8b:8f:69:9e (Qlogic)',
##&nbsp;                               'permanent_port_wwn_vendor' => '21:00:00:e0:8b:8f:69:9e (Qlogic)',
##&nbsp;                               'fc4_types' => 'scsi-fcp:init',
##&nbsp;                               'fabric_port_wwn' => '20:15:00:0d:ec:3e:c0:80',
##&nbsp;                               'class' => '3'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'vsan' => '1',
##&nbsp;            'fcid' => '0x1b0b00'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fcid_info' => [
##&nbsp;                             {
##&nbsp;                               'connected_interface' => 'fc1/24',
##&nbsp;                               'port_ip_addr' => '0.0.0.0',
##&nbsp;                               'symbolic_node_name' => '',
##&nbsp;                               'symbolic_port_name' => 'HSV110 (C)COMPAQ - vrr-eva5000-1 - SR0FBF',
##&nbsp;                               'switch_name_ip_address' => 'vgci9124s38 (10.227.195.58)',
##&nbsp;                               'port_wwn_vendor_' => '50:00:1f:e1:50:01:81:ec (HP)',
##&nbsp;                               'port_type' => 'N',
##&nbsp;                               'fabric_port_wwn' => '20:18:00:0d:ec:3e:c0:80',
##&nbsp;                               'ipa' => 'ff ff ff ff ff ff ff ff',
##&nbsp;                               'permanent_port_wwn_vendor' => '50:00:1f:e1:50:01:81:ec (HP)',
##&nbsp;                               'fc4_types' => 'scsi-fcp 244',
##&nbsp;                               'class' => '3',
##&nbsp;                               'hard_addr' => '0x000000',
##&nbsp;                               'node_wwn' => '50:00:1f:e1:50:01:81:e0',
##&nbsp;                               'node_ip_addr' => '0.0.0.0'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'vsan' => '1',
##&nbsp;            'fcid' => '0x1b0c00'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;

sub show_fcns_database_local {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output=>$output,command=>$command,
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    my $i=-1;
    my $j=0;
    my @arr = get_lines(raw_output=>$output,response_object=>$resp);
    foreach my $line (@arr){
        if ($line =~ /^(VSAN\s\d+)/){
            $i++;
            $key = to_key_name(item=>$1);
        } elsif ($line =~ /--------------------------------------------------------------------------/) {
            next;
        } elsif ($line =~ /^FCID        TYPE  PWWN                    (VENDOR)        FC4-TYPE:FEATURE/) {
            next;
        } elsif ($line =~ /([0-9a-z\:]+)\s+(\w+)\s+([0-9a-z\:]+)\s+\((\w+)\)\s+([0-9a-z-\s\:]+)/ && $command !~ /detail/) {
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"fcid")} = $1;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"type")} = $2;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"pwwn")} = $3;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"vendor")} = $4;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"fc_type:usage")} = $5;
            $j++;
        } elsif ($line =~ /([0-9a-z\:]+)\s+(\w+)\s+([0-9a-z\:]+)\s+\s+([0-9a-z-\s\:]+)/ && $command !~ /detail/) {
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"fcid")} = $1;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"type")} = $2;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"pwwn")} = $3;
            $result_array[$i]->{$key}->[$j]->{to_key_name(item=>"fc_type:usage")} = $4;
            $j++;
        } elsif ($line =~ /------------------------/) {
            next;
        } elsif ($line =~ /(VSAN)\:(\d+)\s+(FCID)\:([0-9a-z]+)/) {
            $i++;
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
            $result_array[$i]->{to_key_name(item=>$3)} = $4;
            $key1 = to_key_name(item=>"fcid_info");
        } elsif ($line =~ /(\w+[-]?[:]?[\s+]?\w+[\s:-][(\w+)]*[\s]?[\S+]*)\s+\:(.*)/) {
            my $k = $1;
            my $val = $2;
            $val = trim_string( 'string' => $val );
            $k = trim_string( 'string' => $k );
            $k =~ s/7mNo previous regular expression.*\[m//;
            $k =~ s/^m//;
            $k =~ s/-/_/g;
            $result_array[$i]->{$key1}->[$j]->{to_key_name(item=>$k)} = $val;
        }
    }
    return \@result_array;
};
1;
