#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_fcs_ie.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse CiscoNetwork show_fcs_ie command
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch show_fcs_ie command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_fcs_ie
##
##  show_fcs_ie(%args)
##
## @author skotagir@netapp.com ,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_fcs_ie;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my ( $error, $error_message );
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(Incomplete\s+command)/i ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output =~ /Error:\s+VSAN\s+not\s+present/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Invalid\s+command/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::show_fcs_ie
##
## @summary Parses the raw output from the "show fcs ie" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_fcs_ie with any of its options as described in the
##    corresponding cdef file 'show_fcs_ie.cdefs'
## @synopsis
##      show_fcs_ie(%args);
## INPUTS TO METHOD CALL 'show_fcs_ie' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan' </td> <td> '0' </td> </tr>
## <tr> <td> 'nwwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show fcs ie
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IE List for VSAN: 1
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:01:00:05:73:fc:fa:69  S(Loc) 0xfffc00 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;[Total 1 IEs in Fabric]
##&nbsp;
##&nbsp;IE List for VSAN: 22
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:16:00:05:73:fc:fa:69  S(Loc) 0xfffc6c 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;20:16:54:7f:ee:06:22:d1  S(Adj) 0xfffc9a 10.228.160.211
##&nbsp;                                        (dpg-mcc-cisco9148-04-B-fabX)
##&nbsp;[Total 2 IEs in Fabric]
##&nbsp;
##&nbsp;IE List for VSAN: 33
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:21:00:05:73:fc:fa:69  S(Loc) 0xfffc00 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;[Total 1 IEs in Fabric]
##&nbsp;
##&nbsp;IE List for VSAN: 98
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:62:00:05:73:fc:fa:69  S(Loc) 0xfffc8d 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;[Total 1 IEs in Fabric]
##&nbsp;
##&nbsp;IE List for VSAN: 101
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:65:00:05:73:fc:fa:69  S(Loc) 0xfffc2b 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;[Total 1 IEs in Fabric]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:01:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc00'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '1',
##&nbsp;            'total_ies_in_fabric' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:16:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc6c'
##&nbsp;                                 },
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Adj)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-B-fabX',
##&nbsp;                                   'ie_wwn' => '20:16:54:7f:ee:06:22:d1',
##&nbsp;                                   'mgmt_addr' => '10.228.160.211',
##&nbsp;                                   'mgmt_id' => '0xfffc9a'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '22',
##&nbsp;            'total_ies_in_fabric' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:21:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc00'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '33',
##&nbsp;            'total_ies_in_fabric' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:62:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc8d'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '98',
##&nbsp;            'total_ies_in_fabric' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:65:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc2b'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '101',
##&nbsp;            'total_ies_in_fabric' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show fcs ie vsan 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IE List for VSAN: 1
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;IE-WWN                   IE     Mgmt-Id  Mgmt-Addr (Switch-name)
##&nbsp;-------------------------------------------------------------------------------
##&nbsp;20:01:00:05:73:fc:fa:69  S(Loc) 0xfffc00 10.228.160.209
##&nbsp;                                        (dpg-mcc-cisco9148-04-A-fabX)
##&nbsp;[Total 1 IEs in Fabric]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ie_attributes' => [
##&nbsp;                                 {
##&nbsp;                                   'ie' => 'S(Loc)',
##&nbsp;                                   'switch_name' => 'dpg-mcc-cisco9148-04-A-fabX',
##&nbsp;                                   'ie_wwn' => '20:01:00:05:73:fc:fa:69',
##&nbsp;                                   'mgmt_addr' => '10.228.160.209',
##&nbsp;                                   'mgmt_id' => '0xfffc00'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'vsan_id' => '1',
##&nbsp;            'total_ies_in_fabric' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show fcs ie  nwwn 20:16:54:7f:ee:06:22:d1 vsan 22
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;IE Attributes
##&nbsp;-------------
##&nbsp;Domain-Id = 0x9a(154)
##&nbsp;Management-Id = 0xfffc9a
##&nbsp;Fabric-Name = 20:16:00:05:73:fc:fa:69
##&nbsp;Logical-Name = dpg-mcc-cisco9148-04-B-fabX
##&nbsp;Management Address List =
##&nbsp;        snmp://10.228.160.211/eth-ip
##&nbsp;        http://10.228.160.211/eth-ip
##&nbsp;Information List:
##&nbsp;        Vendor-Name = Cisco Systems, Inc.
##&nbsp;        Model Name/Number = DS-C9148-16P-K9
##&nbsp;        Release-Code = 5.2(1)
##&nbsp;Switch Zoning Enforcement Status:
##&nbsp;    fwwn:20:01:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:02:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:03:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:04:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:05:54:7f:ee:06:22:d0  [ Hard ON Bcast ON ]
##&nbsp;    fwwn:20:06:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:07:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:08:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:09:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0a:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0b:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0c:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0d:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0e:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:0f:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;    fwwn:20:10:54:7f:ee:06:22:d0  [ NA ]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'information_list' => {
##&nbsp;                                    'vendor-name' => 'Cisco Systems, Inc.',
##&nbsp;                                    'release-code' => '5.2(1)',
##&nbsp;                                    'model_name_number' => 'DS-C9148-16P-K9'
##&nbsp;                                  },
##&nbsp;            'domain-id' => '0x9a(154)',
##&nbsp;            'management_address_list' => {
##&nbsp;                                           'snmp' => '//10.228.160.211/eth-ip',
##&nbsp;                                           'http' => '//10.228.160.211/eth-ip'
##&nbsp;                                         },
##&nbsp;            'logical-name' => 'dpg-mcc-cisco9148-04-B-fabX',
##&nbsp;            'switch_zoning_enforcement_status' => [
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:01:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:02:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:03:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:04:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'ON',
##&nbsp;                                                                   'hard' => 'ON'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:05:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:06:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:07:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:08:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:09:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0a:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0b:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0c:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0d:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0e:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:0f:54:7f:ee:06:22:d0'
##&nbsp;                                                    },
##&nbsp;                                                    {
##&nbsp;                                                      'flags' => {
##&nbsp;                                                                   'bcast' => 'OFF',
##&nbsp;                                                                   'hard' => 'OFF'
##&nbsp;                                                                 },
##&nbsp;                                                      'wwn' => '20:10:54:7f:ee:06:22:d0'
##&nbsp;                                                    }
##&nbsp;                                                  ],
##&nbsp;            'management-id' => '0xfffc9a',
##&nbsp;            'fabric-name' => '20:16:00:05:73:fc:fa:69'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_fcs_ie {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @head );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $count = -1;
    my $limit = 0;

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $cmd =~ /nwwn/ ) {
            if ( $lines[$i] =~ /^(\w+[-](Id|Name))\s+[=]\s+(\S+)$/ ) {
                $result_array[0]
                    ->{ to_key_name( item => $1 ) } = $3;
            } elsif ( $lines[$i] =~ /(\w+)[:](\/\/.*)/ ) {
                $result_array[0]->{management_address_list}
                    ->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $lines[$i] =~ /Vendor|Model|Release/ ) {
                my @values = split( /\s\=\s/, $lines[$i] );
                $values[0] =~ s/\//_/;
                $result_array[0]->{information_list}
                    ->{ to_key_name( item => $values[0] ) } = $values[1];
            } elsif ( $lines[$i]
                =~ /fwwn[:](.*\d+)\s+(\[\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+\])*/
                ) {
                $result_array[0]->{switch_zoning_enforcement_status}->[$limit]
                    ->{wwn} = $1;
                $result_array[0]->{switch_zoning_enforcement_status}->[$limit]
                    ->{flags}->{hard} = $4 ? $4 : 'OFF';
                $result_array[0]->{switch_zoning_enforcement_status}->[$limit]
                    ->{flags}->{bcast} = $6 ? $6 : 'OFF';
                $limit++;
            }
        } else {
            if ( $lines[$i] =~ /(.*)[:]\s+(\d+)/ ) {
                $count++;
                $limit = 0;
                $result_array[$count]->{vsan_id} = $2;
            } elsif ( $lines[$i] =~ /IE-WWN.*/ ) {
                @head = split( /\s+/, $lines[$i] );
                $head[4] =~ s/\(//;
                $head[4] =~ s/\)//;
                for ( my $j = 0; $j <= $#head; $j++ ) {
                    $head[$j] =~ s/-/_/;
                }
            } elsif ( $lines[$i]
                =~ /(\w+[:].*[:]\w+)\s+(\w+\(.*\))\s+(\S+)\s+(.*)/ ) {
                $result_array[$count]->{ie_attributes}->[$limit]
                    ->{ to_key_name( item => $head[0] ) } = $1;
                $result_array[$count]->{ie_attributes}->[$limit]
                    ->{ to_key_name( item => $head[1] ) } = $2;
                $result_array[$count]->{ie_attributes}->[$limit]
                    ->{ to_key_name( item => $head[2] ) } = $3;
                $result_array[$count]->{ie_attributes}->[$limit]
                    ->{ to_key_name( item => $head[3] ) } = $4;
            } elsif ( $lines[$i] =~ /\((.*)\)/ ) {
                $result_array[$count]->{ie_attributes}->[$limit]
                    ->{ to_key_name( item => $head[4] ) } = $1;
                $limit++;
            } elsif ( $lines[$i] =~ /(Total)\s+(\d+)\s+(.*\w+)/ ) {
                my $temp = $1 . " " . $3;
                $result_array[$count]->{ to_key_name( item => $temp ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub show_fcs_ie
1;
