#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_interface.pm#22 $
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC show_interface command
##
## @description
##    This is a library to parse the raw output of CiscoFC show_interface commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_interface
##
##  show_interface(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_interface;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my $error;
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    if ( $raw_output =~ /Invalid command at.*marker/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid\s+command/ ) {
        $error          = "command failed";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::show_interface_brief
##
## @summary Parses the raw output from the "show interface brief" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_interface_brief_detail with any of its options as described in the
##    corresponding cdef file 'show_interface_brief.cdefs'
## @synopsis
##      show_interface_brief(%args);
## INPUTS TO METHOD CALL 'show_interface_brief' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show interface brief
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'port_channel' => '--',
##&nbsp;             'oper_speed_gbps' => '',
##&nbsp;             'status' => 'errDisabled',
##&nbsp;             'sfp' => 'swl',
##&nbsp;             'interface' => 'fc1/1',
##&nbsp;             'admin_truck_mode' => 'off',
##&nbsp;             'vsan' => '1',
##&nbsp;             'admin_mode' => 'auto',
##&nbsp;             'oper_mode' => '--'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'port_channel' => '--',
##&nbsp;             'oper_speed_gbps' => '4',
##&nbsp;             'status' => 'up',
##&nbsp;             'sfp' => 'swl',
##&nbsp;             'interface' => 'fc1/2',
##&nbsp;             'admin_truck_mode' => 'off',
##&nbsp;             'vsan' => '1',
##&nbsp;             'admin_mode' => 'auto',
##&nbsp;             'oper_mode' => 'F'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'speed_gbps' => '1',
##&nbsp;             'status' => 'up',
##&nbsp;             'interface' => 'sup-fc0'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'speed' => '100 Mbps',
##&nbsp;             'status' => 'up',
##&nbsp;             'interface' => 'mgmt0',
##&nbsp;             'ip_address' => '10.227.195.58/23',
##&nbsp;             'mtu' => '1500'
##&nbsp;           }
##&nbsp;         ];
sub show_interface_brief {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my $val;
    my @lines
        = get_lines( raw_output => $output, "response_object" => $resp );
    foreach $val (@lines) {
        if ( $val
            =~ /(\w+\/\d+)\s+(\d+)\s+(\w+)\s+(\w+)\s+(\w+)\s+([\w+|\-]+)\s+([\w+|\-]+)\s+((\d+)?)\s+([\w+|\-]+)/
            ) {
            $i++;
            $result_array[$i]->{"interface"}        = $1;
            $result_array[$i]->{"vsan"}             = $2;
            $result_array[$i]->{"admin_mode"}       = $3;
            $result_array[$i]->{"admin_truck_mode"} = $4;
            $result_array[$i]->{"status"}           = $5;
            $result_array[$i]->{"sfp"}              = $6;
            $result_array[$i]->{"oper_mode"}        = $7;
            $result_array[$i]->{"oper_speed_gbps"}  = $8;
            $result_array[$i]->{"port_channel"}     = $10;
        } elsif ( $val =~ /(\w+\-\w+)\s+(\w+)\s+(\d+)/ ) {
            $i++;
            $result_array[$i]->{"interface"}  = $1;
            $result_array[$i]->{"status"}     = $2;
            $result_array[$i]->{"speed_gbps"} = $3;
        } elsif ( $val
            =~ /(\w+)\s+(\w+)\s+(\d+.\d+.\d+.\d+(\/\d+)?)\s+(\d+\s+\w+)\s+(\d+)/
            ) {
            $i++;
            $result_array[$i]->{"interface"}  = $1;
            $result_array[$i]->{"status"}     = $2;
            $result_array[$i]->{"ip_address"} = $3;
            $result_array[$i]->{"speed"}      = $5;
            $result_array[$i]->{"mtu"}        = $6;
        }
    }
    return \@result_array;
}    # end sub show_interface_brief
## @name ::show_interface_fc
##
## @summary Parses the raw output from the "show interface fc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_interface with any of its options as described in the
##    corresponding cdef file 'show_interface.cdefs'
## @synopsis
##      show_interface_fc(%args);
## INPUTS TO METHOD CALL 'show_interface' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> fc$port </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show interface fc1/3
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'admin_port_mode' => 'auto',
##&nbsp;             'input' => [
##&nbsp;                          {
##&nbsp;                            'frames_per_sec' => '15',
##&nbsp;                            'bytes' => '9364310412',
##&nbsp;                            'errors' => '0',
##&nbsp;                            'crc' => '0',
##&nbsp;                            'byte_per_sec' => '21691',
##&nbsp;                            'unknown_class' => '0',
##&nbsp;                            'bite_per_sec' => '173528',
##&nbsp;                            'discards' => '0',
##&nbsp;                            'too_short' => '0',
##&nbsp;                            'too_long' => '0',
##&nbsp;                            'frames_input' => '6688966'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;             'fc1/3' => 'up',
##&nbsp;             'sfp' => 'short wave laser w/o ofc (sn)',
##&nbsp;             'low_priority_transmit_b2b_credit_remaining' => '3',
##&nbsp;             'receive_b2b_credit_remaining' => '16',
##&nbsp;             'rate_mode' => 'dedicated',
##&nbsp;             'transmit_b2b_credit_remaining' => '3',
##&nbsp;             'snmp_link_state_traps' => 'enabled',
##&nbsp;             'hardware' => 'fibre channel',
##&nbsp;             'port_vsan' => '1',
##&nbsp;             'port_mode' => 'F',
##&nbsp;             'speed_gbps' => '4',
##&nbsp;             'port_wwn' => '20:03:00:0d:ec:3e:c0:80',
##&nbsp;             'trunk_mode' => 'on',
##&nbsp;             'output' => [
##&nbsp;                           {
##&nbsp;                             'frames_per_sec' => '10',
##&nbsp;                             'input_ols' => '0',
##&nbsp;                             'bytes' => '3319887388',
##&nbsp;                             'errors' => '0',
##&nbsp;                             'frames_output' => '4919080',
##&nbsp;                             'lrr' => '0',
##&nbsp;                             'byte_per_sec' => '6840',
##&nbsp;                             'bite_per_sec' => '54720',
##&nbsp;                             'discards' => '0'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;             'interface_last_changed ' => 'sat oct  8 10:22:39 2011'
##&nbsp;             'receive_data_failed_size' => '2112',
##&nbsp;             'fcid' => '0x1b1400',
##&nbsp;             'beacon' => 'turned off'
##&nbsp;           },
##&nbsp;         ];
##&nbsp;
sub show_interface_fc {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( $i, $j ) = 0;
    my ( $val, $value1, $value2, $key, $key1, $name );
    my @lines
        = get_lines( raw_output => $output, "response_object" => $resp );
    foreach $val (@lines) {
        if ( $val =~ /(\w+\/\d+)\s+\w+\s+(\w+)/ ) {
            $result_array[$i]->{"$1"} = $2;
        } elsif ( $val
            =~ /\w+\s+\w+\s+(\w+\s+\w+),\s+\w+\s+\w+\s+(\w+\s+\w+\s+\w+\s+\w+\/\w+\s+\w+\s+\(\w+\))/
            ) {
            $result_array[$i]->{"hardware"} = $1;
            $result_array[$i]->{"sfp"}      = $2;
        } elsif ( $val =~ /Port WWN is\s+((\w+:)+\w+)/ ) {
            $result_array[$i]->{"port_wwn"} = $1;
        } elsif ( $val
            =~ /Admin port mode\s+\w+\s+(\w+), trunk mode\s+\w+\s+(\w+)/ ) {
            $result_array[$i]->{"admin_port_mode"} = $1;
            $result_array[$i]->{"trunk_mode"}      = $2;
        } elsif ( $val =~ /snmp link\s+\w+\s+\w+\s+\w+\s+(\w+)/ ) {
            $result_array[$i]->{"snmp_link_state_traps"} = $1;
        } elsif ( $val =~ /(\d+)\s+(\w+((\s+\w+)*)?\s+B2B(\s+\w+)*)/ ) {
            $result_array[$i]->{ to_key_name( item => $2 ) } = $1;
        } elsif ( $val =~ /Port mode\s+\w+\s+(\w+), FCID\s+\w+\s+(\w+)/ ) {
            $result_array[$i]->{"port_mode"} = $1;
            $result_array[$i]->{"fcid"}      = $2;
        } elsif ( $val =~ /Port vsan\s+\w+\s+(\d+)/ ) {
            $result_array[$i]->{"port_vsan"} = $1;
        } elsif ( $val =~ /Speed\s+\w+\s+(\d+)\s+\w+/ ) {
            $result_array[$i]->{"speed_gbps"} = $1;
        } elsif ( $val =~ /Rate mode\s+\w+\s+(\w+)/ ) {
            $result_array[$i]->{"rate_mode"} = $1;
        } elsif ( $val =~ /Receive(\s+\w+){4}\s+(\d+)/ ) {
            $result_array[$i]->{"receive_data_failed_size"} = $2;
        } elsif ( $val =~ /Beacon\s+\w+\s+(\w+\s+\w+)/ ) {
            $result_array[$i]->{"beacon"} = $1;
        } elsif ( $val
            =~ /(\d+\s+minutes\s+(input|output)\s+rate)\s+(\d+)\s+bits\/sec,\s+(\d+)\s+bytes\/sec,\s+(\d+)\s+frames\/sec/
            ) {
            $name                                                  = $2;
            $result_array[$i]->{"$name"}->[$j]->{"bite_per_sec"}   = $3;
            $result_array[$i]->{"$name"}->[$j]->{"byte_per_sec"}   = $4;
            $result_array[$i]->{"$name"}->[$j]->{"frames_per_sec"} = $5;
        } elsif (
            $val =~ /(\d+)\s+(frames\s+(input|output)),\s+(\d+)\s+bytes/ ) {
            $name                                             = $3;
            $result_array[$i]->{"$name"}->[$j]->{"frames_$3"} = $1;
            $result_array[$i]->{"$name"}->[$j]->{"bytes"}     = $4;
        } elsif ( $val =~ /(\d+)\s+(\w+(\s+\w+)?),\s+(\d+)\s+(\w+(\s+\w+)?)/ )
        {
            $result_array[$i]->{"$name"}->[$j]->{ to_key_name( item => $2 ) }
                = $1;
            $result_array[$i]->{"$name"}->[$j]->{ to_key_name( item => $5 ) }
                = $4;
        } elsif ( $val
            =~ /(\d+)\s+(input|output)\s+\w+,\s+(\d+)\s+\w+,\s+(\d+)\s+\w+,\s+(\d+)\s+\w+\s+\w+/
            ) {
            $result_array[$i]->{"$name"}->[$j]->{"$2_ols"}     = $1;
            $result_array[$i]->{"$name"}->[$j]->{"llr"}        = $3;
            $result_array[$i]->{"$name"}->[$j]->{"nos"}        = $4;
            $result_array[$i]->{"$name"}->[$j]->{"loop_units"} = $5;
        } elsif (
            $val =~ /Interface\s+(\w+\s+)*at\s+(\w+\s+\w+\s+\d+\s+\d+.*)/ ) {
            $result_array[$i]->{"interface_last_changed "} = $2;
        }
    }
    return \@result_array;
} ## end sub show_interface_fc

## @name ::show_interface_fc_capabilities
##
## @summary Parses the raw output from the "show int fc 1/1 capabilities" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_interface_fc_capabilities with any of its options as described in the
##    corresponding cdef file 'show_interface.cdefs'
## @synopsis
##      show_interface_fc_capabilities(%args);
## INPUTS TO METHOD CALL 'show_interface_fc_capabilities' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show int fc 1/1 capabilities
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# show int fc 1/1 capabilities
##&nbsp;fc1/1
##&nbsp; Min Speed is 1 Gbps
##&nbsp; Max Speed is 8 Gbps
##&nbsp; FC-PH Version (high, low)                         (0,6)
##&nbsp; Receive data field size (max/min)                 (2112/256) bytes
##&nbsp; Transmit data field size (max/min)                (2112/128) bytes
##&nbsp; Classes of Service supported are                  Class 2, Class 3, Class F
##&nbsp; Class 2 sequential delivery                       supported
##&nbsp; Class 3 sequential delivery                       supported
##&nbsp; Hold time (max/min)                               (100000/1) micro sec
##&nbsp; BB state change notification                      supported
##&nbsp; Maximum BB state change notifications             14
##&nbsp; Rate Mode change                                  not supported
##&nbsp;
##&nbsp; Rate Mode Capabilities                            Dedicated
##&nbsp;  Receive BB Credit modification supported         yes
##&nbsp;  FX mode Receive BB Credit (min/max/default)      (1/125/32)
##&nbsp;  ISL mode Receive BB Credit (min/max/default)     (2/125/32)
##&nbsp;  Performance buffer modification supported         no
##&nbsp;
##&nbsp; Out of Service capable                             yes
##&nbsp; Beacon mode configurable                           yes
##&nbsp; Extended B2B credit capable                         no
##&nbsp; On demand port activation license supported        yes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'out_of_service_capable' => 'yes',
##&nbsp;            'class_2_sequential_delivery' => 'supported',
##&nbsp;            'max_speed' => '8 Gbps',
##&nbsp;            'performance_buffer_modification_supported' => 'no',
##&nbsp;            'hold_time' => [
##&nbsp;                             {
##&nbsp;                               'min' => '1 micro sec',
##&nbsp;                               'max' => '100000 micro sec'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'receive_data_field_size' => [
##&nbsp;                                           {
##&nbsp;                                             'min' => '256 bytes',
##&nbsp;                                             'max' => '2112 bytes'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'interface_name' => 'fc1/1',
##&nbsp;            'beacon_mode_configurable' => 'yes',
##&nbsp;            'fx_mode_receive_bb_credit' => [
##&nbsp;                                             {
##&nbsp;                                               'min' => '1',
##&nbsp;                                               'max' => '125',
##&nbsp;                                               'default' => '32'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'isl_mode_receive_bb_credit' => [
##&nbsp;                                              {
##&nbsp;                                                'min' => '2',
##&nbsp;                                                'max' => '125',
##&nbsp;                                                'default' => '32'
##&nbsp;                                              }
##&nbsp;                                            ],
##&nbsp;            'extended_b2b_credit_capable' => 'no',
##&nbsp;            'class_3_sequential_delivery' => 'supported',
##&nbsp;            'min_speed' => '1 Gbps',
##&nbsp;            'bb_state_change_notification' => 'supported',
##&nbsp;            'on_demand_port_activation_license_supported' => 'yes',
##&nbsp;            'receive_bb_credit_modification_supported' => 'yes',
##&nbsp;            'rate_mode_change' => 'not supported',
##&nbsp;            'transmit_data_field_size' => [
##&nbsp;                                            {
##&nbsp;                                              'min' => '128 bytes',
##&nbsp;                                              'max' => '2112 bytes'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'classes_of_service_supported_are' => 'Class 2, Class 3, Class F',
##&nbsp;            'maximum_bb_state_change_notifications' => '14',
##&nbsp;            'fc_ph_version' => [
##&nbsp;                                 {
##&nbsp;                                   'high' => '0',
##&nbsp;                                   'low' => '6'
##&nbsp;                                 }
##&nbsp;                               ],
##&nbsp;            'rate_mode_capabilities' => 'Dedicated'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_interface_fc_capabilities {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    shift @lines;
    my @result_array;
    foreach my $line (@lines) {
        if ( $line =~ /(fc\d+\/\d+)/ ) {
            $result_array[0]->{interface_name} = $1;
        } elsif ( $line =~ /(M[in|ax]+\s+Speed)\sis\s(.*)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        } elsif (
            $line =~ /(.*)\s+\((\w+)\/(\w+)\)\s+\((\d+)\/(\d+)\)\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{$2}
                = "$4 $6";
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{$3}
                = "$5 $6";
        } elsif ( $line
            =~ /(.*)\s+\((\w+)\/(\w+)\/(\w+)\)\s+\((\d+)\/(\d+)\/(\d+)\)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{$2} = $5;
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{$3} = $6;
            $result_array[0]->{ to_key_name( item => $1 ) }->[0]->{$4} = $7;
        } elsif ( $line =~ /(.*)\s+\((\w+)\, (\w+)\)\s+\((\d+)\,(\d+)\)/ ) {
            my $temp = $1;
            $temp =~ tr/-/_/;
            $result_array[0]->{ to_key_name( item => $temp ) }->[0]->{$2}
                = $4;
            $result_array[0]->{ to_key_name( item => $temp ) }->[0]->{$3}
                = $5;
        } else {
            my @temp = split( /\s\s+/, $line );
            $result_array[0]->{ to_key_name( item => $temp[0] ) } = $temp[1];
        }
    }
    return \@result_array;
} ## end sub show_interface_fc_capabilities

1;
