#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC show port command
##
## @description
##    This is a library to parse the raw output of CiscoFC switch show port command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_port
##
##  show_port(%args)
##
## @author gupendra@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::CiscoFC::default::show_port;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    elsif ( $raw_output =~
     /(Invalid command.*)|(Invalid input options)|(Valid Port Range 0-47)|
    (This is not a slot based system)/i
      )
    {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::show_port
##
## @summary Parses the raw output from the "show port" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from show_port with any of its options as described in the 
##    corresponding cdef file 'show_port.cdefs'
## @synopsis
##      show_port(%args);
## INPUTS TO METHOD CALL 'show_port' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'fc' </td> <td> 'fiber-channel' </td> <td> '0' </td> </tr> 
## <tr> <td> 'info' </td> <td> 'show-data-structure-info' </td> <td> '1' </td> </tr> 
## <tr> <td> 'interface' </td> <td> 'interface' </td> <td> '1' </td> </tr> 
## <tr> <td> 'internal' </td> <td> 'internal-info' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = show port internal info interface fc 1/1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'operational_information' => [
##&nbsp;                                           {
##&nbsp;                                             'speed' => '4 Gbps',
##&nbsp;                                             'mode' => 'ALL',
##&nbsp;                                             'state' => 'down'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'port_wwn' => '20:01:54:7f:ee:01:2e:10',
##&nbsp;            'sfp' => [
##&nbsp;                       {
##&nbsp;                         'gigabit_eth_compliance_codes' => '0',
##&nbsp;                         'vendor_id' => '0:23:106',
##&nbsp;                         'vendor_revision_level' => 'G2.3',
##&nbsp;                         'vendor_name' => 'CISCO-AVAGO',
##&nbsp;                         'diagmontype_' => '0x68',
##&nbsp;                         'min_speed' => '2000',
##&nbsp;                         'max_speed' => '8000',
##&nbsp;                         'connector_type' => '7',
##&nbsp;                         'vendor_part_num' => 'SFBR-5780APZ-CS2',
##&nbsp;                         'module_type' => '8',
##&nbsp;                         'fc_transmitter_type' => 'Shortwave Laser(3)'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'port_capability_mode' => [
##&nbsp;                                        {
##&nbsp;                                          'min_speed' => '1000',
##&nbsp;                                          'max_speed' => '8000'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'port' => '1/1',
##&nbsp;            'license_file_used' => 'PORT_ACTIVATION_PKG',
##&nbsp;            'port_type' => 'external',
##&nbsp;            'license_scheme' => 'sequential',
##&nbsp;            'fcid' => '0x000000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_port {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => "show port",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $first_line = 'Modes:';
    my @mode_line  = get_lines(
        "raw_output" => $output,
        "first_line" => $first_line
    );
    shift @mode_line;

    my $first_line1 = 'SFP\s+info\s+-';
    my @sfp_line    = get_lines(
        "raw_output" => $output,
        "first_line" => $first_line1
    );
    shift @sfp_line;

    my @lines = get_lines( "raw_output" => $output );
    my @array = ();
    my %hash;
    my %info;
    foreach my $line (@mode_line) {
        if ( $line =~ /Min\s+Speed:\s+(\d+)/ ) {
            $info{"min_speed"} = $1;
        }
        elsif ( $line =~ /Max\s+Speed:\s+(\d+)/ ) {
            $info{"max_speed"} = $1;
            last;
        }
    }
    push( @array, \%info );
    $hash{"port_capability_mode"} = \@array;

    my @array1 = ();
    my %info1  = ();
    while ( my $line1 = shift(@sfp_line) ) {
        if ( $line1 =~ /Trunk Info -/i ) {
            last;
        }
        elsif ( $line1 =~ /(.*)[:|=]\s+(.*)/ ) {
            my $var1 = $1;
            my $var2 = $2;
            $var1 =~ s/\s+/_/g;
            $info1{to_key_name( item => $var1 )} = $var2; 
        }
    }
    push( @array1, \%info1 );
    $hash{"sfp"} = \@array1;

    my @array2 = ();
    foreach my $line2 (@lines) {
        if ( $line2 =~
/Operational Info - state\((\w+)\),\s+mode\((\w+)\),\s+speed\((.*)\),/
          )
        {
            my %info2 = ();
            $info2{"state"} = $1;
            $info2{"mode"}  = $2;
            $info2{"speed"} = $3;
            push( @array2, \%info2 );
            $hash{"operational_information"} = \@array2;
        }
        elsif ( $line2 =~ /fcid\((.*)\),/ ) {
            $hash{"fcid"} = $1;
        }
        elsif ( $line2 =~ /Port\s+WWN\s+is\s+(.*)/ ) {
            $hash{"port_wwn"} = $1;
        }
        elsif ( $line2 =~ /Port\s+type\s+is\s+(\w+)/ ) {
            $hash{"port_type"} = $1;
        }
        elsif ( $line2 =~ /Licensing\s+scheme\s+is\s+(\w+)/ ) {
            $hash{"license_scheme"} = $1;
        }
        elsif ( $line2 =~ /License\s+file\s+used\s+is\s+(\w+)/ ) {
            $hash{"license_file_used"} = $1;
        }
        elsif ( $line2 =~ /\w+(\d+\/\d+)\s+-\s+if_index:.*,\s+phy_port_index:.*/ ) {
            $hash{"port"} = $1;
        }
    }
    push( @result_array, \%hash );

    return \@result_array;
} ## end sub show_port

1;
