#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_version.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse CiscoFC show_version command
##
## @description
##    This is a library to parse the raw output of CiscoFC show_version command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_version
##
##  show_version(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_version;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error );
    if ( $raw_output =~ /Invalid command/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Incomplete command/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid number/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::show_version
##
## @summary Parses the raw output from the "show version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_version with any of its options as described in the
##    corresponding cdef file 'show_version.cdefs'
## @synopsis
##      show_version(%args);
## INPUTS TO METHOD CALL 'show_version' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# show version
##&nbsp;Cisco Nexus Operating System (NX-OS) Software
##&nbsp;TAC support: http://www.cisco.com/tac
##&nbsp;Documents: http://www.cisco.com/en/US/products/ps9372/tsd_products_support_series_home.html
##&nbsp;Copyright (c) 2002-2011, Cisco Systems, Inc. All rights reserved.
##&nbsp;The copyrights to certain works contained herein are owned by
##&nbsp;other third parties and are used and distributed under license.
##&nbsp;Some parts of this software are covered under the GNU Public
##&nbsp;License. A copy of the license is available at
##&nbsp;http://www.gnu.org/licenses/gpl.html.
##&nbsp;
##&nbsp;Software
##&nbsp;  BIOS:      version 1.0.19
##&nbsp;  loader:    version N/A
##&nbsp;  kickstart: version 5.2(1)
##&nbsp;  system:    version 5.2(1)
##&nbsp;  BIOS compile time:       02/01/10
##&nbsp;  kickstart image file is: bootflash:/m9100-s3ek9-kickstart-mz.5.2.1.bin
##&nbsp;  kickstart compile time:  12/25/2020 12:00:00 [07/16/2011 18:02:15]
##&nbsp;  system image file is:    bootflash:/m9100-s3ek9-mz.5.2.1.bin
##&nbsp;  system compile time:     6/7/2011 13:00:00 [07/16/2011 20:17:25]
##&nbsp;
##&nbsp;
##&nbsp;Hardware
##&nbsp;  cisco MDS 9148 FC (1 Slot) Chassis ("1/2/4/8 Gbps FC/Supervisor-3")
##&nbsp;  Motorola, e500v2  with 1036300 kB of memory.
##&nbsp;  Processor Board ID JAF1539CRFB
##&nbsp;
##&nbsp;  Device name: dpg-mcc-cisco9148-04-A-fabY
##&nbsp;  bootflash:    1000944 kB
##&nbsp;Kernel uptime is 12 day(s), 12 hour(s), 40 minute(s), 21 second(s)
##&nbsp;
##&nbsp;Last reset at 760893 usecs after  Wed Aug  8 16:04:19 2012
##&nbsp;
##&nbsp;  Reason: Reset Requested by CLI command reload
##&nbsp;  System version: 5.2(1)
##&nbsp;  Service:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cisco_software' => 'Nexus',
##&nbsp;            'kernel_uptime' => '12 day(s), 12 hour(s), 40 minute(s), 21 second(s)',
##&nbsp;            'software' => [
##&nbsp;                            {
##&nbsp;                              'system_compile_time' => '6/7/2011 13:00:00 [07/16/2011 20:17:25]',
##&nbsp;                              'system_image_file_is' => 'bootflash:/m9100-s3ek9-mz.5.2.1.bin',
##&nbsp;                              'system' => 'version 5.2(1)',
##&nbsp;                              'kickstart' => 'version 5.2(1)',
##&nbsp;                              'bios' => 'version 1.0.19',
##&nbsp;                              'bios_compile_time' => '02/01/10',
##&nbsp;                              'kickstart_image_file_is' => 'bootflash:/m9100-s3ek9-kickstart-mz.5.2.1.bin',
##&nbsp;                              'kickstart_compile_time' => '12/25/2020 12:00:00 [07/16/2011 18:02:15]',
##&nbsp;                              'loader' => 'version N/A'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'gnu_license_info' => 'http://www.gnu.org/licenses/gpl.html.',
##&nbsp;            'hardware' => [
##&nbsp;                            {
##&nbsp;                              'bootflash' => '1000944 kB',
##&nbsp;                              'device_name' => 'dpg-mcc-cisco9148-04-A-fabY',
##&nbsp;                              'processor_board_id' => 'JAF1539CRFB',
##&nbsp;                              'processor' => 'Motorola, e500v2 ',
##&nbsp;                              'chassis' => 'cisco MDS 9148 FC (1 Slot) Chassis ("1/2/4/8 Gbps FC/Supervisor-3")',
##&nbsp;                              'processor_memory' => '1036300 kB'
##&nbsp;                            }
##&nbsp;                          ],
##&nbsp;            'last_reset' => [
##&nbsp;                              {
##&nbsp;                                'system_version' => '5.2(1)',
##&nbsp;                                'reason' => 'Reset Requested by CLI command reload',
##&nbsp;                                'service' => '',
##&nbsp;                                'last_reset_at' => '760893 usecs after  Wed Aug  8 16:04:19 2012'
##&nbsp;                              }
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_version {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my ( $key, $plugins, $i );
    my $j = 0;

    while ( my $line = shift(@lines) ) {
        if ( $line =~ /^(Software|Hardware)/ ) {
            $key = to_key_name( item => "$1" );
            $i = 0;
        } elsif ( $line =~ /(Last reset)\s+at\s+(.*)/ ) {
            $key = to_key_name( item => "$1" );
            $result_array[0]->{$key}->[$i]->{last_reset_at} = $2;
        } elsif ( $line =~ /(^http:\/\/.*)/ ) {
            $result_array[0]->{gnu_license_info} = $1;
        } elsif (
            $line =~ /^Cisco\s+(.*)\s+Operating\s+System\s+.*\s+Software/ ) {
            $result_array[0]->{cisco_software} = "$1";
        } elsif ( $line =~ /(.*):\s+(.*)/ && defined($key) ) {
            my $key1 = $1;
            my $val1 = $2;
            $key1 =~ s/-/_/g;
            $result_array[0]->{$key}->[$i]->{ to_key_name( item => "$key1" ) }
                = $val1;
        } elsif ( $line =~ /(Processor\s+Board\s+ID)\s+(.*)/ ) {
            $result_array[0]->{$key}->[$i]->{ to_key_name( item => "$1" ) }
                = $2;
        } elsif ( $line =~ /(.*)\s+with\s+(.*)\s+of memory/ ) {
            $result_array[0]->{$key}->[$i]->{processor}        = $1;
            $result_array[0]->{$key}->[$i]->{processor_memory} = $2;
        } elsif ( $line =~ /(.*\s+Chassis\s+.*)/ ) {
            $result_array[0]->{$key}->[$i]->{chassis} = $1;
        } elsif ( $line =~ /(.*):\s*(.*)/ && defined($key) ) {
            my $key1 = $1;
            my $val1 = $2;
            $key1 =~ s/-/_/g;
            $result_array[0]->{$key}->[$i]->{ to_key_name( item => "$key1" ) }
                = $val1;
        } elsif ( $line =~ /(Kernel\s+uptime)\s+is\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $2;
        } elsif ( $line =~ /plugin/ ) {
            $key = "plugin";
            while ( $line = shift(@lines) ) {
                $plugins = "$plugins" . "$line";
            }
            my @pluggins = split( /,\s*/, $plugins );
            $result_array[0]->{plugin} = \@pluggins;
        }
    }
    return \@result_array;
} ## end sub show_version
1;
