#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_vsan.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC show_vsan
##
## @description
##    This is a library to parse the raw output of CiscoFC switch show_vsan.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_vsan
##
##  show_vsan(%args)
##
## @author veerap@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_vsan;

use strict;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if (   $raw_output =~ /vsan\s+\d+\s+not\s+configured/
        || $raw_output =~ /Incomplete\s+command/ )
    {
        $error         = "InvalidParamValue";
        $error_message = "Invalid parameter value";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::show_vsan
##
## @summary Parses the raw output from the "show vsan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_vsan with any of its options as described in the
##    corresponding cdef file 'show_vsan.cdefs'
## @synopsis
##      show_vsan(%args);
## INPUTS TO METHOD CALL 'show_vsan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'membership' </td> <td> 'membership' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vsan-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show vsan
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;# show vsan
##&nbsp;vsan 1 information
##&nbsp;         name:VSAN0001  state:suspended
##&nbsp;         interoperability mode:default
##&nbsp;         loadbalancing:src-id/dst-id/oxid
##&nbsp;         operational state:down
##&nbsp;
##&nbsp;vsan 22 information
##&nbsp;         name:mcc  state:active
##&nbsp;         interoperability mode:default
##&nbsp;         loadbalancing:src-id/dst-id/oxid
##&nbsp;         operational state:up
##&nbsp;
##&nbsp;vsan 33 information
##&nbsp;         name:van_unused  state:suspended
##&nbsp;         interoperability mode:default
##&nbsp;         loadbalancing:src-id/dst-id/oxid
##&nbsp;         operational state:down
##&nbsp;
##&nbsp;vsan 111 information
##&nbsp;         name:VSAN0111  state:active
##&nbsp;         interoperability mode:default
##&nbsp;         loadbalancing:src-id/dst-id/oxid
##&nbsp;         operational state:down
##&nbsp;
##&nbsp;vsan 4079:evfp_isolated_vsan
##&nbsp;
##&nbsp;vsan 4094:isolated_vsan
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan_id' => '1',
##&nbsp;            'name' => 'VSAN0001',
##&nbsp;            'loadbalancing' => 'src-id/dst-id/oxid',
##&nbsp;            'interoperability_mode' => 'default',
##&nbsp;            'operational_state' => 'down',
##&nbsp;            'state' => 'suspended'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_id' => '22',
##&nbsp;            'name' => 'mcc',
##&nbsp;            'loadbalancing' => 'src-id/dst-id/oxid',
##&nbsp;            'interoperability_mode' => 'default',
##&nbsp;            'operational_state' => 'up',
##&nbsp;            'state' => 'active'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_id' => '33',
##&nbsp;            'name' => 'van_unused',
##&nbsp;            'loadbalancing' => 'src-id/dst-id/oxid',
##&nbsp;            'interoperability_mode' => 'default',
##&nbsp;            'operational_state' => 'down',
##&nbsp;            'state' => 'suspended'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_id' => '111',
##&nbsp;            'name' => 'VSAN0111',
##&nbsp;            'loadbalancing' => 'src-id/dst-id/oxid',
##&nbsp;            'interoperability_mode' => 'default',
##&nbsp;            'operational_state' => 'down',
##&nbsp;            'state' => 'active'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_id' => '4079',
##&nbsp;            'type' => 'evfp_isolated_vsan'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_id' => '4094',
##&nbsp;            'type' => 'isolated_vsan'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_vsan {
    my %args = @_;
    my (@result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @lines = get_lines( raw_output => $output );
    my $i     = -1;
    my $k     = 0;
    my $j     = 0;
    my $x     = 0;
    my $y     = -1;
    foreach my $line (@lines) {

        if ( $line =~ /vsan\s+(\d+)\s+(information)/ ) {
            $result_array[$k]->{'vsan_id'} = $1;
            $i++;
            $k++;
        }
        elsif ($line =~ /(interoperability mode):(\S+)/
            || $line =~ /(loadbalancing):(\S+)/
            || $line =~ /(operational\s+state):(\S+)/ )
        {
            $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
        }
        elsif ( $line =~ /(name):(\S+)\s+(state):(\S+)/ ) {
            $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            $result_array[$i]->{ to_key_name( item => $3 ) } = $4;
            $j = $i + 1;
        }
        elsif ( $line =~ /vsan\s+(\d+):(\S+)/ ) {
            $result_array[$j]->{'vsan_id'} = $1;
            $result_array[$j]->{'type'}    = $2;
            $j++;

        }
        elsif ( $line =~ /vsan\s+(\d+)\s+(interfaces):(.*)/ ) {
            $y++;
            $x                                      = 0;
            $result_array[$y]->{'vsan_id'}          = $1;
        }
        elsif ( $line =~ /vsan\s+(\d+)\((\w+)\)\s+interfaces:(.*)/ ) {
            $y++;
            $x                                      = 0;
            $result_array[$y]->{'vsan_id'}          = $1;
            $result_array[$y]->{'type'}             = $2;
        }
        elsif ( $line =~ /fc.*\d+/ ) {
            my @arr = split( " ", $line );
            foreach my $var (@arr) {
                $result_array[$y]->{'interfaces'}->[$x] = $var;
                $x++;
            }
        }

    }

    return \@result_array;

}

1;
