#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/showsystem.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC show system and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch show system and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::showsystem
##
##  showsystem_uptime(%args);
##  showsystem_errorid(%args);
##  showsystem_default(%args);
##  showsystem_health(%args);
##  showsystem_health_statistics(%args);
##  showsystem_health_statistics_loopback(%args);
##
## @author benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::showsystem;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /(Invalid command at)|(Incomplete command)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    } elsif ( $raw_output
        =~ /(Invalid hex number)|(Invalid number, expecting 1 at)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    }
    return $error;
} ## end sub _check_error

## @name ::showsystem_uptime
##
## @summary Parses the raw output from the "showsystem_uptime" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system uptime with any of its options as described in the
##    corresponding cdef file 'showsystem_uptime.cdefs'
## @synopsis
##      showsystem_uptime(%args);
## INPUTS TO METHOD CALL 'showsystem_uptime' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show system uptime
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_uptime' => '2 days, 23 hours, 45 minutes, 3 seconds',
##&nbsp;            'system_uptime' => '2 days, 23 hours, 44 minutes, 9 seconds',
##&nbsp;            'system_start_time' => 'Mon Sep 26 13:50:13 2011'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub showsystem_uptime {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key, @result, @lines, %result_hash );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system uptime",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    foreach my $line (@lines) {
        if ( $line =~ /System start time:\s+(.*)/i ) {
            $result_hash{'system_start_time'} = $1;
        } elsif ( $line =~ /(\w+\s+\w+):\s+(.*)/i ) {
            my $uptime = $1;
            my $time   = $2;
            $uptime =~ tr/[A-Z ]/[a-z_]/;
            $result_hash{$uptime} = $time;
        }
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub showsystem_uptime

## @name ::showsystem_errorid
##
## @summary Parses the raw output from the "showsystem_errorid" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system error-id with any of its options as described in the
##    corresponding cdef file 'showsystem_errorid.cdefs'
## @synopsis
##      showsystem_errorid(%args);
## INPUTS TO METHOD CALL 'showsystem_errorid' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'hex-id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show system error-id 0x000000
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'error_description' => 'success',
##&nbsp;            'error_facility' => 'null'
##&nbsp;          }
##&nbsp;];
##&nbsp;
##&nbsp;Command = show system error-id list
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'hex_id' => '0x00000000',
##&nbsp;            'error_message' => 'SYSERR_SUCCESS',
##&nbsp;            'error_description' => 'success'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hex_id' => '0x40050000',
##&nbsp;            'error_message' => 'SYSLOG_UNUSED',
##&nbsp;            'error_description' => 'success'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hex_id' => '0x40190000',
##&nbsp;            'error_message' => 'UNUSED',
##&nbsp;            'error_description' => 'success'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hex_id' => '0x40190008',
##&nbsp;            'error_message' => 'VSAN_MANAGER_FAILURE',
##&nbsp;            'error_description' => 'failure'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hex_id' => '0x401A0014',
##&nbsp;            'error_message' => 'ERR_UNDEFINED',
##&nbsp;            'error_description' => 'undefined'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'hex_id' => '0x401D0004',
##&nbsp;            'error_message' => 'SYSERR_LCM_PSEL_INIT_FAILURE',
##&nbsp;            'error_description' => 'fu_global_init_failure'
##&nbsp;          },
##&nbsp;]
##&nbsp;

sub showsystem_errorid {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key, @result, @lines, %result_hash );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system error-id",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $option;
    if ( $command =~ /show system error-id list/ ) {
        $option = "list";
    } elsif ( $command =~ /show system error-id\s+\d+x\d+/ ) {
        $option = "hex-id";
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    if ( $option eq "hex-id" ) {
        foreach my $line (@lines) {
            if ( $line =~ /(\w+\s+\w+):\s+(.*)/i ) {
                my $error      = $1;
                my $error_text = $2;
                $error      =~ tr/[A-Z ]/[a-z_]/;
                $error_text =~ s/\(|\)//g;
                $result_hash{$error} = $error_text;
            }
        }
        push @result, \%result_hash;
    } elsif ( $option eq "list" ) {
        my $i = -1;
        foreach my $line (@lines) {
            if ( $line =~ /\s*(\w+)\s+\((\w+)\)s*:\s+\"(.*)\"?/i ) {
                $i++;
                $result[$i]->{'hex_id'}            = $1;
                $result[$i]->{'error_message'}     = $2;
                $result[$i]->{'error_description'} = $3;
            }
        }
    }
    return \@result;
} ## end sub showsystem_errorid

## @name ::showsystem_default
##
## @summary Parses the raw output from the "showsystem_default" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system default with any of its options as described in the
##    corresponding cdef file 'showsystem_default.cdefs'
## @synopsis
##      showsystem_default(%args);
## INPUTS TO METHOD CALL 'showsystem_default' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show system default zone
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'gs' => 'read-write',
##&nbsp;            'default_zone' => 'deny',
##&nbsp;            'mode' => 'basic',
##&nbsp;            'distribute' => 'active only'
##&nbsp;          }
##&nbsp;];
##&nbsp;
##&nbsp;Command = show system default switchport
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'port_state' => 'up',
##&nbsp;            'trunk_mode' => 'on',
##&nbsp;            'link_failure_syslog_logging_level' => '5'
##&nbsp;          }
##&nbsp;];
##&nbsp;

sub showsystem_default {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key, @result, @lines, %result_hash );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system default",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $option;
    if ( $command =~ /show system default zone/i ) {
        $option = "zone";
    } elsif ( $command =~ /show system default switchport/i ) {
        $option = "switchport";
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    if ( $option eq "zone" ) {
        foreach my $line (@lines) {
            if ( $line =~ /^(\w+\s+){3}(\S+)\s+(\S+(\s+\S+)?)$/i ) {
                my $zone   = $2;
                my $status = $3;
                $zone =~ tr/-/_/;
                $result_hash{$zone} = $status;
            }
        }
    } elsif ( $option eq "switchport" ) {
        foreach my $line (@lines) {
            if ( $line =~ /^(\w+\s+){2}(.*) is (\w+)/i ) {
                my $switch_port = $2;
                my $status      = $3;
                $switch_port =~ tr/ /_/;
                $result_hash{$switch_port} = $status;
            }
        }
    }
    push @result, \%result_hash;
    return \@result;
} ## end sub showsystem_default

## @name ::showsystem_health
##
## @summary Parses the raw output from the "showsystem_health" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system health with any of its options as described in the
##    corresponding cdef file 'showsystem_health.cdefs'
## @synopsis
##      showsystem_health(%args);
## INPUTS TO METHOD CALL 'showsystem_health' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'module' </td> <td> 'module' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show system health
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'frequency' => '5 Sec',
##&nbsp;            'test' => 'InBand',
##&nbsp;            'status' => 'Running',
##&nbsp;            'action' => 'Enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '10 Sec',
##&nbsp;            'test' => 'Bootflash',
##&nbsp;            'status' => 'Running',
##&nbsp;            'action' => 'Enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '5 Sec',
##&nbsp;            'test' => 'Management Port',
##&nbsp;            'status' => 'Running',
##&nbsp;            'action' => 'Enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '5 Sec',
##&nbsp;            'test' => 'Loopback',
##&nbsp;            'status' => 'Running',
##&nbsp;            'action' => 'Enabled'
##&nbsp;          }
##&nbsp;];
##&nbsp;

sub showsystem_health {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, @result, @lines );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system health",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    foreach my $line (@lines) {
        my %result_hash;
        if ( $line =~ /(\w+|\w+\s+\w+)\s+(\d+\s+\w+)\s+(\w+)\s+(\w+)/ ) {
            $result_hash{'test'}      = $1;
            $result_hash{'frequency'} = $2;
            $result_hash{'status'}    = $3;
            $result_hash{'action'}    = $4;
            push @result, \%result_hash;
        }
    }
    return \@result;
} ## end sub showsystem_health

## @name ::showsystem_health_statistics
##
## @summary Parses the raw output from the "showsystem_health_statistics" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system health statistics with any of its options as described in the
##    corresponding cdef file 'showsystem_health_statistics.cdefs'
## @synopsis
##      showsystem_health_statistics(%args);
## INPUTS TO METHOD CALL 'showsystem_health_statistics' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'module' </td> <td> 'module' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show system health statistics
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'frequency' => '5s',
##&nbsp;            'pass' => '82086',
##&nbsp;            'run' => '82086',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'test_name' => 'InBand'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '10s',
##&nbsp;            'pass' => '41043',
##&nbsp;            'run' => '41043',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'test_name' => 'Bootflash'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '5s',
##&nbsp;            'pass' => '82086',
##&nbsp;            'run' => '82086',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'test_name' => 'Management Port'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'frequency' => '5s',
##&nbsp;            'pass' => '82086',
##&nbsp;            'run' => '82086',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'test_name' => 'Loopback'
##&nbsp;          }
##&nbsp;];
##&nbsp;

sub showsystem_health_statistics {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, @result, @lines );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system health statistics",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    my ( $test, $status );
    foreach my $line (@lines) {
        if ( $line
            =~ /(\w+|\w+\s+\w+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
            ) {
            my %result_hash;
            $result_hash{'frequency'} = $3;
            $result_hash{'run'}       = $4;
            $result_hash{'pass'}      = $5;
            $result_hash{'fail'}      = $6;
            $result_hash{'cfail'}     = $7;
            $result_hash{'errors'}    = $8;
            $result_hash{'test_name'} = $1;
            $result_hash{'status'}    = $2;
            push @result, \%result_hash;
        }
    }

    return \@result;
} ## end sub showsystem_health_statistics

## @name ::showsystem_health_statistics_loopback
##
## @summary Parses the raw output from the "showsystem_health_statistics_loopback" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show system health statistics loopback with any of its options as described in the
##    corresponding cdef file 'showsystem_health_statistics_loopback.cdefs'
## @synopsis
##      showsystem_health_statistics_loopback(%args);
## INPUTS TO METHOD CALL 'showsystem_health_statistics_loopback' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'module' </td> <td> 'module' </td> </tr>
## <tr> <td> 'interface' </td> <td> 'interface' </td> </tr>
## <tr> <td> 'timelog' </td> <td> 'timelog' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = show system health statistics loopback module 1
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pass' => '82086',
##&nbsp;            'run' => '82086',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'port' => '24',
##&nbsp;            'module' => '1'
##&nbsp;          }
##&nbsp;];
##&nbsp;
##&nbsp;Command = show system health statistics loopback interface fc1/1
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pass' => '0',
##&nbsp;            'run' => '0',
##&nbsp;            'errors' => '0',
##&nbsp;            'cfail' => '0',
##&nbsp;            'fail' => '0',
##&nbsp;            'status' => 'Running',
##&nbsp;            'port' => '1',
##&nbsp;            'module' => '1'
##&nbsp;          }
##&nbsp;];
##&nbsp;
##&nbsp;Command = show system health statistics loopback timelog
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'samples' => '82086',
##&nbsp;            'avg' => '608',
##&nbsp;            'min' => '578',
##&nbsp;            'max' => '509699',
##&nbsp;            'module' => '1'
##&nbsp;          }
##&nbsp;];
##&nbsp;

sub showsystem_health_statistics_loopback {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, @result, @lines, %result_hash, $option );

    $error = _check_error(
        raw_output    => $output,
        command       => "show system health statistics loopback",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    if ($command =~ /show\s+system\s+health\s+statistics\s+loopback\s+(\w+)/ )
    {
        $option = $1;
    }

    @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
    );

    my ( $test, $status );
    if ( ( $option =~ /(module)|(interface)/ ) || ( !$option ) ) {
        foreach my $line (@lines) {
            if ( $line
                =~ /(\d+)\s+(\d+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/
                ) {
                $status                = $3;
                $result_hash{'module'} = $1;
                $result_hash{'port'}   = $2;
                $result_hash{'run'}    = $4;
                $result_hash{'pass'}   = $5;
                $result_hash{'fail'}   = $6;
                $result_hash{'cfail'}  = $7;
                $result_hash{'errors'} = $8;
                $result_hash{'status'} = $3;
                push @result, \%result_hash;
            }
        }
    } elsif ( $option =~ /timelog/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
                $result_hash{'module'}  = $1;
                $result_hash{'samples'} = $2;
                $result_hash{'min'}     = $3;
                $result_hash{'max'}     = $4;
                $result_hash{'avg'}     = $5;
                push @result, \%result_hash;
            }
        }
    }
    return \@result;
} ## end sub showsystem_health_statistics_loopback
1;
