#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/showzone.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC showzone and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch showzone and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::showzone
##
##  show_zone(%args)
##  show_zome_ess(%args)
##  show_zome_member(%args)
##  show_zone_policy(%args)
##  show_zone_statistics(%args)
##  show_zone_status(%args)
##  show_zone_internal(%args)
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::showzone;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if ($raw_output =~ /Invalid command/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ($raw_output =~ /Incomplete command/si) {
        $error = "MissingRequiredParam";
        $$error_message = "Required options are missing in the command";
    } elsif ($raw_output =~ /not configured/si) {
        $error = "InvalidParamValue";
        $$error_message = "Invalid param value present in command";
    }

    return $error;
}

## @name ::show_zone
##
## @summary Parses the raw output from the "show zone" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone(%args);
## INPUTS TO METHOD CALL 'show_zone' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## <tr> <td> 'name' </td> <td> 'name' </td> </tr>
## <tr> <td> 'active' </td> <td> 'active' </td> </tr>
## <tr> <td> 'pending' </td> <td> 'pending' </td> </tr>
## <tr> <td> 'pending-diff' </td> <td> 'pending_diff' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;zone name xiv_mod7_v60301a_6a vsan 1
##&nbsp;
##&nbsp;zone name Z_FC1_23_FC1_22_V1 vsan 1
##&nbsp;   interface fc1/23 swwn 20:00:00:0d:ec:3e:c0:80
##&nbsp;   interface fc1/22 swwn 20:00:00:0d:ec:3e:c0:80
##&nbsp;
##&nbsp;zone name Z_FC1_24_FC1_20_V1 vsan 1
##&nbsp;   interface fc1/24 swwn 20:00:00:0d:ec:3e:c0:80
##&nbsp;   interface fc1/20 swwn 20:00:00:0d:ec:3e:c0:80
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan' => '1',
##&nbsp;            'zone_name' => 'xiv_mod7_v60301a_6a'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'members' => [
##&nbsp;                           {
##&nbsp;                             'swwn' => '20:00:00:0d:ec:3e:c0:80',
##&nbsp;                             'interface' => 'fc1/23'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'swwn' => '20:00:00:0d:ec:3e:c0:80',
##&nbsp;                             'interface' => 'fc1/22'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'vsan' => '1',
##&nbsp;            'zone_name' => 'Z_FC1_23_FC1_22_V1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'members' => [
##&nbsp;                           {
##&nbsp;                             'swwn' => '20:00:00:0d:ec:3e:c0:80',
##&nbsp;                             'interface' => 'fc1/24'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'swwn' => '20:00:00:0d:ec:3e:c0:80',
##&nbsp;                             'interface' => 'fc1/20'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'vsan' => '1',
##&nbsp;            'zone_name' => 'Z_FC1_24_FC1_20_V1'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show zone name test_zone
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;zone name test_zone vsan 3
##&nbsp;  pwwn 50:06:01:6f:3c:e0:14:de
##&nbsp;  pwwn 50:0a:09:80:00:02:51:1f [tech7]
##&nbsp;  fcalias name alias1 vsan 3
##&nbsp;    pwwn 50:0a:09:80:00:02:51:1f [tech7]
##&nbsp;    pwwn 50:0a:09:80:00:02:37:1f
##&nbsp;    pwwn 50:01:73:80:01:0a:01:83
##&nbsp;
##&nbsp;  fcalias name alias3 vsan 3
##&nbsp;    pwwn 50:0a:09:80:00:02:37:1f
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'members' => [
##&nbsp;                           {
##&nbsp;                             'pwwn' => '50:06:01:6f:3c:e0:14:de'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'device-alias' => 'tech7',
##&nbsp;                             'pwwn' => '50:0a:09:80:00:02:51:1f'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'members' => [
##&nbsp;                                            {
##&nbsp;                                              'device-alias' => 'tech7',
##&nbsp;                                              'pwwn' => '50:0a:09:80:00:02:51:1f'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'pwwn' => '50:0a:09:80:00:02:37:1f'
##&nbsp;                                            },
##&nbsp;                                            {
##&nbsp;                                              'pwwn' => '50:01:73:80:01:0a:01:83'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                             'vsan' => '3',
##&nbsp;                             'fcalias_name' => 'alias1'
##&nbsp;                           },
##&nbsp;                           {
##&nbsp;                             'members' => [
##&nbsp;                                            {
##&nbsp;                                              'pwwn' => '50:0a:09:80:00:02:37:1f'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;                             'vsan' => '3',
##&nbsp;                             'fcalias_name' => 'alias3'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;            'vsan' => '3',
##&nbsp;            'zone_name' => 'test_zone'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone

sub _show_zone {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    my ($k1, $k2);
    my (@array1, @array2, $flag, $flag2, $var1, $var2);

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if ($line =~ m/.*(zone name)\s+(.*)\s+(vsan)\s+(.*)/) {
            $i++;
            $k1 = -1;
            $flag = 1;
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
            $result_array[$i]->{to_key_name(item=>$3)} = $4;
            next;
        } elsif ($line =~ m/.*(fcalias name)\s+(.*)\s+(vsan)\s+(.*)/) {
            $k1++;
            $k2 = -1;
            $flag = 2;
            $result_array[$i]->{members}->[$k1]->{to_key_name(item=>$1)} = $2;
            $result_array[$i]->{members}->[$k1]->{to_key_name(item=>$3)} = $4;
            next;
        } elsif ($line =~ m/((?:device-alias|domain-id|fcid|fwwn|interface|ip-address|pwwn|symbolic-nodename).*)/) {
            $var1 = $1;
            $var2 = undef;
            $flag2 = 0;
            if ($var1 !~ m/^interface/) {
                if ($var1 =~ m/([A-Za-z0-9:]+\s+[A-Za-z0-9:]+)\s+(.*)/) {
                    $var1 = $1;
                    $var2 = $2;
                }
            }
            if ($var2) {
                $var2 =~ s/[\[\]]//g;
            }
            @array2 = split (/\s+/, $var2);
            if ($#array2 >= 1) {
                $flag2 = 1;
            }
            @array1 = split (/\s+/, $var1);
            if ($flag == 1) {
                $k1++;
                for (my $n=0; $n<=$#array1; $n++) {
                    if ($n % 2 == 0) {
                        $result_array[$i]->{members}->[$k1]->{to_key_name(item=>$array1[$n])} = $array1[$n+1];
                        if ($flag2 == 1 and $#array2 == 1) {
                            $result_array[$i]->{members}->[$k1]->{to_key_name(item=>$array2[0])} = $array2[1];
                        } elsif ($flag2 == 0 and $#array2 == 0) {
                            $result_array[$i]->{members}->[$k1]->{to_key_name(item=>'device-alias')} = $array2[0];
                        }
                    }
                }
            } elsif ($flag == 2) {
                $k2++;
                for (my $n=0; $n<=$#array1; $n++) {
                    if ($n % 2 == 0) {
                        $result_array[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>$array1[$n])} = $array1[$n+1];
                        if ($flag2 == 1 and $#array2 == 1) {
                            $result_array[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>$array2[0])} = $array2[1];
                        } elsif ($flag2 == 0 and $#array2 == 0) {
                            $result_array[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>'device-alias')} = $array2[0];
                        }
                    }
                }
            }
        }
    }

    return \@result_array;
}

## @name ::show_zone_ess
##
## @summary Parses the raw output from the "show zone ess" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_ess with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_ess(%args);
## INPUTS TO METHOD CALL 'show_zone_ess' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone ess
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ESS info on VSAN 1 :
##&nbsp;    Domain : 27, SWWN : 20:01:00:0d:ec:3e:c0:81, Cap1 : 0xfb, Cap2 : 0x0
##&nbsp;Vendor ESS info on VSAN 1 :
##&nbsp;    Domain : 27, SWWN : 20:01:00:0d:ec:3e:c0:81, Cap1 : 0x1, Cap2 : 0x0
##&nbsp;ESS info on VSAN 3 :
##&nbsp;    Domain : 61, SWWN : 20:03:00:0d:ec:3e:c0:81, Cap1 : 0xf9, Cap2 : 0x0
##&nbsp;Vendor ESS info on VSAN 3 :
##&nbsp;    Domain : 61, SWWN : 20:03:00:0d:ec:3e:c0:81, Cap1 : 0x1, Cap2 : 0x0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vendor_ess_info' => [
##&nbsp;                                   {
##&nbsp;                                     'cap2' => '0x0',
##&nbsp;                                     'swwn' => '20:01:00:0d:ec:3e:c0:81',
##&nbsp;                                     'domain' => '27',
##&nbsp;                                     'cap1' => '0x1',
##&nbsp;                                     'vsan' => '1'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'ess_info' => [
##&nbsp;                            {
##&nbsp;                              'cap2' => '0x0',
##&nbsp;                              'swwn' => '20:01:00:0d:ec:3e:c0:81',
##&nbsp;                              'domain' => '27',
##&nbsp;                              'cap1' => '0xfb',
##&nbsp;                              'vsan' => '1'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vendor_ess_info' => [
##&nbsp;                                   {
##&nbsp;                                     'cap2' => '0x0',
##&nbsp;                                     'swwn' => '20:03:00:0d:ec:3e:c0:81',
##&nbsp;                                     'domain' => '61',
##&nbsp;                                     'cap1' => '0x1',
##&nbsp;                                     'vsan' => '3'
##&nbsp;                                   }
##&nbsp;                                 ],
##&nbsp;            'ess_info' => [
##&nbsp;                            {
##&nbsp;                              'cap2' => '0x0',
##&nbsp;                              'swwn' => '20:03:00:0d:ec:3e:c0:81',
##&nbsp;                              'domain' => '61',
##&nbsp;                              'cap1' => '0xf9',
##&nbsp;                              'vsan' => '3'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_ess {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_ess(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_ess

sub _show_zone_ess {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    my (@array1, $flag, $var1);

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if ($line =~ m/^ESS\s+info\s+on\s+VSAN\s+(\d+)/) {
            $i++;
            $flag = 1;
            $result_array[$i]->{ess_info}->[0]->{vsan} = $1;
            next;
        } elsif ($line =~ m/^Vendor\s+ESS\s+info\s+on\s+VSAN\s+(\d+)/) {
            $flag = 2;
            $result_array[$i]->{vendor_ess_info}->[0]->{vsan} = $1;
            next;
        } elsif ($line =~ /.*Domain\s+:\s+(\d+),\s+SWWN\s+:\s+(.*),\s+Cap1\s+:\s+(.*),\s+Cap2\s+:\s+(.*)/) {
            if ($flag == 1) {
                $result_array[$i]->{ess_info}->[0]->{domain} = $1;
                $result_array[$i]->{ess_info}->[0]->{swwn} = $2;
                $result_array[$i]->{ess_info}->[0]->{cap1} = $3;
                $result_array[$i]->{ess_info}->[0]->{cap2} = $4;
            } elsif ($flag == 2) {
                $result_array[$i]->{vendor_ess_info}->[0]->{domain} = $1;
                $result_array[$i]->{vendor_ess_info}->[0]->{swwn} = $2;
                $result_array[$i]->{vendor_ess_info}->[0]->{cap1} = $3;
                $result_array[$i]->{vendor_ess_info}->[0]->{cap2} = $4;
            }
        }
    }

    return \@result_array;
}

## @name ::show_zone_member
##
## @summary Parses the raw output from the "show zone member" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_member with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_member(%args);
## INPUTS TO METHOD CALL 'show_zone_member' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'device-alias' </td> <td> 'device_alias' </td> </tr>
## <tr> <td> 'fcalias' </td> <td> 'fcalias' </td> </tr>
## <tr> <td> 'pwwn' </td> <td> 'pwwn' </td> </tr>
## <tr> <td> 'fcid' </td> <td> 'fcid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone member device-alias tech7
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;device-alias tech7 vsan 1
##&nbsp;  zone Z_FC1_24_FC1_20_V1
##&nbsp;  zone falcon_zone2
##&nbsp;  zone falcon_zone2_non_mkby
##&nbsp;
##&nbsp;device-alias tech7 vsan 3
##&nbsp;  zone test_zone
##&nbsp;  fcalias alias1
##&nbsp;      zone test_zone
##&nbsp;  fcalias alias2
##&nbsp;      zone test_zone
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;         {
##&nbsp;           'device-alias' => 'tech7',
##&nbsp;           'vsan' => '1',
##&nbsp;           'zones' => [
##&nbsp;                        {
##&nbsp;                          'name' => 'Z_FC1_24_FC1_20_V1'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'name' => 'falcon_zone2'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'name' => 'falcon_zone2_non_mkby'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;         },
##&nbsp;         {
##&nbsp;           'device-alias' => 'tech7',
##&nbsp;           'fcalias' => [
##&nbsp;                          {
##&nbsp;                            'name' => 'alias1',
##&nbsp;                            'zones' => [
##&nbsp;                                         'test_zone'
##&nbsp;                                       ]
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'name' => 'alias2',
##&nbsp;                            'zones' => [
##&nbsp;                                         'test_zone'
##&nbsp;                                       ]
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;           'vsan' => '3',
##&nbsp;           'zones' => [
##&nbsp;                        {
##&nbsp;                          'name' => 'test_zone'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;         }
##&nbsp;       ];
##&nbsp;
##&nbsp;Command = show zone member pwwn 50:0a:09:81:00:02:37:2a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;pwwn 50:0a:09:81:00:02:37:2a vsan 1
##&nbsp;  zone vgv3070f18b_0d_184
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'pwwn' => '50:0a:09:81:00:02:37:2a',
##&nbsp;            'vsan' => '1',
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'name' => 'vgv3070f18b_0d_184'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_member {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_member(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_member

sub _show_zone_member {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    my $j = -1;
    my $k;
    my $flag;

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if ($line =~ m/(device-alias|fcalias|fcid|pwwn)\s+([a-zA-Z0-9:]+)\s+(vsan)\s+(\d+)/) {
            $k = -1;
            $i++;
            $flag = 1;
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
            $result_array[$i]->{to_key_name(item=>$3)} = $4;
        } elsif ($line =~ m/fcalias\s+(.*)/) {
            $k = -1;
            $j++;
            $flag = 2;
            $result_array[$i]->{'fcalias'}->[$j]->{'name'} = $1;
        } elsif ($line =~ m/zone\s+(.*)/) {
            $k++;
            if ($flag == 1) {
                $result_array[$i]->{'zones'}->[$k]->{'name'} = $1;
            } elsif ($flag == 2) {
                $result_array[$i]->{'fcalias'}->[$j]->{'zones'}->[$k] = $1;
            }
        }
    }

    return \@result_array;
}

## @name ::show_zone_policy
##
## @summary Parses the raw output from the "show zone policy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_policy with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_policy(%args);
## INPUTS TO METHOD CALL 'show_zone_policy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## <tr> <td> 'pending' </td> <td> 'pending' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone policy
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Vsan: 1
##&nbsp;   Default-zone: deny
##&nbsp;   Distribute: full
##&nbsp;   Broadcast: unsupported
##&nbsp;   Merge control: allow
##&nbsp;   Generic Service: read-write
##&nbsp;
##&nbsp;Vsan: 3
##&nbsp;   Default-zone: deny
##&nbsp;   Distribute: active only
##&nbsp;   Broadcast: unsupported
##&nbsp;   Merge control: allow
##&nbsp;   Generic Service: read-write
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'merge_control' => 'allow',
##&nbsp;            'broadcast' => 'unsupported',
##&nbsp;            'default-zone' => 'deny',
##&nbsp;            'generic_service' => 'read-write',
##&nbsp;            'distribute' => 'full',
##&nbsp;            'vsan' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'merge_control' => 'allow',
##&nbsp;            'broadcast' => 'unsupported',
##&nbsp;            'default-zone' => 'deny',
##&nbsp;            'generic_service' => 'read-write',
##&nbsp;            'distribute' => 'active only',
##&nbsp;            'vsan' => '3'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_policy {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_policy(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_policy

sub _show_zone_policy {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if ($line =~ m/(vsan):\s+(\d+)/i) {
            $i++;
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
        } elsif ($line =~ m/(Default-zone|Distribute|Broadcast|Merge\s+control|Generic\s+Service):\s+(.*)/) {
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
        }
    }

    return \@result_array;
}

## @name ::show_zone_statistics
##
## @summary Parses the raw output from the "show zone statistics" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_statistics with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_statistics(%args);
## INPUTS TO METHOD CALL 'show_zone_statistics' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## <tr> <td> 'lun-zoning' </td> <td> 'lun_zoning' </td> </tr>
## <tr> <td> 'read-only-zoning' </td> <td> 'read_only_zoning' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone statistics
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Statistics For VSAN: 1
##&nbsp;**********************************
##&nbsp;Number of Merge Requests Sent: 0
##&nbsp;Number of Merge Requests Recvd: 0
##&nbsp;Number of Merge Accepts Sent: 0
##&nbsp;Number of Merge Accepts Recvd: 0
##&nbsp;Number of Merge Rejects Sent: 0
##&nbsp;Number of Merge Rejects Recvd: 0
##&nbsp;Number of Change Requests Sent: 0
##&nbsp;Number of Change Requests Recvd: 0
##&nbsp;Number of Change Rejects Sent: 0
##&nbsp;Number of Change Rejects Recvd: 0
##&nbsp;Number of GS Requests Recvd: 0
##&nbsp;Number of GS Requests Rejected: 0
##&nbsp;
##&nbsp;Statistics For VSAN: 3
##&nbsp;**********************************
##&nbsp;Number of Merge Requests Sent: 0
##&nbsp;Number of Merge Requests Recvd: 0
##&nbsp;Number of Merge Accepts Sent: 0
##&nbsp;Number of Merge Accepts Recvd: 0
##&nbsp;Number of Merge Rejects Sent: 0
##&nbsp;Number of Merge Rejects Recvd: 0
##&nbsp;Number of Change Requests Sent: 0
##&nbsp;Number of Change Requests Recvd: 0
##&nbsp;Number of Change Rejects Sent: 0
##&nbsp;Number of Change Rejects Recvd: 0
##&nbsp;Number of GS Requests Recvd: 0
##&nbsp;Number of GS Requests Rejected: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_change_rejects_sent' => '0',
##&nbsp;            'number_of_merge_rejects_recvd' => '0',
##&nbsp;            'number_of_merge_accepts_recvd' => '0',
##&nbsp;            'number_of_change_rejects_recvd' => '0',
##&nbsp;            'number_of_change_requests_sent' => '0',
##&nbsp;            'number_of_gs_requests_recvd' => '0',
##&nbsp;            'number_of_merge_accepts_sent' => '0',
##&nbsp;            'number_of_merge_requests_sent' => '0',
##&nbsp;            'statistics_for_vsan' => '1',
##&nbsp;            'number_of_change_requests_recvd' => '0',
##&nbsp;            'number_of_merge_rejects_sent' => '0',
##&nbsp;            'number_of_gs_requests_rejected' => '0',
##&nbsp;            'number_of_merge_requests_recvd' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'number_of_change_rejects_sent' => '0',
##&nbsp;            'number_of_merge_rejects_recvd' => '0',
##&nbsp;            'number_of_merge_accepts_recvd' => '0',
##&nbsp;            'number_of_change_rejects_recvd' => '0',
##&nbsp;            'number_of_change_requests_sent' => '0',
##&nbsp;            'number_of_gs_requests_recvd' => '0',
##&nbsp;            'number_of_merge_accepts_sent' => '0',
##&nbsp;            'number_of_merge_requests_sent' => '0',
##&nbsp;            'statistics_for_vsan' => '3',
##&nbsp;            'number_of_change_requests_recvd' => '0',
##&nbsp;            'number_of_merge_rejects_sent' => '0',
##&nbsp;            'number_of_gs_requests_rejected' => '0',
##&nbsp;            'number_of_merge_requests_recvd' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_statistics {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_statistics(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_statistics

sub _show_zone_statistics {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    foreach my $line (@arr) {
        next if ($line =~ m/^[\*]+$/);
        next if ($line =~ m/^\n$/);
        if ($line =~ m/(Statistics For VSAN):\s+(\d+)/i) {
            $i++;
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
        } elsif ($line =~ m/(Number[A-Za-z\s]+):\s+(\d+)/) {
            $result_array[$i]->{to_key_name(item=>$1)} = $2;
        }
    }

    return \@result_array;
}

## @name ::show_zone_status
##
## @summary Parses the raw output from the "show zone status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_status with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_status(%args);
## INPUTS TO METHOD CALL 'show_zone_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'global' </td> <td> 'global' </td> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone status vsan 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VSAN: 1 default-zone: deny distribute: full Interop: default
##&nbsp;    mode: enhanced merge-control: allow
##&nbsp;    session: none
##&nbsp;    hard-zoning: enabled broadcast: unsupported
##&nbsp;Default zone:
##&nbsp;    qos: none broadcast: unsupported ronly: disabled
##&nbsp;Full Zoning Database :
##&nbsp;    DB size: 4076 bytes
##&nbsp;    Zonesets:3  Zones:38 Aliases: 0 Attribute-groups: 1
##&nbsp;Active Zoning Database :
##&nbsp;    DB size: 640 bytes
##&nbsp;    Name: VSeries_Zoneset  Zonesets:1  Zones:10
##&nbsp;Status:
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'full_zoning_database' => [
##&nbsp;                                        {
##&nbsp;                                          'zonesets' => '3 ',
##&nbsp;                                          'aliases' => '0 Attribute-groups: 1',
##&nbsp;                                          'zones' => '38',
##&nbsp;                                          'db_size' => '4076 bytes'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'active_zoning_database' => [
##&nbsp;                                          {
##&nbsp;                                            'zonesets' => '1 ',
##&nbsp;                                            'name' => 'VSeries_Zoneset ',
##&nbsp;                                            'zones' => '10',
##&nbsp;                                            'db_size' => '640 bytes'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'default_zone' => [
##&nbsp;                                {
##&nbsp;                                  'broadcast' => 'unsupported',
##&nbsp;                                  'qos' => 'none',
##&nbsp;                                  'ronly' => 'disabled'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'status' => [],
##&nbsp;            'vsan_info' => [
##&nbsp;                             {
##&nbsp;                               'mode' => 'enhanced',
##&nbsp;                               'merge-control' => 'allow',
##&nbsp;                               'broadcast' => 'unsupported',
##&nbsp;                               'default-zone' => 'deny',
##&nbsp;                               'interop' => 'default',
##&nbsp;                               'session' => 'none',
##&nbsp;                               'vsan' => '1',
##&nbsp;                               'distribute' => 'full',
##&nbsp;                               'hard-zoning' => 'enabled'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_status {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_status(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_status

sub _show_zone_status {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    my $i = -1;
    my $key;

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if ($line =~ m/(FC2\s+throttle)\s+is\s+(.*)/) {
            $result_array[0]->{to_key_name(item=>$1)} = $2;
        } elsif ($line =~ m/(Zoneset\s+activate\s+sanity\s+threshold\s+value\s+in\s+bytes):\s+(.*)/) {
            $result_array[0]->{to_key_name(item=>$1)} = $2;
        }
        next if ($line =~ m/Database Not Available/);
        if ($line =~ m/VSAN:\s+\d+/) {
            $i++;
        } elsif ($line =~ m/((?:Default zone)|(?:Full Zoning Database)|(?:Active Zoning Database)|(?:Status))/i) {
            $key = to_key_name(item=>$1);
            $result_array[$i]->{$key} = [];
            next;
        }
        if (($line =~ m/(VSAN):\s+(.*)\s+(default-zone):\s+(.*)\s+(distribute):\s+(.*)\s+(Interop):\s+(.*)/) or ($line =~ m/(mode):\s+(.*)\s+(merge-control):\s+(.*)/) or ($line =~ m/(session):\s+(.*)/) or ($line =~ m/(hard-zoning):\s+(.*)\s+(broadcast):\s+(.*)/) ) {
            if ($1 and $2) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)} = $2;
            }
            if ($3 and $4) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$3)} = $4;
            }
            if ($5 and $6) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$5)} = $6;
            }
            if ($7 and $8) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$7)} = $8;
            }
        } elsif (($line =~ m/(qos):\s+(.*)\s+(broadcast):\s+(.*)\s+(ronly):\s+(.*)/) or ($line =~ m/(DB size):\s+(.*)/) or ($line =~ m/(Zonesets):(.*)\s+(Zones):(.*)\s+(Aliases):\s+(.*)(?:\s+(Attribute-groups):\s+(.*))?/) or ($line =~ m/(Name):\s+(.*)\s+(Zonesets):(.*)\s+(Zones):(.*)/)) {
            if ($1 and $2) {
                $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
            }
            if ($3 and $4) {
                $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
            }
            if ($5 and $6) {
                $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
            }
            if ($7 and $8) {
                $result_array[$i]->{$key}->[0]->{to_key_name(item=>$7)} = $8;
            }
        }
    }

    return \@result_array;
}

## @name ::show_zone_internal
##
## @summary Parses the raw output from the "show zone internal" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_zone_internal with any of its options as described in the
##    corresponding cdef file 'showzone.cdefs'
## @synopsis
##      show_zone_internal(%args);
## INPUTS TO METHOD CALL 'show_zone_internal' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'ifindex-table' </td> <td> 'ifindex_table' </td> </tr>
## <tr> <td> 'rcs-info' </td> <td> 'rcs_info' </td> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr>
## <tr> <td> 'global-info' </td> <td> 'global_info' </td> </tr>
## <tr> <td> 'ddas-table' </td> <td> 'ddas_table' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zone internal
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VSAN: 1 default-zone: deny(rw) distribute: full
##&nbsp;    E_D_TOV: 2000 R_A_TOV: 10000 D_S_TOV: 5000 F_S_TOV: 5000 F_D_TOV: 2000
##&nbsp;    Interop: default IOD: disable bcast: unsupported dflt-bcast: unsupported dflt-qos: 0
##&nbsp;    DBLock:-(F count:0) Ifindex Table Size: 21 Transit Frame Index: 0
##&nbsp;    Total Transit Frame Count: 1833 Transit Discard Count: 0
##&nbsp;Full Database Counters :
##&nbsp;    Zonesets: 3 Zones: 38 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Aliases: 0 Attribute-groups: 1
##&nbsp;    Members: 40 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Session Database Counters (diff) :
##&nbsp;    Zonesets: 0 Zones: 0
##&nbsp;    Aliases: 0 Attribute-groups: 0
##&nbsp;    Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;Active Database Counters :
##&nbsp;    Zonesets: 1 Zones: 10 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Members: 18 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Session Active Database Counters (diff) :
##&nbsp;    Zones: 0 Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;License Info: 0x0
##&nbsp;Full Zoning Database :
##&nbsp;    DB size: 4076 bytes
##&nbsp;    Zonesets:3  Zones:38 Aliases: 0 Attribute-groups: 1
##&nbsp;Active Zoning Database :
##&nbsp;    DB size: 640 bytes
##&nbsp;    Name: VSeries_Zoneset  Zonesets:1  Zones:10
##&nbsp;TCAM Info :
##&nbsp;    cur_seq_num : 2322,  state : 0
##&nbsp;    gen state : 0
##&nbsp;Change protocol info :
##&nbsp;    local domain id = 27,   ACA by 0xff
##&nbsp;    State =       Idle,  reply_cnt = 0, req_sent_cnt = 0, req_pending = 0
##&nbsp;    Remote domains :
##&nbsp;    Status :
##&nbsp;Merge proto info :
##&nbsp;
##&nbsp;VSAN: 3 default-zone: deny(rw) distribute: active only
##&nbsp;    E_D_TOV: 2000 R_A_TOV: 10000 D_S_TOV: 5000 F_S_TOV: 5000 F_D_TOV: 2000
##&nbsp;    Interop: default IOD: disable bcast: unsupported dflt-bcast: unsupported dflt-qos: 0
##&nbsp;    DBLock:-(F count:0) Ifindex Table Size: 0 Transit Frame Index: 0
##&nbsp;    Total Transit Frame Count: 1833 Transit Discard Count: 0
##&nbsp;Full Database Counters :
##&nbsp;    Zonesets: 2 Zones: 4 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Aliases: 6 Attribute-groups: 0
##&nbsp;    Members: 6 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Session Database Counters (diff) :
##&nbsp;    Zonesets: 0 Zones: 0
##&nbsp;    Aliases: 0 Attribute-groups: 0
##&nbsp;    Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;Active Database Counters :
##&nbsp;    Zonesets: 0 Zones: 0 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Session Active Database Counters (diff) :
##&nbsp;    Zones: 0 Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;License Info: 0x0
##&nbsp;Full Zoning Database :
##&nbsp;    DB size: 552 bytes
##&nbsp;    Zonesets:2  Zones:4 Aliases: 6
##&nbsp;Active Zoning Database :
##&nbsp;    Database Not Available
##&nbsp;TCAM Info :
##&nbsp;    cur_seq_num : 2322,  state : 0
##&nbsp;    gen state : 0
##&nbsp;Change protocol info :
##&nbsp;    local domain id = 61,   ACA by 0xff
##&nbsp;    State =       Idle,  reply_cnt = 0, req_sent_cnt = 0, req_pending = 0
##&nbsp;    Remote domains :
##&nbsp;    Status :
##&nbsp;Merge proto info :
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'full_zoning_database' => [
##&nbsp;                                        {
##&nbsp;                                          'zonesets' => '3 ',
##&nbsp;                                          'aliases' => '0',
##&nbsp;                                          'attribute-groups' => '1',
##&nbsp;                                          'zones' => '38',
##&nbsp;                                          'db_size' => '4076 bytes'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'active_database_counters' => [
##&nbsp;                                            {
##&nbsp;                                              'huge_id_zones' => '0',
##&nbsp;                                              'lun_members' => '0',
##&nbsp;                                              'broadcast_zones' => '0',
##&nbsp;                                              'zones' => '10',
##&nbsp;                                              'qos_zones' => '0',
##&nbsp;                                              'members' => '18',
##&nbsp;                                              'zonesets' => '1',
##&nbsp;                                              'ddas_members' => '0',
##&nbsp;                                              'ipv6_members' => '0',
##&nbsp;                                              'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                              'read-only_zones' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'merge_proto_info' => [],
##&nbsp;            'license_info' => [
##&nbsp;                                {
##&nbsp;                                  'license_info' => '0x0'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'full_database_counters' => [
##&nbsp;                                          {
##&nbsp;                                            'huge_id_zones' => '0',
##&nbsp;                                            'lun_members' => '0',
##&nbsp;                                            'broadcast_zones' => '0',
##&nbsp;                                            'zones' => '38',
##&nbsp;                                            'members' => '40',
##&nbsp;                                            'qos_zones' => '0',
##&nbsp;                                            'zonesets' => '3',
##&nbsp;                                            'ddas_members' => '0',
##&nbsp;                                            'aliases' => '0',
##&nbsp;                                            'ipv6_members' => '0',
##&nbsp;                                            'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                            'read-only_zones' => '0',
##&nbsp;                                            'attribute-groups' => '1'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'session_active_database_counters_diff' => [
##&nbsp;                                                         {
##&nbsp;                                                           'members' => '0',
##&nbsp;                                                           'ddas_members' => '0',
##&nbsp;                                                           'lun_members' => '0',
##&nbsp;                                                           'zones' => '0'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'tcam_info' => [
##&nbsp;                             {
##&nbsp;                               'gen_state' => '0 ',
##&nbsp;                               'cur_seq_num' => '2322',
##&nbsp;                               'state' => '0'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'change_protocol_info' => [
##&nbsp;                                        {
##&nbsp;                                          'status' => undef,
##&nbsp;                                          'remote_domains' => undef,
##&nbsp;                                          'reply_cnt' => '0',
##&nbsp;                                          'state' => 'Idle',
##&nbsp;                                          'req_sent_cnt' => '0',
##&nbsp;                                          'req_pending' => '0',
##&nbsp;                                          'aca' => '0xff',
##&nbsp;                                          'local_domain_id' => '27'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'active_zoning_database' => [
##&nbsp;                                          {
##&nbsp;                                            'zonesets' => '1 ',
##&nbsp;                                            'name' => 'VSeries_Zoneset ',
##&nbsp;                                            'zones' => '10',
##&nbsp;                                            'db_size' => '640 bytes'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'vsan_info' => [
##&nbsp;                             {
##&nbsp;                               'transit_discard_count' => '0',
##&nbsp;                               'transit_frame_index' => '0',
##&nbsp;                               'dblock' => [
##&nbsp;                                             {
##&nbsp;                                               'f_count' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                               'interop' => 'default',
##&nbsp;                               'd_s_tov' => '5000',
##&nbsp;                               'dflt-qos' => '0',
##&nbsp;                               'iod' => 'disable',
##&nbsp;                               'ifindex_table_size' => '21',
##&nbsp;                               'f_d_tov' => '2000',
##&nbsp;                               'e_d_tov' => '2000',
##&nbsp;                               'bcast' => 'unsupported',
##&nbsp;                               'total_transit_frame_count' => '1833',
##&nbsp;                               'dflt-bcast' => 'unsupported',
##&nbsp;                               'r_a_tov' => '10000',
##&nbsp;                               'default-zone' => 'deny(rw)',
##&nbsp;                               'f_s_tov' => '5000',
##&nbsp;                               'distribute' => 'full',
##&nbsp;                               'vsan' => '1'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'session_database_counters_diff' => [
##&nbsp;                                                  {
##&nbsp;                                                    'members' => '0',
##&nbsp;                                                    'zonesets' => '0',
##&nbsp;                                                    'ddas_members' => '0',
##&nbsp;                                                    'lun_members' => '0',
##&nbsp;                                                    'aliases' => '0',
##&nbsp;                                                    'attribute-groups' => '0',
##&nbsp;                                                    'zones' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'full_zoning_database' => [
##&nbsp;                                        {
##&nbsp;                                          'db_size' => '552 bytes'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'active_database_counters' => [
##&nbsp;                                            {
##&nbsp;                                              'huge_id_zones' => '0',
##&nbsp;                                              'lun_members' => '0',
##&nbsp;                                              'broadcast_zones' => '0',
##&nbsp;                                              'zones' => '0',
##&nbsp;                                              'qos_zones' => '0',
##&nbsp;                                              'members' => '0',
##&nbsp;                                              'zonesets' => '0',
##&nbsp;                                              'ddas_members' => '0',
##&nbsp;                                              'ipv6_members' => '0',
##&nbsp;                                              'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                              'read-only_zones' => '0'
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'merge_proto_info' => [],
##&nbsp;            'license_info' => [
##&nbsp;                                {
##&nbsp;                                  'license_info' => '0x0'
##&nbsp;                                }
##&nbsp;                              ],
##&nbsp;            'full_database_counters' => [
##&nbsp;                                          {
##&nbsp;                                            'huge_id_zones' => '0',
##&nbsp;                                            'lun_members' => '0',
##&nbsp;                                            'broadcast_zones' => '0',
##&nbsp;                                            'zones' => '4',
##&nbsp;                                            'members' => '6',
##&nbsp;                                            'qos_zones' => '0',
##&nbsp;                                            'zonesets' => '2',
##&nbsp;                                            'ddas_members' => '0',
##&nbsp;                                            'aliases' => '6',
##&nbsp;                                            'ipv6_members' => '0',
##&nbsp;                                            'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                            'read-only_zones' => '0',
##&nbsp;                                            'attribute-groups' => '0'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;            'session_active_database_counters_diff' => [
##&nbsp;                                                         {
##&nbsp;                                                           'members' => '0',
##&nbsp;                                                           'ddas_members' => '0',
##&nbsp;                                                           'lun_members' => '0',
##&nbsp;                                                           'zones' => '0'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'tcam_info' => [
##&nbsp;                             {
##&nbsp;                               'gen_state' => '0 ',
##&nbsp;                               'cur_seq_num' => '2322',
##&nbsp;                               'state' => '0'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'change_protocol_info' => [
##&nbsp;                                        {
##&nbsp;                                          'status' => undef,
##&nbsp;                                          'remote_domains' => undef,
##&nbsp;                                          'reply_cnt' => '0',
##&nbsp;                                          'state' => 'Idle',
##&nbsp;                                          'req_sent_cnt' => '0',
##&nbsp;                                          'req_pending' => '0',
##&nbsp;                                          'aca' => '0xff',
##&nbsp;                                          'local_domain_id' => '61'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'active_zoning_database' => [],
##&nbsp;            'vsan_info' => [
##&nbsp;                             {
##&nbsp;                               'transit_discard_count' => '0',
##&nbsp;                               'transit_frame_index' => '0',
##&nbsp;                               'dblock' => [
##&nbsp;                                             {
##&nbsp;                                               'f_count' => '0'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                               'interop' => 'default',
##&nbsp;                               'd_s_tov' => '5000',
##&nbsp;                               'dflt-qos' => '0',
##&nbsp;                               'iod' => 'disable',
##&nbsp;                               'ifindex_table_size' => '0',
##&nbsp;                               'f_d_tov' => '2000',
##&nbsp;                               'e_d_tov' => '2000',
##&nbsp;                               'bcast' => 'unsupported',
##&nbsp;                               'total_transit_frame_count' => '1833',
##&nbsp;                               'dflt-bcast' => 'unsupported',
##&nbsp;                               'r_a_tov' => '10000',
##&nbsp;                               'default-zone' => 'deny(rw)',
##&nbsp;                               'f_s_tov' => '5000',
##&nbsp;                               'distribute' => 'active only',
##&nbsp;                               'vsan' => '3'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;            'session_database_counters_diff' => [
##&nbsp;                                                  {
##&nbsp;                                                    'members' => '0',
##&nbsp;                                                    'zonesets' => '0',
##&nbsp;                                                    'ddas_members' => '0',
##&nbsp;                                                    'lun_members' => '0',
##&nbsp;                                                    'aliases' => '0',
##&nbsp;                                                    'attribute-groups' => '0',
##&nbsp;                                                    'zones' => '0'
##&nbsp;                                                  }
##&nbsp;                                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
##&nbsp;Command = show zone internal ddas-table
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Device Alias Database State: 1
##&nbsp;Device Alias Mode: basic
##&nbsp;------------------------------------------------------------------------------
##&nbsp;Device Alias Name                                       PWWN
##&nbsp;------------------------------------------------------------------------------
##&nbsp;dev2                                                    50:0a:09:80:00:02:52:1f
##&nbsp;tech7                                                   50:0a:09:80:00:02:51:1f
##&nbsp;
##&nbsp;Total number of entries = 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_alias_mode' => 'basic',
##&nbsp;            'device_alias_database_state' => '1',
##&nbsp;            'total_number_of_entries' => '2',
##&nbsp;            'device_aliases' => [
##&nbsp;                                  {
##&nbsp;                                    'name' => 'dev2',
##&nbsp;                                    'pwwn' => '50:0a:09:80:00:02:52:1f'
##&nbsp;                                  },
##&nbsp;                                  {
##&nbsp;                                    'name' => 'tech7',
##&nbsp;                                    'pwwn' => '50:0a:09:80:00:02:51:1f'
##&nbsp;                                  }
##&nbsp;                                ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show zone internal global-info
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Global Full Database Counters :
##&nbsp;    Zonesets: 5 Zones: 42 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Aliases: 6 Attribute-groups: 1
##&nbsp;    Members: 46 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Global Session Database Counters (diff) :
##&nbsp;    Zonesets: 0 Zones: 0
##&nbsp;    Aliases: 0 Attribute-groups: 0
##&nbsp;    Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;Global Active Database Counters :
##&nbsp;    Zonesets: 1 Zones: 10 Huge id zones: 0
##&nbsp;    Read-only Zones: 0 QoS Zones: 0 Broadcast Zones: 0
##&nbsp;    Members: 18 LUN Members: 0 DDAS Members: 0
##&nbsp;    Adv Zoning3 Members(IPv4 + dom-If): 0 IPv6 Members: 0
##&nbsp;Global Session Active Database Counters (diff) :
##&nbsp;    Zones: 0 Members: 0 LUN Members: 0 DDAS Members: 0
##&nbsp;Global ISSU Info:
##&nbsp;    fs_upgrade = 0 system_upg = 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'global_session_database_counters_diff' => [
##&nbsp;                                                         {
##&nbsp;                                                           'members' => '0',
##&nbsp;                                                           'zonesets' => '0',
##&nbsp;                                                           'ddas_members' => '0',
##&nbsp;                                                           'lun_members' => '0',
##&nbsp;                                                           'aliases' => '0',
##&nbsp;                                                           'attribute-groups' => '0',
##&nbsp;                                                           'zones' => '0'
##&nbsp;                                                         }
##&nbsp;                                                       ],
##&nbsp;            'global_full_database_counters' => [
##&nbsp;                                                 {
##&nbsp;                                                   'huge_id_zones' => '0',
##&nbsp;                                                   'lun_members' => '0',
##&nbsp;                                                   'broadcast_zones' => '0',
##&nbsp;                                                   'zones' => '42',
##&nbsp;                                                   'members' => '46',
##&nbsp;                                                   'qos_zones' => '0',
##&nbsp;                                                   'zonesets' => '5',
##&nbsp;                                                   'ddas_members' => '0',
##&nbsp;                                                   'aliases' => '6',
##&nbsp;                                                   'ipv6_members' => '0',
##&nbsp;                                                   'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                                   'read-only_zones' => '0',
##&nbsp;                                                   'attribute-groups' => '1'
##&nbsp;                                                 }
##&nbsp;                                               ],
##&nbsp;            'global_issu_info' => [
##&nbsp;                                    {
##&nbsp;                                      'fs_upgrade' => '0',
##&nbsp;                                      'system_upg' => '0'
##&nbsp;                                    }
##&nbsp;                                  ],
##&nbsp;            'global_active_database_counters' => [
##&nbsp;                                                   {
##&nbsp;                                                     'huge_id_zones' => '0',
##&nbsp;                                                     'lun_members' => '0',
##&nbsp;                                                     'broadcast_zones' => '0',
##&nbsp;                                                     'zones' => '10',
##&nbsp;                                                     'qos_zones' => '0',
##&nbsp;                                                     'members' => '18',
##&nbsp;                                                     'zonesets' => '1',
##&nbsp;                                                     'ddas_members' => '0',
##&nbsp;                                                     'ipv6_members' => '0',
##&nbsp;                                                     'adv_zoning3_membersipv4_+_dom-if' => '0',
##&nbsp;                                                     'read-only_zones' => '0'
##&nbsp;                                                   }
##&nbsp;                                                 ],
##&nbsp;            'global_session_active_database_counters_diff' => [
##&nbsp;                                                                {
##&nbsp;                                                                  'members' => '0',
##&nbsp;                                                                  'ddas_members' => '0',
##&nbsp;                                                                  'lun_members' => '0',
##&nbsp;                                                                  'zones' => '0'
##&nbsp;                                                                }
##&nbsp;                                                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show zone internal ifindex-table
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;------------------------
##&nbsp;VSAN:1     FCID:0x1b3600
##&nbsp;------------------------
##&nbsp;ifindex                     :fc1/19
##&nbsp;port-wwn                    :50:06:01:6f:3c:e0:14:de
##&nbsp;symbolic-node-name          :
##&nbsp;port-type                   :N
##&nbsp;port-ip-addr                :::
##&nbsp;fabric-port-wwn             :20:13:00:0d:ec:3e:c0:80
##&nbsp;timer-alive                 :FALSE
##&nbsp;------------------------
##&nbsp;VSAN:1     FCID:0x1b3500
##&nbsp;------------------------
##&nbsp;ifindex                     :fc1/18
##&nbsp;port-wwn                    :50:06:01:6e:3c:e0:14:de
##&nbsp;symbolic-node-name          :
##&nbsp;port-type                   :N
##&nbsp;port-ip-addr                :::
##&nbsp;fabric-port-wwn             :20:12:00:0d:ec:3e:c0:80
##&nbsp;timer-alive                 :FALSE
##&nbsp;
##&nbsp;Total number of entries = 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_number_of_entries' => '2',
##&nbsp;            'ifindex_info' => [
##&nbsp;                                {
##&nbsp;                                  'port-type' => 'N',
##&nbsp;                                  'ifindex' => 'fc1/19',
##&nbsp;                                  'fabric-port-wwn' => '20:13:00:0d:ec:3e:c0:80',
##&nbsp;                                  'timer-alive' => 'FALSE',
##&nbsp;                                  'port-wwn' => '50:06:01:6f:3c:e0:14:de',
##&nbsp;                                  'port-ip-addr' => '::',
##&nbsp;                                  'symbolic-node-name' => '',
##&nbsp;                                  'vsan' => '1',
##&nbsp;                                  'fcid' => '0x1b3600'
##&nbsp;                                },
##&nbsp;                                {
##&nbsp;                                  'port-type' => 'N',
##&nbsp;                                  'ifindex' => 'fc1/18',
##&nbsp;                                  'fabric-port-wwn' => '20:12:00:0d:ec:3e:c0:80',
##&nbsp;                                  'timer-alive' => 'FALSE',
##&nbsp;                                  'port-wwn' => '50:06:01:6e:3c:e0:14:de',
##&nbsp;                                  'port-ip-addr' => '::',
##&nbsp;                                  'symbolic-node-name' => '',
##&nbsp;                                  'vsan' => '1',
##&nbsp;                                  'fcid' => '0x1b3500'
##&nbsp;                                },
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show zone internal rcs-info
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;VSAN 1: RCS Support: Local switch: Disabled
##&nbsp;                     Fabric: Disabled
##&nbsp;
##&nbsp;
##&nbsp;VSAN 3: RCS Support: Local switch: Disabled
##&nbsp;                     Fabric: Disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan_info' => [
##&nbsp;                             {
##&nbsp;                               'rcs_support' => 'Local',
##&nbsp;                               'switch' => 'Disabled',
##&nbsp;                               'fabric' => 'Disabled',
##&nbsp;                               'vsan' => '1'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan_info' => [
##&nbsp;                             {
##&nbsp;                               'rcs_support' => 'Local',
##&nbsp;                               'switch' => 'Disabled',
##&nbsp;                               'fabric' => 'Disabled',
##&nbsp;                               'vsan' => '3'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_zone_internal {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zone_internal(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_internal

sub _show_zone_internal {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );

    # Extracting the command option.
    my $option = $command_line;
    $option =~ s/show\zone\sinternal\s?//;
    if ( $option !~ /-\w/ ) {
        $option = "";
    }

    my $flag = 0;
    if ($option =~ m/ddas-table/) {
        $flag = 1;
    } elsif ($option =~ m/rcs-info/) {
        $flag = 2;
    } elsif ($option =~ m/global-info/) {
        $flag = 3;
    } elsif ($option =~ m/ifindex-table/) {
        $flag = 4;
    }

    my $i = -1;
    my $i1 = -1;
    my ($key, $value, $flag1);

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        next if ($line =~ m/Database Not Available/);
        if ($flag == 0) {
            if ($line =~ m/VSAN:.*/) {
                $i++;
                $flag1 = 0;
            } elsif ($line =~ m/(Full Database Counters)/) {
                $flag1 = 1;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Session Database Counters \(diff\))/) {
                $flag1 = 2;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Active Database Counters)\s+:/) {
                $flag1 = 3;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Session Active Database Counters \(diff\))/) {
                $flag1 = 4;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(License Info):\s+(\w+)/) {
                $key = to_key_name(item=>$1);
                $result_array[$i]->{to_key_name(item=>$1)}->[0]->{to_key_name(item=>$1)} = $2;
                next;
            } elsif ($line =~ m/(Full Zoning Database)/) {
                $flag1 = 5;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Active Zoning Database)/) {
                $flag1 = 6;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(TCAM Info)/) {
                $flag1 = 7;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Change protocol info)/) {
                $flag1 = 8;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            } elsif ($line =~ m/(Merge proto info)/) {
                $flag1 = 9;
                $key = to_key_name(item=>$1);
                $result_array[$i]->{$key} = [];
                next;
            }

            if ($flag1 == 0) {
                if (($line =~ m/(VSAN):\s+(.*)\s+(default-zone):\s+(.*)\s+(distribute):\s+(.*)/) or ($line =~ m/(E_D_TOV):\s+(.*)\s+(R_A_TOV):\s+(.*)\s+(D_S_TOV):\s+(.*)\s+(F_S_TOV):\s+(.*)\s+(F_D_TOV):\s+(.*)/) or ($line =~ m/(Total Transit Frame Count):\s+(.*)\s+(Transit Discard Count):\s+(.*)/) ) {
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$3)} = $4;
                    if ($5 and $6) {
                        $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$5)} = $6;
                    }
                    if ($7 and $8) {
                        $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$7)} = $8;
                    }
                    if ($9 and $10) {
                        $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$9)} = $10;
                    }
                } elsif ($line =~ m/(DBLock):-\((F count):(\w+)\) (Ifindex Table Size): (\w+) (Transit Frame Index): (\w+)/) {
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)}->[0]->{to_key_name(item=>$2)} = $3;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$4)} = $5;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$6)} = $7;
                } elsif ($line =~ m/(Interop):\s+(.*)\s+(IOD):\s+(.*)\s+(bcast):\s+(.*)\s+(dflt-bcast):\s+(.*)\s+dfl/) {
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$5)} = $6;
                    $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$7)} = $8;
                } elsif ($line =~ m/t-qos:\s+(.*)/) {
                    $result_array[$i]->{'vsan_info'}->[0]->{'dflt-qos'} = $1;
                }
            } elsif ($flag1 == 1) {
                if (($line =~ m/(Zonesets):\s+(\w+)\s+(Zones):\s+(\w+)\s+(Huge id zones):\s+(\w+)/) or ($line =~ m/(Read-only Zones):\s+(\w+)\s+(QoS Zones):\s+(\w+)\s+(Broadcast Zones):\s+(\w+)/) or ($line =~ m/(Members):\s+(\w+)\s+(LUN Members):\s+(\w+)\s+(DDAS Members):\s+(\w+)/) )  {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                } elsif (($line =~ m/(Aliases):\s+(\w+)\s+(Attribute-groups):\s+(\w+)/) or ($line =~ m/(Adv Zoning3 Members\(IPv4 [+] dom-If\)):\s+(\w+)\s+(IPv6 Members):\s+(\w+)/)) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                }
            } elsif ($flag1 == 2) {
                if (($line =~ m/(Zonesets):\s+(.*)\s+(Zones):\s+(.*)/) or ($line =~ m/(Aliases):\s+(.*)\s+(Attribute-groups):\s+(.*)/) ) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                } elsif ($line =~ m/(Members):\s+(.*)\s+(LUN Members):\s+(.*)\s+(DDAS Members):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                }
            } elsif ($flag1 == 3) {
                if ($line =~ m/(Adv Zoning3 Members\(IPv4 [+] dom-If\)):\s+(.*)\s+(IPv6 Members):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                } elsif (($line =~ m/(Members):\s+(.*)\s+(LUN Members):\s+(.*)\s+(DDAS Members):\s+(.*)/) or ($line =~ m/(Zonesets):\s+(\w+)\s+(Zones):\s+(\w+)\s+(Huge id zones):\s+(\w+)/) or ($line =~ m/(Read-only Zones):\s+(\w+)\s+(QoS Zones):\s+(\w+)\s+(Broadcast Zones):\s+(\w+)/) ) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                }
            } elsif ($flag1 == 4) {
                if ($line =~ m/(Zones):\s+(.*)\s+(Members):\s+(.*)\s+(LUN Members):\s+(.*)\s+(DDAS Members):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$7)} = $8;
                }
            } elsif ($flag1 == 5) {
                if ($line =~ m/(DB size):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                } elsif ($line =~ m/(Zonesets):(.*)\s+(Zones):(.*)\s+(Aliases):\s+(.*)\s+(Attribute-groups):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$7)} = $8;
                }
            } elsif ($flag1 == 6) {
                if ($line =~ m/(DB size):\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                } elsif ($line =~ m/(Name):\s+(.*)\s+(Zonesets):(.*)\s+(Zones):(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                }
            } elsif ($flag1 == 7) {
                if ($line =~ m/(cur_seq_num)\s+:\s+(.*),\s+(state)\s+:\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                } elsif ($line =~ m/(gen state)\s+:\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                }
            } elsif ($flag1 == 8) {
                if ($line =~ m/(local domain id)\s+=\s+(.*),\s+(ACA)\s+by\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                } elsif ($line =~ m/(State)\s+=\s+(.*),\s+(reply_cnt)\s+=\s+(.*),\s+(req_sent_cnt)\s+=\s+(.*),\s+(req_pending)\s+=\s+(.*)/) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$7)} = $8;
                } elsif (($line =~ m/(Remote domains)/) or ($line =~ m/(Status)/)) {
                    $result_array[$i]->{$key}->[0]->{to_key_name(item=>$1)} = undef;
                }
            } elsif ($flag1 == 9) {
            }
        } elsif ($flag == 1) {
            next if ($line =~ m/^\n$/);
            next if ($line =~ m/-------------------------------+/);
            if ($line =~ m/(Device Alias Database State):\s+(.*)/) {
                $result_array[0]->{to_key_name(item=>$1)} = $2;
            } elsif ($line =~ m/(Device Alias Mode):\s+(.*)/) {
                $result_array[0]->{to_key_name(item=>$1)} = $2;
            } elsif ($line =~ m/(Total number of entries)\s+=\s+(.*)/) {
                $result_array[0]->{to_key_name(item=>$1)} = $2;
            } elsif ($line =~ m/(Device Alias Name)\s{10,}(PWWN)/) {
                $key = to_key_name(item=>$1);
                $value = to_key_name(item=>$2);
            } elsif ($line =~ m/(\w+)\s+(.*)/) {
                 $i1++;
                $result_array[0]->{'device_aliases'}->[$i1]->{'name'} = $1;
                $result_array[0]->{'device_aliases'}->[$i1]->{$value} = $2;
            }
        } elsif ($flag == 2) {
            if ($line =~ m/VSAN.*/) {
                $i++;
            }
            if ($line =~ m/(VSAN)\s+(.*):\s+(RCS Support):\s+(.*)\s+(switch):\s+(.*)/) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)} = $2;
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$3)} = $4;
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$5)} = $6;
            } elsif ($line =~ m/(Fabric):\s+(.*)/) {
                $result_array[$i]->{'vsan_info'}->[0]->{to_key_name(item=>$1)} = $2;
            }
        } elsif ($flag == 3) {
            if ($line =~ m/((?:Global Full Database Counters)|(?:Global Session Database Counters \(diff\))|(?:Global Active Database Counters)|(?:Global Session Active Database Counters \(diff\))|(?:Global ISSU Info))/i) {
                $key = to_key_name(item=>$1);
                $result_array[0]->{$key} = [];
                next;
            }
            if ($line =~ m/(Zones):\s+(.*)\s+(Members):\s+(.*)\s+(LUN Members):\s+(.*)\s+(DDAS Members):\s+(.*)/) {
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$7)} = $8;
            } elsif (($line =~ m/(Zonesets):\s+(.*)\s+(Zones):\s+(.*)\s+(Huge id zones):\s+(.*)/) or ($line =~ m/(Read-only Zones):\s+(.*)\s+(QoS Zones):\s+(.*)\s+(Broadcast Zones):\s+(.*)/) or ($line =~ m/(Members):\s+(.*)\s+(LUN Members):\s+(.*)\s+(DDAS Members):\s+(.*)/)) {
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$5)} = $6;
            } elsif (($line =~ m/(Aliases):\s+(.*)\s+(Attribute-groups):\s+(.*)/) or ($line =~ m/(Adv Zoning3 Members\(IPv4 [+] dom-If\)):\s+(.*)\s+(IPv6 Members):\s+(.*)/) or ($line =~ m/(fs_upgrade)\s+=\s+(.*)\s+(system_upg)\s+=\s+(.*)/) or ($line =~ m/(Zonesets):\s+(.*)\s+(Zones):\s+(.*)/)) {
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$1)} = $2;
                $result_array[0]->{$key}->[0]->{to_key_name(item=>$3)} = $4;
            }
        } elsif ($flag == 4) {
            next if ($line =~ m/^\n$/);
            next if ($line =~ m/------------------------/);
            if ($line =~ m/(VSAN):(\d+)\s+(FCID):(.*)/) {
                $i++;
                $result_array[0]->{'ifindex_info'}->[$i]->{to_key_name(item=>$1)} = $2;
                $result_array[0]->{'ifindex_info'}->[$i]->{to_key_name(item=>$3)} = $4;
            } elsif (($line =~ m/(ifindex)\s+:(.*)/) or ($line =~ m/(fabric-port-wwn)\s+:(.*)/) or ($line =~ m/(port-wwn)\s+:(.*)/) or ($line =~ m/(symbolic-node-name)\s+:(.*)/) or ($line =~ m/(port-type)\s+:(.*)/) or ($line =~ m/(port-ip-addr)\s+:(.*)/) or ($line =~ m/(timer-alive)\s+:(.*)/)) {
                $result_array[0]->{'ifindex_info'}->[$i]->{to_key_name(item=>$1)} = $2;
            } elsif ($line =~ m/(Total number of entries)\s+=\s+(\d+)/) {
                $result_array[0]->{to_key_name(item=>$1)} = $2;
            }
        }
    }

    return \@result_array;
}

1;
