#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC showzoneset and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch showzoneset and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::showzoneset
##
##  showzoneset(%args)
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::showzoneset;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if($raw_output =~ /Invalid command/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }elsif($raw_output =~ /Zoneset\s+not\s+present/) {
        $error = "InvalidParamValue";
        $$error_message = "Invalid values are used in the command";
    }elsif($raw_output =~ /(VSAN\s+\d+\s+is\s+not\s+configured)/) {
        $error = "InvalidParamValue";
        $$error_message = "$1";
    }

    return $error;
}

## @name ::show_zoneset
##
## @summary Parses the raw output from the "show zoneset" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from show_zoneset with any of its options as described in the 
##    corresponding cdef file 'showzoneset.cdefs'
## @synopsis
##      show_zoneset(%args);
## INPUTS TO METHOD CALL 'show_zoneset' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan_id' </td> </tr> 
## <tr> <td> 'active' </td> <td> 'active' </td> </tr> 
## <tr> <td> 'name' </td> <td> 'name' </td> </tr> 
## <tr> <td> 'pending' </td> <td> 'pending' </td> </tr> 
## <tr> <td> 'brief' </td> <td> 'brief' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show zoneset vsan 3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;zoneset name test_zoneset vsan 3
##&nbsp;  zone name zone1 vsan 3
##&nbsp;    pwwn 50:0a:09:80:00:02:37:2a
##&nbsp;
##&nbsp;zoneset name test_zone vsan 3
##&nbsp;  zone name zone1 vsan 3
##&nbsp;    pwwn 50:0a:09:80:00:02:37:2a
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan' => '3',
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'members' => [
##&nbsp;                                          {
##&nbsp;                                            'pwwn' => '50:0a:09:80:00:02:37:2a'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                           'vsan' => '3',
##&nbsp;                           'zone_name' => 'zone1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'zone_set_name' => 'test_zoneset'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'vsan' => '3',
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'members' => [
##&nbsp;                                          {
##&nbsp;                                            'pwwn' => '50:0a:09:80:00:02:37:2a'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                           'vsan' => '3',
##&nbsp;                           'zone_name' => 'zone1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'zone_set_name' => 'test_zone'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

##&nbsp;Command = show zoneset name test_zone
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;zoneset name test_zone vsan 3
##&nbsp;  zone name zone1 vsan 3
##&nbsp;    pwwn 50:0a:09:80:00:02:37:2a
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'vsan' => '3',
##&nbsp;            'zones' => [
##&nbsp;                         {
##&nbsp;                           'members' => [
##&nbsp;                                          {
##&nbsp;                                            'pwwn' => '50:0a:09:80:00:02:37:2a'
##&nbsp;                                          }
##&nbsp;                                        ],
##&nbsp;                           'vsan' => '3',
##&nbsp;                           'zone_name' => 'zone1'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'zone_set_name' => 'test_zone'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_zoneset {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _show_zoneset(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub show_zone_set

sub _show_zoneset {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'show zone set', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );

    my $zone_set = -1;
    my ($i,$k1, $k2);
    my (@array1, @array2, $flag, $flag2, $var1, $var2);

    foreach my $line (@arr) {
        next if ($line =~ m/^\n$/);
        if($line =~ /zoneset\s+name\s+(.*)\s+vsan\s+(\d+)/){
            $zone_set++;
            $i = -1;
            $result_array[$zone_set]->{zone_set_name} =$1;
            $result_array[$zone_set]->{vsan} =$2
        }elsif ($line =~ m/.*(zone name)\s+(.*)\s+(vsan)\s+(.*)/) {
            $i++;
            $k1 = -1;
            $flag = 1;
            $result_array[$zone_set]->{zones}->[$i]->{to_key_name(item=>$1)} = $2;
            $result_array[$zone_set]->{zones}->[$i]->{to_key_name(item=>$3)} = $4;
            next;
        } elsif ($line =~ m/.*(fcalias name)\s+(.*)\s+(vsan)\s+(.*)/) {
            $k1++;
            $k2 = -1;
            $flag = 2;
            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>$1)} = $2;
            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>$3)} = $4;
            next;
        } elsif ($line =~ m/((?:device-alias|domain-id|fcid|fwwn|interface|ip-address|pwwn|symbolic-nodename).*)/) {
            $var1 = $1;
            $var2 = undef;
            $flag2 = 0;
            if ($var1 !~ m/^interface/) {
                if ($var1 =~ m/([A-Za-z0-9:]+\s+[A-Za-z0-9:]+)\s+(.*)/) {
                    $var1 = $1;
                    $var2 = $2;
                }
            }
            if ($var2) {
                $var2 =~ s/[\[\]]//g;
            }
            @array2 = split (/\s+/, $var2);
            if ($#array2 >= 1) {
                $flag2 = 1;
            }
            @array1 = split (/\s+/, $var1);
	    if ($flag == 1) {
                $k1++;
                for (my $n=0; $n<=$#array1; $n++) {
                    if ($n % 2 == 0) {
                        $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>$array1[$n])} = $array1[$n+1];
		        if ($flag2 == 1) {
                            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>$array2[0])} = $array2[1];
			    if($#array2 == 2) {
				$result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>'device-alias')} = $array2[2];
			    } 
			} elsif ($flag2 == 0 and $#array2 == 0) {
                            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{to_key_name(item=>'device-alias')} = $array2[0];
                        }
                    } 
                }
            } elsif ($flag == 2) {
                $k2++;
                for (my $n=0; $n<=$#array1; $n++) {
                    if ($n % 2 == 0) {
                        $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>$array1[$n])} = $array1[$n+1];
                        if ($flag2 == 1 and $#array2 == 1) {
                            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>$array2[0])} = $array2[1];
                        } elsif ($flag2 == 0 and $#array2 == 0) {
                            $result_array[$zone_set]->{zones}->[$i]->{members}->[$k1]->{members}->[$k2]->{to_key_name(item=>'device-alias')} = $array2[0];
                        }
                    }
                }
            }
        }
    }

    return \@result_array;
}
1;
