#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC zonename command
##
## @description
##    This is a library to parse the raw output of CiscoFC zonename command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::zonename
##
##  zonename(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::zone;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    if ($raw_output =~ /Invalid\s+command/) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output ;
    } elsif ($raw_output =~ /VSAN\s+(\d+)\s+is\s+not\s+configured|Rename\s+requested\s+for\s+a\s+non\s+existing\s+zone|Target\s+zone\s+name\s+already\s+in\s+use/){
        $error = "InvalidParamValue";
        $$error_message = $raw_output ;
    } elsif ($raw_output =~ /Incomplete\s+command/){
        $error = "MissingRequiredParam";
        $$error_message = $raw_output ;
    }
    return $error;
}

## @name ::zone_name
##
## @summary Parses the raw output from the "zone name" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from zone_name with any of its options as described in the 
##    corresponding cdef file 'zone.cdefs'
## @synopsis
##      zone_name(%args);
## INPUTS TO METHOD CALL 'zone_name' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'zone-name' </td> </tr> 
## <tr> <td> 'vsan' </td> <td> 'vsan-id' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub zone_name {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error = _check_error(raw_output=>$output,error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
    return [] ;
}

## @name ::zone_rename
##
## @summary Parses the raw output from the "zone rename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zone_rename with any of its options as described in the
##    corresponding cdef file 'zone.cdefs'
## @synopsis
##      zone_rename(%args);
## INPUTS TO METHOD CALL 'zone_rename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'currentname' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newname' </td> <td> '0' </td> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = zone rename f6290_109b_0d_ds8000_I3C4T0 f6290_109b_0d_ds8000_I3C4T0_141 vsan 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub zone_rename {
    my %args = @_;
    my ( $error_message, $error );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    # Status returned by switch for this command is
    # 0 even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}

1;

