#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/zoneset.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC zoneset commands
##
## @description
##    This is a library to parse the raw output of CiscoFC zoneset and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::zoneset
##
##  zoneset_name(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::zoneset;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $resp = $args{response_object};
    my $error_message ;
    if($raw_output =~
        /(Invalid\s+command|Zoneset not present|VSAN\s+(\d+)\s+is\s+not\s+active|VSAN\s+(\d+)\s+is\s+not\s+configured|Rename\s+requested\s+for\s+a\s+non\s+existing\s+zoneset|Target\s+zoneset\s+name\s+already\s+in\s+use)/) {
        $error = "InvalidParamValue";
        $error_message = $raw_output ;
    }elsif($raw_output =~ /Incomplete\s+command/){
        $error = "MissingRequiredParam";
        $error_message = $raw_output ;
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
}

## @name ::zoneset_name
##
## @summary Parses the raw output from the "zoneset name" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zoneset_name with any of its options as described in the
##    corresponding cdef file 'zoneset.cdefs'
## @synopsis
##      zoneset_name(%args);
## INPUTS TO METHOD CALL 'zoneset_name' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'zoneset' </td> <td> '0' </td> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = zoneset name zoneset_44 vsan 44
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub zoneset_name {
    my %args = @_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output=>$output, response_object=>$resp);

    return [] ;
}

## @name ::zoneset_activate_name
##
## @summary Parses the raw output from the "zoneset activate name" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zoneset_activate_name with any of its options as described in the
##    corresponding cdef file 'zoneset.cdefs'
## @synopsis
##      zoneset_activate(%args);
## INPUTS TO METHOD CALL 'zoneset_activate_name' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'zoneset' </td> <td> '0' </td> </tr>
## <tr> <td> 'vsan' </td> <td> 'vsan-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = zoneset activate name zoneset_44 vsan 44
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Zoneset activation initiated. check zone status
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub zoneset_activate_name {
    my %args = @_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output=>$output, response_object=>$resp);

    return [] ;
}

## @name ::zoneset_rename
##
## @summary Parses the raw output from the "zoneset rename" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from zoneset_rename with any of its options as described in the 
##    corresponding cdef file 'zoneset.cdefs'
## @synopsis
##      zoneset_rename(%args);
## INPUTS TO METHOD CALL 'zoneset_rename' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'currentname' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'newname' </td> <td> '0' </td> </tr> 
## <tr> <td> 'vsan' </td> <td> 'vsan-id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = zoneset rename vgv6210f118ab_vnx5500a630_01 vgv6210f118ab_vnx5500a630_01_141 vsan 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub zoneset_rename {
    my %args = @_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output=>$output, response_object=>$resp);

    return [] ;
}
1;

