#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for CiscoNetwork Switch CLI commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Switch::CLI::CiscoNetwork;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Switch::CLI);

#
# Module imports - APISet
#
use NACL::APISet::Session::CLI::CiscoNetwork;
use NACL::APISet::_Mixins::CLI qw(_get_absolute_cdef_paths);
use base qw(NACL::APISet::Version::Switch);

#
##Exceptions
##
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

sub new
{
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    my $self = {};
    $self = NACL::APISet::validate_params
      ($class,
       params => \%args,
       spec   =>
             {
              version   => { type => SCALAR, optional => 1, default => 99.99   },
             }
      );
    bless $self, $class;
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
# Method:    clear_privilege
# Objective: To default back to current user
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_privilege {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    
    # For Cisco IOS "enable" is used to set the privilege and
    # "enable 0" is used to clear privilege(0 => lowest)
    $self->enable(level => 0);
    $Log->exit() if $may_exit;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the CiscoNetwork API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API, assigning a default CDEF path
##############################################################################
sub _init_API
{
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;
    
    $self->_get_absolute_cdef_paths();
    
    unless (defined $self->{cdef_path}) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
            "Error in finding CDEF directories " . $self->{cdef_path} . 
            " and the reason: \n$@");
    }

    $self->SUPER::_init_API(%args);

    $Log->exit() if $may_exit;
}

###############################################################################
# Method Name: _invoke
# Objective:   translate cdef schema into actual command and execute
# Details:     This method invokes the command along with its options and returns
#              its output. It does this by getting the CDEF schema, translating
#              this CDEF schema into the actual command to be invoked on the
#              service, and then invoking that on the filer using the Session
#              object.
###############################################################################
sub _invoke {
    my ($self) = shift();
    $Log->enter() if $may_enter;

    $self->api_reconnect_session() if
                   $self->get_connection()->is_disconnected(verify => 0);

    my (%connectrec, $actual_command);

    my %args =
      NACL::APISet->validate_params(
                       params => \@_,
                       spec   => {
                           "privilege-level" => {type => SCALAR, optional => 1},
                               });

    my $privilege       = delete $args{'privilege-level'};

    %args = @_;

    my $method = $args{method};
    my $deconfigure = 0;

    # process parameters
    while (my ($key, $value) = each(%args)) {
        
        if ($key =~ /-?method$/) {
            if (!length($value)) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                                       "Invalid value for \"$key\": " . $value);
            }
        } elsif ($key =~ /-?debug_level$/) {
            if (!length($value) || $value !~ /\d+/) {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                                       "Invalid value for \"$key\": " . $value);
            }

            # delete args that is not needed for translation
            delete $args{$key};
            next;
        } elsif ($key =~ /connectrec\-(.*)/ || $method eq 'execute_command' ) {
            # All options prefixed with a connectrec are meant as options for
            # Connectrec and should not be translated by the CDEFs. These
            # are moved into a separate hash and delete from the args hash
            # so that they are not sent for translation
            $connectrec{$key} = $value;
            delete $args{$key};
        } elsif ($key =~ /deconfigure/) {
            $deconfigure = $value;
            delete $args{$key};
        }
            
    } ## end while (my ($key, $value) ...
	
    # verify mandatory cdef argument
    unless (defined $args{method} and length $args{method}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                                  "Mandatory argument \"cdef\" not specified ");
    }

    my ($command, $file) = ($connectrec{command}, undef);
    
    if ($method ne 'execute_command') {
        
        # Find parser file
        $file = $self->_find_parser_file(method => $method);
        
        # retrieve schema handle
        my $schema_obj = $self->get_schema_obj();

        # translate cdef schema into command
        $command = $schema_obj->translate(%args);

    }

    $connectrec{"connectrec-on_timeout"} = $self->_throw_timeout_exception(
                                before => $connectrec{"connectrec-on_timeout"}  || delete $connectrec{'on_timeout'}, timeout => $connectrec{"connectrec-timeout"} );
    $connectrec{"connectrec-on_idle"} = $self->_throw_timeout_exception(
                                   before => $connectrec{"connectrec-on_idle"}  || delete $connectrec{'on_idle'}, timeout => $connectrec{"connectrec-timeout"} );
	my %connectrec_ptr;
	$self->_move_connectrec_options(
        source => \%connectrec,
        target => \%connectrec_ptr
    );
    my $os_revision = $self->{version_manager}->{os_revision};

    # commands for IOS on Network Cisco switch needs to be handled differently
    if ($self->{set} =~ /CiscoNetwork/ and $os_revision =~ /IOS/i and defined($privilege)) {
        $self->set_privilege('privilege-level' => "enable");
    }
    # To handle IOS command show mac address-table for version 12.2(55)
    if ($command =~ /show mac.address-table/){
        my $major = $self->{version_manager}->{os_revision}->{major};
        my $version = $self->{version_manager}->{os_version}->{version};
        if ($major =~ /IOS/i and $version =~ /12.2\(55\)SE/) {
            $command =~ s/show mac-address-table/show mac address-table/;
        } elsif ($major =~ /Nexus/i and $version =~ /^4.*/) {
            $command =~ s/show mac address-table/show mac-address-table/;
        }
    }

    if ($deconfigure == 1) {
        $command = "no " . $command;
    }

    my $result;

    if ($method ne 'execute_command' ) {
    # execute command
        $result =
      $self->{session}->send_command(
                 command        => $command,
                 method_name    => $method,
                 cdef_path      => $self->{cdef_path},
                 connectrec		=> \%connectrec_ptr,
                 parser_file => $file);
    } else {
        delete $connectrec{method};
          $result = 
      $self->{session}->send_command(
                 connectrec		=> \%connectrec_ptr,
                 command 		=> $command
            );
    }
    
    if ($self->{set} =~ /CiscoNetwork/ and $os_revision =~ /IOS/i and defined($privilege)) {
        $self->clear_privilege();
    }

    $Log->exit() if $may_exit;
    return $result;
}

=head1 NAME

NACL::APISet::Switch::CLI::CiscoNetwork - API Set for CiscoNetwork CLI commands

=head1 SYNOPSIS

use NACL::APISet::Switch::CLI::CiscoNetwork;

=head1 DESCRIPTION

A C<NACL::APISet::Switch::CLI::CiscoNetwork> is used to connect to and send CLI commands
to the CiscoNetwork Switch.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Switch::CLI::CiscoNetwork>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Switch::CLI::CiscoNetwork->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Switch::CLI for details.

=back

=item Returns

A reference to a new API Set.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
