#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork configure terminal
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch configure terminal
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::IOS::configure_terminal
##  
##  configure_mac_address_table(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::IOS::configure_terminal;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if ($raw_output =~ /(Invalid\s+input\s+detected)|(not\s+a\s+switching\s+port)/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    } elsif ($raw_output =~ /(Incomplete\s+command)/i) {
        $$error_message = $raw_output;
        $error = "MissingRequiredParam"
    }

    return $error;
}

## @name ::interface
##
## @summary Parses the raw output from the "interface" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from interface with any of its options as described in the 
##    corresponding cdef file 'configure_terminal.cdefs'
## @synopsis
##      interface(%args);
## INPUTS TO METHOD CALL 'interface' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'entity-type' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'entity-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = interface Gi2/2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub interface {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is  
    # is ZERO even if command failed
    if (not defined($error)) {
        $error = _check_error(raw_output    => $output,
                              error_message => \$error_message);
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

## @name ::shutdown
##
## @summary Parses the raw output from the "shutdown" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from shutdown with any of its options as described in the 
##    corresponding cdef file 'configure_terminal.cdefs'
## @synopsis
##      shutdown(%args);
## INPUTS TO METHOD CALL 'shutdown' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'vlan' </td> <td> 'vlan-list' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = shutdown vlan 2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;VLAN 2 already shutdown
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub shutdown {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is  
    # is ZERO even if command failed
    if (not defined($error)) {
        $error = _check_error(raw_output    => $output,
                              error_message => \$error_message);
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

## @name ::vlan
##
## @summary Parses the raw output from the "vlan" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vlan with any of its options as described in the 
##    corresponding cdef file 'configure_terminal.cdefs'
## @name ::vlan
##
## @summary Parses the raw output from the "vlan" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from vlan with any of its options as described in the 
##    corresponding cdef file 'configure_terminal.cdefs'
## @synopsis
##      vlan(%args);
## INPUTS TO METHOD CALL 'vlan' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'entity-type' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'entity-name' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vlan 3
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub vlan {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is  
    # is ZERO even if command failed
    if (not defined($error)) {
        $error = _check_error(raw_output    => $output,
                              error_message => \$error_message);
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

1;
