#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse NetAppNetwork copy command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch copy command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::copy
##
##  copy(%args)
##
## @author nayankumar.patel@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::copy;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(Incomplete\s+command)/i ) {
        $$error_message = $raw_output;
        $error          = "MissingRequiredParam";
    } elsif ( $raw_output
        =~ /An\s+invalid\s+interface\s+has\s+been\s+used\s+for\s+this\s+command/
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /The\s+group\s+does\s+not\s+exist/ ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output =~ /File\stransfer\sfailed/ ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    }

    return $error;
} ## end sub _check_error

## @name ::copy
##
## @summary Parses the raw output from the "copy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from copy with any of its options as described in the
##    corresponding cdef file 'copy.cdefs'
## @synopsis
##      copy(%args);
## INPUTS TO METHOD CALL 'copy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> <td> '0' </td> </tr>
## <tr> <td> 'noval' </td> <td> 'noval' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'sourcefilename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destfilename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = copy active backup
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Copying active to backup
##&nbsp;Copy operation successful
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'from' => 'active',
##&nbsp;            'to' => 'backup'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = copy nvram:startup-config nvram:backup-config
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;        ];
##&nbsp;
sub copy {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @result_array, $line );
    @result_array = ();
    foreach $line (@lines) {
        next if ( !$line || !defined($line) );
        if ( $line =~ /Copying\s(\w+)\sto\s(\w+)/ ) {
            $result_array[0]->{from} = $1;
            $result_array[0]->{to}   = $2;
        } elsif ( $line =~ /^([a-zA-Z\s]+)\.{4,}\s*(.+)$/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }

    return \@result_array;
} ## end sub copy

1;
