#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/CN1601/script.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse NetAppNetwork script and its subcommands
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch script and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::script
##
##  script_list(%args)
##  script_apply(%args)
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::script;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /(Incomplete\s+command)/i ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output
        =~ /File\s+extension\s+other\s+than\s+'.scr'\s+is\s+not\s+allowed/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Configuration\sscript\s\'.*\'\snot\sfound/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif (
        $raw_output =~ /Configuration\sscript\s\'.*\'\svalidation\sfailed/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::script_list
##
## @summary Parses the raw output from the "script list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from script_list with any of its options as described in the
##    corresponding cdef file 'script.cdefs'
## @synopsis
##      script_list(%args);
## INPUTS TO METHOD CALL 'script_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = script list
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Configuration Script Name        Size(Bytes)
##&nbsp;-------------------------------- -----------
##&nbsp;CN1601_MS_RCF_v1.0.scr                  2523
##&nbsp;running-config.scr                      2454
##&nbsp;net-1961-1_config.scr                    908
##&nbsp;
##&nbsp;3 configuration script(s) found.
##&nbsp;2042 Kbytes free.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'configuration_script_name' => 'CN1601_MS_RCF_v1.0.scr',
##&nbsp;            'size_in_bytes' => '2523'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'configuration_script_name' => 'running-config.scr',
##&nbsp;            'size_in_bytes' => '2454'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'configuration_script_name' => 'net-1961-1_config.scr',
##&nbsp;            'size_in_bytes' => '908'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub script_list {
    my %args = @_;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $i           = 0;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    if ($error_level) {
        _check_error( raw_output => $output, );
    }

    my @Result_array;
    my @lines = get_lines(
        raw_output => $output,
        first_line => 'Configuration\s+Script\s+Name'
    );
    foreach my $line (@lines) {
        next if ( $line =~ /(------)|(Configuration\s+Script\s+Name)/ );
        if ( $line =~ /(.*.scr)\s+(.*)/ ) {
            my $key = $1;
            trim( 'string_ptr' => \$key );
            $Result_array[$i]->{configuration_script_name} = $key;
            $Result_array[ $i++ ]->{size_in_bytes} = $2;
        }
    }
    return \@Result_array;

} ## end sub script_list

## @name ::script_apply
##
## @summary Parses the raw output from the "script apply" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from script_apply with any of its options as described in the
##    corresponding cdef file 'script.cdefs'
## @synopsis
##      script_apply(%args);
## INPUTS TO METHOD CALL 'script_apply' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'script-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = script apply running-config.scr
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Config file 'startup-config' created successfully .
##&nbsp;Configuration Saved!
##&nbsp;
##&nbsp;set prompt "MS:net-1961-1"
##&nbsp;
##&nbsp;telnetcon timeout 60
##&nbsp;
##&nbsp;enable password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 encrypted
##&nbsp;
##&nbsp;network parms 172.22.200.91 255.255.254.0 172.22.200.1
##&nbsp;
##&nbsp;vlan database
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;configure
##&nbsp;
##&nbsp;sntp client mode unicast
##&nbsp;
##&nbsp;sntp server "172.19.2.16"
##&nbsp;
##&nbsp;SNTP Server with specified address already exists.
##&nbsp;
##&nbsp;
##&nbsp;ip domain name "lab.netapp.com"
##&nbsp;
##&nbsp;
##&nbsp;ip name server 172.22.201.46
##&nbsp;
##&nbsp;
##&nbsp;username "admin" password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 level 15 encrypted
##&nbsp;
##&nbsp;line console
##&nbsp;
##&nbsp;serial timeout 60
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;line telnet
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;line ssh
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;spanning-tree configuration name "00-A0-98-4B-8A-40"
##&nbsp;
##&nbsp;interface 0/15
##&nbsp;
##&nbsp;addport 1/1
##&nbsp;
##&nbsp;0/15 interface is part of a port-channel.
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/16
##&nbsp;
##&nbsp;addport 1/1
##&nbsp;
##&nbsp;0/16 interface is part of a port-channel.
##&nbsp;exit
##&nbsp;
##&nbsp;snmp-server sysname "MS:net-1961-1"
##&nbsp;
##&nbsp;interface 0/1
##&nbsp;
##&nbsp;description 'Cluster Node 1'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/16
##&nbsp;
##&nbsp;description 'ISL member port 16'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 1/1
##&nbsp;
##&nbsp;description 'ISL LAG - ports 15-16'
##&nbsp;
##&nbsp;port-channel load-balance 6
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 1/6
##&nbsp;
##&nbsp;description 'NetApp CN1601 Management Switch RCF v1'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;isdp timer 5
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;Configuration script 'running-config.scr' applied.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'applied',
##&nbsp;            'spanning_tree_configuration' => '00-A0-98-4B-8A-40"',
##&nbsp;            'snmp_server_sysname' => 'MS:net-1961-1',
##&nbsp;            'lines' => [
##&nbsp;                         {
##&nbsp;                           'serial_timeout' => '60',
##&nbsp;                           'line' => 'console'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'line' => 'telnet'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'line' => 'ssh'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'config_file' => '\'startup-config\'',
##&nbsp;            'config_file_status' => 'created successfully',
##&nbsp;            'password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;            'interfaces' => [
##&nbsp;                              {
##&nbsp;                                'interface' => '0/15',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'addport 1/1',
##&nbsp;                                                      '0/15 interface is part of a port-channel.',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/16',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'addport 1/1',
##&nbsp;                                                      '0/16 interface is part of a port-channel.',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/1',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'Cluster Node 1\'',
##&nbsp;                                                      'spanning-tree edgeport',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/16',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'ISL member port 16\'',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '1/1',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'ISL LAG - ports 15-16\'',
##&nbsp;                                                      'port-channel load-balance 6',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '1/6',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'NetApp CN1601 Management Switch RCF v1\'',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'sntp_server' => '172.19.2.16',
##&nbsp;            'configuration_script' => 'running-config.scr',
##&nbsp;            'set_prompt' => 'MS:net-1961-1',
##&nbsp;            'enable_password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;            'sntp_client_mode' => 'unicast',
##&nbsp;            'username' => 'admin',
##&nbsp;            'ip_name_server' => '172.22.201.46',
##&nbsp;            'level' => '15',
##&nbsp;            'ip_domain_name' => 'lab.netapp.com',
##&nbsp;            'isdp_timer' => '5',
##&nbsp;            'network_parms' => [
##&nbsp;                                 '172.22.200.91',
##&nbsp;                                 '255.255.254.0',
##&nbsp;                                 '172.22.200.1'
##&nbsp;                               ],
##&nbsp;            'telnetcon_timeout' => '60'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub script_apply {
    my %args = @_;

    my ( @result_array, @packages );
    my $int = -1;
    my $key;
    my $j     = 0;
    my $lines = -1;
    my $con   = 0;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    my $error
        = _check_error( raw_output => $output, response_object => $resp );

    my @lines = get_lines( raw_output => $output, response_object => $resp );
    while ( my $line = shift(@lines) ) {
        next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
        if ( $line =~ /interface\s+(.*)/ && ( $line !~ /part\s+of\s+a/ ) ) {
            $key                                              = "interfaces";
            $result_array[0]->{$key}->[ ++$int ]->{interface} = $1;
            $con                                              = 0;
        } elsif ( $key eq "interfaces" ) {
            $result_array[0]->{interfaces}->[$int]->{'running_config'}
                ->[ $con++ ] = $line;
            $key = undef if ( $line eq "exit" );
        } elsif ( $line =~ /(serial\s+timeout|password)\s+(.*)/
            && $key eq "lines" ) {
            my $key3 = $1;
            my $val3 = $2;
            $key3 =~ s/-/_/g;
            $result_array[0]->{$key}->[$lines]
                ->{ to_key_name( item => "$key3" ) } = $val3;
        } elsif ( $line =~ /line\s+(\w+)/ ) {
            $key = "lines";
            $result_array[0]->{$key}->[ ++$lines ]->{line} = $1;
        } elsif ( $line
            =~ /(System\s+Up\s+Time|System\s+Description|Current\s+SNTP\s+Synchronized\s+Time:)\s+"?(.*)"?/
            ) {
            my $value2 = $2;
            my $key2   = $1;
            $value2 =~ s/"//g;
            $result_array[0]->{ to_key_name( item => "$key2" ) } = $value2;
        } elsif ( $line =~ /(sntp\s+server\s+status)\s+is\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $2;
        } elsif ( $line
            =~ /((spanning-tree\s+configuration|port-channel)\s+name)\s*"?(.*)"?/
            ) {
            my $val4 = $3;
            my $key4 = $2;
            $key4 =~ s/-/_/g;
            $val4 =~ s/"//g;
            $result_array[0]->{ to_key_name( item => "$key4" ) } = $val4;
        } elsif ( $line
            =~ /^!?((ip|snmp-server|System|sntp|serviceport\s+protocol|telnetcon|set\s+prompt|hostname|sshcon)\s*.*)\s+(.*)/
            ) {
            my $value1 = $3;
            my $key1   = $1;
            $key1   =~ s/-/_/;
            $value1 =~ s/"//g;
            $result_array[0]->{ to_key_name( item => "$key1" ) } = $value1;
        } elsif ( $line
            =~ /(Additional\s+Packages|serviceport\s+ip|network parms)\s+(.*)/
            ) {
            my @entries = split( /\s+/, $2 );
            $result_array[0]->{ to_key_name( item => $1 ) } = \@entries;
        } elsif ( $line =~ /(isdp\s+timer|spanning-tree)\s+(.*)/ ) {
            my $key5 = $1;
            my $val5 = $2;
            $key5 =~ s/-/_/g;
            $result_array[0]->{ to_key_name( item => "$key5" ) } = $val5;
        } elsif ( $line
            =~ /(username)\s+"(\w+)"\s+(password)\s+(.*)\s+(level)\s+(.*)\s+encrypted/
            ) {
            $result_array[0]->{"$1"} = $2;
            $result_array[0]->{"$3"} = $4;
            $result_array[0]->{"$5"} = $6;
        } elsif ( $line =~ /((enable)?\s+password)\s+(.*)\s+encrypted/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $3;
        } elsif (
            $line =~ /(Configuration\s+script)\s+'(.*.scr)'\s+(applied)/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) }     = $2;
            $result_array[0]->{ to_key_name( item => "status" ) } = $3;
        } elsif (
            $line =~ /(Config\s+file)\s+(.*)\s+(created\s+successfully)/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $2;
            $result_array[0]->{config_file_status} = $3;
        } elsif ( $line eq "exit" ) {
            $key = undef;
        }
    }

    return \@result_array;
} ## end sub script_apply

## @name ::script_validate
##
## @summary Parses the raw output from the "script validate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from script_validate with any of its options as described in the
##    corresponding cdef file 'script.cdefs'
## @synopsis
##      script_validate(%args);
## INPUTS TO METHOD CALL 'script_validate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'script-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = script validate running-config.scr
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;set prompt "MS:net-1961-1"
##&nbsp;
##&nbsp;telnetcon timeout 60
##&nbsp;
##&nbsp;enable password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 encrypted
##&nbsp;
##&nbsp;network parms 172.22.200.91 255.255.254.0 172.22.200.1
##&nbsp;
##&nbsp;vlan database
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;configure
##&nbsp;
##&nbsp;sntp client mode unicast
##&nbsp;
##&nbsp;sntp server "172.19.2.16"
##&nbsp;
##&nbsp;ip domain name "lab.netapp.com"
##&nbsp;
##&nbsp;
##&nbsp;ip name server 172.22.201.46
##&nbsp;
##&nbsp;
##&nbsp;username "admin" password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 level 15 encrypted
##&nbsp;
##&nbsp;line console
##&nbsp;
##&nbsp;serial timeout 60
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;line telnet
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;line ssh
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;spanning-tree configuration name "00-A0-98-4B-8A-40"
##&nbsp;
##&nbsp;interface 0/15
##&nbsp;
##&nbsp;addport 1/1
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/16
##&nbsp;
##&nbsp;addport 1/1
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;snmp-server sysname "MS:net-1961-1"
##&nbsp;
##&nbsp;interface 0/1
##&nbsp;
##&nbsp;description 'Cluster Node 1'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/2
##&nbsp;
##&nbsp;description 'Cluster Node 2'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/3
##&nbsp;
##&nbsp;description 'Cluster Node 3'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/4
##&nbsp;
##&nbsp;description 'Cluster Node 4'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/5
##&nbsp;
##&nbsp;description 'Cluster Node 5'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/6
##&nbsp;
##&nbsp;description 'Cluster Node 6'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/7
##&nbsp;
##&nbsp;description 'Cluster Node 7'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/8
##&nbsp;
##&nbsp;description 'Cluster Node  8'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/9
##&nbsp;
##&nbsp;description 'cluster Node 9'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/10
##&nbsp;
##&nbsp;description 'Cluster Node 10'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/11
##&nbsp;
##&nbsp;description 'Cluster Node 11'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/12
##&nbsp;
##&nbsp;description 'Cluster Node 12'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/13
##&nbsp;
##&nbsp;description 'Cluster Switch'
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/14
##&nbsp;
##&nbsp;description 'Network uplink'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/15
##&nbsp;
##&nbsp;description 'ISL member port 15'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 0/16
##&nbsp;
##&nbsp;description 'ISL member port 16'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 1/1
##&nbsp;
##&nbsp;description 'ISL LAG - ports 15-16'
##&nbsp;
##&nbsp;port-channel load-balance 6
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;interface 1/6
##&nbsp;
##&nbsp;description 'NetApp CN1601 Management Switch RCF v1'
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;isdp timer 5
##&nbsp;
##&nbsp;exit
##&nbsp;
##&nbsp;Configuration script 'running-config.scr' validated.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;        ];
##&nbsp;
sub script_validate {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
} ## end sub script_validate

1;
