#
#
#Copyright (c)2012-2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse NetAppNetwork show and its subcommands
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch show and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::show
##
##  show_version(%args)
##  show_bootvar(%args)
##  show_port(%args)
##  show_running_config(%args)
##  show_mac_addr_table(%args)
##  show_mac_addr_table_interface(%args)
##  show_interface(%args)
##  show_interface_ethernet(%args)
##  show_storm_control
##  show_process_cpu
##  show_tech_support
##  show_snmpcommunity
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::show;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Invalid\s+input\s+detected/i ) {
        $$error_message = $raw_output;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(Incomplete\s+command)/i ) {
        $$error_message = $raw_output;
        $error          = "MissingRequiredParam";
    } elsif ( $raw_output
        =~ /An\s+invalid\s+interface\s+has\s+been\s+used\s+for\s+this\s+command/
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    } elsif ( $raw_output
        =~ /The\s+group\s+does\s+not\s+exist|The\s+specified\s+interface\s+does\s+not\s+exist./
        ) {
        $$error_message = $raw_output;
        $error          = "InvalidParamValue";
    }

    return $error;
} ## end sub _check_error

## @name ::show_version
##
## @summary Parses the raw output from the "show version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_version with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_version(%args);
## INPUTS TO METHOD CALL 'show_version' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Switch: 1
##&nbsp;
##&nbsp;System Description............................. NetApp CN1601, 1.0.0.2, Linux
##&nbsp;                                                2.6.21.7
##&nbsp;Machine Type................................... NetApp CN1601
##&nbsp;Machine Model.................................. CN1601
##&nbsp;Serial Number.................................. 10610100001
##&nbsp;Burned In MAC Address.......................... 00:A0:98:4B:8A:40
##&nbsp;Software Version............................... 1.0.0.2
##&nbsp;Operating System............................... Linux 2.6.21.7
##&nbsp;Network Processing Device...................... BCM53716_A2
##&nbsp;Part Number.................................... 111-00894
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Additional Packages............................ FASTPATH QOS
##&nbsp;                                                FASTPATH IPv6 Management
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => '10610100001',
##&nbsp;            'operating_system' => 'Linux 2.6.21.7',
##&nbsp;            'network_processing_device' => 'BCM53716_A2',
##&nbsp;            'machine_type' => 'NetApp CN1601',
##&nbsp;            'system_description' => 'NetApp CN1601, 1.0.0.2, Linux 2.6.21.7',
##&nbsp;            'part_number' => '111-00894',
##&nbsp;            'burned_in_mac_address' => '00:A0:98:4B:8A:40',
##&nbsp;            'software_version' => '1.0.0.2',
##&nbsp;            'additional_packages' => [
##&nbsp;                                       'FASTPATH QOS',
##&nbsp;                                       'FASTPATH IPv6 Management'
##&nbsp;                                     ],
##&nbsp;            'machine_model' => 'CN1601'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_version {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    if ( not defined($error) ) {
        $error = _check_error(
            raw_output    => $output,
            command       => "show version",
            error_message => \$error_message
        );
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my @lines
        = get_lines( raw_output => $output, first_line => 'Switch:\s+\d+' );
    shift(@lines) if ( $lines[0] =~ /Switch:\s+\d+/ );
    while ( my $line = shift(@lines) ) {
        next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
        my %hash;
        while ( $line !~ /Switch:\s+\d+/ and ( scalar(@lines) > 0 ) ) {
            if ( $line =~ /^Additional Packages/ ) {
                my @tmp_arr;
                my ( $key, $value ) = ( $line =~ /^(.*\w)\.\.+\s*(.*)/ );
                push( @tmp_arr, $value );
                while ( $lines[0] !~ /^(.*)\.\.+\s*(.*)/
                    and ( scalar(@lines) > 0 ) ) {
                    $line = shift(@lines);
                    next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
                    push( @tmp_arr, $line );
                }
                $hash{ to_key_name( item => $key ) } = \@tmp_arr;
            } else {
                my ( $key, $value ) = ( $line =~ /^(.*\w)\.\.+\s*(.*)/ );
                $hash{ to_key_name( item => $key ) } = $value;
                while ( $lines[0] !~ /^(.*)\.\.+\s*(.*)/
                    and ( scalar(@lines) > 0 ) ) {
                    $line = shift(@lines);
                    next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
                    $hash{ to_key_name( item => $key ) } .= " " . $line;
                }
            }
            $line = shift(@lines);
        }
        push( @result_array, \%hash );
    }

    return \@result_array;

} ## end sub show_version

## @name ::show_bootvar
##
## @summary Parses the raw output from the "show bootvar" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_bootvar with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_bootvar(%args);
## INPUTS TO METHOD CALL 'show_bootvar' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show bootvar
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Image Descriptions
##&nbsp;
##&nbsp; active : N.9.16.4
##&nbsp; backup : 9.2.17.2
##&nbsp;
##&nbsp;
##&nbsp; Images currently available on Flash
##&nbsp;
##&nbsp;--------------------------------------------------------------------
##&nbsp; unit      active      backup     current-active        next-active
##&nbsp;--------------------------------------------------------------------
##&nbsp;
##&nbsp;    1     1.0.0.2     1.0.0.0            1.0.0.2            1.0.0.2
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'image_descriptions' => [
##&nbsp;                                      {
##&nbsp;                                        'active' => 'N.9.16.4',
##&nbsp;                                        'backup' => '9.2.17.2'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'images_currently_available_on_flash' => [
##&nbsp;                                                       {
##&nbsp;                                                         'unit' => '1',
##&nbsp;                                                         'active' => '1.0.0.2',
##&nbsp;                                                         'next_active' => '1.0.0.2',
##&nbsp;                                                         'current_active' => '1.0.0.2',
##&nbsp;                                                         'backup' => '1.0.0.0'
##&nbsp;                                                       }
##&nbsp;                                                     ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_bootvar {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( @values, @key_names, $key, $k );

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        if ( $line
            =~ /(Image\s+Descriptions|Images\s+currently\s+available\s+on\s+Flash)/
            ) {
            $key = to_key_name( item => $1 );
        } elsif ( $line =~ /(active|backup)\s+:/ ) {
            my $key1 = $1;
            $line =~ s/$key1\s+://;
            trim( 'string_ptr' => \$line );
            $result_array[0]->{$key}->[0]->{$key1} = $line;
        } elsif (
            $line =~ /unit\s+active\s+backup\s+current-active\s+next-active/ )
        {
            @key_names = split( /\s+/, $line );
        } elsif ( $line =~ /.*/ && $line !~ /-----/ ) {
            @values = split( /\s+/, $line );
            foreach my $key_name (@key_names) {
                $key_name =~ s/-/_/;
                $result_array[$k]->{$key}->[0]->{$key_name} = $values[0];
                shift @values;
            }
            $k++;
        }
    }
    return \@result_array;
} ## end sub show_bootvar

## @name ::show_port
##
## @summary Parses the raw output from the "show port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_port with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_port(%args);
## INPUTS TO METHOD CALL 'show_port' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> 'description' </td> <td> 'description' </td> <td> '1' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show port all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                  Admin    Physical   Physical   Link   Link    LACP   Actor
##&nbsp; Intf      Type   Mode     Mode       Status     Status Trap    Mode   Timeout
##&nbsp;--------- ------ --------- ---------- ---------- ------ ------- ------ --------
##&nbsp;0/1              Enable    Auto       100 Full   Up     Enable  Enable long
##&nbsp;0/2              Enable    Auto       100 Full   Up     Enable  Enable long
##&nbsp;0/3              Enable    Auto       100 Full   Up     Enable  Enable long
##&nbsp;0/4              Enable    Auto                  Down   Enable  Enable long
##&nbsp;0/5              Enable    Auto                  Down   Enable  Enable long
##&nbsp;0/6              Enable    Auto                  Down   Enable  Enable long
##&nbsp;0/7              Enable    Auto                  Down   Enable  Enable long
##&nbsp;0/8              Enable    Auto                  Down   Enable  Enable long
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/1',
##&nbsp;            'physical_status' => '100 Full',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Up',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/2',
##&nbsp;            'physical_status' => '100 Full',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Up',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/3',
##&nbsp;            'physical_status' => '100 Full',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Up',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/4',
##&nbsp;            'physical_status' => '',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Down',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/5',
##&nbsp;            'physical_status' => '',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Down',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/6',
##&nbsp;            'physical_status' => '',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Down',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/7',
##&nbsp;            'physical_status' => '',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Down',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link_trap' => 'Enable',
##&nbsp;            'lacp_mode' => 'Enable',
##&nbsp;            'actor_timeout' => 'long',
##&nbsp;            'physical_mode' => 'Auto',
##&nbsp;            'interface' => '0/8',
##&nbsp;            'physical_status' => '',
##&nbsp;            'type' => '',
##&nbsp;            'link_status' => 'Down',
##&nbsp;            'admin_mode' => 'Enable'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_port {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( $pos1, $j, $k ) = ( 0, 0, 0 );
    my ( @result_array, @headings, $pos2, $length, $value, $line, $string );
    my $option = $command;
    $option =~ s/show port//;

    if ( $option =~ /description/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(.*)\.\s+(.*)/ ) {
                my $decs_key = $1;
                my $desc_val = $2;
                $decs_key =~ s/[.]/ /g;
                trim( 'string_ptr' => \$decs_key );
                $result_array[0]->{ to_key_name( item => "$decs_key" ) }
                    = $desc_val;
            }
        }
    } else {
        foreach my $line (@lines) {
            if ( $line
                =~ /(Intf\s+Type\s+Mode\s+.*\s+Mode)|(Admin\s+Physical\s+.*\s+Flow)/
                ) {
                @headings = (
                    "interface",       "type",
                    "admin_mode",      "physical_mode",
                    "physical_status", "link_status",
                    "link_trap",       "lacp_mode",
                    "actor_timeout"
                );
            } elsif ( $line =~ /-------/ ) {
                $string = "$line" . " ";
            } elsif ( $line =~ /(\d+\/\d+)\s+.*/ ) {
                while ( $string =~ /\s+/g ) {
                    $pos2 = pos($string);
                    my $len = $pos2 - $pos1;
                    my $val = substr( $line, $pos1, $len );
                    $pos1 = $pos2;
                    trim( 'string_ptr' => \$val );
                    $result_array[$k]->{ $headings[ $j++ ] } = $val;
                }
                $pos1 = 0;
                $j    = 0;
                $k++;
            }
        }
    }
    return \@result_array;
} ## end sub show_port

## @name ::show_running_config
##
## @summary Parses the raw output from the "show running-config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_running_config with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_running_config(%args);
## INPUTS TO METHOD CALL 'show_running_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show running-config
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;!Current Configuration:
##&nbsp;!
##&nbsp;!System Description "NetApp CN1601, 1.0.0.2, Linux 2.6.21.7"
##&nbsp;!System Software Version "1.0.0.2"
##&nbsp;!System Up Time          "36 days 0 hrs 14 mins 49 secs"
##&nbsp;!Additional Packages     FASTPATH QOS
##&nbsp;!Current SNTP Synchronized Time: Mar 15 04:24:20 201 UTC
##&nbsp;!
##&nbsp;set prompt "MS:net-1961-1"
##&nbsp;telnetcon timeout 60
##&nbsp;enable password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 encrypted
##&nbsp;network parms 172.22.200.91 255.255.254.0 172.22.200.1
##&nbsp;vlan database
##&nbsp;exit
##&nbsp;ip ssh server enable
##&nbsp;sshcon timeout 60
##&nbsp;configure
##&nbsp;sntp client mode unicast
##&nbsp;! sntp server status is Success
##&nbsp;sntp server "172.19.2.16"
##&nbsp;ip domain name "lab.netapp.com"
##&nbsp;ip name server 172.22.201.46
##&nbsp;username "admin" password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 level 15 encrypted
##&nbsp;line console
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;serial timeout 60
##&nbsp;exit
##&nbsp;line telnet
##&nbsp;exit
##&nbsp;line ssh
##&nbsp;exit
##&nbsp;spanning-tree configuration name "00-A0-98-4B-8A-40"
##&nbsp;port-channel name 1/1 ISL LAG
##&nbsp;interface 0/15
##&nbsp;addport 1/1
##&nbsp;exit
##&nbsp;interface 0/16
##&nbsp;addport 1/1
##&nbsp;exit
##&nbsp;snmp-server sysname "MS:net-1961-1"
##&nbsp;!
##&nbsp;snmp-server community "netapp"
##&nbsp;interface 0/1
##&nbsp;description 'Cluster Node 1'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/2
##&nbsp;description 'Cluster Node 2'
##&nbsp;spanning-tree edgeport
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;exit
##&nbsp;interface 0/15
##&nbsp;description 'ISL member port 15'
##&nbsp;exit
##&nbsp;interface 0/16
##&nbsp;description 'ISL member port 16'
##&nbsp;exit
##&nbsp;interface 1/1
##&nbsp;description 'ISL LAG - ports 15-16'
##&nbsp;port-channel load-balance 6
##&nbsp;exit
##&nbsp;interface 1/6
##&nbsp;description 'NetApp CN1601 Management Switch RCF v1'
##&nbsp;exit
##&nbsp;isdp timer 5
##&nbsp;exit
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'ip_ssh_server' => 'enable',
##&nbsp;            'spanning_tree_configuration' => '00-A0-98-4B-8A-40',
##&nbsp;            'snmp_server_community' => 'netapp',
##&nbsp;            'snmp_server_sysname' => 'MS:net-1961-1',
##&nbsp;            'lines' => [
##&nbsp;                         {
##&nbsp;                           'serial_timeout' => '60',
##&nbsp;                           'line' => 'console'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'line' => 'telnet'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'line' => 'ssh'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;            'current_sntp_synchronized_time' => 'Mar 15 04:24:20 201 UTC',
##&nbsp;            'password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;            'interfaces' => [
##&nbsp;                              {
##&nbsp;                                'interface' => '0/15',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'addport 1/1',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/16',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'addport 1/1',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/1',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'Cluster Node 1\'',
##&nbsp;                                                      'spanning-tree edgeport',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/2',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'Cluster Node 2\'',
##&nbsp;                                                      'spanning-tree edgeport',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/15',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'ISL member port 15\'',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '0/16',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'ISL member port 16\'',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '1/1',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'ISL LAG - ports 15-16\'',
##&nbsp;                                                      'port-channel load-balance 6',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'interface' => '1/6',
##&nbsp;                                'running_config' => [
##&nbsp;                                                      'description \'NetApp CN1601 Management Switch RCF v1\'',
##&nbsp;                                                      'exit'
##&nbsp;                                                    ]
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'sntp_server' => '172.19.2.16',
##&nbsp;            'additional_packages' => [
##&nbsp;                                       'FASTPATH',
##&nbsp;                                       'QOS'
##&nbsp;                                     ],
##&nbsp;            'sshcon_timeout' => '60',
##&nbsp;            'port_channel' => '1/1 ISL LAG',
##&nbsp;            'system_up_time' => '36 days 0 hrs 14 mins 49 secs',
##&nbsp;            'set_prompt' => 'MS:net-1961-1',
##&nbsp;            'enable_password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;            'system_description' => 'NetApp CN1601, 1.0.0.2, Linux 2.6.21.7',
##&nbsp;            'system_software_version' => '1.0.0.2',
##&nbsp;            'sntp_client_mode' => 'unicast',
##&nbsp;            'username' => 'admin',
##&nbsp;            'level' => '15',
##&nbsp;            'ip_name_server' => '172.22.201.46',
##&nbsp;            'sntp_server_status' => 'Success',
##&nbsp;            'ip_domain_name' => 'lab.netapp.com',
##&nbsp;            'isdp_timer' => '5',
##&nbsp;            'telnetcon_timeout' => '60',
##&nbsp;            'network_parms' => [
##&nbsp;                                 '172.22.200.91',
##&nbsp;                                 '255.255.254.0',
##&nbsp;                                 '172.22.200.1'
##&nbsp;                               ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_running_config {
    my %args = @_;

    my ( @result_array, $error_message, @packages );
    my $int = -1;
    my $key;
    my $j     = 0;
    my $lines = -1;
    my $con   = 0;
    my @lines;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    #To handle the show tech-support command
    if ($output=~/Show Running Config/){
        @lines = get_lines( raw_output => $output, response_object => $resp, first_line => "Show Running Config");
    } else {
        @lines = get_lines( raw_output => $output, response_object => $resp );
    }
    if ($output =~ /description 'NetApp CN1601 Mgmt. Switch RCF\s+(\S+).*/i) {
        $result_array[0]->{'rcf_version'} = $1;
    }
    while ( my $line = shift(@lines) ) {
        last if ($line =~ /Show Debugging/);
        next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
        if ( $line =~ /interface\s+(.*)/ ) {
            $key                                              = "interfaces";
            $result_array[0]->{$key}->[ ++$int ]->{interface} = $1;
            $con                                              = 0;
        } elsif ( $key eq "interfaces" ) {
            $result_array[0]->{interfaces}->[$int]->{'running_config'}
                ->[ $con++ ] = $line;
            $key = undef if ( $line eq "exit" );
        } elsif ( $line =~ /(serial\s+timeout|password)\s+(.*)/
            && $key eq "lines" ) {
            my $key3 = $1;
            my $val3 = $2;
            $key3 =~ s/-/_/g;
            $result_array[0]->{$key}->[$lines]
                ->{ to_key_name( item => "$key3" ) } = $val3;
        } elsif ( $line =~ /line\s+(\w+)/ ) {
            $key = "lines";
            $result_array[0]->{$key}->[ ++$lines ]->{line} = $1;
        } elsif ( $line
            =~ /(System\s+Up\s+Time|System\s+Description|Current\s+SNTP\s+Synchronized\s+Time:)\s+"?(.*)"?/
            ) {
            my $value2 = $2;
            my $key2   = $1;
            $value2 =~ s/"//g;
            $result_array[0]->{ to_key_name( item => "$key2" ) } = $value2;
        } elsif ( $line =~ /(sntp\s+server\s+status)\s+is\s+(.*)/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $2;
        } elsif ( $line
            =~ /((spanning-tree\s+configuration|port-channel)\s+name)\s*"?(.*)"?/
            ) {
            my $val4 = $3;
            my $key4 = $2;
            $val4 =~ s/"//g;
            $key4 =~ s/-/_/g;
            $result_array[0]->{ to_key_name( item => "$key4" ) } = $val4;
        } elsif ( $line
            =~ /^!?((ip|snmp-server|System|sntp|serviceport\s+protocol|sshcon|telnetcon|set\s+prompt|hostname)\s*.*)\s+(.*)/
            ) {
            my $value1 = $3;
            my $key1   = $1;
            $key1   =~ s/-/_/;
            $value1 =~ s/"//g;
            $result_array[0]->{ to_key_name( item => "$key1" ) } = $value1;
        } elsif ( $line
            =~ /(Additional\s+Packages|serviceport\s+ip|network parms)\s+(.*)/
            ) {
            my @entries = split( /\s+/, $2 );
            $result_array[0]->{ to_key_name( item => $1 ) } = \@entries;
        } elsif ( $line =~ /(isdp\s+timer|spanning-tree)\s+(.*)/ ) {
            my $key5 = $1;
            my $val5 = $2;
            $key5 =~ s/-/_/g;
            $result_array[0]->{ to_key_name( item => "$key5" ) } = $val5;
        } elsif ( $line
            =~ /(username)\s+"(\w+)"\s+(password)\s+(.*)\s+(level)\s+(.*)\s+encrypted/
            ) {
            $result_array[0]->{"$1"} = $2;
            $result_array[0]->{"$3"} = $4;
            $result_array[0]->{"$5"} = $6;
        } elsif ( $line =~ /((enable)?\s+password)\s+(.*)\s+encrypted/ ) {
            $result_array[0]->{ to_key_name( item => "$1" ) } = $3;
        } elsif ( $line eq "exit" ) {
            $key = undef;
        }
    }
    return \@result_array;
} ## end sub show_running_config

## @name ::show_mac_addr_table
##
## @summary Parses the raw output from the "show mac-addr-table" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_mac_addr_table with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_mac_addr_table(%args);
## INPUTS TO METHOD CALL 'show_mac_addr_table' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show mac-addr-table
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  ------------
##&nbsp;1        00:03:BA:12:FA:C6   1/1        18       Learned
##&nbsp;1        00:05:73:A5:83:40   1/1        18       Learned
##&nbsp;1        00:05:73:A5:84:C0   1/1        18       Learned
##&nbsp;1        00:05:73:D2:84:40   1/1        18       Learned
##&nbsp;1        00:05:73:D2:86:40   1/1        18       Learned
##&nbsp;1        00:05:9B:76:C5:40   1/1        18       Learned
##&nbsp;1        00:0C:29:1E:B6:C7   1/1        18       Learned
##&nbsp;1        00:0C:29:47:7F:80   1/1        18       Learned
##&nbsp;1        00:0C:29:53:6C:81   1/1        18       Learned
##&nbsp;1        00:0C:29:C2:5C:61   1/1        18       Learned
##&nbsp;1        00:0D:EC:B1:7F:C0   1/1        18       Learned
##&nbsp;1        00:0D:EC:B1:8D:C0   1/1        18       Learned
##&nbsp;1        00:0D:EC:D6:58:80   1/1        18       Learned
##&nbsp;1        00:0D:EC:FE:99:40   1/1        18       Learned
##&nbsp;1        00:22:55:79:5C:C1   1/1        18       Learned
##&nbsp;1        00:25:90:6E:CC:64   1/1        18       Learned
##&nbsp;1        00:25:90:6E:CC:65   1/1        18       Learned
##&nbsp;1        00:30:48:24:24:6D   1/1        18       Learned
##&nbsp;1        00:30:48:24:24:FA   1/1        18       Learned
##&nbsp;1        00:30:48:2E:B2:52   1/1        18       Learned
##&nbsp;1        00:30:48:2E:B3:8B   1/1        18       Learned
##&nbsp;--More-- or (q)uit
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  ------------
##&nbsp;1        00:30:48:2E:B3:95   1/1        18       Learned
##&nbsp;1        00:30:48:2E:B3:A5   1/1        18       Learned
##&nbsp;1        00:30:48:57:1B:60   1/1        18       Learned
##&nbsp;1        00:30:48:60:4B:64   1/1        18       Learned
##&nbsp;1        00:30:48:BE:67:68   1/1        18       Learned
##&nbsp;1        00:30:48:C8:2A:30   1/1        18       Learned
##&nbsp;1        00:30:48:C8:2C:46   1/1        18       Learned
##&nbsp;1        00:30:48:C8:2F:14   1/1        18       Learned
##&nbsp;1        00:30:48:C8:2F:20   1/1        18       Learned
##&nbsp;1        00:30:48:F9:CC:1E   1/1        18       Learned
##&nbsp;1        00:30:48:FC:16:9E   1/1        18       Learned
##&nbsp;1        00:30:48:FC:18:56   1/1        18       Learned
##&nbsp;1        00:50:56:A2:07:CA   1/1        18       Learned
##&nbsp;1        00:50:56:BB:0C:08   1/1        18       Learned
##&nbsp;1        00:50:56:BB:73:E0   1/1        18       Learned
##&nbsp;1        00:A0:98:02:C5:5F   1/1        18       Learned
##&nbsp;1        00:A0:98:13:A3:E9   1/1        18       Learned
##&nbsp;1        00:A0:98:13:A3:EA   1/1        18       Learned
##&nbsp;1        00:A0:98:13:A3:F1   0/4        4        Learned
##&nbsp;1        00:A0:98:13:A3:F2   0/4        4        Learned
##&nbsp;1        00:A0:98:13:CD:BC   1/1        18       Learned
##&nbsp;1        00:A0:98:13:CD:BC   1/1        18       Learned
##&nbsp;--More-- or (q)uit
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  ------------
##&nbsp;1        00:A0:98:14:D7:7E   1/1        18       Learned
##&nbsp;1        00:A0:98:15:79:4E   1/1        18       Learned
##&nbsp;1        00:A0:98:15:B3:CE   1/1        18       Learned
##&nbsp;1        00:A0:98:18:38:61   0/1        1        Learned
##&nbsp;1        00:A0:98:18:38:62   0/1        1        Learned
##&nbsp;1        00:A0:98:18:6C:90   1/1        18       Learned
##&nbsp;1        00:A0:98:18:7B:97   1/1        18       Learned
##&nbsp;1        00:A0:98:18:7B:98   1/1        18       Learned
##&nbsp;1        00:A0:98:23:80:E8   1/1        18       Learned
##&nbsp;1        00:A0:98:25:0C:19   0/3        3        Learned
##&nbsp;1        00:A0:98:25:0C:1C   0/3        3        Learned
##&nbsp;1        00:A0:98:25:0C:71   1/1        18       Learned
##&nbsp;1        00:A0:98:25:0C:74   1/1        18       Learned
##&nbsp;1        00:A0:98:29:78:B2   0/2        2        Learned
##&nbsp;1        00:A0:98:4B:8A:52   1/1        18       Learned
##&nbsp;1        00:A0:98:4B:B3:A1   0/13       13       Learned
##&nbsp;1        00:E0:81:28:C9:51   1/1        18       Learned
##&nbsp;1        08:00:20:A8:6E:59   1/1        18       Learned
##&nbsp;1        08:00:20:A8:F4:6E   1/1        18       Learned
##&nbsp;1        08:00:20:C3:14:1E   1/1        18       Learned
##&nbsp;1        08:00:20:C3:AD:6E   1/1        18       Learned
##&nbsp;--More-- or (q)uit
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  ------------
##&nbsp;1        54:7F:EE:00:8B:80   1/1        18       Learned
##&nbsp;1        54:7F:EE:49:38:C0   1/1        18       Learned
##&nbsp;1        5C:F3:FC:EF:4F:EA   1/1        18       Learned
##&nbsp;600      00:03:BA:12:FA:C6   0/14       14       Learned
##&nbsp;600      00:05:73:A5:83:40   0/14       14       Learned
##&nbsp;600      00:05:73:A5:84:C0   0/14       14       Learned
##&nbsp;600      00:05:73:D2:84:40   0/14       14       Learned
##&nbsp;600      00:05:9B:76:C5:40   0/14       14       Learned
##&nbsp;600      00:0C:29:1E:B6:C7   0/14       14       Learned
##&nbsp;600      00:0C:29:47:7F:80   0/14       14       Learned
##&nbsp;600      00:0C:29:C2:5C:61   0/14       14       Learned
##&nbsp;600      00:0D:EC:B1:7F:C0   0/14       14       Learned
##&nbsp;600      00:0D:EC:B1:8D:C0   0/14       14       Learned
##&nbsp;600      00:0D:EC:D6:58:80   0/14       14       Learned
##&nbsp;600      00:0D:EC:FE:99:40   0/14       14       Learned
##&nbsp;600      00:22:55:79:5C:C1   0/14       14       Learned
##&nbsp;600      00:22:55:EA:7E:DA   0/14       14       Learned
##&nbsp;600      00:25:90:6E:CC:64   0/14       14       Learned
##&nbsp;600      00:25:90:6E:CC:65   0/14       14       Learned
##&nbsp;600      00:30:48:24:24:6D   0/14       14       Learned
##&nbsp;600      00:30:48:24:24:FA   0/14       14       Learned
##&nbsp;--More-- or (q)uit
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  -------
##&nbsp;600      00:30:48:2E:B2:52   0/14       14       Learned
##&nbsp;600      00:30:48:2E:B3:8B   0/14       14       Learned
##&nbsp;600      00:30:48:2E:B3:95   0/14       14       Learned
##&nbsp;600      00:30:48:2E:B3:A5   0/14       14       Learned
##&nbsp;600      00:30:48:2E:B3:B0   0/14       14       Learned
##&nbsp;600      00:30:48:57:1B:60   0/14       14       Learned
##&nbsp;600      00:30:48:60:4B:64   0/14       14       Learned
##&nbsp;600      00:30:48:BE:67:68   0/14       14       Learned
##&nbsp;600      00:30:48:C8:2A:30   0/14       14       Learned
##&nbsp;600      00:30:48:C8:2C:46   0/14       14       Learned
##&nbsp;600      00:30:48:C8:2F:14   0/14       14       Learned
##&nbsp;600      00:30:48:C8:2F:20   0/14       14       Learned
##&nbsp;600      00:30:48:F9:CC:1E   0/14       14       Learned
##&nbsp;600      00:30:48:FC:16:9E   0/14       14       Learned
##&nbsp;600      00:30:48:FC:18:56   0/14       14       Learned
##&nbsp;600      00:50:56:A2:07:CA   0/14       14       Learned
##&nbsp;600      00:50:56:BB:0C:08   0/14       14       Learned
##&nbsp;600      00:50:56:BB:73:E0   0/14       14       Learned
##&nbsp;600      00:A0:98:02:C5:5F   0/14       14       Learned
##&nbsp;600      00:A0:98:13:CD:BC   0/14       14       Learned
##&nbsp;600      00:A0:98:14:D7:7E   0/14       14       Learned
##&nbsp;--More-- or (q)uit
##&nbsp;VLAN ID  MAC Address         Interface  IfIndex  Status
##&nbsp;-------  ------------------  ---------  -------  -------
##&nbsp;600      00:A0:98:15:79:4E   0/14       14       Learned
##&nbsp;600      00:A0:98:15:B3:CE   0/14       14       Learned
##&nbsp;600      00:A0:98:18:6C:90   0/14       14       Learned
##&nbsp;600      00:A0:98:23:80:E8   0/14       14       Learned
##&nbsp;600      00:A0:98:4B:8A:40   3/1        17       Managem
##&nbsp;600      00:E0:81:28:C9:51   0/14       14       Learned
##&nbsp;600      08:00:20:A8:6E:59   0/14       14       Learned
##&nbsp;600      08:00:20:A8:F4:6E   0/14       14       Learned
##&nbsp;600      08:00:20:C3:14:1E   0/14       14       Learned
##&nbsp;600      08:00:20:C3:AD:6E   0/14       14       Learned
##&nbsp;600      54:7F:EE:00:8B:80   0/14       14       Learned
##&nbsp;600      54:7F:EE:49:38:C0   0/14       14       Learned
##&nbsp;600      5C:F3:FC:EF:4F:EA   0/14       14       Learned
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:03:BA:12:FA:C6',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:05:73:A5:83:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:05:73:A5:84:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:05:73:D2:84:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:05:73:D2:86:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:05:9B:76:C5:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0C:29:1E:B6:C7',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0C:29:47:7F:80',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0C:29:53:6C:81',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0C:29:C2:5C:61',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0D:EC:B1:7F:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0D:EC:B1:8D:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0D:EC:D6:58:80',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:0D:EC:FE:99:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:22:55:79:5C:C1',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:25:90:6E:CC:64',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:25:90:6E:CC:65',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:24:24:6D',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:24:24:FA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:2E:B2:52',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:8B',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:95',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:A5',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:57:1B:60',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:60:4B:64',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:BE:67:68',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:C8:2A:30',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:C8:2C:46',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:C8:2F:14',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:C8:2F:20',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:F9:CC:1E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:FC:16:9E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:30:48:FC:18:56',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:50:56:A2:07:CA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:50:56:BB:0C:08',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:50:56:BB:73:E0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:02:C5:5F',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:13:A3:E9',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:13:A3:EA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '4',
##&nbsp;             'mac_address' => '00:A0:98:13:A3:F1',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/4',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '4',
##&nbsp;             'mac_address' => '00:A0:98:13:A3:F2',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/4',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:13:CD:BC',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:14:D7:7E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:15:79:4E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:15:B3:CE',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '1',
##&nbsp;             'mac_address' => '00:A0:98:18:38:61',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '1',
##&nbsp;             'mac_address' => '00:A0:98:18:38:62',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:18:6C:90',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:18:7B:97',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:18:7B:98',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:23:80:E8',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '3',
##&nbsp;             'mac_address' => '00:A0:98:25:0C:19',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/3',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '3',
##&nbsp;             'mac_address' => '00:A0:98:25:0C:1C',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/3',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:25:0C:71',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:25:0C:74',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '2',
##&nbsp;             'mac_address' => '00:A0:98:29:78:B2',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/2',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:A0:98:4B:8A:52',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '13',
##&nbsp;             'mac_address' => '00:A0:98:4B:B3:A1',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/13',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '00:E0:81:28:C9:51',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '08:00:20:A8:6E:59',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '08:00:20:A8:F4:6E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '08:00:20:C3:14:1E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '08:00:20:C3:AD:6E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '54:7F:EE:00:8B:80',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '54:7F:EE:49:38:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '18',
##&nbsp;             'mac_address' => '5C:F3:FC:EF:4F:EA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '1/1',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:03:BA:12:FA:C6',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:05:73:A5:83:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:05:73:A5:84:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:05:73:D2:84:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:05:9B:76:C5:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0C:29:1E:B6:C7',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0C:29:47:7F:80',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0C:29:C2:5C:61',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0D:EC:B1:7F:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0D:EC:B1:8D:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0D:EC:D6:58:80',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:0D:EC:FE:99:40',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:22:55:79:5C:C1',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:22:55:EA:7E:DA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:25:90:6E:CC:64',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:25:90:6E:CC:65',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:24:24:6D',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:24:24:FA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:2E:B2:52',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:8B',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:95',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:A5',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:2E:B3:B0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:57:1B:60',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:60:4B:64',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:BE:67:68',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:C8:2A:30',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:C8:2C:46',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:C8:2F:14',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:C8:2F:20',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:FC:16:9E',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:30:48:FC:18:56',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:50:56:A2:07:CA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:50:56:BB:0C:08',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:50:56:BB:73:E0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '00:A0:98:02:C5:5F',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '54:7F:EE:49:38:C0',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'ifindex' => '14',
##&nbsp;             'mac_address' => '5C:F3:FC:EF:4F:EA',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'interface' => '0/14',
##&nbsp;             'vlan_id' => '600'
##&nbsp;           }
##&nbsp;         ];

sub show_mac_addr_table {
    my %args = @_;

    my ($error_message);

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line =~ /(\d+)\s+(\S+)\s+(\d+\/\d+)\s+(\d+)\s+(\w+)/ ) {
            $result_array[$i]->{'vlan_id'}     = $1;
            $result_array[$i]->{'mac_address'} = $2;
            $result_array[$i]->{'interface'}   = $3;
            $result_array[$i]->{'ifindex'}     = $4;
            $result_array[$i]->{'status'}      = $5;
            $i++;
        }
    }
    return \@result_array;
} ## end sub show_mac_addr_table

## @name ::show_mac_addr_table_interface
##
## @summary Parses the raw output from the "show mac-addr-table interface" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_mac_addr_table_interface with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_mac_addr_table_interface(%args);
## INPUTS TO METHOD CALL 'show_mac_addr_table_interface' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show mac-addr-table interface 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;   MAC Address      VLAN ID      Status
##&nbsp;-----------------  ---------  ------------
##&nbsp;00:A0:98:18:38:61     1        Learned
##&nbsp;00:A0:98:18:38:62     1        Learned
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'mac_address' => '00:A0:98:18:38:61',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'mac_address' => '00:A0:98:18:38:62',
##&nbsp;             'status' => 'Learned',
##&nbsp;             'vlan_id' => '1'
##&nbsp;           }
##&nbsp;         ];

sub show_mac_addr_table_interface {
    my %args = @_;

    my ($error_message);

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line =~ /(\S+)\s+(\d+)\s+(\w+)/ ) {
            $result_array[$i]->{'mac_address'} = $1;
            $result_array[$i]->{'vlan_id'}     = $2;
            $result_array[$i]->{'status'}      = $3;
            $i++;

        }
    }
    return \@result_array;
} ## end sub show_mac_addr_table_interface

## @name ::show_hardware
##
## @summary Parses the raw output from the "show hardware" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_hardware with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_hardware(%args);
## INPUTS TO METHOD CALL 'show_hardware' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show hardware
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Switch: 1
##&nbsp;
##&nbsp;System Description............................. NetApp CN1601, 1.0.0.4, Linux
##&nbsp;                                                2.6.21.7
##&nbsp;Machine Type................................... NetApp CN1601
##&nbsp;Machine Model.................................. CN1601
##&nbsp;Serial Number.................................. 10610100001
##&nbsp;Burned In MAC Address.......................... 00:A0:98:4B:8A:40
##&nbsp;Software Version............................... 1.0.0.4
##&nbsp;Operating System............................... Linux 2.6.21.7
##&nbsp;Network Processing Device...................... BCM53716_A2
##&nbsp;Part Number.................................... 111-00894
##&nbsp;Manufacturer Name.............................. Delta Networks
##&nbsp;Revision....................................... 20
##&nbsp;Date Code...................................... 20110621
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Additional Packages............................ FASTPATH QOS
##&nbsp;                                                FASTPATH IPv6 Management
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => '10610100001',
##&nbsp;            'operating_system' => 'Linux 2.6.21.7',
##&nbsp;            'network_processing_device' => 'BCM53716_A2',
##&nbsp;            'machine_type' => 'NetApp CN1601',
##&nbsp;            'system_description' => 'NetApp CN1601, 1.0.0.4, Linux 2.6.21.7',
##&nbsp;            'date_code' => '20110621',
##&nbsp;            'revision' => '20',
##&nbsp;            'part_number' => '111-00894',
##&nbsp;            'burned_in_mac_address' => '00:A0:98:4B:8A:40',
##&nbsp;            'software_version' => '1.0.0.4',
##&nbsp;            'switch' => '1',
##&nbsp;            'additional_packages' => [
##&nbsp;                                       'FASTPATH QOS',
##&nbsp;                                       'FASTPATH IPv6 Management'
##&nbsp;                                     ],
##&nbsp;            'manufacturer_name' => 'Delta Networks',
##&nbsp;            'machine_model' => 'CN1601'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_hardware {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    if ( not defined($error) ) {
        $error = _check_error(
            raw_output    => $output,
            command       => $command,
            error_message => \$error_message
        );
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my @lines
        = get_lines( raw_output => $output, first_line => 'Switch:\s+\d+' );
    my $header = undef;
    my ( @header_length, @header_keys );
    my @power_supply_header = (
        qw(unit power_supply state part_number_and_revision
            mfg_part_number_and_revision serial_number date_code)
    );
    my @fan_tray_header = (qw(unit fan_tray state part_number_and_revision));
    my @sfp_module_header
        = (qw(unit sfp_module state vendor part_number serial_number));
    my $switch_id = -1;
    my ( $line, $j );
    for ( my $line_id = 0; $line_id < scalar(@lines); $line_id++ ) {
        $line = $lines[$line_id];
        next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
        next if ( !$line || !defined($line) );
        if ( $line =~ /Switch:\s+(\d+)/ ) {
            $switch_id++;
            $result_array[$switch_id]->{switch} = $1;
            next;
        }
        if ( $line =~ /^(.*\w)\.\.+\s*(.*)/ ) {
            $result_array[$switch_id]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /Power\sSupply\sHardware\sData/ ) {
            $header = to_key_name( item => "Power Supply Hardware Data" );
            $j = 0;
            @header_keys = @power_supply_header;
        } elsif ( $line =~ /Fan\sTray\sHardware\sData/ ) {
            $header      = to_key_name( item => "Fan Tray Hardware Data" );
            $j           = 0;
            @header_keys = @fan_tray_header;
        } elsif ( $line =~ /SFP\sModule\sHardware\sData/ ) {
            $header      = to_key_name( item => "SFP Module Hardware Data" );
            $j           = 0;
            @header_keys = @sfp_module_header;
        } elsif ( $line =~ /(Unit|Part\sNumber|SFP)/ ) {
            next;
        } elsif ( $line =~ /^----/ ) {
            @header_length = split( /\s+/, $line );
        } else {
            if ( !defined($header) ) {
                if ( $lines[ $line_id - 1 ] =~ /Additional\sPackages/ ) {
                    my @temp_arr;
                    my $temp
                        = $result_array[$switch_id]->{additional_packages};
                    if ( ref($temp) ) {
                        @temp_arr = ( @$temp, $line );
                    } else {
                        @temp_arr = ( $temp, $line );
                    }
                    $result_array[$switch_id]->{additional_packages}
                        = \@temp_arr;
                } else {
                    $lines[ $line_id - 1 ] =~ m/^(.*\w)\.\.+\s*(.*)/;
                    $result_array[$switch_id]->{ to_key_name( item => $1 ) }
                        .= " " . $line;
                }
            } else {
                my $l = 0;
                for ( my $i = 0; $i < scalar(@header_length); $i++ ) {
                    if ( $i != scalar(@header_length) - 1 ) {
                        $result_array[$switch_id]->{$header}->[$j]
                            ->{ $header_keys[$i] }
                            = substr( $line, $l,
                            length( $header_length[$i] ) );
                    } else {
                        $result_array[$switch_id]->{$header}->[$j]
                            ->{ $header_keys[$i] } = substr( $line, $l );
                    }
                    $l += ( length( $header_length[$i] ) + 2 );
                    $result_array[$switch_id]->{$header}->[$j]
                        ->{ $header_keys[$i] } =~ s/^\s+//;
                    $result_array[$switch_id]->{$header}->[$j]
                        ->{ $header_keys[$i] } =~ s/\s+$//;
                }
                $j++;
            }
        }
    }

    return \@result_array;
} ## end sub show_hardware

## @name ::show_interface
##
## @summary Parses the raw output from the "show interface" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_interface with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_interface(%args);
## INPUTS TO METHOD CALL 'show_interface' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show interface 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Packets Received Without Error................. 159293
##&nbsp;Packets Received With Error.................... 0
##&nbsp;Broadcast Packets Received..................... 305
##&nbsp;Packets Transmitted Without Errors............. 2781428
##&nbsp;Transmit Packet Errors......................... 0
##&nbsp;Collision Frames............................... 0
##&nbsp;Time Since Counters Last Cleared............... 13 day 11 hr 48 min 12 sec
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time_since_counters_last_cleared' => '13 day 11 hr 48 min 12 sec',
##&nbsp;            'packets_transmitted_without_errors' => '2781428',
##&nbsp;            'collision_frames' => '0',
##&nbsp;            'packets_received_with_error' => '0',
##&nbsp;            'broadcast_packets_received' => '305',
##&nbsp;            'transmit_packet_errors' => '0',
##&nbsp;            'packets_received_without_error' => '159293'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_interface {
    my %args = @_;

    my ($error_message);

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    foreach my $line (@lines) {
        if ( $line =~ /(.*\w+)\)?\.+\s+(.*)$/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@result_array;
} ## end sub show_interface

## @name ::show_interface_ethernet
##
## @summary Parses the raw output from the "show interface ethernet" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_interface_ethernet with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_interface_ethernet(%args);
## INPUTS TO METHOD CALL 'show_interface_ethernet' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show interface ethernet 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;Total Packets Received (Octets)................ 3255569764
##&nbsp;Packets Received 64 Octets..................... 2735
##&nbsp;Packets Received 65-127 Octets................. 1699869
##&nbsp;Packets Received 128-255 Octets................ 1096628
##&nbsp;Packets Received 256-511 Octets................ 18948
##&nbsp;Packets Received 512-1023 Octets............... 31518
##&nbsp;Packets Received 1024-1518 Octets.............. 158924
##&nbsp;Packets Received > 1518 Octets................. 341368
##&nbsp;Packets RX and TX 64 Octets.................... 28508
##&nbsp;Packets RX and TX 65-127 Octets................ 4622800
##&nbsp;Packets RX and TX 128-255 Octets............... 1227538
##&nbsp;Packets RX and TX 256-511 Octets............... 313217
##&nbsp;Packets RX and TX 512-1023 Octets.............. 252309
##&nbsp;Packets RX and TX 1024-1518 Octets............. 456488
##&nbsp;Packets RX and TX 1519-2047 Octets............. 1490
##&nbsp;Packets RX and TX 2048-4095 Octets............. 29479
##&nbsp;Packets RX and TX 4096-9216 Octets............. 454438
##&nbsp;
##&nbsp;Total Packets Received Without Errors.......... 3349990
##&nbsp;Unicast Packets Received....................... 3339907
##&nbsp;Multicast Packets Received..................... 8984
##&nbsp;Broadcast Packets Received..................... 1099
##&nbsp;Total Packets Received with MAC Errors......... 0
##&nbsp;Jabbers Received............................... 0
##&nbsp;Fragments Received............................. 0
##&nbsp;Undersize Received............................. 0
##&nbsp;Alignment Errors............................... 0
##&nbsp;FCS Errors..................................... 0
##&nbsp;Overruns....................................... 0
##&nbsp;
##&nbsp;Total Received Packets Not Forwarded........... 0
##&nbsp;802.3x Pause Frames Received................... 0
##&nbsp;Unacceptable Frame Type........................ 0
##&nbsp;
##&nbsp;Total Packets Transmitted (Octets)............. 2190513425
##&nbsp;Packets Transmitted 64 Octets.................. 25773
##&nbsp;Packets Transmitted 65-127 Octets.............. 2922931
##&nbsp;Packets Transmitted 128-255 Octets............. 130910
##&nbsp;Packets Transmitted 256-511 Octets............. 294269
##&nbsp;Packets Transmitted 512-1023 Octets............ 220791
##&nbsp;Packets Transmitted 1024-1518 Octets........... 297564
##&nbsp;Packets Transmitted > 1518 Octets.............. 144039
##&nbsp;Max Frame Size................................. 9216
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;Total Packets Transmitted Successfully......... 4036277
##&nbsp;Unicast Packets Transmitted.................... 3515323
##&nbsp;Multicast Packets Transmitted.................. 496248
##&nbsp;Broadcast Packets Transmitted.................. 24706
##&nbsp;
##&nbsp;Total Transmit Errors.......................... 0
##&nbsp;
##&nbsp;Total Transmit Packets Discarded............... 0
##&nbsp;Single Collision Frames........................ 0
##&nbsp;Multiple Collision Frames...................... 0
##&nbsp;Excessive Collision Frames..................... 0
##&nbsp;
##&nbsp;802.3x Pause Frames Transmitted................ 0
##&nbsp;GVRP PDUs received............................. 0
##&nbsp;GVRP PDUs Transmitted.......................... 0
##&nbsp;GVRP Failed Registrations...................... 0
##&nbsp;GMRP PDUs Received............................. 0
##&nbsp;GMRP PDUs Transmitted.......................... 0
##&nbsp;GMRP Failed Registrations...................... 0
##&nbsp;
##&nbsp;STP BPDUs Transmitted.......................... 0
##&nbsp;STP BPDUs Received............................. 0
##&nbsp;RSTP BPDUs Transmitted......................... 0
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;RSTP BPDUs Received............................ 0
##&nbsp;MSTP BPDUs Transmitted......................... 269530
##&nbsp;MSTP BPDUs Received............................ 0
##&nbsp;
##&nbsp;EAPOL Frames Transmitted....................... 0
##&nbsp;EAPOL Start Frames Received.................... 0
##&nbsp;
##&nbsp;Time Since Counters Last Cleared............... 6 day 5 hr 46 min 47 sec
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'packets_received_65_127_octets' => '36590',
##&nbsp;            'unacceptable_frame_type' => '0',
##&nbsp;            'total_packets_received_without_errors' => '203977',
##&nbsp;            'undersize_received' => '0',
##&nbsp;            'broadcast_packets_received' => '408',
##&nbsp;            'stp_bpdus_received' => '0',
##&nbsp;            'excessive_collision_frames' => '0',
##&nbsp;            'gvrp_pdus_transmitted' => '0',
##&nbsp;            'total_packets_received_with_mac_errors' => '0',
##&nbsp;            'packets_received_1024_1518_octets' => '22480',
##&nbsp;            'mstp_bpdus_transmitted' => '799400',
##&nbsp;            'multicast_packets_transmitted' => '1874968',
##&nbsp;            'max_frame_size' => '1518',
##&nbsp;            'total_received_packets_not_forwarded' => '0',
##&nbsp;            'packets_rx_and_tx_1024_1518_octets' => '169240',
##&nbsp;            'unicast_packets_transmitted' => '356389',
##&nbsp;            'packets_rx_and_tx_2048_4095_octets' => '0',
##&nbsp;            'packets_transmitted_>_1518_octets' => '0',
##&nbsp;            'packets_rx_and_tx_65_127_octets' => '2640061',
##&nbsp;            'packets_transmitted_1024_1518_octets' => '146760',
##&nbsp;            'multiple_collision_frames' => '0',
##&nbsp;            '802.3x_pause_frames_transmitted' => '0',
##&nbsp;            'packets_received_256_511_octets' => '386',
##&nbsp;            'multicast_packets_received' => '26583',
##&nbsp;            'broadcast_packets_transmitted' => '1516924',
##&nbsp;            'packets_received_64_octets' => '95444',
##&nbsp;            'packets_received_128_255_octets' => '48972',
##&nbsp;            'overruns' => '0',
##&nbsp;            'packets_received_>_1518_octets' => '0',
##&nbsp;            'single_collision_frames' => '0',
##&nbsp;            'time_since_counters_last_cleared' => '18 day 12 hr 17 min 7 sec',
##&nbsp;            'jabbers_received' => '0',
##&nbsp;            'mstp_bpdus_received' => '0',
##&nbsp;            '802.3x_pause_frames_received' => '0',
##&nbsp;            'gvrp_failed_registrations' => '0',
##&nbsp;            'total_packets_transmitted_successfully' => '3748281',
##&nbsp;            'packets_rx_and_tx_256_511_octets' => '35623',
##&nbsp;            'rstp_bpdus_received' => '0',
##&nbsp;            'fragments_received' => '0',
##&nbsp;            'packets_transmitted_128_255_octets' => '75253',
##&nbsp;            'fcs_errors' => '0',
##&nbsp;            'packets_transmitted_512_1023_octets' => '327534',
##&nbsp;            'stp_bpdus_transmitted' => '0',
##&nbsp;            'packets_rx_and_tx_1519_2047_octets' => '0',
##&nbsp;            'packets_rx_and_tx_128_255_octets' => '124225',
##&nbsp;            'packets_transmitted_64_octets' => '560026',
##&nbsp;            'gmrp_pdus_transmitted' => '0',
##&nbsp;            'rstp_bpdus_transmitted' => '0',
##&nbsp;            'eapol_frames_transmitted' => '0',
##&nbsp;            'unicast_packets_received' => '176986',
##&nbsp;            'gvrp_pdus_received' => '0',
##&nbsp;            'packets_rx_and_tx_4096_9216_octets' => '0',
##&nbsp;            'gmrp_pdus_received' => '0',
##&nbsp;            'total_packets_received_octets' => '52292432',
##&nbsp;            'packets_received_512_1023_octets' => '105',
##&nbsp;            'total_transmit_packets_discarded' => '0',
##&nbsp;            'packets_rx_and_tx_64_octets' => '655470',
##&nbsp;            'packets_rx_and_tx_512_1023_octets' => '327639',
##&nbsp;            'gmrp_failed_registrations' => '0',
##&nbsp;            'total_transmit_errors' => '0',
##&nbsp;            'packets_transmitted_256_511_octets' => '35237',
##&nbsp;            'total_packets_transmitted_octets' => '756944845',
##&nbsp;            'alignment_errors' => '0',
##&nbsp;            'packets_transmitted_65_127_octets' => '2603471',
##&nbsp;            'eapol_start_frames_received' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_interface_ethernet {
    my %args = @_;

    my ($error_message);

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    foreach my $line (@lines) {
        if ( $line =~ /(.*\w+)\)?\.+\s+(.*)$/ ) {
            my $temp1 = $1;
            my $temp2 = $2;
            $temp1 =~ s/-/_/;
            $result_array[0]->{ to_key_name( item => $temp1 ) } = $temp2;
        }
    }
    return \@result_array;
} ## end sub show_interface_ethernet

## @name ::show_storm_control
##
## @summary Parses the raw output from the "show storm-control" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_storm_control with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_storm_control(%args);
## INPUTS TO METHOD CALL 'show_storm_control' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show storm-control 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;        Bcast   Bcast    Mcast   Mcast    Ucast   Ucast    Flow
##&nbsp; Intf   Mode    Level    Mode    Level    Mode    Level    Mode
##&nbsp;------ ------- -------- ------- -------- ------- -------- ------
##&nbsp;0/1    Enable         2 Disable       5% Disable       5% Enable
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Enable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '2',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Enable',
##&nbsp;            'interface' => '0/1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show storm-control
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Broadcast Storm Control Mode................... Disable
##&nbsp;Broadcast Storm Control Level.................. 5 percent
##&nbsp;Multicast Storm Control Mode................... Disable
##&nbsp;Multicast Storm Control Level.................. 5 percent
##&nbsp;Unicast Storm Control Mode..................... Disable
##&nbsp;Unicast Storm Control Level.................... 5 percent
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'unicast_storm_control_mode' => 'Disable',
##&nbsp;            'unicast_storm_control_level' => '5 percent',
##&nbsp;            'multicast_storm_control_level' => '5 percent',
##&nbsp;            'multicast_storm_control_mode' => 'Disable',
##&nbsp;            'broadcast_storm_control_mode' => 'Disable',
##&nbsp;            'broadcast_storm_control_level' => '5 percent'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show storm-control all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;        Bcast   Bcast    Mcast   Mcast    Ucast   Ucast    Flow
##&nbsp; Intf   Mode    Level    Mode    Level    Mode    Level    Mode
##&nbsp;------ ------- -------- ------- -------- ------- -------- ------
##&nbsp;0/1    Enable         2 Disable       5% Disable       5% Enable
##&nbsp;0/2    Disable       5% Disable       5% Disable       5% Enable
##&nbsp;0/3    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/4    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/5    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/6    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/7    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/8    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/9    Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/10   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/11   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/12   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/13   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/14   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/15   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;0/16   Disable       5% Disable       5% Disable       5% Disable
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Enable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '2',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Enable',
##&nbsp;            'interface' => '0/1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Enable',
##&nbsp;            'interface' => '0/2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/5'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/6'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/8'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/9'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/10'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/12'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/13'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/14'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bcast_mode' => 'Disable',
##&nbsp;            'ucast_level' => '5%',
##&nbsp;            'mcast_level' => '5%',
##&nbsp;            'bcast_level' => '5%',
##&nbsp;            'mcast_mode' => 'Disable',
##&nbsp;            'ucast_mode' => 'Disable',
##&nbsp;            'flow_mode' => 'Disable',
##&nbsp;            'interface' => '0/16'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_storm_control {
    my %args = @_;
    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        my @stats;
        if ( $lines[$i] =~ /(.*\w+)\)?\.+\s+(.*)$/ ) {
            my $temp1 = $1;
            $temp1 =~ s/-/_/;
            $result_array[0]->{ to_key_name( item => $temp1 ) } = $2;
        }
        my ( @all_fields, @values );
        if (   ( $lines[$i] =~ /Bcast/ )
            && ( $lines[ $i + 1 ] =~ /Intf/ ) ) {
            my @fields1 = split( /\s+/, $lines[$i] );
            my @fields2 = split( /\s+/, $lines[ $i + 1 ] );
            $fields2[0] = "interface" if ( $fields2[0] =~ /Intf/ );
            push( @all_fields, lc( $fields2[0] ) );
            for ( my $field = 0; $field <= $#fields2; $field++ ) {
                my $field_name = lc( $fields1[$field] ) . "_"
                    . lc( $fields2[ $field + 1 ] );
                push( @all_fields, $field_name );
            }
        }
        my $line_count = 0;
        if ( $output =~ /Bcast/ ) {
            while ( $i <= $#lines ) {
                if ( $lines[$i] =~ /^\d/ ) {
                    @values = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j < $#all_fields; $j++ ) {
                        $result_array[$line_count]->{ $all_fields[$j] }
                            = $values[$j];
                    }
                    $line_count++;
                }
                $i++;
            }
        }
    }
    return \@result_array;
} ## end sub show_storm_control

## @name ::show_process_cpu
##
## @summary Parses the raw output from the "show process cpu" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_process_cpu with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_process_cpu(%args);
## INPUTS TO METHOD CALL 'show_process_cpu' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'threshold' </td> <td> 'threshold' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show process cpu
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Memory Utilization Report
##&nbsp;status      bytes
##&nbsp;------ ----------
##&nbsp;free     18436096
##&nbsp;alloc    44376064
##&nbsp;
##&nbsp; CPU Utilization:
##&nbsp;
##&nbsp;  PID      Name                    5 Secs     60 Secs    300 Secs
##&nbsp;-----------------------------------------------------------------
##&nbsp;  128    envMonTask                 0.39%       0.13%       0.10%
##&nbsp;  132    osapiTimer                 0.00%       0.06%       0.06%
##&nbsp;  134    _interrupt_thread          0.19%       0.23%       0.18%
##&nbsp;  136    bcmL2X.0                   3.73%       3.51%       3.44%
##&nbsp;  137    bcmCNTR.0                  0.19%       0.08%       0.08%
##&nbsp;  138    bcmTX                      0.00%       0.06%       0.03%
##&nbsp;  140    bcmLINK.0                  0.19%       0.04%       0.03%
##&nbsp;  141    bcmRX                      0.19%       0.17%       0.15%
##&nbsp;  142    cpuUtilMonitorTask         0.78%       0.86%       0.86%
##&nbsp;  146    tap_monitor_task           0.19%       0.03%       0.01%
##&nbsp;  148    simPts_task                0.00%       0.00%       0.01%
##&nbsp;  273    sshd[0]                   18.07%       2.78%       0.59%
##&nbsp;  213    sshd                      37.71%       9.81%       2.37%
##&nbsp;  209    RMONTask                   0.00%       1.86%       1.60%
##&nbsp;  208    isdpTask                   0.19%       0.22%       0.23%
##&nbsp;  196    dhcpsPingTask              0.00%       0.02%       0.02%
##&nbsp;  189    snoopTask                  0.19%       0.36%       0.38%
##&nbsp;  180    dot1s_timer_task           0.00%       0.07%       0.07%
##&nbsp;  161    hapiRxTask                 0.00%       0.02%       0.04%
##&nbsp;  157    dtlTask                    0.39%       0.10%       0.07%
##&nbsp;-----------------------------------------------------------------
##&nbsp; Total CPU Utilization             62.47%      20.50%      10.54%
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_cpu_utilization' => [
##&nbsp;                                         {
##&nbsp;                                           '300_secs' => '10.54%',
##&nbsp;                                           '5_secs' => '62.47%',
##&nbsp;                                           '60_secs' => '20.50%'
##&nbsp;                                         }
##&nbsp;                                       ],
##&nbsp;            'memory_utilization_report' => [
##&nbsp;                                             {
##&nbsp;                                               'alloc_in_bytes' => '44376064',
##&nbsp;                                               'free_in_bytes' => '18436096'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;            'cpu_utilization' => [
##&nbsp;                                   {
##&nbsp;                                     'pid' => '128',
##&nbsp;                                     'name' => 'envMonTask',
##&nbsp;                                     '300_secs' => '0.10%',
##&nbsp;                                     '5_secs' => '0.39%',
##&nbsp;                                     '60_secs' => '0.13%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '132',
##&nbsp;                                     'name' => 'osapiTimer',
##&nbsp;                                     '300_secs' => '0.06%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.06%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '134',
##&nbsp;                                     'name' => '_interrupt_thread',
##&nbsp;                                     '300_secs' => '0.18%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.23%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '136',
##&nbsp;                                     'name' => 'bcmL2X.0',
##&nbsp;                                     '300_secs' => '3.44%',
##&nbsp;                                     '5_secs' => '3.73%',
##&nbsp;                                     '60_secs' => '3.51%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '137',
##&nbsp;                                     'name' => 'bcmCNTR.0',
##&nbsp;                                     '300_secs' => '0.08%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.08%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '138',
##&nbsp;                                     'name' => 'bcmTX',
##&nbsp;                                     '300_secs' => '0.03%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.06%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '140',
##&nbsp;                                     'name' => 'bcmLINK.0',
##&nbsp;                                     '300_secs' => '0.03%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.04%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '141',
##&nbsp;                                     'name' => 'bcmRX',
##&nbsp;                                     '300_secs' => '0.15%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.17%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '142',
##&nbsp;                                     'name' => 'cpuUtilMonitorTask',
##&nbsp;                                     '300_secs' => '0.86%',
##&nbsp;                                     '5_secs' => '0.78%',
##&nbsp;                                     '60_secs' => '0.86%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '146',
##&nbsp;                                     'name' => 'tap_monitor_task',
##&nbsp;                                     '300_secs' => '0.01%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.03%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '148',
##&nbsp;                                     'name' => 'simPts_task',
##&nbsp;                                     '300_secs' => '0.01%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.00%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '273',
##&nbsp;                                     'name' => 'sshd[0]',
##&nbsp;                                     '300_secs' => '0.59%',
##&nbsp;                                     '5_secs' => '18.07%',
##&nbsp;                                     '60_secs' => '2.78%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '213',
##&nbsp;                                     'name' => 'sshd',
##&nbsp;                                     '300_secs' => '2.37%',
##&nbsp;                                     '5_secs' => '37.71%',
##&nbsp;                                     '60_secs' => '9.81%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '209',
##&nbsp;                                     'name' => 'RMONTask',
##&nbsp;                                     '300_secs' => '1.60%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '1.86%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '208',
##&nbsp;                                     'name' => 'isdpTask',
##&nbsp;                                     '300_secs' => '0.23%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.22%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '196',
##&nbsp;                                     'name' => 'dhcpsPingTask',
##&nbsp;                                     '300_secs' => '0.02%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.02%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '189',
##&nbsp;                                     'name' => 'snoopTask',
##&nbsp;                                     '300_secs' => '0.38%',
##&nbsp;                                     '5_secs' => '0.19%',
##&nbsp;                                     '60_secs' => '0.36%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '180',
##&nbsp;                                     'name' => 'dot1s_timer_task',
##&nbsp;                                     '300_secs' => '0.07%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.07%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '161',
##&nbsp;                                     'name' => 'hapiRxTask',
##&nbsp;                                     '300_secs' => '0.04%',
##&nbsp;                                     '5_secs' => '0.00%',
##&nbsp;                                     '60_secs' => '0.02%'
##&nbsp;                                   },
##&nbsp;                                   {
##&nbsp;                                     'pid' => '157',
##&nbsp;                                     'name' => 'dtlTask',
##&nbsp;                                     '300_secs' => '0.07%',
##&nbsp;                                     '5_secs' => '0.39%',
##&nbsp;                                     '60_secs' => '0.10%'
##&nbsp;                                   }
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show process cpu threshold
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;CPU Utilization Monitoring Parameters
##&nbsp;Rising Threshold............................... 0 %
##&nbsp;Rising Interval................................ 0 secs
##&nbsp;Falling Threshold.............................. 0 %
##&nbsp;Falling Interval............................... 0 secs
##&nbsp;
##&nbsp;CPU Free Memory Monitoring Threshold........... 0 KB
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cpu_utilization_monitoring_falling_threshold' => '0 %',
##&nbsp;            'cpu_utilization_monitoring_rising_interval' => '0 secs',
##&nbsp;            'cpu_utilization_monitoring_falling_interval' => '0 secs',
##&nbsp;            'cpu_free_memory_monitoring_threshold' => '0 KB',
##&nbsp;            'cpu_utilization_monitoring_rising_threshold' => '0 %'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_process_cpu {
    my %args = @_;
    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my @result_array;
    my ( $mem_util, @fields, @time_fields );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /(.*\w+)\)?\.+\s+(.*)$/ ) {
            my $temp1 = $1;
            my $temp2 = $2;
            $temp1 =~ s/-/_/;
            my $var = to_key_name( item => $temp1 );
            if ( $lines[$i] !~ /CPU Free Memory Monitoring/ ) {
                $var = 'cpu_utilization_monitoring_' . $var;
            }
            $result_array[0]->{$var} = $temp2;
        }
        if ( $lines[$i] =~ /(Memory Utilization Report)/ ) {
            @fields = split( /\s+/, $lines[ $i + 1 ] );
            $mem_util = to_key_name( item => $1 );
        }
        if ( $lines[$i] =~ /^free\s+([0-9]+)$/ ) {
            $result_array[0]->{$mem_util}->[0]->{"free_in_$fields[1]"} = "$1";
        }
        if ( $lines[$i] =~ /^alloc\s+([0-9]+)$/ ) {
            $result_array[0]->{$mem_util}->[0]->{"alloc_in_$fields[1]"}
                = "$1";
        }
        my ( $cpu_util, @all_fields, $pos );
        if ( $lines[$i] =~ /(CPU Utilization):/ ) {
            $cpu_util = to_key_name( item => $1 );
            $pos = 0;

            while ( $i <= $#lines ) {
                if ( $lines[$i] =~ /PID\s+Name\s+/ ) {
                    @all_fields  = split( /\s\s+/, $lines[$i] );
                    @time_fields = split( /\s\s+/, $' );
                }
                if ( $lines[$i] =~ /^[0-9]/ ) {
                    my %temp;
                    @fields = split( /\s+/, $lines[$i] );
                    for ( my $j = 0; $j <= $#all_fields; $j++ ) {
                        $temp{ to_key_name( item => $all_fields[$j] ) }
                            = $fields[$j];
                    }
                    $result_array[0]->{$cpu_util}->[$pos] = \%temp;
                    $pos++;
                }
                if ( $lines[$i]
                    =~ /(Total CPU Utilization)\s+(\d.*%)\s+(\d.*%)\s+(\d.*%)/
                    ) {
                    my %temp;
                    $cpu_util = to_key_name( item => $1 );
                    my @details = ( $2, $3, $4 );
                    for ( my $k = 0; $k <= $#time_fields; $k++ ) {
                        $temp{ to_key_name( item => $time_fields[$k] ) }
                            = $details[$k];
                    }
                    $result_array[0]->{$cpu_util}->[0] = \%temp;
                }
                $i++;
            }
        }
    }
    return \@result_array;
} ## end sub show_process_cpu

## @name ::show_isdp
##
## @summary Parses the raw output from the "show isdp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_isdp with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_isdp(%args);
## INPUTS TO METHOD CALL 'show_isdp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show isdp
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Timer.......................................... 5
##&nbsp;Hold Time...................................... 180
##&nbsp;Version 2 Advertisements....................... Enabled
##&nbsp;Neighbors table time since last change......... 39 days 14:29:10
##&nbsp;Device ID...................................... net-1960-1
##&nbsp;Device ID format capability.................... Serial Number, Host Name
##&nbsp;Device ID format............................... Host Name
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_id_format_capability' => 'Serial Number, Host Name',
##&nbsp;            'device_id' => 'net-1960-1',
##&nbsp;            'device_id_format' => 'Host Name',
##&nbsp;            'version_2_advertisements' => 'Enabled',
##&nbsp;            'neighbors_table_time_since_last_change' => '39 days 14:29:10',
##&nbsp;            'timer' => '5',
##&nbsp;            'hold_time' => '180'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_isdp {
    my %args = @_;
    my ( $error_message, @result_array );

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /(.*\w+)\)?\.+\s+(.*)$/ ) {
            my $temp1 = $1;
            $temp1 =~ s/-/_/;
            $result_array[0]->{ to_key_name( item => $temp1 ) } = $2;
        }
    }
    return \@result_array;
} ## end sub show_isdp

## @name ::show_isdp_neighbors
##
## @summary Parses the raw output from the "show isdp neighbors" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_isdp_neighbors with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_isdp_neighbors(%args);
## INPUTS TO METHOD CALL 'show_isdp_neighbors' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'detail' </td> <td> 'detail' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show isdp neighbors
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge,
##&nbsp;                  S - Switch, H - Host, I - IGMP, r - Repeater
##&nbsp;Device ID              Intf         Holdtime  Capability   Platform             Port ID
##&nbsp;---------------------- ------------ --------- ------------ -------------------- ------------
##&nbsp;net-vespa-5a           0/1          140       H            FAS2240-2            e0M
##&nbsp;net-spec-8a            0/2          138       H            FAS3140              e0M
##&nbsp;net-absm-1a            0/3          137       H            FAS6280              e0M
##&nbsp;net-car-1a             0/4          132       H            FAS3210              e0M
##&nbsp;f6-edge-sw1.lab.netapp 0/14         166       S I          cisco WS-C4948-10GE  GigabitEther
##&nbsp;net-1961-2             0/15         179       S            CN1601               0/15
##&nbsp;net-1961-2             0/16         179       S            CN1601               0/16
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_id' => 'net-vespa-5a',
##&nbsp;            'holdtime' => '140',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'interface' => '0/1',
##&nbsp;            'platform' => 'FAS2240-2',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'net-spec-8a',
##&nbsp;            'holdtime' => '138',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'interface' => '0/2',
##&nbsp;            'platform' => 'FAS3140',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'net-absm-1a',
##&nbsp;            'holdtime' => '137',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'interface' => '0/3',
##&nbsp;            'platform' => 'FAS6280',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'net-car-1a',
##&nbsp;            'holdtime' => '132',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'interface' => '0/4',
##&nbsp;            'platform' => 'FAS3210',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'f6-edge-sw1.lab.netapp',
##&nbsp;            'holdtime' => '166',
##&nbsp;            'port_id' => 'GigabitEther',
##&nbsp;            'interface' => '0/14',
##&nbsp;            'platform' => 'cisco_w-C4948-10GE',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch',
##&nbsp;                              'IGMP'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'net-1961-2',
##&nbsp;            'holdtime' => '179',
##&nbsp;            'port_id' => '0/15',
##&nbsp;            'interface' => '0/15',
##&nbsp;            'platform' => 'CN1601',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch'
##&nbsp;                            ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'device_id' => 'net-1961-2',
##&nbsp;            'holdtime' => '179',
##&nbsp;            'port_id' => '0/16',
##&nbsp;            'interface' => '0/16',
##&nbsp;            'platform' => 'CN1601',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch'
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show isdp neighbors detail
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Device ID                       net-vespa-5a
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.227
##&nbsp;Capability                      Host
##&nbsp;Platform                        FAS2240-2
##&nbsp;Interface                       0/1
##&nbsp;Port ID                         e0M
##&nbsp;Holdtime                        140
##&nbsp;Advertisement Version           1
##&nbsp;Time since entry last changed   55 days 12:31:57
##&nbsp;Version :
##&nbsp;NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012
##&nbsp;
##&nbsp;Device ID                       net-spec-8a
##&nbsp;Address(es):
##&nbsp;Capability                      Host
##&nbsp;Platform                        FAS3140
##&nbsp;Interface                       0/2
##&nbsp;Port ID                         e0M
##&nbsp;Holdtime                        138
##&nbsp;Advertisement Version           1
##&nbsp;Time since entry last changed   55 days 12:31:54
##&nbsp;Version :
##&nbsp;NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012
##&nbsp;
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Device ID                       net-absm-1a
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.54
##&nbsp;Capability                      Host
##&nbsp;Platform                        FAS6280
##&nbsp;Interface                       0/3
##&nbsp;Port ID                         e0M
##&nbsp;Holdtime                        137
##&nbsp;Advertisement Version           1
##&nbsp;Time since entry last changed   55 days 12:31:53
##&nbsp;Version :
##&nbsp;NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012
##&nbsp;
##&nbsp;Device ID                       net-car-1a
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.23
##&nbsp;Capability                      Host
##&nbsp;Platform                        FAS3210
##&nbsp;Interface                       0/4
##&nbsp;Port ID                         e0M
##&nbsp;Holdtime                        132
##&nbsp;Advertisement Version           1
##&nbsp;Time since entry last changed   55 days 12:31:49
##&nbsp;Version :
##&nbsp;NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012
##&nbsp;
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Device ID                       f6-edge-sw1.lab.netapp.com
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.19.68.105
##&nbsp;Capability                      Switch IGMP
##&nbsp;Platform                        cisco WS-C4948-10GE
##&nbsp;Interface                       0/14
##&nbsp;Port ID                         GigabitEthernet1/27
##&nbsp;Holdtime                        165
##&nbsp;Advertisement Version           2
##&nbsp;Time since entry last changed   55 days 12:32:23
##&nbsp;Version :
##&nbsp;Cisco IOS Software, Catalyst 4500 L3 Switch Software (cat4500-IPBASE-M), Version 12.2(31)SGA4, RELEASE SOFTWARE (fc1)
##&nbsp;Technical Support: http://www.cisco.com/techsupport
##&nbsp;Copyright (c) 1986-2007 by Cisco Systems, Inc.
##&nbsp;Compiled Thu 25-Oct-07 16:15 by hqluong
##&nbsp;
##&nbsp;Device ID                       net-1961-2
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.92
##&nbsp;Capability                      Switch
##&nbsp;Platform                        CN1601
##&nbsp;Interface                       0/15
##&nbsp;Port ID                         0/15
##&nbsp;Holdtime                        178
##&nbsp;Advertisement Version           2
##&nbsp;Time since entry last changed   55 days 12:32:36
##&nbsp;Version :
##&nbsp;1.0.0.4
##&nbsp;
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Device ID                       net-1961-2
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.92
##&nbsp;Capability                      Switch
##&nbsp;Platform                        CN1601
##&nbsp;Interface                       0/16
##&nbsp;Port ID                         0/16
##&nbsp;Holdtime                        178
##&nbsp;Advertisement Version           2
##&nbsp;Time since entry last changed   55 days 12:32:36
##&nbsp;Version :
##&nbsp;1.0.0.4
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'holdtime' => '140',
##&nbsp;            'version' => 'NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012',
##&nbsp;            'advertisement_version' => '1',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:31:57',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-vespa-5a',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.227'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/1',
##&nbsp;            'platform' => 'FAS2240-2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'holdtime' => '138',
##&nbsp;            'version' => 'NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012',
##&nbsp;            'advertisement_version' => '1',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:31:54',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-spec-8a',
##&nbsp;            'interface' => '0/2',
##&nbsp;            'platform' => 'FAS3140'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'holdtime' => '137',
##&nbsp;            'version' => 'NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012',
##&nbsp;            'advertisement_version' => '1',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:31:53',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-absm-1a',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.54'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/3',
##&nbsp;            'platform' => 'FAS6280'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'holdtime' => '132',
##&nbsp;            'version' => 'NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012',
##&nbsp;            'advertisement_version' => '1',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:31:49',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-car-1a',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.23'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/4',
##&nbsp;            'platform' => 'FAS3210'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'technical_support' => 'http://www.cisco.com/techsupport',
##&nbsp;            'holdtime' => '165',
##&nbsp;            'version' => 'Cisco IOS Software, Catalyst 4500 L3 Switch Software (cat4500-IPBASE-M), Version 12.2(31)SGA4, RELEASE SOFTWARE (fc1)',
##&nbsp;            'advertisement_version' => '2',
##&nbsp;            'port_id' => 'GigabitEthernet1/27',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:32:23',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch',
##&nbsp;                              'IGMP'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'f6-edge-sw1.lab.netapp.com',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.19.68.105'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/14',
##&nbsp;            'platform' => 'cisco WS-C4948-10GE'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'holdtime' => '178',
##&nbsp;            'version' => '1.0.0.4',
##&nbsp;            'advertisement_version' => '2',
##&nbsp;            'port_id' => '0/15',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:32:36',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-1961-2',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.92'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/15',
##&nbsp;            'platform' => 'CN1601'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'holdtime' => '178',
##&nbsp;            'version' => '1.0.0.4',
##&nbsp;            'advertisement_version' => '2',
##&nbsp;            'port_id' => '0/16',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:32:36',
##&nbsp;            'capability' => [
##&nbsp;                              'Switch'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-1961-2',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.92'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/16',
##&nbsp;            'platform' => 'CN1601'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show isdp neighbors 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Capability Codes: R - Router, T - Trans Bridge, B - Source Route Bridge,
##&nbsp;                  S - Switch, H - Host, I - IGMP, r - Repeater
##&nbsp;Device ID              Intf         Holdtime  Capability   Platform             Port ID
##&nbsp;---------------------- ------------ --------- ------------ -------------------- ------------
##&nbsp;net-vespa-5a           0/1          139       H            FAS2240-2            e0M
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'device_id' => 'net-vespa-5a',
##&nbsp;            'holdtime' => '139',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'interface' => '0/1',
##&nbsp;            'platform' => 'FAS2240-2',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show isdp neighbors 0/1 detail
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Device ID                       net-vespa-5a
##&nbsp;Address(es):
##&nbsp;    IP Address:                 172.22.200.227
##&nbsp;Capability                      Host
##&nbsp;Platform                        FAS2240-2
##&nbsp;Interface                       0/1
##&nbsp;Port ID                         e0M
##&nbsp;Holdtime                        138
##&nbsp;Advertisement Version           1
##&nbsp;Time since entry last changed   55 days 12:31:57
##&nbsp;Version :
##&nbsp;NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'holdtime' => '138',
##&nbsp;            'version' => 'NetApp Release mainN_120531_0030 Cluster-Mode: Thu May 31 00:58:46 PDT 2012',
##&nbsp;            'advertisement_version' => '1',
##&nbsp;            'port_id' => 'e0M',
##&nbsp;            'time_since_entry_last_changed' => '55 days 12:31:57',
##&nbsp;            'capability' => [
##&nbsp;                              'Host'
##&nbsp;                            ],
##&nbsp;            'device_id' => 'net-vespa-5a',
##&nbsp;            'addresses' => [
##&nbsp;                             '172.22.200.227'
##&nbsp;                           ],
##&nbsp;            'interface' => '0/1',
##&nbsp;            'platform' => 'FAS2240-2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_isdp_neighbors {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my ( @head, @values, %capab_codes, @codes );
    my $count = 0;
    my $pos   = -1;
    my %hash;
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /Capability Codes: / ) {
            my @temp = split( /,\s+/, $' );
            push( @codes, @temp );
            @temp = split( /,\s+/, $lines[ $i + 1 ] );
            push( @codes, @temp );
            foreach my $cod (@codes) {
                @temp = split( /\s+-\s+/, $cod );
                $capab_codes{ $temp[0] } = $temp[1];
            }
            $i = $i + 2;
        }
        if ( $lines[$i] =~ /Device ID\s+Intf/ ) {
            $lines[$i] =~ s/(\w+)\s+ID/$1_id/g;
            $lines[$i] =~ s/Intf/interface/g;
            @head = split( /\s+/, $lines[$i] );
            $i++;
        }
        if ( $lines[$i] =~ /\w+\s+\d\/\d+\s+/ ) {
	    $lines[$i] =~ s/cisco\s+/cisco_/g;
            $lines[$i] =~ s/([R|T|B|S|H|I|r])(\s+){1,7}/$1_/g;
            my @capabs;
            if ( $lines[$i] =~ /(([R|T|B|S|H|I|r]_){1,7})/ ) {
                @capabs = split( /_/, $1 );
                $lines[$i] =~ s/(([R|T|B|S|H|I|r]_){1,7})/-- /g;
            }

            @values = split( /\s+/, $lines[$i] );
	    for ( my $j = 0; $j <= $#head; $j++ ) {
		    if($values[$j]=~/cisco/)
		    {
			    $values[$j] =~ s/_/ /;
		    }
		    $result_array[$count]->{ to_key_name( item => $head[$j] ) }
		    = $values[$j]
			    if ( $head[$j] ne "Capability" );
	    }
            for ( my $k = 0; $k <= $#capabs; $k++ ) {
                foreach my $key ( keys(%capab_codes) ) {
                    if ( $capabs[$k] eq $key ) {
                        $capabs[$k] = $capab_codes{$key};
                    }
                }
            }
            push( @{ $result_array[$count]->{capability} }, @capabs );
            $count++;
        }
        if ( $command =~ /detail/ ) {
            if ( $lines[$i] =~ /:\s+/ ) {
                @head = split( /:\s+/, $lines[$i] );
            } else {
                @head = split( /\s\s+/, $lines[$i] );
            }
            $pos++ if ( $head[0] =~ /Device ID/ );
            if ( $lines[$i] =~ /Version :/ ) {
                $result_array[$pos]->{version} = $lines[ $i + 1 ];
                $i++;
            }
            my $c = 0;
            if (   $lines[$i] =~ /Address\(es\)/
                && $lines[ $i + 1 ] =~ /IP Address/ ) {
                $i++;
                while ( $lines[$i] =~ /IP Address:\s+(.*)/ ) {
                    $i++;
                    $result_array[$pos]->{addresses}->[$c] = $1;
                    $c++;
                }
                $i--;
            }
            my @cap;
            if ( $lines[$i] =~ /Capability/ ) {
                @cap = split( /\s+/, $head[1] );
            }
            chomp( $head[0] );
            $result_array[$pos]->{ to_key_name( item => $head[0] ) }
                = $head[1]
                if ( $head[1] && $head[0] ne "Capability" );
            push( @{ $result_array[$pos]->{capability} }, @cap );
        }
    }
    return \@result_array;
} ## end sub show_isdp_neighbors

## @name ::show_tech_support
##
## @summary Parses the raw output from the "show tech-support" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_tech_support with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_tech_support(%args);
## INPUTS TO METHOD CALL 'show_tech_support' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show tech-support
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; ***************** Show Version ******************
##&nbsp;
##&nbsp;Switch: 1
##&nbsp;
##&nbsp;System Description............................. NetApp CN1601, 1.0.0.4, Linux
##&nbsp;                                                2.6.21.7
##&nbsp;Machine Type................................... NetApp CN1601
##&nbsp;Machine Model.................................. CN1601
##&nbsp;Serial Number.................................. 10610100001
##&nbsp;FRU Number..................................... 111-00894
##&nbsp;Part Number.................................... BCM53716
##&nbsp;Maintenance Level.............................. A
##&nbsp;Manufacturer................................... 0xbc00
##&nbsp;Burned In MAC Address.......................... 00:A0:98:4B:8A:40
##&nbsp;Software Version............................... 1.0.0.4
##&nbsp;Operating System............................... Linux 2.6.21.7
##&nbsp;Network Processing Device...................... BCM53716_A2
##&nbsp;Additional Packages............................ FASTPATH QOS
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp; ***************** Show SysInfo ******************
##&nbsp;
##&nbsp;
##&nbsp; ***************** Show Running Config ***********
##&nbsp;!Current Configuration:
##&nbsp;!
##&nbsp;!System Description "NetApp CN1601, 1.0.0.4, Linux 2.6.21.7"
##&nbsp;!System Software Version "1.0.0.4"
##&nbsp;!System Up Time          "9 days 2 hrs 17 mins 50 secs"
##&nbsp;!Additional Packages     FASTPATH QOS
##&nbsp;!Current SNTP Synchronized Time: Apr 17 10:38:24 201 UTC
##&nbsp;!
##&nbsp;set prompt "net-1961-1"
##&nbsp;telnetcon timeout 60
##&nbsp;enable password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 encrypted
##&nbsp;network parms 172.22.200.91 255.255.254.0 172.22.200.1
##&nbsp;vlan database
##&nbsp;vlan 600
##&nbsp;vlan name 600 "mgmt"
##&nbsp;exit
##&nbsp;network mgmt_vlan 600
##&nbsp;ip ssh server enable
##&nbsp;sshcon timeout 60
##&nbsp;configure
##&nbsp;sntp client mode unicast
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;! sntp server status is Success
##&nbsp;sntp server "172.19.2.16"
##&nbsp;clock timezone -8 minutes 0 zone PST
##&nbsp;ip domain name "lab.netapp.com"
##&nbsp;ip name server 172.22.201.46
##&nbsp;username "admin" password 54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991 level 15 encrypted
##&nbsp;line console
##&nbsp;serial timeout 60
##&nbsp;exit
##&nbsp;line telnet
##&nbsp;exit
##&nbsp;line ssh
##&nbsp;exit
##&nbsp;spanning-tree configuration name "00-A0-98-4B-8A-40"
##&nbsp;interface 0/15
##&nbsp;addport 1/1
##&nbsp;exit
##&nbsp;interface 0/16
##&nbsp;addport 1/1
##&nbsp;exit
##&nbsp;snmp-server sysname "net-1961-1"
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;snmp-server community "netapp"
##&nbsp;snmp-server community "cshm1"
##&nbsp;snmp-server community "cshm1!"
##&nbsp;interface 0/1
##&nbsp;storm-control broadcast level 3
##&nbsp;storm-control multicast level 3
##&nbsp;storm-control unicast level 3
##&nbsp;description 'Cluster Node 1'
##&nbsp;spanning-tree edgeport
##&nbsp;mtu 1800
##&nbsp;exit
##&nbsp;interface 0/2
##&nbsp;description 'Cluster Node 2'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/3
##&nbsp;description 'Cluster Node 3'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/4
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;description 'Cluster Node 4'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/5
##&nbsp;description 'Cluster Node 5'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/6
##&nbsp;description 'Cluster Node 6'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/7
##&nbsp;description 'Cluster Node 7'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/8
##&nbsp;description 'Cluster Node  8'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/9
##&nbsp;description 'cluster Node 9'
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/10
##&nbsp;description 'Cluster Node 10'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/11
##&nbsp;description 'Cluster Node 11'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/12
##&nbsp;description 'Cluster Node 12'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/13
##&nbsp;description 'Cluster Switch'
##&nbsp;spanning-tree edgeport
##&nbsp;exit
##&nbsp;interface 0/14
##&nbsp;description 'Network uplink'
##&nbsp;no spanning-tree port mode
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;vlan pvid 600
##&nbsp;vlan participation exclude 1
##&nbsp;vlan participation include 600
##&nbsp;exit
##&nbsp;interface 0/15
##&nbsp;description 'ISL member port 15'
##&nbsp;exit
##&nbsp;interface 0/16
##&nbsp;description 'ISL member port 16'
##&nbsp;exit
##&nbsp;interface 1/1
##&nbsp;description 'ISL LAG - ports 15-16'
##&nbsp;port-channel load-balance 6
##&nbsp;exit
##&nbsp;interface 1/6
##&nbsp;description 'NetApp CN1601 Mgmt. Switch RCF v1.0 (2012-01-27)'
##&nbsp;exit
##&nbsp;isdp holdtime 10
##&nbsp;isdp timer 10
##&nbsp;exit
##&nbsp;terminal length 0
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;
##&nbsp;Console Break-in Status........................ Enable
##&nbsp;
##&nbsp;--More-- or (q)uit
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'show_running_config_info' => [
##&nbsp;                                            {
##&nbsp;                                              'ip_ssh_server' => 'enable',
##&nbsp;                                              'spanning_tree_configuration' => '00-A0-98-4B-8A-40',
##&nbsp;                                              'snmp_server_community' => 'cshm1!',
##&nbsp;                                              'snmp_server_sysname' => 'net-1961-1',
##&nbsp;                                              'lines' => [
##&nbsp;                                                           {
##&nbsp;                                                             'serial_timeout' => '60',
##&nbsp;                                                             'line' => 'console'
##&nbsp;                                                           },
##&nbsp;                                                           {
##&nbsp;                                                             'line' => 'telnet'
##&nbsp;                                                           },
##&nbsp;                                                           {
##&nbsp;                                                             'line' => 'ssh'
##&nbsp;                                                           }
##&nbsp;                                                         ],
##&nbsp;                                              'current_sntp_synchronized_time' => 'Apr 17 10:38:24 201 UTC',
##&nbsp;                                              'password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;                                              'interfaces' => [
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/15',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'addport 1/1',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/16',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'addport 1/1',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/1',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'storm-control broadcast level 3',
##&nbsp;                                                                                        'storm-control multicast level 3',
##&nbsp;                                                                                        'storm-control unicast level 3',
##&nbsp;                                                                                        'description \'Cluster Node 1\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'mtu 1800',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/2',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 2\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/3',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 3\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/4',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 4\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/5',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 5\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/6',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 6\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/7',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 7\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/8',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node  8\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/9',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'cluster Node 9\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/10',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 10\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/11',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 11\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/12',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Node 12\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/13',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Cluster Switch\'',
##&nbsp;                                                                                        'spanning-tree edgeport',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/14',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'Network uplink\'',
##&nbsp;                                                                                        'no spanning-tree port mode',
##&nbsp;                                                                                        'vlan pvid 600',
##&nbsp;                                                                                        'vlan participation exclude 1',
##&nbsp;                                                                                        'vlan participation include 600',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/15',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'ISL member port 15\'',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '0/16',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'ISL member port 16\'',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '1/1',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'ISL LAG - ports 15-16\'',
##&nbsp;                                                                                        'port-channel load-balance 6',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                },
##&nbsp;                                                                {
##&nbsp;                                                                  'interface' => '1/6',
##&nbsp;                                                                  'running_config' => [
##&nbsp;                                                                                        'description \'NetApp CN1601 Mgmt. Switch RCF v1.0 (2012-01-27)\'',
##&nbsp;                                                                                        'exit'
##&nbsp;                                                                                      ]
##&nbsp;                                                                }
##&nbsp;                                                              ],
##&nbsp;                                              'rcf_version' => 'v1.0',
##&nbsp;                                              'sntp_server' => '172.19.2.16',
##&nbsp;                                              'additional_packages' => [
##&nbsp;                                                                         'FASTPATH',
##&nbsp;                                                                         'QOS'
##&nbsp;                                                                       ],
##&nbsp;                                              'sshcon_timeout' => '60',
##&nbsp;                                              'system_up_time' => '9 days 2 hrs 17 mins 50 secs',
##&nbsp;                                              'set_prompt' => 'net-1961-1',
##&nbsp;                                              'enable_password' => '54efa5bc56b147ac920e6a5cbba5dfe76bd81744dd12ffe9141907f7a6f87a9db9fa55b14573ebe19c8654d0b98a1c393c5635b5ced4d40d1e16f5cdd6e3a991',
##&nbsp;                                              'username' => 'admin',
##&nbsp;                                              'system_description' => 'NetApp CN1601, 1.0.0.4, Linux 2.6.21.7',
##&nbsp;                                              'system_software_version' => '1.0.0.4',
##&nbsp;                                              'sntp_client_mode' => 'unicast',
##&nbsp;                                              'level' => '15',
##&nbsp;                                              'ip_name_server' => '172.22.201.46',
##&nbsp;                                              'ip_domain_name' => 'lab.netapp.com',
##&nbsp;                                              'sntp_server_status' => 'Success',
##&nbsp;                                              'isdp_timer' => '10',
##&nbsp;                                              'telnetcon_timeout' => '60',
##&nbsp;                                              'network_parms' => [
##&nbsp;                                                                   '172.22.200.91',
##&nbsp;                                                                   '255.255.254.0',
##&nbsp;                                                                   '172.22.200.1'
##&nbsp;                                                                 ]
##&nbsp;                                            }
##&nbsp;                                          ],
##&nbsp;            'show_version_info' => [
##&nbsp;                                     {
##&nbsp;                                       'serial_number' => '10610100001',
##&nbsp;                                       'maintenance_level' => 'A',
##&nbsp;                                       'operating_system' => 'Linux 2.6.21.7',
##&nbsp;                                       'network_processing_device' => 'BCM53716_A2',
##&nbsp;                                       'machine_type' => 'NetApp CN1601',
##&nbsp;                                       'system_description' => 'NetApp CN1601, 1.0.0.4, Linux 2.6.21.7',
##&nbsp;                                       'part_number' => 'BCM53716',
##&nbsp;                                       'software_version' => '1.0.0.4',
##&nbsp;                                       'burned_in_mac_address' => '00:A0:98:4B:8A:40',
##&nbsp;                                       'manufacturer' => '0xbc00',
##&nbsp;                                       'additional_packages' => [
##&nbsp;                                                                  'FASTPATH QOS'
##&nbsp;                                                                ],
##&nbsp;                                       'fru_number' => '111-00894',
##&nbsp;                                       'machine_model' => 'CN1601'
##&nbsp;                                     }
##&nbsp;                                   ],
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_tech_support {
    my %args = @_;
    my ( $error_message, @result_array, $result );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $line;
    my %hash;
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $output, response_object => $resp, first_line => 'Switch:\s+\d+' );
    shift(@lines) if ( $lines[0] =~ /Switch:\s+\d+/ );
    while ($line = shift(@lines)) {
        next if ( $line =~ /--More--\s+or\s+\(q\)uit/ );
        last if ($line =~ /Show SysInfo/);
            if ( $line =~ /^Additional Packages/ ) {
                my @values;
                my ( $key, $value ) = ( $line =~ /^(.*\w)\.\.+\s*(.*)/ );
                push( @values, $value );
                if( $lines[0] =~ /^(\w+.*\w)/) {
                    $line = shift(@lines);
                    push( @values, $line );
                }
                $hash{ to_key_name( item => $key ) } = \@values;
            } else {
                my ( $key, $value ) = ( $line =~ /^(.*\w)\.\.+\s*(.*)/ );
                if( $lines[0] =~ /^(\d+\.\d+\.*\d+)/) {
                    $line = shift(@lines);
                    $hash{ to_key_name( item => $key ) } = join(' ',$value,$line);
                } else{
		            $hash{ to_key_name( item => $key ) } = $value;
                }
            }
        }
        $result = show_running_config(@_);
        push (@{$result_array[0]->{'show_version_info'}}, \%hash);
        $result_array[0]->{'show_running_config_info'}= $result;
        return \@result_array;;
    }

sub show_snmpcommunity {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my  @headings = ("SNMP Community Name","Client IP Address","Client IP Mask","Access Mode","Status");
    my (@result_array,$newkey,$key);
    my @lines = get_lines( raw_output => $output );
    my $j = 0;
    for ( my $k = 2; $k <= $#lines; $k++ ) {
        my @values = split( /\s\s+/, $lines[$k] );
        for ( my $i = 0; $i < scalar(@headings); $i++ ) {
            if ( $values[$i] ) {
                $result_array[$j]->{ to_key_name(item => $headings[$i]) } = $values[$i];
            }
        }
        $j++;
    }
    return \@result_array;
}

1;
