#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/default/ping.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary :  ping command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch ping command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::default::ping
##
##  ping(%args)
##
## @author sangavainetapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::default::ping;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ($raw_output =~ /(DNS\s+Lookup\s+failed|Invalid\s+input\s+detected)/i )
    {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /Invalid IPv6 global address/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Incomplete command/ ) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::ping
##
## @summary Parses the raw output from the "ping" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ping with any of its options as described in the
##    corresponding cdef file 'ping.cdefs'
## @synopsis
##      ping(%args);
## INPUTS TO METHOD CALL 'ping' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'ipv6' </td> <td> 'ipv6' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ping 127.0.0.1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Pinging 127.0.0.1 with 0 bytes of data:
##&nbsp;
##&nbsp;Reply From 127.0.0.1: icmp_seq = 0. time= 2692 usec.
##&nbsp;
##&nbsp;----127.0.0.1 PING statistics----
##&nbsp;1 packets transmitted, 1 packets received, 0% packet loss
##&nbsp;round-trip (msec) min/avg/max = 2/2/2
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'time' => '2692 usec',
##&nbsp;            'icmp_seq' => '0',
##&nbsp;            'packets_received' => '1',
##&nbsp;            'avg_round_trip' => '2 msec',
##&nbsp;            'hostname' => '127.0.0.1',
##&nbsp;            'min_round_trip' => '2 msec',
##&nbsp;            'bytes_of_data' => '0',
##&nbsp;            'max_round_trip' => '2 msec',
##&nbsp;            'packet_loss' => '0%',
##&nbsp;            'packets_transmitted' => '1',
##&nbsp;            'ipaddress' => '127.0.0.1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = ping ipv6 ::1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; Pinging ::1 with 64 bytes of data:
##&nbsp;
##&nbsp;
##&nbsp;Send count=3, Receive count=3 from ::1
##&nbsp;Average round trip time = 1.00 ms
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bytes_of_data' => '64',
##&nbsp;            'send_count' => '3',
##&nbsp;            'avg_round_trip' => '1.00 ms',
##&nbsp;            'receive_count' => '3',
##&nbsp;            'hostname' => '::1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ping {
    my %args = @_;

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @result_array;
    my @lines
        = get_lines( 'raw_output' => $output, response_object => $resp );

    _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    foreach my $line (@lines) {
        if ( $line =~ /Pinging (.*) with (\d+) bytes of data/ ) {
            $result_array[0]->{hostname}      = $1;
            $result_array[0]->{bytes_of_data} = $2;
        }
        if ($line =~ /Reply From (.*): icmp_seq = (\d+). time= (\d+\s+\w+)./ )
        {
            $result_array[0]->{ipaddress} = $1;
            $result_array[0]->{icmp_seq}  = $2;
            $result_array[0]->{'time'}    = $3;
        }
        if ( $line
            =~ /(\d+)\s+packets\s+transmitted,\s+(\d+)\s+packets\s+received,\s+(\d+%)\s+packet\s+loss/
            ) {
            $result_array[0]->{packets_transmitted} = $1;
            $result_array[0]->{packets_received}    = $2;
            $result_array[0]->{packet_loss}         = $3;
        }
        if ( $line =~ /round-trip \((\w+)\) min\/avg\/max = (.*)/ ) {
            my @values = split( /\//, $2 );
            $result_array[0]->{min_round_trip} = "$values[0] $1";
            $result_array[0]->{avg_round_trip} = "$values[1] $1";
            $result_array[0]->{max_round_trip} = "$values[2] $1";
        }
        if ( $line =~ /Send count=(\d+), Receive count=(\d+) from/ ) {
            $result_array[0]->{send_count}    = $1;
            $result_array[0]->{receive_count} = $2;
        }
        if ( $line =~ /Average round trip time = (.*)/ ) {
            $result_array[0]->{avg_round_trip} = $1;
        }
    }

    return \@result_array;
} ## end sub ping

1;
