#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse QlogicFC show port commands
##
## @description
##    This is a library to parse the raw output of QlogicFC switch
##    show commands
##
## @synopsis
##  use NACL::APISet::Switch::CLI::QlogicFC::default::show
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::QlogicFC::default::show;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message, $error) = undef;

    # Check for the error
    if ($raw_output =~ /Usage:(.*)/i) {
        $error_message = $raw_output;
        $error         = 'InvalidParam';
    } elsif ($raw_output =~ /not a valid value/i){
        $error_message = $raw_output;
        $error         = 'InvalidParamValue';
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    return $error;
} ## end sub _check_error

## @name :: show_port
##
## @summary Parses the raw output from the "show port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from port_show with any of its options as described in the
##    corresponding cdef file 'show.cdefs'
## @synopsis
##      show_port(%args);
## INPUTS TO METHOD CALL 'show_port' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port-num' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = show port
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;> show port
##&nbsp;
##&nbsp;        Admin       Operational  Login       Config Running     Link     Link
##&nbsp;  Port  State       State        Status      Type   Type        State    Speed
##&nbsp;  ----  -----       -----------  ------      ------ -------     -----    -----
##&nbsp;  0     Offline     Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;  1     Online      Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;  2     Online      Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;  3     Online      Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;  4     Online      Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;  5     Online      Offline      NotLoggedIn GL     Unknown     Inactive Auto
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Offline',
##&nbsp;            'port' => '0',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'loginstatus' => 'NotLoggedIn'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Online',
##&nbsp;            'port' => '1',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'loginstatus' => 'NotLoggedIn'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Online',
##&nbsp;            'port' => '2',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'loginstatus' => 'NotLoggedIn'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Online',
##&nbsp;            'port' => '3',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'loginstatus' => 'NotLoggedIn'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Online',
##&nbsp;            'port' => '4',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'loginstatus' => 'NotLoggedIn'
##&nbsp;          },
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = show port 6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;> show port 6
##&nbsp;
##&nbsp;  Port Number: 6
##&nbsp;  ------------
##&nbsp;  AdminState       Online              OperationalState Offline
##&nbsp;  AsicNumber       0                   PerfTuningMode   Normal
##&nbsp;  AsicPort         6                   PortID           7f0600
##&nbsp;  ConfigType       GL                  PortWWN          20:06:00:c0:dd:07:24:80
##&nbsp;  DiagStatus       Passed              RunningType      Unknown
##&nbsp;  DownstreamISL    False               MediaPartNumber  Unknown
##&nbsp;  EpConnState      None                MediaRevision    Unknown
##&nbsp;  EpIsoReason      NotApplicable       MediaType        NotInstalled
##&nbsp;  IOStreamGuard    Disabled            MediaVendor      Unknown
##&nbsp;  Licensed         True                MediaVendorID    Unknown
##&nbsp;  LinkSpeed        Auto                SupportedSpeeds  1, 2, 4Gb/s
##&nbsp;  LinkState        Inactive            SymbolicName     Port6
##&nbsp;  LoginStatus      NotLoggedIn         SyncStatus       SyncLost
##&nbsp;  MaxCredit        16                  UpstreamISL      False
##&nbsp;  MediaSpeeds      Unknown             XmitterEnabled   True
##&nbsp;
##&nbsp;  ALInit             2                   LIP_F8_F7          0
##&nbsp;  ALInitError        0                   LinkFailures       0
##&nbsp;  BadFrames          0                   Login              0
##&nbsp;  BBCR_FrameFailures 0                   Logout             0
##&nbsp;  BBCR_RRDYFailures  0                   LongFramesIn       0
##&nbsp;  Class2FramesIn     0                   LoopTimeouts       0
##&nbsp;  Class2FramesOut    0                   LossOfSync         0
##&nbsp;  Class2WordsIn      0                   LostFrames         0
##&nbsp;  Class2WordsOut     0                   LostRRDYs          0
##&nbsp;  Class3FramesIn     0                   PrimSeqErrors      0
##&nbsp;  Class3FramesOut    0                   RxLinkResets       0
##&nbsp;  Class3Toss         0                   RxOfflineSeq       0
##&nbsp;  Class3WordsIn      0                   ShortFramesIn      0
##&nbsp;  Class3WordsOut     0                   TotalErrors        0
##&nbsp;  DecodeErrors       0                   TotalLinkResets    0
##&nbsp;  EpConnects         0                   TotalLIPsRecvd     0
##&nbsp;  FBusy              0                   TotalLIPsXmitd     3
##&nbsp;  FlowErrors         0                   TotalOfflineSeq    2
##&nbsp;  FReject            0                   TotalRxFrames      0
##&nbsp;  InvalidCRC         0                   TotalRxWords       0
##&nbsp;  InvalidDestAddr    0                   TotalTxFrames      0
##&nbsp;  LIP_AL_PD_AL_PS    0                   TotalTxWords       0
##&nbsp;  LIP_F7_AL_PS       0                   TxLinkResets       0
##&nbsp;  LIP_F7_F7          0                   TxOfflineSeq       2
##&nbsp;  LIP_F8_AL_PS       0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'portwwn' => '20:06:00:c0:dd:07:24:80',
##&nbsp;            'totalerrors' => '0',
##&nbsp;            'maxcredit' => '16',
##&nbsp;            'bbcr_rrdyfailures' => '0',
##&nbsp;            'class2framesout' => '0',
##&nbsp;            'mediarevision' => 'Unknown',
##&nbsp;            'primseqerrors' => '0',
##&nbsp;            'mediavendorid' => 'Unknown',
##&nbsp;            'logout' => '0',
##&nbsp;            'class2framesin' => '0',
##&nbsp;            'longframesin' => '0',
##&nbsp;            'totalrxframes' => '0',
##&nbsp;            'xmitterenabled' => 'True',
##&nbsp;            'perftuningmode' => 'Normal',
##&nbsp;            'flowerrors' => '0',
##&nbsp;            'login' => '0',
##&nbsp;            'iostreamguard' => 'Disabled',
##&nbsp;            'symbolicname' => 'Port6',
##&nbsp;            'totallipsxmitd' => '3',
##&nbsp;            'epconnects' => '0',
##&nbsp;            'linkfailures' => '0',
##&nbsp;            'looptimeouts' => '0',
##&nbsp;            'configtype' => 'GL',
##&nbsp;            'lip_f7_f7' => '0',
##&nbsp;            'invaliddestaddr' => '0',
##&nbsp;            'linkstate' => 'Inactive',
##&nbsp;            'epconnstate' => 'None',
##&nbsp;            'totaltxwords' => '0',
##&nbsp;            'invalidcrc' => '0',
##&nbsp;            'operationalstate' => 'Offline',
##&nbsp;            'totallipsrecvd' => '0',
##&nbsp;            'fbusy' => '0',
##&nbsp;            'totallinkresets' => '0',
##&nbsp;            'class3framesout' => '0',
##&nbsp;            'mediaspeeds' => 'Unknown',
##&nbsp;            'decodeerrors' => '0',
##&nbsp;            'licensed' => 'True',
##&nbsp;            'class3wordsout' => '0',
##&nbsp;            'class2wordsin' => '0',
##&nbsp;            'lip_f8_f7' => '0',
##&nbsp;            'freject' => '0',
##&nbsp;            'class3wordsin' => '0',
##&nbsp;            'mediapartnumber' => 'Unknown',
##&nbsp;            'episoreason' => 'NotApplicable',
##&nbsp;            'lossofsync' => '0',
##&nbsp;            'totaltxframes' => '0',
##&nbsp;            'lostframes' => '0',
##&nbsp;            'class3toss' => '0',
##&nbsp;            'syncstatus' => 'SyncLost',
##&nbsp;            'class3framesin' => '0',
##&nbsp;            'lip_f7_al_ps' => '0',
##&nbsp;            'totalofflineseq' => '2',
##&nbsp;            'linkspeed' => 'Auto',
##&nbsp;            'rxlinkresets' => '0',
##&nbsp;            'txofflineseq' => '2',
##&nbsp;            'lip_al_pd_al_ps' => '0',
##&nbsp;            'mediavendor' => 'Unknown',
##&nbsp;            'upstreamisl' => 'False',
##&nbsp;            'class2wordsout' => '0',
##&nbsp;            'rxofflineseq' => '0',
##&nbsp;            'alinit' => '2',
##&nbsp;            'mediatype' => 'NotInstalled',
##&nbsp;            'supportedspeeds' => '1, 2, 4Gb/s',
##&nbsp;            'shortframesin' => '0',
##&nbsp;            'downstreamisl' => 'False',
##&nbsp;            'lostrrdys' => '0',
##&nbsp;            'badframes' => '0',
##&nbsp;            'runningtype' => 'Unknown',
##&nbsp;            'adminstate' => 'Online',
##&nbsp;            'asicnumber' => '0',
##&nbsp;            'totalrxwords' => '0',
##&nbsp;            'txlinkresets' => '0',
##&nbsp;            'asicport' => '6',
##&nbsp;            'aliniterror' => '0',
##&nbsp;            'port_number' => '6',
##&nbsp;            'bbcr_framefailures' => '0',
##&nbsp;            'loginstatus' => 'NotLoggedIn',
##&nbsp;            'portid' => '7f0600',
##&nbsp;            'diagstatus' => 'Passed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub show_port {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();

    _check_error(raw_output => $output, response_object => $resp);
    my @result_array;
    my @lines = get_lines(raw_output => $output, response_object => $resp);

    if ($cmd =~ /\d+/) {
        #This is required when show port is executed for specific port.
        foreach my $line (@lines) {
            if ($line =~ /(Port\s+Number):\s+(\d+)/) {
                $result_array[0]->{to_key_name(item => $1)} = $2;
            }

            if ($line =~ /(\w+)\s+(\w+)\s+(\w+)\s+(.*)/) {
                $result_array[0]->{to_key_name(item => $1)} = $2;
                $result_array[0]->{to_key_name(item => $3)} = $4;
            }
        }

    } else {
        #This is required when only show port is executed.
        my $count = 0;
        my @head1 = split(/\s+/, $lines[1]);
        my @head2 = split(/\s+/, $lines[2]);
        unshift(@head1, '');
        @lines = grep (!/(----)/, @lines);
        for my $i (3 .. $#lines) {
            my @values = split(/\s+/, $lines[$i]);
            for my $j (0 .. $#head1) {
                if ($j != 0) {
                    $result_array[$count]
                        ->{to_key_name(item => $head1[$j] . $head2[$j])}
                        = $values[$j];
                } else {
                    $result_array[$count]->{to_key_name(item => $head2[$j])}
                        = $values[$j];
                }
            }
            $count++;

        }
    }

    return \@result_array;
} ## end sub show_port

1;
