#
# Copyright (c) 2011-13 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Version class for finding Version attributes specific to Solaris.
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared


package NACL::APISet::Version::Host::Solaris;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports
#
use NACL::APISet::Session::CLI::Linux;
use NACL::APISet::Response::CLI::ParserUtils;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::Exceptions::Argument qw(:try);

#
# base class
#
use base qw(NACL::APISet::Version::Host);

###############################################################################
# Method:    _get_os_revision
# Objective: Will return Operating System revision
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _find_os_revision {
    my ($self) = @_;
    $Log->enter();

    my ($major, $minor, $session);

    try {
        my $get_linux_session = sub {
            return $self->get_session(
                interface  => "CLI",
                subtype    => "Linux",
            );
        };
        $session = $get_linux_session->();
        ($major, $minor) = $self->_get_major_and_minor_revisions($session);

    } finally {
        # Destroy the session
        $self->destroy_session(session => $session) if (defined $session);
    };

    $self->{os_revision}->{major} = $major;
    $self->{os_revision}->{minor} = $minor;

    $Log->exit();
    return $major;
}

################################################################################
# Subroutine Name  _get_solaris_major_and_minor_revisions
# Objective:       Will store major and minor revision in self if OS is SuSe
# Details:         See POD documentation at the end of this file
################################################################################
sub _get_major_and_minor_revisions {
    my ($self, $session) = @_;

    my ($cmd, $major, $minor);
    
    # store client OS revision
    $cmd = "uname -r";
    my $output = $self->execute_CLI_command(
        connection  => $session->{connection},
        command     => $cmd,
    );
    
    my @output = split(/\n/, $output);
    $major = "Solaris" . $output[0];

    # get Solaris 10 minor revision - note this is the update
    # level w/o the 'u' in front.
    if ($major =~ /5\.11|5\.10|5\.9/) {
        $minor = 0;
        # Get major and minor revision
        $cmd = "cat /etc/release";
        $output = $self->execute_CLI_command(
            connection  => $session->{connection},
            command     => $cmd,
        );
    
        @output = split(/\n/, $output);
        foreach my $line (@output) {
            # Looking for something like one of the following lines:
            # Solaris 9 9/05 HW s9s_u9wos_06b SPARC
            # Solaris 10 10/09 s10s_u8wos_08a SPARC
            # Solaris 10 5/09 s10x_u7wos_08 X86
            # Oracle Solaris 10 8/11 s10s_u10wos_17b SPARC
            # Oracle Solaris 10 8/11 s10x_u10wos_15 X86
            # Oracle Solaris 11 11/11 SPARC
            # Oracle Solaris 11 11/11 X86
            # Oracle Solaris 11.1 SPARC
            # Oracle Solaris 11.1 X86
            #
            if ($line =~ /oracle\s+solaris\s+(\d+)\s+(\d+\/\d+)\s+(sparc|x86)/i) {
                $minor = 0;
            } elsif ($line =~ /oracle\s+solaris\s+(\d+)\.(\d+)\s+(sparc|x86)/i) {
                $minor = $2;
            } elsif ($line =~ /solaris\s+(\d+)\s+\S+\s+s.*_u(\d+)/i) {
                $minor = $2;
            } elsif ($line =~ /solaris\s+(\d+)\s+\S+\s+\S+\s+s.*_u(\d+)/i) { 
                # This accounts for the Solaris 9 9/05 HW s9s_u9wos_06b SPARC
                $minor = $2;
            }
        }
    }

    return ($major, $minor);
}

1;


=head1 NAME

NACL::APISet::Version::Host::Solaris - Version library for Solaris

=head1 SYNOPSIS

 use NACL::APISet::Version::Host::Solaris;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Host::Solaris> is used to connect to Solaris Host and
collect version related attributes. All version related attributes exposed from
this libary can also be set or get by the user.

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Version::Host::Solaris>.

=over

=item Synopsis

$version = NACL::APISet::Version::Host::Solaris->new(%args)

=item Arguments

=over

=item category

(Mandatory)
The category to use.  Valid values include "Host".

=item shell_type

(Mandatory)
The refers to the type of the shell host sent through 'hostobj'.
Valid values include "Host"

=item hostobj

(Mandatory)
The hostrec object created for the resource.

=item conn_overrides

(Optional)
Hash reference to the list of connection overrides to be considered when
establishing the connection to the service.
Valid connection overrides are username, password, connid, conntype, ipaddress,
prompt_username and prompt_password.

=back

=item Returns

A reference to a new Solaris Version object

=back

=cut

