# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Version class for SysManager.
## @author neelkant@netapp.com, ng-sysmgr-automation-core@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Version::Host::SysManager;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports: APISet
#
#
use base qw(NACL::APISet::Version);

#
# Exceptions imported
#
use NATE::BaseException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

#
# Base class
#
use base qw(NACL::APISet::Version);

##############################################################################
# Method:    map_release_to_codename
# Objective: Map release strings to internal codenames
# Details:   See POD documentation at the end of this file.
##############################################################################

sub map_release_to_codename {
    $Log->enter() if $may_enter;
    my $self = shift();
    $Log->exit() if $may_exit;
    return "eaglerare";
}

##############################################################################
# Method:    _find_release
# Objective: Perform the discovery of release information
# Details:   See POD documentation at the end of this file.
##############################################################################
sub _find_release {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    my $Host = $self->{'hostobj'};

    my $Api_Set_cliObj = NACL::APISet->new(
        hostobj   => $Host,
        category  => "Node",
        interface => "CLI",
        set       => "CMode",
    );
    $Api_Set_cliObj->set_timeout( 'connectrec-timeout' => 60 );

    my $hostname =
      $Api_Set_cliObj->execute_raw_command( command => "hostname" );
    chomp($hostname);
    $hostname =~ s/\s+$//;
    my $response = $Api_Set_cliObj->network_interface_show(
        vserver => $hostname,
        role    => "cluster-mgmt"
    );
    my @parsed_output = @{ $response->get_parsed_output() };
    my $Cluster_ip = $parsed_output[0]->{'address'};

    my $API_shell_obj = NACL::APISet->new(
        hostobj            => $Host,
        category           => "Node",
        interface          => "CLI",
        set                => "Systemshell",
        ipaddress          => $Cluster_ip,
        context_definition => {
            username => 'diag',
            password => $Host->{default_password},
        }
    );
    $API_shell_obj->set_timeout( 'connectrec-timeout' => "600" );

    $response = $API_shell_obj->execute_raw_command(
        command => "cat /mroot/etc/www/svcs/sysmgr/htdocs/sysmgr-version.txt" );

    ($self->{'release'}) = $response =~ /version=(\S+)/;
}

1;

=head1 NAME

NACL::APISet::Version::Host::SysManager - Version library for SysManager

=head1 SYNOPSIS

 use NACL::APISet::Version::Host::SysManager;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Host::SysManager> is used to connect to a SysManager Host and
collect version related attributes. All version related attributes exposed from
this libary can also be set or get by the user.

=cut

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::ConnectionFailedException

=item NATE::BaseException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Version::Host::SysManager>.

=over

=item Synopsis

$version = NACL::APISet::Version::Host::SysManager->new(%args)

=item Arguments

=over

=item category

(Mandatory)
The category to use.  Valid values include "Host".

=item shell_type

(Mandatory)
The refers to the type of the shell host sent through 'hostobj'.
Valid values include "Windows" and "Linux".

=item hostobj

(Mandatory)
The hostrec object created for the resource.

=item conn_overrides

(Optional)
Hash reference to the list of connection overrides to be considered when
establishing the connection to the service.
Valid connection overrides are username, password, connid, conntype, ipaddress,
prompt_username and prompt_password.

=back

=item Returns

A reference to a SysManager Version object

=back

=cut

=head1 METHODS


=head2 _find_release

This method discovers release information by creating the appropriate session
and execute various commands like "sysctl", "cat /BUILD", "system image show"
to get information about release.

=over

=item Synopsis

$release = $version_object->get_version_attribute(attribute => "release");

The above method will internally call $self->_find_release() if the information
about release atribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the release information in version object itself.

=back


=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id$
 Last revised $Date$ by $Author$

=cut

