#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Version class for Cluster mode.
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Version::Node::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports: APISet
#
use NACL::APISet::Session::CLI::CMode;
use NACL::APISet::Session::CLI::CMode::Shell;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Version::Node::Compatibility;

#
# Exceptions imported
#
use NATE::BaseException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(:all);
use NATE::ParamSet qw(param_global);
use NATE::Util::Appliance;
#
# Base class
#
use base qw(NACL::APISet::Version::Node);
use Class::MethodMaker [
    hash => 'compatibility',
    scalar => [
        {
            '-default_ctor' => sub {
                $Log->enter() if $may_enter;
                my $self = $_[0];
                my $is_promoted;
                try {
                    $is_promoted = $self->_is_promoted_cmode_priv();
                } catch NATE::BaseException with {
                    if ($Log->may_debug()) {
                        my $exception = shift;
                        $Log->debug('Failed to discover whether the build is promoted '
                            . 'or not: '
                            . $exception->text()
                        );
                    }

                    $is_promoted = $self->_is_promoted_systemshell();
                };

                # promoted builds are guaranteed to be non-debug
                $self->is_debug(0) if ($is_promoted);
                $Log->exit($is_promoted);
                return $is_promoted;
            },
        },
        'is_promoted',
    ],
];

##############################################################################
# Method:    has_uichange
# Objective: Verify whether the service has the specified uichange string
# Details:   See POD documentation at the end of this file.
##############################################################################

sub has_uichange {
    $Log->enter() if $may_enter;

    my $self = shift();
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "uichange" => { type => SCALAR } }
    );

    my @uichanges
        = @{ $self->get_version_attribute( attribute => "uichange_strings" )
        };
    my $has_change = 0;
    $has_change = grep( /^$args{uichange}$/, @uichanges );

    $Log->exit() if $may_exit;
    return $has_change;
}

##############################################################################
# Method:    _find_uichange_strings
# Objective: Perform the discovery of uichange strings information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_uichange_strings {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;
    
    # Query the NACL DB to see if the compatibility information is already stored.
    # Speeds this method up because this way the uichanges are obtained
    # from ONTAP only once per ntest run, rather than for
    # every new APISet object.
    require NACL::Database;
    my $nacl_db = NACL::Database->new();
    my $build_path = $self->get_version_attribute(attribute => 'build_path');
    if ($nacl_db) {
        # Database->new() coded to work only with MySQL, so returns nothing
        # for SQLite (hopefully no one is still using SQLite as this point)
        my $uichanges = $nacl_db->_query_uichanges(build_path => $build_path);
        if (defined $uichanges) {
            $self->{uichange_strings} = $uichanges;
            require NACL::ComponentUtils;
            $Log->comment("Obtained uichanges from DB:\n"
                . NACL::ComponentUtils::_dump_one($uichanges));
            $Log->exit() if $may_exit;
            return;
        }
    }

    # Execute the CMode command "system node uichanges".
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.
    try {

        # Use APISet session class to get the connection object.
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode"
        );

        # Find the information
        $self->execute_CLI_command(
            connection => $session->{connection},
            command    => "set -privilege diagnostic"
        );
        my $command = "system node uichanges local";
        my $output  = $self->execute_CLI_command(
            connection => $session->{connection},
            command    => $command
        );
        my $parsed_output = $self->parse_ngsh_output(
            command    => $command,
            raw_output => $output
        );
        if ( defined $parsed_output ) {

            # Output is parsed successfully. Now get the required field entry
            # if present
            $self->{uichange_strings} = $parsed_output->[0]->{uichanges};

            if ($nacl_db) {
                $nacl_db->_insert_table(
                    table_name => 'uichanges',
                    row        => {
                        build_path => $build_path,
                        ds         => $self->{uichange_strings},
                    }
                );
            }
        }
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };
	$Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_release
# Objective: Perform the discovery of release information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_release {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    # Try running the CMode command "system image show".

    my $session;
    try {
        $self->_command_system_image_show();

    }
    catch NATE::BaseException with {

        # Catch any exception so as to try the possibility of running
        # commands from System shell.
        my $exception = shift;
        $Log->debug( "Error in obtaining release from CMode Shell: "
                . $exception->text() );
    };

    if ( defined $self->{release} ) {
        $Log->exit() if $may_exit;
        return;
    }

    # Get the connection. Using session class itself so that it will
    # do prep_service and it helps us to get a connetion object with
    # the connection overrides specified, if any.
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        #Find the information
        $self->SUPER::_find_release( connection => $session->{connection} );
    }
    finally {

        #Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_wafl_version
# Objective: Perform the discovery of wafl version information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_wafl_version {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my ( $session, $output );
    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode"
        );

        # Find the information
        $self->SUPER::_find_wafl_version( session => $session );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_branch
# Objective: Perform the discovery of branch information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_branch {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;

    # Try running the CMode command "system image show".
  
    if (! $self->is_promoted()) {
        try {
            $self->_command_system_image_show();
        } catch NATE::BaseException with {

            # Catch any exception so as to try the possibility of running
            # commands from System shell.
            my $exception = shift;
            $Log->debug( "Error in obtaining branch from CMode Shell: "
                . $exception->text() );
        };

        if ( defined $self->{branch} ) {
            $Log->exit() if $may_exit;
            return;
        }
    }
    # Try getting the information from systemshell
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_branch( connection => $session->{connection} );
    }

    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_modelinfo
# Objective: Perform the discovery of modelinfo information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_modelinfo {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    # Try running the CMode command "system node show".
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    $self->_command_system_node_show();

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_serialnumber
# Objective: Perform the discovery of serialnumber information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_serialnumber {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    # Try running the CMode command "system node show".
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    $self->_command_system_node_show();

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_change_number
# Objective: Perform the discovery of change number information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_change_number {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;

    # Try running the CMode command "system image show".
    if (! $self->is_promoted()) {
        try {
            $self->_command_system_image_show();

        } catch NATE::BaseException with {

            # Catch any exception so as to try the possibility of running
            # commands from System shell.
            my $exception = shift;
            $Log->debug( "Error in obtaining change number from CMode Shell: "
                . $exception->text() );
        };

        if ( defined $self->{change_number} ) {
            $Log->exit() if $may_exit;
            return;
        }
    }
    # Try to get information from the sytemshell.
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_change_number(
            connection => $session->{connection} );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_client
# Objective: Perform the discovery of client information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_client {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $session;
    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_client( connection => $session->{connection} );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_build_options
# Objective: Perform the discovery of build options information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_build_options {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;

    # Try running the CMode command "system image show".

    try {
        $self->_command_system_image_show();

    }
    catch NATE::BaseException with {

        # Catch any exception so as to try the possibility of running
        # commands from System shell.
        my $exception = shift;
        $Log->debug( "Error in obtaining build_options from CMode Shell: "
                . $exception->text() );
    };

    if ( defined $self->{build_options} ) {
        $Log->exit() if $may_exit;
        return;
    }

    # Try to get information from systemshell.
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_build_options(
            connection => $session->{connection} );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_build_path
# Objective: Perform the discovery of build path information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_build_path {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $session;

    # Try running the CMode command "system image show".
    try {
        $self->_command_system_image_show();

    }
    catch NATE::BaseException with {

        # Catch any exception so as to try the possibility of running
        # commands from System shell.
        my $exception = shift;
        $Log->debug( "Error in obtaining build_path from CMode Shell: "
                . $exception->text() );
    };

    if ( defined $self->{build_path} ) {
        $Log->exit() if $may_exit;
        return;
    }
    # Try getting the information from systemshell.
    # We are not catching any exceptions as we had tried all possibilities to
    # obtain the information and we allow it to fail at this point.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_build_path(
            connection => $session->{connection} );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    _find_node_version
# Objective: Perform the discovery of node version information
# Details:   See POD documentation at the end of this file.
##############################################################################

sub _find_node_version {
    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    # Get the connection. Using session class itself so that it will
    # do prep_service
    my $session;

    # Try running the Nodescope command.
    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode"
        );
        my $output = $self->execute_CLI_command(
            connection => $session->{connection},
            command    => "system node run local version"
        );
        $self->{node_version} = $self->_extract_node_version(
            version => trim_string( string => $output ) );
    }
    catch NATE::BaseException with {

        # Catch any exception so as to try the possibility of running
        # commands from System shell.
        my $exception = shift;
        $Log->debug(
            "Failed to get CMode shell session: " . $exception->text() );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    if ( defined $self->{node_version} ) {
        my %ver_opts = (ontap_version=>$self->{node_version});
        NATE::Util::Appliance::set_version(%ver_opts);
        $Log->exit() if $may_exit;
        return;
    }

    # Try getting the info through Systemshell. We are not catching
    # ConnectionFailedException here as there is no other way in which we
    # can obtain the node version information and allow it to fail if we
    # are not able to establish a session here.

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode::Shell",
            connid    => "c-system"
        );

        # Find the information
        $self->SUPER::_find_node_version(
            connection => $session->{connection} );
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };
    
    if (defined $self->{node_version}) {
        my %ver_opts = (ontap_version=>$self->{node_version});
        NATE::Util::Appliance::set_version(%ver_opts);
    }
    $Log->exit() if $may_exit;
}

##############################################################################
# Method:    setup_compatibility
# Objective: To setup the compatibility by building the compatibility hash
# Details:   See POD documentation at the end of this file.
##############################################################################
sub setup_compatibility {
    my $self = shift();

    # Query the NACL DB to see if the compatibility information is already stored.
    # Speeds this method up because this way the uichanges are obtained
    # from ONTAP (and processed) only once per ntest run, rather than for
    # every new APISet object.
    require NACL::Database;
    my $nacl_db = NACL::Database->new();
    my $build_path = $self->get_version_attribute(attribute => 'build_path');
    if ($nacl_db) {
        # Database->new() coded to work only with MySQL, so returns nothing
        # for SQLite (hopefully no one is still using SQLite as this point)
        my $compatibility = $nacl_db->_query_version_compatibility(
            build_path => $build_path);
        if (defined $compatibility) {
            $self->{compatibility} = $compatibility;
            $Log->exit() if $may_exit;
            return;
        }
    }

    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "interface" => { type => SCALAR, optional => 0 }, }
    );

    # Get the list of uichanges
    my $uichanges
        = $self->get_version_attribute( attribute => "uichange_strings" );

    # Invoke compatibility to build the compatibility hash dynamcially for the
    # uichanges present in the service.
    my $method = "build_compatibility_" . lc( $args{interface} );
    my $compatibility = NACL::APISet::Version::Node::Compatibility->$method(
        uichanges => $uichanges );
    $self->{compatibility} = $compatibility;
    if ($nacl_db) {
        # Store into the NACL DB so that subsequent calls are sped up.
        $nacl_db->_insert_table(
            table_name => 'version_compatibility',
            row        => {
                build_path => $build_path,
                ds         => $compatibility,
            },
        );
    }

    $Log->exit() if $may_exit;
}

sub _command_system_image_show {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;
    my $version;
    my %attribute_to_key = (
        'version'    => 'release',
        'build-info' => 'branch',
        'change'     => 'change_number',
        'build-type' => 'build_options',
        'build-path' => 'build_path',
    );

    # Processing necessary for the required attributes

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode"
        );

        # Find the information
	my $priv = $self->is_promoted() ? 'diagnostic' : 'test';
        $self->execute_CLI_command(
            connection => $session->{connection},
            command    => "set -privilege $priv"
        );
        my $command = "system image show local -iscurrent true";
        my $output  = $self->execute_CLI_command(
            connection => $session->{connection},
            command    => $command
        );
        my $parsed_output = $self->parse_ngsh_output(
            command    => $command,
            raw_output => $output
        );
        if ( defined $parsed_output ) {

            # Output is parsed successfully. Now get all the required
            # fields and process them. 
            if (defined $parsed_output->[0]->{'build-info'}) {
                $parsed_output->[0]->{'build-info'} =~ /"\S+:\s+(\S+)/;
                $parsed_output->[0]->{'build-info'} = $1;
                $parsed_output->[0]->{'build-info'} =~ s/DOT\///;   
            }
            # In some build, release information has got extra suffixes and
            # prefixes. Burt368313
            if (defined $parsed_output->[0]->{version}) {            
                $parsed_output->[0]->{version} =~ s/^"?NetApp Release//;
                $parsed_output->[0]->{version} =~ s/[\w]+\-Mode"?$//;
                $version = $parsed_output->[0]->{version};
            }			
            foreach my $attribute (keys %attribute_to_key) {
                $self->{$attribute_to_key{$attribute}} ||= trim_string(
                string => $parsed_output->[0]->{$attribute} );
            }
        }
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    # For promoted build, the build_path is formed with the version and always 
    # assumed to be under /x/eng/rlse/DOT/
    if ( defined $version &&  $version =~ /^[\d\.PXx]*$/ && $self->is_promoted()) {
        $self->{build_path} = "/x/eng/rlse/DOT/" . $version;
    }

    # Sometimes in Smokes daemon build is used, which makes it difficult to
    # load cdefs, because daemon builds are moving targets.
    # This piece of code finds the path which can be accessible
    if ( defined $self->{build_path} &&  $self->{build_path} =~ /daemon/i ) {
        my $path = NATE::ParamSet::param_global()->get('TESTNAME');
        if ($path =~ s/\/test\/\S+/\//g) {
            $self->{build_path} = $path;
        }
    }

    $Log->exit() if $may_exit;
    return;
}

sub _command_system_node_show {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $session;

    my %attribute_to_key = (
        'model'        => 'modelinfo',
        'serialnumber' => 'serialnumber',
    );

    # Processing necessary for the required attributes

    try {
        $session = $self->get_session(
            interface => "CLI",
            subtype   => "CMode"
        );

        # Find the information

        my $command = "system node show -node local";
        my $output  = $self->execute_CLI_command(
            connection => $session->{connection},
            command    => $command
        );
        my $parsed_output = $self->parse_ngsh_output(
            command    => $command,
            raw_output => $output
        );
        if ( defined $parsed_output ) {

          # Output is parsed successfully. Now get the required field entry if
          # present and process it.

            foreach my $attribute ( keys %attribute_to_key ) {
                $self->{ $attribute_to_key{$attribute} } = trim_string(
                    string => $parsed_output->[0]->{$attribute} );
            }
        }
    }
    finally {

        # Destroy the session
        $self->destroy_session( session => $session ) if ( defined $session );
    };

    $Log->exit() if $may_exit;
    return;
}

1;

=head1 NAME

NACL::APISet::Version::Node::CMode - Version library for cluster mode

=head1 SYNOPSIS

 use NACL::APISet::Version::Node::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Node::CMode> is used to connect to CMode node and 
collect version related attributes. All version related attributes exposed from
this libary can also be set or get by the user. 

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Version::Node::CMode>.

=over 

=item Synopsis

$version = NACL::APISet::Version::Node::CMode->new(%args)

=item Arguments

=over

=item category

(Mandatory)
The category to use.  Valid values include "Node".

=item shell_type

(Mandatory)
The refers to the type of the shell host sent through 'hostobj'.
Valid values include "CMode".

=item hostobj

(Mandatory)
The hostrec object created for the resource.

=item conn_overrides

(Optional)
Hash reference to the list of connection overrides to be considered when
establishing the connection to the service.
Valid connection overrides are username, password, connid, conntype, ipaddress,
prompt_username and prompt_password.

=item console_connection

(Optional)
This paramter is applicable only to 'Node' category, especially if the
shell_type is either 7Mode or CMode. If you already have a console
connection and if you want the version manger to reuse the same connection for
whatever attributes that could be discovered from ngshell/7Mode shell, then you
can specify the existing console connection object to this option.

=back

=item Returns

A reference to a new CMode Version object

=back

=cut

=head1 METHODS

=head2 _find_branch

This method discovers branch information by creating the appropriate session
and execute various commands like "ident", "cat /BUILD" , "system image
show" to get information about branch.

=over 

=item Synopsis

$branch = $version_object->get_version_attribute(attribute => "branch");

The above method will internally call $self->_find_branch() if the information
about branch atribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the branch information in version object itself.

=back

=head2 _find_release

This method discovers release information by creating the appropriate session
and execute various commands like "sysctl", "cat /BUILD", "system image show" 
to get information about release.

=over 

=item Synopsis

$release = $version_object->get_version_attribute(attribute => "release");

The above method will internally call $self->_find_release() if the information
about release atribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the release information in version object itself. 

=back

=head2 _find_modelinfo

This method finds the model information by creating the appropriate session
and execute commands like "system node show" to get information about system 
model.

=over 

=item Synopsis

$release = $version_object->get_version_attribute(attribute => "modelinfo");

The above method will internally call $self->_find_modelinfo() if the information
about modelinfo atribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the model information in version object itself. 

=back

=head2 _find_serialnumber

This method finds the serialnumber information by creating the appropriate session
and execute commands like "system node show" to get information about system 
serial number.

=over 

=item Synopsis

$serialnumber = $version_object->get_version_attribute(attribute => "serialnumber");

The above method will internally call $self->_find_serialnumber() if the information
about serialnumber attribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the system serial number in version object itself. 

=back

=head2 _find_client

This method finds the client information by creating the appropriate session
and execute commands like " cat /BUILD" to get information about CLIENT

=over 

=item Synopsis

$release = $version_object->get_version_attribute(attribute => "client");

The above method will internally call $self->_find_client() if the information
about client atribute is not already present in $self. This method shouldn't be
directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the client information in version object itself. 

=back

=head2 _find_change_number

This method discovers change number associated with the build by creating the 
appropriate session and execute various commands like "ident", "cat /BUILD",
"system image show" to get information about change number.

=over 

=item Synopsis

$change = $version_object->get_version_attribute(attribute => "change_number");

The above method will internally call $self->_find_change_number() if the 
information about change_number atribute is not already present in $self. This
method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the change number information in version object itself. 

=back

=head2 _find_build_options

This method discovers build options by creating the 
appropriate session and execute various commands like "ident", "cat /BUILD",
"system image show" to get information about build options.

=over 

=item Synopsis

$build_options = 
    $version_object->get_version_attribute(attribute => "build_options");

The above method will internally call $self->_find_build_options() if the 
information about build_options atribute is not already present in $self. This
method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the build options information in version object itself. 

=back

=head2 _find_build_path

This method discovers build path by creating the 
appropriate session and execute various commands like "ident", "sysctl",
"system image show" to get information about build path.

=over 

=item Synopsis

$build_path = 
    $version_object->get_version_attribute(attribute => "build_path");

The above method will internally call $self->_find_build_path() if the 
information about build_path atribute is not already present in $self. This
method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the build path information in version object itself.

=back

=head2 _find_node_version

This method discovers node version information by creating the appropriate session
and execute various commands like "sysctl", "version" in Nodescope context
to get information about node version.

=over 

=item Synopsis

$node_version = 
    $version_object->get_version_attribute(attribute => "node_version");

The above method will internally call $self->_find_node_version() if the 
information about node_version atribute is not already present in $self. This
method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the node version information in version object itself. 

=back

=head2 _find_wafl_version

This method discovers wafl version information by running appropriate
commands in Nodescope.

=over 

=item Synopsis

$wafl_version = 
    $version_object->get_version_attribute(attribute => "wafl_version");

The above method will internally call $self->_find_wafl_version() if the 
information about wafl_version atribute is not already present in $self. This
method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the wafl version information in version object itself. 

=back

=head2 _find_uichange_strings

This method discovers uichange strings information by creating the CMode CLI
session and execute "system node uichanges local"to get information
about list of all uichange strings for the local node.

=over 

=item Synopsis

@uichanges = $version_object->get_version_attribute(attribute => 
    "uichange_strings");

The above method will internally call $self->_find_uichange_strings() if the
information about uichange_strings atribute is not already present in $self. 
This method shouldn't be directly invoked by the user.

=item Arguments

Nothing

=item Returns

Nothing. Set the uichange_strings information as an array attribute in 
version manager object itself.

=back

=head2 has_uichange

This method checks whether the service has the specified uichange string
or not.

=over 

=item Synopsis

$is_present = $version_object->has_uichange(uichange => "filer-to-node");

=item Arguments

=over

=item uichange

Required.
The change string

=back

=item Returns

1 if the uichange string is present, else 0.

=back

=head2 is_promoted

Checks whether the service is running a promoted(release) or
development build.

=over 

=item Synopsis

$is_promoted = $version_object->is_promoted();

=item Arguments

Nothing

=item Returns

This method will return 1 for promoted builds and 0 for
development builds.

=back

=head2 setup_compatibility

This method obtains the list of uichange strings from the service and then
initializes the compatibility engine by invoking the
necessary routine from compatibility module which will construct the
compatibility hash dynamically for the uichange strings present in the service.

=over 

=item Synopsis

$is_present = $version_object->setup_compatibility(interface => $interface);

=item Arguments

=over

=item interface

Required.
Interface in which we want to communicate the service. 
Possible values are CLI, ZAPI.

=back

=item Returns

Nothing

=back

=head2 compatibility

=over

=item Synopsis

This method returns the Compatibility hash, which maps uichange strings to
the corresponding command and option name change.
(See L<NACL::APISet::Version::VersionData::CompatibilityDataCli|lib-NACL-APISet-Version-VersionData-CompatibilityDataCli-pm> for more information)

=item Returns

A compatibility hash or hashref based on the context in which the method is
being invoked

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

