#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Version class for finding Version attributes specific to CiscoNetwork.
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here


package NACL::APISet::Version::Switch::CiscoNetwork;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports
#
use NACL::APISet::Session::CLI::CiscoNetwork;
use NACL::APISet::Response::CLI::ParserUtils;

#
# Exceptions imported
#
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;
use NACL::APISet::Exceptions::ConnectionFailedException;
use NACL::APISet::Exceptions::AttributeDiscoveryFailed;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

#
# base class
#
use base qw(NACL::APISet::Version::Switch);

###############################################################################
# Method:    _find_os_revision
# Objective: Will return Operating System revision
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _find_os_revision {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my $os_revision;
    my $session;
    my $os_version;

    try {
        $session = $self->get_session(interface  => "CLI",
                                      subtype    => "CiscoNetwork");

        $self->execute_CLI_command(connection  => $session->{connection},
                                   command     => "terminal length 0");
        $self->execute_CLI_command(connection  => $session->{connection},
                                   command     => "terminal width 511");
        my $output = $self->execute_CLI_command(connection  => $session->{connection},
                                                command     => "show version");

        my @lines = split(/\n/, $output);

        # For Cisco Nexus OS $outpus also contains the commands.
        # Removing it if its there
        shift(@lines) if ($lines[0] =~ /show version/);
        if ($lines[0] =~ /^Cisco\s+(\w+)\s+/i) {
            $os_revision = $1;
            if ($lines[0] =~ /^Cisco.*Version\s+(\S+)\s+/i) {
                $os_version = $1;
                $os_version =~ s/\,//;
            }
        }
        if ($output =~ /(system|NXOS)\:\s+version\s+(\S+)/i){
            $os_version = $2;
        }

    } finally {
        # Destroy the session
        $self->destroy_session(session => $session) if (defined $session);
    };

    $self->{os_revision}->{major} = $os_revision;
    $self->{os_version}->{version} = $os_version;

    $Log->exit() if $may_exit;
    return $os_revision
}

1;

=head1 NAME

NACL::APISet::Version::Switch::CiscoNetwork - Version library for CiscoNetwork

=head1 SYNOPSIS

 use NACL::APISet::Version::Switch::CiscoNetwork;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Switch::CiscoNetwork> is used to connect to CiscoNetwork Switch and
collect version related attributes. All version related attributes exposed from
this libary can also be set or get by the user.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument

=item NACL::APISet::Exceptions::ConnectionFailedException

=item NACL::APISet::Exceptions::AttributeDiscoveryFailed

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Version::Switch::CiscoNetwork>.

=over

=item Synopsis

$version = NACL::APISet::Version::Switch::CiscoNetwork->new(%args)

=item Arguments

=over

=item category

(Mandatory)
The category to use.  Valid values include "Switch".

=item shell_type

(Mandatory)
The refers to the type of the shell host sent through 'hostobj'.
Valid values include "CiscoNetwork".

=item hostobj

(Mandatory)
The hostrec object created for the resource.

=item conn_overrides

(Optional)
Hash reference to the list of connection overrides to be considered when
establishing the connection to the service.
Valid connection overrides are username, password, connid, conntype, ipaddress,
prompt_username and prompt_password.

=back

=item Returns

A reference to a new CiscoNetwork Version object

=back

=cut 
