#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Common routines for CLI APISets
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
package NACL::APISet::_Mixins::CLI;
use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Exceptions::TimeoutException;
use Params::Validate qw(validate_with HASHREF SCALAR ARRAYREF CODEREF UNDEF);
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::CouldNotResyncConnectionException;
use NATE::Exceptions::Argument;
use Data::Dumper;
# use XML::LibXML;
use NACL::GeneralUtils qw(coredump_on_timeout);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        _move_connectrec_options
        get_allowed_options_for_command
        get_privilege
        execute_raw_command
        _find_parser_file
        _get_absolute_cdef_paths
        _check_for_unnamed_parameters
        _throw_timeout_exception
        _is_help_cmd
        _is_contain_question_mark
        _load_version_xml
    );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use Class::MethodMaker [
    scalar => [
        {
            -default_ctor => sub {
                my $content = NATE::Inc::get_file("NACL/APISet/Version/VersionData/version.xml")
                    or NATE::BaseException->throw("Error in reading 'version.xml': $@");
                require XML::LibXML;
                return XML::LibXML->load_xml(string => $content);
            }
        },
        'version_xml_obj',
    ],
];

=head1 NAME

NACL::APISet::_Mixins::CLI

=head1 DESCRIPTION

This package is a mixin containing methods which are common to the
various CLI APISets.

=head1 METHODS

=head2 _move_connectrec_options

    $apiset->_move_connectrec_options(
        source => \%args,
        target => \%connectec
    );

Moves all connectrec options (i.e those that begin with "connectec-") from
the source hash to the target hash. The keys in the target hash do not contain
the leading "connectrec-".

=cut

sub _move_connectrec_options {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    validate_with(
        params => \%args,
        spec   => {
            source => { type => HASHREF },
            target => { type => HASHREF },
        }
    );
    my $source    = $args{source};
    my $target    = $args{target};
    my %deep_copy = %$source;
    while ( my ( $key, $value ) = each %deep_copy ) {
        if ( $key =~ /^connectrec\-(.*)/ ) {
            $target->{$1} = delete $source->{$key};
        }
    }
    $Log->exit() if $may_exit;
}

=head2 get_allowed_options_for_command

    my @allowed_options = $apiset->get_allowed_options_for_command(command => $command);

This method returns an array of option names which are accepted by the command.

=over

=item Options

=over

=item C<< command => $command >>

The command for which we want the list of allowed options. Note that this
should be provided the way the method call would be made (i.e. with all
spaces and hyphens turned into underscores). For example, if we want to
know that options for "aggr status", we should give the command as
"aggr_status".

=back

=back

=cut

sub get_allowed_options_for_command {
    $Log->enter() if $may_enter;
    my $self = shift;
    my @allowed_options
        = $self->get_schema_obj()->get_allowed_options_for_command(@_);
    $Log->exit() if $may_exit;
    return @allowed_options;
}

=head2 get_privilege

    my $current_priv_level = $apiset->get_privilege();

Returns the current privilege level of the APISet.

=cut

sub get_privilege {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $Log->exit() if $may_exit;
    return $self->{'privilege-level'};
}

=head2 execute_raw_command

    my $output = $apiset->execute_raw_command(command => $cmd, ...);

This method executes the command $cmd and returns the raw output. This
method is a thin wrapper around Connectrec->execute and accepts the
same options as that method. This method does not provide the traditional
advantages of APISet methods (output parsing, error checking) and the
command needs to be provided exactly as what needs to be invoked.

=cut

sub execute_raw_command {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $connection = $self->get_connection();
    $self->api_reconnect_session()
        if $connection->is_disconnected( verify => 0 );
    $self->_move_connectrec_options(
        source => \%args,
        target => \%args
    );

    $self->timeout_scaling_factor(
        Tharn::param("APISET_TIMEOUT_SCALING_FACTOR"))
        if defined Tharn::param("APISET_TIMEOUT_SCALING_FACTOR");

    $args{'on_timeout'}
        = $self->_throw_timeout_exception( before => $args{'on_timeout'}, timeout => $args{timeout} );
    $args{'on_idle'}
        = $self->_throw_timeout_exception( before => $args{'on_idle'}, timeout => $args{timeout} );
    my $output = $connection->execute(%args);
    $Log->exit() if $may_exit;
    return $output;
}

=head2 _find_parser_file

    my $parser_file = $apiset->_find_parser_file(method => $method);

This method finds the parser file and returns it.

=over

=item Options

=over

=item C<< method => $method >>

Method name which will get converted to command.

=back

=back

=cut

sub _find_parser_file {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;
    my $schema_obj = $self->get_schema_obj();

    # Determine the CDEF file which was read to construct the command
    # Then change the extension from .cdefs to .pm, so that the appropriate
    # parser file gets loaded.
    my $method    = $args{method};
    my $cdef_obj  = $schema_obj->get_command_definition( command => $method );
    my $cdef_file = $cdef_obj->get_file();
    my ($basename) = ( $cdef_file =~ /(\w+)\.cdefs$/ );
    if ( !$basename ) {
        NATE::BaseException->throw(
            "cdefs file '$cdef_file' did not end in '.cdefs'?");
    }
    my $pm_basename = $basename . ".pm";

    # Cdef_paths is a string, concatenation of
    # different cdef paths, having ";" as delimiter
    my $pm_file = NATE::Inc::find_file( $pm_basename,
        search => [ split( /;/, $self->{cdef_path} ) ], register_for_s2t => 0 );

    # Parser file is needed even if its not present.
    # Defaulting it to cdef file with .pm extention
    if ( !defined $pm_file ) {
        $pm_file = $cdef_file;
        $pm_file =~ s/cdefs$/pm/;
    } else {
        if ( NATE::Inc::package_of($pm_file) eq "NATE::Inc::Plain" ) {
            require $pm_file;
        } else {
            my $code = NATE::Inc::get_file($pm_file, register_for_s2t => 0);
            eval $code;
            if ($@) {
                NATE::BaseException->throw(
                    "Error while evaluating contents of file '$pm_file': $@");
            }
        }
        my $package = filename2packagename($pm_file);
        if ( !$package->can($method) ) {
            $Log->trace(
                "Subroutine $package"."::"."$method was not defined after compiling file '$pm_file'"
            );
        }
    }
    $Log->trace("Parser file found is $pm_file");
    $Log->exit() if $may_exit;
    return $pm_file;
}

sub filename2packagename {
    my ($name) = @_;
    $name =~ s/\.pm$//;

    #since we are interested in only the NACL part following will work fine for all os.
    $name =~ s/[\/\\]/::/g;
    $name =~ s/.*(NACL::.+)/$1/;
    return $name;
}

=head2 _get_absolute_cdef_paths

    my $parser_file = $apiset->_get_absolute_cdef_paths();

This method returns absolute cdef paths.

=cut

sub _get_absolute_cdef_paths {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my @paths = split( /\;/, $self->{cdef_path} );
    my @abs_paths = map { NATE::Inc::find_file($_) } @paths;
    @abs_paths = grep { defined($_) } @abs_paths;
    $self->{cdef_path} = join( ';', @abs_paths );
    $Log->exit() if $may_exit;
}

=head2 _check_for_unnamed_parameters

    my %args = $apiset->_check_for_unnamed_parameters( options =>\@opts ,
                                                       method => $method );

This method will, temporarily, throw a warning if unnamed arguments are passed.
This method will be changed so that the code exceptions out if unnamed arguments
are passed.

=cut

sub _check_for_unnamed_parameters {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    validate_with(
        params => \%opts,
        spec   => {
            options => { type => ARRAYREF },
            method  => { type => SCALAR },
        }
    );
    my $method_name = $opts{method};
    my %args;
    try {
        %args = $self->validate_params(
            params      => $opts{options},
            spec        => {},
            allow_extra => 1
        );
    }
    catch NATE::Exceptions::Argument with {
        my $exception_obj = shift;
        $Log->warn( "The call to method '$method_name' caused the error:\n"
                . $exception_obj->text()
                . "\nPlease make sure to rectify the bug in your\n"
                . "script. This is a temporary warning and if the bug in your script is\n"
                . "not fixed, script failures will occur once this check is removed"
        );
        %args = @{ $opts{options} };
    };
    $Log->exit() if $may_exit;
    return %args;
}
##############################################################################
# Method:    _throw_timeout_exception
# Objective: Throws TimeoutException on timeout or idle connection
# Details:   On timeout of connection, or idle connection this subroutine is
#             executed to throw the TimeoutException
###############################################################################
sub _throw_timeout_exception {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    validate_with(
        params => \%args,
        spec   => {
            timeout => { type => SCALAR | UNDEF,  optional => 1 },
            before  => { type => CODEREF | UNDEF, optional => 1 }
        }
    );
    my $execute_before = $args{before};
    $Log->exit() if $may_exit;
    return sub {
        my $session;
        if ( $self->isa('NACL::APISet') ) {
            $session = $self->{session};
        } elsif ( $self->isa('NACL::APISet::Session') ) {
            $session = $self;
        }
        my $connection  = $session->{connection};
        my $timeout;
        if ( defined $args{timeout} ) {
            $timeout = $args{timeout};
        } else {
            $timeout = $session->get_timeout();
        }
        # the user's timeout handler is executed first
        &$execute_before if $execute_before;

        my $set            = $self->{set} || $args{set};
        my %args_to_resync = ();
        $args_to_resync{'shell_type'} = 'cluster' if $set eq "CMode";

        my $output;
        try {
            $connection->send_intr();
            $output         = $connection->execute_expect(timeout => 60);
            $connection->resync(%args_to_resync);
        } catch NATE::BaseException with {
            my $exception = $_[0];
	    $Log->warn( "Most probably error has occurred as a consequence of timed out of some previous command. Before triaging the issue further, please check the connection state  which may be in 'off-by-one'.");
            NACL::APISet::Exceptions::CouldNotResyncConnectionException
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
        };
        my $message;

        if ( !$execute_before ) {

      # Output will be available only if the default handler alone is executed
      # This is because when custom handlers are specified, we cannot be sure
      # that the output was not already read in the custom handler itself.
            $message = "Timeout has occured.";
            if ( $connection->is_persist ) {
                $message .= "Prompts will be resynced to prepare the connection for the next command execution.\n";
            }
            $message .= "For more details, refer to: http://wikid.netapp.com/w/QA/projects/Libraries_Initiative/Project/Common_Infrastructure/Arch_FAQ#Why_do_my_tests_fail_with_invocation_of_.E2.80.9Cecho_resync.E2.80.9D.3F" . "\n--";
            $message .= "The output obtained so far is:" . "\n--";
            $message .= $output . '--';
        } else {
            $message = 'User defined timeout handler was executed';
        }

        my $dump_core = Tharn::param('NACL_COREDUMP_ON_TIMEOUT');

        if (defined $dump_core and $dump_core == 1) {
            coredump_on_timeout(conn => $connection);
        }

        NACL::APISet::Exceptions::TimeoutException->throw(
            $message,
            timeout => $timeout,
            output  => $output
        );
    }
}
 ## end sub _throw_timeout_exception

sub _is_contain_question_mark {
    $Log->enter() if $may_enter;
    my ($self, %args) =  @_;
    my $contain_question_mark = 0;
    validate_with(
        params => \%args,
        spec   => {
            command => { type => SCALAR },
        }
    );

    if($args{command} =~ /\?/) {
        $contain_question_mark = 1;
    }

    $Log->exit() if $may_exit;
    return $contain_question_mark;
}

sub _is_help_cmd {
    $Log->enter() if $may_enter;
    my ($self, %args) =  @_;
    my $is_help_cmd = 0;
    validate_with(
        params => \%args,
        spec   => {
            command => { type => SCALAR },
        }
    );

    if($args{command} =~ /\?$/ or $args{command} =~ /\?\s*;/) {
        $is_help_cmd = 1;
    }

    $Log->exit() if $may_exit;
    return $is_help_cmd;
}

sub _load_version_xml {
    return __PACKAGE__->version_xml_obj();
}

1;
