
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common routines for APISet exceptions
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::APISet::_Mixins::Exceptions::Common;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with OBJECT ARRAYREF);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        convert
        convert_and_throw
        detect_convert_and_throw
        _detect_helper
    );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );

    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

=head1 NAME

NACL::APISet::_Mixins::Exceptions::Common

=head1 DESCRIPTION

This package is a mixin containing methods which are common to the
various APISet exception.

=head1 METHODS

=head2 convert

    my $newtype_obj = newtype->convert(exception => $exception_obj);
    
Casts an exception to a new exception type while carrying across all
its attributes.

=cut

sub convert {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate( @args,
        { exception => { type => OBJECT, isa => 'NATE::BaseException' } } );

    # Sneaky re-bless. Simplest way to carry across all the attributes
    # of the original object.
    my $obj = bless $opts{exception}, $pkg;
    $Log->exit() if $may_exit;
    return $obj;
}

=head2 convert_and_throw

    newtype->convert_and_throw(exception => $exception_obj);
    
Casts an exception to a new exception type while carrying across all
its attributes and throws the newly casted exception.

=cut

sub convert_and_throw {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $obj = $pkg->convert(@args);
    $Log->exit() if $may_exit;
    $obj->throw();
}

=head2 detect_convert_and_throw

    newtype->detect_convert_and_throw(exception => $exception_obj);
    
Detects if the error is of C<newtype> and if so, converts to this type and
throws.
    
=cut

sub detect_convert_and_throw {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    if ($pkg->detect(exception => $opts{exception})) {
        $Log->exit() if $may_exit;
        $pkg->convert_and_throw(%opts);
    }
    
    $Log->exit() if $may_exit;
}

sub _detect_helper {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            exception => { type => OBJECT, can => 'text' },
            regexes   => { type => ARRAYREF }
        }
    );
    
    my $text = $opts{exception}->text();
    my $regexes = $opts{regexes};
    
    foreach my $regex (@$regexes) {
        if ($text =~ $regex) {
            $Log->exit() if $may_exit;
            return 1;
        }
    }
    
    $Log->exit() if $may_exit;
    return 0;
}

1;
