#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @pod here

package NACL::AppBuilder::Configurator::Configurator;

use strict;
use warnings;

use Params::Validate
    qw(validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF CODEREF);
use NATE::BaseException qw(:try);
use Data::Dumper;
use File::Basename 'dirname';

use Const::Fast;
const my $OLD_VERSION => 11;

use EnGenX;
use NACL::STask::Volume;
use NACL::STask::Snapmirror;
use NACL::C::Cluster;
use NACL::CS::ClusterIdentity;
use NACL::CS::NetworkInterface;
use Hostrec;

use NACL::C::AVANode;

use NATE::Log qw(log_global);

use NATE::ParamSet;

use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
        { -type => 'EnGenX' },
        qw/ engenx /,
    ],
    scalar => [
    qw/ log version xgen /,
    ],
    scalar => [
    qw/ resources /,
    ],
    scalar => [
    qw/ _cluster_ci _vserver_ci _vserver_node_ci /,
    ],
];


=head1 NAME

NACL::AppBuilder::Configurator::Configurator

=head1 SYNOPSIS

 use NACL::AppBuilder::Configurator::Configurator;

=head1 DESCRIPTION

A C<NACL::AppBuilder::Configurator::Configurator> object is used to
configure volumes et al for the igniter.

=head1 METHODS

=head2 new

This is the constructor method.  It returns a new Configurator object.

=over

=item Synopsis

  use NACL::AppBuilder::Configurator::Configurator;
  my $configurator = NACL::AppBuilder::Configurator::Configurator->new();
  my $resources = $configurator->configure_using_preset(
            preset => 'dpsas-sbot',
            params => $reqs, #give params hash for engenx
    );

=back


=cut

sub init {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'version' => { 'type' => SCALAR, default => $OLD_VERSION },
        },
    );

    $self->log(log_global());  #or a global our?
    #$self->log()->comment("init!");
    $self->_cluster_ci({});
    #$self->_vserver_ci({});
    $self->version($opts{version});
	$self->resources({});
    return;
}

#helper method for snapmirror object creation
sub _init_vserver_ci {
    my $self = shift;

    unless ($self->_vserver_ci()) {
        my %vservers = $self->_get_vservers();
        $self->_vserver_ci(\%vservers);
    }
}

#helper method for snapmirror object creation
sub _init_vserver_node_ci {
    my $self = shift;

    unless ($self->_vserver_ci()) {
        $self->_init_vserver_ci();
    }

    unless ($self->_vserver_node_ci()) {
        my $vservers = $self->_vserver_ci();
        my %names;
        foreach my $vserver (keys %{ $vservers }) {
            my $node = $vservers->{$vserver}->command_interface()->node();
            $names{$vserver} = NACL::C::Node->new( name => $node);
        }
        $self->_vserver_node_ci(\%names);
    }
}

sub initialize {
    my $self = shift;
    #$self->engenx();
}


=head2 preset_hash()

Returns $file path for a given preset.


=over

=item Options

=item C<< preset => $preset_name >>

The preset scalar representing an xml file for EnGenX.

The preset is one of:
volume: small debugging file
sm-functional: for snapmirror functional testing
workflow: functional workflows

=back

=cut
sub preset_hash {

    my $dir_xml = 'xml';
    my $dir = '../../../../dp/igniter';
    my %lookup = (
        'volume'     => "$dir_xml/volume.xml",
        'sm-functional'   => "$dir/snapmirror_functional.xml",
        'workflow'   => "$dir/basic_workflow.xml",
        'dpsas-sbot' => "$dir/adrvault-sbot.xml",
        'simple' => "$dir/simple.xml",
    );

    return \%lookup;
}

=head2 add_resources()

Adds resource to internal resource tree/hash


=over

=item Options

=item C<< type => $resource_type >>

type of resource to add

=item C<< resource => $resourse_hashref >>

hashref for the resource.

This is defined to contain necessary keys to create the resource

=back

=cut
sub add_resources {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'type'     => { 'type' => SCALAR, required => 1 },
            'resources' => { 'type' => HASHREF, required => 1 },
        },
    );

    my $store = $self->resources();

    my $type     = $opts{type};
	my $resources= $opts{resources};

	foreach my $name (keys %{$resources}) {
	    $store->{$type}->{$name} = $resources->{$name};
        $self->log()->comment("ADD RESOURCE $name: " . Dumper $resources->{$name});
    }

	#$self->log()->comment("LOCAL RESOURCES:" . Dumper $resources);
    $self->log()->comment("ALL RESOURCES:" . Dumper $self->resources());


	return $opts{resource}; #?

}

=head2 file_for_preset()

Returns $file path for a given preset.

If the value passed to preset ends in .xml it will
use that as a raw file path rather than looking up the preset
file path

=over

=item Options

=item C<< preset => $preset_name >>

The preset scalar representing an xml file for EnGenX

=back

=cut
sub file_for_preset {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'preset' => { 'type' => SCALAR, required => 1 },
        },
    );

    my $lookup = $self->preset_hash();
    my $preset = $opts{preset};
    my $file;
    if ($lookup->{$preset}) {
        $file = $lookup->{$preset};
    } elsif ($preset =~ /\.xml$/) {
        #if they pass an xml file directly, use that
        $self->log()->comment("using custom xml file $preset");
        $file = $preset;
    }


    # If the file is not an absolute path, it is assumed to be relative to the
    # location of this library
    if ($file !~ m{^/}) {
        my $dir = dirname __FILE__;
        $file = "$dir/$file";
    }

    #TODO throw exception for missing file

    return $file;

}

=head2 set_engenx_params()

Sets params hashref passed in into the global paramset object.

(EnGenX won't see it if it's not the global.)

=over

=item Options

=item C<< params => $params_hashref >>

This is a hashref where the keys are the parameter names
and values are the parameter values.

=back

=cut

sub set_engenx_params {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'params' => { 'type' => HASHREF },
        },
        #allow_extra => 1,
    );
    #global is required for engenx to pick it up...
    my $paramset = NATE::ParamSet->new( global => 1);


    my $params = $opts{params};

    foreach my $key (keys %{ $params }) {
        $paramset->set(key => $key, value => $params->{$key});
    }

    return $paramset;
}



=head2 insert_objects()

Inserts objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< objects => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are the NACL objects
created by EnGenX

The objects are inserted into the hash passed by request_names

=item C<< type => $type_name >>

The name of the object type (used for error messages)

=item C<< request_names => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_objects {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'objects'       => { 'type' => HASHREF },
            'type'          => { 'type' => SCALAR },
            'request_names' => { 'type' => ARRAYREF|HASHREF },
            'insert_all'    => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my $request_names = $opts{request_names};
    my @request;

    if (ref $request_names eq 'HASH') {
        @request = keys %{ $request_names };
    } else {
        @request = @{ $request_names };
    }

    my $objects = $opts{objects};

    #either insert all or just requested objects
    my @insert;

    if ($opts{insert_all}) {
        #insert all, for xgen (when you don't know the names of objects
        #in advance.  Then use filter to get just what you want.
        @insert = keys %{$objects};
        $self->log()->comment("Inserting all $opts{type} objects...");
        #in insert all mode, delete old keys as they probably won't match
        if (ref $request_names eq 'HASH') {
            foreach my $key (keys %{$request_names}) {
                $self->log()->trace('insert all so delete requested name:' . $key);
                delete $request_names->{$key};
            }
        }

    } else {
        #just insert requested objects
        @insert = @request;
    }
    my @found;
    foreach my $name (@insert) {
        if ($objects->{$name}) {
            $self->log()->comment("Inserting $opts{type} object $name");
            if (ref $request_names eq 'HASH') {
                $request_names->{$name} = $objects->{$name};
            } else {
                push @found, $objects->{$name};
            }
        } else {
            $self->log()->warn("Cannot find EnGenX $opts{type} $name");
        }
    }

    if (ref $request_names eq 'HASH') {
        return $request_names;
    } else {
        return \@found;
    }

}

=head2 insert_nodes()

Inserts node objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< nodes => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut


sub insert_nodes {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'nodes'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %node_objects = $self->_get_nodes();

    return $self->insert_objects(
        objects       => \%node_objects,
        type          => 'node',
        request_names => $opts{nodes},
        insert_all    => $opts{insert_all},
    );

}


=head2 insert_volumes()

Inserts volume objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< volumes => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut


sub insert_volumes {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'volumes'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %volume_objects = $self->_get_stask_vols();

    return $self->insert_objects(
        objects       => \%volume_objects,
        type          => 'volume',
        request_names => $opts{volumes},
        insert_all    => $opts{insert_all},
    );

}


=head2 insert_aggregates()

Inserts aggregate objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< aggregates => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_aggregates {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'aggregates'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %aggregate_objects = $self->_get_aggregates();

    return $self->insert_objects(
        objects       => \%aggregate_objects,
        type          => 'aggregate',
        request_names => $opts{aggregates},
        insert_all    => $opts{insert_all},
    );

}



=head2 insert_clients()

Inserts client objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< clients => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_clients {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'clients'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %clients_objects = $self->_get_clients();
    $self->log()->comment('client objects stuff:' .  Dumper \%clients_objects);
    return $self->insert_objects(
        objects       => \%clients_objects,
        type          => 'clients',
        request_names => $opts{clients},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_altavaults()

Inserts altavault objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< altavaults => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_altavaults {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'altavaults'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %altavaults_objects = $self->_get_altavaults(altavaults => $opts{altavaults});
    $self->log()->debug('altavault objects stuff:' .  Dumper \%altavaults_objects);
    return $self->insert_objects(
        objects       => \%altavaults_objects,
        type          => 'altavault',
        request_names => $opts{altavaults},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_btrfs_volume()

btrfs_volume objects just keep the key as the value instead of
an object.  This is useful for generating timestamp names
for later use:

- resources:
    - btrfs:
        group_btrfs_volume:
            - btrfs_volume: &EPOCHNAMES
                meta:
                    name_pattern: epnames_${EPOCH_TIME}_${0..}
                    count: 5

... later in exec block ...

                - task:
                    testcase: hw.thpy
                    type: script
                    client: $CLIENTS
                    param:
                        epochnames:
                            resources: *EPOCHNAMES

No objects have been created in this case so pull_resources
elsewhere does not grab these (for now?)

This is called by callback for igniter.

=over

=item Options

=item C<< btrfs_volume => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_btrfs_volume {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'btrfs_volume'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %btrfs_volume = map
	    { my ($n,$v) = split (/\|/,$_); $_ => bless({'node' => $n, 'volume' => $v}, 'NACL::AppBuilder::Configurator::BtrfsVolume') }
		keys %{$opts{btrfs_volume}}; #just save the name as the value

    $self->log()->debug('btrfs_volume objects stuff:' .  Dumper \%btrfs_volume);
    return $self->insert_objects(
        objects       => \%btrfs_volume,
        type          => 'btrfs_volume',
        request_names => $opts{btrfs_volume},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_solidfire_volume()

solidfire_volume objects just keep the key as the value instead of
an object.  This is useful for generating timestamp names
for later use:

- systems_under_test:
    - solidfire: &sfClient1
            meta:
                name: SFCLIENT
                name_value: no_clients

- resources:
    - solidfire
        <<: *sfClient1
        group_sf_account:
            - solidfire_account:
                meta:
                    name_pattern: acct_${EPOCH_TIME}
                group_solidfire_volume:
                    -meta_default:
                        size: 1000000000
                    - solidfire_volume: &VolSrc
                        meta:
                            name_pattern: epnames_${EPOCH_TIME}_${0..}
                            count: 5

... later in exec block ...

                - task:
                    testcase: hw.thpy
                    type: script
                    param:
                        vol_src:
                            resources: *VolSrc

No objects have been created in this case so pull_resources
elsewhere does not grab these (for now?)

This is called by callback for igniter.

=over

=item Options

=item C<< solidfire_volume => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_solidfire_volume {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'solidfire_volume'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );


    #because right now all solidfire stuff is in python we aren't actually creating anything in engenx/xgen
    #until/if that happens, we just want all the info from pull resources to send to the python lib/script so it
    #can do the creation

    my $res = $self->resources();
    $self->log()->debug('solidfire_volume objects stuff resources:' .  Dumper $res->{solidfire_volume});

    my %solidfire_volume = map  { $_ => bless({%{$res->{solidfire_volume}->{$_}}}, 'NACL::AppBuilder::Configurator::SolidfireVolume') }
		keys %{$opts{solidfire_volume}}; #just save the name as the value

    $self->log()->debug('solidfire_volume objects stuff:' .  Dumper \%solidfire_volume);
    return $self->insert_objects(
        objects       => \%solidfire_volume,
        type          => 'solidfire_volume',
        request_names => $opts{solidfire_volume},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_dummy()

Dummy objects just keep the key as the value instead of
an object.  This is useful for generating timestamp names
for later use:

- resources:
    - other:
        group_dummy:
            - dummy: &EPOCHNAMES
                meta:
                    name_pattern: epnames_${EPOCH_TIME}_${0..}
                    count: 5

... later in exec block ...

                - task:
                    testcase: hw.thpy
                    type: script
                    client: $CLIENTS
                    param:
                        epochnames:
                            resources: *EPOCHNAMES

No objects have been created in this case so pull_resources
elsewhere does not grab these (for now?)

This is called by callback for igniter.

=over

=item Options

=item C<< dummy => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_dummy {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'dummy'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %dummy_objects = map { $_, $_ } keys %{$opts{dummy}}; #just save the name as the value
    $self->log()->debug('dummy objects stuff:' .  Dumper \%dummy_objects);
    return $self->insert_objects(
        objects       => \%dummy_objects,
        type          => 'dummy',
        request_names => $opts{dummy},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_btrfs()

btrfs objects just keep the key as the value instead of
an object.  This is useful for generating timestamp names
for later use:


- systems_under_test:
    - btrfs: &btClient1
            meta:
                name: BTCLIENT
                name_value: no_clients
- resources:
    - btrfs: &btrfs1
        <<: *btClient1
        group_btrfs_volumes:
            - btrfs_volume: &EPOCHNAMES
                meta:
                    name_pattern: testep_${EPOCH_TIME}_${0..}
                    count: 5


... later in exec block ...

                - task:
                    testcase: hw.thpy
                    type: script
                    client: $CLIENTS
                    param:
                        epochnames:
                            resources: *EPOCHNAMES
                        btrfs:
                            resources: *btrfs1
... or ....
                        btrfs_client:
                            resources: *btClient1


both btrfs1 and btClient1 show up as strings.

No objects have been created in this case so pull_resources
elsewhere does not grab these (for now?)

This is called by callback for igniter.

=over

=item Options

=item C<< btrfs => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_btrfs {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'btrfs'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %btrfs_objects = map { $_, bless ({'name' => $_}, 'NACL::AppBuilder::Configurator::Btrfs') } keys %{$opts{btrfs}}; #just save the name as the value
    $self->log()->debug('btrfs objects stuff:' .  Dumper \%btrfs_objects);
    return $self->insert_objects(
        objects       => \%btrfs_objects,
        type          => 'btrfs',
        request_names => $opts{btrfs},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_solidfire()

solidfire objects just keep the key as the value instead of
an object.  This is useful for generating timestamp names
for later use:


- systems_under_test:
    - solidfire: &sfClient1
            meta:
                name: SFCLIENT
                name_value: no_clients
- resources:
    - solidfire
        <<: *sfClient1
        group_sf_account:
            - solidfire_account:
                meta:
                    name_pattern: acct_${EPOCH_TIME}
                group_solidfire_volume:
                    -meta_default:
                        size: 1000000000
                    - solidfire_volume: &VolSrc
                        meta:
                            name_pattern: epnames_${EPOCH_TIME}_${0..}
                            count: 5

... later in exec block ...

                - task:
                    testcase: hw.thpy
                    type: script
                    param:
                        vol_src:
                            resources: *VolSrc
... or ....
                        solidfire_client:
                            resources: *sfClient1


both solidfire1 and sfClient1 show up as strings.

No objects have been created in this case so pull_resources
elsewhere does not grab these (for now?)

This is called by callback for igniter.

=over

=item Options

=item C<< solidfire => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_solidfire {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'solidfire'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %solidfire_objects = map { $_, bless ({'name' => $_}, 'NACL::AppBuilder::Configurator::Solidfire') } keys %{$opts{solidfire}}; #just save the name as the value
    $self->log()->debug('solidfire objects stuff:' .  Dumper \%solidfire_objects);
    return $self->insert_objects(
        objects       => \%solidfire_objects,
        type          => 'solidfire',
        request_names => $opts{solidfire},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_clusters()

Inserts cluster objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< clusters => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_clusters {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'clusters'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %cluster_objects = $self->_get_clusters();

    return $self->insert_objects(
        objects       => \%cluster_objects,
        type          => 'cluster',
        request_names => $opts{clusters},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_luns()

Inserts lun objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< luns => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_luns {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'luns'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %lun_objects = $self->_get_luns();
    #$self->log()->comment('lun objects h2  :' . Dumper \%lun_objects);

    return $self->insert_objects(
        objects       => \%lun_objects,
        type          => 'lun',
        request_names => $opts{luns},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_lifs()

Inserts lif objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< lifs => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_lifs {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'lifs'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %lif_objects = $self->_get_lifs();

    return $self->insert_objects(
        objects       => \%lif_objects,
        type          => 'lif',
        request_names => $opts{lifs},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_vservers()

Inserts vserver objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< vservers => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_vservers {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'vservers'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %vserver_objects = $self->_get_vservers();

    return $self->insert_objects(
        objects       => \%vserver_objects,
        type          => 'vserver',
        request_names => $opts{vservers},
        insert_all    => $opts{insert_all},
    );

}

=head2 insert_snapmirrors()

Inserts snapmirror objects into igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< snapmirrors => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub insert_snapmirrors {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'snapmirrors'  => { 'type' => ARRAYREF|HASHREF },
            'insert_all' => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my %snapmirror_objects = $self->_get_snapmirrors();

    return $self->insert_objects(
        objects       => \%snapmirror_objects,
        type          => 'snapmirror',
        request_names => $opts{snapmirrors},
        insert_all    => $opts{insert_all},
    );

}



=head2 pull_resources()

Pull_resources from igniter hashtree/object.
This is called by callback for igniter.

=over

=item Options

=item C<< objects => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are the NACL objects
created by EnGenX

The objects are inserted into the hash passed by request_names

=item C<< type => $type_name >>

The name of the object type (used for error messages)

=item C<< request_names => $empty_object_hashref >>

This is a hashref where the keys are the object names
(usually ontap names) and the values are where the objects will
be inserted.

The objects are found from the configurator objects and have
previously been populated (via EnGenX for now).

=back

=cut

sub pull_resources {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'node'          => { 'type' => HASHREF },
			#'objects'       => { 'type' => HASHREF },
            'type'          => { 'type' => SCALAR },
            'global'        => { 'type' => HASHREF, default => {} },
            'instance'        => { 'type' => CODEREF, default => sub {shift} },
			'default_name'   => { 'type' => BOOLEAN, default => 1 },
			#'request_names' => { 'type' => ARRAYREF|HASHREF },
			#'insert_all'    => { 'type' => BOOLEAN, default => 0 },
        },
    );

    my $node = $opts{node};
    my $type = $opts{type};
	my %global = %{$opts{global}};
	my $instance = $opts{instance};
    my $request_names = $opts{request_names};
    my @request;

    $self->log()->log("FOUND pull_resources $type!");
    $self->log()->log(Dumper $node);

	#get all parameters in the node
	#do we need to filter these at all?
	my %parameters;
	foreach my $key (keys %{$node}) {
		if (!ref $node->{$key}) {
			#keep all scalar values
			if ($key eq 'name') {
				if ($opts{default_name}) {
                    $self->log()->trace("Set name $type to $node->{$key}");
			        $parameters{$type} = $node->{$key};
				}
			} elsif ($key =~ /_value$/) {
			    my $keyname = $key;
				$keyname =~ s/^(.*)_value/$1/;
			    $parameters{$keyname} = $node->{$key};
                $self->log()->log("Set $keyname to $node->{$key}");
			} else {
				if (defined $node->{"${key}_value"} && defined $node->{$key}) {
                    $self->log()->log("Don't overwrite $key value $parameters{$key} with $node->{$key}");
				} else {
					#$self->log()->log("Set $key to $node->{$key}");
			        $parameters{$key} = $node->{$key};
				}
			}
		}
	}

    $self->log()->log( print 'global:' . Dumper \%global);
    $self->log()->log( print 'params:' . Dumper \%parameters);
	

	my $resources = {};
	foreach my $name (keys %{$node->{objects}}) {
		my %instance_parameters;
		($name,%instance_parameters) =  $instance->($name);
        $self->log()->log( print 'instance params:' . Dumper \%instance_parameters);
		$resources->{$name} =
	        {
		        ( $opts{default_name} ? ($type     => $name) : ()),
				%global,
				%parameters,
		        %instance_parameters,

	        };
	}

	$self->add_resources(
		type => $type,
		resources => $resources,
	);
	print Dumper $resources;

}


=head2 configure_using_preset()

Configure testbed using a preset



=over

=item Options

=item C<< preset => $preset >>

This is the name of the preset that the configurator will
use for creating the configuration requested.  Internally the name is mapped
to an xml file that is passed to EnGenX.

For now, preset is one of: workflow, dpsas-sbot

IF the path of an XML file is passed (ie it ends in .xml) it will
use that file instead of a preset.

=item C<< precleanup => $precleanup >>

This is a boolean value that defaults to false.  If true, it will run
engenx cleanup before running setup.

=item C<< params => $params >>

THIS PARAMETER IS NOT CURRENTLY IN USE!

This is a hashref describing configuration parameters that
the configurator will use to change items counts and names.

Internally these are passed down as a Paramset to EnGenX and these must
be synchronized with the EnGenX XML.


=back

=cut

sub configure_using_preset {
    my $self = shift;
    #do we want to validate here also ? Gives clients a better error if so...
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'preset' => { 'type' => SCALAR },
            #'target' => { 'type' => HASHREF, optional => 1 }, #take out
            #'requirements' => { 'type' => HASHREF, optional => 1 }, #take out
            'params' => { 'type' => HASHREF, default => {} },
            'precleanup' => { 'type' => BOOLEAN, default => 0 },
            'cleanup' => { 'type' => BOOLEAN, default => 0 },
            'setup' => { 'type' => BOOLEAN, default => 1 },
        },
        allow_extra => 1,
    );

    my $engenx_obj; #should we use instance var here instead? TODO
    my $xml_file = $self->file_for_preset(preset => $opts{preset});

    #params passed from igniter
    my $params = $opts{params};


    my $paramset = $self->set_engenx_params(params => $params);

    $engenx_obj = new EnGenX( file => $xml_file, );
                            #schema_files => 1,
                            #load_resources => 1,); # XXX - what does this do?

    $self->engenx($engenx_obj); #save it
    $self->log()->comment('executing precleanup') if $opts{precleanup};
    my $preclean = $engenx_obj->cleanup() if $opts{precleanup};
    $self->log()->comment('executing engenx setup') if $opts{setup};
    my $setup = $engenx_obj->setup() if $opts{setup};
    #$self->log()->
    #comment(' after engenx setup ' . Dumper $setup) if $setup;

    my $resources = $engenx_obj->{resource_list};
    $self->log()->comment('executing cleanup') if $opts{cleanup};
    my $clean = $engenx_obj->cleanup() if $opts{cleanup};
    #TODO return this for demo...
    return $resources;
}

=head2 cleanup()

Call engenx cleanup.  Assumes configure has already been run and
thus an engenx object already exists



=over



=back

=cut

sub cleanup {
    my $self = shift;

    my $engenx = $self->engenx();
    if ($engenx) {
        my $clean = $engenx->cleanup();
    } else {
        $self->log()->warn('No engenx object to run cleanup on.  Run configure first');
    }

}

=head2 configure()

Configure testbed using xgen internally



=over

=item Options

=item C<< xgen_config => $xgen_config >>

This is a string giving the class of the xgen config to use.
Defaults to 'NACL::AppBuilder::Xgen::Config::DMG::Base'

=item C<< precleanup => $precleanup >>

This is a boolean value that defaults to false.  If true, it will run
engenx cleanup before running setup.

=item C<< params => $params >>

THIS PARAMETER IS NOT CURRENTLY IN USE!

This is a hashref describing configuration parameters that
the configurator will use to change items counts and names.


Internally these are passed down as a Paramset to EnGenX and these must
be synchronized with the EnGenX XML.


=back

=cut

sub configure {
    my $self = shift;
    #do we want to validate here also ? Gives clients a better error if so...
    my %opts = validate_with(
        params => \@_,
        spec   => {
            #'target' => { 'type' => HASHREF, optional => 1 }, #take out
            #'requirements' => { 'type' => HASHREF, optional => 1 }, #take out
            'xgen_config' => {
                'type' => SCALAR,
				#default => 'NACL::AppBuilder::Xgen::Config::DMG::Base' },
				default => 'NACL::AppBuilder::Xgen::Config::DMG::Base_YML' },
            'params' => { 'type' => HASHREF, default => {} },
            'precleanup' => { 'type' => BOOLEAN, default => 0 },
            'cleanup' => { 'type' => BOOLEAN, default => 0 },
            'setup' => { 'type' => BOOLEAN, default => 1 },
        },
        allow_extra => 1,
    );

    my $engenx_obj; #should we use instance var here instead? TODO
    my $xgen = $opts{xgen_config};

    $self->xgen(1); #we are using xgen

    #params passed from igniter
    my $params = $opts{params};

    $self->log()->trace('params for xgen:' . Dumper $params);
    $self->log()->trace('resources for xgen:' . Dumper $self->resources());

    my $paramset = $self->set_engenx_params(params => $params);

    my $client_res = $self->resources()->{client};
    my %client_params;

    if (defined $client_res->{ISCSI_CLIENTS}->{name}) {
        $client_params{iscsi_clients} = [split(',',  $client_res->{ISCSI_CLIENTS}->{name})];
    } else {
        #this should really be moved to be inside xgen - Base_YML.pm
        my @iscsi;
        foreach my $client (keys %{$client_res}) {
            push @iscsi, $client if $client_res->{$client}->{type} eq 'iscsi';
        }
        if (scalar @iscsi) {
            print "iscsi clients:" . Dumper \@iscsi;
            $client_params{iscsi_clients} = \@iscsi;
        }

    }

    $self->log()->trace('client params:' . Dumper(\%client_params));


    $engenx_obj = new EnGenX(
        config_module => $xgen,
        config_module_args => {
            # This means that if this config has been produced within this test before to
            # reuse the cached version of the config so that the config is not generated
            # from scratch everytime during a test.
            #use_cached_config       => 1,
            use_cached_config       => 0,
            #remove to not display
            display_config          => 2,

            # This means that ONTAP.pm will on look for data aggrs and not create new aggr
            # resources. So aggrs should already exist on the nodes.
            # TODO probably implement this differently
            no_new_aggrs             => 1,
            xgen_params => $params,
			resources => $self->resources(),
            %client_params,
        },
        private_paramset => 1,  #needed for multiple stest runs

    );
                            #schema_files => 1,
                            #load_resources => 1,); # XXX - what does this do?

    $self->engenx($engenx_obj); #save it
    $self->log()->comment('executing engenx precleanup') if $opts{precleanup};
    my $preclean = $engenx_obj->cleanup() if $opts{precleanup};
    $self->log()->comment('executing engenx setup') if $opts{setup};
    my $setup = $engenx_obj->setup() if $opts{setup};
    #$self->log()->
    #comment(' after engenx setup ' . Dumper $setup) if $setup;
    $self->log()->comment('executing cleanup') if $opts{cleanup};
    my $clean = $engenx_obj->cleanup() if $opts{cleanup};

    my $resource_list = $engenx_obj->{resource_list};
    $self->log()->trace('resource list:' . Dumper $resource_list);
    #TODO return this for demo...
    return $resource_list;
}


#TODO - read in "requirements" - ie, what a script needs, yaml file or some representation - then allow queries based on tags/names and return structures... vols, snapmirrors, etc...

#return engenx volumes as stasks
sub _get_stask_vols {
    my $self   = shift;

    my $vols = $self->_get_vols();

    my @stasks = map {
        NACL::STask::Volume->new(
            command_interface => $_->command_interface(),
            'volume'          => $_->volume(),
            'vserver'         => $_->vserver(),
        );
    } @{$vols};


    $self->log()->trace('vol stask objects:' . Dumper \@stasks);
    #convert to hash lookup by vol name
    my %volhash;
    if ($self->_new_version()) {
        %volhash = map { $_->vserver() . '|' . $_->volume() => $_ } @stasks;
    } else {
        %volhash = map { $_->volume() => $_ } @stasks;
    }
    #$self->log()->comment('vol stask objects:' . Dumper \%volhash);

    if (wantarray) {
        return %volhash;
    } else {
        return \@stasks;
    }
}

sub _get_vols {
    my $self = shift;

    my $vol = $self->_get_objects('volume');

    $self->log()->trace('vol objects:' . Dumper $vol);
    return $vol;
}

sub _get_vservers {
    my $self = shift;

    my $vserver = $self->_get_objects('vserver');

    $self->log()->trace('vserver objects:' . Dumper $vserver);

    if (wantarray) {
        #convert to name keys
        my %hash = map { $_->vserver() => $_ } @{$vserver};
        #$self->log()->comment('vserver objects h: ' . Dumper \%hash);
        return %hash;
    } else {
        return $vserver;
    }
}


sub _get_aggregates {
    my $self = shift;

    my $aggregate = $self->_get_objects('aggregate');

    $self->log()->trace('aggregate objects:' . Dumper $aggregate);

    if (wantarray) {
        #convert to name keys
        my %hash = map { $_->aggregate() => $_ } @{$aggregate};
        return %hash;
    } else {
        return $aggregate;
    }
}

sub _get_nodes {
    my $self = shift;

    my $node = $self->_get_objects('node');

    $self->log()->trace('node objects:' . Dumper $node);

    #filter to only keep node objects, engenx give cluster and nodes
    my @only_nodes = grep {ref($_) =~ /node/i} @{$node};

    $self->log()->trace('only node objects:' . Dumper \@only_nodes);

    if (wantarray) {
        #convert to name keys
        my %hash = map { $_->node() => $_ } @only_nodes;
        return %hash;
    } else {
        return \@only_nodes;
    }
}

sub _get_clients {
    my $self = shift;

    my $client = $self->_get_objects('client');

    $self->log()->trace('client objects:' . Dumper $client);

    if (wantarray) {
        #$self->log()->comment('client objects2:' . Dumper $client);
        #convert to name keys
        my %hash;
        foreach my $item (@{$client}) {
            #print "Item:" . Dumper $item;
            $hash{$item->{host}} = $item;
        }
        #my %hash = map { ref $_ && ($_->host() => $_) } @{$client};
        #$self->log()->comment('client objects hash:' . Dumper \%hash);
        return %hash;
    } else {
        return $client;
    }
}

sub _get_altavaults {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            #'altavaults'  => { 'type' => ARRAYREF|HASHREF },
            'altavaults'  => { 'type' => HASHREF },
        },
    );

    my $altavault;

    #no this isn't supported in engenx yet:
    #do it ourselves...
    #my $altavault = $self->_get_objects('ava');
    #check....
    foreach my $ava (keys %{$opts{altavaults}}) {
        my $avanode = NACL::C::AVANode->new(name => $ava);

        $self->log()->trace('altavault node:' . Dumper $avanode);
        $self->log()->comment('ava name:' . $avanode->name());
        push @{$altavault},$avanode;
    }

    $self->log()->trace('altavault objects:' . Dumper $altavault);

    if (wantarray) {
        #$self->log()->trace('altavault objects2:' . Dumper $altavault);
        #convert to name keys
        my %hash = map { ref $_ && ($_->name() => $_) } @{$altavault};
        $self->log()->trace('altavault objects hash:' . Dumper \%hash);
        return %hash;
    } else {
        return $altavault;
    }
}

sub _get_clusters {
    my $self = shift;

    my $cluster = $self->_get_objects('cluster');

    $self->log()->trace('cluster objects:' . Dumper $cluster);

    if (wantarray) {
        #convert to name keys
        my %hash = map { $_->name() => $_ } @{$cluster};
        return %hash;
    } else {
        return $cluster;
    }
}

sub _get_luns {
    my $self = shift;

    my $lun = $self->_get_objects('lun');

    $self->log()->trace('lun objects:' . Dumper $lun);

    if (wantarray) {
        #convert to name keys
        my %hash;
        if ($self->_new_version()) {
			#%hash = map { join('|', ($_->vserver(), $_->lun())) => $_ } @{$lun};

            %hash = map {
			    my $path = $_->path();
			    my ($vol) = $path =~ /^\/[^\/]+\/([^\/]+)\//;
			    join('|', ($_->vserver(), $vol, $_->lun())) => $_
			} @{$lun};
        } else {
            %hash = map { $_->lun() => $_ } @{$lun};
        }
        #$self->log()->comment('lun objects h :' . Dumper \%hash);
        return %hash;
    } else {
        return $lun;
    }
}

sub _get_lifs {
    my $self = shift;

    my $lif = $self->_get_objects('lif');

    $self->log()->trace('lif objects:' . Dumper $lif);

    if (wantarray) {
        #convert to name keys
        my %hash = map { $_->lif() => $_ } @{$lif};
        return %hash;
    } else {
        return $lif;
    }
}

#create snapmirror objects
sub _get_snapmirrors {
    my $self = shift;

    my $engenx = $self->engenx();
    #no this isn't supported in engenx yet:
    #my $snapmirror = $self->_get_objects('snapmirror');
    #do it ourselves...
    my $resources = $engenx->{resource_list}; #?? better way .. TODO
    #$self->log()->comment('resources:' . Dumper $resources);

    my $snapmirror = $resources->get_resources(
        category => '.*',  #seems to fail w/o this?
        type => '^snapmirror$');

    $self->log()->trace('snapmirror resources:' . Dumper $snapmirror);


    my $cluster_ci = $self->_cluster_ci();

    eval {
        #print "stored cluster ci:" . Dumper $cluster_ci;
        foreach my $sm (@{ $snapmirror}) {
            #my $cluster = $sm->{_cluster};
            my $sm_node = $sm->{_source_node};
            #my $nobj = NACL::C::Node->new( name => $sm_node);
            #this will generate a cluster nate host file on the fly
            #if for some reason you depend on a weird config in your
            #cluster natehost file this will not use data from that file
            #this COULD be changed to only do this as a fallback
            #$self->make_cluster_hostrec( node => $nobj);
            $self->make_cluster_hostrec( node => $sm_node);
            #if (!$cluster_ci->{$cluster}) {
            ##$cluster_ci->{$cluster} =
            ##NACL::C::Node->new( name => $cluster); #require cluster natehost file
            #my $nobj = NACL::C::Node->new( name => $sm_node); #require cluster natehost file
            #$self->make_cluster_hostrec( node => $nobj);
            #
            #}
        }
        #print "cluster ci:" . Dumper $cluster_ci;
    };

    $self->_init_vserver_ci();
    $self->_init_vserver_node_ci();
    my $vserver_ci = $self->_vserver_ci();
    my $vserver_node_ci = $self->_vserver_node_ci();

    my $ava_node_ci;
    my %ava_map;
    my %source_map;

    my @sm_objects = map {
        my $dest_path   =  $_->{'DESTINATION-PATH'} || $_->{'destination-path'} ||
		                   $_->{'destination-vserver'} . ':' . $_->{'destination-volume'};
        my $source_path =  $_->{'SOURCE-PATH'} || $_->{'source-path'} ||
		                   $_->{'source-vserver'} . ':' . $_->{'source-volume'};
        #get from name for now
        my ($source_vserver) = $source_path =~ /^([^:]+):/;
        my ($dest_vserver)   = $dest_path   =~ /^([^:]+):/;
        my $source_node_ci = $vserver_node_ci->{$source_vserver};
        my $destination_node_ci = $vserver_node_ci->{$dest_vserver};

        my $ava_ci;
        if (my $ava = $_->{_altavault}) {
            #get from cache or create and save
            if ($ava_node_ci->{$ava}) {
                $ava_ci = $ava_node_ci->{$ava};
            } else {
                $self->log()->debug("Creating AVANode obj. for snapmirror ci ($ava)");
                $ava_ci = NACL::C::AVANode->new(name => $ava);
                $ava_node_ci->{$ava} = $ava_ci;
            }

            #get share name w/o ip, etc
            my ($ava_share) = $dest_path =~ /\/share\/(.*)$/;
            $ava_map{$dest_path} = $ava_share;
            #TODO try not swapping here with new code from snapmirror tomorrow
            #$destination_node_ci = $source_node_ci;
            ###$source_node_ci = $ava_ci;
        }

        #solidfire (or btrfs) source path doesn't even have original name, but vol id
        if ($_->{'_solidfire'} || $_->{'_btrfs'}) {
            $self->log()->trace("btrfs/solidfire source, use original source volume key $_->{_source_volume} instead of $source_path");
            $source_map{$source_path} = $_->{_source_volume};
        }
        #eval {
        #my $source_node =  $vserver_ci->{$source_vserver}->command_interface()->node();
        #print "source node? " . $source_node . "\n";
        #my $dest_node = $vserver_ci->{$dest_vserver}->command_interface()->node();
        #print "dest node? " . $dest_node . "\n";
        #$source_node_ci = NACL::C::Node->new( name => $source_node);
        #$destination_node_ci = NACL::C::Node->new( name => $dest_node);
        #};
        #we try node ci, then cluster, then vserver
        my $source_ci = $source_node_ci || $vserver_ci->{$source_vserver};
        #my $ci = $vserver_ci->{$dest_vserver};
        my $ci =
        #$ava_ci ||  #use ava if there is one - doesn't work till sm->new accepts ava objects
                 $destination_node_ci ||
                 $self->get_cluster_hostrec(cluster => $_->{_cluster} ) ||
                 #$cluster_ci->{$_->{_cluster}} ||
                 $vserver_ci->{$dest_vserver};

        $self->log()->trace("Creating STask::Snapmirror obj. source_path=$source_path dest_path=$dest_path");
        $self->log()->debug('ci'. Dumper $ci);
        $self->log()->debug('source_ci'. Dumper $source_ci);
        my $sm;

        if (!$ava_ci) {
            $sm = NACL::STask::Snapmirror->new(
                command_interface  => $ci,
                source_ci          => $source_ci,
                'source-path'      => $source_path,
                'destination-path' => $dest_path,
            );
        } else {
            $self->log()->trace('This is an AVA SM, construct modified snapmirror obj');
            $sm = NACL::STask::Snapmirror->new(
                #command_interface  => undef,
                command_interface  => $source_ci,
                source_ci          => $source_ci,
                'source-path'      => $source_path,
                'destination-path' => $dest_path,
                'destination_ci'   => $ava_ci,
            );
            #TODO does this work now for ava??  (snapmirror->new)
        }
        $self->log()->debug('pre ava sm' . Dumper $sm);
        #$sm->{destination_ci} = $ava_ci; #TODO add this manually for now till sm->new works for ava obj
        $sm;

    } (@{ $snapmirror });

    #create dummy solidfire objs
    my $res = $self->resources();
    $self->log()->debug('snapmirror objects stuff resources:' .  Dumper $res->{snapmirror});
    my @sf_snapmirror = grep {$_->{solidfire}} (values %{$res->{snapmirror}});
    $self->log()->comment('snapmirror solidfire resources:' .  Dumper \@sf_snapmirror);
    my %sf_snapmirror_lookup;

    my @sf_sm_objects;
    if (scalar @sf_snapmirror) {
        my $vols = $self->_get_vols();
        #$self->log()->debug('snapmirror objects stuff resources:' .  Dumper $vols);

        @sf_sm_objects = map {
            $self->log()->debug('snapmirror objects stuff resources:' .  Dumper $_);
            my $dstvol = $_->{dst_vol};
            $self->log()->debug('snapmirror dst vol:' .  Dumper $dstvol);
            (my $volobj) = grep {$self->log()->debug("vol:$_->{volume}"); $_->{volume} eq $dstvol} @{$vols};
            #$self->log()->debug('solidfire dst vol:' .  Dumper $volobj);
            my $dest_path = "$volobj->{vserver}:$volobj->{volume}";
            $sf_snapmirror_lookup{$dest_path} = $_;

            my $ci = $volobj->{command_interface};
            $self->log()->debug('solidfire SM dest path:' .  Dumper $dest_path);

            my $sm = NACL::STask::Snapmirror->new(
                command_interface  => $ci,
                #source_ci          => $source_ci,
                #'source-path'      => $source_path,
                'destination-path' => $dest_path,
            );
            $self->log()->debug('solidfire sm' . Dumper $sm);

            $sm;
            


        } @sf_snapmirror;

        push(@sm_objects, @sf_sm_objects);
    }

 
    
    $self->log()->trace('sf_snapmirror_lookup:' . Dumper \%sf_snapmirror_lookup);

    $self->log()->trace('snapmirror objects:' . Dumper \@sm_objects);
    if (wantarray) {
        my %smhash = map {
            $self->log()->trace("source-path=$_->{'source_path'} dest-path=$_->{'destination_path'}");
            my $name = join '|', $_->{'source_path'}, $_->{'destination_path'};
            my $sp = $_->{'source_path'};
		    my $dp = $_->{'destination_path'};
            if (defined $ava_map{$_->{'destination_path'}}) {
              $dp = $ava_map{$_->{'destination_path'}};
              $name = join '|', $_->{'source_path'}, $dp;
              $self->log()->trace("AVA destination, only using share name for dest part of name:$name");
            };
            if (defined $source_map{$_->{'source_path'}}) {
              $sp = $source_map{$_->{'source_path'}};
              $name = join '|', $sp, $dp;
              $self->log()->trace("replacing source path");
            };

            if (defined $sf_snapmirror_lookup{$_->{'destination_path'}}) {
                $sp = $sf_snapmirror_lookup{$_->{'destination_path'}}->{src_vol};
                $name = join '|', $sp, $dp;
                $self->log()->trace("replacing source path with solidfire vol name:$sp");
            }

            if ($sp =~ /\/cg\//) {
                # update sp and dp in case of SYNC mirrors
                # updating the keys to match the keys in igniter
                # right now the igniter's hash has keys with vol names in it
                # for SYNC mirrors the source path and dest path have cg name and not vol names
                # so here we are updating the key of the hash to make it look like a non-SYNC mirror
                # should work because we can create only one SYNC / vol
                # Long term fix is to update the keys of the igniter hash to make it look more like ONTAP
                ($sp, $dp) = $_->volume_paths_from_cg;
                $self->log()->trace("SMGR so updated paths to sp=$sp dp=$dp");
            }
			#depending on how the SM was created, it may have vservers in name
			$sp =~ s/^[^:]+:(.*)/$1/;
			$dp =~ s/^[^:]+:(.*)/$1/;
            my $name_wo_vs = join '|', $sp, $dp;
            ($name => $_, $name_wo_vs => $_)
        } @sm_objects;
        #print "sm hash:" . Dumper \%smhash;
        return %smhash;
    } else {
        return \@sm_objects;
    }
}

sub get_cluster_hostrec {
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec => {
            cluster => { type => SCALAR, },
        },
        #allow_extra => 1,
    );

	my $cluster_name = $args{cluster};

    if (my $cluster_ci = $self->_cluster_ci()->{$cluster_name}){
		return $cluster_ci;
	} else {
        $self->log()->warn("unknown cluster ci $cluster_name");
        #TODO try to load cluster natehost file?
        return undef;
	}
}

sub make_cluster_hostrec {
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec => {
            node => { type => OBJECT, },
        },
        #allow_extra => 1,
    );

    my $node_name = $args{node};

    if (my $cluster_component = $self->_cluster_ci()->{$node_name}) {
        #$self->log()->comment('using cached cluster component for node ' . $node_name);
        $self->log()->debug('using cached cluster component for node ' . $node_name);
        return $cluster_component;
    }
    $self->log()->comment('No cached cluster component for ' . $node_name . ', creating cluster hostrec...');
    my $node_component = NACL::C::Node->new( name => $node_name);

    my $clusteridentity = NACL::CS::ClusterIdentity->fetch(
        command_interface => $node_component,
        requested_fields  => [ 'name' ] );

    my $cluster_name = $clusteridentity->name();

    if (my $cluster_ci = $self->_cluster_ci()->{$cluster_name}){
		return $cluster_ci;
	}


    my $clus_mgmt_lif = NACL::CS::NetworkInterface->fetch(
        command_interface => $node_component,
        filter => { role => 'cluster-mgmt' },
    );

    # cannot rely on a cluster nate_host file being available, so create
    # cluster mgmt hostrec from info in node nate_host file
    my $node_hostrec    = $node_component->{hostrec};
    my $cluster_hostrec = Hostrec->new(
        id               => $cluster_name,
        hosttype         => $node_hostrec->{'hosttype'},
        hostname         => $cluster_name,
        default_ip       => $clus_mgmt_lif->address(),
        default_ip_gx    => $clus_mgmt_lif->address(),,
        default_username => $node_hostrec->{'default_username'} || q{admin},
        default_password => $node_hostrec->{'default_password'} ||
                                                               q{netapp1!},
    );
    my $cluster_component = NACL::C::Cluster->new(hostrec => $cluster_hostrec);
    $self->_cluster_ci()->{$cluster_name} = $cluster_component;
    #we need to store it by node name also so we can see if we've created it already
    $self->log()->comment('caching cluster component for node ' . $node_name);
    $self->_cluster_ci()->{$node_name} = $cluster_component;
	return $cluster_component;
}

sub _to_volume_stask {
    NACL::STask::Volume->new(
        command_interface => $_->command_interface(),
        'volume'          => $_->volume(),
        'vserver'         => $_->vserver(),
    );

}

# get objects returned by engenx of a particular type
sub _get_objects {
    my $self   = shift;
    my $type   = shift;

    my $engenx = $self->engenx();
    my $obj = $engenx->get_objects(-type => $type );
    #$self->log()->comment('objects:' . Dumper $obj);
    return $obj;
}

sub _get_ci {
    my $self = shift;
    my $engenx = shift;

}


#returns true if newer version interface (igniter name interfaces)
sub _new_version {
    my $self = shift;

    $self->log()->trace('version #:' . $self->version() . ' - old version:'
                          . $OLD_VERSION);

    return ($self->version() > $OLD_VERSION);
}

#pass this to _walk_tree for igniter to get resources

=head2 return_pull_resources()

Return a function to be passed to the _walk_tree method in
IceKing::Bubblegum in order to fill the igniter hash with
configurator objects

Usage:

  my $bg_data = $bg->get_data()
  my $fill_sub = $configurator->return_pull_resources();
  $bg->_walk_tree($bg_data, $fill_sub);


=cut
sub return_pull_resources {
    my $configurator = shift;

        return sub  {
            my $self = shift;
            my $node = shift;
            my $path = shift;

            #the new igniter has a different matching scheme.  The old
            #igniter version is 11
            if ($configurator->_new_version()) {

                #workaround if object names not known.  If we are using xgen
                #then insert all objects and use filter in yaml
				
                $configurator->log()->log(" in pull resources!");

                my $insert_all = $configurator->xgen();

				#if (ref $node eq 'HASH' && $path->[-1] eq 'objects') {
                if (ref $node eq 'HASH' && defined $node->{'objects'}) {
                    $configurator->log()->comment( 'found child objects:' . Dumper $node );
                    my $type = $path->[-1];
					#my $type = $path->[-2];
                    #look up one more if merged
					#$type = $path->[-3] if $type eq 'MERGED';
                    $type = $path->[-2] if $type eq 'MERGED';

                    $configurator->log()->log(" parent " . (join ',', @$path));
                    $configurator->log()->log(" $type!");

                    my $normalize = {
                        'aggr'       => 'aggregate',
                        'ava'        => 'altavault',
                        'vol'        => 'volume',
                        'sm'         => 'snapmirror',
                        'cl'         => 'cluster',
                        'vs'         => 'vserver',
                    };

					if (defined $normalize->{$type}) {
						$type = $normalize->{$type};
					}
                    $configurator->log()->log(" normalized $type!");


                    if ($type eq 'vol' || $type eq 'volume') { # || $type =~ /vol/) {
                        $configurator->log()->log("FOUND  $type!");
                        $configurator->log()->log(Dumper $node);

						my %parameters;
						foreach my $key (keys %{$node}) {
							if (!ref $node->{$key}) {
								#keep all scalar values
								$parameters{$key} = $node->{$key};
							}
						}

                        $configurator->log()->log( print 'params:' . Dumper \%parameters);

					    #aggr is in a subhash
						my $aggr = $node->{aggr}->{name} || $node->{aggregate}->{name};

                        if ($node->{'aggr-list'} && ref $node->{'aggr-list'} eq 'ARRAY') {
                            my @aggr_list;
                            foreach my $aggr (@{$node->{'aggr-list'}}) {
                                foreach my $aggrname (sort keys %{$aggr->{aggr}->{objects}}) {
                                    push @aggr_list, $aggrname;
                                }
                            }

                            #$parameters{'aggr-list'} = join ',',@aggr_list;
                            $parameters{'aggr-list'} = \@aggr_list;

                            $configurator->log()->trace("aggr-list: " . Dumper $parameters{'aggr-list'});

                        }

						my $resources = {};
						foreach my $name (keys %{$node->{objects}}) {
							#we should get vservers in a better way but this is accurate
							my ($vserver,$volume) = $name =~ /^([^|]+)\|([^|]+)$/;
                            my @junction_path = ();
							#$configurator->log()->log(Dumper $node);

                            # Set the junction-path, if one is not already set
                            # and the default is requested:
                            if ($node->{meta}->{use_default_junction_path} &&
                                    ! $parameters{'junction-path'}) {
                                @junction_path = ('junction-path' => "/$volume");
                            }

                            #$resources->{$volume} =
							$resources->{$name} =
						        {
							        volume    => $volume,
							        ( $aggr ? (aggregate => $aggr) : ()),
							        vserver   => $vserver,
                                    @junction_path,
									%parameters,
						        };
						}

						$configurator->add_resources(
							type => $type,
							resources => $resources,
						);
						print Dumper $resources;
                    } elsif ($type eq 'sm' || $type eq 'snapmirror') {
                        $configurator->log()->debug("Node is : " . Dumper $node);
                        my @item_list;
                        if ($node->{sm_type} eq 'SMGR') {
                            my $filter = $node->{filter} || '.*';
                            $configurator->log()->log("sm_type is SMGR. Filter is $filter");
                            my $hashref;
                            if ((defined $node->{cg_items}[0]->{lun}) &&
                                (defined $node->{cg_items}[0]->{lun}->{objects})) {
                                $configurator->log()->log('Found cg_item lun');
                                $hashref = $node->{cg_items}[0]->{lun}->{objects};
                            } elsif ((defined $node->{cg_items}[0]->{file}) &&
                                    (defined $node->{cg_items}[0]->{file}->{objects})) {
                                $configurator->log()->log('Found cg_item file');
                                $hashref = $node->{cg_items}[0]->{file}->{objects};
                            } else {
                                $configurator->log()->warn('Could not find lun or file cg_items in '.Dumper $node);
                            }
                            $configurator->log()->trace("CG items are : ". Dumper $hashref);
                            my @vserver_source_vol = keys($node->{source_volume}->{objects});
                            if (scalar(@vserver_source_vol) > 1) {
                                $configurator->log()->warn('Un-expected!!! Multiple source vol names found.'.Dumper @vserver_source_vol);
                            }
                            my ($source_vol) = $vserver_source_vol[0] =~ /^(?:[^|]+)\|([^|]+)$/;
                            $configurator->log()->log("Will filter by source volume=$source_vol");
                            foreach my $item_path (keys $hashref) {
                                my ($item) = $item_path =~ /^(?:[^|]+)\|$source_vol\|([^|]+)$/;
                                if (!$item) {
                                    $configurator->log()->log("Item not found. Probably did not match source_vol=$source_vol")
                                } else {
                                    if ($item =~ /$filter/) {
                                        $configurator->log()->log("$item matched $filter");
                                        push (@item_list, $item);
                                    } else {
                                        $configurator->log()->log("$item did not match $filter");
                                    }
                                }
                            }
                            $configurator->log()->log("CG Item list is @item_list");
                        }

                        my $ava_share_dest = 0;
                        my $ava_name;

                        my $btrfs_volume_source = 0;
                        my $btrfs_name;

                        my $solidfire_volume_source = 0;
                        my $solidfire_name;
                        my $solidfire_account;

                        #determine if this is an ontap->ava snapmirror
                        foreach my $key (keys %{$node}) {
                            #match Ricardio.pm snapmirror key field_regex
                            if ($key =~ /dst|destination/) {
                                $configurator->log()->debug("found dest key $key");
                                #SHARE is based on tag name
                                if (grep {$_ =~ /SHARE/} (values %{$node->{$key}->{objects}})) {
                                    $ava_share_dest = 1;
                                    #pull out ava ("ava|volume")
                                    my ($ava,$vol) = split /\|/,(keys %{$node->{$key}->{objects}})[0];
                                    $ava_name = $ava;
                                    $configurator->log()->debug("it's an ava share dest: $ava_name");
                                }
                            } elsif ($key =~ /src|source/) {
                                $configurator->log()->debug("found source key $key");
                                #btrfs_volume is based on tag name
                                if (grep {$_ =~ /BTRFS_VOLUME/} (values %{$node->{$key}->{objects}})) {
                                    $btrfs_volume_source = 1;
                                    #pull out btrfs ("btrfs|volume")
                                    my ($btrfs,$vol) = split /\|/,(keys %{$node->{$key}->{objects}})[0];
                                    $btrfs_name = $btrfs;
                                    $configurator->log()->debug("it's a btrfs volume source: $btrfs_name");
                                } elsif (grep {$_ =~ /SOLIDFIRE_VOLUME/} (values %{$node->{$key}->{objects}})) {
                                    $solidfire_volume_source = 1;
                                    #pull out btrfs ("solidfire|volume")
                                    my ($solidfire,$account, $vol) = split /\|/,(keys %{$node->{$key}->{objects}})[0];
                                    $solidfire_name = $solidfire;
                                    $solidfire_account = $account;
                                    $configurator->log()->debug("it's a solidfire volume source: $solidfire_name");
                                }
                            }
                        }

						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my ($srcvol,$dstvol) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									#my $new_name = $lun;
									my %params;
									$params{src_vol} = $srcvol;
									$params{dst_vol} = $dstvol;
									$params{cg_items} = \@item_list if (scalar(@item_list));
                                    $params{altavault} = $ava_name if $ava_share_dest;
									#$params{solidfire} = $btrfs_name if $btrfs_volume_source;
                                    $params{btrfs} = $btrfs_name if $btrfs_volume_source;
                                    $params{solidfire} = $solidfire_name if $solidfire_volume_source;
                                    $params{solidfire_account} = $solidfire_account if $solidfire_volume_source;
							        $configurator->log()->trace("  sm $srcvol $dstvol ");
									return ($name_value,%params);
							},
						);
                        } elsif ($type eq 'snapmirror_policy') {
                            $configurator->pull_resources(
                                type => $type,
                                node => $node,
                                default_name => 0,
                                global => {
                                    #'aggregate' => $node->{aggr}->{name},
                                },
                                instance => sub {
                                        my $name_value = shift;
                                        my ($vserver,$policy_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
                                        #my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
                                        #my $new_name = $policy_name;
                                        my $new_name = $name_value;
                                        my %params;
                                        $params{vserver} = $vserver;
                                        $params{policy} = $policy_name;
                                        $configurator->log()->trace("  sm policy $vserver $policy_name ");
                                        return ($new_name,%params);
                                },
                            );
                        } elsif ($type eq 'snapmirror_policy_rule') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									#my ($vserver,$policy_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									my ($vserver,$policy_name,$rule) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
                                    #my $new_name = $rule;
									my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
									$params{policy} = $policy_name;
							        $configurator->log()->trace("  sm pol rule $vserver $policy_name $rule ");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'snapshot_policy') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my ($vserver,$policy_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my $new_name = $policy_name;
									my %params;
									$params{vserver} = $vserver;
									$params{policy} = $policy_name;
							        $configurator->log()->trace("  snap policy $vserver  $policy_name");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'volume_efficiency') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
                                    #my ($vserver,$volume) = $name_value =~ /^([^|]+)\|([^|]+)$/;
                                    my ($vserver,$volume,$unique) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
                                    #my $new_name = $policy_name;
									my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
									$params{volume} = $volume;
                                    #if there's a policy object ref get the name out
                                    if (defined $node->{policy}
                                        && ref $node->{policy}
                                        && $node->{policy}->{name}) {
                                        $params{policy} = $node->{policy}->{name};
                                    }
							        $configurator->log()->trace("  volume efficiency vserver=$vserver  volume=$volume");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'volume_efficiency_policy') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my ($vserver,$policy_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
                                    my $new_name = $policy_name;
                                    #my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
                                    #$params{volume} = $volume;
                                    $params{policy} = $policy_name;
							        $configurator->log()->trace("  volume efficiency policy $vserver  $policy_name");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'vserver_nfs') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my $vserver = $name_value;
                                    #my ($vserver,$policy_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
                                    #my ($vserver,$policy_name,$rule) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
							        $configurator->log()->trace("  vserver nfs $vserver  ");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'job_schedule') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my ($cluster,$schedule_name) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my $new_name = $name_value;
									my %params;
									$params{cluster} = $cluster;
									$params{name} = $schedule_name;
							        $configurator->log()->trace("  job schedule $cluster  $new_name");
									return ($new_name,%params);
							},
						);
                    } elsif ($type eq 'share') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							default_name => 0,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							instance => sub {
								    my $name_value = shift;
									my ($ava,$share) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my %params;
									$params{altavault} = $ava;
									$params{name} = $share;
							        $configurator->log()->trace("  share $ava  $share");
									return ($name_value,%params);
							},
						);
                    } elsif ($type eq 'aggr' || $type eq 'aggregate') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
						        'node' => $node->{node}->{name},
							},
						);
						
						#my $node_name = $node->{node}->{name};
						#my $diskcount = $node->{diskcount} || $node->{meta}->{diskcount};
						#
						#my $resources = {};
						#foreach my $name (keys %{$node->{objects}}) {
						#$resources->{$name} =
						#{
						#aggregate => $name,
						#node      => $node_name,
						#diskcount => $diskcount,
						#};
						#}
						#
						#$configurator->add_resources(
						#type => $type,
						#resources => $resources,
						#);
						#print Dumper $resources;

                    } elsif ($type eq 'node') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'owner' && $path->[-3] eq 'aggr') {
                    } elsif ($type eq 'cluster' || $type eq 'cl') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'lun') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							instance => sub {
								    my $name_value = shift;
									#my ($vserver,$lun) = $name_value =~ /^([^|]+)\|([^|]+)$/;
							        my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
									$params{volume} = $volume;
									$params{lun} = $lun;
							        $configurator->log()->trace("  $vserver $lun $new_name");
									return ($new_name,%params);
							},
							global => {
								#'node' => $node->{node}->{name},
							},
						);
                    } elsif ($type eq 'vserver_peer') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							instance => sub {
								    my $name_value = shift;
									my ($vserver,$peer) = $name_value =~ /^([^|]+)\|([^|]+)$/;
                                    #my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my $new_name = $name_value;
									my %params;
									$params{vserver} = $vserver;
                                    if (ref $params{vserver_peer} && defined $params{vserver_peer}->{name}) {
                                        $params{vserver_peer} = $params{vserver_peer}->{name};
                                    }
							        $configurator->log()->trace("vserver peer  $vserver $peer");
									return ($new_name,%params);
							},
							global => {
								#'node' => $node->{node}->{name},
							},
						);
                    } elsif ($type eq 'vs' || $type eq 'vserver') {
					    my @aggrs = sort keys %{$node->{aggr}->{objects}};
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
                                'aggregate' => $node->{aggr}->{name} ||
                                               $aggrs[0],
							},
						);
                    } elsif ($type eq 'lif' || $type eq 'vif') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'client') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'altavault') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'btrfs_volume') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							default_name => 0,
							instance => sub {
								    my $name_value = shift;
									my ($btrfs,$vol) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									#my ($vserver,$volume,$lun) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my %params;
									$params{btrfs} = $btrfs;
									$params{name} = $vol;
							        $configurator->log()->trace("  btrfs_volume $btrfs  $vol");
									return ($name_value,%params);
							},
						);
                    } elsif ($type eq 'btrfs') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'solidfire') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
						);
                    } elsif ($type eq 'solidfire_volume') {
						$configurator->pull_resources(
							type => $type,
							node => $node,
							global => {
								#'aggregate' => $node->{aggr}->{name},
							},
							default_name => 0,
							instance => sub {
								    my $name_value = shift;
									#my ($solidfire,$vol) = $name_value =~ /^([^|]+)\|([^|]+)$/;
									my ($solidfire,$account,$vol) = $name_value =~ /^([^|]+)\|([^|]+)\|(.*)$/;
									my %params;
									$params{solidfire} = $solidfire;
									$params{account} = $account;
									$params{name} = $vol;
							        $configurator->log()->trace("  solidfire_volume $solidfire $account $vol");
									return ($name_value,%params);
							},
						);
                    } else {
                        $configurator->log()->warn(
                            "found unknown resource type: $type ");
                    }

                }

            } else {
                if (ref $node eq 'HASH' && $path->[-1] eq 'objects') {
                    $configurator->log()->comment( 'found objects:' . Dumper $node );
                    my $type = $path->[-2];
                    #look up one more if merged
                    $type = $path->[-3] if $type eq 'MERGED';

                    if ($type eq 'vol') {
                        $configurator->insert_volumes(volumes => $node);
                    } elsif ($type eq 'sm') {
                        $configurator->insert_snapmirrors(snapmirrors => $node);
                    } elsif ($type eq 'aggr') {
                        $configurator->insert_aggregates(aggregates => $node);
                    } elsif ($type eq 'node') {
                        $configurator->insert_nodes(nodes => $node);
                    } elsif ($type eq 'owner' && $path->[-3] eq 'aggr') {
                        $configurator->insert_nodes(nodes => $node);
                    } elsif ($type eq 'cluster') {
                        $configurator->insert_clusters(clusters => $node);
                    } elsif ($type eq 'lun') {
                        $configurator->insert_luns(luns => $node);
                    } else {
                        $configurator->log()->warn(
                            "(old) found unknown resource type: $type\n");
                    }

                }
            }


        };


}

#pass this to _walk_tree for igniter

=head2 return_fill_objects()

Return a function to be passed to the _walk_tree method in
IceKing::Bubblegum in order to fill the igniter hash with
configurator objects

Usage:

  my $bg_data = $bg->get_data()
  my $fill_sub = $configurator->return_fill_objects();
  $bg->_walk_tree($bg_data, $fill_sub);


=cut
sub return_fill_objects {
    my $configurator = shift;

        return sub  {
            my $self = shift;
            my $node = shift;
            my $path = shift;

            #the new igniter has a different matching scheme.  The old
            #igniter version is 11
            if ($configurator->_new_version()) {

                #workaround if object names not known.  If we are using xgen
                #then insert all objects and use filter in yaml

				#my $insert_all = $configurator->xgen();
                my $insert_all = 0;

                if (ref $node eq 'HASH' && $path->[-1] eq 'objects') {
                    $configurator->log()->comment( 'found objects:' . Dumper $node );
                    my $type = $path->[-2];
                    #look up one more if merged
                    $type = $path->[-3] if $type eq 'MERGED';

					my $normalize = {
						'aggr'       => 'aggregate',
						'ava'        => 'altavault',
						'vol'        => 'volume',
						'sm'         => 'snapmirror',
						'cl'         => 'cluster',
						'vs'         => 'vserver',
					};

					if (defined $normalize->{$type}) {
						$type = $normalize->{$type};
					}

                    if ($type eq 'btrfs_volume') { #hopefully we don't depend on the gratuitous /vol/ below but put this first in case...
                        $configurator->insert_btrfs_volume(
                            btrfs_volume => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'solidfire_volume') { #hopefully we don't depend on the gratuitous /vol/ below but put this first in case...
                        $configurator->insert_solidfire_volume(
                            solidfire_volume => $node,
                            insert_all => $insert_all,
                        );
					} elsif ($type eq 'vol' || $type eq 'volume' || $type =~ /vol/) {
                        $configurator->insert_volumes(
                            volumes => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'sm' || $type eq 'snapmirror') {
                        $configurator->insert_snapmirrors(
                            snapmirrors => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'aggr' || $type eq 'aggregate') {
                        $configurator->insert_aggregates(
                            aggregates => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'node') {
                        $configurator->insert_nodes(
                            nodes => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'owner' && $path->[-3] eq 'aggr') {
                        $configurator->insert_nodes(
                            nodes => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'cluster' || $type eq 'cl') {
                        $configurator->insert_clusters(
                            clusters => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'lun') {
                        $configurator->insert_luns(
                            luns => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'vs' || $type eq 'vserver') {
                        $configurator->insert_vservers(
                            vservers => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'lif' || $type eq 'vif') {
                        $configurator->insert_lifs(
                            lifs => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'client') {
                        $configurator->insert_clients(
                            clients => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'altavault') {
                        $configurator->insert_altavaults(
                            altavaults => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'dummy') {
                        $configurator->insert_dummy(
                            dummy => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'btrfs') {
                        $configurator->insert_btrfs(
                            btrfs => $node,
                            insert_all => $insert_all,
                        );
                    } elsif ($type eq 'solidfire') {
                        $configurator->insert_solidfire(
                            solidfire => $node,
                            insert_all => $insert_all,
                        );
                    } else {
                        $configurator->log()->warn(
                            "found unknown object type: $type ");
                    }

                }

            } else {
                if (ref $node eq 'HASH' && $path->[-1] eq 'objects') {
                    $configurator->log()->comment( 'found objects:' . Dumper $node );
                    my $type = $path->[-2];
                    #look up one more if merged
                    $type = $path->[-3] if $type eq 'MERGED';

                    if ($type eq 'vol') {
                        $configurator->insert_volumes(volumes => $node);
                    } elsif ($type eq 'sm') {
                        $configurator->insert_snapmirrors(snapmirrors => $node);
                    } elsif ($type eq 'aggr') {
                        $configurator->insert_aggregates(aggregates => $node);
                    } elsif ($type eq 'node') {
                        $configurator->insert_nodes(nodes => $node);
                    } elsif ($type eq 'owner' && $path->[-3] eq 'aggr') {
                        $configurator->insert_nodes(nodes => $node);
                    } elsif ($type eq 'cluster') {
                        $configurator->insert_clusters(clusters => $node);
                    } elsif ($type eq 'lun') {
                        $configurator->insert_luns(luns => $node);
                    } else {
                        $configurator->log()->warn(
                            "(old) found unknown object type: $type\n");
                    }

                }
            }


        };


}


=head1  SEE ALSO

(Future use, not supported)
This is used in class L<NACL::ServiceAPI::Configurator|lib-NACL-ServiceAPI-Configurator-pm>.  Please see the documentation for more details


=head1 AUTHOR/MAINTAINER

 dpg (ng-dpg-smgr-auto@netapp.com)

=cut

1;

__END__
