# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Event Worker class
## @author rjray@netapp.com
## @status shared
## @pod here

package NACL::AppBuilder::DpgWorkflows::EventWorker;

use 5.014;
use strict;
use warnings;
use namespace::autoclean;

use Data::Dumper;
use File::Basename qw(basename);
use Time::HiRes qw(gettimeofday tv_interval);

use Const::Fast;
use Moose;
use Moose::Util::TypeConstraints;
use Params::Validate qw(validate validation_options SCALAR);
use Scalar::Util qw(blessed refaddr);

use Tharn qw();
use NATE::Log qw(log_global);
use NATE::Process;
use NATE::BaseException;
use NATE::Exceptions::Argument;

# I don't feel like sugaring-up the declaration of $REMOTE_CODE:
## no critic (InputOutput::ProhibitJoinedReadline)

# Set the on_fail behavior for Params::Validate to throw an exception (of the
# NATE::Exceptions::Argument class) on errors instead of a plain croak().
validation_options(
    on_fail => sub {
        NATE::Exceptions::Argument->throw(shift);
    }
);

# The following are not permitted as event names because they represent
# special method names in the Moose-based event classes:
const my @DISALLOWED =>
    qw(
          new
          run
          BUILD
          BUILDARGS
          DEMOLISH
          DESTROY
  );
const my %DISALLOWED => map { $_ => 1 } @DISALLOWED;
# This list *must* match the list of the same name in the __DATA__ code-block:
const my @PASS_ATTRIBUTES =>
    qw(
          class
          event
          client
          interface
          initial_delay
          iteration_delay
          iterations
          runlength
  );
# This table is a mapping of classnames for getting the "name" of an object
# for parameter serialization. Used in _freeze_event_params().
my %CLASS2NAME = map {
    +"NACL::C::$_" => lc $_, "NACL::STask::$_" => lc $_
} (qw(Node Vserver Aggregate Volume Lun Client Cluster Snapshot));
# This is an oddity of how aggregates are represented:
for my $elt (qw(C STask)) {
    $CLASS2NAME{"NACL::${elt}::StorageAggregate"} = 'aggregate';
}
# The block of code that gets sent on all remote calls:
const my $REMOTE_CODE => join q{}, <DATA>;
# Global handle on the NATE::Log logging interface
const my $Log => log_global;

# id: A unique ID for this worker, unique only while the worker exists.
has id => (
    is      => 'ro',
    isa     => 'Int',
    lazy    => 1,
    builder => '_build_id',
);

# client: The name of the client this worker will send the job to.
has client => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
);

# interface: One of 'ZAPI' or 'CLI', determines how the connections to the
#            nodes/clusters should be done.
has interface => (
    is      => 'ro',
    isa     => subtype(
        'Str',
        where   { (($_ eq 'CLI') || ($_ eq 'ZAPI')) },
        message { "Unknown interface type '$_'" }
    ),
    default => 'ZAPI',
);

# username: User-name (if needed) to connect to client
# password: Password (if needed) to connect
has [ qw(username password) ] => (
    is  => 'ro',
    isa => 'Str',
);

# runid: The run-ID that the worker job will be given when it is started.
has runid => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
);

# process_handle: The NATE::Process object that this worker is using.
has process_handle => (
    is       => 'ro',
    isa      => 'NATE::Process',
    writer   => '_set_process_handle',
    init_arg => undef,
    handles  => [ qw(is_running is_stop_completed) ],
);

# class: Name of the class that the event should be found in.
has class => (
    is      => 'ro',
    isa     => subtype(
        'Str',
        where {
            # This matches arbitrary-length Perl class names:
            /^[[:alpha:]]\w*(?:::\w+)*$/
        },
        message { "Invalid Perl class name '$_'" }
    ),
    default => 'NACL::AppBuilder::DpgWorkflows::CoreEvents',
);

# event: Name of the event that will be invoked by this worker.
has event => (
    is       => 'ro',
    isa      => subtype(
        'Str',
        # An event must be a Perl identifier without a leading "_", and also
        # must not match any of the reserved names in %DISALLOWED:
        where   { /^[[:alpha:]]\w*$/ && (! exists $DISALLOWED{$_}) },
        message { "Invalid event name '$_'" }
    ),
    required => 1,
);

# event_params: The set of parameters that will be given to the event.
has event_params => (
    is      => 'ro',
    isa     => 'HashRef',
    reader  => '_event_params_copy',
    default => sub { {} },
    traits  => [ 'Hash' ],
    handles => {
        event_params => 'elements',
    },
);

# initial_delay: Delay, in seconds, before the event should actually start.
# iteration_delay: Delay, in seconds, between iterations of the event.
# event_delay: Delay, in seconds, between steps of the event.
# iterations: Number of times the event should execute before completing.
# runlength: Length of time, in seconds, the event should run. Overrides
#            "iterations".
has [ qw(initial_delay iteration_delay event_delay iterations runlength) ] => (
    is  => 'ro',
    isa => subtype(
        'Int',
        where { $_ >= 0 },
        message { 'Integer value must be equal to or greater than zero' }
    ),
);

# on_start: Optional call-back to be executed when the worker starts.
# on_finish: Optional call-back to be executed when the worker finishes.
has [ qw(on_start on_finish) ] => (
    is  => 'ro',
    isa => 'CodeRef',
);

# started: HiRes value for when the worker was start()'d.
# ended: HiRes value for when the worker finished (stop()'d + wait()'d).
for my $attr (qw(started ended)) {
    has $attr => (
        is       => 'ro',
        isa      => 'ArrayRef',
        reader   => "_get_$attr",
        writer   => "_set_$attr",
        init_arg => undef,
        traits   => [ 'Array' ],
        handles  => {
            $attr => 'elements',
        },
    );
}

# finished: Has this worker already finished?
has finished => (
    is       => 'ro',
    isa      => 'Bool',
    init_arg => undef,
    default  => 0,
    writer   => '_set_finished',
);

# process_on_destroy: What to do with the process_handle if it is still running
#                     when this object is destroyed
has process_on_destroy => (
    is      => 'ro',
    isa     => 'Str',
    default => 'stop',
);

__PACKAGE__->meta->make_immutable;

=head1 NAME

NACL::AppBuilder::DpgWorkflows::EventWorker - Worker class for Event Manager

=head1 SYNOPSIS

    use NACL::AppBuilder::DpgWorkflows::EventWorker;
    
    # Create a worker
    my $worker = NACL::AppBuilder::DpgWorkflows::EventWorker->new(
        client          => $client,
        interface       => 'CLI',     # defaults to 'ZAPI'
        runid           => $runid,
        event           => $event,
        event_params    => $hashref,
        iterations      => 5,
        iteration_delay => 300
    );
    
    $worker->start;
    $worker->wait;

=head1 DESCRIPTION

The B<NACL::AppBuilder::DpgWorkflows::EventWorker> class is used to create
worker processes for the B<NACL::AppBuilder::DpgWorkflows::EventManager> class,
with which it can dispatch the various events it is given. Each worker instance
is designed to handle a single event specification. Such a specification can
loop for a given length of time or number of iterations, but otherwise once an
event has completed the worker should be discarded.

Each worker instance encapsulates a B<NATE::Process> instance to do the actual
remote distributed work. Some of the methods listed below are merely proxied
directly to the B<NATE::Process> instance.

=head1 ATTRIBUTES

The following attributes are defined for instances of this class. Each
attribute value is obtained by calling the attribute name as an accessor. All
attributes are read-only (they cannot be changed after the object is created).

=over

=item id

A numerical ID that identifies this object. Guaranteed to be unique only as
long as the object itself exists.

=item client

The name of the client host that the worker will execute on.

=item interface

One of C<CLI> or C<ZAPI>, the type of interface to use when connecting to a
node or cluster. Defaults to C<ZAPI>.

=item username

The username used when connecting to the client host, if needed.

=item password

The password used when connection to the client host, if needed.

=item runid

The run-ID used for determining the location of logs.

=item process_handle

A B<NATE::Process> object instance that handles the remote execution on the
client host.

=item class

The name of the class that will be used to load the specified event. If not
given at object-creation time, defaults to
B<NACL::AppBuilder::DpgWorkflows::CoreEvents>.

=item event

The name of the event that will be executed on the remote client. See
B<NACL::AppBuilder::DpgWorkflows::CoreEvents> for the basic available events.
Other classes can be event-sources if the class name is given at object
creation time.

=item event_params

A hash-reference of the parameters that will be passed to the event when it is
executed on the remote host. See L</PASSING ARGUMENTS TO EVENTS>, below.

Defaults to an empty hash-reference.

=item initial_delay

The delay, in seconds, before the event is started. Only applied prior to the
initial iteration of the event. Defaults to 0.

=item iterations

The number of times the event should be run before the worker process on the
client machine exits. Defaults to 1. This is ignored if C<runlength> is set
to any non-zero value.

=item iteration_delay

The delay, in seconds, between iterations of the event. This is applied after
an iteration has completed, and is not applied after the last iteration.
Defaults to 0.

=item runlength

The length of time, in seconds, that the event should be allowed to be
repeatedly run. This overrides C<iterations>, but does make use of
C<iteration_delay> between each iteration.

=item started

A two-element list, returned by the B<Time::HiRes> subroutine B<gettimeofday>,
that indicates when the child process actually began execution of the event.

=item ended

A two-element list, returned by the B<Time::HiRes> subroutine B<gettimeofday>,
that indicates when the child process actually completed execution.

=item finished

A boolean value that indicates whether the worker has run and completed its
event. This differs from C<is_running> in that it will only be true after the
remote code has completed (when C<is_running> is no longer true).

=item process_on_destroy

A string value that specifies what the worker object should do with the
encapsulated B<NATE::Process> object when the worker is being destroyed and the
process is still running. Should be one of three values:

=over

=item C<stop>

This is the default value. The B<NATE::Process> object is sent the B<stop>
message, and the worker does a B<wait> to wait for it to finish. If it does not
finish within 60 seconds, B<NATE::Process> will send the remote process a hard
KILL signal.

=item C<kill>

Immediately send the remote process a hard KILL signal, bypassing the usual
B<stop> process.

=item C<detach>

Allow the remote process (and local controlling child process) to continue to
exist after the worker object is destroyed. There will be no way for a
B<NACL::AppBuilder::DpgWorkflows::EventManager> instance to get at the exit
status of the remote process, if this is done.

=back

Any value other than the above will be treated as C<stop>.

=back

All attributes are accessed through methods by the same name as the attribute.
Since all attributes in this class are read-only, the accessors do not permit
modifying the values.

=head1 METHODS

In addition to the attribute accessors, the class provides the following
methods. No subroutines are exported by this package.

=head2 new

    $worker = NACL::AppBuilder::DpgWorkflows::EventWorker->new(
        client             => $client,
        interface          => $interface,
        username           => $user,
        password           => $pass,
        runid              => $runid,
        class              => $class,
        event              => $event,
        event_params       => $event_params,
        on_start           => $callback,
        on_finish          => $callback,
        initial_delay      => $init_delay,
        iterations         => $iterations,
        iteration_delay    => $iter_delay,
        runlength          => $runlength,
        process_on_destroy => $disposition
    );

(Must be called as a class method)

Create a new worker object and return it to the caller. At the point of the
new object being returned, the remote code has not yet been started.

All failures in B<new> throw an exception. See C<Exceptions>, below.

=over

=item Options

The following options are recognized by B<new>:

=over

=item C<< client => $string >>

(Required) The name of the client machine that this worker will run the code
on. May be C<localhost> to specify the same client as is running the test
itself (but this is not recommended). The client must be up and running STAF.

=item C<< interface => $string >>

(Optional) One of C<CLI> or C<ZAPI>, to specify the type of interface that
should be used with any filers/clusters connected to. Defaults to C<ZAPI>.

=item C<< username => $string >>

(Optional) The username to be used when logging in to the client, if needed.
Does not default to anything.

=item C<< password => $string >>

(Optional) The password to be used when logging in to the client, if needed.
Does not default to anything.

=item C<< runid => $string >>

(Required) The run-ID that the worker/client process will use for logging.

=item C<< class => $string >>

(Optional) The class that the event should be found in. If not given, it
defaults to B<NACL::AppBuilder::DpgWorkflows::CoreEvents>. Must match the
following pattern:

    [[:alpha:]]\w*(?:::\w+)*

This matches an arbitrary-length Perl class name.

=item C<< event => $string >>

(Required) The name of the event to be executed on the remote host. Must fit
the following pattern:

    [[:alpha:]]\w*

That is, the name must start with a letter (upper- or lower-case), which is
followed by zero or more characters that are letters, digits, or the C<_>
character.

The event name will be used to invoke a method in the class that is specified
(defaulting to B<NACL::AppBuilder::DpgWorkflows::CoreEvents> class), so it must
match a method name present in that class.

Lastly, the following names are reserved and cannot be used for events:

    new          BUILDARGS
    run          DEMOLISH
    BUILD        DESTROY

=item C<< event_params => $hashref >>

(Optional) A reference to a hash-table of the parameters that will be passed
to the event when it is called. Defaults to an empty hash-table.

=item C<< on_start => $coderef >>

(Optional) A code-reference that will be invoked when the worker is actually
started (with the B<start> method). The callback will be passed two positional
parameters:

=over

=item C<$worker>

The first parameter will be the referent to the
B<NACL::AppBuilder::DpgWorkflows::EventWorker> object that has just been
started.

=item C<$stage>

The second parameter will be a string specifying what stage of operation the
worker is currently signalling. For the C<on_start> callback, this will always
be C<start>.

=back

The return value of the callback is ignored. However, if it dies or throws an
exception, that will not be caught and could terminate the worker or the test
as a whole.

=item C<< on_finish => $coderef >>

(Optional) A code-reference that will be invoked when the worker is actually
finished (either with the B<stop> method or by exiting normally). The callback
will be passed two positional parameters:

=over

=item C<$worker>

The first parameter will be the referent to the
B<NACL::AppBuilder::DpgWorkflows::EventWorker> object that has just finished.

=item C<$stage>

The second parameter will be a string specifying what stage of operation the
worker is currently signalling. For the C<on_finish> callback, this will always
be C<finish>.

=back

The return value of the callback is ignored. However, if it dies or throws an
exception, that will not be caught and could terminate the worker or the test
as a whole.

=item C<< initial_delay => $integer >>

(Optional) If given, the start of the event processing will be delayed by this
many seconds. Note that this delay is only applied to the period B<before>
the first iteration of the event. For delay between iterations of the event,
see C<iteration_delay>, below.

If a parameter by this name appears in the C<event_params> hash table, then
that value will be considered more-specific and will override any value given
at object construction.

=item C<< iterations => $integer >>

(Optional) If given, specifies the number of times that the event will be
executed before the child process exits. This parameter will be ignored if
C<runlength> is given.

If a parameter by this name appears in the C<event_params> hash table, then
that value will be considered more-specific and will override any value given
at object construction.

=item C<< iteration_delay => $integer >>

(Optional) If given, specifies the delay (in seconds) between each iteration of
the event on the client host. Does not trigger on the last iteration. If there
is only one iteration, the delay will not trigger.

If a parameter by this name appears in the C<event_params> hash table, then
that value will be considered more-specific and will override any value given
at object construction.

=item C<< runlength => $runlength >>

(Optional) If given, specifies the length of time (in seconds) that the event
should continuously execute. Each iteration will be subject to
C<iteration_delay>, above, as well. If this option is passed (and is non-zero),
the C<iterations> parameter is ignored.

If a parameter by this name appears in the C<event_params> hash table, then
that value will be considered more-specific and will override any value given
at object construction.

=item C<< process_on_destroy => $string >>

(Optional) One of C<stop>, C<kill> or C<detach>. Determines what this worker
should do with the encapsulated B<NATE::Process> object if it is still running
when the worker is being destroyed. Defaults to C<stop>.

=back

Note that the four values that control the timing and frequency of the event
(C<initial_delay>, C<iterations>, C<iteration_delay> and C<runlength>) may be
overridden by values with the same names in the block of parameters sent to the
event itself (the contents of C<event_params>).

=item Exceptions

The following exceptions may be triggered by this method:

=over

=item Moose Exceptions

The only exceptions that can occur inside of B<new> are type-verification
exceptions or exceptions that stem from an unknown parameter. These exceptions
will be thrown by the Moose system, so they will not match
B<NATE::BaseException>. They will all match B<Moose::Exception> in an C<isa>
test, however. Alternately, you can catch them with the C<otherwise> block of a
C<try> construct.

=back

=back

=cut

# This is the post-constructor hook that Moose will call for each EventWorker
# object that gets created. At this point, the object exists and all parameters
# passed in to new() have been made into attributes.
#
# For this class, this hook creates and stores the NATE::Process object that
# will do the remote execution.
sub BUILD {
    my $self = shift;

    my $process = NATE::Process->new(
        codespec    => $REMOTE_CODE,
        runid       => $self->runid,
        args        => $self->_freeze_event_params,
        client      => $self->client,
        independent => 1,
        ($self->username ? (username => $self->username) : ()),
        ($self->password ? (username => $self->password) : ()),
        oninit => sub {
            my $proc = shift;

            if ($self->is_running) {
                my $runid  = basename $self->runid;
                my $client = $self->client;
                my $event  = $self->event;

                NATE::BaseException->throw(
                    "Attempt to start worker (client $client, event $event, " .
                    "runid $runid) when already running."
                );
            } elsif ($self->finished) {
                my $runid  = basename $self->runid;
                my $client = $self->client;
                my $event  = $self->event;

                NATE::BaseException->throw(
                    "Attempt to start worker (client $client, event $event, " .
                    "runid $runid) when it has already run."
                );
            }

            $self->_set_started([ gettimeofday ]);
            if (my $callback = $self->on_start) {
                $callback->($self, 'start');
            }

            return;
        },
        onexit => sub {
            my $proc = shift;

            #my $worst = $proc->worst_result();
            my $res = ($proc->results())[0];
            my $worst = $res->worst();
            #$Log->comment( "WORST:" . Dumper $worst);

            # We want all results, good or bad, to be non-abortive:
            for my $result ($proc->results) {
                $result->abortive(0);
                $result->store;
                #$Log->log($result);
            }

            if (!$worst) {
                $Log->comment( "no worst event? Generate one..." . Dumper $worst);
                $worst = NATE::Result::Pass->new("no worst event, equating to PASS...");
            }

            #some weird bug where the existing pass doesn't propagate up, but a new
            #pass object will...?
            if ($worst->isa('NATE::Result::Pass')) {
				#$Log->comment( "WORST:is pass");
                $worst = NATE::Result::Pass->new("generated - no errors in events");
            }
			#$Log->comment("log worst now...");
            $Log->log($worst);

            if (my $callback = $self->on_finish) {
                $callback->($self, 'finish');
            }
            $self->_set_ended([ gettimeofday ]);
            $self->_set_finished(1);

            return;
        }
    );

    $self->_set_process_handle($process);

    return;
}

# This is the hook that Moose will call when an object of this class is being
# destroyed.
#
# In this case, we have to see if the NATE::Process handle is still running, and
# if so decide what to do about it (based on the value of "process_on_destroy").
sub DEMOLISH {
    my ($self, $global_destruction) = @_;
    my %known = map { $_ => 1 } (qw(stop kill detach));

    if ($self->is_running) {
        my $disposition = $self->process_on_destroy;

        if (! $known{$disposition}) {
            $disposition = 'stop';
        }

        if ($disposition eq 'stop') {
            # Tell NATE::Process to stop the remote process, then wait for it
            # to complete.
            $self->stop;
            $self->wait;
            # Free up any circular references, etc.
            $self->process_handle->destroy;
        } elsif ($disposition eq 'kill') {
            # All exits, no waiting. KILL the process by raising a signal on
            # it.
            $self->process_handle->raise('KILL');
            # Free up any circular references, etc.
            $self->process_handle->destroy;
        }
        # Do nothing for 'detach'. As-is, the process will continue on.
    } else {
        # If it isn't running, just call the destroy method to free up any
        # resources.
        $self->process_handle->destroy;
    }

    return;
}

=head2 start

    $worker->start;

Start the remote child process on the previously-specified client machine.
Takes no arguments and returns the object referent.

=cut

sub start {
    my $self = shift;

    $self->process_handle->start;

    return $self;
}

=head2 is_running

    if ($worker->is_running) {
        $Log->comment(
            sprintf 'Worker ID #%d is still running...', $worker->id
        );
    }

Determines whether the child process is currently running. Takes no
arguments and returns either a true (1) or false (0) indicating that the
process is (or is not) running.

=head2 stop

    $worker->stop;

Stops the child process on the client machine. Runs asynchronously, so it
returns immediately. Takes no arguments and returns the object referent. See
B<NATE::Process> for specifics on how a process is stopped.

=cut

sub stop {
    my $self = shift;

    $self->process_handle->stop;

    return $self;
}

=head2 is_stop_completed

    if ($worker->is_stop_completed) {
        $Log->comment(
            sprintf 'Worker ID #%d not yet stopped...', $worker->id
        );
    }

Determines whether the previously-made C<stop> request has completed. Takes no
arguments and returns either a true (1) or false (0( indicating that the
process is (or is not) actually stopped.

=head2 wait

    # Start the worker and immediately go into the wait-loop:
    $worker->start->wait;

Wait for the child to complete, returning either when the child has completed
or when the requested timeout has been reached.

=over

=item Options

The following option is recognized by B<wait>:

=over

=item C<< timeout => $integer >>

How long to wait for the child to finish before throwing an exception. The
default value is -1, which indicates no timeout.

=back

=item Exceptions

If the timeout is reached, B<NATE::Process> will exit via B<die>. If you have
the call to C<wait> in a try-catch block, it will be converted to a
B<NATE::BaseException>. However, in some cases B<NATE::Process> will B<die>
with a B<NATE::Result>-based object as the value. The B<try> command will not
convert this to an exception object (since it is already an object).

Also, if you pass an unknown parameter (or non-integer value for C<timeout>) a
B<NATE::Exceptions::Argument> exception will be thrown.

=back

The return value is the object referent.

=cut

sub wait {
    my $self = shift;
    my %opts = validate(
        @_,
        {
            timeout => { type => SCALAR, regex => qr/^\d+$/, optional => 1 },
        }
    );

    $self->process_handle->wait(%opts);

    return $self;
}

=head2 elapsed

    if ($worker->is_running) {
        $Log->comment(
            'Worker for event "%s" still running after %.3f seconds',
            $worker->event, $worker->elapsed
        );
    } elsif ($worker->finished) {
        $Log->comment(
            'Worker for event "%s" ran for a total of %.3f seconds',
            $worker->event, $worker->elapsed
        );
    }

Returns the elapsed time since the worker's B<start> method was called. If the
worker has completed, the value reflects the total time spent in operation.
The tracking of execution time is done with B<Time::HiRes>, so the value you
get here is represented to an average of six decimal places.

=cut

sub elapsed {
    my $self = shift;

    my $ended   = $self->ended ? [ $self->ended ] : undef;

    return tv_interval([ $self->started ], $ended);
}

###############################################################################
# Private methods, not documented to the user.
###############################################################################

###############################################################################
#
#   Sub Name:       _freeze_event_params
#
#   Description:    Create and return an arrayref of the contents of the
#                   event_params attribute that has been "frozen" via careful
#                   use of Data::Dumper.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object of this class
#
#   Returns:        array reference
#
###############################################################################
sub _freeze_event_params {
    my $self = shift;
    my @new_params;

    my %params = $self->event_params;
    local $Data::Dumper::Purity = 1;
    local $Data::Dumper::Terse  = 1;
    local $Data::Dumper::Indent = 0;

    # Add the attributes that are marked for pass-through. Doing it this way
    # makes sure that anything that the user may have passed in as a parameter
    # overrides the attributes.
    for my $attr (@PASS_ATTRIBUTES) {
        if ($attr eq 'class' || $attr eq 'event') {
            # These two parameters cannot be overridden by the user, so we
            # always overwrite:
            $params{$attr} = $self->$attr;
        } else {
            $params{$attr} //= $self->$attr;
        }
    }

    # Scrub the params hash that we currently have:
    my $new_params = _scrub_value(\%params);

    # At this point, the only references anywhere within $new_params should be
    # scalarrefs, hashrefs or arrayrefs. Flatten $new_params into a list, with
    # the refs frozen via Data::Dumper.
    while (my ($key, $value) = each %{$new_params}) {
        my $type = ref $value;

        if (! $type) {
            # Anything non-ref, leave unchanged
            push @new_params, $key, $value;
        } else {
            # The above settings for Data::Dumper will create the smallest
            # (non-compressed) string that can still be eval'd back into the
            # original data. Saving the type info with the data allows a
            # minimal sanity-check on the remote end when thawing.
            my $data = Dumper($value);
            push @new_params, $key, "$type=$data";
        }
    }

    return \@new_params;
}

###############################################################################
#
#   Sub Name:       _scrub_value
#
#   Description:    "Scrub" a value, turning objects of certain classes (from
#                   NACL) into scalar values instead. Also makes copies of
#                   nested array and hash reference values. Will discard and
#                   warn about values that cannot be scrubbed.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $value    in      any       Value to be scrubbed
#
#   Globals:        $Log
#                   %CLASS2NAME
#
#   Returns:        Success: New(ish) value
#                   Failure: throws exception
#
###############################################################################
sub _scrub_value {
    my $value = shift;
    my $type  = ref $value;

    if (blessed $value) {
        # If it is an object, check to see if it is one of the known NACL
        # classes. If so, use the specified method to get the "name" of the
        # object for the remote side to use in reviving the object. If it
        # isn't a known class, for now just throw an exception.
        if (exists $CLASS2NAME{$type}) {
            my $method = $CLASS2NAME{$type};
            $value = $value->$method;
        } else {
            NATE::BaseException->throw(
                "Value meant for remote use was an object of class $type, " .
                'which cannot currently be serialized for transport.'
            );
        }
    } elsif ($type eq 'HASH') {
        my $iceking_value = _scrub_iceking_value($value);
        if (defined $iceking_value) {
            # This was one of the weird hash representations that IceKing
            # creates for some of its resources. The value of $iceking_value
            # is the value we need.
            $value = $iceking_value;
        } else {
            # A regular hashref, which needs to be copied as the individual
            # values themselves are scrubbed.
            $value = { map {
                $_ => _scrub_value($value->{$_})
            } (keys %{$value}) };
        }
    } elsif ($type eq 'ARRAY') {
        # An arrayref needs to be copied as the contents themselves are
        # scrubbed:
        $value = [ map { _scrub_value($_) } @{$value} ];
    } elsif ($type eq 'SCALAR') {
        # A scalar ref should just be copied:
        my $newval = ${$value};
        $value = \$newval;
    } elsif ($type) {
        NATE::BaseException->throw(
            "Value meant for remote use was a reference of type '$type', " .
            'which cannot be properly serialized for transport.'
        );
    }

    # Note that we do nothing if $value isn't a ref. We don't need to.

    return $value;
}

###############################################################################
#
#   Sub Name:       _scrub_iceking_value
#
#   Description:    Look at a hashref and try to determine if it is a "special"
#                   IceKing representation of an object. If so, reduce it to
#                   the value that needs to be returned by _scrub_value for
#                   the sake of _freeze_event_params.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $value    in      any       Hashref to examine
#
#   Returns:        Success: New(ish) value
#                   Failure: throws exception
#
###############################################################################
sub _scrub_iceking_value {
    my $value = shift;
    my $new_value;

    # No need to test for validity, this is only called from within
    # _scrub_value()...

    # Test for a NACL representation:
    if ((exists $value->{name}) && (exists $value->{objects}) &&
            (exists $value->{objects}->{$value->{name}}) &&
            ($value->{objects}->{$value->{name}}->isa('NACL::C::Component'))) {
        $new_value = $value->{name};
    }

    # If none of the tests above matched then we'll return undef, which will
    # tell the caller that this wasn't an IceKing structure.
    return $new_value;
}

###############################################################################
#
#   Sub Name:       _build_id
#
#   Description:    Return a value that can be used as this object's ID
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object of this class
#
#   Returns:        int value
#
###############################################################################
sub _build_id {
    my $self = shift;

    return refaddr $self;
}

=head1 PASSING ARGUMENTS TO EVENTS

When a worker is created, it is given any arguments that should be passed to
the invocation of the event on the remote client. These parameters have to be
encoded for transport from the client running the test to the client that will
be running the event. Because of this, certain things (such as open file
handles or remote connections) cannot be directly passed to an event.

When the worker instance initializes the B<NATE::Process> object that it
encapsulates, it encodes the content of the B<event_params> attribute in a
way that can be cleanly transmitted to the remote client that will execute
the event. During the process of encoding, certain NACL objects are
"converted" (downgraded) to strings containing the resource's name. The
event on the remote client will receive these scalar values in place of the
objects that were specified when the worker was created.

The following NACL classes are converted to names when encountered:

    NACL::C::Aggregate    NACL::STask::Aggregate (also ::StorageAggregate)
    NACL::C::Client       NACL::STask::Client
    NACL::C::Cluster      NACL::STask::Cluster
    NACL::C::Lun          NACL::STask::Lun
    NACL::C::Node         NACL::STask::Node
    NACL::C::Snapshot     NACL::STask::Snapshot
    NACL::C::Volume       NACL::STask::Volume
    NACL::C::Vserver      NACL::STask::Vserver

Any objects that are not from one of these classes will cause an exception to
be thrown during argument encoding. Likewise, any plain reference that is not
SCALAR, ARRAY or HASH will also trigger an exception.

Arguments are decoded before being passed to the event. Events do not have to
deal with decoding parameters, though they will have to convert names that
represent nodes, volumes, etc. back into NACL objects.

=head1 EVENT ITERATION LOOP

When the C<start> method is called, the EventWorker instance sends code to
the chosen client machine via STAF. This code contains the processing loop
that will be used to invoke the chosen remote event. This processing loop is
the I<event iteration loop>.

=head2 Timing Control

The event iteration loop utilizes the C<initial_delay>, C<iterations>,
C<iteration_delay> and C<runlength> attributes of the worker instance. These
values fulfill the following roles:

=over

=item B<initial_delay>

Before the first iteration of the event (and only the first), if this value is
non-zero the worker will wait that many seconds.

=item B<iterations>

If this value is not zero, then it dictates the number of iterations the
event will be put through before the remote process stops execution and
signals the worker instance that it is completed. If C<runlength> is set,
this will always be zero. If both values were passed to the worker creation,
C<runlength> will take precedence.

=item B<runlength>

If this value is not zero, then it dictates the length of time for which the
event will run in a continuous loop. If this value is set, then B<iterations>
will always be zero.

Note that if an initial delay is set with C<initial_delay>, this counts as part
of the overall run-time; something configured to run for 5 minutes with a 2
minute initial delay would have an effective available run-time of only 3
minutes.

=item B<iteration_delay>

If this value is not zero, then each iteration of the event will wait for this
many seconds before starting the next iteration. This delay will not be applied
on the last iteration.

If the run is governed by C<runlength> rather than by C<iterations>, then the
delay is applied after every iteration I<until> an iteration takes the process
past the end-time dictated by C<runlength>. That will be the last iteration,
and no additional delay is applied.

=back

In addition to the timing control, the event iteration loop also manages the
information passed down to the event itself.

=head2 Basic Algorithm

The basic algorithm follwed by the remote code is this:

    Collect paramaters from network caller
    Remove meta-parameters (runlength, etc.)
    Convert remaining parameters from network form to Perl data
    Determine whether using runlength or iterations
    Create event object
    Record start-time
    Record start of event processing
    If (initial_delay > 0)
        Snooze for initial_delay seconds
    Endif
    While (1)
        Increment iteration counter
        Set iteration property on event object to value of counter
        Invoke event
        If (event-level exit requested)
            Exit loop
        Else if ((runlength) and (elapsed time > runlength))
            Exit loop
        Else if (iteration counter > iterations)
            Exit loop
        Else
            If (iteration_delay > 0)
                Snooze for iteration_delay seconds
            Endif
        Endif
    Endwhile
    Record end of event processing
    Create NATE::Result object
    Log result

Note that an event-level exit request is I<not> an exception occurring. When an
exception occurs, it is logged and the event loop attempts to continue. At the
end of processing the total number of caught exceptions will be reported in a
B<NATE::Result::Fail> result object. If there were no exceptions, the result
will be a B<NATE::Result::Pass> object.

An event requests an early exit from the event iteration loop by returning
C<undef> as a return value. Any other return value (including zero) will not
interrupt the loop.

=head2 Logging Within Events

Since the event code is being executed in a NATE/ntest environment, it has
access to the B<NATE::Log> package. From there you can import the B<log_global>
method or use the constructor to build your own logging handle.

=head1 DEVELOPING EVENTS AND EVENT CLASSES

The EventWorker is not much use without collections of events to invoke. The
B<NACL::AppBuilder::DpgWorkflows::CoreEvents> class provides the basic (core)
events that were specified initially. You can add to this class, or create new
classes (provided they implement the needed interface). See the section titled
"DEVELOPING EVENTS AND EVENT CLASSES" in the
B<NACL::AppBuilder::DpgWorkflows::EventBase> man page for coverage of writing
events and event classes.

=head1 SEE ALSO

B<NACL::AppBuilder::DpgWorkflows::CoreEvents>,
B<NACL::AppBuilder::DpgWorkflows::EventManager>, B<NATE::Process>

=head1 AUTHOR

Randy J. Ray E<lt>rjray@netapp.comE<gt>

=cut

1;

__DATA__
{
    use Module::Load;
    use Const::Fast;
    use Tharn;
    use NATE::Log qw(log_global);
    use NATE::Result;
    use NATE::BaseException qw(:try);
    use NACL::AppBuilder::DpgWorkflows::CoreEvents;

    # This list *must* match the const list of the same name at the top of this
    # module:
    const my @PASS_ATTRIBUTES =>
        qw(
              class
              event
              client
              interface
              initial_delay
              iteration_delay
              iterations
              runlength
      );
    my %opts = @_;

    my $Log = log_global;
    # These are the parameters that are used locally, so they are removed from
    # the %opts table entirely. They are not passed down to the event that
    # get eventually called.
    my %params;
    for my $param (@PASS_ATTRIBUTES) {
        $params{$param} = delete $opts{$param};
    }

    # Attempt to load the specified events class, then verify that it provides
    # the interface we need (and that it actually has the specified event).
    my $class = $params{class};
    my $event = $params{event};
    load $class;

    # The class must be a user of this Moose role in order for the worker to
    # correctly interact with instances:
    if (! $class->DOES('NACL::AppBuilder::DpgWorkflows::EventBase')) {
        NATE::BaseException->throw(
            "Event class '$class' is missing the EventBase role"
        );
    }
    # Also, it would be nice if the class actually has the requested method:
    if (! $class->can($event)) {
        NATE::BaseException->throw(
            "Event class '$class' has no method named '$event'"
        );
    }

    # Un-freeze any parameters that were frozen for transport:
    for my $key (keys %opts) {
        if ($opts{$key} =~ /^([A-Za-z_]\w*(?:::\w+)*)=(.*)$/) {
            my $type = $1;
            my $data = eval $2;

            if (ref($data) ne $type) {
                $Log->warn(
                    "Preserved parameter '$key' did not eval to type " .
                    "'$type'. Discarded for sake of security."
                );
                delete $opts{$key};
            } else {
                $opts{$key} = $data;
            }
        }
    }

    my $iterations      = $params{iterations}      || 1;
    my $runlength       = $params{runlength}       || 0;
    my $initial_delay   = $params{initial_delay}   || 0;
    my $iteration_delay = $params{iteration_delay} || 0;
    my $interface       = $params{interface}       || 'ZAPI';
    my $client          = $params{client};
    my $message = qq(On client $client, running event "${class}::$event" for );
    if ($runlength) {
        $message .= "$runlength seconds, with a delay of $iteration_delay " .
            sprintf('second%s ', $iteration_delay == 1 ? q{} : 's') .
            'between iterations.';
    } else {
        $message .= sprintf(
            '%d iteration%s', $iterations, $iterations == 1 ? q{} : 's'
        );
        if ($iterations == 1) {
            $message .= q{.};
        } else {
            $message .= ", with a delay of $iteration_delay " .
                sprintf('second%s', $iteration_delay == 1 ? q{} : 's') .
                'between iterations.';
        }
    }
    $Log->comment($message);

    my $exceptions = 0;
    my $iteration  = 0;
    if ($runlength) {
        # The "runlength" parameter supercedes "iterations". Setting $iterations
        # to 0 will prevent $iteration from catching up to it any time before
        # $runlength has run its course.
        $iterations = 0;
    }

    my $start_time = time;
    my $exit_time  = $start_time + $runlength;
    my $event_handle = NACL::AppBuilder::DpgWorkflows::CoreEvents->new(
        client         => $client,
        interface      => $interface,
        event          => $event,
        max_iterations => $iterations,
        runlength      => $runlength,
        start_time     => $start_time
    );

    # If they asked for an initial delay, do that now:
    if ($initial_delay) {
        $Log->comment("Executing an initial delay of $initial_delay seconds.");
        Tharn::snooze($initial_delay);
    }

    while (1) {
        $iteration++;
        my $exit_requested;

        try {
            $event_handle->iteration($iteration);
            my $retval = $event_handle->run(%opts);
            if (! defined $retval) {
                $exit_requested++;
            }
        } catch NATE::BaseException with {
            my $e = shift;

            # Report the error to the log, but don't exit. At the end, we'll
            # log a final result of pass/fail based on whether $exceptions is
            # zero.
            my $type = ref $e;
            $exceptions++;
            $Log->error(
                "Exception of class $type caught during iteration $iteration:" .
                "\n$e"
            );
        };

        if ($exit_requested ||
                ($runlength && ($exit_time < time)) ||
                ($iterations && ($iteration >= $iterations))) {
            last;
        }

        if ($iteration_delay) {
            Tharn::snooze($iteration_delay);
        }
    }

    $message = sprintf(
        '%d iteration%s', $iteration, $iteration == 1 ? q{} : 's'
    );
    $message .= " completed. Closing remote execution on $params{client}.";
    $Log->comment($message);

    my $result;
    if ($exceptions) {
        $result = NATE::Result::Fail->new(
            sprintf(
                '%d exception%s caught over %d iteration%s.',
                $exceptions, $exceptions == 1 ? q{} : 's',
                $iteration,  $iteration  == 1 ? q{} : 's'
            )
        );
    } else {
        $result = NATE::Result::Pass->new(
            sprintf(
                '%d iteration%s run, no errors detected.',
                $iteration, $iteration == 1 ? q{} : 's'
            )
        );
    }

    $Log->log($result);
}
