# -*-perl-*-
#
# Copyright (c) 2015,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DPG Fiji
## @author ng-smgr-auto@netapp.com
## @status shared
## @pod here

package NACL::AppBuilder::DpgWorkflows::Fiji;

use strict;
use warnings;

use Params::Validate qw(validate_with :types);
use base qw(Exporter);

use NATE::BaseException qw( :try );
use NATE::Log qw(log_global);

use NACL::STask::Fiji;
use NACL::CS::Fiji;

our @EXPORT_OK = qw(
        fiji_clear_all
        fiji_wait
        fiji_set_delay_promote_of_pcs_to_tcs
        fiji_set_pause_before_lrs
        fiji_set_ift_create_fail_during_pcs
        fiji_delay_volmove_transfer_cutover
        fiji_delay_smgr_asynctransfer_cutover
        fiji_delay_local_rollback_of_lrs_to_pcs
        fiji_delay_IFT_resync
        fiji_delay_pcs_create_in_ndo
        fiji_pcs_addtags_error
        fiji_ift_delete_fail
        fiji_mw_hecr_out_of_order_replops
        fiji_unsplit_clones
        fiji_mw_force_decompression
        fiji_pause_protect_after_get_vol_info
        fiji_pause_protect_after_vol_create
        fiji_pause_auto_init
        fiji_wafl_lsa_punch_holes_worker_delay
        fiji_wafl_lsa_indirects_punch_holes_completion_delay
        fiji_pause_init_before_xfer
);

our %EXPORT_TAGS = (
    'all' => \@EXPORT_OK,
    );

my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

=head1 NAME

NACL::AppBuilder::DpgWorkflows::Fiji

=head1 DESCRIPTION

This library contains Fiji's.

=head1 METHODS

=cut

=head2 fiji_clear_all

    Will clear all fijis on node.

=over

=item options

=over

=item node

(Required) Node Object.

=back

=back

=cut

sub fiji_clear_all {
    my %args = validate_with(
        params => \@_,
        allow_extra => 0,
        spec => {
            node => { type => OBJECT, },
        },
    );

    my $node=$args{node};
    NACL::STask::Fiji->clear(command_interface => $node,
                             nacltask_fiji_clear_on_all_nodes => 1,
                             slot              => 'all');

}

=head2 fiji_wait

    Wait for fiji counter to be drained.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item timeout

(Optional) Scalar. Timeout for waiting FIJI being hit.

=back

=back

=cut

sub fiji_wait {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, },
            timeout  => { type => SCALAR, default => '1200'},
        },
    );

    my $node=$args{node};
    $log->comment('Wait for the fiji counter to become non-zero');
    my $count_initial = 0;
    my $count_final   = 0;
    my $timeout = time + $args{timeout};
    do {
        my $fiji_state = NACL::CS::Fiji->fetch( command_interface => $node,
                                            requested_fields=> [ qw(count)], );
        $count_final = $fiji_state->count() if ($fiji_state->count());
    } while ($count_final == $count_initial && time < $timeout);
    $log->comment('fiji counter: ' . $count_final);
    return $count_final;
}

=head2 fiji_set_ift_create_fail_during_pcs
    Method to fail IFT creation during PCS.
    Will return the Fiji object.

=over

=item options

=over

=item node
    (Required) Node Object.

=item repeat
    (Optional) The number of times you want the fiji to be hit during the PCS
    creation process.
    (Default) is 4 because internally PCS is attempted thrice.

=back

=back

=cut

sub fiji_set_ift_create_fail_during_pcs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, },
            repeat => { type => SCALAR, default => 4 },
        },
    );

    my $repeat = $args{repeat};
    my $node=$args{node};

    my $fiji_str = "\'(repeat 4 (repl.inject.error(eq \@key repl_log_cg_init_log) (return true)))\'";
    $log->comment('Setting fiji : ' . $fiji_str . ' on node: ' . $node->name());

    my $fiji_obj =  NACL::STask::Fiji->new(
                        command_interface => $node,
                        tag               => 'IFT_CREATE_FAIL',
                         program          => $fiji_str);
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_set_delay_promote_of_pcs_to_tcs

    Method to introduce delay during promote of PCS to TCS

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item delay

(Required) Delay in sec to be introduced. Default: 60.

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=item relId

(Required) smgr ID. Default: 1.

=back

=back

=cut

sub fiji_set_delay_promote_of_pcs_to_tcs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, },
            delay  => { type => SCALAR, default => 60 },
            repeat => { type => SCALAR, default => 1 },
            relId  => { type => SCALAR, default => 1 },
        },
    );

    my $delay=$args{delay};
    my $repeat=$args{repeat};
    my $node=$args{node};
    my $rel_Id=$args{relId};
    $log->comment('Set fiji to introduce delay of '.$delay.'sec during promotion of PCS to TCS');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to introduce delay during promote',
                    program => "\'(repeat $repeat (repl.relationship.action (&& (eq \@key 'snap_repl_pcs_primary_snapshot_delay') (eq \@relId $rel_Id)) (sleep $delay)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_set_pause_before_lrs

    Pausing the workflow after the event just before the event that creates the LRS(pre xfer afs snapshot).
    That event is this emptiness check event.

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item delay

(Required) Number of sec to pause. Default: 60.

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=back

=back

=cut

sub fiji_set_pause_before_lrs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, },
            delay => { type => SCALAR, default => 60 },
            repeat => { type => SCALAR, default => 1 },
        },
    );

    my $node=$args{node};
    my $delay=$args{delay};
    my $repeat=$args{repeat};
    $log->comment('Set fiji to pause for '.$delay.'sec before LRS(pre xfer afs ss) is created');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to introduce delay during promote',
                    program => "\'(repeat $repeat (rpl.suspend.work.done((eq \@function sm_wf_cggrs_as_init_resync_check_dst_items_emptiness_post_dowork)) (return $delay)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}


=head2 fiji_delay_local_rollback_of_lrs_to_pcs

    This fiji is set on dst node to introduce delay during local rollback of LRS to last PCS.

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Destination Node.

=item delay

(Required) Number of sec to pause. Default: 60.

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=back

=back

=cut

sub fiji_delay_local_rollback_of_lrs_to_pcs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, },
            delay => { type => SCALAR, default => 60 },
            repeat => { type => SCALAR, default => 1 },
        },
    );

    my $node=$args{node};
    my $delay=$args{delay};
    my $repeat=$args{repeat};
    $log->comment('Set fiji to pause for '.$delay.'sec during local rollback of LRS to last PCS');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to introduce delay during rollback',
                    program => "\'(repeat $repeat (repl.writer.lrse.replop.action(eq \@optype 31) (sleep $delay)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_delay_IFT_resync

    This fiji is set on src node to introduce delay during IFT resync.

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item delay

(Required) Number of sec to pause. Default: 60.

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=back

=back

=cut

sub fiji_delay_IFT_resync {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, },
            delay => { type => SCALAR, default => 60 },
            repeat => { type => SCALAR, default => 1 },
        },
    );

    my $node=$args{node};
    my $delay=$args{delay};
    my $repeat=$args{repeat};
    $log->comment('Set fiji to pause for '.$delay.'sec during IFT resync');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to introduce delay during IFT resync',
                    program => "\'(repeat $repeat (repl.action((eq \@key repl_ift_session_gen_replop_delay)) (sleep $delay)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_unsplit_clones

    This fiji is set on src node to do unsplit clones.

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item fsid

(Required) file system id of volume.

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=back

=back

=cut

sub fiji_unsplit_clones {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, optional=>0 },
            fsid  => { type => SCALAR, optional=>0 },
            repeat => { type => SCALAR, default => 1 },
        },
    );

    my $node=$args{node};
    my $fsid=$args{fsid};
    my $repeat=$args{repeat};
    $log->comment('Set fiji to pause for '.$fsid .'to capture unsplit clones');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to capture unsplit clones',
                    program => "\'(repeat $repeat (wafl.sis.fixup.skip.vol (== \@fsid $fsid) (return 1)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_delay_volmove_transfer_cutover

This method is to set or clear fault/fiji injected for volmove prolong process

=over

=item options

=over

=items required

(required) nodes - ARRAYREF, node objects to set/clear fault-injection
(required) point - process point: transfer or cutover
(required) action - set or clear the fault-injection

=items optional

(optional) sleep - delay time

=back

=back

=cut


sub fiji_delay_volmove_transfer_cutover {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            sleep  => { type => SCALAR, optional => 1, default => '300', },
            point  => { type => SCALAR, optional => 0, regex => qr/transfer|cutover/, },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
        },
    );

    my $sleep = $args{sleep};

    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            if ($args{point} eq 'transfer'){
                my $volmove_point = "mgwd.vopl.move.panicordelayinmonitortransfer";
                my $fault_mode    = "GX_FAULT_MODE_ONCE";
                my $type          = "FAULT_ACTION_SLEEP";

                my $cmd =
                    "debug fault-injection set-fault-mode -node local -fault-id $volmove_point -fault-mode $fault_mode -action $type -action-arg $sleep";
                $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

                $cmd = "debug fault-injection enable-fault -node local -fault-id $volmove_point";
                $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

            } else {
                my $volmove_fiji_delay = NACL::STask::Fiji->new(
                        command_interface => $node,
                        tag               => 'volmove_delay',
                        program           =>
                            "\'(move_agent.delay.post.commit.source (eq 1 1) (sleep $sleep))\'",
                );
                $volmove_fiji_delay->set('nodescope-node-name' => $node->node());
            }
        } else { #clear
            if ($args{point} eq 'transfer'){
                #clear FIJI won't stop the current delay.
                my $volmove_point = "mgwd.vopl.move.panicordelayinmonitortransfer";
                my $cmd = "debug fault-injection disable-fault -node local -fault-id $volmove_point";
                $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );
            } else {
                NACL::STask::Fiji->clear(
                    command_interface => $node,
                    slot              => "all",
                );
            }
        }
    }
}


=head2 fiji_delay_smgr_asynctransfer_cutover

This method is to set or clear fiji injected for smgr prolong process

=over

=item options

=over

=items required

(required) nodes - ARRAYREF, node objects to set/clear fault-injection
(required) point - process point: transfer or cutover
(required) action - set or clear the fault-injection

=items optional

(optional) sleep - delay time

=back

=back

=cut


sub fiji_delay_smgr_asynctransfer_cutover {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            sleep  => { type => SCALAR, optional => 1, default => '300', },
            point  => { type => SCALAR, optional => 0, regex => qr/transfer|cutover/, },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
        },
    );

    my $sleep = $args{sleep};

    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            my $program = ($args{point} eq 'cutover')
            ? "\'(repeat 1 (repl.action (eq \@key drl_resync_start_delay) (sleep $sleep)))\'"
            : "\'(repeat 1 (rpl.introduce.delay(eq \@keyword sm_wf_cggrs_as_init_resync_start_source_cg_sync_te_rmt_dowork) (return $sleep)))\'";

            my $smgr_fiji_delay = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'smgr_delay',
                    program           => $program,
            );

            $smgr_fiji_delay->set('nodescope-node-name' => $node->node());

        } else { #clear
            NACL::STask::Fiji->clear(
                command_interface => $node,
                slot              => "all",
            );
        }
    }
}



=head2 fiji_delay_pcs_create_in_ndo

This method is to set or clear fiji injected to delay PCS create in NDO events

=over

=item options

=over

=items required

(required) nodes - ARRAYREF, node objects to set/clear fault-injection
(required) point - process point: transfer or cutover
(required) action - set or clear the fault-injection

=items optional

(optional) sleep - delay time

=back

=back

=cut


sub fiji_delay_pcs_create_in_ndo {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, optional => 0 },
            sleep  => { type => SCALAR, optional => 1, default => '300', },
        },
    );

    my $node  = $args{node};
    my $sleep = $args{sleep};

    $node->refresh_command_interface();

    my $program = "\'(repeat 1 (repl.action(eq \@key snap_repl_pcs_primary_snapshot_delay) (sleep $sleep)))\'";

    my $fiji_obj = NACL::STask::Fiji->new(
            command_interface => $node,
            tag               => 'fiji_delay_pcs_create_in_ndo',
            program           => $program,
    );

    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_pcs_addtags_error

This method is to set the fiji which causes an error while applying tagson the pcs ss.

=over

=item options

=over

=items required

(required) node - ARRAYREF, node objects to set/clear fault-injection
(optional) count - SCALAR, the # of times to repeat the fiji

=back

=back

=cut


sub fiji_pcs_addtags_error {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, optional => 0 },
            count  => { type => SCALAR, optional => 1, default => '1', },
        },
    );

    my $node  = $args{node};
    my $count = $args{count};

    $node->refresh_command_interface();

    my $program = "\'(repeat $count (repl.action (eq \@key snap_repl_pcs_addtags_error) (return true)))\'";

    my $fiji_obj = NACL::STask::Fiji->new(
            command_interface => $node,
            tag               => 'fiji_addtags_error',
            program           => $program,
    );

    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}


=head2 fiji_ift_delete_fail

This method is to set fiji that fails the deletion of IFT

=over

=item options

=over

=item node
    (Required) Node Object.

=back

=back

=cut


sub fiji_ift_delete_fail {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, },
        },
    );

    my $node=$args{node};

    #my $fiji_str = "\'(repeat 4 (repl.inject.error(eq \@key repl_log_cg_init_log) (return true)))\'";
    my $fiji_str= "\'(repl.inject.error(eq \@key 'repl_log_cg_delete_log')(return true))\'";
    $log->comment('Setting fiji : ' . $fiji_str . ' on node: ' . $node->name());

    my $fiji_obj =  NACL::STask::Fiji->new(
                        command_interface => $node,
                        tag               => 'IFT_DELETE_FAIL',
                         program          => $fiji_str);
    $fiji_obj->set('nodescope-node-name' => $node->node());
    return $fiji_obj;
}


=head2 fiji_mw_hecr_out_of_order_replops

    Method to to force the replops out of order.

    This fiji needs to be enabled before beginning the xfer with destination volume <dsid> and <optype>.
    This would buffer the ops for the transfer involving the destination volume <dsid> of <optype> and
    replays them right before replaying clone stitch ops. We also checkpoint this buffer file when this
    fiji is enabled

    Will return the fiji object.

=over

=item options

=over

=item node

(Required) Node Object. Use Source Node.

=item optype

(Required) replop to delay replay, like REPL_FIJI_BUFFER_PUSH_MULTIPLE_INODES (0x40).

                REPL_FIJI_BUFFER_PUSH_INODE           =  0x1,
                REPL_FIJI_BUFFER_PUSH_HOLES           =  0x2,
                REPL_FIJI_BUFFER_PUSH_FUBN            =  0x4,
                REPL_FIJI_BUFFER_PUSH_FUBN_REBUILD    =  0x8,
                REPL_FIJI_BUFFER_PUSH_NAME_FREE       = 0x10,
                REPL_FIJI_BUFFER_PUSH_DIRENT          = 0x20,
                REPL_FIJI_BUFFER_PUSH_MULTIPLE_INODES = 0x40,
                REPL_FIJI_BUFFER_PUSH_MULTIPLE_FUBNS  = 0x80

=item repeat

(Required) Number of times to repeat the fiji. Default: 1.

=item dsid

(Required) destination volume <dsid> . Default: 1.

=back

=back

=cut

sub fiji_mw_hecr_out_of_order_replops {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, },
            optype => { type => SCALAR, default => '0x40' },
            repeat => { type => SCALAR, default => 1 },
            dsid   => { type => SCALAR, default => 1 },
        },
    );

    my $node = $args{node};
    my $dsid = $args{dsid};
    my $optype = $args{optype};
    my $repeat = $args{repeat};

    $log->comment('Set fiji to force replop ' . $optype . ' replay out of order');
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to to force replop replay out of order',
                    program => "\'(repeat $repeat (repl.receiver.writer.replops.buffer.injection(eq \@dsid $dsid) (return $optype)))\'");

    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}

=head2 fiji_mw_force_decompression

FIJI to fail SEND_BLOCKS messages with EDECOMPRESS error and decompression work flow is triggered in Transfer Engine.

=item options

=item node

(Required) Node Object. Use Source Node.

=item repeat

(Required) Number of times to repeat the fiji. Default: 5

=back

=back

=cut

sub fiji_mw_force_decompression {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, },
            repeat => { type => SCALAR, default => 5 },
        },
    );

    my $node   = $args{node};
    my $repeat = $args{repeat};

    my $partner = NACL::STask::Node->get_partner(command_interface => $node);
    my $partner_obj = NACL::STask::Node->new(name => $partner);
    my $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $partner_obj,
                    tag               => 'Fiji to trigger decompression',
                    program => "\'(repeat $repeat (wafl.ng.compressed.write.force.decompression(true) (return 1)))\'");
    $fiji_obj->set('nodescope-node-name' => $partner_obj->node());

    $fiji_obj = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'Fiji to trigger decompression',
                    program => "\'(repeat $repeat (wafl.ng.compressed.write.force.decompression(true) (return 1)))\'");
    $fiji_obj->set('nodescope-node-name' => $node->node());

    return $fiji_obj;
}


=head2 fiji_pause_protect_after_get_vol_info

FIJI to pause snapmirror protect job after GetVvolInfo.

=item options

(Required) nodes - Node Objects to set FIJI.
(Optional) sleep - time to pause.
(Required) action - set or clear the FIJI.

=back

=back

=cut

sub fiji_pause_protect_after_get_vol_info {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            sleep  => { type => SCALAR, optional => 1, default => '60', },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
        },
    );

    my $sleep = $args{sleep};

    my $protect_point = "GX_FAULT_SM_PAUSE_SetupProtectAfterGetVvolInfo";
    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            my $fault_mode    = "GX_FAULT_MODE_ALWAYS";
            my $type          = "FAULT_ACTION_SLEEP";

            my $cmd =
                "debug fault-injection set-fault-mode -node local -fault-id $protect_point -fault-mode $fault_mode -action $type -action-arg $sleep";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

            $cmd = "debug fault-injection enable-fault -node local -fault-id $protect_point";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

        } else { #clear
            #clear FIJI won't stop the current delay.
            my $cmd = "debug fault-injection disable-fault -node local -fault-id $protect_point";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );
        }
    }
}

=head2 fiji_pause_protect_after_vol_create

FIJI to pause protect job after VolPlaceAndCreate.

=item options

(Required) nodes - Node Objects to set FIJI.
(Optional) sleep - time to pause.
(Required) action - set or clear the FIJI.

=back

=back

=cut

sub fiji_pause_protect_after_vol_create {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            sleep  => { type => SCALAR, optional => 1, default => '60', },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
        },
    );

    my $sleep = $args{sleep};

    my $protect_point = "GX_FAULT_SM_PAUSE_SetupProtectAfterVolPlaceAndCreate";
    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            my $fault_mode    = "GX_FAULT_MODE_ALWAYS";
            my $type          = "FAULT_ACTION_SLEEP";

            my $cmd =
                "debug fault-injection set-fault-mode -node local -fault-id $protect_point -fault-mode $fault_mode -action $type -action-arg $sleep";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

            $cmd = "debug fault-injection enable-fault -node local -fault-id $protect_point";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );

        } else { #clear
            #clear FIJI won't stop the current delay.
            my $cmd = "debug fault-injection disable-fault -node local -fault-id $protect_point";
            $node->apiset(set => 'CMode')->execute_raw_command( command => $cmd, timeout => 120 );
        }
    }
}


=head2 fiji_pause_auto_init

FIJI to pause protect job at auto-init phase.

=item options

(Required) nodes - Node Objects to set FIJI.
(Optional) sleep - time to pause.
(Required) action - set or clear the FIJI.
(Optional) repeat - count to repeat.

=back

=back

=cut

sub fiji_pause_auto_init {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
            sleep  => { type => SCALAR, default => '60', },
            repeat => { type => SCALAR, default => '1' },
        },
    );

    my $sleep = $args{sleep};
    my $repeat = $args{repeat};

    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            my $program =
                "\'(repeat $repeat (rpl.suspend.work.done(eq \@function sm_wf_wait_for_te_notification_post_dowork) (return $sleep)))\'";

            my $smgr_fiji_delay = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'auto_init_pause',
                    program           => $program,
            );

            $smgr_fiji_delay->set('nodescope-node-name' => $node->node());

        } else { #clear
            NACL::STask::Fiji->clear(
                command_interface => $node,
                slot              => "all",
            );
        }
    }
}


=head2 fiji_wafl_lsa_punch_holes_worker_delay

FIJI to pause protect job at truncate phase.

=item options

    (Required) node - Node Objects to set FIJI.
    (Optional) sleep - time to pause.
    (Required) file_inode - inode of the file.
    (Required) volume  - volume name to set FIJI

=back

=back

=cut

sub fiji_wafl_lsa_punch_holes_worker_delay {
	my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node  => { type => OBJECT, optional => 0 },
            sleep  => { type => SCALAR, default => '300' },
            file_inode => { type => SCALAR, optional => 0 },
            volume     => { type => SCALAR , optional => 0 },
        },
    );

	my $sleep = $args{sleep};
    my $node = $args{node};
	my $file_inode = $args{file_inode};
	my $volume = $args{volume};

	my $program =
		"\"(wafl.lsa.punch.holes.worker.delay (&& (eq \@vol $volume) (eq \@fileid $file_inode) (eq \@workerid 2)) (return $sleep))\"";
	my $fiji_delay = NACL::STask::Fiji->new(
			command_interface => $node,
			tag               => "lsa_fiji",
			program           => $program,
	);
	$fiji_delay->set('nodescope-node-name' => $node->node());

	return $fiji_delay;
}

=head2 fiji_wafl_lsa_indirects_punch_holes_completion_delay

FIJI to pause protect job at truncate phase.

=item options

    (Required) node - Node Objects to set FIJI.
    (Optional) sleep - time to pause.
    (Optional) repeat - The number of times you want the fiji to be hit
    (Required) volume  - volume name to set FIJI

=back

=back

=cut

sub fiji_wafl_lsa_indirects_punch_holes_completion_delay {
	my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, optional => 0 },
            sleep  => { type => SCALAR, default => '300' },
            repeat => { type => SCALAR, default => '5' },
            volume => { type => SCALAR , optional => 0 },
        },
    );

	my $sleep = $args{sleep};
    my $node = $args{node};
	my $repeat = $args{repeat};
	my $volume = $args{volume};

	my $program =
		"(repeat $repeat (wafl.lsa.indirects.punch.holes.completion.delay (\&& (eq \@vol $volume)) (return $sleep)))";
	my $fiji_delay = NACL::STask::Fiji->new(
			command_interface => $node,
			tag               => "lsa_fiji",
			program           => $program,
	);
	$fiji_delay->set('nodescope-node-name' => $node->node());

	return $fiji_delay;
}

=head2 fiji_pause_init_before_xfer

FIJI to pause protect initialize before transfer.

=item options

(Required) nodes - Node Objects to set FIJI.
(Optional) sleep - time to pause.
(Required) action - set or clear the FIJI.
(Optional) repeat - count to repeat.

=back

=back

=cut

sub fiji_pause_init_before_xfer {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            nodes  => { type => ARRAYREF, optional => 0 },
            action => { type => SCALAR, optional => 0, regex => qr/clear|set/, },
            sleep  => { type => SCALAR, default => '120'},
            repeat => { type => SCALAR, default => '1' },
        },
    );

    my $sleep = $args{sleep};
    my $repeat = $args{repeat};

    foreach my $node (@{$args{nodes}}) {
        $node->refresh_command_interface();
        if($args{action} eq 'set') {
            my $program =
                "\'(repeat $repeat (rpl.suspend.work.done(eq \@function sm_wf_xfer_init_post_dowork) (return $sleep)))\'";

            my $smgr_fiji_delay = NACL::STask::Fiji->new(
                    command_interface => $node,
                    tag               => 'init_before_xfer',
                    program           => $program,
            );

            $smgr_fiji_delay->set('nodescope-node-name' => $node->node());

        } else { #clear
            NACL::STask::Fiji->clear(
                command_interface => $node,
                slot              => "all",
            );
        }
    }
}

1;
