package NACL::AppBuilder::DpgWorkflows::Shiner;
# Compiler directives
#
use strict;
use warnings;

# Module import.
use Hostrec;
use NACL::APISet;
use Data::Dumper;
use Params::Validate qw(validate_with UNDEF OBJECT HASHREF
    SCALAR SCALARREF ARRAYREF BOOLEAN);

use NATE::Log qw(log_global);
my $Log = log_global();


use Time::HiRes qw(gettimeofday);
use Date::Parse;

use NACL::C::Job;
use NACL::C::JobHistory;
use NACL::C::Client;
use NACL::C::Vserver;
use NACL::C::VolumeSnapshot;
use NACL::C::AVANode;

use NACL::CS::Snapmirror;

use NACL::STask::Snapmirror;
use NACL::STask::Node;
use NACL::STask::Aggregate;
use NACL::STask::Volume;
use NACL::STask::VolumeMove;
use NACL::STask::VolumeClone;
use NACL::STask::VolumeSnapshot;
use NACL::STask::Client::Mount;

use NATE::BaseException qw(:try);
use NACL::Exceptions::VerifyFailure;
use NACL::APISet::Exceptions::ResponseException;
use NACL::APISet::Exceptions::CommandFailedException;

use NACL::MTask::Ping;
use NACL::MTask::Traffic;

use Class::MethodMaker [
    new => [ '-init', 'new' ],
    scalar => [
       qw/ ava log /,
    ],
    scalar => [
        #$qw/ ava_api ava_api_root /,
       qw/ ava_node /,
    ],

];

=head1 NAME

NACL::AppBuilder::DpgWorkflows::Shiner

=head1 SYNOPSIS

 use NACL::AppBuilder::DpgWorkflows::Shiner

=head1 DESCRIPTION

A C<NACL::AppBuilder::DpgWorkflows::Shiner> object is used to
manipulate C<NACL::C::AVANode> objects and provides helper
methods for ontap->altavault (ava)  snapmirrors.

=head1 METHODS

=head2 new()

This is the constructor method.  It returns a new Shiner object and
automatically creates an AVANode object internally (if not passed).

=over

=item Synopsis

  use NACL::AppBuilder::DpgWorkflows::Shiner

  #pass in a NACL::C::AVANode object
  my $shiner = NACL::AppBuilder::DpgWorkflows::Shiner->new(ava => $ava_object);

  #OR pass an ava node name
  my $shiner = NACL::AppBuilder::DpgWorkflows::Shiner->new(ava => 'ava_node_name');

=over

=item Options

=item C<< ava => $ava >>

This can be either a C<NACL::C::AVANode> object or simply
the name of the altavault node.


=back

=back


=cut

my $COLON = q{:};
my $UNDERSCORE = q{_};
my $SLASH = q{/};
my $SPLAT = q{*};


sub init {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'ava'      => { 'type' => SCALAR|OBJECT },
            'enable_snapmirror' => { 'type' => SCALAR, default => 1 },
        },
    );

    $self->log(log_global());  #or a global our?
    my $ava = $opts{ava};

    my $ava_node;

    if (ref $ava) {
        $ava_node = $ava;
        $ava = $ava->name();
        $Log->log("Got object. Using AVA $ava");
    } elsif (defined ($ava)) {
        $Log->log("Creating AVANode object for $ava");
        $ava_node =  NACL::C::AVANode->new( name => $ava );
    } else {
        # find looks at AVANODES, AVA1, AVA2
        $Log->log('Look at AVANODES, AVA1, AVA2.... to find an AVANode'); 
        $ava_node =  NACL::C::AVANode->find();
    }

    $self->ava($ava);
    $self->ava_node($ava_node);


    #my $ava_host =  Tharn::host( $ava );
    #$self->ava_api(_get_ava_apiset($ava,$ava_host));
    #$self->ava_api_root(_get_ava_apiset_root($ava,$ava_host));
    #

    $self->ava_node()->show_version();

    if ($opts{enable_snapmirror}) {
        my $sm_state_result = $self->ava_node()->show_snapmirror_state();

        if ($sm_state_result->get_raw_output() =~ /disabled/) {
            # enable snapmirror and restart service
            $Log->log('Enabling snapmirror');
            $self->ava_node()->snapmirror_enable();
            $Log->log('Restarting service');
            $self->ava_node()->service_restart();

            my $new_sm_state_result = $self->ava_node()->show_snapmirror_state();
            NATE::BaseException->throw('Not able to enable snapmirror successfully')
                if ($new_sm_state_result->get_raw_output() !~ /enabled/);
        } else {
            $Log->log('snapmirror already enabled');
        }
    }

    return;
}

=head2 cleanup_ava()

removes ava share on altavault

  $shiner->cleanup_ava(share_name => $share);


=over

=item Options

=item C<< share_name => $share_name >>

name of share to delete


=back

=cut

sub cleanup_ava {
    my $self = shift;

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            share_name => { type => SCALAR },
        }
    );

    my $ava = $self->ava_node();
    my $share_name = delete $args{share_name};
    $Log->log("cleanup ava share $share_name");
    my $ava_ci = $ava->hostrec;

    #if( $ava_ci->is_connected() == 0 ) {
    #$ava_ci->connect();
    #}

    # there is no error message on non-existant share
    # its not clear if there is an error message on failed delete
    # of an existing share either.
    # TODO: do we list out possible shares and grep for it
    # then delete, list again and verify? ugh.

    $ava_ci->execute(
        command     => "snapmirror share delete share-name $share_name",
        match_table => [
            "All the data in $share_name will be deleted. Are you sure you want to continue? \[y\/N\] "
            => "y",
            "$share_name"
            => "y",
        ]
    );

    return;
} ## end sub _cleanup_ava


=head2 enable_sktraces()

enable sktraces on node.
These should be on before doing
ontap->ava snapmirrors for debugging

  $shiner->enable_sktraces(src_node => $ontap_src);

It can also be called statically
  NACL::AppBuilder::DpgWorkflows::enable_sktraces(src_node => $ontap_src);


=over

=item Options

=item C<< src_node => $src_node >>

Ontap source node in the ontap->ava relationship


=back

=cut

sub enable_sktraces {
    my $self_or_pkg = shift;

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => { src_node => { type => OBJECT }, }
    );

    my $srcNode = delete $args{src_node};
    #my $srcNode = $self_or_pkg->{ava_node};

    $srcNode->get_systemshell_apiset()
        ->execute_raw_command(
        command => 'sudo sysctl sysvar.sktrace.SNAPDIFF_enable=16' );

    $srcNode->get_systemshell_apiset()
        ->execute_raw_command(
        command => 'sudo sysctl sysvar.sktrace.REPL_enable=11' );

    $srcNode->get_systemshell_apiset()
        ->execute_raw_command(
        command => 'sudo sysctl sysvar.sktrace.SM_enable=0x0374' );

} # end of _enable_sktraces

=head2 add_whitelist()

Add whitelist ips on ava node for the snapmirror source
node to allow snapmirror source node to connect to the ava.

reigniter will do this automatically but this can be used to
do it for other scripts.

  $shiner->add_whitelist(src_node => $ontap_src);

Note that this will find the intercluster lifs on the node
so those must already exist.

=over

=item Options

=item C<< src_node => $src_node >>

Ontap source node in the ontap->ava relationship


=back

=cut

sub add_whitelist {
    my $self = shift;

    my %opts = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            src_node           => { type => OBJECT },
            inhibit_reboot     => { type => SCALAR, default => 0 },
            inhibit_whitelist  => { type => SCALAR, default => 0 },
        }
    );

    #TODO iterate over array of nodes
    #
    if ($opts{inhibit_whitelist}) {
        $Log->comment('inhibit_whitelist true, skipping add whitelist');
        return;
    }


    my @interfaces = NACL::C::NetworkInterface->find(
        command_interface => $opts{src_node},
        filter            => {
            'role'           => 'intercluster',
            'address-family' => 'ipv4',
        },
    );

    my $ava_node = $self->ava_node();

    $ava_node->show_version();
    my $whitelist_result = $ava_node->show_snapmirror_whitelist();

    #print "WHITELIST:" . Dumper $whitelist_result;

    #my $whitelist = join '', $whitelist_result->get_parsed_output();
    my $whitelist = join '', $whitelist_result->get_raw_output();


    my @address;
    foreach my $iface (@interfaces) {
        push @address, $iface->get_one_state_attribute('address');
    }

    foreach my $addr (@address) {
        if ($whitelist =~ qr/$addr/) {
            $Log->comment("AVA already accepts $addr");
        } else {
            $Log->comment("AVA not yet accepting $addr, add:");

            $ava_node->snapmirror_whitelist_add(ip =>$addr);
        }
    }


}


=head2 ava_ip()

Returns the ip of the altavault

=cut

sub ava_ip {
    my $self = shift;

    return $self->ava_node()->hostobj()->default_ip();

}

=head2 snapmirror_path()

Helper function that generates the snapmirror
destination path in the form <IP>:/share/<share_name>
for use in snapmirror commands.


  $shiner->snapmirror_path(volume => $volume);


=over

=item Options

=item C<< volume => $volume >>

Share name on altavault

=back

=cut

sub snapmirror_path {
    my $self = shift;
    my %opts = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            volume  => {type => SCALAR},
        }
    );


    return $self->ava_ip() . ':/share/' . $opts{volume};
}

sub initialize_snapmirror {
    my $self = shift;
    my %opts = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            src_node    => {type => OBJECT},
            src_volume  => {type => OBJECT},
            snapmirror  => {type => OBJECT},
            snapshot    => {type => SCALAR, default => 'avasnap'},
        }
    );

    my $src_node    = delete $opts{src_node};
    my $src_volume  = delete $opts{src_volume};
    my $src_vserver = $src_node->{vserver};
    my $dst_host    = $self->ava();
    my $sm_rel      = delete $opts{snapmirror};

    if ($src_volume->{type} eq 'RW') {
        my $snapshot = _create_snapshot(
            command_interface => $src_node,
            volume            => $src_volume,
            ss_name           => $opts{snapshot},
        );
    }
    my $ava_ip = $self->ava_ip();

    if ($src_volume->{type} eq 'RW') {
        $sm_rel->initialize( nacltask_wait => 1, 's' => $opts{snapshot}, );
    } else {
        $sm_rel->initialize( nacltask_wait => 1, );
    }


    return $sm_rel;
}


=head2 volume_name()

Generates a volume name with a time stamp.  This is
used internally to generate ava share names as they
have to be globally unique.

  my $vol = $shiner->volume_name();

  my $other_vol = $shiner->volume_name(volume => 'other_vol_');


=over

=item Options

=item C<< volume => $vol_prefix >>

Optional.

String to use a a prefix before the timestamp.

Default is 'vol_dst_'


=back

=cut

sub volume_name {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            'volume' => { 'type' => SCALAR, default => 'vol_dst_' },
        },
    );

    my $name = join '', $opts{volume}, gettimeofday;
    return $name;

}

=head2 validate()

Validates snapmirror data for the ontap->ava relationship
represented by the volumes passed in.

Since there is no way to do this directly on the altavault
side, it restores the altavault volume to a restore volume.
(This must be a primordial volume).  It then does a snapmirror
resync and check between the original source volume and
the restored volume.


  my $vol = $shiner->volume_name();

  $shiner->validate(
      volume => 'other_vol_',
      t);


=over

=item Options

=item C<< src_volume => $src_volume >>

The original source volume object


=item C<< ava_share => $ava_share >>

The name of the ava share to restore from.

This is just the share name, not a destination path.

=item C<< restore_volume => $restore_volume >>

Optional.

The volume object to restore onto.

If not passed, it will generate one with the same
size as the source volume and then delete it afterwards.

=back

=cut

sub validate {
    my $self = shift;

    my %opts = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            src_volume          => { type => OBJECT },
            ava_share           => { type => SCALAR },
            restore_volume      => { type => OBJECT, optional => 1 },
        }
    );
    my $src_volume =  $opts{src_volume};
    my $ava_share =  $opts{ava_share};

    $Log->comment('Validate ontap -> ava snapmirror');

    my $ci = $src_volume->command_interface();

    my $size =  $src_volume->get_one_state_attribute('size');


    my $restore_vol;
    my $created = 0;
    if (! ($restore_vol = $opts{restore_volume})) {
        $Log->comment("create restore volume size '$size'");
        $restore_vol = NACL::STask::Volume->create(
            command_interface => $ci,
            volume            => $self->volume_name(volume => 'restore_'),
            vserver           => $src_volume->vserver,
            type              => 'DP',
            size              => $size,
        );
        $created = 1;
    }

    my $vserver = $src_volume->vserver;

    $Log->comment('restore ava->restore volume');
    NACL::STask::Snapmirror->restore(
        src_node  => $ci,
        'source-path'      => $self->snapmirror_path(volume => $ava_share),
        'destination-path' => $restore_vol,
        #($snapshot ? ('source-snapshot'  => $snapshot) : ()),
        nacltask_wait      => 1,
        #nacltask_verify    => 'true'
    );

    $Log->comment('resync restored volume');
    NACL::STask::Snapmirror->resync(
        command_interface  => $ci,
        'source-path'      => $vserver . $COLON . $src_volume->volume,
        'destination-path' => $vserver . $COLON . $restore_vol->volume,
        type               => 'XDP',
        'nacltask_wait'    => 1,
    );

    $Log->comment('snapmirror check restored volume');
    NACL::STask::Snapmirror->check(
        command_interface  => $ci,
        'source-path'      => $vserver . $COLON . $src_volume->volume,
        'destination-path' => $vserver . $COLON . $restore_vol->volume,
        'nacltask_wait'    => 1,
    );



    $Log->comment('cleanup validation snapmirrors and volumes');
    try {
        NACL::STask::Snapmirror->delete(
            command_interface  => $ci,
            'destination-path' => $SPLAT,
            'source-path'      => $vserver . $COLON . $restore_vol->volume,
        );
    } catch NATE::BaseException with {
        $self->log()->comment("snapmirror does not exist: " .
            $vserver . $COLON . $restore_vol->volume);
    };

    try {
        NACL::STask::Snapmirror->delete(
            command_interface  => $ci,
            'destination-path' => $SPLAT,
            'source-path'      => $vserver . $COLON . $src_volume->volume,
        );
    } catch NATE::BaseException with {
        $self->log()->comment("snapmirror does not exist: " .
            $vserver . $COLON . $src_volume->volume);
    };


    $restore_vol->purge() if $created;
}


sub _create_snapshot {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            command_interface => { type => OBJECT, optional => 0 },
            volume            => { type => OBJECT, optional => 0 },
            ss_name           => { type => SCALAR, optional => 0 },
        }
    );

    my $volume            = delete $args{volume};
    my $ss_name           = delete $args{ss_name};
    my $command_interface = delete $args{command_interface};

    my $snapshot = NACL::C::VolumeSnapshot->create(
        command_interface => $command_interface,
        snapshot          => $ss_name,
        volume            => $volume->{volume},
        vserver           => $volume->{vserver},
    );

    return $snapshot;
}

sub run_wafliron {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node      => { type => OBJECT, optional => 0 },
            aggregate => { type => OBJECT, optional => 0 },
            fix       => { type => SCALAR, optional => 1, default => 0 },
        }
    );

    my $node      = $args{node};
    my $aggregate = $args{aggregate};
    my $fix = $args{fix};

    my $wafliron = NACL::MTask::Wafliron->new(
        command_interface => $node,
        aggregate         => $aggregate,
    );

    if ( $fix == 1 ) {
        # brings aggr back online after it completes.
        $wafliron->start( nacltask_include_mirrors => 1, );
    } else {

        $wafliron->start(
            nacltask_include_mirrors => 1,
            nacltask_optional_commit => 'true',
        );
    
        $wafliron->wait_for_completion();

        if ( !$wafliron->corruption_found() ) {
            $aggregate->online();
        } 
    }

    return;

} # end of run_wafliron

sub SFR_and_check {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec => {
            node_src                  => { type => OBJECT,   optional => 0 },
            restore_source_path       => { type => SCALAR,   optional => 0 },
            restore_destination_path => { type => SCALAR,   optional => 0 },
            restore_source_snapshot   => { type => SCALAR,   optional => 0 },
            file_list                 => { type => ARRAYREF, optional => 0 },
            vol_src                   => { type => OBJECT,   optional => 0 },
            vol_clone                 => { type => OBJECT,   optional => 0 },
            check_target_snapshot     => { type => SCALAR,   optional => 0 },
            },
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        },
    );

    my $node_src                  = $args{node_src};
    my $restore_source_path       = $args{restore_source_path};
    my $restore_destination_path  = $args{restore_destination_path};
    my $restore_source_snapshot   = $args{restore_source_snapshot};
    my $file_list                 = $args{file_list};
    my $vol_src                   = $args{vol_src};
    my $vol_clone                 = $args{vol_clone};
    my $check_target_snapshot     = $args{check_target_snapshot};

    $Log->step( 'SFR Restore' );
    NACL::STask::Snapmirror->restore(
        command_interface       => $node_src,
        'source-path'           => $restore_source_path,
        'destination-path'      => $restore_destination_path,
        'source-snapshot'       => $restore_source_snapshot,
        'create-check-snapshot' => 'true',
        'file-list'             => $file_list,
        nacltask_wait           => 1,
    );

    $self->run_check (
            node                            =>   $node_src,
            restore_destination_volume      =>   $vol_src,
            original_src_volume             =>   $vol_clone,
            snapshot_name                   =>   $check_target_snapshot,
            look_for_check_ss               =>   1,
            file_list                       =>   $file_list);

    return;
}

=head2 run_check()

Run snapmirror check between the snapmirrorCHK ss on the volume to which the files
were restored and the ss on the src volume.

The method will find the snapmirrorCHK* ss on the restore volume.

=item Options

    restore_destination_volume:    OBJECT (Required) The volume on which the files were restored.
    original_src_volume:           OBJECT (Required) The source volume on which the files were originally
                                                     created.
    snapshot_name:                 SCALAR (Required) The ss name on the original src volume.
    look_for_check_ss:             SCALAR (Optional) If checked the method will search for SnapshotCHK* ss
                                                     on the restored vol.
    file_list                      ARRAYREF(Required) The list of files to include in the sm check command.
    node                           OBJECT (Optional)  The node object on which to run the command.

=cut

sub run_check {
    my $self = shift;
    my %args = validate_with (
        params      => \@_,
        allow_extra => 0,
        spec        => {
            restore_destination_volume  =>   { type => OBJECT, optional => 0 },
            original_src_volume         =>   { type => OBJECT, optional => 0 },
            snapshot_name               =>   { type => SCALAR, optional => 0 },
            look_for_check_ss           =>   { type => SCALAR, optional => 0, default => 1 },
            file_list                   =>   { type => ARRAYREF, optional => 0 },
            node                        =>   { type => OBJECT, optional => 0 },
        } );

    my $restore_vol = $args{restore_destination_volume};
    my $ori_src     = $args{original_src_volume};
    my $src_ss      = $args{snapshot_name};
    my $file_list   = $args{file_list};
    my $n           = $args{node};
    my ($check_ss, $target_ss);

    if ($args{look_for_check_ss}) {
        $Log->step( 'Find snapmirrorCHK snapshot in vol: ' . $restore_vol->volume() );
        $check_ss = NACL::STask::VolumeSnapshot->find(
            command_interface => $restore_vol->command_interface(),
            filter            => {
                snapshot => 'snapmirrorCHK*',
                volume   => $restore_vol->volume(),
            });
    }

    if (defined $check_ss) {
        $target_ss = $check_ss->snapshot();
    } else {
        #presume the entire ss was restored so it will have the same name
        #as that on the src vol
        $target_ss = $src_ss;
    }

    my $apiset = NACL::APISet->new(
        name      => $n->node,
        category  => 'Node',
        interface => 'CLI',
        set       => 'CMode',
    );

    $Log->step( 'Run snapmirror check ');
    my $output = $apiset->snapmirror_check(
        'source-path'       => $ori_src->vserver() . ':' . $ori_src->volume(),
        'destination-path ' => $restore_vol->vserver() . ':' . $restore_vol->volume(),
        'source-snapshot'   => $src_ss,
        'target-snapshot'   => $target_ss,
        'file-list'         => join( ',', @{$file_list} ),
    )->get_parsed_output();

    my $job = NACL::C::Job->new(
        command_interface => $n,
        id                => $output->[0]->{job_id},
    );
    $job->watch_progress('method-timeout' => 1800);

    my $jh = NACL::C::JobHistory->find(
        command_interface => $n,
        filter =>
            { 'event-type' => 'Succeeded', id => $output->[0]->{job_id}, }
    );

    if ( $jh->get_one_state_attribute( 'completion' ) =~ /found inconsistencies/i ) {
        NATE::BaseException->throw('Snapmirror check failed.');
    }

    if(defined $check_ss) {
        $Log->step('Delete snapshot to snapmirror check against');
        $check_ss->purge();
    }

    return;
}

sub create_check_clone {

    my $self = shift;
    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node_src        => { type => OBJECT, optional => 0 },
            parent_volume   => { type => OBJECT, optional => 0 },
            parent_snapshot => { type => SCALAR, optional => 0 },
            clone_aggregate => { type => OBJECT, optional => 0 },
            timeout         => { type => SCALAR, optional => 1, default => 1800 },
        },
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        },
    );

    my $node_src        = $args{node_src};
    my $parent_volume   = $args{parent_volume};
    my $parent_snapshot = $args{parent_snapshot};
    my $clone_aggregate = $args{clone_aggregate};
    my $timeout         = $args{timeout};

    my $snap1clone = NACL::STask::VolumeClone->create(
        command_interface => $node_src,
        vserver           => $parent_volume->vserver(),
        'parent-volume'   => $parent_volume,
        'parent-snapshot' => $parent_snapshot,
        nacltask_wait     => 1,
    );

    my $snap1volume = $snap1clone->cast_to_volume();

    NACL::STask::VolumeMove->start(
        command_interface       => $node_src,
        'destination-aggregate' => $clone_aggregate->aggregate(),
        volume                  => $snap1volume->volume(),
        vserver                 => $snap1volume->vserver(),
        nacltask_wait           => 1,
        'method-timeout'        => $timeout,
    );

    return $snap1volume;
}

1;

