# -*-perl-*-
#
# Copyright (c) 2015,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DPG Validations
## @author ng-smgr-auto@netapp.com
## @status shared
## @pod here

package NACL::AppBuilder::DpgWorkflows::Validation;

use strict;
use warnings;

use Params::Validate qw(validate validate_with :types);

use NATE::Log qw(log_global);
use NATE::BaseException qw( :try );

use NACL::Exceptions::UnexpectedOutputException;

use NACL::C::Node;
use NACL::CS::Snapmirror;
use NACL::CS::Lun;
use NACL::CS::VolumeSnapshot;
use NACL::STask::Volume;
use NACL::STask::Aggregate;

use NACL::AppBuilder::DpgWorkflows::StringConstants;

use base qw(Exporter);

our @EXPORT_OK = qw(
        validate_failed_sync_initialization
        check_dedupe_space_saved
        check_compression_space_saved
        compare_dedupe_space_saved
        compare_compression_space_saved
        Validate_PCS_Tags
        Validate_Initialize
        Validate_Update
        Validate_add_cg_item
        Validate_del_cg_item
        Validate_flip_resync_cg_items
	validate_object_packing_packed
	validate_object_packing_flags
        Validate_resync
	get_wafl_composite_stats_counter_aggr
        get_composite_aggr_stats
        Validate_tiering
	Validate_snapmirror_logs
        Validate_appcg_forward_transfer
        Validate_appcg_sfr
	Validate_appcg_restore
        Verify_checkpoint
        lsa_validation
        verify_transferring
        lsa_verify_in_progress
        lsa_count_verify
        lsa_verify_to_complete
        );

our %EXPORT_TAGS = (
    'all' => \@EXPORT_OK,
);

my $SLASH = q{/};

=head1 NAME

NACL::AppBuilder::DpgWorkflows::Validation

=head1 DESCRIPTION

This library contains validations for various snapmirror relationship states.

=head1 METHODS

=cut

=head2 validate_failed_sync_initialization

validate that a failed sync-mirror relationship failed with the
expected conditions

=over

=item example

    my $smgr_relationship = NACL::CS::Snapmirror->fetch();
    validate_failed_sync_initialization(
        'relationship'                 => $smgr_relationship,
        'expected_last_transfer_error' => $text);

=item options

=over

=item relationship

(required) snapmirror component state - the component state object of the
relationship you would like to validate

=item expected_last_transfer_error

(required)  scalar text value - this is the text that the last_transfer_error
will be tested against as a case-insensitive regular expression.

=back

=back

=cut

sub validate_failed_sync_initialization {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            relationship => {
                isa      => 'NACL::CS::Snapmirror',
                optional => 0,
            },
            expected_last_transfer_error => { type => SCALAR, optional => 0 },
        },
    );

    my $sm_state = $args{relationship};
    my $text     = $args{expected_last_transfer_error};

    if (   $sm_state->{'total_transfer_time_secs'} != 0
        || $sm_state->{'last_transfer_error'} !~ /$text/i
        || $sm_state->{'status'} !~ /outofsync/i
        || $sm_state->{'state'} !~ /uninitialized/i

        )
    {
        NACL::Exceptions::UnexpectedState->throw(
            "Unexpected state for smgr relationship, expected: \n"
                . 'total_transfer_time_secs = 0' . "\n"
                . "last_transfer_error = $text" . "\n"
                . 'status = outofsync' . "\n"
                . 'state = uninitialized' . "\n",
            monitoring_fields => [
                'total_transfer_time_secs', 'last_transfer_error',
                'status',                   'state',
            ],
            state => $sm_state->{'total_transfer_time_secs'}
                . $SLASH
                . $sm_state->{'last_transfer_error'}
                . $SLASH
                . $sm_state->{'status'}
                . $SLASH
                . $sm_state->{'state'},
        );
    }

    return 0;
}

=head2 check_dedupe_space_saved

validate that dedupe space savings on volume is greater than zero.

=over

=item example

    check_dedupe_space_saved(vol => $volume);

=item options

=over

=item vol

(required) Volume Object - the volume object whose dedupe savings needs to be evaluated.

=back

=cut

sub check_dedupe_space_saved{
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol       => {type => OBJECT, optional=>0},
            }
        );

	my $vol = $args{'vol'};
    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $space_saved
            = $vol->get_one_state_attribute('dedupe-space-saved');
    $log->comment(
            "Dedupe savings on dst: $space_saved"
        );
        if ($space_saved <= 0) {
            $log->error('No Deduplicated: '
                    . $space_saved
                      );
        }
}

=head2 check_compression_space_saved

   validate that compression space savings on volume is greater than zero.

=over

=item example

    check_compression_space_saved(vol => $volume);

=item options

=over

=item vol

(required) Volume Object - the volume object whose compression savings needs to be evaluated.

=back

=cut

sub check_compression_space_saved{
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol       => {type => OBJECT, optional=>0},
            }
        );

	my $vol = $args{'vol'};
    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;
    my $comp_saved
            = $vol->get_one_state_attribute('compression-space-saved');
    $log->comment(
            "Compression savings on dst: $comp_saved"
        );
        if ($comp_saved <= 0) {
            $log->error('Not Compressed: '
                    . $comp_saved
                      );
        }
}

=head2 compare_dedupe_space_saved

   Compare the dedupe space savings on source and destination volume
   and returns 0 if there is discrepancy else returns 1.

=over

=item example

    compare_dedupe_space_saved(vol_src => $volume_Src, vol_dst=> $volume_dst);

=item options

=over

=item vol

(required) Source Volume Object - the volume object whose dedupe savings needs to be evaluated.

=item vol

(required) Destination Volume Object - the volume object whose dedupe savings needs to be evaluated.

=back

=cut


sub compare_dedupe_space_saved {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_src       => {type => OBJECT, optional=>0},
			vol_dst       => {type => OBJECT, optional=>0},
            }
        );

	my $vol_src = $args{'vol_src'};
	my $vol_dst = $args{'vol_dst'};
    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;
    my ($src_space_saving_actual, $ret_val, $dst_space_saving_actual,
        $src_space_saving_percent, $dst_space_saving_percent);

    $src_space_saving_actual  = $vol_src->get_one_state_attribute('dedupe-space-saved');
    $dst_space_saving_actual  = $vol_dst->get_one_state_attribute('dedupe-space-saved');
    $src_space_saving_percent = $vol_src->get_one_state_attribute('dedupe-space-saved-percent');
    $dst_space_saving_percent = $vol_dst->get_one_state_attribute('dedupe-space-saved-percent');

    $log->comment(
               "Source Space Savings\n\tActual: $src_space_saving_actual\n"
             . "\tPercent: $src_space_saving_percent\n"
             . "Destination Space Savings\n\tActual: $dst_space_saving_actual\n"
             . "\tPercent: $dst_space_saving_percent");
    if ($src_space_saving_actual eq q(-) || $dst_space_saving_actual eq q(-)) {
       $log->error(' No space has been saved by dedupe operation ');
    }
    if (($dst_space_saving_percent > $src_space_saving_percent) && ($dst_space_saving_percent <= 3.5*$src_space_saving_percent) ) {
        $ret_val = 1;
    } else {
	$ret_val = 0;
    }

    return $ret_val;
} ## end sub compare_dedupe_space_saved

=head2 compare_compression_space_saved

   compare the compression space savings on source and destination volumes
   and returns 0 if there is discrepancy else returns 1.

=over

=item example

    compare_compression_space_saved($volume_Src, $volume_dst);

=item options

=over

=item vol

(required) Source Volume Object - the volume object whose dedupe savings needs to be evaluated.

=item vol

(required) Destination Volume Object - the volume object whose dedupe savings needs to be evaluated.

=back

=cut

sub compare_compression_space_saved {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_src       => {type => OBJECT, optional=>0},
			vol_dst       => {type => OBJECT, optional=>0},
            }
        );

	my $vol_src = $args{'vol_src'};
	my $vol_dst = $args{'vol_dst'};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;
    my ($src_space_saving_actual, $ret_val, $dst_space_saving_actual,
        $src_space_saving_percent, $dst_space_saving_percent);

    $src_space_saving_actual  = $vol_src->get_one_state_attribute('compression-space-saved');
    $dst_space_saving_actual  = $vol_dst->get_one_state_attribute('compression-space-saved');
    $src_space_saving_percent = $vol_src->get_one_state_attribute('compression-space-saved-percent');
    $dst_space_saving_percent = $vol_dst->get_one_state_attribute('compression-space-saved-percent');

    $log->comment(
               "Source Compressed Space Savings\n\tActual: $src_space_saving_actual\n"
             . "\tPercent: $src_space_saving_percent\n"
             . "Destination Compressed Space Savings\n\tActual: $dst_space_saving_actual\n"
             . "\tPercent: $dst_space_saving_percent");
    if ($src_space_saving_actual eq q(-) || $dst_space_saving_actual eq q(-)) {
       $log->error(' No space has been saved by dedupe operation ');
    }
    if  (($dst_space_saving_percent > $src_space_saving_percent) && ($dst_space_saving_percent <= 3.5*$src_space_saving_percent)) {
        $ret_val = 1;
    } else {
        $ret_val = 0;
    }

    return $ret_val;
}

=head2 Validate_PCS_Tags

validate the gran.pseudo tag on PCS

=over

=item example

 Validate_PCS_Tags(volname=>$volname, node=>$Node, snapshot=>$snapshot);

=item options

=over

=item volname

(required) Volume name - the volume which hosts the snapshot.

=item node

(required) Node Object - the Node which owns the volume.

=item snapshot

(required) Snapshot name - the snapshot on which tags need to be checked.

=back

=item Exceptions

    NACL::Exceptions::UnexpectedOutputException

=cut



sub Validate_PCS_Tags{
    my %args = validate_with(
        params => \@_,
        spec   => {
            volname       => {type => SCALAR, optional=>0},
            node       => {type => OBJECT, optional=>0},
            snapshot   => {type => SCALAR, optional=>0},

            }
        );

    my $volname = $args{'volname'};
    my $Node = $args{'node'};
    my $SS  = $args{'snapshot'};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $apiset = $Node->get_7m_or_nodescope_apiset();
    my $snaptags_test = "snaptags_test -g -v $volname -S $SS";
    my $snaptags_output = $apiset->execute_command( command => $snaptags_test, );

    NACL::Exceptions::UnexpectedOutputException->throw(text=>"snaptags_test command failed.\nGot $snaptags_output")
        if ($snaptags_output =~ /Can\'t get id of snapshot/i);

    if ( $snaptags_output =~ /pseudo/ ) {
        $log->comment("Found pseudo tag");
        $log->exit() if $may_exit;
        return 1;
    } else {
        $log->comment("pseudo tag not found");
        $log->exit() if $may_exit;
        return 0;
    }

}

sub Validate_Initialize {
   # Validation1: status =InSync, state=snapmirrored, healthy=true, last-tranfer-type =initialize via snapmirror-show
   # Validation2: Snapmirror-snapshot gets cut on the destination after initialize.
   # Validation3: In case of sync validate the "Insync" status via snapmirror-list-destinations
   # Validation4: If snapshot is specified then check if matches exported_snapshot and newest_snapshot

    my %args = validate_with(
            params => \@_,
            spec   => {
                 node_dst          => {type => OBJECT|SCALAR, optional =>  0},
                 node_src          => {type => OBJECT|SCALAR, optional =>  0},
                 sm_obj            => {type => OBJECT|SCALAR, optional => 0},
                 source_cg_name    => {type => SCALAR, optional => 1 },
                 dst_cg_name       => {type => SCALAR, optional => 1 },
                 snapshot          => {type => SCALAR, optional => 1 },
            });

    my $Src_Node_Obj = $args{node_src};
    my $Dst_Node_Obj = $args{node_dst};
    my $o = $args{sm_obj};
    my $Src_Cg_Name = $args{source_cg_name};
    my $Dst_Cg_Name = $args{dst_cg_name};
    my $log = log_global();
my $may_enter = $log->may_enter();
my $may_exit  = $log->may_exit();

    my ($Src_Vserver_Name, $src_vol_name ) = split(/:/,$o->source_path);
    my ($Dst_Vserver_Name, $dst_vol_name ) = split(/:/,$o->destination_path);
    # get the current state of the relationship to verify attributes


    $log->comment('Verify relationship state on the dst');
    my $state = NACL::CS::Snapmirror->fetch(
                    command_interface => $Dst_Node_Obj,
                    filter            => {'source-path'      => $o->source_path,
                                          'destination-path' => $o->destination_path} );

   #if policy is MirrorAll and src has multiple ss then expect
    #last-transfer-type = update
    $log->comment('Fetch all ss on the src');
    my @allss = NACL::CS::VolumeSnapshot->fetch(
                    command_interface       =>  $Src_Node_Obj,
                    filter                  => {
                            vserver         => $Src_Vserver_Name,
                            volume          => $src_vol_name },
                    requested_fields => [qw(snapshot version-uuid)],
                    allow_empty      => 1);

    my $no_of_ss = scalar(@allss);
    $log->comment('Found ' . $no_of_ss . ' on the src');

    my $lasttransfertype = 'initialize';
    if ($state->policy =~ /MirrorAllSnapshots/ && $no_of_ss > 1) {
        $lasttransfertype = 'update';
        $log->comment('....setting last-transfer-type to update');
    }

    $log->comment("Verify state is 'Snapmirrored' after initialize");
    if ( $state->state !~ /Snapmirrored/i ) {
        $log->error("Relationship state is not as expected after initialize\n"
                . "expected:  Snapmirrored\n"
                . "got:  " . $state->state);
    }

    $log->comment("Verify healthy is 'true' after initialize");
    if ( $state->healthy !~ /true/i ) {
        $log->error("Relationship health is not as expected after initialize\n"
                . "expected:  true\n"
                . "got:   " . $state->healthy);
    }

    my $found_current_transfer_type = 0;
    $log->comment("Verify current-transfer-type/last-transfer-type is 'initialize' after initialize");
    if ( $state->current_transfer_type !~ /initialize/i ) {
        $log->comment( "current-transfer-type is not as expected after initialize\n"
                . "expected:  initialize\n"
                . "got: ". $state->current_transfer_type .
                ".\n Check the last_transfer_type field");
    } else {
        $log->comment('Found the expected value for current-transfer-type');
        $found_current_transfer_type = 1;
    }

    if (! $found_current_transfer_type) {
        $log->comment('Checking the last transfer type field');
        if ($state->last_transfer_type !~ /$lasttransfertype/i ) {
           $log->error("last-transfer-type is not as expected after initialize \n"
                       . "expected:  initialize\n"
                       . "got: ". $state->last_transfer_type);
        } else {
           $log->comment('Found the expected value for last-transfer-type');
        }
    }


    # If the relationship type is sync then validate the Insync status

    if($state->policy_type eq 'sync-mirror'){
        $log->comment("Verify status is 'InSync' after initialize");
        if ( $state->status !~ /InSync/i ) {
            $log->error("Relationship status is not as expected after initialize\n"
                    . "expected:  InSync\n"
                    . "got:  $state->status");
        }

        my $newest_ss = $o->get_one_state_attribute('newest-snapshot');
        if ($newest_ss =~ /snapmirrorCG.*/) {
            $log->comment('Snapmirror SS was cut on the destination after initialize operation as expected');
        } else {
            $log->error('Snapmirror SS was not  cut on the destination after initialize operation as expected');
        }
    } else {
        my $exception_text;
        if ($args{snapshot}) {
            # Validation4: If snapshot is specified then check if matches exported_snapshot and newest_snapshot
            $exception_text = _verify_snapshot(sm_obj => $args{sm_obj}, snapshot => $args{snapshot});
            # this should be an exception - but it will mean changing entire routine so logging error now
            $log->error($exception_text) if ($exception_text);
        } else {
            #Snapmirror-snapshot gets cut on the destination after initialize
            $log->log('Verify if snapmirror snapshot got cut on the destination');
            my $exp_snapshot_after_init = $o->get_one_state_attribute('exported-snapshot');

            if ($exp_snapshot_after_init =~ /snapmirror.*/) {
                $log->comment('Snapmirror snapshot got cut on the destination after initialize operation as expected');
            }
            else{
                $log->error('Snapmirror snapshot did not get cut on the destination after initialize operation unexpectedly');
            }
        }
    }

}


=head2 Validate_Update

Will validate a snapmirror update by checking the following
- if snapshot name is provided will check it matches exported_snapshot and newest_snapshot

=cut

sub Validate_Update {

    my %args = validate_with(
            params => \@_,
            spec   => {
                 #node_dst => {type => OBJECT|SCALAR},
                 #node_src => {type => OBJECT|SCALAR},
                 sm_obj   => {type => OBJECT|SCALAR},
                 snapshot => {type => SCALAR|UNDEF, optional => 1},
            });

    my $sm_obj = $args{sm_obj};
    my $snapshot = $args{snapshot};
    my $log = log_global();
    my $may_enter = $log->may_enter();
    my $may_exit  = $log->may_exit();

    my $exception_text;

        if (!defined $snapshot) {
        $log->comment('Fetch the latest ss on the dst');
        my $cmd_interface = $sm_obj->command_interface();

        my $vol = NACL::STask::Volume->find (
                        command_interface       =>  $cmd_interface,
                        filter                  => {
                                volume          => $sm_obj->destination_volume()});

        $log->comment('fetch all ss on the dst vol');
        my @ss = NACL::CS::VolumeSnapshot->fetch(
                        command_interface       =>  $cmd_interface,
                        filter                  => {
                                volume          => $vol->volume(),
                                vserver         => $vol->vserver()});
        my @sorted_ss = NACL::CS::VolumeSnapshot->sort_by_newest( state_objs => \@ss );
        $snapshot = $sorted_ss[0]->snapshot();
        $log->comment('Got back ss: ' . $snapshot . ' as the latest ss');
    }

    # verify if the snapshot specified matches newest-snapshot and exported-snapshot
    if ($snapshot) {
        $exception_text = _verify_snapshot(sm_obj => $args{sm_obj}, snapshot => $snapshot);
    }

    if ($exception_text) {
        NATE::BaseException->throw($exception_text);
    } else {
        $log->log('Validation of snapmirror update succeeded');
    }

    return;
}

=head2 get_wafl_composite_stats_counter_aggr

    This subroutine returns 'wafl composite stats counter show <aggr>' output for a given aggregate
    Command: system node run -node local -command "priv set -q test; wafl composite stats counter show aggr_name;"

=over

=item local_node_api

    Local Node API object

=item aggregate_name

    Name of the aggregate

=item

    usage:
    $dst_aggr_stats = get_wafl_composite_stats_counter_aggr(
        local_node_api => $dst_nodescope_apiset,
        aggregate_name => $dst_aggr_name
    );

=back

=cut

sub get_wafl_composite_stats_counter_aggr {
    my %opts = validate(
        @_,
        {
            local_node_api => {type => OBJECT , isa => "NACL::APISet::Node"},
            aggregate_name => {type => SCALAR},
        }
    );
    my $local_node_api      = $opts{local_node_api};
    my $aggregate_name      = $opts{aggregate_name};

    my $response = $local_node_api->wafl_composite_stats_counter_show(
        'aggr' => $aggregate_name,
    );
    my @parsed_output = @{$response->get_parsed_output()};

    foreach my $entry (@parsed_output) {
        if ($entry->{'Entry Type'} =~ /Aggr/i &&
            $entry->{'Aggr Name'} eq $aggregate_name) {
            return $entry;
        }
    }
    return 0;
} ## end sub get_wafl_composite_stats_counter_aggr

=head2 Validate_tiering

For the given volume and given tiering policy will gather latest composite stats and compare against the originals provided.

Will throw NACL::Exceptions::UnexpectedOutputException if validation failed.
On success, will return diff of the tier stats (of the tier that corresponds to tiering policy)

usage

    # gather original stats
    my($cap_orig,$perf_orig) = get_composite_aggr_stats(aggr=>$dst_aggr);

    # do some operation

    # validate tiering
    my $diff = Validate_tiering(vol_obj=>$vol_obj,cap_orig=>$cap_orig,perf_orig=>$perf_orig);

=cut

sub Validate_tiering {
    my @args = @_;
    my %args = validate_with(
        params => \@args,
        spec   => {
            vol_obj        => { type => OBJECT },
            tiering_policy => { type => SCALAR, optional => 1, regex => qr/none|snapshot-only|backup/ },
            cap_orig       => { type => SCALAR },
            perf_orig      => { type => SCALAR },
        }
    );
    my $log = log_global(1);

    my $tiering_policy = ($args{tiering_policy}) ? $args{tiering_policy} : $args{vol_obj}->get_one_state_attribute('tiering-policy');
    my $data_landing = ($tiering_policy eq 'backup') ? 'capacity' : 'performance';

    my $node = NACL::C::Node->new(name => $args{vol_obj}->get_local_node_name());
    my $aggr_obj = NACL::STask::Aggregate->new(
           command_interface => $node,
           aggregate         => $args{vol_obj}->aggregate(),
    );
    my ($cap_new,$perf_new) = get_composite_aggr_stats(aggr=>$aggr_obj);

    my ($new,$orig);
    if ($data_landing eq 'performance') {
        $orig = $args{perf_orig};
        $new = $perf_new;
    } else {
        $orig = $args{cap_orig};
        $new = $cap_new;
    }

    if ($new <= $orig) {
        my $msg = 'Un-expected result!!!. Expected data to increase in '.$data_landing.
                    ' tier for volume '. $args{vol_obj}->volume().
                    " Original=$orig New=$new";
        #NACL::Exceptions::UnexpectedOutputException->throw(text=>$msg);
        $log->error($msg);
    } else {
        my $msg = 'Got expected result. Expected data to increase in '.$data_landing.
                    ' tier for volume '. $args{vol_obj}->volume().
                    " Original=$orig New=$new";
        $log->log($msg);
    }
    return ($new-$orig);
}

=head2 get_composite_aggr_stats

Will get the cloud and perf used for a composite aggregate
-It takes aggr object ad input and returns perf-used and cloud-used values to the caller
- see the usage in invoke_vaildation.pm(unit test for this function)

e.g my($cap_t1,$perf_t1) = get_composite_aggr_stats(aggr=>$dst_aggr);

=cut

sub get_composite_aggr_stats {
     my %args = validate_with(
            params => \@_,
            spec   => {
                 aggr   => {type => OBJECT},
            });

    my $aggr = $args{aggr};
    my $aggr_node = $aggr->command_interface();

    my $aggr_name = $aggr->get_one_state_attribute('aggregate');
    my $log = log_global();

    my $nodescope_apiset = $aggr_node->get_7m_or_nodescope_apiset();

    # get df -AC output
    my $df_output = $nodescope_apiset->execute_command(command => "df -AC $aggr_name");

    my ($cap_map,$perf_map) = _get_df_cloud_perf_used(output => $df_output);

    my $cap_tier_used = $cap_map->{'df-used'};
    my $perf_tier_used = $perf_map->{'df-used'};

    $log->comment('aggr name :' . $aggr->aggregate());
    $log->comment('capacity map :' . $cap_tier_used);
    $log->comment('performance map :' . $perf_tier_used);

    return($cap_tier_used,$perf_tier_used);

} ## end get_composite_aggr_stats

# Returns aggregate cloud and perf usage as reported by 'df -AC <aggr>' and returns the used column
# NOTE: this method will work only if the "output" is for a particular aggregate
# it will not work as expected if multiple aggregates are found in the output
sub _get_df_cloud_perf_used {
     my %args = validate_with(
            params => \@_,
            spec   => {
                output   => {type => OBJECT|SCALAR},
            });
    my $log = log_global();

    my $cap_map;
    my $perf_map;
    my $output = $args{output};

    my ($perf_used, $cap_used);
    foreach my $line (split(/\n/,$output)) {
        if ($line =~ /Perf\sTier\s+\S+\s+(\d+)/) {
            $perf_used += $1;
        }
        elsif ($line =~ /\S+\s+\S+\s+(\d+)/) {
            $cap_used += $1;
        }
    }
    $perf_map->{'df-used'} = $perf_used;
    $cap_map->{'df-used'} = $cap_used;

    return ($cap_map,$perf_map);
}

# private method used by Initialize and Update Valiate methods
sub _verify_snapshot {
    my %args = validate_with(
            params => \@_,
            spec   => {
                 sm_obj   => {type => OBJECT|SCALAR},
                 snapshot => {type => SCALAR},
            });

    my $sm_obj = $args{sm_obj};
    my $snapshot = $args{snapshot};
    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();

    my $exception_text;
    # verify if the snapshot specified matches newest-snapshot
    my $newest_ss = $sm_obj->get_one_state_attribute('newest-snapshot');
    if ($newest_ss =~ /$snapshot/) {
        $log->log('As expected, found newest-snapshot to be '.$snapshot);
    } else {
        my $txt = 'Field newest_snapshot in snapmirror show did not match. Expected:'.$snapshot.' Got:'.$newest_ss;
        $exception_text .= $txt;
        $log->log($txt);
    }

    # verify if the snapshot specified matches exported-snapshot
    my $exported_ss = $sm_obj->get_one_state_attribute('exported-snapshot');
    if ($exported_ss =~ /$snapshot/) {
        $log->log('As expected, found exported-snapshot to be '.$snapshot);
    } else {
        my $txt = 'Field exported_snapshot in snapmirror show did not match. Expected:'.$snapshot.' Got:'.$newest_ss;
        $exception_text .= $txt;
        $log->log($txt);
    }

    # verify if the transfer-snapshot becomes '-'
    # transfer-snapshot will show the snapshot used while transferring
    # after transfer is completed it will become '-'
    my $transfer_ss = $sm_obj->get_one_state_attribute('transfer-snapshot');
    if ($transfer_ss =~ /-/) {
        $log->log('As expected, found snapshot transfer-snapshot to be -');
    } else {
        my $txt = 'Field transfer_snapshot in snapmirror show did not match. Expected:- Got:'.$transfer_ss;
        $exception_text .= $txt;
        $log->log($txt);
    }

    return ($exception_text);

}

=head2 Validate_resync

validate that the snapmirror relationship is in Idle state after issuing resync.

=over

=item example

 Validate_resync(node_src=>$src_node, node_dst=>$dst_node, sm_obj=>$sm_obj);

=item options

=over

=item node_src

(required) node_src - the source node.

=item node_dst

(required) node_dst - the destination node.

=item sm_obj

(required) Snapmirror object - the snapmirror object.

=cut

sub Validate_resync{
    # Validation1: status =Idle, state=snapmirroed, healthy=true, last-tranfer-type =resync
    # Validation2 : In case of sync validate the "Insync" status via snapmirror-list-destinations

    my %args = validate_with(
        params => \@_,
        spec   => {
            node_src     => {type => OBJECT, },
            node_dst     => {type => OBJECT, },
            sm_obj       => {type => OBJECT, },
            }
        );

    my $Src_Node_Obj     = $args{node_src};
    my $Dst_Node_Obj     = $args{node_dst};
    my $o       = $args{sm_obj};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    # This code is to fetch vserver and volume for cg relationship

    my $state = NACL::CS::Snapmirror->fetch( command_interface => $Dst_Node_Obj,
            filter => {'source-path'      =>  $o->source_path,
            'destination-path' =>  $o->destination_path},
            );


    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
            'relationship-id' => $state->relationship_id,
            },
            requested_fields => [qw(source-path destination-path)]);

    my($Src_Vserver_Name,$src_vol_name) = split(/:/,$list_dst->source_path);
    my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$list_dst->destination_path);

    # get the current state of the relationship to verify attributes
    $log->comment("Verify status is 'Idle' after resync");
    if ( $state->status !~ 'Idle' ) {
        $log->error("Relationship status is not as expected after resync\n"
                . "expected:  Idle\n"
                . "got:  $state->status");
    }

    $log->comment("Verify state is 'Snapmirrored' after resync");
    if ( $state->state !~ 'Snapmirrored' ) {
        $log->error("Relationship state is not as expected after resync\n"
                . "expected:  Snapmirrored\n"
                . "got:  $state->state");
    }

    $log->comment("Verify healthy is 'true' after resync");
    if ( $state->healthy !~ 'true' ) {
        $log->error("Relationship health is not as expected after resync\n"
                . "expected:  true\n"
                . "got:   $state->healthy");
    }

    $log->comment("Verify last-transfer-type is 'resync' after resync");
    if ( $state->last_transfer_type !~ 'resync' ) {
        $log->error( "Last-transfer-type is not as expected after resync\n"
                . "expected:  resync\n"
                . "got: $state->last_transfer_type");
    }
    # for smgr
    if($state->policy_type eq 'sync-mirror'){
        # Validate the InSync status
        _verify_list_destination_status($Src_Node_Obj,$dst_vol_name,$src_vol_name,$Src_Vserver_Name,$Dst_Vserver_Name,'InSync');
    }

}


=head2 Validate_flip_resync_cg_items

Validate that the snapmirror relationship is in Insync state after issuing a flip resync.
This method will also check if the new SRC luns are Read Only and
the new DST luns are RW post a successful flip resync.

=over

=item example

     Validate_flip_resync_cg_items (
                        node_src         => $node_src,
                        node_dst         => $node_dst,
                        src_vol_name     => $vol_src_name,
                        dst_vol_name     => $vol_dst_name,
                        dst_path         => $snapmirror->{'destination_path'},
                        src_path         => $snapmirror->{'source_path'},
                        cg_item_list     => \@cgnames);

=item options

=over

=item node_src

(required) NACL Object for Source Node which owns the volume.

=item node_dst

(required) NACL Object for Destination Node which owns the volume.

=item src_vol_name

(required) The new src vol name where the luns reside.

=item dst_vol_name

(required) The new dst vol name where the luns reside.

=item dst_path

(required) Destination Path - the destination path of snapmirror relationship

=item src_path

(required) Source Path - the source path of snapmirror relationship

=item cg_item_list

(required) List of files/luns to be deleted from cg relationship

=back

=item exceptions

    NATE::BaseException

=cut

sub Validate_flip_resync_cg_items {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node_src     => {type => OBJECT, },
            src_vol_name => {type => SCALAR, },
            dst_vol_name => {type => SCALAR, },
            node_dst     => {type => OBJECT, },
            dst_path     => {type => SCALAR, },
            src_path     => {type => SCALAR, },
            cg_item_list => {type => ARRAYREF, },
            }
        );

    my $node_src     = $args{node_src};
    my $node_dst     = $args{node_dst};
    my $src_path     = $args{src_path};
    my $dest_path    = $args{dst_path};
    my $src_vol_name = $args{src_vol};
    my $dst_vol_name = $args{dst_vol};
    my $cg_item_list = $args{cg_item_list};


    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    $log->step('Verify if CG is Insync');
    my $error_text;
    my $list_destinations = NACL::CS::SnapmirrorListDestinations->fetch( command_interface => $node_src,
                                                                filter => {
                                                                            'destination-path' => $dest_path,
                                                                            'source-path'      => $src_path,
                                                                            },
                                                                allow_empty => 0,
                                                                );
    if ($list_destinations->status() !~ /InSync/){
        my $err = 'Did not find CG relationship InSync (SRC side) after deleting cg items.';
        $log->log($err);
        $error_text .= $err;
    } else {
        $log->log('Snapmirror relationship (SRC side) is in status=InSync');
    }
    # Verify on DST if relationship is InSync check status=InSync and state=Snapmirrored via snapmirror show command
    my $sm_state = NACL::CS::Snapmirror->fetch( command_interface => $node_dst,
                                               filter            => {
                                                      'source-path'      => $src_path,
                                                      'destination-path' => $dest_path,
                                                },
                                                allow_empty => 0,
                                            );
    if ( $sm_state->{unhealthy_reason}    =~ /-/i &&
         $sm_state->{last_transfer_error} =~ /-/i &&
         $sm_state->{status}              =~ /InSync/ &&
         $sm_state->{state}               =~ /Snapmirrored/ ) {
        $log->log('Snapmirror relationship (DST side) is in status=InSync and state=Snapmirrored without errors');
    } else {
        my $err = 'Did not find CG relationship InSync (DST side) after deleting cg items.';
        $log->log($err);
        $error_text .= $err;
    }


    $log->step('Verify that the new src luns are now RW');
    my @lun_items = NACL::CS::Lun->fetch( command_interface => $node_src,
                                          filter            => {
                                                         volume     => $src_vol_name,
                                                         'read-only' => 'false' });

    my @not_found_items;
    foreach my $cg_item (@{$cg_item_list}) {
        if (!grep { $_->path =~ /\/vol\/.*\/$cg_item$/ } @lun_items) {
            push @not_found_items, $cg_item;
        }
    }

    if (scalar(@not_found_items)) {
        my $err = 'Un-expected result!!!. Expected the new src luns to be RW. But found items: ' . Dumper(\@not_found_items)  . ' with unexpected state';
        $log->log($err);
        $error_text .= $err;
    }

    $log->step('Verify that the new dst luns are now RO');
    @lun_items = ();
    @lun_items = NACL::CS::Lun->fetch( command_interface => $node_dst,
                                          filter            => {
                                                         volume     => $dst_vol_name,
                                                         'read-only' => 'true' });

    @not_found_items = ();
    foreach my $cg_item (@{$cg_item_list}) {
        if (!grep { $_->path =~ /\/vol\/.*\/$cg_item$/ } @lun_items) {
            push @not_found_items, $cg_item;
        }
    }

    if (scalar(@not_found_items)) {
        my $err = 'Un-expected result!!!. Expected the new dst luns to be RO. But found items: ' . Dumper(\@not_found_items)  . ' with unexpected state';
        $log->log($err);
        $error_text .= $err;
    }

    # throw an exception if any of the validations fail
    if ($error_text) {
        NATE::BaseException->throw($error_text);
    } else {
        $log->log('Validation after flip-resync succeded');
    }

    return;
}

=head2 Validate_add_cg_item

validate that snapmirror relationship comes to Insync state after addition of cg item
validate that cg items added to cg relationship got replicated to destination

=over

=item example

    Validate_del_cg_item(
                        node_src     => $node_src_obj,
                        node_dst     => $node_dst_obj,
                        dst_path     => $snapmirror_obj->{destination_path},
                        src_path     => $snapmirror_obj->{source_path},
                        dst_vol_name => $vol_dst_name,
                        cg_item_list => \@cg_item_array,
                        validate_expand_only => 0,
                        );

=item options

=over

=item node_src

(required) NACL Object for Source Node which owns the volume.

=item node_dst

(required) NACL Object for Destination Node which owns the volume.

=item dst_path

(required) Destination Path - the destination path of snapmirror relationship

=item src_path

(required) Source Path - the source path of snapmirror relationship

=item cg_item_list

(required) List of files/luns to be deleted from cg relationship

=item validate_expand_only

(required) If set, will skip validation to check if relationship is InSync and also skip validation to check if items became r/w. Default 0.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=cut


sub Validate_add_cg_item {
    _validate_add_del_cg_item(@_,action=>'add');
}

=head2 Validate_del_cg_item

Validate that snapmirror relationship comes to Insync state after deletion of cg item
This validation can be skipped if validate_expand_only is set to 1.
Verify that the -expand output from show has updated list of cg items
Validate that cg items deleted from cg relationship are now read/write on destination
This validation can be skipped if validate_expand_only is set to 1.

Validation for access (r/w or read) is done for lun cg items only.

=over

=item example

    Validate_del_cg_item(
                        node_src     => $node_src_obj,
                        node_dst     => $node_dst_obj,
                        dst_path     => $snapmirror_obj->{destination_path},
                        src_path     => $snapmirror_obj->{source_path},
                        dst_vol_name => $vol_dst_name,
                        cg_item_list => \@cg_item_array,
                        validate_expand_only => 0,
                        );

=item options

=over

=item node_src

(required) NACL Object for Source Node which owns the volume.

=item node_dst

(required) NACL Object for Destination Node which owns the volume.

=item dst_path

(required) Destination Path - the destination path of snapmirror relationship

=item src_path

(required) Source Path - the source path of snapmirror relationship

=item cg_item_list

(required) List of files/luns to be deleted from cg relationship

=item validate_expand_only

(required) If set, will skip validation to check if relationship is InSync and also skip validation to check if items became r/w. Default 0.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=cut

sub Validate_del_cg_item {
_validate_add_del_cg_item(@_,action=>'del');
}

# private sub
sub _validate_add_del_cg_item {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node_src     => {type => OBJECT, },
            node_dst     => {type => OBJECT, },
            dst_path     => {type => SCALAR, },
            src_path     => {type => SCALAR, },
            dst_vol_name => {type => SCALAR, },
            cg_item_list => {type => ARRAYREF, },
            validate_expand_only => {type => SCALAR, default => 0},
            action => {type => SCALAR, regex => qr/(add|del)/},
            }
        );

    my $node_src     = $args{node_src};
    my $node_dst     = $args{node_dst};
    my $src_path     = $args{src_path};
    my $dest_path    = $args{dst_path};
    my $dst_vol_name = $args{dst_vol_name};
    my $cg_item_list = $args{cg_item_list};
    my $validate_expand_only = $args{validate_expand_only};
    my $action = $args{action};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $error_text;
    if (!$validate_expand_only) {
        # Verify on SRC side if relationship is InSync via list-destinations command
        my $list_destinations = NACL::CS::SnapmirrorListDestinations->fetch( command_interface => $node_src,
                                                                    filter => {
                                                                                'destination-path' => $dest_path,
                                                                                'source-path'      => $src_path,
                                                                                },
                                                                    allow_empty => 0,
                                                                    );
        if ($list_destinations->status() !~ /InSync/){
            my $err = 'Did not find CG relationship InSync (SRC side) after delting cg items.';
            $log->log($err);
            $error_text .= $err;
        } else {
            $log->log('Snapmirror relationship (SRC side) is in status=InSync');
        }
        # Verify on DST if relationship is InSync check status=InSync and state=Snapmirrored via snapmirror show command
        my $sm_state = NACL::CS::Snapmirror->fetch( command_interface => $node_dst,
                                                   filter            => {
                                                          'source-path'      => $src_path,
                                                          'destination-path' => $dest_path,
                                                    },
                                                    allow_empty => 0,
                                                );
        if ( $sm_state->{unhealthy_reason}    =~ /-/i &&
             $sm_state->{last_transfer_error} =~ /-/i &&
             $sm_state->{status}              =~ /InSync/ &&
             $sm_state->{state}               =~ /Snapmirrored/ ) {
            $log->log('Snapmirror relationship (DST side) is in status=InSync and state=Snapmirrored without errors');
        } else {
            my $err = 'Did not find CG relationship InSync (DST side) after deleting cg items.';
            $log->log($err);
            $error_text .= $err;
        }
    } else {
        $log->log('Skipping validation to check if relationship is InSync.');
    }

    # Validate that items removed got removed from the snapmirror show - expand
    my @sm_items = NACL::CS::Snapmirror->fetch( command_interface => $node_dst,
                                            filter            => {
                                                                'destination-path' => $dest_path,
                                                                expand             => 'true',
                                                                }
                                            );
    my @found_items;
    foreach my $cg_item (@{$cg_item_list}) {
        if ($action eq 'del') {
            $log->log('Verify cg item '.$cg_item.' missing from the expand output');
            if (grep { $_->destination_path =~ /\/vol\/.*\/$cg_item$/ } @sm_items) {
                push @found_items, $cg_item;
            }
        } else {
            $log->log('Verify cg item '.$cg_item.' present in the expand output');
            if (!grep { $_->destination_path =~ /\/vol\/.*\/$cg_item$/ } @sm_items) {
                push @found_items, $cg_item;
            }
        }
    }

    if (scalar(@found_items)) {
        my $err = ($action eq 'del') ? "Un-expected result!!!. Found items, @found_items in the expand output" :
                                       "Un-expected result!!!. Did not find items, @found_items in the expand output";
        $log->log($err);
        $error_text .= $err;
    } else {
        my $msg = ($action eq 'del') ? 'As expected, CG items removed did not appear in the expand output' :
                                       'As expected, CG items added did appear in the expand output';
        $log->log($msg);
    }

    # Validate that items deleted are now r/w
    # Validate that items added are now r
    if (!$validate_expand_only) {
        my $expected_access = ($action eq 'del') ? 'false' : 'true';
        my $access = ($expected_access eq 'false') ? 'read-write' : 'read';
        $log->log('Validate destination access='.$access.' for '.$action.' items');
        my @lun_items = NACL::CS::Lun->fetch( command_interface => $node_dst,
                                            filter           => {
                                                                volume      => $dst_vol_name,
                                                                'read-only' => $expected_access,
                                                                }
                                            );
        my @not_found_items;
        foreach my $cg_item (@{$cg_item_list}) {
            if (!grep { $_->path =~ /\/vol\/.*\/$cg_item$/ } @lun_items) {
                push @not_found_items, $cg_item;
            }
        }
        if (scalar(@not_found_items)) {
            my $err = "Un-expected result!!!. Expected destination access=$access. Destination access for items @not_found_items was not as expected";
            $log->log($err);
            $error_text .= $err;
        } else {
            my $msg = "Expected destination access=$access. Destination access for $action cg items was found as expected";
            $log->log($msg);
        }
    } else {
        $log->log('Skipping validation to check read and write access of '.$action.' cg items on destination.');
    }

    # throw an exception if any of the validations fail
    if ($error_text) {
        NATE::BaseException->throw($error_text);
    } else {
        $log->log('Validation after '.$action.' cg items succeeded');
    }

    return;
}


=head2 validate_object_packing_packed

Validation whether packing happened or not on File Level and Datawarehouse Level.

=over

=item example

    validate_object_packing_packed(
                        node               => $node,
                        src_vol	   => $vol_src_name,
                        dst_vol_list	   => [$vol_mid_name, $vol_dst_name],
			file_list	   => \@files,
			file_packed_to_vol => {
					     $vol_src_name => 1,
					     $vol_mid_name => 0,
					     $vol_dst_name => 1
					     },
			dw_packed_to_vol   => {
					     $vol_mid_name => 0,
					     $vol_dst_name => 1
					     },
			dw_check => 1
    );

=item options

=over

=item src_vol

(required) OBJECT. Source volume object

=item dst_vol_list

(optional) ARRAYREF. Destination volume objects

=item file_list

(required) ARRAYREF. List of files/luns to be checked

=item file_packed_to_vol

(required) HASHREF. Expecting compaction on files. Keys are vol names as scalars.
Values are corresponding scalars of the expecting packed values. 1: packed.

=item dw_packed_to_vol

(required) HASHREF. Expecting compaction on datawarehouse files. Keys are vol names
as scalars. Values are corresponding scalars of the expecting packed values. 1: packed.

=item dw_check

(optional) SCALAR. Datawarehouse check or not. default to 0.

=back

=item return

    1: fail; 0: pass

=cut


sub validate_object_packing_packed{

    my %args = validate_with(
        params => \@_,
        spec   => {
            src_vol	        => {type => OBJECT, },
            file_list	    => {type => ARRAYREF, },
            dst_vol_list	=> {type => ARRAYREF, optional => 1, default => undef},
	    file_packed_to_vol  => {type => HASHREF, optional => 1, default => undef},
	    dw_packed_to_vol	=> {type => HASHREF, optional => 1, default => undef},
	    dw_check	        => {type => SCALAR, optional => 1, default => 0,},
            }
        );

    my $src_vol	            = $args{src_vol};
    my $dst_vol_list	    = $args{dst_vol_list};
    my $file_list	        = $args{file_list};
    my $file_packed_to_vol  = $args{file_packed_to_vol};
    my $dw_packed_to_vol    = $args{dw_packed_to_vol};
    my $dw_check	        = $args{dw_check};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $result = 0;

    foreach my $file (@{$file_list}) {
	# handle src vol
	my $src_node_obj = NACL::C::Node->new(name => $src_vol->get_local_node_name);
	my $apiset = $src_node_obj->apiset();
        my $src_vol_name = $src_vol->volume();

	$log->comment('Get the 1st vvbn number of $file on src volume ' . $src_vol_name);
	my $cmd = "system node run -node local -command \"priv set test; wafl showvbns -k \/vol\/$src_vol_name\/$file\"";
	my $fbn = $apiset->execute_command(command => $cmd);

	(my $vvbn_number) = $fbn =~ /(\d+)$/m;
	$log->comment("vvbn_number = $vvbn_number\n");

	if($file_packed_to_vol->{$src_vol_name}){
	    $log->comment("Verify compaction on src file $file");
	    my $src_packed = $file_packed_to_vol->{$src_vol_name};
	    if ($fbn =~ /[(]P : \d+[)]/) {
		$log->error("Packed found in source volume file $file") if (!$src_packed);
		$result = 1 if (!$src_packed);
	    } else {
		$log->error("Packed NOT found in source volume file $file") if ($src_packed);
		$result = 1 if ($src_packed);
	    }

	    $log->comment("Print out wafl pack stats");
	    $apiset->execute_command(command => "system node run -node local -command \"priv set test; wafl pack stats\"");
	}

	# handle mid/dst vols
	foreach my $dst_vol (@{$dst_vol_list}) {
	    my $dst_node_obj = NACL::C::Node->new(name => $dst_vol->get_local_node_name);
	    $apiset = $dst_node_obj->apiset();
	    my $dst_vol_name = $dst_vol->volume();

	    $log->comment('Get the 1st vvbn number of $file on dst volume ' . $dst_vol_name);
	    $cmd = "system node run -node local -command \"priv set test; wafl showvbns -k \/vol\/$dst_vol_name\/$file\"";
	    $fbn = $apiset->execute_command(command => $cmd);

	    (my $vvbn_number_dst) = $fbn =~ /(\d+)$/m;
	    $log->comment("vvbn_number = $vvbn_number_dst\n");

	    if($file_packed_to_vol->{$dst_vol_name}){
		$log->comment("Verify packed on dst file $file");
		my $dst_packed = $file_packed_to_vol->{$dst_vol_name};
		if ($fbn =~ /[(]P : \d+[)]/) {
		    $log->error("Packed found in dst volume file $file") if (!$dst_packed);
		    $result = 1 if (!$dst_packed);
		} else {
		    $log->error("Packed NOT found in dst volume file $file") if ($dst_packed);
		    $result = 1 if ($dst_packed);
		}
	    }

	    if($dw_check) {
		$log->comment('Verify compaction on datawarehouse files on volume ' . $dst_vol_name );
		$cmd = "system node run -node local -command \"priv set test; wafl showvbns -k \/vol\/$dst_vol_name\/metadir\/datawarehouse\/wholeVolRep\/dataStore\/map_4k $vvbn_number\"";
		my $dw = $apiset->execute_command(command => $cmd);

		if($dw_packed_to_vol->{$dst_vol_name}){
		    $log->comment("Verify packed on dw file $file");
		    my $dst_dw_packed = $dw_packed_to_vol->{$dst_vol_name};
		    if ($dw =~ /[(]P : \d+[)]/) {
			$log->error("Packed found in volume dw file $file") if (!$dst_dw_packed);
			$result = 1 if (!$dst_dw_packed);
		    } else {
			$log->error("Packed NOT found in volume dw file $file") if ($dst_dw_packed);
			$result = 1 if ($dst_dw_packed);
		    }
		}
	    }

	    $log->comment("Print out wafl pack stats");
	    $apiset->execute_command(command => "system node run -node local -command \"priv set test; wafl pack stats\"");
	}
    }

    $log->exit() if $may_exit;
    return $result;
}



=head2 validate_object_packing_flags

Validation whether packing flags/bootargs been set properly.

=over

=item example

    validate_object_packing_flags(
                        nodes         => [$node1, $node2],
			aggrs_to_node => {  $node1_name => [aggr1, aggr2],
					    $node2_name => [aggr3, aggr4] },
    );

=item options

=over

=item nodes

(required) ARRAYREF. node objs being check.

=item aggrs_to_node

(required) HASHREF. Keys are node names as scalars.
Values are corresponding scalars of the aggrs being checked.

=back

=item return

    1: fail; 0: pass

=cut

sub validate_object_packing_flags{

    my %args = validate_with(
        params => \@_,
        spec   => {
		nodes => {type => ARRAYREF, },
		aggrs_to_node => {type => HASHREF}
            }
        );

    my $nodes = $args{nodes};
    my $aggrs = $args{aggrs_to_node};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $result = 0;

    foreach my $node (@{$nodes}) {
	my $apiset = $node->apiset();

	# check if the platform is non-AFF
	my $cmd = "system node show local -fields is-all-flash-optimized";
	my $retn = $node->apiset()->execute_command(command => $cmd);
	if($retn =~ /false/i) { # non-AFF
	    $log->comment('non-AFFp: need to check bootargs for OP');
	    $cmd = "system node run -node local -command bootargs dump";
	    $retn = $apiset->execute_command(command => $cmd);
	    if ($retn !~ /wafl-enable-packing/i) {
		$log->warn('bootarg wafl-enable-packing NOT found');
		$result = 1;
	    }

	    #$cmd = "system node run -node local -command bootargs dump";
	    #$retn = $apiset->execute_command(command => $cmd);
	    if ($retn !~ /wafl-enable-non-aff-aggr-packing/i) {
		$log->error('bootarg wafl-enable-non-aff-aggr-packing NOT set');
		$result = 1;
	    }
	}

	$cmd = "system node run -node local -command printflag repl_read_compressed";
	$log->comment('check flag repl_read_compressed');
	$retn = $apiset->execute_command(command => $cmd);
	#flag repl_read_compressed,repl_phys_read_compressed_data vaule changed to 1 in FT. burt 1096853
	if ($retn !~ /1/i) {
	    $log->error('flag repl_read_compressed NOT set');
	    $result = 1;
	}

	$cmd = "system node run -node local -command printflag repl_phys_read_compressed_data";
	$log->comment('check flag repl_phys_read_compressed_data');
	$retn = $apiset->execute_command(command => $cmd);
	if ($retn !~ /1/i) {
	    $log->error('flag repl_phys_read_compressed_data NOT set');
	    $result = 1;
	}

	my $aggr_list = $aggrs->{$node->node};
	foreach my $aggr (@{$aggr_list}) {
	    $cmd = "system node run -node local -command wafl pack status -A $aggr";
	    $log->comment('check aggr pack for ' . $aggr);
	    $retn = $apiset->execute_command(command => $cmd);
	    if ($retn !~ /Compaction is enabled on aggregate/i) {
		$log->error('aggr $aggr pack NOT set');
		$result = 1;
	    }
	}
    }

    $log->exit() if $may_exit;
    return $result;
}

=head2 Validate_snapmirror_logs

Validation whether certain logs are shown up in snapmirror logs.

=over

=item example

    Validate_snapmirror_logs(
                        op     => 'Initialize|Update',
			rst_id => $sm_id,
			log_reader => $log_reader,
			val_to_assert => $string_to_check,
			timeout => 600,
    );

=item options

    op  (required): sm operation'Initialize|Update',
    rst_id (required): relationship id,
    log_reader (required): NACL::MTask::SystemLogDetector obj,
    val_to_assert (required): string_to_check,
    timeout (optional): timeout for waiting the logs,

=back

=item return

    1: fail; 0: pass

=cut

sub Validate_snapmirror_logs {
    my %args = validate_with(
        params => \@_,
        spec   => {
	    op 		  => {type => SCALAR},
	    rst_id 	  => {type => SCALAR},
	    log_reader 	  => {type => OBJECT},
	    val_to_assert => {type => SCALAR},
	    timeout 	  => {type => SCALAR, default => '600'},
	    err_to_find   => {type => SCALAR|UNDEF, optional => 1},
        }
    );

    my $log_reader = $args{log_reader};
    my $timeout = $args{timeout};
    my $err_to_find = $args{err_to_find};

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my $result = 1;
    try {
	$log_reader->check(
	    check_for_all_presence => [
		qr/($args{op}).*($args{rst_id}).*($args{val_to_assert})/
	    ],
	    wait_for_log => $timeout,
	);
	$result = 0;
    } catch NATE::BaseException with {
	my $ex = shift;
	if($err_to_find){
	    $log_reader->check(
		check_for_all_presence => [
		    qr/($args{op}).*($args{rst_id}).*($err_to_find)/
		],
	    );
	    $log->warn('Hit expected error: ' . $err_to_find);
	} else {
	    $ex->throw();
	}
    };

    $log->exit() if $may_exit;
    return $result;

} ## end sub _validate

=head2 Validate_appcg_forward_transfer

Validation for AppCG Forward Transfer whether LRSE, LRE, or Transfer Failed.

=over

=item example

    Validate_appcg_forward_transfer(
            src_comp_info => $src_comp_ver,
            dst_comp_info => $dst_comp_ver,
            src_ontap_version => $src_ontap_version,
            dst_ontap_version => $dst_ontap_version,
    );

=item options

    src_ontap_ver  (required): Ontap version for src node ,
    dst_ontap_ver  (required): Ontap version for dst node ,
    comp_info_src_afs  (required): Compression Info of source AFS  ,
    comp_info_dst_afs  (required): Compression Info of dst AFS  ,
    timeout (optional): timeout for waiting the logs,

=back

=item return

    1: fail; 0: pass

=cut

sub Validate_appcg_forward_transfer {
    my %args = validate_with(
        params => \@_,
        spec => {
            src_comp_info => { type => SCALAR },
            dst_comp_info => { type => SCALAR },
            src_ontap_version => { type => SCALAR },
            dst_ontap_version => { type => SCALAR },
        },
    );

   my $src_comp_info = $args{src_comp_info};
   my $dst_comp_info = $args{dst_comp_info};
   my $src_ontap_version = $args{src_ontap_version};
   my $dst_ontap_version = $args{dst_ontap_version};

   my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
   my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
   my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

   my $val_to_assert;
   my $log = log_global();

   if( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'FT_HIGHER' ) {
       $log->log( "Both src and dst nodes have FT or higher release" );
       if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
           $val_to_assert = $val_to_assert_LRSE;
       } else {
           $val_to_assert = $val_to_assert_LRE;
       }
   } elsif ( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'HN_LOWER' ){
       $log->log( "Src node has FT or higher release, and Dst node has HN or lower release" );
       if ( $src_comp_info eq '1028' || $src_comp_info eq '2056' ){
               $val_to_assert = $val_to_assert_LRE;
       } elsif ( $src_comp_info eq '0' || $dst_comp_info eq '0' ||
             $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
       } else {
           $val_to_assert = $val_to_assert_LRE;
       }
   } else {
       $log->log( "Src node has HN or lower release, and Dst node has FT or higher release" );
       if ( $src_comp_info eq '0' or $dst_comp_info eq '0' or $src_comp_info == $dst_comp_info ){
           $val_to_assert = $val_to_assert_LRSE;
       } else {
           $val_to_assert = $val_to_assert_LRE;
       }
   }
} #end of Validate_appcg_forward_transfer

sub Verify_checkpoint {
    my %args = validate_with(
        params => \@_,
        spec => {
            sm_obj => { type => OBJECT, },
            checkpoint => { type => SCALAR, },
            wait_time => { type => SCALAR, default => 300, },
        },
    );

    my $log = log_global();

    my $sm_obj = $args{sm_obj};
    my $checkpoint = $args{checkpoint};

    #--------------------------------------------------------------------
    $log->step( "Verify the checkpoint restarted from the same point. old checkpoint is $checkpoint" );
    #--------------------------------------------------------------------
    $sm_obj->wait_on_attribute(
        'method-timeout'     => $args{wait_time},
        'attribute_to_check' => 'snapshot-checkpoint',
        'till_value'         => qr/[1-9]\d*/,
    );

    my $checkpoint_new = $sm_obj->snapshot_checkpoint;
    my $i              = 3;
    while ( $checkpoint_new !~ /[1-9]/ && $i != 0 ) {
        sleep(10);
        $i--;
        $checkpoint_new = $sm_obj->snapshot_checkpoint;
    }

    $log->log( "New value of snapshot-checkpoint: $checkpoint_new" );

    if ( $checkpoint_new < $checkpoint ) {
       my $error_text
            = "The checkpoint after the update restart is less than the "
        . "checkpoint before the restart. This means the checkpoint "
        . "feature is not working. Fail.";
        $log->error($error_text);
    } else {
        $log->log('Checkpoint validation was good');
    }
}

sub Validate_appcg_restore {
    my %args = validate_with(
        params => \@_,
        spec => {
            src_comp_info => { type => SCALAR },
            dst_comp_info => { type => SCALAR },
            src_ontap_version => { type => SCALAR },
            dst_ontap_version => { type => SCALAR },
            local_restore => { type => SCALAR },
            remote_restore => { type => SCALAR },
        },
    );

   my $log = log_global();
   my $src_comp_info = $args{src_comp_info};
   my $dst_comp_info = $args{dst_comp_info};
   my $src_ontap_version = $args{src_ontap_version};
   my $dst_ontap_version = $args{dst_ontap_version};

   my $local_restore = $args{local_restore};
   my $remote_restore = $args{remote_restore};

   my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
   my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
   my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

   my $val_to_assert;

   if( $local_restore eq '1' ) {
       if( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'FT_HIGHER' ) {
           $log->log( "Both src and dst nodes have FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_failure;
           }
       }
   }

   if( $remote_restore eq '1' ) {
       if( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'FT_HIGHER' ) {
           $log->log( "Both src and dst nodes have FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       } elsif ( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'HN_LOWER' ){
           $log->log( "Src node for restore has FT or higher release, and Dst node for restore has HN or lower release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' ||
                 $src_comp_info == $dst_comp_info  &&
                 ( $src_comp_info ne '1028' || $src_comp_info ne '2056' ||
                   $dst_comp_info ne '1028' || $dst_comp_info ne '2056' )){
                   $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       } else {
           $log->log( "Src node has HN or lower release, and Dst node has FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       }
    }
    $log->log( "The expected string is $val_to_assert" );
    return $val_to_assert;
} #end of Validate_appcg_restore


sub Validate_appcg_sfr {
    my %args = validate_with(
        params => \@_,
        spec => {
            src_comp_info => { type => SCALAR },
            dst_comp_info => { type => SCALAR },
            src_ontap_version => { type => SCALAR },
            dst_ontap_version => { type => SCALAR },
            local_sfr => { type => SCALAR },
            remote_sfr => { type => SCALAR },
        },
    );

   my $log = log_global();
   my $src_comp_info = $args{src_comp_info};
   my $dst_comp_info = $args{dst_comp_info};
   my $src_ontap_version = $args{src_ontap_version};
   my $dst_ontap_version = $args{dst_ontap_version};

   my $local_sfr = $args{local_sfr};
   my $remote_sfr = $args{remote_sfr};

   my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
   my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
   my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

   my $val_to_assert;

   if( $local_sfr eq '1' ) {
       if( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'FT_HIGHER' ) {
           $log->log( "Both src and dst nodes have FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       }
   }

   if( $remote_sfr eq '1' ) {
       if( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'FT_HIGHER' ) {
           $log->log( "Both src and dst nodes have FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       } elsif ( $src_ontap_version eq 'FT_HIGHER' && $dst_ontap_version eq 'HN_LOWER' ){
           $log->log( "Src node has FT or higher release, and Dst node has HN or lower release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' ||
                 $src_comp_info == $dst_comp_info  &&
                 ( $src_comp_info ne '1028' || $src_comp_info ne '2056' ||
                   $dst_comp_info ne '1028' || $dst_comp_info ne '2056' )){
                   $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       } else {
           $log->log( "Src node has HN or lower release, and Dst node has FT or higher release" );
           if ( $src_comp_info eq '0' || $dst_comp_info eq '0' || $src_comp_info == $dst_comp_info ){
               $val_to_assert = $val_to_assert_LRSE;
           } else {
               $val_to_assert = $val_to_assert_LRE;
           }
       }
    }
    $log->log( "The expected string is $val_to_assert" );
    return $val_to_assert;
} #end of Validate_appcg_sfr

=head2 lsa_validation

    Validates physical size and logical size of the file

=item Options

    (Required) logical_file_size  - SCALAR
    (Required) physical_file_size - SCALAR
    (Required) node				  - OBJECT
    (Required) volume_name		  - SCALAR
    (Required) file_name		  - SCALAR

=cut

sub lsa_validation {
	my %args = validate_with(
        params => \@_,
        spec   => {
            logical_file_size  => { type => SCALAR, },
            physical_file_size => { type => SCALAR, },
            node               => { type => OBJECT, },
            volume_name        => { type => SCALAR, },
			file_name          => { type => SCALAR, },
        },
    );
    my $logical_file_size  = $args{logical_file_size};
    my $physical_file_size = $args{physical_file_size};
    my $node               = $args{node};
    my $vol_name           = $args{volume_name};
	my $file_name          = $args{file_name};
	my $result = 0;
    my $log = log_global();

	$log->step(
        'validate physical size logical size on volume '
            . $vol_name );
    my $l_p_size = $node->apiset->system_node_run(
        command =>
            "\"priv set -q test; testsetattr /vol/$vol_name/$file_name -l \"",
        node => $node->node,
    )->get_raw_output();

	$l_p_size =~ /\ssize:\s(\d+)/;
	if ( $1 == $logical_file_size ) {
        $log->log('file matching with logical size '
                . $1
                . ' on volume '
                . $vol_name );
	} else {
        $log->warn(
            'file did not match with logical size  on volume ' . $vol_name );
		$result = 1;
	}

	$l_p_size =~ /\sphysical_size:\s(\d+)/;
    if ( $1 == $physical_file_size ) {
        $log->log('file matching with physical size '
                . $1
                . ' on volume '
                . $vol_name );
    } else {
        $log->warn( 'file did not match with physical size  on volume '
                . $vol_name );
		$result = 1;
	}

	return $result;
}

=head2 verify_transferring

    verify the state the snapmirror relationship is transfering

=item Options

    (Required) sm_objs  - OBJECT

=cut

sub verify_transferring {
    my %args = validate_with(
        params => \@_,
        spec   => { sm_objs => { type => OBJECT, optional => 0 }, 
                    ignore_verify_transferring_err => { type => SCALAR, optional => 1, default => 'false', regex => qr/^(true|false)$/ },
                  },
    );

    my $log = log_global(1);
    my $may_enter = $log->may_enter();
    my $may_exit = $log->may_exit();
    $log->enter() if $may_enter;

    my @smobjs;
    my $sm_obj = $args{'sm_objs'};
    my $ignore_verify_transferring_err = $args{'ignore_verify_transferring_err'};
    try {
        $sm_obj->wait_on_attributes(
            'method-timeout' => 60 * 15,
            attributes       => [
                {   attribute_to_check => 'status',
                    till_value         => qr/Transferring|Finalizing/i
                }
            ]
        );
    } catch NACL::Exceptions::Timeout with {
        my $e = shift;
        my $class = ref $e;

        if ($ignore_verify_transferring_err eq 'true') {

            $log->warn(
                "Caught exception: $e while waiting " .
                "for relationship to become Transferring.  Continuing."
            );
        } else {
            $e->throw();
        }
    }

} ## end sub verify_transfering

=head2 lsa_verify_in_progress

    verify lsa is in progress or no

=item Options


    (Required) node		      - OBJECT
    (Required) vol_name		  - SCALAR


=cut

sub lsa_verify_in_progress {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node     => { type => OBJECT, },
            vol_name => { type => SCALAR, },
        },
    );
    my $node     = $args{node};
    my $vol_name = $args{vol_name};
    my $Log      = log_global();
    $Log->enter();
    my $counter = 30;
    my $lsa_active;
    while (1) {
        my $cmd
            = "volume explore -format volume -dump zdichain-lsa-cnt $vol_name";
        $lsa_active = $node->apiset->execute_raw_command( command => $cmd, );
        if ( $lsa_active !~ /zdichain-lsa-cnt=0/ ) {
            $Log->log("Verified successfully LSA in Progress");
            last;
        } else {
            $Log->log('LSA is still not active, waiting for 1 sec...');
            sleep(1);
            $counter = $counter - 1;
            if ( $counter == 0 ) {
                $Log->error(
                    'Time-out error: Even after 30 sec lsa is still not active'
                );
                $Log->exit();
            } ## end if ( $counter == 0 )
        } ## end else [ if ( $lsa_active !~ /zdichain-lsa-cnt=0/)]
    } ## end while (1)

} ## end sub lsa_verify_in_progress

=head2 lsa_count_verify

    verify lsa count

=item Options


    (Required) node	- OBJECT

=cut

sub lsa_count_verify {
    my %args = validate_with(
        params => \@_,
        spec   => { node => { type => OBJECT }, }
    );
    my $node = $args{node};
    my $Log  = log_global();
    $Log->enter();
    my $cmd       = "wafl stats lsa";
    my $lsa_stats = $node->apiset->system_node_run(
        command => $cmd,
        node    => $node->node(),
    )->get_raw_output();

    if ( $lsa_stats =~ /lsa\.lsa_cnt\s+=\s+(\d+)/ ) {
        $Log->log("LSA COUNT $1");
        return $1;
    } else {
        $Log->error('LSA counter lsa_cnt_cnt not Found');
        $Log->exit();
    }

} ## end sub lsa_count_verify

=head2 lsa_verify_to_complete

    verify lsa is completed or no

=item Options


    (Required) node		      - OBJECT
    (Required) vol_name		  - SCALAR


=cut

sub lsa_verify_to_complete {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node     => { type => OBJECT, },
            vol_name => { type => SCALAR, },
        },
    );
    my $node     = $args{node};
    my $vol_name = $args{vol_name};
    my $Log      = log_global();
    $Log->enter();
    my $counter = 50;
    my $lsa_active;
    while (1) {
        my $cmd
            = "volume explore -format volume -dump zdichain-lsa-cnt $vol_name";
        $lsa_active = $node->apiset->execute_raw_command( command => $cmd, );
        if ( $lsa_active =~ /zdichain-lsa-cnt=0/ ) {
            $Log->log("Verified successfully LSA completed");
            last;
        } else {
            $Log->log('LSA is still active, waiting for 30 sec...');
            sleep(30);
            $counter = $counter - 1;
            if ( $counter == 0 ) {
                $Log->error(
                    'Time-out error: Even after 1500 sec lsa is still active'
                );
                $Log->exit();
            } ## end if ( $counter == 0 )
        } ## end else [ if ( $lsa_active =~ /zdichain-lsa-cnt=0/)]
    } ## end while (1)

} ## end sub lsa_verify_to_complete

1;

