#
# Copyright (c) 2014-2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Client;

use strict;
use warnings;

use San::SanClient;
use Data::Dump qw(dump);
use NATE::STAF qw(staf_global);
use NATE::STAF::Peer;
use NATE::Process;
use NATE::Log qw(log_global);
use NATE::Util::Sockets qw(get_address);
use NATE::Messenger qw(messenger_global);
use NATE::ProcManager;
use NATE::Events qw(eventloop_drain);
use NATE::BaseException qw(:try);
use Sys::Hostname;

use parent qw(NACL::AppBuilder::EnGenX::Client::Setup
              NACL::AppBuilder::EnGenX::Client::Cleanup
              NACL::AppBuilder::EnGenX::Client::Config
              NACL::AppBuilder::EnGenX::Client::Deconfig
              NACL::AppBuilder::EnGenX::Util);

sub new {
    my ($class, %opts) = @_;
    my $self = \%opts;
    bless($self, $class);

    # init or reuse a rep socket and store the endpoint
    $self->{endpoint} = messenger_global()->init(socket_type => 'rep',
                                                 action      => 'bind',
                                                 reuse       => 1);

    return $self;
}

sub xml_to_list {
    my ($self, %args) = @_;

    # make sure there are no duplicate client resources
    my %dup_hash = ();
    my $all_clients = $self->_get_all_client_paramsets();
    foreach my $client_elm (@{$all_clients}) {
        my $client_name = $client_elm->{NAME};

        # detect duplicate clients
        $dup_hash{$client_name}++;
        if ($dup_hash{$client_name} > 1) {
            NATE::BaseException->throw("Detected duplicate client entries for the client " . $client_name);
        }
    }

    # process all groups
    my $multi_clients = $self->_get_all_clients_paramsets();
    my $group_id = 0;
    foreach my $multi_client (@{$multi_clients}) {
        # remove all the CLIENT\d elements from the datastructure
        my $client_elm = {};
        foreach my $key (keys(%{$multi_client})) {
            # register each client as an individual resource
            if ($key =~ /^CLIENT\d*$/) {
                $client_elm = $multi_client->{$key};
                $client_elm->{GROUP_ID} = $group_id;
                # register each client as a resource
                $self->resource_list()->add_resource(
                    category => "CLIENTS",
                    type     => "clients",
                    data     => $client_elm);
                # remove it from the set
                delete($multi_client->{$key});
            }
        }
        # store the group id on the group obj
        $multi_client->{GROUP_ID} = $group_id;

        # create a resource for the global options of the group
        $self->resource_list()->add_resource(
            category => "CLIENTS",
            type => "client_groups",
            data => $multi_client);
        $group_id++;
    }

    return;
}

sub load_resource {
    my ($self, %args) = @_;

    my $staf = staf_global();

    # get the environment from the resource list
    my $environment = $self->resource_list()->get_resources(category => "ENVIRONMENT",
                                                            type => "environment");
    $environment = $environment->[0];

    # obtain all the client resources from the list and prepare the processes that will load each one
    my $client_resources = $self->resource_list()->get_resources(category => "CLIENTS",
                                                                 type => "clients");

    my @procs = ();

    foreach my $client_resource (@{$client_resources}) {
        # Spawn a new process to load this client
        my $proc = NATE::Process->new(codespec => '{ #EnGenX load ' . $client_resource->get_field("NAME") . '
                                                     use NACL::AppBuilder::EnGenX::Client;
                                                     NACL::AppBuilder::EnGenX::Client::_load_client(); }',
                                      runid    => "client__" . $client_resource->get_field("NAME"),
                                      onexit   => \&NATE::Process::on_exit_propagate_worst_result,
                                      client   => 'localhost',
                                      env_to_inherit => ["ENGENX_DISABLE_MD5_VERIFICATION", "USE_MPC_STAGING"],
                                      params   => { "client_ds" => $client_resource->to_hashref(),
                                                    "environment" => $environment->to_hashref(),
                                                    "staf_parent_handle" => $staf->handle(),
                                                    "staf_parent_endpoint" => get_address(Sys::Hostname::hostname())});
        push(@procs, $proc);
    }

    # set up the Event listener and callback methods
    my $filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Client\/SanClient\/\S+\/Load_Complete/);
    };
    my $callback = sub {
        my ($msg, $listener) = @_;

        # parse out the message information
        my ($client_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Client\/SanClient\/(\S+?)\/Load_Complete/);
        my $egx_client_obj = $listener->{egx_client_obj};

        # rebuilt the serialized object
        my $obj = eval($msg->{message}->{obj_serialized});
        if (not defined($obj)) {
            push(@{$egx_client_obj->{load_errors}}, "Could not eval serialized object for client $client_name");
            $egx_client_obj->{parent}->{listeners_complete_count}++;
            return -1;
        }

        # recreate the  SanClient object using the object template
        $obj = SanClient->new(-obj_template => $obj);
        if ($obj->error()) {
            push(@{$egx_client_obj->{load_errors}}, $obj->errmsg());
            $egx_client_obj->{parent}->{listeners_complete_count}++;
            return -1;
        }

        # store the object on the sanmgr resource hash
        $egx_client_obj->{parent}->{_objects}->{client}->{$obj->hostname()} = $obj;

        # validate the md5 for the obj
        if (not $egx_client_obj->_validate_obj_md5_hash(obj => $obj,
                                                        obj_md5_hex_before => $msg->{message}->{obj_md5_hex})) {
            $egx_client_obj->{parent}->{listeners_complete_count}++;
            push(@{$egx_client_obj->{load_errors}}, "MD5 check for client $client_name has failed.");
            return -1;
        }

        $egx_client_obj->{parent}->{listeners_complete_count}++;
        return 0;
    };
    my $listener = {filter         => $filter,
                    callback       => $callback,
                    egx_client_obj => $self};

    # build the fail listener
    my $fail_filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Client\/SanClient\/\S+\/Load_Failed/);
    };
    my $fail_callback = sub {
        my ($msg, $flistener) = @_;

        my $egx_client_obj = $flistener->{egx_client_obj};

        # parse out the message information
        my ($client_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Client\/SanClient\/(\S+?)\/Load_Failed/);
        push(@{$egx_client_obj->{load_errors}}, "Client $client_name failed to load.");
        $egx_client_obj->{parent}->{listeners_complete_count}++;
        return -1;
    };

    my $fail_listener = {filter         => $fail_filter,
                         callback       => $fail_callback,
                         egx_client_obj => $self};

    return ($listener, $fail_listener, @procs);
}

###############################################################################
# Method:    _load_client
# Objective: Load the client object and send it back to the parent
###############################################################################
sub _load_client {
    my ($class) = @_;

    # fetch params passed into the process
    my $global_params = NATE::ParamSet->get_global();
    my $client_ds = $global_params->get("client_ds");
    my $environment = $global_params->get("environment");
    my $staf_parent_handle = $global_params->get("staf_parent_handle");
    my $staf_parent_endpoint = $global_params->get("staf_parent_endpoint");

    my $parent = NATE::STAF::Peer->new(handle   => $staf_parent_handle,
                                       endpoint => $staf_parent_endpoint);

    #############################################
    ### General client parameters
    #############################################
    my @errors = ();

    my $conn_type       = $client_ds->{CONN_TYPE}    // 'THARNHOST';
    my $protocol        = $client_ds->{PROTOCOL}     // push(@errors, "Missing required_field for client 'protocol'");
    my $debug_level     = $client_ds->{DEBUG_LEVEL}  // 0;
    my $load_inits      = $client_ds->{LOAD_INITS}   // 1;
    my $alua_support    = $client_ds->{ALUA_SUPPORT} // 1;
    my $mp_type         = $client_ds->{MP_TYPE}      // "";
    my $lvm_type        = $client_ds->{LVM_TYPE}     // "";
    my $hypervisor_type = $client_ds->{HYPERVISOR}   // "";

    # check for argument errors
    if (scalar(@errors)) {
        require NATE::Exceptions::Argument;
        $parent->message_put(type => "NACL/AppBuilder/EnGenX/Client/SanClient/" . $client_ds->{NAME} . "/Load_Failed",
                             message => {});
        NATE::Exceptions::Argument->throw(join("\n", @errors));
    }

    # initialize the pnate attribute if it doesnt exist
    my $pnate = 0;
    if (defined($environment) and exists($environment->{PNATE})) {
        $pnate = $environment->{PNATE};
    }

    # Create a fake self so that we can get the paramset from the "EnGenX" object
    # using the util method.
    my $fake_self = {};
    bless($fake_self, "NACL::AppBuilder::EnGenX");

    # generate a fake object with all the options built into it, so we
    # can pass in the iscsi type value into the sanclient constructor
    # needed for Linux & ESX
    my $obj = {};
    $fake_self->NACL::AppBuilder::EnGenX::Util::assign_sanclient_options(-client_obj => $obj,
                                                       -datastruct => $client_ds,
                                                       -env_datastruct => $environment);

    # Create client object
    my $cl_obj = SanClient->new(-host                     => $client_ds->{NAME},
                                -conn_type                => $conn_type,
                                -debug_level              => $debug_level,
                                -load_inits               => $load_inits,
                                -mp_type                  => $mp_type,
                                -lvm_type                 => $lvm_type,
                                -protocol                 => $protocol,
                                -pnate_environment        => $pnate,
                                -hypervisor_type          => $hypervisor_type,
                                -alua_support             => $alua_support,
                                -iscsi_type               => $obj->{iscsi}->{CONFIG_ISCSI_TYPE}
                            );

    if ($cl_obj->error()) {
        $parent->message_put(type => "NACL/AppBuilder/EnGenX/Client/SanClient/" . $client_ds->{NAME} . "/Load_Failed",
                             message => {});
        NATE::BaseException->throw("Failed to create SanClient object for " . $client_ds->{NAME} . " : " .
                                       $cl_obj->errmsg());
        return;
    }

    # assign all iscsi_options & fcp_options to this sanclient object
    $fake_self->NACL::AppBuilder::EnGenX::Util::assign_sanclient_options(-client_obj => $cl_obj,
                                                       -datastruct => $client_ds,
                                                       -env_datastruct => $environment);

    # destroy the connection to the sanclient object,
    # we have to do this or else the object we send back to the parent gets messed up
    $cl_obj->disconnect_connection();

    # calculate the md5 for the hypervisor obj
    my $obj_md5_hex = NACL::AppBuilder::EnGenX::Util->_calculate_obj_md5_hash($cl_obj);

    $parent->message_put(type => "NACL/AppBuilder/EnGenX/Client/SanClient/" . $client_ds->{NAME} ."/Load_Complete",
                         message => {obj_md5_hex => $obj_md5_hex,
                                     obj_serialized => Data::Dump::dump($cl_obj)});

    return;
}

################################################################################
# Subroutine Name: _kill_io_instances_subtest
# Objective:       Kill all instances of the client program on any client
################################################################################
sub _kill_io_instances_subtest {
    my ($self, %args) = @_;

    my $client            = delete($args{client})   // NATE::BaseException->throw("Missing required parameter 'client'");
    my $resource          = delete($args{resource}) // NATE::BaseException->throw("Missing required parameter 'resource'");
    my $client_io_program = delete($args{client_io_program}) // "all";

    my ($exe_to_kill, @exes_to_kill);
    my (@subtests);

    # skip kill io processes if the flag is disabled
    if ($resource->{KILL_IO_PROCESSES} == 0) {
        log_global()->log("Skipping kill_io_processes because it " .
                          "is disabled for client '$client'");
        return;
    }

    # determine the process application to kill (all instances)
    if ($client_io_program =~ /dt/ or $client_io_program =~ /all/i) {
        push (@exes_to_kill, "dt");
    }

    if ($client_io_program =~ /sio/ or $client_io_program =~ /all/i) {
        push (@exes_to_kill, "sio");
    }

    if ($client_io_program =~ /blast/ or $client_io_program =~ /all/i) {
        push (@exes_to_kill, "blast");
    }

    if ($client_io_program !~ /sio/   and
        $client_io_program !~ /dt/    and
        $client_io_program !~ /blast/ and
        $client_io_program !~ /all/i) {
        NATE::BaseException->throw("No valid client_io_program passed in (must be 'sio', 'dt', 'blast', or 'all')");
    }

    my @procs;
    foreach $exe_to_kill (@exes_to_kill) {
        # kill the I/O process still on the host (all platforms)
        my $proc = NATE::Process->new(codespec => '{ use San::SanClient::ClientUtil; ' .
                                                  '  &ClientUtil::kill_process_instances("' . $client . '","' . $exe_to_kill . '"); }',
                                      client   => 'localhost',
                                      runid    => "io_task_kill_" . $exe_to_kill . "_" . $client,
                                      onexit   => \&NATE::Process::on_exit_die_worst_result);
        push(@procs, $proc);
    }

    my $pm = NATE::ProcManager->new(proc_info => \@procs, runid => "io_task_kill");
    $pm->start();
    $pm->wait();
    $pm->destroy();

    return;
}

1;
