#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Client::Cleanup;

use strict;
use warnings;

use NATE::Process;
use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);
use NATE::BaseException qw(:try);

sub cleanup {
    my ($self, %args) = @_;
    my $run_id_prefix = delete($args{run_id_prefix}) // "";

    my $client_groups = $self->resource_list()->get_resources(category => "CLIENTS",
                                                              type => "client_groups");

    # clear the client dev list stored on the engenx object
    $self->{_client_dev_list} = undef;

    if (!scalar(@{$client_groups})) {
        log_global()->log("No Clients to cleanup");
    }

    my @procs = ();
    my @required_clients = ();
    my $groups = {};
    foreach my $group (@{$client_groups}) {
        if ($group->{CLEANUP} == 0) {
            log_global()->log("Skipping client group " . $group->{GROUP_ID} . " because cleanup is disabled on it");
            next;
        }

        my $group_id = $group->{GROUP_ID};
        $groups->{$group_id}->{max_failures} = $group->{MAX_FAILURE_COUNT};
        my $required_clients = $group->{REQUIRED_CLIENTS};
        push(@required_clients, split(/\s*,\s*/, $required_clients));

        my $client_list = $self->resource_list()->get_resources(category => "CLIENTS",
                                                                type => "clients",
                                                                data => {GROUP_ID => $group_id});
        foreach my $client (@{$client_list}) {
            if ($client->{CLEANUP} == 0) {
                log_global()->log("Skipping client " . $client->{NAME} . " because cleanup is disabled on it.");
                next;
            }
            my $p = NATE::Process->new(runid    => $client->{NAME},
                                       codespec => sub {
                                           try {
                                               $self->cleanup_client(client => $client,
                                                                     group_id => $group_id);
                                               $self->_cleanup_complete();
                                           } otherwise {
                                               my $e = shift;
                                               $self->_cleanup_failure(error => $e->text(), 
                                                                       client => $client->{NAME},
                                                                       group_id => $group_id);
                                               $e->throw();
                                           };
                                       },
                                       onexit   => \&NATE::Process::on_exit_propagate_worst_result,);

            push(@procs, $p);
        }
    }

    # install a listener to handle a cleanup completion
    my $success_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/Client/Cleanup/Complete' },
        callback => sub {
            my ($msg, $l) = @_;
            $l->{completed}++;
        }
    };

    # install a listener to handle a cleanup failure
    my $someone_failed = 0;
    my @failed_clients;
    my $fail_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/Client/Cleanup/Failure'},
        callback => sub {
            my ($msg, $l) = @_;
            $msg = $msg->message();
            $groups->{$msg->{group_id}}->{failed}++;
            push(@failed_clients, $msg->{client});
            $someone_failed = 1;
            $l->{completed}++;
        }
    };

    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => "$run_id_prefix" . "Client-cleanup");

    # check if we had any failures
    if ($someone_failed) {
        foreach my $group (values %{$groups}) {
            # if the number of clients that failed in this group is beyond the max, then that
            # is a failure
            if ($group->{failed} > $group->{max_failures}) {
                NATE::BaseException->throw("A client group has hit its maximum cleanup " .
                                           "failure threshold");
            }
        }
    }

    # make sure that none of the clients that failed were in the required clients list
    foreach my $client (@failed_clients) {
        if (grep(/^$client$/, @required_clients)) {
            NATE::BaseException->throw("The client $client failed cleanup. Aborting since it " .
                                       "was on the required clients list.");
        }
    }

    return;
}

sub cleanup_client {
    my ($self, %args) = @_;

    my $client   = delete($args{client})   // NATE::BaseException->throw("Missing required parameter 'client'");
    my $group_id = delete($args{group_id}) // NATE::BaseException->throw("Missing required parameter 'group_id'");

    #############################################
    ### General client parameters
    #############################################
    my $client_name = $client->{NAME};
    my $protocol = $client->{PROTOCOL};
    my $hypervisor = $client->{HYPERVISOR};
    my $lvm_type = $client->{LVM_TYPE};

    # Get client object
    my ($hostobj) = $self->get_objects(-type => "client", -names => [$client_name]);
    if ($self->error()) {
        NATE::BaseException->throw("Failed to get client ($client_name) object: " . $self->errmsg());
    }

    # if this is a sanblaze host, then no cleanup should be done
    if (ref($hostobj) =~ /Sanblaze/i) {
        log_global()->log("Bypassing cleanup because this is a sanblaze host");
        return;
    }
    
    # reconnect to the client
    $hostobj->reconnect(-disconnect_existing_conn => 0);
    if ($hostobj->error()) {
        NATE::BaseException->throw("Failed to reconnect to client ($client_name) object: " . $self->errmsg());
    }

    # kill off any running I/O program on the client systems.
    $self->_kill_io_instances_subtest(client => $client_name,
                                      resource => $client);

    # cleanup SAN clients for lvm and non lvm
    if ($protocol =~ /FCP|iSCSI/i) {        
        # first try to create an mpdevmgr and force a device rescan
        if (San::SanClient::MPDevMgr::is_supported($hostobj)) {
            $hostobj->create_mpdevmgr(rescan => 1);
            if ($hostobj->error()) {
                NATE::BaseException->throw("Failed to create an MPDevMgr object for client ($client_name) : " . $hostobj->errmsg());
            }
        }
        
        if ($lvm_type ne "" and $lvm_type ne "none") {
            $self->cleanup_san_lvm(client => $client,
                                   host_obj => $hostobj);
        } else {
            $self->cleanup_san_non_lvm(client => $client,
                                       host_obj => $hostobj);

        }
    }

    # clean up NFS for unix hosts
    if ($protocol =~ /nfs/i) {
        $self->cleanup_nfs(client => $client);
    }

    return;
}

sub cleanup_san_lvm {
    my ($self, %args) = @_;
    my $client = delete $args{client};
    my $hostobj = delete $args{host_obj};

    my $lvm_type = $client->{LVM_TYPE};
    my $io_type = $client->{IO_TYPE};

    my (@vg_objs, $vg_obj, @vol_objs, $vol_obj, $lvm_obj);

    # create a lvm object
    $lvm_obj = $hostobj->create_lvm(-lvm => $lvm_type);
    if ($lvm_obj->error()) {
        NATE::BaseException->throw("Failed to create the LVM object : " . $hostobj->errmsg());
    }

    # cycle through the volume groups / logical voulumes and destroy
    # all the objects. Make sure on filesystems to unmount before
    # trying to destroy
    @vg_objs = $lvm_obj->get_vgs();
    foreach my $vg_obj (@vg_objs) {
        @vol_objs = $vg_obj->get_vols();
        foreach my $vol_obj (@vol_objs) {
            if ($io_type =~ /fs/) {
                if ($vol_obj->is_mounted() == 1) {
                    $vol_obj->unmount();
                    if ($vol_obj->error()) {
                        NATE::BaseException->throw("Failed to unmount the filesystem : " . $vol_obj->errmsg());
                    }
                }
            }
            $vol_obj->destroy();
            if ($vol_obj->error()) {
                NATE::BaseException->throw("Failed to destroy volume object : " . $vol_obj->errmsg());
            }
        }
        $vg_obj->destroy();
        if ($vg_obj->error()) {
            NATE::BaseException->throw("Failed to destroy VG object : " . $vg_obj->errmsg());
        }
    }

    # cleanup any orphaned devices, for now just run on AIX, HPUX, and Solaris.
    # Linux and Windows are implemented with reboot and we want to avoid this.
    if ($hostobj->hosttype() =~ /(hpux|solaris|unix-aix)/i) {
        $hostobj->orphaned_device_cleanup();
        if ($hostobj->error()) {
            NATE::BaseException->throw("Unable to cleanup orphaned devices : " . $hostobj->errmsg());
        }
    }

    return;
}

sub cleanup_san_non_lvm {
    my ($self, %args) = @_;

    my $hostobj = delete $args{host_obj};
    my $client = delete $args{client};
    my $io_type = $client->{IO_TYPE};

    # Create a device manager object
    my $devmgrobj = $hostobj->create_devmgr();
    if ($hostobj->error) {
        NATE::BaseException->throw("Failed to create device manager object : " . $hostobj->errmsg());
    }

    if ($io_type =~ /fs/) {
        foreach my $device ($devmgrobj->get_devices()) {
            $device->unmount_fs();
        }
    }
    # cleanup any orphaned devices, for now just run on AIX and HP
    if ($hostobj->hosttype() =~ /unix-aix/i or $hostobj->hosttype() =~ /hpux/i) {
        $hostobj->orphaned_device_cleanup();
        if ($hostobj->error()) {
            NATE::BaseException->throw("Unable to cleanup orphaned devices : " . $hostobj->errmsg());
        }
    }

}

sub cleanup_nfs {
    my ($self, %args) = @_;
    my $client = delete($args{client});
    my $client_name = $client->{NAME};

    # create the Command interface for this client
    my $client_ci;
    require NACL::C::Client;
    require NACL::STask::Client::Mount;
    $client_ci = new NACL::C::Client(name => $client_name,
                                     apiset_must => {conntype => 'ssh-persist',
                                                     connid => 'ssh'});

    # clean up all mount points related to the nfs target hash
    my @mount_objs = NACL::STask::Client::Mount->find(command_interface => $client_ci,
                                                      apiset_must => {conntype => 'ssh-persist',
                                                                      connid => 'ssh'});

    # clean up all mount points that are owned by EnGenX
    foreach my $mount_obj (@mount_objs) {
        # only purge engenx mount points
        if ($mount_obj->mount_point() =~ /engenx/i) {
            $mount_obj->purge(force => 1);
        }
    }

    return;
}

sub _cleanup_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Client/Cleanup/Failure",
                            message  => \%data);
}

sub _cleanup_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Client/Cleanup/Complete",
                            message  => \%data);
}

1;
