#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Client::Setup;

use strict;
use warnings;

use NATE::Process;
use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);
use NATE::BaseException qw(:try);
use Digest::MD5 qw(md5_hex);
use Data::Dumper;

sub setup {
    my ($self, %args) = @_;

    my $run_id_prefix = delete($args{run_id_prefix}) // "";

    my $client_groups = $self->resource_list()->get_resources(category => "CLIENTS",
                                                              type => "client_groups");

    if (!scalar(@{$client_groups})) {
        log_global()->log("No Clients to setup");
    }

    my @procs = ();
    my $all_dev_list = {};
    my @required_clients = ();
    my $groups = {};
    foreach my $group (@{$client_groups}) {
        if ($group->{SETUP} == 0) {
            log_global()->log("Skipping client group " . $group->{GROUP_ID} . " because setup is disabled on it");
            next;
        }

        my $group_id = $group->{GROUP_ID};
        $groups->{$group_id}->{max_failures} = $group->{MAX_FAILURE_COUNT} // $group->{MAX__SETUP_FAILURE_COUNT};
        my $required_clients = $group->{REQUIRED_CLIENTS};
        push(@required_clients, split(/\s*,\s*/, $required_clients));

        my $client_list = $self->resource_list()->get_resources(category => "CLIENTS",
                                                                type => "clients",
                                                                data => {GROUP_ID => $group_id});
        foreach my $client (@{$client_list}) {
            if ($client->{DEV_LIST} and $client->{DEV_LIST} ne "") {
                # this client already has a dev list,
                # store it and skip setup
                $all_dev_list->{$client->{NAME}} = $client->{DEV_LIST};
                next;
            }
            if ($client->{SETUP} == 0) {
                log_global()->log("Skipping client " . $client->{NAME} . " because setup is disabled on it.");
                next;
            }
            my $p = NATE::Process->new(runid    => $client->{NAME},
                                       codespec => sub {
                                           try {
                                               my $device_list = $self->setup_client(client => $client,
                                                                                     group_id => $group_id);
                                               $self->_setup_complete(client => $client->{NAME},
                                                                      dev_list => $device_list);
                                           } otherwise {
                                               my $e = shift;
                                               $self->_setup_failure(error => $e->text(),
                                                                     client => $client->{NAME},
                                                                     group_id => $group_id);
                                               $e->throw();
                                           };
                                       },
                                       onexit   => \&NATE::Process::on_exit_propagate_worst_result);

            push(@procs, $p);
        }
    }

    # install a listener to handle a setup completion
    my $success_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/Client/Setup/Complete' },
        callback => sub {
            my ($msg, $l) = @_;
            $msg = $msg->message();
            my $client_name = $msg->{client};
            my $dev_list = $msg->{dev_list};
            if ($dev_list) {
                $all_dev_list->{$client_name} = $dev_list;
            }
            $l->{completed}++;
        }
    };

    # install a listener to handle a setup failure
    my $someone_failed = 0;
    my @failed_clients;
    my $fail_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/Client/Setup/Failure'},
        callback => sub {
            my ($msg, $l) = @_;
            $msg = $msg->message();
            $groups->{$msg->{group_id}}->{failed}++;
            push(@failed_clients, $msg->{client});
            $someone_failed = 1;
            $l->{completed}++;
        }
    };

    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => "$run_id_prefix" . "Client-setup");

    # check if we had any failures
    if ($someone_failed) {
        foreach my $group (values %{$groups}) {
            # if the number of clients that failed in this group is beyond the max, then that
            # is a failure
            if ($group->{failed} > $group->{max_failures}) {
                NATE::BaseException->throw("A client group has hit its maximum setup " .
                                           "failure threshold");
            }
        }
    }

    # make sure that none of the clients that failed were in the required clients list
    foreach my $client (@failed_clients) {
        if (grep(/^$client$/, @required_clients)) {
            NATE::BaseException->throw("The client $client failed. Aborting since it " .
                                       "was on the required clients list.");
        }
    }

    # store the client dev list
    $self->{parent}->store_client_dev_list(-device_list => $all_dev_list);

    return;
}

sub setup_client {
    my ($self, %args) = @_;

    my $client   = delete($args{client})   // NATE::BaseException->throw("Missing required parameter 'client'");
    my $group_id = delete($args{group_id}) // NATE::BaseException->throw("Missing required parameter 'group_id'");

    my $client_name = $client->{NAME};
    my $protocol = $client->{PROTOCOL};
    my $ntp_sync = $client->{NTP_SYNC} // 1;
    my $ntp_time_server = $client->{NTP_TIME_SERVER} // "ntp0.eng.netapp.com";
    my $ntp_time_zone = $client->{NTP_TIME_ZONE} // "EST5EDT";

    # Get client object
    my ($hostobj) = $self->get_objects(-type => "client", -names => [$client_name]);
    if ($self->error()) {
        NATE::BaseException->throw("Failed to get client ($client_name) object: " . $self->errmsg());
    }

    # if this is a sanblaze host, then no setup should be done
    if (ref($hostobj) =~ /Sanblaze/i) {
        log_global()->log("Bypassing setup because this is a sanblaze host");
        return;
    }

    # reconnect to the client
    $hostobj->reconnect(-disconnect_existing_conn => 0);
    if ($hostobj->error()) {
        NATE::BaseException->throw("Failed to reconnect to client ($client_name) object: " . $self->errmsg());
    }

    # kill off any running I/O program on the client systems.
    $self->_kill_io_instances_subtest(client => $client_name,
                                      resource => $client);

    # NTP time sync the host
    if ($ntp_sync == 1) {
        if ($hostobj->can("ntp_sync")) {
            $hostobj->ntp_sync(-ntp_server => $ntp_time_server,
                               -ntp_time_zone => $ntp_time_zone);
            if ($hostobj->error()) {
                my @error = $hostobj->errmsg();
                log_global()->warn("Failed to NTP time sync the $client_name (@error)");
                $hostobj->_clear_err();
            }
        } else {
            log_global()->log("Skipping NTP setup on $client_name, there is no ntp_sync method implemented its OS");
        }
    }

    my $device_list = [];

    # setup SAN on this client
    if ($protocol =~ /fcp|iscsi/i) {
        my $list = $self->setup_san(host_obj => $hostobj,
                                    client => $client);
        push(@{$device_list}, @{$list});
    }

    # setup NFS for unix hosts hosts
    if ($protocol =~ /nfs/i) {
        my $list = $self->setup_nfs(host_obj => $hostobj,
                                    client => $client);
        push(@{$device_list}, @{$list});
    }

    # return the client dev list
    return $device_list;
}

sub setup_san {
    my ($self, %args) = @_;

    my $hostobj = delete($args{host_obj});
    my $client = delete($args{client});

    #############################################
    ### General client parameters
    #############################################
    my $client_name = $client->{NAME};
    my $protocol = $client->{PROTOCOL};
    my $io_type = $client->{IO_TYPE};
    my $fs_type = $client->{FS_TYPE} // "";
    my $fs_rotate = $client->{FS_ROTATE} // 0;
    my $lun_count = $client->{LUN_COUNT} // "";
    my $lun_path_regex = eval($client->{LUN_PATH_REGEX}) // undef;
    my $mount_options_pass_thru = $client->{MOUNT_OPTIONS_PASS_THRU} // "";
    my $mount_options_discard = $client->{MOUNT_OPTIONS_DISCARD} // 0;
    my $debug_level = $client->{DEBUG_LEVEL} // 0;

    #############################################
    ### VG parameters
    #############################################
    my $mp_type = $client->{MP_TYPE} // "";
    my $lvm_type = $client->{LVM_TYPE} // "";
    my $vm_vol_size = $client->{VM_VOL_SIZE} // "1g";
    my $vm_num_lvs = $client->{VM_NUM_LVS} // 1;
    my $vm_num_fcp_lvs = $client->{VM_NUM_FCP_LVS} // 0;
    my $vm_num_iscsi_lvs = $client->{VM_NUM_ISCSI_LVS} // 0;
    my $vm_lv_type = $client->{VM_LV_TYPE} // "fs";
    my $vm_vg_layout = $client->{VM_VG_LAYOUT} // "concat";
    my $vm_stripe_size = $client->{VM_STRIPE_SIZE} // "64k";

    # Setup some basic default
    my $vm_vg_name = "n_vg";
    my $cf_mode = "ssi";
    my $device_list = [];
    my $device_data;

    # Setup the protocols array with the protocols specified by caller
    my @protocols = ();
    if ($protocol =~ /fcp/i) {
        push(@protocols, "fcp");
    }
    if ($protocol =~ /iscsi/i) {
        push(@protocols, "iscsi");
    }

    my ($vserver_list_ref, $luns_ref) = $self->{parent}->get_luns_and_vservers_associated_to_client(-client_name => $client_name);

    my @vserver_list = @$vserver_list_ref;
    my @luns = @$luns_ref;

    # default num expected luns to at least 1 lun
    my $num_expected_luns = scalar(@luns) ? scalar(@luns) : 1;

    # first try to create an mpdevmgr and force a scsi device rescan
    if (San::SanClient::MPDevMgr::is_supported($hostobj)) {
        $hostobj->create_mpdevmgr(rescan => 1);
        if ($hostobj->error()) {
            NATE::BaseException->throw("Failed to create an MPDevMgr object for client ($client_name) : " . $hostobj->errmsg());
        }
    }

    # discover resources on the host. This includes configuration
    # information (configuration files) and sanlun information.
    # It wil also run devfsadm to discover LUNs
    $hostobj->discover(-num_expected_luns => $num_expected_luns);
    if ($hostobj->error) {
        NATE::BaseException->throw("Host discover failed : " . $hostobj->errmsg());
    }

    # Create a device manager object
    my $devmgrobj = $hostobj->create_devmgr();
    if ($hostobj->error) {
        NATE::BaseException->throw("Failed to create a device manager object : " . $hostobj->errmsg());
    }

    # get the list of devices pathnames to use
    # on disk and filesystem operations. This is
    # extracted from the object which got this information
    # from sanlun
    my @devices = $devmgrobj->get_free_devices();
    if ($devmgrobj->error()) {
        NATE::BaseException->throw("Unable to run get_free_devices :" .  $devmgrobj->errmsg());
    }
    if (scalar(@devices) > 0) {
        log_global()->log("List of discovered raw devices includes:\n" . Dumper(@devices));
    } else {
        NATE::BaseException->throw("No available devices found.");
    }

    if ($lvm_type ne "" and $lvm_type ne "none") {
        # LVM Case:

        # Make sure the volume size is set
        if ($vm_vol_size eq "") {
            NATE::BaseException->throw("VM volume size is not specified");
        }

        # create a lvm object
        my $lvm_obj = $hostobj->create_lvm(-lvm => $lvm_type);
        if ($hostobj->error) {
          NATE:BaseException->throw("Failed to create LVM object : " . $hostobj->errmsg());
        }

        # after we have configured (and created LVM object), validate it's config
        # to make sure everything is setup properly. Currently only warning
        # later we may want to make this more severe
        $hostobj->validate(-cfo_mode => $cf_mode);
        if ($hostobj->error) {
            log_global()->warn("Validation of host ($client_name) failed");
            $hostobj->_clear_err();
        }

        # Default the vm_num_fcp_lvs and vm_num_iscsi_lvs to vm_num_lvs if they are not defined
        if ($vm_num_fcp_lvs eq "" or $vm_num_fcp_lvs == 0) {
            $vm_num_fcp_lvs = $vm_num_lvs;
        }

        if ($vm_num_iscsi_lvs eq "" or $vm_num_iscsi_lvs == 0) {
            $vm_num_iscsi_lvs = $vm_num_lvs;
        }

        log_global()->log("Getting list of free devices\n\tvservers: @vserver_list\n\tprotocols: @protocols");

        # Collect, init and prepare all devices of protocol type fcp or iscsi
        foreach my $protocol (@protocols) {
            # Get the init states "online", "online invalid", "error", "online thinrclm", or "online thin".
            # "online thinrclm" and "online thin" are the same thing.  Some operating systems use thinrcl
            # others only use thin. "online invalid" and "online invalid thinrclm" are the same depending on vsersion of Veritas.
            my @devices_tmp = $lvm_obj->get_free_devices(-inited_states => ["online invalid", "online invalid thinrclm", "error", "online", "online thinrclm", "online thin"],
                                                         -protocol => $protocol,
                                                         -filers => \@vserver_list);
            if ($lvm_obj->error) {
                NATE::BaseException->throw("Failed to get list of free $protocol devices from LVM : " . $lvm_obj->errmsg());
            }

            if (scalar(@devices_tmp) == 0) {
                NATE::BaseException->throw("Failed to get list of free $protocol devices from LVM (device list was empty)");
            }

            log_global()->log("Initializing (" . scalar(@devices_tmp) . ") $protocol devices");

            # Init list of protocol devices
            $lvm_obj->initialize(-devices => \@devices_tmp);
            if ($lvm_obj->error) {
                my $device_list = join("\n", @devices_tmp);
                NATE::BaseException->throw("Failed to initialize $protocol devices\n:${device_list}\n: " .  $lvm_obj->errmsg());
            }

            log_global()->log("Creating volume group for $protocol with the following devices:" .
                                  join("\n\t", @devices_tmp));
            # Now create a volume group using the list of protocol devices that we discovered.
            $device_data->{$protocol}->{'vg_obj'} = $lvm_obj->create_vg(-devices => \@devices_tmp,
                                                                        -name => $vm_vg_name . "_" . substr($protocol, 0, 1));
            if ($lvm_obj->error) {
                NATE::BaseException->throw("Failed to create vg for $protocol devices : " . $lvm_obj->errmsg());
            }

            if ($protocol =~ /fcp/i) {
                $device_data->{$protocol}->{'num_lvs'} = $vm_num_fcp_lvs;
            } else {
                $device_data->{$protocol}->{'num_lvs'} = $vm_num_iscsi_lvs;
            }

            log_global()->log("Creating " . $device_data->{$protocol}->{'num_lvs'} . " logical volumes on " . $vm_vg_name . "_" . substr($protocol, 0, 1));
        }

        # Go through devices found for each protocol
        foreach my $protocol (keys(%{$device_data})) {
            my $vg_obj     = $device_data->{$protocol}->{'vg_obj'};
            my $num_lvs    = $device_data->{$protocol}->{'num_lvs'};
            my @vg_devices = $vg_obj->get_devices();

            # Special case data for SVM lvm type
            my $num_dev_per_vol;
            my @svm_devices;
            if ($lvm_type =~ /svm/i) {
                foreach my $tmp_device (@vg_devices) {
                    if ($tmp_device =~ /s\d+$/) {
                        $tmp_device =~ s/s\d+$/s0/g;
                    } else {
                        $tmp_device .= "s0";
                    }
                    push @svm_devices, $tmp_device;
                }

                # Calculate the number of devices per volume for SVM
                $num_dev_per_vol = scalar(@svm_devices)/ $num_lvs;
                $num_dev_per_vol = int($num_dev_per_vol);
            }

            log_global()->log("Using vg object for protocol $protocol, creating " .
                                  "$num_lvs logical volumes on vg " .
                                      $vg_obj->get_name());

            my $fs_type_index = 0;
            my $mixed_fs_type_verified = 0;
            my @fs_type_mixed;

            for (my $vol = 1; $vol <= $num_lvs; $vol++) {
                my $volume_name = "nvol-" . $vol . "-" . substr($protocol, 0, 1);

                log_global->log("\nCreating Logical Volume ($volume_name) with parameters:\n" .
                                    "\tLayout: $vm_vg_layout\n" .
                                        "\tVolume Size: $vm_vol_size\n" .
                                            "\tProtocol: $protocol\n");

                my $vol_obj;
                if ($lvm_type =~ /svm/i) {
                    # Special case for SVM lvm type.  We need to tell it what devices to
                    # use for each volume.
                    my $device_list_per_vol;
                    for my $i (1 .. $num_dev_per_vol) {
                        $device_list_per_vol = $device_list_per_vol . shift(@svm_devices) . " ";
                    }

                    $vol_obj = $vg_obj->create_vol(-layout           => $vm_vg_layout,
                                                   -size             => $vm_vol_size,
                                                   -name             => $volume_name,
                                                   -num_dev_per_vol  => $num_dev_per_vol,
                                                   -num_of_vols      => $num_lvs,
                                                   -dev_per_vol_list => $device_list_per_vol);

                    if ($vg_obj->error) {
                        NATE::BaseException->throw("Failed to create volume object : " . $vg_obj->errmsg());
                    }
                } else {
                    $vol_obj = $vg_obj->create_vol(-layout       => $vm_vg_layout,
                                                   -size        => $vm_vol_size,
                                                   -stripe_size => $vm_stripe_size,
                                                   -name        => $volume_name);

                    if ($vg_obj->error) {
                        NATE::BaseException->throw("Failed to create volume object" . $vg_obj->errmsg());
                    }
                }

                my $device_info;

                # Save some attributes of the device onto the device_info hash
                ($device_info->{'vserver'}, $device_info->{'volume'}, $device_info->{'lun'},
                 $device_info->{'protocol'}, $device_info->{'serial'}) =
                     ((), (), (), (), $protocol);

                # Determine if the caller wants filesystems created on the
                # newly created volumes
                if ($io_type =~ /fs/) {
                    if ($vm_lv_type !~ /:|mixed/i) {
                        # Same file system will be created on all the volumes.
                        # Create a fs based on the vm_lv_type requested.
                        $vol_obj->create_fs(-type => $vm_lv_type);
                        if ($vol_obj->error) {
                            NATE::BaseException->throw("Failed to create filesystem" . $vol_obj->errmsg());
                        }
                    } else {
                        # Mixed file systems has been requested by user.
                        # There are 2 options,
                        # 1. When user directly provides colon separated fs list in vm_lv_type.
                        # 2. When vm_lv_type is set to "mixed", create mixed fs from the template provided in host libraries.
                        #    It various from one OS to another. (Currently this option is supported only in Linux).
                        if ($mixed_fs_type_verified == 0) {
                                # Initial setup required for mixed fs. Needs to be done only once in the first iteration.
                            if ($vm_lv_type =~ /:/) {
                                # vm_lv_type is set to colon separated fs by user
                                log_global()->log("Mixed FS requested by user: $vm_lv_type");
                                @fs_type_mixed = split(/:/, $vm_lv_type);
                            } elsif ($vm_lv_type =~ /mixed/i) {
                                # vm_lv_type is set to "mixed".
                                # Get mixed fs type template from host library.
                                if ($hostobj->can("get_mixed_fs_template")) {
                                    # Currently it is supported only on Linux
                                    my @fs_array = $hostobj->get_mixed_fs_template();
                                    my $vm_lv_type_temp = join(":", @fs_array);
                                    log_global()->log("Mixed FS template: $vm_lv_type_temp");
                                    @fs_type_mixed = split(/:/, $vm_lv_type_temp);
                                } else {
                                    my $hostname = $hostobj->{host};
                                    NATE::BaseException->throw("Mixed file systems are not supported yet for $hostname");
                                }
                            }
                            $mixed_fs_type_verified = 1;
                        }

                        $vol_obj->create_fs(-type => $fs_type_mixed[$fs_type_index]);
                        if ($vol_obj->error) {
                            NATE::BaseException->throw("Failed to create filesystem : " . $vol_obj->errmsg());
                        }

                        # increment or reset counter to determine which fs type to use on this loop.
                        if ($fs_type_index == (scalar(@fs_type_mixed) - 1)) {
                            $fs_type_index = 0;
                        } else {
                            $fs_type_index++;
                        }
                    }

                    # Mount the newly created fs
                    if ($vol_obj->is_mounted() == 0) {
                        $vol_obj->mount(-pass_thru => $mount_options_pass_thru,
                                        -discard => $mount_options_discard);
                        if ($vol_obj->error) {
                            NATE::BaseException->throw("Failed to mount filesystem : " . $vol_obj->errmsg());
                        }
                    } else {
                        log_global()->log("file-system (" . $vol_obj->{mount} . ") on $volume_name " .
                                              "is already mounted");
                    }
                    $device_info->{path} = $vol_obj->{mount};
                    $device_info->{io_type} = "fs";
                } else {
                    $device_info->{path} = $vol_obj->get_raw_device();
                    $device_info->{io_type} = "raw";
                }

                push(@{$device_list}, $device_info);
            }
        }
    } else {
        # non LVM case:

        # validate the config
        $hostobj->validate(-cfo_mode => $cf_mode);
        if ($hostobj->error) {
            NATE::BaseException->throw("Validation of host ($client_name) failed : " . $hostobj->errmsg());
        }

        log_global()->log("Getting list of free devices");
        my @devices = ();

        # If the host is a virtual client, only list a single item in the protocols array to gather all
        # devices found for provisioning.
        if ($hostobj->{is_virtual} == 1) {
            log_global()->log("Client ($client_name) is a virtual client, gathering list of all available devices");
            @protocols = ("all");
        }

        # Go through each protocol defined and get the list of devices to use for provisioning
        foreach my $protocol (@protocols) {
            my @device_names;

            # If the protocol is all, use get_free_devices without filters
            if ($protocol eq "all") {
                my @devices_tmp = $devmgrobj->get_devices();
                foreach my $device (@devices_tmp) {
                    push(@device_names, $device->{name});
                }
                $protocol = "";
            } else {
                # Get the init states "online", "online invalid thinrclm", "online invalid", "error", "online thinrclm", or "online thin".
                # "online thinrclm" and "online thin" are the same thing.  Some operating systems use thinrcl
                # others only use thin. "online invalid" and "online invalid thinrclm" are the same depending on vsersion of Veritas.
                @device_names = $devmgrobj->get_free_devices(-inited_states => ["online invalid", "online invalid thinrclm",
                                                                                "error", "online",
                                                                                "online thin", "online thinrclm"],
                                                             -protocol => $protocol,
                                                             -filers => \@vserver_list);
            }
            if ($devmgrobj->error) {
                NATE::BaseException->throw("Failed to get list of free $protocol devices from DeviceMgr : " . $devmgrobj->errmsg());
            }

            if (scalar(@device_names) == 0) {
                NATE::BaseException->throw("Failed to get list of free $protocol devices from DeviceMgr " .
                                               "(device list was empty)");
            }

            # perform lun path regex filters on the discovered devices
            if (defined($lun_path_regex)) {
                my @filtered_out_luns;
                my @filtered_in_luns;
                log_global()->log("Filtering list of devices with lun path regex filter '$lun_path_regex'");
                @device_names = grep {
                    my $dev_name = $_;
                    my $dev_obj = $devmgrobj->get_device($dev_name);
                    my ($vserver, $lun_path) = ($dev_obj->{'lun'} =~ /^\s*(\S+)\s*\:\s*(\S+)\s*$/);
                    $vserver = "" if !$vserver;
                    $lun_path = "" if !$lun_path;
                    if ($lun_path !~ $lun_path_regex) {
                        # filter out luns that dont match the filter
                        push(@filtered_out_luns, "$dev_name\t: $vserver:$lun_path");
                        0;
                    } else {
                        # this lun matches
                        push(@filtered_in_luns, "$dev_name\t: $vserver:$lun_path");
                        1;
                    }
                } @device_names;

                log_global()->log("Device list (protocol:$protocol) :\n" .
                                      "  The following LUNs will be used:\n\t"     . join("\n\t", @filtered_in_luns) . "\n\n" .
                                          "  The following LUNs will not be used:\n\t" . join("\n\t", @filtered_out_luns));
            }

            # Filter the device list if the user specifies a lun count that they want to use
            if ($lun_count ne "") {
                log_global()->log("Filtering list of devices by the specified lun count: $lun_count");
                @device_names = $hostobj->client_lun_filter($lun_count, @device_names);
                if ($hostobj->error) {
                    NATE::BaseException->throw("Failed to apply LUN count ($lun_count) for $protocol client ($client_name) : " .
                                                   $hostobj->errmsg());
                }
                log_global()->log("Device list (protocol:$protocol) has been filtered down to:\n" . Dumper(@device_names));
            }

            # If io_type is filesystem 'fs', format the list of protocol devices found
            if ($io_type =~ /fs/) {
                # format the devices within the @device_names array
                if ($hostobj->isa("San::SanClient::Host::WinHost")) {
                    $hostobj->format(-devices => \@device_names, -type => $fs_type, -fs_rotate => $fs_rotate);
                } else {
                    $hostobj->format(-devices => \@device_names);
                }
                if ($hostobj->error()) {
                    NATE::BaseException->throw("Format failed on $protocol devices : " . $hostobj->errmsg());
                }
            }

            # Go through each device and set the protocol attribute on the device object
            foreach my $device_name (@device_names) {
                my $device = $devmgrobj->get_device($device_name);
                if ($devmgrobj->error) {
                    NATE::BaseException->throw("Failed to get device object for device ($device_name) : " .
                                                   $devmgrobj->errmsg());
                }

                $device->{protocol} = $protocol;
            }

            push(@devices, @device_names);
        }

        my $fs_type_index = 0;
        my $mixed_fs_type_verified = 0;
        my @fs_type_mixed;

        foreach my $device_name (@devices) {
            my $device = $devmgrobj->get_device($device_name);
            my $device_info;

            if ($devmgrobj->error) {
                NATE::BaseException->throw("Failed to get device object for device ($device_name) : " .
                                               $devmgrobj->errmsg());
            }

            # Save some attributes of the device onto the device_info hash
            my ($vserver, $volume, $lun) = ($device->{'lun'} =~ /^(\S+)\:\/vol\/(\S+)\/(\S+)$/);
            push(@{$device_info->{'vserver'}}, $vserver);
            push(@{$device_info->{'volume'}}, $volume);
            push(@{$device_info->{'lun'}}, $lun);
            $device_info->{'protocol'} = $device->{protocol};

            # Push the device serial number onto the device info hash if it is defined
            if (defined($device->{serial}) and $device->{serial} ne "") {
                push(@{$device_info->{'serial'}}, $device->{serial});
            }

            # Running NON CLIENT_VMS
            if ($io_type =~ /fs/) {
                # For windows, format already does the format() call,
                # so there is no need to do again.
                if ($hostobj->hosttype() !~ /windows/i) {
                    if ($fs_type !~ /:|mixed/i) {
                        # Same file system will be created on all the volumes.
                        # Create a fs based on the vm_lv_type requested.
                        $device->create_fs(-type => $fs_type);
                        if ($device->error()) {
                            NATE::BaseException->throw("Create FS failed on device ($device->{name}) :" .
                                                           $device->errmsg());
                        }
                    } else {
                        # Mixed file systems has been requested by user.
                        # There are 2 options,
                        # 1. When user directly provides colon separated fs list in fs_type.
                        # 2. When fs_type is set to "mixed", create mixed fs from the template provided in host libraries.
                        #    It various from one OS to another. (Currently this option is supported only in Linux).
                        if ($mixed_fs_type_verified == 0) {
                                # Initial setup required for mixed fs. Needs to be done only once in the first iteration.
                            if ($fs_type =~ /:/) {
                                # fs_type is set to colon separated fs by user
                                log_global()->log("Mixed FS requested by user: $fs_type");
                                @fs_type_mixed = split(/:/, $fs_type);
                            } elsif ($fs_type =~ /mixed/i) {
                                # fs_type is set to "mixed".
                                # Get mixed fs type template from host library.
                                if ($hostobj->can("get_mixed_fs_template")) {
                                    # Currently it is supported only on Linux
                                    my @fs_array = $hostobj->get_mixed_fs_template();
                                    my $fs_type_temp = join(":", @fs_array);
                                    log_global()->log("Mixed FS template: $fs_type_temp");
                                    @fs_type_mixed = split(/:/, $fs_type_temp);
                                } else {
                                    my $hostname = $hostobj->{host};
                                    NATE::BaseException->throw("Mixed file systems are not supported yet for $hostname");
                                }
                            }
                            $mixed_fs_type_verified = 1;
                        }

                        $device->create_fs(-type => $fs_type_mixed[$fs_type_index]);
                        if ($device->error()) {
                            NATE::BaseException->throw("Create FS failed on device ($device->{name}) : " .
                                                           $device->errmsg());
                        }

                        # increment or reset counter to determine which fs type to use on this loop.
                        if ($fs_type_index == (scalar(@fs_type_mixed) - 1)) {
                            $fs_type_index = 0;
                        } else {
                            $fs_type_index++;
                        }
                    }

                }

                $device->mount_fs(-discard => $mount_options_discard);
                if ($device->error()) {
                    NATE::BaseException->throw("Mount FS failed on device ($device->{name}) : " . $device->errmsg());
                }

                $device->load();
                if ($device->error()) {
                    NATE::BaseException->throw("Load of information failed for device ($device->{name}) : " .
                                                   $device->errmsg());
                }

                $device_info->{path} = $device->get_mount();
                $device_info->{io_type} = "fs";
            } else {
                # For each device in the device list, add the raw
                # name to the fs_return variable.
                $device_info->{path} = $device->get_raw_device();
                $device_info->{io_type} = "raw";
            }

            push(@{$device_list}, $device_info);
        }
    }

    return $device_list;
}

sub setup_nfs {
    my ($self, %args) = @_;
    my $client = delete($args{client});
    my $hostobj = delete($args{host_obj});

    my $client_name = $client->{NAME};
    my $device_list = [];
    my $nfs_target_hash = $self->{parent}->NACL::AppBuilder::EnGenX::Util::get_nfs_target_relationship_hash(client_name => $client_name);

    # create the Command interface for this client
    my $client_ci;
    $client_ci = new NACL::C::Client(name => $client_name,
                                     apiset_must => {conntype => 'ssh-persist',
                                                     connid => 'ssh'});

    # build out the nfs options datastructure up front
    # this will contain a sweet lookup table that we can search through when trying to
    # find matches for nfs_options
    my $all_nfs_options = $self->generate_nfs_options(resource => $client);

    my $match = 0;
    # we should now have a list of volumes to mount and a list of lifs to mount each volume
    foreach my $cluster_name (keys %{$nfs_target_hash}) {
        # skip if this cluster does not match the filter
        next if (!$self->find_nfs_options_match(all_nfs_options => $all_nfs_options,
                                                cluster_name => $cluster_name));
        foreach my $vserver_name (keys %{$nfs_target_hash->{$cluster_name}->{vservers}}) {
            # skip if this vserver does not match the filter
            next if (!$self->find_nfs_options_match(all_nfs_options => $all_nfs_options,
                                                    cluster_name => $cluster_name,
                                                    vserver_name => $vserver_name));

            foreach my $lif_obj (@{$nfs_target_hash->{$cluster_name}->{vservers}->{$vserver_name}->{lif_objs}}) {
                my $lif_address = $lif_obj->address();
                my $lif_name = $lif_obj->lif();
                my $lif_curr_node = $lif_obj->curr_node();

                # skip if the lif address / lif name don't match
                next if (!$self->find_nfs_options_match(all_nfs_options => $all_nfs_options,
                                                        cluster_name => $cluster_name,
                                                        vserver_name => $vserver_name,
                                                        lif_address => $lif_address,
                                                        lif_name => $lif_name));

                foreach my $volume_name (keys %{$nfs_target_hash->{$cluster_name}->{vservers}->{$vserver_name}->{volumes}}) {
                    my $vol_obj = $nfs_target_hash->{$cluster_name}->{vservers}->{$vserver_name}->{volumes}->{$volume_name}->{vol_obj};
                    my $junction_path = $vol_obj->junction_path();
                    my $aggr_obj = $nfs_target_hash->{$cluster_name}->{aggr_objs}->{$vol_obj->aggregate()};
                    my $vol_curr_node = $aggr_obj->nodes()->[0];

                    # discover the affinity between the volume and the lif (local|remote) based
                    # on the node
                    my $actual_affinity;
                    if ($lif_curr_node eq $vol_curr_node) {
                        $actual_affinity = "local";
                    } else {
                        $actual_affinity = "remote";
                    }

                    # now that we have all our fields, lets do one final lookup
                    # to find the nfs options that we can use for this volume
                    my $nfs_options = $self->find_nfs_options_match(all_nfs_options => $all_nfs_options,
                                                                    cluster_name => $cluster_name,
                                                                    vserver_name => $vserver_name,
                                                                    lif_address => $lif_address,
                                                                    lif_name => $lif_name,
                                                                    volume_name => $volume_name,
                                                                    junction_path => $junction_path,
                                                                    volume_lif_affinity => $actual_affinity);

                    # skip this junction path if we could not find an nfs options match
                    next if (!$nfs_options);

                    # translate the nfs_options into parameters that NACL::C::Client::Mount->create() understands
                    my $mount_options = $self->_translate_nfs_options_for_mount(nfs_options => $nfs_options);

                    # construct the mount point path so that it is unique to the cluster/vserver/lif/junction_path
                    # make sure there are no dashes in the mountpoint
                    # note: $junction_path already starts with a /
                    my $mount_point = "/mnt/engenx/${cluster_name}/${vserver_name}/${lif_address}${junction_path}";

                    # mount this volume on this lif
                    my $mount = NACL::C::Client::Mount->create(command_interface => $client_ci,
                                                               server => $lif_address,
                                                               path => $junction_path,
                                                               type => "nfs",
                                                               mount_point => $mount_point,
                                                               apiset_must => {connid => 'ssh',
                                                                               conntype => 'ssh-persist'},
                                                               %{$mount_options});
                    $mount_point = $mount->mount_point();

                    # create a unique sub directory in this mount point

                    my $uuid = md5_hex($mount_point);
                    $uuid = lc($uuid);
                    # chop off the first 24 chars
                    $uuid =~ s/^\S{24}//g;

                    if ($nfs_options->{create_unique_subdir}) {
                        $mount_point = $mount->mount_point() . "/${uuid}";
                        my (@output) = $hostobj->execute("mkdir $mount_point");
                        if ($hostobj->error() and not grep(/File exists/i, @output)) {
                            NATE::BaseException->throw($hostobj->errmsg());
                        }
                        $hostobj->_clear_err();
                    }

                    # create the client dev list entry for this mount point
                    my $device_info;
                    $device_info->{path} = $mount_point;
                    $device_info->{io_type} = "fs";
                    $device_info->{protocol} = "nfs";
                    $device_info->{vserver_name} = $vserver_name;
                    $device_info->{cluster_name} = $cluster_name;
                    $device_info->{volume_name} = $volume_name;
                    $device_info->{qtree_name} = undef;
                    $device_info->{is_qtree} = 0;
                    $device_info->{junction_path} = $junction_path;
                    $device_info->{unique_subdir} = $nfs_options->{create_unique_subdir};
                    $device_info->{lif_name} = $lif_name;
                    $device_info->{lif_address} = $lif_address;
                    $device_info->{lif_volume_affinity} = $actual_affinity;
                    push(@{$device_list}, $device_info);

                    # skip qtrees if this volume explicitly specified to not mount qtrees
                    next if (!$nfs_options->{mount_qtrees});

                    # iterate through all qtrees in this volume and attempt to mount each one as well
                    foreach my $qtree_obj (@{$nfs_target_hash->{$cluster_name}->{vservers}->{$vserver_name}->{volumes}->{$volume_name}->{qtree_objs}}) {
                        my $qtree_name = $qtree_obj->qtree();
                        $nfs_options = undef;
                        $nfs_options = $self->find_nfs_options_match(all_nfs_options => $all_nfs_options,
                                                                     cluster_name => $cluster_name,
                                                                     vserver_name => $vserver_name,
                                                                     lif_address => $lif_address,
                                                                     lif_name => $lif_name,
                                                                     volume_name => $volume_name,
                                                                     qtree_name => $qtree_name,
                                                                     junction_path => $junction_path,
                                                                     volume_lif_affinity => $actual_affinity);

                        # skip this qtree if we could not find an nfs options match
                        next if (!$nfs_options);

                        # construct the mount point path so that it is unique to the cluster/vserver/lif/junction_path
                        # make sure there are no dashes in the mountpoint
                        # note: $junction_path already starts with a /
                        $mount_point = "/mnt/engenx/${cluster_name}/${vserver_name}/${lif_address}${junction_path}__qtree__${qtree_name}";

                        # mount this qtree
                        $mount = NACL::C::Client::Mount->create(command_interface => $client_ci,
                                                                server => $lif_address,
                                                                path => $junction_path . '/' . $qtree_name,
                                                                type => "nfs",
                                                                mount_point => $mount_point,
                                                                apiset_must => {connid => 'ssh',
                                                                                conntype => 'ssh-persist'},
                                                                %{$mount_options});
                        $mount_point = $mount->mount_point();

                        $uuid = md5_hex($mount_point);
                        $uuid = lc($uuid);
                        # chop off the first 24 chars
                        $uuid =~ s/^\S{24}//g;

                        # check if we need to create a unique mount point for this qtree
                        if ($nfs_options->{create_unique_subdir}) {
                            $mount_point = $mount->mount_point() . "/${uuid}";
                            my (@output) = $hostobj->execute("mkdir $mount_point");
                            if ($hostobj->error() and not grep(/File exists/i, @output)) {
                                NATE::BaseException->throw($hostobj->error());
                            }
                            $hostobj->_clear_err();
                        }

                        # create the client dev list entry for this mount point
                        my $device_info;
                        $device_info->{path} = $mount_point;
                        $device_info->{io_type} = "fs";
                        $device_info->{protocol} = "nfs";
                        $device_info->{vserver_name} = $vserver_name;
                        $device_info->{cluster_name} = $cluster_name;
                        $device_info->{volume_name} = $volume_name;
                        $device_info->{qtree_name} = $qtree_name;
                        $device_info->{is_qtree} = 1;
                        $device_info->{junction_path} = $junction_path;
                        $device_info->{unique_subdir} = $nfs_options->{create_unique_subdir};
                        $device_info->{lif_name} = $lif_name;
                        $device_info->{lif_address} = $lif_address;
                        $device_info->{lif_volume_affinity} = $actual_affinity;
                        push(@{$device_list}, $device_info);
                    }
                }
            }
        }
    }

    return $device_list;
}

sub _setup_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Client/Setup/Failure",
                            message  => \%data);
}

sub _setup_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Client/Setup/Complete",
                            message  => \%data);
}

1;
