#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on resource setup
## @author dalmond@netapp.com
## @status private
## @pod here

package NACL::AppBuilder::EnGenX::ClientCluster::setup;

# Compiler directives.
use strict;
use San::SanLog;
use ParamsMgr;
use San::SanUtil;
use NATE::Log qw(log_global);
use NACL::AppBuilder::EnGenX::Util;
use vars qw(@ISA);
@ISA = qw(NACL::AppBuilder::EnGenX::Util);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

###############################################################################
# Method:    setup_single
# Objective: Setup run against a single resource
# Details:   See POD documentation at the end of this file.
###############################################################################
sub setup_single {
    my ( $self, %args ) = @_;

    &ParamsMgr::process_args(%args);
    my $resource = &ParamsMgr::get_arg(
        -arg      => "resource",
        -required => 1,
        -default  => ""
    );
    if ( &ParamsMgr::error() ) {
        $self->_set_err( &ParamsMgr::get_error() );
        &SubIPC::fail( -object => $self );
        return $SanMgr::FALSE;
    }

    # Load up the resource information and get them via get_arg
    my $caller_id = $resource->{NAME} . "|" . time();

    #&ParamsMgr::process_args(%{$resource});
    &ParamsMgr::process_args( %{$resource}, -CALLER => $caller_id );

    #############################################
    ### General client parameters
    #############################################
    my $client_cluster_name = &ParamsMgr::get_arg(
        -arg            => "NAME",
        -required       => 1,
        -default_to_env => 1,
        -CALLER         => $caller_id
    );
    my $cluster_type = &ParamsMgr::get_arg(
        -arg            => "CLUSTER_TYPE",
        -required       => 1,
        -default_to_env => 1,
        -CALLER         => $caller_id
    );
    my $nodes = &ParamsMgr::get_arg(
        -arg            => "SECONDARY_NODES",
        -required       => 1,
        -default_to_env => 1,
        -CALLER         => $caller_id,
    );
    my $lun_path_regex = &ParamsMgr::get_arg(
        -arg            => "LUN_PATH_REGEX",
        -required       => 0,
        -default_to_env => 1,
        -default        => undef,
        -CALLER         => $caller_id
    );
    my $windows_domain_password = &ParamsMgr::get_arg(
        -arg            => "WINDOWS_DOMAIN_PASSWORD",
        -required       => 0,
        -default_to_env => 1,
        -default        => undef,
        -CALLER         => $caller_id
    );
    $lun_path_regex = eval($lun_path_regex);

    if ( &ParamsMgr::error() ) {
        $self->_set_err( &ParamsMgr::get_error() );
        &SubIPC::fail( -object => $self );
        return $SanMgr::FALSE;
    }

    my $device_list;
    my $device_data;
    my @nodes = split(',', $nodes);
    # Get client object
    my ($client_cluster_obj)
        = $self->get_objects( -type => "client_cluster" , -names => [$client_cluster_name] );
    if ( $self->error() ) {
        $self->_set_err( "Failed to get client_cluster ($client_cluster_name) object: ",
            $self->errmsg() );
        &SubIPC::fail( -object => $self );
        return $SanMgr::FALSE;
    }
  if ($cluster_type  =~ /windows/) {
            # Creating the physical cluster.
            #$Log->comment(" Creating cluster $client_cluster_name");
            if (length($client_cluster_name) > 15) {
                $self->_set_err( "For windows, the client cluster name must be 15 characters or less. It will be truncated otherwise.");
                &SubIPC::fail( -object => $self );
                return $SanMgr::FALSE;
            }
            
            $Log->comment("Cluster create starting");
            $client_cluster_obj->cluster_create(windows_domain_password => $windows_domain_password);
            if ($client_cluster_obj->error()) {
                &SubIPC::fail( -object => $self );
                return $SanMgr::FALSE;
            }

            # Add first device to the node. This device will be used as quorum device
            # On Windows, this routine will add all available devices. The quorum will be chosen later.
            $Log->comment("Adding first device");
            $client_cluster_obj->add_device();
            if ($client_cluster_obj->error()) {
                &SubIPC::fail( -object => $self );
                return $SanMgr::FALSE;
            }

            $Log->comment("Creating quorum");
            $client_cluster_obj->create_quorum();
            if ($client_cluster_obj->error()) {
                &SubIPC::fail( -object => $self );
                return $SanMgr::FALSE;
            }

            # add rest of the nodes to the cluster
            foreach my $node (@nodes) {
                $Log->comment("Client Cluster Setup: adding node $node to the cluster");
                $client_cluster_obj->add_node(-host          =>  $node,
                                              -conn_type     =>  "THARNHOST");
                if ($client_cluster_obj->error()) {
                    &SubIPC::fail( -object => $self);
                    return $SanMgr::FALSE;
                }
            }

            $Log->comment("Assigning disks to cluster members");
            $client_cluster_obj->balance_disks_among_groups();
            if ($client_cluster_obj->error()) {
                &SubIPC::fail( -object => $self);
                return $SanMgr::FALSE;
            }
    }

    &SubIPC::pass( -msg => "Client cluster setup complete" );
    return $SanMgr::TRUE;
} ## end sub setup_single

1;
