# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::CIFS::LocalGroup::Membership

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::CIFS::LocalGroup::Membership is used to Config, Setup, Cleanup and
Deconfig CIFS LocalGroup Membership on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::CIFS::LocalGroup::Membership;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL libs
use NACL::STask::VserverCifsUsersAndGroupsLocalGroup;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args
            $Log);

$Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    cifs_service_username =>'',
    cifs_service_password =>'',
    cifs_service_domain   =>'',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
);

=head1 METHODS

=head2 create

(Static method) Request to create a cifs privilege with the following options

  NACL::AppBuilder::EnGenX::Controller::LocalGroup::Membership->create(
            command_interface               => $node,
            vserver                         => $vserver_name,
            'group-name'                    => $name,
            'member-names'                  => $member_names,
            );
=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The name of the vserver.

=item C<< 'group-name' => $name>>

(Required)
The name of the local or Active Directory group.

=item C<< 'member-names' => $member_names>>

(Required)
The Arrayref of member names.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Membership based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "local_group_membership";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

=head2 delete

(Static method) Request to delete a local-group membership with the following options

  NACL::AppBuilder::EnGenX::Controller::LocalGroup::Membership->delete(
            command_interface               => $node,
            vserver                         => $vserver_name,
            'group-name'                    => $name,
            );

=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The name of the vserver.

=item C<< 'group-name' => $name >>

(Required)
The group-name of the local-group.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the delete of the LocalGroup Membership based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "local_group_membership";

    # Super::_create is in Task.pm
    #no-op
    $Log->exit() if $may_exit;

    return $parent_answer;
}

#====================================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the cifs Privilege.
# Details:    Makes the NACL::STask::VserverCIFSUsersAndGroupsLocalGroup->add_members().
#
# Inputs:     NACL::Stask::VserverCIFSUsersAndGroupsLocalGroup->add_members arguments.
# Outputs:    Returns whether the create was successful.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#====================================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Arguments for local-group add-members
    my $member_names = exists($args{'member-names'}) ? delete($args{'member-names'}) : undef;
    my @member_names;
    if ($member_names) {
        @member_names = split(",", $member_names);
        # member-names must be an ARRAYREF
        $args{'member-names'} = \@member_names;
    }

    NACL::STask::VserverCifsUsersAndGroupsLocalGroup->add_members(%args,
                                                                  nacltask_if_exists => $if_exists,
                                                                  _was_created => \$was_created);

    if($was_created) {
        $Log->comment("Added new members to CIFS Local-Group: ". $args{'group-name'});
    } else {
        $Log->comment("Members were already added to CIFS Local-Group: ". $args{'group-name'});
    }

    return (wantarray()) ? ('success' => TRUE, 'duplicated' => !$was_created) : TRUE;
}

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deleting the cifs local-user.
# Details:    Makes the NACL::STask::VserverCifsUsersAndGroupsLocalGroup->remove_members().
#
# Inputs:     NACL::STask::VserverCifsUsersAndGroupsLocalGroup->remove_members arguments.
# Outputs:    Returns whether the delete was successful.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->comment("Members get deleted on deleting the group");

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  NOOP for CIFS
# Details:
# Inputs:     None
# Outputs:    None
# Exceptions: None
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
