# Copyright (c) 2012-2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group is used to Config, Setup, Cleanup and
Deconfig CIFS Local Groups on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL libs
use NACL::STask::VserverCifsUsersAndGroupsLocalGroup;
use NACL::STask::VserverCifsUsersAndGroupsPrivilege;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args
            $Log);

$Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    cifs_service_username =>'',
    cifs_service_password =>'',
    cifs_service_domain   =>'',
    privileges            =>'',
    'member-names'          =>'',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    vserver      => '',
    'group-name' => ''
);

=head1 METHODS

=head2 create

(Static method) Request to create a cifs local-group with the following options

  NACL::AppBuilder::EnGenX::Controller::Local-Group->create(
            command_interface               => $node,
            vserver                         => $vserver_name,
            'group-name'                    => $group_name,
            description                     => $description,
            privileges                      => $privileges,
            'member-names'                  => $member_names
            );
=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The vserver of the cifs service for the local-group.

=item C<< 'group-name' => $group_name >>

(Required)
The group name of the local-group.

=item C<< description => $description >>

(Required)
The description of the local-group.

=item C<< privileges => $privileges >>

(Required)
The privileges of the local-group.

=item C<< 'member-names' => $member_names >>

(Required)
The names of the local-users of the local-group.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Local_Group based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "cifs_local_group";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

=head2 delete

(Static method) Request to delete a cifs local-group with the following options

  NACL::AppBuilder::EnGenX::Controller::Local-Group->delete(
            command_interface               => $node,
            vserver                         => $vserver_name,
            'group-name'                    => $group_name
            );

=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The vserver of the cifs service for the local-group.

=item C<< 'group-name' => $group_name >>

(Required)
The group-name of the local-group.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the delete of the Local_Group based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "cifs_local_group";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the cifs local-group.
# Details:    Makes the NACL::STask::VserverCIFSUsersAndGroupsLocalGroup->create().
#
# Inputs:     NACL::STask::VserverCIFSUsersAndGroupsLocalGroup->create arguments.
# Outputs:    Returns whether the create was successful.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    $args{description} = (exists($args{description}) and $args{description} =~ /^\s+?\".*?\"\s+$/)
                         ? delete($args{description}) : "\"" . delete($args{description}) . "\"";
    
    # create the cifs local group
    NACL::STask::VserverCifsUsersAndGroupsLocalGroup->create(%args,
                                                             nacltask_if_exists => $if_exists,
                                                             _was_created => \$was_created);
    if($was_created) {
        $Log->comment("Created new CIFS Local-Group: ". $args{'group-name'});
    } else {
        NACL::STask::VserverCifsUsersAndGroupsLocalGroup->modify(%args);
        $Log->comment("Modified existing CIFS Local-Group: ". $args{'group-name'});
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? ('success' => TRUE, 'duplicated' => !$was_created) : TRUE;
}

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deleting the cifs local-group.
# Details:    Makes the NACL::STask::VserverCifsUsersAndGroupsLocalGroup->delete().
#
# Inputs:     NACL::STask::VserverCifsUsersAndGroupsLocalGroup->delete arguments.
# Outputs:    Returns whether the delete was successful.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    #Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # delete the cifs service
    NACL::STask::VserverCifsUsersAndGroupsLocalGroup->purge(%args,
                                                            nacltask_if_purged => 'pass',
                                                            _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("Deleted CIFS Local-Group: ". $args{'group-name'}." on vserver". $args{vserver});
    } else {
        $Log->comment("CIFS Local-Group: ". $args{'group-name'}." on vserver". $args{vserver}." was already deleted");
    }

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  NOOP for CIFS
# Details:
# Inputs:     None
# Outputs:    None
# Exceptions: None
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
