# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary CIFS Share Access Control Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::ShareAccessControl is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task. It is used to Create and
Delete ONTAP Vserver CIFS Share Access Control Settings.
All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::VserverCifsShareAccessControl;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args
            $Log);

$Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
    cifs_share => 'share',
    No_Access => 'No_access',
    read => 'Read',
    change => 'Change',
    full_control => 'Full_control',
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
    share             => '',
    'user-or-group'   => '',
);

=head1 PUBLIC METHODS

=head2 create

(Static method) Request creation of one or more CIFS Share Access Policy settings
serially or in parallel and return whether the CIFS Share Access Policy settings.
were created successfully. This method passes all user arguments through to
NACL::AppBuilder::EnGenX::Controller::Task and adds specific arguments used by the EnGenX
framework.

  NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl->create(
            command_interface => $Node,
            vserver           => $Vserver_name,
            'user-or-group'   => $user_or_group_name,
            share             => $share,
            'user-group-type' => $type,
            ...);

=over

=item Options

Either the resource_list is required or the NACL CIFS Share
Access Control Creation Arguments are required but both
should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the Cifs Share Access Control in the resource list
will be created based on the information that is supplied in the resource
list. If the resource list is passed in the NACL Cifs Share Access Control
Creation Arguments should NOT be passed in.

=item C<< NACL CIFS Share Access Control Creation Arguments >>

=item C<< vserver => $Vserver_name >>
(Required)
The vserver the CIFS Share Access Control applies to

=item C<< user-or-group => $user_or_group_name >>
(Required)
The user or group name for the Cifs Share Access Control.

=item C<< share => $Share >>
(Required)
The share name the CIFS Share Access control applies to.

=item C<< user-group-type => $type >>
(Required)
The user group type for the Cifs Share Access Control.

(Optional)
These are arguments that can be passed into NACL::C::VserverCifsShareAccessControl->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Cifs Share Access Control based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "share_access_control";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple CIFS Share Access
Policy settings serially or in parallel and return whether the
CIFS Share Access Policy settings were deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl->delete(
            command_interface => $Node,
            vserver           => $Vserver_name,
            share             => $share,
            'user-or-group'     => $user_or_group_name,
            ...);

=over

=item Options

Either the resource_list is required or the NACL Vserver
Cifs Share Access Control Deletion Arguments are required but both
should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the Cifs Share Access Control in the resource list
will be deleted based on the information that is supplied in the resource
list. If the resource list is passed in the NACL Cifs Share Access Control
Deletion Arguments should NOT be passed in.

=item C<< NACL CIFS Share Access Control Deletion Arguments >>

=item C<< vserver => $Vserver_name >>
(Required)
The vserver the CIFS Share Access Control applies to

=item C<< user-or-group => $user_or_group_name >>
(Required)
The user or group name for the Cifs Share Access Control.

=item C<< share => $Share >>
(Required)
The share name the CIFS Share Access control applies to.

=item C<< user-group-type => $type >>
(Required)
The user group type for the Cifs Share Access Control.

(Optional)
These are arguments that can be passed into NACL::C::VserverCifsShareAccessControl->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Cifs Share Access Control based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "share_access_control";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
} ## end sub delete

=head1 PRIVATE METHODS

=head2 _nacl_create_call

=cut

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of Cifs Share Access Control.
# Details:    Makes the NACL::STask::VserverCifsShareAccessControl->create() call. Also works
#             around some common non-fatal errors that can be return from
#             Cifs Share Access Control creation and then has a retry mechanism to
#             retry vserver creation.
# Inputs:     NACL::STask::VserverCifsShareAccessControl->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the Cifs Share Access Control was a duplicate if a hash is
#             requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }
    # Create the policy rule
    NACL::STask::VserverCifsShareAccessControl->create(%args,
                                            nacltask_if_exists => $if_exists,
                                            _was_created => \$was_created);

    if($was_created) {
        $Log->comment("Created new CIFS Share Access Control for user or group " . $args{'user-or-group'} .
                      " on share " . $args{'share'}. " on vserver " . $args{'vserver'});
    } else {
        $Log->comment("Using existing CIFS Share Access Control for user or group " . $args{'user-or-group'} .
                      " on share " . $args{'share'}. " on vserver " . $args{'vserver'});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_created) : TRUE;

} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of Cifs Share Access Control.
# Details:    Makes the NACL::STask::VserverCifsShareAccessControl->delete() calls. Also
#             works around some common non-fatal errors that can be return from
#             vserver deletion and then has a retry mechanism to retry vserver
#             deletion.
# Inputs:     NACL::STask::VserverCifsShareAccessControl->delete arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }
    NACL::STask::VserverCifsShareAccessControl->purge(%args,
                                        nacltask_if_purged => 'pass',
                                        _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("CIFS Share Access Control was deleted for user or group " . $args{'user-or-group'} .
                      " on share " . $args{'share'}. " on vserver " . $args{'vserver'} );
    } else {
        $Log->comment("CIFS Share Access Control was not deleted for user or group " . $args{'user-or-group'} .
                      " on share " . $args{'share'}. " on vserver " . $args{'vserver'} );
    }

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::Vserver->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::Vserver->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1
