# $Id: //depot/prod/test/main/lib/EnGenX/Controller/Cluster.pm

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Cluster Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::Cluster

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Cluster is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create, Delete
and Modify ONTAP Clusters. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Cluster;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::STask::NetworkOptionsIpv6;
use NACL::STask::SystemNodeAutosupport;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name        => 'clustername'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    _reuse   => '',
    parallel => ''
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one cluster serially or in parallel and
return whether the cluster was created successfully. (Not yet implmented)

Also, request the modification of certain atttributes that are at the
cluster level, but actually have to do with some other part of ONTAP.
This modifications will be done if the corresponding attribute is found in the
resource arguments.

Will return whether the cluster(s) was created successfully.(Not yet implmented)
Also, will return whether the resource(s) was modified successfully.

This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Cluster->create(
            command_interface => $Node,
            clustername=> $cluster_name,
            'mgmt-port'=> $string,
            'mgmt-ip'=> $string
            'mgmt-netmask'=> $string
            'mgmt-gateway'=> $string,
            ipaddr1=> $string,
            netmask=> $string,
            mtu=> $string,
            ipaddr2=> $string,
            "method-timeout"=> $scalar
        );

  NACL::AppBuilder::EnGenX::Controller::Cluster->create(
            command_interface => $Node,
            clustername=> $cluster_name,
            ...
            autosupport       =>"enable"
            ipv6_enabled      => $ipv6_enabled
        );




=over

=item Options

Either the resource_list is required or the NACL Vserver Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed the clusters in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in, the NACL Cluster Creation Arguments should
NOT be passed in.

=item C<< NACL Cluster Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::Cluster->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=item C<< ipv6_enabled => $ipv6_enabled >>

(Optional)
ipv6_enabled controls whether or not ipv6 addresses are enabled for this
cluster and it will be used during NACL calls.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Cluster(s) - NOT YET IMPLEMENTED or succeeds or fails
when modifying the provided attributes.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "cluster";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
} ## end sub create


=head2 delete

(Static method) Request the unjoining of one node from a cluster this could
be done serially or in parallel and return whether the node successfully left
the cluster.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Cluster->unjoin(
            command_interface => $Node,
        );

=over

=item Options

TBD

=over

=item C<< TBD >>

TBD

=back

=item Return

TBD

=item Exceptions

BaseException

=cut
sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    return TRUE;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for Creation of a cluster. Also updates
#             the required attributes.
# Details:    .
# Inputs:     .
# Outputs:    Returns  TRUE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Now, modify the requested attributes:
    # 1 - Autosupport
    if( defined($args{autosupport}) ){
        # The node is optional, so if the user didn't provide one, we can turn
        # it on all the nodes, by default.
        my $node_value = (defined($args{node})) ? $args{node} : '*';
        NACL::STask::SystemNodeAutosupport->modify(
                            command_interface => $args{command_interface},
                                        node  => $node_value,
                                        state => $args{autosupport}
                        );
        $Log->comment("System Node AutoSupport states was modified to be:"
                      . $args{autosupport});
    }

    if( defined($args{ipv6_enabled}) ){
        NACL::STask::NetworkOptionsIpv6->modify(
                            command_interface   => $args{command_interface},
                                        enabled => $args{ipv6_enabled}
                        );
        $Log->comment("Network Options Ipv6 enabled was modified to be: "
                      . $args{ipv6_enabled});
    }

    if( defined($args{'is-ra-processing-enabled'}) ){
        NACL::STask::NetworkOptionsIpv6->modify(
                            command_interface   => $args{command_interface},
                            'is-ra-processing-enabled' => $args{'is-ra-processing-enabled'}
                        );
        $Log->comment("Network Options Ipv6 'is-ra-processing-enabled' was modified to be: "
                      . $args{'is-ra-processing-enabled'});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE) : TRUE;

} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for delete/unjoin a cluster.
# Details:    .
# Inputs:     .
# Outputs:    Returns TRUE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my %args = @_;

    # NOTE: Will add logic to this sub when the public method DELETE is
    #       implemented.

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  currently a noop
# Details:    noop
# Inputs:     none
# Outputs:    TRUE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
