# Copyright (c) 2012-2014 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::DNS

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::DNS is used to Config, Setup, Cleanup and
Deconfig DNS services on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::DNS;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::STask::VserverServicesDns;
use NACL::C::Job;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Exceptions::VserverServicesDns::AlreadyExists;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel      => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver => '',
);

=head1 METHODS

=head2 create

(Static method) Request to add the dns service to a list
of vservers the command interface and vserver name are the
only required perameters.

  NACL::AppBuilder::EnGenX::Controller::DNS->create(
            command_interface   => $node,
            vserver             => $vserver_name,
            domains             => "gdl.englab.netapp.com,rtp.netapp.com",
            'name-servers'      => ["10.60.248.10","10.60.132.40"]
            );

=over

=item Options

The NACL component only requires the user to specify the
command interface, vserver. The ontap defaults will
be used for any option that is not stated.

=over

=item C<< command_interface => $command_interface >>

(Required)
The NACL Command interface to use when running commands.

=item C<< vserver => $vserver >>

(Required)
The vserver to create the DNS service for.

=item C<< domains => domains >>

(Required)
An array referenace or comma delimited scalar of dns domains.

=Item C<< name-servers => name-servers >>

(Required)
An array referenace or comma delimited scalar of dns name-servers.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the DNS service based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {

    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "dns";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;
    return $parent_answer;
}

=head2 delete

(Static method) Request to delete the dns service of the
vserver.

  NACL::AppBuilder::EnGenX::Controller::DNS->delete(
            command_interface               => $node,
            vserver                         => $vserver_name);

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)
The NACL Command interface to use when running commands.

=item C<< vserver => $vserver >>

(Required)
The vserver to create the DNS service for.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the DNS service based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "dns";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the dns service.
# Details:    Makes the NACL::STask::VserverServicesDns->create().
# Inputs:     NACL::STask::VserverServicesDns->create arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the dns service was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # convert the domains parameter and name-servers parameter into an array reference
    # if it is a comma delimited string.
    if (ref($args{domains}) ne "ARRAY") {
        $args{domains} = [split(/\s*,\s*/, $args{domains})];
    }
    if (ref($args{'name-servers'}) ne "ARRAY") {
        $args{'name-servers'} = [split(/\s*,\s*/, $args{'name-servers'})];
    }

    my $attempts = 0;
    my $created = 0;
    my $action = "create";
    while ($attempts < 3 && !$created) {
        try { 
            if ($action eq "create") {
                # create the DNS service
                NACL::STask::VserverServicesDns->create(%args,
                                nacltask_if_exists => $if_exists,
                                _was_created => \$was_created); 
            }
            elsif ($action eq "modify") {
                NACL::STask::VserverServicesDns->modify(%args);
            }
            $created = 1;
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift();
            my $error_msg = $exception_object->stringify();
            if ($error_msg =~ "duplicate") { 
                $action = "modify";
                next;
            }
            else {
                $Log->comment("$action DNS failed exception caught");
            }
            sleep(10);
        };
        $attempts++;
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? ('success' => TRUE, 'duplicated' => !$was_created) : TRUE;
}

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for creation of the dns service.
# Details:    Makes the NACL::STask::VserverServicesDns->delete().
# Inputs:     NACL::STask::VserverServicesDns->delete arguments
# Outputs:    Returns whether the delete was successful or a hash with success
#             and whether the dns service was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;
    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::VserverServicesDns->purge(%args,
                                nacltask_if_purged => 'pass',
                                _was_deleted => \$was_deleted);

    if($was_deleted) {
        $Log->comment("DNS Service deleted on vserver " . $args{vserver});
    } else {
        $Log->comment("DNS Service was not deleted on vserver " . $args{vserver});
    }

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::Vserver->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::Vserver->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
