# $Id: //depot/prod/test/main/lib/EnGenX/Controller/FailoverGroup.pm $

#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::FailoverGroup

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::FailoverGroup is used to Config, Setup, Cleanup and
Deconfig FailoverGroup services on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::FailoverGroup;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::STask::NetworkInterfaceFailoverGroups;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name   => 'failover-group',
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel      => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    'failover-group'  => '',
    node              => '',
    port              => '',
    vserver           => ''
);

=head1 METHODS

=head2 create

(Static method) his method is used to create a new failover group
or add an entry to an existing group.

  NACL::AppBuilder::EnGenX::Controller::FailoverGroup->create(
            command_interface   => $node,
            vserver             => $vserver_name,
            domains             => "gdl.englab.netapp.com,rtp.netapp.com",
            'name-servers'      => ["10.60.248.10","10.60.132.40"]
            );

=over

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< "failover-group" => $failover_group >>

(Required)

Applicable for CMode CLI.

=item C<< "port" => $port >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the FailoverGroup service based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {

    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "FailoverGroup";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;
    
    return $parent_answer;
}

=head2 delete

(Static method) This method is used to remove a port from a
failover group.

  NACL::AppBuilder::EnGenX::Controller::FailoverGroup->delete(
            command_interface               => $node,
            vserver                         => $vserver_name);

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< "failover-group" => $failover_group >>

(Required)

Applicable for CMode CLI.

=item C<< "port" => $port >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the FailoverGroup service based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "FailoverGroup";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the FailoverGroup
# Details:    Makes the NACL::STask::NetworkInterfaceFailoverGroups->create().
# Inputs:     NACL::STask::NetworkInterfaceFailoverGroups->create arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the FailoverGroup was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    my $was_created = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # if the vserver attribute exists (then assume we are FullSteam and later)
    # FullSteam with ipspaces changed how failovergroups work.
    if (defined $args{'vserver'}) {
        # build the targets attribute based on the node port attributes
        $args{'targets'} = [delete($args{'node'}) . ':' . delete($args{'port'})];
    }

    # create the FailoverGroup
    NACL::STask::NetworkInterfaceFailoverGroups->create(%args,
                                                        nacltask_if_exists => $if_exists,
                                                        _was_created       => \$was_created,);
    if($was_created) {
        $Log->comment("FailoverGroup: " . $args{'failover-group'} .
                  " was newly created");
    }
    else {
        if(defined $args{vserver}) {
            # No already exists errors for adding targets.
            NACL::STask::NetworkInterfaceFailoverGroups->add_targets(%args,);
            $Log->comment("FailoverGroup: " . $args{'failover-group'} .
                  " already existed.");   
        }
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? ('success' => TRUE, 'duplicated' => !$was_created) : TRUE;
}

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for creation of the FailoverGroup service.
# Details:    Makes the NACL::STask::NetworkInterfaceFailoverGroups->delete().
# Inputs:     NACL::STask::NetworkInterfaceFailoverGroups->delete arguments
# Outputs:    Returns whether the delete was successful or a hash with success
#             and whether the FailoverGroup service was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;
    my $was_deleted = 0;
    # if vserver is an attribute we'll need to remove a few attributes
    # because we are on a FullSteam or later build
    # there is a subtle caveat here and that is that we may run this method
    # several times for the same failover group, due to the way we are treating
    # each failover group as potentially being more than one resource object
    if ($args{'vserver'}) {
        delete($args{'node'});
        delete($args{'port'});
    }
    

    NACL::STask::NetworkInterfaceFailoverGroups->purge(%args,
                                                       nacltask_if_purged => 'pass',
                                                       _was_deleted       => \$was_deleted);
    if($was_deleted) {
        $Log->comment("FailoverGroup: " . $args{'failover-group'} .
          " was deletd.");  
    } else {
        $Log->comment("FailoverGroup: " . $args{'failover-group'} .
          " did not exist.");     
    }
    $Log->exit() if $may_exit;
    return;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  noop
# Details:    noop
# Inputs:     none
# Outputs:    TRUE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
