# Copyright (c) 2012-2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask.pm

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask.pm is used to Config, Setup, Cleanup and
Deconfig Policy Tasks  on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL libs
use NACL::STask::VserverSecurityFileDirectoryPolicyTask;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args
            $Log);

$Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
    file_directory_policy => 'policy-name',
    'index-num' => 'index'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel      => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
   'policy-name' =>'',
   path          =>'',
   vserver       =>''
);

=head1 METHODS

=head2 create

(Static method) Request to create a policy task with the following options

  NACL::AppBuilder::EnGenX::Controller::PolicyTask.pm->create(
            command_interface               => $node,
            vserver                         => $vserver_name,
            policy-name                     => $policy_name,
            index_num                       => $index_num,
            path                            => $path,
            ntfs-mode                       => $ntfs_mode,
            ntfs_sd                         => $ntfs_sd,
            security-type                   => $security_type
            );

=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The vserver to create the policy task service for.

=item C<< policy-name => $policy_name >>

(Required)
This is the policy name to use for this  Policy Task.

=item C<< index-num => $index_num >>

(Required)
This is the target position for adding this  Policy Task.

=item C<< path => $path >>

(Required)
This is the path for this  Policy Task.

=item C<< ntfs_mode => $ntfs_mode >>

(Optional)
This is the propogation mode for ntfs to use for this  Policy Task.

=item C<< ntfs-sd => $ntfs_sd >>

(Optional)
This is the ntfs security descriptor name to use for this  Policy Task.

=item C<< security-type => $security_type >>

(Optional)
This is the policy name to use for this  Policy Task.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the PolicyTask.pm Targets based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "policy_task";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

=head2 delete

(Static method) Request to delete a policy task service with the following options

  NACL::AppBuilder::EnGenX::Controller::PolicyTask.pm->delete(
            command_interface               => $node,
            vserver                         => $vserver_name,
            policy-name                     => $policy_name,
            );

=over

=item Options

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver >>

(Required)
The vserver to create the policy task.

=item C<< policy-name => $policy_name >>

(Required)
This is the policy-name for the policy task.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the delete of the PolicyTask.pm Targets based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "policy_task";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    $Log->exit() if $may_exit;

    return $parent_answer;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the policy task service.
# Details:    Makes the NACL::STask::VserverSecurityFileDirectoryPolicyTask.pm->create().
#
# Inputs:     NACL::STask::VserverSecurityFileDirectoryPolicyTask.pm->create arguments
# Outputs:    Returns whether the create was successful
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $was_created = 0;
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    $args{'ntfs-sd'} = [split(/,/ , $args{'ntfs-sd'})] if (exists($args{'ntfs-sd'}) and (ref($args{'ntfs-sd'}) ne 'ARRAY'));
    # create the  Policy Task service
    NACL::STask::VserverSecurityFileDirectoryPolicyTask->add(%args,
                                                    nacltask_if_exists => $if_exists,
                                                    _was_created => \$was_created);
    if($was_created) {
        $Log->comment("Policy task added for File-Directory policy " . $args{'policy-name'} ." on vserver: ".$args{vserver});
    } else {
        $Log->comment("Policy task already exists for File-Directory policy " . $args{'policy-name'} . "on vserver: ".$args{vserver}." It will be reused.");
    };

    $Log->exit() if $may_exit;
    return (wantarray()) ? ('success' => TRUE, 'duplicated' => !$was_created) : TRUE;
}

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deleting the policy task
# Details:    Makes the NACL::STask::VserverSecurityFileDirectoryPolicyTask.pm->delete().
#
# Inputs:     NACL::STask::VserverSecurityFileDirectoryPolicyTask.pm->delete arguments
# Outputs:    Returns whether the delete was successful
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # delete the policy task service
    NACL::STask::VserverSecurityFileDirectoryPolicyTask->purge(%args,
                                                    nacltask_if_purged => 'pass',
                                                    _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("Policy Task". $args{'policy-name'}." on Vserver". $args{vserver}." deleted.");
    } else {
        $Log->comment("Vserver $args{vserver} does not have a  Policy Task $args{'policy-name'} to delete.");
    };

    $Log->exit() if $may_exit;
    return;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  NOOP for PolicyTask.pm
# Details:
# Inputs:     None
# Outputs:    None
# Exceptions: None
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
