#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary IPspace Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::IPspace

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::IPspace is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP IPspaces. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::IPspace;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::NetworkIpspace;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args
);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name   => 'ipspace',
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL create method
%DTD_To_NACL_Ignore = (
    parallel      => '',
    delete_policy => ''
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
    force             => '',
    ipspace           => '',
    delete_policy     => ''
);

=head1 METHODS

=head2 create

(Static method) Request creation of one ipspace serially
or in parallel and return whether the ipspace was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::IPspace->create(
            command_interface => $Node,
            ipspace           => $IPspace_name,
            vserver           => $Vserver_name,
            );

=over

=item Options

Either the resource_list is required or the NACL IPspace Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the ipspace in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL IPspace Creation Arguments should
NOT be passed in.

=item C<< NACL IPspace Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::STask::NetworkIpspace->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the IPspace based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "ipspace";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple ipspace serially
or in parallel and return whether the ipspace(s) was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::IPspace->delete(
            command_interface => $Node,
            ipspace           => $Ipspace_name,
            vserver           => $Vserver_name,
            );

=over

=item Options

Either the resource_list is required or the NACL IPspace Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the ipspace in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL IPspace Deletion Arguments should
NOT be passed in.

=item C<< NACL IPspace Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::NetworkIpspace->purge.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the IPspace(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "ipspace";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete


#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a ipspace.
# Details:    Makes the NACL::STask::NetworkIpspace->create() call. Also works
#             around some common non-fatal errors that can be return from
#             ipspace creation and then has a retry mechanism to retry ipspace
#             creation.
# Inputs:     NACL::STask::NetworkIpspace->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the ipspace was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # The Default ipspace is created by ontap and cannot be deleted.
    if($args{ipspace} =~ /^Default$/) {
        $Log->comment("Skipping creation of the Default Ipspace since " .
                      "the Default Ipspace cannot be deleted.\n");
        $Log->exit() if $may_exit;
        return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>TRUE) : TRUE;
    }

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Execute Create
    NACL::STask::NetworkIpspace->create(%args,
                                        nacltask_if_exists => $if_exists,
                                        _was_created => \$was_created);

    if($was_created) {
        $Log->comment("Using newly Created IPspace:" . $args{ipspace});
    } else {
        $Log->comment("Using already Created IPspace:" . $args{ipspace});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_created) : TRUE;

} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a ipspace.
# Details:    Makes the NACL::STask::NetworkIpspace->purge() calls. Also works around
#             some common non-fatal errors that can be return from ipspace
#             deletion and then has a retry mechanism to retry ipspace deletion.
# Inputs:     NACL::STask::NetworkIpspace->purge arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my %args = @_;

    # The Default ipspace is created by ontap and cannot be deleted.
    if($args{ipspace} =~ /^Default$/) {
        $Log->comment("Skipping deletion of the Default Ipspace since " .
                      "the Default Ipspace cannot be deleted.");
        $Log->exit() if $may_exit;
        return TRUE;
    }

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    my $delete_policy = delete $args{delete_policy};

    try {
        NACL::STask::NetworkIpspace->purge(%args,
                                            nacltask_if_purged => 'pass',
                                            _was_deleted       => \$was_deleted);
    } otherwise {
        my $exception = shift;
        if ($delete_policy eq "warn") {
            $Log->warn("Attempted to Delete ipspace " .
                        $args{ipspace} .
                       "Warning and Continuing Test\n" .
                       $exception->text());
        } elsif ($delete_policy eq "fail") {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    if($was_deleted) {
        $Log->comment("Ipspace " .
                       $args{ipspace} .
                      ". was deleted.\n");
    } else {
        $Log->comment("Cannot cleanup ipspace " .
                       $args{ipspace} .
                      ". It does not exist in the cluster" .
                      " or delete_policy=warn and an exception" .
                      " is printed above.\n");
    }

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::NetworkIpspace->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::NetworkIpspace->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
