
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation-core
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Isns is used to Config, Setup, Cleanup and
Deconfig Isns services on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::Isns;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and NACL::AppBuilder::EnGenX Libs
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::STask::VserverIscsiIsns;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (_reuse => 'if_exists');

# Arguments that are used by NACL::AppBuilder::EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel        => '',
    service_enabled => ''
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    vserver           => '',
);

=head1 METHODS

=head2 create

(Static method) Request to add the Isns service to a list
of vservers the command interface and vserver name are the
only required perameters.

  NACL::AppBuilder::EnGenX::Controller::Isns->create(
            command_interfac    => $node,
            vserver                 => $vserver_name,
            address                 => $ip_address,
            status-admin            => $adminitrative_status
            force                   => TRUE
            );

  NACL::AppBuilder::EnGenX::Controller::Isns->create(
            command_interface               => $node,
            vserver                         => $vserver_name,
            );

=over

=item Options

The NACL component only requires the user to specify the
command interface and the vserver. The ontap defaults will
be used for any option that is not stated.

=over

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $vserver_name >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.


=item C<< address => $ip_address >>
(Required)
This is the ip address of the iSNS server



=item C<< status-admin => $status_admin >>

(Optional)
The defualt is to have the service up and running. The user can specify
the service be down on creation.


=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the isns Targets based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "isns";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of the ISNS service and return whether the
services was deleted successfully or not.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the NACL::AppBuilder::EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Isns->delete(
            command_interface => $Node,
            vserver           => $Vserver_name,
            );

=over

=item Options

Either the resource_list is required or the NACL Isns Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed the Isns service in the resource list will be deleted
based on the information that is found at the resource list. If
the resource list is passed in the NACL Isns Deletion Arguments should
NOT be passed in.

=item C<< NACL Lif Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::VserverIscsiIsns->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Lif(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "isns";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the fcp service.
# Details:    Makes the NACL::STask::VserverIscsiIsns->create(). Also works
#             around some common non-fatal errors that can be return from
#             creation and then has a retry mechanism to retry volume
#             creation.
# Inputs:     NACL::STask::VserverIscsiIsns->create arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the volume was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # This creates a NACL command_interface if one is not passed in.
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }
    my $was_created;
    NACL::STask::VserverIscsiIsns->create(
        %args,
        nacltask_if_exists => $if_exists,
        _was_created       => \$was_created
    );
    $Log->comment("Enabled Isns service on vserver " . $args{vserver});
    if ($if_exists eq 'reuse'
        && !$was_created) {

        #Stop the Isns service so that it can be modified
        NACL::STask::VserverIscsiIsns->stop(
            command_interface  => $args{command_interface},
            vserver            => $args{vserver},
            nacltask_if_purged => 'pass'
        );

        # Modify the Isns service if there are new settings
        delete $args{address};
        NACL::STask::VserverIscsiIsns->modify(%args,
            nacltask_if_modified => 'pass');

        if ($args{status_admin} eq "down") {
            $Log->comment("Leaving the Isns service on vserver "
                    . $args{vserver}
                    . " in the admin down state.");
        } elsif ($args{status_admin} eq "up") {
            $Log->comment("The Isns service on vserver "
                    . $args{vserver}
                    . " has been started by the modify action.");
        }
    }

    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return TRUE;

}    #sub

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of the Isns service.
# Details:    Makes the NACL::C::VserverIscsiIsns->delete(). Also works
#             around some common non-fatal errors that can be return from
#             Isns delete and then has a retry mechanism to retry isns service
#             creation.
# Inputs:     NACL::STask::VserverIscsiIsns->delete arguments
# Outputs:    Returns whether the delete was successful or a hash with success
#             and whether the volume was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::VserverIscsiIsns->purge(%args, nacltask_if_purged => 'pass');

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
} ## end sub _get_dtd_nacl_translate

#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
} ## end sub _get_dtd_nacl_ignore

#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
} ## end sub _get_dtd_nacl_delete

1;
