# $Id:

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Lun Controller Layer Module
## @author dl-san-automation
## @status public
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::LunBind

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::LunBind is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP LunBinds. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::LunBind;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::LunBind;
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (parallel => '',);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface         => '',
    apiset_must               => '',
    apiset_should             => '',
    'vvol-path'                 => '',
    'protocol-endpoint-path'    => '',
    vserver                   => '',
    force                     => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple LunBinds serially
or in parallel and return whether the LunBind(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            command_interface           => $Node,
            'vvol-path'                 => $Vvol_path,
            'protocol-endpoint-path'    => $Protocol_endpoint_path,
            vserver                     => $Vserver_name,
            'method-timeout'            => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            command_interface           => $Node,
            'vvol-path'                 => $Vvol_path,
            'protocol-endpoint-path'    => $Protocol_endpoint_path,
            vserver                     => $Vserver_name,
            count                       => 5
            );


  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            command_interfac  => $Node,
            resource_list     => @LunBinds,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            command_interface => $Node,
            resource_list     => @LunBinds,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            resource_list     => @LunBinds,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->create(
            resource_list      => @LunBinds,
            );

=over

=item Options

Either the resource_list is required or the NACL LunBind Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the LunBinds in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Lun Creation Arguments should
NOT be passed in. (This is an array of EnGenX::Resource() objects not
an EnGenX::Resource_List() currently)

=item C<< NACL LunBind Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::Lun->Map.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< count => $lun_count >>

(Optional)
This arguments is used to create multiple LunBinds from the same NACL LunBind
Creation Arguments.  To create the names for the luns to be mapped, the lun name
passed in is appended with the index that it was create at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Lun(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lun_bind";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple LunBinds serially
or in parallel and return whether the LunBind(s) was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            command_interface           => $Node,
            'vvol-path'                 => $Vvol_path,
            'protocol-endpoint-path'    => $Protocol_endpoint_path,
            vserver                     => $Vserver_name,
            'method-timeout'            => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            command_interface           => $Node,
            'vvol-path'                 => $Vvol_path,
            'protocol-endpoint-path'    => $Protocol_endpoint_path,
            vserver                     => $Vserver_name,
            count                       => 5
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            command_interface => $Node,
            resource_list      => @LunBinds,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            command_interface => $Node,
            resource_list      => @LunBinds,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            resource_list      => @LunBinds,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunBind->delete(
            resource_list      => @LunBinds,
            );

=over

=item Options

Either the resource_list is required or the NACL LunBind Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the LunBinds in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL LunBind Deletion Arguments should
NOT be passed in.

=item C<< NACL LunBind Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::C::LunBind->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< count => $lun_count >>

(Optional)
This arguments is used to delete multiple LunBinds from the same NACL LunBind
Deletion Arguments.  To delete the names for the luns to be unmapped, the lun name
passed in is appended with the index that it was deleted at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Lun(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lun_bind";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a LunBinds.
# Details:    Makes the NACL::C::LunBind->create() call. Also works
#             around some common non-fatal errors that can be return from
#             lunbind creation and then has a retry mechanism to retry lunbind
#             creation.
# Inputs:     NACL::C::LunBind->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the lun bind was successful or a hash with success
#             and whether the LunBind was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $duplicated = FALSE;
    delete($args{if_exists});

    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Execute Create
    # There is no error on reuse.
    NACL::STask::LunBind->create(%args,);
    $Log->comment("Comment: Using newly Created LunBind: vserver:" . $args{vserver} .
                  "vvol:" . $args{'vvol-path'} . "mapped to protocol-endpoint:"
                  . $args{'protocol-endpoint-path'});
    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>$duplicated) : TRUE;

} ## end sub _nacl_create_call

#==========#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a Lun.
# Details:    Makes the NACL::C::LunBind->destroy() calls. Also works around
#             some common non-fatal errors that can be return from Lunbind
#             deletion and then has a retry mechanism to retry Lunbind deletion.
# Inputs:     NACL::C::LunBind->destroy() arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    #Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    $args{'force'} = 'TRUE' unless exists($args{'force'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::LunBind->purge(%args,
                                nacltask_if_purged => 'pass',
                                _was_deleted => \$was_deleted,);

    if ($was_deleted) {
        $Log->comment("LunBind: vserver:" . $args{vserver} .
                      "vvol:" . $args{'vvol-path'} . "mapped to protocol-endpoint:"
                      . $args{'protocol-endpoint-path'} . " was newly deleted");
    } else {
        $Log->comment("LunBind: Lun:" . $args{vserver} .
                      "vvol:" . $args{'vvol-path'} . "mapped to protocol-endpoint:"
                      . $args{'protocol-endpoint-path'} . " was already deleted");
    }

    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::LunBind->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::LunBind->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
