# $Id: //depot/prod/test/main/lib/EnGenX/Controller/Lun.pm#2 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Lun Controller Layer Module
## @author dl-san-automation
## @status public
## @pod here

=head1 NAME

EnGenX::Controller::LunMap

=head1 DESCRIPTION

EnGenX::Controller::Lun is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP Luns. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::LunMap;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);
use Digest::MD5;

# NACL and EnGenX Libs
use NACL::STask::Lun;
# Currently no Stask for LunMapping and the method used does not
# return anything useful on reuse so no need to create an STask
# at this time.
use NACL::STask::LunMapping;
use NACL::C::Job;
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use San::SanClient::ClientUtil;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Exceptions::Lun::LunIdAlreadyUsed;
use NACL::C::Exceptions::LunMapping::LunIdAlreadyUsed;
#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    lun               => '',
    qtree             => '',
    volume            => '',
    vserver           => '',
    path              => '',
    igroup            => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple lunmaps serially
or in parallel and return whether the lunmap(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  EnGenX::Controller::LunMap->create(
            command_interface => $Node,
            volume            => ( $Volume_name),
            lun               => $Lun_name,
            vserver           => $Vserver_name,
            igroup            => $Igroup_name,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->create(
            command_interface => $Node,
            volume            => ( $Volume_name),
            lun               => $Lun_name,
            vserver           => $Vserver_name,
            igroup            => $Igroup_name,
            'method-timeout'  => 3600,
            count             => 5
            );

  EnGenX::Controller::LunMap->create(
            command_interfac  => $Node,
            resource_list     => @LunMaps,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->create(
            command_interface => $Node,
            resource_list     => @LunMaps,
            );

  EnGenX::Controller::LunMap->create(
            resource_list     => @LunMaps,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->create(
            resource_list      => @LunMaps,
            );

=over

=item Options

Either the resource_list is required or the NACL LunMap Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the lunmaps in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Lun Creation Arguments should
NOT be passed in. (This is an array of EnGenX::Resource() objects not
an EnGenX::Resource_List() currently)

=item C<< NACL LunMap Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::STask::Lun->Map.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< count => $lun_count >>

(Optional)
This arguments is used to create multiple lunmaps from the same NACL LunMap
Creation Arguments.  To create the names for the luns to be mapped, the lun name
passed in is appended with the index that it was create at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Lun(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lun_map";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple lunmaps serially
or in parallel and return whether the lunmap(s) was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  EnGenX::Controller::LunMap->delete(
            command_interface => $Node,
            volume            => ( $Volume_name),
            lun                  => $Lun_name,
            vserver           => $Vserver_name,
            igroup            => $Igroup_name,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->delete(
            command_interface => $Node,
            volume            => ( $Volume_name),
            lun                  => $Lun_name,
            vserver           => $Vserver_name,
            igroup            => $Igroup_name,
            count             => 5
            );

  EnGenX::Controller::LunMap->delete(
            command_interface => $Node,
            resource_list      => @LunMaps,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->delete(
            command_interface => $Node,
            resource_list      => @LunMaps,
            );

  EnGenX::Controller::LunMap->delete(
            resource_list      => @LunMaps,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::LunMap->delete(
            resource_list      => @LunMaps,
            );

=over

=item Options

Either the resource_list is required or the NACL LunMap Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the lunmaps in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL LunMap Deletion Arguments should
NOT be passed in.

=item C<< NACL LunMap Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::LunMap->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< count => $lun_count >>

(Optional)
This arguments is used to delete multiple lunmaps from the same NACL LunMap
Deletion Arguments.  To delete the names for the luns to be unmapped, the lun name
passed in is appended with the index that it was deleted at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Lun(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lun_map";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a lunmaps.
# Details:    Makes the NACL::STask::Lun->map() call. Also works
#             around some common non-fatal errors that can be return from
#             lun creation and then has a retry mechanism to retry lun
#             creation.
# Inputs:     NACL::STask::Lun->map arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the map was successful or a hash with success
#             and whether the lunmap was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    my $was_mapped = 0;

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    my @igroups = (exists($args{igroups}) and ref($args{igroups}) eq "ARRAY") ? @{delete($args{igroups})} : (delete($args{igroup}));
    if (not scalar(@igroups)) {
        # There are no igroups to map to, this is an error
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Invalid value for 'igroups', it was passed in empty");
    }

    # Get Add reporting nodes parameters
    my $reporting_nodes;
    $reporting_nodes->{nodes} = [split(/,/,delete $args{reporting_nodes})] if (exists($args{reporting_nodes}));
    $reporting_nodes->{all} = delete $args{report_all_nodes} if (exists($args{report_all_nodes}));
    $reporting_nodes->{'local-nodes'} = delete $args{'report_local_nodes'} if (exists($args{'report_local_nodes'}));
    $reporting_nodes->{'destination-aggregate'} = delete $args{report_dest_aggr} if (exists($args{report_dest_aggr}));
    $reporting_nodes->{'destination-volume'} = delete $args{report_dest_vol} if (exists($args{report_dest_vol}));

    # Get Remove reporting nodes parameters
    my $remove_reporting_nodes;
    $remove_reporting_nodes->{'nodes'} = [split(/,/,delete $args{remove_reporting_nodes})] if (exists($args{remove_reporting_nodes}));
    $remove_reporting_nodes->{'remote-nodes'} = delete $args{remove_remote_nodes} if (exists($args{remove_remote_nodes}));

    # check if a lun id was specified
    my $lun_id_range = exists($args{lun_id_range}) ? delete($args{lun_id_range}) : undef;
    my $require_same_lun_id_per_igroup = exists($args{require_same_lun_id_per_igroup}) ? delete($args{require_same_lun_id_per_igroup}) : undef;
    my $global_lun_id = exists($args{'lun-id'}) ? delete($args{'lun-id'}) : undef;

    my $lun_id_override_hash = {};
    my $successful_maps = 0;
    my $lun_id_arr = [];

    # generate a random lun id from the lun id range
    # if the user passed in a valid lun id range
    if ($lun_id_range and $lun_id_range !~ /^\s*$/) {
        $lun_id_arr = $pkg->_generate_lun_id_range($lun_id_range,
                                                   $args{vserver},
                                                   $args{volume},
                                                   $args{qtree},
                                                   $args{lun});
    }

    my $current_igroup = undef;
    my $map_retry = 0;
    # map the lun to each igroup
    foreach my $igroup (@igroups) {
       my $redo = 0;
       if(not(defined($current_igroup)) || $current_igroup ne $igroup) {
            $current_igroup = $igroup;
            $map_retry = 0;
        }

        # update the igroup field for the command invocation
        $args{igroup} = $igroup;

        # by default all igroups will be mapped using the global lun id field
        if (defined($global_lun_id)) {
            $args{'lun-id'} = $global_lun_id;
        }

        # if this igroup requested to use a specific lun id, then override that now
        if (exists($lun_id_override_hash->{$igroup})) {
            $args{'lun-id'} = $lun_id_override_hash->{$igroup};
        } elsif (scalar(@{$lun_id_arr})) {
            # otherwise, if a random lun id was selected, use the next one in the list
            $args{'lun-id'} = $lun_id_arr->[0];
        }

        # instantiate the handle lun id routine
        my $handle_used_lun_id = sub {
            my $e = shift;
            # (1) if the user did not specify an absolute lun id to use for this igroup (lun_map_id_hash->{$igroup})
            # (2) we have a lun id range to work with
            # (3) we havn't exhausted through the lun_id_range yet for this igroup,
            # then grab another lun id from the list
            if (not defined($lun_id_override_hash->{$igroup}) and
                defined(scalar(@{$lun_id_arr})) and
                $map_retry < scalar(@{$lun_id_arr})) {
                # if we had a previous succesful map and the user requires that the lun use the same lun id for
                # all igroups in the map list, then bail out
                if ($successful_maps > 0 and $require_same_lun_id_per_igroup == 1) {
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw("Was not able to map LUN " . $args{lun} . " to the all igroups in the map list " .
                                               "with the same lun id '" . $args{'lun-id'} . "'");
                }

                # rotate the lun id range array
                push(@{$lun_id_arr}, shift(@{$lun_id_arr}));

                # pick a new lun id and increment the retry
                $args{'lun-id'} = $lun_id_arr->[0];
                $map_retry++;
                $redo = 1;
            } else {
                # failed to use this lun-id
                $Log->exit() if $may_exit;

                # Rethrowing caught exception
                $e->throw();
            }
        };

        # map the lun to the igroup now
        try {
            if ($args{command_interface}->version_manager()->is_version_le(release => '8.2.3')) {
                NACL::STask::Lun->map(%args,
                                      nacltask_if_mapped_igroup => $if_exists,
                                      _was_mapped               => \$was_mapped);
            } else {
                NACL::STask::LunMapping->create(%args,
                                                nacltask_if_exists => $if_exists,
                                                _was_created        => \$was_mapped);
            }
        } catch NACL::C::Exceptions::Lun::LunIdAlreadyUsed with {
            my $e = shift;
            # another lun is mapped to this igroup at this lun id
            # pick a new lun id if possible
            $handle_used_lun_id->($e);
        } catch NACL::C::Exceptions::LunMapping::LunIdAlreadyUsed with {
            my $e = shift;
            # another lun is mapped to this igroup at this lun id
            # pick a new lun id if possible
            $handle_used_lun_id->($e);
        };

        # If we mapped it successfully then add it to the number of successful_maps
        $successful_maps++ if($was_mapped);
        if($was_mapped) {
            $Log->comment("Using newly Created LunMap for lun:" . $args{lun} .
                          " in qtree:" . ($args{qtree}||'""') .
                          " in volume:" . $args{volume} .
                          " in vserver:" . $args{vserver}.
                          " mapped to igroup:" . $args{igroup});
        } else {
            $Log->comment("Using already Created LunMap for lun:" . $args{lun} .
                          " in qtree:" . ($args{qtree}||'""') .
                          " in volume:" . $args{volume} .
                          " in vserver:" . $args{vserver}.
                          " mapped to igroup:" . $args{igroup});
        }

        # if the redo flag is set, redo the loop
        redo if $redo;
    }

    # Add reporting nodes for the lun
    if($reporting_nodes) {
        my %params = (%{$reporting_nodes},%args);
        NACL::STask::LunMapping->add_reporting_nodes(%params);
    }

    # Remove reporting nodes for the lun
    if($remove_reporting_nodes) {
        my %params = (%{$remove_reporting_nodes},%args);
        NACL::STask::LunMapping->remove_reporting_nodes(%params);
    }

    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_mapped) : TRUE;
} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a Lun.
# Details:    Makes the NACL::STask::Lun->unmap() calls. Also works around
#             some common non-fatal errors that can be return from Lun
#             deletion and then has a retry mechanism to retry Lun deletion.
# Inputs:     NACL::STask::Lun->unmap arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    #Locals
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    my $was_unmapped = 0;
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    if ($args{command_interface}->version_manager()->is_version_le(release => '8.2.3')) {
        NACL::STask::Lun->unmap(%args,
                                nacltask_if_unmapped_igroup => 'pass',
                                _was_unmapped  => \$was_unmapped);
    } else {
        NACL::STask::LunMapping->purge(%args,
                                       nacltask_if_purged => 'pass',
                                       _was_deleted       => \$was_unmapped);
    }

    NACL::STask::Lun->unmap(%args,
                            nacltask_if_unmapped_igroup => 'pass',
                            _was_unmapped  => \$was_unmapped);
    if($was_unmapped) {
        $Log->comment("LunMap:" . $args{lun} .
                          " in qtree:" . $args{qtree} .
                          " in volume:" . $args{volume} .
                          " in vserver:" . $args{vserver}.
                          " mapped to igroup:" . $args{igroup} .
                          " was deleted");
    } else {
        $Log->comment("Cannot cleanup LunMap:" . $args{lun} .
                          " in qtree:" . $args{qtree} .
                          " in volume:" . $args{volume} .
                          " in vserver:" . $args{vserver}.
                          " mapped to igroup:" . $args{igroup} .
                          " it does not exist in the cluster");
    }

    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::LunMap->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::LunMap->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

###############################################################################
# Method:    _generate_lun_id_range
# Objective: Given a lun id range, generate an array of all the lun ids, randomized
# based on the vserver vol and qtree name specified
# Returns:   0 or 1
###############################################################################
sub _generate_lun_id_range {
    my ($self,
        $lun_id_range,
        $vserver_name,
        $vol_name,
        $qtree_name,
        $lun_name) = @_;

    my @return = ();
    if ($lun_id_range =~ /^(\d+)\-(\d+)$/) {
        my $start = $1;
        my $end = $2;
        for (my $i = $start; $i <= $end; $i++) {
            push(@return, $i);
        }
    }

    $qtree_name = "" if (not defined($qtree_name));

    # create a unique integer based on the md5 hash of the vserver vol qtree
    my $md5s = Digest::MD5::md5_hex($vserver_name . "__" . $vol_name . "__" . $qtree_name . "__" . $lun_name);
    my $md5strsub = substr($md5s, 0, 8);
    $md5strsub =~ tr/a-f/1-6/;

    # set the seed based on the vserver volume and qtree name
    srand($md5strsub);

    &ClientUtil::randomize_array(\@return);

    return \@return;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
