# $Id: //depot/prod/test/main/lib/EnGenX/Controller/NetworkRoute.pm#2 $

#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoute Controller layer module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::NetworkRoute

=head1 DESCRIPTION

EnGenX::Controller::NetworkRoute is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP network routes. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::NetworkRoute;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use NATE::STAF::Peer qw($Parent);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::CS::NetworkInterface;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::C::Job;
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args
    $Mem_Resource
);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'nacltask_if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    gateway           => '',
    vserver           => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple NetworkRoutes serially
or in parallel and return whether the NetworkRoute(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework. lif is only needed if
using a non-ipspace build.

  EnGenX::Controller::NetworkRoute->create(
            command_interface => $Node,
            vserver           => $Vserver_name,
            gateway           => "10.63.9.163"
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::NetworkRoute->create(
            command_interfac  => $Node,
            resource_list      => @NetworkRoutes,
            'method-timeout'  => 3600,
            );

=over

=item Options

Either the resource_list is required or the NACL NetworkRoute Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the NetworkRoutes in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL NetworkRoute Creation Arguments should
NOT be passed in.

=item C<< NACL NetworkRoute Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::NetworkInterface->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< lif >>

(Optional)
lif is only needed if using a non-ipspace build.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the NetworkRoute(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

TODO: The Task::_create_mulptiple sub will fail to create multiple NetworkRoutes because
      the method needs to also provide a unique IP ADDRESS. This unique address
      (or Array of usable address) would be passed by the user. None the less,
      the method needs to accept the array of IPs and use it when making the
      NACL call.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "NetworkRoute";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub create

#==============================================================================
#The delete function is not going to be implemented
#due to the route being deleted upon vserver deletion.
#==============================================================================
sub delete {
    #TODO: no-op for now. May implement in the future
    return;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a NetworkRoute.
# Details:    Makes the NACL::C::NetworkRoute->create() call. Also works
#             around some common non-fatal errors that can be return from
#             NetworkRoute creation and then has a retry mechanism to retry NetworkRoute
#             creation.
# Inputs:     NACL::C::NetworkRoute->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the NetworkRoute was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_created = 0;
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';

    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    if(not defined($args{destination}) or not length($args{destination})) {
        # Set the default destination
        $args{destination} = '0.0.0.0/0';
        if ($args{gateway} =~ /:/){
            $args{destination} = '::/0';
        }
    }

    if ($args{command_interface}->has_uichange(uichange => 'ipspaces-ms4')) {
        #IPSpace build, should use new library
        NACL::STask::NetworkRoute->create(
                                command_interface  => $args{command_interface},
                                'vserver'          => $args{vserver},
                                'gateway'          => $args{gateway},
                                'destination'      => $args{destination},
                                nacltask_if_exists => $if_exists,
                                _was_created       => \$was_created);
    } else {
        # Get information for older command
        my $lif_cs_obj = NACL::CS::NetworkInterface->fetch(
                            command_interface => $args{command_interface},
                            filter => {
                                vserver           => $args{vserver},
                                lif               => $args{lif}},
                            (requested_fields => [ 'routing-group']));

        # Non-IPSpace build, should use older library
        if ($lif_cs_obj->routing_group() ne "-") {
            NACL::STask::NetworkRoutingGroupsRoute->create(
                                command_interface  => $args{command_interface},
                                'vserver'          => $args{vserver},
                                'routing-group'    => $lif_cs_obj->routing_group(),
                                'gateway'          => $args{gateway},
                                'destination'      => $args{destination},
                                nacltask_if_exists => $if_exists,
                                _was_created       => \$was_created);
        }
    }


    if($was_created) {
        $Log->comment("Using newly Created Route with gateway:" . $args{gateway} . " in vserver " . $args{vserver});
    } else {
        $Log->comment("Using already Created Route with gateway:" . $args{gateway} . " in vserver " . $args{vserver});
    }
    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_created) : TRUE;


} ## end sub _nacl_create_call

#==============================================================================
#The _nacl_delete_call function is not going to be implemented
#due to the route being deleted upon vserver deletion.
#==============================================================================
sub _nacl_delete_call {
    #TODO: no-op for now. May implement in the future
    return;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::NetworkRoute->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::NetworkRoute->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}
#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
