# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLogin Controller Layer Module
## @author dl-nacl-dev
## @status public
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::SecurityLogin

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::SecurityLogin is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task. It is used to Set Attributes
of Security login. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::SecurityLogin;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use Tharn qw($Log);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::SecurityLogin;

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
    'user-or-group-name' => '',
    application => '',
    authmethod  => '',
    'method-timeout'  => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple security login(s) serially
or in parallel and return whether the security login(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->create(
            command_interface   => $Node,
            vserver             => $vserver,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->create(
            command_interfac  => $Node,
            resource_list     => @SecurityLogin,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->create(
            resource_list     => @SecurityLogin,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->create(
            resource_list      => @SecurityLogin,
            );

=over

=item Options

Either the resource_list is required or the NACL SecurityLogin Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the on demand tasks in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL SecurityLogin Creation Arguments should
NOT be passed in. (This is an array of EnGenX::Resource() objects not
an EnGenX::Resource_List() currently)

=item C<< NACL SecurityLogin Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::SecurityLogin->create().
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< vserver => $name >>

(Required)
This arguments is to specify the name of the vserver.

=item C<< user-or-group-name => $user_or_group >>

(Required)
This arguments is used to specify the user or group name.

=item C<< application => $application >>

(Required)
This arguments is used to specify the application.

=item C<< authmethod => $authmethod >>

(Required)
This arguments is used to specify the Authentication Method.

=item C<< password => $password >>

(Required)
This arguments is used to specify the password.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the SecurityLogin(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "security_login";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit();
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple SecurityLogin(s) serially
or in parallel and return whether the on demand tasks was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(
            command_interface   => $Node,
            vserver             => $vserver,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(
            command_interface => $Node,
            resource_list      => @SecurityLogin,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(
            command_interface => $Node,
            resource_list      => @SecurityLogin,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(
            resource_list      => @SecurityLogin,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(
            resource_list      => @SecurityLogin,
            );

=over

=item Options

Either the resource_list is required or the NACL Lun Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the on demand tasks in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Security Login
Deletion Arguments should NOT be passed in.

=item C<< NACL SecurityLogin Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::C::SecurityLogin->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< vserver => $name >>

(Required)
This arguments is to specify the name of the vserver.

=item C<< user-or-group-name => $user_or_group >>

(Required)
This arguments is used to specify the user or group name.

=item C<< application => $application >>

(Required)
This arguments is used to specify the application.

=item C<< authmethod => $authmethod >>

(Required)
This arguments is used to specify the Authentication Method.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the task(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "security_login";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);
    $Log->exit();
    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of on demand tasks.
# Details:    Makes the NACL::STask::SecurityLogin->create() call.
# Inputs:     NACL::STask::SecurityLogin->create() arguments
# Outputs:    Returns whether the attribute was set.
#             of the ScannerPool was successful or a hash with success
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Create Scanner Pool
    NACL::STask::SecurityLogin->create(%args,
                                    nacltask_if_exists => $if_exists,
                                    _was_created => \$was_created);

    if($was_created) {
        $Log->comment( "Security Login was  succesfully created for vserver " . $args{vserver});
    } else {
        $Log->comment( "Security Login already exists on vserver " . $args{vserver} . ". It will be reused.");
    }

    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE , 'duplicated' => !$was_created) : TRUE;
} ## end sub _nacl_create_call

#========================================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of on demand task(s).
# Details:    Makes the NACL::STask::SecurityLogin->delete() calls.
# Inputs:     NACL::STask::SecurityLogin->delete arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::SecurityLogin->purge(%args,
                                    nacltask_if_purged => 'pass',
                                    _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("Security Login was deleted on vserver " . $args{vserver});
    } else {
        $Log->comment("Security Login was already deleted on vserver " . $args{vserver});
    }

    $Log->exit();
    return TRUE;

} ## end sub _nacl_delete_call


#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
