# $Id: //depot/prod/test/main/lib/EnGenX/Controller/Snapmirror.pm $

#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Snapmirror is used to Config, Setup, Cleanup and
Deconfig Snapmirror on a vserver.

=cut

package NACL::AppBuilder::EnGenX::Controller::Snapmirror;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::STask::Snapmirror;
use NACL::STask::Vserver;
use NACL::C::Snapmirror;
use NACL::CS::Snapmirror;
use NACL::C::Job;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel      => '',
    engenx_relationship_id => '',
    home_cluster  => '',
);

# Arguments that can be passed into the NACL delete
# command
# Arguments that can be passed into the NACL delete
# command.
%Delete_Args = (
    command_interface       => '',
    apiset_must             => '',
    apiset_should           => '',
    'source-vserver'        => '',
    'source-cluster'        => '',
    'source-volume'         => '',
    'source-path'           => '',
    'destination-vserver'        => '',
    'destination-cluster'        => '',
    'destination-volume'         => '',
    'destination-path'           => '',
    'cg-item-mappings'           => '',
);

=head1 METHODS

=head2 create

(Static method) Request to add the qos policy group service to a list
of vservers the command interface and vserver name are the
only required perameters.

  NACL::AppBuilder::EnGenX::Controller::Snapmirror->create(
            command_interface               => $node,
            home_cluster                      => $home_cluster,
            remote_cluster                    => $remote_cluster,
            );


=over

=item Options

The NACL component only requires the user to specify the
command interface and the vserver. The ontap defaults will
be used for any option that is not stated.

=over

=item C<< home_cluster => $home_cluster >>

(Required)
This is either the name of the home cluster/node or
the a cluster command interface.

=item C<< remote_cluster => $remote_cluster >>

(Required)
This is either the name of the remote cluster/node or
the a cluster command interface.

=item C<< command_interface => $node >>

(Required)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Snapmirror based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut
sub create {

    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "Snapmirror";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

#==============================================================================
#The delete function is not going to be implemented
#due to the options being deleted upon vserver deletion.
#==============================================================================
sub delete {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # TODO Remove $args{'Name'} from everywhere. And ensure
    # no errors occur.
    $args{'Name'} = "Snapmirror";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
}

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of the vserver peer.
# Details:    Makes the NACL::C::VserverSnapmirror->create(). Also works
#             around some common non-fatal errors that can be return from
#             volume creation and then has a retry mechanism to retry volume
#             creation.
# Inputs:     NACL::C::Snapmirror->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the volume was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {

    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    my $was_created = 0;
    my $start_destination_vserver = delete $args{start_destination_vserver};
    if (defined $args{'source-cluster'}) {
        $args{source_ci} =  NACL::STask::Cluster->new(name => $args{'source-cluster'}) if not defined($args{source_ci});
    }

    NACL::STask::Snapmirror->create(%args,
                                    _was_created              => \$was_created,
                                    nacltask_if_exists        => $if_exists,
                                    do_initialize             => 0);

    if($was_created) {
        $Log->comment("Using newly Created Snapmirror");
    } else {
        $Log->comment("Using already Created Snapmirror:");
    }

    if($start_destination_vserver) {
        NACL::STask::Vserver->start(command_interface   => $args{command_interface},
                                    vserver             => $args{'destination-vserver'},
                                    nacltask_if_started => 'pass',);
                                    
    }

    $Log->exit();
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>(not $was_created)) : TRUE;

}#sub

#==============================================================================
#The _nacl_delete_call function is not going to be implemented
#due to the options being deleted upon volume deletion.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (defined $args{'source-cluster'}) {
        $args{source_ci} =  NACL::STask::Cluster->new(name => $args{'source-cluster'}) if not defined($args{source_ci});
    }

    try {
        NACL::STask::Snapmirror->update(%args,);
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        if ($exception->text() =~ /entry doesn't exist|does not exist|Cannot find any Vserver DR relationship|no entries matching/ or
            $exception->text() =~ /Relationship with destination .* not found/)    # CMode
        {
            $Log->comment(
                "Snapmirror: was already deleted");
        } elsif ($exception->text() =~ /Cannot perform this operation when SnapMirror operation state is "Uninitialized"/ or
                 $exception->text() =~ /is not initialized/ or
                 $exception->text() =~ /"update" not supported for "SnapMirror sync CG" relationships/ or
                 $exception->text() =~ /is not a Vserver of subtype "dp-destination"/)    # CMode
        {
            # Ignore this and continue with cleaning up the snapmirror relationship.
            # This happens sometimes when the snapmirror is not initialized.
            # Or the snapmirror has already been broken.
            # Or the relationship is a cg relationship.
        } else {
            $Log->exit();
            # Rethrowing caught exception
            $exception->throw();
        }
    } catch NACL::Exceptions::VerifyFailure with {
        my $exception = shift;
        if ($exception->text() =~ /Retry initialize or perform a SnapMirror resync/)    # CMode
        {
            # Ignore this and continue with cleaning up the snapmirror relationship.
            # This happens sometimes when the snapmirror is not initialized.
        } else {
            $Log->exit();
            # Rethrowing caught exception
            $exception->throw();
        }

    };


    NACL::STask::Snapmirror->purge(%args,
                                    #_was_deleted              => \$was_deleted,
                                    nacltask_if_purged        => 'pass');

    $Log->exit();
    return TRUE;
}

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::Vserver->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     none.
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
