# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Snapshot Policy Controller Layer Module
## @author ng-ptefit-automation-core <ng-ptefit-automation-core@netapp.com>
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP Snapshot Policy. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw( log_global );
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF );

# NACL and EnGenX Libs
use NACL::STask::SnapshotPolicy;

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name   => 'policy',
    _reuse => 'if_exists',
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    policy            => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple snapshot policies serially
or in parallel and return whether the policy or policies was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(
            command_interface   => $command_interface,
            vserver             => $vserver,
            policy              => $policy,
            schedule1           => $schedule1,
            count1              => $count1,
            enabled             => $enabled,
            'method-timeout'    => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(
            command_interface  => $command_interface,
            resource_list      => @SnapshotPolicies,
            'method-timeout'   => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(
            command_interface  => $command_interface,
            resource_list      => @SnapshotPolicies,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(
            resource_list      => @SnapshotPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(
            resource_list      => @SnapshotPolicies,
            );

=over

=item Options

Either the resource_list is required or the NACL SnapshotPolicy Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the policies, the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL SnapshotPolicy Creation Arguments should
NOT be passed in.

=item C<< NACL SnapshotPolicy Creation Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::SnapshotPolicy->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< vserver => $vserver >>

(Required)
the name of the vserver for the Snapshot policy.

=item C<< policy => $policy >>

(Required)
The Snapshot policy name.

=item C<< schedule1 => $schedule1 >>

(Required)
Name of the first schedule associated with the Snapshot policy.

=item C<< count1 => $count1 >>

(Required)
The maximum number of Snapshot copies that can be taken by the first schedule.

=item C<< enabled => $enabled >>

(Required)
Specifies whether the Snapshot policy is enabled.

=item C<< comment => $string >>

(Optional)
a text comment for the Snapshot policy.

=item C<< schedule2 => $schedule_name >>

(Optional)
name of the second schedule associated with the Snapshot policy

=item C<< count2 => $count >>

(Optional)
the maximum number of Snapshot copies that can be taken by the second schedule.

=item C<< schedule3 => $schedule_name >>

(Optional)
name of the third schedule associated with the Snapshot policy.

=item C<< count3 => $count >>

(Optional)
maximum number of Snapshot copies that can be taken by the third schedule.

=item C<< schedule4 => $schedule_name >>

(Optional)
name of the fourth schedule associated with the Snapshot policy.

=item C<< count4 => $count >>

(Optional)
maximum number of Snapshot copies that can be taken by the fourth schedule.

=item C<< schedule5 => $schedule_name >>

(Optional)
name of the fifth schedule associated with the Snapshot policy.

=item C<< count5 => $count >>

(Optional)
maximum number of Snapshot copies that can be taken by the fifth schedule.

=item C<< 'prefix1' => $string >>

(Optional)
the prefix associated with the first schedule.

=item C<< 'prefix2 => $string >>

(Optional)
the prefix associated with the second schedule.

=item C<< 'prefix3' => $string >>

(Optional)
the prefix associated with the third schedule.

=item C<< 'prefix4' => $string >>

(Optional)
the prefix associated with the fourth schedule.

=item C<< 'prefix5' => $string >>

(Optional)
the prefix associated with the fifth schedule.

=item C<< 'snapmirror-label1' => $string >>

(Optional)
the SnapMirror Label of the first schedule associated with the Snapshot policy.

=item C<< 'snapmirror-label2' => $string >>

(Optional)
the SnapMirror Label of the second schedule associated with the Snapshot policy.

=item C<< 'snapmirror-label3' => $string >>

(Optional)
the SnapMirror Label of the third schedule associated with the Snapshot policy.

=item C<< 'snapmirror-label4' => $string >>

(Optional)
the SnapMirror Label of the fourth schedule associated with the Snapshot policy.

=item C<< 'snapmirror-label5' => $string >>

(Optional)
the SnapMirror Label of the fifth schedule associated with the Snapshot policy.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the SnapshotPolicy(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "snapshot_policy";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);
    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple policies serially
or in parallel and return whether the policy or policies was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.


  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(
            command_interface   => $command_interface,
            vserver             => $vserver,
            policy              => $policy,
            'method-timeout'    => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(
            command_interface  => $command_interface,
            resource_list      => @SnapshotPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(
            command_interface => $command_interface,
            resource_list      => @SnapshotPolicies,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(
            resource_list      => @SnapshotPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(
            resource_list      => @SnapshotPolicies,
            );

=over

=item Options

Either the resource_list is required or the NACL SnapshotPolicy Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the SnapshotPolicies , the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL SnapshotPolicy Deletion Arguments should
NOT be passed in.

=item C<< NACL SnapshotPolicy Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::SnapshotPolicy->purge.
If these arguments are passed in the resource_list argument should NOT
be passed in.


=item C<< vserver => $vserver >>

(Required)
the name of the vserver for the Snapshot policy.

=item C<< policy => $policy >>

(Required)
the name of the Snapshot policy.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the policy(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "snapshot_policy";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);
    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub delete


#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a Snapshot Policy.
# Details:    Makes the NACL::STask::SnapshotPolicy->create() call. Also works
#             around some common non-fatal errors that can be return from
#             policy creation.
# Inputs:     NACL::STask::SnapshotPolicy->create arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the policy was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;
    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Execute Create
    NACL::STask::SnapshotPolicy->create(%args,
                             nacltask_if_exists => $if_exists,
                             _was_created => \$was_created);

    if($was_created) {
        $Log->comment("Using newly Created Snapshot Policy ".$args{'policy'}." on vserver ".$args{'vserver'});
    } else {
        $Log->comment("Using already Created Snapshot Policy ".$args{'policy'}." on vserver ".$args{'vserver'});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_created) : TRUE;


} ## end sub _nacl_create_call

#========================================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a SnapshotPolicy.
# Details:    Makes the NACL::STask::SnapshotPolicy->purge() calls. Also works around
#             some common non-fatal errors that can be return from Policy
#             deletion.
# Inputs:     NACL::STask::SnapshotPolicy->purge arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::SnapshotPolicy->purge(%args,
                            nacltask_if_purged => 'pass',
                            _was_deleted => \$was_deleted);

    if($was_deleted) {
        $Log->comment("SnapshotPolicy ".$args{'policy'}." was deleted on vserver ".$args{'vserver'});
    } else {
        $Log->comment("SnapshotPolicy ".$args{'policy'}." was already deleted on vserver ".$args{'vserver'});
    }

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call
#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::SnapshotPolicy->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::SnapshotPolicy->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
