# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Volume Clone Controller Layer Module
## @author ng-rtp-tools
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::Volume::Clone

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Volume::Clone is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP Volumes. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Volume::Clone;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::VolumeClone;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args
);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name   => 'flexclone',
    _reuse => 'if_exists',
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods.
%DTD_To_NACL_Ignore = (
    parallel                 => '',
);

# Arguments that can be passed into the NACL delete
# command.
%Delete_Args = (
    command_interface       => '',
    apiset_must             => '',
    apiset_should           => '',
    vserver                 => '',
    flexclone               => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple volume clones serially
or in parallel and return whether the volume clone(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            command_interface => $Node,
            name              => ( $flexclone_name),
            'parent-volume'   => $volume,
            vserver           => $Vserver_name,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            command_interface => $Node,
            name              => ( $flexclone_name),
            'parent-volume'   => $volume,
            vserver           => $Vserver_name,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            command_interface  => $Node,
            resource_list      => @Vols,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            command_interface => $Node,
            resource_list      => @Vols,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            resource_list      => @Vols,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(
            resource_list      => @Vols,
            );

=over

=item Options

Either the resource_list is required or the NACL Volume Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the volumes in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Volume Creation Arguments should
NOT be passed in.

=item C<< NACL Volume Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::STask::VolumeClone->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< name => $flexclone >>

(Required)
The name of the flex clone.

=item C<< parent-volume => $volume >>

(Required)
The name of the volume to be cloned

=item C<< vserver => $vserver >>

(Required)
The name of the vserver.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method-timeout => $NACL_method_timeout >>

(Optional)
This is the method-timeout that will be used during NACL calls. If a
method-timeout is not passed in then the method-timeout will be defaulted to 3600.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Volume clone(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    $args{'Name'} = "volume-clone";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple volumes serially
or in parallel and return whether the volume(s) was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            command_interface => $Node,
            volume            => ( $Volume_name),
            vserver           => $Vserver_name,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            command_interface => $Node,
            volume            => ( $Volume_name),
            vserver           => $Vserver_name,
            'method-timeout'  => 3600,
            count             => 5
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            command_interface => $Node,
            resource_list      => @Vols,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            command_interface => $Node,
            resource_list      => @Vols,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            resource_list      => @Vols,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(
            resource_list      => @Vols,
            );

=over

=item Options

Either the resource_list is required or the NACL Volume Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the volumes in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Volume Deletion Arguments should
NOT be passed in.

=item C<< NACL Volume Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::Volume->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method-timeout => $NACL_method_timeout >>

(Optional)
This is the method-timeout that will be used during NACL calls. If a
method-timeout is not passed in then the method-timeout will be defaulted to 3600.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Volume(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    $args{'Name'} = "volume-clone";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete


#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a volume clone.
# Details:    Makes the NACL::STask::VolumeClone->create() call.
# Inputs:     NACL::STask::VolumeClone->create arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the volume was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_created = 0;
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # Execute Create
    NACL::STask::VolumeClone->create(%args,
                             nacltask_if_exists => $if_exists,
                             _was_created => \$was_created);

    if($was_created) {
        $Log->comment("Using newly created volume clone : ".$args{flexclone});
    } else {
        $Log->comment("Using already Created volume clone: ".$args{flexclone});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>!$was_created) : TRUE;

} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a volume.
# Details:    Makes the NACL::STask::Volume->purge() call.
# Inputs:     NACL::STask::Volume->purge arguments
# Outputs:    Returns whether the delete was successful.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    $args{volume} = delete($args{flexclone});

    NACL::STask::Volume->purge(%args,
                               _was_deleted => \$was_deleted,
                               nacltask_if_purged => 'pass');

    if($was_deleted) {
        $Log->comment("Volume Clone:" . $args{volume} .
                      " in Vserver:" . $args{vserver} . " was deleted");
    } else {
        $Log->comment("Cannot cleanup Volume clone:" . $args{volume} .
                      " in Vserver:" . $args{vserver} .
                      ". It does not exist in the cluster.\n");
    }
    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::Volume->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::Volume->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
