#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary OnAccessPolicy Controller Layer Module
## @author dl-san-automation
## @status public
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy is a derived class of
NACL::AppBuilder::NACL::AppBuilder::EnGenX::Controller::Task. It is used to Set Attributes
of ONTAP Vserver Vscan OnAccess policies. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);
use Data::Dumper;

# NACL and NACL::AppBuilder::EnGenX Libs
use NACL::STask::VserverVscanOnAccessPolicy;

# Exceptions
use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled qw(:try);

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
    name   => 'policy-name'
);

# Arguments that are used by NACL::AppBuilder::EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (parallel => '',);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
    'method-timeout'  => '',
    'policy-name'     => '',
    revert_to_default => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple vscan(s) serially
or in parallel and return whether the security trace filter(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the NACL::AppBuilder::EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::OnAccessPolicy->create(
            command_interface     => $Node,
            'policy-name'         => $Policy_name,
            'protocol'            => $Protocol,
            'filters'             => $Filters,
            'max-file-size'       => $Max_size,
            'paths-to-exclude'    => $Excluded_paths,
            'file-ext-to-exclude' => $Excluded_exts,
            );

  NACL::AppBuilder::EnGenX::Controller::OnAccessPolicy->create(
            command_interfac  => $Node,
            resource_list     => @OnAccessPolicies,
            'method-timeout'  => 3600,
            );


  NACL::AppBuilder::EnGenX::Controller::OnAccessPolicy->create(
            resource_list     => @OnAccessPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::OnAccessPolicy->create(
            resource_list      => @OnAccessPolicies,
            );

=over

=item Options

Either the resource_list is required or the NACL OnAccessPolicy Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the security trace filters in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL OnAccessPolicy Creation Arguments should
NOT be passed in. (This is an array of NACL::AppBuilder::EnGenX::Resource() objects not
an NACL::AppBuilder::EnGenX::Resource_List() currently)

=item C<< NACL OnAccessPolicy Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::OnAccessPolicy->set().
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< policy-name => $Policy_name >>

(Optional)
This arguments is to specify the name for the on-access-policy.

=item C<< protocol => $Protocol >>

(Optional)
This arguments is used to specify the protocol for the on-access-policy.

=item C<< filters => $Filters >>

(Optional)
This arguments is used to specify the filters for the on-access-policy.

=item C<< max-file-size => $Max_size >>

(Optional)
This arguments is used to specify the max-file-size for the on-access-policy.

=item C<< pahts-to-exclude => $Excluded_paths >>

(Optional)
This arguments is used to specify the paths-to-exclude for the on-access-policy.
=back

=item C<< file-ext-to-exclude => $Excluded_exts >>

(Optional)
This arguments is used to specify the file-extensions toi exclude for the on-access-policy.
=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Vscan::OnAccessPolicy(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "On_Access_Policy";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple OnAccessPolicies serially
or in parallel and return whether the security trace filter(s) was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the NACL::AppBuilder::EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(
            command_interface => $Node,
            vserver           => $Vserver_name,
            'policy-name'     => $Policy_name,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(
            command_interface => $Node,
            resource_list      => @OnAccessPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(
            command_interface => $Node,
            resource_list      => @OnAccessPolicies,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(
            resource_list      => @OnAccessPolicies,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(
            resource_list      => @OnAccessPolicies,
            );

=over

=item Options

Either the resource_list is required or the NACL OnAccessPolicy Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the luns in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Vserver Security Trace Filter
Deletion Arguments should NOT be passed in.

=item C<< NACL Vscan::OnAccessPolicy Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::STask::VserverOnAccessPolicy->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< policy-name => $Policy_name >>

The name of the on-access-policy to delete.

(Optional)
This arguments is used to delete multiple security trace fiters from the same NACL Vserver
Security Trace Filter Deletion Arguments.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Vscan::OnAccessPolicy(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "On_Access_Policy";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a on-access policy(s).
# Details:    Makes the NACL::STask::VserverVscanOnAccessPolicy->create() call. Also works
#             around some common non-fatal errors that can be return from
#             creation.
# Inputs:     NACL::STask::VserverVscanOnAccessPolicy->create() arguments
# Outputs:    Returns whether the attribute was set.
#             of the OnAccessPolicy was successful or a hash with success
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    my $revert_to_default
        = exists $args{revert_to_default}
        ? delete $args{revert_to_default}
        : '1';
    my $duplicated = FALSE;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Create Array Refs for list parameters
    $args{filters} = [split(/,/, $args{filters})] if exists $args{filters};
    $args{'paths-to-exclude'} = [split(/,/, $args{'paths-to-exclude'})]
        if exists $args{'paths-to-exclude'};
    $args{'file-ext-to-exclude'} = [split(/,/, $args{'file-ext-to-exclude'})]
        if exists $args{'file-ext-to-exclude'};

    my $was_created;

    # Create On-Access-Policy
    NACL::STask::VserverVscanOnAccessPolicy->create(
        %args,
        nacltask_if_exists => $if_exists,
        _was_created       => \$was_created
    );

    if (!$was_created and $if_exists eq 'reuse') {
        $duplicated = TRUE;
        $Log->comment("Vserver Vscan On-Access-Policy "
                . $args{'policy-name'}
                . " already exists on "
                . $args{vserver}
                . ". It will be reused.");
    }

    my $default_enabled = 1;

    do {
        # Enable On-Access-Policy
        try {
            NACL::STask::VserverVscanOnAccessPolicy->enable(
                command_interface => $args{command_interface},
                vserver           => $args{vserver},
                'policy-name'     => $args{'policy-name'}
            );

            $Log->comment("Vserver Vscan On-Access-Policy "
                    . $args{'policy-name'}
                    . " succesfully enabled for vserver "
                    . $args{vserver});

            # Set flag to 1 to exit loop after the policy is enabled
            if ($default_enabled == 0) {
                $default_enabled = 1;
                next;
            }
        }
        catch NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled
            with {
            my $exception = shift;
            if ($if_exists eq 'reuse') {
                $duplicated = TRUE;
                $Log->comment("Vserver Vscan On-Access-Policy "
                        . $args{'policy-name'}
                        . " already enabled on "
                        . $args{vserver});
            }
            if (   $exception->enabled_policy_isset()
                && $exception->enabled_policy() ne $args{'policy-name'}) {

# If a policy is already enabled on the vserver it must be deleted. There is a default policy that is enabled by default
# on each vserver. This must be disabled first before the new policy being created can be enabled on the vserver.

                if ($args{'policy-name'} ne $exception->enabled_policy()) {

                    # Disable the existing enabled policy
                    NACL::STask::VserverVscanOnAccessPolicy->disable(
                        command_interface => $args{command_interface},
                        vserver           => $args{vserver},
                        'policy-name'     => $exception->enabled_policy()
                    );

                    $default_enabled = 0;
                    $Log->comment("The On-Access-Policy "
                            . $exception->enabled_policy()
                            . " will be disabled, and the On-Access-Policy "
                            . $args{'policy-name'}
                            . " will be enabled");
                }
            }
            };
    } until ($default_enabled);

    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success' => TRUE) : TRUE;
} ## end sub _nacl_create_call

#========================================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a scanner-pool(s).
# Details:    Makes the NACL::STask::VserverVscanOnAccessPolicy->purge() calls.
# Inputs:     NACL::STask::VserverVscanOnAccessPolicy->delete arguments
# Outputs:    Returns whether the delete was successful
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $if_exists;
    my $revert_to_default
        = exists $args{revert_to_default}
        ? delete $args{revert_to_default}
        : '1';
    my $default_policy_name = "default_CIFS";
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Disable On Access Policy
    NACL::STask::VserverVscanOnAccessPolicy->disable(%args,
        nacltask_if_purged => 'pass');

    # Delete On Access Policy
    NACL::STask::VserverVscanOnAccessPolicy->purge(%args,
        nacltask_if_purged => 'pass');

# If the user specified to revert back to the default policy on deletion, do so now
    if ($revert_to_default) {
        NACL::STask::VserverVscanOnAccessPolicy->enable(
            command_interface  => $args{command_interface},
            vserver            => $args{vserver},
            'policy-name'      => $default_policy_name,
            nacltask_if_exists => 'reuse'
        );
    }
    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
} ## end sub _get_dtd_nacl_translate

#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
} ## end sub _get_dtd_nacl_ignore

#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
} ## end sub _get_dtd_nacl_delete

1;
