# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary OnDemandTask Controller Layer Module
## @author dl-nacl-dev
## @status public
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task. It is used to Set Attributes
of ONTAP Luns. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use Tharn qw($Log);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::VserverVscanOnDemandTask;

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    apiset_must       => '',
    apiset_should     => '',
    vserver           => '',
    'method-timeout'  => '',
    'task-name'    => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple on demand task(s) serially
or in parallel and return whether the on demand task(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->create(
            command_interface   => $Node,
            'task-name'         => $task,
            'scan-paths'         => $Scan_path,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->create(
            command_interfac  => $Node,
            resource_list     => @OnDemandTasks,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->create(
            resource_list     => @OnDemandTasks,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->create(
            resource_list      => @OnDemandTasks,
            );

=over

=item Options

Either the resource_list is required or the NACL OnDemandTask Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the on demand tasks in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL OnDemandTask Creation Arguments should
NOT be passed in. (This is an array of EnGenX::Resource() objects not
an EnGenX::Resource_List() currently)

=item C<< NACL OnDemandTask Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::VserverVscanOnDemandTask->set().
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< task-name => $name >>

(Required)
This arguments is to specify the name of the on demand task.

=item C<< scan-paths => $path >>

(Required)
This arguments is used to specify the scan path.

=item C<< report-path => $report_path >>

(Required)
This arguments is used to specify the path to the reports.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the OnDemandTask(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "on_demand_task";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    $Log->exit();
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple on demand tasks serially
or in parallel and return whether the on demand tasks was deleted successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(
            command_interface   => $Node,
            'task-name'         => $task,
            'scan-paths'         => $Scan_path,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(
            command_interface => $Node,
            resource_list      => @OnDemandTasks,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(
            command_interface => $Node,
            resource_list      => @OnDemandTasks,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(
            resource_list      => @OnDemandTasks,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(
            resource_list      => @OnDemandTasks,
            );

=over

=item Options

Either the resource_list is required or the NACL Lun Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the on demand tasks in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Vserver Security Trace Filter
Deletion Arguments should NOT be passed in.

=item C<< NACL OnDemandTask Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::C::VserverVscanOnDemandTask->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< task-name => $name >>

The name of the on demand task to delete.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the task(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "on_demand_task";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);
    $Log->exit();
    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of on demand tasks.
# Details:    Makes the NACL::STask::VserverVscanOnDemandTask->create() call.
# Inputs:     NACL::STask::VserverVscanOnDemandTask->create() arguments
# Outputs:    Returns whether the attribute was set.
#             of the ScannerPool was successful or a hash with success
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    $args{'cross-junction'} = delete $args{'cross-junction'} || "true";
    $args{'directory-recursion'} = delete $args{'directory-recursion'} || "true";
    $args{'report-log-level'} = delete $args{'report-log-level'} || "error";
    $args{'request-timeout'} = delete $args{'request-timeout'} || "5m";
    $args{'scan-priority'} = delete $args{'scan-priority'} || "low";

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Create Array ref for list parameters
    #$args{'scan-path'} = delete $args{'scan-paths'};
    $args{'scan-paths'} = [split(/,/ , $args{'scan-paths'})];

    #$args{'report-path'} = delete $args{'report-directory'};

    # Create Scanner Pool
    NACL::STask::VserverVscanOnDemandTask->create(%args,
                                    nacltask_if_exists => $if_exists,
                                    _was_created => \$was_created);

    if($was_created) {
        $Log->comment( "Vserver Vscan OnDemandTask " . $args{'task-name'} . " succesfully created for vserver " . $args{vserver});
    } else {
        $Log->comment( "Vserver Vscan OnDemandTask " . $args{'task-name'} . " already exists on " . $args{vserver} . ". It will be reused.");
    }

    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE , 'duplicated' => !$was_created) : TRUE;
} ## end sub _nacl_create_call

#========================================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of on demand task(s).
# Details:    Makes the NACL::C::VserverVscanOnDemandTask->delete() calls.
# Inputs:     NACL::STask::VserverVscanOnDemandTask->delete arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    NACL::STask::VserverVscanOnDemandTask->purge(%args,
                                    nacltask_if_purged => 'pass',
                                    _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("Vserver Vscan OnDemandTask " . $args{'task-name'} . " was deleted on vserver " . $args{vserver});
    } else {
        $Log->comment("Vserver Vscan OnDemandTask " . $args{'task-name'} . " was already deleted on vserver " . $args{vserver});
    }

    $Log->exit();
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::OnDemandTask->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::OnDemandTask->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
