#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Fabric;

use strict;
use warnings;

use NATE::BaseException qw(:try);
use NATE::Messenger qw(messenger_global);
use NATE::Log qw(log_global);
use NATE::ProcManager;
use NATE::Events qw(eventloop_drain);
use NATE::ParamSet qw(param_global);

use San::FabricMgr;
use ParamsMgr;

use parent qw(NACL::AppBuilder::EnGenX::Fabric::Config
              NACL::AppBuilder::EnGenX::Fabric::Deconfig
              NACL::AppBuilder::EnGenX::Util);

sub new {
    my ($class, %opts) = @_;
    my $self = \%opts;
    bless($self, $class);

    # init or reuse a rep socket and store the endpoint
    $self->{endpoint} = messenger_global()->init(socket_type => 'rep',
                                                 action      => 'bind',
                                                 reuse       => 1);

    return $self;
}

sub xml_to_list {
    my ($self, %opts) = @_;

    # get the fabrics from the paramset
    my $fabrics = &ParamsMgr::search(
        -search_path => [],
        -key         => "FABRICS",
        -search_type => "",
        -key_filter  => qr/^FABRIC\d*$/,
        paramset => $self->get_paramset()
    );
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw(&ParamsMgr::errmsg());
    }
    my $fabric_group = &ParamsMgr::search(
        -search_path => [],
        -key => "FABRICS",
        paramset => $self->get_paramset()
    );

    # store each fabric as a resource
    foreach my $fabric_elm (values %{$fabrics}) {

        # find all the switches in this fabric and associate this fabric with that switch resource
        my $fabric = "FABRIC";
        if ($fabric_elm->{_INDEX}) {
            $fabric .= $fabric_elm->{_INDEX};
        }
        my $switches = &ParamsMgr::search(
            -search_path => [],
            -exact_key   => 1,
            -key         => "FABRICS.${fabric}",
            -key_filter  => qr/^SWITCH\d*$/,
            paramset     => $self->get_paramset()
        );
        foreach my $switch_elm (values %{$switches}) {
            my ($switch_res) = $self->resource_list()->get_resources(category => "SWITCHES",
                                                                     type => "switches",
                                                                     data => {NAME => $switch_elm->{NAME}});
            # switch resources can be in multiple fabrics
            push(@{$switch_res->{FABRIC_NAMES}}, $fabric_elm->{NAME});
        }

        $self->resource_list()->add_resource(category => "FABRICS",
                                             type     => "fabrics",
                                             data     => $fabric_elm);
    }

    # add the whole fabric group as a resource
    $self->resource_list()->add_resource(category => "FABRICS",
                                         type     => "fabric_groups",
                                         data     => $fabric_group);
    return;
}

sub load_resource {
    my ($self, %opts) = @_;

    # if there are no fabrics, return now
    # foreach fabric in the resource list, create a fabricmgr object for it
    my $fabrics = $self->resource_list()->get_resources(category => "FABRICS",
                                                        type => "fabrics");
    if (!$fabrics or !@{$fabrics}) {
        log_global()->comment("No Fabric definitions were found so resource load will be skipped");
        return;
    }

    #  find all client objects and hypervisor objects that are doing
    # FCP so we can pass them into the fabricmgr object
    my @client_objs = $self->get_objects(type => "client");
    if ($self->error()) {
        log_global()->comment("No client objects found for fabric configuration");
        $self->clear_error();
        @client_objs = ();
    }

    my @hypervisor_objs = $self->get_objects(type => "hypervisor");
    if ($self->error()) {
        log_global()->comment("No hypervisor objects found for fabric configuration");
        $self->clear_error();
        @hypervisor_objs = ();
    }

    # filter out the client objects that are not doing fcp
    # or that have requested to not perform zoning
    my (@fcp_host_objs) = grep {
        my $obj = $_;
        if ($obj->get_protocol() !~ /fcp/i or $obj->{hypervisor_type} ne "none") {
            # host is not doing fcp or is a virtual machine not capable of doing direct
            # SAN
            0;
        } else {
            if (!$obj->{fcp}->{CONFIG_FCP_PERFORM_ZONING}) {
                # host is doing FCP and wants to be zoned
                log_global()->comment("Was requested to skip FC zoning for client " . $obj->hostname());
                0;
            } else {
                1;
            }
        }
    } @client_objs, @hypervisor_objs;

    # we need clients to do zoning
    if (not scalar(@fcp_host_objs)) {
        NATE::BaseException->throw("No FC clients/hypervisors were found that can be used for FC zoning");
    }

    # get all the clusters and vservers that we will be working with
    my @nacl_clusters = $self->get_objects('type' => "cluster",
                                           'obj_class' => 'nacl');
    if ($self->error()) {
        NATE::BaseException->throw("Failed to get nacl cluster objects, Reason: " . $self->errmsg());
    }
    my @vservers = $self->get_vserver_names(-type => "regular");
    if ($self->error()) {
        NATE::BaseException->throw("Failed to get the vserver names, Reason: " . $self->errmsg());
    }

    # Get the client vserver map
    my ($client_vserver_map)  = $self->{parent}->get_vserver_client_relationship_hash();
    if ($self->{parent}->error) {
        NATE::BaseException->throw("Unable to build the client vserver relationship hash, Reason: " . $self->{parent}->errmsg());
    }

    # foreach fabric in the resource list, create a fabricmgr object for it
    foreach my $fabric (@{$fabrics}) {
        my $fabric_name     = $fabric->get_field("NAME");
        my $restrict_zoning = undef;
        if ($fabric->field_defined("RESTRICT_ZONING_BASED_ON_EXISTING_IGROUPS")) {
            $restrict_zoning = $fabric->get_field("RESTRICT_ZONING_BASED_ON_EXISTING_IGROUPS");
        }

        # get all switches associated to this fabric
        my $switches = $self->resource_list()->get_resources(category => "SWITCHES",
                                                             type => "switches");

        # make a lookup table so I can find switch resources via their name
        my $switch_resources_by_name = {};
        foreach (@{$switches}) {
            $switch_resources_by_name->{$_->get_field("NAME")} = $_;
        }

        # filter out all switches that dont belong to the current fabric
        @{$switches} = grep {
            my $s = $_;
            my $fabs = $s->field_defined("FABRIC_NAMES") ? $s->get_field("FABRIC_NAMES") : undef;
            if (!$fabs) {
                0;
            } elsif (grep(/^$fabric_name$/, @{$fabs})) {
                1;
            } else {
                0;
            }
        } @{$switches};
        if (!scalar(@{$switches})) {
            NATE::BaseException->throw("There are no switches associated to fabric '$fabric_name'");
        }

        # find all the switch elements in the fabric resource
        my $fab_switches = {};
        if ($fabric->{SWITCHES}) {
            # xgen way of doing things with fabric switch members is with a SWITCHES array of names
            foreach my $s (@{$fabric->{SWITCHES}}) {
                $fab_switches->{$s} = $switch_resources_by_name->{$s};
            }
        } else {
            # old deprecated way with XML based resource lists
            # that are using the SWITCH\d* key style on the fabric resource
            foreach my $key (keys %{$fabric}) {
                if ($key =~ /^SWITCH\d*$/) {
                    $fab_switches->{$fabric->{$key}->{NAME}} = $fabric->{$key};
                }
            }
        }

        if (scalar(keys %{$fab_switches}) ne scalar(@{$switches})) {
            NATE::BaseException->throw("There is a mismatch of switches in the fabric and switches registered as resources against this fabric");
        }

        # default some stuff for the fabric map which will be passed into fabricmgr
        my $fabric_map = {};
        $fabric_map->{$fabric_name} = {
            zones_per_node_per_init => $fabric->get_field("ZONES_PER_NODE_PER_INIT") // 1,
            master_switch => $fabric->get_field("MASTER_SWITCH") // "",
            prefix => $fabric->get_field("PREFIX") // "nate",
        };

        # just in case, trim the right most underscore from the prefix
        $fabric_map->{$fabric_name}->{prefix} =~ s/_+$//;

        # intialize the fabric_map for all the switches in this fabric
        my @switch_names = ();
        my @fields_to_copy = ("admin_domain", "virtual_fabric", "fabric_id");
        foreach my $switch (@{$switches}) {
            my $switch_name = $switch->get_field("NAME");
            push(@switch_names, $switch_name);

            foreach my $field (@fields_to_copy) {
                # the order that we will look up the values of the fields will be:
                # 1) the switch tags inside the fabric elements
                # 2) the switch tags outside the fabric elements
                # 3) the fabric element itself
                if (exists $fab_switches->{$switch_name}->{uc($field)} and
                    length $fab_switches->{$switch_name}->{uc($field)}) {
                    # switch tags inside the fabric element
                    $fabric_map->{$fabric_name}->{switches}->{$switch_name}->{$field} =
                        $fab_switches->{$switch_name}->{uc($field)};
                } elsif ($switch->field_defined(uc($field)) and length($switch->get_field(uc($field)))) {
                    # switch tags outside the fabric element
                    $fabric_map->{$fabric_name}->{switches}->{$switch_name}->{$field} =
                        $switch->get_field(uc($field));
                } else {
                    # the fabric element
                    $fabric_map->{$fabric_name}->{switches}->{$switch_name}->{$field} =
                        $fabric->get_field(uc($field)) // "";
                }
            }
        }

        my @switch_objs = $self->get_objects(-type => 'switch',
                                             -names => \@switch_names);
        if ($self->error()) {
            NATE::BaseException->throw("Could not find switch objects, reason " . $self->errmsg());
        }

        if (!scalar(@switch_objs)) {
            NATE::BaseException->throw("There are no switch objects to zone for this fabric");
        }

        # Create the FabricMgr object
        my $fm_obj = San::FabricMgr->new(
            -parent            => $self,
            -client_objs       => \@fcp_host_objs,
            -nacl_cluster_objs => \@nacl_clusters,
            -vservers          => \@vservers,
            -switch_objs       => \@switch_objs,
            -fabric            => $fabric_name,
            -vserver_map       => $client_vserver_map,
            -fabric_map        => $fabric_map,
            -protocols         => ["fcp"],
            -restrict_zoning_based_on_existing_igroups => $restrict_zoning
        );

        # Save the object for future use
        $self->_store_object(type => "fabric",
                             name => $fabric_name,
                             obj  => $fm_obj);
    }
    # mark that we are loaded
    $self->{loaded} = 1;
}

sub _reconnect_all_clients {
    my ($self, %args) = @_;

    my $fabric = delete($args{fabric}) //
        NATE::Exceptions::Argument->throw("Missing required argument 'fabric'");

    # Get fabric object
    my ($fabric_obj) = $self->get_objects(-type => "fabric", -names => [$fabric]);
    if ($self->error()) {
        NATE::BaseException->throw("Could not find fabric object for $fabric");
    }

    # Get all of the client objects off of the fabric and reconnect them.
    my $client_objs = $fabric_obj->get_client_hash();
    if ($fabric_obj->error) {
        NATE::BaseException->throw("Failed to retrieve the client hash for fabric '$fabric' " .
                                   $fabric_obj->errmsg());
    }

    # Iterate over each client objects and reconnect them.
    foreach my $client (keys %$client_objs) {
        my $client_obj = $client_objs->{$client};
        unless (ref($client_obj) =~ /sanclient/i) {
            NATE::BaseException->throw("Invalid data type value for client '$client'");
        }

        $client_obj->reconnect(-disconnect_existing_conn => $SanMgr::FALSE);
        if ($client_obj->error()) {
            NATE::BaseException->throw("Unable to reconnect client '$client' : " . $client_obj->errmsg);
        }
    }

    return;
}

sub _display_missing_wwpns {
    my ($self, %opts) = @_;

    my $fabric_count = delete($opts{fabric_count}) //
        NATE::Exceptions::Argument->throw("Missing required parameter 'fabric_count'");

    # store all missing wwpn messages that we encountered from all fabrics into a single hash.
    # we will only keep the missing wwpn messages that exist in all fabrics
    my %missing_wwpns = ();

    foreach my $fabric_key (keys %{$self->{missing_wwpns}}) {
        foreach my $missing_wwpn (@{$self->{missing_wwpns}->{$fabric_key}}) {
            $missing_wwpns{$missing_wwpn->{msg}}++;
        }
    }

    foreach my $missing_wwpn (keys %missing_wwpns) {
        # if this wwpn was missing in all of the fabrics, then it really was missing
        # remove it if we find it in at least one of the places
        if ($missing_wwpns{$missing_wwpn} < $fabric_count) {
            # we found it in at least one place, so take it out of the hash
            delete($missing_wwpns{$missing_wwpn});
        }
    }

    my $jobid  = param_global()->get('JOBID');
    my $logdir = param_global()->get('LOGDIR');
    my $runid  = param_global()->get('RUNID');
    $runid .= "/" . $self->{last_runid};

    # replace everything after the job id because it is included in runid
    $logdir =~ s/$jobid\S*/$jobid/;

    # Initiator_Aliases_Not_Created logging
    if (scalar (keys %missing_wwpns)) {
        log_global()->warn("There were " . scalar(keys %missing_wwpns) . " Initiator/Target WWPNs that we " .
                           "could not find in any of the provided fabrics.");
        log_global()->redirect(path=>"$logdir/$runid/WWPNs_Not_Found_in_Any_Fabric.log");
        log_global()->comment("This contains all the Initiator/Target WWPNs that were not " .
                              "found in any nameserver of any of the fabrics: " .
                              $self->dumper(\%missing_wwpns));
        log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
        log_global()->reset_path();
    }

    return;
}

sub _create_summary {
    my ($self) = @_;

    my %deleted_hash = ();
    my %created_hash = ();
    my %created_summary = ();

    foreach my $fabric_key (keys %{$self->{summary_info}}) {
        # Sum initiator aliases
        $created_summary{initiator_aliases_created} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{initiator_aliases_created};
        $created_summary{initiator_aliases_reused} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{initiator_aliases_reused};

        # Sum target aliases
        $created_summary{target_aliases_created} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{target_aliases_created};
        $created_summary{target_aliases_reused} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{target_aliases_reused};

        # Sum zones
        $created_summary{zones_created} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{zones_created};
        $created_summary{zones_reused} +=
            $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{totals_info}->{zones_reused};

        # Add the deleted targets to the merged hash for deleted_wwpns_info
        foreach my $wwpn_key ( keys %{$self->{summary_info}->{$fabric_key}->{deleted_wwpns_info_hash}->{vservers}}) {
            $deleted_hash{vservers}{$wwpn_key} =
                $self->{summary_info}->{$fabric_key}->{deleted_wwpns_info_hash}->{vservers}->{$wwpn_key}
                unless exists ($deleted_hash{vservers}{$wwpn_key});
            $deleted_hash{vservers}{$wwpn_key}{count}++;
        }
        # Add the deleted initiators to the merged hash for deleted_wwpns_info
        foreach my $wwpn_key ( keys %{$self->{summary_info}->{$fabric_key}->{deleted_wwpns_info_hash}->{sanclients}}) {
            $deleted_hash{clients}{$wwpn_key} =
                $self->{summary_info}->{$fabric_key}->{deleted_wwpns_info_hash}->{sanclients}->{$wwpn_key}
                unless exists ($deleted_hash{clients}{$wwpn_key});
            $deleted_hash{clients}{$wwpn_key}{count}++;
        }
        # Add the created targets info to the merged hash for aliases and zoned created
        foreach my $cluster ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{vservers}}) {
            foreach my $vserver ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{vservers}->{$cluster}}) {
                foreach my $alias (@{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{new}}) {
                    push(@{$created_summary{vservers}{$cluster}{$vserver}{$fabric_key}{new}},$alias);
                    $created_hash{vservers}{$alias->{wwpn}} = $alias
                        unless exists ($created_hash{vservers}{$alias->{wwpn}});
                    $created_hash{vservers}{$alias->{wwpn}}{count}++;
                }
                foreach my $alias (@{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{reused}}) {
                    push(@{$created_summary{vservers}{$cluster}{$vserver}{$fabric_key}{reused}},$alias);
                    $created_hash{vservers}{$alias->{wwpn}} = $alias
                        unless exists ($created_hash{vservers}{$alias->{wwpn}});
                    $created_hash{vservers}{$alias->{wwpn}}{count}++;
                }
            }
        }
        # Add the created initiator info to the merged hash for aliases and zoned created
        foreach my $client ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{clients}}) {
            foreach my $alias (@{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{clients}->{$client}->{$fabric_key}->{new}}) {
                push(@{$created_summary{clients}{$client}{$fabric_key}{new}},$alias);
                $created_hash{clients}{$alias->{wwpn}} = $alias
                    unless exists ($created_hash{clients}{$alias->{wwpn}});
                $created_hash{clients}{$alias->{wwpn}}{count}++;
            }
            foreach my $alias (@{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{aliases}->{clients}->{$client}->{$fabric_key}->{reused}}) {
                push(@{$created_summary{clients}{$client}{$fabric_key}{reused}},$alias);
                $created_hash{clients}{$alias->{wwpn}} = $alias
                    unless exists ($created_hash{clients}{$alias->{wwpn}});
                $created_hash{clients}{$alias->{wwpn}}{count}++;
            }
        }
        # Add the created target zones info to the merged hash for aliases and zoned created
        foreach my $cluster ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}}) {
            foreach my $vserver ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}->{$cluster}}) {
                foreach my $zone (keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{new}}) {
                    $created_summary{zones}{vservers}{$cluster}{$vserver}{$fabric_key}{new}{$zone} =
                        $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{new}->{$zone};
                }
                foreach my $zone (keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{reused}}) {
                    $created_summary{zones}{vservers}{$cluster}{$vserver}{$fabric_key}{reused}{$zone} =
                        $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{vservers}->{$cluster}->{$vserver}->{$fabric_key}->{reused}->{$zone};
                }
            }
        }
        # Add the created initiator zones info to the merged hash for aliases and zoned created
        foreach my $client ( keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{clients}}) {
            foreach my $zone (keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{clients}->{$client}->{$fabric_key}->{new}}) {
                $created_summary{zones}{clients}{$client}{$fabric_key}{new}{$zone} =
                    $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{clients}->{$client}->{$fabric_key}->{new}->{$zone};
            }
            foreach my $zone (keys %{$self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{clients}->{$client}->{$fabric_key}->{reused}}) {
                $created_summary{zones}{clients}{$client}{$fabric_key}{reused}{$zone} =
                    $self->{summary_info}->{$fabric_key}->{aliases_and_zones_info_hash}->{zones}->{clients}->{$client}->{$fabric_key}->{reused}->{$zone};
            }
        }
    }
    my $fabric_count = scalar (keys %{$self->{summary_info}});
    foreach my $deleted_wwpn (keys %{$deleted_hash{vservers}}) {
        if ($deleted_hash{vservers}{$deleted_wwpn}{count} >= $fabric_count) {
            next;
        }
        foreach my $created_wwpn (keys %{$deleted_hash{vservers}}) {
            if ($created_wwpn eq $deleted_wwpn) {
                delete $deleted_hash{vservers}{$deleted_wwpn};
            }
        }
    }
    foreach my $deleted_wwpn (keys %{$deleted_hash{clients}}) {
        if ($deleted_hash{clients}{$deleted_wwpn}{count} >= $fabric_count) {
            next;
        }
        foreach my $created_wwpn (keys %{$deleted_hash{clients}}) {
            if ($created_wwpn eq $deleted_wwpn) {
                delete $deleted_hash{clients}{$deleted_wwpn};
            }
        }
    }

    my $jobid  = param_global()->get('JOBID');
    my $logdir = param_global()->get('LOGDIR');
    # replace everything after the job id because it is included in runid
    $logdir =~ s/$jobid\S*/$jobid/;
    my $runid  = param_global()->get('RUNID');
    $runid .= "/" . $self->{last_runid};

    # Initiator_Aliases_Created logging
    log_global()->redirect(path=>"$logdir/$runid/Initiator_Aliases_Created.log");
    log_global()->comment($created_summary{initiator_aliases_created}." new initiator aliases");
    log_global()->comment($created_summary{initiator_aliases_reused}." reused initiator aliases");
    log_global()->comment($created_summary{initiator_aliases_created} +
                  $created_summary{initiator_aliases_reused}." total initiator aliases");
    log_global()->comment("This contains all the initiator aliases create by " .
                          "Fabric_Config.\n" .
                          $self->dumper($created_summary{clients}));
    #Logging the following so that the log will PASS
    log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
    # Direct log_global() calls to write back to the original log file
    log_global()->reset_path();

    # Target_Aliases_Created logging
    log_global()->redirect(path=>"$logdir/$runid/Target_Aliases_Created.log");
    log_global()->comment($created_summary{target_aliases_created}." new target aliases");
    log_global()->comment($created_summary{target_aliases_reused}." reused target aliases");
    log_global()->comment($created_summary{target_aliases_created} +
                  $created_summary{target_aliases_reused}." total target aliases");
    log_global()->comment("This contains all the target aliases create by " .
                              "Fabric_Config.\n" .
                                  $self->dumper($created_summary{vservers}));
    #Logging the following so that the log will PASS
    log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
    # Direct log_global() calls to write back to the original log file
    log_global()->reset_path();

    # Zones_Created logging
    log_global()->redirect(path=>"$logdir/$runid/Zones_Created.log");
    log_global()->comment($created_summary{zones_created}." new zones");
    log_global()->comment($created_summary{zones_reused}." reused zones");
    log_global()->comment($created_summary{zones_created} +
                  $created_summary{zones_reused}." total zones");
    log_global()->comment("This contains all the zones create by " .
                              "Fabric_Config.\n" .
                                  $self->dumper($created_summary{zones}));
    #Logging the following so that the log will PASS
    log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
    # Direct log_global() calls to write back to the original log file
    log_global()->reset_path();

    # Target_Aliases_Not_Created logging
    if(scalar (keys %{$deleted_hash{vservers}})) {
        log_global()->warn("There are Target WWPNs that were never used to create aliases");
        log_global()->redirect(path=>"$logdir/$runid/Target_Aliases_Not_Created.log");
        log_global()->comment("This contains all the Target wwpns not used in " .
                                  "Fabric_Config.\n" .
                                      $self->dumper($deleted_hash{vservers}));
        #Logging the following so that the log will PASS
        log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
        # Direct log_global() calls to write back to the original log file
        log_global()->reset_path();
    }

    # Initiator_Aliases_Not_Created logging
    if(scalar (keys %{$deleted_hash{clients}})){
        log_global()->warn("There are Initiator WWPNs that were never used to create aliases");
        log_global()->redirect(path=>"$logdir/$runid/Initiator_Aliases_Not_Created.log");
        log_global()->comment("This contains all the Initiator wwpns not used in " .
                      "Fabric_Config.\n" .
                      $self->dumper($deleted_hash{clients}));
        #Logging the following so that the log will PASS
        log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
        # Direct log_global() calls to write back to the original log file
        log_global()->reset_path();
    }

    # global summary report
    log_global()->redirect(path=>"$logdir/$runid/Summary_Report.log");
    log_global()->comment("There are $fabric_count fabrics");
    log_global()->comment("LOGDIR:" . $logdir);
    log_global()->comment("RUNID:" . $runid);
    log_global()->comment("JOBID:" . $jobid);
    log_global()->comment("Deleted Hash:\n" .
                          "Description: This contains data on all wwpns " .
                          "not used by all the fabrics.\n" .
                          $self->dumper(\%deleted_hash));
    log_global()->comment("Created Summary:\n" .
                          "Description: This contains data on all wwpns " .
                          "used by all the fabrics.\n" .
                          $self->dumper(\%created_summary));
    log_global()->comment("END OF TEST \/ RESULTS SUMMARY\nNo results reported\n");
    log_global()->reset_path();

    return;
}

sub dump_all_hashes {
    my ($self, %args) = @_;

    my $fabric_name = delete($args{fabric})    //
        NATE::Exceptions::Argument->throw("Missing required argument 'fabric'");
    my $subtested   = delete($args{subtested}) // 0;

    my $p = NATE::Process->new(codespec => sub { $self->_dump_all_hashes_subtest(fabric => $fabric_name) },
                               runid => $fabric_name . "_dump_hashes");
    $p->start();
    $p->wait();
    $p->destroy();

    return;
}

sub _dump_all_hashes_subtest {
    my ($self, %args) = @_;

    my $fabric_name = delete($args{fabric})    //
        NATE::Exceptions::Argument->throw("Missing required argument 'fabric'");

    # Get fabric object
    my ($fabric_obj) = $self->get_objects(-type => "fabric", -names => [$fabric_name]);
    if ($self->error()) {
        NATE::BaseException->throw("Failed to get fabric ($fabric_name) object");
    }

    $fabric_obj->dump_hashes();
    if ($fabric_obj->error) {
        NATE::BaseException->throw("Failed to dump the hashes for fabric ($fabric_name) " . $fabric_obj->errmsg());
    }

    return;
}

1;
