#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Fabric::Deconfig;

use strict;
use warnings;

use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);
use NATE::Process;
use NATE::BaseException qw(:try);
use Data::Dump qw(dump);
use Data::Dumper;

sub deconfig {
    my ($self, %args) = @_;

    my $run_id_prefix = delete($args{run_id_prefix}) // "";

    # check to see if deconfig is disabled for all fabrics
    my $fabric_groups = $self->resource_list()->get_resources(category => "FABRICS",
                                                              type => "fabric_groups",
                                                              data => {DECONFIG => 1});
    if (!scalar(@{$fabric_groups})) {
        log_global()->comment("No Fabrics to deconfig");
        return;
    }

    # lazily load the fabric resources only once, and only if
    # fabric deconfig is enabled at the global level
    if (!$self->{loaded}) {
        $self->load_resource();
    }

    # deconfigure all fabric resources that have fabric config enabled
    my $fabrics = $self->resource_list()->get_resources(category => "FABRICS",
                                                        type => "fabrics",
                                                        data => {DECONFIG => 1});
    if (!scalar(@{$fabrics})) {
        log_global()->comment("Skipping fabric deconfig (disabled on each fabric)");
        return;
    }

    my @procs = ();
    foreach my $fabric (@{$fabrics}) {
        my $fabric_name = $fabric->get_field("NAME");
        my $proc = NATE::Process->new(runid => $fabric_name,
                                      codespec => sub {
                                          try {
                                              my $data = $self->_deconfig_single(fabric => $fabric);
                                              $self->_deconfig_complete(name => $fabric_name,
                                                                        %{$data});
                                          } otherwise {
                                              my $e = shift;
                                              my $msg = Dumper($e);
                                              $msg = $e->message_string() if $e->isa("NATE::Result");
                                              $msg = $e->text() if $e->isa("NATE::BaseException");
                                              $self->_deconfig_failure(msg  => $msg,
                                                                       name => $fabric_name);
                                              $e->throw() if $e->isa("NATE::BaseException");
                                          };
                                      },
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result);
        push(@procs, $proc);
    }

    # set up the Event listener and callback methods
    my $success_filter = sub {
        shift->type() eq "NACL/AppBuilder/EnGenX/Fabric/Deconfig/Complete";
    };
    my $success_callback = sub {
        my ($msg, $listener) = @_;

        # parse out the message information
        my ($fabric_name) = $msg->message()->{name};

        $self->{missing_wwpns}->{$fabric_name} = $msg->message()->{missing_wwpns};
        $listener->{completed}++;
    };
    my $success_listener = {filter => $success_filter,
                            callback => $success_callback,
                            completed => 0};

    # build the fail listener
    my @failed_fabrics = ();
    my $fail_filter = sub {
        shift->type() eq 'NACL/AppBuilder/EnGenX/Fabric/Deconfig/Failure';
    };
    my $fail_callback = sub {
        my ($msg, $listener) = @_;

        # parse out the message information
        my ($fabric_name) = $msg->message()->{name};
        push(@failed_fabrics, $fabric_name);
        $listener->{completed}++;
    };
    my $fail_listener = {filter => $fail_filter,
                         callback => $fail_callback,
                         completed => 0};

    # start up all fabric config routines
    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => $run_id_prefix . "Fabric-deconfig");

    # display any warnings for any WWPNs that were not found during fabric deconfig
    $self->_display_missing_wwpns(fabric_count => scalar(@{$fabrics}));

    if (scalar(@failed_fabrics)) {
        NATE::BaseException->throw("The following fabrics failed to deconfigure properly (@failed_fabrics)");
    }

    return;
}

sub _deconfig_single {
    my ($self, %args) = @_;

    my $fabric = delete($args{fabric});
    if (!$fabric) {
        my $err = "Missing required parameter 'fabric'";
        NATE::BaseException->throw($err);
    }

    my $fabric_name = $fabric->get_field("NAME");

    # get the fabric object
    my ($fabric_obj) = $self->get_objects(-type => "fabric", -names => [$fabric_name]);
    if ($self->error()) {
        my $err = "Failed to get fabric object for fabric '$fabric_name'";
        NATE::BaseException->throw($err);
    }

    # Grab the switch objects and reconnect them
    my $switches_hash = $fabric_obj->get_switch_hash();
    if ($fabric_obj->error()) {
        my $err = "Failed to get switch hash for fabric ($fabric_name) " . $fabric_obj->errmsg();
        NATE::BaseException->throw($err);
    }

    foreach my $switch (sort keys %$switches_hash) {
        # Reconnect the switch object in case we are using the same switch
        # in multiple fabrics
        my $obj = $switches_hash->{$switch};
        $obj->reconnect(-disconnect_existing_conn => $SanUtil::FALSE);
        if ($obj->error()) {
            my $err = "Unable to reconnect switch '$switch' " .  $obj->errmsg();
            NATE::BaseException->throw($err);
        }
    }

    # reconnect to all nacl objs
    my @nodes = $self->get_objects(type => 'node', 'obj_class' => 'nacl');
    foreach (@nodes) {
        $_->refresh_command_interface();
    }

    # Build out all of the hashes. Normally you don't have to do this as they are
    # build on demand, however, we want to subtest out the dump of the hashes and
    # we don't want the dump to go throught the build out phase.
    # Dump out the hashes
    $fabric_obj->build_all_hashes();
    if ($fabric_obj->error) {
        # Check to see what kind of error we received.  If we receved one because
        # we don't have enough resources, then just return TRUE and skip
        # deconfig.
        my @errmsg = $fabric_obj->errmsg();
        if (grep(/Not enough targets on node/, @errmsg) > 0
            or grep(/Invalid number of resources for fabric/, @errmsg) > 0) {
            my $errmsg = join("\n", @errmsg);
            log_global()->comment("Skipping fabric deconfig due to lack of resources:\n" . $errmsg);
            return {missing_wwpns => []};
        } else {
            my $err = "Failed to build the hashes for fabric ($fabric_name) @errmsg";
            NATE::BaseException->throw($err);
        }
    }

    # Subtest off the dump of the fabric hashes
    $self->dump_all_hashes(fabric => $fabric_name);

    # Run the "delete" routines within the fabric library
    # Delete the zones
    $fabric_obj->delete_zones();
    if ($fabric_obj->error) {
        my $err = "Failed to delete the zones for fabric ($fabric_name) " . $fabric_obj->errmsg();
        NATE::BaseException->throw($err);
    }

    # Delete the aliases
    $fabric_obj->delete_aliases();
    if ($fabric_obj->error) {
        my $err = "Failed to delete the aliases for fabric ($fabric_name) " . $fabric_obj->errmsg();
        NATE::BaseException->throw($err);
    }

    # Activate the fabric changes
    $fabric_obj->activate_fabric_changes();
    if ($fabric_obj->error) {
        my $err = "Failed to activate the fabric changes for fabric ($fabric_name) " . $fabric_obj->errmsg();
        NATE::BaseException->throw($err);
    }

    my $data = {
        missing_wwpns => $fabric_obj->get_missing_wwpns_macs()
    };



    return $data;
}

sub _deconfig_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Fabric/Deconfig/Failure",
                            message  => \%data);
}

sub _deconfig_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/Fabric/Deconfig/Complete",
                            message  => \%data);
}

1;
