#
# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on fibrebridge
## @author kaaron@netapp.com
## @status private
## @pod here

package NACL::AppBuilder::EnGenX::Fibrebridge;

# Compiler directives.
use strict;

# Module import.
use Data::Dump;

use NATE::Process;
use NATE::STAF::Peer;
use NATE::STAF qw(staf_global);
use NATE::Util::Sockets;
use Sys::Hostname;
use Armadillo;

use parent qw(NACL::AppBuilder::EnGenX::Util);

###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:    _new
# Objective: Create new resource object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new {
    my ($proto, %args) = @_;

    my $self = {};

    my $class = ref($proto) || $proto;
    bless ($self, $class);

    # allow for error stack build ups
    $self->{error_dont_overwrite_errors} = 1;
    $self->{error_stack_style} = "normal";

    &ParamsMgr::process_args(%args);
    my $parent = &ParamsMgr::get_arg(-arg => "parent", -required => 1, -default => "");
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw("Failed to get parent from ParamsMgr.\n" . &ParamsMgr::get_error());
    }

    # Save parent
    $self->{parent} = $parent;

    # Get the set of environment parameters from ParamsMgr
    my $environment = &ParamsMgr::search(search_path => [], key => "ENVIRONMENT");
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw("Couldn't get environment parameters from ParamsMgr.\n" . &ParamsMgr::get_error());
    }

    return $self;
}

###############################################################################
# Method:    setup
# Objective: Launch subtest for all fibrebridges to perform setup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub setup {
    my ($self, %args) = @_;

    # No setup for Fibrebridges

    return $NACL::AppBuilder::EnGenX::TRUE;
}

###############################################################################
# Method:    config
# Objective: Launch subtest for each fibrebridge to perform config
# Details:   See POD documentation at the end of this file.
###############################################################################
sub config {
    my ($self) = @_;

    # No setup for Fibrebridges

    return $NACL::AppBuilder::EnGenX::TRUE;
}

###############################################################################
# Method:    cleanup
# Objective: Launch subtest for each fibrebridge to perform cleanup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cleanup {
    my ($self) = @_;

    # No cleanup for Fibrebridges

    return $NACL::AppBuilder::EnGenX::TRUE;
}

###############################################################################
# Method:    deconfig
# Objective: Launch subtest for each fibrebridge to perform deconfig
# Details:   See POD documentation at the end of this file.
###############################################################################
sub deconfig {
    my ($self) = @_;

    # No decondfig for Fibrebridges

    return $NACL::AppBuilder::EnGenX::TRUE;
}

###############################################################################
# Method:    validate
# Objective: Launch subtest for each fibrebridge to perform validation
# Details:   See POD documentation at the end of this file.
###############################################################################
sub validate {
    my ($self) = @_;

    # TODO

    return $NACL::AppBuilder::EnGenX::TRUE;
}

###############################################################################
# Method:    load_resource
# Objective: Gather list of fibrebridges and create objects for each one
# Details:   See POD documentation at the end of this file.
###############################################################################
sub load_resource {
    my ($self, %args) = @_;

    my $staf = staf_global();

    # Get list of fibrebridges
    my $fibrebridges = &ParamsMgr::search(-search_path => [],
                                          -key => "FIBREBRIDGES",
                                          -key_filter => qr/^FIBREBRIDGE\d*$/);
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw("Failed to get fibrebridges from ParamsMgr.\n" . &ParamsMgr::get_error());
    }

    # make sure there are no duplicate fibrebridges
    my %dup_hash;
    foreach my $fibrebridge (keys(%{$fibrebridges})) {
        $dup_hash{$fibrebridges->{$fibrebridge}->{NAME}}++;
        if ($dup_hash{$fibrebridges->{$fibrebridge}->{NAME}} > 1) {
            NATE::BaseException->throw("Detected duplicate fibrebridge entries for the fibrebridge " .
                                           $fibrebridges->{$fibrebridge}->{NAME});
        }
    }

    # Go through the list and create an object for each one
    my @procs;
    foreach my $fibrebridge (keys(%{$fibrebridges})) {
        # Spawn a new process to load this fibrebridge
        my $proc = NATE::Process->new(codespec => '{ NACL::AppBuilder::EnGenX::Fibrebridge::_load_fibrebridge(); }',
                                      runid => "fibrebridge__" . $fibrebridges->{$fibrebridge}->{NAME},
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result,
                                      params => { "fibrebridges_ds" => $fibrebridges,
                                                  "fibrebridge" => $fibrebridge,
                                                  "debug_level" => $self->{debug_level},
                                                  "staf_parent_handle" => $staf->handle(),
                                                  "staf_parent_endpoint" => NATE::Util::Sockets::get_address(Sys::Hostname::hostname())});
        push(@procs, $proc);
    }

    # set up the Event listener and callback methods
    my $filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /Fibrebridge\//);
    };
    my $callback = sub {
        my ($msg, $listener) = @_;

        my ($atto_name) = ($msg->{type} =~ /Fibrebridge\/(\S+?)\/Load_Complete/);
        my $atto_obj = $listener->{atto_obj};

        # rebuild the Armadillo object
        my $obj = eval($msg->{message}->{obj_serialized});
        $obj = Armadillo->new(-obj_template => $obj);
        if ($obj->error()) {
            $atto_obj->{parent}->{listeners_complete_count}++;
            NATE::BaseException->throw("Failed to instantiate Armadillo object from template" . $obj->errmsg());
        }

        # store the object on the engenx resource hash
        $atto_obj->{parent}->{_objects}->{fibrebridge}->{$obj->get_hostname()} = $obj;

        # validate the md5 for the obj
        if (not $atto_obj->_validate_obj_md5_hash(obj => $obj,
                                                  obj_md5_hex_before => $msg->{message}->{obj_md5_hex})) {
            $atto_obj->{parent}->{listeners_complete_count}++;
            return $NACL::AppBuilder::EnGenX::FALSE;
        }

        $atto_obj->{parent}->{listeners_complete_count}++;
        return $NACL::AppBuilder::EnGenX::TRUE;
    };
    my $listener = {filter => $filter, callback => $callback, atto_obj => $self};

    # build the fail listener
    my $fail_filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /Fibrebridge\/\S+\/Load_Failed/);
    };
    my $fail_callback = sub {
        my ($msg, $flistener) = @_;

        my $atto_obj = $flistener->{atto_obj};

        # parse out the message information
        my ($atto_name) = ($msg->{type} =~ /Fibrebridge\/(\S+?)\/Load_Failed/);
        $atto_obj->{parent}->{listeners_complete_count}++;
        NATE::BaseException->throw("The Fibrebridge object for $atto_name failed to instantiate.");
    };

    my $fail_listener = {filter => $fail_filter, callback => $fail_callback, atto_obj => $self};

    return ($listener, $fail_listener, @procs);
}

sub _load_fibrebridge {
    # fetch params passed into the process
    my $global_params = NATE::ParamSet->get_global();
    my $fibrebridges = $global_params->get("fibrebridges_ds");
    my $fibrebridge = $global_params->get("fibrebridge");
    my $debug_level = $global_params->get("debug_level");
    my $staf_parent_handle = $global_params->get("staf_parent_handle");
    my $staf_parent_endpoint = $global_params->get("staf_parent_endpoint");

    my $parent = NATE::STAF::Peer->new(handle => $staf_parent_handle,
                                       endpoint => $staf_parent_endpoint);
    # Create fibrebridge object.
    my $atto_obj = Armadillo->new(-bridge    => $fibrebridges->{$fibrebridge}->{NAME},
                                  -conn_type => $fibrebridges->{$fibrebridge}->{CONN_TYPE},
                            );
    if ($atto_obj->error()) {
        $parent->message_put(type => "Fibrebridge/" . $fibrebridges->{$fibrebridge}->{NAME} . "/Load_Failed",
                             message => {});
        NATE::BaseException->throw("Failed to create Armadillo object for " . $fibrebridges->{$fibrebridge}->{NAME} . " : " .
                                   $atto_obj->errmsg());
    }

    # destroy the connection to the object,
    # we have to do this or else the object we send back to the parent gets messed up
    $atto_obj->destroy();

    # calculate the md5 for the fibrebridge object
    my $obj_md5_hex = NACL::AppBuilder::EnGenX::Util->_calculate_obj_md5_hash($atto_obj);

    $parent->message_put(type => "Fibrebridge/" . $fibrebridges->{$fibrebridge}->{NAME} . "/Load_Complete",
                         message => {obj_md5_hex => $obj_md5_hex,
                                     obj_serialized => Data::Dump::dump($atto_obj)});

    return $NACL::AppBuilder::EnGenX::TRUE;
}

1;
