#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor;

use strict;
use warnings;

use NATE::Messenger qw(messenger_global);
use NATE::BaseException qw(:try);
use NATE::STAF qw(staf_global);
use NATE::STAF::Peer;
use NATE::Process;
use NATE::ProcManager;
use NATE::Log qw(log_global);
use NATE::Events qw(eventloop_drain);
use Data::Dump qw(dump);
use Sys::Hostname;
use San::SanClient;
use ParamsMgr;
use NATE::Util::Sockets qw(get_address);

use parent qw(NACL::AppBuilder::EnGenX::HyperVisor::Config
              NACL::AppBuilder::EnGenX::HyperVisor::Setup
              NACL::AppBuilder::EnGenX::HyperVisor::Cleanup
              NACL::AppBuilder::EnGenX::HyperVisor::Deconfig
              NACL::AppBuilder::EnGenX::Util);

sub new {
    my ($class, %opts) = @_;
    my $self = \%opts;
    bless ($self, $class);

    # init or reuse a rep socket and store the endpoint
    $self->{endpoint} = messenger_global()->init(socket_type => 'rep',
                                                 action      => 'bind',
                                                 reuse       => 1);

    return $self;
}

sub xml_to_list {
    my ($self, %opts) = @_;

    # find all XML elements related to hypervisors and add them to the resource list
    $self->esx()->xml_to_list();
    $self->hyperv()->xml_to_list();
    $self->cxs()->xml_to_list();

    # hypervisor group
    my $hypervisor_group = &ParamsMgr::search(-search_path => [],
                                              -key => "HYPERVISORS",
                                              paramset => $self->get_paramset());
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw(&ParamsMgr::errmsg());
    }

    # add the entire group of hypervisors as a resource
    $self->resource_list()->add_resource(category => "HYPERVISORS",
                                         type => "hypervisor_groups",
                                         data => $hypervisor_group);

    return;
}

sub load_resource {
    my ($self, %opts) = @_;

    my $staf = staf_global();

    # get the environment from the resource list
    my $environment = $self->resource_list()->get_resources(category => "ENVIRONMENT",
                                                            type => "environment");
    $environment = $environment->[0];

    # find all hypervisors in the resource list and load their respective objects
    # verify that there are no duplicates
    my %dup_hash;
    my @procs = ();
    my $all_hypervisors = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                type => '^hyperv$|^esx$|^cxs$');
    foreach my $hypervisor (@{$all_hypervisors}) {
        my $name = $hypervisor->get_field("NAME");
        $dup_hash{$name}++;
        if ($dup_hash{$name} > 1) {
            NATE::BaseException->throw("Detected duplicate hypervisor entries for the client '$name'");
        }

        my $proc = NATE::Process->new(codespec       => '{ #EnGenX load ' . $name . '
                                                           use NACL::AppBuilder::EnGenX::HyperVisor;
                                                           NACL::AppBuilder::EnGenX::HyperVisor::_load_hypervisor(); }',
                                      runid          => "hypervisor__" . $name,
                                      onexit         => \&NATE::Process::on_exit_propagate_worst_result,
                                      client         => 'localhost',
                                      env_to_inherit => ["ENGENX_DISABLE_MD5_VERIFICATION", "USE_MPC_STAGING"],
                                      params         => { "hypervisor"           => $hypervisor->to_hashref(),
                                                          "environment"          => $environment->to_hashref(),
                                                          "staf_parent_handle"   => $staf->handle(),
                                                          "staf_parent_endpoint" => get_address(Sys::Hostname::hostname())});
        push(@procs, $proc);
    }

    # set up the Event listener and callback methods
    my $filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/SanClient\/\S+\/Load_Complete/);
    };
    my $callback = sub {
        my ($msg, $listener) = @_;

        # parse out the message information
        my ($hypervisor_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/SanClient\/(\S+?)\/Load_Complete/);
        my $egx_hypervisor_obj = $listener->{egx_hypervisor_obj};

        # rebuild the serialized object
        my $obj = eval($msg->{message}->{obj_serialized});
        if (not defined($obj)) {
            push(@{$egx_hypervisor_obj->{load_errors}}, "The SanClient object for hypervisor $hypervisor_name did not get transfered properly");
            $egx_hypervisor_obj->{parent}->{listeners_complete_count}++;
            return -1;
        }
        $obj = SanClient->new(-obj_template => $obj);
        if ($obj->error()) {
            push(@{$egx_hypervisor_obj->{load_errors}}, "Failed to instantiate SanClient object from template of hypervisor $hypervisor_name " . $obj->errmsg());
            $egx_hypervisor_obj->{parent}->{listeners_complete_count}++;
            return -1;
        }

        # store the object on the sanmgr resource hash
        $egx_hypervisor_obj->_store_object(type => "hypervisor",
                                           name => $hypervisor_name,
                                           obj => $obj);

        # validate the md5 for the obj
        if (not $egx_hypervisor_obj->_validate_obj_md5_hash(obj => $obj,
                                                            obj_md5_hex_before => $msg->{message}->{obj_md5_hex})) {
            $egx_hypervisor_obj->{parent}->{listeners_complete_count}++;
            push(@{$egx_hypervisor_obj->{load_errors}}, "MD5 check for hypervisor $hypervisor_name has failed.");
            return -1;
        }

        $egx_hypervisor_obj->{parent}->{listeners_complete_count}++;
        return 0;
    };
    my $listener = {filter => $filter, callback => $callback, egx_hypervisor_obj => $self};

    # build the fail listener
    my $fail_filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/SanClient\/\S+\/Load_Failed/);
    };
    my $fail_callback = sub {
        my ($msg, $flistener) = @_;
        my $egx_hypervisor_obj = $flistener->{egx_hypervisor_obj};

        # parse out the message information
        my ($hypervisor_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/SanClient\/(\S+?)\/Load_Failed/);
        push(@{$egx_hypervisor_obj->{load_errors}}, "The SanClient object for hypervisor $hypervisor_name failed to instantiate.");
        $egx_hypervisor_obj->{parent}->{listeners_complete_count}++;
        return -1;
    };

    my $fail_listener = {filter => $fail_filter, callback => $fail_callback, egx_hypervisor_obj => $self};

    return ($listener, $fail_listener, @procs);
}

sub _load_hypervisor {
    # fetch params passed into the process

    my $global_params = NATE::ParamSet->get_global();
    my $hypervisor = $global_params->get("hypervisor");
    my $environment = $global_params->get("environment");
    my $staf_parent_handle = $global_params->get("staf_parent_handle");
    my $staf_parent_endpoint = $global_params->get("staf_parent_endpoint");

    my @errors = ();
    my $hypervisor_name = $hypervisor->{NAME} // push(@errors, "Missing required field for hypervisor 'name'");
    my $conn_type       = $hypervisor->{CONN_TYPE} // "THARNHOST";
    my $protocol        = $hypervisor->{PROTOCOL} // push(@errors, "Missing required field for hypervisor 'protocol'");
    my $debug_level     = $hypervisor->{DEBUG_LEVEL} // 0;
    my $vma_host        = $hypervisor->{VMA_HOST} // "";
    my $alua_support    = $hypervisor->{ALUA_SUPPORT} // 1;
    my $mp_type         = $hypervisor->{MP_TYPE} // "";
    my $lvm_type        = $hypervisor->{LVM_TYPE} // "";

    my $parent = NATE::STAF::Peer->new(handle => $staf_parent_handle,
                                       endpoint => $staf_parent_endpoint);

    # check for argument errors
    if (@errors) {
        require NATE::Exceptions::Argument;
        $parent->message_put(type => "NACL/AppBuilder/EnGenX/HyperVisor/SanClient/" . $hypervisor_name . "/Load_Failed",
                             message => {});
        NATE::Exceptions::Argument->throw(join("\n", @errors));
    }

    # initialize the pnate attribute if it doesnt exist
    my $pnate = 0;
    if (defined($environment) and exists($environment->{PNATE})) {
        $pnate = $environment->{PNATE};
    }

    # generate a fake object with all the options built into it, so we
    # can pass in the iscsi type value into the sanclient constructor
    # needed for Linux & ESX
    my $fake_self = {};
    bless($fake_self, "EnGenX");
    my $obj = {};
    $fake_self->NACL::AppBuilder::EnGenX::Util::assign_sanclient_options(-client_obj => $obj,
                                                       -datastruct => $hypervisor,
                                                       -env_datastruct => $environment);
    # Create hypervisor SanClient object
    my $hypervisor_obj = SanClient->new(-host              => $hypervisor_name,
                                        -conn_type         => $conn_type,
                                        -debug_level       => $debug_level,
                                        -load_inits        => 1,
                                        -mp_type           => $mp_type,
                                        -lvm_type          => $lvm_type,
                                        -protocol          => $protocol,
                                        -pnate_environment => $pnate,
                                        -vma_host          => $vma_host,
                                        -alua_support      => $alua_support,
                                        -iscsi_type        => $obj->{iscsi}->{CONFIG_ISCSI_TYPE});
    if ($hypervisor_obj->error()) {
        $parent->message_put(type => "NACL/AppBuilder/EnGenX/HyperVisor/SanClient/" . $hypervisor_name . "/Load_Failed",
                             message => {});
        NATE::BaseException->throw("Failed to create SanClient object for $hypervisor_name : " .
                                   $hypervisor_obj->errmsg());
    }

    # assign all iscsi_options & fcp_options to this sanclient object
    $fake_self->NACL::AppBuilder::EnGenX::Util::assign_sanclient_options(-client_obj => $hypervisor_obj,
                                                       -datastruct => $hypervisor,
                                                       -env_datastruct => $environment);

    # destroy the connection to the sanclient object,
    # we have to do this or else the object we send back to the parent gets messed up
    $hypervisor_obj->disconnect_connection();

    # calculate the md5 for the hypervisor obj
    my $obj_md5_hex = NACL::AppBuilder::EnGenX::Util->_calculate_obj_md5_hash($hypervisor_obj);

    $parent->message_put(type => "NACL/AppBuilder/EnGenX/HyperVisor/SanClient/" . $hypervisor_name . "/Load_Complete",
                         message => {obj_md5_hex => $obj_md5_hex,
                                     obj_serialized => Data::Dump::dump($hypervisor_obj)});

    return;
}

sub esx {
    my ($self, %args) = @_;

    if (!$self->{esx}) {
        require NACL::AppBuilder::EnGenX::HyperVisor::ESX;
        $self->{esx} = NACL::AppBuilder::EnGenX::HyperVisor::ESX->new(parent => $self->{parent},
                                                    endpoint => $self->{endpoint});
    }
    return $self->{esx};
}

sub hyperv {
    my ($self, %args) = @_;

    if (!$self->{hyperv}) {
        require NACL::AppBuilder::EnGenX::HyperVisor::HyperV;
        $self->{hyperv} = NACL::AppBuilder::EnGenX::HyperVisor::HyperV->new(parent => $self->{parent},
                                                          endpoint => $self->{endpoint});
    }
    return $self->{hyperv};
}

sub cxs {
    my ($self, %args) = @_;

    if (!$self->{cxs}) {
        require NACL::AppBuilder::EnGenX::HyperVisor::CXS;
        $self->{cxs} = NACL::AppBuilder::EnGenX::HyperVisor::CXS->new(parent => $self->{parent},
                                                    endpoint => $self->{endpoint});
    }
    return $self->{cxs};
}

sub setup_sync {
    my ($self, %args) = @_;

    my $sync_type = delete($args{sync_type});
    my $num = delete($args{num_clients});

    my $old = $self->{_num_sync_clients}->{$sync_type};
    $self->{_num_sync_clients}->{$sync_type} = $num;

    $self->{_sync_listener}->{$sync_type} = messenger_global()->listener_add(listener => {
        filter   => sub { shift->type() eq "${sync_type}/sync_start" ;},
        callback => sub {
            my $m = shift;
            my $l = shift;
            my $type = $l->{sync_type};
            $self->{_sync_table}->{$type}->{$m} = $m;
            if (scalar (keys %{$self->{_sync_table}->{$type}}) >= $self->{_num_sync_clients}->{$type}) {
                # time to sync all the clients
                foreach my $msg (values %{$self->{_sync_table}->{$type}}) {
                    messenger_global()->reply(msg_obj => $msg,
                                              type => "${type}/sync_complete");
                }
                # reset the sync table
                $self->{_sync_table}->{$type} = undef;
                delete $self->{_sync_table}->{$type};
            }
        },
        sync_type => $sync_type
    });

    $self->{_sync_rollback_listener}->{$sync_type} = messenger_global()->listener_add(listener => {
        filter   => sub { shift->type() eq "${sync_type}/sync_rollback" ;},
        callback => sub {
            my $m = shift;
            my $l = shift;
            my $type = $l->{sync_type};
            $self->{_num_sync_clients}->{$type}--;
            if (scalar (keys %{$self->{_sync_table}->{$type}}) >= $self->{_num_sync_clients}->{$type}) {
                # time to sync all the clients if we received a rollback and
                # it made us go over the sync value
                foreach my $msg (values %{$self->{_sync_table}->{$type}}) {
                    messenger_global()->reply(msg_obj => $msg,
                                              type => "${type}/sync_complete");
                }
                # reset the sync table
                $self->{_sync_table}->{$type} = undef;
                delete $self->{_sync_table}->{$type};
            }
        },
        sync_type => $sync_type
    });

    return $old;
}

sub cleanup_sync {
    my ($self, %args) = @_;

    my $type = delete($args{sync_type});

    messenger_global()->listener_remove(listener => $self->{_sync_listener}->{$type});
    delete $self->{_sync_listener}->{$type};
    messenger_global()->listener_remove(listener => $self->{_sync_rollback_listener}->{$type});
    delete $self->{_sync_rollback_listener}->{$type};

    delete $self->{_num_sync_clients}->{$type};
}

sub synchronize {
    my ($self, %args) = @_;

    my $type = delete($args{type});
    my $endpoint = delete($args{endpoint});

    messenger_global()->init(socket_type => "rep", reuse => 1);
    messenger_global()->put(endpoint => $endpoint,
                            type => "${type}/sync_start");
    messenger_global()->get(type => "${type}/sync_complete");
}

sub sync_rollback {
    my ($self, %args) = @_;

    my $type = delete($args{type});
    my $endpoint = delete($args{endpoint});

    messenger_global()->init(socket_type => "rep", reuse => 1);
    messenger_global()->put(endpoint => $endpoint,
                            type => "${type}/sync_rollback");
}

1;
