#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor::CXS::Config;

use strict;
use warnings;

use NATE::Process;
use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);

sub config {
    my ($self, %args) = @_;

    # get all cxs cxsisors with config enabled on them
    my $cxs_resources = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                 type => "cxs",
                                                                 data => {CONFIG => 1});

    # create proc objects for each CXS host that will perform config and return
    # the list of them to the caller
    my $balance_count = 0;
    my @procs = ();
    foreach my $cxs_resource (@{$cxs_resources}) {
        my $host = $cxs_resource->get_field("NAME");
        my $proc = NATE::Process->new(codespec => sub {
                                          try {
                                              $self->_config_single(resource => $cxs_resource,
                                                                   balance => $balance_count++);
                                              $self->_config_complete(name => $host);
                                          } otherwise {
                                              my $e = shift;
                                              $self->_config_failure(name => $host);
                                              $e->throw();
                                          };
                                      },
                                      runid => "CXS-config" . "/" . $host . '-config',
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result);
        push(@procs, $proc);
    }

    return @procs;
}

sub _config_single {
    my ($self, %args) = @_;

    # given a single cxsisor, config this cxsisor
    my $resource = delete($args{resource}) // NATE::BaseException->throw("Missing required parameter 'resource'");
    my $balance = delete($args{balance}) // 0;

    my $client_name = $resource->get_field("NAME");

    # obtain the sanclient object
    my ($cxs_host_obj) = $self->get_objects(-type => "hypervisor",
                                            -names => [$client_name]);

    # get the list of vserver targets that this initiator is associated with
    my ($data) = $self->{parent}->get_target_list(-client_name => $client_name,
                                                  -filter_iscsi_targets_by_portset => $cxs_host_obj->{iscsi}->{CONFIG_ISCSI_FILTER_TARGETS_BY_PORTSET},
                                                  -filter_fcp_targets_by_portset   => $cxs_host_obj->{fcp}->{CONFIG_FCP_FILTER_TARGETS_BY_PORTSET});
    if ($self->{parent}->error()) {
        NATE::BaseException->throw($self->{parent}->errmsg());
    }

    my @iSCSITargets = @{$data->{iscsi_targets}};
    my @FcpTargets = @{$data->{fcp_targets}};

    # reconnect to this host
    $cxs_host_obj->reconnect(-disconnect_existing_conn => $SanMgr::FALSE);
    if ($cxs_host_obj->error()) {
        NATE::BaseException->throw($cxs_host_obj->errmsg());
    }

    my $protocol = $cxs_host_obj->get_protocol();

    # configure iSCSI/FCP
    if ($protocol =~ /iscsi|fcp/i) {
        $cxs_host_obj->configure(-targets => \@iSCSITargets);
        if ($cxs_host_obj->error()) {
            NATE::BaseException->throw("Failed to configure $protocol for the client ($client_name) " . $cxs_host_obj->errmsg());
        }
    }

    # NTP sync the hypervisors
    my $ntp_sync =        $resource->field_defined("NTP_SYNC")        ? $resource->get_field("NTP_SYNC") : 0;
    my $ntp_time_server = $resource->field_defined("NTP_TIME_SERVER") ? $resource->get_field("NTP_TIME_SERVER") : "ntp0.eng.netapp.com";
    my $ntp_time_zone   = $resource->field_defined("NTP_TIME_SERVER") ? $resource->get_field("NTP_TIME_ZONE") : "EST5EDT";
    if ($ntp_sync == 1) {
        $cxs_host_obj->ntp_sync(-ntp_server => $ntp_time_server,
                                -ntp_time_zone => $ntp_time_zone);
        if ($cxs_host_obj->error()) {
            my @error = $cxs_host_obj->errmsg();
            log_global()->warn("Failed to NTP time sync the $client_name (@error)");
            $cxs_host_obj->_clear_err();
        }
    }

    return;
}

sub _config_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/CXS/Config/Failure",
                            message  => \%data);
}

sub _config_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/CXS/Config/Complete",
                            message  => \%data);
}

1;
