#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor::Deconfig;

use strict;
use warnings;

use NATE::Log qw(log_global);
use NATE::BaseException qw(:try);

sub deconfig {
    my ($self, %args) = @_;

    my $run_id_prefix = delete($args{run_id_prefix}) // "";

    # skip setup if its turned off on the entire group
    my $hypervisor_groups = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                  type => "hypervisor_groups");
    if (!scalar(@{$hypervisor_groups})) {
        log_global()->log("No HyperVisors to deconfig");
        return;
    }
    if (!$hypervisor_groups->[0]->get_field("DECONFIG")) {
        log_global()->log("Skipping hypervisor deconfig because it is disabled");
        return;
    }

    my @procs = ();
    my @esx_procs = ();
    my @hyperv_procs = ();
    my @cxs_procs = ();

    # prepare all routines for each hypervisor type
    push(@esx_procs, $self->esx()->deconfig(%args));
    push(@hyperv_procs, $self->hyperv()->deconfig(%args));
    push(@cxs_procs, $self->cxs()->deconfig(%args));

    # install all the synchronization listeners
    if (scalar(@esx_procs)) {
        $self->setup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::ESX",
                          num_clients => scalar(@esx_procs));
        push(@procs, @esx_procs);
    }
    if (scalar(@hyperv_procs)) {
        $self->setup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::HyperV",
                          num_clients => scalar(@hyperv_procs));
        push(@procs, @hyperv_procs);
    }
    if (scalar(@cxs_procs)) {
        $self->setup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::CXS",
                          num_clients => scalar(@cxs_procs));
        push(@procs, @cxs_procs);
    }

    my $device_list = undef;
    my @failed_hypervisors;
    my $fail_listener = {
        filter => sub { shift->type() =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/.+?\/Deconfig\/Failure/i },
        callback => sub {
            my $msg = shift;
            my $l = shift;
            push(@failed_hypervisors, $msg->message()->{name});
            $l->{completed}++;
        },
        completed => 0
    };
    my $success_listener = {
        filter => sub { shift->type() =~ /NACL\/AppBuilder\/EnGenX\/HyperVisor\/.+?\/Deconfig\/Complete/i },
        callback => sub {
            my $msg = shift;
            my $l = shift;
            $l->{completed}++;
        },
        completed => 0
    };

    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => "$run_id_prefix" . "HyperVisor-deconfig");

    if (@failed_hypervisors) {
        NATE::BaseException->throw("The following hypervisors failed deconfig (@failed_hypervisors)");
    }

    # clean up all the synchronization listeners
    if (scalar(@esx_procs)) {
        $self->cleanup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::ESX");
    }
    if (scalar(@hyperv_procs)) {
        $self->cleanup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::HyperV");
    }
    if (scalar(@cxs_procs)) {
        $self->cleanup_sync(sync_type => "NACL::AppBuilder::EnGenX::HyperVisor::CXS");
    }

    return;
}

1;
