#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Config;

use strict;
use warnings;

use NATE::Process;
use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);

sub config {
    my ($self, %args) = @_;

    # get all hyperv hypervisors with config enabled on them
    my $hyperv_resources = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                 type => '^hyperv$',
                                                                 data => {CONFIG => 1});

    # create proc objects for each HYPERV host that will perform config and return
    # the list of them to the caller
    my $balance_count = 0;
    my @procs = ();
    foreach my $hyperv_resource (@{$hyperv_resources}) {
        my $host = $hyperv_resource->get_field("NAME");
        my $proc = NATE::Process->new(codespec => sub {
                                          try {
                                              $self->_config_single(resource => $hyperv_resource,
                                                                   balance => $balance_count++);
                                              $self->_config_complete(name => $host);
                                          } otherwise {
                                              my $e = shift;
                                              $self->_config_failure(name => $host);
                                              $e->throw();
                                          };
                                      },
                                      runid => "HyperV-config" . "/" . $host . '-config',
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result);
        push(@procs, $proc);
    }

    return @procs;
}

sub _config_single {
    my ($self, %args) = @_;

    # given a single hypervisor, config this hypervisor
    my $hyperv_host = delete($args{resource}) // NATE::BaseException->throw("Missing required parameter 'resource'");
    my $balance = delete($args{balance}) // 0;

    my $client_name = $hyperv_host->get_field("NAME");

    # given a single hypervisor, configure this hypervisor
    # By the time this function is done, this client should be
    # ready for fc zoning, or iscsi logins

    # obtain the sanclient object
    my ($hyperv_host_obj) = $self->get_objects(-type => "hypervisor",
                                               -names => [$client_name]);

    # get the list of vserver targets that this initiator is associated with
    my ($data) = $self->{parent}->get_target_list(-client_name => $client_name,
                                                  -filter_iscsi_targets_by_portset => $hyperv_host_obj->{iscsi}->{CONFIG_ISCSI_FILTER_TARGETS_BY_PORTSET},
                                                  -filter_fcp_targets_by_portset   => $hyperv_host_obj->{fcp}->{CONFIG_FCP_FILTER_TARGETS_BY_PORTSET});

    my @iSCSITargets = @{$data->{iscsi_targets}};
    my @FcpTargets = @{$data->{fcp_targets}};

    my $protocol = $hyperv_host_obj->get_protocol();

    # configure iSCSI/FCP
    if ($protocol =~ /iscsi|fcp/i) {
        $hyperv_host_obj->configure(-iscsi_targets => \@iSCSITargets,
                                    -fcp_targets => \@FcpTargets);
        if ($hyperv_host_obj->error()) {
            NATE::BaseException->throw("Failed to configure $protocol for the client ($client_name) " . $hyperv_host_obj->errmsg());
        }
    }

    # NTP sync the hypervisors
    my $ntp_sync = $hyperv_host->field_defined("NTP_SYNC") ? $hyperv_host->get_field("NTP_SYNC") : 0;
    my $ntp_time_server = $hyperv_host->field_defined("NTP_TIME_SERVER") ? $hyperv_host->get_field("NTP_TIME_SERVER") : "ntp0.eng.netapp.com";
    my $ntp_time_zone =   $hyperv_host->field_defined("NTP_TIME_ZONE") ? $hyperv_host->get_field("NTP_TIME_ZONE") : "EST5EDT";

    if ($ntp_sync == 1) {
        $hyperv_host_obj->ntp_sync(-ntp_server => $ntp_time_server,
                                   -ntp_time_zone => $ntp_time_zone);
        if ($hyperv_host_obj->error()) {
            my @error = $hyperv_host_obj->errmsg();
            log_global()->warn("Failed to NTP time sync the $client_name (@error))");
            $hyperv_host_obj->_clear_err();
        }
    }

    return;
}

sub _config_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/HyperV/Config/Failure",
                            message  => \%data);
}

sub _config_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/HyperV/Config/Complete",
                            message  => \%data);
}

1;
