#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Setup;

use strict;
use warnings;

use NATE::Process;
use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
use NATE::Messenger qw(messenger_global);
use Hostrec;

sub setup {
    my ($self, %args) = @_;

    # get all hyperv hypervisors with setup enabled on them
    my $hyperv_resources = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                 type => '^hyperv$',
                                                                 data => {SETUP => 1});

    # create proc objects for each HYPERV host that will perform setup and return
    # the list of them to the caller
    my $balance_count = 0;
    my @procs = ();
    foreach my $hyperv_resource (@{$hyperv_resources}) {
        my $host = $hyperv_resource->get_field("NAME");
        my $proc = NATE::Process->new(codespec => sub {
                                          try {
                                              my $data = $self->_setup_single(resource => $hyperv_resource,
                                                                              balance => $balance_count++);
                                              my %extra = ();
                                              if ($data) {
                                                  $extra{hyperv_client_dev_list} = $data;;
                                              }
                                              $self->_setup_complete(name => $host,
                                                                     %extra);
                                          } otherwise {
                                              my $e = shift;
                                              $self->_setup_failure(name => $host);
                                              $e->throw();
                                          };
                                      },
                                      runid => "HyperV-setup" . "/" . $host . '-setup',
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result);
        push(@procs, $proc);
    }

    return @procs;
}

sub _setup_single {
    my ($self, %args) = @_;

    # given a single hypervisor, setup this hypervisor
    my $hyperv_host = delete($args{resource}) // NATE::BaseException->throw("Missing required parameter 'resource'");
    my $balance = delete($args{balance}) // 0;

    my $hyperv_host_name = $hyperv_host->get_field("NAME");
    my $protocol = $hyperv_host->get_field("PROTOCOL");
    my @protocols;
    if ($protocol =~ /iscsi/i) {
        push(@protocols, "iscsi");
    }
    if ($protocol =~ /fcp/i) {
        push(@protocols, "fcp");
    }

    my $vhd_lun_path_regex_filter = $hyperv_host->field_defined("VHD_LUN_PATH_REGEX") ?
        $hyperv_host->get_field("VHD_LUN_PATH_REGEX") : undef;
    if (defined($vhd_lun_path_regex_filter)) {
        $vhd_lun_path_regex_filter = eval($vhd_lun_path_regex_filter);
    } else {
        $vhd_lun_path_regex_filter = undef;
    }

    # see if requested to allocate luns for the physical host
    my $physical_host_lun_path_regex_filter = $hyperv_host->field_defined("PHYSICAL_HOST_LUN_PATH_REGEX") ?
        $hyperv_host->get_field("PHYSICAL_HOST_LUN_PATH_REGEX") : undef;
    if (defined($physical_host_lun_path_regex_filter)) {
        $physical_host_lun_path_regex_filter = eval($physical_host_lun_path_regex_filter);
    } else {
        $physical_host_lun_path_regex_filter = undef;
    }

    # assume the io type for the hyperv host will be fs
    my $io_type = $hyperv_host->get_field("IO_TYPE") // "fs";

    # get list of vservers that this client will talk to
    my @vserver_list = $self->{parent}->get_vservers_associated_to_client(-client_name => $hyperv_host_name);


    # we also need to obtain the list of cluster objects that this client is related to as well
    my $hash = $self->{parent}->get_vserver_client_relationship_hash();

    # get all VM resources for this esx host
    my $virtual_machines = $self->resource_list()->get_resources(category => "HYPERVISORS",
                                                                 type => "virtual_machines",
                                                                 data => {HYPERVISOR_RESOURCE => $hyperv_host_name});

    my @vm_names = map { $_->get_field("NAME") } @{$virtual_machines};

    my ($hyperv_host_obj) = $self->get_objects(-type => "hypervisor",
                                               -names => [$hyperv_host_name]);

    # obtain VHD and PT total requested counts
    my $total_vhd_lun_count = 0;
    my $total_ptd_lun_count = 0;

    ($total_vhd_lun_count, $total_ptd_lun_count) = $self->_get_vhd_ptd_counts(virtual_machines => $virtual_machines);

    # reconnect to this host
    $hyperv_host_obj->reconnect(-disconnect_existing_conn => 0);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to reconnect to HyperV host '$hyperv_host_name' " . $hyperv_host_obj->errmsg());
    }

    # cleanly shutdown all virtual machines
    $hyperv_host_obj->power_management(-action => "shutdown",
                                       -vm_list => \@vm_names);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to power off VMs " . $hyperv_host_obj->errmsg());
    }

    # discover new luns
    $hyperv_host_obj->discover(-num_expected_luns => $total_vhd_lun_count + $total_ptd_lun_count);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to discover devices" . $hyperv_host_obj->errmsg());
    }

    # clean up empty drive slots on all guests
    $hyperv_host_obj->remove_empty_drive_slots_from_vms(-vm_list => \@vm_names);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to remove empty drive slots from VMs " . $hyperv_host_obj->errmsg());
    }

    # create devmgr object
    my $devmgr_obj = $hyperv_host_obj->create_devmgr(-balance => $balance,
                                                     -vhd_lun_path_regex => undef,
                                                     -lun_path_regex => undef);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to create device manager object " . $hyperv_host_obj->errmsg());
    }

    log_global()->log($self->dumper($devmgr_obj));
    # if specified to provision LUNs to the hyper-v host, then grab devices that match
    # the regular expression specified
    my @device_info_list;
    if (defined($physical_host_lun_path_regex_filter)) {
        my @all_physical_host_luns = $devmgr_obj->get_free_devices(-filters => \@vserver_list,
                                                                   -device_type => "passthrough",
                                                                   -lun_path_regex => $physical_host_lun_path_regex_filter,
                                                                   -protocol => join("|", @protocols));
        if ($devmgr_obj->error()) {
            NATE::BaseException->throw("Unable to get free physical host devices ". $devmgr_obj->errmsg());
        }

        log_global()->log($self->dumper(\@all_physical_host_luns));
        # format these disks and mount them up, and send up the client dev list data
        foreach my $dev_obj (@all_physical_host_luns) {
            my $device_info = {};

            # Save some attributes of the device onto the device_info hash
            if ($dev_obj->get_lun() =~ /^\/vol\/(\S+)\/(\S+)$/) {
                push(@{$device_info->{'volume'}}, $1);
                push(@{$device_info->{'lun'}}, $2);
            }
            push(@{$device_info->{'vserver'}}, $dev_obj->get_filer_name());
            $device_info->{'protocol'} = lc($dev_obj->get_protocol());

            # Push the device serial number onto the device info hash if it is defined
            if (defined($dev_obj->{disk_obj}->{serial}) and $dev_obj->{disk_obj}->{serial} ne "") {
                push(@{$device_info->{'serial'}}, $dev_obj->{disk_obj}->{serial});
            }

            if ($io_type =~ /fs/) {
                $dev_obj->create_fs();
                if ($dev_obj->error()) {
                    NATE::BaseException->throw("Failed to format device " . $dev_obj->errmsg());
                }

                $dev_obj->mount_fs();
                if ($dev_obj->error()) {
                    NATE::BaseException->throw("Failed to assign driveletter to device " . $dev_obj->errmsg());
                }

                # parse out the drive letter
                $device_info->{path} = $dev_obj->get_mount();
                $device_info->{io_type} = "fs";
            } else {
                $device_info->{path} = $dev_obj->get_raw_device();
                $device_info->{io_type} = "raw";
            }
            push(@device_info_list, $device_info);
        }
    }

    # get all vhd devices and store in vhd pool
    my @all_vhd_devices = ();
    # obtain all the VHD devices by using get_free_devices()
    if ($total_vhd_lun_count > 0) {
        @all_vhd_devices = $devmgr_obj->get_free_devices(-filers => \@vserver_list,
                                                         -device_type => "vhd",
                                                         -num_devices => $total_vhd_lun_count,
                                                         -lun_path_regex => $vhd_lun_path_regex_filter,
                                                         -protocol => join("|", @protocols));
        if ($devmgr_obj->error()) {
            NATE::BaseException->throw("Unable to get free VHD devices " . $devmgr_obj->errmsg());
        }
    }

    # foreach vm
    ## if vm wants vhds
    ### take some vhds from the pool
    ### store vhds in datastruct
    ## if vm wants PTDs
    ### take some ptds from the pool
    ### store ptds in datastruct
    foreach my $virtual_machine (@{$virtual_machines}) {
        # allocate some vhd devices for this VM
        my @vhd_devs_for_this_vm = ();
        for (my $i = 0; $i < $virtual_machine->get_field("VIRTUAL_DISKS"); $i++) {
            push(@vhd_devs_for_this_vm, shift(@all_vhd_devices));
            @all_vhd_devices = reverse(@all_vhd_devices);
        }
        $virtual_machine->{vhd_dev_objs} = \@vhd_devs_for_this_vm;

        # allocate some ptd devices for this VM
        my @ptd_devs_for_this_vm = ();
        my $raw_lun_path_regex = $virtual_machine->get_field("RAW_LUN_PATH_REGEX");
        if (defined($raw_lun_path_regex)) {
            $raw_lun_path_regex = eval($raw_lun_path_regex);
        } else {
            $raw_lun_path_regex = undef;
        }

        # figure out the OStype of this virtual machine
        my $vm_name = $virtual_machine->get_field("NAME");
        my $hostrec = Hostrec->new(id => $vm_name);
        my $ostype = $hostrec->hosttype();
        $ostype = "linux" if ($ostype =~ /linux/i);
        $ostype = "windows" if ($ostype =~ /windows/i);
        if (not length($ostype) or $ostype !~ /windows|linux/i) {
            NATE::BaseException->throw("The hosttype '$ostype' of virtual machine '$vm_name' is invalid, only linux and windows hosttypes are supported ");
        }

        @ptd_devs_for_this_vm = $devmgr_obj->get_free_devices(-filers => \@vserver_list,
                                                              -device_type => "passthrough",
                                                              -num_devices => $virtual_machine->get_field("RAW_DISKS"),
                                                              -lun_ostype_regex => qr/$ostype/i,
                                                              -lun_path_regex => $raw_lun_path_regex,
                                                              -protocol => join("|", @protocols));
        if ($devmgr_obj->error()) {
            NATE::BaseException->throw("Unable to get free PassThrough devices " . $devmgr_obj->errmsg());
        }
        $virtual_machine->{ptd_dev_objs} = \@ptd_devs_for_this_vm;
    }

    foreach my $virtual_machine (@{$virtual_machines}) {
        my (@vhds_for_this_guest);
        if (scalar(@{$virtual_machine->{vhd_dev_objs}})) {
            (@vhds_for_this_guest) = $devmgr_obj->create_vhd_files(-vhd_devices => $virtual_machine->{vhd_dev_objs},
                                                                   -num_vhds_per_device => 1,
                                                                   -mount_type => "letter",
                                                                   -vhd_types => [split(/,/, $hyperv_host->get_field("VHD_DISK_TYPE"))]);
            if ($devmgr_obj->error()) {
                NATE::BaseException->throw("Failed to create VHD files " . $devmgr_obj->errmsg());
            }
        }
        $virtual_machine->{vhd_files} = \@vhds_for_this_guest;
    }

    # foreach vm,  attach the devices in a parallel subtest
    my @subtests = ();
    foreach my $virtual_machine (@{$virtual_machines}) {
        my $subtest = NATE::Process->new(codespec => sub { $self->_attach_devices_to_vm(balance => $balance,
                                                                                        hyperv_host_obj => $hyperv_host_obj,
                                                                                        hyperv_host_resource => $hyperv_host,
                                                                                        virtual_machine_resource => $virtual_machine); },
                                         runid => $virtual_machine->get_field("NAME") . '-attach_devices');
                                         #onexit => \&NATE::Process::on_exit_propagate_worst_result);
        $balance += 1;
        $subtest->start();
        push(@subtests, $subtest);
    }

    if (scalar(@subtests) == 0) {
        # nothing to do
        return \@device_info_list;
    }

    # wait for all procs to finish
    my $error = 0;
    foreach (@subtests) {
        $_->wait();
        if ($_->worst_result()) {
            $error = 1;
        }
        $_->destroy();
    }

    # turn all VMs back on, or turn each vm on inside it's own subtest
    $hyperv_host_obj->power_management(-action => "power_on",
                                       -vm_list => \@vm_names);
    if ($hyperv_host_obj->error()) {
        NATE::BaseException->throw("Unable to create device manager object " . $hyperv_host_obj->errmsg());
    }

    if ($error) {
        NATE::BaseException->throw("Encountered an error when attaching disks to a virtual machine.");
    }

    return \@device_info_list;
}

sub _attach_devices_to_vm {
    my ($self, %args) = @_;

    my $hyperv_host = delete($args{hyperv_host_resource});
    my $virtual_machine = delete($args{virtual_machine_resource});
    my $hyperv_host_obj = delete($args{hyperv_host_obj});
    my $balance = delete($args{balance});

    # establish a new random seed for this subtest to allow concurrent access
    # to diskpart
    srand($balance);

    # set up the Load balancing policies for all LUNs
    foreach my $device (@{$virtual_machine->{vhd_dev_objs}}, @{$virtual_machine->{ptd_dev_objs}}) {
        $device->set_lbpolicy();
        if ($device->error()) {
            NATE::BaseException->throw("Failed to set Load Balance policy on device " . $device->errmsg());
        }
    }

    my $devmgr_obj = $hyperv_host_obj->{obj}{devmgr};

    # select the PTD and VHD controller types, default to the Hyperv Host value
    # if no value is specified at the virtual machine level
    my $controller_type;
    if ($virtual_machine->field_defined("CONTROLLER_TYPE")) {
        $controller_type = $virtual_machine->get_field("CONTROLLER_TYPE");
    } else {
        $controller_type = $hyperv_host->get_field("CONTROLLER_TYPE");
    }

    # configure any PTD devices first
    if (scalar(@{$virtual_machine->{ptd_dev_objs}})) {
        $devmgr_obj->configure_passthrough_disks(-passthrough_devices => $virtual_machine->{ptd_dev_objs},
                                                 -controller_types => [split(/,/, $controller_type)]);
        if ($devmgr_obj->error()) {
            NATE::BaseException->throw("Failed to configure passthrough devices " . $devmgr_obj->errmsg());
        }
    }

    # attach VHD files
    if (scalar(@{$virtual_machine->{vhd_files}})) {
        $hyperv_host_obj->add_vhd_files_to_vm(-vhd_files => $virtual_machine->{vhd_files},
                                              -vm_name => $virtual_machine->get_field("NAME"),
                                              -controller_types => [split(/,/, $controller_type)]);
        if ($hyperv_host_obj->error()) {
            NATE::BaseException->throw("Unable to attach VHD files to vm " . $hyperv_host_obj->errmsg());
        }
    }

    # attach PTD handles
    if (scalar(@{$virtual_machine->{ptd_dev_objs}})) {
        $hyperv_host_obj->add_passthrough_disks_to_vm(-passthrough_devices => $virtual_machine->{ptd_dev_objs},
                                                      -vm_name => $virtual_machine->get_field("NAME"));
        if ($hyperv_host_obj->error()) {
            NATE::BaseException->throw("Unable to attach PTD files to vm " . $hyperv_host_obj->errmsg());
        }
    }

    return;
}

sub _get_vhd_ptd_counts {
    my ($self, %args) = @_;

    my $virtual_machines = delete($args{virtual_machines});

    my $total_vhd = 0;
    my $total_ptd = 0;

    # calculate the total vhd and total ptd disk request counts
    foreach my $vm (@{$virtual_machines}) {
        $total_vhd += $vm->get_field("VIRTUAL_DISKS") // 0;
        $total_ptd += $vm->get_field("RAW_DISKS") // 0;
    }

    return ($total_vhd, $total_ptd);
}

sub _setup_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/HyperV/Setup/Failure",
                            message  => \%data);
}

sub _setup_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/HyperVisor/HyperV/Setup/Complete",
                            message  => \%data);
}

1;
