# $Id: //depot/prod/DOT/dev/test/lib/NACL/AppBuilder/EnGenX/ResourceList.pm#1 $

#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide API's to manage EnGenX Resource objects.
## @author dl-san-automation@netapp.com
## @status private
## @pod here

=head1 NAME

ResourceList

=head1 DESCRIPTION

ResourceList provides API's to manage EnGenX Resource Objects.
Resources will be divided first by catagory and then by type.
The categories and types in use can be determined by the user
and can be any valid Perl hash key.

=cut

package NACL::AppBuilder::EnGenX::ResourceList;

# Compiler directives.
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate OBJECT SCALAR SCALARREF ARRAYREF HASHREF UNDEF GLOBREF);

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Resource Objects
use NACL::AppBuilder::EnGenX::Resource;

use vars qw(@ISA);
@ISA = qw();

###############################################################################
#                               Public Methods
###############################################################################

=head1 METHODS

=head2 new

Create a new ResourceList object.

    my $rl_obj = new NACL::AppBuilder::EnGenX::ResourceList();

=over

=item Returns

The ResourceList object created

=item Exceptions

None

=back

=cut

########################################################################
sub new {
    $Log->enter() if $may_enter;

    my ($proto, %args) = @_;

    my $self = {};
    my $class = ref($proto) || $proto;
    bless($self, $class);

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

########################################################################

=head2 add_resource

Add a Resource to a ResourceList.  Note this will create an NACL::AppBuilder::EnGenX::Resource
object and add it to a resource based on the category and type given.

    my $data_ref = {name => "blue_devil",
                    type => "champion",
                    size => "100g" };
    my $resource = $rl_obj->add_rescource(category => "cluster_fas3170_23ab",
                                          type     => volume,
                                          data     => $data_ref);

=over

=item Options

=over

=item C<< category=>$category >>

(Optional) A string that defines the catgory for the resource being added.
Example: "cluster_fas3170_23". Default is "default".

=item C<< type=>$type >>

(Optional) A string that defines the type of redource being added.
Example: "volume". Default is "default".

=item C<< data=>\%data >>

(Required) A Hash ref of data that will be pass through and added to the
Resource object being added.

=back

=item Returns

The NACL::AppBuilder::EnGenX::Resource object created

=item Exceptions

NATE::Exceptions::Argument

=back

=cut

########################################################################
sub add_resource {
    my ($self, @opts) = @_;
    my %args;

    # Process arguments.  Since Validate dies on error, we will catch
    # any errors and throw a standard Arugment exception. Try will promote
    # the die into an exception.
    try {
        %args = Params::Validate::validate( @opts, {
            category => {type => SCALAR, default => "default", optional => 1},
                type => {type => SCALAR, default => "default", optional => 1},
                data => {type => HASHREF, optional => 0},
            });
    }
    otherwise {
        my $ex = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($ex->text());
    };

    my $category = $args{category};
    my $type     = $args{type};
    my $data     = $args{data};

    if ($Log->may_trace()) {
        $Log->trace("add_resource(): category = $category, "
                . "type = $type, data =\n"
                . Dumper($data));
    }

    # Create a new Resource object populated with the hash ref of data
    my $current_resource = $data;
    unless(ref($data) eq 'NACL::AppBuilder::EnGenX::Resource') {
        $current_resource = new NACL::AppBuilder::EnGenX::Resource($data);
    }
    # Push the Resource object onto the array based on category and type
    push(@{$self->{list_data}->{$category}->{$type}}, $current_resource);

    return $current_resource;
} ## end sub add_resource

########################################################################

=head2 get_resources

Get an array of Resource objects from a ResourceList.

    my $data_ref =  {
        "name" => "vol_group_1_*",
        "size" => "100g",
    };
    my $resource = $rl_obj->get_resources(category => "cluster_fas3170_23ab",
                                          type     => volume,
                                          data     => $data_ref);

=over

=item Options

=over

=item C<< category=>$category >>

(Optional) A regex string that defines the catgory for the resource being added.
Example: "cluster_fas3170*". Default is "default".

=item C<< type=>$type >>

(Optional) A regex string that defines the type of redource being added.
Example: "vol_31\d\d_*". Default is ".*".

=item C<< data=>\%data >>

(Optional) A Hash ref of data that will be used to filter resources.
The fields in a resource must match the data given to be returned.
In the hash, the values for each key will be treated as a regex.
Default is undef (don't filter on data).

=item C<< neg_data=>\%neg_data >>

(Optional) A Hash ref of data that will be used to filter resources.
The fields in a resource must NOT match the data given to be returned.
In the hash, the values for each key will be treated as a regex.
Default is undef (don't filter on data).

=back

=item Returns

An array of NACL::AppBuilder::EnGenX::Resource objects

=item Exceptions

NATE::Exceptions::Argument

=back

=cut

########################################################################
sub get_resources {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args;

    # Process arguments.  Since Validate dies on error, we will catch
    # any errors and throw a standard Arugment exception. Try will promote
    # the die into an exception.
    try {
        %args = Params::Validate::validate( @opts, {
            category => {type => SCALAR | SCALARREF,  default => "default", optional => 1},
            type     => {type => SCALAR | SCALARREF,  default  => ".*",     optional => 1},
            data     => {type => HASHREF, optional => 1},
            neg_data => {type => HASHREF, optional => 1},
            });
    }
    otherwise {
        my $ex = shift;
        NATE::Exceptions::Argument->throw($ex->text());
    };

    if ($Log->may_trace()) {
        my $category = $args{category};
        my $type     = $args{type};
        my $data     = $args{data};
        my $neg_data     = $args{neg_data};
        $Log->trace("get_resources(): category = $category, "
                . "type = $type, data =\n"
                . Dumper($data)
                . "neg_data =\n"
                . Dumper($neg_data));
    }

    $Log->exit() if $may_exit;
    return $self->_find_and_process(%args);

} ## end sub get_resources

########################################################################

=head2 get_categories

Get an array of strings that represent the categories in a ResourceList.

    my $list_categories = $rl_obj->get_categories();
    my $list_categories = $rl_obj->get_categories(category = qr/^CLUSTER_/);

=over

=item Options

=over

=item C<< category=>$category >>

(Optional) A regex string that must match any keys to be returned.
Example: "^CLIENT_" or "^CLUSTER_". Default is ".*".

=back

=item Returns

An array of strings

=item Exceptions

NATE::Exceptions::Argument

=back

=cut

########################################################################
sub get_categories {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args;

    # Process arguments.  Since Validate dies on error, we will catch
    # any errors and throw a standard Arugment exception. Try will promote
    # the die into an exception.
    try {
        %args = Params::Validate::validate( @opts, {
            category => {type     => SCALAR | SCALARREF,
                         default  => qr/.*/,
                         optional => 1},
            });
    }
    otherwise {
        my $ex = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($ex->text());
    };

    my $category = $args{category};

    if ($Log->may_trace()) {
        $Log->trace("get_catagories(): category = $category, ");
    }


    # Get all the keys and if the category is ".*" (the default meaning
    # everything) then just a reference to the array.
    my @cat_keys = keys %{$self->{list_data}};

    if ($category eq ".*") {
        $Log->exit() if $may_exit;
        return (wantarray()) ? @cat_keys : \@cat_keys;
    }

    my @filtered_list = ();
    foreach my $key (@cat_keys) {
        next if ($key !~ $category);    # skip if we don't match
        push (@filtered_list, $key);
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? @filtered_list : \@filtered_list;
} ## end sub get_categories

########################################################################

=head2 modify_resources

Modify Resource objects from a ResourceList.

    my $data_ref =  {
        "name" => "vol_group_1_*",
        "size" => "100g"
    };
    my $new_data_ref =  {
        "name" => "vol_renamed_to_me",
    };
    $rl_obj->modifyet_resources(category => "cluster_fas3170_23ab",
                                type     => volume,
                                data     => $data_ref,
                                new)data => $new_data_ref);

=over

=item Options

=over

=item C<< category=>$category >>

(Optional) A regex string that defines the catgory for the resource being added.
Example: "cluster_fas3170*". Default is "default".

=item C<< type=>$type >>

(Optional) A regex string that defines the type of redource being added.
Example: "vol_31\d\d_*". Default is ".*".

=item C<< data=>\%data >>

(Optional) A Hash ref of data that will be used to filter resources.
The fields in a resource must match the data given to be returned.
In the hash, the values for each key will be treated as a regex.
Default is undef (don't filter on data).

=item C<< new_data=>\%new_data >>

(Required) A Hash ref of data that will written to a resource.
In the hash, the values for each key will are not a regex, they are
usually strings.

=back

=item Returns

None

=item Exceptions

NATE::Exceptions::Argument

=back

=cut

########################################################################
sub modify_resources {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args;

    # Process arguments.  Since Validate dies on error, we will catch
    # any errors and throw a standard Arugment exception. Try will promote
    # the die into an exception.
    try {
        %args = Params::Validate::validate( @opts, {
            category => {type => SCALAR | SCALARREF,  default => "default", optional => 1},
            type     => {type => SCALAR | SCALARREF,  default  => ".*",     optional => 1},
            data     => {type => HASHREF, optional => 1},
            new_data => {type => HASHREF, optional => 0},
            });
    }
    otherwise {
        my $ex = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($ex->text());
    };

    if ($Log->may_trace()) {
        my $category = $args{category};
        my $type     = $args{type};
        my $data     = $args{data};
        my $new_data = $args{new_data};
        $Log->trace("get_resources(): category = $category, "
                . "type = $type, data =\n"
                . Dumper($data)
                . "new_data =\n"
                . Dumper($new_data));
    }

    $Log->exit() if $may_exit;
    return $self->_find_and_process(%args);

} ## end sub modify_resources

########################################################################

=head2 merge_with_resource_list

Merge this resource list object with a specified resource list.

NOTE: The resource lists will be concatenated. So if there are duplicates
present between the two resource lists, the result will contain duplicates.

    $rlist_obj->merge_with_resource_list(resource_list => $other_rlist_obj);

=over

=item Options

=over

=item C<< resource_list => $resource_list_obj >>

(Required) The resource list to merge with. Must be a valid C<NACL::AppBuilder::EnGenX::ResourceList> object.

=back

=item Returns

None

=item Exceptions

NATE::Exceptions::Argument

=back

=cut

########################################################################
sub merge_with_resource_list {
    my ($self, %args) = @_;

    my $other_list = $args{resource_list} //
        NATE::Exceptions::Argument->throw("Missing required parameter 'resource_list'");
    unless (ref($other_list) and $other_list->isa("NACL::AppBuilder::EnGenX::ResourceList")) {
        NATE::Exceptions::Argument->throw("Invalid 'resource_list' specified. It must be of type NACL::AppBuilder::EnGenX::ResourceList");
    }

    foreach my $cat ($other_list->get_categories()) {
        foreach my $type (keys %{$other_list->{list_data}->{$cat}}) {
            foreach my $res (@{$other_list->{list_data}->{$cat}->{$type}}) {
                $self->add_resource(category => $cat,
                                    type => $type,
                                    data => $res);
            }
        }
    }
}

###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Description: Method to find and process Reources.  Both get_resources and
#              modify_resources() take a category, type and data to match on
#              and this method is common to both.
#
# Inputs:      category, type, data (hashref), data_type(hashref)
# Outputs:     For get, we return an array of Resources.  For Modify we
#              return TRUE/FALSE
###############################################################################
sub _find_and_process {
    $Log->enter() if $may_enter;
    my ($self, @opts) = @_;
    my %args;
    my @return_array = ();

    # Process arguments.  Since Validate dies on error, we will catch
    # any errors and throw a standard Arugment exception. Try will promote
    # the die into an exception.
    try {
        %args = Params::Validate::validate( @opts, {
            category => {type => SCALAR | SCALARREF,  default => "default", optional => 1},
            type     => {type => SCALAR | SCALARREF,  default  => ".*",     optional => 1},
            data     => {type => HASHREF, optional => 1},
            neg_data => {type => HASHREF, optional => 1},
            new_data => {type => HASHREF, optional => 1},
            });
    }
    otherwise {
        my $ex = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($ex->text());
    };

    my $category = $args{category};
    my $type     = $args{type};
    my $data     = $args{data};
    my $neg_data = $args{neg_data};
    my $new_data = $args{new_data};

    my $action = (caller(1))[3];

    if ($Log->may_trace()) {
        $Log->trace("get_resources(): category = $category, "
                . "type = $type, action = $action data =\n"
                . Dumper($data)
                . "neg_data =\n"
                . Dumper($neg_data)
                . "new_data =\n"
                . Dumper($new_data));
    }

    # Get the hash of categories and do a depth first search to find
    # matching entries
    my $list_data = $self->{list_data};
    foreach my $current_category (keys %{$list_data}) {
        next unless ($current_category =~ $category);

        # If we get here we have a matching category.
        my $category_data = $list_data->{$current_category};
        foreach my $current_type (keys %{$category_data}) {
            next unless ($current_type =~ $type);

            # If we get here we have a matching category and type

            # Need a foreach resource at this point.  If there are data
            # filters will need a foreach data filter to check and see
            # if each of those match the data in the resource.
            if (defined $data or defined $neg_data) {
                my @current_list = @{$category_data->{$current_type}};
                foreach my $resource (@current_list) {
                    my $match = TRUE;
                    foreach my $key (keys %{$data}) {
                        my $value = $data->{$key};

                        # if any of the fields we need to match are not
                        # defined in the resource, this resource won't be
                        # returned
                        if ($resource->field_defined($key) == FALSE) {
                            $match = FALSE;
                            last;
                        }

                        # if any of the fields we need to match are not
                        # a match, this resource won't be returned
                        my $resource_value = $resource->get_field($key);
                        if (ref($resource_value) =~ /ARRAY/i) {
                            my $found = FALSE;
                            foreach my $temp_value (@$resource_value) {
                                if ($temp_value =~ $value) {
                                    $found = TRUE;
                                    last;
                                }
                            }
                            if (not $found) {
                                $match = FALSE;
                                last;
                            }
                        } elsif ($resource_value !~ $value) {
                            $match = FALSE;
                            last;
                        }
                    }    # end foreach data key
                    if ($match) {
                        foreach my $key (keys %{$neg_data}) {
                            my $value = $neg_data->{$key};

                            # if any of the fields we need to NOT match are not
                            # defined in the resource, then go to next key
                            next if ($resource->field_defined($key) == FALSE);

                            # if any of the fields we need to NOT match are
                            # a match, this resource won't be returned
                            my $resource_value = $resource->get_field($key);
                            if (ref($resource_value) =~ /ARRAY/i ) {
                                my $found = FALSE;
                                foreach my $temp_value (@$resource_value) {
                                    if ($temp_value =~ $value) {
                                        $found = TRUE;
                                        last;
                                    }
                                }
                                if ($found) {
                                    $match = FALSE;
                                    last;
                                }
                            } elsif ($resource_value =~ $value) {
                                $match = FALSE;
                                last;
                            }
                        }    # end foreach neg_data key
                    } # end if match
                    if ($match) {
                        if ($action =~ /get_resources/) {
                           push(@return_array, $resource);
                        } else {
                            foreach my $new_key (keys %{$new_data}) {
                                my $new_value = $new_data->{$new_key};
                                $resource->set_field($new_key, $new_value);
                            }
                        }
                    } # end if match
                }    # end foreach resource
            } else {

                # Concat the entire category->type array to the return array
                @return_array = (@return_array,
                                 @{$category_data->{$current_type}});
            }
        }    # end foreach type
    }    # end foreach category

    $Log->exit() if $may_exit;
    return wantarray ? @return_array : \@return_array;
} ## end sub _find_and_process

#
# TODO:
# delete_resources
#    - category, type, resource_key, pattern OR an array of resources
# write_list()
# read_list()
#    - these could read/write subsets if user passes in the same stuff
#      we sent to get_resources() to filter
#    - want to write out the object as a Perl Hash
# modify_resources
#    - category, type, resource_key, pattern OR an array of resources and data
# get_catetories(), get_types()

1;
