#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Services::Base;

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw();
}

use strict;
use warnings;

use Carp qw(confess);

use NATE::Messenger qw(messenger_global);
use NATE::ParamSet qw(param_global);
use NATE::Events qw(eventloop_drain);
use NATE::Process;
use NATE::Log qw(log_global);

use NACL::AppBuilder::EnGenX::Exceptions::ServiceAPIError;

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    my ($class, %args) = @_;

    # store all params on self
    my $self = \%args;

    $self->{engenx_obj} = delete($args{engenx_obj}) // confess "Missing required parameter 'engenx_obj'";

    bless($self, $class);

    if (!exists($self->{runid})) {
        $self->{runid} = ref($self);
        $self->{runid} =~ s/:/_/g;
    }

    return $self;
}

sub init {
    my ($self) = @_;

    messenger_global()->listener_add(listener => {
        filter => sub { shift->type() eq 'svc_ready' },
        callback => sub {
            $self->{svc_endpoint} = shift->message()->{endpoint};
        },
        charges => 1
    });

    # reuse or init a bound rep socket on the messenger
    my $rep = messenger_global()->init(socket_type => 'rep',
                                       action => 'bind',
                                       reuse => 1);

    # spawn the service
    $self->{svc_proc} = NATE::Process->new(
        codespec => sub {$self->_serve()},
        inherit => 1,
        runid => ".engenx_services/" . $self->{runid},
        onexit => \&NATE::Process::on_exit_die_worst_result,
        params => {
            parent_rep => $rep,
        });
    $self->{svc_proc}->start();

    # wait for it to init
    while (!$self->{svc_endpoint}) {
        eventloop_drain();
    }

    return $self->{svc_endpoint};
}

sub _serve {
    my ($self) = @_;

    my $parent_rep = param_global()->get("parent_rep");

    # initialize a rep socket for this service
    my $svc_rep = messenger_global()->init(socket_type => 'rep');

    # perform any pre svc ready routines here
    $self->_pre_svc_ready();

    # tell the parent that we are ready
    messenger_global()->put(endpoint => $parent_rep,
                            type => "svc_ready",
                            message => {endpoint => $svc_rep});

    # perform any post svc ready routines here
    $self->_post_svc_ready();

    # install custom APIs here
    $self->_install_apis();

    # sit in an eventloop forever waiting for svc requests
    while (1) {
        NATE::Events::eventloop_drain();
    }

    return;
}

sub _pre_svc_ready {
    # NOOP
}

sub _post_svc_ready {
    # NOOP
}

sub _install_apis {
    # NOOP
}

sub _common_svcapi {
    my ($self, %args) = @_;

    my $api = delete($args{_api});

    %args = &SanUtil::strip_dash(%args);

    if (!scalar(@{messenger_global()->get_endpoints(filter => {socket_type => 'rep'})})) {
        messenger_global()->init(socket_type => 'rep');
    }
    my $request_id = rand;
    messenger_global()->put(type => "$api",
                            message => {%args, request_id => $request_id},
                            endpoint => $self->{svc_endpoint});

    my $msg;
    my $success;
    my $fail;

    # install listeners for successful and error responses from the service
    $success = messenger_global()->listener_add(listener => {
        filter => sub {
            shift->type() eq "response_${request_id}";
        },
        callback => sub {
            $msg = shift;
            messenger_global()->listener_remove(listener => $success);
            messenger_global()->listener_remove(listener => $fail);
        },
        charges => 1
    });

    $fail = messenger_global()->listener_add(listener => {
        filter => sub {
            shift->type() eq "error_${request_id}";
        },
        callback => sub {
            messenger_global()->listener_remove(listener => $success);
            messenger_global()->listener_remove(listener => $fail);
            NACL::AppBuilder::EnGenX::Exceptions::ServiceAPIError->throw(shift->message());
        },
        charges => 1
    });

    # wait for the message (success or failure) to arrive
    while (!$msg) {
        eventloop_drain();
    }

    undef $success;
    undef $fail;
    
    return $msg->message();
}

1;
