#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::Switch;

use strict;
use warnings;

use NATE::Process;
use NATE::STAF::Peer;
use NATE::STAF qw(staf_global);
use NATE::Util::Sockets;
use Sys::Hostname;
use San::SanSwitch;
use Data::Dump;

use parent qw(NACL::AppBuilder::EnGenX::Util);

sub new {
    my ($class, %opts) = @_;
    my $self = \%opts;
    bless($self, $class);
    return $self;
}

use constant setup => 1;
use constant config => 1;
use constant deconfig => 1;
use constant cleanup => 1;
use constant validate => 1;

sub xml_to_list {
    my ($self, %args) = @_;

    # find all switches in the paramset
    # and register them with the resource list

    # Get list of switches
    my $switches = &ParamsMgr::search(-search_path => [],
                                      -key => "SWITCHES",
                                      -key_filter => qr/^SWITCH\d*$/,
                                      paramset => $self->get_paramset());
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw(&ParamsMgr::errmsg());
    }

    # make sure there are no duplicate switches
    my %dup_hash;
    foreach my $switch (keys(%{$switches})) {
        $dup_hash{$switches->{$switch}->{NAME}}++;
        if ($dup_hash{$switches->{$switch}->{NAME}} > 1) {
            NATE::BaseException->throw("Detected duplicate switch entries for the switch " . $switches->{$switch}->{NAME});
        }
    }

    # store them in the resource list
    foreach my $switch_elm (values %{$switches}) {
        $self->resource_list()->add_resource(category => "SWITCHES",
                                             type => "switches",
                                             data => $switch_elm);
    }
}

sub load_resource {
    my ($self, %args) = @_;

    my $staf = staf_global();

    my $switches = $self->resource_list()->get_resources(category => "SWITCHES",
                                                         type => "switches");

    # Go through the list and create an object for each one
    my @procs = ();

    foreach my $switch_resource (@{$switches}) {
        # Spawn a new process to load this switch
        my $proc = NATE::Process->new(codespec => '{ #EnGenX load ' . $switch_resource->get_field("NAME") . '
                                                     use NACL::AppBuilder::EnGenX::Switch;
                                                     NACL::AppBuilder::EnGenX::Switch::_load_switch(); }',
                                      runid => "switch__" . $switch_resource->get_field("NAME"),
                                      onexit => \&NATE::Process::on_exit_propagate_worst_result,
                                      client => 'localhost',
                                      env_to_inherit => ["ENGENX_DISABLE_MD5_VERIFICATION"],
                                      params => { "switch" => $switch_resource->to_hashref(),
                                                  "staf_parent_handle" => $staf->handle(),
                                                  "staf_parent_endpoint" => NATE::Util::Sockets::get_address(Sys::Hostname::hostname())});
        push(@procs, $proc);
    }

    # set up the Event listener and callback methods
    my $filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Switch\/SanSwitch\/\S+?\/Load_Complete/);
    };
    my $callback = sub {
        my ($msg, $listener) = @_;

        my ($switch_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Switch\/SanSwitch\/(\S+?)\/Load_Complete/);
        my $egx_switch_obj = $listener->{egx_switch_obj};

        # rebuild the SanSwitch object
        my $obj = eval($msg->{message}->{obj_serialized});
        $obj = SanSwitch->new(-obj_template => $obj);
        if ($obj->error()) {
            push(@{$egx_switch_obj->{load_errors}}, "Failed to instantiate SanSwitch object from template : " .
                     $obj->errmsg());
            $egx_switch_obj->{parent}->{listeners_complete_count}++;
            return -1;
        }

        # store the object on the sanmgr resource hash
        $egx_switch_obj->{parent}->{_objects}->{switch}->{$obj->hostname()} = $obj;

        # validate the md5 for the obj
        if (not $egx_switch_obj->_validate_obj_md5_hash(obj => $obj,
                                                       obj_md5_hex_before => $msg->{message}->{obj_md5_hex})) {
            $egx_switch_obj->{parent}->{listeners_complete_count}++;
            push(@{$egx_switch_obj->{load_errors}}, "MD5 check for switch $switch_name has failed.");
            return -1;
        }

        $egx_switch_obj->{parent}->{listeners_complete_count}++;
        return 0;
    };
    my $listener = {filter => $filter,
                    callback => $callback,
                    egx_switch_obj => $self};

    # build the fail listener
    my $fail_filter = sub {
        my $msg = shift;
        return ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Switch\/SanSwitch\/\S+\/Load_Failed/);
    };
    my $fail_callback = sub {
        my ($msg, $flistener) = @_;

        my $egx_switch_obj = $flistener->{egx_switch_obj};

        # parse out the message information
        my ($switch_name) = ($msg->{type} =~ /NACL\/AppBuilder\/EnGenX\/Switch\/SanSwitch\/(\S+?)\/Load_Failed/);
        push(@{$egx_switch_obj->{load_errors}}, "The SanSwitch object for switch $switch_name failed to instantiate.");
        $egx_switch_obj->{parent}->{listeners_complete_count}++;
        return -1;
    };

    my $fail_listener = {filter => $fail_filter,
                         callback => $fail_callback,
                         egx_switch_obj => $self};

    return ($listener, $fail_listener, @procs);
}

sub _load_switch {
    # fetch params passed into the process
    my $global_params = NATE::ParamSet->get_global();
    my $switch = $global_params->get("switch");
    my $staf_parent_handle = $global_params->get("staf_parent_handle");
    my $staf_parent_endpoint = $global_params->get("staf_parent_endpoint");

    my $parent = NATE::STAF::Peer->new(handle => $staf_parent_handle,
                                       endpoint => $staf_parent_endpoint);

    # Create switch object.
    my $sw_obj = SanSwitch->new(-host             => $switch->{NAME},
                                -conn_type        => $switch->{CONN_TYPE},
                                -admin_domain     => $switch->{ADMIN_DOMAIN},
                                -port_retries     => $switch->{PORT_RETRIES},
                                -verify_on_enable => $switch->{VERIFY_ON_ENABLE},
                                -backend_switch   => $switch->{BACKEND_SWITCH},
                                -persistent       => 1,
                                -debug_level      => 0
                            );
    if ($sw_obj->error()) {
        $parent->message_put(type => "NACL/AppBuilder/EnGenX/Switch/SanSwitch/" . $switch->{NAME} . "/Load_Failed",
                             message => {});
        NATE::BaseException->throw("Failed to create SanSwitch object for " . $switch->{NAME} . " : " .
                                   $sw_obj->errmsg());
        return;
    }

    # destroy the connection to the object,
    # we have to do this or else the object we send back to the parent gets messed up
    $sw_obj->disconnect();

    # calculate the md5 for the switch object
    my $obj_md5_hex = NACL::AppBuilder::EnGenX::Util->_calculate_obj_md5_hash($sw_obj);

    $parent->message_put(type => "NACL/AppBuilder/EnGenX/Switch/SanSwitch/" . $switch->{NAME} . "/Load_Complete",
                         message => {obj_md5_hex => $obj_md5_hex,
                                     obj_serialized => Data::Dump::dump($sw_obj)});

    return;
}

1;
