#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work with vpservers
## @author garveyj@netapp.com

package NACL::AppBuilder::EnGenX::VPserver;

# Compiler directives.
use strict;

# Module import.
use San::SanClient;
use NATE::Process;
use NATE::Log qw(log_global);
use NATE::Util::Sockets qw(get_address);
use NATE::Messenger qw(messenger_global);
use NATE::ProcManager;
use NATE::Events qw(eventloop_drain);
use NATE::BaseException qw(:try);
use Hostrec;

use NACL::C::Cluster;
use NACL::CS::NetworkInterface;
use NACL::CS::Vserver;

use Tie::IxHash;
use Data::Dumper;
use parent qw(NACL::AppBuilder::EnGenX::VPserver::Setup
              NACL::AppBuilder::EnGenX::VPserver::Cleanup
              NACL::AppBuilder::EnGenX::Util);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:    new
# Objective: Create new resource object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ( $class, %opts ) = @_;

    my $self = \%opts;
    bless( $self, $class );

    # init or reuse a rep socket and store the endpoint
    $self->{endpoint} = messenger_global()->init(socket_type => 'rep',
                                                 action      => 'bind',
                                                 reuse       => 1);

    # Save parent
    $self->{parent} = $opts{parent};

    return $self;
} ## end sub _new

#==============================================================================
# Method:     xml_to_list
# Objective:  parses the xml for vpservers
#==============================================================================
sub xml_to_list {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    # make sure there are no duplicate client resources
    my %dup_hash = ();

    my $all_vpservers = $self->_get_all_objects_paramsets("VPSERVER");
    foreach my $vpservers (@{$all_vpservers}) {
        my $vpservers_attributes;
        # We are going to add all the vpservers and their attributes, within the vpservers tag, to the ResourceList.
        while (my ($vpservers_key, $vpservers_value) = each(%{$vpservers})) {
            if (ref($vpservers_value) eq 'HASH' and $vpservers_key eq 'VPSERVER') {
                my $vpserver = {};
                while (my ($vpserver_key, $vpserver_value) =  each(%$vpservers_value)) {
                    $vpserver->{lc($vpserver_key)} = $vpserver_value;
                }
                $self->resource_list()->add_resource(
                    category => 'VPSERVER_' . $vpservers_value->{NAME},
                    type     => "vpserver",
                    data     => $vpserver,
                );
            } else {
                # add to vpservers attributes
                $vpservers_attributes->{$vpservers_key} = $vpservers_value;
            }
        }
        # add attributes to resource list
        $self->resource_list()->add_resource(
            category => "VPSERVERS",
            type => "attributes",
            data => $vpservers_attributes,
        );
    }
}

#==============================================================================
# Method:     load_resource
# Objective:  Load VPserver NACL resources into EnGenX
#==============================================================================
sub load_resource {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    # Go through all resources provided by the parent's resource list and create
    # necessary objects for each to be stored on EnGenX for later use
    my $vpserver_resources = $self->resource_list()->get_resources(category => "VPSERVER_*",
                                                                   type => "vpserver");
    if ($vpserver_resources) {
        # we need to maintain vvol provisioning metadata for each vpserver
        my $vvol_provisioning_metadata;
        foreach my $vpserver (@{$vpserver_resources}) {
            my $clusters;

            # Grab any vcenter details for the cluster/vpserver combination here.
            my $vcenter_hostrec = Hostrec->new(id => $vpserver->{vcenter}) if ($vpserver->{vcenter});
            if ($vcenter_hostrec) {
                # need to add the vcenter credentials to the provisioning metadata
                $vvol_provisioning_metadata->{vcenter}->{$vpserver->get_field('vcenter')}->{ip} =
                         $vcenter_hostrec->default_ip;
                $vvol_provisioning_metadata->{vcenter}->{$vpserver->get_field('vcenter')}->{username} =
                         $vcenter_hostrec->default_username // 'Administrator@vsphere.local';
                $vvol_provisioning_metadata->{vcenter}->{$vpserver->get_field('vcenter')}->{password} =
                         $vcenter_hostrec->default_password // 'Admin!23';
            }

            # We need to gather provisioning_metadata for the vpserver here
            my $clusters_to_use = $vpserver->{cluster_filter};
            $clusters = $self->_retrieve_and_filter_nacl_clusters(cluster_filter => $clusters_to_use);
            foreach my $cluster_obj (@{$clusters}) {
                # add this vpserver cluster combination
                $vvol_provisioning_metadata->{vpserver_cluster_map}->{$vpserver->{name}}->{$cluster_obj->name} = {};

                # Grab the defaults for the cluster
                my $cluster_node_hostrec = Hostrec->new(id => $cluster_obj->node,
                                                        hosttype => 'filer-ng-cfe');
                $vvol_provisioning_metadata->{vpserver_cluster_map}->{$vpserver->{name}}->{$cluster_obj->name}->{username} =
                        $cluster_node_hostrec->{conn}->{ssh}->{default_username} // 'admin';
                $vvol_provisioning_metadata->{vpserver_cluster_map}->{$vpserver->{name}}->{$cluster_obj->name}->{password} =
                        $cluster_node_hostrec->{conn}->{ssh}->{default_password} // 'netapp1!';

                # grab the management ip for the cluster
                my $cluster_mgmt_nic = NACL::CS::NetworkInterface->fetch(command_interface => $cluster_obj,
                                                                         filter => {role => 'cluster-mgmt'});
                $vvol_provisioning_metadata->{vpserver_cluster_map}->{$vpserver->{name}}->{$cluster_obj->name}->{ip} = $cluster_mgmt_nic->address;

                # Grab the defaults for the vpserver
                my $vpserver_hostrec = Hostrec->new(id => $vpserver->{name});
                $vvol_provisioning_metadata->{vpserver}->{$vpserver->{name}}->{username} = $vpserver_hostrec->{default_username} // 'vpserver';
                $vvol_provisioning_metadata->{vpserver}->{$vpserver->{name}}->{password} = $vpserver_hostrec->{default_password} // 'netapp1!';
                $vvol_provisioning_metadata->{vpserver}->{$vpserver->{name}}->{application} = "nacl_vpserver_" . $vpserver->{name} . "_setup";

                my $vservers_to_filter = $self->resource_list()->get_resources(category => "CLUSTER_" . $cluster_obj->name,
                                                                               type => 'vserver');

                # this is a hash of the protocols we are allowed by VPserver (we match what the user specifies against this for filtering)
                my %protocol_filters = ('fcp' => {},'iscsi' => {}, 'nfs' => {});

                my @allowed_protocols = split(',', $vpserver->{protocols});
                #####################################################################################
                # Vserver Filter Stage 1 (Filter vservers by 'vserver_filter' param)
                #####################################################################################
                my $vserver_filter = $vpserver->get_field('vserver_filter');
                my $final_vs_hash;
                foreach my $vserver (@{$vservers_to_filter}) {
                    if ($vserver->{name} =~ eval($vserver_filter)) {
                        foreach my $protocol (keys %protocol_filters) {
                            # log this as a valid protocol for this vserver if the $protocol_enabled tag is set
                            push(@{$final_vs_hash->{$vserver->{name}}->{enabled_protocols}}, $protocol) if ($vserver->{$protocol . "_enabled"});
                        }
                    }
                }

                #####################################################################################
                # Vserver Filter Stage 2 (Filter vservers by 'protocol' param)
                #####################################################################################
                # add valid protocols per vserver based on overall protocols permitted for vvol provisioing
                foreach my $vserver (keys %{$final_vs_hash}) {
                    my %original = ();
                    map { $original{$_} = 1 } @allowed_protocols;
                    my @isect = grep { $original{$_} } @{$final_vs_hash->{$vserver}->{enabled_protocols}};
                    # if the protocols for the vserver meet the filter criteria then we can include it in provisioning data
                    if (scalar(@isect)) {
                        foreach my $protocol (@isect) {
                            # metadata mapping used to create containers
                            push(@{$vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$vserver}}, $protocol);
                            # metadata mapping used to mount vvol datastores
                            push(@{$vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$protocol}}, $vserver);
                        }
                    }
                }

                #####################################################################################
                # Vserver Filter Stage 3 (Filter vservers by '(nfs|iscsi|fcp)_vserver_filter' param)
                #####################################################################################
                # map vservers to protocols they are allowed to use
                if ($vpserver->{iscsi_vserver_filter} or
                    $vpserver->{fcp_vserver_filter} or
                    $vpserver->{nfs_vserver_filter}) {
                    foreach my $vserver (keys %{$final_vs_hash}) {
                        my $vserver_protocols = $vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$vserver};

                        # build the list of protocols this vserver is restricted to
                        my @filtered_protocol_list = grep {
                            my $protocol_vserver_regex = $vpserver->{$_ . "_vserver_filter"};
                            if ($vserver =~ eval($protocol_vserver_regex)) {
                                # include
                                1;
                            } else {
                                # don't include
                                0;
                            }
                        } @{$vserver_protocols};

                        # if we have protocols that this vserver should be restricted to update the vserver protocol mappings
                        # update the list of vservers eligible for vvol provisioning
                        if (scalar(@filtered_protocol_list)) {
                            $vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$vserver} = \@filtered_protocol_list;
 
                            # modify/remove the vserver from protocol vserver mappings if it doesn't belong
                            # get list of protocols this vserver is not eligible for
#                            my @protocol_filters = split(',', $vpserver->{protocols});
                            my @protocol_exclusions;
                            foreach my $protocol (@allowed_protocols) {
                                my $matched = grep $_ eq $protocol, @filtered_protocol_list;
                                if (not $matched) {
                                    push(@protocol_exclusions, $protocol);
                                }
                            }

                            # check if this vserver is listed for a protocol it is ineligible for. if so, delete it
                            foreach my $excluded_protocol (@protocol_exclusions) {
                                my $vservers_to_filter = $vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$excluded_protocol};
                                # if this vserver is not specified for use with a disallowed vserver, move on
                                next if (!$vservers_to_filter);
                                my @filtered_vservers = grep {
                                    if ($vserver ne $_) {
                                        # include
                                        1;
                                    } else {
                                        # don't include
                                        0;
                                    }
                                } @{$vservers_to_filter};
                                # set updated protocol to vserver mapping
                                $vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}->{$excluded_protocol} = \@filtered_vservers;
                            }
                        }
                    }
                }

                ###########################################################################################################
                # Volume Filter Stage 1 (Filter out volumes on eligible vservers)
                ###########################################################################################################
                my @vservers_to_filter = keys %{$vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}};
                my $filtered_volumes = $self->_filter_volumes_by_name_and_vserver(cluster => $cluster_obj,
                                                                                  vservers_to_filter => \@vservers_to_filter,
                                                                                  volume_filter => $vpserver->{volume_filter},
                                                                                  vserver_filter => $vpserver->{vserver_filter});

                my @supported_vservers = keys %{$vvol_provisioning_metadata->{vserver_protocol_map}->{$cluster_obj->name}};
                my %volume_vserver_map;
                foreach my $volume (@{$filtered_volumes}) {
                    push(@{ $volume_vserver_map{$volume->{_cluster}}{$volume->{vserver}} }, $volume->{name});
                }
                # merge the volume vserver map with any existing map
                @{$vvol_provisioning_metadata->{volume_vserver_map}}{keys %volume_vserver_map} = values %volume_vserver_map;
                # merge the volume vserver map to self
                @{$self->{provisioning_metadata}->{$vpserver->{name}}}{keys %{$vvol_provisioning_metadata}} = values %{$vvol_provisioning_metadata};
            }
        }
    }

    $Log->exit() if $may_exit;
    return $NACL::AppBuilder::EnGenX::TRUE;
}

#==============================================================================
# Method:     _retrieve_and_filter_nacl_clusters
# Objective:  Retrieves all nacl cluster objects matching the filter
#==============================================================================
sub _retrieve_and_filter_nacl_clusters {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $cluster_filter = $args{cluster_filter};

    my @resource_clusters;
    my $cluster_categories = $self->resource_list()->get_categories(category => "CLUSTER_*");

    foreach my $cluster (@{$cluster_categories}) {
        # we should skip this tag if it not an actual cluster tag
        next if ( ($cluster eq 'CLUSTERS') or !($cluster =~ /CLUSTER_*/) );
        # strip the cluster prefix and push it to list of clusters to filter
        $cluster =~ s/CLUSTER_//;
        push(@resource_clusters, $cluster);
    }

    my @clusters = $self->{parent}->get_objects(type => "node",
                                                names => \@resource_clusters,
                                                obj_class => 'nacl');

    # if we are passed in a non-regex filter convert it to an array
    if (!($cluster_filter =~ /qr\/.*\//)) {
        $cluster_filter = [ split(',', $cluster_filter) ];
    }

    # filter out any clusters not in the resource list
    @clusters = grep {
        my $cluster_name = $_->name;
        grep {
            if ($cluster_name eq $_) {
                # include this cluster
                1;
            } else {
                # dont include this cluster
                0;
            }
        } @resource_clusters;
    } @clusters;

    # filter out the clusters that dont match the regex
    @clusters = grep {
        my $cluster_name = $_->name;
        # $cluster_name can either be an ARRAY of absolute vol names, or
        # a regular expression matching, handle each differently
        if (ref($cluster_filter) eq "ARRAY") {
            # filter out the volumes that are not in the array to filter
            grep {
                if ($cluster_name eq $_) {
                    # include this cluster
                    1;
                } else {
                    # dont include this cluster
                    0;
                }
            } @{$cluster_filter};
        } else {
            # regex filter
            if ($cluster_name =~ eval($cluster_filter)) {
                # include this cluster
                1;
            } else {
                # dont include this cluster
                0;
            }
        }
    } @clusters;

    if (!scalar(@clusters)) {
        NATE::BaseException->throw(text => "No clusters to operate on were found. Clusters must be defined for VVol provisioning.");
    }

    $Log->exit() if $may_exit;
    return \@clusters;
}

#==============================================================================
# Method:     _filter_volumes_by_name_and_vserver
# Objective:  Returns all volume resources matching the resources to filter
#==============================================================================
sub _filter_volumes_by_name_and_vserver {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $volume_hash;
    my @vservers_to_filter = @{$args{vservers_to_filter}};
    my $vserver_filter = $args{vserver_filter};
    my $volume_filter = $args{volume_filter};
    my $cluster_obj = $args{cluster};

    my @volumes_to_filter  = @{$self->resource_list()->get_resources(category => "CLUSTER_" . $cluster_obj->name,
                                                                     type => 'volume')};
    # filter out the volumes that don't match the volume filter on eligible vservers
    @volumes_to_filter = grep {
        my $vserver_name = $_->get_field('vserver');
        my $volume_name = $_->get_field('name');
        grep {
            if ($vserver_name =~ eval($vserver_filter) and
                $volume_name =~ eval($volume_filter)) {
                # include this volume
                1;
            } else {
                # dont include this volume
                0;
            }
        } @vservers_to_filter;
    } @volumes_to_filter;

    if (!scalar(@volumes_to_filter)) {
        NATE::BaseException->throw(text => "No volumes to operate on were found. Volumes must be defined for VVol provisioning.");
    }

    $Log->exit() if $may_exit;
    return \@volumes_to_filter;
}

1;
