#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

# TODO Put some of this libraries functionality into NACL STasks
# Tracking burt <insert burt number>

package NACL::AppBuilder::EnGenX::VPserver::Cleanup;

# Compiler directives.
use strict;

use NACL::AppBuilder::EnGenX::Util;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::Messenger qw(messenger_global);

# Exceptions
use NACL::C::Exceptions::VPserver::StorageAlreadyAdded;
use NATE::BaseException qw(:try);
use Hostrec;
use NATE::Util::Sockets qw(get_fqdn);
use NACL::C::VPserver;
use NACL::C::VPserver::VVolDatastore;
use NACL::C::VPserver::VCenter;
use NACL::C::VPserver::Cluster;
use NACL::C::VPserver::Container;

sub cleanup {
    my ($self, %args) = @_;
    my $run_id_prefix = delete($args{run_id_prefix}) // "";
    my $vpservers = $self->{parent}->{resource_list}->get_resources(category => "VPSERVER_*",
                                                                    type => 'vpserver');
    if (!scalar(@{$vpservers})) {
        log_global()->log("No VPservers to cleanup.");
        return;
    }

    my $vpservers_attributes = $self->{parent}->{resource_list}->get_resources(category => "VPSERVERS",
                                                                               type => 'attributes');
    # skip vpserver cleanup if it is disabled
    if ($vpservers_attributes and
        scalar(@{$vpservers_attributes}) and
        !$vpservers_attributes->[0]->get_field('CLEANUP')) {
        log_global()->log("Skipping VPservers Cleanup.");
        return;
    }

    my @procs = ();
    my $vp_msgs = {};
    foreach my $vpserver (@{$vpservers}) {
        if ($vpserver->get_field('cleanup') == 0) {
            log_global()->log("Skipping VPserver " . $vpserver->get_field('name') . " because cleanup is disabled on it.");
            next;
        }
        my $p = NATE::Process->new(runid    => $vpserver->get_field('name'),
                                   codespec => sub {
                                       try {
                                           $self->cleanup_vpserver(vpserver => $vpserver);
                                           $self->_cleanup_complete();
                                       } otherwise {
                                           my $e = shift;
                                           $self->_cleanup_failure(error => $e->text(), vpserver => $vpserver->get_field('name'));
                                           $e->throw();
                                       };
                                   },
                                   onexit   => \&NATE::Process::on_exit_propagate_worst_result,);

        push(@procs, $p);
    }

    # install a listener to handle a deconfig completion
    my $success_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/VPserver/Cleanup/Complete' },
        callback => sub {
            my ($msg, $l) = @_;
            $l->{completed}++;
        }
    };

    # install a listener to handle a deconfig failure
    my $someone_failed = 0;
    my $fail_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/VPserver/Cleanup/Failure'},
        callback => sub {
            my ($msg, $l) = @_;
            $msg = $msg->message();
            $vp_msgs->{$msg->{vpserver}}->{failed}++;
            $someone_failed = 1;
            $l->{completed}++;
        }
    };


    my $subtest_prefix = &SubIPC::_get_next_subtest_count();
    $run_id_prefix = $subtest_prefix;
    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => "VPserver-cleanup",
                           subtest_count_override => $run_id_prefix);

    # check if we had any failures
    if ($someone_failed) {
        NATE::BaseException->throw("A VPserver has hit an error in cleanup.");
    }

    return;
}

sub cleanup_vpserver {
    my ( $self, %args ) = @_;

    my $resource = $args{vpserver};
    if (!$resource) {
        NATE::BaseException->throw("No vpserver resource was passed in for cleanup.");
        return $NACL::AppBuilder::EnGenX::FALSE;
    }

    my $provisioning_metadata = $self->{provisioning_metadata}->{$resource->get_field('name')};
    if (!$provisioning_metadata) {
        NATE::BaseException->throw("No vpserver provisioning metadata was passed in for cleanup.");
        return $NACL::AppBuilder::EnGenX::FALSE;
    }

    my $engenx_obj = $self->{parent};
    if (!$engenx_obj) {
        NATE::BaseException->throw("No engenx_obj was passed in for vpserver cleanup.");
    }

    # create the vpserver object for this vpserver, using the default credential created within the component call
    my $command_interface = NACL::C::VPserver->new(vpserver => $resource->get_field('name'));
    if ( !$command_interface ) {
        NATE::BaseException->throw("No vpserver command interface was passed in for cleanup.");
    }

    my $esx_host_objs;
    if ($resource->get_field('esx_hosts')) {
        my $host_names = [split(',', $resource->get_field('esx_hosts'))];
        $esx_host_objs = $self->{parent}->get_objects(type => 'hypervisor',
                                                      names => $host_names);
    }

    ##################################################
    ### General provisioning parameters from metadata
    ##################################################
    my @esx_hosts = split(',', $resource->{esx_hosts});
    my @vpserver_clusters = keys(%{$provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}});

    $Log->comment("Removing volumes from containers on vpserver " . $resource->get_field('name') . ".");
    $self->_delete_storage(command_interface => $command_interface,
                           vpserver => $resource->get_field('name'),
                           provisioning_metadata => $provisioning_metadata);

    $Log->comment("Deleting 'nate' containers on vpserver " . $resource->get_field('name') . ".");
    $self->_delete_containers(command_interface => $command_interface,
                              vpserver => $resource->get_field('name'),
                              provisioning_metadata => $provisioning_metadata);

    # Need to delete all clusters
    if ($resource->get_field('unregister_with_ontap')) {
        foreach my $cluster (@vpserver_clusters) {
            $Log->comment("Unregistering vpserver " . $resource->get_field('name') . " with cluster $cluster.");
            my $cluster_ip = $provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}->{$cluster}->{ip};
            NACL::C::VPserver::Cluster->delete(command_interface => $command_interface,
                                               cluster_ip => $cluster_ip);
        }
    } else {
        $Log->comment("Not unregistering the vpserver from the ONTAP cluster.");
    }

    # Need to unregister with vcenter
    if ($resource->get_field('unregister_with_vcenter')) {
        $Log->comment("Unregistering vpserver " . $resource->get_field('name') . " with vcenter and vsc.");
        $command_interface->unregister_with_vcenter(vcenter_ip => $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{ip},
                                                    vcenter_username => $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{username},
                                                    vcenter_password => $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{password},
                                                  );
    } else {
        $Log->comment("Not unregistering the vpserver from vcenter/vsc.");
    }

    return $NACL::AppBuilder::EnGenX::TRUE;
} ## end sub cleanup_single

sub unmount_vvol_datastores_from_host {
    my ($self, %args) = @_;

    my $esx_host_obj = delete($args{esx_host_obj});
    my $datastore_regex_filter = delete($args{datastore_regex_filter}) // qr/nate_(iscsi|fcp|nfs)_container/;
    my $name = $esx_host_obj->hostname();

    # get whatever vpserver resources exist
    my $vpservers = $self->{parent}->{resource_list}->get_resources(category => "VPSERVER_*",
                                                                    type => 'vpserver');
    if (!scalar(@{$vpservers})) {
        $Log->log("No VPservers registered as resources, skipping vvol datastore unmount phase.");
        return;
    }

    # pick the vpserver that is associated to this esx host
    my $vpserver;
    foreach my $vp (@$vpservers) {
        my @esx_hosts = split(',', $vp->{esx_hosts});
        if (grep (/^$name$/, @esx_hosts)) {
            $vpserver = $vp;
            last;
        }
    }
    if (!$vpserver) {
        $Log->log("No VPservers are associated to ESX Host $name");
        return;
    }

    my $vpserver_name = $vpserver->get_field('name');
    my $vcenter = $vpserver->get_field('vcenter');

    # create the vpserver object for this vpserver
    my $command_interface = NACL::C::VPserver->new(vpserver => $vpserver_name);
    if (!$command_interface) {
        NATE::BaseException->throw("Could not create vpserver component object for $vpserver_name");
    }

    # get the esx host moref for this host
    my $esx_host_data = $esx_host_obj->get_host_morefs(vcenter => $vcenter);

    my $esx_host_moref;
    my $hostrec = Hostrec->new(id => $name);
    my $ip = $hostrec->default_ip();
    my $fqdn = get_fqdn($ip)->[0]->{fqdn};
    foreach ($name, $ip, $fqdn) {
        $esx_host_moref = $esx_host_data->{$_};
        next if (!$esx_host_moref);
    }
    log_global()->log("ESX Host $name is not a host of vcenter $vcenter") and
        return if (!$esx_host_moref);

    # get all the datastores that are of type vvol on this vcenter
    my $datastore_info = $esx_host_obj->list_all_datastores(vcenter => $vcenter);
    log_global()->log("No vvol datastores to unmount for host $name on vcenter $vcenter") and
        return if (! exists $datastore_info->{vvol});

    foreach my $datastore (@{$datastore_info->{vvol}}) {
        next if ($datastore->{label} !~ $datastore_regex_filter);
        $Log->log("Unmounting vvol datastore " . $datastore->{label} . " from esx host $name (moref:$esx_host_moref)");
        NACL::C::VPserver::VVolDatastore->unmount_vvol_datastore_on_host(
            command_interface => $command_interface,
            host => $esx_host_moref,
            datastore => $datastore->{moref}
        );
    }
}

sub _delete_storage {
    my ( $self, %args ) = @_;

    my $provisioning_metadata = $args{provisioning_metadata};
    my $storage_list = NACL::C::VPserver::Container->list_storage(command_interface => $args{command_interface});
    foreach my $storage (@{$storage_list}) {
        # match by vpserver name and delete
        if ($storage->{container}->{name} =~ m/nate_(iscsi|fcp|nfs)_container_\d+/) {
            my $container = $storage->{container}->{name};
            my $vserver = $storage->{storageLocation}->{vserverName};
            my $volume = $storage->{storageLocation}->{flexVolName};
            my $cluster = $storage->{storageLocation}->{clusterIp};
            my $cluster_ip = $provisioning_metadata->{vpserver_cluster_map}->{$args{vpserver}}->{$cluster}->{ip};
            $Log->log("Removing storage $cluster:$vserver://$volume from container $container");
            NACL::C::VPserver::Container->delete_storage(command_interface => $args{command_interface},
                                                         container => $container,
                                                         cluster => $cluster,
                                                         vserver => $vserver,
                                                         volume => $volume);
         }
    }
    return $NACL::AppBuilder::EnGenX::TRUE;
}

sub _delete_containers {
    my ( $self, %args ) = @_;

    my $containers = NACL::C::VPserver::Container->list(command_interface => $args{command_interface});
    foreach my $container (@{$containers}) {
        # match by vpserver name and delete
        if ($container->{name} =~ /nate_(iscsi|fcp|nfs)_container_\d+/) {
            $Log->log("Deleting container " . $container->{name});
            NACL::C::VPserver::Container->delete(command_interface => $args{command_interface},
                                                 container => $container->{name});
         }
    }
    return $NACL::AppBuilder::EnGenX::TRUE;
}

sub _cleanup_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/VPserver/Cleanup/Failure",
                            message  => \%data);
}

sub _cleanup_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/VPserver/Cleanup/Complete",
                            message  => \%data);
}

1;
