#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

# TODO Put some of this libraries functionality into NACL STasks
# Tracking burt <insert burt number>

package NACL::AppBuilder::EnGenX::VPserver::Setup;

# Compiler directives.
use strict;
use warnings;

use Tie::IxHash;
use POSIX qw(ceil);
use Digest::MD5 qw(md5);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::Messenger qw(messenger_global);
use NACL::CS::VserverExportPolicyRule;
use NACL::CS::LunIgroup;
use NACL::CS::Volume;

# Exceptions
use NACL::C::Exceptions::VPserver::StorageAlreadyAdded;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::Exceptions::NoElementsFound;
use NATE::BaseException qw(:try);

sub setup {
    my ($self, %args) = @_;
    my $run_id_prefix = delete($args{run_id_prefix}) // "";

    my $vpservers = $self->{parent}->{resource_list}->get_resources(category => "VPSERVER_*",
                                                                    type => 'vpserver');
    if (!scalar(@{$vpservers})) {
        log_global()->log("No VPservers to setup.");
    }

    my $vpservers_attributes = $self->{parent}->{resource_list}->get_resources(category => "VPSERVERS",
                                                                               type => 'attributes');
    # skip vpserver setup if it is disabled
    if ($vpservers_attributes and
        scalar(@{$vpservers_attributes}) and
        !$vpservers_attributes->[0]->get_field('SETUP')) {
        log_global()->log("Skipping VPservers Setup.");
        return;
    }

    my @procs = ();
    my $vp_msgs = {};
    foreach my $vpserver (@{$vpservers}) {
        if ($vpserver->get_field('setup') == 0) {
            log_global()->log("Skipping VPserver " . $vpserver->get_field('name') . " because setup is disabled on it.");
            next;
        }
        my $p = NATE::Process->new(runid    => $vpserver->get_field('name'),
                                   codespec => sub {
                                       try {
                                           $self->setup_vpserver(resource => $vpserver);
                                           $self->_setup_complete();
                                       } otherwise {
                                           my $e = shift;
                                           $self->_setup_failure(error => $e->text(), vpserver => $vpserver->get_field('name'));
                                           $e->throw();
                                       };
                                   },
                                   onexit   => \&NATE::Process::on_exit_propagate_worst_result,);
        push(@procs, $p);
    }

    # install a listener to handle a config completion
    my $success_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/VPserver/Setup/Complete' },
        callback => sub {
            my ($msg, $l) = @_;
            $l->{completed}++;
        }
    };

    # install a listener to handle a config failure
    my $someone_failed = 0;
    my $fail_listener = {
        completed => 0,
        filter => sub { shift->type() eq 'NACL/AppBuilder/EnGenX/VPserver/Setup/Failure' },
        callback => sub {
            my ($msg, $l) = @_;
            $msg = $msg->message();
            $vp_msgs->{$msg->{vpserver}}->{failed}++;
            $someone_failed = 1;
            $l->{completed}++;
        }
    };

    my $subtest_prefix = &SubIPC::_get_next_subtest_count();
    $run_id_prefix = $subtest_prefix;
    $self->_spawn_routines(procs => \@procs,
                           fail_listener => $fail_listener,
                           success_listener => $success_listener,
                           runid => "VPserver-setup",
                           subtest_count_override => $run_id_prefix);

    # check if we had any failures
    if ($someone_failed) {
        NATE::BaseException->throw("A VPserver has hit an error during setup.");
    }

    return;
}

sub setup_vpserver {
    my ( $self, %args ) = @_;

    # we have to requires nacl vvol component libraries here. Smokes is not using NATE Version 5.3
    # NATE Version 5.3 and on contains the NATE::SOAP::Data, and NATE::SOAP libraries that the
    # nacl vvol componets use. CIT's will fail if we blindly import vvol component libraries
    require NACL::C::VPserver;
    require NACL::C::VPserver::VCenter;
    require NACL::C::VPserver::Cluster;
    require NACL::C::VPserver::Container;
    require NACL::C::VPserver::VVolDatastore;

    my $resource = $args{resource};
    if (!$resource) {
        NATE::BaseException->throw("No vpserver resource was passed in for setup.");
        return $NACL::AppBuilder::EnGenX::FALSE;
    }

    my $provisioning_metadata = $self->{provisioning_metadata}->{$resource->get_field('name')};
    if (!$provisioning_metadata) {
        NATE::BaseException->throw( "No vpserver provisioning metadata was passed in for setup.");
        return $NACL::AppBuilder::EnGenX::FALSE;
    }

    my $timeout_override = $resource->get_field('timeout') // 180;
    # create the vpserver object for this vpserver, using the default credential created within the component call
    my $command_interface = NACL::C::VPserver->new(vpserver => $resource->get_field('name'),
                                                   timeout => $timeout_override);
    if (!$command_interface) {
        NATE::BaseException->throw("No vpserver command interface was passed in for setup.");
        return $NACL::AppBuilder::EnGenX::FALSE;
    }

    ##################################################
    ### General provisioning parameters from metadata
    ##################################################
    my $balanced_vol_vserver_map = $self->_get_balanced_volume_vserver_map(provisioning_metadata => $provisioning_metadata);
    $self->{provisioning_metadata}->{$resource->get_field('name')}->{volume_vserver_map} = $balanced_vol_vserver_map;

    my $max_vols_per_container = $resource->get_field('max_vols_per_container');
    my @vpserver_clusters = keys(%{$provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}});

    my $host_names;
    my $esx_host_objs;
    if ($resource->get_field('esx_hosts')) {
        $host_names = [split(',', $resource->get_field('esx_hosts'))];
        $esx_host_objs = $self->{parent}->get_objects(type => 'hypervisor',
                                                      names => $host_names);
    }

    my $vp_registered_clusters = NACL::C::VPserver::Cluster->list(command_interface => $command_interface);

    # Perform VPserver setup for each cluster
    foreach my $cluster (@vpserver_clusters) {
        my $vservers_by_protocol = $provisioning_metadata->{vserver_protocol_map}->{$cluster};
        my $cluster_ip = $provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}->{$cluster}->{ip};
        my $cluster_username = $provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}->{$cluster}->{username};
        my $cluster_password = $provisioning_metadata->{vpserver_cluster_map}->{$resource->get_field('name')}->{$cluster}->{password};
        my $vcenter_ip = $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{ip};
        my $vcenter_username = $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{username};
        my $vcenter_password = $provisioning_metadata->{vcenter}->{$resource->get_field('vcenter')}->{password};
        my $vsc_ip = $resource->get_field('vsc');

        $Log->comment("Registering vpserver " . $resource->get_field('name') .
                      " with vCenter $vcenter_ip ($vcenter_username : $vcenter_password) and VSC ($vsc_ip).");
        my $attempts = 0;
        my $max_retries = 5;
        my $success = 0;
        while (!$success and $attempts < $max_retries) {
            $attempts++;
            # Need to first register the VP with the VSC and Vcenter servers
            try {
                $command_interface->register_with_vcenter_vsc(vcenter_ip => $vcenter_ip,
                                                              vcenter_username => $vcenter_username,
                                                              vcenter_password => $vcenter_password,
                                                              vsc_ip => $vsc_ip);
                $success = 1;
            } otherwise {
                my $e = shift;
                # retry if we time out
                if ($e->text() =~ /timed out|timeout/i) {
                    if ($attempts < $max_retries) {
                        # sleep and retry
                        sleep 2;
                    }
                } else {
                    $e->throw();
                }
            };
        }

        if (!$success) {
            NATE::BaseException->throw("Could not register vpserver with the vCenter and VSC provided");
        }

        # Add the cluster to the VP only if it isn't already added
        # adding clusters after they have already been added can cause waittimes above 10 minutes
        my $cluster_found = 0;
        foreach my $vp_cluster (@{$vp_registered_clusters}) {
            if ($vp_cluster->{name} eq $cluster) {
                $cluster_found = 1;
                last;
            }
        }
        $timeout_override = $resource->get_field('timeout') // 300;
        if (!$cluster_found) {
            # register the cluster
            $Log->comment("Registering cluster $cluster with vpserver " . $resource->get_field('name') . ".");
            NACL::C::VPserver::Cluster->add(command_interface => $command_interface,
                                            cluster_ip => $cluster_ip,
                                            cluster_username => $cluster_username,
                                            cluster_password => $cluster_password,
                                            timeout => $timeout_override);
        } else {
            # rediscover the cluster
            $Log->comment("Rediscovering cluster $cluster");
            NACL::C::VPserver::Cluster->rediscover(command_interface => $command_interface,
                                                   cluster_ip => $cluster_ip,
                                                   timeout => $timeout_override);
        }

        # Create some containers
        $Log->comment("Creating containers for cluster $cluster with vpserver " . $resource->get_field('name') . ".");
        my $created_containers = $self->_create_containers(command_interface => $command_interface,
                                                           provisioning_metadata => $provisioning_metadata,
                                                           vpserver => $resource->get_field('name'),
                                                           cluster => $cluster,
                                                           max_vols_per_container => $max_vols_per_container);

        my $existing_storage = NACL::C::VPserver::Container->list_storage(command_interface => $command_interface);

        # Add storage
        $Log->comment("Adding Storage to containers for cluster $cluster with vpserver " . $resource->get_field('name') . ".");
        my $containers_to_delete = $self->_add_storage_to_containers(command_interface => $command_interface,
                                                                     provisioning_metadata => $provisioning_metadata,
                                                                     vpserver_resource => $resource,
                                                                     created_containers => $created_containers,
                                                                     existing_storage => $existing_storage,
                                                                     cluster => $cluster);

        $existing_storage = NACL::C::VPserver::Container->list_storage(command_interface => $command_interface);
        foreach my $protocol (keys %{$containers_to_delete}) {
            foreach my $container (@{$containers_to_delete->{$protocol}}) {
                # remove this container from the list
                my @filtered_containers = @{$created_containers->{$protocol}};
                @filtered_containers = grep {
                                           if ($container ne $_) {
                                               # include
                                               1;
                                           } else {
                                               # don't include
                                               0;
                                           }
                                       } @filtered_containers;
                $created_containers->{$protocol} = \@filtered_containers;
            }
        }

        if ($esx_host_objs) {
            my @protocols_to_use = split(',', $resource->{protocols});

            # grab any new storage we may have added
            $existing_storage = NACL::C::VPserver::Container->list_storage(command_interface => $command_interface);
            my $initiators_by_protocol = $self->_get_initiators_by_protocol(hosts => $esx_host_objs,
                                                                            protocols_to_use => \@protocols_to_use);

            $Log->comment("Ensuring Protocol Endpoints for cluster $cluster with vpserver " . $resource->get_field('name') . ".");
            # Ensure Protocol Endpoints
            $self->_ensure_protocol_endpoints(command_interface => $command_interface,
                                              provisioning_metadata => $provisioning_metadata,
                                              vpserver => $resource->get_field('name'),
                                              cluster => $cluster,
                                              containers => $created_containers,
                                              initiators_by_protocol => $initiators_by_protocol);

            $Log->comment("Mounting Vvol datastores to hosts for cluster $cluster with vpserver " . $resource->get_field('name') . ".");
            # Mount a specific container to a datastore
            $self->_mount_vvol_datastores(command_interface => $command_interface,
                                          resource => $resource,
                                          provisioning_metadata => $provisioning_metadata,
                                          cluster => $cluster,
                                          hosts => $esx_host_objs,
                                          created_contaners => $created_containers,
                                          existing_storage => $existing_storage,
                                          initiators_by_protocol => $initiators_by_protocol);
        } else {
            $Log->comment("No esx hosts exists to ensure protocol endpoints for or to mount vvol datastores to.");
        }

        # remove containers marked for removal
        foreach my $protocol (keys %{$containers_to_delete}) {
            foreach my $container (@{$containers_to_delete->{$protocol}}) {
                NACL::C::VPserver::Container->delete(command_interface => $args{command_interface},
                                                     container => $container);
            }
        }
    }
    return $NACL::AppBuilder::EnGenX::TRUE;
} ## end sub setup_single

sub _create_containers {
    my ( $self, %args ) = @_;

    my $provisioning_metadata =  $args{provisioning_metadata};
    my $max_vols_per_container = $args{max_vols_per_container};
    my $cluster = $args{cluster};

    my %protocol_hash = (fcp => 0, iscsi => 0, nfs => 0);
    my %created_containers;
    tie %created_containers, 'Tie::IxHash';

    # determine how many containers to create
    foreach my $vserver (keys %{$provisioning_metadata->{volume_vserver_map}->{$cluster}}) {
        my $volume_count = scalar(keys($provisioning_metadata->{volume_vserver_map}->{$cluster}->{$vserver}));
        my $protocols = $provisioning_metadata->{vserver_protocol_map}->{$cluster}->{$vserver};
        my $container_count;
        # all volumes are one protocol
        if (scalar(@{$protocols}) == 1) {
            $container_count = $self->_calculate_container_count(max_vols_per_container => $max_vols_per_container,
                                                                 volume_count => $volume_count);
            $protocol_hash{shift(@{$protocols})} += $container_count;
        }
        # all volumes support two or three protocols
        else {
            my $protocol_count = scalar(@{$protocols});
            my $reserved_volume_count = $volume_count / $protocol_count;
            $volume_count = ceil($reserved_volume_count);
            foreach my $protocol (@{$protocols}) {
                $container_count = $self->_calculate_container_count(max_vols_per_container => $max_vols_per_container,
                                                                     volume_count => $volume_count);
                $protocol_hash{$protocol} += $container_count;
            }
        }
    }

    # create containers
    foreach my $protocol (keys %protocol_hash) {
        next if (!$protocol_hash{$protocol});
        for (my $i = 0; $i < $protocol_hash{$protocol}; $i++) {
            my $container_name = "nate_" . $protocol . "_container_$i";
            push(@{$created_containers{$protocol}}, $container_name);
            my $response = NACL::C::VPserver::Container->create(command_interface => $args{command_interface},
                                                                container => $container_name,
                                                                protocol => $protocol);
        }
    }
    return \%created_containers;
}

sub _calculate_container_count {
    my ( $self, %args ) = @_;

    my $container_count = $args{volume_count} / $args{max_vols_per_container};
    # make sure we grab the ceiling of this division
    $container_count = ceil($container_count);

    # must increment if we have volumes remaining, that need an additional container to hold them
    $container_count++ if($args{volume_count} % $args{max_vols_per_container});

    return $container_count;
}

sub _add_storage_to_containers {
    my ( $self, %args ) = @_;

    my $created_containers = $args{created_containers};
    my $provisioning_metadata = $args{provisioning_metadata};
    my $vpserver = $args{vpserver_resource};
    my $cluster_ip = $provisioning_metadata->{vpserver_cluster_map}->{$vpserver->{name}}->{$args{cluster}}->{ip};
    my $existing_storage = $args{existing_storage};

    # we need a copy of volume vserver map, we need to make modifications to
    # the hash to keep track of volumes that have been added
    my %volume_vserver_map = %{$provisioning_metadata->{volume_vserver_map}->{$args{cluster}}};

    my $container_vol_count;
    my @protocols = split(',', $vpserver->{protocols});
    foreach my $protocol (@protocols) {
       # skip if we don't have any vservers of this protocol
       next if (!$provisioning_metadata->{vserver_protocol_map}->{$args{cluster}}->{$protocol});
       # start adding storage from all vservers of a particular protocol
        my @protocol_vservers = @{$provisioning_metadata->{vserver_protocol_map}->{$args{cluster}}->{$protocol}};
        # get all of the containers of this protocol
        my @containers = @{$created_containers->{$protocol}} if (exists($created_containers->{$protocol}));

        foreach my $container (@containers) {
            my $volumes;
            my $vserver;
            # find the next vserver that has volumes available to add and use that one
            # we are shifting to round robin, and we have a for loop to ensure we look at the whole list
            for (my $i = 0; $i < scalar(@protocol_vservers); $i++) {
                # round-robin the vservers so we don't pile all containers of one protocol in one vserver
                $vserver = shift(@protocol_vservers);
                push(@protocol_vservers, $vserver);
                # get all the volumes for the vserver of this protocol
                $volumes = $volume_vserver_map{$vserver};
                if ($volumes and scalar(@{$volumes})) {
                    last;
                }
            }

            # add as much storage, from this vserver, that we are allowed to the container, or until we run out of storage
            while ($volumes and scalar(@{$volumes})) {
                # if this container has the maximum amount of volumes then we should stop adding volumes
                if (defined($container_vol_count->{$container}) and
                    $container_vol_count->{$container} >= $vpserver->get_field('max_vols_per_container')) {
                    last;
                }
                # grab a volume to add to this container
                my $volume = shift(@{$volumes});
                try {
                    $container_vol_count->{$container}++;
                    $Log->log("Adding volume [cluster:$cluster_ip, vserver:$vserver, volume:$volume] to container $container");
                    NACL::C::VPserver::Container->add_storage(command_interface => $args{command_interface},
                                                              container => $container,
                                                              cluster => $cluster_ip,
                                                              volume => $volume,
                                                              vserver => $vserver);
                } catch NACL::C::Exceptions::VPserver::StorageAlreadyAdded with {
                    # check to see if we are attempting to add the container to the same storage again
                    my $e = shift;
                    my $mapped_to_this_container = 0;
                    foreach my $storage_data (@{$existing_storage}) {
                        if ($container eq $storage_data->{container}->{name} and
                            $cluster_ip eq $storage_data->{storageLocation}->{clusterIp} and
                            $volume eq $storage_data->{storageLocation}->{flexVolName} and
                            $vserver eq $storage_data->{storageLocation}->{vserverName}) {
                            # this volume is attempting to be readded to it's existing container location
                            # we should not error here
                            $mapped_to_this_container = 1;
                            last;
                        }
                    }
                    if (!$mapped_to_this_container) {
                       # rethrow exception
                       $e->throw();
                    }
                } otherwise {
                    my $e = shift;
                    $e->throw();
                };
            }
        }
    }

    # we should delete any containers without storage; so they don't get mounted
    my %containers_to_delete;
    foreach my $protocol (@protocols) {
        foreach my $container (@{$created_containers->{$protocol}}) {
            if (not defined($container_vol_count->{$container})) {
                push(@{$containers_to_delete{$protocol}}, $container);
            }
        }
    }
    return \%containers_to_delete;
}

sub _ensure_protocol_endpoints {
    my ( $self, %args ) = @_;

    my $provisioning_metadata = $args{provisioning_metadata};
    my $initiators = $args{initiators_by_protocol};
    my $cluster_ip = $provisioning_metadata->{vpserver_cluster_map}->{$args{vpserver}}->{$args{cluster}}->{ip};
    my $created_containers = $args{containers};

    # We need to seperate hosts to what protocol they should be ensured for
    foreach my $host (keys %{$initiators}) {
        my $host_initiators = $initiators->{$host};
        foreach my $protocol (keys %{$host_initiators}) {
            my $vservers = $provisioning_metadata->{vserver_protocol_map}->{$args{cluster}}->{$protocol};
            foreach my $vserver (@{$vservers}) {
                # we need to ensure each container we created
                if ($created_containers->{$protocol}) {
                    foreach my $container (@{$created_containers->{$protocol}}) {
                        foreach my $host_initiator_or_ip (@{$host_initiators->{$protocol}}) {
                            $Log->log("Ensuring Protocol Endpoint: " . $args{cluster} . ":$vserver on " .
                                      "container $container (protocol:$protocol) (initiator:$host_initiator_or_ip)");
                            NACL::C::VPserver::Cluster->ensure_pe(command_interface => $args{command_interface},
                                                                  cluster_ip => $cluster_ip,
                                                                  vserver => $vserver,
                                                                  container => $container,
                                                                  protocol => $protocol,
                                                                  hosts => [$host_initiator_or_ip]);
                        }
                    }
                };
            }
        }
    }
    return $NACL::AppBuilder::EnGenX::TRUE;
}

sub _get_initiators_by_protocol {
    my ( $self, %args ) = @_;

    my $esx_hosts = $args{hosts};
    my $protocols_to_use = $args{protocols_to_use};
    my $initiators_by_protocol;

    foreach my $host (@{$esx_hosts}) {
        my $initiators = $host->get_initiators();
        foreach my $initiator (@{$initiators}) {
            # get protocol
            my $protocol = $initiator->get_protocol();
            if ( not (grep {$protocol eq $_} @{$protocols_to_use}) ) {
                # if we aren't filtering to use this protocol we can skip adding this initiator
                next;
            }

            my $initiator_name;
            if ($protocol eq 'fcp') {
                $initiator_name = $self->normalize_fcp_wwn(wwn => $initiator->get_portname);
            } else {
                $initiator_name = $initiator->get_nodename();
            }

            push(@{$initiators_by_protocol->{$host->{phy_host}}->{$protocol}}, $initiator_name);
        }
    }

    # handle initiator/protocol mappings for nfs
    if (grep {'nfs' eq $_} @{$protocols_to_use}) {
        foreach my $host (@{$esx_hosts}) {
            # reconnect to the esx host
            $host->reconnect();
            my $network_interfaces = $host->get_ip_info();
            foreach my $network_interface (keys %{$network_interfaces}) {
                push(@{$initiators_by_protocol->{$host->{phy_host}}->{nfs}}, $network_interfaces->{$network_interface}->{ipv4});
            }
        }
    }
    return $initiators_by_protocol;
}

sub _mount_vvol_datastores {
    my ( $self, %args ) = @_;

    my $provisioning_metadata = $args{provisioning_metadata};
    my $created_containers = $args{created_containers};
    my $initiators_by_protocol = $args{initiators_by_protocol};
    my $existing_storage = $args{existing_storage};
    my $resource = $args{resource};
    my $cluster = $args{cluster};
    my $esx_hosts = $args{hosts};

    $esx_hosts->[0]->reconnect();

    # Get all of the host morefs
    my $host_morefs = $esx_hosts->[0]->get_host_morefs(vcenter => $resource->{vcenter});
    my $network_interfaces = $esx_hosts->[0]->get_ip_info();

    # Get all the container id's
    my $container_ids = $self->_get_container_ids();

    my $cluster_ci = $self->{parent}->get_nacl_objects(category => $cluster,
                                                       type => 'cluster');

    # Get Vserver initiator mapping
    my $vserver_initiator_map = $self->_generate_vvol_vserver_initiator_map(command_interface => $cluster_ci->[0],
                                                                            initiators_by_protocol => $initiators_by_protocol);
    # Get list of compatible containers
    my $compatible_containers = $self->_get_compatible_containers_by_host(hosts => $esx_hosts,
                                                                          initiators_by_protocol => $initiators_by_protocol,
                                                                          vserver_initiator_map => $vserver_initiator_map,
                                                                          network_interfaces => $network_interfaces,
                                                                          existing_storage => $existing_storage);
    my @container_names = keys %{$container_ids};
    my $existing_datastores = $self->_get_datastores_by_container(host => $esx_hosts->[0],
                                                                  vcenter => $resource->{vcenter},
                                                                  containers => \@container_names);
    # mount every compatible vvol datastore to each host
    foreach my $host (@{$esx_hosts}) {
        my $host_moref = $host_morefs->{$host->{esxi_host_fqdn}};
        my $initiator_objs = $host->get_initiators();
        # mount each iscsi/fcp initiator for this host to a datastore on a compatible container
        foreach my $initiator (@{$initiator_objs}) {
            my $initiator_name;
            if ($initiator->get_protocol() eq 'iscsi') {
                $initiator_name = $initiator->get_nodename();
            } else {
                $initiator_name = $self->normalize_fcp_wwn(wwn => $initiator->get_portname);
            }
            # iterate the vservers this iqn is mapped to
            foreach my $vserver (@{$vserver_initiator_map->{$initiator_name}}) {
                # skip if there isn't a compatible container for this vserver/initiator combination
                next if (!$compatible_containers->{$vserver}->{$initiator->get_protocol()});

                foreach my $container (@{$compatible_containers->{$vserver}->{$initiator->get_protocol()}}) {
                    my $container_id = $container_ids->{$container};
                    my $uuid = Digest::MD5::md5_hex($args{cluster} . "__${vserver}");

                    my $datastore = $container . "_ds_$uuid";
                    if ($existing_datastores->{$container}) {
                        # since there is currently a 1:1 mapping between container and datastore
                        # grab the first/only datastore for this container
                        $datastore = $existing_datastores->{$container}->[0];
                    }
                    $self->_mount_vvol_datastore(command_interface => $args{command_interface},
                                                 host => $host->{phy_host},
                                                 host_moref => $host_moref,
                                                 container => $container,
                                                 container_id => $container_id,
                                                 existing_datastores => $existing_datastores,
                                                 datastore => $datastore);
                }
            }
        }

        foreach my $network_interface (keys %{$network_interfaces}) {
            my $host_ip = $network_interfaces->{$network_interface}->{ipv4};
            # iterate the vservers this ip is mapped to
            foreach my $vserver (@{$vserver_initiator_map->{$host_ip}}) {
                # skip if there isn't a compatible container for this vserver/initiator combination
                next if (!$compatible_containers->{$vserver}->{nfs});
                foreach my $container (@{$compatible_containers->{$vserver}->{nfs}}) {
                    my $container_id = $container_ids->{$container};
                    my $uuid = Digest::MD5::md5_hex($args{cluster} . "__${vserver}");

                    my $datastore = $container . "_ds_$uuid";
                    if ($existing_datastores->{$container}) {
                        # since there is currently a 1:1 mapping between container and datastore
                        # grab the first/only datastore for this container
                        $datastore = $existing_datastores->{$container}->[0];
                    }
                    $self->_mount_vvol_datastore(command_interface => $args{command_interface},
                                                 host => $host->{phy_host},
                                                 host_moref => $host_moref,
                                                 container => $container,
                                                 container_id => $container_id,
                                                 existing_datastores => $existing_datastores,
                                                 datastore => $datastore);
                }
            }
        }
    }
    return $NACL::AppBuilder::EnGenX::TRUE;
}

sub _mount_vvol_datastore {
    my ( $self, %args ) = @_;

    my $existing_datastores = $args{existing_datastores};

    # retry variables
    my $mount_wait_interval = 5;
    my $max_cumulative_waittime = 650;
    my $exception;
    my $success = 0;

    $Log->comment("Mounting container " . $args{container_id} .
                  " as datastore " . $args{datastore} .
                  " on host: " . $args{host} . " (" . $args{host_moref} . ").");
    while (!$success and $mount_wait_interval < $max_cumulative_waittime) {
        try {
            NACL::C::VPserver::VVolDatastore->mount_vvol_datastore_on_host(command_interface => $args{command_interface},
                                                                           host_moref => $args{host_moref},
                                                                           container_id => $args{container_id},
                                                                           datastore => $args{datastore});
            push(@{$existing_datastores->{$args{container}}}, $args{datastore}) if (!$existing_datastores->{$args{container}});
            $success = 1;
        } catch NACL::C::Exceptions::VPserver::VVolDatastore::AlreadyExists with {
            # We've already mounted this datastore. Do nothing
            $success = 1;
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            # In larger environments, in some cases, when we attempt to mount vvol datastores the memoryspace on the vcenter
            # doing the mounting gets consumed and the internals start to slow down. When this happens, the vcenter fails
            # to load some apis, and the vpserver call results in the following exception message being thrown:
            # "vCenter API NotFound Exception , , occurred on the server.  See VP server logs for more details."
            # Retrying at staggered intervals seems to work in this instance.
            $exception = shift;
            if ($mount_wait_interval >= $max_cumulative_waittime) {
                # abort
                $exception->throw();
            } else {
                # sleep and retry
                $Log->comment("Failed to mount container " . $args{container_id} .
                              " as datastore " . $args{datastore} .
                              " to host " . $args{host} . " (" . $args{host_moref} . "). " .
                              "Waiting $mount_wait_interval seconds before retrying.");
                sleep($mount_wait_interval);
                $mount_wait_interval *= 2;
            }
        };
    }
}

sub _get_compatible_containers_by_host {
    my ( $self, %args ) = @_;

    my $esx_hosts = $args{hosts};
    my $existing_storage = $args{existing_storage};
    my $vserver_initiator_map = $args{vserver_initiator_map};
    my $initiators_by_protocol = $args{initiators_by_protocol};
    my $network_interfaces = $args{network_interfaces};

    my %compatible_containers;
    tie %compatible_containers, 'Tie::IxHash';
    foreach my $host (@{$esx_hosts}) {
        # Grab compatible containers for using to mount a vvol datastore to the host with this initiator
        # The criteria for a compatible container is:
        # 1) The container must have the same protocol as the current initiator
        # 2) The container must have some storage (flexvols) that exists on, at least, one vserver
        #    which has, at least, one igroup with this host mapped to it.
        foreach my $initiator ($host->get_initiators()) {
            my $protocol = $initiator->get_protocol();
            # Skip mounting of datastore for this initiator if it is not of a protocol that we're operating on
            if (!$initiators_by_protocol->{$host->{phy_host}}->{$protocol}) {
                next;
            }

            my $initiator_name;
            if ($protocol eq 'iscsi') {
                $initiator_name = $initiator->get_nodename;
            } else {
                $initiator_name = $self->normalize_fcp_wwn(wwn => $initiator->get_portname);
            }

            # Create a list of all compatible containers for this iniitiator
            foreach my $storage (@{$existing_storage}) {
                my $vserver = $storage->{storageLocation}->{vserverName};
                my $storage_protocol = $storage->{container}->{protocol};
                my $container_name = $storage->{container}->{name};
                # Containers may only contain flexvolumes from a single vserver
                # We are checking the two criteria described in the comments above
                if (($protocol eq $storage_protocol) and
                    exists($vserver_initiator_map->{$vserver}->{$initiator_name}) and
                    !(grep {$container_name eq $_} @{$compatible_containers{$vserver}{$protocol}})) {
                    push(@{$compatible_containers{$vserver}{$protocol}}, $container_name);
                }
            }
        }

        # do out of band verification for nfs too, if we are operating on nfs protocols
        if ($initiators_by_protocol->{$host->{phy_host}}->{nfs}) {
            foreach my $network_interface (keys %{$network_interfaces}) {
                my $host_ip = $network_interfaces->{$network_interface}->{ipv4};
                # Create a list of all compatible containers for this iniitiator
                foreach my $storage (@{$existing_storage}) {
                    my $vserver = $storage->{storageLocation}->{vserverName};
                    my $storage_protocol = $storage->{container}->{protocol};
                    my $container_name = $storage->{container}->{name};
                    # Containers may only contain flexvolumes from a single vserver
                    # We are checking the two criteria described in the comments above
                    if (('nfs' eq $storage_protocol) and
                        exists($vserver_initiator_map->{$vserver}->{$host_ip}) and
                        !(grep {$container_name eq $_} @{$compatible_containers{$vserver}{nfs}})) {
                        push(@{$compatible_containers{$vserver}{nfs}}, $container_name);
                    }
                }
            }
        }
    }
    return \%compatible_containers;
}

sub _get_datastores_by_container {
    my ( $self, %args ) = @_;

    my %container_datastores;
    my @vvol_datastores;
    my $datastore_info = $args{host}->list_all_datastores(vcenter => $args{vcenter});
    foreach my $ds_info (@{$datastore_info->{vvol}}) {
        push(@vvol_datastores, $ds_info->{label});
    }

    foreach my $datastore (@vvol_datastores) {
        foreach my $container (@{$args{containers}}) {
            if ($datastore =~ /$container/) {
                push(@{$container_datastores{$container}}, $datastore);
            }
        }
    }
    return \%container_datastores;
}

sub _generate_vvol_vserver_initiator_map {
    my ( $self, %args ) = @_;

    my $initiators_by_protocol = $args{initiators_by_protocol};
    # Get an array of initiator names for each host by protocol
    my @initiator_filter;
    my @nfs_ip_filter;
    foreach my $host (keys %{$initiators_by_protocol}) {
        foreach my $protocol (keys %{$initiators_by_protocol->{$host}}) {
            if ($protocol eq 'iscsi') {
                push(@initiator_filter, @{$initiators_by_protocol->{$host}->{$protocol}});
            } elsif ($protocol eq 'fcp') {
                 @initiator_filter = @{$initiators_by_protocol->{$host}->{$protocol}};
            } else {
                push(@nfs_ip_filter, @{$initiators_by_protocol->{$host}->{$protocol}});
            }
        }
    }

    my @igroups = NACL::CS::LunIgroup->fetch(command_interface => $args{command_interface},
                                             allow_empty => 1,
                                             filter => {initiator => \@initiator_filter});

    my @export_policies = NACL::CS::VserverExportPolicyRule->fetch(command_interface => $args{command_interface},
                                                                   allow_empty => 1);

    my $vserver_initiator_map;
    # Add mapping of vserver to initiators
    foreach my $igroup (@igroups) {
        # skip this igroup if it is the protocol endpoint igroup
        if (not ($igroup->igroup =~ m/^vvol-\S+-igroup$/) ) {
            next;
        }
        my @initiators = $igroup->initiator;
        my $vserver = $igroup->vserver;
        foreach my $initiator (@initiators) {
            $vserver_initiator_map->{$vserver}->{$initiator} = {};
            push(@{$vserver_initiator_map->{$initiator}}, $vserver);
        }
    }

    # we need to add all of the nfs ips to the list of initiators by protocol for each host
    foreach my $host (keys %{$initiators_by_protocol}) {
        foreach my $host_ip (@{$initiators_by_protocol->{$host}->{nfs}}) {
            foreach my $export_policy (@export_policies) {
                # push this onto the list if the export policy rule matches and ip for the host
                if ($export_policy->clientmatch eq $host_ip) {
                    $vserver_initiator_map->{$export_policy->vserver}->{$host_ip} = {};
                    push(@{$vserver_initiator_map->{$host_ip}}, $export_policy->vserver);
                }
            }
        }
    }
    return $vserver_initiator_map;
}

sub _get_balanced_volume_vserver_map {
    my ( $self, %args ) = @_;
    my $provisioning_metadata = $args{provisioning_metadata};

    my %node_map;
    tie %node_map, 'Tie::IxHash';
    my $balanced_volume_vserver_map;

    my @cluster_names = keys %{$provisioning_metadata->{volume_vserver_map}};
    my @clusters = $self->{parent}->get_objects(type => "node",
                                                names => \@cluster_names,
                                                obj_class => 'nacl');

    foreach my $cluster_obj (@clusters) {
        my $volume_vserver_map;
        my $vserver_filter;
        my $volume_filter;
        my @vservers =keys %{$provisioning_metadata->{volume_vserver_map}->{$cluster_obj->name}};
        foreach my $vserver (@vservers) {
            $vserver_filter .= "$vserver|";
            foreach my $volume (@{$provisioning_metadata->{volume_vserver_map}->{$cluster_obj->name}->{$vserver}}) {
                $volume_filter .= "$volume|";
            }
        }
        # Remove trailing pipe
        $volume_filter =~ s/\|$//;
        $vserver_filter =~ s/\|$//;

        # grab the volume cs objects
        my @volume_cs_objs = NACL::CS::Volume->fetch(command_interface => $cluster_obj,
                                                     filter => {volume => $volume_filter, vserver => $vserver_filter});

        my @filtered_vol_cs_objs;
        @volume_cs_objs = grep {
            my $vol_name = $_->volume;
            my $vol_vserver = $_->vserver;
            my $volumes = $provisioning_metadata->{volume_vserver_map}->{$cluster_obj->name}->{$vol_vserver};
            if (defined($provisioning_metadata->{volume_vserver_map}->{$cluster_obj->name}->{$vol_vserver}) and (grep {$vol_name eq $_} @{$volumes})) {
                # include
                1;
            } else {
                # don't include
                0;
            }
        } @volume_cs_objs;
        push(@filtered_vol_cs_objs, @volume_cs_objs);
        # map each volume to it's corresponding vserver/node list
        foreach my $vol_cs_obj (@filtered_vol_cs_objs) {
                push(@{$node_map{$cluster_obj->name}{$vol_cs_obj->vserver}{$vol_cs_obj->node}}, $vol_cs_obj->volume);
        }

        # we need to push each flex vol onto the list of volumes for each vserver, mixing by node location within that vserver if possible
        foreach my $vserver (@vservers) {
            # foreach vserver we need to balance out the volumes in the list by node location
            my @vs_nodes = keys %{$node_map{$cluster_obj->name}{$vserver}};
            while (scalar(keys %{$node_map{$cluster_obj->name}{$vserver}})) {
                my $volume;
                my $node = shift(@vs_nodes);

                # if this node still has volumes to use shift one off otherwise move on
                if (scalar(@{$node_map{$cluster_obj->name}{$vserver}{$node}})) {
                    $volume = shift($node_map{$cluster_obj->name}{$vserver}{$node});
                } else {
                    delete $node_map{$cluster_obj->name}{$vserver}{$node};
                    next;
                }
                # push the volume onto the balanced list for the vserver
                push(@{$volume_vserver_map->{$cluster_obj->name}->{$vserver}}, $volume);
                push(@vs_nodes, $node);
            }
        }
        # add the metadata for this cluster to the full balanced list
        $balanced_volume_vserver_map->{$cluster_obj->name} = $volume_vserver_map->{$cluster_obj->name};
    }
    return $balanced_volume_vserver_map;
}

sub _get_container_ids {
    my ( $self, %args ) = @_;

    my %container_ids;
    my $containers_output = NACL::C::VPserver::Container->list(command_interface => $args{command_interface});
    foreach my $container (@{$containers_output}) {
        $container_ids{$container->{name}} = $container->{id};
    }
    return \%container_ids;
}

sub _setup_failure {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/VPserver/Setup/Failure",
                            message  => \%data);
}

sub _setup_complete {
    my ($self, %data) = @_;
    messenger_global()->put(endpoint => $self->{endpoint},
                            type     => "NACL/AppBuilder/EnGenX/VPserver/Setup/Complete",
                            message  => \%data);
}

1;
