# Copyright (c) 2015 NetApp, Inc.
# All rights reserved.

## @summary Configuration data processing for IceKing
##
## @description
##   This class does the heavy lifting for processing an C<IceKing> framework
##   YAML configuration. This includes workarounds for buggy YAML parsers,
##   variable interpolation, scaling and preparing it for object matching.
##
## @notes
##   Everybody loves princess Bubblegum
##
## @author ng-dpg-smgr-auto
##

package NACL::AppBuilder::IceKing::Bubblegum;

use 5.014;
use warnings;
use Moose;
use Readonly;
use Hash::Merge qw(merge);
use Module::Load;
use YAML::Any qw(LoadFile Load);
use Time::HiRes qw(gettimeofday);
use NATE::Log;
use NATE::ParamSet;
use NACL::AppBuilder::IceKing::Util qw(IKDumper);
use NACL::AppBuilder::IceKing::Ricardio;
use NATE::BaseException qw(:try);

our $VERSION = '3.13';
my ($LOG, $PSET, $GLOBAL_ITERATOR, $LESS_BUGGY_YAML);

#<<< sit, perltidy
Readonly my $DEFAULT_OBJ_COUNT    => 1;
    # how many objects to match when no weight value is specified
Readonly my $DELETE_USED_LEAFS    => 1;
    # clean YAML elements which are not needed anymore after processing
Readonly my %DEFAULT_HW_SCALING   =>
    (low_end => 1, mid_range => 1, high_end => 1);
    # fields delimiter for composite keys
Readonly my $FIELD_DELIMITER      => q(|);
    # output the resulting YAML to this file
Readonly my $YAML_RAW_LOGFILE_NAME    => q(RawYAML.log);
Readonly my $YAML_LOGFILE_NAME    => q(ProcessedYAML.log);
Readonly my $YAML_MERGE_KEY       => q(<<);
Readonly my $RANGE_MARKER         => q(##RANGE##);
Readonly my $PLACEHOLDER_TEMPLATE => q(%s_NACL_OBJ_HERE_%04d);
#>>>

has 'data' => (
    is  => 'rw',
    isa => 'Maybe[ArrayRef[Any]]',
);

has 'yaml_version' => (
    is  => 'ro',
    isa => 'Num',
);

has 'hw_scale' => (
    is  => 'rw',
    isa => 'Num',
);

has 'lookup' => (
    is        => 'ro',
    isa       => 'HashRef[Str]',
    predicate => 'has_lookup',
    lazy      => 1,
    default   => sub { {} },
);

has 'seen' => (
    is      => 'ro',
    isa     => 'HashRef[Str]',
    clearer => 'clear_seen',
    lazy    => 1,
    default => sub { {} },
    documentation =>
        q(A hash for keeping track of nodes that are walked through,)
        . q( to avoid infinite loops due to circular references),
);

has 'field_delimiter' => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub { return $FIELD_DELIMITER },
);

has 'keyspecs' => (
    is   => 'ro',
    isa  => 'HashRef[Any]',
    lazy => 1,
    default =>
        sub { return NACL::AppBuilder::IceKing::Ricardio->new()->keyspecs },
);

has 'jake' => (
    is       => 'rw',
    isa      => 'NACL::AppBuilder::IceKing::Jake',
    required => 0,
);

has 'testcode_modules' => (
    is      => 'rw',
    isa     => 'HashRef[Str]',
    required => 0,
);

################################################################################
## Objective: Post-constructor
## Details:   This is a Moose specific post-constructor method, and we use it
##            to load the YAML (file or string), fix parser quirks, scale,
##            and interpolate
## Arguments: yaml_file          - a YAML config file
##               - or -
##            yaml_string        - or a stringified YAML
##            hw_scale           - an absolute HW multipler (for all objects)
##               - or -
##            hw_scaling_factors - a hashref of HW types and factors
##            hw_type            - a key in the scaling_factors hashref
################################################################################
sub BUILD {
    my $self = shift;
    my %args = %{$_[0]};
    # this is how we get the constructor parameters from Moose
    my ($settings, $data, $raw_yaml);
    # settings and data are the two separate YAML 'streams', delimited by the
    # --- separator in the YAML
    Hash::Merge::set_clone_behavior(0);
    $LOG = NATE::Log->new($::LOGSPEC, global => 1);
    $PSET = NATE::ParamSet->new(global => 1);
    $GLOBAL_ITERATOR = 0;
    if (exists $INC{'YAML/XS/LibYAML.pm'}) {
        $LOG->comment('Less-buggy YAML::XS parser is in use');
        $LESS_BUGGY_YAML = 1;
    } else {
        NATE::BaseException->throw('Could not find YAML::XS, required');
        return;
    }
    if (my $file = delete $args{yaml_file}) {
        if (!(eval { ($settings, $data) = LoadFile($file); 1 })) {
            #$LOG->error("Loading config file failed for file '$file': $@");
            NATE::BaseException->throw("Loading config file failed for file '$file': $@");
            return;
        }
        $LOG->log("Original YAML file:$file");

        open(RAWYAML,"<",$file);
        $raw_yaml = join "",<RAWYAML>;
        close RAWYAML;

    } elsif (my $yaml_str = delete $args{yaml_string}) {
        if (!(eval { ($settings, $data) = Load($yaml_str); 1 })) {
            $LOG->error("Loading YAML string failed for '$yaml_str': $@");
            NATE::BaseException->throw("Loading YAML string failed for '$yaml_str': $@");
            return;
        }
        $raw_yaml = $args{yaml_string};
    }

    $LOG->debug('yaml raw: ' . $raw_yaml);
    $LOG->debug('yaml setting: ' . IKDumper($settings));
    $LOG->debug('yaml data: ' . IKDumper($data));

    $self->data($data);
### handle config file versions
    if (!defined $settings->{version} || $settings->{version} < 3) {
        $LOG->error(
            'Sorry, only YAML config versions 3 and higher are supported.');
        return;
    }
    $self->{yaml_version} = $settings->{version};
    $LOG->comment('YAML config v' . $self->yaml_version . ' detected.');

### handle HW scaling
    my ($hw_scale, %hw_scaling_factors);
    if (exists $args{hw_scale}) {
        # hard-coded scaling factor used in constructor
        $hw_scale = $args{hw_scale};
    } else {
        if (   exists $args{hw_scaling_factors}
            || exists $settings->{hw_scaling_factors}) {
            if (exists $args{hw_scaling_factors}) {
                # scaling factors in constructor
                %hw_scaling_factors = %{$args{hw_scaling_factors}};
            } else {
                # scaling factors in YAML
                my @scaling_values
                    = split(/[|]/, $settings->{hw_scaling_factors});
                my @hw_scaling_names = qw(low_end mid_range high_end);
                my $i                = 0;
                %hw_scaling_factors
                    = map { $_ => $scaling_values[$i++] } @hw_scaling_names;
            }
            if (exists $args{hw_type}) {
                $hw_scale = $hw_scaling_factors{$args{hw_type}};
            } else {
                $LOG->warn('Got HW scaling factors but not HW type;'
                        . ' defaulting to \'low_end\'!');
                $hw_scale = $hw_scaling_factors{low_end};
            }
        } elsif (exists $args{hw_type}) {
            $hw_scale = $DEFAULT_HW_SCALING{$args{hw_type}};
        } else {
            $hw_scale = $DEFAULT_HW_SCALING{low_end};
        }
    }
    $self->hw_scale($hw_scale);
    $LOG->comment('HW scale (multiplier) is: ' . $self->hw_scale);
    if (exists $settings->{environment}) {
        $LOG->comment(
            'Loading the environment variable pairs in the Settings section...'
        );
        foreach my $k (keys %{$settings->{environment}}) {
            $self->_update_lookup($k, $settings->{environment}->{$k}, 0);
        }
    } else {
        $LOG->comment(
            'No environment variable pairs in the Settings section to load...');
    }
    $LOG->comment('Processing data structure...');

### Fix common YAML parsing quirks
    if (!$LESS_BUGGY_YAML) {
        # only need fixing if not using YAML::XS
        $LOG->step('Fixing YAML library quirks...');
        $self->_walk_tree($self->data, \&_cb_fix_yaml);
    }

### interpolate env variables (eg. $FILERA, ${FILER})
### Now running interprolate both before and after scale
### (was just after previously)
### This is to be able to get name_list to expand inside
### scale after interpolate is called
###
    $LOG->step('Running interpolate...');
    $self->_walk_tree($self->data, \&_cb_interpolate);

### add scaling factors (HW and local weights)
    $LOG->step('Adding scaling...');
    $self->_walk_tree($self->data, \&_cb_scale);
    $LOG->debug(
        'Data structure (scaled) is: ' . IKDumper($self->data, Maxdepth => 6));

### merge keys YAML issue: http://www.perlmonks.org/?node_id=813726
### needs to happen before scalling because some composite keys need to find
### information in the parent nodes
    $LOG->step('Fixing YAML keys merge...');
    $self->_walk_tree($self->data, \&_cb_merge);
    $LOG->debug(
        'Data structure (merge) is: ' . IKDumper($self->data, Maxdepth => 6));

### interpolate env variables (eg. $FILERA, ${FILER})
### needs to run after &_cb_scale to use the [potentially] new ENV
### variables resulted from <name>/<name>_value meta tuples
    $LOG->step('Running interpolate...again...');
    $self->_walk_tree($self->data, \&_cb_interpolate);

### load modules needed for testcode testcases
    $LOG->step('Loading modules...');
    $self->_walk_tree($self->data->[2]->{execution}, \&_cb_load_modules);

    $LOG->step('Creating log...');
    my $top_log = $PSET->get('TOP_LOGDIR');
    my $runid   = $PSET->get('RUNID');
    if (-f "$top_log/$runid.log") {
        # we are not running with LOGDIR=- so a log file can be created
        # / should work on Windows too ;)
        #

        $LOG->log("Dumping parsed YAML to '$top_log/$runid/$YAML_LOGFILE_NAME'...");
        $LOG->redirect(path => "$top_log/$runid/$YAML_LOGFILE_NAME");
        $LOG->case(
            "Processed YAML data is:\n" . IKDumper($self->data, Deepcopy => 1));
        $LOG->reset_path();
        $LOG->log('...done');

        $LOG->log("Dumping raw YAML to '$top_log/$runid/$YAML_RAW_LOGFILE_NAME'...");
        #we don't want the log line preamble so we have to use open here
        open (WRITEYAML,'>',"$top_log/$runid/$YAML_RAW_LOGFILE_NAME");
        print WRITEYAML $raw_yaml;
        close WRITEYAML;
        $LOG->log('...done');
    } else {
        $LOG->log('Could not redirect log; proabaly LOGDIR=- was used');
    }
    return;
}

################################################################################
## Objective: Returns a placeholder value using the template and iterator
## Details:   Used by _cb_scale to create a string to be replaced by
##            Configurator with actual objects; the value is used for debugging
## Arguments: Object type
################################################################################
sub __construct_placeholder {
    return (sprintf($PLACEHOLDER_TEMPLATE, uc(shift), $GLOBAL_ITERATOR++));
}

sub __iterate_parent {
    my ($parents, $reset) = @_;
    state $parent_iterator = 0; # Perl v5.10 needed
    if ($reset) {
        $LOG->debug("Resetting iterator to $reset");
        $parent_iterator = $reset;
    } else {
        return 0 unless $parents;
        $parent_iterator++;
    }
    if ($parent_iterator >= $parents) {
        $LOG->debug("Reached end of array ($parent_iterator >= $parents)  or wrong reset value)");
        $parent_iterator = 0;
    }
    $LOG->debug("Iterator changed to $parent_iterator");
    return $parent_iterator;
}

################################################################################
## Objective: Updates the lookup table
## Details:   It also checks that a different value doesn't exist already
##            The lookup is used by the Configurator as a dictionary for
##            environment variables to be passed to EnGenX
## Arguments: Environment name and value
################################################################################
sub _update_lookup {
    my ($self, $env_name, $new_val, $overwrite) = @_;
    $overwrite = 1 unless defined $overwrite;
    my $temp_val = $PSET->get($env_name, default => $new_val);
    if ($temp_val ne $new_val) {
        $LOG->trace(
            "'$env_name' value already in the environment ($temp_val) superseeds
            method parameter ($new_val)"
        );
        $new_val = $temp_val;
    }
    if (exists $self->lookup->{$env_name}) {
        $LOG->debug("Lookup pair already exists: '$env_name'"
                . " => '$self->lookup->{$env_name}'");
        if ($self->lookup->{$env_name} ne $new_val) {
            if ($overwrite) {
                $LOG->trace(
                    "Updating '$env_name' => '$new_val' in the lookup table..."
                );
            } else {
                $LOG->warn(
                    " values are different: lookup:'$self->lookup->{$env_name}'"
                        . " <> new:'$new_val'!");
                return;
            }
        } else {
            return;
        }
    } else {
        $LOG->trace("Adding '$env_name' => '$new_val' to the lookup table...");
    }
    $self->lookup->{$env_name} = $new_val;
    $LOG->trace("\t...and in the local environment");
    $PSET->set(key => $env_name, value => $new_val);
    return;
}

################################################################################
## Objective: Uses the lookup table to resolve a variable
## Details:   It updates the lookup with the default value if an entry doesn't
##            exist already
## Arguments: Environment name and a default value
################################################################################
sub _use_lookup {
    my ($self, $env_name, $def_val) = @_;
    $LOG->debug("Called _use_lookup with '$env_name' and default '$def_val'");
    my $env_val = $PSET->get($env_name);
    if ($env_val) {
        $LOG->debug("Found '$env_name' => '$env_val' in ENV");
    } else {
        $env_val = $def_val;
    }
    $self->_update_lookup($env_name, $env_val);
    return $env_val;
}

################################################################################
## Objective: Get the base keyspec object type for an object or alias
## Details:   If it's an alias, return the object type, for an object, return
##            the value that was passed in
## Arguments: Name to search for
################################################################################
sub _get_obj_type {
    my ($self, $name_or_alias) = @_;
    $LOG->debug("Resolving object type for: '$name_or_alias'...");
    my $obj_type;
OBJ_RESOLVE:
    foreach my $objt (keys $self->keyspecs) {
        if ($name_or_alias eq $objt) {
            $obj_type = $objt;
            last OBJ_RESOLVE;
        } else {
            if (defined $self->keyspecs->{$objt}->{alias}) {
                foreach my $alias (@{$self->keyspecs->{$objt}->{alias}}) {
                    if ($name_or_alias eq $alias) {
                        $obj_type = $objt;
                        last OBJ_RESOLVE;
                    }
                }
            }
        }
    }
    if ($obj_type) {
        $LOG->debug("\t...it resolved to: '$obj_type'...");
    } else {
        $LOG->debug("\t...couldn't resolve...");
    }
    return $obj_type;
}

################################################################################
## Objective: Get the parent for a node
## Details:   Since 'data' is a single linked tree, we have to construct the
##            parent object starting from the root
## Arguments: Root object and Array reference of keys representing the path
################################################################################
sub _get_parent {
    my ($root, $aref_keys) = @_;
    my $parent  = $root;
    my @ar_keys = @{$aref_keys};
    while ((my $el = shift(@ar_keys)) && (@ar_keys > 0)) {
        if ($el =~ /\[(.*)\]/) {
            $parent = $parent->[$1];
        } else {
            $parent = $parent->{$el};
        }
    }
    return $parent;
}

## no critic (ProhibitManyArgs)
################################################################################
## Objective: Just broke this out of _generate_key
## Details:   It will generate a [composite] key using the keyspec for the
##            specified object type
## Arguments: too many
################################################################################
sub _process_keyspec {
## use critic
    my ($self, $keyspec, $this_obj, $aref_keys, $name, $obj_type) = @_;
    my ($part, @parts, $count, @range, $key);
    foreach my $token (@{$keyspec->{value}}) {
        if (!$token->{search_parents}) {
            # name can be found in this HASH, no need to search parents
            if (defined $token->{default_name}) {
                # use the name that was passed in
                $part = $name;
            } elsif (defined $token->{field_name}
                && defined $this_obj->{"$token->{field_name}"}) {
                # use the specified field_name
                $part = $this_obj->{"$token->{field_name}"};
            } elsif (defined $token->{field_regex}) {
                $LOG->debug("process keyspec regex '$token->{field_regex}'");
                # find the field name based on a regex
                # (because people seem to do both e.g. src_vol and vol_src)
                my $found;
            REGEX:
                foreach my $k (keys %{$this_obj}) {
                    # if it matches both the type and the regex
                    $LOG->debug("process keyspec key '$k'");
                    if ($k =~ $token->{field_regex} && $k =~ $token->{type}) {
                    $LOG->debug(" match $token->{field_regex} and $token->{type} for keyspec key '$k'");
                        # these fields are there only after scaling
                        $part  = $this_obj->{$k}->{meta}->{name_interpolated};
                        $count = $this_obj->{$k}->{meta}->{count}
                            // ($DEFAULT_OBJ_COUNT + 1);
                        my $rs = $this_obj->{$k}->{meta}->{range_start} // 1;
                    $LOG->debug("process keyspec part '$part' count '$count' rs '$rs'");
                        @range = $rs .. ($rs + $count - 1);
                        # the last regex match will provide the count
                        # TODO: for stuff like sm, get the least count of src and dst vol
                        $count = keys %{$this_obj->{$k}->{objects}};
                    $LOG->debug("process keyspec  count '$count' ");
                        $found = 1;
                        last REGEX;
                    }
                }
                $LOG->warn("Couldn't find a match for '$token->{field_regex}'")
                    if (!$found);
            }
            push @parts, $part if ($part);
        } else {
            my ($at_root, $parent_obj_type, $parent);
            my $obj_search_type = $token->{type};
            my @copy_keys       = @{$aref_keys};
            my $last_key        = $copy_keys[-1];
        SEARCH:
            while (1) {
                if (!@copy_keys) {
                    # we reached the root level and couldn't find it
                    $LOG->warn("Couldn't find a parent of type '$obj_type'");
                    $at_root = 1;
                    last SEARCH;
                }
                $parent          = _get_parent($self->data, \@copy_keys);
                $last_key        = pop @copy_keys;
                $parent_obj_type = $self->_get_obj_type($last_key);
                if ($parent_obj_type) {
                    # the parent was of a known type
                    if ($parent_obj_type eq $obj_search_type) {
                        # in case it was an alias
                        $obj_search_type = $parent_obj_type;
                        last SEARCH;
                    }
                } else {
                    next SEARCH;
                }
            }
            if (!$at_root) {
                # we found something
                push @copy_keys, $last_key;
                my $parent_name;
                if (defined $parent->{$last_key}->{"$token->{field_name}"}) {
                    $parent_name
                        = $parent->{$last_key}->{"$token->{field_name}"};
                } elsif (defined $parent->{$last_key}->{objects}) {
                    # this is a set of objects (e.g. lun's) that have a set of
                    # objects as parent (e.g. vol's); in this case parent has no
                    # 'name' parent attribute so need different matching algorithms
                    $LOG->comment('Found a many to many dependency...');
                    my @keys = sort keys %{$parent->{$last_key}->{objects}};
                    if (!exists($this_obj->{meta}->{distribution})) {
                        #|| $this_obj->{meta}->{distribution} eq 'sequential') {
                        $parent_name = $keys[0];
                        $LOG->comment('...using "broken" matching (just 1st key)');
                    } elsif ($this_obj->{meta}->{distribution} eq 'sequential') {
                        $LOG->comment('...using "sequential" matching (round-robin)');
                        $parent_name = $keys[__iterate_parent(scalar @keys)];
                    } elsif ($this_obj->{meta}->{distribution} eq 'each') {
                        $LOG->warn(
                                  '...using "each" matching (not implemented);'
                                . ' please review the meta->distribution option'
                        );
                        # TODO, if actually needed
                        $parent_name = $keys[0];
                    } elsif ($this_obj->{meta}->{distribution} =~ /\d+/) {
                        $LOG->comment('...using absolute matching (parent #'
                                . $this_obj->{meta}->{distribution}
                                . ')');
                        $parent_name
                            = $keys[$this_obj->{meta}->{distribution} - 1];
                    } elsif ($this_obj->{meta}->{distribution} eq 'random') {
                        $LOG->comment('...using random matching');
                        $parent_name = $keys[rand @keys];
                    }
                    # get last element in case it's a composite key
                    $parent_name
                        = (split(quotemeta $FIELD_DELIMITER, $parent_name))[-1];
                } else {
                    $LOG->debug(
                              "Couldn't find the '$token->{field_name}' field"
                            . " specified in keyspec for object type '$obj_type'"
                            . " at '@copy_keys'");
                }
                $part = $self->_generate_key(
                    $parent->{$last_key}, \@copy_keys,
                    $parent_name,         $token->{type}
                );
                # split in case an object was found and it had a composite key
                $part = (split(quotemeta $FIELD_DELIMITER, $part))[-1];
                push @parts, $part if ($part);
            }
        }
    }
    if (@parts) {
        if (@range) {
            # a range was found, get the start and end element
            push @parts, $range[0];
            push @parts, $range[-1];
        }
        # and append them to the key
        return join($FIELD_DELIMITER, @parts);
    }
    return;
}

################################################################################
## Objective: Generate the key for an object placeholder tuple
## Details:   It will use best effort for object types without keyspecs or
##            call _process_keyspec in case they do have them
## Arguments: The obj, key path array ref, optional: name of obj, type of obj
################################################################################
sub _generate_key {
    my ($self, $this_obj, $aref_keys, $name, $obj_type) = @_;
    my $key;
    $LOG->debug(
        sprintf(
            'gen_key called with: keys "%s", name "%s", obj_type "%s"',
            "@{$aref_keys}",
            $name // 'undef',
            $obj_type // 'undef'
        )
    );
    # try to find out the obj type (if it wasn't passed in)
    $obj_type //= $self->_get_obj_type(@{$aref_keys}[-1]);
    if ($obj_type) {
        if (!defined $self->keyspecs->{$obj_type}) {
            $LOG->trace(
                "Object type '$obj_type' has no entry in \$self->keyspecs!");
            # that's ok, if the default name was passed in or there is a "name" field
            $name //= $this_obj->{name};
        }
        if (defined $self->keyspecs->{$obj_type}->{keyspec}) {
#TODO increment here
            my $keyspec = $self->keyspecs->{$obj_type}->{keyspec};
            # handle prefix if exists
            $key = $keyspec->{prefix}
                if (defined $keyspec->{prefix});
            # if it's a composite key
            if (ref($keyspec->{value}) eq 'ARRAY') {
                $key .= $self->_process_keyspec($keyspec, $this_obj, $aref_keys,
                    $name, $obj_type);
            $LOG->debug("keyspec array process '$key'");

            } else {
                # simple key_spec, only a field_name
                if (defined $keyspec->{field_name}) {
                    my $field = $keyspec->{field_name};
                    if (defined $this_obj->{$field}) {
                        $key .= $this_obj->{$field};
                    } elsif (defined $this_obj->{meta}->{"${field}_value"}) {
                        # dangerous, because name could be overwritten from ENV
                        $key .= $name // $this_obj->{meta}->{"${field}_value"};
                        # the above assumes no "name" entry and an existing
                        # meta "name" passed in to this function as $name
                    } elsif (defined $this_obj->{$YAML_MERGE_KEY}) {
                        $key .= $this_obj->{$YAML_MERGE_KEY}->{name};
                    } else {
                        $key .= "KEYSPEC_${field}_NOT_FOUND";
                        $LOG->warn(
                            "Couldn't find the field specified in keyspec for object type '$obj_type'"
                        );
                    }
                } else {
                    # fall back to the passed in value
                    $key .= $name if ($name);
                }
            }
            # handle suffix if exists
            $key .= $keyspec->{suffix}
                if (defined $keyspec->{suffix});
        } else {
            # no keyspec
            if (defined $this_obj->{objects}) {
                my @keys = keys %{$this_obj->{objects}};
                $key = $keys[rand @keys];
                # get last element in case it's a composite key
                $key = (split(quotemeta $FIELD_DELIMITER, $key))[-1];
            } else {
                $key //= $name // $this_obj->{name}
                    // $this_obj->{meta}->{name_value}
                    // $this_obj->{$YAML_MERGE_KEY}->{name}
                    // 'OBJECT/NAME/META_NOT_FOUND';
            }
        }
    }
    $key //= $name;
    $LOG->debug("gen_key returning key '$key'");
    return $key;
}

################################################################################
## Objective: Check if we saw this reference before
## Details:   It updates the 'seen' hash if not; it is used to work around
##            circular references
## Arguments: Node and key path array reference
################################################################################
sub _was_seen {
    my ($self, $current_node, $aref_keys) = @_;
    my $key = "$current_node";
    if (exists $self->seen->{$key}) {
        $LOG->debug("'$key' was seen before; skipping ($self->seen->{$key})");
        return 1;
    } else {
        my $val = join(q(->), @{$aref_keys});
        $self->seen->{$key} = $val;
        $LOG->debug("'$key' was NOT seen before, adding val = '$val'");
        return 0;
    }
}

################################################################################
## Objective: Visit all nodes in a hash tree recursively and run something
## Details:   It calls an arbitrary callback function on each leaf
## Arguments: Node, callback, key path array reference
################################################################################
sub _walk_tree {
    my ($self, $current_node, $oper, $aref_keys) = @_;
    if (!defined($oper)) {
        $LOG->error('Why walk the data tree without a callback function?!');
        return;
    }
    if (ref($oper) && ref($oper) ne 'CODE') {
        $LOG->error('Callback has to be a method name or code reference!');
        return;
    }
    $aref_keys //= [];
    if (ref($current_node) eq 'HASH') {
        $LOG->debug('Processing HASH leaf');
        if (!$self->_was_seen($current_node, $aref_keys)) {
            $self->$oper($current_node, $aref_keys);
        }
        foreach my $key (keys %{$current_node}) {
            push @{$aref_keys}, $key;
            if (ref($current_node->{$key})) {
                if (!$self->_was_seen($current_node->{$key}, $aref_keys)) {
                    $self->$oper($current_node->{$key}, $aref_keys);
                    $self->_walk_tree($current_node->{$key}, $oper, $aref_keys);
                }
            } else {
                $self->$oper(\$current_node->{$key}, $aref_keys);
            }
            pop @{$aref_keys};
        }
    } elsif (ref($current_node) eq 'ARRAY') {
        $LOG->debug('Processing ARRAY leaf');
        for my $i (0 .. (scalar @{$current_node} - 1)) {
            push @{$aref_keys}, "[$i]";
            if (ref($current_node->[$i])) {
                if (!$self->_was_seen($current_node->[$i], $aref_keys)) {
                    $self->$oper($current_node->[$i], $aref_keys);
                    $self->_walk_tree($current_node->[$i], $oper, $aref_keys);
                }
            } else {
                if (!$self->_was_seen(\$current_node->[$i], $aref_keys)) {
                    $self->$oper(\$current_node->[$i], $aref_keys);
                }
            }
            pop @{$aref_keys};
        }
    } elsif (ref($current_node) =~ /^NACL/) {
        $LOG->debug('Skipping object leaf (' . ref($current_node) . ')');
        return;
    } else {
        $LOG->trace('Invalid leaf ('
                . ref($current_node)
                . ") for '$current_node' @ "
                . join(q(->), @{$aref_keys})
                . q(!));
        return;
    }
    if ((@{$aref_keys} == 0)) {
        $LOG->debug('Finished traversing the tree.');
        $self->clear_seen;
    }
    return;
}

################################################################################
## Objective: Replace a variable with its environment value recursively
## Details:   Should probably use a $Testobj method from NATE instead
## Arguments: The string to intepolate
################################################################################
sub __repl {
    my $val = shift;
    my ($var, $pre, $post);
    my $subst = $val;
    if ($val =~ /\$/) {
        ($pre, $var, $post) = $subst =~ /^(.*)\$[{]?(\w+)[}]?(.*)$/;
        if ($subst = $PSET->get($var)) {
            $subst = __repl($pre) . __repl($subst) . __repl($post);
        } else {
            $subst = "\$$var";
        }
    }
    return $subst;
}

################################################################################
## Objective: Callback function for variable interpolation
## Details:   Automatically called in the post-constructor; processes only HASH
##            elements and SCALAR's
## Arguments: Node and key path array reference
################################################################################
sub _cb_interpolate {
    my ($self, $current_node, $aref_keys) = @_;
    my $subst;
    local $" = q(->);
    if (ref($current_node) eq 'HASH') {
        # handle generated keys (since we have to run after _scale)
        $LOG->debug('Interpolating HASH keys at ' . IKDumper($aref_keys));
        foreach my $k (keys %{$current_node}) {
            $subst = __repl($k);
            $current_node->{$subst} = delete $current_node->{$k}
                if ($subst ne $k);
        }
    } elsif (ref($current_node) eq 'SCALAR') {
        $LOG->debug(
            "Interpolating SCALAR ${$current_node} at " . IKDumper($aref_keys));
        if ($aref_keys->[-1] eq 'name_pattern') {
            $LOG->debug('Skipping name_pattern');
            return;
        }
        if ($aref_keys->[-1] eq 'filter') {
            $LOG->debug('Skipping filter');
            return;
        }
        $subst = __repl(${$current_node});
        if ($subst ne ${$current_node}) {
            $LOG->trace("Resolved '${$current_node}' => '$subst'");
            ${$current_node} = $subst;
        } else {
            if (${$current_node} =~ /\$/) {
                $LOG->trace(
                    "Could not interpolate potentially dynamic '${$current_node}'"
                );
            } else {
                $LOG->debug("Nothing to resolve for '${$current_node}'");
            }
        }
    } elsif (!ref($current_node)) {
        $LOG->error("I was called with a non-reference ($current_node)!");
    } else {
        $LOG->debug('Interpolate called with a non-SCALAR reference');
    }
    return;
}

################################################################################
## Objective: Fix YAML merge since all Perl YAML parsers have this bug
## Details:   It will detect the '<<' key and merge in the values depending on
##            their type (and what other leafs exist in the node)
## Arguments: Node and key path array reference
################################################################################
sub _cb_merge {
    my ($self, $current_node, $aref_keys) = @_;
    if (ref($current_node) eq 'HASH') {
        if (exists $current_node->{$YAML_MERGE_KEY}) {
            $LOG->case("Found merge keys construct ('$YAML_MERGE_KEY')"
                    . ' - due to a buggy YAML parsing library for '
                    . ref($current_node->{$YAML_MERGE_KEY}));
            local $" = q(->);
            $LOG->trace("\tat '@{ $aref_keys }'...");
            if (ref($current_node->{$YAML_MERGE_KEY}) eq 'HASH') {
                $LOG->debug('Current node before replacing HASH:'
                        . IKDumper($current_node, Maxdepth => 2));
                delete $current_node->{meta}
                    if (defined $current_node->{meta}
                    && !%{$current_node->{meta}});
                foreach my $k (keys %{$current_node->{$YAML_MERGE_KEY}}) {
                    $current_node->{$k}
                        = $current_node->{$YAML_MERGE_KEY}->{$k};
                }
                delete $current_node->{$YAML_MERGE_KEY};
                $LOG->debug('Current node after replacing HASH:'
                        . IKDumper($current_node, Maxdepth => 2));
            } elsif (ref($current_node->{$YAML_MERGE_KEY}) eq 'ARRAY') {
                my $parent = _get_parent($self->data, $aref_keys);
                if (keys %{$current_node} == 1) {
                    $LOG->debug('Current node before replacing ARRAY:'
                            . IKDumper($current_node, Maxdepth => 2));
                    $parent->{@{$aref_keys}[-1]}
                        = $current_node->{$YAML_MERGE_KEY};
                    $LOG->debug('Current node after replacing ARRAY:'
                            . IKDumper($current_node, Maxdepth => 2));
                } else {
                    $LOG->warn(
                        'Unsupported YAML merge ARRAY+HASH; check MERGE key at: '
                            . IKDumper($current_node, Maxdepth => 3));
                    $current_node->{'MERGE'} = $current_node->{$YAML_MERGE_KEY};
                    delete $current_node->{$YAML_MERGE_KEY};
                }
            } else {
                $LOG->error('Trying to merge an incorrect type');
            }
        }
    } else {
        $LOG->debug('Nothing to merge for this node type: ' . @{$aref_keys});
    }
    return;
}

################################################################################
## Objective: Fix YAML comments
## Details:   Plain old 'YAML' is the only library known to have this bug
## Arguments: Node and key path array reference
################################################################################
sub _cb_fix_yaml {
    my ($self, $current_node, $aref_keys) = @_;
    if (ref($current_node) eq 'SCALAR') {
        if (${$current_node} && ${$current_node} =~ /(\w+)\s*#.*/) {
            $LOG->comment("Removing comments: '${$current_node}' => '$1'");
            ${$current_node} = $1;
        }
    } else {
        $LOG->debug('Nothing to fix for this node type: ' . @{$aref_keys});
    }
    return;
}

## no critic (ProhibitExcessComplexity, ProhibitDeepNests)
################################################################################
## Objective: Scale and create placeholders for objects
## Details:   Couldn't lower the complexity further, got it below 100 though ;)
## Arguments: Node and key path array reference
################################################################################
sub _cb_scale {
## use critic
    my ($self, $current_node, $aref_keys) = @_;
    my $old_obj_type;
    local $" = q(->);
    $LOG->debug('CB_scale called with '
            . IKDumper($current_node) . ' and '
            . IKDumper($aref_keys, Indent => 0));
    if (ref($current_node) eq 'ARRAY') {
        $LOG->debug('Scale processing ARRAY...');
        my ($meta_base_count, $defa_ref, $deleted, $obj_type,
            $obj_name_or_alias, $this_obj_hash);
# reset here
        __iterate_parent(undef, 0);
        for my $i (0 .. (scalar @{$current_node} - 1)) {
            if (ref($current_node->[$i]) eq 'HASH') {
### handle meta* elements
                if (exists $current_node->[$i]->{meta_base_count}) {
                    $meta_base_count = $current_node->[$i]->{meta_base_count};
                    undef $current_node->[$i];
                    $deleted = 1;
                } elsif (exists $current_node->[$i]->{meta_default}) {
                    if ($defa_ref) {
                        $LOG->trace(
                            'Additional defaults found, superseeding the old ones'
                        );
                        $defa_ref = merge($current_node->[$i]->{meta_default},
                            $defa_ref);
                    } else {
                        $LOG->trace('New defaults found');
                        $defa_ref = delete $current_node->[$i]->{meta_default};
                    }
                    $LOG->debug('Defaults are ' . IKDumper($defa_ref));
                    undef $current_node->[$i];
                    $deleted = 1;
                } else {
### it's a regular node, see if it's a supported type
### and perform other preflight checks
                    $obj_name_or_alias = (keys %{$current_node->[$i]})[0];
                    $obj_type = $self->_get_obj_type($obj_name_or_alias);
                    if (   $obj_type
                        && $old_obj_type
                        && $obj_type ne $old_obj_type) {
                        # defaults get reset here - no bleeding to different
                        # object in a mixed types array
                        undef $defa_ref;
                        $old_obj_type = $obj_type;
                    }
                    if (!$obj_type) {
                        $LOG->trace(
                            'Skipping YAML glue (or an unsupported object) key \''
                                . @{$aref_keys}[-1]
                                . "'\n");
                        next;
                    }
                    if (exists $current_node->[$i]->{$obj_name_or_alias}
                        ->{$YAML_MERGE_KEY}) {
                        $LOG->trace(
                            'Node has merged keys at the root level, skipping');
                        $LOG->trace(
                            "\tat '@{ $aref_keys }\->[$i]->$obj_name_or_alias'..."
                        );
                        next;
                    }
                    # merge in the active defaults
                    $current_node->[$i]->{$obj_name_or_alias}
                        = merge($current_node->[$i]->{$obj_name_or_alias},
                        $defa_ref)
                        if ($defa_ref);
                    $this_obj_hash = $current_node->[$i]->{$obj_name_or_alias};
                    # process META values that may not be part of a name_pattern
                    if (exists $this_obj_hash->{meta}) {
                        foreach my $m (keys %{$this_obj_hash->{meta}}) {
                            if (exists $this_obj_hash->{meta}->{"${m}_value"}) {
                                $self->_use_lookup($this_obj_hash->{meta}->{$m},
                                    $this_obj_hash->{meta}->{"${m}_value"});
                            }
                        }
                    }
### handle counts and scaling
                    my $count;
                    if (exists $this_obj_hash->{meta}->{count}) {
                        $count = $this_obj_hash->{meta}->{count};
                    } else {
                        if (exists $this_obj_hash->{meta}->{weight}) {
                            if ($self->keyspecs->{$obj_type}->{no_scale}) {
                                $LOG->warn(
                                    "Found a 'weight' setting for an '$obj_type' object which is not scalable!"
                                );
                                if (defined $this_obj_hash->{meta}
                                    ->{weight_value}) {
                                    $count = $this_obj_hash->{meta}
                                        ->{weight_value};
                                } else {
                                    $count = $DEFAULT_OBJ_COUNT;
                                }
                            } else {
                                $count = $PSET->get(
                                    $this_obj_hash->{meta}->{weight});
                                if (!$count) {
                                    # if it does not exist in ENV
                                    if (exists $this_obj_hash->{meta}
                                        ->{weight_value}) {
                                        # if a _value is provided
                                        $count = $this_obj_hash->{meta}
                                            ->{weight_value};
                                        $count =~ /(\d*[.]?\d+)(%?)\s*$/;
                                        if ($2) {
                                            # it's a percentage
                                            $LOG->trace(
                                                "Found percentage '$count'...");
                                            if (!$meta_base_count) {
                                                $LOG->warning(
                                                    "No 'meta_base_count' defined! Using $1 as an absolute value"
                                                );
                                                $count = $1;
                                            } else {
                                                $count
                                                    = int($1
                                                        * $meta_base_count
                                                        / 100);
                                            }
                                        } else {
                                            # it's a value
                                            $LOG->trace(
                                                "Found absolute weight '$1'..."
                                            );
                                            $count = $1;
                                        }
                                    } else {
                                        $count = $DEFAULT_OBJ_COUNT;
                                    }
                                    # apply HW scale multiplier
                                    $count = int($count * $self->hw_scale);
                                    $self->_update_lookup(
                                        $this_obj_hash->{meta}->{weight},
                                        $count);
                                }
                            }
                        }
                        $this_obj_hash->{meta}->{count} = $count
                            if ($count);
                    }
                    if ($DELETE_USED_LEAFS) {
                        delete $this_obj_hash->{meta}->{weight};
                        delete $this_obj_hash->{meta}->{weight_value};
                    }
### resolve the key name for this object
                    if (   !defined $self->keyspecs->{$obj_type}
                        || $self->keyspecs->{$obj_type}->{no_obj}
                        || defined $this_obj_hash->{objects}) {
                        $LOG->comment(
                            "Found a 'no_object' entry for object type '$obj_type' - skipping"
                        );
                    } else {
                        my ($name, $obj_key, %objects, @range);
                        if (defined $this_obj_hash->{name}) {
                            $name    = $this_obj_hash->{name};
                            $obj_key = $self->_generate_key($this_obj_hash,
                                [(@{$aref_keys}, "[$i]", $obj_type)], $name);
                            if ($obj_key) {
                                $LOG->debug(
                                    "Object with name '$name' generated key '$obj_key'"
                                ) unless ($name eq $obj_key);
                            } else {
                                $LOG->trace(
                                    "Could not generate a key for $name (has a 'name')"
                                );
                            }
# distribute here
# distribute would entail:
# 1. add the placeholder and just increment for 'sequential'
# 2. and placeholders _for all parents_ and then increment for 'each'
                            if ($this_obj_hash->{meta}->{distribution} eq 'sequential') {
#                                __iterate_parent(scalar @{$current_node});
                            } elsif ($this_obj_hash->{meta}->{distribution} eq 'each') {
                                __iterate_parent(scalar @{$current_node});
                            }
                            if ($DELETE_USED_LEAFS) {
                                delete $this_obj_hash->{meta}->{name_pattern};
                                delete $this_obj_hash->{meta}->{name};
                                delete $this_obj_hash->{meta}->{name_value};
                            }
                        } else {
                            if (defined $this_obj_hash->{meta}->{name}) {
                                $name = $self->_use_lookup(
                                    $this_obj_hash->{meta}->{name},
                                    $this_obj_hash->{meta}->{name_value}
                                );
                                $obj_key
                                    = $self->_generate_key($this_obj_hash,
                                    [(@{$aref_keys}, "[$i]", $obj_type)],
                                    $name);
                                if ($obj_key) {
                                    $LOG->debug(
                                        "Object with META name '$name' generated key '$obj_key'"
                                    ) unless ($name eq $obj_key);
                                } else {
                                    $LOG->warn(
                                        "Could not generate a key for $name (has a 'meta->name')"
                                    );
                                }
                                $this_obj_hash->{name} = $name;
                                if ($DELETE_USED_LEAFS) {
                                    # it had a name, so no need for any name* entries
                                    delete $this_obj_hash->{meta}
                                        ->{name_pattern};
                                    delete $this_obj_hash->{meta}->{name};
                                    delete $this_obj_hash->{meta}->{name_value};
                                }
                            } else {
                                if (exists $this_obj_hash->{meta}
                                    ->{name_pattern}) {
                                    $LOG->debug(
                                        "Object has META name_pattern $this_obj_hash->{meta}->{name_pattern}"
                                    );
                                    my @tokens = split(/\$/,
                                        $this_obj_hash->{meta}->{name_pattern});
                                    my $has_range;
                                    foreach my $t (@tokens) {
                                        my ($val, $post)
                                            = $t =~ /[{](\w+)[}](\w*)/;
                                        if ($t =~ /[{]EPOCH_TIME[}](\w*)/) {
                                            #generate custom timestamp for every entry of epoch_time
                                            $name .= join '', gettimeofday, $2;
                                            $LOG->trace("TIME:" . $name);
                                        } elsif ($t =~ /[{]TIMESTAMP[}](\w*)/) {
                                            #generate a single timestamp for all entries of TIMESTAMP so that
                                            #multiple entries can use this as a globally unique reference value for this run
                                            #(primarily to provide matchine source/dest volume names in different blocks)
                                            if (!$self->{TIMESTAMP}) {
                                                my $timestamp = join '', gettimeofday, $2;
                                                $LOG->trace("generate TIMESTAMP:" . $timestamp);
                                                $self->{TIMESTAMP} = $timestamp;
                                            }
                                            $name .= $self->{TIMESTAMP};
                                            $LOG->trace("TIMESTAMP:" . $name);

                                        } elsif ($val) {
                                            # it's a ${something} token
                                            my $tmp_str;
                                            if (exists $this_obj_hash->{meta}
                                                ->{$val}) {
                                                if (exists
                                                    $this_obj_hash->{meta}
                                                    ->{"${val}_value"}) {
                                                    $tmp_str
                                                        = $self->_use_lookup(
                                                        $this_obj_hash->{meta}
                                                            ->{$val},
                                                        $this_obj_hash->{meta}
                                                            ->{"${val}_value"}
                                                        );
                                                } else {
                                                    $LOG->error(
                                                        "Missing '${val}_value'!"
                                                    );
                                                    $tmp_str
                                                        = "[MISSING_VALUE_$this_obj_hash->{meta}->{$val}]";
                                                }
                                                $name .= $tmp_str;
                                                if ($DELETE_USED_LEAFS) {
                                                    delete
                                                        $this_obj_hash->{meta}
                                                        ->{$val};
                                                    delete
                                                        $this_obj_hash->{meta}
                                                        ->{"${val}_value"};
                                                }
                                            } else {
                                                $LOG->error(
                                                    "Error parsing name_pattern '$this_obj_hash->{meta}->{name_pattern}'!"
                                                );
                                                $name
                                                    .= "[MISSING_TOKEN_VALUE_$val]";
                                            }
                                            $name .= $post if ($post);
                                        } else {
                                            if ($t =~ /[{](\d+)[.][.][}](\w*)/)
                                            {
                                                # it's a ${<number>..} range generator
                                                @range = (
                                                    $1 .. (
                                                              $1
                                                            + $this_obj_hash
                                                            ->{meta}->{count}
                                                            - 1
                                                    )
                                                );
                                                $name .= $RANGE_MARKER . $2;
                                                $has_range = 1;
                                            } else {
                                                # it's just plain text (at the beginning of the pattern)
                                                $name .= $t;
                                            }
                                        }
                                    }
                                    if ($DELETE_USED_LEAFS) {
                                        delete $this_obj_hash->{meta}
                                            ->{name_pattern};
                                    }
                                    # initialize the following two (are being used for sm keys)
                                    $this_obj_hash->{meta}->{name_interpolated}
                                        = $name;
                                    $this_obj_hash->{meta}->{range_start}
                                        = $range[0];
                                    if ($has_range) {
                                        foreach my $r (@range) {
                                            my $obj_name = $name;
                                            $obj_name =~ s/$RANGE_MARKER/$r/g;
                                            $obj_key = $self->_generate_key(
                                                $this_obj_hash, $aref_keys,
                                                $obj_name,      $obj_type
                                            );
                                            if ($obj_key) {
                                                $objects{$obj_key}
                                                    = __construct_placeholder(
                                                    $obj_type);
                                            } else {
                                                $LOG->comment(
                                                    "Could not generate a key for $obj_name (has a name_interpolated; with range)"
                                                );
                                            }
                                        }
                                    } else {
                                        $obj_key
                                            = $self->_generate_key(
                                            $this_obj_hash, $aref_keys, $name,
                                            $obj_type);
                                        if ($obj_key) {
                                            $objects{$obj_key}
                                                = __construct_placeholder(
                                                $obj_type);
                                        } else {
                                            $LOG->comment(
                                                "Could not generate a key for $name (has a name_interpolated; no range)"
                                            );
                                        }
                                    }
                                } elsif (exists $this_obj_hash->{meta}
                                    ->{name_list}) {
                                    $LOG->debug(
                                        "Object has META name_list $this_obj_hash->{meta}->{name_list}"
                                    );
                                    my @list = split /,/, $this_obj_hash->{meta}->{name_list};
                                    $LOG->debug(
                                        "list!*!*!*!*!*!!:"
                                            . IKDumper($this_obj_hash->{meta}->{name_list})
                                    );
                                    $LOG->debug(
                                        "list!!:"
                                            . IKDumper(\@list)
                                    );
                                    $LOG->debug(
                                        "list:". join (', ',@list)
                                    );
                                    if ($DELETE_USED_LEAFS) {
                                        delete $this_obj_hash->{meta}
                                            ->{name_list};
                                    }
                                    # initialize the following two (are being used for sm keys)
                                    $this_obj_hash->{meta}->{name_interpolated}
                                        = $this_obj_hash->{meta}->{name_list};
                                        #= $name;
                                        #$this_obj_hash->{meta}->{range_start}
                                        #= $range[0];
                                    if (1) {
                                        foreach my $name (@list) {
                                            my $obj_name = $name;
                                            #$obj_name =~ s/$RANGE_MARKER/$r/g;
                                            $obj_key = $self->_generate_key(
                                                $this_obj_hash, $aref_keys,
                                                $obj_name,      $obj_type
                                            );
                                            if ($obj_key) {
                                                $objects{$obj_key}
                                                    = __construct_placeholder(
                                                    $obj_type);
                                            } else {
                                                $LOG->comment(
                                                    "Could not generate a key for $obj_name (has a name_interpolated; with range)"
                                                );
                                            }
                                        }
                                        #} else {
                                        #$obj_key
                                        #= $self->_generate_key(
                                        #$this_obj_hash, $aref_keys, $name,
                                        #$obj_type);
                                        #if ($obj_key) {
                                        #$objects{$obj_key}
                                        #= __construct_placeholder(
                                        #$obj_type);
                                        #} else {
                                        #$LOG->comment(
                                        #"Could not generate a key for $name (has a name_interpolated; no range)"
                                        #);
                                        #}
                                    }
                                } elsif (
                                    defined $self->keyspecs->{$obj_type}
                                    ->{keyspec}) {
                                    $obj_key
                                        = $self->_generate_key($this_obj_hash,
                                        $aref_keys, undef, $obj_type);
                                    if ($obj_key) {
                                        if ($obj_key =~ /$RANGE_MARKER/) {
                                            my @composite
                                                = split(
                                                quotemeta $FIELD_DELIMITER,
                                                $obj_key);
                                            # the first and last element are passed as the last two elements in the key (from _gen_key)
                                            my $end   = pop @composite;
                                            my $start = pop @composite;
                                            $obj_key = join($FIELD_DELIMITER,
                                                @composite);
                                            foreach my $r ($start .. $end) {
                                                my $key_name = $obj_key;
                                                $key_name
                                                    =~ s/$RANGE_MARKER/$r/g;
                                                $objects{$key_name}
                                                    = __construct_placeholder(
                                                    $obj_type);
                                            }
                                        } else {
                                            $objects{$obj_key}
                                                = __construct_placeholder(
                                                $obj_type);
                                        }
                                    } else {
                                        $LOG->comment(
                                            "Could not generate a key for $obj_type (has a keyspec)"
                                        );
                                    }
                                } else {
                                    $LOG->error(
                                        "No 'name' or 'name_pattern' could be found for this $obj_type!"
                                            . IKDumper(
                                            $this_obj_hash, Maxdepth => 2
                                            )
                                    );
                                }
                            }
                        }
                        # if a single (not a pattern)!
                        %objects
                            = ($obj_key => __construct_placeholder($obj_type))
                            if (!%objects && $obj_key);
                        $this_obj_hash->{objects} = \%objects
                            if (%objects);
                    }
                    delete $this_obj_hash->{meta}
                        if (defined $this_obj_hash->{meta}
                        && !%{$this_obj_hash->{meta}});
                }
            }
        }
        if ($deleted) {
            $LOG->debug('Cleaning up deleted nodes ('
                    . scalar @{$current_node}
                    . ' found)');
            @{$current_node} = grep {defined} @{$current_node} if ($deleted);
            $LOG->debug(' -> ' . scalar @{$current_node} . ' elements');
        }
    } else {
        # We only scale ARRAY elements
        $LOG->debug(
            'Nothing to scale for this node type: ' . ref($current_node));
    }
    return;
}

################################################################################
## Objective: Find and load testcode modules
## Details:   Look through the execution section, parse testcase entry
## Arguments: Node and key path array reference
################################################################################
sub _cb_load_modules {
    my ($self, $current_node, $aref_keys) = @_;
    my ($test_class, $proc_name, $proc_decoration);
    $LOG->debug('CB_load_modules called with '
            . IKDumper($current_node, Indent => 0, Maxdepth => 2) . ' and '
            . join(q(->), @{$aref_keys}));
    $LOG->debug(' ref($current_node) eq "HASH": ' . (ref($current_node) eq 'HASH' ? 1 : 0));
    if (ref($current_node) eq 'HASH') {
        $LOG->debug(' exists $current_node->{type}: ' . (exists $current_node->{type} ? 1 : 0));
        if (exists $current_node->{type}) {
            $LOG->debug(' $current_node->{type} eq "testcode": ' . ($current_node->{type} eq 'testcode'? 1 : 0));
        }
        if (exists $current_node->{testcase}) {
            $LOG->debug(' $current_node->{testcase} =~ /(.+)->(.+)$/: ' .  ($current_node->{testcase} =~ /(.+)->(.+)$/ ? 1 : 0));
        }
    }
    if (   ref($current_node) eq 'HASH'
        && exists $current_node->{testcase}
        && ((exists $current_node->{type}
                && $current_node->{type} eq 'testcode')
            || $current_node->{testcase} =~ /(.+)->(.+)$/)
        ) {
        if ($current_node->{testcase} =~ /([^()]+)([(].*[)])?/) {
            my ($m1, $m2) = ($1, $2);
            $proc_name = $m1;
            $proc_decoration = (defined $m2) ? $m2 : q();
        } else {
            $proc_name = $current_node->{testcase};
        }
        $LOG->log(sprintf('Testcase: %s, %s',
            $proc_name,
            $proc_decoration ? "decorator: $proc_decoration" : 'no decorator'
        ));
        $proc_name =~ /(.+)->(.+)$/;
        my ($m1, $m2) = ($1, $2);
        $test_class = $m1;
        $proc_name  = $m2;
        $LOG->debug(" Class: $test_class, proc: $proc_name");
        unless ($m1 && $m2) {
            $LOG->trace(q(Couldn't split testcode call for ')
                    . $current_node->{testcase}
                    . "': '$test_class', '$proc_name'");
        } else {
            if ((   exists $self->{testcode_modules}->{$test_class}
                    && $self->{testcode_modules}->{$test_class} ne 'loaded'
                )
                || !exists $self->{testcode_modules}->{$test_class}
                ) {
                if ($PSET->get('NOPRELOAD')) {
                    $LOG->log(
                        "NOPRELOAD parameter detected; not loading '$test_class' at this time"
                    );
                    $self->{testcode_modules}->{$test_class} = 'found';
                } else {
                    $LOG->log(
                        " Trying to load '$test_class' for sub '$proc_name'...");
                    load $test_class;
                    $self->{testcode_modules}->{$test_class} = 'loaded';
                }
            } else {
                $LOG->log("'$test_class' was previously found/loaded; skipping");
            }
        }
    } else {
        $LOG->debug('Nothing to do @ ' . join(q(->), @{$aref_keys}));
    }
    return;
}

__PACKAGE__->meta->make_immutable;
no Moose;
1;

=head1 NAME

NACL::AppBuilder::IceKing::Bubblegum - Configuration data processing for IceKing

=head1 SYNOPSIS

    use NACL::AppBuilder::IceKing::Bubblegum;
    my $bg = NACL::AppBuilder::IceKing::Bubblegum->new(
        yaml_file => 'v31.yml',
        hw_scale => 1,
    #    hw_scaling_factors => { low_end => 6, mid_range => 7, high_end => 8, },
    #    hw_type => 'mid_range',
    );
    say $bg->yaml_version;
    say 'Lookup is ' . ($bg->has_lookup ? '' : 'not') . ' initialized';

=head1 DESCRIPTION

This class does the heavy lifting for processing an C<IceKing> framework YAML
configuration. This includes workarounds for buggy YAML parsers, variable
interpolation, scaling and preparing it for object matching.

=cut

=head1 ATTRIBUTES

=head2 data

The configuration is stored in this hash. It start as a YAML parsed data
structure and is post-processed using _walk_tree and a number of callback
functions (see the _cb_* methods below)

=head2 yaml_version

The version of the configuration file; the current C<NACL::AppBuilder::IceKing::Bubblegum> only
supports configuration version 3 and higher.

=head2 hw_scale

The hardware scaling that is applied to all scalable object sets. It is
basically a [float] multiplier.

=head2 field_delimiter

This is used as "glue" for composite keys in object placeholder tuples. The
Configurator needs this to be ableto extract the constituents and match.

=head2 lookup

A hash containing all key=>value pairs that were instantiated in the config file
(either in the settings stream of the YAML,or through name/name_value meta
pairs. The Configurator needs it to pass it along to EnGenX.

=head2 seen

Internal hash keeping track of what part of the tree has been visited
previously. It is automatically reset at the end of _walk_tree

=head2 keyspecs

A data structure with the keyspec definitions for supported objects. It is
generated from a C<NACL::AppBuilder::IceKing::Ricardio> static method call.

=cut

=head1 METHODS

=head2 BUILD

Moose post-constructor
This is a Moose specific post-constructor method, and we use it to load the YAML
(file or string), fix parser quirks, scale, and interpolate.

=over

=item Arguments

=over

=item yaml_file

(Mutually exclusive with yaml_string)
A YAML config file

=item yaml_string

(Mutually exclusive with yaml_file)
A stringified YAML config

=item hw_scale

(It can replace a combination of hw_scaling_factors and hw_type)
An absolute HW multipler (for all objects)

=item hw_scaling_factors

A hashref of HW types and multiplication factors

=item hw_type

A key in the scaling_factors hashref

=back

=back

=head2 __construct_placeholder

Returns a placeholder value using the template and iterator
Used by _cb_scale to create a string to be replaced by Configurator with actual
objects; the value is used for debugging

=over

=item Arguments

=over

Object type

=back

=back

=head2 _update_lookup

Updates the lookup table; it also checks that a different value doesn't exist
already.
The lookup is used by the Configurator as a dictionary for environment variables
to be passed to EnGenX

=over

=item Arguments

Environment name and value

=back

=head2 _use_lookup

Uses the lookup table to resolve a variable
It updates the lookup with the default value if an entry doesn't exist already

=over

=item Arguments

Environment name and a default value

=back

=head2 _get_obj_type

Get the base keyspec object type for an object or alias
If it's an alias, return the object type, for an object, return the value that
was passed in.

=over

=item Arguments

Name to search for

=back

=head2 _get_parent

Get the parent for a node
Since 'data' is a single linked tree, we have to construct the parent object
starting from the root.

=over

=item Arguments

Root object and Array reference of keys representing the path

=back

=head2 _process_keyspec

Just broke this out of _generate_key
It will generate a [composite] key using the keyspec for the specified object
type.

=over

=item Arguments

too many

=back

=head2 _generate_key

Generate the key for an object placeholder tuple
It will use best effort for object types without keyspecs or call
_process_keyspec in case they do have them.

=over

=item Arguments

The obj, key path array ref, optional: name of obj, type of obj

=back

=head2 _was_seen

Check if we saw this reference before
It updates the 'seen' hash if not; it is used to work around circular
references.

=over

=item Arguments

Node and key path array reference

=back

=head2 _walk_tree

Visit all nodes in a hash tree recursively and run something
It calls an arbitrary callback function on each leaf

=over

=item Arguments

Node, callback, key path array reference

=back

=head2 __repl

Replace a variable with its environment value recursively
Should probably use a $Testobj method from NATE instead

=over

=item Arguments

The string to intepolate

=back

=head2 _cb_interpolate

Callback function for variable interpolation
Automatically called in the post-constructor; processes only HASH elements and
SCALAR's.

=over

=item Arguments

Node and key path array reference

=back

=head2 _cb_merge

Fix YAML merge since all Perl YAML parsers have this bug
It will detect the '<<' key and merge in the values depending on their type (and
what other leafs exist in the node).

=over

=item Arguments

Node and key path array reference

=back

=head2 _cb_fix_yaml

Fix YAML comments
Plain old 'YAML' is the only library known to have this bug

=over

=item Arguments

Node and key path array reference

=back

=head2 _cb_scale

Scale and create placeholders for objects
Couldn't lower the complexity further, got it below 100 though ;)

=over

=item Arguments

Node and key path array reference

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item SMGR Automation Team (ng-dpg-smgr-auto@netapp.com)

=back

$Id:$
Last revised $Date:$ by $Author:$

=cut

