package NACL::AppBuilder::IceKing::Jake;

use 5.014;
use warnings;

use Readonly;
use Tie::IxHash;
use Carp qw(croak);
use File::Basename qw(basename);
use Moose;

use NATE::Log;
use NATE::Result;
use NATE::ParamSet;
use NATE::Process;
use NATE::ProcManager;
use NACL::C::Client;
use NACL::AppBuilder::DpgWorkflows::EventManager;
use NACL::AppBuilder::DpgWorkflows::workflow qw(:all);
use NACL::AppBuilder::DpgWorkflows::Workload qw(:all);
use NACL::AppBuilder::IceKing::Bubblegum;
use NACL::AppBuilder::IceKing::Util qw(IKDumper separator);
use JSON -convert_blessed_universally;

my $VERSION = '3.40';
my ($LOG, $CURRENT_BLOCK_LEVEL, $TASK_PARENT, $TASK_CHILD, $TASK_CHILD_NODE);

our $EVENTMANAGER;
our %RUNID_PARENT = ();

Readonly my $RES_NAME_SEPARATOR => q(_);
Readonly my %GR_NAME_PREFIXES   => (
    serial   => 'Serial',
    parallel => 'Parallel',
);
Readonly my $GR_NAME_MID       => 'Group';
Readonly my $GR_NAME_SEPARATOR => q(_);
Readonly my %EXEC_PREFIXES     => (
    event    => 'Event',
    workflow => 'Workflow',
    Workload => 'Workload',
    testcode => 'Testcode',
    script   => 'Script',
    unknown  => 'UnknownTaskType',
);
Readonly my $EXEC_PRE_SEPARATOR     => q(_);
Readonly my $EVENT_POLLING_INTERVAL => 30;

has 'bubblegum' => (
    is       => 'rw',
    isa      => 'NACL::AppBuilder::IceKing::Bubblegum',
    required => 1,
);

has 'exec_block' => (
    is         => 'rw',
    isa        => 'NATE::ProcManager',
    required   => 1,
    lazy_build => 1,
);

has 'events' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

has 'workflows' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

has 'Workloads' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

has 'scripts' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

has 'codespecs' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

has 'levels' => (
    is       => 'rw',
    isa      => 'Int',
    required => 1,
    default  => 0,
);

use Scalar::Util;

#universal JSON conversion method
#This only converts scalar keys and skips
#other values except for command interfaces so
#that the node/cluster info is passed down.
#Other objects can be added as needed
sub UNIVERSAL::TO_JSON  {
    my $obj = shift;
    my $class = blessed $obj;
    my $out = {};
    $LOG->trace("JSON converting class:" . $class);
    foreach my $key (keys %{$obj}) {
        $LOG->debug("JSON KEY:" . $key);
        if (ref $obj->{$key} eq '') {
            $LOG->debug("converting scalar JSON key:$key");
            $out->{$key} = $obj->{$key};
        } elsif ($key =~ /command_interface|source_ci|destination_ci/) {
            #copy the raw hash, JSON will auto convert it
            $out->{$key} = $obj->{$key};
        }
        else {
            $LOG->trace("skipping non scalar JSON key type:" . ref( $obj->{$key}));
        }

    }
    $out->{__class__} = $class;
    return $out;
}

sub BUILD {
    my $self = shift;
    my %args = %{$_[0]};
    # this is how we get the constructor parameters from Moose
    $LOG = NATE::Log->new($::LOGSPEC, global => 1);
    $LOG->log("Using Jake lib v$VERSION...");
    $self->bubblegum($args{bubblegum});
    $self->bubblegum->jake($self);
    my %pm_opts = (
        proc_info  => [],
        background => 1,
        runid      => $self->bubblegum->data->[2]->{execution}->{name},
        onexit     => \&NATE::Process::on_exit_propagate_worst_result,
    );
    if (defined $self->bubblegum->data->[2]->{execution}->{run_time}) {
        $pm_opts{runtime}
            = $self->bubblegum->data->[2]->{execution}->{run_time};
    }
    if (defined $self->bubblegum->data->[2]->{execution}->{exec_count}) {
        $pm_opts{iterations}
            = $self->bubblegum->data->[2]->{execution}->{exec_count};
    }
    if (defined $self->bubblegum->data->[2]->{execution}->{exec_mode}
        && $self->bubblegum->data->[2]->{execution}->{exec_mode} eq 'parallel')
    {
        $pm_opts{parallel} = 1;
        $pm_opts{iterate_individually} = 1;
    } else {
        #$pm_opts{parallel} = 0; #was this, need below for iterations
        #per nate procmanager docs, this gives serial like operation with iterate individually
        $pm_opts{parallel} = 1;
        $pm_opts{max_active_children} = 1;
        $pm_opts{iterate_individually} = 1;
    }

    $EVENTMANAGER = create_event_manager();
    my $pm_super = NATE::ProcManager->new(%pm_opts);
    $TASK_PARENT = $pm_super;
## no critic (ProtectPrivateSubs)
    $self->bubblegum->_walk_tree($self->bubblegum->data->[2]->{execution},
        \&_cb_exec);
## use critic
# capture the last one

    $LOG->debug('Last/top TASK_CHILD '
            . IKDumper($TASK_CHILD) . 'TASK_CHILD_NODE'
            . IKDumper($TASK_CHILD_NODE));
    $LOG->debug('add children 1');
    $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
    $TASK_PARENT->add_children(
            proc_info => [$TASK_CHILD],
            make_proc_definition($TASK_CHILD_NODE),
        ) if (defined $TASK_CHILD && !$TASK_CHILD->id);
    $self->exec_block($pm_super);
    $LOG->trace('Execution block:');
    $LOG->trace(separator() . IKDumper($self->exec_block()));
    $LOG->trace('    ->    events: ' . $self->events());
    $LOG->trace('    -> workflows: ' . $self->workflows());
    $LOG->trace('    -> Workloads: ' . $self->Workloads());
    $LOG->trace('    ->   scripts: ' . $self->scripts());
    $LOG->trace('    -> codespecs: ' . $self->codespecs());
    $LOG->trace(separator() . '    (' . $self->levels() . ' level(s) deep)');
    return;
}


sub create_event_manager {
    my @clients = NACL::C::Client->find(allow_empty => 1);
    if (!@clients) {
        $LOG->warn('No clients found, using "localhost"');
        push @clients, 'localhost';
    } else {
        @clients =  map { $_->{client} } @clients;
    }
    $LOG->comment('Clients: ' . join(q{, }, @clients));
    my $evt_mgr = NACL::AppBuilder::DpgWorkflows::EventManager->new(
        client_list => \@clients,
        #runid_base  => $runid,
    );

    return $evt_mgr;
}


## no critic (ProhibitExcessComplexity, ProhibitDeepNests)
sub _cb_exec {
## use critic
    my ($bg, $current_node, $aref_keys) = @_;
    $LOG->enter();
    $LOG->trace('CB_exec called with '
            . IKDumper($current_node) . ' and '
            . IKDumper($aref_keys));
    $LOG->trace('Processing ' . ref($current_node));
    if (ref($current_node) eq 'HASH') {
        if (!defined($CURRENT_BLOCK_LEVEL) && (@{$aref_keys} == 0)) {
## We are just starting
            $LOG->debug('Level: starting');
            $CURRENT_BLOCK_LEVEL = 1;
            $bg->jake->levels(1);
        } elsif ((@{$aref_keys} > 0) && ($aref_keys->[-1] eq 'task_group')) {
### We entered a task group section
            $LOG->debug('Level: entering task group:' . $current_node->{name});
            if ($CURRENT_BLOCK_LEVEL > scalar @{$aref_keys}) {
### we went backwards, move up
                $LOG->debug('TG   : and went backwards');
            $LOG->debug(':' . $current_node->{name});
                #task group:
                $LOG->debug('add children 2');
                $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
                $TASK_PARENT->add_children(
                    proc_info => [$TASK_CHILD],
                    make_proc_definition($TASK_CHILD_NODE),
                );
                $CURRENT_BLOCK_LEVEL -= 3;
                while ($CURRENT_BLOCK_LEVEL > scalar @{$aref_keys}) {
                    $TASK_CHILD  = $TASK_PARENT;
                    $TASK_CHILD_NODE = $current_node;
                    $TASK_PARENT = $TASK_PARENT->get_parent();
                    $CURRENT_BLOCK_LEVEL -= 3;
                }
            } else {
### we went deeper
                $LOG->debug('TG   : and went deeper');
            $LOG->debug(':' . $current_node->{name});
                $CURRENT_BLOCK_LEVEL = scalar @{$aref_keys};
                if (($CURRENT_BLOCK_LEVEL / 3) > $bg->jake->levels) {
                    $bg->jake->levels($CURRENT_BLOCK_LEVEL / 3);
                }
                if (defined $TASK_CHILD) {
### we are not at the top level, move down
                    $LOG->debug(
                        'TG   : not at top level, and child exists - point parent to current child'
                    );
            $LOG->debug(':' . $current_node->{name});
                    $LOG->debug('add children 3');
                    $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
                    $TASK_PARENT->add_children(
                        proc_info => [$TASK_CHILD],
                        make_proc_definition($TASK_CHILD_NODE),
                    );
                    $TASK_PARENT = $TASK_CHILD;
                } else {
### We are at the top level and there is no child
                    $LOG->debug(
                        'TG   : not at top level, and there is no child - do nothing'
                    );
            $LOG->debug(':' . $current_node->{name});
                }
            }
### create a new child (we are either at top and need one, or we just went down or up)
            my $gr_parallel = (exists $current_node->{exec_mode}
                    && ($current_node->{exec_mode} eq 'parallel')) ? 1 : 0;
            my $gr_runid = (
                  $gr_parallel
                ? $GR_NAME_PREFIXES{parallel}
                : $GR_NAME_PREFIXES{serial}
                )
                . $GR_NAME_SEPARATOR
                . $GR_NAME_MID
                . $GR_NAME_SEPARATOR
                . $current_node->{name};
            $LOG->debug('TG   : create new PM');
            $LOG->debug(':' . $current_node->{name});
            $TASK_CHILD = NATE::ProcManager->new(
                proc_info  => [],
                runid      => $gr_runid,
                ($gr_parallel ?
                    (
                        parallel   => $gr_parallel,
                        iterate_individually   => 1,
                    ) :
                      #per nate procmanager docs, this gives
                      #serial like operation with iterate individually
                    (
                        #parallel   => 0,
                        parallel   => 1,
                        iterate_individually   => 1,
                        max_active_children    => 1,
                    )
                ),
                background => 0,
            );
            $TASK_CHILD_NODE = $current_node;
        } elsif ((@{$aref_keys} > 0) && ($aref_keys->[-1] eq 'task')) {
### repeat from above
            $LOG->debug('Level: entering task');
            if ($CURRENT_BLOCK_LEVEL > scalar @{$aref_keys}) {
### we went backwards, move up
                $LOG->debug('T    : and went backwards');
                $LOG->debug('add children 4');
                $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
                $TASK_PARENT->add_children(
                    proc_info => [$TASK_CHILD],
                    make_proc_definition($TASK_CHILD_NODE),
                );
                while ($CURRENT_BLOCK_LEVEL > scalar @{$aref_keys}) {
                    $TASK_CHILD = $TASK_PARENT;
                    $TASK_CHILD_NODE = $current_node;
                    $TASK_PARENT = $TASK_PARENT->get_parent() || undef;
                    $CURRENT_BLOCK_LEVEL -= 3;
                }
            } else {
### we went deeper
                $LOG->debug('T    : and went deeper');
                $CURRENT_BLOCK_LEVEL = scalar @{$aref_keys};
            }
### END repeat from above
            my (%p, $max_cnt, $cur_cnt);
            foreach my $res_name (keys %{$current_node->{param}}) {
# ignore non-resource parameters
                $LOG->debug('T    : deal with params');
                unless (
                    ref($current_node->{param}->{$res_name}) eq 'HASH'
                    && ((   exists $current_node->{param}->{$res_name}
                            ->{resources})
                        || (exists $current_node->{param}->{$res_name}
                            ->{resource_group})
                    )
                    ) {
                    $p{$res_name} = $current_node->{param}->{$res_name};
                    $LOG->debug('T    : skip ' . $res_name);
                    next;
                }
                my $res_type;
                my $pattern = $current_node->{param}->{$res_name}->{filter};
                $LOG->debug('T    : filter ' . $pattern) if $pattern;
                $p{$res_name} = [];
                $max_cnt = $current_node->{param}->{$res_name}->{max_count}
                    // 0;
                $cur_cnt = 0;
                if ($res_name
                    =~ /([^$RES_NAME_SEPARATOR]+)($RES_NAME_SEPARATOR?)(\w*)/x)
                {
                    my ($m1, $m3) = ($1, $3);
                    $res_type = (defined $m3) ? $m1 : $res_name;
                }    # should always match

                if (ref($current_node->{param}->{$res_name}->{resource_group})) {
					#$LOG->debug('T    : RG   :' . IKDumper
					#$current_node->{param}->{$res_name}->{resource_group});
				}

                if (ref($current_node->{param}->{$res_name}->{resource_group})
                    eq 'ARRAY') {
                    $LOG->debug('T    : RG   : array');

				}

                if (ref($current_node->{param}->{$res_name}->{resource_group})
                    eq 'HASH') {
                RESOURCE_GROUP:
                    $LOG->debug('T    : RG   : process');
                    foreach my $subgr (
                        keys
                        %{$current_node->{param}->{$res_name}->{resource_group}}
                        ) {
                        for (
                            my $i = 0;
                            $i < scalar @{
                                $current_node->{param}->{$res_name}
                                    ->{resource_group}->{$subgr}
                            };
                            $i++
                            ) {
                            if (exists $current_node->{param}->{$res_name}
                                ->{resource_group}->{$subgr}->[$i]->{$res_type}
                                ->{objects}) {
                                foreach my $l (
                                    keys %{
                                        $current_node->{param}->{$res_name}
                                            ->{resource_group}->{$subgr}->[$i]
                                            ->{$res_type}->{objects}
                                    }
                                    ) {
                                    if ($pattern) {
                                        if ($l =~ /$pattern/) {
                                            $LOG->trace(
                                                "'$l' matched pattern '$pattern'"
                                            );
                                        } else {
                                            $LOG->trace(
                                                "'$l' did not match pattern /$pattern/"
                                            );
                                            next;
                                        }
                                    }
                                    push @{$p{$res_name}},
                                        $current_node->{param}->{$res_name}
                                        ->{resource_group}->{$subgr}->[$i]
                                        ->{$res_type}->{objects}->{$l};
                                    if (++$cur_cnt == $max_cnt) {
                                        $LOG->log(
                                            "Found MAX GROUP '$res_type' elements ($max_cnt)"
                                        );
                                        last RESOURCE_GROUP;
                                    }
                                }
                            } else {
                                $LOG->error('No objects found!');
                            }
                        }
                    }
                } elsif (ref($current_node->{param}->{$res_name}->{resource_group})
                    eq 'ARRAY') {
                RESOURCE_GROUP:
                    $LOG->debug('T    : RG   : process (array)');
                    for (
                        my $j = 0;
                        $j < scalar
                        @{$current_node->{param}->{$res_name}->{resource_group}};
                        $j++
                        ) {
                            $LOG->debug("T    : RG   : res type $res_type " . $bg->_get_obj_type($res_type) );
                            $LOG->debug("T    : RG   : $j : " .
								join ',', keys %{$current_node->{param}->{$res_name}
                                    ->{resource_group}->[$j]});
							foreach my $subgr (
							keys
							%{$current_node->{param}->{$res_name}->{resource_group}->[$j]}
							) {
								#for (
								#my $i = 0;
								#$i < scalar @{
								#$current_node->{param}->{$res_name}
								#->{resource_group}->{$subgr}
								#};
								#$i++
								#) {
                                if ($bg->_get_obj_type($res_type) eq $bg->_get_obj_type($subgr) &&
									exists $current_node->{param}->{$res_name}
                                    ->{resource_group}->[$j]->{$subgr}
                                    ->{objects}) {
                                    foreach my $l (
                                        keys %{
                                            $current_node->{param}->{$res_name}
                                                ->{resource_group}->[$j]
                                                ->{$subgr}->{objects}
                                        }
                                        ) {
                                        if ($pattern) {
                                            if ($l =~ /$pattern/) {
                                                $LOG->trace(
                                                    "'$l' matched pattern '$pattern' - array"
                                                );
                                            } else {
                                                $LOG->trace(
                                                    "'$l' did not match pattern /$pattern/ - array"
                                                );
                                                next;
                                            }
                                        }
                                        push @{$p{$res_name}},
                                            $current_node->{param}->{$res_name}
                                            ->{resource_group}->[$j]
                                            ->{$subgr}->{objects}->{$l};
                                        if (++$cur_cnt == $max_cnt) {
                                            $LOG->log(
                                                "Found MAX GROUP '$res_type' elements ($max_cnt)"
                                            );
                                            last RESOURCE_GROUP;
                                        }
                                    }
                                } else {
                                    $LOG->error('No objects found!');
                                }
								#}
							}
                    }
                } elsif (
                    ref($current_node->{param}->{$res_name}->{resources}) eq
                    'ARRAY') {
                RESOURCE:
                    $LOG->debug('T    : R    : process');
                    for (
                        my $i = 0;
                        $i < scalar
                        @{$current_node->{param}->{$res_name}->{resources}};
                        $i++
                        ) {
# this is for object group references
                        if (exists $current_node->{param}->{$res_name}
                            ->{resources}->[$i]->{$res_type}->{objects}) {
                            foreach my $l (
                                keys %{
                                    $current_node->{param}->{$res_name}
                                        ->{resources}->[$i]->{$res_type}
                                        ->{objects}
                                }
                                ) {
                                if ($pattern) {
                                    if ($l =~ /$pattern/) {
                                        $LOG->trace(
                                            "'$l' matched pattern '$pattern'");
                                    } else {
                                        $LOG->trace(
                                            "'$l' did not match pattern /$pattern/"
                                        );
                                        next;
                                    }
                                }
                                push @{$p{$res_name}},
                                    $current_node->{param}->{$res_name}
                                    ->{resources}->[$i]->{$res_type}->{objects}
                                    ->{$l};
                                if (++$cur_cnt == $max_cnt) {
                                    $LOG->log(
                                        "Found MAX ARRAY '$res_type' elements ($max_cnt)"
                                    );
                                    last RESOURCE;
                                }
                            }
                        } else {
                            $LOG->error('No objects found!');
                        }
                        $LOG->log(
                            "Still haven't reached MAX '$res_type' elements ($cur_cnt < $max_cnt)"
                        ) if ($max_cnt);
                    }
# this is for individual object set references
                } elsif (
                    ref($current_node->{param}->{$res_name}->{resources}) eq
                    'HASH') {
                    $LOG->debug('T    : I    : process');
                    foreach my $l (
                        keys %{
                            $current_node->{param}->{$res_name}->{resources}
                                ->{objects}
                        }
                        ) {
                        push @{$p{$res_name}},
                            $current_node->{param}->{$res_name}->{resources}
                            ->{objects}->{$l};
                        if (++$cur_cnt == $max_cnt) {
                            $LOG->trace(
                                "Found MAX HASH '$res_type' elements ($max_cnt)"
                            );
                            last;
                        } else {
                            $LOG->trace(
                                "Still haven't reached MAX '$res_type' elements ($cur_cnt < $max_cnt)"
                            ) if ($max_cnt);
                        }
                    }
                }
            }
            $current_node->{proc_param} = \%p;
            my ($proc_name, $proc_prefix, $test_class, $testcase_type);
            $LOG->debug('T    : handle testcase types');
            if (defined $current_node->{testcase}) {
                $proc_name = $current_node->{testcase};
            } else {
                $LOG->error('This task has no "testcase:" entry!');
                return;
            }
            my $proc_decoration;
            my $proc_decoration_simple;
            if ($proc_name =~ /([^()]+)([(](.*)[)])?/) {
                my ($m1, $m2, $m3) = ($1, $2, $3);
                $proc_name = $m1;
                $proc_decoration        = (defined $m2) ? $m2 : q();
                $proc_decoration_simple = (defined $m3) ? $m3 : q();
            }
            $LOG->debug("Processing proc_name: '$proc_name'; decoration: '$proc_decoration' simple: '$proc_decoration_simple'");
            if (defined $current_node->{type}) {
                $testcase_type = $current_node->{type};
            } else {
# rudimentary testcase type detection
                if ($proc_name =~ /[.](sh|py|thpy|pl|thpl)$/) {
                    $testcase_type = 'script';
                } elsif ($proc_name =~ /(.+)->(.+)$/) {
                    $testcase_type = 'testcode';
                } else {
                    $testcase_type = 'unknown';
                }
            }
            $LOG->comment("Testcase type is '$testcase_type'");
            $proc_prefix = $EXEC_PREFIXES{$testcase_type} . $EXEC_PRE_SEPARATOR;
            my $script_full_proc_name;
            if ($testcase_type eq 'event') {
                $bg->jake->events($bg->jake->events + 1);
            } elsif ($testcase_type eq 'workflow') {
                $bg->jake->workflows($bg->jake->workflows + 1);
            } elsif ($testcase_type eq 'Workload') {
                $bg->jake->Workloads($bg->jake->Workloads + 1);
            }elsif ($testcase_type eq 'testcode') {
                if ($proc_name =~ /(.+)->(.+)$/) {
                    my ($m1, $m2) = ($1, $2);
                    $test_class = $m1;
                    $proc_name  = $m2;
                } else {
                    $proc_name = "BAD_CODESPEC_FORMAT_$proc_name";
                }
                $bg->jake->codespecs($bg->jake->codespecs + 1);
            } elsif ($testcase_type eq 'script') {
                $script_full_proc_name = $proc_name;
                $proc_name = basename($proc_name);
                $bg->jake->scripts($bg->jake->scripts + 1);
            }
            $LOG->debug('T    : create process');
            my $runid
                = "$EXEC_PREFIXES{$testcase_type}$EXEC_PRE_SEPARATOR$proc_name" . ($proc_decoration_simple ? "$EXEC_PRE_SEPARATOR$proc_decoration_simple" : '');
            $LOG->trace('qc_test_name is '.$current_node->{qc_test_name}) if (defined $current_node->{qc_test_name});
            $LOG->trace('runid is '.$runid);
            my $name = (defined $current_node->{qc_test_name})
                            ? $current_node->{qc_test_name}
                            : $runid;
            $LOG->trace('testcase name is '.$name);

            #we have to create the event worker outside the closure below so
            #the event manager is modified in this thread
            #
            my $event_worker;

            if ($testcase_type eq 'event') {
                #we have to create the event worker here before we fork
                #so clients iterate correctly
                my $runid_prefix;
                if (defined $TASK_CHILD) {
                    $runid_prefix = $TASK_CHILD->runid;
                    #the first time through we need to set it here
                    $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid;
                    #$LOG->comment('CHILD:' . IKDumper($TASK_CHILD));
                } else {
                    $runid_prefix = $TASK_PARENT->runid;
                    #$LOG->comment('PARENT:' . IKDumper($TASK_PARENT));
                }

                my $evt_mgr = $EVENTMANAGER;

                $LOG->debug("RUNID PARENT at runid base for worker:" . IKDumper(\%RUNID_PARENT));

                #recreate the full base path built up by nate::process and nate::procmanagers

                my $runid_base = "$RUNID_PARENT{$runid_prefix}/$runid_prefix/$runid";
                $LOG->trace("runid base for worker ='$runid_base', runid='$runid', prefix='$runid_prefix'");

                $event_worker = $evt_mgr->create_worker(
                    event        => $proc_name,
                    event_params => $current_node->{proc_param},
                    runid_base        => $runid_base,
                );
            }

            my $json = JSON->new();
            $json->allow_blessed(1);
            $json->convert_blessed(1);
            #use NATE::Encode qw(encode_global);


            my $procenv = {};
            #This should be generalized or another way of passing environment down should be implemented.
            $procenv->{FILER_TIMEOUT_SCALING_FACTOR} = $ENV{FILER_TIMEOUT_SCALING_FACTOR};
            $procenv->{NACL_SNAPMIRROR_DP_OPTION} = $ENV{NACL_SNAPMIRROR_DP_OPTION};
            $procenv->{NACL_SNAPMIRROR_DP} = $ENV{NACL_SNAPMIRROR_DP};
            $procenv->{NATE_CONNECTREC_TIMEOUT} = $ENV{NATE_CONNECTREC_TIMEOUT};


            #if it's a python script pass params through env vars, nate doesn't yet support params -> python
            if (($testcase_type eq 'script') && ( ($runid =~ /py$/) || ($runid =~ /py_/) ) ) {  #if python, need env vars, params not supported yet
                $procenv->{'igniter_params'} = $json->encode($current_node->{proc_param});
                $procenv->{'NATE_HOSTS'} = $ENV{'NATE_HOSTS'}; #the same, this param isn't auto propagated
            }

            my $proc = NATE::Process->new(
                runid    => $runid,
                testcase => $name,
                #testcase => (defined $current_node->{qc_test_name})
                #? $current_node->{qc_test_name}
                #: $runid,
                onexit   => \&igniter_exit_handler,
                #onexit   => \&NATE::Process::on_exit_propagate_worst_result,
                (scalar keys %{$procenv} ? (env => $procenv) : () ),
                #if it's a python script pass params through env vars, nate doesn't yet support params -> python
                ($testcase_type eq 'script'
                    ? (params => $current_node->{proc_param},)
                    #? (params => $current_node->{proc_param},
                    #($name =~ /py$/ #if python, need env vars, params not supported yet
                    ##? (env => $current_node->{proc_param}) #raw
                    ##this "works" with raw objects but fails passing via staf
                    ##however the UNIVERSAL::TO_JSON method at the top pulls out the keys and does work
                    #? (env => {'igniter_params' => $json->encode($current_node->{proc_param}),
                    #'NATE_HOSTS' => $ENV{'NATE_HOSTS'},
                    #})
                    ##? (env => {'params' => encode_global(type => 'json')->allow_blessed()->convert_blessed()->encode($current_node->{proc_param})})
                    #: ()))
                    : ()
                ),
                #execute on this client if it's defined
                ( $current_node->{client} ?
                    (client => $current_node->{client} )
                    : ()
                ),
                codespec => $testcase_type eq 'script'
                #? $current_node->{testcase}  #what we really want, but we don't want decorators (decorator1)
                ? $script_full_proc_name
                : sub {
                    my $pset = NATE::ParamSet->new(global => 1);
                    if (exists $current_node->{start_delay}) {
                        my $start_delay;
                        $current_node->{start_delay} =~ /(random[(])(\d+)[)]/;
                        if (defined $2) {
                            $LOG->comment("Found a random delay, max is $2...");
                            $start_delay = eval { int(rand($2)) };
                        } else {
                            $start_delay = $current_node->{start_delay};
                        }
                        $LOG->comment(
                            "Process '$proc_name' delaying start ${start_delay}s..."
                        );
                        Tharn::snooze($start_delay);
                    }
                    $LOG->comment("Process '$proc_name' starting with: "
                            . IKDumper($current_node->{proc_param}, Maxdepth => 3));
                    if ($testcase_type eq 'event') {
# Event codepath
                        $bg->jake->events($bg->jake->events + 1);
                        $LOG->step("Running event '$proc_name'...");
                        my $evt_mgr = $EVENTMANAGER;
                        $evt_mgr->worker_start(worker => $event_worker);
                        $evt_mgr->worker_wait(worker => $event_worker);
                    } elsif ($testcase_type eq 'workflow') {
# Workflow codepath
                        $bg->jake->workflows($bg->jake->workflows + 1);
                        $LOG->step("Running workflow '$proc_name'...");
## no critic (ProhibitNoStrict)
                        no strict 'refs';
## use critic
                        $proc_name->(%{$current_node->{proc_param}});
                        use strict 'refs';
                    } elsif ($testcase_type eq 'Workload') {
# Workload codepath
                        $bg->jake->Workloads($bg->jake->Workloads + 1);
                        $LOG->step("Running Workload '$proc_name'...");
## no critic (ProhibitNoStrict)
                        no strict 'refs';
## use critic
                        $proc_name->(%{$current_node->{proc_param}});
                        use strict 'refs';
                    } elsif ($testcase_type eq 'testcode') {
# Testcode codepath
                        $bg->jake->codespecs($bg->jake->codespecs + 1);
                        $LOG->step("Running testcode '$proc_name'...");
## no critic (ProhibitNoStrict)
                        no strict 'refs';
## use critic
                        $test_class->$proc_name(%{$current_node->{proc_param}});
                        use strict 'refs';
                    } else {
                        $LOG->error(q(Don't know how to run this: ')
                                . $current_node->{testcase}
                                . q('!));
                    }
                    $LOG->comment("Process '$proc_name' finished...");
                },
            );
            if (defined $TASK_CHILD) {
                $LOG->debug('add children 5');
                $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
                $TASK_CHILD->add_children(
                    proc_info => [$proc],
                    make_proc_definition($current_node),
                );
            } else {
                $LOG->debug('add children 6');
                $RUNID_PARENT{$TASK_CHILD->runid} = $TASK_PARENT->runid; #for events
                $TASK_PARENT->add_children(
                    proc_info => [$proc],
                    make_proc_definition($current_node),
                );
            }
        }
    }
    $LOG->exit();
    return;
}

#generates proc_definition key/value for procmanager calls
#see http://web.eng.netapp.com/engineering/nate/releases/latest/html/NATE/ProcManager.html
#for docs under proc_definition
#NOT method
sub make_proc_definition {
    my $current_node = shift;

    $LOG->debug('make_proc_definition:' . IKDumper $current_node);

    return () unless $current_node;

    my @accepted = qw ( iterations iteration_delay iteration_delay_min
        iteration_delay_max interval interval_min
        interval_max runtime max_instances kill_to_stop kill_timeout );

    my %pd = ( proc_definition => [ {} ] );

    foreach my $key (@accepted) {
        if (defined $current_node->{$key}) {
            $pd{proc_definition}->[0]->{$key} = $current_node->{$key};
        }
    }

    if (scalar keys $pd{proc_definition}[0]) {
        $LOG->trace('make_proc_definition result:' . IKDumper \%pd);
        return %pd;
    } else {
        $LOG->trace('make_proc_definition result: empty');
        return ();
    }
}

# handle results
sub igniter_exit_handler {
    my $self = shift;
    $LOG->log('INSIDE IGNITER EXIT HANDLER');
    my $worst_result = $self->worst_result;
    my $name = $self->{testcase}{name};
    my $runid = $self->{runid};
    ($runid) = ($runid=~/\/(.*)$/); # remove the first part which is 01_reigniter
    $LOG->log('runid is '.$runid);
    my $status;

    if ($worst_result) {
        #$LOG->log("WORST RESULT is $worst_result");
        my $result_type = $worst_result->type;
        $LOG->log("WORST RESULT type is $result_type");
        $worst_result->log();
    } else {
        $LOG->warn("Unable to get worst result for testcase $name");
        return;
    }
    if ($name) {
        my @testcases = split(/,/,$name);
        $LOG->log("testcases are @testcases");
        if (scalar (@testcases) > 1 ) {
            $LOG->log('Multiple testcases present');
            foreach my $testcase (@testcases) {
                $LOG->log("Creating PROCESS for $testcase");
                my $proc = NATE::Process->new(
                    runid    => $runid,
                    testcase => $testcase,
                    onexit   => \&NATE::Process::on_exit_propagate_worst_result,
                    codespec => (NATE::Result->is_failure($worst_result))
                    ? sub {
                        $LOG->log('Inside new process to log results');
                        $LOG->log(NATE::Result::Fail->new($testcase));
                        }
                    : sub {
                        $LOG->log('Inside new process to log results');
                        $LOG->log(NATE::Result::Pass->new($testcase));
                        }
                );
                $proc->start();
                $proc->wait();
                $proc->destroy();
            }
        } else {
            $LOG->log('Single qc_test_case name. No subprocess to be launched to log results.');
        }
    } else {
        $LOG->warn('NULL TESTCASES');
    }
}

__PACKAGE__->meta->make_immutable;
no Moose;
1;
