
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-nacl-dev
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::SC::Controller

=head1 DESCRIPTION

NACL::AppBuilder::SC::Controller is used to Config, Setup, Cleanup and
Deconfig a controller.

=cut

package NACL::AppBuilder::SC::Controller;

# Compiler directives.
use strict;

# NATE and Standard Module import
use Tharn;

#use San::SanMgr::Util;
use Data::Dumper;
use NATE::Util::DataTypes qw(TRUE FALSE);
use Tie::IxHash;
use Time::HiRes;

# Exceptions Imported
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use Params::Validate
    qw(OBJECT SCALAR SCALARREF ARRAYREF HASHREF UNDEF GLOBREF);

# NACL libraries in use
use NACL::C::Node;
use NACL::STask::Cluster;

# Load resource specific modules
use NACL::AppBuilder::SC::Controller::Aggregate;
use NACL::AppBuilder::SC::Controller::Vserver;
use NACL::AppBuilder::SC::Controller::Lif;
use NACL::AppBuilder::SC::Controller::Volume;
use NACL::AppBuilder::SC::Controller::Qtree;
use NACL::AppBuilder::SC::Controller::Lun;
use NACL::AppBuilder::SC::Controller::Volume_Efficiency;

use vars qw(@ISA);

@ISA = qw(NACL::AppBuilder::EnGenX::Util);

###############################################################################
#                               Public Methods
###############################################################################

#==============================================================================
# Method:     _new
# Objective:  Create new resource object.
# Details:    Steps: 1)   TBD
# Inputs:     TBD
# Outputs:    TBD
# Exceptions: Argument
#             BaseException
#==============================================================================
sub _new {
    my ($proto, %args) = @_;
    $Log->enter();

    my $self = {};

    my $class = ref($proto) || $proto;
    bless($self, $class);

    &ParamsMgr::process_args(%args);
    my $parent = &ParamsMgr::get_arg(
        -arg      => "parent",
        -required => 1,
        -default  => ""
    );
    if (&ParamsMgr::error()) {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            "Missing required argument parent: " . &ParamsMgr::get_error());
    }

    # Save parent
    $self->{parent} = $parent;

    # Get the set of environment parameters from ParamsMgr
    my $environment = &ParamsMgr::search(
        search_path => [],
        key         => "ENVIRONMENT"
    );
    if (&ParamsMgr::error()) {
        $Log->exit();
        NATE::BaseException->throw(
            "Unable to find ENVIRONMENT: " . &ParamsMgr::get_error());
    }

    $Log->exit();
    return $self;
}    # end sub new()

=head1 METHODS

=head2 setup

Launch subtest for each controller to perform setup

  $self->{sm_controller_object}->setup(
                 run_id_prefix => $self->{run_id_prefix},
                 resource_list => \%{$self->{resource_list}});

=over

=item Options

TBD

=over

=item C<< TBD >>

TBD

=back

=item Return

TBD

=item Exceptions

BaseException

=cut

sub setup {
    my ($self, %args) = @_;
    $Log->enter();
    # Locals
    # Create a dispatch with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %dispatch;
    tie(%dispatch, 'Tie::IxHash');

    # Note: The order of the types (keys) in the hash will be the same at the
    #       time of the execution, so, in this case the first type to be
    #       executed will be Igroup, then Initiator and so on. It is important
    #       to preserve the order.
    %dispatch = (
        'Aggregate' =>
            sub { NACL::AppBuilder::SC::Controller::Aggregate->create(@_) },
        'Vserver' =>
            sub { NACL::AppBuilder::SC::Controller::Vserver->create(@_) },
        'Volume' =>
            sub { NACL::AppBuilder::SC::Controller::Volume->create(@_) },
        'Volume_Efficiency' => sub {
            NACL::AppBuilder::SC::Controller::Volume_Efficiency->create(@_);
        },
        'Qtree' => sub {
            NACL::AppBuilder::SC::Controller::Qtree->create(@_);
        },
        'Lun' =>
            sub { NACL::AppBuilder::SC::Controller::Lun->create(@_) },
        'Lif' => sub { NACL::AppBuilder::SC::Controller::Lif->create(@_) },
    );

    $Log->exit();
    return $self->_controller_execution_helper(
        dispatch_table => \%dispatch,
        action         => "SetUp",
        %args
    );

} ## end sub setup


sub _config {
    my ($self, %args) = @_;
    $Log->enter();

    # Locals
    # Create a dispatch with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %dispatch;
    tie(%dispatch, 'Tie::IxHash');

   # Note: The order of the types (keys) in the hash will be the same at the
   #       time of the execution, so, in this case the first type to be
   #       executed will be Aggregate, then Vserver and so on. It is important
   #       to preserve the order.
    %dispatch = (
        'Volume' =>
            sub { NACL::AppBuilder::SC::Controller::Volume->create(@_) },
        'Lun' =>
            sub { NACL::AppBuilder::SC::Controller::Lun->create(@_) },
        'Lif' => sub { NACL::AppBuilder::SC::Controller::Lif->create(@_) },

    );

    $self->_controller_execution_helper(
        dispatch_table => \%dispatch,
        action         => "Config",
        %args
    );
    $Log->comment("Cluster config completed");
    $Log->exit();
    return TRUE;
} ## end sub _config

sub _deconfig {
    my ($self, %args) = @_;
    $Log->enter();

    # Locals
    # Create a dispatch table with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %dispatch;
    tie(%dispatch, 'Tie::IxHash');

    # Note: The order of the types (keys) in the hash will be the same at the
    #       time of the execution, so, in this case the first type to be
    #       executed will be Lif, then Vserver and so on. It is important
    #       to preserve the order.
    %dispatch = (
        'Lif' => sub { NACL::AppBuilder::SC::Controller::Lif->delete(@_) },
        'Vserver' =>
            sub { NACL::AppBuilder::SC::Controller::Vserver->delete(@_) },
        'Aggregate' =>
            sub { NACL::AppBuilder::SC::Controller::Aggregate->delete(@_) },
    );

    $Log->exit();
    return $self->_controller_execution_helper(
        dispatch_table => \%dispatch,
        action         => "Deconfig",
        %args
    );
} ## end sub _deconfig

=head2 cleanup

Launch subtest for each controller to perform cleanup

  $self->{sm_controller_object}
        ->cleanup(run_id_prefix => $self->{run_id_prefix},
                 resource_list => \%{$self->{resource_list}});

=over

=item Options

TBD

=over

=item C<< TBD >>

TBD

=back

=item Return

TBD

=item Exceptions

BaseException

=cut

sub cleanup {
    my ($self, %args) = @_;
    $Log->enter();

    # Locals
    # Create a dispatch table with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %dispatch;
    tie(%dispatch, 'Tie::IxHash');

    # Note: The order of the types (keys) in the hash will be the same at the
    #       time of the execution, so, in this case the first type to be
    #       executed will be Igroup, then Initiator and so on. It is important
    #       to preserve the order.
    %dispatch = (
        'Lun' => sub { NACL::AppBuilder::SC::Controller::Lun->delete(@_) },
        'Qtree' =>
            sub { NACL::AppBuilder::SC::Controller::Qtree->delete(@_) },
        'Volume_Efficiency' =>
            sub { NACL::AppBuilder::SC::Controller::Volume_Efficiency->delete(@_) },
        'Volume' =>
            sub { NACL::AppBuilder::SC::Controller::Volume->delete(@_) },
        'Lif' => sub { NACL::AppBuilder::SC::Controller::Lif->delete(@_) },
        'Vserver' =>
            sub { NACL::AppBuilder::SC::Controller::Vserver->delete(@_) },
        'Aggregate' =>
            sub { NACL::AppBuilder::SC::Controller::Aggregate->delete(@_) },
    );
    $self->_controller_execution_helper(
        dispatch_table => \%dispatch,
        action         => "CleanUp",
        %args
    );

    $Log->exit();
    return TRUE;
} ## end sub cleanup

=head2 validate

Runs set of validation methods to verify that controllers are
in expected state.  Currently controller validate is a no-op.

  $self->{sm_controller_object}->validate();

=over

=item Options

None

=back

=item Return

TRUE

=item Exceptions

None

=cut

sub validate {
    my ($self, %args) = @_;

    # TODO: Add controller cluster validation
    $Log->comment("\n\nCurrently No cluster validation is done in "
            . "SC::Controller::validate()\n\n");
    return TRUE;
} ## end sub validate

#==============================================================================
# Method:     _controller_execution_helper
# Objective:  Abstract the duplicated code from config, setup, deconfig and
#             cleanup and created a simple way of adding/delting resources to
#             the controller.
# Details:    Steps: 1)   Get all the categories that have information about
#                         the controller, basically all keys that start with
#                         CLUSTER
#                    2)   For each found CLUSTER key get all the TYPEs (vserver,
#                         vol, lun, etc) from the dispatch table.
#                    3)   For each found TYPE get the resources (resource_list)
#                    4)   Create each resource.
# Inputs:     dispatch_table. The hash table that holds the 'key' that is the
#                             type of the resource and the subrutine to be
#                             executed.
#             resourece_list. The SC xml resource list.
#             types. Depending on the caller, there could be different types.
#                    (this discrepancy could be eliminated when setup and
#                     config are consolidated).
#             run_id. Id to identify the subpresses that could be created.
# Outputs:    Boolean, True if no problem was encountered.
# Exceptions: Arguments exception if the passed arguments are wrong.
# TODO: Remove the resource_list argument in the logic and get the resource
#       list from $self ($self->{resource_list}.
#       Also remove the resource list argument on the caller (SC::setup)
#
#==============================================================================
sub _controller_execution_helper {
    my ($self, @opts) = @_;
    $Log->enter();

    # Locals
    my %args = ();

    # Process arguments.
    # Allowing extra arguments to be used when challing
    # ResourceList->get_resources()
    try {
        %args = Params::Validate::validate_with(
            params => \@opts,
            spec   => {
                resource_list  => {type => HASHREF, optional => FALSE},
                dispatch_table => {type => HASHREF, optional => FALSE},
                action         => {type => SCALAR,  optional => FALSE},
                run_id_prefix  => {
                    type     => SCALAR,
                    optional => TRUE,
                    default  => "SC_"
                },
            }
        );
    }
    otherwise {
        my $xcptn = shift;
        $Log->exit();
        NATE::Exceptions::Argument->throw($xcptn->text());
    };

    # Gather values
    # Delete the arguments so that ARGs only holds the information the user
    # intends to use on Resource_List->get_resources()
    my $resource_list  = delete $args{resource_list};
    my $dispatch_table = delete $args{dispatch_table};
    my $action         = delete $args{action};
    my $run_id_prefix  = delete $args{run_id_prefix};
    my @clusters = $resource_list->get_categories(category => '^CLUSTER_');
    # Create the correct filters for cleanup and setup
    # The filter hash will filter out objects that dont need to be cleaned up
    # or setup based on the action to be perform so only need resources are
    # put through the processing engine.
    my %filter_hash = ();
    if ($action =~ /deconfig/i || $action =~ /cleanup/i) {
        %filter_hash = (
            _cleanup      => 1,
            _cleanup_done => 0
        );
    } else {
        %filter_hash = (
            _setup      => 1,
            _setup_done => 0
        );
    }

    # Setup each cluster. Assumes a Resource List:
    # [cluster_<name1> => Aggr => array_of_aggr_resources,
    #                     VOL => array_of_volumes
    # ],
    # [cluster_<name2> => Aggr => array_of_aggr_resources,
    #                     VOL => array_of_volumes
    # ]...
    $Log->debug("Resource List before $action is:\n" . Dumper($resource_list))
        if ($Log->may_debug());

    # Dispatch all the components for each found Cluster
    foreach my $cluster (@clusters) {

        # In order to get the name of the cluster we have to get rid of the
        # 'CLUSTER_' string that is in the top_key name.
        my $cluster_name = substr($cluster, 8);
        $Log->comment("$action cluster '$cluster_name'");

        # Verify if the Cluster has been created
        $self->_cluster_validate(
            action         => $action,
            cluster_name   => $cluster_name,
            dispatch_table => $dispatch_table,
        );

        $Log->debug("SC::Controller::Controller_execution_helper: "
                . "THE DISPATCH TABLE"
                . Dumper($dispatch_table))
            if ($Log->may_debug());

        # Create each requested type
        # Use the keys in the correct order, starting from top to buttom.
        foreach my $type (keys(%{$dispatch_table})) {

          # Gather the parameters needed for calling CREATE or DELETE for each
          # resource.
          # Start with the list of resources:
            my $dispatch_type = $type;
            $type = lc($type);
            my $obj_list_ref = undef;
            if ($type =~ /QOS_POLICY_GROUP/i) {
                my $nacl_cluster_obj
                    = $self->{parent}->{_objects}->{cluster}->{$cluster_name};

# See if the release is SN or after because qos is only supported in SN or after.
                my $release_SN_or_after;
                $release_SN_or_after = $nacl_cluster_obj->version_manager()
                    ->is_version_ge(release => '8.2');
                unless ($release_SN_or_after) {

                    # QOS is not support in RRollingrock and before.
                    next;
                }
            }
            if ($type =~ /Volume_Efficiency/i) {
                $obj_list_ref = $self->_vol_eff_list(
                    resource_list => $resource_list,
                    category      => $cluster,
                    filter_hash   => \%filter_hash
                );
            } elsif ($type =~ /^(FCP|ISCSI|ISNS|CIFS|NFS)$/i) {
                my $service_type = "^" . $type . "_service\$";
                $obj_list_ref = $resource_list->get_resources(
                    category => $cluster,
                    type     => $service_type,
                    data     => {
                        service_enabled => 1,
                        %filter_hash
                    }
                );
            } elsif ($type =~ /INITIATOR/i) {
                $self->_convert_clients_to_initiators(
                    resource_list => $resource_list,
                    category      => $cluster
                );
                $obj_list_ref = $resource_list->get_resources(
                    category => $cluster,
                    type     => "^" . $type . "\$",
                    data     => {
                        name => qr/^iqn\.\S+|
                                               ^[a-fA-F0-9]{16}$|
                                               ^[a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}$
                                              /x,
                        %filter_hash
                    },
                );
            } elsif ($type =~ /Aggregate/i and $action =~ /deconfig/i) {
                $obj_list_ref = $resource_list->get_resources(
                    category => $cluster,
                    type     => "^" . $type . "\$",
                    data     => {%filter_hash},
                    neg_data => {_cleanup_policy => qr/^noaggr$/i}
                );
            } else {
                $obj_list_ref = $resource_list->get_resources(
                    category => $cluster,
                    type     => "^" . $type . "\$",
                    data     => {%filter_hash},
                );
            }

            if (    $action =~ /^config/i
                and scalar(@{$obj_list_ref})
                and $type =~ /Qos_Policy_Group/i) {
                my @temp_list = @{$obj_list_ref};
                @{$obj_list_ref} = $self->_fetch_and_filter_resources(
                    resource_list => \@temp_list,
                    type          => $type,
                    cluster_name  => $cluster_name
                );
            }

            # Now, if we have a list of resources, execute.
            if (scalar(@{$obj_list_ref})) {
                my %params = (
                    resource_list    => $obj_list_ref,
                    'method-timeout' => 3600,
                );

                # At this point we know that the cluster exists, now let's
                # validate that the nodes key actually holds an Array. If not
                # continue, since Task.pm will do a SystemNode->find()
                if (scalar(
                        @{  $self->{nodes_by_cluster}->{$cluster_name}
                                ->{nodes}
                            }
                    )
                    ) {
                    $params{nacl_nodes}
                        = $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
                } else {
                    $Log->warn("Controller::_controller_execution_helper "
                            . "'nodes_by_cluster' has cluster '$cluster_name' "
                            . "defined but there were now NODES associated to "
                            . "this cluster, consequently each "
                            . "Controller::Task:Components will discover a "
                            . "SystemNode to be used as a command_interface "
                            . " for NACL.");
                }

              # The actual execution.
              # This is an implementation of a dispatch table that is just a
              # mapping of keys (used as pointers) to actions (or methods).
              # By calling the dispatch table with an specific key
              # $dispatch_table->{$key}, we are invoking the call to the
              # subroutine that the key holds. Then, we are passing the params
              # $dispatch_table->{$key}->(%params) to that method call.
                $Log->comment("SC::Controller::_controller_execution_helper "
                        . "action: '$action', resources type: '$type', "
                        . "cluster: '$cluster_name'");
                my $start = [Time::HiRes::gettimeofday()];
                $dispatch_table->{$dispatch_type}->(%params);
                my $difference = Time::HiRes::tv_interval($start);
                $SCUtil::OBJ_COUNT += scalar(@$obj_list_ref);

                # Redirect the $Log calls to write to performance.log
                $Log->redirect(
                    path => (Tharn::param('LOGDIR') . '/performance.log'));
                $Log->comment(
                          scalar(@$obj_list_ref)
                        . " objects for "
                        . $dispatch_type . ":"
                        . $action
                        . " took $difference seconds with an average of "
                        . $difference / (scalar(@$obj_list_ref))
                        . " seconds per object    ");

                # Direct $Log calls to write back to the original log file
                $Log->reset_path();
            } else {
                $Log->comment("SC::Controller::_controller_execution_helper "
                        . "Resource list for cluster '$cluster_name'"
                        . "has no $type(s) to $action");
            }
        }    # foreach type
    }    # foreach cluster

    $Log->debug(
        "The Resource List After $action is:\n" . Dumper($resource_list))
        if ($Log->may_debug());

    $Log->comment("Cluster $action completed");
    $Log->exit();
    return TRUE;
} ## end sub _controller_execution_helper

#==============================================================================
# Method:     _fetch_and_filter_resources
# Objective:  Filter out objects from an array of resource that already exists
#                 on the filer.
# Details:    Steps: 1)   Fetch the correct resource type
#                    2)   Compare all the primary keys to check whether the object
#                         matches an existing object.
# Inputs:     resourece_list. An array of resource objects.
#             type            The type of resource
#             cluster_name    Name of the cluster related to the resources
# Outputs:    An array of resources filtered by existence on filer.
# Exceptions: None.
#==============================================================================
sub _fetch_and_filter_resources {
    my ($self, %args) = @_;
    $Log->enter();

    # A hash to convert our types to NACL packages.
    my %type_to_pkg_hash = (
        qos_policy_group => {
            'name' => 'policy-group',
            'pkg'  => 'QosPolicyGroup'
        },
    );

    # Locals
    my $resource_list = $args{resource_list};
    my $type          = lc($args{type});
    my $cluster_name  = $args{cluster_name};
    my $type_info     = $type_to_pkg_hash{$type};
    my @primary_keys
        = ("NACL::C::" . $type_info->{'pkg'})->get_primary_keys();

 # NACL primary keys dont include vserver for qos_policy_group in primary keys
 # but we need it to make sure that the resource is the same as what is on the
 # the filer.
    push(@primary_keys, "vserver") if ($type =~ /qos_policy_group/i);
    my $command_interface
        = $self->{parent}->{_objects}->{cluster}->{$cluster_name};
    $Log->comment("Fetching "
            . $type_info->{'pkg'}
            . " objects from the cluster $cluster_name.");
    my @fetched_objs = ("NACL::CS::" . $type_info->{'pkg'})->fetch(
        allow_empty       => 1,
        command_interface => $command_interface,
        requested_fields  => \@primary_keys
    );

    # Delete Command Interface from CS objects to save space.
    foreach my $obj (@fetched_objs) {
        delete $obj->{command_interface};
    }
    my @filtered_list = ();

    # Only loop through if filer has this type of object.
    if (scalar(@fetched_objs)) {
        my $match_count = 0;
        foreach my $resource (@{$resource_list}) {

            # Match is a count of how many keys matched.
            # If all primary keys matched then we have a match.
            my $match = 0;
            foreach my $obj (@fetched_objs) {
                $match = 0;
                foreach my $key (@primary_keys) {
                    if ($type_info->{name} =~ /^$key$/
                        and exists $resource->{name}) {
                        $match++ if ($resource->{name} eq $obj->$key());
                    } else {
                        $match++ if ($resource->{$key} eq $obj->$key());
                    }
                }
                if ($match == scalar(@primary_keys)) {
                    $match_count++;
                    $resource->{_setup_done} = 1;
                    $resource->{_reused}     = 1;
                    last;
                }
            }
            push(@filtered_list, $resource)
                unless ($match == scalar(@primary_keys));
            last if ($match_count >= scalar(@fetched_objs));
        }
    } else {    # Otherwise return the list passed in.
        return (wantarray()) ? @{$resource_list} : $resource_list;
    }
    $Log->exit();
    return (wantarray()) ? @filtered_list : \@filtered_list;

} ## end sub _fetch_and_filter_resources

#==============================================================================
# Method:     _vol_eff_list
# Objective:  Create the resource list for Volumes Efficiency.
# Details:    Steps: 1)   Get a the resources that hace efficiency turned on
#                    2)   With those volumes, get the Efficiency_options.
# Inputs:     category. The category (key) under wich Volume Efficiency (type)
#                       would be looked for on the ResourceList object,
#             resourece_list. The SC xml resource list.
# Outputs:    An array of resources filtered by Volume Efficiency. The array
#             can be empty if no resources that meet the criteria are found.
# Exceptions: None.
# TODO: Verify that the new array 'vol_concat_array' efectively hold the ref
#       of each resource.
#
#==============================================================================
sub _vol_eff_list {
    my ($self, %args) = @_;
    $Log->enter();

    # Locals
    my $top_key       = $args{category};
    my $resource_list = $args{resource_list};
    my %filter_hash   = %{$args{filter_hash}};

    my @vol_concat_array = ();
    my $volume_list_eff  = $resource_list->get_resources(
        category => $top_key,
        type     => '^volume$',
        data     => {efficiency => 1}
    );

    #Add the efficiency options on volumes that need them.
    foreach my $vol_list (@{$volume_list_eff}) {

        #Add volume efficiency options
        my $volume_eff_ref = $resource_list->get_resources(
            category => $top_key,
            type     => '^efficiency_options$',
            data     => {
                volume  => $vol_list->{name},
                vserver => $vol_list->{vserver},
                %filter_hash
            }
        );
        push(@vol_concat_array, @$volume_eff_ref);
    }

    $Log->exit();
    return (wantarray()) ? @vol_concat_array : \@vol_concat_array;

} ## end sub _vol_eff_list

#==============================================================================
# Method:     _convert_clients_to_initiators
# Objective:  Create the resource list for Initiators.
# Details:    Steps: 1)   Get the initiator resources with client names
#                    2)   Get the initiators from those clients and add them.
# Inputs:     category. The category (key) under wich Initiators (type)
#                       would be looked for on the ResourceList object,
#             resourece_list. The SC xml resource list.
# Outputs:    None
# Exceptions: None.
#
#==============================================================================
sub _convert_clients_to_initiators {
    my ($self, %args) = @_;

    $Log->enter();

    &ParamsMgr::process_args(%args);

    # Grab the resource list. This should have been passed as a
    # reference.
    my $resource_list = &ParamsMgr::get_arg(
        -arg      => "resource_list",
        -required => 1
    );
    my $top_key = &ParamsMgr::get_arg(
        -arg      => "category",
        -required => 1
    );
    if (&ParamsMgr::error()) {
        $Log->exit();
        my $errmsg = join "\n", &ParamsMgr::get_error();
        NATE::BaseException->throw("Argument parsing failed: $errmsg");
    }

    # Get the initiators that have names that dont match know WWPN and IQN
    # formats. We assume the these initiators have client names.
    # neg_data means find resources in the list that do NOT match
    # the data.
    my @initiators_with_client_names = $resource_list->get_resources(
        category => $top_key,
        type     => '^initiator$',
        neg_data => {
            name => qr/^iqn\.\S+|
                                               ^[a-fA-F0-9]{16}$|
                                               ^[a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}$
                                              /x
        },
    );
    foreach my $client (@initiators_with_client_names) {
        my @initiator_list = ();

        next unless (defined($client->{name}));
        my $initiator_data = Clone::clone($client);
        my ($client_obj) = $self->get_objects(
            types => ["client", "hypervisor"],
            names => [$client->{name}]
        );
        my $igroup_protocol = $client->{_protocol};

        # find the paramset datastructure for $client->{name}
        my @filtered_multi_clients = &ParamsMgr::search_root_multi_tags(
            -tag_name => "CLIENT",
            -filter   => "1"
        );

        my $all_hypervisors = &ParamsMgr::search(
            -search_path => [],
            -key         => "HYPERVISORS",
            -key_filter  => qr/^(ESX|HYPERV)\d*$/
        );
        foreach my $clients (@filtered_multi_clients) {
            foreach my $data (values %{$clients}, values %{$all_hypervisors})
            {
                if ($data->{NAME} eq $client->{name}) {

                    # found it, update the datastructure object
                    $client = $data;
                    last;
                }
            }
        }

# first check to see if this client has an initiators field, if it doesn then use the
# initiators from that rather than dynamically searching the client object.
        my @fc_override_init_list    = ();
        my @iscsi_override_init_list = ();
        my $perform_dynamic_lookup   = 1;
        if (exists($client->{INITIATORS})
            and ref($client->{INITIATORS}) eq "HASH") {
            foreach my $key (keys %{$client->{INITIATORS}}) {
                if ($igroup_protocol =~ /mixed|fcp/i) {
                    if ($key =~ /WWPN/) {

                        # pull out the initiator value for this wwpn entry
                        # and push it on the init list
                        push(@fc_override_init_list,
                            $client->{INITIATORS}->{$key}->{VALUE});
                    }
                } elsif ($igroup_protocol =~ /mixed|iscsi/i) {
                    if ($key =~ /ISCSI_NODENAME/) {

                        # pull out the initiator value for this wwpn entry
                        # and push it on the init list
                        push(@iscsi_override_init_list,
                            $client->{INITIATORS}->{$key}->{VALUE});
                    }
                }
            }

            # add the inits to the final initiator list
            push(@initiator_list,
                @fc_override_init_list, @iscsi_override_init_list);

# check to see if we are being requested to fall back to the dynamic lookup algorithm
# in the event that we dont have a wwpn/nodename override for one of the protocols
            $perform_dynamic_lookup = $client->{INITIATORS}->{DYNAMIC_LOOKUP};
        }

        # perform a dynamic lookup for initiators if we were requested to
        if ($perform_dynamic_lookup == 1) {

# search the client dynamically and add any additional initiators that are needed
# for fcp/iscsi
            my @initiator_objs = $client_obj->get_initiators();
            if ($client_obj->error()) {
                NATE::BaseException->throw(
                    "Failed to obtain initators for client "
                        . $client->{NAME},
                    $client_obj->errmsg()
                );
            }

            foreach my $initiator_obj (@initiator_objs) {

                # What protocol is this initiator?
                my $initiator_protocol = $initiator_obj->get_protocol();

                # If it's fcp, get the portname.
                if ($initiator_protocol eq "fcp") {

         # skip this initiator if the user specified an override fcp init list
                    next if (scalar(@fc_override_init_list));

                    if ($igroup_protocol =~ /mixed|fcp/i) {

                        # store the fcp wwpn if the igroup is fcp/mixed
                        push(@initiator_list, $initiator_obj->get_portname());
                    }
                } elsif ($initiator_protocol eq "iscsi") {

       # skip this initiator if the user specified an override iscsi init list
                    next if (scalar(@iscsi_override_init_list));

                    # Else if it's iscsi, get the nodename.
                    if ($igroup_protocol =~ /mixed|iscsi/i) {

                       # store the iscsi nodename if the igroup is iscsi/mixed
                        push(@initiator_list, $initiator_obj->get_nodename());
                    }
                } else {

                    # Else unknown protocol, return error
                    NATE::BaseException->throw("Initiator "
                            . $client->{NAME}
                            . " has an interface that "
                            . "is not using either iSCSI or FCP");
                }
            }
        }

        foreach my $init_name (@initiator_list) {
            $initiator_data->{name} = $init_name;
            $resource_list->add_resource(
                category => $top_key,
                type     => 'initiator',
                data     => Clone::clone($initiator_data)
            );
        }
    }
    $Log->exit();
} ## end sub _convert_clients_to_initiators

#==============================================================================
# Method:     _load_resource
# Objective:  Load Controller NACL resources into SC
# Inputs:     TBD
# Outputs:    TBD
# Exceptions: TBD
#==============================================================================
sub _load_resource {
    my ($self) = @_;
    $Log->enter();

    # Go through all resources provided by caller (XML file) and create
    # necessary objects for each to be stored on SanMgr for later use
    my @nacl_nodes;
    if (exists $self->{parent}->{nodes}) {
        @nacl_nodes = @{delete $self->{parent}->{nodes}};
    } else {

        # Get all the Nodes that are defined.
        @nacl_nodes = NACL::C::Node->find();
    }

    my @nacl_cluster_objs_array;
    if (exists $self->{parent}->{clusters}) {
        @nacl_cluster_objs_array = @{delete $self->{parent}->{clusters}};
    } else {

        # Get all the unique cluster objects from the Nodes that are defined.
        @nacl_cluster_objs_array = NACL::STask::Cluster->find_unique_clusters(
            nodes => \@nacl_nodes);
    }

    # Need to store the cluster objects in nacl_node because SanMgr libraries
    # may search node instead of cluster for the cluster because of how they
    # currently work. When we stop relying on SanMgr we can remove this.
    foreach my $cluster (@nacl_cluster_objs_array) {
        my $cluster_name = $cluster->name();
        $self->{parent}->{_objects}->{nacl_cluster}->{$cluster_name}
            = $cluster;
        $self->{parent}->{_objects}->{nacl_node}->{$cluster_name} = $cluster;

       # Adding cluster and node because SanMgr get_obj relies on it existing.
        $self->{parent}->{_objects}->{cluster}->{$cluster_name} = $cluster;
        $self->{parent}->{_objects}->{node}->{$cluster_name}    = $cluster;

        # Store the name of the nodes under the corresponding cluster. Using
        # these names, we will populate the cluster->nodes hash with the
        # versioned objects.

        #TODO: When Process.pm Supports NODE versioning, use the following
        #      code to assign the retrieve the name of the nodes that each
        #      cluter hash needs to hold.

=break
        my @nodes = $cluster->nodes();
        $self->{nodes_by_cluster}->{$cluster_name}->{names} = \@nodes;
        $self->{nodes_by_cluster}->{$cluster_name}->{nodes} = [];
=cut

        my @node_objs = $cluster->node_objs();
        $self->{nodes_by_cluster}->{$cluster_name}->{nodes} = \@node_objs;
    }

    # Now store all the nodes that we found earlier in nacl_node. These node
    # objects should already be version by this point.
    foreach my $nacl_node_obj (@nacl_nodes) {
        $self->{parent}->{_objects}->{nacl_node}->{$nacl_node_obj->name()}
            = $nacl_node_obj;

        # Adding node because SanMgr get_obj relies on it existing.
        $self->{parent}->{_objects}->{node}->{$nacl_node_obj->name()}
            = $nacl_node_obj;

        #TODO: When Process.pm Supports NODE versioning, use the following
        #      code to assign the versioned nodes to each cluster hash.

=break
        # Add the node objects to its corresponding cluster
        foreach my $cluster_name ( keys(%{$self->{nodes_by_cluster}}) ){
            my $names_array = $self->{nodes_by_cluster}->{$cluster_name}->{names};
            my $nodes_array = $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
            foreach my $found_name (@{$names_array}){
                if ($node_name eq $found_name){
                    push(@{$nodes_array}, $nacl_node_obj);
                    last;
                }
            }
        }# end foreach cluster
=cut

    }

    # Print a friendly output of what nodes have been assigned to what cluster
    foreach my $cls_nm (keys(%{$self->{nodes_by_cluster}})) {
        foreach my $nd (@{$self->{nodes_by_cluster}->{$cls_nm}->{nodes}}) {
            $Log->comment("SC::Controller::_load_resource: "
                    . "FOUND NODES_BY_CLUSTER->[$cls_nm]->[nodes]: " . "NODE["
                    . $nd->{node}
                    . "]");
        }
    }

    $Log->exit();
    return $NACL::AppBuilder::EnGenX::TRUE;
} ## end sub _load_resource

#==============================================================================
# Method:     _cluster_validate
# Objective:  Validate that the cluster has been PREVIOUSLY created.
# Details:    Steps: 1) Validate that the action is Config (no need to see if
#                        the cluster has been created for cleanup)
#                    2) Validate that the provided cluster name is indeed a
#                       configured cluster in the test bed.
#                    3) Add the Cluster key to the dispatch table.
# Inputs:     action. The subroutine the user called.
#             cluster_name. The name of the cluster.
#             dispatch_table. The hash table that holds the 'key' that is the
#                             type of the resource and the subrutine to be
#                             executed.
# Outputs:    Boolean, True if no problem was encountered.
# Exceptions: Arguments exception if the passed arguments are wrong.
#
# TODO: We could configure the cluster here.
#
#==============================================================================
sub _cluster_validate {
    my ($self, %args) = @_;
    $Log->enter();

    #Locals
    my $action       = $args{action};
    my $cluster_name = $args{cluster_name};

    # ONLY VERIFY THE CLUSTER IF THE CALLING ACTION IS CONFIG
    if ($action eq "Config") {

        # AT SC::New::load_resources A CLUSTER IDENTITIY SHOW WAS PERFORMED
        # AND THE NAME OF ALL THE AVAILABLE CLUSTER WERE SAVED, IF THE CLUSTER
        # HAS ALREADY BEEN CREATED, THE NAME OF THE CLUSTER SHOULD BE FOUND.
        if (!exists($self->{nodes_by_cluster}->{$cluster_name})) {

            # IF NOT FOUND THE USER COULD HAVE USED THE NAME OF A NODE (FILER)
            # AS A CLUSTER NAME. SanMgr COMPATIBILITY.
            if (defined(
                    $self->{parent}->{_objects}->{cluster}->{$cluster_name}
                )
                ) {

                # THROW A WARNING
                my $wrnng
                    = "User specified in the XML file at the cluster tag that "
                    . "the name of the cluster is '$cluster_name' when in "
                    . "reality, that is the name of a NODE or Filer. Please "
                    . "modify the 'THARNHOST' or '.NTEST.TB' and specify the "
                    . "name of the cluster like this: CLUSTER_NAME=name_of_the_cluster "
                    . "and use that variable in the cluster name attribute.";

                # TODO: THROW AN ERROR WHEN NO LONGER SUPPORTED
                $Log->warn("SC::Controller::_cluster_validate " . $wrnng);

              # NOW, FIND THE NODES ASSOCIATED TO THIS "CLUSTER NAME". NEED TO
              # LOOK INTO EACH NODE INSIDE OF EACH CLUSTER
                foreach my $cls_nm (keys(%{$self->{nodes_by_cluster}})) {
                    foreach my $nd (
                        @{$self->{nodes_by_cluster}->{$cls_nm}->{nodes}}) {
                        if ($nd->{node} eq $cluster_name) {
                            $self->{nodes_by_cluster}->{$cluster_name}
                                ->{nodes}
                                = $self->{nodes_by_cluster}->{$cls_nm}
                                ->{nodes};
                            last;
                        }
                    }
                }    #foreach cls_nm
            }

          # IF THE CLUSTER NAME IS NOT A NODE NAME, THEN THE CLUSTER IS EITHER
          # NOT CONFIGURED OR NOT CORRECT EITHER WAY, WE NEED TO THROW AN
          # EXCEPTION
            else {

                # TODO: Cluster config call would go here.
                $Log->exit();
                NATE::BaseException->throw(
                          "SC couldn't find a clusters with name "
                        . "'$cluster_name'. At this point SC does "
                        . "not create the cluster for you.  Until "
                        . "that functionality is added all clusters "
                        . "will have to be created prior to setup.");
            }
        }

     # NOW THAT WE KNOW THE CLUSTER IS CONFIGURED, NOW VERIFY THAT THE CURRENT
     # CLUSTER ACTUALLY NEEDS ANY MODIFICATION (VERFIY FLAG) . IF SO, ADD THE
     # CLUSTER KEY TO THE DIPATCH TABLE
        if (defined($self->{parent}->{modify_clusters}->{$cluster_name})) {

            # Trying to preserve the hierarchy of the components, put at the
            # beginning of the Dispatch Table the Cluster type.
            %{$args{dispatch_table}} = (
                'Cluster' => sub {
                    NACL::AppBuilder::SC::Controller::Cluster->create(@_);
                },
                %{$args{dispatch_table}}
            );
        }
    }

    # ELSE,
    # TODO: If the action is CleanUp or DeConfig it should not matter if the
    #       cluster is configured or not. But we do need to verify that the
    #       found nodes (if any) are from the specified cluster name.

    $Log->exit();
    return $NACL::AppBuilder::EnGenX::TRUE;
} ## end sub _cluster_validate

1;
