
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Lif Controller layer module
## @author dl-nacl-dev
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::SC::Controller::Lif

=head1 DESCRIPTION

NACL::AppBuilder::SC::Controller::Lif is a derived class of
NACL::AppBuilder::SC::Controller::Task.  It is used to Create and
Delete ONTAP network interfaces. All methods are static.

=cut

package NACL::AppBuilder::SC::Controller::Lif;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use NATE::STAF::Peer qw($Parent);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::C::NetworkInterface;
use NACL::STask::NetworkInterface;
use NACL::CS::NetworkInterface;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::C::Job;
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#Inherit base class
use base qw( NACL::AppBuilder::SC::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args
    $Mem_Resource
);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name   => 'lif',
    _reuse => 'nacltask_if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    ips     => '',
    ips_key => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    lif               => '',
    vserver           => '',
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple Lifs serially
or in parallel and return whether the lif(s) was created successfully.
This method passes all user arguments through to EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::SC::Controller::Lif->create(
            command_interface => $Node,
            lif               => $Lif_name,
            vserver           => $Vserver_name,
            'data-protocol'   => iscsi
            'home-port'       => e0c
            netmask           =>"255.255.192.0"
            address           => "10.63.9.163"
            role              => "data"
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::SC::Controller::Lif->create(
            command_interface => $Node,
            lif               => $Lif_name,
            vserver           => $Vserver_name,
            'data-protocol'   => iscsi
            'home-port'       => e0c
            netmask           =>"255.255.192.0"
            address           => "10.63.9.163"
            role              => "data"
            count             => 5
            );

  NACL::AppBuilder::SC::Controller::Lif->create(
            command_interfac  => $Node,
            resource_list      => @Lifs,
            'method-timeout'  => 3600,
            );

=over

=item Options

Either the resource_list is required or the NACL Lif Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the lifs in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Lif Creation Arguments should
NOT be passed in.

=item C<< NACL Lif Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::NetworkInterface->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< count => $lif_count >>

(Optional)
This arguments is used to create multiple lifs from the same NACL NetworkInterface
Creation Arguments.  To create the names for the lifs, the lif name
passed in is appended with the index that it was create at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Lif(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

TODO: The Task::_create_mulptiple sub will fail to create multiple LIFs because
      the method needs to also provide a unique IP ADDRESS. This unique address
      (or Array of usable address) would be passed by the user. None the less,
      the method needs to accept the array of IPs and use it when making the
      NACL call.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lif";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple lifs serially
or in parallel and return whether the lif(s) was deleted successfully.
This method passes all user arguments through to EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::SC::Controller::Lif->delete(
            command_interface => $Node,
            lif               => $Lif_name,
            vserver           => $Vserver_name,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::SC::Controller::Lif->delete(
            command_interface => $Node,
            lif               => $Lif_name,
            vserver           => $Vserver_name,
            count             => 5
            );

  NACL::AppBuilder::SC::Controller::Lif->delete(
            command_interface => $Node,
            resource_list      => @Lifs,
            'method-timeout'  => 3600,
            );

=over

=item Options

Either the resource_list is required or the NACL Lif Deletion Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the lifs in the resource list will be deleted
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Lif Deletion Arguments should
NOT be passed in.

=item C<< NACL Lif Deletion Arguments >>

(Required)
These are arguments that can be passed into NACL::C::NetworkInterface->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< count => $lif_count >>

(Optional)
This arguments is used to delete multiple lifs from the same NACL NetworkInterface
Deletion Arguments.  To delete the names for the lifs, the lif name
passed in is appended with the index that it was deleted at starting at 1.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Lif(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lif";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a lif.
# Details:    Makes the NACL::C::Lif->create() call. Also works
#             around some common non-fatal errors that can be return from
#             lif creation and then has a retry mechanism to retry lif
#             creation.
# Inputs:     NACL::C::lif->create arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the lif was a duplicate if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    my $duplicated = FALSE;
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'reuse';
    if (exists($args{'auto-configuration'})) {
        delete($args{address});
        delete($args{netmask});
        delete($args{gateway});
        delete($args{'netmask-length'});
    }

    my $gateway = delete $args{gateway};
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }
    $pkg->_set_ci_defaults(command_interface => $args{command_interface});

    my $was_created;
LABEL:
    {
        try {
            NACL::STask::NetworkInterface->create(
                _was_created       => \$was_created,
                nacltask_if_exists => $if_exists,
                %args
            );
        }
        catch NACL::C::Exceptions::NetworkInterface::IpAlreadyBound with {

            # Delete the Address that didn't work
            $args{address} = '';
            %args = $pkg->_get_lif_params(args => \%args);
            redo LABEL;
        };
    }
    $duplicated = TRUE if ($was_created);
    if (defined($gateway) and length($gateway)) {

        # Set the default destination
        my $destination = '0.0.0.0/0';
        if ($gateway =~ /:/) {
            $destination = '::/0';
        }

        my $lif_cs_obj = NACL::CS::NetworkInterface->fetch(
            command_interface => $args{command_interface},
            filter            => {
                vserver => $args{vserver},
                lif     => $args{lif}
            },
            requested_fields => ['routing-group']
        );
        my $was_created;
        NACL::STask::NetworkRoutingGroupsRoute->create(
            command_interface => $args{command_interface},
            'vserver'         => $args{vserver},
            'routing-group'   => $lif_cs_obj->routing_group(),
            'gateway'         => $gateway,
            'destination'     => $destination,
            '_was_created'    => $was_created
        );
        if ($was_created) {
            $Log->comment("Using an already created Route: "
                    . "\nvserver       "
                    . $args{vserver}
                    . "\nrouting-group "
                    . $lif_cs_obj->routing_group()
                    . "\ngateway       "
                    . $gateway
                    . "\ndestination   "
                    . $destination);
        }
    }
    $Log->exit() if $may_exit;

    # Returning either a list (hash) or scalar TRUE
    return (wantarray())
        ? ('success' => TRUE, 'duplicated' => $duplicated)
        : TRUE;

} ## end sub _nacl_create_call

#==========#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a lif.
# Details:    Makes the NACL::C::lif->delete() calls. Also works around
#             some common non-fatal errors that can be return from lif
#             deletion and then has a retry mechanism to retry lif deletion.
# Inputs:     NACL::C::lif->delete arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    # Locals
    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }
    $pkg->_set_ci_defaults(command_interface => $args{command_interface});

    NACL::STask::NetworkInterface->purge(%args, nacltask_if_purged => 'pass');

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#===============================================================================
# Method:     _get_lif_params
# Objective:  Add or Delete the needed arguments that NACL create call needs.
# Details:    Depending of the Data-Protocol that the lif has, will either do:
#             1) ISCSI - Will populate the 'address' parameter, by either
#                        retrieve the ip from the resource or just passthrough
#                        an existing one
#             2) FCP - Will delete the NETMASK and ADDRESS parameters that are
#                      not needed by NACL.
# Inputs:     args:  The resrouce ARGUMENTS that will be send to NACL call
#             respirces : The resource itself.
# Outputs:    Returns an update version of the arguments.
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _get_lif_params {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    # Locals
    my %args         = %{$opts{args}};
    my $address_flag = TRUE;

    # User is not required to provide attribute 'data-protocol'. So first
    # verify if it even exists. If it does, verify it is an array reference as
    # requested by NACL.
    if (exists($args{'data-protocol'})) {
        $Log->trace("Lif::_get_lif_params: DATA PROTOCOL defined.");
        if (!(ref($args{'data-protocol'}) =~ /ARRAY/i)) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                      "Lif::_get_lif_params: DATA-PROTOCOL must be an"
                    . " ARRAY reference.");
        }

      # Since the DATA-PROTOCOLs attribute is an array, we need to find out if
      # FCP is one of the requested protocols. If it was requested we need to
      # delete the unecessary arguments that all other protocols need.
        foreach my $protocol (@{$args{'data-protocol'}}) {
            if ($protocol =~ /fcp/i) {
                $address_flag = FALSE;
            }
        }
    } else {
        $Log->trace(
                  "Lif::_get_lif_params: Attribute DATA PROTOCOL NOT DEFINED"
                . " FOR LIF: "
                . $opts{resource}->{NAME});
    }

    #if netmask-length exists, netmask isn't needed.
    if (exists($args{'netmask-length'})) {
        delete($args{netmask});
    }

    # If the protocol is not FCP, get an IP ADDRESS if needed.
    if ($address_flag) {

        #Get the resource to work with.
        my $resource;
        if (exists($opts{resource})) {

            # Set the global $Resource variable
            $resource     = $opts{resource};
            $Mem_Resource = $opts{resource};
        } else {
            if (defined($Mem_Resource)) {
                $resource = $Mem_Resource;
            } else {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Lif::_get_lif_params: Missing Resource reference to work with"
                );
            }
        }

        # Verify that an IP address is needed
        if (defined($resource->{ips}) && $args{address} eq '') {

            # If key IPS is present in the resource, means that user passed
            # an IP address range from where to chose the address. So, the
            # Resource needs to pass the array to the parent for him to
            # manage the addresses and avoid collisions.
            #
            # Request and IP Address
            $Parent->message_put(
                type    => "Response",
                message => {
                    status    => "request_ip",
                    resources => $resource,
                }
            );

            # GET THE MESSAGE
            my $ip_mssg = $Parent->message_get(type => "ip_address");

           #IF THE MESSAGE IS AN ARRAY REF, THEN A RESOURCE WITH AN IP ADDRESS
           # WAS RETURNED
            if (ref($ip_mssg) =~ /ARRAY/) {

                # Save the value in the resource that will be
                # returned to the ResourceList
                $resource = $ip_mssg->[0];
                $args{address} = $resource->{ADDRESS};
            }

            # IF THE MESSAGE IS A SCALAR IS AN ERROR
            elsif ($ip_mssg =~ /ERROR: Could not find an IP address/) {
                NACL::AppBuilder::EnGenX::Exceptions::SubProcessFailure->throw($ip_mssg,
                    resource => $resource);
            }
        }    #if ips
    }    #if address
    else {

        # If this is a FCP lif, delete unecessary keys.
        delete $args{address};
        delete $args{netmask};
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? %args : \%args;
}    # sub _get_lif_params

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
} ## end sub _get_dtd_nacl_translate

#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
} ## end sub _get_dtd_nacl_ignore

#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
} ## end sub _get_dtd_nacl_delete

1;
