# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

## @summary Workflow Framework Components data population plugin base class
## @author rjray@netapp.com
## @status shared
## @pod here

package NACL::AppBuilder::WFC::IO::Data;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;

# Since the population plugin classes aren't directly used by script developers
# I'm not worrying with read-only accessors.
use Class::MethodMaker [
    # This will be the local handle (NACL::MTask::Data obj, job obj, etc.)
    scalar => 'handle',
];

=head1 NAME

NACL::AppBuilder::WFC::IO::Data - Base class for data plugins

=head1 SYNOPSIS

This module is not used directly. It is only used as the base class for data
plugins.

=head1 DESCRIPTION

The B<NACL::AppBuilder::WFC::IO::Data> class is the base class for data plugins
that are used with the B<NACL::AppBuilder::WFC::IO> B<data_populate> and
B<data_modify> methods. It provides a few common methods and stubs for the
remaining methods.

Users should not use this module directly. It is only used as the basis for
plugins.

=head1 ATTRIBUTES

This base class provides the following attribute:

=over

=item B<handle>

This attribute is used to store the object handle for whatever method is
being encapsulated by the given plugin. For example, the
B<NACL::AppBuilder::WFC::IO::Data::NaclData> plugin uses this to store the
B<NACL::MTask::Data> handle that it uses.

=back

B<NACL::AppBuilder::WFC::IO::Data> uses B<Class::MethodMaker>, so the
attribute C<handle> may be retrieved or set using B<handle> as a method name.
It can be tested with B<handle_isset>, and may be cleared completely with
B<handle_reset>.

=head1 METHODS

This class provides the following methods. Most of these are stubs that throw
exceptions if they are not overridden by plugins that use this base class. For
the sake of documentation, all methods are documented here fully.

=head2 new

Create a new object of the given class. This cannot be called on the base class,
but should handle creation for all the plugin classes without the need for
being overridden. Note that the user never calls this directly, it is called by
the B<NACL::AppBuilder::WFC::IO> class. Thus, even if overridden things like
the calling arguments will not ever change.

=over

=item Options

No options are recognized by B<new>.

=item Exceptions

The only exception thrown by B<new> is if an attempt is made to use it to
instantiate an object of the B<NACL::AppBuilder::WFC::IO::Data> class, in which
case it will throw a B<NATE::BaseException>.

=back

=cut

###############################################################################
#
#   Sub Name:       new
#
#   Description:    Create an object of the given class. Will not create
#                   object in this class, only in subclasses.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $class    in      scalar    Name of class to bless into
#
#   Returns:        Success:    new object
#                   Failure:    throws exception
#
###############################################################################
sub new {
    $Log->enter;
    my ($class) = @_;

    # Make sure some mistake doesn't lead to someone trying to instantiate this
    # abstract class directly:
    if ($class eq __PACKAGE__) {
        NATE::BaseException->throw(
            __PACKAGE__ . '::new: Cannot be instantiated directly. Must be ' .
            'called from within a base-class.'
        );
    }

    # Create the object
    my $self = bless {}, $class;

    $Log->exit;
    return $self;
}

=head2 data_method

Identifies the "method" that an object instance represents. This is the string
that was used when B<NACL::AppBuilder::WFC::IO::data_populate> or
B<NACL::AppBuilder::WFC::IO::data_modify> was called, to select the given
plugin.

The return value is a string that can be used to select this particular plugin
in future calls to B<data_populate> or B<data_modify> on a
B<NACL::AppBuilder::WFC::IO> handle.

This method may be called as either an instance method or a static method. It
does not need to be overridden by plugins.

=over

=item Options

No options are recognized by B<data_method>.

=item Exceptions

No exceptions are thrown by B<data_method>.

=back

=cut

###############################################################################
#
#   Sub Name:       data_method
#
#   Description:    Like an ID, this identifies the plugin being used
#                   currently for population/modification operations.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      scalar    Class name or an object ref
#
#   Returns:        string
#
###############################################################################
sub data_method {
    $Log->enter;
    my ($self) = @_;
    my $class = ref($self) || $self;

    # If $class (the classname of the object, if called as an instance method)
    # starts with __PACKAGE__ (the classname of this base-class), then the
    # plug-in is one that lives here under NACL::AppBuilder::WFC::IO::Data.
    # In that case, just strip NACL::AppBuilder::WFC::IO::Data:: from the
    # string and return the remainder.
    #
    # If $class doesn't match this, then the plug-in lives somewhere else, so
    # return the full $class value.

    if (__PACKAGE__ eq substr $class, 0, length __PACKAGE__) {
        $class = substr $class, length(__PACKAGE__) + 2;
    }

    $Log->exit;
    return $class;
}

=head2 data_populate

Perform data population. This is the method that the plugin class must implement
in order to do the actual data population. If this method is not overridden,
then the one defined here will throw an exception.

This is the method that the B<NACL::AppBuilder::WFC::IO> handle will ultimately
call in order to do data population.

=over

=item Options

The following options will be passed from the calling IO object:

=over

=item C<< client =E<gt> $client >>

An object of the B<NACL::C::Client> class, that will be used to actually
perform the data population.

=item C<< path =E<gt> $string >>

The path relative to the mount point at which data should be populated.

=item C<< size =E<gt> $string >>

The size of data to be created.

=item C<< async =E<gt> $boolean >>

Whether or not to run the data population asynchronously.

=item C<< arguments =E<gt> $hashref >>

A hash reference of arguments to be passed directly to the underlying population
mechanism (such as B<NACL::MTask::Data::populate>, for example). Each plugin
will be responsible for both providing reasonable default configuration for the
underlying population mechanism, and will also be responsible for correctly
merging in any user-supplied arguments here in such a way as to override any
defaults by the same names.

=back

=item Exceptions

Exceptions will be defined and documented on a per-plugin basis.

=back

=cut

###############################################################################
#
#   Sub Name:       data_populate
#
#   Description:    Stub that throws an exception, in case a sub-class forgets
#                   to implement this method.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object
#
#   Returns:        throws exception
#
###############################################################################
sub data_populate {
    $Log->enter;
    my ($self) = @_;
    my $class = ref $self;

    $Log->exit;
    NATE::BaseException->throw(
        "Class $class did not override the data_populate() method"
    );

    # Not reached, but needed for Perl::Critic:
    return;
}

=head2 data_modify

Perform data modification. This is the method that the plugin class must
implement in order to do the actual data modification. If this method is not
overridden, then the one defined here will throw an exception.

This is the method that the B<NACL::AppBuilder::WFC::IO> handle will ultimately
call in order to do data modification.

=over

=item Options

The following options will be passed from the calling IO object:

=over

=item C<< client =E<gt> $client >>

An object of the B<NACL::C::Client> class, that will be used to actually
perform the data modification.

=item C<< path =E<gt> $string >>

The path relative to the mount point at which data should be modified.

=item C<< size =E<gt> $string >>

The size of data to be modified.

=item C<< async =E<gt> $boolean >>

Whether or not to run the data modification asynchronously.

=item C<< arguments =E<gt> $hashref >>

A hash reference of arguments to be passed directly to the underlying
modification mechanism (such as B<NACL::MTask::Data::modify>, for example).
Each plugin will be responsible for both providing reasonable default
configuration for the underlying modification mechanism, and will also be
responsible for correctly merging in any user-supplied arguments here in such a
way as to override any defaults by the same names.

=back

=item Exceptions

Exceptions will be defined and documented on a per-plugin basis.

=back

=cut

###############################################################################
#
#   Sub Name:       data_modify
#
#   Description:    Stub that throws an exception, in case a sub-class forgets
#                   to implement this method.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object
#
#   Returns:        throws exception
#
###############################################################################
sub data_modify {
    $Log->enter;
    my ($self) = @_;
    my $class = ref $self;

    $Log->exit;
    NATE::BaseException->throw(
        "Class $class did not override the data_modify() method"
    );

    # Not reached, but needed for Perl::Critic:
    return;
}

=head2 data_is_running

Determine if there is currently a population or modification operation running.

This implementation should be sufficient for most plugins. It checks for there
being a value in the C<handle> attribute and returns true or false based on
that.

The return value is a boolean, indicating whether an operation is currently
running.

=over

=item Options

No options are recognized by B<data_is_running>.

=item Exceptions

No exceptions are thrown by B<data_is_running> in this implementation, though
a plugin that implements its own version may throw exceptions.

=back

=cut

###############################################################################
#
#   Sub Name:       data_is_running
#
#   Description:    Boolean predicate to tell if the population object has a
#                   currently-running job. Does not (yet) detect whether the
#                   job has completed on the client. Just returns "true" if
#                   the last populate/modify was done async and neither of
#                   data_wait_for_completion or data_kill have been called yet.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object
#
#   Returns:        boolean
#
###############################################################################
sub data_is_running {
    my ($self) = @_;

    return $self->handle_isset;
}

=head2 data_wait_for_completion

Wait for the current population or modification operation to complete. Does
nothing if there is no current running operation.

The return value is void.

=over

=item Options

The following option is recognized, and should be passed from the calling IO
object:

=over

=item C<< timeout =E<gt> $number >>

Specify a timeout value for the wait, in seconds. If the process takes longer
than this to finish, an exception will be thrown.

=back

=item Exceptions

Exceptions will be defined and documented on a per-plugin basis.

=back

=cut

###############################################################################
#
#   Sub Name:       data_wait_for_completion
#
#   Description:    Stub that throws an exception, in case a sub-class forgets
#                   to implement this method.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object
#
#   Returns:        throws exception
#
###############################################################################
sub data_wait_for_completion {
    $Log->enter;
    my ($self) = @_;
    my $class = ref $self;

    $Log->exit;
    NATE::BaseException->throw(
        "Class $class did not override the data_wait_for_completion() method"
    );

    # Not reached, but needed for Perl::Critic:
    return;
}

=head2 data_kill

Kill a running population or modification operation. Does nothing if there is
no current running operation.

The return value is void.

=over

=item Options

No options are recognized by B<data_kill>.

=item Exceptions

Exceptions will be defined and documented on a per-plugin basis.

=back

=cut

###############################################################################
#
#   Sub Name:       data_kill
#
#   Description:    Stub that throws an exception, in case a sub-class forgets
#                   to implement this method.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     in      ref       Object
#
#   Returns:        throws exception
#
###############################################################################
sub data_kill {
    $Log->enter;
    my ($self) = @_;
    my $class = ref $self;

    $Log->exit;
    NATE::BaseException->throw(
        "Class $class did not override the data_kill() method"
    );

    # Not reached, but needed for Perl::Critic:
    return;
}

=head1 SEE ALSO

B<NACL::AppBuilder::WFC::IO::Data::NaclData>, B<NACL::AppBuilder::WFC::IO>

=head1 AUTHOR

Randy J. Ray E<lt>rjray@netapp.comE<gt>

=cut

1;
