#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Workload::LDX::CIFS_SMB2_1K_IOPS;

use strict;
use warnings;
use Carp qw(confess);
use NATE::Log qw( log_global );

use parent qw(NATE::Workload::LDX);

use swifttest;

use Params::Validate qw(SCALAR ARRAYREF);

use constant WORKLOAD_TAGS => {
    ldx => 1,
    smbv2 => 1,
};

use constant WORKLOAD_DESCRIPTION => 'This workload is a CIFS workload running SMB2 to run ~1K IOPs';

##############################################################################
#  Description: Build a SMB2 project containing reads/writes/locks.
#
##############################################################################
sub build_ldx_project {
    my ($pkg, %opts) = @_;

    my $swt_appliance_ip = $opts{ldx_ip};;
    my $client_port_number = $opts{ldx_ports}->[0];

    # Create project with specified name
    my $project = new swifttest::project("CIFSv2 Trial Test");
    log_global()->log("Created project $project");
    # Create user parameters with users/passwords - load them from specified CSV file
    my $user_parameters = swifttest::user_parameters::load("/u/carteram/swifttest/download/v5.2/test1.csv");
    $project->add_user_parameters(0, $user_parameters);

    # Create data content
    my $data_content = new swifttest::data_content();
    my $root_node = $data_content->get_root_node();

    # Fill data content with 24 nodes that use one of two data content providers
    for (my $file = 1; $file <= 24; $file++) {
        my $file_name = sprintf("FILE%02u.TEST", $file);
        my $content_name = sprintf("::DataContent(%d)", 1 - ($file % 2));
        $root_node->add_node(new swifttest::data_content_fs_node($file_name, $content_name, 1024000, 65534, 65534, 644));
    }

    $data_content->add_provider('random',     '::DataContent(0)');
    $data_content->add_provider('sequential', '::DataContent(1)');

    # Create trace parameters (4MB max)
    my $trace_parameters = new swifttest::trace_parameters(4000000);

    # Create client port
    my $port = new swifttest::port();
    $project->add_port($port);
    $port->set_kind(swifttest::port::CLIENT);
    $port->set_portnum($client_port_number);
    $port->set_appliance($swt_appliance_ip);
    $port->set_datacontent($data_content);
    $port->set_traceparameters($trace_parameters);

    # Create client network
    my $network = new swifttest::net('1.225.0.10', '255.255.224.0', 254, 1, '1.225.0.1');
    $network->set_ipv6('fd20:8b1e:b255:8770::10', '64', 16000, 1, 'fd20:8b1e:b255:8770::1');
    $port->add_net($network);
    $network->set_gw_enabled(1);
    $network->set_ipv6_gw_enabled(1);

    # Create client scenario and loads
    my $scenario = new swifttest::scenario();
    $network->add_scenario($scenario);

    my $load;
    $load = new swifttest::load('ramp_up', 30, 0, 0,100);
    $scenario->add_load($load);
    $load = new swifttest::load('new_scenarios', 45, 10, 0,100);
    $scenario->add_load($load);
    $load = new swifttest::load('scenarios', 95, 100, 0,100);
    $scenario->add_load($load);
    $load = new swifttest::load('ramp_down', 35, 0, 0,100);
    $scenario->add_load($load);

    # Fill client scenario with actions; we specify just changed parameters
    my $action;
    $action = new swifttest::action('SMB2', 'Open SMB2 TCP Connection');
    $scenario->add_action($action);

    # Using set_parameter() method to configure the action
    $action->set_parameter('Input Format', '0');
    $action->set_parameter('Destination IP Address', 'fd20:8b1e:b255:8383::2010');
    $action->set_parameter('Destination IP Port', '445');
    $action->set_parameter('Output Handle', '1: SMB2ConnectionHandle');

    $action = new swifttest::action('SMB2', 'Negotiate');
    $scenario->add_action($action);
    $action->set_parameter('Packet Signing', 'Enabled');
    # Using default SMB 2.002 dialect

    my %SessionSetup = ('GSSAPI' => 'Disable', 'Domain Name' => '= @UP(0,C)', 'User Name' => '= @UP(0,D)',
        'Password' => '= @UP(0,E)', 'NTLM Flags' => '0xA0088237',
        'Output Handle' => '3: SMB2SessionHandle',);
    $action = new swifttest::action('SMB2', 'Session Setup', \%SessionSetup);
    $scenario->add_action($action);

    $action = new swifttest::action('SMB2', 'Tree Connect');
    $scenario->add_action($action);
    $action->set_parameter('Share Name', '= @STRING(\\\\) + @UP(0,A) + @STRING(\\) + @UP(0,B)');
    $action->set_parameter('Output Handle', '4: SMB2ShareHandle');

    #Begin Loop
    $action = new swifttest::action('SWT', 'Begin Loop');
    $scenario->add_action($action);
    $action->set_parameter('Repeat Count', '2048');

    #Create File and name it based on loop count
    $action = new swifttest::action('SMB2', 'Create File');
    $scenario->add_action($action);
    $action->set_parameter('Input Format', '0');
    $action->set_parameter('Path', '= @UP(0,F)');
    $action->set_parameter('Impersonation Level', '2');
    $action->set_parameter('ACL Mode', '0x001F01FF');
    $action->set_parameter('File Attributes', '0x00000080');
    $action->set_parameter('Share Mode', '0x00000003');
    $action->set_parameter('Disposition', '3');
    $action->set_parameter('Create Options', '0x00000000');
    $action->set_parameter('Output Handle', '5: SMB2FileHandle');

    #Delay Execution
    $action = new swifttest::action('SWT', 'Delay Execution');
    $scenario->add_action($action);
    $action->set_parameter('Delay Interval', '5000');

    #Do a File Write
    $action = new swifttest::action('SMB2', 'File Write');
    $scenario->add_action($action);
    $action->set_parameter('File Offset', '0');
    $action->set_parameter('Bytes Per Block', '65500');
    $action->set_parameter('Remaining Bytes to Write', '0');
    $action->set_parameter('Bytes Total', '655000');
    $action->set_parameter('Block Sequence', '0');
    $action->set_parameter('Data Source', '');
    $action->set_parameter('Data Source Offset', '0');

    #File Read
    $action = new swifttest::action('SMB2', 'File Read');
    $scenario->add_action($action);
    $action->set_parameter('File Offset', '0');
    $action->set_parameter('Bytes Per Block', '65500');
    $action->set_parameter('Bytes Total', '655000');
    $action->set_parameter('Block Sequence', '0');
    $action->set_parameter('Data Source', '');
    $action->set_parameter('Data Source Offset', '0');

    #File Lock
    $action = new swifttest::action('SMB2', 'File Lock');
    $scenario->add_action($action);
    $action->set_parameter('Lock Flags', 'Lock Exclusive (no other reads/writes/locks');

    #Delay Execution
    $action = new swifttest::action('SWT', 'Delay Execution');
    $scenario->add_action($action);
    $action->set_parameter('Delay Interval', '5000');

    #Do a File Write
    $action = new swifttest::action('SMB2', 'File Write');
    $scenario->add_action($action);
    $action->set_parameter('File Offset', '0');
    $action->set_parameter('Bytes Per Block', '65500');
    $action->set_parameter('Remaining Bytes to Write', '0');
    $action->set_parameter('Bytes Total', '655000');
    $action->set_parameter('Block Sequence', '0');
    $action->set_parameter('Data Source', '');
    $action->set_parameter('Data Source Offset', '0');

    #File Unlock
    $action = new swifttest::action('SMB2', 'File Lock');
    $scenario->add_action($action);
    $action->set_parameter('Lock Flags', 'Unlock');

    #File Read
    $action = new swifttest::action('SMB2', 'File Read');
    $scenario->add_action($action);
    $action->set_parameter('File Offset', '0');
    $action->set_parameter('Bytes Per Block', '65500');
    $action->set_parameter('Bytes Total', '655000');
    $action->set_parameter('Block Sequence', '0');
    $action->set_parameter('Data Source', '');
    $action->set_parameter('Data Source Offset', '0');

    #End Loop
    $action = new swifttest::action('SWT', 'End Loop');
    $scenario->add_action($action);

    #File Close
    $action = new swifttest::action('SMB2', 'File Close');
    $scenario->add_action($action);
    $action->set_parameter('Close Flags', '0x0000');

    #Tree Disconnect
    $action = new swifttest::action('SMB2', 'Tree Disconnect');
    $scenario->add_action($action);

    #Session Logoff
    $action = new swifttest::action('SMB2', 'Logoff');
    $scenario->add_action($action);

    #Close SMB2 TCP Connection
    $action = new swifttest::action('SMB2', 'Close SMB2 TCP Connection');
    $scenario->add_action($action);

    return $project;

}

1;

__END__

=head1 NAME

PTEFIT::LDX::CIFS_SMB2_1K_IOPS.pm

=head1 DESCRIPTION

L<C<PTEFIT::LDX::CIFS_SMB2_1K_IOPS.pm>> Runs a SMB2 workload that is roughly 1K IOPS.  Contact Aaron Carter for more information on this workload.

=head1 DERIVED FROM

L<C<NATE::Workload::LDX>>

=head1 WORKLOAD TAGS

=over

=item C<< smbv2 => 1 >>

Note: Other tags are inherited from L<C<NATE::Workload::LDX>>

=back

=head1 WORKLOAD OPTIONS

Note: Accepts same options as L<C<NATE::Workload::LDX>>

Note: Does not allow passthrough workload options.

=head1 READONLY WORKLOAD OPTIONS

None.

=head1 AUTHOR/MAINTAINER

 Aaron Carter (carteram@netapp.com)

=cut
