#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Workload::LDX::ISCSI_1K_IOPS;

use warnings;
use Carp qw(confess);
use NATE::Log qw( log_global );

use parent qw(NATE::Workload::LDX);

use swifttest;

use Params::Validate qw(SCALAR ARRAYREF);

use constant WORKLOAD_TAGS => {
    ldx => 1,
    iscsi => 1,
};

use constant WORKLOAD_DESCRIPTION => 'This is a basic ISCSI workload to run ~1K IOPs';

use constant FILE_SIZE => 1024000;

use constant FILE_OWNER_UID => 65534;

use constant FINE_OWNER_GID => 65534;

use constant FILE_ACCESS_MODE => 644;

use constant TRACE_PARAMETER => 4000000;

use constant WORKLOAD_OPTIONS_SPEC => {
    spec => {
        ldx_ip           => { type => SCALAR, optional => 0 },
        ldx_ports        => { type => ARRAYREF, optional => 0 },
        stop_io_on_start => { type => SCALAR, default => 1 },
        ipv4_range       => { type => SCALAR, optional => 0 },
        ipv6_range       => { type => SCALAR, optional => 0 },
        ipv4_netmask     => { type => SCALAR, optional => 0 },
        ipv6_netmask     => { type => SCALAR, optional => 0 },
        ipv4_gw          => { type => SCALAR, optional => 0 },
        ipv6_gw          => { type => SCALAR, optional => 0 },
        iscsi_file        => { type => SCALAR, optional => 0 },
        duration         => { type => SCALAR, optional => 0 },
    },
    allow_extra => 0
};

##############################################################################
#  Description: Build an iSCSI project containing logins/read/writes.
#
##############################################################################
sub build_ldx_project {
    my ($pkg, %opts) = @_;

    my $swt_appliance_ip = $opts{ldx_ip};
    my $client_port_number = $opts{ldx_ports}->[0];
    my $ipv4_range = $opts{ipv4_range};
    my $ipv6_range = $opts{ipv6_range};
    my $ipv4_netmask = $opts{ipv4_netmask};
    my $ipv6_netmask = $opts{ipv6_netmask};
    my $ipv4_gw = $opts{ipv4_gw};
    my $ipv6_gw = $opts{ipv6_gw};
    my $iscsi_file = $opts{iscsi_file};
    my $duration = $opts{duration};

    # Create project with specified name
    my $project = new swifttest::project("ISCSI 1K IOPS Test");
    log_global()->log("Created project $project");
    # Create user parameters with users/passwords - load them from specified User file
    my $user_parameters_0 = swifttest::user_parameters::load($iscsi_file);
    $project->add_user_parameters(0, $user_parameters_0);


    # Create data content
    my $data_content = new swifttest::data_content();
    my $root_node = $data_content->get_root_node();

    # Fill data content with 24 nodes that use one of two data content providers
    for (my $file = 1; $file <= 24; $file++) {
        my $file_name = sprintf("FILE%02u.TEST", $file);
        my $content_name = sprintf("::DataContent(%d)", 1 - ($file % 2));
        $root_node->add_node(new swifttest::data_content_fs_node($file_name, $content_name, FILE_SIZE, FILE_OWNER_UID, FINE_OWNER_GID, FILE_ACCESS_MODE));
    }

    $data_content->add_provider('random',     '::DataContent(0)');
    $data_content->add_provider('sequential', '::DataContent(1)');

    # Create trace parameters (4MB max)
    my $trace_parameters = new swifttest::trace_parameters(TRACE_PARAMETER);

    # Create client port
    my $port = new swifttest::port();
    $project->add_port($port);
    $port->set_kind(swifttest::port::CLIENT);
    $port->set_portnum($client_port_number);
    $port->set_appliance($swt_appliance_ip);
    $port->set_datacontent($data_content);
    $port->set_traceparameters($trace_parameters);

    # Create client network
    my $network = new swifttest::net($ipv4_range, $ipv4_netmask, 254, 1, $ipv4_gw);
    $network->set_ipv6_enabled(1);
    $network->set_ipv6($ipv6_range, $ipv6_netmask, 16000, 1, $ipv6_gw);
    $port->add_net($network);
    $network->set_gw_enabled(1);
    $network->set_ipv6_gw_enabled(1);

    # Create ISCSI scenario
    my $iscsi_scenario = _build_iscsi_scenario($duration);
    $network->add_scenario($iscsi_scenario);

    return $project;
}

##############################################################################
# _build_iscsi_scenario
#
# Description: This method will build and ISCSI scenario to add to the project
#
##############################################################################
sub _build_iscsi_scenario{
	my $dur = shift;
    # Create client scenario and loads
    my $iscsi_scen = new swifttest::scenario();

    my $load = new swifttest::load('ramp_up', 15, 0, 0);
    $iscsi_scen->add_load($load);
    $load = new swifttest::load('scenarios', $dur, 10000, 1000);
    $iscsi_scen->add_load($load);
    $load = new swifttest::load('ramp_down', 35, 0, 0);
    $iscsi_scen->add_load($load);

    my $action = new swifttest::action('iSCSI', 'Open iSCSI Connection');
    $iscsi_scen->add_action($action);

    # Using set_parameter() method to configure the action
    $action->set_parameter('Destination Address', '= @UP(0,A)');
    $action->set_parameter('Destination Port', '3260');
    $action->set_parameter('Output Handle', '1: SCSIConnectionHandle');

    $action = new swifttest::action('iSCSI', 'Login Request');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Target Name', '= @UP(0,B)');

    $action = new swifttest::action('SCSI', 'Report LUNs');
    $iscsi_scen->add_action($action);

    #Begin Loop
    $action = new swifttest::action('SWT', 'Begin Loop');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Repeat Count', '100');

    #Read Capacity
    $action = new swifttest::action('SCSI', 'Read Capacity(10)');
    $iscsi_scen->add_action($action);

    #Delay Execution
    $action = new swifttest::action('SWT', 'Delay Execution');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Delay Interval', '1000');

    #SCSI Write
    $action = new swifttest::action('SCSI', 'Write(10)');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Chunk Size', '1024');
    $action->set_parameter('Total Transfer Size', '1024');

    #Delay Execution
    $action = new swifttest::action('SWT', 'Delay Execution');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Delay Interval', '1000');

    #SCSI Read
    $action = new swifttest::action('SCSI', 'Read(16)');
    $iscsi_scen->add_action($action);
    $action->set_parameter('Chunk Size', '1024');
    $action->set_parameter('Total Transfer Size', '1024');

    #End Loop
    $action = new swifttest::action('SWT', 'End Loop');
    $iscsi_scen->add_action($action);

    #Logout Request
    $action = new swifttest::action('iSCSI', 'Logout Request');
    $iscsi_scen->add_action($action);

    #Close iSCSI Connection
    $action = new swifttest::action('iSCSI', 'Close iSCSI Connection');
    $iscsi_scen->add_action($action);

    return $iscsi_scen;
}

1;

__END__

=head1 NAME

PTEFIT::LDX::ISCSI_1K_IOPS

=head1 DESCRIPTION

L<C<PTEFIT::LDX::ISCSI_1K_IOPS>> Builds a workload with ISCSI with roughly 1K IOPS.  Contact Aaron Carter for more information on this workload.

=head1 DERIVED FROM

L<C<NATE::Workload::LDX>>

=head1 WORKLOAD TAGS

=over

=item C<< iscsi => 1 >>

Note: Other tags are inherited from <NATE::Workload::LDX>>

=back

=head1 WORKLOAD OPTIONS

Note: Accepts same options as L<C<NATE::Workload::LDX>>

Note: Does not allow passthrough workload options.

=head1 READONLY WORKLOAD OPTIONS

None.

=head1 AUTHOR/MAINTAINER

Aaron Carter (carteram@netapp.com)

=cut
