#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Workload::LDX::NFSv3_1K_IOPS;

use warnings;
use Carp qw(confess);
use NATE::Log qw( log_global );

use parent qw(NATE::Workload::LDX);

use swifttest;

use Params::Validate qw(SCALAR ARRAYREF);

use constant WORKLOAD_TAGS => {
    ldx => 1,
    nfsv3 => 1,
};

use constant WORKLOAD_DESCRIPTION => 'This is a NFS workload running NFSv3 to run ~1K IOPs';

use constant FILE_SIZE => 1024000;

use constant FILE_OWNER_UID => 65534;

use constant FINE_OWNER_GID => 65534;

use constant FILE_ACCESS_MODE => 644;

use constant TRACE_PARAMETER => 4000000;

use constant WORKLOAD_OPTIONS_SPEC => {
    spec => {
        ldx_ip           => { type => SCALAR, optional => 0 },
        ldx_ports        => { type => ARRAYREF, optional => 0 },
        stop_io_on_start => { type => SCALAR, default => 1 },
        ipv4_range       => { type => SCALAR, optional => 0 },
        ipv6_range       => { type => SCALAR, optional => 0 },
        ipv4_netmask     => { type => SCALAR, optional => 0 },
        ipv6_netmask     => { type => SCALAR, optional => 0 },
        ipv4_gw          => { type => SCALAR, optional => 0 },
        ipv6_gw          => { type => SCALAR, optional => 0 },
        nfs3_file        => { type => SCALAR, optional => 0 },
        duration         => { type => SCALAR, optional => 0 },
    },
    allow_extra => 0
};

##############################################################################
#  Description: Build a NFSv3 project containing reads/writes/locks.
#
##############################################################################
sub build_ldx_project {
    my ($pkg, %opts) = @_;

    my $swt_appliance_ip = $opts{ldx_ip};
    my $client_port_number = $opts{ldx_ports}->[0];
    my $ipv4_range = $opts{ipv4_range};
    my $ipv6_range = $opts{ipv6_range};
    my $ipv4_netmask = $opts{ipv4_netmask};
    my $ipv6_netmask = $opts{ipv6_netmask};
    my $ipv4_gw = $opts{ipv4_gw};
    my $ipv6_gw = $opts{ipv6_gw};
    my $nfs3_file = $opts{nfs3_file};
    my $duration = $opts{duration};

    # Create project with specified name
    my $project = new swifttest::project("NFSv3 Test");
    log_global()->log("Created project $project");
    # Create user parameters with users/passwords - load them from specified User file
    my $user_parameters_0 = swifttest::user_parameters::load($nfs3_file);
    $project->add_user_parameters(0, $user_parameters_0);


    # Create data content
    my $data_content = new swifttest::data_content();
    my $root_node = $data_content->get_root_node();

    # Fill data content with 24 nodes that use one of two data content providers
    for (my $file = 1; $file <= 24; $file++) {
        my $file_name = sprintf("FILE%02u.TEST", $file);
        my $content_name = sprintf("::DataContent(%d)", 1 - ($file % 2));
        $root_node->add_node(new swifttest::data_content_fs_node($file_name, $content_name, FILE_SIZE, FILE_OWNER_UID, FINE_OWNER_GID, FILE_ACCESS_MODE));
    }

    $data_content->add_provider('random',     '::DataContent(0)');
    $data_content->add_provider('sequential', '::DataContent(1)');

    # Create trace parameters (4MB max)
    my $trace_parameters = new swifttest::trace_parameters(TRACE_PARAMETER);

    # Create client port
    my $port = new swifttest::port();
    $project->add_port($port);
    $port->set_kind(swifttest::port::CLIENT);
    $port->set_portnum($client_port_number);
    $port->set_appliance($swt_appliance_ip);
    $port->set_datacontent($data_content);
    $port->set_traceparameters($trace_parameters);

    # Create client network
    my $network = new swifttest::net($ipv4_range, $ipv4_netmask, 254, 1, $ipv4_gw);
    $network->set_ipv6_enabled(1);
    $network->set_ipv6($ipv6_range, $ipv6_netmask, 16000, 1, $ipv6_gw);
    $port->add_net($network);
    $network->set_gw_enabled(1);
    $network->set_ipv6_gw_enabled(1);
    $network->set_parameter('ArpCachePerSourceIp', '0');
    $network->set_parameter('NdCachePerSourceIp', '0');
    $network->set_parameter('ArpTimeToLive', 1000);
    $network->set_parameter('MACAddressEnabled', '0');

    # Create NFSv3 scenario
    my $nfsv3_scenario = _build_nfsv3_scenario($duration);
    $network->add_scenario($nfsv3_scenario);

    return $project;
}

##############################################################################
# _build_nfsv3_scenario
#
# Description: This method will build and NFSv3 scenario to add to the project
#
#
#
##############################################################################
sub _build_nfsv3_scenario{
    # Create client scenario and loads
    my $nfsv3_scen = new swifttest::scenario();
    my $dur = shift;

    my $load;
    $load = new swifttest::load('ramp_up', 15, 0, 0);
    $nfsv3_scen->add_load($load);
    $load = new swifttest::load('scenarios', $dur, 10000, 1000);
    $nfsv3_scen->add_load($load);
    $load = new swifttest::load('ramp_down', 35, 0, 0);
    $nfsv3_scen->add_load($load);

    # Fill client scenario with actions; we specify just changed parameters
    my $action;
    $action = new swifttest::action('PMAPv2', 'Open Portmapper TCP Connection');
    $nfsv3_scen->add_action($action);

    # Using set_parameter() method to configure the action
    $action->set_parameter('Destination Address', '= @UP(0,A)');
    $action->set_parameter('Destination Port', '111');
    $action->set_parameter('Output Handle', '1: PortmapperConnectionHandle');

    $action = new swifttest::action('PMAPv2', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('PMAPv2', 'Get Program Port');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Version', '3');

    $action = new swifttest::action('MNTv3', 'Open Mount TCP Connection');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Destination Address', '= @UP(0,A)');
    $action->set_parameter('Destination Port', '635');

    $action = new swifttest::action('MNTv3', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('MNTv3', 'Mount Directory');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Directory', '= @STRING(/) + @UP(0,B)');

    $action = new swifttest::action('PMAPv2', 'Get Program Port');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Program', 'NFS');
    $action->set_parameter('Version', '3');

    $action = new swifttest::action('NFSv3', 'Open NFS TCP Connection');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Destination Address', '= @UP(0,A)');

    $action = new swifttest::action('PMAPv2', 'Close Portmapper TCP Connection');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('MNTv3', 'Close Mount TCP Connection');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('NFSv3', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('NFSv3', 'Create Directory');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Subdirectory Name', '= @STRING(fg_dir_0)');

    $action = new swifttest::action('PMAPv2', 'Open Portmapper TCP Connection');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Destination IP Address', '= @UP(0,A)');
    $action->set_parameter('Destination IP Port', '111');

    $action = new swifttest::action('PMAPv2', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('PMAPv2', 'Get Program Port');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Program', 'NFS');
    $action->set_parameter('Version', '3');

    $action = new swifttest::action('MNTv3', 'Open Mount TCP Connection');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Destination Address', '= @UP(0,A)');
    $action->set_parameter('Destination Port', '635');

    $action = new swifttest::action('MNTv3', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('MNTv3', 'Mount Directory');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Directory', '= @STRING(/) + @UP(0,B) + @STRING(/fg_dir_0)');

    $action = new swifttest::action('PMAPv2', 'Get Program Port');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Version', '3');

    $action = new swifttest::action('NFSv3', 'Open NFS TCP Connection');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Destination Address', '= @UP(0,A)');

    $action = new swifttest::action('PMAPv2', 'Close Portmapper TCP Connection');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('MNTv3', 'Close Mount TCP Connection');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('NFSv3', 'Null');
    $nfsv3_scen->add_action($action);

    $action = new swifttest::action('NFSv3', 'Access');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Access Permissions', '0x00000017 (23)');

    $action = new swifttest::action('SWT', 'Delay Execution');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Delay Interval', '30');

    #Begin Loop
    $action = new swifttest::action('SWT', 'Begin Loop');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Repeat Count', '100');

    #Create File and name it based on loop count
    $action = new swifttest::action('NFSv3', 'Create File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('File Name', '= @STRING(NFS3_TEST)');

    $action = new swifttest::action('NFSv3', 'Lookup');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Path', '= @STRING(NFS3_TEST)');

    #Do a File Write
    $action = new swifttest::action('NFSv3', 'Write File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Number of Bytes Per Request', '1024');
    $action->set_parameter('Number of Bytes', '4096');

    #Do a File Read
    $action = new swifttest::action('NFSv3', 'Read File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Number of Bytes Per Request', '1024');
    $action->set_parameter('Number of Bytes', '4096');

    #File Set Info
    $action = new swifttest::action('NFSv3', 'Set File Attributes');
    $nfsv3_scen->add_action($action);

    #File Commit
    $action = new swifttest::action('NFSv3', 'Commit File');
    $nfsv3_scen->add_action($action);

    #Do a File Write
    $action = new swifttest::action('NFSv3', 'Write File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Number of Bytes Per Request', '1024');
    $action->set_parameter('Number of Bytes', '4096');

    #Do a File Read
    $action = new swifttest::action('NFSv3', 'Read File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('Number of Bytes Per Request', '1024');
    $action->set_parameter('Number of Bytes', '4096');

    #File Set Info
    $action = new swifttest::action('NFSv3', 'Set File Attributes');
    $nfsv3_scen->add_action($action);

    #End Loop
    $action = new swifttest::action('SWT', 'End Loop');
    $nfsv3_scen->add_action($action);

    #File Remove
    $action = new swifttest::action('NFSv3', 'Remove File');
    $nfsv3_scen->add_action($action);
    $action->set_parameter('File Name', 'NFS3_TEST*');

    #Directory Remove
    $action = new swifttest::action('NFSv3', 'Remove Directory');
    $nfsv3_scen->add_action($action);

    return $nfsv3_scen;
}

1;

__END__

=head1 NAME

PTEFIT::LDX::NFSv3_1K_IOPS

=head1 DESCRIPTION

L<C<NATE::Workload::LDX>> Builds a workload with all flavors of NFSv3.  Contact Aaron Carter for more information on this workload.

=head1 DERIVED FROM

L<C<PTEFIT::LDX::NFSv3_1K_IOPS>>

=head1 WORKLOAD TAGS

=over

=item C<< nfsv3 => 1 >>

Note: Other tags are inherited from <NATE::Workload::LDX>>

=back

=head1 WORKLOAD OPTIONS

Note: Accepts same options as L<C<NATE::Workload::LDX>>

Note: Does not allow passthrough workload options.

=head1 READONLY WORKLOAD OPTIONS

None.

=head1 AUTHOR/MAINTAINER

Aaron Carter (carteram@netapp.com)

=cut
