#
# Copyright (c) 2014-2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
package NACL::AppBuilder::Xgen::Config::ONTAP;

use strict;
use warnings;

use parent qw( NACL::Service::Xgen::Config );

use NACL::C::Node;

use NATE::Exceptions::Argument;
use NATE::BaseException qw( :try );
use NATE::Log qw( log_global );

use NACL::AppBuilder::Xgen::Exceptions::InvalidPortType;
use NACL::AppBuilder::Xgen::Exceptions::NoAggregatesAvailable;

use NACL::AppBuilder::Xgen::Resource::Cluster;
use NACL::AppBuilder::Xgen::Resource::Node;
use NACL::AppBuilder::Xgen::Resource::Aggregate;
use NACL::AppBuilder::Xgen::Resource::Client;
use NACL::AppBuilder::Xgen::Resource::VirtualMachine;
use NACL::AppBuilder::Xgen::Resource::Port::Eth;
use NACL::AppBuilder::Xgen::Resource::Port::Fcp;
use NACL::AppBuilder::Xgen::Resource::Port::Fcoe;
use NACL::AppBuilder::Xgen::Resource::Fabric;
use NACL::AppBuilder::Xgen::Resource::Switch;

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    my ($class, %args) = @_;
    $Log->enter() if $may_enter;

    my $self = $class->SUPER::new(%args);

    my $nodes = delete($args{nodes});
    my @nacl_node_objs = ();
    if (!$nodes) {
        @nacl_node_objs = NACL::C::Node->find();
    } else {
        @nacl_node_objs = map { NACL::C::Node->new(name => $_); } @{$nodes};
    }

    $self->{nacl_node_objs} = \@nacl_node_objs;

    $Log->exit() if $may_exit;
    return $self;
}

sub build {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

    $self->SUPER::build(%args);

    $self->_create_node_lookup_table(%args);

    $self->generate_cluster_and_node_resources(%args);

    $self->generate_aggregate_resources(%args);

    $self->generate_port_resources(%args);

    $self->_create_port_lookup_table(%args);

    $self->generate_client_resources(%args);

    $self->generate_fabric_resources(%args);

    $Log->exit() if $may_exit;
}

sub get_node_id {
    my ($self, $node_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{node_lookup_table}->{$node_name}->{node_id};
}

sub get_node_resource {
    my ($self, $node_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{node_lookup_table}->{$node_name}->{node_resource};
}

sub get_cluster_name {
    my ($self, $node_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{node_lookup_table}->{$node_name}->{cluster};
}

sub set_cluster_name {
    my ($self, $node_name, $cluster_name) = @_;
    $Log->enter() if $may_enter;

    $self->{node_lookup_table}->{$node_name}->{cluster} = $cluster_name;

    $Log->exit() if $may_exit;
}

sub get_port_id {
    my ($self, $node_name, $port_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{port_lookup_table}->{$node_name}->{$port_name}->{id};
}

sub get_fabric_id {
    my ($self, $node_name, $port_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{port_lookup_table}->{$node_name}->{$port_name}->{fabric_id};
}

sub get_port_lookup_table_by_node {
    my ($self, $node_name) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{port_lookup_table}->{$node_name};
}

sub get_ports_by_node_and_fabric_id {
    my ($self, $node_name, $type, $fabric_id) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{fabric_lookup_table}->{$node_name}->{$type}->{$fabric_id};
}

sub get_fabric_lookup_table_by_node {
    my ($self, $node_name, $type) = @_;
    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;
    return $self->{fabric_lookup_table}->{$node_name}->{$type};
}


sub generate_cluster_and_node_resources {
    my($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $nacl_node_objs = $self->{nacl_node_objs};

    my $cluster_names = {};

    foreach my $nacl_node_obj (@{$nacl_node_objs}) {
        # figure out what cluster $nacl_node_obj belongs to
        # once you do that update the lookup table for this node
        my @clusters =
            NACL::CS::ClusterIdentity->fetch(command_interface => $nacl_node_obj);

        my $node_name = $nacl_node_obj->node();
        my $cluster_name = $clusters[0]->name();

        # associating this node with its cluster name
        $self->set_cluster_name($node_name, $cluster_name);

        # hash of unique cluster names that points to an array
        # of nodes in that cluster
        push(@{$cluster_names->{$cluster_name}}, $node_name);
    }

    foreach my $cluster_name (keys %{$cluster_names}) {
        my $nodes = $cluster_names->{$cluster_name};
        my $cluster_resource =
            new NACL::AppBuilder::Xgen::Resource::Cluster(cluster => $cluster_name,
                                                nodes   => $nodes);

        $cluster_resource->set_shared_attributes(cluster_name => $cluster_name);

        $self->add_resources(resources => [$cluster_resource]);

        foreach my $node (@$nodes) {
            my $node_resource =
                new NACL::AppBuilder::Xgen::Resource::Node(node => $node);
            $node_resource->add_parent_resources(resources => [$cluster_resource]);

            $self->add_node_resource_to_lookup_table($node_resource);
            $self->add_resources(resources => [$node_resource]);
        }
    }

    $Log->exit() if $may_exit;
}

sub generate_aggregate_resources {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $no_new_aggrs = delete($args{no_new_aggrs}) // 1;

    my $nacl_node_objs = $self->{nacl_node_objs};

    my $aggr_info = NACL::AppBuilder::Xgen::Resource::Aggregate->generate_aggr_info(nacl_node_objs => $nacl_node_objs,
	   no_new_aggrs   => $no_new_aggrs);

    # create Aggr objects based on the content of $aggr_info
    my @aggr_resources = ();

    # loop over all nodes in the cluster
    foreach my $node_name (keys(%{$aggr_info})) {

        my $aggr_name_hash = {};

        my $current_aggr_layout = delete($aggr_info->{$node_name}->{current_aggr_layout});

        my $node_resource = $self->get_node_resource($node_name);

        # loop over existing aggregates to build Aggregate Resource Objects
        foreach my $aggr_obj (@{$current_aggr_layout}) {
            my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate    => $aggr_obj->aggregate(),
                                                                      diskcount    => $aggr_obj->diskcount(),
                                                                      nodes        => [$node_name],
                                                                      maxraidsize  => $aggr_obj->maxraidsize());
            $aggr_resource->add_parent_resources(resources => [$node_resource]);

            push (@aggr_resources, $aggr_resource);
            $aggr_name_hash->{$aggr_obj->aggregate()}++;
        }

        my $aggr_index = 0; # suffix of aggregate name

        # if there are no new aggrs to be made go to the next node
        next if not exists($aggr_info->{$node_name}->{aggr_count});

        # dont create new aggrs if told to not do so
        next if ($no_new_aggrs);

        # create Aggreate Resource objects for aggregates we want to create
        for (my $i = 0; $i < $aggr_info->{$node_name}->{aggr_count}; $i++) {
            # base aggregate name
            my $aggr_name = "xgen_aggr_" . $node_name . "_";

            $aggr_name =~ s/-/_/g;

            # check hash of aggregate names until one does not exist
            while ($aggr_name_hash->{$aggr_name . $aggr_index}) {
                $aggr_index++; # increment aggr index if aggr name exists in aggr hash
            }

            $aggr_name = $aggr_name . $aggr_index;
            $aggr_name_hash->{$aggr_name}++; # add new aggr name to aggr hash

            my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate   => $aggr_name,
                                                                      diskcount   => $aggr_info->{$node_name}->{disks_per_aggr},
                                                                      nodes       => [$node_name],
                                                                      maxraidsize => $aggr_info->{$node_name}->{max_raid_size});
            $aggr_resource->add_parent_resources(resources => [$node_resource]);

            push (@aggr_resources, $aggr_resource);
        }
    }

    if (scalar(@aggr_resources)) {
        $self->add_resources(resources => \@aggr_resources);
    } else {
        NACL::AppBuilder::Xgen::Exceptions::NoAggregatesAvailable->throw("There are no aggregate resources availble for use (or that can be created)");
    }

    $Log->exit() if $may_exit;
}

sub generate_port_resources {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $nacl_node_objs = $self->{nacl_node_objs};

    my $port_info = NACL::AppBuilder::Xgen::Resource::Port->generate_port_info(nacl_node_objs => $nacl_node_objs,
                                                                     %args);

    # create Port objects based on the content of $port_info

    # loop over all nodes in the cluster
    foreach my $node_name (keys(%{$port_info})) {
        my $node_resource = $self->get_node_resource($node_name);

        # loop over ports to build Port Resource Objects
        foreach my $port_type (keys(%{$port_info->{$node_name}})) {

            foreach my $port_obj (@{$port_info->{$node_name}->{$port_type}}) {
                my $port_name;
                if ($port_type eq 'eth') {
                    $port_name = $port_obj->port();
                } else {
                    $port_name = $port_obj->adapter();
                }

                my $port_resource;
                my %port_resource_args = (node_name    => $node_name,
                                          port_name    => $port_name,
                                          port_type    => $port_type);
                if ($port_type eq 'eth') {
                    my $ipspace = "";
                    eval {
                        $ipspace = $port_obj->ipspace();
                    };
                    if ($@ or not $ipspace) {
                        # Assume this is a pre ipspace build.
                    } else {
                       $port_resource_args{ipspace} = $ipspace;
                       $port_resource_args{broadcast_domain} = $port_obj->broadcast_domain();
                    }

                    $port_resource = new NACL::AppBuilder::Xgen::Resource::Port::Eth(%port_resource_args);
                } elsif ($port_type eq 'fcp') {
                    # Comes in the format auto-cisco5010-01:vfc9
                    my $switch = $port_obj->switch_port();
                    $switch =~ s/:.*$//i;
                    # Add switch to the port resource args by removing the :port information.
                    $port_resource_args{switch} = $switch;
                    $port_resource = new NACL::AppBuilder::Xgen::Resource::Port::Fcp(%port_resource_args);
                } elsif ($port_type eq 'fcoe') {
                    # Comes in the format auto-cisco5010-01:vfc9
                    my $switch = $port_obj->switch_port();
                    $switch =~ s/:.*$//i;
                    # Add switch to the port resource args by removing the :port information.
                    $port_resource_args{switch} = $switch;
                    $port_resource = new NACL::AppBuilder::Xgen::Resource::Port::Fcoe(%port_resource_args);
                } else {
                    NACL::AppBuilder::Xgen::Exceptions::InvalidPortType->throw("'port_type' should be eth,fcp, " .
                                                                               "or fcoe. Instead it was passed in " .
                                                                               "as $port_type");
                }

                $port_resource->add_parent_resources(resources => [$node_resource]);
                $self->add_resources(resources => [$port_resource]);
            }
        }
    }

    $Log->exit() if $may_exit;
}

sub generate_client_resources {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $iscsi_clients_str = delete($args{iscsi_clients}) // [];
    my $fcp_clients_str   = delete($args{fcp_clients})   // [];
    my $fcoe_clients_str  = delete($args{fcoe_clients})  // [];
    my @iscsi_clients = @{$iscsi_clients_str};
    my @fcp_clients   = @{$fcp_clients_str};
    my @fcoe_clients  = @{$fcoe_clients_str};

    # create a datastructure describing each client
    # and their respective protocols
    my $hash = {};
    foreach my $iscsi_client (@iscsi_clients) {
        push(@{$hash->{$iscsi_client}}, "iscsi");
    }
    foreach my $fcp_client (@fcp_clients) {
        push(@{$hash->{$fcp_client}}, "fcp");
    }
    foreach my $fcoe_client (@fcoe_clients) {
        push(@{$hash->{$fcoe_client}}, "fcoe");
    }

    my @clients = ();
    foreach my $client (keys %{$hash}) {
        my $client_resource =
            new NACL::AppBuilder::Xgen::Resource::Client(name      => $client,
                                                         protocols => $hash->{$client});
        push(@clients, $client_resource);
    }

    # register the client resources if we have any
    if (scalar @clients) {
        $self->add_resources(resources => \@clients);
    }

    # find hypervisor resources in the clients and generate Virtual machine resources for their VMs
    my @vm_resources = ();
    foreach my $cli_res (@clients) {
        if ($cli_res->get_attribute('is_hypervisor')) {
            my $vms = $cli_res->get_attribute('virtual_machines');
            foreach my $vm (@{$vms}) {
                my $vm_res = new NACL::AppBuilder::Xgen::Resource::VirtualMachine(name => $vm,
                                                                                  hypervisor_name => $cli_res->get_attribute("name"),
                                                                                  hypervisor_type => $cli_res->get_attribute("ostype"));
                $vm_res->add_parent_resources(resources => [$cli_res]);
                push(@vm_resources, $vm_res);
            }
        }
    }
    if (scalar(@vm_resources)) {
        $self->add_resources(resources => \@vm_resources);
    }

    $Log->exit() if $may_exit;
}

sub generate_fabric_resources {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $fcp_fabric_data = delete($args{fcp_fabric_data});
    my $fcoe_fabric_data = delete($args{fcoe_fabric_data});

    if ($fcp_fabric_data) {
        $self->_generate_fabric_resource(fabric_data => $fcp_fabric_data,
                                         resource_prefix => "fcp_fabric_");
    }

    if ($fcoe_fabric_data) {
        $self->_generate_fabric_resource(fabric_data => $fcoe_fabric_data,
                                         resource_prefix => "fcoe_fabric_");
    }

    $Log->exit() if $may_exit;
}

sub _generate_fabric_resource {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my $fabric_data = delete($args{fabric_data}) // NATE::Exceptions::Argument->throw("Missing required parameter 'fabric_data'");
    my $resource_prefix = delete($args{resource_refix}) // "";

    # TODO: in the future, using the fabric lookup table, we can discover the fabrics dynamically
    # but lets just assume the user gave us the info for all the fabrics in a static fashion
    # since that is much easier
    my @new_switch_resources = ();
    my @fab_resources = ();

    foreach my $fab (keys %{$fabric_data}) {
        my %extra = ();
        if ($fabric_data->{$fab}->{admin_domain}) {
            $extra{admin_domain} = delete $fabric_data->{$fab}->{admin_domain};
        }
        if ($fabric_data->{$fab}->{fabric_id}) {
            $extra{fabric_id} = delete $fabric_data->{$fab}->{fabric_id};
        }
        if ($fabric_data->{$fab}->{virtual_fabric}) {
            $extra{virtual_fabric} = delete $fabric_data->{$fab}->{virtual_fabric};
        }
        if ($fabric_data->{$fab}->{fabric_prefix}) {
            $extra{prefix} = delete $fabric_data->{$fab}->{fabric_prefix};
        }

        # parse out the switches for this fabric
        my $switches = delete $fabric_data->{$fab}->{switches} // [];
        if (ref($switches) ne "ARRAY" or !scalar(@{$switches})) {
            $Log->comment("There are no switches defined for fabric '${resource_prefix}${fab}', " .
                          "not registering fabric as a resource");
            next;
        }

        # make the fabric resource and register
        my $fab_resource_name = $resource_prefix . $fab;
        my $fab_resource = new NACL::AppBuilder::Xgen::Resource::Fabric(name => $fab_resource_name,
                                                                        switches => $switches,
                                                                        %extra);

        push(@fab_resources, $fab_resource);

        # delete the prefix from extra because the switch doesn't need it
        if (exists $extra{prefix}) { delete $extra{prefix} };

        # make the switch resources and associate them to this fabric.
        # make sure we don't create duplicate switch resources since they
        # can be part of multiple fabrics
        my $switch_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Switch");
        foreach my $new_sw (@{$switches}) {
            # scan existing switch resources
            my $exists = 0;

            foreach my $sw_res (@{$switch_resources}) {
                if ($sw_res->get_attribute("name") eq $new_sw) {
                    $exists = 1;
                    # we found a match with an existing switch, update the fabric names field
                    push(@{$sw_res->get_attribute("fabric_names")}, $fab_resource_name);
                    last;
                }
            }
            foreach my $sw_res (@new_switch_resources) {
                if ($sw_res->get_attribute("name") eq $new_sw) {
                    $exists = 1;
                    # we found a match with an existing switch, update the fabric names field
                    push(@{$sw_res->get_attribute("fabric_names")}, $fab_resource_name);
                    last;
                }
            }
            if (!$exists) {
                # this switch doesnt exist, make a new resource for it, and associate this fabric to it
                push(@new_switch_resources,
                     new NACL::AppBuilder::Xgen::Resource::Switch(name => $new_sw,
                                                                  fabric_names => [$fab_resource_name],
                                                                  %extra));
            }
        }
    }

    if (scalar(@fab_resources)) {
        $self->add_resources(resources => \@fab_resources);
    }

    # if we made new switches, register them
    if (scalar(@new_switch_resources)) {
        $self->add_resources(resources => \@new_switch_resources);
    }

    $Log->exit() if $may_exit;
}

sub _create_node_lookup_table {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

    my $i = 1;
    foreach my $nacl_node_obj (@{$self->{nacl_node_objs}}) {
        my $node = $nacl_node_obj->node();

        $self->{node_lookup_table}->{$node}->{node_id} = $i++;
    }

    $Log->exit() if $may_exit;
}

sub add_node_resource_to_lookup_table {
    my ($self, $resource_to_add) = @_;

    $Log->enter() if $may_enter;

    my $node = $resource_to_add->get_attribute('node');
    $self->{node_lookup_table}->{$node}->{node_resource} = $resource_to_add;

    $Log->exit() if $may_exit;
}

sub _create_port_lookup_table {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

    my $eth_fabric_data  = delete $args{eth_fabric_data}  // {};
    my $fcp_fabric_data  = delete $args{fcp_fabric_data}  // {};
    my $fcoe_fabric_data = delete $args{fcoe_fabric_data} // {};

    my @eth_fabric_data_keys  = sort keys %$eth_fabric_data;
    my @fcp_fabric_data_keys  = sort keys %$fcp_fabric_data;
    my @fcoe_fabric_data_keys = sort keys %$fcoe_fabric_data;

    my $port_lookup_table;
    my $fabric_lookup_table;
    my $i = 1;

    my $eth_port_objs  = $self->get_resources(type => 'NACL::AppBuilder::Xgen::Resource::Port::Eth');
    my $fcp_port_objs  = $self->get_resources(type => 'NACL::AppBuilder::Xgen::Resource::Port::Fcp');
    my $fcoe_port_objs = $self->get_resources(type => 'NACL::AppBuilder::Xgen::Resource::Port::Fcoe');

    if (@eth_fabric_data_keys) {
        # Create an ipspace and broadcast domain to fabric_id mapping.
        my %ipspace_to_fabric_table;
        foreach my $key (@eth_fabric_data_keys) {
            my $ipspace             = $eth_fabric_data->{$key}->{ipspace};
            my $broadcast_domain    = $eth_fabric_data->{$key}->{broadcast_domain};
            $ipspace_to_fabric_table{$ipspace}{$broadcast_domain} = $key;
        }

        foreach my $port_obj (@$eth_port_objs) {
            my $ipspace             = $port_obj->get_attribute('ipspace');
            my $broadcast_domain    = $port_obj->get_attribute('broadcast_domain');
            my $node_name           = $port_obj->get_attribute('node_name');
            my $port_name           = $port_obj->get_attribute('port_name');
            my $port_id = ++$port_lookup_table->{$node_name}->{_eth_id};

            # Only add the port to the hashes if its ipspace and broadcast domain are defined in the eth_fabric_hash.
            if (defined $ipspace_to_fabric_table{$ipspace} and
                defined $ipspace_to_fabric_table{$ipspace}{$broadcast_domain}) {
                # set the fabric id to be corresponding to the ipspace and broadcast domain.
                $port_lookup_table->{$node_name}->{$port_name}->{fabric_id} = $ipspace_to_fabric_table{$ipspace}{$broadcast_domain};

                push(@{$fabric_lookup_table->{$node_name}->{eth}->{$port_lookup_table->{$node_name}->{$port_name}->{fabric_id}}}, $port_name);

                $port_lookup_table->{$node_name}->{$port_name}->{id} = $port_id;
            } else {
                $Log->warn("Port:$port_name Node:$node_name does not belong to any of the ethernet fabrics defined");
            }
        }

        # Remove the _eth_id from the port lookup table that was used for making the port_id.
        foreach my $n (keys %{$port_lookup_table}) {
            delete $port_lookup_table->{$n}->{_eth_id};
        }
    }

    if (@fcp_fabric_data_keys) {
        # Create an switch to fabric_id mapping.
        my %switch_to_fabric_table;
        foreach my $key (@fcp_fabric_data_keys) {
            my @switches = @{$fcp_fabric_data->{$key}{switches}};
            foreach my $switch (@switches) {
                $switch_to_fabric_table{$switch} = $key;
            }
        }

        foreach my $port_obj (@$fcp_port_objs) {
            my $node_name = $port_obj->get_attribute('node_name');
            my $port_name = $port_obj->get_attribute('port_name');
            my $port_id   = ++$port_lookup_table->{$node_name}->{_fcp_id};
            my $switch    = $port_obj->get_attribute('switch');

            if(defined $switch_to_fabric_table{$switch}) {
                $port_lookup_table->{$node_name}->{$port_name}->{id} = $port_id;

                # set the fabric id to be the fabric corresponding to the switch to fabric table.
                $port_lookup_table->{$node_name}->{$port_name}->{fabric_id} = $switch_to_fabric_table{$switch};

                push(@{$fabric_lookup_table->{$node_name}->{fcp}->{$port_lookup_table->{$node_name}->{$port_name}->{fabric_id}}}, $port_name);
            } else {
                $Log->warn("Port:$port_name Node:$node_name does not belong to any of the fcp fabrics defined");
            }
        }
        foreach my $n (keys %{$port_lookup_table}) {
            delete $port_lookup_table->{$n}->{_fcp_id};
        }
    }

    if (@fcoe_fabric_data_keys) {
        # Create an switch to fabric_id mapping.
        my %switch_to_fabric_table;
        foreach my $key (@fcoe_fabric_data_keys) {
            my @switches = @{$fcoe_fabric_data->{$key}->{switches}};
            foreach my $switch (@switches) {
                $switch_to_fabric_table{$switch} = $key;
            }
        }

        foreach my $port_obj (@$fcoe_port_objs) {
            my $node_name = $port_obj->get_attribute('node_name');
            my $port_name = $port_obj->get_attribute('port_name');
            my $port_id   = ++$port_lookup_table->{$node_name}->{_fcoe_id};
            my $switch    = $port_obj->get_attribute('switch');

            if(defined $switch_to_fabric_table{$switch}) {
                $port_lookup_table->{$node_name}->{$port_name}->{id} = $port_id;

                # set the fabric id to be the fabric corresponding to the switch to fabric table.
                $port_lookup_table->{$node_name}->{$port_name}->{fabric_id} = $switch_to_fabric_table{$switch};

                push(@{$fabric_lookup_table->{$node_name}->{fcoe}->{$port_lookup_table->{$node_name}->{$port_name}->{fabric_id}}}, $port_name);
            } else {
                $Log->warn("Port:$port_name Node:$node_name does not belong to any of the fcoe fabrics defined");
            }
        }
        foreach my $n (keys %{$port_lookup_table}) {
            delete $port_lookup_table->{$n}->{_fcoe_id};
        }
    }
    $self->{port_lookup_table} = $port_lookup_table;
    $self->{fabric_lookup_table} = $fabric_lookup_table;
    $Log->exit() if $may_exit;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Config::ONTAP

=head1 AUTHOR/MAINTAINER

=over

=item NACL (dl-nacl-dev@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
