#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AggregateFile Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::AggregateFile

=head1 DESCRIPTION

C<NACL::C::AggregateFile> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP AggregateFile.  A related class is
L<NACL::CS::AggregateFile|lib-NACL-CS-AggregateFile-pm>, which represents
the past state of an ONTAP aggregate file.

This class makes heavy use of the mixin class C<NACL::C::_Mixins::VolumeFileDirectory>.

=cut

package NACL::C::AggregateFile;

use strict;
use warnings;

use base 'NACL::C::Component::ONTAP';

use Params::Validate qw(:all);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use feature 'state';
use NACL::ComponentUtils qw (_optional_scalars);

use NACL::C::StorageAggregate;
use NACL::CS::AggregateFile;
use NACL::C::_Mixins::VolumeFileDirectory
  qw(:file_create :file_delete _call_on_apiset :file_operation);
use NACL::C::Exceptions::AggregateFile::DoesNotExist;
use NACL::C::Exceptions::AggregateFile::BlockIsAHole;

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 path

The path to the file

=over

=cut

=head1 METHODS

=head2 new

    my $file_obj = NACL::C::AggregateFile->new(
        command_interface => $ci,
        path              => $file_path,
    );

Creates a new file object. Note that this does not create a new file
on the filer, it only creates a Perl object.

=cut

use Class::MethodMaker [ scalar => 'path', ];

# Remove the quotes from the path value and store in the
# object.
sub init {
    $Log->enter();
    my ( $self, %opts ) = @_;
    $self->SUPER::init(%opts);
    my $cmode_path =
      $self->_convert_path_to_cmode_format( path => $opts{path} );
    $self->path($cmode_path);
    $Log->exit();
}

=head2 create

    my $file_obj = NACL::C::AggregateFile->create(
        command_interface => $ci,
        path              => $file_path,
        ...
    );

    my @file_objs = NACL::C::AggregateFile->create(
        command_interface => $ci,
        path              => $file_path,
        filecount         => $count,
        ...
    );

(Class method only)

Creates a file at the specified path and returns a Component object
modeling it.

When the "filecount" argument is passed (let's say its value is N), then N
number of files are created and N number of objects are returned. The files
created will be "$filepath.0", "$filepath.1" .. "$filepath.N-1".

Uses a 7Mode CLI or Nodescope-CLI APISet.

Invokes "mkfile".

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $path >>

(Required)
The path to the file.

=item C<< filecount => $count >>

When provided, $count number of files are created and $count number of objects
are returned.

=item C<< size => $size >>

The size of the file to create. If provided as an integer without any units,
then the command assumes to it to be in terms of bytes. If a size suffix is
provided, no space should be present between the size and the suffix. The
allowed suffixes are "b/B", "k/K", "m/M", "g/G". Examples of valid values are:
'10', '100k', '2T'.

=item C<< 'deduplicate-zero-data' => 0 | 1 >>

A value of 1 would detect and deduplicate zeroed data (only valid on volumes
with dedup enabled.)

=item C<< 'step-size-in-blocks' => $num_blocks >>

Number of blocks to skip in between each write.

=item C<< 'stream-name' => $stream >>

A stream is an on disk structure containing a sequence of bytes, analogous to
a file. A stream associated with a file is identified by a name, and data may
be read or written to a file or stream.

=item C<< fragment-file => 0 | 1 >>

Fragment the new file(s) on disk

=item C<< contiguous_fbns => $num_blocks >>

Number of File Blocks(FBNs). It is only valid with fragment-file (-f) option.

=item C<< 'nodescope-node-name' => $node_name >>

(Applicable only for Nodescope-CLI)
Explicitly specify the node on which to run the Nodescope command (the APISet
defaults to running on node "local")

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 0, applicable only for Nodescope-CLI)
If set to 1, the method determines which node the volume hosting the file
is present on and runs the Nodescope CLI command on that particular node.


=back

=back

=cut

sub create {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $objs = $pkg->_create_helper( @args, _is_vol_file => 0 );

    $Log->exit();
    return wantarray ? @$objs : $objs->[0];
}

=head2 delete

    NACL::C::AggregateFile->delete(
        command_interface => $ci,
        path              => $path,
    );

                  or

    $aggregate_file_obj->delete();

                  or

    # Deleting multiple files at once
    NACL::C::AggregateFile->delete(
        command_interface => $ci,
        paths             => \@paths,
    );

(Class or instance method)

Delete a file.

Uses a 7Mode CLI or Nodescope-CLI APIset.

Invokes "rm" in 7Mode and Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for Class Method, Not Applicable for Instance Method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< path => $file_path >>

Path to the file to be deleted.

For class method calls, either this or C<paths> is necessary, for instance
method calls, this argument is not applicable.

=item C<< paths => \@paths >>

(Optional, Applicable only for Class Method)
An array of file paths to be deleted.

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'nodescope-node-name' => $node_name >>

(Applicable only for Nodescope-CLI)
Explicitly specify the node on which to run the Nodescope command (the APISet
defaults to running on node "local")

=item C<< determine_appropriate_nodescope_node => 0 | 1 >>

(Optional, defaults to 0, applicable only for Nodescope-CLI)
If set to 1, the method determines which node the volume hosting the file
is present on and runs the Nodescope CLI command on that particular node.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;

    $pkg_or_obj->_delete_front_end_helper( @args, _is_vol_file => 0 );

    $Log->exit();
}

sub _delete_7mode_cli {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};
    my $ci     = $opts{command_interface};
    my @paths  = @{ $opts{paths} };

    my %api_args;
    if ( $ci->is_cmode() ) {

        # Nodescope
        $pkg_or_obj->_hash_copy_defined(
            source => \%opts,
            target => \%api_args,
            copy   => [qw(nodescope-node-name)]
        );

        if ( @paths == 1 ) {

            # The common case: Only a single path provided
            $api_args{paths} = $paths[0];
            my $hash =
              $pkg_or_obj->_get_node_and_name_ordinal( %opts,
                path => $paths[0] );
            if ( defined $hash->{'nodescope-node-name'} ) {
                $api_args{'nodescope-node-name'} =
                  $hash->{'nodescope-node-name'};
            }
        }
        else {
            # More than a single path provided
            if ( $opts{determine_appropriate_nodescope_node}
                && !$opts{'nodescope-node-name'} )
            {

                my %aggr_hash;
                foreach my $path (@paths) {
                    my $output = $pkg_or_obj->_extract_aggregate_file_from_path(
                        path => $path );
                    my $aggregate = $output->{aggr_name};
                    if ( exists $aggr_hash{$aggregate} ) {
                        push @{ $aggr_hash{$aggregate}{path} }, $path;
                    }
                    else {
                        $aggr_hash{$aggregate} = { path => [$path] };
                    }
                }

                # Remove the multiple files one at a time associated with each aggregate
                foreach my $aggregate ( keys %aggr_hash ) {
                    my $hash =
                      $pkg_or_obj->_get_node_and_name_ordinal( %opts,
                        path => $aggr_hash{$aggregate}{path}[0]
                      );

                    my %local_api_args;
                    $local_api_args{paths} = join ' ',
                      @{ $aggr_hash{$aggregate}{path} };
                    $local_api_args{'nodescope-node-name'} =
                      $hash->{'nodescope-node-name'};
                    $apiset->rm(%local_api_args);
                }

                $Log->exit();
                return;
            }
            else {
                $api_args{paths} = join ' ', @paths;
            }
        }
    }
    else {
        # 7Mode
        $api_args{paths} = join ' ', @paths;
    }

    $apiset->rm(%api_args);
    $Log->exit();
}

=head2 get_local_node_name

    my $node_name = NACL::C::AggregateFile->get_local_node_name(
        command_interface => $ci,
        path              => $file_path,
        ...
    );

    or

    $file_obj->get_local_node_name();

(Applicable only for CMode)
Determines the node on which the aggregate file is present and returns the node
name. This is necessary for Nodescope operations, since they need to
be performed in the context of the node where the aggregate is present.

=cut

sub get_local_node_name {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $output =
      $pkg_or_obj->_extract_aggregate_file_from_path( path => $opts{path} );

    my %common_options;
    $pkg_or_obj->_copy_common_component_params_with_ci(
        source => \%opts,
        target => \%common_options
    );

    my $local_node =
      NACL::C::StorageAggregate->get_local_node_name( %common_options,
        aggregate => $output->{aggr_name} );
    $Log->exit();
    return $local_node;
}

=head2 get_local_node_obj

    my $node_obj = $file_obj->get_local_node_obj();

Similar to C<get_local_node_name>, but returns a C<NACL::STask::Node>
object for the node hosting the aggregate file.

=cut

# Implementation comes from the base-class

=head2 dumpblock_info

    my $dumpblock_cs_obj =   NACL::C::AggregateFile->dumpblock_info(command_interface => $ci,...);

(Class method only) This method executes 'dumpblock info' DBlade command and returns the
information returned by the command in
L<NACL::CS::AggregateFile|lib-NACL-CS-AggregateFile-pm> object.

Supports 7Mode CLI and Nodescope CLI (for CMode).

Uses a 7Mode/Nodescope CLI APIset.

Invokes "dumpblock info" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< file => $file_path >>

(Required) File Path

Applicable for 7Mode CLI/Nodescope .

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

Applicable for 7Mode CLI/Nodescope .

=item C<< determine_appropriate_nodescope_node => 0|1 >>

(Optional, defaults to 0)
When set to 1, the method first determines which node the file is present
and makes the call in the context of that node. Note that when
C<nodescope-node-name> overrides this argument, meaning that if both
C<nodescope-node-name> is provided and C<determine_appropriate_nodescope_node> is
set to 1 then the node name provided in C<nodescope-node-name> is used.

This argument defaults to 0.

Applicable for 7Mode CLI/Nodescope.

=item Command options

All of the options listed under "PROGRAMMATIC NAME" on the
L<dumpblock|lib-NACL-APISet-Node-CLI-7Mode-default-dumpblock-pm--dumpblock_info>
page are accepted by this method.

Applicable for 7Mode CLI/Nodescope.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::AggregateFile::BlockIsAHole>

This exception is thrown when an attempt is made to get the block information
for a block that is a hole.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


=head2 showfh

    my $showfh_cs_obj = NACL::C::AggregateFile->showfh(
        command_interface => $ci,
        path              => $file_path,
        ...
    );

                          or

    my $showfh_cs_obj = $file_obj->showfh(...);

(Class or instance method) This method executes 'showfh' DBlade command and returns the
information returned by the command in
L<NACL::CS::AggregateFileShowfh|lib-NACL-CS-AggregateFileShowfh-pm> object.

Supports 7Mode CLI and Nodescope CLI (for CMode).

Uses a 7Mode/Nodescope CLI APIset.

Invokes "showfh" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< path => $file_path >>

(Required) File Path

Applicable for 7Mode CLI/Nodescope .

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

Applicable for 7Mode CLI/Nodescope .

=item C<< determine_appropriate_nodescope_node => 0|1 >>

(Optional, defaults to 0)
When set to 1, the method first determines which node the file is present
and makes the call in the context of that node. Note that when
C<nodescope-node-name> overrides this argument, meaning that if both
C<nodescope-node-name> is provided and C<determine_appropriate_nodescope_node> is
set to 1 then the node name provided in C<nodescope-node-name> is used.

This argument defaults to 0.

Applicable for 7Mode CLI/Nodescope.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

=head2 testsetattr

Execute testsetattr on the file, without having to depend on apiset and path names
Only works for debug-build.

    my $file = NACL::STask::AggregateFile->create(...);
    my $file_attributes = $file->testsetattr('get-attributes' => 1);
    $file_offset_length = $file->testsetattr('read-offset-and-length' => '0 100');
    $file->testsetattr('write-offset-and-length' => '0 100');

Use it as a pass through to apiset testsetattr.

=cut

 __PACKAGE__->_nodescope_builder(command => 'testsetattr');


=head2 filersio
    filersio support for AggregateFile
=cut

__PACKAGE__->_nodescope_builder(
    command     => 'filersio_asyncio_active',
    map         => { path => 'filename' },
);

__PACKAGE__->_nodescope_builder(
    command     => 'filersio_asyncio_pause',
    map         => { path => 'filename' },
);

__PACKAGE__->_nodescope_builder(command => 'filersio_status');
__PACKAGE__->_nodescope_builder(command => 'filersio_stop');
__PACKAGE__->_nodescope_builder(command => 'filersio_spc1');

=head2 wafl_showvbns

Executes the wafl showvbns command on the file without needing to pass path
and command interface

    my $file = $Aggr->file_create(...);
    $file->wafl_showvbns(...);

This accepts all input accepts by wafl_showvbns apiset

=cut

__PACKAGE__->_cs_call_method_builder(method => 'wafl_showvbns', extra_filters => ['path']);

=head2 hammer

Executes the hammer command on the file without needing to pass filename
and command interface

    my $file = $Aggr->file_create(...);
    $file->hammer(...);

This accepts all input accepts by hammer apiset

=cut

__PACKAGE__->_nodescope_builder(
    command => 'hammer',
    map     => { path => 'file-name' },
);

sub call_on_apiset {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my @ret = $pkg_or_obj->_call_on_apiset(@args);
    $Log->exit();
    return wantarray ? @ret : $ret[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ( path => { type => SCALAR }, );
}

sub _get_node_and_name_ordinal {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params      => \@args,
        allow_extra => 1,
        ignore_primary_keys => 1
    );
    if (  !$opts{'nodescope-node-name'}
        && $opts{determine_appropriate_nodescope_node} )
    {

        my %primary_keys;
        $pkg_or_obj->_copy_primary_keys(
            source => \%opts,
            target => \%primary_keys
        );

        my %common_options;
        $pkg_or_obj->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_options
        );

        my $nodescope_node;
        if (defined $opts{path}) {
            $nodescope_node =
              $pkg_or_obj->get_local_node_name( %common_options, %primary_keys );
        } elsif (defined $opts{aggregate}) {
            $nodescope_node =
                NACL::C::StorageAggregate->get_local_node_name(
                    %common_options,
                    aggregate => $opts{aggregate}
                );
        }

        $Log->exit();
        return { 'nodescope-node-name' => $nodescope_node };
    }

    $Log->exit();
    return {};

}

# Helper method to extract the aggregate and file name
# from the filepath
sub _extract_aggregate_file_from_path {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    state $spec = { path => { type => SCALAR } };
    my %opts = validate_with(
        params => \@args,
        spec   => $spec,
    );
    my $path = $opts{path};

    # Possible for the path to be completely quoted. Example:
    # "/aggr/aggrname/file with spaces". The subsequent regex checks
    # if the path begins with /vol so remove the leading quote, if it exists.
    # Also validate that if there's a leading quote that there's a trailing
    # quote.
    if ( $path =~ s/^"// ) {
        if ( $path !~ /"$/ ) {
            $Log->exit();
            NATE::Exceptions::Argument->throw( "The path provided ('\"$path') "
                  . 'is invalid (it has a double quote at the beginning but '
                  . 'not at the end)' );
        }
    }
    my $output;
    if ( $path =~ m!/aggr/(.*?)/(.*)! ) {
        my ( $aggr_name, $file_name ) = ( $1, $2 );

        if ( $file_name =~ /"$/ ) {
            if ( $file_name !~ /^"/ ) {
                $file_name = '"' . $file_name;
            }
        }
        elsif ( $file_name =~ /[;#\s]/ ) {

            # If the filename has the special characters # or ; or spaces, then
            # it needs to be quoted.
            $file_name = '"' . $file_name . '"';
        }

        $output->{file_name} = $file_name;
        $output->{aggr_name} = $aggr_name;

    }
    else {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            "The path ('$path') is not in the expected format");
    }
    $Log->exit();
    return $output;
}

# CMode path should be completely quoted if it has
# special characters/spaces. (i.e. should be like
# "/aggr/aggrname/file name")

sub _convert_path_to_cmode_format {
    $Log->enter();
    my ($pkg_or_obj) = shift;
    state $spec = { path => { type => SCALAR } };
    my %opts = validate_with( params => \@_, spec => $spec );

    my $output =
      $pkg_or_obj->_extract_aggregate_file_from_path( path => $opts{path} );
    my $file = $output->{file_name};

    my $path = "/aggr/$output->{aggr_name}";

    # If the file-name has quotes (like "file with spaces"), then
    # constructing the path like /aggr/aggrname/"file with spaces" would be
    # incorrect because it would not work on the NGSH. Instead, the
    # entire path should be constructed within quotes: i.e. should be like
    # "/aggr/aggrname/file with spaces".
    my $needs_quoting;
    if ( $file =~ s/^"(.*)"$/$1/ ) {
        $needs_quoting = 1;
    }
    $path .= "/$file";
    $path = '"' . $path . '"' if ($needs_quoting);

    $Log->exit();
    return $path;
}

1;
