#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Directory Operations Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::Directory

=head1 DESCRIPTION

This library is used to perform basic directory system level operations
on a client such as create, delete, open, read etc. and all these
operations would be performed on the local client (from which
the script is executed), so all of these method does not require
command_interface as parameters.

=head1 ATTRIBUTES

=head2 dir

(Required) Name of the directory to be created.

=head2 dir_handle

(Optional) Directory handle returned by open()

=cut

package NACL::C::Client::Directory;
use strict;
use warnings;

use Params::Validate qw(validate SCALAR BOOLEAN GLOBREF);
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use File::Basename qw(fileparse);
use File::Path qw(remove_tree make_path);
use base 'NACL::C::Component::Client';
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Exceptions::Client::Directory::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    scalar => 'dir',
    scalar => 'dir_handle',
];

sub init {
    $Log->enter() if $may_enter;
    my $self            = shift;
    my %validation_spec = $self->_primary_keys_validate_spec();
    validate( @_, \%validation_spec );
    $Log->exit() if $may_exit;
} ## end sub init

=head1 METHODS

=head2 create()

Create a directory(Class Method)

=over

=item Options

=over

=item C<< dir => scalar >>

(Required): Directory to be created and the directory
path is absolute or relative

=item C<< mode => scalar >>

(Optional) Permissions 'mode' to use for the
new directory. Default is '0777'.

=item C<< recursive => boolean >>

Optional. If '0', then an error is signaled unless
the parent directory to the new directory already
exists. If '1', then any 'missing' ancestor
directories of the new directory will be created
before creating the new directory. Default is '0'.

=item Returns

returns directory object containing dir name, on error throws exception

=item Example

	my $dir_obj = NACL::C::Client::Directory->create( dir => $Dir_name, );
    $Log->trace( "dir_status is " . Dumper($dir_obj) );

=back

=back

=over 

=item Exception

=item NACL::C::Exceptions::Client::Directory::AlreadyExists

This type of exception is thrown when an attempt is made to add an already
existing directory

=back

=cut

sub create {
    my $pkg = shift;
    $Log->enter() if $may_enter;

    my %opts = $pkg->_common_validate_with(
        params             => \@_,
        only_static_method => 1,
        additional_spec    => {
            recursive => { type => BOOLEAN, optional => 1, default => 0 },
            mode      => {
                type     => SCALAR,
                optional => 1,
                default  => 0777
            },
        }
    );

    try {
        _create_dir(%opts);
    } catch NATE::BaseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Client::Directory::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $pkg->new( dir => $opts{dir} );
} ## end sub create

#---------------------------------------------------------------------
# Internal function to create directory
#---------------------------------------------------------------------
sub _create_dir {
    my %opts = @_;
    my ( $mode, $dir_name, $recursive, $orig_umask );
    $Log->enter() if $may_enter;

    $dir_name  = $opts{dir};
    $recursive = $opts{recursive};

    # set the umask to '0' to allow user permissions
    $orig_umask = umask(0);

    # convert the mode string to OCTAL (if required)
    if ( defined $opts{mode} and $opts{mode} ne "" ) {
        if ( $opts{mode} =~ /^0/ ) {

            # convert perms to octal
            $mode = oct $opts{mode};
        } else {

            # assume perms are already octal
            $mode = $opts{mode};
        }
    } ## end if ( defined $opts{mode...})

    if ( $Log->may_debug() ) {
        $Log->debug( "Options are dir_name => $dir_name, recursive => "
                . "$recursive, mode => $mode" );
    }

    # Get rid off multiple '/'s
    $dir_name =~ s{(/|\\)+}{$1};

    # Get rid off trailing '/'
    $dir_name =~ s{(/|\\)$}{};

    if ( $dir_name =~ /^.:\\?\*?$/ or $dir_name =~ m{^/\*?$} ) {
        NATE::BaseException->throw("Not creating root directory $dir_name\n");
    }

    my ( undef, $parent, undef ) = fileparse($dir_name);

    # Get rid off trailing '/'
    $parent =~ s{/$}{};

    if ( -e $parent && -d $parent ) {
        $Log->exit() if $may_exit;
        mkdir( $dir_name, $mode )
            or NATE::BaseException->throw(
            "Not able to create directory $dir_name: $!");
    } else {
        if ($recursive) {
            make_path( $opts{dir}, { mode => $mode } )
                or NATE::BaseException->throw(
                "Not able to create the directory $dir_name: $!");
        } else {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("$parent does not exist");
        }
    } ## end else [ if ( -e $parent && -d ...)]

    # Reset the umask to original value
    umask($orig_umask);

    $Log->exit() if $may_exit;
    return 1;
} ## end sub _create_dir

=head2 delete()

Delete a directory(Class or Instance Method)

=over

=item Options

=item C<< dir => scalar >>

 (Required) Directory to be created.
 Pass full path of directory in case of recursive is 1

=item recursive

  Optional. If '0', then only the specified
  directory is removed. If '1' then the directory,
  and all its descendents are also deleted.
  Default is '0'.

=item C<< strict => boolean >>

  Optional. If '0', then any errors generated during
  a recursive remove will be ignored. If '1', then
  a NATE::BaseException will be raised (in the event
  of a failure). Default is '0'.

=item Returns

 Nothing

=item Example

	my $dir_obj  = NACL::C::Client::Directory->create( dir => $Dir_name );
	$dir_obj->delete();

=back

=cut

sub delete {
    my $pkg_or_obj = shift;
    $Log->enter() if $may_enter;

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            recursive => { type => BOOLEAN, optional => 1, default => 0 },
	    strict    => { type => BOOLEAN, optional => 1, default => 0 },
        }
    );

    _delete_dir(%opts);
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_dir {
    my %opts = @_;
    $Log->enter() if $may_enter;
    my ( $dir_name,  $recursive, $strict, $err );
    $dir_name  = $opts{dir};
    $recursive = $opts{recursive};
    $strict    = $opts{strict};
    $Log->trace("dir_name is $dir_name");

    if ( !defined($dir_name) || $dir_name eq "" ) {
        NATE::Exceptions::Argument->throw("Dir is empty string or undefined");
    }

    # Get rid off multiple '/'s
    $dir_name =~ s{/+}{/};

    # Get rid off trailing '/'
    $dir_name =~ s{/$}{};

    # Do not attempt to remove root directory
    if ( $dir_name =~ /^.:\\?\*?$/ or $dir_name =~ m{^/\*?$} ) {
        NATE::BaseException->throw("Not removing root directory $dir_name");
    }

    if ($recursive) {
        $Log->trace("DIR to delete is $dir_name");
        remove_tree($dir_name, {error => \$err});

	# Check for errors
	if ($strict && (@{$err})) {
	    my $error_message = "";
	    foreach my $diag (@{$err}) {
		my ($file, $message) = %{$diag};
		if ($file eq '') {
		    $error_message .= "General error: $message\n";
		} else {
		    $error_message .= "Error unlinking $file: $message\n";
		}
	    }

	    # Raise an exception if an error was encountered
	    if ($error_message ne "") {
		NATE::BaseException->throw(
		    "Errors encountered during recursive remove:\n"
		    . $error_message
		);
	    }

	} elsif ($strict) {
	    $Log->trace("No errors encountered");
	}

    } else {
        rmdir($dir_name)
            or NATE::BaseException->throw(
            "Not able to delete the directory $dir_name: $!");
    }

    $Log->exit() if $may_exit;
} ## end sub _delete_dir

=head2 open()

Open a directory(Class or Instance Method)

=over

=item Options

=item C<< dir => scalar >>

 (Required) Directory for which handle needs to be opened

=item Returns

Returns an object of type NACL::C::Client::Directory which has dir_handle and
directory name stored as its attributes

=back

=cut

sub open {
    my $pkg_or_obj = shift;
    $Log->enter() if $may_enter;
    my ( $dir_name, $dir_handle );
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );
    $dir_name = $opts{dir};
    $dir_handle = $opts{dir_handle} if ( defined $opts{dir_handle} );

    # Open the directory
    opendir( $dir_handle, $dir_name )
        or NATE::BaseException->throw(
        "Not able to open the directory $dir_name: $!");
    $Log->exit() if $may_exit;

    if (ref($pkg_or_obj)) {
       $pkg_or_obj->dir_handle($dir_handle);
       return $pkg_or_obj;
    } else {
       return $pkg_or_obj->new( dir_handle => $dir_handle, dir => $dir_name );
    }
} ## end sub open

=head2 read()

Read a directory(Instance or Class Method)

=over

=item Options

=item C<< dir_handle => $dh >>

 (Required for Class Method) Directory handle returned by opendir().

=item Returns

If used in list context, returns all the rest of the entries in the directory.
If there are no more entries, returns the undefined valuein list context.

=item Example

	my $dir_obj = NACL::C::Client::Directory->open( dir => $Parent_dir, );
	my @content = $dir_obj->read();

=back

=cut

sub read {
    my $pkg_or_obj = shift;
    $Log->enter() if $may_enter;
    my ( $dir_handle, @files );
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_ );
    $dir_handle = $opts{dir_handle};

    # Read the directory
    @files = readdir($dir_handle)
        or NATE::BaseException->throw("Not able to read the directory: $!");
    $Log->exit() if $may_exit;
    return @files;
} ## end sub read

=head2 close()

Close a directory(Instance or Class Method)

=over

=item Options

=item C<< dir_handle => $dh >>

 (Required for Class Method): Directory handle returned by opendir().

=item Returns

Nothing

=item Example

	my $dir_obj = NACL::C::Client::Directory->open( dir => $Dir_name, );
	$dir_obj->close();

=back

=cut

sub close {
    my $pkg_or_obj = shift;
    $Log->enter() if $may_enter;
    my $dir_handle;
    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );
    $dir_handle = $opts{dir_handle};

    # Close the directory
    closedir($dir_handle)
        or NATE::BaseException->throw("Not able to close the directory: $!");
    $Log->exit() if $may_exit;
} ## end sub close

=head2 rename()

Rename a directory(Class or Instance Method)

=over

=item Options

=item C<< dir => scalar >>

 (Required): Directory to be rename.

=item C<< new_dir => scalar >>

 (Required): New name of Directory.

=item Returns

Nothing

=item Example

     NACL::C::Client::Directory->rename(
        dir     => $Dir_name,
        new_dir => $New_dir
    );

=back

=cut

sub rename {
    my $pkg_or_obj = shift;
    $Log->enter() if $may_enter;
    my ( $dir_name, $new_dir_name );
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { new_dir => { type => SCALAR } }
    );
    $dir_name     = $opts{dir};
    $new_dir_name = $opts{new_dir};
    my ( $dir1, $parent1, undef ) = fileparse($dir_name);
    my ( $dir2, $parent2, undef ) = fileparse($new_dir_name);
    $Log->trace("PARENT1 = $parent1, PARENT2 = $parent2");
    $Log->trace("DIR1 = $dir1, DIR2 = $dir2");

    # Rename the source directory to target
    rename( $dir_name, $new_dir_name )
        or NATE::BaseException->throw(
        "Not able to rename the directory $dir_name: $!");

    $Log->exit() if $may_exit;
} ## end sub rename

sub _primary_keys_validate_spec {
    return (
        dir        => { type => SCALAR },
        dir_handle => { type => GLOBREF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

# The same as the base class's version, but we need "no_command_interface"
# globally set for all methods.
sub _common_validate_with {
    $Log->exit() if $may_exit;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->SUPER::_common_validate_with( @_,
        no_command_interface => 1 );
    $Log->exit() if $may_exit;
    return %opts;
} ## end sub _common_validate_with

1;

