# Copyright (c) 2011-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary FileDirOps Component Module
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Client::FileDirOps

=head1 DESCRIPTION

C<NACL::C::Client::FileDirOps> Is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents all file operations.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the FileDirOps
element are the attributes of the FileDirOps ComponentState.

=over

=back

=head2 path

Path to file/dir to get/set its permissions.

=head2 command_interface

See L<NACL::C::Client::command_interface|lib-NACL-C-Client-pm/command_interface>

=cut

###################
# Package
package NACL::C::Client::FileDirOps;

###################
# Everytime
use strict;
use warnings;

###################
# Module includes
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Exceptions::ResponseException qw/:try/;
use NACL::C::Exceptions::Client::PermissionDenied;
use NACL::UnitTest::ParserTester;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Class::MethodMaker [
    scalar => 'path',
];

=head1 METHODS

=head2 new

    my $state = NACL::C::Client::FileDirOps->new( command_interface => $ci );

Return a new file component object with the given attributes.

=cut

=head2 chmod

    NACL::C::Client::FileDirOps->chmod(
        command_interface=>$ci,
        path => $FilePath,
    );

(Class OR Instance method)
This method is used to set file/dir permission.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path to file or dir to list its contents.

=item C<< permission => $string >>

(Optional) Permission to set for file/dir.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub chmod {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    recursive verbose mode
                    save substitute restore
                    setowner findsid verify reset grant deny
                    remove setintegritylevel inheritance
                    inherit-integrity level policy user sid
                    sid-perm old-sid aclfile
                    continue-on-err use-simlink quiet permission
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $file_path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_icacls_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_chmod_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $file_path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _icacls_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $map = { path => "pattern" };
    my $api = 'icacls';

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

sub _chmod_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'chmod';
    my $map = { path => "paths" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 create_dir

    NACL::C::Client::FileDirOps->create_dir(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to create a directory.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path to directory to create.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub create_dir {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    mode make-parents verbose
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_mkdir_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_mkdir_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    my $ret = $pkg->new(
        command_interface => $ci,
        path => $path,
    );
    
    return $ret;
}

sub _mkdir_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'mkdir';

    $pkg->_base_backend_client(
        @args,
        api => $api,
    );

    $Log->exit() if $may_exit;
}

sub _mkdir_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'mkdir';
    my $map = { path => "paths" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 delete_dir

    NACL::C::Client::FileDirOps->delete_dir(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to delete a directory.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path of directory to delete.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub delete_dir {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    recursive quiet-mode ignore_fail_on_non_empty
                    parents verbose
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $file_path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_rmdir_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_rmdir_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $file_path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _rmdir_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'rmdir';

    $pkg->_base_backend_client(
        @args,
        api => $api,
    );

    $Log->exit() if $may_exit;
}

sub _rmdir_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'rmdir';
    my $map = { path => "paths" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 create_file

    NACL::C::Client::FileDirOps->create_file(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to create a file.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path to file to create.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub create_file {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    ascii-file binary-file destination-decryption 
                    verify short-filename
                    cause-prompt network-file access_time no_create
                    modification_time reference stamp
                    set_date help version time no-dereference 
                    adjust-access-time symbolic-link force
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_create_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_touch_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    my $ret = $pkg->new(
        command_interface => $ci,
        path => $path,
    );
    
    return $ret;
}

sub _create_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'copy';
    my $map = { path => "destination" };
    my %other_opts = (
        'suppress-prompt' => 1,
        source            => 'NUL',
    );
    
    $pkg->_base_backend_client(
        @args,
        %other_opts,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

sub _touch_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'touch';

    $pkg->_base_backend_client(
        @args,
        api => $api,
    );

    $Log->exit() if $may_exit;
}

=head2 copy

    NACL::C::Client::FileDirOps->copy(
        command_interface => $ci,
        path              => $Path,
        dest              => $dest_path
    );
    
OR

    $file_ops->copy(dest => $dest_path);

(Class OR Instance method)
This method is used to copy a file from path to dest.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Source file path.

=item C<< dest => $string >>

(Required) Destination file path.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub copy {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    ascii-file binary-file destination-decryption
                    verify short-filename dest network-file
                    recursive archive force
                    interactive link symbolic-link verbose
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $path = $opts{path};
    my $dest = $opts{dest};

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_copy_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_cp_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    my $ret = $pkg->new(
        command_interface => $ci,
        path => $dest,
    );
    
    return $ret;
}

sub _copy_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'copy';
    my $map = { 
        path => "source",
        dest => "destination",
        interactive => "cause-prompt",
    };

    my %other_opts = (
        'suppress-prompt' => 1,
    );
    
    $pkg->_base_backend_client(
        @args,
        %other_opts,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

sub _cp_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'cp';
    my $map = { path => "source" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 move

    NACL::C::Client::FileDirOps->move(
        command_interface => $ci,
        path              => $Path,
    );
    
OR

    $file_ops->copy(path => $Path);

(Class OR Instance method)
This method is move file from path to target location.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< path => $string >>

(Required) Source file path.

=item C<< target => $string >>

(Required) Destination file path.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub move {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    specific-backup normal-backup force interactive 
                    no-clobber strip-trailing-slashes suffix
                    override-backup-suffix target-directory no-target-directory
                    update verbose help version target
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $path = $opts{path};
    my $dest = $opts{target};

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_mv_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    $pkg = $pkg->new(
        command_interface => $ci,
        path => $dest,
    );
    
    return $pkg;
}

sub _mv_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'mv';
    my $map = { path => "source" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 delete_file

    NACL::C::Client::FileDirOps->delete_file(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to delete a file.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) File path.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub delete_file {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (
                    delete-specified-file quiet-mode
                    force interactive recursive
                    prompt-once verbose attributes
                )
            ),
        },
    );

    my $ci = $opts{command_interface};
    my $file_path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_del_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_rm_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::PermissionDenied->detect_convert_and_throw(
            exception => $exception,
            client_ci => $ci,
            file_path => $file_path,
        );

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _del_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'del';
    my $map = {
        path => "files",
        interactive => "cause-prompt",
        force => "force-delete",
    };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

sub _rm_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'rm';
    my $map = { path => "paths" };

    $pkg->_base_backend_client(
        @args,
        api => $api,
        map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 read_test

    NACL::C::Client::FileDirOps->read_test(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to test the read permissions to the directory or file.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path to test the read permission.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub read_test{
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            path    => { type => SCALAR },
        },
    );
    my $ci = $opts{command_interface};
    my $path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_read_test_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_read_test_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::Client::FileDirOps::ReadTestFailed->convert_and_throw(
            exception => $exception
        );
		$Log->exit() if $may_exit;
    };

    $Log->exit() if $may_exit;
    my $ret = $pkg->new(
        command_interface => $ci,
        path => $path,
    );

    return $ret;
}

sub _read_test_windows_cli {
    my ($pkg, @args) = @_;
    $Log->enter();

    my $api = 'dir';

    $pkg->_base_backend_client(
        @args,
        api => $api,
    );

    $Log->exit();
}

sub _read_test_linux_cli {
    my ($pkg, %args) = @_;
    $Log->enter() if $may_enter;

    my $api = 'ls';
    my $map = { path => "paths" };

    $pkg->_base_backend_client(
        %args,
        api => $api,
	map => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 write_test

    NACL::C::Client::FileDirOps->write_test(
        command_interface => $ci,
        path              => $Path,
    );

(Class OR Instance method)
This method is used to test the write permission to a directory or a file.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $string >>

(Required) Path to test the write operations on a file or directory.

=item C<< Options accepted by APISet >>

(Optional) It accepts all the options accepted by respective apisets
L<NACL::APISet|lib-NACL-APISet-pm>

=back

=back

=cut

sub write_test{
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            path    => { type => SCALAR },
        },
    );
    my $ci = $opts{command_interface};
    my $path = $opts{path};
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_write_test_windows_cli",
                    interface => "CLI",
                    set       => "Windows"
                },
                {   method    => "_write_test_linux_cli",
                    interface => "CLI",
                    set       => "Linux|Solaris"
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
	$Log->exit() if $may_exit;
        NACL::C::Exceptions::Client::FileDirOps::WriteTestFailed->convert_and_throw(
            exception => $exception
        );
    };

    $Log->exit() if $may_exit;
    my $ret = $pkg->new(
        command_interface => $ci,
        path => $path,
    );

    return $ret;
}

sub _write_test_windows_cli {
    my ($pkg, %opts) = @_;
    $Log->enter();

    my $api = 'echo';
    my $testFile = 'test_share_'. $opts{'command_interface'}->hostrec->hostname() . '_'.time();

    my $message = 'Hello > '.$opts{path}.'\\'.$testFile;
    try{
	$pkg->_base_backend_client(
            %opts,
	    api => $api,
	    message => $message
	);
	## verifying the creation of the new file
	$pkg->_base_backend_client(
             %opts,
	     api => 'dir',
	     path => $opts{path}.'\\'.$testFile
	);
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
	if($exception->text() =~ /No space left on device|There is not enough space|invalid|incorrect/){
	   $Log->comment('Ignoring the error due to space issue');
	}else{
	   $exception->throw();
        }
    };

    ## deleting the new file created
    $pkg->_base_backend_client(
        %opts,
	api => 'del',
	files => $opts{path}.'\\'.$testFile,
	'force-delete' => 1
    );

    $Log->exit();
}

sub _write_test_linux_cli {
    my ($pkg, %opts) = @_;
    $Log->enter() if $may_enter;

    my $api = 'echo';
    my $map = { path => "paths" };

    my $testFile = 'test_share_'. $opts{'command_interface'}->hostrec->hostname() . '_'.time();

    my $message = 'Hello > '.$opts{path}.'/'.$testFile;
    try{
        $pkg->_base_backend_client(
            %opts,
            api => $api,
            message => $message,
	    map => $map,
        );
        ## verifying the creation of the new file
        $pkg->_base_backend_client(
             %opts,
             api => 'ls',
             paths => $opts{path}.'/'.$testFile
        );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        if($exception->text() =~ /No space left on device|There is not enough space|invalid|incorrect/){
           $Log->comment('Ignoring the error due to space issue');
        }else{
           $exception->throw();
        }
    };

    ## deleting the new file created
    $pkg->_base_backend_client(
        %opts,
        api => 'rm',
        paths => $opts{path}.'/'.$testFile,
        'force-delete' => 1
    );

    $Log->exit() if $may_exit;
}

sub _primary_keys_validate_spec {
    return (
        path => { type => SCALAR},
    );
}

1;
