#
# Copyright (c) 2012-13 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Iscsi Component Module
## @author Vishwanath.Rawat@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Client::Iscsi

=head1 DESCRIPTION

C<NACL::C::Client::Iscsi> Is a derived class of
L<NACL::C::Component::Client|lib-NACL-C-Component-pm>. It represents access to
a Iscsi on a Client Machine.

=head1 ATTRIBUTES

=head2 device

Name of the device or hardisk partition on which you want to make filesystem

=head2 targets

List of the Targets objects for Client to discover through iscsiadm discovery.

=head2 config_iscsi_discovery_type

It specifies what type of discovery method to use.

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

###################
# Package
package NACL::C::Client::Iscsi;

###################

use strict;
use warnings;
use Net::IP qw(ip_is_ipv6);

###################
# Module includes
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with ARRAYREF HASHREF SCALAR BOOLEAN OBJECT);
use NACL::APISet::Exceptions::ResponseException qw/:try/;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use constant ISCSI_PORT => 3260;

use Class::MethodMaker [
    array  => 'targets',
    scalar => 'config_iscsi_discovery_type',
];

=head1 configure
    my $iscsi = NACL::C::Client::Iscsi->configure(
        command_interface => $ci,
        targets => \@targets,
        config_iscsi_discovery_type => $config_iscsi_discovery_type,
    );

(Class method) Request to configure Iscsi on Client and return a new component object that refers to it.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< targets => @array >>

(Required) List of the Targets objects to discover through iscsiadm discovery.

Each target is object of type L<NACL::C::VserverIscsiInterface|lib-NACL-C-VserverIscsiInterface-pm>

=item C<< config_iscsi_version => $string >>

(Required) Iscsi version that need to be configured on Client

Valid values are: open-iscsi, linuxiscsi

=item C<< config_iscsi_discovery_type => $string >>

(Required) It specifies what type of discovery method to use.

Valid values are: sendtargets, static, isns

=item C<< config_iscsi_chap_mode => $string >>

(Optional) Type of CHAP to use.

Valid values are: unidirectional, bidirectional

=item C<< config_iscsi_chap_in_username => $string >>

(Optional) In-bound username to use for CHAP authentication during target discovery.

=item C<< config_iscsi_chap_out_username => $string >>

(Optional) Out-bound username to use for CHAP authentication during target discovery.

=item C<< config_iscsi_chap_in_password => $string >>

(Optional) In-bound password to use for CHAP authentication during target discovery.

=item C<< config_iscsi_chap_out_password => $string >>

(Optional) Out-bound username to use for CHAP authentication during target discovery.

=item C<< config_iscsi_data_digest => $boolean >>

(Optional) Specifies whether to use data digest.

=item C<< config_iscsi_header_digest => $boolean >>

(Optional) Specifies whether to use header digest.

=item C<< config_iscsi_isns_server => $string >>

(Optional) Name of the isns server. It is needed when
config_iscsi_discovery_type is of type isns

=item C<< mp_enable => $boolean >>

(Optional) Tells whether Multipathing is enabled or not

default: 1 (enabled)

=item C<< mp_type => $string >>

(Optional) Tells what type of multipathing is used

default: "dmmp"

=item C<< sanbooted => $boolean >>

(Optional) Tells whether host is Sanbooted or not

default: 0

=back

=back

=cut

sub configure {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            mp_type              => { type => SCALAR, default => "dmmp" },
            mp_enable            => { type => BOOLEAN, default => 1 },
            sanbooted            => { type => SCALAR, default => 0 },
            config_iscsi_version => { type => SCALAR, default => "open-iscsi" },
            alua_support         => { type => SCALAR, default => 1 },
            _optional_scalars(
                qw(
                    config_iscsi_chap_mode config_iscsi_chap_in_username 
                    config_iscsi_chap_out_username config_iscsi_chap_in_password 
                    config_iscsi_chap_out_password config_iscsi_header_digest 
                    config_iscsi_data_digest config_iscsi_isns_server alua_support
                )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   
                method    => "_configure_iscsi_linux",
                interface => "CLI",
                set       => "Linux"
            },
            {   
                method    => "_configure_iscsi_solaris",
                interface => "CLI",
                set       => "Solaris"
            },
        ],
    );

    $Log->exit();
    return $ret;
} ## end sub configure

sub _configure_iscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $command_interface = $opts{command_interface};
    my $version = $opts{config_iscsi_version};
    if ($version =~ /open-iscsi/) {
        $pkg->_configure_openiscsi_linux(%opts);
    } else {
        $pkg->_configure_linuxiscsi_linux(%opts);
    }

    my $obj = $pkg->new(
       command_interface           => $command_interface,
       targets                     => $opts{targets},
       config_iscsi_discovery_type => $opts{config_iscsi_discovery_type},
    );

    $Log->exit();
    return $obj;
} ## end sub _configure_iscsi_linux

sub _configure_openiscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};

    # Configure iscsi config file (/etc/iscsi/iscsid.conf)
    $pkg->_configure_iscsi_conf_file_linux(%opts);

    my $discovery_type = $opts{config_iscsi_discovery_type};
    my $method = "_configure_openiscsi_" . $discovery_type . "_linux";

    $pkg->$method(%opts);

    # configure iscsi options for all the newly added targets
    $pkg->_configure_openiscsi_options_linux(%opts);

    # issue the logins on all the targets
    $pkg->_configure_openiscsi_login_sessions_linux(%opts);

    # issue a rescan on all sessions
    $pkg->_configure_openiscsi_rescan_sessions_linux(%opts);

    # modify the node.startup flag for all sessions of the targets we added to be automatic
    $pkg->_configure_openiscsi_auto_login_at_startup_linux(%opts);

    # Need to recreate the initrd image for sanbooted SuSE systems.
    # This is not required for RHEL and OL.
    $pkg->_recreate_initrd_image(apiset => $apiset, sanbooted => $opts{sanbooted});

    $Log->exit();
}


sub _configure_openiscsi_auto_login_at_startup_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset  = $opts{apiset};
    my @targets = @{$opts{targets}};

    # list out all the sessions
    my $resp = $apiset->iscsiadm(mode => 1, session => 1);
    my @output = split(/\n/, $resp->get_raw_output());

    # Exclude the sessions created by Hardware iscsi (Qlogic cards)
    @output = grep (!/^qla4/, @output);

    # modify the node.startup flag for all sessions of the targets we added to be automatic
    foreach my $target (@targets) {
        my $ip = $target->address();
        my $nodename = $target->target_name();
        my $tpgt = $target->tpgroup_tag();

        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        # if there is no session for this target, then skip.
        if (not grep(/^\s*${ip}\:ISCSI_PORT,${tpgt}\s+${nodename}/i, @output)) {
            next;
        }

        # modify this session's startup setting
        $apiset->iscsiadm(
            mode            => 1,
            node            => 1,
            "target-name"   => $nodename,
            "ip-port"       => "$ip:" . ISCSI_PORT,
            operation       => "update",
            name            => "node.startup",
            value           => "automatic",
        );
    }

    $Log->exit();
}

sub _configure_openiscsi_login_sessions_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my @targets = @{$opts{targets}};

    foreach my $target (@targets) {
        my $ip = $target->address();
        my $nodename = $target->target_name();
        my $tpgt = $target->tpgroup_tag();

        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        try {
            $apiset->iscsiadm(
                mode            => 1,
                node            => 1,
                login           => 1,
                "target-name"   => $nodename,
                "ip-port"       => "$ip:" . ISCSI_PORT . ",$tpgt"
            );
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $ex = shift;
            if ($ex->text() !~ /already exists/) {
                $ex->set_text("Failed to login to $nodename. " . $ex->text());
                $ex->throw();
            }
        }
    };

    $Log->exit();
}

sub _configure_openiscsi_rescan_sessions_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};

    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);

    # issue a rescan on all sessions
    # From RHEL 5.4 and SLES 10.3 onwards, we don't use iscsiadm to rescan iSCSI luns,
    # but instead we use /usr/bin/rescan-scsi-bus.sh.
    if ((($major =~ /sles10/i) and $minor < 3)
     or (($major =~ /rhel5/i) and $minor < 4)
     or ($apiset->rpm(query => 1, package => "sg3_utils"))) {
        $apiset->iscsiadm(mode => 1, session => 1, rescan => 1);
    } else {
        $pkg->_rescan_scsi_bus(protocol => "iscsi");
    }

    $Log->exit();
}

sub _configure_iscsi_conf_file_for_isns_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $isns_server = $opts{config_iscsi_isns_server};
    my $apiset = $opts{apiset};

    $pkg->_change_parameter_value_in_file(
        param   => "isns.address",
        value   => $isns_server,
        file    => "/etc/iscsi/iscsid.conf",
        apiset  => $apiset,
    );

    my $isns_port = 3205;
    $pkg->_change_parameter_value_in_file(
        param   => "isns.port",
        value   => $isns_port,
        file    => "/etc/iscsi/iscsid.conf",
        apiset  => $apiset,
    );

    $Log->exit();
}

sub _configure_iscsi_conf_file_for_chap_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $in_username  = $opts{config_iscsi_chap_in_username};
    my $in_password  = $opts{config_iscsi_chap_in_password};
    my $out_username = $opts{config_iscsi_chap_out_username};
    my $out_password = $opts{config_iscsi_chap_out_password};
    my $apiset = $opts{apiset};

    my %chap_optios = (
        file => "/etc/iscsi/iscsid.conf",
        apiset  => $apiset,
    );
    $pkg->_change_parameter_value_in_file(
        param   => "discovery.sendtargets.auth.authmethod",
        value   => "CHAP",
        %chap_optios,
    );

    $pkg->_change_parameter_value_in_file(
        param   => "discovery.sendtargets.auth.username",
        value   => $in_username,
        %chap_optios,
    );

    $pkg->_change_parameter_value_in_file(
        param   => "discovery.sendtargets.auth.password",
        value   => $in_password,
        %chap_optios,
    );

    # if doing bi directional, update the out credentials
    if ($opts{config_iscsi_chap_mode} =~ /bidirectional/i) {
        $pkg->_change_parameter_value_in_file(
            param   => "discovery.sendtargets.auth.username_in",
            value   => $out_username,
            %chap_optios,
        );

        $pkg->_change_parameter_value_in_file(
            param   => "discovery.sendtargets.auth.password_in",
            value   => $out_password,
            %chap_optios,
        );
    }

    $Log->exit();
}

sub _configure_iscsi_conf_file_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $chap_mode = $opts{config_iscsi_chap_mode};
    # first configure the discovery credentials if requested to do CHAP
    if ($chap_mode =~ /(uni|bi)directional/i) {
        $pkg->_configure_iscsi_conf_file_for_chap_linux(%opts);
    }

    # first add the isns server and isns port to /etc/iscsid.conf
    if ($opts{config_iscsi_discovery_type} =~ /isns/) {
        $pkg->_configure_iscsi_conf_file_for_isns_linux(%opts);
    }

    $Log->exit();
}

sub _configure_openiscsi_sendtargets_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};

    my @targets = @{$opts{targets}};

    # list out all the sendtargets currently on the system
    # this command will return a nonzero status when there are no targets in the list.
    # dont fail in this case.
    my (@list_output, $resp);
    try {
        $resp = $apiset->iscsiadm(mode => 1, discovery => 1);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if (length($resp->get_raw_output())) {
            $ex->set_text("Failed to discover targets. " . $ex->text());
            $ex->throw();
        }
    };

    # add each target with sendtargets
    foreach my $target (@targets) {
        my $ip = $target->address();

        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        #--> iscsiadm -m discovery
        #192.168.1.61:3260 via sendtargets
        #[fe80::a0:98ff:fe01:70aa]:3260 via sendtargets

        $Log->debug("iSCSI Configure: Adding SendTarget '$ip'");

        # add the target. It's ok if it is already addded, this will
        # force iscsiadm to rediscover the targets if they were removed at any point (burt494188)
        $ip = "$ip:" . ISCSI_PORT;
        $apiset->iscsiadm(mode => 1, discovery => 1, type => "sendtargets", 'ip-port' => $ip);
    }

    $Log->exit();
}

sub _configure_openiscsi_static_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my @targets = @{$opts{targets}};

    # list out all the static targets currently on the system
    my @list_output;
    try {
        my $resp = $apiset->iscsiadm(mode => 1, node => 1);
        @list_output = split(/\n/, $resp->get_raw_output());
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() !~ /no records found/) {
            $ex->set_text("Failed to login with error " . $ex->text());
            $ex->throw();
        }
    };

    # add each target manually
    foreach my $target (@targets) {
        my $ip = $target->address();
        my $nodename = $target->target_name();
        my $tpgt = $target->tpgroup_tag();

        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        #--> iscsiadm -m node
        #192.168.1.61:3260,2000 iqn.1992-08.com.netapp:sn.84174947
        #[fe80::a0:98ff:fe01:70aa]:3260,2000 iqn.1992-08.com.netapp:sn.84174947

        # skip if this target is already added.
        if (grep(/^\s*${ip}:ISCSI_PORT,${tpgt}\s+${nodename}/i, @list_output)) {
            $Log->comment("iSCSI Configure: Static target '$nodename,$ip,$tpgt' is already added");
            next;
        } else {
            $Log->comment("iSCSI Configure: Adding Static target '$nodename,$ip,$tpgt'");
        }

        # add the target as a static target
        my $ip_port = $ip . ":" . ISCSI_PORT . "," . $tpgt;
        $apiset->iscsiadm(
            mode          => 1,
            node          => 1,
            operation     => "new",
            'target-name' => $nodename,
            'ip-port'     => $ip_port,
        );
    }

    $Log->exit();
}

sub _configure_openiscsi_isns_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    # my @targets = @{$opts{targets}};

    # need to restart open iscsi in order for the isns server to get updated
    $pkg->restart(%opts);

    # then trigger the update from the isns server
    try {
        $apiset->iscsiadm(mode => 1, discovery => 1, type => "isns");
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        # If the error is 'please specify portal', try the command again
        # but with the newer options for RHEL 5u6 and RHEL 6
        my $isns_server = $opts{config_iscsi_isns_server};
        my $isns_port = 3205;
        my $exception = shift;
        if ($exception->text =~ /Please specify portal/i) {
            $apiset->iscsiadm(mode => 1, discovery => 1, type => "isns", 'ip-port' => "$isns_server:$isns_port");
        }
    };

    $Log->exit();
}

sub _configure_openiscsi_options_linux {

    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset  = $opts{apiset};
    my @targets = @{$opts{targets}};
    my ($out_username, $in_username, $out_password, $in_password, $chap_mode);
    $chap_mode = $opts{config_iscsi_chap_mode};
    if ($chap_mode =~ /(uni|bi)directional/i) {
        $out_username = $opts{config_iscsi_chap_out_username};
        $in_username = $opts{config_iscsi_chap_in_username};
        $out_password = $opts{config_iscsi_chap_out_password};
        $in_password = $opts{config_iscsi_chap_in_password};
    }

    # enable CHAP/HD/DD on all targets passed in if requested
    foreach my $target (@targets) {
        my $ip = $target->address();
        my $nodename = $target->target_name();
        my $tpgt = $target->tpgroup_tag();

        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        my %iscsi_options = (
            mode          => 1,
            node          => 1,
            "ip-port"     => "$ip:" . ISCSI_PORT . ",$tpgt",
            operation     => "update",
            "target-name" => $nodename,
        );

        # enable chap for this target if requested
        if ($chap_mode =~ /(uni|bi)directional/i) {

            # enable chap
            $apiset->iscsiadm(%iscsi_options,
                name      => "node.session.auth.authmethod",
                value     => "CHAP",
            );

            # set up the in username
            $apiset->iscsiadm(%iscsi_options,
                name      => "node.session.auth.username",
                value     => $in_username,
            );

            # set up the in password
            $apiset->iscsiadm(%iscsi_options,
                name      => "node.session.auth.password",
                value     => $in_password,
            );

            # if doing bidirectional, set up the out username & password (open iscsi refers as the 'in' credentials)
            if ($chap_mode =~ /bidirectional/i) {
                $apiset->iscsiadm(%iscsi_options,
                    name      => "node.session.auth.username_in",
                    value     => $out_username,
                );

                $apiset->iscsiadm(%iscsi_options,
                    name      => "node.session.auth.password_in",
                    value     => $out_password,
                );
            }
        }

        # enable header digest if requested
        if (exists $opts{config_iscsi_header_digest} and $opts{config_iscsi_header_digest} == 1) {
            $apiset->iscsiadm(%iscsi_options,
                name      => "node.conn[0].iscsi.HeaderDigest",
                value     => "CRC32C",
            );
        }

        # enable data digest if requested (not supported in redhat and oracle linux)
        my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);
        if (exists $opts{config_iscsi_data_digest} and $opts{config_iscsi_data_digest} == 1 and $major !~ /rhel|ol/i) {
            $apiset->iscsiadm(%iscsi_options,
                name      => "node.conn[0].iscsi.DataDigest",
                value     => "CRC32C",
            );
        }

        # if doing dmmp we need to modify the default setting of replacement_timeout from 120 to 5, all other
        # targets default to 120.
        my $replacement_timeout;
        my $mp_enable = $opts{mp_enable};
        my $mp_type   = $opts{mp_type};
        if ($mp_enable and $mp_type =~ /dmmp/i) {
            $replacement_timeout = 5;
        } else {
            $replacement_timeout = 120;
        }
        $apiset->iscsiadm(%iscsi_options,
            name  => "node.session.timeo.replacement_timeout",
            value => $replacement_timeout,
        );

    }

    $Log->exit();
}

sub _configure_linuxiscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %args = $pkg->_validate_backend_opts(@opts);

    my $apiset = $args{apiset};

    my $allowed_options = [qw( targets mp_type mp_enable )];
    my %opts;
    $pkg->_hash_copy(
        source => \%args,
        copy   => $allowed_options,
        target => \%opts,
    );

    my $mp_enable = $opts{mp_enable};
    my $mp_type = $opts{mp_type};
    my @targets = @{$opts{targets}};

    # Get OS type
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);

    # prepend DiscoveryAddress= in front of every ip address in targets
    # Add the required parameters to iscsi.conf
    my @iscsi_conf;
    my @iscsi_conf_extra_options;

    # Push options specific to rhel4 only
    if ($major =~ /rhel4/i) {
        push (@iscsi_conf_extra_options, 'ConnFailTimeout=5');
    }

    # Push options specific to sles9 only
    elsif ($major =~ /sles9/i) {
        push(@iscsi_conf_extra_options,'HeaderDigest=never', 'DataDigest=never', 'Continous=no'
        );

        # Push the following options only if mp is enabled and mp_type is dmmp
        if ( $mp_enable and $mp_type =~ /dmmp/i) {
            push(@iscsi_conf_extra_options, 'Multipath=portal', 'ConnFailTimeout=5');
        }
    }

    foreach my $target (@targets) {
        $target = "DiscoveryAddress=" .  $target->{ip};
        push (@iscsi_conf, $target);
    }
    push (@iscsi_conf,@iscsi_conf_extra_options);

    # These will be our sentinels for our modifications to the iscsi.conf file
    my $begin_sentinel = "#_#_ START iscsi.conf AUTOMATION ##";
    my $end_sentinel   = "#_#_ END iscsi.conf AUTOMATION ##";
    unshift(@iscsi_conf, $begin_sentinel);
    push(@iscsi_conf, $end_sentinel);

    $apiset->rm(paths => "/etc/iscsi.conf.tmp",force => 1);

    $apiset->touch(path => "/etc/iscsi.conf.tmp");

    # now back up the previous iscsi.conf file in this format,
    # iscsi.conf.bak.<timestamp>
    $apiset->cp(source => "/etc/iscsi.conf", dest => "/etc/iscsi.conf." . time());

    chomp(@iscsi_conf);

    # now write all the lines of elements into the iscsi.conf file on the
    # host.
    foreach my $line (@iscsi_conf) {
        $apiset->execute_raw_command(command => "echo \'$line\' >> /etc/iscsi.conf.tmp");
    }

    $apiset->mv(source => "/etc/iscsi.conf.tmp", target => "/etc/iscsi.conf", force => 1);

    $pkg->restart();

    $Log->exit();
}

sub _configure_iscsi_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $alua_support      = $opts{alua_support};
    my @targets           = @{$opts{targets}};
    
    # Get major minor version
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);
    
    # Need to remove "Solaris" part from $major revision
    # So that we can do Arithmetic comparison on $major
    $major =~ s/solaris//i;
    if ($major > 5.10 || ($major == 5.10 && $minor >= 9)) {
        # We only want to lower the conn-login-max to 60 for ALUA configs
        # since we assume that means Cluster-Mode which has secondary paths.
        # For 7-mode non-alua configs we don't have secondary paths so we don't
        # want the session to go offline so we need to make sure the conn-login-max
        # is set to the default of 180.
        my $conn_login_max = "";
        if ($alua_support) {
            $conn_login_max = "60";
        } else {
            $conn_login_max = "180";
        }

        # set the maximum retry timeout for 10U9 or later to 60.
        $apiset->iscsiadm_modify(
            'direct-object'  => "initiator-node",
            'tunable-param'  => 1,
            'conn-login-max' => $conn_login_max,
        );
    }
    
    my $hostname = $apiset->hostobj()->hostname();
    # set up the iscsi alias for this host
    $apiset->iscsiadm_modify(
        'direct-object' => "initiator-node",
        'node-alias'    => $hostname,
    );
    
    my $discovery_type = $opts{config_iscsi_discovery_type};
    my $method = "_configure_iscsi_" . $discovery_type . "_solaris";

    $pkg->$method(%opts);

    # at this point, perform some final validation to make sure that we have all
    # our targets properly set up
    my $resp = $apiset->iscsiadm_list('direct-object' => "target");
    my @output = split(/\n/, $resp->get_raw_output());

    foreach my $target (@targets) {
        my $nodename = $target->target_name();
        if (not grep(/Target:\s*${nodename}\s*$/, @output)) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "The target ($nodename) is not present in the initiator's target list"
            );
        }
    }
    
    $Log->comment("iSCSI Configure: iSCSI Configuration complete");
    
    my $obj = $pkg->new(
       command_interface           => $command_interface,
       targets                     => $opts{targets},
       config_iscsi_discovery_type => $opts{config_iscsi_discovery_type},
    );

    $Log->exit();
    return $obj;
} ## end sub _configure_iscsi_linux

sub _configure_iscsi_sendtargets_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    
    $Log->comment("iSCSI Configure: Configuring 'Sendtargets' Discovery");
    
    my $resp = $apiset->iscsiadm_list('direct-object' => "discovery-address");
    
    my @list_output = split(/\n/, $resp->get_raw_output());
    
    my @targets = @{$opts{targets}};
    
    foreach my $target (@targets) {
        my $dynamic_addr = $target->address();

        # if ipv6, modify the address to be in brackets
        if (ip_is_ipv6($dynamic_addr)) {
            $dynamic_addr = "[" . $dynamic_addr . "]";
        }

        # check if the discovery address for this target was already added.
        # If it was not added, then add. If it is, then skip.
        if (grep(/$dynamic_addr/i, @list_output) == 0) {
            $Log->comment("Adding iSCSI discovery (send-targets) address $dynamic_addr");
            $apiset->iscsiadm_add(
                'direct-object'     => "discovery-address",
                'discovery-address' => $dynamic_addr,
            );
            
        } else {
            $Log->comment("Skipping add of iSCSI discovery (send-targets) address $dynamic_addr (already exists)");
        }
    }
    
    
    # configure all iscsi options
    $pkg->_configure_iscsi_options_solaris(%opts);
    
    # enable sendtargets discovery
    $apiset->iscsiadm_modify(
        'direct-object' => "discovery",
        'sendtargets'   => "enable",
    );

    # list out the sendtargets addresses that were added
    $apiset->iscsiadm_list('direct-object' => "discovery-address");

    # display what discovery mechanism is enabled.
    $pkg->_verify_discovery_type_is_enabled_solaris(
        command_interface => $opts{command_interface},
        type              => "send_targets",
    );

    $Log->exit();
}

sub _verify_discovery_type_is_enabled_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            command_interface => { isa  => 'NACL::C::Client', type => OBJECT },
            type              => { type => SCALAR },
        },
    );
    
    my $type   = $opts{type};
    my $ci     = $opts{command_interface};
    my $CS_pkg = $pkg->get_CS_package_name();
    my $state  = $CS_pkg->fetch(mode => "discovery", command_interface => $ci);
    
    if ($state->$type() ne "enabled") {
        NACL::APISet::Exceptions::ResponseException->throw(
            text => "Discovery of type $type is not enabled"
        );
    }

    $Log->exit();
}

sub _configure_iscsi_static_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my @targets = @{$opts{targets}};
    
    $Log->comment("iSCSI Configure: Configuring 'Static' Discovery");
    
    my $resp = $apiset->iscsiadm_list('direct-object' => "static-config");
    
    my @list_output = split(/\n/, $resp->get_raw_output());

    foreach my $target (@targets) {
        # define static address for this target and check to see if
        # it was already added. If not added, add, if added, skip.
        my $nodename = $target->target_name();
        my $ip = $target->address();
        my $tpgt = $target->tpgroup_tag();
        
        # if the target ip is ipv6, then add brackets around it.
        if (ip_is_ipv6($ip)) {
            $ip = "[" . $ip . "]";
        }

        my $static_addr = $nodename . "," .
                          $ip . ":" . "3260" . "," .
                          $tpgt;
        if (grep(/$static_addr/i, @list_output) == 0) {
            $Log->comment("Adding iSCSI static address $static_addr");
            $apiset->iscsiadm_add(
                'direct-object' => "static-config",
                'static-target' => $static_addr,
            );
        } else {
            $Log->comment("Skipping add of iSCSI static address $static_addr (already exists)");
        }
    }

    # configure all iscsi options
    $pkg->_configure_iscsi_options_solaris(%opts);

    # enable static discovery
    $apiset->iscsiadm_modify(
        'direct-object' => "discovery",
        'static'        => "enable",
    );

    # list out the static configuration addresses
    $apiset->iscsiadm_list(
        'direct-object' => "static-config",
    );

    # display what discovery mechanism is enabled and
    # validate that the discovery mode is set to what we want.
    $pkg->_verify_discovery_type_is_enabled_solaris(
        command_interface => $opts{command_interface},
        type              => "static",
    );
    
    $Log->exit();
}

sub _configure_iscsi_isns_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};

    my @targets = @{$opts{targets}};
    
    $Log->comment("iSCSI Configure: Configuring 'iSNS' Discovery");

    # list out the isns servers
    my $resp = $apiset->iscsiadm_list('direct-object' => "isns-server");
    my @output = split(/\n/, $resp->get_raw_output());

    # if the isns server is not added added, then add it
    my $isns_server_ip = $opts{config_iscsi_isns_server};
    if (not grep(/iSNS Server IP Address:\s*$isns_server_ip/i, @output)) {
        # add the isns server
        $apiset->iscsiadm_add(
            'direct-object' => "isns-server",
            'isns-server'   => $isns_server_ip,
        );
    }

    # configure all iscsi options
    $pkg->_configure_iscsi_options_solaris(%opts);

    # enable isns
    $apiset->iscsiadm_modify(
        'direct-object' => "discovery",
        'isns'          => "enable",
    );

    # list out the isns server addresses
    $apiset->iscsiadm_list('direct-object' => "isns-server");

    # display what discovery mechanism is enabled and
    # validate that the discovery mode is set to what we want.
    $pkg->_verify_discovery_type_is_enabled_solaris(
        command_interface => $opts{command_interface},
        type              => "isns",
    );
    
    $Log->exit();
}

sub _configure_iscsi_options_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};

    my @targets = @{$opts{targets}};
    
    $Log->comment("iSCSI Configure: Configuring iSCSI Options");

    my ($out_username, $in_username, $out_password, $in_password);
    my $chap_mode = $opts{config_iscsi_chap_mode} || "NOT_SET";
        
    # if doing CHAP, enable chap on the initiator and set up the in password and user name
    if ($chap_mode =~ /(uni|bi)directional/i) {
        $in_username  = $opts{config_iscsi_chap_in_username};
        $in_password  = $opts{config_iscsi_chap_in_password};
        # enable chap on the initiator
        $apiset->iscsiadm_modify(
            "direct-object"  => "initiator-node",
            "authentication" => "CHAP",
        );

        # establish the IN NAME for CHAP (defaults to iqn of host) {CONFIG_ISCSI_CHAP_USERNAME}
        $apiset->iscsiadm_modify(
            "direct-object" => "initiator-node",
            "chap-name"     => $in_username,
        );

        ## establish the IN PASSWORD for CHAP
        # validate that the in password is between 12-16 chars
        $pkg->_verify_param_length_range(
            param       => $in_password,
            length_min  => 12,
            length_max  => 16,
            apiset      => $apiset,
        );

        # set up the chap password for the initiator
        my @match_prompts = (
            "Enter secret:"    => $in_password,
            "Re-enter secret:" => $in_password,
        );
        $apiset->iscsiadm_modify(
            "direct-object"          => "initiator-node",
            "chap-secret"            => 1,
            "connectrec-match_table" => \@match_prompts,
        );
    }

    # foreach unique target nodename (need to know static/send/isns targets),
    # enable CHAP on the target, set up the out password on the
    # target and set up the out username (iqn of initiator)
    # enable header/data digest if needed
    foreach my $target (@targets) {
        my $nodename = $target->target_name();

        # enable CHAP for the target iqn for both uni-directional and bi-directional chap
        if ($chap_mode =~ /(uni|bi)directional/i) {
            $apiset->iscsiadm_modify(
                'direct-object'  => "target-param",
                'authentication' => "CHAP",
                'target'         => $nodename,
            );
        }

        # if doing bi-directional chap, set up the outpass/outusername
        if ($chap_mode =~ /bidirectional/i) {
            $out_username = $opts{config_iscsi_chap_out_username};
            $out_password = $opts{config_iscsi_chap_out_password};
            # enable BI-CHAP for the target iqn
            $apiset->iscsiadm_modify(
                'direct-object'                 => "target-param",
                'target'                        => $nodename,
                'bi-directional-authentication' => "enable",
            );

            # set up the OUT NAME for the target
            # defaults to iqn of host.
            $apiset->iscsiadm_modify(
                'direct-object' => "target-param",
                'target'        => $nodename,
                'chap-name'     => $out_username,
            );

            # set up OUT PASSWORD for CHAP {CONFIG_ISCSI_CHAP_OUT_PASSWORD}
            
            # validate that the out password is between 12-16 chars
            $pkg->_verify_param_length_range(
                param       => $out_password,
                length_min  => 12,
                length_max  => 16,
                apiset      => $apiset,
            );

            
            # Set up the chap password for this target
            my @match_prompts = (
                "Enter secret:"    => $out_password,
                "Re-enter secret:" => $out_password,
            );
            $apiset->iscsiadm_modify(
                "direct-object"          => "initiator-node",
                "chap-secret"            => 1,
                'target'                 => $nodename,
                "connectrec-match_table" => \@match_prompts,
            );
        } else {
            # otherwise we are not doing bi directional chap, so disable it for this target iqn
            $apiset->iscsiadm_modify(
                'direct-object'                 => "target-param",
                'target'                        => $nodename,
                'bi-directional-authentication' => "disable",
            );
        }

        # if doing header digest then enable it
        if (exists $opts{config_iscsi_header_digest} and $opts{config_iscsi_header_digest} == 1) {
            $apiset->iscsiadm_modify(
                'direct-object' => "target-param",
                'target'        => $nodename,
                'headerdigest'  => "CRC32",
            );
        }

        # if doing data digest then enable it
        if (exists $opts{config_iscsi_data_digest} and $opts{config_iscsi_data_digest} == 1) {
            $apiset->iscsiadm_modify(
                'direct-object' => "target-param",
                'target'        => $nodename,
                'datadigest'    => "CRC32",
            );
        }
    }
    
    $Log->exit();
}

=head1 deconfigure

    NACL::C::Client::Iscsi->deconfigure(
        command_interface=>$ci,
        targets => \@targets,
        config_iscsi_discovery_type => $config_iscsi_discovery_type,
    );

    $iscsi->deconfigure();

(Class or Instance method) Request to configure Iscsi on Client and return a new component object that refers to it.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< targets => @array >>

(Required) for make file system on the device or partition of a harddisk.

=item C<< config_iscsi_version => $string >>

(Required) version iscsi that need to be configured on Client.

Valid values are: open-iscsi, linuxiscsi

=item C<< config_iscsi_discovery_type => $string >>

(Required) It specifies what type of discovery method to use.

Valid values are: sendtargets, static, isns

=item C<< mp_enable => $boolean >>

(Optional) Tells whether Multipathing is enabled or not

default: 1 (enabled)

=item C<< mp_type => $string >>

(Optional) Tells what type of multipathing is used

default: "dmmp"

=item C<< sanbooted => $string >>

(Optional) Tells whether host is Sanbooted or not

default: 0

=item C<< clean_level => $string >>

(Optional) clean_level defines the degree of cleanup that need to be performed.
Sometime people does not want to remove the target completely if they want to reuse them.
Its default value is 2 and it cleans-up iscsi target completely by default.

default: 2

=back

=back

=cut

sub deconfigure {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            mp_type              => { type => SCALAR, },
            mp_enable            => { type => BOOLEAN, default => 1},
            sanbooted            => { type => BOOLEAN, default => 0 },
            clean_level          => { type => SCALAR, default => 2 },
            config_iscsi_version => { type => SCALAR, default => "open-iscsi" },
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   
                method    => "_deconfigure_iscsi_linux",
                interface => "CLI",
                set       => "Linux",
                'privilege-level' => "root",
            },
            {   
                method    => "_deconfigure_iscsi_solaris",
                interface => "CLI",
                set       => "Solaris",
                'privilege-level' => "root",
            },
        ],
    );

    $Log->exit();

    return $ret;
} ## end sub deconfigure

sub _deconfigure_iscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $version = $opts{config_iscsi_version};
    if ($version =~ /open-iscsi/) {
        $pkg->_deconfigure_openiscsi_linux(%opts);
    } else {
        $pkg->_deconfigure_linuxiscsi_linux(%opts);
    }

    $Log->exit();

} ## end sub _deconfigure_iscsi_linux

sub _deconfigure_openiscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $clean_level = $opts{clean_level} || 1;

    # Logout iscsi sessions
    $pkg->_deconfigure_iscsi_logout_sessions_linux(%opts);

    # configure iscsi options for all the newly added targets
    $pkg->_deconfigure_openiscsi_options_linux(%opts);

    if ($clean_level == 2) {
        my $discovery_type = $opts{config_iscsi_discovery_type};
        my $method = "_deconfigure_openiscsi_" . $discovery_type . "_linux";

        $pkg->$method(%opts);
    }

    $Log->exit();
}

sub _deconfigure_iscsi_logout_sessions_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $clean_level = $opts{clean_level} || 1;
    # logout from all the targets if clean_level == 2, if we get:
    # No portal found, that is OK, nothing was configured prior
    if ($clean_level == 2) {
        try {
            $apiset->iscsiadm(mode => 1, node => 1, logout => 1);
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $ex = shift;
            if ($ex->text() !~ /No portal found|No session found|No matching sessions found/i) {
                $ex->set_text("Failed to list iscsi sessions. " . $ex->text());
                $ex->throw();
            }
        };
    } else {
        # check for required parameters
        # my @targets = @{$opts{targets}};
        my @targets = @{$opts{targets}};
        if (not scalar(@targets)) {
            # Throw missing param exception
        }

        # otherwise if clean_level == 1, logout from the targets listed in @targets
        foreach my $target (@targets) {
            my $nodename = $target->target_name();

            my $ip = $target->address();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }

            try {
                $apiset->iscsiadm(
                    mode          => 1,
                    node          => 1,
                    logout        => 1,
                    "ip-port"     => "$ip:" . ISCSI_PORT,
                    "target-name" => $nodename,
                );
            } catch NACL::APISet::Exceptions::CommandFailedException with {
                my $ex = shift;
                if ($ex->text() !~ /No portal found|No session found|No matching sessions found/i) {
                    $ex->set_text("Failed to list iscsi sessions. " . $ex->text());
                    $ex->throw();
                }
            };
        }
    }

    $Log->exit();
}

sub _deconfigure_openiscsi_options_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);
    my $clean_level = $opts{clean_level} || 1;
    my @targets = @{$opts{targets}};

    # first check if there are any target records, if there are none, then return immediately
    my $resp;
    try {
        $resp = $apiset->iscsiadm(mode => 1, node => 1);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() !~ /no records found/) {
            $ex->throw();
        }
        $Log->comment("iSCSI Deconfigure: Skipping openiscsi options deconfigure (no targets to deconfigure)");
    };
    my $output = $resp->get_raw_output();
    my @list_output = split(/\n/, $output);

    my %iscsi_options = (
        mode      => 1,
        node      => 1,
        operation => "update",
    );

    # Disable CHAP/HD/DD for all targets if clean_level is 2
    if ($clean_level == 2) {

        $pkg->_deconfigure_openiscsi_disable_chap_linux(%opts);

        # disable header/data digest
        $apiset->iscsiadm(
            name  => "node.conn[0].iscsi.HeaderDigest",
            value => "None",
            %iscsi_options
        );

        # Data Digest is not supported in redhat's and oracle-linux's open-iscsi
        # Command will fail if we try it
        if ($major !~ /rhel|ol/i) {
            $apiset->iscsiadm(
                name  => "node.conn[0].iscsi.DataDigest",
                value => "None",
                %iscsi_options
            );
        }

    } else {
        # for clean_level 1, disable CHAP/HD/DD only for @targets
        foreach my $target (@targets) {
            my $nodename = $target->target_name();
            my $ip = $target->address();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }

            # see if this target is in the list output.
            if (not grep(/^\s*${ip}:3260,\d+ ${nodename}\s*$/, @list_output)) {
                # skip this target since it is not in the output
                next;
            }

            # Disable CHAP
            $pkg->_deconfigure_openiscsi_disable_chap_linux(%opts, nodename => $nodename, ip => $ip);

            # disable header/data digest
            $apiset->iscsiadm(
                name => "node.conn[0].iscsi.HeaderDigest",
                value => "None",
                %iscsi_options
            );

            # Data Digest is not supported in redhat's and oracle linux's open-iscsi
            # This command will fail if we try it
            if ($major !~ /rhel|ol/i) {
                $apiset->iscsiadm(
                    name => "node.conn[0].iscsi.DataDigest",
                    value => "None",
                    %iscsi_options
                );
            }
        }
    }

    $Log->exit();
}

sub _deconfigure_openiscsi_sendtargets_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $clean_level = $opts{clean_level};
    my @targets = @{$opts{targets}};

    # list out all the sendtargets currently on the system
    my $resp;
    try {
        $resp = $apiset->iscsiadm(mode => 1, discovery => 1);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        # status can be non-zero when there are no targets, don't error out
        # if there is no raw_output
        if (length $resp->get_raw_output()) {
            my $ex = shift;
            $ex->throw();
        }
    };

    my @list_output = split (/\n/, $resp->get_raw_output());

    # remove all sendtargets targets if clean_level == 2
    if ($clean_level == 2) {
        # remove all sendtargets that are listed in @list_output
        foreach my $line (@list_output) {
            if ($line =~ /(\S+):3260 via sendtargets/i) {
                my $target_ip = $1;

                $apiset->iscsiadm(mode => 1, discovery => 1, operation => "delete", "ip-port" => $target_ip);
            }
        }

        # comment out the changes we made to the /etc/iscsid.conf file for chap authentication
        my @chap_params = (
            "discovery.sendtargets.auth.authmethod",
            "discovery.sendtargets.auth.username",
            "discovery.sendtargets.auth.password",
            "discovery.sendtargets.auth.username_in",
            "discovery.sendtargets.auth.password_in",
        );

        foreach my $param (@chap_params) {
           $pkg->_comment_out_parameter_in_file(
                param   => $param,
                file    => "/etc/iscsi/iscsid.conf",
                apiset  => $apiset
            );
        }

    } else {
        # check for required parameters
        if (not scalar(@targets)) {
            #Throw missing parameter exception
        }

        # if there are no targets in the output, then return immediately
        if (scalar(@list_output) == 0) {
            $Log->exit();
            return ;
        }

        # remove all sendtargets targets in @targets if clean_level == 1
        foreach my $target (@targets) {
            my $ip = $target->address();

            if (grep(/$ip\:3260 via sendtargets/i, @list_output)) {
                # delete the target because it is in the list of targets currently added
                $apiset->iscsiadm(mode => 1, discovery => 1, operation => "delete", "ip-port" => $ip);
            }
        }
    }

    $Log->exit();
}

sub _deconfigure_openiscsi_static_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $clean_level = $opts{clean_level};
    my @targets = @{$opts{targets}};

    # if there are no targets in the list, return immediately
    my $resp = $apiset->iscsiadm(mode => 1, node => 1);
    my @list_output = split(/\n/, $resp->get_raw_output());
    if (grep(/no records found/i, @list_output)) {
        $Log->exit();
        return ;
    }

    # remove all static targets if clean_level == 2
    if ($clean_level == 2) {
        $apiset->iscsiadm(mode => 1, node => 1, operation => "delete");
    } else {
        # check for required parameters
        if (not scalar(@targets)) {
            # Throw missing parameter ex
        }

        # remove all static targets in @targets if clean_level == 1
        foreach my $target (@targets) {
            my $ip = $target->address();

            my $nodename = $target->target_name();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }

            #--> iscsiadm -m node
            #192.168.1.61:3260,2000 iqn.1992-08.com.netapp:sn.84174947
            #[fe80::a0:98ff:fe01:70aa]:3260,2000 iqn.1992-08.com.netapp:sn.84174947

            # if this target is in the list, remove it
            if (grep(/^\s*${ip}:3260,\d+\s+${nodename}/i, @list_output)) {
                # delete the target because it is in the list of targets currently added
                my $ip_port = "$ip:" . ISCSI_PORT;
                $apiset->iscsiadm(
                    mode          => 1,
                    node          => 1,
                    operation     => "delete",
                    "target-name" => $nodename,
                    "ip-port"     => $ip_port,
                );
            }
        }
    }

    $Log->exit();
}

sub _deconfigure_openiscsi_isns_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset  = $opts{apiset};
    my @targets = @{$opts{targets}};
    # manually edit the iscsid.conf file and remove the isns address field and port
    # isns.address = <ip>
    # isns.port = 3205
    my @isns = ("isns.address", "isns.port");
    foreach my $param (@isns) {
        $pkg->_comment_out_parameter_in_file(
            param   => $param,
            file    => "/etc/iscsi/iscsid.conf",
            apiset  => $apiset
        );
    }

    # first check if there are any target records, if there are none, then return immediately
    my $resp;
    try {
        $resp = $apiset->iscsiadm(mode => 1, node => 1);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() !~ /no records found/) {
            $ex->throw();
        }
    };

    my @list_output = split(/\n/, $resp->get_raw_output());

    my $clean_level = $opts{clean_level};
    # if clean_level == 2, remove all targets by doing iscsiadm -m node -o delete
    if ($clean_level == 2) {
        $apiset->iscsiadm(mode => 1, node => 1, operation => "delete");
    } else {
        # check for required parameters
        if (not scalar(@targets)) {
            # Throw missing ex
        }

        # if clean_level == 1, remove targets in @targets
        foreach my $target (@targets) {
            my $ip = $target->address();
            my $nodename = $target->target_name();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }

            #--> iscsiadm -m node
            #192.168.1.61:3260,2000 iqn.1992-08.com.netapp:sn.84174947
            #[fe80::a0:98ff:fe01:70aa]:3260,2000 iqn.1992-08.com.netapp:sn.84174947

            # if this target is in the list, remove it
            if (grep(/^\s*${ip}:3260,\d+\s+${nodename}/, @list_output)) {
                $apiset->iscsiadm(
                    mode          => 1,
                    node          => 1,
                    operation     => "delete",
                    "target-name" => $nodename,
                    "ip-port"     => "$ip:" . ISCSI_PORT,
                );
            }
        }
    }

    # need to reload the iscsi driver only if not sanbooted and clean level == 2
    # this will make the iscsidaemon forget about the isns server
    if ($opts{sanbooted} == 0 and $clean_level == 2) {
        $pkg->restart();
    }

    $Log->exit();
}


sub _deconfigure_linuxiscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset  = $opts{apiset};
    my @targets = @{$opts{targets}};

    $apiset->rm(paths => "/tmp/iscsi_conf",force => 1);

    $apiset->touch(path => "/tmp/iscsi_conf");

    my @target_ips;
    foreach my $target (@targets) {
        push (@target_ips, $target->{ip});
    }

    my $resp = $apiset->cat(files => "/etc/iscsi.conf");
    my @output = split(/\n/, $resp->get_raw_output);

    # Delete all the lines DiscoveryAddress=ip if ip is present
    # in the target list passed into this method
    foreach my $line (@output) {
        if ($line =~ /DiscoveryAddress/i) {
            my ($ip) = ($line =~ /DiscoveryAddress\=(\d+\.\d+\.\d+\.\d+)/);
            next if grep (/$ip/, @target_ips);
        }

        $apiset->execute_raw_command(command => "echo \'$line\' >> /tmp/iscsi_conf");
    }

    $apiset->mv(source => "/tmp/iscsi_conf", target => "/etc/iscsi.conf", force => 1);

    $Log->exit();
}

sub _deconfigure_openiscsi_disable_chap_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my %iscsi_options = (
        mode      => 1,
        node      => 1,
        operation => "update",
    );

    my $apiset = $opts{apiset};
    if (defined $opts{nodename} and defined $opts{ip}) {
        $iscsi_options{"target-name"} = $opts{nodename};
        $iscsi_options{"ip-port"}     = $opts{ip} . ":" . ISCSI_PORT;
    }

    # disable CHAP
    $apiset->iscsiadm(
        name => "node.session.auth.authmethod",
        value => "None",
        %iscsi_options
    );

    my @chap_params = (
        "node.session.auth.username_in",
        "node.session.auth.password_in",
        "node.session.auth.username",
        "node.session.auth.password",
    );

    # reset the CHAP in/out-credentials
    foreach my $param (@chap_params) {
        $apiset->iscsiadm(
            name => $param,
            value => '""',
            %iscsi_options
        );
    }

    $Log->exit();
}

sub _deconfigure_iscsi_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $command_interface = $opts{command_interface};
    
    $pkg->_deconfigure_iscsi_options_solaris(%opts);
    
    my $discovery_type = $opts{config_iscsi_discovery_type};
    my $method = "_deconfigure_iscsi_" . $discovery_type . "_solaris";

    $pkg->$method(%opts);

    $Log->exit();
} ## end sub _configure_iscsi_linux

sub _deconfigure_iscsi_sendtargets_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset          = $opts{apiset};
    my $ci              = $opts{command_interface};
    my $clean_level     = $opts{clean_level} || 1;
    my $discovery_type  = $opts{config_iscsi_discovery_type};
    my @targets         = @{$opts{targets}};
    
    $Log->comment("iSCSI Deconfigure: Deconfiguring 'Sendtargets' Discovery");

    # list out all the sendtargets discovery targets
    my $resp = $apiset->iscsiadm_list('direct-object' => "discovery-address");
    my @sendtargets_list = split(/\n/, $resp->get_raw_output());

    # if clean_level is 1, then clean up only the targets we know about

    my $already_rebooted = 0;
    
    if ($clean_level == 1) {
        foreach my $target (@targets) {
            # check to see if the ip address for this target is in the list
            my $ip = $target->address();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }

            my $sendtargets_addr = "$ip:" . ISCSI_PORT;

            # if this target is not in the list, then skip the removal
            if (not grep(/$sendtargets_addr/, @sendtargets_list)) {
                next;
            }

            # remove this target from iscsiadm
            $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                command_interface           => $ci,
                already_rebooted            => $already_rebooted,
                apiset                      => $opts{apiset},
                address                     => $sendtargets_addr,
                config_iscsi_discovery_type => $discovery_type,
            );
        }
    }   
    # if clean_level is 2, then clean up all targets we see.
    # try to disable discovery
    elsif ($clean_level == 2) {
        foreach my $line (@sendtargets_list) {
            if ($line =~ /Discovery Address:\s*(\S+):\d*/i) {
                my $sendtargets_addr = $1;
                # if ipv6, modify the address to be in brackets
                if (ip_is_ipv6($sendtargets_addr)) {
                    $sendtargets_addr = "[" . $sendtargets_addr . "]";
                }
                # remove this static target from iscsiadm
                $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                    command_interface           => $ci,
                    already_rebooted            => $already_rebooted,
                    apiset                      => $opts{apiset},
                    address                     => $sendtargets_addr,
                    config_iscsi_discovery_type => $discovery_type,
                );
            }
        }
        
        # try and disable sendtargets discovery, reboot the system if we detect luns are in use
        $pkg->_modify_options_discovery_type_solaris(
            command_interface           => $ci,
            already_rebooted            => $already_rebooted,
            apiset                      => $opts{apiset},
            config_iscsi_discovery_type => $discovery_type,
        );
    }
    
    $Log->exit();
} ## end sub _deconfigure_iscsi_sendtargets_solaris

sub _deconfigure_iscsi_static_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset          = $opts{apiset};
    my $ci              = $opts{command_interface};
    my $clean_level     = $opts{clean_level} || 1;
    my $discovery_type  = $opts{config_iscsi_discovery_type};
    my @targets         = @{$opts{targets}};
    
    $Log->comment("iSCSI Deconfigure: Deconfiguring 'Static' Discovery");

    # list out all the sendtargets discovery targets
    my $resp = $apiset->iscsiadm_list('direct-object' => "static-config");
    my @static_list = split(/\n/, $resp->get_raw_output());

    my $already_rebooted = 0;

    # if clean_level is 1, then clean up only the targets we know about
    if ($clean_level == 1) {
        foreach my $target (@targets) {
            # check to see if the ip address for this target is in the list
            my $ip = $target->address();
            my $nodename = $target->target_name();
            my $tpgt = $target->tpgroup_tag();

            # if the target ip is ipv6, then add brackets around it.
            if (ip_is_ipv6($ip)) {
                $ip = "[" . $ip . "]";
            }
            
            my $static_addr = $nodename . "," . $ip . ISCSI_PORT . "," . $tpgt;
                                  
            # check to see if this ip target is in the list,
            # if it is not, then skip the target
            if (not grep(/$static_addr/, @static_list)) {
                next;
            }

            # remove this target from iscsiadm
            $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                command_interface           => $ci,
                already_rebooted            => $already_rebooted,
                apiset                      => $opts{apiset},
                address                     => $static_addr,
                config_iscsi_discovery_type => $discovery_type,
            );
        }
    }
    # if clean_level is 2, then clean up all targets we see.
    # try to disable static but dont fail if it doesnt work
    elsif ($clean_level == 2) {
        foreach my $line (@static_list) {
            if ($line =~ /Static Configuration Target:\s*(\S+)/i) {
                my $static_addr = $1;
                # remove this static target from iscsiadm
                $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                    command_interface           => $ci,
                    already_rebooted            => $already_rebooted,
                    apiset                      => $opts{apiset},
                    address                     => $static_addr,
                    config_iscsi_discovery_type => $discovery_type,
                );
            }
        }
        # try and disable static discovery, it may error because it can fail if LUNs
        # are mapped to the static iscsi initiator, reboot if this is the case.
        $pkg->_modify_options_discovery_type_solaris(
            command_interface           => $ci,
            already_rebooted            => $already_rebooted,
            apiset                      => $opts{apiset},
            config_iscsi_discovery_type => $discovery_type,
        );
    }
    
    $Log->exit();
} ## end sub _deconfigure_iscsi_static_solaris

sub _deconfigure_iscsi_isns_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset          = $opts{apiset};
    my $ci              = $opts{command_interface};
    my $clean_level     = $opts{clean_level} || 1;
    my $discovery_type  = $opts{config_iscsi_discovery_type};
    my $isns_server     = $opts{config_iscsi_isns_server};
    my @isns_servers    = ($isns_server);
    my $disable_isns    = 0;
    
    $Log->comment("iSCSI Deconfigure: Deconfiguring 'iSNS' Discovery");

    # list out all the sendtargets discovery targets
    my $resp = $apiset->iscsiadm_list('direct-object' => "isns-server");
    my @isns_list = split(/\n/, $resp->get_raw_output());

    my $already_rebooted = 0;

    # if clean_level is 1, then clean up only the isns servers we know about
    if ($clean_level == 1) {
        # if the isns server specified is there, remove it
        if (grep(/iSNS Server IP Address: ${isns_server}\:/, @isns_list)) {
            # remove the isns server we know of
            $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                command_interface           => $ci,
                already_rebooted            => $already_rebooted,
                apiset                      => $opts{apiset},
                address                     => $isns_server,
                config_iscsi_discovery_type => $discovery_type,
            );
            # need to disable isns
            $disable_isns = 1;
        }  
    }   
    # if clean_level is 2, then clean up all isns servers we see.
    # try to disable isns but dont fail if it doesnt work
    elsif ($clean_level == 2) {
        foreach my $line (@isns_list) {
            if ($line =~ /iSNS Server IP Address:\s*(\S+)/i) {
                my $isns_server = $1;

                # remove this isns server from iscsiadm
                $already_rebooted = $pkg->_remove_options_discovery_type_solaris(
                    command_interface           => $ci,
                    already_rebooted            => $already_rebooted,
                    apiset                      => $opts{apiset},
                    address                     => $isns_server,
                    config_iscsi_discovery_type => $discovery_type,
                );
            }
        }
        # need to disable isns
        $disable_isns = 1;
    }
    
    if ($disable_isns == 1) {
        # try and disable sendtargets discovery, reboot the system if we detect luns are in use
        $pkg->_modify_options_discovery_type_solaris(
            command_interface           => $ci,
            already_rebooted            => $already_rebooted,
            apiset                      => $opts{apiset},
            config_iscsi_discovery_type => $discovery_type,
        );
    }
    
    $Log->exit();
} ## end sub _deconfigure_iscsi_isns_solaris

sub _remove_options_discovery_type_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            command_interface => { isa => 'NACL::C::CommandInterface', type => OBJECT },
            apiset            => { isa => 'NACL::APISet', type => OBJECT },
            address           => { type => SCALAR, },
            already_rebooted  => { type => SCALAR, default => 0 },
            config_iscsi_discovery_type => { type => SCALAR, },
        },
    );
    
    my %options_hash;
    
    my $apiset   = $opts{apiset};
    my $rebooted = $opts{already_rebooted};
    my $ci       = $opts{command_interface};
    my $type     = $opts{config_iscsi_discovery_type};
    my $address  = $opts{address};
    
    if ($type =~ /sendtargets/) {
        %options_hash = (
            'direct-object'     => "discovery-address",
            'discovery-address' => $address,
        );
    } if ($type =~ /static/) {
        %options_hash = (
            'direct-object' => "static-config",
            'static-target' => $address,
        );
    } if ($type =~ /isns/) {
        %options_hash = (
            'direct-object' => "isns-server",
            'isns-server'   => $address,
        );
    } 
    
    try {
        $apiset->iscsiadm_remove(%options_hash);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() =~ /logical unit in use/ and (not $rebooted)) {
            NACL::MTask::ClientUtils->reboot(command_interface => $ci);
            $rebooted = 1;
            # Retry one more time
            $apiset->iscsiadm_remove(%options_hash);
        } else {
            $ex->set_text("Unable to remove iscsi discovery of type $type even after reboot");
            $Log->exit();
            $ex->throw();
        }
    }
    
    $Log->exit();
    return $rebooted;
}

sub _modify_options_discovery_type_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            command_interface => { isa => 'NACL::C::CommandInterface', type => OBJECT },
            apiset            => { isa => 'NACL::APISet', type => OBJECT },
            already_rebooted  => { type => SCALAR, default => 0 },
            config_iscsi_discovery_type => { type => SCALAR, },
        },
    );
    
    my $apiset   = $opts{apiset};
    my $rebooted = $opts{already_rebooted};
    my $ci       = $opts{command_interface};
    my $type     = $opts{config_iscsi_discovery_type};
    
    my %iscsi_hash = (
        'direct-object' => "discovery",
    );
    $iscsi_hash{$type} = "disable";
        
    try {
        $apiset->iscsiadm_modify(%iscsi_hash);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() =~ /logical unit in use/ and (not $rebooted)) {
            NACL::MTask::ClientUtils->reboot(command_interface => $ci);
            # Retry one more time
            $apiset->iscsiadm_modify(%iscsi_hash);
        } else {
            $ex->set_text("Unable to disable iscsi discovery of type $type even after reboot");
            $Log->exit();
            $ex->throw();
        }
    }
    
    $Log->exit();
}
sub _disable_target_param_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = validate_with(
        params => \@opts,
        spec   => {
            apiset           => { isa => 'NACL::APISet', },
            target           => { type => SCALAR, },
        },
    );

    my $nodename = $opts{target};
    my $apiset   = $opts{apiset};
    
    my %iscsi_hash = (
        'direct-object' => "target-param",
        'target'        => $nodename,
    );

    # disable chap for this target
    $apiset->iscsiadm_modify(
        'authentication' => "none",
        %iscsi_hash,
    );

    # disable bi-directional chap
    $apiset->iscsiadm_modify(
        'bi-directional-authentication' => "disable",
        %iscsi_hash,
    );

    # disable header digest
    $apiset->iscsiadm_modify(
        'headerdigest' => "none",
        %iscsi_hash,
    );

    # disable data digest
    $apiset->iscsiadm_modify(
        'datadigest' => "none",
        %iscsi_hash,
    );
    
    $Log->exit();
}

sub _deconfigure_iscsi_options_solaris {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my $clean_level = $opts{clean_level} || 1;
    my @targets = @{$opts{targets}};

    # list all the targets
    my $resp = $apiset->iscsiadm_list('direct-object' => "target");
    my @list_output = split(/\n/, $resp->get_raw_output());
    
    # if clean_level == 1 (soft approach, only affect the targets in @targets)
    # disable chap/header/data digest for each unique target in the targets list that we are working with
    if ($clean_level == 1) {    
        foreach my $target (@targets) {
            my $nodename = $target->target_name();
            
            # skip this target if it is not on the host currently
            if (not grep(/Target:\s+${nodename}/, @list_output)) {
                next;
            }
            
            $pkg->_disable_target_param_solaris(
                target => $nodename,
                apiset => $apiset,
            );
        }
    }

    # if clean_level == 2 (hammer approach, affect all targets)
    elsif ($clean_level == 2) {
        foreach my $line (@list_output) {
            if ($line =~ /Target: (\S+)/i) {
                my $nodename = $1;
                $pkg->_disable_target_param_solaris(
                    target => $nodename,
                    apiset => $apiset,
                );
            }
        }
        
        # Disable initiator-side CHAP
        $apiset->iscsiadm_modify(
            'direct-object'  => "initiator-node",
            'authentication' => "none",
        );

        # disable header digest on the initiator
        $apiset->iscsiadm_modify(
            'direct-object'  => "initiator-node",
            'headerdigest' => "none",
        );
        
        # disable data digest on the initiator
        $apiset->iscsiadm_modify(
            'direct-object'  => "initiator-node",
            'datadigest' => "none",
        );
    }
    
    $Log->exit();
}

sub restart {
    $Log->enter();
    my ($pkg, @opts) = @_;
    $pkg->_frontend_log_debug_opts(@opts);
    my %opts = $pkg->_common_validate_with(
        params          => \@opts,
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_restart_iscsi_linux",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );

    $Log->exit();
} ## end sub restart

sub _restart_iscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    $pkg->_stop_iscsi_linux(%opts);

    $pkg->_start_iscsi_linux(%opts);

    $Log->exit();
}


sub start {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_iscsi_linux",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );

    $Log->exit();
} ## end sub start

sub _start_iscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);

    my $api = "service";
    if ($major =~ /rhel7/) {
        $api = "systemctl";
    }

    my $action = "start";
    if ($major =~ /rhel6|ol6/) {
        $action = "force-start";
    }

    my $script = $pkg->_get_iscsi_script_linux(apiset => $opts{apiset});

    try {
        $apiset->$api(script => $script, action => $action);
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        # Dont fail yet. Check the output of status command
        # To make sure its really failed
    };

    Tharn::sleep(3);
    try {
        $apiset->$api(script => $script, action => "status");
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        $Log->exit();
        $ex->set_text("Issued command $api start , but failed to start service. " . $ex->text());
        $ex->throw();
    };

    $Log->exit();
}

sub stop {
    $Log->enter();
    my ($pkg, @opts) = @_;
    $pkg->_frontend_log_debug_opts(@opts);
    my %opts = $pkg->_common_validate_with(
        params          => \@opts,
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_stop_iscsi_linux",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );

    $Log->exit();

} ## end sub stop

sub _stop_iscsi_linux {
    $Log->enter();
    my ($pkg, @opts)  = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset = $opts{apiset};
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);

    my $api = "service";
    if ($major =~ /rhel7/) {
        $api = "systemctl";
    }

    my $script = $pkg->_get_iscsi_script_linux(apiset => $opts{apiset});

    try {
        $apiset->$api(script => $script, action => "stop");
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        # Dont fail yet. Check the output of status command
        # To make sure that its really failed
    };

    try {
        $apiset->$api(script => $script, action => "status");
    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $ex = shift;
        if ($ex->text() =~ /running/) {
            $Log->exit();
            $ex->set_text("Issued command $api stop , but service is still running. " . $ex->text());
            $ex->throw();
        }
    };

    $Log->exit();
}

sub _get_iscsi_script_linux {
    $Log->enter();
    my ($pkg, @opts) = @_;
    $pkg->_frontend_log_debug_opts(@opts);

    my %opts = validate_with(
        params => \@opts,
        spec   => {
            apiset => { isa  => "NACL::APISet", }
        }
    );

    my $apiset = $opts{apiset};
    my ($major, $minor) = $pkg->_get_major_minor(apiset => $apiset);

    my $script = "iscsi";
    if ($major =~ /sles|xen/i) {
        if ($opts{config_iscsi_version} =~ /open-iscsi/i) {
            $script = "open-iscsi";
        } else {
            $script = "iscsi";
        }
    } else {
        if ($major =~ /rhel7/) {
            $script = "iscsid.service";
        } elsif ($major =~ /rhel6|ol6/i) {
            $script = "iscsid";
        } else {
            $script = "iscsi";
        }
    }

    $Log->exit();
    return $script;
}

sub _primary_keys_validate_spec {
    return (
        targets => { type => ARRAYREF },
        config_iscsi_discovery_type => {type => SCALAR },
    );
}

1;
