#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary LV Component Module
## @author dl-nacl-dev@netapp.com, praveent@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::LV

=head1 DESCRIPTION

This library is used to perform operations of creating and deleting Logical
volume on the hosts.

=head1 METHODS

=head2 new

  my $obj = NACL::C::Client::LV->new(   command_interface => $ci,
                                       'lvsize'           => $size,
                                       'lv_name'          => $lv_name,
                                       'vg_name'          => $vgname,);

Return a new LV component object with the given attributes.

=cut

package NACL::C::Client::LV;

use strict;
use warnings;
use NACL::CS::Client::LV;
use NACL::ComponentUtils qw(_optional_scalars );
use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with SCALAR);
use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    scalar => 'lv_name',
    scalar => 'vg_name',
];

=head2 create()

 my $obj = NACL::C::Client::LV->create(command_interface  => $ci,
                                           'lvsize'       => $size,
                                           'lv_name'      => $lv_name,
                                           'vg_name'      => $vgname,);

(Class method) Request creation of a LV (logical volume)  element, and  return a new
component object that refers to it.

Uses Linux-CLI APISet.

Invokes "lvcreate" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device which needs to be converted to volume group.

=item C<< 'lvsize' => $volumes >>

Applicable for Linux-CLI.

(Required): Size of the device which needs to be converted to logical volume.

=item C<< 'pvname' => $volumes >>

Applicable for Linux-CLI.

(Optional): Name of the device which needs to be converted to physical volume.

=item C<< 'lv_name' => $volumes >>

Applicable for Linux-CLI.

(Optional): Name of the device which needs to be converted to logical volume.

=item C<< 'addtag' => $addtag >>

Applicable for Linux-CLI.

(Optional): addtag field.

=item C<< 'alloc-policy' => $policy >>

Applicable for Linux-CLI.

(Optional): alloc-policy field.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional): aautobackup field.

=item C<< 'mirror-consistency' => $mirror >>

Applicable for Linux-CLI.

(Optional): mirror-consistency field.

=item C<< 'contiguous' => $contiguous >>

Applicable for Linux-CLI.

(Optional): contiguous field.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional): debug field.

=item C<< 'schedule' => $schedule >>

Applicable for Linux-CLI.

(Optional): schedule field.

=item C<< 'distributed' => $distributed >>

Applicable for Linux-CLI.

(Optional): distributed field.

=item C<< 'stripes' => $stripes >>

Applicable for Linux-CLI.

(Optional): stripes field.

=item C<< 'stripesize' => $size >>

Applicable for Linux-CLI.

(Optional): stripesize field.

=item C<< 'logicalextents' => $extents >>

Applicable for Linux-CLI.

(Optional): logicalextents field.

=item C<< 'chunksize' => $size >>

Applicable for Linux-CLI.

(Optional): chunksize field.

=item C<< 'persistent' => $persistent >>

Applicable for Linux-CLI.

(Optional): persistent field.

=item C<< 'minor' => $minor >>

Applicable for Linux-CLI.

(Optional): minor field.

=item C<< 'mirrors' => $mirrors >>

Applicable for Linux-CLI.

(Optional): mirrors field.

=item C<< 'nosync' => $boolean >>

Applicable for Linux-CLI.

(Optional): nosync field.

=item C<< 'mirrorlog' => $mirrorlog >>

Applicable for Linux-CLI.

(Optional): mirrorlog field.

=item C<< 'corelog' => $boolean >>

Applicable for Linux-CLI.

(Optional): corelog field.

=item C<< 'regionsize' => $size >>

Applicable for Linux-CLI.

(Optional): regionsize field.

=item C<< 'permission' => $permission >>

Applicable for Linux-CLI.

(Optional): permission field.

=item C<< 'readahead' => $readahead >>

Applicable for Linux-CLI.

(Optional): readahead field.

=item C<< 'snapshot' => $snapshot >>

Applicable for Linux-CLI.

(Optional): snapshot field.

=item C<< 'strict' => $strict >>

Applicable for Linux-CLI.

(Optional): strict field.

=item C<< 'snap-type' => $type >>

Applicable for Linux-CLI.

(Optional): snap-type field.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional): test field.

=item C<< 'timestamp' => $timestamp >>

Applicable for Linux-CLI.

(Optional): timestamp field.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional): verbose field.

=item C<< 'type' => $type >>

Applicable for Linux-CLI.

(Optional): type field.

=item C<< 'zero' => $zero >>

Applicable for Linux-CLI.

(Optional): zero field.

=item C<< 'virtual-size' => $size >>

Applicable for Linux-CLI.

(Optional): virtual-size field.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                  addtag alloc-policy autobackup mirror-consistency contiguous debug
                  schedule distributed stripes stripesize logicalextents lvsize
                  chunksize persistent minor mirrors nosync mirrorlog corelog
                  regionsize permission readahead snapshot strict test
                  snap-type timestamp verbose type zero vgname virtual-size phy-volumes
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_create_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub create

sub _create_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
              addtag alloc-policy autobackup mirror-consistency contiguous debug
              schedule distributed stripes stripesize logicalextents lvsize
              chunksize persistent minor mirrors nosync mirrorlog corelog
              regionsize permission readahead snapshot strict test
              snap-type timestamp verbose type zero virtual-size phy-volumes
              )
        ],
        map => { lv_name => 'lvname',
                 vg_name => 'vgname',},
    );
    $apiset->lvcreate(%linux_args);

    my $obj = $pkg->new(
        command_interface => $command_interface,
        'lv_name'          => "/dev/$opts{vg_name}/$opts{lv_name}",
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_linux_cli

=head2 remove()

 NACL::C::Client::LV->remove(  command_interface   => $ci,
                                 'lv_name'         => $lv_name,
                                 'vg_name'         => $vgname,);

                                        or
 $lv_obj->remove(...);

(Class method or Instance method)  Request deletion of a LV (logical volume)  element, and deletes
component object that refers to it.

Uses Linux-CLI APISet.

Invokes "lvremove" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'lv_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device which needs to be removed from volume group.

=item C<< 'vg_name' => $vg_name >>

Applicable for Linux-CLI.

(Required): Name of the volume group on which logical volume is present.

=item C<< 'autobackup' => $boolean >>

Applicable for Linux-CLI.

(Optional): autobackup field.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional): debug field.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional): verbose field.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional): test field.

=item C<< 'force' => $boolean >>

Applicable for Linux-CLI.

(Optional): force field.

=item C<< 'help' => $boolean >>

Applicable for Linux-CLI.

(Optional): help field.

=item C<< 'snap-lvremove' => $snap-lvremove >>

Applicable for Linux-CLI.

(Optional): snap-lvremove field.

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                  autobackup debug force test verbose snap-lvremove vg_name
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_remove_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub remove

sub _remove_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $lvpath;
    my $prompts_answers = ["Do you really want to remove.*" => "y",];

    if ($opts{lv_name} =~ /^\/dev\/.+?\/.+?$/) {
       $lvpath = $opts{lv_name};
    } elsif($opts{lv_name} && $opts{vgname}) {
        $lvpath = $pkg->_construct_lvpath(
           vgname => $opts{vgname},
           lv_name => $opts{lv_name},
        );
      delete $opts{vgname};
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
          "Pass lv_name either in format '/dev/vgname/lv_name' or pass both lv_name and vg_name params");
    }
    $opts{"lv_name"} = $lvpath;
    my %lvremove_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%lvremove_args,
        copy   => [
            qw (
              autobackup debug force test verbose snap-lvremove
              )
        ],
        map => { lv_name => 'logical-volume-path' },
    );
    $apiset->lvremove(%lvremove_args, 'connectrec-match_table' => $prompts_answers);
    $Log->exit() if $may_exit;
} ## end sub _remove_linux_cli

sub _primary_keys_validate_spec {
    return ( lv_name => { type => SCALAR },
             vg_name => { type => SCALAR , optional => 1}
          );
} ## end sub _primary_keys_validate_spec

sub _construct_lvpath {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { vg_name  => { type => SCALAR },
                    lv_name  => { type => SCALAR },
                  },
    );
    my $lvpath = "/dev/$opts{vg_name}/$opts{lv_name}";
    $Log->exit() if $may_exit;
    return $lvpath;
}

=head2 rename()

 my $obj = NACL::C::Client::LV->rename(  command_interface => $ci,
                                        'lv_name'          => /dev/$vg_name/$lv_name,
                                        'new-lvpath'       => /dev/$vg_name/$lv_name1,);

                                        or
 $obj->rename(...);

(Class method or Instance method)  Request rename of a LV (logical volume) element.

Uses Linux-CLI APISet.

Invokes "lvrename" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'lv_name' => $new-lvpath >>

Applicable for Linux-CLI.

(Required): Name of the device which needs to be renamed.

=item C<< 'autobackup' => $value >>

Applicable for Linux-CLI.

(Optional): autobackup field.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional): debug field.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional): test field.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional): verbose field.

=item C<< 'version' => $boolean >>

Applicable for Linux-CLI.

(Optional): version field.

=item C<< 'force' => $boolean >>

Applicable for Linux-CLI.

(Optional): force field.

=item C<< 'noudevsync' => $boolean >>

Applicable for Linux-CLI.

(Optional): noudevsync field.

=item C<< 'old-lvpath' => $old-lvpath >>

Applicable for Linux-CLI.

(Optional): old-lvpath field.

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    autobackup debug test verbose version force noudevsync new-lvpath
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_rename_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub rename

sub _rename_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                    autobackup debug test verbose version force noudevsync new-lvpath
              )
        ],
        map => { lv_name => 'old-lvpath' },
    );

    $apiset->lvrename(%linux_args);
    my $obj = $pkg->new(
        command_interface => $command_interface,
        'lv_name'         => $opts{'new-lvpath'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _rename_linux_cli

1;
