#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Multipath Component Module
## @author dl-nacl-dev@netapp.com, sshaik@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::Multipath

=head1 DESCRIPTION

C<NACL::C::Client::Multipath> Is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 device

=cut

package NACL::C::Client::Multipath;
use strict;
use warnings;
use NACL::CS::Client::Multipath;
use base 'NACL::C::Component::Client';
use Params::Validate qw(SCALAR validate_with);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

    my $multipath = NACL::C::Client::Multipath->new( command_interface => $ci );

Return a new Multipath component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'device',
];

sub _primary_keys_validate_spec {
    return (
        'device' => { type => SCALAR, optional => 1 },
    );
}
=head2 enable

    NACL::C::Client::Multipath->enable(command_interface=>$ci,);

(Class method) Request to enable multipath.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_enable_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub enable
sub _enable_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts       = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->chkconfig(service => "multipathd", action => "on");
    $Log->exit() if $may_exit;
}    ##end sub _enable_linux_cli

=head2 disable

    NACL::C::Client::Multipath->disable(command_interface=>$ci,);

(Class method) Request to disable multipath.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disable_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub enable
sub _disable_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts       = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->chkconfig(service => "multipathd", action => "off");
    $Log->exit() if $may_exit;
}    ##end sub _enable_linux_cli

=head2 start

    NACL::C::Client::Multipath->start(command_interface=>$ci,);

(Class method) Request to start multipath daemon.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub start_daemon
sub _start_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts       = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->service('script' => "multipathd", 'action' => "start" );
    $Log->exit() if $may_exit;
}    ##end sub _start_daemon_linux_cli

=head2 stop

    NACL::C::Client::Multipath->stop(command_interface=>$ci,);

(Class method) Request to stop multipath daemon.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_stop_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub stop
sub _stop_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts       = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->service('script' => "multipathd", 'action' => "stop" );
    $Log->exit() if $may_exit;
}    ##end sub _stop_linux_cli

=head2 flush

    NACL::C::Client::Multipath->flush(command_interface=>$ci,device => $device);

(Class method) Request to flush the multipath device entries from /dev/mapper
and destroy the mutlipath maps multipath daemon.

=over

=item Options

=over

=item C<< device => $device >>

(Required) The Device name

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub flush {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "device"           => { type => SCALAR, optional => 1 },
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_flush_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub flush
sub _flush_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts     = $pkg_or_obj->_validate_backend_opts(@args);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my $device = $opts{device};
    my %api_opts;
    if ($device) {
        %api_opts = ('flush' => $device);
    } else {
        %api_opts = ('flush-all' => 1);
    }
    $apiset->multipath(%api_opts);
    $Log->exit() if $may_exit;
}    ##end sub _flush_linux_cli

=head2 verify

    NACL::C::Client::Multipath->verify(command_interface=>$ci,);

(Class method) To verify whether multipath is installed or not

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub verify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_verify_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
        ],
    );
    $Log->exit() if $may_exit;
}    ##end sub verify
sub _verify_linux_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,@args) = @_;
    my %opts     = $pkg_or_obj->_validate_backend_opts(@args);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->rpm('query' => '1',package => 'device-mapper-multipath');
    $Log->exit() if $may_exit;
}    ##end sub _verify_linux_cli
1;
