#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary Ndmpcopy Component Module
## @author gmanish@netapp.com, dl-dpg-nb-automation, dl-nacl-dev;
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::Ndmpcopy

=head1 DESCRIPTION

C<NACL::C::Client::Ndmpcopy> is a derived class of
L<NACL::C::Component::Client|lib-NACL-C-Component-Client-pm>.
It represents ndmpcopy operations on client machine.

=cut

package NACL::C::Client::Ndmpcopy;

use strict;
use warnings;
use NACL::ComponentUtils qw (_optional_scalars);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use base 'NACL::C::Component::Client';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 ndmpcopy

    NACL::C::Client::Ndmpcopy->ndmpcopy(
                   command_interface => $ci,
                   source            => $source,
                   destination       => $destination
                  ...);

(Class method)
This method is used to transfers data between filers/vfilers using the
Network Data Management Protocol (NDMP)

Invokes "ndmpcopy" command for client.

=over

=item Options
=back

=item C<< command_interface => $command_interface >>
(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>


=item C<< apiset_must => $ruleset >>
(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>


=item C<< apiset_should => $ruleset >>
(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< "source" => $source >>
(Required for class method) Source path


=item C<< "destination" => $destination >>
(Required for class method) Destination path


=item C<< "method-timeout" => $scalar >>
(Optional)The new timeout value.


=item C<< "engine" => $string >>
(Optional)The engine to be used for backup /restore


=item C<< "source-filer-authentication" => $string >>
(Required)source filer authentication credentials.


=item C<< "destination-filer-authentication" => $string >>
(Required)Destination filer authentication credentials.


=item C<< "all-authentication" => $string >>
(Optional) set if src and dst has same authentication credentials.


=item C<< "source-filer-authentication-type" => $string >>
(Required) Source filer authentication type as text or md5.


=item C<< "destination-filer-authentication-type" => $string >>
(Required)destination filer authentication type.


=item C<< "all-authentication-type" => $string >>
(Optional)set if source and destination filer authentication type are same.


=item C<< "source-control-connection-mode" => $string >>
(Optional)address mode for control connections on source.


=item C<< "destination-control-connection-mode" => $string >>
(Optional)address mode for control connections on destination.


=item C<< "data-connection-mode" => $string >>
(Optional)address mode for all data and control connections.


=item C<< "backup-null" => $boolean >>
(Optional) Backup to null.


=item C<< "file-history" => $boolean >>
(Optional)Enable file history.


=item C<< "incremental-level" => $string >>
(Optional)Incremental level, where <level> is 0..9. Default is 0.


=item C<< "debug-mode" => $boolean >>
(Optional)Enable debug trace to stdout.


=item C<< "advance-help-message" => $boolean >>
(Optional)Display advanced options.


=item C<< "env-variables" => $string >>
(Optional)Add <var=value> pairs to the NDMP environment.


=item C<< "unicode-file-history" => $boolean >>
(Optional)Enable unicode file history.


=item C<< "print-file-history" => $boolean >>
(Optional)Print file history data.


=item C<< "file-history-interval" => $string >>
(Optional)Frequency of sleep. Default is once per file history entry.


=item C<< "file-history-sleep" => $string >>
(Optional)Specify milliseconds to sleep between file history entries.


=item C<< "help-message" => $boolean >>
(Optional)Display this help message.


=item C<< "source" => $string >>
(Optional)source path.


=item C<< "destination" => $string >>
(Optional)destination path.


=item C<< "no-cab " => $boolean >>
(Optional)Do not use the NDMP CAB extension.
=back

=back

=over


=item Exceptions

=item Return
Retruns the ndmpcopy output

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut



sub ndmpcopy {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars qw(
                engine source-filer-authentication destination-filer-authentication all-authentication
                source-filer-authentication-type  destination-filer-authentication-type all-authentication-type
                source-control-connection-mode destination-control-connection-mode data-connection-mode
                backup-null file-history  incremental-level  debug-mode  advance-help-message env-variables
                unicode-file-history 	print-file-history  file-history-interval  file-history-sleep  help-message
                source destination debug-mode  help-message advance-help-message backup-null file-history print-file-history unicode-file-history no-cab
                ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ndmpcopy_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub ndmpcopy

sub _ndmpcopy_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $pkg->_hash_move(
        source => \%opts,
        target => \%opts,
    );

    my $response = $apiset->ndmpcopy(%opts);
    $Log->exit() if $may_exit;
    return $response;
} ## end sub _ndmpcopy_linux_cli

sub _primary_keys_validate_spec {
    return ();
}

1;
