#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary AutoSupport Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::AutoSupport

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for autosupport operations, such as autosupport history, autosupport history-all,
autosupport invoke, autosupport invoke-deploy-logs, autosupport manifest, autosupport modify, autosupport invoke-host

=for html <hr>

=head SYNOPSIS

Show history of upto 50 most recent autosupports for a cluster:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->history(
    command_interface => $ci,
    cluster_name => "cluster001",
);

Show history of all autosupports:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->history_all(
    command_interface => $ci,
);

Invoke an autosupport:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke(
    command_interface => $ci,
    cluster_name => "cluster001",
    node_name => "node001",
    invoke_type => "troubleshooting",
    username => "admin",
    password => "admin123",
);

Invoke a DEPLOY_LOGS autosupport to collect local logs and state:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke_deploy_logs(
    command_interface => $ci,
);

Get details about an Autosupport:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->manifest(
    command_interface => $ci,
    seq_num => "seq_num_id",
);

Modify the configuration for Autosupport:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->modify(
    command_interface => $ci,
    asup_enabled => "true",
    destination_url => $destination_url,
    proxy_url => $proxy_url,
);

Invoke an autosupport specific to this host:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke_host(
    command_interface => $ci,
    hostid => "host001"
    invoke_type => "troubleshooting",
    message => "invoking_autosupport_host",
    username => "admin",
    password => "admin123",
);

=cut

package NACL::C::Client::ONTAPSelectDeploy::AutoSupport;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           AutoSupport Operation methods
###############################################################################

=head history

Return the history of up to the 50 most recent autosupports for a cluster

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster.

=back

=item Examples

# Show history of upto 50 most recent autosupports for a cluster:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->history(
    command_interface => $ci,
    cluster_name => "cluster001",
);

=back

=cut

sub history {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    cluster_name	=> { type => SCALAR },
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_history",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub history

sub _history {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_history' );
    $Log->exit() if $may_exit;
} ## end sub _history

=head history_all

Return the history of all autosupports

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Return the history of all autosupports
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->history_all(
    command_interface => $ci,
);

=back

=cut

sub history_all {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_history_all",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub history_all

sub _history_all {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_history_all' );
    $Log->exit() if $may_exit;
} ## end sub _history_all

=head invoke

Invoke an autosupport specific to this node

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster.

=item C<< node_name => SCALAR >>

 (Required) Name of the node.

=item C<< invoke_type => SCALAR >>

 (Required) Type of the Autosupport {troubleshooting,performance}

=item C<< username => SCALAR >>

 (Required) Host root user name

=item C<< password => SCALAR >>

 (Required) Password for the root user of the host

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Invoke an autosupport specific to this node
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke(
    command_interface => $ci,
    cluster_name => "cluster001",
    node_name => "node001",
    invoke_type => "troubleshooting",
    username => "admin",
    password => "admin123",
);

=back

=cut

sub invoke {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    cluster_name	=> { type => SCALAR },
	    node_name		=> { type => SCALAR },
	    invoke_type		=> { type => SCALAR },
	    username		=> { type => SCALAR },
            password		=> { type => SCALAR },
	    _optional_scalars(
		qw( case message )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_invoke",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub invoke

sub _invoke {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_invoke' );
    $Log->exit() if $may_exit;
} ## end sub _invoke

=head invoke_deploy_logs

Invoke a DEPLOY_LOGS autosupport to collect local logs and state

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

 (Required) for class method, Not Applicable for instance method)

=item C<< case => SCALAR >>

 (Optional) Case Number to include in the subject of the autosupport

=item C<< message => SCALAR >>

 (Optional) Note to appear in the subject of the autosupport

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Invoke a DEPLOY_LOGS autosupport to collect local logs and state
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke_deploy_logs(
    command_interface => $ci,
);

=back

=cut

sub invoke_deploy_logs {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( case message )
            ),
            help                => { type => BOOLEAN, optional => 1 },
        },
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_invoke_deploy_logs",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub invoke_deploy_logs

sub _invoke_deploy_logs {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
        api => 'autosupport_invoke_deploy_logs' );
    $Log->exit() if $may_exit;
} ## end sub _invoke_deploy_logs

=head manifest

Get details about an Autosupport

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< seq_num => SCALAR >>

 (Required) ID of the Autosupport

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Get details about an Autosupport
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->manifest(
    command_interface => $ci,
    seq_num => "seq_num_id",
);

=back

=cut

sub manifest {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    seq_num		=> { type => SCALAR },
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_manifest",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub manifest

sub _manifest {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_manifest' );
    $Log->exit() if $may_exit;
} ## end sub _manifest

=head modify

Replace the configuration for Autosupport.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< asup_enabled => SCALAR >>

 (Optional) Toggle for the Autosupport feature {True,False}

=item C<< chunk_size => SCALAR >>

 (Optional) Size of each transmission chunk in bytes

=item C<< destination_url  => SCALAR >>

 (Optional) HTTPs PUT URL to reach NetApp Autosupport server

=item C<< proxy_url => SCALAR >>

 (Optional) Proxy authentication settings for HTTPS traffic on the
 local network. ('https://user:password@ip:port')

=item C<< local_collection => SCALAR >>

 (Optional) Allows collection to local storage even if "asup_enabled"==false
 {True,False}

=item C<< retry_send_count => SCALAR >>

 (Optional) Max attempts to send an Autosupport

=item C<< retry_send_interval => SCALAR >>

 (Optional) Seconds between Autosupport send retry attempts

=item C<< sender_threads => SCALAR >>

 (Optional) Number of simultaneous sender tasks to allow

=item C<< validate_digital_certificate  => SCALAR >>

 (Optional) Enables hostname validation for digital certificates
 {True,False}

=item C<< product_company => SCALAR >>

 (Optional) Descriptive string that represents the company or
  organization that runs this instance of ONTAPdeploy

=item C<< product_site => SCALAR >>

 (Optional) Description of the location of this ONTAPdeploy instance

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Modify the configuration for Autosupport:
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->modify(
    command_interface => $ci,
    asup_enabled => "true",
    destination_url => $destination_url,
    proxy_url => $proxy_url,
);

=back

=cut

sub modify {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    _optional_scalars(
		qw( asup_enabled chunk_size destination_url proxy_url local_collection
		    retry_send_count retry_send_interval sender_threads
                    validate_digital_certificate product_company product_site help )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_modify' );
    $Log->exit() if $may_exit;
} ## end sub _modify

=head invoke_host

Invoke an autosupport specific to this host

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

 (Required for class method, Not Applicable for instance method)

=item C<< hostid => SCALAR >>

 (Required) Name of the Host

=item C<< invoke_type => SCALAR >>

 (Required) Type of the Autosupport {troubleshooting,performance}

=item C<< message => SCALAR >>

 (Required) Note to appear in the subject of the autosupport

=item C<< username => SCALAR >>

 (Required) Host root user name

=item C<< password => SCALAR >>

 (Required) Password for the root user of the host

=item C<< case_number => SCALAR >>

 (Optional) Case Number to include in the subject of the autosupport

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Invoke an autosupport specific to this host
NACL::C::Client::ONTAPSelectDeploy::AutoSupport->invoke_host(
    command_interface => $ci,
    hostid => "host001"
    invoke_type => "troubleshooting",
    message => "invoking_autosupport_host",
    username => "admin",
    password => "admin123",
);

=back

=cut

sub invoke_host{
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    hostid        => { type => SCALAR },
	    invoke_type    => { type => SCALAR },
	    message        => { type => SCALAR },
	    _optional_scalars(
		qw( username password case_number )
	    ),
	    help	   => { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_invoke_host",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub invoke_host

sub _invoke_host{
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'autosupport_invoke_host' );
    $Log->exit() if $may_exit;
} ## end sub _invoke_host

1;
