#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary File Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::File

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for File operations, such as file delete and file list.

=for html <hr>

=head SYNOPSIS

Delete the specified file if the user has access from ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::File->delete(
    command_interface => $ci,
    file => "file",
    path => "path",
);

List all files in a specified directory if the user has access from ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::File->list(
    command_interface => $ci,
    path => "path",
);

=head ATTRIBUTES

=head file

Name of the file to delete

=head path

Path to the directory

=cut

package NACL::C::Client::ONTAPSelectDeploy::File;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           File Operation methods
###############################################################################

=head delete

Delete the specified file if the user has access from ONTAP Select Deploy.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< file => SCALAR >>

 (Required) Name of the file to delete

=item C<< path => SCALAR >>

 (Optional) File location directory

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Delete the specified file if the user has access from ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::File->delete(
    command_interface => $ci,
    file => "file",
    path => "path",
);

=back

=cut

sub delete {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    file		=> {type => SCALAR,},
            _optional_scalars(
		qw( path )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'file_delete' );
    $Log->exit() if $may_exit;
} ## end sub _delete

=head list

List all files in a specified directory if the user has access from ONTAP Select Deploy.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< path => SCALAR >>

 (Optional) Path to the directory to list all files

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# List all files in a specified directory if the user has access from ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::File->list(
    command_interface => $ci,
    path => "path",
);

=back

=cut

sub list {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    _optional_scalars(
		qw( path )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_list",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub list

sub _list {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'file_list' );
    $Log->exit() if $may_exit;
} ## end sub _list

1;
