#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Host Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::Host

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for Host operations, such as host add, host configure and host remove.

=for html <hr>

=head SYNOPSIS

Add a new host to ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::Host->add(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
    username => "admin",
    password => "admin123",
    vcenter => "vcenter.rtp.netapp.com",
);

Update the configuration of a host:
NACL::C::Client::ONTAPSelectDeploy::Host->configure(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
    location => "RTP",
    serial_number => "AHDUS161"
    evaluation_version => $eval,
    mgmt_network => $mgmt,
    data_network => $data_network,
);

Delete a Host from ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::Host->remove(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
);

=head ATTRIBUTES

=head hostid

IP address or FQDN of the host.

=cut

package NACL::C::Client::ONTAPSelectDeploy::Host;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           Host Operation methods
###############################################################################

=head add

Add a new host to ONTAP Select Deploy

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< hostid => SCALAR >>

 (Required) IP address or FQDN of the Host to add to ONTAP Select Deploy

=item C<< username => SCALAR >>

 (Required) Username with access to this Host

=item C<< password => SCALAR >>

 (Optional) Password for the provided Username

=item C<< vcenter => SCALAR >>

(Optional) IP address or FQDN of a vCenter if these are vCenter credentials

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

#Add a new host to ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::Host->add(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
    username => "admin",
    password => "admin123",
    vcenter => "vcenter.rtp.netapp.com",
);

=back

=cut

sub add {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    hostid		=> { type => SCALAR,},
            username		=> { type => SCALAR},
            password		=> { type => SCALAR, optional => 1,},
            vcenter		=> { type => SCALAR, optional => 1,},
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_add",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

sub _add {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'host_add' );
    $Log->exit() if $may_exit;
} ## end sub _add

=head configure

Update the configuration of a host.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< hostid => SCALAR >>

 (Required) IP address or FQDN of the host to configure

=item C<< location => SCALAR >>

 (Required) A description of the location of this host.

=item C<< serial_number => SCALAR >>

 (Optional) Serial number to be used for the ONTAP Select node on this host.

=item C<< evaluation_version => SCALAR >>

 (Optional) Evaluation version of the instance

=item C<< storage_pool => SCALAR >>

 (Required) The storage pool to use for when deploying an ONTAP Select node on this host.

=item C<< mgmt_network => SCALAR >>

 (Required) Name of the ONTAP Management Network

=item C<< data_network => SCALAR >>

 (Required) Name of the ONTAP Data Network

=item C<< capacity_limit => SCALAR >>

 (Optional) capacity limit for this storage pool in GB.

=item C<< internal_network => SCALAR >>

 (Optional) Name of the ONTAP Internal Network.

=item C<< internal_vlan_id => SCALAR >>

 (Optional) Vlan ID of the ONTAP Internal Network (only for KVM)

=item C<< mgmt_vlan_id => SCALAR >>

 (Optional) Vlan ID of the ONTAP Management Network (only for KVM)

=item C<< data_vlan_id => SCALAR >>

 (Optional) Vlan ID of the ONTAP Data Network (only for KVM)

=item CC<< instance_type => SCALAR >>

 (Optional) name of the instance type to be used for the VM to be deployed on this Host.

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Update the configuration of a host.
NACL::C::Client::ONTAPSelectDeploy::Host->configure(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
    location => "RTP",
    serial_number => "AHDUS1611"
    evaluation_version => $eval,
    mgmt_network => "ONTAP_Management",
    data_network => $data_network,
    capacity_limit => "100GB",
    internal_network => "ONTAP-Internal",
    instance_type => "small",
);

=back

=cut

sub configure {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    hostid		=> { type => SCALAR,},
            location		=> { type => SCALAR,},
            serial_number	=> { type => SCALAR,optional => 1},
	    storage_pool	=> { type => SCALAR,},
            evaluation_version  => { type => SCALAR,optional => 1},
            mgmt_network	=> { type => SCALAR,},
            data_network	=> { type => SCALAR,},
	    _optional_scalars(
		qw( capacity_limit internal_network
		    internal_vlan_id mgmt_vlan_id
		    data_vlan_id instance_type )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_configure",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub configure

sub _configure {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'host_configure' );
    $Log->exit() if $may_exit;
} ## end sub _configure

=head remove

Delete a Host from ONTAP Select Deploy

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< hostid => SCALAR >>

 (Required) IP address or FQDN of the Host to delete from ONTAP Select Deploy

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Delete Host :
NACL::C::Client::ONTAPSelectDeploy::Host->remove(
    command_interface => $ci,
    hostid => "sti-ctl001.gdl.englab.netapp.com",
);

=back

=cut

sub remove {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    hostid		=> { type => SCALAR },
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_remove",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'host_remove' );
    $Log->exit() if $may_exit;
} ## end sub _remove

1;
