#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary  UplinkMonitor Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for uplink monitor operations, such as uplink monitor create, uplink monitor create-using-json,
uplink monitor disable, uplink monitor enable, uplink monitor modify.

=for html <hr>

=head SYNOPSIS

Create Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->create(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
    snmp_username => "admin",
    snmp_password => "admin1",
);

Create Uplink Monitor using JSON file:
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->create_using_json(
    command_interface => $ci,
    json_file => "abc.json",
    path => "/usr/bin",
);

Disable Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->disable(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
);

Enable Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->enable(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
);

Modify Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->modify(
    command_interface => $ci,
    cluster_name => "cluster001",
    node_name => "node001",
    ontap_username => "admin",
    ontap_password => "admin1",
    snmp_username => "admin",
    snmp_password => "admin1",
);

=head ATTRIBUTES

=cut

package NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

###############################################################################
#           Uplink Monitor Operation methods
###############################################################################

=head create

This will create an uplink monitor on all nodes in an ONTAP Select cluster.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster

=item C<< ontap_username => SCALAR >>

 (Required) ONTAP Select administrative user

=item C<< ontap_password => SCALAR >>

 (Required) ONTAP Select administrative password

=item C<< snmp_username => SCALAR >>

 (Required) SNMP username configured on hypervisor host

=item C<< snmp_password => SCALAR >>

 (Required) SNMP password configured on hypervisor host

=back

=item Examples

# Create Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->create(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
    snmp_username => "admin",
    snmp_password => "admin1",
);

=back

=cut

sub create {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    cluster_name    => { type => SCALAR },
	    ontap_username  => { type => SCALAR },
	    ontap_password  => { type => SCALAR },
	    snmp_username   => { type => SCALAR },
	    snmp_password   => { type => SCALAR },
	    help	    => { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'uplink_monitor_create' );
    $Log->exit() if $may_exit;
} ## end sub _create

=head create_using_json

Create the uplink monitor using input JSON.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< json_file => SCALAR >>

 (Required) Name of the file containing uplink monitor create JSON configuration

=item C<< path => SCALAR >>

 (Optional) Path to uplink monitor create JSON file location (Env:Home)

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Create Uplink Monitor using JSON file:
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->create_using_json(
    command_interface => $ci,
    json_file => "abc.json",
    path => "/usr/bin",
);

=back

=cut

sub create_using_json {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    json_file		=> { type => SCALAR },
	    path                => { type => SCALAR, optional => 1 },
            help                => { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_using_json",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create_using_json

sub _create_using_json {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'uplink_monitor_create_using_json' );
    $Log->exit() if $may_exit;
} ## end sub _create_using_json

=head disable

Disable the uplink monitor

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster

=item C<< ontap_username => SCALAR >>

 (Required) ONTAP Select administrative user

=item C<< ontap_password => SCALAR >>

 (Required) ONTAP Select administrative password

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Disable Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->disable(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
);

=back

=cut

sub disable {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    cluster_name	=> { type => SCALAR },
	    ontap_username      => { type => SCALAR },
	    ontap_password      => { type => SCALAR },
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disable",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub disable

sub _disable {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'uplink_monitor_disable' );
    $Log->exit() if $may_exit;
} ## end sub _disable

=head enable

Enable the uplink monitor

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster

=item C<< ontap_username => SCALAR >>

 (Required) ONTAP Select administrative user

=item C<< ontap_password => SCALAR >>

 (Required) ONTAP Select administrative password

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Enable Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->enable(
    command_interface => $ci,
    cluster_name => "cluster001",
    ontap_username => "admin",
    ontap_password => "admin1",
);

=back

=cut

sub enable {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            cluster_name        => { type => SCALAR },
            ontap_username      => { type => SCALAR },
            ontap_password      => { type => SCALAR },
            help                => { type => BOOLEAN, optional => 1 },
        },
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_enable",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub enable

sub _enable {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
        api => 'uplink_monitor_enable' );
    $Log->exit() if $may_exit;
} ## end sub _enable

=head modify

Modify the uplink monitor.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< cluster_name => SCALAR >>

 (Required) Name of the cluster

=item C<< node_name => SCALAR >>

 (Required) Name of the node

=item C<< ontap_username => SCALAR >>

 (Required) ONTAP Select administrative user

=item C<< ontap_password => SCALAR >>

 (Required) ONTAP Select administrative password

=item C<< snmp_username => SCALAR >>

 (Required) SNMP username configured on hypervisor host

=item C<< snmp_password => SCALAR >>

 (Required) SNMP password configured on hypervisor host

=back

=item Examples

# Modify Uplink Monitor
NACL::C::Client::ONTAPSelectDeploy::UplinkMonitor->modify(
    command_interface => $ci,
    cluster_name => "cluster001",
    node_name => "node001",
    ontap_username => "admin",
    ontap_password => "admin1",
    snmp_username => "admin",
    snmp_password => "admin1",
);

=back

=cut

sub modify {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            cluster_name    => { type => SCALAR },
	    node_name       => { type => SCALAR },
            ontap_username  => { type => SCALAR },
            ontap_password  => { type => SCALAR },
            snmp_username   => { type => SCALAR },
            snmp_password   => { type => SCALAR },
            help            => { type => BOOLEAN, optional => 1 },
        },
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
        api => 'uplink_monitor_modify' );
    $Log->exit() if $may_exit;
} ## end sub _modify

1;
