#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Component for File Operations on Remote Machine
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::Remote::File

=head1 DESCRIPTION

This library is used to perform basic file system level operations
on a remote client such as open, close, write, read etc.These
methods work for linux,solaris and windows clients.

It builds on top of C<NACL::C::Client::File>.
This means that any functionality present in
L<NACL::C::Client::File|lib-NACL-C-Client-File-pm> will also be
present in this Component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required, isa 'NACL::C::Component')
A command interface object which represents the client on which we are
manipulating the mount points.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>.

=head2 file_name

(SCALAR) Name/Absolute path of the file to be created

=head2 file_obj
This is the file object that wrap the NACL::C::Client::File object created
in the remote process and we can perform all the file related operation 
using this obejct.

=head2 worker_obj
This is worker object representing the process started in the remoted
client through which remote file operations are performed.

=cut

package NACL::C::Client::Remote::File;
use strict;
use warnings;

use Params::Validate
    qw(validate validate_with SCALAR HASHREF OBJECT SCALARREF);
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use base 'NACL::C::Component';
use NATE::BaseException qw(:try);
use NACL::STask::Client::RPC::Worker;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    new => [ '-hash', '-init', 'new' ],
    scalar => [ { -type => 'NACL::C::Client' }, 'command_interface', ],
    scalar => 'file_name',
    scalar => 'file_obj',
    scalar => 'worker_obj',
];

=head1 METHODS

=head2 open()

Open (or create) a file on remote machine(Class Method)

It returns Client::Remote::File object that contains the following attributes
of the file -

file_obj   => This is the file object that wrap the NACL::C::Client::File object
              and we can perform all the file related operation using this obejct
              that mentioned in L<NACL::C::Client::File|lib-NACL-C-Client-File-pm>

worker_obj => This is worker object that can be use to perform all the
              operations on client mentioned in 
              L<NACL::STask::Client::RPC::Worker|lib-NACL-STask-Client-RPC-Worker-pm>

=over

=item Options

=over

=item  C<< command_interface => $ci >>
(Required)
This is component object representing the client in which the file has
to be created.

=item C<< file_name => $file_name >>
(Required) The absolute or relative path of the file to be open

=item  C<< arguments => $args >>
(Required, HASHREF)
It accepts all the parameters accepted by NACL::C::Client::File->create()

=item Example

 my $fh_obj = NACL::C::Client::Remote::File->open(
     command_interface => $cmd_interface,
     file_name => $file_name,
     arguments => {
         access_mode => read,
         open_flags => 'create-new',
         share_mode  => 'shared-read',
         attributes => 'FILE_FLAG_DELETE_ON_CLOSE'
     }
 );

=back

=back

=cut

sub open {

    my $pkg = shift;
    $Log->enter() if $may_enter;
    my ($worker);
    my %opts = $pkg->_common_validate_with(
        params             => \@_,
        only_static_method => 1,
        additional_spec    => {
            command_interface => { type => OBJECT, isa => 'NACL::C::Client' },
            file_name => { type => SCALAR },
            arguments => { type => HASHREF },
        },
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $client            = $command_interface->hostrec();

    my $file_args = delete $opts{arguments};
    $file_args->{file_name} = $opts{file_name};
    my @args;
    map { push @args, ( $_, $file_args->{$_} ) } keys %$file_args;

    $worker = NACL::STask::Client::RPC::Worker->new( client => $client );

    $worker->start();
    $worker->call( eval => 'use NACL::C::Client::File' );
    my $file_obj = $worker->call(
        package     => 'NACL::C::Client::File',
        method      => 'open',
        args        => \@args,
        wrap_return => 1
    );

    my $self = $pkg->new(
        command_interface => $command_interface,
        file_name         => $opts{file_name},
        file_obj          => $file_obj,
        worker_obj        => $worker
    );

    $Log->debug( sub { "FILEOBJ -> " . Dumper( $self->file_obj() ) } );
    $Log->exit() if $may_exit;
    return $self;

} ## end sub open

=head2 Other Methods

=over

 The rest of the file operation can be done by as given below -
 $fh_obj->rename()
 $fh_obj->close()
 $fh_obj->delete()
 $fh_obj->move()
 $fh_obj->write()

For other file operations check L<NACL::C::Client::File|lib-NACL-C-Client-File-pm>

The method from worker can be call as given below -
 $fh_obj->worker_obj()->method()

=back

=cut

# DESTROY method to kill the worker object
# whenever remote file object goes out of scope, automatic cleanup
# happens for the process started in the remote client
sub DESTROY {
    my $self = shift;
    $Log->trace( sub {"Kill the worker object"} );
    local $@; # preserve errors
    $self->worker_obj()->kill() if ( $self->worker_obj_isset );
}

sub can {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my @opts       = $pkg_or_obj->_can_validation(@_);
    $Log->debug( sub { "OPTS after can_validate -> " . Dumper( \@opts ) } );
    my $code;
    my $method    = $opts[0];
    my $error_ptr = $opts[1];

    if ( $code = $pkg_or_obj->NACL::C::Component::can(@opts) ) {
        $Log->exit() if $may_exit;
        return $code;
    } else {

        $code = sub {
            $Log->debug( sub { "OPTS in Code ref -> " . Dumper( \@_ ) } );

            my ( $obj, %args ) = @_;
            $Log->debug( sub {"Method -> $method"} );
            $obj->file_obj()->$method(%args);
        };
        $Log->exit() if $may_exit;
        return $code;
    } ## end else [ if ( $code = $pkg_or_obj...)]
    $Log->exit() if $may_exit;
    return undef;
} ## end sub can

1;
