#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Client/UserGroup Module
## @author  dl-nacl-dev@netapp.com
## @status public

package NACL::C::Client::UserGroup;

use strict;
use warnings;
use NACL::CS::Client::VG;
use NACL::ComponentUtils qw(_optional_scalars );
use base 'NACL::C::Component::Client';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw( SCALAR OBJECT BOOLEAN );
use NACL::C::Exceptions::Client::UserGroup::UserGroupException qw(:try);

use Class::MethodMaker [
    scalar => 'group',
];

=head2 new

Get a new instance of NACL::C::Client::UserGroup.

=over

=item Usage

$clientutils = NACL::C::Client::UserGroup->new(
    command_interface => $ci,
);

=item Parameters

=item C<<command_interface>>

(Instance method)
Required. Client command interface.

=item Return

<OBJECT> : A NACL::C::Client::UserGroup object.

=back

=cut

=head2 add

    NACL::C::Client::UserGroup->add(
         command_interface => $client,
         group              => $group,
    );

    $cu = NACL::C::Client::UserGroup->new(command_interface => $ci);
    $cu->add(group => $group);

(Class or Instance method)
This method is used to add a local group on the client by  giving group group.
Give warning message if the group exists already.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< group => $group >>

(Required) Local group group on the client.

=item C<< force => $force >>

(Optional) forces the command to exit with success status
in case when group already exists

=item C<< non-unique => $non_unique >>

(Optional) add a group with a non-unique GID.

=item C<< gid => $gid >>

(Optional) Local group id on the client. This must be unique

=item C<< password => $password >>

(Optional) The encrypted password as returned by the crypt.

=item C<< system => $system >>

(Optional) creates a system group

=item Return

Returns the object of type UserGroup with the name as its attributes.

=back

=cut

sub add {
   my ( $pkg, @args ) = @_;
   $pkg->_frontend_log_debug_opts(@args);
   my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            group => { type => SCALAR },
            force => { type => BOOLEAN, optional => 1  },
	    'non-unique' => { type => BOOLEAN, optional => 1},
	    _optional_scalars(
                qw (
                    gid password system
                )
            ),
        },
    );
    my $group = $opts{group};
    my $command_interface = $opts{command_interface};
    try{
      $pkg->call_on_apiset(
         %opts,
         choices => [
             {
                method    => '_add_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
             },
         ],
      );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
		$Log->exit() if $may_exit;
        NACL::C::Exceptions::Client::UserGroup::UserGroupException->detect_convert_and_throw(
            exception => $exception
        );
    };

    return $pkg->new(
        group       => $group,
        command_interface => $command_interface,
    );
	$Log->exit() if $may_exit;
}   ##end sub add

sub _add_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    $pkg->_base_backend_client(
        @args,
        api => 'groupadd',
    );

    $Log->exit() if $may_exit;
}
=head2 modify

    NACL::C::Client::UserGroup->modify(
         command_interface => $client,
         group              => $group,
         groups            => [$group1, ...],
    );

    $cu = NACL::C::Client::UserGroup->new(command_interface => $ci);
    $cu->modify(
         group              => $group,
         groups            => [$group1, ...],
    );

(Class or Instance method)
This method is used to modify user groups for a local user on localhost.
An exception is thrown if no groups given or any given group does not exist.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< new-name => $name >>

(Required) new group name on the client.

=item C<<gid>>

(Optional). The Group id of the UserGroup.

=item C<< group => $group >>

(Required) Local group name on the client.

=item C<< force => $force >>

(Optional) forces the command to exit with success status
in case when group already exists

=item C<< non-unique => $non_unique >>

(Optional) add a group with a non-unique GID.

=item C<< password => $password >>

(Optional) The encrypted password as returned by the crypt.

=item C<< system => $system >>

(Optional) creates a system group

=item Return

Returns the object of type UserGroup with the name as its attributes.

=item Return

=back

=cut

sub modify {
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'new-name'    => { type => SCALAR },
            'non-unique' => { type => BOOLEAN, optional => 1},
	     group    => { type => SCALAR, optional => 1 },
	     _optional_scalars(
                qw (
                    gid password system
                )
            ),
        },
    );
    my $newname = $opts{'new-name'};
    my $command_interface = $opts{command_interface};
    $opts{group} //= $pkg->group();

    try{
      $pkg->call_on_apiset(
          %opts,
          choices => [
             {
                method    => '_modify_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
             },
          ],
      );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
		$Log->exit() if $may_exit;
        NACL::C::Exceptions::Client::UserGroup::UserGroupException->detect_convert_and_throw(
            exception => $exception
        );

    };

    return $pkg->new(
        group       => $newname,
        command_interface => $command_interface,
    );
    $Log->exit() if $may_exit;
}    ##end sub modify

sub _modify_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    $pkg->_base_backend_client(
        @args,
        api => 'groupmod',
    );

    $Log->exit() if $may_exit;
}
=head2 delete

    NACL::C::Client::UserGroup->delete(
         command_interface => $client,
         group              => $group,
    );

    $cu = NACL::C::Client::UserGroup->new(command_interface => $ci);
    $cu->delete(group => $group);

(Class or Instance method) This method is used to delete a local group on the
client by giving group name. Give warning message if the group does not exist.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required) The ( NACL::C::Client object) client.

=item C<< group => $group >>

(Required) Local group name on the client.

=item Return

=back

=cut

sub delete {
   my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            group => { type => SCALAR, optional => 1},
        },
    );

	$opts{group} //= $pkg->group();

    try{
      $pkg->call_on_apiset(
         %opts,
         choices => [
             {
               method    => '_delete_linux_cli',
               interface => 'CLI',
               set       => 'Linux',
             },
         ],
      );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
		$Log->exit() if $may_exit;
        NACL::C::Exceptions::Client::UserGroup::UserGroupException->detect_convert_and_throw(
            exception => $exception
        );
    };
    $Log->exit() if $may_exit;

}    ##end sub delete

sub _delete_linux_cli {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;

    $pkg->_base_backend_client(
        @args,
        api => 'groupdel',
    );

    $Log->exit() if $may_exit;
}

=head2 get_groupid

    $gid = NACL::C::Client::UserGroup->get_groupid(
         command_interface => $client,
         group              => $group,
    );

    $cu = NACL::C::Client::UserGroup->new(command_interface => $ci);
    $gid = $cu->get_groupid(group => $group);

(Class or Instance method)
This method is used to get a local group id on the client by giving group name.
Throws BaseException error if the local group does not exist on the client.

Currently it is implemented to support linux client only. It will throw
exception for non-linux client.

<TODO>: Add non-linux client support and handle exception respectively.

=over

=item  Options

=over

=item C<< command_interface => $ci >>

(Required for class method) The ( NACL::C::Client object) client.

=item C<< group => $group >>

(Required) Local group name on the client.

=item Return

<SCALAR>: Group ID.

=back

=cut

sub get_groupid {
   my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            group => { type => SCALAR},
        },
    );
    my $gid;
    try{
      $gid = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_get_groupid_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
		$Log->exit() if $may_exit;
        NACL::C::Exceptions::Client::UserGroup::UserGroupException->detect_convert_and_throw(
            exception => $exception
        );
    };

    $Log->exit() if $may_exit;
    return $gid;
}    ##end sub get_groupid

sub _get_groupid_linux_cli {

    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my %opts = $pkg->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $group = $opts{group} // $pkg->group();

    my $response = $command_interface->apiset->getent_group(group => $group);

    my $gid  = $response->get_parsed_output()->[0]->{'group_id'};

    $Log->exit() if $may_exit;
    return int($gid);

} ## end sub _get_groupid_linux_cli
1;

