#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary VG Component Module
## @author dl-nacl-dev@netapp.com, praveent@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::VG

=head1 DESCRIPTION

This library is used to perform operations of creating and deleting Volume/
group on the hosts.

=head1 METHODS

=head2 new

  my $obj = NACL::C::Client::VG->new(  command_interface  => $ci,
                                       'vg_name'          => $vgname,
                                       'pv_name'          => $device);

Return a new VG component object with the given attributes.

=cut

package NACL::C::Client::VG;

use strict;
use warnings;
use NACL::CS::Client::VG;
use NACL::ComponentUtils qw(_optional_scalars );
use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    scalar => 'vg_name',
];

=head2 create()

 my $obj = NACL::C::Client::VG->create(command_interface   => $ci,
                                         'vg_name'         => $vgname,
                                         'pv_name'         => $device);

(Class method) Request creation of a VG (volume group)  element, and return a new
component object that refers to it.

Uses Linux-CLI APISet.

Invokes "vgcreate" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'pv_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device which needs to be converted to physical-volumes.

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the volume group  which needs to be created from physical-volumes.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional) autobackup option.

=item C<< 'force' => $boolean >>

Applicable for Linux-CLI.

(Optional) force option.

=item C<< 'max-logical-volumes' => $logical-volumes >>

Applicable for Linux-CLI.

(Optional) max-logical-volumes option.

=item C<< 'max-physical-volumes' => $physical-volumes >>

Applicable for Linux-CLI.

(Optional) max-physical-volumes option.

=item C<< 'physical-extent-size' => $extent-size >>

Applicable for Linux-CLI.

(Optional) physical-extent-size option.

=item C<< 'volume-group-name' => $volume-group >>

Applicable for Linux-CLI.

(Optional) volume-group-name option.

=item C<< 'physical-device-path' => $physical-device >>

Applicable for Linux-CLI.

(Optional) physical-device-path option.

=item C<< 'clustered' => $clustered >>

Applicable for Linux-CLI.

(Optional) clustered option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'help' => $boolean >>

Applicable for Linux-CLI.

(Optional) help option.

=item C<< 'meta-datatype' => $datatype >>

Applicable for Linux-CLI.

(Optional) meta-datatype option.

=item C<< 'vgmeta-datacopies' => $datacopies >>

Applicable for Linux-CLI.

(Optional) vgmeta-datacopies option.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional) test option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=item C<< 'version' => $boolean >>

Applicable for Linux-CLI.

(Optional) version option.

=item C<< 'yes' => $boolean >>

Applicable for Linux-CLI.

(Optional) yes option.

=item C<< 'zero' => $zero >>

Applicable for Linux-CLI.

(Optional) zero option.

=item C<< 'label-sector' => $label >>

Applicable for Linux-CLI.

(Optional) label-sector option.

=item C<< 'metadata-size' => $metadata >>

Applicable for Linux-CLI.

(Optional) metadata-size option.

=item C<< 'pvmetadata-copies' => $pvmetadata >>

Applicable for Linux-CLI.

(Optional) pvmetadata-copies option.

=item C<< 'data-alignment' => $data >>

Applicable for Linux-CLI.

(Optional) data-alignment option.

=item C<< 'data-alignment-offset' => $offset >>

Applicable for Linux-CLI.

(Optional) data-alignment-offset option.

=item C<< 'addtag' => $addtag >>

Applicable for Linux-CLI.

(Optional) addtag option.

=item C<< 'allocation-policy' => $policy >>

Applicable for Linux-CLI.

(Optional) allocation-policy option.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            pv_name => { type => SCALAR },
            _optional_scalars(
                qw(
                  autobackup force max-logical-volumes max-physical-volumes
                  physical-extent-size clustered debug help meta-datatype
                  vgmeta-datacopies test verbose version yes zero label-sector
                  metadata-size pvmetadata-copies data-alignment
                  data-alignment-offset addtag allocation-policy
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_create_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub create

sub _create_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
              autobackup force max-logical-volumes max-physical-volumes
              physical-extent-size clustered debug help meta-datatype
              vgmeta-datacopies test verbose version yes zero label-sector
              metadata-size pvmetadata-copies data-alignment
              data-alignment-offset addtag allocation-policy
              )
        ],
        map => {
            pv_name => 'physical-device-path',
            vg_name => 'volume-group-name'
        },
    );
    $apiset->vgcreate(%linux_args);

    my $obj = $pkg->new(
        command_interface => $command_interface,
        'vg_name'         => $opts{'vg_name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_linux_cli

=head2 remove()

 NACL::C::Client::VG->remove(command_interface   => $ci,
                                'vg_name'        => $vgname,);

                                           or
 $vg_obj->remove(...);

(Class method or Instance method) Request deletion of a VG (volume group)  element, and deletes
component object that refers to it.

Uses Linux-CLI APISet.

Invokes "vgremove" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device which needs to be removed from physical-device-path.

=item C<< 'force' => $boolean >>

Applicable for Linux-CLI.

(Optional) force option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional) test option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                  debug force test verbose
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_remove_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub remove

sub _remove_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
              debug force test verbose
              )
        ],
        map => { vg_name => 'volume-group-name' },
    );
    $apiset->vgremove(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_linux_cli

=head2 change()

 NACL::C::Client::VG->change(command_interface    => $ci,
                                available         => "n",
                                'vg_name'         => $vgname,);

                                           or
 $vg_obj->change(...);

(Class method or Instance method) Request change attributes of a VG (volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgchange" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'ignore-monitoring' => $boolean >>

Applicable for Linux-CLI.

(Optional) ignore-monitoring option.

=item C<< 'monitor' => $monitor >>

Applicable for Linux-CLI.

(Optional) monitor option.

=item C<< 'uuid' => $boolean >>

Applicable for Linux-CLI.

(Optional) uuid option.

=item C<< 'refresh' => $boolean >>

Applicable for Linux-CLI.

(Optional) refresh option.

=item C<< 'resizeable' => $size >>

Applicable for Linux-CLI.

(Optional) resizeable option.

=item C<< 'disable-opening-of-lv' => $size >>

Applicable for Linux-CLI.

(Optional) disable-opening-of-lv option.

=item C<< 'disable-sync-of-stale-physical-extents' => $size >>

Applicable for Linux-CLI.

(Optional) disable-sync-of-stale-physical-extents option.

=item C<< 'activate-vg-if-pv-are-available' => $size >>

Applicable for Linux-CLI.

(Optional) activate-vg-if-pv-are-available option.

=item C<< 'clustered' => $cluster >>

Applicable for Linux-CLI.

(Optional) clustered option.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional) autobackup option.

=item C<< 'available' => $value >>

Applicable for Linux-CLI.

(Optional) available option.

=back

=cut

sub change {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    ignore-monitoring monitor uuid refresh resizeable disable-opening-of-lv
                    disable-sync-of-stale-physical-extents activate-vg-if-pv-are-available
                    clustered autobackup available
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_change_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub change

sub _change_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                 ignore-monitoring monitor uuid refresh resizeable disable-opening-of-lv
                 disable-sync-of-stale-physical-extents activate-vg-if-pv-are-available
                 clustered autobackup available
              )
        ],
        map => { vg_name => 'vg-name' },
    );

    $apiset->vgchange(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _change_linux_cli

=head2 vgimport()

 NACL::C::Client::VG->vgimport(command_interface  => $ci,
                                'vg_name'         => $vgname,);

                                        or
 $vg_obj->vgimport(...);

(Class method or Instance method) Request to import VG(volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgimport" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'all' => $boolean >>

Applicable for Linux-CLI.

(Optional) all option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=back

=cut

sub vgimport {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    all debug verbose
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_vgimport_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub vgimport

sub _vgimport_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                 all debug verbose
              )
        ],
        map => { vg_name => 'vg-name' },
    );

    $apiset->vgimport(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _vgimport_linux_cli

=head2 vgexport()

 NACL::C::Client::VG->vgexport(command_interface => $ci,
                                'vg_name'        => $vgname,);

                                 or
 $vg_obj->vgexport(...);

(Class method or Instance method) Request to export VG(volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgexport" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $volumes >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'all' => $boolean >>

Applicable for Linux-CLI.

(Optional) all option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=back

=cut

sub vgexport {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    all debug verbose
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_vgexport_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub vgexport

sub _vgexport_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                 all debug verbose
              )
        ],
        map => { vg_name => 'vg-name' },
    );

    $apiset->vgexport(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _vgexport_linux_cli

=head2 rename()

 my $obj = NACL::C::Client::VG->rename(command_interface  => $ci,
                                        'vg_name'         => "/dev/$vg_name",
                                        'new-vgname'      => "/dev/$vg_name",);

                                           or
 $obj->rename(...);

(Class method or Instance method) Request to rename of VG(volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgrename" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $new-vgname >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional) autobackup option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional) test option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=item C<< 'old-vgname' => $old-vgname >>

Applicable for Linux-CLI.

(Optional) old-vgname option.

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    autobackup debug test verbose new-vgname
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_rename_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub rename

sub _rename_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                 autobackup debug test verbose new-vgname
              )
        ],
        map => { vg_name => 'old-vgname' },
    );

    $apiset->vgrename(%linux_args);
    my $obj = $pkg->new(
        command_interface => $command_interface,
        'vg_name'         => $opts{'new-vgname'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _rename_linux_cli

=head2 add_device()

 NACL::C::Client::VG->add_device(command_interface => $ci,
                                  'vg_name'        => $vgname,
                                  'pv-name'        => $pv_name,);

                                           or
 $obj->add_device(...);

(Class method or Instance method) Request to add physical volume to VG(volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgextend" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $new-vgname >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional) autobackup option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'restoremissing' => $boolean >>

Applicable for Linux-CLI.

(Optional) restoremissing option.

=item C<< 'force' => $boolean >>

Applicable for Linux-CLI.

(Optional) force option.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional) test option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=item C<< 'pv-name' => $pv-name >>

Applicable for Linux-CLI.

(Optional) pv-name option.

=back

=cut

sub add_device {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    autobackup debug restoremissing force test verbose pv-name
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_add_device_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub add_device

sub _add_device_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                    autobackup debug restoremissing force test verbose pv-name
              )
        ],
        map => { vg_name => 'vg-name' },
    );

    $apiset->vgextend(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _add_device_linux_cli

=head2 remove_device()

 NACL::C::Client::VG->remove_device(command_interface => $ci,
                                     'vg_name'        => $vgname,
                                     'pv-name'        => $pv_name,);

                                           or
 $obj->remove_device(...);

(Class method or Instance method) Request to remove physical device from VG(volume group)  element .

Uses Linux-CLI APISet.

Invokes "vgreduce" command for Linux CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< 'vg_name' => $new-vgname >>

Applicable for Linux-CLI.

(Required): Name of the device whose attributes needs to be changed.

=item C<< 'autobackup' => $backup >>

Applicable for Linux-CLI.

(Optional) autobackup option.

=item C<< 'all' => $backup >>

Applicable for Linux-CLI.

(Optional) all option.

=item C<< 'debug' => $boolean >>

Applicable for Linux-CLI.

(Optional) debug option.

=item C<< 'removemissing' => $boolean >>

Applicable for Linux-CLI.

(Optional) removemissing option.

=item C<< 'test' => $boolean >>

Applicable for Linux-CLI.

(Optional) test option.

=item C<< 'verbose' => $boolean >>

Applicable for Linux-CLI.

(Optional) verbose option.

=item C<< 'pv-name' => $pv-name >>

Applicable for Linux-CLI.

(Optional) pv-name option.

=back

=cut

sub remove_device {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(
                    all autobackup debug removemissing test verbose pv-name
                  )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_remove_device_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub remove_device

sub _remove_device_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [
            qw (
                    all autobackup debug removemissing test verbose pv-name
              )
        ],
        map => { vg_name => 'vg-name' },
    );

    $apiset->vgreduce(%linux_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_device_linux_cli

sub _primary_keys_validate_spec {
    return ( vg_name => { type => SCALAR }, );
} ## end sub _primary_keys_validate_spec
1;
