# $Id: $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Client VLAN Operations Module
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Client::Vlan

=head1 DESCRIPTION

This library vlan interface is used to perform create and delete operations
on client.

=head1 METHODS

=head2 new

  my $obj = NACL::C::Client::Vlan->new(command_interface => $ci,
                                       'interface'       => $Interface,
                                       'vlan-id'         => $Vlan_Id,);

Return a new Vlan component object with the given attributes.

=cut

package NACL::C::Client::Vlan;

use strict;
use warnings;
use NACL::ComponentUtils qw(_optional_scalars );
use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with SCALAR );
use NACL::APISet::Exceptions::ResponseException qw/:try/;
use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::Client::Vlan::AlreadyExists ();
use NACL::C::_Mixins::Vlan qw(:all);

use Class::MethodMaker [
    scalar => 'vlan_id',
    scalar => 'vlan_interface',
    scalar => 'interface',
];

=head2 create()

 my $obj = NACL::C::Client::Vlan->create(command_interface => $ci,
                                         'interface'       => $Interface,
                                         'vlan-id'         => $Vlan_Id,);

(Class method) Request creation of a new Vlan  element, and  return a new
component object that refers to it.
Uses Linux-CLI or Solaris-CLI APISet.

=over

=item Options

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Client|lib-NACL-C-Component-pm/apiset_should>

=item C<< interface => $interface-name >>

Applicable for Linux-CLI and Solaris-CLI.

(Required): Name of interface of which vlan name to be created.

=item C<< vlan-id => scalar >>

Applicable for Linux-CLI and Solaris-CLI.

(Required): Name of vlan-id of which vlan name to be created.

=item C<< vlan-interface => $vlan_interface >>

Applicable for Linux-CLI and Solaris-CLI.

(Optional) Name of vlan-interface to create.

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            interface => { type => SCALAR },
            'vlan-id' => { type => SCALAR },
            "inet6"   => { type => SCALAR, default => 1 },
        },
    );
    my $obj;
    try {
            $obj = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_linux_cli',
                    interface => 'CLI',
                    set       => 'Linux',
                },
                {   method    => '_create_solaris_cli',
                    interface => 'CLI',
                    set       => 'Solaris',
                },
            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Client::Vlan::AlreadyExists->detect_convert_and_throw(
            exception => $exception);
        if ($exception->text() =~ /Invalid argument/) {
            # Might be that the interface doesn't exist or vlan-interface
            # doesn't exist. Determine which one is it.
            my $vlan_interface = $pkg->_get_vlan_interface(%opts);
            my $found = $pkg->find(
                command_interface => $opts{command_interface},
                filter            => {'vlan-interface' => $vlan_interface},
                allow_empty       => 1
                );
            if ($found) {
                # Vlan-interface already exists.
                NACL::C::Exceptions::Client::Vlan::AlreadyExists->convert_and_throw(
                    exception => $exception);
            } else {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        }
    };

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub create

sub _create_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %linux_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%linux_args,
        copy   => [qw ( vlan-id )],
        map    => { 'interface' => 'interface-name' }
    );

    $apiset->vconfig_add( %linux_args, 'privilege-level' => "root" );

    # bring up the newly created vlan interface
    my $vlan_interface = $pkg->_get_vlan_interface(%opts);

    $apiset->ifconfig(
        interface         => $vlan_interface,
        up                => 1,
        'privilege-level' => "root"
    );


    my $obj = $pkg->new(
        command_interface => $command_interface,
        'interface'       => $linux_args{'interface-name'},
        'vlan-interface'  => $vlan_interface,
        'vlan-id'         => $linux_args{'vlan-id'},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_linux_cli

sub _create_solaris_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %solaris_args;
    my ( $vlan_id, $interface, $vlan_interface );

    $solaris_args{'interface'} = $pkg->_get_vlan_interface(%opts);

    $apiset->ifconfig( %solaris_args, plumb => "1", up => "1" );
    $apiset->ifconfig( %solaris_args, up => "1" );

    if ( defined( $opts{inet6} ) && $opts{inet6} ) {
        $apiset->ifconfig(
            %solaris_args,
            inet6 => "1",
            plumb => "1",
            up    => "1"
        );
        $apiset->ifconfig( %solaris_args, inet6 => "1", up => "1" );
    }
    my $obj = $pkg->new(
        command_interface => $command_interface,
        'interface'       => $opts{'interface'},
        'vlan-interface'  => $solaris_args{'interface'},
        'vlan-id'         => $opts{'vlan-id'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub _create_solaris_cli

=head2 delete()

 my $obj = NACL::C::Client::Vlan->delete(command_interface => $Client,
                                         'vlan-name'       => $Interface.".".$Vlan_Id,
                                         );

(Class or instance method) This method deletes an vlan name element.
Uses Linux-CLI or Solaris-CLI APISet.

=over

=item Options

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
(Required) See L<NACL::C::Client|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vlan-id => $vlan_id >>

Applicable for Linux-CLI and Solaris-CLI.

(Required): Name of vlan-id to delete.

=item C<< interface => $interface >>

Applicable for Linux-CLI and Solaris-CLI.

(Required): Name of interface to delete.

=item C<< vlan-interface => $vlan_interface >>

Applicable for Linux-CLI and Solaris-CLI.

(Optional) Name of vlan-interface to delete.

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "vlan-interface" => { type => SCALAR, optional => 1 },
            "interface"      => { type => SCALAR, optional => 1 },
            "vlan-id"        => { type => SCALAR, optional => 1 },
            "inet6"          => { type => SCALAR, default  => 1 },
        },
    );
    if (   !defined( $opts{'vlan-interface'} )
        && !( defined( $opts{interface} ) && defined( $opts{'vlan-id'} ) ) )
    {
        NATE::BaseException->throw(
"vlan-interface and interface, vlan-id are mutual exclusive, so at least one parameter is needed"
        );
    }
    my $obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_delete_linux_cli',
                interface => 'CLI',
                set       => 'Linux',
            },
            {
                method    => '_delete_solaris_cli',
                interface => 'CLI',
                set       => 'Solaris',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}    ##end sub delete

sub _delete_linux_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts   = $pkg->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};
    my %linux_args;
    $linux_args{'vlan-name'} = $pkg->_get_vlan_interface(%opts);

    $apiset->vconfig_rem( %linux_args, 'privilege-level' => "root" );
    $Log->exit() if $may_exit;
}    ##end sub _delete_linux_cli

sub _delete_solaris_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %solaris_args;

    my ( $vlan_id, $interface, $vlan_interface );
    $solaris_args{'interface'} = $pkg->_get_vlan_interface(%opts);

    $apiset->ifconfig( %solaris_args, unplumb => "1" );
    if ( defined( $opts{inet6} ) && $opts{inet6} ) {
        $apiset->ifconfig( %solaris_args, inet6 => "1", unplumb => "1" );
    }
    $Log->exit() if $may_exit;
}    ##end sub _delete_solaris_cli

sub _primary_keys_validate_spec {
    return (
        'interface'      => { type => SCALAR },
        'vlan-id'        => { type => SCALAR },
        'vlan-interface' => { type => SCALAR, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1;

