# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Represents an ONTAP system
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::CommandInterface::ONTAP

=head1 SYNOPSIS

    use base 'NACL::C::CommandInterface::ONTAP';

=head1 DESCRIPTION

NACL::C::CommandInterface::ONTAP is an abstract base class for all
ONTAP Command interfaces (Node, Cluster, Vserver, Vfiler). It is derived from
NACL::C::CommandInterface and it provides tighter, ONTAP-specific APISet
contraints.

=head1 ATTRIBUTES

In addition to those of
L<NACL::C::CommandInterface|lib-NACL-C-CommandInterface-pm/ATTRIBUTES>
ONTAP command interfaces have the following attributes.

=head2 privilege_level

The privilege level at which commands are being invoked for this
command interface. The allowed privilege levels are (in order of highest
to lowest privilege level):

=over

=item CMode

=over

=item test

=item diagnostic

=item advanced

=item admin

=back

=item 7Mode

=over

=item test

=item diag

=item advanced

=item admin

=back

=back

Note that in 7Mode the level is called "diag" while in CMode it is
"diagnostic". This mismatch is handled by the library so that calls are
agnostic. This means that if a call like:

    $ci->privilege_level('diagnostic');

is made for a 7Mode command interface, the implementation will take care
of translating it to "diag" before setting the privilege level. Similarly
a call like:

    $ci->privilege_level('diag');

for a CMode command interface will automatically get internally translated
to "diagnostic" before getting set.

=head2 is_cmode

Is a value of 1 for a CMode machine, is 0 for a 7Mode machine.

=head2 is_vsim

Is a value of 1 for a vsim, else is 0.

=head2 mcc_remote_interface

A command_interface object representing a Node in the remote MCC cluster. This
is used while performing MCC replication verification.

=head2 is_sdot

Is a value of 1 if this models an sDOT system.

=head2 use_version_manager_cache

Is a value of 1 if the version manager cache (see next section below for details)
is being used.

=cut

package NACL::C::CommandInterface::ONTAP;

use strict;
use warnings;
use base qw(NACL::C::CommandInterface NACL::C::Component::ONTAP);

use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;
use Tharn qw( host);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::NetworkInterface;
use NACL::Exceptions::NoElementsFound;
use Params::Validate
    qw(validate_with validate_pos ARRAYREF SCALAR OBJECT BOOLEAN);
use Scalar::Util qw(blessed refaddr);
use autouse 'NACL::GeneralUtils' => qw(_require_nate_version);
use NACL::C::Exceptions::NotInMCCConfig qw(:try);
use NACL::C::_Mixins::ComponentCommandInterfaceONTAP qw(:all);
use NACL::Database;
use NACL::ComponentUtils qw(_dump_one);
use NATE::ParamSet qw(param_global);
use NATE::Database::Test qw(testdb_global);

#
# Since this lib derives NACL::C::Component first
# and then NACL::C::Component::ONTAP ( this should be corrected
# in the near future ) .We need to redirect the calls of
# _common_validate_with and can to NACL::C::Component::ONTAP
#
#
BEGIN {
    no strict 'refs';
    foreach my $method ( qw(_common_validate_with can )) {
        my $method_name = "NACL::C::Component::ONTAP::$method";
        *{$method} = \&$method_name;
    }
    use strict 'refs';
}

# Switch to 1 when we want to enable it by default.
my $default_use_version_manager_cache = 0;

use Class::MethodMaker [
    scalar => [
        {   -default_ctor => sub {
                $Log->enter() if $may_enter;
                my $self    = $_[0];
                my @apisets = $self->_apisets_values();
                foreach my $apiset (@apisets) {

                    # We should not set the privilege on any APISet.
                    # We should only set it for CLI APISets and for the ones
                    # for which the privilege-level is relevant. For example,
                    # Systemshell, Maintenance and Firmware APISets should
                    # not have the privilege set on them. A slightly crude
                    # way of checking if it is relevant is to see if
                    # "fallback_privilege" has been implemented on the APISet
                    if ( $apiset->can('fallback_privilege') ) {
                        $Log->exit() if $may_exit;
                        return $apiset->get_privilege();
                    }
                }

                # If there is currently no CLI APISet in the object, create
                # one and use it to get the privilege level
                my $cli_apiset = $self->apiset(
                    interface => 'CLI',
                    set       => $self->mode()
                );
                $Log->exit() if $may_exit;
                return $cli_apiset->get_privilege();
            },

            -store_cb => sub {
                $Log->enter() if $may_enter;
                my ( $self, $priv ) = @_;
                my @apisets = $self->_apisets_values();
                foreach my $apiset (@apisets) {
                    $self->_handle_priv_settings_agnostic(
                        apiset          => $apiset,
                        privilege_level => $priv
                    );
                }
                $Log->exit() if $may_exit;
                return $priv;
            },
        },
        'privilege_level'
    ],
    scalar => [
        {   -default_ctor => sub { $_[0]->_is_cmode() } },
        'is_cmode'
    ],
    scalar => [
        {   -default_ctor => sub { $_[0]->_is_vsim() } },
        'is_vsim'
    ],
    scalar => [
	{   -default_ctor => sub { $_[0]->_is_fusen() } },
	'is_fusen'
    ],
    scalar => [
        {-type => 'NACL::C::CommandInterface::ONTAP'},
        'mcc_remote_interface',
    ],
    scalar => [
        {   -default_ctor => sub {  $_[0]->_is_sdot() } },
        'is_sdot'
    ],
    scalar => [
        {   -default_ctor => sub {  $_[0]->_is_sdot_multiple_nodes() } },
        'is_sdot_multiple_nodes',
    ],
    scalar => [
        {   -default_ctor => sub {
                my ($self) = $_[0];
                my $use_vm_cache = $self->global_use_version_manager_cache()
                    // param_global()->get('NACL_USE_VERSION_MANAGER_CACHE')
                    // $default_use_version_manager_cache;
                return $use_vm_cache;
            }
        },
        'use_version_manager_cache',
    ],

    # Define this as static because we want it to apply for all objects; it
    # should not be an attribute on individual objects.
    scalar => ['-static', 'global_use_version_manager_cache'],
];

=head1 VERSION MANAGER CACHE

Typically, each new command_interface object will need to query for version
details. This involves running numerous commands, such as "system image show",
"system node uichanges" and "set -privilege ?". There are two issues with this:

=over

=item Re-querying for version details

Since each script is a separate process (and hence have separate memory address
spaces), over the course of a ntest run for a test suite, the version details
for a particular host will need to be re-queried multiple times over. Unless
the version of a host changes during a run, this adds execution time since the
same details are queried for multiple times.

=item Limits number of parallel processes

Since not all version details are available through ZAPI (such as the build-path,
whether the build is promoted or not and the list of uichanges, among others),
it is necessary to query for the version details through the CLI. This
necessitates an SSH connection being established for each host for each test
script. This limits the number of tests that can be run in parallel since it
would be necessary to prevent against hitting the various SSH limits defined in ONTAP.

=back

To alleviate these issues, NACL now supports caching the version manager details.
This cache is persistent for an ntest run, so every test script executed within
a particular ntest run will have access to the cache. In other words, this would
mean that version details for a particular host would need to be queried only
once for the entire test suite, except for cases where the version changes
during the run.

=head2 What if the version changes during the run (such as upgrade/revert)

The implementation of the NACL APIs which could cause a version change
have been updated to invalidate the cache. "Invalidating the cache" involves
removing the entries from the cache to ensure that the subsequent
call will re-query for the version details. This includes, but is not limited
to, the following APIs:

    $command_interface->refresh_command_interface();
    NACL::C/STask::SystemNodeImage->modify()
    NACL::C/STask::SystemNodeImage->update()
    NACL::C/STask::ClusterImage->update()
    NACL::MTask::Revert->revert/perform_revert_cmode/perform_revert_7mode()

Note that the above is not intended to be a comprehensive list of all APIs that
invalidate the cache, so there could be others not listed above.

If it is found that there are other APIs which cause a version change,
and hence would require invalidating the cache, please raise a burt (NACL/nacl_core).

=head2 How to use the version manager cache

The version manager cache is currently not enabled by default. It can be enabled
in any of the following ways:

=over

=item Command-line parameter NACL_USE_VERSION_MANAGER_CACHE

By specifying the command-line parameter C<NACL_USE_VERSION_MANAGER_CACHE=1>,
the version manager cache will be enabled. This allows enabling it for a
test script or for an entire test suite without modifying the code of the
underlying scripts.

As with other parameters, this an be specified on the command-line, in the
.ntest file, or in the .stest file for the suite.

=item Enabling within the script

It is also possible to enable the cache within the script. This would enable
the cache for that script along with any sub-processes spawned by that script.
Enabling it within the script would eliminate the need to remember to specify
the C<NACL_USE_VERSION_MANAGER_CACHE> parameter while running the test. This can
be done by setting the following static variable:

    NACL::C::CommandInterface::ONTAP->global_use_version_manager_cache();

=item Enabling only for a particular command_interface object

The earlier two mechanisms enable the version manager cache on all
command interface objects. It is also possible to enable the cache only
for individual command interface objects by setting the
C<use_version_manager_cache> attribute on the object.

    # Immediately after the call to find()
    my $node NACL::C::Node->find();
    $node->use_version_manager_cache(1);

    # In the call to new()
    NACL::C::Node->new(node => $some_node_name, use_version_manager_cache => 1);

=back

=cut

sub _update_version_manager_cache {
    $Log->enter();
    my ($self, @opts) = @_;
    my ($version_manager) = validate_pos(@opts, {type => OBJECT, isa => 'NACL::APISet::Version'});
    my $global_db = testdb_global();
    if ($global_db) {
        $global_db->insert_version_manager_cache(
            row => {
                version_manager_key => $self->_get_version_manager_key(),
                ds                  => $version_manager,
            },
        );
    }
    $Log->exit();
}

sub _get_from_version_manager_cache {
    $Log->enter();
    my ($self) = @_;
    my $vm;
    my $global_db = testdb_global();
    if ($global_db) {
        my $key = $self->_get_version_manager_key();
        $vm = $global_db->query_version_manager_cache(version_manager_key => $key);
        if ($vm) {
            # Normally, had the version details been queried, then the logs
            # would have the outputs of those commands. However, when obtained
            # from the cache, the log would not show the version details.
            # Let's dump the version manager object here so the logs do have
            # this information.
            # While dumping, remove non-version related data, since that would
            # just be log spam.
            my %vm_deep_copy = %$vm;
            foreach my $field (qw(category cli_connection conn_overrides hostobj
                session_object shell_type compatibility not_parsed)) {
                delete $vm_deep_copy{$field};
            }
            my $uichange_str = delete $vm_deep_copy{uichange_strings};
            $Log->comment("Got version manager from cache for '$key': " . _dump_one(\%vm_deep_copy));
            if (defined $uichange_str) {
                $Log->comment('Got uichange strings from cache: ' . _dump_one($uichange_str));
            }
        }
    }
    $Log->exit();
    return $vm;
}

# Calculate the key into the version manager table. The key is "<type>,<name>",
# for example: "Node,myvsim1".
sub _get_version_manager_key {
    my ($self) = @_;

    # Cannot use $self->name() since NACL::STask::Cluster unfortunately
    # overrides it to look up the cluster identity, which in turn caused
    # an infinite recursion since ->name() invokes ->apiset() which in turn
    # invokes this method, and so on.
    # See burt 1016840. Tested by CommandInterface.thpl/test_vm_infinite_recursion
    my $name_attr = $self->name_attribute();
    return join (',', $self->_get_type(), $self->$name_attr());
}

=head2 invalidate_version_manager_cache

    NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache();

Invalidate the version manager cache by removing all entries from it.
This should be called whenever there is a version change during the
course of the test.

=cut

sub invalidate_version_manager_cache {
    $Log->enter();
    my $global_db = testdb_global();
    if ($global_db) {
        $Log->comment('Invalidating version manager cache');
        $global_db->db_delete(table => 'version_manager_cache');
    }
    $Log->exit();
}

=head2 refresh_command_interface

Provides the same functionality and accepts the same options as documented in
L<NACL::C::CommandInterface::refresh_command_interface|lib-NACL-C-CommandInterface-pm/refresh_command_interface>.
Additionally, it takes care of invalidating the version manager cache unless
C<< no_version_change => 1 >> was provided. It is possible to configure whether
to invalidate the version manager cache by specifying
C<< nacl_invalidate_version_manager_cache => 0|1 >>.

=cut

sub refresh_command_interface {
    $Log->enter();
    my ($self, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            nacl_invalidate_version_manager_cache => {type => SCALAR, default => 1},
        },
        allow_extra => 1,
    );
    my $invalidate_cache = delete $opts{nacl_invalidate_version_manager_cache};
    $self->SUPER::refresh_command_interface(%opts);
    if (!$opts{no_version_change} && $invalidate_cache && $self->use_version_manager_cache()) {
        # Version details could have changed, so invalidate existing cache.
        # Workaround CTL switching NATE_LIB paths in the middle of an ntest
        # run causing the version_manager_cache table to not exist in the DB.
        # This eval block should eventually be removed once CTL updates their
        # tooling.
        # Burt 1013850.
        eval {
            NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache();
        };
        if ($@) {
            $Log->warn('NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache'
                . ' failed with: ' . $@);
        }
    }
}

=head1 METHODS (for users)

=head2 default_apiset_could

Refer to L<NACL::C::CommandInterface|lib-NACL-C-CommandInterface-pm/default_apiset_could>.
The default apiset could for ONTAP components in Node-CLI-CMode.

=cut

sub default_apiset_could {
    return {
        category  => "Node",
        interface => "CLI",
        set       => "CMode",
    };
} ## end sub default_apiset_could

=head2 default_apiset_must

Refer to L<NACL::C::CommandInterface|lib-NACL-C-CommandInterface-pm/default_apiset_must>.
All ONTAP components must be of category "Node".

=cut

# Subclasses of this should also assign the appropriate "set" and
# "set_source" values as well.

sub default_apiset_must {
    return {
        category        => "Node",
        category_source => "command interface is ONTAP-based",
    };
}

=head2 mode

    my $mode = $command_interface_obj->mode();

Returns the mode of the command interface (i.e. CMode or 7Mode)

=cut

sub mode {
    $Log->enter() if $may_enter;
    _require_nate_version(
        version => '4.3.25',
        method  => 'is_cmode'
    );
    my ($self) = @_;

    # Prevent the possibility of infinite recursions
    if ( !$self->can_use_as_command_interface() ) {
        my $name_attribute = $self->name_attribute();
        my $name           = $self->$name_attribute();
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "The host '$name' does not have a "
                . 'hostrec and it could not be dynamically constructed. '
                . 'For this reason the mode of this host cannot be determined'
        );
    }

    my $hostrec = $self->hostrec();
    my $mode;
    if ( $hostrec->is_cmode() ) {
        $mode = 'CMode';
    } else {
        $mode = '7Mode';
    }

    $Log->exit() if $may_exit;
    return $mode;
}

=head2 is_cmode

    my $boolean = $ci_obj->is_cmode();

Returns a 0/1 value describing whether the command interface is for a CMode
host.

=cut

sub _is_cmode {
    $Log->enter() if $may_enter;
    _require_nate_version(
        version => '4.3.25',
        method  => 'is_cmode'
    );
    my ($self) = @_;
    $Log->exit() if $may_exit;
    return $self->hostrec()->is_cmode();
}

=head2 is_7mode

    my $boolean = $ci_obj->is_7mode();

Returns a 0/1 value describing whether the command interface is for a 7Mode
host.

=cut

sub is_7mode {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my $is_7mode = !$self->is_cmode();
    $Log->exit() if $may_exit;
    return $is_7mode;
}

=head2 _is_fusen

    my $boolean = $ci_obj->_is_fusen;

Returns a 0/1 value describing whether the command interface is for _is_fusen.

=cut

sub _is_fusen {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my $value = "";
    my $apiset = $self->command_interface()->get_systemshell_apiset();
    try {
        my $obj = $apiset->is_fusen_mode();
        my $parsed_output = $obj->get_parsed_output();
        $value = ${@$parsed_output[0]}{'fusen_mode'};
    }
    otherwise {
	# suppressing here
    };
    my $is_enabled = ($value =~ /enabled/i) ? 1 : 0;
    $Log->exit() if $may_exit;
    return $is_enabled;
}

=head2 get_command_interfaces_for_lifs

    my @command_interfaces = $ci->get_command_interfaces_for_lifs(
        lif_c_objs  => \@arr_of_interface_c_objs);

    my @command_interfaces = $ci->get_command_interfaces_for_lifs(
        lif_cs_objs => \@arr_of_interface_cs_objs);

This method, when provided a list of NetworkInterface Component or
ComponentState objects, returns an array of command interface objects for
each interface passed in.

Either C<lif_c_objs> or C<lif_cs_objs> should be provided in the call.
Providing both, or neither, results in a C<NATE::Exceptions::Argument> being
thrown.

This method works only for CMode.

=over

=item Options

=over

=item C<< lif_c_objs => \@network_interface_c_objs >>

(Either this or C<lif_cs_objs> is required)

An array of NACL::C::NetworkInterface objects.

=item C<< lif_cs_objs => \@network_interface_cs_objs >>

(Either this or C<lif_c_objs> is required)

An array of NACL::CS::NetworkInterface objects.

=item Other options

The other options accepted are the common component arguments: apiset_must,
apiset_should, method-timeout.

=back

=back

=cut

sub get_command_interfaces_for_lifs {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );
    $self->_throw_exception_if_not_cmode();

    my %opts = validate_with(
        params => \@_,
        spec   => {
            %{ $self->_common_validate_spec_without_ci() },
            lif_c_objs  => { type => ARRAYREF, default => [] },
            lif_cs_objs => { type => ARRAYREF, default => [] },
        },
    );

    my $lif_c_objs  = delete $opts{lif_c_objs};
    my $lif_cs_objs = delete $opts{lif_cs_objs};
    if ( @$lif_c_objs && @$lif_cs_objs ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( 'Either of the arguments '
                . "'lif_c_objs' or 'lif_cs_objs' should be provided but "
                . 'both were' );
    } elsif ( !@$lif_c_objs && !@$lif_cs_objs ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( 'Either of the arguments '
                . "'lif_c_objs' or 'lif_cs_objs' should be provided but "
                . 'neither were' );
    }

    my $throw_exception = sub {
        my %opts = validate_with(
            params => \@_,
            spec   => {
                argument => { type => SCALAR },
                object   => { type => OBJECT },
                type     => { type => SCALAR }
            }
        );
        NATE::Exceptions::Argument->throw( 'All of the values '
                . "provided through the '$opts{argument}' argument should "
                . "be blessed objects of type '$opts{type}' "
                . "but one of the values was provided of type '"
                . ref( $opts{object} )
                . "'" );
    };

    if (@$lif_cs_objs) {
        foreach my $lif_cs_obj (@$lif_cs_objs) {
            if (   !blessed($lif_cs_obj)
                || !$lif_cs_obj->isa('NACL::CS::NetworkInterface') )
            {
                $Log->exit() if $may_exit;
                $throw_exception->(
                    argument => 'lif_cs_objs',
                    object   => $lif_cs_obj,
                    type     => 'NACL::CS::NetworkInterface'
                );
            }
        }
    }

    if (@$lif_c_objs) {
        foreach my $lif_c_obj (@$lif_c_objs) {
            if (   !blessed($lif_c_obj)
                || !$lif_c_obj->isa('NACL::C::NetworkInterface') )
            {
                $Log->exit() if $may_exit;
                $throw_exception->(
                    argument => 'lif_c_objs',
                    object   => $lif_c_obj,
                    type     => 'NACL::C::NetworkInterface'
                );
            }
        }
        foreach my $lif_c_obj (@$lif_c_objs) {
            push @$lif_cs_objs,
                $lif_c_obj->state(
                requested_fields => [qw(vserver lif address)],
                %opts
                );
        }
    }

    my @command_interfaces;
    foreach my $cs_obj (@$lif_cs_objs) {
        push @command_interfaces,
            $self->clone_with_new_hostrec( default_ip => $cs_obj->address() );
    }

    $Log->exit() if $may_exit;
    return @command_interfaces;
}

=head2 get_all_management_interfaces

    my @net_int_cs_objs = $ci->get_all_management_interfaces();

Returns NACL::CS::NetworkInterface objects for each of the interfaces
for this host. These objects can then be further filtered and then
passed on to L<get_command_interfaces_for_lifs|lib-NACL-C-CommandInterface-ONTAP.pm/get_command_interfaces_for_lifs> to obtain command interface
objects.

The only options this method accepts are the common options: apiset_must,
apiset_should, method-timeout.

(Works only for CMode)

=cut

sub get_all_management_interfaces {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    $self->_verify_invocation( style => 'instance_only' );
    $self->_throw_exception_if_not_cmode();
    my $name_attribute = $self->name_attribute();

    my @interface_objs = $self->_get_mgmt_lifs(
        command_interface => $self->command_interface(),
        vserver           => $self->$name_attribute(),
        %opts
    );
    $Log->exit() if $may_exit;
    return @interface_objs;
}

=head2 get_command_interfaces_for_all_mgmt_interfaces

    my $first_command_interface =
        $command_interface->get_command_interfaces_for_all_mgmt_interfaces();

                          or

    my @command_interfaces =
        $command_interface->get_command_interfaces_for_all_mgmt_interfaces();

Given a command interface, this method returns command interface objects
for each of the management IPs of that host.

When it is invoked in scalar context, it returns the first such command
interface object constructed.

(Works only for CMode)

=cut

sub get_command_interfaces_for_all_mgmt_interfaces {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    $self->_verify_invocation( style => 'instance_only' );
    $self->_throw_exception_if_not_cmode();

    my @lifs = $self->get_all_management_interfaces( %opts,
        requested_fields => [qw(vserver lif address)] );
    my @command_interfaces
        = $self->get_command_interfaces_for_lifs( lif_cs_objs => \@lifs );

    $Log->exit() if $may_exit;
    return wantarray ? @command_interfaces : $command_interfaces[0];
}


=head2 has_uichange

    my $boolean = $command_interface->has_uichange(
        uichange => $uichange
    );

    # Perform any operation
    if ( $boolean ) { .. }

(Instance method)
This method is used to determine whether a particular uichange exists
or not. ONTAP generally adds uichange strings to denote the changes in the
UI (say, input option name or output field name is renamed) and at sometimes,
to denote the presence of a feature.

Returns boolean value

    1 : If the specified UI change exists,
    0 : If the specified UI change doesn't exist

The advantage of using this method over
L<has_uichange()|lib-NACL-APISet-Version-Node-CMode-pm/has_uichange>
from the version manager is that,

this method has the additional ability to find the presence/absence
of uichanges even in the vserver context whereas the latter doesn't.
Note that this support needs additional logic to enable finding the
information in the vserver context and so, this support is added only
on request. If this method has such additional support for a uichange,
then the uichange string will be listed under the
L<UICHANGES section|lib-NACL-C-_Version-pm/UICHANGES>. If the specific
uichange string is not listed, this method still supports finding the
presence/absence of the uichange string in the cluster context, just that any
other additional smartness like, finding the information in vserver context
is not supported.

See L<NACL::C::CommandInterface::ONTAP::has_uichange()|lib-NACL-C-CommandInterface/ONTAP-pm/has_uichange>.

=over

=item command_interface

(Required) A component object that represents the host to which to send
commands.

=item uichange

(Required)

Name of the uichange string.

=back

=cut

sub has_uichange {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->_verify_invocation( style => 'instance_only' );
    my $val = NACL::C::_Version->has_uichange(
        command_interface => $self,
        %opts
    );
    $Log->exit() if $may_exit;
    return $val;
}

=head2 get_codename

    my $codename = $self->get_codename();

(Instance method only, for component users)
Returns the codename associated with the release that's being run
on the command interface object.

Any errors that might occur while attempting to obtain the version
details (such as the systemshell being locked) are ignored. In such cases,
this method returns undef. Versioning is supported for both CMode
and 7Mode ONTAP.

=cut

sub get_codename {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );

    my $codename;
    try {
        my $version_mgr = $self->get_version_manager();
        $codename
            = $version_mgr->get_version_attribute( attribute => 'codename' );
    }
    otherwise {
        my $exception = shift;
        my $ref       = ref $exception;
        $Log->debug( "A $ref exception was thrown during versioning "
                . "while attempting to find codename. Error text:\n"
                . $exception->text() );
    };

    $Log->exit() if $may_exit;
    return $codename;
}

=head2 construct_hostrec

    # Construct a hostrec by dynamically determining the IP address
    $ci_obj->construct_hostrec();

    # Construct a hostrec by using a caller-provided IP address
    $ci_obj->construct_hostrec(default_ip => $ip);

This method is used to construct a hostrec object for command interfaces
that do not yet have a hostrec stored within them. (note that a hostrec
object is essential for using this command interface object to issue commands)

This method can be invoked in one of two ways:

=over

=item Without arguments

If there is no nate_host file, then the method will attempt to determine
the IP address of the command interface by running the network interface
command.

=item With IP address provided as argument

In case we know the IP address, instead of having the method run a command
to determine the IP address, we can provide it as argument to the call
(through the C<default_ip> argument). An example of where this would be the
case is where we create a vserver in the script and associate a lif with it.

=back

=over

=item Options

=over

=item C<< default_ip => $ip >>

(Optional)
The IP address of the command interface object.

=back

=back

=cut

sub construct_hostrec {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            default_ip => { type => SCALAR, optional => 1 },
            %{ $self->_common_validate_spec_without_ci() }
        }
    );

    if ( $self->{hostrec} ) {

        # hostrec has already been set, nothing to do
        $Log->exit() if $may_exit;
        return;
    }

    my $name_attribute = $self->name_attribute();
    my $name           = $self->$name_attribute();
    my $host;
    try {
        $host = host $name;
    }
    otherwise {

        my $config_error = shift;

        # Ah, no host file. If an IP address was provided by the caller,
        # let's use that. Else, dynamically determine what its IP is.
        my $address = delete $opts{default_ip};
        if ( !defined $address ) {
            if ( $self->is_command_interface_same_as_self() ) {

                # Command interface uses itself as command interface
                # No way we can determine its IP

                my $no_hostrec_error = $self->_no_hostrec_error(
                    name            => $name,
                    config_error    => $config_error,
                    throw_exception => 0,
                );

                $Log->exit() if $may_exit;
                NATE::BaseException->throw( "$no_hostrec_error\n"
                        . 'Also, a hostrec object could not be dynamically '
                        . 'constructed because this object uses itself '
                        . 'as command interface' );
            }

            if ( $self->command_interface()->is_7mode() ) {

                # Dynamic determination of IP can be done only for CMode
                $Log->exit() if $may_exit;
                $self->_no_hostrec_error(
                    name         => $name,
                    config_error => $config_error
                );
            } else {

                # Need to dynamically determine the IP address
                my $cs;
                try {
                    my @cs_objs = $self->get_all_management_interfaces( %opts,
                        requested_fields => [qw(vserver lif address)] );
                    if ( $self->isa('NACL::C::Cserver') ) {
                        for ( @cs_objs ) {
                            my $cs_obj = $_;
                            $cs = $cs_obj  if ( $cs_obj->{'lif'}  eq 'clus_mgmt' );
                        }
                    }
                    $cs = $cs_objs[0] if ( ! defined $cs );
                }
                catch NACL::Exceptions::NoElementsFound with {
                    my $no_hostrec_error = $self->_no_hostrec_error(
                        name            => $name,
                        config_error    => $config_error,
                        throw_exception => 0,
                    );
                    $Log->exit() if $may_exit;
                    NATE::BaseException->throw( "$no_hostrec_error\n"
                            . 'Also a hostrec object could not be dynamically '
                            . 'constructed because its IP address could '
                            . 'not be determined (maybe it does not '
                            . 'have a management lif associated with it?)' );
                };
                $address = $cs->address();
            }
            $host = host( $name, 'hosttype=' . $self->_hosttype(),
                "default_ip=$address" );
        }
    };

    $Log->exit() if $may_exit;
    return $host;
}

=head2 dynamically_construct_hostrec_for_multiple_objects

    NACL::C::Vserver->dynamically_construct_hostrec_for_multiple_objects(
        objects => \@vserver_objs
    );

    NACL::C::Cluster->dynamically_construct_hostrec_for_multiple_objects(
        objects => \@cluster_objs
    );

For command interfaces that do not have a nate_host file, the IP address
is determined just-in-time (i.e. when the object is about to be used
as a command interface for another component) by running the network
interface command. The disadvantage with this is that if there are
multiple command interfaces for which we need to dynamically determine
the IP address, each of them will result in one command being invoked.

This method when provided multiple command interface objects will determine
the IP addresses of all command interfaces by running a single command
and construct hostrec objects for each of the provided command interface
objects.

=over

=item Options

=over

=item C<< objects => \@arr_of_ci_objs >>

(Required)
This is an array-reference of command interface objects whose hostrec
need to be dynamically constructed.

=back

=back

=cut

sub dynamically_construct_hostrec_for_multiple_objects {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            objects => { type => ARRAYREF },
            %{ $self->_common_validate_spec_without_ci() }
        }
    );

    my ( @names, %objects_keyed_by_name );
    my $objects = $opts{objects};
    foreach my $object ( @{$objects} ) {
        if ( blessed($object) && $object->isa($self) ) {
            push @names, $object->name();
            $objects_keyed_by_name{ $object->name() } = $object;
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw();
        }
    }

    my $filter = join '|', @names;
    my @cs_objects = $self->_get_mgmt_lifs(
        command_interface => $self->command_interface(),
        vserver           => $filter,
        requested_fields  => [qw(vserver lif address)],
        %opts
    );

    my $updated_objects;
    foreach my $cs (@cs_objects) {
        my $object = $objects_keyed_by_name{ $cs->vserver() };
        if ( !$object->{hostrec} ) {
            my $address = $cs->address();
            my $host
                = host( $object->name(), 'hosttype=' . $self->_hosttype(),
                "default_ip=$address" );
            $object->hostrec($host);
            $updated_objects++;
        }
    }
    if ( $updated_objects < @$objects ) {
        my @not_updated_list;
        while ( my ( $name, $object ) = each %objects_keyed_by_name ) {
            if ( !$object->{hostrec} ) {
                push @not_updated_list, $name;
            }
        }
        my $list = join ', ', @not_updated_list;
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( 'The command interface objects for the '
                . 'following hosts did not have their hostrec field updated '
                . "because a management lif could not be determined for them:\n"
                . $list );
    }

    $Log->exit() if $may_exit;
}

=head2 is_command_interface_same_as_self

    my $boolean = $ci->is_command_interface_same_as_self();

Returns 1 if the object is using itself as its command interface, returns 0
otherwise.

=cut

sub is_command_interface_same_as_self {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $Log->exit() if $may_exit;
    return ( refaddr( $self->command_interface() ) eq refaddr($self) )
        ? 1
        : 0;
}

=head2 can_get_version_details

    my $boolean = $ci->can_get_version_details();

Returns 1 if version details could be returned from the object, returns 0
otherwise.

=cut

sub can_get_version_details {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    $Log->exit() if $may_exit;
    return ( !($self->is_vserver() && $self->is_command_interface_same_as_self()) )
        ? 1
        : 0;
}

# Documented in the base class, this handles the privilege level settings
sub apiset {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    my $dummy;
    $opts{_was_new_apiset_created} //= \$dummy;
    my $apiset = $self->SUPER::apiset(%opts);

    # Not using isset() since this field has a -default_ctor. Any field
    # with a -default/-default_ctor setting will result in isset() being
    # true.

    #
    # $apiset being returned by SUPER::apiset can be scalar( 0 or 1 )
    # when check_apiset_already_loaded is being passed in %args
    #
    if (blessed ($apiset) && $apiset->isa('NACL::APISet')) {
        if (${$opts{_was_new_apiset_created}}
            && $apiset->{actual_set} eq 'CMode') {
            # Update the version manager cache only if:
            # 1. A new APISet was created.
            # 2. It was for a CMode-CLI/ZAPI APISet. For Nodescope/Systemshell
            #    APISets, the version manager object will have limited details.
            # 3. If the version manager cache should be used.
            my $version_manager = $apiset->get_version_manager();
            if (defined $version_manager) {
                $self->_update_version_manager_cache($version_manager);
            }
        }
        if ($self->{privilege_level}) {
            $self->_handle_priv_settings_agnostic(
                apiset          => $apiset,
                privilege_level => $self->privilege_level()
            );
        }
    }

    $Log->exit() if $may_exit;
    return $apiset;
}

=head2 get_timezone

    my $zone = $ci->get_timezone();

Returns back the timezone information

=over

=item Options

=over

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds.

=back

=back

=cut

sub get_timezone {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => $self->_common_validate_spec_without_ci()
    );
    require NACL::CS::ClusterDate;
    my $ci;
    if ($self->is_vserver()) {
        my $ci_of_self = $self->command_interface();
        if ($ci_of_self->is_vserver()) {
            # Vserver uses itself as command_interface
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw( 'Timezone cannot be discovered '
                . 'from vserver command_interface. Instead pass Node or Cluster '
                . 'command_interface.'
            );
        } else {
            # Vserver uses some other cluster-scoped object as command_interface
            $ci = $ci_of_self;
        }
    } else {
        $ci = $self;
    }
    my $arr = NACL::CS::ClusterDate->fetch(
        requested_fields  => ['timezone'],
        command_interface => $ci,
        filter            => {
            node => 'local',
        },
        %opts
    );
    my $zone = $arr->timezone();
    $Log->exit() if $may_exit;
    return $zone;

}

=head2 group_by_version

    my $output = $self->group_by_version();

(Instance method only)
This method is used to get all the nodes in a cluster grouped based
on their release version. Returns a GroupByVersion object
having attributes: grouped_by_version, get_highest_version_in_cluster,
get_nodes_in_highest_version, get_lowest_version_in_cluster and
get_nodes_in_lowest_version. Each of these attributes can be accessed
via their accessor methods.
The output looks like the following:
$VAR1 = bless( {
                 'get_lowest_version_in_cluster' => '8.0.1',
                 'get_nodes_in_highest_version' => [
                                                     'fas3170-210-05',
                                                     'fas3170-210-06'
                                                   ],
                 'get_highest_version_in_cluster' => '8.1.2',
                 'get_nodes_in_lowest_version' => [
                                                    'BR.1'
                                                  ],
                 'grouped_by_version' => {
                                           '8.0.1' => 'ARRAY(0xe2de8a0)',
                                           '8.1.2' => 'ARRAY(0x962f000)'
                                         },
                 'node_objs_in_highest_version' => [ $node_obj1, $node_obj2 ],
                 'node_objs_in_lowest_version'  => [ $node_obj3, $node_obj4 ],
               }, 'GroupByVersion' );


=cut

sub group_by_version {
    $Log->enter() if $may_enter;
    require NACL::STask::Node;
    my ($self) = @_;
    $self->_verify_invocation(style => 'instance_only');

    # Will have to collect all the node in the cluster to find
    # and group based on release

    require NACL::CS::SystemNodeImage;
    if ( $self->isa('NACL::C::Vserver') ) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw( 'Nodes cannot be grouped '
                . 'from vserver command_interface. Instead pass Node or Cluster '
                . 'command_interface.' );
    }
    my @states = NACL::CS::SystemNodeImage->fetch(
                    command_interface => $self,
                    filter            => {'iscurrent' => 'true'},
                    requested_fields => ['node','version'],
                );

    my $vm = $self->version_manager();
    my %result;
    foreach my $state (@states) {
        my $release = $vm->map_release_to_numeric_version( release => $state->version() );
        push(@{$result{$release}}, $state->node());
    }

    my @sorted_keys = sort { $b cmp $a } keys %result;
    my $highest = $sorted_keys[0];
	my $lowest = $sorted_keys[-1];
    my @highest_nodes = @{$result{$highest}};
    my @lowest_nodes = @{$result{$lowest}};

    my $obj = GroupByVersion->new();
    $obj->grouped_by_version(%result);
    $obj->get_highest_version_in_cluster($highest);
    $obj->get_lowest_version_in_cluster($lowest);
    $obj->get_nodes_in_highest_version(@highest_nodes);
    $obj->get_nodes_in_lowest_version(@lowest_nodes);

    my @highest_version_node_objs = map { NACL::STask::Node->new(node => $_) }
        @highest_nodes;
    my @lowest_version_node_objs = map { NACL::STask::Node->new(node => $_) }
        @lowest_nodes;

    $obj->node_objs_in_highest_version(@highest_version_node_objs);
    $obj->node_objs_in_lowest_version(@lowest_version_node_objs);

    $Log->exit() if $may_exit;
    return $obj;
}

=head2 get_remote_interface_mcc

    (Usage)
        $command_interface = NACL::C::Node->find();
        $partner_ci        = $command_interface->get_remote_interface_mcc();

(Instance call) This subroutine is used to fetch the command interface object
of the partner MCC cluster.
Return : A object of the same type as that of the command_interface on which
         this method is invoked.

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NotInMCCConfig>

This type of exception is thrown when the filer or node which is in execution
is not MCC configured.

=back

=back

=cut

sub get_remote_interface_mcc {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    my $partner_ci;

    if (defined $opts{remote_interface}) {
        $partner_ci = $opts{remote_interface};
    } else {
        if ($self->mcc_remote_interface_isset()) {
            $partner_ci = $self->mcc_remote_interface();
        } else {
            my $throw_exception = sub {
                NACL::C::Exceptions::NotInMCCConfig->throw(
                'MCC is not configured. Node provided = ' . $self->node());
            };
            try {
                my $mcc_configured   = NACL::CS::MetroclusterNode->fetch(
                    command_interface => $self,
                    filter => { node  => $self->node() },
                    requested_fields => [ qw( dr-partner configuration-state ) ],
                    %opts,
                );
                if ($mcc_configured->configuration_state =~ /configured/i) {
                    my $partner_node = $mcc_configured->dr_partner();
                    $partner_ci   = $self->get_package_name()
                        ->new(node => $partner_node);
                } else {
                    $throw_exception->();
                }
            } catch NACL::Exceptions::NoElementsFound with { # ZAPI
                $throw_exception->();
            } catch NACL::APISet::Exceptions::InvalidParamValueException with { # CLI
                $throw_exception->();
            };
        }
    }

    $self->mcc_remote_interface($partner_ci);
    $Log->exit() if $may_exit;
    return $partner_ci;
}

=head2 is_in_mcc_environment

    my $is_mcc = $command_interface->is_in_mcc_environment();

(Instance call) This subroutine returns 1 if MCC is enabled and is
in a steady state.

=cut

sub is_in_mcc_environment {
    $Log->enter() if $may_enter;
    my ($self,%opts) = @_;
    my $is_mcc = 1;
    require NACL::CS::Metrocluster;

    # If command_interface is a vserver, then can't perform verification
    if ($self->is_vserver()) {
        $is_mcc = 0;
    } else {
        # MCC is in a steady state if local|remote-configuration-state is
        # "configured" and if local|remote-mode is "normal"
        my $mcc_cs = NACL::CS::Metrocluster->fetch(
            %opts,
            command_interface => $self,
            requested_fields  => [qw(local-configuration-state local-mode
                remote-configuration-state remote-mode)],
        );
        foreach my $site (qw(local remote)) {
            my $config_state = "${site}_configuration_state";
            my $mode = "${site}_mode";
            if ($mcc_cs->$config_state() ne 'configured' ||
                $mcc_cs->$mode() ne 'normal') {
                $is_mcc = 0;
                last;
            }
        }
    }
    return $is_mcc;
}

=head2 is_mcc_configured()

    my $is_mcc = $command_interface->is_mcc_configured();

(Instance call) This subroutine returns 1 if MCC is configured, regardless of
state of MCC (steady state or switchover).

=cut

sub is_mcc_configured {
    $Log->enter() if $may_enter;
    my ($self,%opts) = @_;
    require NACL::CS::Metrocluster;

    # If command_interface is a vserver, then can't perform verification
    if ($self->is_vserver()) {
        $Log->exit() if $may_exit;
        return 0;
    } else {
        # MCC is configured if local|remote-configuration-state is "configured"
        my $mcc_cs = NACL::CS::Metrocluster->fetch(
            %opts,
            command_interface => $self,
            requested_fields  => [
                qw(local-configuration-state remote-configuration-state)
            ],
        );
        MCC_LOOP: foreach my $site (qw(local remote)) {
            my $config_state = "${site}_configuration_state";
            if ($mcc_cs->$config_state() eq 'configured') {
                $Log->exit() if $may_exit;
                return 1;
            }
        }
    }
    $Log->exit() if $may_exit;
    return 0;
} # End is_mcc_configured()

# Helps when we'd like to make SUPER calls in mix-ins. Any calls which
# would have been
#    $pkg->SUPER::method()
# would become
#    my $base_class  = $pkg->base_class();
#    my $base_method = "{base_class}::method"
#    $pkg->$base_method();
sub base_class { return __PACKAGE__; }

=head1 METHODS (for developers)

=head2 _create_version_manager

   sub get_version_manager {
        ...
        if (keys %$apisets) {
            ...
        } else {
            $version_manager = $self->_create_version_manager();
        }
        return $version_manager;
    }

(Instance method)
This private method returns the version manager of the appropriate type.
This decision is based on the category ("Node" for ONTAP command interfaces)
and the set.

=cut

sub _create_version_manager {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    use NACL::APISet::Version;
    my $hostrec    = $self->hostrec();
    my $category   = "Node";
    my $shell_type = $self->mode();
    $Log->debug( "Version manager object with parameters "
            . "category = $category; shell_type = $shell_type; "
            . "hostobj type = "
            . $hostrec->hosttype()
            . " created" );

    # Copy any connection overrides so that the connection established by the
    # version manager uses them.
    my %must_should = ($self->apiset_should(), $self->apiset_must());
    require NACL::APISet;
    my %conn_overrides;
    $self->_hash_copy(
        source => \%must_should,
        target => \%conn_overrides,
        copy   => [NACL::APISet->get_conn_override_fields()],
    );

    # If version manager for this object already present in the cache, then
    # reuse it.
    my $vm;
    if ($self->use_version_manager_cache()) {
        $vm = $self->_get_from_version_manager_cache();
    }

    if (defined ($vm)) {
        # If there are connection overrides, set them on the object
        $vm->conn_overrides(%conn_overrides) if (keys %conn_overrides);
    } else {
        $vm = NACL::APISet::Version->new(
            category   => $category,
            hostobj    => $hostrec,
            shell_type => $shell_type,
            conn_overrides => \%conn_overrides,
        );

        # Necessary to set the cli_connection for new version manager objects
        # to prevent the possibility of race condition where a disconnected
        # connection attempts to call set_privilege recursively and fails.
        # The "test_connection_reuse_race_condition" test-case in
        # CommandInterface.thpl tests this.
        if ($self->is_cmode()) {
            try {
                my $session = $vm->get_session(interface => 'CLI', subtype => 'CMode');
                $vm->cli_connection($session->{connection});
            } otherwise {};
        }
    }

    $Log->exit() if $may_exit;
    return $vm;
}

sub _get_mgmt_lifs {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            %{ $pkg_or_obj->_common_validate_spec() },
            vserver          => { type => SCALAR },
            requested_fields => { type => ARRAYREF, default => [] },
            randomize_order  => { type => BOOLEAN, optional => 1 },
        }
    );
    my $vserver = delete $opts{vserver};

    # Now %opts will contain command_interface, requested_fields
    # and the common opts - all of which can be directly
    # passed through to fetch.

    # XXX Frank Radish mentions that going forward, management lifs are no
    # longer going to be identified by role, but more by protocols. At some
    # point we should have a Task method to determine what is a management
    # lif. At this point we should swap this out and make the call to that
    # task. In the meanwhile, let's just go with using the "role" value to
    # determine which lifs are management lifs.
    my @interface_objs;
    try {
        @interface_objs = NACL::CS::NetworkInterface->fetch(
            filter => {
                vserver => $vserver,
                role    => 'node-mgmt|cluster-mgmt|data',
                address => '!-',
            },
            %opts
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        my $exception = shift;
        $exception->set_text( "No management lifs found for '$vserver'\n"
                . $exception->text() );
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return @interface_objs;
}

sub _throw_exception_if_not_cmode {
    $Log->enter() if $may_enter;
    my $self = shift;

    # Nasty infinite recursion if the hostrec has not yet been populated.
    # get_all_management_interfaces -> _throw_exception_if_not_cmode ->
    # is_7mode -> is_cmode -> mode -> hostrec -> construct_hostrec ->
    # get_all_management_interfaces
    # Let's just return if the hostrec hasn't been populated yet.
    if ( !$self->{hostrec} ) {
        $Log->exit() if $may_exit;
        return;
    }
    if ( $self->is_7mode() ) {
        my $caller = ( caller(1) )[3];
        $caller =~ s/.*:://;
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "The method '$caller' is only supported for CMode");
    }
    $Log->exit() if $may_exit;
}

# For CMode the privilege level is "diagnostic",
# but in 7Mode it's "diag". While doing
# "set -privilege diag" works in CMode as well since
# it auto-expands it to "diagnostic", it's best
# we don't rely on that and instead explicitly
# set it to "diagnostic".
# This piece of code makes the call agnostic, so
# regardless of whether the level is set as
# "diag" or "diagnostic" it will work for any APISet.
sub _handle_priv_settings_agnostic {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            apiset          => { type => OBJECT, isa => 'NACL::APISet' },
            privilege_level => { type => SCALAR }
        }
    );
    my $apiset = $opts{apiset};

    if ( $apiset->can('fallback_privilege') ) {
        my $priv = $opts{privilege_level};

        # is_7mode_like - true for 7Mode/Nodescope
        my $is_7mode_like = $apiset->isa('NACL::APISet::Node::CLI::7Mode');
        if ( $priv eq 'diagnostic' && $is_7mode_like ) {
            $priv = 'diag';
        } elsif ( $priv eq 'diag' && !$is_7mode_like ) {
            $priv = 'diagnostic';
        }

        # set_privilege could end up invoking a command, so we will only
        # invoke it if the APISet is not already at that privilege level
        # Note that get_privilege() merely reads from the object, it
        # does not invoke any commands
        if ( $apiset->get_privilege() ne $priv ) {
            $apiset->set_privilege( 'privilege-level' => $priv );
        }
    }

    $Log->exit() if $may_exit;
}

sub _check_if_valid_hostrec {
    $Log->enter() if $may_enter;
    my $self = shift;
    my ($hostrec) = validate_pos( @_, { isa => 'Hostrec' } );
    my $hosttype = $hostrec->hosttype();
    if ( $hosttype !~ /^filer/ ) {
        my $name = $hostrec->hostname();
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "Host '$name' was supposed "
                . "to be a filer/vsim but is actually of type '$hosttype'" );
    }
    $Log->exit() if $may_exit;
}

sub _check_if_hostrec_cmode {
    $Log->enter() if $may_enter;
    _require_nate_version(
        version => '4.3.25',
        method  => 'is_cmode'
    );
    my ( $self, @args ) = @_;
    my ($hostrec) = validate_pos( @args, { isa => 'Hostrec' } );
    if ( !$hostrec->is_cmode() ) {
        my $pkg = $self->get_package_name();
        $Log->exit() if $may_exit;
        NATE::BaseException->throw( "All hosts of type '$pkg' "
                . "should be CMode hosts but the hosttype of host '"
                . $hostrec->hostname()
                . "' is '"
                . $hostrec->hosttype()
                . "'\n" );
    }
    $Log->exit() if $may_exit;
}

sub _is_vsim {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $hosttype = $self->hostrec()->hosttype();
    my $is_vsim;
    if ($hosttype =~ /vsim/) {
        $is_vsim = 1;
    } else {
        $is_vsim = 0;
    }

    $Log->exit($is_vsim);
    return $is_vsim
}

=head2 is_vserver

    my $is_vserver = $ci->is_vserver();

Returns 1 if the CommandInterface is a Vserver. This helper method is useful
because a number of operations that are supported in cluster context are not
supported in vserver context.

=cut

sub is_vserver {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $is_vs = 0;
    if ($self->isa('NACL::C::Vserver')) {
        $is_vs = 1;
    }
    $Log->exit($is_vs);
    return $is_vs;
}

=head2 translate_cm_cli_api_to_command

    my $cmd = $ci->translate_cm_cli_api_to_command(api => $api);

Returns the command name associated with a particular CMode-CLI API. This is
done by looking-up the Schema object. For example, if the API is
"vserver_export_policy_create", then command returned would be
"vserver export-policy create".

=cut

sub translate_cm_cli_api_to_command {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    my $apiset;

    if( defined $opts{apiset_new_options}) {
        my $apiset_new_options = delete $opts{apiset_new_options};
        $apiset = $self->apiset(%$apiset_new_options);
    } else {
        $apiset = $self->apiset(interface => 'CLI', set => 'CMode');
    }

    my $cmd = $apiset->translate_api_to_command(%opts);
    $Log->exit() if $may_exit;
    return $cmd;
}

# Needed for burt 885428. Burt has description of problem and solution.
sub clone_with_new_hostrec {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    my $type = $self->is_cmode() ? 'gx' : '7g';
    foreach my $generic_param (qw(default_ip default_username default_password)) {
        my $specific_param = "${generic_param}_${type}";
        if (exists $opts{$generic_param}) {
            # If the generic param is set, then set the specific param IF the
            # specific param does not already have a value
            $opts{$specific_param} //= $opts{$generic_param};
        } elsif (exists $opts{$specific_param}) {
            $opts{$generic_param} = $opts{$specific_param};
        }
    }
    my $new_ci = $self->SUPER::clone_with_new_hostrec(%opts);
    $Log->exit() if $may_exit;
    return $new_ci;
}

sub _additional_common_validate_spec {
    my ($pkg_or_obj, @opts) = @_;
    my $spec = $pkg_or_obj->SUPER::_additional_common_validate_spec(@opts);
    my $ontap_additional_spec = $pkg_or_obj->_get_additional_common_validate_spec();
    $spec = {%$ontap_additional_spec, %$spec};
    return $spec;
}

sub _is_sdot {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    if(!defined($self->{is_sdot})) {
        $self->_calculate_sdot_values(@args);
    }
    $Log->exit() if $may_exit;
    return $self->is_sdot();
}

sub _is_sdot_multiple_nodes {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    if(!defined($self->{is_sdot_multiple_nodes})) {
        $self->_calculate_sdot_values(@args);
    }
    $Log->exit() if $may_exit;
    return $self->is_sdot_multiple_nodes();
}

sub _calculate_sdot_values {
    my ($self, %opts) = @_;
    my $set_sdot_zero = sub {
        $self->is_sdot(0);
        $self->is_sdot_multiple_nodes(0);
    };
    if ($self->is_7mode()) {
        $set_sdot_zero->();
    } else {
        my @failover_objs = NACL::CS::StorageFailover->fetch(
            command_interface => $self,
            requested_fields => [qw(ha-type)],
            %opts
        );
        my $sdot_flag = 0;
        for(my $i=0; $i<scalar(@failover_objs);$i++) {
           $sdot_flag=1 if($failover_objs[$i]->ha_type() =~ /non_shared_storage/);
           last if($sdot_flag);
           }
        if ($sdot_flag) {
            $self->is_sdot(1);
            if (@failover_objs > 1) {
                $self->is_sdot_multiple_nodes(1);
            } else {
                $self->is_sdot_multiple_nodes(0);
            }
        } else {
            $set_sdot_zero->();
        }
    }
}

1;

package GroupByVersion;

use Class::MethodMaker [
    new    => [ '-hash', 'new', ],
    hash   => 'grouped_by_version',
    scalar => 'get_highest_version_in_cluster',
    scalar => 'get_lowest_version_in_cluster',
    array  => 'get_nodes_in_highest_version',
    array  => 'get_nodes_in_lowest_version',
    array  => 'node_objs_in_highest_version',
    array  => 'node_objs_in_lowest_version',
];

1;
