#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Component

=head1 DESCRIPTION

NACL::C::Component is an abstract base class for "Component" objects
at the NACL Component layer.  The other significant abstract base
class at this layer is NACL::CS::ComponentState, for "ComponentState"
objects; for every derived class of NACL::C::Component, there would
generally be a corresponding derived class of
NACL::CS::ComponentState.

A Component represents a perl object used to influence (query, create,
modify, delete, or otherwise fiddle with) some kind of thing (called
an "element") outside of the perl process.  Almost all methods on a
component (besides internal helper type methods) are expected to go
out to that element and modify it or query its status.

=head1 ATTRIBUTES

=head2 command_interface

(Abstract; almost all derived classes should provide this themselves,
generally also constraining it to be a particular subclass of
NACL::C::CommandInterface appropriate to this component).  An instance of
a subclass of NACL::C::CommandInterface, which represents what element
should receive the commands commands to influence this element.
(Example: if this component is supposed to operate by sending commands
to a particular node, its command_interface would be a component
representing that node).

=head2 I<(keys)>

(Abstract; almost all derived classes should provide these).
Components will generally have one or a few attributes needed to
uniquely identify the given element when talking to the given command
interface.  Components should generally not have any other pieces of
information about the element itself other than these keys (that
belongs in the ComponentState).

=cut

package NACL::C::Component;

use strict;
use warnings;
use feature 'state';

use Params::Validate qw(validate validate_with validate_pos :types);

# Turn Params::Validate failures into NATE::Exceptions::Argument
BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

use Scalar::Util qw(blessed);
use Time::HiRes qw(time);
use NATE::BaseException qw(:try);

use NACL::Exceptions::InvalidPrivilege;
use NATE::Exceptions::Argument;
use NACL::APISet::Exceptions::ConnectionFailedException  ();
use NACL::APISet::Exceptions::ResponseException          ();
use NACL::APISet::Exceptions::NoMatchingEntriesException ();
use NACL::APISet::Exceptions::TimeoutException           ();
use NACL::Exceptions::VerifyFailure                      ();
use NACL::Exceptions::InvalidMethod                      ();
use NACL::Exceptions::NoElementsFound                    ();
use NACL::Exceptions::Timeout                            ();
use NATE::ParamSet qw( param_global);
use Carp qw(cluck);
use autouse 'NACL::GeneralUtils' => qw(nacl_method_retry);
# Make these accessible on components objects
use autouse 'NACL::GeneralUtils' => qw(_copy_common_component_params
    _copy_or_move_common_params _copy_common_component_params_with_ci
    _move_common_component_params coredump_on_timeout);
use NACL::CS::ComponentState;

use Tharn qw( sleep);
use NATE::Log qw(log_global);
my $Log = log_global();
use Data::Dumper;

use NACL::C::UnitNormalization ();

use Class::MethodMaker [ new => [ '-hash', '-init', 'new' ], ];

=head1 METHODS (for component users)

=head2 new

  my $widget = NACL::C::Widget->new(command_interface=>$ci,...);

Discover and return a new component object with the given attributes.

=head2 find

  my $widget  = NACL::C::Widget->find(command_interface=>$ci,...);
  my @widgets = NACL::C::Widget->find(command_interface=>$ci,...);

A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This is analogous to the "fetch" method in the corresponding component
state object, which takes very similar arguments but constructs
component states instead of components.  The component base class
comes with a "find" implementation which is a wrapper around the
corresponding "fetch" method.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Usually Required) The NACL::C::CommandInterface that will be used to
discover the list of elements.  This will also become the value of the
C<command_interface> attribute of the component objects that C<find>
creates and returns, so the methods on those components will continue
to operate by communicating with this command interface.

This option is generally optional for components which are themselves
command interfaces; when not specified, the command interface
components are discovered by examining data passed in by the user
(parameters, host record objects) and they will effectively be
constructed with themselves set to be their own command_interface.

=item C<< filter=>{ attribute1=>$value1, attribute2=>$value2, ... } >>

(Optional) A hashref that limits the components returned to be only
those whose attributes have the given values (If possible, the
commands/requests sent will be optimized to limit discovery
similarly).  The attribute names should match the fields used in the
ComponentState object corresponding to this Component.

=item C<< apiset_must=>$ruleset >>

(Optional) A hashref constraining which API (mode, interface, etc.) on
the given command_interface will be used.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< apiset_should=>$ruleset >>

(Optional) A hashref suggesting which API (mode, interface, etc.) on
the given command_interface is preferred.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< allow_empty=>$boolean >>

(Optional) If false (the default) then throw a NACL::Exceptions::NoElementsFound
exception if no elements are found.  If true, then return undef or an empty list
if no elements are found.

=item C<< 'method-timeout' => $timeout_in_seconds >>

(Optional) We can use this option to specify a large timeout value, which will
be used by all command(s) run, in case the command(s) take a long time to run
due to  large amounts of output.

=back

=back

=cut

sub find {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my $pkg = $pkg_or_obj->get_package_name();

    # If invoked with an object, extract the primary keys and send that
    # as the filter. Also extract the command interface.
    $pkg_or_obj->_extract_from_obj_for_find(arr_ref => \@args);

    my %opts = validate @args, $pkg->_find_validate_spec();
    my @requested_fields = (
        $pkg->get_primary_keys(
            command_interface => $opts{command_interface}
        )
    );

    # This will be the associated CS package
    # Example: NACL::C::Aggregate -> NACL::CS::Aggregate
    my $CS_pkg = $pkg->get_CS_package_name();

    my @state_objs
        = $CS_pkg->fetch( %opts, requested_fields => \@requested_fields );
    my @components;
    foreach my $state_obj (@state_objs) {
        my %new_hash;
        unless (@requested_fields) {

            # Inform the user to go through CS::fetch instead since there
            # are no primary keys for this component
            $Log->warn( 'There are no primary keys for the package '
                    . "'$pkg', hence $CS_pkg->fetch() should be invoked "
                    . "instead of $pkg->find()." );
        }
        foreach my $requested_field (@requested_fields) {
            my $value = $state_obj->$requested_field();
            $new_hash{$requested_field} = $value if ( defined $value );
        }
        %new_hash
            = %{ NACL::C::Component::_convert_to_underscores( \%new_hash ) };
        push @components,
            $pkg->new( %new_hash,
            command_interface => $opts{command_interface}, );
    }

    $Log->exit;
    return wantarray ? @components : $components[0];
} ## end sub find

# If find() was invoked with an object, then extract the primary keys and
# command_interface from it and send that along as the filter.
sub _extract_from_obj_for_find {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { arr_ref => { type => ARRAYREF } },
    );
    if (ref $pkg_or_obj) {
        my %filter = $pkg_or_obj->get_primary_keys_hash();
        my @additional_array = (
            filter            => \%filter,
            command_interface => $pkg_or_obj->command_interface()
        );
        my $arr_ref = $opts{arr_ref};
        @$arr_ref = (@$arr_ref, @additional_array);
    }

    $Log->exit();
}

=head2 find_or_create

    my $foo_obj = NACL::C::Foo->find_or_create(
        command_interface       => $ci,
        %options_common_to_find_and_create,
        create_specific_options => { %options_to_be_used_only_for_create },
        find_specific_options   => { %options_to_be_used_only_for_find },
    );

                                or

    my @foo_objs = NACL::C::Foo->find_or_create(
        command_interface       => $ci,
        %options_common_to_find_and_create,
        create_specific_options => { %options_to_be_used_only_for_create },
        find_specific_options   => { %options_to_be_used_only_for_find },
    );

(For component users)
(Class method)
This method checks whether element(s) matching the specified criteria is/are
present and if so returns them. If there are no elements found matching the
specified criteria, then a new element of that type is created and returned to
the user.

Similar to find, it is callable in either scalar or list context. In scalar
context it returns an object for the first element that matched, in list
context it returns objects for all elements that matched. Even if no matching
elements are found this method is still callable in either scalar or list
context, though in list context it will simply return an array containing one
component object (the one that was created).

=over

=item Options

=over

=item command_interface

(Required, of type 'NACL::C::CommandInterface')
The command interface on which to attempt finding or creating the element

=item find_specific_options

(Optional, HASHREF)
An important consideration is the fact that not all parameters which can be
used as filters for the find can be used in a call to create. (example: we can
search for aggregates with a specified amount of available size but we cannot
create aggregates with a specified amount of available size). All such options
must be sent in the "find_specific_options" parameter.

=item create_specific_options

(Optional, HASHREF)
Similar to "find_specific_options", there might be some options which we're
not interested in while trying to find elements but which have to be provided
for a call to create to succeed (example: we might want any aggregate which a
specified amount of available size regardless of the name, but if it does not
find any matching aggregates the name is required for the aggregate creation).
All such options should be sent in the "create_specific_options" parameter.

=item %options_common_to_find_and_create>

These are options which are common to both find and create.

=item C<< _was_created => \$scalar >>

(Optional)
For cleanup purposes we need to know whether an element was created or not.
We can use this parameter to determine whether the returned object was
for an element that just got created (as opposed to being found).

A reference to a scalar should be passed in, its value gets filled in to
describe whether it was created or not. If its value is "0", then the
element was not created (i.e. was found), if its value is "1" then the
element was created.

Here's a simple example of using this option:

    my $was_created;
    my $aggregate_obj = NACL::STask::Aggregate->find_or_create(
        %other_opts,
        _was_created => \$was_created
    );
    # Value of $was_created now reflects whether the aggregate was created or not

    # Operate on $aggregate_obj here
    # ...
    # Determine whether we need to cleanup the aggregate or not
    if ($was_created) {
        # Need to delete the aggregate since find_or_create created it
        $aggregate_obj->purge();
    }

=item C<< custom_create_command => $cmd >>

(Optional)
If the command to create a new element is not named "create", but is instead
named something else (such as "add"), then this option should be used to
specify the name of this comand.

=back

=back

Here's a specific example to demonstrate using this method:

    NACL::C::Aggregate->find_or_create(
        command_interface       => $node,
        diskcount               => 4,
        node                    => $node,
        create_specific_options => { aggregate => 'aggr1' },
        find_specific_options   => { availsize => '1048576' }
    );

This searches on $node for aggregates which have 4 disks and have 1MB
available size. If none are found, it creates an aggregate named "aggr1" which
has 4 disks on $node.

=cut

sub find_or_create {
    $Log->enter();
    my $pkg = shift;

    if ( $Log->may_debug() ) {
        my %args = @_;
        $Log->debug(
            sub {
                "Opts to 'find_or_create':\n"
                    . NACL::ComponentUtils::Dumper( \%args );
            }
        );
    }

    my %opts = validate_with(
        params => \@_,
        spec   => {
            create_specific_options => { type => HASHREF, default => {} },
            find_specific_options   => { type => HASHREF, default => {} },
            if_exists_with_different_attributes => {
                type      => SCALAR,
                default   => 'die',
                callbacks => {
                    "Value of 'if_exists_with_different_attributes' must "
                        . "be either 'die' or 'purge'" =>
                        sub { $_[0] =~ /^(die)|(purge)$/i }
                }
            },
            _was_created => { type => SCALARREF, optional => 1 },
            custom_create_command => { type => SCALAR, default => 'create' },
        },

        # Allow through options common to both find and create
        allow_extra => 1
    );

    my $create_specific_options = delete $opts{create_specific_options};
    my $find_specific_options   = delete $opts{find_specific_options};
    my $action         = delete $opts{if_exists_with_different_attributes};
    my $create_command = delete $opts{custom_create_command};

    # Was the element created or not (i.e. was found)? Default to 0,
    # set to 1 if created later.
    my $was_created = delete $opts{_was_created};
    $$was_created = 0;
    my $is_stask = $pkg->isa('NACL::STask::STask');

    if ( $action =~ /purge/i && !$pkg->can('purge') ) {
        my $msg
            = 'The action for the argument '
            . "'if_exists_with_different_attributes' was 'purge' "
            . "but purge() has not been implemented on '$pkg'";
        if ( !$is_stask ) {
            $msg .= ' (this is because this is a component package)';
        }
        $Log->exit();
        NATE::Exceptions::Argument->throw("$msg\n");
    }

    my %create_options = ( %opts, %$create_specific_options );
    my %filter_options = (%opts, %$find_specific_options);

   # Divide up the find options to determine what should be part of the filter
    my @common_options = keys %{ $pkg->_common_validate_spec() };
    my %common_opts_with_ci;
    $pkg->_hash_move(
        source => \%filter_options,
        target => \%common_opts_with_ci,
        move   => \@common_options
    );

    # Let's try to figure out if the user provided all the primary keys
    # If they have, then get the state of it, compare it against
    # the find criteria and determine what to do based on
    # 'if_exists_with_different_attributes'
    my $all_pk_specified = 1;
    my $primary_keys     = $pkg->get_primary_keys_options(%filter_options);
    try {

       # _primary_keys_validate_spec is not sufficient to determine
       # if all primary keys were provided (it does not contain information
       # of mutually exclusive sets of attributes, but simply lists them
       # all as optional). Calling new however will tell us if all the primary
       # keys were provided (the "init" method should take care of throwing
       # an exception if we haven't supplied all the necessary parameters)
       #
        $pkg->new( %$primary_keys,
            command_interface => $common_opts_with_ci{command_interface} );

        # Check if any of the primary keys has special characters
        while (my ($key, $value) = each %$primary_keys) {
            if ($pkg->_check_relational_regex_filter(filter_value => $value)) {
                $all_pk_specified = 0;
                last;
            }
        }
    }
    otherwise {

        # One or more of the required primary keys was not provided
        $all_pk_specified = 0;
    };

    # The variable naming here calls it "components". They might be STask
    # objects as well, but technically all STasks are components, so this
    # naming is to just keep the name short
    my ( @components, $need_to_create );
    if ($all_pk_specified) {

        # All primary keys provided - i.e. enough to limit to a
        # single element
        my $CS_pkg = $pkg->get_CS_package_name();
        my $CS_obj = $CS_pkg->fetch(
            %common_opts_with_ci,
            filter           => $primary_keys,
            requested_fields => [ keys %filter_options ],
            allow_empty      => 1
        );
        if ($CS_obj) {

            # Element exists, verify all attributes match
            try {
                $CS_obj->verify_fields(%filter_options);

                # This is going to be revisited once a layer above
                # STasks appears. (Code similar to that in find for
                # turning a CS into a C object will need to be used here)
                # For now let's just use get_<comp_or_task>_instance
                if ($is_stask) {
                    push @components, $CS_obj->get_task_instance();
                } else {
                    push @components, $CS_obj->get_component_instance();
                }
            }
            catch NACL::Exceptions::VerifyFailure with {

                # One or more attributes don't match.
                my $exception = shift;
                if ( $action =~ /^die$/i ) {
                    $Log->exit();
                    $exception->throw();
                } else {

                    # Purge
                    my $task_obj    = $CS_obj->get_task_instance();
                    my %common_opts = %common_opts_with_ci;
                    delete $common_opts{command_interface};
                    $task_obj->purge(%common_opts);
                    $need_to_create = 1;
                }
            };
        } else {
            $need_to_create = 1;
        }
    } else {
        $common_opts_with_ci{filter} = \%filter_options;
        try {
            @components = $pkg->find(%common_opts_with_ci);
        }
        catch NACL::Exceptions::NoElementsFound with {
            $need_to_create = 1;
        };
    }

    if ($need_to_create) {
        push @components, $pkg->$create_command(%create_options);
        $$was_created = 1;
    }

    $Log->exit();
    return wantarray ? @components : $components[0];
}

=head2 state

  my $widget_state  = NACL::C::Widget->state();

(Instance method).  Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on a NACL::C::Foo would return a
NACL::CS::Foo object)

The base class provides an implementation of this (based on the
individual Component's C<_primary_keys_validate_spec> method and the
individual Component State's C<fetch> method)

=over

=item Options

=over

=item C<< requested_fields=>['name1', 'name2', ...] >>

(Optional) An array reference containing the names of component state
fields that must be filled in; others are allowed (but not required)
to be undefined.  If the requested_fields option is undef or not given,
it is assumed that all component state fields should be filled in.  An
optimal component implementation will limit the commands it sends,
their verbosity, their columns, etc., when doing so will still fetch
the given fields, but simpler component implementations may just
ignore this option and always query for all of the state of this
element that they can.

=item C<< apiset_must=>$ruleset >>

(Optional) A hashref constraining which API (mode, interface, etc.) on
the given command_interface will be used.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< apiset_should=>$ruleset >>

(Optional) A hashref suggesting which API (mode, interface, etc.) on
the given command_interface is preferred.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< 'method-timeout' => $timeout_in_seconds >>

(Optional) We can use this option to specify a large timeout value, which will
be used by all command(s) run, in case the command(s) take a long time to run
due to  large amounts of output.

=back

=back

=cut

sub state {
    $Log->enter();
    my ($self, @args) = @_;

    unless (ref $self) {
        NATE::BaseException->throw(
                  "'state' cannot be invoked with a package "
                . "name, it must be invoked only with an object instance" );
    }

    my %opts        = validate @args, $self->_state_validate_spec();
    my $filter_hash = $self->_extract_filter_hash_for_state(%opts);

    my $CS_pkg = $self->get_CS_package_name();

    my $cs_obj = $CS_pkg->fetch(
        %opts,
        command_interface    => $self->command_interface(),
        filter               => $filter_hash,
        _ignore_tharn_params => 1
    );
    $Log->exit();
    return $cs_obj;
} ## end sub state

sub _extract_filter_hash_for_state {
    $Log->enter();
    my $self = $_[0];
    my @filter_fields = $self->get_primary_keys();
    my %filter_hash   = ();
    foreach my $filter_field (@filter_fields) {
        my $field_isset = $filter_field . '_isset';
        if ( $self->$field_isset() ) {
            $filter_hash{$filter_field} = $self->$filter_field();
        }
    }
    $Log->exit();
    return \%filter_hash;
}

=head2 get_one_state_attribute

    my $size = $aggr_obj->get_one_state_attribute('size');

(For component users)
This method can be used to retrieve any one state attribute.
Note that it is a simplified version of doing
    $comp_obj->state(requested_fields => [ qw($field) ])->$field()

=over

=item Options

=over

=item (1)

(Required)
The attribute needs to be specified as the first parameter (un-named,
positional parameter)

=item C<< apiset_must => $ruleset >>

(Optional) A hashref constraining which API (mode, interface, etc.) on
the given command_interface will be used.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< apiset_should => $ruleset >>

(Optional) A hashref suggesting which API (mode, interface, etc.) on
the given command_interface will be used.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< 'method-timeout' => $timeout_in_seconds >>

(Optional) We can use this option to specify a large timeout value, which will
be used by all command(s) run, in case the command(s) take a long time to run
due to  large amounts of output.

=back

=back

=cut

sub get_one_state_attribute {
    $Log->enter();
    my $self      = shift;
    my $attribute = shift;

    # get_one_state_attribute also accepts apiset_(must|should) and
    # method-timeout, which is _common_validate_spec() - command_interface
    my %opts = validate @_, $self->_common_validate_spec_without_ci();
    unless ( defined $attribute ) {
        $Log->exit();
        NATE::Exceptions::Argument->throw( "An attribute should be specified "
                . "as positional param while invoking get_one_state_attribute"
        );
    }

    $Log->exit();
    return $self->state( requested_fields => [$attribute], %opts )
        ->$attribute();
}

=head2 verify_state

    $obj->verify_state(
        %primary_keys,
        %fields_to_be_verified
    );

             or

    $obj->verify_state(%fields_to_be_verified);

(Class or instance method)
This method can be used to verify whether all the state fields of the element
match the values provided. It throws a
L<NACL::Exceptions::VerifyFailure|lib-NACL-Exceptions-VerifyFailure-pm>
exception if one or more of the fields does not match the values we
specified.

=over

=item Options

=over

=item C<primary keys>

(Required for class method, Not Applicable for instance method)
The primary keys of the component.

=item C<fields to verify>

(Required)
This is a hash which specifies the fields to verify and the values which they
should have.

=item C<< nacl_return_boolean => 0|1 >>

(Optional, defaults to 0)
If set to 1, this method will not throw an exception but will instead
return a boolean value (0/1) denoting whether all fields matched.

=back

=back

A simple example call:

    # Verify that aggregate modeled by $aggr_obj is online and is 64-bit
    $aggr_obj->verify_state('block-type' => '64-bit', state => 'online');

    # Use of nacl_return_boolean
    if ($aggr_obj->verify_state('block-type' => '64-bit', state => 'online', nacl_return_boolean => 1)) {
        # ... Is online and 64-bit ...
    }

=cut

sub verify_state {
    $Log->exit();
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { nacl_return_boolean => { type => SCALAR, default => 0 }, },
        allow_extra => 1
    );

    # If can() is invoked with an object, then it will perform extra steps to
    # lookup the CDEFs and help_xml files, which we can skip since we know it
    # isn't a command. The package call is used to prevent this.
    if ($pkg_or_obj->get_package_name()->can('_remove_invalid_verify_state_opts')) {
        %opts = $pkg_or_obj->_remove_invalid_verify_state_opts(%opts);
    }

    my $return_boolean = delete( $opts{nacl_return_boolean} );
    my $CS_pkg         = $pkg_or_obj->get_CS_package_name();
    my %common_opts;
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => \%common_opts,
        move   => [ keys %{ $pkg_or_obj->_common_validate_spec() } ]
    );

    my %pri_keys;
    $pkg_or_obj->_move_primary_keys(
        source => \%opts,
        target => \%pri_keys
    );
    my %fetch_opts;
    $fetch_opts{filter}           = \%pri_keys;
    $fetch_opts{requested_fields} = [ keys %opts ];
    %fetch_opts = ( %fetch_opts, %common_opts );
    my $cs    = $CS_pkg->fetch(%fetch_opts);
    my $match = 1;
    try {
        $cs->verify_fields(%opts);
    }
    catch NACL::Exceptions::VerifyFailure with {
        my $ex = shift;
        $Log->debug("VerifyFailure exception is thrown");
        $match = 0;
        unless ($return_boolean) {
            $Log->exit();
            $ex->throw();
        }
    };
    $Log->exit();
    return $match;
}

=head2 get_primary_keys

    my @primary_keys = $pkg_or_obj->get_primary_keys();

Get an array containing the list of primary keys (keys of
the options hash initializer returned by C<<
$pkg_or_obj->_primary_keys_validate_spec >> ).

=cut

sub get_primary_keys {
    $Log->enter();
    my ( $pkg_or_obj, %opts ) = @_;

    my %primary_keys_spec = $pkg_or_obj->_primary_keys_validate_spec(%opts);
    my @all_keys          = keys %primary_keys_spec;
    unless ( ref $pkg_or_obj ) {

        # Return all keys if invoked as a package method
        $Log->debug(
            "Invoked as a package method. Returning all primary keys");
        $Log->exit();
        return @all_keys;
    } else {
        $Log->debug( "Invoked as an instance method. Returning only "
                . "keys with defined values" );
        my @primary_keys;
        foreach my $key (@all_keys) {
            if ( defined $pkg_or_obj->$key() ) {
                push @primary_keys, $key;
            }
        }
        $Log->exit();
        return @primary_keys;
    }
} ## end sub get_primary_keys

=head2 get_pk_hash_with_ci

    my %vol_hash = $vol_obj->get_pk_hash_with_ci();
    NACL::C::Efficiency->start(%vol_hash);

(For component users)
This method returns a hash containing the primary keys of the component along
with its command_interface.

This facilitates using existing component objects for performing functionality
on separate components. The above example demonstrates how to start efficiency
on a volume for which we have a volume component object.

=cut

sub get_pk_hash_with_ci {
    $Log->enter();
    my $self            = shift;
    my %pk_hash_with_ci = $self->get_primary_keys_hash();
    $pk_hash_with_ci{command_interface} = $self->command_interface();
    $Log->exit();
    return %pk_hash_with_ci;
}

=head2 get_primary_key_string

    my $pk_string = $component->get_primary_key_string();

This method returns a string which is a semi-colon separated list of primary
keys and their values (in the form primary_key=value1;primary_key2=value2;...)

=cut

sub get_primary_key_string {
    my $self         = shift;
    my @primary_keys = $self->get_primary_keys();
    my $str = '';

    foreach my $primary_key ( sort @primary_keys ) {
        my $value = $self->$primary_key();
        $str .= "$primary_key=$value;";
    }

    chop $str;
    return $str;
}

=head2 get_primary_keys_hash

    my %vol_hash = $vol_obj->get_primary_keys_hash();
    my $efficiency = NACL::C::Efficiency->find(
        command_interface => $ci,
        filter            => { %vol_hash }
    );

(For component users)
This method returns a hash which contains the primary keys and their values
for a component object.

This facilitates being able to use existing component objects to get objects
for related elements. The above example demonstrates how to check if
efficiency is enabled on a volume (for which we have a volume component
object).

=cut

sub get_primary_keys_hash {
    $Log->enter();
    my $self         = shift;
    my @primary_keys = $self->get_primary_keys();
    my %pk_hash;
    foreach my $primary_key (@primary_keys) {
        $pk_hash{$primary_key} = $self->$primary_key();
    }
    $Log->exit();
    return %pk_hash;
}

# as get_primary_keys_hash, but getting from options (as needed by
# class method calls) rather than from an object instance.

# also, a little difference for returning a hashref instead of a hash

sub get_primary_keys_options {
    $Log->enter();
    my $pkg_or_obj   = shift;
    my @primary_keys = $pkg_or_obj->get_primary_keys();
    my %opts         = @_;
    my %pk_hash;
    foreach my $primary_key (@primary_keys) {
        if ( defined( $opts{$primary_key} ) ) {
            $pk_hash{$primary_key} = $opts{$primary_key};
        }
    }
    $Log->exit();
    return \%pk_hash;
}

=head2 wait_for_completion

    # Waiting on a job and a single attribute
    $pkg_or_obj->wait_for_completion(
        'method-timeout'   => $large_time,
        job_component      => $job,
        attribute_to_check => $attr,
        till_value         => $val,
        %primary_keys, # Needed only for class calls
        %other_opts    # See wait_on_attribute for other arguments accepted
    );

    # Waiting on a job and multiple attributes
    $pkg_or_obj->wait_for_completion(
        'method-timeout'   => $large_time,
        job_component      => $job,
        # See wait_on_attributes for structure of the "attributes" argument
        attributes         => \@attributes_arrayref,
        %primary_keys,   # Needed only for class calls
        %other_opts
    );

(Class or instance method)
This method is used to wait for completion of a certain operation. This is
done by waiting on a job and on one or more attributes.
Though it is possible to use this method to only wait on a job, or only
on an attribute (or attributes) it is suggested that this method is used
when we would like to wait on both a job and on attribute(s).

If the operation does not complete within the timeout value specified,
then a L<NACL::Exceptions::Timeout|lib-NACL-Exceptions-Timeout-pm>
exception is thrown.

(For only waiting on a job or on attributes, the following methods can be
used directly:

L<wait_on_job|lib-NACL-C-Component-pm/wait_on_job> can be used to wait
on a job.

L<wait_on_attribute|lib-NACL-C-Component-pm/wait_on_attribute> can be used
to wait on a single attribute.

L<wait_on_attributes|lib-NACL-C-Component-pm/wait_on_attributes> can be used
to wait on multiple attributes.
)

This method accepts a job_component as argument along with arguments accepted
by either C<wait_on_attribute> or C<wait_on_attributes>. (Providing
arguments of both C<wait_on_attribute> and C<wait_on_attributes> results
in failure)

If the arguments of C<wait_on_attribute> are provided, then a single
attribute is waited on after the job has completed.

If the arguments of C<wait_on_attribute> are provided, then multiple
attributes are waited on after the job has completed.

=over

=item Options

=over

=item C<primary keys>

If invoked as a class method, the primary keys must be specified.
If invoked as an instance method, the primary keys must not be specified.

=item C<< 'method-timeout' => $time_to_wait >>

(Required)
Time in seconds to wait for the completion of operation.

=item C<< polling_interval => $interval >>

(Optional)
This is the interval (in seconds) after which the value of the attribute will
be polled. If not specified it defaults to 10 seconds.

=item C<< job_component => $job_component >>

(Optional, is required if neither C<attribute_to_check> nor C<attributes>
is provided)

This is a reference to the job component object (is of type C<NACL::C::Job>).
If provided, it uses L<wait_on_job|lib-NACL-C-Component-pm/wait_on_job>
internally to wait on the job.

(In 7Mode, this argument is simply ignored)

=item Arguments accepted by C<wait_on_attribute>

See L<wait_on_attribute|lib-NACL-C-Component-pm/wait_on_attribute>.

(These should not be provided if the arguments of C<wait_on_attributes> have
been provided)

=item Arguments accepted by C<wait_on_attributes>

See L<wait_on_attribute|lib-NACL-C-Component-pm/wait_on_attributes>.

(These should not be provided if the arguments of C<wait_on_attribute> have
been provided)

=item C<< check_job_then_attribute => 0 | 1 >>

(Optional, defaults to 1)
Certain builds appear to suffer from a bug due to which the job entry in the
job table is removed before the operation has completed. This means that the
wait_on_job method will also return before the operation has completed.
This parameter allows us to start polling on the attribute after the job
has claimed to have completed. For builds that do not suffer from this bug,
it will simply need to check the attribute once, so it acts as a verification
that the operation did in fact complete.

This is optional and defaults to 1 if not provided (checks both the job
as well as the attribute). Providing 0 as the value for this would have the
effect of only waiting on the job (if one was provided).

=item C<< apiset_must => $ruleset >>

(Optional)
The apiset_must ruleset.

=item C<< check_error_in_wait_for_initialization => HASHREF >>

parameter check_error_in_wait_for_initialization hold the current value of 'last-transfer-type'
when wait_for_initialization is called from NACL::STask::Snapmirror library.

=item C<< apiset_should => $ruleset >>

(Optional)
The apiset_should ruleset.

=back

=back

=cut

sub wait_for_completion {
    $Log->enter();
    my ($pkg_or_obj, %opts)                         = @_;
    my %wait_on_attributes_validation_spec = ();
    if (exists $opts{attribute_to_check}) {
        %wait_on_attributes_validation_spec = (
            $pkg_or_obj->_common_wait_attr_validate_spec(),
        );
    }
    if (exists $opts{attributes}) {
        %wait_on_attributes_validation_spec = (
             %wait_on_attributes_validation_spec,
             attributes => { type => ARRAYREF, optional => 1 },
             show_all_fields => { type => BOOLEAN, default => 0 },
         );
    }
    %opts = $pkg_or_obj->_common_validate_with(
        params          => [%opts],
        additional_spec => {
            $pkg_or_obj->_common_wait_for_validate_spec(),
            %wait_on_attributes_validation_spec,
            job_component => { type => OBJECT | UNDEF, optional => 1 },
            check_job_and_attribute => { type => SCALAR, default => 1 },
            check_error_in_wait_for_initialization => { type => HASHREF , optional => 1 },

            # Primary keys are necessary for wait_on_attribute(s) but
            # not necessarily for wait_for_completion so making them
            # optional here
            _optional_scalars( $pkg_or_obj->get_primary_keys() ),
        }
    );
    my $check_error = delete $opts{check_error_in_wait_for_initialization};

    if ( $opts{attribute_to_check} && $opts{attributes} ) {
        $Log->exit();
        NATE::Exceptions::Argument->throw( "Either of 'attributes or "
                . "'attribute_to_check' can be provided, but not both" );
    }

    # Call "wait_on_attribute" if "attribute_to_check" is provided.
    # Call "wait_on_attributes" if "attributes" is provided.
    # If neither are provided, then this is undef
    my $wait_for_method_to_call;
    if ( $opts{attribute_to_check} ) {
        $wait_for_method_to_call = 'wait_on_attribute';
    } elsif ( $opts{attributes} ) {
        $wait_for_method_to_call = 'wait_on_attributes';
    }

    my $pkg                     = $pkg_or_obj->get_package_name();
    my $check_job_and_attribute = delete $opts{check_job_and_attribute};
    my $job                     = delete $opts{job_component};
    if ( defined $job ) {
        my %job_opts = %opts;

        if ( !$job->isa('NACL::C::Job') ) {
            my $ref = ref $job;
            $Log->exit();
            NATE::Exceptions::Argument->throw( "'job_component' parameter "
                    . "is supposed to be of type 'NACL::C::Job', but was "
                    . "sent as '$ref'" );
        }

        # Remove the options not valid for the wait_on_job call: these
        # are the ones necessary for waiting on attributes.
        $pkg->_hash_move(
            source => \%job_opts,
            target => {},
            move   => [ keys %wait_on_attributes_validation_spec ]
        );

        my $initial_time = time();
        $pkg->wait_on_job( %job_opts, job_component => $job );
        if ($check_job_and_attribute) {
            if ($wait_for_method_to_call) {
                my $scaling_factor
                    = $opts{command_interface}->timeout_scaling_factor() || 1;
                my $time_elapsed = time() - $initial_time;

                # Take into account the scaling factor by dividing the
                # actual time elapsed by the scaling factor.
                my $time_remaining = $opts{'method-timeout'}
                    - ( $time_elapsed / $scaling_factor );
                $Log->trace("Waited on job for $time_elapsed seconds");
                $Log->trace( 'Polling on attribute(s) now for remaining time '
                        . "($time_remaining seconds)" );

                $opts{check_error_in_wait_for_initialization} = $check_error if(defined $check_error);
                $pkg->$wait_for_method_to_call( %opts,
                    'method-timeout' => $time_remaining );
            }
        }
    } else {
        if ($wait_for_method_to_call) {
            $Log->trace('Waiting on attribute(s)');
            $opts{check_error_in_wait_for_initialization} = $check_error if(defined $check_error);
            $pkg->$wait_for_method_to_call(%opts);
        } else {
            $Log->exit();
            NATE::Exceptions::Argument->throw( "When 'job_component' is not "
                    . "provided either of 'attribute_to_check' or 'attributes' "
                    . 'is mandatory' );
        }
    }
    $Log->exit();
}

=head2 wait_on_attribute

    # More specific examples shown in "Examples" section below
    $pkg_or_obj->wait_on_attribute(
        'method-timeout'   => $time_to_wait,
        attribute_to_check => $attribute,
        till_value         => $till_val | \@till_vals,
        [till_not_value    => $till_not_val,]
        [valid_intermediate_states => \@valid_states ,]
        [invalid_states    => \@invalid_states ,]
        [polling_interval  => $polling_interval, ]
        [%primary_keys]      # Necessary only for class calls
    );

(Class or instance method)
This method can be used to wait for a state attribute to become a particular
value (or one of a number of values).

The value(s) to wait for need to be provided through the C<till_value> parameter.
This method waits for the time specified through C<method-timeout>, polling for
the value of the specified attribute after every delta seconds. If the specified
attribute reaches one of the specified values within the specified time,
the method returns, else it throws a
L<NACL::APISet::Exceptions::TimeoutException|lib-NACL-APISet-Exceptions-TimeoutException-pm>.

This method also specifies a mechanism to specify valid intermediate states
of this attribute. The attribute reaching any value other than the final
state(s) or any of the valid intermediate states results in a
L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>
exception being thrown.

For instances where we need to wait on both a job as well as on an attribute
L<wait_for_completion|lib-NACL-C-Component-pm/wait_for_completion> can be used.

=over

=item Options

=over

=item C<primary keys>

If invoked as a class method, the primary keys must be specified.
If invoked as an instance method, the primary keys must not be specified.

=item C<< 'method-timeout' => $time_to_wait >>

(Required)
Time in seconds to wait for the completion of operation

=item C<< attribute_to_check => $attribute_to_check  >>

The attribute of Component state object whose value need to be polled
to identify the completion of operation

=item C<< till_value => $till_value | \@till_values >>

(Optional, is required if C<job_component> is not provided)
This parameter can be specified as a scalar or as an array-reference

If specified as a scalar, it is the value of the attribute which denotes
the completion of the operation.

If specified as an array-reference, it means that when the attribute reaches
any of the values specified, then the operation has completed.

Note that the values can be specified as either a string or as a regular
expression. When provided as a string, an equality check is performed
between the attribute value obtained from the product and the user-specified
string value. When provided as a regular expression, a regex check is performed
between the product obtained attribute value and the provided regular
expression.

=item C<< till_not_value => $till_not_value >>

(Optional)
This parameter specifies that the method should wait till the attribute's
value becomes any value other than what's specified through this parameter.

This parameter must be a scalar, though it can be either a string or a regex.

For example, while disks are being added to an aggregate, the raidstatus
contains the string "growing". So to wait till disks get added, we need to
wait till raidstatus does not contain "growing".

=item C<< valid_intermediate_states => \@intermediate_states >>

(Optional)
This parameter can be used to specify valid states the attribute can be
in prior to reaching the final state(s). The method will throw a
L<NACL::Exceptions::UnexpectedState|lib-NACL-Exceptions-UnexpectedState-pm>
exception if the attribute value reaches an unexpected state (i.e. any state
other than the final(s) states and the intermediate states)

Note that the values can be specified as either a string or as a regular
expression. When provided as a string, an equality check is performed
between the attribute value obtained from the product and the user-specified
string value. When provided as a regular expression, a regex check is performed
between the product obtained attribute value and the provided regular
expression.

=item C<< invalid_states => \@invalid_states >>

(Optional)
This parameter can be used to specify invalid states, i.e. if the attribute
value becomes any of these values then an error has occured.

Note that the values can be specified as either a string or as a regular
expression. When provided as a string, an equality check is performed
between the attribute value obtained from the product and the user-specified
string value. When provided as a regular expression, a regex check is performed
between the product obtained attribute value and the provided regular
expression.

=item C<< polling_interval => $interval >>

(Optional)
This is the interval (in seconds) after which the value of the attribute will
be polled. If not specified it defaults to 10 seconds.

=item C<< apiset_must => $ruleset >>

(Optional)
The apiset_must ruleset.

=item C<< apiset_should => $ruleset >>

(Optional)
The apiset_should ruleset.

=back

=back

=over

=item Examples

=over

=item Basic example

    $vol_obj->wait_on_attribute(
        attribute_to_check => 'state',
        till_value         => 'online',
        'method-timeout'   => $large_time
    );

In this example, we're waiting for the value of the "state" field to
become "online".

=item Providing valid_intermediate_states

    $vol_obj->wait_on_attribute(
        'method-timeout'   => $large_time,
        attribute_to_check => 'state',
        till_value         => 'online',
        valid_intermediate_states => [ 'creating', 'mounting' ]
    );

Again, waiting for the value of the "state" field to become "online". In
addition, we're also specifying that the only valid intermediate states before
it comes online are "creating" and "mounting", so if the value of the "state"
field reaches any value other than these it will end up failing.

=item Providing valid_intermediate_states as regular expressions

    $vol_quota_obj->wait_on_attribute(
        'method-timeout'   => $large_time,
        attribute_to_check => 'state',
        till_value         => 'on',
        valid_intermediate_states =>
            [ qr(^initializing), qr(^resizing), 'off' ]
    );

Here, we're waiting for the "state" of a volume quota to come "on". Again,
we're using valid_intermediate_states, but we're providing the values
as a combination of strings and regular expressions.

=item Providing till_not_value

    $aggr_obj->wait_on_attribute(
        'method-timeout'   => $large_time,
        attribute_to_check => 'raidstatus',
        till_not_value     => qr/\bgrowing\b/
    );

Checking the raidstatus field till it does B<not> contain "growing".
(When disks are being added to an aggregate, this field contains
"growing". Absence of "growing" means that disks have completed
being added to the aggregate)

=back

=back

=cut

sub wait_on_attribute {
    $Log->enter();
    my $pkg_or_obj      = shift;
    my %additional_spec = $pkg_or_obj->_common_wait_attr_validate_spec();
    my %opts            = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {%additional_spec},

        # Allow through other options such as those defined by
        # _common_wait_for_validate_spec. These will eventually get
        # caught by _wait_on_attributes_helper if they're invalid
        allow_extra => 1
    );

    my $wait_on_attr_obj
        = $pkg_or_obj->_validate_and_return_single_wait_on_attribute(%opts);

    my %all_attribute_entries;
    $all_attribute_entries{ $opts{attribute_to_check} } = $wait_on_attr_obj;

    # Remove all the _common_wait_for_attr attributes since they would have
    # now been moved into the standardized format of all_attribute_entries
    foreach my $key ( keys %additional_spec ) {
        delete $opts{$key};
    }

    my $pkg = $pkg_or_obj->get_package_name();
    $pkg->_wait_on_attributes_helper( %opts,
        all_attribute_entries => \%all_attribute_entries );
    $Log->exit();
}

=head2 wait_on_attributes

    # Using waiting for Snapmirror initialization as an example
    # Wait till the value of state is "Snapmirrored" and status is "Idle"
    $snapmirror_obj->wait_on_attributes(
        'method-timeout' => $time,
        attributes       => [
            {
                attribute_to_check => 'state',
                till_value         => qr/Snapmirrored/i
            },
            {
                attribute_to_check => 'status',
                till_value         => qr/Idle/i
            }
        ]
    );

This method can be used to wait on multiple attributes simultaneously.
This operates similarly to C<wait_on_attribute>, only difference being that
this provides the capability of waiting on more than one attribute at a time.

For instances where we need to wait on both a job as well as on attributes
L<wait_for_completion|lib-NACL-C-Component-pm/wait_for_completion> can be used.

=over

=item Options

=over

=item C<primary keys>

If invoked as a class method, the primary keys must be specified.
If invoked as an instance method, the primary keys must not be specified.

=item C<< 'method-timeout' => $time_to_wait >>

(Required)
Time in seconds to wait for the completion of operation.

=item C<< polling_interval => $interval >>

(Optional)
This is the interval (in seconds) after which the value of the attribute will
be polled. If not specified it defaults to 10 seconds.

=item C<< apiset_must => $ruleset >>

(Optional)
The apiset_must ruleset.

=item C<< apiset_should => $ruleset >>

(Optional)
The apiset_should ruleset.

=item C<< attributes => \@arrayref_of_attribute_options >>

(Required)
This is an array-reference, where each entry is a hash-reference containing
arguments of C<wait_on_attribute> (this would include C<attribute_to_check>,
C<till_value>, C<till_not_value>, C<valid_intermediate_states> and
C<invalid_states>)

=item C<< NACL_SHOW_ALL_FIELDS= 1|0 >>

(Optional)
default 0
This param when set to 1 does not populate -fields in the respective
show command. When set to 0 it appends -fields to the show command.

=back

=back

=cut

sub wait_on_attributes {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %args       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { attributes => { type => ARRAYREF } },

        # Allow through other options such as those defined by
        # _common_wait_for_validate_spec. These will eventually get
        # caught by _wait_on_attributes_helper if they're invalid
        allow_extra => 1
    );
    my @attributes = @{ delete $args{attributes} };
    unless (@attributes) {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            "'attributes' cannot be an empty array-reference");
    }

    my %all_attribute_entries;
    foreach my $attribute_hashref (@attributes) {
        if ( ref($attribute_hashref) eq 'HASH' ) {
            my %opts;
            try {
                %opts = validate_with(
                    params => $attribute_hashref,
                    spec =>
                        { $pkg_or_obj->_common_wait_attr_validate_spec() },
                );
                my $wait_on_attr_obj = $pkg_or_obj
                    ->_validate_and_return_single_wait_on_attribute(%opts);
                $all_attribute_entries{ $attribute_hashref
                        ->{attribute_to_check} } = $wait_on_attr_obj;
            }
            catch NATE::Exceptions::Argument with {
                my $exception = shift;

                # TODO: It would be really nice if I could use the
                # set_ method on exceptions here
                my $new_text
                    = 'One of the entries of the '
                    . "'attributes' argument was passed invalid "
                    . "arguments. Dumper:\n"
                    . Data::Dumper::Dumper($attribute_hashref) . "\n"
                    . $exception->text();
                if ( $exception->can('set_text') ) {
                    $exception->set_text($new_text);
                } else {
                    $exception->{-text} = $new_text;
                }
                $Log->exit();
                $exception->throw();
            };
        } else {
            $Log->exit();
            NATE::Exceptions::Argument->throw( 'All of the entries in the '
                    . "'attributes' argument should be hash-references but "
                    . "one of them was a "
                    . ref($attribute_hashref) );
        }
    }

    my $pkg = $pkg_or_obj->get_package_name();
    $pkg->_wait_on_attributes_helper( %args,
        all_attribute_entries => \%all_attribute_entries );
    $Log->exit();
}

=head2 wait_for_deletion

    $pkg_or_obj->wait_for_deletion(
        'method-timeout'    => $timeout,
        job_component       => $job,
        polling_interval    => $interval,
        [%primary_keys]          # Necessary only for class calls
    );

(Class or instance method)
Certain delete operations appear to take time, with some operating through
a job. This method is provided to wait till the element gets deleted.

If provided a job_component, it will use
L<wait_on_job|lib-NACL-C-Component-pm/wait_on_job> to wait till the job
completes. If not provided a job, it will periodically issue a call to
find() to check if the element still exists or not. It keeps doing this
till the specified timeout is reached.

On timeout a
L<NACL::Exceptions::Timeout|lib-NACL-Exceptions-Timeout-pm>
exception is thrown.

=over

=item Options

=over

=item C<primary keys>

The primary keys need to be specified for class calls.
These should not be specified for instance calls.

=item C<< job_component => $job_component >>

(Optional, is required if C<attribute_to_check> is not provided)
This is a reference to the job component object (is of type C<NACL::C::Job>).
This is aplicable only for CMode operations.

=item C<< 'method-timeout' => $time_to_wait >>

(Required)
Time in seconds to wait for the completion of operation

=item C<< polling_interval => $interval >>

(Optional)
This is the interval (in seconds) after which the value of the attribute will
be polled. If not specified it defaults to 10 seconds.

=item C<< check_job_and_poll => 0 | 1 >>

(Optional, defaults to 1)
Certain builds appear to suffer from a bug due to which the job entry in the
job table is removed before the operation has completed. This means that the
wait_on_job method will also return before the delete operation has completed.
This parameter allows us to start polling the filer by issuing find() calls
to see whether the element has been deleted or not. For builds that do not
suffer from this bug, it will simply need to check the attribute once, so it
acts as a verification that the operation did in fact complete.

=item C<< apiset_must => $ruleset >>

(Optional)
The apiset_must ruleset.

=item C<< apiset_should => $ruleset >>

(Optional)
The apiset_should ruleset.

=back

=back

=cut

sub wait_for_deletion {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            $pkg_or_obj->_common_wait_for_validate_spec(),
            job_component      => { type => OBJECT | UNDEF, optional => 1 },
            check_job_and_poll => { type => SCALAR,         default  => 1 },
        }
    );

    my $pkg                = ref $pkg_or_obj || $pkg_or_obj;
    my $job                = delete $opts{job_component};
    my $check_job_and_poll = delete $opts{check_job_and_poll};
    if ($job) {
        if ( !$job->isa('NACL::C::Job') ) {
            my $ref = ref $job;
            $Log->exit();
            NATE::Exceptions::Argument->throw( "'job_component' parameter "
                    . "is supposed to be of type 'NACL::C::Job', but was "
                    . "sent as '$ref'" );
        } else {
            my $initial_time    = time();
            my %opts_without_pk = %opts;

            # Remove the primary keys before the call to wait_on_job.
            # Just need to have the primary keys moved out of the source,
            # we don't really need it to go anywhere, hence the target
            # being {}.
            $pkg->_hash_move(
                source => \%opts_without_pk,
                target => {},
                move   => [ $pkg_or_obj->get_primary_keys() ]
            );
            $pkg->wait_on_job( %opts_without_pk, job_component => $job );

            if ($check_job_and_poll) {

                # We don't trust the job manager, keep polling till the
                # element is deleted.
                # Determine how long we waited for the job, waiting by
                # polling should be (total_time - time_taken_for_job)
                my $time_elapsed   = time() - $initial_time;
                my $time_remaining = $opts{'method-timeout'} - $time_elapsed;

                $Log->trace("Waited on job for $time_elapsed seconds");
                $Log->trace( 'Polling till deleted for remaining time '
                        . "($time_remaining seconds)" );

                # Override method-timeout to be the time remaining
                $pkg->_poll_till_deleted( %opts,
                    'method-timeout' => $time_remaining );
            }
        }

    } else {
        $pkg->_poll_till_deleted(%opts);
    }

    $Log->exit();
}

=head2 build_associated_object

    # To get a component object
    my $aggr_obj = $vol_obj->build_associated_object(
        type => 'NACL::C::Aggregate');

    # To get a STask object
    my $aggr_obj = $vol_obj->build_associated_object(
        type => 'NACL::STask::Aggregate');

(Instance method)
This uses details provided by this object's relevant CS object to build
component or STask object for elements related to this component.
This would generally be used to obtain objects for elements which are parents
of this component.

In the example shown, we are determining the aggregate on which the volume
(modeled by $vol_obj) and returning a Component/STask object modeling it.
This is done by running state() on this object and extracting the fields
required to build an aggregate component object. This means that the CS object
returned by running state() on this object should return, at the least, all
the primary keys of the object which we want.

This method throws a NATE::Exceptions::Argument exception if an invalid
object type is specified in 'type' input. Any other failures result in a
NATE::BaseException being thrown.

=over

=item Options

=over

=item type

The package name of the component or STask associated object which we
are interested at.

=back

=back

=cut

sub build_associated_object {
    $Log->enter();
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { type => { type => SCALAR }, },
    );
    my $to_component = $opts{type};

    try {
        eval "require $to_component";
    }
    otherwise {
        $Log->exit();
        NATE::Exceptions::Argument->throw( 'Could not import package '
                . "'$to_component' (maybe because the name is invalid?)" );
    };

    my $from_cs = $self->get_CS_package_name();
    try {
        eval "require $from_cs";
    }
    otherwise {
        $Log->exit();
        NATE::Exceptions::Argument->throw( 'Could not import the relevant'
                . "CS package '$from_cs' (maybe because it does not exist?)"
        );
    };

    # Determine if the CS package could fill in the primary keys
    my @to_primary_keys = $to_component->get_primary_keys();
    my @not_present_in_cs;
    foreach my $to_pk (@to_primary_keys) {
        if ( !$from_cs->can($to_pk) ) {
            push @not_present_in_cs, $to_pk;
        }
    }
    if (@not_present_in_cs) {
        my $ref = ref $self;
        $Log->exit();
        NATE::BaseException->throw( 'We cannot build associated object '
                . "of type '$to_component' from this object (of type '$ref') "
                . "because the relevant CS object ('$from_cs') does not fill "
                . 'in one or more the fields required to create the '
                . "$to_component object. Fields not filled in by the CS object:\n"
                . Data::Dumper::Dumper( \@not_present_in_cs )
                . "\n" );
    } else {
        my $cs = $self->state( requested_fields => \@to_primary_keys );
        my $to_component_obj = $cs->build_associated_object(%opts);
        $Log->exit();
        return $to_component_obj;
    }
}

=head2 emerge_contained_object

    # To get a component object
    my $vserver_obj = $vol_obj->emerge_contained_object(
        type => 'NACL::C::Vserver');

    # To get a STask object
    my $vserver_obj = $vol_obj->emerge_contained_object(
        type => 'NACL::STask::Vserver');

(Instance method)
This uses details present in this object itself to build the contained
component or STask object requested for.
This would generally be used to obtain objects whose primary keys
are subset of the primary keys in the object from which this call is invoked.

In the example shown, we are obtaining the vserver Component/STask object
from a volume object as the primary keys of the vserver (vserver) are
subset of primary keys in the volume object (volume, vserver).
This method will try to fetch all the required information to build the
new object from this object itself and so no additional commands will be
invoked in the service.

This method throws a NATE::Exceptions::Argument exception if an invalid
object type is specified in 'type' input. Any other failures result in a
NATE::BaseException being thrown.

=over

=item Options

=over

=item type

The package name of the contained component or STask object which we
are interested at.

=back

=back

=cut

sub emerge_contained_object {
    $Log->enter();
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { type => { type => SCALAR }, },
    );
    my $to_component = $opts{type};

    try {
        eval "require $to_component";
    }
    otherwise {
        $Log->exit();
        NATE::Exceptions::Argument->throw( 'Could not import package '
                . "'$to_component' (maybe because the name is invalid?)" );
    };

    my @to_primary_keys = $to_component->get_primary_keys();
    $Log->debug( "Primary keys of '$to_component' are:\n"
            . Dumper( \@to_primary_keys ) );
    my @not_present_in_me;
    foreach my $to_pk (@to_primary_keys) {
        if ( !$self->can($to_pk) ) {
            push @not_present_in_me, $to_pk;
        }
    }
    if (@not_present_in_me) {
        my $ref = ref $self;
        $Log->exit();
        NATE::BaseException->throw( 'We cannot build contained object '
                . "of type 'to_component' from this object (of type '$ref') "
                . "because "
                . ref($self)
                . " does not fill "
                . 'in one or more the fields required to create the '
                . "$to_component object. Fields not filled in by this object:\n"
                . Data::Dumper::Dumper( \@not_present_in_me )
                . "\n" );
    } else {
        my %new_args;
        foreach my $pk (@to_primary_keys) {
            my $isset = "${pk}_isset";
            if ( $self->$isset() ) {
                $new_args{$pk} = $self->$pk();
            }
        }

        $new_args{command_interface} = $self->command_interface();

        my $to_component_obj;
        try {
            $to_component_obj = $to_component->new(%new_args);
        }
        otherwise {
            my $exception = shift;
            my $ref       = ref $self;
            $Log->exit();
            NATE::BaseException->throw( 'Could not build contained object '
                    . "of type '$to_component' from this object (of type '$ref') "
                    . "due to error while calling $to_component->new(). Error "
                    . "while calling new:\n"
                    . $exception->text() );
        };

        $Log->exit();
        $Log->debug(
            "The contained object of type '$to_component' is found as"
                . Data::Dumper::Dumper(
                { $to_component_obj->get_primary_keys_hash() }
                )
        );
        return $to_component_obj;
    }
}

=head2 get_children_objects

    # To obtain children Component objects
    my @volume_c_objs = $aggr_obj->get_children_objects(
        type => 'NACL::C::Volume');

    # To obtain children STask objects
    my @volume_stask_objs = $aggr_obj->get_children_objects(
        type => 'NACL::STask::Volume');

(Instance method)
Get component/STask objects for elements present under the element represented
by this object. The example above shows how to find all the volumes
present in an aggregate and get objects for all of them.

This method throws a NATE::Exceptions::Argument exception if an invalid
object type is specified in 'type' input. Any other failures result in a
NATE::BaseException being thrown.

=over

=item Options

=over

=item type

The package name of the component or STask for which we want objects.

=back

=back

=cut

sub get_children_objects {
    $Log->enter();
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { type => { type => SCALAR }, },
    );
    my $to_component = $opts{type};

    try {
        eval "require $to_component";
    }
    otherwise {
        $Log->exit();
        NATE::Exceptions::Argument->throw( 'Could not import package '
                . "'$to_component' (maybe because the name is invalid?)\n" );
    };

    my @c_objects;
    try {
        @c_objects = $to_component->find(
            command_interface => $self->command_interface(),
            filter            => { $self->get_primary_keys_hash() },
            allow_empty       => 1
        );
    }
    catch NACL::Exceptions::InvalidFilterField with {
        my $exception = shift;
        $Log->exit();
        NATE::BaseException->throw( "Couldn't find children objects of "
                . "type '$to_component' in the parent '"
                . ref($self) . "'. "
                . "(may be, is this parent invalid?)" );
    };

    if (@c_objects) {
        $Log->exit();
        $Log->debug( "The children objects of type '$to_component' are:\n"
                . NACL::ComponentUtils::Dumper( \@c_objects ) );
        return @c_objects;
    } else {
        $Log->exit();
        NATE::BaseException->throw( "No children of type '$to_component' "
                . 'were found for this object. Dumper of '
                . "this object:\n"
                . Dumper( { $self->get_primary_keys_hash() } )
                . "\n" );
    }
}

=head2 set_fetch_randomization

    NACL::C::Component->set_fetch_randomization();

This method sets the randomization of results for B<all> calls to find/fetch.
This can be overridden for a particular call of find/fetch by sending
"randomize_order => 0".

Note that while this is not specific to a particular component, this method
can be invoked on any component object. For example, this also works:

    $node_obj->set_fetch_randomization();

This does B<not> mean that randomization is set only for Node find/fetch.
Making this method available on component objects is merely for
synctatic convenience.

=cut

sub set_fetch_randomization {
    $Log->enter();
    $NACL::CS::ComponentState::_randomize_fetch = 1;
    $Log->exit();
}

=head2 unset_fetch_randomization

    NACL::C::Component->unset_fetch_randomization();

This method unsets the randomization of results for B<all> calls to
find/fetch.
This can be overridden for a particular call of find/fetch by sending
"randomize_order => 1".

Note that while this is not specific to a particular component, this
method can be invoked on any component object. For example, this also works:

    $node_obj->unset_fetch_randomization();

This does B<not> mean that randomization is unset only for Node find/fetch.
Making this method available on component objects is merely for
synctatic convenience.

=cut


sub unset_fetch_randomization {
    $Log->enter();
    $NACL::CS::ComponentState::_randomize_fetch = 0;
    $Log->exit();
}

=head2 is_c_or_t

    my $c_or_t = NACL::C::Vserver->is_c_or_t(); # Returns "C"

    my $c_or_t = NACL::STask::Volume->is_c_or_t(); # Returns "STask"

    my $c_or_t = $aggr_c_obj->is_c_or_t(); # Returns "C"

    my $c_or_t = $snapmirror_stask_obj->is_c_or_t(); # Returns "STask"

A simple helper method that helps determine whether the package/instance it was
invoked with is a Component (returns "C") or is an STask (returns "STask")
package/instance.

=cut

sub is_c_or_t {
    my $pkg_or_obj = shift;
    my $pkg        = $pkg_or_obj->get_package_name();
    $pkg =~ /NACL::(.+?)::/;
    my $c_or_t = $1;
    return $1;
}

=head2 cast_component_to_task

    my $STask_obj = $component_obj->cast_component_to_task();

(For component and task users)
This method is used to convert a component object into its related STask
(single component task) object.

=cut

sub cast_component_to_task {
    $Log->enter();
    my $self          = $_[0];
    my $C_pkg_name    = ref $self;
    my $Task_pkg_name = $C_pkg_name;
    $Task_pkg_name =~ s/::C::/::STask::/;
    eval "require $Task_pkg_name";
    if ($@) {
        $Log->exit();
        NATE::BaseException->throw( 'We cannot cast a component object '
                . "of type '$C_pkg_name' into a STask object of type "
                . "'$Task_pkg_name' because '$Task_pkg_name' could "
                . 'not be loaded (maybe because it does not exist?)' );
    }
    my $Task_obj = $Task_pkg_name->new( $self->get_pk_hash_with_ci() );
    $Log->exit();
    return $Task_obj;
}

=head2 compare

    if ($Vol_object1->compare($Vol_object2)) {
        $Log->comment("Object references are for the same element");
    }

(Component user facing method)
This method checks whether two object references refer to the same element
and returns 1 if they do. (this is done by comparing the primary key string of
the two objects). If the objects are of different types, then a
NATE::BaseException is thrown.

=cut

sub compare {
    $Log->enter();
    my ( $self, $second_obj ) = @_;
    my $self_type       = ref $self;
    my $second_obj_type = ref $second_obj;

    # Check if the object types match. Since we've introduced aliases
    # (i.e. if a component 'foo' is an alias of component 'bar', 'foo' is a
    # subclass of 'bar' thereby providing access to all of the functionality
    # of 'bar'), object types matching is not merely an equality check of
    # the references, but we also need to check whether one "isa" the other
    if ( $self->isa($second_obj_type) || $second_obj->isa($self_type) ) {
        $Log->exit();
        return ( $self->get_primary_key_string() eq
                $second_obj->get_primary_key_string() ) ? 1 : 0;
    } else {
        $Log->exit();
        NATE::BaseException->throw( 'Two objects of different types cannot '
                . 'be compared. The types of the objects are '
                . "$self_type, $second_obj_type" );
    }
}

=head2 convert_to_bytes

    my $val_in_bytes = $pkg_or_obj->convert_to_bytes($size);

A simple component wrapper around
L<NACL::C::UnitNormalization::convert_to_bytes|lib-NACL-C-UnitNormalization-pm/convert_to_bytes>.
This makes it possible to access this method in all components without
needing to make an explicit imports.

=cut

sub convert_to_bytes {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my $result = NACL::C::UnitNormalization::convert_to_bytes(@args);
    $Log->exit();
    return $result;
}

=head2 convert_to_bytes_ignore_exception

    my $val_in_bytes = $pkg_or_obj->convert_to_bytes_ignore_exception($size);

A simple component wrapper around
L<NACL::C::UnitNormalization::convert_to_bytes|lib-NACL-C-UnitNormalization-pm/convert_to_bytes>.
This makes it possible to access this method in all components without
needing to make an explicit imports.
This method ignores Arguement exception caught due to an invalid value provided as an arguement.

=cut

sub convert_to_bytes_ignore_exception {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my $result;
    try{
       $result = $pkg_or_obj->convert_to_bytes(@args);
    }catch  NATE::Exceptions::Argument with{
       $Log->debug('Size is provided an invalid value');
       $Log->exit();
       $result = $args[0];
    }
    $Log->exit();
    return $result;
}

=head2 various other methods (create, delete, modify, fold, spindle, multiate, ...)

  # As instance (object) method
  my $result  = $widget->whatever(%options);

  # As class (package) method
  my $result  = NACL::C::Widget->whatever(command_interface=>$ci,
                                          (keys) => (values),
                                          %options);

The various methods for changing (creating, modifying, deleting) a
component will generally have a number of options and behaviors in
common.  Most methods should be callable both as a class method and as
an instance method.

=over

=item Options

=over

=item C<< command_interface=>$command_interface, key1=>value1, key2=>value2, ... >>

When a method like this is called as a class method, the fields that
would have been attributes of this object instead become required
options.  This includes the C<command_interface> to communicate with,
and the various attributes to uniquely identify the element when
communicating with that command interface.

=item C<< apiset_must=>$ruleset >>

(Optional) A hashref constraining which API (mode, interface, etc.) on
the given C<command_interface> will be used.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=item C<< apiset_should=>$ruleset >>

(Optional) A hashref suggesting which API (mode, interface, etc.) on
the given C<command_interface> is preferred.  See L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=back

=back

=head2 get_matching_objects

    # @vserver_objs are objects of type NACL::STask::Vserver here
    my @running_vserver_objs = NACL::STask::Vserver->get_matching_objects(
        objects => \@vserver_objs,
        filter  => {'admin-state' => 'running'}
    );

    # Also works in scalar context (returns the first matching object)
    my $running_vserver_objs = NACL::STask::Vserver->get_matching_objects(
        objects => \@vserver_objs,
        filter  => {'admin-state' => 'running'}
    );

Given a list of Component or STask objects and a filter, returns the objects
which match the particular filter.

=over

=item Options

=over

=item C<< objects => \@array_of_objects >>

An array-reference of Component/STask objects. All of these objects need
to be of the same type.

=item C<< filter => \%filter_hash >>

A hash-reference that describes the filter criteria.

=item Common options

The "common options" accepted by all Component methods (such as apiset_should,
apiset_must, method-timeout) are also accepted by this method.

=back

=back

=cut

sub get_matching_objects {
    $Log->enter();
    my ($pkg, @args) = @_;
    state $spec = {
        objects => {type => ARRAYREF},
        filter  => {type => HASHREF},
    };
    my %opts = validate_with(
        params      => \@args,
        spec        => $spec,

        # Allow through common options.
        allow_extra => 1
    );

    my $objects = delete $opts{objects};
    my $filter  = delete $opts{filter};

    my @matched_objects;
    foreach my $object (@$objects) {
        if ($object->verify_state(nacl_return_boolean => 1, %$filter, %opts)) {
            push @matched_objects, $object;
        }
    }

    $Log->exit();
    return wantarray ? @matched_objects : $matched_objects[0];
}

=head1 METHODS (for component implementors)

=head2 _primary_keys_validate_spec

   use Params::Validate qw(SCALAR);
   sub NACL::C::Widget::_primary_keys_validate_spec {
       return (
               "widget-name" => { type=>SCALAR },
              );
   }

(Abstract, all derived classes should provide this).  Returns a hash
initializer (array) for a validation spec (in the hash format used by
the Params::Validate package) that lists the options the component
takes (and corresponding attributes) that describe and keep track of
which element this is a component of.

=head2 init

  sub init { my ($self, @opts) = @_; ... check that @opts are valid options ... }

An instance method, called internally by "new", to check that any
required attributes have been passed to "new".  The base class
provides a generic version of this (based on
_primary_keys_validate_spec).

=cut

sub init {
    $Log->enter();
    my ( $self, %opts ) = @_;

    my %validation_spec = $self->_primary_keys_validate_spec();

    # _primary_keys_validate_spec is written with keys in their hyphenated
    # form. However, when we call new, the attributes are called their
    # underscored form. We need to perform this hyphen to underscore
    # conversion so that validation does not fail.
    %validation_spec = %{ _convert_to_underscores( \%validation_spec ) };
    my %pk_validation_spec = %validation_spec;
    $validation_spec{command_interface}
        = { isa => 'NACL::C::CommandInterface', type => OBJECT };


    # Allow for calls to new() to take either the underscored or hyphenated
    # form of the primary keys
    my $new_opts = _convert_to_underscores( \%opts );

    # Allow passing objects as the value. Example: Volume->new(volume => $vol_obj)
    # Extract the appropriate attribute from the object passed in.
    foreach my $key (keys %pk_validation_spec) {
        my $val = $new_opts->{$key};
        if (blessed ($val)) {
            if ($val->can($key)) {
                my $new_val = $val->$key();
                $new_opts->{$key} = $new_val;
                $self->$key($new_val);
            } else {
                $Log->exit();
                NATE::Exceptions::Argument->throw('In the call to '
                    . ref ($self)
                    . "->new(), the value of the '$key' field was an "
                    . 'object of type '
                    . ref ($val)
                    . ' which is invalid'
                );
            }
        }
    }

    #enable_value_check option checks whether the value of a primary key
    #contains '-', for certain components '-' is still valid
    # Example: DebugNetworkRecovery component
    my %temp_validation_spec = %validation_spec;
    $temp_validation_spec{enable_value_check} = { type => SCALAR, default => 1 };

    %opts = validate_with(
        params      => $new_opts,
        spec        => \%temp_validation_spec,
        allow_extra => 1
    );

    my @fields;
    if ( $opts{enable_value_check} ) {
        foreach my $p_key (keys %validation_spec) {
            if (defined $new_opts->{$p_key} && ($new_opts->{$p_key} eq '-')) {
                push(@fields, $p_key);
            }
        }
        if (@fields) {
            my $fields_str = join( ', ', @fields);
            $Log->exit;
            NATE::Exceptions::Argument->throw(
                "Passing '-' as the value to the primary key(s) $fields_str ".
                "is/are invalid."
            );
        }
    }

    # We need to check if the command_interface provided can indeed be used
    # as a command_interface. Note that the call to hostrec() will fail
    # (i.e. throw a NATE::BaseException) if a hostrec could not be
    # constructed for the host
    try {
        $self->command_interface()->hostrec();
    }
    catch NATE::BaseException with {
        my $exception      = shift;
        my $ci             = $self->command_interface();
        my $ci_type        = ref $ci;
        my $name_attribute = $ci->name_attribute();
        my $name           = $ci->$name_attribute();
        $exception->set_text( "The command interface (name='$name', "
                . "type='$ci_type') passed in the call to new() "
                . 'cannot be used as a command_interface '
                . "for other components because:\n"
                . $exception->text() );
        $Log->exit();
        $exception->throw();
    };

    $Log->exit();
} ## end sub init

=head2 call_on_apiset

    sub my_method {
        my $pkg_or_obj = shift;
        $pkg_or_obj->call_on_apiset(@_, choices => [...]);
    }

A helper for constructing public "front end" methods of components
which are mode- or interface-agnostic.  See
L<lib::NACL::ChooseAPISet|lib-NACL-ChooseAPISet-pm>

=cut

# Copy method definition from NACL::ChooseAPISet
use autouse 'NACL::ChooseAPISet' => qw(call_on_apiset call_on_apiset_with_exceptions);

=head2 parents

(For cleanup implementation only)

This method returns a empty list as of now and is an implementation of parents method
in the base class.

=cut

# this method has been copied to STask/STask but if we want MTasks to also
# resource object properties then this method should be here but it wont do much
# if invoked at the C layer

sub parents {
    $Log->enter();

    $Log->exit();
    return qw();
}

=head2 _hash_move

A helper to select and rename subsets of options (to help in calling
API layer methods).  See L<lib::NACL::ComponentUtils|lib-NACL-ComponentUtils-pm>

=head2 _optional_scalars

  # same as (a=>{type=>SCALAR,optional=>1}, b=>{type=>SCALAR, optional=>1})
  _optional_scalars(qw(a b))

A compact way to represent the Params::Validate spec needed for most
of the options to most component methods, which tend to be optional
strings ('SCALAR' in Params::Validate).  See L<lib::NACL::ComponentUtils|lib-NACL-ComponentUtils-pm>

=head2 AUTOLOAD

    sub a_b_c {
        ...
    }
    my $name = "a-b-c";
    my $result = $self->$name();  # works

The component package has an AUTOLOAD method so that any attribute
declared as "a_b_c" can be accessed using either method name "a-b-c"
or "a_b_c".

=cut

use NACL::ComponentUtils qw(_hash_move
    _optional_scalars
    _hash_copy
    _process_options
    can
    _translate_to_obj_field_name
    get_package_name
    _can_validate_pos_spec
    _can_validation
    _verify_invocation
    _make_attribute_readonly
    _zapi_hash_copy
    _hash_copy_defined
    _hash_move_defined
);

# NATE::Autouse cannot handle AUTOLOAD methods that aren't inherited
# Call ComponentUtils::Autoload without importing it
sub AUTOLOAD {
    our $AUTOLOAD;
    $NACL::ComponentUtils::AUTOLOAD = $AUTOLOAD;
    goto &NACL::ComponentUtils::AUTOLOAD;
}

=head2 _find_validate_spec

   sub find {
        my $pkg  = shift;
        my %opts = validate @_, $pkg->_find_validate_spec();
        ...
    }

The Params::Validate spec describing the arguments that all
implementations of "find" should take.

=cut

sub _find_validate_spec {
    $Log->enter();
    my %find_validate_spec;

    # Other than "filter", "state" accepts all the parameters
    # accepted by "fetch"
    %find_validate_spec
        = %{ NACL::CS::ComponentState->_fetch_validate_spec() };
    delete $find_validate_spec{requested_fields};

    $Log->exit();
    return {%find_validate_spec};
} ## end sub _find_validate_spec

=head2 _state_validate_spec

    sub state {
        my $self = shift;
        my %opts = validate @_, $self->_state_validate_spec();
        ...
    }

The Params::Validate spec describing the arguments that all
implementations of "state" should take.

=cut

sub _state_validate_spec {
    $Log->enter();
    my %state_validate_spec;

    %state_validate_spec
        = %{ NACL::CS::ComponentState->_fetch_validate_spec() };
    foreach (
        qw(filter command_interface allow_empty canned_filters
        randomize_order sort_ascending sort_descending)
        )
    {
        delete $state_validate_spec{$_};
    }

    $Log->exit();
    return {%state_validate_spec};
} ## end sub _state_validate_spec

=head2 _backend_validate_spec

    sub _foo_cmode_cli {
        my $pkg_or_obj = shift;
        my %opts = validate_with(
            params      => \@_,
            spec        => $pkg_or_obj->_backend_validate_spec(),
            allow_extra => 1
    );

The Params::Validate spec describing the arguments taken by the backend
methods. This would have to include command_interface and apiset

=cut

sub _backend_validate_spec {
    return {
        command_interface =>
            { isa => 'NACL::C::CommandInterface', type => OBJECT },
        apiset           => { isa  => 'NACL::APISet' },
        'method-timeout' => { type => SCALAR, optional => 1 }
    };
} ## end sub _backend_validate_spec

# Not documented, as we expect callers to go through
# "_common_validate_with" instead.

# The Params::Validate spec describing the minimum arguments that most
# random class-or-instance methods (modify, delete, etc.) on the
# component should take.

sub _common_validate_spec {
    return {
        command_interface =>
            { isa => 'NACL::C::CommandInterface', type => OBJECT },
        apiset_must      => { type => HASHREF,   default  => {} },
        apiset_should    => { type => HASHREF,   default  => {} },
        'method-timeout' => { type => SCALAR,    optional => 1 },
        apiset_chosen    => { type => SCALARREF, optional => 1 },
        'connectrec-max_idle'  => { type => SCALAR,    optional => 1 },
    };
} ## end sub _common_validate_spec

# These fields cannot be added to _common_validate_spec since they end up
# causing validation failures on the CS side, hence this extra spec for
# options that are common to all methods on this Component on the C side.
sub _additional_common_validate_spec {
    return {cmd_execution_time => {type => SCALARREF, optional => 1}};
}

=head2 _common_validate_with

    sub delete {
        my $pkg_or_obj = shift;
        my %opts = $pkg_or_obj->_common_validate_with
            (params=>\@_,
             additional_spec=>{_optional_scalars qw(a b c d e ...),},
            );
        ...;
    }

_common_validate_with is a generic method for validation of the
inputs to instance/package methods which takes input parameters
equivalent to Params::Validate's C<validate_with>, except:

=over

=item (1)

It's a method (takes a package or object as first argument, which is
then followed by the usual "validate_with" options)

=item (2)

The spec includes options common to almost all component methods
(C<command_interface>, C<apiset_must>, C<apiset_should>).  The caller
may specify additional options with the "additional_spec" option.

=item (3)

If called as a class method (first argument is a package), the allowed
options also include all of the "primary key" options/attributes for
this component (as provided in C<_primary_keys_validate_spec>).  If
called as an instance method, each of the "primary key"
options/attributes are copied out of the object as though they had
been provided as options.  Thus, a method calling
C<_common_validate_with> can go on to find the primary key values in
the returned options hash, regardless of whether that method is called
on a class or on an instance.

=back

=cut

# Making "_get_keys" an alias of the "get_primary_keys" subroutine
no strict 'refs';
*{_get_keys_names} = \&get_primary_keys;
use strict 'refs';

# This includes the primary keys validation spec unless "ignore_primary_keys"
# is set
sub _include_object_attributes_validate_spec {
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            spec                => { type => HASHREF },
            ignore_primary_keys => { type => SCALAR },
            extended_query      => { type => HASHREF | UNDEF, optional => 1 },
            params              => { type => HASHREF },
        },
        allow_extra => 1    # We don't care about the other options
    );
    my %spec   = %{ $opts{spec} };
    my $params = $opts{params};
    if ( !$opts{ignore_primary_keys} ) {
        my %pk_validate_spec = $pkg_or_obj->_primary_keys_validate_spec();
        if ( defined $opts{extended_query}
            && keys %{ $opts{extended_query} } )
        {

            # With extended query the primary keys might or might not be
            # provided, so make them all optional
            $pkg_or_obj->_make_pk_optional( spec => \%spec );
        } else {
            foreach my $key ( keys %pk_validate_spec ) {

                # If any of the primary keys is provided as "*", then all
                # the primary keys become optional
                if ( defined $params->{$key} && $params->{$key} eq '*' ) {
                    $pkg_or_obj->_make_pk_optional( spec => \%spec );
                    last;
                }
            }
        }
        %spec = ( %pk_validate_spec, %spec );
    }

    $Log->exit();
    return %spec;
}

sub _common_validate_with {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            params               => { type     => ARRAYREF },
            additional_spec      => { type     => HASHREF, default => {} },
            ignore_primary_keys  => { type     => SCALAR, default => 0 },
            only_static_method   => { type     => SCALAR, default => 0 },
            allow_extra          => { optional => 1 },
            no_command_interface => { type     => BOOLEAN, default => 0 },
            get_computed_params  => { type => HASHREF, optional => 1 },
            get_computed_spec    => { type => HASHREF, optional => 1 },
            consolidated_spec => { type => HASHREF, optional => 1 },
        },
    );

    if ( $opts{only_static_method} && ref $pkg_or_obj ) {
        my $me = ( caller(1) )[3];
        NATE::BaseException->throw( "'$me' is a static method"
                . ' and should not be invoked with an object' );
    }

    my %params = @{ $opts{params} };

    my %spec = %{ $opts{additional_spec} };

    my $extended_query = $params{extended_query};

    my $consolidated_spec = $opts{consolidated_spec};

    if ( $consolidated_spec ) {
        %spec = %{ $consolidated_spec };
    } else {
        # Include object attributes in the validation spec
        %spec = $pkg_or_obj->_include_object_attributes_validate_spec(
            ignore_primary_keys => $opts{ignore_primary_keys},
            spec                => \%spec,
            extended_query      => $extended_query,
            params              => \%params,
        );

        # The validation spec should be a combination of the hash constructed
        # above, _common_validate_spec and _additional_common_validate_spec.
        # Keep %spec after _common_validate_spec so a user-provided spec can
        # override the validation as provided by _common_validate_spec
        %spec = ( %{ $pkg_or_obj->_common_validate_spec() },
            %{ $pkg_or_obj->_additional_common_validate_spec() },%spec, );

       # Turns out there are components/tasks which don't have a
       # command_interface attribute. This can be specified with the
       # "no_command_interface" option. Let's remove the command_interface
       # added to the validation spec if this option has been specified.
       delete $spec{command_interface} if $opts{no_command_interface};

       # Any argument to a method call can either be of the type specified in
       # the call to _common_validate_with (i.e. scalar or arrayref) or can be
       # an object. This makes OBJECT be a valid type for all arguments
       while ( my ( $key, $value ) = each %spec ) {
            $value->{type} = $value->{type} | OBJECT
                if ( defined $value->{type} );
       }
    }

    if ( defined $params{_expand_objects} ) {
        my $expand_objects = delete $params{_expand_objects};
        if ( ref $expand_objects ne 'ARRAY' ) {
            my $ref = ref $expand_objects;
            $Log->exit();
            NATE::Exceptions::Argument->throw( "The '_expand_objects' "
                    . 'argument should have been an array-reference of '
                    . "values but was instead provided as a '$ref'\n" );
        }
        my %expanded_options;
        foreach my $obj (@$expand_objects) {
            if ( blessed $obj && $obj->isa('NACL::C::Component') ) {
                my %pk_with_ci = $obj->get_pk_hash_with_ci();
                %expanded_options = ( %expanded_options, %pk_with_ci );
            } else {
                my $ref = ref $obj;
                $Log->exit();
                NATE::Exceptions::Argument->throw( 'All of the values '
                        . "for the '_expand_objects' argument should be "
                        . 'either Component or STask objects but one of the '
                        . "arguments passed was of type '$ref'. Dumper of "
                        . "this object:\n"
                        . NACL::ComponentUtils::Dumper($obj)
                        . "\n" );
            }
        }
        %params = ( %expanded_options, %params );
    }

    # If "extended_query" is supplied, then any primary keys that are
    # supplied should be moved into the extended query rather than being
    # sent directly to the command
    if (   ( defined $params{extended_query} )
        && ( keys %{ $params{extended_query} } ) )
    {
        $pkg_or_obj->_move_primary_keys(
            source => \%params,
            target => $params{extended_query}
        );
    }

    if ( ref $pkg_or_obj ) {
        my @keys = $pkg_or_obj->_get_keys_names();
        push @keys, 'command_interface' unless $opts{no_command_interface};
        foreach my $key (@keys) {
            my $key_isset = "${key}_isset";
            if ( $pkg_or_obj->$key_isset() ) {
                my $value_in_obj = $pkg_or_obj->$key();
                if ( $params{$key} ) {

                    # Allow attributes of objects to be provided as
                    # arguments to the method as long as their values
                    # match.
                    # For example, this allows calls like:
                    # $vol_obj->modify(volume => $vol_name, %other_opts);
                    # where $vol_name eq $vol_obj->volume()
                    # This also supports comparisons between strings and
                    # objects, like:
                    # $vol_obj->modify(volume => $vol_obj, %other_opts);
                    # where it will comparse $vol_obj with $vol_obj->volume()
                    my $param_val = $params{$key};
                    if ($key ne 'command_interface' && blessed $param_val) {
                        $param_val = $param_val->$key() if ($param_val->can($key));
                    }
                    if ( $value_in_obj ne $param_val ) {
                        $Log->exit();
                        NATE::Exceptions::Argument->throw(
                                  "'$key' was provided as an argument to the "
                                . 'method but its value does not match '
                                . 'the value of the attribute stored in the '
                                . "object.\n"
                                . "Value passed as argument to method: $param_val"
                                . "\nValue of attribute in object: $value_in_obj"
                        );
                    }
                } else {
                    $params{$key} = $value_in_obj;
                }
            }
        }

    }

    my %validate_with_opts = $pkg_or_obj->_get_validate_with_spec(
	                         params => \%params,
	                         opts   => \%opts);
    %{ $opts{get_computed_spec} } = %spec
		if ( $opts{get_computed_spec} );
    %{ $opts{get_computed_params} } = %params
		if ( $opts{get_computed_params} );

    $Log->exit();
    return validate_with(
        params => \%params,
        spec   => \%spec,
        %validate_with_opts,
    );
} ## end sub _common_validate_with

sub _get_validate_with_spec {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %input = validate_with(
        params => \@args,
        spec => { params   => { type => HASHREF },
                  opts   => { type => HASHREF },
	            }
    );
    my %validate_with_opts;
    my $params = $input{params};
    my $opts = $input{opts};
    if ( exists $params->{allow_extra} ) {
        $validate_with_opts{allow_extra} = $params->{allow_extra};
    } elsif ( exists $opts->{allow_extra} ) {
        $validate_with_opts{allow_extra} = $opts->{allow_extra};
    }
    $Log->exit();
    return %validate_with_opts;
}

=head2 _make_pk_optional

    $pkg_or_obj->_make_pk_optional(spec => \%spec);

Updates the validation spec passed in to have the primary keys be optional.

=cut

sub _make_pk_optional {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { spec => { type => HASHREF } },
    );
    my $spec             = $opts{spec};
    my %pk_validate_spec = $pkg_or_obj->_primary_keys_validate_spec();
    while ( my ( $key, $value ) = each %pk_validate_spec ) {
        $spec->{$key} = $value if ( !exists $spec->{$key} );
        $spec->{$key}{optional} = 1;
    }
    $Log->exit();
}

# Have this default to an empty array so that this method does not need to be
# implemented for those components that don't have primary keys
sub _primary_keys_validate_spec {
    return ();
}

# Convert hyphenated hash keys to their underscored form
# Used by the generic implementation of init
sub _convert_to_underscores {
    my $hash_ref = shift;
    my %hash     = %$hash_ref;
    foreach my $key ( keys %hash ) {
        my $orig_key = $key;
        if ( $key =~ s/-/_/g ) {
            $hash{$key} = delete $hash{$orig_key};
        }
    } ## end foreach my $key ( keys %hash)
    return \%hash;
} ## end sub _convert_to_underscores

=head2 _execute_api_version

    $pkg_or_obj->_execute_api_version(
        apiset_obj      => $apiset,
        api_calls       => [ "api_call_BR", "api_call_RR" ],
        command_options => $cmode_args
    );

(Private method)_execute_api_version takes a arraylist of apicalls wich has different
description on different versions and executes the relavent apicall.
Returns jobids or null .

=over

=item Options

=over

=item C<< apiset_obj=>$apiset >>

(Mandatory) HASHREF The apiset object referring to a node.

=item C<< api_calls=>[ "api_call_BR", "api_call_RR" ] >>

(Mandatory) An array reference containing a list of api calls.

=item C<< command_options=>$command_options >>

(Mandatory)HASHREF of input parameters to execute the command.

=back

=back

=cut

sub _execute_api_version {
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            apiset_obj      => { type => HASHREF },
            api_calls       => { type => ARRAYREF },
            command_options => { type => HASHREF }
        },
    );
    my $index = 0;
    my $found = 0;
    my $response;
    my @api_calls = @{ $opts{api_calls} };
    foreach my $api_call (@api_calls) {
        try {
            $response
                = $opts{apiset_obj}->$api_call( %{ $opts{command_options} } );
            $found = 1;
        }
        catch NACL::APISet::Exceptions::MethodNotFoundException with      {}
            catch NACL::APISet::Exceptions::CommandNotFoundException with {};
        last if $found;
        $index++;
    }

    if ( $index == ( $#api_calls + 1 ) ) {
        NATE::BaseException->throw(
            "Api method's [@api_calls] not supported on this Build.");
    }

    return $response->get_job_ids();

}

=head2 _array_to_scalar_deprication_warning

    $foo->_array_to_scalar_deprication_warning(
      params => ['nm-switch','ns-switch'],
      options => \%opts
    );

This method converts input options having ARRAYREF to comma seprated SCALAR
value. Throws a warning message if user is still using comma seperated
SCALAR values instead of ARRAYREF's.

=cut

sub _array_to_scalar_deprication_warning {
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            params  => { type => ARRAYREF },
            options => { type => HASHREF },
        },
    );

    foreach my $param ( @{ $opts{params} } ) {
        if ( defined $opts{options}->{$param} ) {
            if ( ref $opts{options}->{$param} eq 'ARRAY' ) {
                $opts{options}->{$param} = join ',',
                    @{ $opts{options}->{$param} };
            } else {
                $pkg->_print_deprecated_warning_scalar( $param, "scalar" );
            }
        } ## end if ( defined $opts{options...})
    } ## end foreach my $param ( @{ $opts...})
} ## end sub _array_to_scalar_deprication_warning

=head2 _scalar_to_array_deprication_warning

    $foo->_scalar_to_array_deprication_warning(
      params => ['nm-switch','ns-switch'],
      options => \%opts
    );

This method converts input options having scalar with comma seprated
value to ARRAYREF . Throws a warning message if user is still using
comma seperated SCALAR values instead of ARRAYREF's.

=cut

sub _scalar_to_array_deprication_warning {
    my $pkg  = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            params  => { type => ARRAYREF },
            options => { type => HASHREF },
        },
    );

    foreach my $param ( @{ $opts{params} } ) {
        if ( defined $opts{options}->{$param} ) {
            if ( !ref( $opts{options}->{$param} ) ) {
                my @arr = split /,/, $opts{options}->{$param};
                $opts{options}->{$param} = \@arr;
                $pkg->_print_deprecated_warning_scalar($param);
            }
        } ## end if ( defined $opts{options...})
    } ## end foreach my $param ( @{ $opts...})
} ## end sub _scalar_to_array_deprication_warning

sub _print_deprecated_warning_scalar {
    my $pkg   = shift;
    my $param = shift;
    $Log->warn( cluck ( "Providing a scalar value for the param '$param' is "
            . 'deprecated since this value accepts a list. Provide it '
            . 'as an arrayref of values since support for providing it '
            . 'as a scalar will be removed in an upcoming sprint' ));
} ## end sub _print_deprecated_warning_scalar

sub _print_deprecated_warning_array {
    my $pkg   = shift;
    my $param = shift;
    $Log->warn( cluck("Providing a arrayref value for the param '$param' is "
            . 'deprecated since this value accepts a scalar. Provide it '
            . 'as a scalar value since support for providing it '
            . 'as a arrayref will be removed in an upcoming sprint' ));
} ## end sub _print_deprecated_warning_array

sub _print_deprecation_warning {
    my $pkg  = shift;
    my $warn_message = shift;
    $Log->warn( cluck ( $warn_message ) );
} ## end _print_deprecation_warning

=head2 _set_test_priv_if_test_fields_present

B<DEPRECATED>

The APISet automatically sets to the highest privilege level possible,
so this method is no longer needed. It has been left untouched for
backwards compatibility, but it should no longer be needed.

    # NACL/C/Foo.pm
    sub _operation_mode_cli {
        my $pkg_or_obj = shift;
        my %opts = validate_with(
            params      => \@_,
            spec        => $pkg->_backend_validate_spec(),
            allow_extra => 1
        );
        my $apiset = delete $opts{apiset};
        ...
        $pkg_or_obj->_set_test_priv_if_test_fields_present(
            test_fields => [ qw(field1 field2) ],
            api_opts    => \%opts,
            apiset      => $apiset
        );
        ...
        $apiset->api_call(%opts);
    }

(Component developer only method)
Certain command options are accessible only in 'test' privilege. This method
checks whether any of these options have been provided in the Component method
call and if so sets the API to run in 'test' privilege. Prior to setting to
'test' privilege, it checks whether the build is a promoted build. Promoted
builds have 'test' privilege level disabled, so if it is a promoted build the
call fails (a NACL::Exceptions::InvalidPrivilege exception is thrown)

=over

=item Options

=over

=item C<< test_fields => [ $field1, $field2, ... ] >>

(Required, Arrayref)
This is an arrayref of all of the fields that are accessible only in test
privilege.

=item C<< api_opts => \%opts >>

(Required, Hashref)
This is the reference to the options hash that gets passed to the API call.

=item C<< apiset => $apiset >>

(Required, isa NACL::APISet)
The APISet object used to make the API call.

=back

=item Exceptions thrown

=over

=item NACL::Exceptions::InvalidPrivilege

This exception is thrown if we provide fields which are accessible only
in test privilege but we're running on a promoted build.

=back

=back

=cut

sub _set_test_priv_if_test_fields_present {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   test_fields => { type => ARRAYREF },
            api_opts    => { type => HASHREF },
            apiset      => { isa  => 'NACL::APISet' },
        }
    );
    my $test_fields = $opts{test_fields};
    my $api_opts    = $opts{api_opts};
    my $apiset      = $opts{apiset};

    my ( $need_test_priv, @present_test_fields );

    foreach my $test_field (@$test_fields) {
        if ( exists $api_opts->{$test_field} ) {
            $need_test_priv = 1;
            push @present_test_fields, ("'$test_field'");
        }
    }

    if ($need_test_priv) {
        my ( $is_are, $option_options, $was_were );
        if ( @present_test_fields > 1 ) {
            $is_are         = 'are';
            $option_options = 'options';
            $was_were       = 'were';
        } else {
            $is_are         = 'is';
            $option_options = 'option';
            $was_were       = 'was';
        }

        my $present_test_fields_str;
        $present_test_fields_str = join ', ', @present_test_fields;

        my $version_manager = $apiset->get_version_manager();

        try {
            if ( $version_manager->is_promoted() ) {
                my $release = $version_manager->get_version_attribute(
                    attribute => 'release' );
                $Log->exit();
                NACL::Exceptions::InvalidPrivilege->throw(
                          "The provided $option_options "
                        . "$present_test_fields_str $is_are only accessible in "
                        . "'test' privilege but since this is a promoted build "
                        . "(release: $release), 'test' privilege is disabled"
                );
            } else {
                $Log->debug( "Setting to 'test' privilege for the API call "
                        . "'since the $option_options $present_test_fields_str "
                        . "$was_were provided and $is_are accessible only in "
                        . "'test' privilege" );
                $api_opts->{'privilege-level'} = 'test';
            }
        }
        catch NACL::APISet::Exceptions::ConnectionFailedException with {
            my $exception = shift;
            $Log->debug(
                      'Could not determine if the build is promoted or not '
                    . 'because the required connection (7-system) could not be '
                    . 'established. Setting to test privilege (the API call '
                    . 'will fail if test privilege is inaccessible)'
                    . "\n    Error:\n "
                    . $exception->text() );
            $api_opts->{'privilege-level'} = 'test';
        };
    }
}

=head2 _translate_size_suffixes

    # script.thpl
    ...
    NACL::C::Foo->operation(size => '1KB');
    ...

    # NACL/C/Foo.pm
    sub _operation_mode_interface {
        my $pkg_or_obj = shift;
        my %opts = validate_with(
            params      => \@_,
            spec        => $pkg->_backend_validate_spec(),
            allow_extra => 1
        );
        my $apiset = delete $opts{apiset};
        ...
        $pkg_or_obj->_translate_size_suffixes(
            options_to_translate => [ qw(size maxsize minsize) ],
            api_opts             => \%opts,
            suffix_map => { 'KB' =>'k', 'MB' => 'm', 'GB' => 'g', 'TB' => 't' }
        );
        ...
        # 'size' values gets translated to '1k'
        $apiset->api_call(%opts);
    }

(Component developer only method)
CMode CLI is consistent in accepting the optional size suffixes 'KB', 'MB',
'GB, 'TB' and 'PB'. However other modes and interfaces are not quite so
consistent in the size suffixes they accept. This method translates the
size suffixes to whatever the underlying API understands.

=over

=item Options

=over

=item C<< options_to_translate => [ $field1, $field2, ... ] >>

(Required, Arrayref)
This is an arrayref of all of the fields whose suffixes need to be translated.

=item C<< api_opts => \%opts >>

(Required, Hashref)
This is the reference to the options hash that gets passed to the API call.

=item C<< suffix_map => \%suffix_map >>

(Optional, HASHREF)
This contains the mapping of the size suffixes from the CMode CLI standard
to whatever the API understands. This requires mapping to be provided for
all of 'KB', 'MB', 'GB' and 'TB' (all in uppercase).

OIf not provided, this defaults to the mapping shown in the example.

=back

=back

=cut

sub _translate_size_suffixes {
    $Log->enter();
    my $pkg_or_obj = shift;
    $Log->debug( "Options to '_translate_size_suffixes:\n" . Dumper( {@_} ) );
    my $default_suffix_map
        = { 'KB' => 'k', 'MB' => 'm', 'GB' => 'g', 'TB' => 't' };
    my %opts = validate(
        @_,
        {   options_to_translate => { type => ARRAYREF },
            api_opts             => { type => HASHREF },
            suffix_map => { type => HASHREF, default => $default_suffix_map },
        }
    );
    my $options_to_translate = $opts{options_to_translate};
    my $api_opts             = $opts{api_opts};
    my $suffix_map           = $opts{suffix_map};

    my $num_present_keys;
    foreach my $key (qw(KB MB GB TB)) {
        $num_present_keys++ if ( exists $suffix_map->{$key} );
    }
    if ( $num_present_keys != 4 ) {
        NATE::Exceptions::Argument->throw( 'Internal error: The parameter '
                . "'suffix_map' should contain all of the keys 'KB', 'MB', 'GB'"
                . " and 'TB', however it only contains mappings for the keys "
                . ( keys %$suffix_map ) );
    }

    foreach my $option (@$options_to_translate) {
        if ( exists $api_opts->{$option} ) {
            my $value = $api_opts->{$option};
            if ( $value =~ s/^([+-]?\d+)\s*(\D+)$/$1/ ) {
                my $suffix = $2;

                my $partial_match = sub {
                    my $suffix          = shift;
                    my $suffix_from_map = uc $suffix;
                    $suffix_from_map .= 'B' if ( $suffix_from_map !~ /B$/ );
                    return $suffix_from_map;
                };

             # Check if suffix is any of KB, MB, GB, TB, PB (case-insensitive)
                if ( $suffix =~ /^[kmbgtp]b?$/i ) {
                    my $suffix_from_map = $partial_match->($suffix);
                    if ( $suffix_from_map eq 'PB' ) {
                        my $pb_value_in_tb = $value * 1024;
                        my $new_suffix     = $suffix_map->{'TB'};
                        $api_opts->{$option} = $pb_value_in_tb . $new_suffix;
                    } else {
                        my $new_suffix = $suffix_map->{$suffix_from_map};
                        $api_opts->{$option} = $value . $new_suffix;
                    }
                } else {
                    $Log->exit();
                    NATE::Exceptions::Argument->throw( "The option '$option' "
                            . "can have size suffix 'KB', 'MB', 'GB', 'TB' or  "
                            . "'PB' , but the size suffix provided was '$suffix'"
                    );
                }
            }
        }
    }

    $Log->debug(
        "API opts with translated size suffixes:\n" . Dumper $api_opts);
    $Log->exit();
}

=head2 _print_deprecated_option_used_warning

    # NACL/C/Foo.pm
    if (exists $opts{$deprecated_option}) {
        $pkg_or_obj->_print_deprecated_option_used_warning(
            deprecated_option => $deprecated_option,
            new_option        => $option_name_to_use,
        );
    }

(For component developers)
This method logs a warning message informing the user that a deprecated option
is being used and informing them of the actual option name to use.

=over

=item Options

=over

=item deprecated_option

The name of the deprecated option used

=item new_option

The name of the option they should be using

=back

=back

=cut

sub _print_deprecated_option_used_warning {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            deprecated_option => { type => SCALAR },
            new_option        => { type => SCALAR },
        }
    );
    my $deprecated_option = $opts{deprecated_option};
    my $new_option        = $opts{new_option};

    $Log->warn( cluck("The deprecated option '$deprecated_option' was "
            . "provided. The option '$new_option' should be used instead. "
            . "Support for the deprecated option '$deprecated_option' "
            . 'will be removed in a future sprint' ));
    $Log->exit();
}

=head2 _handle_boolean_options

    # NACL/C/Foo.pm
    sub _bar_mode_interface {
        my $pkg_or_obj = shift;
        my %opts = validate_with(...);
        ...
        $pkg_or_obj->_handle_boolean_options(
            api_options     => \%opts,
            boolean_options => [ qw($opt1 $opt2) ],
            is_7m_cli       => $boolean,
        );
        ...
    }

(For component developers)
This method is used to provide support for boolean options to be provided
through either 1/0 or true/false. In 7Mode CLI, the switch is present in a perl
true value is provided and is not present if a perl false value is provided.
This means that the string 'false' in 7Mode CLI causes the switch to be
present. Hence for 7Mode CLI we need to translate 'false' to 0.

All other modes and interfaces accept the value as true/false. Hence, for other
modes and interfaces we need to translate 1/0 to true/false.

=over

=item Options

=over

=item api_options

(HASHREF, Mandatory)
This is a reference to the opts hash available in the backend. The translation
is performed directly in the backend options.

=item boolean_options

(ARRAYREF, Mandatory)
This is a reference to the array of names of boolean options.

=item is_7m_cli

(BOOLEAN, Mandatory)
This specifies whether the APISet used in 7Mode CLI or not. The translation
performed depends on this value (as described above).

=back

=back

=cut

sub _handle_boolean_options {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            api_options     => { type => HASHREF },
            boolean_options => { type => ARRAYREF },
            is_7m_cli       => { type => BOOLEAN }
        }
    );
    my $api_options     = $opts{api_options};
    my $boolean_options = $opts{boolean_options};
    my $is_7m_cli       = $opts{is_7m_cli};

    foreach my $boolean_opt (@$boolean_options) {
        if ( exists $api_options->{$boolean_opt} ) {
            if ($is_7m_cli) {
                if ( $api_options->{$boolean_opt} eq 'false' ) {
                    $api_options->{$boolean_opt} = 0;
                }
            } else {
                if ( $api_options->{$boolean_opt} =~ /^0$|^1$/ ) {
                    if ( $api_options->{$boolean_opt} == 1 ) {
                        $api_options->{$boolean_opt} = 'true';
                    } elsif ( $api_options->{$boolean_opt} == 0 ) {
                        $api_options->{$boolean_opt} = 'false';
                    }
                }
            }
        }
    }
    $Log->exit();
}

sub get_CS_package_name {
    $Log->enter();
    my $C_pkg_or_obj = shift;
    my $method = shift;

    my $C_pkg_name   = $C_pkg_or_obj->get_package_name();
    $C_pkg_name =~ /NACL::[^:]+::(.*)/;
    my $CS_pkg_name = "NACL::CS::$1";

    if($method) {
        $method =~ s/_(\S)/uc($1)/ge;
        $method =~ s/^(\S)/uc($1)/e;
        $CS_pkg_name .= $method;
    }
    $Log->exit();
    return $CS_pkg_name;
}

=head2 _common_validate_spec_without_ci

    my $cvs_without_ci = $pkg_or_obj->_common_validate_spec_without_ci();

(For component users)
This method returns the validation spec for options common to all component
methods invoked with an object instance.

(As the name indicates, this is the _common_validate_spec without the
command_interface)

=cut

sub _common_validate_spec_without_ci {
    $Log->enter();
    my $pkg_or_obj = shift;
    my $cvs        = $pkg_or_obj->_common_validate_spec();
    delete $cvs->{command_interface};
    $Log->exit();
    return $cvs;
}

# This returns the element name. Ex: for NACL::C::Aggregate, returns "Aggregate"
# Currently the only caller for this is wait_for_completion. Split this off
# into a separate method since it's needed in a couple of places.
sub _determine_element_name {
    $Log->enter();
    my $pkg_or_obj = shift;
    my $element = ref $pkg_or_obj || $pkg_or_obj;
    $element =~ s/.*:://;
    $Log->exit();
    return $element;
}

=head2 _validate_and_return_single_wait_on_attribute

This method is used only by wait_on_attribute and wait_on_attributes.
Providing a single hash-reference containing the options sent as
arguments to wait_on_attribute (attribute_to_check, till_value,
till_not_value, valid_intermediate_states, invalid_states) construct
a C<_WaitOnAttributes> (see end of this file) object. Each of the attributes
is a hash-reference with two keys: strings and regexps.

=cut

sub _validate_and_return_single_wait_on_attribute {
    $Log->enter();
    my ( $pkg_or_obj, %opts ) = @_;
    if ( !defined $opts{attribute_to_check} ) {
        $Log->exit();
        NATE::Exceptions::Argument->throw(
            "'attribute_to_check' argument " . "was not provided" );
    }

    my @till_values;
    if ( ref( $opts{till_value} ) eq 'ARRAY' ) {
        if ( @{ $opts{till_value} } ) {
            @till_values = @{ $opts{till_value} };
        } else {
            $Log->exit();
            NATE::Exceptions::Argument->throw( 'Parameter '
                    . "'till_value' cannot be an empty array-referemce" );
        }
    } else {
        push @till_values, $opts{till_value} if ( defined $opts{till_value} );
    }

    _check_if_each_is_str_or_regexp(
        argument => 'till_value',
        values   => \@till_values
    );

    # If "till_not_value" is specified then negate it and include it
    # in the "till_value" array. For strings the negation is done by
    # turning it into a regex first.
    if ( defined $opts{till_not_value} ) {
        my $till_not_value = $opts{till_not_value};
        if ( ref($till_not_value) =~ /Regexp/i ) {
            push @till_values, qr/^(?!(?s:.*)$till_not_value)/;
        } else {
            my $till_not_regex = qr/^$till_not_value$/;
            push @till_values, qr/^(?!(?s:.*)$till_not_regex)/;
        }
    }

    my $valid_intermediate_states = $opts{valid_intermediate_states};
    _check_if_each_is_str_or_regexp(
        argument => 'valid_intermediate_states',
        values   => $valid_intermediate_states
    );
    my %valid_intermediate_states_hash
        = $pkg_or_obj->_populate_hash_with_string_and_regexp(
        $valid_intermediate_states);

    my $invalid_states = $opts{invalid_states};
    _check_if_each_is_str_or_regexp(
        argument => 'invalid_states',
        values   => $invalid_states
    );

    my %invalid_states_hash
        = $pkg_or_obj->_populate_hash_with_string_and_regexp($invalid_states);

    my %till_values_hash
        = $pkg_or_obj->_populate_hash_with_string_and_regexp( \@till_values );

    $Log->exit();
    return _WaitOnAttributes->new(
        valid_intermediate_states => {%valid_intermediate_states_hash},
        invalid_states            => {%invalid_states_hash},
        till_values               => {%till_values_hash}
    );
}

# This is a function, not a method
# Expects "argument" and "values" to be passed
# "argument" is the name of the argument being validated
# "values" is an array-reference of values
sub _check_if_each_is_str_or_regexp {
    $Log->enter();
    my %opts   = @_;
    my @values = @{ $opts{values} };
    foreach my $value (@values) {
        my $ref = ref $value;
        if ( $ref && ( $ref !~ /Regexp/i ) ) {
            my $argument = $opts{argument};
            $Log->exit();
            NATE::Exceptions::Argument->throw( 'Each of the values provided '
                    . "in the '$argument' argument should be a string or "
                    . 'regular expression, but one of them was provided as a '
                    . "'$ref'. Dumper:\n"
                    . NACL::ComponentUtils::Dumper($value) );
        }
    }
    $Log->exit();
}

=head2 _wait_on_attributes_helper

Helper method for wait_on_attribute and wait_on_attributes. The heavy lifting
of issuing the command, checking for valid/invalid states and periodically
polling is performed here.

This method accepts the arguments described by
C<_common_wait_for_validate_spec> along with an
C<all_attribute_entries> argument. The C<all_attribute_entries> argument
is a hash-reference with each key being an attribute to wait on and each
corresponding value being an object of type C<_WaitOnAttributes>.

=cut

sub _wait_on_attributes_helper {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            $pkg_or_obj->_common_wait_for_validate_spec(),
            all_attribute_entries => { type => HASHREF },
            show_all_fields => { type => BOOLEAN, optional => 1 },
        }
    );
    my $param_global = param_global();
    $opts{show_all_fields} //= $param_global->get('NACL_SHOW_ALL_FIELDS');
    if (!defined $opts{show_all_fields}){
        $opts{show_all_fields} = 0;
    }

    my $polling_interval = $opts{polling_interval};
    my $time_to_wait     = $opts{command_interface}
        ->scale_timeout( 'method-timeout' => $opts{'method-timeout'} );

    while ( my ( $key, $value ) = each %{ $opts{all_attribute_entries} } ) {
        unless ( blessed $value && $value->isa('_WaitOnAttributes') ) {
            $Log->exit();
            my $me = ( caller(0) )[3];
            NATE::Exceptions::Argument->throw( 'Internal error: All of the '
                    . "attributes to $me should have been objects of "
                    . "type '_WaitOnAttributes' but one of them was of "
                    . 'type '
                    . ref($value)
                    . ". Dumper:\n"
                    . Data::Dumper::Dumper($value) );
        }
    }
    my $show_all = $opts{show_all_fields};

    my %all_attribute_entries = %{ $opts{all_attribute_entries} };
    my @attributes_to_check   = keys %all_attribute_entries;

    my $filter = $pkg_or_obj->get_primary_keys_options(%opts);
    my $CS_pkg = $pkg_or_obj->get_CS_package_name();

    my $element = $pkg_or_obj->_determine_element_name();
    my $name    = $pkg_or_obj->_get_component_string($filter);

    my $end_time = time() + $time_to_wait;
    ## check if method-timeout is provided as -1
    $end_time = -1 if($opts{'method-timeout'} == -1 );

    my %common_options;
    $pkg_or_obj->_copy_common_component_params(
        source => \%opts,
        target => \%common_options
    );

    my %fetch_opts = %common_options;
    $fetch_opts{command_interface} = $opts{command_interface};
    $fetch_opts{filter}            = $filter;

    if ( !$show_all ) {
        $fetch_opts{requested_fields} = [@attributes_to_check];
    }
    my $last_transfer_type_base = ${ $opts{check_error_in_wait_for_initialization } }{'last-transfer-type'}     if($opts{check_error_in_wait_for_initialization});
    while (1) {
        my $state_obj;
        nacl_method_retry(
            code       => sub { $state_obj = $CS_pkg->fetch(%fetch_opts); },
            exceptions => 'NACL::Exceptions::NoElementsFound',
            tries_count => 3,
            sleep_time  => 10,
        );
        if (ref $state_obj eq 'NACL::CS::Snapmirror' && !('last-transfer-error' ~~ @attributes_to_check) ) {
            if( defined $state_obj->last_transfer_error && defined $last_transfer_type_base ) {
                my $last_transfer_type = $state_obj->last_transfer_type;
                my $errtext = $state_obj->last_transfer_error;
                if (($last_transfer_type eq $last_transfer_type_base) && $errtext ne '-' ) {
                    NACL::Exceptions::VerifyFailure->throw($errtext);
                }
            }
        }
        # The keys of this hash are the attributes whose values matched
        # in this iteration
        my %attributes_matched_in_iteration;
        while ( my ( $attribute_to_check, $attribute_entry )
            = each %all_attribute_entries )
        {
            my $attribute_value = $state_obj->$attribute_to_check();
            if ( !defined $attribute_value ) {

                # Erroring out if any of the field values is undefined.
                # Earlier, an undefined value would result in the code
                # failing elsewhere with a cryptic validation failure
                $Log->exit();
                NATE::BaseException->throw( 'The value of the attribute '
                        . "'$attribute_to_check' is required for waiting "
                        . 'for completion but no value was returned for it '
                        . 'from the call to fetch()' );
            }

            my %till_values_hash = $attribute_entry->till_values();

            # Has the attribute value become any of the final states?
            if ($pkg_or_obj->_found_in_string_regexp_hash(
                    hashref         => \%till_values_hash,
                    attribute_value => $attribute_value
                )
                )
            {

                # One of the attributes matched in time.
                $attributes_matched_in_iteration{$attribute_to_check} = 1;
                next;
            }

            # Check if an invalid state was reached. We know this if the
            # attribute value:
            # 1) matches an entry in the user-provided "invalid_states" OR
            # 2) Does not match anything provided in the
            # "valid_intermediate_states" parameter. For this condition, we
            # need to first check if this parameter was passed any values.
            # If we perform the check when no values were passed for this
            # parameter, the check would tell us that there was no match,
            # tricking the code into thinking it entered an invalid state.
            my %invalid_states_hash = $attribute_entry->invalid_states();
            my %valid_intermediate_states_hash
                = $attribute_entry->valid_intermediate_states();

            # If the valid_intermediate_state_hash states an empty strings
            # entry and an empty regexps entry, then don't bother searching
            # within it.
            my $need_to_search = sub {
                if (   !keys( %{ $valid_intermediate_states_hash{strings} } )
                    && !@{ $valid_intermediate_states_hash{regexps} } )
                {
                    return 0;
                } else {
                    return 1;
                }
            };
            if ((   $pkg_or_obj->_found_in_string_regexp_hash(
                        hashref         => \%invalid_states_hash,
                        attribute_value => $attribute_value
                    )
                )
                || ($need_to_search->()
                    && !$pkg_or_obj->_found_in_string_regexp_hash(
                        hashref         => \%valid_intermediate_states_hash,
                        attribute_value => $attribute_value
                    )
                )
                )
            {

                # Invalid state reached!

                # Construct a useful error message here
                my %all_valid_fields_hash
                    = ( %valid_intermediate_states_hash, %till_values_hash );
                my $error_text
                    = $element
                    . " '$name' reached the unexpected value '$attribute_value' "
                    . "for the attribute '$attribute_to_check'\n";
                $error_text
                    .= $pkg_or_obj->_construct_error_msg_string_regexp_hash(
                    \%all_valid_fields_hash )if (defined (
                    $pkg_or_obj->_construct_error_msg_string_regexp_hash
                    (\%all_valid_fields_hash )));
                $Log->exit();
                NACL::Exceptions::UnexpectedState->throw(
                    $error_text,
                    monitoring_fields   => $attribute_to_check,
                    $attribute_to_check => $attribute_value
                );
            }
        }

        if (scalar( keys %attributes_matched_in_iteration )
            == @attributes_to_check )
        {

            # Matched all attributes in time, yay!
            last;
        }

        # Simply keeping a track of elapsed time by adding the delta would
        # be roughly correct, but it does not account for the fact that
        # sending the command, it running and returning back takes extra
        # time. This means we could end up waiting for longer than we
        # really should. For this reason we use time() to tell us when
        # we should stop waiting.
        if ( time() > $end_time ) {
            my $error_text
                = "After waiting for $time_to_wait seconds "
                . 'the following attributes being waited on did not reach '
                . "any of the expected value(s).\n\n";
            while ( my ( $attribute_to_check, $attribute_entry )
                = each %all_attribute_entries )
            {
                if ( !exists
                    $attributes_matched_in_iteration{$attribute_to_check} )
                {
                    my %till_values_hash = $attribute_entry->till_values();
                    my $msg
                        = "For attribute '$attribute_to_check':\n"
                        . 'Final value obtained: '
                        . $state_obj->$attribute_to_check() . "\n"
                        . $pkg_or_obj
                        ->_construct_error_msg_string_regexp_hash(
                        \%till_values_hash )
                        . "\n\n";
                    $error_text .= NATE::Util::indent( 4, $msg );
                }
            }

            my $dump_core = Tharn::param('NACL_COREDUMP_ON_TIMEOUT');
            if (defined $dump_core and $dump_core == 1) {
                coredump_on_timeout(command_interface => $opts{command_interface});
            }

            $Log->exit();
            NACL::Exceptions::Timeout->throw($error_text);
        } else {
            $Log->trace("Snoozing for $polling_interval seconds");
            Tharn::snooze($polling_interval);
        }

    }

    $Log->exit();
}

=head2 _populate_hash_with_string_and_regexp

    my %str_regexp_hash  = $pkg_or_obj->_populate_hash_with_string_and_regexp(\@arr);
    ...
    if ($pkg_or_obj->_found_in_string_regexp_hash(hashref => \%str_regexp_hash, attribute_value => $attr))
    {
        # Found in hash
    }

(For developers only, probably only required for wait_on_attribute)
This method takes a single un-named parameter which is an array-reference of
strings and regexps (which the user would have provided in either "till_value"
or "invalid_states" in the call to wait_for_completion/wait_on_attribute).

It returns a hash with the provided values segregated into two keys:
* strings: this is a hash-reference with the keys being all of the strings
provided by the user
* regexps: an array-reference containing all the regular expressions passed

For example, if the array-reference passed in by the user was:
[ "str1", qr/reg1/, "str2", qr/reg2/ ]
it constructs and returns this hash:
(
  'strings' => { 'str1' => 1, 'str2' => 1 },
  'regexps' => [ qr/reg1/, qr/reg2/ ]
)

This data structure is constructed since it allows for a quick search for
whether an attribute value matches anything provided by the user.

=cut

sub _populate_hash_with_string_and_regexp {
    $Log->enter();
    my $pkg_or_obj            = shift;
    my @opts                  = validate_pos @_, { type => ARRAYREF };
    my @arr_of_str_and_regexp = @{ $opts[0] };
    my ( %constructed_hash, %strings, @regexps );
    foreach my $field (@arr_of_str_and_regexp) {
        my $ref = ref $field;
        if ( $ref =~ /Regexp/i ) {
            push @regexps, $field;
        } elsif ( !$ref ) {
            $strings{$field} = $field;
        } else {

            # Should never come here, but let's just leave it in just
            # in case
            $Log->exit();
            NATE::Exceptions::Argument->throw( 'Value provided '
                    . "can be either a string or a regexp, but "
                    . "one of the values provided was of type '$ref'" );
        }
    }
    $constructed_hash{strings} = \%strings;
    $constructed_hash{regexps} = \@regexps;
    $Log->debug( "Constructed hash is:\n" . Dumper( \%constructed_hash ) );
    return %constructed_hash;
}

=head2 _found_in_string_regexp_hash

    my %str_regexp_hash  = $pkg_or_obj->_populate_hash_with_string_and_regexp(\@arr);
    ...
    if ($pkg_or_obj->_found_in_string_regexp_hash(hashref => \%str_regexp_hash, attribute_value => $attr))
    {
        # Found in hash
    }

(For developers only, probably only required for wait_on_attribute)
A helper method required for wait_on_attribute. Given a hash-reference of
strings and regexps (this would have been constructed by
_populate_hash_with_string_and_regexp, explained above) and an attribute value,
check whether there is a match in the hash.

The algorithm is:
* Check if there exists a key in the "strings" portion of the supplied
hash-reference, if so return 1. (This allows us to search with a single
access)
* If it does not exists in the "strings" portion, then we need to iterate
over each of the regular expressions present in "regexps" doing a regex check
till we find a match.

For example, consider these arguments:
hashref => { strings => { 'ready' =>  1, 'running' => 1},
             regexps => [ qr/started/i, qr/on*/ ]
           }
attribute_value => 'online'

This method first checks if the key "online" exists in the strings hash-reference.
Since it does not, it starts iterating through the regexps array of regular
expressions. It does a regex check of the attribute value against each of
these regular expressions till it finds a match. Since
online =~ /on*/
this method will return 1.

=cut

sub _found_in_string_regexp_hash {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   hashref         => { type => HASHREF },
            attribute_value => { type => SCALAR }
        }
    );
    my $hashref         = $opts{hashref};
    my $attribute_value = $opts{attribute_value};

    if ( exists $hashref->{strings}{$attribute_value} ) {
        $Log->exit();
        return 1;
    } else {
        my @regexps_array = @{ $hashref->{regexps} };
        foreach my $regexp (@regexps_array) {
            if ( $attribute_value =~ $regexp ) {
                $Log->exit();
                return 1;
            }
        }
    }

    $Log->exit();
    return 0;
}

# Construct a message stating the valid string and regexp values
# This is used in error messages for when any of the required attribute
# values were not reached
sub _construct_error_msg_string_regexp_hash {
    $Log->enter();
    my $pkg_or_obj         = shift;
    my @opts               = validate_pos @_, { type => HASHREF };
    my %string_regexp_hash = %{ $opts[0] };
    my %strings_hash       = %{ $string_regexp_hash{strings} };
    my @regexps_array      = @{ $string_regexp_hash{regexps} };

    # Form a comma-separated string of all the string values
    my $string_msg = '';
    foreach my $key ( keys %strings_hash ) {
        $string_msg .= "'$key', ";
    }
    if (defined $string_msg) {

        # Chop the trailing comma and space
        chop $string_msg;
        chop $string_msg;
    }
    my $regexps_msg = join ', ', @regexps_array;
    my $error_text;
    if ($string_msg) {
        $error_text .= 'Expected final string values for this attribute are: '
            . "$string_msg\n";
    }
    if ($regexps_msg) {
        $error_text
            .= "Expected final regexps for this attribute are: $regexps_msg\n";
    }
    $Log->debug("Returned error message: $error_text")
       if ( defined $error_text );
    $Log->exit();
    return $error_text;
}

# Bunch of common validation specs required by the various wait_for_*
# routines.

# Validation spec common for all of the wait_for_* routines
sub _common_wait_for_validate_spec {
    return (
        'method-timeout' => { type => SCALAR, default => 60 },
        polling_interval => { type => SCALAR, default => 10 },
    );
}

# Validation spec common to all methods that wait on attributes
sub _common_wait_attr_validate_spec {
    return (
        attribute_to_check => { type => SCALAR, optional => 1 },

        # Params::Validate considers regular expressions to be of type
        # "scalarref object". For this reason, I'm adding both into the
        # allowed types for till_value
        till_value =>
            { type => SCALAR | SCALARREF | OBJECT | ARRAYREF, optional => 1 },
        valid_intermediate_states => { type => ARRAYREF, default => [] },
        invalid_states            => { type => ARRAYREF, default => [] },
        till_not_value            => {
            type      => SCALAR | SCALARREF | OBJECT,
            optional  => 1,
            callbacks => {
                "Value of 'till_not_value' must be a string or a regex" =>
                    sub {
                    return (   ( !ref( $_[0] ) )
                            || ( ( ref $_[0] ) =~ /Regexp/i ) );
                    }
                }

        },
        show_all_fields => { type => BOOLEAN, default => 0 },
    );
}

# A helper for "wait_for_deletion". Keeps issuing find till element is not
# found
sub _poll_till_deleted {
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { $pkg_or_obj->_common_wait_for_validate_spec() }
    );
    my $polling_interval = delete $opts{polling_interval};
    my $total_time       = delete $opts{'method-timeout'};

    my $end_time = time() + $total_time;

    # Get the filter for find, which would be the primary keys
    my $filter = {};
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => $filter,
        move   => [ $pkg_or_obj->get_primary_keys() ]
    );

    my $pkg = ref $pkg_or_obj || $pkg_or_obj;

    while (1) {
        my $found = $pkg->find( %opts, filter => $filter, allow_empty => 1 );
        last if not $found;

       # Again use time(), rather than simply adding on $polling_interval,
       # since that does not take into account time taken to send the command,
       # run it and receive back the response
        if ( time() > $end_time ) {
            my $name = $pkg_or_obj->_get_component_string($filter);
            $Log->exit();
            NACL::Exceptions::Timeout->throw( 'After '
                    . "waiting for $total_time, the $pkg '$name' has "
                    . "not yet gotten deleted\n" );
        }

        $Log->trace("Snoozing for $polling_interval seconds");
        Tharn::snooze($polling_interval);
    }

    $Log->exit();
}

=head2 _get_component_string

    my $primary_keys_hashref = $pkg_or_obj->get_primary_keys_options(%opts);
    my $comp_str = $pkg_or_obj->_get_component_string($primary_keys_hashref);

(For developers only)
Provided a hash-reference containing the primary keys and their values, return
a string of the form "<pk>=<val>,<pk2>=<val2>,...."

Note that this takes a single un-named parameter, which is the primary keys
hash-reference. Even if invoked with an object, this method will NOT
take on extracting the primary keys from it. Providing the primary keys
hash-reference is mandatory.

Specific Example:
    my $str = $vol_pkg_or_obj->_get_component_string(volume => 'vol', vserver => 'vs');
    # $str will be "volume=vol,vserver=vs"

=cut

sub _get_component_string {
    my $pkg_or_obj   = shift;
    my %primary_keys = %{ shift() };
    my $str = '';
    while ( my ( $key, $value ) = each %primary_keys ) {
        $str .= "$key=$value," if ( defined $value );
    }
    chop $str;
    return $str;
}

=head2 _frontend_log_debug_opts

    # NACL::C::Foo
    sub bar {
        $Log->enter();
        my $pkg_or_obj = shift;
        $pkg_or_obj->_frontend_log_debug_opts(@_);
        ...
    }

(For component developers)
This method is used to dump the arguments to component front-end methods
in "debug" log level. C<NACL::ComponentUtils::Dumper> is used to limit
the level of the dumped output to 3 levels.

If we just had this call in our component methods:
    $Log->debug("Opts to 'bar' front end are:\n" . NACL::ComponentUtils::Dumper({@_}));
then the log message would have been computed regardless of whether "debug"
level was enabled. (The message would not have been written to the log if
"debug" was not enabled, but its contents would still have been computed
since the arguments to any method need to be computed prior to being invoked)

This method determines whether debug level logging is enabled and only
if so does it compute the Dumper. Though NACL::ComponentUtils::Dumper
limits it to 3 levels, there is still some amount of computation involved
in constructing this Dumper message, which we're avoiding if "debug" level
is not enabled.

=cut

sub _frontend_log_debug_opts {
    $Log->enter();
    my ( $pkg_or_obj, %opts ) = @_;
    my $write_to_log = sub {
        my $method = ( caller(1) )[3];
        $method =~ s/.*:://;
        $Log->debug( "Opts to '$method' front end are:\n"
                . NACL::ComponentUtils::Dumper( \%opts ) );
    };

    # may_debug() is currently implemented only in nate/last
    # It will be available in nate/current only by the end of June
    # Simplify to write to log if may_log once it's available in current
    if ( $Log->can('may_debug') ) {
        if ( $Log->may_debug() ) {
            $write_to_log->();
        }
    } else {
        $write_to_log->();
    }
    $Log->exit();
}

# Left here so that callers don't break. We need to remove all calls to
# this before this null implementation can be removed.
sub _api_call_log_debug_opts { }

=head2 _validate_backend_opts

    sub _foo_mode_interface {
        my ($pkg_or_obj, @args) = @_;
        my %opts = $pkg_or_obj->_validate_backend_opts(@args);
        ...
    }

(For component developers)
This method validates the options passed to the component back-end method
against
L<_backend_validate_spec|lib-NACL-C-Component-pm/_backend_validate_spec>. It
returns the validation options as a hash.

=cut

sub _validate_backend_opts {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    $Log->exit();
    return %opts;
}

=head2 _handle_component_aggregation

    # NACL/C/StorageAggregate.pm
    sub can {
        my $pkg_or_obj = shift;
        my @opts = $pkg_or_obj->_can_validation(@_);
        my $code;
        if ($code = $pkg_or_obj->SUPER::can(@opts)) {
            return $code;
        } else {
            my $method = $opts[1];
            my
            $code = $pkg_or_obj->_handle_component_aggregation(
                method => $method,
                table  => [
                    {
                        component => 'NACL::C::StorageAggregateWafliron',
                        prefix    => 'wafliron'
                    }
                ],
            );
            if ($code) {
                return $code;
            } else {
                # Do some other processing
            }
        }
    }

(For component developers only)
This method allows for functionality from a related component to be
accessed from the current component.

The example shown above offers all of the methods provided by
NACL::C::StorageAggregateWafliron in the StorageAggregate component, with
the prefix to be provided before the method name.
For example, NACL::C::StorageAggregateWafliron has the methods "start" and
"stop". These can be accessed from the StorageAggregate component by doing

    $aggr->wafliron_start();
    $aggr->wafliron_stop();

If it could resolve the method to one of the linked components, then this
method returns a code-reference. Else it returns undef and updates the
error pointer with a message stating that it couldn't be found in the
linked components.

=over

=item Options

=over

=item C<< method => $method >>

(Required)
The name of the method invoked. This will be the same as $AUTOLOAD.

=item C<< table => $table >>

(Required, ARRAYREF of HASHREFs)
This is a dispatch table. Each hashref contains two entries:

=over

=item * component

The component whose functionality we want to be accessible
from the current component

=item * prefix

The prefix to be placed the method name for us to be able to
access the other component's functionality

=back

=item C<< error_ptr => $error_ptr >>

(Required, reference to a scalar)
A reference to the scalar variable containing the message describing
where the method could not be resolved.

=back

=back

=cut

sub _handle_component_aggregation {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            table     => { type => ARRAYREF },
            method    => { type => SCALAR },
            error_ptr => { type => SCALARREF }
        },
    );
    my $original_method = delete $opts{method};
    my $table           = delete $opts{table};
    my $error_ptr       = delete $opts{error_ptr};

    my $pkg_name = $pkg_or_obj->get_package_name();

    local $Data::Dumper::Terse = 1;

    my $err_str = sub {
        $Log->enter();
        my ( $self, %args ) = @_;
        my $comp_or_task = $args{comp_or_task};
        my $prefix = $args{prefix};
        my $compile_error = $args{compile_error};
        my $err = "Could not locate "
                 . "package '$comp_or_task' (for the prefix "
                 . "'$prefix', the component '$comp_or_task' "
                 . "is looked in for an implementation)\n"
                 . "The script error that occurred while trying  "
                 . "to load the package was:\n"
                 . $compile_error
                 . "\n";
        $Log->exit();
        return $err;
    };
    foreach my $entry (@$table) {
        if ( ref $entry ne 'HASH' ) {
            my $ref = ref $entry;
            $Log->exit();
            NATE::Exceptions::Argument->throw( "Each entry in the 'table' "
                    . 'parameter must be a hashref but an entry was of '
                    . "of type '$ref'. Dumper of 'table':\n"
                    . Data::Dumper::Dumper($table)
                    . "\n" );
        } else {
            my $comp_or_task = $entry->{component};

            # For layers above components, we should be looking for
            # the appropriate library at that layer. Replace "NACL::C::"
            # with "NACL::{$c_or_t}::"
            my $c_or_t = $pkg_name->is_c_or_t();
            $comp_or_task =~ s/::C::/::${c_or_t}::/;
            my $prefix = $entry->{prefix};
            if ( $original_method =~ /^${prefix}_(.*)/ ) {

                # Matched some prefix, check the relevant component
                # for the implementation of the method
                my $specific_method = $1;
                eval "require $comp_or_task";

                if ($@) {
                    if ( $c_or_t eq 'STask' ) {
                         $comp_or_task = $entry->{component};
                         eval "require $comp_or_task";
                         if ( $@ ) {
                             $Log->exit();
                             NATE::BaseException->throw($err_str->(comp_or_task => $comp_or_task,
                                                                   prefix       => $prefix,
                                                                   compile_error => $@ ));
                         }
                    } else {
                         $Log->exit();
                         NATE::BaseException->throw($err_str->(comp_or_task => $comp_or_task,
                                                               prefix       => $prefix,
                                                               compile_error => $@ ));
                    }
                }

                # Keep this check first since we need to do something
                # special for "state"
                if ( $specific_method eq 'state' ) {
                    my $sub = sub {
                        $Log->enter();
                        my ( $self, @opts ) = @_;

                      # Need to make two calls to _common_validate_with here:
                      # The first call is to normalize package/instance calls.
                      # The second is to call the _common_validate_with of
                      # the destination C/STask. This ensures that the
                      # primary keys are filled in and that any package
                      # specific _common_validate_with is invoked
                        my %non_filter_opts
                            = $pkg_or_obj->_common_validate_with(
                            params              => \@opts,
                            allow_extra         => 1,
                            ignore_primary_keys => 1
                            );
                        %non_filter_opts
                            = $comp_or_task->_common_validate_with(
                            params      => [%non_filter_opts],
                            allow_extra => 1
                            );
                        my %filter;
                        $comp_or_task->_move_primary_keys(
                            source => \%non_filter_opts,
                            target => \%filter,
                        );

                        my $comp_or_task_CS
                            = $comp_or_task->get_CS_package_name();
                        my $CS = $comp_or_task_CS->fetch(
                            filter => \%filter,
                            %non_filter_opts
                        );
                        $Log->exit();
                        return $CS;
                    };
                    $Log->exit();
                    return $sub;
                } elsif ( $specific_method eq 'find' ) {
                    my $sub = sub {
                        $Log->enter();
                        my ( $self, @args ) = @_;
                        my %orig_opts = $self->_common_validate_with(
                            params              => \@args,
                            ignore_primary_keys => 1,
                            allow_extra         => 1
                        );
                        my %primary_keys;
                        $self->_move_primary_keys(
                            source => \%orig_opts,
                            target => \%primary_keys
                        );
                        my %opts = validate_with(
                            params => \%orig_opts,
                            spec   => $comp_or_task->_find_validate_spec()
                        );
                        $self->_copy_primary_keys(
                            source => \%primary_keys,
                            target => $opts{filter},
                        );
                        my @objs = $comp_or_task->find(%opts);
                        $Log->exit();
                        return wantarray ? @objs : $objs[0];
                    };
                    $Log->exit();
                    return $sub;
                } elsif ( $comp_or_task->can($specific_method) ) {
                    # Use _common_validate_with to flatten object
                    # attributes/options into a single hash. allow_extra
                    # is necessary since we have no idea what the
                    # validation spec is here (the actual method of the
                    # linked component knows this)
                    my $sub = sub {
                        $Log->enter();
                        my ( $pkg_or_obj, @opts ) = @_;
                        my %specific_opts
                            = $pkg_or_obj->_common_validate_with(
                            params              => \@opts,
                            allow_extra         => 1,
                            ignore_primary_keys => 1
                            );

                        # Now make a package call with the flattened hash of
                        # options. This will now take care of the real
                        # validation
                        my $ret
                            = $comp_or_task->$specific_method(%specific_opts);
                        $Log->exit();
                        return $ret;
                    };
                    $Log->exit();
                    return $sub;
                } else {

                    # Some prefix was matched, but an implementation for the
                    # specific method was not found in the linked component.
                    $$error_ptr
                        .= "No implementation for "
                        . "'$specific_method' was found in the linked "
                        . "package '$comp_or_task'\n";
                    $Log->exit();
                    return undef;
                }
            }
        }
    }

    # None of the prefixes were matched
    $$error_ptr
        .= "No implementation for '$original_method' was "
        . "was found in any of the linked packages. Associated "
        . "components and their prefixes:\n"
        . Data::Dumper::Dumper($table) . "\n";

    $Log->exit();
    return undef;
}

# Providing an empty implementation here so that any over-loaded
# AUTOLOADs will not require defining their own empty DESTROY
sub DESTROY { }

=head2 _find_wrapper_validate_spec

This returns the Params::Validate spec for the C<_find_wrapper_helper> method.

=cut

sub _find_wrapper_validate_spec {
    $Log->enter();
    my $pkg                        = shift;
    my $find_wrapper_validate_spec = $pkg->_find_validate_spec();
    foreach my $find_opt (qw(filter canned_filters)) {
        $find_wrapper_validate_spec->{"extra_$find_opt"}
            = $find_wrapper_validate_spec->{$find_opt};
    }
    $Log->exit();
    return $find_wrapper_validate_spec;
}

=head2 _find_wrapper_helper

    # A simple example of how we'd write a "find_data_vservers"
    sub find_data_vservers {
        my $pkg = shift;
        my @vservers = $pkg->_find_wrapper_helper(
            @_,
            extra_canned_filters => [ qw(data_vservers) ]
        );
        return wantarray ? @vservers : $vservers[0];
    }

This method is a helper for development of wrapper methods round find.

=over

=item Options

=over

=item C<< extra_filter => \%filter >>

(Optional)

An extra filter to be applied to the find() to obtain the appropriate
elements.

=item C<< extra_canned_filters => [ @canned_filters ] >>

(Optional)

Extra canned filters to be applied to the find() to obtain the appropriate
elements.

=back

=back

=cut

sub _find_wrapper_helper {
    $Log->enter();
    my $pkg = shift;
    my %opts = validate @_, $pkg->_find_wrapper_validate_spec();

    my %new_opts = %opts;

    my $extra_filter = delete $new_opts{extra_filter};
    $new_opts{filter} = { %$extra_filter, %{ $opts{filter} } };
    my $extra_canned_filters = delete $new_opts{extra_canned_filters};
    $new_opts{canned_filters}
        = [ @$extra_canned_filters, @{ $opts{canned_filters} } ];

    my @elements = $pkg->find(%new_opts);
    $Log->exit();
    return @elements;
}

=head2 _find_wrapper_generator

    sub can {
        my ($pkg_or_obj, @opts) = @_;
        my $code;
        if ($code = $pkg_or_obj->SUPER::can(@opts)) {
            return $code;
        } else {
            $code = $pkg_or_obj->_find_wrapper_generator(@opts);
            if ($code) {
                return $code;
            } else {
                # Try to resolve some other way
            }
        }
        ...
    }

This method is used to help generate find wrapper methods. This returns
the code-reference to the constructed find wrapper method, else returns
undef (and updates the error pointer) if it could not construct one
(this would happen when the canned filter does not exist).

=cut

sub _find_wrapper_generator {
    $Log->enter();
    my ( $pkg_or_obj, @opts ) = @_;
    my $method = $opts[0];
    if ( $method =~ /^find_(.*)/ ) {
        my $canned_filter = $1;
        my $CS_pkg        = $pkg_or_obj->get_CS_package_name();
        if ( $CS_pkg->can("_canned_filter_$canned_filter") ) {
            my $sub = sub {
                $Log->enter();
                my $pkg      = shift;
                my @vservers = $pkg->_find_wrapper_helper( @_,
                    extra_canned_filters => [$canned_filter] );
                $Log->exit();
                return wantarray ? @vservers : $vservers[0];
            };
            $Log->exit();
            return $sub;
        } else {
            my $error_ptr = $opts[1];
            $$error_ptr = "'$method' is not a find wrapper "
                . "(canned filter '$canned_filter' does not exist)\n";
        }
    }
    $Log->exit();
    return undef;
}

=head2 _can_ci_be_used_for_nodescope

    my $apiset = $pkg_or_obj->_can_ci_be_used_for_nodescope(command_interface => $ci);

This method throws a C<NATE::Exceptions::Argument> exception if the
command interface provided cannot be used for Nodescope operations.
(for example, a NACL::C::Vserver command interface cannot be used for
Nodescope operations)
If it can, then it returns a 7Mode or Nodescope APISet object, depending
on whether the host is 7Mode or CMode respectively.

=over

=item Options

=over

=item C<< command_interface => $ci >>

The command interface object.

=back

=back

=cut

sub _can_ci_be_used_for_nodescope {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            command_interface =>
                { type => OBJECT, isa => 'NACL::C::CommandInterface::ONTAP' }
        },
    );
    my $ci = $opts{command_interface};
    if ( $ci->can('get_7m_or_nodescope_apiset') ) {
        $Log->exit();
        return $ci->get_7m_or_nodescope_apiset();
    } else {
        $Log->exit();
        NATE::Exceptions::Argument->throw( 'The command interface '
                . 'provided ('
                . ref($ci)
                . ') cannot be used for 7Mode/Nodescope method calls' );
    }
}

=head2 _copy_primary_keys

    $pkg_or_obj->_copy_primary_keys(
        source => \%opts,
        target => \%primary_keys
    );

Copies the primary keys from the source hash to the target hash.

=cut

sub _copy_primary_keys {
    $Log->enter();
    my ( $pkg, @args ) = @_;
    $pkg->_hash_copy( @args, copy => [ $pkg->get_primary_keys() ] );
    if (wantarray) {
        my %opts = @args;
        $Log->exit();
        return $opts{target};
    }
    $Log->exit();
}

=head2 _move_primary_keys

    $pkg_or_obj->_move_primary_keys(
        source => \%opts,
        target => \%primary_keys
    );

Moves the primary keys from the source hash to the target hash.

=cut

sub _move_primary_keys {
    $Log->enter();
    my ( $pkg, @args ) = @_;
    $pkg->_hash_move( @args, move => [ $pkg->get_primary_keys() ] );
    if (wantarray) {
        my %opts = @args;
        $Log->exit();
        return $opts{target};
    }
    $Log->exit();
}

=head2 _get_primary_keys_w_or_wo_ci_from_pkg_or_obj

    %primary_keys_w_or_wo_ci = $pkg_or_obj->_get_primary_keys_w_or_wo_ci_from_pkg_or_obj( want_ci => 1 );

This method is used to return the primary keys from the arguments provided.
In addition to common component parameters, there is an additional option "want_ci"
which is specifies, method should return a command_interface or not.
It returns primari keys and command interface (if requested) in hashref of hash format based on the
context being called with.

=over

=item Options

=over

=item C<< want_ci => 0|1 >>

(Optoinal, BOOLEAN)
when this is specified as 1, method returns the command_interface
along with primary keys.

=back

=back

=cut

sub _get_primary_keys_w_or_wo_ci_from_pkg_or_obj {
    $Log->enter();
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params      => \@args,
        additional_spec => {want_ci => {type => BOOLEAN, optional => 1}},
        allow_extra => 1,
    );
    my %primary_keys_w_or_wo_ci;
    my $want_ci = delete $opts{'want_ci'};
    $pkg_or_obj->_copy_primary_keys(
        source => \%opts,
        target => \%primary_keys_w_or_wo_ci
    );
    $primary_keys_w_or_wo_ci{'command_interface'} = $opts{'command_interface'} if($want_ci);
    $Log->exit();
    return wantarray ? %primary_keys_w_or_wo_ci : \%primary_keys_w_or_wo_ci;
}

=head2 _get_package_at_same_layer

    # $pkg = 'NACL::C::Vserver'
    my $pkg = NACL::C::Volume->_get_package_at_same_layer(file => 'Vserver');

    # $pkg = 'NACL::C::Vserver'
    my $pkg = $vol_c_obj->_get_package_at_same_layer(file => 'Vserver');

    # $pkg = 'NACL::STask::Vserver'
    my $pkg = NACL::STask::Volume->_get_package_at_same_layer(file => 'Vserver');

    # $pkg = 'NACL::STask::Vserver'
    my $pkg = $vol_t_obj->_get_package_at_same_layer(file => 'Vserver');

Gets the corresponding package at the same layer (C/STask) as the invoking
package/object.

=cut

sub _get_package_at_same_layer {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { file => { type => SCALAR } }
    );
    my $file = ucfirst $opts{file};
    my $c_or_t = $pkg_or_obj->is_c_or_t();
    my $package_at_same_layer = "NACL::${c_or_t}::$file";
    $Log->exit();
    return $package_at_same_layer;
}

# Why the method from the mix-in doesn't work is one of life's great mysteries.
# For now, doing the ugly thing of repeating the code both here and in
# CS/ComponentState.pm
sub _check_relational_regex_filter {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate (@args, {filter_value => {type => SCALAR|OBJECT|ARRAYREF}});
    my $ret = 0;
    # Check if the filter value is anyone of the type  value*, !value, >=value,
    # <=value, >value, <value,
    # value1..value2 and value1|value2  AND is not
    # within quotes (values within quotes are to be considered as literals)
    my $filter_value_ref = (ref $opts{filter_value} eq 'ARRAY') ? $opts{filter_value} : [$opts{filter_value}];
    foreach my $filter_value (@$filter_value_ref) {
        if ( (   $filter_value =~ /^[!><]/
            || $filter_value =~ /\S+(\.\.|\|)\S+/
            || $filter_value =~ /\*/)
            && $filter_value !~ /^".*"$/ && !(ref($opts{filter_value})
            && !ref $opts{filter_value} eq 'ARRAY')) {
            $ret = 1;
            last;
        }
    }
    $Log->exit();
    return $ret;
}

use Storable qw(freeze thaw);

sub STORABLE_freeze {
    my ($self, $cloning) = @_;
    my $data = {%$self};
    return ('', $data);
}

# See burt 915491.
# Keep a cache of command_interface objects already thawed (key is the UUID),
# so that a subsequent attempt to de-serialize a command_interface object
# can just use the previously de-serialized object.
my %Thawed_Ci_Objs;

# Primary for unit testing
sub _get_thawed_ci_objs {return %Thawed_Ci_Objs};

use NATE::Events qw(call_on_fork_add);

# Reset the cache when new process is spawned
call_on_fork_add(
    {
        callback     => sub {%Thawed_Ci_Objs = ()},
        keep_on_fork => 1,
    },
);

sub STORABLE_thaw {
    my ($self, $cloning, $dummy, $data) = @_;
    if (exists $data->{command_interface} && defined $data->{command_interface}{_uuid}) {
        my $this_uuid = $data->{command_interface}{_uuid};

        # This check should be defined() and not exists(), because if a
        # ci gets garbage collected, then an entry will still exist in the
        # hash but its value will be undef. For such cases, it should use
        # the thawed ci present in $data.
        if (defined $Thawed_Ci_Objs{$this_uuid}) {
            $data->{command_interface} = $Thawed_Ci_Objs{$this_uuid};
        } else {
            $Thawed_Ci_Objs{$this_uuid} = $data->{command_interface};

            # Our cache should not increase the refcount. Not weakening
            # the reference will prevent it from being cleaned up.
            Scalar::Util::weaken ($Thawed_Ci_Objs{$this_uuid});
        }
    }
    %$self = %$data;
}

1;

# A private class used for waiting on attributes. Rather than construct
# hashes of hashes, it would be somewhat easier to deal with objects. This
# way, we can also centralize the validation of the options.

package _WaitOnAttributes;

use strict;
use warnings;

sub init {
    my $self = shift;
    validate_with(
        params => \@_,
        spec   => {
            _is_hashref(
                qw(valid_intermediate_states
                    invalid_states till_values)
            )
        }
    );
}

use Class::MethodMaker [
    new  => [ '-hash', 'init', 'new' ],
    hash => 'valid_intermediate_states',
    hash => 'invalid_states',
    hash => 'till_values'
];

use Params::Validate qw(validate_with HASHREF);

sub _is_hashref {
    my (@names) = @_;
    return ( map { $_ => { type => HASHREF } } @names );
}

1;
