
# Copyright (c) 2012-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Abstract base class for Client components
## @author Vishwanath.rawat@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Component::Client

=head1 SYNOPSIS

    use base 'NACL::C::Component::Client';

=head1 DESCRIPTION

NACL::C::Component::Client is an abstract base class for all
Client Component libraries.
It is derived from L<NACL::C::Component|lib-NACL-C-Component-pm>
and it provides Client specific component base class methods.

=head1 ATTRIBUTES

Same as those of L<NACL::C::Component|lib-NACL-C-Component-pm/ATTRIBUTES>.
(command interfaces are constrained to be sub-classes of
C<NACL::C::Client>.)

=cut

package NACL::C::Component::Client;

use strict;
use warnings;
use NATE::BaseException qw/:try/;
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR BOOLEAN HASHREF SCALARREF ARRAYREF OBJECT);

use base 'NACL::C::Component';

use Class::MethodMaker [
    scalar => [
        {   -type => 'NACL::C::Client' },
        'command_interface'
    ],
];

################################################################################
# Subroutine Name: _change_parameter_value_in_file()
# Objective:       Will change the value of a given parameter in a config file
################################################################################
sub _change_parameter_value_in_file {
    $Log->enter();
    my ($self, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            file   => { type => SCALAR },
            param  => { type => SCALAR },
            value  => { type => SCALAR },
            apiset => { isa  => "NACL::APISet", }
        }
    );

    my $param = $opts{param};
    my $file = $opts{file};
    my $value = delete $opts{value};
    my $apiset = $opts{apiset};

    $self->_comment_out_parameter_in_file(%opts);

    my $cmd =  "perl -p -i.bak -e 'BEGIN{\$x=0}; END{exit \$x}; if (\$x==0) { if (s/^\\s*#?\\s*($param)\\s*=.+/\\1 = $value/) {\$x=1}}' $file";
    try {
        $apiset->execute_raw_command(command => $cmd);
    } catch NATE::BaseException with {
        # If failed, no instances of $param were modified
        # append the $param and value to the end of the file
        my $line = "$param = $value";
        $cmd = "echo \'$line\' >> $file";
        $apiset->execute_raw_command(command => $cmd);
    };

    $Log->exit();

}


################################################################################
# Subroutine Name: _comment_out_parameter_in_file()
# Objective:       will comment out all instances of the given parameter in
#                  a config file
################################################################################
sub _comment_out_parameter_in_file {
    $Log->enter();
    my ($self, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            file   => { type => SCALAR },
            param  => { type => SCALAR },
            apiset => { isa  => "NACL::APISet", }
        }
    );

    my $param = $opts{param};
    my $file = $opts{file};
    my $apiset = $opts{apiset};

    my $cmd = "perl -p -i.bak -e 'END {exit 0}; s/^\\s*#?\\s*($param\\s*=.*)/#\\1/g' $file";
    $apiset->execute_raw_command(command => $cmd);

    $Log->exit();
}

sub _rescan_scsi_bus {
    $Log->enter();
    my ($self, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            option   => { type => SCALAR, optional => 1, },
            protocol => { type => SCALAR },
            apiset   => { isa  => "NACL::APISet", }
        }
    );

    my $apiset = $opts{apiset};
    my $option = $opts{option};
    my $protocol = $opts{protocol};
    if ($option =~ /remove_LUNs/i) {
        # To delete unmapped LUNs from the host, argument is: "-r"
        $apiset->execute_raw_command(command => "/usr/bin/rescan-scsi-bus.sh -r");
    } else {
        # To discover newly mapped luns
        $apiset->execute_raw_command(command => "/usr/bin/rescan-scsi-bus.sh");

        # If LUNs are mapped for the first time,
        # all the LUNs don't get detected after rescan
        # if protocol is iSCSI. So, run the script twice.
        if($protocol =~ /iscsi/i) {
            $apiset->execute_raw_command(command => "/usr/bin/rescan-scsi-bus.sh");
        }
    }

    $Log->exit();
}

sub _recreate_initrd_image {
    $Log->enter();
    my ($self, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            sanbooted => { type => BOOLEAN },
            apiset    => { isa  => "NACL::APISet", }
        }
    );

    my $sanbooted       = $opts{sanbooted};
    my $apiset          = $opts{apiset};
    my $version_manager = $apiset->get_version_manager();
    my $major           = $version_manager->{os_revision}->{major};

    # need to recreate the initrd image for sanbooted systems of SLES.
    # this step is not required for RHEL and OL.
    if ($sanbooted and ($major =~ /sles/i)) {
        # Before modifying the initrd image take a backup

        my $resp =  $apiset->uname("kernel-release" => 1);
        my $kernel_version = $resp->get_parsed_output->[0]->{kernel_release};

        my $source = "/boot/initrd-" . $kernel_version;
        my $target = $source . ".bak";
        $apiset->cp(force => 1, source => $source, target => $target);

        # Recreating the initrd image
        my $services;
        if ($major =~ /sles10/i) {
            $services = "mpath iscsi";
        } elsif ($self->{major_revision} =~ /sles11/i) {
            $services = "multipath iscsi";
        }
        $apiset->mkinitrd(force => 1, services => $services);
    }

    $Log->exit();
}

sub _verify_param_length_range {
    $Log->enter();
    my ($self, @args) = @_;

    my %opts = validate_with(
        params => \@args,
        spec   => {
            param       => { type => SCALAR },
            length_min  => { type => SCALAR },
            length_max  => { type => SCALAR },
            apiset      => { isa  => "NACL::APISet", }
        }
    );
    
    my $min    = $opts{length_min};
    my $max    = $opts{length_max};
    my $length = length($opts{param});
    
    if ($length < $min or $length > $max) {
        NACL::APISet::Exceptions::InvalidParamValueException->throw(text => 
            "Param's length should be in range of $min-$max both inclusive. " . 
            "Current length is $length." 
        );
    }
            
    $Log->exit    
}

sub _get_major_minor {
    $Log->enter();
    my ($self, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            apiset => { isa  => "NACL::APISet", }
        }
    );
    
    my $apiset = $opts{apiset};
    my $version_manager = $apiset->get_version_manager();
    $version_manager->get_version_attribute(attribute => "os_revision");
    
    my $major = $version_manager->{os_revision}->{major};
    my $minor = $version_manager->{os_revision}->{minor};
    
    $Log->exit();
    
    return ($major, $minor);
}

=head2 _base_backend_client

       $pkg_or_obj->_base_backend_cmode_cli(
            @_,
            api  => 'chmod',
       );

This method is the generic base class method for
cli backends for clients.

=over

=item Options

=over

=item C<< api => $api  >>

(Mandatory)  The name of the api command

=item C<< additional_args => \%args >>

(Optional)

Hashref which contains additonal api arguments ,which
are not listed in the cdef file and connectrec match
table which would be used to answer the prompts

=item C<< apiset_response_obj  => \$scalar  >>

(Optional) Scalarref,updates the passed in 
scalarref with the apiset response object

=item C<< map => $map  >>

(default => {}) optional HASHREF to specify the mapping
between "arguments passed to component method" TO "arguments
accepted by apiset"

=back

=back

=cut

sub _base_backend_client {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    my $spec = $pkg_or_obj->_backend_validate_spec();
    my %opts = validate_with(
        params      => \@args,
        spec        => {
            %{$spec},
            api  => {type => SCALAR},
            map  => {type => HASHREF, default => {}},
            additional_args     => {type => HASHREF, default => {}},
            apiset_response_obj => {type => SCALARREF, optional => 1},
        },
        allow_extra => 1,
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $map               = delete $opts{map};
    my $api               = delete $opts{api};
    my $apiset_response_obj  = $opts{apiset_response_obj};

    my @copy = $apiset->get_allowed_options_for_command( command => $api );

    my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => \@copy,
        target => \%api_opts,
        map    => $map,
    );
    
    %api_opts = (%api_opts, %{$opts{additional_args}});
    
    my $response = $apiset->$api(%api_opts);

    if (defined $apiset_response_obj) {
        $$apiset_response_obj = $response; 
    }

    $Log->exit();
}


=head2 get_api_options

       my %target_hash = $pkg_or_obj->get_api_options(
                             api  => $api,
                             apiset => $apiset,
                             options => $options,
                            );

This method returns a hash containing the allowed options
which an api command can take. It deletes the invalid
options from $options hashref and returns the resulting
hash

=over

=item Options

=over

=item C<< api => $api >>

(Mandatory)  The name of the api command

=item C<< apiset => $apiset_obj >>

(Mandatory) HASHREF The apiset object referring to a node.

=back

=back

=cut

sub get_api_options {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            api     => { type => SCALAR },
            options => { type => HASHREF },
            apiset  => {
                type => OBJECT,
                isa  => 'NACL::APISet'
            },
            cdef_ref => {type => SCALARREF, optional => 1},
        },
    );
    my $apiset        = $opts{apiset};
    my $api           = $opts{api};
    my $cdef_ref      = $opts{cdef_ref};
    my %input_options = %{ $opts{options} };
    my $cdef = $apiset->get_command_definition( command => $api );
    $$cdef_ref = $cdef if (defined $cdef_ref);

    my @options = $apiset->get_allowed_options_for_command(command => $api,
        cdef_obj => $cdef);

    ## Adding 'connectrec-match_table' option for component calls
    push @options, 'connectrec-match_table';

    my $alias_map = $cdef->get_alias_map();

    if ( $alias_map ) {
        while (my ($old, $new) = each (%{ $alias_map })) {
            if ( $input_options{$old} ) {
                 $input_options{$new}=delete $input_options{$old};
            }
        }
    }
    my $target = $pkg_or_obj->_hash_copy(
        source => \%input_options,
        copy   => \@options
    );
    my %target_hash =  %{$target};
    foreach my $key ( keys %target_hash ) {
        if ( ref $target_hash{$key} eq "ARRAY" && $key !~ /connectrec-match_table/ ) {
	    if ( $pkg_or_obj -> isa('NACL::C::Client::ONTAPSelectDeploy::NetworkConnectivityCheck') || $pkg_or_obj -> isa('NACL::C::Client::ONTAPSelectDeploy::Cluster')){
		$target_hash{$key} = join( " ", @{ $target_hash{$key} });
	    } else {
		$target_hash{$key} = join( ",", @{ $target_hash{$key} });
	    }
        }
    }
    $Log->exit() if $may_exit;
    return %target_hash;
}

=head2 _base_backend_ontapselectdeploy_cli

       $pkg_or_obj->_base_backend_cmode_cli(
            @_,
            api  => 'cluster_create',
       );

This method is the generic base class method for
ONTAP Select Deploy cli backends.

=over

=item Options

=over

=item C<< api => $api  >>

(Mandatory)  The name of the api command

=item C<< additional_args => \%args >>

(Optional)

Hashref which contains additonal api arguments ,which
are not listed in the cdef file and connectrec match
table which would be used to answer the prompts

=item C<< apiset_response_obj  => \$scalar  >>

(Optional) Scalarref,updates the passed in
scalarref with the apiset response object

=back

=back

=cut

sub _base_backend_ontapselectdeploy_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $spec = $pkg_or_obj->_backend_validate_spec();
    my %opts = validate_with(
        params => \@args,
        spec   => {
            %{$spec},
            api                 => {type => SCALAR},
            additional_args     => {type => HASHREF, default => {}},
            apiset_response_obj => {type => SCALARREF, optional => 1}
        },
        allow_extra => 1
    );
    my $api               = $opts{api};
    my $apiset            = $opts{apiset};
    my $execution_time    = $opts{cmd_execution_time};
    my $command_interface = $opts{command_interface};
    my $apiset_response_obj  = $opts{apiset_response_obj};
    my $cdef;
    my %ontap_selelct_deploy_args = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => $api,
        options => \%opts,
        cdef_ref => \$cdef,
    );

    %ontap_selelct_deploy_args = (%ontap_selelct_deploy_args, %{$opts{additional_args}});
    my $response = $apiset->$api(%ontap_selelct_deploy_args, cdef_obj => $cdef);

    if (defined $apiset_response_obj) {
        $$apiset_response_obj = $response;
    }
    $Log->exit() if $may_exit;

} ## end sub _base_backend_ontapselectdeploy_cli

1;
