#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Cserver Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Cserver

=head1 DESCRIPTION

C<NACL::C::Cserver> is a derived class of
L<NACL::C::CommandInterface|lib-NACL-C-CommandInterface-pm>.  It represents
access to a CMode Cserver. It can only be used as a command interface object
and using it as a component object to perform cserver related operations is
not supported. To perform cserver related operations, you can use
L<NACL::C::Cluster|lib-NACL-C-Cluster-pm> instead.
When C<NACL::C::Cserver> is used as a command interface object,
then the commands would be sent to the lif of the Cserver.

Differences between C<NACL::C::Cserver> and C<NACL::C::Cluster>:

NACL::C::Cserver

=over

=item 1

It can only be used as a command interface object.

=item 2

NACL::C::Cserver->find(command_interface => $ci) would discover the
cserver (a.k.a admin vserver)

=back

NACL::C::Cluster

=over

=item 1

It can be used as both command interface and component object.

=item 2

NACL::C::Cluster->find(command_interface => $ci) would run 'cluster show'
and thus returns objects representing all nodes in the cluster. These
returned objects would have the specified $ci as their
command_interface. If $ci represents a NACL::C::Node object, then using
the returned objects from the above call as command_interfaces would end
up sending the commands/apis to the lif of the node rather than the lif
of the cserver or admin vserver.

=back

=head2 apiset

Fetches the apiset object stored in command_interface of Cserver.
To obtain an APISet object for a particular command interface,
we can use the method 'apiset', which accepts the options
"category", "interface" and "set" to obtain the relevant APISet object.

This could be useful if there are sections of a script for which we have to drop to the API layer
for some reason and instead of having to create an APISet object ourselves,
we can use the existing, cached APISet objects.

The apiset call to cserver can be made in following ways :

Interface is specified to be ZAPI(CLI is the default value).
So the method would select the best defaults for 'category' and 'set' inputs
based on the mode in which the cserver is running.
It returns CM ZAPI APISet for CMode hosts.

my $cserver = NACL::C::Cserver->find();
my $apiset = $cserver->apiset(interface => 'ZAPI');

No inputs specified. apiset() call would return CM CLI APISet for CMode hosts.
my $cserver = NACL::C::Cserver->find();
my $apiset = $cserver->apiset();

=cut

=head1 ATTRIBUTES

=head2 name

The name of the cserver

=head2 cserver

The name of the cserver. Alias to 'name' attribute.

=head2 hostrec

The hostrec describing the host/cserver which is modeled as CommandInterface

=cut

package NACL::C::Cserver;

use strict;
use warnings;
use NACL::C::Cluster;
use NACL::CS::Cserver;
use Params::Validate qw(validate_with SCALAR ARRAYREF BOOLEAN);
use NACL::ComponentUtils qw(_optional_scalars);
use base 'NACL::C::CommandInterface::ONTAP';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Get methods common to Cluster and Cserver
use NACL::C::_Mixins::ClusterCserver qw(:all);

=head1 METHODS 

=head2 new

    my $cserver = NACL::C::Cserver->new([name => $name],
                              [cserver => $admin_vserver_name],
                              [hostrec => $hostrec] );

Specifying any one of the cserver, name, hostrec attributes will populate
the other two.
Returns a new Cserver component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => [
        {   '-default_ctor' => sub {
                $_[0]->_construct_primary_key();
                }
        },
        'cserver'
    ]
];

sub _primary_keys_validate_spec {
    return ( cserver => { type => SCALAR } );
}

=head2 nodes

  NACL::C::Cserver->nodes(command_interface=>$ci,...);

  or

  $cserverobj->nodes();


(Class or instance method) This method returns an array,
containing the name of the nodes contained in the cserver
or in the cluster.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub nodes {

    my $pkg_or_obj = shift;
    my  %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        additional_spec     => { cserver => { type => SCALAR, optional => 1 } },
        ignore_primary_keys => 1
    );

    my $command_interface = delete $opts{command_interface}
        || $pkg_or_obj->command_interface();
    delete $opts{cserver};
    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;
    $pkg  =~ s/Cserver/Cluster/;
    my @state = $pkg->find( command_interface => $command_interface, %opts );
    my @list_of_nodes = map $_->node(), @state;
    return @list_of_nodes;
} ## end sub nodes

=head2 node_objs

  NACL::C::Cserver->node_objs(command_interface=>$ci,...);

  or

  $cserverobj->node_objs();


(Class or instance method) This method returns an array,
containing Node Objects of type NACL::C::Node, for every node present in the
cserver

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub node_objs {

    my $pkg_or_obj = shift;
    my %opts       = @_;

    %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        additional_spec     => { cserver => { type => SCALAR, optional => 1 } },
        ignore_primary_keys => 1
    );

    my $command_interface = delete $opts{command_interface}
        || $pkg_or_obj->command_interface();
    delete $opts{cserver};
    my $pkg = ref($pkg_or_obj) || $pkg_or_obj;
    $pkg  =~ s/Cserver/Cluster/;
    my @list_of_nodes
        = $pkg->nodes( command_interface => $command_interface, %opts );
    my @node_objects;
    require NACL::C::Node;

    @node_objects = map NACL::C::Node->new( name => $_ ), @list_of_nodes;
    return @node_objects;
} ## end sub node_objs

1;
