
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary EventRoute Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::EventRoute

=head1 DESCRIPTION

C<NACL::C::EventRoute> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP EventRoute.  A related class is
L<NACL::CS::EventRoute|lib-NACL-CS-EventRoute-pm>, which represents
the past state of an ONTAP EventRoute.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 messagename

=cut

package NACL::C::EventRoute;

use strict;
use warnings;

use NACL::CS::EventRoute;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NACL::C::Exceptions::EventRoute::DoesNotExist;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 new

  my $EventRoute = NACL::C::EventRoute->new(command_interface=>$ci,
                                   messagename =>$messagename,
                                  );

Return a new EventRoute component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'messagename',];

=head2 find

    my $EventRoute  = NACL::C::EventRoute->find(command_interface => $ci, ...);
    my @EventRoutes = NACL::C::EventRoute->find(command_interface => $ci, ...);

    Uses a CMode CLI or CMode ZAPI APISet.

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $EventRoute_state = $NACL::C::EventRoute->state();

    Uses a CMode CLI or CMode ZAPI APISet.

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::EventRoute would return
a NACL::CS::EventRoute object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::EventRoute->modify(
                                command_interface => $ci,
                                 messagename => $messagename,
                                  ...);

                                 or

    $EventRoute_object->modify(...);

(Class or instance method)
This method is used to modify an event's destination, reporting threshold, or both.

Uses a CMode CLI/ZAPI APISet.

Invokes "ems-routing-modify-iter" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "messagename" => $messagename >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: message-name

<ems-routing-modify-iter>
<attributes>
<ems-routing-info>
<message-name>&value</message-name>
</ems-routing-info>
</attributes>
</ems-routing-modify-iter>

=item C<< "frequencythreshold" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<ems-routing-modify-iter>
<attributes>
<ems-routing-info>
<frequency-threshold>&value</frequency-threshold>
</ems-routing-info>
</attributes>
</ems-routing-modify-iter>

=item C<< "timethreshold" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: time-threshold

<ems-routing-modify-iter>
<attributes>
<ems-routing-info>
<time-threshold>&value</time-threshold>
</ems-routing-info>
</attributes>
</ems-routing-modify-iter>

=item C<< "destinations" => [ $destination1, $destination2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: destinations

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::EventRoute::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing event route.

=item C<< '*' >>

This method takes all other options supported by
"event route modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "destinations" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( frequencythreshold timethreshold
                    )
            ),
        },
        api => 'event_route_modify',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::EventRoute::modify',
        );
    }
    catch NACL::C::Exceptions::Component with {

        # Already classified by the back-end
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::EventRoute::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'event_route_modify'
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api  => 'ems-routing-modify-iter',
        show_cmd => 'event route show',
        copy => [qw(destinations)],
        map  => {
            "frequencythreshold" => "frequency-threshold",
            "messagename"        => "message-name",
            "timethreshold"      => "time-threshold",
        },
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::EventRoute::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 add_destinations

   NACL::C::EventRoute->add_destinations(
                                command_interface => $ci,
                                 messagename => $messagename,
                                  ...);

                                 or

    $EventRoute_object->add_destinations(...);

(Class or instance method)
This method is used to add destination(s) to an event definition.

Uses a CMode CLI/ZAPI APISet.

Invokes "ems-routing-add-destination" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "messagename" => $messagename >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: message-name

=item C<< "destinations" => [ $destination1, $destination2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: destinations

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"event route add-destinations" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_destinations {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {"destinations" => {type => ARRAYREF, optional => 1},},
        cli_cmd => 'event route add-destinations',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_destinations_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_destinations_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::EventRoute::add_destinations',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add_destinations

sub _add_destinations_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'event_route_add_destinations',
    );
    $Log->exit() if $may_exit;
} ## end sub _add_destinations_cmode_cli

sub _add_destinations_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'ems_routing_add_destination',
        copy   => [qw(destinations)],
        map    => {"messagename" => "message-name",},
    );
    $Log->exit() if $may_exit;
} ## end sub _add_destinations_cmode_zapi

=head2 remove_destinations

   NACL::C::EventRoute->remove_destinations(
                                command_interface => $ci,
                                 messagename => $messagename,
                                  ...);

                                 or

    $EventRoute_object->remove_destinations(...);

(Class or instance method)
This method is used to remove destination(s) from an event definition.

Uses a CMode CLI/ZAPI APISet.

Invokes "ems-routing-remove-destination" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "messagename" => $messagename >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: message-name

=item C<< "destinations" => [ $destination1, $destination2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: destinations

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"event route remove-destinations" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_destinations {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {"destinations" => {type => ARRAYREF, optional => 1},},
        cli_cmd  => 'event route remove-destinations',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_destinations_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_destinations_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::EventRoute::remove_destinations',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove_destinations

sub _remove_destinations_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'event_route_remove_destinations',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_destinations_cmode_cli

sub _remove_destinations_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api   => 'ems_routing_remove_destination',
        copy   => [qw(destinations)],
        map    => {"messagename" => "message-name",},
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_destinations_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('messagename' => {type => SCALAR},);
}

1;
