# $Id$
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An abstract base class for the exception that gets thrown when protocols
## are not added to the allowed-protocols of a vserver
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AllowedProtocolsNotAdded

=head1 DESCRIPTION

This is the base class for the exceptions that get thrown when a
service is created on vserver when the service protocol is not added to
allowed-protocols.
All exceptions which are sub-classes of this exception have the attributes
and methods listed below.

=cut

package NACL::C::Exceptions::AllowedProtocolsNotAdded;

use strict;
use warnings;

use base 'NACL::C::Exceptions::Component';

=head1 ATTRIBUTES

Each of the attributes listed below have accessor methods, so (using the
C<vserver> attribute as example) this can be done:

    my $vserver = $exception_obj->vserver();
    
=over

=item C<vserver>

The name of the vserver.

=item C<protocol>

The protocol which needs to be added to the vserver.

=item C<command_interface>

The command_interface on which the protocol needs to be added.

=back

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'protocol',
    scalar => [
        { -type => 'NACL::C::CommandInterface::ONTAP' },
        'command_interface'
    ],
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use NACL::ComponentUtils qw(_ontap_ci_validate_spec _hash_move);
use NACL::STask::Vserver;

=head1 EXAMPLES

(Using C<NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded> for the
examples below: that exception sub-classes this exception)

=head2 Throwing exception

    NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded->throw(
        # Like all other component-level exception, this has the
        # attributes of the APISet exceptions: "text" and "response_object"
        text              => $error_msg,
        response_object   => $resp,
        
        # These are specific to this exception and are mandatory
        vserver           => $vserver_name,
	protocol          => $protocol,
        command_interface => $ci
    );
    
=head2 Catching exception

    try {
        NACL::C::VserverNfs->create(%opts);
    } catch NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded with {
        my $exception = shift;
        
        # Getting the error text is the same as for other exceptions
        $Log->comment('Error is: ' . $exception->text());
        
        # Getting the response object, like any other component-level exception
        my $response_obj = $exception->get_response_object();
        
        # Getting the attributes special to this exception
        my $protocol = $exception->protocol();
        my $command_interface = $exception->command_interface();
    };

=cut

sub new {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => $pkg->_validate_spec(),

        # Options accepted by SUPER::new
        allow_extra => 1
    );
    my $self = $pkg->SUPER::new(%opts);
    $Log->exit() if $may_exit;
    return $self;
}

=head2 add_allowed_protocols

    # An example showing the call failing because the allowed protcol not added,
    # then add the protocol and re-trying the call.
    LOOP: {
        use warnings;
        try {
            NACL::C::VserverNfs->create(command_interface => $ci, %other_opts);
        } catch NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded with {
            my $exception   = shift;
            my $license_obj = $exception->add_allowed_protocols();
            no warnings;
            redo LOOP;
        };
    }

This method can be used to add the missing protocol for which the
exception was thrown. The C<command_interface> , C<allowed_protocols> and 
C<vserver> are extracted automatically from within the exception object and invoke 
L<< NACL::STask::SystemLicense->add|lib-NACL-STask-Vserver-pm/add_to_array_attribute >>

This method returns nothing

=cut

sub add_allowed_protocols {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;
    my $protocol          = $self->protocol();
    my $allowed_protocols = [$protocol];
    NACL::STask::Vserver->add_to_array_attribute(
        command_interface => $self->command_interface(),
        'attribute'       => 'allowed-protocols',
        'list'            => $allowed_protocols,
        vserver           => $self->vserver(),
    );
    $Log->exit() if $may_exit;
}

=head1 FOR DEVELOPERS

This exception introduces new mandatory arguments to the calls to
C<convert> and C<convert_and_throw>.

    my $licensed_exception = NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded->convert(
        exception         => $exception_obj,
        feature           => $feature_name,
        vserver           => $vserver,
        command_interface => $ci
    );
    
    NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded->convert_and_throw(
        exception         => $exception_obj,
        feature           => $feature_name,
        vserver           => $vserver,
        command_interface => $ci
    );

=cut

sub convert {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %validate_spec = $pkg->_validate_spec();
    my %opts          = validate_with(
        params => \@args,
        spec   => \%validate_spec,

        # These are the options for SUPER::convert
        allow_extra => 1
    );
    my %attributes;
    $pkg->_hash_move(
        source => \%opts,
        target => \%attributes,
        move   => [ keys %validate_spec ]
    );
    my $obj = $pkg->SUPER::convert(%opts);
    while ( my ( $key, $value ) = each %attributes ) {
        $obj->$key($value);
    }
    $Log->exit() if $may_exit;
    return $obj;
}

sub _validate_spec {
    my %hash = (
        protocol          => { type => SCALAR },
        vserver           => { type => SCALAR },
        command_interface => _ontap_ci_validate_spec()
    );
    return wantarray ? %hash : {%hash};
}

1;
