#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

=head1 NAME

NACL::C::Exceptions::Client::Mount::FailedToMount

=head1 DESCRIPTION

This exception is thrown when a client failed to mount a volume.

Has gather_diagnostics() defined. If gather_diagnostics is called, then
diagnostic commands will be executed to provide more info for triage

=head1 EXAMPLE

    use NACL::C::Client
    use NACL::C::Client::Mount
    use NACL::C::Exceptions::Client::Mount::FailedToMount qw(:try);

    try {
        NACL::C::Client::Mount->create(
            command_interface => $someClientObj,
            server => "some.ip.address",
            path => "some_junction_path",
        );
    } catch NACL::C::Exceptions::Client::Mount::FailedToMount with {
        my $exception = shift;
        $exception->gather_diagnostics();

        # Propagate the exception further
        $exception->throw();
    };

=head1 This implementation of gather_diagnostics()

In this implementation of gather_diagnostics(), the diagnostics it gathers are:
- rpcinfo on the client that is attempting the mount
- whether the server is pingable from the client
- the mounts already on the client

=cut

use strict;
use warnings;

package NACL::C::Exceptions::Client::Mount::FailedToMount;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::TimeoutException
);

use Class::MethodMaker [
    hash => 'mount_args'
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);

sub _is_linux {
    my $self = shift;
    my %mount_args = $self->mount_args();
    my $client = $mount_args{command_interface}->{version_manager}->{hostobj};

    if ($client->hosttype() =~ /unix/) {
        return 1;
    }
    else {
        return 0;
    }
}

# execute a diagnostic command on the client.
# For ANY exception, print out the info, but do NOT
# let the exception propagate up the call stack.
# We print and ignore the exception because the exception
# was generated by diagnostics, NOT by the actual code
# that is being tested.
sub _exec_client_cmd {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my %mount_args = $self->mount_args();
    my $client_obj = $mount_args{command_interface};
    try {

        # Execute the given command on a NACL::C::Client obj
        $client_obj->execute( command => $args{command} );

    } catch NATE::BaseException with {

        # Print out the exception, then don't do anything else
        my $exception_obj = shift;
        $Log->comment( $exception_obj->stringify() );

    };

    $Log->exit() if $may_exit;
}

# execute a diagnostic command on the node or server.
# For ANY exception, print out the info, but do NOT
# let the exception propagate up the call stack.
# We print and ignore the exception because the exception
# was generated by diagnostics, NOT by the actual code
# that is being tested.
sub _exec_node_cmd {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    my %mount_args = $self->mount_args();
    my $command ;
    if(! defined $args{command}) {
       $Log->warn("Command for Node/server is not passed. " .
                   "Skipping collecting diag info for node/server.");
       return ;
    } else {
       $command = $args{command};
    }
    my $server_ci ;
    if(! defined $mount_args{server_command_interface}) {
       $Log->warn("Command Interface for Node/server is not passed. " .
                   "Skipping collecting diag info for node/server.");
       return ;
    } else {
       $server_ci = $mount_args{server_command_interface};
    }
    my $server_apiset = $server_ci->apiset();
    try {
        # Execute the given command on a node or server obj
        $server_apiset->$command();
    } catch NATE::BaseException with {
        # Print out the exception, then don't do anything else
        my $exception_obj = shift;
        $Log->comment( $exception_obj->stringify() );
    };

    $Log->exit() if $may_exit;
}

# gather_diagnostics should print out commands that help
# a developer triage the source of the mount failure
# from the Client that attempted the Mount.
sub gather_diagnostics {
    my $self = shift;

    $Log->enter() if $may_enter;

    my %mount_args = $self->mount_args();
    my $client = $mount_args{command_interface};
    my $server = $mount_args{server};
    # Perform diagnostic commands
    if ($client->is_windows()) {
        $self->_gather_diag_info_on_time();
        $self->_gather_diag_info_on_user();
        $self->_gather_diag_info_on_ip_hostname(%mount_args);
        $self->_gather_diag_info_on_shares(%mount_args);
        $self->_gather_diag_info_on_ceredentials();
	$self->_gather_diag_info_on_filers();
    }
    else {
        $self->_exec_client_cmd(command => "sudo ping $server -c 5");
        $self->_exec_client_cmd(command => "sudo /usr/sbin/rpcinfo -p $server");
        $self->_exec_client_cmd(command => "sudo /usr/sbin/showmount -e $server");
    }

    # NOTE: should have more diagnostic commands later on

    $Log->exit() if $may_exit;
}

sub _gather_diag_info_on_time {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $Log->comment("Current date and time on the Windows Client : ");
    $self->_exec_client_cmd(command => "echo %DATE% %TIME%");
    $Log->comment('Current date and time on the Filer/Server :');
    $self->_exec_node_cmd(command => "date");
    $Log->exit() if $may_exit;
}

sub _gather_diag_info_on_user {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $Log->comment("Logged in User on Client: " );
    $self->_exec_client_cmd(command => "echo %USERNAME%");
    $Log->comment("Logged in UserDomain on Client: ");
    $self->_exec_client_cmd(command => "echo %USERDOMAIN%");
    $Log->exit() if $may_exit;
}

sub _gather_diag_info_on_ip_hostname {
    $Log->enter() if $may_enter;
    my ($self, %mount_args) = @_;
    my $server = $mount_args{server} // 'dummy';
    my $server_ip = ($server =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) ? $1 : 'dummy';
    $Log->comment("Trying to ping server ip on the Client ");
    $self->_exec_client_cmd(command => "ping $server_ip");

    $Log->exit() if $may_exit;
}

sub _gather_diag_info_on_shares {
    $Log->enter() if $may_enter;
    my ($self, %mount_args) = @_;
    my $server = $mount_args{server} ;
    my $server_ip = ($server =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) ? $1 : '';
    if($server_ip){
       $Log->comment("Trying to view shares ");
       $self->_exec_client_cmd(command => "net view \\$server_ip") ;
       $Log->exit() if $may_exit;
    }
}

sub _gather_diag_info_on_ceredentials {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $Log->comment("Trying to view stored credentials on the Client");
    $self->_exec_client_cmd(command => 'cmdkey /list');
    $Log->exit() if $may_exit;
}

sub _gather_diag_info_on_filers {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $Log->comment("Trying to view NFS , SMB and name mapping details on the node");
    $self->_exec_node_cmd(command => "vserver_cifs_share_show");
    $self->_exec_node_cmd(command => "vserver_nfs_show");
    $Log->exit() if $may_exit;
}
1;
