#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary The base class for all component-level exception
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Component

=head1 DESCRIPTION

This is an abstract base class for all component-level exceptions.

=cut

package NACL::C::Exceptions::Component;

use base 'NACL::APISet::Exceptions::ResponseException';

use NACL::ComponentUtils qw(_hash_move _response_exception_validate_spec);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with);

=head1 Methods for developers

=head2 _new_with_validate_spec

    # In NACL::C::Exceptions::Foo::Bar
    sub new {
        $Log->enter() if $may_enter;
        my ($pkg, @args) = @_;
        my $self = $pkg->_new_with_validate_spec(@args);
        $Log->exit() if $may_exit;
        return $self;
    }
    
Certain component-level exceptions implement a C<_validate_spec> method
which defines the validation spec of the attributes specific to that
exception. This can be used to build the constructor - the options
accepted by the constructor of that exception should satisfy the
validation spec as well as that of its super class. This method
is a helper to achieve this.

=cut

sub _new_with_validate_spec {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => $pkg->_validate_spec(),

        # Options accepted by SUPER::new
        allow_extra => 1
    );
    my $self = $pkg->SUPER::new(%opts);
    $Log->exit() if $may_exit;
    return $self;
}

=head2 _convert_with_validate_spec

    # In NACL::C::Exceptions::Foo::Bar
    sub convert {
        $Log->enter() if $may_enter;
        my ($pkg, @args) = @_;
        my $obj = $pkg->_convert_with_validate_spec(@args);
        $Log->exit() if $may_exit;
        return $obj;
    }
    
Certain component-level exceptions implement a C<_validate_spec> method
which defines the validation spec of the attributes specific to that
exception. This can be used to build the C<convert> method - the options
accepted by it should satisfy the validation spec as well as that of its
super class. This method is a helper to achieve this.

=cut

sub _convert_with_validate_spec {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %validate_spec = $pkg->_validate_spec();
    my %opts          = validate_with(
        params => \@args,
        spec   => \%validate_spec,

        # These are the options for SUPER::convert
        allow_extra => 1
    );
    my %attributes;
    $pkg->_hash_move(
        source => \%opts,
        target => \%attributes,
        move   => [keys %validate_spec]
    );
    my $obj = $pkg->SUPER::convert(%opts);
    while (my ($key, $value) = each %attributes) {
        $obj->$key($value);
    }
    $Log->exit() if $may_exit;
    return $obj;
}

1;
