#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary A component-level exception for when multiple operations fail
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::MultipleCommandFailed

=head1 DESCRIPTION

This type of exception is thrown when a single component method results
in multiple operations failing (this would typically be for "modify" and
"delete" methods that support operating on multiple elements at a time)

=cut

package NACL::C::Exceptions::MultipleCommandFailed;

use strict;
use warnings;
use Params::Validate qw(validate_with SCALAR ARRAYREF HASHREF OBJECT);

# Turn Params::Validate failures into NATE::Exceptions::Argument
BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            NATE::Exceptions::Argument->throw(shift);
        }
    );
}

use base qw(
    NACL::APISet::Exceptions::MultipleCommandFailedException
    NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use Scalar::Util qw(blessed);

=head1 ATTRIBUTES

The attributes listed below should be accessed using the accessor methods
(which are of the same name as the attribute)

In addition to the attributes listed below, the functionality provided
by ResponseException is also present here. However, it is suggested
that the details of the individual errors be accessed using the accessor
methods described below.

(For reference:

The C<text> field contains all the error messages combined into a single
string.

The C<get_response_object> method will return the first response object
in the list of errors.
)

=head2 all_failures

This is an array of objects, containing details of each failure. This can be
accessed in scalar (returns an array-reference) or list context (returns
an array):

    my $all_failures_arrayref = $exception->all_failures();

    my @all_failures = $exception->all_failures();

Each object
contains the following attributes:

=head3 text

The error text.

=head3 primary_keys

This is a hash containing the primary keys that identify the element
for which the operation failed.

Being a hash field, this can be accessed in either scalar context (returns
a hash-reference) or in list context (returns a hash):

    my $primary_keys_hashref = $failure_obj->primary_keys();

    my %primary_keys = $failure_obj->primary_keys();

=head3 response_object

The response object corresponding to the failure.

=head3 error_code

The error-code related to the failure. Note that this field is populated
only for ZAPI.

=cut

use Class::MethodMaker [
    array => 'all_failures',
];

=head1 EXAMPLES

=head2 Throwing exception

When throwing an exception of this type, the C<all_failures> argument
is mandatory. This can be passed as an array-reference of hash-references
or as an array-reference of objects of type C<_SingleCommandFailed>.

=head3 Sending an array-reference of hash-references

    my @all_failures = (
        {
            text => 'Some error',
            response_object => $resp_obj1,
            primary_keys    => { vserver => 'vs1' }
        },
        {
            text => 'Another error',
            response_object => $resp_obj2,
            primary_keys    => { vserver => 'vs2' }
        },
    );
    NACL::C::Exceptions::MultipleCommandFailed->throw(
        all_failures => @all_failures);

=head4 Sending an array-reference of _SingleCommandFailed object

    my @all_failures = (
        _SingleCommandFailed->new(
            text            => 'Some error',
            response_object => $resp_obj1,
            primary_keys    => { vserver => 'vs1' }
        ),
        _SingleCommandFailed->new(
            text            => 'Another error',
            response_object => $resp_obj2,
            primary_keys    => { vserver => 'vs2' }
        ),
    );
    NACL::C::Exceptions::MultipleCommandFailed->throw(
        all_failures => @all_failures);

=head2 Catching exception

    try {
        NACL::C::Vserver->modify(
            command_interface => $node,
            vserver           => '*',
            'aggr-list'       => ['-']
        );
    } catch NACL::C::Exceptions::MultipleCommandFailed with {
        my $exception    = shift;
        my @all_failures = $exception->all_failures();
        my $is_zapi      = $exception->is_zapi();

        foreach my $failure_obj (@all_failures) {
            $Log->comment("Error is:\n" . $failure_obj->text());
            my %primary_keys = $failure_obj->primary_keys();
            $Log->comment("Element on which failure occurred:\n"
                . Dumper(\%primary_keys));
            $Log->comment("Response object:\n"
                . Dumper ($failure_obj->response_object()));
            if ($is_zapi) {
                $Log->comment('Error code is ' . $failure_obj->error_code());
            }
        }
    };

=cut

sub new {
    $Log->enter();
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { all_failures => { type => ARRAYREF } },
        allow_extra => 1
    );

    my %spec = (
        text => { type => SCALAR },
        primary_keys => { type => HASHREF },
        response_object => { type => OBJECT, isa => 'NACL::APISet::Response' },
        error_code      => { type => SCALAR, optional => 1 },
    );

    my @failure_objs;
    foreach my $failure_hash (@{$opts{all_failures}}) {
        if (blessed ($failure_hash)) {
            if ($failure_hash->isa('_SingleCommandFailed')) {
                push @failure_objs, $failure_hash;
            } else {
                $Log->exit();
                NATE::Exceptions::Argument->throw('All of the values '
                    . "provided in the 'all_failures' argument to "
                    . 'MultipleCommandFailed->new should be either '
                    . 'hash-references or objects of type  '
                    . "'_SingleCommandFailed' but one of the values provided "
                    . "was of type '"
                    . ref ($failure_hash)
                    . "'\n");
            }
        } else {
            try {
                validate_with(
                    params => $failure_hash,
                    spec   => \%spec
                );
                push @failure_objs, _SingleCommandFailed->new($failure_hash);
            } catch NATE::Exceptions::Argument with {
                my $exception = shift;
                $exception->set_text('One of the values passed through the '
                    . "'all_failures' argument was invalid:\n"
                    . $exception->text()
                );
                $Log->exit();
                $exception->throw();
            };
        }
    }

    if (!$opts{text} || !$opts{error_messages}) {
        my @text_array;
        foreach my $obj (@failure_objs) {
            push @text_array, $obj->text();
        }
        $opts{error_messages} ||= \@text_array;
        $opts{text} ||= join "\n", @text_array;
    }

    $opts{response_object} ||= $failure_objs[0]->response_object();

    my $self = $pkg->SUPER::new(%opts, all_failures => \@failure_objs);
    $Log->exit();
    return $self;
}

sub get_SingleCommandFailed {
    return _SingleCommandFailed->new;
}

1;

package _SingleCommandFailed;

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => 'text',
    hash   => 'primary_keys',
    scalar => [
        { -type => 'NACL::APISet::Response' },
        'response_object',
    ],
    scalar => 'error_code'
];

1;
