# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkInterface/PortHealthStatusDegraded.pm#1 $
#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for NetworkInterface
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded

=head1 DESCRIPTION

This exception is thrown when we attempt to create a network interface
on the port of which health status is degraded

Has gather_diagnostics() defined. If gather_diagnostics is called, then
diagnostic commands will be executed to provide more info for triaging.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded qw(:try);

   try {
       NACL::C::NetworkInterface->create(
           command_interface  => $Node,
	   node              => $node,
           lif               => $interface_name,
           home-port         => $home_port,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded with {
        # Port Health Status is degraded.
        my $exception = shift;
        my %port_args = {};
        $port_args{'command_interface'} =  $opts{'command_interface'};
        $port_args{'home_port'} = $opts{'home-port'};
        $exception->port_args(%port_args);
        $exception->gather_diagnostics();
   };

=head1 This implementation of gather_diagnostics()

In this implementation of gather_diagnostics(), the diagnostics it gathers are:
- Reachability of port
- Reason for degrade of port

=cut

use strict;
use warnings;

package NACL::C::Exceptions::NetworkInterface::PortHealthStatusDegraded;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);
use Class::MethodMaker [
    hash => [ 'port_args', ],
];

# gather_diagnostics should print out commands that help
# a developer triage the source of the lif creation failure due to degraded port
sub gather_diagnostics {
    $Log->enter() if $may_enter;
    my ($self) = @_;

    $Log->comment("#"x40 . " Diag Info for reachability of port " . "#"x40);
    $self->_gather_diag_info_for_reachability_port();
    $Log->comment("#"x40 . " Finished Gathering Diag Info " . "#"x36);
    $Log->comment("  \n\n  ");
    $Log->comment("#"x40 . " Diag Info for Reason of degrading of port " . "#"x40);
    $self->_gather_diag_info_for_reason_of_degrade();
    $Log->comment("#"x40 . " Finished Gathering Diag Info " . "#"x36);

    $Log->exit() if $may_exit;
}


sub _gather_diag_info_for_reachability_port {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my %port_args = $self->port_args();
    if( defined $port_args{command_interface} && defined $port_args{home_port} ){
        my $apiset = $port_args{command_interface}->apiset;
        my $node_name = $port_args{command_interface}->name;
        $apiset->network_port_broadcast_domain_reachability_show(port => "$node_name".":" .$port_args{home_port});
    }
    else {
        $Log->warn("Either Command Interface or Port Name is not passed. " .
                   "Skipping to collect diag info for reachability of port.");
    }
    $Log->exit() if $may_exit;
}


sub _gather_diag_info_for_reason_of_degrade {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my %port_args = $self->port_args();
    if( defined $port_args{command_interface} && defined $port_args{home_port} ){
        my $apiset = $port_args{command_interface}->apiset;
        $apiset->network_port_show(port => $port_args{home_port});
    }
    else {
        $Log->warn("Either Command Interface or Port Name is not passed. " .
                   "Skipping to collect diag info for reachability of port.");
    }
    $Log->exit() if $may_exit;
}

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Cannot host LIF "%s" on port "%s:%s" because the port's health status is degraded/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
