#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Fiji

=head1 DESCRIPTION

C<NACL::C::Fiji> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Fiji.  A related class is
L<NACL::CS::Fiji|lib-NACL-CS-Fiji-pm>, which represents
the past state of an ONTAP Fiji.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 slot

The name of the slot where fiji has been set,Its optional for all method except clear.

=head2 node

=cut

package NACL::C::Fiji;

use strict;
use warnings;

use NACL::CS::Fiji;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR UNDEF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $Fiji = NACL::C::Fiji->new(
                                   command_interface => $ci,
                                   slot => $slot,
                                  );

Return a new Fiji component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'slot', ];

=head2 find

    my $Fiji  = NACL::C::Fiji->find(command_interface => $ci, ...);
    my @Fijis = NACL::C::Fiji->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Fiji_state = $NACL::C::Fiji->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Fiji would return
a NACL::CS::Fiji object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 fiji_filter

    $fiji_obj = NACL::C::Fiji->fiji_filter(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to fiji filter.Returns the fiji object that contains the
slot number at which fiji has been set.

Uses a 7Mode/Nodescope CLI APISet.

Invokes 'fiji filter' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filter' => $filter >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : filter name ( programmatic name : filter )

=item C<< 'node' => $node >>

Applicable for 7Mode CLI.

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub fiji_filter {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@args,
        additional_spec     => { _optional_scalars(qw(filter)), },
        ignore_primary_keys => 1,
    );
    my $obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_fiji_filter_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub fiji_filter

sub _fiji_filter_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw(node filter)];
    push @$allowed_7mode_options, 'nodescope-node-name';
    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode_args,
    );

    my $text = $apiset->fiji_filter(%mode_args)->get_parsed_output();
    $Log->exit() if $may_exit;
    return $pkg_or_obj->new(
        slot              => $text->[0]->{slot_number},
        command_interface => $command_interface
    );
} ## end sub _fiji_filter_7mode_cli

=head2 fiji_clear

    NACL::C::Fiji->fiji_clear(
                                command_interface => $ci,
                                ...);

                                 or

    $Fiji_object->fiji_clear(...);

(Class or instance method)
This method is used to disable fault.

Uses a 7Mode/Nodescope CLI APISet.

Invokes 'fiji clear' command for 7Mode/Nodescope CLI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'slot' => $slot >>

Applicable for 7Mode CLI. Possible values are either digits or 'all'.

Maps to :

7M CLI : slot name ( programmatic name : slot )

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub fiji_clear {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_fiji_clear_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub fiji_clear

sub _fiji_clear_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw(all slot)];
    push @$allowed_7mode_options, 'nodescope-node-name';
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%cmode_args,
    );

    $apiset->fiji_clear(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _fiji_clear_7mode_cli

=head2 fiji_nvfilter

    NACL::C::Fiji->fiji_nvfilter(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to disable nvram fault.it returns the fiji object
that contains slot number at which fiji has been set.

Uses a 7Mode/Nodescope CLI APISet.

Invokes 'fiji nvfilter' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filter' => $filter >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : filter name ( programmatic name : filter )

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub fiji_nvfilter {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@args,
        additional_spec     => { _optional_scalars(qw(filter)), },
        ignore_primary_keys => 1,
    );
    my $obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_fiji_nvfilter_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub fiji_nvfilter

sub _fiji_nvfilter_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_mode_options = [qw( filter )];
    push @$allowed_mode_options, 'nodescope-node-name';
    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_mode_options,
        target => \%mode_args,
    );

    my $text = $apiset->fiji_nvfilter(%mode_args)->get_parsed_output();
    $Log->exit() if $may_exit;
    return $pkg_or_obj->new(
        slot              => $text->[0]->{slot_number},
        command_interface => $command_interface
    );
} ## end sub _fiji_nvfilter_7mode_cli

=head2 fiji_nvclear

    NACL::C::Fiji->fiji_nvclear(
                                command_interface => $ci,
                                ...);

                                 or

    $Fiji_object->fiji_nvclear(...);

(Class or instance method)
This method is used to disable nvram fault.

Uses a 7Mode/Nodescope CLI APISet.

Invokes 'fiji nvclear' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< 'nodescope-node-name' => $node_name >>

(Optional)
When run in nodescope, the component method defaults to running on the "local"
node. This argument can be used to run the method on node other than "local".

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'all' => $all >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : all ( programmatic name : all )

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub fiji_nvclear {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_fiji_nvclear_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub fiji_nvclear

sub _fiji_nvclear_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_mode_options = [qw( all )];
    push @$allowed_mode_options, 'nodescope-node-name';
    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_mode_options,
        target => \%mode_args,
    );

    $apiset->fiji_nvclear(%mode_args);
    $Log->exit() if $may_exit;
} ## end sub _fiji_nvclear_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        slot => {
            type      => SCALAR | UNDEF,
            optional  => 1,
            callbacks => {
                slot => sub { $_[0] =~ /\d+|all/ }
            }
        }
    );
} ## end sub _primary_keys_validate_spec

# Private Subroutine - will make 'nodescope-node-name' optional for NACL::C::Fiji
sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { additional_spec => { type => HASHREF, default => {} } },
        allow_extra => 1
    );
    unless ( exists $opts{additional_spec}{'nodescope-node-name'} ) {
        my %additional_spec = %{ $opts{additional_spec} };
        $additional_spec{'nodescope-node-name'}
            = { type => SCALAR, optional => 1 };
        $opts{additional_spec} = \%additional_spec;
    } ## end unless ( exists $opts{additional_spec...})
    my %validated_opts = $pkg_or_obj->SUPER::_common_validate_with(%opts);

    $Log->exit() if $may_exit;
    return %validated_opts;
} ## end sub _common_validate_with

1;
