#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

=head1 NAME

NACL::C::HyperV::Network

=head1 DESCRIPTION

C<NACL::C::HyperV::Network> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-HyperV-pm>.  It is used to create,
delete or to manipulate the Network(Virtual Switch) on an HyperV Server.
A related class isL<NACL::CS::HyperV::Network|lib-NACL-CS-HyperV-Network-pm>,
which represents the Virtual Switch on the given HyperV Server.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 name

The name of the Network (virtual Switch)

=cut

package NACL::C::HyperV::Network;

use strict;
use warnings;
use Params::Validate qw(validate_with validate SCALAR BOOLEAN SCALARREF ARRAYREF HASHREF);
use NATE::BaseException qw(:try);
use NACL::APISet;
use NACL::C::HyperV;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use RPC::XML qw(RPC_BOOLEAN RPC_INT RPC_I4 RPC_DOUBLE RPC_STRING);

use base 'NACL::C::Component::HyperV';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $network_obj = NACL::C::HyperV::Network->new(command_interface=>$ci,
                                                    name => $name,
                                                  );

Return a new HyperV::Network component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'name',
];

=head2 create

  my $network_obj  = NACL::C::HyperV::Network->create(
                                                      command_interface => $ci,
                                                      name              => $network
                                              );

(Class method) Request addition of a new Network element, and  return a new
Network component object that refers to it.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< name => $string >>

(Required) Name of the Network to create.

=item C<< tags => $tags >>

(Optional) tags field of Network to create.
It is of type arrayref. E.g.
$tags = [$v1, $v2, ...],

=item C<< VIFs => $VIFs >>

(Optional) VIFs field of Network to create. VIF should be passed as an Opaqueref.
It is of type arrayref. E.g.
$VIFs = [$v1, $v2, ...],

=item C<< PIFs => $PIFs >>

(Optional) PIFs field of Network to create. PIF should be passed as an Opaqueref.
It is of type arrayref. E.g.
$PIFs = [$v1, $v2, ...],

=item C<< allowed_operations => $allowed_operations >>

(Optional) allowed_operations field of Network to create.
It is of type arrayref. E.g.
$allowed_operations = [$v1, $v2, ...],

=item C<< other_config => $other_config >>

(Optional) other_config field of Network to create.
It is of type hashref. E.g.
$other_config = {k1 => v1,k2 => v2, ..},

=item C<< blobs => $blobs >>

(Optional) blobs field of Network to create.
It is of type hashref. E.g.
$blob = {k1 => v1,k2 => v2, ..},

=item C<< uuid => $uuid >>

(Optional) uuid field of Network to create.

=item C<< bridge => $bridge >>

(Optional) bridge field of Network to create.

=item C<< name_description => $name_description >>

(Optional) name_description field of Network to create.

=item C<< mtu => $mtu >>

(optional) Size of the MTU to be set for the Network

=back

=back

=cut

sub create {
    $Log->enter();
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with( params => \@args,
        additional_spec => {
            mtu                => { type => SCALAR, default => 1500 },
            bridge             => { type => SCALAR, optional => 1 },
            name_description   => { type => SCALAR, optional => 1 },
            uuid               => { type => SCALAR, optional => 1 },
            other_config       => { type => HASHREF, default => {} },
            blobs              => { type => HASHREF, optional => 1 },
            tags               => { type => ARRAYREF, optional => 1 },
            VIFs_ref           => { type => ARRAYREF, optional => 1 },
            PIFs_ref           => { type => ARRAYREF, optional => 1 },
            allowed_operations => { type => ARRAYREF, optional => 1 },
        },
    );
    my $ret = $pkg->call_on_apiset( %opts,
                        choices => [
                            {
                              method => '_create_network_xen',
                              interface => 'XAPI',
                              set   => 'Xen',
                            },
                        ],
                   );

    $Log->exit();
    return $ret;
}

sub _create_network_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
    my $req = "network.create";
    my $record = $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [ qw(bridge name_description uuid other_config blobs
                       current_operations tags VIFs PIFs allowed_operations)
        ],
        map => {
                "mtu" => "MTU",
                "name" => "name_label",
        },
    );

    $record->{MTU} = RPC_STRING($record->{MTU});
    my $network_ref = $apiset->simple_request($req, $record);
    
    $Log->exit();
    return $pkg_or_obj->new(
        command_interface => $command_interface,
        name              => $opts{name},
    );
}

=head2 destroy

  NACL::C::HyperV::Network->destroy(command_interface=>$ci, name =>$network);

  OR

  $network_obj->destroy();

(Class or Instance method) Request deletion of a Network (Virtual Switch).
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< name => $name >>

(Required for class method) Name of the Network to destroy.

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub destroy {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args);

    $pkg_or_obj->call_on_apiset( %opts,
        choices => [
            {
              method => '_destroy_network_xen',
              interface => 'XAPI',
              set   => 'Xen',
            },
        ],
    );

    $Log->exit();
}

sub _destroy_network_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $network           = delete $opts{name};

    my $network_ref = $command_interface->_get_network_ref(
                                           name => $network,
                                           apiset  => $apiset,
                                       );

    my $req = "network.destroy";
    $apiset->simple_request($req, $network_ref);

    $Log->exit();
}

sub _primary_keys_validate_spec {
    return ( name   => { type => SCALAR }, );
} ## end sub _primary_keys_validate_spec

1;
