#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

=head1 NAME

NACL::C::HyperV::VIF

=head1 DESCRIPTION

C<NACL::C::HyperV::VIF> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>.  It is used to create,
delete or to manipulate the VIF on an HyperV Server.
A related class isL<NACL::CS::HyperV::VIF|lib-NACL-CS-HyperV-VIF-pm>,
which represents the VIF on the given HyperV Server.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 name

The name of the VIF.

=cut

package NACL::C::HyperV::VIF;

use strict;
use warnings;
use Params::Validate qw(validate_with validate SCALAR BOOLEAN SCALARREF ARRAYREF HASHREF);
use NATE::BaseException qw(:try);
use NACL::APISet;
use NACL::C::HyperV;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use RPC::XML qw(RPC_BOOLEAN RPC_INT RPC_I4 RPC_DOUBLE RPC_STRING);
use base 'NACL::C::Component::HyperV';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $network_obj = NACL::C::HyperV::VIF->new(command_interface=>$ci,
                                                    device  => $device,
                                                    vm_name => $vm_name,
                                                  );

Return a new HyperV::VIF component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'name',
];

=head2 create

  my $network_obj  = NACL::C::HyperV::VIF->create(
                                                      command_interface => $ci,
                                                      device            => $device
                                                      network           => $network
                                                      vm_name           => $vm_name
                                                 );

(Class method) Request addition of a new VIF element, and return a new
VIF component object that refers to it.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< device => $string >>

(Required) Name of the device field of VIF to create.

=item C<< mtu => $mtu >>

(optional) Size of the MTU to be set for the VIF

=item C<< MAC_autogenerated => $boolean >>

(Optional) MAC_autogenerated field of VIF to create.

=item C<< qos_algorithm_type => $qos_algorithm_type >>

(Optional) qos_algorithm_type field of VIF to create.

=item C<< qos_algorithm_params => $qos_algorithm_params >>

(Optional) qos_algorithm_params field of VIF to create.
It is of type hashref. E.g.
$qos_algorithm_params = {k1 => v1,k2 => v2, ..},

=item C<< allowed_operations => $allowed_operations >>

(Optional) allowed_operations field of VIF to create.
It is of type arrayref. E.g.
$allowed_operations = [$v1, $v2, ...],

=item C<< other_config => $other_config >>

(Optional) other_config field of VIF to create.
It is of type hashref. E.g.
$other_config = {k1 => v1,k2 => v2, ..},

=item C<< mac => $mac >>

(Optional) MAC field of VIF to create.

=item C<< network => $network >>

(Optional) network field of VIF to create.
It can either be a network name or network ref

=item C<< vm => $vm >>

(Optional) vm field of VIF to create.
It can either be a VM name or VM ref
    
=item C<< uuid => $uuid >>

(Optional) uuid field of VIF to create.

=back

=back

=cut

sub create {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( params => \@args,
        additional_spec => {
            network               => { type => SCALAR},
            device                => { type => SCALAR},
            vm_name               => { type => SCALAR},
            MAC_autogenerated     => { type => BOOLEAN, default => 1 },
            mtu                   => { type => SCALAR, default => 1500 },
            mac                   => { type => SCALAR, default => '' },
            uuid                  => { type => SCALAR, optional => 1 },
            qos_algorithm_type    => { type => SCALAR, default => '' },
            other_config          => { type => HASHREF, default => {} },
            qos_algorithm_params  => { type => HASHREF, default => {} },
            allowed_operations    => { type => ARRAYREF, optional => 1 },
        },
        ignore_primary_keys       => 1,
    );
    my $ret = $pkg_or_obj->call_on_apiset( %opts,
                        choices => [
                            {
                              method => '_create_vif_xen',
                              interface => 'XAPI',
                              set   => 'Xen',
                            },
                        ],
                   );

    $Log->exit();
    return $ret;
}

sub _create_vif_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    
    my $req = "VIF.create";
    my $record = $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [ qw(qos_algorithm_type uuid qos_algorithm_type
                       other_config qos_algorithm_params allowed_operations
                       device network MAC_autogenerated)
        ],
        map => {
                "mtu"     => "MTU",
                "mac"     => "MAC",
                "vm_name" => "VM",
        },
    );
    
    # Xen API expect MTU and device field to be string (strange i know)
    # using RCP::XML to encode these as string
    $record->{MTU} = RPC_STRING($record->{MTU});
    $record->{device} = RPC_STRING($record->{device});
    
    # VM and Network params are accepted as Refs
    if ($record->{VM} !~ /OpaqueRef:/) {
        $record->{VM} = $command_interface->_get_vm_ref(vm_name => $record->{VM}, apiset  => $apiset);
    }
    if ($record->{network} !~ /OpaqueRef:/) {
        $record->{network} = $command_interface->_get_network_ref(name => $record->{network}, apiset  => $apiset);
    }
    
    my $resp = $apiset->simple_request($req, $record);
    my $vif_ref = $resp->get_parsed_output()->[0];
    
    $Log->exit();
    return $pkg_or_obj->new(
        command_interface => $command_interface,
        name              => $vif_ref,
    );
}

=head2 set_mtu

  NACL::C::HyperV::VIF->set_mtu(command_interface=>$ci, name =>$vif, mtu => $mtu);

  OR

  $vif_obj->set_mtu(mtu => $mtu);

(Class or Instance method) Request to plug VIF to a VM.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< mtu => $mtu >>

(Required for class method) Value of MTU to set for the VIF.

=item C<< name => $name >>

(Required for class method) Name of the VIF.

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub set_mtu {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( 
            params          => \@args,
            additional_spec => {
                name => { type => SCALAR},
                mtu  => { type => SCALAR},
            },
    );

    $pkg_or_obj->call_on_apiset( %opts,
        choices => [
            {
              method => '_set_mtu_vif_xen',
              interface => 'XAPI',
              set   => 'Xen',
            },
        ],
    );

    $Log->exit();
}

sub _set_mtu_vif_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vif_ref           = delete $opts{name};
    
    my $req = "VIF.set_other_config";
    $apiset->simple_request($req, $vif_ref, {MTU => RPC_STRING($opts{mtu})});

    $Log->exit();
}

=head2 plug

  NACL::C::HyperV::VIF->plug(command_interface=>$ci, name =>$vif);

  OR

  $vif_obj->plug();

(Class or Instance method) Request to plug VIF to a VM.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< name => $name >>

(Required for class method) Name of the VIF to plug to a VM.

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub plug {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( 
            params          => \@args,
            additional_spec => {
                name => { type => SCALAR},
            },
    );

    $pkg_or_obj->call_on_apiset( %opts,
        choices => [
            {
              method => '_plug_vif_xen',
              interface => 'XAPI',
              set   => 'Xen',
            },
        ],
    );

    $Log->exit();
}

sub _plug_vif_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vif_ref           = delete $opts{name};
    
    my $req = "VIF.plug";
    $apiset->simple_request($req, $vif_ref);

    $Log->exit();
}

=head2 unplug

  NACL::C::HyperV::VIF->destroy(command_interface=>$ci, name =>$vif);

  OR

  $network_obj->unplug();

(Class or Instance method) Request to unplug of a VIF from a VM.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< name => $name >>

(Required for class method) Name of the VIF to unplug from a VM.

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub unplug {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( 
            params          => \@args,
            additional_spec => {
                name => { type => SCALAR},
            },
    );

    $pkg_or_obj->call_on_apiset( %opts,
        choices => [
            {
              method => '_unplug_vif_xen',
              interface => 'XAPI',
              set   => 'Xen',
            },
        ],
    );

    $Log->exit();
}

sub _unplug_vif_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vif_ref           = delete $opts{name};
    
    my $req = "VIF.unplug";
    $apiset->simple_request($req, $vif_ref);

    $Log->exit();
}

=head2 destroy

  NACL::C::HyperV::VIF->destroy(command_interface=>$ci, name =>$vif);

  OR

  $network_obj->destroy();

(Class or Instance method) Request deletion of a VIF.
Applicable for XAPI Xen.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< name => $name >>

(Required for class method) Name of the VIF to destroy.

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub destroy {
    $Log->enter();
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with( 
            params          => \@args,
            additional_spec => {
                name => { type => SCALAR},
            },
    );

    $pkg_or_obj->call_on_apiset( %opts,
        choices => [
            {
              method => '_destroy_vif_xen',
              interface => 'XAPI',
              set   => 'Xen',
            },
        ],
    );

    $Log->exit();
}

sub _destroy_vif_xen {
    $Log->enter();
    my ($pkg_or_obj, @opts) = @_;
    
    my %opts              = $pkg_or_obj->_validate_backend_opts(@opts);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vif_ref           = delete $opts{name};

    my $req = "VIF.destroy";
    $apiset->simple_request($req, $vif_ref);

    $Log->exit();
}

sub _primary_keys_validate_spec {
    return ( name   => { type => SCALAR }, );
} ## end sub _primary_keys_validate_spec

1;
