#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Ifconfig Component Module
## @author dheeraj.k@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Ifconfig

=head1 DESCRIPTION

C<NACL::C::Ifconfig> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP System Ifconfig.  A related class is
L<NACL::CS::Ifconfig|lib-NACL-CS-Ifconfig-pm>, which represents
the past state of an ONTAP Ifconfig.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 interface

The name of the interface, used to identify it uniquely.

=cut

package NACL::C::Ifconfig;
use strict;
use warnings;
use NACL::CS::Ifconfig;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR);



=head1 METHODS

=head2 new

  my $ifconfig_interface = NACL::C::Ifconfig->new(command_interface=>$ci,
                                   interface =>$interface,
								  );

Return a new Ifconfig component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'interface',
];

=head2 find

    my $Interface  = NACL::C::Interface->find(command_interface => $ci, ...);
	my @Interfaces = NACL::C::Interface->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $Interface_state = $NACL::C::Interface->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Interface would return
a NACL::CS::Interface object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut


=head2 ifconfig

  NACL::C::Ifconfig->ifconfig(command_interface=>$ci,
                        interface => "e0d" , address => "10.72.221.12");

(Class method) Request configuration of interfaces. Doesn't return any
value. 

Uses a 7Mode/Nodescope/Systemshell CLI APISet.

Invokes 'ifconfig' command for 7Mode/Nodescope/Systemshell CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'alias' => $alias >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : alias <value> ( programmatic name : alias )

=item C<< 'unalias' => $unalias >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : -alias <value> ( programmatic name : unalias )

=item C<< 'no_ddns' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : no_ddns ( programmatic name : no_ddns )

=item C<< 'all' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : -a ( programmatic name : all)

=item C<< 'interface' => $interface >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : interface name ( programmatic name : interface)

=item C<< 'address' => $address >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : address ( programmatic name : address)

=item C<< 'netmask' => $netmask >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : netmask <value> ( programmatic name : netmask)

=item C<< 'mtusize' => $mtusize >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : mtusize <value> ( programmatic name : mtusize)

=item C<< 'mediatype' => $mediatype >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : mediatype <value> ( programmatic name : mediatype)

=item C<< 'flowcontrol' => $flowcontrol >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : flowcontrol <value> ( programmatic name : flowcontrol)

=item C<< 'trusted' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : trusted ( programmatic name : trusted )

=item C<< 'untrusted' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : untrusted ( programmatic name : untrusted )

=item C<< 'wins' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : wins ( programmatic name : wins )

=item C<< 'no_wins' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : no_wins ( programmatic name : no_wins )

=item C<< 'nfo' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : nfo ( programmatic name : nfo )

=item C<< 'no_nfo' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : no_nfo ( programmatic name : no_nfo )

=item C<< 'up' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : up ( programmatic name : up )

=item C<< 'down' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : down ( programmatic name : down )

=item C<< 'partner' => $partner >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : partner <value> ( programmatic name : partner)

=item C<< 'no_partner' => $no_partner >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : -partner ( programmatic name : -partner)

=item C<< 'up_private' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : up_private ( programmatic name : up_private )

=item C<< 'prefixlen' => $prefixlen >>

(Optional) Used for ipv6 address configuration.

Applicable for 7Mode CLI.

Maps to :

7M CLI : prefixlen <value> ( programmatic name : prefixlen)

=item C<< 'dad_attempts' => $dad_attempts >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : dad_attempts <value> ( programmatic name : dad_attempts)

=item C<< 'inet' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : inet ( programmatic name : inet )

=item C<< 'inet6' => $boolean >>

Applicable for 7Mode CLI.

Maps to :

7M CLI : inet6 ( programmatic name : inet6 )

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub ifconfig {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw(alias unalias no_ddns all address
                    netmask broadcast mtusize mediatype
                    flowcontrol trusted untrusted wins no_wins
                    nfo no_nfo up down partner no_partner
                    up_private prefixlen dad_attempts inet inet6)
            ),
        },
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_ifconfig_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope|Systemshell",
            },
        ],
    );
} ## end sub ifconfig

sub _ifconfig_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    ## Example: netmask => '64' is stored as prefixlen for ipv6.
    ## The below code is used only when netmask is used instead of prefixlen
    if(defined $opts{address} && $opts{address} =~ /:/) {
        $opts{prefixlen} = delete $opts{netmask} if(!$opts{prefixlen});
    }
	$pkg->_handle_boolean_options(
	    api_options     => \%opts,
	    boolean_options => [ qw(all no_ddns trusted untrusted wins
                                no_wins nfo no_nfo up down up_private inet inet6)],
	    is_7m_cli       => 1
	);

    $apiset->ifconfig(%opts);

} ## end sub _ifconfig_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (  interface => {type => SCALAR });
}
1;
