# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Job Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Job

=head1 DESCRIPTION

C<NACL::C::Job> is a derived class of

L<NACL::C::Component|lib-NACL-C-Component-pm>.  It represents access
to an ONTAP Job.  A related class is

L<NACL::CS::Job|lib-NACL-CS-Job-pm>, which represents
the past state of an ONTAP Job.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 id

=cut

package NACL::C::Job;

use strict;
use warnings;
use NACL::CS::Job;
use NACL::CS::JobShowByNode;
use NACL::CS::JobShowCompleted;
use NACL::ComponentUtils
    qw (_optional_scalars Dumper _retry_logic _retry_spec);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::C::Exceptions::Job::DoesNotExist();
use NACL::C::_Mixins::Job qw(wait_on_job);

=head1 METHODS

=head2 new

 my $Job = NACL::C::Job->new(command_interface=>$ci,id =>$id,);

Return a new Job component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'id',
    scalar => 'vserver',
];

=head2 find

    my $Job  = NACL::C::Job->find(command_interface => $ci, ...);
        my @Jobs = NACL::C::Job->find(command_interface => $ci, ...);

        Uses a CMode CLI or CMode ZAPI APISet.

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $Job_state = $NACL::C::Job->state();

    Uses a CMode CLI or CMode ZAPI APISet.

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Job would return a NACL::CS::Job
object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::Job->delete(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->delete(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes 'job-delete-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
    <job-info>
        <job-id>$value</job-id>
    </job-info>

=item C<< vserver => $vserver >>

(Optional)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:
    <job-info>
        <job-vserver>$value</job-vserver>
    </job-info>


=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'job delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::delete',
    );
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                api => 'job_delete');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-delete-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 softpause

   NACL::C::Job->softpause(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->softpause(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes 'job-soft-pause-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
    <job-info>
        <job-id>$value</job-id>
    </job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : vserver

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job softpause" command in ONTAP.

Applicable for CMode CLI.

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:
    <job-info>
        <job-vserver>$value</job-vserver>
    </job-info>


=item C<< "extended_query" => $string >>

 (Optional)
 Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub softpause {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'job softpause',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_softpause_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_softpause_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::softpause',
    );
    $Log->exit() if $may_exit;
} ## end sub softpause

sub _softpause_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                                      api => 'job_softpause');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub _softpause_cmode_cli

sub _softpause_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-soft-pause-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _softpause_cmode_zapi

=head2 pause

   NACL::C::Job->pause(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->pause(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes 'job-pause-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<job-info>
<job-id>$value</job-id>
</job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI :

<job-info>
<job-vserver>$value</job-vserver>
</job-info>

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job pause" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub pause {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params    => \@args,
        cli_cmd   => 'job pause',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_pause_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_pause_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::Job::pause',
    );
    $Log->exit() if $may_exit;
} ## end sub pause

sub _pause_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
         $pkg_or_obj->_base_backend_cmode_cli(@args,
                              api => 'job_pause');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub _pause_cmode_cli

sub _pause_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-pause-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist',
    );
    $Log->exit() if $may_exit;
} ## end sub _pause_cmode_zapi

=head2 expunge

   NACL::C::Job->expunge(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->expunge(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes 'job-expunge-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

<job-info>
<job-id>$value</job-id>
</job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<job-info>
<job-vserver>$value</job-vserver>
</job-info>

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job expunge" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub expunge {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params    => \@args,
        cli_cmd   => 'job expunge',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_expunge_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_expunge_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::expunge',
    );
    $Log->exit() if $may_exit;
} ## end sub expunge

sub _expunge_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                                api => 'job_expunge');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;

} ## end sub _expunge_cmode_cli

sub _expunge_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-expunge-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _expunge_cmode_zapi

=head2 stop

   NACL::C::Job->stop(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->stop(...);

(Class or instance method)

Uses a CMode CLI or CMode ZAPI APISet.

Invokes 'job-stop-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

<job-info>
<job-id>$value</job-id>
</job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<job-info>
<job-vserver>$value</job-vserver>
</job-info>

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd  => 'job stop',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_stop_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::stop',
    );
    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                      api => 'job_stop');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-stop-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_zapi

=head2 resume

   NACL::C::Job->resume(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->resume(...);

(Class or instance method)

Uses a CMode CLI or CMode ZAPI APISet.

Invokes 'job-resume-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

<job-info>
<job-id>$value</job-id>
</job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<job-info>
<job-vserver>$value</job-vserver>
</job-info>

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job resume" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resume {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'job resume',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resume_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_resume_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::resume',
    );
    $Log->exit() if $may_exit;
} ## end sub resume

sub _resume_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                api => 'job_resume');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_cli

sub _resume_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-resume-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_zapi

=head2 unclaim

   NACL::C::Job->unclaim(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->unclaim(...);

(Class or instance method)

Uses a CMode CLI or CMode ZAPI APISet.

Invokes 'job-unclaim-iter' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

<job-info>
<job-id>$value</job-id>
</job-info>

=item C<< vserver => $vserver >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<job-info>
<job-vserver>$value</job-vserver>
</job-info>

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job unclaim" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unclaim {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd  => 'job unclaim',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unclaim_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_unclaim_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::unclaim',
    );
    $Log->exit() if $may_exit;
} ## end sub unclaim

sub _unclaim_cmode_cli {
    my ($pkg_or_obj, @args) = @_;

    try {
        $pkg_or_obj->_base_backend_cmode_cli(@args,
                api => 'job_unclaim');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = $_[0];
        my $doesnot_exist = 'NACL::C::Exceptions::Job::DoesNotExist';
        $doesnot_exist->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub _unclaim_cmode_cli

sub _unclaim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'job-unclaim-iter',
        show_cmd => 'job show',
        map      => {
            'id'      => 'job-id',
            'vserver' => 'job-vserver'
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception => 'NACL::C::Exceptions::Job::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _unclaim_cmode_zapi

=head2 watch_progress

   NACL::C::Job->watch_progress(
                                command_interface => $ci,
                                 id => $id,
                                  ...);
                                 or

    $Job_object->watch_progress(...);

(Class or instance method)
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)

=item C<< interval => $string >>

=item C<< vserver => $vserver >>

(Required)
Applicable only for CMode CLI

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job watch-progress" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub watch_progress {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( interval )),},
        cli_cmd         => 'job watch-progress',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_watch_progress_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::watch_progress',
    );
} ## end sub watch_progress

sub _watch_progress_cmode_cli {
    my ($pkg_or_obj, %opts) = @_;
    my $apiset              = $opts{apiset};
    # NACL burt 505650, product burt 501814 : job watch-progress keeps
    # waiting even after the job is complete, finishing only when the
    # job becomes history.  To work around this, if job watch-progress
    # prints that the job is complete, break out of it.
    #
    # From ontap source, looks like non-"extended" jobs
    # may just say "Complete" instead of "Complete: \s+ [\d+]"

    my $conn = $apiset->get_connection();

    my $is_resync = 0;
    try {
        $pkg_or_obj->_base_backend_cmode_cli(%opts,
            api => 'job_watch_progress',
            "connectrec-match_table" => [
                "(?m:^|\\r)Complete" => sub {
                    $conn->send_intr() unless ($is_resync);
                    $is_resync = 1;
                    undef;
                },
            ],
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $ex = shift;
        $Log->debug(
            "job watch-progress failed with the reason: " . $ex->text());
        if ($ex->text !~ /Response exception\:\sInterrupted/) {
            $Log->debug("This is not an exception thrown because of "
                    . "intentionally interrupting the command");
            $ex->throw();
        }
    };

    # Resync only on sending interrupt character.
    $conn->resync() if ($is_resync);
} ## end sub _watch_progress_cmode_cli

=head2 kick

   NACL::C::Job->kick(
                                command_interface => $ci,
                                     id => $id,
                                  ...);
                                     or

    $Job_object->pause(...);

(Class or instance method)
Uses a CMode CLI or CMode ZAPI APISet.
Invokes 'job-kick' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< id => $id >>

(Required for class method, Not Applicable for instance method)
Maps to:
    CMode ZAPI: "job-id" of "job-kick"

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
iunderlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< '*' >>

This method takes all other options supported by
"job kick" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=cut

sub kick {
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'job kick',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_kick_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_kick_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::kick',
    );
} ## end sub kick

sub _kick_cmode_cli {
    my ($pkg_or_obj, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $id                = delete $opts{id};
    my $record            = NACL::CS::Job->fetch(
        command_interface => $opts{command_interface},
        filter            => {id => $id}
    );
    $opts{id} = $record->uuid();
    $pkg_or_obj->_base_backend_cmode_cli(%opts,  api => 'job_kick');
} ## end sub _kick_cmode_cli

sub _kick_cmode_zapi {
    my ($pkg_or_obj, %opts) = @_;
    my $command_interface = $opts{command_interface};
    my $id                = delete $opts{id};
    my $record            = NACL::CS::Job->fetch(
        command_interface => $opts{command_interface},
        filter            => {id => $id}
    );
    $opts{"job-uuid"} = $record->uuid();
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'job_kick',
    );
} ## end sub _kick_cmode_zapi

=head2 show_bynode

        my @jobshowbynode = NACL::C::Job->show_bynode(...);
                                or
        my @jobshowbynode = $Job_obj->show_bynode(...);

(Class or instance method)
This method is used to display a list of completed jobs.
It returns objects of type L<NACL::CS::JobShowByNode|lib-NACL-CS-JobShowByNode-pm>.

The options accepted are the same as those accepted by
L<fetch|lib-NACL-CS-ComponentState-pm/fetch>. (requested_fields, filter, allow_empty)

=cut

sub show_bynode {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    my @state_objs = NACL::CS::JobShowByNode->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_bynode

=head2 show_completed

        my @jobshowcompleted = NACL::C::Job->show_completed(...);
                                or
        my @jobshowcompleted = $Job_obj->show_completed(...);

(Class or instance method)
This method is used to display a list of completed jobs.
It returns objects of type L<NACL::CS::JobShowCompleted|lib-NACL-CS-JobShowCompleted-pm>.

The options accepted are the same as those accepted by
L<fetch|lib-NACL-CS-ComponentState-pm/fetch>. (requested_fields, filter, allow_empty)

=cut

sub show_completed {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    my @state_objs = NACL::CS::JobShowCompleted->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_completed

=head2 signal

    NACL::C::Job->signal(
        command_interface => $ci,
        'id' => $job->get_one_state_attribute('uuid'),
        ...
    );

(Class or instance method)
This method is used to signals a pending or running job.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'id' => $uuid >>

(Required)
This is the 'uuid' attribute of the job to be signalled
(despite the inconsistent name).

=item C<< 'val' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"job signal" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub signal {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my $pkg = $pkg_or_obj->get_package_name();
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( id val
                    )
            ),
        },
        ignore_primary_keys => 1,
        no_command_interface => (ref $pkg_or_obj) ? 1 : 0,
        api                 => 'job_signal',
    );
    if (ref $pkg_or_obj) {
	# populate command_interface manually
	$opts{command_interface} = $pkg_or_obj->command_interface();
	# 'id' argument accepts the job 'uuid' (not the 'id')
	$opts{id} = $pkg_or_obj->get_one_state_attribute('uuid');
    }
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_signal_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::signal',
    );

    $Log->exit() if $may_exit;
} ## end sub signal

sub _signal_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_signal');

    $Log->exit() if $may_exit;
} ## end sub _signal_cmode_cli

=head2 show_cluster

    my $show_cluster_obj = NACL::C::Job->show_cluster(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

    my @show_cluster_objs = NACL::C::Job->show_cluster(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_cluster_obj = $Job_obj->show_cluster();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::JobShowCluster|lib-NACL-CS-JobShowCluster-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub show_cluster {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::JobShowCluster;
    my @state_objs = NACL::CS::JobShowCluster->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_cluster

=head2 bulk_remove

    NACL::C::Job->bulk_remove(
                                command_interface => $ci,
                                'id' => $id,
                                ...);

(Class method)
This method is used to removes jobs for a host.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'id' => $id >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"job bulk-remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub bulk_remove {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( id
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'job_bulk_remove',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_bulk_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::bulk_remove',
    );

    $Log->exit() if $may_exit;
} ## end sub bulk_remove

sub _bulk_remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
   
    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_bulk_remove');

    $Log->exit() if $may_exit;
} ## end sub _bulk_remove_cmode_cli

=head2 start_jobthreads

    NACL::C::Job->start_jobthreads(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to starts the Job Manager processing threads.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"job start-jobthreads" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_jobthreads {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'job_start_jobthreads',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_jobthreads_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::start_jobthreads',
    );

    $Log->exit() if $may_exit;
} ## end sub start_jobthreads

sub _start_jobthreads_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_start_jobthreads');

    $Log->exit() if $may_exit;
} ## end sub _start_jobthreads_cmode_cli

=head2 dbg

    NACL::C::Job->dbg(
        command_interface => $ci,
        'id' => $job->get_one_state_attribute('uuid'),
        ...
    );

(Class or instance method)
This method is used to causes a job to dump its debug state.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'id' => $uuid >>

(Required)
This is the 'uuid' attribute of the job to be debugged
(despite the inconsistent name).

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"job dbg" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub dbg {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my $pkg = $pkg_or_obj->get_package_name();
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( id
                    )
            ),
        },
        ignore_primary_keys => 1,
        no_command_interface => (ref $pkg_or_obj) ? 1 : 0,
        api                 => 'job_dbg',
    );
    if (ref $pkg_or_obj) {
	# populate command_interface manually
	$opts{command_interface} = $pkg_or_obj->command_interface();
	# 'id' argument accepts the job 'uuid' (not the 'id')
	$opts{id} = $pkg_or_obj->get_one_state_attribute('uuid');
    }
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_dbg_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::dbg',
    );

    $Log->exit() if $may_exit;
} ## end sub dbg

sub _dbg_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_dbg');

    $Log->exit() if $may_exit;
} ## end sub _dbg_cmode_cli

=head2 do_test

    NACL::C::Job->do_test(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to initiates job testing..
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"job do-test" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub do_test {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'job_do_test',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_do_test_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Job::do_test',
    );

    $Log->exit() if $may_exit;
} ## end sub do_test

sub _do_test_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_do_test');

    $Log->exit() if $may_exit;
} ## end sub _do_test_cmode_cli

sub _primary_keys_validate_spec {
    return (
        id        => {type => SCALAR | OBJECT},
        'vserver' => {type => SCALAR | OBJECT, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

# overriden call_on_apiset to handle product burt 517892
# ONTAP complains that job is not found for an existing job id
# because of the timing issues. This is a rare case hit only when
# executing commands in a fast fashion using the automated scripts.
sub call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $retry    = delete $opts{nacl_retry_count};
    my $response = $pkg_or_obj->_retry_logic(
        method => 'SUPER::call_on_apiset',
        args   => \%opts,
        retry  => $retry,
        exception_type =>
            'NACL::APISet::Exceptions::NoMatchingEntriesException'
    );
    $Log->exit() if $may_exit;
    return $response;
}

# overriden _common_validate_with to accept nacl_retry_count as the valid
# option
sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

   # Add nacl_retry_count to the additional spec as an optional scalar
   # Default is to try 3 times. This is not applicable for all the
   # methods of this job component (say db, bulk_remove, etc.,) but we
   # are still doing it. So if a user tries to pass this option for the
   # commands which doesn't take job id in 'id', validation will pass as we
   # have overriden _common_validate_with and so this code is applicable
   # for all the methods of this component and its inherited classes
   # as well. Since passing this option has no other harmful effects, we
   # are doing it this way as this makes the implementation easy and it also
   # handles any new commands added in job directory which will take job id in
   # 'id' parameter automatically - merits have overcome the demerits!
    $opts{additional_spec}->{'nacl_retry_count'} = $pkg_or_obj->_retry_spec();
    my %args = $pkg_or_obj->SUPER::_common_validate_with(%opts);
    $Log->exit() if $may_exit;
    return %args;
}

sub _find_validate_spec {
    $Log->enter() if $may_enter;
    my $pkg_or_obj         = shift;
    my $find_validate_spec = $pkg_or_obj->SUPER::_find_validate_spec();
    $find_validate_spec->{nacl_retry_count} = $pkg_or_obj->_retry_spec;

    $Log->exit() if $may_exit;
    return $find_validate_spec;
} ## end sub _find_validate_spec

sub _invoke {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $method   = delete $opts{method};
    my @response = $pkg_or_obj->$method(%opts);
    $Log->exit() if $may_exit;
    return @response;
}

1;
