# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobScheduleInterval Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::JobScheduleInterval

=head1 DESCRIPTION

C<NACL::C::JobScheduleInterval> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP JobScheduleInterval.  A related class is
L<NACL::CS::JobScheduleInterval|lib-NACL-CS-JobScheduleInterval-pm>, which represents
the past state of an ONTAP JobScheduleInterval.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 name

=cut

package NACL::C::JobScheduleInterval;

use strict;
use warnings;

use NACL::CS::JobScheduleInterval;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NACL::C::Exceptions::JobScheduleInterval::DoesNotExist;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 new

  my $JobScheduleInterval = NACL::C::JobScheduleInterval->new(command_interface=>$ci,
                                   name =>$name,
                                  );

Return a new JobScheduleInterval component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'name',];

=head2 find

    my $JobScheduleInterval  = NACL::C::JobScheduleInterval->find(command_interface => $ci, ...);
    my @JobScheduleIntervals = NACL::C::JobScheduleInterval->find(command_interface => $ci, ...);

    Uses a CMode CLI or CMode ZAPI APISet.

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $JobScheduleInterval_state = $NACL::C::JobScheduleInterval->state();

    Uses a CMode CLI or CMode ZAPI APISet.

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::JobScheduleInterval would return
a NACL::CS::JobScheduleInterval object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::JobScheduleInterval->delete(
                                command_interface => $ci,
                                 name => $name,
                                  ...);
                                 or

    $JobScheduleInterval_object->delete(...);

(Class or instance method)
This method is used to delete an interval schedule.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-interval-destroy-iter" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI :

<job-schedule-interval-destroy-iter>
<query>
<job-schedule-interval-info>
<job-schedule-name>&value</job-schedule-name>
</job-schedule-interval-info>
</query>
</job-schedule-interval-destroy-iter>

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.


=item C<< '*' >>

This method takes all other options supported by
"job schedule interval delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::JobScheduleInterval::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing job schedule interval.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd  => 'job schedule interval delete',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::JobScheduleInterval::delete',
        );
    }
    catch NACL::C::Exceptions::Component with {

        # Already classified by the back-end
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::JobScheduleInterval::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(@args,
            api => 'job_schedule_interval_delete');

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_iter_helper(
        @args,
        api                  => 'job-schedule-interval-destroy-iter',
        show_cmd             => 'job schedule interval show',
        map                  => {"name" => "job-schedule-name",},
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::JobScheduleInterval::DoesNotExist',
        no_attributes => 1,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $JobScheduleInterval_object    = NACL::C::JobScheduleInterval->create(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

(Class method)
This method is used to create a schedule that runs on an interval.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-interval-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-name

=item C<< "seconds" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-seconds

=item C<< "hours" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-hours

=item C<< "minutes" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-minutes

=item C<< "days" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-days

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"job schedule interval create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params             => \@args,
        only_static_method => 1,
        additional_spec    => {
            _optional_scalars(
                qw( seconds hours minutes days
                    )
            ),
        },
        cli_cmd => 'job schedule interval create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::JobScheduleInterval::create',
    );

    my $obj = $pkg->new(
    command_interface => $opts{command_interface},
    name              => $opts{'name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api     => 'job_schedule_interval_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api => 'job-schedule-interval-create',
        map    => {
            "name"    => "job-schedule-name",
            "days"    => "job-schedule-interval-days",
            "hours"   => "job-schedule-interval-hours",
            "minutes" => "job-schedule-interval-minutes",
            "seconds" => "job-schedule-interval-seconds",
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 modify

   NACL::C::JobScheduleInterval->modify(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

                                 or

    $JobScheduleInterval_object->modify(...);

(Class or instance method)
This method is used to modify an interval schedule.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-interval-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-name

=item C<< "seconds" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-seconds

=item C<< "hours" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-hours

=item C<< "minutes" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-minutes

=item C<< "days" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-interval-days

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"job schedule interval modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( seconds hours minutes days
                    )
            ),
        },
        cli_cmd => 'job schedule interval modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::JobScheduleInterval::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'job_schedule_interval_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        api => 'job-schedule-interval-modify',
        map    => {
            "name"    => "job-schedule-name",
            "days"    => "job-schedule-interval-days",
            "hours"   => "job-schedule-interval-hours",
            "minutes" => "job-schedule-interval-minutes",
            "seconds" => "job-schedule-interval-seconds",
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('name' => {type => SCALAR},);
}

1;
