#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Lun Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Lun

=head1 DESCRIPTION

C<NACL::C::Lun> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Lun.  A related class is
L<NACL::CS::Lun|lib-NACL-CS-Lun-pm>, which represents
the past state of an ONTAP Lun.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 path

=cut

package NACL::C::Lun;

use strict;
use warnings;

use NACL::CS::Lun;
use NACL::C::_Mixins::Lun qw(:all);
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF UNDEF HASHREF SCALARREF OBJECT );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::VolumeFileClone;
use NACL::CS::LunGeometry;
use NACL::CS::LunStats;
use NACL::CS::IOfence;
use NACL::CS::LunMaxsize;
use NACL::C::Exceptions::Lun::AlreadyExists;
use NACL::C::Exceptions::Lun::DoesNotExist;
use NACL::C::Exceptions::Lun::AlreadyMapped;
use NACL::C::Exceptions::Lun::NotMapped;
use NACL::C::Exceptions::Lun::LunIdAlreadyUsed;
use NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume;
use NACL::C::Exceptions::Lun::TooSmall;
use NACL::Exceptions::InvalidChoice;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $Lun = NACL::C::Lun->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   path =>$path,
            );

Return a new Lun component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'path',
];

=head2 find

    my $Lun  = NACL::C::Lun->find(command_interface => $ci, ...);
    my @Luns = NACL::C::Lun->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Lun_state = $NACL::C::Lun->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Lun would return
a NACL::CS::Lun object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 maxsize

   my $obj = NACL::C::Lun->maxsize(
                                command_interface => $ci,
                                filter => {
                                          vserver => $vserver,
                                          volume => $volume,
                                          },
                                  ...);

   $complete_ss_reserve = $obj->complete_ss_reserve();

   (or)

   my @obj = NACL::C::Lun->maxsize(command_interface => $ci);
   $complete_ss_reserve = $obj[0]->complete_ss_reserve();

(Class method)
This method is used to display the maximun possible size of a LUN on a
given volume or qtree..
Uses a CMode CLI, CMode ZAPI APISet or 7Mode CLI APISet.
Returns object(s) of type L<NACL::CS::LunMaxsize|lib-NACL-CS-LunMaxsize-pm>.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< requested_fields => $requested_fields >>

(Optional, ARRAYREF)
The requested_fields parameter passed to fetch.

=item C<< filter => $filter >>

(Optional, HASHREF)
The filter parameter passed to fetch.

=back

=back

=cut

sub maxsize {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    if (ref $pkg) {
        NATE::BaseException->throw("'maxsize' is a static method and should "
                . "not be invoked with an object");
    }
    $Log->debug("Opts to 'maxsize'" . Dumper(\@_));
    my @state_objs = NACL::CS::LunMaxsize->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub maxsize

=head2 lun

   $lun_name = $lun_obj->lun();


(Instance method)
This method is used to get the name of the Lun through the Lun Component object
Uses a CMode CLI, or 7Mode CLI APISet.
Returns the name of the lun as a scalar

=cut

sub lun {

    $Log->enter() if $may_enter;
    my $self = $_[0];

    my $path     = $self->path();
    my @entry    = split "/", $path;
    my $lun_name = $entry[$#entry];

    $Log->exit() if $may_exit;
    return $lun_name;

} ## end sub lun

=head2 volume

   $volume_name = $lun_obj->volume();


(Instance method)
This method is used to get the name of the Volume through the
Lun Component/STask object.
Uses a CMode CLI, or 7Mode CLI APISet.
Returns the name of the volume as a scalar

=cut

sub volume {

    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $path     = $self->path();
    my @entry    = split "/", $path;
    my $volume_name = $entry[2];

    $Log->exit() if $may_exit;
    return $volume_name;

} ## end sub volume

=head2 cast_to_volume

    my $vol_obj = $lun_obj->cast_to_volume();

This method is used to cast the Lun object to Volume object as luns do reside
in volumes so some operations which can be performed on volumes
can be performed on luns as well.

Note that if invoked on a NACL::C::Lun object, a NACL::C::Volume
object is returned, while if is invoked on a NACL::STask::Lun
object, a NACL::STask::Volume object is returned.

=cut

sub cast_to_volume {
    $Log->enter() if $may_enter;
    my $self    = $_[0];
    my $c_or_t  = $self->is_c_or_t();
    my $vol_pkg = "NACL::${c_or_t}::Volume";
    my %pk_hash = $self->get_pk_hash_with_ci();
    $pk_hash{volume} = $self->volume();
    delete $pk_hash{path};
    my $vol_obj = $vol_pkg->new(%pk_hash);
    $Log->exit() if $may_exit;
    return $vol_obj;
} ## end sub cast_to_volume

=head2 cast_to_file

    my $file_obj = $lun_obj->cast_to_file();

This method is used to cast the Lun object to Volume File object as
luns and files are similar for some operations. So, in some cases
luns can be treated as file.

Note that if invoked on a NACL::C::Lun object, a NACL::C::VolumeFile
object is returned, while if is invoked on a NACL::STask::Lun
object, a NACL::STask::VolumeFile object is returned.

=cut

sub cast_to_file {
    $Log->enter() if $may_enter;
    my $self    = $_[0];
    my $c_or_t  = $self->is_c_or_t();
    my $file_pkg = "NACL::${c_or_t}::VolumeFile";
    my %pk_hash = $self->get_pk_hash_with_ci();
    my $file_obj = $file_pkg->new(%pk_hash);
    $Log->exit() if $may_exit;
    return $file_obj;
} ## end sub cast_to_file

# Ensure that the vserver attribute is undef *only* if we're using
# a vserver command interface.
sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;

    # Call the base class's init to do the normal argument checking
    $self->SUPER::init(%opts);

    $self->_vserver_context_init(%opts);
    $Log->exit() if $may_exit;
    return;
} ## end sub init

=head2 create

    my $Lun_object    = NACL::C::Lun->create(
                                command_interface => $ci,
                            vserver => $vserver,
                            path => $path
                                  ...);
    (or)

    my $Lun_object    = NACL::C::Lun->create(
                                command_interface => $ci,
                            vserver => $vserver,
                            volume => $volume,
                                lun => $lun,
                                qtree => $qtree
                                  ...);

(Class method)
This method is used to create a new LUN.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "lun-create-by-size" API for CMode ZAPI if size is defined.

Invokes "lun-create-from-file" API for CMode ZAPI if size is not defined.

Invokes "lun create" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< lun => $lun >>

(Required if path is not provided)
Name of the LUN

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< volume => $volume >>

(Required if path is not provided)
Name of the volume on which the LUN is present

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< qtree => $qtree >>

(Optional, applicable only for CMode in RR onwards, cannot be provided if path
is provided)
The qtree on which the LUN is present.

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< vserver => $vserver >>

(Required in CMode and if path is not provided, not applicable in 7Mode)
Vserver on which the volume is present

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< path => $path >>

(Optional, can only be specified if none of (volume+lun+qtree) are specified)
The path to the LUN. This is either in the form "/vol/$volume/$qtree/$lun" if
it is for a CMode RR build onwards, else it is of the form "/vol/$volume/$lun".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : lun path (Programmatic name: "lun-path")

=item C<< "file-path" => $string >>

If specified, the LUN is created from a particular file, whose path is
to be specified through this option.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: file-name

7M CLI : -f <value> (Programmatic name: "file-path")

=item C<< "extent-size" => $string >>

LUN Extent Size

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -E <value> (Programmatic name: "extent-size")

=item C<< "suffix-size" => $string >>

Lun Suffix Size

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -S <value> (Programmatic name: "suffix-size")

=item C<< "space-resv" => $string >>

Space Reservation.
Builds having UI change "vserver-lun-igroup-portset-rr-0"
accept the option "space-reserve" but the compatibility layer provided by
the APISet will automatically translate "space-resv" to "space-reserve" if
necessary.

Applicable for 7Mode CLI.

=item C<< "space-reserve" => $string >>

Space Reservation.
This option is valid only for builds having UI change
"vserver-lun-igroup-portset-rr-0".

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : space-reservation-enabled

=item C<< "state" => $string >>

Applicable for 7Mode CLI.

=item C<< "size" => $string >>

Lun Size

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : -s <value> (Programmatic name: "size")

=item C<< "prefix-size" => $string >>

LUN Prefix size

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: prefix-size

7M CLI : -P <value> (Programmatic name: "prefix-size")

=item C<< ostype => $string >>

LUN OS Type
Possible values: vmware | hyper_v | windows_2008 | windows_gpt | windows |
linux | xen | solaris | solaris_efi | hpux | aix | netware | openvms

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: ostype

7M CLI : -t <value> (Programmatic name: "ostype")

=item C<< "option" => $string >>

Space Reservation Option

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -o noreserve (Programmatic name: "noreserve")

=item C<< "write" => $string >>

Applicable for 7Mode CLI.

=item C<< 'space-reserve' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: space-reservation-enabled

=item C<< 'qos-policy-group' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: qos-policy-group

=item C<< 'class' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: class

=item C<< 'external-cache' => $string >>

External Cache Policy Name

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: external-cache-policy

=item C<< "space-alloc" => $string >>

Applicable for 7Mode CLI.

=item C<< "noreserve" => $string >>

Applicable for 7Mode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for 7Mode CLI.

=item C<< 'foreign-disk' => $string >>

Foreign Disk Serial number
Applicable for CMode CLI

=item C<< 'caching-policy' => $string >>

Caching Policy Name
Applicable for CMode CLI

=item C<< 'comment' => $string >>

Comment
Applicable for CMode CLI

=item C<< 'P' => $string >>

prefix size
Applicable for CMode CLI

=item C<< 'qos-adaptive-policy-group' => $string >>

Caching policy to apply to the LUN

Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"lun create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item NATE::Exceptions::Argument

CMode ZAPI backend will throw this exception, if none of 'file-path' and
'size' was specified or if both of them were specified.

=item C<NACL::C::Exceptions::Lun::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
existing lun.

=item C<NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume>

This type of exception gets thrown when volume does not have enough space
to create a lun.

=item C<NACL::C::Exceptions::Lun::TooSmall>

This type of exception gets thrown when Lun gets created with small size.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        only_static_method => 1,
        params             => \@args,
        additional_spec    => {
            _optional_scalars(
                qw( file-path extent-size suffix-size space-resv state size
                    prefix-size ostype option write space-alloc space-allocation
                    lun qtree volume P
                    space-reserve

                    qos-policy-group

                    class

                    external-cache
                    foreign-disk caching-policy

                    comment application
		    qos-adaptive-policy-group
                    )
            ),
        },
        cli_cmd => 'lun create',
    );

    my $obj;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $obj = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_check_zapi_create',
                },
            ],
            frontend => 'NACL::C::Lun::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Lun::AlreadyExists->detect_convert_and_throw(
            exception => $exception);
        NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::Lun::TooSmall
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts, api => 'lun_create');

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        path              => $opts{path},
        vserver           => $opts{vserver}
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_cmode_cli

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my %api_opts;

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    if (exists $opts{option}) {
        if ($opts{option} eq 'noreserve') {
            $opts{noreserve} = 1;
        }
    }

    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        copy   => [
            qw(size file-path ostype prefix-size space-alloc noreserve
               suffix-size extent-size)
        ],
        map => {'path' => 'lun-path'},
    );

    $apiset->lun_create(%api_opts);

    my $obj = $pkg->new(
        command_interface => $command_interface,
        path              => $api_opts{'lun-path'}
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_7mode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (defined $opts{size} && defined $opts{'file-path'}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Provide 'file-path' or 'size' not both");
    } elsif (defined $opts{'foreign-disk'} && defined $opts{'size'}) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Provide 'foreign-disk' or 'size' not both");
    }

    my $copy = [
        qw( path size prefix-size ostype class qos-policy-group
            foreign-disk comment application qos-adaptive-policy-group)
    ];
    my %common_opts = (
                          copy => $copy,
                          map  => {
                              'space-reserve'     => 'space-reservation-enabled',
                              'external-cache'    => 'external-cache-policy',
                              'file-path'         => 'file-name',
                               'space-allocation' => 'space-allocation-enabled'
                          },
                          zapi_field_translations => {
                               enable_disable => [qw(space-reserve space-allocation)],
                          },
                      );
    if (defined $opts{size} || defined $opts{'foreign-disk'}) {
        $opts{size} = $pkg->convert_to_bytes($opts{size})
            if (defined $opts{size});
        $pkg->_base_backend_cmode_zapi(
            %opts,
            %common_opts,
            api => 'lun-create-by-size',
        );

    } elsif (defined $opts{'file-path'}) {
        $pkg->_base_backend_cmode_zapi(
            %opts,
            %common_opts,
            api => 'lun-create-from-file',
        );
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
                  "Either of 'file-path' or 'size' or 'foreign-disk'"
                . ' has to be specified');
    }

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        path              => $opts{'path'},
        vserver           => $opts{vserver},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_cmode_zapi

sub _check_zapi_create {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (defined $opts{'suffix-size'}) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('Could not use ZAPI '
                . 'since the option \'suffix-size\' is not supported '
                . 'by the ZAPI lun-create-by-size');
    }
    $Log->exit() if $may_exit;
}

=head2 share

    NACL::C::Lun->share(
                                command_interface => $ci,
                                path => $path,
                                file-access => ( all | none | read | write ),
                        );
     or

    $Lun_object->share(...);

(Class or instance method)
This method is used to enable file system protocol-based access to a lun. By default, all accesses are disallowed.

Uses a 7Mode CLI APISet.

Invokes 'lun share' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path => $path >>

(Required)
The path to the LUN. This can be in the form "/vol/$volume/$lun"

Applicable for 7Mode CLI.

Maps to:

7M CLI: lun path (Programmatic name: "lun-path")

=item C<< "file-access" => ( all | none | read | write ) >>

(Required) Accesses rights to lun.
 By  default, all accesses are disallowed.

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub share {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( file-access
                    )
            ),
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_share_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub share

sub _share_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( file-access
            )
    ];

    my %mode_args;

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode_args,
        map    => {'path' => 'lun-path'},
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api_opts => \%mode_args,
        api      => 'lun_share'
    );

    $apiset->lun_share(%mode_args);
    $Log->exit() if $may_exit;

} ## end sub _share_7mode_cli

=head2 delete

    NACL::C::Lun->delete(command_interface => $ci,
            vserver => $vserver,
            path => $path,
            force => 'true'
                        );
    or

    $Lun_object->delete([force => 'true']);

    or

    NACL::C::Lun->delete(
                         command_interface => $ci,
                         vserver => $vserver,
                         volume => $volume,
                         lun => $lun
                        );


(Class or instance method)
This method is used to delete the lun.

Uses a CMode CLI, 7Mode CLI APISet.

Invokes "lun-destroy" API for CMode ZAPI.

Invokes "lun destroy" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method, Not Applicable
in 7Mode or if path is provided)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< path => $path >>

(Required for class method, Not Applicable for instance method, can only
be specified if none of (volume+vserver+lun+qtree) are specified)
The path to the LUN. This is either in the form "/vol/$volume/$qtree/$lun" if
it is for a CMode RR build onwards, else it is of the form "/vol/$volume/$lun".

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : path

=item C<< force => $string >>

This option specifies to delete the lun even if it is mapped or online.
Possible values: true | false

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: force

7M CLI : -f (Programmatic name: "force")

=item C<< 'destroy-fenced-lun' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'force-fenced' => $string >>

Force Deletion of a Fenced LUN
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: 'destroy-fenced-lun'

=item C<< '*' >>

This method takes all other options supported by
"lun delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Lun::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing lun.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (force lun qtree volume
                    destroy-fenced-lun

                    force-fenced destroy-application-lun
                    )
            )
        },
        cli_cmd => 'lun delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::Lun::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Lun::DoesNotExist->detect_convert_and_throw(
            exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force)],
        is_7m_cli       => 0
    );

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'lun_delete');

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my %api_opts;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force)],
        is_7m_cli       => 1
    );

    $api_opts{'lun-path'} = $opts{'path'};
    $api_opts{force} = $opts{force};

    $apiset->lun_destroy(%api_opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $copy = [qw(force path destroy-fenced-lun destroy-application-lun)];
    my $map = {'force-fenced' => 'destroy-fenced-lun',};
    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => [qw(force)],
        is_7m_cli       => 0
    );
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'lun-destroy',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 map

    NACL::C::Lun->map(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

                                or

    $Lun_object->map(...);

    or


    NACL::C::Lun->map(
                                command_interface => $ci,
                                path => $path
                     );

(Class or instance method)
This method is used to map LUN to all the initiators in the group..

Uses a CMode CLI/ZAPI,7Mode CLI APISet.

Invokes "lun-map" API for CMode ZAPI.

Invokes "lun map" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : lun path (Programmatic name: "lun-path")

=item C<< "force" => $string >>

Applicable for CMode CLI.

=item C<< "igroup" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: initiator-group

7M CLI : initiator group name (Programmatic name: "initiator-group")

=item C<< "lun-id-assigned" => $string >>

Applicable for CMode CLI.

=item C<< "lun-id" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : lun id (Programmatic name: "lun-id")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun map" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Lun::AlreadyMapped>

This type of exception gets thrown if an attempt is made to map a
lun which is already mapped.

=back

=back

=cut

sub map {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = @args;
    %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force igroup lun-id-assigned lun-id lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun map',
    );
    my $ret;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_map_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_map_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_map_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::Lun::map',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Lun::AlreadyMapped
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::Lun::LunIdAlreadyUsed
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub map

sub _map_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cmode_args = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'lun_map',
        options => \%opts,
    );

    $apiset->lun_map(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _map_cmode_cli

sub _map_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(path force lun-id)];
    my $map = {'igroup' => 'initiator-group',};
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-map',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

sub _map_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( lun-id )],
        map    => {
            'path'   => 'lun-path',
            'igroup' => 'initiator-group'
        },
        target => \%args_7mode,
    );

    $apiset->lun_map(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _map_7mode_cli

=head2 modify

    NACL::C::Lun->modify(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

    (or)

    $Lun_object->modify(...);

    (or)

    NACL::C::Lun->modify(
                                command_interface => $ci,
                                path => $path
                        );
(Class or instance method)
This method is used to modify a LUN.

Invokes "lun offline" command for 7Mode CLI if state is offline.

Invokes "lun-set-space-reservation-info, lun-online, lun-offline, lun-set-comment,
lun-set-serial-number, lun-set-device-id, lun-set-qos-policy-group"
API's for CMode ZAPI.

Invokes "lun online, lun offline" commands for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method,Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : lun path (Programmatic name: "lun-path")

=item C<< "device-text-id" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "clear-text-id" => $string >>

Applicable for CMode CLI.

=item C<< "device-binary-id" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : device-id

=item C<< "state" => $string >>

Applicable for CMode CLI.

=item C<< "space-resv-enabled" => $string >>

Applicable for CMode CLI.

=item C<< "size" => $string >>

Applicable for CMode CLI.

=item C<< "device-legacy-id" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "write" => $string >>

Applicable for CMode CLI.

=item C<< "extent-size" => $string >>

Applicable for CMode CLI.

=item C<< "clear-binary-id" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: Invokes the 'lun-unset-device-id' API if the clear-binary-id option
passed.

=item C<< "suffix-size" => $string >>

Applicable for CMode CLI.

=item C<< "serial" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : serial-number of lun-set-serial-number API.

=item C<< "space-alloc-enabled" => $string >>

Applicable for CMode CLI.

=item C<< "space-resv" => $string >>

Space Reservation.
Builds having UI change "vserver-lun-igroup-portset-rr-0"
accept the option "space-reserve" but the compatibility layer provided by
the APISet will automatically translate "space-resv" to "space-reserve" if
necessary.

Applicable for CMode CLI.

=item C<< "space-reserve" => $string >>

Space Reservation.
This option is valid only for builds having UI change
"vserver-lun-igroup-portset-rr-0".

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : enable of lun-set-space-reservation-info API.

=item C<< "prefix-size" => $string >>

Applicable for CMode CLI.

=item C<< "space-alloc" => $string >>

Applicable for CMode CLI.

=item C<< "comment" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : comment of lun-set-comment

=item C<< "select-attribute" => $string >>

Applicable for CMode CLI.

=item C<< 'space-allocation' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: enable of lun-set-space-alloc API

=item C<< 'qos-policy-group' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: qos-policy-group of lun-set-qos-policy-group API

=item C<< 'external-cache' => $string >>

External Cache Policy Name

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: external-cache-policy of lun-set-external-cache-policy API

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'caching-policy' => $string >>

Caching Policy Name
Applicable for CMode CLI

=item C<< 'serial-hex' => $string >>

Serial Number (Hex)

Applicable for CMode CLI

=item C<< 'qos-adaptive-policy-group' => $string >>

Caching policy to apply to the LUN

Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"lun modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( device-text-id clear-text-id device-binary-id state space-resv-enabled size
                    device-legacy-id write extent-size clear-binary-id suffix-size serial
                    space-alloc-enabled space-resv prefix-size space-alloc comment
                    select-attribute space-reserve lun qtree volume

                    space-allocation

                    qos-policy-group

                    external-cache
                    caching-policy serial-hex
		    qos-adaptive-policy-group
                    )
            ),
        },
        cli_cmd => 'lun modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
		check     => '_cmode_zapi_check',
            },
            {   method    => '_modify_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::Lun::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};

    my $allowed_cmode_options = [qw( path comment )];

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {
            'space-reserve'    => 'enable',
            'serial'           => 'serial-number',
            'device-binary-id' => 'device-id',
            'device-text-id'   => 'device-id',
            'device-legacy-id' => 'device-id',
        },
        target => \%cmode_args,
    );

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface,
    );
    if ($opts{'space-reserve'}) {
        if ($opts{'space-reserve'} eq 'enabled') {
            $cmode_args{'enable'} = 'true';
        } elsif ($opts{'space-reserve'} eq 'disabled') {
            $cmode_args{'enable'} = 'false';
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw('Invalid value'
                    . $opts{'space-reserve'}
                    . "passed for space-reserve\n");
        } ## end else [ if ( $opts{'space-reserve'...})]
        $apiset->lun_set_space_reservation_info(%cmode_args);
    } ## end if ( $opts{'space-reserve'...})
    if ($opts{'state'}) {
        if ($opts{'state'} eq 'online') {
            $apiset->lun_online(%cmode_args);
        } elsif ($opts{'state'} eq 'offline') {
            $apiset->lun_offline(%cmode_args);
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                'Invalid value' . $opts{'state'} . "passed for state\n");
        }
    } ## end if ( $opts{'state'} )

    if ($opts{'comment'}) {
        $apiset->lun_set_comment(%cmode_args);
    }
    if ($opts{'serial'}) {
        $apiset->lun_set_serial_number(%cmode_args);
    }
    if ($opts{'device-binary-id'}) {
        # ZAPI requires the device id type to be set when there is any
        # modification to device-id
        $cmode_args{'device-id-type'} = "binary";
        $apiset->lun_set_device_id(%cmode_args);
    } elsif ($opts{'device-legacy-id'}) {
        $cmode_args{'device-id-type'} = "legacy";
        $apiset->lun_set_device_id(%cmode_args);
    } elsif ($opts{'device-text-id'}) {
        $cmode_args{'device-id-type'} = "text";
        $apiset->lun_set_device_id(%cmode_args);
    }

    if (defined $opts{'external-cache'}) {
        $apiset->lun_set_external_cache_policy(
            'external-cache-policy' => $opts{'external-cache'},
            'path'                  => $cmode_args{'path'},
            'vserver-override-name' => $vserver,
        );
    } ## end if ( defined $opts{'external-cache'...})

    if (defined $opts{'clear-binary-id'}) {
        $apiset->lun_unset_device_id(
            'path'                  => $cmode_args{'path'},
            'vserver-override-name' => $vserver,
        );
    } ## end if ( defined $opts{'clear-binary-id'...})

    ######As per Burt 912597 for clear-text-id#############
    if (defined $opts{'clear-text-id'}) {
            $apiset->lun_unset_device_id(
            'path'                  => $cmode_args{'path'},
            'vserver-override-name' => $vserver,
            'device-id-type' => "text",
           );
    } ## end if ( defined $opts{'clear-text-id'...})

    if (defined $opts{'qos-policy-group'}) {
        $apiset->lun_set_qos_policy_group(
            'qos-policy-group'      => $opts{'qos-policy-group'},
            'vserver-override-name' => $vserver,
            'path'                  => $cmode_args{'path'}
        );
    } ## end if ( defined $opts{'qos-policy-group'...})

    if (defined $opts{'qos-adaptive-policy-group'}) {
        $apiset->lun_set_qos_policy_group(
            'qos-adaptive-policy-group'      => $opts{'qos-adaptive-policy-group'},
            'vserver-override-name' => $vserver,
            'path'                  => $cmode_args{'path'}
        );
    } ## end if ( defined $opts{'qos-adaptive-policy-group'...})

    if (defined $opts{'space-allocation'}) {
        my $convertion_opts = {'enabled' => 'true', 'disabled' => 'false'};

        $apiset->lun_set_space_alloc(
            'path'   => $cmode_args{'path'},
            'enable' => $convertion_opts->{$opts{'space-allocation'}},
            'vserver-override-name' => $vserver,
        );
    } ## end if ( defined $opts{'space-allocation'...})

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'path' => 'lun-path',},
        target => \%args_7mode,
    );
    my %orig_args = %opts;
    %opts = %args_7mode;

    if ($orig_args{state} =~ /online/) {

        $apiset->lun_online(%opts);
    } elsif ($orig_args{state} =~ /offline/) {

        $apiset->lun_offline(%opts);
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("The value of 'state' can be either"
                . "of 'online' or 'offline' but was passed as '$orig_args{state}'"
        );
    } ## end else [ if ( $orig_args{state}...)]
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _cmode_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (defined $opts{'serial-hex'}) {
          my $msg = 'Could not use ZAPI implementation of NACL::C::Lun->modify().'
                    .' since the CMode ZAPI does not support "serial-hex" option';

          $Log->comment($msg);
          $Log->exit() if $may_exit;
          NACL::Exceptions::InvalidChoice->throw($msg);
     }


    $Log->exit() if $may_exit;
} ## end sub _cmode_zapi_check

=head2 rescan

    NACL::C::Lun->rescan(
                                command_interface => $ci,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);


(Class method)
This method is used to rescan the volume again to find all the LUN objects..

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "lun-lun-rescan" API for CMode ZAPI.

Invokes "lun rescan" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "volume" => $volume >>

(Required)
Name of the Volume.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: volume

7M CLI : volume name (Programmatic name: "volume")

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "sync-new-identifiers" => $string >>

(Optional)
*Sync New IDs, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: sync-new-identifiers

=item C<< "sync-oovc-to-vol" => $string >>

(Optional)
*Sync OOVC to Vol, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: sync-oovc-to-vol

=item C<< "build-vtoc" => $string >>

(Optional)
*Build VTOC, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: build-vtoc

=item C<< "sync" => $string >>

(Optional)
*Sync, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: sync

=item C<< "build-oovc" => $string >>

(Optional)
*Build OOVC, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: build-oovc

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun rescan" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rescan {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    if (ref $pkg) {
        NATE::BaseException->throw("'rescan' is a static method and should "
                . "not be invoked with an object");
    }
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( sync-new-identifiers sync-oovc-to-vol build-vtoc sync build-oovc vserver volume)
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'lun rescan',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_rescan_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_rescan_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
            {   method    => '_rescan_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::Lun::rescan',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub rescan

sub _rescan_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api => 'lun_rescan');
    $Log->exit() if $may_exit;
} ## end sub _rescan_cmode_cli

sub _rescan_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( sync-new-identifiers volume build-oovc build-vtoc sync-oovc-to-vol
            sync vserver)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
		api    => 'lun_lun_rescan',
		copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _rescan_cmode_zapi

sub _rescan_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw( volume )];
    my %args_7mode;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%args_7mode,
    );

    $apiset->lun_rescan(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _rescan_7mode_cli

=head2 unmap

    NACL::C::Lun->unmap(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

                                or

    $Lun_object->unmap(...);

    (or)


    NACL::C::Lun->unmap(
                                command_interface => $ci,
                                path => $path
                       );

(Class or instance method)
This method is used to remove a previously configured mapping.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "lun-unmap" API for CMode ZAPI.

Invokes "lun unmap" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : path
7M CLI  : lun path (Programmatic name: "lun-path")

=item C<< "igroup" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI: initiator-group

7M CLI : initiator group name (Programmatic name: "initiator-group")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun unmap" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Lun::NotMapped>

This type of exception gets thrown if an attempt is made to unmap a
lun when it is not mapped.

=back

=back

=cut

sub unmap {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( igroup lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun unmap',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_unmap_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_unmap_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_unmap_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::Lun::unmap',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::Lun::NotMapped->detect_convert_and_throw(
            exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub unmap

sub _unmap_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_unmap');
    $Log->exit() if $may_exit;
} ## end sub _unmap_cmode_cli

sub _unmap_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [qw( path )];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'lun_unmap',
            copy   => $allowed_cmode_options,
            map    => {"igroup" => "initiator-group"},
	);
    $Log->exit() if $may_exit;
} ## end sub _unmap_cmode_zapi

sub _unmap_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            'path'   => 'lun-path',
            'igroup' => 'initiator-group'
        },
        target => \%args_7mode,
    );

    $apiset->lun_unmap(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _unmap_7mode_cli

=head2 resize

    NACL::C::Lun->resize(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

     or

    $Lun_object->resize(...);

    or

    NACL::C::Lun->resize(
                                command_interface => $ci,
                                path => $path
                        );

(Class or instance method)
This method is used to changes the size of the LUN to the input value size. This command can be issued even while the LUN is mapped and is online. Note that client-side operations may be needed to ensure that client software recognizes the changed size. The -f flag or manual confirmation is required to reduce the size of a LUN..

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "lun-resize" API for CMode ZAPI.

Invokes "lun resize" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable  for 7Mode CLI

Maps to:

CM ZAPI: path

7M CLI : lun path (Programmatic name: "lun-path")

=item C<< "force" => $string >>

Applicable for Cmode CLI, 7Mode CLI.

Maps to:

7M CLI : -f (Programmatic name: "force")

=item C<< "actual-size" => $string >>

Applicable for CMode CLI.

=item C<< "size" => $string >>

Applicable  for 7Mode CLI

Maps to:

CM ZAPI: size

7M CLI : new size (Programmatic name: "new-size")

=item C<< "newsize" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun resize" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resize {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = @args;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force actual-size size newsize lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun resize',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resize_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_resize_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_resize_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::Lun::resize',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub resize

sub _resize_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_resize');
    $Log->exit() if $may_exit;
} ## end sub _resize_cmode_cli

sub _resize_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( path size force
            )
    ];
	$opts{size} = $pkg_or_obj->convert_to_bytes($opts{size})
        if (defined $opts{size});
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
		api    => 'lun_resize',
		copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _resize_cmode_zapi

sub _resize_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( force )],
        map    => {
            'path' => 'lun-path',
            'size' => 'new-size'
        },
        target => \%args_7mode,
    );

    $apiset->lun_resize(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _resize_7mode_cli

=head2 move

    NACL::C::Lun->move(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

    or

    $Lun_object->move(...);

    or

    NACL::C::Lun->move(
                                command_interface => $ci,
                                path => $path
                      );


(Class or instance method)
This method is used to move (rename) a LUN..

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "lun-move" API for CMode ZAPI.

Invokes "lun move" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : lun path (Programmatic name: "lun-path")

=item C<< "new-lun" => $string >>

Applicable for CMode CLI.

=item C<< "new-qtree" => $string >>

Applicable for CMode CLI.

=item C<< "new-path" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: path

7M CLI : to lun path (Programmatic name: "to-lun-path")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun move" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub move {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = @args;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-lun new-qtree new-path lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun move',
    );

    $Log->warn("The command 'lun move' was renamed to "
                . "'lun move-in-volume' in CLI. Please use "
                . "NACL::C::Lun->move_in_volume instead. "
                . "Support for the old command will be "
                . "removed from the library soon.") if ($opts{command_interface}->has_uichange(
                    uichange => 'lun-move-renamed-to-lun-move-in-volume'));

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_move_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_move_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_move_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::Lun::move',
    );
    if ($opts{command_interface}->is_cmode()) {
        $pkg_or_obj->_set_path(%opts);
    } else {
        $pkg_or_obj->path($opts{'to-lun-path'}) if (ref $pkg_or_obj);
    }
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub move

sub _move_cmode_cli {
    $Log->enter() if $may_enter;
    my $new_path;
    my ($pkg_or_obj, @args) = @_;
    my %opts = @args;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'lun_move');

    $Log->exit() if $may_exit;
} ## end sub _move_cmode_cli

sub _move_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    # CLI accepts 'new-lun' or 'new-qtree' to move the Lun, whereas ZAPI
    # accepts only 'new-path' as parameter. Trying to construct 'new-path'
    # if not already passed.
    $opts{'new-path'} = $pkg->_get_new_path(%opts);
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api => 'lun-move',
        copy => [qw( path new-path )],
    );

    $Log->exit() if $may_exit;
} ## end sub _move_cmode_zapi

sub _set_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (ref $pkg_or_obj) {
        $opts{'path'} = $pkg_or_obj->path();
        my $new_path = $pkg_or_obj->_get_new_path(%opts);
        $pkg_or_obj->path($new_path);
    }
    $Log->exit() if $may_exit;
}

sub _get_new_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (exists $opts{'new-path'}) {
        return $opts{'new-path'};
    } else {
        my $param;
        my ($vol, $qtree, $lun) = $pkg_or_obj->_extract_volume_qtree_lun_from_path(path => $opts{path});
        $param->{qtree}  = $opts{'new-qtree'} if (exists $opts{'new-qtree'});
        $param->{lun}    = $opts{'new-lun'} if (exists $opts{'new-lun'});
        $param->{volume} = $vol;
        $pkg_or_obj->_construct_path(params => $param);
        return $param->{path};
    }
    $Log->exit() if $may_exit;
}

sub _move_7mode_cli {
    $Log->enter() if $may_enter;
    my $new_path;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            'path'     => 'lun-path',
            'new-path' => 'to-lun-path',
        },
        target => \%args_7mode,
    );
    %opts = %args_7mode;

    $apiset->lun_move(%opts);
    $Log->exit() if $may_exit;

} ## end sub _move_7mode_cli

=head2 move_in_volume

    NACL::C::Lun->move_in_volume(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

    or

    $Lun_object->move_in_volume(...);

    or

    NACL::C::Lun->move_in_volume(
                                command_interface => $ci,
                                path => $path
                      );


(Class or instance method)
This method is used to move (rename) a LUN..

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-move" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< "new-lun" => $string >>

Applicable for CMode CLI.

=item C<< "new-qtree" => $string >>

Applicable for CMode CLI.

=item C<< "new-path" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun move" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_throw@
of user guide.

=back

=back

=cut

sub move_in_volume {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = @args;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-lun new-qtree new-path lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun move-in-volume',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_move_in_volume_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_move_cmode_zapi', # lun move remains same in zapi burt#791686
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::move_in_volume',
    );

    $pkg_or_obj->_set_path(@args);
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub move_in_volume

sub _move_in_volume_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'lun_move_in_volume');

    $Log->exit() if $may_exit;
}

=head2 lun_persistent_resv_clear

    NACL::C::Lun->lun_persistent_resv_clear(
                                command_interface => $ci,
                                lun => $lun,
                                qtree => $qtree,
                                volume => $volume,
                                vserver => $vserver,
                                  ...);

    or

    $Lun_object->lun_persistent_resv_clear(...);

    or

    NACL::C::Lun->lun_persistent_resv_clear(
                                command_interface => $ci,
                                path => $path
                      );


(Class or instance method)
This method is used to lun_persistent_resv_clear (rename) a LUN..

Uses a 7Mode CLI APISet.

Invokes 'lun persistent resv clear' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "lun" => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "qtree" => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

7M CLI:

Since this option is not passed to the apilayer call ,there is no mapping
information provided , path will be constructed using this argument
and passed to the apilayer call.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI.

=item C<< "path" => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for 7Mode CLI

=item C<< "new-lun" => $string >>

Applicable for 7Mode CLI.

=item C<< "new-qtree" => $string >>

Applicable for 7Mode CLI.

=item C<< "new-path" => $string >>

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub persistent_resv_clear {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_persistent_resv_clear_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub persistent_resv_clear

sub _persistent_resv_clear_7mode_cli {
    $Log->enter() if $may_enter;
    my $new_path;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %args_7mode;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'path' => 'lun-path',},
        target => \%args_7mode,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api_opts => \%args_7mode,
        api      => 'lun_persistent_resv_clear'
    );

    $apiset->lun_persistent_resv_clear(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _persistent_resv_clear_7mode_cli

=head2 clone

    Examples for class method calls
    # Specifying relative paths, if the option 'volume' is specified.
    my $clone_obj = NACL::C::Lun->clone(
                                command_interface => $ci,
                                vserver => $TestVserver,
                                volume => $Volume,
                                "source-path" => $Lun,
                                "destination-path" => $Clone,
                                "no-reserve" => "true",
                                "snapshot-name" => $SnapshotName
                                 ...);
    # Specifying absolute paths
    my $clone_obj = NACL::C::Lun->clone(
                                command_interface => $ci,
                                vserver => $TestVserver,
                                "source-path" => '/vol/$vol_name/$lun_name',
                                "destination-path" => '/vol/$vol_name/$lun_clone_name',
                                ...
                                );

    Examples for Instance method calls
    # Specifying relative path for destination
    my $clone_obj = $Lun_object->clone('destination-path' => $lun_clone_name, ...);

    # Specifying absolute path for destination
    my $clone_obj = $Lun_object->clone('destination-path' => '/vol/${vol_name}/$lun_clone_name');

(Class or Instance method)

This method is used to create a lun clone. This is achieved by using
VolumeFileClone->create. For an example, documentation of the options accepted
and the APISets for which this has been implemented, please see
L<< NACL::C::VolumeFileClone->create|lib-NACL-C-VolumeFileClone-pm/create >>.

(Note that the C<clone_type> argument accepted by VolumeFile->create is
automatically set to "lun" when calling this method)

=cut

sub clone {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->SUPER::_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw(path)),},
        allow_extra     => 1
    );

    my $path = $opts{path};

    # For instance method calls, assign the value of 'path'
    # based on whether 'volume' is specified by the user or not.
    if (ref $pkg_or_obj && exists $opts{volume}) {

        # common_validate_with would have populated with absolute path
        # Fix it now, since 'volume' option is specified.
        $path =~ /\/vol\/(.+?)\/(.+)/;
        $path = $2;
    }

    my $source_path = $opts{'source-path'} || $path;

    # If both source-path and path are not specified, throw an exception
    # This can be hit for class method calls.
    unless (defined $source_path) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            'Either of path or source-path ' . 'must be specified');
    }

    #Get the volume name.
    my $volume = $opts{volume};
    unless (defined $volume) {
        if ($source_path =~ /^\//) {

            # source_path is in absolute format
            if ($source_path =~ /\/vol\/(.+?)\//) {
                $volume = $1;
            } else {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                    "Path '$source_path' is not in the" . " expected format");
            }
        }

        # else, the source_path is specified as relative path when volume
        # option is not specified. Let's leave it to ONTAP to complain for
        # this
    }

    # If both source-path and path are defined, check if they are
    # not conflicting each other.
    my $get_abs_path = sub {
        my $val = shift;
        unless ($val =~ /^\//) {
            $val = "/vol/${volume}/${val}";
        }
        return $val;
    };
    if (defined($source_path) && defined($path) && $volume) {
        my $abs_src_path = $get_abs_path->($source_path);
        my $abs_path     = $get_abs_path->($path);
        if ($abs_src_path ne $abs_path) {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("The value for "
                    . "source-path '$source_path' conflicts with path '$path'"
            );
        }
    }

    # At this point, source-path will be defined and validated against path
    # No longer, path is needed.
    delete $opts{path};

    # Update source-path back into %opts hash
    $opts{'source-path'} = $source_path;

    NACL::C::VolumeFileClone->create(%opts, clone_type => 'lun');

    my $abs_dest_path = $get_abs_path->($opts{'destination-path'});

    my $obj = NACL::C::Lun->new(
        command_interface => $opts{command_interface},
        path              => $abs_dest_path,
        vserver           => $opts{'vserver'}
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub clone

=head2 testmap

    NACL::C::Lun->testmap(
                                command_interface => $ci,
                                'lun' => $lun,
                                'qtree' => $qtree,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

                                 or

    $Lun_obj->testmap(...);

(Class or instance method)
This method is used to test the Map LUN performance.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'rowcount' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< 'usenextref' => $string >>

Applicable for CMode CLI.

=item C<< 'igroup' => $string >>

Applicable for CMode CLI.

=item C<< 'lun-id-assigned' => $string >>

Applicable for CMode CLI.

=item C<< 'lun-id' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< 'lun' => $string >>

LUN Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun testmap" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub testmap {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( rowcount force usenextref igroup lun-id-assigned lun-id

                    volume lun qtree
                    )
            ),
        },
        cli_cmd => 'lun testmap',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_testmap_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::testmap',
    );

    $Log->exit() if $may_exit;
} ## end sub testmap

sub _testmap_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_testmap');
    $Log->exit() if $may_exit;
} ## end sub _testmap_cmode_cli

=head2 stats

    my $stats_obj = NACL::C::Lun->stats(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @stats_objs = NACL::C::Lun->stats(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $stats_obj = $Lun_obj->stats();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-LunStats-pm/ATTRIBUTES>
of NACL::CS::LunStats as input.
This method returns objects of the type L<NACL::CS::LunStats|lib-NACL-CS-LunStats-pm>.

=cut

sub stats {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::LunStats;
    my @state_objs = NACL::CS::LunStats->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub stats

=head2 geometry

    my $geometry_obj = NACL::C::Lun->geometry(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @geometry_objs = NACL::C::Lun->geometry(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $geometry_obj = $Lun_obj->geometry();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-LunGeometry-pm/ATTRIBUTES>
of NACL::CS::LunGeometry as input.
This method returns objects of the type L<NACL::CS::LunGeometry|lib-NACL-CS-LunGeometry-pm>.

=cut

sub geometry {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::LunGeometry;
    my @state_objs = NACL::CS::LunGeometry->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub geometry

=head2 test_vdisk_size

    NACL::C::Lun->test_vdisk_size(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to test vdisk_cmd_print_size functionality.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-test-vdisk-size" API for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'size' => $string >>

*Size

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : size

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun test-vdisk-size" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test_vdisk_size {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( size
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'lun test-vdisk-size',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_test_vdisk_size_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_test_vdisk_size_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::test_vdisk_size',
    );

    $Log->exit() if $may_exit;
} ## end sub test_vdisk_size

sub _test_vdisk_size_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,  api => 'lun_test_vdisk_size');
    $Log->exit() if $may_exit;
} ## end sub _test_vdisk_size_cmode_cli

sub _test_vdisk_size_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $allowed_cmode_options = [
        qw( size extended_query
            )
    ];
    $pkg->_base_backend_cmode_zapi(@opts,
		api    => 'lun_test_vdisk_size',
		copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _test_vdisk_size_cmode_zapi

=head2 comment

# Class method, set the value
    NACL::C::Lun->comment(
        command_interface => $ci,
    'comment' => $value
    );

# Class method, get the value
    my $value = NACL::C::Lun->comment(
        ...,
    );

# Instance method, set the value
    $lun->comment('comment' => $value);

# Instance method, get the value
    my $value = $lun->comment();

(Class or instance method)
This method is used to hASH(0x2b7030b8c6b0).

Uses a CMode CLI/ZAPI APISet.

This method is used to get or set the value of <comment>. When
<comment> is provided as an argument in the call, then it
gets set to that value. When <comment> is not provided as an
argument, then the current value is returned.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)
LUN Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path possible value(s) are, path

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

(Required for class method, Not Applicable for instance method)
Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name possible value(s) are, vserver name

Applicable for CMode CLI/ZAPI

=item C<< 'comment' => $string >>

Comment possible value(s) are, text

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: comment option of "lun-set-comment" API.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

Applicable for CMode CLI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"lun comment" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub comment {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( comment

                    lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun comment',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_comment_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_comment_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::comment',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub comment

sub _comment_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'lun_comment',
        apiset_response_obj => \$response
    );
    if (!$opts{'comment'}) {
        my $output = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{comment};
    }
    $Log->exit() if $may_exit;
} ## end sub _comment_cmode_cli

sub _comment_cmode_zapi {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, @opts) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@opts);

    my $apiset            = $opts{apiset};
    my $vserver           = $opts{vserver};
    my $command_interface = $opts{command_interface};

    my %cmode_args;

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( path comment )],
        target => \%cmode_args,
    );

    if ($opts{'comment'}) {
        $apiset->lun_set_comment(%cmode_args);
    } else {
        my $response = $apiset->lun_get_comment(%cmode_args);
        my $output   = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{comment};
    } ## end else [ if ( $opts{'comment'} )]
    $Log->exit() if $may_exit;
} ## end sub _comment_cmode_zapi

=head2 offline

    NACL::C::Lun->offline(
                                command_interface => $ci,
                                'lun' => $lun,
                                'qtree' => $qtree,
                                'path' => $path,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                ...);

                                 or

    $Lun_obj->offline(...);

(Class or instance method)
This method is used to disables block-protocol accesses to the LUN(s)..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)
LUN Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path possible value(s) are, path

Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name possible value(s) are, vserver name

Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'lun' => $string >>

LUN Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"lun offline" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub offline {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun qtree volume
                    )
            ),
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'lun offline',
    );
    my $vserver ;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_offline_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_offline_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::Lun::offline',
    );

    $Log->exit() if $may_exit;
} ## end sub offline

sub _offline_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,  api => 'lun_offline');
    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_cli

sub _offline_cmode_zapi_non_iter {
      $Log->enter() if $may_enter;
      my ($pkg_or_obj, @opts) = @_;
      my $copy = [qw(lun volume qtree path)];
      $pkg_or_obj->_base_backend_cmode_zapi(
          @opts,
          api  => 'lun-offline',
          copy => $copy,
      );
      $Log->exit() if $may_exit;
  }

=head2 online

    NACL::C::Lun->online(
                                command_interface => $ci,
                                'lun' => $lun,
                                'qtree' => $qtree,
                                'path' => $path,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                ...);

                                 or

    $Lun_obj->online(...);

(Class or instance method)
This method is used to re-enables block-protocol accesses to the LUN.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)
LUN Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path possible value(s) are, path

Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name possible value(s) are, vserver name

Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'lun' => $string >>

LUN Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI

=item C<< 'force' => $string >>


Forcibly online the LUN, disables checking for LUN mapping conflicts with the cluster partner..
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"lun online" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub online {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun qtree volume

                    force
                    )
            ),
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'lun online',
    );
    my $vserver ;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_online_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_online_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::Lun::online',
    );

    $Log->exit() if $may_exit;
} ## end sub online

sub _online_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,  api => 'lun_online');
    $Log->exit() if $may_exit;
} ## end sub _online_cmode_cli

sub _online_cmode_zapi_non_iter {
     $Log->enter() if $may_enter;
     my ($pkg_or_obj, @opts) = @_;
     my $copy = [qw(lun volume qtree path force)];
     $pkg_or_obj->_base_backend_cmode_zapi(
         @opts,
         api  => 'lun-online',
         copy => $copy,
     );
     $Log->exit() if $may_exit;
 }


=head2 select

# Class method, set the value
    NACL::C::Lun->select(
        ...,
        'select-attribute' => $value
    );

# Class method, get the value
    my $value = NACL::C::Lun->select(
        ...,
    );

# Instance method, set the value
    $lun->select('select-attribute' => $value);

# Instance method, get the value
    my $value = $lun->select();

(Class or instance method)
Uses a CMode CLI/ZAPI APISet.

This method is used to get or set the value of <select-attribute>. When
C<select-attribute> is provided as an argument in the call, then it
gets set to that value. When <select-attribute> is not provided as an
argument, then the current value is returned.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)
LUN Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path possible value(s) are, path

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: path

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name possible value(s) are, volume name

Applicable for CMode CLI/ZAPI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name possible value(s) are, vserver name

Applicable for CMode CLI/ZAPI

=item C<< 'select-attribute' => $string >>

Select Attribute possible value(s) are, active,copy,mirror

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: select-value option of "lun-set-select" API.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'lun' => $string >>

LUN Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by
"lun select" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub select {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( select-attribute

                    lun qtree volume
                    )
            ),
        },
        cli_cmd => 'lun select',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_select_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_select_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::select',
    );
    return $ret;
    $Log->exit() if $may_exit;
} ## end sub select

sub _select_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'lun_select',
        apiset_response_obj => \$response
    );
    if (!$opts{'select-attribute'}) {
        my $output = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{'select-attribute'};
    }
    $Log->exit() if $may_exit;
} ## end sub _select_cmode_cli

sub _select_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@opts);

    my %cmode_args;
    my $apiset            = $opts{apiset};
    my $vserver           = $opts{vserver};
    my $command_interface = $opts{command_interface};

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( path )],
        map    => {'select-attribute' => 'select-value',},
        target => \%cmode_args,
    );

    if ($opts{'select-attribute'}) {
        $apiset->lun_set_select_attribute(%cmode_args);
    } else {
        my $response = $apiset->lun_get_select_attribute(%cmode_args);
        my $output   = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{'select-attribute'};
    } ## end else [ if ( $opts{'select-attribute'...})]

    $Log->exit() if $may_exit;
} ## end sub _select_cmode_zapi

=head2 serial

# Class method, set the value
    NACL::C::Lun->serial(
        ...,
        'serial' => $value
    );

# Class method, get the value
    my $value = NACL::C::Lun->serial(
        ...,
    );

# Instance method, set the value
    $lun->serial('serial' => $value);

# Instance method, get the value
    my $value = $lun->serial();

(Class or instance method)
This method is used to hASH(0x2b7030b8c6b0).

Uses a CMode CLI/ZAPI APISet.

This method is used to get or set the value of <serial>. When
<serial> is provided as an argument in the call, then it
gets set to that value. When <serial> is not provided as an
argument, then the current value is returned.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required for class method, Not Applicable for instance method)
LUN Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path possible value(s) are, path

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: path

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name possible value(s) are, volume name

Applicable for CMode CLI/ZAPI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name possible value(s) are, vserver name

Applicable for CMode CLI

=item C<< 'serial' => $string >>

Serial Number possible value(s) are, text

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: serial-number option of "lun-set-serial" API.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'lun' => $string >>

LUN Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'qtree' => $string >>

Qtree Name possible value(s) are, text

Applicable for CMode CLI/ZAPI

=item C<< 'volume' => $string >>

Volume Name possible value(s) are, volume name

Applicable for CMode CLI/ZAPI

=item C<< 'serial-hex' => $string >>

Serial Number (Hex)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"lun serial" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub serial {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( serial

                    lun qtree volume serial-hex
                    )
            ),
        },
        cli_cmd => 'lun serial',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_serial_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_serial_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
		check     => '_cmode_zapi_check',
            },
        ],
        frontend => 'NACL::C::Lun::serial',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub serial

sub _serial_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'lun_serial',
        apiset_response_obj => \$response
    );
    if (!$opts{'serial'}) {
        my $output = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{serial};
    }
    $Log->exit() if $may_exit;
} ## end sub _serial_cmode_cli

sub _serial_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $apiset            = $opts{apiset};
    my $vserver           = $opts{vserver};
    my $command_interface = $opts{command_interface};

    my %cmode_args;

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( path comment )],
        map    => {'serial-number' => 'serial'},
        target => \%cmode_args,
    );

    if ($opts{'serial'}) {
        $apiset->lun_set_serial_number(%cmode_args);
    } else {
        my $response = $apiset->lun_get_serial_number(%cmode_args);
        my $output   = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{'serial-number'};
    } ## end else [ if ( $opts{'serial'} )]

    $Log->exit() if $may_exit;
} ## end sub _serial_cmode_zapi

=head2 iofence_set

    NACL::C::Lun->iofence_set(
                              ...,
                              'inode-num' => $value
                             );

(Class method)

Uses a 7Mode/Nodescope APISet.

This method is used to set the values for iofence operation.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fsid' => $fsid >>

FSID for the particular volume

Applicable for 7Mode/Nodescope.

=item C<< 'inode-num' => $inode >>

Inode number associated with the volume

Applicable for 7Mode/Nodescope.

=item C<< 'generation-num' => $string >>

Base generation number associated with the volume

Applicable for 7Mode/Nodescope.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_set {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw( fsid inode-num generation-num)),},
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_iofence_set_7mode_cli',
                interface => 'CLI',
                set       => "7Mode|Nodescope",
            },
        ],
        frontend => 'NACL::C::Lun::iofence_set',
    );
    $Log->exit() if $may_exit;
} ## end sub iofence_set

sub _iofence_set_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts                  = $pkg->_validate_backend_opts(@args);
    my $command_interface     = $opts{command_interface};
    my $apiset                = $opts{apiset};
    my $allowed_7mode_options = [qw( fsid inode-num generation-num)];
    my $valid_args            = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
    );
    $apiset->iofence_set(%$valid_args);
    $Log->exit() if $may_exit;
} ## end sub _iofence_set_7mode_cli

=head2 iofence_clear

    NACL::C::Lun->iofence_clear(
                                 ...,
                                 'inode-num' => $value
                                );

(Class method)

Uses a 7Mode/Nodescope APISet.

This method is used to clear the values which are set for iofence operation.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fsid' => $fsid >>

FSID for the particular volume

Applicable for 7Mode/Nodescope.

=item C<< 'inode-num' => $inode >>

Inode number associated with the volume

Applicable for 7Mode/Nodescope.

=item C<< 'generation-num' => $string >>

Base generation number associated with the volume

Applicable for 7Mode/Nodescope.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_clear {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw( fsid inode-num generation-num )),},
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_iofence_clear_7mode_cli',
                interface => 'CLI',
                set       => "7Mode|Nodescope",
            },
        ],
        frontend => 'NACL::C::Lun::iofence_clear',
    );

    $Log->exit() if $may_exit;
} ## end sub iofence_clear

sub _iofence_clear_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts                  = $pkg->_validate_backend_opts(@args);
    my $command_interface     = $opts{command_interface};
    my $apiset                = $opts{apiset};
    my $allowed_7mode_options = [qw( fsid inode-num generation-num)];
    my $valid_args            = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
    );
    $apiset->iofence_clear(%$valid_args);
    $Log->exit() if $may_exit;
} ## end sub _iofence_clear_7mode_cli

=head2 iofence_get

# Class method, get the values
    NACL::C::Lun->iofence_get(
                               ...,
                               'inode-num' => $value
                             );

(Class method)

Uses a 7Mode/Nodescope APISet.

This method is used to get the values for associated with iofence operation.
It will return an array of object with attributes refcnt,proto_fmode,type.

This method returns objects of type
L<NACL::CS::IOfence|lib-NACL-CS-IOfence-pm>.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fsid' => $fsid >>

FSID for the particular volume

Applicable for 7Mode/Nodescope.

=item C<< 'inode-num' => $inode >>

Inode number associated with the volume

Applicable for 7Mode/Nodescope.

=item C<< 'generation-num' => $string >>

Base generation number associated with the volume

Applicable for 7Mode/Nodescope.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub iofence_get {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            {_optional_scalars(qw( fsid inode-num generation-num )),},
        ignore_primary_keys => 1,
    );

    my @ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_iofence_get_7mode_cli',
                interface => 'CLI',
                set       => "7Mode|Nodescope",
            },
        ],
        frontend => 'NACL::C::Lun::iofence_get',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
} ## end sub iofence_get

sub _iofence_get_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);
    my @state_objs;
    my $ci                    = $opts{command_interface};
    my $apiset                = $opts{apiset};
    my $allowed_7mode_options = [qw( fsid inode-num generation-num)];
    my $valid_args            = $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
    );
    my $response = $apiset->iofence_get(%$valid_args);
    my $output   = $response->get_parsed_output();

    foreach my $row (@$output) {
        my $obj = NACL::CS::IOfence->new(command_interface => $ci);
        $obj->_set_fields(row => $row);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _iofence_get_7mode_cli

=head2 debug_print

    NACL::C::Lun->debug_print(
                                command_interface => $ci,
                                'path' => $path,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to prints debug information contained in the SCSI target subsystem associated with a LUN..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'path' => $path >>

(Required)
LUN Path
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

=item C<< 'serial' => $string >>

LUN Serial Number
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "lun debug print" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub debug_print {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( serial vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd                 => 'lun debug print',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_debug_print_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Lun::debug_print',
    );

    $Log->exit() if $may_exit;
}

sub _debug_print_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_debug_print',);
    $Log->exit() if $may_exit;
}

=head2 show_transient_metadata

    my $show_transient_metadata_obj = NACL::C::Lun->show_transient_metadata(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_transient_metadata_objs = NACL::C::Lun->show_transient_metadata(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_transient_metadata_obj = $Lun_obj->show_transient_metadata();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).

=cut

sub show_transient_metadata {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::LunShowTransientMetadata;
    my @state_objs = NACL::CS::LunShowTransientMetadata->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 group_internal_rebuild

    NACL::C::Lun->group_internal_rebuild(
                                command_interface => $ci,
                                    'path' => $path,
                                    'vserver' => $vserver,
                                ...);

(Class method)
This method is used to rebuilds the distributed SCSI target subsystems associated with a LUN..
Uses CMode CLI/ZAPI .
Invokes  lun-group-internal-rebuild  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'path' => $path >>

(Required)
LUN Path
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'serial' => $string >>

LUN Serial Number
Applicable for CMode CLI/ZAPI, non-iter.

Maps to: serial-number

=item C<< '*' >>

This method takes all other options supported by "lun group internal rebuild" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub group_internal_rebuild {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( serial vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'lun group-internal-rebuild',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_group_internal_rebuild_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_group_internal_rebuild_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::Lun::group_internal_rebuild',
    );

    $Log->exit() if $may_exit;
}

sub _group_internal_rebuild_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'lun_group_internal_rebuild',);
    $Log->exit() if $may_exit;
}

sub _group_internal_rebuild_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(vserver path)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-group-internal-rebuild',
        copy => $copy,
        map  => { serial => 'serial-number' },
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | OBJECT | UNDEF, optional => 1},
        'path'    => {type => SCALAR | OBJECT},
    );
} ## end sub _primary_keys_validate_spec

sub _super_common_validate_with {
    my ($pkg_or_obj, @opts) = @_;
    return $pkg_or_obj->SUPER::_common_validate_with(@opts);
}

1;
