# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunPortset  Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunPortset

=head1 DESCRIPTION

C<NACL::C::LunPortset> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunPortset.  A related class is
L<NACL::CS::LunPortset|lib-NACL-CS-LunPortset-pm>, which represents
the past state of an ONTAP LunPortset.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 portset

=cut

package NACL::C::LunPortset;

use strict;
use warnings;

use NACL::CS::LunPortset;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NACL::C::Exceptions::LunPortset::AlreadyExists qw(:try);
use NACL::C::Exceptions::LunPortset::DoesNotExist qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

my $LunPortset = NACL::C::LunPortset->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   portset =>$portset,
                                );
Return a new LunPortset component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'portset',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $LunPortset  = NACL::C::LunPortset->find(command_interface => $ci, ...);
    my @LunPortsets = NACL::C::LunPortset->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $LunPortset_state = $NACL::C::LunPortset->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunPortset would return
a NACL::CS::LunPortset object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 create

    my $LunPortset_object   = NACL::C::LunPortset->create(
                                command_interface => $ci,
                                vserver => $vserver,
                                portset => $portset,
                                  ...);

(Class method)
This method is used to creates a new portset.

Uses a CMode CLI/ZAPI, 7Mode CLI.

Invokes "portset-create" API for CMode ZAPI.

Invokes "portset create" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "portset" => $portset >>

(Required)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : portset-name

7M CLI  : portset name (Programmatic name: "portset-name")

=item C<< "tpg" => $string >>

Applicable for CMode CLI.

=item C<< "fcp" => $string >>

Applicable for CMode CLI.

=item C<< "fc-lif" => $string >>

Applicable for CMode CLI.

=item C<< "protocol-type" => $string >>

Applicable for CMode CLI.

=item C<< "iscsi" => $string >>

Applicable for CMode CLI.

=item C<< "port-name" => [ $port-name1, $port-name2, ... ] >>

Applicable for CMode CLI.


=item C<< "protocol" => $string >>

(Optional)
Expected values : mixed|fcp|iscsi

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : portset-type

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun portset create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        only_static_method => 1,
        params             => \@args,
        additional_spec    => {
            "port-name" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( tpg fcp fc-lif protocol-type protocol portset iscsi
                    )
            ),
        },
        cli_cmd => 'lun portset create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunPortset::create',
        nacl_exceptions => [ qw(AlreadyExists) ],
    );
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{vserver},
        portset           => $opts{'portset'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api => 'lun_portset_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $protocol          = delete $opts{protocol};
    my %cmd_args;
    $pkg->_hash_copy(
        source => \%opts,
        map    => {"portset" => "portset-name"},
        target => \%cmd_args,
    );
    %opts = %cmd_args;

    if ($protocol =~ /fcp/i) {
        $opts{fcp} = 1;
    } else {
        $opts{iscsi} = 1;
    }
    foreach my $array_element (qw(port-name)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ' ', @{$opts{$array_element}};
        }
    }

    $apiset->portset_create(%opts);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        vserver           => $opts{'vserver'},
        portset           => $opts{'portset-name'},
    );
} ## end sub _create_7mode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;
    my $portset_port_names = delete $opts{'port-name'};

    #W.R.T burt#910406, When user do not provide 'protocol' field
    #in ZAPI run then set 'portset-type' as 'mixed' similar to CLI
    $opts{'protocol'} //= 'mixed';
    my $exception;
    try{
        $pkg->_base_backend_cmode_zapi(
            %opts,
            api  => 'portset-create',
            map    => {
                "portset"  => "portset-name",
                "protocol" => "portset-type"
            },
        );
    } otherwise {
        #Burt953020 Don't fail immediately, but execute all ZAPIs first
        $exception = $_[0];
    };
    my $apiset = $opts{apiset};
    my %cmd_args  ;
    $cmd_args{'portset-name'} = $opts{portset};
    $cmd_args{'vserver-override-name'} = $opts{'vserver'} ;
    foreach my $port_name (@{$portset_port_names}) {
        try{
            $cmd_args{'portset-port-name'} = $port_name;
            $apiset->portset_add(%cmd_args);
        } otherwise {
            #Burt953020 Don't fail immediately, but execute all ZAPIs first
            $exception = $_[0] if (!defined $exception);
        };
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 remove

   NACL::C::LunPortset->remove(
                                command_interface => $ci,
                                vserver => $vserver,
                                portset => $portset,
                                  ...);

                            or

    $LunPortset_object->remove(...);

(Class or instance method)
This method is used to remove SCSI TPGs / FCP LIFs from a portset.

Uses a CMode CLI/ZAPI, 7Mode CLI.

Invokes "portset-remove" API for CMode ZAPI.

Invokes "portset remove" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "portset" => $portset >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : portset-name

7M CLI  : portset name (Programmatic name: "portset-name")

=item C<< "port-name" >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : port-name

7M CLI  : port name (Programmatic name: "port-name")

=item C<< "fc-lif" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : portset-port-fclif

=item C<< "tpg" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : portset-port-tpgroup

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun portset remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "port-name" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( fc-lif tpg
                    )
            ),
        },
        cli_cmd => 'lun portset remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunPortset::remove',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( port-name )];
    my %cmd_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"portset" => "portset-name",},
        target => \%cmd_args,
    );
    %opts = %cmd_args;

    foreach my $array_element (qw(port-name)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ' ', @{$opts{$array_element}};
        }
    }

    $apiset->portset_remove(%opts);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_portset_remove');
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%opts) = @_;
    my $arr_ref  = $opts{'port-name'};
    my @port_arr = @$arr_ref;
    my $exception;
    foreach my $element (@port_arr) {
        $opts{'port-name'} = $element;
        try{
            $pkg_or_obj->_base_backend_cmode_zapi(
                %opts,
                api  => 'portset_remove',
                map    => {
                    "portset"   => "portset-name",
                    "port-name" => "portset-port-name",
                    "fc-lif"    => "portset-port-fclif",
                    "tpg"       => "portset-port-tpgroup",
                },
            );
        }otherwise {
            # Burt930360 Don't fail immediately, but execute all ZAPIs first
            $exception = $_[0];
        };
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }

    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

=head2 delete

   NACL::C::LunPortset->delete(
                                command_interface => $ci,
                                vserver => $vserver,
                                portset => $portset,
                                  ...);

                        or

    $LunPortset_object->delete(...);

(Class or instance method)
This method is used to delete the portset.

Uses a CMode CLI/ZAPI, 7Mode CLI.

Invokes "portset-destroy" API for CMode ZAPI.

Invokes "portset destroy" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "portset" => $portset >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : portset-name

7M CLI  : portset name (Programmatic name: "portset-name")

=item C<< "force" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : force

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun portset delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
        },
        cli_cmd => 'lun portset delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_delete_zapi_check',
            },
        ],
        frontend => 'NACL::C::LunPortset::delete',
        nacl_exceptions => [ qw(DoesNotExist) ]
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_portset_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cmd_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {"portset" => "portset-name"},
        target => \%cmd_args,
    );
    %opts = %cmd_args;
    $apiset->portset_destroy(%opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
	my $allowed_cmode_options = [
        qw( force
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'portset_destroy',
            copy   => $allowed_cmode_options,
            map    => {"portset" => "portset-name",},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields_to_check = ('portset');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;
} ## end sub _delete_zapi_check    

=head2 add

   NACL::C::LunPortset->add(
                                command_interface => $ci,
                                vserver => $vserver,
                                portset => $portset,
                                  ...);

                        or

    $LunPortset_object->add(...);

(Class or instance method)
This method is used to add iSCSI TPGs / FCP LIFs to a Portset.

Uses a CMode CLI/ZAPI, 7Mode CLI.

Invokes "portset-add" API for CMode ZAPI.

Invokes "portset add" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "portset" => $portset >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : portset-name

7M CLI  : portset name (Programmatic name: "portset-name")

=item C<< "port-name" >>

Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI : portset-port-name

7M CLI  : port name (Programmatic name: "port-name")

=item C<< "fc-lif" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : portset-port-fclif

=item C<< "tpg" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : portset-port-tpgroup

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"lun portset add" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "port-name" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( fc-lif tpg
                    )
            ),
        },
        cli_cmd => 'lun portset add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunPortset::add',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,  api => 'lun_portset_add');
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_cli

sub _add_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( port-name
            )
    ];
    my %cmd_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"portset" => "portset-name",},
        target => \%cmd_args,
    );
    %opts = %cmd_args;
    foreach my $array_element (qw(port-name)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ' ', @{$opts{$array_element}};
        }
    }
    $apiset->portset_add(%opts);
    $Log->exit() if $may_exit;
} ## end sub _add_7mode_cli

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%opts) = @_;
    my $arr_ref  = $opts{'port-name'};
    my @port_arr = @$arr_ref;
    my $exception;
    foreach my $element (@port_arr) {
        $opts{'port-name'} = $element;
        try{
            $pkg_or_obj->_base_backend_cmode_zapi(
                %opts,
                api  => 'portset_add',
                map    => {
                    "portset"   => "portset-name",
                    "port-name" => "portset-port-name",
                    "fc-lif"    => "portset-port-fclif",
                    "tpg"       => "portset-port-tpgroup",
                },
            );
        }otherwise {
            #Burt930360 Don't fail immediately, but execute all ZAPIs first
            $exception = $_[0];
        };
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'portset' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
